# NNLOJET
Version 1.0.0

* * *

NNLOJET is a parton-level event generator for jet cross sections using the antenna subtraction method. It can be used to compute a large number of jet cross sections and related observables in e+e−, ep and pp collisions at next-to-next-to-leading order in QCD.

NNLOJET contains routines for Monte Carlo phase-space integration, event handling, and analysis. It is interfaced to LHAPDF 6 for evaluating parton distribution functions (PDFs) and the strong coupling constant. Interfaced to APPLfast or PineAPPL, it also allows for the generation of interpolation grids.

NNLOJET has been used for some of the most advanced calculations in jet physics.

## Installation

### Dependencies

* `cmake` build system (minimum version 3.18)
* [`LHAPDF`](https://lhapdf.hepforge.org/) library (minimum version: 6) 
* `python` interpreter for the workflow (minimum version: 3.10)

### Setup

The source files can be downloaded from <https://nnlojet.hepforge.org> as a tarball, which unpacks into a folder `nnlojet-X.Y.Z`. To build NNLOJET requires the following commands:

```
cd nnlojet-X.Y.Z
mkdir build
cd build
cmake ..
make [ -jN ]
make install
```

Compilation can be run in parallel on `N` cores with the `-jN` optional argument. After successful completion of the build and installation, the NNLOJET executable will be located at: `nnlojet-X.Y.Z/install/bin/NNLOJET`. It is recommended to add the location of the NNLOJET bin folder to the system `$PATH` variable.

The `lhapdf-config` utility should be available in the current environment. Alternatively, a directory can be specified by adding `-DLHAPDF_ROOT_DIR=/path/to/LHAPDF` to the `cmake` command.

A path to the `python` interpreter can be specified by adding `-DPython3_ROOT_DIR=/path/to/python` to the `cmake` command.

By default, `OpenMP` is disabled. To enable it, append `-DOPENMP=ON` to the `cmake` command.


### Example runcards
Some example runcards are included in the tarball. To execute them, run

```
cd examples/VJ/1901.11041
NNLOJET --run ZJL.run
```

### Workflow
The main user-facing interface to NNLOJET is the workflow script `nnlojet-run`, which provides an automated way to set up and run calculations. It installed together with the main NNLOJET executable in the `nnlojet-X.Y.Z/install/bin/` folder. Please refer to the NNLOJET manual on instructions how to use the script.
The direct execution of the core executable is possible but requires a more in-depth knowledge of the inner workings of the program, and is therefore only recommended for expert users.

## Citation
If you are using NNLOJET for a scientific paper, please cite:

    NNLOJET Collaboration
    NNLOJET: a parton-level event generator for jet cross sections at NNLO QCD accuracy
    arXiv:240x.xxxx

Please also cite the revelant references for each processes (as included in the `NNLOJET_references_XXX.bib` file which is automatically written when running the process).

## Contact
In case of questions, comments or feedback, please visit

<https://nnlojet.hepforge.org/>

or contact

<nnlojet@projects.hepforge.org>
