

#> Fortran
set(CMAKE_Fortran_PREPROCESS ON)
#set(CMAKE_Fortran_FORMAT FREE)
#> C/C++
set(CMAKE_CXX_STANDARD 11)
#> Fortran-C-interoperability
include(FortranCInterface)
FortranCInterface_VERIFY()
if(CMAKE_Fortran_COMPILER_ID MATCHES "^Intel")
  # use:  cmake -DCMAKE_C_COMPILER=icx -DCMAKE_CXX_COMPILER=icpx -DCMAKE_Fortran_COMPILER=ifx ..
  add_compile_options("$<$<COMPILE_LANGUAGE:Fortran>:-no-wrap-margin;-extend-source;132>")
elseif(CMAKE_Fortran_COMPILER_ID STREQUAL "GNU")
  add_compile_options("$<$<COMPILE_LANGUAGE:Fortran>:-ffixed-line-length-none>")
  if(CMAKE_Fortran_COMPILER_VERSION VERSION_GREATER_EQUAL 10)
    # message(STATUS "!!! ALLOWING ARGUMENT MISMATCH !!!")
    add_compile_options("$<$<COMPILE_LANGUAGE:Fortran>:-fallow-argument-mismatch>")
  endif()
endif()
#> verbose output by default (can also just use `make VERBOSE=1` when compiling)
# set(CMAKE_VERBOSE_MAKEFILE ON)
#> generate `compile_commands.json`
set(CMAKE_EXPORT_COMPILE_COMMANDS ON)


#> default build type is: Release
if (NOT CMAKE_BUILD_TYPE)
  # other options: Debug, Release, RelWithDebInfo, MinSizeRel
  #set(CMAKE_BUILD_TYPE Debug CACHE STRING "Build type" FORCE)
  set(CMAKE_BUILD_TYPE Release CACHE STRING "Build type" FORCE)
endif()
message(STATUS "Build type: ${CMAKE_BUILD_TYPE}")
#> would have thought that CMake adds this automatically
#> but encountered cases where this is not the case?
#> (better be safe than sorry, won't hurt to define twice)
add_compile_definitions($<IF:$<CONFIG:Debug>,DEBUG,NDEBUG>)


#> set up installation directories
if(CMAKE_INSTALL_PREFIX_INITIALIZED_TO_DEFAULT)
  #message(STATUS "overriding default install prefix: ${CMAKE_INSTALL_PREFIX}")
  set(CMAKE_INSTALL_PREFIX "${PROJECT_SOURCE_DIR}/install" CACHE PATH "Install directory" FORCE)
endif()
message(STATUS "Install directory: ${CMAKE_INSTALL_PREFIX}")
include(GNUInstallDirs)
message(STATUS "LIB: ${CMAKE_INSTALL_FULL_LIBDIR}")


#> set up RPATH to work both in the build/install tree
set(CMAKE_SKIP_BUILD_RPATH FALSE)
set(CMAKE_BUILD_WITH_INSTALL_RPATH FALSE)
set(CMAKE_INSTALL_RPATH "${CMAKE_INSTALL_FULL_LIBDIR}")
set(CMAKE_INSTALL_RPATH_USE_LINK_PATH TRUE)


#> helper function to read in `NNLOJET_FILES` lists from different sources
function(nnlojet_read list)
  set(flags APPEND EXISTS)
  set(args VARNAME)
  set(listArgs SOURCES)
  cmake_parse_arguments(arg "${flags}" "${args}" "${listArgs}" ${ARGN})
  if (NOT arg_SOURCES)
    message(FATAL_ERROR "[nnlojet_read]: SOURCES is a required argument")
  endif()
  if (SOURCES IN_LIST arg_KEYWORDS_MISSING_VALUES)
    message(FATAL_ERROR "[nnlojet_read]: SOURCES requires at least one value")
  endif()
  if(arg_VARNAME)
    set(_varname ${arg_VARNAME})
  else()
    set(_varname NNLOJET_FILES)
  endif()
  if (arg_APPEND)
    set(_files ${${list}})
  else()
    set(_files)
  endif()
  foreach(_src ${arg_SOURCES})
    if (IS_DIRECTORY ${_src})
      set(_dir ${_src})
      include(${_src}/CMakeLists.txt)
    else()
      # file(TO_CMAKE_PATH "${_src}" _dir)
      # cmake_path(GET ${_src} PARENT_PATH _dir)
      get_filename_component(_dir ${_src} DIRECTORY)
      include(${_src})
    endif()
    foreach(_file ${${_varname}})
      if (arg_EXISTS)
        if (EXISTS ${_dir}/${_file})
          list(APPEND _files ${_dir}/${_file})
        endif()
      else()
        list(APPEND _files ${_dir}/${_file})
      endif()
    endforeach()
  endforeach()
  set(${list} ${_files} PARENT_SCOPE)
endfunction()


if (APPLE)
  message(STATUS "APPLE device!")
  # check_linker_flag("-Wl,-no_compact_unwind" LINKER_SUPPORTS_NCU)
  # message(STATUS "LINKER_SUPPORTS_NCU: ${LINKER_SUPPORTS_NCU}")
  foreach(_type EXE;MODULE;SHARED;STATIC)
    set(CMAKE_${_type}_LINKER_FLAGS "${CMAKE_${_type}_LINKER_FLAGS} -Wl,-no_compact_unwind -flat_namespace")
    message(STATUS "CMAKE_${_type}_LINKER_FLAGS: ${CMAKE_${_type}_LINKER_FLAGS}")
  endforeach()
  unset(_type)
  # list(APPEND CMAKE_EXE_LINKER_FLAGS "-Wl,-no_compact_unwind")
  # message(STATUS "CMAKE_EXE_LINKER_FLAGS: ${CMAKE_EXE_LINKER_FLAGS}")
endif(APPLE)



#> inspect libraries (https://medium.com/a-42-journey/nm-otool-everything-you-need-to-know-to-build-your-own-7d4fef3d7507):
#> otool -hV libnnlojet_core.dylib
#> (no `TWOLEVEL` here means it was linked with -flat-namespace)

#> get list of source files (missing driver-process files)
#❯ find .. -type f -name "CMakeLists.txt" -exec awk 'BEGIN{qm=0}$1!~/^#/&&NF>0&&qm==0{if($(NF)=="NNLOJET_FILES"){qm=1;next}else{quit}}$1!~/^#/&&$1!~/)/&&qm==1&&NF>0{print}' {} \;

#> strings:
#  character(*), intent(in) :: a
#  character(:), allocatable, intent(out) :: b


#> dokan [ON/OFF]; default: ON
if (NOT DEFINED DOKAN)
  set(DOKAN ON CACHE STRING "dokan" FORCE)
endif()
message(STATUS "dokan: ${DOKAN}")
if ("${DOKAN}")  # <- this should trigger on all "ON" "YES" "1", ... (true constants)
  #> Python  https://cmake.org/cmake/help/v3.12/module/FindPython3.html#module:FindPython3
  #find_package(Python3 COMPONENTS Interpreter Development)
  find_package(Python3 3.10 REQUIRED COMPONENTS Interpreter)
  # message(STATUS "Python3_FOUND: ${Python3_FOUND}")
  # message(STATUS "Python3_VERSION: ${Python3_VERSION}")
  # message(STATUS "Python3_VERSION_MAJOR: ${Python3_VERSION_MAJOR}")
  # message(STATUS "Python3_VERSION_MINOR: ${Python3_VERSION_MINOR}")
  # message(STATUS "Python3_VERSION_PATCH: ${Python3_VERSION_PATCH}")
endif()


#> OpenMP [ON/OFF]; default: OFF
if (NOT DEFINED OPENMP)
  set(OPENMP OFF CACHE STRING "OpenMP" FORCE)
endif()
message(STATUS "OpenMP: ${OPENMP}")
if ("${OPENMP}")  # <- this should trigger on all "ON" "YES" "1", ... (true constants)
  find_package(OpenMP)
  if (OpenMP_Fortran_FOUND)
    message(STATUS "OpenMP_Fortran_VERSION: ${OpenMP_Fortran_VERSION}")
  else()
    message(FATAL_ERROR "could not find OpenMP!")
  endif()
endif()


#> PineAPPL [ON/OFF]; default: OFF
if (NOT DEFINED PINEAPPL)
  set(PINEAPPL OFF CACHE STRING "PineAPPL interface" FORCE)
endif()
message(STATUS "PineAPPL: ${PINEAPPL}")
#if (PINEAPPL STREQUAL "ON")
if ("${PINEAPPL}")  # <- this should trigger on all "ON" "YES" "1", ... (true constants)
  find_package(PkgConfig REQUIRED)
  pkg_check_modules(PINEAPPL REQUIRED pineappl_capi IMPORTED_TARGET)
  if (PINEAPPL_FOUND)
    # add_library(PINEAPPL::PINEAPPL SHARED IMPORTED)
    # set_target_properties(PINEAPPL::PINEAPPL PROPERTIES
    #   IMPORTED_LOCATION "${PINEAPPL_LIBRARY_DIRS}"
    #   LINK_FLAGS "${PINEAPPL_LDFLAGS}"
    # )
    # message(STATUS "PINEAPPL_FOUND: ${PINEAPPL_LDFLAGS}")
    # message(STATUS "PINEAPPL_LIBRARIES: ${PINEAPPL_LIBRARIES}")
    # message(STATUS "PINEAPPL_LIBRARY_DIRS: ${PINEAPPL_LIBRARY_DIRS}")
    # message(STATUS "PINEAPPL_LDFLAGS: ${PINEAPPL_LDFLAGS}")
    # message(STATUS "PINEAPPL_INCLUDE_DIRS: ${PINEAPPL_INCLUDE_DIRS}")
    # message(STATUS "PINEAPPL_CFLAGS: ${PINEAPPL_CFLAGS}")
  else()
    message(FATAL_ERROR "could not find PineAPPL!")
  endif()
endif()


#> APPLfast [ON/OFF]; default: OFF
if (NOT DEFINED APPLFAST)
  set(APPLFAST OFF CACHE STRING "APPLfast interface" FORCE)
endif()
message(STATUS "APPLFAST: ${APPLFAST}")
if ("${APPLFAST}")  # <- this should trigger on all "ON" "YES" "1", ... (true constants)
  find_package(BRIDGE 2 REQUIRED)
  if (BRIDGE_FOUND AND NOT TARGET BRIDGE::BRIDGE)
    add_library(BRIDGE::BRIDGE SHARED IMPORTED)
    set_target_properties(BRIDGE::BRIDGE PROPERTIES
      IMPORTED_LOCATION "${BRIDGE_LIBRARY}"
      # INTERFACE_COMPILE_OPTIONS "${BRIDGE_CXX_FLAGS}"
      INTERFACE_INCLUDE_DIRECTORIES "${BRIDGE_INCLUDE_DIR}"
      INTERFACE_LINK_DIRECTORIES "${BRIDGE_LIBRARY_DIR}"
      # INTERFACE_LINK_LIBRARIES "BRIDGE"
    )
    message(STATUS "BRIDGE_CONFIG: ${BRIDGE_CONFIG}")
    message(STATUS "BRIDGE_VERSION: ${BRIDGE_VERSION}")
  else()
    message(FATAL_ERROR "could not find APPLfast bridge interface!")
  endif()
endif()


