!-------------------------------------------------------------------------------
module BFGfrag_mod
!-------------------------------------------------------------------------------
  implicit none
  private

  !> ifun = 1     gluon
  !>        2     up
  !>        3     down
  !>        4     strange
  !>        5     charm
  !>        6     bottom
  !>        7     top  (invalid)
  public :: frag_bfg

  !> Knots settings
  integer, parameter :: nb_x = 45
  real*8, dimension(nb_x), parameter :: x_grid = (/  &
    &  0.4627d-01,0.5131d-01,0.5690d-01,0.6311d-01,0.6999d-01,0.7762d-01,  &
    &  0.8608d-01,0.9547d-01,0.1059d+00,0.1174d+00,0.1302d+00,0.1444d+00,  &
    &  0.1602d+00,0.1776d+00,0.1970d+00,0.2185d+00,0.2423d+00,0.2687d+00,  &
    &  0.2980d+00,0.3305d+00,0.3666d+00,0.4065d+00,0.4508d+00,0.5000d+00,  &
    &  0.5321d+00,0.5643d+00,0.5964d+00,0.6286d+00,0.6607d+00,0.6929d+00,  &
    &  0.7250d+00,0.7571d+00,0.7893d+00,0.8214d+00,0.8536d+00,0.8857d+00,  &
    &  0.9000d+00,0.9179d+00,0.9300d+00,0.9400d+00,0.9500d+00,0.9600d+00,  &
    &  0.9700d+00,0.9800d+00,0.99d+00 /)
  integer, parameter :: nb_q2_4flav = 8
  real*8, dimension(nb_q2_4flav), parameter :: q2_4flav =(/  &
    &  2.d0,2.8d0,3.9d0,5.4d0,7.5d0,10.5d0,14.5d0,20.24d0 /)
  integer, parameter :: nb_q2_5flav = 22
  real*8, dimension(nb_q2_5flav), parameter :: q2_5flav = (/  &
    &  20.26d0,33.d0,55.d0,93.d0,156.d0,261.d0,  &
    &  435.d0,732.d0,1224.d0,2048.d0,  &
    &  3427.d0,5733.d0,9592.d0,16047.d0,  &
    &  26847.d0,44915.d0,75144.d0,1.257d+5,  &
    &  2.1d+5,3.52d+5,5.89d+5,9.848d+5 /)

  !> derived arrays
  real*8, dimension(nb_x), parameter :: x = log10(x_grid)
  real*8, dimension(nb_q2_4flav), parameter :: y_4flav = log10(q2_4flav)
  real*8, dimension(nb_q2_5flav), parameter :: y_5flav = log10(q2_5flav)


contains


  subroutine frag_bfg(x,Q2,iset,ifun,func)
    implicit none
    double precision, intent(in) :: x, Q2
    integer,intent(in) :: iset, ifun
    double precision, intent(out) :: func
    double precision :: pert, vdm

    call distributionPert(x,Q2,ifun,pert)
    select case (iset)
    case (0)
      vdm = 0d0
    case (1)
      call distributionNonPert_setI(x,Q2,ifun,vdm)
    case (2)
      call distributionNonPert_setII(x,Q2,ifun,vdm)
    case default
      print*, "frag_bfg: invalid iset:", iset
      stop
    end select

    func = x * (pert + vdm)

  end subroutine frag_bfg


!------------------------------------------------------------
!------------- subroutine to compute a point in x and q**2 --
!------------- for the following fragmentation functions:  --
!-------------     gluon,  up   ,down,                     --
!-------------        strange , charm , bottom , top .     --
!------------- range of validity :                         --
!-------------                     1.e-03 < x < .99        --
!-------------                   log10 2  <log10 q**2 < 10 --
!------------------------------------------------------------
!-------------        authors :    p chiappetta
!-------------                     j.ph. guillet
!-------------                     m.greco
!-------------    cgw for q02=2gev2         method p.nason
!-------------    next to leading evolution
!-------------    table for lhc
!         i = 1    gluon
!         i = 2    up ( = antiup )
!         i = 3    down ( = antidown )
!         i = 4    strange ( = antistrange )
!         i = 5    charm ( = anticharm )
!         i = 6    bottom ( = antibottom )
!         i = 7    top ( = antitop )
!
!------------------------------------------------------------
  subroutine distributionPert(z,qstar2,ifun,func)
    implicit real*8 (a-h,o-z)
    implicit integer (i-n)
!------------------------------------------------------------
!-------------   z is x and qstar2 is q square  -------------
    integer ifun
!------------------------------------------------------------
!------------ strfun specifies the desired function----------
!------------------------------------------------------------
!------------------------------------------------------------
!        ifun =  1     gluon           'gluon' (abbrev. 'gl')
!                2     up              'upval' (abbrev. 'up')
!                3     down            'doval' (abbrev. 'do')
!                4     strange         'sbar'  (abbrev. 'sb')
!                5     charm           'cbar ' (abbrev. 'cb')
!                6     bottom          'bbar ' (abbrev. 'bb')
!                7     top             'tbar ' (abbrev. 'tb')
!------------------------------------------------------------
!-------------  func is the output (parton density value)----
!------------------------------------------------------------
    parameter (kfun=6)
    dimension xqdum1(nb_x,nb_q2_4flav,kfun)
    dimension xqdum2(nb_x,nb_q2_5flav,kfun)
    include "BFGfrag_gridPert.inc"
!!!     parameter (iqmax= max(nb_q2_4flav,nb_q2_5flav) )
!!!      not currently supported by g77
    parameter (iqmax=22)
    parameter (igt=4)

    dimension y(iqmax)
    dimension f(nb_x,iqmax)
    dimension q2(iqmax)
    dimension x1a(igt),x2a(igt),ya(igt,igt)

    func = 0d0
    xmin = x_grid(1)
    xmax = x_grid(nb_x)

    !> impose minimum q^2
    if(qstar2.le.20.26d0) then
      q2min=q2_4flav(1)
    else
      q2min=q2_5flav(1)
    endif
    if (qstar2.le.q2min) then
      write(65,*) ' q2 too small!!!!!!!!!'
      stop
    endif

    if ( (ifun < 1) .or. (ifun > 6) ) then
      write(65,*) 'error : invalid ifun'
      stop
    endif

    if(qstar2.le.20.26d0) then
      iq=nb_q2_4flav
      do j=1,iq
        do i=1,nb_x
          f(i,j)=xqdum1(i,j,ifun)
        enddo
        y(j)=y_4flav(j)
      enddo
    else
      iq=nb_q2_5flav
      do j=1,iq
        do i=1,nb_x
          f(i,j)=xqdum2(i,j,ifun)
        enddo
        y(j)=y_5flav(j)
      enddo
    endif

    !write(65,*)'y4flav',y_4flav
    !write(65,*)'y5flav',y_5flav
    !write(65,*)'y',y
    func=0d0
    if(z.gt.xmin.and.z.lt.xmax) then
      a=log10(z)
      b=log10(qstar2)
      call locate1_bfg(x,nb_x,a,jx)
      call locate1_bfg(y,iq,b,jq)
      do ijx=1,igt
        if (jx.eq.1) then
          x1a(ijx)=x(ijx)
          do ijq=1,igt
            if (jq.eq.1) then
              x2a(ijq)=y(ijq)
              ya(ijx,ijq)=f(ijx,ijq)
            else if (jq.eq.(iq-1)) then
              x2a(ijq)=y(jq-3+ijq)
              ya(ijx,ijq)=f(ijx,jq-3+ijq)
            else if (jq.eq.iq) then
              x2a(ijq)=y(jq-4+ijq)
              ya(ijx,ijq)=f(ijx,jq-4+ijq)
            else
              x2a(ijq)=y(jq-2+ijq)
              ya(ijx,ijq)=f(ijx,jq-2+ijq)
            endif
          enddo
        else if (jx.eq.(nb_x-1)) then
          x1a(ijx)=x(jx-3+ijx)
          do ijq=1,igt
            if (jq.eq.1) then
              x2a(ijq)=y(ijq)
              ya(ijx,ijq)=f(jx-3+ijx,ijq)
            else if (jq.eq.(iq-1)) then
              x2a(ijq)=y(jq-3+ijq)
              ya(ijx,ijq)=f(jx-3+ijx,jq-3+ijq)
            else if (jq.eq.iq) then
              x2a(ijq)=y(jq-4+ijq)
              ya(ijx,ijq)=f(jx-3+ijx,jq-4+ijq)
            else
              x2a(ijq)=y(jq-2+ijq)
              ya(ijx,ijq)=f(jx-3+ijx,jq-2+ijq)
            endif
          enddo
        else if (jx.eq.nb_x) then
          x1a(ijx)=x(jx-4+ijx)
          do ijq=1,igt
            if (jq.eq.1) then
              x2a(ijq)=y(ijq)
              ya(ijx,ijq)=f(jx-4+ijx,ijq)
            else if (jq.eq.(iq-1)) then
              x2a(ijq)=y(jq-3+ijq)
              ya(ijx,ijq)=f(jx-4+ijx,jq-3+ijq)
            else if (jq.eq.iq) then
              x2a(ijq)=y(jq-4+ijq)
              ya(ijx,ijq)=f(jx-4+ijx,jq-4+ijq)
            else
              x2a(ijq)=y(jq-2+ijq)
              ya(ijx,ijq)=f(jx-4+ijx,jq-2+ijq)
            endif
          enddo
        else
          x1a(ijx)=x(jx-2+ijx)
          do ijq=1,igt
            if (jq.eq.1) then
              x2a(ijq)=y(ijq)
              ya(ijx,ijq)=f(jx-2+ijx,ijq)
            else if (jq.eq.(iq-1)) then
              x2a(ijq)=y(jq-3+ijq)
              ya(ijx,ijq)=f(jx-2+ijx,jq-3+ijq)
            else if (jq.eq.iq) then
              x2a(ijq)=y(jq-4+ijq)
              ya(ijx,ijq)=f(jx-2+ijx,jq-4+ijq)
            else
              x2a(ijq)=y(jq-2+ijq)
              ya(ijx,ijq)=f(jx-2+ijx,jq-2+ijq)
            endif
          enddo
        endif
      enddo
      call dpolin2_bfg(x1a,x2a,ya,4,4,a,b,yy,dy)
      func = yy
      ! write (8,*) yy,dy
    endif
    ! if (func.lt.0.) func=0.
  end subroutine distributionPert


!------------------------------------------------------------
!------------- subroutine to compute a point in x and q**2 --
!------------- for the following fragmentation functions:  --
!-------------     gluon,  up   ,down,                     --
!-------------        strange , charm , bottom , top .     --
!------------- range of validity :                         --
!-------------                     1.e-03 < x < .99        --
!-------------                   log10 2  <log10 q**2 < 10 --
!------------------------------------------------------------
!-------------        authors :    p chiappetta
!-------------                     j.ph. guillet
!-------------                     m.greco
!-------------    cgw for q02=2gev2         method p.nason
!-------------    next to leading evolution
!-------------    table for lhc
!         i = 1    gluon
!         i = 2    up ( = antiup )
!         i = 3    down ( = antidown )
!         i = 4    strange ( = antistrange )
!         i = 5    charm ( = anticharm )
!         i = 6    bottom ( = antibottom )
!         i = 7    top ( = antitop )
!
!------------------------------------------------------------
  subroutine distributionNonPert_setI(z,qstar2,ifun,func)
    implicit real*8 (a-h,o-z)
    implicit integer (i-n)
!------------------------------------------------------------
!-------------   z is x and qstar2 is q square  -------------
    integer ifun
!------------------------------------------------------------
!------------ strfun specifies the desired function----------
!------------------------------------------------------------
!------------------------------------------------------------
!        ifun =  1     gluon           'gluon' (abbrev. 'gl')
!                2     up              'upval' (abbrev. 'up')
!                3     down            'doval' (abbrev. 'do')
!                4     strange         'sbar'  (abbrev. 'sb')
!                5     charm           'cbar ' (abbrev. 'cb')
!                6     bottom          'bbar ' (abbrev. 'bb')
!                7     top             'tbar ' (abbrev. 'tb')
!------------------------------------------------------------
!-------------  func is the output (parton density value)----
!------------------------------------------------------------
    parameter (kfun=6)
    dimension xqdum1(nb_x,nb_q2_4flav,kfun)
    dimension xqdum2(nb_x,nb_q2_5flav,kfun)
    include "BFGfrag_gridNonPert_setI.inc"
!!!     parameter (iqmax= max(nb_q2_4flav,nb_q2_5flav) )
!!!      not currently supported by g77
    parameter (iqmax=22)
    parameter (igt=4)

    dimension y(iqmax)
    dimension f(nb_x,iqmax)
    dimension q2(iqmax)
    dimension x1a(igt),x2a(igt),ya(igt,igt)

    func = 0d0
    xmin = x_grid(1)
    xmax = x_grid(nb_x)

    !> impose minimum q^2
    if(qstar2.le.20.26d0) then
      q2min=q2_4flav(1)
    else
      q2min=q2_5flav(1)
    endif
    if (qstar2.le.q2min) then
      write(65,*) ' q2 too small!!!!!!!!!'
      stop
    endif

    if ( (ifun < 1) .or. (ifun > 6) ) then
      write(65,*) 'error : invalid ifun'
      stop
    endif

    if(qstar2.le.20.26d0) then
      iq=nb_q2_4flav
      do j=1,iq
        do i=1,nb_x
          f(i,j)=xqdum1(i,j,ifun)
        enddo
        y(j)=y_4flav(j)
      enddo
    else
      iq=nb_q2_5flav
      do j=1,iq
        do i=1,nb_x
          f(i,j)=xqdum2(i,j,ifun)
        enddo
        y(j)=y_5flav(j)
      enddo
    endif

    !write(65,*)'y4flav',y_4flav
    !write(65,*)'y5flav',y_5flav
    !write(65,*)'y',y
    func=0d0
    if(z.gt.xmin.and.z.lt.xmax) then
      a=log10(z)
      b=log10(qstar2)
      call locate1_bfg(x,nb_x,a,jx)
      call locate1_bfg(y,iq,b,jq)
      do ijx=1,igt
        if (jx.eq.1) then
          x1a(ijx)=x(ijx)
          do ijq=1,igt
            if (jq.eq.1) then
              x2a(ijq)=y(ijq)
              ya(ijx,ijq)=f(ijx,ijq)
            else if (jq.eq.(iq-1)) then
              x2a(ijq)=y(jq-3+ijq)
              ya(ijx,ijq)=f(ijx,jq-3+ijq)
            else if (jq.eq.iq) then
              x2a(ijq)=y(jq-4+ijq)
              ya(ijx,ijq)=f(ijx,jq-4+ijq)
            else
              x2a(ijq)=y(jq-2+ijq)
              ya(ijx,ijq)=f(ijx,jq-2+ijq)
            endif
          enddo
        else if (jx.eq.(nb_x-1)) then
          x1a(ijx)=x(jx-3+ijx)
          do ijq=1,igt
            if (jq.eq.1) then
              x2a(ijq)=y(ijq)
              ya(ijx,ijq)=f(jx-3+ijx,ijq)
            else if (jq.eq.(iq-1)) then
              x2a(ijq)=y(jq-3+ijq)
              ya(ijx,ijq)=f(jx-3+ijx,jq-3+ijq)
            else if (jq.eq.iq) then
              x2a(ijq)=y(jq-4+ijq)
              ya(ijx,ijq)=f(jx-3+ijx,jq-4+ijq)
            else
              x2a(ijq)=y(jq-2+ijq)
              ya(ijx,ijq)=f(jx-3+ijx,jq-2+ijq)
            endif
          enddo
        else if (jx.eq.nb_x) then
          x1a(ijx)=x(jx-4+ijx)
          do ijq=1,igt
            if (jq.eq.1) then
              x2a(ijq)=y(ijq)
              ya(ijx,ijq)=f(jx-4+ijx,ijq)
            else if (jq.eq.(iq-1)) then
              x2a(ijq)=y(jq-3+ijq)
              ya(ijx,ijq)=f(jx-4+ijx,jq-3+ijq)
            else if (jq.eq.iq) then
              x2a(ijq)=y(jq-4+ijq)
              ya(ijx,ijq)=f(jx-4+ijx,jq-4+ijq)
            else
              x2a(ijq)=y(jq-2+ijq)
              ya(ijx,ijq)=f(jx-4+ijx,jq-2+ijq)
            endif
          enddo
        else
          x1a(ijx)=x(jx-2+ijx)
          do ijq=1,igt
            if (jq.eq.1) then
              x2a(ijq)=y(ijq)
              ya(ijx,ijq)=f(jx-2+ijx,ijq)
            else if (jq.eq.(iq-1)) then
              x2a(ijq)=y(jq-3+ijq)
              ya(ijx,ijq)=f(jx-2+ijx,jq-3+ijq)
            else if (jq.eq.iq) then
              x2a(ijq)=y(jq-4+ijq)
              ya(ijx,ijq)=f(jx-2+ijx,jq-4+ijq)
            else
              x2a(ijq)=y(jq-2+ijq)
              ya(ijx,ijq)=f(jx-2+ijx,jq-2+ijq)
            endif
          enddo
        endif
      enddo
      call dpolin2_bfg(x1a,x2a,ya,4,4,a,b,yy,dy)
      func = yy
      ! write (8,*) yy,dy
    endif
    ! if (func.lt.0.) func=0.
  end subroutine distributionNonPert_setI


!------------------------------------------------------------
!------------- subroutine to compute a point in x and q**2 --
!------------- for the following fragmentation functions:  --
!-------------     gluon,  up   ,down,                     --
!-------------        strange , charm , bottom , top .     --
!------------- range of validity :                         --
!-------------                     1.e-03 < x < .99        --
!-------------                   log10 2  <log10 q**2 < 10 --
!------------------------------------------------------------
!-------------        authors :    p chiappetta
!-------------                     j.ph. guillet
!-------------                     m.greco
!-------------    cgw for q02=2gev2         method p.nason
!-------------    next to leading evolution
!-------------    table for lhc
!         i = 1    gluon
!         i = 2    up ( = antiup )
!         i = 3    down ( = antidown )
!         i = 4    strange ( = antistrange )
!         i = 5    charm ( = anticharm )
!         i = 6    bottom ( = antibottom )
!         i = 7    top ( = antitop )
!
!------------------------------------------------------------
  subroutine distributionNonPert_setII(z,qstar2,ifun,func)
    implicit real*8 (a-h,o-z)
    implicit integer (i-n)
!------------------------------------------------------------
!-------------   z is x and qstar2 is q square  -------------
    integer ifun
!------------------------------------------------------------
!------------ strfun specifies the desired function----------
!------------------------------------------------------------
!------------------------------------------------------------
!        ifun =  1     gluon           'gluon' (abbrev. 'gl')
!                2     up              'upval' (abbrev. 'up')
!                3     down            'doval' (abbrev. 'do')
!                4     strange         'sbar'  (abbrev. 'sb')
!                5     charm           'cbar ' (abbrev. 'cb')
!                6     bottom          'bbar ' (abbrev. 'bb')
!                7     top             'tbar ' (abbrev. 'tb')
!------------------------------------------------------------
!-------------  func is the output (parton density value)----
!------------------------------------------------------------
    parameter (kfun=6)
    dimension xqdum1(nb_x,nb_q2_4flav,kfun)
    dimension xqdum2(nb_x,nb_q2_5flav,kfun)
    include "BFGfrag_gridNonPert_setII.inc"
!!!     parameter (iqmax= max(nb_q2_4flav,nb_q2_5flav) )
!!!      not currently supported by g77
    parameter (iqmax=22)
    parameter (igt=4)

    dimension y(iqmax)
    dimension f(nb_x,iqmax)
    dimension q2(iqmax)
    dimension x1a(igt),x2a(igt),ya(igt,igt)

    func = 0d0
    xmin = x_grid(1)
    xmax = x_grid(nb_x)

    !> impose minimum q^2
    if(qstar2.le.20.26d0) then
      q2min=q2_4flav(1)
    else
      q2min=q2_5flav(1)
    endif
    if (qstar2.le.q2min) then
      write(65,*) ' q2 too small!!!!!!!!!'
      stop
    endif

    if ( (ifun < 1) .or. (ifun > 6) ) then
      write(65,*) 'error : invalid ifun'
      stop
    endif

    if(qstar2.le.20.26d0) then
      iq=nb_q2_4flav
      do j=1,iq
        do i=1,nb_x
          f(i,j)=xqdum1(i,j,ifun)
        enddo
        y(j)=y_4flav(j)
      enddo
    else
      iq=nb_q2_5flav
      do j=1,iq
        do i=1,nb_x
          f(i,j)=xqdum2(i,j,ifun)
        enddo
        y(j)=y_5flav(j)
      enddo
    endif

    !write(65,*)'y4flav',y_4flav
    !write(65,*)'y5flav',y_5flav
    !write(65,*)'y',y
    func=0d0
    if(z.gt.xmin.and.z.lt.xmax) then
      a=log10(z)
      b=log10(qstar2)
      call locate1_bfg(x,nb_x,a,jx)
      call locate1_bfg(y,iq,b,jq)
      do ijx=1,igt
        if (jx.eq.1) then
          x1a(ijx)=x(ijx)
          do ijq=1,igt
            if (jq.eq.1) then
              x2a(ijq)=y(ijq)
              ya(ijx,ijq)=f(ijx,ijq)
            else if (jq.eq.(iq-1)) then
              x2a(ijq)=y(jq-3+ijq)
              ya(ijx,ijq)=f(ijx,jq-3+ijq)
            else if (jq.eq.iq) then
              x2a(ijq)=y(jq-4+ijq)
              ya(ijx,ijq)=f(ijx,jq-4+ijq)
            else
              x2a(ijq)=y(jq-2+ijq)
              ya(ijx,ijq)=f(ijx,jq-2+ijq)
            endif
          enddo
        else if (jx.eq.(nb_x-1)) then
          x1a(ijx)=x(jx-3+ijx)
          do ijq=1,igt
            if (jq.eq.1) then
              x2a(ijq)=y(ijq)
              ya(ijx,ijq)=f(jx-3+ijx,ijq)
            else if (jq.eq.(iq-1)) then
              x2a(ijq)=y(jq-3+ijq)
              ya(ijx,ijq)=f(jx-3+ijx,jq-3+ijq)
            else if (jq.eq.iq) then
              x2a(ijq)=y(jq-4+ijq)
              ya(ijx,ijq)=f(jx-3+ijx,jq-4+ijq)
            else
              x2a(ijq)=y(jq-2+ijq)
              ya(ijx,ijq)=f(jx-3+ijx,jq-2+ijq)
            endif
          enddo
        else if (jx.eq.nb_x) then
          x1a(ijx)=x(jx-4+ijx)
          do ijq=1,igt
            if (jq.eq.1) then
              x2a(ijq)=y(ijq)
              ya(ijx,ijq)=f(jx-4+ijx,ijq)
            else if (jq.eq.(iq-1)) then
              x2a(ijq)=y(jq-3+ijq)
              ya(ijx,ijq)=f(jx-4+ijx,jq-3+ijq)
            else if (jq.eq.iq) then
              x2a(ijq)=y(jq-4+ijq)
              ya(ijx,ijq)=f(jx-4+ijx,jq-4+ijq)
            else
              x2a(ijq)=y(jq-2+ijq)
              ya(ijx,ijq)=f(jx-4+ijx,jq-2+ijq)
            endif
          enddo
        else
          x1a(ijx)=x(jx-2+ijx)
          do ijq=1,igt
            if (jq.eq.1) then
              x2a(ijq)=y(ijq)
              ya(ijx,ijq)=f(jx-2+ijx,ijq)
            else if (jq.eq.(iq-1)) then
              x2a(ijq)=y(jq-3+ijq)
              ya(ijx,ijq)=f(jx-2+ijx,jq-3+ijq)
            else if (jq.eq.iq) then
              x2a(ijq)=y(jq-4+ijq)
              ya(ijx,ijq)=f(jx-2+ijx,jq-4+ijq)
            else
              x2a(ijq)=y(jq-2+ijq)
              ya(ijx,ijq)=f(jx-2+ijx,jq-2+ijq)
            endif
          enddo
        endif
      enddo
      call dpolin2_bfg(x1a,x2a,ya,4,4,a,b,yy,dy)
      func = yy
      ! write (8,*) yy,dy
    endif
    ! if (func.lt.0.) func=0.
  end subroutine distributionNonPert_setII



  subroutine locate1_bfg(xx,n,x,j)
    implicit real*8 (a-h,o-z)
    implicit integer (i-n)
    dimension xx(n)
    ! write(65,*) 'in locate1_bfg n',n
    ! write(65,*) 'xx ',xx
    ! write(65,*) 'x ',x
    jl=0
    ju=n+1
    10 continue
    if(ju-jl.gt.1)then
      jm=(ju+jl)/2
      if((xx(n).gt.xx(1)).eqv.(x.gt.xx(jm)))then
        jl=jm
      else
        ju=jm
      endif
    go to 10
    endif
    j=jl
    return
  end subroutine locate1_bfg


  subroutine dpolint_bfg(xa,ya,n,x,y,dy)
    implicit real*8 (a-h,o-z)
    implicit integer (i-n)
    parameter (nmax=10)
    dimension xa(n),ya(n),c(nmax),d(nmax)
    ns=1
    dif=abs(x-xa(1))
    do 11 i=1,n
      dift=abs(x-xa(i))
      if (dift.lt.dif) then
        ns=i
        dif=dift
      endif
      c(i)=ya(i)
      d(i)=ya(i)
    11 continue
    y=ya(ns)
    ns=ns-1
    do 13 m=1,n-1
      do 12 i=1,n-m
        ho=xa(i)-x
        hp=xa(i+m)-x
        w=c(i+1)-d(i)
        den=ho-hp
        if(den.eq.0.d0) then
          write(65,*) ' stop in polint den',den
          write(65,*) ' probably problem with compiler'
          write(65,*) ' has to be static in distr* etc '
          ! write(65,*) ' n ',n
          ! write(65,*)  ' xa',xa
          ! write(65,*)  ' x',x
          ! write(65,*)  ' m',m,' i',i
          stop
        endif
        den=w/den
        d(i)=hp*den
        c(i)=ho*den
      12 continue
      if (2*ns.lt.n-m)then
        dy=c(ns+1)
      else
        dy=d(ns)
        ns=ns-1
      endif
      y=y+dy
    13 continue
    return
  end subroutine dpolint_bfg


  subroutine dpolin2_bfg(x1a,x2a,ya,m,n,x1,x2,y,dy)
    implicit real*8 (a-h,o-z)
    implicit integer (i-n)
    parameter (nmax=20,mmax=20)
    dimension x1a(m),x2a(n),ya(m,n),yntmp(nmax),ymtmp(mmax)
    ! write(65,*) 'in dpolin2_bfg m',m,' n ',n
    ! write(65,*) 'x1a ',x1a
    ! write(65,*) 'x2a ',x2a
    ! write(65,*)' ya ', ya
    ! write(65,*) 'x1 ',x1
    ! write(65,*) 'x2 ',x2
    do 12 j=1,m
      do 11 k=1,n
        yntmp(k)=ya(j,k)
      11 continue
      call dpolint_bfg(x2a,yntmp,n,x2,ymtmp(j),dy)
    12 continue
    call dpolint_bfg(x1a,ymtmp,m,x1,y,dy)
    return
  end subroutine dpolin2_bfg


end module BFGfrag_mod
