module CCSFDIS_mod
  use Kinds_mod, only: wp
  implicit none
  private
  public :: CCSF, CC_F3, CC_F2, CC_FL

  real(kind=wp), parameter :: rnup = 2._wp
  real(kind=wp), parameter :: rndown = 3._wp
  real(kind=wp), parameter :: rnf = 5._wp
  real(kind=wp), parameter :: gevfb = 389.37936e9_wp

contains

  real(kind=wp) function CCSF(iord, ix, xi)
    use Process_mod, only: iboson_proc
    use Constants_mod, only: pi
    use Scales_mod
    use KinData_mod, only: xreg1_kin, s4
    use CoeffFunc_av
    use Channels_mod, only: ip1_chan
    use DIS_mod, only: value_q2_dis, value_y_dis
    integer, intent(in) :: ix, iord
    integer idebug
    real(kind=wp), intent(in) :: xi
    integer nflav
    integer i
    !> bino function
    real(kind=wp) :: bino
    real(kind=wp) :: amz, as(nMax_scl), stw, ewfac
    real(kind=wp) :: amzW, CCfac(nMax_scl), emw, ewwidth
    real(kind=wp) :: c2qcox, c2qco1, c2gcox, c2qpscox, c2qdel
    real(kind=wp) :: c2g_new, c2qps_new
    real(kind=wp) :: c3qcox, c3qco1, c3gcox, c3qpscox, c3qdel
    real(kind=wp) :: c3g_new, c3qps_new
    real(kind=wp) :: clqcox, clqco1, clgcox, clqpscox, clqdel
    real(kind=wp) :: cLg_new, cLqps_new
    real(kind=wp) :: f2(nMax_scl), fl(nMax_scl), f3(nMax_scl), dsig(nMax_scl)
    real(kind=wp) :: partons(nMax_scl), facnorm(nMax_scl)
    real(kind=wp) :: xbj, q2, x1curr, x2curr, y
    real(kind=wp) :: rma2, rmb2, rm2, shat, Jxtoy
    real(kind=wp) :: Yp, Ym, dboson, qsign, flavsign
    complex*16 :: zewfac(4), zewnull(4)
    integer :: ipass
    common/parfrac/x1curr, x2curr
!$omp threadprivate(/parfrac/)
    common/eweakZ/amz, zewfac, zewnull
!$omp threadprivate(/eweakZ/)
    common/BZFlav/nflav
!$omp threadprivate(/BZFlav/)
    common/pmasses/rma2, rmb2, rm2(1:7), shat
!$omp threadprivate(/pmasses/)
    common/eweakW/amzW, stw, ewfac
!$omp threadprivate(/eweakW/)
    common/Wmass/emw, ewwidth

    if (ix .ne. 1 .and. ix .ne. 3) then
      write (*, *) "SOMETHING IS WRONG WITH ix ", ix
      stop
    end if

    CCSF = 0._wp

    !> dboson = +1 for W+, -1 for W-, hence - sign here w.r.t iboson
    dboson = -1._wp*iboson_proc !> cast iboson to float

    !ecuts -> return 0 on failure
    ipass = 1
    call ecuts_dis1j(0, 4, ipass)
    if (ipass .eq. 0) return

    call getqcdnorm(ix, partons, facnorm, 4, ipass, 0)
    if (ipass .eq. 0) return

    call eweakV1qq(nflav, s4(2, 3))

    f3 = 0._wp
    f2 = 0._wp
    fl = 0._wp
    dsig = 0._wp

    q2 = value_q2_dis
    y = value_y_dis
    xbj = x1curr
    as = alphas_scl(:)

    do i = 1, nAct_scl
      !> C2/F2
      f2(i) = CC_F2(ip1_chan, ix, iord, muR2_scl(i), muF2_scl(i), q2, xbj, xreg1_kin, dboson, xi)
      !> CL/FL
      fl(i) = CC_FL(ip1_chan, ix, iord, muR2_scl(i), muF2_scl(i), q2, xbj, xreg1_kin, dboson, xi)
      !> C3/F3
      f3(i) = CC_F3(ip1_chan, ix, iord, muR2_scl(i), muF2_scl(i), q2, xbj, xreg1_kin, dboson, xi)
    end do

    !> Global factor for CC DIS
    CCfac(:) = (as(:)/4._wp/pi)**iord*abs(zewfac(1))**2*(pi/2._wp/stw**2._wp)/8._wp*amz**2._wp

    !> Jacobian factor to switch from y to x sampling
    Jxtoy = q2/shat/xbj

    Yp = 1._wp + (1._wp - y)**2._wp
    Ym = 1._wp - (1._wp - y)**2._wp

    dsig(:) = Jxtoy*(Yp*f2(:) - dboson*Ym*f3(:) - y**2._wp*fl(:))/q2**2._wp/y*CCfac

    !> 8*pi is the phase space integal that would otherwise been performed twice
    dsig(:) = gevfb*dsig(:)*8._wp*pi*shat

    CCSF = bino(ix, partons, dsig*facnorm)

    if (CCSF .ne. CCSF) then ! NaN DEBUG
      print *, CCSF, y, q2, f2(1), f3(1), fl(1)
      stop
    end if

  end function CCSF

  real(kind=wp) function CC_F3(ip, ix, iord, renscale, facscale, q2, xbj, xreg, wsign, xi)
    !> Charged-current F3 structure function. Incoming lepton charge given by wsign
    use CoeffFunc_av
    integer, intent(in) :: ip, ix, iord
    real(kind=wp), intent(in) :: renscale, facscale, q2, xbj, xreg, xi, wsign
    real(kind=wp) :: c3qcox, c3qdel, c3qco1
    real(kind=wp) :: fsign, qsign, c3psign, c3vsign
    real(kind=wp) :: c3q_plus, c3q_valence, c3q_diff

    CC_F3 = 0._wp
    c3qdel = 0._wp
    c3qco1 = 0._wp
    c3qcox = 0._wp

    fsign = get_flavsign(ip)
    qsign = get_qsign(ip)
    c3psign = -wsign*fsign
    c3vsign = qsign

    !> This is the delta part
    if (ix .eq. 1) then
      if ((ip .ne. 0) .and. (abs(ip) .ne. 30)) then !> u,d,ub,db init
        c3qdel = c3psign*c3q_plus(iord, xbj, 0, renscale, facscale, q2, 1) &  !> C3+
             & + c3vsign*c3q_valence(iord, xbj, 0, renscale, facscale, q2, 1) !> C3v
        c3qco1 = c3psign*c3q_plus(iord, xreg, 2, renscale, facscale, q2, 1) &  !> C3+
             & + c3vsign*c3q_valence(iord, xreg, 2, renscale, facscale, q2, 1) !> C3v
        CC_F3 = (c3qdel + c3qco1*(1._wp - xbj))
      end if
      !> This is the convolution
    elseif (ix .eq. 3) then
      call inithpls5(xi)
      if ((ip .ne. 0) .and. (abs(ip) .ne. 30)) then !> u,d,ub,db init
        c3qcox = c3psign*c3q_plus(iord, xi, 1, renscale, facscale, q2, 1) &  !> C3+
             & + c3vsign*c3q_valence(iord, xi, 1, renscale, facscale, q2, 1) !> C3v
        CC_F3 = (c3qcox)/xi
      end if
      !> Jacobian factor for rreg1 pick
      !> + compensates for xi in ix.eq.3
      CC_F3 = CC_F3*(1._wp - xbj)*xi
    end if
    CC_F3 = CC_F3*xbj

  end function CC_F3

  real(kind=wp) function CC_FL(ip, ix, iord, renscale, facscale, q2, xbj, xreg, wsign, xi)
    !> Charged-current FL structure function. Incoming lepton charge given by wsign
    use CoeffFunc_av
    integer, intent(in) :: ip, ix, iord
    real(kind=wp), intent(in) :: renscale, facscale, q2, xbj, xreg, xi, wsign
    real(kind=wp) :: clqcox, clqdel, clqco1, clgcox, clqpscox
    real(kind=wp) :: fsign, qsign
    real(kind=wp) :: clq_plus, clq_minus, clg_new, clqps_new
    real(kind=wp) :: cjmsign

    CC_FL = 0._wp
    clqdel = 0._wp
    clqco1 = 0._wp
    clqcox = 0._wp
    clgcox = 0._wp
    clqpscox = 0._wp

    fsign = get_flavsign(ip)
    qsign = get_qsign(ip)
    cjmsign = -wsign*fsign*qsign

    !> This is the delta part
    if (ix .eq. 1) then
      if ((ip .ne. 0) .and. (abs(ip) .ne. 30)) then
        !> c+ = c- = c_EM (clq) up to and including O(alpha)
        clqdel = cjmsign*clq_minus(iord, xbj, 0, renscale, facscale, q2, 1)  & ! Cl-
             & + clq_plus(iord, xbj, 0, renscale, facscale, q2, 1)     ! Cl+
        clqco1 = cjmsign*clq_minus(iord, xreg, 2, renscale, facscale, q2, 1) & ! Cl-
             & + clq_plus(iord, xreg, 2, renscale, facscale, q2, 1)    ! Cl+
        CC_FL = (clqdel + clqco1*(1._wp - xbj))
      end if
      !> This is the convolution
    elseif (ix .eq. 3) then
      call inithpls5(xi)
      if (ip .eq. 0) then !> g init
        !> factor of 4 for final state degeneracy
        clgcox = clg_new(iord, xi, 1, renscale, facscale, q2, 1)*4._wp ! can link to EM clg
        CC_FL = clgcox/xi
      elseif (abs(ip) .ne. 30) then !> u,d,ub,db init
        !> c+ = c- = c_EM (clq) up to and including O(alpha)
        clqcox = cjmsign*clq_minus(iord, xi, 1, renscale, facscale, q2, 1) & ! Cl-
             & + clq_plus(iord, xi, 1, renscale, facscale, q2, 1)    ! Cl+
        CC_FL = (clqcox)/xi
      else !> Q,Qb init
        !> factor of 4 for final state degeneracy
        clqpscox = clqps_new(iord, xi, 1, renscale, facscale, q2, 1)*4._wp ! can link to EM clqps
        CC_FL = (clqpscox)/xi
      end if
      !> Jacobian factor for rreg1 pick
      !> + compensates for xi in ix.eq.3
      CC_FL = CC_FL*(1._wp - xbj)*xi
    end if
    CC_FL = CC_FL*xbj

  end function CC_FL

  real(kind=wp) function CC_F2(ip, ix, iord, renscale, facscale, q2, xbj, xreg, wsign, xi)
    !> Charged-current F2 structure function. Incoming lepton charge given by wsign
    use CoeffFunc_av
    integer, intent(in) :: ip, ix, iord
    real(kind=wp), intent(in) :: renscale, facscale, q2, xbj, xreg, xi, wsign
    real(kind=wp) :: c2qcox, c2qdel, c2qco1, c2gcox, c2qpscox
    real(kind=wp) :: fsign, qsign
    real(kind=wp) :: c2q_plus, c2q_minus, c2g_new, c2qps_new
    real(kind=wp) :: cjmsign

    CC_F2 = 0._wp
    c2qdel = 0._wp
    c2qco1 = 0._wp
    c2gcox = 0._wp
    c2qcox = 0._wp
    c2qpscox = 0._wp

    fsign = get_flavsign(ip)
    qsign = get_qsign(ip)
    cjmsign = -wsign*fsign*qsign

    !> This is the delta part
    if (ix .eq. 1) then
      if ((ip .ne. 0) .and. (abs(ip) .ne. 30)) then
        !> c+ = c- = c_EM (c2q) up to and including O(alpha)
        c2qdel = cjmsign*c2q_minus(iord, xbj, 0, renscale, facscale, q2, 1) & ! C2-
             & + c2q_plus(iord, xbj, 0, renscale, facscale, q2, 1) ! C2+
        c2qco1 = cjmsign*c2q_minus(iord, xreg, 2, renscale, facscale, q2, 1) & !C2-
             & + c2q_plus(iord, xreg, 2, renscale, facscale, q2, 1) !C2+
        CC_F2 = (c2qdel + c2qco1*(1._wp - xbj))
      end if
      !> This is the convolution
    elseif (ix .eq. 3) then
      call inithpls5(xi)
      if (ip .eq. 0) then !> g init
        !> factor of 4 for final state degeneracy
        c2gcox = c2g_new(iord, xi, 1, renscale, facscale, q2, 1)*4._wp ! can link to EM c2g
        CC_F2 = c2gcox/xi
      elseif (abs(ip) .ne. 30) then !> u,d,ub,db init
        !> c+ = c- = c_EM (c2q) up to and including O(alpha)
        c2qcox = cjmsign*c2q_minus(iord, xi, 1, renscale, facscale, q2, 1) & ! C2-
             & + c2q_plus(iord, xi, 1, renscale, facscale, q2, 1) ! C2+
        CC_F2 = (c2qcox)/xi
      else !> Q,Qb init
        !> factor of 4 for final state degeneracy
        c2qpscox = c2qps_new(iord, xi, 1, renscale, facscale, q2, 1)*4._wp ! can link to EM clqps
        CC_F2 = (c2qpscox)/xi
      end if
      !> Jacobian factor for rreg1 pick
      !> + compensates for xi in ix.eq.3
      CC_F2 = CC_F2*(1._wp - xbj)*xi
    end if
    CC_F2 = CC_F2*xbj
  end function CC_F2

  real(kind=wp) function get_flavsign(ip)
    integer :: ip
    !> flavsign => u,ub: +1
    !>             d,db: -1
    !>             g:     0
    !>             Q,Qb:  0
    select case (abs(ip))
    case (20)
      get_flavsign = 1._wp
    case (40)
      get_flavsign = -1._wp
    case (0, 30)
      get_flavsign = 0._wp
    case default
      print *, "Unknown ip for flavsign. Exiting now", ip
      stop
    end select

  end function get_flavsign

  real(kind=wp) function get_qsign(ip)
    integer :: ip
    !> Set up sign conventions
    get_qsign = 0._wp
    !> qsign => q:  +1
    !>          qb: -1
    !>          g:   0
    if (ip .lt. 0._wp) then
      get_qsign = -1._wp
    else if (ip .gt. 0._wp) then
      get_qsign = 1._wp
    end if
  end function get_qsign

end module CCSFDIS_mod

