!-------------------------------------------------------------------------------
! NNLOJET
!-------------------------------------------------------------------------------
!
! MODULE: store & manage elecric chages of active amplitues [external members: ``]
!
! DESCRIPTION:
!> Module providing the storage of charge values for EW subtraction terms
!-------------------------------------------------------------------------------


!-------------------------------------------------------------------------------
module Charges_EW_mod
!-------------------------------------------------------------------------------
  implicit none
  public

  double precision :: Qf1, Qf2, Ql
!$OMP THREADPRIVATE(Qf1, Qf2, Ql)

  integer :: if1, if2
!$OMP THREADPRIVATE(if1, if2)

  ! double precision, parameter, dimension(-6:6) :: enQ =  &
  !   &  (/ -2d0/3d0, +1d0/3d0, -2d0/3d0, +1d0/3d0, -2d0/3d0, +1d0/3d0, 0d0,  &
  !   &     -1d0/3d0, +2d0/3d0, -1d0/3d0, +2d0/3d0, -1d0/3d0, +2d0/3d0 /)

contains


  !-----------------------------------------------------------------------------
  !> @brief
  !> parton charges
  !
  !> @param[in] id LHAPDF id's for partons
  !> @return charges
  !-----------------------------------------------------------------------------
  pure double precision function Qq(id)
    integer, intent(in) :: id
    select case(id)
      case(1,3,5)
        Qq = -1d0/3d0
      case(-1,-3,-5)
        Qq = +1d0/3d0
      case(2,4,6)
        Qq = +2d0/3d0
      case(-2,-4,-6)
        Qq = -2d0/3d0
      case default
        Qq = 0d0
    end select
  end function Qq


  !-----------------------------------------------------------------------------
  !> @brief
  !> update the charge values for the current amplitude
  !
  !> @param[in] amp_type ME type by counting # quark lines (1 = B, 2 = C, ...)
  !-----------------------------------------------------------------------------
  subroutine update_charges(amp_type)
    integer, intent(in) :: amp_type
    integer :: nfB1
    common /BZFlav/ nfB1
!$omp threadprivate(/BZFlav/)
    integer :: nfC1,nfC2
    common /CZFlav/ nfC1, nfC2
!$omp threadprivate(/CZFlav/)

    Ql= -1d0

    select case(amp_type)
      case(1)  ! B-type
        Qf1 = Qq(nfB1)
        Qf2 = 0d0
        if1 = nfB1
        if2 = 0
      case(2)
        Qf1 = Qq(nfC1)
        Qf2 = Qq(nfC2)
        if1 = nfC1
        if2 = nfC2
      case default
        print*, "update_charges: unknown amplitude type:", amp_type
        stop
        Qf1 = 0d0
        Qf2 = 0d0
        if1 = 0
        if2 = 0
    end select

  end subroutine update_charges


end module Charges_EW_mod





