!-------------------------------------------------------------------------------
! NNLOJET
!-------------------------------------------------------------------------------
!
! MODULE: DIS [external members: `_dis`]
!
! DESCRIPTION:
!> Module collecting observable implementations for DIS
!-------------------------------------------------------------------------------

!-------------------------------------------------------------------------------
module EvalDIS_mod
!-------------------------------------------------------------------------------
  use DIS_mod
  use EvalFuncs_mod
  implicit none
  public

contains

  !------------------------------!
  ! getters for global variables !
  !------------------------------!

  real(kind=wp) function eval_Q2_dis(npar)
    integer, intent(in) :: npar
    eval_Q2_dis = value_q2_dis
  end function eval_Q2_dis

  real(kind=wp) function eval_sqrtQ2_dis(npar)
    integer, intent(in) :: npar
    eval_sqrtQ2_dis = value_q_dis
  end function eval_sqrtQ2_dis

  real(kind=wp) function eval_Ee_dis(npar)
    integer, intent(in) :: npar
    eval_Ee_dis = value_Ee_dis
  end function eval_Ee_dis

  real(kind=wp) function eval_thetae_dis(npar)
    integer, intent(in) :: npar
    eval_thetae_dis = value_thetae_dis
  end function eval_thetae_dis

  real(kind=wp) function eval_x_dis(npar)
    integer, intent(in) :: npar
    eval_x_dis = value_x_dis
  end function eval_x_dis

  real(kind=wp) function eval_y_dis(npar)
    integer, intent(in) :: npar
    eval_y_dis = value_y_dis
  end function eval_y_dis

  real(kind=wp) function eval_W2_dis(npar)
    integer, intent(in) :: npar
    eval_W2_dis = value_W2_dis
  end function eval_W2_dis

  real(kind=wp) function eval_W_dis(npar)
    integer, intent(in) :: npar
    eval_W_dis = value_W_dis
  end function eval_W_dis

  real(kind=wp) function eval_cosgammah_dis(npar)
    integer, intent(in) :: npar
    eval_cosgammah_dis = value_cosgammah_dis
  end function eval_cosgammah_dis

  real(kind=wp) function eval_nu_dis(npar)
    integer, intent(in) :: npar
    eval_nu_dis = value_disnu_dis
  end function eval_nu_dis

  real(kind=wp) function eval_resrho_dis(npar)
    integer, intent(in) :: npar
    eval_resrho_dis = value_resrho_dis
  end function eval_resrho_dis

  real(kind=wp) function eval_y_disg(npar)
    integer, intent(in) :: npar
    eval_y_disg = value_y_disg
  end function eval_y_disg

  !-----------------!
  ! DIS observables !
  !-----------------!

  function eval_DIS_Taxis(npar, imem) result(res)
    real(kind=wp) :: res
    integer, intent(in) :: npar, imem
    !> cache for memoization:
    !>   [ Thrust_c(1), JB_c(2) ]
    real(kind=wp), dimension(2), save :: &
      cache_ = [0._wp, 0._wp]
    logical, save :: qcached_ = .false.
!$OMP THREADPRIVATE(cache_, qcached_)
    !>-----
    type(Kin_t), pointer :: kd
    real(kind=wp) :: Tpar, Tn, Bpar, Bn, top, bot
    real(kind=wp), dimension(3, npar - 3) :: pp
    real(kind=wp), dimension(3, 2**(npar - 4)) :: pos_Taxes
    real(kind=wp), dimension(3) :: dis_Taxis, p_cross_Taxis
    integer :: i, j, ncount, n, iTaxis

    res = 0._wp

    if (qcached_ .and. (imem > 0)) then
      res = cache_(imem)
      return
    end if
    if (imem == 0) then
      qcached_ = .false.
      return
    end if

    kd => kin(npar)

    !> vector of thrust axis candidates
    pos_Taxes = 0._wp
    dis_Taxis = 0._wp
    !> vector of particles in current hemisphere
    pp = 0._wp
    ncount = 0

    do j = 4, npar
      ! Select only particles in the current hemisphere (pz > 0)
      if (kd%p(3, j) > 0._wp) then
        ncount = ncount + 1
        do i = 1, 3
          pp(i, ncount) = kd%p(i, j)
        end do
      end if
    end do

    if (ncount .eq. 0) then
      cache_ = [-1._wp, -1._wp]
      qcached_ = .true.
      res = cache_(imem)
      return
    elseif (ncount .eq. 1) then
      cache_ = [0._wp, 0._wp]
      qcached_ = .true.
      res = cache_(imem)
      return
      !> only from here does the Taxis start to have nontrivial importance for T and B
    elseif (ncount .eq. 2) then
      do i = 1, 3
        pos_Taxes(i, 1) = pp(i, 1) + pp(i, 2)
        pos_Taxes(i, 2) = pp(i, 1) - pp(i, 2)
      end do
    elseif (ncount .eq. 3) then
      do i = 1, 3
        pos_Taxes(i, 1) = pp(i, 1) + pp(i, 2) + pp(i, 3)
        pos_Taxes(i, 2) = pp(i, 1) + pp(i, 2) - pp(i, 3)
        pos_Taxes(i, 3) = pp(i, 1) - pp(i, 2) + pp(i, 3)
        pos_Taxes(i, 4) = pp(i, 1) - pp(i, 2) - pp(i, 3)
      end do
    elseif (ncount .eq. 4) then
      do i = 1, 3
        pos_Taxes(i, 1) = pp(i, 1) + pp(i, 2) + pp(i, 3) + pp(i, 4)
        pos_Taxes(i, 2) = pp(i, 1) + pp(i, 2) + pp(i, 3) - pp(i, 4)
        pos_Taxes(i, 3) = pp(i, 1) + pp(i, 2) - pp(i, 3) + pp(i, 4)
        pos_Taxes(i, 4) = pp(i, 1) - pp(i, 2) + pp(i, 3) + pp(i, 4)
        pos_Taxes(i, 5) = pp(i, 1) + pp(i, 2) - pp(i, 3) - pp(i, 4)
        pos_Taxes(i, 6) = pp(i, 1) - pp(i, 2) + pp(i, 3) - pp(i, 4)
        pos_Taxes(i, 7) = pp(i, 1) - pp(i, 2) - pp(i, 3) + pp(i, 4)
        pos_Taxes(i, 8) = pp(i, 1) - pp(i, 2) - pp(i, 3) - pp(i, 4)
      end do
    else
      write (*, *) "Not supported for ncount = ", ncount
      return
    end if

    do n = 1, 2**(ncount - 1)
      pos_Taxes(:, n) = pos_Taxes(:, n)/sqrt(pos_Taxes(1, n)**2 + pos_Taxes(2, n)**2 + pos_Taxes(3, n)**2)
    end do

    !> Identify thrust axis and calculate thrust
    Tpar = 0._wp
    Tn = 0._wp
    iTaxis = 0

    do n = 1, 2**(ncount - 1)
      top = 0._wp
      bot = 0._wp
      do i = 1, ncount
        top = top + sqrt(dot_eucl(pp(:, i), pos_Taxes(:, n))**2)
        bot = bot + sqrt(pp(1, i)**2 + pp(2, i)**2 + pp(3, i)**2)
      end do
      Tn = top/bot
      if (Tn >= Tpar) then
        Tpar = Tn
        iTaxis = n
      end if
    end do

    dis_Taxis = pos_Taxes(:, iTaxis)

    !> Calculate jet broadening now
    Bpar = 0._wp
    p_cross_Taxis = 0._wp
    top = 0._wp
    bot = 0._wp
    do i = 1, ncount
      p_cross_Taxis(1) = pp(2, i)*dis_Taxis(3) - pp(3, i)*dis_Taxis(2)
      p_cross_Taxis(2) = pp(3, i)*dis_Taxis(1) - pp(1, i)*dis_Taxis(3)
      p_cross_Taxis(3) = pp(1, i)*dis_Taxis(2) - pp(2, i)*dis_Taxis(1)
      top = top + sqrt(p_cross_Taxis(1)**2 + p_cross_Taxis(2)**2 + p_cross_Taxis(3)**2)
      bot = bot + sqrt(pp(1, i)**2 + pp(2, i)**2 + pp(3, i)**2)
    end do
    Bpar = top/bot

    !> store in cache & return  [Thrust_c, JB_c]
    cache_ = [1._wp - Tpar, Bpar]

    qcached_ = .true.
    res = cache_(imem)

  end function eval_DIS_Taxis

  real(kind=wp) function eval_dis_dscl(npar)
    integer, intent(in) :: npar
    real(kind=wp) :: ptavg_j12, q2
    logical :: valid
    ptavg_j12 = eval_ptavg_j12(npar)
    q2 = value_q2_dis
    eval_dis_dscl = sqrt((q2 + ptavg_j12**2)/2._wp)
  end function eval_dis_dscl

  real(kind=wp) function eval_dis_dscl3j(npar)
    integer, intent(in) :: npar
    real(kind=wp) :: ptavg_j123, q2
    logical :: valid
    ptavg_j123 = eval_ptavg_j123(npar)
    q2 = value_q2_dis
    eval_dis_dscl3j = sqrt((q2 + ptavg_j123**2)/2._wp)
  end function eval_dis_dscl3j

  real(kind=wp) function eval_dis_dsclj1(npar)
    integer, intent(in) :: npar
    real(kind=wp) :: pt_jet, q2
    logical :: valid
    pt_jet = eval_pt_jet(npar, 1)
    q2 = value_q2_dis
    eval_dis_dsclj1 = sqrt((q2 + pt_jet**2)/2._wp)
  end function eval_dis_dsclj1

  real(kind=wp) function eval_dis_dsclj2(npar)
    integer, intent(in) :: npar
    real(kind=wp) :: pt_jet, q2
    logical :: valid
    pt_jet = eval_pt_jet(npar, 2)
    q2 = value_q2_dis
    eval_dis_dsclj2 = sqrt((q2 + pt_jet**2)/2._wp)
  end function eval_dis_dsclj2

  real(kind=wp) function eval_dis_dsclj3(npar)
    integer, intent(in) :: npar
    real(kind=wp) :: pt_jet, q2
    logical :: valid
    pt_jet = eval_pt_jet(npar, 3)
    q2 = value_q2_dis
    eval_dis_dsclj3 = sqrt((q2 + pt_jet**2)/2._wp)
  end function eval_dis_dsclj3

  real(kind=wp) function eval_dis_dsclj4(npar)
    integer, intent(in) :: npar
    real(kind=wp) :: pt_jet, q2
    logical :: valid
    pt_jet = eval_pt_jet(npar, 4)
    q2 = value_q2_dis
    eval_dis_dsclj4 = sqrt((q2 + pt_jet**2)/2._wp)
  end function eval_dis_dsclj4

  real(kind=wp) function eval_dis_dsclj1l(npar)
    integer, intent(in) :: npar
    real(kind=wp) :: pt_jet, q2
    logical :: valid
    pt_jet = eval_dis_hera_etj1(npar)
    q2 = value_q2_dis
    eval_dis_dsclj1l = sqrt((q2 + pt_jet**2)/2._wp)
  end function eval_dis_dsclj1l

  real(kind=wp) function eval_dis_dsclj2l(npar)
    integer, intent(in) :: npar
    real(kind=wp) :: pt_jet, q2
    logical :: valid
    pt_jet = eval_dis_hera_etj2(npar)
    q2 = value_q2_dis
    eval_dis_dsclj2l = sqrt((q2 + pt_jet**2)/2._wp)
  end function eval_dis_dsclj2l

  real(kind=wp) function eval_dis_dsclj3l(npar)
    integer, intent(in) :: npar
    real(kind=wp) :: pt_jet, q2
    logical :: valid
    pt_jet = eval_dis_hera_etj3(npar)
    q2 = value_q2_dis
    eval_dis_dsclj3l = sqrt((q2 + pt_jet**2)/2._wp)
  end function eval_dis_dsclj3l

  real(kind=wp) function eval_dis_dsclj4l(npar)
    integer, intent(in) :: npar
    real(kind=wp) :: pt_jet, q2
    logical :: valid
    pt_jet = eval_dis_hera_etj4(npar)
    q2 = value_q2_dis
    eval_dis_dsclj4l = sqrt((q2 + pt_jet**2)/2._wp)
  end function eval_dis_dsclj4l

  real(kind=wp) function eval_dis_dsclZEUS(npar)
    integer, intent(in) :: npar
    real(kind=wp) :: etavg_j12, q2
    logical :: valid
    etavg_j12 = eval_etavg_j12(npar)
    q2 = value_q2_dis
    eval_dis_dsclZEUS = sqrt(q2 + etavg_j12**2)
  end function eval_dis_dsclZEUS

  real(kind=wp) function eval_dis_dsclZEUS2(npar)
    integer, intent(in) :: npar
    real(kind=wp) :: etavg_jall, q2
    logical :: valid
    etavg_jall = eval_etavg_jall(npar)
    q2 = value_q2_dis
    eval_dis_dsclZEUS2 = sqrt(q2 + etavg_jall**2)
  end function eval_dis_dsclZEUS2

  real(kind=wp) function eval_dis_hera_deltaeta_j12(npar)
    integer, intent(in) :: npar
    real(kind=wp) :: eta1, eta2
    eta1 = v1_eta(dis%HERAjets(:, 1))
    eta2 = v1_eta(dis%HERAjets(:, 2))
    eval_dis_hera_deltaeta_j12 = abs(eta1 - eta2)
  end function eval_dis_hera_deltaeta_j12

  real(kind=wp) function eval_dis_hera_etaavg_j12(npar)
    integer, intent(in) :: npar
    real(kind=wp) :: eta1, eta2
    eta1 = v1_eta(dis%HERAjets(:, 1))
    eta2 = v1_eta(dis%HERAjets(:, 2))
    eval_dis_hera_etaavg_j12 = 0.5_wp*(eta1 + eta2)
  end function eval_dis_hera_etaavg_j12

  real(kind=wp) function eval_dis_hera_etaj1(npar)
    integer, intent(in) :: npar
    eval_dis_hera_etaj1 = v1_eta(dis%HERAjets(1:4, 1))
  end function eval_dis_hera_etaj1

  real(kind=wp) function eval_dis_hera_etaj2(npar)
    integer, intent(in) :: npar
    eval_dis_hera_etaj2 = v1_eta(dis%HERAjets(1:4, 2))
  end function eval_dis_hera_etaj2

  real(kind=wp) function eval_dis_hera_etaj3(npar)
    integer, intent(in) :: npar
    eval_dis_hera_etaj3 = v1_eta(dis%HERAjets(1:4, 3))
  end function eval_dis_hera_etaj3

  real(kind=wp) function eval_dis_hera_etaj4(npar)
    integer, intent(in) :: npar
    eval_dis_hera_etaj4 = v1_eta(dis%HERAjets(1:4, 4))
  end function eval_dis_hera_etaj4

  real(kind=wp) function eval_dis_hera_etj1(npar)
    integer, intent(in) :: npar
    eval_dis_hera_etj1 = v1_et(dis%HERAjets(1:4, 1))
  end function eval_dis_hera_etj1

  real(kind=wp) function eval_dis_hera_etj2(npar)
    integer, intent(in) :: npar
    eval_dis_hera_etj2 = v1_et(dis%HERAjets(1:4, 2))
  end function eval_dis_hera_etj2

  real(kind=wp) function eval_dis_hera_etj3(npar)
    integer, intent(in) :: npar
    eval_dis_hera_etj3 = v1_et(dis%HERAjets(1:4, 3))
  end function eval_dis_hera_etj3

  real(kind=wp) function eval_dis_hera_etj4(npar)
    integer, intent(in) :: npar
    eval_dis_hera_etj4 = v1_et(dis%HERAjets(1:4, 4))
  end function eval_dis_hera_etj4

  real(kind=wp) function eval_dis_hera_et2j1(npar)
    integer, intent(in) :: npar
    eval_dis_hera_et2j1 = v1_et2(dis%HERAjets(1:4, 1))
  end function eval_dis_hera_et2j1

  real(kind=wp) function eval_dis_hera_et2j2(npar)
    integer, intent(in) :: npar
    eval_dis_hera_et2j2 = v1_et2(dis%HERAjets(1:4, 2))
  end function eval_dis_hera_et2j2

  real(kind=wp) function eval_dis_hera_et2j3(npar)
    integer, intent(in) :: npar
    eval_dis_hera_et2j3 = v1_et2(dis%HERAjets(1:4, 3))
  end function eval_dis_hera_et2j3

  real(kind=wp) function eval_dis_hera_et2j4(npar)
    integer, intent(in) :: npar
    eval_dis_hera_et2j4 = v1_et2(dis%HERAjets(1:4, 4))
  end function eval_dis_hera_et2j4

  real(kind=wp) function eval_dis_hera_ET2byQ2j1(npar)
    integer, intent(in) :: npar
    real(kind=wp) q2
    q2 = value_q2_dis
    eval_dis_hera_ET2byQ2j1 = v1_et2(dis%HERAjets(1:4, 1))/q2
  end function eval_dis_hera_ET2byQ2j1

  real(kind=wp) function eval_dis_hera_ET2byQ2j2(npar)
    integer, intent(in) :: npar
    real(kind=wp) q2
    q2 = value_q2_dis
    eval_dis_hera_ET2byQ2j2 = v1_et2(dis%HERAjets(1:4, 2))/q2
  end function eval_dis_hera_ET2byQ2j2

  real(kind=wp) function eval_dis_hera_ET2byQ2j3(npar)
    integer, intent(in) :: npar
    real(kind=wp) q2
    q2 = value_q2_dis
    eval_dis_hera_ET2byQ2j3 = v1_et2(dis%HERAjets(1:4, 3))/q2
  end function eval_dis_hera_ET2byQ2j3

  real(kind=wp) function eval_dis_hera_ET2byQ2j4(npar)
    integer, intent(in) :: npar
    real(kind=wp) q2
    q2 = value_q2_dis
    eval_dis_hera_ET2byQ2j4 = v1_et2(dis%HERAjets(1:4, 4))/q2
  end function eval_dis_hera_ET2byQ2j4

  real(kind=wp) function eval_dis_hera_eta_j1_0(npar)
    integer, intent(in) :: npar
    eval_dis_hera_eta_j1_0 = v1_eta(dis%raw_HERAjets(:, 1))
  end function eval_dis_hera_eta_j1_0

  real(kind=wp) function eval_dis_hera_xgamma(npar)
    integer, intent(in) :: npar
    real(kind=wp) :: Ej1, Ej2, Pzj1, Pzj2, y, Ee
    Ej1 = dis%HERAjets(4, 1)
    Ej2 = dis%HERAjets(4, 2)
    Pzj1 = dis%HERAjets(3, 1)
    Pzj2 = dis%HERAjets(3, 2)
    y = value_y_dis
    Ee = getEe()
    eval_dis_hera_xgamma = (Ej1 - Pzj1 + Ej2 - Pzj2)/(2._wp*Ee*y)
  end function eval_dis_hera_xgamma

  real(kind=wp) function eval_dis_hera_eta_j2_0(npar)
    integer, intent(in) :: npar
    eval_dis_hera_eta_j2_0 = v1_eta(dis%raw_HERAjets(:, 2))
  end function eval_dis_hera_eta_j2_0

  real(kind=wp) function eval_gammap_eta_j1(npar)
    integer, intent(in) :: npar
    eval_gammap_eta_j1 = v1_eta(dis%HERApgammajets(:, 1))
  end function eval_gammap_eta_j1

  real(kind=wp) function eval_gammap_eta_j2(npar)
    integer, intent(in) :: npar
    eval_gammap_eta_j2 = v1_eta(dis%HERApgammajets(:, 2))
  end function eval_gammap_eta_j2

  real(kind=wp) function eval_DIS_hera_FJ_reweight(npar)
    integer, intent(in) :: npar
    real(kind=wp) :: et2sum, q2
    integer :: i
    et2sum = 0._wp
    q2 = value_q2_dis
    if (kin(npar)%njets .ge. 1) then
      do i = 1, kin(npar)%njets
        et2sum = et2sum + v1_et(dis%HERAjets(1:4, i))**2
      end do
    end if
    eval_DIS_hera_FJ_reweight = q2 + et2sum
  end function eval_DIS_hera_FJ_reweight

  real(kind=wp) function eval_dis_hera_deta1(npar)
    integer, intent(in) :: npar
    eval_dis_hera_deta1 = v1_eta(dis%HERAplus2jets(1:4, 1)) &
                          - v1_eta(dis%HERAplus2jets(1:4, 2))
  end function eval_dis_hera_deta1

  real(kind=wp) function eval_dis_hera_deta2(npar)
    integer, intent(in) :: npar
    eval_dis_hera_deta2 = v1_eta(dis%HERAjets(1:4, 1)) &
                          - v1_eta(dis%HERAplus2jets(1:4, 1))
  end function eval_dis_hera_deta2

  real(kind=wp) function eval_dis_delta_phis(npar)
    use Constants_mod
    integer, intent(in) :: npar
    real(kind=wp) :: phi1, phi2, phi
    phi1 = v1_phi(dis%HERApgammajets(:, 1))
    phi2 = v1_phi(dis%HERApgammajets(:, 2))
    phi = abs(phi1 - phi2)
    if (phi .gt. pi) phi = twopi - phi
    eval_dis_delta_phis = phi
  end function eval_dis_delta_phis

  real(kind=wp) function eval_dis_xi2(npar)
    integer, intent(in) :: npar
    real(kind=wp) :: m12, q2, x
    m12 = eval_minv_j12(npar)
    q2 = value_q2_dis
    x = value_x_dis
    eval_dis_xi2 = x*(1._wp + m12**2/q2)
  end function eval_dis_xi2

  real(kind=wp) function eval_dis_thrust(npar)
    integer, intent(in) :: npar
    real(kind=wp) :: top, bot, absp
    integer :: i, ncount

    top = 0._wp
    bot = 0._wp
    absp = 0._wp
    ncount = 0
    do i = 4, npar
      ! Select only particles in the current hemisphere (pz > 0)
      if (kin(npar)%p(3, i) > 0._wp) then
        absp = sqrt(kin(npar)%p(1, i)**2 + kin(npar)%p(2, i)**2 + kin(npar)%p(3, i)**2)
        top = top + sqrt(kin(npar)%p(3, i)**2)
        bot = bot + absp
        ncount = ncount + 1
      end if
    end do
    if (ncount .eq. 0) then
      eval_dis_thrust = -1._wp
    else
      eval_dis_thrust = 1._wp - top/bot
    end if
  end function eval_dis_thrust

  real(kind=wp) function eval_dis_JB(npar)
    integer, intent(in) :: npar
    real(kind=wp) :: top, bot, absp
    integer :: i, ncount

    top = 0._wp
    bot = 0._wp
    absp = 0._wp
    ncount = 0
    do i = 4, npar
      ! Select only particles in the current hemisphere (pz > 0)
      if (kin(npar)%p(3, i) > 0._wp) then
        ncount = ncount + 1
        absp = sqrt(kin(npar)%p(1, i)**2 + kin(npar)%p(2, i)**2 + kin(npar)%p(3, i)**2)
        top = top + sqrt(kin(npar)%p(1, i)**2 + kin(npar)%p(2, i)**2)
        bot = bot + absp
      end if
    end do

    if (ncount .eq. 0) then
      eval_dis_JB = -1._wp
    else
      eval_dis_JB = top/(2._wp*bot)
    end if
  end function eval_dis_JB

  real(kind=wp) function eval_dis_JM2(npar)
    integer, intent(in) :: npar
    real(kind=wp) :: absptot, absp!,ehtot
    real(kind=wp) :: pxtot, pytot, pztot
    integer :: i, ncount

    pxtot = 0._wp
    pytot = 0._wp
    pztot = 0._wp
    !ehtot = 0._wp !rho_0: replace eh with modulus of 3-momentum
    absp = 0._wp
    absptot = 0._wp
    ncount = 0

    do i = 4, npar
      ! Select only particles in the current hemisphere (pz > 0)
      if (kin(npar)%p(3, i) > 0._wp) then
        ncount = ncount + 1
        pxtot = pxtot + kin(npar)%p(1, i)
        pytot = pytot + kin(npar)%p(2, i)
        pztot = pztot + kin(npar)%p(3, i)
        absp = sqrt(kin(npar)%p(1, i)**2 + kin(npar)%p(2, i)**2 + kin(npar)%p(3, i)**2)
        !ehtot = ehtot + kin(npar)%p(4,i)
        absptot = absptot + absp
      end if
    end do
    if (ncount .eq. 0) then
      eval_dis_JM2 = -1._wp
    else
      eval_dis_JM2 = (absptot**2 - pxtot**2 - pytot**2 - pztot**2)/(2._wp*absptot)**2
    end if
  end function eval_dis_JM2

  real(kind=wp) function eval_dis_C(npar)
    integer, intent(in) :: npar
    real(kind=wp) :: t11, t12, t13, t22, t23, t33
    real(kind=wp) :: Cpar
    t11 = thetadis(1, 1, npar)
    t12 = thetadis(1, 2, npar)
    t13 = thetadis(1, 3, npar)
    t22 = thetadis(2, 2, npar)
    t23 = thetadis(2, 3, npar)
    t33 = thetadis(3, 3, npar)
    Cpar = -t12**2 - t13**2 - t23**2 + t11*t22 + t11*t33 + t22*t33
    eval_dis_C = 3._wp*Cpar
  end function eval_dis_C

  real(kind=wp) function eval_dis_Lgxi2(npar)
    integer, intent(in) :: npar
    real(kind=wp) :: m12, q2, x
    m12 = eval_minv_j12(npar)
    q2 = value_q2_dis
    x = value_x_dis
    eval_dis_Lgxi2 = log10(x*(1._wp + m12**2/q2))
  end function eval_dis_Lgxi2

  real(kind=wp) function eval_dis_etas(npar)
    integer, intent(in) :: npar
    real(kind=wp) :: deta
    deta = eval_deltaeta_j12(npar)
    eval_dis_etas = abs(deta)/2._wp
  end function eval_dis_etas

  real(kind=wp) function eval_dis_visbyW(npar)
    integer, intent(in) :: npar
    real(kind=wp) :: Esum, W
    integer i
    W = value_W_dis
    Esum = 0._wp
    do i = 1, kin(npar)%njets
      Esum = Esum + kin(npar)%p(4, i)
    end do
    eval_dis_visbyW = Esum/W
  end function eval_dis_visbyW

  real(kind=wp) function eval_dis_xi3(npar)
    integer, intent(in) :: npar
    real(kind=wp) :: m123, q2, x
    m123 = eval_minv_j123(npar)
    q2 = value_q2_dis
    x = value_x_dis
    eval_dis_xi3 = x*(1._wp + m123**2/q2)
  end function eval_dis_xi3

  real(kind=wp) function eval_dis_ptl(npar)
    integer, intent(in) :: npar
    eval_dis_ptl = v1_pt(kin(npar)%p(:, 3))
  end function eval_dis_ptl

  real(kind=wp) function eval_disg_xgamma(npar)
    integer, intent(in) :: npar
    real(kind=wp) :: y, Ee, etaj1, etaj2, ptj1, ptj2, s1, s2
    y = value_y_disg
    Ee = getEe()
    etaj1 = eval_eta_jet(npar, 1)
    etaj2 = eval_eta_jet(npar, 2)
    ptj1 = eval_pt_jet(npar, 1)
    ptj2 = eval_pt_jet(npar, 2)

    s1 = ptj1*dexp(-etaj1)
    s2 = ptj2*dexp(-etaj2)
    eval_disg_xgamma = (s1 + s2)/2._wp/y/Ee
  end function eval_disg_xgamma

  real(kind=wp) function eval_disg_xproton(npar)
    integer, intent(in) :: npar
    real(kind=wp) :: Ep, etaj1, etaj2, ptj1, ptj2, s1, s2
    Ep = getEp()
    etaj1 = eval_eta_jet(npar, 1)
    etaj2 = eval_eta_jet(npar, 2)
    ptj1 = eval_pt_jet(npar, 1)
    ptj2 = eval_pt_jet(npar, 2)
    s1 = ptj1*dexp(-etaj1)
    s2 = ptj2*dexp(-etaj2)
    eval_disg_xproton = (s1 + s2)/2._wp/Ep
  end function eval_disg_xproton

  real(kind=wp) function eval_disg_cos(npar)
    integer, intent(in) :: npar
    real(kind=wp) :: etaj1, etaj2
    etaj1 = eval_eta_jet(npar, 1)
    etaj2 = eval_eta_jet(npar, 2)
    eval_disg_cos = dabs(dtanh(etaj1 - etaj2))/2._wp
  end function eval_disg_cos

end module EvalDIS_mod
