!-------------------------------------------------------------------------------
! NNLOJET
!-------------------------------------------------------------------------------
!
! MODULE: Kinds
!
! DESCRIPTION:
!> Define Fortran kinds used in NNLOJET & the working precision (`wp`)
!-------------------------------------------------------------------------------

!-------------------------------------------------------------------------------
module Kinds_mod
!-------------------------------------------------------------------------------
  use, intrinsic :: iso_fortran_env, only: &
    sp => real32, &
    dp => real64, &
    qp => real128, &
    int8, &
    int64
  implicit none
  private

  public :: sp, dp, qp, wp
  public :: int8   ! tiny int (max = 127); for storage optimizations
  public :: int64  ! bigger int; for counters

  !> real number: single, double, and quadruple precision
  !>  https://fortranwiki.org/fortran/show/Real+precision

  !> strictly speaking, these are Fortran 2008
  ! integer, parameter :: sp = real32
  ! integer, parameter :: dp = real64
  ! integer, parameter :: qp = real128

  !> for older standards, one could use instead:
  ! integer, pramaeter :: sp = kind(1.0)
  ! integer, pramaeter :: dp = selected_real_kind(2*precision(1.0_sp))
  ! integer, pramaeter :: qp = selected_real_kind(2*precision(1.0_dp))

  !> better?
  ! integer, parameter :: sp = selected_real_kind(6)
  ! integer, parameter :: dp = selected_real_kind(15)
  ! integer, parameter :: qp = selected_real_kind(33)

  !> our working precision
  integer, parameter :: wp = dp

end module Kinds_mod

