      program NNLOJET
#ifdef USE_OMP
      use omp_lib
#endif
#ifdef USE_OL
      use openloops
#endif
      use Process_mod
      use ProcLib_mod
      use Observables_mod
      use Scales_mod
      use Selectors_mod
      use Histograms_mod
      use PhotonIsolation_mod
      use LeptonIsolation_mod
      use Channels_mod
      use Run_mod
      use MultiRun_mod
      use IOHelper_mod
      use UnitPhase_mod
#ifdef USEFASTJET
      use FastJet_mod, only : reset_fj
#endif
      implicit real*8(a-h,o-z)
      logical swap
      character*10 sy0
      character*20 sindex
      character*128 slogname
      character*20 sruncard
      common /pmasses/rma2,rmb2,rm2(1:7),shat
!$omp threadprivate(/pmasses/)
      common /koppel/ nf,nc,b0
      common /log/slogname
      common /order/ieorder
      common /order2l/ipole
      common /zewfacswap/ swap
!$omp threadprivate(/zewfacswap/)
      common /Qmass/emt,etwidth,emb,ebwidth,emc,ecwidth,emtau,etauwidth

      !> parse runcard and do (serial part) of the modules initialisation
      call readruncard(sy0,sindex)

      !> for now:
      call file_io(logfile = slogname)

      ieorder = 0 ! epsilon order at 1 loop
      ipole = 0 ! epsilon order at 2 loop

      rma2 = 0d0
      rmb2 = 0d0
      if (name_proc(1:7).eq.'HTO2TAU') then
      rm2(1) = emtau**2
      rm2(2) = emtau**2
      else
      rm2(1) = 0d0
      rm2(2) = 0d0
      endif
      rm2(3) = 0d0
      rm2(4) = 0d0
      rm2(5) = 0d0
      rm2(6) = 0d0
      rm2(7) = 0d0
      nc = 3
      nf = 5
      b0 = ((11d0*nc) - (2d0*nf))/6d0
      shat = 0d0

* initialize vegas

      swap = .false.
      call writer()
      call getversion() ! gets repository version from preprocessor to add to the log file

#ifdef USE_OMP
      do i=1,2
         if (i.eq.1) iout = 6
         if (i.eq.2) then
            iout = 500
            open (unit=500,file=trim(slogname),status="old",
     .              position="Append",action="write")
         endif
         write(iout,*) "Maximum available threads: ", OMP_get_num_procs()
         write(iout,*) "Allocated number of threads: ", OMP_get_max_threads()
         if(i.eq.2) close(500)
      enddo
#endif
      call system_clock(itimestart,irate)

#ifdef USE_OL
      call start()
#endif

      !> manually override the sig functions
      if (phase_unit)  call load_sig_plib(sig_unit, sig_zero)

      !> loop over all registered sweeps
      call resetCurrentSweep_run()
      do while (loadNextSweep_run())

        !> loop over all registered contributions
        call resetCurrentContrib_chan()
        do while (loadNextContrib_chan())

          res = 0d0
          err = 0d0
          call vegaswrapper(res,err)

        end do  ! end contrib

      end do  ! end sweep


      !> Module destruction
      call destroy_hist()
      call destroy_sel()
      call destroy_obs()
      call destroy_scl()
      call destroy_isol()
      call destroy_lepisol()
      call destroy_run()
      call destroy_mult()
      call destroy_chan()
      call destroy_plib()
#ifdef USEFASTJET
      call reset_fj()
#endif

#ifdef USE_OL
      call finish()
#endif


      call system_clock(itimeend)

      time = real(itimeend-itimestart)/real(irate)

      do i=1,2
         if (i.eq.1) iout = 6
         if (i.eq.2) then
            iout = 500
            open (unit=500,file=trim(slogname),status="old",
     .              position="Append",action="write")
         endif
         if(time.lt.60d0) then
            write(iout,40) time
         elseif(time.lt.3600d0) then
            write(iout,41) time/60d0
         else
            write(iout,42) time/3600d0
         endif
         if(i.eq.2) close(500)
      enddo

 40   format(/,' Elapsed time = ',g14.6,' seconds ')
 41   format(/,' Elapsed time = ',g14.6,' minutes ')
 42   format(/,' Elapsed time = ',g14.6,' hours ')


      end





      subroutine readinputs(sruncard,sindex,smember)
      use Vegas_mod, only: activate_parallel_sockets
      use Process_mod
      use ProcLib_mod
      use Channels_mod
      use Jets_mod
      use Observables_mod, only : init_obs, digestActive_obs
      use PhotonIsolation_mod
      use LHAPDF_mod, only : init_lhapdf
      implicit real*8(a-h,o-z)
      character*99 sarg
      character*99 sver
      character*99 sruncard
      character*20 sindex
      character*20 smember
      character*99 s_host
      logical passed

      passed = .false.
      sindex = 'No'
      smember = 'No'
      n_sockets = 1
      i_socket = 1
      i_port = 8888
      s_host = "gridui1.dur.scotgrid.ac.uk"

      do ii=1,command_argument_count()
        ! room here for more command line arguments if necessary
        call get_command_argument(ii,sarg)
        call get_command_argument(ii+1,sver)
        select case (trim(sarg))
        case ('-run','--run')
          read(sver,*) sruncard
          passed = .true.
        case('-iseed','--iseed')
          read(sver,*) sindex ! read seed from command line
          print *, "Input seed: ", sindex
        case('-imember','--imember')
          read(sver,*) smember ! read PDF member from command line
          print *, "PDF Member: ", smember
        case('-listobs',"--listobs")
          call init_proc(trim(sver))
          !> need to se sqrts and run setup_proc to get proper channel init
          sqrts_proc = 42d0
          call setup_proc()
          call init_plib()
          call init_obs()
          if (name_proc(1:4) .eq. "EPEM") then
            call init_jet('jade', 0d0, 'v4')
          else
            call init_jet('antikt', 0d0, 'v4')
          end if
          if(name_proc.ne."")then
            print *, ""
            print *, "Printing available observables for process ", trim(name_proc_usr), ":"
            print *, ""
          endif
          if(name_proc(1:1) .eq. "G")then
            call digestObs_isol()
          end if
          call digestActive_jet()
          call digestActive_obs()
          call destroy_plib()
          stop
        case('-listlumi',"--listlumi")
          call init_proc(trim(sver))
          !> need to se sqrts and run setup_proc to get proper channel init
          sqrts_proc = 42d0
          call setup_proc()
          call init_plib()
          call init_chan()
          call enableChannel_chan("ALL")
          call digestLumi_chan()
          call destroy_plib()
          stop
        case('--checkPDF','--checkpdf')
          call init_lhapdf(trim(sver))
          stop
        case('-listprocs',"--listprocs")
            !> List of available processes in public release
            print *, "*"
            print *, "* Available processes up to NNLO accuracy for hadron-hadron colliders:"
            print *, "*    Z      : production of a Z-boson;"
            print *, "*    ZJ     : production of a Z-boson plus a jet;"
            print *, "*    WP     : production of a positvely charged W-boson;"
            print *, "*    WPJ    : production of a positvely charged W-boson plus a jet;"
            print *, "*    WM     : production of a negatively charged W-boson;"
            print *, "*    WMJ    : production of a negatively charged W-boson plus a jet;"
            print *, "*    H      : production of a H-boson;"
            print *, "*    HJ     : production of a H-boson plus a jet;"
            print *, "*    H2     : production of a H-boson with decay to two colour singlets, in particular:"
            print *, "*      - HTO2P     : decay to two photons;"
            ! print *, "*      - HTO2TAU   : decay mode to (tau+tau-);"
            print *, "*    H2J    : production of a H-boson plus a jet with decay to two colour singlets, in particular:"
            print *, "*      - HTO2PJ    : decay to two photons;"
            ! print *, "*      - HTO2TAUJ  : decay mode to (tau+tau-);"
            print *, "*    H3     : production of a H-boson with decay to three colour singlets, in particular:"
            print *, "*      - HTO2L1P   : Dalitz decay to (l+l-) and photon;"
            print *, "*    H3J    : production of a H-boson plus a jet with decay to three colour singlets, in particular:"
            print *, "*      - HTO2L1PJ  : Dalitz decay to (l+l-) and photon;"
            print *, "*    H4     : production of a H-boson with decay to four colour singlets, in particular:"
            print *, "*      - HTO4E     : decay to (l+l-) (l+l-);"
            print *, "*      - HTO2E2MU  : decay to (l1+l1) (l2+l2-);"
            print *, "*      - HTO2L2N   : decay to (l1-nu1b) (l2+nu2);"
            print *, "*    H4J     : production of a H-boson plus a jet with decay to four colour singlets, in particular:"
            print *, "*      - HTO4EJ    : decay to (l+l-) (l+l-);"
            print *, "*      - HTO2E2MUJ : decay to (l1+l1) (l2+l2-);"
            print *, "*      - HTO2L2NJ  : decay to (l1-nu1b) (l2+nu2);"
            print *, "*    GJ     : production of a photon plus a jet;"
            print *, "*    GG     : production of two photons;"
            print *, "*    JJ     : production of two jets;"
            ! print *, "*    VFH,VFHJ         : production of H-boson in vector-boson-fusion channel;"
            ! print *, "*    VFH2,VFH2J       : production of H-boson in vector-boson-fusion channel, with decay mode to gamma-gamma;"
            ! print *, "*    VFH3,VFH3J       : production of H-boson in vector-boson-fusion channel, with decay mode to 2l+gamma;"
            ! print *, "*    VFH4,VFH4J       : production of H-boson in vector-boson-fusion channel, with decay mode to 4l and 2l2nu;"
            print *, "*"
            print *, "* Available processes up to NNLO accuracy for electron-positron colliders:"
            print *, "*    eeJJ   : production of two jets;"
            print *, "*    eeJJJ  : production of three jets;"
            print *, "*"
            print *, "* Available processes up to NNLO accuracy for lepton-hadron colliders:"
            print *, "*    epLJ   : production of a jet through neutral-current exchange;"
            print *, "*    epLJJ  : production of two jets through neutral-current exchange;"
            print *, "*"
            print *, "* Available processes up to NNLO accuracy for electron-hadron colliders:"
            print *, "*    epNJ   : production of a jet through charged-current exchange;"
            print *, "*    epNJJ  : production of two jets through charged-current exchange;"
            print *, "*"
            print *, "* Available processes up to NNLO accuracy for positron-hadron colliders:"
            print *, "*    epNbJ  : production of a jet through charged-current exchange;"
            print *, "*    epNbJJ : production of two jets through charged-current exchange;"
            stop
        case('-listprocsNLO',"--listprocsNLO")
            !> List of available processes in public release
            print *, "*"
            print *, "* Available processes only up to NLO accuracy for hadron-hadron colliders:"
            print *, "*    ZJJ     : production of a Z-boson plus two jets;"
            print *, "*    WJJ     : production of a W-boson plus two jets;"
            print *, "*    HJJ     : production of a H-boson plus two jets;"
            print *, "*    H2JJ    : production of a H-boson plus two jets with decay to two colour singlets, in particular:"
            print *, "*      - HTO2PJJ    : decay to two photons;"
            print *, "*      - HTO2TAUJJ  : decay mode to (tau+tau-);"
            print *, "*    H3JJ    : production of a H-boson plus two jets with decay to three colour singlets, in particular:"
            print *, "*      - HTO2L1PJJ  : Dalitz decay to (l+l-) and photon;"
            print *, "*    H4JJ    : production of a H-boson plus two jets with decay to four colour singlets, in particular:"
            print *, "*      - HTO4EJJ    : decay to (l+l-) (l+l-);"
            print *, "*      - HTO2E2MUJJ : decay to (l1+l1) (l2+l2-);"
            print *, "*      - HTO2L2NJJ  : decay to (l1-nu1b) (l2+nu2);"
            print *, "*    GJJ     : production of a photon plus two jets;"
            print *, "*    JJJ     : production of two jets;"
            stop
        case('-listprocsLO',"--listprocsLO")
            !> List of available processes in public release
            print *, "*"
            print *, "* Available processes only up to LO accuracy for hadron-hadron colliders:"
            print *, "*    ZJJJ     : production of a Z-boson plus three jets;"
            print *, "*    WJJJ     : production of a W-boson plus three jets;"
            print *, "*    HJJJ     : production of a H-boson plus three jets;"
            print *, "*    H2JJJ    : production of a H-boson plus three jets with decay to two colour singlets, in particular:"
            print *, "*      - HTO2PJJJ    : decay to two photons;"
            print *, "*      - HTO2TAUJJJ  : decay mode to (tau+tau-);"
            print *, "*    H3JJJ    : production of a H-boson plus three jets with decay to three colour singlets, in particular:"
            print *, "*      - HTO2L1PJJJ  : Dalitz decay to (l+l-) and photon;"
            print *, "*    H4JJJ    : production of a H-boson plus three jets with decay to four colour singlets, in particular:"
            print *, "*      - HTO4EJJJ    : decay to (l+l-) (l+l-);"
            print *, "*      - HTO2E2MUJJJ : decay to (l1+l1) (l2+l2-);"
            print *, "*      - HTO2L2NJJJ  : decay to (l1-nu1b) (l2+nu2);"
            print *, "*    GJJJ     : production of a photon plus three jets;"
            stop
        case('-help',"-h","--help")
          print *, ""
          print *, "run NNLOJET by providing a runcard:"
          print *, "     ./NNLOJET --run <RUNCARD>"
          print *, ""
          print *, "  available optional flags:"
          print *, "    --help   -  this help screen"
          print *, "    --iseed  -  override the seed number of the runcard"
          print *, "    --imember  -  override the PDF member of the runcard"
          print *, "    --listprocs - query all available processes"
          print *, "    --listobs <PROCESS>  -  query all valid observables"
          print *, "    --listlumi <PROCESS>  -  query all parton luminosities"
          print *, ""
          print *,"  running with sockets (compile with sockets=true)"
          print *,"    --has-sockets - returns 0 if NNLOJET has been
     . compiled with sockets "
          print *,"    --host <host> - server address"
          print *,"    --port <port> "
          print *,"    --sockets <N> - total num of parallel jobs"
          print *,"    --ns <i> - ordinal of this client (i={1..N})"
          stop
        case('-sockets','--sockets')
          read(sver, *) n_sockets
        case('-ns','--ns')
          read(sver, *) i_socket
        case('-port','--port')
          read(sver, *) i_port
        case('-host','--host')
          read(sver, *) s_host
        case('-has-sockets','--has-sockets')
          print*, "Yes"
          stop 0
        end select
      end do

      if (n_sockets > 1) then
         call activate_parallel_sockets(n_sockets, i_socket, s_host, i_port)
         print *, "Sockets active, ", n_sockets, i_socket
      endif

      if ( .not.passed ) then
        print*, ""
        print*, "please provide '--run' flag to the terminal"
        print*, "     ./NNLOJET --run example.run"
        print*, ""
        print*, "or check the help:"
        print*, "     ./NNLOJET --help"
        stop
      end if

      return
      end


      subroutine readruncard(sy0,sindex)
      use IOHelper_mod
      use Parameters_mod
      use Process_mod
      use Run_mod
      use Channels_mod
      use KinData_mod, only : setCacheObsSize_kin
      use Observables_mod, only : init_obs, nUsed_obs
      use Scales_mod
      use Selectors_mod
      use Histograms_mod
      use PhotonIsolation_mod
      use LeptonIsolation_mod
      use MultiRun_mod
      use StrHelper_mod
      use nnlo_bridge_mod, only : iseed_hook
      implicit real*8(a-h,o-z)
      logical flag,qfail
      character*10 sy0
      character*20 sindex
      character*20 smember
      character*40 sline
      character*99 sruncard
      logical y0warning,scalewarning
      common /rseeds/i1,i2
      common /ranno/ idum
      common /tcuts/ymin,y0
      common /nwa/rm12,rg12,inwa
      common /driverwarnings/y0warning,scalewarning
      dimension iseeds(1:2,0:99)
      integer :: iplot
      common /plotmode/ iplot

      y0warning=.false.
      scalewarning=.false.

      data iseeds/
     . 2991, 9281, 3823, 8151, 6239, 8754, 9514, 3746, 4893, 7731,
     . 8635, 4662, 4132, 3764, 5234, 4892, 9753, 7629, 2531, 1638,
     . 9376, 4157, 6472, 8086, 8526, 1264, 4723, 7549, 9472, 4637,
     . 3746, 4682, 9854, 8472, 2332, 7724, 5524, 6352, 3652, 7382,
     . 4431, 3667, 4538, 4888, 1838, 7655, 6764, 8382, 4885, 9980,
     . 9876, 2312, 8743, 5521, 7512, 1123, 9814, 5531, 1249, 6413,
     . 3468, 2814, 4392, 0921,  736, 2881, 4831, 8381, 2319, 4452,
     . 3112,  978, 4791,  541, 1598, 8421, 5798, 4359, 2345, 3258,
     . 3409,   91,  714, 8383, 2553, 7344, 9589, 4931,  941, 3881,
     . 9145, 3885,   12, 4949, 9693, 3919, 9592, 1312, 6667, 2823,
     . 5994, 2864, 4913, 6511, 8726, 2745, 4537, 2371, 6148, 4714,
     . 8414,   28, 3851,  284, 3884, 2813,  847, 1822, 5871, 2442,
     . 4771, 7411, 4728, 2771, 4881, 9831, 1236, 8831,  837, 3771,
     . 5993, 1277, 3441, 8482,  693, 4882, 9391, 3899, 3991, 1134,
     . 2884, 3851, 5918,  761, 9278, 9391,  101, 2391, 8124, 9391,
     .  401, 4313, 3855, 8731, 4721, 3893,  145, 4992, 7573, 8471,
     . 9981, 2383,  921, 3881,  388, 1384, 7628, 7237, 9492, 3884,
     .   28, 8313, 3883, 8281, 1332, 3927, 5452, 7246, 2812, 2831,
     . 7691, 3718, 4881,  331, 6266, 2163, 4772, 7371, 9931, 3881,
     . 7713, 6361, 3488, 9281, 4913, 3813, 3463,   89,  918, 8381 /

      call readinputs(sruncard,sindex,smember) ! reads command line inputs for runcard name

      index=0
      y0=0d0


      open(unit=20,file=sruncard,status='old',err=990)


      !> parse the process specification
      call parse_proc_io(20)

#ifdef USEFASTJET
      !> optional FastJet interface
      call parse_fastjet_io(20)  ! before `init_obs`
#endif

      !> parse the run settings
      call parse_run_io(20, smember)

      !> override iseed if passed via command line
      if ( trim(sindex) /= 'No') then
        index = to_int_str(sindex, qfail)
        if (.not.qfail) iseed_run = index
      end if

      !> pass seed index to the hook
      iseed_hook = iseed_run

      if((iseed_run.gt.999).and.(iseed_run.lt.1000000))then
         iextra = iseed_run/1000 ! integer division
      elseif(iseed_run.le.999)then
         iextra = 0
      else
         write(*,*) "Error, iseed_run request: ", iseed_run, " is too large"
         stop
      endif

      isubindex = iseed_run - iextra*1000

      if(isubindex.lt.0)then
         write(*,*) "Error: define a seed number >= 1"
         stop
      elseif(isubindex.le.99)then
         n=0
         j=10
      elseif((isubindex.ge.100).and.(isubindex.le.999))then
         do j=1,9
            n=j*100
            if((isubindex.ge.n).and.(isubindex.le.n+99))then
               exit
            endif
         enddo
      else
         write(*,*) "Seeding Error: this should never happen"
         stop
      endif

c     for indices > 100, we take multiples of the seeds for < 100
      i1 = iseeds(1,isubindex-n)*(j+1)/11 + iextra
      i2 = iseeds(2,isubindex-n)*(j+1)/11 + iextra

      idum = -abs((i1+i2)/2)

      !> need to set the process before switching on obs
      call init_obs()

      !> optional override of parameters
      !> (setup before param for now)
      !> before initialiseProc because masses needed there!
      call parse_param_io(20)
      call init_param()

      !> need to set up parameters before setup
      call setup_proc()
      call setup_run()
      !> *after* setup_proc!
      !> *before* histogram parsing (for grids, I need channel info)
      call init_chan()  ! here we call selectchannel & allocate memory
      call parse_chan_io(20)

      !> Selectors
      call init_sel()  ! call after parsing the process-block!
      call parse_sel_io(20)
      call init_isol()  ! after selectors
      call init_lepisol()

      !> Scales
      call parse_scl_io(20)  ! *after* RUN parsing

      !> set up multi-runs
      !>  need to be before histogram parsing
      call init_mult()
      call parse_mult_io(20)

      !> Histograms (scales must be initialised before calling init_hist)
      call parse_hist_io(20)

      !> re-weighting function?
      call parse_rewgt_io(20)

      call parse_phase_unit_io(20)

      call parse_pdfi_io(20)

      !> place to do the purge of used observables
      call setCacheObsSize_kin(nUsed_obs)

      if(iplot.eq.0)then ! MC integration
        ! do nothing...
      elseif(iplot.eq.2)then ! Point test => ignore run card and consider all events
         ! call enableChannel_chan("ALL")
         ! actually: take runcard --> do nothing
      elseif(iplot.eq.3)then ! Spike plots
        ! do nothing...
      elseif(iplot.eq.4)then
         fully_inclusive = .true. !> From Selectors_mod
         iplot = 0
      else
         write(*,*) "Invalid iplot: ", iplot
         stop
      endif


      close(unit=20)
      return

 990  write(*,*) "Error in reading input file: ", sruncard
      stop

 991  write(*,*) "Invalid formatting of input file: ", sruncard
      stop

      end


      subroutine getversion()
      use NNLOJET_mod
      implicit none
      integer iout,i
      character*128 slogname
      common /log/slogname

      do i=1,2
         if (i.eq.1) iout = 6
         if (i.eq.2) then
            iout = 500
            open (unit=500,file=trim(slogname),status="old",
     .              position="Append",action="write")
         endif
         write(iout,*) "Repository id: ", version_nnlojet
         if (version_nnlojet.ne."1.0.0") then
         endif
         if(i.eq.2) close(500)
      enddo

      return
      end
