module PhaseNJ_mod
   use KinData_mod, only : dot, boostrest, unboostrest, rotatetoz, unrotatetoz, kin, fillS_kin
   implicit none

   public :: phaseNJ, fillKinVFH, initVFH, destroyVFH

   private
   ! Debug variables
!     integer, public :: puntos
!     !$omp threadprivate(puntos)

   ! Parameters
   logical, parameter :: NOROT = .false., ROTATE = .true.
   integer, parameter :: dp = kind(1.0d0)
   real(dp), parameter :: pi = 3.1415926535897932385d0, &
      costhmin = -1d0, &
      costhmax =  1d0, &
      phimin   =  0d0, &
      phimax   =  2d0*pi
   ! Generic initial partons
   integer, parameter :: a = 1, b = 2
   ! 3 final partons
   integer, parameter :: i3 = 5, j3 = 4, k3 = 3
   ! 4 final partons
   integer, parameter :: i4 = 6, j4 = 5, k4 = 3, l4 = 4

   ! Sampled variables
   real(dp), dimension(10) :: sij, cth, phi

   ! Momentum storage
   real(dp), allocatable, dimension(:, :, :) :: pTemp
   integer, dimension(4) :: wedgecut

   ! Weight
   real(dp) :: wtps

   ! Wedges and rotations flags and variables
   integer, parameter :: wedges_2jlo = 0, wedges_2jnlo = 1, wedges_2jnnlo = 2 
   integer, parameter :: wedges_3jlo = 1, wedges_3jnlo = 3
   logical :: triplecol
   logical :: permute_ab = .false.
   logical :: activeWedges, activeRotations
   integer :: imode = wedges_2jlo
   integer :: nrotations, npartons, Nphase_raw
   integer, dimension(4, 24) :: permut

   ! Make all variables used by this module threadprivate
   !$omp threadprivate(sij, cth, phi, pTemp, wedgecut, wtps)
   ! These shouldn't matter as their value should be the same across all threads but anyway:
   !$omp threadprivate(permute_ab, activeWedges, activeRotations, nrotations, npartons, permut, nphase_raw, triplecol, imode)

   ! number of particles (set in in initVFH)
   integer :: n

   ! External functions
   double precision, external :: dlambda

   ! Global common blocks
   double precision ymin, y0
   common /tcuts/ymin,y0
   double precision rm12, rg12
   integer inwa
   common /nwa/rm12,rg12,inwa
   double precision rmv1,rgv1,rmv2,rgv2
   integer ios1,ios2,isym
   common/vdecay/rmv1,rgv1,rmv2,rgv2,ios1,ios2,isym
   integer mxdim
   common /mxdim/ mxdim
   integer iang
   common /techpar/iang
   integer Nphase
   common /Nphase/Nphase
   !$omp threadprivate(/Nphase/)
   double precision rma2, rmb2, rm2, shat
   common/pmasses/rma2,rmb2,rm2(1:7),shat
   !$omp threadprivate(/pmasses/)

contains

   subroutine phaseNJ(x, wt, ifail)
      use Process_mod, only : n_fs_spectators_proc, n_fs_photons_proc
      real(dp), target, intent(in) :: x(*)
      real(dp), intent(out) :: wt
      integer, intent(out) :: ifail
      ! Arrays and pointers
      real(dp), dimension(:), pointer :: rand_vfh, rand_inwa, rand_p3gen
      real(dp), dimension(:), pointer :: pa, pb, p_out1, p_out2, p_H
      real(dp), dimension(4) :: pg1, pg2, p_out3
      real(dp), dimension(4) :: p_fin1, p_fin2, p_fin3, p_fin4
      real(dp), dimension(4) :: q_fin1, q_fin2, q_fin3, q_fin4
      real(dp), dimension(4) :: sij_in, rvar, rvar2
      real(dp), dimension(4,5), target :: p3gend
      real(dp), dimension(4,3) :: pdecay3
      real(dp), dimension(4,4) :: pdecay4
      ! Variables
      integer :: i, nspec
      !        ! Activate wedges and rotations depending on the number of partons in the final state
      ! This is done at initVFH 
      !        ! 6, no wedges or rotations, nphase = 1
      !        ! 7, wedges, no rotations, nphase = 3 * 2
      !        ! 8, wedges, 4 rotations if iang=1, nphase = nrotation*{12*2, 6}
      !        call activateWedgesRotations(npartons)
      
!      write(*,*) "xxxx"
      
      ! Initialise variables
      wedgecut(:) = 1
      sij(:) = 0d0
      rvar(:) = 0d0
      sij(1) = shat
      wtps = 1d0
      ifail = 0

      nspec = n_fs_spectators_proc + n_fs_photons_proc

      !> Debug, sequential phase space
      !> N=8, inwa=0, roots=8tev: 3.727 * 10^17
!        call sampleLinearAll(x, ifail)
!        if (ifail.eq.1) return
!        if (inwa > 0) then
!           call phaseSequentialBoson(x, wt, ifail)
!        else
!           call phaseSequential(x, wt, ifail)
!        endif
!        wtps = wtps*wt
!        activeWedges = .false.
!        activeRotations = .false.
!        permute_ab = .false.
!        nphase = 1
!        wt = wtps
!        return

      ! Create random variable vectors
      ! Decay of the heavy particle
      rand_inwa => x(2:(3*nspec-3))
      ! 2 -> 3 system
      rand_p3gen => x((3*nspec-2):(3*nspec+1))
      ! Extra partons
      if (npartons >= 3) then
         rand_vfh => x((3*nspec+2):(3*npartons+3*nspec-5))
      endif

      ! Sample Higgs mass
      call sampleInwa(x(1), sij(10))
      if (shat < sij(10)) then
         ifail = 1
         return
      endif

      ! Extra partons:
      ! triplecol: (npartons = 4)
      !      sij(3) => s_out11
      ! else:
      !      sij(6)  => s_out2
      if (npartons >= 3) then
         call sampleVFH(rand_vfh, ifail)
         if (ifail == 1) return
      endif

      ! Generate the 2 -> 3 system
      sij_in = (/sij(1), sij(2), sij(10), sij(5)/)
      call p3generic(rand_p3gen, sij_in, p3gend, ifail)
      if (ifail.eq.1) return
      pa => p3gend(:,1)
      pb => p3gend(:,2)
      p_out1 => p3gend(:, 3)
      p_H    => p3gend(:, 4)
      p_out2 => p3gend(:, 5)

      do i = 1, nrotations
         pTemp(:, a, i) = pa(:)
         pTemp(:, b, i) = pb(:)
      enddo

      ! Decay of the massive boson p_H
      select case(nspec)
      case(2)
         call sampleDecay(rand_inwa)
         rvar = (/sij(10), 0d0, cth(10), phi(10)/)
         call makePsWrapper(rvar, p_H, pg1, pg2, NOROT)
         pTemp(:, n  , 1) = pg2(:)
         pTemp(:, n-1, 1) = pg1(:)
      case(3)
         call decay3(sij(10),p_H,rand_inwa,pdecay3,wtps,ifail)
         if (ifail == 1) return
         wtps = wtps/2d0/pi
         pTemp(:, n  , 1) = pdecay3(:, 3)
         pTemp(:, n-1, 1) = pdecay3(:, 2)
         pTemp(:, n-2, 1) = pdecay3(:, 1)
      case(4)
         call decay4(sij(10),p_H,rand_inwa,pdecay4,wtps,ifail)
         if (ifail == 1) return
         wtps = wtps/2d0/pi
         pTemp(:, n  , 1) = pdecay4(:, 4)
         pTemp(:, n-1, 1) = pdecay4(:, 3)
         pTemp(:, n-2, 1) = pdecay4(:, 2)
         pTemp(:, n-3, 1) = pdecay4(:, 1)
      end select

      select case(npartons)
      case(2)
         pTemp(:, 3, 1) = p_out1(:)
         pTemp(:, 4, 1) = p_out2(:)
      case(3)
         rvar = (/sij(2), 0d0, cth(2), phi(2)/)
         call makePsWrapper(rvar, p_out1, pTemp(:,i3,1), pTemp(:,k3,1), NOROT)
         pTemp(:,j3,1) = p_out2(:)
      case(4)
         if (triplecol) then
            rvar = (/sij(2), sij(3), cth(2), phi(2)/)
            call makePsWrapper(rvar, p_out1, p_out3, p_fin1, NOROT, pa)
            rvar2 = (/sij(3), 0d0, cth(3), phi(3)/)
            call makePsWrapper(rvar2, p_out3, p_fin2, p_fin3, NOROT)
            pTemp(:,j4,1) = p_out2(:)            
            pTemp(:,i4,1) = p_fin1(:)             
            pTemp(:,k4,1) = p_fin2(:)            
            pTemp(:,l4,1) = p_fin3(:)            
            if (activeRotations) then
               ! j is unaffected in all cases
               pTemp(:,j4,2) = p_out2(:)        
               pTemp(:,j4,3) = p_out2(:)        
               pTemp(:,j4,4) = p_out2(:)       

               ! (lk)
               call makePsWrapper(rvar2, p_out3, q_fin2, q_fin3, ROTATE)
               pTemp(:,i4,2) = p_fin1(:)
               pTemp(:,k4,2) = q_fin2(:)        
               pTemp(:,l4,2) = q_fin3(:)         

               ! (ilk;a)
               p_out3 = 0d0
               q_fin2 = 0d0
               q_fin3 = 0d0
               call makePsWrapper(rvar, p_out1, p_out3, q_fin1, ROTATE, pa)
               call makePsWrapper(rvar2, p_out3, q_fin2, q_fin3, NOROT)
               pTemp(:,i4,3) = q_fin1(:)        
               pTemp(:,k4,3) = q_fin2(:)        
               pTemp(:,l4,3) = q_fin3(:)  

               ! (ijk;a) + (jk)
               q_fin2 = 0d0
               q_fin3 = 0d0
               pTemp(:,i4,4) = q_fin1(:)        
               call makePsWrapper(rvar2, p_out3, q_fin2, q_fin3, ROTATE)
               pTemp(:,k4,4) = q_fin2(:)        
               pTemp(:,l4,4) = q_fin3(:)    
            endif
         else
            rvar = (/sij(2), 0d0, cth(2), phi(2)/)
            rvar2 = (/sij(5), 0d0, cth(5), phi(5)/)
            call makePsWrapper(rvar, p_out1, p_fin1, p_fin2, NOROT, pa)
            call makePsWrapper(rvar2, p_out2, p_fin3, p_fin4, NOROT, pb)
            pTemp(:, i4, 1) = p_fin1(:)
            pTemp(:, k4, 1) = p_fin2(:)
            pTemp(:, j4, 1) = p_fin3(:)
            pTemp(:, l4, 1) = p_fin4(:)
            if (activeRotations) then
               ! (ik)
               call makePsWrapper(rvar, p_out1, q_fin1, q_fin2, ROTATE, pa)
               ! (jl)
               call makePsWrapper(rvar2, p_out2, q_fin3, q_fin4, ROTATE, pb)
               ! (ik)
               pTemp(:, i4, 2) = q_fin1(:)
               pTemp(:, k4, 2) = q_fin2(:)
               pTemp(:, j4, 2) = p_fin3(:)
               pTemp(:, l4, 2) = p_fin4(:)
               ! (jl)
               pTemp(:, i4, 3) = p_fin1(:)
               pTemp(:, k4, 3) = p_fin2(:)
               pTemp(:, j4, 3) = q_fin3(:)
               pTemp(:, l4, 3) = q_fin4(:)
               ! (ik) + (jl)
               pTemp(:, i4, 4) = q_fin1(:)
               pTemp(:, k4, 4) = q_fin2(:)
               pTemp(:, j4, 4) = q_fin3(:)
               pTemp(:, l4, 4) = q_fin4(:)
            endif
         endif
      end select

      ! Check wedges
      if (activeWedges) then
         wtps = wtps / (1d0 * nrotations) 
         do i = 1, nrotations
            select case(imode)
            case(wedges_2jnlo)
               call wedge3(i)
            case(wedges_2jnnlo)
               call wedgeAB(i)
            case(wedges_3jnlo)
               call wedgeA3j(i)
            end select
         enddo
      endif

      wt = wtps
   end subroutine

   !-----------------------------------------------------------------------------
   !> @brief
   !>   function to be called by genphase to check whether the point passed                               
   !>   the wedge cuts
   !-----------------------------------------------------------------------------
   subroutine cutPassed(i, n_rot, ipass)
      integer, intent(in) :: i
      integer, intent(inout) :: ipass
      integer, intent(out) :: n_rot
      if (ipass == 0) return
      n_rot = modulo(i, nrotations)
      if (n_rot == 0) n_rot = nrotations
      ipass = wedgecut(n_rot)
   end subroutine

   !-----------------------------------------------------------------------------
   !> @brief
   !>  Given ievt (nphase) and npar, fill up the corresponding kin
   !>                                 
   !-----------------------------------------------------------------------------
   subroutine fillKinVFH(ievt, n_par, ipass)
      integer, intent(in) :: ievt, n_par
      integer, intent(inout) :: ipass
      logical :: ab_swap
      integer :: n_rot, n_perm, i
      real(dp) :: smin, stest, stest2, stest3
      integer :: j
      ! If no wedges, there's nothing to worry about
      if (.not.activeWedges) then
         do i = 1, n_par
            kin(n_par)%p(:, i) = pTemp(:, i, 1)
         enddo
      else
         ! Check whether the ps point passed the wedge and retrieve the rotationtrue.
         call cutPassed(ievt, n_rot, ipass)
         if (ipass.eq.0) return

         ! Recover the permutation index
         n_perm = ceiling((ievt-0.001)/nrotations)

         ! Check whether this is a a<-->b set
         if ( (permute_ab) .and. ( ievt > Nphase_raw ) ) then
            ab_swap = .true.
            n_perm = n_perm - (Nphase_raw/nrotations)
         else
            ab_swap = .false.
         endif

         ! Fill up kin(n_par)

         ! Final partons
         do i = 3, npartons + 2
            kin(n_par)%p(:, permut(i-2, 1)) = pTemp(:, permut(i-2, n_perm), n_rot)
         enddo
         ! Colorless particles
         do i = npartons + 3, n_par
            kin(n_par)%p(:, i) = pTemp(:, i, 1)
         enddo
         ! Initial partons
         if (ab_swap) then 
            kin(n_par)%p(:, 1) = pTemp(:, 2, 1)
            kin(n_par)%p(:, 2) = pTemp(:, 1, 1)
            do i = 1, n_par
               kin(n_par)%p(3, i) = -kin(n_par)%p(3, i)
            enddo
         else
            kin(n_par)%p(:, 1) = pTemp(:, 1, 1)
            kin(n_par)%p(:, 2) = pTemp(:, 2, 1)
         endif

      endif
      call fillS_kin(n_par)

   end subroutine

   !-----------------------------------------------------------------------------
   !> @brief
   !>  Generate the permutations for fillmomenta to fill pperm
   !>   it only needs to be called once at the beginning of a thread
   !>   allocate arrays, so destroyVFH should be called as well
   !>                                      
   !-----------------------------------------------------------------------------
   subroutine initVFH
      use Process_mod, only : n_fs_spectators_proc, n_fs_photons_proc
      use Channels_mod, only : currentContrib_chan
      call currentContrib_chan(npar=n)
      ! Debug
!        puntos = 0
      permut(:,:) = 0
      npartons = n - n_fs_spectators_proc - n_fs_photons_proc - 2
      call activateWedgesRotations(npartons)
      ! Generate permutations for the wedges
      select case(imode)
      case(wedges_2jnlo)
         permut(:, 1) = (/j3, i3, k3, 0/)
         permut(:, 2) = (/i3, k3, j3, 0/)
         permut(:, 3) = (/k3, j3, i3, 0/)
      case(wedges_2jnnlo)
         if (triplecol) then
            ! Permute i over the 4 final state
            ! and permute (k,l) over the 3 possible comb for fixed i
            permut(:,1)  = (/i4,k4,l4,j4/)
            permut(:,2)  = (/k4,i4,l4,j4/)
            permut(:,3)  = (/l4,k4,i4,j4/)
            permut(:,4)  = (/j4,k4,l4,i4/)

            permut(:,5)  = (/i4,k4,j4,l4/)
            permut(:,6)  = (/k4,i4,j4,l4/)
            permut(:,7)  = (/l4,k4,j4,i4/)
            permut(:,8)  = (/j4,k4,i4,l4/)

            permut(:,9)  = (/i4,j4,l4,k4/)
            permut(:,10) = (/k4,j4,l4,i4/)
            permut(:,11) = (/l4,j4,i4,k4/)
            permut(:,12) = (/j4,i4,l4,k4/)
         else
            permut(:,1) = (/i4,k4, j4,l4/)
            permut(:,2) = (/i4,j4, k4,l4/)
            permut(:,3) = (/i4,l4, j4,k4/)
         endif
      case(wedges_3jnlo)
         ! Permute i <-> k,l j <-> k,l and ij <-> kl
            permut(:,1)  = (/i4,k4,l4,j4/)
            permut(:,2)  = (/k4,i4,l4,j4/)
            permut(:,3)  = (/l4,k4,i4,j4/)
            permut(:,4)  = (/i4,j4,l4,k4/)
            permut(:,5)  = (/i4,k4,j4,l4/)
            permut(:,6)  = (/k4,i4,j4,l4/)
      case default
         permut(:, 1) = (/3, 4, 5, 6/)
      end select 
      ! Allocate pTemp
      allocate(pTemp(4, n,nrotations))
   end subroutine 

   subroutine destroyVFH
      deallocate(pTemp)
      !print *, "Points through: ", puntos
   end subroutine

   !-----------------------------------------------------------------------------
   !> @brief
   !>  Generate the 2 -> 3 system
   !>                                      
   !-----------------------------------------------------------------------------
   subroutine p3generic(rvar, rsi, pout, ifail)
      real(dp), dimension(4), intent(inout) :: rvar, rsi
      real(dp), dimension(4,5), intent(out) :: pout
      integer, intent(out) :: ifail
      double precision, external :: dlambda
      real(dp), parameter :: rma2 = 0d0, rmb2 = 0d0
      real(dp), dimension(4) :: pa, pb, pb_boosted
      real(dp) :: roots, s1, s2, s3, Eab, pin, wtem
      real(dp) :: t1, t1max, t1min, rt1
      real(dp) :: t2, t2max, t2min, rt2
      real(dp) :: s23, s23max, s23min, rnum1, rnum2
      real(dp) :: s12, dl1, rg1, rg2, r1, det3, rph, cp
      real(dp), dimension(4) :: p1, p23
      real(dp) :: E1, E23, dlout, p1out, cos1a, sin1a
      real(dp), dimension(4) :: p2, p3, p2rb, p3rb, p2_boosted, p3_boosted
      real(dp), dimension(4,4) :: ubmat, urmat, bmat
      real(dp) :: dl2, cos3b, sin3b, rph1
      real(dp) :: dr1, dr2, dr3, dr4, dr5, dr6, dr7, raux1, raux2, det4
      !
      ! ma -----.------ p1
      ! s0=shat |t1
      !         |       /p2
      ! mb -----.------.
      !  \         s23   \p3
      !   \   t2   
      !    ._____________/
      ! Initialisation
      pa(:) = 0d0
      pb(:) = 0d0
      pout(:,:) = 0d0
      wtem = 1d0
      ! Input data
      shat = rsi(1)
      roots = dsqrt(shat)
      s1 = rsi(2)
      s2 = rsi(3)
      s3 = rsi(4)
      Eab = roots / 2d0
      pin = roots / 2d0

      ! Incoming momenta
      pa(4) = Eab
      pb(4) = Eab
      pa(3) = pin
      pb(3) = -pin

      ! All min/max checks are made before random sampling

      ! Sampling of t1 (cos1a)
      s23min = max((dsqrt(s2) + dsqrt(s3))**2, y0*shat)
      call glimits(shat,t1,s23min,rma2,rmb2,s1, t1max,t1min)
      t1max = min(t1max, - y0*shat)
      if (t1min > t1max) then
         ifail = 1
         return
      endif
      call pickRand(2, rt1, 1d0-t1max, 1d0-t1min, rvar(1),wtem)
      t1 = 1d0-rt1

      ! Sample t2 (cos3b)
      s23max = (s1*t1 - shat*t1 - t1**2)/(s1 - t1) 
      call glimits(s23max, t2, s2, rmb2, t1, s3, t2max,t2min)
      t2max = min(t2max, - y0*shat)
      if (t2min > t2max) then
         ifail = 1
         return
      endif
      call pickRand(2, rt2, 1d0-t2max, 1d0-t2min, rvar(2), wtem)
      t2 = 1d0-rt2

      ! Sample s23
      rnum1  =  s3*(s2 - t1) - (s2 + s3 + t1)*t2 + t2**2
      rnum2  =  (-s3 + t2)*dsqrt(dlambda(s2,t1,t2))
      s23min =  max(-(rnum1 - rnum2)/2d0/t2, s23min)
      if (s23min > s23max) then
         ifail = 1
         return
      endif
      call pickRand(1,s23,s23min,s23max,rvar(3),wtem)

      ! Temporary: sample directly cos instead of s12
      dl1  = dlambda(s23,t1,rmb2)
      rg1  = gres(shat,t1,s23,rma2,rmb2,s1)
      rg2  = gres(s23,t2,s3,t1,rmb2,s2)
      r1   = dsqrt(rg1*rg2)*2d0
      det3 = 2d0 * t1 * shat * (t2 - s3)  &
         + (s23 - t1)*(  &
         + shat * (s23 - s2 + 2d0*s3 -t2)  &
         + (s3 - t2) * (s23 - s1 - 2d0*shat) )
      call pickrand(1, cp, costhmin, costhmax, rvar(4), wtem)
      rph = dacos(cp)
      wtem  = wtem * r1 / dl1 
      s12 = shat + s3 - (det3 - cp*r1) / dl1

      ! Generate p1 and p23
      E1    =  (shat + s1 - s23)/2d0/roots
      E23   =  (shat + s23 - s1)/2d0/roots
      dlout =  dsqrt(dlambda(shat, s23, s1))
      p1out =  dlout/2d0/roots
      cos1a = (t1 - s1 - rma2 + 2d0*Eab*E1)/(2d0*pin*p1out)
      if (dabs(cos1a) > 1d0) then
         print *, "NAN at cos1a"
      endif
      sin1a = dsqrt(1d0 + cos1a) * dsqrt(1d0 - cos1a)
      call pickRand(1,rph1, phimin, phimax, 0d0, wtem)
      p1(4)    =  E1
      p1(1)    =  p1out*sin1a*dcos(rph1)
      p1(2)    =  p1out*sin1a*dsin(rph1)
      p1(3)    =  p1out*cos1a
      p23(4)   =  E23
      p23(1:3) =  - p1(1:3)

      ! Break p23 into p2 and p3
      dl1   = dsqrt(dlambda(s23,rmb2,t1))
      dl2   = dsqrt(dlambda(s23,s3,s2))
      cos3b = (t2-rmb2-s3+(s23+rmb2-t1)*(s23+s3-s2)/2d0/s23) &
         * 2d0*s23/dl1/dl2
      if (dabs(cos3b) > 1d0) then
         print *, "NAN at cos3b"
      endif
      sin3b = dsqrt(1d0-cos3b)*dsqrt(1d0+cos3b)

      ! Create p2 and p3 in the c o m of p23
      ! This frame has its Z axis along pb
      call makePs2cm_nj(s23, cos3b, rph, s3, s2, p3rb, p2rb)

      ! Unrotate frorm the pb z axis
      call boostrest(p23, bmat)
      call boostWrapper(pb_boosted, bmat, pb)
      call unrotatetoz(pb_boosted, urmat)
      call boostWrapper(p2_boosted, urmat, p2rb)
      call boostWrapper(p3_boosted, urmat, p3rb)

      ! Boost back to com frame
      call unboostrest(p23, ubmat)
      call boostWrapper(p2, ubmat, p2_boosted)
      call boostWrapper(p3, ubmat, p3_boosted)

      ! Compute the weight for this 4 invariants ps
      ! t1*t2
      dr1 = s1*(shat-s12+s3) + s23*(s12-s3) &
         -shat*(shat + 2d0*s2 - s12 - s3 -s23) 
      dr2 = shat*(s12*s23 - shat*s2)
      ! t1**2
      dr3 = dlambda(shat, s3, s12)/2d0
      ! t2**2
      dr4 = dlambda(shat, s1, s23)/2d0
      ! t1
      dr5 = s12*(shat*s2-s12*s23+s1*s3) - shat*s3*(2d0*s12-s2-s1) &
         + s3*(s12*s23-s3*s1)
      ! t2
      dr6 = (s23*(shat*s2 - s23*s12) &
         +s1*(shat*s2 - s23*(2d0*shat-s12)) &
         +s3*(shat*s1 - s1**2 + s1*s23))
      ! rest
      dr7 = (s3*s1*(s3*s1/2d0-s12*s23-shat*s2) &
         + (s2*shat - s12*s23)**2/2d0)
      ! Finally
      raux1 = t1**2*dr3 + t2**2*dr4
      raux2 = t1*(dr2+dr5) + t2*(dr2+dr6)
      det4 = (t1*t2*dr1 + raux1 + raux2 + dr7) / 8d0
      if (det4 >= 0d0) then
         !           print *, "det4 > 0?!?", det4
         ifail = 1
         return
      endif
      wtem = wtem / (32d0) / (2d0*pi)**5 

      wtem = wtem / shat ! dsqrt(dlambda(shat, rma2, rmb2)
      wtem = wtem / dsqrt(-det4)
      wtps = wtem * wtps

      pout(:, 1) = pa
      pout(:, 2) = pb
      pout(:, 3) = p1
      pout(:, 4) = p2
      pout(:, 5) = p3

   end subroutine


   !-----------------------------------------------------------------------------
   !> @brief
   !>  Generate the 1 -> 3 decay system
   !>  Similar to the subroutine in phasenewhto3.f but better not make external/INTERFACE links 
   !>  Difference includes the ifail!=ipass definition and common block 
   !-----------------------------------------------------------------------------

      subroutine decay3(rmh2,ph,rr,pd,wtps,ifail)
      implicit real*8(a-h,o-z)
!---- parametrizes the decay of ph to one massive and one massless particles, which then deacy into three massless particles pd(1:3)
!---- requires five random variables rr
!---- settings for H-> Vg: rmv1=M_Z (or M_W), rmv2=0, ios1=1, ios2 no need to define
!---- ios=1: BW parametrization, ios=0: linear parametrization
      integer, intent(out) :: ifail
      real(dp), dimension(5), intent(in) :: rr
      real(dp), dimension(:), intent(in) :: ph
      real(dp), dimension(4,3), intent(out) :: pd
      dimension ubmat(4,4),ubmat12(4,4)
      dimension p12(4),p3(4),p12p(4),p3p(4),p12ap(4),p3ap(4)
      dimension p1p(4),p2p(4),p1ap(4),p2ap(4)
      integer i,j

      r1 = rr(1)
      r2 = rr(2)
      r3 = rr(3)
      r4 = rr(4)
      r5 = rr(5)

      !> ios1(2) -- heavy particle decay type
      !>  0 : use ds12/s12
      !>  1 : use resonance parametrization (with mass rm12, width rg12)
      !>  2 : narrow width inserted: delta function
      !>  3 : on-shell heavy particle no decay
      !>  -1: combine 0 and 1 (mix linear sampling at low s12 and BW sampling for high s12)

      ifail = 1
      s12min = y0*rmh2
      s12max = rmh2-s12min
!      write(6,*) '1:',wtps,r1,s12min,s12max,rmh2
      if (s12min.gt.s12max) return
      if (ios1.eq.0) then
         call pick(1,s12,s12min,s12max,r2,wtps)
      elseif (ios1.eq.1) then
         call pickres(s12,s12min,s12max,rmv1,rgv1,r2,wtps)
      elseif (ios1.eq.2) then
         s12 = rm12**2
         wtps = wtps*pi*rm12*rg12
      elseif (ios1.eq.3) then
         s12 = rm12**2
         wtps = wtps*pi*(4d0*pi)
      elseif (ios1.eq.-1) then
         call pick_lin_res(s12,s12min,s12max,rmv1,rgv1,r2,wtps)
      else
         write(*,*)"ios1 option not supported in decay3"
         stop
      endif
      if(rmh2.lt.s12) return ! Insufficient scattering energy to produce the heavy gauge boson

!      write(6,*) '2:',wtps
!         s12 = rmv1**2

      call pick(1,costh,costhmin,costhmax,r3,wtps)
!      write(6,*) '4a:',wtps,r3,r4
      call pick(1,phi_,phimin,phimax,r1,wtps)
!      write(6,*) '4:',wtps,r3,r4
      wtps = wtps*dsqrt(dlambda(rmh2,s12,rmv2))/rmh2/32d0/pi**2
!      write(6,*) '5:',wtps
      call makeps2com(rmh2,costh,phi_,s12,rmv2,p12p,p3p,p12ap,p3ap)
      call unboostrest(ph,ubmat)
      do i=1,4
         p12(i) = 0d0
         p3(i) = 0d0
         do j=1,4
            p12(i) = p12(i) + ubmat(i,j)*p12p(j)
            p3(i)  = p3(i)  + ubmat(i,j)*p3p(j)
         enddo
      enddo

      call pick(1,costh,costhmin,costhmax,r4,wtps)
      call pick(1,phi_,phimin,phimax,r5,wtps)
      wtps = wtps*dsqrt(dlambda(s12,0d0,0d0))/s12/32d0/pi**2
      wtps = wtps/2d0/pi
      call makeps2com(s12,costh,phi_,0d0,0d0,p1p,p2p,p1ap,p2ap)
      call unboostrest(p12,ubmat12)

      do i=1,4
         pd(i,1) = 0d0
         pd(i,2) = 0d0
         pd(i,3) = 0d0
         do j=1,4
            pd(i,1) = pd(i,1) + ubmat12(i,j)*p1p(j)
            pd(i,2) = pd(i,2) + ubmat12(i,j)*p2p(j)
         enddo
            pd(i,3) = pd(i,3) + p3(i)
      enddo
      ifail =0
      return
      end

   !-----------------------------------------------------------------------------
   !> @brief
   !>  Generate the 1 -> 4 decay system
   !>  Similar to the subroutine in phasenewhto4.f but better not make external/INTERFACE links 
   !>  Difference includes the ifail!=ipass definition and common block 
   !-----------------------------------------------------------------------------
    
   subroutine decay4(rmh2,ph,rr,pd,wtps,ifail)
      implicit real*8(a-h,o-z)
!---- parametrizes the decay of ph to two massive particles, which then deacy into four massless particles pd(1:4)
!---- requires eight random variables rr
!---- settings for H-> VV*: rmv1=rmv2=M_Z (or M_W), ios1=1, ios2=0, isym=1
!---- settings for V*-> VH: rmv1=M_Z, rmv2=M_H, ios1=1, ios2=1, isym=0
!---- ios=1: BW parametrization, ios=0: linear parametrization
!---- isym=1: symmetrize over on-shell and off-shell boson (such that always p1,p2=mu+,mu-, p3,p4=e+,e-)
      integer, intent(out) :: ifail
      real(dp), dimension(8), intent(in) :: rr
      real(dp), dimension(4), intent(in) :: ph
      real(dp), dimension(4,4), intent(out) :: pd
      dimension ubmat(4,4),ubmat12(4,4),ubmat34(4,4)
      dimension p12(4),p34(4),p12p(4),p34p(4),p12ap(4),p34ap(4)
      dimension p1p(4),p2p(4),p3p(4),p4p(4)
      dimension p1ap(4),p2ap(4),p3ap(4),p4ap(4)
      integer :: i, j, kk1, kk2, kk3, kk4

      r1 = rr(1)
      r2 = rr(2)
      r3 = rr(3)
      r4 = rr(4)
      r5 = rr(5)
      r6 = rr(6)
      r7 = rr(7)
      r8 = rr(8)

      !> ios1(2) -- heavy particle decay type
      !>  0 : use ds12/s12
      !>  1 : use resonance parametrization (with mass rm12, width rg12)
      !>  2 : narrow width inserted: delta function
      !>  3 : on-shell heavy particle no decay
      !>  -1: combine 0 and 1 (mix linear sampling at low s12 and BW sampling for high s12)

      ifail = 1
      s12min = y0*rmh2
      s12max = rmh2-s12min
!     write(6,*) '1:',wtps,r1,s12min,s12max,rmh2
      if (s12min.gt.s12max) return
      if (ios1.eq.0) then
         call pick(1,s12,s12min,s12max,r2,wtps)
      elseif (ios1.eq.1) then
         call pickres(s12,s12min,s12max,rmv1,rgv1,r2,wtps)
      elseif (ios1.eq.2) then
         s12 = rmv1**2
         wtps = wtps*pi*rmv1*rgv1
      elseif (ios1.eq.3) then
         s12 = rmv1**2
         wtps = wtps*pi*(4d0*pi)
      elseif (ios1.eq.-1) then
         call pick_lin_res(s12,s12min,s12max,rmv1,rgv1,r2,wtps)
      else
         write(*,*)"ios1 option not supported in decay4"
         stop
      endif
      if(rmh2.lt.s12) return ! Insufficient scattering energy to produce the heavy gauge boson

!      write(6,*) '2:',wtps

      s34min = y0*rmh2
      s34max = dmin1((dsqrt(rmh2)-dsqrt(s12))**2,(1d0-y0)*rmh2)
      if (s34min.gt.s34max) return
      if (ios2.eq.0) then
         call pick(1,s34,s34min,s34max,r3,wtps)
      elseif (ios2.eq.1) then
         call pickres(s34,s34min,s34max,rmv2,rgv2,r3,wtps)
      elseif (ios2.eq.2) then
         s34 = rmv2**2
         wtps = wtps*pi*rmv2*rgv2
      elseif (ios2.eq.3) then
         s34 = rmv2**2
         wtps = wtps*pi*(4d0*pi)
      elseif (ios2.eq.-1) then
         call pick_lin_res(s34,s34min,s34max,rmv2,rgv2,r3,wtps)
      else
         write(*,*)"ios2 option not supported in decay4"
         stop
      endif
!      write(6,*) '3:',wtps

!         s12 = rmv1**2
!         s34 = rmv2**2

      call pick(1,costh,costhmin,costhmax,r4,wtps)
!      write(6,*) '4a:',wtps,r3,r4
      call pick(1,phi_,phimin,phimax,r1,wtps)
!      write(6,*) '4:',wtps,r3,r4

      wtps = wtps*dsqrt(dlambda(rmh2,s12,s34))/rmh2/32d0/pi**2
!      write(6,*) '5:',wtps

      call makeps2com(rmh2,costh,phi_,s12,s34,p12p,p34p,p12ap,p34ap)
      call unboostrest(ph,ubmat)
      do i=1,4
         p12(i) = 0d0
         p34(i) = 0d0
         do j=1,4
            p12(i) = p12(i) + ubmat(i,j)*p12p(j)
            p34(i) = p34(i) + ubmat(i,j)*p34p(j)
         enddo
      enddo

      call pick(1,costh,costhmin,costhmax,r5,wtps)
      call pick(1,phi_,phimin,phimax,r6,wtps)
      wtps = wtps*dsqrt(dlambda(s12,0d0,0d0))/s12/32d0/pi**2
      wtps = wtps/2d0/pi
      call makeps2com(s12,costh,phi_,0d0,0d0,p1p,p2p,p1ap,p2ap)
      call unboostrest(p12,ubmat12)
      call pick(1,costh,costhmin,costhmax,r7,wtps)
      call pick(1,phi_,phimin,phimax,r8,wtps)
      wtps = wtps*dsqrt(dlambda(s34,0d0,0d0))/s34/32d0/pi**2
      wtps = wtps/2d0/pi
      call makeps2com(s34,costh,phi_,0d0,0d0,p3p,p4p,p3ap,p4ap)
      call unboostrest(p34,ubmat34)

      kk1=1
      kk2=2
      kk3=3
      kk4=4
      if (isym.eq.1) then       ! symmetrize the H->VV* kinematics
         if (RAND(0).lt.0.5d0) then
            kk1=3
            kk2=4
            kk3=1
            kk4=2
         endif
      endif

      do i=1,4
         pd(i,1) = 0d0
         pd(i,2) = 0d0
         pd(i,3) = 0d0
         pd(i,4) = 0d0
         do j=1,4
            pd(i,kk1) = pd(i,kk1) + ubmat12(i,j)*p1p(j)
            pd(i,kk2) = pd(i,kk2) + ubmat12(i,j)*p2p(j)
            pd(i,kk3) = pd(i,kk3) + ubmat34(i,j)*p3p(j)
            pd(i,kk4) = pd(i,kk4) + ubmat34(i,j)*p4p(j)
         enddo
      enddo
      ifail = 0
      return
      end





   !-----------------------------------------------------------------------------
   !> @brief
   !>  Generate a 1 -> 2 system in the c.o.m. of 1                                  
   !>                                      
   !-----------------------------------------------------------------------------
   subroutine makePs2cm_nj(s, costh, ph, s1, s2, p1, p2)
      real(dp), intent(in) :: s, costh, ph, s1, s2
      real(dp), dimension(4), intent(out) :: p1, p2
      real(dp) :: smin, sth, sph, cph, roots, roots1
      real(dp) :: E1, E2, pp

      smin = max(s1 + s2, y0*shat)
      if (s < smin) then
         print *, "Improper call of makePs2cm_nj"
         stop
      endif

      sth = dsqrt(1d0 + costh)*dsqrt(1d0 - costh)
      sph = dsin(ph)
      cph = dcos(ph)
      roots = dsqrt(s)

      E1 = (s + s1 - s2)/2d0/roots
      E2 = (s + s2 - s1)/2d0/roots
      roots1 = dsqrt(s1)
      pp = dsqrt(E1 + roots1)*dsqrt(E1 - roots1)

      p1(4) = E1
      p1(1) = pp*sth*cph
      p1(2) = pp*sth*sph
      p1(3) = pp*costh

      p2(4)   = E2
      p2(1:3) = - p1(1:3)
   end subroutine

   !-----------------------------------------------------------------------------
   !> @brief
   !>  calls makeps2cm to generate a 1 -> 2 system in the c.o.m of 1
   !>        only one of the 2 outgoing particles can be massive
   !>  unboost the 2 system back from the c.o.m. frame of 1
   !>  (if provided) rotates the system around the axis of 1 (pij)
   !>                                                 (or another parton, p_ref)
   !-----------------------------------------------------------------------------
   subroutine makePsWrapper(rvar, pij, pi_out, pj_out, rotate_flag, p_ref)
      real(dp), dimension(4), intent(in) :: rvar, pij
      real(dp), dimension(4), intent(out) :: pi_out, pj_out
      logical, intent(in) :: rotate_flag
      real(dp), dimension(4), intent(in), optional :: p_ref
      real(dp), dimension(4) :: p_boosted
      real(dp), dimension(4), target :: p_i, p_j, p_i_boosted, p_j_boosted
      real(dp), dimension(:), pointer :: pfi, pfj
      real(dp), dimension(4,4) :: ubmat, bmat, urmat
      real(dp) :: s_in, s_out, rcos, rph

      ! Get the sampled variables coming in in rvar
      s_in = rvar(1)
      s_out = rvar(2)
      rcos = rvar(3)
      rph = rvar(4)
      if (rotate_flag) rph = rph + pi/2d0

      ! Generate the 1->2 system
      call makePs2cm_nj(s_in, rcos, rph, s_out, 0d0, p_i, p_j)

      ! If given, the frame is rotate with its z axis alongside p_ref
      ! rotate back
      if (present(p_ref)) then
         call boostrest(pij, bmat)
         call boostWrapper(p_boosted, bmat, p_ref)
         call unrotatetoz(p_boosted, urmat)
         call boostWrapper(p_i_boosted, urmat, p_i)
         call boostWrapper(p_j_boosted, urmat, p_j)
         pfi => p_i_boosted
         pfj => p_j_boosted
      else
         pfi => p_i
         pfj => p_j
      endif

      ! Boost pi and pj back to the lab frame
      call unboostrest(pij, ubmat)
      call boostWrapper(pi_out, ubmat, pfi)
      call boostWrapper(pj_out, ubmat, pfj)
   end subroutine

   !-----------------------------------------------------------------------------
   !> @brief
   !>  pa = matrix * pb     
   !>                                      
   !-----------------------------------------------------------------------------
   subroutine boostWrapper(pa, matrix, pb)
      real(dp), dimension(4), intent(out) :: pa
      real(dp), dimension(4,4), intent(in) :: matrix
      real(dp), dimension(4), intent(in) :: pb
      integer :: i
      do i = 1, 4
         pa(i) = sum(matrix(i,:) * pb(:))
      enddo
   end subroutine


   !-----------------------------------------------------------------------------
   !> @brief
   !>  Subroutine to set the activeRotations and activeWedges flags
   !>  and nrotations and nphase parameters
   !-----------------------------------------------------------------------------
   subroutine setRegions_imode(npartons)
      use Process_mod, only : name_proc
      use Channels_mod, only : currentSubContrib_chan
      integer, intent(in) :: npartons
      !> Enable different regions of the phasespace depending on runcard parameters
      !> which enter through common blocks anyway
      !> region info
      integer :: iregion

      call currentSubContrib_chan(1, region=iregion)

      if (name_proc(1:3).ne.'VFH') return

      select case(npartons)
      case(2)
         imode = wedges_2jlo
      case(3)
         imode = wedges_2jnlo
      case(4)
         if((name_proc(1:4).eq.'VFHJ').or.(name_proc.eq.'VFHTO2PJ') &
     &   .or.(name_proc.eq.'VFHTO2TAUJ').or.(name_proc.eq.'VFHTO2E2MUJ') &
     &   .or.(name_proc.eq.'VFHTO2L2NJ').or.(name_proc.eq.'VFHTO4EJ') &
     &   .or.(name_proc.eq.'VFHTO2L1PJ')) then
            imode = wedges_3jnlo
         else
            imode = wedges_2jnnlo
            if (iregion == 1) then
               triplecol = .true.
            else if (iregion == 2) then
               triplecol = .false.
            else
               print *, "Required variable 'iregion' undefined at setRegions_imode"
               stop
            endif
         endif
      end select
   end subroutine setRegions_imode


   subroutine activateWedgesRotations(npartons)
      integer, intent(in) :: npartons

      !> Default values
      activeRotations = .false.
      activeWedges = .false.
      permute_ab = .false.
      tripleCol = .false.
      Nphase_raw = 1
      nrotations = 1

      call setRegions_imode(npartons)

      select case (imode) 
      case (wedges_2jnlo)
         activeWedges = .true.
         permute_ab = .true.
         Nphase_raw = 3
      case (wedges_2jnnlo)
         if (iang == 1) then
            activeRotations = .true.
            nrotations = 4
         endif
         if (triplecol) then
            Nphase_raw = 12*nrotations
         else
            Nphase_raw = 3*nrotations
         endif
         activeWedges = .true.
         permute_ab = .true.
      case (wedges_3jnlo) 
         Nphase_raw = 6
         permute_ab = .true.
         activeWedges = .true.
         !> Make sure triplecol is on even for b-ps
         triplecol = .true.
      end select
   
      if(permute_ab) then
         Nphase = Nphase_raw*2
      else
         Nphase = Nphase_raw
      endif
   end subroutine

   !-----------------------------------------------------------------------------
   !> @brief
   !>  Random picking of massive boson (inwa) or any particle (pickrand)                                
   !>                                      
   !-----------------------------------------------------------------------------
   subroutine sampleInwa(r, s12)
      real(dp), intent(in) :: r
      real(dp), intent(out) :: s12
      real(dp) :: s12max, s12min
      s12max = shat
      s12min = y0*shat
      select case(inwa)
      case(0)
         call pick(1,s12,s12min,s12max,r,wtps)
      case(1)
         call pickres(s12,s12min,s12max,rm12,rg12,r,wtps)
      case(2)
         s12  = rm12**2
         wtps = wtps*pi*rm12*rg12
      case(3)
         s12  = rm12**2
         wtps = wtps*pi*(16d0*pi)
      endselect
      return
   end subroutine

   subroutine pickRand(itype,s,smin,smax,r1,wt)
      integer, intent(in)             :: itype
      double precision, intent(in)    :: smin, smax, r1
      double precision, intent(inout) :: wt
      double precision, intent(out)   :: s
      ! itype = 1  pick linearly
      ! itype = 2  pick logarithmically
      select case(itype)
      case(1)
         s  = smin+r1*(smax-smin)
         wt = wt*(smax-smin)
      case(2)
         s  = smin*(smax/smin)**r1
         wt = wt*log(smax/smin)*s
      endselect
      return
   end subroutine


   !-----------------------------------------------------------------------------
   !> @brief
   !>  Sample the variables necessary for the phasespace
   !>                                      
   !-----------------------------------------------------------------------------
   subroutine sampleDecay(rvec)
      real(dp), dimension(2), intent(in) :: rvec
      ! Sample massive outgoing boson and its decay
      call pickRand(1, cth(10), costhmin, costhmax, rvec(1), wtps)
      call pickRand(1, phi(10), phimin,   phimax,   rvec(2), wtps)
      wtps = wtps/2d0/pi
      wtps = wtps/32d0/pi**2
   end subroutine

   subroutine sampleVFH(rvec, ifail)
      real(dp), dimension(6), intent(in) :: rvec
      integer, intent(inout) :: ifail
      real(dp) :: smin, smax
      smin = y0*shat
      if(((npartons == 3).or.(npartons == 4)).and.(.not.triplecol)) then ! sij(2), p_out1
         ! get the maximum s once the massive boson has been produced
         smax = (dsqrt(shat) - dsqrt(sij(10)))**2
         if (smax <= smin) then
            ifail = 1
            return
         endif
         call pickRand(2, sij(2), smin,     smax,     rvec(1), wtps)
         call pickRand(1, cth(2), costhmin, costhmax, rvec(2), wtps)
         call pickRand(1, phi(2), phimin,   phimax,   rvec(3), wtps)
         wtps = wtps/2d0/pi
         wtps = wtps/32d0/pi**2
      endif

      if ((npartons == 4).and.(triplecol)) then 
         ! sij(2), p_out1 will break into sij(3)->(p_fin2, p_fin3) and p_fin1
         ! Ie, min sij(2) = 3*y0*shat
         smax = (dsqrt(shat) - dsqrt(sij(10)))**2
         smin = 3d0*y0*shat
         if (smax <= smin) then
            ifail = 1
            return
         endif
         call pickRand(2, sij(2), smin,     smax,     rvec(1), wtps)
         call pickRand(1, cth(2), costhmin, costhmax, rvec(2), wtps)
         call pickRand(1, phi(2), phimin,   phimax,   rvec(3), wtps)
         wtps = wtps/2d0/pi
         wtps = wtps/32d0/pi**2
         smax = sij(2) - 2d0*y0*shat
         smin = y0*shat
         if (smax <= smin) then
            ifail = 1
            return
         endif
         call pickRand(2, sij(3), smin,     smax,     rvec(4), wtps)
         call pickRand(1, cth(3), costhmin, costhmax, rvec(5), wtps)
         call pickRand(1, phi(3), phimin,   phimax,   rvec(6), wtps)
         wtps = wtps/2d0/pi 
         wtps = wtps*dsqrt(dlambda(sij(2), 0d0, sij(3)))/sij(2)/32d0/pi**2
      elseif ((npartons == 4).and.(.not.triplecol)) then ! sij(5), for p_out2
         ! Find the max s once the boson and sij(2) have been produced
         smax = (dsqrt(shat) - dsqrt(sij(2)) - dsqrt(sij(10)))**2
         if (smax <= smin) then
            ifail = 1
            return
         endif
         call pickRand(2, sij(5), smin,     smax,     rvec(4), wtps)
         call pickRand(1, cth(5), costhmin, costhmax, rvec(5), wtps)
         call pickRand(1, phi(5), phimin,   phimax,   rvec(6), wtps)
         wtps = wtps/2d0/pi
         wtps = wtps/32d0/pi**2
      endif
   end subroutine

   !-----------------------------------------------------------------------------
   !> @brief
   !>  Function to check whether ij or ji are i_target and j_target
   !> 
   !-----------------------------------------------------------------------------
   pure logical function testij(i,j, i_target, j_target)
      integer, intent(in) :: i, j, i_target, j_target
      if ( ( (i == i_target).and.(j == j_target) ).or. &
           ( (i == j_target).and.(j == i_target) )) then
         testij = .true.
      else 
         testij = .false.
      endif
      return
   end function

   !-----------------------------------------------------------------------------
   !> @brief
   !>  Wedge checking for 3 partons in the final state
   !>    using the same wedges as phase3n                                   
   !> 
   !-----------------------------------------------------------------------------
   subroutine wedge3(irot)
      integer, intent(in) :: irot
      real(dp) :: smin, stest
      integer :: imin, jmin, ii, ij
      real(dp), dimension(5,5) :: sinv
      smin = shat
      imin = 0
      jmin = 0
      do ii = 1, 4
         do ij = ii + 1, 5
            stest = dabs(dot(pTemp(:, ii, irot), &
               &    pTemp(:, ij, irot))) 
            sinv(ii, ij) = stest
            sinv(ij, ii) = stest
            if (stest < smin) then
               imin = ii
               jmin = ij
               smin = stest
            endif
         enddo
      enddo
      wedgecut(irot) = 0
      if ( (imin == a) ) then
         ! s- = sai and sik < sji
         if (jmin == i3) then
            if ( sinv(i3,k3) < sinv(i3,j3) ) wedgecut(irot) = 1
            ! s- = sak and sik < sjk
         else if (jmin == k3) then
            if ( sinv(i3,k3) < sinv(k3,j3) ) wedgecut(irot) = 1
         endif
         ! s- = sik and (sai + sak) < (sbi + sbk)
      else if (((imin == i3).and.(jmin == k3)) &
         & .or.(imin == k3).and.(jmin == i3)) then
         if ( (sinv(a,i3) + sinv(a,k3)) < (sinv(b,i3) + sinv(b,k3)) ) then
            wedgecut(irot) = 1
         endif
      endif
      if (imin*jmin == 0) then
         print *, "Problem @ wedge3 (all invariants > shat)"
         stop
      endif
   end subroutine

   !-----------------------------------------------------------------------------
   !> @brief
   !>  Wedge checking for 4 partons 
   !>     (ps breaks into A and B zones for 2jNNLO)
   !>     (ps does not break for 3jNLO)
   !-----------------------------------------------------------------------------
   subroutine findSmallest_4partons(irot, im1, im2, jm1, jm2, sinv)
      integer, intent(in) :: irot
      integer, intent(out) :: im1, im2, jm1, jm2
      real(dp), dimension(6,6), intent(out) :: sinv
      integer :: ii, ij
      real(dp) :: smin1, smin2, s_a, s_b, stest
      logical :: passed 
      ! Look for the two smallest invariants
      smin1 = shat
      smin2 = shat*(1d0 + y0)
      im1   = 0 ; jm1 = 0
      im2   = 0 ; jm2 = 0
      do ii = 1, 5
         do ij = ii + 1, 6
            stest = dabs(dot(pTemp(:, ii, irot),  &
                             pTemp(:, ij, irot)))
            sinv(ii,ij) = stest
            sinv(ij,ii) = stest
            if ( stest < smin1 ) then
               smin2 = smin1
               smin1 = stest     
               im2 = im1 ; jm2 = jm1
               im1 = ii  ; jm1 = ij
            elseif ( stest < smin2 ) then
               smin2 = stest    
               im2 = ii  ; jm2 = ij
            endif
         enddo
      enddo
      if (((im1*im2*jm1*jm2).eq.0)) then
         print *, "event broken (all invariants > shat)"
         print *, im1, im2, jm1, jm2
         do ii = 1, 6
            print *, pTemp(:, ii, irot)
         enddo
         stop
      endif
   end subroutine


   subroutine wedgeA3j(irot)
      integer, intent(in) :: irot
      real(dp) :: s_a, s_b, s_t1, s_t2
      real(dp), dimension(6,6) :: sinv
      integer :: im1, im2, jm1, jm2
      logical :: passed 

      call findSmallest_4partons(irot, im1, im2, jm1, jm2, sinv)
      passed = .false.

      !> s1 = skl && sa(k+l) < sb(k+l)
      s_a = sinv(a,l4) + sinv(a,k4)
      s_b = sinv(b,l4) + sinv(b,k4)
      if ( testij(im1, jm1, k4, l4) .and. (s_a < s_b) ) then
         passed = .true.
      !> s1 = sbj && sai < sak && sai < sal
      else if ( (testij(im1, jm1, b, j4)) .and. &
           (sinv(a,i4) < sinv(a,k4)) .and. &
           (sinv(a,i4) < sinv(a,l4)) ) then
           passed = .true.
      !> s1 = sai && sbj < sbk && sbj < sbl
      else if ( (testij(im1, jm1, a, i4)) .and. &
           (sinv(b,j4) < sinv(b,k4)) .and. &
           (sinv(b,j4) < sinv(b,l4)) ) then
           passed = .true.
      endif

      if (.not.passed) then
         wedgecut(irot) = 0
      endif

   end subroutine 

   subroutine wedgeAB(irot)
      integer, intent(in) :: irot
      real(dp) :: s_a, s_b
      real(dp), dimension(6,6) :: sinv
      integer :: im1, im2, jm1, jm2
      logical :: passed 

      call findSmallest_4partons(irot, im1, im2, jm1, jm2, sinv)

      passed = .false.

      if (triplecol) then
         ! Wedges for a
         !    l  k 
         !    /  /
         ! ------ i
         ! ------ j
         !  sa(i,k,l) < sb(i,k,l), s1 = skl, s2 € {sik,sil}
         s_a = sinv(a,i4) + sinv(a,l4) + sinv(a,k4)
         s_b = sinv(b,i4) + sinv(b,l4) + sinv(b,k4)
         if ( ( s_a < s_b) .and. &
              testij(im1,jm1, k4, l4) .and. &
              ( testij(im2,jm2, i4, k4).or. &
                testij(im2,jm2, i4, l4)) ) then
            passed = .true.
         ! sai < saj, {s1,s2} € {sak, sal, skl}
         elseif ( ( sinv(a, i4) < sinv(a, j4) ) .and. &
              ( testij(im1,jm1, a, k4) .or. &
                testij(im1,jm1, a, l4) .or. &
                testij(im1,jm1, k4,l4)    ) .and. &
              ( testij(im2,jm2, a, k4) .or. &
                testij(im2,jm2, a, l4) .or. &
                testij(im2,jm2, k4,l4)    ) ) then
            passed = .true.
         endif
      else
         ! Wedges for b 
         !    / i
         ! ----- j
         ! ----- k
         !    \ l
         ! s1 = sik, s2 = sjl, sa(i+k) < sb(i+k)
         if ( testij(im1,jm1, i4, k4) .and. &
              testij(im2,jm2, j4, l4) .and. &
             ((sinv(a,i4)+sinv(a,k4))<(sinv(b,i4)+sinv(b,k4)))) then
            passed = .true.
         ! s1 = sjl, s2 = sik, sb(j+l) < sa(j+l)
         elseif ( testij(im1,jm1, j4, l4) .and. &
                  testij(im2,jm2, i4, k4) .and. &
                  ((sinv(b,j4)+sinv(b,l4))<(sinv(a,j4)+sinv(a,l4)))) then
            passed = .true.
         ! s1 = sai, s2 = sbj, sik < sil (and all a<>b, i<>j, ij<>kl
         ! combinations:
         elseif ( testij(im1,jm1, a,i4) .and. &
                  testij(im2,jm2, b,j4) .and. &
                  (sinv(i4,k4) < sinv(i4,l4)) ) then
            passed = .true.
         elseif ( testij(im1,jm1, a,i4) .and. &
                  testij(im2,jm2, b,l4) .and. &
                  (sinv(i4,k4) < sinv(i4,j4)) ) then
            passed = .true.
         elseif ( testij(im1,jm1, a,k4) .and. &
                  testij(im2,jm2, b,j4) .and. &
                  (sinv(k4,i4) < sinv(k4,l4)) ) then
            passed = .true.
         elseif ( testij(im1,jm1, a,k4) .and. &
                  testij(im2,jm2, b,l4) .and. &
                  (sinv(k4,i4) < sinv(k4,j4)) ) then
            passed = .true.
         elseif ( testij(im1,jm1, b,j4) .and. &
                  testij(im2,jm2, a,i4) .and. &
                  (sinv(j4,l4) < sinv(j4,k4)) ) then
            passed = .true.
         elseif ( testij(im1,jm1, b,j4) .and. &
                  testij(im2,jm2, a,k4) .and. &
                  (sinv(j4,l4) < sinv(j4,i4)) ) then
            passed = .true.
         elseif ( testij(im1,jm1, b,l4) .and. &
                  testij(im2,jm2, a,i4) .and. &
                  (sinv(l4,j4) < sinv(l4,k4)) ) then
            passed = .true.
         elseif ( testij(im1,jm1, b,l4) .and. &
                  testij(im2,jm2, a,k4) .and. &
                  (sinv(l4,j4) < sinv(l4,i4)) ) then
            passed = .true.
         ! {s1,s2} € {min(sai,sak), sjl}
         elseif ( ( testij(im1,jm1, j4,l4) .and. &
                    ( testij(im2,jm2, a, i4).or. &
                      testij(im2,jm2, a, k4) )) .or. &
                  ( testij(im2,jm2, j4,l4) .and. &
                    ( testij(im1,jm1, a, i4).or. &
                      testij(im1,jm1, a, k4) )) ) then
            passed = .true.
         ! {s1,s2} € {min(sbj,sbl), sik}
         elseif ( ( testij(im1,jm1, i4,k4) .and. &
                    ( testij(im2,jm2, b, j4).or. &
                      testij(im2,jm2, b, l4) )) .or. &
                  ( testij(im2,jm2, i4,k4) .and. &
                    ( testij(im1,jm1, b, j4).or. &
                      testij(im1,jm1, b, l4) )) ) then
            passed = .true.
         endif
      endif

      if (.not.passed) then
         wedgecut(irot) = 0
      endif
   end subroutine

   !-----------------------------------------------------------------------------
   !> @brief
   !>   ps functions from Byckling & Kajantie Particle Kinematics                               
   !>                                      
   !-----------------------------------------------------------------------------
   subroutine glimits(x, ydum, z, u, v, w, yp, ym)
      real(dp), intent(in) :: x, ydum, z, u, v, w
      real(dp), intent(out) :: yp, ym
      real(dp) :: dl1, dl2, ycom, yvar
      dl1 = dsqrt(dlambda(x, u, v))
      dl2 = dsqrt(dlambda(x, w, z))
      ycom = u + w - 1d0/2d0*(x+u-v)*(x+w-z)/x
      yvar = dl1*dl2/2d0/x
      ym = ycom - yvar
      yp = ycom + yvar
   end subroutine

   real(dp) function gres(x, y, z, u, v, w)
      real(dp), intent(in) :: x, y, z, u, v, w
      real(dp) :: yp, ym
      call glimits(x, 0d0, z, u, v, w,  yp, ym)
      gres = x*(y-yp)*(y-ym)
   end function

   subroutine phaseSequential(x, wt, ifail)
      real(dp), dimension(mxdim), intent(in) :: x
      real(dp), intent(out) :: wt
      integer, intent(out) :: ifail
      real(dp), dimension(4) :: pa, pb, rvar, pH
      real(dp), dimension(4), target :: p_out1
      real(dp), dimension(:), pointer :: pin
      integer :: i
      wt = 1d0
      call p2generic_nj(x(1),shat,sij(2),0d0, &
                     p_out1,pH,pa,pb,wt)
      pTemp(:,1,1) = pa
      pTemp(:,2,1) = pb
      pTemp(:,3,1) = pH
      pin => p_out1
      do i = 4, n-1
         rvar = (/sij(i-2), sij(i-1), cth(i-2), phi(i-2)/)
         call makePsWrapper(rvar, pin, p_out1, pTemp(:,i,1),NOROT)
         pin => p_out1
      enddo
      pTemp(:, n, 1) = pin
      ifail = 0
   end subroutine
   subroutine phaseSequentialBoson(x, wt, ifail)
      real(dp), dimension(mxdim), intent(in) :: x
      real(dp), intent(out) :: wt
      integer, intent(out) :: ifail
      real(dp), dimension(4) :: pa, pb, rvar, pH
      real(dp), dimension(4), target :: p_out1
      real(dp), dimension(:), pointer :: pin
      integer :: i
      wt = 1d0
      call p2generic_nj(x(1),shat,sij(2),sij(10), &
                     p_out1,pH,pa,pb,wt)
      pTemp(:,1,1) = pa
      pTemp(:,2,1) = pb

      ! "deatach" the massive boson
      rvar = (/sij(10), 0d0, cth(10), phi(10)/)
      call makePsWrapper(rvar, pH, pTemp(:,n,1), pTemp(:,n-1,1), NOROT)
      ! colorful loop
      pin => p_out1
      do i = 3, n-3
         rvar = (/sij(i-1), sij(i), cth(i-1), phi(i-1)/)
         call makePsWrapper(rvar, pin, p_out1, pTemp(:,i,1),NOROT)
         pin => p_out1
      enddo
      pTemp(:, n-2, 1) = pin
      ifail = 0
   end subroutine

   subroutine sampleLinearAll(x, ifail)
      double precision, dimension(mxdim), intent(in) :: x
      integer, intent(out)                           :: ifail
      double precision :: smin, smax
      integer          :: i, j, i_final
      sij = 0d0
      phi = 0d0
      cth = 0d0
      sij(1) = shat
      smin = y0*shat
      smax = shat - y0*shat
      ! "detach" the massive bosson (if any)
      if (inwa > 0) then
         i_final = n-4
         call sampleInwa(x(3*n-13), sij(10))
         call pickRand(1, cth(10), costhmin, costhmax, x(3*n-12), wtps)
         call pickRand(1, phi(10), phimin,   phimax,   x(3*n-11), wtps)
         smax = (dsqrt(shat) - dsqrt(sij(10)))*(dsqrt(shat) - dsqrt(sij(10)))
         wtps = wtps/2d0/pi
         wtps = wtps/32d0/pi**2
      else
         i_final = n-3
      endif
      do i = 2, i_final
         if (smin > smax) then
            ifail = 1
            return
         endif
         j = i + 2*(i-2)
         call pickRand(1, sij(i), smin,     smax,     x(j),   wtps)
         call pickRand(1, cth(i), costhmin, costhmax, x(j+1), wtps)
         call pickRand(1, phi(i), phimin,   phimax,   x(j+2), wtps)
         wtps = wtps/2d0/pi
         smax = sij(i) - y0*shat
         if(i.gt.2) then
            wtps = wtps*dsqrt(dlambda(sij(i-1), 0d0, sij(i)))/sij(i-1)/32d0/pi**2
         else
            wtps = wtps/32d0/pi**2
         endif
      enddo
      return
   end subroutine

   subroutine p2generic_nj(r,slab,s1,s2,p1,p2,pa,pb,wt)
      double precision, intent(in) :: r, slab, s1, s2
      double precision, dimension(4), intent(out) :: p1, p2, pa, pb
      double precision, intent(inout) :: wt
      double precision :: wtem, roots, Eab, pin, pout, E1, E2
      double precision :: ta1min, ta1max, costh, sinth, cp, sp
      double precision :: dlambda, ta1, rph
      p1(:) = 0d0
      p2(:) = 0d0
      pa(:) = 0d0
      pb(:) = 0d0
      wtem  = 1d0
      ta1   = 0d0
      rph   = 0d0

      ! Create initial mom. in the c.o.m.
      roots   = dsqrt(slab)
      Eab     = roots/2d0
      pin     = roots/2d0
      pa(4)   = Eab
      pb(4)   = Eab
      pa(3)   = pin
      pb(1:3) = -pa(1:3)

      E1   = (slab + s1 - s2)/2d0/roots
      E2   = (slab + s2 - s1)/2d0/roots
      pout = dsqrt(dlambda(slab,s1,s2))/2d0/roots
      ! Pick cosine p1-beam
      ta1min = s1 - 2d0*Eab*E1 - 2d0*pin*pout
      ta1max = s1 - 2d0*Eab*E1 + 2d0*pin*pout
      call pickRand(1,ta1,-ta1max,-ta1min,r,wtem)
      costh = (-ta1 - s1 + 2d0*Eab*E1)/(2d0*pin*pout)
      if(dabs(costh).gt.1d0)then
         print *, "Improper call of p2generic_nj @ phaseNJ"
         stop
      endif
      sinth = dsqrt(1d0 - costh**2)
      wtem  = wtem/16d0/pi**2/slab

      ! Pick phi angle between 0 and 2pI
      call pickRand(1,rph,0d0,2d0*pi,0d0,wtem)
      cp = cos(rph)
      sp = sin(rph)

      ! Create outgoing momenta
      p1(4) = E1
      p1(1) = pout*sinth*cp
      p1(2) = pout*sinth*sp
      p1(3) = pout*costh

      p2(4)   = E2
      p2(1:3) = - p1(1:3)

      wt = wt*wtem

      return
   end subroutine


end module

