!-------------------------------------------------------------------------------
! NNLOJET
!-------------------------------------------------------------------------------
!
! MODULE: ProcLib [external members: `_plib`]
!
! DESCRIPTION:
!> Module to manage dynamic process libraries
!-------------------------------------------------------------------------------


!-------------------------------------------------------------------------------
module ProcLib_mod
!-------------------------------------------------------------------------------
  use, intrinsic :: iso_c_binding
  use dlfcn_mod
  implicit none
  public

  character(32) :: name_plib, suff_plib
  type(c_ptr), private :: dl_handle = c_null_ptr
  logical :: qloaded_plib = .false.  ! some processes are hand crafted
  logical :: qmanual_plib = .false.  ! manual override of sig fctns?

  abstract interface
    !>
    subroutine selectchannel_intfce()
    end subroutine selectchannel_intfce
    !>
    subroutine getlumi_intfce(ix, iproc, partons)
      integer, intent(in) :: iproc, ix
      double precision, dimension(7), intent(out) :: partons
    end subroutine
    !>
    function getlumiid_intfce(iproc, ip1, ip2) result(id)
      integer :: id
      integer, intent(in) :: iproc
      integer, intent(out) :: ip1, ip2
    end function
    !>
    subroutine getchannel_intfce(ip, chan_str, me_str, sub_str)
      integer, intent(in) :: ip
      character(:), allocatable, intent(out) :: chan_str, me_str, sub_str
    end subroutine getchannel_intfce
    !>
    subroutine getqcdnorm_intfce(ix, partons, factor, npar, ipass, flav_block)
      integer, intent(in) :: ix, npar, flav_block
      integer, intent(inout) :: ipass
      real(8), intent(out), dimension(7) :: partons, factor !> @fix (deferred size array?)
    end subroutine getqcdnorm_intfce
    !>
    subroutine init_pine_intfce()
    end subroutine init_pine_intfce
    !>
    function sig_intfce(rnorm, x1, x2, wgt) result(sig)
      double precision :: rnorm, x1, x2, wgt
      double precision :: sig
    end function sig_intfce
    !>
    subroutine phaseN_intfce(x, wt, ifail)
      double precision, intent(in) :: x(*)
      double precision, intent(out) :: wt
      integer, intent(out) :: ifail
    end subroutine phaseN_intfce
    !>
    subroutine fillcommonNp_intfce()
    end subroutine fillcommonNp_intfce
    !>
  end interface

  interface load_sig_plib
    module procedure load_sig_by_suff
    module procedure load_sig_by_fctn
  end interface

  interface load_phase_plib
    ! module procedure load_phase_by_name
    module procedure load_phase_by_fctn
  end interface

  procedure(selectchannel_intfce), pointer :: selectchannel_dl => null()
  procedure(getchannel_intfce), pointer :: getchannel_dl => null()
  procedure(getqcdnorm_intfce), pointer :: getqcdnorm_dl => null()
  procedure(getlumi_intfce), pointer :: getlumi_dl => null()
  procedure(getlumiid_intfce), pointer :: getlumiid_dl => null()
  procedure(init_pine_intfce), pointer :: init_pine_dl => null()

  procedure(sig_intfce), pointer :: sigMAT_dl => sig_zero
  procedure(sig_intfce), pointer :: sigSUB_dl => sig_zero

  procedure(phaseN_intfce), pointer :: phaseN_dl => phaseN_null
  procedure(fillcommonNp_intfce), pointer :: fillcommonNp_dl => fillcommonNp_null


contains


  !-----------------------------------------------------------------------------
  !> @brief
  !> load dynamic process library and associate procedures
  !-----------------------------------------------------------------------------
  subroutine init_plib()
    use StrHelper_mod
    use Process_mod, only : name_proc
    type(c_funptr) :: c_fptr
    integer :: status

    qloaded_plib = .false.

    !> map process names to library names
    name_plib = trim(to_lower_str(name_proc))
    select case (name_plib)
    case( "1jet",   "2jet",  &
        & "1jetfc", "2jetfc" )
      name_plib = '2jetfc'

    case ( "gj" )
      name_plib = 'g'

    case ( "wp", "wpj", "wpjj", "wpjjj",  &
         & "wm", "wmj", "wmjj", "wmjjj" )
      name_plib = 'w' // trim(name_plib(3:))

    case ( "diswp", "diswp1j",  &
         & "diswm", "diswm1j" )
      name_plib = 'disw' // trim(name_plib(6:))

    case ( "hto2p", "hto2tau" )
      name_plib = 'h2'
    case ( "hto2pm", "hto2taum" )
      name_plib = 'h2m'
    case ( "hto2pj", "hto2tauj" )
      name_plib = 'h2j'
    case ( "hto2pjm", "hto2taujm" )
      name_plib = 'h2jm'
    case ( "hto2pjj", "hto2taujj" )
      name_plib = 'h2jj'
    case ( "hto2pjjj", "hto2taujjj" )
      name_plib = 'h2jjj'

    case ( "hto2l1p" )
      name_plib = 'h3'
    case ( "hto2l1pm" )
      name_plib = 'h3m'
    case ( "hto2l1pj" )
      name_plib = 'h3j'
    case ( "hto2l1pjm" )
      name_plib = 'h3jm'
    case ( "hto2l1pjj" )
      name_plib = 'h3jj'
    case ( "hto2l1pjjj" )
      name_plib = 'h3jjj'

    case ( "hto4e", "hto2e2mu", "hto2l2n" )
      name_plib = 'h4'
    case ( "hto4em", "hto2e2mum", "hto2l2nm" )
      name_plib = 'h4m'
    case ( "hto4ej", "hto2e2muj", "hto2l2nj" )
      name_plib = 'h4j'
    case ( "hto4ejm", "hto2e2mujm", "hto2l2njm" )
      name_plib = 'h4jm'
    case ( "hto4ejj", "hto2e2mujj", "hto2l2njj" )
      name_plib = 'h4jj'
    case ( "hto4ejjj", "hto2e2mujjj", "hto2l2njjj" )
      name_plib = 'h4jjj'

    case ( "vfhto2p", "vfhto2tau" )
      name_plib = "vfh2"
    case ( "vfhto2pj", "vfhto2tauj" )
      name_plib = "vfh2j"
    case ( "vfhto2l1p" )
      name_plib = "vfh3"
    case ( "vfhto2l1pj" )
      name_plib = "vfh3j"
    case ( "vfhto4e", "vfhto2e2mu", "vfhto2l2n" )
      name_plib = "vfh4"
    case ( "vfhto4ej", "vfhto2e2muj", "vfhto2l2nj" )
      name_plib = "vfh4j"

    !> "hand-crafted" processes
!      case('HINC','HTO2PINC','HTO2TAUINC','HTO2L1PINC',
!     &     'HTO2E2MUINC','HTO4EINC','HTO2L2NEINC')
!         call init_Hinc()
!         call cross(sigVal,sd,sigHinc,ezero,ezero,ezero,ezero
!     .                               ,ezero,ezero,ezero,ezero
!     .                                           ,ezero,ezero)
!      case('HQT','HTO2PQT','HTO2TAUQT','HTO2L1PQT',
!     &     'HTO2E2MUQT','HTO4EQT','HTO2L2NEQT')
!         call init_Hqt()
!         call cross(sigVal,sd,sigHqt,ezero,ezero,ezero,ezero
!     .                               ,ezero,ezero,ezero,ezero
!     .                                           ,ezero,ezero)
!      case('ZQT')
!         call init_Zqt()
!         call cross(sigVal,sd,sigZqt,ezero,ezero,ezero,ezero
!     .                               ,ezero,ezero,ezero,ezero
!     .                                           ,ezero,ezero)
!      case('WPQT', 'WMQT')
!         call init_Wqt()
!         call cross(sigVal,sd,sigWqt,ezero,ezero,ezero,ezero
!     .                               ,ezero,ezero,ezero,ezero
!     .                                           ,ezero,ezero)
!      case('EPEMINC')
!         call init_EEinc()
!         call cross(sigVal,sd,sigEEinc,ezero,ezero,ezero,ezero
!     .                                ,ezero,ezero,ezero,ezero
!     .                                           ,ezero,ezero)
!      case('SFDIS')
!         !>SFDIS has only virtual integration
!         call cross(sigVal,sd,ezero,sigVSFDIS,sigTNLOSFDIS,ezero,ezero
!     .                                ,ezero,ezero,ezero,ezero
!     .                                            ,ezero,ezero)
!      case('SFDISWP')
!         !>SFDIS has only virtual integration
!         call cross(sigVal,sd,ezero,sigVSFDISWp,sigTNLOSFDISWp,ezero,ezero
!     .                                ,ezero,ezero,ezero,ezero
!     .                                            ,ezero,ezero)
!      case('SFDISWM')
!         !>SFDIS has only virtual integration
!         call cross(sigVal,sd,ezero,sigVSFDISWm,sigTNLOSFDISWm,ezero,ezero
!     .                                ,ezero,ezero,ezero,ezero
!     .                                            ,ezero,ezero)
!
!     special process for checking the unit volume of integration
!       case('UNIT1')
!          call cross(sigVal,sd,eunit,ezero,ezero,ezero,ezero
!      .                             ,eunit,ezero,ezero,ezero
!      .                                         ,eunit,ezero)


    end select

    dl_handle = open_dl("libnnlojet_"//trim(name_plib))

    !> suffix for the process functions
    suff_plib = name_plib
    if ( to_lower_str(name_proc(1:2)).eq.'wp' .or.  &
       & to_lower_str(name_proc(1:2)).eq.'wm' .or.  &
       & to_lower_str(name_proc(1:5)).eq.'diswp' .or.  &
       & to_lower_str(name_proc(1:5)).eq.'diswm' ) then
      suff_plib = trim(to_lower_str(name_proc))
    end if
    if ( suff_plib .eq. "g" ) then
      suff_plib = trim(to_lower_str(name_proc))
    end if

    c_fptr = sym_dl(dl_handle, "selectchannel"//trim(suff_plib), mangle=.true.)
    call c_f_procpointer(cptr=c_fptr, fptr=selectchannel_dl)

    c_fptr = sym_dl(dl_handle, "getchannel"//trim(suff_plib), mangle=.true.)
    call c_f_procpointer(cptr=c_fptr, fptr=getchannel_dl)

    c_fptr = sym_dl(dl_handle, "getqcdnorm"//trim(suff_plib), mangle=.true.)
    call c_f_procpointer(cptr=c_fptr, fptr=getqcdnorm_dl)

    c_fptr = sym_dl(dl_handle, "getlumi"//trim(suff_plib), mangle=.true.)
    call c_f_procpointer(cptr=c_fptr, fptr=getlumi_dl)

    c_fptr = sym_dl(dl_handle, "getlumiid"//trim(suff_plib), mangle=.true.)
    call c_f_procpointer(cptr=c_fptr, fptr=getlumiid_dl)

#ifdef USE_PINEAPPL
    c_fptr = sym_dl(dl_handle, "init_pine_lumi"//trim(suff_plib), status=status, mangle=.true.)
    if (status==0) call c_f_procpointer(cptr=c_fptr, fptr=init_pine_dl)
    print*, "init_pine_lumi", trim(suff_plib),":", status
#endif

    ! call load_proc_dl()
    qloaded_plib =.true.

  end subroutine init_plib
  !>>>>>> after init_proc, we'll read in the setting (also process dependent ones)
  !>>>>>> THEN we call init_param
  !>>>>>> THEN we call setup_proc (where igamma is REMOVED)


  !-----------------------------------------------------------------------------
  !> @brief
  !> load `sig` routines with specified suffixes from the dynamic library
  !
  !> @param[in] mat  optional name of ME sig routine
  !> @param[in] sub  optional name of subtraction sig routine
  !-----------------------------------------------------------------------------
  subroutine load_sig_by_suff(mat, sub)
    use StrHelper_mod
    character(*), intent(in), optional :: mat, sub
    type(c_funptr) :: c_fptr
    integer :: status

    !> @todo: hand-crafted processes should load once above and here just return?
    if (.not.qloaded_plib .or. qmanual_plib) return

    sigMAT_dl => sig_zero
    if ( present(mat) ) then
      c_fptr = sym_dl(dl_handle, "sig"//to_lower_str(trim(mat))//trim(suff_plib), status, mangle=.true.)
      if (status==0) call c_f_procpointer(cptr=c_fptr, fptr=sigMAT_dl)
      ! print*, "MAT: sig", trim(mat),":", status
    end if

    sigSUB_dl => sig_zero
    if ( present(sub) ) then
      c_fptr = sym_dl(dl_handle, "sig"//to_lower_str(trim(sub))//trim(suff_plib), status, mangle=.true.)
      if (status==0) call c_f_procpointer(cptr=c_fptr, fptr=sigSUB_dl)
      ! print*, "SUB: sig", trim(sub),":", status
    end if

  end subroutine load_sig_by_suff


  !-----------------------------------------------------------------------------
  !> @brief
  !> set `sig` routines by directly passing functions
  !
  !> @param[in] mat  function for ME sig routine
  !> @param[in] sub  function for subtraction sig routine
  !-----------------------------------------------------------------------------
  subroutine load_sig_by_fctn(mat, sub)
    procedure(sig_intfce) :: mat, sub
    sigMAT_dl => mat
    sigSUB_dl => sub
    qmanual_plib = .true.
  end subroutine load_sig_by_fctn


  !-----------------------------------------------------------------------------
  !> @brief
  !> set phase space routines
  !
  !> @param[in] phaseN  routine for phase-space generation
  !> @param[in] fillcommonNp  routine to populate common blocks
  !-----------------------------------------------------------------------------
  subroutine load_phase_by_fctn(phaseN, fillcommonNp)
    procedure(phaseN_intfce) :: phaseN
    procedure(fillcommonNp_intfce) :: fillcommonNp
    phaseN_dl => phaseN
    fillcommonNp_dl => fillcommonNp
  end subroutine load_phase_by_fctn


  ! !-----------------------------------------------------------------------------
  ! !> @brief
  ! !> set phase space routines that require manual treatment
  ! !
  ! !> @param[in] phaseN  routine for phase-space generation
  ! !> @param[in] fillcommonNp  routine to populate common blocks
  ! !-----------------------------------------------------------------------------
  ! subroutine load_phase_by_name()
  !   use StrHelper_mod
  !   use Process_mod, only : name_proc
  !   ! use PhaseNJ_mod, only : phaseNJ  ! VFH
  !   ! select case (to_upper_str(name_proc))
  !   ! case ("VFH", "VFHJ")
  !   !   phaseN_dl => phaseNJ
  !   !   fillcommonNp_dl => fillcommonNp_null
  !   ! case default
  !   !   print*, "load_phase_by_name: unknown case: ", name_proc
  !   ! end select
  ! end subroutine load_phase_by_name


  !-----------------------------------------------------------------------------
  !> @brief
  !> un-load dynamic process library
  !-----------------------------------------------------------------------------
  subroutine destroy_plib()

    selectchannel_dl => null()
    getchannel_dl => null()
    getqcdnorm_dl => null()
    getlumi_dl => null()
    getlumiid_dl => null()
    init_pine_dl => null()

    sigMAT_dl => sig_zero
    sigSUB_dl => sig_zero

    call close_dl(dl_handle)
    dl_handle = c_null_ptr

    qloaded_plib = .false.
    qmanual_plib = .false.

  end subroutine destroy_plib


  !-----------------------------------------------------------------------------
  !> @brief
  !> compute the parton luminosity
  !
  !> @param[in] ix
  !> @param[in] iproc
  !> @param[in] partons
  !> @param[in] fac optional
  !> @param[in] obsId optional
  !-----------------------------------------------------------------------------
  subroutine getlumi_plib(ix, iproc, partons, fac, obsId)
    use Scales_mod
    use ScaleCoeff_mod, only : isActive_sclc
    integer, intent(in) :: ix, iproc
    double precision, intent(out) :: partons(nMax_scl)
    double precision, optional, intent(in) :: fac
    integer, optional, intent(in) :: obsId

    !> for coefficient functions: not needed
    if ( isActive_sclc() .and.  &
       & .not.present(fac) .and.  &
       & .not.present(obsId) ) then
      partons(:) = 1d0  ! luminosity build up in bino
      return
    end if

    if ( present(fac) .or. present(obsId) ) then
      call updatePDF_scl(withXRegions=(ix/=1), fac=fac, obsId=obsId)
    end if

    call getlumi_dl(ix, iproc, partons)

  end subroutine getlumi_plib


  function sig_zero(rnorm, x1, x2, wgt) result(sig)
    double precision :: rnorm, x1, x2, wgt
    double precision :: sig
    sig = 0d0
  end function sig_zero


  subroutine phaseN_null(x, wt, ifail)
    double precision, intent(in) :: x(*)
    double precision, intent(out) :: wt
    integer, intent(out) :: ifail
    stop "phaseN_null"
  end subroutine phaseN_null


  subroutine fillcommonNp_null()
    stop "fillcommonNp_null"
  end subroutine fillcommonNp_null


  subroutine get_str_chan_inc(ip, chan_str, me_str, sub_str)
    integer, intent(in) :: ip
    character(:), allocatable, intent(out) :: chan_str, me_str, sub_str
    if (ip /= 1) then
      print*, 'get_str_chan_inc: invalid ip:', ip
      stop
    end if
    chan_str = name_plib
    me_str   = name_plib
    sub_str  = name_plib
  end subroutine get_str_chan_inc


end module ProcLib_mod


!//////////////////////////////////////////
!//  wrappers & helpers in global scope  //
!//////////////////////////////////////////


subroutine getqcdnorm(ix, partons, factor, npar, ipass, flav_block)
  ! use Process_mod, only : name_proc
  use ProcLib_mod
  use Scales_mod, only : nMax_scl
  implicit none
  integer, intent(in) :: ix
  double precision, intent(out) :: partons(nMax_scl), factor(nMax_scl)
  integer, intent(in) :: npar
  integer, intent(inout) :: ipass
  integer, intent(in) :: flav_block

  call getqcdnorm_dl(ix, partons, factor, npar, ipass, flav_block)

!> @todo: absorb into makeproc?
! case('SFDIS')
!    call getqcdnormSFDIS(ix, partons, factor, npar, ipass, flav_block)
!    !> all factors are included in SFDIS.f90
!    !> ==> reset all factors set by getqcdnormSFDIS to one
!    factor = 1d0
! case('SFDISWP')
!    call getqcdnormSFDISWp(ix, partons, factor, npar, ipass, flav_block)
!    !> all factors are included in SFDIS.f90
!    !> ==> reset all factors set by getqcdnormSFDIS to one
!    factor = 1d0
! case('SFDISWM')
!    call getqcdnormSFDISWm(ix, partons, factor, npar, ipass, flav_block)
!    !> all factors are included in SFDIS.f90
!    !> ==> reset all factors set by getqcdnormSFDIS to one
!    factor = 1d0

end subroutine getqcdnorm


#ifdef USE_OL
subroutine getqcdnormOL(ix, partons, factor, npar, ipass, flav_block)
  use ProcLib_mod
  use Scales_mod, only : nMax_scl
  implicit none
  integer, intent(in) :: ix
  double precision, intent(out) :: partons(nMax_scl), factor(nMax_scl)
  integer, intent(in) :: npar
  integer, intent(inout) :: ipass
  integer, intent(in) :: flav_block

  stop "getqcdnormOL: not linked yet"
  ! call getqcdnormOL_dl(ix, partons, factor, npar, ipass, flav_block)

end subroutine getqcdnormOL
#endif


pure double precision function get_avg(ip1, ip2)
  implicit none
  integer, intent(in) :: ip1, ip2
  double precision, parameter :: nc = 3d0

  get_avg = 0.25d0

  if(ip1.eq.0) then
    get_avg = get_avg/(nc**2-1d0)
  elseif( (abs(ip1)/=99) .and. (ip1/=7) ) then
    get_avg = get_avg/nc
  endif

  if(ip2.eq.0) then
    get_avg = get_avg/(nc**2-1d0)
  elseif( (abs(ip2)/=99) .and. (ip2/=7) ) then
    get_avg = get_avg/nc
  endif

end function get_avg
