module RESDIS_mod
  use Kinds_mod, only: wp
  use Constants_mod, only: pi
  implicit none
  private
  public :: Rint

  real(kind=wp), parameter :: rnup = 2._wp
  real(kind=wp), parameter :: rndown = 3._wp
  real(kind=wp), parameter :: rnf = 5._wp
!> rnup*eu2+rndown*ed2
  real(kind=wp), parameter :: sumEQ2 = 1.2222222222222222_wp
  real(kind=wp), parameter :: gevfb = 389.37936e9_wp

contains

!> This is only the nnlo correction
!> NB this si only the approximate nnlo, which should agree to 1%
!> level with the real strucutre function
  real(kind=wp) function Rint(iord, ix, xi)
    !use Constants_mod, only : pi
    use Scales_mod
    use KinData_mod, only: xreg1_kin
    use CoeffFunc_av
    use Channels_mod, only: ip1_chan
    use DIS_mod
    integer, intent(in) :: ix, iord
    integer idebug
    real(kind=wp), intent(in) :: xi
    integer nflav, i
    !> bino function
    real(kind=wp) :: bino
    real(kind=wp) :: amz, as!,c2qcox,c2qco1,c2gcox,c2qpscox,c2qdel
    !> debuggig purposes
    !real(kind=wp) :: c2q_new,c2g_new,c2qps_new
    !real(kind=wp) :: cLq_new,cLg_new,cLqps_new
    !real(kind=wp) :: clqcox,clqco1,clgcox,clqpscox,clqdel
    real(kind=wp) :: f2(nMax_scl), fl(nMax_scl), dsig(nMax_scl)
    real(kind=wp) :: partons(nMax_scl), facnorm(nMax_scl)
    real(kind=wp) :: xbj, q2, x1curr, x2curr, y
    real(kind=wp) :: rma2, rmb2, rm2, shat, Jxtoy
    complex*16 :: zewfac(4), zewnull(4)
    integer :: ipass

    real(kind=wp) :: xvegas
    real(kind=wp) :: u, L, rho
    real(kind=wp) :: c2qgcox
    real(kind=wp) :: clqgcox
    real(kind=wp) :: c2qdel, c2qco1, c2qcox
    real(kind=wp) :: clqdel, clqco1, clqcox
    real(kind=wp) :: c2gdel, c2gco1, c2gcox
    real(kind=wp) :: clgdel, clgco1, clgcox
    real(kind=wp) :: c1g(nMax_scl), c1q(nMax_scl)
    real(kind=wp) :: CA, nf, beta0

    parameter(CA=3._wp)
    parameter(nf=5._wp)

    common/parfrac/x1curr, x2curr
!$omp threadprivate(/parfrac/)
    common/eweakZ/amz, zewfac, zewnull
    !$omp threadprivate(/eweakZ/)
    common/BZFlav/nflav
    !$omp threadprivate(/BZFlav/)
    common/pmasses/rma2, rmb2, rm2(1:7), shat
!$omp threadprivate(/pmasses/)
!    common /vdata/xvegas(20)
!!$omp threadprivate(/vdata/)

    beta0 = (11._wp*CA - 2._wp*nf)/(12._wp*pi)

    if (ix .ne. 1 .and. ix .ne. 3) then
      write (*, *) "SOMETHING IS WRONG WITH ix ", ix
      stop
    end if

    !ecuts -> return 0 on failure
    ipass = 1
    call ecuts_dis1j(0, 4, ipass)
    if (ipass .eq. 0) then
      Rint = 0._wp
      return
    end if

    call getqcdnorm(ix, partons, facnorm)
    call eweakV1qq(nflav, 0._wp)

    !f2   = 0._wp
    !fl   = 0._wp
    c1g = 0._wp
    c1q = 0._wp
    dsig = 0._wp

    q2 = value_q2_dis
    y = value_y_dis
    xbj = x1curr

    do i = 1, nAct_scl
      !write(*,*) '****************************************************************'
      !write(*,*) 'iord, ix, xi = : ', iord, ix, xi
      !write(*,*) 'ip1_chan = : ', ip1_chan
      !write(*,*) 'nAct_scl, i = :', nAct_scl, i
      !write(*,*) 'xbj, xreg1_kin, y, q2 = : ', xbj, xreg1_kin, y, q2
      !write(*,*) 'amz = : ', amz
      !write(*,*) 'zewfac = : ', zewfac
      !> alpha s
      as = alphas_scl(i)

      !> Gluon coefficient (has no delta part)
      !> This is the delta part
      !if(ix.eq.1) then
      !   if(ip1_chan.ne.0) then
      !      !c2qdel = c2q_new(iord,xbj,      0,muR2_scl(i),muF2_scl(i),q2)
      !      !c2qco1 = c2q_new(iord,xreg1_kin,2,muR2_scl(i),muF2_scl(i),q2)
      !      c2qgdel = ResCFc2qg(iord,xbj,      0,muR2_scl(i),muF2_scl(i),q2)
      !      c2qgco1 = ResCFc2qg(iord,xreg1_kin,2,muR2_scl(i),muF2_scl(i),q2)
      !      clqgdel = ResCFclqg(iord,xbj,      0,muR2_scl(i),muF2_scl(i),q2)
      !      clqgco1 = ResCFclqg(iord,xreg1_kin,2,muR2_scl(i),muF2_scl(i),q2)
      !      !f2(i) =  (as/4._wp/pi)**iord*abs(zewfac(1))**2*(c2qdel+c2qco1*(1._wp-xbj))
      !      c1g(i) = c2qgdel+c2qgco1 + y**2/(1._wp+(1._wp-y)**2)*(clqgdel+clqgco1)
      !      c1g(i) = c1g(i)*(as/2._wp/pi)**iord*sumEQ2
      !   endif
      !  !> This is the convolution
      !elseif(ix.eq.3) then
      if (ix .eq. 3) then
        !call inithpls5(xi)! Not needed, right?
        if (ip1_chan .eq. 0) then
          !> factor of 2._wp because we are taking u and d separately
          !> would give a double counting for nf pieces
          !c2gcox   = c2g_new(iord,xi,1,muR2_scl(i),muF2_scl(i),q2) / 2._wp
          c2qgcox = ResCFc2qg(iord, xi, 1, muR2_scl(i), muF2_scl(i), q2)/2._wp
          clqgcox = ResCFclqg(iord, xi, 1, muR2_scl(i), muF2_scl(i), q2)/2._wp
          !f2(i) =  (as/4._wp/pi)**iord*sumEQ2*c2gcox/xi
          c1g(i) = c2qgcox + y**2/(1._wp + (1._wp - y)**2)*clqgcox
          c1g(i) = c1g(i)*(as/2._wp/pi)**iord*sumEQ2
          !else
          !   !c2qcox   = c2q_new(iord,xi,1,muR2_scl(i),muF2_scl(i),q2)
          !   c2qgcox   = ResCFc2qg(iord,xi,1,muR2_scl(i),muF2_scl(i),q2)
          !   clqgcox   = ResCFclqg(iord,xi,1,muR2_scl(i),muF2_scl(i),q2)
          !   !c2qpscox = c2qps_new(iord,xi,1,muR2_scl(i),muF2_scl(i),q2)
          !   !f2(i) =  (as/4._wp/pi)**iord*(abs(zewfac(1))**2*c2qcox/xi &
          !   !     &  +sumEQ2*c2qpscox/xi)
          !   c1g(i) = c2qgcox + y**2/(1._wp+(1._wp-y)**2)*clqgcox
          !   c1g(i) = c1g(i)*(as/2._wp/pi)**iord*sumEQ2
        end if
        !> Jacobian factor for rreg1 pick
        !> + compensates for xi in ix.eq.3
        !f2(i)=f2(i)*(1._wp-xbj)*xi
        c1g(i) = c1g(i)*(1._wp - xbj)*xi! Is this needed???
        ! Still have to include the normalization to f
      end if

      !f2(i)=f2(i)*xbj
      c1g(i) = c1g(i)*xbj! Is this needed???

      !> Quark coefficient
      if (ix .eq. 1) then
        if (ip1_chan .ne. 0) then
          !clqdel  = clq_new(iord,xbj,0,muR2_scl(i),muF2_scl(i),q2)
          !clqco1  = clq_new(iord,xreg1_kin,2,muR2_scl(i),muF2_scl(i),q2)
          c2qdel = ResCFc2q(iord, xbj, 0, muR2_scl(i), muF2_scl(i), q2)
          c2qco1 = ResCFc2q(iord, xreg1_kin, 2, muR2_scl(i), muF2_scl(i), q2)
          clqdel = ResCFclq(iord, xbj, 0, muR2_scl(i), muF2_scl(i), q2)
          clqco1 = ResCFclq(iord, xreg1_kin, 2, muR2_scl(i), muF2_scl(i), q2)
          c2gdel = ResCFc2g(iord, xbj, 0, muR2_scl(i), muF2_scl(i), q2)
          c2gco1 = ResCFc2g(iord, xreg1_kin, 2, muR2_scl(i), muF2_scl(i), q2)
          clgdel = ResCFclg(iord, xbj, 0, muR2_scl(i), muF2_scl(i), q2)
          clgco1 = ResCFclg(iord, xreg1_kin, 2, muR2_scl(i), muF2_scl(i), q2)
          !fl(i)   =  (as/4._wp/pi)**iord*abs(zewfac(1))**2*(clqdel+clqco1*(1._wp-xbj))
          c1q(i) = c2qdel + c2qco1 + c2gdel + c2gco1 + &
                   y**2/(1._wp + (1._wp - y)**2)*(clqdel + clqco1 + clgdel + clgco1)
          c1q(i) = c1q(i)*(as/2._wp/pi)**iord*abs(zewfac(1))**2
        end if
        !> This is the convolution
      elseif (ix .eq. 3) then
        !call inithpls5(xi)! Not needed, right?
        !if(ip1_chan.eq.0) then
          !!> factor of 2._wp because we are taking u and d separately
          !!> would give a double counting for nf pieces
        !   !clgcox   = clg_new(iord,xi,1,muR2_scl(i),muF2_scl(i),q2)/2._wp
        !   c2qcox   = ResCFc2q(iord,xi,1,muR2_scl(i),muF2_scl(i),q2) / 2._wp
        !   clqcox   = ResCFclq(iord,xi,1,muR2_scl(i),muF2_scl(i),q2) / 2._wp
        !   c2gcox   = ResCFc2g(iord,xi,1,muR2_scl(i),muF2_scl(i),q2) / 2._wp
        !   clgcox   = ResCFclg(iord,xi,1,muR2_scl(i),muF2_scl(i),q2) / 2._wp
        !   !fl(i) =  (as/4._wp/pi)**iord*sumEQ2*clgcox/xi
        !   c1q(i) = c2qcox-c2qgcox+c2gcox + y**2/(1._wp+(1._wp-y)**2)*(clqcox-clqgcox+clgcox)
        !   c1q(i) = c1q(i)*(as/2._wp/pi)**iord*abs(zewfac(1))**2
        !else
        if (ip1_chan .ne. 0) then
          !clqcox   = clq_new(iord,xi,1,muR2_scl(i),muF2_scl(i),q2)
          !clqpscox = clqps_new(iord,xi,1,muR2_scl(i),muF2_scl(i),q2)
          c2qcox = ResCFc2q(iord, xi, 1, muR2_scl(i), muF2_scl(i), q2)
          clqcox = ResCFclq(iord, xi, 1, muR2_scl(i), muF2_scl(i), q2)
          c2gcox = ResCFc2g(iord, xi, 1, muR2_scl(i), muF2_scl(i), q2)
          clgcox = ResCFclg(iord, xi, 1, muR2_scl(i), muF2_scl(i), q2)
          !fl(i) =  (as/4._wp/pi)**iord*( abs(zewfac(1))**2*clqcox/xi &
          !&  +sumEQ2*clqpscox/xi)
          c1q(i) = c2qcox - c2qgcox + c2gcox + y**2/(1._wp + (1._wp - y)**2)*(clqcox - clqgcox + clgcox)
          c1q(i) = c1q(i)*(as/2._wp/pi)**iord*abs(zewfac(1))**2
        end if
        !> Jacobian factor for rreg1 pick
        !> + compensates for xi in ix.eq.3
        !fl(i)=fl(i)*(1._wp-xbj)*xi
        c1q(i) = c1q(i)*(1._wp - xbj)*xi ! Is this needed???
      end if

      !fl(i)=fl(i)*xbj
      c1q(i) = c1q(i)*xbj

      rho = value_resrho_dis
      L = dlog(1._wp/rho)
      u = beta0*as*L

      !write(*,*) 'Total 2 parts: c2qdel, c2qco1, c2qcox = : ', c2qdel, c2qco1, c2qcox
      !write(*,*) 'Gluon 2 parts: c2qgcox = : ', c2qgcox
      !write(*,*) 'Quark 2 parts: c2qdel, c2qco1, c2qcox-c2qgcox = : ', &
      !         c2qdel, c2qco1, c2qcox-c2qgcox
      !write(*,*) 'Additional quark 2 parts: c2gdel, c2gco1, c2gcox = : ', c2gdel, c2gco1, c2gcox
      !write(*,*) 'Total l parts: clqdel, clqco1, clqcox = : ', clqdel,clqco1,clqcox
      !write(*,*) 'Gluon l parts: clqgcox = : ', clqgcox
      !write(*,*) 'Quarks l parts: clqdel, clqco1, clqcox-clqgcox = : ', &
      !         clqdel, clqco1, clqcox-clqgcox
      !write(*,*) 'Additional quark l parts: clgdel, clgco1, clgcox = : ', clgdel, clgco1, clgcox
      !write(*,*) 'c1q(1) = : ', c1q(1) ! do only 1 scale for now
      !write(*,*) 'c1g(1) = : ', c1g(1)
      !write(*,*) 'u, L, rho = : ', u, L, rho
      !write(*,*) 'xvegas(:) = : ', xvegas(:)
      !write(*,*) '------------------------------------------------------------'
    end do

    !> Jacobian factor to switch from y to x sampling
    !Jxtoy=q2/shat/xbj ! Need this?
    !dsig(:) = Jxtoy*2._wp*pi*amz**2._wp/q2**2._wp/y*( (1._wp+(1._wp-y)**2._wp)*f2(:) - y**2._wp*fl(:))
    !> 8*pi is the phase space integal that would otherwise been performed twice
    !dsig(:)=gevfb*dsig(:)*8._wp*pi*shat

    dsig(:) = c1q(:)*ResS(u)*Sigmaq(L, u) + c1g(:)*Sigmag(L, u)

    Rint = bino(ix, partons, dsig*facnorm)

    !write(*,*) 'facnorm(1) = : ', facnorm(1)
    !write(*,*) 'partons(:) = : ', partons(:)
    !write(*,*) 'dsig(:) = : ', dsig(:)
    !write(*,*) 'Rint = : ', Rint
    !write(*,*) '****************************************************************'
  end function Rint

! Resummation functions. u = beta0*as*L, L = log(1/rho)
  function Resf1(u) result(f1)
    implicit none
    real*8 u, f1, CF, CA, nf, beta0
    real*8 u1, u2, dlu1, dlu2

    parameter(CF=1.333333333333333333_wp)
    parameter(CA=3._wp)
    parameter(nf=5._wp)

    beta0 = (11._wp*CA - 2._wp*nf)/(12._wp*pi)

    u1 = 1._wp - u
    u2 = 1._wp - 2._wp*u
    dlu1 = dlog(u1)
    dlu2 = dlog(u2)

    f1 = -CF/(2._wp*pi*beta0*u)*(u2*dlu2 - 2._wp*u1*dlu1)
  end function Resf1

  function Resf2(u) result(f2)
    implicit none
    real*8 f2, u, ge, CF, CA, nf, K, beta0, beta1
    real*8 u1, u2, dlu1, dlu2

    parameter(ge=0.577215664901532860_wp)
    parameter(CA=3._wp)
    parameter(CF=1.333333333333333333_wp)
    parameter(nf=5._wp)

    beta0 = (11._wp*CA - 2._wp*nf)/(12._wp*pi)
    beta1 = (17._wp*CA**2 - 5._wp*CA*nf - 3._wp*CF*nf)/(24._wp*pi**2)

    u1 = 1._wp - u
    u2 = 1._wp - 2._wp*u
    dlu1 = dlog(u1)
    dlu2 = dlog(u2)

    K = CA*(67._wp/18._wp - pi**2/6._wp) - 5._wp/9._wp*nf

    f2 = -CF*K/(4._wp*pi**2*beta0**2)*(2._wp*dlu1 - dlu2)
    f2 = f2 - 3._wp*CF/(4._wp*pi*beta0)*dlu1
    f2 = f2 - CF*ge/(pi*beta0)*(dlu1 - dlu2)
    f2 = f2 - CF*beta1/(2._wp*pi*beta0**3)*(dlu2 - 2._wp*dlu1 + 1._wp/2._wp*dlu2**2 - dlu1**2)
  end function Resf2

  function ResS(u) result(S)
! Non-global resummation factor S
    real*8 u, t, a, b, c, S, CF, CA, nf, beta0

    parameter(CF=1.333333333333333333_wp)
    parameter(CA=3._wp)
    parameter(nf=5._wp)

    beta0 = (11._wp*CA - 2._wp*nf)/(12._wp*pi)

    t = 1._wp/(4._wp*pi*beta0)*dlog(1._wp/(1._wp - 2._wp*u))
    a = 0.85_wp*CA
    b = 0.86_wp*CA
    c = 1.33_wp

    S = dexp(-CF*CA*pi**2/3._wp*(1._wp + (a*t)**2)/(1._wp + (b*t)**c)*t**2)
  end function ResS

  function ResR(L, u) result(R)
! Radiator R to NLL accuracy
    implicit none
    real*8 L, u, R!,Resf1,Resf2

    R = -L*Resf1(u) - Resf2(u)
  end function ResR

  function ResRp(L, u) result(Rp)
! derivative of ResR
    real*8 L, u, Rp
    real*8 CF, CA, nf, beta0

    parameter(CA=3._wp)
    parameter(CF=1.333333333333333333_wp)
    parameter(nf=5._wp)

    beta0 = (11._wp*CA - 2._wp*nf)/(12._wp*pi)

    Rp = -CF*(dlog(1._wp - 2._wp*u) - dlog(1._wp - u))/(pi*beta0)
  end function ResRp

  function Sigmaq(L, u)
    real*8 Sigmaq, L, u
    !real*8 ResR,ResRp

    Sigmaq = dexp(-ResR(L, u))/gamma(1._wp + ResRp(L, u))
    return
  end function Sigmaq

  function Sigmag(L, u)
    real*8 Sigmag, L, u, CA, CF
    !real*8 Resf1

    parameter(CA=3._wp)
    parameter(CF=1.333333333333333333_wp)
    Sigmag = dexp(L*CA/CF*Resf1(u))
    return
  end function Sigmag

! Coefficient functions in MSbar scheme
! rmu3, fmu3, q2 are actually not needed in all the following coefficient functions. Remove later.
  function ResCFc2q(iord, x, ireg, rmu2, fmu2, q2) result(c2q)
    implicit none
    integer iord, ireg
    real*8 CF, q2, rmu2, fmu2
    real*8 x1, dlx, dl1x!,dlm,dlr
    real*8 dd0, dd1
    real*8 c2q, x
    parameter(CF=1.333333333333333333_wp)

    x1 = 1._wp - x
    dlx = dlog(x)
    dl1x = dlog(x1)

    !dlm = dlog(q2/fmu2)
    !dlr = dlog(rmu2/fmu2)

    if (x .lt. 0.5_wp) then
      if (ireg .eq. 0) then
        dd0 = dl1x
        dd1 = 1._wp/2._wp*dl1x**2
        if (iord .eq. 0) then
          c2q = 1._wp
        elseif (iord .eq. 1) then
          c2q = -3._wp/2._wp*dd0 + 2._wp*dd1 - (pi**2/3._wp + 9._wp/2._wp)
          c2q = CF*c2q
        end if
      elseif (ireg .eq. 1) then
        dd0 = 1._wp/x1
        dd1 = dl1x/x1
        if (iord .eq. 0) then
          c2q = 0._wp
        elseif (iord .eq. 1) then
          c2q = -3._wp/2._wp*dd0 + 2._wp*dd1 - (1._wp + x)*dl1x &
                - (1._wp + x**2)/x1*dlx + 3._wp + 2._wp*x
          c2q = CF*c2q
        end if
      elseif (ireg .eq. 2) then
        dd0 = -1._wp/x1
        dd1 = -dl1x/x1
        if (iord .eq. 0) then
          c2q = 0._wp
        elseif (iord .eq. 1) then
          c2q = -3._wp/2._wp*dd0 + 2._wp*dd1
          c2q = CF*c2q
        end if
      end if
    elseif (x .gt. 0.5_wp) then
      if (ireg .eq. 0) then
        dd0 = dl1x
        dd1 = 1._wp/2._wp*dl1x**2
        if (iord .eq. 0) then
          c2q = 1._wp
        elseif (iord .eq. 1) then
          c2q = (0.5_wp + 6._wp*x**3 - 5._wp*x**2 - 3._wp/2._wp)*dd0 &
                + (1._wp + x**2 + 2._wp)*dd1 - (pi**2/3._wp + 9._wp/2._wp)
          c2q = CF*c2q
        end if
      elseif (ireg .eq. 1) then
        dd0 = 1._wp/x1
        dd1 = dl1x/x1
        if (iord .eq. 0) then
          c2q = 0._wp
        elseif (iord .eq. 1) then
          c2q = (0.5_wp + 6._wp*x**3 - 5._wp*x**2 - 3._wp/2._wp)*dd0 &
                + (1._wp + x**2 + 2._wp)*dd1 - (1._wp + x**2)/(1._wp - x)*dlx - &
                (1._wp + x)*dl1x - (1._wp + x**2)/x1*dlx + 3._wp + 2._wp*x
          c2q = CF*c2q
        end if
      elseif (ireg .eq. 2) then
        dd0 = -1._wp/x1
        dd1 = -dl1x/x1
        if (iord .eq. 0) then
          c2q = 0._wp
        elseif (iord .eq. 1) then
          c2q = (0.5_wp + 6._wp*x**3 - 5._wp*x**2 - 3._wp/2._wp)*dd0 &
                + (1._wp + x**2 + 2._wp)*dd1
          c2q = CF*c2q
        end if
      end if
    end if

  end function ResCFc2q

  function ResCFc2qg(iord, x, ireg, rmu2, fmu2, q2) result(c2qg)
    implicit none
    integer iord, ireg
    real*8 CF, q2, rmu2, fmu2
    real*8 x1, dlx, dl1x!,dlm,dlr
    real*8 dd0, dd1
    real*8 c2qg, x
    parameter(CF=1.333333333333333333_wp)

    x1 = 1._wp - x
    dlx = dlog(x)
    dl1x = dlog(x1)

    !dlm = dlog(q2/fmu2)
    !dlr = dlog(rmu2/fmu2)

    if (x .lt. 0.5_wp) then
      if (ireg .eq. 0) then
        dd0 = dl1x
        dd1 = 1._wp/2._wp*dl1x**2
        if (iord .eq. 0) then
          c2qg = 0._wp
        elseif (iord .eq. 1) then
          c2qg = 0._wp
        end if
      elseif (ireg .eq. 1) then
        dd0 = 1._wp/x1
        dd1 = dl1x/x1
        if (iord .eq. 0) then
          c2qg = 0._wp
        elseif (iord .eq. 1) then
          c2qg = 3._wp*x**3 + (2._wp*x - 5._wp*x**2)/(2._wp*x1) &
                 - (1._wp + x**2)/(1._wp - x)*dl1x
          c2qg = CF*c2qg
        end if
      elseif (ireg .eq. 2) then
        dd0 = -1._wp/x1
        dd1 = -dl1x/x1
        if (iord .eq. 0) then
          c2qg = 0._wp
        elseif (iord .eq. 1) then
          c2qg = 0._wp
        end if
      end if
    elseif (x .gt. 0.5_wp) then
      if (ireg .eq. 0) then
        dd0 = dl1x
        dd1 = 1._wp/2._wp*dl1x**2
        if (iord .eq. 0) then
          c2qg = 0._wp
        elseif (iord .eq. 1) then
          c2qg = 0._wp
        end if
      elseif (ireg .eq. 1) then
        dd0 = 1._wp/x1
        dd1 = dl1x/x1
        if (iord .eq. 0) then
          c2qg = 0._wp
        elseif (iord .eq. 1) then
          c2qg = 3._wp*x**3 + 2._wp + x - 6._wp*x**2 &
                 + (4._wp*x - 3._wp - x**2)/(2._wp*x1) - (1._wp + x**2)/x1*dlx
          c2qg = CF*c2qg
        end if
      elseif (ireg .eq. 2) then
        dd0 = -1._wp/x1
        dd1 = -dl1x/x1
        if (iord .eq. 0) then
          c2qg = 0._wp
        elseif (iord .eq. 1) then
          c2qg = 0._wp
        end if
      end if
    end if
  end function ResCFc2qg

  function ResCFc2g(iord, x, ireg, rmu2, fmu2, q2) result(c2g)
    implicit none
    integer iord, ireg
    real*8 CF, TR, q2, rmu2, fmu2
    real*8 x1, dlx, dl1x, dlm, dlr
    real*8 c2g, x
    parameter(CF=1.333333333333333333_wp)
    parameter(TR=0.5_wp)

    x1 = 1._wp - x
    dlx = dlog(x)
    dl1x = dlog(x1)
    dlm = dlog(q2/fmu2)
    dlr = dlog(rmu2/fmu2)

    if (x .lt. 0.5_wp) then
      if (ireg .eq. 0) then
        if (iord .eq. 0) then
          c2g = 1._wp
        elseif (iord .eq. 1) then
          c2g = 0._wp
        end if
      elseif (ireg .eq. 1) then
        if (iord .eq. 0) then
          c2g = 0._wp
        elseif (iord .eq. 1) then
          c2g = (x1**2 + x**2)*dlog(x1/x) - 8._wp*x**2 + 8._wp*x - 1._wp
          c2g = TR*c2g
        end if
      end if
    elseif (x .gt. 0.5_wp) then
      if (ireg .eq. 0) then
        if (iord .eq. 0) then
          c2g = 1._wp
        elseif (iord .eq. 1) then
          c2g = 0._wp
        end if
      elseif (ireg .eq. 1) then
        if (iord .eq. 0) then
          c2g = 0._wp
        elseif (iord .eq. 1) then
          c2g = (x**2 + (1._wp - x)**2)*(4._wp*x - 2._wp + 2._wp*dl1x - 2._wp*dlx) &
                + 8._wp*x*x1*(1._wp - 2._wp*x)
          c2g = c2g + 2._wp*((x1**2 + x**2)*dlog(x1/x) - 8._wp*x**2 + 8._wp*x - 1._wp)
          c2g = 0.5_wp*TR*c2g
        end if
      end if
    end if

  end function ResCFc2g

  function ResCFclq(iord, x, ireg, rmu2, fmu2, q2) result(clq)
    implicit none
    integer iord, ireg
    real*8 CF, q2, rmu2, fmu2
    real*8 x1, dlx, dl1x, dlm, dlr
    real*8 clq, x
    parameter(CF=1.333333333333333333_wp)

    if (x .lt. 0.5_wp) then
      clq = 0._wp
    else
      if (ireg .eq. 0) then
        if (iord .eq. 0) then
          clq = 1._wp
        elseif (iord .eq. 1) then
          clq = 0._wp
        end if
      elseif (ireg .eq. 1) then
        if (iord .eq. 0) then
          clq = 0._wp
        elseif (iord .eq. 1) then
          clq = CF*(2._wp*x - 4._wp*x**2)
        end if
      elseif (ireg .eq. 2) then
        if (iord .eq. 0) then
          clq = 0._wp
        elseif (iord .eq. 1) then
          clq = 0._wp
        end if
      end if
    end if
  end function ResCFclq

  function ResCFclg(iord, x, ireg, rmu2, fmu2, q2) result(clg)
    implicit none
    integer iord, ireg
    real*8 TR, q2, rmu2, fmu2
    real*8 dlm, dlr
    real*8 clg, x
    parameter(TR=0.5_wp)

    if (x .lt. 0.5_wp) then
      clg = 0._wp
    else
      if (ireg .eq. 0) then
        if (iord .eq. 0) then
          clg = 1._wp
        elseif (iord .eq. 1) then
          clg = 0._wp
        end if
      elseif (ireg .eq. 1) then
        if (iord .eq. 0) then
          clg = 0._wp
        elseif (iord .eq. 1) then
          clg = 0.5_wp*TR*(8._wp*x*(1._wp - x)*(1._wp - 2._wp*x))
        end if
      elseif (ireg .eq. 2) then
        if (iord .eq. 0) then
          clg = 0._wp
        elseif (iord .eq. 1) then
          clg = 0._wp
        end if
      end if
    end if

  end function ResCFclg

  function ResCFclqg(iord, x, ireg, rmu2, fmu2, q2) result(clqg)
    implicit none
    integer iord, ireg
    real*8 CF, q2, rmu2, fmu2
    real*8 dlm, dlr
    real*8 clqg, x
    parameter(CF=1.333333333333333333_wp)

    if (x .lt. 0.5_wp) then
      if (ireg .eq. 0) then
        if (iord .eq. 0) then
          clqg = 0._wp
        elseif (iord .eq. 1) then
          clqg = 0._wp
        end if
      elseif (ireg .eq. 1) then
        if (iord .eq. 0) then
          clqg = 0._wp
        elseif (iord .eq. 1) then
          clqg = CF*2._wp*x**3
        end if
      elseif (ireg .eq. 2) then
        if (iord .eq. 0) then
          clqg = 0._wp
        elseif (iord .eq. 1) then
          clqg = 0._wp
        end if
      end if
    elseif (x .gt. 0.5_wp) then
      if (ireg .eq. 0) then
        if (iord .eq. 0) then
          clqg = 0._wp
        elseif (iord .eq. 1) then
          clqg = 0._wp
        end if
      elseif (ireg .eq. 1) then
        if (iord .eq. 0) then
          clqg = 0._wp
        elseif (iord .eq. 1) then
          clqg = CF*2._wp*x*(1._wp - x)**2
        end if
      elseif (ireg .eq. 2) then
        if (iord .eq. 0) then
          clqg = 0._wp
        elseif (iord .eq. 1) then
          clqg = 0._wp
        end if
      end if
    end if

  end function ResCFclqg

! Expanded resummation coefficients

!real(kind=wp) function LogRintExpd(L)
!  implicit none
!  real*8 G10,G11,G12,G20,G21,G22,G23
!
!  G10 = c1q + c1g! how to best do this?
!  G11 = 3._wp/2._wp*CF
!  G12 = -CF
!  G20 = -1._wp/2._wp*(c1q**2+c1g**2) - c1g*c1g
!  G21 = -3._wp/2._wp*CF*c1g
!  G22 = -CA*c1g + CF*pi**2/2._wp - 151._wp/18._wp*CF + CF*c1q + 3._wp/2._wp*pi*beta0*CF - 1._wp/3._wp*pi**2*CF*CA - 1._wp/3._wp*pi**2*CF
!  G23 = -2._wp*pi*beta0*CF
!
!  LogRintExpd = (alphas/2._wp/pi)*(G10+G11*L+G12*L**2)+(alphas/2._wp/pi)**2*(G20+G21*L+G22*L**2+G23*L**3)
!end function LogRintExpd

end module RESDIS_mod
