module SFDIS_mod
  use Kinds_mod, only: wp
  implicit none
  private
  public :: SF

  real(kind=wp), parameter :: rnup = 2._wp
  real(kind=wp), parameter :: rndown = 3._wp
  real(kind=wp), parameter :: rnf = 5._wp
!> rnup*eu2+rndown*ed2
  real(kind=wp), parameter :: sumEQ2 = 1.2222222222222222_wp
  real(kind=wp), parameter :: gevfb = 389.37936e9_wp
contains

!> This is only the nnlo correction
!> NB this si only the approximate nnlo, which should agree to 1%
!> level with the real strucutre function
  real(kind=wp) function SF(iord, ix, xi)
    use Constants_mod, only: pi
    use Scales_mod
    use KinData_mod, only: xreg1_kin
    use CoeffFunc_av
    use Channels_mod, only: ip1_chan
    use DIS_mod, only: value_q2_dis, value_y_dis
    integer, intent(in) :: ix, iord
    integer idebug
    real(kind=wp), intent(in) :: xi
    integer nflav, i
    real(kind=wp) :: amz, as, c2qcox, c2qco1, c2gcox, c2qpscox, c2qdel
    !> bino function
    real(kind=wp) :: bino
    !> debuggig purposes
    real(kind=wp) :: c2q_new, c2g_new, c2qps_new
    real(kind=wp) :: cLq_new, cLg_new, cLqps_new
    real(kind=wp) :: clqcox, clqco1, clgcox, clqpscox, clqdel
    real(kind=wp) :: f2(nMax_scl), fl(nMax_scl), dsig(nMax_scl)
    real(kind=wp) :: partons(nMax_scl), facnorm(nMax_scl)
    real(kind=wp) :: xbj, q2, x1curr, x2curr, y
    real(kind=wp) :: rma2, rmb2, rm2, shat, Jxtoy
    complex*16 :: zewfac(4), zewnull(4)
    integer :: ipass
    common/parfrac/x1curr, x2curr
!$omp threadprivate(/parfrac/)
    common/eweakZ/amz, zewfac, zewnull
    !$omp threadprivate(/eweakZ/)
    common/BZFlav/nflav
    !$omp threadprivate(/BZFlav/)
    common/pmasses/rma2, rmb2, rm2(1:7), shat
!$omp threadprivate(/pmasses/)

    if (ix .ne. 1 .and. ix .ne. 3) then
      write (*, *) "SOMETHING IS WRONG WITH ix ", ix
      stop
    end if

    SF = 0._wp

    !ecuts -> return 0 on failure
    ipass = 1
    call ecuts_dis1j(0, 4, ipass)
    if (ipass .eq. 0) return

    call getqcdnorm(ix, partons, facnorm, 4, ipass, 0)
    if (ipass .eq. 0) return

    call eweakV1qq(nflav, 0._wp)

    f2 = 0._wp
    fl = 0._wp
    dsig = 0._wp

    q2 = value_q2_dis
    y = value_y_dis
    xbj = x1curr

    !>------------------------// DEBUG
    idebug = 0
    if (idebug > 0) then
    do i = 1, nAct_scl
      !do idebug=1,6
      print *, "debugging order = ", iord
      print *, ""
      print *, "ix = ", ix
      print *, ""
      print *, "xi   = ", xi
      print *, "xbj  = ", xbj
      print *, "xreg = ", xreg1_kin
      print *, ""
      print *, "muR2 = ", muR2_scl(i)
      print *, "muF2 = ", muF2_scl(i)
      print *, "Q2   = ", q2
      print *, ""
      !> the delta-piece (region 0 & 2)
      if (ix == 1) then
        select case (idebug)
        case (1) ! c2q
          print *, ""
          print *, ">>>   region = 0"
          print *, ">>>   c2q - Old:", c2q(iord, xbj, 0, muR2_scl(i), muF2_scl(i), q2)
          print *, ">>>   c2q - New:", c2q_new(iord, xbj, 0, muR2_scl(i), muF2_scl(i), q2, 0)
          print *, ">>>   c2q - AVp:", c2q_av(iord, xbj, 0, qexact=.false., flavchange=0)
          print *, ">>>   c2q - AVe:", c2q_av(iord, xbj, 0, qexact=.true., flavchange=0)
          print *, ""
          print *, ">>>   region = 2"
          print *, ">>>   c2q - Old:", c2q(iord, xreg1_kin, 2, muR2_scl(i), muF2_scl(i), q2)
          print *, ">>>   c2q - New:", c2q_new(iord, xreg1_kin, 2, muR2_scl(i), muF2_scl(i), q2, 0)
          print *, ">>>   c2q - AVp:", c2q_av(iord, xreg1_kin, 2, qexact=.false., flavchange=0)
          print *, ">>>   c2q - AVe:", c2q_av(iord, xreg1_kin, 2, qexact=.true., flavchange=0)
        case (2) ! c2qps
          print *, ""
          print *, ">>>   region = 0"
          print *, ">>>   c2qps - Old:", c2qps(iord, xbj, 0, muR2_scl(i), muF2_scl(i), q2)
          print *, ">>>   c2qps - New:", c2qps_new(iord, xbj, 0, muR2_scl(i), muF2_scl(i), q2, 0)
          print *, ">>>   c2qps - AVp:", c2qps_av(iord, xbj, 0, qexact=.false., flavchange=0)
          print *, ">>>   c2qps - AVe:", c2qps_av(iord, xbj, 0, qexact=.true., flavchange=0)
          print *, ""
          print *, ">>>   region = 2"
          print *, ">>>   c2qps - Old:", c2qps(iord, xreg1_kin, 2, muR2_scl(i), muF2_scl(i), q2)
          print *, ">>>   c2qps - New:", c2qps_new(iord, xreg1_kin, 2, muR2_scl(i), muF2_scl(i), q2, 0)
          print *, ">>>   c2qps - AVp:", c2qps_av(iord, xreg1_kin, 2, qexact=.false., flavchange=0)
          print *, ">>>   c2qps - AVe:", c2qps_av(iord, xreg1_kin, 2, qexact=.true., flavchange=0)
        case (3) ! c2g
          print *, ""
          print *, ">>>   region = 0"
          print *, ">>>   c2g - Old:", c2g(iord, xbj, 0, muR2_scl(i), muF2_scl(i), q2)
          print *, ">>>   c2g - New:", c2g_new(iord, xbj, 0, muR2_scl(i), muF2_scl(i), q2, 0)
          print *, ">>>   c2g - AVp:", c2g_av(iord, xbj, 0, qexact=.false., flavchange=0)
          print *, ">>>   c2g - AVe:", c2g_av(iord, xbj, 0, qexact=.true., flavchange=0)
          print *, ""
          print *, ">>>   region = 2"
          print *, ">>>   c2g - Old:", c2g(iord, xreg1_kin, 2, muR2_scl(i), muF2_scl(i), q2)
          print *, ">>>   c2g - New:", c2g_new(iord, xreg1_kin, 2, muR2_scl(i), muF2_scl(i), q2, 0)
          print *, ">>>   c2g - AVp:", c2g_av(iord, xreg1_kin, 2, qexact=.false., flavchange=0)
          print *, ">>>   c2g - AVe:", c2g_av(iord, xreg1_kin, 2, qexact=.true., flavchange=0)
        case (4) ! cLq
          print *, ""
          print *, ">>>   region = 0"
          print *, ">>>   cLq - Old:", cLq(iord, xbj, 0, muR2_scl(i), muF2_scl(i), q2)
          print *, ">>>   cLq - New:", cLq_new(iord, xbj, 0, muR2_scl(i), muF2_scl(i), q2, 0)
          print *, ">>>   cLq - AVp:", cLq_av(iord, xbj, 0, qexact=.false., flavchange=0)
          print *, ">>>   cLq - AVe:", cLq_av(iord, xbj, 0, qexact=.true., flavchange=0)
          print *, ""
          print *, ">>>   region = 2"
          print *, ">>>   cLq - Old:", cLq(iord, xreg1_kin, 2, muR2_scl(i), muF2_scl(i), q2)
          print *, ">>>   cLq - New:", cLq_new(iord, xreg1_kin, 2, muR2_scl(i), muF2_scl(i), q2, 0)
          print *, ">>>   cLq - AVp:", cLq_av(iord, xreg1_kin, 2, qexact=.false., flavchange=0)
          print *, ">>>   cLq - AVe:", cLq_av(iord, xreg1_kin, 2, qexact=.true., flavchange=0)
        case (5) ! cLqps
          print *, ""
          print *, ">>>   region = 0"
          print *, ">>>   cLqps - Old:", cLqps(iord, xbj, 0, muR2_scl(i), muF2_scl(i), q2)
          print *, ">>>   cLqps - New:", cLqps_new(iord, xbj, 0, muR2_scl(i), muF2_scl(i), q2, 0)
          print *, ">>>   cLqps - AVp:", cLqps_av(iord, xbj, 0, qexact=.false., flavchange=0)
          print *, ">>>   cLqps - AVe:", cLqps_av(iord, xbj, 0, qexact=.true., flavchange=0)
          print *, ""
          print *, ">>>   region = 2"
          print *, ">>>   cLqps - Old:", cLqps(iord, xreg1_kin, 2, muR2_scl(i), muF2_scl(i), q2)
          print *, ">>>   cLqps - New:", cLqps_new(iord, xreg1_kin, 2, muR2_scl(i), muF2_scl(i), q2, 0)
          print *, ">>>   cLqps - AVp:", cLqps_av(iord, xreg1_kin, 2, qexact=.false., flavchange=0)
          print *, ">>>   cLqps - AVe:", cLqps_av(iord, xreg1_kin, 2, qexact=.true., flavchange=0)
        case (6) ! cLg
          print *, ""
          print *, ">>>   region = 0"
          print *, ">>>   cLg - Old:", cLg(iord, xbj, 0, muR2_scl(i), muF2_scl(i), q2)
          print *, ">>>   cLg - New:", cLg_new(iord, xbj, 0, muR2_scl(i), muF2_scl(i), q2, 0)
          print *, ">>>   cLg - AVp:", cLg_av(iord, xbj, 0, qexact=.false., flavchange=0)
          print *, ">>>   cLg - AVe:", cLg_av(iord, xbj, 0, qexact=.true., flavchange=0)
          print *, ""
          print *, ">>>   region = 2"
          print *, ">>>   cLg - Old:", cLg(iord, xreg1_kin, 2, muR2_scl(i), muF2_scl(i), q2)
          print *, ">>>   cLg - New:", cLg_new(iord, xreg1_kin, 2, muR2_scl(i), muF2_scl(i), q2, 0)
          print *, ">>>   cLg - AVp:", cLg_av(iord, xreg1_kin, 2, qexact=.false., flavchange=0)
          print *, ">>>   cLg - AVe:", cLg_av(iord, xreg1_kin, 2, qexact=.true., flavchange=0)

        case default
          stop "muh"
        end select

        !> the convolution part (region 1)
      else
        call inithpls5(xi)
        select case (idebug)
        case (1) ! c2q
          print *, ""
          print *, ">>>   region = 1"
          print *, ">>>   c2q - Old:", c2q(iord, xi, 1, muR2_scl(i), muF2_scl(i), q2)
          print *, ">>>   c2q - New:", c2q_new(iord, xi, 1, muR2_scl(i), muF2_scl(i), q2, 0)
          print *, ">>>   c2q - AVp:", c2q_av(iord, xi, 1, qexact=.false., flavchange=0)
          print *, ">>>   c2q - AVe:", c2q_av(iord, xi, 1, qexact=.true., flavchange=0)
        case (2) ! c2qps
          print *, ""
          print *, ">>>   region = 1"
          print *, ">>>   c2qps - Old:", c2qps(iord, xi, 1, muR2_scl(i), muF2_scl(i), q2)
          print *, ">>>   c2qps - New:", c2qps_new(iord, xi, 1, muR2_scl(i), muF2_scl(i), q2, 0)
          print *, ">>>   c2qps - AVp:", c2qps_av(iord, xi, 1, qexact=.false., flavchange=0)
          print *, ">>>   c2qps - AVe:", c2qps_av(iord, xi, 1, qexact=.true., flavchange=0)
        case (3) ! c2g
          print *, ""
          print *, ">>>   region = 1"
          print *, ">>>   c2g - Old:", c2g(iord, xi, 1, muR2_scl(i), muF2_scl(i), q2)
          print *, ">>>   c2g - New:", c2g_new(iord, xi, 1, muR2_scl(i), muF2_scl(i), q2, 0)
          print *, ">>>   c2g - AVp:", c2g_av(iord, xi, 1, qexact=.false., flavchange=0)
          print *, ">>>   c2g - AVe:", c2g_av(iord, xi, 1, qexact=.true., flavchange=0)
        case (4) ! cLq
          print *, ""
          print *, ">>>   region = 1"
          print *, ">>>   cLq - Old:", cLq(iord, xi, 1, muR2_scl(i), muF2_scl(i), q2)
          print *, ">>>   cLq - New:", cLq_new(iord, xi, 1, muR2_scl(i), muF2_scl(i), q2, 0)
          print *, ">>>   cLq - AVp:", cLq_av(iord, xi, 1, qexact=.false., flavchange=0)
          print *, ">>>   cLq - AVe:", cLq_av(iord, xi, 1, qexact=.true., flavchange=0)
        case (5) ! cLqps
          print *, ""
          print *, ">>>   region = 1"
          print *, ">>>   cLqps - Old:", cLqps(iord, xi, 1, muR2_scl(i), muF2_scl(i), q2)
          print *, ">>>   cLqps - New:", cLqps_new(iord, xi, 1, muR2_scl(i), muF2_scl(i), q2, 0)
          print *, ">>>   cLqps - AVp:", cLqps_av(iord, xi, 1, qexact=.false., flavchange=0)
          print *, ">>>   cLqps - AVe:", cLqps_av(iord, xi, 1, qexact=.true., flavchange=0)
        case (6) ! cLg
          print *, ""
          print *, ">>>   region = 1"
          print *, ">>>   cLg - Old:", cLg(iord, xi, 1, muR2_scl(i), muF2_scl(i), q2)
          print *, ">>>   cLg - New:", cLg_new(iord, xi, 1, muR2_scl(i), muF2_scl(i), q2, 0)
          print *, ">>>   cLg - AVp:", cLg_av(iord, xi, 1, qexact=.false., flavchange=0)
          print *, ">>>   cLg - AVe:", cLg_av(iord, xi, 1, qexact=.true., flavchange=0)

        case default
          stop "muh"
        end select

      end if
      read (*, *)
      !end do
    end do
    end if
    !>------------------------// END DEBUG

    do i = 1, nAct_scl
      !> alpha s
      as = alphas_scl(i)

      !> This is the delta part
      if (ix .eq. 1) then
        if (ip1_chan .ne. 0) then
          c2qdel = c2q_new(iord, xbj, 0, muR2_scl(i), muF2_scl(i), q2, 0)
          c2qco1 = c2q_new(iord, xreg1_kin, 2, muR2_scl(i), muF2_scl(i), q2, 0)
          f2(i) = (as/4._wp/pi)**iord*abs(zewfac(1))**2*(c2qdel + c2qco1*(1._wp - xbj))
        end if
        !> This is the convolution
      elseif (ix .eq. 3) then
        call inithpls5(xi)
        if (ip1_chan .eq. 0) then
          !> factor of 2._wp because we are taking u and d separately
          !> would give a double counting for nf pieces
          c2gcox = c2g_new(iord, xi, 1, muR2_scl(i), muF2_scl(i), q2, 0)/2._wp
          f2(i) = (as/4._wp/pi)**iord*sumEQ2*c2gcox/xi
        else
          c2qcox = c2q_new(iord, xi, 1, muR2_scl(i), muF2_scl(i), q2, 0)
          c2qpscox = c2qps_new(iord, xi, 1, muR2_scl(i), muF2_scl(i), q2, 0)
          f2(i) = (as/4._wp/pi)**iord*(abs(zewfac(1))**2*c2qcox/xi &
               &  + sumEQ2*c2qpscox/xi)
        end if
        !> Jacobian factor for rreg1 pick
        !> + compensates for xi in ix.eq.3
        f2(i) = f2(i)*(1._wp - xbj)*xi
      end if

      f2(i) = f2(i)*xbj
      if (ix .eq. 1) then
        if (ip1_chan .ne. 0) then
          clqdel = clq_new(iord, xbj, 0, muR2_scl(i), muF2_scl(i), q2, 0)
          clqco1 = clq_new(iord, xreg1_kin, 2, muR2_scl(i), muF2_scl(i), q2, 0)
          fl(i) = (as/4._wp/pi)**iord*abs(zewfac(1))**2*(clqdel + clqco1*(1._wp - xbj))
        end if
        !> This is the convolution
      elseif (ix .eq. 3) then
        call inithpls5(xi)
        if (ip1_chan .eq. 0) then
          !> factor of 2._wp because we are taking u and d separately
          !> would give a double counting for nf pieces
          clgcox = clg_new(iord, xi, 1, muR2_scl(i), muF2_scl(i), q2, 0)/2._wp
          fl(i) = (as/4._wp/pi)**iord*sumEQ2*clgcox/xi
        else
          clqcox = clq_new(iord, xi, 1, muR2_scl(i), muF2_scl(i), q2, 0)
          clqpscox = clqps_new(iord, xi, 1, muR2_scl(i), muF2_scl(i), q2, 0)
          fl(i) = (as/4._wp/pi)**iord*(abs(zewfac(1))**2*clqcox/xi &
               &  + sumEQ2*clqpscox/xi)
        end if
        !> Jacobian factor for rreg1 pick
        !> + compensates for xi in ix.eq.3
        fl(i) = fl(i)*(1._wp - xbj)*xi
      end if

      fl(i) = fl(i)*xbj
    end do

    !> Jacobian factor to switch from y to x sampling
    Jxtoy = q2/shat/xbj
    dsig(:) = Jxtoy*2._wp*pi*amz**2._wp/q2**2._wp/y*((1._wp + (1._wp - y)**2._wp)*f2(:) - y**2._wp*fl(:))
    !> 8*pi is the phase space integal that would otherwise been performed twice
    dsig(:) = gevfb*dsig(:)*8._wp*pi*shat
    SF = bino(ix, partons, dsig*facnorm)
  end function SF

  function c2q(iord, x, ireg, rmu2, fmu2, q2)
    implicit none
    integer iord, ireg
    real*8 pi, zeta3, q2, rmu2, fmu2
    real*8 x1, dlx, dl1x, dl1px, dli2mx, dli21mx, dlm, dlr
    real*8 dd0, dd1, dd2, dd3
    real*8 c2q, x
    real*8 dli2
    parameter(pi=3.141592653589793238_wp)
    parameter(zeta3=1.20205690315959429_wp)

    x1 = 1._wp - x
    dlx = dlog(x)
    dl1x = dlog(x1)
    dl1px = dlog(1._wp + x)
    dli2mx = dli2(-x)
    dli21mx = dli2(1._wp - x)

    dlm = dlog(q2/fmu2)
    dlr = dlog(rmu2/fmu2)

    if (ireg .eq. 0) then
      dd0 = dl1x
      dd1 = 1._wp/2._wp*dl1x**2
      dd2 = 1._wp/3._wp*dl1x**3
      dd3 = 1._wp/4._wp*dl1x**4
      if (iord .eq. 0) then
        c2q = 1._wp
      elseif (iord .eq. 1) then
        c2q = -(9._wp + 2._wp/3._wp*pi**2) + 4._wp*dd1 - 3._wp*dd0
        c2q = c2q + dlm*(4._wp*dd0 + 3._wp)
        c2q = 4._wp/3._wp*c2q
      elseif (iord .eq. 2) then
        c2q = 128._wp/9._wp*dd3 - 184._wp/3._wp*dd2 - 31.1052_wp*dd1 &
        &           + 188.641_wp*dd0 - 338.513_wp
        c2q = c2q + rnf*(16._wp/9._wp*dd2 - 232._wp/27._wp*dd1 &
        &           + 6.34888_wp*dd0 + 46.8531_wp)
        c2q = c2q + dlm**2*(&
        &           (16._wp/9._wp)*(16._wp*dd1 + 12._wp*dd0 + 9._wp/2._wp - 4._wp*pi**2/3._wp) &
        &          + 4._wp*(-22._wp/3._wp*dd0 - 11._wp/2._wp) &
        &          + rnf*4._wp/3._wp*(4._wp/3._wp*dd0 + 1._wp))
        c2q = c2q + dlm*( &
        &       (16._wp/9._wp)*(24._wp*dd2 - 12._wp*dd1 - (16._wp/3._wp*pi**2 + 45._wp)*dd0 &
        &            + (40._wp*zeta3 - 2._wp*pi**2 - 51._wp/2._wp)) &
        &           + 4._wp*(-44._wp/3._wp*dd1 + (367._wp/9._wp - 4._wp/3._wp*pi**2)*dd0 &
        &                 + (-12._wp*zeta3 + 44._wp/9._wp*pi**2 + 215._wp/6._wp)) &
        &   + rnf*4._wp/3._wp*(8._wp/3._wp*dd1 - 58._wp/9._wp*dd0 - 8._wp/9._wp*pi**2 - 19._wp/3._wp))
        c2q = c2q + dlr*(11._wp - 2._wp/3._wp*rnf)*4._wp/3._wp*( &
        &       -(9._wp + 2._wp/3._wp*pi**2) + 4._wp*dd1 - 3._wp*dd0 + &
        &           dlm*(4._wp*dd0 + 3._wp))
        !> Call to Moch, Vermaseren, Vogt
!     elseif (iord.eq.3) then
!        c2q = X2NP3C(x,5)
      end if
      c2q = c2q
    elseif (ireg .eq. 1) then
      dd0 = 1._wp/x1
      dd1 = dl1x/x1
      dd2 = dl1x**2/x1
      dd3 = dl1x**3/x1
      if (iord .eq. 0) then
        c2q = 0._wp
      elseif (iord .eq. 1) then
        c2q = 4._wp*dd1 - 3._wp*dd0 - 2._wp*(1._wp + x)*(dl1x - dlx) &
        &            - 4._wp/x1*dlx + 6._wp + 4._wp*x + dlm*(4._wp*dd0 - 2._wp - 2._wp*x)
        c2q = c2q*4._wp/3._wp
      elseif (iord .eq. 2) then
        c2q = 128._wp/9._wp*dd3 - 184._wp/3._wp*dd2 - 31.1052_wp*dd1 &
        &            + 188.641_wp*dd0 - 17.74_wp*dl1x**3 + 72.24_wp*dl1x**2 &
        &            - 628.8_wp*dl1x - 181.0_wp - 806.7_wp*x + 0.719_wp*x*dlx**4 &
        &            + dlx*dl1x*(37.75_wp*dlx - 147.1_wp*dl1x) - 28.384_wp*dlx &
        &            - 20.70_wp*dlx**2 - 80._wp/27._wp*dlx**3
        c2q = c2q + rnf*(16._wp/9._wp*dd2 - 232._wp/27._wp*dd1  &
        &           + 6.34888_wp*dd0 - 1.5_wp*dl1x**2 + 24.87_wp*dl1x - 7.8109_wp &
        &           - 17.82_wp*x - 12.97_wp*x**2 - 0.185_wp*x*dlx**3 &
        &           + 8.113_wp*dlx*dl1x + 16._wp/3._wp*dlx + 20._wp/9._wp*dlx**2)
        c2q = c2q + dlm**2*( &
        &           (16._wp/9._wp)*(16._wp*dd1 + 12._wp*dd0 - 8._wp*(1._wp + x)*dl1x &
        &           - 4._wp*(1._wp + x**2)/x1*dlx + 2._wp*(1._wp + x)*dlx - 10._wp - 2._wp*x) &
        &          + 4._wp*(-22._wp/3._wp*dd0 + 11._wp/3._wp*(1._wp + x)) &
        &          + rnf*4._wp/3._wp*(4._wp/3._wp*dd0 - 2._wp/3._wp*(1._wp + x)))
        c2q = c2q + dlm*( &
        &       (16._wp/9._wp)*(24._wp*dd2 - 12._wp*dd1 - (16._wp/3._wp*pi**2 + 45._wp)*dd0 &
        &           + (1._wp + x**2)/x1*(4._wp*dlx**2 - 24._wp*dlx*dl1x - 6._wp*dlx) &
        &           + (1._wp + x**2)/(1._wp + x1)*(4._wp*dlx**2 - 16._wp*dli2mx &
        &                   - 16._wp*dlx*dl1px - 4._wp/3._wp*pi**2) &
        &           + (1._wp + x)*(4._wp*dli21mx + 4._wp*dlx*dl1x - 12._wp*dl1x**2 &
        &                  - 4._wp*dlx**2 + 8._wp/3._wp*pi**2) + 8._wp*(2._wp + 3._wp*x)*dl1x &
        &           - 2._wp*(3._wp + 11._wp*x)*dlx + 2._wp*(19._wp + 14._wp*x)) &
        &       + 4._wp*(-44._wp/3._wp*dd1 + (367._wp/9._wp - 4._wp/3._wp*pi**2)*dd0 &
        &           + (1._wp + x**2)/x1*(44._wp/3._wp*dlx + 2._wp*dlx**2) &
        &           + (1._wp + x**2)/(1._wp + x)*(8._wp*dli2mx - 2._wp*dlx**2 &
        &                   + 8._wp*dlx*dl1px + 2._wp/3._wp*pi**2) &
        &           + (1._wp + x)*(22._wp/3._wp*dl1x + 2._wp/3._wp*pi**2) &
        &           - 1._wp/9._wp*(164._wp + 434._wp*x)) &
        &       + rnf*4._wp/3._wp*(8._wp/3._wp*dd1 - 58._wp/9._wp*dd0 &
        &            - 8._wp/3._wp*(1._wp + x**2)/x1*dlx - 4._wp/3._wp*(1._wp + x)*dl1x &
        &            + 1._wp/9._wp*(32._wp + 68._wp*x)))
        c2q = c2q + dlr*(11._wp - 2._wp/3._wp*rnf)*4._wp/3._wp*( &
        &        +4._wp*dd1 - 3._wp*dd0 - 2._wp*(1._wp + x)*dl1x  &
        &           - 2._wp*(1._wp + x**2)/x1*dlx + 6._wp + 4._wp*x &
        &           + dlm*(4._wp*dd0 - 2._wp*(1._wp + x)))
!     elseif (irod.eq.3) then
!        c2q = X2NP3A(x,5)
      end if
    elseif (ireg .eq. 2) then
      dd0 = -1._wp/x1
      dd1 = -dl1x/x1
      dd2 = -dl1x**2/x1
      dd3 = -dl1x**3/x1

      if (iord .eq. 0) then
        c2q = 0._wp
      elseif (iord .eq. 1) then
        c2q = +4._wp*dd1 - 3._wp*dd0
        c2q = c2q + dlm*4._wp*dd0
        c2q = 4._wp/3._wp*c2q
      elseif (iord .eq. 2) then
        c2q = 128._wp/9._wp*dd3 - 184._wp/3._wp*dd2 - 31.1052_wp*dd1 &
        &           + 188.641_wp*dd0
        c2q = c2q + rnf*(16._wp/9._wp*dd2 - 232._wp/27._wp*dd1 &
        &           + 6.34888_wp*dd0)
        c2q = c2q + dlm**2*( &
        &           (16._wp/9._wp)*(16._wp*dd1 + 12._wp*dd0) &
        &          + 4._wp*(-22._wp/3._wp*dd0) &
        &          + rnf*4._wp/3._wp*(4._wp/3._wp*dd0))
        c2q = c2q + dlm*( &
        &       (16._wp/9._wp)*(24._wp*dd2 - 12._wp*dd1 - (16._wp/3._wp*pi**2 + 45._wp)*dd0) &
        &           + 4._wp*(-44._wp/3._wp*dd1 + (367._wp/9._wp - 4._wp/3._wp*pi**2)*dd0) &
        &   + rnf*4._wp/3._wp*(8._wp/3._wp*dd1 - 58._wp/9._wp*dd0))
        c2q = c2q + dlr*(11._wp - 2._wp/3._wp*rnf)*4._wp/3._wp*( &
        &          +4._wp*dd1 - 3._wp*dd0 + dlm*(4._wp*dd0))
!     elseif (iorder.eq.3) then
!        c2q = X2NS3B(x,5)
      end if
    end if
    return
  end function c2q

  function c2g(iord, x, ireg, rmu2, fmu2, q2)
    implicit none
    integer iord, ireg
    real*8 pi, zeta3, q2, rmu2, fmu2
    real*8 x1, dlx, dl1x, dl1px, dli2mx, dli21mx, dlm, dlr
    real*8 dd0, dd1, dd2, dd3
    real*8 c2g, x
    real*8 dli2
    parameter(pi=3.141592653589793238_wp)
    parameter(zeta3=1.20205690315959429_wp)

    x1 = 1._wp - x
    dlx = dlog(x)
    dl1x = dlog(x1)
    dl1px = dlog(1._wp + x)
    dli2mx = dli2(-x)
    dli21mx = dli2(1._wp - x)

    dlm = dlog(q2/fmu2)
    dlr = dlog(rmu2/fmu2)

    if (ireg .eq. 0) then
      c2g = 0._wp
    elseif (ireg .eq. 1) then
      if (iord .eq. 0) then
        c2g = 0._wp
      elseif (iord .eq. 1) then
        c2g = (2._wp - 4._wp*x*x1)*(dl1x - dlx) - 2._wp + 16._wp*x*x1
        c2g = c2g + dlm*(2._wp - 4._wp*x*x1)
      elseif (iord .eq. 2) then
        c2g = 58._wp/9._wp*dl1x**3 - 24._wp*dl1x**2 - 34.88_wp*dl1x &
        &          + 30.586_wp - (25.08_wp + 760.3_wp*x + 29.65_wp*dl1x**3)*x1 &
        &          + 1204._wp*x*dlx**2 + dlx*dl1x*(293.8_wp + 711.2_wp*x &
        &          + 1043._wp*dlx) + 115.6_wp*dlx - 7.109_wp*dlx**2 &
        &          + 70._wp/9._wp*dlx**3 + 11.9033_wp*x1/x
        c2g = c2g + dlm**2*( &
        &        2._wp/3._wp*(-4._wp*(1._wp + 4._wp*x**2 - 2._wp*x)*dlx &
        &            + 8._wp*(1._wp + 2._wp*x**2 - 2._wp*x)*dl1x - 2._wp + 8._wp*x) &
        &       + 3._wp/2._wp*(8._wp*(1._wp + 4._wp*x)*dlx &
        &            + 8._wp*(1._wp + 2._wp*x**2 - 2._wp*x)*dl1x &
        &            + 4._wp/3._wp*(3._wp - 31._wp*x**2 + 24._wp*x + 4._wp/x)))
        c2g = c2g + dlm*( &
        &        2._wp/3._wp*(16._wp*(1._wp + 2._wp*x**2 - 2._wp*x)*(dl1x**2 - pi**2/3._wp) &
        &            + 8._wp*(1._wp + 4._wp*x**2 - 2._wp*x)*dlx**2 &
        &            - 8._wp*(3._wp + 8._wp*x**2 - 6._wp*x)*dlx*dl1x &
        &            + 8._wp*(1._wp + 10._wp*x**2 - 6._wp*x)*dlx &
        &            + 8._wp*(1._wp - 2._wp*x)*dli21mx &
        &            - 4._wp*(7._wp + 20._wp*x**2 - 24._wp*x)*dl1x &
        &            + 4._wp*(9._wp + 4._wp*x**2 - 17._wp*x)) &
        &         + 3._wp/2._wp*(32._wp*x*(3._wp - x)*dlx*dl1x &
        &            - 16._wp*(1._wp + 2._wp*x**2 + 2._wp*x)*(dlx*dl1px + dli2mx) &
        &            - 16._wp*(1._wp + 3._wp*x)*dlx**2 + 16._wp*(1._wp + 4._wp*x)*dli21mx &
        &            - 16._wp*(1._wp + 2._wp*x**2)*pi**2/6._wp + 8._wp*x*(25._wp*x - 24._wp)*dlx &
        &            + 8._wp*(1._wp + 2._wp*x**2 - 2._wp*x)*dl1x**2 &
        &            - 8._wp/3._wp*(3._wp + 67._wp*x**2 - 60._wp*x - 4._wp/x)*dl1x &
        &            - 4._wp/9._wp*(165._wp - 407._wp*x**2 + 276._wp*x - 52._wp/x)))
        c2g = c2g + dlr*(11._wp - 2._wp/3._wp*rnf)*( &
        &              (2._wp - 4._wp*x*x1)*(dl1x - dlx) - 2._wp + 16._wp*x*x1 &
        &                 + dlm*(2._wp - 4._wp*x*x1))
      end if
    elseif (ireg .eq. 2) then
      c2g = 0._wp
    end if
    return
  end function c2g

  function c2qps(iord, x, ireg, rmu2, fmu2, q2)
    implicit none
    integer iord, ireg
    real*8 pi, zeta3, q2, rmu2, fmu2
    real*8 x1, dlx, dl1x, dl1px, dli2mx, dli21mx, dlm, dlr
    real*8 dd0, dd1, dd2, dd3
    real*8 c2qps, x
    real*8 dli2
    parameter(pi=3.141592653589793238_wp)
    parameter(zeta3=1.20205690315959429_wp)

    !> lines below are for testing nf independent pieces

    x1 = 1._wp - x
    dlx = dlog(x)
    dl1x = dlog(x1)
    dl1px = dlog(1._wp + x)
    dli2mx = dli2(-x)
    dli21mx = dli2(1._wp - x)

    dlm = dlog(q2/fmu2)
    dlr = dlog(rmu2/fmu2)

    if (ireg .eq. 0) then
      c2qps = 0._wp
    elseif (ireg .eq. 1) then
      if (iord .eq. 0) then
        c2qps = 0._wp
      elseif (iord .eq. 1) then
        c2qps = 0._wp
      elseif (iord .eq. 2) then
        c2qps = (8._wp/3._wp*dl1x**2 - 32._wp/3._wp*dl1x + 9.8937_wp)*x1 &
        &              + (9.57_wp - 13.41_wp*x + 0.08_wp*dl1x**3)*x1**2 &
        &              + 5.667_wp*x*dlx**3 - dlx**2*dl1x*(20.26_wp - 33.93_wp*x) &
        &              + 43.36_wp*x1*dlx - 1.053_wp*dlx**2 + 40._wp/9._wp*dlx**3 &
        &              + 5.2903_wp*x1**2/x
        c2qps = c2qps + dlm**2*2._wp/3._wp*( &
        &         8._wp*(1._wp + x)*dlx + 4._wp/3._wp*(3._wp - 4._wp*x**2 - 3._wp*x + 4._wp/x)) &
        &                    + dlm*2._wp/3._wp*( &
        &         16._wp*(1._wp + x)*(dli21mx + dlx*dl1x - dlx**2) + 32._wp*x**2*dlx &
     &         + 8._wp/3._wp*(3._wp - 4._wp*x**2 - 3._wp*x + 4._wp/x)*dl1x &
     &         - 16._wp/9._wp*(39._wp + 4._wp*x**2 - 30._wp*x - 13._wp/x))
      end if
    end if
    return
  end function c2qps

  function clq(iord, x, ireg, rmu2, fmu2, q2)
    implicit none
    integer iord, ireg
    real*8 pi, zeta3, q2, rmu2, fmu2
    real*8 x1, dlx, dl1x, dl1px, dli2mx, dli21mx, dlm, dlr
    real*8 dd0, dd1, dd2, dd3
    real*8 clq, x
    real*8 dli2
    parameter(pi=3.141592653589793238_wp)
    parameter(zeta3=1.20205690315959429_wp)

    x1 = 1._wp - x
    dlx = dlog(x)
    dl1x = dlog(x1)
    dl1px = dlog(1._wp + x)

    dlm = dlog(q2/fmu2)
    dlr = dlog(rmu2/fmu2)

    if (ireg .eq. 0) then
      clq = 0._wp
      if (iord .eq. 2) clq = clq - 0.012_wp/x1
    elseif (ireg .eq. 1) then
      if (iord .eq. 0) then
        clq = 0._wp
      elseif (iord .eq. 1) then
        clq = 4._wp*x
        clq = clq*4._wp/3._wp
      elseif (iord .eq. 2) then
        clq = 128._wp/9._wp*x*dl1x**2 - 46.50_wp*x*dl1x - 84.094_wp*dlx*dl1x &
        &            + 33.82_wp*x**2 + x*dlx*(32.90_wp + 18.41_wp*dlx) &
        &            - 128.d0/9._wp*dlx &
        &           + 16._wp/27._wp*rnf*(6._wp*x*dl1x - 12._wp*x*dlx - 25._wp*x + 6._wp)
        clq = clq + dlm*( &
        &       (16._wp/9._wp)*(8._wp*x*(2._wp*dl1x - dlx) + 4._wp*(2._wp + x)) &
        &      - 4._wp*(44._wp/3._wp)*x + 4._wp/3._wp*rnf*8._wp/3._wp*x)
        clq = clq + dlr*(11._wp - 2._wp/3._wp*rnf)*4._wp/3._wp*4._wp*x
      end if
    elseif (ireg .eq. 2) then
      clq = 0._wp
    end if
    return
  end function clq

  function clg(iord, x, ireg, rmu2, fmu2, q2)
    implicit none
    integer iord, ireg
    real*8 pi, zeta3, xbj, q2, rmu2, fmu2
    real*8 x1, dlx, dl1x, dl1px, dli2mx, dli21mx, dlm, dlr
    real*8 dd0, dd1, dd2, dd3
    real*8 clg, x
    real*8 dli2
    parameter(pi=3.141592653589793238_wp)
    parameter(zeta3=1.20205690315959429_wp)

    x1 = 1._wp - x
    dlx = dlog(x)
    dl1x = dlog(x1)
    dl1px = dlog(1._wp + x)
    dli2mx = dli2(-x)
    dli21mx = dli2(1._wp - x)

    dlm = dlog(q2/fmu2)
    dlr = dlog(rmu2/fmu2)

    if (ireg .eq. 0) then
      clg = 0._wp
    elseif (ireg .eq. 1) then
      if (iord .eq. 0) then
        clg = 0._wp
      elseif (iord .eq. 1) then
        clg = 8._wp*x*x1
      elseif (iord .eq. 2) then
        clg = (94.74_wp - 49.20_wp*x)*x1*dl1x**2 + 864.8_wp*x1*dl1x &
        &            + 1161._wp*x*dl1x*dlx + 60.06_wp*x*dlx**2 + 39.66_wp*x1*dlx &
        &            - 5.333_wp*(1._wp/x - 1._wp)
        clg = clg + dlm*( &
        &        2._wp/3._wp*(32._wp*x*dlx + 16._wp*(1._wp - 2._wp*x**2 + x)) &
        &       + 3._wp/2._wp*(64._wp*x*x1*dl1x - 128._wp*x*dlx &
        &           - 32._wp/3._wp*(3._wp - 17._wp*x**2 + 15._wp*x - 1._wp/x)))
        clg = clg + dlr*(11._wp - 2._wp/3._wp*rnf)*8._wp*x*x1
      end if
    elseif (ireg .eq. 2) then
      clg = 0._wp
    end if
    return
  end function clg

  function clqps(iord, x, ireg, rmu2, fmu2, q2)
    implicit none
    integer iord, ireg
    real*8 pi, zeta3, q2, rmu2, fmu2
    real*8 x1, dlx, dl1x, dl1px, dli2mx, dli21mx, dlm, dlr
    real*8 dd0, dd1, dd2, dd3
    real*8 clqps, x
    real*8 dli2
    parameter(pi=3.141592653589793238_wp)
    parameter(zeta3=1.20205690315959429_wp)

    x1 = 1._wp - x
    dlx = dlog(x)
    dl1x = dlog(x1)
    dl1px = dlog(1._wp + x)
    dli2mx = dli2(-x)
    dli21mx = dli2(1._wp - x)

    dlm = dlog(q2/fmu2)
    dlr = dlog(rmu2/fmu2)

    if (ireg .eq. 0) then
      clqps = 0._wp
    elseif (ireg .eq. 1) then
      if (iord .eq. 0) then
        clqps = 0._wp
      elseif (iord .eq. 1) then
        clqps = 0._wp
      elseif (iord .eq. 2) then
        clqps = (15.94_wp - 5.212_wp*x)*x1**2*dl1x &
        &          + (0.421_wp + 1.520*x)*dlx**2 + 28.09_wp*x1*dlx &
        &          - (2.370_wp/x - 19.27_wp)*x1**3
        clqps = clqps + dlm*2._wp/3._wp*( &
        &         -32._wp*x*dlx - 32._wp/3._wp*(3._wp - 2._wp*x**2 - 1._wp/x))
      end if
    end if
    return
  end function clqps

end module SFDIS_mod
