module Spikes_mod
   use KinData_mod, only: kin
   implicit none

   private
   public :: collectSpikes, flushSpikes, initialiseSpike

   integer,          parameter :: nbins      = 40
   double precision, parameter :: tolerance  = 1d-2
   double precision, parameter :: oneplus    = 1d0+tolerance
   double precision, parameter :: oneminus   = 1d0-tolerance
   double precision, parameter :: oneplus2s  = 1d0+tolerance*2d0
   double precision, parameter :: oneminus2s = 1d0-tolerance*2d0
   double precision, parameter :: inc        = tolerance/nbins*4d0

   type spike_plot
      integer :: passed   = 0
      integer :: passed2s = 0
      integer :: failed   = 0
      integer :: totale   = 0
      integer :: i, j
      character(3) :: sij           
      ! Store sij/shat and ratio
      double precision, dimension(:), allocatable :: ratio, x0
      ! Store spike plot
      integer, dimension(nbins) :: npoints = 0
   end type spike_plot

   type(spike_plot), save, dimension(:,:), allocatable :: spikeNLO
   double precision, dimension(nbins) :: xpoints

contains
   ! Initialise
   subroutine initialiseSpike(i,j)
      use Run_mod
      integer, intent(in) :: i, j
      character(3) :: sij
      integer :: itmax, nevents

      call resetCurrentSweep_run()
      if (.not.loadNextSweep_run()) then
         print*, "initialiseSpike: failed loading the first sweep"
         stop
      end if
      call currentSweep_run(ncall=nevents, nit=itmax)


      allocate(spikeNLO(1,1))
      allocate(spikeNLO(1,1)%ratio(itmax*2*nevents))
      allocate(spikeNLO(1,1)%x0(itmax*2*nevents))

      spikeNLO(1,1)%i = i
      spikeNLO(1,1)%j = j

      write(sij, '(A,I0,I0)')"s",i,j
      spikeNLO(1,1)%sij = sij

      xpoints = (/((oneminus2s+inc*i), i = 1, nbins)/)
      
   end subroutine initialiseSpike

   ! Print to stdout
   subroutine flushSpikes
      double precision :: maxx, maxn
      integer :: maxn10, ii
      integer :: i, j
      character(:), allocatable :: splot, slegacy
      i = 1
      j = 1

      splot   = "spike_plot_" // trim(spikeNLO(i,j)%sij) // ".agr"
      slegacy = "spike_legacy_" // trim(spikeNLO(i,j)%sij) // ".agr"

      write(*,*)"----------RESULTS------------------------"
      write(*,'(A,I0)')"Passed: ", spikeNLO(i, j)%passed
      write(*,'(A,I0)')"Passed close: ", spikeNLO(i, j)%passed2s
      write(*,'(A,I0)')"Failed: ", spikeNLO(i, j)%failed
      write(*,*)"-----------------------------------------"

      maxx =maxval( spikeNLO(i,j)%x0(:) )
!        minx =minval( spikeNLO(1,1)%x0(:) )
      open (unit=20, file=splot, action="write", status="replace")
      write(20,*)"@version 50123"
      write(20,*)"@with g0"
      write(20,'(A,F9.7,A,F9.7,A,F9.7)')"@  world 0,",oneminus,",",maxx,",",oneplus
      write(20,'(A,F9.7)')"@  xaxis tick major ", maxx/5d0
      write(20,*)'@  xaxis label "y0"'
      write(20,'(A,F9.7)')"@  yaxis tick major ", tolerance/5d0
      write(20,*)'@  yaxis label "ratio"'
      write(20,*)"@  s0 symbol 9"
      write(20,*)"@  s0 symbol size 0.24"
      write(20,*)"@  s0 line type 0"
      do ii = 1, spikeNLO(i,j)%totale
         write(20, *) spikeNLO(i,j)%x0(ii), spikeNLO(i,j)%ratio(ii)
      enddo
      close(20)

      maxn   = real( maxval( spikeNLO(i,j)%npoints(:) ) )
      maxn10 = 10**floor(log10( maxn ) ) 
      maxn10 = maxn10 * ceiling(maxn/maxn10)
      open (unit=30, file=slegacy, action="write", status="replace")
      write(30,*)"@version 50123"
      write(30,*)"@with g0"
      write(30,'(A,F6.4,A,F6.4,A,I0)')"@  world ",xpoints(1) - inc,",0,",xpoints(nbins )+inc,",",maxn10
      write(30,'(A,F9.7)')"@  xaxis tick major ", tolerance/2d0
      write(30,'(A,I0)')"@  yaxis tick major ", maxn10/5
      write(30,*)'@  xaxis label "ratio"'
      write(30,*)'@  yaxis label "N"'
      write(30,'(A,I0,A)')'@  title "Points in: ', spikeNLO(i,j)%passed + spikeNLO(i,j)%passed2s, '"'
      write(30,'(A,I0,A)')'@  subtitle "Points out: ', spikeNLO(i,j)%failed, '"'
      do ii = 1, nbins
         write(30, *) xpoints(ii), spikeNLO(i,1)%npoints(ii)
      enddo
      close(30)
      print *, "'Spike plot' wrote to xmgrace files: ",splot, " and ",slegacy
   end subroutine flushSpikes

   ! Check the ratio dsigma_real / dsigma_subtraction and update passed count
   subroutine collectSpikes(npar, sumR, sumS)
      implicit none
      double precision, intent(in) :: sumR, sumS
      integer, intent(in)          :: npar
      double precision             :: ratio, shat, sijmin
      integer                      :: i, j, n, targeti, targetj ! Todo: OR blocks, several spikeplots?
      logical                      :: spike
      ratio    = sumR/sumS
      if((ratio.ne.ratio).or.(sumS.eq.0d0)) then
         ratio = 0d0
      endif
      i = 1 ; j = 1
      spikeNLO(i,j)%totale = spikeNLO(i,j)%totale + 1
      if ((ratio.lt.oneplus).and.(ratio.gt.oneminus)) then
         spikeNLO(i, j)%passed = spikeNLO(i, j)%passed + 1
         spike = .true.
      elseif ((ratio.lt.oneplus2s).and.(ratio.gt.oneminus2s)) then
         spikeNLO(i, j)%passed2s = spikeNLO(i, j)%passed2s + 1
         spike = .true.
      else
         spikeNLO(i, j)%failed = spikeNLO(i, j)%failed + 1
         spike = .false.
      endif
      if (spike) then
         do n = 1, nbins
            if (ratio.lt.xpoints(n)) then
               spikeNLO(i,j)%npoints(n) = spikeNLO(i,j)%npoints(n) + 1
               exit
            endif
         enddo
      endif
      shat   = kin(npar)%s(1,2)
      targeti = spikeNLO(i,j)%i
      targetj = spikeNLO(i,j)%j
      sijmin = dabs(kin(npar)%s(targeti,targetj))
      spikeNLO(i,j)%x0(spikeNLO(i,j)%totale)    = sijmin/shat
      spikeNLO(i,j)%ratio(spikeNLO(i,j)%totale) = ratio
   end subroutine collectSpikes

end module Spikes_mod
