      !-----------------------------------------------------------------------------
      !> @brief
      !> wrapper to compute weights and fill histograms
      !
      !> @param[in] ix
      !> @param[in] partons parton lumi    & to be binned (*kinwt)
      !> @param[in] kinwt kinematic weight & to be binned (*partons)
      !-----------------------------------------------------------------------------
      double precision function bino(ix, partons, kinwt)
      use Constants_mod
      use Scales_mod
      use ScaleCoeff_mod
      use Reweight_mod
      use Histograms_mod
      use Channels_mod
      use KinData_mod, only : x1_kin, x2_kin, xreg1_kin, xreg2_kin,
     &  zfrag_kin, zregfrag_kin, z2frag_kin, zreg2frag_kin
      use Observables_mod, only : loadFlav_obs
      use Process_mod, only : nPDFs_proc
      use ProcLib_mod, only : getlumi_plib
      use Flavour_mod
      use UnitPhase_mod, only: phase_unit
      use PhotonFragmentation_mod
      use HadronFragmentation_mod
      implicit none
      integer, intent(in) :: ix
      double precision, intent(in) :: partons(nMax_scl), kinwt(nMax_scl)
      double precision :: partons_sub(nMax_scl)

      !> for flavour tagging
      integer :: ip1_flav, ip2_flav

      !> subfac
      double precision :: relfac, subfac

      !> accumulate jacobian factors
      double precision :: jfac

      !> weights for the histograms
      double precision rnormcurr,wgtcurr
      common /histonorm/rnormcurr,wgtcurr
!$omp threadprivate(/histonorm/)

      integer imethod
      common /method/imethod

      logical jflag
      common /jacobian/jflag
!$omp threadprivate(/jacobian/)

      logical virtsflag
      real*8 xreg1,xreg2
      common /xregions/xreg1,xreg2,virtsflag
!$omp threadprivate(/xregions/)

      !> for the scale coefficient recosntruction
      double precision, dimension(nMax_scl) :: as2pi_inp, partons_inp
      integer :: as2pi_power

      bino = 0d0

      ix_chan = ix  ! store it for APPLgrid/fastNLO

      relfac = subfac(0d0)

      !> determine the jacobian factor
      jfac = 1d0
      jfac = jfac * jac_multi_chan()
      if (imethod==2) then
        select case (ix)
        case(2)
          jfac = jfac / xreg2_kin
        case(3)
          jfac = jfac / xreg1_kin
        case(4)
          jfac = jfac / (xreg1_kin*xreg2_kin)
        end select
        if (jflag) then
           if (nPDFs_proc>=1) jfac = jfac / (1d0-x1_kin)
           if (nPDFs_proc>=2) jfac = jfac / (1d0-x2_kin)
        end if
      end if
      !> check if we're in a fragmentation-convolution block!
      if ( isSet_frag(1) ) then
        !> Jacobian for zreg mapping: [0,1] --> [zfrag,1]
        if (.not. phase_unit) jfac = jfac * (1d0-zfrag_kin)
        if (imethod==2) then
          select case (iz_frag)
          case(2)
            jfac = jfac / zregfrag_kin
          end select
        end if
      end if
      if ( isSet_frag(2) ) then
        !> Jacobian for zreg2 mapping: [0,1] --> [z2frag,1]
        if (.not. phase_unit) jfac = jfac * (1d0-z2frag_kin)
        if (imethod==2) then
          select case (iz2_frag)
          case(2)
            jfac = jfac / zreg2frag_kin
          end select
        end if
      end if
      if ( isSet_hfrag() ) then
        !> Jacobian for zreg mapping: [0,1] --> [zfrag,1]
        !> note it is counter to the IS case since
        !> we do not multiply the jacobian by default so
        !> we nedd to multiply it when jflag is not set
        if (.not.jflag .and. virtsflag) jfac = jfac * (1d0-zfrag_kin)
        if (imethod==2) then
          select case (iz_hfrag)
          case(2)
            jfac = jfac / zregfrag_kin
          end select
        end if
      end if

      !>-----
      if (isActive_sclc()) then
      !>----- scale-coefficients

        call get_alphas_scl(as2pi_inp, fac=1d0)
        as2pi_inp = as2pi_inp / twopi
        call getlumi_plib(ix, current_chan, partons_inp, fac=1d0)
        call currentContrib_chan(nas=as2pi_power)

        if (.not.isActive_flav()) then
          !> for the central scale, no scale-reconstruction with logs needed!
          bino = kinwt(1) * jfac * partons_inp(1) * as2pi_inp(1)**as2pi_power
          call writeBuffer_hist( kinwt*rnormcurr*wgtcurr*relfac*jfac,
     &                           ip1_chan, ip2_chan,
     &                           partons=partons_inp, as2pi=as2pi_inp )
        else
           print*, "bino: scale-coeff & flavtag most likely broken:",
     &          "check partons_sub vs. partons_inp"
          stop
          bino = 0d0
          do while (.not.isEmpty_flav())
            call loadFlav_obs()
            call pop_flav(ip1_flav, ip2_flav, partons_sub)
            call writeBuffer_hist( kinwt*rnormcurr*wgtcurr*relfac*jfac,
     &                             ip1_flav, ip2_flav,
     &                             partons=partons_sub, as2pi=as2pi_inp )
            bino = bino + partons_sub(1)
          end do
          bino = bino * kinwt(1) * jfac * as2pi_inp(1)**as2pi_power
        end if


      !>-----
      else
      !>----- no scale-coefficients

        if ( .not.(isActive_flav().and.isActive_hist()) ) then
          bino = kinwt(1) * jfac * partons(1)  ! eventually: * relfac ?
          call writeBuffer_hist( partons*kinwt*rnormcurr*wgtcurr*relfac*jfac,
     &                           ip1_chan, ip2_chan )
        else
          bino = 0d0
          do while (.not.isEmpty_flav()) ! always false when isActive_hist()
            call loadFlav_obs()
            call pop_flav(ip1_flav, ip2_flav, partons_sub)
            call writeBuffer_hist( partons_sub*kinwt*rnormcurr*wgtcurr*relfac*jfac,
     &                             ip1_flav, ip2_flav )
            bino = bino + partons_sub(1)
          end do
          bino = bino * kinwt(1) * jfac
        end if


      end if
      !>-----



      if (isActive_rewgt()) bino = bino * get_rewgt()

      call writeBuffer_multi_chan(bino*rnormcurr*wgtcurr*relfac)

      end function bino




      !> iflav: flavour of the parton that fragments to a photon
      !> icorr: 0 if gluon or final-state, [1|2] if initial-state
      !-----------------------------------------------------------------------------
      !> @brief
      !> dress
      !
      !> @param[in] z momentum fraction of the photon
      !> @param[in] muF2 [GeV^2] fragmentation scale
      !> @return weight
      !-----------------------------------------------------------------------------
      function bino_Dfrag(ix, iflav, icorr, npar, kinwt, flav_block) result(wtsum)
      use Channels_mod, only : ip1_chan, ip2_chan
      use PhotonFragmentation_mod
      use Scales_mod
      use KinData_mod, only : zfrag_kin, zregfrag_kin
      implicit none
      double precision :: z_val
      double precision :: wtsum
      double precision :: bino_Dfrag2
      integer, intent(in) :: ix
      integer, intent(in) :: iflav, icorr
      integer, intent(in) :: npar
      double precision, intent(in) :: kinwt(nMax_scl)
      integer, intent(in) :: flav_block
      integer :: ip1_sav, ip2_sav, ip_unroll, jp_unroll
      integer :: n_unroll
      !integer, pointer :: ip_unroll
      integer :: i, i_unroll, ipass_block
      double precision :: partons(nMax_scl)
      double precision :: facnorm(nMax_scl)
      double precision :: factor(nMax_scl)

      wtsum = bino_Dfrag2(ix, iflav, icorr, -1, -1, npar, kinwt, flav_block)

      end function bino_Dfrag


      !> i1 & i2 independent -> same convention as above (0,1,2) <-> (uncorr,PDF1,PDF2)
      !> i1 correlated to i2 -> set to some final-sate index (>2)
      !> i1 & i2 & IS correalted -> i1c = [1|2], i2c = [1|2] figure out PDF correlation here
      function bino_Dfrag2(ix, i1flav, i1corr, i2flav, i2corr, npar, kinwt, flav_block) result(wtsum)
      use Channels_mod, only : ip1_chan, ip2_chan
      use PhotonFragmentation_mod
      use Scales_mod
      use KinData_mod, only : zfrag_kin, zregfrag_kin, z2frag_kin, zreg2frag_kin
      implicit none
      double precision :: z_val, z2_val
      double precision :: wtsum
      double precision :: bino
      integer, intent(in) :: ix
      integer, intent(in) :: i1flav, i1corr, i2flav, i2corr, npar
      double precision, intent(in) :: kinwt(nMax_scl)
      integer, intent(in) :: flav_block
      integer :: ip1_sav, ip2_sav, ip1_unroll, ip2_unroll
      integer :: jp1_unroll, jp2_unroll
      integer :: i1corr_loc, i2corr_loc
      integer :: n1_unroll, n2_unroll
      !integer, pointer :: ip_unroll
      integer :: i, i1_unroll, i2_unroll, ipass_block
      double precision :: frag_fac
      double precision :: partons(nMax_scl)
      double precision :: facnorm(nMax_scl)
      double precision :: factor(nMax_scl)

      wtsum = 0d0

      !> save to restore at the end
      ip1_sav = ip1_chan
      ip2_sav = ip2_chan

      z_val = get_z_val(iz_frag,  zfrag_kin,  zregfrag_kin)

      !> only one fragmentation function
      if ( (i2flav < 0) .or. (i2corr < 0) ) then
        frag_fac = 1d0
        if ( i1corr == 0 .or. i1corr > 2 ) then
        !> correlation to FS in single frag case means no correlation at all
           i1corr_loc = 0
        else if (i1corr == 1 .or. i1corr == 2 ) then
           i1corr_loc = i1corr
        else
           print*, "bino_Dfrag2: single Dfrag correlation: invalid i1corr: ", i1corr
           stop
        end if
        n1_unroll = n_unroll(i1flav, i1corr_loc, frag_fac)
        do i1_unroll=1,n1_unroll
          jp1_unroll=ip_unroll(i1_unroll, i1flav, i1corr_loc)
          ipass_block = 1
          call getqcdnorm(ix,partons,facnorm, npar, ipass_block, flav_block)
          if (ipass_block /= 0) then
            factor = 0d0
            do i=1,nAct_scl
              factor(i) = D1_frag(jp1_unroll,z_val, muA2_scl(i))
            enddo
            factor = kinwt*facnorm*factor*frag_fac
            wtsum = wtsum + bino(ix,partons,factor)
          end if  ! ipass_block
        end do  ! end i1_unroll

      !> two fragmentation functions
      else
        z2_val = get_z_val(iz2_frag, z2frag_kin, zreg2frag_kin)

        !> two fragmentation functions correlated
        if ( (i1corr > 2) .and. (i2corr > 2) ) then
           if ( i1flav /= i2flav ) then
              !> if we have flavour changing vertices this will probably fail!
            print*, "bino_Dfrag2: F-F corr. invalid state: ", i1flav, i2flav
            stop
          end if

          do i1_unroll=1,nf1_frag
            jp1_unroll = i1flav + 2*(i1_unroll-1)
            jp2_unroll = jp1_unroll
            ipass_block = 1
            call getqcdnorm(ix,partons,facnorm, npar, ipass_block, flav_block)
            if (ipass_block /= 0) then
              factor = 0d0
              do i=1,nAct_scl
                factor(i) = D1_frag(jp1_unroll,z_val, muA2_scl(i))
     &                    * D1_frag(jp2_unroll,z2_val,muA2_scl(i))
              enddo
              factor = factor / DBLE(nf1_frag)
              factor = kinwt*facnorm*factor
              wtsum = wtsum + bino(ix,partons,factor)
            end if  ! ipass_block
          end do

        !> all correlated
        else if ( (mod(ip1_sav,10)==0) .and. (mod(ip2_sav,10)==0) .and.
     &            (i1corr>0) .and. (i2corr>0) ) then
          if ( (i1flav /= i2flav) .or. (i1flav == 0) .or. (i2flav == 0) ) then
            print*, "bino_Dfrag2: 4x corr. invalid state: ", i1flav, i2flav
            stop
          end if

          do i1_unroll=1,nf1_frag
            jp1_unroll = i1flav + 2*(i1_unroll-1)
            jp2_unroll = jp1_unroll
            ip1_chan = sign(jp1_unroll, ip1_sav)
            ip2_chan = sign(jp2_unroll, ip2_sav)
            ipass_block = 1
            call getqcdnorm(ix,partons,facnorm, npar, ipass_block, flav_block)
            if (ipass_block /= 0) then
              factor = 0d0
              do i=1,nAct_scl
                factor(i) = D1_frag(jp1_unroll,z_val, muA2_scl(i))
     &                    * D1_frag(jp2_unroll,z2_val,muA2_scl(i))
              enddo
              factor = kinwt*facnorm*factor
              wtsum = wtsum + bino(ix,partons,factor)
            end if  ! ipass_block
          end do

        !> two independent fragmentations
        else
          frag_fac = 1d0
          if ( i1corr == 0 .or. i1corr > 2 ) then
             i1corr_loc = 0
          else if ( i1corr == 1 .or. i1corr == 2 ) then
             i1corr_loc = i1corr
          else
             print*, "bino_Dfrag2: double Dfrag correlation: invalid i1corr: ", i1corr
             stop
          end if
          if ( i2corr == 0 .or. i2corr > 2 ) then
             i2corr_loc = 0
          else if ( i2corr == 1 .or. i2corr == 2 ) then
             i2corr_loc = i2corr
          else
             print*, "bino_Dfrag2: double Dfrag correlation: invalid i2corr: ", i2corr
             stop
          end if
          n1_unroll = n_unroll(i1flav, i1corr_loc, frag_fac)
          n2_unroll = n_unroll(i2flav, i2corr_loc, frag_fac)
          do i1_unroll=1,n1_unroll
          do i2_unroll=1,n2_unroll
            jp1_unroll=ip_unroll(i1_unroll, i1flav, i1corr_loc)
            jp2_unroll=ip_unroll(i2_unroll, i2flav, i2corr_loc)
            ipass_block = 1
            call getqcdnorm(ix,partons,facnorm, npar, ipass_block, flav_block)
            if (ipass_block /= 0) then
              factor = 0d0
              do i=1,nAct_scl
                factor(i) = D1_frag(jp1_unroll,z_val, muA2_scl(i))
     &                    * D1_frag(jp2_unroll,z2_val,muA2_scl(i))
              enddo
              factor = kinwt*facnorm*factor*frag_fac
              wtsum = wtsum + bino(ix,partons,factor)
            end if  ! ipass_block
          end do  ! end i1_unroll
          end do  ! end i2_unroll

        end if  ! end cases correlation
      end if  ! end two fragmentation functions

      !> restore original state
      ip1_chan = ip1_sav
      ip2_chan = ip2_sav

      contains

      function get_z_val(iz, zfrag, zregfrag) result(zret)
        integer, intent(in) :: iz
        double precision, intent(in) :: zfrag, zregfrag
        double precision :: zret
        zret = 0d0
        select case (iz)
        case (1)
          zret = zfrag
        case (2)
          zret = zfrag / zregfrag
        case default
          stop "bino_Dfrag2::get_z_val: invalid iz!"
        end select
      end function get_z_val

      function n_unroll(iflav, icorr, fac) result(n)
        implicit none
        integer, intent(in) :: iflav, icorr
        double precision, intent(inout) :: fac
        integer :: n
        integer :: ip_in
        if (icorr == 0) then
          if (iflav == 0) then
            n = 1
          else
            !> this assumes there exists only ONE uncorrelated
            !> quark line in the final state!
            n = nf1_frag
            fac = fac / dble(nf1_frag)
          end if
        else  !> correlated
          select case (icorr)
          case (1)
            ip_in = ip1_chan
          case (2)
            ip_in = ip2_chan
          case default
             print *, "n_unroll: invalid icorr: ", icorr
             stop
          end select
          select case (abs(ip_in))
          case (1,2,3,4,5)
            n = 1
            if (iflav/=mod(abs(ip_in)-1,2)+1) stop "iflav inconsistent!"
          case (10,11)
            n = 3
            if (iflav/=1) stop "iflav inconsistent!"
          case (40,41)
            n = 2
            if (iflav/=1) stop "iflav inconsistent!"
          case (20,21)
            n = 2
            if (iflav/=2) stop "iflav inconsistent!"
          case default
            print*, "bino_Dfrag: invalid ip_in = ", ip_in
            stop
          end select
        end if
      end function n_unroll

      function ip_unroll(i_, iflav, icorr) result(id)
        implicit none
        integer, intent(in) :: i_, iflav, icorr
        integer :: id
        id = iflav + 2*(i_-1)
        select case(icorr)
        case (0)
          !> uncorrelated: no PDF flavour to set
        case (1)
          ip1_chan = sign(id, ip1_sav)
        case (2)
          ip2_chan = sign(id, ip2_sav)
        case default
          print*, "bino_Dfrag2::ip_unroll: icorr unknown:", icorr
          print*, "arguments: ", i_, iflav, icorr
          stop
        end select
      end function ip_unroll

      end function bino_Dfrag2


      !> iflav: flavour of the parton that fragments to a photon
      !> icorr: 0 if gluon or final-state, [1|2] if initial-state
      !-----------------------------------------------------------------------------
      !> @brief
      !> dress
      !
      !> @param[in] z momentum fraction of the photon
      !> @param[in] muF2 [GeV^2] fragmentation scale
      !> @return weight
      !-----------------------------------------------------------------------------
      function bino_Hfrag(ix, npar, kinwt, flav_block) result(wtsum)
      use Channels_mod, only : ip1_chan, ip2_chan
      use HadronFragmentation_mod
      use Scales_mod
      use KinData_mod, only : zfrag_kin, zregfrag_kin, setFrag_kin
      implicit none
      double precision :: z_val
      double precision :: wtsum
      double precision :: bino
      integer, intent(in) :: ix
      integer, intent(in) :: npar
      double precision, intent(in) :: kinwt(nMax_scl)
      integer, intent(in) :: flav_block
      !>
      integer :: ip1_sav, ip2_sav, ip_in, ip, ip_unroll
      integer :: n_unroll
      !integer, pointer :: ip_unroll
      integer :: i, i_unroll, j_unroll, ipass_block
      double precision :: partons(nMax_scl)
      double precision :: facnorm(nMax_scl)
      double precision :: factor(nMax_scl)
      double precision :: jac
      double precision :: ckm_fac
      wtsum = 0d0

      !> save to restore at the end
      ip1_sav = ip1_chan
      ip2_sav = ip2_chan

      z_val = 0d0
      select case (iz_hfrag)
      case (1)
         z_val = zfrag_kin
      case (2)
        z_val = zfrag_kin / zregfrag_kin
      case default
        stop "bino_Hfrag: invalid iz_hfrag!"
      end select

      !>In the photon case, this appears in the subt.terms
      !>jac = 1d0/(1d0-zfrag_kin)
      jac = 1d0  !@todo:  to be double-checked

      !!print *,"z_val",z_val

      select case(abs(ip1_hfrag))
      !>-----
      case(0)
      !>----- gluon -> H
        ip = 0
        ipass_block = 1
        call getqcdnorm(ix,partons,facnorm, npar, ipass_block, flav_block)
        if (ipass_block /= 0) then
          factor = 0d0
          do i=1,nAct_scl
            factor(i) = D_hfrag(ip, z_val, muA_scl(i))
          enddo
          factor = kinwt*facnorm*factor*jac
          wtsum = wtsum + bino(ix,partons,factor)
        end if

      !>-----
      case (1,2,3,4) !ARG: added 4 for primed quarks
      !>----- quark -> H
        if (icorr1_hfrag == 0) then
          !> (a) in amplitude it's a gluon (qqb -> g)  =>  1/nf & loop[ip]
          !> (b) correlation *only* with f.s. quarks   =>  1/nf & loop[ip]
          !> could in principle could pull out sum over frag[ip]?
          do i_unroll=1,n_qflav(ip1_hfrag)
            ip = i_qflav(i_unroll, ip1_hfrag)
            ipass_block = 1
            call getqcdnorm(ix,partons,facnorm, npar, ipass_block, flav_block)
            !print*, "in bino.f: facnorm",facnorm
            if (ipass_block /= 0) then
              factor = 0d0
              do i=1,nAct_scl
                factor(i) = D_hfrag(ip, z_val,muA_scl(i))
              enddo
              !print*, "in bino.f: factor(i)",factor(i)
              factor = factor / DBLE(n_qflav(ip1_hfrag))
              factor = kinwt*facnorm*factor*jac
              wtsum = wtsum + bino(ix,partons,factor)
            end if  ! ipass_block
          end do
        else
          if ((icorr1_hfrag/=1).and.(icorr1_hfrag/=2)) then
            print*, "bino_Hfrag: invalid icorr1_hfrag = ", icorr1_hfrag
            stop
          end if
          !> correlation with  at least one i.s. quark =>  loop[ip & i.s. setting]
          !>  (a) just one i.s. unrolling of the flavour with the fragmentation
          !>  (b) both i.s. correlated (imagine D-type amplitudes)
          !> emultiplyPDF already gets the correlation right => just keep the other ip_chan as is :D

          if (icorr1_hfrag==1) then
            ip_in = ip1_chan
          else
            ip_in = ip2_chan
          end if

          do i_unroll=1,n_qflav_in(ip_in)
            ip_unroll = i_qflav_in(i_unroll, ip_in)
            if (icorr1_hfrag==1) then
              ip1_chan = ip_unroll
            else
              ip2_chan = ip_unroll
            end if
            ipass_block = 1
            call getqcdnorm(ix,partons,facnorm, npar, ipass_block, flav_block)
            if (ipass_block /= 0) then
              factor = 0d0
              do j_unroll=1,n_qflav_out(ip1_hfrag, ip_unroll)
                 ip = i_qflav_out(j_unroll, ip1_hfrag, ip_unroll, ckm_fac)
                 do i=1,nAct_scl
                    factor(i) = factor(i) + ckm_fac * D_hfrag(ip, z_val, muA_scl(i))
                 enddo
              enddo
              factor = kinwt*facnorm*factor*jac
              wtsum = wtsum + bino(ix,partons,factor)
            end if  ! ipass_block
          end do

        end if

      case default
        !> ##### should never get here
        print*, "bino_Hfrag: invalid ip1_hfrag:", ip1_hfrag
        stop
      end select

      !> restore original state
      ip1_chan = ip1_sav
      ip2_chan = ip2_sav

      contains

      pure function n_qflav(iflav) result(n)
        implicit none
        integer, intent(in) :: iflav
        integer :: n
        select case (abs(iflav))
        case (1)
          n = 3
        case (2)
          n = 2
        case (3)
          n = 5
        case (4) !ARG: primed quarks
          n = 5
        case default
          n = 0
        end select
      end function n_qflav

      pure function i_qflav(i, iflav) result(id)
        implicit none
        integer, intent(in) :: i, iflav
        integer :: id
        integer, parameter, dimension(3) :: arr_d = [1,3,5]
        integer, parameter, dimension(2) :: arr_u = [2,4]
        integer, parameter, dimension(5) :: arr_q = [1,2,3,4,5]
        select case (abs(iflav))
        case (1)
          id = sign(arr_d(i), iflav)
        case (2)
          id = sign(arr_u(i), iflav)
        case (3)
          id = sign(arr_q(i), iflav)
        case default
          id = 0
        end select
      end function i_qflav

      pure function n_qflav_in(iflav) result(n)
        implicit none
        integer, intent(in) :: iflav
        integer :: n
        select case (abs(iflav))
        case (1,2,3,4,5)
          n = 1
        case (10,11)
          n = 3
        case (20,21)
          n = 2
        case (30,31)
          n = 5
        case (40,41)
          n = 2
        case default
          n = 0
        end select
      end function n_qflav_in

      pure function i_qflav_in(i, iflav) result(id)
        implicit none
        integer, intent(in) :: i, iflav
        integer :: id
        integer, parameter, dimension(3) :: arr_d = [1,3,5]
        integer, parameter, dimension(2) :: arr_u = [2,4]
        integer, parameter, dimension(5) :: arr_q = [1,2,3,4,5]
        select case (abs(iflav))
        case (1,2,3,4,5)
          id = iflav
        case (10,11,40,41)
          id = sign(arr_d(i), iflav)
        case (20,21)
          id = sign(arr_u(i), iflav)
        case (30,31)
          id = sign(arr_q(i), iflav)
        case default
          id = 0
        end select
      end function i_qflav_in

      pure function n_qflav_out(iflav, ip_in) result(n)
        implicit none
        integer, intent(in) :: iflav, ip_in
        integer :: n
        integer, parameter, dimension(3) :: arr_d = [1,3,5]
        integer, parameter, dimension(2) :: arr_u = [2,4]
        !> default case <-> flavour diagonal
        n = 1
        !> catch all flavour-changing cases here
        select case (abs(iflav))
        case (1)
          if ( all(arr_d /= abs(ip_in)) )  n = 3
        case (2)
          if ( all(arr_u /= abs(ip_in)) )  n = 3
        end select
      end function n_qflav_out

      function i_qflav_out(i, iflav, ip_in, ckm_fac) result(id)
        use CKM_mod
        implicit none
        integer, intent(in) :: i, iflav, ip_in
        double precision, intent(out) :: ckm_fac
        integer :: id
        integer, parameter, dimension(3) :: arr_d = [1,3,5]
        integer, parameter, dimension(2) :: arr_u = [2,4]
        integer, parameter, dimension(3,6) :: map_iso =
     &    reshape([2,1,4,3,6,5, 4,3,6,5,2,1, 6,5,2,1,4,3 ], shape(map_iso), order=[2,1])
        !> default case <-> neutral current
        id = sign(ip_in, iflav)
        ckm_fac = 1d0
        !> catch all flavour-changing cases here
        select case (abs(iflav))
        case (1)
          if ( all(arr_d /= abs(ip_in)) ) then
             id = sign(map_iso(i, abs(ip_in)), iflav)
             if (abs(id) <= 5) then
                ckm_fac = CKM(abs(ip_in),abs(id))
             else
                ckm_fac = 0d0
             end if
          end if
        case (2)
          if ( all(arr_u /= abs(ip_in)) ) then
             id = sign(map_iso(i, abs(ip_in)), iflav)
             if (abs(id) <= 5) then
                ckm_fac = CKM(abs(id),abs(ip_in))
             else
                ckm_fac = 0d0
             end if
          end if
        end select
      end function i_qflav_out
      end function bino_Hfrag
