***************************************************
* Boring wrappers...
*
      subroutine ecuts_jet(ilevel,npar,ipass)
      use KinData_mod
      use Proj2Born_mod
      implicit none
      integer ilevel,npar,ipass
      if (isActive_p2b()) then
        if (ilevel == 0) call project_jet_p2b(npar,ipass)
        call ecuts(ilevel,npar_p2b,ipass)
      else
        call ecuts(ilevel,npar,ipass)
      end if
      return
      end

      subroutine ecuts_vj(ilevel,npar,ipass)
      use KinData_mod
      use Observables_mod
      use Proj2Born_mod
      implicit none
      integer ilevel,npar,ipass
      if (isActive_p2b()) then
        ! print*, ilevel, " > called ecuts_vj with P2B:", npar, npar_p2b
        !> unfortunately we don't have one single P2B kinematics (not like DIS)
        !>   imagine: H + 2 partons and an II mapping.
        !>   it preserves the rapidity for the full FS:
        !>     y(H+2)_full = y(H+1)_mapped
        !>   but not for the Higgs itself  =>  always have to map!
        if (ilevel == 0) call project_hdy_p2b(npar,ipass)
        call ecuts(ilevel,npar_p2b,ipass)
        !call printAll_obs(npar_p2b)
      else
        !print*, "called ecuts_vj", npar
        call ecuts(ilevel,npar,ipass)
        !call printAll_obs(npar)
      end if
      !read(*,*)
      return
      end

      subroutine ecuts_vhj(ilevel,npar,ipass)
      use KinData_mod
      implicit none
      integer ilevel,npar,ipass
      call ecuts(ilevel,npar,ipass)
      return
      end

      subroutine ecuts_dis1j(ilevel,npar,ipass)
      use KinData_mod
      use Proj2Born_mod
      implicit none
      integer ilevel,npar,ipass
      if (isActive_p2b()) then
        call ecuts(ilevel,npar_p2b,ipass)
        !print*, "called ecuts_dis1j", npar_p2b
        !read(*,*)
      else
        call ecuts(ilevel,npar,ipass)
        !print*, "called ecuts_dis1j", npar
        !read(*,*)
      end if
      return
      end

      subroutine ecuts_dis(ilevel,npar,ipass)
      use Channels_mod
      use KinData_mod
      use Jets_mod
      use ObsHelper_mod
      use Observables_mod
      use MultiRun_mod, only : applySelectors_mult
      use Scales_mod
      use Selectors_mod
      use PhotonIsolation_mod
      use DIS_mod
      implicit none
      integer ilevel, npar, ipass, counter
      !> transformation matrix HERA to Breit frame
      double precision finv(4,4)
      double precision, parameter, dimension(4) :: gmn = (/-1d0,-1d0,-1d0,1d0/)
      double precision phera(4), valObs
      integer i,j,k, newnjets
      logical qpass(10), qSkipCache
      !> PDF mapping
      integer ifh1,ifs1,ifh2
      integer jfh1,jfs1,jfh2
      common /currentmap/ ifh1,ifs1,ifh2,jfh1,jfs1,jfh2
!$omp threadprivate(/currentmap/)
      integer imethod
      common /method/imethod
      double precision jet_aux(4,4),q2,sumEh,eta_i,eta_j
      integer old_imax,i_max,istart,inext
      double precision eta_max,last_max,pt_max
      integer :: nv

      !> reset stored mapping configuration
      if (ilevel == 0) then
         ifh1 = 0
         ifs1 = 0
         ifh2 = 0

         jfh1 = 0
         jfs1 = 0
         jfh2 = 0
      end if
      !> DIS has only one level (0)
      if ( ilevel /= 0 ) return

      if (ireject_event_chan /= 0) ipass = 0
      if (ipass == 0) return

      !> check for NaN's in momntum set
      !> by definition:  NaN != NaN
      if ( any( kin(npar)%p /= kin(npar)%p ) ) then
         print*, "ecuts_dis: found NaN in momentum set", npar
         call print_xvegas()
         call print_momenta(kin(npar)%npar_from)
         call print_momenta(npar)
         ipass = 0
         ! read(*,*)
         return
      end if

      call currentContrib_chan(nsplit=nv)

      !> If one does not want to use the boost to Breit
      if(isSetBreitFrame) then
         if(.not.useBreitFrame) call ecuts_dis1j(ilevel,npar,ipass)
         return
      endif

      if(getDISEVTSHAPE()) then
         sumEh = 0d0
         do i=4,npar
            if(kin(npar)%p(3,i)>0d0) then
               sumEh=sumEh+kin(npar)%p(4,i)
            endif
         enddo
         q2 = value_q2_dis
         if(sumEh<dsqrt(q2)*getEhfrac()) then
            ipass=0
            return
         endif
      endif


      !----- all "global" cuts in the HERA frame are applied in genphase

      qSkipCache = .false.
      !----- jet clustering in the  Breit frame
      call recombine_jet(npar)
      !> skip everything to do with jets
      if(algo_jet().ne.0 .and. .not.kin(npar)%qcachedObs) then
      !----- additional "extra" cuts for jets in the HERA frame

      !> determine transformation matrix
         finv = 0d0
         do j=1,4
            do i=1,4
               finv(i,j) = gmn(i)*dis%breitT(i,j)*gmn(j)
            end do
         end do

      !> check which jets are rejected in the HERA frame
         qpass = .true.

         dis%rawjets=kin(npar)%njets

         do i=1,kin(npar)%njets
      !> momentum im HERA frame
            phera = 0d0
            do j=1,4
               do k=1,4
                  phera(j) = phera(j) + finv(k,j)*kin(npar)%pjets(k,i)
               end do
        !> copying over HERA jets ordered in Breit frame pt into rawHERA_jets
               dis%raw_HERAjets(j,i)=phera(j)
            end do

        !> check all "extra" observables
        !> these are the jet identifiers in the HERA frame
            if (qpass(i)) then
               valObs   = v1_pt(phera)
               qpass(i) = qpass_jet(idj_hera_ptj, valObs)
            end if

            if (qpass(i)) then
               valObs   = v1_y(phera)
               qpass(i) = qpass_jet(idj_hera_yj, valObs)
               valObs   = abs(valObs)
               qpass(i) = qpass(i) .and. qpass_jet(idj_hera_abs_yj, valObs)
            end if

            if (qpass(i)) then
               valObs   = v1_eta(phera)
               qpass(i) = qpass_jet(idj_hera_etaj, valObs)
               valObs   = abs(valObs)
               qpass(i) = qpass(i) .and. qpass_jet(idj_hera_abs_etaj, valObs)
            end if

            if (qpass(i)) then
               valObs   = v1_et(phera)
               qpass(i) = qpass_jet(idj_hera_etj, valObs)
            end if

            if (qpass(i)) then
               valObs   = phera(3)/getEp()
               qpass(i) = qpass_jet(idj_jets_xhera, valObs)
            end if

            if (qpass(i)) then
               valObs   = phera(4)/getEp()
               qpass(i) = qpass_jet(idj_jets_H1xhera, valObs)
            end if

         end do

      !> trim away those jets and update KinData
         newnjets = 0
         do i=1,kin(npar)%njets
            if (qpass(i)) then
               newnjets = newnjets+1
      !> copy over HERAjets after all acceptance cuts
               dis%HERAjets(1:4,newnjets)=dis%raw_HERAjets(1:4,i)
               if (i/=newnjets) then
                  kin(npar)%pjets(1:4,newnjets) = kin(npar)%pjets(1:4,i)
               end if
            end if
         end do

         kin(npar)%njets = newnjets

      !> If needed order hera jets in eta
         if(gethera_orderEta().and.newnjets.ge.2) then
            !> auxilary jet array
            jet_aux=0d0
            jet_aux(1:4,1:newnjets)=dis%HERAjets(1:4,1:newnjets)

            do i=1,newnjets
               counter=1
               eta_i=v1_eta(jet_aux(1:4,i))
               do j=1,newnjets
                  if(i.eq.j) cycle
                  eta_j=v1_eta(jet_aux(1:4,j))
                  if(eta_i.lt.eta_j) counter=counter+1
               enddo
               dis%HERAjets(1:4,counter)=jet_aux(1:4,i)
#ifndef NDEBUG
               !> debug sanity check
               if(counter.gt.newnjets) stop "Counter too large"
#endif
            enddo

#ifndef NDEBUG
            !> debug
            do i=1,newnjets-1
               if(v1_eta(dis%HERAjets(1:4,i)).lt.
     .              v1_eta(dis%HERAjets(1:4,i+1))) then
                  write(*,*) "Ordering does not work"
                  stop
               endif
            enddo
#endif
         endif
         !> only eval transformation if needed
         if(boostPGAM) then
      !> transform jets into parton gamma rest frame and store
            dis%HERApgammajets(:,:)=0d0
            do i=1,kin(npar)%njets
               do j=1,4
                  do k=1,4
                     dis%HERApgammajets(j,i) = dis%HERApgammajets(j,i)
     $                    + dis%BREITgammaprT(j,k)*kin(npar)%pjets(k,i)
                  end do
               end do
            enddo
         endif

         !> clear the cache since jets have changed!
         qSkipCache = .true.
      endif

      !----- evaluate observables (SKIP JET CLUSTERING)
      call evalAll_obs(npar, skipJetAlgo=.true., skipCache=qSkipCache)

      !----- all observables correctly assigned: cut!
      call applyAll_sel(npar, ipass, skipFlav=.true.)
      if(ipass.eq.0) return

      !----- apply selectors of the current run (nothing if no MultiRun)
      call applySelectors_mult(ipass)
      if(ipass.eq.0) return

      !> update scales & store in KinData cache
      call update_scl(ipass)
      if(ipass.eq.0) return

      !> update alphas arrays
      call updateAlphaS_scl()

      !> update PDF arrays
      if((nv.gt.0).and.(imethod.eq.2))then ! virtual integration with imethod = 2
        call updatePDF_scl(withXRegions=.true.)
      else
        call updatePDF_scl()
      end if

      end

***************************************************
*
*
      subroutine ecuts_epem(ilevel,npar,ipass)
      use KinData_mod
      use Proj2Born_mod
      implicit none
      integer ilevel,npar,ipass
      if (isActive_p2b()) then
        !print*, "ecuts_epem: P2B triggered", ilevel
        call project_epem_p2b(npar,ipass)
        call ecuts(ilevel,npar_p2b,ipass)
      else
        !print*, "ecuts_epem: normal", ilevel
        call ecuts(ilevel,npar,ipass)
      end if
      return
      end


***************************************************
* Main cut routine
*
      subroutine ecuts(ilevel,npar,ipass)
      use KinData_mod
      use Observables_mod
      use Channels_mod
      use MultiRun_mod, only : applySelectors_mult
      use Scales_mod
      use Selectors_mod
      use PhotonIsolation_mod
      use LeptonIsolation_mod
      use PhotonFragmentation_mod
      use HadronFragmentation_mod, only : isActive_hfrag, idx_hfrag
      use Flavour_mod, only: isActive_flav
      use Process_mod, only: nPDFs_proc
      implicit none
      integer ilevel, npar, ipass, ipass_direct
      integer ifh1,ifs1,ifh2
      integer jfh1,jfs1,jfh2
      common /currentmap/ ifh1,ifs1,ifh2,jfh1,jfs1,jfh2
!$omp threadprivate(/currentmap/)
      !> for x[1|2]_pdf observable... need better solution some day
      integer :: igx
      common /gridix/ igx
!$omp threadprivate(/gridix/)
      !> needed for new PDF / scale storage
      integer imethod
      common /method/imethod
      integer :: nv

      igx = ix_chan

      !> reset stored mapping configuration
      if (ilevel == 0) then
         ifh1 = 0
         ifs1 = 0
         ifh2 = 0

         jfh1 = 0
         jfs1 = 0
         jfh2 = 0
      end if

      if (ireject_event_chan /= 0) ipass = 0
      if (ipass == 0) return

      !> check for NaN's in momentum set
      !> by definition:  NaN != NaN
      if ( any( kin(npar)%p /= kin(npar)%p ) ) then
         print*, "ecuts: found NaN in momentum set", npar
         call print_xvegas()
         call print_momenta(kin(npar)%npar_from)
         call print_momenta(npar)
         ireject_event_chan = -2
         ipass = 0
         ! read(*,*)
         return
      end if

      !> ecuts_flav is responsible for observables and scales in flavtag
      if ( isActive_flav() ) return
      !> variable number of photons!
      !>  => cuts applied at higher level: qcdnorm
      if ( (ilevel == 0) .and. (max_nphotons_brems > 0) ) return
      if ( (ilevel == 0) .and. isActive_frag() ) return
      !> if not, skip to save time
      if ( (ilevel == 1)
     &     .and. .not.(max_nphotons_brems > 0)
     &     .and. .not.isActive_frag()
     &   ) return
      !> fragmenting momentum is fixed: can apply cuts immediately
      if ( (ilevel /= 0) .and. isActive_hfrag() ) return


      !> fragmentation-specific branches
      !> check if direct contribution is ON
      !> fragmentation case is dealt with in bino_Dfrag and will never enter here!!!
      ipass_direct = 1
      if ( isActive_frag(ipass_direct = ipass_direct) ) then
         if ( (.not.isSet_frag()) .and. (ipass_direct==0) )  ipass = 0
         if ( ipass == 0 ) return
#ifndef NDEBUG
         if ( (iz_frag/=1) .and. (iz_frag/=2) ) then
            print*, "ecuts: inconsistent iz_frag state:", iz_frag
            stop
         end if
#endif
      end if
      !> only convolution terms contribute to iz_frag==2
      !> (these are the terms with negative nphoton_brems)
      if ((iz_frag==2) .and. (.not.isSet_frag(1))) then
         ipass = 0
         return
      end if
      if ((iz2_frag==2) .and. (.not.isSet_frag(2))) then
         ipass = 0
         return
      end if


      if(.not.fully_inclusive) then

         !> As photon and hadron fragmentation use the same infrastructure in KinData,
         !> in order to run isolated photon + hadron, we assume just the photon direct component
         !> inside the PhotonIsolation mod
         if (isActive_hfrag()) then
            call setFrag_isol(.false.)
         endif

         call applyAll_isol(npar, ipass)
         if(ipass.eq.0) return
         call apply_lepisol(npar, ipass)
         if(ipass.eq.0) return
         !> @todo: double-check but appears to conflict by overwriting internal zfrag infromation
         !> else
         !>    call setFrag_kin(npar, idx_hfrag)

         call evalAll_obs(npar)

         call applyAll_sel(npar, ipass, skipFlav=.true.)
         if(ipass.eq.0) return

         call applySelectors_mult(ipass)
         if(ipass.eq.0) return
      endif

      !>----- only need to update scales and PDFs if cuts are passed

      !> update scales & store in KinData cache
      call update_scl(ipass)
      if(ipass.eq.0) return

      !> update alphas arrays
      call updateAlphaS_scl()

      !> update PDF arrays
      if (nPDFs_proc > 0) then
         call currentContrib_chan(nsplit=nv)
         if ( (nv > 0) .and. (imethod == 2) ) then ! virtual integration with imethod = 2
           call updatePDF_scl(withXRegions=.true.)
         else
           call updatePDF_scl()
         end if
      end if

      end



***************************************************
* flavour-sensitive cut routine
*
      subroutine ecuts_flav(ilevel, npar, ipass)
      use KinData_mod
      use Observables_mod
      use Channels_mod
      use MultiRun_mod, only : applySelectors_mult
      use Scales_mod
      use Selectors_mod
      use PhotonIsolation_mod
      use LeptonIsolation_mod
      use Flavour_mod, only: isActive_flav, has_changed_flav
      use Process_mod, only: nPDFs_proc
      implicit none
      integer ilevel, npar, ipass
      !> needed for new PDF / scale storage
      integer imethod
      common /method/imethod
      integer :: nv

      if (ipass == 0) return

#ifndef NDEBUG
      if ( .not.isActive_flav() )
     .  stop "ecuts_flav: should not be called if flavour-tagging is inactive"
#endif

      if (has_changed_flav(npar)) then

         call applyAll_isol(npar, ipass)
         if (ipass == 0) return
         call apply_lepisol(npar, ipass)
         if (ipass == 0) return

         !> clustering depends on flavour -> cannot cache
         call evalAll_obs(npar, skipCache=.true.)
         ! call printAll_obs(npar, skipCache=.true.)

         call evalFlav_obs(npar)

      end if
      call saveFlav_obs()  ! save to flavour buffer

      call applyAll_sel(npar, ipass, skipFlav=.false.)
      if (ipass == 0) return

      call applySelectors_mult(ipass)
      if (ipass == 0) return


      !> skip evolution if we know things didn't change
      if (.not.has_changed_flav(npar)) return

      !>----- only need to update scales and PDFs if cuts are passed

      !> update scales & store in KinData cache
      call update_scl(ipass)
      if(ipass.eq.0) return

      !> update alphas arrays
      call updateAlphaS_scl()

      !> update PDF arrays
      if (nPDFs_proc > 0) then
         call currentContrib_chan(nsplit=nv)
         if((nv.gt.0).and.(imethod.eq.2))then ! virtual integration with imethod = 2
           call updatePDF_scl(withXRegions=.true.)
         else
           call updatePDF_scl()
         end if
      end if

      end
