******************************************************************
* A general subroutine for extracting a specific phase space from
* the phase space generator. In principle this could be used to
* deal with a different phase space generator to phasenew/phasenewh
*


      subroutine genphase(ipset,ievt,ix,x1,x2,xreg1,xreg2,ifail,rnorm)
      use Process_mod, only: name_proc
      use Run_mod, only: ilips_run
      use DIS_mod, only: isSetBreitFrame,useBreitFrame
      use KinData_mod
      use Observables_mod, only: evalAll_obs, printAll_obs
      implicit real*8(a-h,o-y)
      implicit complex*16(z)
      logical boostBREIT

      !> set global observable `npar_max`
      npar_max = ipset

      if ( (name_proc(2:).eq.'JET' .or.
     &     name_proc(2:).eq.'JETFC' .or.
     &     name_proc(1:2).eq.'GJ' .or.
     &     name_proc.eq.'G_EW' .or.
     &     name_proc.eq.'GGG' .or.
     &     name_proc.eq.'GG_EW').and.(ilips_run.eq.0)) then
      !> incase of GG with qT PS generator (ilips_run) go to genphasenewh
         call genphasenew(ipset,ievt,ix,x1,x2,xreg1,xreg2,ifail)
      elseif( name_proc(:3).eq.'VFH' ) then
         call genphasenewJ(ipset,ievt,ix,x1,x2,xreg1,xreg2,ifail)
      !> new genphase routine for DISG for now, maybe not necessary
      elseif ( name_proc.eq.'DISG' ) then
         call genphasenewDISG(ipset,ievt,ix,x1,x2,xreg1,xreg2,ifail)
      elseif( name_proc(1:3).eq.'DIS' .or.
     &        name_proc(1:5).eq.'SFDIS' ) then
         boostBREIT=.true.
         !>cannot do Breitframe analysis for DIS1j
         if ( name_proc.eq.'DIS1J' .or.
     &        name_proc.eq.'DISWP1J' .or.
     &        name_proc.eq.'DISWM1J' .or.
     &        name_proc.eq.'SFDIS' .or.
     &        name_proc.eq.'SFDISWP' .or.
     &        name_proc.eq.'SFDISWM' ) BOOSTBREIT=.FALSE.
         !> overwrite boostBreit if set in setup
         if(isSetBreitFrame) boostBreit=useBreitFrame
         call genphasedis(ipset,ievt,ix,x1,x2,xreg1,xreg2,ifail,boostBREIT)
      elseif ( name_proc(1:5).eq.'HTO4E' .or.
     &         name_proc(1:2).eq.'HH'.or.
     &         name_proc(1:8).eq.'HTO2E2MU' .or.
     &         name_proc(1:7).eq.'HTO2L2N' .or.
     &         name_proc(1:2).eq.'ZH' .or.
     &         name_proc(1:3).eq.'WMH' .or.
     &         name_proc(1:3).eq.'WPH' ) then
         call genphasenewhto4(ipset,ievt,ix,x1,x2,xreg1,xreg2,ifail)
      elseif ( name_proc(1:7).eq.'HTO2L1P' ) then
         call genphasenewhto3(ipset,ievt,ix,x1,x2,xreg1,xreg2,ifail)
      elseif ( name_proc(1:4).eq.'EPEM' ) then
         if (ilips_run == 1) then
            call genphaseomt(ipset,ievt,ix,x1,x2,xreg1,xreg2,ifail,rnorm)
         else
           if (ipset == 4) then
             call genphasenew(ipset,ievt,ix,x1,x2,xreg1,xreg2,ifail)
           else
             call genphaseepem(ipset,ievt,ix,x1,x2,xreg1,xreg2,ifail)
           end if
         end if
      else
         call genphasenewh(ipset,ievt,ix,x1,x2,xreg1,xreg2,ifail)
      endif


      !> exit in case of failure (P2B: `id_DIS_x` not defined)
      if (ifail /= 0) return


      call clearCacheLocal_kin(ipset)
      kin(ipset)%npar_from = ipset
      call resetFrag_kin(ipset)

#ifdef USE_OL
      do i=1,ipset
         kin(ipset)%pOL(1:3,i)=kin(ipset)%p(1:3,i)
         kin(ipset)%pOL(0,i)=kin(ipset)%p(4,i)
      enddo
#endif
      return
      end


*********************************************************
* Generates a phase space point and boosts into the lab
* frame for phasenew
*

      subroutine genphasenew(ipset,ievt,ix,x1,x2,xreg1,xreg2,ifail)
      use Process_mod, only : name_proc
      use Channels_mod, only : currentSubContrib_chan
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16(z)
      character*128 slogname
      common/pin/pa(4),pb(4)
!$omp threadprivate(/pin/)
      common/eventmom2p/pevt2(4,2,2)
!$omp threadprivate(/eventmom2p/)
      common/eventmom3p/pevt3(4,3,12)
!$omp threadprivate(/eventmom3p/)
      common/eventmom4ap/pevt4a(4,4,96)
!$omp threadprivate(/eventmom4ap/)
      common/eventmom4bp/pevt4b(4,4,24)
!$omp threadprivate(/eventmom4bp/)
      common/eventmom4allp/pevt4all(4,4,48)
!$omp threadprivate(/eventmom4allp/)
      common/eventmom5alln/pevt5alln(4,5,160)
!$omp threadprivate(/eventmom5alln/)
      common/eventmom5an/pevt5an(4,5,240)
!$omp threadprivate(/eventmom5an/)
      common/eventmom5bn/pevt5bn(4,5,120)
!$omp threadprivate(/eventmom5bn/)
      common/eventinv2p/sevt2(2,2,2),tevt2(2,2,2),ievt2(2)
!$omp threadprivate(/eventinv2p/)
      common/eventinv3p/sevt3(3,3,12),tevt3(2,3,12),ievt3(12)
!$omp threadprivate(/eventinv3p/)
      common/eventinv4ap/sevt4a(4,4,96),tevt4a(2,4,96),ievt4a(96)
!$omp threadprivate(/eventinv4ap/)
      common/eventinv4bp/sevt4b(4,4,24),tevt4b(2,4,24),ievt4b(24)
!$omp threadprivate(/eventinv4bp/)
      common/eventinv4allp/sevt4all(4,4,48),tevt4all(2,4,48),ievt4all(48)
!$omp threadprivate(/eventinv4allp/)
      common/eventinv5alln/sevt5alln(5,5,160),tevt5alln(2,5,160),ievt5alln(160)
!$omp threadprivate(/eventinv5alln/)
      common/eventinv5an/sevt5an(5,5,240),tevt5an(2,5,240),ievt5an(240)
!$omp threadprivate(/eventinv5an/)
      common/eventinv5bn/sevt5bn(5,5,120),tevt5bn(2,5,120),ievt5bn(120)
!$omp threadprivate(/eventinv5bn/)

      common /method/imethod
      common /log/slogname
      !> region info
      integer :: iregion

      call currentSubContrib_chan(1, region=iregion)

      if((imethod.eq.1).and.(ipset.eq.4))then
         write(*,*) "imethod=1 is not supported for phase spaces with"
         write(*,*) "2 partons"
         stop
      endif

      ifail=1
      ipass=0

*     check to see if the phase space fell outside the wedge
      if(ipset.eq.5)then
         if(ievt3(ievt).eq.0)then
            return
         endif
      elseif((ipset.eq.6).and.(name_proc(1:3).ne.'GJJ')
     .                   .and.(name_proc(1:4).ne.'3JET')
     .                   .and.(name_proc(1:4).ne.'4JET'))then
         if(iregion.eq.1)then
            if (ievt4a(ievt).eq.0) return
         elseif(iregion.eq.2)then
            if (ievt4b(ievt).eq.0) return
         endif
      elseif((ipset.eq.6).and.(    (name_proc(1:3).eq.'GJJ')
     .                         .or.(name_proc(1:4).eq.'3JET')
     .                         .or.(name_proc(1:4).eq.'4JET') ))then
         if (ievt4all(ievt).eq.0) return
      elseif((ipset.eq.7).and.(   (name_proc(1:4).eq.'5JET')
     .                        .or.(name_proc(1:4).eq.'4JET') ))then
         if (ievt5alln(ievt).eq.0) return
      elseif((ipset.eq.7).and.(   (name_proc(1:4).ne.'5JET')
     .                        .and.(name_proc(1:4).ne.'4JET') ))then
         if (iregion.eq.1) then
            if (ievt5an(ievt).eq.0) return
         elseif (iregion.eq.2)then
            if (ievt5bn(ievt).eq.0) return
         endif
      endif



      if(ipset.eq.4)then
         call p_to_common(2,pa,pb,pevt2(1:4,1:2,ievt))
         call fillS_kin(4)
      elseif(ipset.eq.5)then
         call p_to_common(3,pa,pb,pevt3(1:4,1:3,ievt))
         call fillS_kin(5)
      elseif((ipset.eq.6).and.(name_proc(1:3).ne.'GJJ')
     .                   .and.(name_proc(1:4).ne.'3JET')
     .                   .and.(name_proc(1:4).ne.'4JET'))then
         if(iregion.eq.1)then
            call p_to_common(4,pa,pb,pevt4a(1:4,1:4,ievt))
         elseif(iregion.eq.2)then
            call p_to_common(4,pa,pb,pevt4b(1:4,1:4,ievt))
         else
            write(*,*) "Invalid wedge type for ipset=6:", iregion
            stop
         endif
         call fillS_kin(6)
      elseif((ipset.eq.6).and.(    (name_proc(1:3).eq.'GJJ')
     .                         .or.(name_proc(1:4).eq.'3JET')
     .                         .or.(name_proc(1:4).eq.'4JET') ))then
         call p_to_common(4,pa,pb,pevt4all(1:4,1:4,ievt))
         call fillS_kin(6)
      elseif((ipset.eq.7).and.(   (name_proc(1:4).eq.'5JET')
     .                        .or.(name_proc(1:4).eq.'4JET') ))then
         call p_to_common(5,pa,pb,pevt5alln(1:4,1:5,ievt))
         call fillS_kin(7)
      elseif((ipset.eq.7).and.(   (name_proc(1:4).ne.'5JET')
     .                        .and.(name_proc(1:4).ne.'4JET') ))then
         if(iregion.eq.1)then
            call p_to_common(5,pa,pb,pevt5an(1:4,1:5,ievt))
         elseif(iregion.eq.2)then
            call p_to_common(5,pa,pb,pevt5bn(1:4,1:5,ievt))
         else
            write(*,*) "Invalid wedge type for ipset=7:", iregion
            stop
         endif
         call fillS_kin(7)
      else
         write(*,*) "Invalid ipset: ",ipset
         stop
      endif


      call sijcuts(ipset,ipass)

       if(ipass.eq.1)then
         ifail=0
      else
         return
      endif

      if ( name_proc(1:6).eq.'EPEMJJ' .or. name_proc.eq.'EPEMINC' ) then
      !> boost to lab required for ep em
      !> pa is electron moving in +ve z direction
         etaepem=pa(4)/pb(4)
         call boost_to_lab_epem(ipset,etaepem)
         return
      endif

      if(imethod.eq.2)then
         call boost_to_lab(ipset,x1,x2)
      elseif(imethod.eq.1)then
         if(ix.eq.1)then
            call boost_to_lab(ipset,x1,x2)
         elseif(ix.eq.2)then
            call boost_to_lab(ipset,x1,x2*xreg2)
         elseif(ix.eq.3)then
            call boost_to_lab(ipset,x1*xreg1,x2)
         elseif(ix.eq.4)then
            call boost_to_lab(ipset,x1*xreg1,x2*xreg2)
         endif
      endif

      return
      end


*********************************************************
* Generates a phase space point and boosts into the lab
* frame for phasenew
*

      subroutine genphasenewDISG(ipset,ievt,ix,x1,x2,xreg1,xreg2,ifail)
      use DIS_mod
      use KinData_mod
      use Observables_mod
      use Selectors_mod, only: applyAll_sel
      use Channels_mod, only : currentSubContrib_chan
      implicit real*8(a-h,o-y)
      implicit complex*16(z)
      character*128 slogname
      common /pin/pa(4),pb(4)
!$omp threadprivate(/pin/)
      common /eventmom2p/pevt2(4,2,2)
!$omp threadprivate(/eventmom2p/)
      common /eventmom3p/pevt3(4,3,12)
!$omp threadprivate(/eventmom3p/)
      common/eventmom4ap/pevt4a(4,4,96)
!$omp threadprivate(/eventmom4ap/)
      common/eventmom4bp/pevt4b(4,4,24)
!$omp threadprivate(/eventmom4bp/)
      common/eventinv2p/sevt2(2,2,2),tevt2(2,2,2),ievt2(2)
!$omp threadprivate(/eventinv2p/)
      common/eventinv3p/sevt3(3,3,12),tevt3(2,3,12),ievt3(12)
!$omp threadprivate(/eventinv3p/)
      common/eventinv4ap/sevt4a(4,4,96),tevt4a(2,4,96),ievt4a(96)
!$omp threadprivate(/eventinv4ap/)
      common/eventinv4bp/sevt4b(4,4,24),tevt4b(2,4,24),ievt4b(24)
!$omp threadprivate(/eventinv4bp/)
      common /method/imethod
      common /log/slogname
      !> region info
      integer :: iregion

      call currentSubContrib_chan(1, region=iregion)

      if((imethod.eq.1).and.(ipset.eq.4))then
         write(*,*) "imethod=1 is not supported for phase spaces with"
         write(*,*) "2 partons"
         stop
      endif

      ifail=1
      ipass=0

*     check to see if the phase space fell outside the wedge
      if(ipset.eq.5)then
         if(ievt3(ievt).eq.0)then
            return
         endif
      elseif(ipset.eq.6)then
         if((iregion.eq.1).and.(ievt4a(ievt).eq.0))then
            return
         elseif((iregion.eq.2).and.(ievt4b(ievt).eq.0))then
            return
         endif
      endif

      if(ipset.eq.4)then
         call p_to_common(2,pa,pb,pevt2(1:4,1:2,ievt))
         call fillS_kin(4)
      elseif(ipset.eq.5)then
         call p_to_common(3,pa,pb,pevt3(1:4,1:3,ievt))
         call fillS_kin(5)
      elseif(ipset.eq.6)then
         if(iregion.eq.1)then
            call p_to_common(4,pa,pb,pevt4a(1:4,1:4,ievt))
         elseif(iregion.eq.2)then
            call p_to_common(4,pa,pb,pevt4b(1:4,1:4,ievt))
         else
            write(*,*) "Invalid wedge type for ipset=6:", iregion
            stop
         endif
         call fillS_kin(6)
      else
         write(*,*) "Invalid ipset: ",ipset
         stop
      endif

      call sijcuts(ipset,ipass)

       if(ipass.eq.1)then
         ifail=0
      else
         return
      endif

      value_y_dis = x2

      if(imethod.eq.2)then
         call boost_to_hera_dis(ipset,x1/x2)
      elseif(imethod.eq.1)then
         if(ix.eq.1)then
            call boost_to_hera_dis(ipset,x1/x2)
         elseif(ix.eq.2)then
            call boost_to_hera_dis(ipset,x1/(x2*xreg2))
         elseif(ix.eq.3)then
            call boost_to_hera_dis(ipset,x1*xreg1/x2)
         elseif(ix.eq.4)then
            call boost_to_hera_dis(ipset,x1*xreg1/(x2*xreg2))
         endif
      endif

      return
      end



*********************************************************
* Generates a phase space point for phasenewee.f

      subroutine genphaseepem(ipset,ievt,ix,x1,x2,xreg1,xreg2,ifail)
      use KinData_mod
      use Channels_mod, only : currentSubContrib_chan
      implicit real*8(a-h,o-y)
      implicit complex*16(z)
      common /pinee/pa(4),pb(4)
      common/eventmom3pee/pevt3(4,3,1)
!$omp threadprivate(/eventmom3pee/)
      common/eventmom4pee/pevt4(4,4,12)
!$omp threadprivate(/eventmom4pee/)
      common/eventmom5apee/pevt5a(4,5,120)
!$omp threadprivate(/eventmom5apee/)
      common/eventmom5bpee/pevt5b(4,5,60)
!$omp threadprivate(/eventmom5bpee/)
      common/eventmom5pee/pevt5(4,5,20)
!$omp threadprivate(/eventmom5pee/)
      common/eventinv3pee/sevt3(3,3,1),ievt3(1)
!$omp threadprivate(/eventinv3pee/)
      common/eventinv4pee/sevt4(4,4,12),ievt4(12)
!$omp threadprivate(/eventinv4pee/)
      common/eventinv5apee/sevt5a(5,5,120),ievt5a(120)
!$omp threadprivate(/eventinv5apee/)
      common/eventinv5bpee/sevt5b(5,5,60),ievt5b(60)
!$omp threadprivate(/eventinv5bpee/)
      common/eventinv5pee/sevt5(5,5,20),ievt5(20)
!$omp threadprivate(/eventinv5pee/)
      !> region info
      integer :: iregion

      call currentSubContrib_chan(1, region=iregion)

      ifail=1
      ipass=0


      !> set pa to incoming electron pb to incoming positron

*     check to see if the phase space fell outside the wedge
      if(ipset.eq.5)then
         if(ievt3(ievt).eq.0)then
            return
         endif
      elseif(ipset.eq.6)then
         if(ievt4(ievt).eq.0)then
            return
         endif
      elseif(ipset.eq.7)then
         if (iregion.eq.1)then
            if (ievt5a(ievt).eq.0) return
         elseif (iregion.eq.2)then
            if (ievt5b(ievt).eq.0) return
         endif
      endif

      if(ipset.eq.5)then
         call p_to_common(3,pa,pb,pevt3(1:4,1:3,ievt))
         call fillS_kin(5)
      elseif(ipset.eq.6)then
         call p_to_common(4,pa,pb,pevt4(1:4,1:4,ievt))
         call fillS_kin(6)
      elseif(ipset.eq.7)then
         if(iregion.eq.1)then
            call p_to_common(5,pa,pb,pevt5a(1:4,1:5,ievt))
         elseif(iregion.eq.2)then
            call p_to_common(5,pa,pb,pevt5b(1:4,1:5,ievt))
         else
            write(*,*) "Invalid wedge type for ipset=7: ", iregion
            stop
         endif
         call fillS_kin(7)
      else
         write(*,*) "Invalid ipset: ",ipset
         stop
      endif

      call sijcuts(ipset,ipass)

      if(ipass.eq.1)then
         ifail=0
      else
         return
      endif

      !> boost to lab required for ep em
      !> pa is electron moving in +ve z direction
      etaepem=pa(4)/pb(4)
      call boost_to_lab_epem(ipset,etaepem)
      return
      end



*********************************************************
* Generates a phase space point and boosts into the lab
* frame for phasenewh
*

      subroutine genphasenewh(ipset,ievt,ix,x1,x2,xreg1,xreg2,ifail)
      use KinData_mod
      use Observables_mod
      use Process_mod
      use Channels_mod, only : currentSubContrib_chan
      implicit real*8(a-h,o-y)
      implicit complex*16(z)
      character*128 slogname
      common /pin/pa(4),pb(4)
!$omp threadprivate(/pin/)
      common /eventmom2p/pevt2(4,2,2)
!$omp threadprivate(/eventmom2p/)
      common /eventmom3p/pevt3(4,3,12)
!$omp threadprivate(/eventmom3p/)
      common /eventmom4p/pevt4(4,4,6)
!$omp threadprivate(/eventmom4p/)
      common /eventmom5ap/pevt5a(4,5,48)
!$omp threadprivate(/eventmom5ap/)
      common /eventmom5bp/pevt5b(4,5,24)
!$omp threadprivate(/eventmom5bp/)
      common/eventmom5allp/pevt5all(4,5,48)
!$omp threadprivate(/eventmom5allp/)
      common /eventinv4p/sevt4(4,4,6),tevt4(2,4,6),ievt4(6)
!$omp threadprivate(/eventinv4p/)
      common /eventinv5ap/sevt5a(5,5,48),tevt5a(2,5,48),ievt5a(48)
!$omp threadprivate(/eventinv5ap/)
      common /eventinv5bp/sevt5b(5,5,24),tevt5b(2,5,24),ievt5b(24)
!$omp threadprivate(/eventinv5bp/)
      common /eventinv5allp/sevt5all(5,5,48),tevt5all(2,5,48),ievt5all(48)
!$omp threadprivate(/eventinv5allp/)
      common /method/imethod
      common /log/slogname
      common /vdata/xvegas(20)
!$omp threadprivate(/vdata/)
      !> region info
      integer :: iregion

      call currentSubContrib_chan(1, region=iregion)

      if((imethod.eq.1).and.(ipset.eq.4))then
         write(*,*) "imethod=1 is not supported for phase spaces with"
         write(*,*) "2 partons"
         stop
      endif

      ifail=1
      ipass=0

*     check to see if the phase space fell outside the wedge
      if(ipset.eq.6)then
         if(ievt4(ievt).eq.0)then
            return
         endif
      elseif((ipset.eq.7).and.(name_proc(1:3).ne.'HJJ')
     .                   .and.(name_proc(1:7).ne.'HTO2PJJ')
     .                   .and.(name_proc(1:9).ne.'HTO2TAUJJ')
     .                   .and.(name_proc(1:4).ne.'WPJJ')
     .                   .and.(name_proc(1:4).ne.'WMJJ')
     .                   .and.(name_proc(1:4).ne.'GGJJ')
     .                   .and.(name_proc(1:3).ne.'ZJJ'))then
         if((iregion.eq.1).and.(ievt5a(ievt).eq.0))then
            return
         elseif((iregion.eq.2).and.(ievt5b(ievt).eq.0))then
            return
         endif
      elseif((ipset.eq.7).and.((name_proc(1:3).eq.'HJJ')
     .                     .or.(name_proc(1:7).eq.'HTO2PJJ')
     .                     .or.(name_proc(1:9).eq.'HTO2TAUJJ')
     .                     .or.(name_proc(1:4).eq.'WPJJ')
     .                     .or.(name_proc(1:4).eq.'WMJJ')
     .                     .or.(name_proc(1:4).eq.'GGJJ')
     .                     .or.(name_proc(1:3).eq.'ZJJ')
     .                        ))then
         if(ievt5all(ievt).eq.0)then
            return
         endif
      endif

      if(ipset.eq.4)then
         call p_to_common(2,pa,pb,pevt2(1:4,1:2,ievt))
         call fillS_kin(4)
      elseif(ipset.eq.5)then
         call p_to_common(3,pa,pb,pevt3(1:4,1:3,ievt))
         call fillS_kin(5)
      elseif(ipset.eq.6)then
         call p_to_common(4,pa,pb,pevt4(1:4,1:4,ievt))
         call fillS_kin(6)
      elseif((ipset.eq.7).and.(name_proc(1:3).ne.'HJJ')
     .                   .and.(name_proc(1:7).ne.'HTO2PJJ')
     .                   .and.(name_proc(1:9).ne.'HTO2TAUJJ')
     .                   .and.(name_proc(1:4).ne.'WPJJ')
     .                   .and.(name_proc(1:4).ne.'WMJJ')
     .                   .and.(name_proc(1:4).ne.'GGJJ')
     .                   .and.(name_proc(1:3).ne.'ZJJ'))then
         if(iregion.eq.1)then
            call p_to_common(5,pa,pb,pevt5a(1:4,1:5,ievt))
         elseif(iregion.eq.2)then
            call p_to_common(5,pa,pb,pevt5b(1:4,1:5,ievt))
         else
            write(*,*) "Invalid wedge type for ipset=7: ", iregion
            stop
         endif
         call fillS_kin(7)
      elseif((ipset.eq.7).and.((name_proc(1:3).eq.'HJJ')
     .                     .or.(name_proc(1:7).eq.'HTO2PJJ')
     .                     .or.(name_proc(1:9).eq.'HTO2TAUJJ')
     .                     .or.(name_proc(1:4).eq.'WPJJ')
     .                     .or.(name_proc(1:4).eq.'WMJJ')
     .                     .or.(name_proc(1:4).eq.'GGJJ')
     .                     .or.(name_proc(1:3).eq.'ZJJ')
     .                        ))then
         call p_to_common(5,pa,pb,pevt5all(1:4,1:5,ievt))
         call fillS_kin(7)
      else
         write(*,*) "Invalid ipset: ",ipset
         stop
      endif

      call sijcuts(ipset,ipass)

      if(ipass.eq.1)then
         ifail=0
      else
         return
      endif

      if(imethod.eq.2)then
         call boost_to_lab(ipset,x1,x2)
      elseif(imethod.eq.1)then
         if(ix.eq.1)then
            call boost_to_lab(ipset,x1,x2)
         elseif(ix.eq.2)then
            call boost_to_lab(ipset,x1,x2*xreg2)
         elseif(ix.eq.3)then
            call boost_to_lab(ipset,x1*xreg1,x2)
         elseif(ix.eq.4)then
            call boost_to_lab(ipset,x1*xreg1,x2*xreg2)
         endif
      endif

      return
      end


*********************************************************
* Generates a phase space point and boosts into the lab
* frame for phasenewhto3
*

      subroutine genphasenewhto3(ipset,ievt,ix,x1,x2,xreg1,xreg2,ifail)
      use Process_mod, only : name_proc
      use KinData_mod
      use Channels_mod, only : currentSubContrib_chan
      implicit real*8(a-h,o-y)
      implicit complex*16(z)
      character*128 slogname
      common /pin/pa(4),pb(4)
!$omp threadprivate(/pin/)
      common /eventmom3p/pevt3(4,3,12)
!$omp threadprivate(/eventmom3p/)
      common /eventmom4p/pevt4(4,4,6)
!$omp threadprivate(/eventmom4p/)
      common /eventinv4p/sevt4(4,4,6),tevt4(2,4,6),ievt4(6)
!$omp threadprivate(/eventinv4p/)
      common/eventmom5p/pevt5(4,5,4)
!$omp threadprivate(/eventmom5p/)
      common/eventinv5p/sevt5(5,5,4),tevt5(2,5,4),ievt5(4)
!$omp threadprivate(/eventinv5p/)
      common/eventmom6ap/pevt6a(4,6,24)
!$omp threadprivate(/eventmom6ap/)
      common/eventinv6ap/sevt6a(6,6,24),tevt6a(2,6,24),ievt6a(24)
!$omp threadprivate(/eventinv6ap/)
      common/eventmom6bp/pevt6b(4,6,24)
!$omp threadprivate(/eventmom6bp/)
      common/eventinv6bp/sevt6b(6,6,24),tevt6b(2,6,24),ievt6b(24)
!$omp threadprivate(/eventinv6bp/)
      common/eventmom6allp/pevt6all(4,6,48)
!$omp threadprivate(/eventmom6allp/)
      common/eventinv6allp/sevt6all(6,6,48),tevt6all(2,6,48),ievt6all(48)
!$omp threadprivate(/eventinv6allp/)
      common /method/imethod
      common /log/slogname
      !> region info
      integer :: iregion

      call currentSubContrib_chan(1, region=iregion)

      if((imethod.eq.1).and.(ipset.eq.4))then
         write(*,*) "imethod=1 is not supported for phase spaces with"
         write(*,*) "2 partons"
         stop
      endif

      ifail=1
      ipass=0

*     check to see if the phase space fell outside the wedge
      if(ipset.eq.7)then
         if(ievt5(ievt).eq.0)then
            return
         endif
      elseif((ipset.eq.8).and.(name_proc(8:9).ne.'JJ'))then
         if((iregion.eq.1).and.(ievt6a(ievt).eq.0))then
            return
         elseif((iregion.eq.2).and.(ievt6b(ievt).eq.0))then
            return
         endif
      elseif((ipset.eq.8).and.(name_proc(8:9).eq.'JJ'))then
         if(ievt6all(ievt).eq.0)then
            return
         endif
      endif

      if(ipset.eq.5)then
         call p_to_common3(3,pa,pb,pevt3(1:4,1:3,ievt))
         call fillS_kin(5)
      elseif(ipset.eq.6)then
         call p_to_common3(4,pa,pb,pevt4(1:4,1:4,ievt))
         call fillS_kin(6)
      elseif(ipset.eq.7)then
         call p_to_common3(5,pa,pb,pevt5(1:4,1:5,ievt))
         call fillS_kin(7)
      elseif((ipset.eq.8).and.(name_proc(8:9).ne.'JJ'))then
         if(iregion.eq.1)then
            call p_to_common3(6,pa,pb,pevt6a(1:4,1:6,ievt))
         elseif(iregion.eq.2)then
            call p_to_common3(6,pa,pb,pevt6b(1:4,1:6,ievt))
         else
            write(*,*) "Invalid wedge type for ipset=8: ", iregion
            stop
         endif
         call fillS_kin(8)
      elseif((ipset.eq.8).and.(name_proc(8:9).eq.'JJ'))then
         call p_to_common3(6,pa,pb,pevt6all(1:4,1:6,ievt))
         call fillS_kin(8)
      else
         write(*,*) "Invalid ipset: ",ipset
         stop
      endif

      call sijcuts(ipset,ipass)

      if(ipass.eq.1)then
         ifail=0
      else
         return
      endif

      if(imethod.eq.2)then
         call boost_to_lab(ipset,x1,x2)
      elseif(imethod.eq.1)then
         if(ix.eq.1)then
            call boost_to_lab(ipset,x1,x2)
         elseif(ix.eq.2)then
            call boost_to_lab(ipset,x1,x2*xreg2)
         elseif(ix.eq.3)then
            call boost_to_lab(ipset,x1*xreg1,x2)
         elseif(ix.eq.4)then
            call boost_to_lab(ipset,x1*xreg1,x2*xreg2)
         endif
      endif

      return
      end

*********************************************************
* Generates a phase space point and boosts into the lab
* frame for phasenewhto4
*

      subroutine genphasenewhto4(ipset,ievt,ix,x1,x2,xreg1,xreg2,ifail)
      use Process_mod, only : name_proc
      use KinData_mod
      use Channels_mod, only : currentSubContrib_chan
      implicit real*8(a-h,o-y)
      implicit complex*16(z)
      character*128 slogname
      common /pin/pa(4),pb(4)
!$omp threadprivate(/pin/)
      common /eventmom4p/pevt4(4,4,6)
!$omp threadprivate(/eventmom4p/)
      common /eventinv4p/sevt4(4,4,6),tevt4(2,4,6),ievt4(6)
!$omp threadprivate(/eventinv4p/)
      common/eventmom5p/pevt5(4,5,4)
!$omp threadprivate(/eventmom5p/)
      common/eventinv5p/sevt5(5,5,4),tevt5(2,5,4),ievt5(4)
!$omp threadprivate(/eventinv5p/)
      common/eventmom6p/pevt6(4,6,12)
!$omp threadprivate(/eventmom6p/)
      common/eventinv6p/sevt6(6,6,12),tevt6(2,6,12),ievt6(12)
!$omp threadprivate(/eventinv6p/)
      common/eventmom7ap/pevt7a(4,7,24)
!$omp threadprivate(/eventmom7ap/)
      common/eventinv7ap/sevt7a(7,7,24),tevt7a(2,7,24),ievt7a(24)
!$omp threadprivate(/eventinv7ap/)
      common/eventmom7bp/pevt7b(4,7,24)
!$omp threadprivate(/eventmom7bp/)
      common/eventinv7bp/sevt7b(7,7,24),tevt7b(2,7,24),ievt7b(24)
!$omp threadprivate(/eventinv7bp/)
      common/eventmom7allp/pevt7all(4,7,48)
!$omp threadprivate(/eventmom7allp/)
      common /eventinv7allp/sevt7all(7,7,48),tevt7all(2,7,48),ievt7all(48)
!$omp threadprivate(/eventinv7allp/)
      common /method/imethod
      common /log/slogname
      !> region info
      integer :: iregion

      call currentSubContrib_chan(1, region=iregion)

      if((imethod.eq.1).and.(ipset.eq.4))then
         write(*,*) "imethod=1 is not supported for phase spaces with"
         write(*,*) "2 partons"
         stop
      endif

      ifail=1
      ipass=0

*     check to see if the phase space fell outside the wedge
      if(ipset.eq.8)then
         if(ievt6(ievt).eq.0)then
            return
         endif
      elseif((ipset.eq.9).and.
     .       (name_proc(6:7).ne.'JJ').and.
     .       (name_proc(3:4).ne.'JJ').and.
     .       (name_proc(8:9).ne.'JJ').and.
     .       (name_proc(9:10).ne.'JJ'))then
         if((iregion.eq.1).and.(ievt7a(ievt).eq.0))then
            return
         elseif((iregion.eq.2).and.(ievt7b(ievt).eq.0))then
            return
         endif
      elseif((ipset.eq.9).and.
     .       ((name_proc(6:7).eq.'JJ').or.
     .        (name_proc(3:4).eq.'JJ').or.
     .        (name_proc(8:9).eq.'JJ').or.
     .        (name_proc(9:10).eq.'JJ')))then
         if(ievt7all(ievt).eq.0)then
            return
         endif
      endif



      if(ipset.eq.6)then
         call p_to_common4(4,pa,pb,pevt4(1:4,1:4,ievt))
         call fillS_kin(6)
      elseif(ipset.eq.7)then
         call p_to_common4(5,pa,pb,pevt5(1:4,1:5,ievt))
         call fillS_kin(7)
      elseif(ipset.eq.8)then
         call p_to_common4(6,pa,pb,pevt6(1:4,1:6,ievt))
         call fillS_kin(8)
      elseif((ipset.eq.9).and.
     .       (name_proc(6:7).ne.'JJ').and.
     .       (name_proc(3:4).ne.'JJ').and.
     .       (name_proc(8:9).ne.'JJ').and.
     .       (name_proc(9:10).ne.'JJ'))then
         if(iregion.eq.1)then
            call p_to_common4(7,pa,pb,pevt7a(1:4,1:7,ievt))
         elseif(iregion.eq.2)then
            call p_to_common4(7,pa,pb,pevt7b(1:4,1:7,ievt))
         else
            write(*,*) "Invalid wedge type for ipset=7: ", iregion
            stop
         endif
         call fillS_kin(9)
      elseif((ipset.eq.9).and.
     .       ((name_proc(6:7).eq.'JJ').or.
     .        (name_proc(3:4).eq.'JJ').or.   
     .        (name_proc(8:9).eq.'JJ').or.
     .        (name_proc(9:10).eq.'JJ')))then
         call p_to_common4(7,pa,pb,pevt7all(1:4,1:7,ievt))
         call fillS_kin(9)
      else
         write(*,*) "Invalid ipset: ",ipset
         stop
      endif

      call sijcuts(ipset,ipass)

      if(ipass.eq.1)then
         ifail=0
      else
         return
      endif

      if(imethod.eq.2)then
         call boost_to_lab(ipset,x1,x2)
      elseif(imethod.eq.1)then
         if(ix.eq.1)then
            call boost_to_lab(ipset,x1,x2)
         elseif(ix.eq.2)then
            call boost_to_lab(ipset,x1,x2*xreg2)
         elseif(ix.eq.3)then
            call boost_to_lab(ipset,x1*xreg1,x2)
         elseif(ix.eq.4)then
            call boost_to_lab(ipset,x1*xreg1,x2*xreg2)
         endif
      endif

      return
      end

*********************************************************
* Generates a phase space point and boosts into the lab
* frame for phasenewJ
*
      subroutine genphasenewJ(ipset,ievt,ix,x1,x2,xreg1,xreg2,ifail)
      use Process_mod, only : name_proc
      use KinData_mod
      use PhaseNJ_mod, only : fillKinVFH
      implicit real*8(a-h,o-y)
      implicit complex*16(z)
      character*128 slogname
      integer j
      common/eventinvNbp/pperm(4,10,200)
!$omp threadprivate(/eventinvNbp/)
      common /method/imethod
      common /log/slogname

      if((imethod.eq.1).and.(ipset.eq.4))then
         write(*,*) "imethod=1 is not supported for phase spaces with"
         write(*,*) "2 partons"
         stop
      endif

      ifail = 1
      ipass = 1

      call fillKinVFH(ievt, ipset, ipass)
      if (ipass.eq.0) return

      call sijcuts(ipset,ipass)

      if(ipass.eq.1)then
         ifail=0
      else
         return
      endif

      if(imethod.eq.2)then
         call boost_to_lab(ipset,x1,x2)
      elseif(imethod.eq.1)then
         if(ix.eq.1)then
            call boost_to_lab(ipset,x1,x2)
         elseif(ix.eq.2)then
            call boost_to_lab(ipset,x1,x2*xreg2)
         elseif(ix.eq.3)then
            call boost_to_lab(ipset,x1*xreg1,x2)
         elseif(ix.eq.4)then
            call boost_to_lab(ipset,x1*xreg1,x2*xreg2)
         endif
      endif

      if( (name_proc.eq.'VFH') .or.
     .    (name_proc.eq.'VFHTO2P') .or.
     .    (name_proc.eq.'VFHTO2TAU') )
     .  call clearCache_vbf(ipset)

      if(name_proc.eq.'VFHTO2L1P') call clearCache_vbf3(ipset)

      if( (name_proc.eq.'VFHTO4E') .or.
     .    (name_proc.eq.'VFHTO2E2MU') .or.
     .    (name_proc.eq.'VFHTO2L2N'))
     .  call clearCache_vbf4(ipset)

      return
      end


*********************************************************
* Generates a phase space point and boosts into the lab
* frame
*

      subroutine genphasedis(ipset,ievt,ix,x1,x2,xreg1,xreg2,ifail,boostBreit)
      use Process_mod, only : name_proc, s_proc
      use DIS_mod, only : use_EPRC
      use KinData_mod
      use Observables_mod
      use Selectors_mod, only: applyAll_sel
      use DIS_mod
      use Constants_mod, only : pi
      use Parameters_mod, only : mproton_param, set_value_param
      use Proj2Born_mod
      use Channels_mod, only : currentSubContrib_chan
      implicit real*8(a-h,o-y)
      implicit complex*16(z)
      logical boostBreit
      dimension q(1:4),p(1:4),e(1:4),paux(1:4)
      common /eweakZ/ amzZ,zewfac(4),zewnull(4)
!$omp threadprivate(/eweakZ/)
      common /pin/pa(4),pb(4)
!$omp threadprivate(/pin/)
      common /eventmom2p/pevt2(4,2,2)
!$omp threadprivate(/eventmom2p/)
      common /eventmom3p/pevt3(4,3,12)
!$omp threadprivate(/eventmom3p/)
      common /eventmom4p/pevt4(4,4,6)
!$omp threadprivate(/eventmom4p/)
      common /eventmom5ap/pevt5a(4,5,48)
!$omp threadprivate(/eventmom5ap/)
      common /eventmom5bp/pevt5b(4,5,24)
!$omp threadprivate(/eventmom5bp/)
      common/eventinv2p/sevt2(2,2,2),tevt2(2,2,2),ievt2(2)
!$omp threadprivate(/eventinv2p/)
      common/eventinv3p/sevt3(3,3,12),tevt3(2,3,12),ievt3(12)
!$omp threadprivate(/eventinv3p/)
      common/eventinv4p/sevt4(4,4,6),tevt4(2,4,6),ievt4(6)
!$omp threadprivate(/eventinv4p/)
      common/eventinv5ap/sevt5a(5,5,48),tevt5a(2,5,48),ievt5a(48)
!$omp threadprivate(/eventinv5ap/)
      common/eventinv5bp/sevt5b(5,5,24),tevt5b(2,5,24),ievt5b(24)
!$omp threadprivate(/eventinv5bp/)
      common /method/imethod
      common /vdata/xvegas(20)
!$omp threadprivate(/vdata/)
      !> region info
      integer :: iregion

      call currentSubContrib_chan(1, region=iregion)

      if (imethod.eq.1 .and. (ix.ne.1 .or. ix.ne.3) ) then
         write(*,*) "invalid ix in DIS imethod=1, 1 and 3 only",ix
         stop
      endif

      ifail=1
      ipass=0
      if(ipset.eq.4)then
         if(ievt2(ievt).eq.0) return
         call p_to_common(2,pa,pb,pevt2(1:4,1:2,ievt))
         call fillS_kin(4)
      elseif(ipset.eq.5)then
         if(ievt3(ievt).eq.0) return
         call p_to_common(3,pa,pb,pevt3(1:4,1:3,ievt))
         call fillS_kin(5)
      elseif(ipset.eq.6)then
         if(ievt4(ievt).eq.0) return
         call p_to_common(4,pa,pb,pevt4(1:4,1:4,ievt))
         call fillS_kin(6)
      elseif(ipset.eq.7)then
         if(iregion.eq.1)then
            if(ievt5a(ievt).eq.0) return
            call p_to_common(5,pa,pb,pevt5a(1:4,1:5,ievt))
         elseif(iregion.eq.2)then
            if(ievt5b(ievt).eq.0) return
            call p_to_common(5,pa,pb,pevt5b(1:4,1:5,ievt))
         else
            write(*,*) "Invalid wedge type for ipset=7: ", iregion
            stop
         endif
         call fillS_kin(7)
      else
         write(*,*) "Invalid ipset: ",ipset
         stop
      endif

      if(name_proc(1:4).eq.'UNIT')then ! no further checks required
         ifail=0
         return
      endif

      call sijcuts(ipset,ipass)

      if(ipass.eq.1)then
         ifail=0
      else
         return
      endif

      xx1 =x1
      if(imethod.eq.1 .and. ix.eq.3) then
            xx1 = x1*xreg1
      endif
      !> obtaining relevant variables for breit -> proton gamma transformation
      call setxx1(xx1)
      !> either boost to fixed target or to HERA
      if(fixT) then
         call boost_to_gamnuc(ipset)
      else
         call boost_to_hera_dis(ipset,xx1)
      endif

      !>>>> compute global observables for DIS:
      !>     id_DIS_Q2
      !>     id_DIS_x
      !>     id_DIS_y
      !> define four-momenta
      q(:) = kin(ipset)%p(:,2) - kin(ipset)%p(:,3)
      p(:) = kin(ipset)%p(:,1) / xx1
      e(:) = kin(ipset)%p(:,2)
      !> compute global observables and register
      q2  = -dot(q,q)
      xbj = q2/2d0/dot(q,p)
      y   = q2/s_proc/xbj
      paux(:)=q(:)+p(:)
      w2  = dot(paux,paux)
      !> assuming a massless electron
      thetae = acos(kin(ipset)%p(3,3)/kin(ipset)%p(4,3))
      rnu  = (w2+q2)/2d0/mproton_param()
      cosgammah=((1d0-y)*xbj*p(4)-y*e(4))/((1d0-y)*xbj*p(4)+y*e(4))
      value_q2_dis = q2
      !> Needed for selection of muR
      value_q_dis = sqrt(q2)
      value_Ee_dis = kin(ipset)%p(4,3)
      value_thetae_dis = thetae
      value_x_dis =  xbj
      value_y_dis =  y
      value_W2_dis =  w2
      value_W_dis =  sqrt(w2)
      value_cosgammah_dis =  cosgammah
      value_nu_dis =  rnu

      if(use_EPRC()) then
!$omp critical(aem)
         amzZ=aemrun(q2)
         call set_value_param('alpha', amzZ)
!$omp end critical(aem)
      endif

      if(ipass.eq.0) then
         !> property of global selectors:
         !> if they do not pass selectors, the *whole* event
         !> (including counter-events) are rejected
         !> ===> ifail = 1
         ifail = 1
         return
      end if

      if(boostBreit) then
         call boost_to_breit_dis(ipset,ifail)
         if(boostPGAM) then
            call eval_trans_Breit_to_gamma_proton_cm(ipset)
         endif
      endif

      !> Projection-To-Born
C       print*, "   >>>   Debug P2B   <<<"
C       print*, "      * before:", ipset
C C       call print_momenta(ipset)
C       call evalAll_obs(ipset)
C       call printAll_obs(ipset)
      if (isActive_p2b()) then
         call project_dis_p2b(ipset,ipass)
         if (ipass == 0) ifail = 1
C          print*, "      * P2B:", npar_p2b, "from", ipset
C C          call print_momenta(npar_p2b)
C          call evalAll_obs(npar_p2b)
C          call printAll_obs(npar_p2b)
C          read(*,*)
C       else
C          call print_momenta(ipset)
C          call evalAll_obs(ipset)
C          call printAll_obs(ipset)
      end if

      return
      end


      subroutine p_to_common(n,pa,pb,pevt)
      use Process_mod, only : nPDFs_proc
      use KinData_mod
      implicit none
      double precision pa,pb,pevt
      integer i,j,n
      dimension pa(4),pb(4),pevt(1:4,1:n)


      !> for DIS the final-state lepton should stay in momentum slot 3!
      select case(nPDFs_proc)
      !> hadrons
      !> for the other processes we want the leptons in the *last*
      !> two entries of the momentum array (pahsenewh generates them
      !> in positions 1&2)
      case(2)
         if (n.eq.2) then
         do i=1,4
            p4(i,1) = pa(i)
            p4(i,2) = pb(i)
            do j=1,n
               p4(i,j+2) = pevt(i,j)
            enddo
         enddo
         elseif (n.eq.3) then
         do i=1,4
            p5(i,1) = pa(i)
            p5(i,2) = pb(i)
            p5(i,3) = pevt(i,3)
            do j=1,2
               p5(i,j+3) = pevt(i,j)
            enddo
         enddo
         elseif (n.eq.4) then
         do i=1,4
            p6(i,1) = pa(i)
            p6(i,2) = pb(i)
            p6(i,3) = pevt(i,3)
            p6(i,4) = pevt(i,4)
            do j=1,2
               p6(i,j+4) = pevt(i,j)
            enddo
         enddo
         elseif (n.eq.5) then
         do i=1,4
            p7(i,1) = pa(i)
            p7(i,2) = pb(i)
            p7(i,3) = pevt(i,3)
            p7(i,4) = pevt(i,4)
            p7(i,5) = pevt(i,5)
            do j=1,2
               p7(i,j+5) = pevt(i,j)
            enddo
         enddo
         endif

      !> DIS and epem
      case(1,0)
         if (n.eq.2) then
         do i=1,4
            p4(i,1) = pa(i)
            p4(i,2) = pb(i)
            do j=1,n
               p4(i,j+2) = pevt(i,j)
            enddo
         enddo
         elseif (n.eq.3) then
         do i=1,4
            p5(i,1) = pa(i)
            p5(i,2) = pb(i)
            do j=1,n
               p5(i,j+2) = pevt(i,j)
            enddo
         enddo
         elseif (n.eq.4) then
         do i=1,4
            p6(i,1) = pa(i)
            p6(i,2) = pb(i)
            do j=1,n
               p6(i,j+2) = pevt(i,j)
            enddo
         enddo
         elseif (n.eq.5) then
         do i=1,4
            p7(i,1) = pa(i)
            p7(i,2) = pb(i)
            do j=1,n
               p7(i,j+2) = pevt(i,j)
            enddo
         enddo
         endif
      end select

      return
      end

      subroutine p_to_common3(n,pa,pb,pevt)
      use KinData_mod
      implicit none
      double precision pa,pb,pevt
      integer i,j,n
      dimension pa(4),pb(4),pevt(1:4,1:n)

         if (n.eq.3) then
         do i=1,4
            p5(i,1) = pa(i)
            p5(i,2) = pb(i)
            do j=1,n
               p5(i,j+2) = pevt(i,j)
            enddo
         enddo
         elseif (n.eq.4) then
         do i=1,4
            p6(i,1) = pa(i)
            p6(i,2) = pb(i)
            p6(i,3) = pevt(i,4)
            do j=1,3
               p6(i,j+3) = pevt(i,j)
            enddo
         enddo

         elseif (n.eq.5) then
         do i=1,4
            p7(i,1) = pa(i)
            p7(i,2) = pb(i)
            p7(i,3) = pevt(i,4)
            p7(i,4) = pevt(i,5)
            do j=1,3
               p7(i,j+4) = pevt(i,j)
            enddo
         enddo
         elseif (n.eq.6) then
         do i=1,4
            p8(i,1) = pa(i)
            p8(i,2) = pb(i)
            p8(i,3) = pevt(i,4)
            p8(i,4) = pevt(i,5)
            p8(i,5) = pevt(i,6)
            do j=1,3
               p8(i,j+5) = pevt(i,j)
            enddo
         enddo
         endif

      return
      end

      subroutine p_to_common4(n,pa,pb,pevt)
      use KinData_mod
      implicit none
      double precision pa,pb,pevt
      integer i,j,n
      dimension pa(4),pb(4),pevt(1:4,1:n)

         if (n.eq.4) then
         do i=1,4
            p6(i,1) = pa(i)
            p6(i,2) = pb(i)
            do j=1,n
               p6(i,j+2) = pevt(i,j)
            enddo
         enddo
         elseif (n.eq.5) then
         do i=1,4
            p7(i,1) = pa(i)
            p7(i,2) = pb(i)
            p7(i,3) = pevt(i,5)
            do j=1,4
               p7(i,j+3) = pevt(i,j)
            enddo
         enddo

         elseif (n.eq.6) then
         do i=1,4
            p8(i,1) = pa(i)
            p8(i,2) = pb(i)
            p8(i,3) = pevt(i,5)
            p8(i,4) = pevt(i,6)
            do j=1,4
               p8(i,j+4) = pevt(i,j)
            enddo
         enddo
         elseif (n.eq.7) then
         do i=1,4
            p9(i,1) = pa(i)
            p9(i,2) = pb(i)
            p9(i,3) = pevt(i,5)
            p9(i,4) = pevt(i,6)
            p9(i,5) = pevt(i,7)
            do j=1,4
               p9(i,j+5) = pevt(i,j)
            enddo
         enddo
         endif

      return
      end


      subroutine boost_to_lab_epem(npar,etaepem)
      use KinData_mod
      integer npar
      double precision etaepem,eta,cth,sth
      double precision paux(3:4,3:npar)

      eta = -0.5d0*log(etaepem)
      cth = cosh(eta)
      sth = sinh(eta)
      !> boost in case of unequal ep, em energies
      !> initial state momenta are already set in frame
      do i=3,npar
         paux(3,i) = cth*kin(npar)%p(3,i)-sth*kin(npar)%p(4,i)
         paux(4,i) = cth*kin(npar)%p(4,i)-sth*kin(npar)%p(3,i)
         kin(npar)%p(3,i) = paux(3,i)
         kin(npar)%p(4,i) = paux(4,i)
      enddo
      return
      end


      subroutine boost_to_lab(n,x1,x2)
      use KinData_mod
      use Process_mod, only : beam1_proc, A_beam1, Z_beam1,
     &                        beam2_proc, A_beam2, Z_beam2
      integer n
      double precision x1,x2
      double precision eta,cth,sth
      double precision paux(1:4,1:10)

      if ( abs(beam1_proc) /= abs(beam2_proc) ) then
         eta = -0.5d0*log(x1/x2*dble(Z_beam1)*dble(A_beam2)/dble(Z_beam2)/dble(A_beam1))
      else
         eta = -0.5d0*log(x1/x2)
      end if
      cth = cosh(eta)
      sth = sinh(eta)

      !> @TODO: make this generic!!

      if (n.eq.4) then
         do i = 1,n
            paux(3,i) = cth*p4(3,i)-sth*p4(4,i)
            paux(4,i) = cth*p4(4,i)-sth*p4(3,i)
            p4(3,i) = paux(3,i)
            p4(4,i) = paux(4,i)
         enddo
      elseif (n.eq.5) then
         do i=1,n
            paux(3,i) = cth*p5(3,i)-sth*p5(4,i)
            paux(4,i) = cth*p5(4,i)-sth*p5(3,i)
            p5(3,i) = paux(3,i)
            p5(4,i) = paux(4,i)
         enddo
      elseif (n.eq.6) then
         do i=1,n
            paux(3,i) = cth*p6(3,i)-sth*p6(4,i)
            paux(4,i) = cth*p6(4,i)-sth*p6(3,i)
            p6(3,i) = paux(3,i)
            p6(4,i) = paux(4,i)
         enddo
      elseif (n.eq.7) then
         do i=1,n
            paux(3,i) = cth*p7(3,i)-sth*p7(4,i)
            paux(4,i) = cth*p7(4,i)-sth*p7(3,i)
            p7(3,i) = paux(3,i)
            p7(4,i) = paux(4,i)
         enddo
      elseif (n.eq.8) then
         do i=1,n
            paux(3,i) = cth*p8(3,i)-sth*p8(4,i)
            paux(4,i) = cth*p8(4,i)-sth*p8(3,i)
            p8(3,i) = paux(3,i)
            p8(4,i) = paux(4,i)
         enddo
      elseif (n.eq.9) then
         do i=1,n
            paux(3,i) = cth*p9(3,i)-sth*p9(4,i)
            paux(4,i) = cth*p9(4,i)-sth*p9(3,i)
            p9(3,i) = paux(3,i)
            p9(4,i) = paux(4,i)
         enddo
      elseif (n.eq.10) then
         do i=1,n
            paux(3,i) = cth*p10(3,i)-sth*p10(4,i)
            paux(4,i) = cth*p10(4,i)-sth*p10(3,i)
            p10(3,i) = paux(3,i)
            p10(4,i) = paux(4,i)
         enddo
      endif

      return
      end
