*
************************************************************************
*
      subroutine pick(itype, x, xmin, xmax, ran, wgt)
      use Constants_mod, only : pi
      implicit none
      integer, intent(in)             :: itype
      double precision, intent(out)   :: x
      double precision, intent(in)    :: xmin, xmax, ran
      double precision, intent(inout) :: wgt
      double precision :: y, delta, tansq, rran
      !implicit real*8(a-h,o-z)

      if (xmin >= xmax) then
         !print*, "pick: xmin >= xmax:", xmin, xmax
         wgt = 0d0
         return
      end if

      select case(itype)

      !> y(x) = x
      !> jac  ~ 1
      case(1)
        x = xmin + ran*(xmax-xmin)
        wgt = wgt * (xmax-xmin)

      !> y(x) = ln(x)
      !> jac  ~ x
      case(2)
        x = xmin * (xmax/xmin)**ran
        wgt = wgt * x * log(xmax/xmin)

      !> y(x) = 1/x
      !> jac  ~ x**2
      case(3)
        x = xmax / (1d0 + ran*(xmax/xmin-1d0))
        wgt = wgt * x**2 * (1d0/xmin-1d0/xmax)

      !> y(x) = x**2
      !> jac  ~ 1/x
      case(4)
         x = sqrt(xmin**2 + ran*(xmax**2-xmin**2))
         wgt = wgt * 1d0/(2d0*x) * (xmax**2-xmin**2)

      !> y(x) = sqrt( x - xmin )
      !> jac  ~ sqrt( x - xmin )
      case(10)
         delta = sqrt( xmax - xmin )
         y = ran * delta
         wgt = wgt * delta * 2d0 * y
         x = xmin + y**2

      !> y(x) = sqrt( xmax - x )
      !> jac  ~ sqrt( xmax - x )
      case(11)
         delta = sqrt( xmax - xmin )
         y = ran * delta
         wgt = wgt * delta * 2d0 * y
         x = xmax - y**2

      !> y(x) = arctan( sqrt( ( xmax - x )/( x - xmin ) ) )
      !> jac  ~ sqrt( ( xmax - x ) * ( x - xmin ) )
      case(12)
         y = ran * pi / 2d0
         wgt = wgt * pi / 2d0 * ( xmax - xmin ) * 2d0 * cos(y) * sin(y)
         tansq = tan(y)**2
         x = (xmax + xmin*tansq) / (1d0 + tansq)

      !> y(x) = ln(x)
      !> jac  ~ x
      !> & r -> r^2
      case(22)
        rran = ran*ran
        wgt = wgt * 2d0 * ran

        x = xmin * (xmax/xmin)**rran
        wgt = wgt * x * log(xmax/xmin)

      !> y(x) = ln(x)
      !> jac  ~ x
      !> & r -> r^1.5
      case(23)
        rran = ran**1.5d0
        wgt = wgt * 1.5d0 * sqrt(ran)

        x = xmin * (xmax/xmin)**rran
        wgt = wgt * x * log(xmax/xmin)

      !> error out for invalid itype
      case default
         print*, "pick: unknown itype", itype
         stop

      end select

      return
      end


*
************************************************************************
*
      subroutine pick_qT(itype,pT,qTcut,pTmax,r1,wt)
      implicit real*8(a-h,o-z)
      common/pmasses/rma2,rmb2,rm2(1:7),shat
!$omp threadprivate(/pmasses/)
      common/tcuts/ymin,y0

      rootshat=sqrt(shat)

      select case(itype)

c--XC--default setup sampling log(qT)
      case(0)
               call pick(2,pT,y0*rootshat,pTmax,r1,wt)
            return

c--XC--linear then tanh sampling of pT, dynamic r1cut
      case(1)
         r1cut = 1d0-qTcut/pTmax
         if (r1 < r1cut) then
            ! log
            rr1 = r1 / r1cut
            wt  = wt / r1cut
            call pick_tanh(pT,1d0,4d0,qTcut,pTmax,rr1,wt)
            return
         else
            ! linear
            rr1 = (r1-r1cut) / (1d0-r1cut)
            wt  = wt / (1d0-r1cut)
            call pick(1,pT,y0*rootshat,qTcut,rr1,wt)
            return
         end if


c--XC--depends on the pTmax switch the sampling
c--XC--of full pT region
      case(2)
            if (pTmax.le.10d0*qTcut) then
               call pick(1,pT,y0*rootshat,pTmax,r1,wt)
            else
               call pick(10,pT,y0*rootshat,pTmax,r1,wt)
            endif
            return

c--XC--sqrt then tanh/linear sampling of pT, fixed r1cut.
c--XC--If pTmax is not large compare to qTcut, we use linear sampling
c--XC--as any P.S. in soft region has large weight.
c--xC--When pTmax is much larger than qTcut, we need to intensively sample
c--XC--pT~qTcut for collinear IR divergence. We also need to intensively sample
c--XC--pT~pTmax where HpZ~0 --> yh~0 for the bulk of the total cross section.
      case(3)
         r1cut = 9d-1
         if (r1 < r1cut) then
            ! log
            rr1 = r1 / r1cut
            wt  = wt / r1cut
            if (pTmax.le.10d0*qTcut) then
               call pick(1,pT,qTcut,pTmax,rr1,wt)
            else
               call pick_tanh(pT,1d0,6d0,qTcut,pTmax,rr1,wt)
            endif
            return
         else
            ! sqrt
            rr1 = (r1-r1cut) / (1d0-r1cut)
            wt  = wt / (1d0-r1cut)
            call pick(11,pT,y0*rootshat,qTcut,rr1,wt)
            return
         endif

c--XC--similar idea as case(8) but use different
c--XC--sampling methods for collinear or large yH region
      case(4)
         r1cut = 9d-1
         r2cut = 4d-1
         softlimit = 10d0*qTcut
         collinearlimit = pTmax/3d0
         if (r1 < r1cut) then
             ! log
            rr1 = r1 / r1cut
            wt  = wt / r1cut
            if (pTmax.le.softlimit) then
               call pick(1,pT,qTcut,pTmax,rr1,wt)
            elseif (rr1 < r2cut) then
              rrr1 = rr1 / r2cut
              wt   = wt / r2cut
              call pick(11,pT,collinearlimit,pTmax,rrr1,wt)
            else
              rrr1 = (rr1-r2cut) / (1d0-r2cut)
              wt  = wt / (1d0-r2cut)
              call pick(2,pT,qTcut,collinearlimit,rrr1,wt)
            endif
            return
         else
            ! sqrt
            rr1 = (r1-r1cut) / (1d0-r1cut)
            wt  = wt / (1d0-r1cut)
            call pick(11,pT,y0*rootshat,qTcut,rr1,wt)
            return
         endif

c--XC-similar to case(3) but without soft region
      case(5)
         r1cut = 9d-1
         if (r1 < r1cut) then
            ! log
            rr1 = r1 / r1cut
            wt  = wt / r1cut
            call pick_tanh(pT,1d0,6d0,qTcut,pTmax,rr1,wt)
            return
         else
            ! sqrt
            rr1 = (r1-r1cut) / (1d0-r1cut)
            wt  = wt / (1d0-r1cut)
            call pick(11,pT,y0*rootshat,qTcut,rr1,wt)
            return
         endif

c--XC-similar to case(4) but without soft region
      case(6)
         r1cut = 9d-1
         r2cut = 4d-1
         collinearlimit = pTmax/3d0
         if (r1 < r1cut) then
             ! log
            rr1 = r1 / r1cut
            wt  = wt / r1cut
            if (rr1 < r2cut) then
              rrr1 = rr1 / r2cut
              wt   = wt / r2cut
              call pick(11,pT,collinearlimit,pTmax,rrr1,wt)
            else
              rrr1 = (rr1-r2cut) / (1d0-r2cut)
              wt  = wt / (1d0-r2cut)
              call pick(2,pT,qTcut,collinearlimit,rrr1,wt)
            endif
            return
         else
            ! sqrt
            rr1 = (r1-r1cut) / (1d0-r1cut)
            wt  = wt / (1d0-r1cut)
            call pick(11,pT,y0*rootshat,qTcut,rr1,wt)
            return
         endif


      end select

      return
      end

*
************************************************************************
*

      subroutine pick_tanh_generic(iType,x,slope,tanhmax,xmin,xmax,ran,wt)
      use Constants_mod, only : pi, twopi
      implicit real*8(a-h,o-z)

      if (xmin > xmax) then
         wt = 0d0
         return
      end if

      !> iType -- tanh sampling type
      !> -1 : lower half of tanh
      !>  0 : full tanh
      !>  1 : upper half of tanh
      !>  2 : two tanh joint to a W type of distribution
      select case(iType)

      case(-1)
         call pick_minushalftanh(x,slope,tanhmax,xmin,xmax,ran,wt)

      case(0)
         call pick_tanh(x,slope,tanhmax,xmin,xmax,ran,wt)

      case(1)
         call pick_plushalftanh(x,slope,tanhmax,xmin,xmax,ran,wt)

      case(2)
         call pick_wphi(x,slope,tanhmax,xmin,xmax,ran,wt)

      case default
         print*, "pick_tanh_generic: unknown type:", iType
         stop

      end select
      end


      subroutine pick_tanh(x,slope,tanhmax,xmin,xmax,ran,wt)
      implicit real*8(a-h,o-z)
      !> mapping: y(x) = arctanh(h(ran)) h(ran) = hmin, hmax
      !> we get a jacobian: (1-h)*(1+h)
      !> then normalise to tanh(hmax)-tanh(hmin) and apply to any max or min value
           hmax = tanhmax
           hmin =-hmax
           h = hmin + ran * (hmax - hmin)
           wt = wt * (hmax - hmin)

           rran = tanh(slope*h)/2d0/tanh(slope*hmax) + 1d0/2d0
           wt = wt*(1d0-tanh(slope*h))*(1d0+tanh(slope*h))
     .            /2d0/tanh(slope*hmax)*slope

           x = xmin + rran * (xmax - xmin)
           wt = wt * (xmax - xmin)
      return
      end


      subroutine pick_plushalftanh(x,slope,tanhmax,xmin,xmax,ran,wt)
      implicit real*8(a-h,o-z)
      !> mapping: y(x) = arctanh(h(ran)), h(ran) = [0, tanhmax]
      !> we get a jacobian: (1-h)*(1+h)
      !> then normalise to -tanh(hmin) and apply to any max or min value
           h = tanhmax * ran
           wt = wt * tanhmax

           rran = tanh(slope*h)/tanh(slope*tanhmax)
           wt = wt*(1d0-tanh(slope*h))*(1d0+tanh(slope*h))
     .            /tanh(slope*tanhmax)*slope

           x = xmin + rran * (xmax - xmin)
           wt = wt * (xmax - xmin)
      return
      end


      subroutine pick_minushalftanh(x,slope,tanhmax,xmin,xmax,ran,wt)
      implicit real*8(a-h,o-z)
      !> mapping: y(x) = arctanh(h(ran)), h(ran) = [tanhmin, 0]
      !> we get a jacobian: (1-h)*(1+h)
      !> then normalise to tanh(hmax) and apply to any max or min value
           tanhmin = - tanhmax
           h = tanhmin * (1 - ran)
           wt = wt * ( - tanhmin)

           rran = - tanh(slope*h)/tanh(slope*tanhmin) + 1d0
           wt = wt*(-1d0+tanh(slope*h))*(1d0+tanh(slope*h))
     .            /tanh(slope*tanhmin)*slope

           x = xmin + rran * (xmax - xmin)
           wt = wt * (xmax - xmin)
      return
      end



      subroutine pick_wphi(phi,slope,tanhmax,phimin,phimax,ran,wt)
      implicit real*8(a-h,o-z)
      ! w type distribution sampling of phi, split at pi
        rcut = 5d-1
        split = (phimax - phimin)/2d0 + phimin
        if (ran < rcut) then
           rran = ran/rcut
           wt   = wt/rcut
           call pick_tanh(phi, slope, tanhmax, phimin, split, rran, wt)
           return
        else
           rran = (ran-rcut)/(1d0-rcut)
           wt   = wt/(1d0-rcut)
           call pick_tanh(phi, slope, tanhmax, split, phimax, rran, wt)
           return
        endif
      end


      subroutine pick_wwphi(phi,slope,tanhmax,phimin,phimax,ran,wt)
      implicit real*8(a-h,o-z)
      ! w type distribution sampling of phi, split at pi/2,pi,pi*3/2

        rcut1 = 1d0/4d0
        rcut2 = 1d0/2d0
        rcut3 = 3d0/4d0

        split1 = (phimax - phimin)*rcut1 + phimin
        split2 = (phimax - phimin)*rcut2 + phimin
        split3 = (phimax - phimin)*rcut3 + phimin

        if (ran < rcut1) then
           rran = ran/rcut1
           wt   = wt/rcut1
           call pick_tanh(phi, slope, tanhmax, phimin, split1, rran, wt)
           return
        elseif (ran < rcut2) then
           rran = (ran-rcut1)/(rcut2-rcut1)
           wt   = wt/(rcut2-rcut1)
           call pick_tanh(phi, slope, tanhmax, split1, split2, rran, wt)
           return
        elseif (ran < rcut3) then
           rran = (ran-rcut2)/(rcut3-rcut2)
           wt   = wt/(rcut3-rcut2)
           call pick_tanh(phi, slope, tanhmax, split2, split3, rran, wt)
           return
        elseif (ran < 1d0) then
           rran = (ran-rcut3)/(1d0-rcut3)
           wt   = wt/(1d0-rcut3)
           call pick_tanh(phi, slope, tanhmax, split3, phimax, rran, wt)
           return
        endif
      end

*
************************************************************************
*

      subroutine pick_log_lin(x,xmin,xmax,rcut,xatio,ran,wt)
      implicit real*8(a-h,o-z)
c         rcut  = 8d-1
c         xatio = 5d-1
         xmid = xatio*xmax
         if (ran < rcut) then
             ! log
            rran = ran / rcut
            wt   = wt / rcut
            call pick(2,x,xmin,xmid,rran,wt)
            return
         else
            ! linear
            rran = (ran -rcut) / (1d0-rcut)
            wt  = wt / (1d0-rcut)
            call pick(1,x,xmid,xmax,rran,wt)
            return
         endif

      end

      subroutine pick_halfminuxtanh_lin(x,xmin,xmax,tanhmax,rcut,xatio,ran,wt)
      implicit real*8(a-h,o-z)
c         rcut  = 8d-1
c         xatio = 5d-1
         xmid = xatio*xmax
         if (ran < rcut) then
             ! log
            rran = ran / rcut
            wt   = wt / rcut
            call pick_minushalftanh(x,1d0,tanhmax,xmin,xmid,rran,wt)
            return
         else
            ! linear
            rran = (ran -rcut) / (1d0-rcut)
            wt  = wt / (1d0-rcut)
            call pick(1,x,xmid,xmax,rran,wt)
            return
         endif

      end

      !> mapping: y(x) = arctanh(x)
      !> we get a jacobian: (1-x)*(1+x)
      !> need a technical cutoff to restric range (eps > 0)
      !> -1+eps < costh < 1-eps
      subroutine pickcos(costh,eps,r1,wt)
      implicit none
      double precision :: costh, eps, r1, wt
      double precision :: y, ymin, ymax
      ymax = 0.5d0 * log((2d0-eps)/eps)
      ymin = -ymax
      y = ymin + r1*(ymax-ymin)
      costh = tanh(y)
      wt = wt*(1d0-costh)*(1d0+costh)*(ymax-ymin)
      return
      end


      !> mapping: y(x) = arcsin(x)
      !> we get a jacobian: sqrt( (1-x)*(1+x) )
      !> less aggressive than pickcos, but most
      !> importantly, mapping regular in x=-1,+1
      !> so no technical cut (y0)
      subroutine pickcos_alt(costh,r1,wt)
      use Constants_mod, only : pi
      implicit none
      double precision :: costh, r1, wt
      double precision :: y
      y  = -pi/2d0 + r1*pi
      wt = wt * pi * cos(y)
      costh = sin(y)
      return
      end



      subroutine pick_sinv_generic(s,smin,smax,rm,rg,r1,wt,iResType)
      use Constants_mod, only : pi
      implicit real*8(a-h,o-z)

      if (smin > smax) then
         wt = 0d0
         return
      end if

      !> iResType -- heavy particle decay type
      !>  0 : linear: pick(1)
      !>  1 : Breit-Wigner
      !>  2 : narrow width inserted: delta function (Gamma*M)
      !>  3 : narrow width inserted: delta function (4*pi)
      !> 10 : log: pick(2)
      !> -1 : lin - BW
      !> -2 : BW - lin
      !> -20: BW - log
      !> -3 : lin - BW - lin
      !> -4 : BW - energy dependent width
      select case(iResType)

      case(0)  ! lin
        call pick(1,s,smin,smax,r1,wt)

      case(1)  ! BW
        call pickres(s,smin,smax,rm,rg,r1,wt)
        if (s<smin .or. s>smax) then
           wt = 0d0
           return
        end if

      case(2)  ! NWA (Gamma M)
         s  = rm**2
C          print*, "pick_sinv_generic:", iResType
C          print*, "smin = ", smin
C          print*, "s    = ", s
C          print*, "smax = ", smax
C          read(*,*)
         if (s<smin .or. s>smax) then
            wt = 0d0
            return
         end if
         wt = wt*pi*rm*rg

      case(3)  ! on-shell: (2pi) * (8pi) [Vol 2-pt LIPS]
         s  = rm**2
         if (s<smin .or. s>smax) then
            wt = 0d0
            return
         end if
         wt = wt*16d0*pi**2

      case(4)  ! Q-differential
         s  = rm**2
         if (s<smin .or. s>smax) then
            wt = 0d0
            return
         end if
         wt = wt* 2d0*rm

      case(10)  ! log
        call pick(2,s,smin,smax,r1,wt)

      case(-1)  ! lin - BW
        call pick_lin_res(s,smin,smax,rm,rg,r1,wt)

      case(-2)  ! BW - lin
        call pick_res_lin(s,smin,smax,rm,rg,r1,wt)

      case(-20)  ! BW - log
        call pick_res_log(s,smin,smax,rm,rg,r1,wt)

      case(-3)  ! lin - BW - lin
        call pick_lin_res_lin(s,smin,smax,rm,rg,r1,wt)

      case(-4) ! BW, energy dependent width
        call pickres(s,smin,smax,rm,rg,r1,wt)
        if (s<smin .or. s>smax) then
           wt = 0d0
           return
        end if

      case default
         print*, "pick_sinv_generic: unknown type:", iResType
         stop

      end select
      end


      subroutine pickres(s,smin,smax,rm,rg,r1,wt)
      implicit real*8(a-h,o-z)
      if (smin > smax) then
         print*, "pickres: smin > smax!"
         wt = 0d0
         return
      end if
      zmin = atan((smin-rm**2)/rg/rm)/rg/rm
      zmax = atan((smax-rm**2)/rg/rm)/rg/rm
      z = zmin + r1*(zmax-zmin)
      wt = wt*(zmax-zmin)
      s = rm*rg*tan(rm*rg*z)+rm**2
      dv = 1d0/((s-rm**2)**2+rm**2*rg**2)
      wt = wt/dv
      return
      end


      subroutine pick_lin_res(s,smin,smax,rm,rg,r1,wt)
      implicit real*8(a-h,o-z)
      scut = rm**2 - 5d0*rm*rg  ! region below which we map linearly
      ! print*, "scut = ", scut
      if ( scut < smin ) then
         !> only BW contributes
         ! print*, "... only BW"
         call pickres(s,smin,smax,rm,rg,r1,wt)
         return
      else if ( scut > smax ) then
         !> only lin contributes
         ! print*, "... only lin"
         call pick(1,s,smin,smax,r1,wt)
         return
      else
         !> both regions contribute
         ! print*, "... both: r1 = ", r1
         !> [0, r1cut] <=> [smin, scut] : linear
         !> [r1cut, 1] <=> [scut, smax] : Breit-Wigner
         r1cut = 0.5d0  ! 50/50
         if (r1 < r1cut) then
            ! linear
            ! print*, "... lin"
            rr1 = r1 / r1cut
            wt  = wt / r1cut
            call pick(1,s,smin,scut,rr1,wt)
            return
         else
            ! Breit-Wigner
            ! print*, "... BW"
            rr1 = (r1-r1cut) / (1d0-r1cut)
            wt  = wt / (1d0-r1cut)
            call pickres(s,scut,smax,rm,rg,rr1,wt)
            return
         end if
      end if
      end


      subroutine pick_res_gen(s,smin,smax,rm,rg,r1,wt,ipick)
      implicit real*8(a-h,o-z)
      scut = rm**2 + 5d0*rm*rg  ! region above which we map with a generic mapping [ipick]
      ! print*, "scut = ", scut
      if ( scut < smin ) then
         !> only generic contributes
         ! print*, "... only generic"
         call pick(ipick,s,smin,smax,r1,wt)
         return
      else if ( scut > smax ) then
         !> only BW contributes
         ! print*, "... only BW"
         call pickres(s,smin,smax,rm,rg,r1,wt)
         return
      else
         !> both regions contribute
         ! print*, "... both: r1 = ", r1
         !> [0, r1cut] <=> [smin, scut] : Breit-Wigner
         !> [r1cut, 1] <=> [scut, smax] : generic
         r1cut = 0.5d0  ! 50/50
         if (r1 < r1cut) then
            ! Breit-Wigner
            ! print*, "... BW"
            rr1 = r1 / r1cut
            wt  = wt / r1cut
            call pickres(s,smin,scut,rm,rg,rr1,wt)
            return
         else
            ! generic
            ! print*, "... generic"
            rr1 = (r1-r1cut) / (1d0-r1cut)
            wt  = wt / (1d0-r1cut)
            call pick(ipick,s,scut,smax,rr1,wt)
            return
         end if
      end if
      end


      subroutine pick_res_lin(s,smin,smax,rm,rg,r1,wt)
      implicit real*8(a-h,o-z)
      call pick_res_gen(s,smin,smax,rm,rg,r1,wt,1)
      end

      subroutine pick_res_log(s,smin,smax,rm,rg,r1,wt)
      implicit real*8(a-h,o-z)
      call pick_res_gen(s,smin,smax,rm,rg,r1,wt,2)
      end

      subroutine pick_lin_res_lin(s,smin,smax,rm,rg,r1,wt)
      implicit real*8(a-h,o-z)
      slo = rm**2 - 5d0*rm*rg  ! region below which we map linearly
      shi = rm**2 + 5d0*rm*rg  ! region above which we map linearly
      if ( slo < smin ) then
         !> only 2 regions: BW - lin
         call pick_res_lin(s,smin,smax,rm,rg,r1,wt)
         return
      else if ( shi > smax ) then
         !> only 2 regions: lin - BW
         call pick_lin_res(s,smin,smax,rm,rg,r1,wt)
         return
      else
         !> all three regions contribute
         !> [0,    r1lo] <=> [smin, slo ] : linear
         !> [r1lo, r1hi] <=> [slo,  shi ] : Breit-Wigner
         !> [r1hi, 1   ] <=> [shi,  smax] : linear
         r1lo = 1d0/3d0  ! 33.3%
         r1hi = 2d0/3d0  ! 33.3%
         if (r1 < r1lo) then
            ! linear
            rr1 = r1 / r1lo
            wt  = wt / r1lo
            call pick(1,s,smin,scut,rr1,wt)
            return
         else if (r1 < r1hi) then
            ! Breit-Wigner
            rr1 = (r1-r1lo) / (r1hi-r1lo)
            wt  = wt / (r1hi-r1lo)
            call pickres(s,scut,smax,rm,rg,rr1,wt)
            return
         else
            ! linear
            rr1 = (r1-r1hi) / (1d0-r1hi)
            wt  = wt / (1d0-r1hi)
            call pick(1,s,smin,scut,rr1,wt)
            return
         end if
      end if
      end


*
************************************************************************
*

      subroutine makeps2com(s,costh,phi,rm1sq,rm2sq,p1,p2,p1a,p2a)
      use Constants_mod, only : pi
      implicit real*8(a-h,o-z)
      dimension p1(4),p2(4),p1a(4),p2a(4)

c ---- generates momenta p1,p2 in their centre-of-mass frame
c ---- and momenta p1a,p2a rotated by pi/2 in azimuth

      sinth = sqrt((1d0-costh)*(1d0+costh))
      sp  = sin(phi)
      cp  = cos(phi)
      spa = sin(phi+pi/2d0)
      cpa = cos(phi+pi/2d0)

      smin = rm1sq + rm2sq
      if (s.lt.smin) then
         write(6,*) 'improper call of makeps2com'
         stop
      endif

      roots = sqrt(s)

      e1 = (s + rm1sq - rm2sq)/2d0/roots
      e2 = (s + rm2sq - rm1sq)/2d0/roots
      pp = sqrt(e1**2-rm1sq)

      p1(1) = pp*sinth*cp
      p1(2) = pp*sinth*sp
      p1(3) = pp*costh
      p1(4) = e1

      p1a(1) = pp*sinth*cpa
      p1a(2) = pp*sinth*spa
      p1a(3) = pp*costh
      p1a(4) = e1

      do i=1,3
         p2(i)  = -p1(i)
         p2a(i) = -p1a(i)
      enddo
      p2(4)  = e2
      p2a(4) = e2

      return
      end



      subroutine makeps2com_qT(itype,tanhmax,rqT,phi,s,rm1sq,rm2sq,p1,p2,p1a,p2a,wt)
      use Constants_mod, only : pi
      use Run_mod, only : qtsample_th_run
      implicit real*8(a-h,o-z)
      common/pmasses/rma2,rmb2,rm2(1:7),shat
!$omp threadprivate(/pmasses/)
      common/tcuts/ymin,y0
      dimension p1(4),p2(4),p1a(4),p2a(4)

c ---- generates momenta p1,p2 in their centre-of-mass frame
c ---- and momenta p1a,p2a rotated by pi/2 in azimuth

      rootshat=sqrt(shat)
      if (qtsample_th_run.eq.0d0) then qtsample_th_run = 3d-1 !default value if not specified in runcard

      if (RAND(0).lt.0.5d0) then
          overturn=1d0
      else
          overturn=-1d0
      endif
c--XC Use overturn factor to get the other part of the phase space for different sign of p1(3).
      wt = wt*2d0
c--XC this extra 2 times wpts is for compensating the sampling of only half of z-axis for pt.

      sp  = sin(phi)
      cp  = cos(phi)
      spa = sin(phi+pi/2d0)
      cpa = cos(phi+pi/2d0)

      smin = rm1sq + rm2sq
      if (s.lt.smin) then
         write(6,*) 'improper call of makeps2com'
         stop
      endif

      roots = dsqrt(s)

      e1 = (s + rm1sq - rm2sq)/2d0/roots
      e2 = (s + rm2sq - rm1sq)/2d0/roots
      pp = dsqrt(e1**2-rm1sq)
      pTmax = pp

      if (tanhmax.eq.0d0) then
         call pick_qT(itype,pT,qtsample_th_run,pTmax,rqT,wt)
      else
         call pick_tanh_generic(itype,pT,1d0,tanhmax,y0*rootshat,pTmax,rqT,wt)
      endif

      wt = wt * dsqrt(dlambda(s,rm1sq,rm2sq))/s/32d0/pi**2 * pT
     .       / sqrt((pp-pT)*(pp+pT)) / pp

      p1(1) = pT*cp
      p1(2) = pT*sp
      p1(3) = overturn*sqrt((pp-pT)*(pp+pT))
      p1(4) = e1

      p1a(1) = pT*cpa
      p1a(2) = pT*spa
      p1a(3) = overturn*sqrt((pp-pT)*(pp+pT))
      p1a(4) = e1

      do i=1,3
         p2(i)  = -p1(i)
         p2a(i) = -p1a(i)
      enddo
      p2(4)  = e2
      p2a(4) = e2

      return
      end


      subroutine makeps2com_qT_noqTcut(itype,tanhmax,rqT,phi,s,rm1sq,rm2sq,p1,p2,p1a,p2a,wt)
      use Constants_mod, only : pi
      implicit real*8(a-h,o-z)
      common/pmasses/rma2,rmb2,rm2(1:7),shat
!$omp threadprivate(/pmasses/)
      common/tcuts/ymin,y0
      dimension p1(4),p2(4),p1a(4),p2a(4)

c ---- generates momenta p1,p2 in their centre-of-mass frame
c ---- and momenta p1a,p2a rotated by pi/2 in azimuth

      rootshat=sqrt(shat)

      if (RAND(0).lt.0.5d0) then
          overturn=1d0
      else
          overturn=-1d0
      endif
c--XC Use overturn factor to get the other part of the phase space for different sign of p1(3).
      wt = wt*2d0
c--XC this extra 2 times wpts is for compensating the sampling of only half of z-axis for pt.

      sp  = sin(phi)
      cp  = cos(phi)
      spa = sin(phi+pi/2d0)
      cpa = cos(phi+pi/2d0)

      smin = rm1sq + rm2sq
      if (s.lt.smin) then
         write(6,*) 'improper call of makeps2com'
         stop
      endif

      roots = dsqrt(s)

      e1 = (s + rm1sq - rm2sq)/2d0/roots
      e2 = (s + rm2sq - rm1sq)/2d0/roots
      pp = dsqrt(e1**2-rm1sq)
      pTmax = pp

      if (tanhmax.eq.0d0) then
         call pick_qT(itype,pT,0d0,pTmax,rqT,wt)
      else
         call pick_tanh_generic(itype,pT,1d0,tanhmax,y0*rootshat,pTmax,rqT,wt)
      endif

      wt = wt * dsqrt(dlambda(s,rm1sq,rm2sq))/s/32d0/pi**2 * pT
     .       / sqrt((pp-pT)*(pp+pT)) / pp

      p1(1) = pT*cp
      p1(2) = pT*sp
      p1(3) = overturn*sqrt((pp-pT)*(pp+pT))
      p1(4) = e1

      p1a(1) = pT*cpa
      p1a(2) = pT*spa
      p1a(3) = overturn*sqrt((pp-pT)*(pp+pT))
      p1a(4) = e1

      do i=1,3
         p2(i)  = -p1(i)
         p2a(i) = -p1a(i)
      enddo
      p2(4)  = e2
      p2a(4) = e2

      return
      end

*
************************************************************************
*
      subroutine p2generic(r1,r2,s,rmasq,rmbsq,rm1sq,rm2sq,
     .                     p1,p2,pa,pb,wt)
      use Constants_mod, only : pi, twopi, pi2
c--- generates generic 2-parton phase space in com system
c--- for arbitrary (time-like and space-like) external masses
c--- input: random number r1,r2
c--- input: s, external masses
c--- output: p1(4),p2(4)
c--- processed: phase space weight wt
c--- phase space dR2 = d^3p1/(2*E1*(2*pi)^3)*[p1<->p2]*(2*pi)^4*del(..)
      implicit real*8(a-h,o-z)
      common/pmasses/rma2,rmb2,rm2(1:7),shat
!$omp threadprivate(/pmasses/)
      common/tcuts/ymin,y0
      dimension p1(4),p2(4),pa(4),pb(4)


      roots=dsqrt(s)
      ea = (s+rmasq-rmbsq)/2d0/roots
      eb = (s+rmbsq-rmasq)/2d0/roots
      e1 = (s+rm1sq-rm2sq)/2d0/roots
      e2 = (s+rm2sq-rm1sq)/2d0/roots
      dlin = dsqrt(dlambda(s,rmasq,rmbsq))
      dlout = dsqrt(dlambda(s,rm1sq,rm2sq))
      pin = dlin/2d0/roots
      pout = dlout/2d0/roots


      rmsqmin = min(rm1sq,rm2sq)
      if(rmsqmin.lt.y0*shat)then
      ! Massless particle recoiling off a massive particle, use old sampling method
      ! ta1min ~ 0
         ta1min = rmasq + rm1sq - 2d0*ea*e1 - 2d0*pin*pout
         ta1max = rmasq + rm1sq - 2d0*ea*e1 + 2d0*pin*pout
         call pick(12,ta1,-ta1max,-ta1min,r1,wt)
      else
      ! TM : sampling rat=(ta1max-ta1+y0*shat)/ta1 logarthmically
         ta1max =-( rmasq + rm1sq - 2d0*ea*e1 - 2d0*pin*pout)
         ta1min =-( rmasq + rm1sq - 2d0*ea*e1 + 2d0*pin*pout)
         ratmin = y0*shat/ta1max
         ratmax = (ta1max-ta1min+y0*shat)/ta1min
         call pick(2,rat,ratmin,ratmax,r1,wt)
         ta1 = (ta1max+y0*shat)/(rat+1d0)
         wt  = wt*(ta1max+y0*shat)/(rat+1d0)**2
      endif
c     *****************************************
      ta1 = -ta1
      ta2 = rmasq+rmbsq+rm1sq+rm2sq-s-ta1
      costh = (ta1-rmasq-rm1sq+2d0*ea*e1)/(2d0*pin*pout)
      if (abs(costh).gt.1d0) then
         !print*, r1,r2
         !write(6,*) 'improper call of ps2generic', costh
         !stop
         if (costh < -1d0) costh = -1d0
         if (costh > +1d0) costh = +1d0
         !print*, "re-setting costh to:", costh
      endif
      sinth = dsqrt(1d0-costh**2)
      wt = wt/16d0/pi2/dlin

      call pick(1,ph,0d0,twopi,r2,wt)
      cp = cos(ph)
      sp = sin(ph)

      p1(4) = e1
      p1(1) = pout*sinth*cp
      p1(2) = pout*sinth*sp
      p1(3) = pout*costh

      p2(4) = e2
      do i=1,3
         p2(i) = -p1(i)
      enddo

      pa(4) = ea
      pa(1) = 0d0
      pa(2) = 0d0
      pa(3) = pin

      pb(4) = eb
      do i=1,3
         pb(i) = -pa(i)
      enddo

      return
      end



      subroutine p2generic_alt(itype,r1,r2,s,rmasq,rmbsq,rm1sq,rm2sq,
     .                         p1,p2,pa,pb,wt)
      use Constants_mod, only : pi, twopi, pi2
c--- generates generic 2-parton phase space in com system
c--- for arbitrary (time-like and space-like) external masses
c--- input: random number r1,r2
c--- input: s, external masses
c--- output: p1(4),p2(4)
c--- processed: phase space weight wt
c--- phase space dR2 = d^3p1/(2*E1*(2*pi)^3)*[p1<->p2]*(2*pi)^4*del(..)
      implicit real*8(a-h,o-z)
      common/tcuts/ymin,y0
      dimension p1(4), p2(4), pa(4), pb(4)


      roots = sqrt(s)
      ea = (s+rmasq-rmbsq)/2d0/roots
      eb = (s+rmbsq-rmasq)/2d0/roots
      e1 = (s+rm1sq-rm2sq)/2d0/roots
      e2 = (s+rm2sq-rm1sq)/2d0/roots
      dlin  = sqrt(dlambda(s,rmasq,rmbsq))
      dlout = sqrt(dlambda(s,rm1sq,rm2sq))
      pin  = dlin/2d0/roots
      pout = dlout/2d0/roots

      select case (itype)
      case(1)  ! linear
         call pick(1,costh,-1d0,1d0,r1,wt)
      case(2)  ! sqrt
         call pickcos_alt(costh,r1,wt)
      case(3)  ! log
         call pickcos(costh,y0,r1,wt)
      case(4)  ! arctan
         call pick(12,costh,-1d0,1d0,r1,wt)
      case default
         print*, "p2generic_alt: unknown itype:", itype
         stop
      end select

      sinth = sqrt((1d0-costh)*(1d0+costh))

      call pick(1,ph,0d0,twopi,r2,wt)
      cp = cos(ph)
      sp = sin(ph)

      wt = wt * dlout / (8d0*s) / (4d0*pi2)

      p1(1) = pout*sinth*cp
      p1(2) = pout*sinth*sp
      p1(3) = pout*costh
      p1(4) = e1

      do i=1,3
         p2(i) = -p1(i)
      enddo
      p2(4) = e2

      pa(1) = 0d0
      pa(2) = 0d0
      pa(3) = pin
      pa(4) = ea

      do i=1,3
         pb(i) = -pa(i)
      enddo
      pb(4) = eb

      return
      end





      subroutine p2generic_qT(itype,tanhmax,r1,r2,s,rmasq,rmbsq,rm1sq,rm2sq,
     .                         p1,p2,pa,pb,wt)
      use Constants_mod, only : pi, twopi, pi2
      use Run_mod, only : qtsample_th_run
c--- generates generic 2-parton phase space in com system
c--- for arbitrary (time-like and space-like) external masses
c--- input: random number r1,r2
c--- input: s, external masses
c--- output: p1(4),p2(4)
c--- processed: phase space weight wt
      implicit real*8(a-h,o-z)
      common/pmasses/rma2,rmb2,rm2(1:7),shat
!$omp threadprivate(/pmasses/)
      common/tcuts/ymin,y0
      dimension p1(4), p2(4), pa(4), pb(4)

      if (qtsample_th_run.eq.0d0) then qtsample_th_run = 3d-1 !default value if not specified in runcard

      rootshat=sqrt(shat)

      if (RAND(0).lt.0.5d0) then
          overturn=1d0
      else
          overturn=-1d0
      endif
c--XC Use overturn factor to get the other part of the phase space for different sign of p1(3).
      wt = wt*2d0
c--XC this extra 2 times wpts is for compensating the sampling of only half of z-axis for pt.

      roots = sqrt(s)
      ea = (s+rmasq-rmbsq)/2d0/roots
      eb = (s+rmbsq-rmasq)/2d0/roots
      e1 = (s+rm1sq-rm2sq)/2d0/roots
      e2 = (s+rm2sq-rm1sq)/2d0/roots
      dlin  = sqrt(dlambda(s,rmasq,rmbsq))
      dlout = sqrt(dlambda(s,rm1sq,rm2sq))
      pin  = dlin/2d0/roots
      pout = dlout/2d0/roots

      pTmax = pout

      if (tanhmax.eq.0d0) then
         call pick_qT(itype,pT,qtsample_th_run,pTmax,r1,wt)
      else
         call pick_tanh_generic(itype,pT,1d0,tanhmax,y0*rootshat,pTmax,r1,wt)
      endif

      call pick(1,ph,0d0,twopi,r2,wt)
      cp = cos(ph)
      sp = sin(ph)

      wt =  wt * dlout / (8d0*s) / (4d0*pi2) * pT
     .       / sqrt((pout-pT)*(pout+pT)) / pout
c--XC PS weight for sampling pT

      p1(1) = pT*cp
      p1(2) = pT*sp
      p1(3) = overturn*sqrt((pout-pT)*(pout+pT))
      p1(4) = e1
c--XC p1 always assign to colourless massive particle that does not become collinear with pa before wedge swapping
c--XC to get p2 collinear with pa, here we keep p1(3) negative
c--XC since here we make a choice of the sign of p1(3), we always need to do wedge swapping when using p2generic_qT
c      write(*,*)"p1 =",p1(1),p1(2),p1(3),p1(4)

      do i=1,3
         p2(i) = -p1(i)
      enddo
      p2(4) = e2

      pa(1) = 0d0
      pa(2) = 0d0
      pa(3) = pin
      pa(4) = ea

      do i=1,3
         pb(i) = -pa(i)
      enddo
      pb(4) = eb

      return
      end


      subroutine p2generic_qT_noy0cut(itype,tanhmax,r1,r2,s,rmasq,rmbsq,rm1sq,rm2sq,
     .                         p1,p2,pa,pb,wt)
      use Constants_mod, only : pi, twopi, pi2
      use Run_mod, only : qtsample_th_run
c--- generates generic 2-parton phase space in com system
c--- for arbitrary (time-like and space-like) external masses
c--- input: random number r1,r2
c--- input: s, external masses
c--- output: p1(4),p2(4)
c--- processed: phase space weight wt
      implicit real*8(a-h,o-z)
      dimension p1(4), p2(4), pa(4), pb(4)

      if (qtsample_th_run.eq.0d0) then qtsample_th_run = 3d-1 !default value if not specified in runcard

      if (RAND(0).lt.0.5d0) then
          overturn=1d0
      else
          overturn=-1d0
      endif
c--XC Use overturn factor to get the other part of the phase space for different sign of p1(3).
      wt = wt*2d0
c--XC this extra 2 times wpts is for compensating the sampling of only half of z-axis for pt.

      roots = sqrt(s)
      ea = (s+rmasq-rmbsq)/2d0/roots
      eb = (s+rmbsq-rmasq)/2d0/roots
      e1 = (s+rm1sq-rm2sq)/2d0/roots
      e2 = (s+rm2sq-rm1sq)/2d0/roots
      dlin  = sqrt(dlambda(s,rmasq,rmbsq))
      dlout = sqrt(dlambda(s,rm1sq,rm2sq))
      pin  = dlin/2d0/roots
      pout = dlout/2d0/roots

      pTmax = pout

      if (tanhmax.eq.0d0) then
         call pick_qT(itype,pT,qtsample_th_run,pTmax,r1,wt)
      else
         call pick_tanh_generic(itype,pT,1d0,tanhmax,0d0,pTmax,r1,wt)
      endif

      call pick(1,ph,0d0,twopi,r2,wt)
      cp = cos(ph)
      sp = sin(ph)

      wt =  wt * dlout / (8d0*s) / (4d0*pi2) * pT
     .       / sqrt((pout-pT)*(pout+pT)) / pout
c--XC PS weight for sampling pT

      p1(1) = pT*cp
      p1(2) = pT*sp
      p1(3) = overturn*sqrt((pout-pT)*(pout+pT))
      p1(4) = e1
c--XC p1 always assign to colourless massive particle that does not become collinear with pa before wedge swapping
c--XC to get p2 collinear with pa, here we keep p1(3) negative
c--XC since here we make a choice of the sign of p1(3), we always need to do wedge swapping when using p2generic_qT_noy0cut
c      write(*,*)"p1 =",p1(1),p1(2),p1(3),p1(4)

      do i=1,3
         p2(i) = -p1(i)
      enddo
      p2(4) = e2

      pa(1) = 0d0
      pa(2) = 0d0
      pa(3) = pin
      pa(4) = ea

      do i=1,3
         pb(i) = -pa(i)
      enddo
      pb(4) = eb

      return
      end

*
************************************************************************
*

      real*8 function dlambda(a,b,c)
      implicit real*8(a-h,o-z)

      if (dabs(c).lt.min(dabs(a),dabs(b))) then
         dlambda = (a-b)**2 + c*c - 2d0*(a+b)*c
      elseif(dabs(b).lt.min(dabs(a),dabs(c))) then
         dlambda = (a-c)**2 + b*b - 2d0*(a+c)*b
      else
         dlambda = (b-c)**2 + a*a - 2d0*(b+c)*a
      endif
c      dlambda = a*a + b*b + c*c - 2d0*a*b - 2d0*a*c - 2d0*b*c
      return
      end


      subroutine fillinv(n,p,pa,pb,sij,tij)
      use KinData_mod, only : dot
      implicit real*8(a-h,o-z)
      dimension p(4,1:n),pa(4),pb(4),sij(1:n,1:n),tij(1:2,1:n)

      do i=1,n
         sij(i,i) = 0d0
         tij(1,i) = -2d0*dot(pa(1),p(1,i))
         tij(2,i) = -2d0*dot(pb(1),p(1,i))
         do j=i+1,n
            sij(i,j) = 2d0*dot(p(1,i),p(1,j))
            sij(j,i) = sij(i,j)
         enddo
      enddo
      return
      end


