

      subroutine phase2dis(x,wt,ifail)
      use DIS_mod, only: id_Q2_dis
      use Selectors_mod, only : getExtremes_sel
      use UnitPhase_mod, only : phase_unit, nocuts
      implicit real*8(a-h,o-z)
      dimension p1(4),p2(4),p3(4),v(4),v1(4),q(4)
      dimension pqa(4)
      dimension x(*)
      parameter(pi=3.141592653589793238d0)
      common/pmasses/rma2,rmb2,rm2(1:7),shat
!$omp threadprivate(/pmasses/)
      common/pin/pa(4),pb(4)
!$omp threadprivate(/pin/)
      common/pmom/p(4,7)
!$omp threadprivate(/pmom/)
      common/invar2/sij2(2,2),tij2(2,2)
!$omp threadprivate(/invar2/)
      common/pmomang2/pang2(4,2,1)
!$omp threadprivate(/pmomang2/)
      common/invarang2/sijang2(2,2,1),tijang2(2,2,1)
!$omp threadprivate(/invarang2/)
      common /tcuts/ymin,y0
      common /method/imethod
      common /Nphase/Nphase
!$omp threadprivate(/Nphase/)
      common /nwa/rm12,rg12,inwa

*
* phase space for 2->2 scattering, a+b -> 1+2 with q=pb-p1
*
* incoming momenta along z-axis, pa is +z
*
*
* massless case: integrates to P_2 = 1/8/pi
*

cLIPS      print*, "phase2dis", x(1), wt, ifail

      Nphase=1

      r1 = x(1)
      ifail =1
      wt = 0d0
      wtps = 1d0

      sqa = 0d0
      q2pmin = y0*shat
      q2pmax = shat-sqa
      if (.not.(phase_unit.and.nocuts)) then
         call getExtremes_sel(id_Q2_dis, minVal=q2pmin, maxVal=q2pmax)
      end if

      if (q2pmin.ge.q2pmax) return

      if(.not. phase_unit) then
         if (inwa.eq.1) then
            call pickres(q2, q2pmin, q2pmax, rm12, rg12, r1, wtps)
         else
            call pick(3,q2,q2pmin,q2pmax,r1,wtps)
         endif
      else
         call pick(1,q2,q2pmin,q2pmax,r1,wtps)
      endif

      wtps = wtps/8d0/pi/shat

      q2s = -q2
      call disframe0(sqa,0d0,q2,pa,pb,q,p1)
      do i=1,4
         p2(i) = pa(i)+pb(i)-p1(i)
      enddo

      do i=1,4
         p(i,1) = p1(i)
         p(i,2) = p2(i)
         pang2(i,1,1) = p1(i)
         pang2(i,2,1) = p2(i)
      enddo

      call fillinv(2,pang2(1:4,1:2,1),pa,pb,sijang2(1:2,1:2,1)
     .                                     ,tijang2(1:2,1:2,1))

c      write(6,101) p1(1:4)
c      write(6,102) p2(1:4)
c      write(6,103) p3(1:4)
c      write(6,105) pa(1:4)
c      write(6,106) pb(1:4)
c      write(6,109) v1(1:4)
c      write(6,*) q2/dot(q,q)

 101  format('p1:',4f14.6)
 102  format('p2:',4f14.6)
 103  format('p3:',4f14.6)
 105  format('pa:',4f14.6)
 106  format('pb:',4f14.6)
 109  format('pp:',4f14.6)

     
      ifail =0
      wt = wtps
      return
      end




      subroutine fillcommon2pdis
      implicit real*8(a-h,o-z)
      common/pmomang2/pang2(4,2,1)
!$omp threadprivate(/pmomang2/)
      common/invarang2/sijang2(2,2,1),tijang2(2,2,1)
!$omp threadprivate(/invarang2/)
      common/eventmom2p/pevt2(4,2,2)
!$omp threadprivate(/eventmom2p/)
      common/eventinv2p/sevt2(2,2,2),tevt2(2,2,2),ievt2(2)
!$omp threadprivate(/eventinv2p/)

c---- pang3,pang3g and sija,tija are input
c---- this subroutine fills
c----       pevt2(component,pnum,ievent)             : 4-momenta of events
c----       sevt2(i,j,ievent),tevt2([ab],i,ievent)   : invariants of events
c----
c---- ievent = 1 fixed

      do j=1,2
         do k=1,4
            pevt2(k,j,1)    = pang2(k,j,1)
         enddo
      enddo

      do j=1,2
         do k=1,2
            sevt2(j,k,1)    = sijang2(j,k,1)
         enddo
         tevt2(1,j,1)   = tijang2(1,j,1)
         tevt2(2,j,1)   = tijang2(2,j,1)
      enddo
      ievt2(1) = 1

c---- a few tests here
c      write(6,101) pevt3(1:4,1,1)
c      write(6,102) pevt3(1:4,2,1)
c      write(6,103) pevt3(1:4,3,1)
c      write(6,104) pevt3(1:4,1,1)+pevt3(1:4,2,1)+pevt3(1:4,3,1)

c      it =1
c      ta1 = -pevt3(4,1,it)+pevt3(3,1,it)
c      ta2 = -pevt3(4,2,it)+pevt3(3,2,it)
c      ta3 = -pevt3(4,3,it)+pevt3(3,3,it)
c      write(6,110) ta1/tevt3(1,1,it),ta2/tevt3(1,2,it),ta3/tevt3(1,3,it)

 101  format('p1:',4f14.6)
 102  format('p2:',4f14.6)
 103  format('p3:',4f14.6)
 104  format('p4:',4f14.6)
 110  format(3g18.11)
 111  format(2g18.11)

      return
      end


      subroutine phase3dis(x,wt,ifail)
      use KinData_mod, only : boostrest,unboostrest,rotatetoz,unrotatetoz
      use DIS_mod, only: id_Q2_dis
      use Selectors_mod, only : getExtremes_sel
      use UnitPhase_mod, only : phase_unit, nocuts
      implicit real*8(a-h,o-z)
      dimension p1(4),p2(4),p3(4),v(4),v1(4),q(4)
      dimension p1g(4),p2g(4),p3g(4),pag(4),pbg(4),pqg(4)
      dimension p1p(4),p2p(4),p3p(4),pap(4),pbp(4),qp(4)
      dimension p1a(4),p2a(4),p3a(4)
      dimension p1ap(4),p2ap(4),p3ap(4)
      dimension p23(4),p23p(4)
      dimension pqa(4)
      dimension x(*)
      dimension bmat(4,4),ubmat(4,4),rmat(4,4),urmat(4,4),fmat(4,4)
      parameter(pi=3.141592653589793238d0)
      common/pmasses/rma2,rmb2,rm2(1:7),shat
!$omp threadprivate(/pmasses/)
      common/pin/pa(4),pb(4)
!$omp threadprivate(/pin/)
      common/pmom/p(4,7)
!$omp threadprivate(/pmom/)
      common/invar3/sij3(3,3),tij3(2,3)
!$omp threadprivate(/invar3/)
      common/pmomang3/pang3(4,3,1)
!$omp threadprivate(/pmomang3/)
      common/pmomang3g/pang3g(4,3,1)
!$omp threadprivate(/pmomang3g/)
      common/invarang3/sijang3(3,3,1),tijang3(2,3,1)
!$omp threadprivate(/invarang3/)
      common /tcuts/ymin,y0
      common /method/imethod
      common /Nphase/Nphase
!$omp threadprivate(/Nphase/)
      common /nwa/rm12,rg12,inwa

*
* phase space for 2->3 scattering, a+b -> 1+2+3 with q=pb-p1
*
* incoming momenta along z-axis, pa is +z
*
*
* massless case: integrates to P_3 = 1/256/pi**3*sab
*

cLIPS      print*, "phase3dis", x(1), wt, ifail

      Nphase = 1


      r1 = x(1)
      r2 = x(2)
      r3 = x(3)
      r4 = x(4)
      ifail =1
      wt = 0d0
      wtps = 1d0

      sqamin = y0*shat
      sqamax = shat
      if (sqamin.ge.sqamax) return
      call pick(1,sqa,sqamin,sqamax,r1,wtps)
      wtps = wtps/8d0/pi/shat

      q2pmin = y0*shat
      q2pmax = shat-sqa
      if (.not.(phase_unit.and.nocuts)) then
         call getExtremes_sel(id_Q2_dis, minVal=q2pmin, maxVal=q2pmax)
      end if

      if (q2pmin.ge.q2pmax) return
      if(phase_unit) then
         call pick(1,q2,q2pmin,q2pmax,r3,wtps)
      else
         if(imethod.eq.2) then
            if (inwa.eq.1) then
               call pickres(q2, q2pmin, q2pmax, rm12, rg12, r3, wtps)
            else
               call pick(3,q2,q2pmin,q2pmax,r3,wtps)
            endif
         else  !imethod 1's range in q2 too large for logarithm to adopt
            call pick(2,q2,q2pmin,q2pmax,r3,wtps)
         endif
      endif

      wtps = wtps/2d0/pi

      q2s = -q2
      call p2generic_alt(2,r4,r2,sqa,0d0,q2s,rm2(2),rm2(3),
     .                   p2g,p3g,pag,pqg,wtps)
      call disframe(sqa,0d0,q2,pa,pb,q,p1,fmat)


      do i=1,4
         p2(i) = 0d0
         p3(i) = 0d0
         v(i) = 0d0
         do j=1,4
            p2(i) = p2(i) + fmat(i,j)*p2g(j)
            p3(i) = p3(i) + fmat(i,j)*p3g(j)
            v(i) = v(i) + fmat(i,j)*pag(j)
         enddo
         v1(i) = p1(i)+p2(i)+p3(i)
      enddo

      do i=1,4
         p(i,1) = p1(i)
         p(i,2) = p2(i)
         p(i,3) = p3(i)
         pang3(i,1,1) = p1(i)
         pang3(i,2,1) = p2(i)
         pang3(i,3,1) = p3(i)
         pang3g(i,2,1) = p2g(i)
         pang3g(i,3,1) = p3g(i)
      enddo

      call fillinv(3,pang3(1:4,1:3,1),pa,pb,sijang3(1:3,1:3,1)
     .                                     ,tijang3(1:2,1:3,1))

c      write(6,101) p1(1:4)
c      write(6,102) p2(1:4)
c      write(6,103) p3(1:4)
c      write(6,105) pa(1:4)
c      write(6,106) pb(1:4)
c      write(6,109) v1(1:4)
c      write(6,*) q2/dot(q,q)

 101  format('p1:',4f14.6)
 102  format('p2:',4f14.6)
 103  format('p3:',4f14.6)
 105  format('pa:',4f14.6)
 106  format('pb:',4f14.6)
 109  format('pp:',4f14.6)

      ifail =0
      wt = wtps

      return
      end

      subroutine fillcommon3pdis
      implicit real*8(a-h,o-z)
      common/pmomang3/pang3(4,3,1)
!$omp threadprivate(/pmomang3/)
      common/pmomang3g/pang3g(4,3,1)
!$omp threadprivate(/pmomang3g/)
      common/invarang3/sijang3(3,3,1),tijang3(2,3,1)
!$omp threadprivate(/invarang3/)
      common/eventmom3p/pevt3(4,3,12)
!$omp threadprivate(/eventmom3p/)
      common/eventmom3pg/pevt3g(4,3,1)
!$omp threadprivate(/eventmom3pg/)
      common/eventinv3p/sevt3(3,3,12),tevt3(2,3,12),ievt3(12)
!$omp threadprivate(/eventinv3p/)

c---- pang3,pang3g and sija,tija are input
c---- this subroutine fills
c----       pevt3(component,pnum,ievent)             : 4-momenta of events
c----       pevt3g(component,pnum,ievent)            : 4-momenta of events in g*-pa frame
c----       sevt3(i,j,ievent),tevt3([ab],i,ievent)   : invariants of events
c----
c---- ievent = 1 fixed

      do j=1,3
         do k=1,4
            pevt3(k,j,1)    = pang3(k,j,1)
            pevt3g(k,j,1)    = pang3g(k,j,1)
         enddo
      enddo

      do j=1,3
         do k=1,3
            sevt3(j,k,1)    = sijang3(j,k,1)
         enddo
         tevt3(1,j,1)   = tijang3(1,j,1)
         tevt3(2,j,1)   = tijang3(2,j,1)
      enddo
      ievt3(1) = 1

c---- a few tests here
c      write(6,101) pevt3(1:4,1,1)
c      write(6,102) pevt3(1:4,2,1)
c      write(6,103) pevt3(1:4,3,1)
c      write(6,104) pevt3(1:4,1,1)+pevt3(1:4,2,1)+pevt3(1:4,3,1)

c      it =1
c      ta1 = -pevt3(4,1,it)+pevt3(3,1,it)
c      ta2 = -pevt3(4,2,it)+pevt3(3,2,it)
c      ta3 = -pevt3(4,3,it)+pevt3(3,3,it)
c      write(6,110) ta1/tevt3(1,1,it),ta2/tevt3(1,2,it),ta3/tevt3(1,3,it)

 101  format('p1:',4f14.6)
 102  format('p2:',4f14.6)
 103  format('p3:',4f14.6)
 104  format('p4:',4f14.6)
 110  format(3g18.11)
 111  format(2g18.11)

      return
      end


      subroutine phase4dis(x,wt,ifail)
      use KinData_mod, only : boostrest,unboostrest,rotatetoz,unrotatetoz,dot
      use DIS_mod, only: id_Q2_dis
      use Selectors_mod, only : getExtremes_sel
      use UnitPhase_mod, only : phase_unit, nocuts
      implicit real*8(a-h,o-z)
      dimension p1(4),p2(4),p3(4),p4(4),v(4),v1(4),q(4)
      dimension p1p(4),p2p(4),p3p(4),p4p(4),pap(4)
      dimension p1a(4),p2a(4),p3a(4),p4a(4)
      dimension p1g(4),p2g(4),p3g(4),p4g(4),pag(4),pqg(4)
      dimension p1ag(4),p2ag(4),p3ag(4),p4ag(4)
      dimension p1ap(4),p2ap(4),p3ap(4),p4ap(4)
      dimension p34(4),p34p(4),p34g(4),papg(4)
      dimension x(*)
      dimension bmat(4,4),ubmat(4,4),rmat(4,4),urmat(4,4),fmat(4,4)
      parameter(pi=3.141592653589793238d0)
      common/pmasses/rma2,rmb2,rm2(1:7),shat
!$omp threadprivate(/pmasses/)
      common/pin/pa(4),pb(4)
!$omp threadprivate(/pin/)
      common/pmom/p(4,7)
!$omp threadprivate(/pmom/)
      common/pmomg/pg(4,5)
!$omp threadprivate(/pmomg/)
      common/invar4/sij4(4,4),tij4(2,4)
!$omp threadprivate(/invar4/)
      common/pmomang4/pang4(4,4,2)
!$omp threadprivate(/pmomang4/)
      common/invarang4/sijang4(4,4,2),tijang4(2,4,2),iacc(2)
!$omp threadprivate(/invarang4/)
      common/pmomang4g/pang4g(4,4,2)
!$omp threadprivate(/pmomang4g/)
      common /tcuts/ymin,y0
      common /method/imethod
      common /techpar/iang
      common /Nphase/Nphase
!$omp threadprivate(/Nphase/)
      common /nwa/rm12,rg12,inwa

c---  corresponds to the phase space(3,4;a)

*
* phase space for 2->4 scattering, a+b -> 1+2+3+4 with q=pb-p1
*
* incoming momenta along z-axis, pa is +z
*
*
* massless case: integrates to P_4 = 1/24576/pi**5*sab^2
*

cLIPS      print*, "phase4dis", x(1), wt, ifail

      if(imethod.eq.1) then
         write(*,*) "imethod 1 not implemented for phase4"
         write(*,*) "Please change to imethod 2 in run card"
         stop
      endif

      wtps = 1d0

      if(iang.eq.0)then
         Nphase=3
      else
         Nphase=6
         wtps=wtps/2d0
      endif

      r1 = x(1)
      r2 = x(2)
      r3 = x(3)
      r4 = x(4)
      r5 = x(5)
      r6 = x(6)
      r7 = x(7)
      ifail =1
      wt = 0d0

      sqamin = y0*shat
      sqamax = shat
      if (sqamin.ge.sqamax) return
      call pick(1,sqa,sqamin,sqamax,r1,wtps)
      phimin = 0d0
      phimax = 2d0*pi
      call pick(1,phi,phimin,phimax,r2,wtps)
      wtps = wtps/16d0/pi**2/shat

      q2pmin = y0*shat
      q2pmax = shat-sqa
      if (.not.(phase_unit.and.nocuts)) then
         call getExtremes_sel(id_Q2_dis, minVal=q2pmin, maxVal=q2pmax)
      end if

      if (q2pmin.ge.q2pmax) return

      if(phase_unit) then
         call pick(1,q2,q2pmin,q2pmax,r3,wtps)
      else
         if (inwa.eq.1) then
            call pickres(q2, q2pmin, q2pmax, rm12, rg12, r3, wtps)
         else
            call pick(3,q2,q2pmin,q2pmax,r3,wtps)
         endif
      endif
      wtps = wtps/2d0/pi

      s34min = dmax1(y0*shat,(dsqrt(rm2(3))+dsqrt(rm2(4)))**2)
      s34max = dmin1((dsqrt(sqa)-dsqrt(rm2(2)))**2,(1d0-y0)*shat)
      if (s34min.ge.s34max) return
      call pick(12,s34,s34min,s34max,r4,wtps)
      wtps = wtps/2d0/pi
      q2s = -q2
      call p2generic_alt(2,r5,r7,sqa,0d0,q2s,rm2(2),s34,
     .                   p2g,p34g,pag,pqg,wtps)

C       costhmin = -1d0
C       costhmax = 1d0
C       call pick(1,costh,costhmin,costhmax,r6,wtps)
C      call pickcos_alt(costh,r6,wtps)
      call pickcos(costh,y0,r6,wtps)

      wtps = wtps*dsqrt(dlambda(s34,rm2(3),rm2(4)))/s34/16d0/pi
      call makeps2com(s34,costh,phi,rm2(3),rm2(4),p3p,p4p,p3ap,p4ap)

      call boostrest(p34g,bmat)
      call unboostrest(p34g,ubmat)
      do i=1,4
         v(i) = 0d0
         papg(i) = 0d0
         do j=1,4
            papg(i) = papg(i) + bmat(i,j)*pag(j)
         enddo
      enddo
      call unrotatetoz(papg,urmat)
      do i=1,4
         do j=1,4
            fmat(i,j) = 0d0
            do k=1,4
               fmat(i,j) = fmat(i,j)+ubmat(i,k)*urmat(k,j)
            enddo
         enddo
      enddo

      v1(4) = e1p
      v1(1) = 0d0
      v1(2) = 0d0
      v1(3) = e1p
      do i=1,4
         p3g(i) = 0d0
         p4g(i) = 0d0
         p3ag(i) = 0d0
         p4ag(i) = 0d0
         v(i) = 0d0
         do j=1,4
            p3g(i) = p3g(i) + fmat(i,j)*p3p(j)
            p4g(i) = p4g(i) + fmat(i,j)*p4p(j)
            p3ag(i) = p3ag(i) + fmat(i,j)*p3ap(j)
            p4ag(i) = p4ag(i) + fmat(i,j)*p4ap(j)
            v(i) = v(i) + fmat(i,j)*v1(j)
         enddo
      enddo

c ---- momenta in the gamma* p centre-of-mass frame
      do i=1,4
         pg(i,2) = p2g(i)
         pg(i,3) = p3g(i)
         pg(i,4) = p4g(i)
         p2ag(i) = p2g(i)
         pang4g(i,2,1) = p2g(i)
         pang4g(i,3,1) = p3g(i)
         pang4g(i,4,1) = p4g(i)
         pang4g(i,2,2) = p2ag(i)
         pang4g(i,3,2) = p3ag(i)
         pang4g(i,4,2) = p4ag(i)
      enddo

      call disframe(sqa,0d0,q2,pa,pb,q,p1,fmat)
      do i=1,4
         p2(i) = 0d0
         p3(i) = 0d0
         p4(i) = 0d0
         v(i) = 0d0
         p2a(i) = 0d0
         p3a(i) = 0d0
         p4a(i) = 0d0
         do j=1,4
            p2(i) = p2(i) + fmat(i,j)*p2g(j)
            p3(i) = p3(i) + fmat(i,j)*p3g(j)
            p4(i) = p4(i) + fmat(i,j)*p4g(j)
            v(i) = v(i) + fmat(i,j)*pag(j)
            p3a(i) = p3a(i) + fmat(i,j)*p3ag(j)
            p4a(i) = p4a(i) + fmat(i,j)*p4ag(j)
         enddo
         p2a(i) = p2(i)
         v1(i) = p1(i)+p2a(i)+p3a(i)+p4a(i)
      enddo
      do i=1,4
         p(i,1) = p1(i)
         p(i,2) = p2(i)
         p(i,3) = p3(i)
         p(i,4) = p4(i)
         pang4(i,1,1) = p1(i)
         pang4(i,1,2) = p1(i)
         pang4(i,2,1) = p2(i)
         pang4(i,2,2) = p2(i)
         pang4(i,3,1) = p3(i)
         pang4(i,3,2) = p3a(i)
         pang4(i,4,1) = p4(i)
         pang4(i,4,2) = p4a(i)
      enddo

c      write(6,*) s34/dot(pang4(1:4,3,2),pang4(1:4,4,2))

c      write(6,101) p1(1:4)
c      write(6,101) pa(1:4)
c      write(6,102) p2(1:4)
c      write(6,102) p2a(1:4)
c      write(6,103) p3(1:4)
c      write(6,103) p3a(1:4)
c      write(6,104) p4(1:4)
c      write(6,104) p4a(1:4)
c      write(6,105) pa(1:4)
c      write(6,106) pb(1:4)
c      write(6,109) v1(1:4)
c      write(6,*) q2/dot(q,q)

      do i=1,2
         call fillinv(4,pang4(1:4,1:4,i),pa,pb,sijang4(1:4,1:4,i)
     .                                        ,tijang4(1:2,1:4,i))
      enddo
c      write(6,*) 'h:',sijang4(2,3,1),sijang4(2,3,2)
      do i=1,2
         call acceptcuts4dis(sijang4(1:4,1:4,i),tijang4(1:2,1:4,i),
     .                       iacc(i),sqa)
      enddo
      if (iacc(1)+iacc(2).eq.0) return
      ifail =0
      wt = wtps




 101  format('p1:',4f14.9)
 102  format('p2:',4f14.9)
 103  format('p3:',4f14.9)
 104  format('p4:',4f14.9)
 105  format('pa:',4f14.9)
 106  format('pb:',4f14.9)
 109  format('pp:',4f14.9)

      return
      end


      subroutine acceptcuts4dis(sij4,tij4,ipass,sqa)
      implicit real*8(a-h,o-z)
      dimension sij4(1:4,1:4),tij4(1:2,1:4)
      common/pmasses/rma2,rmb2,rm2(1:7),shat
!$omp threadprivate(/pmasses/)
      common /tcuts/ymin,y0

      ipass=0

      sa2 = -tij4(1,2)
      sa3 = -tij4(1,3)
      sa4 = -tij4(1,4)
      s23 =  sij4(2,3)
      s24 =  sij4(2,4)
      s34 =  sij4(3,4)
      smin = min(sa2,sa3,sa4,s23,s24,s34)
      if (smin.lt.y0*sqa) return
      ipass = 0
      if (smin.eq.s34) ipass=1
      if (smin.eq.sa3.and.s34.ge.s23) ipass=1
      if (smin.eq.sa4.and.s34.ge.s24) ipass=1
      return
      end


      subroutine fillcommon4pdis
      implicit real*8(a-h,o-z)
      common/pmomang4/pang4(4,4,2)
!$omp threadprivate(/pmomang4/)
      common/pmomang4g/pang4g(4,4,2)
!$omp threadprivate(/pmomang4g/)
      common/invarang4/sijang4(4,4,2),tijang4(2,4,2),iacc(2)
!$omp threadprivate(/invarang4/)
      common/eventmom4p/pevt4(4,4,6)
!$omp threadprivate(/eventmom4p/)
      common/eventmom4pg/pevt4g(4,4,6)
!$omp threadprivate(/eventmom4pg/)
      common/eventinv4p/sevt4(4,4,6),tevt4(2,4,6),ievt4(6)
!$omp threadprivate(/eventinv4p/)
      dimension i1(1:4,1:3)

      data i1/1,2,3,4,
     .        1,3,4,2,
     .        1,4,2,3/

c---- pang4,pang4g and sija,tija are input and correspond to an event in (3,4;a)
c---- this subroutine fills
c----       pevt4(component,pnum,ievent)             : 4-momenta of events
c----       pevt4g(component,pnum,ievent)            : 4-momenta of events in g*-pa frame
c----       sevt4(i,j,ievent),tevt4([ab],i,ievent)   : invariants of events
c----
c---- ievent
c---- 1: (3,4;a)
c---- 2: (4,2;a)
c---- 3: (2,3;a)
c----
c---- 4-6: (i,j;x) with (i,j) system rotated around a-axis
c----
c---- full phave space coverage is obtained by summing all 3 events
c---- angular average is obtained by averaging:
c----        M2 = 1/2* ( sum_ievent=1..6   M2(ievent))
c----

      do i=1,3
         do j=1,4
            do k=1,4
               pevt4(k,i1(j,i),i)    = pang4(k,j,1)
               pevt4(k,i1(j,i),i+3)  = pang4(k,j,2)
               pevt4g(k,i1(j,i),i)   = pang4g(k,j,1)
               pevt4g(k,i1(j,i),i+3) = pang4g(k,j,2)
            enddo
         enddo
      enddo

      do i=1,3
         do j=1,4
            do k=1,4
               sevt4(i1(j,i),i1(k,i),i)    = sijang4(j,k,1)
               sevt4(i1(j,i),i1(k,i),i+3)  = sijang4(j,k,2)
            enddo
            tevt4(1,i1(j,i),i)    = tijang4(1,j,1)
            tevt4(2,i1(j,i),i)    = tijang4(2,j,1)
            tevt4(1,i1(j,i),i+3)  = tijang4(1,j,2)
            tevt4(2,i1(j,i),i+3)  = tijang4(2,j,2)
         enddo
         ievt4(i)   = iacc(1)
         ievt4(i+3) = iacc(2)
      enddo

c---- a few tests here
c      write(6,101) pevt4(1:4,1,1)
c      write(6,102) pevt4(1:4,2,1)
c      write(6,103) pevt4(1:4,3,1)
c      write(6,104) pevt4(1:4,4,1)
c      write(6,101) pevt4(1:4,1,4)
c      write(6,102) pevt4(1:4,2,4)
c      write(6,103) pevt4(1:4,3,4)
c      write(6,104) pevt4(1:4,4,4)
c      write(6,105) pevt4(1:4,1,4)+pevt4(1:4,2,4)
c     .            +pevt4(1:4,3,4)+pevt4(1:4,4,4)

c      it =3

c      s12 = 2d0*dot(pevt4(1,1,it),pevt4(1,2,it))
c      s13 = 2d0*dot(pevt4(1,1,it),pevt4(1,3,it))
c      s23 = 2d0*dot(pevt4(1,2,it),pevt4(1,3,it))
c      s14 = 2d0*dot(pevt4(1,1,it),pevt4(1,4,it))
c      s24 = 2d0*dot(pevt4(1,2,it),pevt4(1,4,it))
c      s34 = 2d0*dot(pevt4(1,3,it),pevt4(1,4,it))
c      ta1 = -pevt4(4,1,it)+pevt4(3,1,it)
c      ta2 = -pevt4(4,2,it)+pevt4(3,2,it)
c      ta3 = -pevt4(4,3,it)+pevt4(3,3,it)
c      ta4 = -pevt4(4,4,it)+pevt4(3,4,it)
c      tb1 = -pevt4(4,1,it)-pevt4(3,1,it)
c      tb2 = -pevt4(4,2,it)-pevt4(3,2,it)
c      tb3 = -pevt4(4,3,it)-pevt4(3,3,it)
c      tb4 = -pevt4(4,4,it)-pevt4(3,4,it)

c      it = it+3
c      write(6,110) ta1/tevt4(1,1,it),ta2/tevt4(1,2,it),ta3/tevt4(1,3,it)
c     .            ,ta4/tevt4(1,4,it)
c      write(6,110) tb1/tevt4(2,1,it),tb2/tevt4(2,2,it),tb3/tevt4(2,3,it)
c     .            ,tb4/tevt4(2,4,it)
c      write(6,110) s12/sevt4(1,2,it),s13/sevt4(1,3,it),s23/sevt4(2,3,it)
c     .            ,s14/sevt4(1,4,it)
c      write(6,111) s24/sevt4(2,4,it),s34/sevt4(3,4,it)

 101  format('p1:',4f14.6)
 102  format('p2:',4f14.6)
 103  format('p3:',4f14.6)
 104  format('p4:',4f14.6)
 105  format('p5:',4f14.6)
 110  format(4g18.11)
 111  format(2g18.11)

      return
      end



      subroutine phase5adis(x,wt,ifail)
      use KinData_mod, only : boostrest,unboostrest,rotatetoz,unrotatetoz
      use Selectors_mod, only : getExtremes_sel
      use UnitPhase_mod, only : phase_unit, nocuts
      use DIS_mod, only : PSold, id_Q2_dis
      implicit real*8(a-h,o-z)
      dimension p1(4),p2(4),p3(4),p4(4),p5(4),v(4),q(4),v1(4)
      dimension p1g(4),p2g(4),p3g(4),p4g(4),p5g(4),pag(4),pqg(4)
      dimension p1ag(4),p2ag(4),p3ag(4),p4ag(4),p5ag(4)
      dimension p1p(4),p2p(4),p3p(4),p4p(4),p5p(4),papg(4),
     .          pappg(4),paappg(4),pap(4),parb(4)
      dimension p1ap(4),p2ap(4),p3ap(4),p4ap(4),p5ap(4)
      dimension p345g(4),p45p(4),p45g(4)
      dimension p345pg(4),p45ag(4)
      dimension p45a(4),p45ap(4),p4tg(4),p5tg(4),p4tag(4),p5tag(4)
      dimension x(*)
      dimension bmat(4,4),ubmat(4,4),rmat(4,4),urmat(4,4),fmat(4,4)
      parameter(pi=3.141592653589793238d0)
      common/pmasses/rma2,rmb2,rm2(1:7),shat
!$omp threadprivate(/pmasses/)
      common/pin/pa(4),pb(4)
!$omp threadprivate(/pin/)
      common/pmom/p(4,7)
!$omp threadprivate(/pmom/)
      common/pmomg/pg(4,5)
!$omp threadprivate(/pmomg/)
      common/pmomang5g/pang5g(4,5,4)
!$omp threadprivate(/pmomang5g/)
      common/pmomang5/pang5(4,5,4)
!$omp threadprivate(/pmomang5/)
      common/invar5/sij5(5,5),tij5(2,5)
!$omp threadprivate(/invar5/)
      common/invarang5/sijang5(5,5,4),tijang5(2,5,4),iacc(4)
!$omp threadprivate(/invarang5/)
      common/tcuts/ymin,y0
      common /techpar/iang
      common /Nphase/Nphase
!$omp threadprivate(/Nphase/)
      common /nwa/rm12,rg12,inwa

* this is (3;4,5;a)
*
* phase space for 2->5 scattering, a+b -> 1+2+3+4+5 with q=pb-p1
* with sequential splitting (345) -> 3+(45) -> 3+4+5
*
* (345) system is rotated with respect to pa axis
* (45)  system is rotated with respect to pa axis
*
* default momenta             : pang(i,j,1)
* (45) rotated                : pang(i,j,2)
* (345) rotated               : pang(i,j,3)
* (345) rotated, (45) rotated : pang(i,j,4)
*
* incoming momenta along z-axis, pa is +z
*
*
* massless case: integrates to P_5 = 1/4718592/pi**7*sab^3 = 0.7016789753e-10*sab^3
*
* incoming momenta along z-axis, pa is +z
*

cLIPS      print*, "phase5adis", x(1), wt, ifail

      wtps = 1d0

      if(iang.eq.0)then
         Nphase=12
      else
         Nphase=48
         wtps=wtps/4d0
      endif


      r1 = x(1)
      r2 = x(2)
      r3 = x(3)
      r4 = x(4)
      r5 = x(5)
      r6 = x(6)
      r7 = x(7)
      r8 = x(8)
      r9 = x(9)
      r10 = x(10)
      wt = 0d0
      ifail =1

      sqamin = y0*shat
      sqamax = shat
      if (sqamin.ge.sqamax) return

      call pick(1,sqa,sqamin,sqamax,r1,wtps)
      wtps = wtps/8d0/pi/shat

      q2pmin = y0*shat
      q2pmax = shat-sqa
      if (.not.(phase_unit.and.nocuts)) then
         call getExtremes_sel(id_Q2_dis, minVal=q2pmin, maxVal=q2pmax)
      end if

      if (q2pmin.ge.q2pmax) return
      if(phase_unit) then
         call pick(1,q2,q2pmin,q2pmax,r3,wtps)
      else
         if (inwa.eq.1) then
            call pickres(q2, q2pmin, q2pmax, rm12, rg12, r3, wtps)
         else
            call pick(3,q2,q2pmin,q2pmax,r3,wtps)
         endif
      endif
      wtps = wtps/2d0/pi

      s345min = dmax1(y0*shat,
     .           (dsqrt(rm2(3))+dsqrt(rm2(4))+dsqrt(rm2(5)))**2)
      s345max = dmin1((dsqrt(sqa)-dsqrt(rm2(2)))**2,(1d0-y0)*shat)
      if (s345max.le.s345min) return
      if(PSold) then
         call pick(1,s345,s345min,s345max,r4,wtps)
      else
         call pick(2,s345,s345min,s345max,r4,wtps)
      endif

      wtps = wtps/2d0/pi
      s45min = dmax1(y0*shat,(dsqrt(rm2(4))+dsqrt(rm2(5)))**2)
      s45max = dmin1(s345,(1d0-y0)*shat)
      if (s45max.le.s45min) return
      if(PSold) then
         call pick(1,s45,s45min,s45max,r5,wtps)
      else
         call pick(2,s45,s45min,s45max,r5,wtps)
      endif
      wtps = wtps/2d0/pi
      q2s = -q2
      call p2generic_alt(2,r6,r2,sqa,0d0,q2s,rm2(2),s345,
     .                   p2g,p345g,pag,pqg,wtps)

* TODO: change of variables only works in the massless case.
*       Add a condition s.t. we return to the old calculations
*       for massive partons

***********************************************
*     TM 03/05/2015: switched to sampling
*     rat=sa3/(sa3max-sa3+y0*shat) logarithmically
*     from costh


      phimin = 0d0
      phimax = 2d0*pi
      call pick(1,phi,phimin,phimax,r8,wtps)

c      costhmin = -1d0
c      costhmax = 1d0
c      if(PSold) then
c         call pick(1,costh,costhmin,costhmax,r7,wtps)
c      else
c*     boost pa into the s345 rest frame
c         call boostrest(p345g,bmat)
c         do i=1,4
c            v(i) = 0.d0
c            pap(i) = 0.d0
c            do j=1,4
c               pap(i) = pap(i) + bmat(i,j)*pag(j)
c            enddo
c         enddo
c*     rotate pap into the frame
c         call rotatetoz(pap,rmat)
c
c         do i=1,4
c            parb(i) = 0.d0
c            do j=1,4
c               do k=1,4
c                  parb(i) = parb(i) + rmat(i,k)*bmat(k,j)*pag(j)
c               enddo
c            enddo
c         enddo
c
c         bound = 2.d0*(s345+rm2(3)-s45)/sqrt(s345)*parb(4)
c         sa3min = y0*shat
c         sa3max = min(bound,(1.d0-y0)*shat)
c         if(sa3max.le.sa3min) return
c
c*     jacobian for costh -> sa3
c         wtps = wtps/(s345+rm2(3)-s45)*sqrt(s345)/parb(4)
c
c*     rat=sa3/(sa3max-sa3+y0*shat)
c         ratmin=y0*shat/sa3max
c         ratmax=sa3max/(y0*shat)
c
c         call pick(2,rat,ratmin,ratmax,r7,wtps)
c
c         sa3=rat*(sa3max+y0*shat)/(rat+1d0)
c*      wtps=wtps*(sa3max-sa3)**2/sa3max
c         wtps=wtps*(sa3max-sa3+y0*shat)**2/(sa3max+y0*shat)
c         costh=-(sa3-(s345-s45)*parb(4)/s345**0.5)/(2d0*parb(4)*((s345-s45)/2d0/s345**0.5))
c      endif

c      call pickcos_alt(costh,r7,wtps)
      call pickcos(costh,y0,r7,wtps)

      call makeps2com(s345,costh,phi,rm2(3),s45,p3p,p45p,p3ap,p45ap)
      wtps = wtps*dsqrt(dlambda(s345,rm2(3),s45))/s345/32d0/pi**2

*******************************************************************

c---- rotate the (345) system
      call boostrest(p345g,bmat)
      call unboostrest(p345g,ubmat)
      do i=1,4
         v(i) = 0d0
         papg(i) = 0d0
         do j=1,4
            papg(i) = papg(i) + bmat(i,j)*pag(j)
         enddo
      enddo
      call unrotatetoz(papg,urmat)
      do i=1,4
         do j=1,4
            fmat(i,j) = 0d0
            do k=1,4
               fmat(i,j) = fmat(i,j)+ubmat(i,k)*urmat(k,j)
            enddo
         enddo
      enddo
      do i=1,4
         p3g(i) = 0d0
         p45g(i) = 0d0
         p3ag(i) = 0d0
         p45ag(i) = 0d0
         v(i) = 0d0
         do j=1,4
            p3g(i) = p3g(i) + fmat(i,j)*p3p(j)
            p45g(i) = p45g(i) + fmat(i,j)*p45p(j)
            p3ag(i) = p3ag(i) + fmat(i,j)*p3ap(j)
            p45ag(i) = p45ag(i) + fmat(i,j)*p45ap(j)
            v(i) = v(i) + fmat(i,j)*papg(j)
         enddo
      enddo

C      if(PSold) then
C         call pick(1,costh,costhmin,costhmax,r9,wtps)
C      else
C*****************************************************************
C*     TM 03/05/2015: switched to sampling rat=sa5/sa4
C*     from costh
C
C*     boost pa into the s45 rest frame
C         call boostrest(p45g,bmat)
C         do i=1,4
C            v(i) = 0.d0
C            pap(i) = 0.d0
C            do j=1,4
C               pap(i) = pap(i) + bmat(i,j)*pag(j)
C            enddo
C         enddo
C
C*     rotate pap into the frame
C         call rotatetoz(pap,rmat)
C         do i=1,4
C            parb(i) = 0.d0
C            do j=1,4
C               do k=1,4
C                  parb(i) = parb(i) + rmat(i,k)*bmat(k,j)*pag(j)
C               enddo
C            enddo
C         enddo
C
C         E2=2*(s45**0.5/2d0)*parb(4)
C
C         wtps = wtps/E2
C
C         ratmin=2*E2/(2*E2-y0*shat) -1d0
C         ratmax=2*E2/(y0*shat)-1d0
C
C         if(ratmin.ge.ratmax) return
C*     pick the ratio rat=sa5/sa4 logarthmically
C         call pick(2,rat,ratmin,ratmax,r9,wtps)
C
C         sa4=2*E2/(rat+1d0)
C         sa5=2*E2-sa4
C
C         wtps=wtps*sa4**2/(2*E2)
C         costh=-(sa4-2*(s45**0.5)*parb(4)/2d0)/(2*(s45**0.5)*parb(4)/2d0)
C      endif
      call pickcos_alt(costh,r9,wtps)

      call pick(1,phi,phimin,phimax,r10,wtps)
      call makeps2com(s45,costh,phi,rm2(4),rm2(5),p4p,p5p,p4ap,p5ap)
      wtps = wtps*dsqrt(dlambda(s45,rm2(4),rm2(5)))/s45/32d0/pi**2

*************************************************


c---- rotate the (45) system
      call boostrest(p45g,bmat)
      call unboostrest(p45g,ubmat)
      do i=1,4
         v(i) = 0d0
         pappg(i) = 0d0
         do j=1,4
            pappg(i) = pappg(i) + bmat(i,j)*pag(j)
         enddo
      enddo
      call unrotatetoz(pappg,urmat)
      do i=1,4
         do j=1,4
            fmat(i,j) = 0d0
            do k=1,4
               fmat(i,j) = fmat(i,j)+ubmat(i,k)*urmat(k,j)
            enddo
         enddo
      enddo
      do i=1,4
         p4g(i) = 0d0
         p5g(i) = 0d0
         p4ag(i) = 0d0
         p5ag(i) = 0d0
         v(i) = 0d0
         do j=1,4
            p4g(i) = p4g(i) + fmat(i,j)*p4p(j)
            p5g(i) = p5g(i) + fmat(i,j)*p5p(j)
            p4ag(i) = p4ag(i) + fmat(i,j)*p4ap(j)
            p5ag(i) = p5ag(i) + fmat(i,j)*p5ap(j)
            v(i) = v(i) + fmat(i,j)*papg(j)
         enddo
      enddo

c---- rotate the (45ang) system
      call boostrest(p45ag,bmat)
      call unboostrest(p45ag,ubmat)
      do i=1,4
         v(i) = 0d0
         paappg(i) = 0d0
         do j=1,4
            paappg(i) = paappg(i) + bmat(i,j)*pag(j)
         enddo
      enddo
      call unrotatetoz(paappg,urmat)
      do i=1,4
         do j=1,4
            fmat(i,j) = 0d0
            do k=1,4
               fmat(i,j) = fmat(i,j)+ubmat(i,k)*urmat(k,j)
            enddo
         enddo
      enddo
      do i=1,4
         p4tg(i) = 0d0
         p5tg(i) = 0d0
         p4tag(i) = 0d0
         p5tag(i) = 0d0
         v(i) = 0d0
         do j=1,4
            p4tg(i) = p4tg(i) + fmat(i,j)*p4p(j)
            p5tg(i) = p5tg(i) + fmat(i,j)*p5p(j)
            p4tag(i) = p4tag(i) + fmat(i,j)*p4ap(j)
            p5tag(i) = p5tag(i) + fmat(i,j)*p5ap(j)
            v(i) = v(i) + fmat(i,j)*papg(j)
         enddo
      enddo

      do i=1,4
         pg(i,2) = p2g(i)
         pg(i,3) = p3g(i)
         pg(i,4) = p4g(i)
         pg(i,5) = p5g(i)
         pang5g(i,2,1) = p2g(i)
         pang5g(i,3,1) = p3g(i)
         pang5g(i,4,1) = p4g(i)
         pang5g(i,5,1) = p5g(i)
         pang5g(i,2,2) = p2g(i)
         pang5g(i,3,2) = p3g(i)
         pang5g(i,4,2) = p4ag(i)
         pang5g(i,5,2) = p5ag(i)
         pang5g(i,2,3) = p2g(i)
         pang5g(i,3,3) = p3ag(i)
         pang5g(i,4,3) = p4tg(i)
         pang5g(i,5,3) = p5tg(i)
         pang5g(i,2,4) = p2g(i)
         pang5g(i,3,4) = p3ag(i)
         pang5g(i,4,4) = p4tag(i)
         pang5g(i,5,4) = p5tag(i)
      enddo

      call disframe(sqa,0d0,q2,pa,pb,q,p1,fmat)
      do k1=1,4
         do i=1,4
            pang5(i,1,k1) = p1(i)
         enddo
         do k2=2,5
            do i=1,4
               pang5(i,k2,k1) = 0d0
               do j=1,4
                 pang5(i,k2,k1)=pang5(i,k2,k1)+fmat(i,j)*pang5g(j,k2,k1)
               enddo
            enddo
         enddo
      enddo

      do i=1,4
         p2(i) = pang5(i,2,1)
         p3(i) = pang5(i,3,1)
         p4(i) = pang5(i,4,1)
         p5(i) = pang5(i,5,1)
      enddo

      k10 = 4
      do i=1,4
         do k=1,5
            p(i,k) = pang5(i,k,1)
         enddo
         v1(i) = pang5(i,1,k10)+pang5(i,2,k10)+pang5(i,3,k10)
     .          +pang5(i,4,k10)+pang5(i,5,k10)
      enddo

      do i=1,4
         call fillinv(5,pang5(1:4,1:5,i),pa,pb,sijang5(1:5,1:5,i)
     .                                        ,tijang5(1:2,1:5,i))
      enddo

      do i=1,4
        call acceptcuts5adis(sijang5(1:5,1:5,i),tijang5(1:2,1:5,i),
     .                       iacc(i),sqa)
      enddo
      if (iacc(1)+iacc(2)+iacc(3)+iacc(4).eq.0) return
      ifail =0
      wt = wtps

c      write(6,101) pang5(1:4,1,k10)
c      write(6,102) pang5(1:4,2,k10)
c      write(6,103) pang5(1:4,3,k10)
c      write(6,104) pang5(1:4,4,k10)
c      write(6,105) pang5(1:4,5,k10)
c      write(6,106) pa(1:4)
c      write(6,107) pb(1:4)
c      write(6,109) v1(1:4)
c      write(6,*) q2/dot(q,q)

 101  format('p1:',4f14.9)
 102  format('p2:',4f14.9)
 103  format('p3:',4f14.9)
 104  format('p4:',4f14.9)
 105  format('p5:',4f14.9)
 106  format('pa:',4f14.9)
 107  format('pb:',4f14.9)
 109  format('pp:',4f14.9)

      return
      end



      subroutine acceptcuts5adis(sij5,tij5,ipass,sqa)
      implicit real*8(a-h,o-z)
      dimension sij5(1:5,1:5),tij5(1:2,1:5)
      common/pmasses/rma2,rmb2,rm2(1:7),shat
!$omp threadprivate(/pmasses/)
      common /tcuts/ymin,y0

      ipass = 0
      sa2 = -tij5(1,2)
      sa3 = -tij5(1,3)
      sa4 = -tij5(1,4)
      sa5 = -tij5(1,5)
      s23 =  sij5(2,3)
      s24 =  sij5(2,4)
      s25 =  sij5(2,5)
      s34 =  sij5(3,4)
      s35 =  sij5(3,5)
      s45 =  sij5(4,5)

      if (min(sa2,sa3,sa4,sa5).lt.y0*sqa) return
      if (min(s23,s24,s25,s34,s35,s45).lt.y0*sqa) return

      smin1 = sqa
      smin2 = sqa

      do i=2,5
         do j=i+1,5
            if (sij5(i,j).lt.smin2) then
               smin2 = sij5(i,j)
               if (smin2.lt.smin1) then
                  stmp  = smin1
                  smin1 = smin2
                  smin2 = stmp
               endif
            endif
         enddo
         if (-tij5(1,i).lt.smin2) then
            smin2 = -tij5(1,i)
            if (smin2.lt.smin1) then
               stmp  = smin1
               smin1 = smin2
               smin2 = stmp
            endif
         endif
      enddo

      s245 = s24+s25+s45
      s345 = s34+s35+s45
      if (smin1.eq.s45.and.smin2.eq.min(s34,s35)) ipass=1
      if (s345.le.s245) then
         if (smin1.eq.sa4.and.smin2.eq.sa5) ipass=1
         if (smin1.eq.sa5.and.smin2.eq.sa4) ipass=1
         if (smin1.eq.s45.and.smin2.eq.sa4) ipass=1
         if (smin1.eq.sa4.and.smin2.eq.s45) ipass=1
         if (smin1.eq.s45.and.smin2.eq.sa5) ipass=1
         if (smin1.eq.sa5.and.smin2.eq.s45) ipass=1
      endif

      return
      end


      subroutine phase5bdis(x,wt,ifail)
      use KinData_mod, only : boostrest,unboostrest,rotatetoz,unrotatetoz
      use Selectors_mod, only : getExtremes_sel
      use UnitPhase_mod, only : phase_unit, nocuts
      use DIS_mod, only : PSold, id_Q2_dis
      implicit real*8(a-h,o-z)
      dimension p1(4),p2(4),p3(4),p4(4),p5(4),v(4),q(4),v1(4)
      dimension p1g(4),p2g(4),p3g(4),p4g(4),p5g(4),pag(4),pqg(4)
      dimension p1a(4),p2a(4),p3a(4),p4a(4),p5a(4)
      dimension p1ag(4),p2ag(4),p3ag(4),p4ag(4),p5ag(4)
      dimension p23g(4),p45g(4)
      dimension p1b(4),p2b(4),p3b(4),p4b(4),p5b(4)
      dimension p1c(4),p2c(4),p3c(4),p4c(4),p5c(4)
      dimension p23(4),p45(4),p45a(4),p45p(4),p45ap(4)
      dimension p1p(4),p2p(4),p3p(4),p4p(4),p5p(4),pap(4),pbp(4),
     .          p1ap(4),p2ap(4),p3ap(4),p4ap(4),p5ap(4),pap1(4),parb(4)
      dimension papg(4)
      dimension x(*)
      dimension bmat(4,4),ubmat(4,4),rmat(4,4),urmat(4,4),fmat(4,4)
      parameter(pi=3.141592653589793238d0)
      common/pmasses/rma2,rmb2,rm2(1:7),shat
!$omp threadprivate(/pmasses/)
      common/pin/pa(4),pb(4)
!$omp threadprivate(/pin/)
      common/pmom/p(4,7)
!$omp threadprivate(/pmom/)
      common/pmomg/pg(4,5)
!$omp threadprivate(/pmomg/)
      common/pmomang5/pang5(4,5,4)
!$omp threadprivate(/pmomang5/)
      common/pmomang5g/pang5g(4,5,4)
!$omp threadprivate(/pmomang5g/)
      common/invar5/sij5(5,5),tij5(2,5)
!$omp threadprivate(/invar5/)
      common/invarang5/sijang5(5,5,4),tijang5(2,5,4),iacc(4)
!$omp threadprivate(/invarang5/)
      common /tcuts/ymin,y0
      common /techpar/iang
      common /Nphase/Nphase
!$omp threadprivate(/Nphase/)
      common /nwa/rm12,rg12,inwa

* This is (2,3;a;4,5;a)
*
* phase space for 2->5 scattering, a+b -> 1+2+3+4+5 with q = pb-p1
* with splittings (23) -> 2+3, (45) -> 4+5
* (23) system is rotated with respect to pa axis
* (45) system is rotated with respect to pa axis
*
* default momenta             : pang(i,j,1)
* (45) rotated                : pang(i,j,2)
* (23) rotated                : pang(i,j,3)
* (23) rotated, (45) rotated  : pang(i,j,4)
*
* incoming momenta along z-axis, pa is +z
*
*
* massless case: integrates to P_5 = 1/4718592/pi**7*sab^3 = 0.7016789753e-10*sab^3
*

cLIPS      print*, "phase5bdis", x(1), wt, ifail

      wtps = 1d0

      if(iang.eq.0)then
         Nphase=3
      else
         Nphase=12
         wtps=wtps/4d0
      endif

      r1 = x(1)
      r2 = x(2)
      r3 = x(3)
      r4 = x(4)
      r5 = x(5)
      r6 = x(6)
      r7 = x(7)
      r8 = x(8)
      r9 = x(9)
      r10 = x(10)
      wt = 0d0
      ifail =1


      sqamin = y0*shat
      sqamax = shat
      if (sqamin.ge.sqamax) return
      call pick(1,sqa,sqamin,sqamax,r1,wtps)
      wtps = wtps/8d0/pi/shat

      q2pmin = y0*shat
      q2pmax = shat-sqa
      if (.not.(phase_unit.and.nocuts)) then
         call getExtremes_sel(id_Q2_dis, minVal=q2pmin, maxVal=q2pmax)
      end if

      if (q2pmin.ge.q2pmax) return
      if(.not. phase_unit) then
         if (inwa.eq.1) then
            call pickres(q2, q2pmin, q2pmax, rm12, rg12, r3, wtps)
         else
            call pick(3,q2,q2pmin,q2pmax,r3,wtps)
         endif
      else
         call pick(1,q2,q2pmin,q2pmax,r3,wtps)
      endif

      wtps = wtps/2d0/pi

      s23min = dmax1(y0*shat,(dsqrt(rm2(2))+dsqrt(rm2(3)))**2)
      s23max = dmin1((dsqrt(sqa)-dsqrt(rm2(4))-dsqrt(rm2(5)))**2
     .              ,(1d0-5d0*y0)*shat)
      if (s23min.ge.s23max) return
      if(PSold) then
         call pick(1,s23,s23min,s23max,r4,wtps)
      else
         call pick(2,s23,s23min,s23max,r4,wtps)
      endif

      wtps = wtps/2d0/pi
      s45min = dmax1(y0*shat,(dsqrt(rm2(4))+dsqrt(rm2(5)))**2)
      s45max = dmin1((dsqrt(sqa)-dsqrt(s23))**2,(1d0-5d0*y0)*shat)
      if (s45min.ge.s45max) return
      if(PSold) then
         call pick(1,s45,s45min,s45max,r5,wtps)
      else
         call pick(12,s45,s45min,s45max,r5,wtps)
      endif
      wtps = wtps/2d0/pi
      q2s = -q2
      call p2generic_alt(2,r6,r2,sqa,0d0,q2s,s23,s45,
     .                   p23g,p45g,pag,pqg,wtps)


***********************************************

      phimin = 0d0
      phimax = 2d0*pi
      call pick(1,phi,phimin,phimax,r8,wtps)

C      costhmin = -1d0
C      costhmax = 1d0
C      if(PSold) then
C         call pick(1,costh,costhmin,costhmax,r7,wtps)
C      else
C***********************************************
C*     switched to sampling rat=sa3/sa2 from costh
C
C*     boost pb into the s23 rest frame
C         call boostrest(p23g,bmat)
C         do i=1,4
C            v(i) = 0.d0
C            pap1(i) = 0.d0
C            do j=1,4
C               pap1(i) = pap1(i) + bmat(i,j)*pag(j)
C            enddo
C         enddo
C
C*     rotate pbp into the frame
C         call rotatetoz(pap1,rmat)
C         do i=1,4
C            parb(i) = 0.d0
C            do j=1,4
C               do k=1,4
C                  parb(i) = parb(i) + rmat(i,k)*bmat(k,j)*pag(j)
C               enddo
C            enddo
C         enddo
C
C         E2=2*(s23**0.5/2d0)*parb(4)
C         wtps = wtps/E2
C
C         ratmin=2*E2/(2*E2-y0*shat) -1d0
C         ratmax=2*E2/(y0*shat)-1d0
C
C*     pick the ratio rat=sb5/sb4 logarthmically
C         call pick(2,rat,ratmin,ratmax,r7,wtps)
C
C         sa2=2*E2/(rat+1d0)
C         sa3=2*E2-sa2
C
C         wtps=wtps*sa2**2/(2*E2)
C         costh=-(sa2-2*(s23**0.5)*parb(4)/2d0)/(2*(s23**0.5)*parb(4)/2d0)
C      endif

c      call pickcos_alt(costh,r7,wtps)
      call pickcos(costh,y0,r7,wtps)

      call makeps2com(s23,costh,phi,rm2(2),rm2(3),p2p,p3p,p2ap,p3ap)
      wtps = wtps*dsqrt(dlambda(s23,rm2(2),rm2(3)))/s23/32d0/pi**2


      call pick(1,phi,phimin,phimax,r10,wtps)

C      if(PSold) then
C         call pick(1,costh,costhmin,costhmax,r9,wtps)
C      else
C***********************************************
C*     TM 03/05/2015: switched to sampling rat=sa5/sa4
C*     from costh
C
C*     boost pb into the s45 rest frame
C         call boostrest(p45g,bmat)
C         do i=1,4
C            v(i) = 0.d0
C            pap1(i) = 0.d0
C            do j=1,4
C               pap1(i) = pap1(i) + bmat(i,j)*pag(j)
C            enddo
C         enddo
C
C*     rotate pbp into the frame
C         call rotatetoz(pap1,rmat)
C         do i=1,4
C            parb(i) = 0.d0
C            do j=1,4
C               do k=1,4
C                  parb(i) = parb(i) + rmat(i,k)*bmat(k,j)*pag(j)
C               enddo
C            enddo
C         enddo
C
C         E2=2*(s45**0.5/2d0)*parb(4)
C         wtps = wtps/E2
C
C         ratmin=2*E2/(2*E2-y0*shat) -1d0
C         ratmax=2*E2/(y0*shat)-1d0
C
C         if(ratmin.ge.ratmax) return
C*     pick the ratio rat=sb5/sb4 logarthmically
C         call pick(2,rat,ratmin,ratmax,r9,wtps)
C
C         sa4=2*E2/(rat+1d0)
C         sa5=2*E2-sa4
C
C         wtps=wtps*sa4**2/(2*E2)
C         costh=-(sa4-2*(s45**0.5)*parb(4)/2d0)/(2*(s45**0.5)*parb(4)/2d0)
C
C*************************************************************************
C      endif

c      call pickcos_alt(costh,r9,wtps)
      call pickcos(costh,y0,r9,wtps)

      call makeps2com(s45,costh,phi,rm2(4),rm2(5),p4p,p5p,p4ap,p5ap)
      wtps = wtps*dsqrt(dlambda(s45,rm2(4),rm2(5)))/s45/32d0/pi**2

      call boostrest(p23g,bmat)
      call unboostrest(p23g,ubmat)
      do i=1,4
         v(i) = 0d0
         papg(i) = 0d0
         do j=1,4
            papg(i) = papg(i) + bmat(i,j)*pag(j)
         enddo
      enddo
      call unrotatetoz(papg,urmat)
      do i=1,4
         do j=1,4
            fmat(i,j) = 0d0
            do k=1,4
               fmat(i,j) = fmat(i,j)+ubmat(i,k)*urmat(k,j)
            enddo
         enddo
      enddo
      do i=1,4
         p2g(i) = 0d0
         p3g(i) = 0d0
         p2ag(i) = 0d0
         p3ag(i) = 0d0
         v(i) = 0d0
         do j=1,4
            p2g(i) = p2g(i) + fmat(i,j)*p2p(j)
            p3g(i) = p3g(i) + fmat(i,j)*p3p(j)
            p2ag(i) = p2ag(i) + fmat(i,j)*p2ap(j)
            p3ag(i) = p3ag(i) + fmat(i,j)*p3ap(j)
         enddo
      enddo

      call boostrest(p45g,bmat)
      call unboostrest(p45g,ubmat)
      do i=1,4
         papg(i) = 0d0
         do j=1,4
            papg(i) = papg(i) + bmat(i,j)*pag(j)
         enddo
      enddo
      call unrotatetoz(papg,urmat)
      do i=1,4
         do j=1,4
            fmat(i,j) = 0d0
            do k=1,4
               fmat(i,j) = fmat(i,j)+ubmat(i,k)*urmat(k,j)
            enddo
         enddo
      enddo
      do i=1,4
         p4g(i) = 0d0
         p5g(i) = 0d0
         p4ag(i) = 0d0
         p5ag(i) = 0d0
         v(i) = 0d0
         do j=1,4
            p4g(i) = p4g(i) + fmat(i,j)*p4p(j)
            p5g(i) = p5g(i) + fmat(i,j)*p5p(j)
            p4ag(i) = p4ag(i) + fmat(i,j)*p4ap(j)
            p5ag(i) = p5ag(i) + fmat(i,j)*p5ap(j)
         enddo
      enddo

      do i=1,4
         pg(i,2) = p2g(i)
         pg(i,3) = p3g(i)
         pg(i,4) = p4g(i)
         pg(i,5) = p5g(i)
         pang5g(i,2,1) = p2g(i)
         pang5g(i,3,1) = p3g(i)
         pang5g(i,4,1) = p4g(i)
         pang5g(i,5,1) = p5g(i)
         pang5g(i,2,2) = p2g(i)
         pang5g(i,3,2) = p3g(i)
         pang5g(i,4,2) = p4ag(i)
         pang5g(i,5,2) = p5ag(i)
         pang5g(i,2,3) = p2ag(i)
         pang5g(i,3,3) = p3ag(i)
         pang5g(i,4,3) = p4g(i)
         pang5g(i,5,3) = p5g(i)
         pang5g(i,2,4) = p2ag(i)
         pang5g(i,3,4) = p3ag(i)
         pang5g(i,4,4) = p4ag(i)
         pang5g(i,5,4) = p5ag(i)
      enddo

      call disframe(sqa,0d0,q2,pa,pb,q,p1,fmat)
      do k1=1,4
         do i=1,4
            pang5(i,1,k1) = p1(i)
         enddo
         do k2=2,5
            do i=1,4
               pang5(i,k2,k1) = 0d0
               do j=1,4
                 pang5(i,k2,k1)=pang5(i,k2,k1)+fmat(i,j)*pang5g(j,k2,k1)
               enddo
            enddo
         enddo
      enddo

      k10 = 4
      do i=1,4
         do k=1,5
            p(i,k) = pang5(i,k,1)
         enddo
         v1(i) = pang5(i,1,k10)+pang5(i,2,k10)+pang5(i,3,k10)
     .          +pang5(i,4,k10)+pang5(i,5,k10)
      enddo

      do i=1,4
         call fillinv(5,pang5(1:4,1:5,i),pa,pb,sijang5(1:5,1:5,i)
     .                                        ,tijang5(1:2,1:5,i))
      enddo

      do i=1,4
        call acceptcuts5bdis(sijang5(1:5,1:5,i),tijang5(1:2,1:5,i),
     .                       iacc(i),sqa)
      enddo
      if (iacc(1)+iacc(2)+iacc(3)+iacc(4).eq.0) return
      ifail =0
      wt = wtps



      ifail =0
      wt = wtps

c      write(6,101) pang5(1:4,1,k10)
c      write(6,102) pang5(1:4,2,k10)
c      write(6,103) pang5(1:4,3,k10)
c      write(6,104) pang5(1:4,4,k10)
c      write(6,105) pang5(1:4,5,k10)
c      write(6,106) pa(1:4)
c      write(6,107) pb(1:4)
c      write(6,109) v1(1:4)
c      write(6,*) q2/dot(q,q)

 101  format('p1:',4f14.9)
 102  format('p2:',4f14.9)
 103  format('p3:',4f14.9)
 104  format('p4:',4f14.9)
 105  format('p5:',4f14.9)
 106  format('pa:',4f14.9)
 107  format('pb:',4f14.9)
 109  format('pp:',4f14.9)

 120  format(4e19.11)
      return
      end


      subroutine acceptcuts5bdis(sij5,tij5,ipass,sqa)
      implicit real*8(a-h,o-z)
      dimension sij5(1:5,1:5),tij5(1:2,1:5)
      common/pmasses/rma2,rmb2,rm2(1:7),shat
!$omp threadprivate(/pmasses/)
      common /tcuts/ymin,y0

      ipass = 0

      sa2 = -tij5(1,2)
      sa3 = -tij5(1,3)
      sa4 = -tij5(1,4)
      sa5 = -tij5(1,5)
      s23 =  sij5(2,3)
      s24 =  sij5(2,4)
      s25 =  sij5(2,5)
      s34 =  sij5(3,4)
      s35 =  sij5(3,5)
      s45 =  sij5(4,5)

      if (min(sa2,sa3,sa4,sa5).lt.y0*sqa) return
      if (min(s23,s24,s25,s34,s35,s45).lt.y0*sqa) return

      smin1 = sqa
      smin2 = sqa

      do i=2,5
         do j=i+1,5
            if (sij5(i,j).lt.smin2) then
               smin2 = sij5(i,j)
               if (smin2.lt.smin1) then
                  stmp  = smin1
                  smin1 = smin2
                  smin2 = stmp
               endif
            endif
         enddo
         if (-tij5(1,i).lt.smin2) then
            smin2 = -tij5(1,i)
            if (smin2.lt.smin1) then
               stmp  = smin1
               smin1 = smin2
               smin2 = stmp
            endif
         endif
      enddo
      if (smin1.eq.s23.and.smin2.eq.s45) ipass=1
      if (smin2.eq.s23.and.smin1.eq.s45) ipass=1
      if (smin1.eq.s45.and.(smin2.eq.sa2.or.smin2.eq.sa3)) ipass=1
      if (smin2.eq.s45.and.(smin1.eq.sa2.or.smin1.eq.sa3)) ipass=1
      if (smin1.eq.s23.and.(smin2.eq.sa4.or.smin2.eq.sa5)) ipass=1
      if (smin2.eq.s23.and.(smin1.eq.sa4.or.smin1.eq.sa5)) ipass=1

      return
      end

      subroutine fillcommon5apdis
      implicit real*8(a-h,o-z)
      common/pmomang5/pang5(4,5,4)
!$omp threadprivate(/pmomang5/)
      common/pmomang5g/pang5g(4,5,4)
!$omp threadprivate(/pmomang5g/)
      common/invarang5/sijang5(5,5,4),tijang5(2,5,4),iacc(4)
!$omp threadprivate(/invarang5/)
      common/eventmom5ap/pevt5a(4,5,48)
!$omp threadprivate(/eventmom5ap/)
      common/eventmom5apg/pevt5ag(4,5,48)
!$omp threadprivate(/eventmom5apg/)
      common/eventinv5ap/sevt5a(5,5,48),tevt5a(2,5,48),ievt5a(48)
!$omp threadprivate(/eventinv5ap/)
      dimension i1(1:5,1:12)

      data i1/1,2,3,4,5,
     .        1,3,2,4,5,
     .        1,4,2,3,5,
     .        1,2,4,3,5,
     .        1,5,2,3,4,
     .        1,2,5,3,4,
     .        1,4,3,2,5,
     .        1,3,4,2,5,
     .        1,5,3,2,4,
     .        1,3,5,2,4,
     .        1,5,4,2,3,
     .        1,4,5,2,3/
c---- pang5,pang5g and sijang are input and correspond to an event in (3;4,5;a)
c---- this subroutine fills
c----       pevt5a(ievent,component,pnum)            : 4-momenta of events
c----       pevt5ag(ievent,component,pnum)           : 4-momenta of events in g*-pa frame
c----       sevt5a(i,j,ievent),tevt5a([ab],i,ievent) : invariants of events
c----
c----  1: (3;45;a)
c----  2: (2;45;a)
c----  3: (2;35;a)
c----  4: (4;35;a)
c----  5: (2;34;a)
c----  6: (5;34;a)
c----  7: (3;25;a)
c----  8: (4;25;a)
c----  9: (3;24;a)
c---- 10: (5;24;a)
c---- 11: (4;23;a)
c---- 12: (5;23;a)

c---- 13-24: with (45) system rotated
c---- 25-36: with (345) system rotated
c---- 37-48: with (45) and (345) system rotated

c---- full phave space coverage is obtained by summing ievent=1..12
c---- angular average is obtained by averaging:
c----        M2 = 1/4* ( sum_ievent=1..48   M2(ievent))
c----

      do i=1,12
         do j=1,5
            do k=1,4
               pevt5a(k,i1(j,i),i)     = pang5(k,j,1)
               pevt5a(k,i1(j,i),i+12)  = pang5(k,j,2)
               pevt5a(k,i1(j,i),i+24)  = pang5(k,j,3)
               pevt5a(k,i1(j,i),i+36)  = pang5(k,j,4)
               pevt5ag(k,i1(j,i),i)    = pang5g(k,j,1)
               pevt5ag(k,i1(j,i),i+12) = pang5g(k,j,2)
               pevt5ag(k,i1(j,i),i+24) = pang5g(k,j,3)
               pevt5ag(k,i1(j,i),i+36) = pang5g(k,j,4)
            enddo
         enddo
      enddo

      do i=1,12
         do j=1,5
            do k=1,5
               sevt5a(i1(j,i),i1(k,i),i)    = sijang5(j,k,1)
               sevt5a(i1(j,i),i1(k,i),i+12) = sijang5(j,k,2)
               sevt5a(i1(j,i),i1(k,i),i+24) = sijang5(j,k,3)
               sevt5a(i1(j,i),i1(k,i),i+36) = sijang5(j,k,4)
            enddo
            tevt5a(1,i1(j,i),i)    = tijang5(1,j,1)
            tevt5a(2,i1(j,i),i)    = tijang5(2,j,1)
            tevt5a(1,i1(j,i),i+12) = tijang5(1,j,2)
            tevt5a(2,i1(j,i),i+12) = tijang5(2,j,2)
            tevt5a(1,i1(j,i),i+24) = tijang5(1,j,3)
            tevt5a(2,i1(j,i),i+24) = tijang5(2,j,3)
            tevt5a(1,i1(j,i),i+36) = tijang5(1,j,4)
            tevt5a(2,i1(j,i),i+36) = tijang5(2,j,4)
         enddo
         ievt5a(i)    = iacc(1)
         ievt5a(i+12) = iacc(2)
         ievt5a(i+24) = iacc(3)
         ievt5a(i+36) = iacc(4)
      enddo

c---- a few tests here
c      it = 1
c      write(6,101) pevt4b(1:4,1,it)
c      write(6,102) pevt4b(1:4,2,it)
c      write(6,103) pevt4b(1:4,3,it)
c      write(6,104) pevt4b(1:4,4,it)
c      write(6,101) pevt4b(1:4,1,it+1)
c      write(6,102) pevt4b(1:4,2,it+1)
c      write(6,103) pevt4b(1:4,3,it+1)
c      write(6,104) pevt4b(1:4,4,it+1)
c      write(6,101) pevt4b(1:4,1,it+2)
c      write(6,102) pevt4b(1:4,2,it+2)
c      write(6,103) pevt4b(1:4,3,it+2)
c      write(6,104) pevt4b(1:4,4,it+2)
c      write(6,101) pevt4b(1:4,1,it+3)
c      write(6,102) pevt4b(1:4,2,it+3)
c      write(6,103) pevt4b(1:4,3,it+3)
c      write(6,104) pevt4b(1:4,4,it+3)
c      write(6,101) pevt4b(1:4,1,it+4)
c      write(6,102) pevt4b(1:4,2,it+4)
c      write(6,103) pevt4b(1:4,3,it+4)
c      write(6,104) pevt4b(1:4,4,it+4)
c      write(6,101) pevt4b(1:4,1,it+5)
c      write(6,102) pevt4b(1:4,2,it+5)
c      write(6,103) pevt4b(1:4,3,it+5)
c      write(6,104) pevt4b(1:4,4,it+5)
c      write(6,*)
c      do it =1,12
c         write(6,*) it
c      s23 = 2d0*dot(pevt5a(1,2,it),pevt5a(1,3,it))
c      s24 = 2d0*dot(pevt5a(1,2,it),pevt5a(1,4,it))
c      s25 = 2d0*dot(pevt5a(1,2,it),pevt5a(1,5,it))
c      s34 = 2d0*dot(pevt5a(1,3,it),pevt5a(1,4,it))
c      s35 = 2d0*dot(pevt5a(1,3,it),pevt5a(1,5,it))
c      s45 = 2d0*dot(pevt5a(1,4,it),pevt5a(1,5,it))
c      ta2 = -pevt5a(4,2,it)+pevt5a(3,2,it)
c      ta3 = -pevt5a(4,3,it)+pevt5a(3,3,it)
c      ta4 = -pevt5a(4,4,it)+pevt5a(3,4,it)
c      ta5 = -pevt5a(4,5,it)+pevt5a(3,5,it)
c      tb2 = -pevt5a(4,2,it)-pevt5a(3,2,it)
c      tb3 = -pevt5a(4,3,it)-pevt5a(3,3,it)
c      tb4 = -pevt5a(4,4,it)-pevt5a(3,4,it)
c      tb5 = -pevt5a(4,5,it)-pevt5a(3,5,it)

c      it2 = it+12
c      write(6,107) ta2/tevt5a(1,2,it2),ta3/tevt5a(1,3,it2),
c     .             ta4/tevt5a(1,4,it2),ta5/tevt5a(1,5,it2)
c      write(6,107) tb2/tevt5a(2,2,it2),tb3/tevt5a(2,3,it2),
c     .             tb4/tevt5a(2,4,it2),tb5/tevt5a(2,5,it2)
c      write(6,108) s23/sevt5a(2,3,it2),s24/sevt5a(2,4,it2),
c     .             s34/sevt5a(3,4,it2)
c      write(6,107) s25/sevt5a(2,5,it2),s35/sevt5a(3,5,it2),
c     .             s45/sevt5a(4,5,it2),
c     .   (s34+s35+s45)/(sevt5a(3,4,it2)+sevt5a(3,5,it2)+sevt5a(4,5,it2))
c      write(6,*)
c      enddo


 101  format('p1:',4f14.6)
 102  format('p2:',4f14.6)
 103  format('p3:',4f14.6)
 104  format('p4:',4f14.6)
 107  format(4g18.13)
 108  format(3g18.13)


      return
      end




      subroutine fillcommon5bpdis
      implicit real*8(a-h,o-z)
      common/pmomang5/pang5(4,5,4)
!$omp threadprivate(/pmomang5/)
      common/pmomang5g/pang5g(4,5,4)
!$omp threadprivate(/pmomang5g/)
      common/invarang5/sijang5(5,5,4),tijang5(2,5,4),iacc(4)
!$omp threadprivate(/invarang5/)
      common/eventmom5bp/pevt5b(4,5,24)
!$omp threadprivate(/eventmom5bp/)
      common/eventmom5bpg/pevt5bg(4,5,24)
!$omp threadprivate(/eventmom5bpg/)
      common/eventinv5bp/sevt5b(5,5,24),tevt5b(2,5,24),ievt5b(24)
!$omp threadprivate(/eventinv5bp/)
      dimension i1(1:5,1:3)

      data i1/1,2,3,4,5,
     .        1,2,4,3,5,
     .        1,2,5,3,4/

c---- pang5,pang5g and sijang are input and correspond to an event in (2,3;a;4,5;a)
c---- this subroutine fills
c----       pevt5b(ievent,component,pnum)            : 4-momenta of events
c----       pevt5bg(ievent,component,pnum)           : 4-momenta of events in g*-pa frame
c----       sevt5b(i,j,ievent),tevt5b([ab],i,ievent) : invariants of events
c----
c---- ievent
c---- 1: (2,3;a;4,5;a)
c---- 2: (2,4;a;3,5;a)
c---- 3: (2,5;a;3,4;a)
c----  4- 6: with (45) system rotated
c----  7- 9: with (23) system rotated
c---- 10-12: with (23) and (45) system rotated

c---- full phave space coverage is obtained by summing ievent=1..3
c---- angular average is obtained by averaging:
c----        M2 = 1/4* ( sum_ievent=1..12   M2(ievent))
c----

      do i=1,3
         do j=1,5
            do k=1,4
               pevt5b(k,i1(j,i),i)    = pang5(k,j,1)
               pevt5b(k,i1(j,i),i+3)  = pang5(k,j,2)
               pevt5b(k,i1(j,i),i+6)  = pang5(k,j,3)
               pevt5b(k,i1(j,i),i+9)  = pang5(k,j,4)
               pevt5bg(k,i1(j,i),i)   = pang5g(k,j,1)
               pevt5bg(k,i1(j,i),i+3) = pang5g(k,j,2)
               pevt5bg(k,i1(j,i),i+6) = pang5g(k,j,3)
               pevt5bg(k,i1(j,i),i+9) = pang5g(k,j,4)
            enddo
         enddo
      enddo

      do i=1,3
         do j=1,5
            do k=1,5
               sevt5b(i1(j,i),i1(k,i),i)   = sijang5(j,k,1)
               sevt5b(i1(j,i),i1(k,i),i+3) = sijang5(j,k,2)
               sevt5b(i1(j,i),i1(k,i),i+6) = sijang5(j,k,3)
               sevt5b(i1(j,i),i1(k,i),i+9) = sijang5(j,k,4)
            enddo
            tevt5b(1,i1(j,i),i)   = tijang5(1,j,1)
            tevt5b(2,i1(j,i),i)   = tijang5(2,j,1)
            tevt5b(1,i1(j,i),i+3) = tijang5(1,j,2)
            tevt5b(2,i1(j,i),i+3) = tijang5(2,j,2)
            tevt5b(1,i1(j,i),i+6) = tijang5(1,j,3)
            tevt5b(2,i1(j,i),i+6) = tijang5(2,j,3)
            tevt5b(1,i1(j,i),i+9) = tijang5(1,j,4)
            tevt5b(2,i1(j,i),i+9) = tijang5(2,j,4)
         enddo
         ievt5b(i)    = iacc(1)
         ievt5b(i+3)  = iacc(2)
         ievt5b(i+6)  = iacc(3)
         ievt5b(i+9)  = iacc(4)
      enddo

c---- a few tests here
c      it = 1
c      write(6,101) pevt4b(1:4,1,it)
c      write(6,102) pevt4b(1:4,2,it)
c      write(6,103) pevt4b(1:4,3,it)
c      write(6,104) pevt4b(1:4,4,it)
c      write(6,101) pevt4b(1:4,1,it+1)
c      write(6,102) pevt4b(1:4,2,it+1)
c      write(6,103) pevt4b(1:4,3,it+1)
c      write(6,104) pevt4b(1:4,4,it+1)
c      write(6,101) pevt4b(1:4,1,it+2)
c      write(6,102) pevt4b(1:4,2,it+2)
c      write(6,103) pevt4b(1:4,3,it+2)
c      write(6,104) pevt4b(1:4,4,it+2)
c      write(6,101) pevt4b(1:4,1,it+3)
c      write(6,102) pevt4b(1:4,2,it+3)
c      write(6,103) pevt4b(1:4,3,it+3)
c      write(6,104) pevt4b(1:4,4,it+3)
c      write(6,101) pevt4b(1:4,1,it+4)
c      write(6,102) pevt4b(1:4,2,it+4)
c      write(6,103) pevt4b(1:4,3,it+4)
c      write(6,104) pevt4b(1:4,4,it+4)
c      write(6,101) pevt4b(1:4,1,it+5)
c      write(6,102) pevt4b(1:4,2,it+5)
c      write(6,103) pevt4b(1:4,3,it+5)
c      write(6,104) pevt4b(1:4,4,it+5)
c      write(6,*)
c      do it =1,3
c         write(6,*) it
c      s23 = 2d0*dot(pevt5b(1,2,it),pevt5b(1,3,it))
c      s24 = 2d0*dot(pevt5b(1,2,it),pevt5b(1,4,it))
c      s25 = 2d0*dot(pevt5b(1,2,it),pevt5b(1,5,it))
c      s34 = 2d0*dot(pevt5b(1,3,it),pevt5b(1,4,it))
c      s35 = 2d0*dot(pevt5b(1,3,it),pevt5b(1,5,it))
c      s45 = 2d0*dot(pevt5b(1,4,it),pevt5b(1,5,it))
c      ta2 = -pevt5b(4,2,it)+pevt5b(3,2,it)
c      ta3 = -pevt5b(4,3,it)+pevt5b(3,3,it)
c      ta4 = -pevt5b(4,4,it)+pevt5b(3,4,it)
c      ta5 = -pevt5b(4,5,it)+pevt5b(3,5,it)
c      tb2 = -pevt5b(4,2,it)-pevt5b(3,2,it)
c      tb3 = -pevt5b(4,3,it)-pevt5b(3,3,it)
c      tb4 = -pevt5b(4,4,it)-pevt5b(3,4,it)
c      tb5 = -pevt5b(4,5,it)-pevt5b(3,5,it)
c
c      it2 = it+9
c      write(6,107) ta2/tevt5b(1,2,it2),ta3/tevt5b(1,3,it2),
c     .             ta4/tevt5b(1,4,it2),ta5/tevt5b(1,5,it2)
c      write(6,107) tb2/tevt5b(2,2,it2),tb3/tevt5b(2,3,it2),
c     .             tb4/tevt5b(2,4,it2),tb5/tevt5b(2,5,it2)
c      write(6,108) s23/sevt5b(2,3,it2),s24/sevt5b(2,4,it2),
c     .             s34/sevt5b(3,4,it2)
c      write(6,108) s25/sevt5b(2,5,it2),s35/sevt5b(3,5,it2),
c     .             s45/sevt5b(4,5,it2)
c      write(6,*)
c      enddo


 101  format('p1:',4f14.6)
 102  format('p2:',4f14.6)
 103  format('p3:',4f14.6)
 104  format('p4:',4f14.6)
 107  format(4g18.13)
 108  format(3g18.13)


      return
      end


      subroutine disframe(sqa,phi,q2,pa,pb,q,p1,fmat)
      use DIS_mod, only : dis
      use KinData_mod, only : boostrest,unboostrest,rotatetoz,unrotatetoz
c ---- input sqa (=(q+pa)^2), phi and q2=-q.q
c ---- computes outgoing electron momentum p1 in (pa,pb) com frame
c ---- and Lorentz transformation fmat from (q+pa)_com to (pa+pb)_com
      implicit real*8(a-h,o-z)
      dimension p1(4),bmat(4,4),ubmat(4,4),urmat(4,4),fmat(4,4)
      dimension q(4),pa(4),pb(4),pqa(4),pap(4)
      common/pmasses/rma2,rmb2,rm2(1:7),shat
!$omp threadprivate(/pmasses/)

      roots = sqrt(shat)
      ea = roots/2d0
      eb = ea
      pa(4) = ea
      pa(1) = 0d0
      pa(2) = 0d0
      pa(3) = ea
      pb(4) = eb
      pb(1) = 0d0
      pb(2) = 0d0
      pb(3) = -eb

      z = 1d0/(1d0+sqa/q2)
      e1  = 0.5d0*(roots-q2/roots*(1d0-z)/z)
      p1z = 0.5d0*(q2/roots*(1d0+z)/z-roots)
      p1t = sqrt(e1**2-p1z**2)
      cp = cos(phi)
      sp = sin(phi)
      p1(4) = e1
      p1(1) = p1t*cp
      p1(2) = p1t*sp
      p1(3) = p1z
      do i=1,4
         q(i) = pb(i)-p1(i)
         pqa(i) = pa(i) + q(i)
      enddo
      call boostrest(pqa,bmat)
      call unboostrest(pqa,ubmat)
      do i=1,4
         pap(i) =0d0
         do j=1,4
            pap(i) = pap(i) + bmat(i,j)*pa(j)
         enddo
      enddo
      call unrotatetoz(pap,urmat)
      do i=1,4
         do j=1,4
            fmat(i,j) = 0d0
            do k=1,4
               fmat(i,j) = fmat(i,j)+ubmat(i,k)*urmat(k,j)
            enddo
         enddo
      enddo

      return
      end



      subroutine disframe0(sqa,phi,q2,pa,pb,q,p1)
c ---- input sqa (=(q+pa)^2), phi and q2=-q.q
c ---- computes outgoing electron momentum p1 in (pa,pb) com frame
      implicit real*8(a-h,o-z)
      dimension p1(4)
      dimension q(4),pa(4),pb(4),pqa(4),pap(4)
      common/pmasses/rma2,rmb2,rm2(1:7),shat
!$omp threadprivate(/pmasses/)

      roots = sqrt(shat)
      ea = roots/2d0
      eb = ea
      pa(4) = ea
      pa(1) = 0d0
      pa(2) = 0d0
      pa(3) = ea
      pb(4) = eb
      pb(1) = 0d0
      pb(2) = 0d0
      pb(3) = -eb

      z = 1d0/(1d0+sqa/q2)
      e1  = 0.5d0*(roots-q2/roots*(1d0-z)/z)
      p1z = 0.5d0*(q2/roots*(1d0+z)/z-roots)
      p1t = sqrt(e1**2-p1z**2)
      cp = cos(phi)
      sp = sin(phi)
      p1(4) = e1
      p1(1) = p1t*cp
      p1(2) = p1t*sp
      p1(3) = p1z
      do i=1,4
         q(i) = pb(i)-p1(i)
      enddo

      return
      end
