      subroutine decay4(rmh2,ph,rr,pd,wtps,ipass)
      use KinData_mod, only : boostrest,unboostrest,rotatetoz,unrotatetoz
      use Constants_mod, only : pi, twopi
      implicit real*8(a-h,o-z)
c---- parametrizes the decay of ph to two massive particles, which then deacy into four massless particles pd(1:4)
c---- requires eight random variables rr
c---- settings for H-> VV*: rmv1=rmv2=M_Z (or M_W), ios1=1, ios2=0, isym=1
c---- settings for V*-> HV: rmv1=M_H, rmv2=M_V, ios1=3 (only on-shell H for the moment), ios2=1, isym=0
c---- ios=1: BW parametrization, ios=0: linear parametrization
c---- isym=1: symmetrize over on-shell and off-shell boson (such that always p1,p2=mu+,mu-, p3,p4=e+,e-)
      dimension ph(4),rr(8),pd(4,4)
      dimension ubmat(4,4),ubmat12(4,4),ubmat34(4,4)
      dimension p12(4),p34(4),p12p(4),p34p(4),p12ap(4),p34ap(4)
      dimension p1p(4),p2p(4),p3p(4),p4p(4)
      dimension p1ap(4),p2ap(4),p3ap(4),p4ap(4)
      common/vdecay/rmv1,rgv1,rmv2,rgv2,ios1,ios2,isym
      common /tcuts/ymin,y0

      r1 = rr(1)
      r2 = rr(2)
      r3 = rr(3)
      r4 = rr(4)
      r5 = rr(5)
      r6 = rr(6)
      r7 = rr(7)
      r8 = rr(8)

      ipass = 0

      s12min = y0*rmh2
      s12max = rmh2-s12min
      if (s12min > s12max) return
      call pick_sinv_generic(s12,s12min,s12max,rmv1,rgv1,r2,wtps,ios1)
      if(wtps==0d0) return

      if(rmh2.lt.s12) return ! Insufficient scattering energy to produce the heavy gauge boson

      s34min = y0*rmh2
      s34max = dmin1((sqrt(rmh2)-sqrt(s12))**2,(1d0-y0)*rmh2)
      if (s34min > s34max) return
      call pick_sinv_generic(s34,s34min,s34max,rmv2,rgv2,r3,wtps,ios2)
      if (wtps.eq.0d0) return

      costhmin = -1d0
      costhmax = 1d0
      phimin = 0d0
      phimax = 2d0*pi
      call pick(1,costh,costhmin,costhmax,r4,wtps)
c      write(6,*) '4a:',wtps,r3,r4
      call pick(1,phi,phimin,phimax,r1,wtps)
c      write(6,*) '4:',wtps,r3,r4

      wtps = wtps*dsqrt(dlambda(rmh2,s12,s34))/rmh2/32d0/pi**2
c      write(6,*) '5:',wtps

      call makeps2com(rmh2,costh,phi,s12,s34,p12p,p34p,p12ap,p34ap)
      call unboostrest(ph,ubmat)
      do i=1,4
         p12(i) = 0d0
         p34(i) = 0d0
         do j=1,4
            p12(i) = p12(i) + ubmat(i,j)*p12p(j)
            p34(i) = p34(i) + ubmat(i,j)*p34p(j)
         enddo
      enddo

      call pick(1,costh,costhmin,costhmax,r5,wtps)
      call pick(1,phi,phimin,phimax,r6,wtps)
      wtps = wtps*dsqrt(dlambda(s12,0d0,0d0))/s12/32d0/pi**2
      wtps = wtps/2d0/pi
      call makeps2com(s12,costh,phi,0d0,0d0,p1p,p2p,p1ap,p2ap)
      call unboostrest(p12,ubmat12)
      call pick(1,costh,costhmin,costhmax,r7,wtps)
      call pick(1,phi,phimin,phimax,r8,wtps)
      wtps = wtps*dsqrt(dlambda(s34,0d0,0d0))/s34/32d0/pi**2
      wtps = wtps/2d0/pi
      call makeps2com(s34,costh,phi,0d0,0d0,p3p,p4p,p3ap,p4ap)
      call unboostrest(p34,ubmat34)

      i1=1
      i2=2
      i3=3
      i4=4
      if (isym.eq.1) then       ! symmetrize the H->VV* kinematics
         if (RAND(0).lt.0.5d0) then
            i1=3
            i2=4
            i3=1
            i4=2
         endif
      endif

      do i=1,4
         pd(i,1) = 0d0
         pd(i,2) = 0d0
         pd(i,3) = 0d0
         pd(i,4) = 0d0
         do j=1,4
            pd(i,i1) = pd(i,i1) + ubmat12(i,j)*p1p(j)
            pd(i,i2) = pd(i,i2) + ubmat12(i,j)*p2p(j)
            pd(i,i3) = pd(i,i3) + ubmat34(i,j)*p3p(j)
            pd(i,i4) = pd(i,i4) + ubmat34(i,j)*p4p(j)
         enddo
      enddo
      ipass =1
      return
      end




      subroutine phase4vv(x, wt, ifail)
      use KinData_mod, only : boostrest,unboostrest,rotatetoz,unrotatetoz
      implicit real*8(a-h, o-z)
      double precision x, wt
      integer ifail
      dimension x(*)
      dimension p1(4),p2(4),p3(4),p4(4),v(4),v1(4),q(4)
      dimension p1p(4),p2p(4),p3p(4),p4p(4)
      dimension p1ap(4),p2ap(4),p3ap(4),p4ap(4)
      dimension ubmat12(4,4),ubmat34(4,4)
      dimension pdecay(4,4)
      dimension bmat(4,4),ubmat(4,4),rmat(4,4),urmat(4,4),fmat(4,4)
      dimension p12(4),p34(4),p12p(4),p34p(4),p12ap(4),p34ap(4)
      dimension xveg(8)
      parameter(pi=3.141592653589793238d0)
      ! Common Blocks
      common/pmasses/rma2,rmb2,rm2(1:7),shat
!$omp threadprivate(/pmasses/)
      common /tcuts/ymin,y0
      common /pmom/p(4,7)
!$omp threadprivate(/pmom/)
      common /pmomang4/pang4(4,4,4)
!$omp threadprivate(/pmomang4/)
      common /invarang4/sijang4(4,4,4),tijang4(2,4,4),iacc(4)
!$omp threadprivate(/invarang4/)
      common /pin/pa(4),pb(4)
!$omp threadprivate(/pin/)
      common /nwa/rm12,rg12,inwa
      common /Nphase/Nphase
!$omp threadprivate(/Nphase/)
      common/vdecay/rmv1,rgv1,rmv2,rgv2,ios1,ios2,isym

cLIPS      print*, "phase4vv", x(1), wt, ifail

      wtps = 1d0
      Nphase=1

      ifail = 1
      wt = 0d0


      r1 = x(1)
      r2 = x(2)
      r3 = x(3)
      r4 = x(4)
      r5 = x(5)
      r6 = x(6)
      r7 = x(7)

      s12min = y0*shat
      s12max = shat-s12min

      if (s12min > s12max) return
      call pick_sinv_generic(s12,s12min,s12max,rmv1,rgv1,r1,wtps,ios1)
      if (wtps == 0d0) return
      if (shat < s12) return ! Insufficient scattering energy to produce the heavy gauge boson

      s34min = y0*shat
      s34max = dmin1((dsqrt(shat)-dsqrt(s12))**2,(1d0-y0)*shat)
      if (s34min.gt.s34max) return
      call pick_sinv_generic(s34,s34min,s34max,rmv2,rgv2,r2,wtps,ios2)
      if (wtps.eq.0d0) return

      call p2generic(r3,0d0,shat,rma2,rmb2,s12,s34,p12,p34,pa,pb,wtps)
      if(wtps.eq.0d0) return

      costhmin = -1d0
      costhmax = 1d0
      phimin = 0d0
      phimax = 2d0*pi

      call pick(1,costh,costhmin,costhmax,r4,wtps)
      call pick(1,phi,phimin,phimax,r5,wtps)
      wtps = wtps*dsqrt(dlambda(s12,0d0,0d0))/s12/32d0/pi**2
      wtps = wtps/2d0/pi
      call makeps2com(s12,costh,phi,0d0,0d0,p1p,p2p,p1ap,p2ap)
      call unboostrest(p12,ubmat12)
      call pick(1,costh,costhmin,costhmax,r6,wtps)
      call pick(1,phi,phimin,phimax,r7,wtps)
      wtps = wtps*dsqrt(dlambda(s34,0d0,0d0))/s34/32d0/pi**2
      wtps = wtps/2d0/pi
      call makeps2com(s34,costh,phi,0d0,0d0,p3p,p4p,p3ap,p4ap)
      call unboostrest(p34,ubmat34)

      i1=1
      i2=2
      i3=3
      i4=4
      if (isym.eq.1) then       ! symmetrize the H->VV* kinematics
         if (RAND(0).lt.0.5d0) then
            i1=3
            i2=4
            i3=1
            i4=2
         endif
      endif

      do i=1,4
         pdecay(i,1) = 0d0
         pdecay(i,2) = 0d0
         pdecay(i,3) = 0d0
         pdecay(i,4) = 0d0
         do j=1,4
            pdecay(i,i1) = pdecay(i,i1) + ubmat12(i,j)*p1p(j)
            pdecay(i,i2) = pdecay(i,i2) + ubmat12(i,j)*p2p(j)
            pdecay(i,i3) = pdecay(i,i3) + ubmat34(i,j)*p3p(j)
            pdecay(i,i4) = pdecay(i,i4) + ubmat34(i,j)*p4p(j)
         enddo
      enddo

      do i=1,4
         do j=1,4
            p(i,j) = pdecay(i,j)
            pang4(i,j,1) = p(i,j)
         enddo
      enddo

      call fillinv(4,pang4(1:4,1:4,1),pa,pb,sijang4(1:4,1:4,1)
     .                                     ,tijang4(1:2,1:4,1))

      if (wtps.ne.0d0 .and. .not.isnan(wtps)) ifail = 0
      wt = wtps

      return
      end

      subroutine fillcommon4pvv
      implicit real*8(a-h, o-z)
      common /pmomang4/pang4(4,4,4)
!$omp threadprivate(/pmomang4/)
      common/invarang4/sijang4(4,4,4),tijang4(2,4,4),iacc(4)
!$omp threadprivate(/invarang4/)
      common/eventmom4p/pevt4(4,4,6)
!$omp threadprivate(/eventmom4p/)
      common/eventinv4p/sevt4(4,4,6),tevt4(2,4,6),ievt4(6)
!$omp threadprivate(/eventinv4p/)

      do j=1,4
         do k=1,4
            pevt4(k,j,1) = pang4(k,j,1)
         enddo
      enddo

      do j=1,4
         do k=1,4
            sevt4(k,j,1) = sijang4(k,j,1)
         enddo
         tevt4(1,j,1) = tijang4(1,j,1)
         tevt4(2,j,1) = tijang4(2,j,1)
      enddo
      ievt4(1) = 1

      return
      end


      subroutine phase5vv(x,wt,ifail)
      use Process_mod, only : s_proc
      use KinData_mod, only : boostrest,unboostrest,rotatetoz,unrotatetoz
      implicit real*8(a-h,o-z)
      dimension p1(4),p2(4),p3(4),p4(4),p5(4),v(4),v1(4),q(4)
      dimension pqa(4),p1234(4)
      dimension pdecay(1:4,1:4)
      dimension x(*)
      dimension bmat(4,4),ubmat(4,4),rmat(4,4),urmat(4,4),fmat(4,4)
      parameter(pi=3.141592653589793238d0)
      common /pmasses/rma2,rmb2,rm2(1:7),shat
!$omp threadprivate(/pmasses/)
      common /pin/pa(4),pb(4)
!$omp threadprivate(/pin/)
      common/pmom/p(4,7)
!$omp threadprivate(/pmom/)
      common/pmomang5/pang5(4,5,4)
!$omp threadprivate(/pmomang5/)
      common/invar5/sij5(5,5),tij5(2,5)
!$omp threadprivate(/invar5/)
      common/invarang5/sijang5(5,5,4),tijang5(2,5,4),iacc(4)
!$omp threadprivate(/invarang5/)
      common /nwa/rm1234,rg1234,inwa
      common /tcuts/ymin,y0
      common /mxdim/ mxdim
      common /Nphase/Nphase
!$omp threadprivate(/Nphase/)

cLIPS      print*, "phase5vv", x(1), wt, ifail

      wtps = 1d0
      Nphase = 1
*
* phase space for 2->5 scattering, a+b -> 1+2+3+4+5 with ph=p1+p2+p3+p4 and pv12 = p1+p2, pv34 = p3+p4
*
* incoming momenta along z-axis, pa is +z
*
* massless case: integrates to P_5 = 1/4718592/pi**7*sab^3 = 0.7016789753e-10*sab^3

      r1 = x(1)
      r2 = x(2)

      ifail =1
      wt = 0d0
      
      s1234min=y0*shat
      s1234max=(1d0 - y0)*shat
      if (s1234min > s1234max) return
      call pick_sinv_generic(s1234,s1234min,s1234max,rm1234,rg1234,r1,wtps,inwa)
      if (wtps == 0d0) return

      wtps = wtps/2d0/pi
c      write(6,*) '1:',wtps
      call p2generic(r2,0d0,shat,rma2,rmb2,s1234,rm2(5),p1234,p5,pa,pb,wtps)
      if(wtps.eq.0d0) return

c      write(6,*) '2:',wtps

      call decay4(s1234,p1234,x(3:10),pdecay,wtps,ipass)
      if (ipass.eq.0) return
c      write(6,*) '3:',wtps
      do i=1,4
         do j=1,4
            p(i,j) = pdecay(i,j)
            pang5(i,j,1) = p(i,j)
         enddo
         p(i,5) = p5(i)
         pang5(i,5,1) = p(i,5)
      enddo

      if(p5(4).lt.0d0)then ! negative energy jets
         ifail=1
         return
      endif

      call fillinv(5,pang5(1:4,1:5,1),pa,pb,sijang5(1:5,1:5,1)
     .                                     ,tijang5(1:2,1:5,1))

c      write(6,101) p1(1:4)
c      write(6,102) p2(1:4)
c      write(6,103) p3(1:4)
c      write(6,105) v(1:4)
c      write(6,106) pb(1:4)
c      write(6,109) v1(1:4)
c      write(6,*) q2/dot(q,q)
c      write(6,109) p1(1:4) + p2(1:4) + p3(1:4)

 101  format('p1:',4f14.6)
 102  format('p2:',4f14.6)
 103  format('p3:',4f14.6)
 105  format('pa:',4f14.6)
 106  format('pb:',4f14.6)
 109  format('pp:',4f14.6)

      if (wtps.ne.0d0) ifail = 0
      wt = wtps

      return
      end


      subroutine fillcommon5pvv
      implicit real*8(a-h,o-z)
      common/pmomang5/pang5(4,5,4)
!$omp threadprivate(/pmomang5/)
      common/invarang5/sijang5(5,5,4),tijang5(2,5,4),iacc(4)
!$omp threadprivate(/invarang5/)
      common/eventmom5p/pevt5(4,5,4)
!$omp threadprivate(/eventmom5p/)
      common/eventinv5p/sevt5(5,5,4),tevt5(2,5,4),ievt5(4)
!$omp threadprivate(/eventinv5p/)

c---- pang3,pang3g and sija,tija are input
c---- this subroutine fills
c----       pevt3(component,pnum,ievent)             : 4-momenta of events
c----       sevt3(i,j,ievent),tevt3([ab],i,ievent)   : invariants of events
c----
c---- ievent = 1 fixed

      do j=1,5
         do k=1,4
            pevt5(k,j,1)    = pang5(k,j,1)
         enddo
      enddo

      do j=1,5
         do k=1,5
            sevt5(j,k,1)    = sijang5(j,k,1)
         enddo
         tevt5(1,j,1)   = tijang5(1,j,1)
         tevt5(2,j,1)   = tijang5(2,j,1)
      enddo
      ievt5(1) = 1

c---- a few tests here
c      write(6,101) pevt3(1:4,1,1)
c      write(6,102) pevt3(1:4,2,1)
c      write(6,103) pevt3(1:4,3,1)
c      write(6,104) pevt3(1:4,1,1)+pevt3(1:4,2,1)+pevt3(1:4,3,1)

c      it =1
c      ta1 = -pevt3(4,1,it)+pevt3(3,1,it)
c      ta2 = -pevt3(4,2,it)+pevt3(3,2,it)
c      ta3 = -pevt3(4,3,it)+pevt3(3,3,it)
c      write(6,110) ta1/tevt3(1,1,it),ta2/tevt3(1,2,it),ta3/tevt3(1,3,it)

 101  format('p1:',4f14.6)
 102  format('p2:',4f14.6)
 103  format('p3:',4f14.6)
 104  format('p4:',4f14.6)
 110  format(3g18.11)
 111  format(2g18.11)

      return
      end





      subroutine phase6vv(x,wt,ifail)
      use Process_mod, only : s_proc
      use KinData_mod, only : boostrest,unboostrest,rotatetoz,unrotatetoz
      use Constants_mod, only : pi, twopi
      implicit real*8(a-h,o-z)
      dimension p1(4),p2(4),p3(4),p4(4),p5(4),p6(4),v(4),v1(4),q(4)
      dimension p5p(4),p6p(4),pap(4)
      dimension p5a(4),p6a(4)
      dimension p5ap(4),p6ap(4)
      dimension p56(4),parb(4)
      dimension pqa(4),p1234(4)
      dimension pdecay(1:4,1:4)
      dimension x(*)
      dimension bmat(4,4),ubmat(4,4),rmat(4,4),urmat(4,4),fmat(4,4)
      common /pmasses/rma2,rmb2,rm2(1:7),shat
!$omp threadprivate(/pmasses/)
      common /pin/pa(4),pb(4)
!$omp threadprivate(/pin/)
      common/pmom/p(4,7)
!$omp threadprivate(/pmom/)
      common/pmomang6/pang6(4,6,4)
!$omp threadprivate(/pmomang6/)
      common/invar6/sij6(6,6),tij6(2,6)
!$omp threadprivate(/invar6/)
      common/invarang6/sijang6(6,6,4),tijang6(2,6,4),iacc(4)
!$omp threadprivate(/invarang6/)
      common /nwa/rm1234,rg1234,inwa
      common /tcuts/ymin,y0
      common /techpar/iang
      common /mxdim/ mxdim
      common /Nphase/Nphase
!$omp threadprivate(/Nphase/)

cLIPS      print*, "phase6vv", x(1), wt, ifail

      wtps = 1d0
      if(iang.eq.0)then
         Nphase=2
      else
         Nphase=4
         wtps=wtps/2d0
      endif

*
* phase space for 2->6 scattering, a+b -> 1+2+3+4+5+6 with ph=p1+p2+p3+p4 and pv12 = p1+p2, pv34 = p3+p4
*
* incoming momenta along z-axis, pa is +z
*
      r1 = x(1)
      r2 = x(2)
      r3 = x(3)
      r4 = x(4)
      r5 = x(5)

      ifail =1
      wt = 0d0

      s1234min=y0*shat
      s1234max=(1d0 - y0)*shat
      if (s1234min > s1234max) return
      call pick_sinv_generic(s1234,s1234min,s1234max,rm1234,rg1234,r1,wtps,inwa)
      if (wtps == 0d0) return


      if(shat.lt.s1234) return ! Insufficient scattering energy to produce the heavy particle
      wtps = wtps/2d0/pi
c      write(6,*) '1:',wtps

      s56min = dmax1(y0*shat,(dsqrt(rm2(5))+dsqrt(rm2(6)))**2)
      s56max = dmin1((dsqrt(shat)-dsqrt(s1234))**2,(1d0-y0)*shat)
      if (s56min.ge.s56max) return
      call pick(10,s56,s56min,s56max,r2,wtps)
      wtps = wtps/2d0/pi
      call p2generic_alt(3,r3,0d0,shat,rma2,rmb2,s1234,s56,p1234,p56,pa,pb,wtps)
      if(wtps.eq.0d0) return

c      write(6,*) '2:',wtps


**************************************************
*     TM 24/06/2015: switched to sampling
*     rat=sa3/(sa3max-sa3+y0*shat) logarithmically
*     from costh
*
!      call boostrest(p56,bmat)
!      call unboostrest(p56,ubmat)
!      do i=1,4
!         v(i) = 0d0
!         pap(i) = 0d0
!         do j=1,4
!            pap(i) = pap(i) + bmat(i,j)*pa(j)
!         enddo
!      enddo
!
!      call rotatetoz(pap,rmat)
!      do i=1,4
!         parb(i) = 0.d0
!         do j=1,4
!            do k=1,4
!               parb(i) = parb(i) + rmat(i,k)*bmat(k,j)*pa(j)
!            enddo
!         enddo
!      enddo
!
!      bound = 2.d0*(s56+rm2(5))/sqrt(s56)*parb(4)
!      sa5min = y0*shat
!      sa5max = min(bound,(1.d0-y0)*shat)
!      if(sa5max.le.sa5min) return
!
!*     jacobian for costh -> sa5
!      wtps = wtps/(s56)*sqrt(s56)/parb(4)
!
!*     rat=sa5/(sa5max-sa5+y0*shat)
!      ratmin=y0*shat/sa5max
!      ratmax=sa5max/(y0*shat)
!
!      call pick(2,rat,ratmin,ratmax,r4,wtps)
!
!      sa5=rat*(sa5max+y0*shat)/(rat+1d0)
!      wtps=wtps*(sa5max-sa5+y0*shat)**2/(sa5max+y0*shat)
!
!      costh=-(sa5-s56*parb(4)/s56**0.5)/(2d0*parb(4)*(s56/2d0/s56**0.5))


c      call pick(1,costh,-1d0,1d0,r4,wtps)
c      call pickcos(costh,y0,r4,wtps)
      call pickcos_alt(costh,r4,wtps)
      call pick(1,phi,0d0,twopi,r5,wtps)
      call makeps2com(s56,costh,phi,rm2(5),rm2(6),p5p,p6p,p5ap,p6ap)
      wtps = wtps*dsqrt(dlambda(s56,rm2(5),rm2(6)))/s56/32d0/pi**2

*****************************************

      call boostrest(p56,bmat)
      call unboostrest(p56,ubmat)
      do i=1,4
         v(i) = 0d0
         pap(i) = 0d0
         do j=1,4
            pap(i) = pap(i) + bmat(i,j)*pa(j)
         enddo
      enddo
      call unrotatetoz(pap,urmat)
      do i=1,4
         do j=1,4
            fmat(i,j) = 0d0
            do k=1,4
               fmat(i,j) = fmat(i,j)+ubmat(i,k)*urmat(k,j)
            enddo
         enddo
      enddo

      do i=1,4
         p5(i) = 0d0
         p6(i) = 0d0
         p5a(i) = 0d0
         p6a(i) = 0d0
         do j=1,4
            p5(i) = p5(i) + fmat(i,j)*p5p(j)
            p6(i) = p6(i) + fmat(i,j)*p6p(j)
            p5a(i) = p5a(i) + fmat(i,j)*p5ap(j)
            p6a(i) = p6a(i) + fmat(i,j)*p6ap(j)
         enddo
      enddo

      if((p5(4).lt.0).or.(p5a(4).lt.0).or.(p6(4).lt.0).or.(p6a(4).lt.0))then
         ifail = 1
         return
      endif

**************************************************


      call decay4(s1234,p1234,x(6:13),pdecay,wtps,ipass)
      if (ipass.eq.0) return
c      write(6,*) '3:',wtps


      do i=1,4
         do j=1,4
            p(i,j) = pdecay(i,j)
            pang6(i,j,1) = p(i,j)
            pang6(i,j,2) = p(i,j)
         enddo
         p(i,5) = p5(i)
         p(i,6) = p6(i)
         pang6(i,5,1) = p5(i)
         pang6(i,6,1) = p6(i)
         pang6(i,5,2) = p5a(i)
         pang6(i,6,2) = p6a(i)
      enddo

      call fillinv(6,p,pa,pb,sij6,tij6)
      do i=1,2
         call fillinv(6,pang6(1:4,1:6,i),pa,pb,sijang6(1:6,1:6,i)
     .                                        ,tijang6(1:2,1:6,i))
      enddo

      do i=1,2
         call acceptcuts6(sijang6(1:6,1:6,i),tijang6(1:2,1:6,i),iacc(i))
      enddo
      if (iacc(1)+iacc(2).eq.0) return

      ifail =0
      wt = wtps

      return
      end



      subroutine acceptcuts6(sij6,tij6,ipass)
      implicit real*8(a-h,o-z)
      dimension sij6(1:6,1:6),tij6(1:2,1:6)
      common/pmasses/rma2,rmb2,rm2(1:7),shat
!$omp threadprivate(/pmasses/)
      common /tcuts/ymin,y0

      ipass = 0

      sa5 = -tij6(1,5)
      sa6 = -tij6(1,6)
      sb5 = -tij6(2,5)
      sb6 = -tij6(2,6)


      rinvmin = min(sa5,sa6,sb5,sb6)
      if (rinvmin.lt.y0*shat) return

      ipass = 0
* single out on 5||a, 6||a
      if (rinvmin.eq.sa5.or.rinvmin.eq.sa6) ipass=1
      return
      end



      subroutine fillcommon6pvv
      implicit real*8(a-h,o-z)
      common/pmomang6/pang6(4,6,4)
!$omp threadprivate(/pmomang6/)
      common/invarang6/sijang6(6,6,4),tijang6(2,6,4),iacc(4)
!$omp threadprivate(/invarang6/)
      common/eventmom6p/pevt6(4,6,12)
!$omp threadprivate(/eventmom6p/)
      common/eventinv6p/sevt6(6,6,12),tevt6(2,6,12),ievt6(12)
!$omp threadprivate(/eventinv6p/)

c---- pang6 and sija,tija are input and correspond to an event in (3,4;a)
c---- this subroutine fills
c----       pevt6(component,pnum,ievent)             : 4-momenta of events
c----       sevt6(i,j,ievent),tevt6([ab],i,ievent)   : invariants of events
c----
c---- ievent
c---- 1: (5,6;a)
c---- 2: (5,6;b)
c----
c---- 5-6 (i,j;x) with (i,j) system rotated around x-axis
c----
c---- full phave space coverage is obtained by summing all 2 events
c---- angular average is obtained by averaging:
c----        M2 = 1/2* ( sum_ievent=1..4   M2(ievent))
c----


      do j=1,6
         do k=1,4
            pevt6(k,j,1) = pang6(k,j,1)
            pevt6(k,j,3) = pang6(k,j,2)
            if (k.ne.3) then
               pevt6(k,j,2) = pang6(k,j,1)
               pevt6(k,j,4) = pang6(k,j,2)
            else
               pevt6(k,j,2) = -pang6(k,j,1)
               pevt6(k,j,4) = -pang6(k,j,2)
            endif
         enddo
      enddo


      do j=1,6
         do k=1,6
            sevt6(j,k,1) = sijang6(j,k,1)
            sevt6(j,k,2) = sijang6(j,k,1)
            sevt6(j,k,3) = sijang6(j,k,2)
            sevt6(j,k,4) = sijang6(j,k,2)
         enddo
         tevt6(1,j,1) = tijang6(1,j,1)
         tevt6(2,j,1) = tijang6(2,j,1)
         tevt6(1,j,3) = tijang6(1,j,2)
         tevt6(2,j,3) = tijang6(2,j,2)
         tevt6(1,j,2) = tijang6(2,j,1)
         tevt6(2,j,2) = tijang6(1,j,1)
         tevt6(1,j,4) = tijang6(2,j,2)
         tevt6(2,j,4) = tijang6(1,j,2)
      enddo
      ievt6(1) = iacc(1)
      ievt6(2) = iacc(1)
      ievt6(3) = iacc(2)
      ievt6(4) = iacc(2)


c---- a few tests here
c      write(6,101) pevt6(1:4,1,1)
c      write(6,102) pevt6(1:4,2,1)
c      write(6,103) pevt6(1:4,3,1)
c      write(6,104) pevt6(1:4,4,1)
c      write(6,105) pevt6(1:4,5,1)
c      write(6,106) pevt6(1:4,6,1)
c      write(6,101) pevt6(1:4,1,4)
c      write(6,102) pevt6(1:4,2,4)
c      write(6,103) pevt6(1:4,3,4)
c      write(6,104) pevt6(1:4,4,4)
c      write(6,105) pevt6(1:4,5,4)
c      write(6,106) pevt6(1:4,6,4)

c      it =2
c
c      s12 = 2d0*dot(pevt6(1,1,it),pevt6(1,2,it))
c      s13 = 2d0*dot(pevt6(1,1,it),pevt6(1,3,it))
c      s23 = 2d0*dot(pevt6(1,2,it),pevt6(1,3,it))
c      s14 = 2d0*dot(pevt6(1,1,it),pevt6(1,4,it))
c      s24 = 2d0*dot(pevt6(1,2,it),pevt6(1,4,it))
c      s34 = 2d0*dot(pevt6(1,3,it),pevt6(1,4,it))
c      ta1 = -pevt6(4,1,it)+pevt6(3,1,it)
c      ta2 = -pevt6(4,2,it)+pevt6(3,2,it)
c      ta3 = -pevt6(4,3,it)+pevt6(3,3,it)
c      ta4 = -pevt6(4,4,it)+pevt6(3,4,it)
c      tb1 = -pevt6(4,1,it)-pevt6(3,1,it)
c      tb2 = -pevt6(4,2,it)-pevt6(3,2,it)
c      tb3 = -pevt6(4,3,it)-pevt6(3,3,it)
c      tb4 = -pevt6(4,4,it)-pevt6(3,4,it)

c      it = it+2
c      write(6,110) ta1/tevt6(1,1,it),ta2/tevt6(1,2,it),ta3/tevt6(1,3,it)
c     .            ,ta4/tevt6(1,4,it)
c      write(6,110) tb1/tevt6(2,1,it),tb2/tevt6(2,2,it),tb3/tevt6(2,3,it)
c     .            ,tb4/tevt6(2,4,it)
c      write(6,110) s12/sevt6(1,2,it),s13/sevt6(1,3,it),s23/sevt6(2,3,it)
c     .            ,s14/sevt6(1,4,it)
c      write(6,111) s24/sevt6(2,4,it),s34/sevt6(3,4,it)

 101  format('p1:',4f14.6)
 102  format('p2:',4f14.6)
 103  format('p3:',4f14.6)
 104  format('p4:',4f14.6)
 105  format('p5:',4f14.6)
 106  format('p6:',4f14.6)
 110  format(4g18.11)
 111  format(2g18.11)

      return
      end



      subroutine phase7avv(x,wt,ifail)
      use Process_mod, only : s_proc
      use KinData_mod, only : boostrest,unboostrest,rotatetoz,unrotatetoz
      implicit real*8(a-h,o-z)
      dimension p1(4),p2(4),p3(4),p4(4),p5(4),p6(4),p7(4),v(4),q(4),v1(4)
      dimension p1p(4),p2p(4),p3p(4),p4p(4),p5p(4),p6p(4),p7p(4),pap(4),
     .          p3pp(4),p3app(4),papp(4)
      dimension p1ap(4),p2ap(4),p3ap(4),p4ap(4),p5ap(4),p6ap(4),p7ap(4)
      dimension p5a(4),p6a(4),p7a(4)
      dimension p1234(4),p567(4),p67p(4),p67(4)
      dimension p567p(4)
      dimension p67a(4),p67ap(4),p6t(4),p7t(4),p6ta(4),p7ta(4)
      dimension pdecay(1:4,1:4)
      dimension x(*)
      dimension bmat(4,4),ubmat(4,4),rmat(4,4),urmat(4,4),fmat(4,4)
      dimension parb(4)
      double precision p6pp(4),p7pp(4)
      parameter(pi=3.141592653589793238d0)
      common/pmasses/rma2,rmb2,rm2(1:7),shat
!$omp threadprivate(/pmasses/)
      common /nwa/rm1234,rg1234,inwa
      common/pin/pa(4),pb(4)
!$omp threadprivate(/pin/)
      common/pmom/p(4,7)
!$omp threadprivate(/pmom/)
      common/pmomang7/pang7(4,7,4)
!$omp threadprivate(/pmomang7/)
      common/invar7/sij7(7,7),tij7(2,7)
!$omp threadprivate(/invar7/)
      common/invarang7/sijang7(7,7,4),tijang7(2,7,4),iacc(4)
!$omp threadprivate(/invarang7/)
      common/tcuts/ymin,y0
      common /techpar/iang
      common /Nphase/Nphase
!$omp threadprivate(/Nphase/)

cLIPS      print*, "phase7avv", x(1), wt, ifail

      wtps = 1d0

      if(iang.eq.0)then
         Nphase=6
      else
         Nphase=24
         wtps=wtps/4d0
      endif


* this is (5;6,7;a)
*
* phase space for 2->7 scattering, a+b -> 1+2+3+4+5+6+7 with q=p1+p2+p3+p4
* with sequential splitting (567) -> 5+(67) -> 5+6+7
*
* (567) system is rotated with respect to pa axis
* (67)  system is rotated with respect to p3 axis
*
* default momenta             : pang(i,j,1)
* (67) rotated                : pang(i,j,2)
* (567) rotated               : pang(i,j,3)
* (567) rotated, (67) rotated : pang(i,j,4)
*
*
* incoming momenta along z-axis, pa is +z
*
      r1 = x(1)
      r2 = x(2)
      r3 = x(3)
      r4 = x(4)
      r5 = x(5)
      r6 = x(6)
      r7 = x(7)
      r8 = x(8)
      wt = 0d0
      ifail =1


      s1234min=y0*shat
      s1234max=(1d0 - y0)*shat
c      s12min = dmax1(y0*shat,(dsqrt(rm2(1))+dsqrt(rm2(2)))**2)
c      s12max = dmin1((dsqrt(shat)-dsqrt(rm2(3))-dsqrt(rm2(4)))**2
c     .              ,(1d0-5d0*y0)*shat)
      if (s1234min > s1234max) return
      call pick_sinv_generic(s1234,s1234min,s1234max,rm1234,rg1234,r1,wtps,inwa)
      if (wtps == 0d0) return
      if (shat < s1234) return    ! Insufficient scattering energy to produce the heavy particle
      wtps = wtps/2d0/pi


      s567min = dmax1(y0*shat,
     .           (dsqrt(rm2(5))+dsqrt(rm2(6))+dsqrt(rm2(7)))**2)
      s567max = dmin1((dsqrt(shat)-dsqrt(s1234))**2,(1d0-y0)*shat)
      if (s567max.le.s567min) return
      call pick(10,s567,s567min,s567max,r2,wtps)
      wtps = wtps/2d0/pi
      s67min = dmax1(y0*shat,(dsqrt(rm2(6))+dsqrt(rm2(7)))**2)
      s67max = dmin1(s567,(1d0-y0)*shat)
      if (s67max.le.s67min) return
*      call pick(1,s67,s67min,s67max,r5,wtps)
*     sample log(s67)
      call pick(2,s67,s67min,s67max,r3,wtps)
      wtps = wtps/2d0/pi

      call p2generic_alt(3,r4,0d0,shat,0d0,0d0,s1234,
     .               s567,p1234,p567,pa,pb,wtps)
      if(wtps.eq.0d0) return

      if(p567(4).lt.0d0)then
         ifail = 1
         return
      endif

* TODO: change of variables only works in the massless case.
*       Add a condition s.t. we return to the old calculations
*       for massive partons

***********************************************
*     TM 03/05/2015: switched to sampling
*     rat=sa3/(sa3max-sa3+y0*shat) logarithmically
*     from costh

      costhmin = -1d0
      costhmax = 1d0
      phimin = 0d0
      phimax = 2d0*pi
c      call pick(1,costh,costhmin,costhmax,r7,wtps)
c      call pickcos(costh,y0,r7,wtps)


*     boost pa into the s345 rest frame
      call boostrest(p567,bmat)
      do i=1,4
         v(i) = 0.d0
         pap(i) = 0.d0
         do j=1,4
            pap(i) = pap(i) + bmat(i,j)*pa(j)
         enddo
      enddo

*     rotate pap into the frame
      call rotatetoz(pap,rmat)

      do i=1,4
         parb(i) = 0.d0
         do j=1,4
            do k=1,4
               parb(i) = parb(i) + rmat(i,k)*bmat(k,j)*pa(j)
            enddo
         enddo
      enddo

      bound = 2.d0*(s567+rm2(5)-s67)/sqrt(s567)*parb(4)
      sa5min = y0*shat
      sa5max = min(bound,(1.d0-y0)*shat)
      if(sa5max.le.sa5min) return

*     jacobian for costh -> sa5
      wtps = wtps/(s567+rm2(5)-s67)*sqrt(s567)/parb(4)

*     rat=sa5/(sa5max-sa5+y0*shat)
      ratmin=y0*shat/sa5max
      ratmax=sa5max/(y0*shat)

      call pick(2,rat,ratmin,ratmax,r5,wtps)

      sa5=rat*(sa5max+y0*shat)/(rat+1d0)
*      wtps=wtps*(sa3max-sa3)**2/sa3max

      wtps=wtps*(sa5max-sa5+y0*shat)**2/(sa5max+y0*shat)

      call pick(1,phi,phimin,phimax,r6,wtps)

      costh=-(sa5-(s567-s67)*parb(4)/s567**0.5)/(2d0*parb(4)*((s567-s67)/2d0/s567**0.5))
      call makeps2com(s567,costh,phi,rm2(5),s67,p5p,p67p,p5ap,p67ap)
      wtps = wtps*dsqrt(dlambda(s567,rm2(3),s67))/s567/32d0/pi**2


***********************************************

c---- rotate the (345) system
      call boostrest(p567,bmat)
      call unboostrest(p567,ubmat)
      do i=1,4
         v(i) = 0d0
         pap(i) = 0d0
         do j=1,4
            pap(i) = pap(i) + bmat(i,j)*pa(j)
         enddo
      enddo
      call unrotatetoz(pap,urmat)
      do i=1,4
         do j=1,4
            fmat(i,j) = 0d0
            do k=1,4
               fmat(i,j) = fmat(i,j)+ubmat(i,k)*urmat(k,j)
            enddo
         enddo
      enddo
      do i=1,4
         p5(i) = 0d0
         p67(i) = 0d0
         p5a(i) = 0d0
         p67a(i) = 0d0
         v(i) = 0d0
         do j=1,4
            p5(i) = p5(i) + fmat(i,j)*p5p(j)
            p67(i) = p67(i) + fmat(i,j)*p67p(j)
            p5a(i) = p5a(i) + fmat(i,j)*p5ap(j)
            p67a(i) = p67a(i) + fmat(i,j)*p67ap(j)
            v(i) = v(i) + fmat(i,j)*pap(j)
         enddo
      enddo


***********************************************
*     TM 03/05/2015: switched to sampling rat=sa5/sa4
*     from costh

*     boost pa into the s45 rest frame
      call boostrest(p67,bmat)
      do i=1,4
         v(i) = 0.d0
         pap(i) = 0.d0
         do j=1,4
            pap(i) = pap(i) + bmat(i,j)*pa(j)
         enddo
      enddo

*     rotate pap into the frame
      call rotatetoz(pap,rmat)
      do i=1,4
         parb(i) = 0.d0
         do j=1,4
            do k=1,4
               parb(i) = parb(i) + rmat(i,k)*bmat(k,j)*pa(j)
            enddo
         enddo
      enddo

c      call pick(1,costh,costhmin,costhmax,r9,wtps)
c      sa6min = y0*shat
c      sa6max = 2*s67**0.5*parb(4)-y0*shat
c      call pick(1,sa6,sa6min,sa6max,r9,wtps)

      E2=2*(s67**0.5/2d0)*parb(4)

      wtps = wtps/E2

      ratmin=2*E2/(2*E2-y0*shat) -1d0
      ratmax=2*E2/(y0*shat)-1d0

*     pick the ratio rat=sa5/sa4 logarthmically
      call pick(2,rat,ratmin,ratmax,r7,wtps)

      sa6=2*E2/(rat+1d0)
      sa7=2*E2-sa6

      wtps=wtps*sa6**2/(2*E2)
      costh=-(sa6-2*(s67**0.5)*parb(4)/2d0)/(2*(s67**0.5)*parb(4)/2d0)

      call pick(1,phi,phimin,phimax,r8,wtps)
      call makeps2com(s67,costh,phi,rm2(6),rm2(7),p6p,p7p,p6ap,p7ap)
      wtps = wtps*dsqrt(dlambda(s67,rm2(6),rm2(7)))/s67/32d0/pi**2

***********************************************

c---- rotate the (67) system
      call boostrest(p67,bmat)
      call unboostrest(p67,ubmat)
      do i=1,4
         v(i) = 0d0
         papp(i) = 0d0
         do j=1,4
            papp(i) = papp(i) + bmat(i,j)*pa(j)
         enddo
      enddo
      call unrotatetoz(papp,urmat)
      do i=1,4
         do j=1,4
            fmat(i,j) = 0d0
            do k=1,4
               fmat(i,j) = fmat(i,j)+ubmat(i,k)*urmat(k,j)
            enddo
         enddo
      enddo

      do i=1,4
         p6(i) = 0d0
         p7(i) = 0d0
         p6a(i) = 0d0
         p7a(i) = 0d0
         v(i) = 0d0
         do j=1,4
            p6(i) = p6(i) + fmat(i,j)*p6p(j)
            p7(i) = p7(i) + fmat(i,j)*p7p(j)
            p6a(i) = p6a(i) + fmat(i,j)*p6ap(j)
            p7a(i) = p7a(i) + fmat(i,j)*p7ap(j)
            v(i) = v(i) + fmat(i,j)*pap(j)
         enddo
      enddo

c---- rotate the (67ang) system
      call boostrest(p67a,bmat)
      call unboostrest(p67a,ubmat)
      do i=1,4
         v(i) = 0d0
         papp(i) = 0d0
         do j=1,4
            papp(i) = papp(i) + bmat(i,j)*pa(j)
         enddo
      enddo
      call unrotatetoz(papp,urmat)
      do i=1,4
         do j=1,4
            fmat(i,j) = 0d0
            do k=1,4
               fmat(i,j) = fmat(i,j)+ubmat(i,k)*urmat(k,j)
            enddo
         enddo
      enddo
      do i=1,4
         p6t(i) = 0d0
         p7t(i) = 0d0
         p6ta(i) = 0d0
         p7ta(i) = 0d0
         v(i) = 0d0
         do j=1,4
            p6t(i) = p6t(i) + fmat(i,j)*p6p(j)
            p7t(i) = p7t(i) + fmat(i,j)*p7p(j)
            p6ta(i) = p6ta(i) + fmat(i,j)*p6ap(j)
            p7ta(i) = p7ta(i) + fmat(i,j)*p7ap(j)
            v(i) = v(i) + fmat(i,j)*pap(j)
         enddo
      enddo


      call decay4(s1234,p1234,x(9:16),pdecay,wtps,ipass)
      if (ipass.eq.0) return


      do i=1,4
         do j=1,4
            p(i,j) = pdecay(i,j)
            pang7(i,j,1) = p(i,j)
            pang7(i,j,2) = p(i,j)
            pang7(i,j,3) = p(i,j)
            pang7(i,j,4) = p(i,j)
         enddo
         p(i,5) = p5(i)
         p(i,6) = p6(i)
         p(i,7) = p7(i)
         pang7(i,5,1) = p5(i)
         pang7(i,6,1) = p6(i)
         pang7(i,7,1) = p7(i)

         pang7(i,5,2) = p5(i)
         pang7(i,6,2) = p6a(i)
         pang7(i,7,2) = p7a(i)

         pang7(i,5,3) = p5a(i)
         pang7(i,6,3) = p6t(i)
         pang7(i,7,3) = p7t(i)

         pang7(i,5,4) = p5a(i)
         pang7(i,6,4) = p6ta(i)
         pang7(i,7,4) = p7ta(i)
      enddo

      do i=1,4
         call fillinv(7,pang7(1:4,1:7,i),pa,pb,sijang7(1:7,1:7,i)
     .                                        ,tijang7(1:2,1:7,i))
      enddo

      do i=1,4
        call acceptcuts7a(sijang7(1:7,1:7,i),tijang7(1:2,1:7,i),iacc(i),itechpass)
c        if(itechpass.eq.0) return ! failed a technical cut, bin the whole set
      enddo
      if (iacc(1)+iacc(2)+iacc(3)+iacc(4).eq.0) return

      ifail =0
      wt = wtps


c
c      k10 = 1
c      write(6,101) pang5(1:4,1,k10)
c      write(6,102) pang5(1:4,2,k10)
c      write(6,103) pang5(1:4,3,k10)
c      write(6,104) pang5(1:4,4,k10)
c      write(6,105) pang5(1:4,5,k10)
c      write(6,106) pa(1:4)
c      write(6,107) pb(1:4)
c      write(6,109) v1(1:4)
c      write(6,*) q2/dot(q,q)
c

 101  format('p1:',4f14.9)
 102  format('p2:',4f14.9)
 103  format('p3:',4f14.9)
 104  format('p4:',4f14.9)
 105  format('p5:',4f14.9)
 106  format('p6:',4f14.9)
 107  format('p7:',4f14.9)
 108  format('pa:',4f14.9)
 109  format('pb:',4f14.9)
 110  format('pp:',4f14.9)

      return
      end



      subroutine acceptcuts7a(sij7,tij7,ipass,itechpass)
      implicit real*8(a-h,o-z)
      dimension sij7(1:7,1:7),tij7(1:2,1:7)
      common/pmasses/rma2,rmb2,rm2(1:7),shat
!$omp threadprivate(/pmasses/)
      common /tcuts/ymin,y0

      ipass = 0
      itechpass = 0

      sa5 = -tij7(1,5)
      sa6 = -tij7(1,6)
      sa7 = -tij7(1,7)
      sb5 = -tij7(2,5)
      sb6 = -tij7(2,6)
      sb7 = -tij7(2,7)
      s56 =  sij7(5,6)
      s57 =  sij7(5,7)
      s67 =  sij7(6,7)

      if (min(sa5,sa6,sa7,sb5,sb6,sb7).lt.y0*shat) return
      if (min(s56,s57,s67).lt.y0*shat) return

      itechpass = 1

      smin1 = shat
      smin2 = shat

      do i=5,7
         do j=i+1,7
            if (sij7(i,j).lt.smin2) then
               smin2 = sij7(i,j)
               if (smin2.lt.smin1) then
                  stmp  = smin1
                  smin1 = smin2
                  smin2 = stmp
               endif
            endif
         enddo
         if (-tij7(1,i).lt.smin2) then
            smin2 = -tij7(1,i)
            if (smin2.lt.smin1) then
               stmp  = smin1
               smin1 = smin2
               smin2 = stmp
            endif
         endif
         if (-tij7(2,i).lt.smin2) then
            smin2 = -tij7(2,i)
            if (smin2.lt.smin1) then
               stmp  = smin1
               smin1 = smin2
               smin2 = stmp
            endif
         endif
      enddo

c---- these are the new selection cuts for phase7a
      if (smin1.eq.s67.and.smin2.eq.sa6) ipass=1
      if (smin2.eq.s67.and.smin1.eq.sa6) ipass=1
      if (smin1.eq.s67.and.smin2.eq.sa7) ipass=1
      if (smin2.eq.s67.and.smin1.eq.sa7) ipass=1
      if (smin1.eq.s67.and.smin2.eq.min(s56,s57)) then
         if ((sa6+sa7).lt.(sb6+sb7)) ipass=1
      endif
      if (smin1.eq.sa6.and.smin2.eq.sa7) ipass=1
      if (smin2.eq.sa6.and.smin1.eq.sa7) ipass=1

      return
      end




      subroutine fillcommon7apvv
      implicit real*8(a-h,o-z)
      common/pmomang7/pang7(4,7,4)
!$omp threadprivate(/pmomang7/)
      common/invarang7/sijang7(7,7,4),tijang7(2,7,4),iacc(4)
!$omp threadprivate(/invarang7/)
      common/eventmom7ap/pevt7a(4,7,24)
!$omp threadprivate(/eventmom7ap/)
      common/eventinv7ap/sevt7a(7,7,24),tevt7a(2,7,24),ievt7a(24)
!$omp threadprivate(/eventinv7ap/)
      dimension i1(1:7,1:3)

      data i1/1,2,3,4,5,6,7,
     .        1,2,3,4,7,5,6,
     .        1,2,3,4,6,7,5/
c---- pang7 and sijang are input and correspond to an event in (5;6,7;a)
c---- this subroutine fills
c----       pevt7a(ievent,component,pnum)            : 4-momenta of events
c----       sevt7a(i,j,ievent),tevt7a([ab],i,ievent) : invariants of events
c----
c----  1: (5;67;a)
c----  2: (5;67;b)
c----  3: (7;56;a)
c----  4: (7;56;b)
c----  5: (6;75;a)
c----  6: (6;75;b)

c----  7-12: with (67) system rotated
c---- 13-18: with (567) system rotated
c---- 19-24: with (67) and (567) system rotated

c---- full phave space coverage is obtained by summing ievent=1..6
c---- angular average is obtained by averaging:
c----        M2 = 1/4* ( sum_ievent=1..24   M2(ievent))
c----

      do i=1,3
         do j=1,7
            do k=1,4
               pevt7a(k,i1(j,i),2*i-1)   = pang7(k,j,1)
               pevt7a(k,i1(j,i),2*i+5)   = pang7(k,j,2)
               pevt7a(k,i1(j,i),2*i+11)  = pang7(k,j,3)
               pevt7a(k,i1(j,i),2*i+17)  = pang7(k,j,4)
               if (k.ne.3) then
                  pevt7a(k,i1(j,i),2*i)     = pang7(k,j,1)
                  pevt7a(k,i1(j,i),2*i+6)   = pang7(k,j,2)
                  pevt7a(k,i1(j,i),2*i+12)  = pang7(k,j,3)
                  pevt7a(k,i1(j,i),2*i+18)  = pang7(k,j,4)
               else
                  pevt7a(k,i1(j,i),2*i)     = -pang7(k,j,1)
                  pevt7a(k,i1(j,i),2*i+6)   = -pang7(k,j,2)
                  pevt7a(k,i1(j,i),2*i+12)  = -pang7(k,j,3)
                  pevt7a(k,i1(j,i),2*i+18)  = -pang7(k,j,4)
               endif
            enddo
         enddo
      enddo

      do i=1,3
         do j=1,7
            do k=1,7
               sevt7a(i1(j,i),i1(k,i),2*i-1)  = sijang7(j,k,1)
               sevt7a(i1(j,i),i1(k,i),2*i+5)  = sijang7(j,k,2)
               sevt7a(i1(j,i),i1(k,i),2*i+11) = sijang7(j,k,3)
               sevt7a(i1(j,i),i1(k,i),2*i+17) = sijang7(j,k,4)
               sevt7a(i1(j,i),i1(k,i),2*i)    = sijang7(j,k,1)
               sevt7a(i1(j,i),i1(k,i),2*i+6)  = sijang7(j,k,2)
               sevt7a(i1(j,i),i1(k,i),2*i+12) = sijang7(j,k,3)
               sevt7a(i1(j,i),i1(k,i),2*i+18) = sijang7(j,k,4)
            enddo
            tevt7a(1,i1(j,i),2*i-1)  = tijang7(1,j,1)
            tevt7a(2,i1(j,i),2*i-1)  = tijang7(2,j,1)
            tevt7a(1,i1(j,i),2*i+5)  = tijang7(1,j,2)
            tevt7a(2,i1(j,i),2*i+5)  = tijang7(2,j,2)
            tevt7a(1,i1(j,i),2*i+11) = tijang7(1,j,3)
            tevt7a(2,i1(j,i),2*i+11) = tijang7(2,j,3)
            tevt7a(1,i1(j,i),2*i+17) = tijang7(1,j,4)
            tevt7a(2,i1(j,i),2*i+17) = tijang7(2,j,4)
            tevt7a(1,i1(j,i),2*i)    = tijang7(2,j,1)
            tevt7a(2,i1(j,i),2*i)    = tijang7(1,j,1)
            tevt7a(1,i1(j,i),2*i+6)  = tijang7(2,j,2)
            tevt7a(2,i1(j,i),2*i+6)  = tijang7(1,j,2)
            tevt7a(1,i1(j,i),2*i+12) = tijang7(2,j,3)
            tevt7a(2,i1(j,i),2*i+12) = tijang7(1,j,3)
            tevt7a(1,i1(j,i),2*i+18) = tijang7(2,j,4)
            tevt7a(2,i1(j,i),2*i+18) = tijang7(1,j,4)
         enddo
         ievt7a(2*i-1)  = iacc(1)
         ievt7a(2*i)    = iacc(1)
         ievt7a(2*i+5)  = iacc(2)
         ievt7a(2*i+6)  = iacc(2)
         ievt7a(2*i+11) = iacc(3)
         ievt7a(2*i+12) = iacc(3)
         ievt7a(2*i+17) = iacc(4)
         ievt7a(2*i+18) = iacc(4)
      enddo


 101  format('p1:',4f14.6)
 102  format('p2:',4f14.6)
 103  format('p3:',4f14.6)
 104  format('p4:',4f14.6)
 107  format(4g18.13)
 108  format(3g18.13)


      return
      end




      subroutine phase7bvv(x,wt,ifail)
      use Process_mod, only : s_proc
      use KinData_mod, only : boostrest,unboostrest,rotatetoz,unrotatetoz
      implicit real*8(a-h,o-z)
      dimension p1(4),p2(4),p3(4),p4(4),p5(4),p6(4),p7(4),v(4),q(4),v1(4)
      dimension p1p(4),p2p(4),p3p(4),p4p(4),p5p(4),p6p(4),p7p(4),pap(4),
     .          pbp(4),pbap(4)
      dimension parb(4),pbrb(4)
      dimension p1ap(4),p2ap(4),p3ap(4),p4ap(4),p5ap(4),p6ap(4),p7ap(4)
      dimension p3a(4),p4a(4),p5a(4),p6a(4),p7a(4)
      dimension p1234(4),p567(4),p67p(4),p67(4)
      dimension p567p(4)
      dimension p67a(4),p67ap(4),p6t(4),p7t(4),p6ta(4),p7ta(4)
      dimension pdecay(1:4,1:4)
      dimension x(*)
      dimension bmat(4,4),ubmat(4,4),rmat(4,4),urmat(4,4),fmat(4,4)
      parameter(pi=3.141592653589793238d0)
      common/pmasses/rma2,rmb2,rm2(1:7),shat
!$omp threadprivate(/pmasses/)
      common /nwa/rm1234,rg1234,inwa
      common/pin/pa(4),pb(4)
!$omp threadprivate(/pin/)
      common/pmom/p(4,7)
!$omp threadprivate(/pmom/)
      common/pmomang7/pang7(4,7,4)
!$omp threadprivate(/pmomang7/)
      common/invar7/sij7(7,7),tij7(2,7)
!$omp threadprivate(/invar7/)
      common/invarang7/sijang7(7,7,4),tijang7(2,7,4),iacc(4)
!$omp threadprivate(/invarang7/)
      common/tcuts/ymin,y0
      common /techpar/iang
      common /Nphase/Nphase
!$omp threadprivate(/Nphase/)

cLIPS      print*, "phase7bvv", x(1), wt, ifail

      wtps = 1d0

      if(iang.eq.0)then
         Nphase = 6
      else
         Nphase = 24
         wtps=wtps/4d0
      endif


* this is (5;a;6,7;b)
*
* phase space for 2->7 scattering, a+b -> 1+2+3+4+5+6+7 with q=p1+p2+p3+p4
* with sequential splitting (567) -> 5+(76) -> 5+6+7
*
* (567) system is rotated with respect to pa axis
* (67)  system is rotated with respect to pb axis
*
* default momenta             : pang(i,j,1)
* (67) rotated                : pang(i,j,2)
* (567) rotated               : pang(i,j,3)
* (567) rotated, (67) rotated : pang(i,j,4)
*
* incoming momenta along z-axis, pa is +z
*
      r1 = x(1)
      r2 = x(2)
      r3 = x(3)
      r4 = x(4)
      r5 = x(5)
      r6 = x(6)
      r7 = x(7)
      r8 = x(8)
      wt = 0d0
      ifail =1


      s1234min=y0*shat
      s1234max=(1d0 - y0)*shat
c      s1234min = dmax1(y0*shat,(dsqrt(rm2(1))+dsqrt(rm2(2)))**2)
c      s1234max = dmin1((dsqrt(shat)-dsqrt(rm2(3))-dsqrt(rm2(4)))**2
c     .              ,(1d0-5d0*y0)*shat)
      if (s1234min > s1234max) return
      if (wtps == 0d0) return
      call pick_sinv_generic(s1234,s1234min,s1234max,rm1234,rg1234,r1,wtps,inwa)
      if (shat < s1234) return ! Insufficient scattering energy to produce the heavy particle
      wtps = wtps/2d0/pi


      s567min = dmax1(y0*shat,
     .           (dsqrt(rm2(5))+dsqrt(rm2(6))+dsqrt(rm2(7)))**2)
      s567max = dmin1((dsqrt(shat)-dsqrt(s1234))**2,(1d0-y0)*shat)
      if (s567max.le.s567min) return
      call pick(10,s567,s567min,s567max,r2,wtps)
      wtps = wtps/2d0/pi
      s67min = dmax1(y0*shat,(dsqrt(rm2(6))+dsqrt(rm2(7)))**2)
      s67max = dmin1(s567,(1d0-y0)*shat)
      if (s67max.le.s67min) return
      call pick(2,s67,s67min,s67max,r3,wtps)
      wtps = wtps/2d0/pi

      call p2generic_alt(3,r4,0d0,shat,0d0,0d0,s1234,
     .               s567,p1234,p567,pa,pb,wtps)
      if(wtps.eq.0d0) return

      if(p567(4).lt.0d0)then
         ifail = 1
         return
      endif

* TODO: change of variables only works in the massless case.
*       Add a condition s.t. we return to the old calculations
*       for massive partons

***********************************************
*     TM 03/05/2015: switched to sampling sa5
*     from costh

c      costhmin = -1d0
c      costhmax = 1d0
      phimin = 0d0
      phimax = 2d0*pi
c      call pick(1,costh,costhmin,costhmax,r7,wtps)

      call boostrest(p567,bmat)
      do i=1,4
         v(i) = 0.d0
         pap(i) = 0.d0
         do j=1,4
            pap(i) = pap(i) + bmat(i,j)*pa(j)
         enddo
      enddo

      call rotatetoz(pap,rmat)

      do i=1,4
         parb(i) = 0.d0
         do j=1,4
            do k=1,4
               parb(i) = parb(i) + rmat(i,k)*bmat(k,j)*pa(j)
            enddo
         enddo
      enddo

      bound = 2.d0*(s567+rm2(5)-s67)/sqrt(s567)*parb(4)
      sa5min = y0*shat
      sa5max = min(bound,(1.d0-y0)*shat)

      wtps = wtps/(s567+rm2(5)-s67)*sqrt(s567)/parb(4)
      if(sa5max.le.sa5min) return

      call pick(2,sa5,sa5min,sa5max,r5,wtps)
      call pick(1,phi,phimin,phimax,r6,wtps)

      costh=-(sa5-(s567-s67)*parb(4)/s567**0.5)/(2d0*parb(4)*((s567-s67)/2d0/s567**0.5))

      call makeps2com(s567,costh,phi,rm2(5),s67,p5p,p67p,p5ap,p67ap)
      wtps = wtps*dsqrt(dlambda(s567,rm2(5),s67))/s567/32d0/pi**2

***********************************************

c---- rotate the (567) system
      call boostrest(p567,bmat)
      call unboostrest(p567,ubmat)
      do i=1,4
         v(i) = 0d0
         pap(i) = 0d0
         do j=1,4
            pap(i) = pap(i) + bmat(i,j)*pa(j)
         enddo
      enddo
      call unrotatetoz(pap,urmat)
      do i=1,4
         do j=1,4
            fmat(i,j) = 0d0
            do k=1,4
               fmat(i,j) = fmat(i,j)+ubmat(i,k)*urmat(k,j)
            enddo
         enddo
      enddo
      do i=1,4
         p5(i) = 0d0
         p67(i) = 0d0
         p5a(i) = 0d0
         p67a(i) = 0d0
         v(i) = 0d0
         do j=1,4
            p5(i) = p5(i) + fmat(i,j)*p5p(j)
            p67(i) = p67(i) + fmat(i,j)*p67p(j)
            p5a(i) = p5a(i) + fmat(i,j)*p5ap(j)
            p67a(i) = p67a(i) + fmat(i,j)*p67ap(j)
            v(i) = v(i) + fmat(i,j)*pap(j)
         enddo
      enddo

***********************************************
*     TM 03/05/2015: switched to sampling rat=sb5/sb4
*     from costh

*     boost pb into the s67 rest frame
      call boostrest(p67,bmat)
      do i=1,4
         v(i) = 0.d0
         pbp(i) = 0.d0
         do j=1,4
            pbp(i) = pbp(i) + bmat(i,j)*pb(j)
         enddo
      enddo

*     rotate pbp into the frame
      call rotatetoz(pbp,rmat)
      do i=1,4
         pbrb(i) = 0.d0
         do j=1,4
            do k=1,4
               pbrb(i) = pbrb(i) + rmat(i,k)*bmat(k,j)*pb(j)
            enddo
         enddo
      enddo

      E2=2*(s67**0.5/2d0)*pbrb(4)
      wtps = wtps/E2

      ratmin=2*E2/(2*E2-y0*shat) -1d0
      ratmax=2*E2/(y0*shat)-1d0

*     pick the ratio rat=sb5/sb4 logarthmically
      call pick(2,rat,ratmin,ratmax,r7,wtps)

      sb6=2*E2/(rat+1d0)
      sb7=2*E2-sb6

      wtps=wtps*sb6**2/(2*E2)
      costh=-(sb6-2*(s67**0.5)*pbrb(4)/2d0)/(2*(s67**0.5)*pbrb(4)/2d0)

      call pick(1,phi,phimin,phimax,r8,wtps)
      call makeps2com(s67,costh,phi,rm2(6),rm2(7),p6p,p7p,p6ap,p7ap)
      wtps = wtps*dsqrt(dlambda(s67,rm2(6),rm2(7)))/s67/32d0/pi**2



c---- rotate the (67) system
      call boostrest(p67,bmat)
      call unboostrest(p67,ubmat)
      do i=1,4
         v(i) = 0d0
         pbp(i) = 0d0
         do j=1,4
            pbp(i) = pbp(i) + bmat(i,j)*pb(j)
         enddo
      enddo
      call unrotatetoz(pbp,urmat)
      do i=1,4
         do j=1,4
            fmat(i,j) = 0d0
            do k=1,4
               fmat(i,j) = fmat(i,j)+ubmat(i,k)*urmat(k,j)
            enddo
         enddo
      enddo
      do i=1,4
         p6(i) = 0d0
         p7(i) = 0d0
         p6a(i) = 0d0
         p7a(i) = 0d0
         v(i) = 0d0
         do j=1,4
            p6(i) = p6(i) + fmat(i,j)*p6p(j)
            p7(i) = p7(i) + fmat(i,j)*p7p(j)
            p6a(i) = p6a(i) + fmat(i,j)*p6ap(j)
            p7a(i) = p7a(i) + fmat(i,j)*p7ap(j)
            v(i) = v(i) + fmat(i,j)*pbp(j)
         enddo
      enddo

c      write(6,107) v(1:4)

c---- rotate the (67ang) system
      call boostrest(p67a,bmat)
      call unboostrest(p67a,ubmat)
      do i=1,4
         v(i) = 0d0
         pbap(i) = 0d0
         do j=1,4
            pbap(i) = pbap(i) + bmat(i,j)*pb(j)
         enddo
      enddo
      call unrotatetoz(pbap,urmat)
      do i=1,4
         do j=1,4
            fmat(i,j) = 0d0
            do k=1,4
               fmat(i,j) = fmat(i,j)+ubmat(i,k)*urmat(k,j)
            enddo
         enddo
      enddo
      do i=1,4
         p6t(i) = 0d0
         p7t(i) = 0d0
         p6ta(i) = 0d0
         p7ta(i) = 0d0
         v(i) = 0d0
         do j=1,4
            p6t(i) = p6t(i) + fmat(i,j)*p6p(j)
            p7t(i) = p7t(i) + fmat(i,j)*p7p(j)
            p6ta(i) = p6ta(i) + fmat(i,j)*p6ap(j)
            p7ta(i) = p7ta(i) + fmat(i,j)*p7ap(j)
            v(i) = v(i) + fmat(i,j)*pbap(j)
         enddo
      enddo


      call decay4(s1234,p1234,x(9:16),pdecay,wtps,ipass)
      if (ipass.eq.0) return


      do i=1,4
         do j=1,4
            p(i,j) = pdecay(i,j)
            pang7(i,j,1) = p(i,j)
            pang7(i,j,2) = p(i,j)
            pang7(i,j,3) = p(i,j)
            pang7(i,j,4) = p(i,j)
         enddo
         p(i,5) = p5(i)
         p(i,6) = p6(i)
         p(i,7) = p7(i)
         pang7(i,5,1) = p5(i)
         pang7(i,6,1) = p6(i)
         pang7(i,7,1) = p7(i)

         pang7(i,5,2) = p5(i)
         pang7(i,6,2) = p6a(i)
         pang7(i,7,2) = p7a(i)

         pang7(i,5,3) = p5a(i)
         pang7(i,6,3) = p6t(i)
         pang7(i,7,3) = p7t(i)

         pang7(i,5,4) = p5a(i)
         pang7(i,6,4) = p6ta(i)
         pang7(i,7,4) = p7ta(i)
      enddo


      do i=1,4
         call fillinv(7,pang7(1:4,1:7,i),pa,pb,sijang7(1:7,1:7,i)
     .                                        ,tijang7(1:2,1:7,i))
      enddo
      do i=1,4
        call acceptcuts7b(sijang7(1:7,1:7,i),tijang7(1:2,1:7,i),iacc(i),itechpass)
c        if(itechpass.eq.0) return ! failed a technical cut, bin the whole set
      enddo

      if (iacc(1)+iacc(2)+iacc(3)+iacc(4).eq.0) return


      ifail =0
      wt = wtps

 101  format('p1:',4f14.9)
 102  format('p2:',4f14.9)
 103  format('p3:',4f14.9)
 104  format('p4:',4f14.9)
 105  format('p5:',4f14.9)
 106  format('pa:',4f14.9)
 107  format('pb:',4f14.9)
 109  format('pp:',4f14.9)

      return
      end



      subroutine acceptcuts7b(sij7,tij7,ipass,itechpass)
      implicit real*8(a-h,o-z)
      dimension sij7(1:7,1:7),tij7(1:2,1:7)
      common /pmasses/rma2,rmb2,rm2(1:7),shat
!$omp threadprivate(/pmasses/)
      common /tcuts/ymin,y0

      ipass = 0
      itechpass=0

      sa5 = -tij7(1,5)
      sa6 = -tij7(1,6)
      sa7 = -tij7(1,7)
      sb5 = -tij7(2,5)
      sb6 = -tij7(2,6)
      sb7 = -tij7(2,7)
      s56 =  sij7(5,6)
      s57 =  sij7(5,7)
      s67 =  sij7(6,7)

      if (min(sa5,sa6,sa7,sb5,sb6,sb7).lt.y0*shat) return
      if (min(s56,s57,s67).lt.y0*shat) return

      itechpass=1

      smin1 = shat
      smin2 = shat

      do i=5,7
         do j=i+1,7
            if (sij7(i,j).lt.smin2) then
               smin2 = sij7(i,j)
               if (smin2.lt.smin1) then
                  stmp  = smin1
                  smin1 = smin2
                  smin2 = stmp
               endif
            endif
         enddo
         if (-tij7(1,i).lt.smin2) then
            smin2 = -tij7(1,i)
            if (smin2.lt.smin1) then
               stmp  = smin1
               smin1 = smin2
               smin2 = stmp
            endif
         endif
         if (-tij7(2,i).lt.smin2) then
            smin2 = -tij7(2,i)
            if (smin2.lt.smin1) then
               stmp  = smin1
               smin1 = smin2
               smin2 = stmp
            endif
         endif
      enddo

c---- these are the new selection cuts for phase7b
      if (smin1.eq.sa5.and.smin2.eq.min(sb5,sb6,sb7)) ipass=1
      if (smin1.eq.s67.and.smin2.eq.sa5) ipass=1
      if (smin2.eq.s67.and.smin1.eq.sa5) ipass=1

      return
      end


      subroutine fillcommon7bpvv
      implicit real*8(a-h,o-z)
      common/pmomang7/pang7(4,7,4)
!$omp threadprivate(/pmomang7/)
      common/invarang7/sijang7(7,7,4),tijang7(2,7,4),iacc(4)
!$omp threadprivate(/invarang7/)
      common/eventmom7bp/pevt7b(4,7,24)
!$omp threadprivate(/eventmom7bp/)
      common/eventinv7bp/sevt7b(7,7,24),tevt7b(2,7,24),ievt7b(24)
!$omp threadprivate(/eventinv7bp/)
      dimension i1(1:7,1:3)

      data i1/1,2,3,4,5,6,7,
     .        1,2,3,4,7,5,6,
     .        1,2,3,4,6,7,5/
c---- pang7 and sijang are input and correspond to an event in (5;a;6,7;b)
c---- this subroutine fills
c----       pevt7a(ievent,component,pnum)            : 4-momenta of events
c----       sevt7a(i,j,ievent),tevt5a([ab],i,ievent) : invariants of events
c----
c----  1: (5;a;67;b)
c----  2: (5;b;67;a)
c----  3: (7;a;56;b)
c----  4: (7;b;56;a)
c----  5: (6;a;75;b)
c----  6: (6;b;75;a)

c----  7-12: with (67) system rotated
c---- 13-18: with (567) system rotated
c---- 19-24: with (67) and (567) system rotated

c---- full phave space coverage is obtained by summing ievent=1..6
c---- angular average is obtained by averaging:
c----        M2 = 1/4* ( sum_ievent=1..24   M2(ievent))
c----

      do i=1,3
         do j=1,7
            do k=1,4
               pevt7b(k,i1(j,i),2*i-1)   = pang7(k,j,1)
               pevt7b(k,i1(j,i),2*i+5)   = pang7(k,j,2)
               pevt7b(k,i1(j,i),2*i+11)  = pang7(k,j,3)
               pevt7b(k,i1(j,i),2*i+17)  = pang7(k,j,4)
               if (k.ne.3) then
                  pevt7b(k,i1(j,i),2*i)     = pang7(k,j,1)
                  pevt7b(k,i1(j,i),2*i+6)   = pang7(k,j,2)
                  pevt7b(k,i1(j,i),2*i+12)  = pang7(k,j,3)
                  pevt7b(k,i1(j,i),2*i+18)  = pang7(k,j,4)
               else
                  pevt7b(k,i1(j,i),2*i)     = -pang7(k,j,1)
                  pevt7b(k,i1(j,i),2*i+6)   = -pang7(k,j,2)
                  pevt7b(k,i1(j,i),2*i+12)  = -pang7(k,j,3)
                  pevt7b(k,i1(j,i),2*i+18)  = -pang7(k,j,4)
               endif
            enddo
         enddo
      enddo

      do i=1,3
         do j=1,7
            do k=1,7
               sevt7b(i1(j,i),i1(k,i),2*i-1)  = sijang7(j,k,1)
               sevt7b(i1(j,i),i1(k,i),2*i+5)  = sijang7(j,k,2)
               sevt7b(i1(j,i),i1(k,i),2*i+11) = sijang7(j,k,3)
               sevt7b(i1(j,i),i1(k,i),2*i+17) = sijang7(j,k,4)
               sevt7b(i1(j,i),i1(k,i),2*i)    = sijang7(j,k,1)
               sevt7b(i1(j,i),i1(k,i),2*i+6)  = sijang7(j,k,2)
               sevt7b(i1(j,i),i1(k,i),2*i+12) = sijang7(j,k,3)
               sevt7b(i1(j,i),i1(k,i),2*i+18) = sijang7(j,k,4)
            enddo
            tevt7b(1,i1(j,i),2*i-1)  = tijang7(1,j,1)
            tevt7b(2,i1(j,i),2*i-1)  = tijang7(2,j,1)
            tevt7b(1,i1(j,i),2*i+5)  = tijang7(1,j,2)
            tevt7b(2,i1(j,i),2*i+5)  = tijang7(2,j,2)
            tevt7b(1,i1(j,i),2*i+11) = tijang7(1,j,3)
            tevt7b(2,i1(j,i),2*i+11) = tijang7(2,j,3)
            tevt7b(1,i1(j,i),2*i+17) = tijang7(1,j,4)
            tevt7b(2,i1(j,i),2*i+17) = tijang7(2,j,4)
            tevt7b(1,i1(j,i),2*i)    = tijang7(2,j,1)
            tevt7b(2,i1(j,i),2*i)    = tijang7(1,j,1)
            tevt7b(1,i1(j,i),2*i+6)  = tijang7(2,j,2)
            tevt7b(2,i1(j,i),2*i+6)  = tijang7(1,j,2)
            tevt7b(1,i1(j,i),2*i+12) = tijang7(2,j,3)
            tevt7b(2,i1(j,i),2*i+12) = tijang7(1,j,3)
            tevt7b(1,i1(j,i),2*i+18) = tijang7(2,j,4)
            tevt7b(2,i1(j,i),2*i+18) = tijang7(1,j,4)
         enddo
         ievt7b(i)    = iacc(1)
         ievt7b(i+3)  = iacc(1)
         ievt7b(i+6)  = iacc(2)
         ievt7b(i+9)  = iacc(2)
         ievt7b(i+12) = iacc(3)
         ievt7b(i+15) = iacc(3)
         ievt7b(i+18) = iacc(4)
         ievt7b(i+21) = iacc(4)
      enddo


 101  format('p1:',4f14.6)
 102  format('p2:',4f14.6)
 103  format('p3:',4f14.6)
 104  format('p4:',4f14.6)
 107  format(4g18.13)
 108  format(3g18.13)


      return
      end


! Phase space for 2 -> 5 scattering with a heavy particle mediator that later on
! decays to additional two heavy particles, whih consequently decay into 3 and 2
! final massless particles.
! Schematically, where H and V are the heavy particles, and a and b are the
! incoming particles:
!
! (ab) ->  H V
! Offshell V -> (4) (5)
! Offshell H -> (1) (23) -> (1) (2) (3)
      subroutine phase5h3v2(x, wt, ifail)
          use KinData_mod,   only: unboostrest
          use Constants_mod, only: pi
          use UnitPhase_mod, only: phase_unit
          implicit none
          real(8), intent(in)  :: x(20)
          real(8), intent(out) :: wt
          integer, intent(out) :: ifail
          integer :: ios1, ios2, isym, ievt5(4), Nphase
          real(8) :: pdec(4,3), p4(4), p5(4), p123(4), p45(4),
     .               pa(4), pb(4), dummypa(4), dummypb(4),
     .               pevt5(4,5,4), sevt5(5,5,4), tevt5(2,5,4),
     .               s123, s123min, s123max, s45, s45min, s45max,
     .               ubmat123(4,4), ubmat45(4,4),
     .               rmv1, rgv1, rmv2, rgv2, rma2, rmb2, rm2(7), shat,
     .               ymin, y0, wtps

          ! Common blocks
          common/eventmom5p/ pevt5
          !$omp threadprivate(/eventmom5p/)
          common/eventinv5p/ sevt5, tevt5, ievt5
          !$omp threadprivate(/eventinv5p/)
          common /Nphase/    Nphase
          !$omp threadprivate(/Nphase/)
          common /pin/       pa, pb
          !$omp threadprivate(/pin/)
          common /pmasses/   rma2, rmb2, rm2, shat
          !$omp threadprivate(/pmasses/)
          common /vdecay/    rmv1, rgv1, rmv2, rgv2, ios1, ios2, isym
          common /tcuts/     ymin, y0

          ifail  = 1
          Nphase = 2 ! s23 < s13 and s23 < s13 later
          wt     = 0d0
          wtps   = 1d0

          ! For testing unit phase
          if(phase_unit) then
            ios1 = 0
            ios2 = 0
          end if

          ! Generate invariant mass of three particles (eg. the invariant mass
          ! of a Higgs, later decaying to three partons like b--bb--g)
          s123min = y0*shat
          s123max = (1d0 - y0)*shat
          call pick_sinv_generic(s123, s123min, s123max, rmv1, rgv1, x(1), wtps, ios1)
          if (wtps .eq. 0d0) return

          ! Generate invariant mass of two particles
          ! (eg. invariant mass of a vector boson later decaying into a lepton pair)
          s45min = y0*shat
          s45max = (sqrt(shat)-sqrt(s123))**2
          call pick_sinv_generic(s45, s45min, s45max, rmv2, rgv2, x(2), wtps, ios2)
          if (wtps.eq.0d0) return

          ! Generate momenta p123 and p45 for the corresponding unboosting matrices
          call p2generic(x(3),0d0,shat,rma2,rmb2,s123,s45,p123,p45,pa,pb,wtps)
          call unboostrest(p123, ubmat123)
          call unboostrest(p45, ubmat45)

          ! Decay Higgs to three partons
          call decay3im(s123, x(4:8), pdec, wtps)

          ! Generate momenta p4 and p5 (dummypa and dummypb are not used)
          call p2generic(x(9),x(10),s45,0d0,0d0,rm2(4),rm2(5),p4,p5,dummypa,dummypb,wtps)
          wtps = wtps/(2d0*pi)

          ! Unboost everything to the lab frame + different ordering due to
          ! p_to_common4 that is later called
          pevt5(:,5,1) = matmul(ubmat123,pdec(:,1))
          pevt5(:,1,1) = matmul(ubmat123,pdec(:,2))
          pevt5(:,2,1) = matmul(ubmat123,pdec(:,3))
          pevt5(:,3,1) = matmul(ubmat45,p4)
          pevt5(:,4,1) = matmul(ubmat45,p5)

          call fillinv(5, pevt5(:,:,1), pa, pb, sevt5(:,:,1), tevt5(:,:,1))

          ! Only accept if s23 < s13 (note the different ordering introduced
          ! just before), the other case will be accounted for in subroutine
          ! fillcommon5h3v2
          if (sevt5(1,2,1).gt.sevt5(2,5,1)) return

          if (wtps.ne.0d0 .and. .not.isnan(wtps)) then
              ifail = 0
              wt    = wtps
          end if

      end subroutine phase5h3v2

      subroutine fillcommon5h3v2
          real(8) :: pevt5(4,5,4), sevt5(5,5,4), tevt5(2,5,4)
          integer :: ievt5(4)
          common/eventmom5p/ pevt5
          !$omp threadprivate(/eventmom5p/)
          common/eventinv5p/ sevt5, tevt5, ievt5
          !$omp threadprivate(/eventinv5p/)
          interface
              subroutine momswap(pmat, smat, tmat, i, j, newpmat, newsmat, newtmat)
                  real(8), dimension(:,:), intent(in)  :: pmat, smat, tmat
                  integer, intent(in) :: i, j
                  real(8), dimension(:,:), intent(out) :: newpmat, newsmat, newtmat
              end subroutine momswap
          end interface

          ! Original construction already stored
          ievt5(1) = 1

          ! Swap 1<->2 (here: 1<->5)
          ! s13 < s23
          call momswap(pevt5(:,:,1), sevt5(:,:,1), tevt5(:,:,1), 1, 5,
     .                 pevt5(:,:,2), sevt5(:,:,2), tevt5(:,:,2))
          ievt5(2) = 1

      end subroutine fillcommon5h3v2

! Phase space for 2 -> 6 scattering with a heavy particle mediator that later on
! decays to additional two heavy particles, which consequently decay into 4 and
! 2 final massless particles. Schematically, where H and V are the heavy
! particles, and a and b are the incoming particles:
!
! (ab) ->  H V
! Offshell V -> (5) (6)
! Offshell H -> (1) (2) (3) (4)
!
! A-type decay:  H -> (134) (2) -> (13) (4) (2) -> (1) (3) (4) (2)
! B-type decay:  H -> (13) (24) -> (1) (3) (4) (2)
      subroutine phase6h4v2(x, wt, ifail)
          use KinData_mod,   only: unboostrest, unrotatetoz
          use Constants_mod, only: pi, pi2
          use UnitPhase_mod, only: phase_unit
          implicit none
          real(8), intent(in)  :: x(20)
          real(8), intent(out) :: wt
          integer, intent(out) :: ifail
          character*1          :: stype
          integer :: ios1, ios2, inwa, isym, iacc(4), Nphase, iang, i, maxrot
          real(8) :: p5(4), p6(4), p1234(4), p56(4), pa(4), pb(4), dummypa(4),
     .               dummypb(4), pang6(4,6,4), pdec(4,4,2), s1234, s56, smin,
     .               smax, ubmat1234(4,4), ubmat56(4,4), rmv1, rgv1, rmv2, rgv2,
     .               rma2, rmb2, rm2(7), shat, sijang6(6,6,4), tijang6(2,6,4),
     .               ymin, y0, wtps

          ! Common blocks
          common /invarang6/ sijang6, tijang6, iacc
          !$omp threadprivate(/invarang6/)
          common /Nphase/    Nphase
          !$omp threadprivate(/Nphase/)
          common /pin/       pa, pb
          !$omp threadprivate(/pin/)
          common/pmomang6/   pang6
          !$omp threadprivate(/pmomang6/)
          common /pmasses/   rma2, rmb2, rm2, shat
          !$omp threadprivate(/pmasses/)
          common /vdecay/    rmv1, rgv1, rmv2, rgv2, ios1, ios2, isym
          common /tcuts/     ymin, y0
          common /techpar/   iang
          common /stype/     stype

          ifail = 1
          wt    = 0d0

          if (iang.eq.0) then
              maxrot = 1
              Nphase = 6
              wtps   = 1d0
          else
              maxrot = 2
              Nphase = 12
              wtps   = 1d0/2d0
          endif

          ! For testing unit phase
          if(phase_unit) then
            ios1 = 0
            ios2 = 0
          end if

          ! Generate invariant mass of four particles (eg. the invariant mass
          ! of a Higgs, later decaying to four partons like b--bb--g--g)
          smin = y0*shat
          smax = (1d0 - y0)*shat
          call pick_sinv_generic(s1234, smin, smax, rmv1, rgv1, x(1), wtps, ios1)
          if (wtps .eq. 0d0) return

          ! Generate invariant mass of two particles
          ! (eg. invariant mass of a vector boson later decaying into a lepton pair)
          smin = y0*shat
          smax = (sqrt(shat)-sqrt(s1234))**2
          call pick_sinv_generic(s56, smin, smax, rmv2, rgv2, x(2), wtps, ios2)
          if (wtps.eq.0d0) return

          ! Generate momenta p1234 and p56 for the corresponding unboosting matrices
          call p2generic(x(3),0d0,shat,rma2,rmb2,s1234,s56,p1234,p56,pa,pb,wtps)
          call unboostrest(p1234, ubmat1234)
          call unboostrest(p56, ubmat56)

          ! Decay (56) -> (5) (6)
          call p2generic(x(4),x(5),s56,0d0,0d0,0d0,0d0,p5,p6,dummypa,dummypb,wtps)
          wtps = wtps/(2d0*pi)

          ! Decay (1234) -> (1) (2) (3) (4) with A- or B-type sampling
          select case(stype)
              case('a')
                  call decay31(s1234, x(6:13), pdec, wtps)
              case('b')
                  call decay22(s1234, x(6:13), pdec, wtps)
          end select
          if (wtps.eq.0d0) return

          ! Unboost everything (+ reordering to match p_to_common4) and filling
          ! the invariants sijang6 and tijang6
          do i=1, maxrot
              pang6(:,5,i) = matmul(ubmat1234, pdec(:,1,i))
              pang6(:,2,i) = matmul(ubmat1234, pdec(:,2,i))
              pang6(:,6,i) = matmul(ubmat1234, pdec(:,3,i))
              pang6(:,1,i) = matmul(ubmat1234, pdec(:,4,i))
              pang6(:,3,i) = matmul(ubmat56, p5)
              pang6(:,4,i) = matmul(ubmat56, p6)

              call fillinv(6, pang6(:,:,i), pa, pb, sijang6(:,:,i), tijang6(:,:,i))
          end do

          ifail = 0
          wt    = wtps

          contains

              ! A-type decay sampling
              ! Generate the phase space of 4 massless particles decaying with a
              ! chain of (1234) -> (134) (2) -> (13) (4) (2) -> (1) (3) (4) (2)
              ! Another set of momenta is also generated where the momenta of
              ! particles 1 and 3 are rotated by 90 degress around their
              ! collinear (13) axis (provided that rotations are enabled).
              subroutine decay31(s, y, pdec, wt)
                  real(8), intent(in)    :: s, y(8)
                  real(8), intent(out)   :: pdec(4,4,2)
                  real(8), intent(inout) :: wt
                  real(8)                :: s13, s134, p13(4), p134(4), smin, smax,
     .                                      dummypa(4), dummypb(4), ubmat13(4,4),
     .                                      ubmat134(4,4), urmat13(4,4), p2(4),
     .                                      p4(4), cthmin, cthmax, cth,
     .                                      phimin, phimax, phi, p1(4), p3(4),
     .                                      p1rot(4), p3rot(4)


                  ! Decay (1234) -> (134) (2)
                  smin = y0*s
                  smax = (1d0 - y0)*s
                  call pick(2, s134, smin, smax, y(1), wt) ! Logarithmic pick
                  if (wt.eq.0d0) return

                  ! Generate momenta p134 and p2 plus the corresponding unboosting matrix
                  call p2generic(y(2),y(3),s1234,0d0,0d0,s134,0d0,p134,p2,dummypa,dummypb,wt)
                  wt = wt/(2d0*pi)
                  call unboostrest(p134, ubmat134)

                  ! Decay (134) -> (13) (4)
                  smin = y0*s134
                  smax = (1d0 - y0)*s134
                  call pick(2, s13, smin, smax, y(4), wt) ! Logarithmic pick
                  if (wt.eq.0d0) return

                  ! Generate momenta p13 and p4 plus the corresponding
                  ! unboosting and unrotating matrices
                  call p2generic(y(5),y(6),s134,0d0,0d0,s13,0d0,p13,p4,dummypa,dummypb,wt)
                  wt = wt/(2d0*pi)
                  call unboostrest(p13, ubmat13)
                  call unrotatetoz(p13, urmat13)

                  ! Decay (13) -> (1) (3)
                  ! Generate rotated p1 and p3 as well with makeps2com
                  cthmin = -1d0
                  cthmax = 1d0
                  phimin = 0d0
                  phimax = 2d0*pi
                  call pick(1, cth, cthmin, cthmax, y(7), wt) ! Linear pick
                  call pick(1, phi, phimin, phimax, y(8), wt) ! Linear pick
                  wt = wt/32d0/pi2
                  wt = wt/2d0/pi
                  call makeps2com(s13, cth, phi, 0d0, 0d0, p1, p3, p1rot, p3rot)

                  pdec(:,1,1) = matmul(ubmat134, matmul(ubmat13, matmul(urmat13, p1)))
                  pdec(:,2,1) = p2
                  pdec(:,3,1) = matmul(ubmat134, matmul(ubmat13, matmul(urmat13, p3)))
                  pdec(:,4,1) = matmul(ubmat134, p4)

                  if (iang.eq.1) then
                      pdec(:,1,2) = matmul(ubmat134, matmul(ubmat13, matmul(urmat13, p1rot)))
                      pdec(:,2,2) = p2
                      pdec(:,3,2) = matmul(ubmat134, matmul(ubmat13, matmul(urmat13, p3rot)))
                      pdec(:,4,2) = matmul(ubmat134, p4)
                  end if

              end subroutine decay31


              ! B-type decay sampling
              ! Generate the phase space of 4 massless particles decaying with a
              ! chain of (1234) -> (13) (24) -> (1) (3) (4) (2)
              ! Another set of momenta is also generated where the momenta of
              ! particles 1 and 3 are rotated by 90 degress around their
              ! collinear (13) axis (provided that rotations are enabled).
              subroutine decay22(s, y, pdec, wt)
                  real(8), intent(in)    :: s, y(8)
                  real(8), intent(out)   :: pdec(4,4,2)
                  real(8), intent(inout) :: wt
                  real(8)                :: s13, s24, p13(4), p24(4), smin, smax,
     .                                      dummypa(4), dummypb(4), ubmat13(4,4),
     .                                      ubmat24(4,4), urmat13(4,4), p2(4),
     .                                      p4(4), cthmin, cthmax, cth,
     .                                      phimin, phimax, phi, p1(4), p3(4),
     .                                      p1rot(4), p3rot(4)

                  ! Decay (1234) -> (13) (24)
                  ! Pick s13
                  smin = y0*s
                  smax = (1d0 - y0)*s
                  call pick(2, s13, smin, smax, y(1), wt) ! Logarithmic pick
                  if (wt.eq.0d0) return
                  wt = wt/(2d0*pi)

                  ! Pick s24
                  smin = y0*s
                  smax = (sqrt(s) - sqrt(s13))**2
                  call pick(2, s24, smin, smax, y(2), wt) ! Logarithmic pick
                  if (wt.eq.0d0) return

                  ! Generate momenta p13 and p24 plus the corresponding
                  ! unboosting and unrotating matrices
                  call p2generic(y(3),y(4),s,0d0,0d0,s13,s24,p13,p24,dummypa,dummypb,wt)
                  call unboostrest(p13, ubmat13)
                  call unrotatetoz(p13, urmat13)
                  call unboostrest(p24, ubmat24)

                  ! Decay (24) -> (2) (4)
                  call p2generic(y(5),y(6),s24,0d0,0d0,0d0,0d0,p2,p4,dummypa,dummypb,wt)
                  wt = wt/(2d0*pi)

                  ! Decay (13) -> (1) (3)
                  ! Generate rotated p1 and p3 as well with makeps2com
                  cthmin = -1d0
                  cthmax = 1d0
                  phimin = 0d0
                  phimax = 2d0*pi
                  call pick(1, cth, cthmin, cthmax, y(7), wt) ! Linear pick
                  call pick(1, phi, phimin, phimax, y(8), wt) ! Linear pick
                  wt = wt/32d0/pi2
                  wt = wt/2d0/pi
                  call makeps2com(s13, cth, phi, 0d0, 0d0, p1, p3, p1rot, p3rot)

                  pdec(:,1,1) = matmul(ubmat13, matmul(urmat13, p1))
                  pdec(:,2,1) = matmul(ubmat24, p2)
                  pdec(:,3,1) = matmul(ubmat13, matmul(urmat13, p3))
                  pdec(:,4,1) = matmul(ubmat24, p4)

                  if (iang.eq.1) then
                      pdec(:,1,2) = matmul(ubmat13, matmul(urmat13, p1rot))
                      pdec(:,2,2) = matmul(ubmat24, p2)
                      pdec(:,3,2) = matmul(ubmat13, matmul(urmat13, p3rot))
                      pdec(:,4,2) = matmul(ubmat24, p4)
                  end if

              end subroutine decay22

      end subroutine phase6h4v2

      subroutine acceptcuts6h4v2(sij6, ipass)
          implicit none
          real(8), intent(in)  :: sij6(6,6)
          integer, intent(out) :: ipass
          real(8)              :: s13, s14, s23, s24, s34, s12
          real(8)              :: s134, s234, s123, s124, smin
          character*1          :: stype
          common /stype/ stype

          ipass = 0

          ! Different names due to reordering in phase6h4v2a
          s12 = sij6(2,5)
          s13 = sij6(5,6)
          s14 = sij6(1,5)
          s23 = sij6(2,6)
          s24 = sij6(1,2)
          s34 = sij6(1,6)

          smin = min(s12, s13, s14, s23, s24, s34)

          ! The smallest shall be s13
          if (smin.eq.s13) then

              s134 = s13 + s14 + s34
              s234 = s23 + s24 + s34
              s123 = s12 + s13 + s23
              s124 = s12 + s14 + s24

              smin = min(s134, s234, s123, s124)

              select case(stype)
                  case('a')
                      ! The smallest shall be s134 or s123 in region A
                      if ((smin.eq.s134).or.(smin.eq.s123)) ipass = 1
                  case('b')
                      ! The smallest shall be s234 or s124 in region B
                      if ((smin.eq.s234).or.(smin.eq.s124)) ipass = 1
              end select
          end if

      end subroutine acceptcuts6h4v2

      subroutine fillcommon6h4v2
          implicit none
          real(8)     :: pang6(4,6,4), sijang6(6,6,4), tijang6(2,6,4),
     .                   pevt6(4,6,12), sevt6(6,6,12), tevt6(2,6,12),
     .                   ptemp(4,6), stemp(6,6), ttemp(2,6)
          integer     :: ievt6(12), iacc(4), iang, i, maxrot
          character*1 :: stype
          common/techpar/ iang
          common/stype/ stype
          common/pmomang6/   pang6
          !$omp threadprivate(/pmomang6/)
          common/invarang6/  sijang6, tijang6, iacc
          !$omp threadprivate(/invarang6/)
          common/eventmom6p/ pevt6
          !$omp threadprivate(/eventmom6p/)
          common/eventinv6p/ sevt6, tevt6, ievt6
          !$omp threadprivate(/eventinv6p/)
          interface
              subroutine momswap(pmat, smat, tmat, i, j, newpmat, newsmat, newtmat)
                  real(8), dimension(:,:), intent(in)  :: pmat, smat, tmat
                  integer, intent(in) :: i, j
                  real(8), dimension(:,:), intent(out) :: newpmat, newsmat, newtmat
              end subroutine momswap
          end interface

          if (iang.eq.0) then
            maxrot = 1
          else
            maxrot = 2
          end if

          ievt6 = 0

          ! For the original, and the rotated sets
          do i = 1, maxrot

              ! Do the generated momenta make it into wedges A1 or B1?
              call acceptcuts6h4v2(sijang6(:,:,i), iacc(i))

              if (iacc(i).eq.0) cycle

              ! Original construction
              ! Wedge A1: minimal invariants: s13 and s134||s123
              ! Wedge B1: minimal invariants: s13 and s234||s124
              pevt6(:,:,6*i-5) = pang6(:,:,i)
              tevt6(:,:,6*i-5) = tijang6(:,:,i)
              sevt6(:,:,6*i-5) = sijang6(:,:,i)

              ! Swap 3<->4 (here: 1<->6)
              ! Wedge A2: minimal invariants: s14 and s134||s124
              ! Wedge B2: minimal invariants: s14 and s234||s123
              call momswap(pang6(:,:,i),     sijang6(:,:,i),   tijang6(:,:,i), 1, 6,
     .                     pevt6(:,:,6*i-4), sevt6(:,:,6*i-4), tevt6(:,:,6*i-4))

              ! Swap 1<->2 (here: 2<->5)
              ! Wedge A3: minimal invariants: s23 and s234||s123
              ! Wedge B3: minimal invariants: s23 and s134||s124
              call momswap(pang6(:,:,i),     sijang6(:,:,i),   tijang6(:,:,i), 2, 5,
     .                     pevt6(:,:,6*i-3), sevt6(:,:,6*i-3), tevt6(:,:,6*i-3))

              ! Swap 1<->2 (here: 2<->5) and swap 3<->4 (here: 1<->6)
              ! Wedge A4: minimal invariants: s24 and s234||s124
              ! Wedge B4: minimal invariants: s24 and s134||s123
              call momswap(pang6(:,:,i),     sijang6(:,:,i),   tijang6(:,:,i), 2, 5,
     .                     ptemp, stemp, ttemp)
              call momswap(ptemp, stemp, ttemp, 1, 6,
     .                     pevt6(:,:,6*i-2), sevt6(:,:,6*i-2), tevt6(:,:,6*i-2))

              ! Swap 1<->4 (here: 1<->5)
              ! Wedge A5: minimal invariants: s34 and s134||s234
              ! Wedge B5: minimal invariants: s34 and s123||s124
              call momswap(pang6(:,:,i),     sijang6(:,:,i),   tijang6(:,:,i), 1, 5,
     .                     pevt6(:,:,6*i-1), sevt6(:,:,6*i-1), tevt6(:,:,6*i-1))

              ! Swap 2<->3 (here: 2<->6)
              ! Wedge A6: minimal invariants: s12 and s124||s123
              ! Wedge B6: minimal invariants: s12 and s234||s134
              call momswap(pang6(:,:,i),   sijang6(:,:,i), tijang6(:,:,i), 2, 6,
     .                     pevt6(:,:,6*i), sevt6(:,:,6*i), tevt6(:,:,6*i))

              ievt6((6*i-5):(6*i)) = 1

          enddo

      end subroutine fillcommon6h4v2

! Phase space for 2 -> 6 scattering with an initial state radiated parton and
! a heavy particle mediator that later on decays to additional two heavy
! particles, which consequently decay into 3 and 2 final massless particles.
! Schematically, where H and V are the heavy particles, and a and b are the
! incoming particles:
!
! (ab) -> (6) V
! Offshell V -> V H
! Offshell V -> (4) (5)
! Offshell H -> (1) (2) (3)
      subroutine phase6h3v2(x, wt, ifail)
          use Constants_mod, only: pi
          use KinData_mod,   only: unboostrest
          use UnitPhase_mod, only: phase_unit
          implicit none
          real(8), intent(in)  :: x(20)
          real(8), intent(out) :: wt
          integer, intent(out) :: ifail
          integer :: ios1, ios2, isym, inwa, Nphase, ievt6(12)
          real(8) :: pdec(4,3), p4(4), p5(4), p6(4), p123(4), p45(4), p12345(4),
     .               pa(4), pb(4), dummypa(4), dummypb(4),
     .               pevt6(4,6,12), sevt6(6,6,12), tevt6(2,6,12),
     .               s123, s123min, s123max, s12345, s12345min, s12345max,
     .               s45, s45min, s45max,
     .               ubmat123(4,4), ubmat12345(4,4), ubmat45(4,4),
     .               rmv1, rgv1, rmv2, rgv2, rma2, rmb2, rm2(7), shat,
     .               rm12345, rg12345,
     .               ymin, y0, wtps

          ! COMMON BLOCKS
          common /Nphase/    Nphase
          !$omp threadprivate(/Nphase/)
          common/eventmom6p/ pevt6
          !$omp threadprivate(/eventmom6p/)
          common/eventinv6p/ sevt6, tevt6, ievt6
          !$omp threadprivate(/eventinv6p/)
          common /pin/       pa, pb
          !$omp threadprivate(/pin/)
          common /pmasses/   rma2, rmb2, rm2, shat
          !$omp threadprivate(/pmasses/)
          common /vdecay/    rmv1, rgv1, rmv2, rgv2, ios1, ios2, isym
          common /nwa/       rm12345, rg12345, inwa
          common /tcuts/     ymin, y0

          ifail  = 1
          Nphase = 2
          wt     = 0d0
          wtps   = 1d0

          ! For testing unit phase
          if(phase_unit) then
            ios1 = 0
            ios2 = 0
          end if

          ! Sample vector boson
          s12345min = y0*shat
          s12345max = (1d0 - y0)*shat
          if (s12345min.ge.s12345max) return
          call pick_sinv_generic(s12345,s12345min,s12345max,rm12345,rg12345,x(1),wtps,inwa)

          ! Generate invariant mass of three particles
          s123min = y0*s12345
          s123max = (1d0 - y0)*s12345
          call pick_sinv_generic(s123, s123min, s123max, rmv1, rgv1, x(2), wtps, ios1)
          if (wtps .eq. 0d0) return

          ! Generate invariant mass of two particles
          s45min = y0*s12345
          s45max = (sqrt(s12345)-sqrt(s123))**2
          call pick_sinv_generic(s45, s45min, s45max, rmv2, rgv2, x(3), wtps, ios2)
          if (wtps.eq.0d0) return

          ! Radiate (6) from the initial state
          call p2generic(x(4),0d0,shat,0d0,0d0,s12345,0d0,p12345,p6,pa,pb,wtps)
          call unboostrest(p12345, ubmat12345)
          wtps = wtps/2d0/pi

          ! Generate momenta p123 and p45 for the corresponding unboosting matrices
          call p2generic(x(5),x(6),s12345,0d0,0d0,s123,s45,p123,p45,dummypa,dummypb,wtps)
          call unboostrest(p123, ubmat123)
          call unboostrest(p45, ubmat45)

          ! Decay Higgs to three partons
          call decay3im(s123, x(7:11), pdec, wtps)

          ! Generate momenta p4 and p5 (dummypa and dummypb are not used)
          call p2generic(x(12),x(13),s45,0d0,0d0,0d0,0d0,p4,p5,dummypa,dummypb,wtps)
          wtps = wtps/(2d0*pi)

          ! Unboost everything to the lab frame + different ordering due to
          ! p_to_common4 that is later called
          pevt6(:,5,1) = p6
          pevt6(:,6,1) = matmul(ubmat12345,matmul(ubmat123,pdec(:,1)))
          pevt6(:,1,1) = matmul(ubmat12345,matmul(ubmat123,pdec(:,2)))
          pevt6(:,2,1) = matmul(ubmat12345,matmul(ubmat123,pdec(:,3)))
          pevt6(:,3,1) = matmul(ubmat12345,matmul(ubmat45,p4))
          pevt6(:,4,1) = matmul(ubmat12345,matmul(ubmat45,p5))

          call fillinv(6, pevt6(:,:,1), pa, pb, sevt6(:,:,1), tevt6(:,:,1))

          ! Only accept if s23 < s13
          if (sevt6(1,2,1).gt.sevt6(2,6,1)) return

          if (wtps.ne.0d0 .and. .not.isnan(wtps)) then
              ifail = 0
              wt    = wtps
          end if

      end subroutine phase6h3v2

      subroutine fillcommon6h3v2
          implicit none
          real(8) :: pevt6(4,6,12), sevt6(6,6,12), tevt6(2,6,12)
          integer :: ievt6(12)
          common/eventmom6p/ pevt6
          !$omp threadprivate(/eventmom6p/)
          common/eventinv6p/ sevt6, tevt6, ievt6
          !$omp threadprivate(/eventinv6p/)
          interface
              subroutine momswap(pmat, smat, tmat, i, j, newpmat, newsmat, newtmat)
                  real(8), dimension(:,:), intent(in)  :: pmat, smat, tmat
                  integer, intent(in) :: i, j
                  real(8), dimension(:,:), intent(out) :: newpmat, newsmat, newtmat
              end subroutine momswap
          end interface

          ! Original construction already stored in [pst]evt(:,:,1)
          ievt6(1) = 1

          ! Swap 1<->2 (here: 1<->6)
          ! s13 < s23
          call momswap(pevt6(:,:,1), sevt6(:,:,1), tevt6(:,:,1), 1, 6,
     .                 pevt6(:,:,2), sevt6(:,:,2), tevt6(:,:,2))
          ievt6(2) = 1

      end subroutine fillcommon6h3v2

      ! General decay subroutine to three massless final partons with sampling
      ! order (123) -> 1 (23) -> 1 2 3
      subroutine decay3im(s, r, p, wtps)
          use Constants_mod, only: pi
          use KinData_mod,   only: unboostrest
          implicit none
          real(8), intent(in)    :: s, r(5)
          real(8), intent(out)   :: p(4,3)
          real(8), intent(inout) :: wtps
          real(8)                :: s23, s23min, s23max, dummypa(4), dummypb(4),
     .                              y0, ymin, p1(4), p2(4), p3(4), p23(4),
     .                              ubmat(4,4)

          ! COMMON BLOCKS
          common /tcuts/ ymin, y0

          ! Decay first (123) -> 1 (23)
          s23min = y0*s
          s23max = (1d0 - y0)*s
          call pick(2, s23, s23min, s23max, r(1), wtps) ! Logarithmic pick
          if (wtps.eq.0d0) return

          ! Generate momenta p1 and p23, plus the unboosting matrix for p23
          call p2generic(r(2),r(3),s,0d0,0d0,0d0,s23,p1,p23,dummypa,dummypb,wtps)
          wtps = wtps/(2d0*pi)
          call unboostrest(p23, ubmat)

          ! Generate momenta p2 and p3 (dummypa and dummypb are not used)
          call p2generic(r(4),r(5),s23,0d0,0d0,0d0,0d0,p2,p3,dummypa,dummypb,wtps)
          wtps = wtps/(2d0*pi)

          p(:,1) = p1
          p(:,2) = matmul(ubmat, p2)
          p(:,3) = matmul(ubmat, p3)

      end subroutine decay3im

      ! Momentum index-swapping subroutine
      subroutine momswap(pmat, smat, tmat, i, j, newpmat, newsmat, newtmat)
          real(8), dimension(:,:), intent(in)  :: pmat, smat, tmat
          integer, intent(in) :: i, j
          real(8), dimension(:,:), intent(out) :: newpmat, newsmat, newtmat
          integer :: k, l, m, n, npart

          ! Number of particles
          npart = size(pmat,2)

          ! Some input consistency checks
          if (size(pmat,1).ne.4) then
              print*, "Number of momentum-components is not 4!"
              stop
          endif

          if ((size(smat,1).ne.npart).or.(size(smat,2).ne.npart)) then
              print*, "Matrix containing the s invariants is not squared!"
              stop
          endif

          if ((size(tmat,1).ne.2).or.(size(tmat,2).ne.npart)) then
              print*, "Matrix containing the t invariants is not correctly shaped!"
              stop
          endif

          newpmat = pmat
          ! Swap p-matrix components
          newpmat(:,i) = pmat(:,j)
          newpmat(:,j) = pmat(:,i)

          newtmat = tmat
          ! Swap t-matrix components
          newtmat(:,i) = tmat(:,j)
          newtmat(:,j) = tmat(:,i)

          ! Swap s-matrix components
          do k = 1, npart
              do l = 1, npart
                  if (k .eq. i) then
                      m = j
                  else if (k .eq. j) then
                      m = i
                  else
                      m = k
                  end if

                  if (l .eq. i) then
                      n = j
                  else if (l .eq. j) then
                      n = i
                  else
                      n = l
                  end if

                  newsmat(k,l) = smat(m,n)

              end do
          end do

      end subroutine momswap


      subroutine phase7allvv(x,wt,ifail)
      use Constants_mod, only : pi, twopi
      use Process_mod, only : s_proc
      use KinData_mod, only : boostrest,unboostrest,rotatetoz,unrotatetoz
      implicit real*8(a-h,o-z)
      dimension p1(4),p2(4),p3(4),p4(4),p5(4),p6(4),p7(4),v(4),q(4),v1(4)
      dimension p1p(4),p2p(4),p3p(4),p4p(4),p5p(4),p6p(4),p7p(4),pap(4),
     .          p3pp(4),p3app(4),papp(4)
      dimension p1ap(4),p2ap(4),p3ap(4),p4ap(4),p5ap(4),p6ap(4),p7ap(4)
      dimension p5a(4),p6a(4),p7a(4)
      dimension p1234(4),p567(4),p67p(4),p67(4)
      dimension p567p(4)
      dimension p67a(4),p67ap(4),p6t(4),p7t(4),p6ta(4),p7ta(4)
      dimension pdecay(1:4,1:4)
      dimension x(*)
      dimension bmat(4,4),ubmat(4,4),rmat(4,4),urmat(4,4),fmat(4,4)
      dimension parb(4)
      double precision p6pp(4),p7pp(4)
      common/pmasses/rma2,rmb2,rm2(1:7),shat
!$omp threadprivate(/pmasses/)
      common /nwa/rm1234,rg1234,inwa
      common/pin/pa(4),pb(4)
!$omp threadprivate(/pin/)
      common/pmom/p(4,7)
!$omp threadprivate(/pmom/)
      common/pmomang7/pang7(4,7,4)
!$omp threadprivate(/pmomang7/)
      common/invar7/sij7(7,7),tij7(2,7)
!$omp threadprivate(/invar7/)
      common/invarang7/sijang7(7,7,4),tijang7(2,7,4),iacc(4)
!$omp threadprivate(/invarang7/)
      common/tcuts/ymin,y0
      common /techpar/iang
      common /Nphase/Nphase
!$omp threadprivate(/Nphase/)

cLIPS      print*, "phase7allvv", x(1), wt, ifail

      wtps = 1d0

      if(iang.eq.0)then
         Nphase=6
      else
         Nphase=24
         wtps=wtps/4d0
      endif


* this is (5;6,7;a)
*
* phase space for 2->7 scattering, a+b -> 1+2+3+4+5+6+7 with q=p1+p2+p3+p4
* with sequential splitting (567) -> 5+(67) -> 5+6+7
*
* (567) system is rotated with respect to pa axis
* (67)  system is rotated with respect to pa axis
*
* default momenta             : pang(i,j,1)
* (67) rotated                : pang(i,j,2)
* (567) rotated               : pang(i,j,3)
* (567) rotated, (67) rotated : pang(i,j,4)
*
* incoming momenta along z-axis, pa is +z
*
*
* massless case: integrates to P_5 = 1/4718592/pi**7*sab^3 = 0.7016789753e-10*sab^3
*
* incoming momenta along z-axis, pa is +z
*
      r1 = x(1)
      r2 = x(2)
      r3 = x(3)
      r4 = x(4)
      r5 = x(5)
      r6 = x(6)
      r7 = x(7)
      r8 = x(8)
      wt = 0d0
      ifail =1


      s1234min=y0*shat
      s1234max=(1d0 - y0)*shat
      if (s1234min > s1234max) return
      call pick_sinv_generic(s1234,s1234min,s1234max,rm1234,rg1234,r1,wtps,inwa)
      if (wtps == 0d0) return

      if(shat.lt.s1234) return    ! Insufficient scattering energy to produce the heavy particle
      wtps = wtps/2d0/pi


      s567min = dmax1(y0*shat,
     .           (dsqrt(rm2(5))+dsqrt(rm2(6))+dsqrt(rm2(7)))**2)
      s567max = dmin1((dsqrt(shat)-dsqrt(s1234))**2,(1d0-y0)*shat)
      if (s567max.le.s567min) return
      call pick(1,s567,s567min,s567max,r2,wtps)
      wtps = wtps/2d0/pi
      s67min = dmax1(y0*shat,(dsqrt(rm2(6))+dsqrt(rm2(7)))**2)
      s67max = dmin1(s567,(1d0-y0)*shat)
      if (s67max.le.s67min) return
*      call pick(1,s45,s45min,s45max,r5,wtps)
*     sample log(s45)
      call pick(2,s67,s67min,s67max,r3,wtps)
      wtps = wtps/2d0/pi

      call p2generic_alt(2,r4,0d0,shat,0d0,0d0,s1234,s567,
     .                   p1234,p567,pa,pb,wtps)
      if(wtps.eq.0d0) return

      if(p567(4).lt.0d0)then
          ifail = 1
          return
      endif

* TODO: change of variables only works in the massless case.
*       Add a condition s.t. we return to the old calculations
*       for massive partons

      costhmin = -1d0
      costhmax = 1d0
      call pickcos(costh,y0,r5,wtps)

      call pick(1,phi,0d0,twopi,r6,wtps)
      call makeps2com(s567,costh,phi,rm2(5),s67,p5p,p67p,p5ap,p67ap)
      wtps = wtps*dsqrt(dlambda(s567,rm2(5),s67))/s567/32d0/pi**2

***********************************************

c---- rotate the (567) system
      call boostrest(p567,bmat)
      call unboostrest(p567,ubmat)
      do i=1,4
         v(i) = 0d0
         pap(i) = 0d0
         do j=1,4
            pap(i) = pap(i) + bmat(i,j)*pa(j)
         enddo
      enddo
      call unrotatetoz(pap,urmat)
      do i=1,4
         do j=1,4
            fmat(i,j) = 0d0
            do k=1,4
               fmat(i,j) = fmat(i,j)+ubmat(i,k)*urmat(k,j)
            enddo
         enddo
      enddo
      do i=1,4
         p5(i) = 0d0
         p67(i) = 0d0
         p5a(i) = 0d0
         p67a(i) = 0d0
         v(i) = 0d0
         do j=1,4
            p5(i) = p5(i) + fmat(i,j)*p5p(j)
            p67(i) = p67(i) + fmat(i,j)*p67p(j)
            p5a(i) = p5a(i) + fmat(i,j)*p5ap(j)
            p67a(i) = p67a(i) + fmat(i,j)*p67ap(j)
            v(i) = v(i) + fmat(i,j)*pap(j)
         enddo
      enddo

      call pickcos_alt(costh,r7,wtps)
*      call pickcos(costh,y0,r9,wtps)

      call pick(1,phi,0d0,twopi,r8,wtps)
      call makeps2com(s67,costh,phi,rm2(6),rm2(7),p6p,p7p,p6ap,p7ap)
      wtps = wtps*dsqrt(dlambda(s67,rm2(6),rm2(7)))/s67/32d0/pi**2

***********************************************


c---- rotate the (67) system
      call boostrest(p67,bmat)
      call unboostrest(p67,ubmat)
      do i=1,4
         v(i) = 0d0
         papp(i) = 0d0
         do j=1,4
            papp(i) = papp(i) + bmat(i,j)*pa(j)
         enddo
      enddo
      call unrotatetoz(papp,urmat)
      do i=1,4
         do j=1,4
            fmat(i,j) = 0d0
            do k=1,4
               fmat(i,j) = fmat(i,j)+ubmat(i,k)*urmat(k,j)
            enddo
         enddo
      enddo

      do i=1,4
         p6(i) = 0d0
         p7(i) = 0d0
         p6a(i) = 0d0
         p7a(i) = 0d0
         v(i) = 0d0
         do j=1,4
            p6(i) = p6(i) + fmat(i,j)*p6p(j)
            p7(i) = p7(i) + fmat(i,j)*p7p(j)
            p6a(i) = p6a(i) + fmat(i,j)*p6ap(j)
            p7a(i) = p7a(i) + fmat(i,j)*p7ap(j)
            v(i) = v(i) + fmat(i,j)*pap(j)
         enddo
      enddo

c---- rotate the (67ang) system
      call boostrest(p67a,bmat)
      call unboostrest(p67a,ubmat)
      do i=1,4
         v(i) = 0d0
         papp(i) = 0d0
         do j=1,4
            papp(i) = papp(i) + bmat(i,j)*pa(j)
         enddo
      enddo
      call unrotatetoz(papp,urmat)
      do i=1,4
         do j=1,4
            fmat(i,j) = 0d0
            do k=1,4
               fmat(i,j) = fmat(i,j)+ubmat(i,k)*urmat(k,j)
            enddo
         enddo
      enddo
      do i=1,4
         p6t(i) = 0d0
         p7t(i) = 0d0
         p6ta(i) = 0d0
         p7ta(i) = 0d0
         v(i) = 0d0
         do j=1,4
            p6t(i) = p6t(i) + fmat(i,j)*p6p(j)
            p7t(i) = p7t(i) + fmat(i,j)*p7p(j)
            p6ta(i) = p6ta(i) + fmat(i,j)*p6ap(j)
            p7ta(i) = p7ta(i) + fmat(i,j)*p7ap(j)
            v(i) = v(i) + fmat(i,j)*pap(j)
         enddo
      enddo

      call decay4(s1234,p1234,x(9:16),pdecay,wtps,ipass)
      if (ipass.eq.0) return

      do i=1,4
         do j=1,4
           p(i,j) = pdecay(i,j)
           pang7(i,j,1) = p(i,j)
           pang7(i,j,2) = p(i,j)
           pang7(i,j,3) = p(i,j)
           pang7(i,j,4) = p(i,j)
         enddo
         p(i,5) = p5(i)
         p(i,6) = p6(i)
         p(i,7) = p7(i)
         pang7(i,5,1) = p5(i)
         pang7(i,6,1) = p6(i)
         pang7(i,7,1) = p7(i)

         pang7(i,5,2) = p5(i)
         pang7(i,6,2) = p6a(i)
         pang7(i,7,2) = p7a(i)

         pang7(i,5,3) = p5a(i)
         pang7(i,6,3) = p6t(i)
         pang7(i,7,3) = p7t(i)

         pang7(i,5,4) = p5a(i)
         pang7(i,6,4) = p6ta(i)
         pang7(i,7,4) = p7ta(i)
      enddo

      do i=1,4
         call fillinv(7,pang7(1:4,1:7,i),pa,pb,sijang7(1:7,1:7,i)
     .                                        ,tijang7(1:2,1:7,i))
      enddo

      do i=1,4
        call acceptcuts7all(sijang7(1:7,1:7,i),tijang7(1:2,1:7,i),iacc(i),itechpass)
c        if(itechpass.eq.0) return ! failed a technical cut, bin the whole set
      enddo
      if (iacc(1)+iacc(2)+iacc(3)+iacc(4).eq.0) return

      ifail =0
      wt = wtps


c
c      k10 = 1
c      write(6,101) pang5(1:4,1,k10)
c      write(6,102) pang5(1:4,2,k10)
c      write(6,103) pang5(1:4,3,k10)
c      write(6,104) pang5(1:4,4,k10)
c      write(6,105) pang5(1:4,5,k10)
c      write(6,106) pa(1:4)
c      write(6,107) pb(1:4)
c      write(6,109) v1(1:4)
c      write(6,*) q2/dot(q,q)
c

  101  format('p1:',4f14.9)
  102  format('p2:',4f14.9)
  103  format('p3:',4f14.9)
  104  format('p4:',4f14.9)
  105  format('p5:',4f14.9)
  106  format('p6:',4f14.9)
  107  format('p7:',4f14.9)
  108  format('pa:',4f14.9)
  109  format('pb:',4f14.9)
  110  format('pp:',4f14.9)

      return
      end


      subroutine acceptcuts7all(sij7,tij7,ipass,itechpass)
      implicit real*8(a-h,o-z)
      dimension sij7(1:7,1:7),tij7(1:2,1:7)
      common/pmasses/rma2,rmb2,rm2(1:7),shat
!$omp threadprivate(/pmasses/)
      common /tcuts/ymin,y0
      integer ipassIF, ipassFF

      ipass = 0
      itechpass = 0
      ipassIF = 0
      ipassFF = 0

      sa5 = -tij7(1,5)
      sa6 = -tij7(1,6)
      sa7 = -tij7(1,7)
      sb5 = -tij7(2,5)
      sb6 = -tij7(2,6)
      sb7 = -tij7(2,7)
      s56 =  sij7(5,6)
      s57 =  sij7(5,7)
      s67 =  sij7(6,7)

      if (min(sa5,sa6,sa7,sb5,sb6,sb7).lt.y0*shat) return
      if (min(s56,s57,s67).lt.y0*shat) return

      itechpass = 1

* single out on 5||a, 6||a, 7||a
      rinvmina = min(sa5,sa6,sa7)
      rinvminb = min(sb5,sb6,sb7)
      if (rinvmina.lt.rinvminb) ipassIF=1

* single out on 6||7
      rinvmin  = min(s56,s57,s67)
      if (rinvmin.eq.s67) ipassFF=1

      if ((ipassIF.eq.1).and.(ipassFF.eq.1)) ipass=1

      return
      end





      subroutine fillcommon7allpvv
*XC based on fillcommon5aph and duplicated for further upgrade
      implicit real*8(a-h,o-z)
      common/pmomang7/pang7(4,7,4)
!$omp threadprivate(/pmomang7/)
      common/invarang7/sijang7(7,7,4),tijang7(2,7,4),iacc(4)
!$omp threadprivate(/invarang7/)
      common/eventmom7allp/pevt7all(4,7,48)
!$omp threadprivate(/eventmom7allp/)
      common/eventinv7allp/sevt7all(7,7,48),tevt7all(2,7,48),ievt7all(48)
!$omp threadprivate(/eventinv7allp/)
      dimension i1(1:7,1:3)

      data i1/1,2,3,4,5,6,7,
     .        1,2,3,4,7,5,6,
     .        1,2,3,4,6,7,5/
c---- pang7 and sijang are input and correspond to an event in (5;6,7;a)
c---- this subroutine fills
c----       pevt7all(ievent,component,pnum)            : 4-momenta of events
c----       sevt7all(i,j,ievent),tevt7all([ab],i,ievent) : invariants of events
c----
c----  1: (5;67;a)
c----  2: (5;67;b)
c----  3: (7;56;a)
c----  4: (7;56;b)
c----  5: (6;75;a)
c----  6: (6;75;b)

c----  7-12: with (67) system rotated
c---- 13-18: with (567) system rotated
c---- 19-24: with (67) and (567) system rotated

c---- full phase space coverage is obtained by summing ievent=1..6
c---- angular average is obtained by averaging:
c----        M2 = 1/4* ( sum_ievent=1..24   M2(ievent))
c----

      do i=1,3
         do j=1,7
            do k=1,4
               pevt7all(k,i1(j,i),2*i-1)   = pang7(k,j,1)
               pevt7all(k,i1(j,i),2*i+5)   = pang7(k,j,2)
               pevt7all(k,i1(j,i),2*i+11)  = pang7(k,j,3)
               pevt7all(k,i1(j,i),2*i+17)  = pang7(k,j,4)
               if (k.ne.3) then
                  pevt7all(k,i1(j,i),2*i)     = pang7(k,j,1)
                  pevt7all(k,i1(j,i),2*i+6)   = pang7(k,j,2)
                  pevt7all(k,i1(j,i),2*i+12)  = pang7(k,j,3)
                  pevt7all(k,i1(j,i),2*i+18)  = pang7(k,j,4)
               else
                  pevt7all(k,i1(j,i),2*i)     = -pang7(k,j,1)
                  pevt7all(k,i1(j,i),2*i+6)   = -pang7(k,j,2)
                  pevt7all(k,i1(j,i),2*i+12)  = -pang7(k,j,3)
                  pevt7all(k,i1(j,i),2*i+18)  = -pang7(k,j,4)
               endif
            enddo
         enddo
      enddo

      do i=1,3
         do j=1,7
            do k=1,7
               sevt7all(i1(j,i),i1(k,i),2*i-1)  = sijang7(j,k,1)
               sevt7all(i1(j,i),i1(k,i),2*i+5)  = sijang7(j,k,2)
               sevt7all(i1(j,i),i1(k,i),2*i+11) = sijang7(j,k,3)
               sevt7all(i1(j,i),i1(k,i),2*i+17) = sijang7(j,k,4)
               sevt7all(i1(j,i),i1(k,i),2*i)    = sijang7(j,k,1)
               sevt7all(i1(j,i),i1(k,i),2*i+6)  = sijang7(j,k,2)
               sevt7all(i1(j,i),i1(k,i),2*i+12) = sijang7(j,k,3)
               sevt7all(i1(j,i),i1(k,i),2*i+18) = sijang7(j,k,4)
            enddo
            tevt7all(1,i1(j,i),2*i-1)  = tijang7(1,j,1)
            tevt7all(2,i1(j,i),2*i-1)  = tijang7(2,j,1)
            tevt7all(1,i1(j,i),2*i+5)  = tijang7(1,j,2)
            tevt7all(2,i1(j,i),2*i+5)  = tijang7(2,j,2)
            tevt7all(1,i1(j,i),2*i+11) = tijang7(1,j,3)
            tevt7all(2,i1(j,i),2*i+11) = tijang7(2,j,3)
            tevt7all(1,i1(j,i),2*i+17) = tijang7(1,j,4)
            tevt7all(2,i1(j,i),2*i+17) = tijang7(2,j,4)
            tevt7all(1,i1(j,i),2*i)    = tijang7(2,j,1)
            tevt7all(2,i1(j,i),2*i)    = tijang7(1,j,1)
            tevt7all(1,i1(j,i),2*i+6)  = tijang7(2,j,2)
            tevt7all(2,i1(j,i),2*i+6)  = tijang7(1,j,2)
            tevt7all(1,i1(j,i),2*i+12) = tijang7(2,j,3)
            tevt7all(2,i1(j,i),2*i+12) = tijang7(1,j,3)
            tevt7all(1,i1(j,i),2*i+18) = tijang7(2,j,4)
            tevt7all(2,i1(j,i),2*i+18) = tijang7(1,j,4)
         enddo
         ievt7all(2*i-1)  = iacc(1)
         ievt7all(2*i)    = iacc(1)
         ievt7all(2*i+5)  = iacc(2)
         ievt7all(2*i+6)  = iacc(2)
         ievt7all(2*i+11) = iacc(3)
         ievt7all(2*i+12) = iacc(3)
         ievt7all(2*i+17) = iacc(4)
         ievt7all(2*i+18) = iacc(4)
      enddo


 101  format('p1:',4f14.6)
 102  format('p2:',4f14.6)
 103  format('p3:',4f14.6)
 104  format('p4:',4f14.6)
 107  format(4g18.13)
 108  format(3g18.13)


      return
      end


      subroutine phase6vh_v2l_h4l(x, wt, ifail)
          use Constants_mod, only : pi
          use KinData_mod, only : unboostrest
          implicit none
          double precision, external :: dlambda
          real(8), intent(in)  :: x(20)
          real(8), intent(out) :: wt
          integer, intent(out) :: ifail
          integer ::   ios1, ios2, ios3, ios4, Nphase, isym,
     .     i,j,ievt6(12),iacc(4),itechpass, i1,i2,i3,i4
          real(8) ::   s1234, s56, s12, s34, wtps, r1, r2, r3, r4 ,r5,
     .                 r6,r7, r8, r9, r10, r11, r12, r13, s56min, s56max,
     .                 s12min, s12max, s34min, s34max, s1234min,
     .                 s1234max, costhmin, costhmax,
     .                 phimin, phimax, costh, phi, p1234(4), p56(4),
     .                 p12(4), p34(4), p1p(4), p2p(4), p3p(4),
     .                 p4p(4), p5p(4), p6p(4), p1ap(4), p2ap(4),
     .                 p3ap(4), p4ap(4), p5ap(4), p6ap(4), pdecay(4,6),
     .                 p(4,6), ubmat56(4,4), ubmat1234(4,4),
     .                 ubmat12(4,4), ubmat34(4,4), dummypa(4),dummypb(4),
     .                 pa(4), pb(4), ymin, y0, rma2, rmb2, shat,
     .                 rmv1, rgv1, rmv2, rgv2, rgv3, rmv3, rgv4, rmv4,
     .                 sevt6(6,6,12),tevt6(2,6,12),pevt6(4,6,12),rm2(7),
     .                 sij6(6,6),tij6(2,6),pang6(4,6,4),sijang6(6,6,4),
     .                 tijang6(2,6,4)
          common/pin/pa, pb
          !$omp threadprivate(/pin/)
          common/tcuts/ymin, y0
          common/pmasses/rma2, rmb2, rm2, shat
          !$omp threadprivate(/pmasses/)
          common /vdecay/rmv1, rgv1, rmv2, rgv2, ios1, ios2, isym
          common /Nphase/Nphase
          !$omp threadprivate(/Nphase/)
          common /hdecay/rmv3, rgv3, rmv4, rgv4, ios3, ios4
          common/invar6/sij6,tij6
          common/eventmom6p/pevt6
          !$omp threadprivate(/eventmom6p/)
          common/eventinv6p/ sevt6, tevt6, ievt6
          !$omp threadprivate(/eventinv6p/)
          common/invarang6/ sijang6, tijang6, iacc
          !$omp threadprivate(/invarang6/)
          common/pmomang6/ pang6
          !$omp threadprivate(/pmomang6/)
!!! u + _b -> l- l+ l- l+ nu l
          wtps = 1d0
          Nphase=2

          ifail=1
          wt = 0d0

          r1 = x(1)
          r2 = x(2)
          r3 = x(3)
          r4 = x(4)
          r5 = x(5)
          r6 = x(6)
          r7 = x(7)
          r8 = x(8)
          r9 = x(9)
          r10= x(10)
          r11= x(11)
          r12= x(12)
          r13= x(13)

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

           s1234min = y0*shat ! Limit for the mass of the Higgs
           s1234max = shat-s1234min

           if (s1234min.gt.s1234max) return
           call pick_sinv_generic(s1234,s1234min,s1234max,rmv1,rgv1,r1,wtps,ios1) ! pick the mass of the first Z with Breit-Wigner
           if (wtps.eq.0d0) return

           s56min = y0*shat ! Limit for the mass of the Z
           s56max = (dsqrt(shat)-dsqrt(s1234))**2

           if (s56min.gt.s56max) return
           call pick_sinv_generic(s56,s56min,s56max,rmv2,rgv2,r2,wtps,ios2) ! pick the mass of the first Z with Breit-Wigner
           if (wtps.eq.0d0) return

           call p2generic(r3,0d0,shat,rma2,rmb2,s1234,s56,p1234,p56,pa,pb,wtps) ! generate the momentum pa,pb,p12,p34
           if(wtps.eq.0d0) return
           wtps=wtps/2d0/pi

!!!!!! Decay of the vector boson s56 to electron and neutrino

           costhmin = -1d0
           costhmax = 1d0
           phimin = 0d0
           phimax = 2d0*pi

           call pick(1,costh,costhmin,costhmax,r4,wtps) ! Choose theta
           call pick(1,phi,phimin,phimax,r5,wtps ) ! Choose phi

           wtps = wtps*dsqrt(dlambda(s56,0d0,0d0))/s56/32d0/pi**2
           wtps=wtps/2d0/pi

           call makeps2com(s56,costh,phi,0d0,0d0,p5p,p6p,p5ap,p6ap) !Generate momentum p5,p6

           call unboostrest(p56,ubmat56) ! Create unboostrest matrix

!!!!!! Decay of the Higgs to 4 charged leptons

           call unboostrest(p1234,ubmat1234) ! Create unboost matrix for p1234

           s12min = y0*s1234
           s12max = s1234-s12min

           if (s12min.gt.s12max) return
           call pick_sinv_generic(s12,s12min,s12max,rmv3,rgv3,r6,wtps,ios3)
           if (wtps.eq.0d0) return

           s34min = y0*s1234
           s34max = (dsqrt(s1234)-dsqrt(s12))**2

           if (s34min.gt.s34max) return
           call pick_sinv_generic(s34,s34min,s34max,rmv4,rgv4,r7,wtps,ios4)
           if (wtps.eq.0d0) return

           call p2generic(r8,r9,s1234,0d0,0d0,s12,s34,p12,p34,dummypa,dummypb,wtps)
           if(wtps.eq.0d0) return

           costhmin = -1d0
           costhmax = 1d0
           phimin = 0d0
           phimax = 2d0*pi

           call pick(1,costh,costhmin,costhmax,r10,wtps)
           call pick(1,phi,phimin,phimax,r11,wtps)

           wtps = wtps*dsqrt(dlambda(s12,0d0,0d0))/s12/32d0/pi**2
           wtps = wtps/2d0/pi

           call makeps2com(s12,costh,phi,0d0,0d0,p1p,p2p,p1ap,p2ap)
           call unboostrest(p12,ubmat12)

           call pick(1,costh,costhmin,costhmax,r12,wtps)
           call pick(1,phi,phimin,phimax,r13,wtps)

           wtps = wtps*dsqrt(dlambda(s34,0d0,0d0))/s34/32d0/pi**2
           wtps = wtps/2d0/pi

           call makeps2com(s34,costh,phi,0d0,0d0,p3p,p4p,p3ap,p4ap)
           call unboostrest(p34,ubmat34)

!!!!!! Reboost everything

! Initialise the matrix of momentum

! Back to s12,s34 and s56 frame

          ! We can swap both Z0 s12 <-> s34
          ! It means exchange (3,4) <-> (5,6)
          i1=1
          i2=2
          i3=3
          i4=4
          !if (isym.eq.1) then       ! symmetrize the H->VV* kinematics
          !   if (RAND(0).lt.0.5d0) then
          !      i1=3
          !      i2=4
          !      i3=1
          !      i4=2
          !   endif
          !endif

          pang6(:,i1,1) = matmul(ubmat1234,matmul(ubmat12,p1p))
          pang6(:,i2,1) = matmul(ubmat1234,matmul(ubmat12,p2p))
          pang6(:,i3,1) = matmul(ubmat1234,matmul(ubmat34,p3p))
          pang6(:,i4,1) = matmul(ubmat1234,matmul(ubmat34,p4p))
          pang6(:,5,1) = matmul(ubmat56,p5p)
          pang6(:,6,1) = matmul(ubmat56,p6p)

          !do i=1,6
          !print*,'i,momenta_phasenewhto4',i,pang6(:,i,1)
          !read*
          !enddo

          call fillinv(6,pang6(:,:,1),pa,pb,sijang6(:,:,1), tijang6(:,:,1))
          ! Only accept if s12 < s34 - sij = 2 p(i)*p(j)
          if (sijang6(1,2,1).gt.sijang6(3,4,1)) return

          wt = wtps
          ifail = 0

      end subroutine phase6vh_v2l_h4l


      subroutine fillcommon6vh_v2l_h4l
          implicit none
          real(8) :: pevt6(4,6,12), sevt6(6,6,12), tevt6(2,6,12),
     .                pang6(4,6,4), sijang6(6,6,4), tijang6(2,6,4),
     .               pa(4),pb(4)
          integer :: ievt6(12),iacc(4)
          common/pin/pa, pb
          !$omp threadprivate(/pin/)
          common/eventmom6p/pevt6
          !$omp threadprivate(/eventmom6p/)
          common/eventinv6p/ sevt6, tevt6, ievt6
          !$omp threadprivate(/eventinv6p/)
          common/invarang6/ sijang6, tijang6, iacc
          !$omp threadprivate(/invarang6/)
          common/pmomang6/ pang6
          !$omp threadprivate(/pmomang6/)

          interface
              subroutine momswap(pmat, smat, tmat, i, j, newpmat, newsmat, newtmat)
                  real(8), dimension(:,:), intent(in)  :: pmat, smat, tmat
                  integer, intent(in) :: i, j
                  real(8), dimension(:,:), intent(out) :: newpmat, newsmat, newtmat
              end subroutine momswap
          end interface

          call acceptcuts_v2l_h4l(sijang6(1:6,1:6,1),iacc(1))

          if (iacc(1)==0) return
          ! Original construction in evt(:,:,1)
             ievt6(1)=iacc(1)

             pevt6(:,5,1)=pang6(:,1,1)
             pevt6(:,6,1)=pang6(:,2,1)
             pevt6(:,1,1)=pang6(:,3,1)
             pevt6(:,2,1)=pang6(:,4,1)
             pevt6(:,3,1)=pang6(:,5,1)
             pevt6(:,4,1)=pang6(:,6,1)

          call fillinv(6,pevt6(:,:,1),pa,pb,sevt6(:,:,1), tevt6(:,:,1))

          ! Swap 1 <-> 3 and 2 <-> 4 in pang6, so 1 <-> 5 and 2 <-> 6
             ievt6(2)=1
             call momswap(pevt6(:,:,1), sevt6(:,:,1), tevt6(:,:,1), 1, 5,
     .                    pevt6(:,:,2), sevt6(:,:,2), tevt6(:,:,2))
             call momswap(pevt6(:,:,1), sevt6(:,:,1), tevt6(:,:,1), 2, 6,
     .                    pevt6(:,:,2), sevt6(:,:,2), tevt6(:,:,2))

      end subroutine fillcommon6vh_v2l_h4l


      subroutine acceptcuts_v2l_h4l(sij6, ipass)
          implicit none
          real(8), intent(in) :: sij6(6,6)
          integer, intent(out) :: ipass

          ipass = 1

      end subroutine acceptcuts_v2l_h4l


