      !> dedicated LIPS for epem -> jets (all massless momenta)
      !> tuned for sampling deeply unresolved limits





      subroutine init_omt()
      implicit none
      double precision :: E
      !> common blocks
      double precision :: rnorm_sav,Qsqr,Qinv,q1(4),q2(4)
      common/lips_omt/rnorm_sav,Qsqr,Qinv,q1,q2
!$omp threadprivate(/lips_omt/)
      double precision :: rma2,rmb2,rm2(1:7),shat
      common/pmasses/rma2,rmb2,rm2,shat
!$omp threadprivate(/pmasses/)
      double precision :: pa(4),pb(4)
      common /pinomt/pa,pb
!$omp threadprivate(/pinomt/)

      rnorm_sav = 0d0  ! reset

      Qsqr = shat
      Qinv = sqrt(Qsqr)
      E = 0.5d0 * Qinv

      pa(1:2) = 0d0
      pa(3)   = +E
      pa(4)   = +E
      pb(1:2) = 0d0
      pb(3)   = -E
      pb(4)   = +E

      q1(1:2) = 0d0
      q1(3)   = +0.5d0
      q1(4)   = +0.5d0
      q2(1:2) = 0d0
      q2(3)   = -0.5d0
      q2(4)   = +0.5d0

      end subroutine init_omt


      subroutine sample_q(isample, x, wt, qnrm, qrot)
      use Constants_mod, only : pi, twopi
      use Run_mod, only : tcut_run
      implicit none
      integer, intent(in) :: isample
      double precision, intent(in) :: x(3)
      double precision, intent(inout) :: wt
      double precision, intent(out) :: qnrm(1:4), qrot(1:4)
      double precision :: sqrt_tcut, u, v, t, lt, y, phi

      select case(isample)

      case (1)
        sqrt_tcut = sqrt(tcut_run)
        call pick(10, u, sqrt_tcut, 1d0, x(1), wt)
        call pick(10, v, sqrt_tcut, 1d0, x(2), wt)
        call pick(1,  phi, 0d0, twopi,  x(3), wt)
        wt = wt*u*v

        qnrm(1) = u*v*cos(phi)
        qnrm(2) = u*v*sin(phi)
        qnrm(3) = 0.5d0*(u**2 - v**2)
        qnrm(4) = 0.5d0*(u**2 + v**2)

        qrot(1) = u*v*cos(phi+0.25d0*pi)
        qrot(2) = u*v*sin(phi+0.25d0*pi)
        qrot(3) = qnrm(3)
        qrot(4) = qnrm(4)

      case (2)
        call pick(2, t, tcut_run, 1d0, x(1), wt)
        lt = log(t)  ! < 0
        call pick(1, y, lt, -lt,       x(2), wt)
        call pick(1, phi, 0d0, twopi,  x(3), wt)
        wt = wt*t/2d0

        qnrm(1) = t*cos(phi)
        qnrm(2) = t*sin(phi)
        qnrm(3) = t*sinh(y)
        qnrm(4) = t*cosh(y)

        qrot(1) = t*cos(phi+0.25d0*pi)
        qrot(2) = t*sin(phi+0.25d0*pi)
        qrot(3) = qnrm(3)
        qrot(4) = qnrm(4)

      case default
        print*, "sample_q: invalid isample = ", isample
        stop

      end select

      end subroutine sample_q



      subroutine phase2omt(x, wt, ifail)
      use Constants_mod, only : pi, pi2, twopi
      implicit none
      double precision, intent(in) :: x(1)
      double precision, intent(out) :: wt
      integer, intent(out) :: ifail
      double precision :: wtps, E
      double precision :: costh, sinth
      !> common blocks
      double precision :: rnorm_sav,Qsqr,Qinv,q1(4),q2(4)
      common/lips_omt/rnorm_sav,Qsqr,Qinv,q1,q2
!$omp threadprivate(/lips_omt/)
      integer :: Nphase
      common /Nphase/Nphase
!$omp threadprivate(/Nphase/)
      double precision :: pang2(4,2,1),wt2(1)
      common /pmomang2omt/pang2,wt2
!$omp threadprivate(/pmomang2omt/)

      ifail = 1
      wt = 0d0

      wtps = 1d0
      Nphase = 1

      call init_omt()
      E = 0.5d0 * Qinv

      call pick(1, costh, -1d0, 1d0, x(1), wtps)
      if (wtps == 0d0) return
      wtps = wtps * twopi  ! trivial phi <-> beam azimuth symmetry
      sinth = sqrt((1d0+costh)*(1d0-costh))

      pang2(1,1,1) = 0d0  ! phi = pi/2
      pang2(2,1,1) = E * sinth
      pang2(3,1,1) = E * costh
      pang2(4,1,1) = E

      pang2(1:3,2,1) = - pang2(1:3,1,1)
      pang2(4,  2,1) = E

      wt2(1) = 1d0
      wtps = wtps / (32d0*pi2)

      ifail = 0
      wt = wtps

      end subroutine phase2omt


      subroutine fillcommon2omt(ievt, wt)
      implicit none
      integer, intent(in) :: ievt
      double precision, intent(out) :: wt
      !> common blocks
      double precision :: pang2(4,2,1),wt2(1)
      common /pmomang2omt/pang2,wt2
!$omp threadprivate(/pmomang2omt/)
      double precision :: pevt2(4,2)
      common/eventmom2omt/pevt2
!$omp threadprivate(/eventmom2omt/)

      pevt2(1:4,1:2) = pang2(1:4,1:2,ievt)

      wt = wt2(ievt)

      end subroutine fillcommon2omt



      subroutine phase3omt(x, wt, ifail)
      use Constants_mod, only : pi, pi2, twopi
      use Process_mod, only : decay_type_proc
      use KinData_mod, only : dot
      implicit none
      double precision, intent(in) :: x(4)
      double precision, intent(out) :: wt
      integer, intent(out) :: ifail
      double precision :: wtps
      double precision :: costh, sinth, prot(2:3)
      double precision :: qang(4,3:3,0:1), Qaux(4), Qrst(4)
      double precision :: qnrm(4,3), zsqr, zinv, dot_qQaux
      integer :: i, j, k
      integer, parameter :: qrot(3:3,2) =
     &  reshape([
     &    0,  ! k=1  ->  3nrm
     &    1   ! k=2  ->  3rot
     &  ],[1,2])
      !> common blocks
      double precision :: rnorm_sav,Qsqr,Qinv,q1(4),q2(4)
      common/lips_omt/rnorm_sav,Qsqr,Qinv,q1,q2
!$omp threadprivate(/lips_omt/)
      integer :: iang
      common /techpar/iang
      integer :: Nphase
      common /Nphase/Nphase
!$omp threadprivate(/Nphase/)
      double precision :: pang3(4,3,2),wt3(2)
      common /pmomang3omt/pang3,wt3
!$omp threadprivate(/pmomang3omt/)

      ifail = 1
      wt = 0d0

      wtps = 1d0
      if (iang == 0) then
        Nphase = 3
      else
        Nphase = 3 * 2
        wtps = wtps/2d0
      endif

      call init_omt()

      qang(:,:,:) = 0d0

      call sample_q(decay_type_proc, x(1:3), wtps, qang(1:4,3,0), qang(1:4,3,1))
      call pick(1, costh, -1d0, 1d0, x(4), wtps)
      if (wtps == 0d0) return
      wtps = wtps * twopi  ! trivial phi <-> beam azimuth symmetry
      sinth = sqrt((1d0+costh)*(1d0-costh))

      pang3 = 0d0; wt3 = 0d0

      Qrst(1:4) = [0d0, 0d0, 0d0, 1d0]

      do k = 1,2

        Qaux(1:4) = Qrst(1:4)
        do j = 3,3
          Qaux(1:4) = Qaux(1:4) + qang(1:4,j,qrot(j,k))
        end do

        ! zsqr = dot(Qaux,Qaux)
        zsqr = (Qaux(4)+Qaux(3))*(Qaux(4)-Qaux(3)) - (Qaux(1)**2+Qaux(2)**2)
        zinv = sqrt(zsqr)
        Qaux(1:4)   = Qaux(1:4) / zinv
        qnrm(1:4,1) = q1(1:4)   / zinv
        qnrm(1:4,2) = q2(1:4)   / zinv
        do j = 3,3
          qnrm(1:4,j) = qang(1:4,j,qrot(j,k)) / zinv
        end do

        do j = 1,3
          dot_qQaux = dot(qnrm(:,j),Qaux)
          pang3(:,j,k) = qnrm(:,j)
     &      - (dot_qQaux + qnrm(4,j)) / (1d0 + Qaux(4)) * (Qaux(:) + Qrst(:))
     &      + 2d0 * dot_qQaux * Qrst(:)
        end do

        do j = 1,3
          prot(2) = + pang3(2,j,k)*costh + pang3(3,j,k)*sinth
          prot(3) = - pang3(2,j,k)*sinth + pang3(3,j,k)*costh
          pang3(2:3,j,k) = prot(2:3)
          pang3(1:4,j,k) = pang3(1:4,j,k) * Qinv
        end do

        wt3(k) = 1d0 / zsqr**3

      end do  ! k

      wtps = wtps * Qsqr / (2d0**3 * twopi**5)

      ifail = 0
      wt = wtps

      end subroutine phase3omt


      subroutine fillcommon3omt(ievt, wt)
      implicit none
      integer, intent(in) :: ievt
      double precision, intent(out) :: wt
      integer :: i, j, k
      integer, parameter :: iwedge(3,3) =
     &  reshape([
     &    1,2, 3,
     &    1,3, 2,
     &    2,3, 1
     &  ],[3,3])
      !> common blocks
      double precision :: pang3(4,3,2),wt3(2)
      common /pmomang3omt/pang3,wt3
!$omp threadprivate(/pmomang3omt/)
      double precision :: pevt3(4,3)
      common/eventmom3omt/pevt3
!$omp threadprivate(/eventmom3omt/)

      i = mod((ievt-1),3)+1  ! inner loop: wedges
      k = int((ievt-1)/3)+1  ! outer loop: rotations

      do j=1,3
        pevt3(1:4,iwedge(j,i)) = pang3(1:4,j,k)
      end do

      wt = wt3(k)

      end subroutine fillcommon3omt


      subroutine phase4omt(x, wt, ifail)
      use Constants_mod, only : pi, pi2, twopi
      use Run_mod, only : tcut_run
      use Process_mod, only : decay_type_proc
      use KinData_mod, only : dot
      implicit none
      double precision, intent(in) :: x(7)
      double precision, intent(out) :: wt
      integer, intent(out) :: ifail
      double precision :: wtps
      double precision :: costh, sinth, prot(2:3)
      double precision :: qang(4,3:4,0:1), Qaux(4), Qrst(4)
      double precision :: qnrm(4,4), zsqr, zinv, dot_qQaux, dot_qiqj
      integer :: i, j, k, m, n
      integer, parameter :: qrot(3:4,4) =
     &  reshape([
     &    0,0,  ! k=1  ->  [3nrm,4nrm]
     &    0,1,  ! k=2  ->  [3nrm,4rot]
     &    1,0,  ! k=3  ->  [3rot,4nrm]
     &    1,1   ! k=4  ->  [3rot,4rot]
     &  ],[2,4])
      !> common blocks
      double precision :: rnorm_sav,Qsqr,Qinv,q1(4),q2(4)
      common/lips_omt/rnorm_sav,Qsqr,Qinv,q1,q2
!$omp threadprivate(/lips_omt/)
      integer :: iang
      common /techpar/iang
      integer :: Nphase
      common /Nphase/Nphase
!$omp threadprivate(/Nphase/)
      double precision :: pang4(4,4,4),wt4(4)
      common /pmomang4omt/pang4,wt4
!$omp threadprivate(/pmomang4omt/)

      ifail = 1
      wt = 0d0

      wtps = 1d0
      if (iang == 0) then
        Nphase = 6
      else
        Nphase = 6 * 4
        wtps = wtps/4d0
      endif

      call init_omt()

      qang(:,:,:) = 0d0

      call sample_q(decay_type_proc, x(1:3), wtps, qang(1:4,3,0), qang(1:4,3,1))
      call sample_q(decay_type_proc, x(4:6), wtps, qang(1:4,4,0), qang(1:4,4,1))
      call pick(1, costh, -1d0, 1d0, x(7), wtps)
      if (wtps == 0d0) return
      wtps = wtps * twopi  ! trivial phi <-> beam azimuth symmetry
      sinth = sqrt((1d0+costh)*(1d0-costh))

      pang4 = 0d0; wt4 = 0d0

      Qrst(1:4) = [0d0, 0d0, 0d0, 1d0]

      rots: do k = 1,4

        do m = 3,4
          do n = m+1,4
            dot_qiqj = dot(qang(1:4,m,qrot(m,k)), qang(1:4,n,qrot(n,k)))
            if ( (2d0*dot_qiqj >= 1d0) .or. (2d0*dot_qiqj < tcut_run) ) then
              wt4(k) = 0d0
              cycle rots
            end if
          end do
        end do

        Qaux(1:4) = Qrst(1:4)
        do j = 3,4
          Qaux(1:4) = Qaux(1:4) + qang(1:4,j,qrot(j,k))
        end do

        ! zsqr = dot(Qaux,Qaux)
        zsqr = (Qaux(4)+Qaux(3))*(Qaux(4)-Qaux(3)) - (Qaux(1)**2+Qaux(2)**2)
        zinv = sqrt(zsqr)
        Qaux(1:4)   = Qaux(1:4) / zinv
        qnrm(1:4,1) = q1(1:4)   / zinv
        qnrm(1:4,2) = q2(1:4)   / zinv
        do j = 3,4
          qnrm(1:4,j) = qang(1:4,j,qrot(j,k)) / zinv
        end do

        do j = 1,4
          dot_qQaux = dot(qnrm(:,j),Qaux)
          pang4(:,j,k) = qnrm(:,j)
     &      - (dot_qQaux + qnrm(4,j)) / (1d0 + Qaux(4)) * (Qaux(:) + Qrst(:))
     &      + 2d0 * dot_qQaux * Qrst(:)
        end do

        do j = 1,4
          prot(2) = + pang4(2,j,k)*costh + pang4(3,j,k)*sinth
          prot(3) = - pang4(2,j,k)*sinth + pang4(3,j,k)*costh
          pang4(2:3,j,k) = prot(2:3)
          pang4(1:4,j,k) = pang4(1:4,j,k) * Qinv
        end do

        wt4(k) = 1d0 / zsqr**4

      end do rots  ! k

      wtps = wtps * Qsqr**2 / (2d0**3 * twopi**8)

      ifail = 0
      wt = wtps

      end subroutine phase4omt


      subroutine fillcommon4omt(ievt, wt)
      implicit none
      integer, intent(in) :: ievt
      double precision, intent(out) :: wt
      integer :: i, j, k
      integer, parameter :: iwedge(4,6) =
     &  reshape([
     &    1,2, 3,4,
     &    1,3, 2,4,
     &    1,4, 2,3,
     &    2,3, 1,4,
     &    2,4, 1,3,
     &    3,4, 1,2
     &  ],[4,6])
      !> common blocks
      double precision :: pang4(4,4,4),wt4(4)
      common /pmomang4omt/pang4,wt4
!$omp threadprivate(/pmomang4omt/)
      double precision :: pevt4(4,4)
      common/eventmom4omt/pevt4
!$omp threadprivate(/eventmom4omt/)

      i = mod((ievt-1),6)+1  ! inner loop: wedges
      k = int((ievt-1)/6)+1  ! outer loop: rotations

      do j=1,4
        pevt4(1:4,iwedge(j,i)) = pang4(1:4,j,k)
      end do

      wt = wt4(k)

      end subroutine fillcommon4omt


      subroutine phase5omt(x, wt, ifail)
      use Constants_mod, only : pi, pi2, twopi
      use Run_mod, only : tcut_run
      use Process_mod, only : decay_type_proc
      use KinData_mod, only : dot
      implicit none
      double precision, intent(in) :: x(10)
      double precision, intent(out) :: wt
      integer, intent(out) :: ifail
      double precision :: wtps
      double precision :: costh, sinth, prot(2:3)
      double precision :: qang(4,3:5,0:1), Qaux(4), Qrst(4)
      double precision :: qnrm(4,5), zsqr, zinv, dot_qQaux, dot_qiqj
      integer :: i, j, k, m, n
      integer, parameter :: qrot(3:5,8) =
     &  reshape([
     &    0,0,0,  ! k=1  ->  [3N,4N,5N]
     &    0,0,1,  ! k=2  ->  [3N,4N,5R]
     &    0,1,0,  ! k=3  ->  [3N,4R,5N]
     &    1,0,0,  ! k=4  ->  [3R,4N,5N]
     &    0,1,1,  ! k=5  ->  [3N,4R,5R]
     &    1,0,1,  ! k=6  ->  [3R,4N,5R]
     &    1,1,0,  ! k=7  ->  [3R,4R,5N]
     &    1,1,1   ! k=8  ->  [3R,4R,5R]
     &  ],[3,8])
      !> common blocks
      double precision :: rnorm_sav,Qsqr,Qinv,q1(4),q2(4)
      common/lips_omt/rnorm_sav,Qsqr,Qinv,q1,q2
!$omp threadprivate(/lips_omt/)
      integer :: iang
      common /techpar/iang
      integer :: Nphase
      common /Nphase/Nphase
!$omp threadprivate(/Nphase/)
      double precision :: pang5(4,5,10),wt5(10)
      common /pmomang5omt/pang5,wt5
!$omp threadprivate(/pmomang5omt/)

      ifail = 1
      wt = 0d0

      wtps = 1d0
      if (iang == 0) then
        Nphase = 10
      else
        Nphase = 10 * 8
        wtps = wtps/8d0
      endif

      call init_omt()

      qang(:,:,:) = 0d0

      call sample_q(decay_type_proc, x(1:3), wtps, qang(1:4,3,0), qang(1:4,3,1))
      call sample_q(decay_type_proc, x(4:6), wtps, qang(1:4,4,0), qang(1:4,4,1))
      call sample_q(decay_type_proc, x(7:9), wtps, qang(1:4,5,0), qang(1:4,5,1))
      call pick(1, costh, -1d0, 1d0, x(10), wtps)
      if (wtps == 0d0) return
      wtps = wtps * twopi  ! trivial phi <-> beam azimuth symmetry
      sinth = sqrt((1d0+costh)*(1d0-costh))

      pang5 = 0d0; wt5 = 0d0

      Qrst(1:4) = [0d0, 0d0, 0d0, 1d0]

      rots: do k = 1,8

        do m = 3,5
          do n = m+1,5
            dot_qiqj = dot(qang(1:4,m,qrot(m,k)), qang(1:4,n,qrot(n,k)))
            if ( (2d0*dot_qiqj >= 1d0) .or. (2d0*dot_qiqj < tcut_run) ) then
              wt5(k) = 0d0
              cycle rots
            end if
          end do
        end do

        Qaux(1:4) = Qrst(1:4)
        do j = 3,5
          Qaux(1:4) = Qaux(1:4) + qang(1:4,j,qrot(j,k))
        end do

        ! zsqr = dot(Qaux,Qaux)
        zsqr = (Qaux(4)+Qaux(3))*(Qaux(4)-Qaux(3)) - (Qaux(1)**2+Qaux(2)**2)
        zinv = sqrt(zsqr)
        Qaux(1:4)   = Qaux(1:4) / zinv
        qnrm(1:4,1) = q1(1:4)   / zinv
        qnrm(1:4,2) = q2(1:4)   / zinv
        do j = 3,5
          qnrm(1:4,j) = qang(1:4,j,qrot(j,k)) / zinv
        end do

        do j = 1,5
          dot_qQaux = dot(qnrm(:,j),Qaux)
          pang5(:,j,k) = qnrm(:,j)
     &      - (dot_qQaux + qnrm(4,j)) / (1d0 + Qaux(4)) * (Qaux(:) + Qrst(:))
     &      + 2d0 * dot_qQaux * Qrst(:)
        end do

        do j = 1,5
          prot(2) = + pang5(2,j,k)*costh + pang5(3,j,k)*sinth
          prot(3) = - pang5(2,j,k)*sinth + pang5(3,j,k)*costh
          pang5(2:3,j,k) = prot(2:3)
          pang5(1:4,j,k) = pang5(1:4,j,k) * Qinv
        end do

        wt5(k) = 1d0 / zsqr**5

      end do rots  ! k

      wtps = wtps * Qsqr**3 / (2d0**3 * twopi**11)

      ifail = 0
      wt = wtps

      end subroutine phase5omt


      subroutine fillcommon5omt(ievt, wt)
      implicit none
      integer, intent(in) :: ievt
      double precision, intent(out) :: wt
      integer :: i, j, k
      integer, parameter :: iwedge(5,10) =
     &  reshape([
     &    1,2, 3,4,5,
     &    1,3, 2,4,5,
     &    1,4, 2,3,5,
     &    1,5, 2,3,4,
     &    2,3, 1,4,5,
     &    2,4, 1,3,5,
     &    2,5, 1,3,4,
     &    3,4, 1,2,5,
     &    3,5, 1,2,4,
     &    4,5, 1,2,3
     &  ],[5,10])
      !> common blocks
      double precision :: pang5(4,5,10),wt5(10)
      common /pmomang5omt/pang5,wt5
!$omp threadprivate(/pmomang5omt/)
      double precision :: pevt5(4,5)
      common/eventmom5omt/pevt5
!$omp threadprivate(/eventmom5omt/)

      i = mod((ievt-1),10)+1  ! inner loop: wedges
      k = int((ievt-1)/10)+1  ! outer loop: rotations

      do j=1,5
        pevt5(1:4,iwedge(j,i)) = pang5(1:4,j,k)
      end do

      wt = wt5(k)

      end subroutine fillcommon5omt




      subroutine genphaseomt(ipset,ievt,ix,x1,x2,xreg1,xreg2,ifail,rnorm)
      use KinData_mod
      use Channels_mod, only : currentSubContrib_chan
      implicit none
      integer, intent(in) :: ipset,ievt,ix
      double precision, intent(in) :: x1,x2,xreg1,xreg2
      integer, intent(out) :: ifail
      double precision, intent(inout) :: rnorm
      double precision :: wt
      integer :: iregion  ! region info
      !> common blocks
      double precision :: rnorm_sav,Qsqr,Qinv,q1(4),q2(4)
      common/lips_omt/rnorm_sav,Qsqr,Qinv,q1,q2
!$omp threadprivate(/lips_omt/)
      double precision :: pa(4),pb(4)
      common /pinomt/pa,pb
!$omp threadprivate(/pinomt/)
      !>  LIPS 2
      double precision :: pevt2(4,2)
      common/eventmom2omt/pevt2
!$omp threadprivate(/eventmom2omt/)
      !>  LIPS 3
      double precision :: pevt3(4,3)
      common/eventmom3omt/pevt3
!$omp threadprivate(/eventmom3omt/)
      !>  LIPS 4
      double precision :: pevt4(4,4)
      common/eventmom4omt/pevt4
!$omp threadprivate(/eventmom4omt/)
      !>  LIPS 5
      double precision :: pevt5(4,5)
      common/eventmom5omt/pevt5
!$omp threadprivate(/eventmom5omt/)

      ifail = 1

      call currentSubContrib_chan(1, region=iregion)

      if (rnorm_sav == 0d0) then
        rnorm_sav = rnorm
      end if

      !> load phase space point
      wt = 0d0
      select case (ipset)
      case (4)
        call fillcommon2omt(ievt,wt)
        if (wt == 0d0) return
        call p_to_common(2,pa,pb,pevt2)
      case (5)
        call fillcommon3omt(ievt,wt)
        if (wt == 0d0) return
        call p_to_common(3,pa,pb,pevt3)
      case (6)
        call fillcommon4omt(ievt,wt)
        if (wt == 0d0) return
        call p_to_common(4,pa,pb,pevt4)
      case (7)
        call fillcommon5omt(ievt,wt)
        if (wt == 0d0) return
        call p_to_common(5,pa,pb,pevt5)
      case default
        print*, "genphaseomt: invalid ipset =", ipset
        stop
      end select

      call fillS_kin(ipset)

      rnorm = rnorm_sav * wt
      ifail = 0

      end subroutine genphaseomt

