

      double precision function sig(x,wgt,sigMAT,sigSUB)
      use ieee_arithmetic, only : ieee_is_nan
      use Process_mod, only : nPDFs_proc, n_fs_spectators_proc
      use Channels_mod, only : currentContrib_chan, flushBuffer_multi_chan, ireject_event_chan
      use UnitPhase_mod, only: phase_unit
      use Run_mod, only: pole_check_run, print_max_weight_run
      use KinData_mod
      use Mapping_mod
      use Histograms_mod
      use FPA_mod
      use Spikes_mod, only: collectSpikes
      use MultiRun_mod
      use Checks_mod, only : point_chk, isActive_point_chk
      use PhotonFragmentation_mod, only : isActive_frag, nz_frag
      use HadronFragmentation_mod, only : isActive_hfrag
      !use Observables_mod
      implicit real*8(a-h,o-z)
      dimension x(mxdim)
      logical virtsflag
      double precision, external :: sigMAT, sigSUB
      common /vdata/xvegas(20)
!$omp threadprivate(/vdata/)
      common /parfrac/x1,x2
!$omp threadprivate(/parfrac/)
      common /xregions/xreg1,xreg2,virtsflag
!$omp threadprivate(/xregions/)
      common /method/imethod
      common /mxdim/mxdim
      common /Nphase/Nphase
!$omp threadprivate(/Nphase/)
      common /iphase/iphase
!$omp threadprivate(/iphase/)
      common /plotmode/iplot
      !> multi-run settings
      integer :: irun, igroup
      !> info on the current contribution
      integer :: n, ndim, nv, nfrag
      !> to extract random numbers from the tail
      integer :: itail_x
      !> fragmentation
      double precision :: zfrag, zregfrag

      call currentContrib_chan(npar=n, ndim=ndim, nsplit=nv, nfrag=nfrag)

      virtsflag = .false.

      sig    = 0d0
      sumMAT = 0d0
      sumSUB = 0d0

      xvegas(1:ndim) = x(1:ndim)
      itail_x = ndim
      if ( nv > 0 ) then
         !> rreg2
         if ( nPDFs_proc==2 ) then
            rreg2 = x(itail_x)
            itail_x = itail_x-1
         else
            rreg2 = 0d0
         end if
         !> rreg1
         if ( nPDFs_proc>=1 ) then
            rreg1 = x(itail_x)
            itail_x = itail_x-1
         else
            rreg1 = 0d0
         end if
      else
         rreg1 = 0d0
         rreg2 = 0d0
      end if
      !> r2
      if ( (nPDFs_proc==2) .or.
     &    ((nPDFs_proc==1).and.(n_fs_spectators_proc==0))
     &   ) then
         r2 = x(itail_x)
         itail_x = itail_x-1
      else
         r2 = 0d0
      end if
      !> r1
      if ( nPDFs_proc>=1 ) then
         r1 = x(itail_x)
         itail_x = itail_x-1
      else
         r1 = 0d0
      end if

      call fillparmom(n,1,r1,r2,rreg1,rreg2,rnorm,ifail)
      if (ifail.eq.1) goto 999

      !> determine extra convolution variable for fragmentation
      !> *must* come after fillparmom for jacobians!
      if ( isActive_hfrag() ) then
         zfrag = x(itail_x)
         itail_x = itail_x-1
         select case (nfrag)
         case (1)
            call setFragFrac_kin(zfrag)
         case (2)
            zregfrag = zfrag + (1d0-zfrag) * x(itail_x)
            !> this would interfere with pure-QCD subtractions.
            !> Jacobian will be applied in `bino` instead
            ! if (.not. phase_unit) rnorm = rnorm * (1d0-zfrag)
            itail_x = itail_x-1
            call setFragFrac_kin(zfrag, zregfrag)
         case default
            stop "sig: only support ONE fragmentation convolution!"
         end select
      end if
      if ( isActive_frag() ) then
         if (nz_frag >= 1) then
            zfrag = x(itail_x)
            itail_x = itail_x-1
            if (nv > 0) then
               zregfrag = zfrag + (1d0-zfrag) * x(itail_x)
               itail_x = itail_x-1
               call setFragFrac_kin(zfrag, zregfrag)
            else
               call setFragFrac_kin(zfrag)
            end if
         end if
         if (nz_frag >= 2) then
            zfrag = x(itail_x)
            itail_x = itail_x-1
            if (nv > 0) then
               zregfrag = zfrag + (1d0-zfrag) * x(itail_x)
               itail_x = itail_x-1
               call setFrag2Frac_kin(zfrag, zregfrag)
            else
               call setFrag2Frac_kin(zfrag)
            end if
         end if
      end if

      ! call print_xvegas()

      !> start the loop over all registered runs
      ireject_event_chan = 0
      call resetCurrentRun_mult()
      do while (loadNextRun_mult())
         call currentRun_mult(irun=irun, run_group=igroup)
         ! print*, "irun = ", irun, " / ", ntot_mult()
         do iphase = 1, Nphase
            ! if (iphase==1) call print_xvegas
            ! print*, "iphase = ", iphase, " / ", Nphase
            !!! genphase needs to be called *after* load_mult (P2B) !!!
            call genphase(n,iphase,1,x1,x2,xreg1,xreg2,ifail,rnorm)
            if (ifail /= 0) cycle
            ! print*, "genphase successful...", n
            ! call print_momenta(n)

            sMAT = 0d0;  sSUB = 0d0
            call flushCache_map()
            sMAT = sigMAT(rnorm,x1,x2,wgt)
            sSUB = sigSUB(rnorm,x1,x2,wgt)

            sumMAT = sumMAT + sMAT
            sumSUB = sumSUB + sSUB

C              call print_momenta(n)
C              !call printAll_obs(n)
C              call print_xvegas()
C              !print*, "iphase", iphase,"/",Nphase
C              print*, "sMAT", sMAT
C              print*, "sSUB", sSUB
C              print*, "x1", x1
C              print*, "x2", x2
C              print*, "xreg1", xreg1, (xreg1>x1)
C              print*, "xreg2", xreg2, (xreg2>x2)
C              !if ((1d0-xreg1)<1d-5) read(*,*)
C              !if ((1d0-xreg2)<1d-5) read(*,*)
C              read(*,*)
            call point_chk(n,rnorm,x1,x2,wgt, sigMAT,sigSUB)

            if(pole_check_run)then
               if((abs(sumMAT)+abs(sumSUB)).gt.1D-20)then
                  call polecheck(n,rnorm,x1,x2,wgt,sigMAT,sigSUB)
               endif
            endif

         end do  ! iphase
      end do  ! irun

      sig = sumMAT - sumSUB

****  debug code  ****

      if (print_max_weight_run) then
         call printmaxwgt(n, sumMAT*wgt, sumSUB*wgt, rnorm)
      endif


      if(iplot.eq.2) call pointcheck(n,rnorm,x1,x2,wgt,sigMAT,sigSUB)
      if(iplot.eq.3) then
         if (sumR.ne.0d0) then
            call collectSpikes(n, sumMAT, sumSUB)
         endif
      endif

      if ( ieee_is_nan(sig) ) then
         print*, ""
         print*, "###################"
         print*, "# Encountered NaN #"
         print*, "###################"
         call print_momenta(n)
         call print_xvegas
         print*, "throwing away this phasespace point..."
         ireject_event_chan = -2  ! NaN
         sig = 0d0 ! reset for safety
      endif

*********************
*     exit
 999  continue

      if (.not.phase_unit .and. .not.isActive_point_chk()) then
         !> for checking weights against VEGAS
         call flushBuffer_hist(vegas_wsig=wgt*sig)
         call flushBuffer_multi_chan(wgt, sig)
      else
         call flushBuffer_hist()
      endif

      return
      end

************************************************
* A small subroutine for testing the pole pieces
* of a matrix element and subtraction term.
*
* NB: This is only intended as a quick test to make
* sure we have dressed the subtraction term with the
* PDFs correctly. We should do more thorough testing
* in the check code.
*
      subroutine polecheck(npar,rnorm,x1,x2,wgt,sigMAT,sigSUB)
      use Process_mod
      use Channels_mod
      implicit none
      integer, intent(in) :: npar
      double precision, intent(in) :: rnorm, x1, x2, wgt
      !> external functions
      double precision, external :: sigMAT,sigSUB
      !> common block for 1-loop MEs
      integer :: ieorder
      common /order/ieorder
      !> common block for 2-loop MEs
      integer :: ipole
      common /order2l/ipole
      !> local variables
      double precision :: mat, sub
      integer :: ichan
      character(:), allocatable :: abbr_str
      character(:), allocatable :: chan_str, me_str, sub_str
      integer :: id_ch, nsub_ch, nV_ch, nR_ch, nO_ch, npar_ch


!$omp critical(polechk)


      do ichan=1,ntot_chan
         if (.not.qactive_chan(ichan)) cycle
         !> channel information
         call get_chan(ichan, chan_str, me_str, sub_str)
         call info_chan(ichan, id_ch, nsub_ch, abbr_str, nV_ch, nR_ch, nO_ch, npar_ch)
         if (npar /= npar_ch) cycle

         !> switch on one single channel
         qselect_chan(:)     = .false.
         qselect_chan(ichan) = .true.

         !> 1-loop?
         if (nV_ch == 1) then
            write(*,*)
            write(*,'(10("*"),1x,"channel ",I4,":",T30,A)') ichan, chan_str
            do ieorder=-2,-1
               write(*,*) "Pole check ieorder: ", ieorder
               mat = sigMAT(rnorm, x1, x2, wgt)
               sub = sigSUB(rnorm, x1, x2, wgt)
               if((mat.ne.0d0).and.(sub.ne.0d0))then
                  write(*,*) "matrix:      ", mat
                  write(*,*) "subtraction: ", sub
                  write(*,*) "ratio:       ", mat/sub
               elseif((mat.ne.0d0).or.(sub.ne.0d0))then
                  write(*,*) "matrix:      ", mat
                  write(*,*) "subtraction: ", sub
               else
                  write(*,*) "no pole"
               endif
            end do  ! ieorder=-2,-1
         end if

         ieorder = 0  ! reset

         !> 2-loop?
         if (nV_ch == 2) then
            write(*,*)
            write(*,'(10("*"),1x,"channel ",I4,":",T30,A)') ichan, chan_str
            ipole = -4
c--   check the 1/e^4 pole
            write(*,*) "Pole check ipole: ", ipole
            mat = sigMAT(rnorm, x1, x2, wgt)
            sub = sigSUB(rnorm, x1, x2, wgt)
            if((mat.ne.0d0).and.(sub.ne.0d0))then
               write(*,*) "matrix:      ", mat
               write(*,*) "subtraction: ", sub
               write(*,*) "ratio:       ", mat/sub
            elseif((mat.ne.0d0).or.(sub.ne.0d0))then
               write(*,*) "matrix:      ", mat
               write(*,*) "subtraction: ", sub
            else
               write(*,*) "no pole"
               ipole = -3
c---  if no 1/e^4 pole, check the 1/e^3 pole
               write(*,*) "Pole check ipole: ", ipole
               mat = sigMAT(rnorm, x1, x2, wgt)
               sub = sigSUB(rnorm, x1, x2, wgt)
               if((mat.ne.0d0).and.(sub.ne.0d0))then
                  write(*,*) "matrix:      ", mat
                  write(*,*) "subtraction: ", sub
                  write(*,*) "ratio:       ", mat/sub
               elseif((mat.ne.0d0).or.(sub.ne.0d0))then
                  write(*,*) "matrix:      ", mat
                  write(*,*) "subtraction: ", sub
               else
                  write(*,*) "no pole"
                  ipole = -2
c---  if no 1/e^3 pole, check the 1/e^2 pole
                  write(*,*) "Pole check ipole: ", ipole
                  mat = sigMAT(rnorm, x1, x2, wgt)
                  sub = sigSUB(rnorm, x1, x2, wgt)
                  if((mat.ne.0d0).and.(sub.ne.0d0))then
                     write(*,*) "matrix:      ", mat
                     write(*,*) "subtraction: ", sub
                     write(*,*) "ratio:       ", mat/sub
                  elseif((mat.ne.0d0).or.(sub.ne.0d0))then
                     write(*,*) "matrix:      ", mat
                     write(*,*) "subtraction: ", sub
                  else
                     write(*,*) "no pole"
                     ipole = -1
c---  if no 1/e^2 pole, check the 1/e pole
                     write(*,*) "Pole check ipole: ", ipole
                     mat = sigMAT(rnorm, x1, x2, wgt)
                     sub = sigSUB(rnorm, x1, x2, wgt)
                     if((mat.ne.0d0).and.(sub.ne.0d0))then
                        write(*,*) "matrix:      ", mat
                        write(*,*) "subtraction: ", sub
                        write(*,*) "ratio:       ", mat/sub
                     elseif((mat.ne.0d0).or.(sub.ne.0d0))then
                        write(*,*) "matrix:      ", mat
                        write(*,*) "subtraction: ", sub
                      else
                        write(*,*) "no pole"
                     endif
                  endif
               endif
            endif
         end if

      end do  ! ichan=1,ntot_chan

      !> switch all channels back on
      qselect_chan(:) = .true.

      write(*,*)
      write(*,*) "Pole check complete"
      write(*,*) "Press any key to continue with next phasespace point..."
      read(*,*)
!$omp end critical(polechk)

      end subroutine

**********************************************************************
*   keeps track of the highest weight and/or the largest real
*   corrections in the Monte Carlo integration and prints out the
*   phasespace point whenever a new highest weight/ real ME is encountered.
*
      subroutine printmaxwgt(npar,wgtR,wgtS,wgtPS)
      implicit real*8(a-h,o-y)
      !---
      real*8 maxwgt, maxwgtR, maxwgtS
      save   maxwgt, maxwgtR, maxwgtS
      data   maxwgt, maxwgtR, maxwgtS /3 * 0d0/
      logical qmaxwgt, qmaxwgtR, qmaxwgtS
      data    qmaxwgt, qmaxwgtR, qmaxwgtS /3 * .false./
      !---
      real*8 minwgt, minwgtR, minwgtS
      save   minwgt, minwgtR, minwgtS
      data   minwgt, minwgtR, minwgtS /3 * 0d0/
      logical qminwgt, qminwgtR, qminwgtS
      data    qminwgt, qminwgtR, qminwgtS /3 * .false./
      !---
      common /parfrac/x1,x2
!$omp threadprivate(/parfrac/)
      logical virtsflag
      common /xregions/xreg1,xreg2,virtsflag
!$omp threadprivate(/xregions/)
      common /vdata/xvegas(20)
!$omp threadprivate(/vdata/)

      wgt = wgtR - wgtS

C --- comment out the line that you do not want
      qmaxwgt  = wgt .gt.maxwgt
      !qmaxwgtR = wgtR.gt.maxwgtR
      !qmaxwgtS = wgtS.gt.maxwgtS
      !---
      !qminwgt  = wgt .lt.minwgt
      !qminwgtR = wgtR.lt.minwgtR
      !qminwgtS = wgtS.lt.minwgtS

      if ( qmaxwgt  .or. qminwgt  .or.
     &     qmaxwgtR .or. qminwgtR .or.
     &     qmaxwgtS .or. qminwgtS ) then
!$omp critical(prtmaxwgt)
         if(qmaxwgt)  maxwgt  = wgt
         if(qmaxwgtR) maxwgtR = wgtR
         if(qmaxwgtS) maxwgtS = wgtS
         if(qminwgt)  minwgt  = wgt
         if(qminwgtR) minwgtR = wgtR
         if(qminwgtS) minwgtS = wgtS
         print*, ""
         if(qmaxwgt)  print*, "******* new max weight: ", wgt
         if(qmaxwgtR) print*, "******* new max real ME:", wgtR
         if(qmaxwgtS) print*, "******* new max subtraction term:", wgtS
         if(qminwgt)  print*, "******* new min weight: ", wgt
         if(qminwgtR) print*, "******* new min real ME:", wgtR
         if(qminwgtS) print*, "******* new min subtraction term:", wgtS
         print*, "* R        = ", wgtR
         print*, "* S        = ", wgtS
         print*, "* R/S  S/R = ", wgtR/wgtS, wgtS/wgtR
         print*, "* cancellation of ~",
     .           int(anint(-log10(abs((wgtR-wgtS)/(wgtS+wgtR))*2d0))),
     .           "digit(s)"
C          print*, "* PS weight = ",wgtPS
C          print*, "* phasespace: "
C          call print_momenta(npar)
         print*, "* x1,    x2    ", x1, x2
         print*, "* xreg1, xreg2 ", xreg1, xreg2
         call print_xvegas()
         !read(*,*)
!$omp end critical(prtmaxwgt)
      endif

      end


********************************************************
* This subroutine populates the momentum common blocks for
* a given ipset
* Note that for the case of double real integration the
* iregion parameter is used to denote the 'a' or 'b' group of
* wedges
*
      subroutine fillparmom(ipset,ix,r1,r2,rreg1,rreg2,rnorm,ifail)
      use Process_mod
      use ProcLib_mod
      use Run_mod, only : ilips_run
      use Channels_mod
      use PhaseNJ_mod,    only : phaseNJ
      use UnitPhase_mod
      use KinData_mod, only : setParFrac_kin
      use sigHinc_mod
      use sigHqt_mod
      use sigZqt_mod
      use sigGGqt_mod
      use sigWqt_mod
      use Selectors_mod, only : getExtremes_sel
      implicit real*8(a-h,o-y)
      implicit complex*16(z)
      parameter(fbGeV2=389379365600d0)
      parameter(pi=3.141592653589793238d0)
      common /parfrac/x1,x2
!$omp threadprivate(/parfrac/)
      logical virtsflag
      common /xregions/xreg1,xreg2,virtsflag
!$omp threadprivate(/xregions/)
      common /vdata/xvegas(20)
!$omp threadprivate(/vdata/)
      common /scut/smin
!$omp threadprivate(/scut/)
      common /tcuts/ymin,y0
      common /pmasses/rma2,rmb2,rm2(1:7),shat
!$omp threadprivate(/pmasses/)
      common /method/imethod
      common /techpar/iang
      common /nwa/rm12,rg12,inwa
      logical qnaive
      common /s12_min/s12_min
      !> info on the current contribution
      integer :: nv, ndim
      !> region info
      integer :: iregion

      call currentContrib_chan(npar=n, ndim=ndim, nsplit=nv)



      call currentSubContrib_chan(1, region=iregion)

C --- r1,r2 are random numbers [0,1] from which x1,x2 (PDF convolution) are derived
C --- x1,x2 are stored in the /parfrac/ common block

C --- rreg1,rreg2 are random numbers [0,1] from which xreg1,xreg2 are derived
C --- xreg1,xreg2 are stored in the /xregions/ common block

C --- need to be careful with the fillparmom calles in the integrated antennae
C --- which is why the mapping only works with imethod=2

      ifail = 0
      wtWW=1d0
      rnorm = 0d0

      ! x[1|2] & xreg[1|2] mapping
      qnaive = .false. ! switch to a naive mapping as has been done before
      wtjac = 1d0
c nigel to validate against Joao
c      qnaive = .true.
C --- imethod = 1
      if(nPDFs_proc == 0) then
         shat = s_proc
         !> skip the whole block for epem
         goto 100
      endif

      if(imethod.eq.1)then
         x1    = r1
         x2    = r2
         xreg1 = rreg1
         xreg2 = rreg2

C --- imethod = 2
      else

!      if(name_proc.eq."GJ") qnaive=.true.

      if(.not.qnaive)then
         shatmin = s12_min
         shatmax = s_proc

!----- 2->2 scattering: need the option of a Breit-Wigner mapping
         if ( ( (ipset.eq.4) .and. (name_proc(1:2).ne.'GJ')
     &                       .and. (name_proc.ne.'GG')
     &                       .and. (name_proc.ne.'GG_FRAG')
     &                       .and. (name_proc.ne.'G_EW')
     &                       .and. (name_proc.ne.'GG_EW')
     &                       .and. (name_proc(2:).ne.'JET')
     &                       .and. (name_proc(2:).ne.'JETFC') 
     &                       .and. (nPDFs_proc == 2) ) .or.
     &        ( (ipset.eq.5) .and. (name_proc(1:7).eq.'HTO2L1P') ) .or.
     &        ( (ipset.eq.6) .and. (name_proc(1:5).eq.'HTO4E') ) .or.
     &        ( (ipset.eq.6) .and. (name_proc(1:8).eq.'HTO2E2MU') ) .or.
     &        ( (ipset.eq.6) .and. (name_proc(1:7).eq.'HTO2L2N') ) ) then

!* off-shell bosons
            if (( inwa >= -2  .and.  inwa <= 1 ).or.(inwa .eq. -4).or.(inwa .eq. 10)) then
! dx[1|2] --> dshat, dlogx
               wtjac = wtjac / s_proc ! tau --> shat
               call pick_sinv_generic(shat,shatmin,shatmax,rm12,rg12,r1,wtjac,inwa)
               tau   = shat/shatmax
               wtjac = -wtjac * log(tau) ! dx/x --> dlog(x)
               x1    = tau**r2
               x2    = tau / x1
               yt    = 0.5d0*log(x1/x2)

         !* on-shell bosons
            elseif ( (inwa >= 2) .and. (inwa <= 4) ) then
! NWA (doesn't use r1)
               ratio = rm12/sqrts_proc
               ymax  = -log(ratio)
               yt    = -ymax + 2d0*r2*ymax
               x1    = ratio*exp(+yt)
               x2    = ratio*exp(-yt)
               wtjac = 2d0*ymax/s_proc
               select case (inwa)
               case (2)
                  wtjac = wtjac*pi*rm12*rg12
               case (3)
                  wtjac = wtjac * 16d0*pi*pi
               case (4)
                  wtjac = wtjac * 2d0*rm12
               case default
                  print*, "fillparmon: invalid inwa = ", inwa, " in on-shell branch"
                  stop
               end select
            else
               write(*,*)"fillparmon: inwa =",inwa,"not supported"
               stop
            endif

!----- 2->n scattering (n>2) or 2->2 with no resonance
         else
            !> in photoproduction we need another convolution variable
            if (nPDFs_proc.eq.1 .and. name_proc.ne.'DISG') then
               x1min = shatmin / s_proc
               x1max = shatmax / s_proc
               call pick(2,x1,x1min,x1max,r1,wtjac) ! logarithmic
               x2=0d0
            else
               taumin = shatmin / s_proc
               taumax = shatmax / s_proc
               ! if (taumin.eq.0d0) then
               ! if ( shatmin < 1d0 ) then
               if ( taumin < 1d-8 ) then
                 call pick(1,tau,taumin,taumax,r1,wtjac) ! linear
               else
                 call pick(2,tau,taumin,taumax,r1,wtjac) ! log
                 ! call pick(23,tau,taumin,taumax,r1,wtjac) ! log & more
               end if
               !> This is only to compare against SHERPA
               !> take it out again once comparision works
               if(name_proc.eq.'DISG') then
                  x1=tau
                  if(nv.eq.0)then ! only real integration
                     x2=1d0
                     xreg1 = 0d0
                     xreg2 = 0d0
                     call setParFrac_kin(x1, x2)
                  else          ! include virtual integration
                     xreg1 = x1 + rreg1*(1d0-x1)
                     xreg2 = rreg2
                     x2    = rreg2
                     wtjac = wtjac*(1d0-x1)
                     call setParFrac_kin(x1, x2, xreg1, xreg2)
                  endif
                  goto 500
               else
                  x1     = tau**r2
                  x2     = tau / x1
                  wtjac  = -wtjac*log(tau)
               endif

            endif

         endif
      else
         if(name_proc(2:).eq.'JET' .or. name_proc(2:).eq.'JETFC')then
            x1=r1
            x2=r2
         else
            write(*,*)"fillparmom: qnaive is set to
     .                             true in core/sig.f (stopping)"
            stop
         endif
      endif

      !* remaining xreg[1|2] mapping
      if(nv.eq.0)then ! only real integration
        xreg1 = 0d0
        xreg2 = 0d0
        call setParFrac_kin(x1, x2)
      else ! include virtual integration
        call pick(1, xreg1, x1, 1d0, rreg1, wtjac)
        call pick(1, xreg2, x2, 1d0, rreg2, wtjac)
        call setParFrac_kin(x1, x2, xreg1, xreg2)
      endif
 500  continue
      !> inclusive (y-diff) Higgs production
      if ( name_proc.eq.'HINC' .or.
     &     name_proc.eq.'HTO2PINC'  .or.
     &     name_proc.eq.'HTO2TAUINC'  .or.
     &     name_proc.eq.'HTO2L1PINC'  .or.
     &     name_proc.eq.'HTO2E2MUINC'  .or.
     &     name_proc.eq.'HTO4EINC'  .or.
     &     name_proc.eq.'HTO2L2NINC' ) then
         if(nv.eq.0) stop "fillparmom: Hinc needs virtual integrations"
         !> valiables fixing the Born kinematics
         call set_kinematics_Hinc(rm12, yt)
         !> RapidiX needs xireg in [0,1] with a manual cut xireg >= xi
         !>  => override (don't care about jac: computed manually)
         xreg1 = rreg1
         xreg2 = rreg2
         call setParFrac_kin(x1, x2, xreg1, xreg2)
      endif
      !> qT Higgs from SCET
      if ( name_proc.eq.'HQT' .or.
     &     name_proc.eq.'HTO2PQT'  .or.
     &     name_proc.eq.'HTO2TAUQT'  .or.
     &     name_proc.eq.'HTO2L1PQT'  .or.
     &     name_proc.eq.'HTO2E2MUQT'  .or.
     &     name_proc.eq.'HTO4EQT'  .or.
     &     name_proc.eq.'HTO2L2NQT' ) then
         if (nv==0) stop "fillparmom: Hqt needs virtual integrations"
         !> store the Born variables (for checks?)
         call set_kinematics_Hqt(rm12, yt)
         !> sample qT & phi <-> 2 rand #'s
         !> the last four entries in the random number array are reserved for the
         !> x1, x2, x1_reg, x2_reg sampling.
         !> phase-space sampling pops from the head
         !>  ==>  take the 5th & 6th ran # from the back
         call sample_recoil_Hqt(xvegas(ndim-4), xvegas(ndim-5))
         !stop "QT NOT DONE"
      endif
      !> qT Z-boson from SCET
      if ( name_proc.eq.'ZQT' ) then
         if (nv==0) stop "fillparmom: Zqt needs virtual integrations"
         !> store the Born variables (for checks?)
         call set_kinematics_Zqt(sqrt(shat), yt)
         !> sample qT & phi <-> 2 rand #'s
         !> the last four entries in the random number array are reserved for the
         !> x1, x2, x1_reg, x2_reg sampling.
         !> phase-space sampling pops from the head
         !>  ==>  take the 5th & 6th ran # from the back
         call sample_recoil_Zqt(xvegas(ndim-4), xvegas(ndim-5))
         !stop "QT NOT DONE"
      endif
      !> qT di-photon from SCET
      if ( name_proc.eq.'GGQT' ) then
         if (nv==0) stop "fillparmom: GGqt needs virtual integrations"
         !> store the Born variables (for checks?)
         call set_kinematics_GGqt(sqrt(shat), yt)
         !> sample qT & phi <-> 2 rand #'s
         !> the last four entries in the random number array are reserved for the
         !> x1, x2, x1_reg, x2_reg sampling.
         !> phase-space sampling pops from the head
         !>  ==>  take the 5th & 6th ran # from the back
         call sample_recoil_GGqt(xvegas(ndim-4), xvegas(ndim-5))
         !stop "QT NOT DONE"
      endif
      !> qT W-boson from SCET
      if ( name_proc.eq.'WPQT' .or. name_proc.eq.'WMQT' ) then
         if (nv==0) stop "fillparmom: Wqt needs virtual integrations"
         !> store the Born variables (for checks?)
         call set_kinematics_Wqt(sqrt(shat), yt)
         !> sample qT & phi <-> 2 rand #'s
         !> the last four entries in the random number array are reserved for the
         !> x1, x2, x1_reg, x2_reg sampling.
         !> phase-space sampling pops from the head
         !>  ==>  take the 5th & 6th ran # from the back
         call sample_recoil_Wqt(xvegas(ndim-4), xvegas(ndim-5))
         !stop "QT NOT DONE"
      endif


      endif
C --- imethod endif


      tau = x1*x2
      !> single out photoproduction -> 0 spectators
      if (nPDFs_proc.eq.1.and.n_fs_spectators_proc.ne.0) then
         tau = x1
      else
         tau = x1*x2
      endif

      if(imethod.eq.2)then
         shat = tau*s_proc
      elseif(imethod.eq.1)then
         if(ix.eq.1)then
            shat = tau*s_proc
         elseif(ix.eq.2)then
            shat = xreg2*tau*s_proc
         elseif(ix.eq.3)then
            shat = xreg1*tau*s_proc
         elseif(ix.eq.4)then
            shat = xreg1*xreg2*tau*s_proc
         else
            write(*,*) "Invalid region: ", ix
            stop
         endif
      else
         write(*,*) "invalid method: ", imethod
         stop
      endif

 100  continue

      if ( name_proc.ne."SFDIS"  .and.
     &     name_proc.ne."SFDISWP"  .and.
     &     name_proc.ne."SFDISWM"  .and.
     &     name_proc.ne."RESDIS" .and.
     &     name_proc.ne."HINC" .and.
     &     name_proc.ne."HTO2PINC"  .and.
     &     name_proc.ne."HTO2TAUINC"  .and.
     &     name_proc.ne."HTO2L1PINC"  .and.
     &     name_proc.ne."HTO2E2MUINC"  .and.
     &     name_proc.ne."HTO4EINC"  .and.
     &     name_proc.ne."HTO2L2NINC" ) then
         flux = fbGeV2/2d0/shat
      else
         !> could potentially take flux factor out in SF
         flux = 1d0
      endif

      wtps=0d0

      call phaseN_dl(xvegas,wtps,ifail)
      call fillcommonNp_dl()

      smin = shat*y0
      rnorm = flux*wtps*wtjac*wtWW

      if(phase_unit)then
         rnorm = wtps ! overwrite weight with just the phase space weight when calculating the unit phase space and remove x1 and x2 values
      endif


      if(ifail.eq.1) return

      return
      end


************************************************************************
*     prints the raw vegas variables for debugging
*
      subroutine print_xvegas()
      use Channels_mod
      implicit none
      double precision xvegas
      common /vdata/xvegas(20)
!$omp threadprivate(/vdata/)
      !> info on the current contribution
      integer :: i, n, ndim, nv

      call currentContrib_chan(npar=n, ndim=ndim, nsplit=nv)

!$omp critical(printxvegas)
      write(*,*) "* raw vegas variables: "

      do i=1,ndim
         write(6,50) i,xvegas(i)
      enddo

!$omp end critical(printxvegas)

 50   format(11x,'x('i2') = ',f32.30,'d0')
      return
      end



**********************************************
* An additional technical cut on the polynomial
* Q3 = x1*(x1+x2)**2 -4*x2, which is a
* known region of numerical instability as Q3 -> 0
* this will shift x1 and x2 by order of the technical
* cut in the event that it is too close to Q3
*
* y0 is set to 1d-9
*
      subroutine Q3_check
      use Process_mod, only : nPDFs_proc
      use KinData_mod
      implicit none
      logical virtsflag
      double precision x1,x2,Q3_1,Q3_2,y0
      double precision r1,rn
      integer ix
      common /xregions/x1,x2,virtsflag
!$omp threadprivate(/xregions/)

      if(nPDFs_proc.ne.2) return

      Q3_1 = x1*(x1+x2)**2 - 4*x2
      Q3_2 = x2*(x1+x2)**2 - 4*x1

      y0=1d-9 ! tested to be ok for Z+1jet VV integration

      if((abs(Q3_1).lt.y0).or.(abs(Q3_2).lt.y0))then
         write(*,*) "Q3 check failed"
         write(*,*) "Input x1,x2: ", x1,x2
         write(*,*) "Q3_1,Q3_2: ", Q3_1,Q3_2
         write(*,*) "Technical cut: ", y0
         r1 = rn()
         if(r1.lt.0.5d0)then
            x1=x1-y0
            x2=x2+y0
         else
            x1=x1+y0
            x2=x2-y0
         endif
         if(x1.gt.1d0) x1 = x1-y0
         if(x1.lt.0d0) x1 = x1+y0
         if(x2.gt.1d0) x2 = x2-y0
         if(x2.lt.0d0) x2 = x2+y0
         write(*,*) "New x1,x2: ", x1,x2
      endif


      return
      end
