***************************************************
* a wrapper subroutine for the new vegas subroutine
*
*

      subroutine vegaswrapper(sigVal,sigErr)
      use Run_mod
      use Channels_mod
      use Histograms_mod
      use IOHelper_mod, only : file_io
      use Spikes_mod
      use Vegas_mod, only: vegasnr
      use LIPS_mod
      use ProcLib_mod
      implicit real*8(a-h,o-y)
      implicit complex*16(z)
      external sig
      logical readin,writeout
      character*128 gridfile
      common /mxdim/ mxdim
      common /gridfilename/gridfile
      common /plotmode/iplot
      common /vegasgrid/ readin,writeout
      logical wilsonflag
      common /wilsonc/wilsonflag
      common /npars/ npars,nspec,n
!$omp threadprivate(/wilsonc/)
      !> info on the current sweep
      integer :: irun, ncall, nit
      !> info on the current contribution
      integer :: id_contr, nsub, nVirt, nReal
      integer :: ndim,nv,norder,npar
      !> filenames
      character(128) :: new_gridfile
      character(128) :: histogram_prefix, histogram_suffix
      !
      logical :: qexists_grid
      !
      integer :: iregion

      !> get information on the current contribution
      call currentContrib_chan(id=id_contr, nsub=nsub,
     &                         nV=nVirt, nR=nReal, nO=norder,
     &                         npar=npar, ndim=ndim, nsplit=nv)

      mxdim = ndim  ! really needed?
      n = npar

      call clear_hist()
      call setTechcuts(npar)

      call currentSubContrib_chan(1, region=iregion)
      call init_lips(npar, iregion)

      call file_io( gridfile=new_gridfile,
     &              histogram_prefix=histogram_prefix,
     &              histogram_suffix=histogram_suffix )

      call setPrefix_hist( trim(histogram_prefix) )
      call setSuffix_hist( trim(histogram_suffix) )

      gridfile = new_gridfile
      inquire(file=new_gridfile, exist=qexists_grid)

      !> get information on the current sweep
      call currentSweep_run(type=irun, ncall=ncall, nit=nit)
      !> irun = 1: warmup
      !>      = 2: production
      select case (irun)
      case(1)  ! warmup
        print*, "warmup..."
        if (isActive_multi_chan()) then
          print*, "  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"
          print*, "  !!!  ATTENTION - warmup numbers using multi-channel  !!!"
          print*, "  !!!              *not* suitable for validation       !!!"
          print*, "  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"
        end if
        init = 0
        call setActive_hist(.false.)
        ! call setActive_hist(.true.)  ! only for debugging
        if (qexists_grid .and. .not.reset_vegas_grid_run) then
          readin   = .true.
        else
          readin   = .false.
        end if
        writeout = .true.
      case(2)  ! production
        print*, "production..."
        init = 1  ! turn off adaption: freeze the grid
        call setActive_hist(.true.)
        readin   = .true.
        writeout = .false.
      case default
        print*, "unknown irun:", irun
        stop
      end select



      call vegasnr(ndim,sig,init,ncall,nit,sigVal,sigErr,chi2a,sigMAT_dl,sigSUB_dl)

      if ( (irun==1) .and. isActive_multi_chan() ) then
        print*, "  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"
        print*, "  !!!  ATTENTION - warmup numbers using multi-channel  !!!"
        print*, "  !!!              *not* suitable for validation       !!!"
        print*, "  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"
      end if

      !> after first warmup integration keep grid
      !> @todo: when we integrate multiple channels this might be bad?
      reset_vegas_grid_run = .false.

      !> clean up for spike testing
      if (irun==2 .and. iplot==3) call flushSpikes()

      call destroy_lips()

      end




c     do parallel initialization of modules
c     > has to be called once at the beginning of a parallel block
      subroutine init_parallel()
      use Process_mod
      use KinData_mod
      use Jets_mod
      use Scales_mod
      use Observables_mod
      use Selectors_mod
      use Histograms_mod
      use PhotonIsolation_mod
      use PhotonFragmentation_mod
      use HadronFragmentation_mod, only : init_hfrag
      use Proj2Born_mod, only : setActive_p2b
      use MultiRun_mod, only : min_npar_p2b_mult
      use Channels_mod
      use PDFInterp_mod, only : initGrids_pdfi
      use PDFCache_mod, only : initCache_pdfc
      use PhaseNJ_mod, only : initVFH
      use Reweight_mod, only : setLocal_rewgt
      use Mapping_mod
      use Flavour_mod, only : init_flav
      use PhotonFragmentation_mod, only : iz_frag
      use Run_mod
#ifdef USEFASTJET
      use FastJet_mod, only : init_fj, get_n_fj
#endif
      implicit none
      integer :: npar, nR, min_npar, max_npar
      integer :: max_npartons, max_nphotons
      call currentContrib_chan(npar=npar, nR=nR)
      !> all sizes optimized for the specific integration
      min_npar = 2+n_fs_spectators_proc+n_fs_photons_proc+n_fs_partons_proc  ! QCD Born
      max_npar = npar
      max_npartons = n_fs_partons_proc + nR
      max_nphotons = n_fs_photons_proc + nR + max_nphotons_brems  ! better be safe than sorry
      call initBuffer_isol(max_nphotons, max_npartons)
      !> for P2B we can go lower than the "natural" Born level
      if (min_npar_p2b_mult > 0) then
        min_npar = min(min_npar, min_npar_p2b_mult)
      end if

      call init_kin( nmin = min_npar
     &             , nmax = max_npar
     &             , jetOffsetLower = ibegin_partons_proc-1
     &             , jetOffsetUpper = (npar-nR)-(ibegin_partons_proc-1+n_fs_partons_proc)
     &             , nphotons = n_fs_photons_proc + 4  ! + 4 for H->4l
#ifdef USEFASTJET
     &             , nfj = get_n_fj()
#endif
     &             )  ! end init_kin
      call initBuffer_jet(max_npartons)
      call initBuffer_obs(max_npartons)
      call setRcone_jet()  ! copy inputRcone to local Rcone[Sq] variables
      call setMu_scl()  ! copy inputMu[R|F] values into local storage
      call initBuffer_hist()
      call initBuffer_sel()
      call initBuffer_multi_chan()
      call initVFH
      !> default: set Proj2Born off
      call setActive_p2b(.false.)
      call init_flav(min_npar, max_npar, nAct_scl, nUsed_obs)
      call initGrids_pdfi()
      call initCache_pdfc()
      call init_map(max_npar)
      !> init it once in the parallel block
      !> (invalid setting to trigger warnings for debugging)
      iz_frag = -1
      call init_pdf_run()
      call init_hfrag()
#ifdef USEFASTJET
      call init_fj()
#endif
      end


c     do parallel destruction of modules
c     > has to be called once at the end of a parallel block
      subroutine destroy_parallel()
      use KinData_mod
      use Jets_mod
      use Scales_mod
      use Observables_mod
      use Selectors_mod
      use Channels_mod
      use Histograms_mod
      use PhotonIsolation_mod
      use HadronFragmentation_mod, only : destroy_hfrag
      use PDFInterp_mod, only : destroyGrids_pdfi
      use PDFCache_mod, only : destroyCache_pdfc
      use PhaseNJ_mod, only : destroyVFH
      use Mapping_mod
      use Flavour_mod, only : destroy_flav
      use Run_mod
#ifdef USEFASTJET
      use FastJet_mod, only : destroy_fj
#endif
      implicit none
      call destroy_kin()
      call destroyBuffer_jet()
      call destroyBuffer_obs()
      call unsetMu_scl()  ! de-allocate threadprivate storage
      call destroyBuffer_hist()
      call destroyBuffer_isol()
      call destroyBuffer_sel()
      call destroyBuffer_multi_chan()
      call destroyGrids_pdfi()
      call destroyCache_pdfc()
      call destroyVFH()
      call destroy_flav()
      call destroy_map()
      call destroy_pdf_run()
      call destroy_hfrag()
#ifdef USEFASTJET
      call destroy_fj()
#endif
      end



