      subroutine writer
      use Observables_mod
      use Scales_mod
      use Selectors_mod
      use PhotonIsolation_mod
      use LeptonIsolation_mod
      use HadronFragmentation_mod
      use Histograms_mod
      use MultiRun_mod
      use PDFInterp_mod
      use Process_mod
      use Run_mod
      use Flavour_mod
      use DIS_mod
      use Channels_mod
      use Process_mod
      use Parameters_mod
      use Mapping_mod
#ifdef USEFASTJET
      use FastJet_mod, only : digest_fj
#endif
      implicit real*8 (a-h,o-z)
      character*128 slogname
      character*50 sp1,sM1
      character*50 sS1
      character*20 sscale
      character*15 sfac
      logical y0warning,scalewarning
      common /nwa/rm12,rg12,inwa
      common /Zmass/ emz,ezwidth
      common /Wmass/ emw,ewwidth
      common /Hmass/ emh,ehwidth
      common /eweakW/ amz,stw,ewfac
!$omp threadprivate(/eweakW/)
      common /techpar/iang
      common /tcuts/ymin,y0
      common /log/slogname
      common /driverwarnings/y0warning,scalewarning
      character(len=*), parameter :: fmt_line =  '(1x,"*","****************************************************************************************************","*")'
      character(len=*), parameter :: fmt_str =  '(1x,"*",1x,A,T103,"*")'
      character(len=*), parameter :: fmt_section =  '(1x,"*",3x,A,T103,"*")'
      character(len=*), parameter :: fmt_param = '(1x,"*",1x,A,T64,1es26.20E2,T103,"*")'
      character(len=*), parameter :: fmt_int = '(1x,"*",1x,A,T64,I1,T103,"*")'

      print*, "   _  ___  ____   ____     ____________"
      print*, "  / |/ / |/ / /  / __ \__ / / __/_  __/"
      print*, " /    /    / /__/ /_/ / // / _/  / /   "
      print*, "/_/|_/_/|_/____/\____/\___/___/ /_/    "
      print*, "                                       "

      do i=1,2
         if (i.eq.1) iout = 6 ! output to screen
         if (i.eq.2) iout = 500 ! output to log, must not be the same number as the histogramming files
         if (i.eq.2) open (unit=500,file=trim(slogname),action="write")

      write(iout,fmt_line)
      write(iout,fmt_str) ""
      write(iout,fmt_str) " NNLOJET: A multiprocess parton level event generator "
      write(iout,fmt_str) ""
      write(iout,fmt_line)

      if(iboson_proc.ge.-1 .and. iboson_proc.le.1) then
         write(iout,fmt_str) "Active bosons:"
         if (iboson_proc.eq.0) then
            if(igamma_proc.eq.0) then
               write(iout,fmt_str) "  Z0/gamma boson"
            elseif(igamma_proc.eq.2) then
               write(iout,fmt_str) "  Z0 boson only"
            else
               write(iout,fmt_str) "  gamma only"
            endif
         endif
         if (iboson_proc.eq.-1) write(iout,fmt_str) "  W- boson"
         if (iboson_proc.eq.1)  write(iout,fmt_str) "  W+ boson"
      endif
      
      write(iout,fmt_str) ""
      write(iout,fmt_section) "> List of active processes <"
      write(iout,fmt_str) ""
      call digest_chan(iout)
      write(iout,fmt_str) ""

      write(iout,fmt_line)
   
      write(iout,fmt_str) ""
      write(iout,fmt_section) "> Input technical parameters <"
      write(iout,fmt_str) ""
      if (qtcut_veto_run) then
         write(iout,fmt_param) "y0[veto]",y0
      else
         write(iout,fmt_param) "y0",y0
      endif
      if (iang.eq.1) write(iout,fmt_str) "including angular rotations"
      if (iang.eq.0) write(iout,fmt_str) "no angular rotations"
      if ( isActiveCache_map() ) then
         write(iout,fmt_str) "including phase space caching"
      else
         write(iout,fmt_str) "no phase space caching"
      endif
      write(iout,fmt_str) ""

      if(use_EPRC()) write(iout,fmt_str) "EPRC routine is used for alpha_em."
      if(gethera_orderEta()) write(iout,fmt_str) "Jets are ordered in eta in HERA frame."
      if(isSetPol) write(iout,fmt_param) "Electron beam polarisation set to.", Pol

      if(getDISEVTSHAPE()) then
         write(iout,fmt_str) "DIS event shape cut is on:"
         write(iout,fmt_param) "Energy fraction is:",getEhfrac()
      endif

      if (isSetBreitFrame) then
         if(useBreitFrame) then
            write(iout,fmt_str) "Clustering in Breit Frame is set on."
         else
            write(iout,fmt_str) "Clustering in Breit Frame is set off."
         endif
      endif

      write(iout,fmt_str) ""
      write(iout,fmt_section) "> Input physics parameters <"
      write(iout,fmt_str) ""

      write(iout,fmt_param) "Z boson mass",  emz
      write(iout,fmt_param) "Z boson width", ezwidth
      write(iout,fmt_param) "W boson mass",  emw
      write(iout,fmt_param) "W boson width", ewwidth
      write(iout,fmt_param) "H boson mass",  emh
      write(iout,fmt_param) "H boson width", ehwidth
      write(iout,fmt_param) "sin^2(thw)",    stw
      write(iout,fmt_param) "1/alpha",       1d0/amz

      if (qalpha0_hard_photons) then
         write(iout,fmt_int) "rescaling of alpha active:", n_fs_photons_proc
         write(iout,fmt_param) "1/alpha0", 1d0/alpha0_param()
      end if

      write(iout,fmt_str) ""
      write(iout,fmt_section) "> Input kinematical parameters <"
      write(iout,fmt_str) ""
      call digest_proc(iout)  ! process, beams
      write(iout,fmt_str) ""
      call digest_run(iout)   ! order, pdf, warnings, sweeps
      call digest_flav(iout)  ! flavour tagging
      call digest_hfrag(iout) ! hadron fragmentation

#ifdef USEFASTJET
      write(iout,fmt_str) ""
      call digest_fj(iout)
#endif

      write(iout,fmt_str) ""
      write(iout,fmt_section) "> Scales <"
      write(iout,fmt_str) ""
      call digest_scl(iout)

      write(iout,fmt_str) ""
      write(iout,fmt_section) "> Selectors <"
      write(iout,fmt_str) ""
      call digest_sel(iout)

      call digest_isol(iout)
      call digest_lepisol(iout)

      write(iout,fmt_str) ""
      write(iout,fmt_section) "> Histograms <"
      write(iout,fmt_str) ""
      call digest_hist(iout)

      call digest_mult(iout)
      call digest_pdfi(iout)

      write(iout,fmt_str) ""
      write(iout,fmt_line)

      if(y0warning)then
         write(iout,fmt_str) "!WARNING: Your y0 cut may be too large/small!"
      endif
      if(scalewarning)then
         write(iout,fmt_str) "!WARNING: requested additional scales are being ignored because a warmup run has been selected!"
      endif
      if((((name_proc(1:3).eq.'DIS').or.(name_proc(3:5).eq.'DIS').or.(name_proc(3:5).eq.'DIS')).and.(name_proc.ne.'DISG'))
     &     .and.(inwa.eq.1))then
         write(iout,fmt_str) "!WARNING: Breit Wigner resonance sampling enabled for DIS process by using inwa=1."
         write(iout,fmt_str) "          Please check that this is intended if considering photon exchange (it probably isn't!)."
      endif
      if(i.eq.2) close (500)

      enddo

 10   format(1x,A,A,1pe12.4,A,A)
 11   format(1x,A,A,i2,A,A)
 12   format(1x,A,A,A,A,A)
 20   format(1x,A,A,1f12.4,A,A)
 21   format(1x,A,A,1e30.20,A,A)
 22   format(1x,"*",I10,8x,A8,5x,1f12.4,A15,5x,1f12.4,A15,10x,"*")

      return
      end
