!-------------------------------------------------------------------------------
! NNLOJET
!-------------------------------------------------------------------------------
!
! MODULE: sigGGqt [external members: `_GGqt`]
!
! DESCRIPTION:
!> Module providing qT-subtraction components for GG-production up to N3LO
!-------------------------------------------------------------------------------

!#define GGQTN3LO

!-------------------------------------------------------------------------------
module sigGGqt_mod
!-------------------------------------------------------------------------------
  use Constants_mod
  use Scales_mod, only : nMax_scl
  implicit none
  private

  public :: init_GGqt
  public :: set_qtcut_GGqt, qtcut_GGqt
  public :: set_boost_GGqt
  public :: set_kinematics_GGqt, sample_recoil_GGqt
  !> the differential cross section to be integrated
  public :: sigGGqt

  !> run settings
  double precision :: qtcut_ = 1d0  ! [GeV]
  integer :: qtcutmultiplier_ = 1  ! multiplier of qtcut_ if boost > 1, default value is 1
  logical :: qboost_ = .true.

  !----- local setting for the cross section:

  integer, parameter :: npar_  = 4
  integer, parameter :: nc = 3
  integer, parameter :: nf = 5
  integer, parameter :: nup = 2
  integer, parameter :: ndn = 3

  !> variables setting the kinematics
  double precision :: Q_, Y_, uons_
!$OMP THREADPRIVATE(Q_, Y_, uons_)
  !> variables for the recoil kinematics (& jacobian weight)
  double precision :: QT_, PHI_, WGT_
!$OMP THREADPRIVATE(QT_, PHI_, WGT_)

  !> variables setting the channels
  !>  we use bit flags to switch on/off partonic channels
  integer, parameter :: ibit_gg_   = 0  !fAgBg up
  integer, parameter :: ibit_ug_   = 1  !fAqBg up
  integer, parameter :: ibit_dg_   = 2  !fAqBg down
  integer, parameter :: ibit_qpg_  = 3  !fAqpBg up
  integer, parameter :: ibit_ubg_  = 4  !fAqbBg up
  integer, parameter :: ibit_dbg_  = 5  !fAqbBg down
  integer, parameter :: ibit_qbpg_ = 6  !fAqbpBg up
  integer, parameter :: ibit_gu_   = 7  !fAgBq up
  integer, parameter :: ibit_gd_   = 8  !fAgBq down
  integer, parameter :: ibit_gqp_  = 9  !fAgBqp up
  integer, parameter :: ibit_gub_  = 10  !fAgBqb up
  integer, parameter :: ibit_gdb_  = 11  !fAgBqb down
  integer, parameter :: ibit_gqbp_ = 12  !fAgBqbp up
  integer, parameter :: ibit_uub_  = 13  !fAqBqb up
  integer, parameter :: ibit_ddb_  = 14  !fAqBqb down
  integer, parameter :: ibit_uqbp_ = 15  !fAqBqbp up
  integer, parameter :: ibit_dqbp_ = 16  !fAqBqbp down
  integer, parameter :: ibit_qpub_ = 17  !fAqpBqb up
  integer, parameter :: ibit_qpdb_ = 18  !fAqpBqb down
  integer, parameter :: ibit_uu_   = 19  !fAqBq up
  integer, parameter :: ibit_dd_   = 20  !fAqBq down
  integer, parameter :: ibit_uqp_  = 21  !fAqBqp up
  integer, parameter :: ibit_dqp_  = 22  !fAqBqp down
  integer, parameter :: ibit_qpu_  = 23  !fAqpBq up
  integer, parameter :: ibit_qpd_  = 24  !fAqpBq down
  integer, parameter :: ibit_ubub_ = 25  !fAqbBqb up
  integer, parameter :: ibit_dbdb_ = 26  !fAqbBqb down
  integer, parameter :: ibit_ubqbp_= 27  !fAqbBqbp up
  integer, parameter :: ibit_dbqbp_= 28  !fAqbBqbp down
  integer, parameter :: ibit_qbpub_= 29  !fAqbpBqb up
  integer, parameter :: ibit_qbpdb_= 30  !fAqbpBqb down
  integer, parameter :: ibit_ubu_  = 31  !fAqbBq up
  integer, parameter :: ibit_dbd_  = 32  !fAqbBq down
  integer, parameter :: ibit_ubqp_ = 33  !fAqbBqp up
  integer, parameter :: ibit_dbqp_ = 34  !fAqbBqp down
  integer, parameter :: ibit_qbpu_ = 35  !fAqbpBq up
  integer, parameter :: ibit_qbpd_ = 36  !fAqbpBq down
  integer, parameter :: nbit_chan_ = 36

  !> local cache of variables that are expensive to compute
  double precision :: x1_, x2_, z1_, z2_
  double precision :: omx1_, omx2_, dlomx1_, dlomx2_
  !# double precision :: denz1_, denz2_, denomz1_, denomz2_, denopz1_, denopz2_
  double precision, dimension(nMax_scl) :: dlQTcut_, dlQ_, dlQT_, dlR_
  double precision, dimension(nMax_scl) :: as_
  double precision :: hra1_(-1:1),hra2_(-1:1,-1:1),hra3_(-1:1,-1:1,-1:1),  &
                    & hra4_(-1:1,-1:1,-1:1,-1:1),  &
                    & hra5_(-1:1,-1:1,-1:1,-1:1,-1:1)
  double precision :: hrb1_(-1:1),hrb2_(-1:1,-1:1),hrb3_(-1:1,-1:1,-1:1),  &
                    & hrb4_(-1:1,-1:1,-1:1,-1:1),  &
                    & hrb5_(-1:1,-1:1,-1:1,-1:1,-1:1)
  double precision :: hrc1_(-1:1),hrc2_(-1:1,-1:1),hrc3_(-1:1,-1:1,-1:1),  &
                    & hrc4_(-1:1,-1:1,-1:1,-1:1),  &
                    & hrc5_(-1:1,-1:1,-1:1,-1:1,-1:1),  &
                    & hrc6_(-1:1,-1:1,-1:1,-1:1,-1:1,-1:1)
!$OMP THREADPRIVATE(x1_, x2_, z1_, z2_)
!$OMP THREADPRIVATE(omx1_, omx2_, dlomx1_, dlomx2_)
!# $OMP THREADPRIVATE(denz1_, denz2_, denomz1_, denomz2_, denopz1_, denopz2_)
!$OMP THREADPRIVATE(dlQTcut_, dlQ_, dlQT_, dlR_)
!$OMP THREADPRIVATE(as_)
!$OMP THREADPRIVATE(hra1_, hra2_, hra3_, hra4_, hra5_)
!$OMP THREADPRIVATE(hrb1_, hrb2_, hrb3_, hrb4_, hrb5_)
!$OMP THREADPRIVATE(hrc1_, hrc2_, hrc3_, hrc4_, hrc5_)


contains


  !-----------------------------------------------------------------------------
  !> @brief
  !> set parmeters of the cross section
  !-----------------------------------------------------------------------------
  subroutine init_GGqt()
    use Process_mod, only : name_proc
    integer :: i

    print*, "init_GGqt: setting up SCET... ", name_proc

  end subroutine init_GGqt


  !-----------------------------------------------------------------------------
  !> @brief
  !> set the upper value for the qtcut value (cumulant)
  !
  !> @param[in] Q the invariant mass
  !-----------------------------------------------------------------------------
  subroutine set_qtcut_GGqt(qtcut)
    double precision, intent(in) :: qtcut
    if (qtcut <= 0d0) then
      print*, "set_qtcut_GGqt: invalid qt cut:", qtcut
      stop
    end if
    print*, "set_qtcut_GGqt: ", qtcut
    ! read(*,*)
    qtcut_ = qtcut
    ! this is the value used in accumulated contribution
    ! never book histogram selector with ptz lower than this value
  end subroutine set_qtcut_GGqt


  !-----------------------------------------------------------------------------
  !> @brief
  !> getter for the qt cut value
  !-----------------------------------------------------------------------------
  double precision function qtcut_GGqt()
    qtcut_GGqt = qtcut_*dble(qtcutmultiplier_)
  end function qtcut_GGqt

  !-----------------------------------------------------------------------------
  !> @brief
  !> switch on/off the boost and the associated local counterterm
  !> If multiplier is presented, we expect small qtcut_ value:
  !>    The allowed pT sampling is [qtcut_, qtcut_*qtcutmultiplier_]
  !>    This is to avoid large power correction cancellation in the zero bin in region [tcut,qtcut_]
  !>    By only taking accumGGqt_ result in the zero bin and only bin diffGGqt_ in the [qtcut_, qtcut_*qtcutmultiplier_] region.
  !>    To enable histograms from multiple qtcut_ values in one run with stable zero bin
  !
  !> @param[in] active
  !> @param[in] multiplier
  !-----------------------------------------------------------------------------
  subroutine set_boost_GGqt(active,multiplier)
    logical, intent(in) :: active
    integer, intent(in), optional :: multiplier
    qboost_ = active
    if ( (present(multiplier)) .and. qboost_) then
      if (multiplier.le.1) then
         write(*,*) "Invalid (<1) boost multiplier in set_boost_GGqt: ", multiplier
         stop
      else
        qtcutmultiplier_ = multiplier
      endif
    end if
    print*, "set_boost_GGqt qboost_: ", qboost_
    print*, "set_boost_GGqt qtcutmultiplier_: ", qtcutmultiplier_
  end subroutine set_boost_GGqt

  !-----------------------------------------------------------------------------
  !> @brief
  !> set the variables that define the Born kinematics
  !> of the colour-less final state (= H)
  !
  !> @param[in] Q the invariant mass
  !> @param[in] Y the rapidity
  !-----------------------------------------------------------------------------
  subroutine set_kinematics_GGqt(Q, Y)
    double precision, intent(in) :: Q, Y
    ! print*, "set_kinematics_GGqt: ", Q, Y
    Q_   = Q
    Y_   = Y
  end subroutine set_kinematics_GGqt


  !-----------------------------------------------------------------------------
  !> @brief
  !> sample the recoil kinematics (qt, phi)
  !> of the colour-less final state (= H)
  !
  !> @param[in] r1 random number [0,1] for qT  sampling in  [0, qtcut_] or [qtcut_,qtcut_*boostmultiply] region
  !> @param[in] r2 random number [0,1] for phi sampling in  [0, 2pi]
  !-----------------------------------------------------------------------------
  subroutine sample_recoil_GGqt(r1, r2)
    use Run_mod, only : tcut_run
    double precision, intent(in) :: r1, r2
    double precision :: qtmin, qtmax
    ! print*, "sample_recoil_GGqt: ", r1, r2
    qtmax = qtcut_GGqt()
    if (qtmax.ne.qtcut_) then
      !qtcutmultiplier_ is active and larger than 1, only sample QT_ from qtcut_
        qtmin = max(tcut_run * Q_, qtcut_)
    else
        qtmin = tcut_run * Q_
    endif

    WGT_ = 1d0
    !>----- QT
    !> linear mapping
    ! QT_  = r1 * qtcut_
    ! WGT_ = WGT_ * qtcut_
    !> log mapping (requires non-vanishing lower bound)
    QT_ = qtmin * (qtmax/qtmin)**r1
    WGT_ = WGT_ * QT_ * log(qtmax/qtmin)
    !>----- PHI
    PHI_ = r2 * twopi
    !> this is WRONG, as PHI was inclusive!
    !> WGT_ = WGT_ * twopi
    ! print*, " > qtmin = ", qtmin
    ! print*, " > QT    = ", QT_
    ! print*, " > PHI   = ", PHI_
  end subroutine sample_recoil_GGqt


  !-----------------------------------------------------------------------------
  !> @brief
  !> the differential cross section
  !
  !> @param[in] rnorm
  !> @param[in] x1
  !> @param[in] x2
  !> @param[in] wgt
  !-----------------------------------------------------------------------------
  double precision function sigGGqt(rnorm, x1_in, x2_in, wgt)
    use KinData_mod
    use Channels_mod
    use Scales_mod
    use Parameters_mod, only : alpha_param
    double precision, intent(in) :: rnorm, x1_in, x2_in, wgt

    double precision :: x1curr,x2curr
    common /parfrac/x1curr,x2curr
!$omp threadprivate(/parfrac/)

    double precision :: xreg1,xreg2
    logical :: virtsflag
    common /xregions/xreg1,xreg2,virtsflag
!$omp threadprivate(/xregions/)

    logical :: jflag
    common /jacobian/jflag
!$omp threadprivate(/jacobian/)

    double precision :: rnormcurr,wgtcurr
    common /histonorm/rnormcurr,wgtcurr
!$omp threadprivate(/histonorm/)

    !> the flavour-dependence of the Born MEs
    double precision, external :: B0g0GG
    double precision, external :: B2g1GG
    integer :: nfB
    common /BZFlav/ nfB
!$omp threadprivate(/BZFlav/)
    double precision :: facB, facBgg, born_ddb, born_dbd, born_uub, born_ubu, born_gg

    double precision, external :: bino

    double precision, dimension(nMax_scl) :: kinwt, partons
    double precision :: fac, wt, wtsum
    integer :: ipass, ix, iorder, norder, itype

    ! print*, "sigGGqt: ", rnorm, x1_in, x2_in, wgt

    current_chan = 1  ! only one channel

    x1curr = x1_in
    x2curr = x2_in
    rnormcurr = rnorm  ! 1d0  ! <- override?
    wgtcurr = wgt

    sigGGqt = 0d0
    wtsum = 0d0
    kinwt = 0d0
    wt    = 0d0
    fac   = 1d0
    virtsflag = .true.
    ! jflag = .false.  ! <- already have the 1/(1-xi) in our expressions
    jflag = .true.  ! <- for stability, we pulled it out

    ! if ( (abs(1d0-xreg1_kin) < 1d-7) .or. (abs(1d0-xreg2_kin) < 1d-7) ) return

    type_loop : do itype=1,3
    ! print*,  "itype = ", itype
    if ( (itype>1) .and. ( .not.qboost_ ) ) cycle type_loop
    select case (itype)
    case (1)
      ipass = 1
      call ecuts_vj(0,npar_,ipass)
      if (ipass==0) cycle type_loop
      fac = 1d0
      call init_Bornscale_()
      !> cache varibles depend on scales => AFTER ecuts!
      call init_cache_()
      !> cache the Born MEs (diffevernt flavours & permutations):
      facB   = (4d0*pi*alpha_param())**2*nc*4d0 / (nc*nc*2d0*2d0)
      facBgg = (4d0*pi*alpha_param())**2*16d0*(nc**2-1d0) / ((nc**2-1d0)**2*2d0*2d0)
      nfB = 1  ! down-type
      born_ddb = facB * B0g0GG(1,2,3,4)
      born_dbd = facB * B0g0GG(2,1,3,4)
      nfB = 2  ! up-type
      born_uub = facB * B0g0GG(1,2,3,4)
      born_ubu = facB * B0g0GG(2,1,3,4)
      !gg channel loop-induced
      born_gg  = facBgg * B2g1GG(1,2,3,4)
      if (qtcut_<QT_) cycle type_loop
      !skip zero pt bin (accumXX) if one uses qtcutmultiplier_ > 1
      !But still need Born ME initialised above for diffXX in case(3)
    case (2)
      !> eveything same as case 1, just include Jac
      !> if sampled QT_ value is larger than qtcut_, we accumulate the weight from diffGGqt.inc
      !> if sampled QT_ value is smaller than qtcut_, we subtract the weight from diffGGqt.inc,
      !> since it has been included in accumGGqt.inc.
      if ( (ipass==0) .or. (qtcut_<QT_) ) cycle type_loop
      fac = - WGT_
    case (3)
      call init_Bornscale_()
      call apply_boost_()  ! must come last & only once!
      ipass = 1
      call ecuts_vj(0,npar_,ipass)
      if ( ipass==0 ) cycle type_loop
      fac = + WGT_
      !> need to recompute scale-dependent cached vars
      call init_cache_()
    end select



    order_loop : do iorder=0,3
    ! print*, "iorder = ", iorder, get_order_inc_chan(iorder)
    if (.not.get_order_inc_chan(iorder)) cycle order_loop
    do ix=1,4
      ! print*, "ix = ", ix

      !--- q qb -> GG + X
      select case(itype)
      case (1)
        continue  !#killed: call accumGGqt_qqb_qqb(ix, iorder, kinwt)
      case (2,3)
        continue  !#killed: call diffGGqt_qqb_qqb(ix, iorder, kinwt)
      end select
      call getlumi_(ibit_uub_, ix, partons)
      wt = bino(ix, partons, fac * born_uub * kinwt * as_**iorder)
      wtsum = wtsum + wt
      call getlumi_(ibit_ddb_, ix, partons)
      wt = bino(ix, partons, fac * born_ddb * kinwt * as_**iorder)
      wtsum = wtsum + wt

      ! !--- q qb -> GG + X  (this is zero up to N3LO)
      ! select case(itype)
      ! case (1)
      !   call accumGGqt_qqb_qbq(ix, iorder, kinwt)
      ! case (2,3)
      !   call diffGGqt_qqb_qbq(ix, iorder, kinwt)
      ! end select
      ! call getlumi_(ibit_uub_, ix, partons)
      ! wt = bino(ix, partons, fac * born_ubu * kinwt * as_**iorder)
      ! wtsum = wtsum + wt
      ! call getlumi_(ibit_ddb_, ix, partons)
      ! wt = bino(ix, partons, fac * born_dbd * kinwt * as_**iorder)
      ! wtsum = wtsum + wt

      !--- q qb -> GG + X nfv2 contribution
      select case(itype)
      case (1)
        continue  !#killed: call accumGGqt_qqbnfv2_qqb(ix, iorder, kinwt)
      case (2,3)
        continue  !#killed: call diffGGqt_qqbnfv2_qqb(ix, iorder, kinwt)
      end select
      call getlumi_(ibit_uub_, ix, partons)
      wt = bino(ix, partons, fac * 11d0/4d0*born_uub * kinwt * as_**iorder)
      !11/4 is (\sum_nf=5 eq(i)^2)/eq(up)^2=11/9*(4/9)^-1 
      !For nfv2 contribution has two photon connected to a closed quark loop of nf=5 instead of external up-typ quark
      wtsum = wtsum + wt
      call getlumi_(ibit_ddb_, ix, partons)
      wt = bino(ix, partons, fac * 11d0*born_ddb * kinwt * as_**iorder)
      !11 is (\sum_nf=5 eq(i)^2)/eq(down)^2=11/9*(1/9)^-1 
      !For nfv2 contribution has two photon connected to a closed quark loop of nf=5 instead of external down-typ quark
      wtsum = wtsum + wt

      !--- qb q -> GG + X nfv2 contribution
      select case(itype)
      case (1)
        continue  !#killed: call accumGGqt_qbqnfv2_qbq(ix, iorder, kinwt)
      case (2,3)
        continue  !#killed: call diffGGqt_qbqnfv2_qbq(ix, iorder, kinwt)
      end select
      call getlumi_(ibit_ubu_, ix, partons)
      wt = bino(ix, partons, fac * 11d0/4d0*born_ubu * kinwt * as_**iorder)
      !11/4 is (\sum_nf=5 eq(i)^2)/eq(up)^2=11/9*(4/9)^-1 
      !For nfv2 contribution has two photon connected to a closed quark loop of nf=5 instead of external up-typ quark
      wtsum = wtsum + wt
      call getlumi_(ibit_dbd_, ix, partons)
      wt = bino(ix, partons, fac * 11d0*born_dbd * kinwt * as_**iorder)
      !11 is (\sum_nf=5 eq(i)^2)/eq(down)^2=11/9*(1/9)^-1 
      !For nfv2 contribution has two photon connected to a closed quark loop of nf=5 instead of external down-typ quark
      wtsum = wtsum + wt

      !--- q qb -> GG + X nfv contribution
      select case(itype)
      case (1)
        continue  !#killed: call accumGGqt_qqbnfv_qqb(ix, iorder, kinwt)
      case (2,3)
        continue  !#killed: call diffGGqt_qqbnfv_qqb(ix, iorder, kinwt)
      end select
      call getlumi_(ibit_uub_, ix, partons)
      wt = bino(ix, partons, fac * 1d0/2d0*born_uub * kinwt * as_**iorder)
      !1/2 is (\sum_nf=5 eq(i))/eq(up)=1/3*(2/3)^-1 
      !For nfv contribution has one photon connected to a closed quark loop of nf=5 instead of external up-typ quark
      wtsum = wtsum + wt
      call getlumi_(ibit_ddb_, ix, partons)
      wt = bino(ix, partons, fac * (-1d0)*born_ddb * kinwt * as_**iorder)
      !-1 is (\sum_nf=5 eq(i))/eq(down)=1/3*(-1/3)^-1 
      !For nfv contribution has one photon connected to a closed quark loop of nf=5 instead of external down-typ quark
      wtsum = wtsum + wt

      !--- qb q -> GG + X nfv contribution
      select case(itype)
      case (1)
        continue  !#killed: call accumGGqt_qbqnfv_qbq(ix, iorder, kinwt)
      case (2,3)
        continue  !#killed: call diffGGqt_qbqnfv_qbq(ix, iorder, kinwt)
      end select
      call getlumi_(ibit_ubu_, ix, partons)
      wt = bino(ix, partons, fac * 1d0/2d0*born_ubu * kinwt * as_**iorder)
      !1/2 is (\sum_nf=5 eq(i))/eq(up)=1/3*(2/3)^-1 
      !For nfv contribution has one photon connected to a closed quark loop of nf=5 instead of external up-typ quark
      wtsum = wtsum + wt
      call getlumi_(ibit_dbd_, ix, partons)
      wt = bino(ix, partons, fac * (-1d0)*born_dbd * kinwt * as_**iorder)
      !-1 is (\sum_nf=5 eq(i))/eq(down)=1/3*(-1/3)^-1 
      !For nfv contribution has one photon connected to a closed quark loop of nf=5 instead of external down-typ quark
      wtsum = wtsum + wt

      !--- q qb' -> GG + X
      select case(itype)
      case (1)
        continue  !#killed: call accumGGqt_qqbp_qqb(ix, iorder, kinwt)
      case (2,3)
        continue  !#killed: call diffGGqt_qqbp_qqb(ix, iorder, kinwt)
      end select
      call getlumi_(ibit_uqbp_, ix, partons)
      wt = bino(ix, partons, fac * born_uub * kinwt * as_**iorder)
      wtsum = wtsum + wt
      call getlumi_(ibit_dqbp_, ix, partons)
      wt = bino(ix, partons, fac * born_ddb * kinwt * as_**iorder)
      wtsum = wtsum + wt

      ! !--- q qb' -> GG + X
      ! select case(itype)
      ! case (1)
      !   call accumGGqt_qqbp_qbq(ix, iorder, kinwt)
      ! case (2,3)
      !   call diffGGqt_qqbp_qbq(ix, iorder, kinwt)
      ! end select
      ! call getlumi_(ibit_uqbp_, ix, partons)
      ! wt = bino(ix, partons, fac * born_ubu * kinwt * as_**iorder)
      ! wtsum = wtsum + wt
      ! call getlumi_(ibit_dqbp_, ix, partons)
      ! wt = bino(ix, partons, fac * born_dbd * kinwt * as_**iorder)
      ! wtsum = wtsum + wt

      !--- q' qb -> GG + X
      select case(itype)
      case (1)
        continue  !#killed: call accumGGqt_qpqb_qqb(ix, iorder, kinwt)
      case (2,3)
        continue  !#killed: call diffGGqt_qpqb_qqb(ix, iorder, kinwt)
      end select
      call getlumi_(ibit_qpub_, ix, partons)
      wt = bino(ix, partons, fac * born_uub * kinwt * as_**iorder)
      wtsum = wtsum + wt
      call getlumi_(ibit_qpdb_, ix, partons)
      wt = bino(ix, partons, fac * born_ddb * kinwt * as_**iorder)
      wtsum = wtsum + wt

      ! !--- q' qb -> GG + X
      ! select case(itype)
      ! case (1)
      !   call accumGGqt_qpqb_qbq(ix, iorder, kinwt)
      ! case (2,3)
      !   call diffGGqt_qpqb_qbq(ix, iorder, kinwt)
      ! end select
      ! call getlumi_(ibit_qpub_, ix, partons)
      ! wt = bino(ix, partons, fac * born_ubu * kinwt * as_**iorder)
      ! wtsum = wtsum + wt
      ! call getlumi_(ibit_qpdb_, ix, partons)
      ! wt = bino(ix, partons, fac * born_dbd * kinwt * as_**iorder)
      ! wtsum = wtsum + wt

      ! !--- qb q -> GG + X  (this is zero up to N3LO)
      ! select case(itype)
      ! case (1)
      !   call accumGGqt_qbq_qqb(ix, iorder, kinwt)
      ! case (2,3)
      !   call diffGGqt_qbq_qqb(ix, iorder, kinwt)
      ! end select
      ! call getlumi_(ibit_ubu_, ix, partons)
      ! wt = bino(ix, partons, fac * born_uub * kinwt * as_**iorder)
      ! wtsum = wtsum + wt
      ! call getlumi_(ibit_dbd_, ix, partons)
      ! wt = bino(ix, partons, fac * born_ddb * kinwt * as_**iorder)
      ! wtsum = wtsum + wt

      !--- qb q -> GG + X
      select case(itype)
      case (1)
        continue  !#killed: call accumGGqt_qbq_qbq(ix, iorder, kinwt)
      case (2,3)
        continue  !#killed: call diffGGqt_qbq_qbq(ix, iorder, kinwt)
      end select
      call getlumi_(ibit_ubu_, ix, partons)
      wt = bino(ix, partons, fac * born_ubu * kinwt * as_**iorder)
      wtsum = wtsum + wt
      call getlumi_(ibit_dbd_, ix, partons)
      wt = bino(ix, partons, fac * born_dbd * kinwt * as_**iorder)
      wtsum = wtsum + wt

      ! !--- qb q' -> GG + X  (this is zero up to N3LO)
      ! select case(itype)
      ! case (1)
      !   call accumGGqt_qbqp_qqb(ix, iorder, kinwt)
      ! case (2,3)
      !   call diffGGqt_qbqp_qqb(ix, iorder, kinwt)
      ! end select
      ! call getlumi_(ibit_ubqp_, ix, partons)
      ! wt = bino(ix, partons, fac * born_uub * kinwt * as_**iorder)
      ! wtsum = wtsum + wt
      ! call getlumi_(ibit_dbqp_, ix, partons)
      ! wt = bino(ix, partons, fac * born_ddb * kinwt * as_**iorder)
      ! wtsum = wtsum + wt

      !--- q qb -> GG + X nfv2 contribution
      select case(itype)
      case (1)
        continue  !#killed: call accumGGqt_qqbnfv2_qbq(ix, iorder, kinwt)
      case (2,3)
        continue  !#killed: call diffGGqt_qqbnfv2_qbq(ix, iorder, kinwt)
      end select
      call getlumi_(ibit_uub_, ix, partons)
      wt = bino(ix, partons, fac * 11d0/4d0*born_ubu * kinwt * as_**iorder)
      !11/4 is (\sum_nf=5 eq(i)^2)/eq(up)^2=11/9*(4/9)^-1 
      !For nfv2 contribution has two photon connected to a closed quark loop of nf=5 instead of external up-typ quark
      wtsum = wtsum + wt
      call getlumi_(ibit_ddb_, ix, partons)
      wt = bino(ix, partons, fac * 11d0*born_dbd * kinwt * as_**iorder)
      !11 is (\sum_nf=5 eq(i)^2)/eq(down)^2=11/9*(1/9)^-1 
      !For nfv2 contribution has two photon connected to a closed quark loop of nf=5 instead of external down-typ quark
      wtsum = wtsum + wt

      !--- qb q' -> GG + X  (this is zero up to N3LO)
      select case(itype)
      case (1)
        continue  !#killed: call accumGGqt_qbqp_qbq(ix, iorder, kinwt)
      case (2,3)
        continue  !#killed: call diffGGqt_qbqp_qbq(ix, iorder, kinwt)
      end select
      call getlumi_(ibit_ubqp_, ix, partons)
      wt = bino(ix, partons, fac * born_ubu * kinwt * as_**iorder)
      wtsum = wtsum + wt
      call getlumi_(ibit_dbqp_, ix, partons)
      wt = bino(ix, partons, fac * born_dbd * kinwt * as_**iorder)
      wtsum = wtsum + wt

      ! !--- qb' q -> GG + X  (this is zero up to N3LO)
      ! select case(itype)
      ! case (1)
      !   call accumGGqt_qbpq_qqb(ix, iorder, kinwt)
      ! case (2,3)
      !   call diffGGqt_qbpq_qqb(ix, iorder, kinwt)
      ! end select
      ! call getlumi_(ibit_qbpu_, ix, partons)
      ! wt = bino(ix, partons, fac * born_uub * kinwt * as_**iorder)
      ! wtsum = wtsum + wt
      ! call getlumi_(ibit_qbpd_, ix, partons)
      ! wt = bino(ix, partons, fac * born_ddb * kinwt * as_**iorder)
      ! wtsum = wtsum + wt

      !--- qb' q -> GG + X  (this is zero up to N3LO)
      select case(itype)
      case (1)
        continue  !#killed: call accumGGqt_qbpq_qbq(ix, iorder, kinwt)
      case (2,3)
        continue  !#killed: call diffGGqt_qbpq_qbq(ix, iorder, kinwt)
      end select
      call getlumi_(ibit_qbpu_, ix, partons)
      wt = bino(ix, partons, fac * born_ubu * kinwt * as_**iorder)
      wtsum = wtsum + wt
      call getlumi_(ibit_qbpd_, ix, partons)
      wt = bino(ix, partons, fac * born_dbd * kinwt * as_**iorder)
      wtsum = wtsum + wt

      !--- q g -> GG + X
      select case(itype)
      case (1)
        continue  !#killed: call accumGGqt_qg_qqb(ix, iorder, kinwt)
      case (2,3)
        continue  !#killed: call diffGGqt_qg_qqb(ix, iorder, kinwt)
      end select
      call getlumi_(ibit_ug_, ix, partons)
      wt = bino(ix, partons, fac * born_uub * kinwt * as_**iorder)
      wtsum = wtsum + wt
      call getlumi_(ibit_dg_, ix, partons)
      wt = bino(ix, partons, fac * born_ddb * kinwt * as_**iorder)
      wtsum = wtsum + wt

      !--- q g -> GG + X
      select case(itype)
      case (1)
        continue  !#killed: call accumGGqt_qg_qbq(ix, iorder, kinwt)
      case (2,3)
        continue  !#killed: call diffGGqt_qg_qbq(ix, iorder, kinwt)
      end select
      call getlumi_(ibit_ug_, ix, partons)
      wt = bino(ix, partons, fac * born_ubu * kinwt * as_**iorder)
      wtsum = wtsum + wt
      call getlumi_(ibit_dg_, ix, partons)
      wt = bino(ix, partons, fac * born_dbd * kinwt * as_**iorder)
      wtsum = wtsum + wt

      !--- q' g -> GG + X
      select case(itype)
      case (1)
        continue  !#killed: call accumGGqt_qpg_qqb(ix, iorder, kinwt)
      case (2,3)
        continue  !#killed: call diffGGqt_qpg_qqb(ix, iorder, kinwt)
      end select
      call getlumi_(ibit_qpg_, ix, partons)
      wt = bino(ix, partons, fac * (nup*born_uub+ndn*born_ddb) * kinwt * as_**iorder)
      wtsum = wtsum + wt

      !--- q' g -> GG + X
      select case(itype)
      case (1)
        continue  !#killed: call accumGGqt_qpg_qbq(ix, iorder, kinwt)
      case (2,3)
        continue  !#killed: call diffGGqt_qpg_qbq(ix, iorder, kinwt)
      end select
      call getlumi_(ibit_qpg_, ix, partons)
      wt = bino(ix, partons, fac * (nup*born_ubu+ndn*born_dbd) * kinwt * as_**iorder)
      wtsum = wtsum + wt

      !--- g q -> GG + X
      select case(itype)
      case (1)
        continue  !#killed: call accumGGqt_gq_qqb(ix, iorder, kinwt)
      case (2,3)
        continue  !#killed: call diffGGqt_gq_qqb(ix, iorder, kinwt)
      end select
      call getlumi_(ibit_gu_, ix, partons)
      wt = bino(ix, partons, fac * born_uub * kinwt * as_**iorder)
      wtsum = wtsum + wt
      call getlumi_(ibit_gd_, ix, partons)
      wt = bino(ix, partons, fac * born_ddb * kinwt * as_**iorder)
      wtsum = wtsum + wt

      !--- g q -> GG + X
      select case(itype)
      case (1)
        continue  !#killed: call accumGGqt_gq_qbq(ix, iorder, kinwt)
      case (2,3)
        continue  !#killed: call diffGGqt_gq_qbq(ix, iorder, kinwt)
      end select
      call getlumi_(ibit_gu_, ix, partons)
      wt = bino(ix, partons, fac * born_ubu * kinwt * as_**iorder)
      wtsum = wtsum + wt
      call getlumi_(ibit_gd_, ix, partons)
      wt = bino(ix, partons, fac * born_dbd * kinwt * as_**iorder)
      wtsum = wtsum + wt

      !--- g q' -> GG + X
      select case(itype)
      case (1)
        continue  !#killed: call accumGGqt_gqp_qqb(ix, iorder, kinwt)
      case (2,3)
        continue  !#killed: call diffGGqt_gqp_qqb(ix, iorder, kinwt)
      end select
      call getlumi_(ibit_gqp_, ix, partons)
      wt = bino(ix, partons, fac * (nup*born_uub+ndn*born_ddb) * kinwt * as_**iorder)
      wtsum = wtsum + wt

      !--- g q' -> GG + X
      select case(itype)
      case (1)
        continue  !#killed: call accumGGqt_gqp_qbq(ix, iorder, kinwt)
      case (2,3)
        continue  !#killed: call diffGGqt_gqp_qbq(ix, iorder, kinwt)
      end select
      call getlumi_(ibit_gqp_, ix, partons)
      wt = bino(ix, partons, fac * (nup*born_ubu+ndn*born_dbd) * kinwt * as_**iorder)
      wtsum = wtsum + wt


      !--- q g -> GG + X nfv2 contribution
      select case(itype)
      case (1)
        continue  !#killed: call accumGGqt_qgnfv2_qqb(ix, iorder, kinwt)
      case (2,3)
        continue  !#killed: call diffGGqt_qgnfv2_qqb(ix, iorder, kinwt)
      end select
      call getlumi_(ibit_ug_, ix, partons)
      wt = bino(ix, partons, fac * 11d0/4d0*born_uub * kinwt * as_**iorder)
      !11/4 is (\sum_nf=5 eq(i)^2)/eq(up)^2=11/9*(4/9)^-1 
      !For nfv2 contribution has two photon connected to a closed quark loop of nf=5 instead of external up-typ quark
      wtsum = wtsum + wt
      call getlumi_(ibit_dg_, ix, partons)
      wt = bino(ix, partons, fac * 11d0*born_ddb * kinwt * as_**iorder)
      !11 is (\sum_nf=5 eq(i)^2)/eq(down)^2=11/9*(1/9)^-1 
      !For nfv2 contribution has two photon connected to a closed quark loop of nf=5 instead of external down-typ quark
      wtsum = wtsum + wt

      !--- g q -> GG + X nfv2 contribution
      select case(itype)
      case (1)
        continue  !#killed: call accumGGqt_gqnfv2_qbq(ix, iorder, kinwt)
      case (2,3)
        continue  !#killed: call diffGGqt_gqnfv2_qbq(ix, iorder, kinwt)
      end select
      call getlumi_(ibit_gu_, ix, partons)
      wt = bino(ix, partons, fac * 11d0/4d0*born_ubu * kinwt * as_**iorder)
      !11/4 is (\sum_nf=5 eq(i)^2)/eq(up)^2=11/9*(4/9)^-1 
      !For nfv2 contribution has two photon connected to a closed quark loop of nf=5 instead of external up-typ quark
      wtsum = wtsum + wt
      call getlumi_(ibit_gd_, ix, partons)
      wt = bino(ix, partons, fac * 11d0*born_dbd * kinwt * as_**iorder)
      !11 is (\sum_nf=5 eq(i)^2)/eq(down)^2=11/9*(1/9)^-1 
      !For nfv2 contribution has two photon connected to a closed quark loop of nf=5 instead of external down-typ quark
      wtsum = wtsum + wt

      !--- qb g -> GG + X
      select case(itype)
      case (1)
        continue  !#killed: call accumGGqt_qbg_qqb(ix, iorder, kinwt)
      case (2,3)
        continue  !#killed: call diffGGqt_qbg_qqb(ix, iorder, kinwt)
      end select
      call getlumi_(ibit_ubg_, ix, partons)
      wt = bino(ix, partons, fac * born_uub * kinwt * as_**iorder)
      wtsum = wtsum + wt
      call getlumi_(ibit_dbg_, ix, partons)
      wt = bino(ix, partons, fac * born_ddb * kinwt * as_**iorder)
      wtsum = wtsum + wt

      !--- qb g -> GG + X
      select case(itype)
      case (1)
        continue  !#killed: call accumGGqt_qbg_qbq(ix, iorder, kinwt)
      case (2,3)
        continue  !#killed: call diffGGqt_qbg_qbq(ix, iorder, kinwt)
      end select
      call getlumi_(ibit_ubg_, ix, partons)
      wt = bino(ix, partons, fac * born_ubu * kinwt * as_**iorder)
      wtsum = wtsum + wt
      call getlumi_(ibit_dbg_, ix, partons)
      wt = bino(ix, partons, fac * born_dbd * kinwt * as_**iorder)
      wtsum = wtsum + wt

      !--- qb' g -> GG + X
      select case(itype)
      case (1)
        continue  !#killed: call accumGGqt_qbpg_qqb(ix, iorder, kinwt)
      case (2,3)
        continue  !#killed: call diffGGqt_qbpg_qqb(ix, iorder, kinwt)
      end select
      call getlumi_(ibit_qbpg_, ix, partons)
      wt = bino(ix, partons, fac * (nup*born_uub+ndn*born_ddb) * kinwt * as_**iorder)
      wtsum = wtsum + wt

      !--- qb' g -> GG + X
      select case(itype)
      case (1)
        continue  !#killed: call accumGGqt_qbpg_qbq(ix, iorder, kinwt)
      case (2,3)
        continue  !#killed: call diffGGqt_qbpg_qbq(ix, iorder, kinwt)
      end select
      call getlumi_(ibit_qbpg_, ix, partons)
      wt = bino(ix, partons, fac * (nup*born_ubu+ndn*born_dbd) * kinwt * as_**iorder)
      wtsum = wtsum + wt

      !--- g qb -> GG + X
      select case(itype)
      case (1)
        continue  !#killed: call accumGGqt_gqb_qqb(ix, iorder, kinwt)
      case (2,3)
        continue  !#killed: call diffGGqt_gqb_qqb(ix, iorder, kinwt)
      end select
      call getlumi_(ibit_gub_, ix, partons)
      wt = bino(ix, partons, fac * born_uub * kinwt * as_**iorder)
      wtsum = wtsum + wt
      call getlumi_(ibit_gdb_, ix, partons)
      wt = bino(ix, partons, fac * born_ddb * kinwt * as_**iorder)
      wtsum = wtsum + wt

      !--- g qb -> GG + X
      select case(itype)
      case (1)
        continue  !#killed: call accumGGqt_gqb_qbq(ix, iorder, kinwt)
      case (2,3)
        continue  !#killed: call diffGGqt_gqb_qbq(ix, iorder, kinwt)
      end select
      call getlumi_(ibit_gub_, ix, partons)
      wt = bino(ix, partons, fac * born_ubu * kinwt * as_**iorder)
      wtsum = wtsum + wt
      call getlumi_(ibit_gdb_, ix, partons)
      wt = bino(ix, partons, fac * born_dbd * kinwt * as_**iorder)
      wtsum = wtsum + wt

      !--- g qb' -> GG + X
      select case(itype)
      case (1)
        continue  !#killed: call accumGGqt_gqbp_qqb(ix, iorder, kinwt)
      case (2,3)
        continue  !#killed: call diffGGqt_gqbp_qqb(ix, iorder, kinwt)
      end select
      call getlumi_(ibit_gqbp_, ix, partons)
      wt = bino(ix, partons, fac * (nup*born_uub+ndn*born_ddb) * kinwt * as_**iorder)
      wtsum = wtsum + wt

      !--- g qb' -> GG + X
      select case(itype)
      case (1)
        continue  !#killed: call accumGGqt_gqbp_qbq(ix, iorder, kinwt)
      case (2,3)
        continue  !#killed: call diffGGqt_gqbp_qbq(ix, iorder, kinwt)
      end select
      call getlumi_(ibit_gqbp_, ix, partons)
      wt = bino(ix, partons, fac * (nup*born_ubu+ndn*born_dbd) * kinwt * as_**iorder)
      wtsum = wtsum + wt

      !--- g qb -> GG + X nfv2 contribution
      select case(itype)
      case (1)
        continue  !#killed: call accumGGqt_gqbnfv2_qqb(ix, iorder, kinwt)
      case (2,3)
        continue  !#killed: call diffGGqt_gqbnfv2_qqb(ix, iorder, kinwt)
      end select
      call getlumi_(ibit_gub_, ix, partons)
      wt = bino(ix, partons, fac * 11d0/4d0*born_uub * kinwt * as_**iorder)
      !11/4 is (\sum_nf=5 eq(i)^2)/eq(up)^2=11/9*(4/9)^-1 
      !For nfv2 contribution has two photon connected to a closed quark loop of nf=5 instead of external up-typ quark
      wtsum = wtsum + wt
      call getlumi_(ibit_gdb_, ix, partons)
      wt = bino(ix, partons, fac * 11d0*born_ddb * kinwt * as_**iorder)
      !11 is (\sum_nf=5 eq(i)^2)/eq(down)^2=11/9*(1/9)^-1 
      !For nfv2 contribution has two photon connected to a closed quark loop of nf=5 instead of external down-typ quark
      wtsum = wtsum + wt

      !--- qb g -> GG + X nfv2 contribution
      select case(itype)
      case (1)
        continue  !#killed: call accumGGqt_qbgnfv2_qbq(ix, iorder, kinwt)
      case (2,3)
        continue  !#killed: call diffGGqt_qbgnfv2_qbq(ix, iorder, kinwt)
      end select
      call getlumi_(ibit_ubg_, ix, partons)
      wt = bino(ix, partons, fac * 11d0/4d0*born_ubu * kinwt * as_**iorder)
      !11/4 is (\sum_nf=5 eq(i)^2)/eq(up)^2=11/9*(4/9)^-1 
      !For nfv2 contribution has two photon connected to a closed quark loop of nf=5 instead of external up-typ quark
      wtsum = wtsum + wt
      call getlumi_(ibit_dbg_, ix, partons)
      wt = bino(ix, partons, fac * 11d0*born_dbd * kinwt * as_**iorder)
      !11 is (\sum_nf=5 eq(i)^2)/eq(down)^2=11/9*(1/9)^-1 
      !For nfv2 contribution has two photon connected to a closed quark loop of nf=5 instead of external down-typ quark
      wtsum = wtsum + wt

      !--- g g -> GG + X
      select case(itype)
      case (1)
        continue  !#killed: call accumGGqt_gg_qqb(ix, iorder, kinwt)
      case (2,3)
        continue  !#killed: call diffGGqt_gg_qqb(ix, iorder, kinwt)
      end select
      call getlumi_(ibit_gg_, ix, partons)
      wt = bino(ix, partons, fac * (nup*born_uub+ndn*born_ddb) * kinwt * as_**iorder)
      wtsum = wtsum + wt

      !--- g g -> GG + X
      select case(itype)
      case (1)
        continue  !#killed: call accumGGqt_gg_qbq(ix, iorder, kinwt)
      case (2,3)
        continue  !#killed: call diffGGqt_gg_qbq(ix, iorder, kinwt)
      end select
      call getlumi_(ibit_gg_, ix, partons)
      wt = bino(ix, partons, fac * (nup*born_ubu+ndn*born_dbd) * kinwt * as_**iorder)
      wtsum = wtsum + wt

      !--- g g -> GG + X loop-induced
      select case(itype)
      case (1)
        continue  !#killed: call accumGGqt_gg_gg(ix, iorder, kinwt)
      case (2,3)
        continue  !#killed: call diffGGqt_gg_gg(ix, iorder, kinwt)
      end select
      call getlumi_(ibit_gg_, ix, partons)
      wt = bino(ix, partons, fac * born_gg * kinwt * as_**iorder)
      wtsum = wtsum + wt

      !--- q q -> GG + X
      select case(itype)
      case (1)
        continue  !#killed: call accumGGqt_qq_qqb(ix, iorder, kinwt)
      case (2,3)
        continue  !#killed: call diffGGqt_qq_qqb(ix, iorder, kinwt)
      end select
      call getlumi_(ibit_uu_, ix, partons)
      wt = bino(ix, partons, fac * born_uub * kinwt * as_**iorder)
      wtsum = wtsum + wt
      call getlumi_(ibit_dd_, ix, partons)
      wt = bino(ix, partons, fac * born_ddb * kinwt * as_**iorder)
      wtsum = wtsum + wt

      !--- q q -> GG + X
      select case(itype)
      case (1)
        continue  !#killed: call accumGGqt_qq_qbq(ix, iorder, kinwt)
      case (2,3)
        continue  !#killed: call diffGGqt_qq_qbq(ix, iorder, kinwt)
      end select
      call getlumi_(ibit_uu_, ix, partons)
      wt = bino(ix, partons, fac * born_ubu * kinwt * as_**iorder)
      wtsum = wtsum + wt
      call getlumi_(ibit_dd_, ix, partons)
      wt = bino(ix, partons, fac * born_dbd * kinwt * as_**iorder)
      wtsum = wtsum + wt

      !--- q q' -> GG + X
      select case(itype)
      case (1)
        continue  !#killed: call accumGGqt_qqp_qqb(ix, iorder, kinwt)
      case (2,3)
        continue  !#killed: call diffGGqt_qqp_qqb(ix, iorder, kinwt)
      end select
      call getlumi_(ibit_uqp_, ix, partons)
      wt = bino(ix, partons, fac * born_uub * kinwt * as_**iorder)
      wtsum = wtsum + wt
      call getlumi_(ibit_dqp_, ix, partons)
      wt = bino(ix, partons, fac * born_ddb * kinwt * as_**iorder)
      wtsum = wtsum + wt

      !--- q q' -> GG + X
      select case(itype)
      case (1)
        continue  !#killed: call accumGGqt_qqp_qbq(ix, iorder, kinwt)
      case (2,3)
        continue  !#killed: call diffGGqt_qqp_qbq(ix, iorder, kinwt)
      end select
      call getlumi_(ibit_uqp_, ix, partons)
      wt = bino(ix, partons, fac * born_ubu * kinwt * as_**iorder)
      wtsum = wtsum + wt
      call getlumi_(ibit_dqp_, ix, partons)
      wt = bino(ix, partons, fac * born_dbd * kinwt * as_**iorder)
      wtsum = wtsum + wt

      !--- q' q -> GG + X
      select case(itype)
      case (1)
        continue  !#killed: call accumGGqt_qpq_qqb(ix, iorder, kinwt)
      case (2,3)
        continue  !#killed: call diffGGqt_qpq_qqb(ix, iorder, kinwt)
      end select
      call getlumi_(ibit_qpu_, ix, partons)
      wt = bino(ix, partons, fac * born_uub * kinwt * as_**iorder)
      wtsum = wtsum + wt
      call getlumi_(ibit_qpd_, ix, partons)
      wt = bino(ix, partons, fac * born_ddb * kinwt * as_**iorder)
      wtsum = wtsum + wt

      !--- q' q -> GG + X
      select case(itype)
      case (1)
        continue  !#killed: call accumGGqt_qpq_qbq(ix, iorder, kinwt)
      case (2,3)
        continue  !#killed: call diffGGqt_qpq_qbq(ix, iorder, kinwt)
      end select
      call getlumi_(ibit_qpu_, ix, partons)
      wt = bino(ix, partons, fac * born_ubu * kinwt * as_**iorder)
      wtsum = wtsum + wt
      call getlumi_(ibit_qpd_, ix, partons)
      wt = bino(ix, partons, fac * born_dbd * kinwt * as_**iorder)
      wtsum = wtsum + wt

      !--- qb qb -> GG + X
      select case(itype)
      case (1)
        continue  !#killed: call accumGGqt_qbqb_qqb(ix, iorder, kinwt)
      case (2,3)
        continue  !#killed: call diffGGqt_qbqb_qqb(ix, iorder, kinwt)
      end select
      call getlumi_(ibit_ubub_, ix, partons)
      wt = bino(ix, partons, fac * born_uub * kinwt * as_**iorder)
      wtsum = wtsum + wt
      call getlumi_(ibit_dbdb_, ix, partons)
      wt = bino(ix, partons, fac * born_ddb * kinwt * as_**iorder)
      wtsum = wtsum + wt

      !--- qb qb -> GG + X
      select case(itype)
      case (1)
        continue  !#killed: call accumGGqt_qbqb_qbq(ix, iorder, kinwt)
      case (2,3)
        continue  !#killed: call diffGGqt_qbqb_qbq(ix, iorder, kinwt)
      end select
      call getlumi_(ibit_ubub_, ix, partons)
      wt = bino(ix, partons, fac * born_ubu * kinwt * as_**iorder)
      wtsum = wtsum + wt
      call getlumi_(ibit_dbdb_, ix, partons)
      wt = bino(ix, partons, fac * born_dbd * kinwt * as_**iorder)
      wtsum = wtsum + wt

      ! !--- qb qb' -> GG + X  (is zero up to N3LO)
      ! select case(itype)
      ! case (1)
      !   call accumGGqt_qbqbp_qqb(ix, iorder, kinwt)
      ! case (2,3)
      !   call diffGGqt_qbqbp_qqb(ix, iorder, kinwt)
      ! end select
      ! call getlumi_(ibit_ubqbp_, ix, partons)
      ! wt = bino(ix, partons, fac * born_uub * kinwt * as_**iorder)
      ! wtsum = wtsum + wt
      ! call getlumi_(ibit_dbqbp_, ix, partons)
      ! wt = bino(ix, partons, fac * born_ddb * kinwt * as_**iorder)
      ! wtsum = wtsum + wt

      !--- qb qb' -> GG + X
      select case(itype)
      case (1)
        continue  !#killed: call accumGGqt_qbqbp_qbq(ix, iorder, kinwt)
      case (2,3)
        continue  !#killed: call diffGGqt_qbqbp_qbq(ix, iorder, kinwt)
      end select
      call getlumi_(ibit_ubqbp_, ix, partons)
      wt = bino(ix, partons, fac * born_ubu * kinwt * as_**iorder)
      wtsum = wtsum + wt
      call getlumi_(ibit_dbqbp_, ix, partons)
      wt = bino(ix, partons, fac * born_dbd * kinwt * as_**iorder)
      wtsum = wtsum + wt

      !--- qb' qb -> GG + X
      select case(itype)
      case (1)
        continue  !#killed: call accumGGqt_qbpqb_qqb(ix, iorder, kinwt)
      case (2,3)
        continue  !#killed: call diffGGqt_qbpqb_qqb(ix, iorder, kinwt)
      end select
      call getlumi_(ibit_qbpub_, ix, partons)
      wt = bino(ix, partons, fac * born_uub * kinwt * as_**iorder)
      wtsum = wtsum + wt
      call getlumi_(ibit_qbpdb_, ix, partons)
      wt = bino(ix, partons, fac * born_ddb * kinwt * as_**iorder)
      wtsum = wtsum + wt

      ! !--- qb' qb -> GG + X  (is zero up to N3LO)
      ! select case(itype)
      ! case (1)
      !   call accumGGqt_qbpqb_qbq(ix, iorder, kinwt)
      ! case (2,3)
      !   call diffGGqt_qbpqb_qbq(ix, iorder, kinwt)
      ! end select
      ! call getlumi_(ibit_qbpub_, ix, partons)
      ! wt = bino(ix, partons, fac * born_ubu * kinwt * as_**iorder)
      ! wtsum = wtsum + wt
      ! call getlumi_(ibit_qbpdb_, ix, partons)
      ! wt = bino(ix, partons, fac * born_dbd * kinwt * as_**iorder)
      ! wtsum = wtsum + wt

    end do  ! end do ix
    end do order_loop
    end do type_loop

    sigGGqt = rnorm * wtsum

    !> in case of a NaN, reject the full event
    if ( sigGGqt /= sigGGqt ) then
      ireject_event_chan = -2
      sigGGqt = 0d0
    end if

    ! print*, "sigGGqt = ", sigGGqt
    ! read(*,*)
  end function sigGGqt


  !-----------------------------------------------------------------------------
  !> @brief
  !> boost the final state using QT_ & PHI_
  !-----------------------------------------------------------------------------
  subroutine apply_boost_()
    use KinData_mod
    type(Kin_t), pointer :: kd
    integer :: i
    double precision :: Q2, chY, shY, MT
    double precision :: pX(4), pXb(4), r(4)
    double precision :: sXXb, siX, sir

    Q2 = Q_**2
    chY = cosh(Y_)
    shY = sinh(Y_)
    MT  = sqrt( Q2 + QT_**2 )

    pX(1) = 0d0
    pX(2) = 0d0
    pX(3) = Q_ * shY
    pX(4) = Q_ * chY

    pXb(1) = QT_ * cos(PHI_)
    pXb(2) = QT_ * sin(PHI_)
    pXb(3) = MT * shY
    pXb(4) = MT * chY

    r(:) = pX(:) + pXb(:)

    sXXb = 2d0*Q_*(MT+Q_)

    !> prettify code
    kd => kin(npar_)

    do i = 3, npar_
      !> scalar products
      sir = 2d0*dot(kd%p(:,i), r)
      siX = 2d0*dot(kd%p(:,i), pX)
      kd%p(:,i) = kd%p(:,i)  &
                & - sir / sXXb * r(:)  &
                & + siX / Q2 * pXb(:)
    end do

    call fillS_kin(npar_)
    call clearCacheLocal_kin(npar_)

  end subroutine apply_boost_

  !-----------------------------------------------------------------------------
  !> @brief
  !> initialise the Born variables that enters the SCET hard function
  !> Note: those variables stay the same in linear power correction after boost
  !-----------------------------------------------------------------------------
  subroutine init_Bornscale_()
    use KinData_mod, only : s4

    uons_ = abs(s4(1,3)/Q_**2)
!     if(uons_.ge.1d0)then
!       print*,"========init_Bornscale_=========="
!       print*,"s4(1,3) = ",s4(1,3)
!       print*,"Q_^2    = ",Q_**2
!       print*,"s4(1,2) = ",s4(1,2)
!       print*,"uons_   = ",uons_
! !     stop
!     endif

  end subroutine init_Bornscale_
  !-----------------------------------------------------------------------------
  !> @brief
  !> initialise the local variables
  !-----------------------------------------------------------------------------
  subroutine init_cache_()
    use KinData_mod, only : x1_kin, x2_kin, xreg1_kin, xreg2_kin, s4
    use Scales_mod
    use Parameters_mod, only : mt_param
    use LHAPDF_mod, only : alphasQ_lhapdf
    integer :: iscl, idmap
    !> dummy arguments for calling hplog5
    integer :: nw  ! max weight for hplog: [1,5]
    complex(8) :: hc1(-1:1),  &
                & hc2(-1:1,-1:1),  &
                & hc3(-1:1,-1:1,-1:1),  &
                & hc4(-1:1,-1:1,-1:1,-1:1),  &
                & hc5(-1:1,-1:1,-1:1,-1:1,-1:1),  &
                & hc6(-1:1,-1:1,-1:1,-1:1,-1:1,-1:1)
    double precision :: hi1(-1:1),  &
                      & hi2(-1:1,-1:1),  &
                      & hi3(-1:1,-1:1,-1:1),  &
                      & hi4(-1:1,-1:1,-1:1,-1:1),  &
                      & hi5(-1:1,-1:1,-1:1,-1:1,-1:1),  &
                      & hi6(-1:1,-1:1,-1:1,-1:1,-1:1,-1:1)
    ! print*, "init_cache_"

    x1_ = x1_kin
    x2_ = x2_kin
    z1_ = xreg1_kin
    z2_ = xreg2_kin

    omx1_ = (1D0 - x1_)
    omx2_ = (1D0 - x2_)
    dlomx1_ = log(omx1_)
    dlomx2_ = log(omx2_)

    ! denz1_ = 1d0 / z1_
    ! denz2_ = 1d0 / z2_
    ! denomz1_ = 1D0 / (1D0 - z1_)
    ! denomz2_ = 1D0 / (1D0 - z2_)
    ! denopz1_ = 1D0 / (1D0 + z1_)
    ! denopz2_ = 1D0 / (1D0 + z2_)

    dlQ_(:)     = 0d0
    dlQT_(:)    = 0d0
    dlQTcut_(:) = 0d0
    do iscl=1,nAct_scl
      idmap = idmap_scl(iscl, withF=.true.)
      if (idmap == iscl) then
        dlQ_(iscl)     =  2D0 * log(Q_     / muF_scl(iscl))
        dlQT_(iscl)    =  2D0 * log(QT_    / muF_scl(iscl))
        dlQTcut_(iscl) =  2D0 * log(qtcut_ / muF_scl(iscl))
      else
        dlQ_(iscl)     = dlQ_(idmap)
        dlQT_(iscl)    = dlQT_(idmap)
        dlQTcut_(iscl) = dlQTcut_(idmap)
      end if
    enddo

    dlR_(:)    = 0d0
    do iscl=1,nAct_scl
      dlR_(iscl) = 2D0 * log(muR_scl(iscl) / muF_scl(iscl))
    end do

    as_ = alphas_scl / (4d0*pi)

    !> harmonic polylogs
    !> @todo optimise depending on the order that is requested!
    nw = 5

    hra1_ = 0d0
    hra2_ = 0d0
    hra3_ = 0d0
    hra4_ = 0d0
    hra5_ = 0d0
    call hplog5( z1_, nw,  &
               & hc1,   hc2,   hc3,   hc4,   hc5,     &
               & hra1_, hra2_, hra3_, hra4_, hra5_,   &
               & hi1,   hi2,   hi3,   hi4,   hi5,     &
               & -1, +1)

    hrb1_ = 0d0
    hrb2_ = 0d0
    hrb3_ = 0d0
    hrb4_ = 0d0
    hrb5_ = 0d0
    call hplog5( z2_, nw,  &
               & hc1,   hc2,   hc3,   hc4,   hc5,     &
               & hrb1_, hrb2_, hrb3_, hrb4_, hrb5_,   &
               & hi1,   hi2,   hi3,   hi4,   hi5,     &
               & -1, +1)


    hrc1_ = 0d0
    hrc2_ = 0d0
    hrc3_ = 0d0
    hrc4_ = 0d0
    hrc5_ = 0d0
    hrc6_ = 0d0

#ifdef GGQTN3LO
    nw = 6
    call hplog6( uons_, nw,  &
               & hc1,   hc2,   hc3,   hc4,   hc5,   hc6,     &
               & hrc1_, hrc2_, hrc3_, hrc4_, hrc5_, hrc6_,   &
               & hi1,   hi2,   hi3,   hi4,   hi5,   hi6,     &
               & -1, +1)
#else
    call hplog5( uons_, nw,  &
               & hc1,   hc2,   hc3,   hc4,   hc5,     &
               & hrc1_, hrc2_, hrc3_, hrc4_, hrc5_,   &
               & hi1,   hi2,   hi3,   hi4,   hi5,     &
               & -1, +1)
#endif

  end subroutine init_cache_


!> load the qT-accumulated expressions
!#killed: #include "accumGGqt.inc"


!> load the qT-differential expressions
!#killed: #include "diffGGqt.inc"


  subroutine getlumi_(ichan, ix, partons)
    use Scales_mod
    use Channels_mod
    use CKM_mod
    use Process_mod
    integer, intent(in) :: ichan, ix
    double precision, intent(out), dimension(nMax_scl) :: partons
    double precision, dimension(:,:), pointer :: f1, f2
    double precision :: f1lumi(-6:7,1:nMax_scl), f2lumi(-6:7,1:nMax_scl)

    select case (ix)
    case (1)
       f1(-6:7,1:nMax_scl) => f1_scl
       f2(-6:7,1:nMax_scl) => f2_scl
    case (2)
       f1(-6:7,1:nMax_scl) => f1_scl
       f2(-6:7,1:nMax_scl) => f2x2_scl
    case (3)
       f1(-6:7,1:nMax_scl) => f1x1_scl
       f2(-6:7,1:nMax_scl) => f2_scl
    case (4)
       f1(-6:7,1:nMax_scl) => f1x1_scl
       f2(-6:7,1:nMax_scl) => f2x2_scl
    end select

    f1lumi(-6:7,1:nMax_scl) = f1(-6:7,1:nMax_scl)
    f2lumi(-6:7,1:nMax_scl) = f2(-6:7,1:nMax_scl)

    if ( beam1_proc==pid_ap ) then
      f1lumi(-6,:) = f1( 6,:)
      f1lumi(-5,:) = f1( 5,:)
      f1lumi(-4,:) = f1( 4,:)
      f1lumi(-3,:) = f1( 3,:)
      f1lumi(-2,:) = f1( 2,:)
      f1lumi(-1,:) = f1( 1,:)
      f1lumi( 6,:) = f1(-6,:)
      f1lumi( 5,:) = f1(-5,:)
      f1lumi( 4,:) = f1(-4,:)
      f1lumi( 3,:) = f1(-3,:)
      f1lumi( 2,:) = f1(-2,:)
      f1lumi( 1,:) = f1(-1,:)    
    end if

    if ( beam2_proc==pid_ap ) then
      f2lumi(-6,:) = f2( 6,:)
      f2lumi(-5,:) = f2( 5,:)
      f2lumi(-4,:) = f2( 4,:)
      f2lumi(-3,:) = f2( 3,:)
      f2lumi(-2,:) = f2( 2,:)
      f2lumi(-1,:) = f2( 1,:)
      f2lumi( 6,:) = f2(-6,:)
      f2lumi( 5,:) = f2(-5,:)
      f2lumi( 4,:) = f2(-4,:)
      f2lumi( 3,:) = f2(-3,:)
      f2lumi( 2,:) = f2(-2,:)
      f2lumi( 1,:) = f2(-1,:)    
    end if

    partons = 0d0

    select case (ichan)
    case ( ibit_gg_ )
      ip1_chan = 0
      ip2_chan = 0
      partons(1:nInp_scl) =  &
        &   f1lumi(0,1:nInp_scl)*f2lumi(0,1:nInp_scl)
    case ( ibit_ug_ )
      ip1_chan = 1
      ip2_chan = 0
      partons(1:nInp_scl) =  &
        &   f1lumi(2,1:nInp_scl)*f2lumi(0,1:nInp_scl)  &
        & + f1lumi(4,1:nInp_scl)*f2lumi(0,1:nInp_scl)
    case ( ibit_dg_ )
      ip1_chan = 1
      ip2_chan = 0
      partons(1:nInp_scl) =  &
        &   f1lumi(1,1:nInp_scl)*f2lumi(0,1:nInp_scl)  &
        & + f1lumi(3,1:nInp_scl)*f2lumi(0,1:nInp_scl)  &
        & + f1lumi(5,1:nInp_scl)*f2lumi(0,1:nInp_scl)
    case ( ibit_qpg_ )
      ip1_chan = 1
      ip2_chan = 0
      partons(1:nInp_scl) =  &
        &   f1lumi(1,1:nInp_scl)*f2lumi(0,1:nInp_scl)  &
        & + f1lumi(2,1:nInp_scl)*f2lumi(0,1:nInp_scl)  &
        & + f1lumi(3,1:nInp_scl)*f2lumi(0,1:nInp_scl)  &
        & + f1lumi(4,1:nInp_scl)*f2lumi(0,1:nInp_scl)  &
        & + f1lumi(5,1:nInp_scl)*f2lumi(0,1:nInp_scl)
    case ( ibit_ubg_ )
      ip1_chan = -1
      ip2_chan = 0
      partons(1:nInp_scl) =  &
        &   f1lumi(-2,1:nInp_scl)*f2lumi(0,1:nInp_scl)  &
        & + f1lumi(-4,1:nInp_scl)*f2lumi(0,1:nInp_scl)
    case ( ibit_dbg_ )
      ip1_chan = -1
      ip2_chan = 0
      partons(1:nInp_scl) =  &
        &   f1lumi(-1,1:nInp_scl)*f2lumi(0,1:nInp_scl)  &
        & + f1lumi(-3,1:nInp_scl)*f2lumi(0,1:nInp_scl)  &
        & + f1lumi(-5,1:nInp_scl)*f2lumi(0,1:nInp_scl)
    case ( ibit_qbpg_ )
      ip1_chan = -1
      ip2_chan = 0
      partons(1:nInp_scl) =  &
        &   f1lumi(-1,1:nInp_scl)*f2lumi(0,1:nInp_scl)  &
        & + f1lumi(-2,1:nInp_scl)*f2lumi(0,1:nInp_scl)  &
        & + f1lumi(-3,1:nInp_scl)*f2lumi(0,1:nInp_scl)  &
        & + f1lumi(-4,1:nInp_scl)*f2lumi(0,1:nInp_scl)  &
        & + f1lumi(-5,1:nInp_scl)*f2lumi(0,1:nInp_scl)
    case ( ibit_gu_ )
      ip1_chan = 0
      ip2_chan = 1
      partons(1:nInp_scl) =  &
        &   f1lumi(0,1:nInp_scl)*f2lumi(2,1:nInp_scl)  &
        & + f1lumi(0,1:nInp_scl)*f2lumi(4,1:nInp_scl)
    case ( ibit_gd_ )
      ip1_chan = 0
      ip2_chan = 1
      partons(1:nInp_scl) =  &
        &   f1lumi(0,1:nInp_scl)*f2lumi(1,1:nInp_scl)  &
        & + f1lumi(0,1:nInp_scl)*f2lumi(3,1:nInp_scl)  &
        & + f1lumi(0,1:nInp_scl)*f2lumi(5,1:nInp_scl)
    case ( ibit_gqp_ )
      ip1_chan = 0
      ip2_chan = 1
      partons(1:nInp_scl) =  &
        &   f1lumi(0,1:nInp_scl)*f2lumi(1,1:nInp_scl)  &
        & + f1lumi(0,1:nInp_scl)*f2lumi(2,1:nInp_scl)  &
        & + f1lumi(0,1:nInp_scl)*f2lumi(3,1:nInp_scl)  &
        & + f1lumi(0,1:nInp_scl)*f2lumi(4,1:nInp_scl)  &
        & + f1lumi(0,1:nInp_scl)*f2lumi(5,1:nInp_scl)
    case ( ibit_gub_ )
      ip1_chan = 0
      ip2_chan = -1
      partons(1:nInp_scl) =  &
        &   f1lumi(0,1:nInp_scl)*f2lumi(-2,1:nInp_scl)  &
        & + f1lumi(0,1:nInp_scl)*f2lumi(-4,1:nInp_scl)
    case ( ibit_gdb_ )
      ip1_chan = 0
      ip2_chan = -1
      partons(1:nInp_scl) =  &
        &   f1lumi(0,1:nInp_scl)*f2lumi(-1,1:nInp_scl)  &
        & + f1lumi(0,1:nInp_scl)*f2lumi(-3,1:nInp_scl)  &
        & + f1lumi(0,1:nInp_scl)*f2lumi(-5,1:nInp_scl)
    case ( ibit_gqbp_ )
      ip1_chan = 0
      ip2_chan = -1
      partons(1:nInp_scl) =  &
        &   f1lumi(0,1:nInp_scl)*f2lumi(-1,1:nInp_scl)  &
        & + f1lumi(0,1:nInp_scl)*f2lumi(-2,1:nInp_scl)  &
        & + f1lumi(0,1:nInp_scl)*f2lumi(-3,1:nInp_scl)  &
        & + f1lumi(0,1:nInp_scl)*f2lumi(-4,1:nInp_scl)  &
        & + f1lumi(0,1:nInp_scl)*f2lumi(-5,1:nInp_scl)
    case ( ibit_uub_ )
      ip1_chan =  1
      ip2_chan = -1
      partons(1:nInp_scl) =  &
        &   f1lumi(2,1:nInp_scl)*f2lumi(-2,1:nInp_scl)  &
        & + f1lumi(4,1:nInp_scl)*f2lumi(-4,1:nInp_scl)
    case ( ibit_ddb_ )
      ip1_chan =  1
      ip2_chan = -1
      partons(1:nInp_scl) =  &
        &   f1lumi(1,1:nInp_scl)*f2lumi(-1,1:nInp_scl)  &
        & + f1lumi(3,1:nInp_scl)*f2lumi(-3,1:nInp_scl)  &
        & + f1lumi(5,1:nInp_scl)*f2lumi(-5,1:nInp_scl)
    case ( ibit_uqbp_ )
      ip1_chan =  1
      ip2_chan = -1
      partons(1:nInp_scl) =  &
        &   f1lumi(2,1:nInp_scl)*f2lumi(-1,1:nInp_scl)  &
        & + f1lumi(2,1:nInp_scl)*f2lumi(-2,1:nInp_scl)  &
        & + f1lumi(2,1:nInp_scl)*f2lumi(-3,1:nInp_scl)  &
        & + f1lumi(2,1:nInp_scl)*f2lumi(-4,1:nInp_scl)  &
        & + f1lumi(2,1:nInp_scl)*f2lumi(-5,1:nInp_scl)  &
        & + f1lumi(4,1:nInp_scl)*f2lumi(-1,1:nInp_scl)  &
        & + f1lumi(4,1:nInp_scl)*f2lumi(-2,1:nInp_scl)  &
        & + f1lumi(4,1:nInp_scl)*f2lumi(-3,1:nInp_scl)  &
        & + f1lumi(4,1:nInp_scl)*f2lumi(-4,1:nInp_scl)  &
        & + f1lumi(4,1:nInp_scl)*f2lumi(-5,1:nInp_scl)
    case ( ibit_dqbp_ )
      ip1_chan =  1
      ip2_chan = -1
      partons(1:nInp_scl) =  &
        &   f1lumi(1,1:nInp_scl)*f2lumi(-1,1:nInp_scl)  &
        & + f1lumi(1,1:nInp_scl)*f2lumi(-2,1:nInp_scl)  &
        & + f1lumi(1,1:nInp_scl)*f2lumi(-3,1:nInp_scl)  &
        & + f1lumi(1,1:nInp_scl)*f2lumi(-4,1:nInp_scl)  &
        & + f1lumi(1,1:nInp_scl)*f2lumi(-5,1:nInp_scl)  &
        & + f1lumi(3,1:nInp_scl)*f2lumi(-1,1:nInp_scl)  &
        & + f1lumi(3,1:nInp_scl)*f2lumi(-2,1:nInp_scl)  &
        & + f1lumi(3,1:nInp_scl)*f2lumi(-3,1:nInp_scl)  &
        & + f1lumi(3,1:nInp_scl)*f2lumi(-4,1:nInp_scl)  &
        & + f1lumi(3,1:nInp_scl)*f2lumi(-5,1:nInp_scl)  &
        & + f1lumi(5,1:nInp_scl)*f2lumi(-1,1:nInp_scl)  &
        & + f1lumi(5,1:nInp_scl)*f2lumi(-2,1:nInp_scl)  &
        & + f1lumi(5,1:nInp_scl)*f2lumi(-3,1:nInp_scl)  &
        & + f1lumi(5,1:nInp_scl)*f2lumi(-4,1:nInp_scl)  &
        & + f1lumi(5,1:nInp_scl)*f2lumi(-5,1:nInp_scl)
    case ( ibit_qpub_ )
      ip1_chan =  1
      ip2_chan = -1
      partons(1:nInp_scl) =  &
        &   f1lumi(1,1:nInp_scl)*f2lumi(-2,1:nInp_scl)  &
        & + f1lumi(2,1:nInp_scl)*f2lumi(-2,1:nInp_scl)  &
        & + f1lumi(3,1:nInp_scl)*f2lumi(-2,1:nInp_scl)  &
        & + f1lumi(4,1:nInp_scl)*f2lumi(-2,1:nInp_scl)  &
        & + f1lumi(5,1:nInp_scl)*f2lumi(-2,1:nInp_scl)  &
        & + f1lumi(1,1:nInp_scl)*f2lumi(-4,1:nInp_scl)  &
        & + f1lumi(2,1:nInp_scl)*f2lumi(-4,1:nInp_scl)  &
        & + f1lumi(3,1:nInp_scl)*f2lumi(-4,1:nInp_scl)  &
        & + f1lumi(4,1:nInp_scl)*f2lumi(-4,1:nInp_scl)  &
        & + f1lumi(5,1:nInp_scl)*f2lumi(-4,1:nInp_scl)
    case ( ibit_qpdb_ )
      ip1_chan =  1
      ip2_chan = -1
      partons(1:nInp_scl) =  &
        &   f1lumi(1,1:nInp_scl)*f2lumi(-1,1:nInp_scl)  &
        & + f1lumi(2,1:nInp_scl)*f2lumi(-1,1:nInp_scl)  &
        & + f1lumi(3,1:nInp_scl)*f2lumi(-1,1:nInp_scl)  &
        & + f1lumi(4,1:nInp_scl)*f2lumi(-1,1:nInp_scl)  &
        & + f1lumi(5,1:nInp_scl)*f2lumi(-1,1:nInp_scl)  &
        & + f1lumi(1,1:nInp_scl)*f2lumi(-3,1:nInp_scl)  &
        & + f1lumi(2,1:nInp_scl)*f2lumi(-3,1:nInp_scl)  &
        & + f1lumi(3,1:nInp_scl)*f2lumi(-3,1:nInp_scl)  &
        & + f1lumi(4,1:nInp_scl)*f2lumi(-3,1:nInp_scl)  &
        & + f1lumi(5,1:nInp_scl)*f2lumi(-3,1:nInp_scl)  &
        & + f1lumi(1,1:nInp_scl)*f2lumi(-5,1:nInp_scl)  &
        & + f1lumi(2,1:nInp_scl)*f2lumi(-5,1:nInp_scl)  &
        & + f1lumi(3,1:nInp_scl)*f2lumi(-5,1:nInp_scl)  &
        & + f1lumi(4,1:nInp_scl)*f2lumi(-5,1:nInp_scl)  &
        & + f1lumi(5,1:nInp_scl)*f2lumi(-5,1:nInp_scl)
    case ( ibit_uu_ )
      ip1_chan =  1
      ip2_chan =  1
      partons(1:nInp_scl) =  &
        &   f1lumi(2,1:nInp_scl)*f2lumi(2,1:nInp_scl)  &
        & + f1lumi(4,1:nInp_scl)*f2lumi(4,1:nInp_scl)
    case ( ibit_dd_ )
      ip1_chan =  1
      ip2_chan =  1
      partons(1:nInp_scl) =  &
        &   f1lumi(1,1:nInp_scl)*f2lumi(1,1:nInp_scl)  &
        & + f1lumi(3,1:nInp_scl)*f2lumi(3,1:nInp_scl)  &
        & + f1lumi(5,1:nInp_scl)*f2lumi(5,1:nInp_scl)
    case ( ibit_uqp_ )
      ip1_chan =  1
      ip2_chan =  1
      partons(1:nInp_scl) =  &
        &   f1lumi(2,1:nInp_scl)*f2lumi(1,1:nInp_scl)  &
        & + f1lumi(2,1:nInp_scl)*f2lumi(2,1:nInp_scl)  &
        & + f1lumi(2,1:nInp_scl)*f2lumi(3,1:nInp_scl)  &
        & + f1lumi(2,1:nInp_scl)*f2lumi(4,1:nInp_scl)  &
        & + f1lumi(2,1:nInp_scl)*f2lumi(5,1:nInp_scl)  &
        & + f1lumi(4,1:nInp_scl)*f2lumi(1,1:nInp_scl)  &
        & + f1lumi(4,1:nInp_scl)*f2lumi(2,1:nInp_scl)  &
        & + f1lumi(4,1:nInp_scl)*f2lumi(3,1:nInp_scl)  &
        & + f1lumi(4,1:nInp_scl)*f2lumi(4,1:nInp_scl)  &
        & + f1lumi(4,1:nInp_scl)*f2lumi(5,1:nInp_scl)
    case ( ibit_dqp_ )
      ip1_chan =  1
      ip2_chan =  1
      partons(1:nInp_scl) =  &
        &   f1lumi(1,1:nInp_scl)*f2lumi(1,1:nInp_scl)  &
        & + f1lumi(1,1:nInp_scl)*f2lumi(2,1:nInp_scl)  &
        & + f1lumi(1,1:nInp_scl)*f2lumi(3,1:nInp_scl)  &
        & + f1lumi(1,1:nInp_scl)*f2lumi(4,1:nInp_scl)  &
        & + f1lumi(1,1:nInp_scl)*f2lumi(5,1:nInp_scl)  &
        & + f1lumi(3,1:nInp_scl)*f2lumi(1,1:nInp_scl)  &
        & + f1lumi(3,1:nInp_scl)*f2lumi(2,1:nInp_scl)  &
        & + f1lumi(3,1:nInp_scl)*f2lumi(3,1:nInp_scl)  &
        & + f1lumi(3,1:nInp_scl)*f2lumi(4,1:nInp_scl)  &
        & + f1lumi(3,1:nInp_scl)*f2lumi(5,1:nInp_scl)  &
        & + f1lumi(5,1:nInp_scl)*f2lumi(1,1:nInp_scl)  &
        & + f1lumi(5,1:nInp_scl)*f2lumi(2,1:nInp_scl)  &
        & + f1lumi(5,1:nInp_scl)*f2lumi(3,1:nInp_scl)  &
        & + f1lumi(5,1:nInp_scl)*f2lumi(4,1:nInp_scl)  &
        & + f1lumi(5,1:nInp_scl)*f2lumi(5,1:nInp_scl)
    case ( ibit_qpu_ )
      ip1_chan =  1
      ip2_chan =  1
      partons(1:nInp_scl) =  &
        &   f1lumi(1,1:nInp_scl)*f2lumi(2,1:nInp_scl)  &
        & + f1lumi(2,1:nInp_scl)*f2lumi(2,1:nInp_scl)  &
        & + f1lumi(3,1:nInp_scl)*f2lumi(2,1:nInp_scl)  &
        & + f1lumi(4,1:nInp_scl)*f2lumi(2,1:nInp_scl)  &
        & + f1lumi(5,1:nInp_scl)*f2lumi(2,1:nInp_scl)  &
        & + f1lumi(1,1:nInp_scl)*f2lumi(4,1:nInp_scl)  &
        & + f1lumi(2,1:nInp_scl)*f2lumi(4,1:nInp_scl)  &
        & + f1lumi(3,1:nInp_scl)*f2lumi(4,1:nInp_scl)  &
        & + f1lumi(4,1:nInp_scl)*f2lumi(4,1:nInp_scl)  &
        & + f1lumi(5,1:nInp_scl)*f2lumi(4,1:nInp_scl)
    case ( ibit_qpd_ )
      ip1_chan =  1
      ip2_chan =  1
      partons(1:nInp_scl) =  &
        &   f1lumi(1,1:nInp_scl)*f2lumi(1,1:nInp_scl)  &
        & + f1lumi(2,1:nInp_scl)*f2lumi(1,1:nInp_scl)  &
        & + f1lumi(3,1:nInp_scl)*f2lumi(1,1:nInp_scl)  &
        & + f1lumi(4,1:nInp_scl)*f2lumi(1,1:nInp_scl)  &
        & + f1lumi(5,1:nInp_scl)*f2lumi(1,1:nInp_scl)  &
        & + f1lumi(1,1:nInp_scl)*f2lumi(3,1:nInp_scl)  &
        & + f1lumi(2,1:nInp_scl)*f2lumi(3,1:nInp_scl)  &
        & + f1lumi(3,1:nInp_scl)*f2lumi(3,1:nInp_scl)  &
        & + f1lumi(4,1:nInp_scl)*f2lumi(3,1:nInp_scl)  &
        & + f1lumi(5,1:nInp_scl)*f2lumi(3,1:nInp_scl)  &
        & + f1lumi(1,1:nInp_scl)*f2lumi(5,1:nInp_scl)  &
        & + f1lumi(2,1:nInp_scl)*f2lumi(5,1:nInp_scl)  &
        & + f1lumi(3,1:nInp_scl)*f2lumi(5,1:nInp_scl)  &
        & + f1lumi(4,1:nInp_scl)*f2lumi(5,1:nInp_scl)  &
        & + f1lumi(5,1:nInp_scl)*f2lumi(5,1:nInp_scl)
    case ( ibit_ubub_ )
      ip1_chan =  -1
      ip2_chan =  -1
      partons(1:nInp_scl) =  &
        &   f1lumi(-2,1:nInp_scl)*f2lumi(-2,1:nInp_scl)  &
        & + f1lumi(-4,1:nInp_scl)*f2lumi(-4,1:nInp_scl)
    case ( ibit_dbdb_ )
      ip1_chan =  -1
      ip2_chan =  -1
      partons(1:nInp_scl) =  &
        &   f1lumi(-1,1:nInp_scl)*f2lumi(-1,1:nInp_scl)  &
        & + f1lumi(-3,1:nInp_scl)*f2lumi(-3,1:nInp_scl)  &
        & + f1lumi(-5,1:nInp_scl)*f2lumi(-5,1:nInp_scl)
    case ( ibit_ubqbp_ )
      ip1_chan = -1
      ip2_chan = -1
      partons(1:nInp_scl) =  &
        &   f1lumi(-2,1:nInp_scl)*f2lumi(-1,1:nInp_scl)  &
        & + f1lumi(-2,1:nInp_scl)*f2lumi(-2,1:nInp_scl)  &
        & + f1lumi(-2,1:nInp_scl)*f2lumi(-3,1:nInp_scl)  &
        & + f1lumi(-2,1:nInp_scl)*f2lumi(-4,1:nInp_scl)  &
        & + f1lumi(-2,1:nInp_scl)*f2lumi(-5,1:nInp_scl)  &
        & + f1lumi(-4,1:nInp_scl)*f2lumi(-1,1:nInp_scl)  &
        & + f1lumi(-4,1:nInp_scl)*f2lumi(-2,1:nInp_scl)  &
        & + f1lumi(-4,1:nInp_scl)*f2lumi(-3,1:nInp_scl)  &
        & + f1lumi(-4,1:nInp_scl)*f2lumi(-4,1:nInp_scl)  &
        & + f1lumi(-4,1:nInp_scl)*f2lumi(-5,1:nInp_scl)
    case ( ibit_dbqbp_ )
      ip1_chan = -1
      ip2_chan = -1
      partons(1:nInp_scl) =  &
        &   f1lumi(-1,1:nInp_scl)*f2lumi(-1,1:nInp_scl)  &
        & + f1lumi(-1,1:nInp_scl)*f2lumi(-2,1:nInp_scl)  &
        & + f1lumi(-1,1:nInp_scl)*f2lumi(-3,1:nInp_scl)  &
        & + f1lumi(-1,1:nInp_scl)*f2lumi(-4,1:nInp_scl)  &
        & + f1lumi(-1,1:nInp_scl)*f2lumi(-5,1:nInp_scl)  &
        & + f1lumi(-3,1:nInp_scl)*f2lumi(-1,1:nInp_scl)  &
        & + f1lumi(-3,1:nInp_scl)*f2lumi(-2,1:nInp_scl)  &
        & + f1lumi(-3,1:nInp_scl)*f2lumi(-3,1:nInp_scl)  &
        & + f1lumi(-3,1:nInp_scl)*f2lumi(-4,1:nInp_scl)  &
        & + f1lumi(-3,1:nInp_scl)*f2lumi(-5,1:nInp_scl)  &
        & + f1lumi(-5,1:nInp_scl)*f2lumi(-1,1:nInp_scl)  &
        & + f1lumi(-5,1:nInp_scl)*f2lumi(-2,1:nInp_scl)  &
        & + f1lumi(-5,1:nInp_scl)*f2lumi(-3,1:nInp_scl)  &
        & + f1lumi(-5,1:nInp_scl)*f2lumi(-4,1:nInp_scl)  &
        & + f1lumi(-5,1:nInp_scl)*f2lumi(-5,1:nInp_scl)
    case ( ibit_qbpub_ )
      ip1_chan = -1
      ip2_chan = -1
      partons(1:nInp_scl) =  &
        &   f1lumi(-1,1:nInp_scl)*f2lumi(-2,1:nInp_scl)  &
        & + f1lumi(-2,1:nInp_scl)*f2lumi(-2,1:nInp_scl)  &
        & + f1lumi(-3,1:nInp_scl)*f2lumi(-2,1:nInp_scl)  &
        & + f1lumi(-4,1:nInp_scl)*f2lumi(-2,1:nInp_scl)  &
        & + f1lumi(-5,1:nInp_scl)*f2lumi(-2,1:nInp_scl)  &
        & + f1lumi(-1,1:nInp_scl)*f2lumi(-4,1:nInp_scl)  &
        & + f1lumi(-2,1:nInp_scl)*f2lumi(-4,1:nInp_scl)  &
        & + f1lumi(-3,1:nInp_scl)*f2lumi(-4,1:nInp_scl)  &
        & + f1lumi(-4,1:nInp_scl)*f2lumi(-4,1:nInp_scl)  &
        & + f1lumi(-5,1:nInp_scl)*f2lumi(-4,1:nInp_scl)
    case ( ibit_qbpdb_ )
      ip1_chan = -1
      ip2_chan = -1
      partons(1:nInp_scl) =  &
        &   f1lumi(-1,1:nInp_scl)*f2lumi(-1,1:nInp_scl)  &
        & + f1lumi(-2,1:nInp_scl)*f2lumi(-1,1:nInp_scl)  &
        & + f1lumi(-3,1:nInp_scl)*f2lumi(-1,1:nInp_scl)  &
        & + f1lumi(-4,1:nInp_scl)*f2lumi(-1,1:nInp_scl)  &
        & + f1lumi(-5,1:nInp_scl)*f2lumi(-1,1:nInp_scl)  &
        & + f1lumi(-1,1:nInp_scl)*f2lumi(-3,1:nInp_scl)  &
        & + f1lumi(-2,1:nInp_scl)*f2lumi(-3,1:nInp_scl)  &
        & + f1lumi(-3,1:nInp_scl)*f2lumi(-3,1:nInp_scl)  &
        & + f1lumi(-4,1:nInp_scl)*f2lumi(-3,1:nInp_scl)  &
        & + f1lumi(-5,1:nInp_scl)*f2lumi(-3,1:nInp_scl)  &
        & + f1lumi(-1,1:nInp_scl)*f2lumi(-5,1:nInp_scl)  &
        & + f1lumi(-2,1:nInp_scl)*f2lumi(-5,1:nInp_scl)  &
        & + f1lumi(-3,1:nInp_scl)*f2lumi(-5,1:nInp_scl)  &
        & + f1lumi(-4,1:nInp_scl)*f2lumi(-5,1:nInp_scl)  &
        & + f1lumi(-5,1:nInp_scl)*f2lumi(-5,1:nInp_scl)
    case ( ibit_ubu_ )
      ip1_chan = -1
      ip2_chan =  1
      partons(1:nInp_scl) =  &
        &   f1lumi(-2,1:nInp_scl)*f2lumi(2,1:nInp_scl)  &
        & + f1lumi(-4,1:nInp_scl)*f2lumi(4,1:nInp_scl)
    case ( ibit_dbd_ )
      ip1_chan = -1
      ip2_chan =  1
      partons(1:nInp_scl) =  &
        &   f1lumi(-1,1:nInp_scl)*f2lumi(1,1:nInp_scl)  &
        & + f1lumi(-3,1:nInp_scl)*f2lumi(3,1:nInp_scl)  &
        & + f1lumi(-5,1:nInp_scl)*f2lumi(5,1:nInp_scl)
    case ( ibit_ubqp_ )
      ip1_chan = -1
      ip2_chan =  1
      partons(1:nInp_scl) =  &
        &   f1lumi(-2,1:nInp_scl)*f2lumi(1,1:nInp_scl)  &
        & + f1lumi(-2,1:nInp_scl)*f2lumi(2,1:nInp_scl)  &
        & + f1lumi(-2,1:nInp_scl)*f2lumi(3,1:nInp_scl)  &
        & + f1lumi(-2,1:nInp_scl)*f2lumi(4,1:nInp_scl)  &
        & + f1lumi(-2,1:nInp_scl)*f2lumi(5,1:nInp_scl)  &
        & + f1lumi(-4,1:nInp_scl)*f2lumi(1,1:nInp_scl)  &
        & + f1lumi(-4,1:nInp_scl)*f2lumi(2,1:nInp_scl)  &
        & + f1lumi(-4,1:nInp_scl)*f2lumi(3,1:nInp_scl)  &
        & + f1lumi(-4,1:nInp_scl)*f2lumi(4,1:nInp_scl)  &
        & + f1lumi(-4,1:nInp_scl)*f2lumi(5,1:nInp_scl)
    case ( ibit_dbqp_ )
      ip1_chan = -1
      ip2_chan =  1
      partons(1:nInp_scl) =  &
        &   f1lumi(-1,1:nInp_scl)*f2lumi(1,1:nInp_scl)  &
        & + f1lumi(-1,1:nInp_scl)*f2lumi(2,1:nInp_scl)  &
        & + f1lumi(-1,1:nInp_scl)*f2lumi(3,1:nInp_scl)  &
        & + f1lumi(-1,1:nInp_scl)*f2lumi(4,1:nInp_scl)  &
        & + f1lumi(-1,1:nInp_scl)*f2lumi(5,1:nInp_scl)  &
        & + f1lumi(-3,1:nInp_scl)*f2lumi(1,1:nInp_scl)  &
        & + f1lumi(-3,1:nInp_scl)*f2lumi(2,1:nInp_scl)  &
        & + f1lumi(-3,1:nInp_scl)*f2lumi(3,1:nInp_scl)  &
        & + f1lumi(-3,1:nInp_scl)*f2lumi(4,1:nInp_scl)  &
        & + f1lumi(-3,1:nInp_scl)*f2lumi(5,1:nInp_scl)  &
        & + f1lumi(-5,1:nInp_scl)*f2lumi(1,1:nInp_scl)  &
        & + f1lumi(-5,1:nInp_scl)*f2lumi(2,1:nInp_scl)  &
        & + f1lumi(-5,1:nInp_scl)*f2lumi(3,1:nInp_scl)  &
        & + f1lumi(-5,1:nInp_scl)*f2lumi(4,1:nInp_scl)  &
        & + f1lumi(-5,1:nInp_scl)*f2lumi(5,1:nInp_scl)
    case ( ibit_qbpu_ )
      ip1_chan = -1
      ip2_chan =  1
      partons(1:nInp_scl) =  &
        &   f1lumi(-1,1:nInp_scl)*f2lumi(2,1:nInp_scl)  &
        & + f1lumi(-2,1:nInp_scl)*f2lumi(2,1:nInp_scl)  &
        & + f1lumi(-3,1:nInp_scl)*f2lumi(2,1:nInp_scl)  &
        & + f1lumi(-4,1:nInp_scl)*f2lumi(2,1:nInp_scl)  &
        & + f1lumi(-5,1:nInp_scl)*f2lumi(2,1:nInp_scl)  &
        & + f1lumi(-1,1:nInp_scl)*f2lumi(4,1:nInp_scl)  &
        & + f1lumi(-2,1:nInp_scl)*f2lumi(4,1:nInp_scl)  &
        & + f1lumi(-3,1:nInp_scl)*f2lumi(4,1:nInp_scl)  &
        & + f1lumi(-4,1:nInp_scl)*f2lumi(4,1:nInp_scl)  &
        & + f1lumi(-5,1:nInp_scl)*f2lumi(4,1:nInp_scl)
    case ( ibit_qbpd_ )
      ip1_chan = -1
      ip2_chan =  1
      partons(1:nInp_scl) =  &
        &   f1lumi(-1,1:nInp_scl)*f2lumi(1,1:nInp_scl)  &
        & + f1lumi(-2,1:nInp_scl)*f2lumi(1,1:nInp_scl)  &
        & + f1lumi(-3,1:nInp_scl)*f2lumi(1,1:nInp_scl)  &
        & + f1lumi(-4,1:nInp_scl)*f2lumi(1,1:nInp_scl)  &
        & + f1lumi(-5,1:nInp_scl)*f2lumi(1,1:nInp_scl)  &
        & + f1lumi(-1,1:nInp_scl)*f2lumi(3,1:nInp_scl)  &
        & + f1lumi(-2,1:nInp_scl)*f2lumi(3,1:nInp_scl)  &
        & + f1lumi(-3,1:nInp_scl)*f2lumi(3,1:nInp_scl)  &
        & + f1lumi(-4,1:nInp_scl)*f2lumi(3,1:nInp_scl)  &
        & + f1lumi(-5,1:nInp_scl)*f2lumi(3,1:nInp_scl)  &
        & + f1lumi(-1,1:nInp_scl)*f2lumi(5,1:nInp_scl)  &
        & + f1lumi(-2,1:nInp_scl)*f2lumi(5,1:nInp_scl)  &
        & + f1lumi(-3,1:nInp_scl)*f2lumi(5,1:nInp_scl)  &
        & + f1lumi(-4,1:nInp_scl)*f2lumi(5,1:nInp_scl)  &
        & + f1lumi(-5,1:nInp_scl)*f2lumi(5,1:nInp_scl)
    case default
      print*, "sigGGqt getlumi_: invalid ichan:", ichan
      stop
    end select
  end subroutine getlumi_


end module sigGGqt_mod


