#define USE_COMPLEX_MASS_SCHEME


*************************************************
* This subroutine initialises all hard coded parameters
* relevant for electroweak type processes. These parameters
* are then passed to the relevant processes through common
* blocks
*
      subroutine initeweak()
      use Process_mod, only: igamma_proc
      use DIS_mod, only : use_EPRC
      use CKM_mod, only : CKM, fillckm
      implicit real*8(a-h,o-y)
      implicit complex*16(z)
      real*8 nTau(-5:5)
      common /Zmass/ emz,ezwidth
      common /Wmass/ emw,ewwidth
      common /LR/ ele,ere,sin2w,enQ(-5:5),enL(5),enR(5)
#ifdef USE_COMPLEX_MASS_SCHEME
      common /zLR/ zele,zere,zsin2w,zenL(5),zenR(5)
#endif
      common /eweakW/ amzW,stw,ewfac
!$omp threadprivate(/eweakW/)
      common /eweakZ/ amzZ,zewfac(4),zewnull(4)
!$omp threadprivate(/eweakZ/)
      common/plotmode/iplot
      integer ckmmode
      common/CKMmode/ ckmmode


      if (iplot == 0) then
         print*, "initeweak: do not call me in the driver"
         stop
      end if


      !> initialise variables
      zewfac=0d0
      zewnull=0d0
      amzW=0d0
      stw=0d0
      ewfac=0d0

      amz = 7.55638390728735360d-3  ! default MCFM
      !AH amz = 7.56239563669733588d-3  ! G_mu scheme
      !amz = 7.5467711139788835d-3   ! default MadGraph
      !amz = 0.0075624689019847557986d0  ! MATRIX comparison
      if (igamma_proc.eq.1) then
!     alpha_em changed from 1/137.035989 to 1/137.035999084
!     (NIST recommended value)
         aem=1d0/137.035999084d0
         amz = aem
         amw = aem
      endif
      !> use EPRC routines
      if(use_EPRC()) call EPRC_INIT(.false.)

#ifdef USE_OL
      stw = 1d0 - (emw/emz)**2
#else
      stw =0.22264585341299603d0 !MCFM value
#endif
      !AH stw = 0.22289722252391819d0  ! on-shell scheme


      call fillckm(ckmmode)

!        amz = 0.00755590715381540752d0 ! proVBF value
!        stw = 0.22263562357711094532d0 ! proVBF value
      !>----------------------------------------------------------------
      !> Gmu-scheme + Delta-rho corrections + effective alpha



!       amz = 7.77906438417484d-3
!       stw = 0.2292834081112d0
      amz = 7.77906438417484d-3
      stw = 0.2292834081112d0
      !> Gmu-scheme (no universal h.o. corrections)
      !amz = 7.5623956366973385d-3
      !stw = 0.2228972225239183d0
      !> other input parameters
      !>   GF = 1.1663787 10^-5;  (* Fermi constant [GeV^-2] *)
      !>   alpha0 = 7.2973525664 10^-3; (* fine-structure constant *)
      !>   alphas = 0.1182; (* strong coupling constant alphas(MZ) *)
      !>   MW = 80.385; (* on-shell W-boson mass [GeV] *)
      !>   GW = 2.085; (* on-shell W-boson width [GeV] *)
      !>   MZ = 91.1876; (* on-shell Z-boson mass [GeV] *)
      !>   GZ = 2.4952; (* on-shell Z-boson width [GeV] *)
      !>   MH = 125.09; (* on-shell Higgs-boson mass [GeV] *)
      !>   mt = 173.21; (* "Direct Measurements" ~ on-shell/pole mass top-quark mass [GeV] *)
      !>----------------------------------------------------------------
      !> pole masses / widths
      MW = 80.357973609877561216d0
      GW = 2.0842989982782196279d0
      MZ = 91.153480619182758460d0
      GZ = 2.4942663787728243108d0
      !>----------------------------------------------------------------

      !>>>>>> ONLY USED FOR FLAVTAG VALIDATION <<<<<<!
C       amz = 0.00754677d0
C       stw = 0.222249d0
C       MW = 80.4185d0
C       GW = 2.047600d0
C       MZ = 91.1876d0
C       GZ = 2.441404d0
      !>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<!


      amzW = amz
      amzZ = amz ! fortran doesn't like two parameters being the same in different common blocks




      if(iplot.ne.0)then
*     if spike testing, initialise hard coded values. Otherwise take from runcard
         emz = 91.1876d0
         ezwidth = 2.4952d0
         emw = 80.398d0
         ewwidth = 2.1054d0
         ! Madgraph loop
         !emw = 80.419002445756163d0
         !ewwidth = 2.0476000000000001d0
         !ezwidth = 2.4414039999999999d0
         !emz = 91.188000000000002d0
         !stw = 0.22224648578577766d0
      endif

*     ewcharges,iQ(0) is not a valid flavour
      enQ(-5) = +0.333333333333333d0
      enQ(-4) = -0.666666666666667d0
      enQ(-3) = +0.333333333333333d0
      enQ(-2) = -0.666666666666667d0
      enQ(-1) = +0.333333333333333d0
      enQ(0)  = +0d0
      enQ(+1) = -0.333333333333333d0
      enQ(+2) = +0.666666666666667d0
      enQ(+3) = -0.333333333333333d0
      enQ(+4) = +0.666666666666667d0
      enQ(+5) = -0.333333333333333d0

*     A parameter used in the coupling
      data nTau/1d0,-1d0,1d0,-1d0,1d0,0d0,-1d0,1d0,-1d0,1d0,-1d0/
      sin2w = 2d0*sqrt(stw*(1d0-stw))
      do j=1,5
         enL(j) = (ntau(j)-2d0*enQ(j)*stw)/sin2w
         enR(j) = -2d0*enQ(j)*stw/sin2w
      enddo

      ele=(-1d0-2d0*(-1d0)*stw)/sin2w
      ere=(-2d0*(-1d0)*stw)/sin2w



!////////// the complex-mass scheme
#ifdef USE_COMPLEX_MASS_SCHEME

      zMW2 = DCMPLX(emw**2, -ewwidth*emw)
      zMZ2 = DCMPLX(emz**2, -ezwidth*emz)
      zcw2 = zMW2 / zMZ2
      zsw2 = 1d0 - zcw2

      zsin2w = 2d0*sqrt(zsw2*zcw2)

      do j=1,5
         zenL(j) = (ntau(j)-2d0*enQ(j)*zsw2)/zsin2w
         zenR(j) = -2d0*enQ(j)*zsw2/zsin2w
      enddo

      zele=(-1d0-2d0*(-1d0)*zsw2)/zsin2w
      zere=(-2d0*(-1d0)*zsw2)/zsin2w

      !> for W-production
      stw = abs(zsw2)

#endif



      return
      end

**********************************************
* Calculates the relavent electroweak constants for the q qbar -> g + Z/gamma* processes
* nflav is an integer corresponding to the quark flavour, s is the invariant of the two electrons (necessary for off-shell calculations)
*
* NOTE: THIS SUBROUTINE ONLY WORKS WITH ONE QQBAR PAIR. For two q qbar pairs, see the subroutine eweakZ2qq
*

      subroutine eweakV1qq(nflav,s)
      use Process_mod, only : iboson_proc
      implicit real*8(a-h,o-y)
      implicit complex*16(z)
      common /eweakZ/ amz,zewfac(4),zewnull(4)
!$omp threadprivate(/eweakZ/)
      common /eweakW/ amz2,stw,ewfac
!$omp threadprivate(/eweakW/)

      if(iboson_proc.eq.0)then
         call ewcalc(nflav,s,zewfac) ! Z boson coupling
      elseif(abs(iboson_proc).eq.1)then
         call eweakW1qq(0,0,s) ! W boson coupling
         zewfac(1) = dcmplx(ewfac**0.5)
         zewfac(2:4) = 0d0
      else
        write(*,*)"Error in eweakV1qq: iboson_proc parameter not initialised"
        stop
      endif

      zewnull = 0d0 ! a term used in the common block for the 2qqbar pair case

      end




      subroutine eweakV2qq(nflav1,nflav2,s)
      use Process_mod, only : iboson_proc
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /eweakW/ amz2,stw,ewfac
!$omp threadprivate(/eweakW/)
      common /zewfacswap/ swap
!$omp threadprivate(/zewfacswap/)
      logical swap


      if(iboson_proc.eq.0)then
         call ewcalc(nflav1,s,zewfac1) ! Z boson coupling
         call ewcalc(nflav2,s,zewfac2)
      elseif(abs(iboson_proc).eq.1)then
         call eweakW1qq(0,0,s) ! W boson coupling
         zewfac1(1) = ewfac**0.5
         zewfac1(2:4) = 0d0
         zewfac2=0d0
         if (swap) then
         zewfac2(1) = ewfac**0.5
         zewfac2(2:4) = 0d0
         zewfac1=0d0
         endif
      endif


      end

************************************************************
*
*  Calculates a single unsquared electroweak factor
*

      subroutine ewcalc(nflav,s,factor)
      use Process_mod, only: igamma_proc, ilepton_proc
      use DIS_mod, only : Pol
      use Parameters_mod, only : width_V_scheme_param
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      common /Zmass/ emz,ezwidth
      common /nwa/rm12,rg12,inwa
      common /LR/ ele,ere,sin2w,enQ(-5:5),enL(5),enR(5)
#ifdef USE_COMPLEX_MASS_SCHEME
      common /zLR/ zele,zere,zsin2w,zenL(5),zenR(5)
#endif
      common /eweakZ/ amz,zewfac(4),zewnull(4)
!$omp threadprivate(/eweakZ/)
      common /axial/zewax(2)
!$omp threadprivate(/axial/)
      complex*16 factor(4)

*     A test to see if parameters have been initialised
      if ((enR(1).eq.0d0).or.(enL(1).eq.0d0))then
         call initeweak
      endif
      Q  = enQ(nflav)
      eL = enL(nflav)
      eR = enR(nflav)

#ifdef USE_COMPLEX_MASS_SCHEME
      zeL = zenL(nflav)
      zeR = zenR(nflav)
#endif
      qe = -1d0

      if (width_V_scheme_param().eq.-4) then      ! Energy dependent Z width
         !> Eqns 1.6, 1.7 in http://www.sciencedirect.com/science/article/pii/0370269388916279
         zprop = s/dcmplx((s - emz**2), s*ezwidth/emz)
      else
         zprop = s/dcmplx((s - emz**2), ezwidth*emz)
      endif

c ---- restrict to photon exchange for NC
      if ( igamma_proc==1 ) zprop=(0d0,0d0)
c ---- NC: only Z or neutrino decay
      rf = 1d0
      if ( igamma_proc==2 .or. ilepton_proc == 0 ) then
        rf = 0d0
      endif

***  #ifdef USE_COMPLEX_MASS_SCHEME
      if ( width_V_scheme_param() == 4 ) then
*     Left handed quark, left handed electron
      factor(1) = (rf*Q*qe+zeL*zele*zprop)*(1d0-Pol)**0.5
*     Left handed quark, right handed electron
      factor(2) = (rf*Q*qe+zeL*zere*zprop)*(1d0+Pol)**0.5
*     Right handed quark, left handed electron
      factor(3) = (rf*Q*qe+zeR*zele*zprop)*(1d0-Pol)**0.5
*     Right handed quark, right handed electron
      factor(4) = (rf*Q*qe+zeR*zere*zprop)*(1d0+Pol)**0.5

      zewax(1) = zele*zprop/zsin2w
      zewax(2) = zere*zprop/zsin2w
***  #else
      else
*     Left handed quark, left handed electron
      factor(1) = (rf*Q*qe+eL*ele*zprop)*(1d0-Pol)**0.5
*     Left handed quark, right handed electron
      factor(2) = (rf*Q*qe+eL*ere*zprop)*(1d0+Pol)**0.5
*     Right handed quark, left handed electron
      factor(3) = (rf*Q*qe+eR*ele*zprop)*(1d0-Pol)**0.5
*     Right handed quark, right handed electron
      factor(4) = (rf*Q*qe+eR*ere*zprop)*(1d0+Pol)**0.5

      zewax(1) = ele*zprop/sin2w
      zewax(2) = ere*zprop/sin2w
***  #endif
      end if
      end


************************************************************
*
*  Calculates a single unsquared electroweak factor for a vector coupling.
*
*  This occurs where the Z boson couples to a loop instead of the active quark pair
*

      subroutine ewcalcvec(s)
      use Process_mod, only : igamma_proc, ilepton_proc
      use Parameters_mod, only : width_V_scheme_param
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      common /vector/zewvec(2)
!$omp threadprivate(/vector/)
      common /Zmass/ emz,ezwidth
      common /LR/ ele,ere,sin2w,enQ(-5:5),enL(5),enR(5)
#ifdef USE_COMPLEX_MASS_SCHEME
      common /zLR/ zele,zere,zsin2w,zenL(5),zenR(5)
#endif
      common /nwa/rm12,rg12,inwa
      complex*16 factor(2)

      factor = 0d0

*     A test to see if parameters have been initialised
      if ((enR(1).eq.0d0).or.(enL(1).eq.0))then
         call initeweak
      endif

      if (width_V_scheme_param().eq.-4) then      ! Energy dependent Z width
         !> Eqns 1.6, 1.7 in http://www.sciencedirect.com/science/article/pii/0370269388916279
         zprop = s/dcmplx((s - emz**2),s*ezwidth/emz)
      else
         zprop = s/dcmplx((s - emz**2),ezwidth*emz)
      endif

      qe=-1

c ---- restrict to photon exchange for NC
      if ( igamma_proc==1 ) zprop=(0d0,0d0)
c ---- NC: only Z or neutrino decay
      rf = 1d0
      if ( igamma_proc==2 .or. ilepton_proc == 0 ) then
        rf = 0d0
      endif


***  #ifdef USE_COMPLEX_MASS_SCHEME
      if ( width_V_scheme_param() == 4 ) then
      do j=1,5
         factor(1) = factor(1)+rf*enQ(j)*qe+0.5d0*(zenL(j)+zenR(j))*zele*zprop
         factor(2) = factor(2)+rf*enQ(j)*qe+0.5d0*(zenL(j)+zenR(j))*zere*zprop
      enddo
***  #else
      else
      do j=1,5
         factor(1) = factor(1)+rf*enQ(j)*qe+0.5d0*(enL(j)+enR(j))*ele*zprop
         factor(2) = factor(2)+rf*enQ(j)*qe+0.5d0*(enL(j)+enR(j))*ere*zprop
      enddo
***  #endif
      end if

      zewvec = factor

      end
