//
//   @file    gridwrap.cxx
//
//
//   @author M.Sutton
//
//   Copyright (C) 2013 M.Sutton (sutt@cern.ch)
//
//   $Id: gridwrap.cxx, v0.0   Thu 21 2015 15:46:33 CEST sutt $

#include <string>
#include <cstdlib>
#include <vector>

/// function pointer hooks - set to 0 when no functions defined and applgrid not linked
void (*appl_initptr)( const int& igrid, const int& id, const std::string& gridname,
                      const int& nbins, const double* bins  ) = 0;

void (*appl_fillptr)( const int& igrid, const int& id, 
                      const double& obs, 
                      const double* wt, 
                      const double* wtref,
                      const double* coeff ) = 0;

void (*appl_fillrefptr)( const int& igrid, const int& id,
                         const double& obs,
                         const double* wt,
                         const double* coeff ) = 0;

void (*appl_termptr)( const int& igrid, const int& id, const double& norm ) = 0;



/// book grids
extern "C" void  book_grid_( const int& igrid, const int& id,
                             const int& nbins, const double& lo, const double& hi,
                             const char* s, size_t len ) {
   if ( appl_initptr ) {
      std::string _s( s, 0, len );
      std::vector<double> bins;; bins.reserve(nbins+1);
      for ( int i=0 ; i<=nbins ; i++ ) bins.push_back( (lo*(nbins-i) + hi*i)/nbins );
      appl_initptr( igrid, id, _s, nbins, &bins[0] );
   }
}

/// book grids
extern "C" void  book_grid_varbins_( const int& igrid, const int& id,
                                     const int& nbins, const double* bins,
                                     const char* s, size_t len ) {
   if ( appl_initptr ) {
      std::string _s( s, 0, len );
      appl_initptr( igrid, id, _s, nbins, bins );
   }
}


extern "C" void  book_grid__( const int& igrid, const int& id,
                              const int& nbins, const double& lo, const double& hi,
                              const char* s, size_t len  ) {
   book_grid_( igrid, id, nbins, lo, hi, s, len );
}

extern "C" void  book_grid_varbins__( const int& igrid, const int& id,
                                      const int& nbins, const double* bins,
                                      const char* s, size_t len  ) {
   book_grid_varbins_( igrid, id, nbins, bins, s, len );
}


/// fill grids
extern "C" void  fill_grid_( const int& igrid, const int& id, 
                             const double& obs, const double* wt, 
                             const double* wtref,
                             const double* coeff ) {
   if ( appl_fillptr ) appl_fillptr( igrid, id, obs, wt, wtref, coeff );
}

extern "C" void  fill_grid__( const int& igrid, const int& id, 
                              const double& obs, const double* wt,
                              const double* wtref,
                              const double* coeff ) {
   fill_grid_( igrid, id, obs, wt, wtref, coeff );
}


/// write grids
extern "C" void  write_grid_( const int& igrid, const int& id, const double& norm ) { 
  if ( appl_termptr ) appl_termptr( igrid, id, norm );
}

extern "C" void  write_grid__( const int& igrid, const int& id, const double& norm ) { 
  write_grid_( igrid, id, norm ); 
}



int setupConfigPath() {
  return setenv( "CONFIGDIR", (std::string(CONFIGDIR)+"/process").c_str(), 1 );
}

static int idummy = setupConfigPath();
