!-------------------------------------------------------------------------------
! NNLOJET
!-------------------------------------------------------------------------------
!
! MODULE: CoeffFunc_av [external members: `_av`]
!
! DESCRIPTION:
!> 2-loop and 3-loop coefficient functions by A. Vogt. 
!> note: mu_r = mu_f = Q
!>       only photon exchange
!>
!> patch xc*.f files with module wrappers using:
!> for i in *.f; do modname=$(sed 's/.f//g' <<< $i); gsed -i -e "1i\      MODULE ${modname}\\n      CONTAINS" -e "\$a\\      END MODULE ${modname}" $i; done
!>
!> only for validation, didn't care about performance *at all*
!-------------------------------------------------------------------------------
module CoeffFunc_av
  public
  
  integer, parameter :: nf_int = 5


contains
  

  function c2q_av(iorder, x, iregion, qexact, flavchange) result(res)
    use xc2ns2p
    use xc2ns3p
    use xc2ns3e
    double precision, intent(in)  :: x
    integer, intent(in)           :: iorder, iregion, flavchange
    double precision              :: res
    logical, intent(in), optional :: qexact
    logical                       :: qex
    double precision              :: CA, CB, CC
    
    res = 0d0

    qex = .false.
    if ( present(qexact) ) qex = qexact

    !> NNLO
    if (iorder == 2) then
      select case(iregion)
      case(0)
         res = C2NN2C(x, nf_int)
      case(1)
         res = C2NN2A(x, nf_int) + C2NS2B(x, nf_int)
      case(2)
         res = - C2NS2B(x, nf_int)
      end select

    !> N3LO
    else if (iorder == 3) then

      !> exact
      if (qex) then
        !> need to call in order (A,B,C) for it to work
        CA = X2NP3A(x, nf_int, flavchange)
        CB = X2NS3B(x, nf_int)
        CC = X2NP3C(x, nf_int, flavchange)
        select case(iregion)
        case(0)
           res = CC
        case(1)
           res = CA + CB
        case(2)
           res = - CB
        end select

      !> parametrisation
      else
        select case(iregion)
        case(0)
           res = C2NP3C(x, nf_int, flavchange)
        case(1)
           res = C2NP3A(x, nf_int, flavchange) + C2NS3B(x, nf_int)
        case(2)
           res = - C2NS3B(x, nf_int)
        end select
      end if

    else
      stop "CoeffFunc only has NNLO and N3LO implemented!"
    end if

  end function c2q_av


  function c2qps_av(iorder, x, iregion, qexact, flavchange) result(res)
    use xc2sg2p
    use xc2sg3p
    use xc2ps3e
    double precision, intent(in)  :: x
    integer, intent(in)           :: iorder, iregion, flavchange
    double precision              :: res
    logical, intent(in), optional :: qexact
    logical                       :: qex
    double precision              :: CA, CB, CC
    
    res = 0d0

    qex = .false.
    if ( present(qexact) ) qex = qexact

    !> NNLO
    if (iorder == 2) then
      select case(iregion)
      case(0)
         res = 0d0
      case(1)
         res = C2S2A(x, nf_int)
      case(2)
         res = 0d0
      end select

    !> N3LO
    else if (iorder == 3) then
      !> exact
      if (qex) then
        !> need to call in order (A,B,C) for it to work
        CA = x2s3a(x, nf_int, flavchange)
        CB = 0
        CC = X2S3C(x, nf_int, flavchange)
        select case(iregion)
        case(0)
           res = CC
        case(1)
           res = CA + CB
        case(2)
           res = - CB
        end select

      !> parametrisation
      else       
         select case(iregion)
         case(0)
            res = C2S3C(x, nf_int, flavchange)
         case(1)
            res = C2S3A(x, nf_int)
         case(2)
            res = 0d0
         end select
      endif
    else
      stop "CoeffFunc only has NNLO and N3LO implemented!"
    end if

    res = res / dble(nf_int)  ! nf factor in the driver
  end function c2qps_av


  function c2g_av(iorder, x, iregion, qexact, flavchange) result(res)
    use xc2sg2p
    use xc2sg3p
    use xc2gl3e
    double precision, intent(in)  :: x
    integer, intent(in)           :: iorder, iregion, flavchange
    double precision              :: res
    logical, intent(in), optional :: qexact
    logical                       :: qex
    double precision              :: CA, CB, CC
    
    res = 0d0

    qex = .false.
    if ( present(qexact) ) qex = qexact

    !> NNLO
    if (iorder == 2) then
      select case(iregion)
      case(0)
         res = C2G2C(x, nf_int)
      case(1)
         res = C2G2A(x, nf_int)
      case(2)
         res = 0d0
      end select

    !> N3LO
    else if (iorder == 3) then
      !> exact
      if (qex) then
        !> need to call in order (A,B,C) for it to work
        CA = X2G3A(x, nf_int, flavchange)
        CB = 0
        CC = 0
        select case(iregion)
        case(0)
           res = CC
        case(1)
           res = CA + CB
        case(2)
           res = - CB
        end select
     else
        select case(iregion)
        case(0)
           res = C2G3C(x, nf_int)
        case(1)
           res = C2G3A(x, nf_int)
        case(2)
           res = 0d0
        end select
     endif

    else
     stop "CoeffFunc only has NNLO and N3LO implemented!"
    end if

    res = res / dble(nf_int)  ! nf factor in the driver
  end function c2g_av


  function clq_av(iorder, x, iregion, qexact, flavchange) result(res)
    use xclns2p
    use xclns3p
    use xclns3e
    double precision, intent(in)  :: x
    integer, intent(in)           :: iorder, iregion, flavchange
    double precision              :: res
    logical, intent(in), optional :: qexact
    logical                       :: qex
    double precision              :: CA, CB, CC
    
    res = 0d0

    qex = .false.
    if ( present(qexact) ) qex = qexact

    !> NNLO
    if (iorder == 2) then
      select case(iregion)
      case(0)
         res = CLNN2C(x)
      case(1)
         res = CLNN2A(x, nf_int)
      case(2)
         res = 0d0
      end select

    !> N3LO
    else if (iorder == 3) then
      !> exact
      if (qex) then
        !> need to call in order (A,B,C) for it to work
        CA = XLNP3A(x, nf_int, flavchange)
        CB = 0
        CC = 0
        select case(iregion)
        case(0)
           res = CC
        case(1)
           res = CA + CB
        case(2)
           res = - CB
        end select
     else
        select case(iregion)
        case(0)
           res = CLNP3C(x, nf_int)
        case(1)
           res = CLNP3A(x, nf_int, flavchange)
        case(2)
           res = 0d0
        end select
     endif
    else
      stop "CoeffFunc only has NNLO and N3LO implemented!"
    end if

  end function clq_av


  function clqps_av(iorder, x, iregion, qexact, flavchange) result(res)
    use xclsg2p
    use xclsg3p
    use xclps3e
    double precision, intent(in)  :: x
    integer, intent(in)           :: iorder, iregion, flavchange
    double precision              :: res
    logical, intent(in), optional :: qexact
    logical                       :: qex
    double precision              :: CA, CB, CC
    
    res = 0d0

    qex = .false.
    if ( present(qexact) ) qex = qexact

    !> NNLO
    if (iorder == 2) then
      select case(iregion)
      case(0)
         res = 0d0
      case(1)
         res = CLS2A(x, nf_int)
      case(2)
         res = 0d0
      end select

    !> N3LO
    else if (iorder == 3) then
       !> exact
       if (qex) then
        !> need to call in order (A,B,C) for it to work
          CA = XLS3A(x, nf_int, flavchange)
          CB = 0
          CC = 0
          select case(iregion)
          case(0)
             res = CC
          case(1)
             res = CA + CB
          case(2)
             res = - CB
          end select
       else
          select case(iregion)
          case(0)
             res = 0d0
          case(1)
             res = CLS3A(x, nf_int)
          case(2)
             res = 0d0
          end select
       endif
    else
      stop "CoeffFunc only has NNLO and N3LO implemented!"
    end if

    res = res / dble(nf_int)  ! nf factor in the driver
  end function clqps_av


  function clg_av(iorder, x, iregion, qexact, flavchange) result(res)
    use xclsg2p
    use xclsg3p
    use xclgl3e
    double precision, intent(in)  :: x
    integer, intent(in)           :: iorder, iregion, flavchange
    double precision              :: res
    logical, intent(in), optional :: qexact
    logical                       :: qex
    double precision              :: CA, CB, CC
    
    res = 0d0

    qex = .false.
    if ( present(qexact) ) qex = qexact

    !> NNLO
    if (iorder == 2) then
      select case(iregion)
      case(0)
         res = 0d0
      case(1)
         res = CLG2A(x, nf_int)
      case(2)
         res = 0d0
      end select

    !> N3LO
    else if (iorder == 3) then
       !> exact
       if (qex) then
        !> need to call in order (A,B,C) for it to work
          CA = XLG3A(x, nf_int, flavchange)
          CB = 0
          CC = 0
          select case(iregion)
          case(0)
             res = CC
          case(1)
             res = CA + CB
          case(2)
             res = - CB
          end select
       else
          select case(iregion)
          case(0)
             res = 0d0
          case(1)
             res = CLG3A(x, nf_int)
          case(2)
             res = 0d0
          end select
       endif
    else
      stop "CoeffFunc only has NNLO and N3LO implemented!"
    end if

    res = res / dble(nf_int)  ! nf factor in the driver
  end function clg_av

!> c3q
  function c3q_plus_av(iorder, x, iregion, qexact) result(res)
    use xc3ns2e
    use xc3ns3e
    use xcdf2e
    use xcdf3p
    double precision, intent(in)  :: x
    integer, intent(in)           :: iorder, iregion
    double precision              :: res
    logical, intent(in), optional :: qexact
    logical                       :: qex
    double precision              :: CA, CB, CC, CDIFF
    
    res = 0d0

    qex = .false.
    if ( present(qexact) ) qex = qexact

    !> NNLO
    if (iorder == 2) then !> Exact only implemented
       CA = X3NM2A(x, nf_int)
       CB = X3NS2B(x, nf_int)
       CC = X3NS2C(x, nf_int)
       CDIFF = XC3DFF2(x)
       
      select case(iregion)
      case(0)
         res = CC
      case(1)
         res = CA + CB + CDIFF
      case(2)
         res = -CB
      end select

      res = res 

    !> N3LO
    elseif(iorder == 3) then
       CA = X3NM3A(x, nf_int, 0)
       CB = X3NS3B(x, nf_int)
       CC = X3NS3C(x, nf_int)
       CDIFF = C3Q3DF(x, nf_int, 0)

      select case(iregion)
      case(0)
         res = CC
      case(1)
         res = CA + CB + CDIFF
      case(2)
         res = - CB
      end select

      res = res
      
    end if
    
  end function c3q_plus_av
  
  function c3q_minus_av(iorder, x, iregion, qexact) result(res)
    use xc3ns2e
    use xc3ns3e
    double precision, intent(in)  :: x
    integer, intent(in)           :: iorder, iregion
    double precision              :: res
    logical, intent(in), optional :: qexact
    logical                       :: qex
    double precision              :: CA, CB, CC
    
    res = 0d0

    qex = .false.
    if ( present(qexact) ) qex = qexact

    !> NNLO
    if (iorder == 2) then !> Exact only implemented
       CA = X3NM2A(x, nf_int)
       CB = X3NS2B(x, nf_int)
       CC = X3NS2C(x, nf_int)

      select case(iregion)
      case(0)
         res = CC
      case(1)
         res = CA +CB 
      case(2)
         res = -CB
      end select

      res = res 

    !> N3LO
    elseif(iorder == 3) then
       CA = X3NM3A(x, nf_int, 0)
       CB = X3NS3B(x, nf_int)
       CC = X3NS3C(x, nf_int)

      select case(iregion)
      case(0)
         res = CC
      case(1)
         res = CA +CB 
      case(2)
         res = -CB
      end select

      res = res 
    end if

  end function c3q_minus_av

  
  function c3q_valence_av(iorder, x, iregion, qexact) result(res)
    use xc3ns2e
    use xc3ns3e
    double precision, intent(in)  :: x
    integer, intent(in)           :: iorder, iregion
    double precision              :: res
    logical, intent(in), optional :: qexact
    logical                       :: qex
    double precision              :: CA, CB, CC
    
    res = 0d0

    qex = .false.
    if ( present(qexact) ) qex = qexact

    !> NNLO
    if (iorder == 2) then !> Exact only implemented
       CA = X3NM2A(x, nf_int)
       CB = X3NS2B(x, nf_int)
       CC = X3NS2C(x, nf_int)

      select case(iregion)
      case(0)
         res = CC
      case(1)
         res = CA + CB 
      case(2)
         res = -CB
      end select

      res = res 

    !> N3LO
    elseif(iorder == 3) then
       CA = X3NM3A(x, nf_int, 1)
       CB = X3NS3B(x, nf_int)
       CC = X3NS3C(x, nf_int)

      select case(iregion)
      case(0)
         res = CC
      case(1)
         res = CA +CB 
      case(2)
         res = -CB
      end select

      res = res 
    end if
  
  end function c3q_valence_av

  
 
end module CoeffFunc_av

