c--- libGJRB   --------------------------------------------------------------
c
c     This library contains all the real emission B-type MEs for gamma+jet.
c     The MEs are wrapped from jet libB.f. Note that compared to the jets we
c     have reversed the quark line. Then we also have to swap the gluon
c     ordering!
c
c----------------------------------------------------------------------------

      function fullB2g0G(iqb,ig1,ig2,iq,iG)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16(z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /eweakZ/ amz,zewfac(4),zewnull(4)
!$omp threadprivate(/eweakZ/)

      if(iqb.le.2.and.iq.le.2) then
         ave = 1/4d0/nc**2
      elseif(iqb.gt.2.and.iq.gt.2) then
         ave = 1/4d0/(nc**2-1d0)**2
      else   
         ave = 1/4d0/nc/(nc**2-1d0)
      endif   

      fac = ave*(4d0*pi*as)**2*(4d0*pi*amz)*4d0*nc*(nc**2-1d0)

      FullB2g0G = ( B2g0G(iqb,ig1,ig2,iq,iG)
     .             +B2g0G(iqb,ig2,ig1,iq,iG))
     .     -1d0/dble(nc)**2*(
     .              Bt2g0G(iqb,ig1,ig2,iq,iG))
      
      FullB2g0G = fac*FullB2g0G
      
      return
      end

*************************************************************

      function B2g0G(iqb,ig1,ig2,iq,iG)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16(z)
      common /eweakZ/ amz,zewfac(4),zenwull(4)
!$omp threadprivate(/eweakZ/)
      common /BZFlav/nflav
!$omp threadprivate(/BZFlav/)

      call eweakV1qq(nflav,0d0)
      B2g0G=abs(zewfac(1))**2*Bt3g0(iq,iG,ig2,ig1,iqb)/2d0

      return
      end

      
************************************************************************
* A wrapper function for B2g0G which inherits the flavour of the first
* pair of quarks from the C0g0G common block.
*
* This must be called in the maple code for 4 quark processes where the
* reduced matrix element has the flavour of the first pair of quarks


      function f1B2g0G(i1,i2,i3,i4,i5)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /CZFlav/nf1,nf2
!$omp threadprivate(/CZFlav/)
      common /BZFlav/nf1g
!$omp threadprivate(/BZFlav/)

      if(nf1.ne.0)then
         nf1g = nf1
         f1B2g0G = B2g0G(i1,i2,i3,i4,i5)
      else
         print *, "Error in f1B2g0G: flavours have not been initialised"
         stop
      endif

      return
      end

************************************************************************
* A wrapper function for B2g0G which inherits the flavour of the second
* pair of quarks from the C0g0G common block.
*
* This must be called in the maple code for 4 quark processes where the
* reduced matrix element has the flavour of the second pair of quarks
*

      function f2B2g0G(i1,i2,i3,i4,i5)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /CZFlav/nf1,nf2
!$omp threadprivate(/CZFlav/)
      common /BZFlav/nf1g
!$omp threadprivate(/BZFlav/)

      if(nf2.ne.0)then
         nf1g = nf2
         f2B2g0G = B2g0G(i1,i2,i3,i4,i5)
      else
         print *, "Error in f2B2g0G: flavours have not been initialised"
         stop
      endif

      return
      end

************************************************************************
* Symmetrised B2g0G across the qqb pair.
*
*
      function sB2g0G(i1,i2,i3,i4,i5)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      sB2g0G = 0.5d0*(B2g0G(i1,i2,i3,i4,i5)
     .               +B2g0G(i4,i3,i2,i1,i5))

      return
      end

*******************************************************************
* Symmetrised B2g0G across the qqb pair, taking the flavour from the
* second quark pair of the C-type matrix elements.
*

      function sf2B2g0G(i1,i2,i3,i4,i5)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      sf2B2g0G = 0.5d0*(f2B2g0G(i1,i2,i3,i4,i5)
     .                 +f2B2g0G(i4,i3,i2,i1,i5))

      return
      end

***************************************************************************

      function Bt2g0G(iqb,ig1,ig2,iq,iG)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16(z)
      common /eweakZ/ amz,zewfac(4),zenwull(4)
!$omp threadprivate(/eweakZ/)
      common /BZFlav/nflav
!$omp threadprivate(/BZFlav/)

      call eweakV1qq(nflav,0d0)

      Bt2g0G=abs(zewfac(1))**2*Btt3g0(iq,iG,ig2,ig1,iqb)/2d0

      return
      end

      
************************************************************************
* A wrapper function for Bt2g0G which inherits the flavour of the first
* pair of quarks from the C0g0G common block.
*
* This must be called in the maple code for 4 quark processes where the
* reduced matrix element has the flavour of the first pair of quarks


      function f1Bt2g0G(i1,i2,i3,i4,i5)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /CZFlav/nf1,nf2
!$omp threadprivate(/CZFlav/)
      common /BZFlav/nf1g
!$omp threadprivate(/BZFlav/)

      if(nf1.ne.0)then
         nf1g = nf1
         f1Bt2g0G = Bt2g0G(i1,i2,i3,i4,i5)
      else
         print *, "Error in f1Bt2g0G: flavours haven't been initialised"
         stop
      endif

      return
      end

************************************************************************
* A wrapper function for Bt2g0G which inherits the flavour of the second
* pair of quarks from the C0g0G common block.
*
* This must be called in the maple code for 4 quark processes where the
* reduced matrix element has the flavour of the second pair of quarks
*

      function f2Bt2g0G(i1,i2,i3,i4,i5)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /CZFlav/nf1,nf2
!$omp threadprivate(/CZFlav/)
      common /BZFlav/nf1g
!$omp threadprivate(/BZFlav/)

      if(nf2.ne.0)then
         nf1g = nf2
         f2Bt2g0G = Bt2g0G(i1,i2,i3,i4,i5)
      else
         print *, "Error in f2Bt2g0G: flavours haven't been initialised"
         stop
      endif

      return
      end

*******************************************************************
* Symmetrised Bt2g0G across the qqb pair, taking the flavour from
* the first quark pair of the C-type matrix elements.
*
*

      function sf1Bt2g0G(i1,i2,i3,i4,i5)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      sf1Bt2g0G = 0.5d0*(f1Bt2g0G(i1,i2,i3,i4,i5)
     .                 +f1Bt2g0G(i4,i3,i2,i1,i5))

      return
      end

*******************************************************************
* Symmetrised Bt2g0G across the qqb pair, taking the flavour from the
* second quark pair of the C-type matrix elements.
*

      function sf2Bt2g0G(i1,i2,i3,i4,i5)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      sf2Bt2g0G = 0.5d0*(f2Bt2g0G(i1,i2,i3,i4,i5)
     .                  +f2Bt2g0G(i4,i3,i2,i1,i5))

      return
      end


*******************************************************************
* Symmetrised Bt2g0G across the qqb pair.
*
*
      function sBt2g0G(i1,i2,i3,i4,i5)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      sBt2g0G = 0.5d0*(Bt2g0G(i1,i2,i3,i4,i5)
     .                +Bt2g0G(i4,i3,i2,i1,i5))

      return
      end

***************************************************************************


