c--- libGJRRC  --------------------------------------------------------------
c
c     This library contains all the double-real C-type MEs for gamma+jet.
c
c     Amplitudes taken from:
c      
c     V. Del Duca, W. B. Kilgore, F. Maltoni
c     Multiphoton amplitudes for next-to-leading order QCD
c     Nucl. Phys. B566 (2000) 252-274
c     Nucl. Phys. B574 (2000) 851-873
c     DOI:10.1016/S0550-3213(99)90663-6
c     [hep-ph/9910253]
c      
c----------------------------------------------------------------------------

      function fullC1g0G(iqb1,ig1,iq2,iqb2,iq1,iG)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16(z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      
      if(ig1.gt.2) then
         ave = 1/4d0/nc**2
      else
         ave = 1/4d0/nc/(nc**2-1d0)
      endif   

      fac = ave*(4d0*pi*as)**3*(4d0*pi*amz)*4d0*nc**2*(nc**2-1d0)

      FullC1g0G = 1d0/dble(nc)*C1g0G(iqb1,ig1,iq2,iqb2,iq1,iG)
     .     +1d0/dble(nc)**3*(
     .            Ct1g0G(iqb1,ig1,iq2,iqb2,iq1,iG))

      FullC1g0G = fac*FullC1g0G
      
      return
      end

*************************************************************

      function C1g0G(iqb1,ig1,iq2,iqb2,iq1,iG)
      use KinData_mod
      implicit real*8 (a-h,o-y)
      implicit complex*16(z)

      wt=0d0

      do ihel=1,8
      do iflip=0,1
            
         ztreea = zC1g0Ga(iqb1,iq1,iqb2,iq2,ig1,iG,ihel,iflip) ! (1,0)
         ztreeb = zC1g0Gb(iqb1,iq1,iqb2,iq2,ig1,iG,ihel,iflip) ! (0,1)
         
         wt = wt + real(ztreea*dconjg(ztreea))
     .        + real(ztreeb*dconjg(ztreeb))
         
      enddo
      enddo

      C1g0G = wt/2d0

      return
      end

**********************************************************************
* A wrapper function for the C1g0G matrix element symmetrised over the
* second quark pair. This is required for certain subtraction terms
* (e.g. qg initiated nf piece).
*
* qg configuration => symmetrised
* qbg configuration => symmetrised
* anything else => not symmetrised

      function sC1g0G(i1,i2,i3,i4,i5,i6)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common/nonpisflag/nonpis
      
      select case(nonpis)
      case(0)
         if((i2.le.2).and.((i1.le.2).or.(i5.le.2)))then
            sC1g0G = 0.5d0*(C1g0G(i1,i2,i3,i4,i5,i6)
     .           +C1g0G(i1,i2,i4,i3,i5,i6))
         else
            sC1g0G = C1g0G(i1,i2,i3,i4,i5,i6)
         endif
      case(1)
         if(((i1.eq.1).or.(i5.eq.1)))then
            sC1g0G = 0.5d0*(C1g0G(i1,i2,i3,i4,i5,i6)
     .           +C1g0G(i1,i2,i4,i3,i5,i6))
         else
            sC1g0G = C1g0G(i1,i2,i3,i4,i5,i6)
         endif
      case default
         sC1g0G = C1g0G(i1,i2,i3,i4,i5,i6)
      end select

      return
      end
      
*******************************************************************************
 
      function Ct1g0G(iqb1,ig1,iq2,iqb2,iq1,iG)
      use KinData_mod
      implicit real*8 (a-h,o-y)
      implicit complex*16(z)

      wt=0d0

      do ihel=1,8
      do iflip=0,1

         ztreeat = zCt1g0Ga(iqb1,iq1,iqb2,iq2,ig1,iG,ihel,iflip) ! (1,0)
         ztreebt = zCt1g0Gb(iqb1,iq1,iqb2,iq2,ig1,iG,ihel,iflip) ! (0,1)
         zintabt = ztreeat +ztreebt

         wt = wt + real(ztreeat*dconjg(ztreeat))
     .        + real(ztreebt*dconjg(ztreebt))
     .        - 2d0*real(zintabt*dconjg(zintabt))
      enddo
      enddo

      Ct1g0G = wt/2d0

      return
      end

**********************************************************************
* A wrapper function for the Ct1g0G matrix element symmetrised over the
* second quark pair. This is required for certain subtraction terms
* (e.g. qg initiated nf/nc**2 piece)
*
* qg configuration => symmetrised
* qbg configuration => symmetrised
* anything else => not symmetrised

      function sCt1g0G(i1,i2,i3,i4,i5,i6)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common/nonpisflag/nonpis
      select case(nonpis)
      case(0)
         if((i2.le.2).and.((i1.le.2).or.(i5.le.2)))then
            sCt1g0G = 0.5d0*(Ct1g0G(i1,i2,i3,i4,i5,i6)
     .           +Ct1g0G(i1,i2,i4,i3,i5,i6))
         else
            sCt1g0G = Ct1g0G(i1,i2,i3,i4,i5,i6)
         endif
      case(1)
         if((i1.eq.1).or.(i5.eq.1))then
            sCt1g0G = 0.5d0*(Ct1g0G(i1,i2,i3,i4,i5,i6)
     .           +Ct1g0G(i1,i2,i4,i3,i5,i6))
         else
            sCt1g0G = Ct1g0G(i1,i2,i3,i4,i5,i6)
         endif
      case(2)
            sCt1g0G = Ct1g0G(i1,i2,i3,i4,i5,i6)
      end select

      return
      end

********************************************************************************
*
*     leading colour amplitude for 2q2Q1g1G, (k,l)=(1,0) ordering , see hep-ph/9910253
*     (qb,q,Qb,Q,g,G)
      
      function zC1g0Ga(i1,i2,i3,i4,i5,i6,ihel,iflip)
      use KinData_mod

      implicit real*8 (a-h,o-y)
      implicit complex*16(z)

      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)

! MHV

      call eweakV2qq(nf1,nf2,0d0)

      if(ihel.le.4)then

         zi = (0d0,1d0)

         if(iflip.eq.0)then

            za12 = zA6(i1,i2)
            za13 = zA6(i1,i3)
            za14 = zA6(i1,i4)
            za21 = zA6(i2,i1)
            za23 = zA6(i2,i3)
            za24 = zA6(i2,i4)
            za25 = zA6(i2,i5)
            za26 = zA6(i2,i6)
            za34 = zA6(i3,i4)
            za43 = zA6(i4,i3)
            za46 = zA6(i4,i6)
            za53 = zA6(i5,i3)
            za61 = zA6(i6,i1)
            za63 = zA6(i6,i3)

         elseif(iflip.eq.1)then

            za12 = -zB6(i1,i2)
            za13 = -zB6(i1,i3)
            za14 = -zB6(i1,i4)
            za21 = -zB6(i2,i1)
            za23 = -zB6(i2,i3)
            za24 = -zB6(i2,i4)
            za25 = -zB6(i2,i5)
            za26 = -zB6(i2,i6)
            za34 = -zB6(i3,i4)
            za43 = -zB6(i4,i3)
            za46 = -zB6(i4,i6)
            za53 = -zB6(i5,i3)
            za61 = -zB6(i6,i1)
            za63 = -zB6(i6,i3)

         endif

         zpf = real(zewfac1(1))*za21/za26/za61
     .       + real(zewfac2(1))*za43/za46/za63

         zfa = zi*za23/za12/za34/za25/za53

         if(ihel.eq.1)then
            zwt = -zpf*za13**2*zfa
         endif

         if(ihel.eq.2)then
            zwt = zpf*za14**2*zfa
         endif

         if(ihel.eq.3)then
            zwt = zpf*za23**2*zfa
         endif

         if(ihel.eq.4)then
            zwt = -zpf*za24**2*zfa
         endif

      endif
      
! N-MHV (For N-MHV A01 and A10 are interchanged, in contrast to how it's presented in the paper.)
! For the C-terms this doesn't matter, as only squares of A01 / A10 or the sum appears. But for D it makes a difference.
      
      if(ihel.eq.5)then
         zwt = real(zewfac1(1))*zC1g0G1(i1,i2,i3,i4,i5,i6,iflip)
     .       + real(zewfac2(1))*zC1g0G2(i3,i4,i1,i2,i5,i6,iflip)
      endif

      if(ihel.eq.6)then
         zwt = real(zewfac1(1))*zC1g0G3(i1,i2,i3,i4,i5,i6,iflip)
     .       + real(zewfac2(1))*zC1g0G3(i4,i3,i2,i1,i5,i6,iflip)
      endif

      if(ihel.eq.7)then
         zwt = real(zewfac1(1))*zC1g0G2(i2,i1,i4,i3,i5,i6,iflip)
     .       + real(zewfac2(1))*zC1g0G1(i4,i3,i2,i1,i5,i6,iflip)
      endif

      if(ihel.eq.8)then
         zwt = real(zewfac1(1))*zC1g0G4(i2,i1,i4,i3,i5,i6,iflip)
     .       + real(zewfac2(1))*zC1g0G4(i3,i4,i1,i2,i5,i6,iflip)
      endif
      
      if(ihel.gt.8)then
         zwt = (0d0,0d0)
      endif

      zC1g0Ga = zwt

      return
      end

********************************************************************************
*
*     leading colour amplitude for 2q2Q1g1G, (k,l)=(0,1) ordering , see hep-ph/9910253
*     (qb,q,Qb,Q,g,G)

      function zC1g0Gb(i1,i2,i3,i4,i5,i6,ihel,iflip)
      use KinData_mod

      implicit real*8 (a-h,o-y)
      implicit complex*16(z)

      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)

! MHV

      call eweakV2qq(nf1,nf2,0d0)

      if(ihel.le.4)then

         zi = (0d0,1d0)

         if(iflip.eq.0)then

            za12 = zA6(i1,i2)
            za13 = zA6(i1,i3)
            za14 = zA6(i1,i4)
            za21 = zA6(i2,i1)
            za23 = zA6(i2,i3)
            za24 = zA6(i2,i4)
            za26 = zA6(i2,i6)
            za34 = zA6(i3,i4)
            za43 = zA6(i4,i3)
            za46 = zA6(i4,i6)
            za61 = zA6(i6,i1)
            za63 = zA6(i6,i3)

            za41 = zA6(i4,i1)   !
            za45 = zA6(i4,i5)   ! Need these for (0,1)
            za51 = zA6(i5,i1)   !

         elseif(iflip.eq.1)then

            za12 = -zB6(i1,i2)
            za13 = -zB6(i1,i3)
            za14 = -zB6(i1,i4)
            za21 = -zB6(i2,i1)
            za23 = -zB6(i2,i3)
            za24 = -zB6(i2,i4)
            za26 = -zB6(i2,i6)
            za34 = -zB6(i3,i4)
            za43 = -zB6(i4,i3)
            za46 = -zB6(i4,i6)
            za61 = -zB6(i6,i1)
            za63 = -zB6(i6,i3)

            za41 = -zB6(i4,i1)   !
            za45 = -zB6(i4,i5)   ! Need these for (0,1)
            za51 = -zB6(i5,i1)   !

         endif

         zpf = real(zewfac1(1))*za21/za26/za61
     .       + real(zewfac2(1))*za43/za46/za63

         zfa = zi*za41/za12/za34/za45/za51 ! for MHV the only difference between (1,0) and (0,1) is in this line...

         if(ihel.eq.1)then
            zwt = -zpf*za13**2*zfa
         endif

         if(ihel.eq.2)then
            zwt = zpf*za14**2*zfa
         endif

         if(ihel.eq.3)then
            zwt = zpf*za23**2*zfa
         endif

         if(ihel.eq.4)then
            zwt = -zpf*za24**2*zfa
         endif

      endif

! N-MHV (For N-MHV A01 and A10 are interchanged, in contrast to how it's presented in the paper.)
! For the C-terms this doesn't matter, as only squares of A01 / A10 or the sum appears. But for D it makes a difference.
      
      if(ihel.eq.5)then
         zwt = real(zewfac1(1))*zC1g0G2(i1,i2,i3,i4,i5,i6,iflip)
     .       + real(zewfac2(1))*zC1g0G1(i3,i4,i1,i2,i5,i6,iflip)
      endif

      if(ihel.eq.6)then
         zwt = real(zewfac1(1))*zC1g0G4(i1,i2,i3,i4,i5,i6,iflip)
     .       + real(zewfac2(1))*zC1g0G4(i4,i3,i2,i1,i5,i6,iflip)
      endif

      if(ihel.eq.7)then
         zwt = real(zewfac1(1))*zC1g0G1(i2,i1,i4,i3,i5,i6,iflip)
     .       + real(zewfac2(1))*zC1g0G2(i4,i3,i2,i1,i5,i6,iflip)
      endif

      if(ihel.eq.8)then
         zwt = real(zewfac1(1))*zC1g0G3(i2,i1,i4,i3,i5,i6,iflip)
     .       + real(zewfac2(1))*zC1g0G3(i3,i4,i1,i2,i5,i6,iflip)
      endif

      if(ihel.gt.8)then
         zwt = (0d0,0d0)
      endif

      zC1g0Gb = zwt

      return
      end

*******************************************************************************
*
*     subleading colour amplitude for 2q2Q1g1G, (k,l)=(1,0) ordering , see hep-ph/9910253
*     (qb,q,Qb,Q,g,G)

      function zCt1g0Ga(i1,i2,i3,i4,i5,i6,ihel,iflip)
      use KinData_mod

      implicit real*8 (a-h,o-y)
      implicit complex*16(z)

      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)

! MHV

      call eweakV2qq(nf1,nf2,0d0)

      if(ihel.le.4)then

         zi = (0d0,1d0)

         if(iflip.eq.0)then

            za12 = zA6(i1,i2)
            za13 = zA6(i1,i3)
            za14 = zA6(i1,i4)
            za21 = zA6(i2,i1)
            za23 = zA6(i2,i3)
            za24 = zA6(i2,i4)
            za25 = zA6(i2,i5)
            za26 = zA6(i2,i6)
            za34 = zA6(i3,i4)
            za43 = zA6(i4,i3)
            za46 = zA6(i4,i6)
            za61 = zA6(i6,i1)
            za63 = zA6(i6,i3)

            za51 = zA6(i5,i1)

         elseif(iflip.eq.1)then

            za12 = -zB6(i1,i2)
            za13 = -zB6(i1,i3)
            za14 = -zB6(i1,i4)
            za21 = -zB6(i2,i1)
            za23 = -zB6(i2,i3)
            za24 = -zB6(i2,i4)
            za25 = -zB6(i2,i5)
            za26 = -zB6(i2,i6)
            za34 = -zB6(i3,i4)
            za43 = -zB6(i4,i3)
            za46 = -zB6(i4,i6)
            za61 = -zB6(i6,i1)
            za63 = -zB6(i6,i3)

            za51 = -zB6(i5,i1)

        endif

         zpf = real(zewfac1(1))*za21/za26/za61
     .       + real(zewfac2(1))*za43/za46/za63

         zfa = zi*za21/za12/za34/za25/za51

         if(ihel.eq.1)then
            zwt = -zpf*za13**2*zfa
         endif

         if(ihel.eq.2)then
            zwt = zpf*za14**2*zfa
         endif

         if(ihel.eq.3)then
            zwt = zpf*za23**2*zfa
         endif

         if(ihel.eq.4)then
            zwt = -zpf*za24**2*zfa
         endif

      endif

! N-MHV

      if(ihel.eq.5)then
         zwt = real(zewfac1(1))*zCt1g0G1(i1,i2,i3,i4,i5,i6,iflip)
     .       + real(zewfac2(1))*zCt1g0G2(i3,i4,i1,i2,i5,i6,iflip)
      endif

      if(ihel.eq.6)then
         zwt = real(zewfac1(1))*zCt1g0G1(i1,i2,i4,i3,i5,i6,iflip)
     .       - real(zewfac2(1))*zCt1g0G2(i4,i3,i1,i2,i5,i6,iflip)
      endif

      if(ihel.eq.7)then
         zwt = real(zewfac1(1))*zCt1g0G1(i2,i1,i4,i3,i5,i6,iflip)
     .       + real(zewfac2(1))*zCt1g0G2(i4,i3,i2,i1,i5,i6,iflip)
      endif

      if(ihel.eq.8)then
         zwt = real(zewfac1(1))*zCt1g0G1(i2,i1,i3,i4,i5,i6,iflip)
     .       - real(zewfac2(1))*zCt1g0G2(i3,i4,i2,i1,i5,i6,iflip)
      endif

      if(ihel.gt.8)then
         zwt=(0d0,0d0)
      endif

      zCt1g0Ga = zwt

      return
      end

********************************************************************************
*
*     subleading colour amplitude for 2q2Q1g1G, (k,l)=(0,1) ordering , see hep-ph/9910253
*     (qb,q,Qb,Q,g,G)

      function zCt1g0Gb(i1,i2,i3,i4,i5,i6,ihel,iflip)
      use KinData_mod

      implicit real*8 (a-h,o-y)
      implicit complex*16(z)

      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)

! MHV

      call eweakV2qq(nf1,nf2,0d0)

      if(ihel.le.4)then

         zi = (0d0,1d0)

         if(iflip.eq.0)then

            za12 = zA6(i1,i2)
            za13 = zA6(i1,i3)
            za14 = zA6(i1,i4)
            za21 = zA6(i2,i1)
            za23 = zA6(i2,i3)
            za24 = zA6(i2,i4)
            za26 = zA6(i2,i6)
            za34 = zA6(i3,i4)
            za43 = zA6(i4,i3)
            za46 = zA6(i4,i6)
            za53 = zA6(i5,i3)
            za61 = zA6(i6,i1)
            za63 = zA6(i6,i3)

            za45 = zA6(i4,i5)

         elseif(iflip.eq.1)then

            za12 = -zB6(i1,i2)
            za13 = -zB6(i1,i3)
            za14 = -zB6(i1,i4)
            za21 = -zB6(i2,i1)
            za23 = -zB6(i2,i3)
            za24 = -zB6(i2,i4)
            za26 = -zB6(i2,i6)
            za34 = -zB6(i3,i4)
            za43 = -zB6(i4,i3)
            za46 = -zB6(i4,i6)
            za53 = -zB6(i5,i3)
            za61 = -zB6(i6,i1)
            za63 = -zB6(i6,i3)

            za45 = -zB6(i4,i5)

         endif

         zpf = real(zewfac1(1))*za21/za26/za61
     .       + real(zewfac2(1))*za43/za46/za63

         zfa = zi*za43/za12/za34/za45/za53 ! for MHV the only difference between (1,0) and (0,1) is in this line...

         if(ihel.eq.1)then
            zwt = -zpf*za13**2*zfa
         endif

         if(ihel.eq.2)then
            zwt = zpf*za14**2*zfa
         endif

         if(ihel.eq.3)then
            zwt = zpf*za23**2*zfa
         endif

         if(ihel.eq.4)then
            zwt = -zpf*za24**2*zfa
         endif

      endif

! N-MHV

      if(ihel.eq.5)then
         zwt = real(zewfac1(1))*zCt1g0G2(i1,i2,i3,i4,i5,i6,iflip)
     .       + real(zewfac2(1))*zCt1g0G1(i3,i4,i1,i2,i5,i6,iflip)
      endif

      if(ihel.eq.6)then
         zwt = -real(zewfac1(1))*zCt1g0G2(i1,i2,i4,i3,i5,i6,iflip)
     .       + real(zewfac2(1))*zCt1g0G1(i4,i3,i1,i2,i5,i6,iflip)
      endif

      if(ihel.eq.7)then
         zwt = real(zewfac1(1))*zCt1g0G2(i2,i1,i4,i3,i5,i6,iflip)
     .       + real(zewfac2(1))*zCt1g0G1(i4,i3,i2,i1,i5,i6,iflip)
      endif

      if(ihel.eq.8)then
         zwt = -real(zewfac1(1))*zCt1g0G2(i2,i1,i3,i4,i5,i6,iflip)
     .       + real(zewfac2(1))*zCt1g0G1(i3,i4,i2,i1,i5,i6,iflip)
      endif

      if(ihel.gt.8)then
         zwt=(0d0,0d0)
      endif

      zCt1g0Gb = zwt

      return
      end

*******************************************************************************
*
*      f1( qb q Qb Q g G ) from hep-ph/9910253
*

      function zC1g0G1(i1,i2,i3,i4,i5,i6,iflip)
      use KinData_mod

      implicit real*8 (a-h,o-y)
      implicit complex*16(z)

      zi=(0d0,1d0)

      s34=s6(i3,i4)
      s12=s6(i1,i2)

      t126=s6(i1,i2)+s6(i1,i6)+s6(i2,i6)
      t134=s6(i1,i3)+s6(i1,i4)+s6(i3,i4)

      if(iflip.eq.0)then

         za12=zA6(i1,i2)
         za14=zA6(i1,i4)
         za16=zA6(i1,i6)
         za24=zA6(i2,i4)
         za31=zA6(i3,i1)
         za36=zA6(i3,i6)
         za45=zA6(i4,i5)
         za52=zA6(i5,i2)
         za54=zA6(i5,i4)
         za62=zA6(i6,i2)
         za64=zA6(i6,i4)
         za65=zA6(i6,i5)

         zb31=zB6(i3,i1)
         zb35=zB6(i3,i5)
         zb52=zB6(i5,i2)
         zb36=zB6(i3,i6)
         zb62=zB6(i6,i2)
         zb65=zB6(i6,i5)


      elseif(iflip.eq.1)then

         za12=-zB6(i1,i2)
         za14=-zB6(i1,i4)
         za16=-zB6(i1,i6)
         za24=-zB6(i2,i4)
         za31=-zB6(i3,i1)
         za36=-zB6(i3,i6)
         za45=-zB6(i4,i5)
         za52=-zB6(i5,i2)
         za54=-zB6(i5,i4)
         za62=-zB6(i6,i2)
         za64=-zB6(i6,i4)
         za65=-zB6(i6,i5)

         zb31=-zA6(i3,i1)
         zb35=-zA6(i3,i5)
         zb52=-zA6(i5,i2)
         zb36=-zA6(i3,i6)
         zb62=-zA6(i6,i2)
         zb65=-zA6(i6,i5)

      endif

      zd1 = za16*za62*zb35*zb52*s34
      zd2 = zb52*za62*s34*t134
      zd3 = za16*za62*zb35*s34*t126

      zn1c = (zb31*za12+zb36*za62)*(zb31*za14+zb36*za64)
      zn2c = za52*zb31*(zb62*za24+zb65*za54)
      zn3c = za45*(zb31*za12+zb36*za62)*(zb31*za12+zb36*za62)

      zwtc = zn1c/zd1+zn2c/zd2+zn3c/zd3

      zC1g0G1 = zi*zwtc

      return
      end

********************************************************************************
*
*      f2( qb q Qb Q g G ) from hep-ph/9910253
*

      function zC1g0G2(i1,i2,i3,i4,i5,i6,iflip)
      use KinData_mod

      implicit real*8 (a-h,o-y)
      implicit complex*16(z)

      zi=(0d0,1d0)

      s34=s6(i3,i4)

      t126=s6(i1,i2)+s6(i1,i6)+s6(i2,i6)
      t156=s6(i1,i5)+s6(i1,i6)+s6(i5,i6)

      if(iflip.eq.0)then

         za12=zA6(i1,i2)
         za14=zA6(i1,i4)
         za15=zA6(i1,i5)
         za16=zA6(i1,i6)
         za24=zA6(i2,i4)
         za31=zA6(i3,i1)
         za36=zA6(i3,i6)
         za42=zA6(i4,i2)
         za52=zA6(i5,i2)
         za53=zA6(i5,i3)
         za62=zA6(i6,i2)
         za65=zA6(i6,i5)

         zb15=zB6(i1,i5)
         zb16=zB6(i1,i6)
         zb54=zB6(i5,i4)

         zb34=zB6(i3,i4)
         za43=zA6(i4,i3)
         zb21=zB6(i2,i1)
         zb41=zB6(i4,i1)
         za13=zA6(i1,i3)
         za32=zA6(i3,i2)
         zb42=zB6(i4,i2)
         zb52=zB6(i5,i2)
         zb32=zB6(i3,i2)
         zb14=zB6(i1,i4)
         zb31=zB6(i3,i1)
         zb36=zB6(i3,i6)

      elseif(iflip.eq.1)then

         za12=-zB6(i1,i2)
         za14=-zB6(i1,i4)
         za15=-zB6(i1,i5)
         za16=-zB6(i1,i6)
         za24=-zB6(i2,i4)
         za31=-zB6(i3,i1)
         za36=-zB6(i3,i6)
         za42=-zB6(i4,i2)
         za52=-zB6(i5,i2)
         za53=-zB6(i5,i3)
         za62=-zB6(i6,i2)
         za65=-zB6(i6,i5)

         zb15=-zA6(i1,i5)
         zb16=-zA6(i1,i6)
         zb54=-zA6(i5,i4)

         zb34=-zA6(i3,i4)
         za43=-zB6(i4,i3)
         zb21=-zA6(i2,i1)
         zb41=-zA6(i4,i1)
         za13=-zB6(i1,i3)
         za32=-zB6(i3,i2)
         zb42=-zA6(i4,i2)
         zb52=-zA6(i5,i2)
         zb32=-zA6(i3,i2)
         zb14=-zA6(i1,i4)
         zb31=-zA6(i3,i1)
         zb36=-zA6(i3,i6)


      endif

      zd1 = za16*za62*zb15*zb54*s34
      zd2 = za16*zb15*s34*t156
      zd3 = za16*za62*zb54*s34*t126

      zn1c = (zb14*za42+zb15*za52)*(zb31*za12+zb36*za62)
      zn2c = zb16*za24*(zb31*za15+zb36*za65)
      zn3c = za53*(zb31*za12+zb36*za62)*(zb31*za12+zb36*za62)

      zwtc = zn1c/zd1+zn2c/zd2+zn3c/zd3

      zC1g0G2 = zi*zwtc

      return
      end

*******************************************************************************
*
*      f3( qb q Qb Q g G ) from hep-ph/9910253
*

      function zC1g0G3(i1,i2,i3,i4,i5,i6,iflip)
      use KinData_mod

      implicit real*8 (a-h,o-y)
      implicit complex*16(z)

      zi=(0d0,1d0)

      s34=s6(i3,i4)

      t126=s6(i1,i2)+s6(i1,i6)+s6(i2,i6)
      t134=s6(i1,i3)+s6(i1,i4)+s6(i3,i4)
      t146=s6(i1,i4)+s6(i1,i6)+s6(i4,i6)

      if(iflip.eq.0)then

         za12=zA6(i1,i2)
         za16=zA6(i1,i6)
         za23=zA6(i2,i3)
         za41=zA6(i4,i1)
         za46=zA6(i4,i6)
         za52=zA6(i5,i2)
         za53=zA6(i5,i3)
         za54=zA6(i5,i4)
         za62=zA6(i6,i2)
         za65=zA6(i6,i5)

         zb35=zB6(i3,i5)
         zb41=zB6(i4,i1)
         zb52=zB6(i5,i2)

         zb46=zB6(i4,i6)
         zb62=zB6(i6,i2)
         zb65=zB6(i6,i5)

      elseif(iflip.eq.1)then

         za12=-zB6(i1,i2)
         za16=-zB6(i1,i6)
         za23=-zB6(i2,i3)
         za41=-zB6(i4,i1)
         za46=-zB6(i4,i6)
         za52=-zB6(i5,i2)
         za53=-zB6(i5,i3)
         za54=-zB6(i5,i4)
         za62=-zB6(i6,i2)
         za65=-zB6(i6,i5)

         zb35=-zA6(i3,i5)
         zb41=-zA6(i4,i1)
         zb52=-zA6(i5,i2)

         zb46=-zA6(i4,i6)
         zb62=-zA6(i6,i2)
         zb65=-zA6(i6,i5)

      endif

! same denominators as f1, zd2 & zd3 interchanged in paper, but not here!

      zd1 = za16*za62*zb35*zb52*s34
      zd2 = zb52*za62*s34*t134
      zd3 = za16*za62*zb35*s34*t126

      zn1c = t146*(zb41*za12+zb46*za62)
      zn2c = za52*zb41*(zb62*za23+zb65*za53)
      zn3c = za54*(zb41*za12+zb46*za62)*(zb41*za12+zb46*za62)

      zwtc = -zn1c/zd1+zn2c/zd2+zn3c/zd3

      zC1g0G3 = zi*zwtc

      return
      end

********************************************************************************
*
*      f4( qb q Qb Q g G ) from hep-ph/9910253
*

      function zC1g0G4(i1,i2,i3,i4,i5,i6,iflip)
      use KinData_mod

      implicit real*8 (a-h,o-y)
      implicit complex*16(z)

      zi=(0d0,1d0)

      s34=s6(i3,i4)

      t126=s6(i1,i2)+s6(i1,i6)+s6(i2,i6)
      t156=s6(i1,i5)+s6(i1,i6)+s6(i5,i6)

      if(iflip.eq.0)then

         za12=zA6(i1,i2)
         za15=zA6(i1,i5)
         za16=zA6(i1,i6)
         za23=zA6(i2,i3)
         za35=zA6(i3,i5)
         za41=zA6(i4,i1)
         za46=zA6(i4,i6)
         za62=zA6(i6,i2)
         za65=zA6(i6,i5)

         zb15=zB6(i1,i5)
         zb16=zB6(i1,i6)
         zb41=zB6(i4,i1)
         zb54=zB6(i5,i4)
         zb46=zB6(i4,i6)

      elseif(iflip.eq.1)then

         za12=-zB6(i1,i2)
         za15=-zB6(i1,i5)
         za16=-zB6(i1,i6)
         za23=-zB6(i2,i3)
         za35=-zB6(i3,i5)
         za41=-zB6(i4,i1)
         za46=-zB6(i4,i6)
         za62=-zB6(i6,i2)
         za65=-zB6(i6,i5)

         zb15=-zA6(i1,i5)
         zb16=-zA6(i1,i6)
         zb41=-zA6(i4,i1)
         zb54=-zA6(i5,i4)
         zb46=-zA6(i4,i6)

      endif

      zd1 = za16*za62*zb15*zb54*s34
      zd2 = za16*zb15*s34*t156
      zd3 = za16*za62*zb54*s34*t126

      zn1 = zb41*za23*(zb41*za12+zb46*za62)
      zn2 = zb16*za23*(zb41*za15+zb46*za65)
      zn3 = za35*(zb41*za12+zb46*za62)*(zb41*za12+zb46*za62)

      zwt = zn1/zd1+zn2/zd2+zn3/zd3

      zC1g0G4 = zi*zwt

      return
      end

********************************************************************************
*
*      g1( qb q Qb Q g G ) from hep-ph/9910253
*

      function zCt1g0G1(i1,i2,i3,i4,i5,i6,iflip)
      use KinData_mod

      implicit real*8 (a-h,o-y)
      implicit complex*16(z)

      zi=(0d0,1d0)

      s34=s6(i3,i4)

      t134=s6(i1,i3)+s6(i1,i4)+s6(i3,i4)
      t156=s6(i1,i5)+s6(i1,i6)+s6(i5,i6)

      if(iflip.eq.0)then

         za12=zA6(i1,i2)
         za16=zA6(i1,i6)
         za13=zA6(i1,i3)
         za15=zA6(i1,i5)
         za24=zA6(i2,i4)
         za31=zA6(i3,i1)
         za34=zA6(i3,i4)
         za36=zA6(i3,i6)
         za52=zA6(i5,i2)
         za54=zA6(i5,i4)
         za62=zA6(i6,i2)
         za64=zA6(i6,i4)
         za65=zA6(i6,i5)

         zb15=zB6(i1,i5)
         zb16=zB6(i1,i6)
         zb31=zB6(i3,i1)
         zb52=zB6(i5,i2)

         zb13=zB6(i1,i3)
         zb36=zB6(i3,i6)
         zb62=zB6(i6,i2)
         zb65=zB6(i6,i5)

      elseif(iflip.eq.1)then

         za12=-zB6(i1,i2)
         za16=-zB6(i1,i6)
         za13=-zB6(i1,i3)
         za15=-zB6(i1,i5)
         za24=-zB6(i2,i4)
         za31=-zB6(i3,i1)
         za34=-zB6(i3,i4)
         za36=-zB6(i3,i6)
         za52=-zB6(i5,i2)
         za54=-zB6(i5,i4)
         za62=-zB6(i6,i2)
         za64=-zB6(i6,i4)
         za65=-zB6(i6,i5)

         zb15=-zA6(i1,i5)
         zb16=-zA6(i1,i6)
         zb31=-zA6(i3,i1)
         zb52=-zA6(i5,i2)

         zb13=-zA6(i1,i3)
         zb36=-zA6(i3,i6)
         zb62=-zA6(i6,i2)
         zb65=-zA6(i6,i5)

      endif

      zd1 = za16*za62*zb15*zb52*s34
      zd2 = za16*zb15*s34*t156
      zd3 = zb52*za62*s34*t134

      zn1c = (zb13*za34+zb16*za64)*(zb31*za12+zb36*za62)
      zn2c = zb16*za24*(zb31*za15+zb36*za65)
      zn3c = za52*zb31*(zb62*za24+zb65*za54)

      zwtc = zn1c/zd1+zn2c/zd2+zn3c/zd3

      zCt1g0G1 = zi*zwtc

      return
      end

********************************************************************************
*
*      g2( qb q Qb Q g G ) from hep-ph/9910253
*

      function zCt1g0G2(i1,i2,i3,i4,i5,i6,iflip)
      use KinData_mod

      implicit real*8 (a-h,o-y)
      implicit complex*16(z)

      zi=(0d0,1d0)

      t126=s6(i1,i2)+s6(i1,i6)+s6(i2,i6)

      if(iflip.eq.0)then

         za12=zA6(i1,i2)
         za16=zA6(i1,i6)
         za31=zA6(i3,i1)
         za36=zA6(i3,i6)
         za62=zA6(i6,i2)

         zb35=zB6(i3,i5)
         zb54=zB6(i5,i4)

         zb31=zB6(i3,i1)
         zb36=zB6(i3,i6)

      elseif(iflip.eq.1)then

         za12=-zB6(i1,i2)
         za16=-zB6(i1,i6)
         za31=-zB6(i3,i1)
         za36=-zB6(i3,i6)
         za62=-zB6(i6,i2)

         zb35=-zA6(i3,i5)
         zb54=-zA6(i5,i4)

         zb31=-zA6(i3,i1)
         zb36=-zA6(i3,i6)

      endif

      zd1 = za16*za62*zb35*zb54*t126

      zn1c = (zb31*za12+zb36*za62)*(zb31*za12+zb36*za62)

      zwtc = -zn1c/zd1

      zCt1g0G2 = zi*zwtc

      return
      end

********************************************************************************
