c--- libGJRVB  --------------------------------------------------------------
c
c     This library contains all the real-virtual B-type MEs for gamma+jet.
c
c     Amplitudes taken from:
c      
c     [1] A. Signer
c         One-loop corrections to five-parton amplitudes with external photons
c         Phys.Lett. B357 (1995) 204-210c     
c         DOI:10.1016/0370-2693(95)00905-Z
c         [hep-ph/9507442]      
c
c     and:
c
c     [2] A. Signer
c         Helicity method for next-to-leading order corrections in QCD
c         (Doctoral Thesis)
c         Diss. Naturwiss. ETH Zürich, Nr. 11143, 1995
c         DOI:10.3929/ethz-a-001503525
c      
c----------------------------------------------------------------------------

      function fullB2g1G(iqb,ig1,ig2,iq,iG,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16(z)
      parameter(pi=3.141592653589793238d0)
      parameter(qnf=0.333333333333333333d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /eweakZ/ amz,zewfac(4),zewnull(4)
!$omp threadprivate(/eweakZ/)

      if(iqb.le.2.and.iq.le.2) then
         ave = 1d0/4d0/nc**2
      elseif(iqb.gt.2.and.iq.gt.2) then
         ave = 1d0/4d0/(nc**2-1d0)**2
      else   
         ave = 1d0/4d0/nc/(nc**2-1d0)
      endif   
      
      fac = ave*(4d0*pi*as)**3*(4d0*pi*amz)*4d0*nc**2*(nc**2-1d0)/(8d0*pi**2)

      FullB2g1G = ( B2g1G(iqb,ig1,ig2,iq,iG,renscale2)
     .             +B2g1G(iqb,ig2,ig1,iq,iG,renscale2))
     .     +1d0/dble(nc)*dble(nf)*(
     .              Bh2g1G(iqb,ig1,ig2,iq,iG,renscale2)
     .             +Bh2g1G(iqb,ig2,ig1,iq,iG,renscale2))
     .     -1d0/dble(nc)**2*(
     .              Bt2g1G(iqb,ig1,ig2,iq,iG,renscale2)
     .             +Bt2g1G(iqb,ig2,ig1,iq,iG,renscale2))
     .     -1d0/dble(nc)**2*(
     .              Btt2g1G(iqb,ig1,ig2,iq,iG,renscale2))
     .     -1d0/dble(nc)**3*dble(nf)*(
     .              Bth2g1G(iqb,ig1,ig2,iq,iG,renscale2))
     .     +1d0/dble(nc)**3*(nc**2-4d0)*qnf*(
     .              Bv2g1G(iqb,ig1,ig2,iq,iG,renscale2))
     .     +1d0/dble(nc)**4*(
     .              Bttt2g1G(iqb,ig1,ig2,iq,iG,renscale2))
      
      FullB2g1G = fac*FullB2g1G
      
      return
      end

*************************************************************

      function B2g1G(iqb,ig1,ig2,iq,iG,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16(z)
      parameter(pi=3.141592653589793238d0)
      common /eweakZ/ amz,zewfac(4),zewnull(4)
!$omp threadprivate(/eweakZ/)
      common /BZFlav/nflav
!$omp threadprivate(/BZFlav/)
      common/order/iorder

      call eweakV1qq(nflav,0d0)

      B2g1G = 0d0

      zi = (0d0,1d0)
      zwt = 0d0

c     Note the reversed quark string.

      do ihel=1,6
         do iflip=0,1
            zwt = zwt 
     .           + dconjg(zb1gam012(iq,ig2,ig1,iqb,iG,ihel,iflip))
     .           * zB1gaml12(iq,ig2,ig1,iqb,iG,ihel,iflip,renscale2)
         enddo
      enddo

      B2g1G = abs(zewfac(1))**2*real(zi*zwt)/2d0

      if (iorder .eq. -1) then
c     UV counter term at order 1/e to renormalise the matrix element in the MS bar scheme
         subuv = 11d0/3d0
         B2g1G = B2g1G - subuv*B2g0G(iqb,ig1,ig2,iq,iG)
      elseif (iorder .eq. 0)then
c     pi**2/4d0 comes from a correction for C(e)
c     1/2 comes from a conversion of four-dimensional helicity scheme (FDH) to 't Hooft-Veltman (HV) scheme
c     see hep-ph/9708239 equation 6.4
         e0 = pi**2/4d0 - 0.5d0
         B2g1G = B2g1G + e0*B2g0G(iqb,ig1,ig2,iq,iG)
      endif

      return
      end

**********************************************
* A wrapper function for returning the normal
* matrix element for situations where either quark or
* antiquark are in the initial state. Otherwise
* it will return a line reversal symmetrised matrix element
*

      function sB2g1G(iqb,ig1,ig2,iq,iG,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      if((iqb.le.2).or.(iq.le.2))then
        sB2g1G = B2g1G(iqb,ig1,ig2,iq,iG,renscale2)
      else
        sB2g1G = 0.5d0*(B2g1G(iqb,ig1,ig2,iq,iG,renscale2)
     .                 +B2g1G(iq,ig2,ig1,iqb,iG,renscale2))
      endif

      return
      end

*************************************************************

      function Bt2g1G(iqb,ig1,ig2,iq,iG,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16(z)
      parameter(pi=3.141592653589793238d0)
      common /eweakZ/ amz,zewfac(4),zewnull(4)
!$omp threadprivate(/eweakZ/)
      common /BZFlav/nflav
!$omp threadprivate(/BZFlav/)
      common/order/iorder

      call eweakV1qq(nflav,0d0)

      Bt2g1G = 0d0

      zi = (0d0,1d0)
      zwt = 0d0

      do ihel=1,6
         do iflip=0,1
            zwt = zwt 
     .           + dconjg(zb1gam012(iq,ig2,ig1,iqb,iG,ihel,iflip))
     .           * zB1gams12(iq,ig2,ig1,iqb,iG,ihel,iflip,renscale2)

         enddo
      enddo

c     the "-" compensates the "-" in qcdnormGJ.f; then it coincides with the structure of Bt2g1Z, as it should
      Bt2g1G = abs(zewfac(1))**2*real(-zi*zwt)/2d0

      if (iorder .eq. 0)then
c     pi**2/12d0 comes from a correction for C(e)
c     1/2 comes from a conversion of four-dimensional helicity scheme (FDH) to 't Hooft-Veltman (HV) scheme
c     see hep-ph/9708239 equation 6.4
         e0 = pi**2/12d0 - 0.5d0
         Bt2g1G = Bt2g1G + e0*B2g0G(iqb,ig1,ig2,iq,iG)
      endif

      return
      end


**********************************************
* A wrapper function for returning the normal
* matrix element for situations where either quark or
* antiquark are in the initial state. Otherwise
* it will return a line reversal symmetrised matrix element
*

      function sBt2g1G(iqb,ig1,ig2,iq,iG,renscale2)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      if((iqb.le.2).or.(iq.le.2))then
        sBt2g1G = Bt2g1G(iqb,ig1,ig2,iq,iG,renscale2)
      else
        sBt2g1G = 0.5d0*(Bt2g1G(iqb,ig1,ig2,iq,iG,renscale2)
     .                  +Bt2g1G(iq,ig2,ig1,iqb,iG,renscale2))
      endif

      return
      end

*************************************************************

      function Btt2g1G(iqb,ig1,ig2,iq,iG,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16(z)
      parameter(pi=3.141592653589793238d0)
      common /eweakZ/ amz,zewfac(4),zewnull(4)
!$omp threadprivate(/eweakZ/)
      common /BZFlav/nflav
!$omp threadprivate(/BZFlav/)
      common/order/iorder

      call eweakV1qq(nflav,0d0)

      Btt2g1G = 0d0

      zi = (0d0,1d0)
      zwt1 = 0d0
      zwt2 = 0d0

      do ihel=1,6
         do iflip=0,1
            zwt1 = zwt1 
     .             + dconjg(  zb1gam012(iq,ig2,ig1,iqb,iG,ihel,iflip)
     .                      + zb1gam021(iq,ig2,ig1,iqb,iG,ihel,iflip))
     .             *(  zB1gaml12(iq,ig2,ig1,iqb,iG,ihel,iflip,renscale2)
     .               + zB1gaml21(iq,ig2,ig1,iqb,iG,ihel,iflip,renscale2))
            zwt2 = zwt2 
     .             + dconjg(  zb1gam012(iq,ig2,ig1,iqb,iG,ihel,iflip)
     .                      + zb1gam021(iq,ig2,ig1,iqb,iG,ihel,iflip))
     .             * zB1gamtr(iq,ig2,ig1,iqb,iG,ihel,iflip,renscale2) 
         enddo
      enddo

      Btt2g1G = abs(zewfac(1))**2*real(zi*zwt1-zi*zwt2)/2d0

      if (iorder .eq. -1) then
c     UV counter term at order 1/e to renormalise the matrix element in the MS bar scheme
         subuv = 11d0/3d0
         Btt2g1G = Btt2g1G - subuv*Bt2g0G(iqb,ig1,ig2,iq,iG)
      elseif (iorder .eq. 0)then
c     pi**2/4d0 comes from a correction for C(e)
c     1/2 comes from a conversion of four-dimensional helicity scheme (FDH) to 't Hooft-Veltman (HV) scheme
c     see hep-ph/9708239 equation 6.4
         e0 = pi**2/4d0 - 0.5d0
         Btt2g1G = Btt2g1G + e0*Bt2g0G(iqb,ig1,ig2,iq,iG)
      endif

      return
      end

**********************************************
* A wrapper function for returning the normal
* matrix element for situations where either quark or
* antiquark are in the initial state. Otherwise
* it will return a line reversal symmetrised matrix element
*

      function sBtt2g1Gc(iqb,ig1,ig2,iq,iG,renscale2)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      if((iqb.le.2).or.(iq.le.2))then
        sBtt2g1Gc = Btt2g1G(iqb,ig1,ig2,iq,iG,renscale2)
      else
        sBtt2g1Gc = 0.5d0*(Btt2g1G(iqb,ig1,ig2,iq,iG,renscale2)
     .                  +Btt2g1G(iq,ig2,ig1,iqb,iG,renscale2))
      endif

      return
      end

*************************************************************

      function Bttt2g1G(iqb,ig1,ig2,iq,iG,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16(z)
      parameter(pi=3.141592653589793238d0)
      common /eweakZ/ amz,zewfac(4),zewnull(4)
!$omp threadprivate(/eweakZ/)
      common /BZFlav/nflav
!$omp threadprivate(/BZFlav/)
      common/order/iorder

      call eweakV1qq(nflav,0d0)

      Bttt2g1G = 0d0

      zi = (0d0,1d0)
      zwt = 0d0

      do ihel=1,6
         do iflip=0,1
            zwt = zwt
     .            + dconjg(  zb1gam012(iq,ig2,ig1,iqb,iG,ihel,iflip)
     .                     + zb1gam021(iq,ig2,ig1,iqb,iG,ihel,iflip))
     .            *(  zB1gams12(iq,ig2,ig1,iqb,iG,ihel,iflip,renscale2)
     .              + zB1gams21(iq,ig2,ig1,iqb,iG,ihel,iflip,renscale2))
         enddo
      enddo

      Bttt2g1G = abs(zewfac(1))**2*real(-zi*zwt)/2d0

      if (iorder .eq. 0)then
c     pi**2/12d0 comes from a correction for C(e)
c     1/2 comes from a conversion of four-dimensional helicity scheme (FDH) to 't Hooft-Veltman (HV) scheme
c     see hep-ph/9708239 equation 6.4
         e0 = pi**2/12d0 - 0.5d0
         Bttt2g1G = Bttt2g1G + e0*Bt2g0G(iqb,ig1,ig2,iq,iG)
      endif

      return
      end

**********************************************
* A wrapper function for returning the normal
* matrix element for situations where either quark or
* antiquark are in the initial state. Otherwise
* it will return a line reversal symmetrised matrix element
*

      function sBttt2g1G(iqb,ig1,ig2,iq,iG,renscale2)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      if((iqb.le.2).or.(iq.le.2))then
        sBttt2g1G = Bttt2g1G(iqb,ig1,ig2,iq,iG,renscale2)
      else
        sBttt2g1G = 0.5d0*(Bttt2g1G(iqb,ig1,ig2,iq,iG,renscale2)
     .                  +Bttt2g1G(iq,ig2,ig1,iqb,iG,renscale2))
      endif

      return
      end

*************************************************************

      function Bh2g1G(iqb,ig1,ig2,iq,iG,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16(z)
      parameter(pi=3.141592653589793238d0)
      common /eweakZ/ amz,zewfac(4),zewnull(4)
!$omp threadprivate(/eweakZ/)
      common /BZFlav/nflav
!$omp threadprivate(/BZFlav/)
      common/order/iorder

      call eweakV1qq(nflav,0d0)

      Bh2g1G = 0d0

      zi = (0d0,1d0)
      zwt = 0d0

      do ihel=1,6
         do iflip=0,1
            zwt = zwt
     .           + dconjg(zb1gam012(iq,ig1,ig2,iqb,iG,ihel,iflip))
     .           * zB1gamX12(iq,ig1,ig2,iqb,iG,ihel,iflip)
         enddo
      enddo

      Bh2g1G = abs(zewfac(1))**2*real(zi*zwt)/2d0

      if (iorder .eq. -1) then
c     UV counter term at order 1/e to renormalise the matrix element in the MS bar scheme
         subuv = -2d0/3d0
         Bh2g1G = Bh2g1G - subuv*B2g0G(iqb,ig1,ig2,iq,iG)
c     Note: it's the UV counterterm which introduces an e-pole, unrenorm.: Bh2g1G(iorder=-1)=0 
      endif

      return
      end

**********************************************
* A wrapper function for returning the normal
* matrix element for situations where either quark or
* antiquark are in the initial state. Otherwise
* it will return a line reversal symmetrised matrix element
*

      function sBh2g1G(iqb,ig1,ig2,iq,iG,renscale2)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      if((iqb.le.2).or.(iq.le.2))then
        sBh2g1G = Bh2g1G(iqb,ig1,ig2,iq,iG,renscale2)
      else
        sBh2g1G = 0.5d0*(Bh2g1G(iqb,ig1,ig2,iq,iG,renscale2)
     .                  +Bh2g1G(iq,ig2,ig1,iqb,iG,renscale2))
      endif

      return
      end

*************************************************************

      function Bth2g1G(iqb,ig1,ig2,iq,iG,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16(z)
      parameter(pi=3.141592653589793238d0)
      common /eweakZ/ amz,zewfac(4),zewnull(4)
!$omp threadprivate(/eweakZ/)
      common /BZFlav/nflav
!$omp threadprivate(/BZFlav/)
      common/order/iorder

      call eweakV1qq(nflav,0d0)

      Bth2g1G = 0d0

c     Note: the unrenormalised Bth2g1G is =0, because B1gamX12=-B1gamX21 and Bth2g1G~(B1gamX12+B1gamX21)

      if (iorder .eq. -1) then
c     UV counter term at order 1/e to renormalise the matrix element in the MS bar scheme
         subuv = -2d0/3d0
         Bth2g1G = - subuv*Bt2g0G(iqb,ig1,ig2,iq,iG)
      endif

      return
      end

**********************************************
* A wrapper function for returning the normal
* matrix element for situations where either quark or
* antiquark are in the initial state. Otherwise
* it will return a line reversal symmetrised matrix element
*

      function sBth2g1G(iqb,ig1,ig2,iq,iG,renscale2)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      if((iqb.le.2).or.(iq.le.2))then
        sBth2g1G = Bth2g1G(iqb,ig1,ig2,iq,iG,renscale2)
      else
        sBth2g1G = 0.5d0*(Bth2g1G(iqb,ig1,ig2,iq,iG,renscale2)
     .                  +Bth2g1G(iq,ig2,ig1,iqb,iG,renscale2))
      endif

      return
      end

*************************************************************

      function Bv2g1G(iqb,ig1,ig2,iq,iG,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16(z)
      parameter(pi=3.141592653589793238d0)
      common /eweakZ/ amz,zewfac(4),zewnull(4)
!$omp threadprivate(/eweakZ/)
      common /BZFlav/nflav
!$omp threadprivate(/BZFlav/)
      common/order/iorder

      call eweakV1qq(nflav,0d0)

      Bv2g1G = 0d0

      zi = (0d0,1d0)
      zwt = 0d0

      do ihel=1,6
         do iflip=0,1
            zwt = zwt
     .           + dconjg( zb1gam012(iq,ig1,ig2,iqb,iG,ihel,iflip)
     .                    +zb1gam021(iq,ig1,ig2,iqb,iG,ihel,iflip))
     .           * zBNFh(iq,ig1,ig2,iqb,iG,ihel,iflip,renscale2)
         enddo
      enddo

      Bv2g1G = real(zewfac(1))*real(-zi*1d0/2d0*zwt)/2d0

      Bv2g1G = -Bv2g1G

      return
      end

*************************************************************
*
*     tree level 2q2g1G amplitude b_{12}^{1gamma,(0)} from [2]
*
*     - See also eq. 7 in [1] hep-ph/9507442.
*     - We can recycle zBt123tree from jet libB1.f or code it directly,
*       as it is not complicated.
*
      function zb1gam012(iq,ig1,ig2,iqb,iG,ihel,iflip)
      use KinData_mod
  
      implicit real*8 (a-h,o-y)
      implicit complex*16(z)

      zi=(0d0,1d0)

      i1 = ig1
      i2 = ig2
      i3 = iG
      i4 = iq
      i5 = iqb

*                          q  qb g1 g2 G              q  qb  g1 g2 G
*     ihel = 1   iflip =0  +  -| -  +  +   iflip = 1  -  +| +  -  -
*     ihel = 2   iflip =0  +  -| +  -  +   iflip = 1  -  +| -  +  -
*     ihel = 3   iflip =0  +  -| +  +  -   iflip = 1  -  +| -  -  +
*     ihel = 4   iflip =0  -  +| -  +  +   iflip = 1  +  -| +  -  -
*     ihel = 5   iflip =0  -  +| +  -  +   iflip = 1  +  -| -  +  -
*     ihel = 6   iflip =0  -  +| +  +  -   iflip = 1  +  -| -  -  +
*
      if(iflip.eq.0)then

         za12=zA5(i1,i2)
         za25=zA5(i2,i5)
         za34=zA5(i3,i4)
         za41=zA5(i4,i1)
         za42=zA5(i4,i2)
         za43=zA5(i4,i3)
         za51=zA5(i5,i1)
         za52=zA5(i5,i2)
         za53=zA5(i5,i3)

      elseif(iflip.eq.1)then

         za12=-zB5(i1,i2)
         za25=-zB5(i2,i5)
         za34=-zB5(i3,i4)
         za41=-zB5(i4,i1)
         za42=-zB5(i4,i2)
         za43=-zB5(i4,i3)
         za51=-zB5(i5,i1)
         za52=-zB5(i5,i2)
         za53=-zB5(i5,i3)
  
      endif   

      zd = za41*za12*za25*za53*za34

      if(ihel.eq.1)then
         zn = za41*za51**3
      elseif(ihel.eq.2)then   
         zn = za42*za52**3
      elseif(ihel.eq.3)then
         zn = za43*za53**3
      elseif(ihel.eq.4)then
         zn = -za51*za41**3
      elseif(ihel.eq.5)then
         zn = -za52*za42**3
      elseif(ihel.eq.6)then
         zn = -za53*za43**3
      endif   

      zb1gam012 = zi*zn/zd 

      return
      end

*************************************************************
*
*     tree level 2q2g1G amplitude b_{12}^{1gamma,(0)} [2]
*     with reversed gluon ordering
*

      function zb1gam021(iq,ig1,ig2,iqb,iG,ihel,iflip)
      use KinData_mod
  
      implicit real*8 (a-h,o-y)
      implicit complex*16(z)
      dimension ihel2(6)
      
      ihel2(1)=2
      ihel2(2)=1
      ihel2(3)=3
      ihel2(4)=5
      ihel2(5)=4
      ihel2(6)=6

      zb1gam021 = zb1gam012(iq,ig2,ig1,iqb,iG,ihel2(ihel),iflip)

      return
      end

*************************************************************
*
*     function B_{12}^{1gamma,l} from [2],
*     see eqs. 5.76, 5.80, 5.84
*

      function zB1gaml12(iq,ig1,ig2,iqb,iG,ihel,iflip,renscale2)
      use KinData_mod
  
      implicit real*8 (a-h,o-y)
      implicit complex*16(z)
      common/order/iorder

      zi=(0d0,1d0)

*                          q  qb g1 g2 G              q  qb  g1 g2 G
*     ihel = 1   iflip =0  +  -| -  +  +   iflip = 1  -  +| +  -  -
*     ihel = 2   iflip =0  +  -| +  -  +   iflip = 1  -  +| -  +  -
*     ihel = 3   iflip =0  +  -| +  +  -   iflip = 1  -  +| -  -  +
*     ihel = 4   iflip =0  -  +| -  +  +   iflip = 1  +  -| +  -  -
*     ihel = 5   iflip =0  -  +| +  -  +   iflip = 1  +  -| -  +  -
*     ihel = 6   iflip =0  -  +| +  +  -   iflip = 1  +  -| -  -  +
*
*     (hq,hqb,hg1,hg2,hG) = (hqb,hq,hg2,hg1,hG)|q<->qb,g1<->g2 
*
*     =>
*
*     (ihel=4) = (ihel=2)|q<->qb,g1<->g2
*     (ihel=5) = (ihel=1)|q<->qb,g1<->g2
*     (ihel=6) = (ihel=3)|q<->qb,g1<->g2
*

      if(ihel.le.3)then
         j1 = ig1
         j2 = ig2
         j3 = iG
         j4 = iqb
         j5 = iq
         sign = 1d0
      elseif(ihel.ge.4)then
         j1 = ig2
         j2 = ig1
         j3 = iG
         j4 = iq
         j5 = iqb 
         sign = 1d0
      endif

      if(iflip.eq.0)then
        za12=zA5(j1,j2)
        za21=zA5(j2,j1)
        za13=zA5(j1,j3)
        za31=zA5(j3,j1)
        za14=zA5(j1,j4)
        za41=zA5(j4,j1)
        za15=zA5(j1,j5)
        za51=zA5(j5,j1)
        za23=zA5(j2,j3)
        za32=zA5(j3,j2)
        za24=zA5(j2,j4)
        za42=zA5(j4,j2)
        za25=zA5(j2,j5)
        za52=zA5(j5,j2)
        za34=zA5(j3,j4)
        za43=zA5(j4,j3)
        za35=zA5(j3,j5)
        za53=zA5(j5,j3)
        za45=zA5(j4,j5)
        za54=zA5(j5,j4)

        zb12=zB5(j1,j2)
        zb21=zB5(j2,j1)
        zb13=zB5(j1,j3)
        zb31=zB5(j3,j1)
        zb14=zB5(j1,j4)
        zb41=zB5(j4,j1)
        zb15=zB5(j1,j5)
        zb51=zB5(j5,j1)
        zb23=zB5(j2,j3)
        zb32=zB5(j3,j2)
        zb24=zB5(j2,j4)
        zb42=zB5(j4,j2)
        zb25=zB5(j2,j5)
        zb52=zB5(j5,j2)
        zb34=zB5(j3,j4)
        zb43=zB5(j4,j3)
        zb35=zB5(j3,j5)
        zb53=zB5(j5,j3)
        zb45=zB5(j4,j5)
        zb54=zB5(j5,j4)

      elseif(iflip.eq.1)then
        za12=-zB5(j1,j2)
        za21=-zB5(j2,j1)
        za13=-zB5(j1,j3)
        za31=-zB5(j3,j1)
        za14=-zB5(j1,j4)
        za41=-zB5(j4,j1)
        za15=-zB5(j1,j5)
        za51=-zB5(j5,j1)
        za23=-zB5(j2,j3)
        za32=-zB5(j3,j2)
        za24=-zB5(j2,j4)
        za42=-zB5(j4,j2)
        za25=-zB5(j2,j5)
        za52=-zB5(j5,j2)
        za34=-zB5(j3,j4)
        za43=-zB5(j4,j3)
        za35=-zB5(j3,j5)
        za53=-zB5(j5,j3)
        za45=-zB5(j4,j5)
        za54=-zB5(j5,j4)

        zb12=-zA5(j1,j2)
        zb21=-zA5(j2,j1)
        zb13=-zA5(j1,j3)
        zb31=-zA5(j3,j1)
        zb14=-zA5(j1,j4)
        zb41=-zA5(j4,j1)
        zb15=-zA5(j1,j5)
        zb51=-zA5(j5,j1)
        zb23=-zA5(j2,j3)
        zb32=-zA5(j3,j2)
        zb24=-zA5(j2,j4)
        zb42=-zA5(j4,j2)
        zb25=-zA5(j2,j5)
        zb52=-zA5(j5,j2)
        zb34=-zA5(j3,j4)
        zb43=-zA5(j4,j3)
        zb35=-zA5(j3,j5)
        zb53=-zA5(j5,j3)
        zb45=-zA5(j4,j5)
        zb54=-zA5(j5,j4)
      endif

        s12=s5(j1,j2)
        s13=s5(j1,j3)
        s14=s5(j1,j4)
        s15=s5(j1,j5)
        s21=s5(j2,j1)
        s23=s5(j2,j3)
        s24=s5(j2,j4)
        s25=s5(j2,j5)
        s31=s5(j3,j1)
        s32=s5(j3,j2)
        s34=s5(j3,j4)
        s35=s5(j3,j5)
        s41=s5(j4,j1)
        s42=s5(j4,j2)
        s43=s5(j4,j3)
        s45=s5(j4,j5)
        s51=s5(j5,j1)
        s52=s5(j5,j2)
        s53=s5(j5,j3)
        s54=s5(j5,j4)

      if(ihel.eq.1.or.ihel.eq.5)then
         zwt0 = zi*zb1gam012(j5,j1,j2,j4,j3,1,iflip)
     .         *( 0.5d0*(zzLograt(renscale2,-s42))**2
     .          + 0.5d0*(zzLograt(renscale2,-s51))**2
     .          + 0.5d0*(zzLograt(renscale2,-s12))**2
     .          + (3d0/2d0)*(zzLograt(renscale2,-s51))
     .          + 3d0 + zzCFGJ(s41,s42,s21) + zzCFGJ(s13,s15,s53)
     .          + (za42*za51/za41/za52)**3*zzCFGJ(s52,s51,s12)
     .          + (za42*za31/za41/za32)**3*zzCFGJ(s23,s24,s43))

     .         + za45**2*za51*zb52**2/(2d0*za43*za52*za53*zb12*s43)
     .         + za43*zb23**2/(2d0*za52*za53*zb51*zb12)
     .         - za41**2*za51/(2d0*za43*za52*za53*za12)
     .           * zL0(s43,s51,renscale2)
     .         + za43*za12*za13*zb23/(za53*za23**2)
     .           * zL1(s42,s51,renscale2)
     .         + 2d0*za41*za13*zb23/(za53*za23)*zL1(s42,s51,renscale2)

     .         +( za42*za51**2*zb52*(za42*za51 + za41*za52)
     .            /(za43*za52**2*za53*za12)
     .          + za13*zb23*(za42*za13 + za41*za23)
     .            /(za53*za23**2))
     .          * zL1(s43,s51,renscale2)

     .         - 2d0*za45*za41*za51*zb52/(za43*za52*za53)
     .           * zL1(s43,s12,renscale2)
     .         + za45**2*za51*za12*zb52/(za43*za52**2*za53)
     .           * zL1(s43,s12,renscale2)
     .         - za42*za13**2*zb23**2/(2d0*za53*za23)
     .           * zL2(s42,s51,renscale2)
     .         - za43*za12**2*zb23**2/(2d0*za52*za23)
     .           * zL2(s43,s51,renscale2)
     .         - za45**2*za51*za12*zb52**2/(2d0*za43*za52*za53)
     .           * zL2(s12,s43,renscale2)

         zwt1 = zi*zb1gam012(j5,j1,j2,j4,j3,1,iflip)
     .         *( (zzLograt(renscale2,-s42))
     .          + (zzLograt(renscale2,-s51))
     .          + (zzLograt(renscale2,-s12))
     .          + (3d0/2d0))

         zwt2 = zi*zb1gam012(j5,j1,j2,j4,j3,1,iflip)*3d0

      elseif(ihel.eq.2.or.ihel.eq.4)then
         zwt0 = zi*zb1gam012(j5,j1,j2,j4,j3,2,iflip)
     .         *( 0.5d0*(zzLograt(renscale2,-s42))**2
     .          + 0.5d0*(zzLograt(renscale2,-s51))**2
     .          + 0.5d0*(zzLograt(renscale2,-s12))**2
     .          + (3d0/2d0)*(zzLograt(renscale2,-s43))
     .          + 3d0 + za42*za51/za41/za52*zzCFGJ(s41,s42,s21)
     .          + zzCFGJ(s52,s51,s12)
     .          + za51*za32/za52/za31*zzCFGJ(s13,s15,s53)
     .          + zzCFGJ(s23,s24,s43))
     .         + za45**2*za52*zb51**2/(2d0*za43*za51*za53*zb12*s43)
     .         - zb13**2/(2d0*za51*zb42*zb12)
     .         + 2d0*za45*za42*za52*zb51/(za43*za51*za53)
     .           * zL1(s43,s12,renscale2)
     .         - za45**2*za52*za12*zb51**2/(2d0*za43*za51*za53)
     .           * zL2(s12,s43,renscale2)

         zwt1 = zi*zb1gam012(j5,j1,j2,j4,j3,2,iflip)
     .         *( (zzLograt(renscale2,-s42))
     .          + (zzLograt(renscale2,-s51))
     .          + (zzLograt(renscale2,-s12))
     .          + (3d0/2d0))

         zwt2 = zi*zb1gam012(j5,j1,j2,j4,j3,2,iflip)*3d0

      elseif(ihel.eq.3.or.ihel.eq.6)then
         zwt0 = zi*zb1gam012(j5,j1,j2,j4,j3,3,iflip)
     .         *( 0.5d0*(zzLograt(renscale2,-s42))**2
     .          + 0.5d0*(zzLograt(renscale2,-s51))**2
     .          + 0.5d0*(zzLograt(renscale2,-s12))**2
     .          + (3d0/2d0)*(zzLograt(renscale2,-s53))
     .          + 3d0 + zzCFGJ(s41,s42,s21)
     .          + zzCFGJ(s45,s43,s35)
     .          + zzCFGJ(s52,s51,s12))
     .         - zb51*zb52/(3d0*za12*zb43*zb53)
     .         - zb45*zb52*zb12/(2d0*za12*zb42*zb43*zb53)
     .         + za13**2*zb12**2/(2d0*za51*za12*zb42*s53)
     .         - 2d0*za43*za13*zb12/(za51*za12)*zL1(s42,s53,renscale2)
     .         - za42*za13**2*zb12**2/(2d0*za51*za12)
     .           * zL2(s42,s53,renscale2)

         zwt1 = zi*zb1gam012(j5,j1,j2,j4,j3,3,iflip)
     .         *( (zzLograt(renscale2,-s42))
     .          + (zzLograt(renscale2,-s51))
     .          + (zzLograt(renscale2,-s12))
     .          + (3d0/2d0))

         zwt2 = zi*zb1gam012(j5,j1,j2,j4,j3,3,iflip)*3d0

      endif

      if(iorder.eq.0)then
         zB1gaml12 = sign*zwt0
      elseif(iorder.eq.-1)then
         zB1gaml12 = sign*zwt1
      elseif(iorder.eq.-2)then
         zB1gaml12 = sign*zwt2
      endif

      return
      end

*************************************************************
*
*     function B_{12}^{1gamma,l} from [2]
*     with reversed gluon ordering
*

      function zB1gaml21(iq,ig1,ig2,iqb,iG,ihel,iflip,renscale2)
      use KinData_mod

      implicit real*8 (a-h,o-y)
      implicit complex*16(z)
      dimension ihel2(6)

      ihel2(1)=2
      ihel2(2)=1
      ihel2(3)=3
      ihel2(4)=5
      ihel2(5)=4
      ihel2(6)=6

      zB1gaml21 = zB1gaml12(iq,ig2,ig1,iqb,iG,ihel2(ihel),iflip,renscale2)

      return
      end

*************************************************************
*
*     function B_{12}^{1gamma,s} from [2]
*     see eq. 5.66
*     already coded in jet libB1.f as zBt3g1AAc but there with
*     correction factors which we include later
*
      function zB1gams12(iq,ig1,ig2,iqb,iG,ihel,iflip,renscale2)
      use KinData_mod

      implicit real*8 (a-h,o-y)
      implicit complex*16(z)
      dimension ihel2(6), iflip2(6,0:1)

*
*     We can in principle use zBt3g1AAc(iq,iG,ig1,ig2,iqb)
*     from jet libB1.f, however, there the correction factor
*     for DRED -> CDR is already included. If we go one layer
*     deeper we see that the primitive amplitudes zB123s and
*     zB123m from jet libB1.f already contain the correction
*     for c_gamma, which is why we cannot build a simple wrapper.
*     We therefore code them new as zB123sG and zB123mG, without
*     correction (see below).
*
*                          q  qb g1 g2 G              q  qb  g1 g2 G
*     ihel = 1   iflip =0  +  -| -  +  +   iflip = 1  -  +| +  -  -
*     ihel = 2   iflip =0  +  -| +  -  +   iflip = 1  -  +| -  +  -
*     ihel = 3   iflip =0  +  -| +  +  -   iflip = 1  -  +| -  -  +
*     ihel = 4   iflip =0  -  +| -  +  +   iflip = 1  +  -| +  -  -
*     ihel = 5   iflip =0  -  +| +  -  +   iflip = 1  +  -| -  +  -
*     ihel = 6   iflip =0  -  +| +  +  -   iflip = 1  +  -| -  -  +
*

      if(ihel.eq.1)then

        zwt=zB123sG(iq,ig1,ig2,iqb,iG,1,iflip,renscale2)
     .     +zB123sG(iq,ig1,iG,iqb,ig2,1,iflip,renscale2)
     .     +zB123sG(iq,iG,ig1,iqb,ig2,2,iflip,renscale2)
     .     +zB123mG(iq,ig1,ig2,iqb,iG,1,iflip,renscale2)
     .     +zB123mG(iq,ig1,iG,iqb,ig2,1,iflip,renscale2)
     .     +zB123mG(iq,iG,ig1,iqb,ig2,2,iflip,renscale2)

      elseif(ihel.eq.2)then

        zwt=zB123sG(iq,ig1,ig2,iqb,iG,2,iflip,renscale2)
     .     +zB123sG(iq,ig1,iG,iqb,ig2,3,iflip,renscale2)
     .     +zB123sG(iq,iG,ig1,iqb,ig2,3,iflip,renscale2)
     .     +zB123mG(iq,ig1,ig2,iqb,iG,2,iflip,renscale2)
     .     +zB123mG(iq,ig1,iG,iqb,ig2,3,iflip,renscale2)
     .     +zB123mG(iq,iG,ig1,iqb,ig2,3,iflip,renscale2)

      elseif(ihel.eq.3)then

        zwt=zB123sG(iq,ig1,ig2,iqb,iG,3,iflip,renscale2)
     .     +zB123sG(iq,ig1,iG,iqb,ig2,2,iflip,renscale2)
     .     +zB123sG(iq,iG,ig1,iqb,ig2,1,iflip,renscale2)
     .     +zB123mG(iq,ig1,ig2,iqb,iG,3,iflip,renscale2)
     .     +zB123mG(iq,ig1,iG,iqb,ig2,2,iflip,renscale2)
     .     +zB123mG(iq,iG,ig1,iqb,ig2,1,iflip,renscale2)

      elseif(ihel.eq.4)then

        zwt=zB123sG(iq,ig1,ig2,iqb,iG,4,iflip,renscale2)
     .     +zB123sG(iq,ig1,iG,iqb,ig2,4,iflip,renscale2)
     .     +zB123sG(iq,iG,ig1,iqb,ig2,5,iflip,renscale2)
     .     +zB123mG(iq,ig1,ig2,iqb,iG,4,iflip,renscale2)
     .     +zB123mG(iq,ig1,iG,iqb,ig2,4,iflip,renscale2)
     .     +zB123mG(iq,iG,ig1,iqb,ig2,5,iflip,renscale2)

      elseif(ihel.eq.5)then

        zwt=zB123sG(iq,ig1,ig2,iqb,iG,5,iflip,renscale2)
     .     +zB123sG(iq,ig1,iG,iqb,ig2,6,iflip,renscale2)
     .     +zB123sG(iq,iG,ig1,iqb,ig2,6,iflip,renscale2)
     .     +zB123mG(iq,ig1,ig2,iqb,iG,5,iflip,renscale2)
     .     +zB123mG(iq,ig1,iG,iqb,ig2,6,iflip,renscale2)
     .     +zB123mG(iq,iG,ig1,iqb,ig2,6,iflip,renscale2)

      elseif(ihel.eq.6)then

        zwt=zB123sG(iq,ig1,ig2,iqb,iG,6,iflip,renscale2)
     .     +zB123sG(iq,ig1,iG,iqb,ig2,5,iflip,renscale2)
     .     +zB123sG(iq,iG,ig1,iqb,ig2,4,iflip,renscale2)
     .     +zB123mG(iq,ig1,ig2,iqb,iG,6,iflip,renscale2)
     .     +zB123mG(iq,ig1,iG,iqb,ig2,5,iflip,renscale2)
     .     +zB123mG(iq,iG,ig1,iqb,ig2,4,iflip,renscale2)

      endif

      zB1gams12 = zwt

      return
      end

*************************************************************
*
*     function B_{12}^{1gamma,s} from [2]
*     with reversed gluon ordering
*

      function zB1gams21(iq,ig1,ig2,iqb,iG,ihel,iflip,renscale2)
      use KinData_mod

      implicit real*8 (a-h,o-y)
      implicit complex*16(z)
      dimension ihel2(6)

      ihel2(1)=2
      ihel2(2)=1
      ihel2(3)=3
      ihel2(4)=5
      ihel2(5)=4
      ihel2(6)=6

      zB1gams21 = zB1gams12(iq,ig2,ig1,iqb,iG,ihel2(ihel),iflip,renscale2)

      return
      end

*************************************************************
*
*     function B_{tr}^{1gamma} from [2]
*     see eq. 5.71
*

      function zB1gamtr(iq,ig1,ig2,iqb,iG,ihel,iflip,renscale2)
      use KinData_mod

      implicit real*8 (a-h,o-y)
      implicit complex*16(z)

      zB1gamtr = zb1qNF0(iq,ig1,ig2,iqb,iG,ihel,iflip,renscale2)
     .          + zb1qbNF0(iq,ig1,ig2,iqb,iG,ihel,iflip,renscale2)
     .          + zbb31(iq,ig1,ig2,iqb,iG,ihel,iflip,renscale2)

      return
      end

*************************************************************
*
*     function B_{12}^{1gamma,ext} in [1] hep-ph/9507442
*     see eqs. 13, 14, 15, but also eqs. 5.77, 5.81, 5.86 in [2]
*
      function zB1gamX12(iq,ig1,ig2,iqb,iG,ihel,iflip)
      use KinData_mod

      implicit real*8 (a-h,o-y)
      implicit complex*16(z)
      common/order/iorder

      zi=(0d0,1d0)

      if(ihel.le.3)then
         i1 = ig1
         i2 = ig2
         i3 = iG
         i4 = iqb
         i5 = iq
         sign = 1d0
      elseif(ihel.ge.4)then
         i1 = ig2
         i2 = ig1
         i3 = iG
         i4 = iq
         i5 = iqb
         sign = 1d0
      endif

      if(iflip.eq.0)then
         zb51=zB5(i5,i1)
         zb52=zB5(i5,i2)
         za12=zA5(i1,i2)
         zb43=zB5(i4,i3)
         zb53=zB5(i5,i3)
      elseif(iflip.eq.1)then
         zb51=-zA5(i5,i1)
         zb52=-zA5(i5,i2)
         za12=-zB5(i1,i2)
         zb43=-zA5(i4,i3)
         zb53=-zA5(i5,i3)
      endif

      if(ihel.eq.1.or.ihel.eq.5)then
         zwt0 = 0d0
      elseif(ihel.eq.2.or.ihel.eq.4)then
         zwt0 = 0d0
      elseif(ihel.eq.3.or.ihel.eq.6)then
         zwt0 = zb51*zb52/(3d0*za12*zb43*zb53)
      endif

      zwt1 = 0d0

      zwt2 = 0d0

      if(iorder.eq.0)then
         zB1gamX12 = sign*zwt0
      elseif(iorder.eq.-1)then
         zB1gamX12 = sign*zwt1
      elseif(iorder.eq.-2)then
         zB1gamX12 = sign*zwt2
      endif

      return
      end

*************************************************************
*
*     function B_{12}^{1gamma,ext} from [1] hep-ph/9507442
*     with reversed gluon ordering
*

      function zB1gamX21(iq,ig1,ig2,iqb,iG,ihel,iflip)
      use KinData_mod

      implicit real*8 (a-h,o-y)
      implicit complex*16(z)
      dimension ihel2(6)

      ihel2(1)=2
      ihel2(2)=1
      ihel2(3)=3
      ihel2(4)=5
      ihel2(5)=4
      ihel2(6)=6

      zB1gamX21 = zB1gamX12(iq,ig2,ig1,iqb,iG,ihel2(ihel),iflip)

      return
      end

*************************************************************
*
*     this function is closely related to B_{tr}^{1gamma,NF} in [2],
*     see eqs. 5.78, 5.82, 5.85
*

      function zBNFh(iq,ig1,ig2,iqb,iG,ihel,iflip,renscale2)
      use KinData_mod

      implicit real*8 (a-h,o-y)
      implicit complex*16(z)

      if(ihel.eq.1)then
         zwt = zBNF(iq,ig1,ig2,iqb,iG,1,iflip,renscale2)
     .       + zBNF(iq,ig1,iG,iqb,ig2,1,iflip,renscale2)
      elseif(ihel.eq.2)then
         zwt = zBNF(iq,ig1,ig2,iqb,iG,2,iflip,renscale2)
     .       + zBNF(iq,iG,ig2,iqb,ig1,2,iflip,renscale2)
      elseif(ihel.eq.3)then
         zwt = zBNF(iq,ig1,ig2,iqb,iG,3,iflip,renscale2)
     .       + zBNF(iq,ig2,ig1,iqb,iG,3,iflip,renscale2)
      elseif(ihel.eq.4)then
         zwt = zBNF(iq,ig1,ig2,iqb,iG,4,iflip,renscale2)
     .       + zBNF(iq,ig1,iG,iqb,ig2,4,iflip,renscale2)
      elseif(ihel.eq.5)then
         zwt = zBNF(iq,ig1,ig2,iqb,iG,5,iflip,renscale2)
     .       + zBNF(iq,iG,ig2,iqb,ig1,5,iflip,renscale2)
      elseif(ihel.eq.6)then
         zwt = zBNF(iq,ig1,ig2,iqb,iG,6,iflip,renscale2)
     .       + zBNF(iq,ig2,ig1,iqb,iG,6,iflip,renscale2)
      endif

      zBNFh = zwt

      return
      end

************************************************************************
*     This is B_123^m from [2] (5.13,5.17,5.21),
*     initially coded as in jet libB1.f, but without any correcion-factors
*     (for c_gamma etc.)
*     now it is coded with consistent helicity (ihel, iflip) notation
*
      function zB123mG(iq,ig1,ig2,iqb,iG,ihel,iflip,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /order/iorder
      parameter(pi=3.141592653589793238d0)

        zi = (0d0,1d0)
        zone = (1d0,0d0)

*
*                          q  qb g1 g2 G              q  qb  g1 g2 G
*     ihel = 1   iflip =0  +  -| -  +  +   iflip = 1  -  +| +  -  -
*     ihel = 2   iflip =0  +  -| +  -  +   iflip = 1  -  +| -  +  -
*     ihel = 3   iflip =0  +  -| +  +  -   iflip = 1  -  +| -  -  +
*     ihel = 4   iflip =0  -  +| -  +  +   iflip = 1  +  -| +  -  -
*     ihel = 5   iflip =0  -  +| +  -  +   iflip = 1  +  -| -  +  -
*     ihel = 6   iflip =0  -  +| +  +  -   iflip = 1  +  -| -  -  +
*

      if(ihel.le.3)then
         j1 = iq
         j2 = ig1
         j3 = ig2
         j4 = iG
         j5 = iqb
         sign = 1d0
      elseif(ihel.ge.4)then
         j1 = iqb
         j2 = iG
         j3 = ig2
         j4 = ig1
         j5 = iq
         sign = 1d0
      endif

      if(iflip.eq.0)then
        za12=zA5(j1,j2)
        za21=zA5(j2,j1)
        za13=zA5(j1,j3)
        za31=zA5(j3,j1)
        za14=zA5(j1,j4)
        za41=zA5(j4,j1)
        za15=zA5(j1,j5)
        za51=zA5(j5,j1)
        za23=zA5(j2,j3)
        za32=zA5(j3,j2)
        za24=zA5(j2,j4)
        za42=zA5(j4,j2)
        za25=zA5(j2,j5)
        za52=zA5(j5,j2)
        za34=zA5(j3,j4)
        za43=zA5(j4,j3)
        za35=zA5(j3,j5)
        za53=zA5(j5,j3)
        za45=zA5(j4,j5)
        za54=zA5(j5,j4)

        zb12=zB5(j1,j2)
        zb21=zB5(j2,j1)
        zb13=zB5(j1,j3)
        zb31=zB5(j3,j1)
        zb14=zB5(j1,j4)
        zb41=zB5(j4,j1)
        zb15=zB5(j1,j5)
        zb51=zB5(j5,j1)
        zb23=zB5(j2,j3)
        zb32=zB5(j3,j2)
        zb24=zB5(j2,j4)
        zb42=zB5(j4,j2)
        zb25=zB5(j2,j5)
        zb52=zB5(j5,j2)
        zb34=zB5(j3,j4)
        zb43=zB5(j4,j3)
        zb35=zB5(j3,j5)
        zb53=zB5(j5,j3)
        zb45=zB5(j4,j5)
        zb54=zB5(j5,j4)

      elseif(iflip.eq.1)then
        za12=-zB5(j1,j2)
        za21=-zB5(j2,j1)
        za13=-zB5(j1,j3)
        za31=-zB5(j3,j1)
        za14=-zB5(j1,j4)
        za41=-zB5(j4,j1)
        za15=-zB5(j1,j5)
        za51=-zB5(j5,j1)
        za23=-zB5(j2,j3)
        za32=-zB5(j3,j2)
        za24=-zB5(j2,j4)
        za42=-zB5(j4,j2)
        za25=-zB5(j2,j5)
        za52=-zB5(j5,j2)
        za34=-zB5(j3,j4)
        za43=-zB5(j4,j3)
        za35=-zB5(j3,j5)
        za53=-zB5(j5,j3)
        za45=-zB5(j4,j5)
        za54=-zB5(j5,j4)

        zb12=-zA5(j1,j2)
        zb21=-zA5(j2,j1)
        zb13=-zA5(j1,j3)
        zb31=-zA5(j3,j1)
        zb14=-zA5(j1,j4)
        zb41=-zA5(j4,j1)
        zb15=-zA5(j1,j5)
        zb51=-zA5(j5,j1)
        zb23=-zA5(j2,j3)
        zb32=-zA5(j3,j2)
        zb24=-zA5(j2,j4)
        zb42=-zA5(j4,j2)
        zb25=-zA5(j2,j5)
        zb52=-zA5(j5,j2)
        zb34=-zA5(j3,j4)
        zb43=-zA5(j4,j3)
        zb35=-zA5(j3,j5)
        zb53=-zA5(j5,j3)
        zb45=-zA5(j4,j5)
        zb54=-zA5(j5,j4)
      endif

        s12=s5(j1,j2)
        s13=s5(j1,j3)
        s14=s5(j1,j4)
        s15=s5(j1,j5)
        s21=s5(j2,j1)
        s23=s5(j2,j3)
        s24=s5(j2,j4)
        s25=s5(j2,j5)
        s31=s5(j3,j1)
        s32=s5(j3,j2)
        s34=s5(j3,j4)
        s35=s5(j3,j5)
        s41=s5(j4,j1)
        s42=s5(j4,j2)
        s43=s5(j4,j3)
        s45=s5(j4,j5)
        s51=s5(j5,j1)
        s52=s5(j5,j2)
        s53=s5(j5,j3)
        s54=s5(j5,j4)

      if(ihel.eq.1.or.ihel.eq.6)then
        ztree=za12*za52**(3)/(za12*za23*za34*za45*za51)

!     original amplitude in [2]
!        ze0 = -1.d0*za52**(3)/(2.d0*za51*za54*za23*za34)
!     .      + za52*za53*za12*zb13/(2.d0*za54*za13*za34*s54)
!     .      + za52*za12*za12*zb14/(2.d0*za13*za14*za23*s54)
!     .      - za12*za12*zb13*zb14/(2.d0*za14*za34*zb54*s23)
!     .      - za52*za54*za12*zb34/(2.d0*za51*za14*za34*s23)
!     .      - za53*zb13*zb13*zb34/(2.d0*za34*zb12*zb23*s54)
!     .      - za52**(3)*zL0(s51,s54,renscale2)/(za51*za54*za23*za34)
!     .      - za52*za52*za12*zL0(s23,s51,renscale2)/(2.d0*za51*za14*za23*za34)
!     .      - za52*za52*za53*za12
!     .      *zL0(s23,s54,renscale2)/(2.d0*za51*za54*za13*za23*za34)
!     .      + za52*za12*za24*zb14*zL1(s51,s23,renscale2)/(za14*za23*za34)
!     .      - za51*za51*(za24**(3))*zb14
!     .      *zL1(s51,s23,renscale2)/(za54*za14*za14*za23*za34)
!     .      + za52*za52*zb34*zL1(s51,s23,renscale2)/(za51*za34)
!     .      + za52*za53*za12*zb13*zL1(s54,s12,renscale2)/(za54*za13*za34)
!     .      + za51*za53*za12*za23*zb13
!     .      *zL1(s54,s12,renscale2)/(za54*za13*za13*za34)
!     .      + za52*za12*za12*zb14*zL1(s54,s23,renscale2)/(za13*za14*za23)
!     .      - za51*za51*za23*za23*zb13
!     .      *zL1(s54,s23,renscale2)/(za54*za13*za13*za34)
!     .      + za51*za12*za24*za24*zb14
!     .      *zL1(s54,s23,renscale2)/(za14*za14*za23*za34)
!     .      + za54*za23*za24*zb34*zb34
!     .      *zL2(s51,s23,renscale2)/(2.d0*za14*za34)
!     .      + za53*za53*za12*za12*zb13*zb13
!     .      *zL2(s12,s54,renscale2)/(2.d0*za54*za13*za34)
!     .      - za54*((za12)**(3))*zb14*zb14
!     .      *zL2(s23,s54,renscale2)/(2.d0*za13*za14*za23)

!     amplitude corrected to remove angular instability in s54
        ztmp = -1.d0*za52**(3)/(2.d0*za51*za54*za23*za34)
     .      + za52*za12*za12*zb14/(2.d0*za13*za14*za23*s54)
     .      - za12*za12*zb13*zb14/(2.d0*za14*za34*zb54*s23)
     .      - za52*za54*za12*zb34/(2.d0*za51*za14*za34*s23)
     .      - za53*zb13*zb13*zb34/(2.d0*za34*zb12*zb23*s54)
     .      - za52**(3)*zL0(s51,s54,renscale2)/(za51*za54*za23*za34)
     .      - za52*za52*za12*zL0(s23,s51,renscale2)/(2.d0*za51*za14*za23*za34)
     .      - za52*za52*za53*za12
     .      *zL0(s23,s54,renscale2)/(2.d0*za51*za54*za13*za23*za34)
     .      + za52*za12*za24*zb14*zL1(s51,s23,renscale2)/(za14*za23*za34)
     .      - za51*za51*(za24**(3))*zb14
     .      *zL1(s51,s23,renscale2)/(za54*za14*za14*za23*za34)
     .      + za52*za52*zb34*zL1(s51,s23,renscale2)/(za51*za34)
     .      + za52*za53*za12*zb13*zL1(s54,s12,renscale2)/(za54*za13*za34)
     .      + za51*za53*za12*za23*zb13
     .      *zL1(s54,s12,renscale2)/(za54*za13*za13*za34)
     .      + za52*za12*za12*zb14*zL1(s54,s23,renscale2)/(za13*za14*za23)
     .      - za51*za51*za23*za23*zb13
     .      *zL1(s54,s23,renscale2)/(za54*za13*za13*za34)
     .      + za51*za12*za24*za24*zb14
     .      *zL1(s54,s23,renscale2)/(za14*za14*za23*za34)
     .      + za54*za23*za24*zb34*zb34
     .      *zL2(s51,s23,renscale2)/(2.d0*za14*za34)
     .      - za54*((za12)**(3))*zb14*zb14
     .      *zL2(s23,s54,renscale2)/(2.d0*za13*za14*za23)
     .  +za53*za12*zb13/(2.d0*za54*za13*za34)*(-za53*za12*zb13*zL2(s54,s12,renscale2)
     .-zb41/zb45/zb21)

        ze0 = ztmp
        ze1 = 0d0
        ze2 = 0d0

      elseif(ihel.eq.2.or.ihel.eq.5)then

        ztree=za13*za53**(3)/(za12*za23*za34*za45*za51)

        ze0 = za52*za34*zb14*zb24**(2)/(2.d0*za24*zb34*s51*s23)
     .  + za53*za53*za13*zb14/(2.d0*za54*za12*za23*s51)
     .  - za13*za13*zb14*zb14/(2.d0*za12*za14*za23*zb51*zb34)
     .  + za13*zb52*zb14*zb14*zb24/(2.d0*za14*zb51*zb54*zb34*s23)
     .  - za52*zb24*zb24*s54/(2.d0*za12*za24*zb23*zb34*s51)
     .  + za53*za23*zb24*zL1(s51,s23,renscale2)/(za12*za24)
     .  +  za53*za23*zb24*zL1(s51,s34,renscale2)/(za12*za24)
     .  + za52*za23*za34*zb24*zL1(s51,s23,renscale2)/(za12*za24*za24)
     .  + za52*za23*za34*zb24*zL1(s51,s34,renscale2)/(za12*za24*za24)
     .  + za53*za13*za13*zb14*zL1(s54,s23,renscale2)/(za12*za14*za23)
     .  + za51*za13*za13*za34*zb14
     .      *zL1(s54,s23,renscale2)/(za12*za14*za14*za23)
     .  - za54*za23*za23*zb24*zb24*zL2(s51,s23,renscale2)/(2.d0*za12*za24)
     .  - za51*za51*za13*za34*za34*zb14*zb14
     .      *zL2(s23,s51,renscale2)/(2.d0*za54*za12*za14*za23)
     .  - za54*za13*za13*za13*zb14*zb14
     .      *zL2(s23,s54,renscale2)/(2.d0*za12*za14*za23)
     .  -za52*za23*za34*zb24*zb24*zL2(s34,s51,renscale2)/(2.d0*za12*za24)

        ze1 = 0d0
        ze2 = 0d0

      elseif(ihel.eq.3.or.ihel.eq.4)then

        ztree=za14*za54**(3)/(za12*za23*za34*za45*za51)

        ze0 = -1.d0*zb13*zb23/(2.d0*za23*zb54*zb34)
     . - za54*za24*zb23*zL1(s51,s34,renscale2)/(za12*za23)
     . + za52*za24*za34*zb23*zb23*zL2(s51,s34,renscale2)/(2.d0*za12*za23)

        ze1 = 0d0
        ze2 = 0d0

      endif

      if(iorder.eq.-2)then
          zB123mG=sign*ze2
      elseif(iorder.eq.-1)then
          zB123mG=sign*ze1
      elseif(iorder.eq.0)then
          zB123mG=sign*ze0
      endif

      return
      end

************************************************************************
*
*     This is B_123^s from [2] (5.11,5.15,5.19),
*     initially coded as in jet libB1.f, but without any correcion-factors
*     (for c_gamma etc.)
*     now it is coded with consistent helicity (ihel, iflip) notation
*
      function zB123sG(iq,ig1,ig2,iqb,iG,ihel,iflip,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /order/iorder
      parameter(pi=3.141592653589793238d0)

        zi = (0d0,1d0)
        zone = (1d0,0d0)

*
*                          q  qb g1 g2 G              q  qb  g1 g2 G
*     ihel = 1   iflip =0  +  -| -  +  +   iflip = 1  -  +| +  -  -
*     ihel = 2   iflip =0  +  -| +  -  +   iflip = 1  -  +| -  +  -
*     ihel = 3   iflip =0  +  -| +  +  -   iflip = 1  -  +| -  -  +
*     ihel = 4   iflip =0  -  +| -  +  +   iflip = 1  +  -| +  -  -
*     ihel = 5   iflip =0  -  +| +  -  +   iflip = 1  +  -| -  +  -
*     ihel = 6   iflip =0  -  +| +  +  -   iflip = 1  +  -| -  -  +
*

      if(ihel.le.3)then
         j1 = iq
         j2 = ig1
         j3 = ig2
         j4 = iG
         j5 = iqb
         sign = 1d0
      elseif(ihel.ge.4)then
         j1 = iqb
         j2 = iG
         j3 = ig2
         j4 = ig1
         j5 = iq
         sign = 1d0
      endif

      if(iflip.eq.0)then
        za12=zA5(j1,j2)
        za21=zA5(j2,j1)
        za13=zA5(j1,j3)
        za31=zA5(j3,j1)
        za14=zA5(j1,j4)
        za41=zA5(j4,j1)
        za15=zA5(j1,j5)
        za51=zA5(j5,j1)
        za23=zA5(j2,j3)
        za32=zA5(j3,j2)
        za24=zA5(j2,j4)
        za42=zA5(j4,j2)
        za25=zA5(j2,j5)
        za52=zA5(j5,j2)
        za34=zA5(j3,j4)
        za43=zA5(j4,j3)
        za35=zA5(j3,j5)
        za53=zA5(j5,j3)
        za45=zA5(j4,j5)
        za54=zA5(j5,j4)

        zb12=zB5(j1,j2)
        zb21=zB5(j2,j1)
        zb13=zB5(j1,j3)
        zb31=zB5(j3,j1)
        zb14=zB5(j1,j4)
        zb41=zB5(j4,j1)
        zb15=zB5(j1,j5)
        zb51=zB5(j5,j1)
        zb23=zB5(j2,j3)
        zb32=zB5(j3,j2)
        zb24=zB5(j2,j4)
        zb42=zB5(j4,j2)
        zb25=zB5(j2,j5)
        zb52=zB5(j5,j2)
        zb34=zB5(j3,j4)
        zb43=zB5(j4,j3)
        zb35=zB5(j3,j5)
        zb53=zB5(j5,j3)
        zb45=zB5(j4,j5)
        zb54=zB5(j5,j4)

      elseif(iflip.eq.1)then
        za12=-zB5(j1,j2)
        za21=-zB5(j2,j1)
        za13=-zB5(j1,j3)
        za31=-zB5(j3,j1)
        za14=-zB5(j1,j4)
        za41=-zB5(j4,j1)
        za15=-zB5(j1,j5)
        za51=-zB5(j5,j1)
        za23=-zB5(j2,j3)
        za32=-zB5(j3,j2)
        za24=-zB5(j2,j4)
        za42=-zB5(j4,j2)
        za25=-zB5(j2,j5)
        za52=-zB5(j5,j2)
        za34=-zB5(j3,j4)
        za43=-zB5(j4,j3)
        za35=-zB5(j3,j5)
        za53=-zB5(j5,j3)
        za45=-zB5(j4,j5)
        za54=-zB5(j5,j4)

        zb12=-zA5(j1,j2)
        zb21=-zA5(j2,j1)
        zb13=-zA5(j1,j3)
        zb31=-zA5(j3,j1)
        zb14=-zA5(j1,j4)
        zb41=-zA5(j4,j1)
        zb15=-zA5(j1,j5)
        zb51=-zA5(j5,j1)
        zb23=-zA5(j2,j3)
        zb32=-zA5(j3,j2)
        zb24=-zA5(j2,j4)
        zb42=-zA5(j4,j2)
        zb25=-zA5(j2,j5)
        zb52=-zA5(j5,j2)
        zb34=-zA5(j3,j4)
        zb43=-zA5(j4,j3)
        zb35=-zA5(j3,j5)
        zb53=-zA5(j5,j3)
        zb45=-zA5(j4,j5)
        zb54=-zA5(j5,j4)
      endif

        s12=s5(j1,j2)
        s13=s5(j1,j3)
        s14=s5(j1,j4)
        s15=s5(j1,j5)
        s21=s5(j2,j1)
        s23=s5(j2,j3)
        s24=s5(j2,j4)
        s25=s5(j2,j5)
        s31=s5(j3,j1)
        s32=s5(j3,j2)
        s34=s5(j3,j4)
        s35=s5(j3,j5)
        s41=s5(j4,j1)
        s42=s5(j4,j2)
        s43=s5(j4,j3)
        s45=s5(j4,j5)
        s51=s5(j5,j1)
        s52=s5(j5,j2)
        s53=s5(j5,j3)
        s54=s5(j5,j4)

      if(ihel.eq.1.or.ihel.eq.6)then

        ztree=za12*za52**(3)/(za12*za23*za34*za45*za51)

        ze0 = -ztree
     . *( 0.5d0*(zzLograt(renscale2,-s15))**(2)
     .  + (3.d0/2.d0)*(zzLograt(renscale2,-s45))
     .  + (7.d0/2.d0)*zone
     .  + (za51*za42)**(3)/(za52*za41)**(3)*zzCFGJ(s14,s15,s54)
     .  - ((za53*za12)/(za52*za13)-1d0*zone
     .      +(za53*za12)/(za52*za13)*(za51*za32)/(za52*za31))
     .  *(za51*za32)/(za52*za31)*zzCFGJ(s13,s12,s23))

        ze1 = -ztree
     . *( zzLograt(renscale2,-s15) + (3d0/2d0)*zone)

        ze2 = -ztree*1.d0

      elseif(ihel.eq.2.or.ihel.eq.5)then

        ztree=za13*za53**(3)/(za12*za23*za34*za45*za51)

        ze0 = -ztree
     .  *(0.5d0*(zzLograt(renscale2,-s15))**(2)
     .  + (3.d0/2.d0)*(zzLograt(renscale2,-s15))
     .  + (7.d0/2.d0)*zone)

     .  - za51**2*za13*za34**2*zb14*zL1(s51,s23,renscale2)
     .     /(za54*za12*za14**(2)*za23)

     .  - ztree*za51**(3)*za43**(3)
     .       *zzCFGJ(s14,s15,s54)/(za53**(3)*za41**(3))
     .  - ztree*za51*za32
     .       *zzCFGJ(s52,s51,s12)/(za52*za31)
     .  - ztree*za51*za32/(za52*za31)*(za52*za43/za53/za42)**(3)
     .       *zzCFGJ(s24,s23,s34)

        ze1 = -ztree
     . *( zzLograt(renscale2,-s15) + (3.d0/2.d0)*zone)

        ze2 = -ztree*1.d0

      elseif(ihel.eq.3.or.ihel.eq.4)then

        ztree=za14*za54**(3)/(za12*za23*za34*za45*za51)

        ze0  = -ztree
     .*(  0.5d0*(zzLograt(renscale2,-s15))**(2)
     .  + 3.d0/2.d0*(zzLograt(renscale2,-s15))
     .  + (7.d0/2.d0)*zone
     . + za51*za42*zzCFGJ(s52,s51,s12)/(za52*za41)
     . + za51*za43*zzCFGJ(s53,s54,s43)/(za53*za41) )

        ze1  = -ztree
     .  *(zzLograt(renscale2,-s15) + (3.d0/2.d0)*zone)

        ze2  = -ztree*1.d0

      endif

      if(iorder.eq.-2)then
          zB123sG=sign*ze2
      elseif(iorder.eq.-1)then
          zB123sG=sign*ze1
      elseif(iorder.eq.0)then
          zB123sG=sign*ze0
      endif

      return
      end

*************************************************************
*
*     function b_q^(1) (5.22) from [2], with NF=0 and divided by i*c_gamma, corresponds to zB3g1AAq in jet libB1.f
*
      function zb1qNF0(iq,ig1,ig2,iqb,iG,ihel,iflip,renscale2)
      use KinData_mod

      implicit real*8 (a-h,o-y)
      implicit complex*16(z)

      zi = (0d0,1d0)

      if(ihel.eq.1)then
         i1=ig1
         i2=ig2
         i3=iG
         i4=iqb
         i5=iq
         sign = 1d0
      elseif(ihel.eq.2)then
         i1=ig2
         i2=iG
         i3=ig1
         i4=iqb
         i5=iq
         sign = 1d0
      elseif(ihel.eq.3)then
         i1=iG
         i2=ig1
         i3=ig2
         i4=iqb
         i5=iq
         sign = 1d0
      elseif(ihel.eq.4)then
         i1=ig1
         i2=iG
         i3=ig2
         i4=iq
         i5=iqb
         sign = 1d0
      elseif(ihel.eq.5)then
         i1=ig2
         i2=ig1
         i3=iG
         i4=iq
         i5=iqb
         sign = 1d0
      elseif(ihel.eq.6)then
         i1=iG
         i2=ig2
         i3=ig1
         i4=iq
         i5=iqb
         sign = 1d0
      endif

      zwt = -zi*zBNF(i5,i1,i2,i4,i3,1,iflip,renscale2)
     .      + zb0123(i5,i1,i2,i4,i3,1,iflip)
     .       * zBq123(i5,i1,i2,i4,i3,iflip,renscale2)
     .      + zb0231(i5,i1,i2,i4,i3,1,iflip)
     .       * zBq231(i5,i1,i2,i4,i3,iflip,renscale2)
     .      + zb0312(i5,i1,i2,i4,i3,1,iflip)
     .       * zBq312(i5,i1,i2,i4,i3,iflip,renscale2)

      zb1qNF0 = -sign*zi*zwt

      return
      end

*************************************************************
*
*     function b_qb^(1) from [2], with NF=0 and divided by i*c_gamma
*     derived using table 5.2
*     corresponds to zB3g1AAqb in jet libB1.f
*
      function zb1qbNF0(iq,ig1,ig2,iqb,iG,ihel,iflip,renscale2)
      use KinData_mod

      implicit real*8 (a-h,o-y)
      implicit complex*16(z)
      dimension ihel2(6)

      ihel2(1)=4
      ihel2(2)=5
      ihel2(3)=6
      ihel2(4)=1
      ihel2(5)=2
      ihel2(6)=3

      zb1qbNF0 = zb1qNF0(iqb,ig1,ig2,iq,iG,ihel2(ihel),iflip,renscale2)

      return
      end

*************************************************************
*
*     function b_1^(1) (5.23) from [2],
*     divided by i*c_gamma, corresponds to zB3g1AAd1 in jet libB1.f
*

      function zbb11(iq,ig1,ig2,iqb,iG,ihel,iflip,renscale2)
      use KinData_mod

      implicit real*8 (a-h,o-y)
      implicit complex*16(z)

      zi=(0d0,1d0)

      if(ihel.eq.1)then
         j1=ig1
         j2=ig2
         j3=iG
         j4=iqb
         j5=iq
         ihel2=1
         sign = 1d0
      elseif(ihel.eq.2)then
         j1=ig1
         j2=ig2
         j3=iG
         j4=iqb
         j5=iq
         ihel2=2
         sign = 1d0
      elseif(ihel.eq.3)then
         j1=ig1
         j2=iG
         j3=ig2
         j4=iqb
         j5=iq
         ihel2=2
         sign = 1d0
      elseif(ihel.eq.4)then
         j1=ig1
         j2=ig2
         j3=iG
         j4=iq
         j5=iqb
         ihel2=1
         sign = 1d0
      elseif(ihel.eq.5)then
         j1=ig1
         j2=ig2
         j3=iG
         j4=iq
         j5=iqb
         ihel2=2
         sign = 1d0
      elseif(ihel.eq.6)then
         j1=ig1
         j2=iG
         j3=ig2
         j4=iq
         j5=iqb
         ihel2=2
         sign = 1d0
      endif

      zwt = 2d0*zi*zBNF(j5,j1,j2,j4,j3,ihel2,iflip,renscale2)
     .     + zb0123(j5,j1,j2,j4,j3,ihel2,iflip)
     .      *zB1123(j5,j1,j2,j4,j3,ihel2,iflip,renscale2)
     .     + zb0132(j5,j1,j2,j4,j3,ihel2,iflip)
     .      *zB1132(j5,j1,j2,j4,j3,ihel2,iflip,renscale2)
     .     + zb0231(j5,j1,j2,j4,j3,ihel2,iflip)
     .      *zB1231(j5,j1,j2,j4,j3,ihel2,iflip,renscale2)
     .     + zb0321(j5,j1,j2,j4,j3,ihel2,iflip)
     .      *zB1321(j5,j1,j2,j4,j3,ihel2,iflip,renscale2)

      zbb11 = -sign*zi*zwt

      return
      end

*************************************************************
*
*     function b_3^(1) from [2], with NF=0 and divided by i*c_gamma
*     derived using table 5.2
*     corresponds to zB3g1AAd3 in jet libB1.f
*

      function zbb31(iq,ig1,ig2,iqb,iG,ihel,iflip,renscale2)
      use KinData_mod

      implicit real*8 (a-h,o-y)
      implicit complex*16(z)
      dimension ihel2(6)

      ihel2(1)=3
      ihel2(2)=2
      ihel2(3)=1
      ihel2(4)=6
      ihel2(5)=5
      ihel2(6)=4

      zbb31 = zbb11(iq,iG,ig2,iqb,ig1,ihel2(ihel),iflip,renscale2)

      return
      end

*************************************************************
*
*     function B^{NF} from [2],
*     see eqs. 5.25, 5.26
*     corresponds to zBnfmpp and zBnfpmp,
*     or better zBh3g1AAq and zBh3g1AAqb in jet libB1.f
*

      function zBNF(iq,ig1,ig2,iqb,iG,ihel,iflip,renscale2)
      use KinData_mod

      implicit real*8 (a-h,o-y)
      implicit complex*16(z)
      common/order/iorder

      if(ihel.eq.1)then
         j1=ig1
         j2=ig2
         j3=iG
         j4=iqb
         j5=iq
         sign = 1d0
      elseif(ihel.eq.2)then
c     Note that this is an argument ordering equivalent to the
c     exchange 1<->2 given in (5.26) in [2],
c     because B^{NF} is symmetric wrt. swaping same-helicity
c     "gluons".
         j1=ig2
         j2=iG
         j3=ig1
         j4=iqb
         j5=iq
         sign = 1d0
      elseif(ihel.eq.3)then
         j1=iG
         j2=ig1
         j3=ig2
         j4=iqb
         j5=iq
         sign = 1d0
      elseif(ihel.eq.4)then
         j1=ig1
         j2=ig2
         j3=iG
         j4=iq
         j5=iqb
         sign = 1d0
      elseif(ihel.eq.5)then
         j1=ig2
         j2=iG
         j3=ig1
         j4=iq
         j5=iqb
         sign = 1d0
      elseif(ihel.eq.6)then
         j1=iG
         j2=ig1
         j3=ig2
         j4=iq
         j5=iqb
         sign = 1d0
      endif

      if(iflip.eq.0)then
        za12=zA5(j1,j2)
        za21=zA5(j2,j1)
        za13=zA5(j1,j3)
        za31=zA5(j3,j1)
        za14=zA5(j1,j4)
        za41=zA5(j4,j1)
        za15=zA5(j1,j5)
        za51=zA5(j5,j1)
        za23=zA5(j2,j3)
        za32=zA5(j3,j2)
        za24=zA5(j2,j4)
        za42=zA5(j4,j2)
        za25=zA5(j2,j5)
        za52=zA5(j5,j2)
        za34=zA5(j3,j4)
        za43=zA5(j4,j3)
        za35=zA5(j3,j5)
        za53=zA5(j5,j3)
        za45=zA5(j4,j5)
        za54=zA5(j5,j4)

        zb12=zB5(j1,j2)
        zb21=zB5(j2,j1)
        zb13=zB5(j1,j3)
        zb31=zB5(j3,j1)
        zb14=zB5(j1,j4)
        zb41=zB5(j4,j1)
        zb15=zB5(j1,j5)
        zb51=zB5(j5,j1)
        zb23=zB5(j2,j3)
        zb32=zB5(j3,j2)
        zb24=zB5(j2,j4)
        zb42=zB5(j4,j2)
        zb25=zB5(j2,j5)
        zb52=zB5(j5,j2)
        zb34=zB5(j3,j4)
        zb43=zB5(j4,j3)
        zb35=zB5(j3,j5)
        zb53=zB5(j5,j3)
        zb45=zB5(j4,j5)
        zb54=zB5(j5,j4)

      elseif(iflip.eq.1)then
        za12=-zB5(j1,j2)
        za21=-zB5(j2,j1)
        za13=-zB5(j1,j3)
        za31=-zB5(j3,j1)
        za14=-zB5(j1,j4)
        za41=-zB5(j4,j1)
        za15=-zB5(j1,j5)
        za51=-zB5(j5,j1)
        za23=-zB5(j2,j3)
        za32=-zB5(j3,j2)
        za24=-zB5(j2,j4)
        za42=-zB5(j4,j2)
        za25=-zB5(j2,j5)
        za52=-zB5(j5,j2)
        za34=-zB5(j3,j4)
        za43=-zB5(j4,j3)
        za35=-zB5(j3,j5)
        za53=-zB5(j5,j3)
        za45=-zB5(j4,j5)
        za54=-zB5(j5,j4)

        zb12=-zA5(j1,j2)
        zb21=-zA5(j2,j1)
        zb13=-zA5(j1,j3)
        zb31=-zA5(j3,j1)
        zb14=-zA5(j1,j4)
        zb41=-zA5(j4,j1)
        zb15=-zA5(j1,j5)
        zb51=-zA5(j5,j1)
        zb23=-zA5(j2,j3)
        zb32=-zA5(j3,j2)
        zb24=-zA5(j2,j4)
        zb42=-zA5(j4,j2)
        zb25=-zA5(j2,j5)
        zb52=-zA5(j5,j2)
        zb34=-zA5(j3,j4)
        zb43=-zA5(j4,j3)
        zb35=-zA5(j3,j5)
        zb53=-zA5(j5,j3)
        zb45=-zA5(j4,j5)
        zb54=-zA5(j5,j4)
      endif

        s12=s5(j1,j2)
        s13=s5(j1,j3)
        s14=s5(j1,j4)
        s15=s5(j1,j5)
        s21=s5(j2,j1)
        s23=s5(j2,j3)
        s24=s5(j2,j4)
        s25=s5(j2,j5)
        s31=s5(j3,j1)
        s32=s5(j3,j2)
        s34=s5(j3,j4)
        s35=s5(j3,j5)
        s41=s5(j4,j1)
        s42=s5(j4,j2)
        s43=s5(j4,j3)
        s45=s5(j4,j5)
        s51=s5(j5,j1)
        s52=s5(j5,j2)
        s53=s5(j5,j3)
        s54=s5(j5,j4)

         zwt0 = - za41*zb52*zb23/(za23*zb12*s45)
     .         + za42*za13*zb51*zb23**2/(za23**2*zb12*zb13*s45)
     .         - 2d0*za42*za43*za12*za13*zb23/(za45*za23**3)
     .           *( zL1(s45,s12,renscale2) + zL1(s45,s13,renscale2))
     .         + 2d0*za42*za43*za12*za13/(za45*za23**4)*zzCFGJ(s23,s21,s13)
     .         + za41**2/(za45*za23**2)*zzCFGJ(s23,s21,s13)
     .         - za43**2*za12**2*zb23**2/(za45*za23**2)
     .           * zL2(s45,s12,renscale2)
     .         - za42**2*za13**2*zb23**2/(za45*za23**2)
     .           * zL2(s45,s13,renscale2)

         zwt1 = 0d0

         zwt2 = 0d0

      if(iorder.eq.0)then
         zBNF = sign*zwt0
      elseif(iorder.eq.-1)then
         zBNF = sign*zwt1
      elseif(iorder.eq.-2)then
         zBNF = sign*zwt2
      endif

      return
      end

*************************************************************
*
*     This is the the 2q3g tree amplitude b_{123}^{(0)},
*     see eq. 5.8 in [2].
*     It has already been coded in jet libB1.f as zB123tree.
*
      function zb0123(iq,ig1,ig2,iqb,iG,ihel,iflip)
      use KinData_mod

      implicit real*8 (a-h,o-y)
      implicit complex*16(z)

      zi = (0d0,1d0)

*
*                          q  qb g1 g2 G              q  qb  g1 g2 G
*     ihel = 1   iflip =0  +  -| -  +  +   iflip = 1  -  +| +  -  -
*     ihel = 2   iflip =0  +  -| +  -  +   iflip = 1  -  +| -  +  -
*     ihel = 3   iflip =0  +  -| +  +  -   iflip = 1  -  +| -  -  +
*     ihel = 4   iflip =0  -  +| -  +  +   iflip = 1  +  -| +  -  -
*     ihel = 5   iflip =0  -  +| +  -  +   iflip = 1  +  -| -  +  -
*     ihel = 6   iflip =0  -  +| +  +  -   iflip = 1  +  -| -  -  +
*
         i1 = iq
         i2 = ig1
         i3 = ig2
         i4 = iG
         i5 = iqb

      if(iflip.eq.0)then
         za12 = zA5(i1,i2)
         za23 = zA5(i2,i3)
         za34 = zA5(i3,i4)
         za45 = zA5(i4,i5)
         za51 = zA5(i5,i1)

         za13 = zA5(i1,i3)
         za14 = zA5(i1,i4)
         za52 = zA5(i5,i2)
         za53 = zA5(i5,i3)
         za54 = zA5(i5,i4)
      elseif(iflip.eq.1)then
         za12 = -zB5(i1,i2)
         za23 = -zB5(i2,i3)
         za34 = -zB5(i3,i4)
         za45 = -zB5(i4,i5)
         za51 = -zB5(i5,i1)

         za13 = -zB5(i1,i3)
         za14 = -zB5(i1,i4)
         za52 = -zB5(i5,i2)
         za53 = -zB5(i5,i3)
         za54 = -zB5(i5,i4)
      endif

      if(ihel.eq.1)then
         znum = za12*za52**3
      elseif(ihel.eq.2)then
         znum = za13*za53**3
      elseif(ihel.eq.3)then
         znum = za14*za54**3
      elseif(ihel.eq.4)then
         znum = -za52*za12**3
      elseif(ihel.eq.5)then
         znum = -za53*za13**3
      elseif(ihel.eq.6)then
         znum = -za54*za14**3
      endif

      zden = za12*za23*za34*za45*za51

      zb0123 = -zi*znum/zden

      return
      end

*************************************************************
*
*     see [2], table 5.1
*

      function zb0132(iq,ig1,ig2,iqb,iG,ihel,iflip)
            use KinData_mod

      implicit real*8 (a-h,o-y)
      implicit complex*16(z)

      dimension ihel2(6)

      ihel2(1) = 1
      ihel2(2) = 3
      ihel2(3) = 2
      ihel2(4) = 4
      ihel2(5) = 6
      ihel2(6) = 5

      zb0132 = zb0123(iq,ig1,iG,iqb,ig2,ihel2(ihel),iflip)

      return
      end

*************************************************************
*
*     see [2], table 5.1
*

      function zb0213(iq,ig1,ig2,iqb,iG,ihel,iflip)
            use KinData_mod

      implicit real*8 (a-h,o-y)
      implicit complex*16(z)

      dimension ihel2(6)

      ihel2(1) = 2
      ihel2(2) = 1
      ihel2(3) = 3
      ihel2(4) = 5
      ihel2(5) = 4
      ihel2(6) = 6

      zb0213 = zb0123(iq,ig2,ig1,iqb,iG,ihel2(ihel),iflip)

      return
      end

*************************************************************
*
*     see [2], table 5.1
*

      function zb0231(iq,ig1,ig2,iqb,iG,ihel,iflip)
            use KinData_mod

      implicit real*8 (a-h,o-y)
      implicit complex*16(z)

      dimension ihel2(6)

      ihel2(1) = 3
      ihel2(2) = 1
      ihel2(3) = 2
      ihel2(4) = 6
      ihel2(5) = 4
      ihel2(6) = 5

      zb0231 = zb0123(iq,ig2,iG,iqb,ig1,ihel2(ihel),iflip)

      return
      end

*************************************************************
*
*     see [2], table 5.1
*

      function zb0312(iq,ig1,ig2,iqb,iG,ihel,iflip)
            use KinData_mod

      implicit real*8 (a-h,o-y)
      implicit complex*16(z)

      dimension ihel2(6)

      ihel2(1) = 2
      ihel2(2) = 3
      ihel2(3) = 1
      ihel2(4) = 5
      ihel2(5) = 6
      ihel2(6) = 4

      zb0312 = zb0123(iq,iG,ig1,iqb,ig2,ihel2(ihel),iflip)

      return
      end

*************************************************************
*
*     see [2], table 5.1
*

      function zb0321(iq,ig1,ig2,iqb,iG,ihel,iflip)
            use KinData_mod

      implicit real*8 (a-h,o-y)
      implicit complex*16(z)

      dimension ihel2(6)

      ihel2(1) = 3
      ihel2(2) = 2
      ihel2(3) = 1
      ihel2(4) = 6
      ihel2(5) = 5
      ihel2(6) = 4

      zb0321 = zb0123(iq,iG,ig2,iqb,ig1,ihel2(ihel),iflip)

      return
      end

*************************************************************
*
*     function B_{123}^{q} from [2],
*     see eq. 5.27
*

      function zBq123(iq,ig1,ig2,iqb,iG,iflip,renscale2)
      use KinData_mod

      implicit real*8 (a-h,o-y)
      implicit complex*16(z)
      common/order/iorder

      j1=ig1
      j2=ig2
      j3=iG
      j4=iqb
      j5=iq

      if(iflip.eq.0)then
        za12=zA5(j1,j2)
        za21=zA5(j2,j1)
        za13=zA5(j1,j3)
        za31=zA5(j3,j1)
        za14=zA5(j1,j4)
        za41=zA5(j4,j1)
        za15=zA5(j1,j5)
        za51=zA5(j5,j1)
        za23=zA5(j2,j3)
        za32=zA5(j3,j2)
        za24=zA5(j2,j4)
        za42=zA5(j4,j2)
        za25=zA5(j2,j5)
        za52=zA5(j5,j2)
        za34=zA5(j3,j4)
        za43=zA5(j4,j3)
        za35=zA5(j3,j5)
        za53=zA5(j5,j3)
        za45=zA5(j4,j5)
        za54=zA5(j5,j4)

        zb12=zB5(j1,j2)
        zb21=zB5(j2,j1)
        zb13=zB5(j1,j3)
        zb31=zB5(j3,j1)
        zb14=zB5(j1,j4)
        zb41=zB5(j4,j1)
        zb15=zB5(j1,j5)
        zb51=zB5(j5,j1)
        zb23=zB5(j2,j3)
        zb32=zB5(j3,j2)
        zb24=zB5(j2,j4)
        zb42=zB5(j4,j2)
        zb25=zB5(j2,j5)
        zb52=zB5(j5,j2)
        zb34=zB5(j3,j4)
        zb43=zB5(j4,j3)
        zb35=zB5(j3,j5)
        zb53=zB5(j5,j3)
        zb45=zB5(j4,j5)
        zb54=zB5(j5,j4)

      elseif(iflip.eq.1)then
        za12=-zB5(j1,j2)
        za21=-zB5(j2,j1)
        za13=-zB5(j1,j3)
        za31=-zB5(j3,j1)
        za14=-zB5(j1,j4)
        za41=-zB5(j4,j1)
        za15=-zB5(j1,j5)
        za51=-zB5(j5,j1)
        za23=-zB5(j2,j3)
        za32=-zB5(j3,j2)
        za24=-zB5(j2,j4)
        za42=-zB5(j4,j2)
        za25=-zB5(j2,j5)
        za52=-zB5(j5,j2)
        za34=-zB5(j3,j4)
        za43=-zB5(j4,j3)
        za35=-zB5(j3,j5)
        za53=-zB5(j5,j3)
        za45=-zB5(j4,j5)
        za54=-zB5(j5,j4)

        zb12=-zA5(j1,j2)
        zb21=-zA5(j2,j1)
        zb13=-zA5(j1,j3)
        zb31=-zA5(j3,j1)
        zb14=-zA5(j1,j4)
        zb41=-zA5(j4,j1)
        zb15=-zA5(j1,j5)
        zb51=-zA5(j5,j1)
        zb23=-zA5(j2,j3)
        zb32=-zA5(j3,j2)
        zb24=-zA5(j2,j4)
        zb42=-zA5(j4,j2)
        zb25=-zA5(j2,j5)
        zb52=-zA5(j5,j2)
        zb34=-zA5(j3,j4)
        zb43=-zA5(j4,j3)
        zb35=-zA5(j3,j5)
        zb53=-zA5(j5,j3)
        zb45=-zA5(j4,j5)
        zb54=-zA5(j5,j4)
      endif

        s12=s5(j1,j2)
        s13=s5(j1,j3)
        s14=s5(j1,j4)
        s15=s5(j1,j5)
        s21=s5(j2,j1)
        s23=s5(j2,j3)
        s24=s5(j2,j4)
        s25=s5(j2,j5)
        s31=s5(j3,j1)
        s32=s5(j3,j2)
        s34=s5(j3,j4)
        s35=s5(j3,j5)
        s41=s5(j4,j1)
        s42=s5(j4,j2)
        s43=s5(j4,j3)
        s45=s5(j4,j5)
        s51=s5(j5,j1)
        s52=s5(j5,j2)
        s53=s5(j5,j3)
        s54=s5(j5,j4)

      zwt0 = - 0.5d0*(zzLograt(renscale2,-s45))**2
     .      + 0.5d0*(zzLograt(renscale2,-s41))**2
     .      + 0.5d0*(zzLograt(renscale2,-s53))**2
     .      - 0.5d0*(zzLograt(renscale2,-s13))**2
     .      - za43*za12/za42/za13*zzCFGJ(s41,s45,s51)
     .      - za45*za21/za41/za25*zzCFGJ(s42,s45,s52)
     .      + za43*za52/za42/za53*(zzCFGJ(s43,s45,s53) + zzCFGJ(s52,s53,s32))
     .      - za51*za32/za52/za31*zzCFGJ(s42,s45,s52)
     .      + zzCFGJ(s42,s41,s12)
     .      - za41*za32/za42/za31*zzCFGJ(s41,s43,s31)
     .      + za43*za51/za41/za53*zzCFGJ(s51,s54,s41)
     .      - (za45*za31/za41/za35)**2*zzCFGJ(s53,s54,s43)
     .      + za45*za32/za42/za35*(zzCFGJ(s45,s43,s35) + zzCFGJ(s23,s25,s53))
     .      + za45*za31/za41/za35*zzCFGJ(s13,s14,s43)
     .      - za43*za12/za42/za13*zzCFGJ(s12,s13,s32)
     .      + za43*za21/za41/za23*zzCFGJ(s23,s21,s13)
     .       *( 2d0*za42*za51/za41/za52 + za42*za53/za43/za52
     .        - za43*za21/za41/za23*za45*za32/za42/za35)

      zwt1 = - (zzLograt(renscale2,-s45))
     .      + (zzLograt(renscale2,-s41))
     .      + (zzLograt(renscale2,-s53))
     .      - (zzLograt(renscale2,-s13))

      zwt2 =0d0

      if(iorder.eq.0)then
         zBq123 = zwt0
      elseif(iorder.eq.-1)then
         zBq123 = zwt1
      elseif(iorder.eq.-2)then
         zBq123 = zwt2
      endif

      return
      end

*************************************************************
*
*     function B_{231}^{q} from [2],
*     see eq. 5.28
*

      function zBq231(iq,ig1,ig2,iqb,iG,iflip,renscale2)
      use KinData_mod

      implicit real*8 (a-h,o-y)
      implicit complex*16(z)
      common/order/iorder

      j1=ig1
      j2=ig2
      j3=iG
      j4=iqb
      j5=iq

      if(iflip.eq.0)then
        za12=zA5(j1,j2)
        za21=zA5(j2,j1)
        za13=zA5(j1,j3)
        za31=zA5(j3,j1)
        za14=zA5(j1,j4)
        za41=zA5(j4,j1)
        za15=zA5(j1,j5)
        za51=zA5(j5,j1)
        za23=zA5(j2,j3)
        za32=zA5(j3,j2)
        za24=zA5(j2,j4)
        za42=zA5(j4,j2)
        za25=zA5(j2,j5)
        za52=zA5(j5,j2)
        za34=zA5(j3,j4)
        za43=zA5(j4,j3)
        za35=zA5(j3,j5)
        za53=zA5(j5,j3)
        za45=zA5(j4,j5)
        za54=zA5(j5,j4)

        zb12=zB5(j1,j2)
        zb21=zB5(j2,j1)
        zb13=zB5(j1,j3)
        zb31=zB5(j3,j1)
        zb14=zB5(j1,j4)
        zb41=zB5(j4,j1)
        zb15=zB5(j1,j5)
        zb51=zB5(j5,j1)
        zb23=zB5(j2,j3)
        zb32=zB5(j3,j2)
        zb24=zB5(j2,j4)
        zb42=zB5(j4,j2)
        zb25=zB5(j2,j5)
        zb52=zB5(j5,j2)
        zb34=zB5(j3,j4)
        zb43=zB5(j4,j3)
        zb35=zB5(j3,j5)
        zb53=zB5(j5,j3)
        zb45=zB5(j4,j5)
        zb54=zB5(j5,j4)

      elseif(iflip.eq.1)then
        za12=-zB5(j1,j2)
        za21=-zB5(j2,j1)
        za13=-zB5(j1,j3)
        za31=-zB5(j3,j1)
        za14=-zB5(j1,j4)
        za41=-zB5(j4,j1)
        za15=-zB5(j1,j5)
        za51=-zB5(j5,j1)
        za23=-zB5(j2,j3)
        za32=-zB5(j3,j2)
        za24=-zB5(j2,j4)
        za42=-zB5(j4,j2)
        za25=-zB5(j2,j5)
        za52=-zB5(j5,j2)
        za34=-zB5(j3,j4)
        za43=-zB5(j4,j3)
        za35=-zB5(j3,j5)
        za53=-zB5(j5,j3)
        za45=-zB5(j4,j5)
        za54=-zB5(j5,j4)

        zb12=-zA5(j1,j2)
        zb21=-zA5(j2,j1)
        zb13=-zA5(j1,j3)
        zb31=-zA5(j3,j1)
        zb14=-zA5(j1,j4)
        zb41=-zA5(j4,j1)
        zb15=-zA5(j1,j5)
        zb51=-zA5(j5,j1)
        zb23=-zA5(j2,j3)
        zb32=-zA5(j3,j2)
        zb24=-zA5(j2,j4)
        zb42=-zA5(j4,j2)
        zb25=-zA5(j2,j5)
        zb52=-zA5(j5,j2)
        zb34=-zA5(j3,j4)
        zb43=-zA5(j4,j3)
        zb35=-zA5(j3,j5)
        zb53=-zA5(j5,j3)
        zb45=-zA5(j4,j5)
        zb54=-zA5(j5,j4)
      endif

        s12=s5(j1,j2)
        s13=s5(j1,j3)
        s14=s5(j1,j4)
        s15=s5(j1,j5)
        s21=s5(j2,j1)
        s23=s5(j2,j3)
        s24=s5(j2,j4)
        s25=s5(j2,j5)
        s31=s5(j3,j1)
        s32=s5(j3,j2)
        s34=s5(j3,j4)
        s35=s5(j3,j5)
        s41=s5(j4,j1)
        s42=s5(j4,j2)
        s43=s5(j4,j3)
        s45=s5(j4,j5)
        s51=s5(j5,j1)
        s52=s5(j5,j2)
        s53=s5(j5,j3)
        s54=s5(j5,j4)

      zwt0 = - 0.5d0*(zzLograt(renscale2,-s45))**2
     .      + 0.5d0*(zzLograt(renscale2,-s42))**2
     .      + 0.5d0*(zzLograt(renscale2,-s51))**2
     .      - 0.5d0*(zzLograt(renscale2,-s12))**2
     .      + za45*za32/za42/za35*za52*za31/za51/za32
     .       *(zzCFGJ(s43,s41,s13) - zzCFGJ(s45,s41,s15))
     .      - (za45*za31/za41/za35)**2
     .       *(zzCFGJ(s53,s54,s43) + za53*za21/za51/za23*zzCFGJ(s23,s21,s13))
     .      + za42*za51/za41/za52*za43*za51/za41/za53*zzCFGJ(s52,s54,s42)
     .      - za51*za23/za53/za21*zzCFGJ(s53,s54,s43)
     .      + (za43*za51/za41/za53)**2*zzCFGJ(s53,s51,s13)
     .      + za45*za12/za42/za15*zzCFGJ(s12,s15,s52)

      zwt1 = - (zzLograt(renscale2,-s45))
     .      + (zzLograt(renscale2,-s42))
     .      + (zzLograt(renscale2,-s51))
     .      - (zzLograt(renscale2,-s12))

      zwt2 =0d0

      if(iorder.eq.0)then
         zBq231 = zwt0
      elseif(iorder.eq.-1)then
         zBq231 = zwt1
      elseif(iorder.eq.-2)then
         zBq231 = zwt2
      endif

      return
      end

*************************************************************
*
*     function B_{312}^{q} from [2],
*     see eq. 5.29
*

      function zBq312(iq,ig1,ig2,iqb,iG,iflip,renscale2)
      use KinData_mod

      implicit real*8 (a-h,o-y)
      implicit complex*16(z)
      common/order/iorder

      j1=ig1
      j2=ig2
      j3=iG
      j4=iqb
      j5=iq

      if(iflip.eq.0)then
        za12=zA5(j1,j2)
        za21=zA5(j2,j1)
        za13=zA5(j1,j3)
        za31=zA5(j3,j1)
        za14=zA5(j1,j4)
        za41=zA5(j4,j1)
        za15=zA5(j1,j5)
        za51=zA5(j5,j1)
        za23=zA5(j2,j3)
        za32=zA5(j3,j2)
        za24=zA5(j2,j4)
        za42=zA5(j4,j2)
        za25=zA5(j2,j5)
        za52=zA5(j5,j2)
        za34=zA5(j3,j4)
        za43=zA5(j4,j3)
        za35=zA5(j3,j5)
        za53=zA5(j5,j3)
        za45=zA5(j4,j5)
        za54=zA5(j5,j4)

        zb12=zB5(j1,j2)
        zb21=zB5(j2,j1)
        zb13=zB5(j1,j3)
        zb31=zB5(j3,j1)
        zb14=zB5(j1,j4)
        zb41=zB5(j4,j1)
        zb15=zB5(j1,j5)
        zb51=zB5(j5,j1)
        zb23=zB5(j2,j3)
        zb32=zB5(j3,j2)
        zb24=zB5(j2,j4)
        zb42=zB5(j4,j2)
        zb25=zB5(j2,j5)
        zb52=zB5(j5,j2)
        zb34=zB5(j3,j4)
        zb43=zB5(j4,j3)
        zb35=zB5(j3,j5)
        zb53=zB5(j5,j3)
        zb45=zB5(j4,j5)
        zb54=zB5(j5,j4)

      elseif(iflip.eq.1)then
        za12=-zB5(j1,j2)
        za21=-zB5(j2,j1)
        za13=-zB5(j1,j3)
        za31=-zB5(j3,j1)
        za14=-zB5(j1,j4)
        za41=-zB5(j4,j1)
        za15=-zB5(j1,j5)
        za51=-zB5(j5,j1)
        za23=-zB5(j2,j3)
        za32=-zB5(j3,j2)
        za24=-zB5(j2,j4)
        za42=-zB5(j4,j2)
        za25=-zB5(j2,j5)
        za52=-zB5(j5,j2)
        za34=-zB5(j3,j4)
        za43=-zB5(j4,j3)
        za35=-zB5(j3,j5)
        za53=-zB5(j5,j3)
        za45=-zB5(j4,j5)
        za54=-zB5(j5,j4)

        zb12=-zA5(j1,j2)
        zb21=-zA5(j2,j1)
        zb13=-zA5(j1,j3)
        zb31=-zA5(j3,j1)
        zb14=-zA5(j1,j4)
        zb41=-zA5(j4,j1)
        zb15=-zA5(j1,j5)
        zb51=-zA5(j5,j1)
        zb23=-zA5(j2,j3)
        zb32=-zA5(j3,j2)
        zb24=-zA5(j2,j4)
        zb42=-zA5(j4,j2)
        zb25=-zA5(j2,j5)
        zb52=-zA5(j5,j2)
        zb34=-zA5(j3,j4)
        zb43=-zA5(j4,j3)
        zb35=-zA5(j3,j5)
        zb53=-zA5(j5,j3)
        zb45=-zA5(j4,j5)
        zb54=-zA5(j5,j4)
      endif

        s12=s5(j1,j2)
        s13=s5(j1,j3)
        s14=s5(j1,j4)
        s15=s5(j1,j5)
        s21=s5(j2,j1)
        s23=s5(j2,j3)
        s24=s5(j2,j4)
        s25=s5(j2,j5)
        s31=s5(j3,j1)
        s32=s5(j3,j2)
        s34=s5(j3,j4)
        s35=s5(j3,j5)
        s41=s5(j4,j1)
        s42=s5(j4,j2)
        s43=s5(j4,j3)
        s45=s5(j4,j5)
        s51=s5(j5,j1)
        s52=s5(j5,j2)
        s53=s5(j5,j3)
        s54=s5(j5,j4)

      zwt0 = - 0.5d0*(zzLograt(renscale2,-s45))**2
     .      + 0.5d0*(zzLograt(renscale2,-s43))**2
     .      + 0.5d0*(zzLograt(renscale2,-s52))**2
     .      - 0.5d0*(zzLograt(renscale2,-s23))**2
     .      + za45*za21/za41/za25*zzCFGJ(s45,s42,s25)
     .      - za43*za21/za41/za23*zzCFGJ(s43,s42,s23)
     .      + (za42*za51/za41/za52*za45*za21/za41/za25
     .         -(za45*za21/za41/za25)**2)*zzCFGJ(s52,s54,s42)
     .      - 2d0*za42*za51/za41/za52*za45*za31/za41/za35
     .       *zzCFGJ(s53,s54,s43)
     .      + (za45*za21/za41/za25)**3*za52*za31/za51/za32
     .       *zzCFGJ(s52,s51,s12)
     .      + za42*za51/za41/za52*zzCFGJ(s51,s52,s21)
     .      - za42*za31/za41/za32*zzCFGJ(s13,s12,s23)
     .      + za45*za21/za41/za25*(za43*za21/za41/za23)**2
     .       *zzCFGJ(s23,s24,s43)
     .      + za42*za53/za43/za52*zzCFGJ(s23,s21,s13)

      zwt1 = - (zzLograt(renscale2,-s45))
     .      + (zzLograt(renscale2,-s43))
     .      + (zzLograt(renscale2,-s52))
     .      - (zzLograt(renscale2,-s23))

      zwt2 =0d0

      if(iorder.eq.0)then
         zBq312 = zwt0
      elseif(iorder.eq.-1)then
         zBq312 = zwt1
      elseif(iorder.eq.-2)then
         zBq312 = zwt2
      endif

      return
      end

*************************************************************
*
*     function B_{123}^{1} from [2],
*     see eqs. 5.30, 5.34
*

      function zB1123(iq,ig1,ig2,iqb,iG,ihel,iflip,renscale2)
      use KinData_mod

      implicit real*8 (a-h,o-y)
      implicit complex*16(z)
      common/order/iorder

      j1=ig1
      j2=ig2
      j3=iG
      j4=iqb
      j5=iq

      if(iflip.eq.0)then
        za12=zA5(j1,j2)
        za21=zA5(j2,j1)
        za13=zA5(j1,j3)
        za31=zA5(j3,j1)
        za14=zA5(j1,j4)
        za41=zA5(j4,j1)
        za15=zA5(j1,j5)
        za51=zA5(j5,j1)
        za23=zA5(j2,j3)
        za32=zA5(j3,j2)
        za24=zA5(j2,j4)
        za42=zA5(j4,j2)
        za25=zA5(j2,j5)
        za52=zA5(j5,j2)
        za34=zA5(j3,j4)
        za43=zA5(j4,j3)
        za35=zA5(j3,j5)
        za53=zA5(j5,j3)
        za45=zA5(j4,j5)
        za54=zA5(j5,j4)

        zb12=zB5(j1,j2)
        zb21=zB5(j2,j1)
        zb13=zB5(j1,j3)
        zb31=zB5(j3,j1)
        zb14=zB5(j1,j4)
        zb41=zB5(j4,j1)
        zb15=zB5(j1,j5)
        zb51=zB5(j5,j1)
        zb23=zB5(j2,j3)
        zb32=zB5(j3,j2)
        zb24=zB5(j2,j4)
        zb42=zB5(j4,j2)
        zb25=zB5(j2,j5)
        zb52=zB5(j5,j2)
        zb34=zB5(j3,j4)
        zb43=zB5(j4,j3)
        zb35=zB5(j3,j5)
        zb53=zB5(j5,j3)
        zb45=zB5(j4,j5)
        zb54=zB5(j5,j4)

      elseif(iflip.eq.1)then
        za12=-zB5(j1,j2)
        za21=-zB5(j2,j1)
        za13=-zB5(j1,j3)
        za31=-zB5(j3,j1)
        za14=-zB5(j1,j4)
        za41=-zB5(j4,j1)
        za15=-zB5(j1,j5)
        za51=-zB5(j5,j1)
        za23=-zB5(j2,j3)
        za32=-zB5(j3,j2)
        za24=-zB5(j2,j4)
        za42=-zB5(j4,j2)
        za25=-zB5(j2,j5)
        za52=-zB5(j5,j2)
        za34=-zB5(j3,j4)
        za43=-zB5(j4,j3)
        za35=-zB5(j3,j5)
        za53=-zB5(j5,j3)
        za45=-zB5(j4,j5)
        za54=-zB5(j5,j4)

        zb12=-zA5(j1,j2)
        zb21=-zA5(j2,j1)
        zb13=-zA5(j1,j3)
        zb31=-zA5(j3,j1)
        zb14=-zA5(j1,j4)
        zb41=-zA5(j4,j1)
        zb15=-zA5(j1,j5)
        zb51=-zA5(j5,j1)
        zb23=-zA5(j2,j3)
        zb32=-zA5(j3,j2)
        zb24=-zA5(j2,j4)
        zb42=-zA5(j4,j2)
        zb25=-zA5(j2,j5)
        zb52=-zA5(j5,j2)
        zb34=-zA5(j3,j4)
        zb43=-zA5(j4,j3)
        zb35=-zA5(j3,j5)
        zb53=-zA5(j5,j3)
        zb45=-zA5(j4,j5)
        zb54=-zA5(j5,j4)
      endif

        s12=s5(j1,j2)
        s13=s5(j1,j3)
        s14=s5(j1,j4)
        s15=s5(j1,j5)
        s21=s5(j2,j1)
        s23=s5(j2,j3)
        s24=s5(j2,j4)
        s25=s5(j2,j5)
        s31=s5(j3,j1)
        s32=s5(j3,j2)
        s34=s5(j3,j4)
        s35=s5(j3,j5)
        s41=s5(j4,j1)
        s42=s5(j4,j2)
        s43=s5(j4,j3)
        s45=s5(j4,j5)
        s51=s5(j5,j1)
        s52=s5(j5,j2)
        s53=s5(j5,j3)
        s54=s5(j5,j4)

        zwta = - 0.5d0*(zzLograt(renscale2,-s41))**2
     .         + 0.5d0*(zzLograt(renscale2,-s42))**2
     .         + 0.5d0*(zzLograt(renscale2,-s13))**2
     .         - 0.5d0*(zzLograt(renscale2,-s23))**2

      if(ihel.eq.1)then
         zwt0 = zwta
     .        - za41*za32/za42/za31
     .         *( zzCFGJ(s41,s45,s51)-zzCFGJ(s41,s42,s21)-zzCFGJ(s41,s43,s31)
     .           +zzCFGJ(s23,s25,s53)-zzCFGJ(s23,s21,s13))
     .        - za43*za12/za42/za13
     .         *( zzCFGJ(s43,s45,s53)-zzCFGJ(s43,s41,s13)-zzCFGJ(s43,s42,s23))
     .        - za43*za51/za41/za53*zzCFGJ(s52,s53,s32)
     .        - za45*za31/za41/za35*(zzCFGJ(s13,s14,s43)-zzCFGJ(s13,s15,s53))
     .        + ( za43*za12/za42/za13
     .           + za42*za51/za41/za52*za43*za12/za42/za13)
     .         * zzCFGJ(s12,s13,s32)
     .        - zzCFGJ(s13,s12,s23)
     .        + za45*za32/za42/za35*zzCFGJ(s23,s24,s43)
     .        - 3d0*za43*za21/za41/za23*zzCFGJ(s23,s21,s13)
      elseif(ihel.eq.2)then
         zwt0 = zwta
     .         - za51*za32/za52/za31*(zzCFGJ(s41,s45,s51)-zzCFGJ(s23,s21,s13))
     .        + za42*za51/za41/za52
     .         *( zzCFGJ(s42,s45,s52)-zzCFGJ(s42,s41,s12)-zzCFGJ(s13,s12,s23))
     .        + za45*za21/za41/za25*za45*za32/za42/za35
     .         *( zzCFGJ(s45,s42,s25)-zzCFGJ(s43,s42,s23)-zzCFGJ(s45,s41,s15))
     .        + za41*za32/za42/za31*zzCFGJ(s41,s43,s31)
     .        - zzCFGJ(s42,s43,s32)
     .        + za43*za52/za42/za53*(zzCFGJ(s52,s54,s42)-zzCFGJ(s52,s51,s12))
     .        - za43*za12/za42/za13*(za43*za52/za42/za53)**2
     .         * zzCFGJ(s53,s54,s43)
     .        - za45*za21/za41/za25*(za45*za32/za42/za35)**3
     .         * zzCFGJ(s53,s52,s23)
     .        - za45*za32/za42/za35*(za41*za32/za42/za31)**2
     .         * zzCFGJ(s13,s14,s43)
     .        + za45*za32/za42/za35*zzCFGJ(s23,s24,s43)
      endif

        zwt1 = - (zzLograt(renscale2,-s41))
     .         + (zzLograt(renscale2,-s42))
     .         + (zzLograt(renscale2,-s13))
     .         - (zzLograt(renscale2,-s23))

      zwt2 = 0d0

      if(iorder.eq.0)then
         zB1123 = zwt0
      elseif(iorder.eq.-1)then
         zB1123 = zwt1
      elseif(iorder.eq.-2)then
         zB1123 = zwt2
      endif

      return
      end

*************************************************************
*
*     function B_{132}^{1} from [2],
*     see eqs. 5.31, 5.35
*

      function zB1132(iq,ig1,ig2,iqb,iG,ihel,iflip,renscale2)
      use KinData_mod

      implicit real*8 (a-h,o-y)
      implicit complex*16(z)
      common/order/iorder

      j1=ig1
      j2=ig2
      j3=iG
      j4=iqb
      j5=iq

      if(iflip.eq.0)then
        za12=zA5(j1,j2)
        za21=zA5(j2,j1)
        za13=zA5(j1,j3)
        za31=zA5(j3,j1)
        za14=zA5(j1,j4)
        za41=zA5(j4,j1)
        za15=zA5(j1,j5)
        za51=zA5(j5,j1)
        za23=zA5(j2,j3)
        za32=zA5(j3,j2)
        za24=zA5(j2,j4)
        za42=zA5(j4,j2)
        za25=zA5(j2,j5)
        za52=zA5(j5,j2)
        za34=zA5(j3,j4)
        za43=zA5(j4,j3)
        za35=zA5(j3,j5)
        za53=zA5(j5,j3)
        za45=zA5(j4,j5)
        za54=zA5(j5,j4)

        zb12=zB5(j1,j2)
        zb21=zB5(j2,j1)
        zb13=zB5(j1,j3)
        zb31=zB5(j3,j1)
        zb14=zB5(j1,j4)
        zb41=zB5(j4,j1)
        zb15=zB5(j1,j5)
        zb51=zB5(j5,j1)
        zb23=zB5(j2,j3)
        zb32=zB5(j3,j2)
        zb24=zB5(j2,j4)
        zb42=zB5(j4,j2)
        zb25=zB5(j2,j5)
        zb52=zB5(j5,j2)
        zb34=zB5(j3,j4)
        zb43=zB5(j4,j3)
        zb35=zB5(j3,j5)
        zb53=zB5(j5,j3)
        zb45=zB5(j4,j5)
        zb54=zB5(j5,j4)

      elseif(iflip.eq.1)then
        za12=-zB5(j1,j2)
        za21=-zB5(j2,j1)
        za13=-zB5(j1,j3)
        za31=-zB5(j3,j1)
        za14=-zB5(j1,j4)
        za41=-zB5(j4,j1)
        za15=-zB5(j1,j5)
        za51=-zB5(j5,j1)
        za23=-zB5(j2,j3)
        za32=-zB5(j3,j2)
        za24=-zB5(j2,j4)
        za42=-zB5(j4,j2)
        za25=-zB5(j2,j5)
        za52=-zB5(j5,j2)
        za34=-zB5(j3,j4)
        za43=-zB5(j4,j3)
        za35=-zB5(j3,j5)
        za53=-zB5(j5,j3)
        za45=-zB5(j4,j5)
        za54=-zB5(j5,j4)

        zb12=-zA5(j1,j2)
        zb21=-zA5(j2,j1)
        zb13=-zA5(j1,j3)
        zb31=-zA5(j3,j1)
        zb14=-zA5(j1,j4)
        zb41=-zA5(j4,j1)
        zb15=-zA5(j1,j5)
        zb51=-zA5(j5,j1)
        zb23=-zA5(j2,j3)
        zb32=-zA5(j3,j2)
        zb24=-zA5(j2,j4)
        zb42=-zA5(j4,j2)
        zb25=-zA5(j2,j5)
        zb52=-zA5(j5,j2)
        zb34=-zA5(j3,j4)
        zb43=-zA5(j4,j3)
        zb35=-zA5(j3,j5)
        zb53=-zA5(j5,j3)
        zb45=-zA5(j4,j5)
        zb54=-zA5(j5,j4)
      endif

        s12=s5(j1,j2)
        s13=s5(j1,j3)
        s14=s5(j1,j4)
        s15=s5(j1,j5)
        s21=s5(j2,j1)
        s23=s5(j2,j3)
        s24=s5(j2,j4)
        s25=s5(j2,j5)
        s31=s5(j3,j1)
        s32=s5(j3,j2)
        s34=s5(j3,j4)
        s35=s5(j3,j5)
        s41=s5(j4,j1)
        s42=s5(j4,j2)
        s43=s5(j4,j3)
        s45=s5(j4,j5)
        s51=s5(j5,j1)
        s52=s5(j5,j2)
        s53=s5(j5,j3)
        s54=s5(j5,j4)

        zwta = - 0.5d0*(zzLograt(renscale2,-s41))**2
     .         + 0.5d0*(zzLograt(renscale2,-s43))**2
     .         + 0.5d0*(zzLograt(renscale2,-s12))**2
     .         - 0.5d0*(zzLograt(renscale2,-s23))**2

      if(ihel.eq.1)then
         zwt0 = zwta
     .        - za42*za13/za43/za12
     .         *( zzCFGJ(s42,s45,s52)-zzCFGJ(s42,s41,s12)-zzCFGJ(s42,s43,s32))
     .        - za42*za51/za41/za52*zzCFGJ(s53,s52,s23)
     .        - za45*za21/za41/za25*(zzCFGJ(s12,s14,s42)-zzCFGJ(s12,s15,s52))
     .        - 3d0*za45*za31/za41/za35*za42*za31/za41/za32
     .         * zzCFGJ(s23,s24,s43)
      elseif(ihel.eq.2)then
         zwt0 = zwta
     .        + za42*za51/za41/za52*zzCFGJ(s43,s45,s53)
     .        - za43*za51/za41/za53*zzCFGJ(s43,s41,s13)
     .        - za42*za53/za43/za52*zzCFGJ(s43,s42,s23)
     .        +( za43*za12/za42/za13-za41*za32/za42/za31)
     .         *zzCFGJ(s13,s12,s23)
     .        - zzCFGJ(s12,s13,s32)
      endif

        zwt1 = - (zzLograt(renscale2,-s41))
     .         + (zzLograt(renscale2,-s43))
     .         + (zzLograt(renscale2,-s12))
     .         - (zzLograt(renscale2,-s23))

      zwt2 = 0d0

      if(iorder.eq.0)then
         zB1132 = zwt0
      elseif(iorder.eq.-1)then
         zB1132 = zwt1
      elseif(iorder.eq.-2)then
         zB1132 = zwt2
      endif

      return
      end

*************************************************************
*
*     function B_{231}^{1} from [2],
*     see eqs. 5.32, 5.36
*

      function zB1231(iq,ig1,ig2,iqb,iG,ihel,iflip,renscale2)
      use KinData_mod

      implicit real*8 (a-h,o-y)
      implicit complex*16(z)
      common/order/iorder

      j1=ig1
      j2=ig2
      j3=iG
      j4=iqb
      j5=iq

      if(iflip.eq.0)then
        za12=zA5(j1,j2)
        za21=zA5(j2,j1)
        za13=zA5(j1,j3)
        za31=zA5(j3,j1)
        za14=zA5(j1,j4)
        za41=zA5(j4,j1)
        za15=zA5(j1,j5)
        za51=zA5(j5,j1)
        za23=zA5(j2,j3)
        za32=zA5(j3,j2)
        za24=zA5(j2,j4)
        za42=zA5(j4,j2)
        za25=zA5(j2,j5)
        za52=zA5(j5,j2)
        za34=zA5(j3,j4)
        za43=zA5(j4,j3)
        za35=zA5(j3,j5)
        za53=zA5(j5,j3)
        za45=zA5(j4,j5)
        za54=zA5(j5,j4)

        zb12=zB5(j1,j2)
        zb21=zB5(j2,j1)
        zb13=zB5(j1,j3)
        zb31=zB5(j3,j1)
        zb14=zB5(j1,j4)
        zb41=zB5(j4,j1)
        zb15=zB5(j1,j5)
        zb51=zB5(j5,j1)
        zb23=zB5(j2,j3)
        zb32=zB5(j3,j2)
        zb24=zB5(j2,j4)
        zb42=zB5(j4,j2)
        zb25=zB5(j2,j5)
        zb52=zB5(j5,j2)
        zb34=zB5(j3,j4)
        zb43=zB5(j4,j3)
        zb35=zB5(j3,j5)
        zb53=zB5(j5,j3)
        zb45=zB5(j4,j5)
        zb54=zB5(j5,j4)

      elseif(iflip.eq.1)then
        za12=-zB5(j1,j2)
        za21=-zB5(j2,j1)
        za13=-zB5(j1,j3)
        za31=-zB5(j3,j1)
        za14=-zB5(j1,j4)
        za41=-zB5(j4,j1)
        za15=-zB5(j1,j5)
        za51=-zB5(j5,j1)
        za23=-zB5(j2,j3)
        za32=-zB5(j3,j2)
        za24=-zB5(j2,j4)
        za42=-zB5(j4,j2)
        za25=-zB5(j2,j5)
        za52=-zB5(j5,j2)
        za34=-zB5(j3,j4)
        za43=-zB5(j4,j3)
        za35=-zB5(j3,j5)
        za53=-zB5(j5,j3)
        za45=-zB5(j4,j5)
        za54=-zB5(j5,j4)

        zb12=-zA5(j1,j2)
        zb21=-zA5(j2,j1)
        zb13=-zA5(j1,j3)
        zb31=-zA5(j3,j1)
        zb14=-zA5(j1,j4)
        zb41=-zA5(j4,j1)
        zb15=-zA5(j1,j5)
        zb51=-zA5(j5,j1)
        zb23=-zA5(j2,j3)
        zb32=-zA5(j3,j2)
        zb24=-zA5(j2,j4)
        zb42=-zA5(j4,j2)
        zb25=-zA5(j2,j5)
        zb52=-zA5(j5,j2)
        zb34=-zA5(j3,j4)
        zb43=-zA5(j4,j3)
        zb35=-zA5(j3,j5)
        zb53=-zA5(j5,j3)
        zb45=-zA5(j4,j5)
        zb54=-zA5(j5,j4)
      endif

        s12=s5(j1,j2)
        s13=s5(j1,j3)
        s14=s5(j1,j4)
        s15=s5(j1,j5)
        s21=s5(j2,j1)
        s23=s5(j2,j3)
        s24=s5(j2,j4)
        s25=s5(j2,j5)
        s31=s5(j3,j1)
        s32=s5(j3,j2)
        s34=s5(j3,j4)
        s35=s5(j3,j5)
        s41=s5(j4,j1)
        s42=s5(j4,j2)
        s43=s5(j4,j3)
        s45=s5(j4,j5)
        s51=s5(j5,j1)
        s52=s5(j5,j2)
        s53=s5(j5,j3)
        s54=s5(j5,j4)

        zwta = - 0.5d0*(zzLograt(renscale2,-s51))**2
     .         + 0.5d0*(zzLograt(renscale2,-s53))**2
     .         + 0.5d0*(zzLograt(renscale2,-s12))**2
     .         - 0.5d0*(zzLograt(renscale2,-s23))**2

      if(ihel.eq.1)then
         zwt0 = zwta
     .        + za51*za23/za53/za21
     .         *( zzCFGJ(s51,s52,s21)+zzCFGJ(s51,s53,s31)-zzCFGJ(s51,s54,s41))
     .        + (za43*za51/za41/za53)**2
     .         *( zzCFGJ(s53,s54,s43)-zzCFGJ(s53,s51,s13))
     .        - (za43*za21/za41/za23)**2*za45*za32/za42/za35
     .         * zzCFGJ(s23,s24,s43)
     .        +( za43*za21/za41/za23-za42*za31/za41/za32
     .          + (za42*za31/za41/za32)**2*za51*za23/za53/za21)
     .         * zzCFGJ(s23,s21,s13)
      elseif(ihel.eq.2)then
         zwt0 = zwta
     .        - 3d0*za41*za32/za42/za31*zzCFGJ(s13,s12,s23)
     .        - za41*za53/za43/za51*zzCFGJ(s53,s51,s13)
     .        + za51*za23/za53/za21*za41*za52/za42/za51
     .         * zzCFGJ(s23,s21,s13)
     .        + za41*za23/za43/za21*zzCFGJ(s41,s42,s21)
      endif

        zwt1 = - (zzLograt(renscale2,-s51))
     .         + (zzLograt(renscale2,-s53))
     .         + (zzLograt(renscale2,-s12))
     .         - (zzLograt(renscale2,-s23))

      zwt2 = 0d0

      if(iorder.eq.0)then
         zB1231 = zwt0
      elseif(iorder.eq.-1)then
         zB1231 = zwt1
      elseif(iorder.eq.-2)then
         zB1231 = zwt2
      endif

      return
      end

*************************************************************
*
*     function B_{321}^{1} from [2],
*     see eqs. 5.33, 5.37
*

      function zB1321(iq,ig1,ig2,iqb,iG,ihel,iflip,renscale2)
      use KinData_mod

      implicit real*8 (a-h,o-y)
      implicit complex*16(z)
      common/order/iorder

      j1=ig1
      j2=ig2
      j3=iG
      j4=iqb
      j5=iq

      if(iflip.eq.0)then
        za12=zA5(j1,j2)
        za21=zA5(j2,j1)
        za13=zA5(j1,j3)
        za31=zA5(j3,j1)
        za14=zA5(j1,j4)
        za41=zA5(j4,j1)
        za15=zA5(j1,j5)
        za51=zA5(j5,j1)
        za23=zA5(j2,j3)
        za32=zA5(j3,j2)
        za24=zA5(j2,j4)
        za42=zA5(j4,j2)
        za25=zA5(j2,j5)
        za52=zA5(j5,j2)
        za34=zA5(j3,j4)
        za43=zA5(j4,j3)
        za35=zA5(j3,j5)
        za53=zA5(j5,j3)
        za45=zA5(j4,j5)
        za54=zA5(j5,j4)

        zb12=zB5(j1,j2)
        zb21=zB5(j2,j1)
        zb13=zB5(j1,j3)
        zb31=zB5(j3,j1)
        zb14=zB5(j1,j4)
        zb41=zB5(j4,j1)
        zb15=zB5(j1,j5)
        zb51=zB5(j5,j1)
        zb23=zB5(j2,j3)
        zb32=zB5(j3,j2)
        zb24=zB5(j2,j4)
        zb42=zB5(j4,j2)
        zb25=zB5(j2,j5)
        zb52=zB5(j5,j2)
        zb34=zB5(j3,j4)
        zb43=zB5(j4,j3)
        zb35=zB5(j3,j5)
        zb53=zB5(j5,j3)
        zb45=zB5(j4,j5)
        zb54=zB5(j5,j4)

      elseif(iflip.eq.1)then
        za12=-zB5(j1,j2)
        za21=-zB5(j2,j1)
        za13=-zB5(j1,j3)
        za31=-zB5(j3,j1)
        za14=-zB5(j1,j4)
        za41=-zB5(j4,j1)
        za15=-zB5(j1,j5)
        za51=-zB5(j5,j1)
        za23=-zB5(j2,j3)
        za32=-zB5(j3,j2)
        za24=-zB5(j2,j4)
        za42=-zB5(j4,j2)
        za25=-zB5(j2,j5)
        za52=-zB5(j5,j2)
        za34=-zB5(j3,j4)
        za43=-zB5(j4,j3)
        za35=-zB5(j3,j5)
        za53=-zB5(j5,j3)
        za45=-zB5(j4,j5)
        za54=-zB5(j5,j4)

        zb12=-zA5(j1,j2)
        zb21=-zA5(j2,j1)
        zb13=-zA5(j1,j3)
        zb31=-zA5(j3,j1)
        zb14=-zA5(j1,j4)
        zb41=-zA5(j4,j1)
        zb15=-zA5(j1,j5)
        zb51=-zA5(j5,j1)
        zb23=-zA5(j2,j3)
        zb32=-zA5(j3,j2)
        zb24=-zA5(j2,j4)
        zb42=-zA5(j4,j2)
        zb25=-zA5(j2,j5)
        zb52=-zA5(j5,j2)
        zb34=-zA5(j3,j4)
        zb43=-zA5(j4,j3)
        zb35=-zA5(j3,j5)
        zb53=-zA5(j5,j3)
        zb45=-zA5(j4,j5)
        zb54=-zA5(j5,j4)
      endif

        s12=s5(j1,j2)
        s13=s5(j1,j3)
        s14=s5(j1,j4)
        s15=s5(j1,j5)
        s21=s5(j2,j1)
        s23=s5(j2,j3)
        s24=s5(j2,j4)
        s25=s5(j2,j5)
        s31=s5(j3,j1)
        s32=s5(j3,j2)
        s34=s5(j3,j4)
        s35=s5(j3,j5)
        s41=s5(j4,j1)
        s42=s5(j4,j2)
        s43=s5(j4,j3)
        s45=s5(j4,j5)
        s51=s5(j5,j1)
        s52=s5(j5,j2)
        s53=s5(j5,j3)
        s54=s5(j5,j4)

        zwta = - 0.5d0*(zzLograt(renscale2,-s51))**2
     .         + 0.5d0*(zzLograt(renscale2,-s52))**2
     .         + 0.5d0*(zzLograt(renscale2,-s13))**2
     .         - 0.5d0*(zzLograt(renscale2,-s23))**2

      if(ihel.eq.1)then
         zwt0 = zwta
     .        + (za42*za51/za41/za52)**2
     .         *( zzCFGJ(s52,s54,s42)-zzCFGJ(s52,s51,s12))
     .        - zzCFGJ(s13,s12,s23)
     .        + za51*za32/za52/za31*zzCFGJ(s23,s25,s53)
      elseif(ihel.eq.2)then
         zwt0 = zwta 
     .        + (za41*za52/za42/za51)**2*za41*za32/za42/za31
     .         *( zzCFGJ(s51,s52,s21)-zzCFGJ(s51,s54,s41))
     .        + za51*za32/za52/za31*zzCFGJ(s51,s53,s31)
     .        + (za43*za52/za42/za53)**3*za53*za12/za52/za13
     .         * zzCFGJ(s53,s52,s23)
     .        - zzCFGJ(s52,s53,s32)
     .        + za53*za12/za52/za13*zzCFGJ(s12,s13,s32)
     .        + za45*za23/za43/za25*(zzCFGJ(s23,s25,s53)-zzCFGJ(s13,s15,s53))
     .        - za41*za52/za42/za51*(za41*za32/za42/za31)**2
     .         * zzCFGJ(s13,s12,s23)
      endif   

        zwt1 = - (zzLograt(renscale2,-s51))
     .         + (zzLograt(renscale2,-s52))
     .         + (zzLograt(renscale2,-s13))
     .         - (zzLograt(renscale2,-s23))

      zwt2 = 0d0

      if(iorder.eq.0)then
         zB1321 = zwt0
      elseif(iorder.eq.-1)then
         zB1321 = zwt1
      elseif(iorder.eq.-2)then   
         zB1321 = zwt2
      endif   

      return
      end

*************************************************************
