c--- libGJRVC  --------------------------------------------------------------
c
c     This library contains all the real-virtual C-type MEs for gamma+jet.
c
c     Amplitudes taken from:
c      
c     A. Signer
c     One-loop corrections to five-parton amplitudes with external photons
c     Phys.Lett. B357 (1995) 204-210c     
c     DOI:10.1016/0370-2693(95)00905-Z
c     [hep-ph/9507442]      
c
c     and:
c
c     A. Signer
c     Helicity method for next-to-leading order corrections in QCD
c     (Doctoral Thesis)
c     Diss. Naturwiss. ETH Zürich, Nr. 11143, 1995
c     DOI:10.3929/ethz-a-001503525
c     
c----------------------------------------------------------------------------

      function fullC0g1G(iqb1,iq2,iqb2,iq1,iG,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16(z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4),zewnull(4)
!$omp threadprivate(/eweakZ/)
      
      ave = 1/4d0/nc**2

      fac = ave*(4d0*pi*as)**3*(4d0*pi*amz)*4d0*nc*(nc**2-1d0)/(8d0*pi**2)

      FullC0g1G = C0g1G(iqb1,iq2,iqb2,iq1,iG,renscale2)
     .     +1d0/dble(nc)*dble(nf)*(
     .            Ch0g1G(iqb1,iq2,iqb2,iq1,iG,renscale2))
     .     +1d0/dble(nc)**2*(
     .            Ct0g1G(iqb1,iq2,iqb2,iq1,iG,renscale2))

      FullC0g1G = fac*FullC0g1G
      
      return
      end

*************************************************************

c
c     Note: There is an error in Signers's thesis:
c     in eq. 5.96 u_1^(1) has to be N times the quantity as it is defined there.
c     The same is true for eq. 19 in hep-ph/9507442.
c

      function C0g1G(iqb1,iq2,iqb2,iq1,iG,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16(z)
      parameter(pi=3.141592653589793238d0)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4),zewnull(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      common/order/iorder

      call eweakV2qq(nf1,nf2,0d0)

      C0g1G=0d0

      zi = (0d0,1d0)
      zwt = 0d0

      do ihel=1,4
         do iflip=0,1
            zwt = zwt 
     .           + dconjg( zc1gam0(iqb1,iq2,iqb2,iq1,iG,ihel,iflip))
     .           * zC1gaml2(iqb1,iq2,iqb2,iq1,iG,ihel,iflip,renscale2)
         enddo
      enddo

      C0g1G = real(zi*zwt)/2d0
      
      if (iorder .eq. -1) then
c     UV counter term at order 1/e to renormalise the matrix element in the MS bar scheme
         subuv = 11d0/3d0
         C0g1G = C0g1G - subuv*C0g0G(iqb1,iq2,iqb2,iq1,iG)
      elseif (iorder .eq. 0)then
c     pi**2/6d0 comes from a correction for C(e)
c     2/3 comes from a conversion of four-dimensional helicity scheme (FDH) to 't Hooft-Veltman (HV) scheme
c     see hep-ph/9708239 equation 6.5
         e0 = pi**2/6d0 - 2d0/3d0
         C0g1G = C0g1G + e0*C0g0G(iqb1,iq2,iqb2,iq1,iG)
      endif

      return
      end

**********************************************
* A wrapper function for returning the normal
* matrix element for situations where either of the quarks
* in the second quark line are in the initial state.
* Otherwise it will symmetrise over the quark pair.
*

      function sC0g1G(iqb1,iq2,iqb2,iq1,iG,renscale2)
      use Process_mod, only : nPDFs_proc
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      !> For DIS we always symmetrise this ME/ Z exchange
      if((nPDFs_proc.ne.1.and.((iq2.le.2).or.(iqb2.le.2))))then
         sC0g1G=C0g1G(iqb1,iq2,iqb2,iq1,iG,renscale2)
      elseif(nPDFs_proc.eq.0) then
         sC0g1G=0.25d0*(C0g1G(iqb1,iq2,iqb2,iq1,iG,renscale2)+C0g1G(iqb1,iqb2,iq2,iq1,iG,renscale2)+
     $    C0g1G(iq1,iq2,iqb2,iqb1,iG,renscale2)+C0g1G(iq1,iqb2,iq2,iqb1,iG,renscale2))
      else
         sC0g1G=0.5d0*(C0g1G(iqb1,iq2,iqb2,iq1,iG,renscale2)+C0g1G(iqb1,iqb2,iq2,iq1,iG,renscale2))
      endif

      return
      end

*************************************************************

      function Ct0g1G(iqb1,iq2,iqb2,iq1,iG,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16(z)
      parameter(pi=3.141592653589793238d0)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4),zewnull(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      common/order/iorder

      call eweakV2qq(nf1,nf2,0d0)

      Ct0g1G=0d0

      zi = (0d0,1d0)
      zwt = 0d0

      do ihel=1,4
         do iflip=0,1
            zwt = zwt 
     .           + dconjg( zc1gam0(iqb1,iq2,iqb2,iq1,iG,ihel,iflip))
     .           * zC1gams2(iqb1,iq2,iqb2,iq1,iG,ihel,iflip,renscale2)
         enddo
      enddo

      Ct0g1G = real(zi*zwt)/2d0

      if (iorder .eq. 0)then
c     pi**2/6d0 comes from a correction for C(e)
c     1 comes from a conversion of four-dimensional helicity scheme (FDH) to 't Hooft-Veltman (HV) scheme
c     see hep-ph/9708239 equation 6.5
         e0 = pi**2/6d0 - 1d0 
         Ct0g1G = Ct0g1G - e0*C0g0G(iqb1,iq2,iqb2,iq1,iG)
      endif

      return
      end

**********************************************
* A wrapper function for returning the normal
* matrix element for situations where either of the quarks
* in the second quark line are in the initial state.
* Otherwise it will symmetrise over the quark pair.
*

      function sCt0g1G(iqb1,iq2,iqb2,iq1,iG,renscale2)
      use Process_mod, only : nPDFs_proc
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      !> For DIS we always symmetrise this ME/ Z exchange
      if((nPDFs_proc.ne.1.and.((iq2.le.2).or.(iqb2.le.2))))then
         sCt0g1G=Ct0g1G(iqb1,iq2,iqb2,iq1,iG,renscale2)
      elseif(nPDFs_proc.eq.0) then
         sCt0g1G=0.25d0*(Ct0g1G(iqb1,iq2,iqb2,iq1,iG,renscale2)+Ct0g1G(iqb1,iqb2,iq2,iq1,iG,renscale2)+
     $    Ct0g1G(iq1,iq2,iqb2,iqb1,iG,renscale2)+Ct0g1G(iq1,iqb2,iq2,iqb1,iG,renscale2))
      else
         sCt0g1G=0.5d0*(Ct0g1G(iqb1,iq2,iqb2,iq1,iG,renscale2)+Ct0g1G(iqb1,iqb2,iq2,iq1,iG,renscale2))
      endif

      return
      end

*************************************************************

      function Ch0g1G(iqb1,iq2,iqb2,iq1,iG,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16(z)
      parameter(pi=3.141592653589793238d0)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4),zewnull(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      common/order/iorder

      call eweakV2qq(nf1,nf2,0d0)

      Ch0g1G=0d0

      zi = (0d0,1d0)
      zwt = 0d0

      do ihel=1,4
         do iflip=0,1
            zwt = zwt 
     .           + dconjg( zc1gam0(iqb1,iq2,iqb2,iq1,iG,ihel,iflip))
     .           * zC1gamNF(iqb1,iq2,iqb2,iq1,iG,ihel,iflip,renscale2)
         enddo
      enddo

      Ch0g1G = real(zi*zwt)/2d0

      if (iorder .eq. -1) then
c     UV counter term at order 1/e to renormalise the matrix element in the MS bar scheme
         subuv = -2d0/3d0
         Ch0g1G = Ch0g1G - subuv*C0g0G(iqb1,iq2,iqb2,iq1,iG)
         Ch0g1G = 0d0 
c     Note: counter term is such that the e-1 coefficient vansihes
      endif

      return
      end

**********************************************
* A wrapper function for returning the normal
* matrix element for situations where either of the quarks
* in the second quark line are in the initial state.
* Otherwise it will symmetrise over the quark pair.
*

      function sCh0g1G(iqb1,iq2,iqb2,iq1,iG,renscale2)
      use Process_mod, only : nPDFs_proc
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      !> For DIS we always symmetrise this ME/ Z exchange
      if((nPDFs_proc.ne.1.and.((iq2.le.2).or.(iqb2.le.2))))then
         sCh0g1G=Ch0g1G(iqb1,iq2,iqb2,iq1,iG,renscale2)
      elseif(nPDFs_proc.eq.0) then
         sCh0g1G=0.25d0*(Ch0g1G(iqb1,iq2,iqb2,iq1,iG,renscale2)+Ch0g1G(iqb1,iqb2,iq2,iq1,iG,renscale2)+
     $    Ch0g1G(iq1,iq2,iqb2,iqb1,iG,renscale2)+Ch0g1G(iq1,iqb2,iq2,iqb1,iG,renscale2))
      else
         sCh0g1G=0.5d0*(Ch0g1G(iqb1,iq2,iqb2,iq1,iG,renscale2)+Ch0g1G(iqb1,iqb2,iq2,iq1,iG,renscale2))
      endif

      return
      end

*************************************************************

      function zC1gaml1(iqb1,iq2,iqb2,iq1,iG,ihel,iflip,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16(z)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4),zewnull(4)
!$omp threadprivate(/eweakZ/)
      dimension ihel2(4)

      ihel2(1)=1
      ihel2(2)=3
      ihel2(3)=2
      ihel2(4)=4

*     ordering (qb,Qb,Q,q,G) + ordering (Qb,qb,q,Q,G)
*
*     the second ordering comes from the d-amplitudes in eq. 3.8 (Signer's thesis) or
*     eq. 18 in hep-ph/9507442, using eq. 5.98 (thesis) or eq. 21 (paper)

      zwt = real(zewfac1(1))*zc1gam1u1l(iqb1,iq2,iqb2,iq1,iG,ihel,iflip,renscale2)
     .     +real(zewfac2(2))*zc1gam1u1l(iqb2,iq1,iqb1,iq2,iG,ihel2(ihel),iflip,renscale2)

      zC1gaml1 = zwt

      return
      end

*************************************************************

      function zC1gaml2(iqb1,iq2,iqb2,iq1,iG,ihel,iflip,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16(z)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4),zewnull(4)
!$omp threadprivate(/eweakZ/)
      dimension ihel2(4)


      ihel2(1)=1
      ihel2(2)=3
      ihel2(3)=2
      ihel2(4)=4

*     ordering (qb,Qb,Q,q,G)
      zc1gam1u2la = zc1gam1uA(iqb1,iq2,iqb2,iq1,iG,ihel,iflip,renscale2)
     .             -zc1gam1u1l(iqb1,iq2,iqb2,iq1,iG,ihel,iflip,renscale2)

*     ordering (Qb,qb,q,Q,G)
      zc1gam1u2lb = zc1gam1uA(iqb2,iq1,iqb1,iq2,iG,ihel2(ihel),iflip,renscale2)
     .             -zc1gam1u1l(iqb2,iq1,iqb1,iq2,iG,ihel2(ihel),iflip,renscale2)

      zwt = real(zewfac1(1))*zc1gam1u2la
     .     +real(zewfac2(2))*zc1gam1u2lb

      zC1gaml2 = zwt

      return
      end

*************************************************************

      function zC1gams1(iqb1,iq2,iqb2,iq1,iG,ihel,iflip,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16(z)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4),zewnull(4)
!$omp threadprivate(/eweakZ/)
      dimension ihel2(4)

      ihel2(1)=1
      ihel2(2)=3
      ihel2(3)=2
      ihel2(4)=4

*     ordering (qb,Qb,Q,q,G)
      zc1gam1u1sa = zc1gam1uA(iqb1,iq2,iqb2,iq1,iG,ihel,iflip,renscale2)
     .             +zc1gam1uB(iqb1,iq2,iqb2,iq1,iG,ihel,iflip,renscale2)

*     ordering (Qb,qb,q,Q,G)
      zc1gam1u1sb = zc1gam1uA(iqb2,iq1,iqb1,iq2,iG,ihel2(ihel),iflip,renscale2)
     .             +zc1gam1uB(iqb2,iq1,iqb1,iq2,iG,ihel2(ihel),iflip,renscale2)

      zwt = real(zewfac1(1))*zc1gam1u1sa
     .     +real(zewfac2(2))*zc1gam1u1sb

      zC1gams1 = zwt

      return
      end

*************************************************************

      function zC1gams2(iqb1,iq2,iqb2,iq1,iG,ihel,iflip,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16(z)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4),zewnull(4)
!$omp threadprivate(/eweakZ/)
      common/order/iorder
      dimension ihel2(4)

      ihel2(1)=1
      ihel2(2)=3
      ihel2(3)=2
      ihel2(4)=4

*     ordering (qb,Qb,Q,q,G)
      zc1gam1u2sa = -2d0*zc1gam1uA(iqb1,iq2,iqb2,iq1,iG,ihel,iflip,renscale2)
     .             -zc1gam1uB(iqb1,iq2,iqb2,iq1,iG,ihel,iflip,renscale2)

*     ordering (Qb,qb,q,Q,G)
      zc1gam1u2sb = -2d0*zc1gam1uA(iqb2,iq1,iqb1,iq2,iG,ihel2(ihel),iflip,renscale2)
     .             -zc1gam1uB(iqb2,iq1,iqb1,iq2,iG,ihel2(ihel),iflip,renscale2)

      zwt = real(zewfac1(1))*zc1gam1u2sa
     .     +real(zewfac2(2))*zc1gam1u2sb

      zC1gams2 = zwt

      return
      end

*************************************************************

      function zC1gamNF(iqb1,iq2,iqb2,iq1,iG,ihel,iflip,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16(z)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4),zewnull(4)
!$omp threadprivate(/eweakZ/)
      common/order/iorder
      dimension ihel2(4)

      ihel2(1)=1
      ihel2(2)=3
      ihel2(3)=2
      ihel2(4)=4

*     ordering (qb,Qb,Q,q,G) + ordering (Qb,qb,q,Q,G)

      zwt = real(zewfac1(1))*zc1gam1uNF(iqb1,iq2,iqb2,iq1,iG,ihel,iflip,renscale2)
     .     +real(zewfac2(2))*zc1gam1uNF(iqb2,iq1,iqb1,iq2,iG,ihel2(ihel),iflip,renscale2)

      zC1gamNF = zwt

      return
      end

*************************************************************
*
*     this is u_A from hep-ph/9507442 (eq. 36, see eq. 5.111 in Signer's thesis)
*

      function zc1gam1uA(iqb1,iq2,iqb2,iq1,iG,ihel,iflip,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16(z)
      common/order/iorder

      zi = (0d0,1d0)
      zone = (1d0,0d0)

***********************************************
*             qQG                             *
*     ihel=1  +++    =  X                     *
*     ihel=2  -++    =  X|q1<->qb1            *
*     ihel=3  +-+    = -X|q2<->qb2            *
*     ihel=4  --+    = -X|q1<->qb1,q2<->qb2   *
***********************************************

      if(ihel.eq.1)then
         j1 = iqb1
         j2 = iq1
         j3 = iG
         j4 = iqb2
         j5 = iq2
         signfac = 1d0
      elseif(ihel.eq.2)then
         j1 = iq1
         j2 = iqb1
         j3 = iG
         j4 = iqb2
         j5 = iq2
         signfac = 1d0
      elseif(ihel.eq.3)then
         j1 = iqb1
         j2 = iq1
         j3 = iG
         j4 = iq2
         j5 = iqb2
         signfac = -1d0
      elseif(ihel.eq.4)then
         j1 = iq1
         j2 = iqb1
         j3 = iG
         j4 = iq2
         j5 = iqb2
         signfac = -1d0
      endif   

      if(iflip.eq.0)then
        za12=zA5(j1,j2)
        za21=zA5(j2,j1)
        za13=zA5(j1,j3)
        za31=zA5(j3,j1)
        za14=zA5(j1,j4)
        za41=zA5(j4,j1)
        za15=zA5(j1,j5)
        za51=zA5(j5,j1)
        za23=zA5(j2,j3)
        za32=zA5(j3,j2)
        za24=zA5(j2,j4)
        za42=zA5(j4,j2)
        za25=zA5(j2,j5)
        za52=zA5(j5,j2)
        za34=zA5(j3,j4)
        za43=zA5(j4,j3)
        za35=zA5(j3,j5)
        za53=zA5(j5,j3)
        za45=zA5(j4,j5)
        za54=zA5(j5,j4)

        zb12=zB5(j1,j2)
        zb21=zB5(j2,j1)
        zb13=zB5(j1,j3)
        zb31=zB5(j3,j1)
        zb14=zB5(j1,j4)
        zb41=zB5(j4,j1)
        zb15=zB5(j1,j5)
        zb51=zB5(j5,j1)
        zb23=zB5(j2,j3)
        zb32=zB5(j3,j2)
        zb24=zB5(j2,j4)
        zb42=zB5(j4,j2)
        zb25=zB5(j2,j5)
        zb52=zB5(j5,j2)
        zb34=zB5(j3,j4)
        zb43=zB5(j4,j3)
        zb35=zB5(j3,j5)
        zb53=zB5(j5,j3)
        zb45=zB5(j4,j5)
        zb54=zB5(j5,j4)

      elseif(iflip.eq.1)then
        za12=-zB5(j1,j2)
        za21=-zB5(j2,j1)
        za13=-zB5(j1,j3)
        za31=-zB5(j3,j1)
        za14=-zB5(j1,j4)
        za41=-zB5(j4,j1)
        za15=-zB5(j1,j5)
        za51=-zB5(j5,j1)
        za23=-zB5(j2,j3)
        za32=-zB5(j3,j2)
        za24=-zB5(j2,j4)
        za42=-zB5(j4,j2)
        za25=-zB5(j2,j5)
        za52=-zB5(j5,j2)
        za34=-zB5(j3,j4)
        za43=-zB5(j4,j3)
        za35=-zB5(j3,j5)
        za53=-zB5(j5,j3)
        za45=-zB5(j4,j5)
        za54=-zB5(j5,j4)

        zb12=-zA5(j1,j2)
        zb21=-zA5(j2,j1)
        zb13=-zA5(j1,j3)
        zb31=-zA5(j3,j1)
        zb14=-zA5(j1,j4)
        zb41=-zA5(j4,j1)
        zb15=-zA5(j1,j5)
        zb51=-zA5(j5,j1)
        zb23=-zA5(j2,j3)
        zb32=-zA5(j3,j2)
        zb24=-zA5(j2,j4)
        zb42=-zA5(j4,j2)
        zb25=-zA5(j2,j5)
        zb52=-zA5(j5,j2)
        zb34=-zA5(j3,j4)
        zb43=-zA5(j4,j3)
        zb35=-zA5(j3,j5)
        zb53=-zA5(j5,j3)
        zb45=-zA5(j4,j5)
        zb54=-zA5(j5,j4)
      endif
        s12=s5(j1,j2)
        s13=s5(j1,j3)
        s14=s5(j1,j4)
        s15=s5(j1,j5)
        s21=s5(j2,j1)
        s23=s5(j2,j3)
        s24=s5(j2,j4)
        s25=s5(j2,j5)
        s31=s5(j3,j1)
        s32=s5(j3,j2)
        s34=s5(j3,j4)
        s35=s5(j3,j5)
        s41=s5(j4,j1)
        s42=s5(j4,j2)
        s43=s5(j4,j3)
        s45=s5(j4,j5)
        s51=s5(j5,j1)
        s52=s5(j5,j2)
        s53=s5(j5,j3)
        s54=s5(j5,j4)

      ztree = zc1gam0u0(j1,j5,j4,j2,j3,1,iflip)

      if(iorder.eq.0)then
         zwt = -zi*ztree
     .         *( 0.5d0*(zzLograt(renscale2,-s14))**2
     .          + 0.5d0*(zzLograt(renscale2,-s25))**2
     .          - 0.5d0*(zzLograt(renscale2,-s15))**2
     .          - 0.5d0*(zzLograt(renscale2,-s24))**2)
     .         - za12*za24*zb23/za25/za23*zL1(s13,s45,renscale2)
     .         - za15*za45*zb53/za25/za53*zL1(s13,s24,renscale2)
     .         - za15*za43*zb53/za23/za53*zL1(s15,s24,renscale2)
     .         + za12*za14/za13/za25/za23*zL0(s45,s24,renscale2)
     .         -zi*ztree
     .         *( zzCFGJ(s15,s14,s45)
     .          - (za42*za51/za41/za52)**2*zzCFGJ(s25,s24,s45)
     .          + zzCFGJ(s43,s41,s13)
     .          - zzCFGJ(s43,s42,s23)
     .          + zzCFGJ(s42,s45,s52)
     .          - zzCFGJ(s41,s45,s51)
     .          + zzCFGJ(s53,s52,s23)
     .          - (za43*za51/za41/za53)**2*zzCFGJ(s53,s51,s13))
      elseif(iorder.eq.-1)then
         zwt = -zi*ztree
     .         *( (zzLograt(renscale2,-s14))
     .          + (zzLograt(renscale2,-s25))
     .          - (zzLograt(renscale2,-s15))
     .          - (zzLograt(renscale2,-s24)))
      elseif(iorder.eq.-2)then
         zwt = 0d0
      endif

      zc1gam1uA = signfac*zwt

      return
      end

*************************************************************
*
*     this is u_B from hep-ph/9507442 (eq. 35, see eq. 5.110 in Signer's thesis)
*

      function zc1gam1uB(iqb1,iq2,iqb2,iq1,iG,ihel,iflip,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16(z)
      common/order/iorder


      zi = (0d0,1d0)
      zone = (1d0,0d0)

***********************************************
*             qQG                             *
*     ihel=1  +++    =  X                     *
*     ihel=2  -++    = -X|q1<->qb1            *
*     ihel=3  +-+    =  X|q2<->qb2            *
*     ihel=4  --+    = -X|q1<->qb1,q2<->qb2   *
***********************************************

      if(ihel.eq.1)then
         j1 = iqb1
         j2 = iq1
         j3 = iG
         j4 = iqb2
         j5 = iq2
         signfac = 1d0
      elseif(ihel.eq.2)then
         j1 = iq1
         j2 = iqb1
         j3 = iG
         j4 = iqb2
         j5 = iq2
         signfac = -1d0
      elseif(ihel.eq.3)then
         j1 = iqb1
         j2 = iq1
         j3 = iG
         j4 = iq2
         j5 = iqb2
         signfac = 1d0
      elseif(ihel.eq.4)then
         j1 = iq1
         j2 = iqb1
         j3 = iG
         j4 = iq2
         j5 = iqb2
         signfac = -1d0
      endif

      if(iflip.eq.0)then
        za12=zA5(j1,j2)
        za21=zA5(j2,j1)
        za13=zA5(j1,j3)
        za31=zA5(j3,j1)
        za14=zA5(j1,j4)
        za41=zA5(j4,j1)
        za15=zA5(j1,j5)
        za51=zA5(j5,j1)
        za23=zA5(j2,j3)
        za32=zA5(j3,j2)
        za24=zA5(j2,j4)
        za42=zA5(j4,j2)
        za25=zA5(j2,j5)
        za52=zA5(j5,j2)
        za34=zA5(j3,j4)
        za43=zA5(j4,j3)
        za35=zA5(j3,j5)
        za53=zA5(j5,j3)
        za45=zA5(j4,j5)
        za54=zA5(j5,j4)

        zb12=zB5(j1,j2)
        zb21=zB5(j2,j1)
        zb13=zB5(j1,j3)
        zb31=zB5(j3,j1)
        zb14=zB5(j1,j4)
        zb41=zB5(j4,j1)
        zb15=zB5(j1,j5)
        zb51=zB5(j5,j1)
        zb23=zB5(j2,j3)
        zb32=zB5(j3,j2)
        zb24=zB5(j2,j4)
        zb42=zB5(j4,j2)
        zb25=zB5(j2,j5)
        zb52=zB5(j5,j2)
        zb34=zB5(j3,j4)
        zb43=zB5(j4,j3)
        zb35=zB5(j3,j5)
        zb53=zB5(j5,j3)
        zb45=zB5(j4,j5)
        zb54=zB5(j5,j4)

      elseif(iflip.eq.1)then
        za12=-zB5(j1,j2)
        za21=-zB5(j2,j1)
        za13=-zB5(j1,j3)
        za31=-zB5(j3,j1)
        za14=-zB5(j1,j4)
        za41=-zB5(j4,j1)
        za15=-zB5(j1,j5)
        za51=-zB5(j5,j1)
        za23=-zB5(j2,j3)
        za32=-zB5(j3,j2)
        za24=-zB5(j2,j4)
        za42=-zB5(j4,j2)
        za25=-zB5(j2,j5)
        za52=-zB5(j5,j2)
        za34=-zB5(j3,j4)
        za43=-zB5(j4,j3)
        za35=-zB5(j3,j5)
        za53=-zB5(j5,j3)
        za45=-zB5(j4,j5)
        za54=-zB5(j5,j4)

        zb12=-zA5(j1,j2)
        zb21=-zA5(j2,j1)
        zb13=-zA5(j1,j3)
        zb31=-zA5(j3,j1)
        zb14=-zA5(j1,j4)
        zb41=-zA5(j4,j1)
        zb15=-zA5(j1,j5)
        zb51=-zA5(j5,j1)
        zb23=-zA5(j2,j3)
        zb32=-zA5(j3,j2)
        zb24=-zA5(j2,j4)
        zb42=-zA5(j4,j2)
        zb25=-zA5(j2,j5)
        zb52=-zA5(j5,j2)
        zb34=-zA5(j3,j4)
        zb43=-zA5(j4,j3)
        zb35=-zA5(j3,j5)
        zb53=-zA5(j5,j3)
        zb45=-zA5(j4,j5)
        zb54=-zA5(j5,j4)
      endif
        s12=s5(j1,j2)
        s13=s5(j1,j3)
        s14=s5(j1,j4)
        s15=s5(j1,j5)
        s21=s5(j2,j1)
        s23=s5(j2,j3)
        s24=s5(j2,j4)
        s25=s5(j2,j5)
        s31=s5(j3,j1)
        s32=s5(j3,j2)
        s34=s5(j3,j4)
        s35=s5(j3,j5)
        s41=s5(j4,j1)
        s42=s5(j4,j2)
        s43=s5(j4,j3)
        s45=s5(j4,j5)
        s51=s5(j5,j1)
        s52=s5(j5,j2)
        s53=s5(j5,j3)
        s54=s5(j5,j4)

      ztree = zc1gam0u0(j1,j5,j4,j2,j3,1,iflip)

      if(iorder.eq.0)then
         zwt = zi*ztree
     .          *( 0.5d0*(zzLograt(renscale2,-s12))**2
     .           + 0.5d0*(zzLograt(renscale2,-s45))**2
     .           + 3d0*(zzLograt(renscale2,-s45))
     .           + 13d0/2d0*zone)
     .         - za12*za12*za45*zb25*zb25/2d0/za13/za23
     .          * zL2(s45,s13,renscale2)
     .         - za12*za43*zb23*zb53/za23
     .          * zL2(s12,s45,renscale2)
     .         + za12*za24*za43*zb23/za23/za23/za45
     .          *( zL1(s45,s12,renscale2) + zL1(s45,s13,renscale2))
     .         + za14*za14/2d0/za13/za23/za45*zL0(s45,s13,renscale2)
     .         + za12*za12*zb25*zb25/2d0/za13/za23/zb45/s13
     .         - za14*zb53/za23/s45
     .         +zi*ztree
     .          *( zzCFGJ(s13,s12,s23)
     .           + (za43*za21/za41/za23)**2*zzCFGJ(s23,s21,s13))
      elseif(iorder.eq.-1)then
         zwt = zi*ztree
     .         *( (zzLograt(renscale2,-s12))
     .          + (zzLograt(renscale2,-s45))
     .          + 3d0*zone)
      elseif(iorder.eq.-2)then
         zwt = zi*ztree*2d0
      endif

      zc1gam1uB = signfac*zwt

      return
      end

*************************************************************
*
*     this is u_1^l from hep-ph/9507442
*     (eqs. 31, 32, see eqs. 5.106, 5.107 in Signer's thesis)
*

      function zc1gam1u1l(iqb1,iq2,iqb2,iq1,iG,ihel,iflip,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16(z)
      common/order/iorder


      zi = (0d0,1d0)
      zone = (1d0,0d0)

***********************************************
*             qQG                             *
*     ihel=1  +++    =  X                     *
*     ihel=2  -++    = -Y|q1<->qb1,q2<->qb2   *
*     ihel=3  +-+    =  Y                     *
*     ihel=4  --+    = -X|q1<->qb1,q2<->qb2   *
***********************************************

      if(ihel.eq.1.or.ihel.eq.3)then
         j1 = iqb1
         j2 = iq1
         j3 = iG
         j4 = iqb2
         j5 = iq2
         signfac = 1d0
      elseif(ihel.eq.2.or.ihel.eq.4)then
         j1 = iq1
         j2 = iqb1
         j3 = iG
         j4 = iq2
         j5 = iqb2
         signfac = -1d0
      endif

      if(iflip.eq.0)then
        za12=zA5(j1,j2)
        za21=zA5(j2,j1)
        za13=zA5(j1,j3)
        za31=zA5(j3,j1)
        za14=zA5(j1,j4)
        za41=zA5(j4,j1)
        za15=zA5(j1,j5)
        za51=zA5(j5,j1)
        za23=zA5(j2,j3)
        za32=zA5(j3,j2)
        za24=zA5(j2,j4)
        za42=zA5(j4,j2)
        za25=zA5(j2,j5)
        za52=zA5(j5,j2)
        za34=zA5(j3,j4)
        za43=zA5(j4,j3)
        za35=zA5(j3,j5)
        za53=zA5(j5,j3)
        za45=zA5(j4,j5)
        za54=zA5(j5,j4)

        zb12=zB5(j1,j2)
        zb21=zB5(j2,j1)
        zb13=zB5(j1,j3)
        zb31=zB5(j3,j1)
        zb14=zB5(j1,j4)
        zb41=zB5(j4,j1)
        zb15=zB5(j1,j5)
        zb51=zB5(j5,j1)
        zb23=zB5(j2,j3)
        zb32=zB5(j3,j2)
        zb24=zB5(j2,j4)
        zb42=zB5(j4,j2)
        zb25=zB5(j2,j5)
        zb52=zB5(j5,j2)
        zb34=zB5(j3,j4)
        zb43=zB5(j4,j3)
        zb35=zB5(j3,j5)
        zb53=zB5(j5,j3)
        zb45=zB5(j4,j5)
        zb54=zB5(j5,j4)

      elseif(iflip.eq.1)then
        za12=-zB5(j1,j2)
        za21=-zB5(j2,j1)
        za13=-zB5(j1,j3)
        za31=-zB5(j3,j1)
        za14=-zB5(j1,j4)
        za41=-zB5(j4,j1)
        za15=-zB5(j1,j5)
        za51=-zB5(j5,j1)
        za23=-zB5(j2,j3)
        za32=-zB5(j3,j2)
        za24=-zB5(j2,j4)
        za42=-zB5(j4,j2)
        za25=-zB5(j2,j5)
        za52=-zB5(j5,j2)
        za34=-zB5(j3,j4)
        za43=-zB5(j4,j3)
        za35=-zB5(j3,j5)
        za53=-zB5(j5,j3)
        za45=-zB5(j4,j5)
        za54=-zB5(j5,j4)

        zb12=-zA5(j1,j2)
        zb21=-zA5(j2,j1)
        zb13=-zA5(j1,j3)
        zb31=-zA5(j3,j1)
        zb14=-zA5(j1,j4)
        zb41=-zA5(j4,j1)
        zb15=-zA5(j1,j5)
        zb51=-zA5(j5,j1)
        zb23=-zA5(j2,j3)
        zb32=-zA5(j3,j2)
        zb24=-zA5(j2,j4)
        zb42=-zA5(j4,j2)
        zb25=-zA5(j2,j5)
        zb52=-zA5(j5,j2)
        zb34=-zA5(j3,j4)
        zb43=-zA5(j4,j3)
        zb35=-zA5(j3,j5)
        zb53=-zA5(j5,j3)
        zb45=-zA5(j4,j5)
        zb54=-zA5(j5,j4)
      endif
        s12=s5(j1,j2)
        s13=s5(j1,j3)
        s14=s5(j1,j4)
        s15=s5(j1,j5)
        s21=s5(j2,j1)
        s23=s5(j2,j3)
        s24=s5(j2,j4)
        s25=s5(j2,j5)
        s31=s5(j3,j1)
        s32=s5(j3,j2)
        s34=s5(j3,j4)
        s35=s5(j3,j5)
        s41=s5(j4,j1)
        s42=s5(j4,j2)
        s43=s5(j4,j3)
        s45=s5(j4,j5)
        s51=s5(j5,j1)
        s52=s5(j5,j2)
        s53=s5(j5,j3)
        s54=s5(j5,j4)

      if(ihel.eq.1.or.ihel.eq.4)then
         ztree = zc1gam0u0(j1,j5,j4,j2,j3,1,iflip)   
         if(iorder.eq.0)then
            zfinite = za13*za24*za24*zb23*zb23/2d0/za23/za45
     .                * zL2(s13,s45,renscale2)
     .               - za14*za24*zb23/za23/za45
     .                * zL1(s13,s45,renscale2)
     .               + za12*za14*zb25/2d0/za13/za23/s45
     .               + za14*za24*zb23/2d0/za23/za45/s45
     .               - zi*ztree
     .                *( zzCFGJ(s15,s14,s45) + zzCFGJ(s24,s25,s54)
     .                  + zzCFGJ(s43,s41,s13) + zzCFGJ(s53,s52,s23))     
         endif   
      elseif(ihel.eq.2.or.ihel.eq.3)then
         ztree = zc1gam0u0(j1,j5,j4,j2,j3,3,iflip)   
         if(iorder.eq.0)then
            zfinite = -za13*za25*za25*zb23*zb23/2d0/za23/za45
     .                 * zL2(s13,s45,renscale2)
     .                - za12*za14*za25*zb24/za13/za24/za23
     .                 * zL1(s13,s45,renscale2)
     .                - za15*za15/za13/za23/za45
     .                 * zL0(s13,s45,renscale2)
     .                - za14*za45*zb43/za24/za43
     .                 * zL1(s13,s25,renscale2)
     .                - za12*za15/za13/za24/za23
     .                 * zL0(s13,s25,renscale2)
     .                + za14*za53*zb43/za23/za43
     .                 * zL1(s14,s25,renscale2)
     .                + za12*za15*zb24/2d0/za13/za23/s45
     .                - za15*za25*zb23/2d0/za23/za45/s45
     .                - zi*ztree
     .                 *( zzCFGJ(s15,s14,s45)
     .                   +(za14*za25/za15/za24)**2*zzCFGJ(s24,s25,s54)
     .                   +(za14*za35/za15/za34)**2*zzCFGJ(s43,s41,s13)
     .                   + zzCFGJ(s53,s52,s23))
         endif   
      endif   

      if(iorder.eq.0)then
         zwt = -zi*ztree
     .         *( 0.5d0*(zzLograt(renscale2,-s14))**2
     .          + 0.5d0*(zzLograt(renscale2,-s25))**2
     .          - 2d0/3d0*(zzLograt(renscale2,-s45))
     .          - 29d0/18d0*zone)
     .         + zfinite 
      elseif(iorder.eq.-1)then
         zwt = -zi*ztree
     .         *( (zzLograt(renscale2,-s14))
     .          + (zzLograt(renscale2,-s25))
     .          - 2d0/3d0*zone)
      elseif(iorder.eq.-2)then   
         zwt = -zi*ztree*2d0
      endif

      zc1gam1u1l = signfac*zwt

      return
      end

*************************************************************
*
*     this is u^NF from hep-ph/9507442 (eq. 30, see eq. 5.105 in Signer's thesis) 
*

      function zc1gam1uNF(iqb1,iq2,iqb2,iq1,iG,ihel,iflip,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16(z)
      common/order/iorder

      zi = (0d0,1d0)
      zone = (1d0,0d0)

      s45 = s5(iqb2,iq2)

      if(iorder.eq.0)then
         zfac = 2d0/3d0*(zzLograt(renscale2,-s45))+10d0/9d0*zone
      elseif(iorder.eq.-1)then
         zfac = 2d0/3d0*zone
      elseif(iorder.eq.-2)then
         zfac = 0d0
      endif    

      zc1gam1uNF = zi*zfac*zc1gam0u0(iqb1,iq2,iqb2,iq1,iG,ihel,iflip)
   
      return
      end

*************************************************************
