c--- libGJRVD  --------------------------------------------------------------
c
c     This library contains all the real-virtual D-type MEs for gamma+jet.
c
c----------------------------------------------------------------------------

      function fullD0g1G(iqb1,iq2,iqb2,iq1,iG,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16(z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4),zewnull(4)
!$omp threadprivate(/eweakZ/)
      
      ave = 1/4d0/nc**2

      fac = ave*(4d0*pi*as)**3*(4d0*pi*amz)*4d0*(nc**2-1d0)/(8d0*pi**2)

      FullD0g1G = D0g1G(iqb1,iq2,iqb2,iq1,iG,renscale2)
     .     +1d0/dble(nc)*dble(nf)*(
     .            Dh0g1G(iqb1,iq2,iqb2,iq1,iG,renscale2))
     .     +1d0/dble(nc)**2*(
     .            Dt0g1G(iqb1,iq2,iqb2,iq1,iG,renscale2))

      FullD0g1G = fac*FullD0g1G
      
      return
      end

*************************************************************

      function D0g1G(iqb1,iq2,iqb2,iq1,iG,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16(z)
      parameter(pi=3.141592653589793238d0)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4),zewnull(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      common/order/iorder
      dimension ihelD(2)

      call eweakV2qq(nf1,nf2,0d0)

c     interfernce only if quarks have same helicity
      ihelD(1)=1
      ihelD(2)=4
      
      D0g1G=0d0

      zi = (0d0,1d0)

      zwt = 0d0

      do ii=1,2
         do iflip=0,1
            zord12 = dconjg( zc1gam0(iqb1,iq2,iqb2,iq1,iG,ihelD(ii),iflip))
     .              * zC1gaml1(iqb2,iq2,iqb1,iq1,iG,ihelD(ii),iflip,renscale2)

            zord21 = dconjg( zc1gam0(iqb2,iq2,iqb1,iq1,iG,ihelD(ii),iflip))
     .              * zC1gaml1(iqb1,iq2,iqb2,iq1,iG,ihelD(ii),iflip,renscale2)

            zwt = zwt + zord12 + zord21
         enddo
      enddo

      D0g1G = real(zi*zwt)/2d0

      if (iorder .eq. -1) then
c     UV counter term at order 1/e to renormalise the matrix element in the MS bar scheme
         subuv = -11d0/3d0
         D0g1G = D0g1G + subuv*D0g0G(iqb1,iq2,iqb2,iq1,iG)
      elseif (iorder .eq. 0)then
c     pi**2/6d0 comes from a correction for C(e)
c     2/3 comes from a conversion of four-dimensional helicity scheme (FDH) to 't Hooft-Veltman (HV) scheme
c     see hep-ph/9708239 equation 6.5
         e0 = (pi**2/6d0 - 2d0/3d0)
         D0g1G = D0g1G + e0*D0g0G(iqb1,iq2,iqb2,iq1,iG)
      endif

      D0g1G = -D0g1G

      return
      end

*************************************************************

      function Dt0g1G(iqb1,iq2,iqb2,iq1,iG,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16(z)
      parameter(pi=3.141592653589793238d0)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4),zewnull(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      common/order/iorder
      dimension ihelD(2)

      call eweakV2qq(nf1,nf2,0d0)

c     interfernce only if quarks have same helicity
      ihelD(1)=1
      ihelD(2)=4
      
      Dt0g1G=0d0

      zi = (0d0,1d0)

      zwt = 0d0

      do ii=1,2
         do iflip=0,1
            zord12 = dconjg( zc1gam0(iqb1,iq2,iqb2,iq1,iG,ihelD(ii),iflip))
     .              * zC1gams1(iqb2,iq2,iqb1,iq1,iG,ihelD(ii),iflip,renscale2)

            zord21 = dconjg( zc1gam0(iqb2,iq2,iqb1,iq1,iG,ihelD(ii),iflip))
     .              * zC1gams1(iqb1,iq2,iqb2,iq1,iG,ihelD(ii),iflip,renscale2)

            zwt = zwt + zord12 + zord21
         enddo
      enddo

      Dt0g1G = real(zi*zwt)/2d0

      if (iorder .eq. 0)then
c     pi**2/6d0 comes from a correction for C(e)
c     1 comes from a conversion of four-dimensional helicity scheme (FDH) to 't Hooft-Veltman (HV) scheme
c     see hep-ph/9708239 equation 6.5
         e0 = (pi**2/6d0 - 1d0)
         Dt0g1G = Dt0g1G - e0*D0g0G(iqb1,iq2,iqb2,iq1,iG)
      endif

      Dt0g1G = -Dt0g1G

      return
      end

*************************************************************

      function Dh0g1G(iqb1,iq2,iqb2,iq1,iG,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16(z)
      parameter(pi=3.141592653589793238d0)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4),zewnull(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      common/order/iorder
      dimension ihelD(2)

      call eweakV2qq(nf1,nf2,0d0)

c     interfernce only if quarks have same helicity
      ihelD(1)=1
      ihelD(2)=4
      
      Dh0g1G=0d0

      zi = (0d0,1d0)

      zwt = 0d0

      do ii=1,2
         do iflip=0,1
            zord12 = dconjg( zc1gam0(iqb1,iq2,iqb2,iq1,iG,ihelD(ii),iflip))
     .              * zC1gamNF(iqb2,iq2,iqb1,iq1,iG,ihelD(ii),iflip,renscale2)

            zord21 = dconjg( zc1gam0(iqb2,iq2,iqb1,iq1,iG,ihelD(ii),iflip))
     .              * zC1gamNF(iqb1,iq2,iqb2,iq1,iG,ihelD(ii),iflip,renscale2)

            zwt = zwt + zord12 + zord21
         enddo
      enddo

      Dh0g1G = real(-zi*zwt)/2d0

      if (iorder .eq. -1) then
c     UV counter term at order 1/e to renormalise the matrix element in the MS bar scheme
         subuv = 2d0/3d0
         Dh0g1G = Dh0g1G + subuv*D0g0G(iqb1,iq2,iqb2,iq1,iG)
         Dh0g1G = 0d0 
c     Note: counter term is such that the e-1 coefficient vansihes
      endif

      Dh0g1G = -Dh0g1G

      return
      end

*************************************************************
