c--- libGJVVA  --------------------------------------------------------------
c
c     This library contains the loop induced (gg -> gG) ME for gamma+jet.
c
c----------------------------------------------------------------------------

      double precision function FullA3g2G(i1,i2,i3,iG,renscale2)
      implicit none
      integer :: i1,i2,i3,iG
      double precision :: renscale2,ave,A3g2G,facVV,facA3g2G
      double precision :: pi,qnf
      parameter(pi=3.141592653589793238d0)
      parameter(qnf=0.333333333333333333d0)
      integer :: nf,nc
      double precision :: b0,as,amz
      double complex :: zewfac,zewnull
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /eweakZ/ amz,zewfac(4),zewnull(4)
!$omp threadprivate(/eweakZ/)
      
      if(iG.gt.2)then
         ave=1d0/4d0/(nc**2-1d0)**2
      else
         ave=1d0/4d0/(nc**2-1d0)
      endif   

      facVV = ave*(4d0*pi*as)**3*(4d0*pi*amz)
     .     *nc**2*(nc**2-1d0)/(16d0*pi**4) ! (a factor 1/4 absorbed into A3g2G)


      facA3g2G = qnf**2*(nc**2-4d0)/nc**3*facVV
      
      FullA3g2G = facA3g2G*A3g2G(i1,i2,i3,iG,renscale2)
      
      end function FullA3g2G

!###############################################

      double precision function FullA2g2GG(i1,i2,iG1,iG2,renscale2)
      implicit none
      integer :: i1,i2,iG1,iG2
      double precision :: renscale2,ave,A3g2G,facVV,facA2g2GG
      double precision :: pi,q2nf
      parameter(pi=3.141592653589793238d0)
      parameter(q2nf=1.22222222222222222d0)
      integer :: nf,nc
      double precision :: b0,as,amz
      double complex :: zewfac,zewnull
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /eweakZ/ amz,zewfac(4),zewnull(4)
!$omp threadprivate(/eweakZ/)
      
      if(iG1.gt.2.and.iG2.gt.2)then
         ave=1d0/4d0/(nc**2-1d0)**2
      elseif(iG1.le.2.and.iG2.le.2)then
         ave=1d0/4d0
      else
         ave=1d0/4d0/(nc**2-1d0)
      endif   

      facVV = ave*(4d0*pi*as)**2*(4d0*pi*amz)**2
     .     *nc**3/(16d0*pi**4) ! (a factor 1/4 absorbed into A3g2G)

      facA2g2GG = q2nf**2*(nc**2-1d0)/nc**3*facVV

      FullA2g2GG = facA2g2GG*A3g2G(i1,i2,iG1,iG2,renscale2)

!      write(*,*) "FullA2g2GG", ave,facA2g2GG,FullA2g2GG
      
      end function FullA2g2GG

!###############################################
      
      double precision function A3g2G(i1,i2,i3,iG,renscale2)
      implicit none
      integer :: i1,i2,i3,iG,ihel,iflip
      double precision :: renscale2,ampsq
      double complex :: zamp,zAgam41w
      integer :: ieorder
      common/order/ieorder
      
      ampsq =0d0

      do iflip=0,1
         do ihel=1,8
            zamp = zAgam41w(i1,i2,i3,iG,ihel,iflip,renscale2)
            ampsq = ampsq + dreal(dconjg(zamp)*zamp)
         enddo
      enddo

      if(ieorder.le.-1)then
         A3g2G = 0d0
      elseif(ieorder.eq.0)then   
         A3g2G = ampsq
      endif

      A3g2G = A3g2G/4d0 ! does this factor belong here? 
      
c     charge factor (sum q)^2 = qnf^2 = 1/9 in qcdnorm
      
      end function A3g2G

!###############################################
!
!     a wrapper generating all helicity configurations
!     out of the 4 indepedent ones (ihel=1,2,3,4)
!      
      double complex function zAgam41w(i1,i2,i3,i4,ihel,iflip,renscale2)
      implicit none
      integer :: i1,i2,i3,i4,ihel,iflip
      double precision :: renscale2
      double complex :: zAgam41

      if(ihel.eq.1)then         ! ++++
         zAgam41w = zAgam41(i1,i2,i3,i4,1,iflip,renscale2)
      elseif(ihel.eq.2)then     ! -+++
         zAgam41w = zAgam41(i1,i2,i3,i4,2,iflip,renscale2)
      elseif(ihel.eq.3)then     ! --++
         zAgam41w = zAgam41(i1,i2,i3,i4,3,iflip,renscale2)
      elseif(ihel.eq.4)then     ! -+-+
         zAgam41w = zAgam41(i1,i2,i3,i4,4,iflip,renscale2)
      elseif(ihel.eq.5)then     ! +-++
         zAgam41w = zAgam41(i2,i3,i4,i1,2,iflip,renscale2)
      elseif(ihel.eq.6)then     ! ++-+
         zAgam41w = zAgam41(i3,i4,i1,i2,2,iflip,renscale2)
      elseif(ihel.eq.7)then     ! +++-
         zAgam41w = zAgam41(i4,i1,i2,i3,2,iflip,renscale2)
      elseif(ihel.eq.8)then     ! -++-
         zAgam41w = zAgam41(i4,i1,i2,i3,3,iflip,renscale2)         
      endif   
      
      end function zAgam41w 
      
!###############################################
!
!     the 3g1G loop amplitude, constructed following hep-ph/9409393
!      
      double complex function zAgam41(i1,i2,i3,i4,ihel,iflip,renscale2)
      use KinData_mod      
      integer :: i1,i2,i3,i4,ihel,iflip
      double precision :: renscale2
      double precision :: s12,s13,s14,s23,s24,s34
      double complex :: zFfs,zzLograt,zi,zA4gtree
      double complex :: za12,za13,za14,za23,za24,za34
      double complex :: zb12,zb13,zb14,zb23,zb24,zb34
      double precision :: pi
      parameter(pi=3.141592653589793238d0)
      
      s12=s4(i1,i2)
      s13=s4(i1,i3)
      s14=s4(i1,i4)
      s23=s4(i2,i3)
      s24=s4(i2,i4)
      s34=s4(i3,i4)
      
************************************
*             iflip=0   iflip=1    *
*             gggg      gggg       *
*     ihel=1  ++++      ----       *
*     ihel=2  -+++      +---       *
*     ihel=3  --++      ++--       *
*     ihel=4  -+-+      +-+-       *
************************************

      if(iflip.eq.0)then
         za12=zA4(i1,i2)
         za13=zA4(i1,i3)
         za14=zA4(i1,i4)
         za23=zA4(i2,i3)
         za24=zA4(i2,i4)
         za34=zA4(i3,i4)
         zb12=zB4(i1,i2)
         zb13=zB4(i1,i3)
         zb14=zB4(i1,i4)
         zb23=zB4(i2,i3)
         zb24=zB4(i2,i4)
         zb34=zB4(i3,i4)         
      elseif(iflip.eq.1)then
         za12=-zB4(i1,i2)
         za13=-zB4(i1,i3)
         za14=-zB4(i1,i4)
         za23=-zB4(i2,i3)
         za24=-zB4(i2,i4)
         za34=-zB4(i3,i4)
         zb12=-zA4(i1,i2)
         zb13=-zA4(i1,i3)
         zb14=-zA4(i1,i4)
         zb23=-zA4(i2,i3)
         zb24=-zA4(i2,i4)
         zb34=-zA4(i3,i4)                  
      endif   

      zi = (0d0,1d0)

      if(ihel.eq.1)then ! 1/(4pi)^2 absorbed into facVV
         zAgam41 = -zi/3d0*(
     .        -s12*s23/za12/za23/za34/za14
     .        +s12*s24/za12/za24/za34/za13
     .        +s14*s24/za14/za24/za23/za13)
      elseif(ihel.eq.2)then ! 1/(4pi)^2 absorbed into facVV
         zAgam41 = -zi/3d0*(
     .        -za24*zb24**3/zb12/za23/za34/zb14
     .        +za23*zb23**3/zb12/za24/za34/zb13
     .        +za34*zb34**3/zb14/za24/za23/zb13)
      elseif(ihel.eq.3)then
         zAgam41 = 2d0/3d0*zzLograt(renscale2,-s23)
     .        *zA4gtree(i1,i2,i3,i4,3,iflip)
     .        +2d0/3d0*zzLograt(renscale2,-s24)
     .        *zA4gtree(i1,i2,i4,i3,3,iflip)
     .        +(1d0/3d0
     .        *(zzLograt(renscale2,-s14)+zzLograt(renscale2,-s24))
     .        -zFfs(i1,i4,i2,i3))
     .        *zA4gtree(i1,i4,i2,i3,4,iflip)
      elseif(ihel.eq.4)then
         zAgam41 = (1d0/3d0
     .        *(zzLograt(renscale2,-s12)+zzLograt(renscale2,-s23))
     .        -zFfs(i1,i2,i3,i4))
     .        *zA4gtree(i1,i2,i3,i4,4,iflip)
     .        +2d0/3d0*zzLograt(renscale2,-s12)
     .        *zA4gtree(i3,i1,i2,i4,3,iflip)         
     .        +2d0/3d0*zzLograt(renscale2,-s14)
     .        *zA4gtree(i3,i1,i4,i2,3,iflip)
      endif   
            
      end function zAgam41

!###############################################      
!
!     the 4g tree amplitude
!     
      double complex function zA4gtree(i1,i2,i3,i4,ihel,iflip)
      use KinData_mod
      integer :: i1,i2,i3,i4,ihel,iflip
      double complex :: zi,zd
      double complex :: za12,za13,za14,za23,za24,za34
      double complex :: zb12,zb13,zb14,zb23,zb24,zb34
      
************************************
*             iflip=0   iflip=1    *
*             gggg      gggg       *
*     ihel=1  ++++      ----       *
*     ihel=2  -+++      +---       *
*     ihel=3  --++      ++--       *
*     ihel=4  -+-+      +-+-       *
************************************

      if(iflip.eq.0)then
         za12=zA4(i1,i2)
         za13=zA4(i1,i3)
         za14=zA4(i1,i4)
         za23=zA4(i2,i3)
         za24=zA4(i2,i4)
         za34=zA4(i3,i4)
         zb12=zB4(i1,i2)
         zb13=zB4(i1,i3)
         zb14=zB4(i1,i4)
         zb23=zB4(i2,i3)
         zb24=zB4(i2,i4)
         zb34=zB4(i3,i4)         
      elseif(iflip.eq.1)then
         za12=-zB4(i1,i2)
         za13=-zB4(i1,i3)
         za14=-zB4(i1,i4)
         za23=-zB4(i2,i3)
         za24=-zB4(i2,i4)
         za34=-zB4(i3,i4)
         zb12=-zA4(i1,i2)
         zb13=-zA4(i1,i3)
         zb14=-zA4(i1,i4)
         zb23=-zA4(i2,i3)
         zb24=-zA4(i2,i4)
         zb34=-zA4(i3,i4)                  
      endif   

      zi = (0d0,1d0)

      zd = -za12*za23*za34*za14
      
      if(ihel.eq.1)then
         zA4gtree = (0d0,0d0)
      elseif(ihel.eq.2)then
         zA4gtree = (0d0,0d0)         
      elseif(ihel.eq.3)then
         zA4gtree = zi*za12**4/zd         
      elseif(ihel.eq.4)then
         zA4gtree = zi*za13**4/zd         
      endif   
      
      end function zA4gtree

!###############################################
!
!     finite, scale independent pieces of A^f+A^s
!     (see Appendix II of hep-ph/9409393) 
!      
      double complex function zFfs(i1,i2,i3,i4)
      use KinData_mod      
      integer :: i1,i2,i3,i4
      double precision :: s12,s13,s14,s23,s24,s34
      double complex :: zFf,zFs,zzLograt
      double precision :: pi
      parameter(pi=3.141592653589793238d0)      

      s12=s4(i1,i2)
      s13=s4(i1,i3)
      s14=s4(i1,i4)
      s23=s4(i2,i3)
      s24=s4(i2,i4)
      s34=s4(i3,i4)

      zFf = -0.5d0*s12*s23/s13**2*(zzLograt(-s12,-s23)**2+pi**2)
     .     +0.5d0*(s12-s23)/s13*zzLograt(-s12,-s23) 

      zFs = (-s12*s23/s13**2)*(-s12*s23/s13**2
     .     *(zzLograt(-s12,-s23)**2+pi**2)
     .     +(s12-s23)/s13*(1d0+s13**2/s12/s23/6d0)
     .     *zzLograt(-s12,-s23)+1d0)

      zFfs = zFf + zFs
      
      end function zFfs

!###############################################
