c--- libGJborn --------------------------------------------------------------
c
c     This library contains all the Born level MEs for gamma+jet.
c
c----------------------------------------------------------------------------

      function fullB1g0G(iqb,ig1,iq,iG)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16(z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /eweakZ/ amz,zewfac(4),zewnull(4)
!$omp threadprivate(/eweakZ/)

      if(ig1.gt.2) then
         ave = 1/4d0/nc**2
      else
         ave = 1/4d0/nc/(nc**2-1d0)
      endif   

      fac = ave*(4d0*pi*as)*(4d0*pi*amz)*4d0*(nc**2-1d0)
      
      FullB1g0G = B1g0G(iqb,ig1,iq,iG)
      
      FullB1g0G = fac*FullB1g0G

      return
      end

*************************************************************

      function B1g0G(iqb,ig1,iq,iG)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16(z)
      common /eweakZ/ amz,zewfac(4),zewnull(4)
!$omp threadprivate(/eweakZ/)
      common /BZFlav/nflav
!$omp threadprivate(/BZFlav/)
      dimension ihel2(2)

      call eweakV1qq(nflav,0d0)

      zwt = 0d0
      
      ihel2(1) = 2
      ihel2(2) = 1

      do iflip=0,1
         do ihel=1,2
            ztree = zA4tree(iqb,ig1,iG,iq,ihel,iflip)
     .            + zA4tree(iqb,iG,ig1,iq,ihel2(ihel),iflip) 
            zwt = zwt + dconjg(ztree)*ztree
         enddo
      enddo

      B1g0G = abs(zewfac(1))**2*real(zwt)/2d0

      return
      end
      
************************************************************************
* A wrapper function for B1g0G which inherits the flavour of the first
* pair of quarks from the C0g0G common block.
*
*This must be called in the maple code for 4 quark processes where the
* reduced matrix element has the flavour of the first pair of quarks
*
* NB: This will need further modification for NNLO 4quark+1g processes.

      function f1B1g0G(i1,i2,i3,i4)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /CZFlav/nf1,nf2
!$omp threadprivate(/CZFlav/)
      common /BZFlav/nf1g
!$omp threadprivate(/BZFlav/)

      if(nf1.ne.0)then
         nf1g = nf1
         f1B1g0G = B1g0G(i1,i2,i3,i4)
      else
         print *, "Error in f1B1g0G: flavours have not been initialised"
         stop
      endif

      return
      end

************************************************************************
* A wrapper function for B1g0G which inherits the flavour of the second
* pair of quarks from the C0g0G common block.
*
* This must be called in the maple code for 4 quark processes where the
* reduced matrix element has the flavour of the second pair of quarks
*
* NB: This will need further modification for NNLO 4quark+1g processes.

      function f2B1g0G(i1,i2,i3,i4)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /CZFlav/nf1,nf2
!$omp threadprivate(/CZFlav/)
      common /BZFlav/nf1g
!$omp threadprivate(/BZFlav/)

      if(nf2.ne.0)then
         nf1g = nf2
         f2B1g0G = B1g0G(i1,i2,i3,i4)
      else
         print *, "Error in f2B1g0G: flavours have not been initialised"
         stop
      endif

      return
      end

************************************************************************
*
* A symmetrised version of the B1g0G over the qqb pair.
*

      function sB1g0G(i1,i2,i3,i4)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      sB1g0G = 0.5d0*(B1g0G(i1,i2,i3,i4)
     .               +B1g0G(i3,i2,i1,i4))


      return
      end

************************************************************************
*
* A symmetrised version of the B1g0G over the qqb pair, taking the
* flavour from the second quark pair of C-type matrix elements
*
      function sf1B1g0G(i1,i2,i3,i4)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      sf1B1g0G = 0.5d0*(f1B1g0G(i1,i2,i3,i4)
     .                 +f1B1g0G(i3,i2,i1,i4))


      return
      end

      function sf2B1g0G(i1,i2,i3,i4)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      sf2B1g0G = 0.5d0*(f2B1g0G(i1,i2,i3,i4)
     .                 +f2B1g0G(i3,i2,i1,i4))


      return
      end
      
*************************************************************

      function zA4tree(iqb,ig1,ig2,iq,ihel,iflip)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16(z)
      parameter(pi=3.141592653589793238d0)
      common /eweakZ/ amz,zewfac(4),zewnull(4)
!$omp threadprivate(/eweakZ/)
      common /BZFlav/nflav
!$omp threadprivate(/BZFlav/)

      call eweakV1qq(nflav,0d0)

      zi = (0d0,1d0)
      
      j1 = iqb
      j2 = iq
      j3 = ig1
      j4 = ig2
     
         za12=zA4(j1,j2)
         za21=zA4(j2,j1)
         za13=zA4(j1,j3)
         za31=zA4(j3,j1)
         za14=zA4(j1,j4)
         za41=zA4(j4,j1)
         za23=zA4(j2,j3)
         za32=zA4(j3,j2)
         za24=zA4(j2,j4)
         za42=zA4(j4,j2)
         za34=zA4(j3,j4)
         za43=zA4(j4,j3)

c                     qb q g1 g2
c    iflip=0 ihel=1   -+-+
c    iflip=0 ihel=2   -++-
c    iflip=1 ihel=1   +-+-
c    iflip=1 ihel=2   +--+

         if(ihel.eq.1.and.iflip.eq.0)then
            zn = za13**3*za23
         elseif(ihel.eq.2.and.iflip.eq.0)then
            zn = za14**3*za24
         elseif(ihel.eq.1.and.iflip.eq.1)then
            zn = -za24**3*za14
         elseif(ihel.eq.2.and.iflip.eq.1)then
            zn = -za23**3*za13
         endif      

      zd = za12*za23*za34*za41

      zA4tree = zi*zn/zd

      return
      end

*************************************************************
