#define USE_COMPLEX_MASS_SCHEME


*************************************************************

      function D0g0_EW(iqb1,iq2,iqb2,iq1)
      use Parameters_mod
      use KinData_mod
      implicit none
      integer, parameter :: nc = 3
      integer :: iqb1,iq1,iqb2,iq2
      real(8) :: D0g0_EW,s,t,u
      complex(8) :: vprop, zamp

      real(8) :: emz,ezwidth
      common /Zmass/ emz,ezwidth
      real(8) :: emw,ewwidth
      common /Wmass/ emw,ewwidth
      real(8) :: amzW,stw,ewfac
      common /eweakW/ amzW,stw,ewfac
!$omp threadprivate(/eweakW/)
      real(8) :: ele,ere,sin2w,enQ(-5:5),enL(5),enR(5)
      common /LR/ ele,ere,sin2w,enQ,enL,enR
#ifdef USE_COMPLEX_MASS_SCHEME
      complex(8) :: zele,zere,zsin2w,zenL(5),zenR(5)
      common /zLR/ zele,zere,zsin2w,zenL,zenR
#endif
      integer :: nfC1,nfC2
      common /CZFlav/ nfC1,nfC2
!$omp threadprivate(/CZFlav/)

      s = s4(iq1,iqb1)
      t = s4(iq1,iqb2)
      u = s4(iq1,iq2)
      
      D0g0_EW = (nc**2-1d0)/2d0  ! colour factor
      !> kinematic factor (gs^2 * e^2 stripped)
      D0g0_EW = D0g0_EW * 4d0 * u**2 / s / t
      
      if (nfC1==nfC2) then
        !> neutral-current (photon + Z-boson)
        vprop = s / dcmplx((s - emz**2), ezwidth*emz)
     .        + t / dcmplx((t - emz**2), ezwidth*emz)
        zamp  = 4d0*enQ(nfC1)**2 + vprop*( zenL(nfC1)**2 + zenR(nfC1)**2 )
      else
        !> charged current (W-boson)
        vprop = s / dcmplx((s - emw**2), ewwidth*emw)
        ! use zere*zsin2w = 2*sin^2(th_W) (complex, see broadcast_param)
        zamp  = vprop / zere / zsin2w 
      end if

      D0g0_EW = D0g0_EW * 2d0 * real(zamp)

      return
      end
      
*************************************************************
*     A wrapper for the NC matrix element (needed for EW+frag subtraction)
      
      function D0g0_EWNC(iqb1,iq2,iqb2,iq1)
      implicit none
      integer :: iqb1,iq1,iqb2,iq2
      real(8) :: D0g0_EW,D0g0_EWNC
      integer :: nfC1,nfC2
      common /CZFlav/ nfC1,nfC2
!$omp threadprivate(/CZFlav/)

      if (nfC1.ne.nfC2) then
         print *, "D0g0_EWNC called with non-identical flavours!",nfC1,nfC2
         stop
      endif
      
      D0g0_EWNC = D0g0_EW(iqb1,iq2,iqb2,iq1)
      
      return
      end

*************************************************************
*     A wrapper for the CC matrix element (needed for EW+frag subtraction)
      
      function D0g0_EWCC(iqb1,iq2,iqb2,iq1)
      implicit none
      integer :: iqb1,iq1,iqb2,iq2
      real(8) :: D0g0_EW,D0g0_EWCC
      integer :: nfC1,nfC2
      common /CZFlav/ nfC1,nfC2
!$omp threadprivate(/CZFlav/)

      if (nfC1.eq.nfC2) then
         print *, "D0g0_EWCC called with identical flavours!",nfC1,nfC2
         stop
      endif
      
      D0g0_EWCC = D0g0_EW(iqb1,iq2,iqb2,iq1)
      
      return
      end
