************************************************************************
!XC This file is based on MCFM-8.2 source code.
!  J.~M.~Campbell and R.~K.~Ellis,
!  %``An Update on vector boson pair production at hadron colliders,''
!  Phys.\ Rev.\ D {\bf 60} (1999) 113006
!  doi:10.1103/PhysRevD.60.113006
!  [hep-ph/9905386].

!  J.~M.~Campbell, R.~K.~Ellis and W.~T.~Giele,
!  %``A Multi-Threaded Version of MCFM,''
!  Eur.\ Phys.\ J.\ C {\bf 75} (2015) no.6,  246
!  doi:10.1140/epjc/s10052-015-3461-2
!  [arXiv:1503.06182 [physics.comp-ph]].

!  R.~Boughezal, J.~M.~Campbell, R.~K.~Ellis, C.~Focke, W.~Giele, X.~Liu, F.~Petriello and C.~Williams,
!  %``Color singlet production at NNLO in MCFM,''
!  Eur.\ Phys.\ J.\ C {\bf 77} (2017) no.1,  7
!  doi:10.1140/epjc/s10052-016-4558-y
!  [arXiv:1605.08011 [hep-ph]].

!XC The reference for Sushi is https://sushi.hepforge.org/
!  R.~V.~Harlander, S.~Liebler and H.~Mantler,
!  %``SusHi: A program for the calculation of Higgs production in gluon fusion and bottom-quark annihilation in the Standard Model and the MSSM,''
!  Comput.\ Phys.\ Commun.\  {\bf 184} (2013) 1605
!  doi:10.1016/j.cpc.2013.02.006
!  [arXiv:1212.3249 [hep-ph]].

!  R.~V.~Harlander, S.~Liebler and H.~Mantler,
!  %``SusHi Bento: Beyond NNLO and the heavy-top limit,''
!  Comput.\ Phys.\ Commun.\  {\bf 212} (2017) 239
!  doi:10.1016/j.cpc.2016.10.015
!  [arXiv:1605.03190 [hep-ph]].
************************************************************************
! This file is part of SusHi.
! 
! It includes the calculation of loop integrals
! as well as logarithms and polylogarithms.
! 
c-{{{ without QCD-Loop

c-{{{ 2-point-function

      function Integral2(ma,mq2)
      implicit none
      double complex Integral2
      double precision ma,m1,mq2,re,im,Pi
      parameter (Pi=3.141592653589793238462643d0)
      double complex rslt(0:2)
      m1=ma/mq2
      if(m1.eq.0.d0) then
         re = -2.d0
         im = 0.d0
      elseif(m1.lt.0.d0) then
         re = -2.d0*dsqrt(1.d0-4.d0/m1)*
     &        dlog((dsqrt(-m1)+dsqrt(-m1+4.d0))/2.d0)
         im = 0.d0
      elseif(m1.lt.4.d0) then
         re = -2.d0*dsqrt(4.d0/m1-1.d0)*dasin(dsqrt(m1)/2.d0)
         im = 0.d0
      elseif(m1.gt.4.d0) then
         re = -2.d0*dsqrt(1-4.d0/m1)*(dlog((dsqrt(m1)+dsqrt(m1-4.d0))/
     &(2.d0)))
         im = 2.d0*dsqrt(1.d0-4.d0/m1)*Pi/2.d0
      endif
      Integral2 = (1.d0,0.d0)*re + (0.d0,1.d0)*im
      end

c-}}}
c-{{{ 3-point-function

      function Integral3(ma,mb,mq2)
      implicit none
      double complex Integral3
      double precision ma,mb,mq2,m1,m2,re,im,Pi
      parameter (Pi=3.141592653589793238462643D0)
      double complex rslt(0:2)
      m1=ma/mq2
      m2=mb/mq2

      if(m1.eq.0.d0) then
         re=0.d0
         im=0.d0
      elseif(m1.lt.0.d0) then
         re=(dlog(-((-2.d0+m1+dsqrt(m1*(-4+m1)))/2.d0))**2)/4.d0
         im=0.d0
      elseif(m1.lt.4.d0) then
         re=-dasin(dsqrt(m1/4.d0))**2
         im=0.d0
      elseif(m1.gt.4.d0) then
         re=(dlog((-2.d0+m1+dsqrt(m1*(-4.d0+m1)))/(2.d0))**2-Pi**2)/4.d0
         im=-Pi*dlog((-2.d0+m1+dsqrt(m1*(-4.d0+m1)))/2.d0)/(m1-m2)
      endif

      if(m2.eq.0.d0) then
      elseif(m2.lt.0.d0) then
         re=re-(dlog(-((-2+m2+dsqrt(m2*(-4+m2)))/(2.d0)))**2)/4.d0
      elseif(m2.lt.4.d0) then
         re=re+dasin(dsqrt(m2/4.d0))**2
      elseif(m2.gt.4.d0) then
        re=re-(dlog((-2.d0+m2+dsqrt(m2*(-4.d0+m2)))/2.d0)**2-Pi**2)/4.d0
        im=im+Pi*dlog((-2.d0+m2+dsqrt(m2*(-4.d0+m2)))/(2.d0))/(m1-m2)
      endif
      Integral3=((1.d0,0.d0)*2.d0/(m1-m2)*re+(0.d0,1.d0)*im)/mq2

      end

c-}}}
c-{{{ 4-point-function

      function Integral4(ma,mb,mc,mq2)
      implicit none
      double complex Integral4,wrz,p(0:3)
      double precision ma,mb,mc,mq2,m1,m2,m3,x(1:2),pn(0:3),tmp,
     &dli2,arg,zeta2,zeta3,pi
      integer i,k
      double complex s_cli2, cli2
      COMMON/ZETACONST/ZETA2,ZETA3
      parameter (Pi=3.141592653589793238462643D0)
      double complex rslt(0:2)

      m1=ma/mq2
      m2=mb/mq2
      m3=mc/mq2
      pn(0)=0.d0
      pn(1)=m1
      pn(2)=m1+m2+m3
      pn(3)=m3

      tmp=1+m1*m3/m2/2.d0
      if(tmp**2.eq.1.d0) then
         Integral4=1/6.d0/mq2**2
         return
      endif

      x(2)=-tmp-dsqrt(tmp**2-1)
      x(1)=1/x(2)

      Integral4=(2.d0,0.d0)*(2*dli2(1+x(2))+dlog(-x(2))**2/2)
      do i=1,3
         wrz=cdsqrt((1.d0,0.d0)*(1-pn(i)/2.d0)**2-1)
         if(pn(i).eq.0.d0) then
            p(i) = 1.d0
         elseif(pn(i).gt.0.d0) then
            p(i)=1/(1-pn(i)/2.d0-wrz)
         else
            p(i)=1-pn(i)/2.d0+wrz
         endif

         do k=1,2
            if(pn(i).le.0.d0) then
               arg = real(p(i)*x(k))
               Integral4=Integral4+(-1)**(i+k)*
     &              (-2*dli2(1+1/arg)-dlog(-arg)**2/2)
            elseif(pn(i).ge.4.d0) then
               arg = real(p(i)*x(k))
               Integral4=Integral4+(-1)**(i+k) * (
     &              dli2(-1/arg)-zeta2-dlog(1+1/arg)*dlog(arg)
     &              -(dlog(arg)-(0.d0,1.d0)*Pi)**2/4.d0
     &              -(0.d0,1.d0)*Pi*dlog(1+arg))*2.d0
            else
               Integral4=Integral4+(-1)**(i+k)*(s_cli2(1+p(i)*x(k))
     &              +s_cli2(1+x(k)/p(i)))
            endif
         enddo
      enddo
      Integral4=Integral4/dsqrt(tmp**2-1)/m2/mq2**2/2.d0

      end

c-}}}

c-}}}

c-{{{ complex dilogarithm (from HIGLU)

      COMPLEX*16 FUNCTION s_CLI2(X)
C--COMPLEX DILOGARITHM (SPENCE-FUNCTION)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMPLEX*16 X,Y,SUSHI_CLI2
      COMMON/ZETACONST/ZETA2,ZETA3
      s_CLI2 = 0.D0
      ZERO=1.D-40
      XR=DREAL(X)
      XI=DIMAG(X)
      R2=XR*XR+XI*XI
      IF(R2.LE.ZERO)THEN
        s_CLI2=X
        RETURN
      ENDIF
      RR=XR/R2
      IF(R2.EQ.1.D0.AND.XI.EQ.0.D0)THEN
        IF(XR.EQ.1.D0)THEN
          s_CLI2=DCMPLX(ZETA2)
        ELSE
          s_CLI2=-DCMPLX(ZETA2/2.D0)
        ENDIF
        RETURN
      ELSEIF(R2.GT.1.D0.AND.RR.GT.0.5D0)THEN
        Y=(X-1.D0)/X
       s_CLI2=SUSHI_CLI2(Y)+ZETA2-CDLOG(X)
     &     *CDLOG(1.D0-X)+0.5D0*CDLOG(X)**2
        RETURN
      ELSEIF(R2.GT.1.D0.AND.RR.LE.0.5D0)THEN
        Y=1.D0/X
        s_CLI2=-SUSHI_CLI2(Y)-ZETA2-0.5D0*CDLOG(-X)**2
        RETURN
      ELSEIF(R2.LE.1.D0.AND.XR.GT.0.5D0)THEN
        Y=1.D0-X
        s_CLI2=-SUSHI_CLI2(Y)+ZETA2-CDLOG(X)*CDLOG(1.D0-X)
       RETURN
      ELSEIF(R2.LE.1.D0.AND.XR.LE.0.5D0)THEN
        Y=X
        s_CLI2=SUSHI_CLI2(Y)
        RETURN
      ENDIF
      END

      COMPLEX*16 FUNCTION SUSHI_CLI2(X)
C--TAYLOR-EXPANSION FOR COMPLEX DILOGARITHM (SPENCE-FUNCTION)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMPLEX*16 X,Z
      COMMON/BERNOULLI/B2(18),B12(18),B3(18),Once
!$omp threadprivate(/BERNOULLI/)
      COMMON/POLY/NBER
      N=NBER-1
      Z=-CDLOG(1.D0-X)
      SUSHI_CLI2=B2(NBER)
      DO 111 I=N,1,-1
        SUSHI_CLI2=Z*SUSHI_CLI2+B2(I)
111   CONTINUE
      SUSHI_CLI2=Z**2*SUSHI_CLI2+Z
      RETURN
      END

      SUBROUTINE SUSHI_BERNINI(N)
C--INITIALIZATION OF COEFFICIENTS FOR POLYLOGARITHMS
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION B(18),PB(19)
      COMMON/BERNOULLI/B2(18),B12(18),B3(18),Once
!$omp threadprivate(/BERNOULLI/)
      COMMON/ZETACONST/ZETA2,ZETA3
      COMMON/POLY/NBER
 
      NBER=N
      PI=4.D0*DATAN(1.D0)
      Once=1d0

      B(1)=-1.D0/2.D0
      B(2)=1.D0/6.D0
      B(3)=0.D0
      B(4)=-1.D0/30.D0
      B(5)=0.D0
      B(6)=1.D0/42.D0
      B(7)=0.D0
      B(8)=-1.D0/30.D0
      B(9)=0.D0
      B(10)=5.D0/66.D0
      B(11)=0.D0
      B(12)=-691.D0/2730.D0
      B(13)=0.D0
      B(14)=7.D0/6.D0
      B(15)=0.D0
      B(16)=-3617.D0/510.D0
      B(17)=0.D0
      B(18)=43867.D0/798.D0
      ZETA2=PI**2/6.D0
      ZETA3=1.202056903159594D0
 
      DO 995 I=1,18
        B2(I)=B(I)/SUSHI_FACULT(I+1)
        B12(I)=DFLOAT(I+1)/SUSHI_FACULT(I+2)*B(I)/2.D0
        PB(I+1)=B(I)
        B3(I)=0.D0
995   CONTINUE
      PB(1)=1.D0
      DO 996 I=1,18
      DO 996 J=0,I
       B3(I)=B3(I)+PB(J+1)*PB(I-J+1)/SUSHI_FACULT(I-J)/SUSHI_FACULT(J+1)
     .                                            /DFLOAT(I+1)
996   CONTINUE
 
      RETURN
      END
 
      DOUBLE PRECISION FUNCTION SUSHI_FACULT(N)
C--DOUBLE PRECISION VERSION OF FACULTY
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      SUSHI_FACULT=1.D0
      IF(N.EQ.0)RETURN
      DO 999 I=1,N
        SUSHI_FACULT=SUSHI_FACULT*DFLOAT(I)
999   CONTINUE
      RETURN
      END

