
      !###########################################
      !########### 2-loop library ################
      !###########################################

      module libH3p2loop
      implicit none

      contains
      real*8 function fullA3g2H(s5,i1,i2,i3,order,col1L,col2L
     . ,renscale2,cSub,ieorder)
       use h3pC_Coeff,only:initG0LC,initG1LC,initG2LC
     . ,setCataniSub,setIEorder
      implicit none
      !Function to evaluate the matrix element for h+3g process
      !See Kauffman, Desai, Risal, PHYS. REV. D 55, 4005 (1997)

      !colour1L(1) : !N    (A3g1H)
      !colour1L(2) : !N^-1 (At3g1H) !returns 0d0
      !colour1L(3) : !Nf   (Ah3g1H)

      !colour2L(1) : !N^2  (A3g2H)
      !colour2L(2) : !1    (At3g2H) !returns 0d0
      !colour2L(3) : !N^-2 (Att3g2H) !returns 0d0
      !colour2L(4) : !Nf/N (Aht3g2H)
      !colour2L(5) : !NNf  (Ah3g2H)
      !colour2L(6) : !Nf^2 (Ahh3g2H)

      !qcdNorm(1 ) = 1.d0
      !qcdNorm(2 ) = Nc
      !qcdNorm(3 ) = 1.d0/Nc
      !qcdNorm(4 ) = Nf
      !qcdNorm(5 ) = Nc**2
      !qcdNorm(6 ) = 1.d0
      !qcdNorm(7 ) = 1.d0/Nc**2
      !qcdNorm(8 ) = Nf/Nc
      !qcdNorm(9 ) = Nf*Nc
      !qcdNorm(10) = Nf**2
      !-------------------
c      real*8, dimension(4,5), intent(in) :: p5
      real*8, dimension(5,5), intent(in) :: s5
      integer, intent(in) :: order!loop order
      integer, intent(in) :: i1,i2,i3
      logical, intent(in), dimension(3) :: col1L!see above
      logical, intent(in), dimension(6) :: col2L!see above
      logical, intent(in), optional :: cSub!Switch for Catani sub-
                                           ! traction at 1-loop
      integer, intent(in), optional :: ieorder!Set the epsilon order
                                              ! order of the coeff.
      real*8, intent(in) :: renscale2
      !internal
      real*8, dimension(3,3) :: li
      logical, dimension(3) :: colour1L
      logical, dimension(6) :: colour2L
      real*8, dimension(4,3) :: p
      complex*16, dimension(8)   :: coeff0L
      complex*16, dimension(8,3,0:2) :: coeff1L
      complex*16, dimension(8,3) :: coeff1LS
      complex*16, dimension(8,6) :: coeff2L
      real*8, dimension(8) :: helSq
      real*8 fin0L,fin1L,fin2L
      integer :: i,j,k,l,m
      real*8 :: Nc
      integer :: Nf
      integer, dimension(3,3) :: mask
      integer, dimension(0:2,0:2) :: maskeps
      integer, dimension(0:2) :: veceps
      real*8 :: s123
      real*8, dimension(6,3) :: cmat
      real*8, dimension(6)   :: cvec
      real*8, dimension(3)   :: cvec1L
      real*8 :: lr
      logical :: check
      logical :: donothing = .false.


      Nc = 3.d0
      Nf = 5
c      p = p5(:,1:3)
      coeff0L  = (0.d0,0.d0)
      coeff1L  = (0.d0,0.d0)
      coeff1LS = (0.d0,0.d0)
      coeff2L  = (0.d0,0.d0)
      helSq = 0.d0
      fin0L = 0.d0
      fin1L = 0.d0
      fin2L = 0.d0
      mask = 0
      maskeps = 0
      veceps = 0
      check = .false.

      colour1L = col1L
      colour2L = col2L
      li = s5(1:3,1:3)

      s123 = li(i1,i2) + li(i1,i3) + li(i2,i3)
      lr   = log(renscale2/s123)

      !matrix for b0*sig1*lr
      cmat      = 0d0
      cmat(1,1) = 55d0/12d0
      cmat(5,1) = - 5d0/6d0
      cmat(5,3) = cmat(1,1)
      cmat(6,3) = cmat(5,1)
      cmat      = cmat*lr

      !vector for b1*sig0*lr + b0^2*sig0*lr^2
      cvec    = 0d0
      !cvec(1) = 17d0/4d0*lr + 605d0/96d0*lr**2 !n*b1
      cvec(1) = 85d0/12d0*lr + 605d0/96d0*lr**2 !(n+2)*b1
      !cvec(4) = 3d0/8d0*lr !n*b1
      cvec(4) = 5d0/8d0*lr !(n+2)*b1
      !cvec(5) = - 13d0/8d0*lr - 55d0/24d0*lr**2 !n*b1
      cvec(5) = - 65d0/24d0*lr - 55d0/24d0*lr**2 !(n+2)*b1
      cvec(6) = 5d0/24d0*lr**2

      !vector for b0*sig0*lr
      cvec1L    = 0d0
      cvec1L(1) = 11d0/4d0
      cvec1L(3) = -1d0/2d0
      cvec1L    = cvec1L*lr

      if(present(cSub)) then
         call setCataniSub(cSub)
      else!default
         call setCataniSub(.true.)
      endif
      if(present(ieorder)) then
         if((ieorder==4).and.(order==2)) then
            call setIEorder(ieorder)
            maskeps = 0
            maskeps(2,2) = 1
         elseif((ieorder==3).and.(order==2)) then
            call setIEorder(ieorder)
            maskeps = 0
            maskeps(1,2) = 1
            maskeps(2,1) = 1
         elseif((ieorder==2).and.(order==1)) then
            call setIEorder(ieorder)
            veceps = 0
            veceps(2) = 1
         elseif((ieorder==1).and.(order==1)) then
            call setIEorder(ieorder)
            veceps = 0
            veceps(1) = 1
         elseif(ieorder==0) then
            call setIEorder(ieorder)
            maskeps = 0
            veceps = 0
            maskeps(0,0) = 1
            veceps(0) = 1
         else
            fullA3g2H = 0d0
            return
         endif
      else!default
         call setIEorder(0)
         maskeps = 0
         veceps = 0
         maskeps(0,0) = 1
         veceps(0) = 1
      endif

      !------------------------------------------
      !squares of helicity amplitudes
      helSq(1) = helSqG(li,1,1,1,i1,i2,i3)
      helSq(2) = helSqG(li,1,1,0,i1,i2,i3)
      helSq(3) = helSqG(li,1,0,1,i1,i2,i3)
      helSq(4) = helSqG(li,1,0,0,i1,i2,i3)
      helSq(5) = helSqG(li,0,1,1,i1,i2,i3)
      helSq(6) = helSqG(li,0,1,0,i1,i2,i3)
      helSq(7) = helSqG(li,0,0,1,i1,i2,i3)
      helSq(8) = helSqG(li,0,0,0,i1,i2,i3)
      if(order==2) then
         !Switch on the required 1loop self-interferences
         if(colour2L(1)) then
            colour1L(1) = .true.
            mask(1,1) = 1
         endif
         if(colour2L(2)) then
            colour1L(1) = .true.
            colour1L(2) = .true.
            mask(1,2) = 1
            mask(2,1) = 1
         endif
         if(colour2L(3)) then
            colour1L(2) = .true.
            mask(2,2) = 1
         endif
         if(colour2L(4)) then
            colour1L(2) = .true.
            colour1L(3) = .true.
            mask(2,3) = 1
            mask(3,2) = 1
         endif
         if(colour2L(5)) then
            colour1L(1) = .true.
            colour1L(3) = .true.
            mask(1,3) = 1
            mask(3,1) = 1
         endif
         if(colour2L(6)) then
            colour1L(3) = .true.
            mask(3,3) = 1
         endif
      endif
      if(order>=0) then
         call initG0LC(li)!initialise 0L alpha/beta coeffs.
         coeff0L(1) = helCoeffG(1,1,1,i1,i2,i3,1)
         coeff0L(2) = helCoeffG(1,1,0,i1,i2,i3,1)
         coeff0L(3) = helCoeffG(1,0,1,i1,i2,i3,1)
         coeff0L(4) = helCoeffG(1,0,0,i1,i2,i3,1)
         coeff0L(5) = helCoeffG(0,1,1,i1,i2,i3,1)
         coeff0L(6) = helCoeffG(0,1,0,i1,i2,i3,1)
         coeff0L(7) = helCoeffG(0,0,1,i1,i2,i3,1)
         coeff0L(8) = helCoeffG(0,0,0,i1,i2,i3,1)
      endif
      if(order>=1) then
         if(order>1) call setCataniSub(.true.)
         do l = 0,2
            check = .false.
            do j = 0,2
               if(maskeps(l,j)==1) check = .true.
            enddo
            if(check.or.(veceps(l)==1)) then
               call setIEorder(l)
               call initG1LC(li) !initialise 1L alpha/beta coeffs.
               do i = 1,3
                  if(colour1L(i)) then
                     coeff1L(1,i,l) = helCoeffG(1,1,1,i1,i2,i3,i+1)
                     coeff1L(2,i,l) = helCoeffG(1,1,0,i1,i2,i3,i+1)
                     coeff1L(3,i,l) = helCoeffG(1,0,1,i1,i2,i3,i+1)
                     coeff1L(4,i,l) = helCoeffG(1,0,0,i1,i2,i3,i+1)
                     coeff1L(5,i,l) = helCoeffG(0,1,1,i1,i2,i3,i+1)
                     coeff1L(6,i,l) = helCoeffG(0,1,0,i1,i2,i3,i+1)
                     coeff1L(7,i,l) = helCoeffG(0,0,1,i1,i2,i3,i+1)
                     coeff1L(8,i,l) = helCoeffG(0,0,0,i1,i2,i3,i+1)
                     if(present(ieorder))then
                        if(
     .    ((order==2).and.(ieorder>2)).or.
     .    ((order==1).and.(ieorder>0))
     .                  ) then
                           donothing = .true.
                        endif
                     endif
                     if(.not.(donothing))then
                        do k = 1,8
                           if(order>1) coeff1LS(k,i) = coeff1L(k,i,0)
                           coeff1L(k,i,0) = coeff1L(k,i,0)
     .                                   + cvec1L(i)*coeff0L(k)
                        enddo
                     endif
                     donothing = .false.
                  endif
               enddo
            endif
         enddo
      endif
      if(order==2) then
         call setIEorder(ieorder)
         call initG2LC(li)!initialise 2L alpha/beta coeffs.
         do i = 1,6
            if(colour2L(i)) then
               coeff2L(1,i) = helCoeffG(1,1,1,i1,i2,i3,i+4)
               coeff2L(2,i) = helCoeffG(1,1,0,i1,i2,i3,i+4)
               coeff2L(3,i) = helCoeffG(1,0,1,i1,i2,i3,i+4)
               coeff2L(4,i) = helCoeffG(1,0,0,i1,i2,i3,i+4)
               coeff2L(5,i) = helCoeffG(0,1,1,i1,i2,i3,i+4)
               coeff2L(6,i) = helCoeffG(0,1,0,i1,i2,i3,i+4)
               coeff2L(7,i) = helCoeffG(0,0,1,i1,i2,i3,i+4)
               coeff2L(8,i) = helCoeffG(0,0,0,i1,i2,i3,i+4)
               if(present(ieorder))then
                if(ieorder>2)then
                  donothing = .true.
                endif
               endif
               if(.not.(donothing))then
                  do k = 1,8
                     do j = 1,3
                        coeff2L(k,i) =   coeff2L(k,i)
     .                       + cmat(i,j)*coeff1LS(k,j)
                     enddo
                     coeff2L(k,i) = coeff2L(k,i) + cvec(i)*coeff0L(k)
                  enddo
               endif
               donothing = .false.
            endif
         enddo
      endif
      !------------------------------------------
      if(order==2) then
         do i = 1,8   !sum over helicities
            do j = 1,6!sum over colour structures
               fin2L = fin2L
     . + 2.d0*real(coeff2L(i,j)*conjg(coeff0L(i)))
     . *helSq(i)!*qcdNorm(j+4)*qcdNorm(1)
            enddo
         enddo
         do i = 1,8            !sum over helicities
            do j = 1,3         !sum over colour structures
               do k = 1,3      !sum over colour structures
                  do l = 0,2   !sum over poles
                     do m = 0,2!sum over poles
                        fin2L = fin2L
     .                       + real(coeff1L(i,j,l)*conjg(coeff1L(i,k,m))
     .                       *helSq(i)*mask(j,k)*maskeps(l,m))!*qcdNorm(j+1)*qcdNorm(k+1))
                     enddo
                  enddo
               enddo
            enddo
         enddo
      endif
      if(order>=1) then
         do i = 1,8   !sum over helicities
            do j = 1,3!sum over colour structures
               do k = 0,2!sum over poles
                  fin1L = fin1L
     .          + 2.d0*real(coeff1L(i,j,k)*conjg(coeff0L(i))*veceps(k))
     .          *helSq(i) !*qcdNorm(j+1)*qcdNorm(1)
               enddo
            enddo
         enddo
      endif
      if(order>=0) then
         fin0L = sum(helSq*real(coeff0L*conjg(coeff0L)))!*qcdNorm(1)**2
      endif
      !------------------------------------------
      if(order==2) then
         fullA3g2H = fin2L
      elseif(order==1) then
         fullA3g2H = fin1L
      elseif(order==0) then
         fullA3g2H = fin0L
      endif

      end function full A3g2H

!###################################################

      real*8 function fullB1g2H(s5,i1,i2,i3,order,col1L,col2L
     . ,renscale2,cSub,ieorder)
      use h3pC_Coeff,only:initQ0LC,initQ1LC,initQ2LC
     . ,setCataniSub,setIEorder
      !Function to evaluate the matrix element for h+2qg process
      !Helicities label OUTGOING states
      !"i3" is always the gluon
      !i1,i2,i3 = '123','213':qqb,qbq
      !           '132','231':qg,gq
      !           '312','321':qbg,gqb

      !colour1L(1) : !N    (B1g1H)
      !colour1L(2) : !N^-1 (Bt1g1H)
      !colour1L(3) : !Nf   (Bh1g1H)

      !colour2L(1) : !N^2  (B1g2H)
      !colour2L(2) : !1    (Bt1g2H)
      !colour2L(3) : !N^-2 (Btt1g2H)
      !colour2L(4) : !Nf/N (Bht1g2H)
      !colour2L(5) : !NNf  (Bh1g2H)
      !colour2L(6) : !Nf^2 (Bhh1g2H)

      !qcdNorm(1 ) = 1.d0
      !qcdNorm(2 ) = Nc
      !qcdNorm(3 ) = 1.d0/Nc
      !qcdNorm(4 ) = Nf
      !qcdNorm(5 ) = Nc**2
      !qcdNorm(6 ) = 1.d0
      !qcdNorm(7 ) = 1.d0/Nc**2
      !qcdNorm(8 ) = Nf/Nc
      !qcdNorm(9 ) = Nf*Nc
      !qcdNorm(10) = Nf**2
      !-------------------
c      real*8, dimension(4,5), intent(in) :: p5
      real*8, dimension(5,5), intent(in) :: s5
      integer, intent(in) :: order!loop order
      integer, intent(in) :: i1,i2,i3
      logical, intent(in), dimension(3) :: col1L!see above
      logical, intent(in), dimension(6) :: col2L!see above
      logical, intent(in), optional :: cSub!Switch for Catani sub-
                                           ! traction at 1-loop
      integer, intent(in), optional :: ieorder!Set the epsilon order
                                              ! order of the coeff.
      real*8, intent(in) :: renscale2
      !internal
      real*8, dimension(3,3) :: li
      logical, dimension(3) :: colour1L
      logical, dimension(6) :: colour2L
      real*8, dimension(4,3) :: p
      complex*16, dimension(4)   :: coeff0L
      complex*16, dimension(4,3,0:2) :: coeff1L
      complex*16, dimension(4,3) :: coeff1LS
      complex*16, dimension(4,6) :: coeff2L
      real*8, dimension(4) :: helSq
      real*8 fin0L,fin1L,fin2L
      integer :: i,j,k,l,m
      real*8 :: Nc
      integer :: Nf
      integer, dimension(3,3) :: mask
      integer, dimension(0:2,0:2) :: maskeps
      integer, dimension(0:2) :: veceps
      real*8 :: s123
      real*8, dimension(6,3) :: cmat
      real*8, dimension(6)   :: cvec
      real*8, dimension(3)   :: cvec1L
      real*8 :: lr
      logical :: check
      logical :: donothing = .false.


      Nc = 3.d0
      Nf = 5
c      p = p5(:,1:3)
      coeff0L  = (0.d0,0.d0)
      coeff1L  = (0.d0,0.d0)
      coeff1LS = (0.d0,0.d0)
      coeff2L  = (0.d0,0.d0)
      helSq = 0.d0
      fin0L = 0.d0
      fin1L = 0.d0
      fin2L = 0.d0
      mask = 0
      maskeps = 0
      veceps = 0
      check = .false.

      colour1L = col1L
      colour2L = col2L
      li = s5(1:3,1:3)

      s123 = li(i1,i2) + li(i1,i3) + li(i2,i3)
      lr   = log(renscale2/s123)

      !matrix for b0*sig1*lr
      cmat      = 0d0
      cmat(1,1) = 55d0/12d0
      cmat(4,2) = -5d0/6d0
      cmat(2,2) = cmat(1,1)
      cmat(5,3) = cmat(1,1)
      cmat(5,1) = cmat(4,2)
      cmat(6,3) = cmat(4,2)
      cmat      = cmat*lr

      !vector for b1*sig0*lr + b0^2*sig0*lr^2
      cvec    = 0d0
      !cvec(1) = 17d0/4d0*lr + 605d0/96d0*lr**2
      cvec(1) = 85d0/12d0*lr + 605d0/96d0*lr**2
      !cvec(4) = 3d0/8d0*lr
      cvec(4) = 5d0/8d0*lr
      !cvec(5) = - 13d0/8d0*lr - 55d0/24d0*lr**2
      cvec(5) = - 65d0/24d0*lr - 55d0/24d0*lr**2
      cvec(6) = 5d0/24d0*lr**2

      !vector for b0*sig0*lr
      cvec1L    = 0d0
      cvec1L(1) = 11d0/4d0
      cvec1L(3) = -1d0/2d0
      cvec1L    = cvec1L*lr

      if(present(cSub)) then
         call setCataniSub(cSub)
      else!default
         call setCataniSub(.true.)
      endif
      if(present(ieorder)) then
         if((ieorder==4).and.(order==2)) then
            call setIEorder(ieorder)
            maskeps = 0
            maskeps(2,2) = 1
         elseif((ieorder==3).and.(order==2)) then
            call setIEorder(ieorder)
            maskeps = 0
            maskeps(1,2) = 1
            maskeps(2,1) = 1
         elseif((ieorder==2).and.(order==1)) then
            call setIEorder(ieorder)
            veceps = 0
            veceps(2) = 1
         elseif((ieorder==1).and.(order==1)) then
            call setIEorder(ieorder)
            veceps = 0
            veceps(1) = 1
         elseif(ieorder==0) then
            call setIEorder(ieorder)
            maskeps = 0
            veceps = 0
            maskeps(0,0) = 1
            veceps(0) = 1
         else
            fullB1g2H = 0d0
            return
         endif
      else!default
         call setIEorder(0)
         maskeps = 0
         veceps = 0
         maskeps(0,0) = 1
         veceps(0) = 1
      endif

      !------------------------------------------
      !squares of helicity amplitudes
      helSq(1) = helSqQ(li,1,0,1,i1,i2,i3)
      helSq(2) = helSqQ(li,1,0,0,i1,i2,i3)
      helSq(3) = helSqQ(li,0,1,1,i1,i2,i3)
      helSq(4) = helSqQ(li,0,1,0,i1,i2,i3)
      if(order==2) then!Switch on the required 1loop self-interferences
         if(colour2L(1)) then
            colour1L(1) = .true.
            mask(1,1) = 1
         endif
         if(colour2L(2)) then
            colour1L(1) = .true.
            colour1L(2) = .true.
            mask(1,2) = 1
            mask(2,1) = 1
         endif
         if(colour2L(3)) then
            colour1L(2) = .true.
            mask(2,2) = 1
         endif
         if(colour2L(4)) then
            colour1L(2) = .true.
            colour1L(3) = .true.
            mask(2,3) = 1
            mask(3,2) = 1
         endif
         if(colour2L(5)) then
            colour1L(1) = .true.
            colour1L(3) = .true.
            mask(1,3) = 1
            mask(3,1) = 1
         endif
         if(colour2L(6)) then
            colour1L(3) = .true.
            mask(3,3) = 1
         endif
      endif
      if(order>=0) then
         call initQ0LC(li,i1,i2,i3)!initialise 0L gamma coeffs.
         coeff0L(1) = helCoeffQ(1,0,1,i1,i2,i3,1)
         coeff0L(2) = helCoeffQ(1,0,0,i1,i2,i3,1)
         coeff0L(3) = helCoeffQ(0,1,1,i1,i2,i3,1)
         coeff0L(4) = helCoeffQ(0,1,0,i1,i2,i3,1)
      endif
      if(order>=1) then
         if(order>1) call setCataniSub(.true.)
         do l = 0,2
            check = .false.
            do j = 0,2
               if(maskeps(l,j)==1) check = .true.
            enddo
            if(check.or.(veceps(l)==1)) then
               call setIEorder(l)
               call initQ1LC(li,i1,i2,i3) !initialise 1L gamma coeffs.
               do i = 1,3
                  if(colour1L(i)) then
                     coeff1L(1,i,l) = helCoeffQ(1,0,1,i1,i2,i3,i+1)
                     coeff1L(2,i,l) = helCoeffQ(1,0,0,i1,i2,i3,i+1)
                     coeff1L(3,i,l) = helCoeffQ(0,1,1,i1,i2,i3,i+1)
                     coeff1L(4,i,l) = helCoeffQ(0,1,0,i1,i2,i3,i+1)
                     if(present(ieorder))then
                      if(
     .                    ((order==2).and.(ieorder>2)).or.
     .                    ((order==1).and.(ieorder>0)))then
                        donothing = .true.
                      endif
                     endif
                     if(.not.(donothing))then
                        do k = 1,4
                           if(order>1) coeff1LS(k,i) = coeff1L(k,i,0)
                           coeff1L(k,i,0) =  coeff1L(k,i,0)
     .                                   + cvec1L(i)*coeff0L(k)
                        enddo
                     endif
                     donothing = .false.
                  endif
               enddo
            endif
         enddo
      endif
      if(order==2) then
         call setIEorder(ieorder)
         call initQ2LC(li,i1,i2,i3)!initialise 2L gamma coeffs.
         do i = 1,6
            if(colour2L(i)) then
               coeff2L(1,i) = helCoeffQ(1,0,1,i1,i2,i3,i+4)
               coeff2L(2,i) = helCoeffQ(1,0,0,i1,i2,i3,i+4)
               coeff2L(3,i) = helCoeffQ(0,1,1,i1,i2,i3,i+4)
               coeff2L(4,i) = helCoeffQ(0,1,0,i1,i2,i3,i+4)
               if(present(ieorder))then
                if(ieorder>2)then
                  donothing = .true.
                endif
               endif
               if(.not.(donothing))then
                  do k = 1,4
                     do j = 1,3
                        coeff2L(k,i) =   coeff2L(k,i)
     .                       + cmat(i,j)*coeff1LS(k,j)
                     enddo
                     coeff2L(k,i) = coeff2L(k,i) + cvec(i)*coeff0L(k)
                  enddo
               endif
               donothing = .false.
            endif
         enddo
      endif
      !------------------------------------------
      if(order==2) then
         do i = 1,4   !sum over helicities
            do j = 1,6!sum over colour structures
               fin2L = fin2L
     . + 2.d0*real(coeff2L(i,j)*conjg(coeff0L(i)))
     . *helSq(i)!*qcdNorm(j+4)*qcdNorm(1)
            enddo
         enddo
         do i = 1,4      !sum over helicities
            do j = 1,3   !sum over colour structures
               do k = 1,3!sum over colour structures
                  do l = 0,2   !sum over poles
                     do m = 0,2!sum over poles
                        fin2L = fin2L
     .                       + real(coeff1L(i,j,l)*conjg(coeff1L(i,k,m))
     .                       *helSq(i)*mask(j,k)*maskeps(l,m))!*qcdNorm(j+1)*qcdNorm(k+1))
                     enddo
                  enddo
               enddo
            enddo
         enddo
      endif
      if(order>=1) then
         do i = 1,4   !sum over helicities
            do j = 1,3!sum over colour structures
               do k = 0,2!sum over poles
                  fin1L = fin1L
     .           + 2.d0*real(coeff1L(i,j,k)*conjg(coeff0L(i))*veceps(k))
     .                 *helSq(i) !*qcdNorm(j+1)*qcdNorm(1)
               enddo
            enddo
         enddo
      endif
      if(order>=0) then
         fin0L = sum(helSq*real(coeff0L*conjg(coeff0L)))!*qcdNorm(1)**2
      endif
      !------------------------------------------
      if(order==2) then
         fullB1g2H = fin2L ! + logs*fin1L + logs**2*fin0L
      elseif(order==1) then
         fullB1g2H = fin1L ! + logs*fin0L
      elseif(order==0) then
         fullB1g2H = fin0L
      endif

      end function fullB1g2H

!###################################################

      real*8 function helSqG(s,h1,h2,h3,i1,i2,i3)
      implicit none
      !function which returns helicity amplitudes squared
      !h=1 for positive hel., h=0 for negative
      !---------------------------------------
      real*8, dimension(3,3), intent(in) :: s
      integer, intent(in) :: h1,h2,h3
      integer, intent(in) :: i1,i2,i3
      !internal
      integer :: disc


      disc = h1*4+h2*2+h3

      helicity : select case (disc)

      case(7)!ppp
         helSqG = SqGpp(s,i1,i2,i3)

      case(6)!ppm
         helSqG = SqGmp(s,i3,i1,i2)

      case(5)!pmp
         helSqG = SqGmp(s,i2,i3,i1)

      case(4)!pmm
         helSqG = SqGmp(s,i1,i2,i3)

      !------------------------------------

      case(3)!mpp
         helSqG = SqGmp(s,i1,i2,i3)

      case(2)!mpm
         helSqG = SqGmp(s,i2,i3,i1)

      case(1)!mmp
         helSqG = SqGmp(s,i3,i1,i2)

      case(0)!mmm
         helSqG = SqGpp(s,i1,i2,i3)

      case default
         helSqG = 0.d0

      end select helicity

      end function helSqG

!###################################################

      complex*16 function helCoeffG(h1,h2,h3,i1,i2,i3,col)
      implicit none
      !function which returns helicity coefficients
      !h=1 for positive hel., h=0 for negative
      !---------------------------------------
      integer, intent(in) :: h1,h2,h3
      integer, intent(in) :: i1,i2,i3
      integer, intent(in) :: col
      !internal
      integer :: disc


      disc = h1*4+h2*2+h3

      helicity : select case (disc)

      case(7)!ppp
         helCoeffG = CGpp(i1,i2,i3,col)

      case(6)!ppm
         helCoeffG = CGmp(i3,i1,i2,col)

      case(5)!pmp
         helCoeffG = CGmp(i2,i3,i1,col)

      case(4)!pmm
         helCoeffG = -conjg(CGmp(i1,i2,i3,col)) !change parity

      !------------------------------------

      case(3)!mpp
         helCoeffG = CGmp(i1,i2,i3,col)

      case(2)!mpm
         helCoeffG = -conjg(CGmp(i2,i3,i1,col)) !change parity

      case(1)!mmp
         helCoeffG = -conjg(CGmp(i3,i1,i2,col)) !change parity

      case(0)!mmm
         helCoeffG = -conjg(CGpp(i1,i2,i3,col)) !change parity

      case default
         helCoeffG = (0.d0,0.d0)

      end select helicity

      end function helCoeffG

!###################################################

      real*8 function helSqQ(s,h1,h2,h3,i1,i2,i3)
      implicit none
      !function which returns g2q helicity amplitudes squared
      !h=1 for positive hel., h=0 for negative
      !---------------------------------------
      real*8, dimension(3,3), intent(in) :: s
      integer, intent(in) :: h1,h2,h3
      integer, intent(in) :: i1,i2,i3
      !internal
      integer disc


      disc = h1*4+h2*2+h3

      helicity : select case (disc)

      case(5)!pmp
         helSqQ = SqQpm(s,i2,i1,i3)

      case(4)!pmm
         helSqQ = SqQpm(s,i1,i2,i3)

      case(3)!mpp
         helSqQ = SqQpm(s,i1,i2,i3)

      case(2)!mpm
         helSqQ = SqQpm(s,i2,i1,i3)

      case default
         helSqQ = 0.d0

      end select helicity

      !Check if q (qb) is in the IS, while q (q) is in the FS
      if(s(i1,i2).gt.0d0) then
         helSqQ = helSqQ
      else
         helSqQ = -helSqQ
      endif

c      !> MM: old implementation
c       if(i3==3) then
c          helSqQ = helSqQ
c       else
c          helSqQ = -helSqQ!odd number of fermions
c       endif              ! continued to initial state


      end function helSqQ

!###################################################

      complex*16 function helCoeffQ(h1,h2,h3,i1,i2,i3,col)
      use h3pC_Coeff,only:gammaCC
      implicit none
      !function which returns g2q helicity coefficients
      !h=1 for positive hel., h=0 for negative
      !---------------------------------------
      integer, intent(in) :: h1,h2,h3
      integer, intent(in) :: i1,i2,i3
      integer, intent(in) :: col
      !internal
      integer disc


      disc = h1*4+h2*2+h3

      helicity : select case (disc)

      case(5)!pmp
         helCoeffQ = gammaCC(1,col)

      case(4)!pmm
         helCoeffQ = -conjg(gammaCC(2,col)) !change parity

      case(3)!mpp
         helCoeffQ = gammaCC(2,col)

      case(2)!mpm
         helCoeffQ = -conjg(gammaCC(1,col)) !change parity

      case default
         helCoeffQ = (0.d0,0.d0)

      end select helicity

      end function helCoeffQ

!###################################################

      real*8 function SqGpp(s,i1,i2,i3)
      implicit none
      !All plus h3g helicity amp. squared
      !----------------------------------
      real*8, dimension(3,3), intent(in) :: s
      integer, intent(in) :: i1,i2,i3
      !internal
      real*8 :: mHsq


      mHsq = s(i1,i2) + s(i1,i3) + s(i2,i3)
      SqGpp = mHsq**4/s(i1,i2)/s(i2,i3)/s(i3,i1)

      end function SqGpp

!###################################################

      real*8 function SqGmp(s,i1,i2,i3)
      implicit none
      !-++ h3g helicity amp. squared
      !-----------------------------
      real*8, dimension(3,3), intent(in) :: s
      integer, intent(in) :: i1,i2,i3


      SqGmp = s(i2,i3)**3/s(i1,i2)/s(i1,i3)

      end function SqGmp

!###################################################

      complex*16 function CGpp(i1,i2,i3,col)
      use h3pC_Coeff, only:alphaCC
      implicit none
      !All plus h3g helicity coefficient
      !---------------------------------
      integer, intent(in) :: i1,i2,i3
      integer, intent(in) :: col


      CGpp = alphaCC(i1,i2,col)

      end function CGpp

!###################################################

      complex*16 function CGmp(i1,i2,i3,col)
      use h3pC_Coeff, only:betaCC
      implicit none
      !-++ h3g helicity coefficient
      !----------------------------
      integer, intent(in) :: i1,i2,i3
      integer, intent(in) :: col


      CGmp = betaCC(i2,i3,col)

      end function CGmp

!###################################################

      real*8 function SqQpm(s,i1,i2,i3)
      implicit none
      !returns the helicity amp squared
      ! for qqBargH
      !------------
      real*8, dimension(3,3), intent(in) :: s
      integer, intent(in) :: i1,i2,i3


      SqQpm = s(i2,i3)**2/s(i1,i2)

      end function SqQpm

      end module libH3p2loop
