************************************************************************
!XC The 1-loop ME functions are based on MCFM-8.0 and MCFM-8.2 source code 

!Corresponding references are:
! For MCFM:
!  J.~M.~Campbell and R.~K.~Ellis,
!  %``An Update on vector boson pair production at hadron colliders,''
!  Phys.\ Rev.\ D {\bf 60} (1999) 113006
!  doi:10.1103/PhysRevD.60.113006
!  [hep-ph/9905386].

!  J.~M.~Campbell, R.~K.~Ellis and W.~T.~Giele,
!  %``A Multi-Threaded Version of MCFM,''
!  Eur.\ Phys.\ J.\ C {\bf 75} (2015) no.6,  246
!  doi:10.1140/epjc/s10052-015-3461-2
!  [arXiv:1503.06182 [physics.comp-ph]].

!  R.~Boughezal, J.~M.~Campbell, R.~K.~Ellis, C.~Focke, W.~Giele, X.~Liu, F.~Petriello and C.~Williams,
!  %``Color singlet production at NNLO in MCFM,''
!  Eur.\ Phys.\ J.\ C {\bf 77} (2017) no.1,  7
!  doi:10.1140/epjc/s10052-016-4558-y
!  [arXiv:1605.08011 [hep-ph]].

! For EHSV related functions:
!  R.~K.~Ellis, I.~Hinchliffe, M.~Soldate and J.~J.~van der Bij,
!  %``Higgs Decay to tau+ tau-: A Possible Signature of Intermediate Mass Higgs Bosons at the SSC,''
!  Nucl.\ Phys.\ B {\bf 297} (1988) 221.
!  doi:10.1016/0550-3213(88)90019-3
************************************************************************
*
* two gluon one higgs zero quark 1-loop level matrix elements with massive quark loop
**
* summed over colour orderings, summed and squared
*
************************************************************************
      function A2g1mH(i1,i2,i3,i4)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /Hmass/emh,ehwidth
      common /Qmass/emt,etwidth,emb,ebwidth,emc,ecwidth,emtau,etauwidth
      double precision , dimension(6) :: emq
      common /npars/npars,nspec,n
      real*8, pointer :: s12
      integer k,noq

      ipset = nspec+2
      s12 => kin(ipset)%s(i1,i2)

      emq(1)=emt
      emq(2)=emb
      emq(3)=emc
      noq=1

c      hmass=emh
      hmass=sqrt(s12)
      pisq=pi*pi
      zamphprod=0d0

      do k=1,noq
      tn = 4d0*(emq(k)/hmass)**2
      zamphprod=zamphprod+(3d0*tn/4d0)*(2d0+(tn-1d0)*zw2(hmass**2,emq(k))/2d0)
      enddo

      hprod=zamphprod*conjg(zamphprod)
      A2g1mH=2d0*s12**2*hprod

      return
      end

************************************************************************
*
* three gluon one higgs zero quark 1-loop level matrix elements with massive quark loop
**
* summed over colour orderings, summed and squared
*
************************************************************************
      double precision function ehsvm3(s,t,u,s45)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
c---Matrix element squared Eqn 2.2 of EHSV
      integer k,noq
      common /Hmass/emh,ehwidth
      common /Qmass/emt,etwidth,emb,ebwidth,emc,ecwidth,emtau,etauwidth
      double precision , dimension(6) :: emq

      emq(1)=emt
      emq(2)=emb
      emq(3)=emc
      noq=1

         zamppp1=0d0
         zamppp2=0d0
         zamppp3=0d0
         zamppm1=0d0

      do k=1,noq
         zamppp1=zamppp1+zehsva2(s,t,u,emq(k))
         zamppp2=zamppp2+zehsva2(u,s,t,emq(k))
         zamppp3=zamppp3+zehsva2(t,u,s,emq(k))
         zamppm1=zamppm1+zehsva4(s,t,u,emq(k))
      enddo

      ehsvm3=
     . zamppp1*conjg(zamppp1)+zamppp2*conjg(zamppp2)+zamppp3*conjg(zamppp3)
     . +zamppm1*conjg(zamppm1)

      ehsvm3=s45**4/s/t/u*ehsvm3

      return
      end
************************************************************************
      double complex function zehsva2(s,t,u,emq)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      zehsva2=zehsvb2(s,t,u,emq)+zehsvb2(s,u,t,emq)
      return
      end
************************************************************************
      double complex function zehsva4(s,t,u,emq)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      zehsva4=zehsvb4(s,t,u,emq)+zehsvb4(u,s,t,emq)+zehsvb4(t,u,s,emq)
      return
      end
************************************************************************
      double complex function zehsvb2(s,t,u,emq)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /Hmass/emh,ehwidth

      hmass2=s+t+u
      emq2=emq**2

      zehsvb2=emq2/hmass2**2*(s*(u-s)/(s+u)
     . +2d0*u*t*(u+2d0*s)/(s+u)**2*(zw1(t,emq)-zw1(hmass2,emq))
     . +(emq2-0.25d0*s)
     . *(0.5d0*zw2(s,emq)+0.5d0*zw2(hmass2,emq)-zw2(t,emq)+zw3(s,t,u,hmass2,emq))
     . +s**2*(2d0*emq2/(s+u)**2-0.5d0/(s+u))*(zw2(t,emq)-zw2(hmass2,emq))
     . +0.5d0*u*t/s*(zw2(hmass2,emq)-2d0*zw2(t,emq))
     . +0.125d0*(s-12d0*emq2-4d0*u*t/s)*zw3(t,s,u,hmass2,emq))
      return
      end
************************************************************************
      double complex function zehsvb4(s,t,u,emq)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /Hmass/emh,ehwidth

      emq2=emq**2
      hmass2=s+t+u

      zehsvb4=emq2/hmass2*(-2d0/3d0
     . +(emq2/hmass2-0.25d0)*(zw2(t,emq)-zw2(hmass2,emq)+zw3(s,t,u,hmass2,emq)))
      return
      end


************************************************************************
************************************************************************
*
* three gluon one higgs zero quark 1-loop level matrix elements with massive quark loop
**
* summed over colour orderings, summed and squared
*
************************************************************************
************************************************************************
*XC For massive Top quark only, Hm Real contribution is stable for y0=1d-6 to 1d-12
************************************************************************
*XC For massive Top and Bottom quark (and their interference terms),
*XC this ME is stable for HpTm LO at small pT (0.1 GeV still consistent for y0=1d-7 down to 1d-10), 
*XC stable for Hm Real contribution (consistent for y0=1d-6 down to 1d-9, unstable from 1d-10).
************************************************************************
*XC When including massive Charm quark, only HJm LO results are stable.

      function A3g1mH(i1,i2,i3,i4,i5)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer NHEL(3,4),PERM(5)
      integer PERMb(5),PERMc(5)
      integer i,k,noq
      double precision , dimension(6) :: emq
      common /Qmass/emt,etwidth,emb,ebwidth,emc,ecwidth,emtau,etauwidth
      COMMON/BERNOULLI/B2(18),B12(18),B3(18),Once
!$omp threadprivate(/BERNOULLI/)
      COMMON/ZETACONST/ZETA2,ZETA3
      COMMON/POLY/NBER

      DATA (NHEL(IHEL,   1),IHEL=1, 3) /  1, 1, 1/
      DATA (NHEL(IHEL,   2),IHEL=1, 3) /  1, 1,-1/
      DATA (NHEL(IHEL,   3),IHEL=1, 3) /  1,-1, 1/
      DATA (NHEL(IHEL,   4),IHEL=1, 3) /  1,-1,-1/

c      DATA (NHEL(IHEL,   2),IHEL=1, 3) / -1,-1,-1/
c      DATA (NHEL(IHEL,   4),IHEL=1, 3) / -1,-1, 1/
c      DATA (NHEL(IHEL,   6),IHEL=1, 3) / -1, 1,-1/
c      DATA (NHEL(IHEL,   8),IHEL=1, 3) / -1, 1, 1/
*XC above four helicities are not independent after moduler square

      emq(1)=emt
      emq(2)=emb
      emq(3)=emc
      noq=1

        perm(1)=i1
        perm(2)=i2
        perm(3)=i3
        perm(4)=i4
        perm(5)=i5

        permb(1)=i1
        permb(2)=i3
        permb(3)=i2
        permb(4)=i4
        permb(5)=i5

        permc(1)=i2
        permc(2)=i1
        permc(3)=i3
        permc(4)=i4
        permc(5)=i5

          if (Once .ne. 1d0) then
            call sushi_bernini(18)
          endif

        amp2=0d0

       do i=1,4
          zamp = (0d0,0d0)
          do k=1,noq
            if(i.eq.1) then
             zamp=zamp+zA3g1mHppp(perm(1),emq(k))
            endif
            if(i.eq.2) then
             zamp=zamp+zA3g1mHppm(permb(1),emq(k))
            endif
            if(i.eq.3) then
             zamp=zamp+zA3g1mHppm(perm(1),emq(k))
            endif
            if(i.eq.4) then
             zamp=zamp+zA3g1mHmmp(permc(1),emq(k))
            endif
          enddo
          amp2=amp2+36d0*zamp*dconjg(zamp)
       !     write(*,*)"amp2=",amp2
       enddo

       A3g1mH=amp2

      return
      end

************************************************************************
      complex*16 function zA3g1mHppp(PERM,emq)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer perm(5),ipset
      common /npars/npars,nspec,n
      complex*16, pointer :: za12,za13,za23
      complex*16, pointer :: za21,za31,za32
      complex*16, pointer :: zb12,zb13,zb23
      complex*16, pointer :: zb21,zb31,zb32

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)

      ipset = nspec+3

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za23 => kin(ipset)%zA(j2,j3)

      zA3g1mHppp=2d0*sqrt(2d0) * zc1(j1,j2,j3,emq) / za12 / za13 / za23

*      ztemp=zA3g1mHppp
*      write(*,*)"zA3g1mHppp",zA3g1mHppp

      return
      end

************************************************************************
      complex*16 function zA3g1mHppm(PERM,emq)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer hel(4),perm(5),ipset
      common /npars/npars,nspec,n
      complex*16, pointer :: za12,za13,za23
      complex*16, pointer :: za21,za31,za32
      complex*16, pointer :: zb12,zb13,zb23
      complex*16, pointer :: zb21,zb31,zb32

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)

      ipset = nspec+3

      za12 => kin(ipset)%zA(j1,j3)
      zb21 => kin(ipset)%zB(j3,j1)
      zb31 => kin(ipset)%zB(j2,j1)
      zb32 => kin(ipset)%zB(j2,j3)


      zA3g1mHppm=2d0*sqrt(2d0) * zc2(j1,j3,j2,emq) * zb21 / zb31 / zb32 / za12**2

*      ztemp=zA3g1mHppm
*      write(*,*)"zA3g1mHppm",zA3g1mHppm

      return
      end

************************************************************************
      complex*16 function zA3g1mHmmp(PERM,emq)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer hel(4),perm(5),ipset
      common /npars/npars,nspec,n
      complex*16, pointer :: za12,za13,za23
      complex*16, pointer :: za21,za31,za32
      complex*16, pointer :: zb12,zb13,zb23
      complex*16, pointer :: zb21,zb31,zb32

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)

      ipset = nspec+3

      za12 => kin(ipset)%zB(j1,j3)
      zb21 => kin(ipset)%zA(j3,j1)
      zb31 => kin(ipset)%zA(j2,j1)
      zb32 => kin(ipset)%zA(j2,j3)

      zA3g1mHmmp=2d0*sqrt(2d0) * zc2(j1,j3,j2,emq) * zb21 / zb31 / zb32 / za12**2

*      ztemp=zA3g1mHmmp
*      write(*,*)"zA3g1mHmmp",zA3g1mHmmp

      return
      end

************************************************************************
      complex*16 function zc1(j1,j2,j3,emq)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer ipset
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s23,s21,s31,s32

      ipset = nspec+3

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s23 => kin(ipset)%s(j2,j3)

      zc1=zc1smh(s12,s13,s23,emq**2)*emq**2/16d0

!      write(*,*)"zc1=",zc1/emq**2

      return
      end

************************************************************************
      function zc1smh(s,t,u,m2)
      implicit none
      double complex zc1smh
      double precision s,t,u,m2
      double complex Integral3,Integral4

      zc1smh=(8.d0*(s + t + u) - 
     &  (-4.d0*m2 + s + t + u)*
     &   (2.d0*((t + u)*Integral3(s,s + t + u,m2)
     &        + (s + u)*Integral3(t,s + t + u,m2)
     &        + (s + t)*Integral3(u,s + t + u,m2))
     &   + s*u*Integral4(s,t,u,m2)
     &   + s*t*Integral4(s,u,t,m2)
     &   + t*u*Integral4(t,s,u,m2)
     &       )
     &       )
      end

************************************************************************
      complex*16 function zc2(j1,j2,j3,emq)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer hel(4),perm(5),ipset
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s23,s21,s31,s32

      ipset = nspec+3

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s23 => kin(ipset)%s(j2,j3)

      zc2=zc2smh(s12,s13,s23,emq**2)*emq**2/16d0
 
      return
      end

************************************************************************
      function zc2smh(s,t,u,m2)
      implicit none
      double complex zc2smh
      double precision s,t,u,m2
      double complex Integral2,Integral3,Integral4
      zc2smh=(8.d0*(-((s*t)/(s + t)) + s**2/(s + u) + 
     &     t*u*(((2.d0*s + u)*Integral2(t,m2))/(s + u)**2 + 
     &     ((2.d0*s + t)*Integral2(u,m2))/(s + t)**2 - 
     &     ((2.d0*s + t)/(s + t)**2 + 
     &        (2.d0*s + u)/(s + u)**2)*
     &      Integral2(s + t + u,m2) + 
     &  ((t*Integral3(0.d0,t,m2) + 
     &       u*Integral3(0.d0,u,m2)))/(2.d0*s))) + 
     &  2.d0*((4.d0*m2 - s)*(t + u)*Integral3(s,s + t + u,m2) + 
     &  (-s**2 - 2.d0*(2.d0*m2 + t)*u - (2.d0*t*u**2)/s + 
     &     s*(4.d0*m2 + u - (8.d0*m2*u)/(s + u)))*
     &   Integral3(t,s + t + u,m2) + 
     &  ((-2.d0*t**2*u)/s - (-4.d0*m2+ s)*s + 
     &     t*(-4.d0*m2 - 2.d0*u + s - (8.d0*m2*s)/(s + t)))*
     &   Integral3(u,s + t + u,m2)) +  
     &  (4.d0*m2 - s)*s*u*Integral4(s,t,u,m2) + 
     &  (4.d0*m2 - s)*s*t*Integral4(s,u,t,m2) + 
     & t*u*(-12.d0*m2 + s - (4.d0*t*u)/s)*Integral4(t,s,u,m2))
      end


************************************************************************
*
* four gluon one higgs zero quark 1-loop level matrix elements with massive quark loop
**
* summed over colour orderings, summed and squared
* Need inputs from colour decomposed OL
************************************************************************
      function A4g1mH(i1,i2,i3,i4,i5,i6)
#ifdef USE_OL  
      use openloops
      use KinData_mod
      use Channels_mod, only: OLiproc_chan
      use Process_mod
#endif 
!Need the dynamic value of alphas_scl from driver for MC run
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      real(8) :: MW,MZ
      integer :: OL_id_A4g1mHOL 
      real(8) :: m2arr(6)
!Here we know the colour basis dimension is 6 for A4g1mH from loop_colbasis_dim of OpenLoops
      parameter(pi=3.141592653589793238d0)
      parameter(nc=3d0)
      parameter(nf=5d0)
      integer :: ncolb,colelemsz,nhel

c      write(*,*)"Hello A4g1mH"
      A4g1mH = 0d0
      m2arr(:) = 0d0
#ifdef USE_OL 
      select case ( name_proc ) 
      case('HJM')
           OL_id_A4g1mHOL = OLiproc_chan(8)
!     OL_id_A4g1mHOL=register_process("g g -> g g H  ",12) same as OLiproc_chan(8) inside registerOLHJm()
      case('HJJM','HJJMREW')
c           write(*,*)"name_proc=",name_proc
c           write(*,*)"OLiproc_chan(1)=",OLiproc_chan(1)
           OL_id_A4g1mHOL = OLiproc_chan(1)
!     OL_id_A4g1mHOL=register_process("g g -> g g H  ",12) same as OLiproc_chan(1) inside registerOLHJJm()
      case default
         write(*,*) "Invalid process in crossProc: ", name_proc
         stop
      end select

!$omp critical(eval_ol)
      call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
      !write(*,*)"kin(6)%pOLHinc=",kin(6)%pOLHinc
!Above parameters matches the OpenLoops pre-factor and we want to remove them for NNLOJET convention
!MM: this function (at the moment) requires the public_beta version of OLs 
       call evaluate_loop_colvect2(OL_id_A4g1mHOL,kin(6)%pOLHinc,m2arr)
c      write(*,*)"m2arr after evaluate",m2arr
      if(i1.eq.1 .and. i2.eq.2 .and. i3.eq.3 .and. i4.eq.4) then
          temp = m2arr(1)
      elseif(i1.eq.1 .and. i2.eq.4 .and. i3.eq.3 .and. i4.eq.2) then
          temp = m2arr(1)
      elseif(i1.eq.1 .and. i2.eq.2 .and. i3.eq.4 .and. i4.eq.3) then
          temp = m2arr(2)
      elseif(i1.eq.1 .and. i2.eq.3 .and. i3.eq.4 .and. i4.eq.2) then
          temp = m2arr(2)
      elseif(i1.eq.1 .and. i2.eq.3 .and. i3.eq.2 .and. i4.eq.4) then
          temp = m2arr(3)
      elseif(i1.eq.1 .and. i2.eq.4 .and. i3.eq.2 .and. i4.eq.3) then
          temp = m2arr(3)
      elseif(i1.eq.2 .and. i2.eq.3 .and. i3.eq.4 .and. i4.eq.1) then
          temp = m2arr(1)
      elseif(i1.eq.2 .and. i2.eq.1 .and. i3.eq.4 .and. i4.eq.3) then
          temp = m2arr(1)
      elseif(i1.eq.2 .and. i2.eq.4 .and. i3.eq.3 .and. i4.eq.1) then
          temp = m2arr(2)
      elseif(i1.eq.2 .and. i2.eq.1 .and. i3.eq.3 .and. i4.eq.4) then
          temp = m2arr(2)
      elseif(i1.eq.2 .and. i2.eq.4 .and. i3.eq.1 .and. i4.eq.3) then
          temp = m2arr(3)
      elseif(i1.eq.2 .and. i2.eq.3 .and. i3.eq.1 .and. i4.eq.4) then
          temp = m2arr(3)
      else
          write(*,*)"OL permutation not found in A4g1mH"
          write(*,*)"i1,i2,i3,i4,i5,i6 =",i1,i2,i3,i4,i5,i6
          stop
      endif

!Calculate the vev used in OL:
      call get_parameter("alpha", alpha)
      call get_parameter("alphas", alpha_s)
      call get_parameter("w_mass", MW)
      call get_parameter("z_mass", MZ)
!$omp end critical(eval_ol)
      cw   = MW/MZ
      cw2  = cw**2d0  
      sw2  = 1d0 - cw2
      sw   = sqrt(sw2)
      vev  = 2d0*MW*sw/sqrt(4d0*pi*alpha)
      !write(*,*)"vev=",vev
      C0=(alpha_s/6d0/pi/vev)**2
      prefactor = (dble(nc)**2-1d0)*C0*(4d0*pi*alpha_s)**2*2d0
      !write(*,*)"prefactor=",prefactor
      A4g1mH = temp/prefactor
#endif

      return
      end

ccc This is reweighted 1-loop libarary. 
************************************************************************
      function A3g1mHrew(i1,i2,i3,i4,i5,renscale2)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer i1,i2,i3,i4,i5

      A3g1mHrew = A3g1H(i1,i2,i3,i4,i5,renscale2)*A3g1mH(i1,i2,i3,i4,i5)/A3g0H(i1,i2,i3,i4,i5)

      return
      end

************************************************************************
      function Ah3g1mHrew(i1,i2,i3,i4,i5,renscale2)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer i1,i2,i3,i4,i5

      Ah3g1mHrew = Ah3g1H(i1,i2,i3,i4,i5,renscale2)*A3g1mH(i1,i2,i3,i4,i5)/A3g0H(i1,i2,i3,i4,i5)

      return
      end

************************************************************************
      function A4g1mHrew(i1,i2,i3,i4,i5,i6,renscale2)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer i1,i2,i3,i4,i5,i6

      A4g1mHrew = A4g1H(i1,i2,i3,i4,i5,i6,renscale2)*A4g1mH(i1,i2,i3,i4,i5,i6)/A4g0H(i1,i2,i3,i4,i5,i6)

      return
      end

************************************************************************
      function Ah4g1mHrew(i1,i2,i3,i4,i5,i6,renscale2)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer i1,i2,i3,i4,i5,i6

      Ah4g1mHrew = Ah4g1H(i1,i2,i3,i4,i5,i6,renscale2)*A4g1mH(i1,i2,i3,i4,i5,i6)/A4g0H(i1,i2,i3,i4,i5,i6)

      return
      end

