************************************************************************
!XC The B1g1mH function is based on MCFM-8.0 source code

!Corresponding references are:
! For MCFM:
!  J.~M.~Campbell and R.~K.~Ellis,
!  %``An Update on vector boson pair production at hadron colliders,''
!  Phys.\ Rev.\ D {\bf 60} (1999) 113006
!  doi:10.1103/PhysRevD.60.113006
!  [hep-ph/9905386].

!  J.~M.~Campbell, R.~K.~Ellis and W.~T.~Giele,
!  %``A Multi-Threaded Version of MCFM,''
!  Eur.\ Phys.\ J.\ C {\bf 75} (2015) no.6,  246
!  doi:10.1140/epjc/s10052-015-3461-2
!  [arXiv:1503.06182 [physics.comp-ph]].

!  R.~Boughezal, J.~M.~Campbell, R.~K.~Ellis, C.~Focke, W.~Giele, X.~Liu, F.~Petriello and C.~Williams,
!  %``Color singlet production at NNLO in MCFM,''
!  Eur.\ Phys.\ J.\ C {\bf 77} (2017) no.1,  7
!  doi:10.1140/epjc/s10052-016-4558-y
!  [arXiv:1605.08011 [hep-ph]].

! For EHSV related functions:
!  R.~K.~Ellis, I.~Hinchliffe, M.~Soldate and J.~J.~van der Bij,
!  %``Higgs Decay to tau+ tau-: A Possible Signature of Intermediate Mass Higgs Bosons at the SSC,''
!  Nucl.\ Phys.\ B {\bf 297} (1988) 221.
!  doi:10.1016/0550-3213(88)90019-3
************************************************************************
*
* one gluon one qqbar pair one higgs 1-loop level matrix elements with massive quark loop
**
* summed over colour orderings, summed and squared
*
* tested to be stable for all processes with massive T,B,C quarks and interference effects
************************************************************************
      function B1g1mH(i1,i3,i2,i4,i5)
      use KinData_mod
      implicit double precision (a-h,o-z)
      common /Hmass/emh,ehwidth
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s23

      ipset = nspec+3
      s12 => kin(ipset)%s(i1,i2)
      s13 => kin(ipset)%s(i1,i3)
      s23 => kin(ipset)%s(i2,i3)

c      s45=emh**2
      s45=s12+s13+s23

      if(i3.eq.1 .or. i3 .eq.2) then
      B1g1mH= -9d0/2d0*ehsvm4(s12,s13,s23,s45)
      else
      B1g1mH=  9d0/2d0*ehsvm4(s12,s13,s23,s45)
      endif

      return
      end
************************************************************************
      double precision function ehsvm4(s,t,u,s45)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer k,noq
      double precision , dimension(6) :: emq
      common /Hmass/emh,ehwidth
      common /Qmass/emt,etwidth,emb,ebwidth,emc,ecwidth

      emq(1)=emt
      emq(2)=emb
      emq(3)=emc
      noq=1

         zamp=0d0
      do k=1,noq
         zamp=zamp+zehsva5(s,t,u,emq(k))
      enddo

      ehsvm4=zamp*conjg(zamp)
      ehsvm4=(u**2+t**2)/s*s45**2/(u+t)**2*ehsvm4

      return
      end

************************************************************************
      double complex function zehsva5(s,t,u,emq)
C     ehsv:EqnA.14
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      hmass2=s+t+u
      emq2=emq**2

      zehsva5=emq2/hmass2*(4d0+4d0*s/(u+t)*(zw1(s,emq)-zw1(hmass2,emq))
     . +(1d0-4d0*emq2/(u+t))*(zw2(s,emq)-zw2(hmass2,emq)))
      return
      end

************************************************************************
*
* colour leading one quark pair two gluon one higgs 1-loop level matrix elements with massive quark loop
**
* summed over colour orderings, summed and squared
* Need inputs from colour decomposed OL
************************************************************************
      function B2g1mH(iq1,i3,i4,iqbar2,i5,i6)
#ifdef USE_OL  
      use openloops
      use KinData_mod
      use Channels_mod, only: OLiproc_chan
      use Process_mod
#endif 
!Need the dynamic value of alphas_scl from driver for MC run
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      real(8) :: MW,MZ
      integer :: OL_id_B2g1mHOL 
      real(8) :: m2arr(3)
!Here we know the colour basis dimension is 3 for B2g1mH from loop_colbasis_dim of OpenLoops
      parameter(pi=3.141592653589793238d0)
      parameter(nc=3d0)
      parameter(nf=5d0)

      !write(*,*)"Hello B2g1mH"
      B2g1mH = 0d0
      m2arr(:) = 0d0
      temp = 0d0
#ifdef USE_OL 
!$omp critical(eval_ol)
      select case ( name_proc ) 
      case('HJM')
         if(iq1.eq.1 .and. i3.eq.2 .and. i4.eq.3 .and. iqbar2.eq.4 ) then
           OL_id_B2g1mHOL = OLiproc_chan(9)
!          OLiproc_chan(9) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect2(OL_id_B2g1mHOL,kin(6)%pOLHinc,m2arr)
           temp = m2arr(2)
         elseif(iq1.eq.1 .and. i3.eq.3 .and. i4.eq.2 .and. iqbar2.eq.4 ) then
           OL_id_B2g1mHOL = OLiproc_chan(9)
!          OLiproc_chan(9) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect2(OL_id_B2g1mHOL,kin(6)%pOLHinc,m2arr)
           temp = m2arr(1)
         elseif(iq1.eq.1 .and. i3.eq.2 .and. i4.eq.4 .and. iqbar2.eq.3 ) then
           OL_id_B2g1mHOL = OLiproc_chan(9)
!          OLiproc_chan(9) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 4, 3, 5, 6 /) )
            call evaluate_loop_colvect2(OL_id_B2g1mHOL,kin(6)%pOLHinc,m2arr)
           temp = m2arr(2)
         elseif(iq1.eq.1 .and. i3.eq.4 .and. i4.eq.2 .and. iqbar2.eq.3 ) then
           OL_id_B2g1mHOL = OLiproc_chan(9)
!          OLiproc_chan(9) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 4, 3, 5, 6 /) )
            call evaluate_loop_colvect2(OL_id_B2g1mHOL,kin(6)%pOLHinc,m2arr)
           temp = m2arr(1)
         elseif(iq1.eq.1 .and. i3.eq.3 .and. i4.eq.4 .and. iqbar2.eq.2 ) then
           OL_id_B2g1mHOL = OLiproc_chan(10)
!          OLiproc_chan(10) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect2(OL_id_B2g1mHOL,kin(6)%pOLHinc,m2arr)
           temp = m2arr(2)
         elseif(iq1.eq.1 .and. i3.eq.4 .and. i4.eq.3 .and. iqbar2.eq.2 ) then
           OL_id_B2g1mHOL = OLiproc_chan(10)
!          OLiproc_chan(10) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect2(OL_id_B2g1mHOL,kin(6)%pOLHinc,m2arr)
           temp = m2arr(1)
         elseif(iq1.eq.4 .and. i3.eq.3 .and. i4.eq.2 .and. iqbar2.eq.1 ) then
           OL_id_B2g1mHOL = OLiproc_chan(9)
!          OLiproc_chan(9) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect2(OL_id_B2g1mHOL,kin(6)%pOLHinc,m2arr)
           temp = m2arr(2)
         elseif(iq1.eq.4 .and. i3.eq.2 .and. i4.eq.3 .and. iqbar2.eq.1 ) then
           OL_id_B2g1mHOL = OLiproc_chan(9)
!          OLiproc_chan(9) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect2(OL_id_B2g1mHOL,kin(6)%pOLHinc,m2arr)
           temp = m2arr(1)
         elseif(iq1.eq.3 .and. i3.eq.4 .and. i4.eq.2 .and. iqbar2.eq.1 ) then
           OL_id_B2g1mHOL = OLiproc_chan(9)
!          OLiproc_chan(9) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 4, 3, 5, 6 /) )
            call evaluate_loop_colvect2(OL_id_B2g1mHOL,kin(6)%pOLHinc,m2arr)
           temp = m2arr(2)
         elseif(iq1.eq.3 .and. i3.eq.2 .and. i4.eq.4 .and. iqbar2.eq.1 ) then
           OL_id_B2g1mHOL = OLiproc_chan(9)
!          OLiproc_chan(9) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 4, 3, 5, 6 /) )
            call evaluate_loop_colvect2(OL_id_B2g1mHOL,kin(6)%pOLHinc,m2arr)
           temp = m2arr(1)
         elseif(iq1.eq.2 .and. i3.eq.4 .and. i4.eq.3 .and. iqbar2.eq.1 ) then
           OL_id_B2g1mHOL = OLiproc_chan(10)
!          OLiproc_chan(10) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect2(OL_id_B2g1mHOL,kin(6)%pOLHinc,m2arr)
           temp = m2arr(2)
         elseif(iq1.eq.2 .and. i3.eq.3 .and. i4.eq.4 .and. iqbar2.eq.1 ) then
           OL_id_B2g1mHOL = OLiproc_chan(10)
!          OLiproc_chan(10) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect2(OL_id_B2g1mHOL,kin(6)%pOLHinc,m2arr)
           temp = m2arr(1)
         elseif(iq1.eq.3 .and. i3.eq.1 .and. i4.eq.2 .and. iqbar2.eq.4 ) then
           OL_id_B2g1mHOL = OLiproc_chan(13)
!          OLiproc_chan(13) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect2(OL_id_B2g1mHOL,kin(6)%pOLHinc,m2arr)
           temp = m2arr(2)
         elseif(iq1.eq.3 .and. i3.eq.2 .and. i4.eq.1 .and. iqbar2.eq.4 ) then
           OL_id_B2g1mHOL = OLiproc_chan(13)
!          OLiproc_chan(13) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect2(OL_id_B2g1mHOL,kin(6)%pOLHinc,m2arr)
           temp = m2arr(1)
         elseif(iq1.eq.4 .and. i3.eq.1 .and. i4.eq.2 .and. iqbar2.eq.3 ) then
           OL_id_B2g1mHOL = OLiproc_chan(13)
!          OLiproc_chan(13) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 4, 3, 5, 6 /) )
            call evaluate_loop_colvect2(OL_id_B2g1mHOL,kin(6)%pOLHinc,m2arr)
           temp = m2arr(2)
         elseif(iq1.eq.4 .and. i3.eq.2 .and. i4.eq.1 .and. iqbar2.eq.3 ) then
           OL_id_B2g1mHOL = OLiproc_chan(13)
!          OLiproc_chan(13) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 4, 3, 5, 6 /) )
            call evaluate_loop_colvect2(OL_id_B2g1mHOL,kin(6)%pOLHinc,m2arr)
           temp = m2arr(1)
         elseif(iq1.eq.2 .and. i3.eq.3 .and. i4.eq.1 .and. iqbar2.eq.4 ) then
           OL_id_B2g1mHOL = OLiproc_chan(11)
!          OLiproc_chan(11) inside registerOLHJm()
!          OLiproc_chan(11)=register_process("g 2 -> g 2 H  ",12)
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect2(OL_id_B2g1mHOL,kin(6)%pOLHinc,m2arr)
           temp = m2arr(1)
         elseif(iq1.eq.2 .and. i3.eq.1 .and. i4.eq.3 .and. iqbar2.eq.4 ) then
           OL_id_B2g1mHOL = OLiproc_chan(11)
!          OLiproc_chan(11) inside registerOLHJm()
!          OLiproc_chan(11)=register_process("g 2 -> g 2 H  ",12)
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect2(OL_id_B2g1mHOL,kin(6)%pOLHinc,m2arr)
           temp = m2arr(2)
         elseif(iq1.eq.4 .and. i3.eq.1 .and. i4.eq.3 .and. iqbar2.eq.2 ) then
           OL_id_B2g1mHOL = OLiproc_chan(11)
!          OLiproc_chan(11) inside registerOLHJm()
!          OLiproc_chan(11)=register_process("g 2 -> g 2 H  ",12)
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect2(OL_id_B2g1mHOL,kin(6)%pOLHinc,m2arr)
           temp = m2arr(1)
         elseif(iq1.eq.4 .and. i3.eq.3 .and. i4.eq.1 .and. iqbar2.eq.2 ) then
           OL_id_B2g1mHOL = OLiproc_chan(11)
!          OLiproc_chan(11) inside registerOLHJm()
!          OLiproc_chan(11)=register_process("g 2 -> g 2 H  ",12)
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect2(OL_id_B2g1mHOL,kin(6)%pOLHinc,m2arr)
           temp = m2arr(2)
         elseif(iq1.eq.2 .and. i3.eq.4 .and. i4.eq.1 .and. iqbar2.eq.3 ) then
           OL_id_B2g1mHOL = OLiproc_chan(11)
!          OLiproc_chan(11) inside registerOLHJm()
!          OLiproc_chan(11)=register_process("g 2 -> g 2 H  ",12)
           call fill_pOLHinc(6, (/ 1, 2, 4, 3, 5, 6 /) )
            call evaluate_loop_colvect2(OL_id_B2g1mHOL,kin(6)%pOLHinc,m2arr)
           temp = m2arr(1)
         elseif(iq1.eq.2 .and. i3.eq.1 .and. i4.eq.4 .and. iqbar2.eq.3 ) then
           OL_id_B2g1mHOL = OLiproc_chan(11)
!          OLiproc_chan(11) inside registerOLHJm()
!          OLiproc_chan(11)=register_process("g 2 -> g 2 H  ",12)
           call fill_pOLHinc(6, (/ 1, 2, 4, 3, 5, 6 /) )
            call evaluate_loop_colvect2(OL_id_B2g1mHOL,kin(6)%pOLHinc,m2arr)
           temp = m2arr(2)
         elseif(iq1.eq.3 .and. i3.eq.1 .and. i4.eq.4 .and. iqbar2.eq.2 ) then
           OL_id_B2g1mHOL = OLiproc_chan(11)
!          OLiproc_chan(11) inside registerOLHJm()
!          OLiproc_chan(11)=register_process("g 2 -> g 2 H  ",12)
           call fill_pOLHinc(6, (/ 1, 2, 4, 3, 5, 6 /) )
            call evaluate_loop_colvect2(OL_id_B2g1mHOL,kin(6)%pOLHinc,m2arr)
           temp = m2arr(1)
         elseif(iq1.eq.3 .and. i3.eq.4 .and. i4.eq.1 .and. iqbar2.eq.2 ) then
           OL_id_B2g1mHOL = OLiproc_chan(11)
!          OLiproc_chan(11) inside registerOLHJm()
!          OLiproc_chan(11)=register_process("g 2 -> g 2 H  ",12)
           call fill_pOLHinc(6, (/ 1, 2, 4, 3, 5, 6 /) )
            call evaluate_loop_colvect2(OL_id_B2g1mHOL,kin(6)%pOLHinc,m2arr)
           temp = m2arr(2)
         else
           write(*,*)"OL_id_B2g1mHOL crossing in HJM not found"
           write(*,*)"iq1,i3,i4,iqbar2,i5,i6 =",iq1,i3,i4,iqbar2,i5,i6
           stop
         endif
      case('HJJM','HJJMREW')
         if(iq1.eq.1 .and. i3.eq.2 .and. i4.eq.3 .and. iqbar2.eq.4 ) then
           OL_id_B2g1mHOL = OLiproc_chan(2)
!          OLiproc_chan(2) inside registerOLHJJm()
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect2(OL_id_B2g1mHOL,kin(6)%pOLHinc,m2arr)
           temp = m2arr(2)
         elseif(iq1.eq.1 .and. i3.eq.3 .and. i4.eq.2 .and. iqbar2.eq.4 ) then
           OL_id_B2g1mHOL = OLiproc_chan(2)
!          OLiproc_chan(2) inside registerOLHJJm()
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect2(OL_id_B2g1mHOL,kin(6)%pOLHinc,m2arr)
           temp = m2arr(1)
         elseif(iq1.eq.1 .and. i3.eq.2 .and. i4.eq.4 .and. iqbar2.eq.3 ) then
           OL_id_B2g1mHOL = OLiproc_chan(2)
!          OLiproc_chan(2) inside registerOLHJJm()
           call fill_pOLHinc(6, (/ 1, 2, 4, 3, 5, 6 /) )
            call evaluate_loop_colvect2(OL_id_B2g1mHOL,kin(6)%pOLHinc,m2arr)
           temp = m2arr(2)
         elseif(iq1.eq.1 .and. i3.eq.4 .and. i4.eq.2 .and. iqbar2.eq.3 ) then
           OL_id_B2g1mHOL = OLiproc_chan(2)
!          OLiproc_chan(2) inside registerOLHJJm()
           call fill_pOLHinc(6, (/ 1, 2, 4, 3, 5, 6 /) )
            call evaluate_loop_colvect2(OL_id_B2g1mHOL,kin(6)%pOLHinc,m2arr)
           temp = m2arr(1)
         elseif(iq1.eq.1 .and. i3.eq.3 .and. i4.eq.4 .and. iqbar2.eq.2 ) then
           OL_id_B2g1mHOL = OLiproc_chan(3)
!          OLiproc_chan(3) inside registerOLHJJm()
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect2(OL_id_B2g1mHOL,kin(6)%pOLHinc,m2arr)
           temp = m2arr(2)
         elseif(iq1.eq.1 .and. i3.eq.4 .and. i4.eq.3 .and. iqbar2.eq.2 ) then
           OL_id_B2g1mHOL = OLiproc_chan(3)
!          OLiproc_chan(3) inside registerOLHJJm()
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect2(OL_id_B2g1mHOL,kin(6)%pOLHinc,m2arr)
           temp = m2arr(1)
         elseif(iq1.eq.4 .and. i3.eq.3 .and. i4.eq.2 .and. iqbar2.eq.1 ) then
           OL_id_B2g1mHOL = OLiproc_chan(2)
!          OLiproc_chan(2) inside registerOLHJJm()
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect2(OL_id_B2g1mHOL,kin(6)%pOLHinc,m2arr)
           temp = m2arr(2)
         elseif(iq1.eq.4 .and. i3.eq.2 .and. i4.eq.3 .and. iqbar2.eq.1 ) then
           OL_id_B2g1mHOL = OLiproc_chan(2)
!          OLiproc_chan(2) inside registerOLHJJm()
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect2(OL_id_B2g1mHOL,kin(6)%pOLHinc,m2arr)
           temp = m2arr(1)
         elseif(iq1.eq.3 .and. i3.eq.4 .and. i4.eq.2 .and. iqbar2.eq.1 ) then
           OL_id_B2g1mHOL = OLiproc_chan(2)
!          OLiproc_chan(2) inside registerOLHJJm()
           call fill_pOLHinc(6, (/ 1, 2, 4, 3, 5, 6 /) )
            call evaluate_loop_colvect2(OL_id_B2g1mHOL,kin(6)%pOLHinc,m2arr)
           temp = m2arr(2)
         elseif(iq1.eq.3 .and. i3.eq.2 .and. i4.eq.4 .and. iqbar2.eq.1 ) then
           OL_id_B2g1mHOL = OLiproc_chan(2)
!          OLiproc_chan(2) inside registerOLHJJm()
           call fill_pOLHinc(6, (/ 1, 2, 4, 3, 5, 6 /) )
            call evaluate_loop_colvect2(OL_id_B2g1mHOL,kin(6)%pOLHinc,m2arr)
           temp = m2arr(1)
         elseif(iq1.eq.2 .and. i3.eq.4 .and. i4.eq.3 .and. iqbar2.eq.1 ) then
           OL_id_B2g1mHOL = OLiproc_chan(3)
!          OLiproc_chan(3) inside registerOLHJJm()
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect2(OL_id_B2g1mHOL,kin(6)%pOLHinc,m2arr)
           temp = m2arr(2)
         elseif(iq1.eq.2 .and. i3.eq.3 .and. i4.eq.4 .and. iqbar2.eq.1 ) then
           OL_id_B2g1mHOL = OLiproc_chan(3)
!          OLiproc_chan(3) inside registerOLHJJm()
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect2(OL_id_B2g1mHOL,kin(6)%pOLHinc,m2arr)
           temp = m2arr(1)
         elseif(iq1.eq.3 .and. i3.eq.1 .and. i4.eq.2 .and. iqbar2.eq.4 ) then
           OL_id_B2g1mHOL = OLiproc_chan(6)
!          OLiproc_chan(6) inside registerOLHJJm()
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect2(OL_id_B2g1mHOL,kin(6)%pOLHinc,m2arr)
           temp = m2arr(2)
         elseif(iq1.eq.3 .and. i3.eq.2 .and. i4.eq.1 .and. iqbar2.eq.4 ) then
           OL_id_B2g1mHOL = OLiproc_chan(6)
!          OLiproc_chan(6) inside registerOLHJJm()
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect2(OL_id_B2g1mHOL,kin(6)%pOLHinc,m2arr)
           temp = m2arr(1)
         elseif(iq1.eq.4 .and. i3.eq.1 .and. i4.eq.2 .and. iqbar2.eq.3 ) then
           OL_id_B2g1mHOL = OLiproc_chan(6)
!          OLiproc_chan(6) inside registerOLHJJm()
           call fill_pOLHinc(6, (/ 1, 2, 4, 3, 5, 6 /) )
            call evaluate_loop_colvect2(OL_id_B2g1mHOL,kin(6)%pOLHinc,m2arr)
           temp = m2arr(2)
         elseif(iq1.eq.4 .and. i3.eq.2 .and. i4.eq.1 .and. iqbar2.eq.3 ) then
           OL_id_B2g1mHOL = OLiproc_chan(6)
!          OLiproc_chan(6) inside registerOLHJJm()
           call fill_pOLHinc(6, (/ 1, 2, 4, 3, 5, 6 /) )
            call evaluate_loop_colvect2(OL_id_B2g1mHOL,kin(6)%pOLHinc,m2arr)
           temp = m2arr(1)
         elseif(iq1.eq.2 .and. i3.eq.3 .and. i4.eq.1 .and. iqbar2.eq.4 ) then
           OL_id_B2g1mHOL = OLiproc_chan(4)
!          OLiproc_chan(4) inside registerOLHJJm()
!          OLiproc_chan(4)=register_process("g 2 -> g 2 H  ",12)
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect2(OL_id_B2g1mHOL,kin(6)%pOLHinc,m2arr)
           temp = m2arr(1)
         elseif(iq1.eq.2 .and. i3.eq.1 .and. i4.eq.3 .and. iqbar2.eq.4 ) then
           OL_id_B2g1mHOL = OLiproc_chan(4)
!          OLiproc_chan(4) inside registerOLHJJm()
!          OLiproc_chan(4)=register_process("g 2 -> g 2 H  ",12)
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect2(OL_id_B2g1mHOL,kin(6)%pOLHinc,m2arr)
           temp = m2arr(2)
         elseif(iq1.eq.4 .and. i3.eq.1 .and. i4.eq.3 .and. iqbar2.eq.2 ) then
           OL_id_B2g1mHOL = OLiproc_chan(4)
!          OLiproc_chan(4) inside registerOLHJJm()
!          OLiproc_chan(4)=register_process("g 2 -> g 2 H  ",12)
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect2(OL_id_B2g1mHOL,kin(6)%pOLHinc,m2arr)
           temp = m2arr(1)
         elseif(iq1.eq.4 .and. i3.eq.3 .and. i4.eq.1 .and. iqbar2.eq.2 ) then
           OL_id_B2g1mHOL = OLiproc_chan(4)
!          OLiproc_chan(4) inside registerOLHJJm()
!          OLiproc_chan(4)=register_process("g 2 -> g 2 H  ",12)
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect2(OL_id_B2g1mHOL,kin(6)%pOLHinc,m2arr)
           temp = m2arr(2)
         elseif(iq1.eq.2 .and. i3.eq.4 .and. i4.eq.1 .and. iqbar2.eq.3 ) then
           OL_id_B2g1mHOL = OLiproc_chan(4)
!          OLiproc_chan(4) inside registerOLHJJm()
!          OLiproc_chan(4)=register_process("g 2 -> g 2 H  ",12)
           call fill_pOLHinc(6, (/ 1, 2, 4, 3, 5, 6 /) )
            call evaluate_loop_colvect2(OL_id_B2g1mHOL,kin(6)%pOLHinc,m2arr)
           temp = m2arr(1)
         elseif(iq1.eq.2 .and. i3.eq.1 .and. i4.eq.4 .and. iqbar2.eq.3 ) then
           OL_id_B2g1mHOL = OLiproc_chan(4)
!          OLiproc_chan(4) inside registerOLHJJm()
!          OLiproc_chan(4)=register_process("g 2 -> g 2 H  ",12)
           call fill_pOLHinc(6, (/ 1, 2, 4, 3, 5, 6 /) )
            call evaluate_loop_colvect2(OL_id_B2g1mHOL,kin(6)%pOLHinc,m2arr)
           temp = m2arr(2)
         elseif(iq1.eq.3 .and. i3.eq.1 .and. i4.eq.4 .and. iqbar2.eq.2 ) then
           OL_id_B2g1mHOL = OLiproc_chan(4)
!          OLiproc_chan(4) inside registerOLHJJm()
!          OLiproc_chan(4)=register_process("g 2 -> g 2 H  ",12)
           call fill_pOLHinc(6, (/ 1, 2, 4, 3, 5, 6 /) )
            call evaluate_loop_colvect2(OL_id_B2g1mHOL,kin(6)%pOLHinc,m2arr)
           temp = m2arr(1)
         elseif(iq1.eq.3 .and. i3.eq.4 .and. i4.eq.1 .and. iqbar2.eq.2 ) then
           OL_id_B2g1mHOL = OLiproc_chan(4)
!          OLiproc_chan(4) inside registerOLHJJm()
!          OLiproc_chan(4)=register_process("g 2 -> g 2 H  ",12)
           call fill_pOLHinc(6, (/ 1, 2, 4, 3, 5, 6 /) )
            call evaluate_loop_colvect2(OL_id_B2g1mHOL,kin(6)%pOLHinc,m2arr)
           temp = m2arr(2)
         else
           write(*,*)"OL_id_B2g1mHOL crossing in HJJM not found"
           write(*,*)"iq1,i3,i4,iqbar2,i5,i6 =",iq1,i3,i4,iqbar2,i5,i6
           stop
         endif
      case default
         write(*,*) "Invalid process in crossProc: ", name_proc
         stop
      end select
!Calculate the vev used in OL:
      call get_parameter("alpha", alpha)
      call get_parameter("alphas", alpha_s)
      call get_parameter("w_mass", MW)
      call get_parameter("z_mass", MZ)
!$omp end critical(eval_ol)
#endif 
      cw   = MW/MZ
      cw2  = cw**2d0  
      sw2  = 1d0 - cw2
      sw   = sqrt(sw2)
      vev  = 2d0*MW*sw/sqrt(4d0*pi*alpha)
      !write(*,*)"vev=",vev
      C0=(alpha_s/6d0/pi/vev)**2
      prefactor = (dble(nc)**2-1d0)*C0*(4d0*pi*alpha_s)**2/2d0
      !write(*,*)"prefactor=",prefactor
      !write(*,*)"kin(6)%pOLHinc=",kin(6)%pOLHinc
!Above parameters matches the OpenLoops pre-factor and we want to remove them for NNLOJET convention
      B2g1mH = temp/prefactor
      return
      end

************************************************************************
*
* colour sub-leading one quark pair two gluon one higgs 1-loop level matrix elements with massive quark loop
**
* summed over colour orderings, summed and squared
* Need inputs from colour decomposed OL
************************************************************************
      function Bt2g1mH(iq1,i3,i4,iqbar2,i5,i6)
#ifdef USE_OL  
      use openloops
      use KinData_mod
      use Channels_mod, only: OLiproc_chan
      use Process_mod
#endif 
!Need the dynamic value of alphas_scl from driver for MC run
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      real(8) :: MW,MZ
      integer :: OL_id_Bt2g1mHOL
      complex(8) :: zm2arr(3,16)
!Here we know the colour basis dimension is 6 for A4g1mH from loop_colbasis_dim of OpenLoops
      parameter(pi=3.141592653589793238d0)
      parameter(nc=3d0)
      parameter(nf=5d0)

      !write(*,*)"Hello B2g1mH"
      Bt2g1mH = 0d0
      zm2arr(:,:) = (0d0,0d0)
      temp = 0d0
#ifdef USE_OL 
!$omp critical(eval_ol)
      select case ( name_proc ) 
      case('HJM')
         if(iq1.eq.1 .and. i3.eq.2 .and. i4.eq.3 .and. iqbar2.eq.4 ) then
           OL_id_Bt2g1mHOL = OLiproc_chan(9)
!          OLiproc_chan(9) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect(OL_id_Bt2g1mHOL,kin(6)%pOLHinc,zm2arr,nhelnonv)
           do i = 1,16
              temp = temp + (zm2arr(1,i)+zm2arr(2,i))
     .              *dconjg((zm2arr(1,i)+zm2arr(2,i)))
           enddo
         elseif(iq1.eq.1 .and. i3.eq.3 .and. i4.eq.2 .and. iqbar2.eq.4 ) then
           OL_id_Bt2g1mHOL = OLiproc_chan(9)
!          OLiproc_chan(9) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect(OL_id_Bt2g1mHOL,kin(6)%pOLHinc,zm2arr,nhelnonv)
           do i = 1,16
              temp = temp + (zm2arr(1,i)+zm2arr(2,i))
     .              *dconjg((zm2arr(1,i)+zm2arr(2,i)))
           enddo
         elseif(iq1.eq.1 .and. i3.eq.2 .and. i4.eq.4 .and. iqbar2.eq.3 ) then
           OL_id_Bt2g1mHOL = OLiproc_chan(9)
!          OLiproc_chan(9) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 4, 3, 5, 6 /) )
            call evaluate_loop_colvect(OL_id_Bt2g1mHOL,kin(6)%pOLHinc,zm2arr,nhelnonv)
           do i = 1,16
              temp = temp + (zm2arr(1,i)+zm2arr(2,i))
     .              *dconjg((zm2arr(1,i)+zm2arr(2,i)))
           enddo
         elseif(iq1.eq.1 .and. i3.eq.4 .and. i4.eq.2 .and. iqbar2.eq.3 ) then
           OL_id_Bt2g1mHOL = OLiproc_chan(9)
!          OLiproc_chan(9) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 4, 3, 5, 6 /) )
            call evaluate_loop_colvect(OL_id_Bt2g1mHOL,kin(6)%pOLHinc,zm2arr,nhelnonv)
           do i = 1,16
              temp = temp + (zm2arr(1,i)+zm2arr(2,i))
     .              *dconjg((zm2arr(1,i)+zm2arr(2,i)))
           enddo
         elseif(iq1.eq.1 .and. i3.eq.3 .and. i4.eq.4 .and. iqbar2.eq.2 ) then
           OL_id_Bt2g1mHOL = OLiproc_chan(10)
!          OLiproc_chan(10) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect(OL_id_Bt2g1mHOL,kin(6)%pOLHinc,zm2arr,nhelnonv)
           do i = 1,16
              temp = temp + (zm2arr(1,i)+zm2arr(2,i))
     .              *dconjg((zm2arr(1,i)+zm2arr(2,i)))
           enddo
         elseif(iq1.eq.1 .and. i3.eq.4 .and. i4.eq.3 .and. iqbar2.eq.2 ) then
           OL_id_Bt2g1mHOL = OLiproc_chan(10)
!          OLiproc_chan(10) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect(OL_id_Bt2g1mHOL,kin(6)%pOLHinc,zm2arr,nhelnonv)
           do i = 1,16
              temp = temp + (zm2arr(1,i)+zm2arr(2,i))
     .              *dconjg((zm2arr(1,i)+zm2arr(2,i)))
           enddo
         elseif(iq1.eq.4 .and. i3.eq.3 .and. i4.eq.2 .and. iqbar2.eq.1 ) then
           OL_id_Bt2g1mHOL = OLiproc_chan(9)
!          OLiproc_chan(9) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect(OL_id_Bt2g1mHOL,kin(6)%pOLHinc,zm2arr,nhelnonv)
           do i = 1,16
              temp = temp + (zm2arr(1,i)+zm2arr(2,i))
     .              *dconjg((zm2arr(1,i)+zm2arr(2,i)))
           enddo
         elseif(iq1.eq.4 .and. i3.eq.2 .and. i4.eq.3 .and. iqbar2.eq.1 ) then
           OL_id_Bt2g1mHOL = OLiproc_chan(9)
!          OLiproc_chan(9) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect(OL_id_Bt2g1mHOL,kin(6)%pOLHinc,zm2arr,nhelnonv)
           do i = 1,16
              temp = temp + (zm2arr(1,i)+zm2arr(2,i))
     .              *dconjg((zm2arr(1,i)+zm2arr(2,i)))
           enddo
         elseif(iq1.eq.3 .and. i3.eq.4 .and. i4.eq.2 .and. iqbar2.eq.1 ) then
           OL_id_Bt2g1mHOL = OLiproc_chan(9)
!          OLiproc_chan(9) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 4, 3, 5, 6 /) )
            call evaluate_loop_colvect(OL_id_Bt2g1mHOL,kin(6)%pOLHinc,zm2arr,nhelnonv)
           do i = 1,16
              temp = temp + (zm2arr(1,i)+zm2arr(2,i))
     .              *dconjg((zm2arr(1,i)+zm2arr(2,i)))
           enddo
         elseif(iq1.eq.3 .and. i3.eq.2 .and. i4.eq.4 .and. iqbar2.eq.1 ) then
           OL_id_Bt2g1mHOL = OLiproc_chan(9)
!          OLiproc_chan(9) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 4, 3, 5, 6 /) )
            call evaluate_loop_colvect(OL_id_Bt2g1mHOL,kin(6)%pOLHinc,zm2arr,nhelnonv)
           do i = 1,16
              temp = temp + (zm2arr(1,i)+zm2arr(2,i))
     .              *dconjg((zm2arr(1,i)+zm2arr(2,i)))
           enddo
         elseif(iq1.eq.2 .and. i3.eq.4 .and. i4.eq.3 .and. iqbar2.eq.1 ) then
           OL_id_Bt2g1mHOL = OLiproc_chan(10)
!          OLiproc_chan(10) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect(OL_id_Bt2g1mHOL,kin(6)%pOLHinc,zm2arr,nhelnonv)
           do i = 1,16
              temp = temp + (zm2arr(1,i)+zm2arr(2,i))
     .              *dconjg((zm2arr(1,i)+zm2arr(2,i)))
           enddo
         elseif(iq1.eq.2 .and. i3.eq.3 .and. i4.eq.4 .and. iqbar2.eq.1 ) then
           OL_id_Bt2g1mHOL = OLiproc_chan(10)
!          OLiproc_chan(10) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect(OL_id_Bt2g1mHOL,kin(6)%pOLHinc,zm2arr,nhelnonv)
           do i = 1,16
              temp = temp + (zm2arr(1,i)+zm2arr(2,i))
     .              *dconjg((zm2arr(1,i)+zm2arr(2,i)))
           enddo
         elseif(iq1.eq.3 .and. i3.eq.1 .and. i4.eq.2 .and. iqbar2.eq.4 ) then
           OL_id_Bt2g1mHOL = OLiproc_chan(13)
!          OLiproc_chan(13) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect(OL_id_Bt2g1mHOL,kin(6)%pOLHinc,zm2arr,nhelnonv)
           do i = 1,16
              temp = temp + (zm2arr(1,i)+zm2arr(2,i))
     .              *dconjg((zm2arr(1,i)+zm2arr(2,i)))
           enddo
         elseif(iq1.eq.3 .and. i3.eq.2 .and. i4.eq.1 .and. iqbar2.eq.4 ) then
           OL_id_Bt2g1mHOL = OLiproc_chan(13)
!          OLiproc_chan(13) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect(OL_id_Bt2g1mHOL,kin(6)%pOLHinc,zm2arr,nhelnonv)
           do i = 1,16
              temp = temp + (zm2arr(1,i)+zm2arr(2,i))
     .              *dconjg((zm2arr(1,i)+zm2arr(2,i)))
           enddo
         elseif(iq1.eq.4 .and. i3.eq.1 .and. i4.eq.2 .and. iqbar2.eq.3 ) then
           OL_id_Bt2g1mHOL = OLiproc_chan(13)
!          OLiproc_chan(13) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 4, 3, 5, 6 /) )
            call evaluate_loop_colvect(OL_id_Bt2g1mHOL,kin(6)%pOLHinc,zm2arr,nhelnonv)
           do i = 1,16
              temp = temp + (zm2arr(1,i)+zm2arr(2,i))
     .              *dconjg((zm2arr(1,i)+zm2arr(2,i)))
           enddo
         elseif(iq1.eq.4 .and. i3.eq.2 .and. i4.eq.1 .and. iqbar2.eq.3 ) then
           OL_id_Bt2g1mHOL = OLiproc_chan(13)
!          OLiproc_chan(13) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 4, 3, 5, 6 /) )
            call evaluate_loop_colvect(OL_id_Bt2g1mHOL,kin(6)%pOLHinc,zm2arr,nhelnonv)
           do i = 1,16
              temp = temp + (zm2arr(1,i)+zm2arr(2,i))
     .              *dconjg((zm2arr(1,i)+zm2arr(2,i)))
           enddo
         elseif(iq1.eq.2 .and. i3.eq.1 .and. i4.eq.3 .and. iqbar2.eq.4 ) then
           OL_id_Bt2g1mHOL = OLiproc_chan(11)
!          OLiproc_chan(11) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect(OL_id_Bt2g1mHOL,kin(6)%pOLHinc,zm2arr,nhelnonv)
           do i = 1,16
              temp = temp + (zm2arr(1,i)+zm2arr(2,i))
     .              *dconjg((zm2arr(1,i)+zm2arr(2,i)))
           enddo
         elseif(iq1.eq.2 .and. i3.eq.3 .and. i4.eq.1 .and. iqbar2.eq.4 ) then
           OL_id_Bt2g1mHOL = OLiproc_chan(11)
!          OLiproc_chan(11) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect(OL_id_Bt2g1mHOL,kin(6)%pOLHinc,zm2arr,nhelnonv)
           do i = 1,16
              temp = temp + (zm2arr(1,i)+zm2arr(2,i))
     .              *dconjg((zm2arr(1,i)+zm2arr(2,i)))
           enddo
         elseif(iq1.eq.4 .and. i3.eq.3 .and. i4.eq.1 .and. iqbar2.eq.2 ) then
           OL_id_Bt2g1mHOL = OLiproc_chan(11)
!          OLiproc_chan(11) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect(OL_id_Bt2g1mHOL,kin(6)%pOLHinc,zm2arr,nhelnonv)
           do i = 1,16
              temp = temp + (zm2arr(1,i)+zm2arr(2,i))
     .              *dconjg((zm2arr(1,i)+zm2arr(2,i)))
           enddo
         elseif(iq1.eq.4 .and. i3.eq.1 .and. i4.eq.3 .and. iqbar2.eq.2 ) then
           OL_id_Bt2g1mHOL = OLiproc_chan(11)
!          OLiproc_chan(11) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect(OL_id_Bt2g1mHOL,kin(6)%pOLHinc,zm2arr,nhelnonv)
           do i = 1,16
              temp = temp + (zm2arr(1,i)+zm2arr(2,i))
     .              *dconjg((zm2arr(1,i)+zm2arr(2,i)))
           enddo
         elseif(iq1.eq.2 .and. i3.eq.1 .and. i4.eq.4 .and. iqbar2.eq.3 ) then
           OL_id_Bt2g1mHOL = OLiproc_chan(11)
!          OLiproc_chan(11) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 4, 3, 5, 6 /) )
            call evaluate_loop_colvect(OL_id_Bt2g1mHOL,kin(6)%pOLHinc,zm2arr,nhelnonv)
           do i = 1,16
              temp = temp + (zm2arr(1,i)+zm2arr(2,i))
     .              *dconjg((zm2arr(1,i)+zm2arr(2,i)))
           enddo
         elseif(iq1.eq.2 .and. i3.eq.4 .and. i4.eq.1 .and. iqbar2.eq.3 ) then
           OL_id_Bt2g1mHOL = OLiproc_chan(11)
!          OLiproc_chan(11) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 4, 3, 5, 6 /) )
            call evaluate_loop_colvect(OL_id_Bt2g1mHOL,kin(6)%pOLHinc,zm2arr,nhelnonv)
           do i = 1,16
              temp = temp + (zm2arr(1,i)+zm2arr(2,i))
     .              *dconjg((zm2arr(1,i)+zm2arr(2,i)))
           enddo
         elseif(iq1.eq.3 .and. i3.eq.4 .and. i4.eq.1 .and. iqbar2.eq.2 ) then
           OL_id_Bt2g1mHOL = OLiproc_chan(11)
!          OLiproc_chan(11) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 4, 3, 5, 6 /) )
            call evaluate_loop_colvect(OL_id_Bt2g1mHOL,kin(6)%pOLHinc,zm2arr,nhelnonv)
           do i = 1,16
              temp = temp + (zm2arr(1,i)+zm2arr(2,i))
     .              *dconjg((zm2arr(1,i)+zm2arr(2,i)))
           enddo
         elseif(iq1.eq.3 .and. i3.eq.1 .and. i4.eq.4 .and. iqbar2.eq.2 ) then
           OL_id_Bt2g1mHOL = OLiproc_chan(11)
!          OLiproc_chan(11) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 4, 3, 5, 6 /) )
            call evaluate_loop_colvect(OL_id_Bt2g1mHOL,kin(6)%pOLHinc,zm2arr,nhelnonv)
           do i = 1,16
              temp = temp + (zm2arr(1,i)+zm2arr(2,i))
     .              *dconjg((zm2arr(1,i)+zm2arr(2,i)))
           enddo
         else
           write(*,*)"OL_id_Bt2g1mHOL crossing in HJM not found"
           write(*,*)"iq1,i3,i4,iqbar2,i5,i6 =",iq1,i3,i4,iqbar2,i5,i6
           stop
         endif

      case('HJJM','HJJMREW')
         if(iq1.eq.1 .and. i3.eq.2 .and. i4.eq.3 .and. iqbar2.eq.4 ) then
           OL_id_Bt2g1mHOL = OLiproc_chan(2)
!          OLiproc_chan(9) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect(OL_id_Bt2g1mHOL,kin(6)%pOLHinc,zm2arr,nhelnonv)
           do i = 1,16
              temp = temp + (zm2arr(1,i)+zm2arr(2,i))
     .              *dconjg((zm2arr(1,i)+zm2arr(2,i)))
           enddo
         elseif(iq1.eq.1 .and. i3.eq.3 .and. i4.eq.2 .and. iqbar2.eq.4 ) then
           OL_id_Bt2g1mHOL = OLiproc_chan(2)
!          OLiproc_chan(9) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect(OL_id_Bt2g1mHOL,kin(6)%pOLHinc,zm2arr,nhelnonv)
           do i = 1,16
              temp = temp + (zm2arr(1,i)+zm2arr(2,i))
     .              *dconjg((zm2arr(1,i)+zm2arr(2,i)))
           enddo
         elseif(iq1.eq.1 .and. i3.eq.2 .and. i4.eq.4 .and. iqbar2.eq.3 ) then
           OL_id_Bt2g1mHOL = OLiproc_chan(2)
!          OLiproc_chan(9) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 4, 3, 5, 6 /) )
            call evaluate_loop_colvect(OL_id_Bt2g1mHOL,kin(6)%pOLHinc,zm2arr,nhelnonv)
           do i = 1,16
              temp = temp + (zm2arr(1,i)+zm2arr(2,i))
     .              *dconjg((zm2arr(1,i)+zm2arr(2,i)))
           enddo
         elseif(iq1.eq.1 .and. i3.eq.4 .and. i4.eq.2 .and. iqbar2.eq.3 ) then
           OL_id_Bt2g1mHOL = OLiproc_chan(2)
!          OLiproc_chan(9) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 4, 3, 5, 6 /) )
            call evaluate_loop_colvect(OL_id_Bt2g1mHOL,kin(6)%pOLHinc,zm2arr,nhelnonv)
           do i = 1,16
              temp = temp + (zm2arr(1,i)+zm2arr(2,i))
     .              *dconjg((zm2arr(1,i)+zm2arr(2,i)))
           enddo
         elseif(iq1.eq.1 .and. i3.eq.3 .and. i4.eq.4 .and. iqbar2.eq.2 ) then
           OL_id_Bt2g1mHOL = OLiproc_chan(3)
!          OLiproc_chan(10) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect(OL_id_Bt2g1mHOL,kin(6)%pOLHinc,zm2arr,nhelnonv)
           do i = 1,16
              temp = temp + (zm2arr(1,i)+zm2arr(2,i))
     .              *dconjg((zm2arr(1,i)+zm2arr(2,i)))
           enddo
         elseif(iq1.eq.1 .and. i3.eq.4 .and. i4.eq.3 .and. iqbar2.eq.2 ) then
           OL_id_Bt2g1mHOL = OLiproc_chan(3)
!          OLiproc_chan(10) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect(OL_id_Bt2g1mHOL,kin(6)%pOLHinc,zm2arr,nhelnonv)
           do i = 1,16
              temp = temp + (zm2arr(1,i)+zm2arr(2,i))
     .              *dconjg((zm2arr(1,i)+zm2arr(2,i)))
           enddo
         elseif(iq1.eq.4 .and. i3.eq.3 .and. i4.eq.2 .and. iqbar2.eq.1 ) then
           OL_id_Bt2g1mHOL = OLiproc_chan(2)
!          OLiproc_chan(9) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect(OL_id_Bt2g1mHOL,kin(6)%pOLHinc,zm2arr,nhelnonv)
           do i = 1,16
              temp = temp + (zm2arr(1,i)+zm2arr(2,i))
     .              *dconjg((zm2arr(1,i)+zm2arr(2,i)))
           enddo
         elseif(iq1.eq.4 .and. i3.eq.2 .and. i4.eq.3 .and. iqbar2.eq.1 ) then
           OL_id_Bt2g1mHOL = OLiproc_chan(2)
!          OLiproc_chan(9) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect(OL_id_Bt2g1mHOL,kin(6)%pOLHinc,zm2arr,nhelnonv)
           do i = 1,16
              temp = temp + (zm2arr(1,i)+zm2arr(2,i))
     .              *dconjg((zm2arr(1,i)+zm2arr(2,i)))
           enddo
         elseif(iq1.eq.3 .and. i3.eq.4 .and. i4.eq.2 .and. iqbar2.eq.1 ) then
           OL_id_Bt2g1mHOL = OLiproc_chan(2)
!          OLiproc_chan(9) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 4, 3, 5, 6 /) )
            call evaluate_loop_colvect(OL_id_Bt2g1mHOL,kin(6)%pOLHinc,zm2arr,nhelnonv)
           do i = 1,16
              temp = temp + (zm2arr(1,i)+zm2arr(2,i))
     .              *dconjg((zm2arr(1,i)+zm2arr(2,i)))
           enddo
         elseif(iq1.eq.3 .and. i3.eq.2 .and. i4.eq.4 .and. iqbar2.eq.1 ) then
           OL_id_Bt2g1mHOL = OLiproc_chan(2)
!          OLiproc_chan(9) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 4, 3, 5, 6 /) )
            call evaluate_loop_colvect(OL_id_Bt2g1mHOL,kin(6)%pOLHinc,zm2arr,nhelnonv)
           do i = 1,16
              temp = temp + (zm2arr(1,i)+zm2arr(2,i))
     .              *dconjg((zm2arr(1,i)+zm2arr(2,i)))
           enddo
         elseif(iq1.eq.2 .and. i3.eq.4 .and. i4.eq.3 .and. iqbar2.eq.1 ) then
           OL_id_Bt2g1mHOL = OLiproc_chan(3)
!          OLiproc_chan(10) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect(OL_id_Bt2g1mHOL,kin(6)%pOLHinc,zm2arr,nhelnonv)
           do i = 1,16
              temp = temp + (zm2arr(1,i)+zm2arr(2,i))
     .              *dconjg((zm2arr(1,i)+zm2arr(2,i)))
           enddo
         elseif(iq1.eq.2 .and. i3.eq.3 .and. i4.eq.4 .and. iqbar2.eq.1 ) then
           OL_id_Bt2g1mHOL = OLiproc_chan(3)
!          OLiproc_chan(10) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect(OL_id_Bt2g1mHOL,kin(6)%pOLHinc,zm2arr,nhelnonv)
           do i = 1,16
              temp = temp + (zm2arr(1,i)+zm2arr(2,i))
     .              *dconjg((zm2arr(1,i)+zm2arr(2,i)))
           enddo
         elseif(iq1.eq.3 .and. i3.eq.1 .and. i4.eq.2 .and. iqbar2.eq.4 ) then
           OL_id_Bt2g1mHOL = OLiproc_chan(6)
!          OLiproc_chan(13) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect(OL_id_Bt2g1mHOL,kin(6)%pOLHinc,zm2arr,nhelnonv)
           do i = 1,16
              temp = temp + (zm2arr(1,i)+zm2arr(2,i))
     .              *dconjg((zm2arr(1,i)+zm2arr(2,i)))
           enddo
         elseif(iq1.eq.3 .and. i3.eq.2 .and. i4.eq.1 .and. iqbar2.eq.4 ) then
           OL_id_Bt2g1mHOL = OLiproc_chan(6)
!          OLiproc_chan(13) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect(OL_id_Bt2g1mHOL,kin(6)%pOLHinc,zm2arr,nhelnonv)
           do i = 1,16
              temp = temp + (zm2arr(1,i)+zm2arr(2,i))
     .              *dconjg((zm2arr(1,i)+zm2arr(2,i)))
           enddo
         elseif(iq1.eq.4 .and. i3.eq.1 .and. i4.eq.2 .and. iqbar2.eq.3 ) then
           OL_id_Bt2g1mHOL = OLiproc_chan(6)
!          OLiproc_chan(13) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 4, 3, 5, 6 /) )
            call evaluate_loop_colvect(OL_id_Bt2g1mHOL,kin(6)%pOLHinc,zm2arr,nhelnonv)
           do i = 1,16
              temp = temp + (zm2arr(1,i)+zm2arr(2,i))
     .              *dconjg((zm2arr(1,i)+zm2arr(2,i)))
           enddo
         elseif(iq1.eq.4 .and. i3.eq.2 .and. i4.eq.1 .and. iqbar2.eq.3 ) then
           OL_id_Bt2g1mHOL = OLiproc_chan(6)
!          OLiproc_chan(13) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 4, 3, 5, 6 /) )
            call evaluate_loop_colvect(OL_id_Bt2g1mHOL,kin(6)%pOLHinc,zm2arr,nhelnonv)
           do i = 1,16
              temp = temp + (zm2arr(1,i)+zm2arr(2,i))
     .              *dconjg((zm2arr(1,i)+zm2arr(2,i)))
           enddo
         elseif(iq1.eq.2 .and. i3.eq.1 .and. i4.eq.3 .and. iqbar2.eq.4 ) then
           OL_id_Bt2g1mHOL = OLiproc_chan(4)
!          OLiproc_chan(4) inside registerOLHJJm()
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect(OL_id_Bt2g1mHOL,kin(6)%pOLHinc,zm2arr,nhelnonv)
           do i = 1,16
              temp = temp + (zm2arr(1,i)+zm2arr(2,i))
     .              *dconjg((zm2arr(1,i)+zm2arr(2,i)))
           enddo
         elseif(iq1.eq.2 .and. i3.eq.3 .and. i4.eq.1 .and. iqbar2.eq.4 ) then
           OL_id_Bt2g1mHOL = OLiproc_chan(4)
!          OLiproc_chan(4) inside registerOLHJJm()
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect(OL_id_Bt2g1mHOL,kin(6)%pOLHinc,zm2arr,nhelnonv)
           do i = 1,16
              temp = temp + (zm2arr(1,i)+zm2arr(2,i))
     .              *dconjg((zm2arr(1,i)+zm2arr(2,i)))
           enddo
         elseif(iq1.eq.4 .and. i3.eq.3 .and. i4.eq.1 .and. iqbar2.eq.2 ) then
           OL_id_Bt2g1mHOL = OLiproc_chan(4)
!          OLiproc_chan(4) inside registerOLHJJm()
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect(OL_id_Bt2g1mHOL,kin(6)%pOLHinc,zm2arr,nhelnonv)
           do i = 1,16
              temp = temp + (zm2arr(1,i)+zm2arr(2,i))
     .              *dconjg((zm2arr(1,i)+zm2arr(2,i)))
           enddo
         elseif(iq1.eq.4 .and. i3.eq.1 .and. i4.eq.3 .and. iqbar2.eq.2 ) then
           OL_id_Bt2g1mHOL = OLiproc_chan(4)
!          OLiproc_chan(4) inside registerOLHJJm()
           call fill_pOLHinc(6, (/ 1, 2, 3, 4, 5, 6 /) )
            call evaluate_loop_colvect(OL_id_Bt2g1mHOL,kin(6)%pOLHinc,zm2arr,nhelnonv)
           do i = 1,16
              temp = temp + (zm2arr(1,i)+zm2arr(2,i))
     .              *dconjg((zm2arr(1,i)+zm2arr(2,i)))
           enddo
         elseif(iq1.eq.2 .and. i3.eq.1 .and. i4.eq.4 .and. iqbar2.eq.3 ) then
           OL_id_Bt2g1mHOL = OLiproc_chan(4)
!          OLiproc_chan(11) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 4, 3, 5, 6 /) )
            call evaluate_loop_colvect(OL_id_Bt2g1mHOL,kin(6)%pOLHinc,zm2arr,nhelnonv)
           do i = 1,16
              temp = temp + (zm2arr(1,i)+zm2arr(2,i))
     .              *dconjg((zm2arr(1,i)+zm2arr(2,i)))
           enddo
         elseif(iq1.eq.2 .and. i3.eq.4 .and. i4.eq.1 .and. iqbar2.eq.3 ) then
           OL_id_Bt2g1mHOL = OLiproc_chan(4)
!          OLiproc_chan(11) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 4, 3, 5, 6 /) )
            call evaluate_loop_colvect(OL_id_Bt2g1mHOL,kin(6)%pOLHinc,zm2arr,nhelnonv)
           do i = 1,16
              temp = temp + (zm2arr(1,i)+zm2arr(2,i))
     .              *dconjg((zm2arr(1,i)+zm2arr(2,i)))
           enddo
         elseif(iq1.eq.3 .and. i3.eq.4 .and. i4.eq.1 .and. iqbar2.eq.2 ) then
           OL_id_Bt2g1mHOL = OLiproc_chan(4)
!          OLiproc_chan(11) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 4, 3, 5, 6 /) )
            call evaluate_loop_colvect(OL_id_Bt2g1mHOL,kin(6)%pOLHinc,zm2arr,nhelnonv)
           do i = 1,16
              temp = temp + (zm2arr(1,i)+zm2arr(2,i))
     .              *dconjg((zm2arr(1,i)+zm2arr(2,i)))
           enddo
         elseif(iq1.eq.3 .and. i3.eq.1 .and. i4.eq.4 .and. iqbar2.eq.2 ) then
           OL_id_Bt2g1mHOL = OLiproc_chan(4)
!          OLiproc_chan(11) inside registerOLHJm()
           call fill_pOLHinc(6, (/ 1, 2, 4, 3, 5, 6 /) )
            call evaluate_loop_colvect(OL_id_Bt2g1mHOL,kin(6)%pOLHinc,zm2arr,nhelnonv)
           do i = 1,16
              temp = temp + (zm2arr(1,i)+zm2arr(2,i))
     .              *dconjg((zm2arr(1,i)+zm2arr(2,i)))
           enddo
         else
           write(*,*)"OL_id_Bt2g1mHOL crossing in HJJM not found"
           write(*,*)"iq1,i3,i4,iqbar2,i5,i6 =",iq1,i3,i4,iqbar2,i5,i6
           stop
         endif

      case default
         write(*,*) "Invalid process in crossProc: ", name_proc
         stop
      end select
!Calculate the vev used in OL:
      call get_parameter("alpha", alpha)
      call get_parameter("alphas", alpha_s)
      call get_parameter("w_mass", MW)
      call get_parameter("z_mass", MZ)
!$omp end critical(eval_ol)
      cw   = MW/MZ
      cw2  = cw**2d0  
      sw2  = 1d0 - cw2
      sw   = sqrt(sw2)
      vev  = 2d0*MW*sw/sqrt(4d0*pi*alpha)
      !write(*,*)"vev=",vev
      C0=(alpha_s/6d0/pi/vev)**2
      prefactor = (dble(nc)**2-1d0)*C0*(4d0*pi*alpha_s)**2/2d0
      !write(*,*)"prefactor=",prefactor
      !write(*,*)"kin(6)%pOLHinc=",kin(6)%pOLHinc
!Above parameters matches the OpenLoops pre-factor and we want to remove them for NNLOJET convention
      Bt2g1mH = temp/prefactor
#endif

      return
      end





ccc This is reweighted 1-loop libarary. 
************************************************************************
      function B1g1mHrew(i1,i2,i3,i4,i5,renscale2)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer i1,i2,i3,i4,i5

      B1g1mHrew = B1g1H(i1,i2,i3,i4,i5,renscale2)*B1g1mH(i1,i2,i3,i4,i5)/B1g0H(i1,i2,i3,i4,i5)

      return
      end
************************************************************************
      function Bh1g1mHrew(i1,i2,i3,i4,i5,renscale2)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer i1,i2,i3,i4,i5

      Bh1g1mHrew = Bh1g1H(i1,i2,i3,i4,i5,renscale2)*B1g1mH(i1,i2,i3,i4,i5)/B1g0H(i1,i2,i3,i4,i5)

      return                                                            
      end

************************************************************************
      function Bt1g1mHrew(i1,i2,i3,i4,i5,renscale2)
      implicit double precision (a-h,o-y)                               
      implicit complex*16 (z)                                           
      integer i1,i2,i3,i4,i5                                            

      Bt1g1mHrew = Bt1g1H(i1,i2,i3,i4,i5,renscale2)*B1g1mH(i1,i2,i3,i4,i5)/B1g0H(i1,i2,i3,i4,i5)

      return                                                            
      end

************************************************************************
      function B2g1mHrew(i1,i2,i3,i4,i5,i6,renscale2)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer i1,i2,i3,i4,i5,i6

      B2g1mHrew = B2g1H(i1,i2,i3,i4,i5,i6,renscale2)*B2g1mH(i1,i2,i3,i4,i5,i6)/B2g0H(i1,i2,i3,i4,i5,i6)

      return
      end

************************************************************************
      function Bt2g1mHrew(i1,i2,i3,i4,i5,i6,renscale2)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer i1,i2,i3,i4,i5,i6

      Bt2g1mHrew = Bt2g1H(i1,i2,i3,i4,i5,i6,renscale2)*B2g1mH(i1,i2,i3,i4,i5,i6)/B2g0H(i1,i2,i3,i4,i5,i6)

      return
      end

************************************************************************
      function Bh2g1mHrew(i1,i2,i3,i4,i5,i6,renscale2)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer i1,i2,i3,i4,i5,i6

      Bh2g1mHrew = Bh2g1H(i1,i2,i3,i4,i5,i6,renscale2)*B2g1mH(i1,i2,i3,i4,i5,i6)/B2g0H(i1,i2,i3,i4,i5,i6)

      return
      end

************************************************************************
      function Btt2g1mHrew(i1,i2,i3,i4,i5,i6,renscale2)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer i1,i2,i3,i4,i5,i6

      Btt2g1mHrew = Btt2g1H(i1,i2,i3,i4,i5,i6,renscale2)*Bt2g1mH(i1,i2,i3,i4,i5,i6)/Bt2g0H(i1,i2,i3,i4,i5,i6)

      return
      end

************************************************************************
      function Bttt2g1mHrew(i1,i2,i3,i4,i5,i6,renscale2)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer i1,i2,i3,i4,i5,i6

      Bttt2g1mHrew = Bttt2g1H(i1,i2,i3,i4,i5,i6,renscale2)*Bt2g1mH(i1,i2,i3,i4,i5,i6)/Bt2g0H(i1,i2,i3,i4,i5,i6)

      return
      end

************************************************************************
      function Btth2g1mHrew(i1,i2,i3,i4,i5,i6,renscale2)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer i1,i2,i3,i4,i5,i6

      Btth2g1mHrew = Btth2g1H(i1,i2,i3,i4,i5,i6,renscale2)*Bt2g1mH(i1,i2,i3,i4,i5,i6)/Bt2g0H(i1,i2,i3,i4,i5,i6)

      return
      end

************************************************************************
      function Bhh2g1mHrew(i1,i2,i3,i4,i5,i6,renscale2)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer i1,i2,i3,i4,i5,i6

      Bhh2g1mHrew = Bhh2g1H(i1,i2,i3,i4,i5,i6,renscale2)*Bt2g1mH(i1,i2,i3,i4,i5,i6)/Bt2g0H(i1,i2,i3,i4,i5,i6)

      return
      end

************************************************************************
      function Btt2g1mHFrew(i1,i3,i4,i2,i5,i6,renscale2)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer i1,i2,i3,i4,i5,i6

      Btt2g1mHFrew=Btt2g1mHrew(i1,i3,i4,i2,i5,i6,renscale2)-Bhh2g1mHrew(i1,i3,i4,i2,i5,i6,renscale2)

      return
      end
