*************************************************************************
!XC Tree level ME for Higgs plus up to one gluon with two qqbar pairs
!XC for one gluon with two qqbar pair analytical results calculated using BCFW method: 
!  R.~Britto, F.~Cachazo and B.~Feng,
!  %``Generalized unitarity and one-loop amplitudes in N=4 super-Yang-Mills,''
!  Nucl.\ Phys.\ B {\bf 725} (2005) 275
!  doi:10.1016/j.nuclphysb.2005.07.014
!  [hep-th/0412103].

!  R.~Britto, F.~Cachazo, B.~Feng and E.~Witten,
!  %``Direct proof of tree-level recursion relation in Yang-Mills theory,''
!  Phys.\ Rev.\ Lett.\  {\bf 94} (2005) 181602
!  doi:10.1103/PhysRevLett.94.181602
!  [hep-th/0501052].

!  R.~Britto, F.~Cachazo, B.~Feng and E.~Witten,
!  %``Direct proof of tree-level recursion relation in Yang-Mills theory,''
!  Phys.\ Rev.\ Lett.\  {\bf 94} (2005) 181602
!  doi:10.1103/PhysRevLett.94.181602
!  [hep-th/0501052].

************************************************************************
*XC two q qbar with different flavor and one higgs tree level matrix elements
*XC summed over all colour orderings, summed and squared
*
      function FullC0g0H(iq1,iQbar3,iQ4,iqbar2,i5,i6)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
*XC (as/3d0/pi/239d0)**2=as/3d0*pi/246d0 for gluon gluon fusion to higgs effective
*XC coupling constant
      stop "set value for chgg correctly!"
      fac=(4d0*pi*as)**2*(nc**2-1d0)*chgg**2/4d0
*XC form (3.1) of hep-th/0412275 coefficient squared

      FullC0g0H=
     .            +C0g0H(iq1,iQbar3,iQ4,iqbar2,i5,i6)

      FullC0g0H=fac*FullC0g0H
      return
      end
************************************************************************
*XC  two q qbar with different flavor and one higgs amplitude
*XC  one colour ordering, summed and squared  with both phi and cphi field
      function C0g0H(iq1,iQbar3,iQ4,iqbar2,i5,i6)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer NHEL(4,2),PERM(6)

      DATA (NHEL(IHEL,   1),IHEL=1, 4) / -1,-1, 1, 1/
      DATA (NHEL(IHEL,   2),IHEL=1, 4) / -1, 1,-1, 1/

        perm(1)=iq1
        perm(2)=iQbar3
        perm(3)=iQ4
        perm(4)=iqbar2
        perm(5)=i5
        perm(6)=i6

        amp2=0d0
        do i=1,2
          zamp=zC0g0HMHV(PERM(1),NHEL(1,i))
          amp2=amp2+2d0*zamp*dconjg(zamp)
        enddo
        C0g0H=amp2

      return
      end
************************************************************************
*XC two q qbar with identical flavor and one higgs tree level matrix elements
*XC summed over all colour orderings, summed and squared
      function FullD0g0H(iq1,iQbar3,iQ4,iqbar2,i5,i6)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)

*XC (as/3d0/pi/239d0)**2=as/3d0*pi/246d0 for gluon gluon fusion to higgs effective
*XC coupling constant
      stop "set value for chgg correctly!"
      fac=(4d0*pi*as)**2*(nc**2-1d0)*chgg**2/4d0
*XC form (3.1) of hep-th/0412275 coefficient squared

      FullD0g0H=
     .            +C0g0H(iq1,iQbar3,iQ4,iqbar2,i5,i6)
     .            +C0g0H(iq1,iqbar2,iQ4,iQbar3,i5,i6)
     .            -D0g0H(iq1,iQbar3,iQ4,iqbar2,i5,i6)/Nc

      FullD0g0H=fac*FullD0g0H
      return
      end
************************************************************************
*XC  two q qbar pairs with identical flavor and one higgs amplitude
*XC one colour ordering, summed and squared  with both phi and cphi field
      function D0g0H(iq1,iQbar3,iQ4,iqbar2,i5,i6)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /koppel/ nf,nc,b0
      integer NHEL(4,1),PERMA(6)

      DATA (NHEL(IHEL,   1),IHEL=1, 4) /  1,-1, 1,-1/

        perma(1)=iq1
        perma(2)=iQbar3
        perma(3)=iQ4
        perma(4)=iqbar2
        perma(5)=i5
        perma(6)=i6

        amp2=0d0
            zampa=zD0g0HMHVA(PERMA(1),NHEL(1,1))
            zampb=zD0g0HMHVB(PERMA(1),NHEL(1,1))
            zamp=-2d0*real(zampa*dconjg(zampb))
        amp2=amp2+2d0*zamp

        D0g0H=amp2
      return
      end
************************************************************************
* XC two quark pair (nonidentical) and one gluon one higgs tree level matrix elements
* XC summed over colour orderings, summed and squared
* XC gluon in final state
      function FullC1g0H(iq1,i5,iQbar4,iQ3,iqbar2,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
*XC C=(as/6d0/pi/239d0) and C/2 for gluon gluon fusion to higgs effective
*XC coupling constant
*XC (2.1) of hep-th/0411092v2
*XC use the value form MG5 that C=5.235154617065361E-005 and we need to
*XC take (C/2)**2 after squaring
      stop "set value for chgg correctly!"
      fac=(4d0*pi*as)**3*(nc**2-1d0)*nc*chgg**2/4d0
*XC form (3.1) of hep-th/0412275 coefficient squared

      FullC1g0H=
     .            +C1g0HF(iq1,i5,iQbar4,iQ3,iqbar2,i6,i7)
     .            +Ct1g0HF(iq1,i5,iQbar4,iQ3,iqbar2,i6,i7)/nc**2

      FullC1g0H=fac*FullC1g0H

      return
      end
************************************************************************
      function FullC1g0Havg(iq1,i5,iQbar4,iQ3,iqbar2,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)

      stop "set value for chgg correctly!"
      fac=(4d0*pi*as)**3*(nc**2-1d0)*nc*chgg**2/4d0

      FullC1g0Havg=
     .            +C1g0HF(iq1,i5,iQbar4,iQ3,iqbar2,i6,i7)
     .            +Ct1g0HFavg(iq1,i5,iQbar4,iQ3,iqbar2,i6,i7)/nc**2

*      FullC1g0Havg=fac*FullC1g0Havg

      return
      end
************************************************************************
* XC two quark pair (nonidentical) and one gluon one higgs tree level matrix elements
* XC summed over colour orderings, summed and squared, color leading contribution
      function C1g0HF(iq1,i5,iQbar4,iQ3,iqbar2,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)

      C1g0HF=
     .            +C1g0Ha(iq1,i5,iQbar4,iQ3,iqbar2,i6,i7)
     .            +C1g0Hb(iq1,iQbar4,iQ3,i5,iqbar2,i6,i7)

      return
      end

************************************************************************
* XC two quark pair (nonidentical) and one gluon one higgs tree level matrix elements
* XC summed over colour orderings, summed and squared, color sub-leading contribution.
* XC avargaing over the final state Q, Qbar warping.
      function Ct1g0HFavg(iq1,i5,iQbar4,iQ3,iqbar2,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)

      Ct1g0HFavg=
     .   1d0/2d0*(+Ct1g0HF(iq1,i5,iQbar4,iQ3,iqbar2,i6,i7)
     .            +Ct1g0HF(iq1,i5,iQ3,iQbar4,iqbar2,i6,i7)
     .           )

      return
      end
************************************************************************
* XC two quark pair (nonidentical) and one gluon one higgs tree level matrix elements
* XC summed over colour orderings, summed and squared, color sub-leading contribution
      function Ct1g0HF(iq1,i5,iQbar4,iQ3,iqbar2,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)

      if(iq1.ne.1 .and. iq1 .ne.2 .and. iqbar2.ne.1 .and. iqbar2 .ne.2) then
        Ct1g0HF=1d0/2d0*(
     .            +Ct1g0Ha(iq1,i5,iqbar2,iQ3,iQbar4,i6,i7)
     .            +Ct1g0Hb(iq1,iqbar2,iQ3,i5,iQbar4,i6,i7)
     .            -Ctt1g0H(iq1,i5,iQbar4,iQ3,iqbar2,i6,i7)

     .            +Ct1g0Ha(iqbar2,i5,iq1,iQ3,iQbar4,i6,i7)
     .            +Ct1g0Hb(iqbar2,iq1,iQ3,i5,iQbar4,i6,i7)
     .            -Ctt1g0H(iqbar2,i5,iQbar4,iQ3,iq1,i6,i7)
     .                 )
      elseif(iQ3.ne.1 .and. iQ3 .ne.2 .and. iQbar4.ne.1 .and. iQbar4 .ne.2) then
        Ct1g0HF=1d0/2d0*(
     .            +Ct1g0Ha(iq1,i5,iqbar2,iQ3,iQbar4,i6,i7)
     .            +Ct1g0Hb(iq1,iqbar2,iQ3,i5,iQbar4,i6,i7)
     .            -Ctt1g0H(iq1,i5,iQbar4,iQ3,iqbar2,i6,i7)

     .            +Ct1g0Ha(iq1,i5,iqbar2,iQbar4,iQ3,i6,i7)
     .            +Ct1g0Hb(iq1,iqbar2,iQbar4,i5,iQ3,i6,i7)
     .            -Ctt1g0H(iq1,i5,iQ3,iQbar4,iqbar2,i6,i7)
     .                 )
      else
      Ct1g0HF=
     .            +Ct1g0Ha(iq1,i5,iqbar2,iQ3,iQbar4,i6,i7)
     .            +Ct1g0Hb(iq1,iqbar2,iQ3,i5,iQbar4,i6,i7)
     .            -Ctt1g0H(iq1,i5,iQbar4,iQ3,iqbar2,i6,i7)

      endif

      return
      end
************************************************************************
*XC two quark pair (nonidentical) and one gluon one higgs tree level amplitude
*XC colour leading part for A1 order, summed and squared  with both phi and cphi field
      function C1g0Ha(iq1,i5,iQbar4,iQ3,iqbar2,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer NHEL(5,16),PERM(7)

      DATA (NHEL(IHEL,   1),IHEL=1, 5) / -1, 1, 1,-1, 1/
      DATA (NHEL(IHEL,   2),IHEL=1, 5) / -1, 1,-1, 1, 1/
      DATA (NHEL(IHEL,   3),IHEL=1, 5) / -1,-1, 1,-1, 1/
      DATA (NHEL(IHEL,   4),IHEL=1, 5) / -1,-1,-1, 1, 1/
        perm(1)=iq1
        perm(2)=i5
        perm(3)=iQbar4
        perm(4)=iQ3
        perm(5)=iqbar2
        perm(6)=i6
        perm(7)=i7

         amp2=0d0
       do i=1,4
           if(i.eq.1) then
            zamp=zC1g0Ha1mppmp(PERM(1))
           endif
           if(i.eq.2) then
            zamp=zC1g0Ha1mpmpp(PERM(1))
           endif
           if(i.eq.3) then
            zamp=zC1g0Ha1mmpmp(PERM(1))
           endif
           if(i.eq.4) then
            zamp=zC1g0Ha1mmmpp(PERM(1))
           endif
          amp2=amp2+2d0*zamp*dconjg(zamp)
       enddo

         C1g0Ha=amp2

   10 format(" C1g0Ha   ",e12.4,6i3)
      return
      end
************************************************************************
*XC two quark pair (nonidentical) and one gluon one higgs tree level amplitude
*XC colour leading part for A2 order, summed and squared  with both phi and cphi field
      function C1g0Hb(iq1,iQbar4,iQ3,i5,iqbar2,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer NHEL(5,16),PERM(7)

      DATA (NHEL(IHEL,   1),IHEL=1, 5) / -1, 1,-1, 1, 1/
      DATA (NHEL(IHEL,   2),IHEL=1, 5) / -1,-1, 1, 1, 1/
      DATA (NHEL(IHEL,   3),IHEL=1, 5) / -1, 1,-1,-1, 1/
      DATA (NHEL(IHEL,   4),IHEL=1, 5) / -1,-1, 1,-1, 1/

        perm(1)=iq1
        perm(2)=iQbar4
        perm(3)=iQ3
        perm(4)=i5
        perm(5)=iqbar2
        perm(6)=i6
        perm(7)=i7

         amp2=0d0
       do i=1,4
           if(i.eq.1) then
            zamp=zC1g0Ha2mpmpp(PERM(1))
           endif
           if(i.eq.2) then
            zamp=zC1g0Ha2mmppp(PERM(1))
           endif
           if(i.eq.3) then
            zamp=zC1g0Ha2mpmmp(PERM(1))
           endif
           if(i.eq.4) then
            zamp=zC1g0Ha2mmpmp(PERM(1))
           endif
          amp2=amp2+2d0*zamp*dconjg(zamp)
       enddo

        C1g0Hb=amp2

   10 format(" C1g0Hb   ",e12.4,6i3)
      return
      end
************************************************************************
*XC two quark pair (nonidentical) and one gluon one higgs tree level amplitude
*XC colour leading part for B1 order, summed and squared  with both phi and cphi field
      function Ct1g0Ha(iq1,i5,iqbar2,iQ3,iQbar4,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer NHEL(5,16),PERM(7),perma(7)

      DATA (NHEL(IHEL,   1),IHEL=1, 5) / -1, 1, 1,-1, 1/
      DATA (NHEL(IHEL,   2),IHEL=1, 5) / -1, 1, 1, 1,-1/
      DATA (NHEL(IHEL,   3),IHEL=1, 5) / -1,-1, 1,-1, 1/
      DATA (NHEL(IHEL,   4),IHEL=1, 5) / -1,-1, 1, 1,-1/

        perm(1)=iq1
        perm(2)=i5
        perm(3)=iqbar2
        perm(4)=iQ3
        perm(5)=iQbar4
        perm(6)=i6
        perm(7)=i7

         amp2=0d0
       do i=1,4
           if(i.eq.1) then
            zamp=zC1g0Hb1mppmp(PERM(1))
           endif
           if(i.eq.2) then
            zamp=zC1g0Hb1mpppm(PERM(1))
           endif
           if(i.eq.3) then
            zamp=zC1g0Hb1mmpmp(PERM(1))
           endif
           if(i.eq.4) then
            zamp=zC1g0Hb1mmppm(PERM(1))
           endif
          amp2=amp2+2d0*zamp*dconjg(zamp)
       enddo
         Ct1g0Ha=amp2

   10 format(" Ct1g0Ha   ",e12.4,6i3)
      return
      end
************************************************************************
*XC two quark pair (nonidentical) and one gluon one higgs tree level amplitude
*XC colour leading part for B2 order, summed and squared  with both phi and cphi field
      function Ct1g0Hb(iq1,iqbar2,iQ3,i5,iQbar4,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer NHEL(5,16),PERM(7),perma(7)

      DATA (NHEL(IHEL,   1),IHEL=1, 5) / -1, 1,-1, 1, 1/
      DATA (NHEL(IHEL,   2),IHEL=1, 5) / -1, 1, 1, 1,-1/
      DATA (NHEL(IHEL,   3),IHEL=1, 5) / -1, 1,-1,-1, 1/
      DATA (NHEL(IHEL,   4),IHEL=1, 5) / -1, 1, 1,-1,-1/

        perm(1)=iq1
        perm(2)=iqbar2
        perm(3)=iQ3
        perm(4)=i5
        perm(5)=iQbar4
        perm(6)=i6
        perm(7)=i7

         amp2=0d0
       do i=1,4
           if(i.eq.1) then
            zamp=zC1g0Hb2mpmpp(PERM(1))
           endif
           if(i.eq.2) then
            zamp=zC1g0Hb2mpppm(PERM(1))
           endif
           if(i.eq.3) then
            zamp=zC1g0Hb2mpmmp(PERM(1))
           endif
           if(i.eq.4) then
            zamp=zC1g0Hb2mppmm(PERM(1))
           endif
          amp2=amp2+2d0*zamp*dconjg(zamp)
       enddo

         Ct1g0Hb=amp2

   10 format(" Ct1g0Hb   ",e12.4,6i3)
      return
      end
************************************************************************
*XC two quark pair (nonidentical) and one gluon one higgs tree level amplitude
*XC colour mixing part for (A1+A2)(B1+B2)^*+(B1+B2)(A1+A2)^*, summed and squared  with both phi and cphi field
*XC gluon in final state
      function Ctt1g0H(iq1,i5,iQbar4,iQ3,iqbar2,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer NHEL(5,16),PERMA1(7),PERMA2(7),PERMB1(7),PERMB2(7)
      DATA (NHEL(IHEL,   1),IHEL=1, 5) / -1, 1, 1,-1, 1/
      DATA (NHEL(IHEL,   2),IHEL=1, 5) / -1, 1,-1, 1, 1/
      DATA (NHEL(IHEL,   3),IHEL=1, 5) / -1,-1, 1,-1, 1/
      DATA (NHEL(IHEL,   4),IHEL=1, 5) / -1,-1,-1, 1, 1/

        permb1(1)=iq1
        permb1(2)=i5
        permb1(3)=iqbar2
        permb1(4)=iQ3
        permb1(5)=iQbar4
        permb1(6)=i6
        permb1(7)=i7

        permb2(1)=iq1
        permb2(2)=iqbar2
        permb2(3)=iQ3
        permb2(4)=i5
        permb2(5)=iQbar4
        permb2(6)=i6
        permb2(7)=i7

         amp2=0d0
       do i=1,4
           if(i.eq.1) then
            zamp=zC1g0Hb1mppmp(PERMB1(1))-zC1g0Hb2mpmpp(PERMB2(1))
           endif
           if(i.eq.2) then
            zamp=zC1g0Hb1mpppm(PERMB1(1))-zC1g0Hb2mpppm(PERMB2(1))
           endif
           if(i.eq.3) then
            zamp=zC1g0Hb1mmpmp(PERMB1(1))-zC1g0Hb2mpmmp(PERMB2(1))
           endif
           if(i.eq.4) then
            zamp=zC1g0Hb1mmppm(PERMB1(1))-zC1g0Hb2mppmm(PERMB2(1))
           endif

          amp2=amp2+4d0*zamp*conjg(zamp)
*XC here 4d0*zamp= 2d0*2d0*zamp, the first 2d0 is for double real part of (A1+A2)*(B1+B2)^\dagger
*XC which equals to (A1+A2)*(B1+B2)^\dagger+(B1+B2)*(A1+A2)^\dagger, this could also be seen as two
*XC times of the |A1+A2|^{2} as A1+A2=B1+B2
*XC the second 2d0 is for the other opposite helicity
       enddo
         Ctt1g0H=amp2
   10 format(" C1g0Hcm   ",e12.4,6i3)
      return
      end
************************************************************************
*XC two quark pair (identical) and one gluon one higgs tree level matrix elements
*XC summed over colour orderings, summed and squared
      function FullD1g0H(iq1,i5,iQbar4,iQ3,iqbar2,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
* C=(as/6d0/pi/239d0) and C/2 for gluon gluon fusion to higgs effective
* coupling constant
*XC (2.1) of hep-th/0411092v2
*XC use the value form MG5 that C=5.235154617065361E-005 and we need to
*XC take (C/2)**2 after squaring
      stop "set value for chgg correctly!"
      fac=(4d0*pi*as)**3*(nc**2-1d0)*nc*chgg**2/4d0
*XC form (3.1) of hep-th/0412275 coefficient squared

      FullD1g0H=
     .            +C1g0Ha(iq1,i5,iQbar4,iQ3,iqbar2,i6,i7)
     .            +C1g0Hb(iq1,iQbar4,iQ3,i5,iqbar2,i6,i7)
     .            +Ct1g0Ha(iq1,i5,iqbar2,iQ3,iQbar4,i6,i7)/nc**2
     .            +Ct1g0Hb(iq1,iqbar2,iQ3,i5,iQbar4,i6,i7)/nc**2
     .            -Ctt1g0H(iq1,i5,iQbar4,iQ3,iqbar2,i6,i7)/nc**2

     .            +C1g0Ha(iq1,i5,iqbar2,iQ3,iQbar4,i6,i7)
     .            +C1g0Hb(iq1,iqbar2,iQ3,i5,iQbar4,i6,i7)
     .            +Ct1g0Ha(iq1,i5,iQbar4,iQ3,iqbar2,i6,i7)/nc**2
     .            +Ct1g0Hb(iq1,iQbar4,iQ3,i5,iqbar2,i6,i7)/nc**2
     .            -Ctt1g0H(iq1,i5,iqbar2,iQ3,iQbar4,i6,i7)/nc**2

     .            -D1g0H(iq1,i5,iQbar4,iQ3,iqbar2,i6,i7)/nc
     .            +Dt1g0H(iq1,i5,iQbar4,iQ3,iqbar2,i6,i7)/nc
     .            +Dt1g0H(iq1,i5,iQbar4,iQ3,iqbar2,i6,i7)/nc**3

      FullD1g0H=fac*FullD1g0H
      return
      end
************************************************************************
*XC two quark pair (identical) and one gluon one higgs tree level matrix elements
*XC summed over colour orderings, summed and squared
      function FD1g0H(iq1,i5,iQbar4,iQ3,iqbar2,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
* C=(as/6d0/pi/239d0) and C/2 for gluon gluon fusion to higgs effective
* coupling constant
*XC (2.1) of hep-th/0411092v2
*XC use the value form MG5 that C=5.235154617065361E-005 and we need to
*XC take (C/2)**2 after squaring
      stop "set value for chgg correctly!"
      fac=(4d0*pi*as)**3*(nc**2-1d0)*nc*chgg**2/4d0
*XC form (3.1) of hep-th/0412275 coefficient squared

      FD1g0H=
     .            -D1g0H(iq1,i5,iQbar4,iQ3,iqbar2,i6,i7)/nc
     .            +Dt1g0H(iq1,i5,iQbar4,iQ3,iqbar2,i6,i7)/nc
     .            +Dt1g0H(iq1,i5,iQbar4,iQ3,iqbar2,i6,i7)/nc**3

*      FD1g0H=fac*FD1g0H
      return
      end

************************************************************************
*XC two quark pair (identical) and one gluon one higgs tree level matrix elements
*XC summed over colour orderings, summed and squared
      function D1g0HF(iq1,i5,iQbar4,iQ3,iqbar2,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)

      D1g0HF=
     .            +D1g0H(iq1,i5,iQbar4,iQ3,iqbar2,i6,i7)
     .            -Dt1g0H(iq1,i5,iQbar4,iQ3,iqbar2,i6,i7)

      return
      end
************************************************************************
*XC two quark pair (identical) and one gluon one higgs tree level amplitude
*XC colour mixing part for (A1+A2)(B1+B2)^*+(B1+B2)(A1+A2)^*, summed and squared  with both phi and cphi field
      function D1g0H(iq1,i5,iQbar4,iQ3,iqbar2,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /koppel/ nf,nc,b0
      integer NHEL(5,2),PERMA1(7),PERMA2(7),PERMB1(7),PERMB2(7),PERMAA1(7),PERMAA2(7),PERMBB1(7),PERMBB2(7)

      DATA (NHEL(IHEL,   1),IHEL=1, 5) / -1, 1, 1,-1, 1/
      DATA (NHEL(IHEL,   2),IHEL=1, 5) / -1,-1, 1,-1, 1/

        perma1(1)=iq1
        perma1(2)=i5
        perma1(3)=iQbar4
        perma1(4)=iQ3
        perma1(5)=iqbar2
        perma1(6)=i6
        perma1(7)=i7

        perma2(1)=iq1
        perma2(2)=iQbar4
        perma2(3)=iQ3
        perma2(4)=i5
        perma2(5)=iqbar2
        perma2(6)=i6
        perma2(7)=i7

        permb1(1)=iq1
        permb1(2)=i5
        permb1(3)=iqbar2
        permb1(4)=iQ3
        permb1(5)=iQbar4
        permb1(6)=i6
        permb1(7)=i7

        permb2(1)=iq1
        permb2(2)=iqbar2
        permb2(3)=iQ3
        permb2(4)=i5
        permb2(5)=iQbar4
        permb2(6)=i6
        permb2(7)=i7


         amp2=0d0
       do i=1,2
           if(i.eq.1) then
            zamp=-real(zC1g0Ha1mppmp(PERMA1(1))*conjg(zC1g0Hb1mppmp(PERMA1(1)))
     .        +zC1g0Ha2mpmpp(PERMA2(1))*conjg(zC1g0Hb2mpmpp(PERMA2(1)))
     .        +zC1g0Hb1mppmp(PERMB1(1))*conjg(zC1g0Ha1mppmp(PERMB1(1)))
     .        +zC1g0Hb2mpmpp(PERMB2(1))*conjg(zC1g0Ha2mpmpp(PERMB2(1))))
           endif

           if(i.eq.2) then
            zamp=-real(zC1g0Ha1mmpmp(PERMA1(1))*conjg(zC1g0Hb1mmpmp(PERMA1(1)))
     .        +zC1g0Ha2mpmmp(PERMA2(1))*conjg(zC1g0Hb2mpmmp(PERMA2(1)))
     .        +zC1g0Hb1mmpmp(PERMB1(1))*conjg(zC1g0Ha1mmpmp(PERMB1(1)))
     .        +zC1g0Hb2mpmmp(PERMB2(1))*conjg(zC1g0Ha2mpmmp(PERMB2(1))))
           endif

          amp2=amp2+4d0*zamp
       enddo

       if(i5.eq.1.or.i5.eq.2) then
         D1g0H=-amp2
       else
*XC gluon in final state
         D1g0H=amp2
*XC gluon in initial state
       endif

   10 format(" D1g0H   ",e12.4,6i3)
      return
      end
************************************************************************
*XC two quark pair (identical) and one gluon one higgs tree level amplitude
*XC colour mixing part for (A1+A2)(B1+B2)^*+(B1+B2)(A1+A2)^*, summed and squared  with both phi and cphi field
      function Dt1g0H(iq1,i5,iQbar4,iQ3,iqbar2,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /koppel/ nf,nc,b0
      integer NHEL(5,2),PERMA1(7),PERMA2(7),PERMB1(7),PERMB2(7),PERMAA1(7),PERMAA2(7),PERMBB1(7),PERMBB2(7)

      DATA (NHEL(IHEL,   1),IHEL=1, 5) / -1, 1, 1,-1, 1/
      DATA (NHEL(IHEL,   2),IHEL=1, 5) / -1,-1, 1,-1, 1/

        perma1(1)=iq1
        perma1(2)=i5
        perma1(3)=iQbar4
        perma1(4)=iQ3
        perma1(5)=iqbar2
        perma1(6)=i6
        perma1(7)=i7

        perma2(1)=iq1
        perma2(2)=iQbar4
        perma2(3)=iQ3
        perma2(4)=i5
        perma2(5)=iqbar2
        perma2(6)=i6
        perma2(7)=i7

        permaa1(1)=iq1
        permaa1(2)=i5
        permaa1(3)=iqbar2
        permaa1(4)=iQ3
        permaa1(5)=iQbar4
        permaa1(6)=i6
        permaa1(7)=i7

        permaa2(1)=iq1
        permaa2(2)=iqbar2
        permaa2(3)=iQ3
        permaa2(4)=i5
        permaa2(5)=iQbar4
        permaa2(6)=i6
        permaa2(7)=i7

         amp2=0d0
       do i=1,2
           if(i.eq.1) then
            zamp=real(
     .    +(zC1g0Ha1mppmp(PERMA1(1))-zC1g0Ha2mpmpp(PERMA2(1)))
     .    *dconjg(zC1g0Ha1mppmp(PERMAA1(1))-zC1g0Ha2mpmpp(PERMAA2(1)))
     .   )
           endif

           if(i.eq.2) then
            zamp=real(
     .    +(zC1g0Ha1mmpmp(PERMA1(1))-zC1g0Ha2mpmmp(PERMA2(1)))
     .    *dconjg(zC1g0Ha1mmpmp(PERMAA1(1))-zC1g0Ha2mpmmp(PERMAA2(1)))
     .    )
           endif

          amp2=amp2+4d0*zamp
       enddo

         Dt1g0H=-amp2
   10 format(" Dt1g0H   ",e12.4,6i3)
      return
      end
*****************************************************************************************
*****************************************************************************************
************************** Library for four quark tree amplitudes ***********************
*****************************************************************************************
*****************************************************************************************
      complex*16 function zC0g0HMHV(PERM,HEL)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer hel(4),perm(6)
      integer p(2),m(2),ipset
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43
      complex*16, pointer :: zamm12,zbpp12

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      l=0
      k=0
      do i=1,4
        if(HEL(i).eq.-1) then
          l=l+1
          m(l)=perm(i)
        endif
        if(HEL(i).eq.1) then
          k=k+1
          p(k)=perm(i)
        endif
      enddo

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za34 => kin(ipset)%zA(j3,j4)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za43 => kin(ipset)%zA(j4,j3)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb34 => kin(ipset)%zB(j3,j4)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb43 => kin(ipset)%zB(j4,j3)

      zamm12=>kin(ipset)%zA(m(1),m(2))
      zbpp12=>kin(ipset)%zB(p(1),p(2))

      zC0g0HMHV=zamm12**2/za23/za14+zbpp12**2/zb23/zb14
      return
      end
************************************************************************
      complex*16 function zD0g0HMHVA(PERM,HEL)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer hel(4),perm(6)
      integer p(2),m(2),ipset
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43
      complex*16, pointer :: zamm12,zbpp12

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      l=0
      k=0
      do i=1,4
        if(HEL(i).eq.-1) then
          l=l+1
          m(l)=perm(i)
        endif
        if(HEL(i).eq.1) then
          k=k+1
          p(k)=perm(i)
        endif
      enddo

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za34 => kin(ipset)%zA(j3,j4)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za43 => kin(ipset)%zA(j4,j3)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb34 => kin(ipset)%zB(j3,j4)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb43 => kin(ipset)%zB(j4,j3)

      zamm12=>kin(ipset)%zA(m(1),m(2))
      zbpp12=>kin(ipset)%zB(p(1),p(2))

      zD0g0HMHVA=zamm12**2/za23/za14+zbpp12**2/zb23/zb14

      return
      end
************************************************************************
      complex*16 function zD0g0HMHVB(PERM,HEL)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer hel(4),perm(6)
      integer p(2),m(2),ipset
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43
      complex*16, pointer :: zamm12,zbpp12

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      l=0
      k=0
      do i=1,4
        if(HEL(i).eq.-1) then
          l=l+1
          m(l)=perm(i)
        endif
        if(HEL(i).eq.1) then
          k=k+1
          p(k)=perm(i)
        endif
      enddo

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za34 => kin(ipset)%zA(j3,j4)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za43 => kin(ipset)%zA(j4,j3)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb34 => kin(ipset)%zB(j3,j4)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb43 => kin(ipset)%zB(j4,j3)

      zamm12=>kin(ipset)%zA(m(1),m(2))
      zbpp12=>kin(ipset)%zB(p(1),p(2))

      zD0g0HMHVB=zamm12**2/za12/za43+zbpp12**2/zb12/zb43

      return
      end
*****************************************************************************************
*****************************************************************************************
**************** Library for four quark and one gluon tree amplitudes********************
*****************************************************************************************
*****************************************************************************************
************************************************************************
      complex*16 function zC1g0Ha1mppmp(PERM)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer hel(5),perm(7),ipset
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s15,s23,s24,s25,s34,s35,s45
      real*8, pointer :: s21,s31,s41,s51,s32,s42,s52,s43,s53,s54
      complex*16, pointer :: za12,za13,za14,za15,za23,za24,za25,za34,za35,za45
      complex*16, pointer :: za21,za31,za41,za51,za32,za42,za52,za43,za53,za54
      complex*16, pointer :: zb12,zb13,zb14,zb15,zb23,zb24,zb25,zb34,zb35,zb45
      complex*16, pointer :: zb21,zb31,zb41,zb51,zb32,zb42,zb52,zb43,zb53,zb54

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)
      j7=perm(7)

      ipset = nspec+5

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s15 => kin(ipset)%s(j1,j5)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s25 => kin(ipset)%s(j2,j5)
      s34 => kin(ipset)%s(j3,j4)
      s35 => kin(ipset)%s(j3,j5)
      s45 => kin(ipset)%s(j4,j5)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s51 => kin(ipset)%s(j5,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s52 => kin(ipset)%s(j5,j2)
      s43 => kin(ipset)%s(j4,j3)
      s53 => kin(ipset)%s(j5,j3)
      s54 => kin(ipset)%s(j5,j4)

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za15 => kin(ipset)%zA(j1,j5)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za25 => kin(ipset)%zA(j2,j5)
      za34 => kin(ipset)%zA(j3,j4)
      za35 => kin(ipset)%zA(j3,j5)
      za45 => kin(ipset)%zA(j4,j5)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za51 => kin(ipset)%zA(j5,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za52 => kin(ipset)%zA(j5,j2)
      za43 => kin(ipset)%zA(j4,j3)
      za53 => kin(ipset)%zA(j5,j3)
      za54 => kin(ipset)%zA(j5,j4)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb15 => kin(ipset)%zB(j1,j5)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb25 => kin(ipset)%zB(j2,j5)
      zb34 => kin(ipset)%zB(j3,j4)
      zb35 => kin(ipset)%zB(j3,j5)
      zb45 => kin(ipset)%zB(j4,j5)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb51 => kin(ipset)%zB(j5,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb52 => kin(ipset)%zB(j5,j2)
      zb43 => kin(ipset)%zB(j4,j3)
      zb53 => kin(ipset)%zB(j5,j3)
      zb54 => kin(ipset)%zB(j5,j4)

      t123=s12+s13+s23
      t124=s12+s14+s24
      t134=s13+s14+s34
      t145=s14+s45+s15
      t125=s12+s25+s15
      t234=s23+s34+s24
      t345=s34+s45+s35
      t1235=s12+s13+s15+s23+s25+s35
      t1234=s12+s13+s14+s23+s24+s34
      t1245=s12+s14+s15+s24+s25+s45
      t1345=s13+s14+s15+s34+s35+s45

      s67=s12+s13+s14+s15+s23+s24+S25+s34+s35+s45

      zC1g0Ha1mppmp=
     .           -(za42*zb25+za43*zb35)**2*(za32*zb25+za34*zb45)/za23/za34/zb15/t234/(za23*zb35+za24*zb45)
*A
     .           +zb23**2*za14**2*zb15*(za14*zb42+za15*zb52)/t145/s15/(za41*zb12+za45*zb52)
     .           /(za14*zb43+za15*zb53)
*B
     .           -(za13*zb32+za14*zb42+za15*zb52)**2*za34*zb35**2*(zb23*t345-zb12*(za14*zb43+za15*zb53))
     .           /s34/t1345/t345/(za14*zb43+za15*zb53)/(zb25*t345-zb12*(za13*zb35+za14*zb45))
*C(1)
     .           +s67**2*zb25**3*za34*zb35**3/zb51/s34/(zb35*t125+zb34*(za41*zb15+za42*zb25))
     .           /(zb53*za31*zb15+zb53*za32*zb25+zb54*za41*zb15+zb54*za42*zb25)
     .           /(zb25*t345-zb12*(za13*zb35+za14*zb45))
*C(2)
     .           -(za12*zb25+za13*zb35+za14*zb45)**2*za34*zb35**2/za12/s34/(za23*zb35+za24*zb45)
     .           /(zb53*za31*zb15+zb53*za32*zb25+zb54*za41*zb15+zb54*za42*zb25)
*C(3)
     .           +zb25**3*(za41*zb13+za42*zb23+za45*zb53)**2/zb15/t125/(za41*zb12+za45*zb52)
     .           /(zb43*(za41*zb15+za42*zb25)+t125*zb53)
*D
     .           +za31*za14**2/za12/za23/za34/za51
*phi part

      return
      end
************************************************************************
      complex*16 function zC1g0Ha1mpmpp(PERM)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer hel(5),perm(7),ipset
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s15,s23,s24,s25,s34,s35,s45
      real*8, pointer :: s21,s31,s41,s51,s32,s42,s52,s43,s53,s54
      complex*16, pointer :: za12,za13,za14,za15,za23,za24,za25,za34,za35,za45
      complex*16, pointer :: za21,za31,za41,za51,za32,za42,za52,za43,za53,za54
      complex*16, pointer :: zb12,zb13,zb14,zb15,zb23,zb24,zb25,zb34,zb35,zb45
      complex*16, pointer :: zb21,zb31,zb41,zb51,zb32,zb42,zb52,zb43,zb53,zb54

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)
      j7=perm(7)

      ipset = nspec+5

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s15 => kin(ipset)%s(j1,j5)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s25 => kin(ipset)%s(j2,j5)
      s34 => kin(ipset)%s(j3,j4)
      s35 => kin(ipset)%s(j3,j5)
      s45 => kin(ipset)%s(j4,j5)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s51 => kin(ipset)%s(j5,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s52 => kin(ipset)%s(j5,j2)
      s43 => kin(ipset)%s(j4,j3)
      s53 => kin(ipset)%s(j5,j3)
      s54 => kin(ipset)%s(j5,j4)

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za15 => kin(ipset)%zA(j1,j5)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za25 => kin(ipset)%zA(j2,j5)
      za34 => kin(ipset)%zA(j3,j4)
      za35 => kin(ipset)%zA(j3,j5)
      za45 => kin(ipset)%zA(j4,j5)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za51 => kin(ipset)%zA(j5,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za52 => kin(ipset)%zA(j5,j2)
      za43 => kin(ipset)%zA(j4,j3)
      za53 => kin(ipset)%zA(j5,j3)
      za54 => kin(ipset)%zA(j5,j4)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb15 => kin(ipset)%zB(j1,j5)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb25 => kin(ipset)%zB(j2,j5)
      zb34 => kin(ipset)%zB(j3,j4)
      zb35 => kin(ipset)%zB(j3,j5)
      zb45 => kin(ipset)%zB(j4,j5)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb51 => kin(ipset)%zB(j5,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb52 => kin(ipset)%zB(j5,j2)
      zb43 => kin(ipset)%zB(j4,j3)
      zb53 => kin(ipset)%zB(j5,j3)
      zb54 => kin(ipset)%zB(j5,j4)

      t123=s12+s13+s23
      t124=s12+s14+s24
      t134=s13+s14+s34
      t145=s14+s45+s15
      t125=s12+s25+s15
      t234=s23+s34+s24
      t345=s34+s45+s35
      t1235=s12+s13+s15+s23+s25+s35
      t1234=s12+s13+s14+s23+s24+s34
      t1245=s12+s14+s15+s24+s25+s45
      t1345=s13+s14+s15+s34+s35+s45

      s67=s12+s13+s14+s15+s23+s24+S25+s34+s35+s45

      zC1g0Ha1mpmpp=
     .            -(za31*zb12+za34*zb42+za35*zb52)**2*(za41*zb12+za43*zb32+za45*zb52)*za15*zb25**3
     .            /za34/s15/(za41*zb12+za45*zb52)/(zb25*t345+zb21*(za13*zb35+za14*zb45))
     .            /(zb21*za13*zb32+zb21*za14*zb42+zb25*za53*zb32+zb25*za54*zb42)
*A(1)
     .            +s67**2*zb24**3*zb25**3*za15/zb43/s15/(t125*zb24+(zb21*za13+zb25*za53)*zb34)
     .            /(zb25*t234+zb15*(za13*zb32+za14*zb42))
     .            /(zb21*za13*zb32+zb21*za14*zb42+zb25*za53*zb32+zb25*za54*zb42)
*A(2)
     .            +(za31*zb14+za32*zb24+za35*zb54)**2*za15*zb25**3/s15/t125/(za31*zb15+za32*zb25)
     .            /(t125*zb24+(zb21*za13+zb25*za53)*zb34)
*A(3)
     .            +za34*zb42**3*(za12*zb25+za13*zb35+za14*zb45)**2/s34/t234/(za12*zb24+za13*zb34)
     .            /(zb15*(za13*zb32+za14*zb42)+t234*zb25)
*B
     .            +zb45**2*(za13*zb32+za14*zb42+za15*zb52)**2*(zb32*t345+zb12*(za14*zb43+za15*zb53))
     .            /zb34/t345/t1345/(za14*zb43+za15*zb53)/(zb52*t345+zb12*(za13*zb35+za14*zb45))
*C
     .            -(za14*zb42+za15*zb52)**3/za15/t145/(za14*zb43+za15*zb53)/(za41*zb12+za45*zb52)
*D
     .            +zb45**2*za13**3*zb23/za12/s23/(za12*zb24+za13*zb34)/(za31*zb15+za32*zb25)
*E
     .            +za13**3/za12/za23/za34/za15
*cphi part

      return
      end
************************************************************************
      complex*16 function zC1g0Ha1mmpmp(PERM)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer hel(5),perm(7),perma(7)

          perma(1)=perm(3)
          perma(2)=perm(2)
          perma(3)=perm(1)
          perma(4)=perm(5)
          perma(5)=perm(4)
          perma(6)=perm(6)
          perma(7)=perm(7)


      zC1g0Ha1mmpmp=-conjg(zC1g0Ha1mppmp(perma))

*XC related to zC1g0Ha1mppmp amplitude by new symmetry in two quark pairs.
*XC for two separated quark lines in the colour decomposed factor, we could swarp the overall line order.
      return
      end
************************************************************************
      complex*16 function zC1g0Ha1mmmpp(PERM)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer hel(5),perm(7),ipset
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s15,s23,s24,s25,s34,s35,s45
      real*8, pointer :: s21,s31,s41,s51,s32,s42,s52,s43,s53,s54
      complex*16, pointer :: za12,za13,za14,za15,za23,za24,za25,za34,za35,za45
      complex*16, pointer :: za21,za31,za41,za51,za32,za42,za52,za43,za53,za54
      complex*16, pointer :: zb12,zb13,zb14,zb15,zb23,zb24,zb25,zb34,zb35,zb45
      complex*16, pointer :: zb21,zb31,zb41,zb51,zb32,zb42,zb52,zb43,zb53,zb54

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)
      j7=perm(7)

      ipset = nspec+5

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s15 => kin(ipset)%s(j1,j5)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s25 => kin(ipset)%s(j2,j5)
      s34 => kin(ipset)%s(j3,j4)
      s35 => kin(ipset)%s(j3,j5)
      s45 => kin(ipset)%s(j4,j5)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s51 => kin(ipset)%s(j5,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s52 => kin(ipset)%s(j5,j2)
      s43 => kin(ipset)%s(j4,j3)
      s53 => kin(ipset)%s(j5,j3)
      s54 => kin(ipset)%s(j5,j4)

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za15 => kin(ipset)%zA(j1,j5)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za25 => kin(ipset)%zA(j2,j5)
      za34 => kin(ipset)%zA(j3,j4)
      za35 => kin(ipset)%zA(j3,j5)
      za45 => kin(ipset)%zA(j4,j5)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za51 => kin(ipset)%zA(j5,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za52 => kin(ipset)%zA(j5,j2)
      za43 => kin(ipset)%zA(j4,j3)
      za53 => kin(ipset)%zA(j5,j3)
      za54 => kin(ipset)%zA(j5,j4)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb15 => kin(ipset)%zB(j1,j5)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb25 => kin(ipset)%zB(j2,j5)
      zb34 => kin(ipset)%zB(j3,j4)
      zb35 => kin(ipset)%zB(j3,j5)
      zb45 => kin(ipset)%zB(j4,j5)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb51 => kin(ipset)%zB(j5,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb52 => kin(ipset)%zB(j5,j2)
      zb43 => kin(ipset)%zB(j4,j3)
      zb53 => kin(ipset)%zB(j5,j3)
      zb54 => kin(ipset)%zB(j5,j4)

      t123=s12+s13+s23
      t145=s14+s45+s15
      t125=s12+s25+s15
      t234=s23+s34+s24
      t345=s34+s45+s35
      t1235=s12+s13+s15+s23+s25+s35
      t1234=s12+s13+s14+s23+s24+s34
      t1245=s12+s14+s15+s24+s25+s45
      t1345=s13+s14+s15+s34+s35+s45

      s67=s12+s13+s14+s15+s23+s24+S25+s34+s35+s45

      zC1g0Ha1mmmpp=
     .               -za13**2*zb34*(za31*zb15+za32*zb25+za34*zb45)**2
     .               *(za32*zb21*za13+za32*zb24*za43+za35*zb51*za13+za35*zb54*za43)/s34
     .               /(za31*zb12+za34*zb42)/(za31*zb15+za34*zb45)/(za31*zb12+za34*zb42+za35*zb52)
     .               /(za13*t125-za34*(za12*zb24+za15*zb54))
*A(1)
     .               +s67**2*za13**3*zb34/za51/s34/t1345/(za13*zb32+za14*zb42+za15*zb52)
     .               /(za31*zb12+za34*zb42+za35*zb52)
*A(2)
     .               +(za12*zb25+za13*zb35+za14*zb45)**2*za13**2*zb34/s34
     .               /(za13*zb32+za14*zb42+za15*zb52)/(za13*zb32+za14*zb42)/(za13*zb35+za14*zb45)
*A(3)
     .               +za12**2*za25*(za31*zb14+za32*zb24+za35*zb54)**2*(za12*zb24+za15*zb54)
     .               /za15/t125/(za21*zb14+za25*zb54)/(za51*zb14+za52*zb24)
     .               /(za34*(za12*zb24+za15*zb54)+t125*za31)
*B
     .               -za23*(za12*zb24+za13*zb34)**2*(za12*zb23+za14*zb43)/za15/zb34/s23/t234
     .               /(za13*zb32+za14*zb42)
*C
     .               -za23**2*za15*zb45**2*(za21*zb15+za24*zb45)/s15/(za21*zb14+za25*zb54)
     .               /(za31*zb15+za34*zb45)/t145
*D
     .               +za12*(za31*zb14+za32*zb24)**2/s12/(za31*zb12+za34*zb42)/(za51*zb14+za52*zb24)
*E
     .               -za12**2*zb45**2/t345/zb34/(za13*zb35+za14*zb45)
*F
     .               -zb13*zb45**2/zb12/zb23/zb34/zb15
*cphi part

      return
      end
************************************************************************
      complex*16 function zC1g0Ha2mpmmp(PERM)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer hel(5),perm(7),perma(7)

          perma(1)=perm(5)
          perma(2)=perm(4)
          perma(3)=perm(3)
          perma(4)=perm(2)
          perma(5)=perm(1)
          perma(6)=perm(6)
          perma(7)=perm(7)

      zC1g0Ha2mpmmp=dconjg(zC1g0Ha1mppmp(PERMA))

      return
      end
************************************************************************
      complex*16 function zC1g0Ha2mmpmp(PERM)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer hel(5),perm(7),perma(7)

          perma(1)=perm(5)
          perma(2)=perm(4)
          perma(3)=perm(3)
          perma(4)=perm(2)
          perma(5)=perm(1)
          perma(6)=perm(6)
          perma(7)=perm(7)

      zi=(0d0,1d0)

      zC1g0Ha2mmpmp=-dconjg(zi*zC1g0Ha1mpmpp(PERMA))

      return
      end
************************************************************************
      complex*16 function zC1g0Ha2mpmpp(PERM)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer hel(5),perm(7),perma(7)

          perma(1)=perm(5)
          perma(2)=perm(4)
          perma(3)=perm(3)
          perma(4)=perm(2)
          perma(5)=perm(1)
          perma(6)=perm(6)
          perma(7)=perm(7)

      zC1g0Ha2mpmpp=dconjg(zC1g0Ha1mmpmp(PERMA))

      return
      end
************************************************************************
*
      complex*16 function zC1g0Ha2mmppp(PERM)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer hel(5),perm(7),perma(7)

          perma(1)=perm(5)
          perma(2)=perm(4)
          perma(3)=perm(3)
          perma(4)=perm(2)
          perma(5)=perm(1)
          perma(6)=perm(6)
          perma(7)=perm(7)
      zi=(0d0,1d0)

      zC1g0Ha2mmppp=-dconjg(zi*zC1g0Ha1mmmpp(PERMA))

      return
      end
************************************************************************
      complex*16 function zC1g0Hb1mmpmp(PERM)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer hel(5),perm(7),ipset
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s15,s23,s24,s25,s34,s35,s45
      real*8, pointer :: s21,s31,s41,s51,s32,s42,s52,s43,s53,s54
      complex*16, pointer :: za12,za13,za14,za15,za23,za24,za25,za34,za35,za45
      complex*16, pointer :: za21,za31,za41,za51,za32,za42,za52,za43,za53,za54
      complex*16, pointer :: zb12,zb13,zb14,zb15,zb23,zb24,zb25,zb34,zb35,zb45
      complex*16, pointer :: zb21,zb31,zb41,zb51,zb32,zb42,zb52,zb43,zb53,zb54

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)
      j7=perm(7)

      ipset = nspec+5

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s15 => kin(ipset)%s(j1,j5)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s25 => kin(ipset)%s(j2,j5)
      s34 => kin(ipset)%s(j3,j4)
      s35 => kin(ipset)%s(j3,j5)
      s45 => kin(ipset)%s(j4,j5)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s51 => kin(ipset)%s(j5,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s52 => kin(ipset)%s(j5,j2)
      s43 => kin(ipset)%s(j4,j3)
      s53 => kin(ipset)%s(j5,j3)
      s54 => kin(ipset)%s(j5,j4)

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za15 => kin(ipset)%zA(j1,j5)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za25 => kin(ipset)%zA(j2,j5)
      za34 => kin(ipset)%zA(j3,j4)
      za35 => kin(ipset)%zA(j3,j5)
      za45 => kin(ipset)%zA(j4,j5)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za51 => kin(ipset)%zA(j5,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za52 => kin(ipset)%zA(j5,j2)
      za43 => kin(ipset)%zA(j4,j3)
      za53 => kin(ipset)%zA(j5,j3)
      za54 => kin(ipset)%zA(j5,j4)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb15 => kin(ipset)%zB(j1,j5)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb25 => kin(ipset)%zB(j2,j5)
      zb34 => kin(ipset)%zB(j3,j4)
      zb35 => kin(ipset)%zB(j3,j5)
      zb45 => kin(ipset)%zB(j4,j5)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb51 => kin(ipset)%zB(j5,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb52 => kin(ipset)%zB(j5,j2)
      zb43 => kin(ipset)%zB(j4,j3)
      zb53 => kin(ipset)%zB(j5,j3)
      zb54 => kin(ipset)%zB(j5,j4)

      t123=s12+s13+s23
      t145=s14+s45+s15
      t125=s12+s25+s15
      t234=s23+s34+s24
      t345=s34+s45+s35
      t1235=s12+s13+s15+s23+s25+s35
      t1234=s12+s13+s14+s23+s24+s34
      t1245=s12+s14+s15+s24+s25+s45
      t1345=s13+s14+s15+s34+s35+s45

      zC1g0Hb1mmpmp=
     .            +za14**2*(za21*zb13+za24*zb43+za25*zb53)**2/za45/t145/t1345
     .            /(za14*zb43+za15*zb53)
*A
     .            -za12**2*za45*zb35**2/s45/t345/(za14*zb43+za15*zb53)
*B
     .            +za12*(za41*zb13+za42*zb23)**2/za45/zb23/s12/t123
*C
     .            -zb35**2/zb12/zb23/zb54
*cphi part

      return
      end
************************************************************************
      complex*16 function zC1g0Hb1mppmp(PERM)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer hel(5),perm(7),perma(7)

          perma(1)=perm(3)
          perma(2)=perm(2)
          perma(3)=perm(1)
          perma(4)=perm(5)
          perma(5)=perm(4)
          perma(6)=perm(6)
          perma(7)=perm(7)

      zC1g0Hb1mppmp=-dconjg(zC1g0Hb1mmpmp(PERMA))

      return
      end
************************************************************************
      complex*16 function zC1g0Hb1mpppm(PERM)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer hel(5),perm(7),perma(7)

          perma(1)=perm(3)
          perma(2)=perm(2)
          perma(3)=perm(1)
          perma(4)=perm(4)
          perma(5)=perm(5)
          perma(6)=perm(6)
          perma(7)=perm(7)

      zi=(0d0,1d0)

      zC1g0Hb1mpppm=dconjg(zi*zC1g0Hb1mmpmp(PERMA))

      return
      end
************************************************************************
      complex*16 function zC1g0Hb1mmppm(PERM)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer hel(5),perm(7),perma(7)

          perma(1)=perm(1)
          perma(2)=perm(2)
          perma(3)=perm(3)
          perma(4)=perm(5)
          perma(5)=perm(4)
          perma(6)=perm(6)
          perma(7)=perm(7)

      zi=(0d0,1d0)

      zC1g0Hb1mmppm=zi*zC1g0Hb1mmpmp(PERMA)

      return
      end
************************************************************************
      complex*16 function zC1g0Hb2mpmmp(PERM)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer hel(5),perm(7),perma(7)

          perma(1)=perm(3)
          perma(2)=perm(4)
          perma(3)=perm(5)
          perma(4)=perm(1)
          perma(5)=perm(2)
          perma(6)=perm(6)
          perma(7)=perm(7)

      zi=(0d0,1d0)

      zC1g0Hb2mpmmp=-zC1g0Hb1mmpmp(PERMA)

      return
      end
************************************************************************
      complex*16 function zC1g0Hb2mppmm(PERM)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer hel(5),perm(7),perma(7)

          perma(1)=perm(5)
          perma(2)=perm(4)
          perma(3)=perm(3)
          perma(4)=perm(1)
          perma(5)=perm(2)
          perma(6)=perm(6)
          perma(7)=perm(7)

      zi=(0d0,1d0)

      zC1g0Hb2mppmm=zi*zC1g0Hb1mmpmp(PERMA)

      return
      end
************************************************************************
      complex*16 function zC1g0Hb2mpppm(PERM)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer hel(5),perm(7),perma(7)

          perma(1)=perm(3)
          perma(2)=perm(4)
          perma(3)=perm(5)
          perma(4)=perm(2)
          perma(5)=perm(1)
          perma(6)=perm(6)
          perma(7)=perm(7)

      zi=(0d0,1d0)

      zC1g0Hb2mpppm=dconjg(zi*zC1g0Hb1mmpmp(PERMA))

      return
      end
************************************************************************
      complex*16 function zC1g0Hb2mpmpp(PERM)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer hel(5),perm(7),perma(7)

          perma(1)=perm(5)
          perma(2)=perm(4)
          perma(3)=perm(3)
          perma(4)=perm(2)
          perma(5)=perm(1)
          perma(6)=perm(6)
          perma(7)=perm(7)

      zC1g0Hb2mpmpp=dconjg(zC1g0Hb1mmpmp(PERMA))

      return
      end
************************************************************************
      complex*16 function zC1g0Ha2mppmm(PERM)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer hel(5),perm(7),perma(7)

          perma(1)=perm(2)
          perma(2)=perm(4)
          perma(3)=perm(3)
          perma(4)=perm(5)
          perma(5)=perm(1)
          perma(6)=perm(6)
          perma(7)=perm(7)

      zi=(0d0,1d0)

      zC1g0Ha2mppmm=-conjg(zi*zC1g0Ha1mpmpp(PERMA))

      return
      end
************************************************************************
      complex*16 function zC1g0Ha1mpppm(PERM)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer hel(5),perm(7),perma(7)

          perma(1)=perm(1)
          perma(2)=perm(2)
          perma(3)=perm(5)
          perma(4)=perm(4)
          perma(5)=perm(3)
          perma(6)=perm(6)
          perma(7)=perm(7)

      zi=(0d0,1d0)

      zC1g0Ha1mpppm=zi*zC1g0Ha1mpmpp(PERMA)

      return
      end
************************************************************************
*
      complex*16 function zC1g0Ha1mmppm(PERM)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer hel(5),perm(7),perma(7)

          perma(1)=perm(1)
          perma(2)=perm(2)
          perma(3)=perm(5)
          perma(4)=perm(4)
          perma(5)=perm(3)
          perma(6)=perm(6)
          perma(7)=perm(7)
      zi=(0d0,1d0)

      zC1g0Ha1mmppm=zi*zC1g0Ha1mmmpp(PERMA)

      return
      end
************************************************************************
      complex*16 function zC1g0Ha2mpppm(PERM)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer hel(5),perm(7),perma(7)

          perma(1)=perm(2)
          perma(2)=perm(4)
          perma(3)=perm(3)
          perma(4)=perm(5)
          perma(5)=perm(1)
          perma(6)=perm(6)
          perma(7)=perm(7)
      zi=(0d0,1d0)

      zC1g0Ha2mpppm=-conjg(zi*zC1g0Ha1mmmpp(PERMA))

      return
      end
************************************************************************
      complex*16 function zC1g0Hb1mmmpp(PERM)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer hel(5),perm(7),perma(7)

          perma(1)=perm(1)
          perma(2)=perm(2)
          perma(3)=perm(5)
          perma(4)=perm(3)
          perma(5)=perm(4)
          perma(6)=perm(6)
          perma(7)=perm(7)

      zi=(0d0,1d0)

      zC1g0Hb1mmmpp=zi*zC1g0Hb1mmpmp(PERMA)

      return
      end
************************************************************************
      complex*16 function zC1g0Hb1mpmpp(PERM)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer hel(5),perm(7),perma(7)

          perma(1)=perm(5)
          perma(2)=perm(2)
          perma(3)=perm(1)
          perma(4)=perm(4)
          perma(5)=perm(3)
          perma(6)=perm(6)
          perma(7)=perm(7)

      zi=(0d0,1d0)

      zC1g0Hb1mpmpp=conjg(zi*zC1g0Hb1mmpmp(PERMA))

      return
      end

************************************************************************
      complex*16 function zC1g0Hb2mmpmp(PERM)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer hel(5),perm(7),perma(7)

          perma(1)=perm(2)
          perma(2)=perm(4)
          perma(3)=perm(3)
          perma(4)=perm(1)
          perma(5)=perm(5)
          perma(6)=perm(6)
          perma(7)=perm(7)

      zi=(0d0,1d0)

      zC1g0Hb2mmpmp=-zi*zC1g0Hb1mmpmp(PERMA)

      return
      end
************************************************************************
      complex*16 function zC1g0Hb2mmppp(PERM)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer hel(5),perm(7),perma(7)

          perma(1)=perm(3)
          perma(2)=perm(4)
          perma(3)=perm(2)
          perma(4)=perm(5)
          perma(5)=perm(1)
          perma(6)=perm(6)
          perma(7)=perm(7)

      zi=(0d0,1d0)

      zC1g0Hb2mmppp=-conjg(zi*zC1g0Hb1mmpmp(PERMA))

      return
      end


