************************************************************************
!XC 1-loop amplitudes based on MCFM-6.8 source code 
!  J.~M.~Campbell and R.~K.~Ellis,
!  %``An Update on vector boson pair production at hadron colliders,''
!  Phys.\ Rev.\ D {\bf 60} (1999) 113006
!  doi:10.1103/PhysRevD.60.113006
!  [hep-ph/9905386].

!  J.~M.~Campbell, R.~K.~Ellis and W.~T.~Giele,
!  %``A Multi-Threaded Version of MCFM,''
!  Eur.\ Phys.\ J.\ C {\bf 75} (2015) no.6,  246
!  doi:10.1140/epjc/s10052-015-3461-2
!  [arXiv:1503.06182 [physics.comp-ph]].

!XC The corresponding reference for 1-loop amplitudes are 
!  S.~Badger, J.~M.~Campbell, R.~K.~Ellis and C.~Williams,
!  %``Analytic results for the one-loop NMHV Hqqgg amplitude,''
!  JHEP {\bf 0912} (2009) 035
!  doi:10.1088/1126-6708/2009/12/035
!  [arXiv:0910.4481 [hep-ph]].

!  S.~Badger, E.~W.~Nigel Glover, P.~Mastrolia and C.~Williams,
!  %``One-loop Higgs plus four gluon amplitudes: Full analytic results,''
!  JHEP {\bf 1001} (2010) 036
!  doi:10.1007/JHEP01(2010)036
!  [arXiv:0909.4475 [hep-ph]].

!  L.~J.~Dixon and Y.~Sofianatos,
!  %``Analytic one-loop amplitudes for a Higgs boson plus four partons,''
!  JHEP {\bf 0908} (2009) 058
!  doi:10.1088/1126-6708/2009/08/058
!  [arXiv:0906.0008 [hep-ph]].

!  E.~W.~N.~Glover, P.~Mastrolia and C.~Williams,
!  %``One-loop phi-MHV amplitudes using the unitarity bootstrap: The General helicity case,''
!  JHEP {\bf 0808} (2008) 017
!  doi:10.1088/1126-6708/2008/08/017
!  [arXiv:0804.4149 [hep-ph]].

!  S.~D.~Badger, E.~W.~N.~Glover and K.~Risager,
!  %``One-loop phi-MHV amplitudes using the unitarity bootstrap,''
!  JHEP {\bf 0707} (2007) 066
!  doi:10.1088/1126-6708/2007/07/066
!  [arXiv:0704.3914 [hep-ph]].

!XC To improve numerical stability, more details in this code are explained in
!  X.~Chen,
!  %``Hadronic Production of a Higgs Boson in Association with a Jet at Next-to-Next-to-Leading Order,''
!  http://etheses.dur.ac.uk/11158/1/PhDthesisXuanChen.pdf 
************************************************************************
************************************************************************
*XC two q qbar pairs with non-identical flavor and one higgs one-loop level matrix elements
*XC summed over all colour orderings, summed and squared
*
      function FullC0g1H(iq1,iQbar3,iQ4,iqbar2,i5,i6,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /order/ieorder

*XC (as/3d0/pi/239d0)**2=as/3d0*pi/246d0 for gluon gluon fusion to higgs effective
*XC coupling constant
      stop "set value for chgg correctly!"
      fac=(4d0*pi*as)**3*(nc**2-1d0)*nc*chgg**2/4d0

      FullC0g1H=
     .            +C0g1H(iq1,iQbar3,iQ4,iqbar2,i5,i6,renscale2)
     .            -Ct0g1H(iq1,iQbar3,iQ4,iqbar2,i5,i6,renscale2)/nc**2
     .            +Ch0g1H(iq1,iQbar3,iQ4,iqbar2,i5,i6,renscale2)*nf/nc

      FullC0g1H=fac*FullC0g1H

      return
      end
************************************************************************
*XC two q qbar pairs with identical flavor and one higgs one-loop level matrix elements
*XC summed over all colour orderings, summed and squared
*
      function FullD0g1H(iq1,iQbar3,iQ4,iqbar2,i5,i6,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /order/ieorder
*XC (as/3d0/pi/239d0)**2=as/3d0*pi/246d0 for gluon gluon fusion to higgs effective
*XC coupling constant
      stop "set value for chgg correctly!"
      fac=(4d0*pi*as)**3*(nc**2-1d0)*nc*chgg**2/4d0

      FullD0g1H=
     .            +C0g1H(iq1,iQbar3,iQ4,iqbar2,i5,i6,renscale2)
     .            -Ct0g1H(iq1,iQbar3,iQ4,iqbar2,i5,i6,renscale2)/nc**2
     .            +Ch0g1H(iq1,iQbar3,iQ4,iqbar2,i5,i6,renscale2)*nf/nc

     .            +C0g1H(iq1,iqbar2,iQ4,iQbar3,i5,i6,renscale2)
     .            -Ct0g1H(iq1,iqbar2,iQ4,iQbar3,i5,i6,renscale2)/nc**2
     .            +Ch0g1H(iq1,iqbar2,iQ4,iQbar3,i5,i6,renscale2)*nf/nc

     .   -1d0/nc*(+D0g1H(iq1,iqbar2,iQ4,iQbar3,i5,i6,renscale2)
     .            +Dt0g1H(iq1,iqbar2,iQ4,iQbar3,i5,i6,renscale2)/nc**2
     .            -Dh0g1H(iq1,iqbar2,iQ4,iQbar3,i5,i6,renscale2)*nf/nc
     .           )


      FullD0g1H=fac*FullD0g1H

      return
      end

************************************************************************

      function C0g1HF(iq1,iQbar3,iQ4,iqbar2,i5,i6,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)

      if(iq1.ne.1 .and. iq1 .ne.2 .and. iqbar2.ne.1 .and. iqbar2 .ne.2) then
      C0g1HF=1d0/2d0*(
     .            +C0g1H(iq1,iQbar3,iQ4,iqbar2,i5,i6,renscale2)
     .            +C0g1H(iqbar2,iQbar3,iQ4,iq1,i5,i6,renscale2)
     .               )
      elseif(iQbar3.ne.1 .and. iQbar3 .ne.2 .and. iQ4.ne.1 .and. iQ4 .ne.2) then
      C0g1HF=1d0/2d0*(
     .            +C0g1H(iq1,iQbar3,iQ4,iqbar2,i5,i6,renscale2)
     .            +C0g1H(iq1,iQ4,iQbar3,iqbar2,i5,i6,renscale2)
     .               )
      else
      C0g1HF=
     .            +C0g1H(iq1,iQbar3,iQ4,iqbar2,i5,i6,renscale2)
      endif

      return
      end
************************************************************************
      double precision function C0g1H(i1,i3,i4,i2,i5,i6,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
C-----Author: Keith Ellis
C-----July  2005
C----Matrix element squared for the virtual corrections to the process
C----q(p1)+r(p3) -> q(p2)+r(p4)+Higgs
C----summed over incoming/outgoing colors and spins.
C    with a factor of gsq*ason2pi*Asq removed.
      common /order/ieorder
      parameter(pi=3.141592653589793238d0)
      double complex L0,L1,Lsm1,Lsm1_2me,lnrat
      double precision s123,s124,s134,s234
      double precision fun1C,fun2C,fun3C,fun4C,fun5C,fun6C,fun7C,
     . fun8C,fun9C,fun10C,fun11C,fun12C
      double precision Hqarb,Hqarbsq,mhsq,musq
      integer i1,i2,i3,i4,j1,j2,j3,j4
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43

      j1=i3
      j2=i4
      j3=i2
      j4=i1

      musq=renscale2
      epinv=1d0

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      s123=s12+s13+s23
      s124=s12+s14+s24
      s234=s23+s24+s34
      s134=s13+s14+s34

      mhsq=s12+s13+s14+s23+s24+s34

      Hqarb=C0g0H(j4,j1,j2,j3,i5,i6)/4d0
      if(ieorder.eq.-2)then
          Hqarbvsq=epinv**2*(-2.D0)*Hqarb
      elseif(ieorder.eq.-1)then
          Hqarbvsq=+epinv * (
     &     - 3.D0
     &     + dble(lnrat(-s14,musq))
     &     + dble(lnrat(-s23,musq)))*Hqarb
      elseif(ieorder.eq.0)then
         Hqarbvsq =
     &     + 80.D0/9.D0
     &     - dble(Lsm1_2me(s123,s234,s23,mhsq))
     &     - dble(Lsm1_2me(s124,s134,s14,mhsq))
     &     - 13.D0/6.D0*dble(lnrat(-s34,musq))

      Hqarbvsq = Hqarbvsq
     &     - 13.D0/6.D0*dble(lnrat(-s12,musq))
     &     - 1.D0/2.D0*dble(lnrat(-s14,musq)**2)
     &     - 1.D0/2.D0*dble(lnrat(-s23,musq)**2)
     &

      Hqarbvsq=Hqarb*Hqarbvsq
C---end of terms proportional to lowest order

      Hqarbvsq=Hqarbvsq
     .  +1d0/2d0
     . *(Dble(Lsm1(-s12,-s123,-s23,-s123))*fun1C(j1,j2,j3,j4)
     .  +Dble(Lsm1(-s12,-s124,-s14,-s124))*fun1C(j2,j1,j4,j3)
     .  +Dble(Lsm1(-s34,-s234,-s23,-s234))*fun1C(j2,j1,j4,j3)
     .  +Dble(Lsm1(-s34,-s134,-s14,-s134))*fun1C(j1,j2,j3,j4))

      Hqarbvsq=Hqarbvsq
     . +fun2C(j1,j2,j3,j4)*dble(L1(-s123,-s12))*0.5d0
     . +fun2C(j2,j1,j4,j3)*dble(L1(-s124,-s12))*0.5d0
     . +fun2C(j4,j3,j2,j1)*dble(L1(-s234,-s34))*0.5d0
     . +fun2C(j3,j4,j1,j2)*dble(L1(-s134,-s34))*0.5d0

     . +fun3C(j1,j2,j3,j4)*dble(L0(-s123,-s12))*0.5d0
     . +fun3C(j2,j1,j4,j3)*dble(L0(-s124,-s12))*0.5d0
     . +fun3C(j4,j3,j2,j1)*dble(L0(-s234,-s34))*0.5d0
     . +fun3C(j3,j4,j1,j2)*dble(L0(-s134,-s34))*0.5d0

     . +fun5C(j1,j2,j3,j4)*dble(L0(-s124,-s14))*0.5d0*(-1d0/2d0)
     . +fun5C(j2,j1,j4,j3)*dble(L0(-s123,-s23))*0.5d0*(-1d0/2d0)
     . +fun5C(j4,j3,j2,j1)*dble(L0(-s134,-s14))*0.5d0*(-1d0/2d0)
     . +fun5C(j3,j4,j1,j2)*dble(L0(-s234,-s23))*0.5d0*(-1d0/2d0)

      Hqarbvsq=Hqarbvsq
     . +0.5d0*dble(lnrat(-s123,-s12))*fun6C(j1,j2,j3,j4)
     . +0.5d0*dble(lnrat(-s124,-s12))*fun6C(j2,j1,j4,j3)
     . +0.5d0*dble(lnrat(-s234,-s34))*fun6C(j4,j3,j2,j1)
     . +0.5d0*dble(lnrat(-s134,-s34))*fun6C(j3,j4,j1,j2)

     . +0.5d0*dble(lnrat(-s123,-s23))*fun7C(j1,j2,j3,j4)
     . +0.5d0*dble(lnrat(-s124,-s14))*fun7C(j2,j1,j4,j3)
     . +0.5d0*dble(lnrat(-s234,-s23))*fun7C(j4,j3,j2,j1)
     . +0.5d0*dble(lnrat(-s134,-s14))*fun7C(j3,j4,j1,j2)

     . +0.5d0*dble(lnrat(-s12,-s14))*fun8C(j1,j2,j3,j4)
     . +0.5d0*dble(lnrat(-s12,-s14))*fun8C(j2,j1,j4,j3)
     . +0.5d0*dble(lnrat(-s34,-s23))*fun8C(j4,j3,j2,j1)
     . +0.5d0*dble(lnrat(-s34,-s23))*fun8C(j3,j4,j1,j2)

      Hqarbvsq=Hqarbvsq

     . +fun12C(j1,j2,j3,j4)*0.5d0
     . +fun12C(j2,j1,j4,j3)*0.5d0
     . +fun12C(j4,j3,j2,j1)*0.5d0
     . +fun12C(j3,j4,j1,j2)*0.5d0

      endif

      if(ieorder.eq.0)then
        ren=2d0*pi**2/12d0
      else
        ren=0d0
      endif

       C0g1H=4d0*Hqarbvsq+C0g0H(i1,i3,i4,i2,i5,i6)*ren
*XC c_gamma fix up
      return
      end

************************************************************************
      double precision function Ct0g1Havg(i1,i3,i4,i2,i5,i6,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)

       Ct0g1Havg=1d0/2d0*(Ct0g1H(i1,i3,i4,i2,i5,i6,renscale2)
     .                   +Ct0g1H(i1,i4,i3,i2,i5,i6,renscale2))

      return
      end

************************************************************************

      function Ct0g1HF(iq1,iQbar3,iQ4,iqbar2,i5,i6,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)

      if(iq1.ne.1 .and. iq1 .ne.2 .and. iqbar2.ne.1 .and. iqbar2 .ne.2) then
      Ct0g1HF=1d0/2d0*(
     .            +Ct0g1H(iq1,iQbar3,iQ4,iqbar2,i5,i6,renscale2)
     .            +Ct0g1H(iqbar2,iQbar3,iQ4,iq1,i5,i6,renscale2)
     .               )
      elseif(iQbar3.ne.1 .and. iQbar3 .ne.2 .and. iQ4.ne.1 .and. iQ4 .ne.2) then
      Ct0g1HF=1d0/2d0*(
     .            +Ct0g1H(iq1,iQbar3,iQ4,iqbar2,i5,i6,renscale2)
     .            +Ct0g1H(iq1,iQ4,iQbar3,iqbar2,i5,i6,renscale2)
     .               )
      else
      Ct0g1HF=
     .            +Ct0g1H(iq1,iQbar3,iQ4,iqbar2,i5,i6,renscale2)
      endif

      return
      end

************************************************************************
      double precision function Ct0g1H(i1,i3,i4,i2,i5,i6,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
C-----Author: Keith Ellis
C-----July  2005
C----Matrix element squared for the virtual corrections to the process
C----q(p1)+r(p3) -> q(p2)+r(p4)+Higgs
C----summed over incoming/outgoing colors and spins.
C    with a factor of gsq*ason2pi*Asq removed.
      common /order/ieorder
      parameter(pi=3.141592653589793238d0)
      double complex L0,L1,Lsm1,Lsm1_2me,lnrat
      double precision s123,s124,s134,s234
      double precision fun1C,fun2C,fun3C,fun4C,fun5C,fun6C,fun7C,
     . fun8C,fun9C,fun10C,fun11C,fun12C
      double precision Hqarb,Hqarbsq,mhsq,musq
      integer i1,i2,i3,i4,j1,j2,j3,j4
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43

      j1=i3
      j2=i4
      j3=i2
      j4=i1

      musq=renscale2
      epinv=1d0

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      s123=s12+s13+s23
      s124=s12+s14+s24
      s234=s23+s24+s34
      s134=s13+s14+s34

      mhsq=s12+s13+s14+s23+s24+s34

      Hqarb=C0g0H(j4,j1,j2,j3,i5,i6)/4d0

      if(ieorder.eq.-2)then

          Hqarbvsq=epinv**2*(2.D0)*Hqarb

      elseif(ieorder.eq.-1)then

          Hqarbvsq=+epinv * (
     &     + 3.D0
     &     - dble(lnrat(-s34,musq))
     &     - dble(lnrat(-s12,musq))
     &     + 2.D0*dble(lnrat(-s13,musq))
     &     + 2.D0*dble(lnrat(-s24,musq))
     &     - 2.D0*dble(lnrat(-s14,musq))
     &     - 2.D0*dble(lnrat(-s23,musq)))*Hqarb

      elseif(ieorder.eq.0)then

      Hqarbvsq =
     &     + 8.D0
     &     + dble(Lsm1_2me(s134,s234,s34,mhsq))
     &     - 2.D0*dble(Lsm1_2me(s123,s134,s13,mhsq))
     &     + 2.D0*dble(Lsm1_2me(s123,s234,s23,mhsq))
     &     + dble(Lsm1_2me(s123,s124,s12,mhsq))
     &     + 2.D0*dble(Lsm1_2me(s124,s134,s14,mhsq))
     &     - 2.D0*dble(Lsm1_2me(s124,s234,s24,mhsq))
     &     - 3.D0/2.D0*dble(lnrat(-s34,musq))
     &

      Hqarbvsq = Hqarbvsq
     &     - 3.D0/2.D0*dble(lnrat(-s12,musq))
     &     - dble(lnrat(-s13,musq)**2)
     &     - dble(lnrat(-s24,musq)**2)
     &     + dble(lnrat(-s14,musq)**2)
     &     + dble(lnrat(-s23,musq)**2)
     &     + 1.D0/2.D0*dble(lnrat(-s34,musq)**2)
     &     + 1.D0/2.D0*dble(lnrat(-s12,musq)**2)
     &

      Hqarbvsq=Hqarb*Hqarbvsq

C---end of terms proportional to lowest order

      Hqarbvsq=Hqarbvsq+(
     . +Dble(Lsm1(-s12,-s123,-s13,-s123))*fun1C(j2,j1,j3,j4)
     . +Dble(Lsm1(-s12,-s124,-s24,-s124))*fun1C(j1,j2,j4,j3)
     . +Dble(Lsm1(-s34,-s234,-s24,-s234))*fun1C(j2,j1,j3,j4)
     . +Dble(Lsm1(-s34,-s134,-s13,-s134))*fun1C(j1,j2,j4,j3))
     . -(Dble(Lsm1(-s12,-s123,-s23,-s123))*fun1C(j1,j2,j3,j4)
     .  +Dble(Lsm1(-s12,-s124,-s14,-s124))*fun1C(j2,j1,j4,j3)
     .  +Dble(Lsm1(-s34,-s234,-s23,-s234))*fun1C(j2,j1,j4,j3)
     .  +Dble(Lsm1(-s34,-s134,-s14,-s134))*fun1C(j1,j2,j3,j4))

      Hqarbvsq=Hqarbvsq
     . +fun2C(j1,j2,j3,j4)*dble(L1(-s123,-s12))*0.5d0
     . +fun2C(j2,j1,j4,j3)*dble(L1(-s124,-s12))*0.5d0
     . +fun2C(j4,j3,j2,j1)*dble(L1(-s234,-s34))*0.5d0
     . +fun2C(j3,j4,j1,j2)*dble(L1(-s134,-s34))*0.5d0

     . +fun4C(j1,j2,j3,j4)*dble(L0(-s123,-s12))*0.5d0
     . +fun4C(j2,j1,j4,j3)*dble(L0(-s124,-s12))*0.5d0
     . +fun4C(j4,j3,j2,j1)*dble(L0(-s234,-s34))*0.5d0
     . +fun4C(j3,j4,j1,j2)*dble(L0(-s134,-s34))*0.5d0

     . +fun5C(j1,j2,j3,j4)*dble(L0(-s124,-s14))*0.5d0
     . +fun5C(j2,j1,j4,j3)*dble(L0(-s123,-s23))*0.5d0
     . +fun5C(j4,j3,j2,j1)*dble(L0(-s134,-s14))*0.5d0
     . +fun5C(j3,j4,j1,j2)*dble(L0(-s234,-s23))*0.5d0

     . -fun5C(j1,j2,j4,j3)*dble(L0(-s123,-s13))*0.5d0
     . -fun5C(j2,j1,j3,j4)*dble(L0(-s124,-s24))*0.5d0
     . -fun5C(j4,j3,j1,j2)*dble(L0(-s234,-s24))*0.5d0
     . -fun5C(j3,j4,j2,j1)*dble(L0(-s134,-s13))*0.5d0


      Hqarbvsq=Hqarbvsq
     . +0.5d0*dble(lnrat(-s123,-s12))*fun9C(j1,j2,j3,j4)
     . +0.5d0*dble(lnrat(-s124,-s12))*fun9C(j2,j1,j4,j3)
     . +0.5d0*dble(lnrat(-s234,-s34))*fun9C(j4,j3,j2,j1)
     . +0.5d0*dble(lnrat(-s134,-s34))*fun9C(j3,j4,j1,j2)

     . +0.5d0*dble(lnrat(-s123,-s13))*fun10C(j1,j2,j3,j4)
     . +0.5d0*dble(lnrat(-s124,-s24))*fun10C(j2,j1,j4,j3)
     . +0.5d0*dble(lnrat(-s234,-s24))*fun10C(j4,j3,j2,j1)
     . +0.5d0*dble(lnrat(-s134,-s13))*fun10C(j3,j4,j1,j2)

     . +0.5d0*dble(lnrat(-s12,-s13))*fun11C(j1,j2,j3,j4)
     . +0.5d0*dble(lnrat(-s12,-s23))*fun11C(j2,j1,j4,j3)
     . +0.5d0*dble(lnrat(-s34,-s24))*fun11C(j4,j3,j2,j1)
     . +0.5d0*dble(lnrat(-s34,-s14))*fun11C(j3,j4,j1,j2)


      Hqarbvsq=Hqarbvsq

     . -0.5d0*dble(lnrat(-s123,-s23))*fun10C(j2,j1,j3,j4)
     . -0.5d0*dble(lnrat(-s124,-s14))*fun10C(j1,j2,j4,j3)
     . -0.5d0*dble(lnrat(-s234,-s23))*fun10C(j3,j4,j2,j1)
     . -0.5d0*dble(lnrat(-s134,-s14))*fun10C(j4,j3,j1,j2)

     . -0.5d0*dble(lnrat(-s12,-s13))*fun11C(j2,j1,j3,j4)
     . -0.5d0*dble(lnrat(-s12,-s14))*fun11C(j1,j2,j4,j3)
     . -0.5d0*dble(lnrat(-s34,-s23))*fun11C(j3,j4,j2,j1)
     . -0.5d0*dble(lnrat(-s34,-s24))*fun11C(j4,j3,j1,j2)


     . +fun12C(j1,j2,j3,j4)*0.5d0
     . +fun12C(j2,j1,j4,j3)*0.5d0
     . +fun12C(j4,j3,j2,j1)*0.5d0
     . +fun12C(j3,j4,j1,j2)*0.5d0

      endif

      if(ieorder.eq.0)then
        ren=2d0*pi**2/12d0
      else
        ren=0d0
      endif

       Ct0g1H=-4d0*Hqarbvsq+C0g0H(i1,i3,i4,i2,i5,i6)*ren
*XC c_gamma fix up
      return
      end
************************************************************************
      double precision function Ch0g1H(i1,i3,i4,i2,i5,i6,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
C-----Author: Keith Ellis
C-----July  2005
C----Matrix element squared for the virtual corrections to the process
C----q(p1)+r(p3) -> q(p2)+r(p4)+Higgs
C----summed over incoming/outgoing colors and spins.
C    with a factor of gsq*ason2pi*Asq removed.
      common /order/ieorder
      double complex L0,L1,Lsm1,Lsm1_2me,lnrat
      double precision s123,s124,s134,s234
      double precision fun1C,fun2C,fun3C,fun4C,fun5C,fun6C,fun7C,
     . fun8C,fun9C,fun10C,fun11C,fun12C
      double precision Hqarb,Hqarbsq,mhsq,musq
      integer i1,i2,i3,i4,j1,j2,j3,j4
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43

      j1=i3
      j2=i4
      j3=i2
      j4=i1

      musq=renscale2
      epinv=1d0

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      s123=s12+s13+s23
      s124=s12+s14+s24
      s234=s23+s24+s34
      s134=s13+s14+s34

      mhsq=s12+s13+s14+s23+s24+s34

      Hqarb=C0g0H(j4,j1,j2,j3,i5,i6)/4d0
      if(ieorder.eq.-2)then

          Hqarbvsq=0d0

      elseif(ieorder.eq.-1)then

          Hqarbvsq=0d0

      elseif(ieorder.eq.0)then

      Hqarbvsqnf =
     &     - 20.D0/9.D0
     &     + 2.D0/3.D0*dble(lnrat( - s34,musq))
     &     + 2.D0/3.D0*dble(lnrat(-s12,musq))
     &

      Hqarbvsqnf=Hqarb*Hqarbvsqnf

      Ch0g1H=4d0*Hqarbvsqnf

      endif

      return
      end

************************************************************************
      double precision function D0g1H(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)
C     Amplitude squared with certain factors removed
C     for the process H --> q(p1)+a(p2)+q(p3)+a(p4)
      common /order/ieorder
      parameter(pi=3.141592653589793238d0)
      double complex L0,L1,Lsm1,Lsm1_2me,lnrat
      double precision s123,s124,s134,s234
      double precision fun1D,fun2D,fun3D,fun4D,fun5D,fun6D,fun7D,fun8D
      double precision Hqaqa,Hqaqasq,mhsq,musq
      integer i1,i2,i3,i4
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43

      j1=i1
      j2=i2
      j3=i3
      j4=i4

      musq=renscale2
      epinv=1d0

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      s123=s12+s13+s23
      s124=s12+s14+s24
      s234=s23+s24+s34
      s134=s13+s14+s34

      mhsq=s12+s13+s14+s23+s24+s34

      Hqaqa=-D0g0H(i1,i2,i3,i4,i5,i6)/4d0

      if(ieorder.eq.-2)then
          Hqaqavsq = + epinv**2*(-2.D0)*Hqaqa
      elseif(ieorder.eq.-1)then
          Hqaqavsq =  + epinv * (
     &     - 3.D0
     &     + dble(lnrat( - s24,musq))
     &     + dble(lnrat( - s13,musq))
     &     )*Hqaqa
      elseif(ieorder.eq.0)then
      Hqaqavsq =
     &     + 80.D0/9.D0
     &     - dble(Lsm1_2me(s123,s134,s13,mhsq))
     &     - dble(Lsm1_2me(s124,s234,s24,mhsq))
     &     - 13.D0/12.D0*dble(lnrat( - s14,musq))
     &
      Hqaqavsq = Hqaqavsq
     &     - 13.D0/12.D0*dble(lnrat( - s23,musq))
     &     - 13.D0/12.D0*dble(lnrat( - s12,musq))
     &     - 13.D0/12.D0*dble(lnrat( - s34,musq))
     &     - 1.D0/2.D0*dble(lnrat( - s24,musq)**2)
     &     - 1.D0/2.D0*dble(lnrat( - s13,musq)**2)

      Hqaqavsq =Hqaqavsq*Hqaqa

      Hqaqavsq = Hqaqavsq+(
     & -dble(Lsm1(-s12,-s124,-s24,-s124))*fun1D(s12,s13,s14,s23,s24,s34)
     & -dble(Lsm1(-s23,-s234,-s24,-s234))*fun1D(s23,s13,s34,s12,s24,s14)
     & -dble(Lsm1(-s14,-s124,-s24,-s124))*fun1D(s14,s13,s12,s34,s24,s23)
     & -dble(Lsm1(-s34,-s234,-s24,-s234))*fun1D(s34,s13,s23,s14,s24,s12)
     & -dble(Lsm1(-s34,-s134,-s13,-s134))*fun1D(s12,s13,s14,s23,s24,s34)
     & -dble(Lsm1(-s14,-s134,-s13,-s134))*fun1D(s23,s13,s34,s12,s24,s14)
     & -dble(Lsm1(-s12,-s123,-s13,-s123))*fun1D(s14,s13,s12,s34,s24,s23)
     & -dble(Lsm1(-s23,-s123,-s13,-s123))*fun1D(s34,s13,s23,s14,s24,s12))

      Hqaqavsq = Hqaqavsq+(
     & + dble(L1(-s134,-s34))*fun3D(s34,s13,s23,s14,s24,s12)
     & + dble(L1(-s134,-s14))*fun3D(s14,s13,s12,s34,s24,s23)
     & + dble(L1(-s234,-s34))*fun3D(s34,s24,s14,s23,s13,s12)
     & + dble(L1(-s124,-s14))*fun3D(s14,s24,s34,s12,s13,s23)
     & + dble(L1(-s123,-s12))*fun3D(s12,s13,s14,s23,s24,s34)
     & + dble(L1(-s123,-s23))*fun3D(s23,s13,s34,s12,s24,s14)
     & + dble(L1(-s124,-s12))*fun3D(s12,s24,s23,s14,s13,s34)
     & + dble(L1(-s234,-s23))*fun3D(s23,s24,s12,s34,s13,s14))

      Hqaqavsq = Hqaqavsq+(
     & + dble(L0(-s134,-s34))*fun4D(s34,s13,s23,s14,s24,s12)
     & + dble(L0(-s134,-s14))*fun4D(s14,s13,s12,s34,s24,s23)
     & + dble(L0(-s234,-s34))*fun4D(s34,s24,s14,s23,s13,s12)
     & + dble(L0(-s124,-s14))*fun4D(s14,s24,s34,s12,s13,s23)
     & + dble(L0(-s123,-s12))*fun4D(s12,s13,s14,s23,s24,s34)
     & + dble(L0(-s123,-s23))*fun4D(s23,s13,s34,s12,s24,s14)
     & + dble(L0(-s124,-s12))*fun4D(s12,s24,s23,s14,s13,s34)
     & + dble(L0(-s234,-s23))*fun4D(s23,s24,s12,s34,s13,s14))

      Hqaqavsq = Hqaqavsq+(
     & +dble(lnrat(-s134,-s34))*fun6D(s34,s13,s23,s14,s24,s12)
     & +dble(lnrat(-s134,-s14))*fun6D(s14,s13,s12,s34,s24,s23)
     & +dble(lnrat(-s234,-s34))*fun6D(s34,s24,s14,s23,s13,s12)
     & +dble(lnrat(-s124,-s14))*fun6D(s14,s24,s34,s12,s13,s23)
     & +dble(lnrat(-s123,-s12))*fun6D(s12,s13,s14,s23,s24,s34)
     & +dble(lnrat(-s123,-s23))*fun6D(s23,s13,s34,s12,s24,s14)
     & +dble(lnrat(-s124,-s12))*fun6D(s12,s24,s23,s14,s13,s34)
     & +dble(lnrat(-s234,-s23))*fun6D(s23,s24,s12,s34,s13,s14))

      Hqaqavsq = Hqaqavsq+(
     & +fun8D(s34,s13,s23,s14,s24,s12)
     & +fun8D(s14,s13,s12,s34,s24,s23)
     & +fun8D(s34,s24,s14,s23,s13,s12)
     & +fun8D(s14,s24,s34,s12,s13,s23)
     & +fun8D(s12,s13,s14,s23,s24,s34)
     & +fun8D(s23,s13,s34,s12,s24,s14)
     & +fun8D(s12,s24,s23,s14,s13,s34)
     & +fun8D(s23,s24,s12,s34,s13,s14))

      endif

      if(ieorder.eq.0)then
        ren=2d0*pi**2/12d0
      else
        ren=0d0
      endif

      D0g1H=-4d0*Hqaqavsq+D0g0H(i1,i2,i3,i4,i5,i6)*ren
*XC c_gamma fix up
      return
      end
************************************************************************
      double precision function Dt0g1H(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)
C     Amplitude squared with certain factors removed
C     for the process H --> q(p1)+a(p2)+q(p3)+a(p4)
      common /order/ieorder
      parameter(pi=3.141592653589793238d0)
      double complex L0,L1,Lsm1,Lsm1_2me,lnrat
      double precision s123,s124,s134,s234
      double precision fun1D,fun2D,fun3D,fun4D,fun5D,fun6D,fun7D,fun8D
      double precision Hqaqa,Hqaqasq,mhsq,musq
      integer i1,i2,i3,i4
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43

      j1=i1
      j2=i2
      j3=i3
      j4=i4

      musq=renscale2
      epinv=1d0

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      s123=s12+s13+s23
      s124=s12+s14+s24
      s234=s23+s24+s34
      s134=s13+s14+s34

      mhsq=s12+s13+s14+s23+s24+s34

      Hqaqa=-D0g0H(i1,i2,i3,i4,i5,i6)/4d0

      if(ieorder.eq.-2)then
          Hqaqavsq = + epinv**2*(+2.D0)*Hqaqa
      elseif(ieorder.eq.-1)then
          Hqaqavsq = + epinv * (
     &     + 3.D0
     &     - dble(lnrat( - s14,musq))
     &     - dble(lnrat( - s23,musq))
     &     - dble(lnrat( - s12,musq))
     &     - dble(lnrat( - s34,musq))
     &     + dble(lnrat( - s24,musq))
     &     + dble(lnrat( - s13,musq))
     &     )*Hqaqa
      elseif(ieorder.eq.0)then
      Hqaqavsq =
     &     + 8.D0
     &     + dble(Lsm1_2me(s123,s234,s23,mhsq))
     &     + dble(Lsm1_2me(s123,s124,s12,mhsq))
     &     + dble(Lsm1_2me(s134,s234,s34,mhsq))
     &     + dble(Lsm1_2me(s124,s134,s14,mhsq))
     &     - dble(Lsm1_2me(s123,s134,s13,mhsq))
     &     - dble(Lsm1_2me(s124,s234,s24,mhsq))
     &     - 3.D0/4.D0*dble(lnrat( - s14,musq))
     &
      Hqaqavsq = Hqaqavsq
     &     + 1.D0/2.D0*dble(lnrat( - s14,musq)**2)
     &     - 3.D0/4.D0*dble(lnrat( - s23,musq))
     &     + 1.D0/2.D0*dble(lnrat( - s23,musq)**2)
     &     - 3.D0/4.D0*dble(lnrat( - s12,musq))
     &     + 1.D0/2.D0*dble(lnrat( - s12,musq)**2)
     &     - 3.D0/4.D0*dble(lnrat( - s34,musq))
     &     + 1.D0/2.D0*dble(lnrat( - s34,musq)**2)
     &     - 1.D0/2.D0*dble(lnrat( - s24,musq)**2)
     &     - 1.D0/2.D0*dble(lnrat( - s13,musq)**2)

      Hqaqavsq =Hqaqavsq*Hqaqa

      Hqaqavsq = Hqaqavsq+(
     & -dble(Lsm1(-s12,-s124,-s24,-s124))*fun1D(s12,s13,s14,s23,s24,s34)
     & -dble(Lsm1(-s23,-s234,-s24,-s234))*fun1D(s23,s13,s34,s12,s24,s14)
     & -dble(Lsm1(-s14,-s124,-s24,-s124))*fun1D(s14,s13,s12,s34,s24,s23)
     & -dble(Lsm1(-s34,-s234,-s24,-s234))*fun1D(s34,s13,s23,s14,s24,s12)
     & -dble(Lsm1(-s34,-s134,-s13,-s134))*fun1D(s12,s13,s14,s23,s24,s34)
     & -dble(Lsm1(-s14,-s134,-s13,-s134))*fun1D(s23,s13,s34,s12,s24,s14)
     & -dble(Lsm1(-s12,-s123,-s13,-s123))*fun1D(s14,s13,s12,s34,s24,s23)
     & -dble(Lsm1(-s23,-s123,-s13,-s123))*fun1D(s34,s13,s23,s14,s24,s12))

c--- NB: in sub-formula below, I have replaced the line:
c     & -dble(Lsm1(-s12,-s124,-s24,-s124))*fun2D(s12,s24,s23,s14,s13,s34)
c---     with the one:
c     & -dble(Lsm1(-s12,-s124,-s14,-s124))*fun2D(s12,s24,s23,s14,s13,s34)
c--- This is as prescribed by Eq. (37) of EGZ and restores 1<>3 symmetry

      Hqaqavsq = Hqaqavsq+(
     & -dble(Lsm1(-s12,-s123,-s23,-s123))*fun2D(s12,s13,s14,s23,s24,s34)
     & -dble(Lsm1(-s23,-s123,-s12,-s123))*fun2D(s23,s13,s34,s12,s24,s14)
c     & -dble(Lsm1(-s12,-s124,-s24,-s124))*fun2D(s12,s24,s23,s14,s13,s34)
     & -dble(Lsm1(-s12,-s124,-s14,-s124))*fun2D(s12,s24,s23,s14,s13,s34)
     & -dble(Lsm1(-s14,-s124,-s12,-s124))*fun2D(s14,s24,s34,s12,s13,s23)
     & -dble(Lsm1(-s34,-s234,-s23,-s234))*fun2D(s23,s24,s12,s34,s13,s14)
     & -dble(Lsm1(-s23,-s234,-s34,-s234))*fun2D(s34,s24,s14,s23,s13,s12)
     & -dble(Lsm1(-s34,-s134,-s14,-s134))*fun2D(s34,s13,s23,s14,s24,s12)
     & -dble(Lsm1(-s34,-s134,-s14,-s134))*fun2D(s14,s13,s12,s34,s24,s23))


      Hqaqavsq = Hqaqavsq+(
     & + dble(L1(-s134,-s34))*fun3D(s34,s13,s23,s14,s24,s12)
     & + dble(L1(-s134,-s14))*fun3D(s14,s13,s12,s34,s24,s23)
     & + dble(L1(-s234,-s34))*fun3D(s34,s24,s14,s23,s13,s12)
     & + dble(L1(-s124,-s14))*fun3D(s14,s24,s34,s12,s13,s23)
     & + dble(L1(-s123,-s12))*fun3D(s12,s13,s14,s23,s24,s34)
     & + dble(L1(-s123,-s23))*fun3D(s23,s13,s34,s12,s24,s14)
     & + dble(L1(-s124,-s12))*fun3D(s12,s24,s23,s14,s13,s34)
     & + dble(L1(-s234,-s23))*fun3D(s23,s24,s12,s34,s13,s14))

      Hqaqavsq = Hqaqavsq+(
     & + dble(L0(-s134,-s34))*fun5D(s34,s13,s23,s14,s24,s12)
     & + dble(L0(-s134,-s14))*fun5D(s14,s13,s12,s34,s24,s23)
     & + dble(L0(-s234,-s34))*fun5D(s34,s24,s14,s23,s13,s12)
     & + dble(L0(-s124,-s14))*fun5D(s14,s24,s34,s12,s13,s23)
     & + dble(L0(-s123,-s12))*fun5D(s12,s13,s14,s23,s24,s34)
     & + dble(L0(-s123,-s23))*fun5D(s23,s13,s34,s12,s24,s14)
     & + dble(L0(-s124,-s12))*fun5D(s12,s24,s23,s14,s13,s34)
     & + dble(L0(-s234,-s23))*fun5D(s23,s24,s12,s34,s13,s14))

      Hqaqavsq = Hqaqavsq+(
     & +dble(lnrat(-s134,-s34))*fun7D(s34,s13,s23,s14,s24,s12)
     & +dble(lnrat(-s134,-s14))*fun7D(s14,s13,s12,s34,s24,s23)
     & +dble(lnrat(-s234,-s34))*fun7D(s34,s24,s14,s23,s13,s12)
     & +dble(lnrat(-s124,-s14))*fun7D(s14,s24,s34,s12,s13,s23)
     & +dble(lnrat(-s123,-s12))*fun7D(s12,s13,s14,s23,s24,s34)
     & +dble(lnrat(-s123,-s23))*fun7D(s23,s13,s34,s12,s24,s14)
     & +dble(lnrat(-s124,-s12))*fun7D(s12,s24,s23,s14,s13,s34)
     & +dble(lnrat(-s234,-s23))*fun7D(s23,s24,s12,s34,s13,s14))


      Hqaqavsq = Hqaqavsq+(
     & +fun8D(s34,s13,s23,s14,s24,s12)
     & +fun8D(s14,s13,s12,s34,s24,s23)
     & +fun8D(s34,s24,s14,s23,s13,s12)
     & +fun8D(s14,s24,s34,s12,s13,s23)
     & +fun8D(s12,s13,s14,s23,s24,s34)
     & +fun8D(s23,s13,s34,s12,s24,s14)
     & +fun8D(s12,s24,s23,s14,s13,s34)
     & +fun8D(s23,s24,s12,s34,s13,s14))

      endif

      if(ieorder.eq.0)then
        ren=2d0*pi**2/12d0
      else
        ren=0d0
      endif

      Dt0g1H=-4d0*Hqaqavsq-D0g0H(i1,i2,i3,i4,i5,i6)*ren
*XC c_gamma fix up
      return
      end
************************************************************************
      double precision function Dh0g1H(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)
C     Amplitude squared with certain factors removed
C     for the process H --> q(p1)+a(p2)+q(p3)+a(p4)
      common /order/ieorder
      double complex L0,L1,Lsm1,Lsm1_2me,lnrat
      double precision s123,s124,s134,s234
      double precision fun1D,fun2D,fun3D,fun4D,fun5D,fun6D,fun7D,fun8D
      double precision Hqaqa,Hqaqasq,mhsq,musq
      integer i1,i2,i3,i4
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43

      j1=i1
      j2=i2
      j3=i3
      j4=i4

      musq=renscale2
      epinv=1d0

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      s123=s12+s13+s23
      s124=s12+s14+s24
      s234=s23+s24+s34
      s134=s13+s14+s34

      mhsq=s12+s13+s14+s23+s24+s34

      Hqaqa=-D0g0H(i1,i2,i3,i4,i5,i6)/4d0

      if(ieorder.eq.-2)then
          Hqaqavsq = 0d0
      elseif(ieorder.eq.-1)then
          Hqaqavsq = 0d0
      elseif(ieorder.eq.0)then
      Hqaqavsq =
     &     - 40.D0/9.D0*0.5d0
     &     + 2.D0/3.D0*dble(lnrat( - s14,musq))*0.5d0
     &
      Hqaqavsq = Hqaqavsq
     &     + 2.D0/3.D0*dble(lnrat( - s23,musq))*0.5d0
     &     + 2.D0/3.D0*dble(lnrat( - s12,musq))*0.5d0
     &     + 2.D0/3.D0*dble(lnrat( - s34,musq))*0.5d0

      Hqaqavsq =Hqaqavsq*Hqaqa

      endif

      Dh0g1H=4d0*Hqaqavsq

      return
      end
************************************************************************
      double precision function fun1C(i1,i2,i3,i4)
      use KinData_mod
      implicit double precision (a-h,o-y)
C-----Author: Keith Ellis
C-----July  2005
C----Matrix element squared for the virtual corrections to the process
C----q(p1)+r(p3) -> q(p2)+r(p4)+Higgs
C----summed over incoming/outgoing colors and spins.
C    with a factor of gsq*ason2pi*Asq removed.
      double complex L0,L1,Lsm1,Lsm1_2me,lnrat
      double precision Hqarb,Hqarbsq,mhsq
      integer i1,i2,i3,i4
      logical CheckEGZ
      common/CheckEGZ/CheckEGZ
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43

      j1=i1
      j2=i2
      j3=i3
      j4=i4

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      fun1C=
     . -s12*s34/(2d0*s13**2)
     . -s24**2/(2d0*s12*s34)
     . +(3d0*s13*s24-s23**2+s14*s23
     . -s14**2-s13**2)/(s12*s34)
     . -s14**2*s23**2/(2d0*s12*s13**2*s34)
     .   -2d0*(s13*s24-s14*s23)**2
     .  /(s12**2*s34**2)+s24/s13
     .  +s14*s23/s13**2-2d0


      return
      end

************************************************************************
      double precision function fun2C(i1,i2,i3,i4)
      use KinData_mod
      implicit double precision (a-h,o-y)
C-----Author: Keith Ellis
C-----July  2005
C----Matrix element squared for the virtual corrections to the process
C----q(p1)+r(p3) -> q(p2)+r(p4)+Higgs
C----summed over incoming/outgoing colors and spins.
C    with a factor of gsq*ason2pi*Asq removed.
      double complex L0,L1,Lsm1,Lsm1_2me,lnrat
      double precision Hqarb,Hqarbsq,mhsq
      integer i1,i2,i3,i4
      logical CheckEGZ
      common/CheckEGZ/CheckEGZ
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43

      j1=i1
      j2=i2
      j3=i3
      j4=i4

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      fun2C =
     . (s12*s34*(s12*s34+s23
     . *(s24+2d0*s23-s14))+s23**2
     . *(s24+s14)**2)
     .  /(2*s12**3*s34)

      return
      end

************************************************************************
      double precision function fun3C(i1,i2,i3,i4)
      use KinData_mod
      implicit double precision (a-h,o-y)
C-----Author: Keith Ellis
C-----July  2005
C----Matrix element squared for the virtual corrections to the process
C----q(p1)+r(p3) -> q(p2)+r(p4)+Higgs
C----summed over incoming/outgoing colors and spins.
C    with a factor of gsq*ason2pi*Asq removed.
      double complex L0,L1,Lsm1,Lsm1_2me,lnrat
      double precision Hqarb,Hqarbsq,mhsq
      integer i1,i2,i3,i4
      logical CheckEGZ
      common/CheckEGZ/CheckEGZ
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43

      j1=i1
      j2=i2
      j3=i3
      j4=i4

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      fun3C =
     . s34/(2d0*s23)+s23*(s24+s14)
     . *(s24+4d0*s23+3d0*s14)
     . /(2d0*s12**2*s34)
     . +(3d0*s24+4d0*s23)/(2d0*s12)

      return
      end

************************************************************************
      double precision function fun4C(i1,i2,i3,i4)
      use KinData_mod
      implicit double precision (a-h,o-y)
C-----Author: Keith Ellis
C-----July  2005
C----Matrix element squared for the virtual corrections to the process
C----q(p1)+r(p3) -> q(p2)+r(p4)+Higgs
C----summed over incoming/outgoing colors and spins.
C    with a factor of gsq*ason2pi*Asq removed.
      double complex L0,L1,Lsm1,Lsm1_2me,lnrat
      double precision Hqarb,Hqarbsq,mhsq
      integer i1,i2,i3,i4
      logical CheckEGZ
      common/CheckEGZ/CheckEGZ
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43

      j1=i1
      j2=i2
      j3=i3
      j4=i4

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      fun4C=
     . -2d0*s34/s23-s23*(s24+s14)
     . *(s24+2d0*s23+5d0*s14)
     . /(2d0*s12**2*s34)
     . -(4d0*s24+6*s23-3d0*s14)/(2d0*s12)

      return
      end
************************************************************************
      double precision function fun5C(i1,i2,i3,i4)
      use KinData_mod
      implicit double precision (a-h,o-y)
C-----Author: Keith Ellis
C-----July  2005
C----Matrix element squared for the virtual corrections to the process
C----q(p1)+r(p3) -> q(p2)+r(p4)+Higgs
C----summed over incoming/outgoing colors and spins.
C    with a factor of gsq*ason2pi*Asq removed.
      double complex L0,L1,Lsm1,Lsm1_2me,lnrat
      double precision Hqarb,Hqarbsq,mhsq
      integer i1,i2,i3,i4
      logical CheckEGZ
      common/CheckEGZ/CheckEGZ
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43

      j1=i1
      j2=i2
      j3=i3
      j4=i4

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      fun5C =
     . -(s24**2*(s34**2-s24*s34
     . +s13*s34-s13*s24+s14*s23)
     . +s14*s23*(2d0*s24*s34
     . -s13*s24+s14*s23))
     . /(s14*s24**2*s34)

      return
      end

************************************************************************
      double precision function fun6C(i1,i2,i3,i4)
      use KinData_mod
      implicit double precision (a-h,o-y)
C-----Author: Keith Ellis
C-----July  2005
C----Matrix element squared for the virtual corrections to the process
C----q(p1)+r(p3) -> q(p2)+r(p4)+Higgs
C----summed over incoming/outgoing colors and spins.
C    with a factor of gsq*ason2pi*Asq removed.
      double complex L0,L1,Lsm1,Lsm1_2me,lnrat
      double precision Hqarb,Hqarbsq,mhsq
      integer i1,i2,i3,i4
      logical CheckEGZ
      common/CheckEGZ/CheckEGZ
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43

      j1=i1
      j2=i2
      j3=i3
      j4=i4

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      fun6C=
     . s12*s34/(2d0*s13*s23)
     . +(4d0*s23*s24+2d0*s14*s24
     . -3d0*s13*s24+3d0*s14*s23)
     . /(2d0*s12*s34)+s14**2*s23
     . /(2d0*s12*s13*s34)-s14/s13+0.5d0

      return
      end
************************************************************************
      double precision function fun7C(i1,i2,i3,i4)
      use KinData_mod
      implicit double precision (a-h,o-y)
C-----Author: Keith Ellis
C-----July  2005
C----Matrix element squared for the virtual corrections to the process
C----q(p1)+r(p3) -> q(p2)+r(p4)+Higgs
C----summed over incoming/outgoing colors and spins.
C    with a factor of gsq*ason2pi*Asq removed.
      double complex L0,L1,Lsm1,Lsm1_2me,lnrat
      double precision Hqarb,Hqarbsq,mhsq
      integer i1,i2,i3,i4
      logical CheckEGZ
      common/CheckEGZ/CheckEGZ
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43

      j1=i1
      j2=i2
      j3=i3
      j4=i4

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      fun7C=
     . (s14**2*s23**2+s13*s34**2*s12
     . -s13*s14*s23*s24)
     . /2/s13**2/s34/s12

      return
      end
************************************************************************
      double precision function fun8C(i1,i2,i3,i4)
      use KinData_mod
      implicit double precision (a-h,o-y)
C-----Author: Keith Ellis
C-----July  2005
C----Matrix element squared for the virtual corrections to the process
C----q(p1)+r(p3) -> q(p2)+r(p4)+Higgs
C----summed over incoming/outgoing colors and spins.
C    with a factor of gsq*ason2pi*Asq removed.
      double complex L0,L1,Lsm1,Lsm1_2me,lnrat
      double precision Hqarb,Hqarbsq,mhsq
      integer i1,i2,i3,i4
      logical CheckEGZ
      common/CheckEGZ/CheckEGZ
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43

      j1=i1
      j2=i2
      j3=i3
      j4=i4

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      fun8C=
     . (s14*s23-s13*s24)/s12/s34/2d0

      return
      end
************************************************************************
      double precision function fun9C(i1,i2,i3,i4)
      use KinData_mod
      implicit double precision (a-h,o-y)
C-----Author: Keith Ellis
C-----July  2005
C----Matrix element squared for the virtual corrections to the process
C----q(p1)+r(p3) -> q(p2)+r(p4)+Higgs
C----summed over incoming/outgoing colors and spins.
C    with a factor of gsq*ason2pi*Asq removed.
      double complex L0,L1,Lsm1,Lsm1_2me,lnrat
      double precision Hqarb,Hqarbsq,mhsq
      integer i1,i2,i3,i4
      logical CheckEGZ
      common/CheckEGZ/CheckEGZ
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43

      j1=i1
      j2=i2
      j3=i3
      j4=i4

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      fun9C=
     . +s13*s24**2/(s12*s23*s34)
     . -s12*s34/(s13*s23)-(2d0*s24**2
     . +2d0*s23*s24+5d0*s14*s24
     . -5d0*s13*s24+5d0*s14*s23)
     . /(2d0*s12*s34)
     . -s14**2*s23/(s12*s13*s34)
     . -2d0*s24/s23+2d0*s14/s13-1d0

      return
      end
************************************************************************
      double precision function fun10C(i1,i2,i3,i4)
      use KinData_mod
      implicit double precision (a-h,o-y)
C-----Author: Keith Ellis
C-----July  2005
C----Matrix element squared for the virtual corrections to the process
C----q(p1)+r(p3) -> q(p2)+r(p4)+Higgs
C----summed over incoming/outgoing colors and spins.
C    with a factor of gsq*ason2pi*Asq removed.
      double complex L0,L1,Lsm1,Lsm1_2me,lnrat
      double precision Hqarb,Hqarbsq,mhsq
      integer i1,i2,i3,i4
      logical CheckEGZ
      common/CheckEGZ/CheckEGZ
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43

      j1=i1
      j2=i2
      j3=i3
      j4=i4

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      fun10C=
     . (s12*s23*s34**2+s13**2*s24**2
     . -s13*s14*s23*s24)
     . /(s12*s23**2*s34)

      return
      end
************************************************************************
      double precision function fun11C(i1,i2,i3,i4)
      use KinData_mod
      implicit double precision (a-h,o-y)
C-----Author: Keith Ellis
C-----July  2005
C----Matrix element squared for the virtual corrections to the process
C----q(p1)+r(p3) -> q(p2)+r(p4)+Higgs
C----summed over incoming/outgoing colors and spins.
C    with a factor of gsq*ason2pi*Asq removed.
      double complex L0,L1,Lsm1,Lsm1_2me,lnrat
      double precision Hqarb,Hqarbsq,mhsq
      integer i1,i2,i3,i4
      logical CheckEGZ
      common/CheckEGZ/CheckEGZ
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43

      j1=i1
      j2=i2
      j3=i3
      j4=i4

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      fun11C=2d0*s13*s24/s12/s34

      return
      end
************************************************************************
      double precision function fun12C(i1,i2,i3,i4)
      use KinData_mod
      implicit double precision (a-h,o-y)
C-----Author: Keith Ellis
C-----July  2005
C----Matrix element squared for the virtual corrections to the process
C----q(p1)+r(p3) -> q(p2)+r(p4)+Higgs
C----summed over incoming/outgoing colors and spins.
C    with a factor of gsq*ason2pi*Asq removed.
      double complex L0,L1,Lsm1,Lsm1_2me,lnrat
      double precision Hqarb,Hqarbsq,mhsq
      integer i1,i2,i3,i4
      logical CheckEGZ
      common/CheckEGZ/CheckEGZ
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43

      j1=i1
      j2=i2
      j3=i3
      j4=i4

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      fun12C=
     . +s13*(s13*(s14-s24)
     . +2d0*s14*s23)/2/s12**2
     . /s34+s14/s12/2d0

      return
      end
************************************************************************
      double precision function fun1D(s12,s13,s14,s23,s24,s34)
      use KinData_mod
      implicit none
C     Amplitude squared with certain factors removed
C     for the process H --> q(p1)+a(p2)+q(p3)+a(p4)
      double complex L0,L1,Lsm1,Lsm1_2me,lnrat
      double precision s123,s124,s134,s234,s12,s13,s14,s23,s24,s34

      fun1D=(
     &  - 2.D0*s12*s13*s24*s34+ s12*s13**2*s34
     &  - 4.D0*s12*s14*s23*s34+ s12*s24**2*s34
     &  + 2.D0*s12**2*s34**2- 2.D0*s13*s14*s23*s24
     &  - s13*s24**3+ s13**2*s14*s23
     &  - s13**3*s24+ s14*s23*s24**2+ 2.D0*s14**2*s23**2)
     & /(4d0*s12*s23*s14*s34)

      return
      end
************************************************************************
      double precision function fun2D(s12,s13,s14,s23,s24,s34)
      use KinData_mod
      implicit none
C     Amplitude squared with certain factors removed
C     for the process H --> q(p1)+a(p2)+q(p3)+a(p4)
      double complex L0,L1,Lsm1,Lsm1_2me,lnrat
      double precision s123,s124,s134,s234,s12,s13,s14,s23,s24,s34

      fun2D=(
     &  - 2.D0*s12*s13*s14*s23*s24*s34
     &  + 2.D0*s12*s13**2*s14*s23*s34
     &  - 4.D0*s12*s13**2*s24**2*s34
     &  + 3.D0*s12*s13**3*s24*s34
     &  - 2.D0*s12*s13**4*s34
     &  + s12*s14**2*s23**2*s34
     &  + 3.D0*s12**2*s13*s24*s34**2
     &  - 3.D0*s12**2*s13**2*s34**2
     &  + s12**2*s14*s23*s34**2
     &  - s12**3*s34**3
     &  + 3.D0*s13*s14**2*s23**2*s24
     &  - 4.D0*s13**2*s14*s23*s24**2
     &  - 3.D0*s13**2*s14**2*s23**2
     &  + 3.D0*s13**3*s14*s23*s24
     &  + 2.D0*s13**3*s24**3
     &  - 2.D0*s13**4*s14*s23
     &  + 2.D0*s13**5*s24
     &  - s14**3*s23**3)
     & /(8d0*s12*s13**2*s23*s14*s34)

      return
      end
************************************************************************
      double precision function fun3D(s12,s13,s14,s23,s24,s34)
      use KinData_mod
      implicit none
C     Amplitude squared with certain factors removed
C     for the process H --> q(p1)+a(p2)+q(p3)+a(p4)
      double complex L0,L1,Lsm1,Lsm1_2me,lnrat
      double precision s123,s124,s134,s234,s12,s13,s14,s23,s24,s34

      fun3D=(
     &  - 2.D0*s12*s14*s23*s34
     &  + s12*s23*s24*s34
     &  + s12*s23**2*s34
     &  + s12**2*s34**2
     &  + s14*s23**2*s24
     &  + s14*s23**3
     &  + s14**2*s23**2
     &  + s23**3*s24)
     & /(8d0*s12**2*s14*s23)

      return
      end
************************************************************************
      double precision function fun4D(s12,s13,s14,s23,s24,s34)
      use KinData_mod
      implicit none
C     Amplitude squared with certain factors removed
C     for the process H --> q(p1)+a(p2)+q(p3)+a(p4)
      double complex L0,L1,Lsm1,Lsm1_2me,lnrat
      double precision s123,s124,s134,s234,s12,s13,s14,s23,s24,s34

      fun4D=(
     &  - 2.D0*s12*s23*s34+ 5.D0*s12*s24*s34- 3.D0*s14*s23*s24
     &  + 6.D0*s14*s23**2+ 4.D0*s23*s24**2- s23**2*s24)
     & /(8d0*s14*s12*s23)

      return
      end
************************************************************************
      double precision function fun5D(s12,s13,s14,s23,s24,s34)
      use KinData_mod
      implicit none
C     Amplitude squared with certain factors removed
C     for the process H --> q(p1)+a(p2)+q(p3)+a(p4)
      double complex L0,L1,Lsm1,Lsm1_2me,lnrat
      double precision s123,s124,s134,s234,s12,s13,s14,s23,s24,s34

      fun5D=(
     &  + s12*s14*s23**2*s24*s34- 6.D0*s12*s14*s23**3*s34
     &  + 2.D0*s12*s14**2*s23**2*s34- 2.D0*s12*s23**2*s24**2*s34
     &  + 3.D0*s12*s23**3*s24*s34+ 2.D0*s12**2*s14*s23*s34**2
     &  - 3.D0*s12**2*s23*s24*s34**2+ 2.D0*s12**2*s23**2*s34**2
     &  - 2.D0*s12**3*s34**3- 2.D0*s14*s23**3*s24**2
     &  - 4.D0*s14**2*s23**3*s24- 2.D0*s14**3*s23**3)
     & /(8d0*s12**2*s23**2*s14*s34)

      return
      end
************************************************************************
      double precision function fun6D(s12,s13,s14,s23,s24,s34)
      use KinData_mod
      implicit none
C     Amplitude squared with certain factors removed
C     for the process H --> q(p1)+a(p2)+q(p3)+a(p4)
      double complex L0,L1,Lsm1,Lsm1_2me,lnrat
      double precision s123,s124,s134,s234,s12,s13,s14,s23,s24,s34

      fun6D=(- 3.D0*s12*s34+ 3.D0*s13*s24
     &  + 5.D0*s14*s23- 2.D0*s23*s24+ 4.D0*s24**2)
     & /(8d0*s14*s23)

      return
      end
************************************************************************
      double precision function fun7D(s12,s13,s14,s23,s24,s34)
      use KinData_mod
      implicit none
C     Amplitude squared with certain factors removed
C     for the process H --> q(p1)+a(p2)+q(p3)+a(p4)
      double complex L0,L1,Lsm1,Lsm1_2me,lnrat
      double precision s123,s124,s134,s234,s12,s13,s14,s23,s24,s34

      fun7D=(
     &  - 7.D0*s12*s13*s14*s23**2*s34
     &  - 2.D0*s12*s13*s23*s24**2*s34
     &  + 2.D0*s12*s13*s23**2*s24*s34
     &  - s12*s13**2*s23*s24*s34
     &  + 2.D0*s12*s14**2*s23**2*s34
     &  + s12**2*s13*s23*s34**2
     &  + 2.D0*s12**2*s14*s23*s34**2
     &  - 2.D0*s12**3*s34**3
     &  - 2.D0*s13*s14*s23**2*s24**2
     &  - 2.D0*s14**3*s23**3)
     & /(8d0*s12*s13*s23**2*s14*s34)

      return
      end
************************************************************************
      double precision function fun8D(s12,s13,s14,s23,s24,s34)
      use KinData_mod
      implicit none
C     Amplitude squared with certain factors removed
C     for the process H --> q(p1)+a(p2)+q(p3)+a(p4)
      double complex L0,L1,Lsm1,Lsm1_2me,lnrat
      double precision s123,s124,s134,s234,s12,s13,s14,s23,s24,s34

      fun8D=(
     &  - 2.D0*s12*s13*s14*s24- 2.D0*s12*s13*s23*s24
     &  + s12*s13*s23**2+ s12*s13*s24**2
     &  + s12*s13**2*s24+ 2.D0*s12*s14*s23**2
     &  + 2.D0*s12*s14*s34**2+ 2.D0*s12*s14**2*s23
     &  + s12*s14**2*s24+ 2.D0*s12*s23*s34**2+s12**2*s13*s23
     &  + s12**2*s14*s24+ 2.D0*s12**2*s14*s34
     &  + 2.D0*s12**2*s23*s34- 2.D0*s13*s14*s24*s34+ s13*s14*s24**2
     &  + s13*s14*s34**2 + s13*s14**2*s34
     &  - 2.D0*s13*s23*s24*s34+ s13*s23*s24**2
     &  + s13*s24**2*s34+ s13**2*s14*s24
     &  + s13**2*s23*s24+ s13**2*s24*s34
     &  + 2.D0*s14*s23**2*s34+ 2.D0*s14**2*s23*s34
     &  + s23*s24*s34**2  + s23**2*s24*s34)
     &  /(64d0*s12*s14*s23*s34)

      return
      end

