************************************************************************
!XC zffDDHK and zfWDDHK functions are based on MCFM-6.8 source code
!  J.~M.~Campbell and R.~K.~Ellis,
!  %``An Update on vector boson pair production at hadron colliders,''
!  Phys.\ Rev.\ D {\bf 60} (1999) 113006
!  doi:10.1103/PhysRevD.60.113006
!  [hep-ph/9905386].

!XC Loop functions based on hep-ph/9701342
!  A.~Djouadi, V.~Driesen, W.~Hollik and A.~Kraft,
!  %``The Higgs photon - Z boson coupling revisited,''
!  Eur.\ Phys.\ J.\ C {\bf 1} (1998) 163
!  doi:10.1007/BF01245806
!  [hep-ph/9701342].
***************************************************************

      complex*16 function zffDDHK(s123,s12,msq)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      double precision msq

      zffDDHK=zC0DDHK(s123,s12,msq)+4d0*zC2DDHK(s123,s12,msq)
      return
      end


      complex*16 function zfWDDHK(s123,s12,msq)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      double precision msq
c----- msq here should only be equal to wmass**2

      zfWDDHK=2d0*(s123/msq*(1d0-2d0*msq/s12)+2d0*(1d0-6d0*msq/s12))*zC2DDHK(s123,s12,msq)
     &       +4d0*(1d0-4d0*msq/s12)*zC0DDHK(s123,s12,msq)
      return
      end


***************************************************************
********************sub-layer functions************************
***************************************************************


      complex*16 function zC0DDHK(s123,s12,msq)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      double precision msq

      tauZ=4d0*msq/s12
      tauH=4d0*msq/s123
      zC0DDHK=-2d0*tauZ*tauH/(tauZ-tauH)*(zFDDHK(tauZ)-zFDDHK(tauH))
C-----changed sign in order to agree with normal QCDLoop definition of scalar integral
      zC0DDHK=-zC0DDHK/(4d0*msq)

      return
      end


      complex*16 function zC2DDHK(s123,s12,msq)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      double precision msq

      tauZ=4d0*msq/s12
      tauH=4d0*msq/s123
      zC2DDHK=tauZ*tauH/(2d0*(tauZ-tauH))
     & +tauZ*tauH**2/(2d0*(tauZ-tauH)**2)
     & *(tauZ*(zFDDHK(tauZ)-zFDDHK(tauH))+2d0*(zGDDHK(tauZ)-zGDDHK(tauH)))
C-----changed sign in order to agree with QCDLoop result
      zC2DDHK=-zC2DDHK/(4d0*msq)

      return
      end


***************************************************************
******************sub-sub-layer functions**********************
***************************************************************


      complex*16 function zFDDHK(tau)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      zi=(0d0,1d0)

      if (tau >= 1d0) then
      root1otau=1d0/sqrt(tau)
      zFDDHK=asin(root1otau)**2
      else
      root1mtau=sqrt(1d0-tau)
      xlog=log((1d0+root1mtau)/(1d0-root1mtau))
      zFDDHK=-1d0/4d0*(xlog-zi*pi)**2
      endif
      return
      end

      complex*16 function zGDDHK(tau)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      zi=(0d0,1d0)

      if (tau >= 1d0) then
      root1mtau=sqrt(tau-1d0)
      root1otau=1d0/sqrt(tau)
      zGDDHK=root1mtau*asin(root1otau)
      else
      root1mtau=sqrt(1d0-tau)
      xlog=log((1d0+root1mtau)/(1d0-root1mtau))
      zGDDHK=root1mtau/2d0*(xlog-zi*pi)
      endif
      return
      end



