************************************************************************
!XC The HZZ2e2mu0 function is based on PhysRevD.37.3193
!  E.~W.~N.~Glover, J.~Ohnemus and S.~S.~D.~Willenbrock,
!  %``Higgs Boson Decay to One Real and One Virtual $W$ Boson,''
!  Phys.\ Rev.\ D {\bf 37} (1988) 3193.
!  doi:10.1103/PhysRevD.37.3193

!XC The HZZ2e2mu0 function is cross checked with code from MCFM-6.8
!  J.~M.~Campbell and R.~K.~Ellis,
!  %``An Update on vector boson pair production at hadron colliders,''
!  Phys.\ Rev.\ D {\bf 60} (1999) 113006
!  doi:10.1103/PhysRevD.60.113006
!  [hep-ph/9905386].

************************************************************************
*
*  higgs decay to ZZ* then decay to four leptons (l1+,l1-,l2+,l2-)
* i1=l1-(e-), i2=l1+(e+), i3=l2-(mu-), i4=l2+(mu+).
* s(i1,i2)=mZ^2(mZ*^2) and s(i3,i4)=mZ*^2(mZ^2) (invariant mass swap if isym=1)
**
*
************************************************************************
      function HZZ2e2mu0(i1,i2,i3,i4,ipset)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /Zmass/ emz,ezwidth
      common /Wmass/ emw,ewwidth
      common /LR/ ele,ere,sin2w,enQ(-5:5),enL(5),enR(5)
      common /eweakW/ amzW,stw,ewfac
!$omp threadprivate(/eweakW/)
      common /eweakZ/ amzZ,zewfac(4),zewnull(4)
!$omp threadprivate(/eweakZ/)
      common /Hmass/emh,ehwidth
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43

      s12 => kin(ipset)%s(i1,i2)
      s13 => kin(ipset)%s(i1,i3)
      s14 => kin(ipset)%s(i1,i4)
      s23 => kin(ipset)%s(i2,i3)
      s24 => kin(ipset)%s(i2,i4)
      s34 => kin(ipset)%s(i3,i4)

      s21 => kin(ipset)%s(i2,i1)
      s31 => kin(ipset)%s(i3,i1)
      s41 => kin(ipset)%s(i4,i1)
      s32 => kin(ipset)%s(i3,i2)
      s42 => kin(ipset)%s(i4,i2)
      s43 => kin(ipset)%s(i4,i3)

      s1234=s12+s13+s14+s23+s24+s34

      if((amzW.eq.0).or.(stw.eq.0))then
         call initeweak
      endif

      propHnarrow=1d0/((s1234-emh**2)**2+(emh*ehwidth)**2)
c      propHnarrow=pi/emh/ehwidth
c      propHnarrow=1d0
*Higgs propagator in Breit-Wigner resonance formalism

      propZ1=1d0/((s12-emz**2)**2+(emz*ezwidth)**2)
c      propZ1=pi/emz/ezwidth
c      propZ1=1d0
*Z propagator in Breit-Wigner resonance formalism

      propZ2=1d0/((s34-emz**2)**2+(emz*ezwidth)**2)
c      propZ2=pi/emz/ezwidth
c      propZ2=1d0
*Z propagator in Breit-Wigner resonance formalism


      HZZ2e2mu0=propHnarrow*propZ1*propZ2
     .       *((ele**4+ere**4)*s13*s24 + 2d0*ele**2*ere**2*s14*s23)
*H-->(Z-->e-(1)e+(2))(Z-->mu-(3)mu+(4))decay ME extended from PhysRevD.37.3193 by Nigel cross checked with MCFM gg_hZZg.f
      return
      end
