************************************************************************
!XC The HZg2l1g0 function is based on MCFM-6.8 source code
!  J.~M.~Campbell and R.~K.~Ellis,
!  %``An Update on vector boson pair production at hadron colliders,''
!  Phys.\ Rev.\ D {\bf 60} (1999) 113006
!  doi:10.1103/PhysRevD.60.113006
!  [hep-ph/9905386].

!XC Loop functions zfWDDHK and zffDDHK are based hep-ph/9701342
!  A.~Djouadi, V.~Driesen, W.~Hollik and A.~Kraft,
!  %``The Higgs photon - Z boson coupling revisited,''
!  Eur.\ Phys.\ J.\ C {\bf 1} (1998) 163
!  doi:10.1007/BF01245806
!  [hep-ph/9701342].

************************************************************************
*
*  higgs decay to Z/gamma* + gamma then decay to two leptons + one photon
* i1=l1-(e-), i2=l1+(e+), i3=gamma.
* s(i1,i2)=m_Z^2(m_gamma*^2)
**
*
************************************************************************
      function HZg2l1g0(i1,i2,i3,ipset)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /Zmass/ emz,ezwidth
      common /Wmass/ emw,ewwidth
      common /LR/ ele,ere,sin2w,enQ(-5:5),enL(5),enR(5)
      common /eweakW/ amzW,stw,ewfac
!$omp threadprivate(/eweakW/)
      common /eweakZ/ amzZ,zewfac(4),zewnull(4)
!$omp threadprivate(/eweakZ/)
      common /Hmass/emh,ehwidth
      common /Qmass/emt,etwidth,emb,ebwidth,emc,ecwidth,emtau,etauwidth
      real*8, pointer :: s12,s13,s23,s21,s31,s32
      double precision v1(2),mt_eff,nc,mwsq,mtsq
      complex*16 zF(2)
      parameter(nc=3d0)
      parameter(mt_eff=169.47396236888420d0)
*XC the mt_eff value here is the running top mass at 125GeV (mH) scale with MSbar scheme.
*here we assume mt_msbar=166 GeV and alphas=0.118 and the running of mt_eff only use the LO beta function
*the running mt_eff with NLO beta function is mt_eff=169.74113270998026 which cause less than 0.04% difference for HZg2eg0

      s12 => kin(ipset)%s(i1,i2)
      s13 => kin(ipset)%s(i1,i3)
      s23 => kin(ipset)%s(i2,i3)

      s21 => kin(ipset)%s(i2,i1)
      s31 => kin(ipset)%s(i3,i1)
      s32 => kin(ipset)%s(i3,i2)

      s123=s12+s13+s23

      if((amzW.eq.0).or.(stw.eq.0))then
         call initeweak
      endif

      zi=(0d0,1d0)

      propHnarrow=1d0/((s123-emh**2)**2+(emh*ehwidth)**2)
c      propHnarrow=pi/emh/ehwidth
c      propHnarrow=1d0
*Higgs propagator in Breit-Wigner resonance formalism

      mwsq=emw**2
      mtsq=emt**2
      v1(1)=ele
      v1(2)=ere
      Qf=enQ(2)
      cotw=sqrt((1d0-stw)/stw)
      vfbar=1d0/2d0*(enL(2)+enR(2))

      zpropZnarrow=s12/(s12-emz**2+zi*emz*ezwidth)
C  -- j labels the helicity of the lepton line
      do j=1,2
      zF(j)=zfWDDHK(s123,s12,mwsq)
     & *(-1d0+cotw*v1(j)*zpropZnarrow)
     & +zffDDHK(s123,s12,mtsq)
     & *4d0*Qf*nc*(mt_eff**2/s12)
     & *(-Qf+vfbar*v1(j)*zpropZnarrow)
      enddo
c  include both fermion and W loop for Higgs decay

      HZg2l1g0=propHnarrow*1d0/2d0*s12*(s13**2+s23**2)
     &       *(zF(1)*Dconjg(zF(1))+zF(2)*Dconjg(zF(2)))
*H-->(Z/gamma*-->e-(1)e+(2))+gamma decay ME calculated by R.K.Ellis in MCFM
*also reference from hep-ph/9701342
      return
      end
