************************************************************************
!XC The HWW2l2n0 function is taken from PhysRevD.37.3193
!  E.~W.~N.~Glover, J.~Ohnemus and S.~S.~D.~Willenbrock,
!  %``Higgs Boson Decay to One Real and One Virtual $W$ Boson,''
!  Phys.\ Rev.\ D {\bf 37} (1988) 3193.
!  doi:10.1103/PhysRevD.37.3193

!XC then cross checked against MCFM-6.8
!  J.~M.~Campbell and R.~K.~Ellis,
!  %``An Update on vector boson pair production at hadron colliders,''
!  Phys.\ Rev.\ D {\bf 60} (1999) 113006
!  doi:10.1103/PhysRevD.60.113006
!  [hep-ph/9905386].

************************************************************************
*
*  higgs decay to WW* then decay to two lepton (l1,l2) plus two neutrino (n1,n2)
* i1=l1+(l1-), i2=n1(n1bar), i3=l2-(l2+), i4=n2bar(n2).
* s(i1,i2)=mW^2 and s(i3,i4)=mW*^2
**
*
************************************************************************
      function HWW2l2n0(i1,i2,i3,i4,ipset)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /Zmass/ emz,ezwidth
      common /Wmass/ emw,ewwidth
      common /LR/ ele,ere,sin2w,enQ(-5:5),enL(5),enR(5)
      common /eweakW/ amzW,stw,ewfac
!$omp threadprivate(/eweakW/)
      common /eweakZ/ amzZ,zewfac(4),zewnull(4)
!$omp threadprivate(/eweakZ/)
      common /Hmass/emh,ehwidth
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43

      s12 => kin(ipset)%s(i1,i2)
      s13 => kin(ipset)%s(i1,i3)
      s14 => kin(ipset)%s(i1,i4)
      s23 => kin(ipset)%s(i2,i3)
      s24 => kin(ipset)%s(i2,i4)
      s34 => kin(ipset)%s(i3,i4)

      s21 => kin(ipset)%s(i2,i1)
      s31 => kin(ipset)%s(i3,i1)
      s41 => kin(ipset)%s(i4,i1)
      s32 => kin(ipset)%s(i3,i2)
      s42 => kin(ipset)%s(i4,i2)
      s43 => kin(ipset)%s(i4,i3)

      s1234=s12+s13+s14+s23+s24+s34

      if((amzW.eq.0).or.(stw.eq.0))then
         call initeweak
      endif

      propHnarrow=1d0/((s1234-emh**2)**2+(emh*ehwidth)**2)
c      propHnarrow=pi/emh/ehwidth
c      propHnarrow=1d0
*Higgs propagator in Breit-Wigner resonance formalism

      propW1=1d0/((s12-emw**2)**2+(emw*ewwidth)**2)
c      propW1=pi/emw/ewwidth
c      propW1=1d0
*W propagator in Breit-Wigner resonance formalism

      propW2=1d0/((s34-emw**2)**2+(emw*ewwidth)**2)
c      propW2=pi/emw/ewwidth
c      propW2=1d0
*W propagator in Breit-Wigner resonance formalism

      HWW2l2n0=s14*s23*propHnarrow*propW1*propW2
*decay ME taken from PhysRevD.37.3193 cross checked with MCFM gg_hWWg.f

      return
      end

