************************************************************************
!XC Special functions involved in Higgs+parton 1-loop amplitudes are based on MCFM-6.8 source code:
!  J.~M.~Campbell and R.~K.~Ellis,
!  %``An Update on vector boson pair production at hadron colliders,''
!  Phys.\ Rev.\ D {\bf 60} (1999) 113006
!  doi:10.1103/PhysRevD.60.113006
!  [hep-ph/9905386].

!  J.~M.~Campbell, R.~K.~Ellis and W.~T.~Giele,
!  %``A Multi-Threaded Version of MCFM,''
!  Eur.\ Phys.\ J.\ C {\bf 75} (2015) no.6,  246
!  doi:10.1140/epjc/s10052-015-3461-2
!  [arXiv:1503.06182 [physics.comp-ph]].

!XC The corresponding reference for special functions are:

!  L.~J.~Dixon and Y.~Sofianatos,
!  %``Analytic one-loop amplitudes for a Higgs boson plus four partons,''
!  JHEP {\bf 0908} (2009) 058
!  doi:10.1088/1126-6708/2009/08/058
!  [arXiv:0906.0008 [hep-ph]].

!  S.~D.~Badger, E.~W.~N.~Glover and K.~Risager,
!  %``One-loop phi-MHV amplitudes using the unitarity bootstrap,''
!  JHEP {\bf 0707} (2007) 066
!  doi:10.1088/1126-6708/2007/07/066
!  [arXiv:0704.3914 [hep-ph]].

!  G.~Duplancic and B.~Nizic,
!  %``Dimensionally regulated one loop box scalar integrals with massless internal lines,''
!  Eur.\ Phys.\ J.\ C {\bf 20} (2001) 357
!  doi:10.1007/s100520100675
!  [hep-ph/0006249].

!  Z.~Bern, L.~J.~Dixon and D.~A.~Kosower,
!  %``One loop amplitudes for e+ e- to four partons,''
!  Nucl.\ Phys.\ B {\bf 513} (1998) 3
!  doi:10.1016/S0550-3213(97)00703-7
!  [hep-ph/9708239].

!  N.~I.~Usyukina and A.~I.~Davydychev,
!  %``New results for two loop off-shell three point diagrams,''
!  Phys.\ Lett.\ B {\bf 332} (1994) 159
!  doi:10.1016/0370-2693(94)90874-5
!  [hep-ph/9402223].

!  Z.~Bern, L.~J.~Dixon and D.~A.~Kosower,
!  %``Dimensionally regulated pentagon integrals,''
!  Nucl.\ Phys.\ B {\bf 412} (1994) 751
!  doi:10.1016/0550-3213(94)90398-0
!  [hep-ph/9306240].

************************************************************************
      double complex function zab2(j1,j2,j3,j4,iset)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      zab2=kin(iset)%zA(j1,j2)*kin(iset)%zB(j2,j4)+kin(iset)%zA(j1,j3)*kin(iset)%zB(j3,j4)

      return
      end
************************************************************************
      double complex function zba2(j1,j2,j3,j4,iset)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      zba2=kin(iset)%zB(j1,j2)*kin(iset)%zA(j2,j4)+kin(iset)%zB(j1,j3)*kin(iset)%zA(j3,j4)

      return
      end
************************************************************************
      double precision function ss3(j1,j2,j3,iset)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      ss3=kin(iset)%s(j1,j2)+kin(iset)%s(j2,j3)+kin(iset)%s(j3,j1)

      return
      end
************************************************************************
      double complex function  zab3(j1,j2,j3,j4,iset)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      zab3=kin(iset)%zA(j1,j2)*kin(iset)%zB(j2,j3)*kin(iset)%zA(j3,j4)

      return
      end
************************************************************************

      double complex function  zba3(j1,j2,j3,j4,iset)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      zba3=kin(iset)%zB(j1,j2)*kin(iset)%zA(j2,j3)*kin(iset)%zB(j3,j4)

      return
      end

************************************************************************
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      double complex function F31m(s,renscale2)
      common /order/ieorder
      double precision s,epinv,renscale2
      double complex lnrat
      integer ieorder
      epinv=1d3
*      write(*,*)"facscale=",facscale
      if(ieorder.eq.-2)then
        F31m=1d0
      elseif(ieorder.eq.-1)then
        F31m=-lnrat(-s,renscale2)
      elseif(ieorder.eq.0)then
        F31m=+0.5d0*lnrat(-s,renscale2)**2
      endif

      return
      end
************************************************************************
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      double complex function F41m(psq,s,t,renscale2)
cw--- note: ordering of arguments to function is taken from, e.g.
cw---       arXiV:0804.4149v3 (App. B) and not hep-ph/0607139 (Eq. 21)
      use KinData_mod
      implicit none
      double precision s,t,psq,renscale2
      double complex F31m,F41mF
      integer ieorder
      common /order/ieorder

      F41m=2d0*(F31m(s,renscale2)+F31m(t,renscale2)-F31m(psq,renscale2)+F41mF(psq,s,t))

      return
      end
************************************************************************
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      double complex function F41mF(psq,s,t)
cw--- note: ordering of arguments to function is taken from, e.g.
cw---       arXiV:0804.4149v3 (App. B) and not hep-ph/0607139 (Eq. 21)
      use KinData_mod
      implicit none
      double precision s,t,psq
      double complex Lsm1
      integer ieorder
      common /order/ieorder

      if(ieorder.eq.-2)then
        F41mF=0d0
      elseif(ieorder.eq.-1)then
        F41mF=0d0
      elseif(ieorder.eq.0)then
        F41mF=Lsm1(-s,-psq,-t,-psq)
      endif

      return
      end
************************************************************************
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      double complex function F42me(psq,qsq,s,t,renscale2)
cw--- note: ordering of arguments to function is taken from, e.g.
cw---       arXiV:0804.4149v3 (App. B) and not hep-ph/0607139 (Eq. 22)
      use KinData_mod
      implicit none
      double precision s,t,psq,qsq,renscale2
      double complex F31m,F42meF
      integer ieorder
      common /order/ieorder
cw--- NOTE: checked on 8/30/09 that this agrees with the expression above
      F42me=2d0*(F31m(s,renscale2)+F31m(t,renscale2)-F31m(psq,renscale2)
     .          -F31m(qsq,renscale2)+F42meF(psq,qsq,s,t))

      return
      end
************************************************************************
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      double complex function F42meF(psq,qsq,s,t)
cw--- note: ordering of arguments to function is taken from, e.g.
cw---       arXiV:0804.4149v3 (App. B) and not hep-ph/0607139 (Eq. 22)
      use KinData_mod
      implicit none
      double precision s,t,psq,qsq
      double complex Lsm1_2me
      integer ieorder
      common /order/ieorder

      if(ieorder.eq.-2)then
        F42meF=0d0
      elseif(ieorder.eq.-1)then
        F42meF=0d0
      elseif(ieorder.eq.0)then
        F42meF=Lsm1_2me(s,t,psq,qsq)
      endif

      return
      end
************************************************************************
      double complex function W(mhsq,s234,s12,s23,s34,s14)
      implicit none
      double complex F41mF_BGMW,F42mhF_BGMW
      double precision mhsq,s234,s12,s23,s34,s14
cw--- note reference to BGMW version of the finite piece of the
cw--- box integrals, which is twice the definition in previous papers
cw--- Note: second term has 3rd and 4th arguments switched
cw---       compared with BGMW paper
      W=F41mF_BGMW(s234,s23,s34)
     . +F42mhF_BGMW(mhsq,s23,s14,s234)
     . +F42mhF_BGMW(mhsq,s34,s12,s234)

      return
      end
************************************************************************
cw--- these are just aliases to previously-defined finite pieces
cw--- of the box functions, but with an additional factor of two
      double complex function F41mF_BGMW(psq,s,t)
      use KinData_mod
      implicit none
      double complex F41mF
      double precision psq,s,t

      F41mF_BGMW=2d0*F41mF(psq,s,t)

      return
      end
************************************************************************
      double complex function F42mhF_BGMW(psq,qsq,s,t)
      use KinData_mod
      implicit none
      double complex F42mhF
      double precision psq,qsq,s,t

      F42mhF_BGMW=2d0*F42mhF(psq,qsq,s,t)

      return
      end
******************************
      double complex function F42mhF(psq,qsq,s,t)
      use KinData_mod
      implicit none

      double precision s,t,psq,qsq
      double complex Lsm1_2mht

      F42mhF=Lsm1_2mht(s,t,psq,qsq)

      return
      end
************************************************************************
      double complex function F33m(p1sq,p2sq,p3sq)
      use KinData_mod
      implicit none
      double precision p1sq,p2sq,p3sq
      double complex I3m,zf33m

      zf33m=I3m(p1sq,p2sq,p3sq)

      F33m=zf33m

      return
      end
************************************************************************
cw--- Combinations of bubble integrals, 0704.3914v3 Eq. (3.21)
cw--- The hatted versions of these functions are further defined
cw--- by Eqs. (3.25), (3.26) and (3.27)
      double complex function BGRL0(s,t)
      use KinData_mod
      implicit none
      double complex lnrat
      double precision s,t
      BGRL0=lnrat(-s,-t)
      return
      end
*******************************
      double complex function BGRL1(s,t)
      use KinData_mod
      implicit none
      double complex lnrat
      double precision s,t
      BGRL1=lnrat(-s,-t)/dcmplx(s-t)
      return
      end
*******************************
      double complex function BGRL2(s,t)
      use KinData_mod
      implicit none
      double complex lnrat
      double precision s,t
      BGRL2=lnrat(-s,-t)/dcmplx(s-t)**2
      return
      end
*******************************
      double complex function BGRL2hat(s,t)
      use KinData_mod
      implicit none
      double complex lnrat
      double precision s,t,ratio,onemratio

      ratio=t/s
      onemratio=abs(1d0-ratio)

      if(onemratio.ge.2d-5)then
      BGRL2hat=lnrat(-s,-t)/dcmplx(s-t)**2
     . -dcmplx(0.5d0*(1d0/s+1d0/t)/(s-t))
      elseif(onemratio.le.2d-5)then
      BGRL2hat=    -(ratio**(-1d0)-1d0)*(6d0*s**2d0)**(-1d0)
     .             -(ratio**(-1d0)-1d0)**2d0*(12d0*s**2d0)**(-1d0)
     .             -(ratio**(-1d0)-1d0)**3d0*(30d0*s**2d0)**(-1d0)
     .             +(ratio**(-1d0)-1d0)**4d0*(60d0*s**2d0)**(-1d0)
     .             -(ratio**(-1d0)-1d0)**5d0*(105d0*s**2d0)**(-1d0)

      endif

      return
      end
*******************************
      double complex function BGRL3(s,t)
      use KinData_mod
      implicit none
      double complex lnrat
      double precision s,t
      BGRL3=lnrat(-s,-t)/dcmplx(s-t)**3
      return
      end
*******************************
      double complex function BGRL3hat(s,t)
      use KinData_mod
      implicit none
      double complex lnrat
      double precision s,t,ratio,onemratio

      ratio=t/s
      onemratio=abs(1d0-ratio)

      if(onemratio.ge.1d-5)then
      BGRL3hat=lnrat(-s,-t)/dcmplx(s-t)**3
     . -dcmplx(0.5d0*(1d0/s+1d0/t)/(s-t)**2)
      elseif(onemratio.le.1d-5)then
      BGRL3hat=    -(6d0*s**3d0)**(-1d0)
     .             -(ratio**(-1d0)-1d0)*(4d0*s**3d0)**(-1d0)
     .             -(ratio**(-1d0)-1d0)**2d0*(20d0*s**3d0)**(-1d0)
     .             +(ratio**(-1d0)-1d0)**3d0*(60d0*s**3d0)**(-1d0)
     .             -(ratio**(-1d0)-1d0)**4d0*(140d0*s**3d0)**(-1d0)

      endif

      return
      end
************************************************************************
************************************************************************
**********************Sub-layer functions*******************************
************************************************************************
************************************************************************
      double precision function htheta(x)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
CW--- define Heaviside theta function (=1 for x>0) and (0 for x < 0)

      htheta=0.5d0+0.5d0*sign(1d0,x)

      return
      end
************************************************************************
      double complex function L0(x,y)
      use KinData_mod
      implicit none
      double complex Lnrat,cone
      double precision x,y,denom
      cone=(1d0,0d0)

      denom=1d0-x/y
      if (abs(denom) .lt. 1d-7) then
      L0=-cone-dcmplx(denom*(0.5d0+denom/3d0))
      else
      L0=Lnrat(x,y)/dcmplx(denom)
      endif
      return
      end
************************************************************************
      double complex function L1(x,y)
      use KinData_mod
      implicit none
      double precision x,y,denom
      double complex L0,cone
      cone=(1d0,0d0)

      denom=1d0-x/y
      if (abs(denom) .lt. 1d-7) then
      L1=-0.5d0*cone-dcmplx(denom/3d0*(1d0+0.75d0*denom))
      else
      L1=(L0(x,y)+cone)/dcmplx(denom)
      endif
      return
      end
************************************************************************
      double complex function L2(x,y)
      use KinData_mod
      implicit none
      double complex Lnrat
      double precision x,y,r,denom

      r=x/y
      denom=1d0-r
      if (abs(denom) .lt. 1d-7) then
      L2=(dcmplx(10d0)+denom*(dcmplx(15d0)+dcmplx(18d0)*denom))
     . /dcmplx(60d0)
      else
      L2=(Lnrat(x,y)-dcmplx(0.5d0*(r-1d0/r)))/dcmplx(denom)**3
      endif
      return
      end
************************************************************************
      double complex function Lnrat(x,y)
************************************************************************
*     Author: R.K. Ellis                                               *
*     August, 1998.                                                    *
c     Lnrat(x,y)=log(x-i*ep)-log(y-i*ep)                               *
c     this function is hard-wired for sign of epsilon we must adjust   *
c     sign of x and y to get the right sign for epsilon                *
************************************************************************
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
CW--- define Heaviside theta function (=1 for x>0) and (0 for x < 0)
      zimpi=(0d0,3.14159265358979311599d0)
      Lnrat=dcmplx(dlog(abs(x/y)))-zimpi*dcmplx((htheta(-x)-htheta(-y)))

      return
      end
************************************************************************
      double complex function zLnrat(x,y)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      double complex Lnrat
      zimpi=(0d0,3.14159265358979311599d0)

      zLnrat=Lnrat(x,y)

      return
      end
************************************************************************
      double complex function Lsm1_2me(s,t,m1sq,m3sq)
      use KinData_mod
      implicit complex*16 (z)
cw---- formula taken from G.~Duplancic and B~Nizic [arXiv:hep-ph/0006249 v2]
cw---- Eq 71
cw---- Lsm1_2me notation follows from
cw----  Z.~Bern, L.~J.~Dixon and D.~A.~Kosower,
cw----  %``Dimensionally regulated pentagon integrals,''
cw----  Nucl.\ Phys.\ B {\bf 412}, 751 (1994)
cw----  [arXiv:hep-ph/9306240].
cw----  %%CITATION = HEP-PH 9306240;%%
cw----  Eqs. (I.13)
Cw---- analytic continuation has been checked by calculating numerically.
      integer j
      double precision s,t,m1sq,m3sq,ddilog,arg(4),omarg(4),f2me,htheta
      double complex Li2(4),wlog(4)
      double precision pi,pisq,pisqo6
      pi=3.14159265358979311599d0
      pisq=pi*pi
      pisqo6=pisq/6d0
      zimpi=(0d0,3.14159265358979311599d0)

CW--- define Heaviside theta function (=1 for x>0) and (0 for x < 0)

      f2me=(s+t-m1sq-m3sq)/(s*t-m1sq*m3sq)

      arg(1)=f2me*s
      arg(2)=f2me*t
      arg(3)=f2me*m1sq
      arg(4)=f2me*m3sq

      do j=1,4
         omarg(j)=1d0-arg(j)
         wlog(j)=log(abs(arg(j)))
     .     +zimpi*dcmplx(htheta(-arg(j))*sign(1d0,f2me))
         if (omarg(j) .gt. 1d0) then
             Li2(j)=dcmplx(pisqo6-ddilog(arg(j)))
     .       -wlog(j)*dcmplx(log(omarg(j)))
          else
             Li2(j)=dcmplx(ddilog(omarg(j)))
         endif
      enddo
      Lsm1_2me=Li2(1)+Li2(2)-Li2(3)-Li2(4)

      return
      end
************************************************************************
      double complex function Lsm1(x1,y1,x2,y2)
      use KinData_mod
      implicit none
      double precision x1,x2,y1,y2,r1,r2,omr1,omr2,ddilog
      double complex dilog1,dilog2,Lnrat
      double precision pi,pisq,pisqo6
      pi=3.14159265358979311599d0
      pisq=pi*pi
      pisqo6=pisq/6d0

      r1=x1/y1
      r2=x2/y2
      omr1=1d0-r1
      omr2=1d0-r2
      if (omr1 .gt. 1d0) then
         dilog1=dcmplx(pisqo6-ddilog(r1))-Lnrat(x1,y1)*dcmplx(log(omr1))
      else
         dilog1=dcmplx(ddilog(omr1))
      endif
      if (omr2 .gt. 1d0) then
         dilog2=dcmplx(pisqo6-ddilog(r2))-Lnrat(x2,y2)*dcmplx(log(omr2))
      else
          dilog2=dcmplx(ddilog(omr2))
      endif
      lsm1=dilog1+dilog2+Lnrat(x1,y1)*Lnrat(x2,y2)-dcmplx(pisqo6)
      return
      end

************************************************************************
      double complex function Lsm1_2mht(s,t,m1sq,m2sq)
      use KinData_mod
      implicit none
      double precision s,t,m1sq,m2sq,ddilog,r1,r2,omr1,omr2
      double complex Lnrat,dilog1,dilog2
      double precision pi,pisq,pisqo6
      pi=3.14159265358979311599d0
      pisq=pi*pi
      pisqo6=pisq/6d0

      r1=m1sq/t
      r2=m2sq/t
      omr1=1d0-r1
      omr2=1d0-r2

      if (omr1 .gt. 1d0) then
      dilog1=dcmplx(pisqo6-ddilog(r1))-Lnrat(-m1sq,-t)*dcmplx(log(omr1))
      else
      dilog1=dcmplx(ddilog(omr1))
      endif
      if (omr2 .gt. 1d0) then
      dilog2=dcmplx(pisqo6-ddilog(r2))-Lnrat(-m2sq,-t)*dcmplx(log(omr2))
      else
      dilog2=dcmplx(ddilog(omr2))
      endif
      lsm1_2mht=-dilog1-dilog2
     & +0.5d0*(Lnrat(-s,-m1sq)*Lnrat(-s,-m2sq)-Lnrat(-s,-t)**2)
      return
      end
************************************************************************
      double complex function I3m(s1,s2,s3)
CW     This is the function I3m, a massless triangle with all three external
CW     lines offshell defined in BDK
CW     %\cite{Bern:1997sc}
CW     \bibitem{Bern:1997sc}
CW     Z.~Bern, L.~J.~Dixon and D.~A.~Kosower,
CW     %``One-loop amplitudes for e+ e- to four partons,''
CW     Nucl.\ Phys.\ B {\bf 513}, 3 (1998)
CW     [arXiv:hep-ph/9708239].
CW     %%CITATION = HEP-PH 9708239;%%
CW     defined in their equation II.9
CW     \int da_1 da_2 da_3 /(-a_1*a_2*s1-a_2*a_3*s2-a_3*a_1*s3)
      implicit none
      double precision s1,s2,s3,smax,smid,smin,del3,rtdel3
      double precision i3m1a,flag
      double complex i3m1b,zf33m

      smax=max(s1,s2,s3)
      smin=min(s1,s2,s3)
      smid=s1+s2+s3-smax-smin
      del3=s1**2+s2**2+s3**2-2d0*(s1*s2+s2*s3+s3*s1)

      if (del3 .gt. 0) then
      rtdel3=sqrt(del3)
         if (smax .lt. 0) then
cw---case all negative
             flag=0d0
             i3m=i3m1b(smax,smid,smin,rtdel3,flag)
         elseif (smin .gt. 0) then
cw---case all positive
             flag=0d0
             i3m=-i3m1b(-smin,-smid,-smax,rtdel3,flag)
         elseif ((smid .lt. 0) .and. (smin .lt. 0)) then
cw---case two negative and one positive
             flag=+1d0
             zf33m=i3m1b(smin,smid,smax,rtdel3,flag)
             i3m=zf33m

         elseif ((smax .gt. 0).and.(smid .gt. 0)) then
cw---case two positive and one negative
             flag=-1d0
             i3m=-i3m1b(-smax,-smid,-smin,rtdel3,flag)
         endif
      elseif (del3 .lt. 0) then
      rtdel3=sqrt(-del3)
         if (smax .lt. 0) then
cw---case all negative
             i3m=+dcmplx(i3m1a(+s1,+s2,+s3,rtdel3))
         elseif (smin .gt. 0) then
cw---case all positive
             i3m=-dcmplx(i3m1a(-s1,-s2,-s3,rtdel3))
          endif
      endif

      return
      end
***********************************
      double precision function I3m1a(s1,s2,s3,rtmdel)
      implicit none
CW     symmetric form of Lu and Perez
CW     %\cite{Lu:1992ny}
cw     \bibitem{Lu:1992ny}
cw     H.~J.~Lu and C.~A.~Perez,
cw     %``Massless one loop scalar three point integral and associated Clausen,
cw     %Glaisher and L functions,''
cw     SLAC-PUB-5809
      double precision s1,s2,s3,d1,d2,d3,rtmdel,arg1,arg2,arg3,dclaus

      d1=s1-s2-s3
      d2=s2-s3-s1
      d3=s3-s1-s2

      arg1=2d0*datan(rtmdel/d1)
      arg2=2d0*datan(rtmdel/d2)
      arg3=2d0*datan(rtmdel/d3)
      i3m1a=2d0/rtmdel*(Dclaus(arg1)+Dclaus(arg2)+Dclaus(arg3))

      end
***********************************
      double complex function I3m1b(s1,s2,s3,rtdel,flag)
      implicit none
CW     form of Ussyukina and Davydychev
CW %\cite{Usyukina:1994iw}
CW \bibitem{Usyukina:1994iw}
CW   N.~I.~Usyukina and A.~I.~Davydychev,
CW   %``New results for two loop off-shell three point diagrams,''
CW  Phys.\ Lett.\ B {\bf 332}, 159 (1994)
CW  [arXiv:hep-ph/9402223].
CW  %%CITATION = HEP-PH 9402223;%%

      double precision s1,s2,s3,d3,temp,ddilog,xlog,ylog,rat,pi
      double precision x,y,rho,rtdel,argx,argy,argdlx,argdly,flag
      double complex impi
      pi=3.14159265358979311599d0
      impi=(0d0,3.14159265358979311599d0)
      d3=s3-s1-s2
      x=s1/s3
      y=s2/s3
      rat=0.5d0*(d3+rtdel)/s3
      if (abs(rat) .lt. 1d-3) rat=2d0*s1*s2/(s3*(d3-rtdel))
      rho=1d0/rat
      argx=rho*x
      argy=rho*y
      argdlx=-argx
      argdly=-argy

      if ((argdlx .gt. 1d0) .or. (argdly .gt. 1d0)) then
      stop
      endif

      xlog=log(abs(argx))
      ylog=log(abs(argy))
      temp=xlog*ylog+pi**2/3d0+(ylog-xlog)*log((1d0+argy)/(1d0+argx))
     & +2d0*(ddilog(argdlx)+ddilog(argdly))
      I3m1b=Dcmplx(temp-abs(flag)*pi**2)+impi*Dcmplx(flag*(xlog+ylog))
      I3m1b=-I3m1b/Dcmplx(rtdel)
      end
************************************************************************
      double complex function Lsm1DS(s,t,msq)
c--- This is an implementation of Eq. (B.2) from DS, 0906.0008
      use KinData_mod
      implicit none
      double precision s,t,msq,r1,r2,omr1,omr2,ddilog,pi,pisq,pisqo6
      double complex dilog1,dilog2,Lnrat
      pi=3.14159265358979311599d0
      pisq=pi*pi
      pisqo6=pisq/6d0

      r1=s/msq
      r2=t/msq
      omr1=1d0-r1
      omr2=1d0-r2
      if (omr1 .gt. 1d0) then
       dilog1=dcmplx(pisqo6-ddilog(r1))
     & -Lnrat(-s,-msq)*dcmplx(log(omr1))
      else
       dilog1=dcmplx(ddilog(omr1))
      endif
      if (omr2 .gt. 1d0) then
       dilog2=dcmplx(pisqo6-ddilog(r2))
     & -Lnrat(-t,-msq)*dcmplx(log(omr2))
      else
       dilog2=dcmplx(ddilog(omr2))
      endif
      Lsm1DS=dilog1+dilog2+Lnrat(-s,-msq)*Lnrat(-t,-msq)-dcmplx(pisqo6)
      return
      end
************************************************************************
************************************************************************
**********************Sub-Sub-layer functions***************************
************************************************************************
************************************************************************
      DOUBLE PRECISION FUNCTION DDILOG(X)
      use KinData_mod

      DOUBLE PRECISION X,Y,T,S,A,PI3,PI6,ZERO,ONE,HALF,MALF,MONE,MTWO
      DOUBLE PRECISION C(0:18),H,ALFA,B0,B1,B2

      DATA ZERO /0.0D0/, ONE /1.0D0/
      DATA HALF /0.5D0/, MALF /-0.5D0/, MONE /-1.0D0/, MTWO /-2.0D0/
      DATA PI3 /3.289868133696453D0/, PI6 /1.644934066848226D0/

      DATA C( 0) / 0.4299669356081370D0/
      DATA C( 1) / 0.4097598753307711D0/
      DATA C( 2) /-0.0185884366501460D0/
      DATA C( 3) / 0.0014575108406227D0/
      DATA C( 4) /-0.0001430418444234D0/
      DATA C( 5) / 0.0000158841554188D0/
      DATA C( 6) /-0.0000019078495939D0/
      DATA C( 7) / 0.0000002419518085D0/
      DATA C( 8) /-0.0000000319334127D0/
      DATA C( 9) / 0.0000000043454506D0/
      DATA C(10) /-0.0000000006057848D0/
      DATA C(11) / 0.0000000000861210D0/
      DATA C(12) /-0.0000000000124433D0/
      DATA C(13) / 0.0000000000018226D0/
      DATA C(14) /-0.0000000000002701D0/
      DATA C(15) / 0.0000000000000404D0/
      DATA C(16) /-0.0000000000000061D0/
      DATA C(17) / 0.0000000000000009D0/
      DATA C(18) /-0.0000000000000001D0/

      IF(X .EQ. ONE) THEN
       DDILOG=PI6
       RETURN
      ELSE IF(X .EQ. MONE) THEN
       DDILOG=MALF*PI6
       RETURN
      END IF
      T=-X
      IF(T .LE. MTWO) THEN
       Y=MONE/(ONE+T)
       S=ONE
       A=-PI3+HALF*(LOG(-T)**2-LOG(ONE+ONE/T)**2)
      ELSE IF(T .LT. MONE) THEN
       Y=MONE-T
       S=MONE
       A=LOG(-T)
       A=-PI6+A*(A+LOG(ONE+ONE/T))
      ELSE IF(T .LE. MALF) THEN
       Y=(MONE-T)/T
       S=ONE
       A=LOG(-T)
       A=-PI6+A*(MALF*A+LOG(ONE+T))
      ELSE IF(T .LT. ZERO) THEN
       Y=-T/(ONE+T)
       S=MONE
       A=HALF*LOG(ONE+T)**2
      ELSE IF(T .LE. ONE) THEN
       Y=T
       S=ONE
       A=ZERO
      ELSE
       Y=ONE/T
       S=MONE
       A=PI6+HALF*LOG(T)**2
      END IF

      H=Y+Y-ONE
      ALFA=H+H
      B1=ZERO
      B2=ZERO
      DO 1 I = 18,0,-1
      B0=C(I)+ALFA*B1-B2
      B2=B1
    1 B1=B0
      DDILOG=-(S*(B0-H*B2)+A)
      RETURN
      END
************************************************************************
*CW $Id: dclaus64.F,v 1.2 1996/04/02 16:23:45 mclareni Exp $
*
*cw $Log: dclaus64.F,v $
*cw Revision 1.2  1996/04/02 16:23:45  mclareni
*cw More precise dclaus64 (C326), test added and C344 removed from TESTALL
*
*cw Revision 1.1.1.1  1996/04/01 15:02:03  mclareni
*cw Mathlib gen
*
      DOUBLE PRECISION FUNCTION DCLAUS(X)
      use KinData_mod
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)

      DIMENSION A(0:8),B(0:13)

      PARAMETER (R1 = 1d0, HF =R1/2d0)
      PARAMETER (PI = 3.14159 26535 89793 24D0)
      PARAMETER (PI2 = 2d0*PI, PIH = PI/2d0, RPIH = 2d0/PI)

      DATA A( 0) / 0.02795 28319 73575 6613D0/
      DATA A( 1) / 0.00017 63088 74389 8116D0/
      DATA A( 2) / 0.00000 12662 74146 1157D0/
      DATA A( 3) / 0.00000 00117 17181 8134D0/
      DATA A( 4) / 0.00000 00001 23006 4129D0/
      DATA A( 5) / 0.00000 00000 01395 2729D0/
      DATA A( 6) / 0.00000 00000 00016 6908D0/
      DATA A( 7) / 0.00000 00000 00000 2076D0/
      DATA A( 8) / 0.00000 00000 00000 0027D0/

      DATA B( 0) / 0.63909 70888 57265 341D0/
      DATA B( 1) /-0.05498 05693 01851 716D0/
      DATA B( 2) /-0.00096 12619 45950 606D0/
      DATA B( 3) /-0.00003 20546 86822 550D0/
      DATA B( 4) /-0.00000 13294 61695 426D0/
      DATA B( 5) /-0.00000 00620 93601 824D0/
      DATA B( 6) /-0.00000 00031 29600 656D0/
      DATA B( 7) /-0.00000 00001 66351 954D0/
      DATA B( 8) /-0.00000 00000 09196 527D0/
      DATA B( 9) /-0.00000 00000 00524 004D0/
      DATA B(10) /-0.00000 00000 00030 580D0/
      DATA B(11) /-0.00000 00000 00001 820D0/
      DATA B(12) /-0.00000 00000 00000 110D0/
      DATA B(13) /-0.00000 00000 00000 007D0/

      V=MOD(ABS(X),PI2)
      S=SIGN(R1,X)
      IF(V .GT. PI) THEN
       V=PI2-V
       S=-S
      ENDIF
      IF(V .EQ. 0d0 .OR. V .EQ. PI) THEN
       H=0d0
      ELSEIF(V .LT. PIH) THEN
       U=RPIH*V
       H=2d0*U**2-1d0
       ALFA=H+H
       B1=0d0
       B2=0d0
       DO 1 I = 8,0,-1
       B0=A(I)+ALFA*B1-B2
       B2=B1
    1  B1=B0
       H=V*(1d0-LOG(V)+HF*V**2*(B0-H*B2))
      ELSE
       U=RPIH*V-2d0
       H=2d0*U**2-1d0
       ALFA=H+H
       B1=0d0
       B2=0d0
       DO 2 I = 13,0,-1
       B0=B(I)+ALFA*B1-B2
       B2=B1
    2  B1=B0
       H=(PI-V)*(B0-H*B2)
      ENDIF
      DCLAUS=S*H
      RETURN
      END


************************************************************************
************************************************************************
************************************************************************
************************************************************************
      double complex function zw1(s,emq)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)

      emq2=emq**2

      rat=4d0*emq2/s
      temp=dsqrt(dabs(1d0/rat))
      if (rat .lt. 0d0) then
          zw1=2d0*dsqrt(1d0-rat)*asinh(temp)
      elseif (rat .gt. 1d0) then
          zw1=2d0*dsqrt(rat-1d0)*asin(temp)
      else
          temp=2d0*acosh(temp)
          zw1=dsqrt(1d0-rat)*dcmplx(temp,-pi)
       endif
      return
      end
************************************************************************
      double complex function zw2(s,emq)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)

      emq2=emq**2

      rat=s/(4d0*emq2)
      tempr=dsqrt(dabs(rat))
      if (rat .lt. 0d0) then
          tempr=asinh(tempr)
          zw2=4d0*tempr**2
      elseif (rat .gt. 1d0) then
          tempr=acosh(tempr)
          tempi=-4d0*tempr*pi
          tempr=+4d0*tempr**2-pi**2
          zw2=dcmplx(tempr,tempi)
      else
          tempr=asin(tempr)
          zw2=-4d0*tempr**2
      endif
      return
      end
************************************************************************
      double complex function zw3(s,t,u,varg,emq)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      zw3=zi3(s,t,u,varg,emq)-zi3(s,t,u,s,emq)-zi3(s,t,u,u,emq)
      return
      end
************************************************************************
      double complex function zi3(s,t,u,varg,emq)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      double complex cli2
      parameter(pi=3.141592653589793238d0)

      emq2=emq**2

      zim=(0d0,1d0)

      rat=4d0*emq2/varg
      if (rat .lt. 0d0) then
           be=0.5d0*(1d0+dsqrt(1d0+4d0*t*emq2/(u*s)))
           ga=0.5d0*(1d0+dsqrt(1d0-rat))
           arg1=ga/(ga+be-1d0)
           arg2=(ga-1d0)/(ga+be-1d0)
           arg3=(be-ga)/be
           arg4=(be-ga)/(be-1d0)
           zi3=2d0/(2d0*be-1d0)
     .     *(-ddilog(arg1)+ddilog(arg2)+ddilog(arg3)-ddilog(arg4)
     .     +0.5d0*(dlog(be)**2-dlog(be-1d0)**2)
     .     +dlog(ga)*dlog((ga+be-1d0)/be)
     .     +dlog(ga-1d0)*dlog((be-1d0)/(ga+be-1d0)))
      elseif (rat .gt. 1d0) then
           be=0.5d0*(1d0+dsqrt(1d0+4d0*t*emq2/(u*s)))
           al=dsqrt(rat-1d0)
           r=dsqrt((al**2+1d0)/(al**2+(2d0*be-1d0)**2))
           arg=r*(al**2+2d0*be-1d0)/(1d0+al**2)
           if (arg .ge. 1d0) then
             phi=0d0
           else
             phi=dacos(arg)
           endif
           arg=r*(al**2-2d0*be+1d0)/(1d0+al**2)
           if (arg .ge. 1d0) then
             theta=0d0
           else
             theta=dacos(arg)
           endif
           zth=r*dcmplx(cos(theta),sin(theta))
           zph=r*dcmplx(cos(phi),sin(phi))
           zi3=2d0/(2d0*be-1d0)
     .     *(2d0*dble(cli2(zth))-2d0*dble(cli2(zph))
     .     +(phi-theta)*(phi+theta-pi))
      else
           be=0.5d0*(1d0+dsqrt(1d0+4d0*t*emq2/(u*s)))
           ga=0.5d0*(1d0+dsqrt(1d0-rat))
           arg1=ga/(ga+be-1d0)
           arg2=(ga-1d0)/(ga+be-1d0)
           arg3=ga/(ga-be)
           arg4=(ga-1d0)/(ga-be)

           zi3=2d0/(2d0*be-1d0)
     .     *(-ddilog(arg1)+ddilog(arg2)+ddilog(arg3)-ddilog(arg4)
     .     +dlog(ga/(1d0-ga))*dlog((ga+be-1d0)/(be-ga))
     .     -zim*pi*dlog((ga+be-1d0)/(be-ga)))
      endif

      return
      end
************************************************************************
      double precision function acosh(y)
      implicit none
      double precision y
      acosh=dlog(y+dsqrt(y**2-1d0))
      return
      end
************************************************************************
      double precision function asinh(y)
      implicit none
      double precision y
      asinh=dlog(y+dsqrt(y**2+1d0))
      return
      end

