      !-------------------------------------------------------------------------------
      ! NNLOJET
      !-------------------------------------------------------------------------------
      !
      ! MODULE: VBF Amplitude C0g VBF
      !
      ! DESCRIPTION:
      !>    Amplitudes for 2q to 2q
      !>    i1 --->--- i4
      !>    i2 --->--- i3
      !-------------------------------------------------------------------------------

      !-------------------------------------------------------------------------------
      module C0gVBF_mod
      !-------------------------------------------------------------------------------
         use commonVBF_mod
         use KinData_mod
!           use commonVBF_mod, only :: propagator_vbf, coupling_vbf
!           use KinData_mod, only :: kin

         implicit none

         public :: C0g0VBF, D0g0VBF

         private 
         integer, parameter :: dp = kind(1.0d0)

         private :: zC0g0VBF
         public :: C0g0VBF_vbf

         integer npars, nspec, n
         common /npars/npars,nspec,n
         integer :: nfC1, nfC2
         common /CZFlav/ nfC1,nfC2
!$omp threadprivate(/CZFlav/)

      contains
         real(dp) function C0g0VBF(i1, i3, i2, i4, i5, i6, iboson)
            integer, intent(in) :: i1, i3, i2, i4, i5, i6, iboson
            real(dp), dimension(4) :: glr, amp
            complex(dp), dimension(4) :: zamp
            real(dp) :: s14, s23, rMcom
            !> 
            !> Base configuration
            !> q(i1) Q(i2) --> Q(i3) q(i4) H(i5,i6)
            !>  Square matrix element of:
            !>  i1 >_____________> i4
            !>         ---- H
            !>  i2 >_____________> i3
            !>

            C0g0VBF = 0d0

            !> Make sure the couplings are attached to the right current
            if (any( (/i1,i4/) < 3 )) then
               call coupling_vbf(nfC1, nfC2, iboson, glr)
            else
               call coupling_vbf(nfC2, nfC1, iboson, glr)
            endif

            !> Create the propagators for the two weak bosns
            s14 = kin(nspec+4)%s(i1, i4)
            s23 = kin(nspec+4)%s(i2, i3)
            call propagator_vbf(s14, s23, iboson, rMcom)

            !> Create the amplitude J(i1,i4)·J(i2,i3)
            call C0g0VBF_vbf(i1,i3,i2,i4,zamp)

            amp = (glr*abs(zamp))**2

            C0g0VBF = 2d0*rMcom*sum(amp)

            return
         end function

         real(dp) function D0g0VBF(i1, i3, i2, i4, i5, i6, iboson)
            integer, intent(in) :: i1, i3, i2, i4, i5, i6, iboson
            real(dp), dimension(4) :: glr
            complex(dp), dimension(4) :: zampA, zampB
            !> Only LL and RR helicity configurations will be allowed
            real(dp), dimension(2) :: amp
            complex(dp), dimension(2) :: zampA_Z,zampB_Z,zampA_W,zampB_W
            complex(dp) :: cMcomA_Z, cMcomB_Z
            complex(dp) :: cMcomA_W, cMcomB_W
            real(dp) :: s14, s23, s24, s13
            !> 
            !> Base configuration
            !> q(i1) q(i2) --> q(i3) q(i4) H(i5,i6)
            !>
            !> Same flavour interference between:
            !>  i1 >_____________> i4
            !>         ---- H            = A
            !>  i2 >_____________> i3
            !> and
            !>  i1 >_____________> i3
            !>         ---- H            = B
            !>  i2 >_____________> i4
            !>

            D0g0VBF = 0d0
            amp(:) = 0d0

            !> Only same-helicity configuration is != 0
            !> ie, only glr(1) = RR and glr(4) = LL are possible
            call coupling_vbf(nfC1, nfC2, iboson, glr)

            !> Generate the interfering amplitudes
            call C0g0VBF_vbf(i1,i3,i2,i4,zampA)            
            call C0g0VBF_vbf(i1,i4,i2,i3,zampB)            

            !> Generate the momentum transfer for the propagators
            !> A
            s14 = kin(nspec+4)%s(i1, i4)
            s23 = kin(nspec+4)%s(i2, i3)
            !> B
            s13 = kin(nspec+4)%s(i1, i3)
            s24 = kin(nspec+4)%s(i2, i4)

            select case(iboson)
            case(0) 
               !> 2 Z bosons, same coupling in both amplitudes
               call c_propagator_vbf(s14, s23, iboson, cMcomA_Z)
               call c_propagator_vbf(s13, s24, iboson, cMcomB_Z)

               zampA_Z(1) = glr(1)*cMcomA_Z*zampA(1)
               zampB_Z(1) = glr(1)*cMcomB_Z*zampB(1)
               zampA_Z(2) = glr(4)*cMcomA_Z*zampA(4)
               zampB_Z(2) = glr(4)*cMcomB_Z*zampB(4)

               amp(:) = real(zampA_Z(:)*dconjg(zampB_Z(:)))

            case(2) 
               !> Z/W interference, necessary to generate (A_WxB_Z + A_ZxB_W)

               !> Z propagator x ZZH
               call c_propagator_vbf(s14, s23, 0, cMcomA_Z)
               call c_propagator_vbf(s13, s24, 0, cMcomB_Z)
               !> W propagator x WWH
               call c_propagator_vbf(s14, s23, 1, cMcomA_W)
               call c_propagator_vbf(s13, s24, 1, cMcomB_W)

               !> Only glr(4) == LL allowed!
               !> and only the Z can appear in the S channel

               !> A_W x B_Z
               if (i1*i4 > 2) then
                  zampA_W(1) =        cMcomA_W*zampA(4)
                  zampB_Z(1) = glr(4)*cMcomB_Z*zampB(4)
                  amp(1) = real(zampA_W(1)*dconjg(zampB_Z(1)))
               endif

               !> A_Z x B_W
               if (i2*i3 > 2) then
                  zampA_Z(2) = glr(4)*cMcomA_Z*zampA(4)
                  zampB_W(2) =        cMcomB_W*zampB(4)
                  amp(2) = real(zampB_W(2)*dconjg(zampA_Z(2)))
               endif

            case default
               print *, "Case ", iboson, " not considered in C0gVBF.f"
               stop
            end select

            D0g0VBF = 2d0*sum(amp)

            return
         end function

         complex(dp) function zC0g0VBF(i1, i3, i2, i4)
            integer, intent(in) :: i1, i2, i3, i4
            ! + + + +

            zC0g0VBF = kin(nspec+4)%zA(i1,i2)*kin(nspec+4)%zB(i3,i4)

            return
         end function zC0g0VBF

         subroutine C0g0VBF_vbf(i1, i3, i2, i4, zamp)
            !> Wrapper over all the possible configurations
            integer, intent(in) :: i1, i2, i3, i4
            complex(8), intent(out) :: zamp(4)

            zamp(1) = zC0g0VBF(i1,i3,i2,i4) ! ++ ++
            zamp(2) = zC0g0VBF(i1,i2,i3,i4) ! ++ --
            zamp(3) = dconjg(zamp(2))
            zamp(4) = dconjg(zamp(1))
!              zamp(3) = zC0g0VBF(i4,i3,i2,i1) ! -- ++
!              zamp(4) = zC0g0VBF(i4,i2,i3,i1) ! -- --

            return
         end subroutine C0g0VBF_vbf

      end module C0gVBF_mod


