      !-------------------------------------------------------------------------------
      ! NNLOJET
      !-------------------------------------------------------------------------------
      !
      ! MODULE: VBF Ampltiude C1g
      !
      ! DESCRIPTION:
      !>    Amplitudes for 2 to 3
      !>    i1-i4, quars ; i5, gluon
      !>           i5
      !>          /
      !>    i1 --->--- i4
      !>    i2 --->--- i3
      !-------------------------------------------------------------------------------

      !-------------------------------------------------------------------------------
      module C1gVBF_mod
         !-------------------------------------------------------------------------------
         use commonVBF_mod
         use KinData_mod
         implicit none

         public :: C1g0VBFs0, C1g0VBF

         integer, parameter :: dp = kind(1.0d0)

         complex(dp), parameter :: I = dcmplx(0d0, 1d0)
         real(dp), parameter :: pi = 3.14159265358979d0
         complex(dp), parameter :: ipi = dcmplx(0d0, pi)
         real(dp), parameter :: b0 = 11d0/6d0

         !> Flag for allowing s channel in gluon initiated
         logical, public :: s_channel = .false.

         public :: C1g0VBF_vbf
         public :: C1g1VBF_vbf
         public :: Ct1g1VBF_vbf

         public :: clearC1gVBF_vbf

         !> Common Blocks
         integer npars, nspec, n
         common /npars/npars,nspec,n
         integer :: nfC1, nfC2
         common /CZFlav/ nfC1,nfC2
!$omp threadprivate(/CZFlav/)

         private 

      contains
         !-----------------------------------------------------------------------------
         !> @brief
         !>
         !> Public interfaces for 4 q 1 gluon VBF Matrix Elements
         !>
         !-----------------------------------------------------------------------------
         real(dp) function C1g0VBF(i1,i5,i3,i2,i4,i6,i7,iboson)
            integer, intent(in) :: i1,i5,i3,i2,i4,i6,i7, iboson
            real(dp) :: r1, r2
!              integer si1, si2, si, sj, sg1, sg2, si7
!              common /spiketesting/ si1,si2,si,sj,sg1,sg2,si7
!              if (si1.eq.0) then
!                 si1 = i1 ; si2 = i2 ; si = i4 ; sj = i3; 
!                 sg1 = i5 ; sg2 = 99 ; si7 = 0
!              endif
            !
            ! q(i1) + Q(i2) ----> Q(i3) q(i4) + g(i5) + H(i6,i7)
            ! Sum over J emitted by q and Q
            !

            r1 = 0d0
            r2 = 0d0

            ! For gluon initiated:
            ! Base sit       i3
            !               /
            !              /
            ! i5 ccccccccc-------- i2
            ! i1 ----------------- i4
            !

            !> gluon s channel only (debug)
!              if ( any( (/i1,i4/) < 3 ) ) then
!                 r1 = C1g0VBFs0(i1,i5,i3,i2,i4,i6,i7,iboson)
!              else
!                 r2 = C1g0VBFs0(i2,i5,i4,i1,i3,i6,i7,iboson)
!              endif


            if ((i5 > 2).or.(s_channel)) then
               r1 = C1g0VBFs0(i1,i5,i3,i2,i4,i6,i7,iboson)
               r2 = C1g0VBFs0(i2,i5,i4,i1,i3,i6,i7,iboson)
            else
               if (any( (/i1,i4/) < 3 )) then
                  r2 = C1g0VBFs0(i2,i5,i4,i1,i3,i6,i7,iboson, .true.)
               else 
                  r1 = C1g0VBFs0(i1,i5,i3,i2,i4,i6,i7,iboson, .false.)
               endif
            endif

            C1g0VBF = r1 + r2


            return
         end function
         real(dp) function C1g0VBFs0(i1,i5,i3,i2,i4,i6,i7,iboson, swapf)

            integer, intent(in) :: i1,i5,i3,i2,i4,i6,i7,iboson
            logical, intent(in), optional :: swapf
            real(dp), dimension(4) :: glr
            real(dp) :: s14, s15, s45, q1, s23, rMcom
            complex(dp), dimension(2, 4) :: zamp
            real(dp), dimension(2,4) :: amp
            integer :: ii
            !> 
            !> Base configuration
            !> q(i1) Q(i2) --> Q(i3) q(i4) g(i5) H(i6,i7) with i5
            !> emitted from the q(i1---i4) line
            !>  Square matrix element of:
            !>           / i5
            !>  i1 >_____________> i4
            !>         ---- H
            !>  i2 >_____________> i3
            !>

            C1g0VBFs0 = 0d0

            !> Make sure the couplings are attached to the right current
            if (any( (/i1,i4/) == 1 )) then
!              if (swapf) then
               call coupling_vbf(nfC1, nfC2, iboson, glr)
            else
               call coupling_vbf(nfC2, nfC1, iboson, glr)
            endif

            !> Create the propagators for the two weak bosns
            s14    = kin(nspec+5)%s(i1, i4)
            s15    = kin(nspec+5)%s(i1, i5)
            s45    = kin(nspec+5)%s(i4, i5)
            q1     = s14 + s15 + s45

            s23    = kin(nspec+5)%s(i2, i3)
            call propagator_vbf(q1, s23, iboson, rMcom)

            !> Create the amplitude J(i1,(i5),i4)·J(i2,i3)
            call C1g0VBF_vbf(i1,i5,i3,i2,i4,zamp)

            do ii = 1,4
               zamp(:,ii) = zamp(:,ii)*glr(ii)
            enddo

            amp = abs(zamp)**2

            C1g0VBFs0 = rMcom*(sum(amp))

            return
         end function

         !-----------------------------------------------------------------------------
         !> @brief
         !
         !> @param[in] i1-i5 initial partons id
         !> @param[out] zamp(2,4) amplitude
         !>                zamp(:,1) RR (i1-i4 ; i2-i3)
         !>                zamp(:,2) RL
         !>                zamp(:,3) LR
         !>                zamp(:,4) LL
         !>                zamp(1,:) helicity + gluon
         !>                zamp(2,:) helicity - gluon
         !-----------------------------------------------------------------------------
         subroutine C1g0VBF_vbf(i1, i5, i3, i2, i4, zamp)
            integer, intent(in) :: i1, i2, i3, i4, i5
            complex(dp), intent(out) :: zamp(2,4)

c           if(ampl_C1gVBF(i1,i5,i3,i2,i4)%set)then
c              zamp = ampl_C1gVBF(i1,i5,i3,i2,i4)%s
c              return
c           endif

            zamp(1,1) = zC1g0VBFp(i1,i5,i3,i2,i4)
            zamp(2,1) = zC1g0VBFm(i1,i5,i3,i2,i4)
            ! + + - -
            zamp(1,2) = zC1g0VBFp(i1,i5,i2,i3,i4)
            zamp(2,2) = zC1g0VBFm(i1,i5,i2,i3,i4)
            ! - - + +
            zamp(1,3) = zC1g0VBFp(i4,i5,i3,i2,i1)
            zamp(2,3) = zC1g0VBFm(i4,i5,i3,i2,i1)
            ! - -  - -
            zamp(1,4) = zC1g0VBFp(i4,i5,i2,i3,i1)
            zamp(2,4) = zC1g0VBFm(i4,i5,i2,i3,i1)

c           ampl_C1gVBF(i1,i5,i3,i2,i4)%s = zamp
c
c           ! Point to .true.
c           if(flags%a)then
c              ampl_C1gVBF(i1,i5,i3,i2,i4)%set => flags%a
c           elseif(flags%b)then
c              ampl_C1gVBF(i1,i5,i3,i2,i4)%set => flags%b
c           endif
c           flags%valid = .true.

            return
         end subroutine C1g0VBF_vbf

         subroutine C1g1VBF_vbf(i1, i5, i3, i2, i4, renscale2, zamp)
            integer, intent(in) :: i1, i2, i3, i4, i5
            real(dp), intent(in) :: renscale2
            complex(dp), intent(out) :: zamp(2,4)
            integer :: i
            complex(dp) zfact(4)

            call alphabetag(i4,i1,i5,renscale2,zfact)
            zamp(1,1) = zC1g1VBFp(i1,i5,i3,i2,i4,zfact)
            zamp(1,2) = zC1g1VBFp(i1,i5,i2,i3,i4,zfact)
            zamp(2,3) = zC1g1VBFm(i4,i5,i3,i2,i1,zfact)
            zamp(2,4) = zC1g1VBFm(i4,i5,i2,i3,i1,zfact)

            call alphabetag(i1,i4,i5,renscale2,zfact)
            zamp(2,1) = zC1g1VBFm(i1,i5,i3,i2,i4,zfact)
            zamp(2,2) = zC1g1VBFm(i1,i5,i2,i3,i4,zfact)
            zamp(1,3) = zC1g1VBFp(i4,i5,i3,i2,i1,zfact)
            zamp(1,4) = zC1g1VBFp(i4,i5,i2,i3,i1,zfact)

            return
         end subroutine C1g1VBF_vbf

         subroutine Ct1g1VBF_vbf(i1, i5, i3, i2, i4, renscale2, zamp)
            integer, intent(in) :: i1, i2, i3, i4, i5
            real(dp), intent(in) :: renscale2
            complex(dp), intent(out) :: zamp(2,4)
            complex(dp) zfact(4)

            call alphabetagSL(i4,i1,i5,renscale2,zfact)
            zamp(1,1) = zC1g1VBFp(i1,i5,i3,i2,i4,zfact)
            zamp(1,2) = zC1g1VBFp(i1,i5,i2,i3,i4,zfact)
            zamp(2,3) = zC1g1VBFm(i4,i5,i3,i2,i1,zfact)
            zamp(2,4) = zC1g1VBFm(i4,i5,i2,i3,i1,zfact)
            call alphabetagSL(i1,i4,i5,renscale2,zfact)
            zamp(2,1) = zC1g1VBFm(i1,i5,i3,i2,i4,zfact)
            zamp(2,2) = zC1g1VBFm(i1,i5,i2,i3,i4,zfact)
            zamp(1,3) = zC1g1VBFp(i4,i5,i3,i2,i1,zfact)
            zamp(1,4) = zC1g1VBFp(i4,i5,i2,i3,i1,zfact)

            return
         end subroutine Ct1g1VBF_vbf

         !-----------------------------------------------------------------------------
         !> @brief
         !> Private functions for 1 gluon no loops, 1 gluon 1 loop
         !> and coefficients for the 1 loop amplitude
         !>
         !> @param[in] i1-i5 initial partons id
         !>
         !>
         !-----------------------------------------------------------------------------

         pure complex(dp) function zC1g0VBFp(i1,i5,i3,i2,i4)
            integer, intent(in) :: i1, i2, i3, i4, i5
            complex(dp) :: zd, zu, z1, z2
            ! + + + +  g+
            zd = kin(nspec+5)%zA(i4,i5)*kin(nspec+5)%zA(i1,i5)
            zu = sqrt(2d0)*kin(nspec+5)%zA(i1,i2)
            z1 = kin(nspec+5)%zA(i1,i4)*kin(nspec+5)%zB(i3,i4)
            z2 = kin(nspec+5)%zA(i1,i5)*kin(nspec+5)%zB(i3,i5)

            zC1g0VBFp = zu*(z1+z2)/zd

            return
         end function zC1g0VBFp

         pure complex(dp) function zC1g0VBFm(i1,i5,i3,i2,i4)
            integer, intent(in) :: i1, i2, i3, i4, i5
            complex(dp) :: zd, zu, z1, z2
            ! + + + +  g-
            zd = kin(nspec+5)%zB(i4,i5)*kin(nspec+5)%zB(i1,i5)
            zu = sqrt(2d0)*kin(nspec+5)%zB(i4,i3)
            z1 = kin(nspec+5)%zB(i1,i4)*kin(nspec+5)%zA(i1,i2)
            z2 = kin(nspec+5)%zB(i4,i5)*kin(nspec+5)%zA(i2,i5)

            zC1g0VBFm = zu*(z1+z2)/zd

            return
         end function zC1g0VBFm

         pure complex(dp) function zC1g1VBFp(i1,i5,i3,i2,i4,zfact)
            integer, intent(in) :: i1, i2, i3, i4, i5
            complex(dp), intent(in)  :: zfact(4)
            complex(dp) :: zup, zd1, zd3, zd4
            complex(dp) :: za(4), ztotal(4)
            ! g +

            za(:) = 0d0

            zup   = kin(nspec+5)%zA(i1,i2)
            zd1   = kin(nspec+5)%zA(i4,i5)*kin(nspec+5)%zA(i1,i5)/zup
            za(1) = kin(nspec+5)%zA(i1,i4)*kin(nspec+5)%zB(i3,i4)/zd1
            za(2) = kin(nspec+5)%zA(i1,i5)*kin(nspec+5)%zB(i3,i5)/zd1

            zd3    = kin(nspec+5)%zA(i4,i5)*kin(nspec+5)%zB(i1,i5)
            za(3)  = kin(nspec+5)%zA(i4,i2)*kin(nspec+5)%zB(i4,i5)*kin(nspec+5)%zB(i5,i3)/zd3

            zd4    =  kin(nspec+5)%zA(i4,i5)*kin(nspec+5)%zB(i1,i4)/kin(nspec+5)%zB(i4,i5)
            za(4)  = (kin(nspec+5)%zA(i1,i2)*kin(nspec+5)%zB(i1,i3)
     &          + kin(nspec+5)%zA(i2,i4)*kin(nspec+5)%zB(i3,i4)
     &          + kin(nspec+5)%zA(i2,i5)*kin(nspec+5)%zB(i3,i5))/zd4

            ztotal    = (zfact*za)
            zC1g1VBFp = sqrt(2d0)*sum(ztotal)

            return
         end function zC1g1VBFp

         pure complex(dp) function zC1g1VBFm(i4,i5,i2,i3,i1,zfact)
            integer, intent(in) :: i1, i2, i3, i4, i5
            complex(dp), intent(in)  :: zfact(4)
            complex(dp) :: zup, zd1, zd3, zd4
            complex(dp) :: za(4), ztotal(4)
            ! g -

            za(:) = 0d0

            zup   = kin(nspec+5)%zB(i1,i2)
            zd1   = kin(nspec+5)%zB(i1,i5)*kin(nspec+5)%zB(i4,i5)/zup
            za(1) = kin(nspec+5)%zB(i4,i1)*kin(nspec+5)%zA(i4,i3)/zd1
            za(2) = kin(nspec+5)%zB(i1,i5)*kin(nspec+5)%zA(i3,i5)/zd1

            zd3    = kin(nspec+5)%zB(i4,i5)*kin(nspec+5)%zA(i1,i5)
            za(3)  = kin(nspec+5)%zB(i4,i2)*kin(nspec+5)%zA(i4,i5)*kin(nspec+5)%zA(i5,i3)/zd3

            zd4    =  kin(nspec+5)%zB(i4,i5)*kin(nspec+5)%zA(i1,i4)/kin(nspec+5)%zA(i4,i5)
            za(4)  = (kin(nspec+5)%zB(i1,i2)*kin(nspec+5)%zA(i1,i3)
     &          + kin(nspec+5)%zB(i2,i4)*kin(nspec+5)%zA(i3,i4)
     &          + kin(nspec+5)%zB(i2,i5)*kin(nspec+5)%zA(i3,i5))/zd4

            ztotal    = (zfact*za)
            zC1g1VBFm = sqrt(2d0)*sum(ztotal)

            return
         end function zC1g1VBFm

         subroutine alphabetag(i1,i2,i3,renscale2,zfact)
            integer, intent(in) :: i1, i2, i3
            real(dp), intent(in) :: renscale2
            complex(dp), intent(out) :: zfact(4)
            complex(dp) :: zzlog, zH2, zzlograt
            complex(dp) :: zalpha, zbeta, zgamma, zdelta, zdls123
            complex(dp) :: zH0y, zdls13
            complex(dp) :: zH0z, zdls23
            complex(dp) :: zzL0
            real(dp) :: s12, s13, s23, s123, ry, rz
            integer :: ieorder
            common/order/ieorder

            zalpha   = (0d0,0d0)
            zbeta    = (0d0,0d0)
            zgamma   = (0d0,0d0)
            zdelta   = (0d0,0d0)
            zfact(:) = (0d0,0d0)

            s12  = kin(nspec+5)%s(i1,i2)
            s13  = kin(nspec+5)%s(i1,i3)
            s23  = kin(nspec+5)%s(i2,i3)
            s123 = s12 + s23 + s13

            if (s123 > 0) then
               if (s13 > 0) then
                  zdls13  = zzlograt(renscale2,(-s13))
                  zdls23  = zzlograt(renscale2,(-s23))
               else
                  if (s13 > s23) then
                     zdls13  = zzlograt(renscale2,(-s13))
                     zdls23  = zzlograt(renscale2,(-s23))
                  else
                     zdls13  = zzlograt(-renscale2,s13)
                     zdls23  = zzlograt(-renscale2,s23)
                  endif
               endif
               zdls123 = zzlograt(renscale2,(-s123))
            else
               zdls13  = zzlograt(renscale2,(-s13))
               zdls23  = zzlograt(renscale2,(-s23))
               if (s13 > s23) then
                  zdls123 = zzlograt(renscale2,-s123)
               else
                  zdls123 = zzlograt(-renscale2,s123)
               endif
            endif

            zH0y = zzlograt(-s13,-s123)
            zH0z = zzlograt(-s23,-s123)
            zzL0 = zzL2(-s123,-s23,-s13)

            !ALPHA
            if(ieorder.eq.-2)then
               zalpha = -1
            elseif(ieorder.eq.-1)then
               zalpha = -1.0d0/2.0d0*b0 + (1.0d0/2.0d0)*zH0y + (1.0d0/2.0d0)*zH0z
     .   - zdls123 - 3.0d0/4.0d0
            elseif(ieorder.eq.0)then
               zalpha = -1.0d0/4.0d0*s23**2*zdls123/(s12 + s13)**2 + (1.0d0/4.0d0
     .   )*s23**2*zdls23/(s12 + s13)**2 - s23*zdls123/(s12 + s13) + s23*
     .   zdls23/(s12 + s13) - 1.0d0/4.0d0*s23/(s12 + s13) + (1.0d0/2.0d0)*
     .   zH0y*zdls123 + (1.0d0/2.0d0)*zH0z*zdls123 - 1.0d0/2.0d0*zdls123**
     .   2d0 - 3.0d0/4.0d0*zdls123 - 1.0d0/4.0d0*zzL0 - 2d0 + (1.0d0/12.0d0)*
     .   pi**2 - 1.0d0/4.0d0*s13*s23*zH0y/s12**2 - 1.0d0/4.0d0*s13*s23*
     .   zH0z/s12**2 + (1.0d0/2.0d0)*s13*s23*zdls123/s12**2 - 1.0d0/4.0d0*
     .   s13*s23*zdls13/s12**2 - 1.0d0/4.0d0*s13*s23*zdls23/s12**2
            endif

            !BETA
            if(ieorder.eq.-2)then
               zbeta = -1
            elseif(ieorder.eq.-1)then
               zbeta = -1.0d0/2.0d0*b0 + (1.0d0/2.0d0)*zH0y + (1.0d0/2.0d0)*zH0z
     .   - zdls123 - 3.0d0/4.0d0
            elseif(ieorder.eq.0)then
               zbeta = -1.0d0/2.0d0*s12**2*zdls123/(s12 + s13)**2 + (1.0d0/2.0d0)
     .   *s12**2*zdls23/(s12 + s13)**2 + (1.0d0/2.0d0)*s12*s23*zdls123/(
     .   s12 + s13)**2 - 1.0d0/2.0d0*s12*s23*zdls23/(s12 + s13)**2 + s12*
     .   zdls123/(s12 + s13) - s12*zdls23/(s12 + s13) + (1.0d0/2.0d0)*s13
     .   **2*zdls123/(s12 + s13)**2 - 1.0d0/2.0d0*s13**2*zdls23/(s12 + s13
     .   )**2 + (1.0d0/2.0d0)*s13*s23*zdls123/(s12 + s13)**2 - 1.0d0/2.0d0
     .   *s13*s23*zdls23/(s12 + s13)**2 - 1.0d0/4.0d0*s23*zdls123/(s12 +
     .   s13) + (1.0d0/4.0d0)*s23*zdls23/(s12 + s13) + (1.0d0/2.0d0)*zH0y*
     .   zdls123 + (1.0d0/2.0d0)*zH0z*zdls123 - 1.0d0/2.0d0*zdls123**2 -
     .   1.0d0/4.0d0*zdls123 - 1.0d0/2.0d0*zdls23 - 1.0d0/4.0d0*zzL0 -
     .   3.0d0/2.0d0 + (1.0d0/12.0d0)*pi**2 + (1.0d0/4.0d0)*s23*zH0y/s12 +
     .   (1.0d0/4.0d0)*s23*zH0z/s12 - 1.0d0/2.0d0*s23*zdls123/s12 + (1.0d0
     .   /4.0d0)*s23*zdls13/s12 + (1.0d0/4.0d0)*s23*zdls23/s12 - 1.0d0/
     .   4.0d0*s13*s23*zH0y/s12**2 - 1.0d0/4.0d0*s13*s23*zH0z/s12**2 + (
     .   1.0d0/2.0d0)*s13*s23*zdls123/s12**2 - 1.0d0/4.0d0*s13*s23*zdls13/
     .   s12**2 - 1.0d0/4.0d0*s13*s23*zdls23/s12**2
            endif

            !GAMMA
            if(ieorder.eq.-2)then
               zgamma = 0
            elseif(ieorder.eq.-1)then
               zgamma = 0
            elseif(ieorder.eq.0)then
               zgamma = (1.0d0/4.0d0)*s23**2*zdls123/(s12 + s13)**2 - 1.0d0/4.0d0
     .   *s23**2*zdls23/(s12 + s13)**2 + (1.0d0/4.0d0)*s23*zdls123/(s12 +
     .   s13) - 1.0d0/4.0d0*s23*zdls23/(s12 + s13) + (1.0d0/4.0d0)*s23/(
     .   s12 + s13) - 1.0d0/4.0d0*s23*zH0y/s12 - 1.0d0/4.0d0*s23*zH0z/s12
     .   + (1.0d0/2.0d0)*s23*zdls123/s12 - 1.0d0/4.0d0*s23*zdls13/s12 -
     .   1.0d0/4.0d0*s23*zdls23/s12
            endif

            !DELTA
            if(ieorder.eq.-2)then
               zdelta = 0
            elseif(ieorder.eq.-1)then
               zdelta = 0
            elseif(ieorder.eq.0)then
               zdelta = -1.0d0/4.0d0*s12*s23*zdls123/(s12 + s13)**2 + (1.0d0/
     .   4.0d0)*s12*s23*zdls23/(s12 + s13)**2 - 1.0d0/2.0d0*s12*zdls123/(
     .   s12 + s13) + (1.0d0/2.0d0)*s12*zdls23/(s12 + s13) - 1.0d0/4.0d0*
     .   s12/(s12 + s13)
            endif


            zfact(1) = 2d0*zalpha
            zfact(2) = 2d0*zbeta
            zfact(3) = 2d0*zgamma
            zfact(4) = 2d0*zdelta

            return
         end subroutine alphabetag

         subroutine alphabetagSL(i1,i2,i3,renscale2,zfact)
            integer, intent(in) :: i1, i2, i3
            real(dp), intent(in) :: renscale2
            complex(dp), intent(out) :: zfact(4)
            complex(dp) :: zzlog, zH2, zt1, zt2, zt3, zzlograt
            complex(dp) :: zalpha, zbeta, zgamma, zdelta, zdls123
            complex(dp) :: zH1x, zH0x, zH01x, zrx, zdls12
            complex(dp) :: zH1y, zH0y, zH01y, zry, zdls13
            complex(dp) :: zH1z, zH0z, zH01z, zrz, zdls23
            real(dp) :: s12, s13, s23, s123, ry, rz,rx
            integer :: ieorder
            common/order/ieorder

            zfact(:) = (0d0,0d0)
            zalpha   = (0d0,0d0)
            zbeta    = (0d0,0d0)
            zgamma   = (0d0,0d0)
            zdelta   = (0d0,0d0)
            zH01x    = (0d0,0d0)
            zH01y    = (0d0,0d0)
            zH01z    = (0d0,0d0)

            s12  = kin(nspec+5)%s(i1,i2)
            s13  = kin(nspec+5)%s(i1,i3)
            s23  = kin(nspec+5)%s(i2,i3)
            s123 = s12 + s23 + s13

            zdls12  = zzlograt(renscale2,(-s12))

            if (s123 > 0) then
               if (s13 > 0) then
                  zdls13  = zzlograt(renscale2,(-s13))
                  zdls23  = zzlograt(renscale2,(-s23))
               else
                  if (s13 > s23) then
                     zdls13  = zzlograt(renscale2,(-s13))
                     zdls23  = zzlograt(renscale2,(-s23))
                  else
                     zdls13  = zzlograt(-renscale2,s13)
                     zdls23  = zzlograt(-renscale2,s23)
                  endif
               endif
               zdls123 = zzlograt(renscale2,(-s123))
            else
               zdls13  = zzlograt(renscale2,(-s13))
               zdls23  = zzlograt(renscale2,(-s23))
               if (s13 > s23) then
                  zdls123 = zzlograt(renscale2,-s123)
               else
                  zdls123 = zzlograt(-renscale2,s123)
               endif
            endif

            rx = s12/s123
            ry = s13/s123
            rz = s23/s123

            zrx = dcmplx(rx, 0d0)
            zry = dcmplx(ry, 0d0)
            zrz = dcmplx(rz, 0d0)

            zH01x =  zH2(0,1,zrx)
            zH0x  =  zzlog(rx)
            zH1x  = -zzlog(1d0-rx)

            zH01y =  zH2(0,1,zry)
            zH0y  =  zzlog(ry)
            zH1y  = -zzlog(1d0-ry)

            zH01z =  zH2(0,1,zrz)
            zH0z  =  zzlog(rz)
            zH1z  = -zzlog(1d0-rz)

            ! Collect terms for numerical stability in the singular limits
            ! any small changes break this stability, needs more work
            ! (only works for s12 > s123)
            !
            zt1 = zH1z - zdls12 + zdls123
            zt2 = pi*pi/6d0 - (zH01z+zH01x) - zH1x*(zdls12 - zdls123)
            zt3 = zt2 - zdls23*zt1 + zdls123*zt1

            !ALPHA
            if(ieorder.eq.-2)then
               zalpha = 1.0d0/2.0d0
            elseif(ieorder.eq.-1)then
               zalpha = zdls12/2d0 + 3.0d0/4.0d0
            elseif(ieorder.eq.0)then
               zalpha = s23**2*zdls123/(4d0*(s12 + s13)**2) - s23**2*zdls23/(4d0*(s12
     .   + s13)**2) + s23*zdls123/(s12 + s13) - s23*zdls23/(s12 + s13) +
     .   s23/(4d0*(s12 + s13)) - zH01x/4d0 - zH01y/2d0 + zH01z/4d0 - zH1x*zdls12/4
     .   + zH1x*zdls123/4d0 + zH1y*zdls123/2d0 - zH1y*zdls13/2d0 + zdls12**2/4d0 -
     .   zdls12*zdls123/2d0 + zdls12*zdls13/2d0 + zdls123**2/2d0 - zdls123*
     .   zdls13/2d0 - zdls123*zt1/4d0 + 3d0*zdls123/4d0 + zdls23*zt1/4d0 + 3d0*zt3/4d0 +
     .   2d0 + s23**2*zdls123/(2d0*s13*(s12 + s13)) - s23**2*zdls23/(2d0*s13*(
     .   s12 + s13)) - s23*zdls12/(2d0*s13) + s23*zdls123/(2d0*s13) + s23*zt3/
     .   s13 + s23**2*zt3/(2d0*s13**2)
            endif

            !BETA
            if(ieorder.eq.-2)then
               zbeta = 1.0d0/2.0d0
            elseif(ieorder.eq.-1)then
               zbeta = zdls12/2d0 + 3.0d0/4.0d0
            elseif(ieorder.eq.0)then
               zbeta = s12**2*zdls123/(2d0*(s12 + s13)**2) - s12**2*zdls23/(2d0*(s12
     .   + s13)**2) + s12*s13*zdls12/(2d0*(s13 + s23)**2) - s12*s13*zdls123/
     .   (2d0*(s13 + s23)**2) - s12*s23*zdls123/(2d0*(s12 + s13)**2) + s12*s23
     .   *zdls23/(2d0*(s12 + s13)**2) - s12*zdls123/(s12 + s13) + s12*zdls23
     .   /(s12 + s13) - s12*s23*zdls123/(2d0*s13*(s12 + s13)) + s12*s23*
     .   zdls23/(2d0*s13*(s12 + s13)) + s12*zdls12/(2d0*s13) - s12*zdls123/(2d0*
     .   s13) - s12*zt3/s13 - s12*s23*zt3/(2d0*s13**2) - s13**2*zdls123/(2d0*(
     .   s12 + s13)**2) + s13**2*zdls23/(2d0*(s12 + s13)**2) - s13*s23*
     .   zdls123/(2d0*(s12 + s13)**2) + s13*s23*zdls23/(2d0*(s12 + s13)**2) -
     .   s23*zdls12/(2d0*(s13 + s23)) + s23*zdls123/(2d0*(s13 + s23)) - s23*
     .   zdls123/(4d0*(s12 + s13)) + s23*zdls23/(4d0*(s12 + s13)) + s23/(2d0*(
     .   s13 + s23)) - zH01x/4d0 - zH01y/2d0 + zH01z/4d0 - zH1x*zdls12/4d0 + zH1x*
     .   zdls123/4d0 + zH1y*zdls123/2d0 - zH1y*zdls13/2d0 + zdls12**2/4d0 - zdls12
     .   *zdls123/2d0 + zdls12*zdls13/2d0 + zdls12 + zdls123**2/2d0 - zdls123*
     .   zdls13/2d0 - zdls123*zt1/4d0 - 3d0*zdls123/4d0 + zdls23*zt1/4d0 + zdls23/2
     .   - zt3/4d0 + 3.0d0/2.0d0 - s23*zt3/(2d0*s13)
            endif

            !GAMMA
            if(ieorder.eq.-2)then
               zgamma = 0
            elseif(ieorder.eq.-1)then
               zgamma = 0
            elseif(ieorder.eq.0)then
               zgamma = -s12*s13*zdls12/(2d0*(s13 + s23)**2) + s12*s13*zdls123/(2d0*(
     .   s13 + s23)**2) - s12*s23*zdls123/(2d0*s13*(s12 + s13)) + s12*s23*
     .   zdls23/(2d0*s13*(s12 + s13)) + s12*zdls12/(2d0*s13) - s12*zdls123/(2d0*
     .   s13) - s12*s23*zt3/(2d0*s13**2) - s23**2*zdls123/(4d0*(s12 + s13)**2
     .   ) + s23**2*zdls23/(4d0*(s12 + s13)**2) + s23*zdls12/(2d0*(s13 + s23
     .   )) - s23*zdls123/(2d0*(s13 + s23)) - 3d0*s23*zdls123/(4d0*(s12 + s13))
     .   + 3d0*s23*zdls23/(4d0*(s12 + s13)) - s23/(2d0*(s13 + s23)) - s23/(4d0*(
     .   s12 + s13)) - s23**2*zdls123/(2d0*s13*(s12 + s13)) + s23**2*zdls23/
     .   (2d0*s13*(s12 + s13)) + s23*zdls12/(2d0*s13) - s23*zdls123/(2d0*s13) -
     .   s23*zt3/(2d0*s13) - s23**2*zt3/(2d0*s13**2)
            endif

            !DELTA
            if(ieorder.eq.-2)then
               zdelta = 0
            elseif(ieorder.eq.-1)then
               zdelta = 0
            elseif(ieorder.eq.0)then
               zdelta = s12*s23*zdls123/(4d0*(s12 + s13)**2) - s12*s23*zdls23/(4d0*(
     .   s12 + s13)**2) + s12*zdls123/(2d0*(s12 + s13)) - s12*zdls23/(2d0*(s12
     .   + s13)) + s12/(4d0*(s12 + s13)) + s12*s23*zdls123/(2d0*s13*(s12 + s13
     .   )) - s12*s23*zdls23/(2d0*s13*(s12 + s13)) - s12*zdls12/(2d0*s13) +
     .   s12*zdls123/(2d0*s13) + s12*zt3/(2d0*s13) + s12*s23*zt3/(2d0*s13**2)
            endif


            zfact(1)  = -2d0*zalpha
            zfact(2)  = -2d0*zbeta
            zfact(3)  = -2d0*zgamma
            zfact(4)  = -2d0*zdelta

            return
         end subroutine alphabetagSL

         complex(dp) function zzL0(s123,s13,s23)
            real(dp), intent(in) :: s123,s13,s23
            real(dp) :: dli2, r1, r2
            complex(dp) :: zt1, zt2, zt3
            complex(dp) :: zdilog1, zdilog2
            complex(dp) :: zzlograt
            r1 = s123/s13
            r2 = s123/s23
            zdilog1 = dli2(1d0-r1)
            zdilog2 = dli2(1d0-r2)
            zt1 = -dcmplx(pi*pi/3d0,0d0)
            zt2 = -zzlograt(-s23,-s13)**2
            zt3 = -2d0*zdilog1 - 2d0*zdilog2
            zzL0 = zt1 + zt2 + zt3
            zzL0 = -zzL0
         end function zzL0

         complex(dp) function zzL2(s123, s13, s23)
            real(dp), intent(in) :: s123,s13,s23
            real(dp) :: y, z, r1, r2
            complex(dp) :: zt1, zt2, zt3
            complex(dp) :: zzlograt
            y    =  s13/s123
            z    =  s23/s123
            r1   =  1d0 - y
            r2   =  1d0 - z
            zt1  =  (zzlograt(s13,s123) + zzlograt(s23,s123))**2
            zt2  =  dcmplx(pi*pi/3d0)
            zt3  = 2d0*(zzli2m1(s13,s123) + zzli2m1(s23,s123))
            zzL2 =  zt1-zt2+zt3
         end function zzL2

         complex(dp) function zzli2m1(x,y) ! Li2 (1 - x/y)
            real(dp), intent(in) :: x, y
            real(dp) :: r, dli2
            complex(dp) :: zt1, zt2, zt3, zzlograt
            r       =  1 - x/y
            zt1     =  dcmplx(dli2(r),0d0)
            zt2     =  dcmplx(Aimag(zzlograt(-x,-y)),0d0)
            zt3     =  newlm1(-x,-y)*I*rtheta(-x/y)
            zzli2m1 =  zt1 + zt2*zt3
         end function zzli2m1

         pure complex(dp) function newlm1(x,y) ! log (1 - x/y)
            real(dp), intent(in) :: x, y
            real(dp) ::  ar, im
            ar     =  1d0 - x/y
            im     =  (rtheta(x) - rtheta(y))*rtheta(-ar)
            newlm1 =  dlog(abs(ar)) - ipi/2d0*im
         end function newlm1

         pure real(dp) function rtheta(t)
            real(dp), intent(in) :: t
            rtheta = 0.5d0+0.5d0*sign(1d0,t)
         end function rtheta

         subroutine clearC1gVBF_vbf()
         end subroutine clearC1gVBF_vbf

      end module C1gVBF_mod



























