      double precision function C2g0VBF(i1,i5,i6,i3,i2,i4,i7,i8,iboson)
      ! Gluons: i5, i6
      use KinData_mod
      use commonVBF_mod
      use C2gVBF_mod
      implicit none
      integer, intent(in) :: i1, i5, i6, i3, i2, i4, i7, i8, iboson
      double precision :: amp1(4,4), glr(4), amp2(4,4), ampNadj(4,4)
      complex(8) :: zamp1(4,4), zamp2(4,4), zampNadj(4,4)
      double precision :: s1k, s1l, s1i, s2j, s2k, s2l,
     .                    sik, sil, skl, sjk, sjl, q1, q2, qN1, qN2,
     .                    rMcom1, rMcom2, rMcomNadj,
     .                    rM1Tot, rM2Tot, rMnadj
      integer :: i, j
      logical :: leg1, leg2
      ! Common blocks
      double precision :: ele, ere, sin2w, enq, enL, enR
      common/LR/ele,ere,sin2w,enq(-5:5),enL(5),enR(5)
      integer npars, nspec, n
      common /npars/npars,nspec,n
      integer :: nfC1, nfC2
      common /CZFlav/ nfC1,nfC2
!$omp threadprivate(/CZFlav/)
      integer :: si1, si2, si, sj, sg1, sg2, si7
      common /spiketesting/ si1,si2,si,sj,sg1,sg2,si7
      if (si1.eq.0) then
         si1 = i1
         si2 = i2
         si = i4
         sj = i3
         sg1 = i5
         sg2 = i6
         si7 = 0
      endif

      !
      ! Sums the following contributions:
      ! (i1 - (i5 - i6) - i4, i2-i3)^2
      ! (i1-i3, i2 - (i5 - i6) - i3)^2
      !
      !
      !        / i5 / i6
      ! i1 -- / ---/ ----- i4
      !
      ! i2 --------------- i3
      !          
      !                 
      ! i1 --------------- i4
      !
      ! i2 --\----\------- i3
      !       \i5  \i6
      !        
      !

      !
      !          / i5
      ! i1 ---- / ------ i4
      !
      ! i2 ---- \ ------ i3
      !          \ i6


      ! (gluons crossing to the initial state also accepted)

      zamp1(:,:) = (0d0,0d0)
      zamp2(:,:) = (0d0,0d0)
      zampNadj(:,:) = (0d0,0d0)
      glr(:)     =  0d0
      amp1(:,:)  =  0d0
      amp2(:,:)  =  0d0
      ampNadj(:,:) = 0d0
      leg1 = .true.
      leg2 = .true.

      s1k = kin(nspec+6)%s(i1, i5)
      s1l = kin(nspec+6)%s(i1,i6)
      s1i = kin(nspec+6)%s(i1,i4)
      sik = kin(nspec+6)%s(i4,i5)
      sil = kin(nspec+6)%s(i4,i6)

      skl = kin(nspec+6)%s(i5,i6)

      s2j = kin(nspec+6)%s(i2,i3)
      s2k = kin(nspec+6)%s(i2,i5)
      s2l = kin(nspec+6)%s(i2,i6)
      sjk = kin(nspec+6)%s(i3,i5)
      sjl = kin(nspec+6)%s(i3,i6)

      ! Couplings
      call coupling_vbf(nfC1, nfC2, iboson, glr)

      ! Check for initial gluon
      ! (1,2,l,j,k,i) is C(k,2,l,j;1,i) ordering for the adj case
      !              and C(2,j,k;1,l,i) ordering for the nadj case
      ! (l,1,2,j,k,i) is C(1,l,i;2,j,k)

      !> One gluon radiated from each leg
      if ( (i5 < 3).and.(i6 < 3) ) then ! gg initiated
         qN1 = s1k + s1i + sik
         qN2 = s2l + s2j + sjl
         call C2g0VBFnadj_vbf(i1,i5,i3,i2,i6,i4,zampNadj)
         leg1 = .false.
         leg2 = .false.
      elseif ( (i5 < 3).and.(i6 >= 3) ) then 
         ! qg or gq initiated
         ! Make sure we don't cross the wrong gluon
         if( (i1 < 3).or.(i4 < 3) ) then
            qN1 = s1l + s1i + sil
            qN2 = s2k + s2j + sjk
            call C2g0VBFnadj_vbf(i1,i6,i3,i2,i5,i4,zampNadj)
            leg1 = .false.
         else
            qN1 = s1k + s1i + sik
            qN2 = s2l + s2j + sjl
            call C2g0VBFnadj_vbf(i1,i5,i3,i2,i6,i4,zampNadj)
            leg2 = .false.
         endif
      elseif ( (i6 < 3).and.(i5 >= 3) ) then ! qg or gq initiated
         ! (non-adjacent piece already computed for the i5-i6 ordering)
         if( (i1 < 3).or.(i4 < 3) ) then
            leg1 = .false.
         else
            leg2 = .false.
         endif
      else ! qQ initiated
         qN1 = s1k + s1i + sik
         qN2 = s2l + s2j + sjl
         call C2g0VBFnadj_vbf(i1,i5,i3,i2,i6,i4,zampNadj)
      endif
      ! Propagator
      call propagator_vbf(qN1, qN2, iboson, rMcomNadj)
      

      !> Both gluons radiated from the same leg
      ! Propagators
      q1  = s1k + s1l + s1i + skl + sik + sil
      q2  = s2l + s2k + s2j + skl + sjk + sjl
      call propagator_vbf(q1, s2j, iboson, rMcom1)
      call propagator_vbf(s1i, q2, iboson, rMcom2)

      !> Both gluons between legs 1 and i
      if(leg1) call C2g0VBFadj_vbf(i1,i5,i6,i3,i2,i4, zamp1)

      !> Both gluons between legs 2 and j
      if(leg2) call C2g0VBFadj_vbf(i2,i5,i6,i4,i1,i3, zamp2)

      do i = 1,4
         amp1(:,i) = abs(zamp1(:,i)*glr(i))**2
         amp2(:,i) = abs(zamp2(:,i)*glr(i))**2
         ampNadj(:,i) = abs(zampNadj(:,i)*glr(i))**2
      enddo

      rM1Tot = 2d0*rMcom1*sum(amp1)
      rM2Tot = 2d0*rMcom2*sum(amp2)
      rMnadj = 2d0*rMcomNadj*sum(ampNadj)

      C2g0VBF = rM1Tot + rM2Tot
      C2g0VBF = C2g0VBF + rMnadj

      return
      end function


      double precision function C2g0WFH(i1,i5,i6,i3,i2,i4,i7,i8)
      implicit none
      double precision C2g0VBF
      integer i1,i5,i6,i3,i2,i4,i7,i8

      C2g0WFH = C2g0VBF(i1,i5,i6,i3,i2,i4,i7,i8,1)

      return
      end function

      double precision function C2g0ZFH(i1,i5,i6,i3,i2,i4,i7,i8)
      implicit none
      double precision C2g0VBF
      integer i1,i5,i6,i3,i2,i4,i7,i8

      C2g0ZFH = C2g0VBF(i1,i5,i6,i3,i2,i4,i7,i8,0)

      return
      end function

      !>
      !> Symmetrised version of C2g0VFH over i2-i3
      !>
      double precision function sC2g0WFH(i1,i5,i6,i3,i2,i4,i7,i8)
         implicit none
         double precision C2g0WFH
         integer i1,i5,i6,i3,i2,i4,i7,i8

         if ((i5 > 2).and.(i6 > 2)) then
            sC2g0WFH = C2g0WFH(i1,i5,i6,i3,i2,i4,i7,i8)
         elseif ((i5 < 3).and.(i6 < 3)) then
            sC2g0WFH = C2g0WFH(i1,i5,i6,i3,i2,i4,i7,i8)
         else
            if ((i1 > 2).and.(i4 > 2)) then
               sC2g0WFH = (C2g0WFH(i1, i5,i6,i3,i2, i4,i7,i8) 
     .                  +  C2g0WFH(i4, i5,i6,i3,i2, i1,i7,i8))/2d0
            else
               sC2g0WFH = (C2g0WFH(i1,i5,i6, i3,i2, i4,i7,i8) 
     .                  +  C2g0WFH(i1,i5,i6, i2,i3, i4,i7,i8))/2d0
            endif
         endif
         return
      end function sC2g0WFH

      double precision function sC2g0ZFH(i1,i5,i6,i3,i2,i4,i7,i8)
         implicit none
         double precision C2g0ZFH
         integer i1,i5,i6,i3,i2,i4,i7,i8
         
         if ((i5 > 2).and.(i6 > 2)) then
            sC2g0ZFH = C2g0ZFH(i1,i5,i6,i3,i2,i4,i7,i8)
         elseif ((i5 < 3).and.(i6 < 3)) then
            sC2g0ZFH = C2g0ZFH(i1,i5,i6,i3,i2,i4,i7,i8)
         else
            if ((i1 > 2).and.(i4 > 2)) then
               sC2g0ZFH = (C2g0ZFH(i1, i5,i6,i3,i2, i4,i7,i8) 
     .                  +  C2g0ZFH(i4, i5,i6,i3,i2, i1,i7,i8))/2d0
            else
               sC2g0ZFH = (C2g0ZFH(i1,i5,i6, i3,i2, i4,i7,i8) 
     .                  +  C2g0ZFH(i1,i5,i6, i2,i3, i4,i7,i8))/2d0
            endif
         endif
         return
      end function sC2g0ZFH

      ! Utilities
      subroutine clearCache_vbf(n)
      use C2gVBF_mod
      if(n.eq.8) call clearC2gVBF_vbf()
      return
      end subroutine clearCache_vbf

      subroutine clearCache_vbf3(n)
      use C2gVBF_mod
      if(n.eq.9) call clearC2gVBF_vbf()
      return
      end subroutine clearCache_vbf3

      subroutine clearCache_vbf4(n)
      use C2gVBF_mod
      if(n.eq.10) call clearC2gVBF_vbf()
      return
      end subroutine clearCache_vbf4
