      !-------------------------------------------------------------------------------
      ! NNLOJET
      !-------------------------------------------------------------------------------
      !
      ! MODULE: VBF Ampltiude C2g
      !
      ! DESCRIPTION:
      !>    Amplitudes for 2 to 3
      !>    i1-i4, quars ; i5, i6, gluons
      !>        i5   i6
      !>        /   /
      !>    i1 --->--- i4
      !>    i2 --->--- i3
      !-------------------------------------------------------------------------------

      !-------------------------------------------------------------------------------
      module C2gVBF_mod
      !-------------------------------------------------------------------------------
      implicit none

         private

         public :: C2g0VBFadj_vbf, C2g0VBFnadj_vbf, clearC2gVBF_vbf
         type flags_vbf
            logical :: a     = .false.
            logical :: b     = .false.
            logical :: valid = .false.
         end type flags_vbf
         type(flags_vbf), save, target :: flags

         type store_vbf
            complex(8), dimension(4,4) :: s_adj
            complex(8), dimension(4,4) :: s_nadj
            logical,pointer :: is_set_adj, is_set_nadj
         end type store_vbf
         type(store_vbf), save, dimension(6,6,6,6,6,6) :: ampl_C2gVBF
!$omp threadprivate(flags, ampl_C2gVBF)
         integer npars, nspec, n
         common /npars/npars,nspec,n

      contains
         !-----------------------------------------------------------------------------
         !> @brief
         !> Public subroutines
         !
         !> @param[in] i1-i5 initial partons ids
         !> @param[out] zamp(4,4) amplitude
         !>                zamp(:,1) RR (i1-i4 ; i2-i3)
         !>                zamp(:,2) RL
         !>                zamp(:,3) LR
         !>                zamp(:,4) LL
         !>                helicity configurations for the gluons:
         !>                zamp(1,:) + gluon (i5) + gluon (i6)
         !>                zamp(2,:) + gluon (i5) - gluon (i6)
         !>                zamp(3,:) - gluon (i5) + gluon (i6)
         !>                zamp(4,:) - gluon (i5) - gluon (i6)
         !-----------------------------------------------------------------------------
         subroutine C2g0VBFadj_vbf(i1, i5, i6, i3, i2, i4, zamp)
            integer, intent(in) :: i1, i5, i6, i3, i2, i4
            complex(8), intent(out) :: zamp(4,4)
            integer :: i, j

            if(associated(ampl_C2gVBF(i1,i5,i6,i3,i2,i4)%is_set_adj))then
               if(ampl_C2gVBF(i1, i5, i6, i3, i2, i4)%is_set_adj)then
                     zamp = ampl_C2gVBF(i1, i5, i6, i3, i2, i4)%s_adj
                     return
               endif
            endif

            call C2g0adjWrapper(zamp(:,1), i1, i5, i6, i3, i2, i4) ! RR
            call C2g0adjWrapper(zamp(:,2), i1, i5, i6, i2, i3, i4) ! RL

            do i = 1,2
               do j = 1,4
                  zamp(5-j,5-i) = conjg(zamp(j,i))
               enddo
            enddo

            ampl_C2gVBF(i1, i5, i6, i3, i2, i4)%s_adj = zamp
            if(flags%a)then
               ampl_C2gVBF(i1, i5, i6, i3, i2, i4)%is_set_adj => flags%a
            elseif(flags%b)then
               ampl_C2gVBF(i1, i5, i6, i3, i2, i4)%is_set_adj => flags%b
            endif
            flags%valid = .true.

            return
         end subroutine C2g0VBFadj_vbf

         subroutine C2g0VBFnadj_vbf(i1, i5, i3, i2, i6, i4, zamp)
            integer, intent(in) :: i1, i5, i6, i3, i2, i4
            complex(8), intent(out) :: zamp(4,4)
            integer :: i, j

            if(associated(ampl_C2gVBF(i1,i5,i3,i2,i6,i4)%is_set_nadj))then
               if(ampl_C2gVBF(i1, i5, i3, i2, i6, i4)%is_set_nadj)then
                     zamp = ampl_C2gVBF(i1, i5, i3, i2, i6, i4)%s_nadj
                     return
               endif
            endif

            call C2g0nadjWrapper(zamp(:,1), i1, i5, i3, i2, i6, i4) ! RR
            call C2g0nadjWrapper(zamp(:,2), i1, i5, i2, i3, i6, i4) ! RL

            do i = 1,2
               do j = 1,4
                  zamp(5-j,5-i) = conjg(zamp(j,i))
               enddo
            enddo

            ampl_C2gVBF(i1, i5, i3, i2, i6, i4)%s_nadj = zamp
            if(flags%a)then
               ampl_C2gVBF(i1,i5,i3,i2,i6,i4)%is_set_nadj => flags%a
            elseif(flags%b)then
               ampl_C2gVBF(i1,i5,i3,i2,i6,i4)%is_set_nadj => flags%b
            endif
            flags%valid = .true.

            return
         end subroutine C2g0VBFnadj_vbf

         !-----------------------------------------------------------------------------
         !> @brief
         !> Private function to compute sijk
         !>
         !> @param[in] i,j,k id of the partons
         !>
         !-----------------------------------------------------------------------------

         pure double precision function s8ijk(i,j,k)
            use KinData_mod, only: kin
            integer, intent(in) :: i,j,k

            s8ijk = kin(nspec+6)%s(i,j) + kin(nspec+6)%s(j,k) + kin(nspec+6)%s(i,k)

            return
         end function s8ijk

         !-----------------------------------------------------------------------------
         !> @brief
         !> Private functions for 2 gluon no loops,
         !> adj for both gluons rad. from the same leg
         !> nadj for each gluon rad. from a a different leg
         !>
         !> @param[in] i1-i6 initial partons id
         !>
         !-----------------------------------------------------------------------------
         subroutine C2g0adjWrapper(zamp, i1, i2, i3, i4, i5, i6)
            integer, intent(in) :: i1, i2, i3, i4, i5, i6
            complex(8), intent(out) :: zamp(4)
            zamp(1) = zC2g0VBFadjpp(i1,i2,i3,i4,i5,i6)
            zamp(2) = zC2g0VBFadjpm(i1,i2,i3,i4,i5,i6)
            zamp(3) = zC2g0VBFadjmp(i1,i2,i3,i4,i5,i6)
            zamp(4) = zC2g0VBFadjmm(i1,i2,i3,i4,i5,i6)
            return
         end subroutine C2g0adjWrapper

         pure complex(8) function zC2g0VBFadjpp(i1,i5,i6,i3,i2,i4)
            use KinData_mod, only: kin
            integer, intent(in) :: i1, i2, i3, i4, i5, i6
            complex(8) z1, z2, z3, z4, zd

            z1 = kin(nspec+6)%zA(i1, i4)*kin(nspec+6)%zB(i4, i3)
            z2 = kin(nspec+6)%zA(i1, i5)*kin(nspec+6)%zB(i3, i5)
            z3 = kin(nspec+6)%zA(i1,i6)*kin(nspec+6)%zB(i3, i6)
            z4 = kin(nspec+6)%zA(i1,i2)
            zd = kin(nspec+6)%zA(i1,i5)*kin(nspec+6)%zA(i5,i6)
     &          *kin(nspec+6)%zA(i6,i4)

            zC2g0VBFadjpp = (+z1 -z2 -z3)*z4/zd

            return
         end function zC2g0VBFadjpp

         pure complex(8) function zC2g0VBFadjmm(i1,i5,i6,i3,i2,i4)
            use KinData_mod, only: kin
            integer, intent(in) :: i1, i2, i3, i4, i5, i6
            complex(8) z1, z2, zd

            z1 = - kin(nspec+6)%zA(i1, i2)*kin(nspec+6)%zB(i1, i4)
     &           - kin(nspec+6)%zA(i2, i5)*kin(nspec+6)%zB(i4, i5)
     &           - kin(nspec+6)%zA(i2,i6)*kin(nspec+6)%zB(i4, i6)
            z2 =   kin(nspec+6)%zB(i4,i3)
            zd =   kin(nspec+6)%zB(i1,i5)*kin(nspec+6)%zB(i5,i6)
     &            *kin(nspec+6)%zB(i6,i4)

            zC2g0VBFadjmm = (z1 * z2)/zd

            return
         end function zC2g0VBFadjmm

         pure complex(8) function zC2g0VBFadjpm(i1,i5,i6,i3,i2,i4)
            use KinData_mod, only: kin
            integer, intent(in) :: i1, i2, i3, i4, i5, i6
            complex(8) z1, z2, z1d, z2d, zT


            z1d = + kin(nspec+6)%zA(i1,i5)**2*kin(nspec+6)%zA(i5,i6)
     &              *kin(nspec+6)%zB(i5,i4)*s8ijk(i1,i6,i5)
     &           + kin(nspec+6)%zA(i1,i5)*kin(nspec+6)%zA(i1,i6)
     &              *kin(nspec+6)%zA(i5,i6)*kin(nspec+6)%zB(i6,i4)*s8ijk(i1,i6,i5)

            z2d = + kin(nspec+6)%zA(i6,i1)*kin(nspec+6)%zB(i6,i4)**2
     &             *kin(nspec+6)%zB(i6,i5)*s8ijk(i4,i6,i5)
     &          - kin(nspec+6)%zA(i5,i1)*kin(nspec+6)%zB(i6,i4)
     &             *kin(nspec+6)%zB(i6,i5)*kin(nspec+6)%zB(i4,i5)*s8ijk(i4,i6,i5)

            z1  = 1d0/z1d
            z2  = 1d0/z2d

            zT = + kin(nspec+6)%zA(i1,i6)**3*kin(nspec+6)%zA(i1,i2)
     &             *kin(nspec+6)%zB(i1,i4)*kin(nspec+6)%zB(i4,i3)*z1
     &        + kin(nspec+6)%zA(i1,i6)**3*kin(nspec+6)%zA(i6,i2)
     &          *kin(nspec+6)%zB(i6,i4)*kin(nspec+6)%zB(i4,i3)*z1
     &        - kin(nspec+6)%zA(i1,i6)**3*kin(nspec+6)%zA(i5,i2)
     &          *kin(nspec+6)%zB(i4,i3)*kin(nspec+6)%zB(i4,i5)*z1
     &        - kin(nspec+6)%zA(i1,i6)*kin(nspec+6)%zA(i1,i2)
     &          *kin(nspec+6)%zB(i6,i3)*kin(nspec+6)%zB(i4,i5)**3*z2
     &        - kin(nspec+6)%zA(i1,i4)*kin(nspec+6)%zA(i1,i2)
     &          *kin(nspec+6)%zB(i4,i3)*kin(nspec+6)%zB(i4,i5)**3*z2
     &        + kin(nspec+6)%zA(i1,i5)*kin(nspec+6)%zA(i1,i2)
     &          *kin(nspec+6)%zB(i4,i5)**3*kin(nspec+6)%zB(i3,i5)*z2

            zC2g0VBFadjpm = zT

            return
         end function zC2g0VBFadjpm

         pure complex(8) function zC2g0VBFadjmp(i1,i5,i6,i3,i2,i4)
            use KinData_mod, only: kin
            integer, intent(in) :: i1, i2, i3, i4, i5, i6
            complex(8) z1, z2, z1d, z2d, zT

            z1d = + kin(nspec+6)%zA(i5,i4)*kin(nspec+6)%zB(i1,i5)**2
     &             *kin(nspec+6)%zB(i5,i6)*s8ijk(i1,i6,i5)
     &         + kin(nspec+6)%zA(i6,i4)*kin(nspec+6)%zB(i1,i5)
     &           *kin(nspec+6)%zB(i1,i6)*kin(nspec+6)%zB(i5,i6)*s8ijk(i1,i6,i5)

            z2d = - kin(nspec+6)%zA(i5,i6)*kin(nspec+6)%zA(i5,i4)
     &             *kin(nspec+6)%zA(i6,i4)*kin(nspec+6)%zB(i5,i1)*s8ijk(i4,i6,i5)
     &         - kin(nspec+6)%zA(i5,i6)*kin(nspec+6)%zA(i6,i4)**2
     &             *kin(nspec+6)%zB(i6,i1)*s8ijk(i4,i6,i5)

            z1 = 1d0/z1d
            z2 = 1d0/z2d

            zT = - kin(nspec+6)%zA(i1,i6)*kin(nspec+6)%zA(i1,i2)
     &            *kin(nspec+6)%zB(i1,i6)**3*kin(nspec+6)%zB(i6,i3)*z1
     &        + kin(nspec+6)%zA(i1,i6)*kin(nspec+6)%zA(i5,i2)
     &         *kin(nspec+6)%zB(i1,i6)**2*kin(nspec+6)%zB(i6,i3)
     &         *kin(nspec+6)%zB(i6,i5)*z1
     &        + kin(nspec+6)%zA(i1,i5)*kin(nspec+6)%zA(i1,i2)
     &         *kin(nspec+6)%zB(i1,i6)**3*kin(nspec+6)%zB(i3,i5)*z1
     &        - kin(nspec+6)%zA(i1,i5)*kin(nspec+6)%zA(i1,i2)
     &         *kin(nspec+6)%zB(i1,i6)**2*kin(nspec+6)%zB(i1,i3)
     &         *kin(nspec+6)%zB(i6,i5)*z1
     &        - kin(nspec+6)%zA(i1,i5)*kin(nspec+6)%zA(i5,i2)
     &         *kin(nspec+6)%zB(i1,i6)**2*kin(nspec+6)%zB(i6,i5)
     &         *kin(nspec+6)%zB(i3,i5)*z1
     &        + kin(nspec+6)%zA(i1,i5)*kin(nspec+6)%zA(i5,i2)
     &         *kin(nspec+6)%zB(i1,i6)*kin(nspec+6)%zB(i1,i3)
     &         *kin(nspec+6)%zB(i6,i5)**2*z1
     &        + kin(nspec+6)%zA(i1,i4)*kin(nspec+6)%zA(i1,i2)
     &         *kin(nspec+6)%zB(i1,i6)**3*kin(nspec+6)%zB(i3,i4)*z1
     &        - kin(nspec+6)%zA(i1,i4)*kin(nspec+6)%zA(i5,i2)
     &         *kin(nspec+6)%zB(i1,i6)**2*kin(nspec+6)%zB(i6,i5)
     &         *kin(nspec+6)%zB(i3,i4)*z1
     &        - kin(nspec+6)%zA(i1,i2)*kin(nspec+6)%zA(i6,i5)**2
     &         *kin(nspec+6)%zA(i5,i4)*kin(nspec+6)%zB(i1,i6)
     &         *kin(nspec+6)%zB(i6,i3)*z2
     &        - kin(nspec+6)%zA(i1,i2)*kin(nspec+6)%zA(i6,i5)
     &         *kin(nspec+6)%zA(i5,i4)**2*kin(nspec+6)%zB(i1,i6)
     &         *kin(nspec+6)%zB(i3,i4)*z2
     &        + kin(nspec+6)%zA(i1,i2)*kin(nspec+6)%zA(i6,i5)
     &         *kin(nspec+6)%zA(i5,i4)**2*kin(nspec+6)%zB(i1,i4)
     &         *kin(nspec+6)%zB(i6,i3)*z2
     &        - kin(nspec+6)%zA(i1,i2)*kin(nspec+6)%zA(i6,i5)
     &         *kin(nspec+6)%zB(i1,i6)**2*kin(nspec+6)%zB(i6,i3)
     &         *kin(nspec+6)%zB(i6,i5)*z1
     &        + kin(nspec+6)%zA(i1,i2)*kin(nspec+6)%zA(i5,i4)**3
     &         *kin(nspec+6)%zB(i1,i4)*kin(nspec+6)%zB(i3,i4)*z2
     &        - kin(nspec+6)%zA(i1,i2)*kin(nspec+6)%zA(i5,i4)
     &         *kin(nspec+6)%zB(i1,i6)**2*kin(nspec+6)%zB(i6,i5)
     &         *kin(nspec+6)%zB(i3,i4)*z1
     &        + kin(nspec+6)%zA(i6,i5)**2*kin(nspec+6)%zA(i5,i4)
     &         *kin(nspec+6)%zA(i5,i2)*kin(nspec+6)%zB(i6,i3)
     &         *kin(nspec+6)%zB(i6,i5)*z2
     &
            zT = zT + kin(nspec+6)%zA(i6,i5)**2*kin(nspec+6)%zA(i5,i4)
     &               *kin(nspec+6)%zA(i4,i2)*kin(nspec+6)%zB(i6,i3)
     &               *kin(nspec+6)%zB(i6,i4)*z2
     &        + kin(nspec+6)%zA(i6,i5)*kin(nspec+6)%zA(i6,i2)
     &         *kin(nspec+6)%zA(i5,i4)**2*kin(nspec+6)%zB(i6,i3)
     &         *kin(nspec+6)%zB(i6,i4)*z2
     &        + kin(nspec+6)%zA(i6,i5)*kin(nspec+6)%zA(i5,i4)**2
     &         *kin(nspec+6)%zA(i5,i2)*kin(nspec+6)%zB(i6,i3)
     &         *kin(nspec+6)%zB(i5,i4)*z2
     &        + kin(nspec+6)%zA(i6,i5)*kin(nspec+6)%zA(i5,i4)**2
     &         *kin(nspec+6)%zA(i5,i2)*kin(nspec+6)%zB(i6,i5)
     &         *kin(nspec+6)%zB(i3,i4)*z2
     &        + kin(nspec+6)%zA(i6,i5)*kin(nspec+6)%zA(i5,i4)**2
     &         *kin(nspec+6)%zA(i4,i2)*kin(nspec+6)%zB(i6,i4)
     &         *kin(nspec+6)%zB(i3,i4)*z2
     &        + kin(nspec+6)%zA(i6,i5)*kin(nspec+6)%zA(i5,i2)
     &         *kin(nspec+6)%zB(i1,i6)*kin(nspec+6)%zB(i6,i3)
     &         *kin(nspec+6)%zB(i6,i5)**2*z1
     &        + kin(nspec+6)%zA(i6,i2)*kin(nspec+6)%zA(i5,i4)**3
     &         *kin(nspec+6)%zB(i6,i4)*kin(nspec+6)%zB(i3,i4)*z2
     &        + kin(nspec+6)%zA(i5,i4)**3*kin(nspec+6)%zA(i5,i2)
     &         *kin(nspec+6)%zB(i3,i4)*kin(nspec+6)%zB(i5,i4)*z2
     &        + kin(nspec+6)%zA(i5,i4)*kin(nspec+6)%zA(i5,i2)
     &         *kin(nspec+6)%zB(i1,i6)*kin(nspec+6)%zB(i6,i5)**2
     &         *kin(nspec+6)%zB(i3,i4)*z1

            zC2g0VBFadjmp = zT

            return
         end function zC2g0VBFadjmp

         subroutine C2g0nadjWrapper(zamp, i1, i2, i3, i4, i5, i6)
            integer, intent(in) :: i1, i2, i3, i4, i5, i6
            complex(8), intent(out) :: zamp(4)
            zamp(1) = zC2g0VBFnadjpp(i1,i2,i3,i4,i5,i6)
            zamp(2) = zC2g0VBFnadjpm(i1,i2,i3,i4,i5,i6)
            zamp(3) = zC2g0VBFnadjmp(i1,i2,i3,i4,i5,i6)
            zamp(4) = zC2g0VBFnadjmm(i1,i2,i3,i4,i5,i6)
            return
         end subroutine C2g0nadjWrapper

         pure complex(8) function zC2g0VBFnadjpp(i1,i5,i3,i2,i6,i4)
            use KinData_mod, only: kin
            integer, intent(in) :: i1, i2, i3, i4, i5, i6
            complex(8) z1, z2, zd

            z1 = - kin(nspec+6)%zA(i1,i5)*kin(nspec+6)%zA(i2,i6)
     &             *kin(nspec+6)%zB(i5,i6)
     &        - kin(nspec+6)%zA(i1,i5)*kin(nspec+6)%zA(i2,i3)
     &         *kin(nspec+6)%zB(i5,i3)
     &        - kin(nspec+6)%zA(i1,i4)*kin(nspec+6)%zA(i2,i6)
     &         *kin(nspec+6)%zB(i4,i6)
     &        - kin(nspec+6)%zA(i1,i4)*kin(nspec+6)%zA(i2,i3)
     &         *kin(nspec+6)%zB(i4,i3)

            z2 = kin(nspec+6)%zA(i1,i2)
            zd = kin(nspec+6)%zA(i1,i5)*kin(nspec+6)%zA(i5,i4)
     &          *kin(nspec+6)%zA(i2,i6)*kin(nspec+6)%zA(i6,i3)

            zC2g0VBFnadjpp = (z1*z2)/zd

         return
         end function zC2g0VBFnadjpp

         pure complex(8) function zC2g0VBFnadjmp(i1,i5,i3,i2,i6,i4)
            use KinData_mod, only: kin
            integer, intent(in) :: i1, i2, i3, i4, i5, i6
            complex(8) z1, zd

            z1 = - kin(nspec+6)%zA(i1,i2)*kin(nspec+6)%zA(i2,i6)
     &            *kin(nspec+6)%zB(i4,i1)*kin(nspec+6)%zB(i4,i6)
     &        - kin(nspec+6)%zA(i1,i2)*kin(nspec+6)%zA(i2,i3)
     &         *kin(nspec+6)%zB(i4,i1)*kin(nspec+6)%zB(i4,i3)
     &        - kin(nspec+6)%zA(i5,i2)*kin(nspec+6)%zA(i2,i6)
     &         *kin(nspec+6)%zB(i4,i5)*kin(nspec+6)%zB(i4,i6)
     &        - kin(nspec+6)%zA(i5,i2)*kin(nspec+6)%zA(i2,i3)
     &         *kin(nspec+6)%zB(i4,i5)*kin(nspec+6)%zB(i4,i3)

            zd = kin(nspec+6)%zB(i1,i5)*kin(nspec+6)%zB(i5,i4)
     &          *kin(nspec+6)%zA(i2,i6)*kin(nspec+6)%zA(i6,i3)

            zC2g0VBFnadjmp = z1/zd

            return
         end function zC2g0VBFnadjmp

         complex(8) function zC2g0VBFnadjpm(i1,i5,i3,i2,i6,i4)
            use KinData_mod, only: kin
            integer, intent(in) :: i1, i2, i3, i4, i5, i6
            complex(8) z1, zd

            z1 = + kin(nspec+6)%zA(i1,i5)*kin(nspec+6)%zA(i1,i2)
     &            *kin(nspec+6)%zB(i3,i5)*kin(nspec+6)%zB(i3,i2)
     &        + kin(nspec+6)%zA(i1,i5)*kin(nspec+6)%zA(i1,i6)
     &         *kin(nspec+6)%zB(i3,i5)*kin(nspec+6)%zB(i3,i6)
     &        + kin(nspec+6)%zA(i1,i4)*kin(nspec+6)%zA(i1,i2)
     &         *kin(nspec+6)%zB(i3,i4)*kin(nspec+6)%zB(i3,i2)
     &        + kin(nspec+6)%zA(i1,i4)*kin(nspec+6)%zA(i1,i6)
     &         *kin(nspec+6)%zB(i3,i4)*kin(nspec+6)%zB(i3,i6)

            zd = kin(nspec+6)%zA(i1,i5)*kin(nspec+6)%zA(i5,i4)
     &          *kin(nspec+6)%zB(i2,i6)*kin(nspec+6)%zB(i6,i3)

            zC2g0VBFnadjpm = z1/zd

         return
         end function zC2g0VBFnadjpm

         pure complex(8) function zC2g0VBFnadjmm(i1,i5,i3,i2,i6,i4)
            use KinData_mod, only: kin
            integer, intent(in) :: i1, i2, i3, i4, i5, i6
            complex(8) z1, z2, zd

            z1 = + kin(nspec+6)%zA(i1,i2)*kin(nspec+6)%zB(i4,i1)
     &            *kin(nspec+6)%zB(i2,i3)
     &        + kin(nspec+6)%zA(i1,i6)*kin(nspec+6)%zB(i4,i1)
     &         *kin(nspec+6)%zB(i6,i3)
     &        + kin(nspec+6)%zA(i5,i2)*kin(nspec+6)%zB(i4,i5)
     &         *kin(nspec+6)%zB(i2,i3)
     &        + kin(nspec+6)%zA(i5,i6)*kin(nspec+6)%zB(i4,i5)
     &         *kin(nspec+6)%zB(i6,i3)

            z2 = kin(nspec+6)%zB(i4,i3)
            zd = kin(nspec+6)%zB(i1,i5)*kin(nspec+6)%zB(i5,i4)
     &          *kin(nspec+6)%zB(i2,i6)*kin(nspec+6)%zB(i6,i3)

            zC2g0VBFnadjmm = (z1*z2)/zd

         return
         end function zC2g0VBFnadjmm

         subroutine clearC2gVBF_vbf()
            !> If the last point was valid, swap the values of 
            !> true and false, instead of having to iterate over the 
            !> entire array
            if(flags%valid)then
               flags%valid = .false.
               if(flags%a)then
                  flags%a = .false.
                  flags%b = .true.
               else
                  flags%b = .false.
                  flags%a = .true.
               endif
            endif
         return
         end subroutine clearC2gVBF_vbf


      end module C2gVBF_mod
