      !-------------------------------------------------------------------------------
      ! NNLOJET
      !-------------------------------------------------------------------------------
      !
      ! MODULE: VBF Amplitude C0g VBF
      !
      ! DESCRIPTION:
      !>    Amplitudes for 2q to 4q
      !>        i5 i6
      !>         \/
      !>         /
      !>    i1 --->--- i4
      !>    i2 --->--- i3
      !-------------------------------------------------------------------------------

      !-------------------------------------------------------------------------------
      module E0gVBF_mod
         !-------------------------------------------------------------------------------
         use KinData_mod
         use commonVBF_mod
         implicit none
         private

         public :: E0g0VBF, F0g0VBF
         public :: skip_f 

         integer, parameter :: dp = kind(1.0d0)

         !> flag to skip F-type ME
         logical, parameter :: skip_f = .true.

         integer npars, nspec, n
         common /npars/npars,nspec,n
         integer :: nfC1, nfC2
         common /CZFlav/ nfC1,nfC2
!$omp threadprivate(/CZFlav/)
      contains

         !> Public  interfaces
         real(dp) function E0g0VBF(i1,i3,i2,i4,i5,i6, i7, i8, iboson)
            integer, intent(in) :: i1,i3,i2,i4,i5,i6, i7, i8, iboson
            real(dp) :: r1, r2
!           integer si1, si2, si, sj, sg1, sg2, si7
!           common /spiketesting/ si1,si2,si,sj,sg1,sg2, si7
!           if (si1.eq.0) then
!              si1 = i1
!              si2 = i2
!              si = i4
!              sj = i3
!              sg1 = i5
!              sg2 = i6
!              si7 = 1
!           endif

            E0g0VBF = 0d0

            if (((i1.lt.3).or.(i4.lt.3)).and.((i2.lt.3).or.(i3.lt.3))) then
               ! Initial particles are attached to the Z/W sum over qqb
               ! pair emitted from both legs
               ! i5-i6 from i1-i4
               r1 = E0g0VBFbase(i2, i4, i6, i5, i1, i3, iboson)
               ! i5-i6 from i2-i3
               r2 = E0g0VBFbase(i1, i3, i6, i5, i2, i4, iboson)
               E0g0VBF = r1 + r2
            else
               if ((i2.lt.3).or.(i3.lt.3)) then
                  ! i1-i4 from i5-i6
                  E0g0VBF = E0g0VBFbase(i2, i4, i6, i5, i1, i3, iboson)
               else
                  ! i3-i2 from i5-i6
                  E0g0VBF = E0g0VBFbase(i1, i3, i6, i5, i2, i4, iboson)
               endif
            end if
            return
         end function E0g0VBF

         real(dp) function F0g0VBF(i1, i3, i2, i4, i5, i6, i7, i8, iboson)
            integer, intent(in) :: i1, i3, i2, i4, i5, i6, i7, i8, iboson
            real(dp) :: r1, r2
!              integer si1, si2, si, sj, sg1, sg2, si7
!              common /spiketesting/ si1,si2,si,sj,sg1,sg2, si7
!              if (si1.eq.0) then
!                 si1 = i1
!                 si2 = i2
!                 si = i4
!                 sj = i3
!                 sg1 = i5
!                 sg2 = i6
!                 si7 = 2
!              endif

            F0g0VBF = 0d0

            if (skip_f) return

            !> If it is a qR initiated ME, return 0
            if ((i5 < 3) .or. (i6 < 3)) return

            !> Upper leg (i5-i6 (and i4) quarks radiated from i1-4)
            r1 = F0g0VBFbase(i2,   i4, i6, i5, i1,    i3, iboson)

            !> Lower leg
            r2 = F0g0VBFbase(i1,   i3, i6, i5, i2,    i4, iboson)

            F0g0VBF = r1 + r2

            return
         end function F0g0VBF

         !>
         !> Private functions
         !>

         real(dp) function E0g0VBFbase(i1, i3, i6, i5, i2, i4, iboson)
            integer, intent(in) :: i1,i3,i6,i5,i2,i4, iboson
            real(dp), dimension(4) :: glr
            real(dp), dimension(2,4) :: amp
            complex(dp), dimension(2,4) :: zamp
            real(dp) :: s2j, s2k, s2l, sjk, sjl, skl, s1i, q2
            real(dp) :: rMcom
            integer :: i
            ! Base situation
            ! i1 ------------ i4
            !        |_ _ _ H (i7,i8)
            !        |
            ! i2 ------------ i3
            !         |
            !        / \
            !       i6 i5

            s2j = kin(nspec+6)%s(i2,i3)
            s2k = kin(nspec+6)%s(i2,i6)
            s2l = kin(nspec+6)%s(i2,i5)
            sjk = kin(nspec+6)%s(i3,i6)
            sjl = kin(nspec+6)%s(i3,i5)
            skl = kin(nspec+6)%s(i5,i6)
            s1i = kin(nspec+6)%s(i1,i4)
            q2  = s2k + s2l + s2j + skl + sjk + sjl
            call propagator_vbf(s1i, q2, iboson, rMcom)
            call coupling_vbf(nfC1, nfC2, iboson, glr(:))

            call zamplE0g(i2,i5,i6,i3, i1,i4,zamp(1, 1)) ! + +     + +  - +
            call zamplE0g(i2,i6,i5,i3, i1,i4,zamp(2, 1)) ! + +     + +  + -
            call zamplE0g(i3,i5,i6,i2, i1,i4,zamp(1, 2)) ! + +     - -  - +
            call zamplE0g(i3,i6,i5,i2, i1,i4,zamp(2, 2)) ! + +     - -  + -

            zamp(1, 3) = conjg(zamp(2, 2))
            zamp(2, 3) = conjg(zamp(1, 2))
            zamp(1, 4) = conjg(zamp(2, 1))
            zamp(2, 4) = conjg(zamp(1, 1))

            do i = 1, 4
               zamp(:, i) = zamp(:,i)*glr(i)
            enddo

            amp = abs(zamp)**2

            E0g0VBFbase = rMcom*sum(amp)

            return
         end function E0g0VBFbase

         real(dp) function F0g0VBFbase(i1, i3, i6, i5, i2, i4, iboson)
            integer, intent(in) :: i1,i3,i6,i5,i2,i4,iboson
            real(dp), dimension(4) :: glr(4), amp1(4), amp2(4), glr2(4)
            complex(dp), dimension(4) :: zamp1(4), zamp2(4)
            complex(dp), dimension(4) :: zamp3(4), zamp4(4)
            real(dp) :: s2j, s2k, s2l, sjk, sjl, skl, s1i, q2
            real(dp) :: rMcom
            ! Base situation
            ! i1 ------------ i4
            !        |_ _ _ H (i7,i8)
            !        |
            ! i2 ------------ i3
            !         |
            !        / \
            !       i6 i5

            s1i = kin(nspec+6)%s(i1,i4)

            s2j = kin(nspec+6)%s(i2,i3)
            skl = kin(nspec+6)%s(i5,i6)
            sjk = kin(nspec+6)%s(i3,i6)
            sjl = kin(nspec+6)%s(i3,i5)
            s2k = kin(nspec+6)%s(i2,i6)
            s2l = kin(nspec+6)%s(i2,i5)

            q2  = s2j + skl + sjk + sjl + s2k + s2l

            call propagator_vbf(s1i, q2, iboson, rMcom)
            call coupling_vbf(nfC1, nfC2, iboson, glr)


            zamp1(:) = (0d0,0d0)
            zamp2(:) = (0d0,0d0)
            zamp3(:) = (0d0,0d0)
            zamp4(:) = (0d0,0d0)
            amp1 (:) = 0d0
            amp2 (:) = 0d0

            ! A                                           ! 1 i    2 j   k l
            call zamplE0g(i2, i6,i5, i3,  i1,i4,zamp1(1)) ! + +    + +   - + !
            call zamplE0g(i3, i5,i6, i2,  i1,i4,zamp1(2)) ! + +    - -   + - !

            zamp1(3) = conjg(zamp1(2))
            zamp1(4) = conjg(zamp1(1))

            ! B
            call zamplE0g(i2, i6,i3, i5,  i1,i4,zamp2(1)) ! + +  + +   - + !
            call zamplE0g(i5, i3,i6, i2,  i1,i4,zamp2(2)) ! + +  - -   + - !
            zamp2(3) = conjg(zamp2(2))
            zamp2(4) = conjg(zamp2(1))

            ! D                                           ! 1 i  2 j   k l
            call zamplE0g(i6, i2,i5, i3,  i1,i4,zamp4(1)) ! + +  + +   - + !
            call zamplE0g(i3, i5,i2, i6,  i1,i4,zamp4(2)) ! + +  - -   + - !
            zamp4(3) = conjg(zamp4(2))
            zamp4(4) = conjg(zamp4(1))

            if (iboson.eq.0) then
               ! Z boson allows for extra combinations
               ! since the 2-j-k-l system behaves as a D-type ME
               call zamplE0g(i6, i2,i3, i5,  i1,i4,zamp3(1)) ! + +    + +   - + !
               call zamplE0g(i5, i3,i2, i6,  i1,i4,zamp3(2)) ! + +    - -   + - !
               zamp3(3) = conjg(zamp3(2))
               zamp3(4) = conjg(zamp3(1))

               call coupling_mix_f(i2, i3, glr2)
               amp2 = (glr * glr2)*real( (zamp3) * conjg(zamp1) )
            endif
            amp1 = glr**2*real( (zamp2 + zamp4) * conjg(zamp1) )
            F0g0VBFbase = rMcom*(sum(amp1) + sum(amp2))

            return
         end function F0g0VBFbase

         !> Mix coupling
         subroutine coupling_mix_f(ip1, ip2, glr)
            integer, intent(in) ::  ip1, ip2
            real(dp), dimension(4), intent(out) :: glr
            real(dp), dimension(4) :: tmp
            integer :: ntmp
            integer, parameter :: iboson = 0 ! Only happens for Z
            integer, parameter :: nc = 3d0, nup = 2d0, nd = 3d0
            if (min(ip1,ip2) == 1) then
               ntmp = nfC1
            elseif (min(ip1,ip2) == 2) then
               ntmp = nfC2
            endif
            call coupling_vbf(ntmp, 1, iboson, glr)
            call coupling_vbf(ntmp, 2, iboson, tmp)
            glr = nc*(nd*glr + nup*tmp)
            return
         end subroutine

         !> Basic amplitude
         subroutine zamplE0g(i1, ik, il, ii,   i2, ij, zamp)
            ! i2 ------------ ij
            !        |_ _ _ H (i7,i8)
            !        |
            ! i1 ------------ ii
            !         |
            !        / \
            !       ik il
            integer, intent(in) :: i1, ij, il, ik, i2, ii
            complex(dp), intent(out) :: zamp
            complex(dp) :: z1, z2, z1d, z2d
            real(dp), pointer, dimension(:,:) :: s
            complex(dp), pointer, dimension(:,:) :: za, zb

            s => kin(nspec+6)%s
            zA => kin(nspec+6)%zA
            zB => kin(nspec+6)%zB

            ! Base helicity is (1+ j+ k- l+ 2+ i+)
            z1 = (0d0,0d0)
            z2 = (0d0,0d0)
            z1d = DCMPLX(sqrt(2d0)/(s(ik,il) + s(ii,ik) + s(ii,il)),0d0)
            z2d = DCMPLX(sqrt(2d0)/(s(ik,il) + s(i1,ik) + s(i1,il)),0d0)

            z1 = + 1.D0*zA(il,ik)*zA(i2,i1)*zB(il,ij)*zB(il,ii)*z1d
     &           + 1.D0*zA(ik,ii)*zA(i2,i1)*zB(il,ii)*zB(ij,ii)*z1d
            z2 = + 1.D0*zA(ik,i2)*zA(ik,i1)*zB(il,ik)*zB(ij,ii)*z2d
     &           - 1.D0*zA(ik,i1)*zA(i2,i1)*zB(il,i1)*zB(ij,ii)*z2d

            zamp = (z1 + z2)/DCMPLX(s(ik,il),0d0)

            return
         end subroutine zamplE0g

      end module E0gVBF_mod

