*********************************************************************
*
*
* Quark pair one loop level virtual correction matrix element for
*
* q(i1) + Qb(i3) -> g(i2) + nu(i4) + e+(i5)
*
* summed and squared over helicities, all colour levels including overall factors
*
* The flavour of the quark pair in this matrix element is determined from the flav variable in the common block /BWFlav/

      function FullB1g1W(i1,i2,i3,i4,i5,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /eweakW/ amz,stw,ewfac
!$omp threadprivate(/eweakW/)
      common /order/iorder
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /BWFlav/nf1,nf2
!$omp threadprivate(/BWFlav/)

      if((amz.eq.0).or.(stw.eq.0))then
         call initeweak
      endif

      fac = get_avg(i1,i3)
      fac = fac*2d0*(nc**2-1d0)/(2d0*nc)*nc**2*(4*pi*as)*(4*pi*amz/stw)**2

      fac = fac*as/(2*pi)


      FullB1g1W = B1g1W(i1,i2,i3,i4,i5,renscale2)
     .           -(1d0/nc**2)*Bt1g1W(i1,i2,i3,i4,i5,renscale2)


      if (iorder.eq.-1)then
         FullB1g1W = FullB1g1W
     .              +dble(nf)/3d0*1d0/3d0*B1g0W(i1,i2,i3,i4,i5) ! renormalisation nf component
      endif

      FullB1g1W = fac*FullB1g1W

      return
      end

*********************************************************************
*
*
* Quark pair one loop level virtual correction matrix element for
*
* q(i1) + Qb(i3) -> g(i2) + nu(i4) + e+(i5)
*
* summed and squared over helicities, single colour ordering without
* overall factors
*
* The flavour of the quark pair in this matrix element is determined from the flav variable in the common block /BWFlav/

      function B1g1W(i1,i2,i3,i4,i5,renscale2)
      use KinData_mod
      use B1gZ, only : zampB1g1Z
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /eweakW/ amz,stw,ewfac
!$omp threadprivate(/eweakW/)
      common /order/iorder
      common /koppel/ nf,nc,b0
      common /BWFlav/nf1,nf2
!$omp threadprivate(/BWFlav/)
      double complex zamp(4)

      zamp = zampB1g1Z(i1,i2,i3,i5,i4,renscale2)
      B1g1W = dble(dconjg(zamp(1))*zamp(2))+dble(dconjg(zamp(3))*zamp(4))

      s = s5(i4,i5)

      call eweakW1qq(nf1,nf2,s)

      B1g1W = ewfac*B1g1W

c     A UV counter term at order 1/epsilon which returns the matrix element in the MS bar scheme.
      if (iorder .eq. -1) then
         subuv = 11d0/6d0
         B1g1W = B1g1W - subuv*B1g0W(i1,i2,i3,i4,i5)
      elseif (iorder .eq. 0) then
c     Correction for C(e)
         e0 = -pi**2/6d0 + 0.5d0
c         e0 = -(1d0/6d0)
         B1g1W = B1g1W - e0*B1g0W(i1,i2,i3,i4,i5)
      endif


      return
      end


