***********************************************************************
*
* Quark pair one loop level virtual correction matrix element for
*
* q(i1) + Qb(i4) -> g(i2) + g(i3) + W+ -> g(i2) + nu(i4) + e+(i5)
*
* summed and squared over helicities, summed over all colour orderings and all colour levels
*
* The flavour of the quark pair in this matrix element is determined from the flav variable in the common block /B2gFlav/

      function FullB2g1W(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /eweakW/ amz,stw,ewfac
!$omp threadprivate(/eweakW/)
      common /koppel/ nf,nc,b0
      common /order/iorder
      common /strong/ as
!$omp threadprivate(/strong/)
      common /BWFlav/nf1,nf2
!$omp threadprivate(/BWFlav/)

      if((amz.eq.0).or.(stw.eq.0))then
         call initeweak
      endif

      fac = get_avg(i1,i4)

      fac = fac*2d0*(4d0*pi*as)**2*(4d0*pi*amz/stw)**2*(nc**2-1d0)/(2d0*nc)*as/(2*pi)*nc**3


      if((i2.le.2).and.(i3.le.2))then ! gg initial state configuration, nf type contribution
         nup = 2

         nf1=2 ! Up type contribution
         nf2=1

      FullB2g1W = nup*(B2g1W(i1,i2,i3,i4,i5,i6,renscale2)
     .           +B2g1W(i1,i3,i2,i4,i5,i6,renscale2)
     .           -(1d0/nc**2)*(Bt2g1W(i1,i2,i3,i4,i5,i6,renscale2)
     .                        +Bt2g1W(i1,i3,i2,i4,i5,i6,renscale2)
     .                        +Btt2g1W(i1,i2,i3,i4,i5,i6,renscale2)
     .                        -Bhh2g1W(i1,i2,i3,i4,i5,i6,renscale2))
     .           +(1d0/nc**4)*Bttt2g1W(i1,i2,i3,i4,i5,i6,renscale2)
     .           +(dble(nf)/nc)*(Bh2g1W(i1,i2,i3,i4,i5,i6,renscale2)
     .                          +Bh2g1W(i1,i3,i2,i4,i5,i6,renscale2))
     .           -(dble(nf)/nc**3)*Bth2g1W(i1,i2,i3,i4,i5,i6,renscale2)
     .           -(1d0/nc)*(B2g1Wtop(i1,i2,i3,i4,i5,i6,renscale2)
     .                     +B2g1Wtop(i1,i3,i2,i4,i5,i6,renscale2))
     .           +(1d0/nc**3)*Bt2g1Wtop(i1,i2,i3,i4,i5,i6,renscale2))

      else

      if((i1.le.2).and.(i4.le.2))then
         ! qqb initial state => extra factor of a half
         fac = 0.5d0*fac
      endif

      FullB2g1W = B2g1W(i1,i2,i3,i4,i5,i6,renscale2)
     .           +B2g1W(i1,i3,i2,i4,i5,i6,renscale2)
     .           -(1d0/nc**2)*(Bt2g1W(i1,i2,i3,i4,i5,i6,renscale2)
     .                        +Bt2g1W(i1,i3,i2,i4,i5,i6,renscale2)
     .                        +Btt2g1W(i1,i2,i3,i4,i5,i6,renscale2)
     .                        -Bhh2g1W(i1,i2,i3,i4,i5,i6,renscale2))
     .           +(1d0/nc**4)*Bttt2g1W(i1,i2,i3,i4,i5,i6,renscale2)
     .           +(dble(nf)/nc)*(Bh2g1W(i1,i2,i3,i4,i5,i6,renscale2)
     .                          +Bh2g1W(i1,i3,i2,i4,i5,i6,renscale2))
     .           -(dble(nf)/nc**3)*Bth2g1W(i1,i2,i3,i4,i5,i6,renscale2)
     .           -(1d0/nc)*(B2g1Wtop(i1,i2,i3,i4,i5,i6,renscale2)
     .                     +B2g1Wtop(i1,i3,i2,i4,i5,i6,renscale2))
     .           +(1d0/nc**3)*Bt2g1Wtop(i1,i2,i3,i4,i5,i6,renscale2)

      endif

c     04/05/2023 GS: Remember!
c     the conversion to ’t Hooft-Veltman scheme implies to add:
c         - 1/2 * (1 - 1/nc^2) * FullB2g0W
c     according to eq.(5.5) of hep-ph/9409393.
c     such a term is splitted across the layers above,
c     see inside B2g1W, Bt2g1W, Btt2g1W, Bttt2g1W
      
      FullB2g1W = fac*FullB2g1W

      return
      end

***********************************************************************
*
* Quark pair one loop level virtual correction matrix element for
*
* q(i1) + Qb(i4) -> g(i2) + g(i3) + W+ -> g(i2) + nu(i4) + e+(i5)
*
* summed and squared over helicities, single colour ordering without
* overall factors
*
* The flavour of the quark pair in this matrix element is determined from the flav variable in the common block /B2gFlav/

      function B2g1W(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      use B2gZ, only : zampB2g1Z,zampB2g0Z
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /eweakW/ amz,stw,ewfac
!$omp threadprivate(/eweakW/)
      common /order/iorder
      common /koppel/ nf,nc,b0
      common /BWFlav/nf1,nf2
!$omp threadprivate(/BWFlav/)
      double complex zamp(2,2),zampTree(2,2)

      s = s6(i5,i6)

      call eweakW1qq(nf1,nf2,s)

      zamp     = zampB2g1Z(i1,i2,i3,i4,i5,i6,renscale2)
      zampTree = zampB2g0Z(i1,i2,i3,i4,i5,i6)

      B2g1W=0d0

      do ii=1,2
         do jj=1,2
            B2g1W=B2g1W+dble(dconjg(zampTree(ii,jj))*zamp(ii,jj))
         enddo
      enddo

      B2g1W = ewfac*B2g1W

      if (iorder .eq. -1) then
c     A UV counter term at order 1/e to renormalise the matrix element in the MS bar scheme
         subuv = 11d0/3d0
         B2g1W = B2g1W - subuv*B2g0W(i1,i2,i3,i4,i5,i6)
      elseif (iorder .eq. 0)then
c     Correction for C(e)
         e0 = -pi**2/4d0 + 0.5d0
         B2g1W = B2g1W - e0*B2g0W(i1,i2,i3,i4,i5,i6)
      endif

      return
      end

***********************************************************************
*
* Quark pair one loop level virtual correction matrix element for
*
* q(i1) + Qb(i4) -> g(i2) + g(i3) + W+ -> g(i2) + nu(i4) + e+(i5)
*
* summed and squared over helicities, single colour ordering without
* overall factors, nf contribution
*
* The flavour of the quark pair in this matrix element is determined from the flav variable in the common block /B2gFlav/

      function Bh2g1W(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      use B2gZ, only : zampB2g0Z,zampBh2g1Z
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /eweakW/ amz,stw,ewfac
!$omp threadprivate(/eweakW/)
      common /order/iorder
      common /koppel/nf,nc,b0
      common /BWFlav/nf1,nf2
!$omp threadprivate(/BWFlav/)
      double complex zampnfloop(2,2),zampTree(2,2)

      Bh2g1W = 0d0

      s = s6(i5,i6)

      call eweakW1qq(nf1,nf2,s)

      zampTree = zampB2g0Z(i1,i2,i3,i4,i5,i6)

      zampnfLoop = zampBh2g1Z(i1,i2,i3,i4,i5,i6,renscale2)

      do ii=1,2
         do jj=1,2
            Bh2g1W = Bh2g1W+dble(dconjg(zampTree(ii,jj))*zampnfLoop(ii,jj))
         enddo
      enddo

      Bh2g1W = ewfac*Bh2g1W

      if (iorder .eq. -1) then
c     A UV counter term at order 1/e to renormalise the matrix element in the MS bar scheme
         subuv = -(2d0/3d0)
         Bh2g1W = Bh2g1W - subuv*B2g0W(i1,i2,i3,i4,i5,i6)
c      elseif (iorder .eq. 0)then
c     Correction for C(e), work this out numerically once this RV subtraction term has been calculated.
      endif


      return
      end

***********************************************************************
*
* Quark pair one loop level virtual correction matrix element for
*
* q(i1) + Qb(i4) -> g(i2) + g(i3) + W+ -> g(i2) + nu(i4) + e+(i5)
*
* summed and squared over helicities, single colour ordering without
* overall factors, finite top corrections
*
* The flavour of the quark pair in this matrix element is determined from the flav variable in the common block /B2gFlav/

      function B2g1Wtop(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      use B2gZ, only : zampB2g0Z,zFaxqqbgg
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /eweakW/ amz,stw,ewfac
!$omp threadprivate(/eweakW/)
      common /masstop/emt
      common /order/iorder

      if (emt.eq.0d0)then
         write(*,*) "Error in finite top correction to B2g1Z,
     . the mass of the top has not been defined"
         stop
      endif

      B2g1Wtop = 0d0

      if (iorder.eq.0)then
         B2g1Wtop = (1d0/20d0)*s6(i2,i3)/emt**2*Bh2g1W(i1,i2,i3,i4,i5,i6,renscale2)
      endif

      return
      end



