****************************************************************
* quark pair tree level matrix elements for q + Qb -> g + g + g + W+ -> nu + e+
*
*
* summed over colour orderings, summed and squared
*
* q(i1) + Qb(i5) -> g(i2) + g(i3) + g(i4) + nu(i6) + e+(i7)
*
* If the overall charge of the process is negative then the momenta of the electron
* and neutrino are reversed.
*
* Including overall factors
*
      function FullB3g0W(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)
      common /BWFlav/ nf1,nf2
!$omp threadprivate(/BWFlav/)
      common /eweakW/ amz,stw,ewfac
!$omp threadprivate(/eweakW/)

      fac = get_avg(i1,i5)

      fac = fac*(4d0*pi*as)**3*(4d0*pi*amz/stw)**2*nc**3*(nc**2-1)/(2d0*nc)*2d0

      if((i1.le.2).and.(i5.le.2))then
         fac = fac/6d0          ! statistical factor
      elseif((i1.le.2).or.(i5.le.2))then
         fac = fac/2d0
      endif

      FullB3g0W = B3g0W(i1,i2,i3,i4,i5,i6,i7)
     .           +B3g0W(i1,i2,i4,i3,i5,i6,i7)
     .           +B3g0W(i1,i4,i2,i3,i5,i6,i7)
     .           +B3g0W(i1,i3,i4,i2,i5,i6,i7)
     .           +B3g0W(i1,i3,i2,i4,i5,i6,i7)
     .           +B3g0W(i1,i4,i3,i2,i5,i6,i7)
     .        -(1d0/nc**2)*(Bt3g0W(i1,i2,i3,i4,i5,i6,i7)
     .           +Bt3g0W(i1,i2,i4,i3,i5,i6,i7)
     .           +Bt3g0W(i1,i3,i2,i4,i5,i6,i7)
     .           +Bt3g0W(i1,i3,i4,i2,i5,i6,i7)
     .           +Bt3g0W(i1,i4,i2,i3,i5,i6,i7)
     .           +Bt3g0W(i1,i4,i3,i2,i5,i6,i7))
     .     +((1d0/nc**2)+(1d0/nc**4))*Btt3g0W(i1,i2,i3,i4,i5,i6,i7)


      FullB3g0W = fac*FullB3g0W

c     write(*,*) "FullB3g0Z",FullB3g0Z,i1,i2,i3,i4,i5,i6,i7

      return
      end

****************************************************************
* quark pair tree level matrix elements for q + Qb -> g + g + g + W+ -> nu + e+
*
* single colour ordering, leading colour, summed and squared.
*
* q(i1) + Qb(i5) -> g(i2) + g(i3) + g(i4) + nu(i6) + e+(i7)
*
* If the overall charge of the process is negative then the momenta of the electron
* and neutrino are reversed.
*
*
      function B3g0W(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      use B3gZ, only : zampB3g0Zvec
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /proc/ isub(7)
      common /BWFlav/ nf1,nf2
!$omp threadprivate(/BWFlav/)
      common /eweakW/ amz,stw,ewfac
!$omp threadprivate(/eweakW/)
      complex*16 zampB3g0WvecLL(2,2,2)

      s67 = s7(i6,i7)

      call eweakW1qq(nf1,nf2,s67)

      zampB3g0WvecLL = zampB3g0Zvec(i1,i2,i3,i4,i5,i6,i7)

      B3g0W = 0d0

      do i=1,2
         do j=1,2
            do k=1,2
               B3g0W = B3g0W + abs(zampB3g0WvecLL(i,j,k))**2
            enddo
         enddo
      enddo

      B3g0W = ewfac*B3g0W

      return
      end

****************************************************************
* quark pair tree level matrix elements for q + Qb -> g + g + g + W+ -> nu + e+
*
* single colour ordering, subleading colour, summed and squared.
*
* q(i1) + Qb(i5) -> g(i2) + g(i3) + g(i4) + nu(i6) + e+(i7)
*
* If the overall charge of the process is negative then the momenta of the electron
* and neutrino are reversed.
*
* Note that gluon (i2) is abelian-like.
*

      function Bt3g0W(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      use B3gZ, only : zampB3g0Zvec
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /proc/ isub(7)
      common /BWFlav/ nf1,nf2
!$omp threadprivate(/BWFlav/)
      common /eweakW/ amz,stw,ewfac
!$omp threadprivate(/eweakW/)
      complex*16 zampLL1(2,2,2),zampLL2(2,2,2),zampLL3(2,2,2)

      s67 = s7(i6,i7)

      call eweakW1qq(nf1,nf2,s67)

      zampLL1 = zampB3g0Zvec(i1,i2,i3,i4,i5,i6,i7)
      zampLL2 = zampB3g0Zvec(i1,i3,i4,i2,i5,i6,i7)
      zampLL3 = zampB3g0Zvec(i1,i3,i2,i4,i5,i6,i7)

      Bt3g0W = 0d0

      do i=1,2
         do j=1,2
            do k=1,2
               zampB3g0ZvecLL = zampLL2(j,k,i)+zampLL3(j,i,k)+zampLL1(i,j,k)
               Bt3g0W = Bt3g0W + abs(zampB3g0ZvecLL)**2
            enddo
         enddo
      enddo

      Bt3g0W = ewfac*Bt3g0W

      return
      end

****************************************************************
* quark pair tree level matrix elements for q + Qb -> g + g + g + W+ -> nu + e+
*
* single colour ordering, sub-subleading colour, summed and squared.
*
* q(i1) + Qb(i5) -> g(i2) + g(i3) + g(i4) + nu(i6) + e+(i7)
*
* If the overall charge of the process is negative then the momenta of the electron
* and neutrino are reversed.
*
* Note that all the gluons are abelian-like.
*

      function Btt3g0W(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      use B3gZ, only : zampB3g0Zvec
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /proc/ isub(7)
      common /BWFlav/ nf1,nf2
!$omp threadprivate(/BWFlav/)
      common /eweakW/ amz,stw,ewfac
!$omp threadprivate(/eweakW/)
      complex*16 zampLL1(2,2,2),zampLL2(2,2,2),zampLL3(2,2,2)
      complex*16 zampLL4(2,2,2),zampLL5(2,2,2),zampLL6(2,2,2)

      s67 = s7(i6,i7)

      call eweakW1qq(nf1,nf2,s67)

      zampLL1 = zampB3g0Zvec(i1,i2,i3,i4,i5,i6,i7)
      zampLL2 = zampB3g0Zvec(i1,i2,i4,i3,i5,i6,i7)
      zampLL3 = zampB3g0Zvec(i1,i4,i2,i3,i5,i6,i7)
      zampLL4 = zampB3g0Zvec(i1,i3,i4,i2,i5,i6,i7)
      zampLL5 = zampB3g0Zvec(i1,i3,i2,i4,i5,i6,i7)
      zampLL6 = zampB3g0Zvec(i1,i4,i3,i2,i5,i6,i7)

      Btt3g0W = 0d0

      do i=1,2
         do j=1,2
            do k=1,2
               zampB3g0ZvecLL =+zampLL1(i,j,k)
     .                      +zampLL2(i,k,j)
     .                      +zampLL3(k,i,j)
     .                      +zampLL4(j,k,i)
     .                      +zampLL5(j,i,k)
     .                      +zampLL6(k,j,i)

               Btt3g0W = Btt3g0W + abs(zampB3g0ZvecLL)**2

            enddo
         enddo
      enddo

      Btt3g0W = ewfac*Btt3g0W

      return
      end
