***********************************************************************
*
* Quark pair one loop level virtual correction matrix element for
*
* q(i1) + Qb(i4) -> g(i2) + g(i3) + W+ -> g(i2) + nu(i4) + e+(i5)
*
* summed and squared over helicities, single colour ordering without
* overall factors and subleading colour
*
* The flavour of the quark pair in this matrix element is determined from the flav variable in the common block /B2gFlav/

      function Bt2g1W(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      use B2gZ
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /eweakW/ amz,stw,ewfac
!$omp threadprivate(/eweakW/)
      common /order/iorder
      common /koppel/ nf,nc,b0
      common /BWFlav/nf1g,nf2g
!$omp threadprivate(/BWFlav/)
      double complex zampTree(2,2),zampqqbgg1(2,2)
      integer ihel(6,4)

      Bt2g1W = 0d0

      s= s6(i5,i6)

      call eweakW1qq(nf1g,nf2g,s)

      zampTree = zampB2g0Z(i1,i2,i3,i4,i5,i6)
      zampqqbgg1 = zampBtt2g1Z(i1,i2,i3,i4,i5,i6,renscale2)

      do ii=1,2
         do jj=1,2
            Bt2g1W = Bt2g1W+dble(Dconjg(zampTree(ii,jj))*zampqqbgg1(ii,jj))
         enddo
      enddo

      Bt2g1W = ewfac*Bt2g1W

c$$$      if (iorder.eq.-1)then
c$$$         subuv = 11d0/6d0
c$$$         Bt2g1W = Bt2g1W - subuv*Bt2g0W(i1,i2,i3,i4,i5,i6)
c      if (iorder.eq.0)then
c         e0 = -pi**2/12d0  -1d0/2d0! pi**2/12 factor from C_gamma expansion
c         Bt2g1W = Bt2g1W - e0*B2g0W(i1,i2,i3,i4,i5,i6)
c      endif

c     (change 24.10) e0 part to be added back 
c      (C(e) expansion related part has to be there !!
c     1/2 comes from a conversion of four-dimensional helicity
c      scheme (FDH) to t Hooft-Veltman (HV) scheme
c     see hep-ph/9708239v1 equation 6.4  has to be there !!
      if (iorder.eq.0)then
         e0 = -pi**2/12d0       ! pi**2/12 factor from C_gamma expansion
         e0=e0+1d0/2d0
         Bt2g1W = Bt2g1W - e0*B2g0W(i1,i2,i3,i4,i5,i6)
      endif
      
      return
      end

***********************************************************************
*
* Quark pair one loop level virtual correction matrix element for
*
* q(i1) + Qb(i4) -> g(i2) + g(i3) + W+ -> g(i2) + nu(i4) + e+(i5)
*
* summed and squared over helicities, single colour ordering without
* overall factors, subleading nf contribution
*
* The flavour of the quark pair in this matrix element is determined from the flav variable in the common block /B2gFlav/
*
* !!THIS FUNCTION has not been validated against MCFM, probably due to numerical stability.
* !!The single unresolved IR behaviour of Bth2g1W has been checked for both implicit and explicit poles
* !!and agree with RV antenna subtraction terms.
*

      function Bth2g1W(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      use B2gZ, only : zampB2g0Z,zampBh2g1Z
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /eweakW/ amz,stw,ewfac
!$omp threadprivate(/eweakW/)
      common /order/iorder
      common /koppel/ nf,nc,b0
      common /BWFlav/nf1,nf2
!$omp threadprivate(/BWFlav/)
      double complex zampnfLoop(2,2),zampTree(2,2)

      Bth2g1W = 0d0

      s = s6(i5,i6)

      call eweakW1qq(nf1,nf2,s)

      zampTree = zampB2g0Z(i1,i2,i3,i4,i5,i6)
 ! note the gluon swapping
     .          +zampB2g0Z(i1,i3,i2,i4,i5,i6)

      zampnfLoop = zampBh2g1Z(i1,i2,i3,i4,i5,i6,renscale2)
! note the gluon swapping
     .            +zampBh2g1Z(i1,i3,i2,i4,i5,i6,renscale2)

      do ii=1,2
         do jj=1,2
            Bth2g1W = Bth2g1W+dble(dconjg(zampTree(ii,jj))*zampnfLoop(ii,jj))
         enddo
      enddo

      Bth2g1W = ewfac*Bth2g1W

      if (iorder.eq.-1) then
         subuv = 2d0/3d0
         Bth2g1W=subuv*Bt2g0W(i1,i2,i3,i4,i5,i6)
      endif
      
      return
      end

***********************************************************************
*
* Quark pair one loop level virtual correction matrix element for
*
* q(i1) + Qb(i4) -> g(i2) + g(i3) + W+ -> g(i2) + nu(i4) + e+(i5)
*
* summed and squared over helicities, single colour ordering without
* overall factors, finite top corrections subleading colour
*
* The flavour of the quark pair in this matrix element is determined from the flav variable in the common block /B2gFlav/

      function Bt2g1Wtop(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      use B2gZ, only : zampB2g0Z,zFaxqqbgg,zFaxslqqbgg
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /masstop/emt
      common /order/iorder
      common /koppel/ nf,nc,b0

      if (emt.eq.0d0)then
         write(*,*) "Error in finite top correction to B2g1Z,
     . the mass of the top has not been defined"
         stop
      endif

      Bt2g1Wtop = 0d0

      if (iorder.eq.0)then
         Bt2g1Wtop = (1d0/20d0)*s6(i2,i3)/emt**2*Bth2g1W(i1,i2,i3,i4,i5,i6,renscale2)
      endif

      return
      end

