**********************************************************************
*
* Returns the subleading colour matrix element for the process
*
* q(i1) + Qbar(i4) ->  + q`(i3) + Qb`(i2) + nu(i5) + e+(i6)
*
* There is only a subleading colour term for matching quark pairs
*
* Ignoring overall factors
*
* The W boson couples to the i1 and i4 pair of quarks
*


      function D0g0W(i1,i2,i3,i4,i5,i6)
      use KinData_mod
      use C0gZ, only : zampC0g0Z
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      complex*16 zampA,zampB

      D0g0W = 0d0

      if(nf1.eq.nf2)then

      call eweakV2qq(nf1,nf2,s6(i5,i6))

c     q L, e L
      zampA = zewfac1(1)*zampC0g0Z(i1,i4,i3,i2,i5,i6)
      zampB = zewfac1(1)*zampC0g0Z(i1,i2,i3,i4,i5,i6)
      D0g0W =-2d0*(+dble(zampA*dconjg(zampB)))

      else
         write(*,*) "nf1!=nf2 in D0g0W"
         stop
      endif

      return
      end

**********************************************************************
*
* Returns the subleading colour matrix element for the process
*
* q(i1) + Qbar(i4) ->  + q`(i3) + Qb`(i2) + nu(i5) + e+(i6)
*
* There is only a subleading colour term for matching quark pairs
*
* Ignoring overall factors
*
* The W boson couples to the i1 and i4 pair of quarks
*
* Sets nf2 = nf1, for use in subtraction terms where nf1 != nf2

      function f1D0g0W(i1,i2,i3,i4,i5,i6)
      use KinData_mod
      use C0gZ, only : zampC0g0Z
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      complex*16 zampA,zampB

      nf2orig = nf2 ! store original value of nf2
      nf2 = nf1

      f1D0g0W = 0d0

      if(nf1.eq.nf2)then

      call eweakV2qq(nf1,nf2,s6(i5,i6))

c     q L, e L
      zampA = zewfac1(1)*zampC0g0Z(i1,i4,i3,i2,i5,i6)
      zampB = zewfac1(1)*zampC0g0Z(i1,i2,i3,i4,i5,i6)
      f1D0g0W =-2d0*(+dble(zampA*dconjg(zampB)))
      else
         write(*,*) "nf1!=nf2 in D0g0W"
         stop
      endif

      nf2 = nf2orig ! reset nf2

      return
      end

