****************************************************************************
* A wrapper function for the C1g0W matrix element symmetrised over the
* second quark pair. This is required for certain subtraction terms
* (e.g. qg initiated nf piece).
*
* qg configuration => symmetrised
* qbg configuration => symmetrised
* anything else => not symmetrised
*
* This is the analogue of the sC1g0Z symmetrisation wrapper for the W case

      function sC1g0W(i1,i2,i3,i4,i5,i6,i7)
      use Process_mod, only : nPDFs_proc
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      select case(nPDFs_proc)
      case(2)
      if((i2.le.2).and.((i1.le.2).or.(i5.le.2)))then
         sC1g0W = 0.5d0*(C1g0W(i1,i2,i3,i4,i5,i6,i7)
     .                  +C1g0W(i1,i2,i4,i3,i5,i6,i7))
!qpg or qpbg initiated.
!Symmetrised over the other quark pair.
      elseif((i2.le.2).and.((i3.le.2).or.(i4.le.2)))then
         sC1g0W = 0.5d0*(C1g0W(i1,i2,i3,i4,i5,i6,i7)
     .                  +C1g0W(i5,i2,i3,i4,i1,i6,i7))
      else
         sC1g0W = C1g0W(i1,i2,i3,i4,i5,i6,i7)
      endif
      case(1)
c     symmetrising for q initiated case for DIS W exchange
         if(((i1.eq.1).or.(i5.eq.1)))then
            sC1g0W = 0.5d0*(C1g0W(i1,i2,i3,i4,i5,i6,i7)
     .           +C1g0W(i1,i2,i4,i3,i5,i6,i7))
         elseif(((i3.eq.1).or.(i4.eq.1)))then
            sC1g0W = 0.5d0*(C1g0W(i1,i2,i3,i4,i5,i6,i7)
     .           +C1g0W(i5,i2,i3,i4,i1,i6,i7))
         else
            sC1g0W = C1g0W(i1,i2,i3,i4,i5,i6,i7)
         endif

      case default
         sC1g0W = C1g0W(i1,i2,i3,i4,i5,i6,i7)
      end select

      return
      end

****************************************************************************
* A wrapper function for the Ct1g0W matrix element symmetrised over the
* second quark pair. This is required for certain subtraction terms
* (e.g. qg initiated nf/nc**2 piece)
* Note: we include Ctt1g0W in this function as well!
*
* qg configuration => symmetrised
* qbg configuration => symmetrised
* anything else => not symmetrised
*
* This is the analogue of the sCt1g0Z symmetrisation wrapper for the W case

      function sCt1g0W(i1,i2,i3,i4,i5,i6,i7)
      use Process_mod, only : nPDFs_proc
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      select case(nPDFs_proc)

      case(2)
      if((i2.le.2).and.((i1.le.2).or.(i5.le.2)))then
         sCt1g0W = 0.5d0*(Ct1g0W(i1,i2,i3,i4,i5,i6,i7)
     .        +Ct1g0W(i1,i2,i4,i3,i5,i6,i7)
     .        -Ctt1g0W(i1,i2,i3,i4,i5,i6,i7)
     .        -Ctt1g0W(i1,i2,i4,i3,i5,i6,i7))
!qpg or qpbg initiated.
!Symmetrised over the other quark pair.
      elseif((i2.le.2).and.((i3.le.2).or.(i4.le.2)))then
         sCt1g0W = 0.5d0*(Ct1g0W(i1,i2,i3,i4,i5,i6,i7)
     .        +Ct1g0W(i5,i2,i3,i4,i1,i6,i7)
     .        -Ctt1g0W(i1,i2,i3,i4,i5,i6,i7)
     .        -Ctt1g0W(i5,i2,i3,i4,i1,i6,i7))
      else
         sCt1g0W = Ct1g0W(i1,i2,i3,i4,i5,i6,i7)
     .            -Ctt1g0W(i1,i2,i3,i4,i5,i6,i7)
      endif
      case(1)
c     symmetrising for q initiated case for DIS W exchange
         if(((i1.eq.1).or.(i5.eq.1)))then
            sCt1g0W = 0.5d0*(Ct1g0W(i1,i2,i3,i4,i5,i6,i7)
     .           +Ct1g0W(i1,i2,i4,i3,i5,i6,i7)
     .           -Ctt1g0W(i1,i2,i3,i4,i5,i6,i7)
     .           -Ctt1g0W(i1,i2,i4,i3,i5,i6,i7))
         elseif(((i3.eq.1).or.(i4.eq.1)))then
            sCt1g0W = 0.5d0*(Ct1g0W(i1,i2,i3,i4,i5,i6,i7)
     .           +Ct1g0W(i5,i2,i3,i4,i1,i6,i7)
     .           -Ctt1g0W(i1,i2,i3,i4,i5,i6,i7)
     .           -Ctt1g0W(i5,i2,i3,i4,i1,i6,i7))
         else
            sCt1g0W = Ct1g0W(i1,i2,i3,i4,i5,i6,i7)
         endif

      case default
         sCt1g0W = Ct1g0W(i1,i2,i3,i4,i5,i6,i7)
      end select

      return
      end
****************************************************************************
* Function defined for the unsymmetrised WJ process. For consistency
* between the symmetrised and unsymmetrised WJ process, we
* renamed the MEs sXXgXW -> XXgXW.
* But sCt1g0W contains Ctt1g0W contribution so we define here
* u(nsymmetrysed)Ct1g0W to take that into account
* 

      function uCt1g0W(i1,i2,i3,i4,i5,i6,i7)
      use Process_mod, only : nPDFs_proc
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      uCt1g0W = Ct1g0W(i1,i2,i3,i4,i5,i6,i7)
     .     -Ctt1g0W(i1,i2,i3,i4,i5,i6,i7)

      return
      end



****************************************************************************
* A matrix element wrapper for two non-identical qqb pairs and an electron pair.
*
* q(i1) + qb(i5) -> Q(i4) + Qb(i3) + g(i2) + e+(i6) + e-(i7)
*
* At tree level, leading colour. Note that this includes both colour
* orderings for the gluon position. Whilst this matrix element can be decomposed
* further, there is little point given that the subtraction term must be summed over
* both colour orderings anyway. -> gluon production from both quark lines is included
*
* This changes the quark line flavours so that they are compatible with the equivalent
* Z ME.
*
* The W boson couples to the i1 and i5 pair of quarks.
*

      function C1g0W(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      use C1gZ, only : genC1g0Zamp
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer orignf2
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)

      if (nf2.le.2) then
      C1g0W = C1g0Z(i1,i2,i3,i4,i5,i6,i7)
c The C type amplitudes can't take nf2 = 4, so it needs to be swapped here.
c It should always be different to nf1 such that all amplitudes are included in
c the matrix element, i.e (nf1,nf2) = (1,2) or (2,1)
      elseif (nf2.eq.4) then
         orignf2 = nf2
         if (nf1.eq.1) then
            nf2 = 2
         elseif (nf1.eq.2)then
            nf2 = 1
         endif
         C1g0W = C1g0Z(i1,i2,i3,i4,i5,i6,i7)
         nf2 = orignf2
      endif
      return
      end

****************************************************************************
* A matrix element wrapper for two non-identical qqb pairs and an electron pair.
*
* q(i1) + qb(i5) -> Q(i4) + Qb(i3) + g(i2) + e+(i6) + e-(i7)
*
* At tree level, subleading colour.
* This changes the quark line flavours so that they are compatible with the
* equivalent Z ME.
*
* The W boson couples to the i1 and i5 pair of quarks.
*

      function Ct1g0W(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      use C1gZ, only : genC1g0Zamp
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer orignf2
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)

      if (nf2.le.2) then
      Ct1g0W = Ct1g0Z(i1,i2,i3,i4,i5,i6,i7)
c The C type amplitudes can't take nf2 = 4, so it needs to be swapped here.
c It should always be different to nf1 such that all amplitudes are included in
c the matrix element, i.e (nf1,nf2) = (1,2) or (2,1)
      elseif (nf2.eq.4) then
         orignf2 = nf2
         if (nf1.eq.1) then
            nf2 = 2
         elseif (nf1.eq.2)then
            nf2 = 1
         endif
         Ct1g0W = Ct1g0Z(i1,i2,i3,i4,i5,i6,i7)
         nf2 = orignf2
      endif
      return
      end

****************************************************************************
* A matrix element wrapper for two non-identical qqb pairs and an electron pair.
*
* q(i1) + qb(i5) -> Q(i4) + Qb(i3) + g(i2) + e+(i6) + e-(i7)
*
* At tree level, subsubleading colour.
* This changes the quark line flavours so that they are compatible with the
* equivalent Z ME.
*
* The W boson couples to the i1 and i5 pair of quarks.
*

      function Ctt1g0W(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      use C1gZ, only : genC1g0Zamp
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer orignf2
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)

      if (nf2.le.2) then
      Ctt1g0W = Ctt1g0Z(i1,i2,i3,i4,i5,i6,i7)
c The C type amplitudes can't take nf2 = 4, so it needs to be swapped here.
c It should always be different to nf1 such that all amplitudes are included in
c the matrix element, i.e (nf1,nf2) = (1,2) or (2,1)
      elseif (nf2.eq.4) then
         orignf2 = nf2
         if (nf1.eq.1) then
            nf2 = 2
         elseif (nf1.eq.2)then
            nf2 = 1
         endif
         Ctt1g0W = Ctt1g0Z(i1,i2,i3,i4,i5,i6,i7)
         nf2 = orignf2
      endif
      return
      end

************************************************************************
* A wrapper function for B0g0Z which inherits the flavour of the first
* pair of quarks from the C0g0Z common block.

      function f1B0g0W(i1,i2,i3,i4)
      use KinData_mod
      implicit double precision (a-h,o-z)
      common /CZFlav/nf1,nf2
!$omp threadprivate(/CZFlav/)
      common /BZFlav/nf1g
!$omp threadprivate(/BZFlav/)

      if(nf1.ne.0)then
         nf1g = nf1
         f1B0g0W = B0g0W(i1,i2,i3,i4)
      else
         write(*,*) "Error in f1B0g0W:
     $flavours have not been initialised"
         stop
      endif

      return
      end

****************************************************************************
* A wrapper function for the C0g0W matrix element symmetrised over the
* second quark pair. This is required for certain subtraction terms
* (e.g. qg initiated nf piece).
*
* qg configuration => symmetrised
* qbg configuration => symmetrised
* anything else => not symmetrised
*
* This is the analogue of the sC0g0Z symmetrisation wrapper for the W case

      function sC0g0W(i1,i2,i3,i4,i5,i6)
      use Process_mod, only : nPDFs_proc
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      select case(nPDFs_proc)
      case(2)
         stop "sC0g0W Not implemented for non-DIS" 
      case(1)
c     symmetrising for q initiated case for DIS W exchange
         if(((i1.eq.1).or.(i4.eq.1)))then
            sC0g0W = 0.5d0*(C0g0W(i1,i2,i3,i4,i5,i6)
     .           +C0g0W(i1,i3,i2,i4,i5,i6))
         elseif(((i2.eq.1).or.(i3.eq.1)))then
            sC0g0W = 0.5d0*(C0g0W(i1,i2,i3,i4,i5,i6)
     .           +C0g0W(i4,i2,i3,i1,i5,i6))
         else
            sC0g0W = C0g0W(i1,i2,i3,i4,i5,i6)
         endif

      case default
         sC0g0W = C0g0W(i1,i2,i3,i4,i5,i6)
      end select

      return
      end
