*     Matrix elements for D0g1Wa/b, Dt0g1Wa/b terms. a/b are charge conjugate-esque
*     processes.
*
*     Validated against Madgraph for W+ production DW 04/17.
*
****************************************************
* Returns the virtual one-loop matrix element for the processes
*
*      u(i1) ub(i2) to ub(i3) d(i4) nu(i5)  lp(i6)
*      d(i1) db(i2) to db(i3) u(i4) lm(i5) nub(i6)
*
* Ignoring overall factors, leading colour interference quark contribution
*
* The W boson couples to the i1 and i4 pair of quarks
*

      double precision function D0g1Wa(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      use C0gZ
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /order/iorder
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      complex*16 zampTree,zampTreeA
      complex*16 zampTreeA2
      complex*16 zampLoop,zampLoopA(2,2,2)
      complex*16 zampLoopA2(2,2,2)
      complex*16 zampLoop2,zampTree2

      call eweakV2qq(nf1,nf2,s6(i5,i6))

      if (nf1.ne.nf2)then
         write(*,*)"nf1 != nf2 in D0g1Wa. "
         write(*,*)"Both quark lines must be the same flavour for interference terms."
         stop
      endif

!     Line reversal for D0g1Wa type ME included in these amplitude calls

      zampTreeA = dconjg(zampC0g0Z(i4,i3,i2,i1,i6,i5))
      zampLoopA = zampC0g1ZLoop(i4,i1,i3,i2,i6,i5,renscale2,2)

c     Crossed tree amplitudes

      zampTreeA2 = dconjg(zampC0g0Z(i4,i1,i2,i3,i6,i5))
      zampLoopA2 = zampC0g1ZLoop(i4,i3,i1,i2,i6,i5,renscale2,2)

!     Fill only the LLL amplitudes. These correspond to our RRR amplitudes because ?
!     But it works...
      zampLoop =+zewfac1(1)*zampLoopA(2,2,2)
      zampTree =+zewfac1(1)*zampTreeA

      zampLoop2 =+zewfac1(1)*zampLoopA2(2,2,2)
      zampTree2 =+zewfac1(1)*zampTreeA2

      D0g1Wa= -dble(zampTree*dconjg(zampLoop))
     .        -dble(zampTree2*dconjg(zampLoop2))

      if(iorder.eq.-1)then
!     UV renormalisation (MSbar scheme)
         subuv = 11d0/3d0
         D0g1Wa = D0g1Wa + subuv*D0g0Wa(i1,i2,i3,i4,i5,i6)

      elseif (iorder.eq.0)then
!     The conversion from FDH -> HV is +2/3, see hep-ph/9708239v1 equation 6.5
!     +pi**2/6 from Cgamma expansion.
         e0 = +pi**2/6d0 - (2d0/3d0)
         D0g1Wa = D0g1Wa - e0*D0g0Wa(i1,i2,i3,i4,i5,i6)
      endif

      return
      end

****************************************************
* Returns the virtual one-loop matrix element for the processes
*
*      u(i1) ub(i2) to ub(i3) d(i4) nu(i5)  lp(i6)
*      d(i1) db(i2) to db(i3) u(i4) lm(i5) nub(i6)
*
* Ignoring overall factors, subleading colour interference quark contribution
*
* The W boson couples to the i1 and i4 pair of quarks

      double precision function Dt0g1Wa(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      use C0gZ
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /order/iorder
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      complex*16 zampTreeA
      complex*16 zampTreeA2
      complex*16 zampLoopA(2,2,2)
      complex*16 zampLoopA2(2,2,2)
      complex*16 zampTree,zampLoop
      complex*16 zampLoop2,zampTree2

      if (nf1.ne.nf2)then
         write(*,*)"nf1 != nf2 in Dt0g1Wa. "
         write(*,*)"Both quark lines must be the same flavour for interference terms."
         stop
      endif

      call eweakV2qq(nf1,nf2,s6(i5,i6))

!     Line reversal for D0g1Wa type ME included in these amplitude calls
      zampTreeA = dconjg(zampC0g0Z(i4,i3,i2,i1,i6,i5))

!     1/N^2 amplitudes in A6;2 (e.q 2.8) hep-ph/9610370v1
      zampLoopA =+zampC0g1ZLoop(i4,i1,i3,i2,i6,i5,renscale2,2)
     .           +zampC0g1ZLoop(i4,i2,i3,i1,i6,i5,renscale2,1)
     .           -zampC0g1ZLoopSL(i4,i2,i3,i1,i6,i5,renscale2)

c     Crossed amplitudes

      zampTreeA2 = dconjg(zampC0g0Z(i4,i1,i2,i3,i6,i5))

      zampLoopA2 =+zampC0g1ZLoop(i4,i3,i1,i2,i6,i5,renscale2,2)
     .            +zampC0g1ZLoop(i4,i2,i1,i3,i6,i5,renscale2,1)
     .            -zampC0g1ZLoopSL(i4,i2,i1,i3,i6,i5,renscale2)


!     Fill only the LLL amplitudes. These correspond to our RRR amplitudes because ?
!     But it works...
      zampTree=+zewfac1(1)*zampTreeA2
      zampLoop=+zewfac1(1)*zampLoopA2(2,2,2)

      zampTree2=+zewfac1(1)*zampTreeA
      zampLoop2=+zewfac1(1)*zampLoopA(2,2,2)

      Dt0g1Wa= -dble(zampTree*dconjg(zampLoop))
     .         -dble(zampTree2*dconjg(zampLoop2))

      if (iorder.eq.0)then
!     The conversion from FDH -> HV is +1, see hep-ph/9708239v1 equation 6.5
!     -pi**2/6 from Cgamma expansion.
         e0 = -pi**2/6d0+1d0
         Dt0g1Wa = Dt0g1Wa - e0*D0g0Wa(i1,i2,i3,i4,i5,i6)
      endif

      return
      end


********************************************************
*     Purely tree level amplitudes, so can be passed straight through to the Z case
*     with line reversal

* FIX 23/2/18 -> Need a - sign in some invariants for the Wa case due to the crossings used
      
      double precision function Dh0g1Wa(i4,i3,i2,i1,i6,i5,renscale2)
      use KinData_mod
      use C0gZ
      implicit none      
      common /eweakZ/ amz,zewfac1,zewfac2
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      common /order/iorder
      integer, intent(in) :: i1,i2,i3,i4,i5,i6
      double precision, intent(in) :: renscale2
      integer :: nf1,nf2,iorder
      double precision :: amz
      complex*16,dimension(4) :: zewfac1,zewfac2 
      complex*16 :: zzlograt
      complex*16 :: zfacB,zfacB2,zfacA,zfacA2
      complex*16 zampTreeA(2,2,2),zampTreeB(2,2,2)
      complex*16 zampTree(2,2,2),zampLoopnf(2,2,2)
      complex*16 zampTreeA2(2,2,2),zampTreeB2(2,2,2)
      complex*16 zampTree2(2,2,2),zampLoopnf2(2,2,2)

      Dh0g1Wa = 0d0

      call eweakV2qq(nf1,nf2,s6(i5,i6))
      if(nf1.eq.nf2)then

      zampTreeA = zampC0g1ZTree(i1,i2,i3,i4,i5,i6)
      zampTreeB = zampC0g1ZTree(i2,i1,i4,i3,i5,i6)
      zampTreeA2 = zampC0g1ZTree(i1,i4,i3,i2,i5,i6)
      zampTreeB2 = zampC0g1ZTree(i4,i1,i2,i3,i5,i6)

      if(iorder.eq.0)then

*          We must compute the tree level matrix element before the electroweak interference. The reason for this is that the factors that multiply the QCD amplitudes are scale dependent.
         !> These s6 invariants have a relative -ve sign from those in D0g1Z.f used by Dh0g1Wb
         zfacA = 2d0/3d0*zzLogRat(renscale2,s6(i2,i3))+(10d0/9d0)
         zfacB = 2d0/3d0*zzLogRat(renscale2,s6(i1,i4))+(10d0/9d0)
         zfacA2 = 2d0/3d0*zzLogRat(renscale2,s6(i3,i4))+(10d0/9d0)
         zfacB2 = 2d0/3d0*zzLogRat(renscale2,s6(i1,i2))+(10d0/9d0)

         zampTree(1,1,1) = +zewfac1(1)*zampTreeA(1,1,1)
         zampLoopnf(1,1,1) = +zfacA2*zewfac1(1)*zampTreeA2(1,1,1)
         zampTree2(1,1,1) = +zewfac1(1)*zampTreeA2(1,1,1)
         zampLoopnf2(1,1,1) = +zfacA*zewfac1(1)*zampTreeA(1,1,1)

         Dh0g1Wa = Dh0g1Wa -dble(zampTree(1,1,1)*dconjg(zampLoopnf(1,1,1)))
     .        -dble(zampTree2(1,1,1)*dconjg(zampLoopnf2(1,1,1)))

      endif
      endif

      return
      end

****************************************************
* Returns the virtual one-loop matrix element for the processes
*
*      u(i1) db(i2) to db(i3) d(i4) nu(i5)  lp(i6)
*      d(i1) ub(i2) to ub(i3) u(i4) lm(i5) nub(i6)
*
* Ignoring overall factors, leading colour interference quark contribution
*
* The W boson couples to the i1 and i4 pair of quarks
*

      double precision function D0g1Wb(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      use C0gZ
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /order/iorder
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      complex*16 zampTree,zampTreeA
      complex*16 zampTreeA2
      complex*16 zampLoop,zampLoopA(2,2,2)
      complex*16 zampLoopA2(2,2,2)
      complex*16 zampLoop2,zampTree2

      call eweakV2qq(nf1,nf2,s6(i5,i6))

      if (nf1.ne.nf2)then
         write(*,*)"nf1 != nf2 in D0g1Wb. "
         write(*,*)"Both quark lines must be the same flavour for interference terms."
         stop
      endif

         zampTreeA = -zampC0g0Z(i1,i4,i3,i2,i5,i6)
         zampLoopA = zampC0g1ZLoop(i4,i3,i1,i2,i5,i6,renscale2,2)

!     Crossed amplitudes

         zampTreeA2 = -zampC0g0Z(i1,i2,i3,i4,i5,i6)
         zampLoopA2 = zampC0g1ZLoop(i2,i3,i1,i4,i5,i6,renscale2,2)

!     Fill only the LLL amplitudes. These correspond to our RRL amplitudes because ?
!     But it works...
         zampLoop =+zewfac1(1)*zampLoopA(2,2,1)
         zampTree =+zewfac1(1)*zampTreeA

         zampLoop2 =+zewfac1(1)*zampLoopA2(2,2,1)
         zampTree2 =+zewfac1(1)*zampTreeA2

         D0g1Wb = -dble(zampTree*dconjg(zampLoop))
     .            -dble(zampTree2*dconjg(zampLoop2))

      if(iorder.eq.-1)then
!     UV renormalisation (MSbar scheme)
         subuv = 11d0/3d0
         D0g1Wb = D0g1Wb + subuv*D0g0Wb(i1,i2,i3,i4,i5,i6) !Need crossing for D0g1Wb types

      elseif (iorder.eq.0)then
!     The conversion from FDH -> HV is +2/3, see hep-ph/9708239v1 equation 6.5
!     +pi**2/6 from Cgamma expansion.
         e0 = +pi**2/6d0 - (2d0/3d0)
         D0g1Wb = D0g1Wb - e0*D0g0Wb(i1,i2,i3,i4,i5,i6) !Need crossing for D0g1Wb types
      endif

      return
      end

****************************************************
* Returns the virtual one-loop matrix element for the process
*
*      u(i1) db(i2) to db(i3) d(i4) nu(i5)  lp(i6)
*      d(i1) ub(i2) to ub(i3) u(i4) lm(i5) nub(i6)
*
* Ignoring overall factors, subleading colour interference quark contribution
*
* The W boson couples to the i1 and i4 pair of quarks

      double precision function Dt0g1Wb(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      use C0gZ
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /order/iorder
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      complex*16 zampTreeA
      complex*16 zampTreeA2
      complex*16 zampLoopA(2,2,2)
      complex*16 zampLoopA2(2,2,2)
      complex*16 zampTree,zampLoop
      complex*16 zampLoop2,zampTree2

      if (nf1.ne.nf2)then
         write(*,*)"nf1 != nf2 in Dt0g1Wb. "
         write(*,*)"Both quark lines must be the same flavour for interference terms."
         stop
      endif

      call eweakV2qq(nf1,nf2,s6(i5,i6))

         zampTreeA = -zampC0g0Z(i1,i4,i3,i2,i5,i6)

!     1/N^2 amplitudes in A6;2 (e.q 2.8) hep-ph/9610370v1

         zampLoopA =+zampC0g1ZLoop(i4,i3,i1,i2,i5,i6,renscale2,2)
     .              +zampC0g1ZLoop(i4,i2,i1,i3,i5,i6,renscale2,1)
     .              -zampC0g1ZLoopSL(i4,i2,i1,i3,i5,i6,renscale2)

!     Crossed tree amplitudes
         zampTreeA2 = -zampC0g0Z(i1,i2,i3,i4,i5,i6)

         zampLoopA2 =+zampC0g1ZLoop(i2,i3,i1,i4,i5,i6,renscale2,2)
     .               +zampC0g1ZLoop(i2,i4,i1,i3,i5,i6,renscale2,1)
     .               -zampC0g1ZLoopSL(i2,i4,i1,i3,i5,i6,renscale2)

!     Fill only the LLL amplitudes. These correspond to our RRL amplitudes because ?
!     But it works...
      zampTree=+zewfac1(1)*zampTreeA2
      zampLoop=+zewfac1(1)*zampLoopA2(2,2,1)

      zampTree2=+zewfac1(1)*zampTreeA
      zampLoop2=+zewfac1(1)*zampLoopA(2,2,1)

      Dt0g1Wb= -dble(zampTree*dconjg(zampLoop))
     .         -dble(zampTree2*dconjg(zampLoop2))

      if (iorder.eq.0)then
!     The conversion from FDH -> HV is +1, see hep-ph/9708239v1 equation 6.5
!     -pi**2/6 from Cgamma expansion.
         e0 = -pi**2/6d0+1d0
         Dt0g1Wb = Dt0g1Wb - e0*D0g0Wb(i1,i2,i3,i4,i5,i6)
      endif

      return
      end

********************************************************
*     Purely tree level amplitudes, so can be passed straight through to the Z case

      double precision function Dh0g1Wb(i1,i2,i3,i4,i5,i6,renscale)
      use Process_mod, only : iboson_proc
      implicit none
      integer i1,i2,i3,i4,i5,i6
      double precision Dh0g1Z
      double precision renscale

      if(abs(iboson_proc).eq.1)then
          Dh0g1Wb=Dh0g1Z(i1,i2,i3,i4,i5,i6,renscale)
      else
         write(*,*) "Error in Dh0g1Wb: iboson_proc parameter incorrectly initialised"
         stop
      endif

      return
      end

*********************************************************
*                     W+/- Wrappers                     *
*********************************************************

      double precision function D0g1Wpa(i1,i2,i3,i4,i5,i6,renscale)
      use Process_mod, only : iboson_proc
      implicit none
      integer i1,i2,i3,i4,i5,i6
      double precision D0g1Wa
      double precision renscale

      if(abs(iboson_proc).eq.1)then
          D0g1Wpa=D0g1Wa(i1,i2,i3,i4,i5,i6,renscale)
      else
         write(*,*) "Error in D0g1Wpa: iboson_proc parameter incorrectly initialised"
         stop
      endif

      return
      end


      double precision function D0g1Wma(i1,i2,i3,i4,i5,i6,renscale)
      use Process_mod, only : iboson_proc
      implicit none
      integer i1,i2,i3,i4,i5,i6
      double precision D0g1Wa
      double precision renscale

      if(abs(iboson_proc).eq.1)then
          D0g1Wma=D0g1Wa(i1,i2,i3,i4,i5,i6,renscale)
      else
         write(*,*) "Error in D0g1Wma: iboson_proc parameter incorrectly initialised"
         stop
      endif

      return
      end

      double precision function D0g1Wpb(i1,i2,i3,i4,i5,i6,renscale)
      use Process_mod, only : iboson_proc
      implicit none
      integer i1,i2,i3,i4,i5,i6
      double precision D0g1Wb
      double precision renscale

      if(abs(iboson_proc).eq.1)then
          D0g1Wpb=D0g1Wb(i1,i2,i3,i4,i5,i6,renscale)
      else
         write(*,*) "Error in D0g1Wpb: iboson_proc parameter incorrectly initialised"
         stop
      endif

      return
      end


      double precision function D0g1Wmb(i1,i2,i3,i4,i5,i6,renscale)
      use Process_mod, only : iboson_proc
      implicit none
      integer i1,i2,i3,i4,i5,i6
      double precision D0g1Wb
      double precision renscale

      if(abs(iboson_proc).eq.1)then
          D0g1Wmb=D0g1Wb(i1,i2,i3,i4,i5,i6,renscale)
      else
         write(*,*) "Error in D0g1Wmb: iboson_proc parameter incorrectly initialised"
         stop
      endif

      return
      end

************************************************************

      double precision function Dt0g1Wpa(i1,i2,i3,i4,i5,i6,renscale)
      use Process_mod, only : iboson_proc
      implicit none
      integer i1,i2,i3,i4,i5,i6
      double precision Dt0g1Wa
      double precision renscale

      if(abs(iboson_proc).eq.1)then
          Dt0g1Wpa=Dt0g1Wa(i1,i2,i3,i4,i5,i6,renscale)
      else
         write(*,*) "Error in Dt0g1Wpa: iboson_proc parameter incorrectly initialised"
         stop
      endif

      return
      end


      double precision function Dt0g1Wma(i1,i2,i3,i4,i5,i6,renscale)
      use Process_mod, only : iboson_proc
      implicit none
      integer i1,i2,i3,i4,i5,i6
      double precision Dt0g1Wa
      double precision renscale

      if(abs(iboson_proc).eq.1)then
          Dt0g1Wma=Dt0g1Wa(i1,i2,i3,i4,i5,i6,renscale)
      else
         write(*,*) "Error in Dt0g1Wma: iboson_proc parameter incorrectly initialised"
         stop
      endif

      return
      end

      double precision function Dt0g1Wpb(i1,i2,i3,i4,i5,i6,renscale)
      use Process_mod, only : iboson_proc
      implicit none
      integer i1,i2,i3,i4,i5,i6
      double precision Dt0g1Wb
      double precision renscale

      if(abs(iboson_proc).eq.1)then
          Dt0g1Wpb=Dt0g1Wb(i1,i2,i3,i4,i5,i6,renscale)
      else
         write(*,*) "Error in Dt0g1Wpb: iboson_proc parameter incorrectly initialised"
         stop
      endif

      return
      end


      double precision function Dt0g1Wmb(i1,i2,i3,i4,i5,i6,renscale)
      use Process_mod, only : iboson_proc
      implicit none
      integer i1,i2,i3,i4,i5,i6
      double precision Dt0g1Wb
      double precision renscale

      if(abs(iboson_proc).eq.1)then
          Dt0g1Wmb=Dt0g1Wb(i1,i2,i3,i4,i5,i6,renscale)
      else
         write(*,*) "Error in Dt0g1Wmb: iboson_proc parameter incorrectly initialised"
         stop
      endif

      return
      end

************************************************************

      double precision function Dh0g1Wpa(i1,i2,i3,i4,i5,i6,renscale)
      use Process_mod, only : iboson_proc
      implicit none
      integer i1,i2,i3,i4,i5,i6
      double precision Dh0g1Wa
      double precision renscale

      if(abs(iboson_proc).eq.1)then
          Dh0g1Wpa=Dh0g1Wa(i1,i2,i3,i4,i5,i6,renscale)
      else
         write(*,*) "Error in Dh0g1Wpa: iboson_proc parameter incorrectly initialised"
         stop
      endif

      return
      end


      double precision function Dh0g1Wma(i1,i2,i3,i4,i5,i6,renscale)
      use Process_mod, only : iboson_proc
      implicit none
      integer i1,i2,i3,i4,i5,i6
      double precision Dh0g1Wa
      double precision renscale

      if(abs(iboson_proc).eq.1)then
          Dh0g1Wma=Dh0g1Wa(i1,i2,i3,i4,i5,i6,renscale)
      else
         write(*,*) "Error in Dh0g1Wma: iboson_proc parameter incorrectly initialised"
         stop
      endif

      return
      end

      double precision function Dh0g1Wpb(i1,i2,i3,i4,i5,i6,renscale)
      use Process_mod, only : iboson_proc
      implicit none
      integer i1,i2,i3,i4,i5,i6
      double precision Dh0g1Wb
      double precision renscale

      if(abs(iboson_proc).eq.1)then
          Dh0g1Wpb=Dh0g1Wb(i1,i2,i3,i4,i5,i6,renscale)
      else
         write(*,*) "Error in Dh0g1Wpb: iboson_proc parameter incorrectly initialised"
         stop
      endif

      return
      end


      double precision function Dh0g1Wmb(i1,i2,i3,i4,i5,i6,renscale)
      use Process_mod, only : iboson_proc
      implicit none
      integer i1,i2,i3,i4,i5,i6
      double precision Dh0g1Wb
      double precision renscale

      if(abs(iboson_proc).eq.1)then
          Dh0g1Wmb=Dh0g1Wb(i1,i2,i3,i4,i5,i6,renscale)
      else
         write(*,*) "Error in Dh0g1Wmb: iboson_proc parameter incorrectly initialised"
         stop
      endif

      return
      end
