****************************************************
* quark pair tree level matrix elements for q + qbar --> e^- + e^+
*
*
* summed over colour orderings, summed and squared
*
*                    q(1) qbar(2) e-(3) e+(4)
*

      function FullB0g0Z(i1,i2,i3,i4)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)
      common /eweakZ/ amz,zewfac(4),zewnull(4)
!$omp threadprivate(/eweakZ/)

      fac = 0.25d0/nc**2

      fac = fac * (4d0*(4d0*pi*amz)**2)*nc

      FullB0g0Z = fac*B0g0Z(i1,i2,i3,i4)


c      write(*,*) "FullB0g0Z",FullB0g0Z,i1,i2,i3,i4

      return
      end

****************************************************
* A wrapper function to sum over the four helicity combinations
* of the quark/lepton pair
*
* q(1) q(2) -> e+(3) e-(4)
*

      function B0g0Z(i1,i2,i3,i4)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      double precision B0g0Zvec(4)
      common /eweakZ/ amz,zewfac(4),zewnull(4)
!$omp threadprivate(/eweakZ/)
      common /BZFlav/nflav
!$omp threadprivate(/BZFlav/)

      s = s4(i3,i4)
      t = s4(i1,i3)
      u = s4(i1,i4)

      call eweakV1qq(nflav,s)

      B0g0Zvec(1) = (u/s)**2
      B0g0Zvec(2) = (t/s)**2
      B0g0Zvec(3) = B0g0Zvec(2)
      B0g0Zvec(4) = B0g0Zvec(1)

      B0g0Z = sum(abs(zewfac)**2*B0g0Zvec)

      return
      end

*     symmetrised B0g0Z
      function sB0g0Z(i1,i2,i3,i4)
      implicit double precision (a-h,o-y)

      sB0g0Z=0.5d0*(B0g0Z(i1,i2,i3,i4)+B0g0Z(i2,i1,i3,i4))
      return
      end

************************************************************************
* A wrapper function for B0g0Z which inherits the flavour of the first
* pair of quarks from the C0g0Z common block.

      function f1B0g0Z(i1,i2,i3,i4)
      use KinData_mod
      implicit double precision (a-h,o-z)
      common /CZFlav/nf1,nf2
!$omp threadprivate(/CZFlav/)
      common /BZFlav/nf1g
!$omp threadprivate(/BZFlav/)

      if(nf1.ne.0)then
         nf1g = nf1
         f1B0g0Z = B0g0Z(i1,i2,i3,i4)
      else
         write(*,*) "Error in f1B0g0Z:
     $flavours have not been initialised"
         stop
      endif

      return
      end

************************************************************************
* A wrapper function for B0g0Z which inherits the flavour of the second
* pair of quarks from the C0g0Z common block.

      function f2B0g0Z(i1,i2,i3,i4)
      use KinData_mod
      implicit double precision (a-h,o-z)
      common /CZFlav/nf1,nf2
!$omp threadprivate(/CZFlav/)
      common /BZFlav/nf1g
!$omp threadprivate(/BZFlav/)

      if(nf1.ne.0)then
         nf1g = nf2
         f2B0g0Z = B0g0Z(i1,i2,i3,i4)
      else
         write(*,*) "Error in f2B0g0Z:
     $flavours have not been initialised"
         stop
      endif

      return
      end

* symmetrised

      function sf1B0g0Z(i1,i2,i3,i4)
      use KinData_mod
      implicit double precision (a-h,o-z)
      common /CZFlav/nf1,nf2
!$omp threadprivate(/CZFlav/)
      common /BZFlav/nf1g
!$omp threadprivate(/BZFlav/)

      if(nf1.ne.0)then
         nf1g = nf1
         sf1B0g0Z = 0.5d0*(B0g0Z(i1,i2,i3,i4)+B0g0Z(i2,i1,i3,i4))
      else
         write(*,*) "Error in f1B0g0Z:
     $flavours have not been initialised"
         stop
      endif

      return
      end


* symmetrised

      function sf2B0g0Z(i1,i2,i3,i4)
      use KinData_mod
      implicit double precision (a-h,o-z)
      common /CZFlav/nf1,nf2
!$omp threadprivate(/CZFlav/)
      common /BZFlav/nf1g
!$omp threadprivate(/BZFlav/)

      if(nf1.ne.0)then
         nf1g = nf2
         sf2B0g0Z = 0.5d0*(B0g0Z(i1,i2,i3,i4)+B0g0Z(i2,i1,i3,i4))
      else
         write(*,*) "Error in sf2B0g0Z:
     $flavours have not been initialised"
         stop
      endif

      return
      end

