****************************************************
* Quark pair tree level matrix elements for q + qbar -> g + e^- + e^+
*
* summed over colour orderings, summed and squared
*
*     q(1)  g(2) qbar(3) e-(4) e+(5)
*

      function FullB1g0Z(i1,i2,i3,i4,i5)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16(z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)
      common /eweakZ/ amz,zewfac(4),zewnull(4)
!$omp threadprivate(/eweakZ/)

*     if(isub(1).eq.0)return

      fac = get_avg(i1,i3)

      fac = fac*(4d0*(nc**2 - 1)*((4d0*pi*amz)**2)*4d0*pi*as)

      FullB1g0Z = fac*B1g0Z(i1,i2,i3,i4,i5)

c     write(*,*) "FullB1g0Z",FullB1g0Z,i1,i2,i3,i4,i5

      return
      end

************************************************************************
*
* Quark pair tree level matrix elements for q + qbar -> g + e^- + e^+
*
* summed over colour orderings, summed and squared, ignoring overall factors
*
*     q(1) g(2) qbar(3) e-(4) e+(5)
*
* The flavour of the quark pair in this matrix element is determined from the flav variable in the common block /BZFlav/

      function B1g0Z(i1,i2,i3,i4,i5)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      double complex zamps(4)
      parameter(pi=3.141592653589793238d0)
      double precision B1g0Zvec(4)
      common /eweakZ/ amz,zewfac(4),zewnull(4)
!$omp threadprivate(/eweakZ/)
      common /BZFlav/nflav
!$omp threadprivate(/BZFlav/)

      za34 = zA5(i3,i4)

      za14 = zA5(i1,i4)
      za32 = zA5(i3,i2)
      za31 = zA5(i3,i1)
      zb51 = zB5(i5,i1)
      zb52 = zB5(i5,i2)

      zb51 = zB5(i5,i1)
      zb12 = zB5(i1,i2)
      zb32 = zB5(i3,i2)
      za34 = zA5(i3,i4)
      zb31 = zB5(i3,i1)
      za42 = zA5(i4,i2)
      zb12 = zB5(i1,i2)

      za35 = zA5(i3,i5)
      za12 = zA5(i1,i2)
      za31 = zA5(i3,i1)
      zb41 = zB5(i4,i1)
      za32 = zA5(i3,i2)
      zb42 = zB5(i4,i2)

      za52 = zA5(i5,i2)

      s54 = s5(i4,i5)

      call eweakV1qq(nflav,s54)

      zamps(1) = za34/za12/za32*(za31*zb51+za32*zb52)
      zamps(2) = zb51/zb12/zb32*(za34*zb31+za42*zb12)

      zamps(3) = za35/za12/za32*(za31*zb41+za32*zb42)
      zamps(4) = zb41/zb12/zb32*(za35*zb31+za52*zb12)


      B1g0Zvec(1) = (abs(zamps(1))**2+abs(zamps(2))**2)/s54**2
      B1g0Zvec(2) = (abs(zamps(3))**2+abs(zamps(4))**2)/s54**2
      B1g0Zvec(3) = B1g0Zvec(2)
      B1g0Zvec(4) = B1g0Zvec(1)


      B1g0Zvec = abs(zewfac)**2*B1g0Zvec

      B1g0Z = sum(B1g0Zvec)


      return
      end





************************************************************************
* A wrapper function for B1g0Z which inherits the flavour of the first
* pair of quarks from the C0g0Z common block.
*
* This must be called in the maple code for 4 quark processes where the
* reduced matrix element has the flavour of the first pair of quarks
*
* NB: This will need further modification for NNLO 4quark+1g processes.

      function f1B1g0Z(i1,i2,i3,i4,i5)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /CZFlav/nf1,nf2
!$omp threadprivate(/CZFlav/)
      common /BZFlav/nf1g
!$omp threadprivate(/BZFlav/)

      if(nf1.ne.0)then
         nf1g = nf1
         f1B1g0Z = B1g0Z(i1,i2,i3,i4,i5)
      else
         write(*,*) "Error in f1B1g0Z: flavours have not been initialised"
         stop
      endif
      return
      end

************************************************************************
* A wrapper function for B1g0Z which inherits the flavour of the second
* pair of quarks from the C0g0Z common block.
*
* This must be called in the maple code for 4 quark processes where the
* reduced matrix element has the flavour of the second pair of quarks
*
* NB: This will need further modification for NNLO 4quark+1g processes.

      function f2B1g0Z(i1,i2,i3,i4,i5)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /CZFlav/nf1,nf2
!$omp threadprivate(/CZFlav/)
      common /BZFlav/nf1g
!$omp threadprivate(/BZFlav/)

      if(nf2.ne.0)then
         nf1g = nf2
         f2B1g0Z = B1g0Z(i1,i2,i3,i4,i5)
      else
         write(*,*) "Error in f2B1g0Z: flavours have not been initialised"
         stop
      endif

      return
      end

************************************************************************
*
* A symmetrised version of the B1g0Z over the qqb pair.
*

      function sB1g0Z(i1,i2,i3,i4,i5)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      sB1g0Z = 0.5d0*(B1g0Z(i1,i2,i3,i4,i5)
     .               +B1g0Z(i3,i2,i1,i4,i5))


      return
      end

************************************************************************
*
* A symmetrised version of the B1g0Z over the qqb pair, taking the
* flavour from the first quark pair of C-type matrix elements
*

      function sf1B1g0Z(i1,i2,i3,i4,i5)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      sf1B1g0Z = 0.5d0*(f1B1g0Z(i1,i2,i3,i4,i5)
     .                 +f1B1g0Z(i3,i2,i1,i4,i5))


      return
      end

************************************************************************
*
* A symmetrised version of the B1g0Z over the qqb pair, taking the
* flavour from the second quark pair of C-type matrix elements
*

      function sf2B1g0Z(i1,i2,i3,i4,i5)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      sf2B1g0Z = 0.5d0*(f2B1g0Z(i1,i2,i3,i4,i5)
     .                 +f2B1g0Z(i3,i2,i1,i4,i5))


      return
      end

*******************************************************************
* A flavour averaged version of B1g0Z. This is required in certain
* subtraction terms at RV and VV.
*

      function fB1g0Z(i1,i2,i3,i4,i5)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer ni(2)
      double complex zamps(4)
      parameter(pi=3.141592653589793238d0)
      double precision B1g0Zvec(4),ewfacstore(4)
      data ni /3,2/ ! this is nup and ndown, hard coded for now
      common /koppel/ nf,nc,b0
      common /eweakZ/ amz,zewfac(4),zewnull(4)
!$omp threadprivate(/eweakZ/)
      common /BZFlav/nflav
!$omp threadprivate(/BZFlav/)

      za34 = zA5(i3,i4)

      za14 = zA5(i1,i4)
      za32 = zA5(i3,i2)
      za31 = zA5(i3,i1)
      zb51 = zB5(i5,i1)
      zb52 = zB5(i5,i2)

      zb51 = zB5(i5,i1)
      zb12 = zB5(i1,i2)
      zb32 = zB5(i3,i2)
      za34 = zA5(i3,i4)
      zb31 = zB5(i3,i1)
      za42 = zA5(i4,i2)
      zb12 = zB5(i1,i2)

      za35 = zA5(i3,i5)
      za12 = zA5(i1,i2)
      za31 = zA5(i3,i1)
      zb41 = zB5(i4,i1)
      za32 = zA5(i3,i2)
      zb42 = zB5(i4,i2)

      za52 = zA5(i5,i2)

      s54 = s5(i4,i5)



      zamps(1) = za34/za12/za32*(za31*zb51+za32*zb52)
      zamps(2) = zb51/zb12/zb32*(za34*zb31+za42*zb12)

      zamps(3) = za35/za12/za32*(za31*zb41+za32*zb42)
      zamps(4) = zb41/zb12/zb32*(za35*zb31+za52*zb12)

      B1g0Zvec(1) = (abs(zamps(1))**2+abs(zamps(2))**2)/s54**2
      B1g0Zvec(2) = (abs(zamps(3))**2+abs(zamps(4))**2)/s54**2
      B1g0Zvec(3) = B1g0Zvec(2)
      B1g0Zvec(4) = B1g0Zvec(1)

      ewfacstore=0d0

      do i=1,2
         call eweakV1qq(i,s54)
         ewfacstore = ewfacstore+(dble(ni(i))/dble(nf))*abs(zewfac)**2
      enddo

      B1g0Zvec = ewfacstore*B1g0Zvec

      fB1g0Z = sum(B1g0Zvec)

      return
      end


