*********************************************************************
*
*
* Quark pair one loop level virtual correction matrix element for
*
* q(i1) qb(i3) -> g(i2) e-(i4) e+(i5)
*
* summed and squared over helicities, all colour levels including overall factors
*
* The flavour of the quark pair in this matrix element is determined from the flav variable in the common block /BZFlav/

      function FullB1g1Z(i1,i2,i3,i4,i5,renscale2)
      use KinData_mod
      use B1gZ, only : zampB1g1Z
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /eweakZ/ amz,zewfac(4),zewnull(4)
!$omp threadprivate(/eweakZ/)
      common /order/iorder
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /BZFlav/nf1g
!$omp threadprivate(/BZFlav/)

      fac = get_avg(i1,i3)
      fac = fac*8d0*(nc**2-1d0)/(2d0*nc)*nc**2*(4*pi*as)*(4*pi*amz)**2
      fac = fac*as/(2*pi)


      FullB1g1Z = B1g1Z(i1,i2,i3,i4,i5,renscale2)
     .           -(1d0/nc**2)*Bt1g1Z(i1,i2,i3,i4,i5,renscale2)

      if (iorder.eq.-1)then
         FullB1g1Z = FullB1g1Z
     .              +dble(nf)/nc*Bh1g1Z(i1,i2,i3,i4,i5,renscale2) ! renormalisation nf component
      endif

      FullB1g1Z = fac*FullB1g1Z

      return
      end

*********************************************************************
*
*
* Quark pair one loop level virtual correction matrix element for
*
* q(i1) qb(i3) -> g(i2) e-(i4) e+(i5)
*
* summed and squared over helicities, single colour ordering without
* overall factors
*
* The flavour of the quark pair in this matrix element is determined from the flav variable in the common block /BZFlav/

      function B1g1Z(i1,i2,i3,i4,i5,renscale2)
      use KinData_mod
      use B1gZ, only : zampB1g1Z
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /eweakZ/ amz,zewfac(4),zewnull(4)
!$omp threadprivate(/eweakZ/)
      common /order/iorder
      common /koppel/ nf,nc,b0
      common /BZFlav/nf1g
!$omp threadprivate(/BZFlav/)
      double precision B1g1Zvec(4)
      double complex zamp(4)
      integer ihel(5,4)

      ihel = reshape((/i1,i2,i3,i5,i4,
     .                 i1,i2,i3,i4,i5,
     .                 i3,i2,i1,i5,i4,
     .                 i3,i2,i1,i4,i5/),(/5,4/))

      do i=1,4
         zamp = zampB1g1Z(ihel(1,i),ihel(2,i),ihel(3,i),ihel(4,i),ihel(5,i),renscale2)
         B1g1Zvec(i) = dble(dconjg(zamp(1))*zamp(2))+dble(dconjg(zamp(3))*zamp(4))

      enddo

      s = s5(i4,i5)

      call eweakV1qq(nf1g,s)

c      write(6,*) 'old:',B1g1Zvec(1:4)

      B1g1Zvec = abs(zewfac)**2*B1g1Zvec

      B1g1Z  = sum(B1g1Zvec)

c     A UV counter term at order 1/epsilon which returns the matrix element in the MS bar scheme.
      if (iorder .eq. -1) then
         subuv = 11d0/6d0
         B1g1Z = B1g1Z - subuv*B1g0Z(i1,i2,i3,i4,i5)
      elseif (iorder .eq. 0) then
c     pi**2/6d0 comes from a correction for C(e)
c     1/2 comes from a conversion of four-dimensional helicity scheme (FDH) to 't Hooft-Veltman (HV) scheme
c     see hep-ph/9708239v1 equation 6.4
         e0 = -pi**2/6d0 + 0.5d0
         B1g1Z = B1g1Z - e0*B1g0Z(i1,i2,i3,i4,i5)
      endif

      return
      end


*****************************************************************************
*    Symmetrised B1g1Z

      double precision function csB1g1Z(i1,i2,i3,i4,i5,renscale2)
      implicit none
      integer i1,i2,i3,i4,i5
      double precision B1g1Z,renscale2
      if(i2.gt.2) then
         csB1g1Z=B1g1Z(i1,i2,i3,i4,i5,renscale2)
      else
         csB1g1Z=0.5d0*(B1g1Z(i1,i2,i3,i4,i5,renscale2)+
     $     B1g1Z(i3,i2,i1,i4,i5,renscale2))
      endif

      return
      end

*****************************************************************************
*    Symmetrised Bt1g1Z

      double precision function csBt1g1Z(i1,i2,i3,i4,i5,renscale2)
      implicit none
      integer i1,i2,i3,i4,i5
      double precision Bt1g1Z,renscale2

      if(i2.gt.2) then
         csBt1g1Z=Bt1g1Z(i1,i2,i3,i4,i5,renscale2)
      else
         csBt1g1Z=0.5d0*(Bt1g1Z(i1,i2,i3,i4,i5,renscale2)+
     $     Bt1g1Z(i3,i2,i1,i4,i5,renscale2))
      endif

      return
      end

*****************************************************************************
*    Symmetrised Bh1g1Z

      double precision function csBh1g1Z(i1,i2,i3,i4,i5,renscale2)
      implicit none
      integer i1,i2,i3,i4,i5
      double precision Bh1g1Z,renscale2

      if(i2.gt.2) then
         csBh1g1Z=Bh1g1Z(i1,i2,i3,i4,i5,renscale2)
      else
         csBh1g1Z=0.5d0*(Bh1g1Z(i1,i2,i3,i4,i5,renscale2)+
     $     Bh1g1Z(i3,i2,i1,i4,i5,renscale2))
      endif
      return
      end


*****************************************************************************
*    Catani subtracted B1g1Z matrix element.
*    This should really only be used for debugging
*

      function B1g1ZCC(i1,i2,i3,i4,i5,renscale2)
      use KinData_mod
      implicit none
      double precision pi
      parameter(pi=3.141592653589793238d0)
      integer i1,i2,i3,i4,i5,iorder
      double precision renscale2,B1g1ZCC,B1g0Z,B1g1Z
      common /order/iorder

      if(iorder.eq.0)then
         B1g1ZCC=B1g1Z(i1,i2,i3,i4,i5,renscale2)-
     .          (-0.5d0*(log(abs(s5(i1,i2)/renscale2))**2 + log(abs(s5(i2,i3)/renscale2))**2)
     .          +5d0/3d0*(log(abs(s5(i1,i2)/renscale2)) + log(abs(s5(i2,i3)/renscale2)))
     .          )*B1g0Z(i1,i2,i3,i4,i5)

         if((i1.le.2).and.(i3.le.2))then ! qqb
            B1g1ZCC = B1g1ZCC-pi**2/6d0*B1g0Z(i1,i2,i3,i4,i5)
         elseif((i2.le.2).and.((i1.le.2).or.(i3.le.2)))then ! qg or qbg
            B1g1ZCC = B1g1ZCC-pi**2*2d0/3d0*B1g0Z(i1,i2,i3,i4,i5)
         else
            print *, "invalid initial state in B1g1ZCC: ", i1,i2,i3,i4,i5
            stop
         endif
      else
         B1g1ZCC=0d0
      endif


      return
      end

*********************************************************************
*
*
* Quark pair one loop level virtual correction matrix element for
*
* q(i1) qb(i3) -> g(i2) e-(i4) e+(i5)
*
* summed and squared over helicities, single colour ordering without
* overall factors, nf conponent (beta,F piece)
*

      function Bh1g1Z(i1,i2,i3,i4,i5,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /order/iorder

      Bh1g1Z = 0d0

      if(iorder.eq.-1)then
         Bh1g1Z = (1d0/3d0)*B1g0Z(i1,i2,i3,i4,i5) ! renormalisation nf component
      endif

      return
      end




************************************************************************
*
* A symmetrised version of the B1g1Z over the qqb pair.
*

      function sB1g1Z(i1,i2,i3,i4,i5,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      sB1g1Z = 0.5d0*(B1g1Z(i1,i2,i3,i4,i5,renscale2)
     .               +B1g1Z(i3,i2,i1,i4,i5,renscale2))

      return
      end

************************************************************************
*
* A symmetrised version of the Bh1g1Z over the qqb pair.
*

      function sBh1g1Z(i1,i2,i3,i4,i5,renscale2)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /order/iorder

      sBh1g1Z = 0d0

      if(iorder.eq.-1)then
         sBh1g1Z = 0.5d0*(Bh1g1Z(i1,i2,i3,i4,i5,renscale2)
     .                   +Bh1g1Z(i3,i2,i1,i4,i5,renscale2))
      endif

      return
      end



      function sBt1g1Z(i1,i2,i3,i4,i5,renscale2)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      sBt1g1Z = 0.5d0*(Bt1g1Z(i1,i2,i3,i4,i5,renscale2)
     .                +Bt1g1Z(i3,i2,i1,i4,i5,renscale2))

      return
      end

************************************************************************
* A wrapper function for B1g1Z which inherits the flavour of the first
* pair of quarks from the C0g0Z common block.
*
* This must be called in the maple code for 4 quark processes where the
* reduced matrix element has the flavour of the first pair of quarks
*

      function f1B1g1Z(i1,i2,i3,i4,i5,renscale2)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /CZFlav/nf1,nf2
!$omp threadprivate(/CZFlav/)
      common /BZFlav/nf1g
!$omp threadprivate(/BZFlav/)

      if(nf1.ne.0)then
         nf1g = nf1
         f1B1g1Z = B1g1Z(i1,i2,i3,i4,i5,renscale2)
      else
         write(*,*) "Error in f1B1g1Z: flavours have not been initialised"
         stop
      endif

      return
      end

      function sf1B1g1Z(i1,i2,i3,i4,i5,renscale2)
      implicit double precision (a-h,o-y)
      sf1B1g1Z= 0.5d0*(f1B1g1Z(i1,i2,i3,i4,i5,renscale2)+f1B1g1Z(i3,i2,i1,i4,i5,renscale2))
      return
      end

************************************************************************
* A wrapper function for B1g1Z which inherits the flavour of the second
* pair of quarks from the C0g0Z common block.
*
* This must be called in the maple code for 4 quark processes where the
* reduced matrix element has the flavour of the second pair of quarks
*

      function f2B1g1Z(i1,i2,i3,i4,i5,renscale2)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /CZFlav/nf1,nf2
!$omp threadprivate(/CZFlav/)
      common /BZFlav/nf1g
!$omp threadprivate(/BZFlav/)

      if(nf2.ne.0)then
         nf1g = nf2
         f2B1g1Z = B1g1Z(i1,i2,i3,i4,i5,renscale2)
      else
         write(*,*) "Error in f2B1g0Z: flavours have not been initialised"
         stop
      endif

      return
      end

      function sf2B1g1Z(i1,i2,i3,i4,i5,renscale2)
      implicit double precision (a-h,o-y)
      sf2B1g1Z= 0.5d0*(f1B1g1Z(i1,i2,i3,i4,i5,renscale2)+f1B1g1Z(i3,i2,i1,i4,i5,renscale2))
      return
      end


************************************************************************
* A wrapper function for Bh1g1Z which inherits the flavour of the first
* pair of quarks from the C0g0Z common block.
*
* This must be called in the maple code for 4 quark processes where the
* reduced matrix element has the flavour of the second pair of quarks
*

      function f1Bh1g1Z(i1,i2,i3,i4,i5,renscale2)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /order/iorder
      common /CZFlav/nf1,nf2
!$omp threadprivate(/CZFlav/)
      common /BZFlav/nf1g
!$omp threadprivate(/BZFlav/)

      if(nf2.ne.0)then
         nf1g = nf1
         f1Bh1g1Z = Bh1g1Z(i1,i2,i3,i4,i5,renscale2)
      else
         write(*,*) "Error in f1Bh1g0Z: flavours have not been initialised"
         stop
      endif

      return
      end

************************************************************************
* A wrapper function for Bh1g1Z which inherits the flavour of the second
* pair of quarks from the C0g0Z common block.
*
* This must be called in the maple code for 4 quark processes where the
* reduced matrix element has the flavour of the second pair of quarks
*

      function f2Bh1g1Z(i1,i2,i3,i4,i5,renscale2)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /order/iorder
      common /CZFlav/nf1,nf2
!$omp threadprivate(/CZFlav/)
      common /BZFlav/nf1g
!$omp threadprivate(/BZFlav/)

      if(nf2.ne.0)then
         nf1g = nf2
         f2Bh1g1Z = Bh1g1Z(i1,i2,i3,i4,i5,renscale2)
      else
         write(*,*) "Error in f2Bh1g0Z: flavours have not been initialised"
         stop
      endif

      return
      end
