************************************************************************
*
* Quark pair tree level matrix elements for q + qbar -> g + e^- + e^+
*
* summed over colour orderings, summed and squared, ignoring overall factors
*
*     q(1) g(2) qbar(3) e-(4) e+(5)
*
* The flavour of the quark pair in this matrix element is determined from the flav variable in the common block /BZFlav/

      function B1g0Znew(i1,i2,i3,i4,i5)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      double complex zamps(4)
      parameter(pi=3.141592653589793238d0)
      double precision B1g0Zvec(4)
      common /eweakZ/ amz,zewfac(4),zewnull(4)
!$omp threadprivate(/eweakZ/)
      common /ampbasic/zampA(1:4),zampB(1:4),zampC(1:4)
!$omp threadprivate(/ampbasic/)
      common /BZFlav/nflav
!$omp threadprivate(/BZFlav/)
c      common /B1gFlav/nflav
      common /zcoeff/zal(1:4),zbe(1:4),zga(1:4)
!$omp threadprivate(/zcoeff/)

      s54 = s5(i5,i4)
      call eweakV1qq(nflav,s54)
      nloop = 0
      rs2 = 0d0
      call helcoeff(i1,i2,i3,i4,i5,nloop,rs2,0)
      call amp5basic(i1,i2,i3,i4,i5)
      do i=1,4
         zamps(i) = zal(i)*zampA(i)+zbe(i)*zampB(i)+zga(i)*zampC(i)
      enddo

      B1g0Zvec(1) = (abs(zamps(1))**2+abs(zamps(2))**2)/s54**2
      B1g0Zvec(2) = (abs(zamps(3))**2+abs(zamps(4))**2)/s54**2
      B1g0Zvec(3) = B1g0Zvec(2)
      B1g0Zvec(4) = B1g0Zvec(1)

      B1g0Zvec = abs(zewfac)**2*B1g0Zvec
      
      B1g0Znew = sum(B1g0Zvec)

      return
      end


      function B1g1Znew(i1,i2,i3,i4,i5,rs2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      double precision B1gvec(4)
      nloop = 1
      icolor = 1
      call makem0mn(i1,i2,i3,i4,i5,nloop,icolor,rs2,B1gvec)
      B1g1Znew = sum(B1gvec)

      return
      end


      function Bt1g1Znew(i1,i2,i3,i4,i5,rs2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      double precision B1gvec(4)
      nloop = 1
      icolor = 2
      call makem0mn(i1,i2,i3,i4,i5,nloop,icolor,rs2,B1gvec)
      Bt1g1Znew = sum(B1gvec)

      return
      end

      function Bh1g1Znew(i1,i2,i3,i4,i5,rs2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      double precision B1gvec(4)
      nloop = 1
      icolor = 3
      call makem0mn(i1,i2,i3,i4,i5,nloop,icolor,rs2,B1gvec)
      Bh1g1Znew = sum(B1gvec)

      return
      end

c----- the two loop contributions should be multipled in QCDNORM with a factor
c----- factor1vv = factor3/(8d0*pi)**2
c----- and the following relatvive factors:
c----- B1g2Z - 1d0/nc**2*Bt1g2Z + 1/nc**4*Btt1g2Z + nf/nc*Bh1g2Z + nf/nc**3*Bht1g2Z + nf**2/nc**2*Bhh1g2Z + (4/nc-nc)*Bv1g2Z

c TM 06/06/2015, switched Bt1g2Z -> -Bt1g2Z to be consistant with RR and RV

      function B1g2Z(i1,i2,i3,i4,i5,rs2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      double precision B1gvec(4)
      nloop = 2
      icolor = 1
      call makem0mn(i1,i2,i3,i4,i5,nloop,icolor,rs2,B1gvec)
      B1g2Z = sum(B1gvec)
C       print*, "B1g2Z: ", B1g2Z
      call makem1m1(i1,i2,i3,i4,i5,icolor,rs2,B1gvec)
      B1g2Z = B1g2Z + sum(B1gvec)
C       print*, "B1g2Z: ", B1g2Z
C       read*

      return
      end

      function Bt1g2Z(i1,i2,i3,i4,i5,rs2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      double precision B1gvec(4)
      nloop = 2
      icolor = 2
      call makem0mn(i1,i2,i3,i4,i5,nloop,icolor,rs2,B1gvec)
      Bt1g2Z = sum(B1gvec)
      call makem1m1(i1,i2,i3,i4,i5,icolor,rs2,B1gvec)
      Bt1g2Z = Bt1g2Z + sum(B1gvec)

c TM 06/06/2015 sign change
      Bt1g2Z = -Bt1g2Z
      return
      end

      function Btt1g2Z(i1,i2,i3,i4,i5,rs2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      double precision B1gvec(4)
      nloop = 2
      icolor = 3
      call makem0mn(i1,i2,i3,i4,i5,nloop,icolor,rs2,B1gvec)
      Btt1g2Z = sum(B1gvec)
      call makem1m1(i1,i2,i3,i4,i5,icolor,rs2,B1gvec)
      Btt1g2Z = Btt1g2Z + sum(B1gvec)

      return
      end

      function Bh1g2Z(i1,i2,i3,i4,i5,rs2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      double precision B1gvec(4)
      nloop = 2
      icolor = 4
      call makem0mn(i1,i2,i3,i4,i5,nloop,icolor,rs2,B1gvec)
      Bh1g2Z = sum(B1gvec)
      call makem1m1(i1,i2,i3,i4,i5,icolor,rs2,B1gvec)
      Bh1g2Z = Bh1g2Z + sum(B1gvec)

      return
      end

      function Bht1g2Z(i1,i2,i3,i4,i5,rs2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      double precision B1gvec(4)
      nloop = 2
      icolor = 5
      call makem0mn(i1,i2,i3,i4,i5,nloop,icolor,rs2,B1gvec)
      Bht1g2Z = sum(B1gvec)
      call makem1m1(i1,i2,i3,i4,i5,icolor,rs2,B1gvec)
      Bht1g2Z = Bht1g2Z + sum(B1gvec)

      return
      end

      function Bhh1g2Z(i1,i2,i3,i4,i5,rs2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      double precision B1gvec(4)
      nloop = 2
      icolor = 6
      call makem0mn(i1,i2,i3,i4,i5,nloop,icolor,rs2,B1gvec)
      Bhh1g2Z = sum(B1gvec)
      call makem1m1(i1,i2,i3,i4,i5,icolor,rs2,B1gvec)
      Bhh1g2Z = Bhh1g2Z + sum(B1gvec)

      return
      end


      function Bv1g2Z(i1,i2,i3,i4,i5,rs2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      double precision B1gvec(4)
      nloop = 2
      icolor = 7
      call makem0mn(i1,i2,i3,i4,i5,nloop,icolor,rs2,B1gvec)
      Bv1g2Z = sum(B1gvec)

      return
      end

      subroutine makem0mn(i1,i2,i3,i4,i5,nloop,icolor,rs2,B1gvec)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      double complex zamps0(8),zamps0d(8),zamps1(8)
      parameter(pi=3.141592653589793238d0)
      double precision B1gvec(4)
      common /eweakZ/ amz,zewfac(4),zewnull(4)
!$omp threadprivate(/eweakZ/)
      common /vector/zewvec(2)
!$omp threadprivate(/vector/)
      common /ampbasic/zampA(1:4),zampB(1:4),zampC(1:4)
!$omp threadprivate(/ampbasic/)
      common /BZFlav/nflav
!$omp threadprivate(/BZFlav/)
c      common /B1gFlav/nflav
      common /zcoeff/zal(1:4),zbe(1:4),zga(1:4)
!$omp threadprivate(/zcoeff/)
      s54 = s5(i5,i4)
      call eweakV1qq(nflav,s54)
      call amp5basic(i1,i2,i3,i4,i5)
      call helcoeff(i1,i2,i3,i4,i5,0,rs2,0)
      do i=1,4
         zamps0(i) = zal(i)*zampA(i)+zbe(i)*zampB(i)+zga(i)*zampC(i)
         zamps0d(i) = dconjg(zamps0(i))
         zamps0(i+4) = dconjg(zal(i))*zampA(i)
     .                +dconjg(zbe(i))*zampB(i)+dconjg(zga(i))*zampC(i)
         zamps0d(i+4) = dconjg(zamps0(i+4))
      enddo
      call helcoeff(i1,i2,i3,i4,i5,nloop,rs2,icolor)
      do i=1,4
         zamps1(i) = zal(i)*zampA(i)+zbe(i)*zampB(i)+zga(i)*zampC(i)
         zamps1(i+4) = dconjg(zal(i))*zampA(i)
     .                +dconjg(zbe(i))*zampB(i)+dconjg(zga(i))*zampC(i)
      enddo

      B1gvec(1) = 2d0*( dreal(zamps0d(1)*zamps1(1))
     .                  + dreal(zamps0d(2)*zamps1(2)) )/s54**2
      B1gvec(2) = 2d0*( dreal(zamps0d(3)*zamps1(3))
     .                  + dreal(zamps0d(4)*zamps1(4)) )/s54**2
      B1gvec(3) = 2d0*( dreal(zamps0d(7)*zamps1(7))
     .                  + dreal(zamps0d(8)*zamps1(8)) )/s54**2
      B1gvec(4) = 2d0*( dreal(zamps0d(5)*zamps1(5))
     .                  + dreal(zamps0d(6)*zamps1(6)) )/s54**2
      if (icolor.le.6.or.nloop.le.1) then      
         B1gvec = abs(zewfac)**2*B1gvec
      else
         do ii=1,4
            if ((ii.eq.1).or.(ii.eq.3))then
               jj = 1
            else
               jj = 2
            endif
            B1gvec(ii) = dconjg(zewfac(ii))*zewvec(jj)*B1gvec(ii)
         enddo
      endif
      return
      end



      subroutine makem1m1(i1,i2,i3,i4,i5,icolor,rs2,B1gvec)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      double complex zamps0(8),zamps0d(8),zamps1(8),zamps1d(8)
      parameter(pi=3.141592653589793238d0)
      double precision B1gvec(4)
      common /eweakZ/ amz,zewfac(4),zewnull(4)
!$omp threadprivate(/eweakZ/)
      common /ampbasic/zampA(1:4),zampB(1:4),zampC(1:4)
!$omp threadprivate(/ampbasic/)
      common /BZFlav/nflav
!$omp threadprivate(/BZFlav/)
c      common /B1gFlav/nflav
      common /zcoeff/zal(1:4),zbe(1:4),zga(1:4)
!$omp threadprivate(/zcoeff/)
      common /zcoeff2/zal2(1:4),zbe2(1:4),zga2(1:4)
!$omp threadprivate(/zcoeff2/)
      s54 = s5(i5,i4)
      call eweakV1qq(nflav,s54)
      call amp5basic(i1,i2,i3,i4,i5)
      call helcoeff(i1,i2,i3,i4,i5,11,rs2,icolor)
      do i=1,4
         zamps1(i) = zal(i)*zampA(i)+zbe(i)*zampB(i)+zga(i)*zampC(i)
         zamps1(i+4) = dconjg(zal(i))*zampA(i)
     .                +dconjg(zbe(i))*zampB(i)+dconjg(zga(i))*zampC(i)
         zamps1d(i) = zal2(i)*zampA(i)+zbe2(i)*zampB(i)+zga2(i)*zampC(i)
         zamps1d(i) = dconjg(zamps1d(i))
         zamps1d(i+4) = dconjg(zal2(i))*zampA(i)
     .                 +dconjg(zbe2(i))*zampB(i)+dconjg(zga2(i))*zampC(i)
         zamps1d(i+4) = dconjg(zamps1d(i+4))
      enddo

      B1gvec(1) = ( dreal(zamps1d(1)*zamps1(1))
     .              + dreal(zamps1d(2)*zamps1(2)) )/s54**2
      B1gvec(2) = ( dreal(zamps1d(3)*zamps1(3))
     .              + dreal(zamps1d(4)*zamps1(4)) )/s54**2
      B1gvec(3) = ( dreal(zamps1d(7)*zamps1(7))
     .              + dreal(zamps1d(8)*zamps1(8)) )/s54**2
      B1gvec(4) = ( dreal(zamps1d(5)*zamps1(5))
     .              + dreal(zamps1d(6)*zamps1(6)) )/s54**2

      B1gvec = abs(zewfac)**2*B1gvec
      return
      end



      subroutine helcoeff(i1,i2,i3,i4,i5,nloop,rs2,icolor)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit double complex (z)
      dimension u(2),v(2),kinregion(2)
      common /order/ieorder
      common /order2l/ipole
      double complex zamps(4),za(1:30)
      double complex a_a1,a_b1,a_c1,
     $               b_a1,b_b1,b_c1,
     $               c_a1,c_b1,c_c1,
     $               A_a2,A_b2,A_c2,
     $               B_a2,B_b2,B_c2,
     $               C_a2,C_b2,C_c2,
     $               D_a2,D_b2,D_c2,
     $               E_a2,E_b2,E_c2,
     $               F_a2,F_b2,F_c2,
     $               G_a2,G_b2,G_c2

      parameter(pi=3.141592653589793238d0)
      common /zcoeff/zal(1:4),zbe(1:4),zga(1:4)
!$omp threadprivate(/zcoeff/)
      common /zcoeff2/zal2(1:4),zbe2(1:4),zga2(1:4)
!$omp threadprivate(/zcoeff2/)

c      ipole = 0   ! finite parts
c      ipole = -4  ! 1/e^4 pole
c      ipole = -3  ! 1/e^3 pole (only for NF-type pieces)

      if (nloop.eq.0) then
         do i=1,4
            zal(i) = (1d0,0d0)
            zbe(i) = (1d0,0d0)
            zga(i) = (0d0,0d0)
         enddo
         return
      endif

      zone = (1d0,0d0)
      zero = (0d0,0d0)
      zipi = (0d0,pi)
      srt2 = dsqrt(2d0)

      if (ipole.eq.-4) then
         do i=1,2
            if (nloop.eq.1) then
               if (icolor.eq.1) then ! N
                  zal(i) = -zone
                  zbe(i) = -zone
                  zga(i) = zero
               elseif (icolor.eq.2) then ! -1/N
                  zal(i) = -0.5d0*zone
                  zbe(i) = -0.5d0*zone
                  zga(i) = zero
               elseif (icolor.eq.3) then ! NF
                  zal(i) = zero
                  zbe(i) = zero
                  zga(i) = zero
               endif
            endif
            if (nloop.eq.11) then
               if (icolor.eq.1) then ! N^2
                  zal(i) = -zone
                  zbe(i) = -zone
                  zga(i) = zero
                  zal2(i) = zal(i)
                  zbe2(i) = zbe(i)
                  zga2(i) = zga(i)
               elseif (icolor.eq.2) then ! N^0
                  zal(i) = -srt2*zone
                  zbe(i) = -srt2*zone
                  zga(i) = zero
                  zal2(i) = 0.5d0*srt2*zone
                  zbe2(i) = 0.5d0*srt2*zone
                  zga2(i) = zero
               elseif (icolor.eq.3) then ! N^-2
                  zal(i) = 0.5d0*zone
                  zbe(i) = 0.5d0*zone
                  zga(i) = zero
                  zal2(i) = zal(i)
                  zbe2(i) = zbe(i)
                  zga2(i) = zga(i)
               elseif (icolor.eq.4) then ! NF*N
                  zal(i) = -srt2*zone
                  zbe(i) = -srt2*zone
                  zga(i) = zero
                  zal2(i) = zero
                  zbe2(i) = zero
                  zga2(i) = zero
               elseif (icolor.eq.5) then ! NF/N
                  zal(i) = 0.5d0*srt2*zone
                  zbe(i) = 0.5d0*srt2*zone
                  zga(i) = zero
                  zal2(i) = zero
                  zbe2(i) = zero
                  zga2(i) = zero
               elseif (icolor.eq.6) then ! NF^2
                  zal(i) = zero
                  zbe(i) = zero
                  zga(i) = zero
                  zal2(i) = zal(i)
                  zbe2(i) = zbe(i)
                  zga2(i) = zga(i)
               endif
            endif
            if (nloop.eq.2) then
               if (icolor.eq.1) then ! N^2
                  zal(i) = 0.5d0*zone
                  zbe(i) = 0.5d0*zone
                  zga(i) = zero
               elseif (icolor.eq.2) then ! N^0
                  zal(i) = -0.5d0*zone
                  zbe(i) = -0.5d0*zone
                  zga(i) = zero
               elseif (icolor.eq.3) then ! N^-2
                  zal(i) = zone/8d0
                  zbe(i) = zone/8d0
                  zga(i) = zero
               elseif (icolor.eq.4) then ! NF*N
                  zal(i) = zero
                  zbe(i) = zero
                  zga(i) = zero
               elseif (icolor.eq.5) then ! NF/N
                  zal(i) = zero
                  zbe(i) = zero
                  zga(i) = zero
               elseif (icolor.eq.6) then ! NF^2
                  zal(i) = zero
                  zbe(i) = zero
                  zga(i) = zero
               elseif (icolor.eq.7) then ! NFZ
                  zal(i) = zero
                  zbe(i) = zero
                  zga(i) = zero
               endif
            endif
            zal(i+2) = zal(i)
            zbe(i+2) = zbe(i)
            zga(i+2) = zga(i)
            zal2(i+2) = zal2(i)
            zbe2(i+2) = zbe2(i)
            zga2(i+2) = zga2(i)
         enddo
         return
      endif

      if (ipole.eq.-3) then
         do i=1,2
            if (nloop.eq.11) then
               if (icolor.eq.1) then ! N^2
                  zal(i) = zero
                  zbe(i) = zero
                  zga(i) = zero
                  zal2(i) = zal(i)
                  zbe2(i) = zbe(i)
                  zga2(i) = zga(i)
               elseif (icolor.eq.2) then ! N^0
                  zal(i) = zero
                  zbe(i) = zero
                  zga(i) = zero
                  zal2(i) = zero
                  zbe2(i) = zero
                  zga2(i) = zero
               elseif (icolor.eq.3) then ! N^-2
                  zal(i) = zero
                  zbe(i) = zero
                  zga(i) = zero
                  zal2(i) = zal(i)
                  zbe2(i) = zbe(i)
                  zga2(i) = zga(i)
               elseif (icolor.eq.4) then ! NF*N
                  zal(i) = -srt2*zone
                  zbe(i) = -srt2*zone
                  zga(i) = zero
                  zal2(i) = srt2/6d0*zone
                  zbe2(i) = srt2/6d0*zone
                  zga2(i) = zero
               elseif (icolor.eq.5) then ! NF/N
                  zal(i) = 0.5d0*srt2*zone
                  zbe(i) = 0.5d0*srt2*zone
                  zga(i) = zero
                  zal2(i) = srt2/6d0*zone
                  zbe2(i) = srt2/6d0*zone
                  zga2(i) = zero
               elseif (icolor.eq.6) then ! NF^2
                  zal(i) = zero
                  zbe(i) = zero
                  zga(i) = zero
                  zal2(i) = zal(i)
                  zbe2(i) = zbe(i)
                  zga2(i) = zga(i)
               endif
            endif
            if (nloop.eq.2) then
               if (icolor.eq.1) then ! N^2
                  zal(i) = zero
                  zbe(i) = zero
                  zga(i) = zero
               elseif (icolor.eq.2) then ! N^0
                  zal(i) = zero
                  zbe(i) = zero
                  zga(i) = zero
               elseif (icolor.eq.3) then ! N^-2
                  zal(i) = zero
                  zbe(i) = zero
                  zga(i) = zero
               elseif (icolor.eq.4) then ! NF*N
                  zal(i) = -5d0/12d0*zone
                  zbe(i) = -5d0/12d0*zone
                  zga(i) = zero
               elseif (icolor.eq.5) then ! NF/N
                  zal(i) = 5d0/24d0*zone
                  zbe(i) = 5d0/24d0*zone
                  zga(i) = zero
               elseif (icolor.eq.6) then ! NF^2
                  zal(i) = zero
                  zbe(i) = zero
                  zga(i) = zero
               elseif (icolor.eq.7) then ! NFZ
                  zal(i) = zero
                  zbe(i) = zero
                  zga(i) = zero
               endif
            endif
            zal(i+2) = zal(i)
            zbe(i+2) = zbe(i)
            zga(i+2) = zga(i)
            zal2(i+2) = zal2(i)
            zbe2(i+2) = zbe2(i)
            zga2(i+2) = zga2(i)
         enddo
         return
      endif

      if (ipole.eq.-2) then
         do i=1,2
            if (nloop.eq.11) then
               if (icolor.eq.1) then ! N^2
                  zal(i) = zero
                  zbe(i) = zero
                  zga(i) = zero
                  zal2(i) = zal(i)
                  zbe2(i) = zbe(i)
                  zga2(i) = zga(i)
               elseif (icolor.eq.2) then ! N^0
                  zal(i) = zero
                  zbe(i) = zero
                  zga(i) = zero
                  zal2(i) = zero
                  zbe2(i) = zero
                  zga2(i) = zero
               elseif (icolor.eq.3) then ! N^-2
                  zal(i) = zero
                  zbe(i) = zero
                  zga(i) = zero
                  zal2(i) = zal(i)
                  zbe2(i) = zbe(i)
                  zga2(i) = zga(i)
               elseif (icolor.eq.4) then ! NF*N
                  zal(i) = zero
                  zbe(i) = zero
                  zga(i) = zero
                  zal2(i) = zero
                  zbe2(i) = zero
                  zga2(i) = zero
               elseif (icolor.eq.5) then ! NF/N
                  zal(i) = zero
                  zbe(i) = zero
                  zga(i) = zero
                  zal2(i) = zero
                  zbe2(i) = zero
                  zga2(i) = zero
               elseif (icolor.eq.6) then ! NF^2
                  zal(i) = -1d0/6d0*zone
                  zbe(i) = -1d0/6d0*zone
                  zga(i) = zero
                  zal2(i) = zal(i)
                  zbe2(i) = zbe(i)
                  zga2(i) = zga(i)
               endif
            endif
            if (nloop.eq.2) then
               if (icolor.eq.1) then ! N^2
                  zal(i) = zero
                  zbe(i) = zero
                  zga(i) = zero
               elseif (icolor.eq.2) then ! N^0
                  zal(i) = zero
                  zbe(i) = zero
                  zga(i) = zero
               elseif (icolor.eq.3) then ! N^-2
                  zal(i) = zero
                  zbe(i) = zero
                  zga(i) = zero
               elseif (icolor.eq.4) then ! NF*N
                  zal(i) = zero
                  zbe(i) = zero
                  zga(i) = zero
               elseif (icolor.eq.5) then ! NF/N
                  zal(i) = zero
                  zbe(i) = zero
                  zga(i) = zero
               elseif (icolor.eq.6) then ! NF^2
                  zal(i) = 1d0/24d0*zone
                  zbe(i) = 1d0/24d0*zone
                  zga(i) = zero
               elseif (icolor.eq.7) then ! NFZ
                  zal(i) = zero
                  zbe(i) = zero
                  zga(i) = zero
               endif
            endif
            zal(i+2) = zal(i)
            zbe(i+2) = zbe(i)
            zga(i+2) = zga(i)
            zal2(i+2) = zal2(i)
            zbe2(i+2) = zbe2(i)
            zga2(i+2) = zga2(i)
         enddo
         return
      endif



      s12 = s5(i1,i3)
      s13 = s5(i1,i2)
      s23 = s5(i2,i3)
      s45 = s5(i4,i5)
c      write(6,*) s12,s13,s23,s45,rs2

      zl12 = zzLogRat(-s12,rs2)
      zl13 = zzLogRat(-s13,rs2)
      zl23 = zzLogRat(-s23,rs2)
      dlren = dlog(abs(s45)/rs2)

c----- determine the kinematical region
      if (s45.gt.0d0) then
         if (s12.gt.0d0) iregion = 2  ! 2a
         if (s13.gt.0d0) iregion = 3  ! 3a
         if (s23.gt.0d0) iregion = 4  ! 4a
         if (min(s12,s13,s23).gt.0d0) iregion = 1  ! 1a
      else
         if (s12.gt.0d0.and.s13.gt.s45.and.s23.gt.s45) iregion = 5  ! 1b
         if (s12.gt.0d0.and.s13.le.s45.and.s23.le.s45) iregion = 14 ! 2d
         if (s12.gt.0d0.and.s13.le.s45.and.s23.gt.s45) iregion = 11 ! 3c
         if (s12.gt.0d0.and.s13.gt.s45.and.s23.le.s45) iregion = 12 ! 4c
         if (s13.gt.0d0.and.s12.gt.s45.and.s23.gt.s45) iregion = 9  ! 1c
         if (s13.gt.0d0.and.s12.le.s45.and.s23.gt.s45) iregion = 6  ! 2b
         if (s13.gt.0d0.and.s12.le.s45.and.s23.le.s45) iregion = 15 ! 3d
         if (s13.gt.0d0.and.s12.gt.s45.and.s23.le.s45) iregion = 8  ! 4b
         if (s23.gt.0d0.and.s12.gt.s45.and.s13.gt.s45) iregion = 13 ! 1d
         if (s23.gt.0d0.and.s12.le.s45.and.s13.gt.s45) iregion = 10 ! 2c
         if (s23.gt.0d0.and.s12.gt.s45.and.s13.le.s45) iregion = 7  ! 3b
         if (s23.gt.0d0.and.s12.le.s45.and.s13.le.s45) iregion = 16 ! 4d
      endif

      if(iregion.eq.1) then      ! 1a
         kinregion(1) = 1
         kinregion(2) = 1
         u(1) =  s13/s45
         v(1) =  s23/s45
         u(2) = v(1)
         v(2) = u(1)
      elseif(iregion.eq.2) then  ! 2a
         kinregion(1) = 2
         kinregion(2) = 2
         u(1) = -s13/s12
         v(1) =  s45/s12
         u(2) = -s23/s12
         v(2) = v(1)
      elseif(iregion.eq.3) then  ! 3a
         kinregion(1) = 3
         kinregion(2) = 4
         u(1) = -s23/s13
         v(1) =  s45/s13
         u(2) = u(1)
         v(2) = v(1)
      elseif(iregion.eq.4) then  ! 4a
         kinregion(1) = 4
         kinregion(2) = 3
         u(1) = -s13/s23
         v(1) =  s45/s23
         u(2) = u(1)
         v(2) = v(1)
      elseif(iregion.eq.5) then  ! 1b
         kinregion(1) = 5
         kinregion(2) = 5
         u(1) = (s12+s23)/s45
         v(1) = -s12/s45
         u(2) = (s12+s13)/s45
         v(2) = v(1)
      elseif(iregion.eq.6) then  ! 2b (note different definition from NPB640)
         kinregion(1) = 6
         kinregion(2) = 10
         u(1) = -(s13+s23)/s12
         v(1) = s23/s12
         u(2) = u(1)
         v(2) = v(1)
      elseif(iregion.eq.7) then  ! 3b
         kinregion(1) = 7
         kinregion(2) = 8
         u(1) = (s13+s23)/s13
         v(1) = s12/s13
         u(2) = u(1)
         v(2) = v(1)
      elseif(iregion.eq.8) then  ! 4b
         kinregion(1) = 8
         kinregion(2) = 7
         u(1) = (s13+s23)/s23
         v(1) = s12/s23
         u(2) = u(1)
         v(2) = v(1)
      elseif(iregion.eq.9) then  ! 1c
         kinregion(1) = 9
         kinregion(2) = 13
         u(1) = (s12+s13)/s45
         v(1) = -s13/s45
         u(2) = u(1)
         v(2) = v(1)
      elseif(iregion.eq.10) then ! 2c
         kinregion(1) = 10
         kinregion(2) = 6
         u(1) = -(s13+s23)/s12
         v(1) = s13/s12
         u(2) = u(1)
         v(2) = v(1)
      elseif(iregion.eq.11) then ! 3c
         kinregion(1) = 11
         kinregion(2) = 12
         u(1) = -(s12+s23)/s13
         v(1) = s23/s13
         u(2) = u(1)
         v(2) = v(1)
      elseif(iregion.eq.12) then ! 4c
         kinregion(1) = 12
         kinregion(2) = 11
         u(1) = -(s12+s13)/s23
         v(1) = s13/s23
         u(2) = u(1)
         v(2) = v(1)
      elseif(iregion.eq.13) then ! 1d
         kinregion(1) = 13
         kinregion(2) = 9
         u(1) = (s12+s23)/s45
         v(1) = -s23/s45
         u(2) = u(1)
         v(2) = v(1)
      elseif(iregion.eq.14) then ! 2d
         kinregion(1) = 14
         kinregion(2) = 14
         u(1) = (s12+s13)/s12
         v(1) = -s45/s12
         u(2) = (s12+s23)/s12
         v(2) = v(1)
      elseif(iregion.eq.15) then ! 3d
         kinregion(1) = 15
         kinregion(2) = 16
         u(1) = (s13+s23)/s13
         v(1) = -s45/s13
         u(2) = u(1)
         v(2) = v(1)
      elseif(iregion.eq.16) then ! 4d
         kinregion(1) = 16
         kinregion(2) = 15
         u(1) = (s13+s23)/s23
         v(1) = -s45/s23
         u(2) = u(1)
         v(2) = v(1)
      endif

      do i=1,2
c         write(6,*) iregion,u(i),v(i),1d0-u(i)-v(i)
         omv = 1d0-v(i)
c ---- displacement for problematic exceptional endpoint
         if (omv.lt.1d-3) then
            v(i) = v(i)-1d-3
            omv = 1d0-v(i)
         endif
c ---- series approximation for u->0 or 1-u-v->0
         deldis = 1d-3*omv
         adis = min(1d-1*omv,1d-2)
         omumv = 1d0-u(i)-v(i)
         if (kinregion(i).ge.5.and.u(i).lt.deldis.and.u(i).le.omumv) then
c            write(6,*) '1:',u(i),v(i),1d0-u(i)-v(i)
            call makecoeftaylorU(kinregion(i),u(i),v(i),adis,-2d0*adis,za)
         elseif (kinregion(i).ge.5.and.omumv.lt.deldis) then
c            write(6,*) '2:',u(i),v(i),1d0-u(i)-v(i)
            call makecoeftaylorU(kinregion(i),u(i),v(i),adis,2d0*adis,za)
         else
            call makecoef(kinregion(i),u(i),v(i),za)
         endif

         if (nloop.eq.1.or.nloop.eq.-1) then
            if (icolor.eq.1) then ! N
               zal(i) = za(1) - 11d0/12d0*dlren*zone
               zbe(i) = za(2) - 11d0/12d0*dlren*zone
               zga(i) = za(3)
            elseif (icolor.eq.2) then ! -1/N (note the '-' here)
               zal(i) = -za(4)
               zbe(i) = -za(5)
               zga(i) = -za(6)
            elseif (icolor.eq.3) then ! NF
               zal(i) = za(7) + 1d0/6d0*dlren*zone
               zbe(i) = za(8) + 1d0/6d0*dlren*zone
               zga(i) = za(9)
            endif
         endif
         if (nloop.eq.11) then
            if (icolor.eq.1) then ! N^2
               zal(i) = za(1) - 11d0/12d0*dlren*zone
               zbe(i) = za(2) - 11d0/12d0*dlren*zone
               zga(i) = za(3)
               zal2(i) = zal(i)
               zbe2(i) = zbe(i)
               zga2(i) = zga(i)
             elseif (icolor.eq.2) then ! N^0
               zal(i) = srt2*(za(1) - 11d0/12d0*dlren*zone)
               zbe(i) = srt2*(za(2) - 11d0/12d0*dlren*zone)
               zga(i) = srt2* za(3)
               zal2(i) = srt2*za(4)
               zbe2(i) = srt2*za(5)
               zga2(i) = srt2*za(6)
             elseif (icolor.eq.3) then ! N^-2
               zal(i) = za(4)
               zbe(i) = za(5)
               zga(i) = za(6)
               zal2(i) = zal(i)
               zbe2(i) = zbe(i)
               zga2(i) = zga(i)
            elseif (icolor.eq.4) then ! NF*N
               zal(i) = srt2*(za(1) - 11d0/12d0*dlren*zone)
               zbe(i) = srt2*(za(2) - 11d0/12d0*dlren*zone)
               zga(i) = srt2* za(3)
               zal2(i) = srt2*(za(7) + 1d0/6d0*dlren*zone)
               zbe2(i) = srt2*(za(8) + 1d0/6d0*dlren*zone)
               zga2(i) = srt2* za(9)
            elseif (icolor.eq.5) then ! NF/N
               zal(i) = srt2*za(4)
               zbe(i) = srt2*za(5)
               zga(i) = srt2*za(6)
               zal2(i) = srt2*(za(7) + 1d0/6d0*dlren*zone)
               zbe2(i) = srt2*(za(8) + 1d0/6d0*dlren*zone)
               zga2(i) = srt2*za(9)
            elseif (icolor.eq.6) then ! NF^2
               zal(i) = za(7) + 1d0/6d0*dlren*zone
               zbe(i) = za(8) + 1d0/6d0*dlren*zone
               zga(i) = za(9)
               zal2(i) = zal(i)
               zbe2(i) = zbe(i)
               zga2(i) = zga(i)
            endif
         endif
         if (nloop.eq.2) then
c---- this is eq(2.38) of NPB642(2002)227 expanded out according to color factors
            if (icolor.eq.1) then ! N^2
               zal(i) = za(10) + dlren*(-11d0/4d0*za(1)-17d0/12d0*zone+121d0/96d0*dlren*zone)
               zbe(i) = za(11) + dlren*(-11d0/4d0*za(2)-17d0/12d0*zone+121d0/96d0*dlren*zone)
               zga(i) = za(12) + dlren*(-11d0/4d0*za(3))
            elseif (icolor.eq.2) then ! N^0
               zal(i) = za(13) + dlren*(-11d0/4d0*za(4))
               zbe(i) = za(14) + dlren*(-11d0/4d0*za(5))
               zga(i) = za(15) + dlren*(-11d0/4d0*za(6))
            elseif (icolor.eq.3) then ! N^-2
               zal(i) = za(16)
               zbe(i) = za(17)
               zga(i) = za(18)
            elseif (icolor.eq.4) then ! NF*N
               zal(i) = za(19) + dlren*(-11d0/4d0*za(7)+1d0/2d0*za(1)+13d0/24d0*zone-11d0/24d0*dlren*zone)
               zbe(i) = za(20) + dlren*(-11d0/4d0*za(8)+1d0/2d0*za(2)+13d0/24d0*zone-11d0/24d0*dlren*zone)
               zga(i) = za(21) + dlren*(-11d0/4d0*za(9)+1d0/2d0*za(3))
            elseif (icolor.eq.5) then ! NF/N
               zal(i) = za(22) + dlren*(1d0/2d0*za(4)-1d0/8d0*zone)
               zbe(i) = za(23) + dlren*(1d0/2d0*za(5)-1d0/8d0*zone)
               zga(i) = za(24) + dlren*(1d0/2d0*za(6))
            elseif (icolor.eq.6) then  ! NF^2
               zal(i) = za(25) + dlren*(1d0/2d0*za(7)+1d0/24d0*dlren*zone)
               zbe(i) = za(26) + dlren*(1d0/2d0*za(8)+1d0/24d0*dlren*zone)
               zga(i) = za(27) + dlren*(1d0/2d0*za(9))
            elseif (icolor.eq.7) then ! NFZ
               zal(i) = za(28)
               zbe(i) = za(29)
               zga(i) = za(30)
            endif
         endif
         zal(i+2) = zal(i)
         zbe(i+2) = zbe(i)
         zga(i+2) = zga(i)
         zal2(i+2) = zal2(i)
         zbe2(i+2) = zbe2(i)
         zga2(i+2) = zga2(i)
      enddo

      icatani=1
      if (icatani.eq.0) return

c-----re-introducing the Catani operators at one loop here:
      if (nloop.eq.1) then
         if (ieorder.eq.-2.and.icolor.eq.1) then
            do i=1,4
               zal(i) = -zone
               zbe(i) = -zone
               zga(i) = zero
            enddo
         elseif (ieorder.eq.-2.and.icolor.eq.2) then
            do i=1,4
               zal(i) = -0.5d0*zone
               zbe(i) = -0.5d0*zone
               zga(i) = zero
            enddo
         elseif (ieorder.eq.-2.and.icolor.eq.3) then
            do i=1,4
               zal(i) = zero
               zbe(i) = zero
               zga(i) = zero
            enddo
         elseif (ieorder.eq.-1.and.icolor.eq.1) then
            do i=1,4
               zal(i) = -5d0/3d0*zone
               zal(i) = zal(i) + 0.5d0*(zl13+zl23)
               zbe(i) = -5d0/3d0*zone
               zbe(i) = zbe(i) + 0.5d0*(zl13+zl23)
               zga(i) = zero
            enddo
         elseif (ieorder.eq.-1.and.icolor.eq.2) then
            do i=1,4
               zal(i) = -3d0/4d0*zone
               zal(i) = zal(i) + 0.5d0*zl12
               zbe(i) = -3d0/4d0*zone
               zbe(i) = zbe(i) + 0.5d0*zl12
               zga(i) = zero
            enddo
         elseif (ieorder.eq.-1.and.icolor.eq.3) then
            do i=1,4
               zal(i) = 1d0/6d0*zone
               zbe(i) = 1d0/6d0*zone
               zga(i) = zero
            enddo
         elseif (ieorder.eq.0.and.icolor.eq.1) then
            do i=1,4
               zal(i) = zal(i) - 0.25d0*(zl13**2+zl23**2)
               zal(i) = zal(i) + 5d0/6d0*(zl13+zl23)
               zal(i) = zal(i) + pi**2/12d0*zone
               zbe(i) = zbe(i) - 0.25d0*(zl13**2+zl23**2)
               zbe(i) = zbe(i) + 5d0/6d0*(zl13+zl23)*zone
               zbe(i) = zbe(i) + pi**2/12d0*zone
c               zal(i) = zal(i) - 11d0/12d0*dlren*zone
c               zbe(i) = zbe(i) - 11d0/12d0*dlren*zone
            enddo
         elseif (ieorder.eq.0.and.icolor.eq.2) then
            do i=1,4
               zal(i) = zal(i) - 0.25d0*zl12**2
               zal(i) = zal(i) + 3d0/4d0*zl12
               zal(i) = zal(i) + pi**2/24d0*zone
               zbe(i) = zbe(i) - 0.25d0*zl12**2
               zbe(i) = zbe(i) + 3d0/4d0*zl12
               zbe(i) = zbe(i) + pi**2/24d0*zone
               zga(i) = zga(i)
            enddo
         elseif (ieorder.eq.0.and.icolor.eq.3) then
            do i=1,4
               zal(i) = zal(i) - 1d0/12d0*(zl13+zl23)
               zbe(i) = zbe(i) - 1d0/12d0*(zl13+zl23)
c               zal(i) = zal(i) + 1d0/6d0*dlren*zone
c               zbe(i) = zbe(i) + 1d0/6d0*dlren*zone
            enddo
         endif
      endif

      return
      end


      subroutine makecoeftaylorU(ireg,u,v,del,a,za)
      use KinData_mod
      implicit none
      integer ireg,i
      real*8 u,v,del,a,u0,um,up,x
      complex*16 za(1:30),za0(1:30),zam(1:30),zap(1:30)

      u0 = u-a
      um = u0-del
      up = u0+del
      x = u-u0
c      write(6,*) v,1d0-u-v,u0,um,up,del
      call makecoef(ireg,u0,v,za0)
      call makecoef(ireg,um,v,zam)
      call makecoef(ireg,up,v,zap)

      do i=1,30
         za(i) = za0(i) + x*(zap(i)-zam(i))/2d0/del
     .          + x**2*(zap(i)-2d0*za0(i)+zam(i))/2d0/del**2
      enddo
      return
      end


      subroutine makecoef(ireg,u,v,za)
      use KinData_mod
      implicit none
      integer ireg
      real*8 u,v
      complex*16 za(1:30)

      if (ireg.eq.1)  call makecoef1a(u,v,za)
      if (ireg.eq.2)  call makecoef2a(u,v,za)
      if (ireg.eq.3)  call makecoef3a(u,v,za)
      if (ireg.eq.4)  call makecoef4a(u,v,za)
      if (ireg.eq.5)  call makecoef1b(u,v,za)
      if (ireg.eq.6)  call makecoef2b(u,v,za)
      if (ireg.eq.7)  call makecoef3b(u,v,za)
      if (ireg.eq.8)  call makecoef4b(u,v,za)
      if (ireg.eq.9)  call makecoef1c(u,v,za)
      if (ireg.eq.10) call makecoef2c(u,v,za)
      if (ireg.eq.11) call makecoef3c(u,v,za)
      if (ireg.eq.12) call makecoef4c(u,v,za)
      if (ireg.eq.13) call makecoef1d(u,v,za)
      if (ireg.eq.14) call makecoef2d(u,v,za)
      if (ireg.eq.15) call makecoef3d(u,v,za)
      if (ireg.eq.16) call makecoef4d(u,v,za)

      return
      end



      subroutine amp5basic(i1,i2,i3,i4,i5)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /ampbasic/zampA(1:4),zampB(1:4),zampC(1:4)
!$omp threadprivate(/ampbasic/)

      zampA(1) = zA5(i3,i4)/zA5(i1,i2)/zA5(i3,i2)*zA5(i3,i1)*zB5(i5,i1)
      zampB(1) = zA5(i3,i4)/zA5(i1,i2)*zB5(i5,i2)
      zampC(1) = zA5(i1,i4)*zB5(i1,i2)*zB5(i2,i5)/zA5(i1,i2)/zB5(i3,i2)

      zampA(2) = -zB5(i1,i5)/zB5(i3,i2)/zB5(i1,i2)*zB5(i1,i3)*zA5(i4,i3)
      zampB(2) = -zB5(i1,i5)/zB5(i3,i2)*zA5(i4,i2)
      zampC(2) = -zB5(i3,i5)*zA5(i3,i2)*zA5(i2,i4)/zB5(i3,i2)/zA5(i1,i2)

      zampA(3) = zA5(i3,i5)/zA5(i1,i2)/zA5(i3,i2)*zA5(i3,i1)*zB5(i4,i1)
      zampB(3) = zA5(i3,i5)/zA5(i1,i2)*zB5(i4,i2)
      zampC(3) = zA5(i1,i5)*zB5(i1,i2)*zB5(i2,i4)/zA5(i1,i2)/zB5(i3,i2)

      zampA(4) = -zB5(i1,i4)/zB5(i3,i2)/zB5(i1,i2)*zB5(i1,i3)*zA5(i5,i3)
      zampB(4) = -zB5(i1,i4)/zB5(i3,i2)*zA5(i5,i2)
      zampC(4) = -zB5(i3,i4)*zA5(i3,i2)*zA5(i2,i5)/zB5(i3,i2)/zA5(i1,i2)

      return
      end



