************************************************************************
* The flavour of the quark pair in this matrix element is determined from the flav variable in the common block /BZFlav/
* The correct vector and axial vector contribution based on 2211.13596 and 2306.10170.
      function B1g1ZPS(i1,i2,i3,i4,i5,rs2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      double precision B1gvec(1:4)
      nloop = 1
      icolor = 4
      call makem0mnPS(i1,i2,i3,i4,i5,nloop,icolor,rs2,B1gvec)
      B1g1ZPS = sum(B1gvec)
      return
      end

c----- the two loop contributions should be multipled in QCDNORM with a factor
c----- factor1vv = factor3/(8d0*pi)**2/nc
c----- and the following relatvive factors:
c----- B1g2ZPS - 1d0/nc**2*Bt1g2ZPS + nf/nc*Bh1g2ZPS 

      
      function B1g2ZPS(i1,i2,i3,i4,i5,rs2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      double precision B1gvec(1:4)
      nloop = 2
      icolor = 8
      call makem0mnPS(i1,i2,i3,i4,i5,nloop,icolor,rs2,B1gvec)
      B1g2ZPS = sum(B1gvec)
      call makem1m1PS(i1,i2,i3,i4,i5,icolor,rs2,B1gvec)
      B1g2ZPS = B1g2ZPS + sum(B1gvec)
      return
      end

      function Bt1g2ZPS(i1,i2,i3,i4,i5,rs2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      double precision B1gvec(1:4)
      nloop = 2
      icolor = 9
      call makem0mnPS(i1,i2,i3,i4,i5,nloop,icolor,rs2,B1gvec)
      Bt1g2ZPS = sum(B1gvec)
      call makem1m1PS(i1,i2,i3,i4,i5,icolor,rs2,B1gvec)
      Bt1g2ZPS = Bt1g2ZPS + sum(B1gvec)
      return
      end

      function Bh1g2ZPS(i1,i2,i3,i4,i5,rs2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      double precision B1gvec(1:4)
      nloop = 2
      icolor = 10
      call makem0mnPS(i1,i2,i3,i4,i5,nloop,icolor,rs2,B1gvec)
      Bh1g2ZPS = sum(B1gvec)
      call makem1m1PS(i1,i2,i3,i4,i5,icolor,rs2,B1gvec)
      Bh1g2ZPS = Bh1g2ZPS + sum(B1gvec)
      return
      end

************************************************************************
* A wrapper function for B1g1ZPS which inherits the flavour of the first
* pair of quarks from the C0g1Zax common block.
*
* This must be called in the maple code for 4 quark processes where the
* reduced matrix element has the flavour of the first pair of quarks
*
      function f1B1g1ZPS(i1,i2,i3,i4,i5,rs2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /CZFlav/nf1,nf2
!$omp threadprivate(/CZFlav/)
      common /BZFlav/nf1g
!$omp threadprivate(/BZFlav/)

      if(nf1.ne.0)then
         nf1g = nf1
         f1B1g1ZPS = B1g1ZPS(i1,i2,i3,i4,i5,rs2)
      else
         write(*,*) "Error in f1B1g1ZPS: flavours have not been initialised"
         stop
      endif
      return
      end

************************************************************************
* A wrapper function for B1g1ZPS which inherits the flavour of the second
* pair of quarks from the C0g1Zax common block.
*
* This must be called in the maple code for 4 quark processes where the
* reduced matrix element has the flavour of the second pair of quarks
*
      function f2B1g1ZPS(i1,i2,i3,i4,i5,rs2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /CZFlav/nf1,nf2
!$omp threadprivate(/CZFlav/)
      common /BZFlav/nf1g
!$omp threadprivate(/BZFlav/)

      if(nf2.ne.0)then
         nf1g = nf2
         f2B1g1ZPS = B1g1ZPS(i1,i2,i3,i4,i5,rs2)
      else
         write(*,*) "Error in f2B1g1ZPS: flavours have not been initialised"
         stop
      endif
      return
      end

************************************************************************
      subroutine makem0mnPS(i1,i2,i3,i4,i5,nloop,icolor,rs2,B1gvec)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      double complex zamps0(8),zamps0d(8),zamps1(8),zcoupl(4)
      parameter(pi=3.141592653589793238d0)
      double precision B1gvec(1:4)
      common /eweakZ/ amz,zewfac(4),zewnull(4)
!$omp threadprivate(/eweakZ/)
      common /ampbasic/zampA(1:4),zampB(1:4),zampC(1:4)
!$omp threadprivate(/ampbasic/)
      common /BZFlav/nflav
!$omp threadprivate(/BZFlav/)
      common /axial/zewax(2)
!$omp threadprivate(/axial/)
c     common /B1gFlav/nflav
      common /zcoeff/zal(1:4),zbe(1:4),zga(1:4)
!$omp threadprivate(/zcoeff/)
      common /masslog/dlmt2q2     
!$omp threadprivate(/masslog/)
      common /Qmass/emt,etwidth,emb,ebwidth,emc,ecwidth,emtau,etauwidth
      s54 = s5(i5,i4)
      dlmt2q2 = dlog(emt**2/s54)
      call eweakV1qq(nflav,s54)
      call amp5basic(i1,i2,i3,i4,i5)
      call helcoeff(i1,i2,i3,i4,i5,0,rs2,0)
      do i=1,4
         zamps0(i) = zal(i)*zampA(i)+zbe(i)*zampB(i)+zga(i)*zampC(i)
         zamps0d(i) = dconjg(zamps0(i))
         zamps0(i+4) = dconjg(zal(i))*zampA(i)
     .                +dconjg(zbe(i))*zampB(i)+dconjg(zga(i))*zampC(i)
         zamps0d(i+4) = dconjg(zamps0(i+4))
c                  write(6,*) "0:",zal(i),zbe(i),zga(i)
c                  write(6,*) zamps0(i),zamps0(i+4)
c                  write(6,*) zampA(i)
c                  write(6,*) zampB(i)
c                  write(6,*) zampC(i)                  
      enddo
      call helcoeffPS(i1,i2,i3,i4,i5,nloop,rs2,icolor)
      do i=1,4
         zamps1(i) = zal(i)*zampA(i)+zbe(i)*zampB(i)+zga(i)*zampC(i)
         zamps1(i+4) = dconjg(zal(i))*zampA(i)
     .                +dconjg(zbe(i))*zampB(i)+dconjg(zga(i))*zampC(i)
c                  write(6,*) "1:",zal(i),zbe(i),zga(i)
c                  write(6,*) zamps1(i),zamps1(i+4)
c                  write(6,*) zampA(i)
c                  write(6,*) zampB(i)
c                  write(6,*) zampC(i)                  
      enddo
c      write(6,*) abs(zamps0(1:4)**2)

      B1gvec(1) = 2d0*( real(zamps0d(1)*zamps1(1))
     .                  + real(zamps0d(2)*zamps1(2)) )/s54**2
      B1gvec(2) = 2d0*( real(zamps0d(3)*zamps1(3))
     .                  + real(zamps0d(4)*zamps1(4)) )/s54**2
      B1gvec(3) = 2d0*( real(zamps0d(7)*zamps1(7))
     .                  + real(zamps0d(8)*zamps1(8)) )/s54**2
      B1gvec(4) = 2d0*( real(zamps0d(5)*zamps1(5))
     .                  + real(zamps0d(6)*zamps1(6)) )/s54**2
      zcoupl(1) =  zewfac(1)*zewax(1)/2d0    ! '-' sign 
      zcoupl(2) =  zewfac(2)*zewax(2)/2d0
      zcoupl(3) =  -zewfac(3)*zewax(1)/2d0     ! '+' sign 
      zcoupl(4) =  -zewfac(4)*zewax(2)/2d0

      B1gvec = real(zcoupl)*B1gvec

      
      return
      end



      subroutine makem1m1PS(i1,i2,i3,i4,i5,icolor,rs2,B1gvec)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      double complex zamps0(8),zamps0d(8),zamps1(8),zcoupl(4)
      parameter(pi=3.141592653589793238d0)
      double precision B1gvec(1:4)
      common /eweakZ/ amz,zewfac(4),zewnull(4)
!$omp threadprivate(/eweakZ/)
      common /ampbasic/zampA(1:4),zampB(1:4),zampC(1:4)
!$omp threadprivate(/ampbasic/)
      common /BZFlav/nflav
!$omp threadprivate(/BZFlav/)
      common /axial/zewax(2)
!$omp threadprivate(/axial/)
c     common /B1gFlav/nflav
      common /zcoeff/zal(1:4),zbe(1:4),zga(1:4)
!$omp threadprivate(/zcoeff/)
      common /masslog/dlmt2q2     
!$omp threadprivate(/masslog/)
      common /Qmass/emt,etwidth,emb,ebwidth,emc,ecwidth,emtau,etauwidth
      s54 = s5(i5,i4)
      dlmt2q2 = dlog(emt**2/s54)
      call eweakV1qq(nflav,s54)
      call amp5basic(i1,i2,i3,i4,i5)
      
      call helcoeff(i1,i2,i3,i4,i5,-1,rs2,icolor-7)
      do i=1,4
         zamps0(i) = zal(i)*zampA(i)+zbe(i)*zampB(i)+zga(i)*zampC(i)
         zamps0d(i) = dconjg(zamps0(i))
         zamps0(i+4) = dconjg(zal(i))*zampA(i)
     .                +dconjg(zbe(i))*zampB(i)+dconjg(zga(i))*zampC(i)
         zamps0d(i+4) = dconjg(zamps0(i+4))
c                  write(6,*) "0:",zal(i),zbe(i),zga(i)
c                  write(6,*) zamps0(i),zamps0(i+4)
c                  write(6,*) zampA(i)
c                  write(6,*) zampB(i)
c                  write(6,*) zampC(i)                  
      enddo
      call helcoeffPS(i1,i2,i3,i4,i5,1,rs2,4)
      do i=1,4
         zamps1(i) = zal(i)*zampA(i)+zbe(i)*zampB(i)+zga(i)*zampC(i)
         zamps1(i+4) = dconjg(zal(i))*zampA(i)
     .                +dconjg(zbe(i))*zampB(i)+dconjg(zga(i))*zampC(i)
c                  write(6,*) "1:",zal(i),zbe(i),zga(i)
c                  write(6,*) zamps1(i),zamps1(i+4)
c                  write(6,*) zampA(i)
c                  write(6,*) zampB(i)
c                  write(6,*) zampC(i)                  
      enddo
c      write(6,*) abs(zamps0(1:4)**2)

      B1gvec(1) = 2d0*( real(zamps0d(1)*zamps1(1))
     .                  + real(zamps0d(2)*zamps1(2)) )/s54**2
      B1gvec(2) = 2d0*( real(zamps0d(3)*zamps1(3))
     .                  + real(zamps0d(4)*zamps1(4)) )/s54**2
      B1gvec(3) = 2d0*( real(zamps0d(7)*zamps1(7))
     .                  + real(zamps0d(8)*zamps1(8)) )/s54**2
      B1gvec(4) = 2d0*( real(zamps0d(5)*zamps1(5))
     .                  + real(zamps0d(6)*zamps1(6)) )/s54**2
      zcoupl(1) =  zewfac(1)*zewax(1)/2d0    ! '-' sign 
      zcoupl(2) =  zewfac(2)*zewax(2)/2d0
      zcoupl(3) =  -zewfac(3)*zewax(1)/2d0     ! '+' sign 
      zcoupl(4) =  -zewfac(4)*zewax(2)/2d0

      B1gvec = real(zcoupl)*B1gvec

      
      return
      end




      subroutine helcoeffPS(i1,i2,i3,i4,i5,nloop,rs2,icolor)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit double complex (z)
      dimension u(2),v(2),kinregion(2)
      common /order/ieorder
      common /order2l/ipole
      double complex zamps(4),za(1:15)
      double complex d_a1,d_b1,d_c1,
     $               G_a2,G_b2,G_c2,
     $               H_a2,H_b2,H_c2,
     $               I_a2,I_b2,I_c2,
     $               J_a2,J_b2,J_c2

      parameter(pi=3.141592653589793238d0)
      common /zcoeff/zal(1:4),zbe(1:4),zga(1:4)
!$omp threadprivate(/zcoeff/)
      common /zcoeff2/zal2(1:4),zbe2(1:4),zga2(1:4)
!$omp threadprivate(/zcoeff2/)

c      ipole = 0   ! finite parts
c      ipole = -4  ! 1/e^4 pole
c      ipole = -3  ! 1/e^3 pole (only for NF-type pieces)

      if (nloop.eq.0) then
         do i=1,4
            zal(i) = (1d0,0d0)
            zbe(i) = (1d0,0d0)
            zga(i) = (0d0,0d0)
         enddo
         return
      endif
      if (ieorder.ne.0) then
         do i=1,4
            zal(i) = (0d0,0d0)
            zbe(i) = (0d0,0d0)
            zga(i) = (0d0,0d0)
         enddo
         return
      endif

      zone = (1d0,0d0)
      zero = (0d0,0d0)
      zipi = (0d0,pi)
      srt2 = dsqrt(2d0)

      s12 = s5(i1,i3)
      s13 = s5(i1,i2)
      s23 = s5(i2,i3)
      s45 = s5(i4,i5)
c      write(6,*) s12,s13,s23,s45,rs2

      zl12 = zzLogRat(-s12,rs2)
      zl13 = zzLogRat(-s13,rs2)
      zl23 = zzLogRat(-s23,rs2)
      dlren = dlog(abs(s45)/rs2)

c----- determine the kinematical region
      if (s45.gt.0d0) then
         if (s12.gt.0d0) iregion = 2  ! 2a
         if (s13.gt.0d0) iregion = 3  ! 3a
         if (s23.gt.0d0) iregion = 4  ! 4a
         if (min(s12,s13,s23).gt.0d0) iregion = 1  ! 1a
      else
         if (s12.gt.0d0.and.s13.gt.s45.and.s23.gt.s45) iregion = 5  ! 1b
         if (s12.gt.0d0.and.s13.le.s45.and.s23.le.s45) iregion = 14 ! 2d
         if (s12.gt.0d0.and.s13.le.s45.and.s23.gt.s45) iregion = 11 ! 3c
         if (s12.gt.0d0.and.s13.gt.s45.and.s23.le.s45) iregion = 12 ! 4c
         if (s13.gt.0d0.and.s12.gt.s45.and.s23.gt.s45) iregion = 9  ! 1c
         if (s13.gt.0d0.and.s12.le.s45.and.s23.gt.s45) iregion = 6  ! 2b
         if (s13.gt.0d0.and.s12.le.s45.and.s23.le.s45) iregion = 15 ! 3d
         if (s13.gt.0d0.and.s12.gt.s45.and.s23.le.s45) iregion = 8  ! 4b
         if (s23.gt.0d0.and.s12.gt.s45.and.s13.gt.s45) iregion = 13 ! 1d
         if (s23.gt.0d0.and.s12.le.s45.and.s13.gt.s45) iregion = 10 ! 2c
         if (s23.gt.0d0.and.s12.gt.s45.and.s13.le.s45) iregion = 7  ! 3b
         if (s23.gt.0d0.and.s12.le.s45.and.s13.le.s45) iregion = 16 ! 4d
         write(*,*) 'kinematical region not implemented',iregion
         stop
      endif

      if(iregion.eq.1) then      ! 1a
         kinregion(1) = 1
         kinregion(2) = 1
         u(1) =  s13/s45
         v(1) =  s23/s45
         u(2) = v(1)
         v(2) = u(1)
      elseif(iregion.eq.2) then  ! 2a
         kinregion(1) = 2
         kinregion(2) = 2
         u(1) = -s13/s12
         v(1) =  s45/s12
         u(2) = -s23/s12
         v(2) = v(1)
      elseif(iregion.eq.3) then  ! 3a
         kinregion(1) = 3
         kinregion(2) = 4
         u(1) = -s23/s13
         v(1) =  s45/s13
         u(2) = u(1)
         v(2) = v(1)
      elseif(iregion.eq.4) then  ! 4a
         kinregion(1) = 4
         kinregion(2) = 3
         u(1) = -s13/s23
         v(1) =  s45/s23
         u(2) = u(1)
         v(2) = v(1)
      elseif(iregion.eq.5) then  ! 1b
         kinregion(1) = 5
         kinregion(2) = 5
         u(1) = (s12+s23)/s45
         v(1) = -s12/s45
         u(2) = (s12+s13)/s45
         v(2) = v(1)
      elseif(iregion.eq.6) then  ! 2b (note different definition from NPB640)
         kinregion(1) = 6
         kinregion(2) = 10
         u(1) = -(s13+s23)/s12
         v(1) = s23/s12
         u(2) = u(1)
         v(2) = v(1)
      elseif(iregion.eq.7) then  ! 3b
         kinregion(1) = 7
         kinregion(2) = 8
         u(1) = (s13+s23)/s13
         v(1) = s12/s13
         u(2) = u(1)
         v(2) = v(1)
      elseif(iregion.eq.8) then  ! 4b
         kinregion(1) = 8
         kinregion(2) = 7
         u(1) = (s13+s23)/s23
         v(1) = s12/s23
         u(2) = u(1)
         v(2) = v(1)
      elseif(iregion.eq.9) then  ! 1c
         kinregion(1) = 9
         kinregion(2) = 13
         u(1) = (s12+s13)/s45
         v(1) = -s13/s45
         u(2) = u(1)
         v(2) = v(1)
      elseif(iregion.eq.10) then ! 2c
         kinregion(1) = 10
         kinregion(2) = 6
         u(1) = -(s13+s23)/s12
         v(1) = s13/s12
         u(2) = u(1)
         v(2) = v(1)
      elseif(iregion.eq.11) then ! 3c
         kinregion(1) = 11
         kinregion(2) = 12
         u(1) = -(s12+s23)/s13
         v(1) = s23/s13
         u(2) = u(1)
         v(2) = v(1)
      elseif(iregion.eq.12) then ! 4c
         kinregion(1) = 12
         kinregion(2) = 11
         u(1) = -(s12+s13)/s23
         v(1) = s13/s23
         u(2) = u(1)
         v(2) = v(1)
      elseif(iregion.eq.13) then ! 1d
         kinregion(1) = 13
         kinregion(2) = 9
         u(1) = (s12+s23)/s45
         v(1) = -s23/s45
         u(2) = u(1)
         v(2) = v(1)
      elseif(iregion.eq.14) then ! 2d
         kinregion(1) = 14
         kinregion(2) = 14
         u(1) = (s12+s13)/s12
         v(1) = -s45/s12
         u(2) = (s12+s23)/s12
         v(2) = v(1)
      elseif(iregion.eq.15) then ! 3d
         kinregion(1) = 15
         kinregion(2) = 16
         u(1) = (s13+s23)/s13
         v(1) = -s45/s13
         u(2) = u(1)
         v(2) = v(1)
      elseif(iregion.eq.16) then ! 4d
         kinregion(1) = 16
         kinregion(2) = 15
         u(1) = (s13+s23)/s23
         v(1) = -s45/s23
         u(2) = u(1)
         v(2) = v(1)
      endif

      do i=1,2
c         write(6,*) iregion,u(i),v(i),1d0-u(i)-v(i)
         omv = 1d0-v(i)
c ---- displacement for problematic exceptional endpoint
         if (omv.lt.1d-3) then
            v(i) = v(i)-1d-3
            omv = 1d0-v(i)
         endif
         call makecoefPS(kinregion(i),u(i),v(i),za)

         if (nloop.eq.1) then
            zal(i) = za(1)
            zbe(i) = za(2)
            zga(i) = za(3)
         endif
         if (nloop.eq.2) then
            if (icolor.eq.8) then ! N
               zal(i) = za(7) - 11d0/12d0*dlren*za(1)
               zbe(i) = za(8) - 11d0/12d0*dlren*za(2)
               zga(i) = za(9) - 11d0/12d0*dlren*za(3)
            elseif (icolor.eq.9) then ! -1/N (note the '-' here)
               zal(i) = -za(10)
               zbe(i) = -za(11)
               zga(i) = -za(12)
            elseif (icolor.eq.10) then ! NF
               zal(i) = za(13) + 1d0/6d0*dlren*za(1)
               zbe(i) = za(14) + 1d0/6d0*dlren*za(2)
               zga(i) = za(15) + 1d0/6d0*dlren*za(3)
            endif
         endif
         zal(i+2) = zal(i)
         zbe(i+2) = zbe(i)
         zga(i+2) = zga(i)
c         zal2(i+2) = zal2(i)
c         zbe2(i+2) = zbe2(i)
c         zga2(i+2) = zga2(i)
      enddo

      return
      end


      subroutine makecoefPS(ireg,u,v,za)
      use KinData_mod
      implicit none
      integer ireg
      real*8 u,v
      complex*16 za(1:15)

      if (ireg.eq.1)  call makecoefPS1a(u,v,za)
      if (ireg.eq.2)  call makecoefPS2a(u,v,za)
      if (ireg.eq.3)  call makecoefPS3a(u,v,za)
      if (ireg.eq.4)  call makecoefPS4a(u,v,za)
c      if (ireg.eq.5)  call makecoef1b(u,v,za)
c      if (ireg.eq.6)  call makecoef2b(u,v,za)
c      if (ireg.eq.7)  call makecoef3b(u,v,za)
c      if (ireg.eq.8)  call makecoef4b(u,v,za)
c      if (ireg.eq.9)  call makecoef1c(u,v,za)
c      if (ireg.eq.10) call makecoef2c(u,v,za)
c      if (ireg.eq.11) call makecoef3c(u,v,za)
c      if (ireg.eq.12) call makecoef4c(u,v,za)
c      if (ireg.eq.13) call makecoef1d(u,v,za)
c      if (ireg.eq.14) call makecoef2d(u,v,za)
c      if (ireg.eq.15) call makecoef3d(u,v,za)
c      if (ireg.eq.16) call makecoef4d(u,v,za)

      return
      end




