***************************************************

      module B1gZ

      contains

***********************************************************************
*
* Quark pair leading colour one loop level amplitude
*
* q(1) q(2) -> g(3) e+(4) e-(5)
*
* Returns a 4 component array where elements (1) and (3) are tree level helicity amplitudes and elements (2) and (4) are their respective loop level interference terms.

      function zampB1g1Z(i1,i2,i3,i4,i5,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /proc/ isub(7)
      common /order/iorder
      double complex zampB1g1Z(4)

      za14 = zA5(i1,i4)
      za32 = zA5(i3,i2)
      za21 = zA5(i2,i1)
      za45 = zA5(i4,i5)
      zb14 = zB5(i1,i4)
      zb32 = zB5(i3,i2)
      zb21 = zB5(i2,i1)
      zb45 = zB5(i4,i5)
      za13 = zA5(i1,i3)
      zb35 = zB5(i3,i5)
      za54 = zA5(i5,i4)

      zb12 = zB5(i1,i2)
      zb23 = zB5(i2,i3)
      zb54 = zB5(i5,i4)
      zb31 = zB5(i3,i1)
      zb45 = zB5(i4,i5)

      s12  = s5(i1,i2)
      s23  = s5(i2,i3)
      s45  = s5(i4,i5)

      zampB1g1Z(1) = -za14**2/(za32*za21*za45) ! Tree amplitude

      zl23 = zzLogRat(renscale2,-s23)
      zl12 = zzLogRat(renscale2,-s12)

      if (iorder .eq. -2) then
         zVcc = -2d0
         zVsc =  DCMPLX(0d0,0d0)
         zFcc =  DCMPLX(0d0,0d0)
         zFsc =  DCMPLX(0d0,0d0)
      elseif (iorder .eq. -1) then
         zVcc = -zl23 -zl12 -2d0
         zVsc = 0.5d0
         zFcc = DCMPLX(0d0,0d0)
         zFsc = DCMPLX(0d0,0d0)
      elseif (iorder .eq. 0) then

         zVcc = -0.5d0*(zl12**2 + zl23**2) - 2d0*zl12  -4d0
         zVsc = 0.5d0*zl12 + 1d0
         zFcc = -zampB1g1Z(1)*(dconjg(zLsm1(-s23,-s45,-s12,-s45))
     .       -2d0*za13*zb35*za54/za14*dconjg(zL0d(-s12,-s45))/s45)

         zFsc =  za14*za13*zb35*za54/(za32*za21*za45)
     .       *dconjg(zL0d(-s12,-s45))/s45+0.5d0*(za13*zb35)**2*za45
     .       /(za32*za21)*dconjg(zL1d(-s12,-s45))/s45**2

      endif

      zampB1g1Z(2) = (zVcc+zVsc)*zampB1g1Z(1)+zFcc+zFsc

      zampB1g1Z(3) = -zb35**2/(zb12*zb23*zb54) ! Tree amplitude

      if (iorder .eq. -2) then
         zVcc = -2d0
         zVsc =  0d0
      elseif (iorder .eq. -1) then
         zVcc = -zl23 -zl12 -2d0
         zVsc = 0.5d0
      elseif (iorder .eq. 0) then
         zVcc = -0.5d0*(zl12**2 + zl23**2) - 2d0*zl23 - 4d0
         zVsc = 0.5d0*zl23 + 1d0
         zFcc= -zampB1g1Z(3)*(dconjg(zLsm1(-s12,-s45,-s23,-s45))
     .     -2d0*zb31*za14*zb45/zb35*dconjg(zL0d(-s23,-s45)/s45))

         zFsc = zb35*zb31*za14*zb45/(zb12*zb23*zb54)
     .      *dconjg(zL0d(-s23,-s45))/s45+0.5d0*(zb31*za14)**2*zb54
     .      /(zb12*zb23)*dconjg(zL1d(-s23,-s45))/s45**2

      endif

      zampB1g1Z(4) = (zVcc+zVsc)*zampB1g1Z(3)+zFcc+zFsc

      return
      end function

***********************************************************************
*
* Quark pair subleading colour one loop level amplitude
*
* q(1) q(2) -> g(3) e+(4) e-(5)
*
* Returns a 4 component array where elements (1) and (3) are tree level helicity amplitudes and elements (2) and (4) are their respectie loop level interference terms.

      function zampBt1g1Z(i1,i2,i3,i4,i5,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /proc/ isub(7)
      common /order/iorder
      double complex zampBt1g1Z(4)

      zb41 = zB5(i4,i1)
      zb21 = zB5(i2,i1)
      zb32 = zB5(i3,i2)
      zb54 = zB5(i5,i4)
      zb13 = zB5(i1,i3)
      zb42 = zB5(i4,i2)
      zb43 = zB5(i4,i3)
      zb23 = zB5(i2,i3)
      za23 = zA5(i2,i3)
      zb34 = zB5(i3,i4)
      za32 = zA5(i3,i2)
      zb31 = zB5(i3,i1)
      zb24 = zB5(i2,i4)
      za52 = zA5(i5,i2)
      za51 = zA5(i5,i1)
      za21 = zA5(i2,i1)
      za53 = zA5(i5,i3)
      za13 = zA5(i1,i3)
      za54 = zA5(i5,i4)

      za12 = zA5(i1,i2)
      za45 = zA5(i4,i5)
      za31 = zA5(i3,i1)
      za15 = zA5(i1,i5)
      zb12 = zB5(i1,i2)
      za25 = zA5(i2,i5)
      zb41 = zB5(i4,i1)
      zb45 = zB5(i4,i5)

      s12  = s5(i1,i2)
      s45  = s5(i4,i5)
      s13  = s5(i1,i3)
      s23  = s5(i2,i3)

      zampBt1g1Z(1) = -zb41**2/(zb21*zb32*zb54)

      zl13 = zzlogRat(renscale2,-s13)
      zl12 = zzlogRat(renscale2,-s12)
      zl45 = zzlogRat(renscale2,-s45)

      if (iorder .eq. -2) then
         zVcc = DCMPLX(-1d0,0d0)
         zVsc = DCMPLX(0d0,0d0)
         zFcc = DCMPLX(0d0,0d0)
         zFsc = DCMPLX(0d0,0d0)
      elseif (iorder .eq. -1) then
         zVcc = -zl13 - 2d0
         zVsc = 0.5d0
         zFcc = DCMPLX(0d0,0d0)
         zFsc = DCMPLX(0d0,0d0)
      elseif (iorder .eq. 0) then
         zVcc = -0.5d0*zl13**2 - 2d0*zl45 - 4d0
         zVsc =  0.5d0*zl45 + 0.5d0

         zFcc=zampBt1g1Z(1)*zLsm1(-s13,-s45,-s23, -s45)
     .        +zb41*(zb13*zb42-zb43*zb21)
     .        /(zb21*zb23**2*zb54)
     .        *zLsm1(-s13,-s45,-s12,-s45)
     .        +2d0*za23*zb43*zb41/(zb23*zb54)
     .        *zL0d(-s12,-s45)/s45

         zFsc=zb43**2*zb21/(zb23**3*zb54)
     . *zLsm1(-s13,-s45,-s12,-s45)
     . -0.5d0*(zb34*za23)**2*zb21/(zb23*zb54)
     . *zL1d(-s45,-s12)/s12**2
     . +zb43**2*zb21*za32/(zb23**2*zb54)
     . *zL0d(-s45,-s12)/s12
     . -zb31*za23*zb24*za52/zb23
     . *zL1d(-s45,-s13)/s13**2
     . -zb31*za23*zb42*zb43/(zb23**2*zb54)
     . *zL0d(-s45,-s13)/s13
     . -0.5d0*za52*(za23*za51+za21*za53)
     . /(za13*za21*zb23*za54)

      endif

      zampBt1g1Z(2) = dconjg(zVsc+zVcc)*zampBt1g1Z(1)-zFsc-zFcc

      zampBt1g1Z(3) = -za53**2/(za23*za12*za45)

      if (iorder .eq. -2) then
         zVcc = DCMPLX(-1d0,0d0)
         zVsc = DCMPLX(0d0,0d0)
         zFcc = DCMPLX(0d0,0d0)
         zFsc = DCMPLX(0d0,0d0)
      elseif (iorder .eq. -1) then
         zVcc = -zl13 -2d0
         zVsc = 0.5d0
         zFcc = DCMPLX(0d0,0d0)
         zFsc = DCMPLX(0d0,0d0)
      elseif (iorder .eq. 0) then
         zVcc = -0.5d0*zl13**2 - 2d0*zl45 - 4d0
         zVsc = 0.5d0*zl45 + 0.5d0

         zFcc=zampBt1g1Z(3)*zLsm1(-s13,-s45,-s12, -s45)
     . +za53*(za31*za52-za51*za23)
     .  /(za23*za21**2*za45)
     . *zLsm1(-s13,-s45,-s23,-s45)
     . +2d0*zb21*za51*za53/(za21*za45)
     . *zL0d(-s23,-s45)/s45

         zFsc = za51**2*za23/(za21**3*za45)
     . *zLsm1(-s13,-s45,-s23,-s45)
     . -0.5d0*(za15*zb21)**2*za23/(za21*za45)
     . *zL1d(-s45,-s23)/s23**2
     . +za51**2*za23*zb12/(za21**2*za45)
     . *zL0d(-s45,-s23)/s23
     . -za13*zb21*za25*zb42/za21
     . *zL1d(-s45,-s13)/s13**2
     . -za13*zb21*za52*za51/(za21**2*za45)
     . *zL0d(-s45,-s13)/s13
     . -0.5d0*zb42*(zb21*zb43+zb23*zb41)
     . /(zb31*zb23*za21*zb45)

      endif

      zampBt1g1Z(4) = dconjg(zVcc+zVsc)*zampBt1g1Z(3)-zFcc-zFsc

c      write(*,*) abs(zampBt1g1Z)**2

      return
      end function

      end module B1gZ
