********************************************************************
*
* Quark pair tree level leading colour matrix element for
*
*     q + qb ->  g + g + g + e- + e+
*
* summed over colour orderings, including overall factors
*
*     q(1) g(2) g(3) g(4) qbar(5) e-(6) e+(7)
*
* NOTE: In the case of a gg initial state, this function only checks the
* i2 and i3 for this configuration. When running this function, make sure that this is where you put your initial state gluons!!
*

      function FullB3g0Z(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)
      common /BZFlav/ nf3g
!$omp threadprivate(/BZFlav/)
      common /eweakZ/ amz,zewfac(4),zewnull(4)
!$omp threadprivate(/eweakZ/)

      fac = get_avg(i1,i5)

      fac = fac*(4d0*pi*as)**3*(4d0*pi*amz)**2*nc**3*(nc**2-1)/(2d0*nc)*8d0

      if((i2.le.2).and.(i3.le.2))then ! gg initial state, nf contribution
         nup = 2
         ndo = 3

         nf3g = 2 ! up type contribution

         FullB3g0Z = nup*(B3g0Z(i1,i2,i3,i4,i5,i6,i7)
     .           +B3g0Z(i1,i2,i4,i3,i5,i6,i7)
     .           +B3g0Z(i1,i4,i2,i3,i5,i6,i7)
     .           +B3g0Z(i1,i3,i4,i2,i5,i6,i7)
     .           +B3g0Z(i1,i3,i2,i4,i5,i6,i7)
     .           +B3g0Z(i1,i4,i3,i2,i5,i6,i7)

     .           -(1d0/nc**2)*(Bt3g0Z(i1,i2,i3,i4,i5,i6,i7)
     .           +Bt3g0Z(i1,i2,i4,i3,i5,i6,i7)
     .           +Bt3g0Z(i1,i3,i2,i4,i5,i6,i7)
     .           +Bt3g0Z(i1,i3,i4,i2,i5,i6,i7)
     .           +Bt3g0Z(i1,i4,i2,i3,i5,i6,i7)
     .           +Bt3g0Z(i1,i4,i3,i2,i5,i6,i7))

     .           +((1d0/nc**2)+(1d0/nc**4))*Btt3g0Z(i1,i2,i3,i4,i5,i6,i7))


         nf3g = 1 ! down type contribution

         FullB3g0Z = FullB3g0Z
     .      +ndo*(B3g0Z(i1,i2,i3,i4,i5,i6,i7)
     .           +B3g0Z(i1,i2,i4,i3,i5,i6,i7)
     .           +B3g0Z(i1,i4,i2,i3,i5,i6,i7)
     .           +B3g0Z(i1,i3,i4,i2,i5,i6,i7)
     .           +B3g0Z(i1,i3,i2,i4,i5,i6,i7)
     .           +B3g0Z(i1,i4,i3,i2,i5,i6,i7)

     .           -(1d0/nc**2)*(Bt3g0Z(i1,i2,i3,i4,i5,i6,i7)
     .           +Bt3g0Z(i1,i2,i4,i3,i5,i6,i7)
     .           +Bt3g0Z(i1,i3,i2,i4,i5,i6,i7)
     .           +Bt3g0Z(i1,i3,i4,i2,i5,i6,i7)
     .           +Bt3g0Z(i1,i4,i2,i3,i5,i6,i7)
     .           +Bt3g0Z(i1,i4,i3,i2,i5,i6,i7))

     .           +((1d0/nc**2)+(1d0/nc**4))*Btt3g0Z(i1,i2,i3,i4,i5,i6,i7))

      else

         if((i1.le.2).and.(i5.le.2))then
            fac = fac/6d0       ! statistical factor
         elseif((i1.le.2).or.(i5.le.2))then
            fac = fac/2d0
         endif

         FullB3g0Z = B3g0Z(i1,i2,i3,i4,i5,i6,i7)
     .           +B3g0Z(i1,i2,i4,i3,i5,i6,i7)
     .           +B3g0Z(i1,i4,i2,i3,i5,i6,i7)
     .           +B3g0Z(i1,i3,i4,i2,i5,i6,i7)
     .           +B3g0Z(i1,i3,i2,i4,i5,i6,i7)
     .           +B3g0Z(i1,i4,i3,i2,i5,i6,i7)

     .           -(1d0/nc**2)*(Bt3g0Z(i1,i2,i3,i4,i5,i6,i7)
     .           +Bt3g0Z(i1,i2,i4,i3,i5,i6,i7)
     .           +Bt3g0Z(i1,i3,i2,i4,i5,i6,i7)
     .           +Bt3g0Z(i1,i3,i4,i2,i5,i6,i7)
     .           +Bt3g0Z(i1,i4,i2,i3,i5,i6,i7)
     .           +Bt3g0Z(i1,i4,i3,i2,i5,i6,i7))

     .           +((1d0/nc**2)+(1d0/nc**4))*Btt3g0Z(i1,i2,i3,i4,i5,i6,i7)

      endif

      FullB3g0Z = fac*FullB3g0Z

c     write(*,*) "FullB3g0Z",FullB3g0Z,i1,i2,i3,i4,i5,i6,i7

      return
      end

********************************************************************
*
* Quark pair tree level matrix element for
*
*     q + qb ->  g + g + g + e- e+
*
* One colour ordering, summed over spins and squared
*
*  q(1) g(2) g(3) g(4) qbar(5) e-(6) e+(7)
*

      function B3g0Z(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      use B3gZ, only : zampB3g0Zvec
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /proc/ isub(7)
      common /eweakZ/ amz,zewfac(4),zewnull(4)
!$omp threadprivate(/eweakZ/)
      common /BZFlav/ nf3g
!$omp threadprivate(/BZFlav/)
      double precision B3g0Zvec(4)
      complex*16 zampB3g0ZvecLL(2,2,2),zampB3g0ZvecLR(2,2,2)

      s67 = s7(i6,i7)

      call eweakV1qq(nf3g,s67)

      do i=1,4
         B3g0Zvec(i) = 0d0
      enddo


      zampB3g0ZvecLL = zampB3g0Zvec(i1,i2,i3,i4,i5,i6,i7)
      zampB3g0ZvecLR = zampB3g0Zvec(i1,i2,i3,i4,i5,i7,i6)

      do i=1,2
         do j=1,2
            do k=1,2
               B3g0Zvec(1) = B3g0Zvec(1) + abs(zampB3g0ZvecLL(i,j,k))**2
               B3g0Zvec(2) = B3g0Zvec(2) + abs(zampB3g0ZvecLR(i,j,k))**2
            enddo
         enddo
      enddo

      B3g0Zvec(3) = B3g0Zvec(2)
      B3g0Zvec(4) = B3g0Zvec(1)

      B3g0Zvec = abs(zewfac)**2*B3g0Zvec

      B3g0Z = sum(B3g0Zvec)


      return
      end



************************************************************************
* A wrapper function for B3g0Z which inherits the flavour of the first
* pair of quarks from the C0g0Z common block.
*
* This must be called in the maple code for 4 quark processes where the
* reduced matrix element has the flavour of the first pair of quarks


      function f1B3g0Z(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /CZFlav/nf1,nf2
!$omp threadprivate(/CZFlav/)
      common /BZFlav/nf1g
!$omp threadprivate(/BZFlav/)

      if(nf1.ne.0)then
         nf1g = nf1
         f1B3g0Z = B3g0Z(i1,i2,i3,i4,i5,i6,i7)
      else
         write(*,*) "Error in f1B3g0Z: flavours have not been initialised"
         stop
      endif

      return
      end

************************************************************************
* A wrapper function for B3g0Z which inherits the flavour of the second
* pair of quarks from the C0g0Z common block.
*
* This must be called in the maple code for 4 quark processes where the
* reduced matrix element has the flavour of the second pair of quarks
*

      function f2B3g0Z(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /CZFlav/nf1,nf2
!$omp threadprivate(/CZFlav/)
      common /BZFlav/nf1g
!$omp threadprivate(/BZFlav/)

      if(nf2.ne.0)then
         nf1g = nf2
         f2B3g0Z = B3g0Z(i1,i2,i3,i4,i5,i6,i7)
      else
         write(*,*) "Error in f2B3g0Z: flavours have not been initialised"
         stop
      endif

      return
      end



**********************************************
* A wrapper function for returning the normal
* matrix element for situations where either quark or
* antiquark are in the initial state. Otherwise
* it will return a line reversal symmetrised matrix element
*
*

      function sB3g0Z(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      if((i1.le.2).or.(i5.le.2))then
         sB3g0Z=B3g0Z(i1,i2,i3,i4,i5,i6,i7)
      else
         sB3g0Z=0.5d0*(B3g0Z(i1,i2,i3,i4,i5,i6,i7)+B3g0Z(i5,i4,i3,i2,i1,i6,i7))
      endif

      return
      end


      function sBt3g0Z(i1,i2,i3,i4,i5,i6,i7)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      if((i1.le.2).or.(i5.le.2))then
         sBt3g0Z=Bt3g0Z(i1,i2,i3,i4,i5,i6,i7)
      else
         sBt3g0Z=0.5d0*(Bt3g0Z(i1,i2,i3,i4,i5,i6,i7)+Bt3g0Z(i5,i4,i3,i2,i1,i6,i7))
      endif

      return
      end


      function sBtt3g0Z(i1,i2,i3,i4,i5,i6,i7)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      if((i1.le.2).or.(i5.le.2))then
         sBtt3g0Z=Btt3g0Z(i1,i2,i3,i4,i5,i6,i7)
      else
         sBtt3g0Z=0.5d0*(Btt3g0Z(i1,i2,i3,i4,i5,i6,i7)+Btt3g0Z(i5,i4,i3,i2,i1,i6,i7))
      endif

      return
      end


************************************************************************
* Re of interference of two orderings, summed over helicities
* q g g g qb ep em

      function B3g0epemint(i1,i2,i3,i4,i5,i6,i7,j1,j2,j3,j4,j5,j6,j7)
      use KinData_mod
      implicit real*8 (a-h,o-y)
      implicit complex*16(z)
      dimension jhel(8)
      common /proc/ isub(7)
      common /eweakZ/ amz,zewfac(4),zewnull(4)
!$omp threadprivate(/eweakZ/)
      common /BZFlav/ nf3g
!$omp threadprivate(/BZFlav/)
      double precision B3g0Zintvec(4)

      zwt = (0d0,0d0)
      zwt1 = (0d0,0d0)

       zwt2 = (0d0,0d0)
       zwt3 = (0d0,0d0)

c      if(j2.eq.i2.and.j3.eq.i3.and.j4.eq.i4)then
c         jhel(1)=1
c         jhel(2)=2
c         jhel(3)=3
c         jhel(4)=4
c         jhel(5)=5
c         jhel(6)=6
c         jhel(7)=7
c         jhel(8)=8
c  6 mean 6helicity, except +++ and ---

      if(j2.eq.i2.and.j3.eq.i4.and.j4.eq.i3)then
         jhel(1)=1
         jhel(2)=3
         jhel(3)=2
         jhel(4)=4
         jhel(5)=5
         jhel(6)=7
         jhel(7)=6
         jhel(8)=8

      elseif(j2.eq.i3.and.j3.eq.i2.and.j4.eq.i4)then
         jhel(1)=1
         jhel(2)=2
         jhel(3)=4
         jhel(4)=3
         jhel(5)=5
         jhel(6)=6
         jhel(7)=8
         jhel(8)=7
      elseif(j2.eq.i3.and.j3.eq.i4.and.j4.eq.i2)then
         jhel(1)=1
         jhel(2)=3
         jhel(3)=4
         jhel(4)=2
         jhel(5)=5
         jhel(6)=7
         jhel(7)=8
         jhel(8)=6

      elseif(j2.eq.i4.and.j3.eq.i2.and.j4.eq.i3)then
         jhel(1)=1
         jhel(2)=4
         jhel(3)=2
         jhel(4)=3
         jhel(5)=5
         jhel(6)=8
         jhel(7)=6
         jhel(8)=7
      elseif(j2.eq.i4.and.j3.eq.i3.and.j4.eq.i2)then
         jhel(1)=1
         jhel(2)=4
         jhel(3)=3
         jhel(4)=2
         jhel(5)=5
         jhel(6)=8
         jhel(7)=7
         jhel(8)=6
      endif



      do i=1,4
         B3g0Zintvec(i) = 0d0
      enddo


      s67 = s7(i6,i7)

      call eweakV1qq(nf3g,s67)

      do ihel=1,8


         ztmp1 = zB3g0epem(i1,i2,i3,i4,i5,i6,i7,ihel)! Divide by photon propagator
         ztmp2 = zB3g0epem(j1,j2,j3,j4,j5,j6,j7,jhel(ihel)) ! Divide by photon propagator
         zwt = zwt + ztmp2*dconjg(ztmp1)

         ztmp3 = zB3g0epem(i1,i2,i3,i4,i5,i7,i6,ihel) ! Divide by photon propagator
         ztmp4 = zB3g0epem(j1,j2,j3,j4,j5,j7,j6,jhel(ihel)) ! Divide by photon propagator
         zwt1 = zwt1 + ztmp4*dconjg(ztmp3)

c         ztmp5 = zB3g0epem(i5,i2,i3,i4,i1,i6,i7,ihel)! Divide by photon propagator
c         ztmp6 = zB3g0epem(j5,j2,j3,j4,j1,j6,j7,jhel(ihel)) ! Divide by photon propagator
c         zwt2 = zwt2 + ztmp6*dconjg(ztmp5)
c
c         ztmp7 = zB3g0epem(i5,i2,i3,i4,i1,i7,i6,ihel) ! Divide by photon propagator
c         ztmp8 = zB3g0epem(j5,j2,j3,j4,j1,j7,j6,jhel(ihel)) ! Divide by photon propagator
c         zwt3 = zwt3 + ztmp8*dconjg(ztmp7)



      enddo
c      wt = dreal(zwt)

               B3g0Zintvec(1) = dreal(zwt)

c      wt1 = dreal(zwt1)

               B3g0Zintvec(2) = dreal(zwt1)


      B3g0Zintvec(3) =  B3g0Zintvec(2)      ! dreal(zwt2)
      B3g0Zintvec(4) =  B3g0Zintvec(1)         ! dreal(zwt3)   

      B3g0Zintvec = abs(zewfac)**2*B3g0Zintvec

c      write(*,*) zewfac(1),zewfac(2),zewfac(3),zewfac(4)

      B3g0epemint = sum(B3g0Zintvec)


c       B3g0epemint =  wt

      return
      end


***********************************************************************
* Amplitude with ordering qb q g g g ep em
*
      function zB3g0epem(i1,i2,i3,i4,i5,i6,i7,iHEL)
      use KinData_mod
      use B3gZ
      implicit real*8 (a-h,o-y)
      implicit complex*16(z)

      zi=(0d0,1d0)



* MHV amps

      if(ihel.eq.1)then
         zwt= zampB3g0Zppp(i1,i2,i3,i4,i5,i6,i7)
      endif

      if(ihel.eq.2)then
         zwt= zampB3g0Zppm(i1,i2,i3,i4,i5,i6,i7)
      endif

      if(ihel.eq.3)then
         zwt= zampB3g0Zpmp(i1,i2,i3,i4,i5,i6,i7)

       endif

      if(ihel.eq.4)then
         zwt= zampB3g0Zmpp(i1,i2,i3,i4,i5,i6,i7)

      endif

* MHV-bar amps
* ---
      if(ihel.eq.5)then
         zwt= dconjg(zampB3g0Zppp(i5,i4,i3,i2,i1,i7,i6))
      endif
* --+
      if(ihel.eq.6)then
         zwt= -dconjg(zampB3g0Zmpp(i5,i4,i3,i2,i1,i7,i6))
      endif
* -+-
      if(ihel.eq.7)then
         zwt= -dconjg(zampB3g0Zpmp(i5,i4,i3,i2,i1,i7,i6))
      endif
* +--
      if(ihel.eq.8)then
         zwt= -dconjg(zampB3g0Zppm(i5,i4,i3,i2,i1,i7,i6))
      endif


      zB3g0epem=zi*zwt/s7(i6,i7)
c       zB3g0epem=zwt/s7(i6,i7)

      return
      end

*************************************************************************  



