***************************************************
      module B3gZ

      contains

********************************************************************
*
* Quark pair tree level helicity amplitudes for
*
*     q + qb ->  g + g + g  + e- + e+
*
*     q(1) g(2) g(3) g(4) qbar(5) e-(6) e+(7)
*
* One colour ordering and helicity, ignoring overall factors
*
* Returns a 2x2x2 array where each element denotes the combinations of
* the helicities of the gluons. The elements are given by
*
* (1,1,1) => LLL
* (1,1,2) => LLR
* (1,2,1) => LRL
* (1,2,2) => LRR
* (2,1,1) => RLL
* (2,1,2) => RLR
* (2,2,1) => RRL
* (2,2,2) => RRR
*
* These amplitudes were calculated using the BCFW recursion relations and
* are available in hep-th/0507161
*

      function zampB3g0Zvec(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)
      complex*16 zampB3g0Zvec(2,2,2)

      zampB3g0Zvec(1,1,1)=dconjg(zampB3g0Zppp(i5,i4,i3,i2,i1,i7,i6))
      zampB3g0Zvec(1,1,2)=-dconjg(zampB3g0Zmpp(i5,i4,i3,i2,i1,i7,i6))
      zampB3g0Zvec(1,2,1)=-dconjg(zampB3g0Zpmp(i5,i4,i3,i2,i1,i7,i6))
      zampB3g0Zvec(1,2,2)=zampB3g0Zmpp(i1,i2,i3,i4,i5,i6,i7)
      zampB3g0Zvec(2,1,1)=-dconjg(zampB3g0Zppm(i5,i4,i3,i2,i1,i7,i6))
      zampB3g0Zvec(2,1,2)=zampB3g0Zpmp(i1,i2,i3,i4,i5,i6,i7)
      zampB3g0Zvec(2,2,1)=zampB3g0Zppm(i1,i2,i3,i4,i5,i6,i7)
      zampB3g0Zvec(2,2,2)=zampB3g0Zppp(i1,i2,i3,i4,i5,i6,i7)

      zampB3g0Zvec = zampB3g0Zvec/s7(i6,i7) ! Divide by photon propagator


      return
      end function


      function zampB3g0Zppp(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)


      zampB3g0Zppp = zA7(i5,i6)**2*zB7(i6,i7)
     .                     /(zA7(i5,i4)*zA7(i4,i3)*zA7(i3,i2)*zA7(i2,i1))

      return
      end function

      function zampB3g0Zmpp(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

c     statement functions
      zt2a(i1,i2,i3,i4) = zA7(i1,i2)*zB7(i2,i4)+zA7(i1,i3)*zB7(i3,i4)

      zt2b(i1,i2,i3,i4) = zB7(i1,i2)*zA7(i2,i4)+zB7(i1,i3)*zA7(i3,i4)

      zt3a(i1,i2,i3,i4,i5,i6)=zA7(i1,i2)*zB7(i2,i4)*zA7(i4,i6)
     .                       +zA7(i1,i2)*zB7(i2,i5)*zA7(i5,i6)
     .                       +zA7(i1,i3)*zB7(i3,i4)*zA7(i4,i6)
     .                       +zA7(i1,i3)*zB7(i3,i5)*zA7(i5,i6)

      zt3b(i1,i2,i3,i4,i5,i6)=zB7(i1,i2)*zA7(i2,i4)*zB7(i4,i6)
     .                       +zB7(i1,i2)*zA7(i2,i5)*zB7(i5,i6)
     .                       +zB7(i1,i3)*zA7(i3,i4)*zB7(i4,i6)
     .                       +zB7(i1,i3)*zA7(i3,i5)*zB7(i5,i6)

      t3(i1,i2,i3) = s7(i1,i2) + s7(i1,i3) + s7(i2,i3)
      t4(i1,i2,i3,i4) = s7(i1,i2) + s7(i1,i3) + s7(i1,i4)
     .                 +s7(i2,i3) + s7(i2,i4) + s7(i3,i4)

      zampB3g0Zmpp = +zA7(i5,i2)**3*zB7(i1,i7)*zA7(i7,i6)*zB7(i7,i1)
     .                     /(t4(i2,i3,i4,i5)*zA7(i5,i4)*zA7(i4,i3)*zA7(i3,i2)
     .                      *zt2a(i5,i6,i7,i1))

     .                     -zB7(i3,i1)**3*zA7(i5,i6)*zB7(i7,i6)*zA7(i6,i5)
     .                     /(t3(i1,i2,i3)*zA7(i5,i4)*zt2a(i4,i2,i3,i1)*zB7(i3,i2)*zB7(i2,i1))

     .                     +zt2a(i2,i4,i3,i1)**3*zA7(i5,i6)*zB7(i7,i6)*zA7(i6,i5)
     .                     /(t3(i2,i3,i4)*t4(i1,i2,i3,i4)*zA7(i4,i3)*zA7(i3,i2)
     .                      *zt2a(i4,i3,i2,i1)*zt2a(i5,i6,i7,i1))

      return
      end function

      function zampB3g0Zpmp(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

c     statement functions
      zt2a(i1,i2,i3,i4) = zA7(i1,i2)*zB7(i2,i4)+zA7(i1,i3)*zB7(i3,i4)

      zt2b(i1,i2,i3,i4) = zB7(i1,i2)*zA7(i2,i4)+zB7(i1,i3)*zA7(i3,i4)

      zt3a(i1,i2,i3,i4,i5,i6)=zA7(i1,i2)*zB7(i2,i4)*zA7(i4,i6)
     .                       +zA7(i1,i2)*zB7(i2,i5)*zA7(i5,i6)
     .                       +zA7(i1,i3)*zB7(i3,i4)*zA7(i4,i6)
     .                       +zA7(i1,i3)*zB7(i3,i5)*zA7(i5,i6)

      zt3b(i1,i2,i3,i4,i5,i6)=zB7(i1,i2)*zA7(i2,i4)*zB7(i4,i6)
     .                       +zB7(i1,i2)*zA7(i2,i5)*zB7(i5,i6)
     .                       +zB7(i1,i3)*zA7(i3,i4)*zB7(i4,i6)
     .                       +zB7(i1,i3)*zA7(i3,i5)*zB7(i5,i6)

      t3(i1,i2,i3) = s7(i1,i2) + s7(i1,i3) + s7(i2,i3)
      t4(i1,i2,i3,i4) = s7(i1,i2) + s7(i1,i3) + s7(i1,i4)
     .                 +s7(i2,i3) + s7(i2,i4) + s7(i3,i4)



      zampB3g0Zpmp = -zA7(i5,i3)**3*zA7(i3,i1)*
     .                      (zA7(i3,i2)*zB7(i2,i7)*zA7(i7,i6)
     .                      +zA7(i3,i1)*zB7(i1,i7)*zA7(i7,i6))
     .                      *zt2a(i3,i1,i2,i7)
     .                     /(zA7(i5,i4)*zA7(i4,i3)*zA7(i3,i2)*zA7(i2,i1)
     .                      *zt3a(i1,i6,i7,i4,i5,i3)*zt3a(i3,i2,i1,i7,i6,i5))

     .                     +zA7(i3,i1)*zt2a(i3,i1,i2,i4)**3*zA7(i5,i6)*zB7(i7,i6)*zA7(i6,i5)
     .                     /(t4(i1,i2,i3,i4)*t3(i1,i2,i3)*zA7(i3,i2)*zA7(i2,i1)*
     .                      (zA7(i3,i2)*zB7(i2,i6)*zA7(i6,i5)
     .                      +zA7(i3,i2)*zB7(i2,i7)*zA7(i7,i5)
     .                      +zA7(i3,i1)*zB7(i1,i6)*zA7(i6,i5)
     .                      +zA7(i3,i1)*zB7(i1,i7)*zA7(i7,i5))*zt2a(i1,i2,i3,i4))

     .                    -zA7(i5,i3)**3*zt2a(i3,i4,i5,i2)*
     .                     (zB7(i2,i5)*zA7(i5,i6)
     .                     +zB7(i2,i4)*zA7(i4,i6)
     .                     +zB7(i2,i3)*zA7(i3,i6))
     .                    *(zB7(i7,i6)*zA7(i6,i5)*zB7(i5,i2)
     .                     +zB7(i7,i6)*zA7(i6,i4)*zB7(i4,i2)
     .                     +zB7(i7,i6)*zA7(i6,i3)*zB7(i3,i2))
     .                    /(t4(i2,i3,i4,i5)*t3(i3,i4,i5)
     .                     *zA7(i5,i4)*zA7(i4,i3)*zt3a(i1,i6,i7,i5,i4,i3)
     .                     *zt2a(i5,i3,i4,i2))

     .                    -zB7(i4,i2)**4*zA7(i5,i6)*zB7(i7,i6)*zA7(i6,i5)
     .                    /(t3(i2,i3,i4)*zt2a(i1,i2,i3,i4)*zt2a(i5,i3,i4,i2)
     .                     *zB7(i4,i3)*zB7(i3,i2))

      return
      end function

      function zampB3g0Zppm(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

      zt2a(i1,i2,i3,i4) = zA7(i1,i2)*zB7(i2,i4)+zA7(i1,i3)*zB7(i3,i4)

      zt2b(i1,i2,i3,i4) = zB7(i1,i2)*zA7(i2,i4)+zB7(i1,i3)*zA7(i3,i4)

      zt3a(i1,i2,i3,i4,i5,i6)=zA7(i1,i2)*zB7(i2,i4)*zA7(i4,i6)
     .                       +zA7(i1,i2)*zB7(i2,i5)*zA7(i5,i6)
     .                       +zA7(i1,i3)*zB7(i3,i4)*zA7(i4,i6)
     .                       +zA7(i1,i3)*zB7(i3,i5)*zA7(i5,i6)

      zt3b(i1,i2,i3,i4,i5,i6)=zB7(i1,i2)*zA7(i2,i4)*zB7(i4,i6)
     .                       +zB7(i1,i2)*zA7(i2,i5)*zB7(i5,i6)
     .                       +zB7(i1,i3)*zA7(i3,i4)*zB7(i4,i6)
     .                       +zB7(i1,i3)*zA7(i3,i5)*zB7(i5,i6)

      t3(i1,i2,i3) = s7(i1,i2) + s7(i1,i3) + s7(i2,i3)
      t4(i1,i2,i3,i4) = s7(i1,i2) + s7(i1,i3) + s7(i1,i4)
     .                 +s7(i2,i3) + s7(i2,i4) + s7(i3,i4)


      zampB3g0Zppm=      -zA7(i4,i1)*(zA7(i4,i3)*zB7(i3,i7)*zA7(i7,i6)
     .                                 +zA7(i4,i2)*zB7(i2,i7)*zA7(i7,i6)
     .                                 +zA7(i4,i1)*zB7(i1,i7)*zA7(i7,i6))
     .                                *(zB7(i7,i3)*zA7(i3,i4)
     .                                 +zB7(i7,i2)*zA7(i2,i4)
     .                                 +zB7(i7,i1)*zA7(i1,i4))
     .                     /(t4(i1,i2,i3,i4)*zA7(i4,i3)*zA7(i3,i2)
     .                      *zA7(i2,i1)*zt2a(i1,i7,i6,i5))

     .                    +zB7(i5,i3)*(zB7(i3,i5)*zA7(i5,i6)+zB7(i3,i4)*zA7(i4,i6))
     .                               *(zB7(i7,i6)*zA7(i6,i5)*zB7(i5,i3)
     .                                +zB7(i7,i6)*zA7(i6,i4)*zB7(i4,i3))
     .                    /(t3(i5,i4,i3)*zA7(i2,i1)*zB7(i5,i4)*zB7(i4,i3)
     .                     *zt2a(i2,i3,i4,i5))

     .                    +zt2a(i4,i2,i3,i5)*(zA7(i4,i3)*zB7(i3,i5)*zA7(i5,i6)
     .                                       +zA7(i4,i3)*zB7(i3,i4)*zA7(i4,i6)
     .                                       +zA7(i4,i3)*zB7(i3,i2)*zA7(i2,i6)
     .                                       +zA7(i4,i2)*zB7(i2,i5)*zA7(i5,i6)
     .                                       +zA7(i4,i2)*zB7(i2,i4)*zA7(i4,i6)
     .                                       +zA7(i4,i2)*zB7(i2,i3)*zA7(i3,i6))
     .                                      *(zB7(i7,i6)*zA7(i6,i5)*zB7(i5,i3)*zA7(i3,i4)
     .                                       +zB7(i7,i6)*zA7(i6,i4)*zB7(i4,i3)*zA7(i3,i4)
     .                                       +zB7(i7,i6)*zA7(i6,i2)*zB7(i2,i3)*zA7(i3,i4)
     .                                       +zB7(i7,i6)*zA7(i6,i5)*zB7(i5,i2)*zA7(i2,i4)
     .                                       +zB7(i7,i6)*zA7(i6,i4)*zB7(i4,i2)*zA7(i2,i4)
     .                                       +zB7(i7,i6)*zA7(i6,i3)*zB7(i3,i2)*zA7(i2,i4))
     .                    /(t4(i5,i4,i3,i2)*t3(i2,i3,i4)*zA7(i4,i3)*zA7(i3,i2)
     .                   *zt2a(i1,i7,i6,i5)*zt2a(i2,i4,i3,i5))

      return
      end function

      end module B3gZ
