***********************************************************************
*
* Quark pair one loop level virtual correction subleading colour matrix element for
*
* q(1) qb(3) -> g(2) e+(4) e-(5)
*
* summed and squared over helicities, single colour ordering without
* overall factors
*
* The flavour of the quark pair in this matrix element is determined from the flav variable in the common block /BZFlav/

      function Bt1g1Z(i1,i2,i3,i4,i5,renscale2)
      use KinData_mod
      use B1gZ, only : zampBt1g1Z
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      double precision Bt1g1Zvec(4)
      parameter(pi=3.141592653589793238d0)
      common /order/iorder
      common /koppel/ nf,nc,b0
      common /BZFlav/nfb1g
!$omp threadprivate(/BZFlav/)
      double complex zamp(4)
      common /eweakZ/ amz,zewfac(4),zewnull(4)
!$omp threadprivate(/eweakZ/)
      integer ihel(5,4)

      ihel = reshape((/i1,i2,i3,i5,i4,
     .                 i1,i2,i3,i4,i5,
     .                 i3,i2,i1,i5,i4,
     .                 i3,i2,i1,i4,i5/),(/5,4/))

      do i=1,4
         zamp = zampBt1g1Z(ihel(1,i),ihel(2,i),ihel(3,i),ihel(4,i),ihel(5,i),renscale2)
         Bt1g1Zvec(i) = dble(dconjg(zamp(1))*zamp(2))+dble(dconjg(zamp(3))*zamp(4))
      enddo

      s = s5(i4,i5)

      call eweakV1qq(nfb1g,s)

      Bt1g1Zvec = abs(zewfac)**2*Bt1g1Zvec

      Bt1g1Z = sum(Bt1g1Zvec)

      if(iorder.eq.0)then
c     pi**2/6d0 comes from a correction for C(e)
c     1/2 comes from a conversion of four-dimensional helicity scheme (FDH) to 't Hooft-Veltman (HV) scheme
c     see hep-ph/9708239v1 equation 6.4
         e0 =-pi**2/12d0 + 0.5d0 ! factor from C_gamma expansion
         Bt1g1Z = Bt1g1Z-e0*B1g0Z(i1,i2,i3,i4,i5)
      endif

      return
      end

************************************************************************
* A wrapper function for Bt1g1Z which inherits the flavour of the first
* pair of quarks from the C0g0Z common block.
*
* This must be called in the maple code for 4 quark processes where the
* reduced matrix element has the flavour of the first pair of quarks
*

      function f1Bt1g1Z(i1,i2,i3,i4,i5,renscale2)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /CZFlav/nf1,nf2
!$omp threadprivate(/CZFlav/)
      common /BZFlav/nf1g
!$omp threadprivate(/BZFlav/)

      if(nf1.ne.0)then
         nf1g = nf1
         f1Bt1g1Z = Bt1g1Z(i1,i2,i3,i4,i5,renscale2)
      else
         write(*,*) "Error in f1Bt1g1Z: flavours have not been initialised"
         stop
      endif

      return
      end


************************************************************************
* A wrapper function for B1g1Z which inherits the flavour of the second
* pair of quarks from the C0g0Z common block.
*
* This must be called in the maple code for 4 quark processes where the
* reduced matrix element has the flavour of the second pair of quarks
*

      function f2Bt1g1Z(i1,i2,i3,i4,i5,renscale2)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /CZFlav/nf1,nf2
!$omp threadprivate(/CZFlav/)
      common /BZFlav/nf1g
!$omp threadprivate(/BZFlav/)

      if(nf2.ne.0)then
         nf1g = nf2
         f2Bt1g1Z = Bt1g1Z(i1,i2,i3,i4,i5,renscale2)
      else
         write(*,*) "Error in f2Bt1g0Z: flavours have not been initialised"
         stop
      endif

      return
      end
