****************************************************
* A wrapper function to sum over the four helicity combinations
* of the quark/lepton pair for the process
*
* q(1) qbar(4) -> g(2) g(3) e+(5) e-(6)
*

      function Bt2g0Z(i1,i2,i3,i4,i5,i6)
      use KinData_mod
      use B2gZ, only : zampB2g0Z
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /eweakZ/ amz,zewfac(4),zewnull(4)
!$omp threadprivate(/eweakZ/)
      common /BZFlav/ nfB2g
!$omp threadprivate(/BZFlav/)
      double precision Bt2g0Zvec(4)
      complex*16 zampBt2g0ZLLa(2,2),zampBt2g0ZLLb(2,2)
      complex*16 zampBt2g0ZLRa(2,2),zampBt2g0ZLRb(2,2)

      Bt2g0Zvec = 0d0

      zampBt2g0ZLLa = zampB2g0Z(i1,i2,i3,i4,i5,i6)
      zampBt2g0ZLLb = zampB2g0Z(i1,i3,i2,i4,i5,i6)
      zampBt2g0ZLRa = zampB2g0Z(i1,i2,i3,i4,i6,i5)
      zampBt2g0ZLRb = zampB2g0Z(i1,i3,i2,i4,i6,i5)


      do i=1,2
         do j=1,2

            zampBt2g0Za = zampBt2g0ZLLa(i,j) + zampBt2g0ZLLb(j,i)
            zampBt2g0Zb = zampBt2g0ZLRa(i,j) + zampBt2g0ZLRb(j,i)

            Bt2g0Zvec(1) = Bt2g0Zvec(1) + abs(zampBt2g0Za)**2
            Bt2g0Zvec(2) = Bt2g0Zvec(2) + abs(zampBt2g0Zb)**2

         enddo
      enddo

      Bt2g0Zvec(3) = Bt2g0Zvec(2)
      Bt2g0Zvec(4) = Bt2g0Zvec(1)

      s = s6(i5,i6)

      call eweakV1qq(nfB2g,s)

      Bt2g0Zvec = abs(zewfac)**2*Bt2g0Zvec

      Bt2g0Z = sum(Bt2g0Zvec)

      return
      end

************************************************************************
* A wrapper function for Bt2g0Z which inherits the flavour of the first
* pair of quarks from the C0g0Z common block.
*
* This must be called in the maple code for 4 quark processes where the
* reduced matrix element has the flavour of the first pair of quarks


      function f1Bt2g0Z(i1,i2,i3,i4,i5,i6)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /CZFlav/nf1,nf2
!$omp threadprivate(/CZFlav/)
      common /BZFlav/nf1g
!$omp threadprivate(/BZFlav/)

      if(nf1.ne.0)then
         nf1g = nf1
         f1Bt2g0Z = Bt2g0Z(i1,i2,i3,i4,i5,i6)
      else
         write(*,*) "Error in f1Bt2g0Z: flavours have not been initialised"
         stop
      endif

      return
      end

************************************************************************
* A wrapper function for Bt2g0Z which inherits the flavour of the second
* pair of quarks from the C0g0Z common block.
*
* This must be called in the maple code for 4 quark processes where the
* reduced matrix element has the flavour of the second pair of quarks
*

      function f2Bt2g0Z(i1,i2,i3,i4,i5,i6)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /CZFlav/nf1,nf2
!$omp threadprivate(/CZFlav/)
      common /BZFlav/nf1g
!$omp threadprivate(/BZFlav/)

      if(nf2.ne.0)then
         nf1g = nf2
         f2Bt2g0Z = Bt2g0Z(i1,i2,i3,i4,i5,i6)
      else
         write(*,*) "Error in f2Bt2g0Z: flavours have not been initialised"
         stop
      endif

      return
      end

*******************************************************************
* Symmetrised B2g0Z across the qqb pair, taking the flavour from
* the first quark pair of the C-type matrix elements.
*
*

      function sf1Bt2g0Z(i1,i2,i3,i4,i5,i6)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      sf1Bt2g0Z = 0.5d0*(f1Bt2g0Z(i1,i2,i3,i4,i5,i6)
     .                 +f1Bt2g0Z(i4,i3,i2,i1,i5,i6))

      return
      end

*******************************************************************
* Symmetrised B2g0Z across the qqb pair, taking the flavour from the
* second quark pair of the C-type matrix elements.
*

      function sf2Bt2g0Z(i1,i2,i3,i4,i5,i6)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      sf2Bt2g0Z = 0.5d0*(f2Bt2g0Z(i1,i2,i3,i4,i5,i6)
     .                  +f2Bt2g0Z(i4,i3,i2,i1,i5,i6))

      return
      end


****************************************************
* A flavour averaged version of Bt2g0Z. This is required in certain
* subtraction terms at RV
*

      function fBt2g0Z(i1,i2,i3,i4,i5,i6)
      use KinData_mod
      use B2gZ, only : zampB2g0Z
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /koppel/nf,nc,b0
      common /eweakZ/ amz,zewfac(4),zewnull(4)
!$omp threadprivate(/eweakZ/)
      common /BZFlav/ nfB2g
!$omp threadprivate(/BZFlav/)
      integer ni(2)
      data ni /3,2/ ! this is nup and ndown, hard coded for now
      double precision Bt2g0Zvec(4),ewfacstore(4)
      complex*16 zampBt2g0ZLLa(2,2),zampBt2g0ZLLb(2,2)
      complex*16 zampBt2g0ZLRa(2,2),zampBt2g0ZLRb(2,2)

      Bt2g0Zvec=0d0

      zampBt2g0ZLLa = zampB2g0Z(i1,i2,i3,i4,i5,i6)
      zampBt2g0ZLLb = zampB2g0Z(i1,i3,i2,i4,i5,i6)
      zampBt2g0ZLRa = zampB2g0Z(i1,i2,i3,i4,i6,i5)
      zampBt2g0ZLRb = zampB2g0Z(i1,i3,i2,i4,i6,i5)

      do i=1,2
         do j=1,2

            zampBt2g0Za = zampBt2g0ZLLa(i,j) + zampBt2g0ZLLb(j,i)
            zampBt2g0Zb = zampBt2g0ZLRa(i,j) + zampBt2g0ZLRb(j,i)

            Bt2g0Zvec(1) = Bt2g0Zvec(1) + abs(zampBt2g0Za)**2
            Bt2g0Zvec(2) = Bt2g0Zvec(2) + abs(zampBt2g0Zb)**2

         enddo
      enddo

      Bt2g0Zvec(3) = Bt2g0Zvec(2)
      Bt2g0Zvec(4) = Bt2g0Zvec(1)

      s56 = s6(i5,i6)

      ewfacstore = 0d0

      do i=1,2
         call eweakV1qq(i,s56)
         ewfacstore = ewfacstore + (dble(ni(i))/dble(nf))*abs(zewfac)**2
      enddo

      Bt2g0Zvec = ewfacstore*Bt2g0Zvec

      fBt2g0Z = sum(Bt2g0Zvec)

      return
      end


