***********************************************************************
*
* Quark pair one loop level virtual correction matrix element for
*
* q(i1) qb(i4) -> g(i2) g(i3) e+(i5) e-(i6)
*
* summed and squared over helicities, single colour ordering without
* overall factors and subleading colour
*
* The flavour of the quark pair in this matrix element is determined from the flav variable in the common block /BZFlav/

      function Bt2g1Z(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      use B2gZ
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /eweakZ/ amz,zewfac(4),zewnull(4)
!$omp threadprivate(/eweakZ/)
      common /order/iorder
      common /koppel/ nf,nc,b0
      common /BZFlav/nf2g
!$omp threadprivate(/BZFlav/)
      double precision Bt2g1Zvec(4)
      double complex zampTree(2,2),zampqqbgg1(2,2)
      integer ihel(6,4)

      Bt2g1Z = 0d0

      do ii=1,4
         Bt2g1Zvec(ii) = 0d0
      enddo

      s= s6(i5,i6)

      call eweakV1qq(nf2g,s)

      ihel = reshape((/i1,i2,i3,i4,i5,i6,
     .                 i1,i2,i3,i4,i6,i5,
     .                 i1,i2,i3,i4,i6,i5,
     .                 i1,i2,i3,i4,i5,i6/),(/6,4/))

      do i=1,4
         if (i.le.2) then

            zampTree = zampB2g0Z(ihel(1,i),ihel(2,i),ihel(3,i)
     .                          ,ihel(4,i),ihel(5,i),ihel(6,i))

            zampqqbgg1 = zampBtt2g1Z(ihel(1,i),ihel(2,i),ihel(3,i)
     .                              ,ihel(4,i),ihel(5,i),ihel(6,i),renscale2)

         else
            call swapzAB_kin(6)
            zampTree = zampB2g0Z(ihel(1,i),ihel(2,i),ihel(3,i)
     .                          ,ihel(4,i),ihel(5,i),ihel(6,i))

            zampqqbgg1 = zampBtt2g1Z(ihel(1,i),ihel(2,i),ihel(3,i)
     .                              ,ihel(4,i),ihel(5,i),ihel(6,i),renscale2)
            call swapzAB_kin(6)
         endif

            do ii=1,2
               do jj=1,2
                  Bt2g1Zvec(i) = Bt2g1Zvec(i)+dble(Dconjg(zampTree(ii,jj))*zampqqbgg1(ii,jj))
               enddo
            enddo
      enddo

      Bt2g1Zvec = abs(zewfac)**2*Bt2g1Zvec

      Bt2g1Z = sum(Bt2g1Zvec)

      if (iorder.eq.0)then
c     pi**2/6d0 comes from a correction for C(e)
c     1/2 comes from a conversion of four-dimensional helicity scheme (FDH) to t Hooft-Veltman (HV) scheme
c     see hep-ph/9708239v1 equation 6.4
         e0 = -pi**2/12d0 + 0.5d0
         Bt2g1Z = Bt2g1Z - e0*B2g0Z(i1,i2,i3,i4,i5,i6)
      endif

      return
      end

***********************************************************************
*
* Quark pair one loop level virtual correction matrix element for
*
* q(i1) qb(i4) -> g(i2) g(i3) e+(i5) e-(i6)
*
* summed and squared over helicities, single colour ordering without
* overall factors, subleading nf contribution
*
* The flavour of the quark pair in this matrix element is determined from the flav variable in the common block /BZFlav/
*
*

      function Bth2g1Z(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      use B2gZ, only : zampB2g0Z,zampBh2g1Z
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /eweakZ/ amz,zewfac(4),zewnull(4)
!$omp threadprivate(/eweakZ/)
      common /order/iorder
      common /koppel/nf,nc,b0
      common /BZFlav/nf2g
!$omp threadprivate(/BZFlav/)
      double precision Bth2g1Zvec(4)
      double complex zampnfLoop(2,2),zampTree(2,2)
      integer ihel(6,4)

c      write(6,*) 'here Bth2g1Z', sqrt(renscale2)
      
      Bth2g1Z = 0d0
      do ii=1,4
         Bth2g1Zvec(ii) = 0d0
      enddo

      if(iorder.eq.0) then

      call eweakV1qq(nf2g,s)

      !> Nothing to be done, the finite part of the amplitude is 0
      !> To get the photon-like amplitude, one would sum over the two gluonic (leading-nf) amplitudes swapping the gluons
      !> but this sum is identically 0

      Bth2g1Zvec = abs(zewfac)**2*Bth2g1Zvec
      Bth2g1Z = sum(Bth2g1Zvec)
      elseif (iorder .eq. -1) then
         subuv =2d0/3d0
         Bth2g1Z=subuv*Bt2g0Z(i1,i2,i3,i4,i5,i6)
      endif


      return
      end

***********************************************************************
*
* Quark pair one loop level virtual correction matrix element for
*
* q(i1) qb(i4) -> g(i2) g(i3) e+(i5) e-(i6)
*
* summed and squared over helicities, single colour ordering without
* overall factors, axial coupling contribution subleading colour
*
* The flavour of the quark pair in this matrix element is determined from the flav variable in the common block /BZFlav/

      function Bt2g1Zax(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      use B2gZ, only : zampB2g0Z,zFaxqqbgg,zFaxslqqbgg
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /eweakZ/ amz,zewfac(4),zewnull(4)
!$omp threadprivate(/eweakZ/)
      common /axial/zewax(2)
!$omp threadprivate(/axial/)
      common /order/iorder
      common /koppel/ nf,nc,b0
      common /BZFlav/nf2g
!$omp threadprivate(/BZFlav/)
      double complex zBt2g1Zaxvec(4)
      double complex zampaxloop(2,2),zampTree(2,2)
      double complex ztemp1(2,2),ztemp2(2,2),ztemp3(2,2)
      integer ihel(6,4)

      Bt2g1Zax = 0d0
      do ii=1,4
         zBt2g1Zaxvec(ii) = 0d0
      enddo

      if(iorder.eq.0)then

      s = s6(i5,i6)

      call eweakV1qq(nf2g,s)

      ihel = reshape((/i1,i2,i3,i4,i5,i6,
     .                 i1,i2,i3,i4,i6,i5,
     .                 i1,i2,i3,i4,i6,i5,
     .                 i1,i2,i3,i4,i5,i6/),(/6,4/))

      do i=1,4
         if (i.le.2) then


            zampTree = zampB2g0Z(ihel(1,i),ihel(2,i),ihel(3,i)
     .           ,ihel(4,i),ihel(5,i),ihel(6,i))
     .                +Transpose(zampB2g0Z(ihel(1,i),ihel(3,i),ihel(2,i) ! note the gluon swapping
     .           ,ihel(4,i),ihel(5,i),ihel(6,i)))

            zampaxLoop = 2d0*zFaxqqbgg(ihel(1,i),ihel(2,i),ihel(3,i)
     .           ,ihel(4,i),ihel(5,i),ihel(6,i),renscale2)
     .                  +2d0*Transpose(zFaxqqbgg(ihel(1,i),ihel(3,i),ihel(2,i)
     .           ,ihel(4,i),ihel(5,i),ihel(6,i),renscale2))
     .                  -1d0*zFaxslqqbgg(ihel(1,i),ihel(2,i),ihel(3,i)
     .           ,ihel(4,i),ihel(5,i),ihel(6,i),renscale2)

            ztemp1 = 2d0*zFaxqqbgg(ihel(1,i),ihel(2,i),ihel(3,i)
     .           ,ihel(4,i),ihel(5,i),ihel(6,i),renscale2)
            ztemp2 = +2d0*zFaxqqbgg(ihel(1,i),ihel(3,i),ihel(2,i)
     .           ,ihel(4,i),ihel(5,i),ihel(6,i),renscale2)
            ztemp3 = -1d0*zFaxslqqbgg(ihel(1,i),ihel(2,i),ihel(3,i)
     .           ,ihel(4,i),ihel(5,i),ihel(6,i),renscale2)
         else
            call swapzAB_kin(6)
            zampTree = zampB2g0Z(ihel(1,i),ihel(2,i),ihel(3,i)
     .           ,ihel(4,i),ihel(5,i),ihel(6,i))
     .                +Transpose(zampB2g0Z(ihel(1,i),ihel(3,i),ihel(2,i) ! note the gluon swapping
     .           ,ihel(4,i),ihel(5,i),ihel(6,i)))

            zampaxLoop = 2d0*zFaxqqbgg(ihel(1,i),ihel(2,i),ihel(3,i)
     .           ,ihel(4,i),ihel(5,i),ihel(6,i),renscale2)
     .                  +2d0*Transpose(zFaxqqbgg(ihel(1,i),ihel(3,i),ihel(2,i)
     .           ,ihel(4,i),ihel(5,i),ihel(6,i),renscale2))
     .                  -1d0*zFaxslqqbgg(ihel(1,i),ihel(2,i),ihel(3,i)
     .           ,ihel(4,i),ihel(5,i),ihel(6,i),renscale2)

           ztemp1 = 2d0*zFaxqqbgg(ihel(1,i),ihel(2,i),ihel(3,i)
     .           ,ihel(4,i),ihel(5,i),ihel(6,i),renscale2)
            ztemp2 = +2d0*zFaxqqbgg(ihel(1,i),ihel(3,i),ihel(2,i)
     .           ,ihel(4,i),ihel(5,i),ihel(6,i),renscale2)
            ztemp3 = -1d0*zFaxslqqbgg(ihel(1,i),ihel(2,i),ihel(3,i)
     .           ,ihel(4,i),ihel(5,i),ihel(6,i),renscale2)
            call swapzAB_kin(6)
         endif

         do ii=1,2
            do jj=1,2
               if (i.le.2)then
                  zBt2g1Zaxvec(i) = zBt2g1Zaxvec(i)
     .                 -dconjg(zampTree(ii,jj))*zampaxLoop(ii,jj)
               else
                  zBt2g1Zaxvec(i) = zBt2g1Zaxvec(i)
     .                 +dconjg(zampTree(ii,jj))*zampaxLoop(ii,jj)
               endif
            enddo
         enddo
      enddo

      do ii=1,4
         if ((ii.eq.1).or.(ii.eq.3))then
            jj = 1
         else
            jj = 2
         endif
         zBt2g1Zaxvec(ii) = dconjg(zewfac(ii))*zewax(jj)*zBt2g1Zaxvec(ii)
      enddo

      Bt2g1Zax = dble(sum(zBt2g1Zaxvec))
      endif

      return
      end

***********************************************************************
*
* Quark pair one loop level virtual correction matrix element for
*
* q(i1) qb(i4) -> g(i2) g(i3) e+(i5) e-(i6)
*
* summed and squared over helicities, single colour ordering without
* overall factors, finite top corrections subleading colour
*
* The flavour of the quark pair in this matrix element is determined from the flav variable in the common block /BZFlav/

      function Bt2g1Ztop(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      use Parameters_mod
      use B2gZ, only : zampB2g0Z,zFaxqqbgg,zFaxslqqbgg
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /eweakZ/ amz,zewfac(4),zewnull(4)
!$omp threadprivate(/eweakZ/)
      common /order/iorder
      common /koppel/ nf,nc,b0
      double precision :: emt,etwidth,emb,ebwidth,emc,ecwidth,emtau,etauwidth
      common /Qmass/ emt,etwidth,emb,ebwidth,emc,ecwidth,emtau,etauwidth

      if (emt.eq.0d0)then
         write(*,*) "Error in finite top correction to B2g1Z,
     . the mass of the top has not been defined"
         stop
      endif

      Bt2g1Ztop = 0d0

      if (iorder.eq.0)then
         Bt2g1Ztop = (1d0/20d0)*s6(i2,i3)/emt**2*Bth2g1Z(i1,i2,i3,i4,i5,i6,renscale2)
      endif

      return
      end
