********************************************************************
*
* sub leading colour Quark pair tree level leading colour matrix element for
*
*     q + qb ->  g + g + g + e- + e+
*
* summed over colour orderings, summed and squared
*
*     q(1) g(2) g(3) g(4) qbar(5) e-(6) e+(7)
*

      function FullBt3g0Z(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)
      common /eweakZ/ amz,zewfac(4),zewnull(4)
!$omp threadprivate(/eweakZ/)
      double precision FullBt3g0Zvec(4)

      fac = get_avg(i1,i5)

      fac = fac*(4d0*pi*as)**3*(4d0*pi*amz)**2*nc**3*(nc**2-1)/(2d0*nc)*8d0

      if((i1.le.2).and.(i5.le.2))then
         fac = fac/6d0 ! statistical factor
      elseif((i1.le.2).or.(i5.le.2))then
         fac = fac/2d0
      endif

      FullBt3g0Z = Bt3g0Z(i1,i2,i3,i4,i5,i6,i7)
     .            +Bt3g0Z(i1,i2,i4,i3,i5,i6,i7)
     .            +Bt3g0Z(i1,i3,i2,i4,i5,i6,i7)
     .            +Bt3g0Z(i1,i3,i4,i2,i5,i6,i7)
     .            +Bt3g0Z(i1,i4,i2,i3,i5,i6,i7)
     .            +Bt3g0Z(i1,i4,i3,i2,i5,i6,i7)

      FullBt3g0Z =  fac*FullBt3g0Z

c     write(*,*) "FullBt3g0Z",FullBt3g0Z,i1,i2,i3,i4,i5,i6,i7

      return
      end

********************************************************************
*
* Sub leading colour Quark pair tree level matrix element for
*
*     q + qb ->  g + g + g + e- e+
*
* One colour ordering, summed over spins and squared
*
*  q(1) g~(2) g(3) g(4) qbar(5) e-(6) e+(7)
*
* Note that gluon g(2) is abelian

      function Bt3g0Z(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      use B3gZ, only : zampB3g0Zvec
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /proc/ isub(7)
      common /eweakZ/ amz,zewfac(4),zewnull(4)
!$omp threadprivate(/eweakZ/)
      common /BZFlav/nf3g
!$omp threadprivate(/BZFlav/)
      double precision Bt3g0Zvec(4)
      complex*16 zampLL1(2,2,2),zampLL2(2,2,2),zampLL3(2,2,2)
      complex*16 zampLR1(2,2,2),zampLR2(2,2,2),zampLR3(2,2,2)

      s67 = s7(i6,i7)

      call eweakV1qq(nf3g,s67)

      do i=1,4
         Bt3g0Zvec(i) = 0d0
      enddo

      zampLL1 = zampB3g0Zvec(i1,i2,i3,i4,i5,i6,i7)
      zampLL2 = zampB3g0Zvec(i1,i3,i4,i2,i5,i6,i7)
      zampLL3 = zampB3g0Zvec(i1,i3,i2,i4,i5,i6,i7)

      zampLR1 = zampB3g0Zvec(i1,i2,i3,i4,i5,i7,i6)
      zampLR2 = zampB3g0Zvec(i1,i3,i4,i2,i5,i7,i6)
      zampLR3 = zampB3g0Zvec(i1,i3,i2,i4,i5,i7,i6)

      do i=1,2
         do j=1,2
            do k=1,2
               zampB3g0ZvecLL = zampLL2(j,k,i)+zampLL3(j,i,k)+zampLL1(i,j,k)
               zampB3g0ZvecLR = zampLR2(j,k,i)+zampLR3(j,i,k)+zampLR1(i,j,k)
               Bt3g0Zvec(1) = Bt3g0Zvec(1) + abs(zampB3g0ZvecLL)**2
               Bt3g0Zvec(2) = Bt3g0Zvec(2) + abs(zampB3g0ZvecLR)**2
            enddo
         enddo
      enddo

      Bt3g0Zvec(3) = Bt3g0Zvec(2)
      Bt3g0Zvec(4) = Bt3g0Zvec(1)

      Bt3g0Zvec = abs(zewfac)**2*Bt3g0Zvec

      Bt3g0Z = sum(Bt3g0Zvec)

      return
      end

************************************************************************
* A wrapper function for Bt3g0Z which inherits the flavour of the first
* pair of quarks from the C0g0Z common block.
*
* This must be called in the maple code for 4 quark processes where the
* reduced matrix element has the flavour of the first pair of quarks


      function f1Bt3g0Z(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /CZFlav/nf1,nf2
!$omp threadprivate(/CZFlav/)
      common /BZFlav/nf1g
!$omp threadprivate(/BZFlav/)

      if(nf1.ne.0)then
         nf1g = nf1
         f1Bt3g0Z = Bt3g0Z(i1,i2,i3,i4,i5,i6,i7)
      else
         write(*,*) "Error in f1Bt3g0Z: flavours have not been initialised"
         stop
      endif

      return
      end

************************************************************************

* A wrapper function for Bt3g0Z which inherits the flavour of the second
* pair of quarks from the C0g0Z common block.
*
* This must be called in the maple code for 4 quark processes where the
* reduced matrix element has the flavour of the second pair of quarks
*

      function f2Bt3g0Z(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /CZFlav/nf1,nf2
!$omp threadprivate(/CZFlav/)
      common /BZFlav/nf1g
!$omp threadprivate(/BZFlav/)

      if(nf2.ne.0)then
         nf1g = nf2
         f2Bt3g0Z = Bt3g0Z(i1,i2,i3,i4,i5,i6,i7)
      else
         write(*,*) "Error in f2Bt3g0Z: flavours have not been initialised"
         stop
      endif

      return
      end



************************************************************************      



