****************************************************
* Returns the leading colour matrix element for the process
*
* q(i1) + qbar(i4) ->  + Q(i3) + Qbar(i2) + e+(5) + e-(6)
*
* The common block /CZFlav/ denotes the flavour of the quark pair i1 and i4 (nf1) and the flavour of the quark pair i2 and i3 (nf2)
*
* The Z boson couples to the i1 and i4 pair of quarks.
*

      function FullC0g0Z(i1,i2,i3,i4,i5,i6)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)

      fac = 0.25d0/nc**2

      fac = 4d0*fac*(nc**2-1d0)*(4d0*pi*as)**2*(4d0*pi*amz)**2

      FullC0g0Z = C0g0Z(i1,i2,i3,i4,i5,i6)

      if((i1.le.2).and.(i2.le.2).and.(nf1.eq.nf2))then ! qqb initial state identical flavour

         nup = 2
         ndo = 3

         nf2 = 2 ! Up type contribution

         FullC0g0Z = FullC0g0Z
     .              +nup*C0g0Z(i1,i4,i3,i2,i5,i6)

         nf2 = 1 ! Down type contribution

         FullC0g0Z = FullC0g0Z
     .              +ndo*C0g0Z(i1,i4,i3,i2,i5,i6)

         nf2 = nf1 ! reset

         FullC0g0Z = FullC0g0Z ! subleading colour
     .              -(1d0/nc)*D0g0Z(i1,i2,i3,i4,i5,i6)

      elseif(nf1.eq.nf2)then ! identical flavour, other initial state configurations
         if (((i1.le.2).and.(i3.le.2)).or.((i2.le.2).and.(i4.le.2)))then ! qbqb or qq initial state
            fac = 0.5d0*fac     ! statistical factor
         endif

         FullC0g0Z = FullC0g0Z
     .              +C0g0Z(i1,i4,i3,i2,i5,i6)
     .              -(1d0/nc)*D0g0Z(i1,i2,i3,i4,i5,i6)

      endif

      FullC0g0Z = fac*FullC0g0Z

      return
      end

****************************************************
* Returns the leading colour matrix element for the process
*
* q(1) + qbar(4) -> Q(3) + Qbar(2) + e+(5) + e-(6)
*
* Ignoring overall factors
*
* The Z boson couples to the i1 and i4 pair of quarks
*

      function C0g0Z(i1,i2,i3,i4,i5,i6)
      use KinData_mod
      use C0gZ, only : zampC0g0ZGen
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      complex*16 zampqRbA(2,2),zampqRbB(2,2)
      complex*16 zamp(2,2,2)

      C0g0Z = 0d0

      call eweakV2qq(nf1,nf2,s6(i5,i6))

      call zampC0g0ZGen(i1,i3,i2,i4,i5,i6,zampqRbA,zampqRbB)

c     q L, e L
      zamp(1,1,1) = zewfac1(1)*zampqRbA(1,1)
     .             +zewfac2(1)*zampqRbB(1,1)

c     q L, e R
      zamp(1,1,2) = zewfac1(2)*dconjg(zampqRbA(2,2))
     .             +zewfac2(2)*dconjg(zampqRbB(2,2))

c     q R, e L
      zamp(2,2,1) = zewfac1(3)*zampqRbA(2,2)
     .             +zewfac2(3)*zampqRbB(2,2)

c     q R, e R
      zamp(2,2,2) = zewfac1(4)*dconjg(zampqRbA(1,1))
     .             +zewfac2(4)*dconjg(zampqRbB(1,1))

      zamp(1,2,1) = zewfac1(1)*zampqRbA(1,2)
     .             -zewfac2(3)*zampqRbB(1,2)

      zamp(1,2,2) = zewfac1(2)*dconjg(zampqRbA(2,1))
     .             -zewfac2(4)*dconjg(zampqRbB(2,1))

      zamp(2,1,1) = zewfac1(3)*zampqRbA(2,1)
     .             -zewfac2(1)*zampqRbB(2,1)

      zamp(2,1,2) = zewfac1(4)*dconjg(zampqRbA(1,2))
     .             -zewfac2(2)*dconjg(zampqRbB(1,2))

c      write(6,*) '0:'
c      do ii=1,2
c         do kk=1,2
c            do jj=1,2
c               write(6,*) ii,jj,kk,zamp(ii,jj,kk)
c            enddo
c         enddo
c      enddo
      
      C0g0Z = sum(abs(zamp)**2)

c      C0g0Z = abs(zamp(1,1,1)**2)
      
      return
      end

      function sC0g0Z(i1,i2,i3,i4,i5,i6)
      implicit double precision (a-h,o-y)
      !> Only used for DIS1j
      if ((i2.le.2).or.(i3.le.2)) then ! No symmetrisation for qp,qpb init production [CC]
         sC0g0Z = C0g0Z(i1,i2,i3,i4,i5,i6)
      else
         sC0g0Z=0.5d0*(C0g0Z(i1,i2,i3,i4,i5,i6)+C0g0Z(i1,i3,i2,i4,i5,i6))
      endif
      return
      end


**********************************************************************
