****************************************************
* Returns the virtual one-loop matrix element for the process
*
* q(1) + qbar(4) -> Q(2) + Qbar(3) + e+(5) + e-(6)
*
* Full colour ordering, overall factors included.
*
* The Z boson couples to the i1 and i4 pair of quarks
*

      function FullC0g1Z(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /order/iorder
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)


      fac = 0.25d0/nc**2
      fac = 4d0*fac*(nc**2-1d0)*(4d0*pi*as)**2*(4d0*pi*amz)**2
      fac = fac*nc*0.5d0*as/(2*pi)

      FullC0g1Z = C0g1Z(i1,i2,i3,i4,i5,i6,renscale2)
     .           +dble(nf)/nc*Ch0g1Z(i1,i2,i3,i4,i5,i6,renscale2)
     .           +1d0/nc*C0g1Ztop(i1,i2,i3,i4,i5,i6,renscale2)
     .           +(1d0/nc**2)*Ct0g1Z(i1,i2,i3,i4,i5,i6,renscale2)
     .           +(1d0/nc)*C0g1Zax(i1,i2,i3,i4,i5,i6,renscale2)


      if((i1.le.2).and.(i2.le.2).and.(nf1.eq.nf2))then ! qqb initial state identical flavour

         nup = 2
         ndo = 3

         nf2 = 2 ! Up type contribution

         FullC0g1Z = FullC0g1Z
     .              +nup*(C0g1Z(i1,i4,i3,i2,i5,i6,renscale2)
     .                   +dble(nf)/nc*Ch0g1Z(i1,i4,i3,i2,i5,i6,renscale2)
     .                   +1d0/nc*C0g1Ztop(i1,i4,i3,i2,i5,i6,renscale2)
     .                   +(1d0/nc**2)*Ct0g1Z(i1,i4,i3,i2,i5,i6,renscale2)
     .                   +(1d0/nc)*C0g1Zax(i1,i4,i3,i2,i5,i6,renscale2))

         nf2 = 1 ! Down type contribution

         FullC0g1Z = FullC0g1Z
     .              +ndo*(C0g1Z(i1,i4,i3,i2,i5,i6,renscale2)
     .                   +dble(nf)/nc*Ch0g1Z(i1,i4,i3,i2,i5,i6,renscale2)
     .                   +1d0/nc*C0g1Ztop(i1,i4,i3,i2,i5,i6,renscale2)
     .                   +(1d0/nc**2)*Ct0g1Z(i1,i4,i3,i2,i5,i6,renscale2)
     .                   +(1d0/nc)*C0g1Zax(i1,i4,i3,i2,i5,i6,renscale2))


         nf2 = nf1 ! reset

         FullC0g1Z = FullC0g1Z ! subleading colour
     .              +(1d0/nc)*(D0g1Z(i1,i2,i3,i4,i5,i6,renscale2)
     .              +dble(nf)/nc*Dh0g1Z(i1,i2,i3,i4,i5,i6,renscale2)
     .              +1d0/nc*D0g1Ztop(i1,i2,i3,i4,i5,i6,renscale2)
     .              +(1d0/nc**2)*Dt0g1Z(i1,i2,i3,i4,i5,i6,renscale2)
     .              +(1d0/nc)*D0g1Zax(i1,i2,i3,i4,i5,i6,renscale2))


      elseif(nf1.eq.nf2)then ! identical flavour, other initial state configurations
         if (((i1.le.2).and.(i3.le.2)).or.((i2.le.2).and.(i4.le.2)))then ! qbqb or qq initial state
            fac = 0.5d0*fac     ! statistical factor
         endif

         FullC0g1Z = FullC0g1Z
     .              +C0g1Z(i1,i4,i3,i2,i5,i6,renscale2)
     .              +dble(nf)/nc*Ch0g1Z(i1,i4,i3,i2,i5,i6,renscale2)
     .              +1d0/nc*C0g1Ztop(i1,i4,i3,i2,i5,i6,renscale2)
     .              +(1d0/nc**2)*Ct0g1Z(i1,i4,i3,i2,i5,i6,renscale2)
     .              +(1d0/nc)*C0g1Zax(i1,i4,i3,i2,i5,i6,renscale2)
     .              +(1d0/nc)*(D0g1Z(i1,i2,i3,i4,i5,i6,renscale2)
     .              +dble(nf)/nc*Dh0g1Z(i1,i2,i3,i4,i5,i6,renscale2)
     .              +1d0/nc*D0g1Ztop(i1,i2,i3,i4,i5,i6,renscale2)
     .              +(1d0/nc**2)*Dt0g1Z(i1,i2,i3,i4,i5,i6,renscale2)
     .              +(1d0/nc)*D0g1Zax(i1,i2,i3,i4,i5,i6,renscale2))


      endif

      FullC0g1Z = 2d0*fac*FullC0g1Z

      return
      end
c$$$
c$$$      function FullC0g1Z(i1,i2,i3,i4,i5,i6,renscale2)
c$$$      use KinData_mod
c$$$      implicit double precision (a-h,o-y)
c$$$      implicit complex*16 (z)
c$$$      parameter(pi=3.141592653589793238d0)
c$$$      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
c$$$!$omp threadprivate(/eweakZ/)
c$$$      common /koppel/ nf,nc,b0
c$$$      common /strong/ as
c$$$!$omp threadprivate(/strong/)
c$$$      common /order/iorder
c$$$      common /CZFlav/ nf1,nf2
c$$$!$omp threadprivate(/CZFlav/)
c$$$
c$$$      ! Things left to do:
c$$$      ! * need to include an identical factor, idfac, for when the two partons are of the same flavour and charge (qq/qbqb)
c$$$
c$$$      fac = 4d0*(nc**2-1)*0.25/nc**2*(4*pi*amz)**2*(4*pi*as)**2
c$$$      fac = fac*nc*0.5d0*as/(2*pi)
c$$$
c$$$      FullC0g1Z = 0d0
c$$$
c$$$      if((nf1.eq.0).or.(nf2.eq.0))then
c$$$         write(*,*) "Flavours in FullC0g1Z have not been initialised"
c$$$         stop
c$$$      endif
c$$$
c$$$      FullC0g1Z =+C0g1Z(i1,i2,i3,i4,i5,i6,renscale2)
c$$$     .           +(1d0/nc**2)*(Ct0g1Z(i1,i2,i3,i4,i5,i6,renscale2)
c$$$     .                        +Ct0g1Zax(i1,i2,i3,i4,i5,i6,renscale2))
c$$$     .           +(dble(nf)/nc)*Ch0g1Z(i1,i2,i3,i4,i5,i6,renscale2)
c$$$
c$$$     .           +(1d0/nc)*(D0g1Z(i1,i2,i3,i4,i5,i6,renscale2)
c$$$     .                     +D0g1Zax(i1,i2,i3,i4,i5,i6,renscale2)
c$$$     .           +C0g1Ztop(i1,i2,i3,i4,i5,i6,renscale2)
c$$$     .           +(1d0/nc)*D0g1Ztop(i1,i2,i3,i4,i5,i6,renscale2)
c$$$     .           +(1d0/nc**2)*Dt0g1Z(i1,i2,i3,i4,i5,i6,renscale2)
c$$$     .           +(dble(nf)/nc)*Dh0g1Z(i1,i2,i3,i4,i5,i6,renscale2))
c$$$
c$$$      if((i1.le.2).and.(i2.le.2).and.(nf1.eq.nf2))then ! same flavour initial state, add nf contribution
c$$$         nf2 = 2 ! set to up type quarks
c$$$         nu = 2 ! number of up quarks, fixed at nf = 5
c$$$         nd = 3 ! number of down quarks, fixed at nf = 5
c$$$
c$$$         FullC0g1Z=FullC0g1Z
c$$$     .            +dble(nu)*(C0g1Z(i1,i4,i3,i2,i5,i6,renscale2)
c$$$     .            +(1d0/nc**2)*Ct0g1Z(i1,i4,i3,i2,i5,i6,renscale2)
c$$$     .            +(dble(nf)/nc)*Ch0g1Z(i1,i4,i3,i2,i5,i6,renscale2)
c$$$     .            +(1d0/nc)*C0g1Ztop(i1,i4,i3,i2,i5,i6,renscale2)
c$$$     .            +(1d0/nc)*D0g1Zax(i1,i4,i3,i2,i5,i6,renscale2))
c$$$
c$$$         nf2 = 1 ! set to down type quarks
c$$$         FullC0g1Z=FullC0g1Z
c$$$     .            +dble(nd)*(C0g1Z(i1,i4,i3,i2,i5,i6,renscale2)
c$$$     .            +(1d0/nc**2)*Ct0g1Z(i1,i4,i3,i2,i5,i6,renscale2)
c$$$     .            +(dble(nf)/nc)*Ch0g1Z(i1,i4,i3,i2,i5,i6,renscale2)
c$$$     .            +(1d0/nc)*C0g1Ztop(i1,i4,i3,i2,i5,i6,renscale2)
c$$$     .            +(1d0/nc)*D0g1Zax(i1,i4,i3,i2,i5,i6,renscale2))
c$$$
c$$$         ! reset flavour
c$$$         nf2 = nf1
c$$$      endif
c$$$
c$$$      FullC0g1Z = 2d0*fac*FullC0g1Z
c$$$
c$$$      return
c$$$      end


****************************************************
* Returns the virtual one-loop matrix element for the process
*
* q(1) + qbar(4) -> Q(2) + Qbar(3) + e+(5) + e-(6)
*
* Ignoring overall factors, leading colour only
*
* The Z boson couples to the i1 and i4 pair of quarks
*

      function C0g1Z(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      use C0gZ
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /order/iorder
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      complex*16 zampqRbA(2,2),zampqRbB(2,2)
      complex*16 zampTreeA(2,2,2),zampTreeB(2,2,2)
      complex*16 zampLoopA(2,2,2),zampLoopB(2,2,2)
      complex*16 zampTree(2,2,2),zampLoop(2,2,2)

      C0g1Z = 0d0
      call eweakV2qq(nf1,nf2,s6(i5,i6))

      zampTreeA = zampC0g1ZTree(i1,i2,i3,i4,i5,i6)
      zampTreeB = zampC0g1ZTree(i2,i1,i4,i3,i5,i6)

      zampLoopA = zampC0g1ZLoop(i1,i3,i4,i2,i5,i6,renscale2,1)
      zampLoopB = zampC0g1ZLoop(i2,i4,i3,i1,i5,i6,renscale2,1)


      do ii=1,2
         do kk=1,2
            do jj=1,2

               if ((ii.eq.1).and.(kk.eq.1))then
                  i = 1
               elseif((ii.eq.1).and.(kk.eq.2))then
                  i = 2
               elseif((ii.eq.2).and.(kk.eq.1))then
                  i = 3
               else
                  i = 4
               endif

               if ((jj.eq.1).and.(kk.eq.1))then
                  j = 1
               elseif((jj.eq.1).and.(kk.eq.2))then
                  j = 2
               elseif((jj.eq.2).and.(kk.eq.1))then
                  j = 3
               else
                  j = 4
               endif

               zampTree(ii,jj,kk) = zewfac1(i)*zampTreeA(ii,jj,kk)
     .                             -zewfac2(j)*zampTreeB(3-jj,3-ii,kk)

               zampLoop(ii,jj,kk) = zewfac1(i)*zampLoopA(ii,jj,kk)
     .                             -zewfac2(j)*zampLoopB(3-jj,3-ii,kk)

               C0g1Z = C0g1Z + dble(zampTree(ii,jj,kk)*dconjg(zampLoop(ii,jj,kk)))

            enddo
         enddo
      enddo
c      write(6,*) '1:',iorder
c      do ii=1,2
c         do kk=1,2
c            do jj=1,2
c               write(6,*) ii,jj,kk,zampTree(ii,jj,kk),zampLoop(ii,jj,kk)
c            enddo
c         enddo
c      enddo

cccc  here
c      C0g1Z = dble(zampTree(1,1,1)*dconjg(zampLoop(1,1,1)))
cccc      
      if (iorder.eq.-1)then
         subuv = 11d0/3d0  ! renormalisation to the MS bar scheme
         C0g1Z = C0g1Z - subuv*C0g0Z(i1,i2,i3,i4,i5,i6)
      elseif (iorder.eq.0)then
!     The conversion from FDH -> HV is -2d0/3d0, see hep-ph/9708239v1 equation 6.5
         e0 = -pi**2/6d0 + (2d0/3d0)
         C0g1Z = C0g1Z - e0*C0g0Z(i1,i2,i3,i4,i5,i6)
      endif

      return
      end

****************************************************
* Returns the virtual one-loop matrix element for the process
*
* q(1) + qbar(4) -> Q(2) + Qbar(3) + e+(5) + e-(6)
*
* Ignoring overall factors, subleading colour.
*
* The Z boson couples to the i1 and i4 pair of quarks
*
      function Ct0g1Z(i1,i2,i3,i4,i5,i6,renscale2)
      use Process_mod, only : nPDFs_proc
      use KinData_mod
      use C0gZ
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /axial/zewax(2)
!$omp threadprivate(/axial/)
      common /order/iorder
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      complex*16 zampTreeA(2,2,2),zampTreeB(2,2,2)
      complex*16 zampLoopA(2,2,2),zampLoopB(2,2,2)
      complex*16 zampTree(2,2,2),zampLoop(2,2,2)

      Ct0g1Z = 0d0

      call eweakV2qq(nf1,nf2,s6(i5,i6))

      zampTreeA = zampC0g1ZTree(i1,i2,i3,i4,i5,i6)
      zampTreeB = zampC0g1ZTree(i2,i1,i4,i3,i5,i6)

!1/N^2 amplitudes in A6;1 (e.q 2.8) hep-ph/9610370v1
      zampLoopA =-2d0*zampC0g1ZLoop(i1,i2,i4,i3,i5,i6,renscale2,2)
     .           -2d0*zampC0g1ZLoop(i1,i3,i4,i2,i5,i6,renscale2,1)
     .           +zampC0g1ZLoopSL(i1,i3,i4,i2,i5,i6,renscale2)


      zampLoopB =-2d0*zampC0g1ZLoop(i2,i1,i3,i4,i5,i6,renscale2,2)
     .           -2d0*zampC0g1ZLoop(i2,i4,i3,i1,i5,i6,renscale2,1)
     .           +zampC0g1ZLoopSL(i2,i4,i3,i1,i5,i6,renscale2)

      do ii=1,2
         do kk=1,2
            do jj=1,2

               if ((ii.eq.1).and.(kk.eq.1))then
                  i = 1
               elseif((ii.eq.1).and.(kk.eq.2))then
                  i = 2
               elseif((ii.eq.2).and.(kk.eq.1))then
                  i = 3
               else
                  i = 4
               endif

               if ((jj.eq.1).and.(kk.eq.1))then
                  j = 1
               elseif((jj.eq.1).and.(kk.eq.2))then
                  j = 2
               elseif((jj.eq.2).and.(kk.eq.1))then
                  j = 3
               else
                  j = 4
               endif

               zampTree(ii,jj,kk) = zewfac1(i)*zampTreeA(ii,jj,kk)
     .                             -zewfac2(j)*zampTreeB(3-jj,3-ii,kk)
               zampLoop(ii,jj,kk) = zewfac1(i)*zampLoopA(ii,jj,kk)
     .                             -zewfac2(j)*zampLoopB(3-jj,3-ii,kk)

               Ct0g1Z = Ct0g1Z + dble(zampTree(ii,jj,kk)*dconjg(zampLoop(ii,jj,kk)))
            enddo
         enddo
      enddo
c      write(6,*) '2:',iorder
c      do ii=1,2
c         do kk=1,2
c            do jj=1,2
c               write(6,*) ii,jj,kk,zampTree(ii,jj,kk),zampLoop(ii,jj,kk)
c            enddo
c         enddo
c      enddo
cccc here
c      Ct0g1Z = dble(zampTree(1,1,1)*dconjg(zampLoop(1,1,1)))
      if (iorder.eq.0)then
!     The conversion from FDH -> HV is +1, see hep-ph/9708239v1 equation 6.5
         e0 = +pi**2/6d0 -1d0
         Ct0g1Z = Ct0g1Z - e0*C0g0Z(i1,i2,i3,i4,i5,i6)
      endif

      if(nPDFs_proc.eq.1) Ct0g1Z=-Ct0g1Z            !added - JN to be consistent with Ct1g0Z

      return
      end

****************************************************
* Returns the virtual one-loop matrix element for the process
*
* q(1) + qbar(4) -> Q(2) + Qbar(3) + e+(5) + e-(6)
*
* Ignoring overall factors, nf loop contribution leading colour
*
* The Z boson couples to the i1 and i4 pair of quarks
*
      function Ch0g1Z(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      use C0gZ
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      common /order/iorder
      complex*16 zampTreeA(2,2,2),zampTreeB(2,2,2)
      complex*16 zampTree(2,2,2),zampLoopnf(2,2,2)

      Ch0g1Z = 0d0

      zampTreeA = zampC0g1ZTree(i1,i2,i3,i4,i5,i6)
      zampTreeB = zampC0g1ZTree(i2,i1,i4,i3,i5,i6)

      call eweakV2qq(nf1,nf2,s6(i5,i6))

      if(iorder.eq.0)then

*          We must compute the tree level matrix element before the electroweak interference. The reason for this is that the factors that multiply the QCD amplitudes are scale dependent.

         zfacA = 2d0/3d0*zzLogRat(renscale2,-s6(i2,i3))+(10d0/9d0)
         zfacB = 2d0/3d0*zzLogRat(renscale2,-s6(i1,i4))+(10d0/9d0)

         do ii=1,2
            do kk=1,2
               do jj=1,2

                  if ((ii.eq.1).and.(kk.eq.1))then
                     i = 1
                  elseif((ii.eq.1).and.(kk.eq.2))then
                     i = 2
                  elseif((ii.eq.2).and.(kk.eq.1))then
                     i = 3
                  else
                     i = 4
                  endif

                  if ((jj.eq.1).and.(kk.eq.1))then
                     j = 1
                  elseif((jj.eq.1).and.(kk.eq.2))then
                     j = 2
                  elseif((jj.eq.2).and.(kk.eq.1))then
                     j = 3
                  else
                     j = 4
                  endif

                  zampTree(ii,jj,kk) = zewfac1(i)*zampTreeA(ii,jj,kk)
     .                                -zewfac2(j)*zampTreeB(3-jj,3-ii,kk)
                  zampLoopnf(ii,jj,kk) = zfacA*zewfac1(i)*zampTreeA(ii,jj,kk)
     .                                  -zfacB*zewfac2(j)*zampTreeB(3-jj,3-ii,kk)

                  Ch0g1Z = Ch0g1Z + dble(zampTree(ii,jj,kk)*dconjg(zampLoopnf(ii,jj,kk)))

               enddo
            enddo
         enddo
cccc here
c      Ch0g1Z = dble(zampTree(1,1,1)*dconjg(zampLoopnf(1,1,1)))

      endif
c      write(6,*) '3:',iorder
c      do ii=1,2
c         do kk=1,2
c            do jj=1,2
c               write(6,*) ii,jj,kk,zampTree(ii,jj,kk),zampLoopnf(ii,jj,kk)
c            enddo
c         enddo
c      enddo

!     TM: 23/09/2015 flipped sign of Ch0g1Z to match that of HJ convention

      Ch0g1Z = -Ch0g1Z


      return
      end

****************************************************
* Returns the virtual one-loop matrix element for the process
*
* q(1) + qbar(4) -> Q(2) + Qbar(3) + e+(5) + e-(6)
*
* Ignoring overall factors, finite top corrections
*
* The Z boson couples to the i1 and i4 pair of quarks
*
* Only valid in the large top mass region (low incoming s),
* as the corrections are expansions in sij/mt**2

      function C0g1Ztop(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      use Parameters_mod
      use C0gZ
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      double precision :: emt,etwidth,emb,ebwidth,emc,ecwidth,emtau,etauwidth
      common /Qmass/ emt,etwidth,emb,ebwidth,emc,ecwidth,emtau,etauwidth
      common /order/iorder
      complex*16 zampTreeA(2,2,2),zampTreeB(2,2,2)
      complex*16 zampTree(2,2,2),zampLoopnf(2,2,2)

      if(emt.eq.0d0)then
         write(*,*) "Mass of the top has not been initialised. This is
     . required for the C0g1Ztop function"
         stop
      endif

      C0g1Ztop = 0d0

      zampTreeA = zampC0g1ZTree(i1,i2,i3,i4,i5,i6)
      zampTreeB = zampC0g1ZTree(i2,i1,i4,i3,i5,i6)

      call eweakV2qq(nf1,nf2,s6(i5,i6))

      if(iorder.eq.0)then

*          We must compute the tree level matrix element before the electroweak interference. The reason for this is that the factors that multiply the QCD amplitudes are scale dependent.

*     We will have to keep an eye on this term, in limits with 1||3 and 2||4 we have a 0/0 factor with the tree level amplitude. If this is a problem, we can create a new amplitude function and cancel the invariants analytically.

         zfacA = zzselftop(s6(i2,i3))
         zfacB = zzselftop(s6(i1,i4))

         do ii=1,2
            do kk=1,2
               do jj=1,2

                  if ((ii.eq.1).and.(kk.eq.1))then
                     i = 1
                  elseif((ii.eq.1).and.(kk.eq.2))then
                     i = 2
                  elseif((ii.eq.2).and.(kk.eq.1))then
                     i = 3
                  else
                     i = 4
                  endif

                  if ((jj.eq.1).and.(kk.eq.1))then
                     j = 1
                  elseif((jj.eq.1).and.(kk.eq.2))then
                     j = 2
                  elseif((jj.eq.2).and.(kk.eq.1))then
                     j = 3
                  else
                     j = 4
                  endif

                  zampTree(ii,jj,kk) = zewfac1(i)*zampTreeA(ii,jj,kk)
     .                                -zewfac2(j)*zampTreeB(3-jj,3-ii,kk)
                  zampLoopnf(ii,jj,kk) = zfacA*zewfac1(i)*zampTreeA(ii,jj,kk)
     .                                  -zfacB*zewfac2(j)*zampTreeB(3-jj,3-ii,kk)

                  C0g1Ztop = C0g1Ztop + dble(zampTree(ii,jj,kk)*dconjg(zampLoopnf(ii,jj,kk)))

               enddo
            enddo
         enddo
      endif


      return
      end

****************************************************
* Returns the virtual one-loop matrix element for the process
*
* q(1) + qbar(4) -> Q(2) + Qbar(3) + e+(5) + e-(6)
*
* Ignoring overall factors, axial contribution
*
* The Z boson couples to the i1 and i4 pair of quarks
*
* Only valid in the large top mass region (low incoming s),
* as the corrections are expansions in sij/mt**2

      function C0g1Zax(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      use C0gZ
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /axial/zewax(2)
!$omp threadprivate(/axial/)
      common /order/iorder
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      complex*16 zampTree(2,2,2),zampTreeA(2,2,2),zampTreeB(2,2,2)
      complex*16 zampAx(2,2,2)

      C0g1Zax = 0d0

      zampAx =  zampC0g1ZaxA(i1,i2,i4,i3,i5,i6)

      zampTreeA = zampC0g1ZTree(i1,i2,i3,i4,i5,i6)
      zampTreeB = zampC0g1ZTree(i2,i1,i4,i3,i5,i6)

      call eweakV2qq(nf1,nf2,s6(i5,i6))
      if (iorder.eq.0)then
c         write(6,*) '1: '
      do ii=1,2
         do kk=1,2
            do jj=1,2

               if ((ii.eq.1).and.(kk.eq.1))then
                  i = 1
               elseif((ii.eq.1).and.(kk.eq.2))then
                  i = 2
               elseif((ii.eq.2).and.(kk.eq.1))then
                  i = 3
               else
                  i = 4
               endif

               if ((jj.eq.1).and.(kk.eq.1))then
                  j = 1
               elseif((jj.eq.1).and.(kk.eq.2))then
                  j = 2
               elseif((jj.eq.2).and.(kk.eq.1))then
                  j = 3
               else
                  j = 4
               endif

               zampTree(ii,jj,kk) = zewfac1(i)*zampTreeA(ii,jj,kk)
     .                             -zewfac2(j)*zampTreeB(3-jj,3-ii,kk)
c               zampTree(ii,jj,kk) = (-1)**(ii+jj)*zampTree(ii,jj,kk)  ! to adjust to tree-level sign convention
c               write(6,*) ii,jj,kk,zampTree(ii,jj,kk)
               zampAx(ii,jj,kk) = zewax(kk)*zampAx(ii,jj,kk)

               C0g1Zax = C0g1Zax + dble(zampTree(ii,jj,kk)*dconjg(zampAx(ii,jj,kk)))

            enddo
         enddo
      enddo
cccc here
c      C0g1Zax = dble(zampTree(1,1,1)*dconjg(zampAx(1,1,1)))
      endif
      return
      end

**********************************************
* A wrapper function for returning the normal
* matrix element for situations where either of the quarks
* in the second quark line are in the initial state.
* Otherwise it will symmetrise over the quark pair.
*

      function sC0g1Z(i1,i2,i3,i4,i5,i6,renscale2)
      use Process_mod, only : nPDFs_proc
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      !> For DIS we always symmetrise this ME/ Z exchange
      !> as we always have a secondary quark pair in the final state
      if((nPDFs_proc.ne.1.and.((i2.le.2).or.(i3.le.2))))then
         sC0g1Z=C0g1Z(i1,i2,i3,i4,i5,i6,renscale2)
      elseif(nPDFs_proc.eq.0) then
         sC0g1Z=0.25d0*(C0g1Z(i1,i2,i3,i4,i5,i6,renscale2)+C0g1Z(i1,i3,i2,i4,i5,i6,renscale2)+
     $    C0g1Z(i4,i2,i3,i1,i5,i6,renscale2)+C0g1Z(i4,i3,i2,i1,i5,i6,renscale2))
      else
         sC0g1Z=0.5d0*(C0g1Z(i1,i2,i3,i4,i5,i6,renscale2)+C0g1Z(i1,i3,i2,i4,i5,i6,renscale2))
      endif

      return
      end

