      module C0gZ

      contains

****************************************************
* Returns all the helicity amplitudes for a given set of momenta, taking into
* account all possible helicity combinations of the qqbar pairs.
*
*

      subroutine zampC0g0ZGen(i1,i2,i3,i4,i5,i6,zampC0g0ZA,zampC0g0ZB)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      complex*16 zampC0g0ZA(2,2),zampC0g0ZB(2,2)

c     q L, qbar L
      zampC0g0ZA(1,1) = zampC0g0Z(i1,i3,i2,i4,i5,i6)
      zampC0g0ZB(1,1) = zampC0g0Z(i2,i4,i1,i3,i5,i6)

c     q R, qbar R
      zampC0g0ZA(2,2) =-zampC0g0Z(i4,i2,i3,i1,i5,i6)
      zampC0g0ZB(2,2) =-zampC0g0Z(i3,i1,i4,i2,i5,i6)

c     q L, qbar R
      zampC0g0ZA(1,2) = zampC0g0Z(i1,i2,i3,i4,i5,i6)
      zampC0g0ZB(1,2) = zampC0g0Z(i3,i4,i1,i2,i5,i6)

c     q R. qbar L
      zampC0g0ZA(2,1) =-zampC0g0Z(i4,i3,i2,i1,i5,i6)
      zampC0g0ZB(2,1) =-zampC0g0Z(i2,i1,i4,i3,i5,i6)

      end subroutine


***********************************************************************


****************************************************
* Returns the specific helicity amplitude for a given set of momenta
* relevant for the process
*
* q(1) + qbar(2) -> Q(3) + Qbar(4) + e-(5) + e+(6)
*
*
*

      function zampC0g0Z(i1,i2,i3,i4,i5,i6)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /proc/ isub(7)

      zt2a(i1,i2,i3,i4) = zA6(i1,i2)*zB6(i2,i4)+zA6(i1,i3)*zB6(i3,i4)

      s123 = s6(i1,i2) + s6(i1,i3) + s6(i2,i3)
      s234 = s6(i2,i3) + s6(i2,i4) + s6(i3,i4)
      prop = s6(i2,i3)*s6(i5,i6)

      zb13 = zB6(i1,i3)
      za54 = zA6(i5,i4)
      za24 = zA6(i2,i4)
      zb61 = zB6(i6,i1)

      zampC0g0Z =+zb13*za54*zt2a(i2,i1,i3,i6)/(prop*s123)
     .           +za24*zb61*zt2a(i5,i2,i4,i3)/(prop*s234)

c      print *, abs(zb13),abs(za54),abs(za24),abs(zb61)

      return
      end function


******************************************************
* Returns a 2x2x2 array of tree level amplitudes to be used
* in C0g1Z
*
*

      function zampC0g1ZTree(i1,i2,i3,i4,i5,i6)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /proc/ isub(7)
      complex*16 zampC0g1ZTree(2,2,2)
      complex*16 zampqRbA(2,2),zampqRbB(2,2)

      call zampC0g0ZGen(i1,i3,i2,i4,i5,i6,zampqRbA,zampqRbB)

      zampC0g1ZTree(1,1,1) = zampqRbA(1,1)
      zampC0g1ZTree(1,2,1) =-zampqRbA(1,2)
      zampC0g1ZTree(1,1,2) = dconjg(zampqRbA(2,2)) ! Note the swapping
      zampC0g1ZTree(1,2,2) =-dconjg(zampqRbA(2,1))
      zampC0g1ZTree(2,1,1) = dconjg(zampC0g1ZTree(1,2,2))
      zampC0g1ZTree(2,2,1) = dconjg(zampC0g1ZTree(1,1,2))
      zampC0g1ZTree(2,1,2) = dconjg(zampC0g1ZTree(1,2,1))
      zampC0g1ZTree(2,2,2) = dconjg(zampC0g1ZTree(1,1,1))

      return
      end function

******************************************************
* Returns a 2x2x2 array of tree level amplitudes to be used
* in C0g1Z
*
*

      function zampC0g1ZLoop(i1,i2,i3,i4,i5,i6,renscale2,iconfig)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /proc/ isub(7)
      common /order/iorder
      complex*16 zampC0g1ZLoop(2,2,2),zampTree(2,2,2),zLoop(2,2,2)

      do ii=1,2
         do jj=1,2
            do kk=1,2
               zLoop(ii,jj,kk) = 0d0
            enddo
         enddo
      enddo

      if (iorder.eq.0)then
         if (iconfig.eq.1)then
            zLoop(1,1,1) =-zampC0g1Zpm(i1,i4,i2,i3,i5,i6)
            call swapzAB_kin(6)
            zLoop(1,2,1) = zampC0g1Zpp(i3,i2,i4,i1,i6,i5)
            call swapzAB_kin(6)

            zLoop(1,1,2) =-zampC0g1Zpm(i1,i4,i2,i3,i6,i5)

            call swapzAB_kin(6)
            zLoop(1,2,2) = zampC0g1Zpp(i3,i2,i4,i1,i5,i6)
            call swapzAB_kin(6)

            zLoop(2,1,1) = zampC0g1Zpp(i3,i2,i4,i1,i5,i6)

            call swapzAB_kin(6)
            zLoop(2,2,1) =-zampC0g1Zpm(i1,i4,i2,i3,i6,i5)
            call swapzAB_kin(6)

            zLoop(2,1,2) = zampC0g1Zpp(i3,i2,i4,i1,i6,i5)

            call swapzAB_kin(6)
            zLoop(2,2,2) =-zampC0g1Zpm(i1,i4,i2,i3,i5,i6)
            call swapzAB_kin(6)
         elseif(iconfig.eq.2)then ! swap pp<->pm
            zLoop(1,1,1) =-zampC0g1Zpp(i1,i4,i2,i3,i5,i6)
            call swapzAB_kin(6)
            zLoop(1,2,1) = zampC0g1Zpm(i3,i2,i4,i1,i6,i5)
            call swapzAB_kin(6)

            zLoop(1,1,2) =-zampC0g1Zpp(i1,i4,i2,i3,i6,i5)

            call swapzAB_kin(6)
            zLoop(1,2,2) = zampC0g1Zpm(i3,i2,i4,i1,i5,i6)
            call swapzAB_kin(6)

            zLoop(2,1,1) = zampC0g1Zpm(i3,i2,i4,i1,i5,i6)

            call swapzAB_kin(6)
            zLoop(2,2,1) =-zampC0g1Zpp(i1,i4,i2,i3,i6,i5)
            call swapzAB_kin(6)

            zLoop(2,1,2) = zampC0g1Zpm(i3,i2,i4,i1,i6,i5)

            call swapzAB_kin(6)
            zLoop(2,2,2) =-zampC0g1Zpp(i1,i4,i2,i3,i5,i6)
            call swapzAB_kin(6)
         else
            write(*,*) "Invalid configuration passed to zampC0g1ZLoop",iconfig
            stop
         endif
      endif

      zl14 = zzLogRat(renscale2,-s6(i1,i4))
      zl23 = zzLogRat(renscale2,-s6(i2,i3))
      zl24 = zzLogRat(renscale2,-s6(i2,i4))
      zl56 = zzLogRat(renscale2,-s6(i5,i6))

      if(iorder.eq.-2)then
         zIPoles = -2d0
      elseif(iorder.eq.-1)then
         zIPoles = (2d0/3d0)-zl14-zl23
      else
         zIPoles = 10d0/9d0 - 0.5d0*zl14**2-0.5d0*zl23**2+13d0/6d0*zl24
     .        -3d0/2d0*zl56
      endif


      if (iconfig.eq.1)then
         zampTree =  zampC0g1ZTree(i1,i4,i2,i3,i5,i6)
      elseif(iconfig.eq.2)then
         zampTree = -zampC0g1ZTree(i1,i2,i4,i3,i5,i6)
      endif

      zampC0g1ZLoop = (zIPoles*zampTree)+zLoop

      return
      end function

******************************************************
* Returns a 2x2x2 array of tree level amplitudes to be used
* in C0g1Z
*
*

      function zampC0g1ZLoopSL(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /proc/ isub(7)
      common /order/iorder
      complex*16 zampC0g1ZLoopSL(2,2,2),zampTree(2,2,2),zLoop(2,2,2)

      do ii=1,2
         do jj=1,2
            do kk=1,2
               zLoop(ii,jj,kk) = 0d0
            enddo
         enddo
      enddo

      if (iorder.eq.0)then
         call swapzAB_kin(6)
         zLoop(1,1,1) =  zampC0g1Zsl(i4,i2,i3,i1,i6,i5)
         call swapzAB_kin(6)

         call swapzAB_kin(6)
         zLoop(1,2,1) = -zampC0g1Zsl(i2,i4,i3,i1,i6,i5)
         call swapzAB_kin(6)

         call swapzAB_kin(6)
         zLoop(1,1,2) =  zampC0g1Zsl(i4,i2,i3,i1,i5,i6)
         call swapzAB_kin(6)

         call swapzAB_kin(6)
         zLoop(1,2,2) = -zampC0g1Zsl(i2,i4,i3,i1,i5,i6)
         call swapzAB_kin(6)

         zLoop(2,1,1) = -zampC0g1Zsl(i2,i4,i3,i1,i5,i6)
         zLoop(2,2,1) =  zampC0g1Zsl(i4,i2,i3,i1,i5,i6)
         zLoop(2,1,2) = -zampC0g1Zsl(i2,i4,i3,i1,i6,i5)
         zLoop(2,2,2) =  zampC0g1Zsl(i4,i2,i3,i1,i6,i5)
      endif

      zl24 = zzLogRat(renscale2,-s6(i2,i4))
      zl13 = zzLogRat(renscale2,-s6(i1,i3))

      if(iorder.eq.-2)then
         zIPoles = -2d0
      elseif(iorder.eq.-1)then
         zIPoles = -3d0-zl24-zl13
      else
         zIPoles = -15d0/2d0-0.5d0*zl24**2-0.5d0*zl13**2-(3d0/2d0)*zl24
     .        -(3d0/2d0)*zl13
      endif


      zampTree =  zampC0g1ZTree(i1,i4,i2,i3,i5,i6)
      zampC0g1ZLoopSL = -((zIPoles*zampTree)+zLoop)


      return
      end function


      function zampC0g1Zpm(i1,i2,i3,i4,i5,i6)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)

      t(i2,i3,i4) = s6(i2,i3) + s6(i2,i4) + s6(i3,i4)

      s56  = s6(i5,i6)
      za15 = zA6(i1,i5)
      za56 = zA6(i5,i6)
      zb13 = zB6(i1,i3)
      zb23 = zB6(i2,i3)
      za12 = zA6(i1,i2)
      zb24 = zB6(i2,i4)
      za13 = zA6(i1,i3)
      zb34 = zB6(i3,i4)
      za25 = zA6(i2,i5)
      za45 = zA6(i4,i5)
      za23 = zA6(i2,i3)
      za24 = zA6(i2,i4)
      zb16 = zB6(i1,i6)
      zb36 = zB6(i3,i6)
      zb46 = zB6(i4,i6)
      zb56 = zB6(i5,i6)
      s12  = s6(i1,i2)
      s23  = s6(i2,i3)
      zb12 = zB6(i1,i2)
      za34 = zA6(i3,i4)
      zb26 = zB6(i2,i6)
      s34  = s6(i3,i4)
      zb14 = zB6(i1,i4)
      za35 = zA6(i3,i5)
      za14 = zA6(i1,i4)

      zampC0g1Zpm = 0d0

      zs3 = -2d0*zL0d(-t(i2,i3,i4),-s56)/s56*za15/za56*zb13
     ./zb23/(za12*zb24+za13*zb34)*(-za25
     .*zb23+za45*zb34)-2*zL0d(-t(i1,i2,i3),-s56)/
     .s56/za23*za24/(za12*zb24+za13*zb34)*(-za12*zb16
     .+za23*zb36)*zb46/zb56
      zs4 = zs3+zLsm1(-s12,-t(i1,i2,i3),-s23,-t(i1,i2,i3))
     .*(-za45**2/za56*zb13**2/(za24*zb12
     .+za34*zb13)/zb23/t(i1,i2,i3)+za12**2/za13**2
     ./za23*(-za13*zb16-za23*zb26)**2/(za12*zb24
     .+za13*zb34)/zb56/t(i1,i2,i3))
      zs2 = zs4+zLsm1_2mh(s34,t(i1,i2,i3),s12,s56)*(-za45**2
     ./za56*zb13**2/(za24*zb12+za34*zb13)
     ./zb23/t(i1,i2,i3)+1d0/za23/(-za13*zb14-za23*
     .zb24)**2*(-za13*zb16-za23*zb26)**2/(za12*zb24
     .+za13*zb34)*(-za12*zb14+za23*zb34)**2
     ./zb56/t(i1,i2,i3))+zL0d(-t(i1,i2,i3),-s12)/s12*za12/za13
     .*zb13/(-za13*zb14-
     .za23*zb24)*(-za13*zb16-za23*zb26)**2
     ./zb56/t(i1,i2,i3)
      zs3 = zs2+zL0d(-t(i1,i2,i3),-s23)/s23*za12/za13*zb13*
     .(-za13*zb16-za23*zb26)/(za12*zb24+za13*
     .zb34)*(za12*zb26+za13*zb36)/zb56/t(i1,i2,i3)
     .-zL1d(-t(i1,i2,i3),-s23)/s23**2*za23*zb13**2/(za12
     .*zb24+za13*zb34)*(za12*zb26+za13*zb36)**2
     ./zb56/t(i1,i2,i3)/2d0
      zs4 = zs3+zL0d(-t(i1,i2,i3),-s23)/s23*zb13/(za12*zb24
     .+za13*zb34)*(za12*zb26+za13*zb36)
     .*(-za12*zb16+za23*zb36)/zb56/t(i1,i2,i3)
      zs5 = zs4-zL1d(-s56,-t(i1,i2,i3))/za23*za24**2/(za12
     .*zb24+za13*zb34)*zb46**2/zb56/t(i1,i2,i3)/2d0
      zs6 = zs5
      zs8 = zL0d(-t(i1,i2,i3),-s56)/s56*za24/(-za13*zb14
     .-za23*zb24)/(za12*zb24+za13*zb34)*zb46**2
     ./zb56*t(i1,i2,i3)
      zs10 = zI3m(s12,s34,s56)
      zs12 = -1d0/(-za13*zb14-za23*zb24)/(za12*zb24
     .+za13*zb34)*(za23*zb34*(-za35*zb36-za45
     .*zb46)-za12*zb14*(-za35*zb36+za45*zb46))/2d0
     .-2d0*za12*za45*zb13/(za12*zb24+za13*zb34)
     .*zb46/t(i1,i2,i3)
      zs13 = zs12
      zs15 = (za45**2/za56*zb13**2/(za24*zb12+za34
     .*zb13)/zb23-1d0/za23/(za12*zb24+za13*zb34)
     .*(-za12*zb16+za23*zb36)**2/zb56)/t(i1,i2,i3)**2
     .*(2d0*s12*s56+(-s12+s34-s56)*t(i1,i2,i3))/2d0
      zs16 = (-s12-s34+s56)/(s12**2-2d0*s12*s34
     .+s34**2-2d0*s12*s56-2d0*s34*s56+s56**2)
     ./(-za13*zb14-za23*zb24)*(-za15*zb16
     .-za25*zb26)/(za12*zb24+za13*zb34)*(-
     .s56*za12*zb14-(-za12*zb14+za23*zb34)*t(i1,i2,i3))/2d0
      zs14 = zs15+zs16
      zs11 = zs13+zs14
      zs9 = zs10*zs11
      zs7 = zs8+zs9
      zs1 = zs6+zs7
      zs4 = zs1
      zs6 = zzlogRat(-s12,-s56)/(s12**2-2d0*s12*s34+s34**2
     .-2d0*s12*s56-2d0*s34*s56+s56**2)/(-
     .za13*zb14-za23*zb24)*(2d0*s12*(-za25*zb16
     .+za15*(-za23*zb13-za24*zb14)/(-za13
     .*zb23-za14*zb24)*zb26)+za12*(zb16**2
     .+(-za23*zb13-za24*zb14)/(-za13*zb23
     .-za14*zb24)*zb26**2)/zb56*t(i1,i2,i3)+1/
     .za56*zb12*(za25**2+za15**2*(-za23*zb13
     .-za24*zb14)/(-za13*zb23-za14*zb24))*t(i1,i2,i4))
      zs7 = -zLsm1(-s34,-t(i2,i3,i4),-s23,-t(i2,i3,i4))*(1/za56
     ./zb23/zb24**2*(za35*zb23+za45*zb24)**2
     .*zb34**2/(za12*zb24+za13*zb34)/t(i2,i3,i4)
     .-1d0/za23*za24**2/(za24*zb12+za34*zb13)*
     .zb16**2/zb56/t(i2,i3,i4))
      zs5 = zs6+zs7
      zs3 = zs4+zs5
      zs4 = zs3-zLsm1_2mh(s12,t(i2,i3,i4),s34,s56)*(1d0/za56
     ./zb23/(-za13*zb23-za14*zb24)**2*(za35
     .*zb23+za45*zb24)**2/(za12*zb24+za13
     .*zb34)*(za12*zb23-za14*zb34)**2/t(i2,i3,i4)
     .-1/za23*za24**2/(za24*zb12+za34*zb13)*zb16**2
     ./zb56/t(i2,i3,i4))
      zs2 = zs4-zL0d(-t(i2,i3,i4),-s34)/s34*za24/za56/zb24
     ./(-za13*zb23-za14*zb24)*(za35*zb23
     .+za45*zb24)**2*zb34/t(i2,i3,i4)+zL1d(-s56,-t(i2,i3,i4))
     .*za15**2/za56*zb13**2/zb23/(za12*zb24+
     .za13*zb34)/t(i2,i3,i4)/2
      zs3 = zs2-zL0d(-t(i2,i3,i4),-s23)/s23*za24/za56/zb24
     .*(za35*zb23+za45*zb24)*zb34/(za12*zb24
     .+za13*zb34)*(-za25*zb24-za35*zb34)
     ./t(i2,i3,i4)+zL1d(-t(i2,i3,i4),-s23)/s23**2*za24**2/za56
     .*zb23/(za12*zb24+za13*zb34)*
     .(-za25*zb24-za35*zb34)**2/t(i2,i3,i4)/2d0
      zs4 = zs3-zL0d(-t(i2,i3,i4),-s23)/s23*za24/za56/(za12
     .*zb24+za13*zb34)*(-za25*zb24-za35*zb34)
     .*(-za25*zb23+za45*zb34)/t(i2,i3,i4)
      zs5 = zs4-zL0d(-t(i2,i3,i4),-s56)/s56*za15**2/za56
     .*zb13/(-za13*zb23-za14*zb24)/(za12*zb24
     .+za13*zb34)*t(i2,i3,i4)
      zs6 = zs5
      zs8 = -zzlogRat(-s34,-s56)/(s12**2-2*s12*s34+
     .s34**2-2*s12*s56-2*s34*s56+s56**2)/(
     .-za13*zb23-za14*zb24)*(2d0*s34*(-za45*
     .zb36+za35*(-za14*zb13-za24*zb23)/(-za13
     .*zb14-za23*zb24)*zb46)+za34*(zb36**2
     .+(-za14*zb13-za24*zb23)/(-za13*zb14-za23
     .*zb24)*zb46**2)/zb56*t(i1,i3,i4)+1d0
     ./za56*(za45**2+za35**2*(-za14*zb13-za24*zb23)
     ./(-za13*zb14-za23*zb24))*zb34*t(i2,i3,i4))
      zs10 = -dcmplx(1d0,0d0)
      zs12 = zI3m(s34,s12,s56)
      zs14 = -1d0/(-za13*zb23-za14*zb24)/(za12*zb24+za13
     .*zb34)*(za12*zb23*(-za15*zb16-za25*zb26)
     .-za14*(za15*zb16-za25*zb26)*zb34)/2d0
     .-2d0*za15*za24*zb16*zb34
     ./(za12*zb24+za13*zb34)/t(i2,i3,i4)
      zs15 = zs14
      zs17 = (-1d0/za56/zb23/(za12*zb24+za13*zb34)*(-za25
     .*zb23+za45*zb34)**2+1d0/za23*za24**2/(za24
     .*zb12+za34*zb13)*zb16**2/
     .zb56)/t(i2,i3,i4)**2*(2*s34*s56+(s12-s34-
     .s56)*t(i2,i3,i4))/2d0
      zs18 = (-s12-s34+s56)/(s12**2-2d0*s12*s34+s34**2
     .-2d0*s12*s56-2d0*s34*s56+s56**2)/(-za13*zb23
     .-za14*zb24)/(za12*zb24+
     .za13*zb34)*(-za35*zb36-za45*zb46)*(-
     .s56*za14*zb34-(za12*zb23-za14*zb34)*t(i2,i3,i4))/2
      zs16 = zs17+zs18
      zs13 = zs15+zs16
      zs11 = zs12*zs13
      zs9 = zs10*zs11
      zs7 = zs8+zs9
      zampC0g1Zpm=zs6+zs7

      return
      end function

      function zampC0g1Zpp(i1,i2,i3,i4,i5,i6)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)

      t(j2,j3,j4) = s6(j2,j3) + s6(j2,j4) + s6(j3,j4)

      za13 = zA6(i1,i3)
      zb16 = zB6(i1,i6)
      za23 = zA6(i2,i3)
      zb26 = zB6(i2,i6)
      za12 = zA6(i1,i2)
      zb24 = zB6(i2,i4)
      zb34 = zB6(i3,i4)
      zb56 = zB6(i5,i6)
      s23  = s6(i2,i3)
      zb12 = zB6(i1,i2)
      zb36 = zB6(i3,i6)
      s56  = s6(i5,i6)
      za34 = zA6(i3,i4)
      zb46 = zB6(i4,i6)
      s12  = s6(i1,i2)
      s34  = s6(i3,i4)
      za45 = zA6(i4,i5)
      za56 = zA6(i5,i6)
      za24 = zA6(i2,i4)
      zb13 = zB6(i1,i3)
      zb23 = zB6(i2,i3)
      za35 = zA6(i3,i5)
      za25 = zA6(i2,i5)
      za15 = zA6(i1,i5)

      zs2 = -2d0*(-za13*zb16-za23*zb26)/(za12*zb24
     .+za13*zb34)/zb56*(zL0d(-s23,-t(i1,i2,i3))*zb12*
     .(za12*zb26+za13*zb36)/t(i1,i2,i3)**2+zL0d(-s56,-t(i1,i2,i3))
     ./za23*za34*zb46/t(i1,i2,i3))

      zs3 = (zLsm1(-s12,-t(i1,i2,i3),-s23,-t(i1,i2,i3))+zLsm1_2mht(s34,t(i1,i2,i3),s12,s56))
     .*(-za45**2/za56*zb12**2/(za24*zb12+za34*zb13)/zb23
     ./t(i1,i2,i3)+1d0/za23*(-za13*zb16-za23*zb26)**2
     ./(za12*zb24+za13*zb34)/zb56/t(i1,i2,i3))-1d0/za23/(za12
     .*zb24+za13*zb34)*(zL1d(-t(i1,i2,i3),-s23)/s23**2*
     .za23**2*zb12**2*(za12*zb26+za13*zb36)**2
     .+zL1d(-s56,-t(i1,i2,i3))*za34**2*zb46**2)/zb56/t(i1,i2,i3)/2
      zs1 = zs2+zs3
      zs2 = zs1-2d0/za56*(za35*zb23+za45*zb24)/(za12*zb24
     .+za13*zb34)*(-zL0d(-s23,-t(i2,i3,i4))*za34*(-za25*zb24
     .-za35*zb34)/t(i2,i3,i4)**2+zL0d(-s56,-t(i2,i3,i4))*za15*
     .zb12/zb23/t(i2,i3,i4))
      zt0 = zs2-(zLsm1(-s34,-t(i2,i3,i4),-s23,-t(i2,i3,i4))
     .+zLsm1_2mht(s12,t(i2,i3,i4),s34,s56))*(1d0/za56/zb23
     .*(za35*zb23+za45*zb24)**2/(za12*zb24+za13
     .*zb34)/t(i2,i3,i4)-1/za23*za34**2/(za24*zb12+za34
     .*zb13)*zb16**2/zb56/t(i2,i3,i4))+1d0/za56/zb23
     ./(za12*zb24+za13*zb34)*(zL1d(-s56,-t(i2,i3,i4))
     .*za15**2*zb12**2+zL1d(-t(i2,i3,i4),-s23)/s23**2
     .*za34**2*zb23**2*(-za25*zb24-za35*zb34)**2)
     ./t(i2,i3,i4)/2d0

      zampC0g1Zpp = zt0

      return
      end function

      function zampC0g1Zsl(i1,i2,i3,i4,i5,i6)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)

      t(j2,j3,j4) = s6(j2,j3) + s6(j2,j4) + s6(j3,j4)

      s34  = s6(i3,i4)
      s12  = s6(i1,i2)
      s56  = s6(i5,i6)
      za45 = zA6(i4,i5)
      za56 = zA6(i5,i6)
      zb12 = zB6(i1,i2)
      zb13 = zB6(i1,i3)
      za14 = zA6(i1,i4)
      za24 = zA6(i2,i4)
      zb23 = zB6(i2,i3)
      za12 = zA6(i1,i2)
      za13 = zA6(i1,i3)
      zb14 = zB6(i1,i4)
      za23 = zA6(i2,i3)
      zb24 = zB6(i2,i4)
      zb16 = zB6(i1,i6)
      zb26 = zB6(i2,i6)
      zb34 = zB6(i3,i4)
      zb56 = zB6(i5,i6)
      zb36 = zB6(i3,i6)
      zb46 = zB6(i4,i6)
      za25 = zA6(i2,i5)
      za35 = zA6(i3,i5)
      zb45 = zB6(i4,i5)
      za36 = zA6(i3,i6)
      zb15 = zB6(i1,i5)
      zb35 = zB6(i3,i5)
      za46 = zA6(i4,i6)

      zs4 = zLsm1_2mh(s34,t(i1,i2,i3),s12,s56)*(za45**2/za56
     ./zb12*zb13**2/(-za14*zb13-za24*zb23)/t(i1,i2,i3)
     .-1/za12/(-za13*zb14-za23*zb24)**3*(-za13
     .*zb16-za23*zb26)**2*(-za12*zb14+za23
     .*zb34)**2/zb56/t(i1,i2,i3))-3.D0/4.D0*(zzlogRat(-s34,-s56)
     .+zzlogRat(-t(i1,i2,i3),-s56))/za12/(-za13*zb14-za23
     .*zb24)*(-za12*zb16+za23*zb36)**2/zb56/t(i1,i2,i3)

      zs3 = zs4-zL0d(-t(i1,i2,i3),-s56)/s56/za12*za23*za24/(-za13
     .*zb14-za23*zb24)**2*zb46**2/zb56*t(i1,i2,i3)
     .+zL1d(-s56,-t(i1,i2,i3))/za12*za24**2/(-za13*zb14
     .-za23*zb24)*zb46**2/zb56/t(i1,i2,i3)/2

      zs4 = zs3+(zzlogRat(-s34,-s56)+zzlogRat(-t(i1,i2,i3),-s56))
     ./za12*za23/(-za13*zb14-za23*zb24)**2
     .*(-za12*zb16+za23*zb36)*zb46/zb56/2


      zs2 = zs4+2*zL0d(-t(i1,i2,i3),-s56)/s56/za12*za24/
     .(-za13*zb14-za23*zb24)*(-za12*zb16+za23*zb36)
     .*zb46/zb56+3.D0/4.D0*(zzlogRat(-s34,-s56)+zzlogRat(-t(i1,i2,i4),-s56))
     ./za56/zb12*(za25*zb12+za45*zb14)**2/(-za13
     .*zb14-za23*zb24)/t(i1,i2,i4)-(zzlogRat(-s34,-s56)
     .+zzlogRat(-t(i1,i2,i4),-s56))*za35/za56/zb12*zb14*(za25*zb12
     .+za45*zb14)/(-za13*zb14-za23*zb24)**2/2
      zs3 = zs2+zL0d(-t(i1,i2,i4),-s56)/s56*za35**2/za56
     ./zb12*zb13*zb14/(-za13*zb14-za23*zb24)**2*
     .t(i1,i2,i4)-zL1d(-s56,-t(i1,i2,i4))*za35**2/za56/zb12*zb13**2
     ./(-za13*zb14-za23*zb24)/t(i1,i2,i4)/2-2*
     .zL0d(-t(i1,i2,i4),-s56)/s56*za35/za56/zb12*zb13
     .*(za25*zb12+za45*zb14)/(-za13*zb14-za23*zb24)

      zs5 = zs3
      zs8 = zI3m(s12,s34,s56)
      zs10 = za23*(za25*zb12+za35*zb13)/(-za13*zb14-za23*zb24)**2
     .*zb46+(-s12+s34-s56)/(s12**2-2*s12*s34
     .+s34**2-2*s12*s56-2*s34*s56+s56**2)*
     .za23*((s12-s34-s56)*za25*zb12+(-s12
     .-s34+s56)*za56*zb16)/(-za13*zb14-za23*zb24)*
     .*2*zb46-1/(s12**2-2*s12*s34+s34**2-2*s12*s56-
     .2*s34*s56+s56**2)*za56*zb12/(-za13*zb14
     .-za23*zb24)*(za23*zb36+za24*zb46)**2
      zs11 = zs10-2*za45*zb13/(-za13*zb14-za23*zb24)*(-za13*zb16
     .-za23*zb26)*(-za12*zb14+za23*zb34)/t(i1,i2,i3)**2
      zs9 = zs11+s34/(s12**2-2*s12*s34+s34**2-2*s12*s56
     .-2*s34*s56+s56**2)*za25*zb16/(-za13*zb14
     .-za23*zb24)*(t(i1,i2,i3)-t(i1,i2,i4))/2+(-s12**2+2*s12
     .*s34-s34**2+3*s34*(-s12+s34-s56)+2*s12
     .*s56+2*s34*s56-s56**2)/(s12**2-2*s12*
     .s34+s34**2-2*s12*s56-2*s34*s56+s56**2)**2
     .*(-za23*zb13-za24*zb14)/(-za13*zb14-za23*zb24)*(-za35
     .*zb36-za45*zb46)*(t(i1,i2,i3)-t(i1,i2,i4))/2
      zs7 = zs8*zs9
      zs9 = -dcmplx(1d0,0d0)
      zs11 = zzlogRat(-s12,-s34)
      zs13 = -1/(s12**2-2*s12*s34+s34**2-2*s12*s56-2*
     .s34*s56+s56**2)*za12*(-za35*zb13+za45
     .*zb14)*zb16/(-za13*zb14-za23*zb24)+
     .3.D0/2.D0*(-s12-s34+s56)/(s12**2-2*s12*s34
     .+s34**2-2*s12*s56-2*s34*s56+s56**2)**2
     .*(-za23*zb13-za24*zb14)
     ./(-za13*zb14-za23*zb24)*(-za35*zb36-
     .za45*zb46)*(-t(i1,i2,i3)+t(i1,i2,i4))
      zs12 = zs13-1/(s12**2-2*s12*s34+s34**2-2*s12*s56
     .-2*s34*s56+s56**2)*za25/za56/(-
     .za13*zb14-za23*zb24)*(-za35*zb13*t(i1,i2,i3)+za45
     .*zb14*t(i1,i2,i4)+(-s12+s34-s56)*za35*zb14
     ./(-za13*zb14-za23*zb24)*t(i1,i2,i4))+1/(s12**2
     .-2*s12*s34+s34**2-2*s12*s56-2*s34*s56
     .+s56**2)*za12*za35*zb14/(-za13*zb14
     .-za23*zb24)**2*(-2*za25*zb12*zb56+zb16*(-t(i1,i2,i3)+t(i1,i2,i4)))
      zs10 = zs11*zs12
      zs8 = zs9*zs10
      zs6 = zs7+zs8
      zs4 = zs5+zs6
      zs5 = zs4+1/(s12**2-2*s12*s34+s34**2-2*s12*
     .s56-2*s34*s56+s56**2)/za12*za25/za56*((-s12
     .+s34-s56)*za25-2*za12*za56*zb16)/(-za13
     .*zb14-za23*zb24)*(-t(i1,i2,i3)+t(i1,i2,i4))/4
      zs6 = zs5
      zs9 = zzlogRat(-s12,-s34)
      zs11 = 1/(s12**2-2*s12*s34+s34**2-2*s12*s56-2*s34*
     .s56+s56**2)*za25*zb12/(-za13*zb14-za23
     .*zb24)*(za23*zb36-za24*zb46)-1/(s12**2-2*s12
     .*s34+s34**2-2*s12*s56-2*s34*s56+s56**2)
     .*za23*zb12/(-za13*zb14-za23*zb24)**2*zb46
     .*(-2*za12*za56*zb16+za25*(t(i1,i2,i3)-t(i1,i2,i4)))
      zs12 = zs11
      zs14 = 3.D0/2.D0*(-s12-s34+s56)/(s12**2-2*s12*s34+s34**2
     .-2*s12*s56-2*s34*s56+s56**2)**2*(-za23*zb13
     .-za24*zb14)/(-za13*zb14-za23*zb24)*(-za35
     .*zb36-za45*zb46)*(t(i1,i2,i3)-t(i1,i2,i4))
      zs15 = 1/(s12**2-2*s12*s34+s34**2-2*s12*s56-2*s34
     .*s56+s56**2)*zb16/(-za13*zb14-za23*zb24)
     ./zb56*(za23*zb36*t(i1,i2,i3)+(-s12+s34-s56)*za23
     ./(-za13*zb14-za23*zb24)*zb46*t(i1,i2,i3)-za24*zb46*t(i1,i2,i4))
      zs13 = zs14+zs15
      zs10 = zs12+zs13
      zs8 = zs9*zs10
      zs9 = -1/(s12**2-2*s12*s34+s34**2-2*s12*s56
     .-2*s34*s56+s56**2)/zb12*zb16/(-za13*zb14
     .-za23*zb24)/zb56*((-s12+s34-s56)*zb16
     .-2*za25*zb12*zb56)*(t(i1,i2,i3)-t(i1,i2,i4))/4
      zs7 = zs8+zs9
      zs1 = zs6+zs7
      zs3 = zs1-zL0d(-t(i3,i5,i6),-s12)/s12*za35*za45/za56/zb12
     .*zb14**2/(-za35*zb45-za36*zb46)**2*t(i3,i5,i6)
     .+zL1d(-s12,-t(i3,i5,i6))*za45**2/za56/zb12*zb14**2
     ./(-za35*zb45-za36*zb46)/t(i3,i5,i6)/2-3.D0/4.D0*(zzlogRat(-s34,-s12)
     .+zzlogRat(-t(i3,i5,i6),-s12))/za56/zb12*
     .(za35*zb13-za56*zb16)**2/(-za35*zb45-
     .za36*zb46)/t(i3,i5,i6)
      zs4 = zs3+(zzlogRat(-s34,-s12)+zzlogRat(-t(i3,i5,i6),-s12))
     .*za35/za56/zb12*zb14*(za35*zb13-za56*zb16)/(-za35
     .*zb45-za36*zb46)**2/2
      zs5 = zs4+2*zL0d(-t(i3,i5,i6),-s12)/s12*za45/za56/zb12*zb14*
     .(za35*zb13-za56*zb16)/(-za35*zb45-za36*zb46)
      zs2 = zs5+zLsm1_2mh(s34,t(i3,i5,i6),s56,s12)*(-1/za56/zb12*(-za35
     .*zb15-za36*zb16)**2/(-za35*zb45-za36*zb46)**3
     .*(-za35*zb34-za56*zb46)**2/t(i3,i5,i6)+
     .1/za12*za24**2*zb36**2/(-za45*zb35-za46
     .*zb36)/zb56/t(i3,i5,i6))+3.D0/4.D0*(zzlogRat(-s34,-s12)
     .+zzlogRat(-t(i4,i5,i6),-s12))/za12/(-za35*zb45-za36
     .*zb46)/zb56*(za24*zb46+za25*zb56)**2/t(i4,i5,i6)
      zs4 = zs2-(zzlogRat(-s34,-s12)+zzlogRat(-t(i4,i5,i6),-s12))
     ./za12*za23*zb46/(-za35*zb45-za36*zb46)**2/
     .zb56*(za24*zb46+za25*zb56)/2
      zs3 = zs4+zL0d(-t(i4,i5,i6),-s12)/s12/za12*za23**2*zb36*zb46
     ./(-za35*zb45-za36*zb46)**2/zb56*t(i4,i5,i6)-zL1d(-s12,-t(i4,i5,i6))
     ./za12*za23**2*zb36**2/(-za35*zb45-za36*zb46)
     ./zb56/t(i4,i5,i6)/2-2*zL0d(-t(i4,i5,i6),-s12)/s12/za12*za23
     .*zb36/(-za35*zb45-za36*zb46)/zb56*(za24
     .*zb46+za25*zb56)
      zs5 = zs3
      zs8 = -dcmplx(1d0,0d0)
      zs10 = zzlogRat(-s56,-s34)
      zs12 = -1/(s12**2-2*s12*s34+s34**2-2*s12*s56
     .-2*s34*s56+s56**2)*za56*zb16*(-za23*zb36
     .+za24*zb46)/(-za35*zb45-za36*zb46)+3.D0/2.D0*
     .(s12-s34-s56)/(s12**2-2*s12*s34+s34**2
     .-2*s12*s56-2*s34*s56+s56**2)**2*(-za23
     .*zb13-za24*zb14)/(-za35*zb45-za36*zb46)
     .*(-za35*zb36-za45*zb46)*(-t(i3,i5,i6)+t(i4,i5,i6))
      zs11 = zs12-1/(s12**2-2*s12*s34+s34**2-2*s12*s56
     .-2*s34*s56+s56**2)/za12*za25/(-za35*zb45
     .-za36*zb46)*(-za23*zb36*t(i3,i5,i6)+za24*zb46
     .*t(i4,i5,i6)+(-s12+s34-s56)*za23*zb46/(-za35
     .*zb45-za36*zb46)*t(i4,i5,i6))-1/(s12**2-2*s12
     .*s34+s34**2-2*s12*s56-2*s34*s56+s56**2)
     .*za23*za56*zb46/(-za35*zb45-za36*zb46)**2
     .*(2*za25*zb12*zb56-zb16*(-t(i3,i5,i6)+t(i4,i5,i6)))
      zs9 = zs10*zs11
      zs7 = zs8*zs9
      zs9 = zzlogRat(-s56,-s34)
      zs11 = 1/(s12**2-2*s12*s34+s34**2-2*s12*s56-2*s34*
     .s56+s56**2)*za25*(za35*zb13-za45*zb14)
     ./(-za35*zb45-za36*zb46)*zb56+1/(s12**2
     .-2*s12*s34+s34**2-2*s12*s56-2*s34*s56
     .+s56**2)*za35*zb14/(-za35*zb45-za36*zb46)**2
     .*zb56*(2*za12*za56*zb16-za25*(t(i3,i5,i6)-t(i4,i5,i6)))
      zs12 = zs11
      zs14 = 3.D0/2.D0*(s12-s34-s56)/(s12**2-2*s12*s34
     .+s34**2-2*s12*s56-2*s34*s56+s56**2)**2*
     .(-za23*zb13-za24*zb14)/(-za35*zb45-za36*zb46)
     .*(-za35*zb36-za45*zb46)*(t(i3,i5,i6)-t(i4,i5,i6))
      zs15 = 1/(s12**2-2*s12*s34+s34**2-2*s12*s56-
     .2*s34*s56+s56**2)/zb12*zb16/(-za35*zb45
     .-za36*zb46)*(za35*zb13*t(i3,i5,i6)+(-s12+s34
     .-s56)*za35*zb14/(-za35*zb45-za36*zb46)*
     .t(i3,i5,i6)-za45*zb14*t(i4,i5,i6))
      zs13 = zs14+zs15
      zs10 = zs12+zs13
      zs8 = zs9*zs10
      zs6 = zs7+zs8
      zs4 = zs5+zs6
      zs6 = zs4
      zs8 = zI3m(s56,s34,s12)
      zs10 = -1/(s12**2-2*s12*s34+s34**2-2*s12*s56-2*s34
     .*s56+s56**2)*za12*(za35*zb13+za45*zb14)**2
     ./(-za35*zb45-za36*zb46)*zb56+za35*zb14
     ./(-za35*zb45-za36*zb46)**2*(za23*zb36+za25
     .*zb56)+(-s12+s34-s56)/(s12**2-2*s12*s34+s34**2-2*
     .s12*s56-2*s34*s56+s56**2)*za35*zb14
     ./(-za35*zb45-za36*zb46)**2*((s12-s34-s56)*za12
     .*zb16+(-s12-s34+s56)*za25*zb56)
      zs11 = zs10-2*za24*(-za35*zb15-za36*zb16)*zb36/
     .(-za35*zb45-za36*zb46)*(-za35*zb34-za56
     .*zb46)/t(i3,i5,i6)**2
      zs9 = zs11+s34/(s12**2-2*s12*s34+s34**2-2*s12*s56
     .-2*s34*s56+s56**2)*za25*zb16/(-za35*
     .zb45-za36*zb46)*(t(i3,i5,i6)-t(i4,i5,i6))/2+(-s12**2
     .+2*s12*s34-s34**2+3*s34*(-s12+s34-s56)
     .+2*s12*s56+2*s34*s56-s56**2)/(s12**2
     .-2*s12*s34+s34**2-2*s12*s56-2*s34*s56
     .+s56**2)**2*(-za23*zb13-za24*zb14)/(-za35
     .*zb45-za36*zb46)*(-za35*zb36-za45*zb46)*
     .(t(i3,i5,i6)-t(i4,i5,i6))/2
      zs7 = zs8*zs9
      zs5 = zs6+zs7
      zt0 = zs5-1/(s12**2-2*s12*s34+s34**2-2*s12*
     .s56-2*s34*s56+s56**2)/za12*za25/za56*(-(-s12
     .+s34-s56)*za25+2*za12*za56*zb16)/(-za35
     .*zb45-za36*zb46)*(-t(i3,i5,i6)+t(i4,i5,i6))/4+1/(s12**2
     .-2*s12*s34+s34**2-2*s12*s56-2*s34*s56
     .+s56**2)/zb12*zb16/(-za35*zb45-za36*zb46)
     ./zb56*(-(-s12+s34-s56)*zb16+2*za25*zb12*
     .zb56)*(t(i3,i5,i6)-t(i4,i5,i6))/4

      zampC0g1Zsl = zt0

      return
      end function

      function zampC0g1Zax(i1,i2,i3,i4,i5,i6)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)

      s12  = s6(i1,i2)
      s34  = s6(i3,i4)
      s56  = s6(i5,i6)
      zb63 = zB6(i6,i3)
      za42 = zA6(i4,i2)
      za25 = zA6(i2,i5)
      za12 = zA6(i1,i2)
      zb61 = zB6(i6,i1)
      zb13 = zB6(i1,i3)
      za45 = zA6(i4,i5)
      zb12 = zB6(i1,i2)
      za24 = zA6(i2,i4)
      za34 = zA6(i3,i4)
      zb31 = zB6(i3,i1)
      zb34 = zB6(i3,i4)

      zampC0g1Zax = +2d0*(
C      .     +zfmt(s12,s34,s56)
     .     -zFzip(s12,s34,s56))/s56
     . *(zb63*za42*za25/za12
     . -zb61*zb13*za45/zb12)
     .    +2d0*(
C      .     +zfmt(s34,s12,s56)
     .     -zFzip(s34,s12,s56))/s56
     . *(zb61*za24*za45/za34
     . -zb63*zb31*za25/zb34)

      return
      end function

      function zampC0g1ZaxA(i1,i2,i3,i4,i5,i6)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      complex*16 zampC0g1ZaxA(2,2,2)

      zampC0g1ZaxA(1,1,1) = zampC0g1Zax(i1,i3,i4,i2,i5,i6)
      zampC0g1ZaxA(1,2,1) = -zampC0g1Zax(i1,i3,i2,i4,i5,i6)
      zampC0g1ZaxA(1,1,2) = zampC0g1Zax(i1,i3,i4,i2,i6,i5)
      zampC0g1ZaxA(1,2,2) = -zampC0g1Zax(i1,i3,i2,i4,i6,i5)
      call swapzAB_kin(6)
      zampC0g1ZaxA(2,1,1) = zampC0g1Zax(i1,i3,i2,i4,i6,i5)
      zampC0g1ZaxA(2,2,1) = -zampC0g1Zax(i1,i3,i4,i2,i6,i5)
      zampC0g1ZaxA(2,1,2) = zampC0g1Zax(i1,i3,i2,i4,i5,i6)
      zampC0g1ZaxA(2,2,2) = -zampC0g1Zax(i1,i3,i4,i2,i5,i6)
      call swapzAB_kin(6)


      return
      end function


      end module C0gZ


