******************************************************************
* The full matrix element for the 4-quark + gluon + Z process,
*
* q(i1) + qb(i5) -> Q(i4) + Qb(i3) + g(i2) + e+(i6) + e-(i7)
*
* At tree level, all colour orderings including overall factors.
*
* The Z boson couples to the i1 and i5 pair of quarks.
*

      function FullC1g0Z(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      use C1gZ, only : genC1g0Zamp
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)

      FullC1g0Z = 0d0

      if(i2.le.2)then
         fac = 0.25d0/nc/(nc**2-1d0) !aveqg
         nf2 = nf1
      else
         fac = 0.25d0/nc**2 ! aveqq for all other configurations
      endif

      fac = fac*4d0*(4*pi*as)**3*(4*pi*amz)**2*8d0*(nc**2-1)/(2*nc)*nc**2/4d0

      FullC1g0Z = C1g0Z(i1,i2,i3,i4,i5,i6,i7)
     .           -(1d0/nc)*(D1g0Z(i1,i2,i3,i4,i5,i6,i7)
     .                     -Dt1g0Z(i1,i2,i3,i4,i5,i6,i7))
     .           +(1d0/nc**2)*(+Ct1g0Z(i1,i2,i3,i4,i5,i6,i7)
     .                         -Ctt1g0Z(i1,i2,i3,i4,i5,i6,i7))
     .           +(1d0/nc**3)*Dt1g0Z(i1,i2,i3,i4,i5,i6,i7)



      if((i1.le.2).and.(i3.le.2).and.(nf1.eq.nf2))then ! nf type contribution

         nup = 2
         ndo = 3

         nf2 = 2

         FullC1g0Z = FullC1g0Z
     .              +nup*(C1g0Z(i1,i2,i5,i4,i3,i6,i7)
     .              +(1d0/nc**2)*(Ct1g0Z(i1,i2,i5,i4,i3,i6,i7)
     .                          -Ctt1g0Z(i1,i2,i5,i4,i3,i6,i7)))

         nf2 = 1
         FullC1g0Z = FullC1g0Z
     .              +ndo*(C1g0Z(i1,i2,i5,i4,i3,i6,i7)
     .              +(1d0/nc**2)*(Ct1g0Z(i1,i2,i5,i4,i3,i6,i7)
     .                          -Ctt1g0Z(i1,i2,i5,i4,i3,i6,i7)))

         ! reset flavour
         nf2 = nf1
      elseif(i2.le.2)then ! gluon initial state
         nup = 2
         ndo = 3


         FullC1g0Z = FullC1g0Z
     .              +C1g0Z(i1,i2,i5,i4,i3,i6,i7)
     .              +(1d0/nc**2)*(Ct1g0Z(i1,i2,i5,i4,i3,i6,i7)
     .                          -Ctt1g0Z(i1,i2,i5,i4,i3,i6,i7))

         FullC1g0Z = 0.5d0*FullC1g0Z

         if((nf1.eq.1).or.(nf1.eq.3).or.(nf1.eq.5))then
            nup = 2
            ndo = 2
         elseif((nf1.eq.2).or.(nf1.eq.4))then
            nup = 1
            ndo = 3
         endif

         nf2 = 2

         FullC1g0Z = FullC1g0Z
     .              +nup*(C1g0Z(i1,i2,i3,i4,i5,i6,i7)
     .              +(1d0/nc**2)*(Ct1g0Z(i1,i2,i3,i4,i5,i6,i7)
     .                          -Ctt1g0Z(i1,i2,i3,i4,i5,i6,i7)))

         nf2 = 1
         FullC1g0Z = FullC1g0Z
     .              +ndo*(C1g0Z(i1,i2,i3,i4,i5,i6,i7)
     .              +(1d0/nc**2)*(Ct1g0Z(i1,i2,i3,i4,i5,i6,i7)
     .                          -Ctt1g0Z(i1,i2,i3,i4,i5,i6,i7)))

         ! reset flavour
         nf2 = nf1
      elseif(nf1.eq.nf2)then ! Other momentum ordering

         FullC1g0Z = FullC1g0Z
     .              +C1g0Z(i1,i2,i5,i4,i3,i6,i7)
     .              +(1d0/nc**2)*(Ct1g0Z(i1,i2,i5,i4,i3,i6,i7)
     .                          -Ctt1g0Z(i1,i2,i5,i4,i3,i6,i7))
         fac = fac*0.5d0 ! Symmetry factor
      endif

      FullC1g0Z = fac*FullC1g0Z

      return
      end




******************************************************************
* The matrix element for two non-identical qqb pairs and an electron pair.
*
* q(i1) + qb(i5) -> Q(i4) + Qb(i3) + g(i2) + e+(i6) + e-(i7)
*
* At tree level, leading colour. Note that this includes both colour
* orderings for the gluon position. Whilst this matrix element can be decomposed
* further, there is little point given that the subtraction term must be summed over both colour orderings anyway.
*
* The Z boson couples to the i1 and i5 pair of quarks.
*

      function C1g0Z(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      use C1gZ, only : genC1g0Zamp
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      complex*16 zampB1_1234(2,2,2,2,2,2),zampB2_1234(2,2,2,2,2,2)
      complex*16 zampB1_3214(2,2,2,2,2,2),zampB2_3214(2,2,2,2,2,2)
      complex*16 zampB1_3412(2,2,2,2,2,2),zampB2_3412(2,2,2,2,2,2)
      complex*16 zampB1_1432(2,2,2,2,2,2),zampB2_1432(2,2,2,2,2,2)

      C1g0Z = 0d0

      call genC1g0Zamp(i4,i1,i5,i2,i3,i6,i7,zampB1_1234,zampB2_1234)
      call genC1g0Zamp(i4,i1,i3,i2,i5,i6,i7,zampB1_3214,zampB2_3214)
      call genC1g0Zamp(i1,i4,i3,i2,i5,i6,i7,zampB1_3412,zampB2_3412)
      call genC1g0Zamp(i1,i4,i5,i2,i3,i6,i7,zampB1_1432,zampB2_1432)

      call genC1g0ZA(zampB1_1234,zampB2_1234,
     .               zampB1_3214,zampB2_3214,
     .               zampB1_3412,zampB2_3412,
     .               zampB1_1432,zampB2_1432,A)

      call genC1g0ZG(zampB1_1234,zampB2_1234,
     .               zampB1_3214,zampB2_3214,
     .               zampB1_3412,zampB2_3412,
     .               zampB1_1432,zampB2_1432,G)



      C1g0Z = A+G

      return
      end

******************************************************************
* The matrix element for two non-identical qqb pairs and an electron pair.
*
* q(i1) + qb(i5) -> Q(i4) + Qb(i3) + g(i2) + e+(i6) + e-(i7)
*
* At tree level, subleading colour
*
* The Z boson couples to the i1 and i5 pair of quarks.
*

      function Ct1g0Z(i1,i2,i3,i4,i5,i6,i7)
      use Process_mod, only : nPDFs_proc
      use KinData_mod
      use C1gZ, only : genC1g0Zamp
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)

      complex*16 zampB1_1234(2,2,2,2,2,2),zampB2_1234(2,2,2,2,2,2)
      complex*16 zampB1_3214(2,2,2,2,2,2),zampB2_3214(2,2,2,2,2,2)
      complex*16 zampB1_3412(2,2,2,2,2,2),zampB2_3412(2,2,2,2,2,2)
      complex*16 zampB1_1432(2,2,2,2,2,2),zampB2_1432(2,2,2,2,2,2)

      Ct1g0Z = 0d0

      call genC1g0Zamp(i4,i1,i5,i2,i3,i6,i7,zampB1_1234,zampB2_1234)
      call genC1g0Zamp(i4,i1,i3,i2,i5,i6,i7,zampB1_3214,zampB2_3214)
      call genC1g0Zamp(i1,i4,i3,i2,i5,i6,i7,zampB1_3412,zampB2_3412)
      call genC1g0Zamp(i1,i4,i5,i2,i3,i6,i7,zampB1_1432,zampB2_1432)

      call genC1g0ZA(zampB1_1234,zampB2_1234,
     .               zampB1_3214,zampB2_3214,
     .               zampB1_3412,zampB2_3412,
     .               zampB1_1432,zampB2_1432,A)

      call genC1g0ZD(zampB1_1234,zampB2_1234,
     .               zampB1_3214,zampB2_3214,
     .               zampB1_3412,zampB2_3412,
     .               zampB1_1432,zampB2_1432,D)


!     TM 17/09/2015: changed Ct1g0Z -> -Ct1g0Z to match Xuan's convention
!      Ct1g0Z = A+D
      if(nPDFs_proc.eq.2) then
         Ct1g0Z = -A-D
      else
         Ct1g0Z = A+D
      endif
      return
      end

******************************************************************
* The matrix element for two non-identical qqb pairs and an electron pair.
*
* q(i1) + qb(i5) -> Q(i4) + Qb(i3) + g(i2) + e+(i6) + e-(i7)
*
* At tree level, subsubleading colour
*
* The Z boson couples to the i1 and i5 pair of quarks.
*

      function Ctt1g0Z(i1,i2,i3,i4,i5,i6,i7)
      use Process_mod, only : nPDFs_proc
      use KinData_mod
      use C1gZ, only : genC1g0Zamp
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)

      complex*16 zampB1_1234(2,2,2,2,2,2),zampB2_1234(2,2,2,2,2,2)
      complex*16 zampB1_3214(2,2,2,2,2,2),zampB2_3214(2,2,2,2,2,2)
      complex*16 zampB1_3412(2,2,2,2,2,2),zampB2_3412(2,2,2,2,2,2)
      complex*16 zampB1_1432(2,2,2,2,2,2),zampB2_1432(2,2,2,2,2,2)

      Ctt1g0Z = 0d0

      call genC1g0Zamp(i4,i1,i5,i2,i3,i6,i7,zampB1_1234,zampB2_1234)
      call genC1g0Zamp(i4,i1,i3,i2,i5,i6,i7,zampB1_3214,zampB2_3214)
      call genC1g0Zamp(i1,i4,i3,i2,i5,i6,i7,zampB1_3412,zampB2_3412)
      call genC1g0Zamp(i1,i4,i5,i2,i3,i6,i7,zampB1_1432,zampB2_1432)


      call genC1g0ZF(zampB1_1234,zampB2_1234,
     .               zampB1_3214,zampB2_3214,
     .               zampB1_3412,zampB2_3412,
     .               zampB1_1432,zampB2_1432,F)

      call genC1g0ZG(zampB1_1234,zampB2_1234,
     .               zampB1_3214,zampB2_3214,
     .               zampB1_3412,zampB2_3412,
     .               zampB1_1432,zampB2_1432,G)

!     TM 17/09/2015: changed Ctt1g0Z -> -Ctt1g0Z to match Xuan's convention

      if(nPDFs_proc.eq.2) then
         Ctt1g0Z = 2d0*(F+G)
      else
         Ctt1g0Z = -2d0*(F+G)
      endif
      return
      end

******************************************************************
* The matrix element for two identical qqb pairs and an electron pair.
*
* q(i1) + qb(i5) -> Q(i4) + Qb(i3) + g(i2) + e+(i6) + e-(i7)
*
* At tree level, leading colour
*
* The Z boson couples to the i1 and i5 pair of quarks.
*
*

      function D1g0Z(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      use C1gZ, only : genC1g0Zamp
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      common /zewfacswap/ swap
!$omp threadprivate(/zewfacswap/)
      logical swap
      complex*16 zampB1_1234(2,2,2,2,2,2),zampB2_1234(2,2,2,2,2,2)
      complex*16 zampB1_3214(2,2,2,2,2,2),zampB2_3214(2,2,2,2,2,2)
      complex*16 zampB1_3412(2,2,2,2,2,2),zampB2_3412(2,2,2,2,2,2)
      complex*16 zampB1_1432(2,2,2,2,2,2),zampB2_1432(2,2,2,2,2,2)

      D1g0Z = 0d0

      if (nf1.eq.nf2)then

         swap = .TRUE.
         call genC1g0Zamp(i4,i1,i5,i2,i3,i6,i7,zampB1_1432,zampB2_1432)
         call genC1g0Zamp(i1,i4,i5,i2,i3,i6,i7,zampB1_1234,zampB2_1234)
         swap = .FALSE.
         call genC1g0Zamp(i1,i4,i3,i2,i5,i6,i7,zampB1_3214,zampB2_3214)
         call genC1g0Zamp(i4,i1,i3,i2,i5,i6,i7,zampB1_3412,zampB2_3412)

         call genC1g0ZB(zampB1_1234,zampB2_1234,
     .        zampB1_3214,zampB2_3214,
     .        zampB1_3412,zampB2_3412,
     .        zampB1_1432,zampB2_1432,B)

         call genC1g0ZE(zampB1_1234,zampB2_1234,
     .        zampB1_3214,zampB2_3214,
     .        zampB1_3412,zampB2_3412,
     .        zampB1_1432,zampB2_1432,E)


         D1g0Z = 2d0*B+E

      endif



      return
      end

******************************************************************
* The matrix element for two identical qqb pairs and an electron pair.
*
* q(i1) + qb(i5) -> Q(i4) + Qb(i3) + g(i2) + e+(i6) + e-(i7)
*
* At tree level, subleading colour
*
* The Z boson couples to the i1 and i5 pair of quarks.
*

      function Dt1g0Z(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      use C1gZ, only : genC1g0Zamp
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      common /zewfacswap/ swap
!$omp threadprivate(/zewfacswap/)
      logical swap
      complex*16 zampB1_1234(2,2,2,2,2,2),zampB2_1234(2,2,2,2,2,2)
      complex*16 zampB1_3214(2,2,2,2,2,2),zampB2_3214(2,2,2,2,2,2)
      complex*16 zampB1_3412(2,2,2,2,2,2),zampB2_3412(2,2,2,2,2,2)
      complex*16 zampB1_1432(2,2,2,2,2,2),zampB2_1432(2,2,2,2,2,2)

      Dt1g0Z = 0d0

      if (nf1.eq.nf2)then
         swap = .TRUE.
         call genC1g0Zamp(i4,i1,i5,i2,i3,i6,i7,zampB1_1432,zampB2_1432)
         call genC1g0Zamp(i1,i4,i5,i2,i3,i6,i7,zampB1_1234,zampB2_1234)
         swap = .FALSE.
         call genC1g0Zamp(i1,i4,i3,i2,i5,i6,i7,zampB1_3214,zampB2_3214)
         call genC1g0Zamp(i4,i1,i3,i2,i5,i6,i7,zampB1_3412,zampB2_3412)

         call genC1g0ZB(zampB1_1234,zampB2_1234,
     .        zampB1_3214,zampB2_3214,
     .        zampB1_3412,zampB2_3412,
     .        zampB1_1432,zampB2_1432,B)

         call genC1g0ZC(zampB1_1234,zampB2_1234,
     .        zampB1_3214,zampB2_3214,
     .        zampB1_3412,zampB2_3412,
     .        zampB1_1432,zampB2_1432,C)

         call genC1g0ZE(zampB1_1234,zampB2_1234,
     .        zampB1_3214,zampB2_3214,
     .        zampB1_3412,zampB2_3412,
     .        zampB1_1432,zampB2_1432,E)


         Dt1g0Z = B+C+E
      endif

      return
      end




***************************************************
* Generates the sub-component of the C1g0Z matrix element A,
* the notation is as according to Phys.Rev. D59 (1999) 014020
*
*

      subroutine genC1g0ZA(zampB1_1234,zampB2_1234,
     .                     zampB1_3214,zampB2_3214,
     .                     zampB1_3412,zampB2_3412,
     .                     zampB1_1432,zampB2_1432,A)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      complex*16 zampB1_1234(2,2,2,2,2,2),zampB2_1234(2,2,2,2,2,2)
      complex*16 zampB1_3214(2,2,2,2,2,2),zampB2_3214(2,2,2,2,2,2)
      complex*16 zampB1_3412(2,2,2,2,2,2),zampB2_3412(2,2,2,2,2,2)
      complex*16 zampB1_1432(2,2,2,2,2,2),zampB2_1432(2,2,2,2,2,2)

      A = 0d0
      do kk=1,2
         do ll=1,2
            do mm=1,2
               do nn=1,2
                  A = A
     .                 +abs(zampB1_1234(nf2,nf1,kk,ll,mm,nn))**2
     .                 +abs(zampB2_1234(nf2,nf1,kk,ll,mm,nn))**2
     .                 +abs(zampB1_3412(nf1,nf2,ll,kk,mm,nn))**2
     .                 +abs(zampB2_3412(nf1,nf2,ll,kk,mm,nn))**2


               enddo
            enddo
         enddo
      enddo

      return
      end

***************************************************
* Generates the sub-component of the C1g0Z matrix element B,
* the notation is as according to Phys.Rev. D59 (1999) 014020
*
*

      subroutine genC1g0ZB(zampB1_1234,zampB2_1234,
     .                     zampB1_3214,zampB2_3214,
     .                     zampB1_3412,zampB2_3412,
     .                     zampB1_1432,zampB2_1432,B)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      complex*16 zampB1_1234(2,2,2,2,2,2),zampB2_1234(2,2,2,2,2,2)
      complex*16 zampB1_3214(2,2,2,2,2,2),zampB2_3214(2,2,2,2,2,2)
      complex*16 zampB1_3412(2,2,2,2,2,2),zampB2_3412(2,2,2,2,2,2)
      complex*16 zampB1_1432(2,2,2,2,2,2),zampB2_1432(2,2,2,2,2,2)

      B = 0d0

      do ll=1,2
         do mm=1,2
            do nn=1,2
               B = B-2d0*dble(
     .              +zampB1_1234(nf2,nf1,ll,ll,mm,nn)
     .              *Dconjg(zampB1_1432(nf2,nf1,ll,ll,mm,nn))
     .              +zampB2_1234(nf2,nf1,ll,ll,mm,nn)
     .              *Dconjg(zampB2_3214(nf1,nf2,ll,ll,mm,nn))
     .              +zampB1_3412(nf1,nf2,ll,ll,mm,nn)
     .              *Dconjg(zampB1_3214(nf1,nf2,ll,ll,mm,nn))
     .              +zampB2_3412(nf2,nf1,ll,ll,mm,nn)
     .              *Dconjg(zampB2_1432(nf2,nf1,ll,ll,mm,nn)))
            enddo
         enddo
      enddo


      return
      end

***************************************************
* Generates the sub-component of the C1g0Z matrix element C,
* the notation is as according to Phys.Rev. D59 (1999) 014020
*
*

      subroutine genC1g0ZC(zampB1_1234,zampB2_1234,
     .                     zampB1_3214,zampB2_3214,
     .                     zampB1_3412,zampB2_3412,
     .                     zampB1_1432,zampB2_1432,C)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      complex*16 zampB1_1234(2,2,2,2,2,2),zampB2_1234(2,2,2,2,2,2)
      complex*16 zampB1_3214(2,2,2,2,2,2),zampB2_3214(2,2,2,2,2,2)
      complex*16 zampB1_3412(2,2,2,2,2,2),zampB2_3412(2,2,2,2,2,2)
      complex*16 zampB1_1432(2,2,2,2,2,2),zampB2_1432(2,2,2,2,2,2)

      C = 0d0


      do ll=1,2
         do mm=1,2
            do nn=1,2
               C = C-2d0*dble(
     .              +zampB1_1234(nf2,nf1,ll,ll,mm,nn)
     .              *Dconjg(zampB1_3214(nf1,nf2,ll,ll,mm,nn))
     .              +zampB2_1234(nf2,nf1,ll,ll,mm,nn)
     .              *Dconjg(zampB2_1432(nf2,nf1,ll,ll,mm,nn))
     .              +zampB1_3412(nf1,nf2,ll,ll,mm,nn)
     .              *Dconjg(zampB1_1432(nf2,nf1,ll,ll,mm,nn))
     .              +zampB2_3412(nf1,nf2,ll,ll,mm,nn)
     .              *Dconjg(zampB2_3214(nf1,nf2,ll,ll,mm,nn)))
            enddo
         enddo
      enddo

      return
      end

***************************************************
* Generates the sub-component of the C1g0Z matrix element D,
* the notation is as according to Phys.Rev. D59 (1999) 014020
*
*

      subroutine genC1g0ZD(zampB1_1234,zampB2_1234,
     .                     zampB1_3214,zampB2_3214,
     .                     zampB1_3412,zampB2_3412,
     .                     zampB1_1432,zampB2_1432,D)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      complex*16 zampB1_1234(2,2,2,2,2,2),zampB2_1234(2,2,2,2,2,2)
      complex*16 zampB1_3214(2,2,2,2,2,2),zampB2_3214(2,2,2,2,2,2)
      complex*16 zampB1_3412(2,2,2,2,2,2),zampB2_3412(2,2,2,2,2,2)
      complex*16 zampB1_1432(2,2,2,2,2,2),zampB2_1432(2,2,2,2,2,2)

      D = 0d0

      do kk=1,2
         do ll=1,2
            do mm=1,2
               do nn=1,2
                  D = D+2d0*dble(
     .                       +zampB1_1234(nf2,nf1,kk,ll,mm,nn)
     .                       *Dconjg(zampB2_1234(nf2,nf1,kk,ll,mm,nn))
     .                       +zampB1_3412(nf1,nf2,ll,kk,mm,nn)
     .                       *Dconjg(zampB2_3412(nf1,nf2,ll,kk,mm,nn)))

               enddo
            enddo
         enddo
      enddo

      return
      end

***************************************************
* Generates the sub-component of the C1g0Z matrix element E,
* the notation is as according to Phys.Rev. D59 (1999) 014020
*
*

      subroutine genC1g0ZE(zampB1_1234,zampB2_1234,
     .                     zampB1_3214,zampB2_3214,
     .                     zampB1_3412,zampB2_3412,
     .                     zampB1_1432,zampB2_1432,E)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      complex*16 zampB1_1234(2,2,2,2,2,2),zampB2_1234(2,2,2,2,2,2)
      complex*16 zampB1_3214(2,2,2,2,2,2),zampB2_3214(2,2,2,2,2,2)
      complex*16 zampB1_3412(2,2,2,2,2,2),zampB2_3412(2,2,2,2,2,2)
      complex*16 zampB1_1432(2,2,2,2,2,2),zampB2_1432(2,2,2,2,2,2)


      E = 0d0

      do ll=1,2
         do mm=1,2
            do nn=1,2
               E = E-2d0*dble(
     .              +(zampB1_1234(nf2,nf1,ll,ll,mm,nn)
     .              +zampB1_3412(nf1,nf2,ll,ll,mm,nn))
     .              *Dconjg(zampB2_3214(nf1,nf2,ll,ll,mm,nn)
     .              +zampB2_1432(nf2,nf1,ll,ll,mm,nn))
     .              +(zampB2_1234(nf2,nf1,ll,ll,mm,nn)
     .              +zampB2_3412(nf1,nf2,ll,ll,mm,nn))
     .              *Dconjg(zampB1_3214(nf1,nf2,ll,ll,mm,nn)
     .              +zampB1_1432(nf2,nf1,ll,ll,mm,nn)))
            enddo
         enddo
      enddo


      return
      end

***************************************************
* Generates the sub-component of the C1g0Z matrix element F,
* the notation is as according to Phys.Rev. D59 (1999) 014020
*
*

      subroutine genC1g0ZF(zampB1_1234,zampB2_1234,
     .                     zampB1_3214,zampB2_3214,
     .                     zampB1_3412,zampB2_3412,
     .                     zampB1_1432,zampB2_1432,F)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      complex*16 zampB1_1234(2,2,2,2,2,2),zampB2_1234(2,2,2,2,2,2)
      complex*16 zampB1_3214(2,2,2,2,2,2),zampB2_3214(2,2,2,2,2,2)
      complex*16 zampB1_3412(2,2,2,2,2,2),zampB2_3412(2,2,2,2,2,2)
      complex*16 zampB1_1432(2,2,2,2,2,2),zampB2_1432(2,2,2,2,2,2)

      F = 0d0

      do kk=1,2
         do ll=1,2
            do mm=1,2
               do nn=1,2
                  F = F+2d0*dble(
     .                       +zampB1_1234(nf2,nf1,kk,ll,mm,nn)
     .                       *Dconjg(zampB1_3412(nf1,nf2,ll,kk,mm,nn))
     .                       +zampB2_1234(nf2,nf1,kk,ll,mm,nn)
     .                       *Dconjg(zampB2_3412(nf1,nf2,ll,kk,mm,nn)))

               enddo
            enddo
         enddo
      enddo


      return
      end

***************************************************
* Generates the sub-component of the C1g0Z matrix element G,
* the notation is as according to Phys.Rev. D59 (1999) 014020
*
*

      subroutine genC1g0ZG(zampB1_1234,zampB2_1234,
     .                     zampB1_3214,zampB2_3214,
     .                     zampB1_3412,zampB2_3412,
     .                     zampB1_1432,zampB2_1432,G)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      complex*16 zampB1_1234(2,2,2,2,2,2),zampB2_1234(2,2,2,2,2,2)
      complex*16 zampB1_3214(2,2,2,2,2,2),zampB2_3214(2,2,2,2,2,2)
      complex*16 zampB1_3412(2,2,2,2,2,2),zampB2_3412(2,2,2,2,2,2)
      complex*16 zampB1_1432(2,2,2,2,2,2),zampB2_1432(2,2,2,2,2,2)

      G = 0d0

      do kk=1,2
         do ll=1,2
            do mm=1,2
               do nn=1,2
                  G = G+2d0*dble(
     .                 +zampB1_1234(nf2,nf1,kk,ll,mm,nn)
     .                 *Dconjg(zampB2_3412(nf1,nf2,ll,kk,mm,nn))
     .                 +zampB2_1234(nf2,nf1,kk,ll,mm,nn)
     .                 *Dconjg(zampB1_3412(nf1,nf2,ll,kk,mm,nn)))

               enddo
            enddo
         enddo
      enddo

      return
      end

**********************************************************************
* A wrapper function for the C1g0Z matrix element symmetrised over the
* second quark pair. This is required for certain subtraction terms
* (e.g. qg initiated nf piece).
*
* qg configuration => symmetrised
* qbg configuration => symmetrised
* anything else => not symmetrised

      function sC1g0Z(i1,i2,i3,i4,i5,i6,i7)
      use Process_mod, only : nPDFs_proc
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      select case(nPDFs_proc)
      case(2)
         if((i2.le.2).and.((i1.le.2).or.(i5.le.2)))then
            sC1g0Z = 0.5d0*(C1g0Z(i1,i2,i3,i4,i5,i6,i7)
     .           +C1g0Z(i1,i2,i4,i3,i5,i6,i7))
         else
            sC1g0Z = C1g0Z(i1,i2,i3,i4,i5,i6,i7)
         endif
      case(1)
c     symmetrising for q initiated case for DIS Z exchange
         if(((i1.eq.1).or.(i5.eq.1)))then
            sC1g0Z = 0.5d0*(C1g0Z(i1,i2,i3,i4,i5,i6,i7)
     .           +C1g0Z(i1,i2,i4,i3,i5,i6,i7))
         else
            sC1g0Z = C1g0Z(i1,i2,i3,i4,i5,i6,i7)
         endif
      case default
         sC1g0Z = C1g0Z(i1,i2,i3,i4,i5,i6,i7)
      end select

      return
      end


**********************************************************************
* A wrapper function for the Ct1g0Z matrix element symmetrised over the
* second quark pair. This is required for certain subtraction terms
* (e.g. qg initiated nf/nc**2 piece)
* Note: we include Ctt1g0Z in this function as well!
*
* qg configuration => symmetrised
* qbg configuration => symmetrised
* anything else => not symmetrised


      function sCt1g0Z(i1,i2,i3,i4,i5,i6,i7)
      use Process_mod, only : nPDFs_proc
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      select case(nPDFs_proc)
      case(2)
         if((i2.le.2).and.((i1.le.2).or.(i5.le.2)))then
            sCt1g0Z = 0.5d0*(Ct1g0Z(i1,i2,i3,i4,i5,i6,i7)
     .           +Ct1g0Z(i1,i2,i4,i3,i5,i6,i7)
     .           -Ctt1g0Z(i1,i2,i3,i4,i5,i6,i7)
     .           -Ctt1g0Z(i1,i2,i4,i3,i5,i6,i7))
         else
            sCt1g0Z = Ct1g0Z(i1,i2,i3,i4,i5,i6,i7)
     .           -Ctt1g0Z(i1,i2,i3,i4,i5,i6,i7)
         endif
      case(1)
         if((i1.eq.1).or.(i5.eq.1))then
            sCt1g0Z = 0.5d0*(Ct1g0Z(i1,i2,i3,i4,i5,i6,i7)
     .           +Ct1g0Z(i1,i2,i4,i3,i5,i6,i7)
     .           -Ctt1g0Z(i1,i2,i3,i4,i5,i6,i7)
     .           -Ctt1g0Z(i1,i2,i4,i3,i5,i6,i7))
         else
            sCt1g0Z = Ct1g0Z(i1,i2,i3,i4,i5,i6,i7)
     .           -Ctt1g0Z(i1,i2,i3,i4,i5,i6,i7)
         endif
      case(0)
            sCt1g0Z = Ct1g0Z(i1,i2,i3,i4,i5,i6,i7)
     .           -Ctt1g0Z(i1,i2,i3,i4,i5,i6,i7)
      end select

      return
      end

***********************************************************
* A wrapper function for (D1g0Z-Dt1g0Z). This is convenient
* to use in the driver program
*

      function D1g0Zc(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)


      D1g0Zc = D1g0Z(i1,i2,i3,i4,i5,i6,i7)
     .        -Dt1g0Z(i1,i2,i3,i4,i5,i6,i7)

      return
      end


*****************************************************************************
* The matrix element for two non-identical qqb pairs and an electron pair.
*
* q(i1) + qb(i5) -> Q(i4) + Qb(i3) + g(i2) + e+(i6) + e-(i7)
*
* At tree level, leading colour-a: gluon in string between Q and qb
*
* The Z boson couples to the i1 and i5 pair of quarks.
*


      function C1g0Za(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      use astore
      use C1gZ!, only : genC1g0Zamp      
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

      complex*16 zm1_1234(2,2,2,2,2,2),zm2_1234(2,2,2,2,2,2),zm3_1234(2,2,2,2,2,2),zm4_1234(2,2,2,2,2,2)

c      complex*16 zampB1_1234(2,2,2,2,2,2),zampB2_1234(2,2,2,2,2,2)
c      complex*16 zampB1_3214(2,2,2,2,2,2),zampB2_3214(2,2,2,2,2,2)
      complex*16 zm1_3412(2,2,2,2,2,2),zm2_3412(2,2,2,2,2,2),zm3_3412(2,2,2,2,2,2),zm4_3412(2,2,2,2,2,2)

      call genC1g0Zamp1(i1,i4,i3,i2,i5,i6,i7,zm1_1234,zm2_1234,zm3_1234,zm4_1234)
c      call genC1g0Zamp(i4,i1,i3,i2,i5,i6,i7,zampB1_3214,zampB2_3214)
      call genC1g0Zamp1(i4,i1,i5,i2,i3,i6,i7,zm1_3412,zm2_3412,zm3_3412,zm4_3412)

      zi=(0d0,1d0)

           A = 0d0
      z1 = (0d0,0d0)

      do kk=1,2
         do ll=1,2
            do mm=1,2
               do nn=1,2

                  z1 = +zi*(zm1_1234(nf1,nf2,kk,ll,mm,nn)+zm3_1234(nf1,nf2,ll,kk,mm,nn))
     .                 +zi*(zm2_3412(nf2,nf1,ll,kk,mm,nn)+zm4_3412(nf2,nf1,kk,ll,mm,nn))

                  A = A +abs(z1)**2
                  
               enddo
            enddo

         enddo
      enddo
           C1g0Za = A
 
      return
      end



*****************************************************************************
* The matrix element for two non-identical qqb pairs and an electron pair.
*
* q(i1) + qb(i5) -> Q(i4) + Qb(i3) + g(i2) + e+(i6) + e-(i7)
*
* At tree level, leading colour-b: gluon in string between q and Qb
*
* The Z boson couples to the i1 and i5 pair of quarks.

      function C1g0Zb(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      use astore
      use C1gZ!, only : genC1g0Zamp      
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

      complex*16 zm1_1234(2,2,2,2,2,2),zm2_1234(2,2,2,2,2,2),zm3_1234(2,2,2,2,2,2),zm4_1234(2,2,2,2,2,2)

c      complex*16 zampB1_1234(2,2,2,2,2,2),zampB2_1234(2,2,2,2,2,2)
c      complex*16 zampB1_3214(2,2,2,2,2,2),zampB2_3214(2,2,2,2,2,2)
      complex*16 zm1_3412(2,2,2,2,2,2),zm2_3412(2,2,2,2,2,2),zm3_3412(2,2,2,2,2,2),zm4_3412(2,2,2,2,2,2)

      call genC1g0Zamp1(i1,i4,i3,i2,i5,i6,i7,zm1_1234,zm2_1234,zm3_1234,zm4_1234)
c      call genC1g0Zamp(i4,i1,i3,i2,i5,i6,i7,zampB1_3214,zampB2_3214)
      call genC1g0Zamp1(i4,i1,i5,i2,i3,i6,i7,zm1_3412,zm2_3412,zm3_3412,zm4_3412)

      zi=(0d0,1d0)

           A = 0d0
      z1 = (0d0,0d0)

      do kk=1,2
         do ll=1,2
            do mm=1,2
               do nn=1,2


                 z1 = +zi*(zm2_1234(nf1,nf2,kk,ll,mm,nn)+zm4_1234(nf1,nf2,ll,kk,mm,nn))
     .                  +zi*(zm1_3412(nf2,nf1,ll,kk,mm,nn)+zm3_3412(nf2,nf1,kk,ll,mm,nn))

                  A = A +abs(z1)**2

               enddo
            enddo

         enddo
      enddo
           C1g0Zb = A


      return
      end


*****************************************************************************
* The matrix element for two non-identical qqb pairs and an electron pair.
*
* q(i1) + qb(i5) -> Q(i4) + Qb(i3) + g(i2) + e+(i6) + e-(i7)
*
* At tree level, subleading colour-c: gluon in string between q and qb
*
* The Z boson couples to the i1 and i5 pair of quarks.

      function C1g0Zc(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      use astore
      use C1gZ!, only : genC1g0Zamp      
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

      complex*16 zm1_1234(2,2,2,2,2,2),zm2_1234(2,2,2,2,2,2),zm3_1234(2,2,2,2,2,2),zm4_1234(2,2,2,2,2,2)

c      complex*16 zampB1_1234(2,2,2,2,2,2),zampB2_1234(2,2,2,2,2,2)
c      complex*16 zampB1_3214(2,2,2,2,2,2),zampB2_3214(2,2,2,2,2,2)
      complex*16 zm1_3412(2,2,2,2,2,2),zm2_3412(2,2,2,2,2,2),zm3_3412(2,2,2,2,2,2),zm4_3412(2,2,2,2,2,2)

      call genC1g0Zamp1(i1,i4,i3,i2,i5,i6,i7,zm1_1234,zm2_1234,zm3_1234,zm4_1234)
c      call genC1g0Zamp(i4,i1,i3,i2,i5,i6,i7,zampB1_3214,zampB2_3214)
      call genC1g0Zamp1(i4,i1,i5,i2,i3,i6,i7,zm1_3412,zm2_3412,zm3_3412,zm4_3412)

      zi=(0d0,1d0)

           A = 0d0
      z1 = (0d0,0d0)

      do kk=1,2
         do ll=1,2
            do mm=1,2
               do nn=1,2


                  z1 = +zi*(zm1_1234(nf1,nf2,kk,ll,mm,nn)+zm2_1234(nf1,nf2,kk,ll,mm,nn))
     .               +zi*(zm3_1234(nf1,nf2,ll,kk,mm,nn)+zm4_1234(nf1,nf2,ll,kk,mm,nn))

                  A = A +abs(z1)**2

               enddo
            enddo

         enddo
      enddo
           C1g0Zc = A


      return
      end


*****************************************************************************
* The matrix element for two non-identical qqb pairs and an electron pair.
*
* q(i1) + qb(i5) -> Q(i4) + Qb(i3) + g(i2) + e+(i6) + e-(i7)
*
* At tree level, subleading colour-d: gluon in string between Q and Qb
*
* The Z boson couples to the i1 and i5 pair of quarks.

      function C1g0Zd(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      use astore
      use C1gZ!, only : genC1g0Zamp      
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

      complex*16 zm1_1234(2,2,2,2,2,2),zm2_1234(2,2,2,2,2,2),zm3_1234(2,2,2,2,2,2),zm4_1234(2,2,2,2,2,2)

c      complex*16 zampB1_1234(2,2,2,2,2,2),zampB2_1234(2,2,2,2,2,2)
c      complex*16 zampB1_3214(2,2,2,2,2,2),zampB2_3214(2,2,2,2,2,2)
      complex*16 zm1_3412(2,2,2,2,2,2),zm2_3412(2,2,2,2,2,2),zm3_3412(2,2,2,2,2,2),zm4_3412(2,2,2,2,2,2)

      call genC1g0Zamp1(i1,i4,i3,i2,i5,i6,i7,zm1_1234,zm2_1234,zm3_1234,zm4_1234)
c      call genC1g0Zamp(i4,i1,i3,i2,i5,i6,i7,zampB1_3214,zampB2_3214)
      call genC1g0Zamp1(i4,i1,i5,i2,i3,i6,i7,zm1_3412,zm2_3412,zm3_3412,zm4_3412)

      zi=(0d0,1d0)

           A = 0d0
      z1 = (0d0,0d0)

      do kk=1,2
         do ll=1,2
            do mm=1,2
               do nn=1,2


                  z1 =  +zi*(zm1_3412(nf2,nf1,ll,kk,mm,nn)+zm2_3412(nf2,nf1,ll,kk,mm,nn))
     .               +zi*(zm3_3412(nf2,nf1,kk,ll,mm,nn)+zm4_3412(nf2,nf1,kk,ll,mm,nn))

                  A = A +abs(z1)**2

               enddo
            enddo

         enddo
      enddo
           C1g0Zd = A


      return
      end


*****************************************************************************

* The interference term between the leading-color-a and subleading-d.

      function C1g0Zad(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      use astore      
      use C1gZ!, only : genC1g0Zamp      
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

      complex*16 zm1_1234(2,2,2,2,2,2),zm2_1234(2,2,2,2,2,2),zm3_1234(2,2,2,2,2,2),zm4_1234(2,2,2,2,2,2)

c      complex*16 zampB1_1234(2,2,2,2,2,2),zampB2_1234(2,2,2,2,2,2)
c      complex*16 zampB1_3214(2,2,2,2,2,2),zampB2_3214(2,2,2,2,2,2)
      complex*16 zm1_3412(2,2,2,2,2,2),zm2_3412(2,2,2,2,2,2),zm3_3412(2,2,2,2,2,2),zm4_3412(2,2,2,2,2,2)

      call genC1g0Zamp1(i1,i4,i3,i2,i5,i6,i7,zm1_1234,zm2_1234,zm3_1234,zm4_1234)
c      call genC1g0Zamp(i4,i1,i3,i2,i5,i6,i7,zampB1_3214,zampB2_3214)
      call genC1g0Zamp1(i4,i1,i5,i2,i3,i6,i7,zm1_3412,zm2_3412,zm3_3412,zm4_3412)

      zi=(0d0,1d0)

           A = 0d0
      z1 = (0d0,0d0)
      z2 = (0d0,0d0)

      do kk=1,2
         do ll=1,2
            do mm=1,2
               do nn=1,2

               
                   z1=  +zi*(zm1_1234(nf1,nf2,kk,ll,mm,nn)+zm4_3412(nf2,nf1,kk,ll,mm,nn))
     .                 +zi*(zm2_3412(nf2,nf1,ll,kk,mm,nn)+zm3_1234(nf1,nf2,ll,kk,mm,nn))

                   z2= +zi*(zm1_3412(nf2,nf1,ll,kk,mm,nn)+zm2_3412(nf2,nf1,ll,kk,mm,nn))
     .                  +zi*(zm3_3412(nf2,nf1,kk,ll,mm,nn)+zm4_3412(nf2,nf1,kk,ll,mm,nn))


                  A = A +2d0*real(z1*dconjg(z2))

               enddo
            enddo
         enddo
      enddo
           C1g0Zad = A


      return
      end


*****************************************************************************



      function C1g0Zac(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      use astore      
      use C1gZ!, only : genC1g0Zamp      
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

      complex*16 zm1_1234(2,2,2,2,2,2),zm2_1234(2,2,2,2,2,2),zm3_1234(2,2,2,2,2,2),zm4_1234(2,2,2,2,2,2)

c      complex*16 zampB1_1234(2,2,2,2,2,2),zampB2_1234(2,2,2,2,2,2)
c      complex*16 zampB1_3214(2,2,2,2,2,2),zampB2_3214(2,2,2,2,2,2)
      complex*16 zm1_3412(2,2,2,2,2,2),zm2_3412(2,2,2,2,2,2),zm3_3412(2,2,2,2,2,2),zm4_3412(2,2,2,2,2,2)

      call genC1g0Zamp1(i1,i4,i3,i2,i5,i6,i7,zm1_1234,zm2_1234,zm3_1234,zm4_1234)
c      call genC1g0Zamp(i4,i1,i3,i2,i5,i6,i7,zampB1_3214,zampB2_3214)
      call genC1g0Zamp1(i4,i1,i5,i2,i3,i6,i7,zm1_3412,zm2_3412,zm3_3412,zm4_3412)

      zi=(0d0,1d0)

           A = 0d0
      z1 = (0d0,0d0)
      z2 = (0d0,0d0)


      do kk=1,2
         do ll=1,2
            do mm=1,2
               do nn=1,2


                   z1=  +zi*(zm1_1234(nf1,nf2,kk,ll,mm,nn)+zm4_3412(nf2,nf1,kk,ll,mm,nn))
     .                 +zi*(zm2_3412(nf2,nf1,ll,kk,mm,nn)+zm3_1234(nf1,nf2,ll,kk,mm,nn))
                   z2=  +zi*(zm1_1234(nf1,nf2,kk,ll,mm,nn)+zm2_1234(nf1,nf2,kk,ll,mm,nn))
     .               +zi*(zm3_1234(nf1,nf2,ll,kk,mm,nn)+zm4_1234(nf1,nf2,ll,kk,mm,nn))


                  A = A +2d0*real(z1*dconjg(z2))

               enddo
            enddo
         enddo
      enddo

           C1g0Zac = A

      return
      end


*****************************************************************************



      function C1g0Zab(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      use astore      
      use C1gZ!, only : genC1g0Zamp      
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)

      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

      complex*16 zm1_1234(2,2,2,2,2,2),zm2_1234(2,2,2,2,2,2),zm3_1234(2,2,2,2,2,2),zm4_1234(2,2,2,2,2,2)

c      complex*16 zampB1_1234(2,2,2,2,2,2),zampB2_1234(2,2,2,2,2,2)
c      complex*16 zampB1_3214(2,2,2,2,2,2),zampB2_3214(2,2,2,2,2,2)
      complex*16 zm1_3412(2,2,2,2,2,2),zm2_3412(2,2,2,2,2,2),zm3_3412(2,2,2,2,2,2),zm4_3412(2,2,2,2,2,2)

      call genC1g0Zamp1(i1,i4,i3,i2,i5,i6,i7,zm1_1234,zm2_1234,zm3_1234,zm4_1234)
c      call genC1g0Zamp(i4,i1,i3,i2,i5,i6,i7,zampB1_3214,zampB2_3214)
      call genC1g0Zamp1(i4,i1,i5,i2,i3,i6,i7,zm1_3412,zm2_3412,zm3_3412,zm4_3412)

      zi=(0d0,1d0)

           A = 0d0
      z1 = (0d0,0d0)
      z2 = (0d0,0d0)

      do kk=1,2
         do ll=1,2
            do mm=1,2
               do nn=1,2

                   z1= +zi*(zm1_1234(nf1,nf2,kk,ll,mm,nn)+zm4_3412(nf2,nf1,kk,ll,mm,nn))
     .                 +zi*(zm2_3412(nf2,nf1,ll,kk,mm,nn)+zm3_1234(nf1,nf2,ll,kk,mm,nn))

                   z2= +zi*(zm2_1234(nf1,nf2,kk,ll,mm,nn)+zm4_1234(nf1,nf2,ll,kk,mm,nn))
     .                  +zi*(zm1_3412(nf2,nf1,ll,kk,mm,nn)+zm3_3412(nf2,nf1,kk,ll,mm,nn))


                  A = A +2d0*real(z1*dconjg(z2))

               enddo
            enddo
         enddo
      enddo

           C1g0Zab = A


      return
      end


*****************************************************************************



      function C1g0Zbd(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      use astore      
      use C1gZ!, only : genC1g0Zamp      
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)

      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

      complex*16 zm1_1234(2,2,2,2,2,2),zm2_1234(2,2,2,2,2,2),zm3_1234(2,2,2,2,2,2),zm4_1234(2,2,2,2,2,2)

c      complex*16 zampB1_1234(2,2,2,2,2,2),zampB2_1234(2,2,2,2,2,2)
c      complex*16 zampB1_3214(2,2,2,2,2,2),zampB2_3214(2,2,2,2,2,2)
      complex*16 zm1_3412(2,2,2,2,2,2),zm2_3412(2,2,2,2,2,2),zm3_3412(2,2,2,2,2,2),zm4_3412(2,2,2,2,2,2)

      call genC1g0Zamp1(i1,i4,i3,i2,i5,i6,i7,zm1_1234,zm2_1234,zm3_1234,zm4_1234)
c      call genC1g0Zamp(i4,i1,i3,i2,i5,i6,i7,zampB1_3214,zampB2_3214)
      call genC1g0Zamp1(i4,i1,i5,i2,i3,i6,i7,zm1_3412,zm2_3412,zm3_3412,zm4_3412)

      zi=(0d0,1d0)

           A = 0d0
      z1 = (0d0,0d0)
      z2 = (0d0,0d0)

      do kk=1,2
         do ll=1,2
            do mm=1,2
               do nn=1,2


                      z1= +zi*(zm2_1234(nf1,nf2,kk,ll,mm,nn)+zm4_1234(nf1,nf2,ll,kk,mm,nn))
     .                  +zi*(zm1_3412(nf2,nf1,ll,kk,mm,nn)+zm3_3412(nf2,nf1,kk,ll,mm,nn))

                      z2= +zi*(zm1_3412(nf2,nf1,ll,kk,mm,nn)+zm2_3412(nf2,nf1,ll,kk,mm,nn))
     .                  +zi*(zm3_3412(nf2,nf1,kk,ll,mm,nn)+zm4_3412(nf2,nf1,kk,ll,mm,nn))



                  A = A +2d0*real(z1*dconjg(z2))

               enddo
            enddo
         enddo
      enddo

           C1g0Zbd = A


      return
      end


*****************************************************************************



      function C1g0Zbc(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      use astore      
      use C1gZ!, only : genC1g0Zamp      
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

      complex*16 zm1_1234(2,2,2,2,2,2),zm2_1234(2,2,2,2,2,2),zm3_1234(2,2,2,2,2,2),zm4_1234(2,2,2,2,2,2)

c      complex*16 zampB1_1234(2,2,2,2,2,2),zampB2_1234(2,2,2,2,2,2)
c      complex*16 zampB1_3214(2,2,2,2,2,2),zampB2_3214(2,2,2,2,2,2)
      complex*16 zm1_3412(2,2,2,2,2,2),zm2_3412(2,2,2,2,2,2),zm3_3412(2,2,2,2,2,2),zm4_3412(2,2,2,2,2,2)

      call genC1g0Zamp1(i1,i4,i3,i2,i5,i6,i7,zm1_1234,zm2_1234,zm3_1234,zm4_1234)
c      call genC1g0Zamp(i4,i1,i3,i2,i5,i6,i7,zampB1_3214,zampB2_3214)
      call genC1g0Zamp1(i4,i1,i5,i2,i3,i6,i7,zm1_3412,zm2_3412,zm3_3412,zm4_3412)

      zi=(0d0,1d0)

           A = 0d0
      z1 = (0d0,0d0)
      z2 = (0d0,0d0)

      do kk=1,2
         do ll=1,2
            do mm=1,2
               do nn=1,2

                   z1 = +zi*(zm2_1234(nf1,nf2,kk,ll,mm,nn)+zm4_1234(nf1,nf2,ll,kk,mm,nn))
     .                  +zi*(zm1_3412(nf2,nf1,ll,kk,mm,nn)+zm3_3412(nf2,nf1,kk,ll,mm,nn))

                   z2 =  +zi*(zm1_1234(nf1,nf2,kk,ll,mm,nn)+zm2_1234(nf1,nf2,kk,ll,mm,nn))
     .               +zi*(zm3_1234(nf1,nf2,ll,kk,mm,nn)+zm4_1234(nf1,nf2,ll,kk,mm,nn))

                  A = A +2d0*real(z1*dconjg(z2))

               enddo
            enddo
         enddo
      enddo

           C1g0Zbc = A

      return
      end


*****************************************************************************


      function C1g0Zgam(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      use astore
      use C1gZ!, only : genC1g0Zamp      
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

      complex*16 zm1_1234(2,2,2,2,2,2),zm2_1234(2,2,2,2,2,2),zm3_1234(2,2,2,2,2,2),zm4_1234(2,2,2,2,2,2)

c      complex*16 zampB1_1234(2,2,2,2,2,2),zampB2_1234(2,2,2,2,2,2)
c      complex*16 zampB1_3214(2,2,2,2,2,2),zampB2_3214(2,2,2,2,2,2)
      complex*16 zm1_3412(2,2,2,2,2,2),zm2_3412(2,2,2,2,2,2),zm3_3412(2,2,2,2,2,2),zm4_3412(2,2,2,2,2,2)
      
      call genC1g0Zamp1(i1,i4,i3,i2,i5,i6,i7,zm1_1234,zm2_1234,zm3_1234,zm4_1234)

      call genC1g0Zamp1(i4,i1,i5,i2,i3,i6,i7,zm1_3412,zm2_3412,zm3_3412,zm4_3412)

      zi=(0d0,1d0)

           A = 0d0
      z1 = (0d0,0d0)
      z2 = (0d0,0d0)

      do kk=1,2
         do ll=1,2
            do mm=1,2
               do nn=1,2

                   z1 = +zi*(zm2_1234(nf1,nf2,kk,ll,mm,nn)+zm4_1234(nf1,nf2,ll,kk,mm,nn))
     .                  +zi*(zm1_3412(nf2,nf1,ll,kk,mm,nn)+zm3_3412(nf2,nf1,kk,ll,mm,nn))
     .                   +zi*(zm1_1234(nf1,nf2,kk,ll,mm,nn)+zm4_3412(nf2,nf1,kk,ll,mm,nn))
     .                 +zi*(zm2_3412(nf2,nf1,ll,kk,mm,nn)+zm3_1234(nf1,nf2,ll,kk,mm,nn))


                  A = A +real(z1*dconjg(z1))

               enddo
            enddo
         enddo
      enddo

           C1g0Zgam = A

      return
      end


*****************************************************************************


      function C1g0ZY(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      use astore
      use C1gZ!, only : genC1g0Zamp      
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)


      C1g0ZY=C1g0Zc(i1,i2,i3,i4,i5,i6,i7)
     .     +C1g0Zd(i1,i2,i3,i4,i5,i6,i7)
     .  - 2d0*C1g0Zgam(i1,i2,i3,i4,i5,i6,i7)      

      C1g0ZY= -C1g0ZY

      return
      end

*****************************************************************************


      function D1g0Za(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      use astore
      use C1gZ!, only : genC1g0Zamp      
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

      complex*16 zm1_1234(2,2,2,2,2,2),zm2_1234(2,2,2,2,2,2),zm3_1234(2,2,2,2,2,2),zm4_1234(2,2,2,2,2,2)

      complex*16 zm1_1432(2,2,2,2,2,2),zm2_1432(2,2,2,2,2,2),zm3_1432(2,2,2,2,2,2),zm4_1432(2,2,2,2,2,2)
      complex*16 zm1_3214(2,2,2,2,2,2),zm2_3214(2,2,2,2,2,2),zm3_3214(2,2,2,2,2,2),zm4_3214(2,2,2,2,2,2)
      complex*16 zm1_3412(2,2,2,2,2,2),zm2_3412(2,2,2,2,2,2),zm3_3412(2,2,2,2,2,2),zm4_3412(2,2,2,2,2,2)


      call genC1g0Zamp1(i1,i4,i3,i2,i5,i6,i7,zm1_1234,zm2_1234,zm3_1234,zm4_1234)
      call genC1g0Zamp1(i1,i4,i5,i2,i3,i6,i7,zm1_3214,zm2_3214,zm3_3214,zm4_3214)
      call genC1g0Zamp1(i4,i1,i5,i2,i3,i6,i7,zm1_3412,zm2_3412,zm3_3412,zm4_3412)
      call genC1g0Zamp1(i4,i1,i3,i2,i5,i6,i7,zm1_1432,zm2_1432,zm3_1432,zm4_1432)

c      call genC1g0Zamp1(i4,i1,i5,i2,i3,i6,i7,zampB11_1234,zampB22_1234)  2 4 exchange qb Qb 
c      call genC1g0Zamp1(i1,i4,i3,i2,i5,i6,i7,zampB11_3412,zampB22_3412)

      zi=(0d0,1d0)

           A = 0d0
      z1 = (0d0,0d0)
      z2 = (0d0,0d0)

      do ll=1,2
         do mm=1,2
            do nn=1,2

                   z1= +zi*(zm1_1234(nf1,nf2,ll,ll,mm,nn)+zm4_3412(nf2,nf1,ll,ll,mm,nn))
     .                 +zi*(zm2_3412(nf2,nf1,ll,ll,mm,nn)+zm3_1234(nf1,nf2,ll,ll,mm,nn))

                   z2= +zi*(zm1_3214(nf2,nf1,ll,ll,mm,nn)+zm4_1432(nf1,nf2,ll,ll,mm,nn))
     .                 +zi*(zm2_1432(nf1,nf2,ll,ll,mm,nn)+zm3_3214(nf2,nf1,ll,ll,mm,nn))     

               A = A+2d0*real(z1*dconjg(z2))

            enddo
         enddo
      enddo


           D1g0Za = A

      return
      end


******************************************************
 


      function D1g0Zb(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      use astore
      use C1gZ!, only : genC1g0Zamp      
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

      complex*16 zm1_1234(2,2,2,2,2,2),zm2_1234(2,2,2,2,2,2),zm3_1234(2,2,2,2,2,2),zm4_1234(2,2,2,2,2,2)

      complex*16 zm1_1432(2,2,2,2,2,2),zm2_1432(2,2,2,2,2,2),zm3_1432(2,2,2,2,2,2),zm4_1432(2,2,2,2,2,2)
      complex*16 zm1_3214(2,2,2,2,2,2),zm2_3214(2,2,2,2,2,2),zm3_3214(2,2,2,2,2,2),zm4_3214(2,2,2,2,2,2)
      complex*16 zm1_3412(2,2,2,2,2,2),zm2_3412(2,2,2,2,2,2),zm3_3412(2,2,2,2,2,2),zm4_3412(2,2,2,2,2,2)

      call genC1g0Zamp1(i1,i4,i3,i2,i5,i6,i7,zm1_1234,zm2_1234,zm3_1234,zm4_1234)
      call genC1g0Zamp1(i1,i4,i5,i2,i3,i6,i7,zm1_3214,zm2_3214,zm3_3214,zm4_3214)
      call genC1g0Zamp1(i4,i1,i5,i2,i3,i6,i7,zm1_3412,zm2_3412,zm3_3412,zm4_3412)
      call genC1g0Zamp1(i4,i1,i3,i2,i5,i6,i7,zm1_1432,zm2_1432,zm3_1432,zm4_1432)

c      call genC1g0Zamp1(i4,i1,i5,i2,i3,i6,i7,zampB11_1234,zampB22_1234)  2 4 exchange qb Qb 
c      call genC1g0Zamp1(i1,i4,i3,i2,i5,i6,i7,zampB11_3412,zampB22_3412)

      zi=(0d0,1d0)

           A = 0d0
      z1 = (0d0,0d0)
      z2 = (0d0,0d0) 

      do ll=1,2
         do mm=1,2
            do nn=1,2

                  
                   z1= +zi*(zm2_1234(nf1,nf2,ll,ll,mm,nn)+zm4_1234(nf1,nf2,ll,ll,mm,nn))
     .                  +zi*(zm1_3412(nf2,nf1,ll,ll,mm,nn)+zm3_3412(nf2,nf1,ll,ll,mm,nn))

                   z2=  +zi*(zm2_3214(nf2,nf1,ll,ll,mm,nn)+zm4_3214(nf2,nf1,ll,ll,mm,nn))
     .                  +zi*(zm1_1432(nf1,nf2,ll,ll,mm,nn)+zm3_1432(nf1,nf2,ll,ll,mm,nn))    

               A = A+2d0*real(z1*dconjg(z2))

            enddo
         enddo
      enddo


           D1g0Zb = A

      return
      end




***************************************************************
 


      function D1g0Zintc(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      use astore
      use C1gZ!, only : genC1g0Zamp      
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

      complex*16 zm1_1234(2,2,2,2,2,2),zm2_1234(2,2,2,2,2,2),zm3_1234(2,2,2,2,2,2),zm4_1234(2,2,2,2,2,2)

      complex*16 zm1_1432(2,2,2,2,2,2),zm2_1432(2,2,2,2,2,2),zm3_1432(2,2,2,2,2,2),zm4_1432(2,2,2,2,2,2)
      complex*16 zm1_3214(2,2,2,2,2,2),zm2_3214(2,2,2,2,2,2),zm3_3214(2,2,2,2,2,2),zm4_3214(2,2,2,2,2,2)
      complex*16 zm1_3412(2,2,2,2,2,2),zm2_3412(2,2,2,2,2,2),zm3_3412(2,2,2,2,2,2),zm4_3412(2,2,2,2,2,2)

      call genC1g0Zamp1(i1,i4,i3,i2,i5,i6,i7,zm1_1234,zm2_1234,zm3_1234,zm4_1234)
      call genC1g0Zamp1(i1,i4,i5,i2,i3,i6,i7,zm1_3214,zm2_3214,zm3_3214,zm4_3214)
      call genC1g0Zamp1(i4,i1,i5,i2,i3,i6,i7,zm1_3412,zm2_3412,zm3_3412,zm4_3412)
      call genC1g0Zamp1(i4,i1,i3,i2,i5,i6,i7,zm1_1432,zm2_1432,zm3_1432,zm4_1432)

c      call genC1g0Zamp1(i4,i1,i5,i2,i3,i6,i7,zampB11_1234,zampB22_1234)  2 4 exchange qb Qb 
c      call genC1g0Zamp1(i1,i4,i3,i2,i5,i6,i7,zampB11_3412,zampB22_3412)

      zi=(0d0,1d0)

           A = 0d0
      z1 = (0d0,0d0)
      z2 = (0d0,0d0)

      do ll=1,2
         do mm=1,2
            do nn=1,2

                   z1=  +zi*(zm1_1234(nf1,nf2,ll,ll,mm,nn)+zm2_1234(nf1,nf2,ll,ll,mm,nn))
     .               +zi*(zm3_1234(nf1,nf2,ll,ll,mm,nn)+zm4_1234(nf1,nf2,ll,ll,mm,nn))

                   z2=  +zi*(zm1_3214(nf2,nf1,ll,ll,mm,nn)+zm2_3214(nf2,nf1,ll,ll,mm,nn))
     .               +zi*(zm3_3214(nf2,nf1,ll,ll,mm,nn)+zm4_3214(nf2,nf1,ll,ll,mm,nn))  

               A = A+2d0*real(z1*dconjg(z2))

            enddo
         enddo
      enddo


           D1g0Zintc = A

      return
      end


**************************************************************
 


      function D1g0Zd(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      use astore
      use C1gZ!, only : genC1g0Zamp      
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

      complex*16 zm1_1234(2,2,2,2,2,2),zm2_1234(2,2,2,2,2,2),zm3_1234(2,2,2,2,2,2),zm4_1234(2,2,2,2,2,2)

      complex*16 zm1_1432(2,2,2,2,2,2),zm2_1432(2,2,2,2,2,2),zm3_1432(2,2,2,2,2,2),zm4_1432(2,2,2,2,2,2)
      complex*16 zm1_3214(2,2,2,2,2,2),zm2_3214(2,2,2,2,2,2),zm3_3214(2,2,2,2,2,2),zm4_3214(2,2,2,2,2,2)
      complex*16 zm1_3412(2,2,2,2,2,2),zm2_3412(2,2,2,2,2,2),zm3_3412(2,2,2,2,2,2),zm4_3412(2,2,2,2,2,2)

      call genC1g0Zamp1(i1,i4,i3,i2,i5,i6,i7,zm1_1234,zm2_1234,zm3_1234,zm4_1234)
      call genC1g0Zamp1(i1,i4,i5,i2,i3,i6,i7,zm1_3214,zm2_3214,zm3_3214,zm4_3214)
      call genC1g0Zamp1(i4,i1,i5,i2,i3,i6,i7,zm1_3412,zm2_3412,zm3_3412,zm4_3412)
      call genC1g0Zamp1(i4,i1,i3,i2,i5,i6,i7,zm1_1432,zm2_1432,zm3_1432,zm4_1432)

c      call genC1g0Zamp1(i4,i1,i5,i2,i3,i6,i7,zampB11_1234,zampB22_1234)  2 4 exchange qb Qb 
c      call genC1g0Zamp1(i1,i4,i3,i2,i5,i6,i7,zampB11_3412,zampB22_3412)

      zi=(0d0,1d0)

           A = 0d0
      z1 = (0d0,0d0)
      z2 = (0d0,0d0)

      do ll=1,2
         do mm=1,2
            do nn=1,2

                   z1= +zi*(zm1_3412(nf2,nf1,ll,ll,mm,nn)+zm2_3412(nf2,nf1,ll,ll,mm,nn))
     .                  +zi*(zm3_3412(nf2,nf1,ll,ll,mm,nn)+zm4_3412(nf2,nf1,ll,ll,mm,nn))

                   z2= +zi*(zm1_1432(nf1,nf2,ll,ll,mm,nn)+zm2_1432(nf1,nf2,ll,ll,mm,nn))
     .                  +zi*(zm3_1432(nf1,nf2,ll,ll,mm,nn)+zm4_1432(nf1,nf2,ll,ll,mm,nn))  

               A = A+2d0*real(z1*dconjg(z2))

            enddo
         enddo
      enddo


           D1g0Zd = A

      return
      end




*******************************************************************



      function D1g0Zab(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      use astore
      use C1gZ!, only : genC1g0Zamp      
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

      complex*16 zm1_1234(2,2,2,2,2,2),zm2_1234(2,2,2,2,2,2),zm3_1234(2,2,2,2,2,2),zm4_1234(2,2,2,2,2,2)
      complex*16 zm1_1432(2,2,2,2,2,2),zm2_1432(2,2,2,2,2,2),zm3_1432(2,2,2,2,2,2),zm4_1432(2,2,2,2,2,2)
      complex*16 zm1_3214(2,2,2,2,2,2),zm2_3214(2,2,2,2,2,2),zm3_3214(2,2,2,2,2,2),zm4_3214(2,2,2,2,2,2)
      complex*16 zm1_3412(2,2,2,2,2,2),zm2_3412(2,2,2,2,2,2),zm3_3412(2,2,2,2,2,2),zm4_3412(2,2,2,2,2,2)

      call genC1g0Zamp1(i1,i4,i3,i2,i5,i6,i7,zm1_1234,zm2_1234,zm3_1234,zm4_1234)
      call genC1g0Zamp1(i1,i4,i5,i2,i3,i6,i7,zm1_3214,zm2_3214,zm3_3214,zm4_3214)
      call genC1g0Zamp1(i4,i1,i5,i2,i3,i6,i7,zm1_3412,zm2_3412,zm3_3412,zm4_3412)
      call genC1g0Zamp1(i4,i1,i3,i2,i5,i6,i7,zm1_1432,zm2_1432,zm3_1432,zm4_1432)

c      call genC1g0Zamp1(i4,i1,i5,i2,i3,i6,i7,zampB11_1234,zampB22_1234)  2 4 exchange qb Qb 
c      call genC1g0Zamp1(i1,i4,i3,i2,i5,i6,i7,zampB11_3412,zampB22_3412)

      zi=(0d0,1d0)

           A = 0d0
      z1 = (0d0,0d0)
      z2 = (0d0,0d0)

      do ll=1,2
         do mm=1,2
            do nn=1,2

                   z1= +zi*(zm1_1234(nf1,nf2,ll,ll,mm,nn)+zm4_3412(nf2,nf1,ll,ll,mm,nn))
     .              +zi*(zm2_3412(nf2,nf1,ll,ll,mm,nn)+zm3_1234(nf1,nf2,ll,ll,mm,nn))

                   z2=  +zi*(zm2_3214(nf2,nf1,ll,ll,mm,nn)+zm4_3214(nf2,nf1,ll,ll,mm,nn))
     .                  +zi*(zm1_1432(nf1,nf2,ll,ll,mm,nn)+zm3_1432(nf1,nf2,ll,ll,mm,nn))

               A = A+2d0*real(z1*dconjg(z2))

            enddo
         enddo
      enddo


           D1g0Zab = A

      return
      end


*******************************************************************



      function D1g0Zac(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      use astore
      use C1gZ!, only : genC1g0Zamp      
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

      complex*16 zm1_1234(2,2,2,2,2,2),zm2_1234(2,2,2,2,2,2),zm3_1234(2,2,2,2,2,2),zm4_1234(2,2,2,2,2,2)
      complex*16 zm1_1432(2,2,2,2,2,2),zm2_1432(2,2,2,2,2,2),zm3_1432(2,2,2,2,2,2),zm4_1432(2,2,2,2,2,2)
      complex*16 zm1_3214(2,2,2,2,2,2),zm2_3214(2,2,2,2,2,2),zm3_3214(2,2,2,2,2,2),zm4_3214(2,2,2,2,2,2)
      complex*16 zm1_3412(2,2,2,2,2,2),zm2_3412(2,2,2,2,2,2),zm3_3412(2,2,2,2,2,2),zm4_3412(2,2,2,2,2,2)

      call genC1g0Zamp1(i1,i4,i3,i2,i5,i6,i7,zm1_1234,zm2_1234,zm3_1234,zm4_1234)
      call genC1g0Zamp1(i1,i4,i5,i2,i3,i6,i7,zm1_3214,zm2_3214,zm3_3214,zm4_3214)
      call genC1g0Zamp1(i4,i1,i5,i2,i3,i6,i7,zm1_3412,zm2_3412,zm3_3412,zm4_3412)
      call genC1g0Zamp1(i4,i1,i3,i2,i5,i6,i7,zm1_1432,zm2_1432,zm3_1432,zm4_1432)

c      call genC1g0Zamp1(i4,i1,i5,i2,i3,i6,i7,zampB11_1234,zampB22_1234)  2 4 exchange qb Qb 
c      call genC1g0Zamp1(i1,i4,i3,i2,i5,i6,i7,zampB11_3412,zampB22_3412)

      zi=(0d0,1d0)

           A = 0d0
      z1 = (0d0,0d0)
      z2 = (0d0,0d0)

      do ll=1,2
         do mm=1,2
            do nn=1,2

                   z1= +zi*(zm1_1234(nf1,nf2,ll,ll,mm,nn)+zm4_3412(nf2,nf1,ll,ll,mm,nn))
     .                 +zi*(zm2_3412(nf2,nf1,ll,ll,mm,nn)+zm3_1234(nf1,nf2,ll,ll,mm,nn))

                   z2= +zi*(zm1_3214(nf2,nf1,ll,ll,mm,nn)+zm2_3214(nf2,nf1,ll,ll,mm,nn))
     .               +zi*(zm3_3214(nf2,nf1,ll,ll,mm,nn)+zm4_3214(nf2,nf1,ll,ll,mm,nn))    

               A = A+2d0*real(z1*dconjg(z2))

            enddo
         enddo
      enddo


           D1g0Zac = A

      return
      end


*******************************************************************



      function D1g0Zad(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      use astore
      use C1gZ!, only : genC1g0Zamp      
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

      complex*16 zm1_1234(2,2,2,2,2,2),zm2_1234(2,2,2,2,2,2),zm3_1234(2,2,2,2,2,2),zm4_1234(2,2,2,2,2,2)
      complex*16 zm1_1432(2,2,2,2,2,2),zm2_1432(2,2,2,2,2,2),zm3_1432(2,2,2,2,2,2),zm4_1432(2,2,2,2,2,2)
      complex*16 zm1_3214(2,2,2,2,2,2),zm2_3214(2,2,2,2,2,2),zm3_3214(2,2,2,2,2,2),zm4_3214(2,2,2,2,2,2)
      complex*16 zm1_3412(2,2,2,2,2,2),zm2_3412(2,2,2,2,2,2),zm3_3412(2,2,2,2,2,2),zm4_3412(2,2,2,2,2,2)

      call genC1g0Zamp1(i1,i4,i3,i2,i5,i6,i7,zm1_1234,zm2_1234,zm3_1234,zm4_1234)
      call genC1g0Zamp1(i1,i4,i5,i2,i3,i6,i7,zm1_3214,zm2_3214,zm3_3214,zm4_3214)
      call genC1g0Zamp1(i4,i1,i5,i2,i3,i6,i7,zm1_3412,zm2_3412,zm3_3412,zm4_3412)
      call genC1g0Zamp1(i4,i1,i3,i2,i5,i6,i7,zm1_1432,zm2_1432,zm3_1432,zm4_1432)

c      call genC1g0Zamp1(i4,i1,i5,i2,i3,i6,i7,zampB11_1234,zampB22_1234)  2 4 exchange qb Qb 
c      call genC1g0Zamp1(i1,i4,i3,i2,i5,i6,i7,zampB11_3412,zampB22_3412)

      zi=(0d0,1d0)

           A = 0d0
      z1 = (0d0,0d0)
      z2 = (0d0,0d0)

      do ll=1,2
         do mm=1,2
            do nn=1,2

                   z1= +zi*(zm1_1234(nf1,nf2,ll,ll,mm,nn)+zm4_3412(nf2,nf1,ll,ll,mm,nn))
     .                +zi*(zm2_3412(nf2,nf1,ll,ll,mm,nn)+zm3_1234(nf1,nf2,ll,ll,mm,nn))

                   z2= +zi*(zm1_1432(nf1,nf2,ll,ll,mm,nn)+zm2_1432(nf1,nf2,ll,ll,mm,nn))
     .                  +zi*(zm3_1432(nf1,nf2,ll,ll,mm,nn)+zm4_1432(nf1,nf2,ll,ll,mm,nn))  

               A = A+2d0*real(z1*dconjg(z2))

            enddo
         enddo
      enddo


           D1g0Zad = A

      return
      end


*******************************************************************



      function D1g0Zba(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      use astore
      use C1gZ!, only : genC1g0Zamp      
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

      complex*16 zm1_1234(2,2,2,2,2,2),zm2_1234(2,2,2,2,2,2),zm3_1234(2,2,2,2,2,2),zm4_1234(2,2,2,2,2,2)

      complex*16 zm1_1432(2,2,2,2,2,2),zm2_1432(2,2,2,2,2,2),zm3_1432(2,2,2,2,2,2),zm4_1432(2,2,2,2,2,2)
      complex*16 zm1_3214(2,2,2,2,2,2),zm2_3214(2,2,2,2,2,2),zm3_3214(2,2,2,2,2,2),zm4_3214(2,2,2,2,2,2)
      complex*16 zm1_3412(2,2,2,2,2,2),zm2_3412(2,2,2,2,2,2),zm3_3412(2,2,2,2,2,2),zm4_3412(2,2,2,2,2,2)

      call genC1g0Zamp1(i1,i4,i3,i2,i5,i6,i7,zm1_1234,zm2_1234,zm3_1234,zm4_1234)
      call genC1g0Zamp1(i1,i4,i5,i2,i3,i6,i7,zm1_3214,zm2_3214,zm3_3214,zm4_3214)
      call genC1g0Zamp1(i4,i1,i5,i2,i3,i6,i7,zm1_3412,zm2_3412,zm3_3412,zm4_3412)
      call genC1g0Zamp1(i4,i1,i3,i2,i5,i6,i7,zm1_1432,zm2_1432,zm3_1432,zm4_1432)

c      call genC1g0Zamp1(i4,i1,i5,i2,i3,i6,i7,zampB11_1234,zampB22_1234)  2 4 exchange qb Qb 
c      call genC1g0Zamp1(i1,i4,i3,i2,i5,i6,i7,zampB11_3412,zampB22_3412)

      zi=(0d0,1d0)

           A = 0d0
      z1 = (0d0,0d0)
      z2 = (0d0,0d0)

      do ll=1,2
         do mm=1,2
            do nn=1,2

                   z1= +zi*(zm2_1234(nf1,nf2,ll,ll,mm,nn)+zm4_1234(nf1,nf2,ll,ll,mm,nn))
     .                  +zi*(zm1_3412(nf2,nf1,ll,ll,mm,nn)+zm3_3412(nf2,nf1,ll,ll,mm,nn))

                   z2=  +zi*(zm1_3214(nf2,nf1,ll,ll,mm,nn)+zm4_1432(nf1,nf2,ll,ll,mm,nn))
     .                 +zi*(zm2_1432(nf1,nf2,ll,ll,mm,nn)+zm3_3214(nf2,nf1,ll,ll,mm,nn))   

               A = A+2d0*real(z1*dconjg(z2))

            enddo
         enddo
      enddo


           D1g0Zba = A

      return
      end


*******************************************************************



      function D1g0Zbc(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      use astore
      use C1gZ!, only : genC1g0Zamp      
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

      complex*16 zm1_1234(2,2,2,2,2,2),zm2_1234(2,2,2,2,2,2),zm3_1234(2,2,2,2,2,2),zm4_1234(2,2,2,2,2,2)
      complex*16 zm1_1432(2,2,2,2,2,2),zm2_1432(2,2,2,2,2,2),zm3_1432(2,2,2,2,2,2),zm4_1432(2,2,2,2,2,2)
      complex*16 zm1_3214(2,2,2,2,2,2),zm2_3214(2,2,2,2,2,2),zm3_3214(2,2,2,2,2,2),zm4_3214(2,2,2,2,2,2)
      complex*16 zm1_3412(2,2,2,2,2,2),zm2_3412(2,2,2,2,2,2),zm3_3412(2,2,2,2,2,2),zm4_3412(2,2,2,2,2,2)

      call genC1g0Zamp1(i1,i4,i3,i2,i5,i6,i7,zm1_1234,zm2_1234,zm3_1234,zm4_1234)
      call genC1g0Zamp1(i1,i4,i5,i2,i3,i6,i7,zm1_3214,zm2_3214,zm3_3214,zm4_3214)
      call genC1g0Zamp1(i4,i1,i5,i2,i3,i6,i7,zm1_3412,zm2_3412,zm3_3412,zm4_3412)
      call genC1g0Zamp1(i4,i1,i3,i2,i5,i6,i7,zm1_1432,zm2_1432,zm3_1432,zm4_1432)

c      call genC1g0Zamp1(i4,i1,i5,i2,i3,i6,i7,zampB11_1234,zampB22_1234)  2 4 exchange qb Qb 
c      call genC1g0Zamp1(i1,i4,i3,i2,i5,i6,i7,zampB11_3412,zampB22_3412)

      zi=(0d0,1d0)

           A = 0d0
      z1 = (0d0,0d0)
      z2 = (0d0,0d0)

      do ll=1,2
         do mm=1,2
            do nn=1,2

                   z1= +zi*(zm2_1234(nf1,nf2,ll,ll,mm,nn)+zm4_1234(nf1,nf2,ll,ll,mm,nn))
     .                  +zi*(zm1_3412(nf2,nf1,ll,ll,mm,nn)+zm3_3412(nf2,nf1,ll,ll,mm,nn))

                   z2= +zi*(zm1_3214(nf2,nf1,ll,ll,mm,nn)+zm2_3214(nf2,nf1,ll,ll,mm,nn))
     .               +zi*(zm3_3214(nf2,nf1,ll,ll,mm,nn)+zm4_3214(nf2,nf1,ll,ll,mm,nn))    

               A = A+2d0*real(z1*dconjg(z2))

            enddo
         enddo
      enddo


           D1g0Zbc = A

      return
      end


*******************************************************************



      function D1g0Zbd(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      use astore
      use C1gZ!, only : genC1g0Zamp      
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

      complex*16 zm1_1234(2,2,2,2,2,2),zm2_1234(2,2,2,2,2,2),zm3_1234(2,2,2,2,2,2),zm4_1234(2,2,2,2,2,2)

      complex*16 zm1_1432(2,2,2,2,2,2),zm2_1432(2,2,2,2,2,2),zm3_1432(2,2,2,2,2,2),zm4_1432(2,2,2,2,2,2)
      complex*16 zm1_3214(2,2,2,2,2,2),zm2_3214(2,2,2,2,2,2),zm3_3214(2,2,2,2,2,2),zm4_3214(2,2,2,2,2,2)
      complex*16 zm1_3412(2,2,2,2,2,2),zm2_3412(2,2,2,2,2,2),zm3_3412(2,2,2,2,2,2),zm4_3412(2,2,2,2,2,2)

      call genC1g0Zamp1(i1,i4,i3,i2,i5,i6,i7,zm1_1234,zm2_1234,zm3_1234,zm4_1234)
      call genC1g0Zamp1(i1,i4,i5,i2,i3,i6,i7,zm1_3214,zm2_3214,zm3_3214,zm4_3214)
      call genC1g0Zamp1(i4,i1,i5,i2,i3,i6,i7,zm1_3412,zm2_3412,zm3_3412,zm4_3412)
      call genC1g0Zamp1(i4,i1,i3,i2,i5,i6,i7,zm1_1432,zm2_1432,zm3_1432,zm4_1432)

c      call genC1g0Zamp1(i4,i1,i5,i2,i3,i6,i7,zampB11_1234,zampB22_1234)  2 4 exchange qb Qb 
c      call genC1g0Zamp1(i1,i4,i3,i2,i5,i6,i7,zampB11_3412,zampB22_3412)

      zi=(0d0,1d0)

           A = 0d0
      z1 = (0d0,0d0)
      z2 = (0d0,0d0)

      do ll=1,2
         do mm=1,2
            do nn=1,2

                   z1= +zi*(zm2_1234(nf1,nf2,ll,ll,mm,nn)+zm4_1234(nf1,nf2,ll,ll,mm,nn))
     .                  +zi*(zm1_3412(nf2,nf1,ll,ll,mm,nn)+zm3_3412(nf2,nf1,ll,ll,mm,nn))

                   z2= +zi*(zm1_1432(nf1,nf2,ll,ll,mm,nn)+zm2_1432(nf1,nf2,ll,ll,mm,nn))
     .                  +zi*(zm3_1432(nf1,nf2,ll,ll,mm,nn)+zm4_1432(nf1,nf2,ll,ll,mm,nn))  

               A = A+2d0*real(z1*dconjg(z2))

            enddo
         enddo
      enddo


           D1g0Zbd = A

      return
      end


*******************************************************************



      function D1g0Zca(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      use astore
      use C1gZ!, only : genC1g0Zamp      
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

      complex*16 zm1_1234(2,2,2,2,2,2),zm2_1234(2,2,2,2,2,2),zm3_1234(2,2,2,2,2,2),zm4_1234(2,2,2,2,2,2)

      complex*16 zm1_1432(2,2,2,2,2,2),zm2_1432(2,2,2,2,2,2),zm3_1432(2,2,2,2,2,2),zm4_1432(2,2,2,2,2,2)
      complex*16 zm1_3214(2,2,2,2,2,2),zm2_3214(2,2,2,2,2,2),zm3_3214(2,2,2,2,2,2),zm4_3214(2,2,2,2,2,2)
      complex*16 zm1_3412(2,2,2,2,2,2),zm2_3412(2,2,2,2,2,2),zm3_3412(2,2,2,2,2,2),zm4_3412(2,2,2,2,2,2)

      call genC1g0Zamp1(i1,i4,i3,i2,i5,i6,i7,zm1_1234,zm2_1234,zm3_1234,zm4_1234)
      call genC1g0Zamp1(i1,i4,i5,i2,i3,i6,i7,zm1_3214,zm2_3214,zm3_3214,zm4_3214)
      call genC1g0Zamp1(i4,i1,i5,i2,i3,i6,i7,zm1_3412,zm2_3412,zm3_3412,zm4_3412)
      call genC1g0Zamp1(i4,i1,i3,i2,i5,i6,i7,zm1_1432,zm2_1432,zm3_1432,zm4_1432)

c      call genC1g0Zamp1(i4,i1,i5,i2,i3,i6,i7,zampB11_1234,zampB22_1234)  2 4 exchange qb Qb 
c      call genC1g0Zamp1(i1,i4,i3,i2,i5,i6,i7,zampB11_3412,zampB22_3412)

      zi=(0d0,1d0)

           A = 0d0
      z1 = (0d0,0d0)
      z2 = (0d0,0d0)

      do ll=1,2
         do mm=1,2
            do nn=1,2

                   z1= +zi*(zm1_1234(nf1,nf2,ll,ll,mm,nn)+zm2_1234(nf1,nf2,ll,ll,mm,nn))
     .               +zi*(zm3_1234(nf1,nf2,ll,ll,mm,nn)+zm4_1234(nf1,nf2,ll,ll,mm,nn))

                   z2=  +zi*(zm1_3214(nf2,nf1,ll,ll,mm,nn)+zm4_1432(nf1,nf2,ll,ll,mm,nn))
     .                 +zi*(zm2_1432(nf1,nf2,ll,ll,mm,nn)+zm3_3214(nf2,nf1,ll,ll,mm,nn))   

               A = A+2d0*real(z1*dconjg(z2))

            enddo
         enddo
      enddo


           D1g0Zca = A

      return
      end


*******************************************************************



      function D1g0Zcb(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      use astore
      use C1gZ!, only : genC1g0Zamp      
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

      complex*16 zm1_1234(2,2,2,2,2,2),zm2_1234(2,2,2,2,2,2),zm3_1234(2,2,2,2,2,2),zm4_1234(2,2,2,2,2,2)

      complex*16 zm1_1432(2,2,2,2,2,2),zm2_1432(2,2,2,2,2,2),zm3_1432(2,2,2,2,2,2),zm4_1432(2,2,2,2,2,2)
      complex*16 zm1_3214(2,2,2,2,2,2),zm2_3214(2,2,2,2,2,2),zm3_3214(2,2,2,2,2,2),zm4_3214(2,2,2,2,2,2)
      complex*16 zm1_3412(2,2,2,2,2,2),zm2_3412(2,2,2,2,2,2),zm3_3412(2,2,2,2,2,2),zm4_3412(2,2,2,2,2,2)

      call genC1g0Zamp1(i1,i4,i3,i2,i5,i6,i7,zm1_1234,zm2_1234,zm3_1234,zm4_1234)
      call genC1g0Zamp1(i1,i4,i5,i2,i3,i6,i7,zm1_3214,zm2_3214,zm3_3214,zm4_3214)
      call genC1g0Zamp1(i4,i1,i5,i2,i3,i6,i7,zm1_3412,zm2_3412,zm3_3412,zm4_3412)
      call genC1g0Zamp1(i4,i1,i3,i2,i5,i6,i7,zm1_1432,zm2_1432,zm3_1432,zm4_1432)

c      call genC1g0Zamp1(i4,i1,i5,i2,i3,i6,i7,zampB11_1234,zampB22_1234)  2 4 exchange qb Qb 
c      call genC1g0Zamp1(i1,i4,i3,i2,i5,i6,i7,zampB11_3412,zampB22_3412)

      zi=(0d0,1d0)

           A = 0d0
      z1 = (0d0,0d0)
      z2 = (0d0,0d0)

      do ll=1,2
         do mm=1,2
            do nn=1,2

                   z1=+zi*(zm1_1234(nf1,nf2,ll,ll,mm,nn)+zm2_1234(nf1,nf2,ll,ll,mm,nn))
     .               +zi*(zm3_1234(nf1,nf2,ll,ll,mm,nn)+zm4_1234(nf1,nf2,ll,ll,mm,nn))

                   z2= +zi*(zm2_3214(nf2,nf1,ll,ll,mm,nn)+zm4_3214(nf2,nf1,ll,ll,mm,nn))
     .                  +zi*(zm1_1432(nf1,nf2,ll,ll,mm,nn)+zm3_1432(nf1,nf2,ll,ll,mm,nn))    

               A = A+2d0*real(z1*dconjg(z2))

            enddo
         enddo
      enddo


           D1g0Zcb = A

      return
      end


*******************************************************************



      function D1g0Zcd(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      use astore
      use C1gZ!, only : genC1g0Zamp      
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

      complex*16 zm1_1234(2,2,2,2,2,2),zm2_1234(2,2,2,2,2,2),zm3_1234(2,2,2,2,2,2),zm4_1234(2,2,2,2,2,2)
      complex*16 zm1_1432(2,2,2,2,2,2),zm2_1432(2,2,2,2,2,2),zm3_1432(2,2,2,2,2,2),zm4_1432(2,2,2,2,2,2)
      complex*16 zm1_3214(2,2,2,2,2,2),zm2_3214(2,2,2,2,2,2),zm3_3214(2,2,2,2,2,2),zm4_3214(2,2,2,2,2,2)
      complex*16 zm1_3412(2,2,2,2,2,2),zm2_3412(2,2,2,2,2,2),zm3_3412(2,2,2,2,2,2),zm4_3412(2,2,2,2,2,2)

      call genC1g0Zamp1(i1,i4,i3,i2,i5,i6,i7,zm1_1234,zm2_1234,zm3_1234,zm4_1234)
      call genC1g0Zamp1(i1,i4,i5,i2,i3,i6,i7,zm1_3214,zm2_3214,zm3_3214,zm4_3214)
      call genC1g0Zamp1(i4,i1,i5,i2,i3,i6,i7,zm1_3412,zm2_3412,zm3_3412,zm4_3412)
      call genC1g0Zamp1(i4,i1,i3,i2,i5,i6,i7,zm1_1432,zm2_1432,zm3_1432,zm4_1432)

c      call genC1g0Zamp1(i4,i1,i5,i2,i3,i6,i7,zampB11_1234,zampB22_1234)  2 4 exchange qb Qb 
c      call genC1g0Zamp1(i1,i4,i3,i2,i5,i6,i7,zampB11_3412,zampB22_3412)

      zi=(0d0,1d0)

           A = 0d0
      z1 = (0d0,0d0)
      z2 = (0d0,0d0)

      do ll=1,2
         do mm=1,2
            do nn=1,2

                   z1= +zi*(zm1_1234(nf1,nf2,ll,ll,mm,nn)+zm2_1234(nf1,nf2,ll,ll,mm,nn))
     .               +zi*(zm3_1234(nf1,nf2,ll,ll,mm,nn)+zm4_1234(nf1,nf2,ll,ll,mm,nn))

                   z2=  +zi*(zm1_1432(nf1,nf2,ll,ll,mm,nn)+zm2_1432(nf1,nf2,ll,ll,mm,nn))
     .                  +zi*(zm3_1432(nf1,nf2,ll,ll,mm,nn)+zm4_1432(nf1,nf2,ll,ll,mm,nn)) 

               A = A+2d0*real(z1*dconjg(z2))

            enddo
         enddo
      enddo


           D1g0Zcd = A

      return
      end


*******************************************************************



      function D1g0Zda(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      use astore
      use C1gZ!, only : genC1g0Zamp      
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

      complex*16 zm1_1234(2,2,2,2,2,2),zm2_1234(2,2,2,2,2,2),zm3_1234(2,2,2,2,2,2),zm4_1234(2,2,2,2,2,2)
      complex*16 zm1_1432(2,2,2,2,2,2),zm2_1432(2,2,2,2,2,2),zm3_1432(2,2,2,2,2,2),zm4_1432(2,2,2,2,2,2)
      complex*16 zm1_3214(2,2,2,2,2,2),zm2_3214(2,2,2,2,2,2),zm3_3214(2,2,2,2,2,2),zm4_3214(2,2,2,2,2,2)
      complex*16 zm1_3412(2,2,2,2,2,2),zm2_3412(2,2,2,2,2,2),zm3_3412(2,2,2,2,2,2),zm4_3412(2,2,2,2,2,2)

      call genC1g0Zamp1(i1,i4,i3,i2,i5,i6,i7,zm1_1234,zm2_1234,zm3_1234,zm4_1234)
      call genC1g0Zamp1(i1,i4,i5,i2,i3,i6,i7,zm1_3214,zm2_3214,zm3_3214,zm4_3214)
      call genC1g0Zamp1(i4,i1,i5,i2,i3,i6,i7,zm1_3412,zm2_3412,zm3_3412,zm4_3412)
      call genC1g0Zamp1(i4,i1,i3,i2,i5,i6,i7,zm1_1432,zm2_1432,zm3_1432,zm4_1432)

c      call genC1g0Zamp1(i4,i1,i5,i2,i3,i6,i7,zampB11_1234,zampB22_1234)  2 4 exchange qb Qb 
c      call genC1g0Zamp1(i1,i4,i3,i2,i5,i6,i7,zampB11_3412,zampB22_3412)

      zi=(0d0,1d0)

           A = 0d0
      z1 = (0d0,0d0)
      z2 = (0d0,0d0)

      do ll=1,2
         do mm=1,2
            do nn=1,2 

                   z1= +zi*(zm1_3412(nf2,nf1,ll,ll,mm,nn)+zm2_3412(nf2,nf1,ll,ll,mm,nn))
     .                  +zi*(zm3_3412(nf2,nf1,ll,ll,mm,nn)+zm4_3412(nf2,nf1,ll,ll,mm,nn))

                   z2= +zi*(zm1_3214(nf2,nf1,ll,ll,mm,nn)+zm4_1432(nf1,nf2,ll,ll,mm,nn))
     .                 +zi*(zm2_1432(nf1,nf2,ll,ll,mm,nn)+zm3_3214(nf2,nf1,ll,ll,mm,nn)) 

               A = A+2d0*real(z1*dconjg(z2))

            enddo
         enddo
      enddo


           D1g0Zda = A

      return
      end


*******************************************************************



      function D1g0Zdb(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      use astore
      use C1gZ!, only : genC1g0Zamp      
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

      complex*16 zm1_1234(2,2,2,2,2,2),zm2_1234(2,2,2,2,2,2),zm3_1234(2,2,2,2,2,2),zm4_1234(2,2,2,2,2,2)

      complex*16 zm1_1432(2,2,2,2,2,2),zm2_1432(2,2,2,2,2,2),zm3_1432(2,2,2,2,2,2),zm4_1432(2,2,2,2,2,2)
      complex*16 zm1_3214(2,2,2,2,2,2),zm2_3214(2,2,2,2,2,2),zm3_3214(2,2,2,2,2,2),zm4_3214(2,2,2,2,2,2)
      complex*16 zm1_3412(2,2,2,2,2,2),zm2_3412(2,2,2,2,2,2),zm3_3412(2,2,2,2,2,2),zm4_3412(2,2,2,2,2,2)

      call genC1g0Zamp1(i1,i4,i3,i2,i5,i6,i7,zm1_1234,zm2_1234,zm3_1234,zm4_1234)
      call genC1g0Zamp1(i1,i4,i5,i2,i3,i6,i7,zm1_3214,zm2_3214,zm3_3214,zm4_3214)
      call genC1g0Zamp1(i4,i1,i5,i2,i3,i6,i7,zm1_3412,zm2_3412,zm3_3412,zm4_3412)
      call genC1g0Zamp1(i4,i1,i3,i2,i5,i6,i7,zm1_1432,zm2_1432,zm3_1432,zm4_1432)

c      call genC1g0Zamp1(i4,i1,i5,i2,i3,i6,i7,zampB11_1234,zampB22_1234)  2 4 exchange qb Qb 
c      call genC1g0Zamp1(i1,i4,i3,i2,i5,i6,i7,zampB11_3412,zampB22_3412)

      zi=(0d0,1d0)

           A = 0d0
      z1 = (0d0,0d0)
      z2 = (0d0,0d0)

      do ll=1,2
         do mm=1,2
            do nn=1,2

                   z1= +zi*(zm1_3412(nf2,nf1,ll,ll,mm,nn)+zm2_3412(nf2,nf1,ll,ll,mm,nn))
     .                  +zi*(zm3_3412(nf2,nf1,ll,ll,mm,nn)+zm4_3412(nf2,nf1,ll,ll,mm,nn))

                   z2= +zi*(zm2_3214(nf2,nf1,ll,ll,mm,nn)+zm4_3214(nf2,nf1,ll,ll,mm,nn))
     .                  +zi*(zm1_1432(nf1,nf2,ll,ll,mm,nn)+zm3_1432(nf1,nf2,ll,ll,mm,nn))    


               A = A+2d0*real(z1*dconjg(z2))

            enddo
         enddo
      enddo


           D1g0Zdb = A

      return
      end


*******************************************************************




      function D1g0Zdc(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      use astore
      use C1gZ!, only : genC1g0Zamp      
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

      complex*16 zm1_1234(2,2,2,2,2,2),zm2_1234(2,2,2,2,2,2),zm3_1234(2,2,2,2,2,2),zm4_1234(2,2,2,2,2,2)
      complex*16 zm1_1432(2,2,2,2,2,2),zm2_1432(2,2,2,2,2,2),zm3_1432(2,2,2,2,2,2),zm4_1432(2,2,2,2,2,2)
      complex*16 zm1_3214(2,2,2,2,2,2),zm2_3214(2,2,2,2,2,2),zm3_3214(2,2,2,2,2,2),zm4_3214(2,2,2,2,2,2)
      complex*16 zm1_3412(2,2,2,2,2,2),zm2_3412(2,2,2,2,2,2),zm3_3412(2,2,2,2,2,2),zm4_3412(2,2,2,2,2,2)

      call genC1g0Zamp1(i1,i4,i3,i2,i5,i6,i7,zm1_1234,zm2_1234,zm3_1234,zm4_1234)
      call genC1g0Zamp1(i1,i4,i5,i2,i3,i6,i7,zm1_3214,zm2_3214,zm3_3214,zm4_3214)
      call genC1g0Zamp1(i4,i1,i5,i2,i3,i6,i7,zm1_3412,zm2_3412,zm3_3412,zm4_3412)
      call genC1g0Zamp1(i4,i1,i3,i2,i5,i6,i7,zm1_1432,zm2_1432,zm3_1432,zm4_1432)

c      call genC1g0Zamp1(i4,i1,i5,i2,i3,i6,i7,zampB11_1234,zampB22_1234)  2 4 exchange qb Qb 
c      call genC1g0Zamp1(i1,i4,i3,i2,i5,i6,i7,zampB11_3412,zampB22_3412)

      zi=(0d0,1d0)

           A = 0d0
      z1 = (0d0,0d0)
      z2 = (0d0,0d0)

      do ll=1,2
         do mm=1,2
            do nn=1,2

                   z1= +zi*(zm1_3412(nf2,nf1,ll,ll,mm,nn)+zm2_3412(nf2,nf1,ll,ll,mm,nn))
     .                  +zi*(zm3_3412(nf2,nf1,ll,ll,mm,nn)+zm4_3412(nf2,nf1,ll,ll,mm,nn))

                   z2= +zi*(zm1_3214(nf2,nf1,ll,ll,mm,nn)+zm2_3214(nf2,nf1,ll,ll,mm,nn))
     .               +zi*(zm3_3214(nf2,nf1,ll,ll,mm,nn)+zm4_3214(nf2,nf1,ll,ll,mm,nn))    


               A = A+2d0*real(z1*dconjg(z2))

            enddo
         enddo
      enddo


           D1g0Zdc = A

      return
      end


*******************************************************************


      function D1g0Zgam(i1,i2,i3,i4,i5,i6,i7)

      use KinData_mod
      use astore
      use C1gZ!, only : genC1g0Zamp      
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

      complex*16 zm1_1234(2,2,2,2,2,2),zm2_1234(2,2,2,2,2,2),zm3_1234(2,2,2,2,2,2),zm4_1234(2,2,2,2,2,2)
      complex*16 zm1_1432(2,2,2,2,2,2),zm2_1432(2,2,2,2,2,2),zm3_1432(2,2,2,2,2,2),zm4_1432(2,2,2,2,2,2)
      complex*16 zm1_3214(2,2,2,2,2,2),zm2_3214(2,2,2,2,2,2),zm3_3214(2,2,2,2,2,2),zm4_3214(2,2,2,2,2,2)
      complex*16 zm1_3412(2,2,2,2,2,2),zm2_3412(2,2,2,2,2,2),zm3_3412(2,2,2,2,2,2),zm4_3412(2,2,2,2,2,2)

      call genC1g0Zamp1(i1,i4,i3,i2,i5,i6,i7,zm1_1234,zm2_1234,zm3_1234,zm4_1234)
      call genC1g0Zamp1(i1,i4,i5,i2,i3,i6,i7,zm1_3214,zm2_3214,zm3_3214,zm4_3214)
      call genC1g0Zamp1(i4,i1,i5,i2,i3,i6,i7,zm1_3412,zm2_3412,zm3_3412,zm4_3412)
      call genC1g0Zamp1(i4,i1,i3,i2,i5,i6,i7,zm1_1432,zm2_1432,zm3_1432,zm4_1432)

c      call genC1g0Zamp1(i4,i1,i5,i2,i3,i6,i7,zampB11_1234,zampB22_1234) i3 i5 exchange qb Qb 
c      call genC1g0Zamp1(i1,i4,i3,i2,i5,i6,i7,zampB11_3412,zampB22_3412)

      zi=(0d0,1d0)

           A = 0d0
      z1 = (0d0,0d0)
      z2 = (0d0,0d0)
      do ll=1,2
         do mm=1,2
            do nn=1,2

                   z1= +zi*(zm2_1234(nf1,nf2,ll,ll,mm,nn)+zm4_1234(nf1,nf2,ll,ll,mm,nn))
     .                  +zi*(zm1_3412(nf2,nf1,ll,ll,mm,nn)+zm3_3412(nf2,nf1,ll,ll,mm,nn))
     .                   +zi*(zm1_1234(nf1,nf2,ll,ll,mm,nn)+zm4_3412(nf2,nf1,ll,ll,mm,nn))
     .                 +zi*(zm2_3412(nf2,nf1,ll,ll,mm,nn)+zm3_1234(nf1,nf2,ll,ll,mm,nn))

                   z2= +zi*(zm2_3214(nf2,nf1,ll,ll,mm,nn)+zm4_3214(nf2,nf1,ll,ll,mm,nn))
     .                  +zi*(zm1_1432(nf1,nf2,ll,ll,mm,nn)+zm3_1432(nf1,nf2,ll,ll,mm,nn))
     .                   +zi*(zm1_3214(nf2,nf1,ll,ll,mm,nn)+zm4_1432(nf1,nf2,ll,ll,mm,nn))
     .                 +zi*(zm2_1432(nf1,nf2,ll,ll,mm,nn)+zm3_3214(nf2,nf1,ll,ll,mm,nn))  



                   A = A+2d0*real(z2*dconjg(z1))


            enddo
         enddo
      enddo


           D1g0Zgam = A

      return
      end

********************************************************************************
      function D1g0ZX(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      use astore
      use C1gZ!, only : genC1g0Zamp      
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

      complex*16 zm1_1234(2,2,2,2,2,2),zm2_1234(2,2,2,2,2,2),zm3_1234(2,2,2,2,2,2),zm4_1234(2,2,2,2,2,2)
      complex*16 zm1_1432(2,2,2,2,2,2),zm2_1432(2,2,2,2,2,2),zm3_1432(2,2,2,2,2,2),zm4_1432(2,2,2,2,2,2)
      complex*16 zm1_3214(2,2,2,2,2,2),zm2_3214(2,2,2,2,2,2),zm3_3214(2,2,2,2,2,2),zm4_3214(2,2,2,2,2,2)
      complex*16 zm1_3412(2,2,2,2,2,2),zm2_3412(2,2,2,2,2,2),zm3_3412(2,2,2,2,2,2),zm4_3412(2,2,2,2,2,2)

      call genC1g0Zamp1(i1,i4,i3,i2,i5,i6,i7,zm1_1234,zm2_1234,zm3_1234,zm4_1234)
      call genC1g0Zamp1(i1,i4,i5,i2,i3,i6,i7,zm1_3214,zm2_3214,zm3_3214,zm4_3214)
      call genC1g0Zamp1(i4,i1,i5,i2,i3,i6,i7,zm1_3412,zm2_3412,zm3_3412,zm4_3412)
      call genC1g0Zamp1(i4,i1,i3,i2,i5,i6,i7,zm1_1432,zm2_1432,zm3_1432,zm4_1432)

c      call genC1g0Zamp1(i4,i1,i5,i2,i3,i6,i7,zampB11_1234,zampB22_1234)  2 4 exchange qb Qb 
c      call genC1g0Zamp1(i1,i4,i3,i2,i5,i6,i7,zampB11_3412,zampB22_3412)

      zi=(0d0,1d0)

           A = 0d0
           A1 = 0d0
      zc = (0d0,0d0)
      za = (0d0,0d0)
      zd = (0d0,0d0)
      zb = (0d0,0d0)

      zc1 = (0d0,0d0)
      za1 = (0d0,0d0)
      zd1 = (0d0,0d0)
      zb1 = (0d0,0d0)

      do ll=1,2
         do mm=1,2
            do nn=1,2

            
                  zc = +zi*(zm1_1234(nf1,nf2,ll,ll,mm,nn)+zm2_1234(nf1,nf2,ll,ll,mm,nn))
     .                +zi*(zm3_1234(nf1,nf2,ll,ll,mm,nn)+zm4_1234(nf1,nf2,ll,ll,mm,nn))

                   za= +zi*(zm1_3214(nf2,nf1,ll,ll,mm,nn)+zm4_1432(nf1,nf2,ll,ll,mm,nn))
     .                 +zi*(zm2_1432(nf1,nf2,ll,ll,mm,nn)+zm3_3214(nf2,nf1,ll,ll,mm,nn)) 

                   zd= +zi*(zm1_3412(nf2,nf1,ll,ll,mm,nn)+zm2_3412(nf2,nf1,ll,ll,mm,nn))
     .                  +zi*(zm3_3412(nf2,nf1,ll,ll,mm,nn)+zm4_3412(nf2,nf1,ll,ll,mm,nn))

                   zb= +zi*(zm2_3214(nf2,nf1,ll,ll,mm,nn)+zm4_3214(nf2,nf1,ll,ll,mm,nn))
     .                  +zi*(zm1_1432(nf1,nf2,ll,ll,mm,nn)+zm3_1432(nf1,nf2,ll,ll,mm,nn))        
    
                   A = A+2d0*real(zb*dconjg(zc))
                   A = A+2d0*real(za*dconjg(zd))


                  zc1 = +zi*(zm1_3214(nf2,nf1,ll,ll,mm,nn)+zm2_3214(nf2,nf1,ll,ll,mm,nn))
     .                +zi*(zm3_3214(nf2,nf1,ll,ll,mm,nn)+zm4_3214(nf2,nf1,ll,ll,mm,nn))

                   za1=  +zi*(zm1_1234(nf1,nf2,ll,ll,mm,nn)+zm4_3412(nf2,nf1,ll,ll,mm,nn))
     .                +zi*(zm2_3412(nf2,nf1,ll,ll,mm,nn)+zm3_1234(nf1,nf2,ll,ll,mm,nn)) 

                   zd1= +zi*(zm1_1432(nf1,nf2,ll,ll,mm,nn)+zm2_1432(nf1,nf2,ll,ll,mm,nn))
     .                  +zi*(zm3_1432(nf1,nf2,ll,ll,mm,nn)+zm4_1432(nf1,nf2,ll,ll,mm,nn))

                   zb1=  +zi*(zm2_1234(nf1,nf2,ll,ll,mm,nn)+zm4_1234(nf1,nf2,ll,ll,mm,nn))
     .                 +zi*(zm1_3412(nf2,nf1,ll,ll,mm,nn)+zm3_3412(nf2,nf1,ll,ll,mm,nn))        

                   A1 = A1+2d0*real(zb1*dconjg(zc1))
                   A1 = A1+2d0*real(za1*dconjg(zd1))


            enddo
         enddo
      enddo


           D1g0ZX = (A+A1)

      return
      end


*******************************************************************
* nfg1,nfg2,nfg3 means the three non-identical flavour of three quark pairs
* in E-type process. 
* nfg1 corresponds to the first quark pair that couples to the Z boson.

      function f1f3C1g0Zb(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)       
      common /EZFlav/nfg1,nfg2,nfg3
!$omp threadprivate(/EZFlav/)
      common /BZFlav/nf1g
!$omp threadprivate(/BZFlav/)
      integer :: nf1g1,nf1g2,nf1g3
      if(nfg1.ne.0 .and. nfg2.ne.0.and. nfg3.ne.0)then
        nf1g1=nfg1
        nf1g2=nfg2
        nf1g3=nfg3

         nf1 = nf1g1
         nf2 = nf1g3
         f1f3C1g0Zb = C1g0Zb(i1,i2,i3,i4,i5,i6,i7)
      else
         write(*,*) "Error in f1f3b: flavours have not been initialised"
         stop
      endif
      nfg1 = nf1g1
      nfg2 = nf1g2
      nfg3 = nf1g3

      return

      end
*******************************************************************

      function f1f3C1g0Za(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)       
      common /EZFlav/nfg1,nfg2,nfg3
!$omp threadprivate(/EZFlav/)
      common /BZFlav/nf1g
!$omp threadprivate(/BZFlav/)
      integer :: nf1g1,nf1g2,nf1g3

      if(nfg1.ne.0 .and. nfg2.ne.0 .and. nfg3.ne.0)then
        nf1g1=nfg1
        nf1g2=nfg2
        nf1g3=nfg3

         nf1 = nf1g1
         nf2 = nf1g3
         f1f3C1g0Za = C1g0Za(i1,i2,i3,i4,i5,i6,i7)
      else
         write(*,*) "Error in f1f3a: flavours have not been initialised"
         stop
      endif
      nfg1 = nf1g1
      nfg2 = nf1g2
      nfg3 = nf1g3

      return

      end
*******************************************************************
      function f1f2C1g0Za(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      common /EZFlav/nfg1,nfg2,nfg3
!$omp threadprivate(/EZFlav/)
      common /BZFlav/nf1g
!$omp threadprivate(/BZFlav/)
      integer :: nf1g1,nf1g2,nf1g3      

      if(nfg1.ne.0 .and. nfg2.ne.0.and. nfg3.ne.0)then
        nf1g1=nfg1
        nf1g2=nfg2
        nf1g3=nfg3

         nf1 = nf1g1
         nf2 = nf1g2
         f1f2C1g0Za = C1g0Za(i1,i2,i3,i4,i5,i6,i7)

      else
         write(*,*) "Error in f1f2a: flavours have not been initialised"
         stop
      endif
      nfg1 = nf1g1
      nfg2 = nf1g2
      nfg3 = nf1g3
      return

      end
*******************************************************************
      function f1f2C1g0Zb(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      common /EZFlav/nfg1,nfg2,nfg3
!$omp threadprivate(/EZFlav/)
      common /BZFlav/nf1g
!$omp threadprivate(/BZFlav/)
      integer :: nf1g1,nf1g2,nf1g3
      if(nfg1.ne.0 .and. nfg2.ne.0.and. nfg3.ne.0)then
        nf1g1=nfg1
        nf1g2=nfg2
        nf1g3=nfg3

         nf1 = nf1g1
         nf2 = nf1g2
         f1f2C1g0Zb = C1g0Zb(i1,i2,i3,i4,i5,i6,i7)
      else
         write(*,*) "Error in f1f2b: flavours have not been initialised"
         stop
      endif
      nfg1 = nf1g1
      nfg2 = nf1g2
      nfg3 = nf1g3
      return

      end

*******************************************************************

      function f3f2C1g0Za(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      common /EZFlav/nfg1,nfg2,nfg3
!$omp threadprivate(/EZFlav/)
      common /BZFlav/nf1g
!$omp threadprivate(/BZFlav/)
      integer :: nf1g1,nf1g2,nf1g3
      if(nfg1.ne.0 .and. nfg2.ne.0.and. nfg3.ne.0)then
        nf1g1=nfg1
        nf1g2=nfg2
        nf1g3=nfg3

         nf1 = nf1g3
         nf2 = nf1g2
         f3f2C1g0Za = C1g0Za(i1,i2,i3,i4,i5,i6,i7)
      else
         write(*,*) "Error in f3f2a: flavours have not been initialised"
         stop
      endif
      nfg1 = nf1g1
      nfg2 = nf1g2
      nfg3 = nf1g3
      return

      end

*******************************************************************
      function f3f2C1g0Zb(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      common /EZFlav/nfg1,nfg2,nfg3
!$omp threadprivate(/EZFlav/)
      common /BZFlav/nf1g
!$omp threadprivate(/BZFlav/)
      integer :: nf1g1,nf1g2,nf1g3
      if(nfg1.ne.0 .and. nfg2.ne.0.and. nfg3.ne.0)then
        nf1g1=nfg1
        nf1g2=nfg2
        nf1g3=nfg3

         nf1 = nf1g3
         nf2 = nf1g2
         f3f2C1g0Zb = C1g0Zb(i1,i2,i3,i4,i5,i6,i7)
      else
         write(*,*) "Error in f3f2b: flavours have not been initialised"
         stop
      endif
      nfg1 = nf1g1
      nfg2 = nf1g2
      nfg3 = nf1g3
      return

      end

*******************************************************************
      function f1f3C1g0Zc(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      common /EZFlav/nfg1,nfg2,nfg3
!$omp threadprivate(/EZFlav/)
      common /BZFlav/nf1g
!$omp threadprivate(/BZFlav/)
      integer :: nf1g1,nf1g2,nf1g3
      if(nfg1.ne.0 .and. nfg2.ne.0.and. nfg3.ne.0)then
        nf1g1=nfg1
        nf1g2=nfg2
        nf1g3=nfg3

         nf1 = nf1g1
         nf2 = nf1g3
         f1f3C1g0Zc = C1g0Zc(i1,i2,i3,i4,i5,i6,i7)
      else
         write(*,*) "Error in f1f3c: flavours have not been initialised"
         stop
      endif
      nfg1 = nf1g1
      nfg2 = nf1g2
      nfg3 = nf1g3

      return

      end
*******************************************************************

      function f1f3C1g0Zd(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      common /EZFlav/nfg1,nfg2,nfg3
!$omp threadprivate(/EZFlav/)
      common /BZFlav/nf1g
!$omp threadprivate(/BZFlav/)
      integer :: nf1g1,nf1g2,nf1g3
      if(nfg1.ne.0 .and. nfg2.ne.0.and. nfg3.ne.0)then
        nf1g1=nfg1
        nf1g2=nfg2
        nf1g3=nfg3

         nf1 = nf1g1
         nf2 = nf1g3
         f1f3C1g0Zd = C1g0Zd(i1,i2,i3,i4,i5,i6,i7)
      else
         write(*,*) "Error in f1f3d: flavours have not been initialised"
         stop
      endif
      nfg1 = nf1g1
      nfg2 = nf1g2
      nfg3 = nf1g3

      return

      end
*******************************************************************
      function f1f2C1g0Zc(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      common /EZFlav/nfg1,nfg2,nfg3
!$omp threadprivate(/EZFlav/)
      common /BZFlav/nf1g
!$omp threadprivate(/BZFlav/)
      integer :: nf1g1,nf1g2,nf1g3

      if(nfg1.ne.0 .and. nfg2.ne.0.and. nfg3.ne.0)then
        nf1g1=nfg1
        nf1g2=nfg2
        nf1g3=nfg3

         nf1 = nf1g1
         nf2 = nf1g2
         f1f2C1g0Zc = C1g0Zc(i1,i2,i3,i4,i5,i6,i7)
      else
         write(*,*) "Error in f1f2c: flavours have not been initialised"
         stop
      endif
      nfg1 = nf1g1
      nfg2 = nf1g2
      nfg3 = nf1g3
      return

      end
*******************************************************************
      function f1f2C1g0Zd(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      common /EZFlav/nfg1,nfg2,nfg3
!$omp threadprivate(/EZFlav/)
      common /BZFlav/nf1g
!$omp threadprivate(/BZFlav/)
      integer :: nf1g1,nf1g2,nf1g3
      if(nfg1.ne.0 .and. nfg2.ne.0.and. nfg3.ne.0)then
        nf1g1=nfg1
        nf1g2=nfg2
        nf1g3=nfg3

         nf1 = nf1g1
         nf2 = nf1g2
         f1f2C1g0Zd = C1g0Zd(i1,i2,i3,i4,i5,i6,i7)
      else
         write(*,*) "Error in f1f2d: flavours have not been initialised"
         stop
      endif
      nfg1 = nf1g1
      nfg2 = nf1g2
      nfg3 = nf1g3
      return

      end

*******************************************************************

      function f3f2C1g0Zc(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      common /EZFlav/nfg1,nfg2,nfg3
!$omp threadprivate(/EZFlav/)
      common /BZFlav/nf1g
!$omp threadprivate(/BZFlav/)
      integer :: nf1g1,nf1g2,nf1g3
      if(nfg1.ne.0 .and. nfg2.ne.0.and. nfg3.ne.0)then
        nf1g1=nfg1
        nf1g2=nfg2
        nf1g3=nfg3

         nf1 = nf1g3
         nf2 = nf1g2
         f3f2C1g0Zc = C1g0Zc(i1,i2,i3,i4,i5,i6,i7)
      else
         write(*,*) "Error in f3f2c: flavours have not been initialised"
         stop
      endif
      nfg1 = nf1g1
      nfg2 = nf1g2
      nfg3 = nf1g3
      return

      end

*******************************************************************
      function f3f2C1g0Zd(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      common /EZFlav/nfg1,nfg2,nfg3
!$omp threadprivate(/EZFlav/)
      common /BZFlav/nf1g
!$omp threadprivate(/BZFlav/)
      integer :: nf1g1,nf1g2,nf1g3

      if(nfg1.ne.0 .and. nfg2.ne.0.and. nfg3.ne.0)then
        nf1g1=nfg1
        nf1g2=nfg2
        nf1g3=nfg3

         nf1 = nf1g3
         nf2 = nf1g2
         f3f2C1g0Zd = C1g0Zd(i1,i2,i3,i4,i5,i6,i7)
      else
         write(*,*) "Error in f3f2d: flavours have not been initialised"
         stop
      endif
      nfg1 = nf1g1
      nfg2 = nf1g2
      nfg3 = nf1g3
      return

      end


*******************************************************************

      function f1f3C1g0Zgam(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      common /EZFlav/nfg1,nfg2,nfg3
!$omp threadprivate(/EZFlav/)
      common /BZFlav/nf1g
!$omp threadprivate(/BZFlav/)
      integer :: nf1g1,nf1g2,nf1g3
      if(nfg1.ne.0 .and. nfg2.ne.0.and. nfg3.ne.0)then
        nf1g1=nfg1
        nf1g2=nfg2
        nf1g3=nfg3

         nf1 = nf1g1
         nf2 = nf1g3
         f1f3C1g0Zgam = C1g0Zgam(i1,i2,i3,i4,i5,i6,i7)
      else
         write(*,*) "Error in f1f3gam: flavours have not been initialised"
         stop
      endif
      nfg1 = nf1g1
      nfg2 = nf1g2
      nfg3 = nf1g3

      return

      end
*******************************************************************



      function f3f2C1g0Zgam(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      common /EZFlav/nfg1,nfg2,nfg3
!$omp threadprivate(/EZFlav/)
      common /BZFlav/nf1g
!$omp threadprivate(/BZFlav/)
      integer :: nf1g1,nf1g2,nf1g3
      if(nfg1.ne.0 .and. nfg2.ne.0.and. nfg3.ne.0)then
        nf1g1=nfg1
        nf1g2=nfg2
        nf1g3=nfg3

         nf1 = nf1g3
         nf2 = nf1g2
         f3f2C1g0Zgam = C1g0Zgam(i1,i2,i3,i4,i5,i6,i7)
      else
         write(*,*) "Error in f3f2gam: flavours have not been initialised"
         stop
      endif
      nfg1 = nf1g1
      nfg2 = nf1g2
      nfg3 = nf1g3
      return

      end

*******************************************************************
      function f1f2C1g0Zgam(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      common /EZFlav/nfg1,nfg2,nfg3
!$omp threadprivate(/EZFlav/)
      common /BZFlav/nf1g
!$omp threadprivate(/BZFlav/)
      integer :: nf1g1,nf1g2,nf1g3

      if(nfg1.ne.0 .and. nfg2.ne.0.and. nfg3.ne.0)then
        nf1g1=nfg1
        nf1g2=nfg2
        nf1g3=nfg3

         nf1 = nf1g1
         nf2 = nf1g2
         f1f2C1g0Zgam = C1g0Zgam(i1,i2,i3,i4,i5,i6,i7)
      else
         write(*,*) "Error in f1f2gam: flavours have not been initialised"
         stop
      endif
      nfg1 = nf1g1
      nfg2 = nf1g2
      nfg3 = nf1g3
      return

      end
*******************************************************************
* nfg1,nfg2,nfg3 means the two non-identical flavour of three quark pairs
* in F-type process.
* f1f2 means the flavors of first two quark pairs are identical.

      function f1f2D1g0Zgam(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      common /EZFlav/nfg1,nfg2,nfg3
!$omp threadprivate(/EZFlav/)
      common /BZFlav/nf1g
!$omp threadprivate(/BZFlav/)
      integer :: nf1g1,nf1g2,nf1g3
        nf1g1=nfg1
        nf1g2=nfg2
        nf1g3=nfg3
      if(nfg1.eq.nfg2)then

         nf1 = nf1g1
         nf2 = nf1g2

         f1f2D1g0Zgam = D1g0Zgam(i1,i2,i3,i4,i5,i6,i7)

      else

      f1f2D1g0Zgam = 0d0

      endif

      nfg1 = nf1g1
      nfg2 = nf1g2
      nfg3 = nf1g3
      return

      end

*******************************************************************
* nfg1,nfg2,nfg3 means the two non-identical flavour of three quark pairs
* in F-type process.
* f1f3 means the flavors of first quark pair and the third quark pair are identical.

      function f1f3D1g0Zgam(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      common /EZFlav/nfg1,nfg2,nfg3
!$omp threadprivate(/EZFlav/)
      common /BZFlav/nf1g
!$omp threadprivate(/BZFlav/)
      integer :: nf1g1,nf1g2,nf1g3
        nf1g1=nfg1
        nf1g2=nfg2
        nf1g3=nfg3
      if(nfg1.eq.nfg3)then

         nf1 = nf1g1
         nf2 = nf1g3

         f1f3D1g0Zgam = D1g0Zgam(i1,i2,i3,i4,i5,i6,i7)

      else

      f1f3D1g0Zgam = 0d0

      endif

      nfg1 = nf1g1
      nfg2 = nf1g2
      nfg3 = nf1g3
      return

      end      
*******************************************************************
* nfg1,nfg2,nfg3 means the two non-identical flavour of three quark pairs
* in F-type process.
* f2f3 means the flavors of second quark pair and the third quark pair are identical.

      function f2f3D1g0Zgam(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      common /EZFlav/nfg1,nfg2,nfg3
!$omp threadprivate(/EZFlav/)
      common /BZFlav/nf1g
!$omp threadprivate(/BZFlav/)
      integer :: nf1g1,nf1g2,nf1g3
        nf1g1=nfg1
        nf1g2=nfg2
        nf1g3=nfg3
      if(nfg2.eq.nfg3)then

         nf1 = nf1g2
         nf2 = nf1g3

         f2f3D1g0Zgam = D1g0Zgam(i1,i2,i3,i4,i5,i6,i7)

      else

      f2f3D1g0Zgam = 0d0

      endif

      nfg1 = nf1g1
      nfg2 = nf1g2
      nfg3 = nf1g3
      return

      end      

*******************************************************************
* nfg1,nfg2,nfg3 means the two non-identical flavour of three quark pairs
* in F-type process.

      function f1f2D1g0ZX(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)      
      common /EZFlav/nfg1,nfg2,nfg3
!$omp threadprivate(/EZFlav/)
      common /BZFlav/nf1g
!$omp threadprivate(/BZFlav/)
      integer :: nf1g1,nf1g2,nf1g3
        nf1g1=nfg1
        nf1g2=nfg2
        nf1g3=nfg3
      if(nfg1.eq.nfg2)then

         nf1 = nf1g1
         nf2 = nf1g2
         f1f2D1g0ZX = D1g0ZX(i1,i2,i3,i4,i5,i6,i7)

      else

         f1f2D1g0ZX = 0d0

      endif
      nfg1 = nf1g1
      nfg2 = nf1g2
      nfg3 = nf1g3
      return

      end

*******************************************************************
* nfg1,nfg2,nfg3 means the two non-identical flavour of three quark pairs
* in F-type process.

      function f1f3D1g0ZX(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)      
      common /EZFlav/nfg1,nfg2,nfg3
!$omp threadprivate(/EZFlav/)
      common /BZFlav/nf1g
!$omp threadprivate(/BZFlav/)
      integer :: nf1g1,nf1g2,nf1g3
        nf1g1=nfg1
        nf1g2=nfg2
        nf1g3=nfg3
      if(nfg1.eq.nfg3)then

         nf1 = nf1g1
         nf2 = nf1g3
         f1f3D1g0ZX = D1g0ZX(i1,i2,i3,i4,i5,i6,i7)

      else

         f1f3D1g0ZX = 0d0

      endif
      nfg1 = nf1g1
      nfg2 = nf1g2
      nfg3 = nf1g3
      return

      end

*******************************************************************
* nfg1,nfg2,nfg3 means the two non-identical flavour of three quark pairs
* in F-type process.

      function f2f3D1g0ZX(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)      
      common /EZFlav/nfg1,nfg2,nfg3
!$omp threadprivate(/EZFlav/)
      common /BZFlav/nf1g
!$omp threadprivate(/BZFlav/)
      integer :: nf1g1,nf1g2,nf1g3
        nf1g1=nfg1
        nf1g2=nfg2
        nf1g3=nfg3
      if(nfg2.eq.nfg3)then

         nf1 = nf1g2
         nf2 = nf1g3
         f2f3D1g0ZX = D1g0ZX(i1,i2,i3,i4,i5,i6,i7)

      else

         f2f3D1g0ZX = 0d0

      endif
      nfg1 = nf1g1
      nfg2 = nf1g2
      nfg3 = nf1g3
      return

      end      
