      module C1gZ

      contains


*************************************************************
* Two quark pair tree level helicity amplitudes for
*
* q + qb -> Q + Qb + e- + e+
*
* One colour ordering and helicity, ignoring overall factors. Unlike other
* matrix elements, the electroweak factors are included at this level. The reason
* why is because we need to establish the interference pattern.
*
* The first two indices denote the type of quarks in the process (1 implies
* up type, 2 implies down type.
*
* The remaining indices denote the helicity configuration of the amplitude in question
*
      subroutine genC1g0Zamp(i1,i2,i3,i4,i5,i6,i7,zmb1,zmb2)
      use KinData_mod
      use astore
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      common /eweakZ/amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      complex*16 zzA1(2,2,2,2),zzA2(2,2,2,2)
      complex*16 zzB3(2,2,2,2),zzB4(2,2,2,2)
      complex*16 zm1(2,2,2,2,2,2),zm2(2,2,2,2,2,2),zm3(2,2,2,2,2,2),zm4(2,2,2,2,2,2)
      complex*16 zmb1(2,2,2,2,2,2),zmb2(2,2,2,2,2,2)
      logical exists

      call checkamp(i1,i2,i3,i4,i5,exists,zzA1,zzA2,zzB3,zzB4)

      if(.not.exists)then
         call swapzAB_kin(7)    ! mixed up za <-> zb in the amplitudes
         zzA1 = zampA1(i1,i2,i3,i4,i5,i6,i7)
         zzA2 = zampA2(i1,i2,i3,i4,i5,i6,i7)

         zzB3 = zampA3(i2,i1,i5,i4,i3,i6,i7)
         zzB4 = zampA4(i2,i1,i5,i4,i3,i6,i7)
         call swapzAB_kin(7)
         call addamp(i1,i2,i3,i4,i5,zzA1,zzA2,zzB3,zzB4)
      endif

      zmb1 = 0d0
      zmb2 = 0d0

      call eweakV2qq(nf1,nf2,s7(i6,i7))

      do ii=1,2
         do jj=1,2
            do kk=1,2
               do ll=1,2
                  if ((ii.eq.1).and.(ll.eq.1))then
                     i = 1 ! LL
                  else if ((ii.eq.1).and.(ll.eq.2)) then
                     i = 2 ! LR
                  else if ((ii.eq.2).and.(ll.eq.1)) then
                     i = 3 ! RL
                  else
                     i = 4 ! RR
                  endif

                  if ((jj.eq.1).and.(ll.eq.1))then
                     j = 1 ! LL
                  else if ((jj.eq.1).and.(ll.eq.2)) then
                     j = 2 ! LR
                  else if ((jj.eq.2).and.(ll.eq.1)) then
                     j = 3 ! RL
                  else
                     j = 4 ! RR
                  endif

                  zm1(nf1,nf2,ii,jj,kk,ll)=zewfac1(i)*zzA1(ii,jj,kk,ll)
                  zm2(nf1,nf2,ii,jj,kk,ll)=zewfac1(i)*zzA2(ii,jj,kk,ll)
                  zm3(nf1,nf2,jj,ii,kk,ll)=zewfac2(j)*zzB3(jj,ii,kk,ll)
                  zm4(nf1,nf2,jj,ii,kk,ll)=zewfac2(j)*zzB4(jj,ii,kk,ll)
                  if(nf1.ne.nf2)then
                     zm1(nf2,nf1,ii,jj,kk,ll)=zewfac2(i)*zzA1(ii,jj,kk,ll)
                     zm2(nf2,nf1,ii,jj,kk,ll)=zewfac2(i)*zzA2(ii,jj,kk,ll)
                     zm3(nf2,nf1,jj,ii,kk,ll)=zewfac1(j)*zzB3(jj,ii,kk,ll)
                     zm4(nf2,nf1,jj,ii,kk,ll)=zewfac1(j)*zzB4(jj,ii,kk,ll)
                  endif
               enddo
            enddo
         enddo
      enddo


      do kk=1,2
         do ll=1,2
            do mm=1,2
               do nn=1,2
                  zmb1(nf1,nf2,kk,ll,mm,nn) =zm1(nf1,nf2,kk,ll,mm,nn)
     .                 +zm3(nf1,nf2,ll,kk,mm,nn)
                  zmb2(nf1,nf2,kk,ll,mm,nn) =zm2(nf1,nf2,kk,ll,mm,nn)
     .                 +zm4(nf1,nf2,ll,kk,mm,nn)

                  if(nf1.ne.nf2)then
                     zmb1(nf2,nf1,kk,ll,mm,nn) =zm1(nf2,nf1,kk,ll,mm,nn)
     .                    +zm3(nf2,nf1,ll,kk,mm,nn)
                     zmb2(nf2,nf1,kk,ll,mm,nn) =zm2(nf2,nf1,kk,ll,mm,nn)
     .                    +zm4(nf2,nf1,ll,kk,mm,nn)
                  endif

               enddo
            enddo
         enddo
      enddo


      return
      end subroutine




****************************************************************************



      subroutine genC1g0Zamp1(i1,i2,i3,i4,i5,i6,i7,zm1,zm2,zm3,zm4)
      use KinData_mod
      use astore
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      common /eweakZ/amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      complex*16 zzA1(2,2,2,2),zzA2(2,2,2,2)
      complex*16 zzB3(2,2,2,2),zzB4(2,2,2,2)
      complex*16 zm1(2,2,2,2,2,2),zm2(2,2,2,2,2,2),zm3(2,2,2,2,2,2),zm4(2,2,2,2,2,2)
c      complex*16 zmb11(2,2,2,2,2,2),zmb22(2,2,2,2,2,2)
      logical exists

      call checkamp(i1,i2,i3,i4,i5,exists,zzA1,zzA2,zzB3,zzB4)

      if(.not.exists)then
         call swapzAB_kin(7)    ! mixed up za <-> zb in the amplitudes
         zzA1 = zampA1(i1,i2,i3,i4,i5,i6,i7)
         zzA2 = zampA2(i1,i2,i3,i4,i5,i6,i7)

         zzB3 = zampA3(i2,i1,i5,i4,i3,i6,i7)
         zzB4 = zampA4(i2,i1,i5,i4,i3,i6,i7)
         call swapzAB_kin(7)
         call addamp(i1,i2,i3,i4,i5,zzA1,zzA2,zzB3,zzB4)
      endif

c      zmb11 = 0d0
c      zmb22 = 0d0

      call eweakV2qq(nf1,nf2,s7(i6,i7))

      do ii=1,2
         do jj=1,2
            do kk=1,2
               do ll=1,2
                  if ((ii.eq.1).and.(ll.eq.1))then
                     i = 1 ! LL  ii is 1st quark ,ll is lepton
                  else if ((ii.eq.1).and.(ll.eq.2)) then
                     i = 2 ! LR
                  else if ((ii.eq.2).and.(ll.eq.1)) then
                     i = 3 ! RL
                  else
                     i = 4 ! RR
                  endif

                  if ((jj.eq.1).and.(ll.eq.1))then
                     j = 1 ! LL
                  else if ((jj.eq.1).and.(ll.eq.2)) then
                     j = 2 ! LR
                  else if ((jj.eq.2).and.(ll.eq.1)) then
                     j = 3 ! RL
                  else
                     j = 4 ! RR
                  endif

                  zm1(nf1,nf2,ii,jj,kk,ll)=zewfac1(i)*zzA1(ii,jj,kk,ll)
                  zm2(nf1,nf2,ii,jj,kk,ll)=zewfac1(i)*zzA2(ii,jj,kk,ll)
                  zm3(nf1,nf2,jj,ii,kk,ll)=zewfac2(j)*zzB3(jj,ii,kk,ll)
                  zm4(nf1,nf2,jj,ii,kk,ll)=zewfac2(j)*zzB4(jj,ii,kk,ll)
                  if(nf1.ne.nf2)then
                     zm1(nf2,nf1,ii,jj,kk,ll)=zewfac2(i)*zzA1(ii,jj,kk,ll)
                     zm2(nf2,nf1,ii,jj,kk,ll)=zewfac2(i)*zzA2(ii,jj,kk,ll)
                     zm3(nf2,nf1,jj,ii,kk,ll)=zewfac1(j)*zzB3(jj,ii,kk,ll)
                     zm4(nf2,nf1,jj,ii,kk,ll)=zewfac1(j)*zzB4(jj,ii,kk,ll)
                  endif
               enddo
            enddo
         enddo
      enddo


      return
      end subroutine



*****************************************





*****************************************
* A function that returns an array of the A1 amplitudes from PRD 014020
*
* The helicity configurations are
*
* 1st index => 1st quark helicity
* 2nd index => 2nd quark helicity
* 3rd index => gluon polarisation
* 4th index => lepton helicity


      function zampA1(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)
      complex*16 zampA1(2,2,2,2)

      zampA1(1,1,1,1) = -zampA1pmpmp(i1,i2,i3,i4,i5,i6,i7)
      zampA1(1,1,1,2) = -zampA1pmpmp(i1,i2,i3,i4,i5,i7,i6)
      zampA1(1,2,1,1) = -zampA1pmpmp(i1,i3,i2,i4,i5,i6,i7)
      zampA1(1,2,1,2) = -zampA1pmpmp(i1,i3,i2,i4,i5,i7,i6)
      call swapzAB_kin(7)
      call transposezA_kin(7)
      call transposezB_kin(7)
      zampA1(2,1,2,1) = -zampA1pmpmp(i1,i3,i2,i4,i5,i7,i6)
      zampA1(2,1,2,2) = -zampA1pmpmp(i1,i3,i2,i4,i5,i6,i7)
      zampA1(2,2,2,1) = -zampA1pmpmp(i1,i2,i3,i4,i5,i7,i6)
      zampA1(2,2,2,2) = -zampA1pmpmp(i1,i2,i3,i4,i5,i6,i7)
      call transposezA_kin(7)
      call transposezB_kin(7)
      call swapzAB_kin(7)
      zampA1(1,1,2,1) = -zampA1pmpmm(i1,i2,i3,i4,i5,i6,i7)
      zampA1(1,1,2,2) = -zampA1pmpmm(i1,i2,i3,i4,i5,i7,i6)
      zampA1(1,2,2,1) = -zampA1pmpmm(i1,i3,i2,i4,i5,i6,i7)
      zampA1(1,2,2,2) = -zampA1pmpmm(i1,i3,i2,i4,i5,i7,i6)
      call swapzAB_kin(7)
      call transposezA_kin(7)
      call transposezB_kin(7)
      zampA1(2,1,1,1) = -zampA1pmpmm(i1,i3,i2,i4,i5,i7,i6)
      zampA1(2,1,1,2) = -zampA1pmpmm(i1,i3,i2,i4,i5,i6,i7)
      zampA1(2,2,1,1) = -zampA1pmpmm(i1,i2,i3,i4,i5,i7,i6)
      zampA1(2,2,1,2) = -zampA1pmpmm(i1,i2,i3,i4,i5,i6,i7)
      call transposezA_kin(7)
      call transposezB_kin(7)
      call swapzAB_kin(7)

      return
      end function


*****************************************
* A function that returns an array of the A3 amplitudes from PRD 014020
*
* The helicity configurations are
*
* 1st index => 1st quark helicity
* 2nd index => 2nd quark helicity
* 3rd index => gluon polarisation
* 4th index => lepton helicity


      function zampA2(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)
      complex*16 zampA2(2,2,2,2)

      zampA2(1,1,1,1) = -zampA2pmpmp(i1,i2,i3,i4,i5,i6,i7)
      zampA2(1,1,1,2) = -zampA2pmpmp(i1,i2,i3,i4,i5,i7,i6)
      zampA2(1,2,1,1) = -zampA2pmpmp(i1,i3,i2,i4,i5,i6,i7)
      zampA2(1,2,1,2) = -zampA2pmpmp(i1,i3,i2,i4,i5,i7,i6)
      call swapzAB_kin(7)
      call transposezA_kin(7)
      call transposezB_kin(7)
      zampA2(2,1,2,1) = -zampA2pmpmp(i1,i3,i2,i4,i5,i7,i6)
      zampA2(2,1,2,2) = -zampA2pmpmp(i1,i3,i2,i4,i5,i6,i7)
      zampA2(2,2,2,1) = -zampA2pmpmp(i1,i2,i3,i4,i5,i7,i6)
      zampA2(2,2,2,2) = -zampA2pmpmp(i1,i2,i3,i4,i5,i6,i7)
      call transposezA_kin(7)
      call transposezB_kin(7)
      call swapzAB_kin(7)
      zampA2(1,1,2,1) = -zampA2pmpmm(i1,i2,i3,i4,i5,i6,i7)
      zampA2(1,1,2,2) = -zampA2pmpmm(i1,i2,i3,i4,i5,i7,i6)
      zampA2(1,2,2,1) = -zampA2pmpmm(i1,i3,i2,i4,i5,i6,i7)
      zampA2(1,2,2,2) = -zampA2pmpmm(i1,i3,i2,i4,i5,i7,i6)
      call swapzAB_kin(7)
      call transposezA_kin(7)
      call transposezB_kin(7)
      zampA2(2,1,1,1) = -zampA2pmpmm(i1,i3,i2,i4,i5,i7,i6)
      zampA2(2,1,1,2) = -zampA2pmpmm(i1,i3,i2,i4,i5,i6,i7)
      zampA2(2,2,1,1) = -zampA2pmpmm(i1,i2,i3,i4,i5,i7,i6)
      zampA2(2,2,1,2) = -zampA2pmpmm(i1,i2,i3,i4,i5,i6,i7)
      call transposezA_kin(7)
      call transposezB_kin(7)
      call swapzAB_kin(7)

      return
      end function

*****************************************
* A function that returns an array of the A3 amplitudes from PRD 014020
*
* The helicity configurations are
*
* 1st index => 1st quark helicity
* 2nd index => 2nd quark helicity
* 3rd index => gluon polarisation
* 4th index => lepton helicity


      function zampA3(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)
      complex*16 zampA3(2,2,2,2)

      zero = dcmplx(0d0,0d0)

      do ii=1,2
         do jj=1,2
            do kk=1,2
               do ll=1,2
                  zampA3(ii,jj,kk,ll) = zero
               enddo
            enddo
         enddo
      enddo

      zampA3(1,1,1,1) = -zampA3pmpmp(i1,i2,i3,i4,i5,i6,i7)
      zampA3(1,1,1,2) = -zampA3pmpmp(i1,i2,i3,i4,i5,i7,i6)
      zampA3(1,2,2,1) = -zampA3pmmpm(i1,i3,i2,i4,i5,i6,i7)
      zampA3(1,2,2,2) = -zampA3pmmpm(i1,i3,i2,i4,i5,i7,i6)

      call swapzAB_kin(7)
      call transposezA_kin(7)
      call transposezB_kin(7)
      zampA3(2,2,2,1) = -zampA3pmpmp(i1,i2,i3,i4,i5,i7,i6)
      zampA3(2,2,2,2) = -zampA3pmpmp(i1,i2,i3,i4,i5,i6,i7)
      zampA3(2,1,1,1) = -zampA3pmmpm(i1,i3,i2,i4,i5,i7,i6)
      zampA3(2,1,1,2) = -zampA3pmmpm(i1,i3,i2,i4,i5,i6,i7)
      call transposezA_kin(7)
      call transposezB_kin(7)
      call swapzAB_kin(7)

      return
      end function

*****************************************
* A function that returns an array of the A3 amplitudes from PRD 014020
*
* The helicity configurations are
*
* 1st index => 1st quark helicity
* 2nd index => 2nd quark helicity
* 3rd index => gluon polarisation
* 4th index => lepton helicity


      function zampA4(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)
      complex*16 zampA4(2,2,2,2)

      zero = dcmplx(0d0,0d0)

      do ii=1,2
         do jj=1,2
            do kk=1,2
               do ll=1,2
                  zampA4(ii,jj,kk,ll) = zero
               enddo
            enddo
         enddo
      enddo

      zampA4(1,1,2,1) = -zampA4pmpmm(i1,i2,i3,i4,i5,i6,i7)
      zampA4(1,1,2,2) = -zampA4pmpmm(i1,i2,i3,i4,i5,i7,i6)
      zampA4(1,2,1,1) = -zampA4pmmpp(i1,i3,i2,i4,i5,i6,i7)
      zampA4(1,2,1,2) = -zampA4pmmpp(i1,i3,i2,i4,i5,i7,i6)
      call swapzAB_kin(7)
      call transposezA_kin(7)
      call transposezB_kin(7)
      zampA4(2,2,1,2) = -zampA4pmpmm(i1,i2,i3,i4,i5,i6,i7)
      zampA4(2,2,1,1) = -zampA4pmpmm(i1,i2,i3,i4,i5,i7,i6)
      zampA4(2,1,2,2) = -zampA4pmmpp(i1,i3,i2,i4,i5,i6,i7)
      zampA4(2,1,2,1) = -zampA4pmmpp(i1,i3,i2,i4,i5,i7,i6)
      call transposezA_kin(7)
      call transposezB_kin(7)
      call swapzAB_kin(7)


      return
      end function




*****************************************
* A50 in PRD 014020
*

      function zampA1pmpmp(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)


c     statement function

      zt2a(j1,j2,j3,j4) = zA7(j1,j2)*zB7(j2,j4)+zA7(j1,j3)*zB7(j3,j4)

      zb54 = zB7(i5,i4)
      za71 = zA7(i7,i1)
      za24 = zA7(i2,i4)
      s54  = s7(i5,i4)
      s32  = s7(i3,i2)
      zb56 = zB7(i5,i6)
      za21 = zA7(i2,i1)
      zb13 = zB7(i1,i3)
      za54 = zA7(i5,i4)
      za42 = zA7(i4,i2)
      zb43 = zB7(i4,i3)
      za32 = zA7(i3,i2)
      zb34 = zB7(i3,i4)
      s67 = s7(i6,i7)


      s234 = s7(i2,i3) + s7(i2,i4) + s7(i3,i4)
      s167 = s7(i1,i6) + s7(i1,i7) + s7(i6,i7)
      s567 = s7(i5,i6) + s7(i5,i7) + s7(i6,i7)
      s123 = s7(i1,i2) + s7(i2,i3) + s7(i1,i3)

      zampA1pmpmp = -zb54*zt2a(i2,i5,i4,i3)*zt2a(i2,i1,i7,i6)*za71
     .  /(za24*s54*s32*s167)
     . -zb56*zt2a(i7,i5,i6,i4)*za21**2*zb13
     .  /(za24*s32*s123*s567)
     . -zt2a(i2,i5,i4,i6)*zt2a(i7,i1,i2,i3)*za21
     .  /(za54*za42*s32*s123)
     . +zb43*zt2a(i2,i3,i4,i5)*zt2a(i2,i1,i7,i6)*za71
     .  /(za24*s32*s234*s167)
     . +zb56
     . *(zt2a(i7,i5,i6,i3)*za32+zt2a(i7,i5,i6,i4)*za42)
     . *zb34*za21/(za24*s32*s234*s567)
      zampA1pmpmp = zampA1pmpmp/s67
      return
      end function

*****************************************
* A53 in PRD 014020
*


      function zampA2pmpmp(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)


c     statement function

      zt2a(j1,j2,j3,j4) = zA7(j1,j2)*zB7(j2,j4)+zA7(j1,j3)*zB7(j3,j4)

      zb43 = zB7(i4,i3)
      za71 = zA7(i7,i1)
      za24 = zA7(i2,i4)
      s32  = s7(i3,i2)
      zb56 = zB7(i5,i6)
      zb14 = zB7(i1,i4)
      za21 = zA7(i2,i1)
      s14  = s7(i1,i4)
      zb53 = zB7(i5,i3)
      za41 = zA7(i4,i1)
      za32 = zA7(i3,i2)
      za42 = zA7(i4,i2)
      zb34 = zB7(i3,i4)
      s67 = s7(i6,i7)


      s167 = s7(i1,i6) + s7(i1,i7) + s7(i6,i7)
      s234 = s7(i2,i3) + s7(i2,i4) + s7(i3,i4)
      s567 = s7(i5,i6) + s7(i5,i7) + s7(i6,i7)
      s235 = s7(i2,i3) + s7(i2,i5) + s7(i3,i5)

      zampA2pmpmp = -zb43*zt2a(i2,i3,i4,i5)*zt2a(i2,i1,i7,i6)*za71
     .  /(za24*s32*s234*s167)
     . -zb56*zt2a(i7,i5,i6,i3)*zb14*za21**2
     .  /(za24*s14*s32*s567)
     . -zb53*zt2a(i2,i5,i3,i4)*zt2a(i2,i1,i7,i6)*za71
     .  /(za24*s32*s235*s167)
     . -zb53*zt2a(i2,i5,i3,i6)*za71*za21
     .  /(za24*za41*s32*s235)
     . -zb56
     .  *(zt2a(i7,i5,i6,i3)*za32+zt2a(i7,i5,i6,i4)*za42)
     . *zb34*za21/(za24*s32*s234*s567)


      zampA2pmpmp = zampA2pmpmp/s67
      return
      end function

*****************************************
* A56 in PRD 014020
*


      function zampA3pmpmp(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

c     statement function

      zt2a(j1,j2,j3,j4) = zA7(j1,j2)*zB7(j2,j4)+zA7(j1,j3)*zB7(j3,j4)

      zb43 = zB7(i4,i3)
      za71 = zA7(i7,i1)
      za24 = zA7(i2,i4)
      s34  = s7(i3,i4)
      zb56 = zB7(i5,i6)
      za32 = zA7(i3,i2)
      za42 = zA7(i4,i2)
      zb34 = zB7(i3,i4)
      za21 = zA7(i2,i1)
      s67 = s7(i6,i7)


      s234 = s7(i2,i3) + s7(i2,i4) + s7(i3,i4)
      s167 = s7(i1,i6) + s7(i1,i7) + s7(i6,i7)
      s567 = s7(i5,i6) + s7(i5,i7) + s7(i6,i7)

      zampA3pmpmp = -zb43*zt2a(i2,i3,i4,i5)*zt2a(i2,i1,i7,i6)*za71
     . /(za24*s34*s234*s167)
     . -zb56
     . *(zt2a(i7,i5,i6,i3)*za32+zt2a(i7,i5,i6,i4)*za42)
     . *zb34*za21/(za24*s34*s234*s567)


      zampA3pmpmp = zampA3pmpmp/s67

      return
      end function

*****************************************
* A59 in PRD 014020
*


      function zampA4pmmpp(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

c     statement function

      zt2a(j1,j2,j3,j4) = zA7(j1,j2)*zB7(j2,j4)+zA7(j1,j3)*zB7(j3,j4)

      zb43 = zB7(i4,i3)
      za71 = zA7(i7,i1)
      za24 = zA7(i2,i4)
      s34  = s7(i3,i4)
      zb56 = zB7(i5,i6)
      za32 = zA7(i3,i2)
      za42 = zA7(i4,i2)
      zb34 = zB7(i3,i4)
      za21 = zA7(i2,i1)
      s67 = s7(i6,i7)


      s234 = s7(i2,i3) + s7(i2,i4) + s7(i3,i4)
      s167 = s7(i1,i6) + s7(i1,i7) + s7(i6,i7)
      s567 = s7(i5,i6) + s7(i5,i7) + s7(i6,i7)

      zampA4pmmpp=+zb43*zt2a(i2,i3,i4,i5)*zt2a(i2,i1,i7,i6)
     . *za71/(za24*s34*s234*s167)
     .  +zb56
     . *(zt2a(i7,i5,i6,i3)*za32+zt2a(i7,i5,i6,i4)*za42)
     . *zb34*za21/(za24*s34*s234*s567)

      zampA4pmmpp = zampA4pmmpp/s67
      return
      end function

*****************************************
* A51 in PRD 014020
*

      function zampA1pmpmm(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

c     statement function

      zt2a(j1,j2,j3,j4) = zA7(j1,j2)*zB7(j2,j4)+zA7(j1,j3)*zB7(j3,j4)

      zb53 = zB7(i5,i3)
      za45 = zA7(i4,i5)
      za71 = zA7(i7,i1)
      zb34 = zB7(i3,i4)
      s54  = s7(i5,i4)
      s32  = s7(i3,i2)
      zb56 = zB7(i5,i6)
      za21 = zA7(i2,i1)
      zb54 = zB7(i5,i4)
      zb43 = zB7(i4,i3)
      za42 = zA7(i4,i2)
      zb32 = zB7(i3,i2)
      zb16 = zB7(i1,i6)
      za41 = zA7(i4,i1)
      za27 = zA7(i2,i7)
      zb76 = zB7(i7,i6)
      za47 = zA7(i4,i7)
      s67 = s7(i6,i7)


      s123 = s7(i1,i2) + s7(i1,i3) + s7(i2,i3)
      s234 = s7(i2,i3) + s7(i2,i4) + s7(i3,i4)
      s567 = s7(i5,i6) + s7(i5,i7) + s7(i6,i7)
      s167 = s7(i1,i6) + s7(i1,i7) + s7(i6,i7)

      zampA1pmpmm = +zb53**2*za45*zt2a(i2,i1,i7,i6)*za71
     .  /(zb34*s54*s32*s167)
     . +zb56*zt2a(i7,i5,i6,i3)*zt2a(i4,i1,i2,i3)*za21
     .  /(zb34*s32*s123*s567)
     . -zb53*zb56*zt2a(i7,i1,i2,i3)*za21
     .  /(zb54*zb43*s32*s123)
     . +zb53*za42
     .  *(+zb32*za21*zb16+zb34*za41*zb16
     .    +zb32*za27*zb76+zb34*za47*zb76)
     . *za71/(zb34*s32*s234*s167)
     . -zb56*zt2a(i7,i5,i6,i3)*za42*zt2a(i1,i2,i4,i3)
     .  /(zb34*s32*s234*s567)

      zampA1pmpmm = zampA1pmpmm/s67
      return
      end function

*****************************************
* A54 in PRD 014020
*

      function zampA2pmpmm(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

c     statement function

      zt2a(j1,j2,j3,j4) = zA7(j1,j2)*zB7(j2,j4)+zA7(j1,j3)*zB7(j3,j4)

      zb53 = zB7(i5,i3)
      za42 = zA7(i4,i2)
      zb32 = zB7(i3,i2)
      za21 = zA7(i2,i1)
      zb16 = zB7(i1,i6)
      zb34 = zB7(i3,i4)
      za41 = zA7(i4,i1)
      za27 = zA7(i2,i7)
      zb76 = zB7(i7,i6)
      za47 = zA7(i4,i7)
      za71 = zA7(i7,i1)
      s32  = s7(i3,i2)
      za25 = zA7(i2,i5)
      zb56 = zB7(i5,i6)
      zb41 = zB7(i4,i1)
      s14  = s7(i1,i4)
      s67 = s7(i6,i7)


      s234 = s7(i2,i3) + s7(i2,i4) + s7(i3,i4)
      s167 = s7(i1,i6) + s7(i1,i7) + s7(i6,i7)
      s235 = s7(i2,i3) + s7(i2,i5) + s7(i3,i5)
      s567 = s7(i5,i6) + s7(i5,i7) + s7(i6,i7)

      zampA2pmpmm = -zb53*za42*
     . (+zb32*za21*zb16+zb34*za41*zb16
     .  +zb32*za27*zb76+zb34*za47*zb76)
     . *za71/(zb34*s32*s234*s167)
     . +zb53**2*za25*zt2a(i4,i1,i7,i6)*za71
     .  /(zb34*s32*s235*s167)
     . +zb56*zt2a(i7,i5,i6,i3)*za42*zt2a(i1,i2,i4,i3)
     .  /(zb34*s32*s234*s567)
     . -zb53*zt2a(i2,i5,i3,i6)*zt2a(i7,i1,i4,i3)
     .  /(zb34*zb41*s32*s235)
     . +zb56*zt2a(i7,i5,i6,i3)*za41*zt2a(i2,i1,i4,i3)
     .  /(zb34*s14*s32*s567)

      zampA2pmpmm = zampA2pmpmm/s67

      return
      end function

*****************************************
* A57 in PRD 014020
*

      function zampA3pmmpm(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

      zt2a(j1,j2,j3,j4) = zA7(j1,j2)*zB7(j2,j4)+zA7(j1,j3)*zB7(j3,j4)

      zb53 = zB7(i5,i3)
      za42 = zA7(i4,i2)
      zb32 = zB7(i3,i2)
      za21 = zA7(i2,i1)
      zb16 = zB7(i1,i6)
      zb34 = zB7(i3,i4)
      za41 = zA7(i4,i1)
      za27 = zA7(i2,i7)
      zb76 = zB7(i7,i6)
      za47 = zA7(i4,i7)
      za71 = zA7(i7,i1)
      s24  = s7(i2,i4)
      zb56 = zB7(i5,i6)
      s67 = s7(i6,i7)


      s234 = s7(i2,i3) + s7(i2,i4) + s7(i3,i4)
      s167 = s7(i1,i6) + s7(i1,i7) + s7(i6,i7)
      s567 = s7(i5,i6) + s7(i5,i7) + s7(i6,i7)

      zampA3pmmpm=-zb53*za42*
     . (+zb32*za21*zb16+zb34*za41*zb16
     .  +zb32*za27*zb76+zb34*za47*zb76)
     . *za71/(zb34*s24*s234*s167)
     . +zb56*zt2a(i7,i5,i6,i3)*za42*zt2a(i1,i2,i4,i3)
     . /(zb34*s24*s234*s567)


      zampA3pmmpm = zampA3pmmpm/s67
      return
      end function

*****************************************
* A58 in PRD 014020
*

      function zampA4pmpmm(i1,i2,i3,i4,i5,i6,i7)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

      zt2a(j1,j2,j3,j4) = zA7(j1,j2)*zB7(j2,j4)+zA7(j1,j3)*zB7(j3,j4)

      zb53 = zB7(i5,i3)
      za42 = zA7(i4,i2)
      zb32 = zB7(i3,i2)
      za21 = zA7(i2,i1)
      zb16 = zB7(i1,i6)
      za27 = zA7(i2,i7)
      zb76 = zB7(i7,i6)
      zb34 = zB7(i3,i4)
      za41 = zA7(i4,i1)
      za47 = zA7(i4,i7)
      za71 = zA7(i7,i1)
      s24  = s7(i2,i4)
      zb56 = zB7(i5,i6)
      s67 = s7(i6,i7)


      s567 = s7(i5,i6) + s7(i5,i7) + s7(i6,i7)
      s167 = s7(i1,i6) + s7(i6,i7) + s7(i1,i7)
      s234 = s7(i2,i3) + s7(i2,i4) + s7(i3,i4)

      zampA4pmpmm = zb53*za42*
     . (+zb32*za21*zb16+zb32*za27*zb76
     .  +zb34*za41*zb16+zb34*za47*zb76)
     . *za71/(zb34*s24*s234*s167)
     . -zb56*zt2a(i7,i5,i6,i3)*za42*zt2a(i1,i2,i4,i3)
     .  /(zb34*s24*s234*s567)

      zampA4pmpmm = zampA4pmpmm/s67

      return
      end function


      end module C1gZ
