**********************************************************************
*
* Returns the subleading colour matrix element for the process
*
* q(1) + qbar(2) -> q(3) + qbar(4) + e-(5) + e+(6)
*
* There is only a subleading colour term for matching quark pairs
*
* Ignoring overall factors
*
* The Z boson couples to the i1 and i4 pair of quarks
*


      function D0g0Z(i1,i2,i3,i4,i5,i6)
      use KinData_mod
      use C0gZ, only : zampC0g0ZGen
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      complex*16 zampqqbA(2,2),zampqqbB(2,2),zampqRbA(2,2),zampqRbB(2,2)
      complex*16 zampA(2,2,2),zampB(2,2,2)

      D0g0Z = 0d0

      if(nf1.eq.0)then
         write(*,*) "flavour in D0g0Z has not been initialised"
         stop
      endif

      if(nf1.eq.nf2)then

      call eweakV2qq(nf1,nf2,s6(i5,i6))
c     zampqqbA and zampqqbB are the two possible helicity orderings

      call zampC0g0ZGen(i1,i3,i2,i4,i5,i6,zampqqbA,zampqqbB)
      call zampC0g0ZGen(i1,i3,i4,i2,i5,i6,zampqRbA,zampqRbB)


c     q L, e L
      zampA(1,1,1) = zewfac1(1)*zampqRbA(1,1) + zewfac2(1)*zampqRbB(1,1)
      zampB(1,1,1) = zewfac1(1)*zampqqbA(1,1) + zewfac2(1)*zampqqbB(1,1)

c     q L, e R
      zampA(1,1,2) = zewfac1(2)*zampqRbA(2,2) + zewfac2(2)*zampqRbB(2,2)
      zampB(1,1,2) = zewfac1(2)*zampqqbA(2,2) + zewfac2(2)*zampqqbB(2,2)

c     q R, e L
      zampA(2,2,1) = zewfac1(3)*zampqRbA(2,2) + zewfac2(3)*zampqRbB(2,2)
      zampB(2,2,1) = zewfac1(3)*zampqqbA(2,2) + zewfac2(3)*zampqqbB(2,2)

c     q R, e R
      zampA(2,2,2) = zewfac1(4)*zampqRbA(1,1) + zewfac2(4)*zampqRbB(1,1)
      zampB(2,2,2) = zewfac1(4)*zampqqbA(1,1) + zewfac2(4)*zampqqbB(1,1)


      D0g0Z =-2d0*(+dble(zampA(1,1,1)*dconjg(zampB(1,1,1)))
     .       +dble(zampA(1,1,2)*dconjg(zampB(1,1,2)))
     .       +dble(zampA(2,2,1)*dconjg(zampB(2,2,1)))
     .       +dble(zampA(2,2,2)*dconjg(zampB(2,2,2))))

      endif


      return
      end


************************************************************
* A wrapper function for D0g0Z where it inherits the flavours
* from the matrix element B2g0Z
*
*

      function BD0g0Z(i1,i2,i3,i4,i5,i6)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /BZFlav/ nf2g
!$omp threadprivate(/BZFlav/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)

      nf1 = nf2g
      nf2 = nf2g

      BD0g0Z = D0g0Z(i1,i2,i3,i4,i5,i6)

      return
      end


