****************************************************
* Returns the virtual one-loop matrix element for the process
*
* q(1) + qbar(4) -> Q(2) + Qbar(3) + e+(5) + e-(6)
*
* Ignoring overall factors, leading colour identical quark contribution including axial contribution
*
* The Z boson couples to the i1 and i4 pair of quarks


      function D0g1Z(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      use C0gZ
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /order/iorder
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      complex*16 zampTree(2,2,2),zampTreeA(2,2,2),zampTreeB(2,2,2)
      complex*16 zampTreeA2(2,2,2),zampTreeB2(2,2,2)
      complex*16 zampLoop(2,2,2),zampLoopA(2,2,2),zampLoopB(2,2,2)
      complex*16 zampLoopA2(2,2,2),zampLoopB2(2,2,2)
      complex*16 zampLoop2(2,2,2),zampTree2(2,2,2)

      D0g1Z = 0d0
      call eweakV2qq(nf1,nf2,s6(i5,i6))

      if(nf1.eq.nf2)then ! same flavour

      zampTreeA = zampC0g1ZTree(i1,i2,i3,i4,i5,i6)
      zampTreeB = zampC0g1ZTree(i2,i1,i4,i3,i5,i6)

      zampLoopA = zampC0g1ZLoop(i1,i4,i2,i3,i5,i6,renscale2,2)
      zampLoopB = zampC0g1ZLoop(i4,i1,i3,i2,i5,i6,renscale2,2)

c     Crossed tree amplitudes

      zampTreeA2 = zampC0g1ZTree(i1,i4,i3,i2,i5,i6)
      zampTreeB2 = zampC0g1ZTree(i4,i1,i2,i3,i5,i6)

      zampLoopA2 = zampC0g1ZLoop(i1,i2,i4,i3,i5,i6,renscale2,2)
      zampLoopB2 = zampC0g1ZLoop(i2,i1,i3,i4,i5,i6,renscale2,2)

      do ii=1,2
         do kk=1,2
            jj=ii
            if ((ii.eq.1).and.(kk.eq.1))then
               i = 1
            elseif((ii.eq.1).and.(kk.eq.2))then
               i = 2
            elseif((ii.eq.2).and.(kk.eq.1))then
               i = 3
            else
               i = 4
            endif

            if ((jj.eq.1).and.(kk.eq.1))then
               j = 1
            elseif((jj.eq.1).and.(kk.eq.2))then
               j = 2
            elseif((jj.eq.2).and.(kk.eq.1))then
               j = 3
            else
               j = 4
            endif

            zampLoop(ii,jj,kk) =+zewfac1(i)*zampLoopA(ii,jj,kk)
     .                          -zewfac2(j)*zampLoopB(3-jj,3-ii,kk)

            zampTree(ii,jj,kk) =+zewfac1(i)*zampTreeA(ii,jj,kk)
     .                          -zewfac2(j)*zampTreeB(3-jj,3-ii,kk)

            zampLoop2(ii,jj,kk) =+zewfac1(i)*zampLoopA2(ii,jj,kk)
     .                           -zewfac2(j)*zampLoopB2(3-jj,3-ii,kk)

            zampTree2(ii,jj,kk) =+zewfac1(i)*zampTreeA2(ii,jj,kk)
     .                           -zewfac2(j)*zampTreeB2(3-jj,3-ii,kk)


            D0g1Z=D0g1Z -dble(zampTree(ii,jj,kk)*dconjg(zampLoop(ii,jj,kk)))
     .                  -dble(zampTree2(ii,jj,kk)*dconjg(zampLoop2(ii,jj,kk)))
         enddo
      enddo

      if(iorder.eq.-1)then      ! UV renormalisation (MSbar scheme)
         subuv = 11d0/3d0
         D0g1Z = D0g1Z + subuv*D0g0Z(i1,i2,i3,i4,i5,i6)

      elseif (iorder.eq.0)then
!     The conversion from FDH -> HV is +2/3, see hep-ph/9708239v1 equation 6.5
         e0 = -(2d0/3d0) + pi**2/6d0
         D0g1Z = D0g1Z - e0*D0g0Z(i1,i2,i3,i4,i5,i6)
      endif

      endif
      return
      end

****************************************************
* Returns the virtual one-loop matrix element for the process
*
* q(1) + qbar(4) -> Q(2) + Qbar(3) + e+(5) + e-(6)
*
* Ignoring overall factors, subleading colour identical quark contribution
*
* The Z boson couples to the i1 and i4 pair of quarks

      function Dt0g1Z(i1,i2,i3,i4,i5,i6,renscale2)
      use Process_mod, only : nPDFs_proc
      use KinData_mod
      use C0gZ
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /order/iorder
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      complex*16 zampTreeA(2,2,2),zampTreeB(2,2,2)
      complex*16 zampTreeA2(2,2,2),zampTreeB2(2,2,2)
      complex*16 zampLoopA(2,2,2),zampLoopB(2,2,2)
      complex*16 zampLoopA2(2,2,2),zampLoopB2(2,2,2)
      complex*16 zampTree(2,2,2),zampLoop(2,2,2)
      complex*16 zampLoop2(2,2,2),zampTree2(2,2,2)

      if(nPDFs_proc.ne.2) iaxialflag=0

      Dt0g1Z = 0d0

      if (nf1.eq.nf2)then       ! same flavour

         call eweakV2qq(nf1,nf2,s6(i5,i6))

         zampTreeA = zampC0g1ZTree(i1,i2,i3,i4,i5,i6)
         zampTreeB = zampC0g1ZTree(i2,i1,i4,i3,i5,i6)

!     1/N^2 amplitudes in A6;2 (e.q 2.8) hep-ph/9610370v1

         zampLoopA =+zampC0g1ZLoop(i1,i4,i2,i3,i5,i6,renscale2,2)
     .              +zampC0g1ZLoop(i1,i3,i2,i4,i5,i6,renscale2,1)
     .              -zampC0g1ZLoopSL(i1,i3,i2,i4,i5,i6,renscale2)

         zampLoopB =+zampC0g1ZLoop(i4,i1,i3,i2,i5,i6,renscale2,2)
     .              +zampC0g1ZLoop(i4,i2,i3,i1,i5,i6,renscale2,1)
     .              -zampC0g1ZLoopSL(i4,i2,i3,i1,i5,i6,renscale2)


c     Crossed tree amplitudes
         zampTreeA2 = zampC0g1ZTree(i1,i4,i3,i2,i5,i6)
         zampTreeB2 = zampC0g1ZTree(i4,i1,i2,i3,i5,i6)

         zampLoopA2 =+zampC0g1ZLoop(i1,i2,i4,i3,i5,i6,renscale2,2)
     .               +zampC0g1ZLoop(i1,i3,i4,i2,i5,i6,renscale2,1)
     .               -zampC0g1ZLoopSL(i1,i3,i4,i2,i5,i6,renscale2)


         zampLoopB2 =+zampC0g1ZLoop(i2,i1,i3,i4,i5,i6,renscale2,2)
     .               +zampC0g1ZLoop(i2,i4,i3,i1,i5,i6,renscale2,1)
     .               -zampC0g1ZLoopSL(i2,i4,i3,i1,i5,i6,renscale2)

      do ii=1,2
         do kk=1,2
            do jj=1,2

               if ((ii.eq.1).and.(kk.eq.1))then
                  i = 1
               elseif((ii.eq.1).and.(kk.eq.2))then
                  i = 2
               elseif((ii.eq.2).and.(kk.eq.1))then
                  i = 3
               else
                  i = 4
               endif

               if ((jj.eq.1).and.(kk.eq.1))then
                  j = 1
               elseif((jj.eq.1).and.(kk.eq.2))then
                  j = 2
               elseif((jj.eq.2).and.(kk.eq.1))then
                  j = 3
               else
                  j = 4
               endif

               if (ii.eq.jj)then ! same helicity

                   zampTree(ii,jj,kk)=+zewfac1(i)*zampTreeA2(ii,jj,kk)
     .                               -zewfac2(j)*zampTreeB2(3-jj,3-ii,kk)

                  zampLoop(ii,jj,kk)=+zewfac1(i)*zampLoopA2(ii,jj,kk)
     .                               -zewfac2(j)*zampLoopB2(3-jj,3-ii,kk)

                  zampTree2(ii,jj,kk)=+zewfac1(i)*zampTreeA(ii,jj,kk)
     .                                -zewfac2(j)*zampTreeB(3-jj,3-ii,kk)

                  zampLoop2(ii,jj,kk)=+zewfac1(i)*zampLoopA(ii,jj,kk)
     .                                -zewfac2(j)*zampLoopB(3-jj,3-ii,kk)

                  Dt0g1Z=Dt0g1Z -dble(zampTree(ii,jj,kk)*dconjg(zampLoop(ii,jj,kk)))
     .                 -dble(zampTree2(ii,jj,kk)*dconjg(zampLoop2(ii,jj,kk)))

               endif
            enddo
         enddo
      enddo
      endif

      if (iorder.eq.0)then
!     The conversion from FDH -> HV is +1, see hep-ph/9708239v1 equation 6.5 !-pi**2 from Cgamma expansion.
         e0 = -pi**2/6d0 + 1d0
         Dt0g1Z = Dt0g1Z - e0*D0g0Z(i1,i2,i3,i4,i5,i6)
      endif

      return
      end

****************************************************
* Returns the virtual one-loop matrix element for the process
*
* q(1) + qbar(4) -> Q(2) + Qbar(3) + e+(5) + e-(6)
*
* Ignoring overall factors, nf loop contribution subleading colour
*
* The Z boson couples to the i1 and i4 pair of quarks

      function Dh0g1Z(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      use C0gZ
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      common /order/iorder
      complex*16 zampTreeA(2,2,2),zampTreeB(2,2,2)
      complex*16 zampTree(2,2,2),zampLoopnf(2,2,2)
      complex*16 zampTreeA2(2,2,2),zampTreeB2(2,2,2)
      complex*16 zampTree2(2,2,2),zampLoopnf2(2,2,2)

      Dh0g1Z = 0d0

      call eweakV2qq(nf1,nf2,s6(i5,i6))

      if(nf1.eq.nf2)then

      zampTreeA = zampC0g1ZTree(i1,i2,i3,i4,i5,i6)
      zampTreeB = zampC0g1ZTree(i2,i1,i4,i3,i5,i6)
      zampTreeA2 = zampC0g1ZTree(i1,i4,i3,i2,i5,i6)
      zampTreeB2 = zampC0g1ZTree(i4,i1,i2,i3,i5,i6)

      if(iorder.eq.0)then

*          We must compute the tree level matrix element before the electroweak interference. The reason for this is that the factors that multiply the QCD amplitudes are scale dependent.

         zfacA = 2d0/3d0*zzLogRat(renscale2,-s6(i2,i3))+(10d0/9d0)
         zfacB = 2d0/3d0*zzLogRat(renscale2,-s6(i1,i4))+(10d0/9d0)
         zfacA2 = 2d0/3d0*zzLogRat(renscale2,-s6(i3,i4))+(10d0/9d0)
         zfacB2 = 2d0/3d0*zzLogRat(renscale2,-s6(i1,i2))+(10d0/9d0)

         do ii=1,2
            do kk=1,2
               do jj=1,2

                  if ((ii.eq.1).and.(kk.eq.1))then
                     i = 1
                  elseif((ii.eq.1).and.(kk.eq.2))then
                     i = 2
                  elseif((ii.eq.2).and.(kk.eq.1))then
                     i = 3
                  else
                     i = 4
                  endif

                  if ((jj.eq.1).and.(kk.eq.1))then
                     j = 1
                  elseif((jj.eq.1).and.(kk.eq.2))then
                     j = 2
                  elseif((jj.eq.2).and.(kk.eq.1))then
                     j = 3
                  else
                     j = 4
                  endif

                  if (ii.eq.jj)then

                     zamptree(ii,jj,kk) = +zewfac1(i)*zampTreeA(ii,jj,kk)
     .                                    -zewfac2(j)*zampTreeB(3-jj,3-ii,kk)

                     zampLoopnf(ii,jj,kk) = +zfacA2*zewfac1(i)*zampTreeA2(ii,jj,kk)
     .                                      -zfacB2*zewfac2(j)*zampTreeB2(3-jj,3-ii,kk)

                     zampTree2(ii,jj,kk) = +zewfac1(i)*zampTreeA2(ii,jj,kk)
     .                                     -zewfac2(j)*zampTreeB2(3-jj,3-ii,kk)

                     zampLoopnf2(ii,jj,kk) = +zfacA*zewfac1(i)*zampTreeA(ii,jj,kk)
     .                                       -zfacB*zewfac2(j)*zampTreeB(3-jj,3-ii,kk)

                     Dh0g1Z = Dh0g1Z -dble(zampTree(ii,jj,kk)*dconjg(zampLoopnf(ii,jj,kk)))
     .                    -dble(zampTree2(ii,jj,kk)*dconjg(zampLoopnf2(ii,jj,kk)))

                  endif
               enddo
            enddo
         enddo
      endif
      endif

      return
      end

****************************************************
* Returns the virtual one-loop matrix element for the process
*
* q(1) + qbar(4) -> Q(2) + Qbar(3) + e+(5) + e-(6)
*
* Ignoring overall factors, finite top correction
*
* The Z boson couples to the i1 and i4 pair of quarks
*
* Only valid in the large top mass region (low incoming s),
* as the corrections are expansions in sij/mt**2

      function D0g1Ztop(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      use Parameters_mod
      use C0gZ
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      double precision :: emt,etwidth,emb,ebwidth,emc,ecwidth,emtau,etauwidth
      common /Qmass/ emt,etwidth,emb,ebwidth,emc,ecwidth,emtau,etauwidth
      common /order/iorder
      parameter(pi=3.141592653589793238d0)
      complex*16 zampTreeA(2,2,2),zampTreeB(2,2,2)
      complex*16 zampTree(2,2,2),zampLoopnf(2,2,2)
      complex*16 zampTreeA2(2,2,2),zampTreeB2(2,2,2)
      complex*16 zampTree2(2,2,2),zampLoopnf2(2,2,2)

      D0g1Ztop = 0d0

      if(emt.eq.0d0)then
         write(*,*) "Mass of the top has not been initialised. This is
     . required for the C0g1Ztop function"
         stop
      endif

      if(nf1.eq.nf2)then

      zampTreeA = zampC0g1ZTree(i1,i2,i3,i4,i5,i6)
      zampTreeB = zampC0g1ZTree(i2,i1,i4,i3,i5,i6)
      zampTreeA2 = zampC0g1ZTree(i1,i4,i3,i2,i5,i6)
      zampTreeB2 = zampC0g1ZTree(i4,i1,i2,i3,i5,i6)

      call eweakV2qq(nf1,nf2,s6(i5,i6))

      if(iorder.eq.0)then

*     We must compute the tree level matrix element before the electroweak interference. The reason for this is that the factors that multiply the QCD amplitudes are scale dependent.

         zfacA = zzselftop(s6(i2,i3))
         zfacB = zzselftop(s6(i1,i4))
         zfacA2 =zzselftop(s6(i3,i4))
         zfacB2 =zzselftop(s6(i1,i2))

         do ii=1,2
            do kk=1,2
               do jj=1,2

                  if ((ii.eq.1).and.(kk.eq.1))then
                     i = 1
                  elseif((ii.eq.1).and.(kk.eq.2))then
                     i = 2
                  elseif((ii.eq.2).and.(kk.eq.1))then
                     i = 3
                  else
                     i = 4
                  endif

                  if ((jj.eq.1).and.(kk.eq.1))then
                     j = 1
                  elseif((jj.eq.1).and.(kk.eq.2))then
                     j = 2
                  elseif((jj.eq.2).and.(kk.eq.1))then
                     j = 3
                  else
                     j = 4
                  endif

                  if (ii.eq.jj)then

                     zampTree(ii,jj,kk) = zewfac1(i)*zampTreeA(ii,jj,kk)
     .                                   -zewfac2(j)*zampTreeB(3-jj,3-ii,kk)

                     zampLoopnf(ii,jj,kk) = zfacA2*zewfac1(i)*zampTreeA2(ii,jj,kk)
     .                                     -zfacB2*zewfac2(j)*zampTreeB2(3-jj,3-ii,kk)

                     zampTree2(ii,jj,kk) = zewfac1(i)*zampTreeA2(ii,jj,kk)
     .                                    -zewfac2(j)*zampTreeB2(3-jj,3-ii,kk)

                     zampLoopnf2(ii,jj,kk) = zfacA*zewfac1(i)*zampTreeA(ii,jj,kk)
     .                                      -zfacB*zewfac2(j)*zampTreeB(3-jj,3-ii,kk)

                     D0g1Ztop = D0g1Ztop +dble(zampTree(ii,jj,kk)*dconjg(zampLoopnf(ii,jj,kk)))
     .                    +dble(zampTree2(ii,jj,kk)*dconjg(zampLoopnf2(ii,jj,kk)))

                  endif
               enddo
            enddo
         enddo
      endif
      endif

      return
      end


***********************************************************
* Returns the virtual one-loop matrix element for the process
*
* q(1) + qbar(4) -> Q(2) + Qbar(3) + e+(5) + e-(6)
*
* Ignoring overall factors, subleading colour axial correction
*
* The Z boson couples to the i1 and i4 pair of quarks
*
* Only valid in the large top mass region (low incoming s),
* as the corrections are expansions in sij/mt**2

      function D0g1Zax(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      use C0gZ
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /eweakZ/ amz,zewfac1(4),zewfac2(4)
!$omp threadprivate(/eweakZ/)
      common /axial/zewax(2)
!$omp threadprivate(/axial/)
      common /order/iorder
      common /CZFlav/ nf1,nf2
!$omp threadprivate(/CZFlav/)
      complex*16 zampTreeA(2,2,2),zampTreeB(2,2,2)
      complex*16 zampTree(2,2,2),zampLoop(2,2,2)
      complex*16 zampAx(2,2,2),zampAx2(2,2,2)
      complex*16 zampTree2(2,2,2),zampTreeA2(2,2,2),zampTreeB2(2,2,2)

      D0g1Zax=0d0

      if (iorder.eq.0)then
      if(nf1.eq.nf2)then

         zampAx2 = zampC0g1ZaxA(i1,i2,i4,i3,i5,i6)
         zampAx = zampC0g1ZaxA(i1,i4,i2,i3,i5,i6)

         zampTreeA = zampC0g1ZTree(i1,i2,i3,i4,i5,i6)
         zampTreeB = zampC0g1ZTree(i2,i1,i4,i3,i5,i6)
         zampTreeA2 = zampC0g1ZTree(i1,i4,i3,i2,i5,i6)
         zampTreeB2 = zampC0g1ZTree(i4,i1,i2,i3,i5,i6)

         call eweakV2qq(nf1,nf2,s6(i5,i6))

         do ii=1,2
            do kk=1,2
               jj=ii

               if ((ii.eq.1).and.(kk.eq.1))then
                  i = 1
               elseif((ii.eq.1).and.(kk.eq.2))then
                  i = 2
               elseif((ii.eq.2).and.(kk.eq.1))then
                  i = 3
               else
                  i = 4
               endif

               if ((jj.eq.1).and.(kk.eq.1))then
                  j = 1
               elseif((jj.eq.1).and.(kk.eq.2))then
                  j = 2
               elseif((jj.eq.2).and.(kk.eq.1))then
                  j = 3
               else
                  j = 4
               endif

               zampTree(ii,jj,kk) =  zewfac1(i)*zampTreeA(ii,jj,kk)
     .                              -zewfac2(j)*zampTreeB(3-jj,3-ii,kk)
               zampTree2(ii,jj,kk) = zewfac1(i)*zampTreeA2(ii,jj,kk)
     .                              -zewfac2(j)*zampTreeB2(3-jj,3-ii,kk)
               D0g1Zax = D0g1Zax +dble(zampTree(ii,jj,kk)*dconjg(zewax(kk)*zampAx(ii,jj,kk)))
     .                           +dble(zampTree2(ii,jj,kk)*dconjg(zewax(kk)*zampAx2(ii,jj,kk)))
            enddo
         enddo

      endif
      endif
      return
      end


