************************************************************
* Note that this function is similar to zzlog in src/core, however
* it also takes into account the signs of the x,y for the
* correct determination of the sign of epsilon
*
      function zzLograt(x,y)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      htheta(t)=0.5d0+0.5d0*sign(1d0,t)
      zzLogRat = dcmplx(dlog(abs(x/y)))-(0d0,pi)*dcmplx(htheta(-x)-htheta(-y))
      return
      end

      function zlsm1(x1,y1,x2,y2)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      pi2o6 = pi**2/6d0
      r1 = x1/y1
      r2 = x2/y2
      if (r1 .lt. 0d0) then
         zdilog1 = dcmplx(pi2o6 - dli2(r1)) - zzlogRat(x1,y1)*dcmplx(log(1d0-r1))
      else
         zdilog1 = dcmplx(dli2(1d0-r1))
      endif
      if (r2 .lt. 0d0) then
         zdilog2 = dcmplx(pi2o6 - dli2(r2)) - zzlogRat(x2,y2)*dcmplx(log(1d0-r2))
      else
         zdilog2 = dcmplx(dli2(1d0-r2))
      endif
      zlsm1 = zdilog1 + zdilog2 + zzlogRat(x1,y1)*zzlogRat(x2,y2) - dcmplx(pi2o6)
      return
      end

      function zL0d(x,y)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      denom = 1d0 - x/y
      if (abs(denom) .lt. 1d-7) then
         zL0d = -dcmplx(1d0) - dcmplx(denom*(0.5d0 + denom/3d0))
      else
         zL0d = zzlogRat(x,y)/dcmplx(denom)
      endif
      return
      end

      function zL1d(x,y)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      denom = 1d0 - x/y
      if (abs(denom) .lt. 1d-7) then
         zL1d = -dcmplx(0.5d0) - dcmplx(denom/3d0*(1d0 + 0.75d0*denom))
      else
         zL1d = (zL0d(x,y)+dcmplx(1d0))/dcmplx(denom)
      endif
      return
      end

      function zL2d(x,y)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      denom = 1d0 - x/y
      if (abs(denom) .lt. 1d-7) then
         zL2d = -dcmplx(1d0/6d0) + dcmplx(denom/2d0*(0.5d0 + 0.6d0*denom))
      else
         zL2d = (zL0d(x,y)+dcmplx(0.5d0*(1d0+y/x)))/dcmplx(denom**2)
      endif
      return
      end

      function zLsm1_2me(s,t,em1sq,em3sq)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      double precision arg(5),omarg(5)
      double complex zlog(5),zLi2(5)

      htheta(x) = (1d0 + sign(1d0,x))/2d0
 
      arg1 = em1sq/s
      arg2 = em1sq/t
      arg3 = em3sq/s
      arg4 = em3sq/t
      arg5 = arg1*arg4

      omarg1 = 1d0-arg1
      omarg2 = 1d0-arg2
      omarg3 = 1d0-arg3
      omarg4 = 1d0-arg4
      omarg5 = 1d0-arg5

      if (omarg1.gt.1d0) then
         zdilog1 = dcmplx(pi**2/6d0-dli2(arg1))
     &   - zzlogRat(-em1sq,-s)*dcmplx(log(omarg1))
      else
         zdilog1=dcmplx(dli2(omarg1))
      endif
      if (omarg2.gt.1d0) then
         zdilog2 = dcmplx(pi**2/6d0-dli2(arg2))
     &   - zzlogRat(-em1sq,-t)*dcmplx(log(omarg2))
      else
         zdilog2=dcmplx(dli2(omarg2))
      endif
      if (omarg3.gt.1d0) then
         zdilog3 = dcmplx(pi**2/6d0-dli2(arg3))
     &   - zzlogRat(-em3sq,-s)*dcmplx(log(omarg3))
      else
         zdilog3=dcmplx(dli2(omarg3))
      endif
      if (omarg4.gt.1d0) then
         zdilog4 = dcmplx(pi**2/6d0-dli2(arg4))
     &   - zzlogRat(-em3sq,-t)*dcmplx(log(omarg4))
      else
         zdilog4=dcmplx(dli2(omarg4))
      endif
      if (omarg5.gt.1d0) then
         zdilog5 = dcmplx(pi**2/6d0-dli2(arg5))
     &   - (zzlogRat(-em1sq,-s)+zzlogRat(-em3sq,-t))*dcmplx(log(omarg5))
      else
         zdilog5=dcmplx(dli2(omarg5))
      endif
      zLsm1_2me = - zdilog1 - zdilog2 - zdilog3 - zdilog4 + zdilog5
     &            - 0.5d0*(zzlogRat(-s,-t))**2

      return
      end

      function zI3m(s1,s2,s3)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
C     This is the function zI3m, a massless triangle with all three external
C     lines offshell defined in BDK. Obtained from MCFM i3m.f

      smax = max(s1,s2,s3)
      smin = min(s1,s2,s3)
      smid = s1+s2+s3-smax-smin
      del3 = s1**2+s2**2+s3**2-2d0*(s1*s2+s2*s3+s3*s1)



      if (del3 .gt. 0) then
         rtdel3 = sqrt(del3)
         if (smax .lt. 0) then
c     case all negative
            flag=0d0
            zI3m = zI3m1b(smax,smid,smin,rtdel3,flag)
         elseif (smin .gt. 0) then
c     case all positive
            flag=0d0
            zI3m = -zI3m1b(-smin,-smid,-smax,rtdel3,flag)
         elseif ((smid .lt. 0) .and. (smin .lt. 0)) then
c     case of two negative and one positive
            flag=+1d0
            zI3m = zI3m1b(smin,smid,smax,rtdel3,flag)
         elseif ((smax .gt. 0).and.(smid .gt. 0)) then
c     case of two positive and one negative
            flag=-1d0
            zI3m = -zI3m1b(-smax,-smid,-smin,rtdel3,flag)
         endif
      elseif (del3 .lt. 0) then
         rtdel3 = sqrt(-del3)
         if (smax .lt. 0) then
c     case all negative
            zI3m=dcmplx(eI3m1a(s1,s2,s3,rtdel3))
         elseif (smin .gt. 0) then
            zI3m=-dcmplx(eI3m1a(-s1,-s2,-s3,rtdel3))
         endif
      endif

      return
      end

      function eI3m1a(s1,s2,s3,rtmdel)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)

      d1=s1-s2-s3
      d2=s2-s3-s1
      d3=s3-s1-s2

      arg1=2d0*datan(rtmdel/d1)
      arg2=2d0*datan(rtmdel/d2)
      arg3=2d0*datan(rtmdel/d3)

      eI3m1a=2d0/rtmdel*(Dclaus(arg1)+Dclaus(arg2)+Dclaus(arg3))

      end

      function zI3m1b(s1,s2,s3,rtdel,flag)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)

      d3=s3-s1-s2
      x=s1/s3
      y=s2/s3
      rat=0.5d0*(d3+rtdel)/s3
      if (abs(rat) .lt. 1d-3) rat=2d0*s1*s2/(s3*(d3-rtdel))
      rho=1d0/rat
      argx=rho*x
      argy=rho*y
      argdlx=-argx
      argdly=-argy


      if ((argdlx .gt. 1d0) .or. (argdly .gt. 1d0)) then
      write(*,*) 'problems with call of I3m1b'
      write(*,*) 'argdlx',argdlx
      write(*,*) 'argdly',argdly
      stop
      endif

      xlog=log(abs(argx))
      ylog=log(abs(argy))
      temp=xlog*ylog+pi**2/3d0+(ylog-xlog)*log((1d0+argy)/(1d0+argx))
     . +2d0*(dli2(argdlx)+dli2(argdly))
      zI3m1b=Dcmplx(temp-abs(flag)*pi**2)+dcmplx(0,pi)*Dcmplx(flag*(xlog+ylog))
      zI3m1b=-zI3m1b/Dcmplx(rtdel)


      end


      function zLsm1_2mht(s,t,em1sq,em2sq)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)

      r1=em1sq/t
      r2=em2sq/t
      omr1 = 1d0-r1
      omr2 = 1d0-r2

      if (omr1 .gt. 1d0) then
         zdilog1=dcmplx(pi**2/6d0-dli2(r1)) - zzlogRat(-em1sq,-t)*dcmplx(log(omr1))
      else
         zdilog1=dcmplx(dli2(omr1))
      endif

      if (omr2 .gt. 1d0) then
         zdilog2=dcmplx(pi**2/6d0-dli2(r2)) - zzlogRat(-em2sq,-t)*dcmplx(log(omr2))
      else
         zdilog2=dcmplx(dli2(omr2))
      endif

      zlsm1_2mht=-zdilog1-zdilog2+0.5d0*(zzlogRat(-s,-em1sq)*zzlogRat(-s,-em2sq)
     .           -zzlogRat(-s,-t)**2)

      return
      end

      function zLsm1_2mh(s,t,em1sq,em2sq)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      zLsm1_2mh=zLsm1_2mht(s,t,em1sq,em2sq)
     .         +(0.5d0*(s-em1sq-em2sq)+em1sq*em2sq/t)*zI3m(s,em1sq,em2sq)

      return
      end

      function zLs0d(x1,y1,x2,y2)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      r1 = x1/y1
      r2 = x2/y2
      zLs0d = zLsm1(x1,y1,x2,y2)/dcmplx(1d0-r1-r2)
      return
      end

      function zLs1d(x1,y1,x2,y2)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      r1 = x1/y1
      r2 = x2/y2
      zLs1d=(zLs0d(x1,y1,x2,y2)+zL0d(x1,y1)+zL0d(x2,y2))/dcmplx(1d0-r1-r2)
      return
      end

      function zLs2d(x1,y1,x2,y2)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      r1 = x1/y1
      r2 = x2/y2
      zLs2d=(zLs1d(x1,y1,x2,y2)+(1d0/2d0)*(zL1d(x1,y1)+zL1d(x2,y2)))/dcmplx(1d0-r1-r2)
      return
      end

      function zLs3d(x1,y1,x2,y2)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      r1 = x1/y1
      r2 = x2/y2
      zLs3d=(zLs2d(x1,y1,x2,y2)+(1d0/3d0)*(zL2d(x1,y1)+zL2d(x2,y2))-(1d0/6d0)*(1d0/r1+1d0/r2))/dcmplx(1d0-r1-r2)
      return
      end

      function zFzip(s12,s34,s56)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
C     Implementation of BDKW Nucl Phys. 513, 3 (1998)
C     Eqn. (B15).
      Del12=s12-s34-s56
      Del34=s34-s12-s56
      Del56=s56-s12-s34
      Del3=s12*Del12+s34*Del34+s56*Del56

      zFzip=Dcmplx((3d0*s34*Del34/Del3**2-1d0/Del3)*s12*s56)
     . *zI3m(s12,s34,s56)
     . +Dcmplx((3d0*s56*Del56/Del3**2-0.5d0/Del3)*s12)
     . *zzlogRat(-s12,-s34)
     . +Dcmplx((3d0*s12*Del12/Del3**2-0.5d0/Del3)*s56)
     . *zzlogRat(-s56,-s34)
     . -Dcmplx(0.5d0*Del34/Del3)

      return
      end


      function zzselftop(s)
      use Parameters_mod
       implicit double precision (a-h,o-y)
       implicit complex*16 (z)
       double precision :: emt,etwidth,emb,ebwidth,emc,ecwidth,emtau,etauwidth
       common /Qmass/ emt,etwidth,emb,ebwidth,emc,ecwidth,emtau,etauwidth

       if(emt.eq.0)then
         write(*,*) "Error in zzselftop function,"
         write(*,*) "the mass of the top quark has not been defined"
         stop
       endif

       x=s/emt**2
       a1 = -(2d0/15d0)
       a2 = (3d0/28d0)
       a3 = (4d0/27d0)
       a4 = (15d0/88d0)
       a5 = (12d0/65d0)
       a6 = (7d0/36d0)
       a7 = (24d0/119d0)

       ans = a1*x*(1d0+a2*x*(1d0+a3*x*(1d0+a4*x*(1d0+a5*x*(1d0+a6*x*(1d0+a7*x))))))
       zzselftop = dcmplx(ans)

       return
       end

****************************************************
*     finite top mass corrections to axial
*     expanded through to (s/mt2)**6
*
       function zFmt(s12,s34,s56)
       use Parameters_mod
       implicit double precision (a-h,o-y)
       implicit complex*16 (z)
       double precision :: emt,etwidth,emb,ebwidth,emc,ecwidth,emtau,etauwidth
       common /Qmass/ emt,etwidth,emb,ebwidth,emc,ecwidth,emtau,etauwidth

       if(emt.eq.0)then
         write(*,*) "Error in zFmt function,"
         write(*,*) "the mass of the top quark has not been defined"
         stop
       endif

c      Previous implementation expanded through to (s/mt2)**2
       zFmt = dcmplx((1d0+(2d0*s34+s12+s56)/15d0/emt**2)/(24d0*emt**2))

       return
       end
