************************************************************************
!  All 2-loop amplitudes in this file are based on
! 
!  E.~W.~N.~Glover, C.~Oleari and M.~E.~Tejeda-Yeomans,
!  %``Two loop QCD corrections to gluon-gluon scattering,''
!  Nucl. Phys. B \textbf{605} (2001), 467-485
!  doi:10.1016/S0550-3213(01)00210-3
!  [arXiv:hep-ph/0102201 [hep-ph]].
!  
!  E.~W.~N.~Glover and M.~E.~Tejeda-Yeomans,
!  %``One loop QCD corrections to gluon-gluon scattering at NNLO,''
!  JHEP \textbf{05} (2001), 010
!  doi:10.1088/1126-6708/2001/05/010
!  [arXiv:hep-ph/0104178 [hep-ph]].
!  
************************************************************************
*
* four gluon two loop matrix elements interfered with tree level
*                     plus one-loop matrix elements squared
*
*                     g g g g
*
* summed over colour orderings, summed and squared
*
      function FullA4g2(ig1,ig2,ig3,ig4,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)

c      write(*,*)" hello FullA4g2"
      fac=(as*nc/2d0/pi)**2*(4d0*pi*as*nc)**2*(nc**2-1d0)*2d0
      rf=dble(nf)/dble(nc)

      FullA4g2=         +A4g2(ig1,ig2,ig3,ig4,renscale2)      
     .              +rf*Ah4g2(ig1,ig2,ig3,ig4,renscale2)              
     .           +rf**2*Ahh4g2(ig1,ig2,ig3,ig4,renscale2) 
     .     + 1d0/dble(nc)**2*(
     .                 +At4g2(ig1,ig2,ig3,ig4,renscale2)      
     .              +rf*Ath4g2(ig1,ig2,ig3,ig4,renscale2)              
     .           +rf**2*Athh4g2(ig1,ig2,ig3,ig4,renscale2))
     .     + 1d0/dble(nc)**4*(
     .          +rf**2*Atthh4g2(ig1,ig2,ig3,ig4,renscale2) )   

      FullA4g2=fac*FullA4g2

*c      write(*,*)"FullA4g2",FullA4g2
      return
      end
************************************************************************
*
      function A4g2(ig1,ig2,ig3,ig4,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)

      A4g2 =0d0
      if((ig1.eq.1).and.(ig2.eq.2).and.(ig3.eq.3).and.(ig4.eq.4))then
        A4g2 = finiteA4g2X(ig1,ig2,ig3,ig4,renscale2)
      endif
      
      A4g2=A4g2/2d0

      return
      end
*
*
************************************************************************
*      
      function Ah4g2(ig1,ig2,ig3,ig4,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)

      Ah4g2 =0d0
      if((ig1.eq.1).and.(ig2.eq.2).and.(ig3.eq.3).and.(ig4.eq.4))then
        Ah4g2 = finiteAh4g2X(ig1,ig2,ig3,ig4,renscale2)
      endif

      Ah4g2=Ah4g2/2d0

      return
      end
*
*
************************************************************************
*
      function Ahh4g2(ig1,ig2,ig3,ig4,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)

      Ahh4g2 =0d0
      if((ig1.eq.1).and.(ig2.eq.2).and.(ig3.eq.3).and.(ig4.eq.4))then
        Ahh4g2 = finiteAhh4g2X(ig1,ig2,ig3,ig4,renscale2)
      endif

      Ahh4g2=Ahh4g2/2d0

      return
      end
*
*
************************************************************************
*
      function At4g2(ig1,ig2,ig3,ig4,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)

      At4g2 =0d0
      if((ig1.eq.1).and.(ig2.eq.2).and.(ig3.eq.3).and.(ig4.eq.4))then
        At4g2 = finiteAt4g2X(ig1,ig2,ig3,ig4,renscale2)
      endif

      At4g2=At4g2/2d0

      return
      end
*
*
************************************************************************
*
      function Ath4g2(ig1,ig2,ig3,ig4,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)

      Ath4g2 =0d0
      if((ig1.eq.1).and.(ig2.eq.2).and.(ig3.eq.3).and.(ig4.eq.4))then
        Ath4g2 = finiteAth4g2X(ig1,ig2,ig3,ig4,renscale2)
      endif

      Ath4g2=Ath4g2/2d0

      return
      end
*
*
************************************************************************
*
      function Athh4g2(ig1,ig2,ig3,ig4,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)

      Athh4g2 =0d0
      if((ig1.eq.1).and.(ig2.eq.2).and.(ig3.eq.3).and.(ig4.eq.4))then
        Athh4g2 = finiteAthh4g2X(ig1,ig2,ig3,ig4,renscale2)
      endif

      Athh4g2=Athh4g2/2d0

      return
      end
*
*
************************************************************************
*
      function Atthh4g2(ig1,ig2,ig3,ig4,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)

      Atthh4g2 =0d0
      if((ig1.eq.1).and.(ig2.eq.2).and.(ig3.eq.3).and.(ig4.eq.4))then
        Atthh4g2 = finiteAtthh4g2X(ig1,ig2,ig3,ig4,renscale2)
      endif

      Atthh4g2=Atthh4g2/2d0
 
      return
      end
*
*
************************************************************************
*
      function finiteA4g2X(ig1,ig2,ig3,ig4,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)
      double precision dli2x,dli2omx,dli3x,dli3omx,dli4x,dli4omx,dli4xx
      parameter(pi=3.141592653589793238d0)
      parameter(zeta3=1.202056903159594285d0)
      common /order2l/ieorder
      s12 =s4(ig1,ig2)
      s23 =s4(ig2,ig3)
      s13 =s4(ig1,ig3)
      dlx   =log(-s23/s12)
      dlomx =log(-s13/s12)
      dls =log(s12/renscale2)
      dli2x   =rli2(-s23/s12)
      dli2omx =rli2(-s13/s12)
      dli3x   =rli3(-s23/s12)
      dli3omx =rli3(-s13/s12)
      dli4x   =rli4(-s23/s12)
      dli4omx =rli4(-s13/s12)
      dli4xx  =rli4(-s13/s23)
      if(ieorder.eq.-4)then
      finiteA4g2X =  0

      elseif(ieorder.eq.-3)then
      finiteA4g2X =  0

      elseif(ieorder.eq.-2)then
      finiteA4g2X =  0

      elseif(ieorder.eq.-1)then
      finiteA4g2X =  0

      elseif(ieorder.eq.0)then
      finiteA4g2X =  + zeta3 * ( 1168.D0/3.D0 + 1672.D0/9.D0*s12**(-2)*
     &    s23**2 + 1672.D0/9.D0*s12**(-1)*s23 + 1408.D0/9.D0*s13**(-2)*
     &    s23**2 + 1864.D0/9.D0*s13**(-1)*s23 + 1864.D0/9.D0*s13*
     &    s23**(-1) + 1408.D0/9.D0*s13**2*s23**(-2) )
      finiteA4g2X = finiteA4g2X + pi**2 * (  - 208.D0/9.D0 + 142.D0/9.D0
     &    *s12**(-2)*s23**2 + 142.D0/9.D0*s12**(-1)*s23 - 170.D0/9.D0*
     &    s13**(-2)*s23**2 + 86.D0/3.D0*s13**(-1)*s23 + 86.D0/3.D0*s13*
     &    s23**(-1) - 170.D0/9.D0*s13**2*s23**(-2) )
      finiteA4g2X = finiteA4g2X + pi**4 * ( 23.D0/3.D0 + s12**(-4)*
     &    s23**4 + 2.D0*s12**(-3)*s23**3 + 61.D0/15.D0*s12**(-2)*s23**2
     &     + 14.D0/3.D0*s12**(-1)*s23 + 26.D0/15.D0*s13**(-2)*s23**2 + 
     &    496.D0/45.D0*s13**(-1)*s23 + 332.D0/45.D0*s13*s23**(-1) + 26.D
     &    0/15.D0*s13**2*s23**(-2) )
      finiteA4g2X = finiteA4g2X + dli4xx * ( 128.D0 + 256.D0*s12**(-1)*
     &    s23 + 96.D0*s13**(-1)*s23 - 96.D0*s13*s23**(-1) )
      finiteA4g2X = finiteA4g2X + dli4omx * (  - 128.D0 + 96.D0*
     &    s12**(-1)*s23 - 96.D0*s13**(-1)*s23 - 256.D0*s13*s23**(-1) )
      finiteA4g2X = finiteA4g2X + dli4x * (  - 224.D0 - 96.D0*s12**(-1)
     &    *s23 - 256.D0*s13**(-1)*s23 - 96.D0*s13*s23**(-1) )
      finiteA4g2X = finiteA4g2X + dli3omx * (  - 304.D0/3.D0 - 88.D0/3.D
     &    0*s12**(-2)*s23**2 - 80.D0*s12**(-1)*s23 - 304.D0/3.D0*
     &    s13**(-1)*s23 - 64.D0/3.D0*s13*s23**(-1) + 88.D0/3.D0*s13**2*
     &    s23**(-2) )
      finiteA4g2X = finiteA4g2X + dli3x * (  - 152.D0/3.D0 - 88.D0/3.D0
     &    *s12**(-2)*s23**2 + 64.D0/3.D0*s12**(-1)*s23 + 88.D0/3.D0*
     &    s13**(-2)*s23**2 - 64.D0/3.D0*s13**(-1)*s23 - 304.D0/3.D0*s13
     &    *s23**(-1) )
      finiteA4g2X = finiteA4g2X + dli2x*pi**2 * ( 16.D0/3.D0 + 32.D0/3.D
     &    0*s12**(-1)*s23 - 32.D0/3.D0*s13**(-1)*s23 + 32.D0/3.D0*s13*
     &    s23**(-1) )
      finiteA4g2X = finiteA4g2X + dls * ( 5038.D0/3.D0 + 4840.D0/9.D0*
     &    s12**(-2)*s23**2 + 4840.D0/9.D0*s12**(-1)*s23 + 4840.D0/9.D0*
     &    s13**(-2)*s23**2 + 4840.D0/9.D0*s13**(-1)*s23 + 4840.D0/9.D0*
     &    s13*s23**(-1) + 4840.D0/9.D0*s13**2*s23**(-2) )
      finiteA4g2X = finiteA4g2X + dls*zeta3 * (  - 96.D0 - 32.D0*
     &    s12**(-2)*s23**2 - 32.D0*s12**(-1)*s23 - 32.D0*s13**(-2)*
     &    s23**2 - 32.D0*s13**(-1)*s23 - 32.D0*s13*s23**(-1) - 32.D0*
     &    s13**2*s23**(-2) )
      finiteA4g2X = finiteA4g2X + dls*pi**2 * (  - 242.D0/3.D0 - 44.D0/
     &    9.D0*s12**(-2)*s23**2 - 44.D0/9.D0*s12**(-1)*s23 - 440.D0/9.D0
     &    *s13**(-2)*s23**2 - 440.D0/9.D0*s13**(-1)*s23 - 440.D0/9.D0*
     &    s13*s23**(-1) - 440.D0/9.D0*s13**2*s23**(-2) )
      finiteA4g2X = finiteA4g2X + dls**2 * ( 484.D0 + 484.D0/3.D0*
     &    s12**(-2)*s23**2 + 484.D0/3.D0*s12**(-1)*s23 + 484.D0/3.D0*
     &    s13**(-2)*s23**2 + 484.D0/3.D0*s13**(-1)*s23 + 484.D0/3.D0*
     &    s13*s23**(-1) + 484.D0/3.D0*s13**2*s23**(-2) )
      finiteA4g2X = finiteA4g2X + dlomx * ( 5038.D0/9.D0 + 4564.D0/27.D0
     &    *s12**(-2)*s23**2 + 2452.D0/27.D0*s12**(-1)*s23 + 5392.D0/27.D
     &    0*s13**(-2)*s23**2 + 5392.D0/27.D0*s13**(-1)*s23 + 6676.D0/27.
     &    D0*s13*s23**(-1) + 4564.D0/27.D0*s13**2*s23**(-2) )
      finiteA4g2X = finiteA4g2X + dlomx*zeta3 * (  - 40.D0 - 8.D0*
     &    s12**(-2)*s23**2 - 48.D0*s12**(-1)*s23 - 16.D0*s13**(-2)*
     &    s23**2 - 64.D0*s13**(-1)*s23 - 48.D0*s13*s23**(-1) - 8.D0*
     &    s13**2*s23**(-2) )
      finiteA4g2X = finiteA4g2X + dlomx*pi**2 * (  - 52.D0/3.D0 - 4.D0*
     &    s12**(-3)*s23**3 - 109.D0/9.D0*s12**(-2)*s23**2 - 100.D0/3.D0
     &    *s12**(-1)*s23 - 374.D0/9.D0*s13**(-2)*s23**2 + 2.D0/3.D0*
     &    s13**(-1)*s23 + 146.D0/9.D0*s13*s23**(-1) - 299.D0/9.D0*
     &    s13**2*s23**(-2) + 8.D0*s13**3*s23**(-3) )
      finiteA4g2X = finiteA4g2X + dlomx*dli3omx * ( 128.D0 + 80.D0*
     &    s12**(-1)*s23 + 96.D0*s13**(-1)*s23 + 80.D0*s13*s23**(-1) )
      finiteA4g2X = finiteA4g2X + dlomx*dli3x * (  - 104.D0 + 128.D0*
     &    s12**(-1)*s23 + 48.D0*s13**(-1)*s23 - 48.D0*s13*s23**(-1) )
      finiteA4g2X = finiteA4g2X + dlomx*dli2x * (  - 304.D0/3.D0 - 88.D0
     &    /3.D0*s12**(-2)*s23**2 - 80.D0*s12**(-1)*s23 - 304.D0/3.D0*
     &    s13**(-1)*s23 - 64.D0/3.D0*s13*s23**(-1) + 88.D0/3.D0*s13**2*
     &    s23**(-2) )
      finiteA4g2X = finiteA4g2X + dlomx*dls * ( 968.D0/3.D0 + 484.D0/3.D
     &    0*s12**(-2)*s23**2 + 308.D0/3.D0*s12**(-1)*s23 + 220.D0*s13*
     &    s23**(-1) + 484.D0/3.D0*s13**2*s23**(-2) )
      finiteA4g2X = finiteA4g2X + dlomx**2 * (  - 1094.D0/9.D0 + 332.D0/
     &    9.D0*s12**(-2)*s23**2 + 4.D0*s12**(-1)*s23 - 536.D0/9.D0*
     &    s13**(-2)*s23**2 - 536.D0/9.D0*s13**(-1)*s23 + 628.D0/9.D0*
     &    s13*s23**(-1) + 332.D0/9.D0*s13**2*s23**(-2) )
      finiteA4g2X = finiteA4g2X + dlomx**2*pi**2 * ( 146.D0/3.D0 + 2.D0
     &    *s12**(-4)*s23**4 + 4.D0*s12**(-3)*s23**3 + 26.D0/3.D0*
     &    s12**(-2)*s23**2 + 52.D0/3.D0*s12**(-1)*s23 + 16.D0/3.D0*
     &    s13**(-2)*s23**2 + 52.D0/3.D0*s13**(-1)*s23 + 32.D0*s13*
     &    s23**(-1) + 80.D0/3.D0*s13**2*s23**(-2) + 8.D0*s13**3*
     &    s23**(-3) + 4.D0*s13**4*s23**(-4) )
      finiteA4g2X = finiteA4g2X + dlomx**2*dls * (  - 132.D0 - 44.D0*
     &    s12**(-2)*s23**2 - 44.D0*s12**(-1)*s23 - 88.D0*s13**(-2)*
     &    s23**2 - 88.D0*s13**(-1)*s23 - 44.D0*s13*s23**(-1) - 44.D0*
     &    s13**2*s23**(-2) )
      finiteA4g2X = finiteA4g2X + dlomx**3 * (  - 562.D0/9.D0 - 4.D0*
     &    s12**(-3)*s23**3 - 406.D0/9.D0*s12**(-2)*s23**2 - 620.D0/9.D0
     &    *s12**(-1)*s23 - 176.D0/9.D0*s13**(-2)*s23**2 - 176.D0/9.D0*
     &    s13**(-1)*s23 - 28.D0/3.D0*s13*s23**(-1) - 298.D0/9.D0*s13**2
     &    *s23**(-2) + 4.D0*s13**3*s23**(-3) )
      finiteA4g2X = finiteA4g2X + dlomx**4 * ( 22.D0 + s12**(-4)*s23**4
     &     + 2.D0*s12**(-3)*s23**3 + 9.D0*s12**(-2)*s23**2 + 22.D0/3.D0
     &    *s12**(-1)*s23 + 8.D0*s13**(-2)*s23**2 + 8.D0*s13**(-1)*s23
     &     + 26.D0/3.D0*s13*s23**(-1) + 9.D0*s13**2*s23**(-2) + 2.D0*
     &    s13**3*s23**(-3) + s13**4*s23**(-4) )
      finiteA4g2X = finiteA4g2X + dlx * ( 638.D0 + 4564.D0/27.D0*
     &    s12**(-2)*s23**2 + 6676.D0/27.D0*s12**(-1)*s23 + 4564.D0/27.D0
     &    *s13**(-2)*s23**2 + 6676.D0/27.D0*s13**(-1)*s23 + 5392.D0/27.D
     &    0*s13*s23**(-1) + 5392.D0/27.D0*s13**2*s23**(-2) )
      finiteA4g2X = finiteA4g2X + dlx*zeta3 * (  - 8.D0*s12**(-2)*
     &    s23**2 + 32.D0*s12**(-1)*s23 - 8.D0*s13**(-2)*s23**2 - 48.D0*
     &    s13**(-1)*s23 - 64.D0*s13*s23**(-1) - 16.D0*s13**2*s23**(-2)
     &     )
      finiteA4g2X = finiteA4g2X + dlx*pi**2 * (  - 5.D0/9.D0 + 4.D0*
     &    s12**(-3)*s23**3 - 5.D0*s12**(-2)*s23**2 + 74.D0/3.D0*
     &    s12**(-1)*s23 + 8.D0*s13**(-3)*s23**3 - 85.D0/3.D0*s13**(-2)*
     &    s23**2 + 38.D0/3.D0*s13**(-1)*s23 - 146.D0/9.D0*s13*s23**(-1)
     &     - 374.D0/9.D0*s13**2*s23**(-2) )
      finiteA4g2X = finiteA4g2X + dlx*dli3omx * (  - 232.D0 - 128.D0*
     &    s12**(-1)*s23 - 48.D0*s13**(-1)*s23 + 48.D0*s13*s23**(-1) )
      finiteA4g2X = finiteA4g2X + dlx*dli3x * ( 48.D0 - 80.D0*s12**(-1)
     &    *s23 + 80.D0*s13**(-1)*s23 + 96.D0*s13*s23**(-1) )
      finiteA4g2X = finiteA4g2X + dlx*dli2x * ( 152.D0/3.D0 + 88.D0/3.D0
     &    *s12**(-2)*s23**2 - 64.D0/3.D0*s12**(-1)*s23 - 88.D0/3.D0*
     &    s13**(-2)*s23**2 + 64.D0/3.D0*s13**(-1)*s23 + 304.D0/3.D0*s13
     &    *s23**(-1) )
      finiteA4g2X = finiteA4g2X + dlx*dls * ( 1144.D0/3.D0 + 484.D0/3.D0
     &    *s12**(-2)*s23**2 + 220.D0*s12**(-1)*s23 + 484.D0/3.D0*
     &    s13**(-2)*s23**2 + 220.D0*s13**(-1)*s23 )
      finiteA4g2X = finiteA4g2X + dlx*dlomx * ( 2482.D0/9.D0 + 28.D0*
     &    s12**(-2)*s23**2 + 28.D0*s12**(-1)*s23 + 536.D0/9.D0*
     &    s13**(-2)*s23**2 + 472.D0/9.D0*s13**(-1)*s23 + 472.D0/9.D0*
     &    s13*s23**(-1) + 536.D0/9.D0*s13**2*s23**(-2) )
      finiteA4g2X = finiteA4g2X + dlx*dlomx*pi**2 * ( 56.D0/3.D0 - 4.D0
     &    *s12**(-4)*s23**4 - 8.D0*s12**(-3)*s23**3 - 20.D0*s12**(-2)*
     &    s23**2 - 32.D0*s12**(-1)*s23 - 40.D0/3.D0*s13**(-2)*s23**2 - 
     &    16.D0/3.D0*s13**(-1)*s23 + 64.D0/3.D0*s13*s23**(-1) - 40.D0/3.
     &    D0*s13**2*s23**(-2) )
      finiteA4g2X = finiteA4g2X + dlx*dlomx*dls * ( 132.D0 + 88.D0*
     &    s13**(-2)*s23**2 + 88.D0*s13**(-1)*s23 + 88.D0*s13*s23**(-1)
     &     + 88.D0*s13**2*s23**(-2) )
      finiteA4g2X = finiteA4g2X + dlx*dlomx**2 * (  - 38.D0/3.D0 + 12.D0
     &    *s12**(-3)*s23**3 + 10.D0/3.D0*s12**(-2)*s23**2 + 12.D0*
     &    s12**(-1)*s23 - 44.D0/3.D0*s13**(-2)*s23**2 - 160.D0/3.D0*
     &    s13**(-1)*s23 + 76.D0/3.D0*s13*s23**(-1) + 88.D0*s13**2*
     &    s23**(-2) )
      finiteA4g2X = finiteA4g2X + dlx*dlomx**3 * (  - 68.D0/3.D0 - 4.D0
     &    *s12**(-4)*s23**4 - 8.D0*s12**(-3)*s23**3 - 20.D0*s12**(-2)*
     &    s23**2 - 8.D0/3.D0*s12**(-1)*s23 - 16.D0*s13**(-2)*s23**2 + 8.
     &    D0/3.D0*s13**(-1)*s23 - 16.D0/3.D0*s13*s23**(-1) - 16.D0*
     &    s13**2*s23**(-2) )
      finiteA4g2X = finiteA4g2X + dlx**2 * (  - 266.D0/3.D0 + 332.D0/9.D
     &    0*s12**(-2)*s23**2 + 628.D0/9.D0*s12**(-1)*s23 + 332.D0/9.D0*
     &    s13**(-2)*s23**2 + 628.D0/9.D0*s13**(-1)*s23 - 536.D0/9.D0*
     &    s13*s23**(-1) - 536.D0/9.D0*s13**2*s23**(-2) )
      finiteA4g2X = finiteA4g2X + dlx**2*pi**2 * ( 146.D0/3.D0 + 2.D0*
     &    s12**(-4)*s23**4 + 4.D0*s12**(-3)*s23**3 + 26.D0/3.D0*
     &    s12**(-2)*s23**2 + 52.D0/3.D0*s12**(-1)*s23 + 4.D0*s13**(-4)*
     &    s23**4 + 8.D0*s13**(-3)*s23**3 + 80.D0/3.D0*s13**(-2)*s23**2
     &     + 40.D0*s13**(-1)*s23 + 28.D0/3.D0*s13*s23**(-1) + 16.D0/3.D0
     &    *s13**2*s23**(-2) )
      finiteA4g2X = finiteA4g2X + dlx**2*dls * (  - 132.D0 - 44.D0*
     &    s12**(-2)*s23**2 - 44.D0*s12**(-1)*s23 - 44.D0*s13**(-2)*
     &    s23**2 - 44.D0*s13**(-1)*s23 - 88.D0*s13*s23**(-1) - 88.D0*
     &    s13**2*s23**(-2) )
      finiteA4g2X = finiteA4g2X + dlx**2*dlomx * ( 52.D0/3.D0 - 12.D0*
     &    s12**(-3)*s23**3 - 10.D0/3.D0*s12**(-2)*s23**2 - 188.D0/3.D0*
     &    s12**(-1)*s23 + 176.D0/3.D0*s13**(-2)*s23**2 + 140.D0/3.D0*
     &    s13**(-1)*s23 + 48.D0*s13*s23**(-1) - 44.D0/3.D0*s13**2*
     &    s23**(-2) )
      finiteA4g2X = finiteA4g2X + dlx**2*dlomx**2 * ( 10.D0 + 6.D0*
     &    s12**(-4)*s23**4 + 12.D0*s12**(-3)*s23**3 + 30.D0*s12**(-2)*
     &    s23**2 + 56.D0*s12**(-1)*s23 + 24.D0*s13**(-2)*s23**2 + 20.D0
     &    *s13**(-1)*s23 - 4.D0*s13*s23**(-1) + 24.D0*s13**2*s23**(-2)
     &     )
      finiteA4g2X = finiteA4g2X + dlx**3 * (  - 104.D0/3.D0 + 4.D0*
     &    s12**(-3)*s23**3 - 298.D0/9.D0*s12**(-2)*s23**2 - 28.D0/3.D0*
     &    s12**(-1)*s23 + 4.D0*s13**(-3)*s23**3 - 298.D0/9.D0*s13**(-2)
     &    *s23**2 - 28.D0/3.D0*s13**(-1)*s23 - 176.D0/9.D0*s13*
     &    s23**(-1) - 176.D0/9.D0*s13**2*s23**(-2) )
      finiteA4g2X = finiteA4g2X + dlx**3*dlomx * (  - 172.D0/3.D0 - 4.D0
     &    *s12**(-4)*s23**4 - 8.D0*s12**(-3)*s23**3 - 20.D0*s12**(-2)*
     &    s23**2 - 72.D0*s12**(-1)*s23 - 16.D0*s13**(-2)*s23**2 - 64.D0/
     &    3.D0*s13**(-1)*s23 + 56.D0/3.D0*s13*s23**(-1) - 16.D0*s13**2*
     &    s23**(-2) )
      finiteA4g2X = finiteA4g2X + dlx**4 * ( 28.D0 + s12**(-4)*s23**4
     &     + 2.D0*s12**(-3)*s23**3 + 9.D0*s12**(-2)*s23**2 + 58.D0/3.D0
     &    *s12**(-1)*s23 + s13**(-4)*s23**4 + 2.D0*s13**(-3)*s23**3 + 9.
     &    D0*s13**(-2)*s23**2 + 38.D0/3.D0*s13**(-1)*s23 + 4.D0*s13*
     &    s23**(-1) + 8.D0*s13**2*s23**(-2) )
      finiteA4g2X = finiteA4g2X - 27496.D0/27.D0 - 26344.D0/81.D0*
     &    s12**(-2)*s23**2 - 26344.D0/81.D0*s12**(-1)*s23 - 26344.D0/81.
     &    D0*s13**(-2)*s23**2 - 26344.D0/81.D0*s13**(-1)*s23 - 26344.D0/
     &    81.D0*s13*s23**(-1) - 26344.D0/81.D0*s13**2*s23**(-2)

      endif
      return
      end
************************************************************************
*
      function finiteAh4g2X(ig1,ig2,ig3,ig4,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)
      double precision dli2x,dli2omx,dli3x,dli3omx,dli4x,dli4omx,dli4xx
      parameter(pi=3.141592653589793238d0)
      parameter(zeta3=1.202056903159594285d0)
      common /order2l/ieorder
      s12 =s4(ig1,ig2)
      s23 =s4(ig2,ig3)
      s13 =s4(ig1,ig3)
      dlx   =log(-s23/s12)
      dlomx =log(-s13/s12)
      dls =log(s12/renscale2)
      dli2x   =rli2(-s23/s12)
      dli2omx =rli2(-s13/s12)
      dli3x   =rli3(-s23/s12)
      dli3omx =rli3(-s13/s12)
      dli4x   =rli4(-s23/s12)
      dli4omx =rli4(-s13/s12)
      dli4xx  =rli4(-s13/s23)
      if(ieorder.eq.-4)then
      finiteAh4g2X =  0

      elseif(ieorder.eq.-3)then
      finiteAh4g2X =  0

      elseif(ieorder.eq.-2)then
      finiteAh4g2X =  0

      elseif(ieorder.eq.-1)then
      finiteAh4g2X =  0

      elseif(ieorder.eq.0)then
      finiteAh4g2X =  + zeta3 * ( 620.D0/3.D0 - 16.D0/9.D0*s12**(-2)*
     &    s23**2 - 16.D0/9.D0*s12**(-1)*s23 + 32.D0/9.D0*s13**(-2)*
     &    s23**2 - 388.D0/9.D0*s13**(-1)*s23 - 388.D0/9.D0*s13*
     &    s23**(-1) + 32.D0/9.D0*s13**2*s23**(-2) )
      finiteAh4g2X = finiteAh4g2X + pi**2 * ( 94.D0/3.D0 + 298.D0/27.D0
     &    *s12**(-2)*s23**2 + 298.D0/27.D0*s12**(-1)*s23 - 20.D0/27.D0*
     &    s13**(-2)*s23**2 - 1097.D0/27.D0*s13**(-1)*s23 - 1097.D0/27.D0
     &    *s13*s23**(-1) - 20.D0/27.D0*s13**2*s23**(-2) )
      finiteAh4g2X = finiteAh4g2X + pi**4 * (  - 16.D0/3.D0 - 2.D0*
     &    s12**(-4)*s23**4 - 4.D0*s12**(-3)*s23**3 - 7.D0*s12**(-2)*
     &    s23**2 - 27.D0/5.D0*s12**(-1)*s23 - 739.D0/90.D0*s13**(-1)*
     &    s23 - 67.D0/18.D0*s13*s23**(-1) )
      finiteAh4g2X = finiteAh4g2X + dli4xx * (  - 112.D0 - 224.D0*
     &    s12**(-1)*s23 - 48.D0*s13**(-1)*s23 + 48.D0*s13*s23**(-1) )
      finiteAh4g2X = finiteAh4g2X + dli4omx * ( 112.D0 - 48.D0*
     &    s12**(-1)*s23 + 48.D0*s13**(-1)*s23 + 224.D0*s13*s23**(-1) )
      finiteAh4g2X = finiteAh4g2X + dli4x * ( 160.D0 + 48.D0*s12**(-1)*
     &    s23 + 224.D0*s13**(-1)*s23 + 48.D0*s13*s23**(-1) )
      finiteAh4g2X = finiteAh4g2X + dli3omx * ( 280.D0/3.D0 + 16.D0/3.D0
     &    *s12**(-2)*s23**2 + 52.D0*s12**(-1)*s23 + 280.D0/3.D0*
     &    s13**(-1)*s23 + 124.D0/3.D0*s13*s23**(-1) - 16.D0/3.D0*s13**2
     &    *s23**(-2) )
      finiteAh4g2X = finiteAh4g2X + dli3x * ( 140.D0/3.D0 + 16.D0/3.D0*
     &    s12**(-2)*s23**2 - 124.D0/3.D0*s12**(-1)*s23 - 16.D0/3.D0*
     &    s13**(-2)*s23**2 + 124.D0/3.D0*s13**(-1)*s23 + 280.D0/3.D0*
     &    s13*s23**(-1) )
      finiteAh4g2X = finiteAh4g2X + dli2x*pi**2 * (  - 32.D0/3.D0 - 64.D
     &    0/3.D0*s12**(-1)*s23 + 64.D0/3.D0*s13**(-1)*s23 - 64.D0/3.D0*
     &    s13*s23**(-1) )
      finiteAh4g2X = finiteAh4g2X + dls * (  - 4198.D0/9.D0 - 3496.D0/
     &    27.D0*s12**(-2)*s23**2 - 3496.D0/27.D0*s12**(-1)*s23 - 3496.D0
     &    /27.D0*s13**(-2)*s23**2 - 3496.D0/27.D0*s13**(-1)*s23 - 3496.D
     &    0/27.D0*s13*s23**(-1) - 3496.D0/27.D0*s13**2*s23**(-2) )
      finiteAh4g2X = finiteAh4g2X + dls*pi**2 * ( 44.D0/3.D0 + 8.D0/9.D0
     &    *s12**(-2)*s23**2 + 8.D0/9.D0*s12**(-1)*s23 + 80.D0/9.D0*
     &    s13**(-2)*s23**2 - 19.D0/9.D0*s13**(-1)*s23 - 19.D0/9.D0*s13*
     &    s23**(-1) + 80.D0/9.D0*s13**2*s23**(-2) )
      finiteAh4g2X = finiteAh4g2X + dls**2 * (  - 176.D0 - 176.D0/3.D0*
     &    s12**(-2)*s23**2 - 176.D0/3.D0*s12**(-1)*s23 - 176.D0/3.D0*
     &    s13**(-2)*s23**2 - 176.D0/3.D0*s13**(-1)*s23 - 176.D0/3.D0*
     &    s13*s23**(-1) - 176.D0/3.D0*s13**2*s23**(-2) )
      finiteAh4g2X = finiteAh4g2X + dlomx * (  - 4198.D0/27.D0 - 400.D0/
     &    9.D0*s12**(-2)*s23**2 - 1886.D0/27.D0*s12**(-1)*s23 - 1096.D0/
     &    27.D0*s13**(-2)*s23**2 - 1096.D0/27.D0*s13**(-1)*s23 - 514.D0/
     &    27.D0*s13*s23**(-1) - 400.D0/9.D0*s13**2*s23**(-2) )
      finiteAh4g2X = finiteAh4g2X + dlomx*zeta3 * ( 16.D0 + 24.D0*
     &    s12**(-1)*s23 + 24.D0*s13**(-1)*s23 + 64.D0*s13*s23**(-1) )
      finiteAh4g2X = finiteAh4g2X + dlomx*pi**2 * (  - 140.D0/3.D0 + 8.D
     &    0*s12**(-3)*s23**3 + 118.D0/9.D0*s12**(-2)*s23**2 + 32.D0*
     &    s12**(-1)*s23 + 68.D0/9.D0*s13**(-2)*s23**2 - 32.D0*s13**(-1)
     &    *s23 - 575.D0/9.D0*s13*s23**(-1) - 142.D0/9.D0*s13**2*
     &    s23**(-2) - 16.D0*s13**3*s23**(-3) )
      finiteAh4g2X = finiteAh4g2X + dlomx*dli3omx * (  - 112.D0 - 88.D0
     &    *s12**(-1)*s23 - 48.D0*s13**(-1)*s23 - 88.D0*s13*s23**(-1) )
      finiteAh4g2X = finiteAh4g2X + dlomx*dli3x * ( 64.D0 - 112.D0*
     &    s12**(-1)*s23 - 24.D0*s13**(-1)*s23 + 24.D0*s13*s23**(-1) )
      finiteAh4g2X = finiteAh4g2X + dlomx*dli2x * ( 280.D0/3.D0 + 16.D0/
     &    3.D0*s12**(-2)*s23**2 + 52.D0*s12**(-1)*s23 + 280.D0/3.D0*
     &    s13**(-1)*s23 + 124.D0/3.D0*s13*s23**(-1) - 16.D0/3.D0*s13**2
     &    *s23**(-2) )
      finiteAh4g2X = finiteAh4g2X + dlomx*dls * (  - 352.D0/3.D0 - 176.D
     &    0/3.D0*s12**(-2)*s23**2 - 166.D0/3.D0*s12**(-1)*s23 - 62.D0*
     &    s13*s23**(-1) - 176.D0/3.D0*s13**2*s23**(-2) )
      finiteAh4g2X = finiteAh4g2X + dlomx**2 * ( 50.D0 - 134.D0/9.D0*
     &    s12**(-2)*s23**2 + 23.D0/3.D0*s12**(-1)*s23 + 80.D0/9.D0*
     &    s13**(-2)*s23**2 + 80.D0/9.D0*s13**(-1)*s23 - 337.D0/9.D0*s13
     &    *s23**(-1) - 134.D0/9.D0*s13**2*s23**(-2) )
      finiteAh4g2X = finiteAh4g2X + dlomx**2*pi**2 * (  - 32.D0 - 4.D0*
     &    s12**(-4)*s23**4 - 8.D0*s12**(-3)*s23**3 - 14.D0*s12**(-2)*
     &    s23**2 - 56.D0/3.D0*s12**(-1)*s23 - 5.D0*s13**(-1)*s23 - 97.D0
     &    /3.D0*s13*s23**(-1) - 28.D0*s13**2*s23**(-2) - 16.D0*s13**3*
     &    s23**(-3) - 8.D0*s13**4*s23**(-4) )
      finiteAh4g2X = finiteAh4g2X + dlomx**2*dls * ( 46.D0 + 8.D0*
     &    s12**(-2)*s23**2 + 19.D0*s12**(-1)*s23 + 16.D0*s13**(-2)*
     &    s23**2 + 16.D0*s13**(-1)*s23 - 3.D0*s13*s23**(-1) + 8.D0*
     &    s13**2*s23**(-2) )
      finiteAh4g2X = finiteAh4g2X + dlomx**3 * ( 37.D0/9.D0 + 8.D0*
     &    s12**(-3)*s23**3 + 172.D0/9.D0*s12**(-2)*s23**2 + 383.D0/9.D0
     &    *s12**(-1)*s23 + 32.D0/9.D0*s13**(-2)*s23**2 + 32.D0/9.D0*
     &    s13**(-1)*s23 - 85.D0/3.D0*s13*s23**(-1) - 44.D0/9.D0*s13**2*
     &    s23**(-2) - 8.D0*s13**3*s23**(-3) )
      finiteAh4g2X = finiteAh4g2X + dlomx**4 * (  - 31.D0/3.D0 - 2.D0*
     &    s12**(-4)*s23**4 - 4.D0*s12**(-3)*s23**3 - 7.D0*s12**(-2)*
     &    s23**2 - 17.D0/6.D0*s12**(-1)*s23 - 43.D0/6.D0*s13*s23**(-1)
     &     - 7.D0*s13**2*s23**(-2) - 4.D0*s13**3*s23**(-3) - 2.D0*
     &    s13**4*s23**(-4) )
      finiteAh4g2X = finiteAh4g2X + dlx * (  - 3512.D0/27.D0 - 400.D0/9.
     &    D0*s12**(-2)*s23**2 - 514.D0/27.D0*s12**(-1)*s23 - 400.D0/9.D0
     &    *s13**(-2)*s23**2 - 514.D0/27.D0*s13**(-1)*s23 - 1096.D0/27.D0
     &    *s13*s23**(-1) - 1096.D0/27.D0*s13**2*s23**(-2) )
      finiteAh4g2X = finiteAh4g2X + dlx*zeta3 * (  - 8.D0 - 24.D0*
     &    s12**(-1)*s23 + 64.D0*s13**(-1)*s23 + 24.D0*s13*s23**(-1) )
      finiteAh4g2X = finiteAh4g2X + dlx*pi**2 * (  - 592.D0/9.D0 - 8.D0
     &    *s12**(-3)*s23**3 - 10.D0*s12**(-2)*s23**2 - 110.D0/3.D0*
     &    s12**(-1)*s23 - 16.D0*s13**(-3)*s23**3 - 50.D0/3.D0*s13**(-2)
     &    *s23**2 - 57.D0*s13**(-1)*s23 - 148.D0/9.D0*s13*s23**(-1) + 
     &    68.D0/9.D0*s13**2*s23**(-2) )
      finiteAh4g2X = finiteAh4g2X + dlx*dli3omx * ( 176.D0 + 112.D0*
     &    s12**(-1)*s23 + 24.D0*s13**(-1)*s23 - 24.D0*s13*s23**(-1) )
      finiteAh4g2X = finiteAh4g2X + dlx*dli3x * (  - 24.D0 + 88.D0*
     &    s12**(-1)*s23 - 88.D0*s13**(-1)*s23 - 48.D0*s13*s23**(-1) )
      finiteAh4g2X = finiteAh4g2X + dlx*dli2x * (  - 140.D0/3.D0 - 16.D0
     &    /3.D0*s12**(-2)*s23**2 + 124.D0/3.D0*s12**(-1)*s23 + 16.D0/3.D
     &    0*s13**(-2)*s23**2 - 124.D0/3.D0*s13**(-1)*s23 - 280.D0/3.D0*
     &    s13*s23**(-1) )
      finiteAh4g2X = finiteAh4g2X + dlx*dls * (  - 362.D0/3.D0 - 176.D0/
     &    3.D0*s12**(-2)*s23**2 - 62.D0*s12**(-1)*s23 - 176.D0/3.D0*
     &    s13**(-2)*s23**2 - 62.D0*s13**(-1)*s23 )
      finiteAh4g2X = finiteAh4g2X + dlx*dlomx * (  - 602.D0/9.D0 - 20.D0
     &    *s12**(-2)*s23**2 - 20.D0*s12**(-1)*s23 - 80.D0/9.D0*
     &    s13**(-2)*s23**2 + 296.D0/9.D0*s13**(-1)*s23 + 296.D0/9.D0*
     &    s13*s23**(-1) - 80.D0/9.D0*s13**2*s23**(-2) )
      finiteAh4g2X = finiteAh4g2X + dlx*dlomx*pi**2 * (  - 88.D0/3.D0
     &     + 8.D0*s12**(-4)*s23**4 + 16.D0*s12**(-3)*s23**3 + 28.D0*
     &    s12**(-2)*s23**2 + 28.D0*s12**(-1)*s23 - 14.D0/3.D0*s13**(-1)
     &    *s23 - 34.D0*s13*s23**(-1) )
      finiteAh4g2X = finiteAh4g2X + dlx*dlomx*dls * (  - 24.D0 - 16.D0*
     &    s13**(-2)*s23**2 + 6.D0*s13**(-1)*s23 + 6.D0*s13*s23**(-1) - 
     &    16.D0*s13**2*s23**(-2) )
      finiteAh4g2X = finiteAh4g2X + dlx*dlomx**2 * ( 158.D0/3.D0 - 24.D0
     &    *s12**(-3)*s23**3 - 100.D0/3.D0*s12**(-2)*s23**2 - 42.D0*
     &    s12**(-1)*s23 + 8.D0/3.D0*s13**(-2)*s23**2 + 169.D0/3.D0*
     &    s13**(-1)*s23 + 104.D0/3.D0*s13*s23**(-1) - 16.D0*s13**2*
     &    s23**(-2) )
      finiteAh4g2X = finiteAh4g2X + dlx*dlomx**3 * ( 16.D0/3.D0 + 8.D0*
     &    s12**(-4)*s23**4 + 16.D0*s12**(-3)*s23**3 + 28.D0*s12**(-2)*
     &    s23**2 + 16.D0/3.D0*s12**(-1)*s23 - 10.D0*s13**(-1)*s23 - 6.D0
     &    *s13*s23**(-1) )
      finiteAh4g2X = finiteAh4g2X + dlx**2 * ( 247.D0/9.D0 - 134.D0/9.D0
     &    *s12**(-2)*s23**2 - 337.D0/9.D0*s12**(-1)*s23 - 134.D0/9.D0*
     &    s13**(-2)*s23**2 - 337.D0/9.D0*s13**(-1)*s23 + 80.D0/9.D0*s13
     &    *s23**(-1) + 80.D0/9.D0*s13**2*s23**(-2) )
      finiteAh4g2X = finiteAh4g2X + dlx**2*pi**2 * (  - 98.D0/3.D0 - 4.D
     &    0*s12**(-4)*s23**4 - 8.D0*s12**(-3)*s23**3 - 14.D0*s12**(-2)*
     &    s23**2 - 20.D0*s12**(-1)*s23 - 8.D0*s13**(-4)*s23**4 - 16.D0*
     &    s13**(-3)*s23**3 - 28.D0*s13**(-2)*s23**2 - 109.D0/3.D0*
     &    s13**(-1)*s23 - s13*s23**(-1) )
      finiteAh4g2X = finiteAh4g2X + dlx**2*dls * ( 35.D0 + 8.D0*
     &    s12**(-2)*s23**2 - 3.D0*s12**(-1)*s23 + 8.D0*s13**(-2)*s23**2
     &     - 3.D0*s13**(-1)*s23 + 16.D0*s13*s23**(-1) + 16.D0*s13**2*
     &    s23**(-2) )
      finiteAh4g2X = finiteAh4g2X + dlx**2*dlomx * ( 116.D0/3.D0 + 24.D0
     &    *s12**(-3)*s23**3 + 100.D0/3.D0*s12**(-2)*s23**2 + 266.D0/3.D0
     &    *s12**(-1)*s23 - 32.D0/3.D0*s13**(-2)*s23**2 - 20.D0/3.D0*
     &    s13**(-1)*s23 - 37.D0*s13*s23**(-1) + 8.D0/3.D0*s13**2*
     &    s23**(-2) )
      finiteAh4g2X = finiteAh4g2X + dlx**2*dlomx**2 * ( 4.D0 - 12.D0*
     &    s12**(-4)*s23**4 - 24.D0*s12**(-3)*s23**3 - 42.D0*s12**(-2)*
     &    s23**2 - 58.D0*s12**(-1)*s23 + 3.D0*s13**(-1)*s23 + 15.D0*s13
     &    *s23**(-1) )
      finiteAh4g2X = finiteAh4g2X + dlx**3 * (  - 82.D0/3.D0 - 8.D0*
     &    s12**(-3)*s23**3 - 44.D0/9.D0*s12**(-2)*s23**2 - 85.D0/3.D0*
     &    s12**(-1)*s23 - 8.D0*s13**(-3)*s23**3 - 44.D0/9.D0*s13**(-2)*
     &    s23**2 - 85.D0/3.D0*s13**(-1)*s23 + 32.D0/9.D0*s13*s23**(-1)
     &     + 32.D0/9.D0*s13**2*s23**(-2) )
      finiteAh4g2X = finiteAh4g2X + dlx**3*dlomx * ( 116.D0/3.D0 + 8.D0
     &    *s12**(-4)*s23**4 + 16.D0*s12**(-3)*s23**3 + 28.D0*s12**(-2)*
     &    s23**2 + 72.D0*s12**(-1)*s23 + 2.D0*s13**(-1)*s23 - 18.D0*s13
     &    *s23**(-1) )
      finiteAh4g2X = finiteAh4g2X + dlx**4 * (  - 103.D0/6.D0 - 2.D0*
     &    s12**(-4)*s23**4 - 4.D0*s12**(-3)*s23**3 - 7.D0*s12**(-2)*
     &    s23**2 - 33.D0/2.D0*s12**(-1)*s23 - 2.D0*s13**(-4)*s23**4 - 4.
     &    D0*s13**(-3)*s23**3 - 7.D0*s13**(-2)*s23**2 - 55.D0/6.D0*
     &    s13**(-1)*s23 + 2.D0*s13*s23**(-1) )
      finiteAh4g2X = finiteAh4g2X + 593.D0/3.D0 + 466.D0/9.D0*s12**(-2)
     &    *s23**2 + 466.D0/9.D0*s12**(-1)*s23 + 466.D0/9.D0*s13**(-2)*
     &    s23**2 + 466.D0/9.D0*s13**(-1)*s23 + 466.D0/9.D0*s13*
     &    s23**(-1) + 466.D0/9.D0*s13**2*s23**(-2)

      endif
      return
      end
************************************************************************
*
      function finiteAhh4g2X(ig1,ig2,ig3,ig4,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)
      double precision dli2x,dli2omx,dli3x,dli3omx,dli4x,dli4omx,dli4xx
      parameter(pi=3.141592653589793238d0)
      parameter(zeta3=1.202056903159594285d0)
      common /order2l/ieorder
      s12 =s4(ig1,ig2)
      s23 =s4(ig2,ig3)
      s13 =s4(ig1,ig3)
      dlx   =log(-s23/s12)
      dlomx =log(-s13/s12)
      dls =log(s12/renscale2)
      dli2x   =rli2(-s23/s12)
      dli2omx =rli2(-s13/s12)
      dli3x   =rli3(-s23/s12)
      dli3omx =rli3(-s13/s12)
      dli4x   =rli4(-s23/s12)
      dli4omx =rli4(-s13/s12)
      dli4xx  =rli4(-s13/s23)
      if(ieorder.eq.-4)then
      finiteAhh4g2X =  0

      elseif(ieorder.eq.-3)then
      finiteAhh4g2X =  0

      elseif(ieorder.eq.-2)then
      finiteAhh4g2X =  0

      elseif(ieorder.eq.-1)then
      finiteAhh4g2X =  0

      elseif(ieorder.eq.0)then
      finiteAhh4g2X =  + pi**2 * ( 55.D0/9.D0 + 62.D0/27.D0*s12**(-2)*
     &    s23**2 + 62.D0/27.D0*s12**(-1)*s23 + 92.D0/27.D0*s13**(-2)*
     &    s23**2 + 146.D0/27.D0*s13**(-1)*s23 + 146.D0/27.D0*s13*
     &    s23**(-1) + 92.D0/27.D0*s13**2*s23**(-2) )
      finiteAhh4g2X = finiteAhh4g2X + pi**4 * ( 1.D0/4.D0 + s12**(-4)*
     &    s23**4 + 2.D0*s12**(-3)*s23**3 + 2.D0*s12**(-2)*s23**2 + 
     &    s12**(-1)*s23 )
      finiteAhh4g2X = finiteAhh4g2X + dls * ( 268.D0/9.D0 + 160.D0/27.D0
     &    *s12**(-2)*s23**2 + 160.D0/27.D0*s12**(-1)*s23 + 160.D0/27.D0
     &    *s13**(-2)*s23**2 + 160.D0/27.D0*s13**(-1)*s23 + 160.D0/27.D0
     &    *s13*s23**(-1) + 160.D0/27.D0*s13**2*s23**(-2) )
      finiteAhh4g2X = finiteAhh4g2X + dls*pi**2 * ( 2.D0*s13**(-1)*s23
     &     + 2.D0*s13*s23**(-1) )
      finiteAhh4g2X = finiteAhh4g2X + dls**2 * ( 16.D0 + 16.D0/3.D0*
     &    s12**(-2)*s23**2 + 16.D0/3.D0*s12**(-1)*s23 + 16.D0/3.D0*
     &    s13**(-2)*s23**2 + 16.D0/3.D0*s13**(-1)*s23 + 16.D0/3.D0*s13*
     &    s23**(-1) + 16.D0/3.D0*s13**2*s23**(-2) )
      finiteAhh4g2X = finiteAhh4g2X + dlomx * ( 268.D0/27.D0 + 80.D0/27.
     &    D0*s12**(-2)*s23**2 - 80.D0/27.D0*s12**(-1)*s23 + 80.D0/9.D0*
     &    s13*s23**(-1) + 80.D0/27.D0*s13**2*s23**(-2) )
      finiteAhh4g2X = finiteAhh4g2X + dlomx*pi**2 * ( 1.D0/3.D0 - 4.D0*
     &    s12**(-3)*s23**3 - 6.D0*s12**(-2)*s23**2 - 20.D0/3.D0*
     &    s12**(-1)*s23 + 2.D0/3.D0*s13**(-1)*s23 + 12.D0*s13*s23**(-1)
     &     + 12.D0*s13**2*s23**(-2) + 8.D0*s13**3*s23**(-3) )
      finiteAhh4g2X = finiteAhh4g2X + dlomx*dls * ( 32.D0/3.D0 + 16.D0/
     &    3.D0*s12**(-2)*s23**2 + 20.D0/3.D0*s12**(-1)*s23 + 4.D0*s13*
     &    s23**(-1) + 16.D0/3.D0*s13**2*s23**(-2) )
      finiteAhh4g2X = finiteAhh4g2X + dlomx**2 * ( 92.D0/9.D0 + 26.D0/3.
     &    D0*s12**(-2)*s23**2 + 28.D0/3.D0*s12**(-1)*s23 + 8.D0*s13*
     &    s23**(-1) + 26.D0/3.D0*s13**2*s23**(-2) )
      finiteAhh4g2X = finiteAhh4g2X + dlomx**2*pi**2 * ( 3.D0/2.D0 + 2.D
     &    0*s12**(-4)*s23**4 + 4.D0*s12**(-3)*s23**3 + 4.D0*s12**(-2)*
     &    s23**2 + 2.D0*s12**(-1)*s23 + 4.D0*s13*s23**(-1) + 8.D0*
     &    s13**2*s23**(-2) + 8.D0*s13**3*s23**(-3) + 4.D0*s13**4*
     &    s23**(-4) )
      finiteAhh4g2X = finiteAhh4g2X + dlomx**2*dls * (  - 4.D0 - 2.D0*
     &    s12**(-1)*s23 + 2.D0*s13*s23**(-1) )
      finiteAhh4g2X = finiteAhh4g2X + dlomx**3 * (  - 2.D0 - 4.D0*
     &    s12**(-3)*s23**3 - 6.D0*s12**(-2)*s23**2 - 20.D0/3.D0*
     &    s12**(-1)*s23 + 20.D0/3.D0*s13*s23**(-1) + 6.D0*s13**2*
     &    s23**(-2) + 4.D0*s13**3*s23**(-3) )
      finiteAhh4g2X = finiteAhh4g2X + dlomx**4 * ( 1.D0/2.D0 + 
     &    s12**(-4)*s23**4 + 2.D0*s12**(-3)*s23**3 + 2.D0*s12**(-2)*
     &    s23**2 + s12**(-1)*s23 + s13*s23**(-1) + 2.D0*s13**2*
     &    s23**(-2) + 2.D0*s13**3*s23**(-3) + s13**4*s23**(-4) )
      finiteAhh4g2X = finiteAhh4g2X + dlx * ( 428.D0/27.D0 + 80.D0/27.D0
     &    *s12**(-2)*s23**2 + 80.D0/9.D0*s12**(-1)*s23 + 80.D0/27.D0*
     &    s13**(-2)*s23**2 + 80.D0/9.D0*s13**(-1)*s23 )
      finiteAhh4g2X = finiteAhh4g2X + dlx*pi**2 * ( 5.D0 + 4.D0*
     &    s12**(-3)*s23**3 + 6.D0*s12**(-2)*s23**2 + 20.D0/3.D0*
     &    s12**(-1)*s23 + 8.D0*s13**(-3)*s23**3 + 12.D0*s13**(-2)*
     &    s23**2 + 12.D0*s13**(-1)*s23 + 2.D0/3.D0*s13*s23**(-1) )
      finiteAhh4g2X = finiteAhh4g2X + dlx*dls * ( 28.D0/3.D0 + 16.D0/3.D
     &    0*s12**(-2)*s23**2 + 4.D0*s12**(-1)*s23 + 16.D0/3.D0*
     &    s13**(-2)*s23**2 + 4.D0*s13**(-1)*s23 )
      finiteAhh4g2X = finiteAhh4g2X + dlx*dlomx * (  - 44.D0/9.D0 - 12.D
     &    0*s12**(-2)*s23**2 - 12.D0*s12**(-1)*s23 )
      finiteAhh4g2X = finiteAhh4g2X + dlx*dlomx*pi**2 * (  - 1.D0 - 4.D0
     &    *s12**(-4)*s23**4 - 8.D0*s12**(-3)*s23**3 - 8.D0*s12**(-2)*
     &    s23**2 - 4.D0*s12**(-1)*s23 )
      finiteAhh4g2X = finiteAhh4g2X + dlx*dlomx*dls * (  - 4.D0*
     &    s13**(-1)*s23 - 4.D0*s13*s23**(-1) )
      finiteAhh4g2X = finiteAhh4g2X + dlx*dlomx**2 * ( 5.D0 + 12.D0*
     &    s12**(-3)*s23**3 + 18.D0*s12**(-2)*s23**2 + 16.D0*s12**(-1)*
     &    s23 - 2.D0*s13*s23**(-1) )
      finiteAhh4g2X = finiteAhh4g2X + dlx*dlomx**3 * (  - 1.D0 - 4.D0*
     &    s12**(-4)*s23**4 - 8.D0*s12**(-3)*s23**3 - 8.D0*s12**(-2)*
     &    s23**2 - 4.D0*s12**(-1)*s23 )
      finiteAhh4g2X = finiteAhh4g2X + dlx**2 * ( 86.D0/9.D0 + 26.D0/3.D0
     &    *s12**(-2)*s23**2 + 8.D0*s12**(-1)*s23 + 26.D0/3.D0*s13**(-2)
     &    *s23**2 + 8.D0*s13**(-1)*s23 )
      finiteAhh4g2X = finiteAhh4g2X + dlx**2*pi**2 * ( 3.D0/2.D0 + 2.D0
     &    *s12**(-4)*s23**4 + 4.D0*s12**(-3)*s23**3 + 4.D0*s12**(-2)*
     &    s23**2 + 2.D0*s12**(-1)*s23 + 4.D0*s13**(-4)*s23**4 + 8.D0*
     &    s13**(-3)*s23**3 + 8.D0*s13**(-2)*s23**2 + 4.D0*s13**(-1)*s23
     &     )
      finiteAhh4g2X = finiteAhh4g2X + dlx**2*dls * (  - 2.D0 + 2.D0*
     &    s12**(-1)*s23 + 2.D0*s13**(-1)*s23 )
      finiteAhh4g2X = finiteAhh4g2X + dlx**2*dlomx * (  - 5.D0 - 12.D0*
     &    s12**(-3)*s23**3 - 18.D0*s12**(-2)*s23**2 - 16.D0*s12**(-1)*
     &    s23 - 2.D0*s13**(-1)*s23 )
      finiteAhh4g2X = finiteAhh4g2X + dlx**2*dlomx**2 * ( 3.D0/2.D0 + 6.
     &    D0*s12**(-4)*s23**4 + 12.D0*s12**(-3)*s23**3 + 12.D0*
     &    s12**(-2)*s23**2 + 6.D0*s12**(-1)*s23 )
      finiteAhh4g2X = finiteAhh4g2X + dlx**3 * ( 8.D0/3.D0 + 4.D0*
     &    s12**(-3)*s23**3 + 6.D0*s12**(-2)*s23**2 + 20.D0/3.D0*
     &    s12**(-1)*s23 + 4.D0*s13**(-3)*s23**3 + 6.D0*s13**(-2)*s23**2
     &     + 20.D0/3.D0*s13**(-1)*s23 )
      finiteAhh4g2X = finiteAhh4g2X + dlx**3*dlomx * (  - 1.D0 - 4.D0*
     &    s12**(-4)*s23**4 - 8.D0*s12**(-3)*s23**3 - 8.D0*s12**(-2)*
     &    s23**2 - 4.D0*s12**(-1)*s23 )
      finiteAhh4g2X = finiteAhh4g2X + dlx**4 * ( 1.D0/2.D0 + s12**(-4)*
     &    s23**4 + 2.D0*s12**(-3)*s23**3 + 2.D0*s12**(-2)*s23**2 + 
     &    s12**(-1)*s23 + s13**(-4)*s23**4 + 2.D0*s13**(-3)*s23**3 + 2.D
     &    0*s13**(-2)*s23**2 + s13**(-1)*s23 )
      finiteAhh4g2X = finiteAhh4g2X + 805.D0/27.D0 + 472.D0/81.D0*
     &    s12**(-2)*s23**2 + 472.D0/81.D0*s12**(-1)*s23 + 472.D0/81.D0*
     &    s13**(-2)*s23**2 + 472.D0/81.D0*s13**(-1)*s23 + 472.D0/81.D0*
     &    s13*s23**(-1) + 472.D0/81.D0*s13**2*s23**(-2)

      endif
      return
      end
************************************************************************
*
      function finiteAt4g2X(ig1,ig2,ig3,ig4,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)
      double precision dli2x,dli2omx,dli3x,dli3omx,dli4x,dli4omx,dli4xx
      parameter(pi=3.141592653589793238d0)
      parameter(zeta3=1.202056903159594285d0)
      common /order2l/ieorder
      s12 =s4(ig1,ig2)
      s23 =s4(ig2,ig3)
      s13 =s4(ig1,ig3)
      dlx   =log(-s23/s12)
      dlomx =log(-s13/s12)
      dls =log(s12/renscale2)
      dli2x   =rli2(-s23/s12)
      dli2omx =rli2(-s13/s12)
      dli3x   =rli3(-s23/s12)
      dli3omx =rli3(-s13/s12)
      dli4x   =rli4(-s23/s12)
      dli4omx =rli4(-s13/s12)
      dli4xx  =rli4(-s13/s23)
      if(ieorder.eq.-4)then
      finiteAt4g2X =  0

      elseif(ieorder.eq.-3)then
      finiteAt4g2X =  0

      elseif(ieorder.eq.-2)then
      finiteAt4g2X =  0

      elseif(ieorder.eq.-1)then
      finiteAt4g2X =  0

      elseif(ieorder.eq.0)then
      finiteAt4g2X =  + zeta3 * (  - 5600.D0 - 448.D0*s13**(-1)*s23 - 
     &    448.D0*s13*s23**(-1) )
      finiteAt4g2X = finiteAt4g2X + pi**2 * (  - 88.D0 - 48.D0*
     &    s12**(-2)*s23**2 - 48.D0*s12**(-1)*s23 + 48.D0*s13**(-2)*
     &    s23**2 + 96.D0*s13**(-1)*s23 + 96.D0*s13*s23**(-1) + 48.D0*
     &    s13**2*s23**(-2) )
      finiteAt4g2X = finiteAt4g2X + pi**4 * ( 1496.D0/15.D0 + 12.D0*
     &    s12**(-4)*s23**4 + 24.D0*s12**(-3)*s23**3 + 644.D0/15.D0*
     &    s12**(-2)*s23**2 + 36.D0/5.D0*s12**(-1)*s23 + 80.D0*s13**(-2)
     &    *s23**2 + 1216.D0/15.D0*s13**(-1)*s23 + 64.D0/5.D0*s13*
     &    s23**(-1) + 832.D0/15.D0*s13**2*s23**(-2) )
      finiteAt4g2X = finiteAt4g2X + dli4xx * ( 288.D0 + 576.D0*
     &    s12**(-1)*s23 - 384.D0*s13**(-2)*s23**2 - 1152.D0*s13**(-1)*
     &    s23 + 1152.D0*s13*s23**(-1) + 384.D0*s13**2*s23**(-2) )
      finiteAt4g2X = finiteAt4g2X + dli4omx * (  - 288.D0 + 384.D0*
     &    s12**(-2)*s23**2 - 384.D0*s12**(-1)*s23 - 384.D0*s13**(-2)*
     &    s23**2 + 384.D0*s13**(-1)*s23 - 576.D0*s13*s23**(-1) )
      finiteAt4g2X = finiteAt4g2X + dli4x * ( 480.D0 + 384.D0*s12**(-2)
     &    *s23**2 + 1152.D0*s12**(-1)*s23 - 576.D0*s13**(-1)*s23 + 384.D
     &    0*s13*s23**(-1) - 384.D0*s13**2*s23**(-2) )
      finiteAt4g2X = finiteAt4g2X + dli3omx * ( 896.D0 + 448.D0*
     &    s12**(-1)*s23 + 896.D0*s13**(-1)*s23 + 448.D0*s13*s23**(-1) )
      finiteAt4g2X = finiteAt4g2X + dli3x * ( 448.D0 - 448.D0*s12**(-1)
     &    *s23 + 448.D0*s13**(-1)*s23 + 896.D0*s13*s23**(-1) )
      finiteAt4g2X = finiteAt4g2X + dli2x*pi**2 * ( 176.D0 + 352.D0*
     &    s12**(-1)*s23 - 192.D0*s13**(-2)*s23**2 - 544.D0*s13**(-1)*
     &    s23 + 544.D0*s13*s23**(-1) + 192.D0*s13**2*s23**(-2) )
      finiteAt4g2X = finiteAt4g2X + dlomx * (  - 384.D0*s12**(-1)*s23
     &     + 384.D0*s13*s23**(-1) )
      finiteAt4g2X = finiteAt4g2X + dlomx*zeta3 * (  - 192.D0 + 192.D0*
     &    s12**(-1)*s23 + 384.D0*s13**(-2)*s23**2 + 768.D0*s13**(-1)*
     &    s23 - 1248.D0*s13*s23**(-1) - 384.D0*s13**2*s23**(-2) )
      finiteAt4g2X = finiteAt4g2X + dlomx*pi**2 * (  - 1432.D0/3.D0 - 
     &    48.D0*s12**(-3)*s23**3 - 72.D0*s12**(-2)*s23**2 - 2816.D0/3.D0
     &    *s12**(-1)*s23 - 1648.D0/3.D0*s13**(-1)*s23 + 1288.D0/3.D0*
     &    s13*s23**(-1) - 208.D0*s13**2*s23**(-2) + 96.D0*s13**3*
     &    s23**(-3) )
      finiteAt4g2X = finiteAt4g2X + dlomx*dli3omx * ( 288.D0 - 384.D0*
     &    s12**(-2)*s23**2 + 288.D0*s12**(-1)*s23 - 768.D0*s13**(-1)*
     &    s23 + 1056.D0*s13*s23**(-1) + 384.D0*s13**2*s23**(-2) )
      finiteAt4g2X = finiteAt4g2X + dlomx*dli3x * ( 288.D0 + 288.D0*
     &    s12**(-1)*s23 - 384.D0*s13**(-2)*s23**2 - 768.D0*s13**(-1)*
     &    s23 + 768.D0*s13*s23**(-1) + 384.D0*s13**2*s23**(-2) )
      finiteAt4g2X = finiteAt4g2X + dlomx*dli2x * ( 896.D0 + 448.D0*
     &    s12**(-1)*s23 + 896.D0*s13**(-1)*s23 + 448.D0*s13*s23**(-1) )
      finiteAt4g2X = finiteAt4g2X + dlomx**2 * ( 5192.D0/3.D0 + 1936.D0/
     &    3.D0*s12**(-2)*s23**2 + 4256.D0/3.D0*s12**(-1)*s23 - 128.D0*
     &    s13*s23**(-1) + 1936.D0/3.D0*s13**2*s23**(-2) )
      finiteAt4g2X = finiteAt4g2X + dlomx**2*pi**2 * ( 248.D0 + 24.D0*
     &    s12**(-4)*s23**4 + 48.D0*s12**(-3)*s23**3 + 152.D0*s12**(-2)*
     &    s23**2 + 312.D0*s12**(-1)*s23 - 96.D0*s13**(-1)*s23 + 144.D0*
     &    s13*s23**(-1) + 208.D0*s13**2*s23**(-2) + 96.D0*s13**3*
     &    s23**(-3) + 48.D0*s13**4*s23**(-4) )
      finiteAt4g2X = finiteAt4g2X + dlomx**2*dli2x * (  - 192.D0*
     &    s12**(-2)*s23**2 - 192.D0*s12**(-1)*s23 + 192.D0*s13*
     &    s23**(-1) + 192.D0*s13**2*s23**(-2) )
      finiteAt4g2X = finiteAt4g2X + dlomx**3 * (  - 1568.D0/3.D0 - 48.D0
     &    *s12**(-3)*s23**3 - 424.D0*s12**(-2)*s23**2 - 3656.D0/3.D0*
     &    s12**(-1)*s23 + 1544.D0/3.D0*s13*s23**(-1) - 280.D0*s13**2*
     &    s23**(-2) + 48.D0*s13**3*s23**(-3) )
      finiteAt4g2X = finiteAt4g2X + dlomx**4 * ( 132.D0 + 12.D0*
     &    s12**(-4)*s23**4 + 24.D0*s12**(-3)*s23**3 + 92.D0*s12**(-2)*
     &    s23**2 + 164.D0*s12**(-1)*s23 - 4.D0*s13*s23**(-1) + 92.D0*
     &    s13**2*s23**(-2) + 24.D0*s13**3*s23**(-3) + 12.D0*s13**4*
     &    s23**(-4) )
      finiteAt4g2X = finiteAt4g2X + dlx * ( 384.D0 + 384.D0*s12**(-1)*
     &    s23 + 384.D0*s13**(-1)*s23 )
      finiteAt4g2X = finiteAt4g2X + dlx*zeta3 * (  - 384.D0 - 192.D0*
     &    s12**(-1)*s23 - 384.D0*s13**(-2)*s23**2 - 1248.D0*s13**(-1)*
     &    s23 + 768.D0*s13*s23**(-1) + 384.D0*s13**2*s23**(-2) )
      finiteAt4g2X = finiteAt4g2X + dlx*pi**2 * ( 512.D0 + 48.D0*
     &    s12**(-3)*s23**3 + 72.D0*s12**(-2)*s23**2 + 864.D0*s12**(-1)*
     &    s23 + 96.D0*s13**(-3)*s23**3 - 208.D0*s13**(-2)*s23**2 + 504.D
     &    0*s13**(-1)*s23 - 400.D0*s13*s23**(-1) )
      finiteAt4g2X = finiteAt4g2X + dlx*dli3omx * (  - 288.D0*s12**(-1)
     &    *s23 + 384.D0*s13**(-2)*s23**2 + 768.D0*s13**(-1)*s23 - 768.D0
     &    *s13*s23**(-1) - 384.D0*s13**2*s23**(-2) )
      finiteAt4g2X = finiteAt4g2X + dlx*dli3x * (  - 384.D0 - 384.D0*
     &    s12**(-2)*s23**2 - 1056.D0*s12**(-1)*s23 + 384.D0*s13**(-2)*
     &    s23**2 + 1056.D0*s13**(-1)*s23 - 768.D0*s13*s23**(-1) )
      finiteAt4g2X = finiteAt4g2X + dlx*dli2x * (  - 448.D0 + 448.D0*
     &    s12**(-1)*s23 - 448.D0*s13**(-1)*s23 - 896.D0*s13*s23**(-1) )
      finiteAt4g2X = finiteAt4g2X + dlx*dlomx * (  - 184.D0 - 3872.D0/3.
     &    D0*s12**(-2)*s23**2 - 3872.D0/3.D0*s12**(-1)*s23 + 4640.D0/3.D
     &    0*s13**(-1)*s23 + 4640.D0/3.D0*s13*s23**(-1) )
      finiteAt4g2X = finiteAt4g2X + dlx*dlomx*pi**2 * ( 256.D0 - 48.D0*
     &    s12**(-4)*s23**4 - 96.D0*s12**(-3)*s23**3 - 240.D0*s12**(-2)*
     &    s23**2 - 64.D0*s12**(-1)*s23 - 192.D0*s13**(-2)*s23**2 + 160.D
     &    0*s13**(-1)*s23 + 576.D0*s13*s23**(-1) )
      finiteAt4g2X = finiteAt4g2X + dlx*dlomx*dli2x * ( 384.D0*
     &    s13**(-2)*s23**2 + 384.D0*s13**(-1)*s23 - 384.D0*s13*
     &    s23**(-1) - 384.D0*s13**2*s23**(-2) )
      finiteAt4g2X = finiteAt4g2X + dlx*dlomx**2 * ( 2120.D0 + 144.D0*
     &    s12**(-3)*s23**3 + 568.D0*s12**(-2)*s23**2 + 2200.D0*
     &    s12**(-1)*s23 + 1648.D0*s13**(-1)*s23 - 48.D0*s13*s23**(-1)
     &     + 704.D0*s13**2*s23**(-2) )
      finiteAt4g2X = finiteAt4g2X + dlx*dlomx**3 * (  - 112.D0 - 48.D0*
     &    s12**(-4)*s23**4 - 96.D0*s12**(-3)*s23**3 - 368.D0*s12**(-2)*
     &    s23**2 - 368.D0*s12**(-1)*s23 + 64.D0*s13**(-1)*s23 + 288.D0*
     &    s13*s23**(-1) )
      finiteAt4g2X = finiteAt4g2X + dlx**2 * ( 2872.D0/3.D0 + 1936.D0/3.
     &    D0*s12**(-2)*s23**2 - 128.D0*s12**(-1)*s23 + 1936.D0/3.D0*
     &    s13**(-2)*s23**2 - 128.D0*s13**(-1)*s23 )
      finiteAt4g2X = finiteAt4g2X + dlx**2*pi**2 * ( 88.D0 + 24.D0*
     &    s12**(-4)*s23**4 + 48.D0*s12**(-3)*s23**3 + 120.D0*s12**(-2)*
     &    s23**2 - 40.D0*s12**(-1)*s23 + 48.D0*s13**(-4)*s23**4 + 96.D0
     &    *s13**(-3)*s23**3 + 208.D0*s13**(-2)*s23**2 + 80.D0*s13**(-1)
     &    *s23 + 32.D0*s13**2*s23**(-2) )
      finiteAt4g2X = finiteAt4g2X + dlx**2*dli2x * ( 192.D0*s12**(-2)*
     &    s23**2 + 192.D0*s12**(-1)*s23 - 192.D0*s13**(-2)*s23**2 - 192.
     &    D0*s13**(-1)*s23 )
      finiteAt4g2X = finiteAt4g2X + dlx**2*dlomx * (  - 104.D0 - 144.D0
     &    *s12**(-3)*s23**3 + 136.D0*s12**(-2)*s23**2 - 1048.D0*
     &    s12**(-1)*s23 + 704.D0*s13**(-2)*s23**2 - 496.D0*s13**(-1)*
     &    s23 + 752.D0*s13*s23**(-1) )
      finiteAt4g2X = finiteAt4g2X + dlx**2*dlomx**2 * ( 288.D0 + 72.D0*
     &    s12**(-4)*s23**4 + 144.D0*s12**(-3)*s23**3 + 264.D0*s12**(-2)
     &    *s23**2 + 264.D0*s12**(-1)*s23 + 288.D0*s13**(-2)*s23**2 - 96.
     &    D0*s13*s23**(-1) )
      finiteAt4g2X = finiteAt4g2X + dlx**3 * ( 320.D0 + 48.D0*s12**(-3)
     &    *s23**3 - 280.D0*s12**(-2)*s23**2 + 1544.D0/3.D0*s12**(-1)*
     &    s23 + 48.D0*s13**(-3)*s23**3 - 280.D0*s13**(-2)*s23**2 + 1544.
     &    D0/3.D0*s13**(-1)*s23 )
      finiteAt4g2X = finiteAt4g2X + dlx**3*dlomx * (  - 112.D0 - 48.D0*
     &    s12**(-4)*s23**4 - 96.D0*s12**(-3)*s23**3 - 176.D0*s12**(-2)*
     &    s23**2 - 176.D0*s12**(-1)*s23 - 128.D0*s13**(-2)*s23**2 + 288.
     &    D0*s13**(-1)*s23 - 128.D0*s13*s23**(-1) - 64.D0*s13**2*
     &    s23**(-2) )
      finiteAt4g2X = finiteAt4g2X + dlx**4 * ( 60.D0 + 12.D0*s12**(-4)*
     &    s23**4 + 24.D0*s12**(-3)*s23**3 + 92.D0*s12**(-2)*s23**2 + 20.
     &    D0*s12**(-1)*s23 + 12.D0*s13**(-4)*s23**4 + 24.D0*s13**(-3)*
     &    s23**3 + 76.D0*s13**(-2)*s23**2 - 52.D0*s13**(-1)*s23 + 48.D0
     &    *s13*s23**(-1) + 16.D0*s13**2*s23**(-2) )
      finiteAt4g2X = finiteAt4g2X + 24.D0

      endif
      return
      end
************************************************************************
*
      function finiteAth4g2X(ig1,ig2,ig3,ig4,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)
      double precision dli2x,dli2omx,dli3x,dli3omx,dli4x,dli4omx,dli4xx
      parameter(pi=3.141592653589793238d0)
      parameter(zeta3=1.202056903159594285d0)
      common /order2l/ieorder
      s12 =s4(ig1,ig2)
      s23 =s4(ig2,ig3)
      s13 =s4(ig1,ig3)
      dlx   =log(-s23/s12)
      dlomx =log(-s13/s12)
      dls =log(s12/renscale2)
      dli2x   =rli2(-s23/s12)
      dli2omx =rli2(-s13/s12)
      dli3x   =rli3(-s23/s12)
      dli3omx =rli3(-s13/s12)
      dli4x   =rli4(-s23/s12)
      dli4omx =rli4(-s13/s12)
      dli4xx  =rli4(-s13/s23)
      if(ieorder.eq.-4)then
      finiteAth4g2X =  0

      elseif(ieorder.eq.-3)then
      finiteAth4g2X =  0

      elseif(ieorder.eq.-2)then
      finiteAth4g2X =  0

      elseif(ieorder.eq.-1)then
      finiteAth4g2X =  0

      elseif(ieorder.eq.0)then
      finiteAth4g2X =  + zeta3 * ( 2356.D0/3.D0 + 32.D0*s12**(-2)*
     &    s23**2 + 32.D0*s12**(-1)*s23 + 32.D0*s13**(-2)*s23**2 + 188.D0
     &    /3.D0*s13**(-1)*s23 + 188.D0/3.D0*s13*s23**(-1) + 32.D0*
     &    s13**2*s23**(-2) )
      finiteAth4g2X = finiteAth4g2X + pi**2 * ( 47.D0/3.D0 + 6.D0*
     &    s12**(-2)*s23**2 + 6.D0*s12**(-1)*s23 - 16.D0*s13**(-2)*
     &    s23**2 - 26.D0*s13**(-1)*s23 - 26.D0*s13*s23**(-1) - 16.D0*
     &    s13**2*s23**(-2) )
      finiteAth4g2X = finiteAth4g2X + pi**4 * (  - 31.D0/3.D0 - 4.D0*
     &    s12**(-4)*s23**4 - 8.D0*s12**(-3)*s23**3 - 14.D0*s12**(-2)*
     &    s23**2 - 139.D0/15.D0*s12**(-1)*s23 - 391.D0/90.D0*s13**(-1)*
     &    s23 + 65.D0/18.D0*s13*s23**(-1) )
      finiteAth4g2X = finiteAth4g2X + dli4xx * (  - 88.D0 - 176.D0*
     &    s12**(-1)*s23 + 48.D0*s13**(-1)*s23 - 48.D0*s13*s23**(-1) )
      finiteAth4g2X = finiteAth4g2X + dli4omx * ( 88.D0 + 48.D0*
     &    s12**(-1)*s23 - 48.D0*s13**(-1)*s23 + 176.D0*s13*s23**(-1) )
      finiteAth4g2X = finiteAth4g2X + dli4x * ( 40.D0 - 48.D0*s12**(-1)
     &    *s23 + 176.D0*s13**(-1)*s23 - 48.D0*s13*s23**(-1) )
      finiteAth4g2X = finiteAth4g2X + dli3omx * (  - 184.D0/3.D0 - 92.D0
     &    /3.D0*s12**(-1)*s23 - 184.D0/3.D0*s13**(-1)*s23 - 92.D0/3.D0*
     &    s13*s23**(-1) )
      finiteAth4g2X = finiteAth4g2X + dli3x * (  - 92.D0/3.D0 + 92.D0/3.
     &    D0*s12**(-1)*s23 - 92.D0/3.D0*s13**(-1)*s23 - 184.D0/3.D0*s13
     &    *s23**(-1) )
      finiteAth4g2X = finiteAth4g2X + dli2x*pi**2 * (  - 44.D0/3.D0 - 
     &    88.D0/3.D0*s12**(-1)*s23 + 160.D0/3.D0*s13**(-1)*s23 - 160.D0/
     &    3.D0*s13*s23**(-1) )
      finiteAth4g2X = finiteAth4g2X + dls * ( 24.D0 + 8.D0*s12**(-2)*
     &    s23**2 + 8.D0*s12**(-1)*s23 + 8.D0*s13**(-2)*s23**2 + 8.D0*
     &    s13**(-1)*s23 + 8.D0*s13*s23**(-1) + 8.D0*s13**2*s23**(-2) )
      finiteAth4g2X = finiteAth4g2X + dlomx * ( 8.D0 + 128.D0*s12**(-1)
     &    *s23 + 8.D0*s13**(-2)*s23**2 + 8.D0*s13**(-1)*s23 - 128.D0*
     &    s13*s23**(-1) )
      finiteAth4g2X = finiteAth4g2X + dlomx*zeta3 * ( 72.D0 - 64.D0*
     &    s12**(-1)*s23 - 32.D0*s13**(-1)*s23 + 168.D0*s13*s23**(-1) )
      finiteAth4g2X = finiteAth4g2X + dlomx*pi**2 * ( 302.D0/9.D0 + 16.D
     &    0*s12**(-3)*s23**3 + 24.D0*s12**(-2)*s23**2 + 1402.D0/9.D0*
     &    s12**(-1)*s23 + 731.D0/9.D0*s13**(-1)*s23 - 1043.D0/9.D0*s13*
     &    s23**(-1) + 16.D0*s13**2*s23**(-2) - 32.D0*s13**3*s23**(-3) )
      finiteAth4g2X = finiteAth4g2X + dlomx*dli3omx * (  - 72.D0 - 104.D
     &    0*s12**(-1)*s23 + 64.D0*s13**(-1)*s23 - 104.D0*s13*s23**(-1)
     &     )
      finiteAth4g2X = finiteAth4g2X + dlomx*dli3x * (  - 144.D0 - 72.D0
     &    *s12**(-1)*s23 + 32.D0*s13**(-1)*s23 - 32.D0*s13*s23**(-1) )
      finiteAth4g2X = finiteAth4g2X + dlomx*dli2x * (  - 184.D0/3.D0 - 
     &    92.D0/3.D0*s12**(-1)*s23 - 184.D0/3.D0*s13**(-1)*s23 - 92.D0/
     &    3.D0*s13*s23**(-1) )
      finiteAth4g2X = finiteAth4g2X + dlomx**2 * (  - 1988.D0/3.D0 - 
     &    734.D0/3.D0*s12**(-2)*s23**2 - 1592.D0/3.D0*s12**(-1)*s23 + 
     &    124.D0/3.D0*s13*s23**(-1) - 734.D0/3.D0*s13**2*s23**(-2) )
      finiteAth4g2X = finiteAth4g2X + dlomx**2*pi**2 * (  - 44.D0 - 8.D0
     &    *s12**(-4)*s23**4 - 16.D0*s12**(-3)*s23**3 - 28.D0*s12**(-2)*
     &    s23**2 - 118.D0/3.D0*s12**(-1)*s23 + 1.D0/3.D0*s13**(-1)*s23
     &     - 163.D0/3.D0*s13*s23**(-1) - 56.D0*s13**2*s23**(-2) - 32.D0
     &    *s13**3*s23**(-3) - 16.D0*s13**4*s23**(-4) )
      finiteAth4g2X = finiteAth4g2X + dlomx**2*dli2x * ( 16.D0 + 8.D0*
     &    s12**(-1)*s23 + 16.D0*s13**(-1)*s23 + 8.D0*s13*s23**(-1) )
      finiteAth4g2X = finiteAth4g2X + dlomx**3 * ( 814.D0/9.D0 + 16.D0*
     &    s12**(-3)*s23**3 + 88.D0*s12**(-2)*s23**2 + 2455.D0/9.D0*
     &    s12**(-1)*s23 - 1303.D0/9.D0*s13*s23**(-1) + 40.D0*s13**2*
     &    s23**(-2) - 16.D0*s13**3*s23**(-3) )
      finiteAth4g2X = finiteAth4g2X + dlomx**4 * (  - 20.D0/3.D0 - 4.D0
     &    *s12**(-4)*s23**4 - 8.D0*s12**(-3)*s23**3 - 14.D0*s12**(-2)*
     &    s23**2 - 61.D0/6.D0*s12**(-1)*s23 - 59.D0/6.D0*s13*s23**(-1)
     &     - 14.D0*s13**2*s23**(-2) - 8.D0*s13**3*s23**(-3) - 4.D0*
     &    s13**4*s23**(-4) )
      finiteAth4g2X = finiteAth4g2X + dlx * (  - 120.D0 - 128.D0*
     &    s12**(-1)*s23 - 128.D0*s13**(-1)*s23 + 8.D0*s13*s23**(-1) + 8.
     &    D0*s13**2*s23**(-2) )
      finiteAth4g2X = finiteAth4g2X + dlx*zeta3 * ( 136.D0 + 64.D0*
     &    s12**(-1)*s23 + 168.D0*s13**(-1)*s23 - 32.D0*s13*s23**(-1) )
      finiteAth4g2X = finiteAth4g2X + dlx*pi**2 * (  - 358.D0/3.D0 - 16.
     &    D0*s12**(-3)*s23**3 - 24.D0*s12**(-2)*s23**2 - 452.D0/3.D0*
     &    s12**(-1)*s23 - 32.D0*s13**(-3)*s23**3 + 16.D0*s13**(-2)*
     &    s23**2 - 121.D0*s13**(-1)*s23 + 71.D0*s13*s23**(-1) )
      finiteAth4g2X = finiteAth4g2X + dlx*dli3omx * (  - 72.D0 + 72.D0*
     &    s12**(-1)*s23 - 32.D0*s13**(-1)*s23 + 32.D0*s13*s23**(-1) )
      finiteAth4g2X = finiteAth4g2X + dlx*dli3x * ( 32.D0 + 104.D0*
     &    s12**(-1)*s23 - 104.D0*s13**(-1)*s23 + 64.D0*s13*s23**(-1) )
      finiteAth4g2X = finiteAth4g2X + dlx*dli2x * ( 92.D0/3.D0 - 92.D0/
     &    3.D0*s12**(-1)*s23 + 92.D0/3.D0*s13**(-1)*s23 + 184.D0/3.D0*
     &    s13*s23**(-1) )
      finiteAth4g2X = finiteAth4g2X + dlx*dlomx * ( 272.D0/3.D0 + 1468.D
     &    0/3.D0*s12**(-2)*s23**2 + 1468.D0/3.D0*s12**(-1)*s23 - 572.D0
     &    *s13**(-1)*s23 - 572.D0*s13*s23**(-1) )
      finiteAth4g2X = finiteAth4g2X + dlx*dlomx*pi**2 * (  - 4.D0/3.D0
     &     + 16.D0*s12**(-4)*s23**4 + 32.D0*s12**(-3)*s23**3 + 56.D0*
     &    s12**(-2)*s23**2 + 128.D0/3.D0*s12**(-1)*s23 - 30.D0*
     &    s13**(-1)*s23 - 78.D0*s13*s23**(-1) )
      finiteAth4g2X = finiteAth4g2X + dlx*dlomx*dli2x * (  - 16.D0 - 32.
     &    D0*s12**(-1)*s23 - 16.D0*s13**(-1)*s23 + 16.D0*s13*s23**(-1)
     &     )
      finiteAth4g2X = finiteAth4g2X + dlx*dlomx**2 * (  - 1082.D0/3.D0
     &     - 48.D0*s12**(-3)*s23**3 - 136.D0*s12**(-2)*s23**2 - 1430.D0/
     &    3.D0*s12**(-1)*s23 - 275.D0*s13**(-1)*s23 + 257.D0/3.D0*s13*
     &    s23**(-1) - 128.D0*s13**2*s23**(-2) )
      finiteAth4g2X = finiteAth4g2X + dlx*dlomx**3 * ( 16.D0/3.D0 + 16.D
     &    0*s12**(-4)*s23**4 + 32.D0*s12**(-3)*s23**3 + 56.D0*s12**(-2)
     &    *s23**2 + 92.D0/3.D0*s12**(-1)*s23 + 10.D0/3.D0*s13**(-1)*s23
     &     - 10.D0*s13*s23**(-1) )
      finiteAth4g2X = finiteAth4g2X + dlx**2 * (  - 1130.D0/3.D0 - 734.D
     &    0/3.D0*s12**(-2)*s23**2 + 124.D0/3.D0*s12**(-1)*s23 - 734.D0/
     &    3.D0*s13**(-2)*s23**2 + 124.D0/3.D0*s13**(-1)*s23 )
      finiteAth4g2X = finiteAth4g2X + dlx**2*pi**2 * (  - 92.D0/3.D0 - 
     &    8.D0*s12**(-4)*s23**4 - 16.D0*s12**(-3)*s23**3 - 28.D0*
     &    s12**(-2)*s23**2 - 50.D0/3.D0*s12**(-1)*s23 - 16.D0*s13**(-4)
     &    *s23**4 - 32.D0*s13**(-3)*s23**3 - 56.D0*s13**(-2)*s23**2 - 
     &    49.D0*s13**(-1)*s23 - s13*s23**(-1) )
      finiteAth4g2X = finiteAth4g2X + dlx**2*dli2x * (  - 8.D0 + 8.D0*
     &    s12**(-1)*s23 - 8.D0*s13**(-1)*s23 - 16.D0*s13*s23**(-1) )
      finiteAth4g2X = finiteAth4g2X + dlx**2*dlomx * ( 176.D0/3.D0 + 48.
     &    D0*s12**(-3)*s23**3 + 8.D0*s12**(-2)*s23**2 + 318.D0*
     &    s12**(-1)*s23 - 128.D0*s13**(-2)*s23**2 + 349.D0/3.D0*
     &    s13**(-1)*s23 - 641.D0/3.D0*s13*s23**(-1) )
      finiteAth4g2X = finiteAth4g2X + dlx**2*dlomx**2 * (  - 58.D0 - 24.
     &    D0*s12**(-4)*s23**4 - 48.D0*s12**(-3)*s23**3 - 84.D0*
     &    s12**(-2)*s23**2 - 98.D0*s12**(-1)*s23 + 15.D0*s13**(-1)*s23
     &     + 19.D0*s13*s23**(-1) )
      finiteAth4g2X = finiteAth4g2X + dlx**3 * (  - 331.D0/3.D0 - 16.D0
     &    *s12**(-3)*s23**3 + 40.D0*s12**(-2)*s23**2 - 1303.D0/9.D0*
     &    s12**(-1)*s23 - 16.D0*s13**(-3)*s23**3 + 40.D0*s13**(-2)*
     &    s23**2 - 1303.D0/9.D0*s13**(-1)*s23 )
      finiteAth4g2X = finiteAth4g2X + dlx**3*dlomx * ( 64.D0/3.D0 + 16.D
     &    0*s12**(-4)*s23**4 + 32.D0*s12**(-3)*s23**3 + 56.D0*s12**(-2)
     &    *s23**2 + 260.D0/3.D0*s12**(-1)*s23 - 26.D0*s13**(-1)*s23 - 
     &    14.D0/3.D0*s13*s23**(-1) )
      finiteAth4g2X = finiteAth4g2X + dlx**4 * (  - 61.D0/6.D0 - 4.D0*
     &    s12**(-4)*s23**4 - 8.D0*s12**(-3)*s23**3 - 14.D0*s12**(-2)*
     &    s23**2 - 103.D0/6.D0*s12**(-1)*s23 - 4.D0*s13**(-4)*s23**4 - 
     &    8.D0*s13**(-3)*s23**3 - 14.D0*s13**(-2)*s23**2 - 47.D0/6.D0*
     &    s13**(-1)*s23 - 2.D0*s13*s23**(-1) )
      finiteAth4g2X = finiteAth4g2X - 118.D0 - 110.D0/3.D0*s12**(-2)*
     &    s23**2 - 110.D0/3.D0*s12**(-1)*s23 - 110.D0/3.D0*s13**(-2)*
     &    s23**2 - 110.D0/3.D0*s13**(-1)*s23 - 110.D0/3.D0*s13*
     &    s23**(-1) - 110.D0/3.D0*s13**2*s23**(-2)

      endif
      return
      end
************************************************************************
*
      function finiteAthh4g2X(ig1,ig2,ig3,ig4,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)
      double precision dli2x,dli2omx,dli3x,dli3omx,dli4x,dli4omx,dli4xx
      parameter(pi=3.141592653589793238d0)
      parameter(zeta3=1.202056903159594285d0)
      common /order2l/ieorder
      s12 =s4(ig1,ig2)
      s23 =s4(ig2,ig3)
      s13 =s4(ig1,ig3)
      dlx   =log(-s23/s12)
      dlomx =log(-s13/s12)
      dls =log(s12/renscale2)
      dli2x   =rli2(-s23/s12)
      dli2omx =rli2(-s13/s12)
      dli3x   =rli3(-s23/s12)
      dli3omx =rli3(-s13/s12)
      dli4x   =rli4(-s23/s12)
      dli4omx =rli4(-s13/s12)
      dli4xx  =rli4(-s13/s23)
      if(ieorder.eq.-4)then
      finiteAthh4g2X =  0

      elseif(ieorder.eq.-3)then
      finiteAthh4g2X =  0

      elseif(ieorder.eq.-2)then
      finiteAthh4g2X =  0

      elseif(ieorder.eq.-1)then
      finiteAthh4g2X =  0

      elseif(ieorder.eq.0)then
      finiteAthh4g2X =  + pi**2 * (  - 6.D0 - 4.D0*s12**(-2)*s23**2 - 4.
     &    D0*s12**(-1)*s23 - 8.D0*s13**(-2)*s23**2 - 8.D0*s13**(-1)*s23
     &     - 8.D0*s13*s23**(-1) - 8.D0*s13**2*s23**(-2) )
      finiteAthh4g2X = finiteAthh4g2X + pi**4 * (  - 1.D0/2.D0 - 2.D0*
     &    s12**(-4)*s23**4 - 4.D0*s12**(-3)*s23**3 - 4.D0*s12**(-2)*
     &    s23**2 - 2.D0*s12**(-1)*s23 )
      finiteAthh4g2X = finiteAthh4g2X + dlomx * (  - 8.D0/3.D0*
     &    s12**(-1)*s23 + 8.D0/3.D0*s13*s23**(-1) )
      finiteAthh4g2X = finiteAthh4g2X + dlomx*pi**2 * (  - 14.D0/3.D0
     &     + 8.D0*s12**(-3)*s23**3 + 12.D0*s12**(-2)*s23**2 + 8.D0/3.D0
     &    *s12**(-1)*s23 - 40.D0/3.D0*s13*s23**(-1) - 24.D0*s13**2*
     &    s23**(-2) - 16.D0*s13**3*s23**(-3) )
      finiteAthh4g2X = finiteAthh4g2X + dlomx**2 * ( 152.D0/3.D0 + 28.D0
     &    /3.D0*s12**(-2)*s23**2 + 36.D0*s12**(-1)*s23 - 52.D0/3.D0*s13
     &    *s23**(-1) + 28.D0/3.D0*s13**2*s23**(-2) )
      finiteAthh4g2X = finiteAthh4g2X + dlomx**2*pi**2 * (  - 3.D0 - 4.D
     &    0*s12**(-4)*s23**4 - 8.D0*s12**(-3)*s23**3 - 8.D0*s12**(-2)*
     &    s23**2 - 4.D0*s12**(-1)*s23 - 8.D0*s13*s23**(-1) - 16.D0*
     &    s13**2*s23**(-2) - 16.D0*s13**3*s23**(-3) - 8.D0*s13**4*
     &    s23**(-4) )
      finiteAthh4g2X = finiteAthh4g2X + dlomx**3 * (  - 8.D0/3.D0 + 8.D0
     &    *s12**(-3)*s23**3 + 12.D0*s12**(-2)*s23**2 - 12.D0*s13**2*
     &    s23**(-2) - 8.D0*s13**3*s23**(-3) )
      finiteAthh4g2X = finiteAthh4g2X + dlomx**4 * (  - 1.D0 - 2.D0*
     &    s12**(-4)*s23**4 - 4.D0*s12**(-3)*s23**3 - 4.D0*s12**(-2)*
     &    s23**2 - 2.D0*s12**(-1)*s23 - 2.D0*s13*s23**(-1) - 4.D0*
     &    s13**2*s23**(-2) - 4.D0*s13**3*s23**(-3) - 2.D0*s13**4*
     &    s23**(-4) )
      finiteAthh4g2X = finiteAthh4g2X + dlx * ( 8.D0/3.D0 + 8.D0/3.D0*
     &    s12**(-1)*s23 + 8.D0/3.D0*s13**(-1)*s23 )
      finiteAthh4g2X = finiteAthh4g2X + dlx*pi**2 * (  - 10.D0/3.D0 - 8.
     &    D0*s12**(-3)*s23**3 - 12.D0*s12**(-2)*s23**2 - 8.D0/3.D0*
     &    s12**(-1)*s23 - 16.D0*s13**(-3)*s23**3 - 24.D0*s13**(-2)*
     &    s23**2 - 40.D0/3.D0*s13**(-1)*s23 )
      finiteAthh4g2X = finiteAthh4g2X + dlx*dlomx * ( 8.D0/3.D0 - 56.D0/
     &    3.D0*s12**(-2)*s23**2 - 56.D0/3.D0*s12**(-1)*s23 + 160.D0/3.D0
     &    *s13**(-1)*s23 + 160.D0/3.D0*s13*s23**(-1) )
      finiteAthh4g2X = finiteAthh4g2X + dlx*dlomx*pi**2 * ( 2.D0 + 8.D0
     &    *s12**(-4)*s23**4 + 16.D0*s12**(-3)*s23**3 + 16.D0*s12**(-2)*
     &    s23**2 + 8.D0*s12**(-1)*s23 )
      finiteAthh4g2X = finiteAthh4g2X + dlx*dlomx**2 * ( 22.D0/3.D0 - 
     &    24.D0*s12**(-3)*s23**3 - 36.D0*s12**(-2)*s23**2 - 16.D0/3.D0*
     &    s12**(-1)*s23 + 16.D0/3.D0*s13**(-1)*s23 - 16.D0/3.D0*s13*
     &    s23**(-1) )
      finiteAthh4g2X = finiteAthh4g2X + dlx*dlomx**3 * ( 2.D0 + 8.D0*
     &    s12**(-4)*s23**4 + 16.D0*s12**(-3)*s23**3 + 16.D0*s12**(-2)*
     &    s23**2 + 8.D0*s12**(-1)*s23 )
      finiteAthh4g2X = finiteAthh4g2X + dlx**2 * ( 24.D0 + 28.D0/3.D0*
     &    s12**(-2)*s23**2 - 52.D0/3.D0*s12**(-1)*s23 + 28.D0/3.D0*
     &    s13**(-2)*s23**2 - 52.D0/3.D0*s13**(-1)*s23 )
      finiteAthh4g2X = finiteAthh4g2X + dlx**2*pi**2 * (  - 3.D0 - 4.D0
     &    *s12**(-4)*s23**4 - 8.D0*s12**(-3)*s23**3 - 8.D0*s12**(-2)*
     &    s23**2 - 4.D0*s12**(-1)*s23 - 8.D0*s13**(-4)*s23**4 - 16.D0*
     &    s13**(-3)*s23**3 - 16.D0*s13**(-2)*s23**2 - 8.D0*s13**(-1)*
     &    s23 )
      finiteAthh4g2X = finiteAthh4g2X + dlx**2*dlomx * ( 2.D0/3.D0 + 24.
     &    D0*s12**(-3)*s23**3 + 36.D0*s12**(-2)*s23**2 + 16.D0/3.D0*
     &    s12**(-1)*s23 - 16.D0/3.D0*s13**(-1)*s23 + 16.D0/3.D0*s13*
     &    s23**(-1) )
      finiteAthh4g2X = finiteAthh4g2X + dlx**2*dlomx**2 * (  - 3.D0 - 
     &    12.D0*s12**(-4)*s23**4 - 24.D0*s12**(-3)*s23**3 - 24.D0*
     &    s12**(-2)*s23**2 - 12.D0*s12**(-1)*s23 )
      finiteAthh4g2X = finiteAthh4g2X + dlx**3 * ( 4.D0/3.D0 - 8.D0*
     &    s12**(-3)*s23**3 - 12.D0*s12**(-2)*s23**2 - 8.D0*s13**(-3)*
     &    s23**3 - 12.D0*s13**(-2)*s23**2 )
      finiteAthh4g2X = finiteAthh4g2X + dlx**3*dlomx * ( 2.D0 + 8.D0*
     &    s12**(-4)*s23**4 + 16.D0*s12**(-3)*s23**3 + 16.D0*s12**(-2)*
     &    s23**2 + 8.D0*s12**(-1)*s23 )
      finiteAthh4g2X = finiteAthh4g2X + dlx**4 * (  - 1.D0 - 2.D0*
     &    s12**(-4)*s23**4 - 4.D0*s12**(-3)*s23**3 - 4.D0*s12**(-2)*
     &    s23**2 - 2.D0*s12**(-1)*s23 - 2.D0*s13**(-4)*s23**4 - 4.D0*
     &    s13**(-3)*s23**3 - 4.D0*s13**(-2)*s23**2 - 2.D0*s13**(-1)*s23
     &     )
      finiteAthh4g2X = finiteAthh4g2X - 16.D0

      endif
      return
      end
************************************************************************
*
      function finiteAtthh4g2X(ig1,ig2,ig3,ig4,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)
      double precision dli2x,dli2omx,dli3x,dli3omx,dli4x,dli4omx,dli4xx
      parameter(pi=3.141592653589793238d0)
      parameter(zeta3=1.202056903159594285d0)
      common /order2l/ieorder
      s12 =s4(ig1,ig2)
      s23 =s4(ig2,ig3)
      s13 =s4(ig1,ig3)
      dlx   =log(-s23/s12)
      dlomx =log(-s13/s12)
      dls =log(s12/renscale2)
      dli2x   =rli2(-s23/s12)
      dli2omx =rli2(-s13/s12)
      dli3x   =rli3(-s23/s12)
      dli3omx =rli3(-s13/s12)
      dli4x   =rli4(-s23/s12)
      dli4omx =rli4(-s13/s12)
      dli4xx  =rli4(-s13/s23)
      if(ieorder.eq.-4)then
      finiteAtthh4g2X =  0

      elseif(ieorder.eq.-3)then
      finiteAtthh4g2X =  0

      elseif(ieorder.eq.-2)then
      finiteAtthh4g2X =  0

      elseif(ieorder.eq.-1)then
      finiteAtthh4g2X =  0

      elseif(ieorder.eq.0)then
      finiteAtthh4g2X =  + pi**2 * ( 18.D0 + 12.D0*s12**(-2)*s23**2 + 
     &    12.D0*s12**(-1)*s23 + 24.D0*s13**(-2)*s23**2 + 24.D0*
     &    s13**(-1)*s23 + 24.D0*s13*s23**(-1) + 24.D0*s13**2*s23**(-2)
     &     )
      finiteAtthh4g2X = finiteAtthh4g2X + pi**4 * ( 3.D0/2.D0 + 6.D0*
     &    s12**(-4)*s23**4 + 12.D0*s12**(-3)*s23**3 + 12.D0*s12**(-2)*
     &    s23**2 + 6.D0*s12**(-1)*s23 )
      finiteAtthh4g2X = finiteAtthh4g2X + dlomx * (  - 24.D0*s12**(-1)*
     &    s23 + 24.D0*s13*s23**(-1) )
      finiteAtthh4g2X = finiteAtthh4g2X + dlomx*pi**2 * ( 6.D0 - 24.D0*
     &    s12**(-3)*s23**3 - 36.D0*s12**(-2)*s23**2 - 24.D0*s12**(-1)*
     &    s23 + 48.D0*s13*s23**(-1) + 72.D0*s13**2*s23**(-2) + 48.D0*
     &    s13**3*s23**(-3) )
      finiteAtthh4g2X = finiteAtthh4g2X + dlomx**2 * ( 24.D0 + 36.D0*
     &    s12**(-2)*s23**2 + 36.D0*s12**(-1)*s23 + 36.D0*s13*s23**(-1)
     &     + 36.D0*s13**2*s23**(-2) )
      finiteAtthh4g2X = finiteAtthh4g2X + dlomx**2*pi**2 * ( 9.D0 + 12.D
     &    0*s12**(-4)*s23**4 + 24.D0*s12**(-3)*s23**3 + 24.D0*s12**(-2)
     &    *s23**2 + 12.D0*s12**(-1)*s23 + 24.D0*s13*s23**(-1) + 48.D0*
     &    s13**2*s23**(-2) + 48.D0*s13**3*s23**(-3) + 24.D0*s13**4*
     &    s23**(-4) )
      finiteAtthh4g2X = finiteAtthh4g2X + dlomx**3 * (  - 24.D0*
     &    s12**(-3)*s23**3 - 36.D0*s12**(-2)*s23**2 - 24.D0*s12**(-1)*
     &    s23 + 24.D0*s13*s23**(-1) + 36.D0*s13**2*s23**(-2) + 24.D0*
     &    s13**3*s23**(-3) )
      finiteAtthh4g2X = finiteAtthh4g2X + dlomx**4 * ( 3.D0 + 6.D0*
     &    s12**(-4)*s23**4 + 12.D0*s12**(-3)*s23**3 + 12.D0*s12**(-2)*
     &    s23**2 + 6.D0*s12**(-1)*s23 + 6.D0*s13*s23**(-1) + 12.D0*
     &    s13**2*s23**(-2) + 12.D0*s13**3*s23**(-3) + 6.D0*s13**4*
     &    s23**(-4) )
      finiteAtthh4g2X = finiteAtthh4g2X + dlx * ( 24.D0 + 24.D0*
     &    s12**(-1)*s23 + 24.D0*s13**(-1)*s23 )
      finiteAtthh4g2X = finiteAtthh4g2X + dlx*pi**2 * ( 18.D0 + 24.D0*
     &    s12**(-3)*s23**3 + 36.D0*s12**(-2)*s23**2 + 24.D0*s12**(-1)*
     &    s23 + 48.D0*s13**(-3)*s23**3 + 72.D0*s13**(-2)*s23**2 + 48.D0
     &    *s13**(-1)*s23 )
      finiteAtthh4g2X = finiteAtthh4g2X + dlx*dlomx * (  - 24.D0 - 72.D0
     &    *s12**(-2)*s23**2 - 72.D0*s12**(-1)*s23 )
      finiteAtthh4g2X = finiteAtthh4g2X + dlx*dlomx*pi**2 * (  - 6.D0
     &     - 24.D0*s12**(-4)*s23**4 - 48.D0*s12**(-3)*s23**3 - 48.D0*
     &    s12**(-2)*s23**2 - 24.D0*s12**(-1)*s23 )
      finiteAtthh4g2X = finiteAtthh4g2X + dlx*dlomx**2 * ( 18.D0 + 72.D0
     &    *s12**(-3)*s23**3 + 108.D0*s12**(-2)*s23**2 + 72.D0*s12**(-1)
     &    *s23 )
      finiteAtthh4g2X = finiteAtthh4g2X + dlx*dlomx**3 * (  - 6.D0 - 24.
     &    D0*s12**(-4)*s23**4 - 48.D0*s12**(-3)*s23**3 - 48.D0*
     &    s12**(-2)*s23**2 - 24.D0*s12**(-1)*s23 )
      finiteAtthh4g2X = finiteAtthh4g2X + dlx**2 * ( 24.D0 + 36.D0*
     &    s12**(-2)*s23**2 + 36.D0*s12**(-1)*s23 + 36.D0*s13**(-2)*
     &    s23**2 + 36.D0*s13**(-1)*s23 )
      finiteAtthh4g2X = finiteAtthh4g2X + dlx**2*pi**2 * ( 9.D0 + 12.D0
     &    *s12**(-4)*s23**4 + 24.D0*s12**(-3)*s23**3 + 24.D0*s12**(-2)*
     &    s23**2 + 12.D0*s12**(-1)*s23 + 24.D0*s13**(-4)*s23**4 + 48.D0
     &    *s13**(-3)*s23**3 + 48.D0*s13**(-2)*s23**2 + 24.D0*s13**(-1)*
     &    s23 )
      finiteAtthh4g2X = finiteAtthh4g2X + dlx**2*dlomx * (  - 18.D0 - 
     &    72.D0*s12**(-3)*s23**3 - 108.D0*s12**(-2)*s23**2 - 72.D0*
     &    s12**(-1)*s23 )
      finiteAtthh4g2X = finiteAtthh4g2X + dlx**2*dlomx**2 * ( 9.D0 + 36.
     &    D0*s12**(-4)*s23**4 + 72.D0*s12**(-3)*s23**3 + 72.D0*
     &    s12**(-2)*s23**2 + 36.D0*s12**(-1)*s23 )
      finiteAtthh4g2X = finiteAtthh4g2X + dlx**3 * ( 12.D0 + 24.D0*
     &    s12**(-3)*s23**3 + 36.D0*s12**(-2)*s23**2 + 24.D0*s12**(-1)*
     &    s23 + 24.D0*s13**(-3)*s23**3 + 36.D0*s13**(-2)*s23**2 + 24.D0
     &    *s13**(-1)*s23 )
      finiteAtthh4g2X = finiteAtthh4g2X + dlx**3*dlomx * (  - 6.D0 - 24.
     &    D0*s12**(-4)*s23**4 - 48.D0*s12**(-3)*s23**3 - 48.D0*
     &    s12**(-2)*s23**2 - 24.D0*s12**(-1)*s23 )
      finiteAtthh4g2X = finiteAtthh4g2X + dlx**4 * ( 3.D0 + 6.D0*
     &    s12**(-4)*s23**4 + 12.D0*s12**(-3)*s23**3 + 12.D0*s12**(-2)*
     &    s23**2 + 6.D0*s12**(-1)*s23 + 6.D0*s13**(-4)*s23**4 + 12.D0*
     &    s13**(-3)*s23**3 + 12.D0*s13**(-2)*s23**2 + 6.D0*s13**(-1)*
     &    s23 )
      finiteAtthh4g2X = finiteAtthh4g2X + 48.D0

      endif
      return
      end
