*****************************************************************************************
*
*  identical four quark one loop matrix elements interfered with tree level
*
*                      q1  qb2  q2  qb1
*
* summed over colour orderings
*
* additional functions: D0g1subA, D0g1subB, D0g1subC, D0g1subD 
*
*   |0> = (T1, T2)^{T},    |1> = (L1, L2)^{T},     <0| = (T1,T2),     <1| = (L1, L2)
*
*   |0b> = (T2b, T1b)^{T}, |1b> = (L2b, L1b)^{T},  <0b| = (T2b,T1b),  <1b| = (L2b, L1b)
*
*   <a|b> = Ma(i)*Mb(j) * CC(i,j),    where CC(1,1)=CC(2,2) = N^2 and CC(1,2)=CC(2,1)=N
*
*   T2 = -1/N*T1 and T2b = -1/N*T1b  such that
*
*   <0b|1>                                   = V*T1b*L2
*   <1|0b> = charge conjugate of <0b|1>
*   <0|1b> = flip of <0b|1>                  = V*T1*L2b 
*
*   D0g1subA = T1b*L1
*   D0g1subB = T1b*L2
*   D0g1subC = T1*L1b
*   D0g1subD = T1*L2b
*
*   D0g1 = <0b|1>+<0|1b>+CC = 2*(D0g1subB+D0g1subD)*(-1)  
*
*G M01Aqq = DM0111qq -(AAA(s13,s23,s12,e)-1/N*BBB(s13,s23,s12,e))*(DM0011qq-M0011qq);
*G M01Bqq = DM0112qq -(BBB(s23,s13,s12,e)-1/N*AAA(s23,s13,s12,e))*(DM0011qq-M0011qq);
*G M01Cqq = DM0122qqa-(AAA(s23,s13,s12,e)-1/N*BBB(s23,s13,s12,e))*(DM0011qqa-M0011qqa);
*G M01Dqq = DM0121qqa-(BBB(s13,s23,s12,e)-1/N*AAA(s13,s23,s12,e))*(DM0011qqa-M0011qqa);
*
*****************************************************************************************




************************************************************************
*
      function D0g1subA(iq1,iqb2,iq2,iqb1,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)

      if((iq1.eq.1).and.(iqb2.eq.2).and.(iq2.eq.3).and.(iqb1.eq.4))then
        D0g1subA=D0g1subA1(iq1,iqb1,iq2,iqb2,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.2).and.(iq2.eq.4).and.(iqb1.eq.3))then
        D0g1subA=D0g1subA1(iq1,iqb1,iq2,iqb2,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.3).and.(iq2.eq.2).and.(iqb1.eq.4))then
        D0g1subA=D0g1subA2(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.3).and.(iq2.eq.4).and.(iqb1.eq.2))then
        D0g1subA=D0g1subA1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.4).and.(iq2.eq.2).and.(iqb1.eq.3))then
        D0g1subA=D0g1subA2(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.4).and.(iq2.eq.3).and.(iqb1.eq.2))then
        D0g1subA=D0g1subA1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.1).and.(iq2.eq.3).and.(iqb1.eq.4))then
        D0g1subA=D0g1subA1(iq1,iqb1,iq2,iqb2,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.1).and.(iq2.eq.4).and.(iqb1.eq.3))then
        D0g1subA=D0g1subA1(iq1,iqb1,iq2,iqb2,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.3).and.(iq2.eq.1).and.(iqb1.eq.4))then
        D0g1subA=D0g1subA2(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.3).and.(iq2.eq.4).and.(iqb1.eq.1))then
        D0g1subA=D0g1subA1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.4).and.(iq2.eq.1).and.(iqb1.eq.3))then
        D0g1subA=D0g1subA2(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.4).and.(iq2.eq.3).and.(iqb1.eq.1))then
        D0g1subA=D0g1subA1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.1).and.(iq2.eq.2).and.(iqb1.eq.4))then
        D0g1subA=D0g1subA1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.1).and.(iq2.eq.4).and.(iqb1.eq.2))then
        D0g1subA=D0g1subA2(iqb1,iq2,iqb2,iq1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.2).and.(iq2.eq.1).and.(iqb1.eq.4))then
        D0g1subA=D0g1subA1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.2).and.(iq2.eq.4).and.(iqb1.eq.1))then
        D0g1subA=D0g1subA2(iqb1,iq2,iqb2,iq1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.4).and.(iq2.eq.1).and.(iqb1.eq.2))then
        D0g1subA=D0g1subA1(iqb2,iq2,iqb1,iq1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.4).and.(iq2.eq.2).and.(iqb1.eq.1))then
        D0g1subA=D0g1subA1(iqb2,iq2,iqb1,iq1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.1).and.(iq2.eq.2).and.(iqb1.eq.3))then
        D0g1subA=D0g1subA1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.1).and.(iq2.eq.3).and.(iqb1.eq.2))then
        D0g1subA=D0g1subA2(iqb1,iq2,iqb2,iq1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.2).and.(iq2.eq.1).and.(iqb1.eq.3))then
        D0g1subA=D0g1subA1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.2).and.(iq2.eq.3).and.(iqb1.eq.1))then
        D0g1subA=D0g1subA2(iqb1,iq2,iqb2,iq1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.3).and.(iq2.eq.1).and.(iqb1.eq.2))then
        D0g1subA=D0g1subA1(iqb1,iq1,iqb2,iq2,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.3).and.(iq2.eq.2).and.(iqb1.eq.1))then
        D0g1subA=D0g1subA1(iqb1,iq1,iqb2,iq2,renscale2)
      endif
      return
      end
*
************************************************************************
*
      function Dh0g1subA(iq1,iqb2,iq2,iqb1,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)

      if((iq1.eq.1).and.(iqb2.eq.2).and.(iq2.eq.3).and.(iqb1.eq.4))then
        Dh0g1subA=Dh0g1subA1(iq1,iqb1,iq2,iqb2,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.2).and.(iq2.eq.4).and.(iqb1.eq.3))then
        Dh0g1subA=Dh0g1subA1(iq1,iqb1,iq2,iqb2,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.3).and.(iq2.eq.2).and.(iqb1.eq.4))then
        Dh0g1subA=Dh0g1subA2(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.3).and.(iq2.eq.4).and.(iqb1.eq.2))then
        Dh0g1subA=Dh0g1subA1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.4).and.(iq2.eq.2).and.(iqb1.eq.3))then
        Dh0g1subA=Dh0g1subA2(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.4).and.(iq2.eq.3).and.(iqb1.eq.2))then
        Dh0g1subA=Dh0g1subA1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.1).and.(iq2.eq.3).and.(iqb1.eq.4))then
        Dh0g1subA=Dh0g1subA1(iq1,iqb1,iq2,iqb2,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.1).and.(iq2.eq.4).and.(iqb1.eq.3))then
        Dh0g1subA=Dh0g1subA1(iq1,iqb1,iq2,iqb2,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.3).and.(iq2.eq.1).and.(iqb1.eq.4))then
        Dh0g1subA=Dh0g1subA2(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.3).and.(iq2.eq.4).and.(iqb1.eq.1))then
        Dh0g1subA=Dh0g1subA1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.4).and.(iq2.eq.1).and.(iqb1.eq.3))then
        Dh0g1subA=Dh0g1subA2(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.4).and.(iq2.eq.3).and.(iqb1.eq.1))then
        Dh0g1subA=Dh0g1subA1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.1).and.(iq2.eq.2).and.(iqb1.eq.4))then
        Dh0g1subA=Dh0g1subA1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.1).and.(iq2.eq.4).and.(iqb1.eq.2))then
        Dh0g1subA=Dh0g1subA2(iqb1,iq2,iqb2,iq1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.2).and.(iq2.eq.1).and.(iqb1.eq.4))then
        Dh0g1subA=Dh0g1subA1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.2).and.(iq2.eq.4).and.(iqb1.eq.1))then
        Dh0g1subA=Dh0g1subA2(iqb1,iq2,iqb2,iq1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.4).and.(iq2.eq.1).and.(iqb1.eq.2))then
        Dh0g1subA=Dh0g1subA1(iqb2,iq2,iqb1,iq1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.4).and.(iq2.eq.2).and.(iqb1.eq.1))then
        Dh0g1subA=Dh0g1subA1(iqb2,iq2,iqb1,iq1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.1).and.(iq2.eq.2).and.(iqb1.eq.3))then
        Dh0g1subA=Dh0g1subA1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.1).and.(iq2.eq.3).and.(iqb1.eq.2))then
        Dh0g1subA=Dh0g1subA2(iqb1,iq2,iqb2,iq1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.2).and.(iq2.eq.1).and.(iqb1.eq.3))then
        Dh0g1subA=Dh0g1subA1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.2).and.(iq2.eq.3).and.(iqb1.eq.1))then
        Dh0g1subA=Dh0g1subA2(iqb1,iq2,iqb2,iq1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.3).and.(iq2.eq.1).and.(iqb1.eq.2))then
        Dh0g1subA=Dh0g1subA1(iqb1,iq1,iqb2,iq2,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.3).and.(iq2.eq.2).and.(iqb1.eq.1))then
        Dh0g1subA=Dh0g1subA1(iqb1,iq1,iqb2,iq2,renscale2)
      endif
      return
      end
************************************************************************
*
      function Dt0g1subA(iq1,iqb2,iq2,iqb1,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)

      if((iq1.eq.1).and.(iqb2.eq.2).and.(iq2.eq.3).and.(iqb1.eq.4))then
        Dt0g1subA=Dt0g1subA1(iq1,iqb1,iq2,iqb2,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.2).and.(iq2.eq.4).and.(iqb1.eq.3))then
        Dt0g1subA=Dt0g1subA1(iq1,iqb1,iq2,iqb2,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.3).and.(iq2.eq.2).and.(iqb1.eq.4))then
        Dt0g1subA=Dt0g1subA2(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.3).and.(iq2.eq.4).and.(iqb1.eq.2))then
        Dt0g1subA=Dt0g1subA1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.4).and.(iq2.eq.2).and.(iqb1.eq.3))then
        Dt0g1subA=Dt0g1subA2(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.4).and.(iq2.eq.3).and.(iqb1.eq.2))then
        Dt0g1subA=Dt0g1subA1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.1).and.(iq2.eq.3).and.(iqb1.eq.4))then
        Dt0g1subA=Dt0g1subA1(iq1,iqb1,iq2,iqb2,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.1).and.(iq2.eq.4).and.(iqb1.eq.3))then
        Dt0g1subA=Dt0g1subA1(iq1,iqb1,iq2,iqb2,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.3).and.(iq2.eq.1).and.(iqb1.eq.4))then
        Dt0g1subA=Dt0g1subA2(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.3).and.(iq2.eq.4).and.(iqb1.eq.1))then
        Dt0g1subA=Dt0g1subA1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.4).and.(iq2.eq.1).and.(iqb1.eq.3))then
        Dt0g1subA=Dt0g1subA2(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.4).and.(iq2.eq.3).and.(iqb1.eq.1))then
        Dt0g1subA=Dt0g1subA1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.1).and.(iq2.eq.2).and.(iqb1.eq.4))then
        Dt0g1subA=Dt0g1subA1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.1).and.(iq2.eq.4).and.(iqb1.eq.2))then
        Dt0g1subA=Dt0g1subA2(iqb1,iq2,iqb2,iq1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.2).and.(iq2.eq.1).and.(iqb1.eq.4))then
        Dt0g1subA=Dt0g1subA1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.2).and.(iq2.eq.4).and.(iqb1.eq.1))then
        Dt0g1subA=Dt0g1subA2(iqb1,iq2,iqb2,iq1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.4).and.(iq2.eq.1).and.(iqb1.eq.2))then
        Dt0g1subA=Dt0g1subA1(iqb2,iq2,iqb1,iq1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.4).and.(iq2.eq.2).and.(iqb1.eq.1))then
        Dt0g1subA=Dt0g1subA1(iqb2,iq2,iqb1,iq1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.1).and.(iq2.eq.2).and.(iqb1.eq.3))then
        Dt0g1subA=Dt0g1subA1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.1).and.(iq2.eq.3).and.(iqb1.eq.2))then
        Dt0g1subA=Dt0g1subA2(iqb1,iq2,iqb2,iq1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.2).and.(iq2.eq.1).and.(iqb1.eq.3))then
        Dt0g1subA=Dt0g1subA1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.2).and.(iq2.eq.3).and.(iqb1.eq.1))then
        Dt0g1subA=Dt0g1subA2(iqb1,iq2,iqb2,iq1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.3).and.(iq2.eq.1).and.(iqb1.eq.2))then
        Dt0g1subA=Dt0g1subA1(iqb1,iq1,iqb2,iq2,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.3).and.(iq2.eq.2).and.(iqb1.eq.1))then
        Dt0g1subA=Dt0g1subA1(iqb1,iq1,iqb2,iq2,renscale2)
      endif
      return
      end


************************************************************************
*
      function D0g1subB(iq1,iqb2,iq2,iqb1,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)

      if((iq1.eq.1).and.(iqb2.eq.2).and.(iq2.eq.3).and.(iqb1.eq.4))then
        D0g1subB=D0g1subB1(iq1,iqb1,iq2,iqb2,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.2).and.(iq2.eq.4).and.(iqb1.eq.3))then
        D0g1subB=D0g1subB1(iq1,iqb1,iq2,iqb2,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.3).and.(iq2.eq.2).and.(iqb1.eq.4))then
        D0g1subB=D0g1subB2(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.3).and.(iq2.eq.4).and.(iqb1.eq.2))then
        D0g1subB=D0g1subB1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.4).and.(iq2.eq.2).and.(iqb1.eq.3))then
        D0g1subB=D0g1subB2(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.4).and.(iq2.eq.3).and.(iqb1.eq.2))then
        D0g1subB=D0g1subB1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.1).and.(iq2.eq.3).and.(iqb1.eq.4))then
        D0g1subB=D0g1subB1(iq1,iqb1,iq2,iqb2,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.1).and.(iq2.eq.4).and.(iqb1.eq.3))then
        D0g1subB=D0g1subB1(iq1,iqb1,iq2,iqb2,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.3).and.(iq2.eq.1).and.(iqb1.eq.4))then
        D0g1subB=D0g1subB2(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.3).and.(iq2.eq.4).and.(iqb1.eq.1))then
        D0g1subB=D0g1subB1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.4).and.(iq2.eq.1).and.(iqb1.eq.3))then
        D0g1subB=D0g1subB2(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.4).and.(iq2.eq.3).and.(iqb1.eq.1))then
        D0g1subB=D0g1subB1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.1).and.(iq2.eq.2).and.(iqb1.eq.4))then
        D0g1subB=D0g1subB1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.1).and.(iq2.eq.4).and.(iqb1.eq.2))then
        D0g1subB=D0g1subB2(iqb1,iq2,iqb2,iq1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.2).and.(iq2.eq.1).and.(iqb1.eq.4))then
        D0g1subB=D0g1subB1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.2).and.(iq2.eq.4).and.(iqb1.eq.1))then
        D0g1subB=D0g1subB2(iqb1,iq2,iqb2,iq1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.4).and.(iq2.eq.1).and.(iqb1.eq.2))then
        D0g1subB=D0g1subB1(iqb2,iq2,iqb1,iq1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.4).and.(iq2.eq.2).and.(iqb1.eq.1))then
        D0g1subB=D0g1subB1(iqb2,iq2,iqb1,iq1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.1).and.(iq2.eq.2).and.(iqb1.eq.3))then
        D0g1subB=D0g1subB1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.1).and.(iq2.eq.3).and.(iqb1.eq.2))then
        D0g1subB=D0g1subB2(iqb1,iq2,iqb2,iq1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.2).and.(iq2.eq.1).and.(iqb1.eq.3))then
        D0g1subB=D0g1subB1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.2).and.(iq2.eq.3).and.(iqb1.eq.1))then
        D0g1subB=D0g1subB2(iqb1,iq2,iqb2,iq1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.3).and.(iq2.eq.1).and.(iqb1.eq.2))then
        D0g1subB=D0g1subB1(iqb1,iq1,iqb2,iq2,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.3).and.(iq2.eq.2).and.(iqb1.eq.1))then
        D0g1subB=D0g1subB1(iqb1,iq1,iqb2,iq2,renscale2)
      endif
      return
      end
*
************************************************************************
*
      function Dh0g1subB(iq1,iqb2,iq2,iqb1,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)

      if((iq1.eq.1).and.(iqb2.eq.2).and.(iq2.eq.3).and.(iqb1.eq.4))then
        Dh0g1subB=Dh0g1subB1(iq1,iqb1,iq2,iqb2,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.2).and.(iq2.eq.4).and.(iqb1.eq.3))then
        Dh0g1subB=Dh0g1subB1(iq1,iqb1,iq2,iqb2,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.3).and.(iq2.eq.2).and.(iqb1.eq.4))then
        Dh0g1subB=Dh0g1subB2(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.3).and.(iq2.eq.4).and.(iqb1.eq.2))then
        Dh0g1subB=Dh0g1subB1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.4).and.(iq2.eq.2).and.(iqb1.eq.3))then
        Dh0g1subB=Dh0g1subB2(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.4).and.(iq2.eq.3).and.(iqb1.eq.2))then
        Dh0g1subB=Dh0g1subB1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.1).and.(iq2.eq.3).and.(iqb1.eq.4))then
        Dh0g1subB=Dh0g1subB1(iq1,iqb1,iq2,iqb2,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.1).and.(iq2.eq.4).and.(iqb1.eq.3))then
        Dh0g1subB=Dh0g1subB1(iq1,iqb1,iq2,iqb2,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.3).and.(iq2.eq.1).and.(iqb1.eq.4))then
        Dh0g1subB=Dh0g1subB2(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.3).and.(iq2.eq.4).and.(iqb1.eq.1))then
        Dh0g1subB=Dh0g1subB1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.4).and.(iq2.eq.1).and.(iqb1.eq.3))then
        Dh0g1subB=Dh0g1subB2(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.4).and.(iq2.eq.3).and.(iqb1.eq.1))then
        Dh0g1subB=Dh0g1subB1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.1).and.(iq2.eq.2).and.(iqb1.eq.4))then
        Dh0g1subB=Dh0g1subB1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.1).and.(iq2.eq.4).and.(iqb1.eq.2))then
        Dh0g1subB=Dh0g1subB2(iqb1,iq2,iqb2,iq1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.2).and.(iq2.eq.1).and.(iqb1.eq.4))then
        Dh0g1subB=Dh0g1subB1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.2).and.(iq2.eq.4).and.(iqb1.eq.1))then
        Dh0g1subB=Dh0g1subB2(iqb1,iq2,iqb2,iq1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.4).and.(iq2.eq.1).and.(iqb1.eq.2))then
        Dh0g1subB=Dh0g1subB1(iqb2,iq2,iqb1,iq1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.4).and.(iq2.eq.2).and.(iqb1.eq.1))then
        Dh0g1subB=Dh0g1subB1(iqb2,iq2,iqb1,iq1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.1).and.(iq2.eq.2).and.(iqb1.eq.3))then
        Dh0g1subB=Dh0g1subB1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.1).and.(iq2.eq.3).and.(iqb1.eq.2))then
        Dh0g1subB=Dh0g1subB2(iqb1,iq2,iqb2,iq1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.2).and.(iq2.eq.1).and.(iqb1.eq.3))then
        Dh0g1subB=Dh0g1subB1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.2).and.(iq2.eq.3).and.(iqb1.eq.1))then
        Dh0g1subB=Dh0g1subB2(iqb1,iq2,iqb2,iq1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.3).and.(iq2.eq.1).and.(iqb1.eq.2))then
        Dh0g1subB=Dh0g1subB1(iqb1,iq1,iqb2,iq2,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.3).and.(iq2.eq.2).and.(iqb1.eq.1))then
        Dh0g1subB=Dh0g1subB1(iqb1,iq1,iqb2,iq2,renscale2)
      endif
      return
      end
************************************************************************
*
      function Dt0g1subB(iq1,iqb2,iq2,iqb1,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)

      if((iq1.eq.1).and.(iqb2.eq.2).and.(iq2.eq.3).and.(iqb1.eq.4))then
        Dt0g1subB=Dt0g1subB1(iq1,iqb1,iq2,iqb2,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.2).and.(iq2.eq.4).and.(iqb1.eq.3))then
        Dt0g1subB=Dt0g1subB1(iq1,iqb1,iq2,iqb2,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.3).and.(iq2.eq.2).and.(iqb1.eq.4))then
        Dt0g1subB=Dt0g1subB2(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.3).and.(iq2.eq.4).and.(iqb1.eq.2))then
        Dt0g1subB=Dt0g1subB1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.4).and.(iq2.eq.2).and.(iqb1.eq.3))then
        Dt0g1subB=Dt0g1subB2(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.4).and.(iq2.eq.3).and.(iqb1.eq.2))then
        Dt0g1subB=Dt0g1subB1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.1).and.(iq2.eq.3).and.(iqb1.eq.4))then
        Dt0g1subB=Dt0g1subB1(iq1,iqb1,iq2,iqb2,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.1).and.(iq2.eq.4).and.(iqb1.eq.3))then
        Dt0g1subB=Dt0g1subB1(iq1,iqb1,iq2,iqb2,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.3).and.(iq2.eq.1).and.(iqb1.eq.4))then
        Dt0g1subB=Dt0g1subB2(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.3).and.(iq2.eq.4).and.(iqb1.eq.1))then
        Dt0g1subB=Dt0g1subB1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.4).and.(iq2.eq.1).and.(iqb1.eq.3))then
        Dt0g1subB=Dt0g1subB2(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.4).and.(iq2.eq.3).and.(iqb1.eq.1))then
        Dt0g1subB=Dt0g1subB1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.1).and.(iq2.eq.2).and.(iqb1.eq.4))then
        Dt0g1subB=Dt0g1subB1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.1).and.(iq2.eq.4).and.(iqb1.eq.2))then
        Dt0g1subB=Dt0g1subB2(iqb1,iq2,iqb2,iq1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.2).and.(iq2.eq.1).and.(iqb1.eq.4))then
        Dt0g1subB=Dt0g1subB1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.2).and.(iq2.eq.4).and.(iqb1.eq.1))then
        Dt0g1subB=Dt0g1subB2(iqb1,iq2,iqb2,iq1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.4).and.(iq2.eq.1).and.(iqb1.eq.2))then
        Dt0g1subB=Dt0g1subB1(iqb2,iq2,iqb1,iq1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.4).and.(iq2.eq.2).and.(iqb1.eq.1))then
        Dt0g1subB=Dt0g1subB1(iqb2,iq2,iqb1,iq1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.1).and.(iq2.eq.2).and.(iqb1.eq.3))then
        Dt0g1subB=Dt0g1subB1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.1).and.(iq2.eq.3).and.(iqb1.eq.2))then
        Dt0g1subB=Dt0g1subB2(iqb1,iq2,iqb2,iq1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.2).and.(iq2.eq.1).and.(iqb1.eq.3))then
        Dt0g1subB=Dt0g1subB1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.2).and.(iq2.eq.3).and.(iqb1.eq.1))then
        Dt0g1subB=Dt0g1subB2(iqb1,iq2,iqb2,iq1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.3).and.(iq2.eq.1).and.(iqb1.eq.2))then
        Dt0g1subB=Dt0g1subB1(iqb1,iq1,iqb2,iq2,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.3).and.(iq2.eq.2).and.(iqb1.eq.1))then
        Dt0g1subB=Dt0g1subB1(iqb1,iq1,iqb2,iq2,renscale2)
      endif
      return
      end


************************************************************************
*
      function D0g1subC(iq1,iqb2,iq2,iqb1,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)

      if((iq1.eq.1).and.(iqb2.eq.2).and.(iq2.eq.3).and.(iqb1.eq.4))then
        D0g1subC=D0g1subC1(iq1,iqb1,iq2,iqb2,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.2).and.(iq2.eq.4).and.(iqb1.eq.3))then
        D0g1subC=D0g1subC1(iq1,iqb1,iq2,iqb2,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.3).and.(iq2.eq.2).and.(iqb1.eq.4))then
        D0g1subC=D0g1subC2(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.3).and.(iq2.eq.4).and.(iqb1.eq.2))then
        D0g1subC=D0g1subC1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.4).and.(iq2.eq.2).and.(iqb1.eq.3))then
        D0g1subC=D0g1subC2(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.4).and.(iq2.eq.3).and.(iqb1.eq.2))then
        D0g1subC=D0g1subC1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.1).and.(iq2.eq.3).and.(iqb1.eq.4))then
        D0g1subC=D0g1subC1(iq1,iqb1,iq2,iqb2,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.1).and.(iq2.eq.4).and.(iqb1.eq.3))then
        D0g1subC=D0g1subC1(iq1,iqb1,iq2,iqb2,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.3).and.(iq2.eq.1).and.(iqb1.eq.4))then
        D0g1subC=D0g1subC2(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.3).and.(iq2.eq.4).and.(iqb1.eq.1))then
        D0g1subC=D0g1subC1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.4).and.(iq2.eq.1).and.(iqb1.eq.3))then
        D0g1subC=D0g1subC2(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.4).and.(iq2.eq.3).and.(iqb1.eq.1))then
        D0g1subC=D0g1subC1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.1).and.(iq2.eq.2).and.(iqb1.eq.4))then
        D0g1subC=D0g1subC1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.1).and.(iq2.eq.4).and.(iqb1.eq.2))then
        D0g1subC=D0g1subC2(iqb1,iq2,iqb2,iq1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.2).and.(iq2.eq.1).and.(iqb1.eq.4))then
        D0g1subC=D0g1subC1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.2).and.(iq2.eq.4).and.(iqb1.eq.1))then
        D0g1subC=D0g1subC2(iqb1,iq2,iqb2,iq1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.4).and.(iq2.eq.1).and.(iqb1.eq.2))then
        D0g1subC=D0g1subC1(iqb2,iq2,iqb1,iq1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.4).and.(iq2.eq.2).and.(iqb1.eq.1))then
        D0g1subC=D0g1subC1(iqb2,iq2,iqb1,iq1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.1).and.(iq2.eq.2).and.(iqb1.eq.3))then
        D0g1subC=D0g1subC1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.1).and.(iq2.eq.3).and.(iqb1.eq.2))then
        D0g1subC=D0g1subC2(iqb1,iq2,iqb2,iq1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.2).and.(iq2.eq.1).and.(iqb1.eq.3))then
        D0g1subC=D0g1subC1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.2).and.(iq2.eq.3).and.(iqb1.eq.1))then
        D0g1subC=D0g1subC2(iqb1,iq2,iqb2,iq1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.3).and.(iq2.eq.1).and.(iqb1.eq.2))then
        D0g1subC=D0g1subC1(iqb1,iq1,iqb2,iq2,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.3).and.(iq2.eq.2).and.(iqb1.eq.1))then
        D0g1subC=D0g1subC1(iqb1,iq1,iqb2,iq2,renscale2)
      endif
      return
      end
*
************************************************************************
*
      function Dh0g1subC(iq1,iqb2,iq2,iqb1,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)

      if((iq1.eq.1).and.(iqb2.eq.2).and.(iq2.eq.3).and.(iqb1.eq.4))then
        Dh0g1subC=Dh0g1subC1(iq1,iqb1,iq2,iqb2,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.2).and.(iq2.eq.4).and.(iqb1.eq.3))then
        Dh0g1subC=Dh0g1subC1(iq1,iqb1,iq2,iqb2,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.3).and.(iq2.eq.2).and.(iqb1.eq.4))then
        Dh0g1subC=Dh0g1subC2(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.3).and.(iq2.eq.4).and.(iqb1.eq.2))then
        Dh0g1subC=Dh0g1subC1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.4).and.(iq2.eq.2).and.(iqb1.eq.3))then
        Dh0g1subC=Dh0g1subC2(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.4).and.(iq2.eq.3).and.(iqb1.eq.2))then
        Dh0g1subC=Dh0g1subC1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.1).and.(iq2.eq.3).and.(iqb1.eq.4))then
        Dh0g1subC=Dh0g1subC1(iq1,iqb1,iq2,iqb2,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.1).and.(iq2.eq.4).and.(iqb1.eq.3))then
        Dh0g1subC=Dh0g1subC1(iq1,iqb1,iq2,iqb2,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.3).and.(iq2.eq.1).and.(iqb1.eq.4))then
        Dh0g1subC=Dh0g1subC2(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.3).and.(iq2.eq.4).and.(iqb1.eq.1))then
        Dh0g1subC=Dh0g1subC1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.4).and.(iq2.eq.1).and.(iqb1.eq.3))then
        Dh0g1subC=Dh0g1subC2(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.4).and.(iq2.eq.3).and.(iqb1.eq.1))then
        Dh0g1subC=Dh0g1subC1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.1).and.(iq2.eq.2).and.(iqb1.eq.4))then
        Dh0g1subC=Dh0g1subC1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.1).and.(iq2.eq.4).and.(iqb1.eq.2))then
        Dh0g1subC=Dh0g1subC2(iqb1,iq2,iqb2,iq1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.2).and.(iq2.eq.1).and.(iqb1.eq.4))then
        Dh0g1subC=Dh0g1subC1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.2).and.(iq2.eq.4).and.(iqb1.eq.1))then
        Dh0g1subC=Dh0g1subC2(iqb1,iq2,iqb2,iq1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.4).and.(iq2.eq.1).and.(iqb1.eq.2))then
        Dh0g1subC=Dh0g1subC1(iqb2,iq2,iqb1,iq1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.4).and.(iq2.eq.2).and.(iqb1.eq.1))then
        Dh0g1subC=Dh0g1subC1(iqb2,iq2,iqb1,iq1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.1).and.(iq2.eq.2).and.(iqb1.eq.3))then
        Dh0g1subC=Dh0g1subC1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.1).and.(iq2.eq.3).and.(iqb1.eq.2))then
        Dh0g1subC=Dh0g1subC2(iqb1,iq2,iqb2,iq1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.2).and.(iq2.eq.1).and.(iqb1.eq.3))then
        Dh0g1subC=Dh0g1subC1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.2).and.(iq2.eq.3).and.(iqb1.eq.1))then
        Dh0g1subC=Dh0g1subC2(iqb1,iq2,iqb2,iq1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.3).and.(iq2.eq.1).and.(iqb1.eq.2))then
        Dh0g1subC=Dh0g1subC1(iqb1,iq1,iqb2,iq2,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.3).and.(iq2.eq.2).and.(iqb1.eq.1))then
        Dh0g1subC=Dh0g1subC1(iqb1,iq1,iqb2,iq2,renscale2)
      endif
      return
      end
************************************************************************
*
      function Dt0g1subC(iq1,iqb2,iq2,iqb1,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)

      if((iq1.eq.1).and.(iqb2.eq.2).and.(iq2.eq.3).and.(iqb1.eq.4))then
        Dt0g1subC=Dt0g1subC1(iq1,iqb1,iq2,iqb2,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.2).and.(iq2.eq.4).and.(iqb1.eq.3))then
        Dt0g1subC=Dt0g1subC1(iq1,iqb1,iq2,iqb2,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.3).and.(iq2.eq.2).and.(iqb1.eq.4))then
        Dt0g1subC=Dt0g1subC2(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.3).and.(iq2.eq.4).and.(iqb1.eq.2))then
        Dt0g1subC=Dt0g1subC1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.4).and.(iq2.eq.2).and.(iqb1.eq.3))then
        Dt0g1subC=Dt0g1subC2(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.4).and.(iq2.eq.3).and.(iqb1.eq.2))then
        Dt0g1subC=Dt0g1subC1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.1).and.(iq2.eq.3).and.(iqb1.eq.4))then
        Dt0g1subC=Dt0g1subC1(iq1,iqb1,iq2,iqb2,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.1).and.(iq2.eq.4).and.(iqb1.eq.3))then
        Dt0g1subC=Dt0g1subC1(iq1,iqb1,iq2,iqb2,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.3).and.(iq2.eq.1).and.(iqb1.eq.4))then
        Dt0g1subC=Dt0g1subC2(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.3).and.(iq2.eq.4).and.(iqb1.eq.1))then
        Dt0g1subC=Dt0g1subC1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.4).and.(iq2.eq.1).and.(iqb1.eq.3))then
        Dt0g1subC=Dt0g1subC2(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.4).and.(iq2.eq.3).and.(iqb1.eq.1))then
        Dt0g1subC=Dt0g1subC1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.1).and.(iq2.eq.2).and.(iqb1.eq.4))then
        Dt0g1subC=Dt0g1subC1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.1).and.(iq2.eq.4).and.(iqb1.eq.2))then
        Dt0g1subC=Dt0g1subC2(iqb1,iq2,iqb2,iq1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.2).and.(iq2.eq.1).and.(iqb1.eq.4))then
        Dt0g1subC=Dt0g1subC1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.2).and.(iq2.eq.4).and.(iqb1.eq.1))then
        Dt0g1subC=Dt0g1subC2(iqb1,iq2,iqb2,iq1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.4).and.(iq2.eq.1).and.(iqb1.eq.2))then
        Dt0g1subC=Dt0g1subC1(iqb2,iq2,iqb1,iq1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.4).and.(iq2.eq.2).and.(iqb1.eq.1))then
        Dt0g1subC=Dt0g1subC1(iqb2,iq2,iqb1,iq1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.1).and.(iq2.eq.2).and.(iqb1.eq.3))then
        Dt0g1subC=Dt0g1subC1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.1).and.(iq2.eq.3).and.(iqb1.eq.2))then
        Dt0g1subC=Dt0g1subC2(iqb1,iq2,iqb2,iq1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.2).and.(iq2.eq.1).and.(iqb1.eq.3))then
        Dt0g1subC=Dt0g1subC1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.2).and.(iq2.eq.3).and.(iqb1.eq.1))then
        Dt0g1subC=Dt0g1subC2(iqb1,iq2,iqb2,iq1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.3).and.(iq2.eq.1).and.(iqb1.eq.2))then
        Dt0g1subC=Dt0g1subC1(iqb1,iq1,iqb2,iq2,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.3).and.(iq2.eq.2).and.(iqb1.eq.1))then
        Dt0g1subC=Dt0g1subC1(iqb1,iq1,iqb2,iq2,renscale2)
      endif
      return
      end


************************************************************************
*
      function D0g1subD(iq1,iqb2,iq2,iqb1,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)

      if((iq1.eq.1).and.(iqb2.eq.2).and.(iq2.eq.3).and.(iqb1.eq.4))then
        D0g1subD=D0g1subD1(iq1,iqb1,iq2,iqb2,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.2).and.(iq2.eq.4).and.(iqb1.eq.3))then
        D0g1subD=D0g1subD1(iq1,iqb1,iq2,iqb2,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.3).and.(iq2.eq.2).and.(iqb1.eq.4))then
        D0g1subD=D0g1subD2(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.3).and.(iq2.eq.4).and.(iqb1.eq.2))then
        D0g1subD=D0g1subD1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.4).and.(iq2.eq.2).and.(iqb1.eq.3))then
        D0g1subD=D0g1subD2(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.4).and.(iq2.eq.3).and.(iqb1.eq.2))then
        D0g1subD=D0g1subD1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.1).and.(iq2.eq.3).and.(iqb1.eq.4))then
        D0g1subD=D0g1subD1(iq1,iqb1,iq2,iqb2,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.1).and.(iq2.eq.4).and.(iqb1.eq.3))then
        D0g1subD=D0g1subD1(iq1,iqb1,iq2,iqb2,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.3).and.(iq2.eq.1).and.(iqb1.eq.4))then
        D0g1subD=D0g1subD2(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.3).and.(iq2.eq.4).and.(iqb1.eq.1))then
        D0g1subD=D0g1subD1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.4).and.(iq2.eq.1).and.(iqb1.eq.3))then
        D0g1subD=D0g1subD2(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.4).and.(iq2.eq.3).and.(iqb1.eq.1))then
        D0g1subD=D0g1subD1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.1).and.(iq2.eq.2).and.(iqb1.eq.4))then
        D0g1subD=D0g1subD1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.1).and.(iq2.eq.4).and.(iqb1.eq.2))then
        D0g1subD=D0g1subD2(iqb1,iq2,iqb2,iq1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.2).and.(iq2.eq.1).and.(iqb1.eq.4))then
        D0g1subD=D0g1subD1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.2).and.(iq2.eq.4).and.(iqb1.eq.1))then
        D0g1subD=D0g1subD2(iqb1,iq2,iqb2,iq1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.4).and.(iq2.eq.1).and.(iqb1.eq.2))then
        D0g1subD=D0g1subD1(iqb2,iq2,iqb1,iq1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.4).and.(iq2.eq.2).and.(iqb1.eq.1))then
        D0g1subD=D0g1subD1(iqb2,iq2,iqb1,iq1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.1).and.(iq2.eq.2).and.(iqb1.eq.3))then
        D0g1subD=D0g1subD1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.1).and.(iq2.eq.3).and.(iqb1.eq.2))then
        D0g1subD=D0g1subD2(iqb1,iq2,iqb2,iq1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.2).and.(iq2.eq.1).and.(iqb1.eq.3))then
        D0g1subD=D0g1subD1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.2).and.(iq2.eq.3).and.(iqb1.eq.1))then
        D0g1subD=D0g1subD2(iqb1,iq2,iqb2,iq1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.3).and.(iq2.eq.1).and.(iqb1.eq.2))then
        D0g1subD=D0g1subD1(iqb1,iq1,iqb2,iq2,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.3).and.(iq2.eq.2).and.(iqb1.eq.1))then
        D0g1subD=D0g1subD1(iqb1,iq1,iqb2,iq2,renscale2)
      endif
      return
      end
*
************************************************************************
*
      function Dh0g1subD(iq1,iqb2,iq2,iqb1,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)

      if((iq1.eq.1).and.(iqb2.eq.2).and.(iq2.eq.3).and.(iqb1.eq.4))then
        Dh0g1subD=Dh0g1subD1(iq1,iqb1,iq2,iqb2,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.2).and.(iq2.eq.4).and.(iqb1.eq.3))then
        Dh0g1subD=Dh0g1subD1(iq1,iqb1,iq2,iqb2,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.3).and.(iq2.eq.2).and.(iqb1.eq.4))then
        Dh0g1subD=Dh0g1subD2(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.3).and.(iq2.eq.4).and.(iqb1.eq.2))then
        Dh0g1subD=Dh0g1subD1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.4).and.(iq2.eq.2).and.(iqb1.eq.3))then
        Dh0g1subD=Dh0g1subD2(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.4).and.(iq2.eq.3).and.(iqb1.eq.2))then
        Dh0g1subD=Dh0g1subD1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.1).and.(iq2.eq.3).and.(iqb1.eq.4))then
        Dh0g1subD=Dh0g1subD1(iq1,iqb1,iq2,iqb2,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.1).and.(iq2.eq.4).and.(iqb1.eq.3))then
        Dh0g1subD=Dh0g1subD1(iq1,iqb1,iq2,iqb2,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.3).and.(iq2.eq.1).and.(iqb1.eq.4))then
        Dh0g1subD=Dh0g1subD2(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.3).and.(iq2.eq.4).and.(iqb1.eq.1))then
        Dh0g1subD=Dh0g1subD1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.4).and.(iq2.eq.1).and.(iqb1.eq.3))then
        Dh0g1subD=Dh0g1subD2(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.4).and.(iq2.eq.3).and.(iqb1.eq.1))then
        Dh0g1subD=Dh0g1subD1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.1).and.(iq2.eq.2).and.(iqb1.eq.4))then
        Dh0g1subD=Dh0g1subD1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.1).and.(iq2.eq.4).and.(iqb1.eq.2))then
        Dh0g1subD=Dh0g1subD2(iqb1,iq2,iqb2,iq1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.2).and.(iq2.eq.1).and.(iqb1.eq.4))then
        Dh0g1subD=Dh0g1subD1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.2).and.(iq2.eq.4).and.(iqb1.eq.1))then
        Dh0g1subD=Dh0g1subD2(iqb1,iq2,iqb2,iq1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.4).and.(iq2.eq.1).and.(iqb1.eq.2))then
        Dh0g1subD=Dh0g1subD1(iqb2,iq2,iqb1,iq1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.4).and.(iq2.eq.2).and.(iqb1.eq.1))then
        Dh0g1subD=Dh0g1subD1(iqb2,iq2,iqb1,iq1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.1).and.(iq2.eq.2).and.(iqb1.eq.3))then
        Dh0g1subD=Dh0g1subD1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.1).and.(iq2.eq.3).and.(iqb1.eq.2))then
        Dh0g1subD=Dh0g1subD2(iqb1,iq2,iqb2,iq1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.2).and.(iq2.eq.1).and.(iqb1.eq.3))then
        Dh0g1subD=Dh0g1subD1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.2).and.(iq2.eq.3).and.(iqb1.eq.1))then
        Dh0g1subD=Dh0g1subD2(iqb1,iq2,iqb2,iq1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.3).and.(iq2.eq.1).and.(iqb1.eq.2))then
        Dh0g1subD=Dh0g1subD1(iqb1,iq1,iqb2,iq2,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.3).and.(iq2.eq.2).and.(iqb1.eq.1))then
        Dh0g1subD=Dh0g1subD1(iqb1,iq1,iqb2,iq2,renscale2)
      endif
      return
      end
************************************************************************
*
      function Dt0g1subD(iq1,iqb2,iq2,iqb1,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)

      if((iq1.eq.1).and.(iqb2.eq.2).and.(iq2.eq.3).and.(iqb1.eq.4))then
        Dt0g1subD=Dt0g1subD1(iq1,iqb1,iq2,iqb2,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.2).and.(iq2.eq.4).and.(iqb1.eq.3))then
        Dt0g1subD=Dt0g1subD1(iq1,iqb1,iq2,iqb2,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.3).and.(iq2.eq.2).and.(iqb1.eq.4))then
        Dt0g1subD=Dt0g1subD2(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.3).and.(iq2.eq.4).and.(iqb1.eq.2))then
        Dt0g1subD=Dt0g1subD1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.4).and.(iq2.eq.2).and.(iqb1.eq.3))then
        Dt0g1subD=Dt0g1subD2(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.1).and.(iqb2.eq.4).and.(iq2.eq.3).and.(iqb1.eq.2))then
        Dt0g1subD=Dt0g1subD1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.1).and.(iq2.eq.3).and.(iqb1.eq.4))then
        Dt0g1subD=Dt0g1subD1(iq1,iqb1,iq2,iqb2,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.1).and.(iq2.eq.4).and.(iqb1.eq.3))then
        Dt0g1subD=Dt0g1subD1(iq1,iqb1,iq2,iqb2,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.3).and.(iq2.eq.1).and.(iqb1.eq.4))then
        Dt0g1subD=Dt0g1subD2(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.3).and.(iq2.eq.4).and.(iqb1.eq.1))then
        Dt0g1subD=Dt0g1subD1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.4).and.(iq2.eq.1).and.(iqb1.eq.3))then
        Dt0g1subD=Dt0g1subD2(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.2).and.(iqb2.eq.4).and.(iq2.eq.3).and.(iqb1.eq.1))then
        Dt0g1subD=Dt0g1subD1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.1).and.(iq2.eq.2).and.(iqb1.eq.4))then
        Dt0g1subD=Dt0g1subD1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.1).and.(iq2.eq.4).and.(iqb1.eq.2))then
        Dt0g1subD=Dt0g1subD2(iqb1,iq2,iqb2,iq1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.2).and.(iq2.eq.1).and.(iqb1.eq.4))then
        Dt0g1subD=Dt0g1subD1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.2).and.(iq2.eq.4).and.(iqb1.eq.1))then
        Dt0g1subD=Dt0g1subD2(iqb1,iq2,iqb2,iq1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.4).and.(iq2.eq.1).and.(iqb1.eq.2))then
        Dt0g1subD=Dt0g1subD1(iqb2,iq2,iqb1,iq1,renscale2)
      elseif((iq1.eq.3).and.(iqb2.eq.4).and.(iq2.eq.2).and.(iqb1.eq.1))then
        Dt0g1subD=Dt0g1subD1(iqb2,iq2,iqb1,iq1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.1).and.(iq2.eq.2).and.(iqb1.eq.3))then
        Dt0g1subD=Dt0g1subD1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.1).and.(iq2.eq.3).and.(iqb1.eq.2))then
        Dt0g1subD=Dt0g1subD2(iqb1,iq2,iqb2,iq1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.2).and.(iq2.eq.1).and.(iqb1.eq.3))then
        Dt0g1subD=Dt0g1subD1(iq1,iqb2,iq2,iqb1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.2).and.(iq2.eq.3).and.(iqb1.eq.1))then
        Dt0g1subD=Dt0g1subD2(iqb1,iq2,iqb2,iq1,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.3).and.(iq2.eq.1).and.(iqb1.eq.2))then
        Dt0g1subD=Dt0g1subD1(iqb1,iq1,iqb2,iq2,renscale2)
      elseif((iq1.eq.4).and.(iqb2.eq.3).and.(iq2.eq.2).and.(iqb1.eq.1))then
        Dt0g1subD=Dt0g1subD1(iqb1,iq1,iqb2,iq2,renscale2)
      endif
      return
      end

************************************************************************
*
      function D0g1subA1(iq1,iqb2,iq2,iqb1,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      parameter(zeta3=1.202056903159594285d0)
      common /order/ieorder
      s12 =s4(iq1,iqb1)
      s23 =s4(iq1,iqb2)
      s13 =s4(iq1,iq2)
      dlx   =log(-s23/s12)
      dlomx =log(-s13/s12)
      dls =log(s12/renscale2)
      if(ieorder.eq.-2)then
      D0g1subA1 =  - 2.D0 - 2.D0*s12**(-1)*s23 + 2.D0*s13*s23**(-1)

      elseif(ieorder.eq.-1)then
      D0g1subA1 =  + dls * ( 2.D0 + 2.D0*s12**(-1)*s23 - 2.D0*s13*
     &    s23**(-1) )
      D0g1subA1 = D0g1subA1 + dlx * ( 2.D0 + 2.D0*s12**(-1)*s23 - 2.D0*s13*
     &    s23**(-1) )
      D0g1subA1 = D0g1subA1 - 3.D0 - 3.D0*s12**(-1)*s23 + 3.D0*s13*s23**(-1)

      elseif(ieorder.eq.0)then
      D0g1subA1 =  + pi**2 * ( 1.D0/6.D0 + 1.D0/6.D0*s12**(-1)*s23 - 1.D0/
     &    6.D0*s13*s23**(-1) )
      D0g1subA1 = D0g1subA1 + dls * (  - 2.D0/3.D0 - 2.D0/3.D0*s12**(-1)*s23
     &     + 2.D0/3.D0*s13*s23**(-1) )
      D0g1subA1 = D0g1subA1 + dls**2 * (  - 1.D0 - s12**(-1)*s23 + s13*
     &    s23**(-1) )
      D0g1subA1 = D0g1subA1 + dlx * (  - s13*s23**(-1) )
      D0g1subA1 = D0g1subA1 + dlx*dls * (  - 2.D0 - 2.D0*s12**(-1)*s23 + 2.D0
     &    *s13*s23**(-1) )
      D0g1subA1 = D0g1subA1 + dlx**2 * (  - 1.D0/2.D0 + 1.D0/2.D0*s13*
     &    s23**(-1) )
      D0g1subA1 = D0g1subA1 + 13.D0/9.D0 + 13.D0/9.D0*s12**(-1)*s23 - 13.D0/9.
     &    D0*s13*s23**(-1)

      endif
      return
      end
************************************************************************
*
      function D0g1subA2(iq1,iqb2,iq2,iqb1,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      parameter(zeta3=1.202056903159594285d0)
      common /order/ieorder
      s12 =s4(iq1,iq2)
      s23 =s4(iq1,iqb2)
      s13 =s4(iq1,iqb1)
      dlx   =log(-s23/s12)
      dlomx =log(-s13/s12)
      dls =log(s12/renscale2)
      if(ieorder.eq.-2)then
      D0g1subA2 =  - 4.D0 - 2.D0*s13**(-1)*s23 - 2.D0*s13*s23**(-1)

      elseif(ieorder.eq.-1)then
      D0g1subA2 =  + dls * ( 4.D0 + 2.D0*s13**(-1)*s23 + 2.D0*s13*
     &    s23**(-1) )
      D0g1subA2 = D0g1subA2 + dlx * ( 4.D0 + 2.D0*s13**(-1)*s23 + 2.D0*s13*
     &    s23**(-1) )
      D0g1subA2 = D0g1subA2 - 6.D0 - 3.D0*s13**(-1)*s23 - 3.D0*s13*s23**(-1)

      elseif(ieorder.eq.0)then
      D0g1subA2 =  + pi**2 * ( 4.D0/3.D0 + 7.D0/6.D0*s13**(-1)*s23 + 2.D0/
     &    3.D0*s13*s23**(-1) )
      D0g1subA2 = D0g1subA2 + dls * (  - 4.D0/3.D0 - 2.D0/3.D0*s13**(-1)*s23
     &     - 2.D0/3.D0*s13*s23**(-1) )
      D0g1subA2 = D0g1subA2 + dls**2 * (  - 2.D0 - s13**(-1)*s23 - s13*
     &    s23**(-1) )
      D0g1subA2 = D0g1subA2 + dlomx * (  - 7.D0/3.D0 - 2.D0/3.D0*s13**(-1)*
     &    s23 - 5.D0/3.D0*s13*s23**(-1) )
      D0g1subA2 = D0g1subA2 + dlomx**2 * ( 1.D0 + s13**(-1)*s23 + 1.D0/2.D0*
     &    s13*s23**(-1) )
      D0g1subA2 = D0g1subA2 + dlx * ( 1.D0 + s13*s23**(-1) )
      D0g1subA2 = D0g1subA2 + dlx*dls * (  - 4.D0 - 2.D0*s13**(-1)*s23 - 2.D0
     &    *s13*s23**(-1) )
      D0g1subA2 = D0g1subA2 + dlx*dlomx * (  - 2.D0 - 2.D0*s13**(-1)*s23 - 
     &    s13*s23**(-1) )
      D0g1subA2 = D0g1subA2 + dlx**2 * (  - 1.D0 - 1.D0/2.D0*s13*s23**(-1) )
      D0g1subA2 = D0g1subA2 + 26.D0/9.D0 + 13.D0/9.D0*s13**(-1)*s23 + 13.D0/9.
     &    D0*s13*s23**(-1)

      endif
      return
      end
************************************************************************
*
      function Dh0g1subA1(iq1,iqb2,iq2,iqb1,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      parameter(zeta3=1.202056903159594285d0)
      common /order/ieorder
      s12 =s4(iq1,iqb1)
      s23 =s4(iq1,iqb2)
      s13 =s4(iq1,iq2)
      dlx   =log(-s23/s12)
      dlomx =log(-s13/s12)
      dls =log(s12/renscale2)
      if(ieorder.eq.-2)then
      Dh0g1subA1 =  0

      elseif(ieorder.eq.-1)then
      Dh0g1subA1 =  0

      elseif(ieorder.eq.0)then
      Dh0g1subA1 =  + dls * ( 2.D0/3.D0 + 2.D0/3.D0*s12**(-1)*s23 - 2.D0/3.
     &    D0*s13*s23**(-1) )
      Dh0g1subA1 = Dh0g1subA1 - 10.D0/9.D0 - 10.D0/9.D0*s12**(-1)*s23 + 10.D0/
     &    9.D0*s13*s23**(-1)

      endif
      return
      end
************************************************************************
*
      function Dh0g1subA2(iq1,iqb2,iq2,iqb1,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      parameter(zeta3=1.202056903159594285d0)
      common /order/ieorder
      s12 =s4(iq1,iq2)
      s23 =s4(iq1,iqb2)
      s13 =s4(iq1,iqb1)
      dlx   =log(-s23/s12)
      dlomx =log(-s13/s12)
      dls =log(s12/renscale2)
      if(ieorder.eq.-2)then
      Dh0g1subA2 =  0

      elseif(ieorder.eq.-1)then
      Dh0g1subA2 =  0

      elseif(ieorder.eq.0)then
      Dh0g1subA2 =  + dls * ( 4.D0/3.D0 + 2.D0/3.D0*s13**(-1)*s23 + 2.D0/3.
     &    D0*s13*s23**(-1) )
      Dh0g1subA2 = Dh0g1subA2 + dlomx * ( 4.D0/3.D0 + 2.D0/3.D0*s13**(-1)*s23
     &     + 2.D0/3.D0*s13*s23**(-1) )
      Dh0g1subA2 = Dh0g1subA2 - 20.D0/9.D0 - 10.D0/9.D0*s13**(-1)*s23 - 10.D0/
     &    9.D0*s13*s23**(-1)

      endif
      return
      end
************************************************************************
*
      function Dt0g1subA1(iq1,iqb2,iq2,iqb1,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      parameter(zeta3=1.202056903159594285d0)
      common /order/ieorder
      s12 =s4(iq1,iqb1)
      s23 =s4(iq1,iqb2)
      s13 =s4(iq1,iq2)
      dlx   =log(-s23/s12)
      dlomx =log(-s13/s12)
      dls =log(s12/renscale2)
      if(ieorder.eq.-2)then
      Dt0g1subA1 =  + 2.D0 + 2.D0*s12**(-1)*s23 - 2.D0*s13*s23**(-1)

      elseif(ieorder.eq.-1)then
      Dt0g1subA1 =  + dls * (  - 2.D0 - 2.D0*s12**(-1)*s23 + 2.D0*s13*
     &    s23**(-1) )
      Dt0g1subA1 = Dt0g1subA1 + dlomx * ( 4.D0 + 4.D0*s12**(-1)*s23 - 4.D0*
     &    s13*s23**(-1) )
      Dt0g1subA1 = Dt0g1subA1 + dlx * (  - 4.D0 - 4.D0*s12**(-1)*s23 + 4.D0*
     &    s13*s23**(-1) )
      Dt0g1subA1 = Dt0g1subA1 + 3.D0 + 3.D0*s12**(-1)*s23 - 3.D0*s13*
     &    s23**(-1)

      elseif(ieorder.eq.0)then
      Dt0g1subA1 =  + pi**2 * (  - 7.D0/6.D0 - 7.D0/6.D0*s12**(-1)*s23 + 7.
     &    D0/6.D0*s13*s23**(-1) )
      Dt0g1subA1 = Dt0g1subA1 + dls * (  - 3.D0 - 3.D0*s12**(-1)*s23 + 3.D0*
     &    s13*s23**(-1) )
      Dt0g1subA1 = Dt0g1subA1 + dls**2 * ( 1.D0 + s12**(-1)*s23 - s13*
     &    s23**(-1) )
      Dt0g1subA1 = Dt0g1subA1 + dlomx*dls * (  - 4.D0 - 4.D0*s12**(-1)*s23 + 
     &    4.D0*s13*s23**(-1) )
      Dt0g1subA1 = Dt0g1subA1 + dlx * ( 2.D0*s13*s23**(-1) )
      Dt0g1subA1 = Dt0g1subA1 + dlx*dls * ( 4.D0 + 4.D0*s12**(-1)*s23 - 4.D0*
     &    s13*s23**(-1) )
      Dt0g1subA1 = Dt0g1subA1 + dlx**2 * ( 1.D0 - s13*s23**(-1) )
      Dt0g1subA1 = Dt0g1subA1 + 8.D0 + 8.D0*s12**(-1)*s23 - 8.D0*s13*
     &    s23**(-1)

      endif
      return
      end
************************************************************************
*
      function Dt0g1subA2(iq1,iqb2,iq2,iqb1,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      parameter(zeta3=1.202056903159594285d0)
      common /order/ieorder
      s12 =s4(iq1,iq2)
      s23 =s4(iq1,iqb2)
      s13 =s4(iq1,iqb1)
      dlx   =log(-s23/s12)
      dlomx =log(-s13/s12)
      dls =log(s12/renscale2)
      if(ieorder.eq.-2)then
      Dt0g1subA2 =  + 4.D0 + 2.D0*s13**(-1)*s23 + 2.D0*s13*s23**(-1)

      elseif(ieorder.eq.-1)then
      Dt0g1subA2 =  + dls * (  - 4.D0 - 2.D0*s13**(-1)*s23 - 2.D0*s13*
     &    s23**(-1) )
      Dt0g1subA2 = Dt0g1subA2 + dlomx * (  - 4.D0 - 2.D0*s13**(-1)*s23 - 2.D0
     &    *s13*s23**(-1) )
      Dt0g1subA2 = Dt0g1subA2 + dlx * (  - 8.D0 - 4.D0*s13**(-1)*s23 - 4.D0*
     &    s13*s23**(-1) )
      Dt0g1subA2 = Dt0g1subA2 + 6.D0 + 3.D0*s13**(-1)*s23 + 3.D0*s13*
     &    s23**(-1)

      elseif(ieorder.eq.0)then
      Dt0g1subA2 =  + pi**2 * ( 5.D0/3.D0 - 1.D0/6.D0*s13**(-1)*s23 + 5.D0/
     &    6.D0*s13*s23**(-1) )
      Dt0g1subA2 = Dt0g1subA2 + dls * (  - 6.D0 - 3.D0*s13**(-1)*s23 - 3.D0*
     &    s13*s23**(-1) )
      Dt0g1subA2 = Dt0g1subA2 + dls**2 * ( 2.D0 + s13**(-1)*s23 + s13*
     &    s23**(-1) )
      Dt0g1subA2 = Dt0g1subA2 + dlomx * (  - 4.D0 - 3.D0*s13**(-1)*s23 - s13*
     &    s23**(-1) )
      Dt0g1subA2 = Dt0g1subA2 + dlomx*dls * ( 4.D0 + 2.D0*s13**(-1)*s23 + 2.D0
     &    *s13*s23**(-1) )
      Dt0g1subA2 = Dt0g1subA2 + dlomx**2 * ( 4.D0 + s13**(-1)*s23 + 2.D0*s13*
     &    s23**(-1) )
      Dt0g1subA2 = Dt0g1subA2 + dlx * (  - 2.D0 - 2.D0*s13*s23**(-1) )
      Dt0g1subA2 = Dt0g1subA2 + dlx*dls * ( 8.D0 + 4.D0*s13**(-1)*s23 + 4.D0*
     &    s13*s23**(-1) )
      Dt0g1subA2 = Dt0g1subA2 + dlx*dlomx * ( 4.D0 + 4.D0*s13**(-1)*s23 + 2.D0
     &    *s13*s23**(-1) )
      Dt0g1subA2 = Dt0g1subA2 + dlx**2 * ( 2.D0 + s13*s23**(-1) )
      Dt0g1subA2 = Dt0g1subA2 + 16.D0 + 8.D0*s13**(-1)*s23 + 8.D0*s13*
     &    s23**(-1)

      endif
      return
      end
************************************************************************
*
      function D0g1subB1(iq1,iqb2,iq2,iqb1,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      parameter(zeta3=1.202056903159594285d0)
      common /order/ieorder
      s12 =s4(iq1,iqb1)
      s23 =s4(iq1,iqb2)
      s13 =s4(iq1,iq2)
      dlx   =log(-s23/s12)
      dlomx =log(-s13/s12)
      dls =log(s12/renscale2)
      if(ieorder.eq.-2)then
      D0g1subB1 =  + 2.D0 + 2.D0*s12**(-1)*s23 - 2.D0*s13*s23**(-1)

      elseif(ieorder.eq.-1)then
      D0g1subB1 =  + dls * (  - 2.D0 - 2.D0*s12**(-1)*s23 + 2.D0*s13*
     &    s23**(-1) )
      D0g1subB1 = D0g1subB1 + dlomx * (  - 2.D0 - 2.D0*s12**(-1)*s23 + 2.D0*
     &    s13*s23**(-1) )
      D0g1subB1 = D0g1subB1 + 3.D0 + 3.D0*s12**(-1)*s23 - 3.D0*s13*s23**(-1)

      elseif(ieorder.eq.0)then
      D0g1subB1 =  + pi**2 * (  - 1.D0/6.D0 - 1.D0/6.D0*s12**(-1)*s23 + 1.D
     &    0/6.D0*s13*s23**(-1) )
      D0g1subB1 = D0g1subB1 + dls * ( 2.D0/3.D0 + 2.D0/3.D0*s12**(-1)*s23 - 2.
     &    D0/3.D0*s13*s23**(-1) )
      D0g1subB1 = D0g1subB1 + dls**2 * ( 1.D0 + s12**(-1)*s23 - s13*s23**(-1)
     &     )
      D0g1subB1 = D0g1subB1 + dlomx*dls * ( 2.D0 + 2.D0*s12**(-1)*s23 - 2.D0*
     &    s13*s23**(-1) )
      D0g1subB1 = D0g1subB1 - 13.D0/9.D0 - 13.D0/9.D0*s12**(-1)*s23 + 13.D0/9.
     &    D0*s13*s23**(-1)

      endif
      return
      end
************************************************************************
*
      function D0g1subB2(iq1,iqb2,iq2,iqb1,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      parameter(zeta3=1.202056903159594285d0)
      common /order/ieorder
      s12 =s4(iq1,iq2)
      s23 =s4(iq1,iqb2)
      s13 =s4(iq1,iqb1)
      dlx   =log(-s23/s12)
      dlomx =log(-s13/s12)
      dls =log(s12/renscale2)
      if(ieorder.eq.-2)then
      D0g1subB2 =  + 4.D0 + 2.D0*s13**(-1)*s23 + 2.D0*s13*s23**(-1)

      elseif(ieorder.eq.-1)then
      D0g1subB2 =  + dls * (  - 4.D0 - 2.D0*s13**(-1)*s23 - 2.D0*s13*
     &    s23**(-1) )
      D0g1subB2 = D0g1subB2 + 6.D0 + 3.D0*s13**(-1)*s23 + 3.D0*s13*s23**(-1)

      elseif(ieorder.eq.0)then
      D0g1subB2 =  + pi**2 * (  - 7.D0/3.D0 - 7.D0/6.D0*s13**(-1)*s23 - 7.D
     &    0/6.D0*s13*s23**(-1) )
      D0g1subB2 = D0g1subB2 + dls * ( 4.D0/3.D0 + 2.D0/3.D0*s13**(-1)*s23 + 2.
     &    D0/3.D0*s13*s23**(-1) )
      D0g1subB2 = D0g1subB2 + dls**2 * ( 2.D0 + s13**(-1)*s23 + s13*s23**(-1)
     &     )
      D0g1subB2 = D0g1subB2 + dlomx * ( 4.D0/3.D0 + 2.D0/3.D0*s13**(-1)*s23
     &     + 2.D0/3.D0*s13*s23**(-1) )
      D0g1subB2 = D0g1subB2 + dlomx**2 * (  - 2.D0 - s13**(-1)*s23 - s13*
     &    s23**(-1) )
      D0g1subB2 = D0g1subB2 - 26.D0/9.D0 - 13.D0/9.D0*s13**(-1)*s23 - 13.D0/9.
     &    D0*s13*s23**(-1)

      endif
      return
      end
************************************************************************
*
      function Dh0g1subB1(iq1,iqb2,iq2,iqb1,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      parameter(zeta3=1.202056903159594285d0)
      common /order/ieorder
      s12 =s4(iq1,iqb1)
      s23 =s4(iq1,iqb2)
      s13 =s4(iq1,iq2)
      dlx   =log(-s23/s12)
      dlomx =log(-s13/s12)
      dls =log(s12/renscale2)
      if(ieorder.eq.-2)then
      Dh0g1subB1 =  0

      elseif(ieorder.eq.-1)then
      Dh0g1subB1 =  0

      elseif(ieorder.eq.0)then
      Dh0g1subB1 =  + dls * (  - 2.D0/3.D0 - 2.D0/3.D0*s12**(-1)*s23 + 2.D0
     &    /3.D0*s13*s23**(-1) )
      Dh0g1subB1 = Dh0g1subB1 + 10.D0/9.D0 + 10.D0/9.D0*s12**(-1)*s23 - 10.D0/
     &    9.D0*s13*s23**(-1)

      endif
      return
      end
************************************************************************
*
      function Dh0g1subB2(iq1,iqb2,iq2,iqb1,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      parameter(zeta3=1.202056903159594285d0)
      common /order/ieorder
      s12 =s4(iq1,iq2)
      s23 =s4(iq1,iqb2)
      s13 =s4(iq1,iqb1)
      dlx   =log(-s23/s12)
      dlomx =log(-s13/s12)
      dls =log(s12/renscale2)
      if(ieorder.eq.-2)then
      Dh0g1subB2 =  0

      elseif(ieorder.eq.-1)then
      Dh0g1subB2 =  0

      elseif(ieorder.eq.0)then
      Dh0g1subB2 =  + dls * (  - 4.D0/3.D0 - 2.D0/3.D0*s13**(-1)*s23 - 2.D0
     &    /3.D0*s13*s23**(-1) )
      Dh0g1subB2 = Dh0g1subB2 + dlomx * (  - 4.D0/3.D0 - 2.D0/3.D0*s13**(-1)*
     &    s23 - 2.D0/3.D0*s13*s23**(-1) )
      Dh0g1subB2 = Dh0g1subB2 + 20.D0/9.D0 + 10.D0/9.D0*s13**(-1)*s23 + 10.D0/
     &    9.D0*s13*s23**(-1)

      endif
      return
      end
************************************************************************
*
      function Dt0g1subB1(iq1,iqb2,iq2,iqb1,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      parameter(zeta3=1.202056903159594285d0)
      common /order/ieorder
      s12 =s4(iq1,iqb1)
      s23 =s4(iq1,iqb2)
      s13 =s4(iq1,iq2)
      dlx   =log(-s23/s12)
      dlomx =log(-s13/s12)
      dls =log(s12/renscale2)
      if(ieorder.eq.-2)then
      Dt0g1subB1 =  - 2.D0 - 2.D0*s12**(-1)*s23 + 2.D0*s13*s23**(-1)

      elseif(ieorder.eq.-1)then
      Dt0g1subB1 =  + dls * ( 2.D0 + 2.D0*s12**(-1)*s23 - 2.D0*s13*
     &    s23**(-1) )
      Dt0g1subB1 = Dt0g1subB1 + dlomx * (  - 2.D0 - 2.D0*s12**(-1)*s23 + 2.D0
     &    *s13*s23**(-1) )
      Dt0g1subB1 = Dt0g1subB1 + dlx * ( 2.D0 + 2.D0*s12**(-1)*s23 - 2.D0*s13*
     &    s23**(-1) )
      Dt0g1subB1 = Dt0g1subB1 - 3.D0 - 3.D0*s12**(-1)*s23 + 3.D0*s13*
     &    s23**(-1)

      elseif(ieorder.eq.0)then
      Dt0g1subB1 =  + pi**2 * ( 7.D0/6.D0 + 7.D0/6.D0*s12**(-1)*s23 - 7.D0/
     &    6.D0*s13*s23**(-1) )
      Dt0g1subB1 = Dt0g1subB1 + dls * ( 3.D0 + 3.D0*s12**(-1)*s23 - 3.D0*s13*
     &    s23**(-1) )
      Dt0g1subB1 = Dt0g1subB1 + dls**2 * (  - 1.D0 - s12**(-1)*s23 + s13*
     &    s23**(-1) )
      Dt0g1subB1 = Dt0g1subB1 + dlomx*dls * ( 2.D0 + 2.D0*s12**(-1)*s23 - 2.D0
     &    *s13*s23**(-1) )
      Dt0g1subB1 = Dt0g1subB1 + dlx * (  - s13*s23**(-1) )
      Dt0g1subB1 = Dt0g1subB1 + dlx*dls * (  - 2.D0 - 2.D0*s12**(-1)*s23 + 2.D
     &    0*s13*s23**(-1) )
      Dt0g1subB1 = Dt0g1subB1 + dlx**2 * (  - 1.D0/2.D0 + 1.D0/2.D0*s13*
     &    s23**(-1) )
      Dt0g1subB1 = Dt0g1subB1 - 8.D0 - 8.D0*s12**(-1)*s23 + 8.D0*s13*
     &    s23**(-1)

      endif
      return
      end
************************************************************************
*
      function Dt0g1subB2(iq1,iqb2,iq2,iqb1,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      parameter(zeta3=1.202056903159594285d0)
      common /order/ieorder
      s12 =s4(iq1,iq2)
      s23 =s4(iq1,iqb2)
      s13 =s4(iq1,iqb1)
      dlx   =log(-s23/s12)
      dlomx =log(-s13/s12)
      dls =log(s12/renscale2)
      if(ieorder.eq.-2)then
      Dt0g1subB2 =  - 4.D0 - 2.D0*s13**(-1)*s23 - 2.D0*s13*s23**(-1)

      elseif(ieorder.eq.-1)then
      Dt0g1subB2 =  + dls * ( 4.D0 + 2.D0*s13**(-1)*s23 + 2.D0*s13*
     &    s23**(-1) )
      Dt0g1subB2 = Dt0g1subB2 + dlomx * ( 4.D0 + 2.D0*s13**(-1)*s23 + 2.D0*
     &    s13*s23**(-1) )
      Dt0g1subB2 = Dt0g1subB2 + dlx * ( 4.D0 + 2.D0*s13**(-1)*s23 + 2.D0*s13*
     &    s23**(-1) )
      Dt0g1subB2 = Dt0g1subB2 - 6.D0 - 3.D0*s13**(-1)*s23 - 3.D0*s13*
     &    s23**(-1)

      elseif(ieorder.eq.0)then
      Dt0g1subB2 =  + pi**2 * (  - 2.D0/3.D0 + 1.D0/6.D0*s13**(-1)*s23 - 1.
     &    D0/3.D0*s13*s23**(-1) )
      Dt0g1subB2 = Dt0g1subB2 + dls * ( 6.D0 + 3.D0*s13**(-1)*s23 + 3.D0*s13*
     &    s23**(-1) )
      Dt0g1subB2 = Dt0g1subB2 + dls**2 * (  - 2.D0 - s13**(-1)*s23 - s13*
     &    s23**(-1) )
      Dt0g1subB2 = Dt0g1subB2 + dlomx * ( 5.D0 + 3.D0*s13**(-1)*s23 + 2.D0*
     &    s13*s23**(-1) )
      Dt0g1subB2 = Dt0g1subB2 + dlomx*dls * (  - 4.D0 - 2.D0*s13**(-1)*s23 - 
     &    2.D0*s13*s23**(-1) )
      Dt0g1subB2 = Dt0g1subB2 + dlomx**2 * (  - 3.D0 - s13**(-1)*s23 - 3.D0/2.
     &    D0*s13*s23**(-1) )
      Dt0g1subB2 = Dt0g1subB2 + dlx * ( 1.D0 + s13*s23**(-1) )
      Dt0g1subB2 = Dt0g1subB2 + dlx*dls * (  - 4.D0 - 2.D0*s13**(-1)*s23 - 2.D
     &    0*s13*s23**(-1) )
      Dt0g1subB2 = Dt0g1subB2 + dlx*dlomx * (  - 2.D0 - 2.D0*s13**(-1)*s23 - 
     &    s13*s23**(-1) )
      Dt0g1subB2 = Dt0g1subB2 + dlx**2 * (  - 1.D0 - 1.D0/2.D0*s13*s23**(-1)
     &     )
      Dt0g1subB2 = Dt0g1subB2 - 16.D0 - 8.D0*s13**(-1)*s23 - 8.D0*s13*
     &    s23**(-1)

      endif
      return
      end
************************************************************************
*
      function D0g1subC1(iq1,iqb2,iq2,iqb1,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      parameter(zeta3=1.202056903159594285d0)
      common /order/ieorder
      s12 =s4(iq1,iqb1)
      s23 =s4(iq1,iqb2)
      s13 =s4(iq1,iq2)
      dlx   =log(-s23/s12)
      dlomx =log(-s13/s12)
      dls =log(s12/renscale2)
      if(ieorder.eq.-2)then
      D0g1subC1 =  - 2.D0 - 2.D0*s12**(-1)*s23 + 2.D0*s13*s23**(-1)

      elseif(ieorder.eq.-1)then
      D0g1subC1 =  + dls * ( 2.D0 + 2.D0*s12**(-1)*s23 - 2.D0*s13*
     &    s23**(-1) )
      D0g1subC1 = D0g1subC1 - 3.D0 - 3.D0*s12**(-1)*s23 + 3.D0*s13*s23**(-1)

      elseif(ieorder.eq.0)then
      D0g1subC1 =  + pi**2 * ( 7.D0/6.D0 + 7.D0/6.D0*s12**(-1)*s23 - 7.D0/
     &    6.D0*s13*s23**(-1) )
      D0g1subC1 = D0g1subC1 + dls * (  - 2.D0/3.D0 - 2.D0/3.D0*s12**(-1)*s23
     &     + 2.D0/3.D0*s13*s23**(-1) )
      D0g1subC1 = D0g1subC1 + dls**2 * (  - 1.D0 - s12**(-1)*s23 + s13*
     &    s23**(-1) )
      D0g1subC1 = D0g1subC1 + dlx * (  - 5.D0/3.D0 - 5.D0/3.D0*s12**(-1)*s23
     &     + 2.D0/3.D0*s13*s23**(-1) )
      D0g1subC1 = D0g1subC1 + dlx**2 * ( 1.D0/2.D0*s12**(-1)*s23 - s13*
     &    s23**(-1) )
      D0g1subC1 = D0g1subC1 + 13.D0/9.D0 + 13.D0/9.D0*s12**(-1)*s23 - 13.D0/9.
     &    D0*s13*s23**(-1)

      endif
      return
      end
************************************************************************
*
      function D0g1subC2(iq1,iqb2,iq2,iqb1,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      parameter(zeta3=1.202056903159594285d0)
      common /order/ieorder
      s12 =s4(iq1,iq2)
      s23 =s4(iq1,iqb2)
      s13 =s4(iq1,iqb1)
      dlx   =log(-s23/s12)
      dlomx =log(-s13/s12)
      dls =log(s12/renscale2)
      if(ieorder.eq.-2)then
      D0g1subC2 =  - 4.D0 - 2.D0*s13**(-1)*s23 - 2.D0*s13*s23**(-1)

      elseif(ieorder.eq.-1)then
      D0g1subC2 =  + dls * ( 4.D0 + 2.D0*s13**(-1)*s23 + 2.D0*s13*
     &    s23**(-1) )
      D0g1subC2 = D0g1subC2 + dlomx * ( 4.D0 + 2.D0*s13**(-1)*s23 + 2.D0*s13*
     &    s23**(-1) )
      D0g1subC2 = D0g1subC2 - 6.D0 - 3.D0*s13**(-1)*s23 - 3.D0*s13*s23**(-1)

      elseif(ieorder.eq.0)then
      D0g1subC2 =  + pi**2 * ( 4.D0/3.D0 + 2.D0/3.D0*s13**(-1)*s23 + 7.D0/
     &    6.D0*s13*s23**(-1) )
      D0g1subC2 = D0g1subC2 + dls * (  - 4.D0/3.D0 - 2.D0/3.D0*s13**(-1)*s23
     &     - 2.D0/3.D0*s13*s23**(-1) )
      D0g1subC2 = D0g1subC2 + dls**2 * (  - 2.D0 - s13**(-1)*s23 - s13*
     &    s23**(-1) )
      D0g1subC2 = D0g1subC2 + dlomx * ( 1.D0 + s13**(-1)*s23 )
      D0g1subC2 = D0g1subC2 + dlomx*dls * (  - 4.D0 - 2.D0*s13**(-1)*s23 - 2.D
     &    0*s13*s23**(-1) )
      D0g1subC2 = D0g1subC2 + dlomx**2 * (  - 1.D0 - 1.D0/2.D0*s13**(-1)*s23
     &     )
      D0g1subC2 = D0g1subC2 + dlx * (  - 7.D0/3.D0 - 5.D0/3.D0*s13**(-1)*s23
     &     - 2.D0/3.D0*s13*s23**(-1) )
      D0g1subC2 = D0g1subC2 + dlx*dlomx * (  - 2.D0 - s13**(-1)*s23 - 2.D0*
     &    s13*s23**(-1) )
      D0g1subC2 = D0g1subC2 + dlx**2 * ( 1.D0 + 1.D0/2.D0*s13**(-1)*s23 + s13
     &    *s23**(-1) )
      D0g1subC2 = D0g1subC2 + 26.D0/9.D0 + 13.D0/9.D0*s13**(-1)*s23 + 13.D0/9.
     &    D0*s13*s23**(-1)

      endif
      return
      end
************************************************************************
*
      function Dh0g1subC1(iq1,iqb2,iq2,iqb1,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      parameter(zeta3=1.202056903159594285d0)
      common /order/ieorder
      s12 =s4(iq1,iqb1)
      s23 =s4(iq1,iqb2)
      s13 =s4(iq1,iq2)
      dlx   =log(-s23/s12)
      dlomx =log(-s13/s12)
      dls =log(s12/renscale2)
      if(ieorder.eq.-2)then
      Dh0g1subC1 =  0

      elseif(ieorder.eq.-1)then
      Dh0g1subC1 =  0

      elseif(ieorder.eq.0)then
      Dh0g1subC1 =  + dls * ( 2.D0/3.D0 + 2.D0/3.D0*s12**(-1)*s23 - 2.D0/3.
     &    D0*s13*s23**(-1) )
      Dh0g1subC1 = Dh0g1subC1 + dlx * ( 2.D0/3.D0 + 2.D0/3.D0*s12**(-1)*s23
     &     - 2.D0/3.D0*s13*s23**(-1) )
      Dh0g1subC1 = Dh0g1subC1 - 10.D0/9.D0 - 10.D0/9.D0*s12**(-1)*s23 + 10.D0/
     &    9.D0*s13*s23**(-1)

      endif
      return
      end
************************************************************************
*
      function Dh0g1subC2(iq1,iqb2,iq2,iqb1,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      parameter(zeta3=1.202056903159594285d0)
      common /order/ieorder
      s12 =s4(iq1,iq2)
      s23 =s4(iq1,iqb2)
      s13 =s4(iq1,iqb1)
      dlx   =log(-s23/s12)
      dlomx =log(-s13/s12)
      dls =log(s12/renscale2)
      if(ieorder.eq.-2)then
      Dh0g1subC2 =  0

      elseif(ieorder.eq.-1)then
      Dh0g1subC2 =  0

      elseif(ieorder.eq.0)then
      Dh0g1subC2 =  + dls * ( 4.D0/3.D0 + 2.D0/3.D0*s13**(-1)*s23 + 2.D0/3.
     &    D0*s13*s23**(-1) )
      Dh0g1subC2 = Dh0g1subC2 + dlx * ( 4.D0/3.D0 + 2.D0/3.D0*s13**(-1)*s23
     &     + 2.D0/3.D0*s13*s23**(-1) )
      Dh0g1subC2 = Dh0g1subC2 - 20.D0/9.D0 - 10.D0/9.D0*s13**(-1)*s23 - 10.D0/
     &    9.D0*s13*s23**(-1)

      endif
      return
      end
************************************************************************
*
      function Dt0g1subC1(iq1,iqb2,iq2,iqb1,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      parameter(zeta3=1.202056903159594285d0)
      common /order/ieorder
      s12 =s4(iq1,iqb1)
      s23 =s4(iq1,iqb2)
      s13 =s4(iq1,iq2)
      dlx   =log(-s23/s12)
      dlomx =log(-s13/s12)
      dls =log(s12/renscale2)
      if(ieorder.eq.-2)then
      Dt0g1subC1 =  + 2.D0 + 2.D0*s12**(-1)*s23 - 2.D0*s13*s23**(-1)

      elseif(ieorder.eq.-1)then
      Dt0g1subC1 =  + dls * (  - 2.D0 - 2.D0*s12**(-1)*s23 + 2.D0*s13*
     &    s23**(-1) )
      Dt0g1subC1 = Dt0g1subC1 + dlomx * ( 4.D0 + 4.D0*s12**(-1)*s23 - 4.D0*
     &    s13*s23**(-1) )
      Dt0g1subC1 = Dt0g1subC1 + dlx * (  - 2.D0 - 2.D0*s12**(-1)*s23 + 2.D0*
     &    s13*s23**(-1) )
      Dt0g1subC1 = Dt0g1subC1 + 3.D0 + 3.D0*s12**(-1)*s23 - 3.D0*s13*
     &    s23**(-1)

      elseif(ieorder.eq.0)then
      Dt0g1subC1 =  + pi**2 * (  - 1.D0/6.D0 - 1.D0/6.D0*s12**(-1)*s23 + 1.
     &    D0/6.D0*s13*s23**(-1) )
      Dt0g1subC1 = Dt0g1subC1 + dls * (  - 3.D0 - 3.D0*s12**(-1)*s23 + 3.D0*
     &    s13*s23**(-1) )
      Dt0g1subC1 = Dt0g1subC1 + dls**2 * ( 1.D0 + s12**(-1)*s23 - s13*
     &    s23**(-1) )
      Dt0g1subC1 = Dt0g1subC1 + dlomx*dls * (  - 4.D0 - 4.D0*s12**(-1)*s23 + 
     &    4.D0*s13*s23**(-1) )
      Dt0g1subC1 = Dt0g1subC1 + dlx * (  - 1.D0 - s12**(-1)*s23 + 3.D0*s13*
     &    s23**(-1) )
      Dt0g1subC1 = Dt0g1subC1 + dlx*dls * ( 2.D0 + 2.D0*s12**(-1)*s23 - 2.D0*
     &    s13*s23**(-1) )
      Dt0g1subC1 = Dt0g1subC1 + dlx*dlomx * (  - 4.D0 - 4.D0*s12**(-1)*s23 + 
     &    4.D0*s13*s23**(-1) )
      Dt0g1subC1 = Dt0g1subC1 + dlx**2 * ( 3.D0 + 2.D0*s12**(-1)*s23 - s13*
     &    s23**(-1) )
      Dt0g1subC1 = Dt0g1subC1 + 8.D0 + 8.D0*s12**(-1)*s23 - 8.D0*s13*
     &    s23**(-1)

      endif
      return
      end
************************************************************************
*
      function Dt0g1subC2(iq1,iqb2,iq2,iqb1,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      parameter(zeta3=1.202056903159594285d0)
      common /order/ieorder
      s12 =s4(iq1,iq2)
      s23 =s4(iq1,iqb2)
      s13 =s4(iq1,iqb1)
      dlx   =log(-s23/s12)
      dlomx =log(-s13/s12)
      dls =log(s12/renscale2)
      if(ieorder.eq.-2)then
      Dt0g1subC2 =  + 4.D0 + 2.D0*s13**(-1)*s23 + 2.D0*s13*s23**(-1)

      elseif(ieorder.eq.-1)then
      Dt0g1subC2 =  + dls * (  - 4.D0 - 2.D0*s13**(-1)*s23 - 2.D0*s13*
     &    s23**(-1) )
      Dt0g1subC2 = Dt0g1subC2 + dlomx * (  - 8.D0 - 4.D0*s13**(-1)*s23 - 4.D0
     &    *s13*s23**(-1) )
      Dt0g1subC2 = Dt0g1subC2 + dlx * (  - 4.D0 - 2.D0*s13**(-1)*s23 - 2.D0*
     &    s13*s23**(-1) )
      Dt0g1subC2 = Dt0g1subC2 + 6.D0 + 3.D0*s13**(-1)*s23 + 3.D0*s13*
     &    s23**(-1)

      elseif(ieorder.eq.0)then
      Dt0g1subC2 =  + pi**2 * ( 5.D0/3.D0 + 5.D0/6.D0*s13**(-1)*s23 - 1.D0/
     &    6.D0*s13*s23**(-1) )
      Dt0g1subC2 = Dt0g1subC2 + dls * (  - 6.D0 - 3.D0*s13**(-1)*s23 - 3.D0*
     &    s13*s23**(-1) )
      Dt0g1subC2 = Dt0g1subC2 + dls**2 * ( 2.D0 + s13**(-1)*s23 + s13*
     &    s23**(-1) )
      Dt0g1subC2 = Dt0g1subC2 + dlomx * (  - 2.D0 - 2.D0*s13**(-1)*s23 )
      Dt0g1subC2 = Dt0g1subC2 + dlomx*dls * ( 8.D0 + 4.D0*s13**(-1)*s23 + 4.D0
     &    *s13*s23**(-1) )
      Dt0g1subC2 = Dt0g1subC2 + dlomx**2 * ( 2.D0 + s13**(-1)*s23 )
      Dt0g1subC2 = Dt0g1subC2 + dlx * (  - 4.D0 - s13**(-1)*s23 - 3.D0*s13*
     &    s23**(-1) )
      Dt0g1subC2 = Dt0g1subC2 + dlx*dls * ( 4.D0 + 2.D0*s13**(-1)*s23 + 2.D0*
     &    s13*s23**(-1) )
      Dt0g1subC2 = Dt0g1subC2 + dlx*dlomx * ( 4.D0 + 2.D0*s13**(-1)*s23 + 4.D0
     &    *s13*s23**(-1) )
      Dt0g1subC2 = Dt0g1subC2 + dlx**2 * ( 4.D0 + 2.D0*s13**(-1)*s23 + s13*
     &    s23**(-1) )
      Dt0g1subC2 = Dt0g1subC2 + 16.D0 + 8.D0*s13**(-1)*s23 + 8.D0*s13*
     &    s23**(-1)

      endif
      return
      end
************************************************************************
*
      function D0g1subD1(iq1,iqb2,iq2,iqb1,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      parameter(zeta3=1.202056903159594285d0)
      common /order/ieorder
      s12 =s4(iq1,iqb1)
      s23 =s4(iq1,iqb2)
      s13 =s4(iq1,iq2)
      dlx   =log(-s23/s12)
      dlomx =log(-s13/s12)
      dls =log(s12/renscale2)
      if(ieorder.eq.-2)then
      D0g1subD1 =  + 2.D0 + 2.D0*s12**(-1)*s23 - 2.D0*s13*s23**(-1)

      elseif(ieorder.eq.-1)then
      D0g1subD1 =  + dls * (  - 2.D0 - 2.D0*s12**(-1)*s23 + 2.D0*s13*
     &    s23**(-1) )
      D0g1subD1 = D0g1subD1 + dlomx * (  - 2.D0 - 2.D0*s12**(-1)*s23 + 2.D0*
     &    s13*s23**(-1) )
      D0g1subD1 = D0g1subD1 + 3.D0 + 3.D0*s12**(-1)*s23 - 3.D0*s13*s23**(-1)

      elseif(ieorder.eq.0)then
      D0g1subD1 =  + pi**2 * (  - 7.D0/6.D0 - 7.D0/6.D0*s12**(-1)*s23 + 7.D
     &    0/6.D0*s13*s23**(-1) )
      D0g1subD1 = D0g1subD1 + dls * ( 2.D0/3.D0 + 2.D0/3.D0*s12**(-1)*s23 - 2.
     &    D0/3.D0*s13*s23**(-1) )
      D0g1subD1 = D0g1subD1 + dls**2 * ( 1.D0 + s12**(-1)*s23 - s13*s23**(-1)
     &     )
      D0g1subD1 = D0g1subD1 + dlomx*dls * ( 2.D0 + 2.D0*s12**(-1)*s23 - 2.D0*
     &    s13*s23**(-1) )
      D0g1subD1 = D0g1subD1 + dlx * ( 2.D0/3.D0 + 2.D0/3.D0*s12**(-1)*s23 - 2.
     &    D0/3.D0*s13*s23**(-1) )
      D0g1subD1 = D0g1subD1 + dlx*dlomx * ( 2.D0 + 2.D0*s12**(-1)*s23 - 2.D0*
     &    s13*s23**(-1) )
      D0g1subD1 = D0g1subD1 + dlx**2 * (  - 1.D0 - s12**(-1)*s23 + s13*
     &    s23**(-1) )
      D0g1subD1 = D0g1subD1 - 13.D0/9.D0 - 13.D0/9.D0*s12**(-1)*s23 + 13.D0/9.
     &    D0*s13*s23**(-1)

      endif
      return
      end
************************************************************************
*
      function D0g1subD2(iq1,iqb2,iq2,iqb1,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      parameter(zeta3=1.202056903159594285d0)
      common /order/ieorder
      s12 =s4(iq1,iq2)
      s23 =s4(iq1,iqb2)
      s13 =s4(iq1,iqb1)
      dlx   =log(-s23/s12)
      dlomx =log(-s13/s12)
      dls =log(s12/renscale2)
      if(ieorder.eq.-2)then
      D0g1subD2 =  + 4.D0 + 2.D0*s13**(-1)*s23 + 2.D0*s13*s23**(-1)

      elseif(ieorder.eq.-1)then
      D0g1subD2 =  + dls * (  - 4.D0 - 2.D0*s13**(-1)*s23 - 2.D0*s13*
     &    s23**(-1) )
      D0g1subD2 = D0g1subD2 + 6.D0 + 3.D0*s13**(-1)*s23 + 3.D0*s13*s23**(-1)

      elseif(ieorder.eq.0)then
      D0g1subD2 =  + pi**2 * (  - 7.D0/3.D0 - 7.D0/6.D0*s13**(-1)*s23 - 7.D
     &    0/6.D0*s13*s23**(-1) )
      D0g1subD2 = D0g1subD2 + dls * ( 4.D0/3.D0 + 2.D0/3.D0*s13**(-1)*s23 + 2.
     &    D0/3.D0*s13*s23**(-1) )
      D0g1subD2 = D0g1subD2 + dls**2 * ( 2.D0 + s13**(-1)*s23 + s13*s23**(-1)
     &     )
      D0g1subD2 = D0g1subD2 + dlx * ( 4.D0/3.D0 + 2.D0/3.D0*s13**(-1)*s23 + 2.
     &    D0/3.D0*s13*s23**(-1) )
      D0g1subD2 = D0g1subD2 + dlx**2 * (  - 2.D0 - s13**(-1)*s23 - s13*
     &    s23**(-1) )
      D0g1subD2 = D0g1subD2 - 26.D0/9.D0 - 13.D0/9.D0*s13**(-1)*s23 - 13.D0/9.
     &    D0*s13*s23**(-1)

      endif
      return
      end
************************************************************************
*
      function Dh0g1subD1(iq1,iqb2,iq2,iqb1,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      parameter(zeta3=1.202056903159594285d0)
      common /order/ieorder
      s12 =s4(iq1,iqb1)
      s23 =s4(iq1,iqb2)
      s13 =s4(iq1,iq2)
      dlx   =log(-s23/s12)
      dlomx =log(-s13/s12)
      dls =log(s12/renscale2)
      if(ieorder.eq.-2)then
      Dh0g1subD1 =  0

      elseif(ieorder.eq.-1)then
      Dh0g1subD1 =  0

      elseif(ieorder.eq.0)then
      Dh0g1subD1 =  + dls * (  - 2.D0/3.D0 - 2.D0/3.D0*s12**(-1)*s23 + 2.D0
     &    /3.D0*s13*s23**(-1) )
      Dh0g1subD1 = Dh0g1subD1 + dlx * (  - 2.D0/3.D0 - 2.D0/3.D0*s12**(-1)*
     &    s23 + 2.D0/3.D0*s13*s23**(-1) )
      Dh0g1subD1 = Dh0g1subD1 + 10.D0/9.D0 + 10.D0/9.D0*s12**(-1)*s23 - 10.D0/
     &    9.D0*s13*s23**(-1)

      endif
      return
      end
************************************************************************
*
      function Dh0g1subD2(iq1,iqb2,iq2,iqb1,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      parameter(zeta3=1.202056903159594285d0)
      common /order/ieorder
      s12 =s4(iq1,iq2)
      s23 =s4(iq1,iqb2)
      s13 =s4(iq1,iqb1)
      dlx   =log(-s23/s12)
      dlomx =log(-s13/s12)
      dls =log(s12/renscale2)
      if(ieorder.eq.-2)then
      Dh0g1subD2 =  0

      elseif(ieorder.eq.-1)then
      Dh0g1subD2 =  0

      elseif(ieorder.eq.0)then
      Dh0g1subD2 =  + dls * (  - 4.D0/3.D0 - 2.D0/3.D0*s13**(-1)*s23 - 2.D0
     &    /3.D0*s13*s23**(-1) )
      Dh0g1subD2 = Dh0g1subD2 + dlx * (  - 4.D0/3.D0 - 2.D0/3.D0*s13**(-1)*
     &    s23 - 2.D0/3.D0*s13*s23**(-1) )
      Dh0g1subD2 = Dh0g1subD2 + 20.D0/9.D0 + 10.D0/9.D0*s13**(-1)*s23 + 10.D0/
     &    9.D0*s13*s23**(-1)

      endif
      return
      end
************************************************************************
*
      function Dt0g1subD1(iq1,iqb2,iq2,iqb1,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      parameter(zeta3=1.202056903159594285d0)
      common /order/ieorder
      s12 =s4(iq1,iqb1)
      s23 =s4(iq1,iqb2)
      s13 =s4(iq1,iq2)
      dlx   =log(-s23/s12)
      dlomx =log(-s13/s12)
      dls =log(s12/renscale2)
      if(ieorder.eq.-2)then
      Dt0g1subD1 =  - 2.D0 - 2.D0*s12**(-1)*s23 + 2.D0*s13*s23**(-1)

      elseif(ieorder.eq.-1)then
      Dt0g1subD1 =  + dls * ( 2.D0 + 2.D0*s12**(-1)*s23 - 2.D0*s13*
     &    s23**(-1) )
      Dt0g1subD1 = Dt0g1subD1 + dlomx * (  - 2.D0 - 2.D0*s12**(-1)*s23 + 2.D0
     &    *s13*s23**(-1) )
      Dt0g1subD1 = Dt0g1subD1 + dlx * ( 2.D0 + 2.D0*s12**(-1)*s23 - 2.D0*s13*
     &    s23**(-1) )
      Dt0g1subD1 = Dt0g1subD1 - 3.D0 - 3.D0*s12**(-1)*s23 + 3.D0*s13*
     &    s23**(-1)

      elseif(ieorder.eq.0)then
      Dt0g1subD1 =  + pi**2 * ( 1.D0/6.D0 + 1.D0/6.D0*s12**(-1)*s23 - 1.D0/
     &    6.D0*s13*s23**(-1) )
      Dt0g1subD1 = Dt0g1subD1 + dls * ( 3.D0 + 3.D0*s12**(-1)*s23 - 3.D0*s13*
     &    s23**(-1) )
      Dt0g1subD1 = Dt0g1subD1 + dls**2 * (  - 1.D0 - s12**(-1)*s23 + s13*
     &    s23**(-1) )
      Dt0g1subD1 = Dt0g1subD1 + dlomx*dls * ( 2.D0 + 2.D0*s12**(-1)*s23 - 2.D0
     &    *s13*s23**(-1) )
      Dt0g1subD1 = Dt0g1subD1 + dlx * ( 2.D0 + 2.D0*s12**(-1)*s23 - 3.D0*s13*
     &    s23**(-1) )
      Dt0g1subD1 = Dt0g1subD1 + dlx*dls * (  - 2.D0 - 2.D0*s12**(-1)*s23 + 2.D
     &    0*s13*s23**(-1) )
      Dt0g1subD1 = Dt0g1subD1 + dlx*dlomx * ( 2.D0 + 2.D0*s12**(-1)*s23 - 2.D0
     &    *s13*s23**(-1) )
      Dt0g1subD1 = Dt0g1subD1 + dlx**2 * (  - 2.D0 - 3.D0/2.D0*s12**(-1)*s23
     &     + s13*s23**(-1) )
      Dt0g1subD1 = Dt0g1subD1 - 8.D0 - 8.D0*s12**(-1)*s23 + 8.D0*s13*
     &    s23**(-1)

      endif
      return
      end
************************************************************************
*
      function Dt0g1subD2(iq1,iqb2,iq2,iqb1,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      parameter(zeta3=1.202056903159594285d0)
      common /order/ieorder
      s12 =s4(iq1,iq2)
      s23 =s4(iq1,iqb2)
      s13 =s4(iq1,iqb1)
      dlx   =log(-s23/s12)
      dlomx =log(-s13/s12)
      dls =log(s12/renscale2)
      if(ieorder.eq.-2)then
      Dt0g1subD2 =  - 4.D0 - 2.D0*s13**(-1)*s23 - 2.D0*s13*s23**(-1)

      elseif(ieorder.eq.-1)then
      Dt0g1subD2 =  + dls * ( 4.D0 + 2.D0*s13**(-1)*s23 + 2.D0*s13*
     &    s23**(-1) )
      Dt0g1subD2 = Dt0g1subD2 + dlomx * ( 4.D0 + 2.D0*s13**(-1)*s23 + 2.D0*
     &    s13*s23**(-1) )
      Dt0g1subD2 = Dt0g1subD2 + dlx * ( 4.D0 + 2.D0*s13**(-1)*s23 + 2.D0*s13*
     &    s23**(-1) )
      Dt0g1subD2 = Dt0g1subD2 - 6.D0 - 3.D0*s13**(-1)*s23 - 3.D0*s13*
     &    s23**(-1)

      elseif(ieorder.eq.0)then
      Dt0g1subD2 =  + pi**2 * (  - 2.D0/3.D0 - 1.D0/3.D0*s13**(-1)*s23 + 1.
     &    D0/6.D0*s13*s23**(-1) )
      Dt0g1subD2 = Dt0g1subD2 + dls * ( 6.D0 + 3.D0*s13**(-1)*s23 + 3.D0*s13*
     &    s23**(-1) )
      Dt0g1subD2 = Dt0g1subD2 + dls**2 * (  - 2.D0 - s13**(-1)*s23 - s13*
     &    s23**(-1) )
      Dt0g1subD2 = Dt0g1subD2 + dlomx * ( 1.D0 + s13**(-1)*s23 )
      Dt0g1subD2 = Dt0g1subD2 + dlomx*dls * (  - 4.D0 - 2.D0*s13**(-1)*s23 - 
     &    2.D0*s13*s23**(-1) )
      Dt0g1subD2 = Dt0g1subD2 + dlomx**2 * (  - 1.D0 - 1.D0/2.D0*s13**(-1)*
     &    s23 )
      Dt0g1subD2 = Dt0g1subD2 + dlx * ( 5.D0 + 2.D0*s13**(-1)*s23 + 3.D0*s13*
     &    s23**(-1) )
      Dt0g1subD2 = Dt0g1subD2 + dlx*dls * (  - 4.D0 - 2.D0*s13**(-1)*s23 - 2.D
     &    0*s13*s23**(-1) )
      Dt0g1subD2 = Dt0g1subD2 + dlx*dlomx * (  - 2.D0 - s13**(-1)*s23 - 2.D0*
     &    s13*s23**(-1) )
      Dt0g1subD2 = Dt0g1subD2 + dlx**2 * (  - 3.D0 - 3.D0/2.D0*s13**(-1)*s23
     &     - s13*s23**(-1) )
      Dt0g1subD2 = Dt0g1subD2 - 16.D0 - 8.D0*s13**(-1)*s23 - 8.D0*s13*
     &    s23**(-1)

      endif
      return
      end
