!************************************************************************
!  All tree-level amplitudes in this file are based on
!  
!  F.~A.~Berends and W.~Giele,
!  %``The Six Gluon Process as an Example of Weyl-Van Der Waerden Spinor Calculus,''
!  Nucl. Phys. B \textbf{294} (1987), 700-732
!  doi:10.1016/0550-3213(87)90604-3
!
!  M.~L.~Mangano and S.~J.~Parke,
!  %``Multiparton amplitudes in gauge theories,''
!  Phys. Rept. \textbf{200} (1991), 301-367
!  doi:10.1016/0370-1573(91)90091-Y
!  [arXiv:hep-th/0509223 [hep-th]].
!
!  J.G.M. Kuijf,
!  Multiparton production at hadron colliders
!  PhD thesis, 
!  Leiden (1991).
************************************************************************
*
* four gluon zero quark tree level matrix elements
*
*
* summed over colour orderings, summed and squared
*
      function FullA4g0(i1,i2,i3,i4)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
*
*      write(*,*)" hello FullA4g0"
      fac=(4d0*pi*as*nc)**2*(nc**2-1d0)*2d0

      FullA4g0=+A4g0(i1,i2,i3,i4)
     .         +A4g0(i1,i3,i4,i2)
     .         +A4g0(i1,i4,i2,i3)
      FullA4g0=fac*FullA4g0
*      write(*,*)"FullA4g0",FullA4g0
*
      return
      end

************************************************************************
*
* four gluon zero quark tree level matrix elements
*
* one colour ordering, summed and squared
*
*
      function A4g0(i1,i2,i3,i4)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
*
      s12=s4(i1,i2)
      s13=s4(i1,i3)
      s14=s4(i1,i4)
      s23=s4(i2,i3)
      s24=s4(i2,i4)
      s34=s4(i3,i4)

      hs=(s12**4+s13**4+s14**4+s23**4+s24**4+s34**4)

      A4g0=hs/s12/s23/s34/s14
*      write(*,*)"A4g0",A4g0,i1,i2,i3,i4
*
      return
      end


************************************************************************
*
* four gluon zero quark tree level matrix elements
*
* interference between two colour orderings, M4g0(i1,i2,i3,i4)*M4g0(j1,j2,j3,j4)
* relevant for subleading colour subtraction terms
*
*
      function A4g0int(i1,i2,i3,i4,j1,j2,j3,j4)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      
      A4g0int=0d0
      
      l1=i1
      l2=i2
      l3=i3
      l4=i4

* cyclic symmetry      
      if((i1.eq.1).and.(i2.eq.3).and.(i3.eq.4).and.(i4.eq.2)) then
         l1=1; l2=2; l3=4; l4=3;
      elseif((i1.eq.1).and.(i2.eq.4).and.(i3.eq.2).and.(i4.eq.3)) then
         l1=1; l2=3; l3=2; l4=4;
      elseif((i1.eq.1).and.(i2.eq.4).and.(i3.eq.3).and.(i4.eq.2)) then
         l1=1; l2=2; l3=3; l4=4;
      elseif((i1.eq.2).and.(i2.eq.1).and.(i3.eq.3).and.(i4.eq.4)) then
         l1=1; l2=2; l3=4; l4=3;
      elseif((i1.eq.2).and.(i2.eq.1).and.(i3.eq.4).and.(i4.eq.3)) then
         l1=1; l2=2; l3=3; l4=4;
      elseif((i1.eq.2).and.(i2.eq.3).and.(i3.eq.1).and.(i4.eq.4)) then
         l1=1; l2=3; l3=2; l4=4;
      elseif((i1.eq.2).and.(i2.eq.3).and.(i3.eq.4).and.(i4.eq.1)) then
         l1=1; l2=2; l3=3; l4=4;
      elseif((i1.eq.2).and.(i2.eq.4).and.(i3.eq.1).and.(i4.eq.3)) then
         l1=1; l2=3; l3=2; l4=4;
      elseif((i1.eq.2).and.(i2.eq.4).and.(i3.eq.3).and.(i4.eq.1)) then
         l1=1; l2=2; l3=4; l4=3;
      elseif((i1.eq.3).and.(i2.eq.1).and.(i3.eq.2).and.(i4.eq.4)) then
         l1=1; l2=2; l3=4; l4=3;
      elseif((i1.eq.3).and.(i2.eq.1).and.(i3.eq.4).and.(i4.eq.2)) then
         l1=1; l2=3; l3=2; l4=4;
      elseif((i1.eq.3).and.(i2.eq.2).and.(i3.eq.1).and.(i4.eq.4)) then
         l1=1; l2=2; l3=3; l4=4;
      elseif((i1.eq.3).and.(i2.eq.2).and.(i3.eq.4).and.(i4.eq.1)) then
         l1=1; l2=3; l3=2; l4=4;
      elseif((i1.eq.3).and.(i2.eq.4).and.(i3.eq.1).and.(i4.eq.2)) then
         l1=1; l2=2; l3=3; l4=4;
      elseif((i1.eq.3).and.(i2.eq.4).and.(i3.eq.2).and.(i4.eq.1)) then
         l1=1; l2=2; l3=4; l4=3;
      elseif((i1.eq.4).and.(i2.eq.1).and.(i3.eq.2).and.(i4.eq.3)) then
         l1=1; l2=2; l3=3; l4=4;
      elseif((i1.eq.4).and.(i2.eq.1).and.(i3.eq.3).and.(i4.eq.2)) then
         l1=1; l2=3; l3=2; l4=4;
      elseif((i1.eq.4).and.(i2.eq.2).and.(i3.eq.1).and.(i4.eq.3)) then
         l1=1; l2=2; l3=4; l4=3;
      elseif((i1.eq.4).and.(i2.eq.2).and.(i3.eq.3).and.(i4.eq.1)) then
         l1=1; l2=3; l3=2; l4=4;
      elseif((i1.eq.4).and.(i2.eq.3).and.(i3.eq.1).and.(i4.eq.2)) then
         l1=1; l2=2; l3=4; l4=3;
      elseif((i1.eq.4).and.(i2.eq.3).and.(i3.eq.2).and.(i4.eq.1)) then
         l1=1; l2=2; l3=3; l4=4;      
      endif
        

      k1=j1
      k2=j2
      k3=j3
      k4=j4

* cyclic symmetry
      if((j1.eq.1).and.(j2.eq.3).and.(j3.eq.4).and.(j4.eq.2)) then
         k1=1; k2=2; k3=4; k4=3;
      elseif((j1.eq.1).and.(j2.eq.4).and.(j3.eq.2).and.(j4.eq.3)) then
         k1=1; k2=3; k3=2; k4=4;
      elseif((j1.eq.1).and.(j2.eq.4).and.(j3.eq.3).and.(j4.eq.2)) then
         k1=1; k2=2; k3=3; k4=4;
      elseif((j1.eq.2).and.(j2.eq.1).and.(j3.eq.3).and.(j4.eq.4)) then
         k1=1; k2=2; k3=4; k4=3;
      elseif((j1.eq.2).and.(j2.eq.1).and.(j3.eq.4).and.(j4.eq.3)) then
         k1=1; k2=2; k3=3; k4=4;
      elseif((j1.eq.2).and.(j2.eq.3).and.(j3.eq.1).and.(j4.eq.4)) then
         k1=1; k2=3; k3=2; k4=4;
      elseif((j1.eq.2).and.(j2.eq.3).and.(j3.eq.4).and.(j4.eq.1)) then
         k1=1; k2=2; k3=3; k4=4;
      elseif((j1.eq.2).and.(j2.eq.4).and.(j3.eq.1).and.(j4.eq.3)) then
         k1=1; k2=3; k3=2; k4=4;
      elseif((j1.eq.2).and.(j2.eq.4).and.(j3.eq.3).and.(j4.eq.1)) then
         k1=1; k2=2; k3=4; k4=3;
      elseif((j1.eq.3).and.(j2.eq.1).and.(j3.eq.2).and.(j4.eq.4)) then
         k1=1; k2=2; k3=4; k4=3;
      elseif((j1.eq.3).and.(j2.eq.1).and.(j3.eq.4).and.(j4.eq.2)) then
         k1=1; k2=3; k3=2; k4=4;
      elseif((j1.eq.3).and.(j2.eq.2).and.(j3.eq.1).and.(j4.eq.4)) then
         k1=1; k2=2; k3=3; k4=4;
      elseif((j1.eq.3).and.(j2.eq.2).and.(j3.eq.4).and.(j4.eq.1)) then
         k1=1; k2=3; k3=2; k4=4;
      elseif((j1.eq.3).and.(j2.eq.4).and.(j3.eq.1).and.(j4.eq.2)) then
         k1=1; k2=2; k3=3; k4=4;
      elseif((j1.eq.3).and.(j2.eq.4).and.(j3.eq.2).and.(j4.eq.1)) then
         k1=1; k2=2; k3=4; k4=3;
      elseif((j1.eq.4).and.(j2.eq.1).and.(j3.eq.2).and.(j4.eq.3)) then
         k1=1; k2=2; k3=3; k4=4;
      elseif((j1.eq.4).and.(j2.eq.1).and.(j3.eq.3).and.(j4.eq.2)) then
         k1=1; k2=3; k3=2; k4=4;
      elseif((j1.eq.4).and.(j2.eq.2).and.(j3.eq.1).and.(j4.eq.3)) then
         k1=1; k2=2; k3=4; k4=3;
      elseif((j1.eq.4).and.(j2.eq.2).and.(j3.eq.3).and.(j4.eq.1)) then
         k1=1; k2=3; k3=2; k4=4;
      elseif((j1.eq.4).and.(j2.eq.3).and.(j3.eq.1).and.(j4.eq.2)) then
         k1=1; k2=2; k3=4; k4=3;
      elseif((j1.eq.4).and.(j2.eq.3).and.(j3.eq.2).and.(j4.eq.1)) then
         k1=1; k2=2; k3=3; k4=4;
      endif

* helicity sum
      call m4g0mmpp(l1,l2,l3,l4,ztree1)
      if(k2.eq.l2) call m4g0mmpp(k1,k2,k3,k4,ztree2)
      if(k3.eq.l2) call m4g0mpmp(k1,k2,k3,k4,ztree2)
      if(k4.eq.l2) call m4g0mmpp(k4,k1,k2,k3,ztree2)
      A4g0int=A4g0int+Real(ztree1*dconjg(ztree2))

      call m4g0mpmp(l1,l2,l3,l4,ztree1)
      if(k2.eq.l3) call m4g0mmpp(k1,k2,k3,k4,ztree2)
      if(k3.eq.l3) call m4g0mpmp(k1,k2,k3,k4,ztree2)
      if(k4.eq.l3) call m4g0mmpp(k4,k1,k2,k3,ztree2)
      A4g0int=A4g0int+Real(ztree1*dconjg(ztree2))

      call m4g0mmpp(l4,l1,l2,l3,ztree1)
      if(k2.eq.l4) call m4g0mmpp(k1,k2,k3,k4,ztree2)
      if(k3.eq.l4) call m4g0mpmp(k1,k2,k3,k4,ztree2)
      if(k4.eq.l4) call m4g0mmpp(k4,k1,k2,k3,ztree2)
      A4g0int=A4g0int+Real(ztree1*dconjg(ztree2))

      call m4g0mmpp(l2,l3,l4,l1,ztree1)
      if(k2.eq.l4) call m4g0mmpp(k3,k4,k1,k2,ztree2)
      if(k3.eq.l4) call m4g0mpmp(k2,k3,k4,k1,ztree2)
      if(k4.eq.l4) call m4g0mmpp(k2,k3,k4,k1,ztree2)
      A4g0int=A4g0int+Real(ztree1*dconjg(ztree2))

      call m4g0mpmp(l2,l3,l4,l1,ztree1)
      if(k2.eq.l3) call m4g0mmpp(k3,k4,k1,k2,ztree2)
      if(k3.eq.l3) call m4g0mpmp(k2,k3,k4,k1,ztree2)
      if(k4.eq.l3) call m4g0mmpp(k2,k3,k4,k1,ztree2)
      A4g0int=A4g0int+Real(ztree1*dconjg(ztree2))

      call m4g0mmpp(l3,l4,l1,l2,ztree1)
      if(k2.eq.l2) call m4g0mmpp(k3,k4,k1,k2,ztree2)
      if(k3.eq.l2) call m4g0mpmp(k4,k1,k2,k3,ztree2)
      if(k4.eq.l2) call m4g0mmpp(k3,k2,k1,k4,ztree2)
      A4g0int=A4g0int+Real(ztree1*dconjg(ztree2))

      return
      end



************************************************************************
*
* four gluon zero quark tree-level helicity amplitude
*
      subroutine m4g0mmpp(i1,i2,i3,i4,ztree)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      zi=(0d0,1d0)

      za12=zA4(i1,i2)
      za23=zA4(i2,i3)
      za34=zA4(i3,i4)
      za41=zA4(i4,i1)

      ztree=zi*za12**(4)/(za12*za23*za34*za41)

      end

************************************************************************
*
* four gluon zero quark tree-level helicity amplitude
*
      subroutine m4g0mpmp(i1,i2,i3,i4,ztree)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      zi=(0d0,1d0)

      za12=zA4(i1,i2)
      za23=zA4(i2,i3)
      za34=zA4(i3,i4)
      za41=zA4(i4,i1)

      za13=zA4(i1,i3)

      ztree=zi*za13**(4)/(za12*za23*za34*za41)

      end


************************************************************************
*
* five gluon zero quark tree level matrix elements
*
*
* summed over colour orderings, summed and squared
*
      function FullA5g0(i1,i2,i3,i4,i5)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)


*      write(*,*)" hello FullA5g0"
      fac=(4d0*pi*as*nc)**3*(nc**2-1d0)*2d0

      FullA5g0=+A5g0(i1,i2,i3,i4,i5)
     .         +A5g0(i1,i2,i3,i5,i4)
     .         +A5g0(i1,i2,i4,i3,i5)
     .         +A5g0(i1,i2,i4,i5,i3)
     .         +A5g0(i1,i2,i5,i3,i4)
     .         +A5g0(i1,i2,i5,i4,i3)
     .         +A5g0(i1,i3,i2,i4,i5)
     .         +A5g0(i1,i3,i2,i5,i4)
     .         +A5g0(i1,i4,i2,i3,i5)
     .         +A5g0(i1,i4,i2,i5,i3)
     .         +A5g0(i1,i5,i2,i3,i4)
     .         +A5g0(i1,i5,i2,i4,i3)
      FullA5g0=fac*FullA5g0
*      write(*,*)"FullA5g0",FullA5g0
*
      return
      end
*
************************************************************************
*
* five gluon zero quark tree level matrix elements
*
* one colour ordering, summed and squared
*
*
      function A5g0(i1,i2,i3,i4,i5)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
*
*      write(*,*)" hello A5g0"

      s12=s5(i1,i2)
      s13=s5(i1,i3)
      s14=s5(i1,i4)
      s15=s5(i1,i5)
      s23=s5(i2,i3)
      s24=s5(i2,i4)
      s25=s5(i2,i5)
      s34=s5(i3,i4)
      s35=s5(i3,i5)
      s45=s5(i4,i5)

      hs=2d0*(s12**4+s13**4+s14**4+s15**4+s23**4
     .       +s24**4+s25**4+s34**4+s35**4+s45**4)

      A5g0=hs/s12/s23/s34/s45/s15

*      write(*,10)A5g0,i1,i2,i3,i4,i5
   10 format(" A5g0   ",e12.4,5i3)
*
      return
      end



************************************************************************
*
* five gluon zero quark tree level matrix elements
*
* one colour ordering, summed and squared
*
      function At5g0(i1,i2,i3,i4,i5)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
*
*      write(*,*)" hello At5g0"

      !OBS this is 2*Re(M50(1,2,3,4,5)*M50s(1,4,2,5,3) which is the only inteference
      !of 5 gluon tree level amplitudes that we need to evaluate for dsigma_a_RR at SLC
      !it corresponds to the only two orderings with no two or three particle poles in common*


       hs= -((s5(i1,i3)*s5(i1,i5)*s5(i2,i4)*s5(i2,i5)*s5(i3,i4)
     & - s5(i1,i3)*s5(i1,i4)*s5(i2,i5)**2*s5(i3,i4)
     & + s5(i1,i4)*s5(i1,i5)*s5(i2,i3)*s5(i2,i4)*s5(i3,i5)
     & - s5(i1,i3)*s5(i1,i5)*s5(i2,i4)**2*s5(i3,i5)
     & - s5(i1,i4)**2*s5(i2,i3)*s5(i2,i5)*s5(i3,i5)
     & + s5(i1,i3)*s5(i1,i4)*s5(i2,i4)*s5(i2,i5)*s5(i3,i5)
     & + s5(i1,i2)*s5(i1,i4)*s5(i2,i5)*s5(i3,i4)*s5(i3,i5)
     & - s5(i1,i2)*s5(i1,i4)*s5(i2,i4)*s5(i3,i5)**2
     & + s5(i1,i3)*s5(i1,i4)*s5(i2,i3)*s5(i2,i5)*s5(i4,i5)
     & - s5(i1,i3)**2*s5(i2,i4)*s5(i2,i5)*s5(i4,i5)
     & + s5(i1,i2)*s5(i1,i3)*s5(i2,i4)*s5(i3,i5)*s5(i4,i5))*
     & (s5(i1,i2)**4 + s5(i1,i3)**4 + s5(i1,i4)**4 + s5(i1,i5)**4
     & + s5(i2,i3)**4 + s5(i2,i4)**4 + s5(i2,i5)**4 + s5(i3,i4)**4
     & + s5(i3,i5)**4 + s5(i4,i5)**4))

      denom=s5(i1,i2)*s5(i1,i3)*s5(i1,i4)*s5(i1,i5)*s5(i2,i3)
     &*s5(i2,i4)*s5(i2,i5)*s5(i3,i4)*s5(i3,i5)*s5(i4,i5)

      At5g0=2d0*hs/denom

*      write(*,10)At5g0,i1,i2,i3,i4,i5
   10 format(" At5g0   ",e12.4,5i3)
*
      return
      end


************************************************************************
*
* six gluon zero quark tree level matrix elements
*
*
* summed over colour orderings, summed and squared
*
      function FullA6g0(i1,i2,i3,i4,i5,i6)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)

*      write(*,*)" hello FullA6g0"
      fac=(4d0*pi*as*nc)**4*(nc**2-1d0)*2d0

      ansLC=
     .     +A6g0(i1,i2,i3,i4,i5,i6)
     .     +A6g0(i1,i2,i3,i6,i5,i4)
     .     +A6g0(i1,i2,i4,i3,i6,i5)
     .     +A6g0(i1,i2,i5,i4,i3,i6)
     .     +A6g0(i1,i2,i6,i5,i4,i3)
     .     +A6g0(i1,i2,i6,i3,i4,i5)
     .     +A6g0(i1,i2,i5,i6,i3,i4)
     .     +A6g0(i1,i2,i4,i5,i6,i3)
     .     +A6g0(i1,i2,i3,i5,i4,i6)
     .     +A6g0(i1,i2,i6,i4,i5,i3)
     .     +A6g0(i1,i2,i5,i4,i6,i3)
     .     +A6g0(i1,i2,i3,i6,i4,i5)
     .     +A6g0(i1,i2,i4,i6,i3,i5)
     .     +A6g0(i1,i2,i5,i3,i6,i4)
     .     +A6g0(i1,i2,i6,i3,i5,i4)
     .     +A6g0(i1,i2,i4,i5,i3,i6)
     .     +A6g0(i1,i2,i3,i4,i6,i5)
     .     +A6g0(i1,i2,i5,i6,i4,i3)
     .     +A6g0(i1,i2,i4,i6,i5,i3)
     .     +A6g0(i1,i2,i3,i5,i6,i4)
     .     +A6g0(i1,i2,i6,i5,i3,i4)
     .     +A6g0(i1,i2,i4,i3,i5,i6)
     .     +A6g0(i1,i2,i5,i3,i4,i6)
     .     +A6g0(i1,i2,i6,i4,i3,i5)

     .     +A6g0(i1,i3,i2,i4,i5,i6)
     .     +A6g0(i1,i3,i2,i6,i5,i4)
     .     +A6g0(i1,i3,i2,i5,i6,i4)
     .     +A6g0(i1,i3,i2,i4,i6,i5)
     .     +A6g0(i1,i3,i2,i5,i4,i6)
     .     +A6g0(i1,i3,i2,i6,i4,i5)
     .     +A6g0(i1,i4,i2,i3,i5,i6)
     .     +A6g0(i1,i4,i2,i6,i5,i3)
     .     +A6g0(i1,i4,i2,i5,i6,i3)
     .     +A6g0(i1,i4,i2,i3,i6,i5)
     .     +A6g0(i1,i4,i2,i5,i3,i6)
     .     +A6g0(i1,i4,i2,i6,i3,i5)
     .     +A6g0(i1,i5,i2,i3,i4,i6)
     .     +A6g0(i1,i5,i2,i6,i4,i3)
     .     +A6g0(i1,i5,i2,i4,i6,i3)
     .     +A6g0(i1,i5,i2,i3,i6,i4)
     .     +A6g0(i1,i5,i2,i4,i3,i6)
     .     +A6g0(i1,i5,i2,i6,i3,i4)
     .     +A6g0(i1,i6,i2,i3,i4,i5)
     .     +A6g0(i1,i6,i2,i5,i4,i3)
     .     +A6g0(i1,i6,i2,i4,i5,i3)
     .     +A6g0(i1,i6,i2,i3,i5,i4)
     .     +A6g0(i1,i6,i2,i4,i3,i5)
     .     +A6g0(i1,i6,i2,i5,i3,i4)

     .     +A6g0(i1,i3,i4,i2,i5,i6)
     .     +A6g0(i1,i4,i3,i2,i6,i5)
     .     +A6g0(i1,i3,i5,i2,i4,i6)
     .     +A6g0(i1,i5,i3,i2,i6,i4)
     .     +A6g0(i1,i3,i6,i2,i4,i5)
     .     +A6g0(i1,i6,i3,i2,i5,i4)
     .     +A6g0(i1,i3,i4,i2,i6,i5)
     .     +A6g0(i1,i4,i3,i2,i5,i6)
     .     +A6g0(i1,i3,i5,i2,i6,i4)
     .     +A6g0(i1,i5,i3,i2,i4,i6)
     .     +A6g0(i1,i3,i6,i2,i5,i4)
     .     +A6g0(i1,i6,i3,i2,i4,i5)


       ansSLC=
     .     +At6g0(i1,i2,i3,i4,i5,i6)
     .     +At6g0(i1,i2,i3,i4,i6,i5)
     .     +At6g0(i1,i2,i3,i5,i4,i6)
     .     +At6g0(i1,i2,i3,i5,i6,i4)
     .     +At6g0(i1,i2,i3,i6,i4,i5)
     .     +At6g0(i1,i2,i3,i6,i5,i4)
     .     +At6g0(i1,i2,i4,i3,i5,i6)
     .     +At6g0(i1,i2,i4,i3,i6,i5)
     .     +At6g0(i1,i2,i4,i5,i3,i6)
     .     +At6g0(i1,i2,i4,i5,i6,i3)
     .     +At6g0(i1,i2,i4,i6,i3,i5)
     .     +At6g0(i1,i2,i4,i6,i5,i3)
     .     +At6g0(i1,i2,i5,i3,i4,i6)
     .     +At6g0(i1,i2,i5,i3,i6,i4)
     .     +At6g0(i1,i2,i5,i4,i3,i6)
     .     +At6g0(i1,i2,i5,i4,i6,i3)
     .     +At6g0(i1,i2,i5,i6,i3,i4)
     .     +At6g0(i1,i2,i5,i6,i4,i3)
     .     +At6g0(i1,i2,i6,i3,i4,i5)
     .     +At6g0(i1,i2,i6,i3,i5,i4)
     .     +At6g0(i1,i2,i6,i4,i3,i5)
     .     +At6g0(i1,i2,i6,i4,i5,i3)
     .     +At6g0(i1,i2,i6,i5,i3,i4)
     .     +At6g0(i1,i2,i6,i5,i4,i3)

     .     +At6g0(i1,i3,i2,i4,i5,i6)
     .     +At6g0(i1,i3,i2,i4,i6,i5)
     .     +At6g0(i1,i3,i2,i5,i4,i6)
     .     +At6g0(i1,i3,i2,i5,i6,i4)
     .     +At6g0(i1,i3,i2,i6,i4,i5)
     .     +At6g0(i1,i3,i2,i6,i5,i4)
     .     +At6g0(i1,i4,i2,i3,i5,i6)
     .     +At6g0(i1,i4,i2,i3,i6,i5)
     .     +At6g0(i1,i4,i2,i5,i3,i6)
     .     +At6g0(i1,i4,i2,i5,i6,i3)
     .     +At6g0(i1,i4,i2,i6,i3,i5)
     .     +At6g0(i1,i4,i2,i6,i5,i3)
     .     +At6g0(i1,i5,i2,i3,i4,i6)
     .     +At6g0(i1,i5,i2,i3,i6,i4)
     .     +At6g0(i1,i5,i2,i4,i3,i6)
     .     +At6g0(i1,i5,i2,i4,i6,i3)
     .     +At6g0(i1,i5,i2,i6,i3,i4)
     .     +At6g0(i1,i5,i2,i6,i4,i3)
     .     +At6g0(i1,i6,i2,i3,i4,i5)
     .     +At6g0(i1,i6,i2,i3,i5,i4)
     .     +At6g0(i1,i6,i2,i4,i3,i5)
     .     +At6g0(i1,i6,i2,i4,i5,i3)
     .     +At6g0(i1,i6,i2,i5,i3,i4)
     .     +At6g0(i1,i6,i2,i5,i4,i3)

     .     +At6g0(i1,i3,i4,i2,i5,i6)
     .     +At6g0(i1,i3,i4,i2,i6,i5)
     .     +At6g0(i1,i3,i5,i2,i4,i6)
     .     +At6g0(i1,i3,i5,i2,i6,i4)
     .     +At6g0(i1,i3,i6,i2,i4,i5)
     .     +At6g0(i1,i3,i6,i2,i5,i4)
     .     +At6g0(i1,i4,i3,i2,i5,i6)
     .     +At6g0(i1,i4,i3,i2,i6,i5)
     .     +At6g0(i1,i4,i5,i2,i3,i6)
     .     +At6g0(i1,i4,i6,i2,i3,i5)
     .     +At6g0(i1,i5,i3,i2,i4,i6)
     .     +At6g0(i1,i5,i4,i2,i3,i6)



      FullA6g0=fac*(ansLC+ansSLC*2d0/nc**2)

*      write(*,*)"FullA6g0",FullA6g0

*
      return
      end

*
************************************************************************
*
* six gluon zero quark subleading colour tree level matrix elements
*
*
* summed over colour orderings, summed and squared
*
      function FullAt6g0(i1,i2,i3,i4,i5,i6)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)

*      write(*,*)" hello FullAt6g0"
      fac=(4d0*pi*as*nc)**4*(nc**2-1d0)*2d0*2d0/nc**2

      FullAt6g0=
     .     +At6g0(i1,i2,i3,i4,i5,i6)
     .     +At6g0(i1,i2,i3,i6,i5,i4)
     .     +At6g0(i1,i2,i4,i3,i6,i5)
     .     +At6g0(i1,i2,i5,i4,i3,i6)
     .     +At6g0(i1,i2,i6,i5,i4,i3)
     .     +At6g0(i1,i2,i6,i3,i4,i5)
     .     +At6g0(i1,i2,i5,i6,i3,i4)
     .     +At6g0(i1,i2,i4,i5,i6,i3)
     .     +At6g0(i1,i2,i3,i5,i4,i6)
     .     +At6g0(i1,i2,i6,i4,i5,i3)
     .     +At6g0(i1,i2,i5,i4,i6,i3)
     .     +At6g0(i1,i2,i3,i6,i4,i5)
     .     +At6g0(i1,i2,i4,i6,i3,i5)
     .     +At6g0(i1,i2,i5,i3,i6,i4)
     .     +At6g0(i1,i2,i6,i3,i5,i4)
     .     +At6g0(i1,i2,i4,i5,i3,i6)
     .     +At6g0(i1,i2,i3,i4,i6,i5)
     .     +At6g0(i1,i2,i5,i6,i4,i3)
     .     +At6g0(i1,i2,i4,i6,i5,i3)
     .     +At6g0(i1,i2,i3,i5,i6,i4)
     .     +At6g0(i1,i2,i6,i5,i3,i4)
     .     +At6g0(i1,i2,i4,i3,i5,i6)
     .     +At6g0(i1,i2,i5,i3,i4,i6)
     .     +At6g0(i1,i2,i6,i4,i3,i5)

     .     +At6g0(i1,i3,i2,i4,i5,i6)
     .     +At6g0(i1,i3,i2,i6,i5,i4)
     .     +At6g0(i1,i3,i2,i5,i6,i4)
     .     +At6g0(i1,i3,i2,i4,i6,i5)
     .     +At6g0(i1,i3,i2,i5,i4,i6)
     .     +At6g0(i1,i3,i2,i6,i4,i5)
     .     +At6g0(i1,i4,i2,i3,i5,i6)
     .     +At6g0(i1,i4,i2,i6,i5,i3)
     .     +At6g0(i1,i4,i2,i5,i6,i3)
     .     +At6g0(i1,i4,i2,i3,i6,i5)
     .     +At6g0(i1,i4,i2,i5,i3,i6)
     .     +At6g0(i1,i4,i2,i6,i3,i5)
     .     +At6g0(i1,i5,i2,i3,i4,i6)
     .     +At6g0(i1,i5,i2,i6,i4,i3)
     .     +At6g0(i1,i5,i2,i4,i6,i3)
     .     +At6g0(i1,i5,i2,i3,i6,i4)
     .     +At6g0(i1,i5,i2,i4,i3,i6)
     .     +At6g0(i1,i5,i2,i6,i3,i4)
     .     +At6g0(i1,i6,i2,i3,i4,i5)
     .     +At6g0(i1,i6,i2,i5,i4,i3)
     .     +At6g0(i1,i6,i2,i4,i5,i3)
     .     +At6g0(i1,i6,i2,i3,i5,i4)
     .     +At6g0(i1,i6,i2,i4,i3,i5)
     .     +At6g0(i1,i6,i2,i5,i3,i4)

     .     +At6g0(i1,i3,i4,i2,i5,i6)
     .     +At6g0(i1,i4,i3,i2,i6,i5)
     .     +At6g0(i1,i3,i5,i2,i4,i6)
     .     +At6g0(i1,i5,i3,i2,i6,i4)
     .     +At6g0(i1,i3,i6,i2,i4,i5)
     .     +At6g0(i1,i6,i3,i2,i5,i4)
     .     +At6g0(i1,i3,i4,i2,i6,i5)
     .     +At6g0(i1,i4,i3,i2,i5,i6)
     .     +At6g0(i1,i3,i5,i2,i6,i4)
     .     +At6g0(i1,i5,i3,i2,i4,i6)
     .     +At6g0(i1,i3,i6,i2,i5,i4)
     .     +At6g0(i1,i6,i3,i2,i4,i5)



      FullAt6g0=fac*FullAt6g0
*      write(*,*)"FullAt6g0",FullAt6g0

*
      return
      end

*
************************************************************************
*
* six gluon zero quark tree level matrix elements
*
* one colour ordering, summed and squared
*
*
      function A6g0(i1,i2,i3,i4,i5,i6)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer NHEL(6,25),PERM(6)

      DATA (NHEL(IHEL,   1),IHEL=1, 6) / 1, 1,-1,-1,-1,-1/
      DATA (NHEL(IHEL,   2),IHEL=1, 6) / 1,-1, 1,-1,-1,-1/
      DATA (NHEL(IHEL,   3),IHEL=1, 6) / 1,-1,-1, 1,-1,-1/
      DATA (NHEL(IHEL,   4),IHEL=1, 6) / 1,-1,-1,-1, 1,-1/
      DATA (NHEL(IHEL,   5),IHEL=1, 6) / 1,-1,-1,-1,-1, 1/

      DATA (NHEL(IHEL,   6),IHEL=1, 6) / 1,-1,-1, 1, 1, 1/
      DATA (NHEL(IHEL,   7),IHEL=1, 6) / 1,-1, 1,-1, 1, 1/
      DATA (NHEL(IHEL,   8),IHEL=1, 6) / 1,-1, 1, 1,-1, 1/
      DATA (NHEL(IHEL,   9),IHEL=1, 6) / 1,-1, 1, 1, 1,-1/
      DATA (NHEL(IHEL,  10),IHEL=1, 6) / 1, 1,-1,-1, 1, 1/
      DATA (NHEL(IHEL,  11),IHEL=1, 6) / 1, 1,-1, 1,-1, 1/
      DATA (NHEL(IHEL,  12),IHEL=1, 6) / 1, 1,-1, 1, 1,-1/
      DATA (NHEL(IHEL,  13),IHEL=1, 6) / 1, 1, 1,-1,-1, 1/
      DATA (NHEL(IHEL,  14),IHEL=1, 6) / 1, 1, 1,-1, 1,-1/
      DATA (NHEL(IHEL,  15),IHEL=1, 6) / 1, 1, 1, 1,-1,-1/

      DATA (NHEL(IHEL,  16),IHEL=1, 6) / 1, 1, 1,-1,-1,-1/
      DATA (NHEL(IHEL,  17),IHEL=1, 6) / 1, 1,-1,-1,-1, 1/
      DATA (NHEL(IHEL,  18),IHEL=1, 6) / 1,-1,-1,-1, 1, 1/
      DATA (NHEL(IHEL,  19),IHEL=1, 6) / 1, 1,-1, 1,-1,-1/
      DATA (NHEL(IHEL,  20),IHEL=1, 6) / 1, 1,-1,-1, 1,-1/
      DATA (NHEL(IHEL,  21),IHEL=1, 6) / 1,-1, 1,-1,-1, 1/
      DATA (NHEL(IHEL,  22),IHEL=1, 6) / 1,-1, 1, 1,-1,-1/
      DATA (NHEL(IHEL,  23),IHEL=1, 6) / 1,-1,-1, 1,-1, 1/
      DATA (NHEL(IHEL,  24),IHEL=1, 6) / 1,-1,-1, 1, 1,-1/
      DATA (NHEL(IHEL,  25),IHEL=1, 6) / 1,-1, 1,-1, 1,-1/

*
*      write(*,*)" hello A6g0"
c Helicity loop for leading color


        perm(1)=i1
        perm(2)=i2
        perm(3)=i3
        perm(4)=i4
        perm(5)=i5
        perm(6)=i6
        amp2=0d0
        do i=1,25
          numplus=0
          do j=1,6
             if(NHEL(j,i).eq.1) numplus=numplus+1
          enddo
          if(numplus.eq.2) then
             zamp=zA6gMHVb(PERM(1),NHEL(1,i))
          endif
          if(numplus.eq.3) then
             zamp=zA6gNMHV(PERM(1),NHEL(1,i))
          endif
          if(numplus.eq.4) then
             zamp=zA6gMHVa(PERM(1),NHEL(1,i))
          endif
*         write(*,*)(NHEL(k,i),k=1,6)
*         write(*,*)" value ",zamp
          amp2=amp2+2d0*zamp*dconjg(zamp)
        enddo
        A6g0=amp2

*      write(*,10)A6g0,i1,i2,i3,i4,i5,i6
   10 format(" A6g0   ",e12.4,6i3)
*
      return
      end




************************************************************************
*
* six gluon zero quark subleading colour tree level matrix elements
*
* one colour ordering, summed and interfered
*
*
      function At6g0(i1,i2,i3,i4,i5,i6)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer NHEL(6,25),NHEL2(6,25),NHEL3(6,25),NHEL4(6,25)
      integer PERM(6),PERM2(6),PERM3(6),PERM4(6)

      DATA (NHEL(IHEL,   1),IHEL=1, 6) / 1, 1,-1,-1,-1,-1/
      DATA (NHEL(IHEL,   2),IHEL=1, 6) / 1,-1, 1,-1,-1,-1/
      DATA (NHEL(IHEL,   3),IHEL=1, 6) / 1,-1,-1, 1,-1,-1/
      DATA (NHEL(IHEL,   4),IHEL=1, 6) / 1,-1,-1,-1, 1,-1/
      DATA (NHEL(IHEL,   5),IHEL=1, 6) / 1,-1,-1,-1,-1, 1/

      DATA (NHEL(IHEL,   6),IHEL=1, 6) / 1,-1,-1, 1, 1, 1/
      DATA (NHEL(IHEL,   7),IHEL=1, 6) / 1,-1, 1,-1, 1, 1/
      DATA (NHEL(IHEL,   8),IHEL=1, 6) / 1,-1, 1, 1,-1, 1/
      DATA (NHEL(IHEL,   9),IHEL=1, 6) / 1,-1, 1, 1, 1,-1/
      DATA (NHEL(IHEL,  10),IHEL=1, 6) / 1, 1,-1,-1, 1, 1/
      DATA (NHEL(IHEL,  11),IHEL=1, 6) / 1, 1,-1, 1,-1, 1/
      DATA (NHEL(IHEL,  12),IHEL=1, 6) / 1, 1,-1, 1, 1,-1/
      DATA (NHEL(IHEL,  13),IHEL=1, 6) / 1, 1, 1,-1,-1, 1/
      DATA (NHEL(IHEL,  14),IHEL=1, 6) / 1, 1, 1,-1, 1,-1/
      DATA (NHEL(IHEL,  15),IHEL=1, 6) / 1, 1, 1, 1,-1,-1/

      DATA (NHEL(IHEL,  16),IHEL=1, 6) / 1, 1, 1,-1,-1,-1/
      DATA (NHEL(IHEL,  17),IHEL=1, 6) / 1, 1,-1,-1,-1, 1/
      DATA (NHEL(IHEL,  18),IHEL=1, 6) / 1,-1,-1,-1, 1, 1/
      DATA (NHEL(IHEL,  19),IHEL=1, 6) / 1, 1,-1, 1,-1,-1/
      DATA (NHEL(IHEL,  20),IHEL=1, 6) / 1, 1,-1,-1, 1,-1/
      DATA (NHEL(IHEL,  21),IHEL=1, 6) / 1,-1, 1,-1,-1, 1/
      DATA (NHEL(IHEL,  22),IHEL=1, 6) / 1,-1, 1, 1,-1,-1/
      DATA (NHEL(IHEL,  23),IHEL=1, 6) / 1,-1,-1, 1,-1, 1/
      DATA (NHEL(IHEL,  24),IHEL=1, 6) / 1,-1,-1, 1, 1,-1/
      DATA (NHEL(IHEL,  25),IHEL=1, 6) / 1,-1, 1,-1, 1,-1/

*
*      write(*,*)" hello At6g0"
c Helicity loop for subleading color

        perm(1)=i1
        perm(2)=i2
        perm(3)=i3
        perm(4)=i4
        perm(5)=i5
        perm(6)=i6

        perm2(1)=i1
        perm2(2)=i3
        perm2(3)=i5
        perm2(4)=i2
        perm2(5)=i6
        perm2(6)=i4

        perm3(1)=i1
        perm3(2)=i4
        perm3(3)=i2
        perm3(4)=i6
        perm3(5)=i3
        perm3(6)=i5

        perm4(1)=i1
        perm4(2)=i3
        perm4(3)=i6
        perm4(4)=i4
        perm4(5)=i2
        perm4(6)=i5

        do ii=1,25
           NHEL2(1,II)=NHEL(1,II)
           NHEL2(2,II)=NHEL(3,II)
           NHEL2(3,II)=NHEL(5,II)
           NHEL2(4,II)=NHEL(2,II)
           NHEL2(5,II)=NHEL(6,II)
           NHEL2(6,II)=NHEL(4,II)

           NHEL3(1,II)=NHEL(1,II)
           NHEL3(2,II)=NHEL(4,II)
           NHEL3(3,II)=NHEL(2,II)
           NHEL3(4,II)=NHEL(6,II)
           NHEL3(5,II)=NHEL(3,II)
           NHEL3(6,II)=NHEL(5,II)

           NHEL4(1,II)=NHEL(1,II)
           NHEL4(2,II)=NHEL(3,II)
           NHEL4(3,II)=NHEL(6,II)
           NHEL4(4,II)=NHEL(4,II)
           NHEL4(5,II)=NHEL(2,II)
           NHEL4(6,II)=NHEL(5,II)
        enddo

        amp2=0d0
        do i=1,25
          numplus=0
          do j=1,6
             if(NHEL(j,i).eq.1) numplus=numplus+1
          enddo
          if(numplus.eq.2) then
             zampc=dconjg(zA6gMHVb(PERM(1),NHEL(1,i)))
             zamp1=zA6gMHVb(PERM2(1),NHEL2(1,i))
             zamp2=zA6gMHVb(PERM3(1),NHEL3(1,i))
             zamp3=zA6gMHVb(PERM4(1),NHEL4(1,i))
          endif
          if(numplus.eq.3) then
             zampc=dconjg(zA6gNMHV(PERM(1),NHEL(1,i)))
             zamp1=zA6gNMHV(PERM2(1),NHEL2(1,i))
             zamp2=zA6gNMHV(PERM3(1),NHEL3(1,i))
             zamp3=zA6gNMHV(PERM4(1),NHEL4(1,i))
          endif
          if(numplus.eq.4) then
             zampc=dconjg(zA6gMHVa(PERM(1),NHEL(1,i)))
             zamp1=zA6gMHVa(PERM2(1),NHEL2(1,i))
             zamp2=zA6gMHVa(PERM3(1),NHEL3(1,i))
             zamp3=zA6gMHVa(PERM4(1),NHEL4(1,i))
          endif
          amp2=amp2+2d0*dreal(zampc*(zamp1+zamp2+zamp3))
        enddo
        At6g0=amp2

*      write(*,10)At6g0,i1,i2,i3,i4,i5,i6
   10 format(" At6g0   ",e12.4,6i3)
*
      return
      end



************************************************************************
*
* six gluon zero quark tree level matrix elements
*
*
* summed over colour orderings, summed and squared
*
      function FullA6g0XYZ(i1,i2,i3,i4,i5,i6)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)

*      write(*,*)" hello FullA6g0XYZ"
      fac=(4d0*pi*as*nc)**4*(nc**2-1d0)*2d0

      FullA6g0XYZ=
     .     +A6g0X(i1,i2,i3,i4,i5,i6)
     .     +A6g0X(i1,i2,i3,i5,i4,i6)
     .     +A6g0X(i1,i2,i3,i4,i6,i5)

     .     +A6g0Y(i1,i3,i2,i4,i5,i6)
     .     +A6g0Y(i1,i4,i2,i5,i6,i3)
     .     +A6g0Y(i1,i5,i2,i6,i3,i4)
     .     +A6g0Y(i1,i6,i2,i3,i4,i5)

     .     +A6g0Z(i1,i3,i4,i2,i5,i6)
     .     +A6g0Z(i1,i3,i4,i2,i6,i5)
     .     +A6g0Z(i1,i3,i5,i2,i4,i6)
     .     +A6g0Z(i1,i3,i5,i2,i6,i4)
     .     +A6g0Z(i1,i3,i6,i2,i4,i5)
     .     +A6g0Z(i1,i3,i6,i2,i5,i4)

      FullA6g0XYZ=fac*FullA6g0XYZ
*      write(*,*)"FullA6g0XYZ",FullA6g0XYZ

*
      return
      end

************************************************************************
*
* six gluon zero quark tree level matrix elements
*
*
* summed over colour orderings, summed and squared
*
      function A6g0X(i1,i2,i3,i4,i5,i6)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)

*      write(*,*)" hello A6g0X"

      A6g0X=+A6g0(i1,i2,i3,i4,i5,i6)
     .      +A6g0(i1,i2,i6,i5,i4,i3)
     .      +A6g0(i1,i2,i4,i5,i6,i3)
     .      +A6g0(i1,i2,i3,i6,i5,i4)
     .      +A6g0(i1,i2,i5,i6,i3,i4)
     .      +A6g0(i1,i2,i4,i3,i6,i5)
     .      +A6g0(i1,i2,i6,i3,i4,i5)
     .      +A6g0(i1,i2,i5,i4,i3,i6)

*      write(*,*)"A6g0X",A6g0X

*
      return
      end
      function A6g0Y(i1,i3,i2,i4,i5,i6)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)

*      write(*,*)" hello A6g0Y"

      A6g0Y=
     .      +A6g0(i1,i3,i2,i4,i5,i6)
     .      +A6g0(i1,i3,i2,i4,i6,i5)
     .      +A6g0(i1,i3,i2,i5,i4,i6)
     .      +A6g0(i1,i3,i2,i5,i6,i4)
     .      +A6g0(i1,i3,i2,i6,i4,i5)
     .      +A6g0(i1,i3,i2,i6,i5,i4)

*      write(*,*)"A6g0Y",A6g0Y

*
      return
      end

      function A6g0Z(i1,i3,i4,i2,i5,i6)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)

*      write(*,*)" hello A6g0Z"

      A6g0Z=
     .      +A6g0(i1,i3,i4,i2,i5,i6)
     .      +A6g0(i1,i4,i3,i2,i6,i5)

*      write(*,*)"A6g0Z",A6g0Z

*
      return
      end
************************************************************************
*
      complex*16 function zA6gMHVa(PERM,HEL)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      integer hel(6),perm(6)
      integer k(2)

      zi=(0d0,1d0)

      j=0
      do i=1,6
        if(HEL(i).eq.-1) then
           j=j+1
           k(j)=perm(i)
        endif
      enddo

      zA6gMHVa=zi*zA6(k(1),k(2))**(4)
     . /zA6(perm(1),perm(2))/zA6(perm(2),perm(3))
     . /zA6(perm(3),perm(4))/zA6(perm(4),perm(5))
     . /zA6(perm(5),perm(6))/zA6(perm(6),perm(1))

*      hs=s6(k(1),k(2))**4
*      den=s6(perm(1),perm(2))*s6(perm(2),perm(3))
*     .   *s6(perm(3),perm(4))*s6(perm(4),perm(5))
*     .   *s6(perm(5),perm(6))*s6(perm(6),perm(1))
*      write(*,*)" mhva ",zA6gMHVa*dconjg(zA6gMHVa),hs/den
      return
      end
************************************************************************
*
      complex*16 function zA6gMHVb(PERM,HEL)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      integer hel(6),perm(6)
      integer k(2)

      zi=(0d0,1d0)

      j=0
      do i=1,6
        if(HEL(i).eq.1) then
           j=j+1
           k(j)=perm(i)
        endif
      enddo

      zA6gMHVb=zi*zB6(k(1),k(2))**(4)
     . /zB6(perm(1),perm(2))/zB6(perm(2),perm(3))
     . /zB6(perm(3),perm(4))/zB6(perm(4),perm(5))
     . /zB6(perm(5),perm(6))/zB6(perm(6),perm(1))

*      hs=s6(k(1),k(2))**4
*      den=s6(perm(1),perm(2))*s6(perm(2),perm(3))
*     .   *s6(perm(3),perm(4))*s6(perm(4),perm(5))
*     .   *s6(perm(5),perm(6))*s6(perm(6),perm(1))
*      write(*,*)" mhvb ",zA6gMHVb*dconjg(zA6gMHVb),hs/den
      return
      end
************************************************************************
*
      complex*16 function zA6gNMHV(PERM,HEL)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      integer hel(6),perm(6)
      integer permaux(6), helaux(6)
      integer permgood(6),helgood(6)
      integer aux1,aux2,aux3(6),aux4(6)

      do i=1,6
         PERMaux(i)=PERM(i)
         HELaux(i)=HEL(i)
      enddo

      j=0
c
c perform cyclic permutations of the helicity configuration
c untill it finds the right configuration: +++---; ++-+--; +-+-+-
      do i=1,6
         if( HELaux(1) .eq. 1 .and. HELaux(2).eq.1 .and.
     &       HELaux(3) .eq. 1 )
     & then
            j=1
           do k=1,6
              PERMgood(k)=PERMaux(k)
              HELgood(k)=HELaux(k)
           enddo
         endif

         if( HELaux(1) .eq. 1 .and. HELaux(2).eq.1 .and.
     &       HELaux(3) .eq.-1 .and. HELaux(4).eq.1) then
            j=2
           do k=1,6
              PERMgood(k)=PERMaux(k)
              HELgood(k)=HELaux(k)
           enddo
         endif

         if( HELaux(1). eq.1 .and. HELaux(2).eq.-1 .and.
     &       HELaux(3) .eq.1 .and. HELaux(4).eq.-1 .and.
     &       HELaux(5). eq.1) then
            j=3
           do k=1,6
              PERMgood(k)=PERMaux(k)
              HELgood(k)=HELaux(k)
           enddo
         endif

c do the cyclic permutation
        aux1=HELaux(1)
        aux2=PERMaux(1)
        do k=1,5
           HELaux(k)=HELaux(k+1)
           PERMaux(k)=PERMaux(k+1)
        enddo
        HELaux(6)=aux1
        PERMaux(6)=aux2
      enddo

c if j=0 we need to do a line reversal and do the previous check again
      if(j.eq.0) then
         do k=1,6
           aux3(k)=HELaux(k)
           aux4(k)=PERMaux(k)
        enddo
        do k=1,6
           HELaux(k)=aux3(7-k)
           PERMaux(k)=aux4(7-k)
        enddo

c perform cyclic permutations of the helicity configuration
c untill it finds the right configuration: +++---; ++-+--; +-+-+-
         do i=1,6
            if(HELaux(1).eq.1 .and. HELaux(2).eq.1 .and.
     &         HELaux(3).eq.1 ) then
               j=1
              do k=1,6
                 PERMgood(k)=PERMaux(k)
                 HELgood(k)=HELaux(k)
              enddo
            endif

            if(HELaux(1).eq. 1 .and. HELaux(2).eq.1 .and.
     &         HELaux(3).eq.-1 .and. HELaux(4).eq.1) then
               j=2
              do k=1,6
                 PERMgood(k)=PERMaux(k)
                 HELgood(k)=HELaux(k)
              enddo
            endif

            if(HELaux(1).eq.1 .and. HELaux(2).eq.-1 .and.
     &         HELaux(3).eq.1 .and. HELaux(4).eq.-1 .and.
     &         HELaux(5).eq.1) then
               j=3
              do k=1,6
                 PERMgood(k)=PERMaux(k)
                 HELgood(k)=HELaux(k)
              enddo
            endif

c do the cyclic permutation
           aux1=HELaux(1)
           aux2=PERMaux(1)
           do k=1,5
              HELaux(k)=HELaux(k+1)
              PERMaux(k)=PERMaux(k+1)
           enddo
           HELaux(6)=aux1
           PERMaux(6)=aux2
         enddo
      endif

      if(j.eq.1) then
         zA6gNMHV=zA6gpppmmm(PERMgood)
      endif
      if(j.eq.2) then
         zA6gNMHV=zA6gppmpmm(PERMgood)
      endif
      if(j.eq.3) then
         zA6gNMHV=zA6gpmpmpm(PERMgood)
      endif

      zi=(0d0,1d0)
      zA6gNMHV=zi*zA6gNMHV

      return
      end
************************************************************************
*
      complex*16 function zA6gpppmmm(perm)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer perm(6)

      t123=S6(perm(1),perm(2))+S6(perm(1),perm(3))
     &+S6(perm(2),perm(3))
      t234=S6(perm(2),perm(3))+S6(perm(2),perm(4))
     &+S6(perm(3),perm(4))
      t345=S6(perm(3),perm(4))+S6(perm(3),perm(5))
     &+S6(perm(4),perm(5))

      zb12=zB6(perm(1),perm(2))
      zb13=zB6(perm(1),perm(3))
      zb34=zB6(perm(3),perm(4))
      zb35=zB6(perm(3),perm(5))
      zb45=zB6(perm(4),perm(5))
      zb56=zB6(perm(5),perm(6))

      zb61=zB6(perm(6),perm(1))
      zb53=zB6(perm(5),perm(3))
      zb54=zB6(perm(5),perm(4))

      za12=zA6(perm(1),perm(2))
      za23=zA6(perm(2),perm(3))
      za24=zA6(perm(2),perm(4))
      za34=zA6(perm(3),perm(4))
      za46=zA6(perm(4),perm(6))
      za56=zA6(perm(5),perm(6))

      za61=zA6(perm(6),perm(1))
      za32=zA6(perm(3),perm(2))
      za42=zA6(perm(4),perm(2))

      zA6gpppmmm=1d0/(zb53*za32+zb54*za42)
     &*((zb12*za24+zb13*za34)**(3)/(za23*za34*zb56*zb61*t234)
     &+(zb34*za46+zb35*za56)**(3)/(za61*za12*zb34*zb45*t345))

*      write(*,*)" zA6gpppmmm ",perm,zA6gpppmmm

      return
      end
************************************************************************
*
      complex*16 function zA6gppmpmm(perm)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer perm(6)

      t123=S6(perm(1),perm(2))+S6(perm(1),perm(3))
     &+S6(perm(2),perm(3))
      t234=S6(perm(2),perm(3))+S6(perm(2),perm(4))
     &+S6(perm(3),perm(4))
      t345=S6(perm(3),perm(4))+S6(perm(3),perm(5))
     &+S6(perm(4),perm(5))

      za12=zA6(perm(1),perm(2))
      za13=zA6(perm(1),perm(3))
      za23=zA6(perm(2),perm(3))
      za34=zA6(perm(3),perm(4))
      za35=zA6(perm(3),perm(5))
      za36=zA6(perm(3),perm(6))
      za45=zA6(perm(4),perm(5))
      za56=zA6(perm(5),perm(6))

      za31=zA6(perm(3),perm(1))
      za61=zA6(perm(6),perm(1))
      za32=zA6(perm(3),perm(2))
      za53=zA6(perm(5),perm(3))
      za54=zA6(perm(5),perm(4))

      zb12=zB6(perm(1),perm(2))
      zb14=zB6(perm(1),perm(4))
      zb16=zB6(perm(1),perm(6))
      zb23=zB6(perm(2),perm(3))
      zb24=zB6(perm(2),perm(4))
      zb26=zB6(perm(2),perm(6))
      zb34=zB6(perm(3),perm(4))
      zb45=zB6(perm(4),perm(5))
      zb46=zB6(perm(4),perm(6))
      zb56=zB6(perm(5),perm(6))

      zb61=zB6(perm(6),perm(1))
      zb32=zB6(perm(3),perm(2))
      zb42=zB6(perm(4),perm(2))

      zA6gppmpmm=zb24**(4)*za56**(3)
     &/(zb23*zb34*za61*t234*(za12*zb24+za13*zb34)
     &*(za53*zb32+za54*zb42))
     &+(za31*zb14+za32*zb24)**(4)
     &/(za12*za23*zb45*zb56*t123
     &*(za12*zb24+za13*zb34)*(za31*zb16+za32*zb26))
     &+zb12**(3)*za35**(4)
     &/(zb61*za34*za45*t345*(za53*zb32+za54*zb42)
     &*(za34*zb46+za35*zb56))

*      write(*,*)" zA6gppmpmm ",perm,zA6gppmpmm

      return
      end
************************************************************************
*
      complex*16 function zA6gpmpmpm(perm)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer perm(6)

      t123=S6(perm(1),perm(2))+S6(perm(1),perm(3))
     &+S6(perm(2),perm(3))
      t234=S6(perm(2),perm(3))+S6(perm(2),perm(4))
     &+S6(perm(3),perm(4))
      t345=S6(perm(3),perm(4))+S6(perm(3),perm(5))
     &+S6(perm(4),perm(5))

      za12=zA6(perm(1),perm(2))
      za23=zA6(perm(2),perm(3))
      za24=zA6(perm(2),perm(4))
      za26=zA6(perm(2),perm(6))
      za34=zA6(perm(3),perm(4))
      za45=zA6(perm(4),perm(5))
      za46=zA6(perm(4),perm(6))
      za56=zA6(perm(5),perm(6))

      za61=zA6(perm(6),perm(1))
      za42=zA6(perm(4),perm(2))
      za62=zA6(perm(6),perm(2))
      za43=zA6(perm(4),perm(3))
      za64=zA6(perm(6),perm(4))
      za65=zA6(perm(6),perm(5))

      zb12=zB6(perm(1),perm(2))
      zb13=zB6(perm(1),perm(3))
      zb15=zB6(perm(1),perm(5))
      zb23=zB6(perm(2),perm(3))
      zb34=zB6(perm(3),perm(4))
      zb35=zB6(perm(3),perm(5))
      zb45=zB6(perm(4),perm(5))
      zb56=zB6(perm(5),perm(6))

      zb21=zB6(perm(2),perm(1))
      zb31=zB6(perm(3),perm(1))
      zb61=zB6(perm(6),perm(1))
      zb43=zB6(perm(4),perm(3))
      zb53=zB6(perm(5),perm(3))

      zA6gpmpmpm=zb13**(4)*za46**(4)
     &/(zb12*zb23*za45*za56*t123
     &*(za61*zb13+za62*zb23)
     &*(za42*zb21+za43*zb31))
     &+za26**(4)*zb35**(4)
     &/(za61*za12*zb34*zb45*t345
     &*(za64*zb43+za65*zb53)
     &*(za23*zb35+za24*zb45))
     &+zb15**(4)*za24**(4)
     &/(za23*za34*zb56*zb61*t234
     &*(za42*zb21+za43*zb31)
     &*(za23*zb35+za24*zb45))

*      write(*,*)" zA6gpmpmpm ",perm,zA6gpmpmpm
      return
      end
************************************************************************
* Full LC (N^0) colour level of FullA6g0
      function A6g0full(i1,i2,i3,i4,i5,i6)
      implicit double precision (a-h,o-z)

      A6g0full = +A6g0(i1,i2,i3,i4,i5,i6)
     .           +A6g0(i1,i2,i3,i6,i5,i4)
     .           +A6g0(i1,i2,i4,i3,i6,i5)
     .           +A6g0(i1,i2,i5,i4,i3,i6)
     .           +A6g0(i1,i2,i6,i5,i4,i3)
     .           +A6g0(i1,i2,i6,i3,i4,i5)
     .           +A6g0(i1,i2,i5,i6,i3,i4)
     .           +A6g0(i1,i2,i4,i5,i6,i3)
     .           +A6g0(i1,i2,i3,i5,i4,i6)
     .           +A6g0(i1,i2,i6,i4,i5,i3)
     .           +A6g0(i1,i2,i5,i4,i6,i3)
     .           +A6g0(i1,i2,i3,i6,i4,i5)
     .           +A6g0(i1,i2,i4,i6,i3,i5)
     .           +A6g0(i1,i2,i5,i3,i6,i4)
     .           +A6g0(i1,i2,i6,i3,i5,i4)
     .           +A6g0(i1,i2,i4,i5,i3,i6)
     .           +A6g0(i1,i2,i3,i4,i6,i5)
     .           +A6g0(i1,i2,i5,i6,i4,i3)
     .           +A6g0(i1,i2,i4,i6,i5,i3)
     .           +A6g0(i1,i2,i3,i5,i6,i4)
     .           +A6g0(i1,i2,i6,i5,i3,i4)
     .           +A6g0(i1,i2,i4,i3,i5,i6)
     .           +A6g0(i1,i2,i5,i3,i4,i6)
     .           +A6g0(i1,i2,i6,i4,i3,i5)

     .           +A6g0(i1,i3,i2,i4,i5,i6)
     .           +A6g0(i1,i3,i2,i6,i5,i4)
     .           +A6g0(i1,i3,i2,i5,i6,i4)
     .           +A6g0(i1,i3,i2,i4,i6,i5)
     .           +A6g0(i1,i3,i2,i5,i4,i6)
     .           +A6g0(i1,i3,i2,i6,i4,i5)
     .           +A6g0(i1,i4,i2,i3,i5,i6)
     .           +A6g0(i1,i4,i2,i6,i5,i3)
     .           +A6g0(i1,i4,i2,i5,i6,i3)
     .           +A6g0(i1,i4,i2,i3,i6,i5)
     .           +A6g0(i1,i4,i2,i5,i3,i6)
     .           +A6g0(i1,i4,i2,i6,i3,i5)
     .           +A6g0(i1,i5,i2,i3,i4,i6)
     .           +A6g0(i1,i5,i2,i6,i4,i3)
     .           +A6g0(i1,i5,i2,i4,i6,i3)
     .           +A6g0(i1,i5,i2,i3,i6,i4)
     .           +A6g0(i1,i5,i2,i4,i3,i6)
     .           +A6g0(i1,i5,i2,i6,i3,i4)
     .           +A6g0(i1,i6,i2,i3,i4,i5)
     .           +A6g0(i1,i6,i2,i5,i4,i3)
     .           +A6g0(i1,i6,i2,i4,i5,i3)
     .           +A6g0(i1,i6,i2,i3,i5,i4)
     .           +A6g0(i1,i6,i2,i4,i3,i5)
     .           +A6g0(i1,i6,i2,i5,i3,i4)

     .           +A6g0(i1,i3,i4,i2,i5,i6)
     .           +A6g0(i1,i4,i3,i2,i6,i5)
     .           +A6g0(i1,i3,i5,i2,i4,i6)
     .           +A6g0(i1,i5,i3,i2,i6,i4)
     .           +A6g0(i1,i3,i6,i2,i4,i5)
     .           +A6g0(i1,i6,i3,i2,i5,i4)
     .           +A6g0(i1,i3,i4,i2,i6,i5)
     .           +A6g0(i1,i4,i3,i2,i5,i6)
     .           +A6g0(i1,i3,i5,i2,i6,i4)
     .           +A6g0(i1,i5,i3,i2,i4,i6)
     .           +A6g0(i1,i3,i6,i2,i5,i4)
     .           +A6g0(i1,i6,i3,i2,i4,i5)

      return
      end
************************************************************************
* Full SLC (1/N^2) colour level of FullA6g0
      function At6g0full(i1,i2,i3,i4,i5,i6)
      implicit double precision (a-h,o-z)

      At6g0full = 2d0*(
     .            +At6g0(i1,i2,i3,i4,i5,i6)
     .            +At6g0(i1,i2,i3,i4,i6,i5)
     .            +At6g0(i1,i2,i3,i5,i4,i6)
     .            +At6g0(i1,i2,i3,i5,i6,i4)
     .            +At6g0(i1,i2,i3,i6,i4,i5)
     .            +At6g0(i1,i2,i3,i6,i5,i4)
     .            +At6g0(i1,i2,i4,i3,i5,i6)
     .            +At6g0(i1,i2,i4,i3,i6,i5)
     .            +At6g0(i1,i2,i4,i5,i3,i6)
     .            +At6g0(i1,i2,i4,i5,i6,i3)
     .            +At6g0(i1,i2,i4,i6,i3,i5)
     .            +At6g0(i1,i2,i4,i6,i5,i3)
     .            +At6g0(i1,i2,i5,i3,i4,i6)
     .            +At6g0(i1,i2,i5,i3,i6,i4)
     .            +At6g0(i1,i2,i5,i4,i3,i6)
     .            +At6g0(i1,i2,i5,i4,i6,i3)
     .            +At6g0(i1,i2,i5,i6,i3,i4)
     .            +At6g0(i1,i2,i5,i6,i4,i3)
     .            +At6g0(i1,i2,i6,i3,i4,i5)
     .            +At6g0(i1,i2,i6,i3,i5,i4)
     .            +At6g0(i1,i2,i6,i4,i3,i5)
     .            +At6g0(i1,i2,i6,i4,i5,i3)
     .            +At6g0(i1,i2,i6,i5,i3,i4)
     .            +At6g0(i1,i2,i6,i5,i4,i3)

     .            +At6g0(i1,i3,i2,i4,i5,i6)
     .            +At6g0(i1,i3,i2,i4,i6,i5)
     .            +At6g0(i1,i3,i2,i5,i4,i6)
     .            +At6g0(i1,i3,i2,i5,i6,i4)
     .            +At6g0(i1,i3,i2,i6,i4,i5)
     .            +At6g0(i1,i3,i2,i6,i5,i4)
     .            +At6g0(i1,i4,i2,i3,i5,i6)
     .            +At6g0(i1,i4,i2,i3,i6,i5)
     .            +At6g0(i1,i4,i2,i5,i3,i6)
     .            +At6g0(i1,i4,i2,i5,i6,i3)
     .            +At6g0(i1,i4,i2,i6,i3,i5)
     .            +At6g0(i1,i4,i2,i6,i5,i3)
     .            +At6g0(i1,i5,i2,i3,i4,i6)
     .            +At6g0(i1,i5,i2,i3,i6,i4)
     .            +At6g0(i1,i5,i2,i4,i3,i6)
     .            +At6g0(i1,i5,i2,i4,i6,i3)
     .            +At6g0(i1,i5,i2,i6,i3,i4)
     .            +At6g0(i1,i5,i2,i6,i4,i3)
     .            +At6g0(i1,i6,i2,i3,i4,i5)
     .            +At6g0(i1,i6,i2,i3,i5,i4)
     .            +At6g0(i1,i6,i2,i4,i3,i5)
     .            +At6g0(i1,i6,i2,i4,i5,i3)
     .            +At6g0(i1,i6,i2,i5,i3,i4)
     .            +At6g0(i1,i6,i2,i5,i4,i3)

     .            +At6g0(i1,i3,i4,i2,i5,i6)
     .            +At6g0(i1,i3,i4,i2,i6,i5)
     .            +At6g0(i1,i3,i5,i2,i4,i6)
     .            +At6g0(i1,i3,i5,i2,i6,i4)
     .            +At6g0(i1,i3,i6,i2,i4,i5)
     .            +At6g0(i1,i3,i6,i2,i5,i4)
     .            +At6g0(i1,i4,i3,i2,i5,i6)
     .            +At6g0(i1,i4,i3,i2,i6,i5)
     .            +At6g0(i1,i4,i5,i2,i3,i6)
     .            +At6g0(i1,i4,i6,i2,i3,i5)
     .            +At6g0(i1,i5,i3,i2,i4,i6)
     .            +At6g0(i1,i5,i4,i2,i3,i6)
     .            )

      return
      end
************************************************************************

