!************************************************************************
!  All tree-level amplitudes in this file are based on
!  
!  M.~L.~Mangano and S.~J.~Parke,
!  %``Multiparton amplitudes in gauge theories,''
!  Phys. Rept. \textbf{200} (1991), 301-367
!  doi:10.1016/0370-1573(91)90091-Y
!  [arXiv:hep-th/0509223 [hep-th]].
!
!  J.G.M. Kuijf,
!  Multiparton production at hadron colliders
!  PhD thesis, 
!  Leiden (1991).
************************************************************************
*
* two gluon two quark tree level matrix elements
*
*                      q  g  g  qb
*
* summed over colour orderings, summed and squared
*
      function FullB2g0(iq,ig1,ig2,iqb)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)

c      write(*,*)" hello FullB2g0"
      fac=(4d0*pi*as*nc)**2*(nc**2-1d0)/nc

      FullB2g0=     +B2g0(iq,ig1,ig2,iqb)
     .              +B2g0(iq,ig2,ig1,iqb)
     .  -1d0/dble(nc**(2))*Bt2g0(iq,ig1,ig2,iqb)

      FullB2g0=fac*FullB2g0
c      write(*,*)"FullB2g0",FullB2g0
*
      return
      end
*
************************************************************************
*
* two gluon two quark tree level matrix elements
*
* one colour ordering, summed and squared
*
*                    q   g   g  qb
*
*
      function B2g0(iq,ig1,ig2,iqb)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
*
      sq1 =s4(iq,ig1)
      sq2 =s4(iq,ig2)
      s1qb=s4(iqb,ig1)
      s2qb=s4(iqb,ig2)
      s12 =s4(ig1,ig2)
      sqqb=s4(iq,iqb)

      B2g0=2d0*(sq1**3*sq2+sq1*sq2**3)/sq1**2/s12**2

      B2g0=abs(B2g0)

*      write(*,*)"B2g0",B2g0,iq,ig1,ig2,iqb
*
      return
      end
*
* Symmetrised version of B2g0
*
      function sB2g0(iq,ig1,ig2,iqb)
      implicit double precision (a-h,o-z)
      sB2g0=B2g0(iq,ig1,ig2,iqb)+B2g0(iqb,ig2,ig1,iq)
      return
      end

*
************************************************************************
*
* two gluon two quark tree level matrix elements
*
* subleading colour ordering, summed and squared
*
*
      function Bt2g0(iq,ig1,ig2,iqb)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
*

      sq1 =s4(iq,ig1)
      sq2 =s4(iq,ig2)
      s1qb=s4(iqb,ig1)
      s2qb=s4(iqb,ig2)
      s12 =s4(ig1,ig2)
      sqqb=s4(iq,iqb)

      hs=2d0*(sq1**3*sq2+sq1*sq2**3)

      Bt2g0=hs/sq1**2/sq2**2

      Bt2g0=abs(Bt2g0)
*
      return
      end

************************************************************************
* function used for R term reduced matrix element

      function BRt2g0(iq,ig1,ig2,iqb)
      use KinData_mod
      implicit double precision (a-h,o-z)
*
      wt = Bt2g0(iq,ig1,ig2,iqb)
     .   - B2g0(iq,ig1,ig2,iqb)
     .   - B2g0(iq,ig2,ig1,iqb)

      BRt2g0=wt
*
      return
      end

************************************************************************
* Re of interference of two orderings, summed over helicities
* q g g qb

      function B2g0int(i1,i2,i3,i4,j1,j2,j3,j4)
      use KinData_mod
      implicit real*8 (a-h,o-y)
      implicit complex*16(z)
      dimension jhel(6)

      zwt = (0d0,0d0)

      if(j2.eq.i2.and.j3.eq.i3)then
         jhel(1)=1
         jhel(2)=2
      elseif(j2.eq.i3.and.j3.eq.i2)then
         jhel(1)=2
         jhel(2)=1
      endif

      do ihel=1,2
         ztmp1 = zB2g0(i4,i1,i2,i3,ihel)
         ztmp2 = zB2g0(j4,j1,j2,j3,jhel(ihel))
         zwt = zwt + ztmp1*dconjg(ztmp2)
      enddo

      wt = 2d0*dreal(zwt)

      B2g0int = wt

      return
      end
************************************************************************
* Amplitude with ordering qb q g g g
*
      function zB2g0(i1,i2,i3,i4,iHEL)
      use KinData_mod

      implicit real*8 (a-h,o-y)
      implicit complex*16(z)

      zi=(0d0,1d0)

      za12=zA4(i1,i2)
      za13=zA4(i1,i3)
      za14=zA4(i1,i4)
      za23=zA4(i2,i3)
      za24=zA4(i2,i4)
      za34=zA4(i3,i4)
   
      za21=zA4(i2,i1)
      za31=zA4(i3,i1)
      za41=zA4(i4,i1)
      za32=zA4(i3,i2)
      za42=zA4(i4,i2)
      za43=zA4(i4,i3)
   
      zb12=zB4(i1,i2)
      zb13=zB4(i1,i3)
      zb14=zB4(i1,i4)
      zb23=zB4(i2,i3)
      zb24=zB4(i2,i4)
      zb34=zB4(i3,i4)

      zb21=zB4(i2,i1)
      zb31=zB4(i3,i1)
      zb41=zB4(i4,i1)
      zb32=zB4(i3,i2)
      zb42=zB4(i4,i2)
      zb43=zB4(i4,i3)

      if(ihel.eq.1)then
         zwt=za23**(3)*za13
     . /za12/za23/za34/za41
      endif

      if(ihel.eq.2)then
         zwt=za24**(3)*za14
     . /za12/za23/za34/za41
      endif

      zB2g0=zi*zwt

      return
      end

************************************************************************
*
* three gluon two quark tree level matrix elements
*
*
* summed over colour orderings, summed and squared
*
      function FullB3g0(iq,ig1,ig2,ig3,iqb)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)

c      write(*,*)" hello FullB3g0"
      fac=(4d0*pi*as*nc)**3*(nc**2-1d0)/nc

      FullB3g0=         +B3g0(iq,ig1,ig2,ig3,iqb)
     .                  +B3g0(iq,ig1,ig3,ig2,iqb)
     .                  +B3g0(iq,ig2,ig1,ig3,iqb)
     .                  +B3g0(iq,ig2,ig3,ig1,iqb)
     .                  +B3g0(iq,ig3,ig1,ig2,iqb)
     .                  +B3g0(iq,ig3,ig2,ig1,iqb)
     .-1d0/dble(nc)**2*(Bt3g0(iq,ig1,ig2,ig3,iqb)
     .                  +Bt3g0(iq,ig1,ig3,ig2,iqb)
     .                  +Bt3g0(iq,ig2,ig1,ig3,iqb)
     .                  +Bt3g0(iq,ig2,ig3,ig1,iqb)
     .                  +Bt3g0(iq,ig3,ig1,ig2,iqb)
     .                  +Bt3g0(iq,ig3,ig2,ig1,iqb))
     .+1d0/dble(nc)**(2)*(1d0+1d0/dble(nc)**(2))*Btt3g0(iq,ig1,ig2,ig3,iqb)

      FullB3g0=fac*FullB3g0
*      write(*,*)"FullB3g0",FullB3g0
*
      return
      end
*
************************************************************************
*
* three gluon two quark tree level matrix elements
*
* one colour ordering, summed and squared
*
*                     q  g  g  g  qb
*
*
      function B3g0(iq,ig1,ig2,ig3,iqb)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
*
      sq1 =s5(iq,ig1)
      sq2 =s5(iq,ig2)
      sq3 =s5(iq,ig3)
      s1qb=s5(iqb,ig1)
      s2qb=s5(iqb,ig2)
      s3qb=s5(iqb,ig3)
      s12 =s5(ig1,ig2)
      s13 =s5(ig1,ig3)
      s23 =s5(ig2,ig3)
      sqqb=s5(iq,iqb)

      hs=2d0*(sq1**3*s1qb+sq2**3*s2qb+sq3**3*s3qb
     .       +sq1*s1qb**3+sq2*s2qb**3+sq3*s3qb**3)

      B3g0=hs/sq1/s12/s23/s3qb/sqqb

      B3g0=abs(B3g0)
*      write(*,*)"B3g0",B3g0,iq,ig1,ig2,ig3,iqb
*
      return
      end

*
************************************************************************
*
* three gluon two quark tree level matrix elements
*
*         q ph g1 g2 qb
*
* one colour ordering, summed and squared
*
*
      function Bt3g0(iq,ip,ig2,ig3,iqb)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)

      sq1 =s5(iq,ip)
      sq2 =s5(iq,ig2)
      sq3 =s5(iq,ig3)
      s1qb=s5(iqb,ip)
      s2qb=s5(iqb,ig2)
      s3qb=s5(iqb,ig3)
      s12 =s5(ip,ig2)
      s13 =s5(ip,ig3)
      s23 =s5(ig2,ig3)
      sqqb=s5(iq,iqb)

      hs=2d0*(sq1**3*s1qb+sq2**3*s2qb+sq3**3*s3qb
     .        +sq1*s1qb**3+sq2*s2qb**3+sq3*s3qb**3)

      Bt3g0=hs/sq2/s23/s3qb/sq1/s1qb
      Bt3g0=abs(Bt3g0)
*c      write(*,*)"Bt3g0",Bt3g0,iq,ig1,ig2,ig3,iqb
*
      return
      end
*
************************************************************************
*
* three gluon two quark tree level matrix elements
*
* one colour ordering, summed and squared
*
*
      function Btt3g0(iq,ig1,ig2,ig3,iqb)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
*

      sq1 =s5(iq,ig1)
      sq2 =s5(iq,ig2)
      sq3 =s5(iq,ig3)
      s1qb=s5(iqb,ig1)
      s2qb=s5(iqb,ig2)
      s3qb=s5(iqb,ig3)
      s12 =s5(ig1,ig2)
      s13 =s5(ig1,ig3)
      s23 =s5(ig2,ig3)
      sqqb=s5(iq,iqb)

      hs=2d0*(sq1**3*s1qb+sq2**3*s2qb+sq3**3*s3qb
     .        +sq1*s1qb**3+sq2*s2qb**3+sq3*s3qb**3)

      Btt3g0=hs*sqqb/sq1/sq2/sq3/s1qb/s2qb/s3qb
      Btt3g0=abs(Btt3g0)
*c      write(*,*)"Btt3g0",Btt3g0,iq,ig1,ig2,ig3,iqb
*
      return
      end
************************************************************************
* function used for reduced matrix element for R term
*
      function BRt3g0(iq,ig1,ig2,ig3,iqb)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
*
      wt = Bt3g0(iq,ig1,ig2,ig3,iqb)
     .   + Bt3g0(iq,ig1,ig3,ig2,iqb)
     .   - Btt3g0(iq,ig1,ig2,ig3,iqb)

      BRt3g0 = wt
*
      return
      end
*
************************************************************************
*
* four gluon two quark tree level matrix elements
*
*                     q  g  g  g  g  qb
*
* summed over colour orderings, summed and squared
*
      function FullB4g0(iq,ig1,ig2,ig3,ig4,iqb)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      dimension B(24),Bt(24),Btt(12)

c      write(*,*)" hello FullB4g0"
      fac=(4d0*pi*as*nc)**4*(nc**2-1d0)/nc

* SLC=0d0 ~ leading colour SLC=1d0 ~ Full colour
      SLC=1d0

      call B4g0ME(iq,ig1,ig2,ig3,ig4,iqb,B,Bt,Btt,Bb,R)

      wtb=0d0
      wtbt=0d0
      wtbtt=0d0

      do i=1,24
         wtb=wtb+B(i)
         wtbt=wtbt+Bt(i)
      enddo
      do i=1,12
         wtbtt=wtbtt+Btt(i)
      enddo

* leading colour

      wt = wtb

* subleading colour terms

      wt=wt+(-1d0/dble(nc**2)*(wtbt-Bb+R)
     .       +1d0/dble(nc**4)*(wtbtt-3d0*Bb)
     .       -1d0/dble(nc**6)*Bb)*SLC


      wt=wt*fac
      FullB4g0=wt


*c      write(*,*)"FullB4g0",FullB4g0
*

      return
      end
************************************************************************
* four gluon two quark tree level squared partial amplitude, summed over
* helicities for a set colour ordering
*
      function B4g0(iq,ig1,ig2,ig3,ig4,iqb)
      use KinData_mod
      implicit real*8 (a-h,o-y)
      implicit complex*16 (z)

*      write(*,*)" hello B4g0"

      wt=0d0

      do ii=1,14
         ztmp=zB4g0(iqb,iq,ig1,ig2,ig3,ig4,ii)
         wt=wt+2d0*ztmp*dconjg(ztmp)
      enddo

      B4g0=wt

*      write(*,*)"B4g0",B4g0

      return
      end

************************************************************************
* four gluon two quark tree level squared partial amplitude, summed over
* helicities for a set colour ordering
*
      function BR4g0(iq,ig1,ig2,ig3,ig4,iqb)
      use KinData_mod
      implicit real*8 (a-h,o-y)
      implicit complex*16 (z)
      dimension B(24),Bt(24),Btt(12)

*      write(*,*)" hello BR4g0"

      call B4g0ME(iq,ig1,ig2,ig3,ig4,iqb,B,Bt,Btt,Bb,R)
 
      wt=R
      BR4g0=wt

*      write(*,*)"BR4g0",BR4g0

      return
      end

************************************************************************
* leading colour 2q4g matrix elements, summed over helicities and colour

      function B4g0LC(iq,i1,i2,i3,i4,iqb)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      dimension wt(24)

c      write(*,*)" hello B4g0LC"
      fac=(4d0*pi*as*nc)**4*(nc**2-1d0)/nc

      wtmp=0d0

      wt(1)= B4g0(iq,i1,i2,i3,i4,iqb)
      wt(2)= B4g0(iq,i1,i2,i4,i3,iqb)
      wt(3)= B4g0(iq,i1,i3,i2,i4,iqb)
      wt(4)= B4g0(iq,i1,i3,i4,i2,iqb)
      wt(5)= B4g0(iq,i1,i4,i2,i3,iqb)
      wt(6)= B4g0(iq,i1,i4,i3,i2,iqb)
      wt(7)= B4g0(iq,i2,i1,i3,i4,iqb)
      wt(8)= B4g0(iq,i2,i1,i4,i3,iqb)
      wt(9)= B4g0(iq,i2,i3,i1,i4,iqb)
      wt(10)=B4g0(iq,i2,i3,i4,i1,iqb)
      wt(11)=B4g0(iq,i2,i4,i1,i3,iqb)
      wt(12)=B4g0(iq,i2,i4,i3,i1,iqb)
      wt(13)=B4g0(iq,i3,i1,i2,i4,iqb)
      wt(14)=B4g0(iq,i3,i1,i4,i2,iqb)
      wt(15)=B4g0(iq,i3,i2,i1,i4,iqb)
      wt(16)=B4g0(iq,i3,i2,i4,i1,iqb)
      wt(17)=B4g0(iq,i3,i4,i1,i2,iqb)
      wt(18)=B4g0(iq,i3,i4,i2,i1,iqb)
      wt(19)=B4g0(iq,i4,i1,i2,i3,iqb)
      wt(20)=B4g0(iq,i4,i1,i3,i2,iqb)
      wt(21)=B4g0(iq,i4,i2,i1,i3,iqb)
      wt(22)=B4g0(iq,i4,i2,i3,i1,iqb)
      wt(23)=B4g0(iq,i4,i3,i1,i2,iqb)
      wt(24)=B4g0(iq,i4,i3,i2,i1,iqb)

      do ii=1,24
         wtmp=wtmp+wt(ii)
      enddo

      B4g0LC=wtmp
      return
      end

************************************************************************
* leading colour 2q4g matrix elements, summed over helicities and colour

      function qgFullB4g0LC(iq,i1,i2,i3,i4,iqb)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)

c      write(*,*)" hello qgB4g0LC"

      wt = B4g0qgLC(iq,i1,i2,i3,i4,iqb)
     .    +B4g0qgLC(iq,i1,i4,i2,i3,iqb)
     .    +B4g0qgLC(iq,i1,i3,i4,i2,iqb)

      qgFullB4g0LC=wt
      return
      end
************************************************************************
* leading colour 2q4g matrix elements, summed over helicities and colour
* the relevant block for the qg subtraction

      function B4g0qgLC(iq,i1,i2,i3,i4,iqb)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      dimension wt(24)

      idebug=0

c      write(*,*)" hello B4g0LC"
      fac=(4d0*pi*as*nc)**4*(nc**2-1d0)/nc

      wtmp=0d0

      wt(1)=B4g0(iq,i1,i2,i3,i4,iqb)
      wt(2)=B4g0(iq,i4,i1,i2,i3,iqb)
      wt(3)=B4g0(iq,i3,i4,i1,i2,iqb)
      wt(4)=B4g0(iq,i2,i3,i4,i1,iqb)

      wt(5)=B4g0(iq,i4,i3,i2,i1,iqb)
      wt(6)=B4g0(iq,i1,i4,i3,i2,iqb)
      wt(7)=B4g0(iq,i2,i1,i4,i3,iqb)
      wt(8)=B4g0(iq,i3,i2,i1,i4,iqb)


      do ii=1,8
         wtmp=wtmp+wt(ii)
      enddo

      if(idebug.eq.1)then
      write(*,*)
      write(*,*) iq,i1,i2,i3,i4,iqb,wt(1)
      write(*,*) iq,i4,i1,i2,i3,iqb,wt(2)
      write(*,*) iq,i3,i4,i1,i2,iqb,wt(3)
      write(*,*) iq,i2,i3,i4,i1,iqb,wt(4)
      write(*,*)
      write(*,*) iq,i4,i3,i2,i1,iqb,wt(5)
      write(*,*) iq,i1,i4,i3,i2,iqb,wt(6)
      write(*,*) iq,i2,i1,i4,i3,iqb,wt(7)
      write(*,*) iq,i3,i2,i1,i4,iqb,wt(8)
      write(*,*)
      endif

      wtmp=fac*wtmp

      B4g0qgLC=wtmp
      return
      end
************************************************************************
*
* four gluon two quark tree level matrix elements
*
*
* summed over colour orderings, summed and squared
*
      function B4g0X(iq,ig1,ig2,ig3,ig4,iqb)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      dimension B(24),Bt(24),Btt(12)

      call B4g0ME(iq,ig1,ig2,ig3,ig4,iqb,B,Bt,Btt,Bb,R)

*      write(*,*)" hello B4g0X"

*      B4g0X=  +B4g0(iq,ig1,ig2,ig3,ig4,iqb)
*     .        +B4g0(iq,ig2,ig3,ig4,ig1,iqb)
*     .        +B4g0(iq,ig3,ig4,ig1,ig2,iqb)
*     .        +B4g0(iq,ig4,ig1,ig2,ig3,iqb)

*     .        +B4g0(iq,ig4,ig3,ig2,ig1,iqb)
*     .        +B4g0(iq,ig3,ig2,ig1,ig4,iqb)
*     .        +B4g0(iq,ig2,ig1,ig4,ig3,iqb)
*     .        +B4g0(iq,ig1,ig4,ig3,ig2,iqb)


       if(ig2.eq.4.and.ig3.eq.5.and.ig4.eq.6)then
         B4g0X=B(1)+B(6)+B(8)+B(10)+B(15)+B(17)+B(19)+B(24)
       elseif(ig2.eq.5.and.ig3.eq.4.and.ig4.eq.6)then
         B4g0X=B(3)+B(5)+B(9)+B(11)+B(14)+B(16)+B(20)+B(22)
       elseif(ig2.eq.4.and.ig3.eq.6.and.ig4.eq.5)then
         B4g0X=B(2)+B(4)+B(7)+B(12)+B(13)+B(18)+B(21)+B(23)
       endif
*      write(*,*)"B4g0X",B4g0X

*
      return
      end
************************************************************************
*
      subroutine B4g0ME(iq,ig1,ig2,ig3,ig4,iqb,B,Bt,Btt,Bb,R)
      use KinData_mod
      use Process_mod

      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer aux
      COMPLEX*16 M(24,14),zR(24)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      dimension B(24),Bt(24),Btt(12),j1(3),j2(8),j3(11),j4(23)

      do J=1,14
      M(1,j)=zB4g0(iqb,iq,ig1,ig2,ig3,ig4,J)
      M(2,j)=zB4g0(iqb,iq,ig1,ig2,ig4,ig3,J)
      M(3,j)=zB4g0(iqb,iq,ig1,ig3,ig2,ig4,J)
      M(4,j)=zB4g0(iqb,iq,ig1,ig3,ig4,ig2,J)
      M(5,j)=zB4g0(iqb,iq,ig1,ig4,ig2,ig3,J)
      M(6,j)=zB4g0(iqb,iq,ig1,ig4,ig3,ig2,J)

      M(7,j)=zB4g0(iqb,iq,ig2,ig1,ig3,ig4,J)
      M(8,j)=zB4g0(iqb,iq,ig2,ig1,ig4,ig3,J)
      M(9,j)=zB4g0(iqb,iq,ig2,ig3,ig1,ig4,J)
      M(10,j)=zB4g0(iqb,iq,ig2,ig3,ig4,ig1,J)
      M(11,j)=zB4g0(iqb,iq,ig2,ig4,ig1,ig3,J)
      M(12,j)=zB4g0(iqb,iq,ig2,ig4,ig3,ig1,J)

      M(13,j)=zB4g0(iqb,iq,ig3,ig1,ig2,ig4,J)
      M(14,j)=zB4g0(iqb,iq,ig3,ig1,ig4,ig2,J)
      M(15,j)=zB4g0(iqb,iq,ig3,ig2,ig1,ig4,J)
      M(16,j)=zB4g0(iqb,iq,ig3,ig2,ig4,ig1,J)
      M(17,j)=zB4g0(iqb,iq,ig3,ig4,ig1,ig2,J)
      M(18,j)=zB4g0(iqb,iq,ig3,ig4,ig2,ig1,J)

      M(19,j)=zB4g0(iqb,iq,ig4,ig1,ig2,ig3,J)
      M(20,j)=zB4g0(iqb,iq,ig4,ig1,ig3,ig2,J)
      M(21,j)=zB4g0(iqb,iq,ig4,ig2,ig1,ig3,J)
      M(22,j)=zB4g0(iqb,iq,ig4,ig2,ig3,ig1,J)
      M(23,j)=zB4g0(iqb,iq,ig4,ig3,ig1,ig2,J)
      M(24,j)=zB4g0(iqb,iq,ig4,ig3,ig2,ig1,J)
      enddo

      do II=1,24
         B(II)=0D0
         Bt(II)=0D0
         zR(ii)=(0d0,0d0)
      enddo
      do ii=1,12
         Btt(ii)=0d0
      enddo
      Bb=0d0
      R=0d0
      !> initialise helicity orderings
      call inithel()
      do i=1,14

      call B4g0heliords(i,j1,j2,j3,j4)

* Zero photon squared amplitudes

      do J=1,24
         B(J)=B(J)+2d0*M(J,i)*DCONJG(M(J,i))
      enddo

* One photon squared amplitudes

*3;4,5,6
      ztmp=M(1,i)+M(7,j1(1))+M(9,j1(2))+M(10,j1(3))
      Bt(1)=Bt(1)+2d0*ztmp*dconjg(ztmp)
*3;4,6,5
      ztmp=M(2,i)+M(8,j1(1))+M(11,j1(2))+M(12,j1(3))
      Bt(2)=Bt(2)+2d0*ztmp*dconjg(ztmp)
*3;5,4,6
      ztmp=M(3,i)+M(13,j1(1))+M(15,j1(2))+M(16,j1(3))
      Bt(3)=Bt(3)+2d0*ztmp*dconjg(ztmp)
*3;5,6,4
      ztmp=M(4,i)+M(14,j1(1))+M(17,j1(2))+M(18,j1(3))
      Bt(4)=Bt(4)+2d0*ztmp*dconjg(ztmp)
*3;6,4,5
      ztmp=M(5,i)+M(19,j1(1))+M(21,j1(2))+M(22,j1(3))
      Bt(5)=Bt(5)+2d0*ztmp*dconjg(ztmp)
*3;6,5,4
      ztmp=M(6,i)+M(20,j1(1))+M(23,j1(2))+M(24,j1(3))
      Bt(6)=Bt(6)+2d0*ztmp*dconjg(ztmp)
*4;3,5,6
      ztmp=M(7,i)+M(1,j1(1))+M(3,j1(2))+M(4,j1(3))
      Bt(7)=Bt(7)+2d0*ztmp*dconjg(ztmp)
*4;3,6,5
      ztmp=M(8,i)+M(2,j1(1))+M(5,j1(2))+M(6,j1(3))
      Bt(8)=Bt(8)+2d0*ztmp*dconjg(ztmp)
*4;5,3,6
      ztmp=M(9,i)+M(15,j1(1))+M(13,j1(2))+M(14,j1(3))
      Bt(9)=Bt(9)+2d0*ztmp*dconjg(ztmp)
*4;5,6,3
      ztmp=M(10,i)+M(16,j1(1))+M(18,j1(2))+M(17,j1(3))
      Bt(10)=Bt(10)+2d0*ztmp*dconjg(ztmp)
*4;6,3,5
      ztmp=M(11,i)+M(21,j1(1))+M(19,j1(2))+M(20,j1(3))
      Bt(11)=Bt(11)+2d0*ztmp*dconjg(ztmp)
*4;6,5,3
      ztmp=M(12,i)+M(22,j1(1))+M(24,j1(2))+M(23,j1(3))
      Bt(12)=Bt(12)+2d0*ztmp*dconjg(ztmp)
*5;3,4,6
      ztmp=M(13,i)+M(3,j1(1))+M(1,j1(2))+M(2,j1(3))
      Bt(13)=Bt(13)+2d0*ztmp*dconjg(ztmp)
*5;3,6,4
      ztmp=M(14,i)+M(4,j1(1))+M(6,j1(2))+M(5,j1(3))
      Bt(14)=Bt(14)+2d0*ztmp*dconjg(ztmp)
*5;4,3,6
      ztmp=M(15,i)+M(9,j1(1))+M(7,j1(2))+M(8,j1(3))
      Bt(15)=Bt(15)+2d0*ztmp*dconjg(ztmp)
*5;4,6,3
      ztmp=M(16,i)+M(10,j1(1))+M(12,j1(2))+M(11,j1(3))
      Bt(16)=Bt(16)+2d0*ztmp*dconjg(ztmp)
*5;6,3,4
      ztmp=M(17,i)+M(23,j1(1))+M(20,j1(2))+M(19,j1(3))
      Bt(17)=Bt(17)+2d0*ztmp*dconjg(ztmp)
*5;6,5,4
      ztmp=M(18,i)+M(24,j1(1))+M(22,j1(2))+M(21,j1(3))
      Bt(18)=Bt(18)+2d0*ztmp*dconjg(ztmp)
*6;3,4,5
      ztmp=M(19,i)+M(5,j1(1))+M(2,j1(2))+M(1,j1(3))
      Bt(19)=Bt(19)+2d0*ztmp*dconjg(ztmp)
*6;3,5,4
      ztmp=M(20,i)+M(6,j1(1))+M(4,j1(2))+M(3,j1(3))
      Bt(20)=Bt(20)+2d0*ztmp*dconjg(ztmp)
*6;4,3,5
      ztmp=M(21,i)+M(11,j1(1))+M(8,j1(2))+M(7,j1(3))
      Bt(21)=Bt(21)+2d0*ztmp*dconjg(ztmp)
*6;4,5,3
      ztmp=M(22,i)+M(12,j1(1))+M(10,j1(2))+M(9,j1(3))
      Bt(22)=Bt(22)+2d0*ztmp*dconjg(ztmp)
*6;5,3,4
      ztmp=M(23,i)+M(17,j1(1))+M(14,j1(2))+M(13,j1(3))
      Bt(23)=Bt(23)+2d0*ztmp*dconjg(ztmp)
*6;5,4,3
      ztmp=M(24,i)+M(18,j1(1))+M(16,j1(2))+M(15,j1(3))
      Bt(24)=Bt(24)+2d0*ztmp*dconjg(ztmp)

* Two photon squared amplitudes

*3,4;5,6
      ztmp=M(1,i)+M(3,j3(1))+M(13,j3(2))+M(4,j3(3))+M(14,j3(4))+M(17,j3(5))
     .+M(7,j3(6))+M(9,j3(7))+M(15,j3(8))+M(10,j3(9))+M(16,j3(10))+M(18,j3(11))
      Btt(1)=Btt(1)+2d0*ztmp*dconjg(ztmp)
*3,4;6,5
      ztmp=M(2,i)+M(5,j3(1))+M(19,j3(2))+M(6,j3(3))+M(20,j3(4))+M(23,j3(5))
     .+M(8,j3(6))+M(11,j3(7))+M(21,j3(8))+M(12,j3(9))+M(22,j3(10))+M(24,j3(11))
      Btt(2)=Btt(2)+2d0*ztmp*dconjg(ztmp)
*3,5;4,6
      ztmp=M(3,i)+M(1,j3(1))+M(7,j3(2))+M(2,j3(3))+M(8,j3(4))+M(11,j3(5))
     .+M(13,j3(6))+M(15,j3(7))+M(9,j3(8))+M(16,j3(9))+M(10,j3(10))+M(12,j3(11))
      Btt(3)=Btt(3)+2d0*ztmp*dconjg(ztmp)
*3,5;6,4
      ztmp=M(4,i)+M(6,j3(1))+M(20,j3(2))+M(5,j3(3))+M(19,j3(4))+M(21,j3(5))
     .+M(14,j3(6))+M(17,j3(7))+M(23,j3(8))+M(18,j3(9))+M(24,j3(10))+M(22,j3(11))
      Btt(4)=Btt(4)+2d0*ztmp*dconjg(ztmp)
*3,6;4,5
      ztmp=M(5,i)+M(2,j3(1))+M(8,j3(2))+M(1,j3(3))+M(7,j3(4))+M(9,j3(5))
     .+M(19,j3(6))+M(21,j3(7))+M(11,j3(8))+M(22,j3(9))+M(12,j3(10))+M(10,j3(11))
      Btt(5)=Btt(5)+2d0*ztmp*dconjg(ztmp)
*3,6;5,4
      ztmp=M(6,i)+M(4,j3(1))+M(14,j3(2))+M(3,j3(3))+M(13,j3(4))+M(15,j3(5))
     .+M(20,j3(6))+M(23,j3(7))+M(17,j3(8))+M(24,j3(9))+M(18,j3(10))+M(16,j3(11))
      Btt(6)=Btt(6)+2d0*ztmp*dconjg(ztmp)
*4,5;3,6
      ztmp=M(9,i)+M(7,j3(1))+M(1,j3(2))+M(8,j3(3))+M(2,j3(4))+M(5,j3(5))
     .+M(15,j3(6))+M(13,j3(7))+M(3,j3(8))+M(14,j3(9))+M(4,j3(10))+M(6,j3(11))
      Btt(7)=Btt(7)+2d0*ztmp*dconjg(ztmp)
*4,5;6,3
      ztmp=M(10,i)+M(12,j3(1))+M(22,j3(2))+M(11,j3(3))+M(21,j3(4))+M(19,j3(5))
     .+M(16,j3(6))+M(18,j3(7))+M(24,j3(8))+M(17,j3(9))+M(23,j3(10))+M(20,j3(11))
      Btt(8)=Btt(8)+2d0*ztmp*dconjg(ztmp)
*4,6;3,5
      ztmp=M(11,i)+M(8,j3(1))+M(2,j3(2))+M(7,j3(3))+M(1,j3(4))+M(3,j3(5))
     .+M(21,j3(6))+M(19,j3(7))+M(5,j3(8))+M(20,j3(9))+M(6,j3(10))+M(4,j3(11))
      Btt(9)=Btt(9)+2d0*ztmp*dconjg(ztmp)
*4,6;5,3
      ztmp=M(12,i)+M(10,j3(1))+M(16,j3(2))+M(9,j3(3))+M(15,j3(4))+M(13,j3(5))
     .+M(22,j3(6))+M(24,j3(7))+M(18,j3(8))+M(23,j3(9))+M(17,j3(10))+M(14,j3(11))
      Btt(10)=Btt(10)+2d0*ztmp*dconjg(ztmp)
*5,6;3,4
      ztmp=M(17,i)+M(14,j3(1))+M(4,j3(2))+M(13,j3(3))+M(3,j3(4))+M(1,j3(5))
     .+M(23,j3(6))+M(20,j3(7))+M(6,j3(8))+M(19,j3(9))+M(5,j3(10))+M(2,j3(11))
      Btt(11)=Btt(11)+2d0*ztmp*dconjg(ztmp)
*5,6;4,3
      ztmp=M(18,i)+M(16,j3(1))+M(10,j3(2))+M(15,j3(3))+M(9,j3(4))+M(7,j3(5))
     .+M(24,j3(6))+M(22,j3(7))+M(12,j3(8))+M(21,j3(9))+M(11,j3(10))+M(8,j3(11))
      Btt(12)=Btt(12)+2d0*ztmp*dconjg(ztmp)

* Remainder interferences

*3,4,5,6
      ztmp=M(8,j2(1))+M(11,j2(2))+M(12,j2(3))+M(14,j2(4))+M(16,j2(5))+M(20,j2(6))+M(21,j2(7))+M(24,j2(8))
      zR(1)=zR(1)+2d0*ztmp*dconjg(M(1,i))
*3,4,6,5
      ztmp=M(7,j2(1))+M(9,j2(2))+M(10,j2(3))+M(20,j2(4))+M(22,j2(5))+M(14,j2(6))+M(15,j2(7))+M(18,j2(8))
      zR(2)=zR(2)+2d0*ztmp*dconjg(M(2,i))
*3,5,4,6
      ztmp=M(14,j2(1))+M(17,j2(2))+M(18,j2(3))+M(8,j2(4))+M(10,j2(5))+M(19,j2(6))+M(23,j2(7))+M(22,j2(8))
      zR(3)=zR(3)+2d0*ztmp*dconjg(M(3,i))
*3,5,6,4
      ztmp=M(13,j2(1))+M(15,j2(2))+M(16,j2(3))+M(19,j2(4))+M(24,j2(5))+M(8,j2(6))+M(9,j2(7))+M(12,j2(8))
      zR(4)=zR(4)+2d0*ztmp*dconjg(M(4,i))
*3,6,4,5
      ztmp=M(20,j2(1))+M(23,j2(2))+M(24,j2(3))+M(7,j2(4))+M(12,j2(5))+M(13,j2(6))+M(17,j2(7))+M(16,j2(8))
      zR(5)=zR(5)+2d0*ztmp*dconjg(M(5,i))
*3,6,5,4
      ztmp=M(19,j2(1))+M(21,j2(2))+M(22,j2(3))+M(13,j2(4))+M(18,j2(5))+M(7,j2(6))+M(11,j2(7))+M(10,j2(8))
      zR(6)=zR(6)+2d0*ztmp*dconjg(M(6,i))
*4,3,5,6
      ztmp=M(2,j2(1))+M(5,j2(2))+M(6,j2(3))+M(16,j2(4))+M(14,j2(5))+M(22,j2(6))+M(19,j2(7))+M(23,j2(8))
      zR(7)=zR(7)+2d0*ztmp*dconjg(M(7,i))
*4,3,6,5
      ztmp=M(1,j2(1))+M(3,j2(2))+M(4,j2(3))+M(22,j2(4))+M(20,j2(5))+M(16,j2(6))+M(13,j2(7))+M(17,j2(8))
      zR(8)=zR(8)+2d0*ztmp*dconjg(M(8,i))
*4,5,3,6
      ztmp=M(16,j2(1))+M(18,j2(2))+M(17,j2(3))+M(2,j2(4))+M(4,j2(5))+M(21,j2(6))+M(24,j2(7))+M(20,j2(8))
      zR(9)=zR(9)+2d0*ztmp*dconjg(M(9,i))
*4,5,6,3
      ztmp=M(15,j2(1))+M(13,j2(2))+M(14,j2(3))+M(21,j2(4))+M(23,j2(5))+M(2,j2(6))+M(3,j2(7))+M(6,j2(8))
      zR(10)=zR(10)+2d0*ztmp*dconjg(M(10,i))
*4,6,3,5
      ztmp=M(22,j2(1))+M(24,j2(2))+M(23,j2(3))+M(1,j2(4))+M(6,j2(5))+M(15,j2(6))+M(18,j2(7))+M(14,j2(8))
      zR(11)=zR(11)+2d0*ztmp*dconjg(M(11,i))
*4,6,5,3
      ztmp=M(21,j2(1))+M(19,j2(2))+M(20,j2(3))+M(15,j2(4))+M(17,j2(5))+M(1,j2(6))+M(5,j2(7))+M(4,j2(8))
      zR(12)=zR(12)+2d0*ztmp*dconjg(M(12,i))
*5,3,4,6
      ztmp=M(4,j2(1))+M(6,j2(2))+M(5,j2(3))+M(10,j2(4))+M(8,j2(5))+M(24,j2(6))+M(20,j2(7))+M(21,j2(8))
      zR(13)=zR(13)+2d0*ztmp*dconjg(M(13,i))
*5,3,6,4
      ztmp=M(3,j2(1))+M(1,j2(2))+M(2,j2(3))+M(24,j2(4))+M(19,j2(5))+M(10,j2(6))+M(7,j2(7))+M(11,j2(8))
      zR(14)=zR(14)+2d0*ztmp*dconjg(M(14,i))
*5,4,3,6
      ztmp=M(10,j2(1))+M(12,j2(2))+M(11,j2(3))+M(4,j2(4))+M(2,j2(5))+M(23,j2(6))+M(22,j2(7))+M(19,j2(8))
      zR(15)=zR(15)+2d0*ztmp*dconjg(M(15,i))
*5,4,6,3
      ztmp=M(9,j2(1))+M(7,j2(2))+M(8,j2(3))+M(23,j2(4))+M(21,j2(5))+M(4,j2(6))+M(1,j2(7))+M(5,j2(8))
      zR(16)=zR(16)+2d0*ztmp*dconjg(M(16,i))
*5,6,3,4
      ztmp=M(24,j2(1))+M(22,j2(2))+M(21,j2(3))+M(3,j2(4))+M(5,j2(5))+M(9,j2(6))+M(12,j2(7))+M(8,j2(8))
      zR(17)=zR(17)+2d0*ztmp*dconjg(M(17,i))
*5,6,5,4
      ztmp=M(23,j2(1))+M(20,j2(2))+M(19,j2(3))+M(9,j2(4))+M(11,j2(5))+M(3,j2(6))+M(6,j2(7))+M(2,j2(8))
      zR(18)=zR(18)+2d0*ztmp*dconjg(M(18,i))
*6,3,4,5
      ztmp=M(6,j2(1))+M(4,j2(2))+M(3,j2(3))+M(12,j2(4))+M(7,j2(5))+M(18,j2(6))+M(14,j2(7))+M(15,j2(8))
      zR(19)=zR(19)+2d0*ztmp*dconjg(M(19,i))
*6,3,5,4
      ztmp=M(5,j2(1))+M(2,j2(2))+M(1,j2(3))+M(18,j2(4))+M(13,j2(5))+M(12,j2(6))+M(8,j2(7))+M(9,j2(8))
      zR(20)=zR(20)+2d0*ztmp*dconjg(M(20,i))
*6,4,3,5
      ztmp=M(12,j2(1))+M(10,j2(2))+M(9,j2(3))+M(6,j2(4))+M(1,j2(5))+M(17,j2(6))+M(16,j2(7))+M(13,j2(8))
      zR(21)=zR(21)+2d0*ztmp*dconjg(M(21,i))
*6,4,5,3
      ztmp=M(11,j2(1))+M(8,j2(2))+M(7,j2(3))+M(17,j2(4))+M(15,j2(5))+M(6,j2(6))+M(2,j2(7))+M(3,j2(8))
      zR(22)=zR(22)+2d0*ztmp*dconjg(M(22,i))
*6,5,3,4
      ztmp=M(18,j2(1))+M(16,j2(2))+M(15,j2(3))+M(5,j2(4))+M(3,j2(5))+M(11,j2(6))+M(10,j2(7))+M(7,j2(8))
      zR(23)=zR(23)+2d0*ztmp*dconjg(M(23,i))
*6,5,4,3
      ztmp=M(17,j2(1))+M(14,j2(2))+M(13,j2(3))+M(11,j2(4))+M(9,j2(5))+M(5,j2(6))+M(4,j2(7))+M(1,j2(8))
      zR(24)=zR(24)+2d0*ztmp*dconjg(M(24,i))

* four photon squared amplitude
      ztmp=M(1,i)
      DO j=2,24
      ztmp=ztmp+M(j,j4(j-1))
      ENDDO
      Bb=Bb+2d0*ztmp*dconjg(ztmp)

      enddo

      ztmp2=(0d0,0d0)
      do ii=1,24
         ztmp2=ztmp2+zR(ii)
      enddo
      R=dreal(ztmp2)

   

      return
      end
************************************************************************
* 1 photon 3 gluon squared amplitude, summed over helicity for a single ordering

      function Bt4g0ord(iq,ig1,ig2,ig3,ig4,iqb)
      use KinData_mod
      use Process_mod

      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      dimension zM(24,14),j1(3),j2(8),j3(11),j4(23)

      do J=1,14
      zM(1,j)=zB4g0(iqb,iq,ig1,ig2,ig3,ig4,J)
      zM(2,j)=zB4g0(iqb,iq,ig1,ig2,ig4,ig3,J)
      zM(3,j)=zB4g0(iqb,iq,ig1,ig3,ig2,ig4,J)
      zM(4,j)=zB4g0(iqb,iq,ig1,ig3,ig4,ig2,J)
      zM(5,j)=zB4g0(iqb,iq,ig1,ig4,ig2,ig3,J)
      zM(6,j)=zB4g0(iqb,iq,ig1,ig4,ig3,ig2,J)

      zM(7,j)=zB4g0(iqb,iq,ig2,ig1,ig3,ig4,J)
      zM(8,j)=zB4g0(iqb,iq,ig2,ig1,ig4,ig3,J)
      zM(9,j)=zB4g0(iqb,iq,ig2,ig3,ig1,ig4,J)
      zM(10,j)=zB4g0(iqb,iq,ig2,ig3,ig4,ig1,J)
      zM(11,j)=zB4g0(iqb,iq,ig2,ig4,ig1,ig3,J)
      zM(12,j)=zB4g0(iqb,iq,ig2,ig4,ig3,ig1,J)

      zM(13,j)=zB4g0(iqb,iq,ig3,ig1,ig2,ig4,J)
      zM(14,j)=zB4g0(iqb,iq,ig3,ig1,ig4,ig2,J)
      zM(15,j)=zB4g0(iqb,iq,ig3,ig2,ig1,ig4,J)
      zM(16,j)=zB4g0(iqb,iq,ig3,ig2,ig4,ig1,J)
      zM(17,j)=zB4g0(iqb,iq,ig3,ig4,ig1,ig2,J)
      zM(18,j)=zB4g0(iqb,iq,ig3,ig4,ig2,ig1,J)

      zM(19,j)=zB4g0(iqb,iq,ig4,ig1,ig2,ig3,J)
      zM(20,j)=zB4g0(iqb,iq,ig4,ig1,ig3,ig2,J)
      zM(21,j)=zB4g0(iqb,iq,ig4,ig2,ig1,ig3,J)
      zM(22,j)=zB4g0(iqb,iq,ig4,ig2,ig3,ig1,J)
      zM(23,j)=zB4g0(iqb,iq,ig4,ig3,ig1,ig2,J)
      zM(24,j)=zB4g0(iqb,iq,ig4,ig3,ig2,ig1,J)
      enddo

      zwt = (0d0,0d0)


      !> initialise helicity orderings
      call inithel()
      do i=1,14

      call B4g0heliords(i,j1,j2,j3,j4)

      ztmp=zM(1,i)+zM(7,j1(1))+zM(9,j1(2))+zM(10,j1(3))
      zwt=zwt+ztmp*dconjg(ztmp)

      enddo

      wt = 2d0*dreal(zwt)
      Bt4g0ord=wt


      return
      end


************************************************************************
* 1 photon 3 gluon squared amplitude, summed over helicity for a single ordering
* ig1 is photon
      function Bt4g0(iq,ig1,ig2,ig3,ig4,iqb)
      use KinData_mod
      use Constants_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      dimension zM(4,14)
      integer j1(3,14)
      data j1 /2,3,4,
     .     1,1,1,
     .     3,2,2,
     .     4,4,3,
     .     5,5,6,
     .     6,7,7,
     .     8,8,8,
     .     7,6,5,
     .     9,13,11,
     .     10,14,12,
     .     14,10,10,
     .     13,9,9,
     .     12,12,14,
     .     11,11,13/

      do J=1,14
      zM(1,j)=zB4g0(iqb,iq,ig1,ig2,ig3,ig4,J)
      zM(2,j)=zB4g0(iqb,iq,ig2,ig1,ig3,ig4,J)
      zM(3,j)=zB4g0(iqb,iq,ig2,ig3,ig1,ig4,J)
      zM(4,j)=zB4g0(iqb,iq,ig2,ig3,ig4,ig1,J)
      enddo
 
      zwt = (0d0,0d0)
      do i=1,14

      ztmp=zM(1,i)+zM(2,j1(1,i))+zM(3,j1(2,i))+zM(4,j1(3,i))
      zwt=zwt+ztmp*dconjg(ztmp)
      
      enddo
      

      wt = 2d0*dreal(zwt)
      Bt4g0=wt

      return
      end



************************************************************************
* 2 photon 2 gluon squared amplitude, summed over helicity for a single ordering

      function Btt4g0ord(iq,ig1,ig2,ig3,ig4,iqb)
      use KinData_mod
      use Process_mod

      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      dimension zM(24,14),j1(3),j2(8),j3(11),j4(23)

      do J=1,14
      zM(1,j)=zB4g0(iqb,iq,ig1,ig2,ig3,ig4,J)
      zM(2,j)=zB4g0(iqb,iq,ig1,ig2,ig4,ig3,J)
      zM(3,j)=zB4g0(iqb,iq,ig1,ig3,ig2,ig4,J)
      zM(4,j)=zB4g0(iqb,iq,ig1,ig3,ig4,ig2,J)
      zM(5,j)=zB4g0(iqb,iq,ig1,ig4,ig2,ig3,J)
      zM(6,j)=zB4g0(iqb,iq,ig1,ig4,ig3,ig2,J)

      zM(7,j)=zB4g0(iqb,iq,ig2,ig1,ig3,ig4,J)
      zM(8,j)=zB4g0(iqb,iq,ig2,ig1,ig4,ig3,J)
      zM(9,j)=zB4g0(iqb,iq,ig2,ig3,ig1,ig4,J)
      zM(10,j)=zB4g0(iqb,iq,ig2,ig3,ig4,ig1,J)
      zM(11,j)=zB4g0(iqb,iq,ig2,ig4,ig1,ig3,J)
      zM(12,j)=zB4g0(iqb,iq,ig2,ig4,ig3,ig1,J)

      zM(13,j)=zB4g0(iqb,iq,ig3,ig1,ig2,ig4,J)
      zM(14,j)=zB4g0(iqb,iq,ig3,ig1,ig4,ig2,J)
      zM(15,j)=zB4g0(iqb,iq,ig3,ig2,ig1,ig4,J)
      zM(16,j)=zB4g0(iqb,iq,ig3,ig2,ig4,ig1,J)
      zM(17,j)=zB4g0(iqb,iq,ig3,ig4,ig1,ig2,J)
      zM(18,j)=zB4g0(iqb,iq,ig3,ig4,ig2,ig1,J)

      zM(19,j)=zB4g0(iqb,iq,ig4,ig1,ig2,ig3,J)
      zM(20,j)=zB4g0(iqb,iq,ig4,ig1,ig3,ig2,J)
      zM(21,j)=zB4g0(iqb,iq,ig4,ig2,ig1,ig3,J)
      zM(22,j)=zB4g0(iqb,iq,ig4,ig2,ig3,ig1,J)
      zM(23,j)=zB4g0(iqb,iq,ig4,ig3,ig1,ig2,J)
      zM(24,j)=zB4g0(iqb,iq,ig4,ig3,ig2,ig1,J)
      enddo

      zwt = (0d0,0d0)


      !> initialise helicity orderings
      call inithel()
      do i=1,14

      call B4g0heliords(i,j1,j2,j3,j4)

      ztmp = zM(1,i) + zM(3,j3(1)) + zM(13,j3(2)) + zM(4,j3(3)) + zM(14,j3(4)) + zM(17,j3(5))
     .     + zM(7,j3(6)) + zM(9,j3(7)) + zM(15,j3(8)) + zM(10,j3(9)) + zM(16,j3(10)) + zM(18,j3(11))
      zwt = zwt + ztmp*dconjg(ztmp)

      enddo

      wt = 2d0*dreal(zwt)
      Btt4g0ord=wt

      return
      end


************************************************************************
* 2 photon 2 gluon squared amplitude

      function Btt4g0(iq,ig1,ig2,ig3,ig4,iqb)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      dimension zM(12,14),j3(11,14)
      data j3 /1,  2,  1,  2,  3,  2,  3,  3,  4,  4,  4,
     .  3,  3,  4,  4,  4,  1,  1,  2,  1,  2,  3,
     .  2,  1,  2,  1,  1,  3,  2,  1,  2,  1,  1,
     .  4,  4,  3,  3,  2,  4,  4,  4,  3,  3,  2,
     .  5,  5,  6,  6,  7,  5,  5,  5,  6,  6,  7, 
     .  7,  8,  7,  8,  8,  6,  7,  8,  7,  8,  8, 
     .  6,  6,  5,  5,  5,  8,  8,  7,  8,  7,  6,
     .  8,  7,  8,  7,  6,  7,  6,  6,  5,  5,  5,
     .  13, 12, 11, 14, 10,  9, 13, 12, 11, 14, 10, 
     .  14, 11, 12, 13,  9, 10, 14, 11, 12, 13,  9, 
     .  11, 14, 13, 12, 12, 14, 10, 10, 10, 10, 14, 
     .  12, 13, 14, 11, 11, 13,  9,  9,  9,  9, 13,
     .  9,  9,  9,  9, 13, 12, 12, 13, 14, 11, 11, 
     .  10, 10, 10, 10, 14, 11, 11, 14, 13, 12, 12/
 
      do J=1,14
      zM(1,j)=zB4g0(iqb,iq,ig1,ig2,ig3,ig4,J)
      zM(2,j)=zB4g0(iqb,iq,ig1,ig3,ig2,ig4,J)
      zM(3,j)=zB4g0(iqb,iq,ig1,ig3,ig4,ig2,J)

      zM(4,j)=zB4g0(iqb,iq,ig2,ig1,ig3,ig4,J)
      zM(5,j)=zB4g0(iqb,iq,ig2,ig3,ig1,ig4,J)
      zM(6,j)=zB4g0(iqb,iq,ig2,ig3,ig4,ig1,J)

      zM(7,j)=zB4g0(iqb,iq,ig3,ig1,ig2,ig4,J)
      zM(8,j)=zB4g0(iqb,iq,ig3,ig1,ig4,ig2,J)
      zM(9,j)=zB4g0(iqb,iq,ig3,ig2,ig1,ig4,J)
      zM(10,j)=zB4g0(iqb,iq,ig3,ig2,ig4,ig1,J)
      zM(11,j)=zB4g0(iqb,iq,ig3,ig4,ig1,ig2,J)
      zM(12,j)=zB4g0(iqb,iq,ig3,ig4,ig2,ig1,J)

      enddo

      zwt = (0d0,0d0)
      do i=1,14
      ztmp = zM(1,i) + zM(2,j3(1,i)) + zM(7,j3(2,i)) + zM(3,j3(3,i)) + zM(8,j3(4,i)) + zM(11,j3(5,i))
     .     + zM(4,j3(6,i)) + zM(5,j3(7,i)) + zM(9,j3(8,i)) + zM(6,j3(9,i)) + zM(10,j3(10,i)) + zM(12,j3(11,i))
      zwt = zwt + ztmp*dconjg(ztmp)
      enddo

      wt = 2d0*dreal(zwt)
      Btt4g0=wt

      return
      end
      
      function Bttt4g0(iq,ig1,ig2,ig3,ig4,iqb)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      dimension zM(24,14),j4(23,14)

      data j4 / 1, 1, 1, 1, 1, 2, 2, 3, 4, 3, 4, 2, 2, 3, 4, 3, 4, 2, 2, 3, 4, 3, 4,
     .     2,3,4,3,4,1,1,1,1,1,1,3,4,2,2,4,3,3,4,2,2,4,3,
     .      4,2,2,4,3,3,4,2,2,4,3,1,1,1,1,1,1,4,3,4,3,2,2,
     .     3,4,3,2,2,4,3,4,3,2,2,4,3,4,3,2,2,1,1,1,1,1,1,
     .     6,5,6,7,7,5,6,5,6,7,7,5,6,5,6,7,7,8,8,8,8,8,8,
     .     5,7,7,5,6,6,5,7,7,5,6,8,8,8,8,8,8,5,6,5,6,7,7,
     .     7, 6, 5, 6, 5, 8, 8, 8, 8, 8, 8, 6, 5, 7, 7, 5, 6, 6, 5, 7, 7, 5, 6, 
     .     8,8,8,8,8,7,7,6,5,6,5,7,7,6,5,6,5,7,7,6,5,6,5,
     .     9,13,11,13,11,9, 9,13,11,13,11,12,14,12,14,10,10,12,14,12,14,10,10,
     .     10,14,12,14,12,10,10,14,12,14,12,11,13,11,13, 9, 9,11,13,11,13, 9, 9,
     .     13,11,13, 9, 9,14,12,10,10,12,14,14,12,10,10,12,14, 9, 9,13,11,13,11,
     .     14,12,14,10,10,13,11, 9, 9,11,13,13,11, 9, 9,11,13,10,10,14,12,14,12,
     .     11, 9, 9,11,13,12,14,12,14,10,10, 9, 9,13,11,13,11,14,12,10,10,12,14,
     .     12,10,10,12,14,11,13,11,13, 9, 9,10,10,14,12,14,12,13,11, 9, 9,11,13 /

      do J=1,14
      zM(1,j)=zB4g0(iqb,iq,ig1,ig2,ig3,ig4,J)
      zM(2,j)=zB4g0(iqb,iq,ig1,ig2,ig4,ig3,J)
      zM(3,j)=zB4g0(iqb,iq,ig1,ig3,ig2,ig4,J)
      zM(4,j)=zB4g0(iqb,iq,ig1,ig3,ig4,ig2,J)
      zM(5,j)=zB4g0(iqb,iq,ig1,ig4,ig2,ig3,J)
      zM(6,j)=zB4g0(iqb,iq,ig1,ig4,ig3,ig2,J)

      zM(7,j)=zB4g0(iqb,iq,ig2,ig1,ig3,ig4,J)
      zM(8,j)=zB4g0(iqb,iq,ig2,ig1,ig4,ig3,J)
      zM(9,j)=zB4g0(iqb,iq,ig2,ig3,ig1,ig4,J)
      zM(10,j)=zB4g0(iqb,iq,ig2,ig3,ig4,ig1,J)
      zM(11,j)=zB4g0(iqb,iq,ig2,ig4,ig1,ig3,J)
      zM(12,j)=zB4g0(iqb,iq,ig2,ig4,ig3,ig1,J)

      zM(13,j)=zB4g0(iqb,iq,ig3,ig1,ig2,ig4,J)
      zM(14,j)=zB4g0(iqb,iq,ig3,ig1,ig4,ig2,J)
      zM(15,j)=zB4g0(iqb,iq,ig3,ig2,ig1,ig4,J)
      zM(16,j)=zB4g0(iqb,iq,ig3,ig2,ig4,ig1,J)
      zM(17,j)=zB4g0(iqb,iq,ig3,ig4,ig1,ig2,J)
      zM(18,j)=zB4g0(iqb,iq,ig3,ig4,ig2,ig1,J)

      zM(19,j)=zB4g0(iqb,iq,ig4,ig1,ig2,ig3,J)
      zM(20,j)=zB4g0(iqb,iq,ig4,ig1,ig3,ig2,J)
      zM(21,j)=zB4g0(iqb,iq,ig4,ig2,ig1,ig3,J)
      zM(22,j)=zB4g0(iqb,iq,ig4,ig2,ig3,ig1,J)
      zM(23,j)=zB4g0(iqb,iq,ig4,ig3,ig1,ig2,J)
      zM(24,j)=zB4g0(iqb,iq,ig4,ig3,ig2,ig1,J)
      enddo

      zwt = (0d0,0d0)

      do i=1,14
      ztmp=zM(1,i)
      do j=2,24
         ztmp=ztmp+zM(j,j4(j-1,i))
      enddo
      zwt = zwt + ztmp*dconjg(ztmp)

      enddo

      wt = 2d0*dreal(zwt)
      Bttt4g0=wt

      return
      end
************************************************************************
* 4 photon squared amplitude, summed over helicity

      function Bb4g0ord(iq,ig1,ig2,ig3,ig4,iqb)
      use KinData_mod
      use Process_mod

      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      dimension zM(24,14),j1(3),j2(8),j3(11),j4(23)

      do J=1,14
      zM(1,j)=zB4g0(iqb,iq,ig1,ig2,ig3,ig4,J)
      zM(2,j)=zB4g0(iqb,iq,ig1,ig2,ig4,ig3,J)
      zM(3,j)=zB4g0(iqb,iq,ig1,ig3,ig2,ig4,J)
      zM(4,j)=zB4g0(iqb,iq,ig1,ig3,ig4,ig2,J)
      zM(5,j)=zB4g0(iqb,iq,ig1,ig4,ig2,ig3,J)
      zM(6,j)=zB4g0(iqb,iq,ig1,ig4,ig3,ig2,J)

      zM(7,j)=zB4g0(iqb,iq,ig2,ig1,ig3,ig4,J)
      zM(8,j)=zB4g0(iqb,iq,ig2,ig1,ig4,ig3,J)
      zM(9,j)=zB4g0(iqb,iq,ig2,ig3,ig1,ig4,J)
      zM(10,j)=zB4g0(iqb,iq,ig2,ig3,ig4,ig1,J)
      zM(11,j)=zB4g0(iqb,iq,ig2,ig4,ig1,ig3,J)
      zM(12,j)=zB4g0(iqb,iq,ig2,ig4,ig3,ig1,J)

      zM(13,j)=zB4g0(iqb,iq,ig3,ig1,ig2,ig4,J)
      zM(14,j)=zB4g0(iqb,iq,ig3,ig1,ig4,ig2,J)
      zM(15,j)=zB4g0(iqb,iq,ig3,ig2,ig1,ig4,J)
      zM(16,j)=zB4g0(iqb,iq,ig3,ig2,ig4,ig1,J)
      zM(17,j)=zB4g0(iqb,iq,ig3,ig4,ig1,ig2,J)
      zM(18,j)=zB4g0(iqb,iq,ig3,ig4,ig2,ig1,J)

      zM(19,j)=zB4g0(iqb,iq,ig4,ig1,ig2,ig3,J)
      zM(20,j)=zB4g0(iqb,iq,ig4,ig1,ig3,ig2,J)
      zM(21,j)=zB4g0(iqb,iq,ig4,ig2,ig1,ig3,J)
      zM(22,j)=zB4g0(iqb,iq,ig4,ig2,ig3,ig1,J)
      zM(23,j)=zB4g0(iqb,iq,ig4,ig3,ig1,ig2,J)
      zM(24,j)=zB4g0(iqb,iq,ig4,ig3,ig2,ig1,J)
      enddo

      zwt = (0d0,0d0)


      !> initialise helicity orderings
      call inithel()
      do i=1,14

      call B4g0heliords(i,j1,j2,j3,j4)

      ztmp=zM(1,i)
      do j=2,24
         ztmp=ztmp+zM(j,j4(j-1))
      enddo
      zwt = zwt + ztmp*dconjg(ztmp)

      enddo

      wt = 2d0*dreal(zwt)
      Bb4g0ord=wt

      return
      end

************************************************************************
* R term, summed over helicity for a single ordering

      function BR4g0ord(iq,ig1,ig2,ig3,ig4,iqb)
      use KinData_mod

      implicit double precision (a-h,o-y)
!      implicit complex*16 (z)
    
      wt =   B4g0int(iq,ig1,ig2,ig3,ig4,iqb,iq,ig2,ig1,ig4,ig3,iqb)
     .     + B4g0int(iq,ig1,ig2,ig3,ig4,iqb,iq,ig2,ig4,ig1,ig3,iqb)
     .     + B4g0int(iq,ig1,ig2,ig3,ig4,iqb,iq,ig2,ig4,ig3,ig1,iqb)
     .     + B4g0int(iq,ig1,ig2,ig3,ig4,iqb,iq,ig3,ig1,ig4,ig2,iqb)
     .     + B4g0int(iq,ig1,ig2,ig3,ig4,iqb,iq,ig3,ig2,ig4,ig1,iqb)
     .     + B4g0int(iq,ig1,ig2,ig3,ig4,iqb,iq,ig4,ig1,ig3,ig2,iqb)
     .     + B4g0int(iq,ig1,ig2,ig3,ig4,iqb,iq,ig4,ig2,ig1,ig3,iqb)
     .     + B4g0int(iq,ig1,ig2,ig3,ig4,iqb,iq,ig4,ig3,ig2,ig1,iqb)

      BR4g0ord = wt

      return
      end

************************************************************************
* general interference of b4g0 amplitudes, summed over helicity

      function B4g0int(iq,ig1,ig2,ig3,ig4,iqb,jq,jg1,jg2,jg3,jg4,jqb)
      use KinData_mod

      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      dimension zM1(14),zM2(14),j4(23,14)
      data j4 / 1, 1, 1, 1, 1, 2, 2, 3, 4, 3, 4, 2, 2, 3, 4, 3, 4, 2, 2, 3, 4, 3, 4,
     .     2,3,4,3,4,1,1,1,1,1,1,3,4,2,2,4,3,3,4,2,2,4,3,
     .      4,2,2,4,3,3,4,2,2,4,3,1,1,1,1,1,1,4,3,4,3,2,2,
     .     3,4,3,2,2,4,3,4,3,2,2,4,3,4,3,2,2,1,1,1,1,1,1,
     .     6,5,6,7,7,5,6,5,6,7,7,5,6,5,6,7,7,8,8,8,8,8,8,
     .     5,7,7,5,6,6,5,7,7,5,6,8,8,8,8,8,8,5,6,5,6,7,7,
     .     7, 6, 5, 6, 5, 8, 8, 8, 8, 8, 8, 6, 5, 7, 7, 5, 6, 6, 5, 7, 7, 5, 6, 
     .     8,8,8,8,8,7,7,6,5,6,5,7,7,6,5,6,5,7,7,6,5,6,5,
     .     9,13,11,13,11,9, 9,13,11,13,11,12,14,12,14,10,10,12,14,12,14,10,10,
     .     10,14,12,14,12,10,10,14,12,14,12,11,13,11,13, 9, 9,11,13,11,13, 9, 9,
     .     13,11,13, 9, 9,14,12,10,10,12,14,14,12,10,10,12,14, 9, 9,13,11,13,11,
     .     14,12,14,10,10,13,11, 9, 9,11,13,13,11, 9, 9,11,13,10,10,14,12,14,12,
     .     11, 9, 9,11,13,12,14,12,14,10,10, 9, 9,13,11,13,11,14,12,10,10,12,14,
     .     12,10,10,12,14,11,13,11,13, 9, 9,10,10,14,12,14,12,13,11, 9, 9,11,13 /

      do j=1,14

      zM1(j)=zB4g0(iqb,iq,ig1,ig2,ig3,ig4,j)
      
      if(jg1.eq.ig1.and.jg2.eq.ig2.and.jg3.eq.ig3.and.jg4.eq.ig4)then
         zM2(j)=zB4g0(jqb,jq,jg1,jg2,jg3,jg4,j)
      elseif(jg1.eq.ig1.and.jg2.eq.ig2.and.jg3.eq.ig4.and.jg4.eq.ig3)then
         zM2(j)=zB4g0(jqb,jq,jg1,jg2,jg3,jg4,j4(1,j))
      elseif(jg1.eq.ig1.and.jg2.eq.ig3.and.jg3.eq.ig2.and.jg4.eq.ig4)then
         zM2(j)=zB4g0(jqb,jq,jg1,jg2,jg3,jg4,j4(2,j))
      elseif(jg1.eq.ig1.and.jg2.eq.ig3.and.jg3.eq.ig4.and.jg4.eq.ig2)then
         zM2(j)=zB4g0(jqb,jq,jg1,jg2,jg3,jg4,j4(3,j))
      elseif(jg1.eq.ig1.and.jg2.eq.ig4.and.jg3.eq.ig2.and.jg4.eq.ig3)then
         zM2(j)=zB4g0(jqb,jq,jg1,jg2,jg3,jg4,j4(4,j))
      elseif(jg1.eq.ig1.and.jg2.eq.ig4.and.jg3.eq.ig3.and.jg4.eq.ig2)then
         zM2(j)=zB4g0(jqb,jq,jg1,jg2,jg3,jg4,j4(5,j))
*
      elseif(jg1.eq.ig2.and.jg2.eq.ig1.and.jg3.eq.ig3.and.jg4.eq.ig4)then
         zM2(j)=zB4g0(jqb,jq,jg1,jg2,jg3,jg4,j4(6,j))
      elseif(jg1.eq.ig2.and.jg2.eq.ig1.and.jg3.eq.ig4.and.jg4.eq.ig3)then
         zM2(j)=zB4g0(jqb,jq,jg1,jg2,jg3,jg4,j4(7,j))
      elseif(jg1.eq.ig2.and.jg2.eq.ig3.and.jg3.eq.ig1.and.jg4.eq.ig4)then
         zM2(j)=zB4g0(jqb,jq,jg1,jg2,jg3,jg4,j4(8,j))
      elseif(jg1.eq.ig2.and.jg2.eq.ig3.and.jg3.eq.ig4.and.jg4.eq.ig1)then
         zM2(j)=zB4g0(jqb,jq,jg1,jg2,jg3,jg4,j4(9,j))
      elseif(jg1.eq.ig2.and.jg2.eq.ig4.and.jg3.eq.ig1.and.jg4.eq.ig3)then
         zM2(j)=zB4g0(jqb,jq,jg1,jg2,jg3,jg4,j4(10,j))
      elseif(jg1.eq.ig2.and.jg2.eq.ig4.and.jg3.eq.ig3.and.jg4.eq.ig1)then
         zM2(j)=zB4g0(jqb,jq,jg1,jg2,jg3,jg4,j4(11,j))
*
      elseif(jg1.eq.ig3.and.jg2.eq.ig1.and.jg3.eq.ig2.and.jg4.eq.ig4)then
         zM2(j)=zB4g0(jqb,jq,jg1,jg2,jg3,jg4,j4(12,j))
      elseif(jg1.eq.ig3.and.jg2.eq.ig1.and.jg3.eq.ig4.and.jg4.eq.ig2)then
         zM2(j)=zB4g0(jqb,jq,jg1,jg2,jg3,jg4,j4(13,j))
      elseif(jg1.eq.ig3.and.jg2.eq.ig2.and.jg3.eq.ig1.and.jg4.eq.ig4)then
         zM2(j)=zB4g0(jqb,jq,jg1,jg2,jg3,jg4,j4(14,j))
      elseif(jg1.eq.ig3.and.jg2.eq.ig2.and.jg3.eq.ig4.and.jg4.eq.ig1)then
         zM2(j)=zB4g0(jqb,jq,jg1,jg2,jg3,jg4,j4(15,j))
      elseif(jg1.eq.ig3.and.jg2.eq.ig4.and.jg3.eq.ig1.and.jg4.eq.ig2)then
         zM2(j)=zB4g0(jqb,jq,jg1,jg2,jg3,jg4,j4(16,j))
      elseif(jg1.eq.ig3.and.jg2.eq.ig4.and.jg3.eq.ig2.and.jg4.eq.ig1)then
         zM2(j)=zB4g0(jqb,jq,jg1,jg2,jg3,jg4,j4(17,j))
*
      elseif(jg1.eq.ig4.and.jg2.eq.ig1.and.jg3.eq.ig2.and.jg4.eq.ig3)then
         zM2(j)=zB4g0(jqb,jq,jg1,jg2,jg3,jg4,j4(18,j))
      elseif(jg1.eq.ig4.and.jg2.eq.ig1.and.jg3.eq.ig3.and.jg4.eq.ig2)then
         zM2(j)=zB4g0(jqb,jq,jg1,jg2,jg3,jg4,j4(19,j))
      elseif(jg1.eq.ig4.and.jg2.eq.ig2.and.jg3.eq.ig1.and.jg4.eq.ig3)then
         zM2(j)=zB4g0(jqb,jq,jg1,jg2,jg3,jg4,j4(20,j))
      elseif(jg1.eq.ig4.and.jg2.eq.ig2.and.jg3.eq.ig3.and.jg4.eq.ig1)then
         zM2(j)=zB4g0(jqb,jq,jg1,jg2,jg3,jg4,j4(21,j))
      elseif(jg1.eq.ig4.and.jg2.eq.ig3.and.jg3.eq.ig1.and.jg4.eq.ig2)then
         zM2(j)=zB4g0(jqb,jq,jg1,jg2,jg3,jg4,j4(22,j))
      elseif(jg1.eq.ig4.and.jg2.eq.ig3.and.jg3.eq.ig2.and.jg4.eq.ig1)then
         zM2(j)=zB4g0(jqb,jq,jg1,jg2,jg3,jg4,j4(23,j))
      endif

      enddo

      zwt = (0d0,0d0)
      
      do i=1,14
      ztmp1 = zM1(i)
      ztmp2 = zM2(i)
      zwt = zwt + ztmp1*dconjg(ztmp2)
      enddo

      wt = 2d0*dreal(zwt)
      B4g0int = wt

      return
      end

************************************************************************
* Re of interference of two orderings, summed over helicities
* q g g g qb

      function B3g0int(i1,i2,i3,i4,i5,j1,j2,j3,j4,j5)
      use KinData_mod
      implicit real*8 (a-h,o-y)
      implicit complex*16(z)
      dimension jhel(6)

      zwt = (0d0,0d0)

      if(j2.eq.i2.and.j3.eq.i3.and.j4.eq.i4)then
         jhel(1)=1
         jhel(2)=2
         jhel(3)=3
         jhel(4)=4
         jhel(5)=5
         jhel(6)=6
      elseif(j2.eq.i2.and.j3.eq.i4.and.j4.eq.i3)then
         jhel(1)=1
         jhel(2)=3
         jhel(3)=2
         jhel(4)=5
         jhel(5)=4
         jhel(6)=6
      elseif(j2.eq.i3.and.j3.eq.i2.and.j4.eq.i4)then
         jhel(1)=2
         jhel(2)=1
         jhel(3)=3
         jhel(4)=4
         jhel(5)=6
         jhel(6)=5
      elseif(j2.eq.i3.and.j3.eq.i4.and.j4.eq.i2)then
         jhel(1)=3
         jhel(2)=1
         jhel(3)=2
         jhel(4)=5
         jhel(5)=6
         jhel(6)=4
      elseif(j2.eq.i4.and.j3.eq.i2.and.j4.eq.i3)then
         jhel(1)=2
         jhel(2)=3
         jhel(3)=1
         jhel(4)=6
         jhel(5)=4
         jhel(6)=5
      elseif(j2.eq.i4.and.j3.eq.i3.and.j4.eq.i2)then
         jhel(1)=3
         jhel(2)=2
         jhel(3)=1
         jhel(4)=6
         jhel(5)=5
         jhel(6)=4
      endif

      do ihel=1,6
         ztmp1 = zB3g0(i5,i1,i2,i3,i4,ihel)
         ztmp2 = zB3g0(j5,j1,j2,j3,j4,jhel(ihel))
         zwt = zwt + ztmp1*dconjg(ztmp2)
      enddo

      wt = 2d0*dreal(zwt)

      B3g0int = wt

      return
      end
************************************************************************
* Amplitude with ordering qb q g g g g
*
      function zB3g0(i1,i2,i3,i4,i5,iHEL)
      use KinData_mod

      implicit real*8 (a-h,o-y)
      implicit complex*16(z)

      zi=(0d0,1d0)

      za12=zA5(i1,i2)
      za13=zA5(i1,i3)
      za14=zA5(i1,i4)
      za15=zA5(i1,i5)
      za23=zA5(i2,i3)
      za24=zA5(i2,i4)
      za25=zA5(i2,i5)
      za34=zA5(i3,i4)
      za35=zA5(i3,i5)
      za45=zA5(i4,i5)  

      za21=zA5(i2,i1)
      za31=zA5(i3,i1)
      za41=zA5(i4,i1)
      za51=zA5(i5,i1)
      za32=zA5(i3,i2)
      za42=zA5(i4,i2)
      za52=zA5(i5,i2)
      za43=zA5(i4,i3)
      za53=zA5(i5,i3)
      za54=zA5(i5,i4)    

      zb12=zB5(i1,i2)
      zb13=zB5(i1,i3)
      zb14=zB5(i1,i4)
      zb15=zB5(i1,i5)
      zb23=zB5(i2,i3)
      zb24=zB5(i2,i4)
      zb25=zB5(i2,i5)
      zb34=zB5(i3,i4)
      zb35=zB5(i3,i5)
      zb45=zB5(i4,i5)

      zb21=zB5(i2,i1)
      zb31=zB5(i3,i1)
      zb41=zB5(i4,i1)
      zb51=zB5(i5,i1)
      zb32=zB5(i3,i2)
      zb42=zB5(i4,i2)
      zb52=zB5(i5,i2)
      zb43=zB5(i4,i3)
      zb53=zB5(i5,i3)
      zb54=zB5(i5,i4)
  

* MHV amps

      if(ihel.eq.1)then
         zwt=za23**(3)*za13
     . /za12/za23/za34/za45/za51
      endif

      if(ihel.eq.2)then
         zwt=za24**(3)*za14
     . /za12/za23/za34/za45/za51
      endif

      if(ihel.eq.3)then
         zwt=za25**(3)*za15
     . /za12/za23/za34/za45/za51
      endif


* MHV-bar amps

      if(ihel.eq.4)then
         zwt=zb15**(3)*zb25
     . /zb12/zb23/zb34/zb45/zb51
      endif

      if(ihel.eq.5)then
         zwt=zb14**(3)*zb24
     . /zb12/zb23/zb34/zb45/zb51
      endif

      if(ihel.eq.6)then
         zwt=zb13**(3)*zb23
     . /zb12/zb23/zb34/zb45/zb51
      endif


      zB3g0=zi*zwt

      return
      end

************************************************************************
* Amplitude with ordering qb q g g g g
*
      function zB4g0(i1,i2,i3,i4,i5,i6,iHEL)
      use KinData_mod

      implicit real*8 (a-h,o-y)
      implicit complex*16(z)

      zi=(0d0,1d0)

      t123=s6(i1,i2)+s6(i1,i3)+s6(i2,i3)
      t234=s6(i2,i3)+s6(i2,i4)+s6(i3,i4)
      t345=s6(i3,i4)+s6(i3,i5)+s6(i4,i5)

      za12=zA6(i1,i2)
      za13=zA6(i1,i3)
      za14=zA6(i1,i4)
      za15=zA6(i1,i5)
      za16=zA6(i1,i6)
      za23=zA6(i2,i3)
      za24=zA6(i2,i4)
      za25=zA6(i2,i5)
      za26=zA6(i2,i6)
      za34=zA6(i3,i4)
      za35=zA6(i3,i5)
      za36=zA6(i3,i6)
      za45=zA6(i4,i5)
      za46=zA6(i4,i6)
      za56=zA6(i5,i6)

      za21=zA6(i2,i1)
      za31=zA6(i3,i1)
      za41=zA6(i4,i1)
      za51=zA6(i5,i1)
      za61=zA6(i6,i1)
      za32=zA6(i3,i2)
      za42=zA6(i4,i2)
      za52=zA6(i5,i2)
      za62=zA6(i6,i2)
      za43=zA6(i4,i3)
      za53=zA6(i5,i3)
      za63=zA6(i6,i3)
      za54=zA6(i5,i4)
      za64=zA6(i6,i4)
      za65=zA6(i6,i5)

      zb12=zB6(i1,i2)
      zb13=zB6(i1,i3)
      zb14=zB6(i1,i4)
      zb15=zB6(i1,i5)
      zb16=zB6(i1,i6)
      zb23=zB6(i2,i3)
      zb24=zB6(i2,i4)
      zb25=zB6(i2,i5)
      zb26=zB6(i2,i6)
      zb34=zB6(i3,i4)
      zb35=zB6(i3,i5)
      zb36=zB6(i3,i6)
      zb45=zB6(i4,i5)
      zb46=zB6(i4,i6)
      zb56=zB6(i5,i6)

      zb21=zB6(i2,i1)
      zb31=zB6(i3,i1)
      zb41=zB6(i4,i1)
      zb51=zB6(i5,i1)
      zb61=zB6(i6,i1)
      zb32=zB6(i3,i2)
      zb42=zB6(i4,i2)
      zb52=zB6(i5,i2)
      zb62=zB6(i6,i2)
      zb43=zB6(i4,i3)
      zb53=zB6(i5,i3)
      zb63=zB6(i6,i3)
      zb54=zB6(i5,i4)
      zb64=zB6(i6,i4)
      zb65=zB6(i6,i5)

* MHV amps

      if(ihel.eq.1)then
         zwt=za23**(3)*za13
     . /za12/za23/za34/za45/za56/za61
      endif

      if(ihel.eq.2)then
         zwt=za24**(3)*za14
     . /za12/za23/za34/za45/za56/za61
      endif

      if(ihel.eq.3)then
         zwt=za25**(3)*za15
     . /za12/za23/za34/za45/za56/za61
      endif

      if(ihel.eq.4)then
         zwt=za26**(3)*za16
     . /za12/za23/za34/za45/za56/za61
      endif

* MHV-bar amps

      if(ihel.eq.5)then
         zwt=zb16**(3)*zb26
     . /zb12/zb23/zb34/zb45/zb56/zb61
      endif

      if(ihel.eq.6)then
         zwt=zb15**(3)*zb25
     . /zb12/zb23/zb34/zb45/zb56/zb61
      endif

      if(ihel.eq.7)then
         zwt=zb14**(3)*zb24
     . /zb12/zb23/zb34/zb45/zb56/zb61
      endif

      if(ihel.eq.8)then
         zwt=zb13**(3)*zb23
     . /zb12/zb23/zb34/zb45/zb56/zb61
      endif

* N-MHV amps

      if(ihel.eq.9)then
      zn1=((za45*zb51+za46*zb61)**(2))*(za45*zb52+za46*zb62)
      zn2=((za23*zb35+za24*zb45)**(2))*(za13*zb35+za14*zb45)
      zd1=t123*zb12*zb23*za45*za56*(zb34*za46+zb35*za56)
      zd2=t345*za12*za61*zb34*zb45*(zb34*za46+zb35*za56)
      zwt=-(zn1/zd1)+(zn2/zd2)
      endif

      if(ihel.eq.10)then
      zn1=(za25*zb54+za26*zb64)**(3)
      zn2=(za54*zb41+za53*zb31)**(3)
      zd1=t123*za12*za23*zb45*zb56*(za34*zb46+za35*zb56)
      zd2=t345*za34*za45*zb61*zb12*(za34*zb46+za35*zb56)
      zwt=(zn1/zd1)-(zn2/zd2)
      endif

      if(ihel.eq.11)then
      zn1=((za34*zb41+za35*zb51)**(3))*(za34*zb42+za35*zb52)
      zn2=za13*za23**(2)*zb45**(3)
      zn3=zb24*(za62*zb24+za63*zb34)**(3)
      zd1=t345*zb61*zb12*za34*za45*(za34*zb46+za35*zb56)*(zb23*za35+zb24*za45)
      zd2=t123*zb56*za12*(za12*zb24+za13*zb34)*(za31*zb16+za32*zb26)
      zd3=t234*zb23*zb34*za56*(za12*zb24+za13*zb34)*(zb23*za35+zb24*za45)
      zwt=-(zn1/zd1)+(zn2/zd2)+(zn3/zd3)
      endif

      if(ihel.eq.12)then
      zn1=(za15*(za51*zb13+za56*zb63)**(3))
      zn2=zb62*zb61**(2)*za45**(3)
      zn3=(za14*zb46+za15*zb56)*(za24*zb46+za25*zb56)**(3)
      zd1=t234*zb34*za16*za56*(za51*zb12+za56*zb62)*(zb45*za51+zb46*za61)
      zd2=t345*zb12*za34*(za34*zb46+za35*zb56)*(za51*zb12+za56*zb62)
      zd3=t123*zb45*zb56*za12*za23*(za34*zb46+za35*zb56)*(za15*zb54+za16*zb64)
      zwt=-(zn1/zd1)+(zn2/zd2)+(zn3/zd3)

      endif

      if(ihel.eq.13)then
      zn1=za32**(2)*za31*zb46**(4)
      zn2=zb24*za15*(za51*zb14+za56*zb64)**(3)
      zn3=za35**(4)*zb61**(2)*zb62
      zd1=t123*zb45*za12*zb56*(za34*zb46+za35*zb56)*(za15*zb54+za16*zb64)
      zd2=t234*zb23*zb34*za16*za56*(za15*zb54+za16*zb64)*(za51*zb12+za56*zb62)
      zd3=t345*zb12*za34*za45*(za31*zb16+za32*zb26)*(za51*zb12+za56*zb62)
      zwt=-(zn1/zd1)-(zn2/zd2)-(zn3/zd3)
      endif

      if(ihel.eq.14)then
      zn1=za46**(4)*zb13**(3)
      zn2=zb35**(4)*za62**(3)
      zn3=za42**(3)*zb15**(3)*(za41*zb15+za46*zb65)
      zd1=t123*zb12*za45*za56*(za61*zb13+za62*zb23)*(zb12*za24+zb13*za34)
      zd2=t345*zb34*zb45*za12*(za65*zb53+za64*zb43)*(za23*zb35+za24*zb45)
      zd3=t234*zb56*zb61*za34*za23*(za23*zb35+za24*zb45)*(za43*zb31+za42*zb21)
      zwt=-(zn1/zd1)+(zn2/zd2)+(zn3/zd3)
      endif

      zB4g0=zi*zwt

      return
      end

************************************************************************
*
      subroutine B4g0heliords(i,j1,j2,j3,j4)
      use KinData_mod
      implicit integer(a-z)
      common/nheljets/nhel(6,14,24)
      dimension j1(3),j2(8),j3(11),j4(23)
* j1 ~ one photon
* j2 ~ R term
* j3 ~ two photon
* j4 ~ four photon

      do kk=2,24
      do jj=1,14
      do ll=3,6
      if(nhel(ll,i,1).ne.nhel(ll,jj,kk))then
      exit
      else
      if(ll.eq.6)then

      if(kk.eq.2)then
      j4(1)=jj
      endif
      if(kk.eq.3)then
      j3(1)=jj
      j4(2)=jj
      endif
      if(kk.eq.4)then
      j3(3)=jj
      j4(3)=jj
      endif
      if(kk.eq.5)then
      j4(4)=jj
      endif
      if(kk.eq.6)then
      j4(5)=jj
      endif
      if(kk.eq.7)then
      j1(1)=jj
      j3(6)=jj
      j4(6)=jj
      endif
      if(kk.eq.8)then
      j2(1)=jj
      j4(7)=jj
      endif
      if(kk.eq.9)then
      j1(2)=jj
      j3(7)=jj
      j4(8)=jj
      endif
      if(kk.eq.10)then
      j1(3)=jj
      j3(9)=jj
      j4(9)=jj
      endif
      if(kk.eq.11)then
      j2(2)=jj
      j4(10)=jj
      endif
      if(kk.eq.12)then
      j2(3)=jj
      j4(11)=jj
      endif
      if(kk.eq.13)then
      j3(2)=jj
      j4(12)=jj
      endif
      if(kk.eq.14)then
      j2(4)=jj
      j3(4)=jj
      j4(13)=jj
      endif
      if(kk.eq.15)then
      j3(8)=jj
      j4(14)=jj
      endif
      if(kk.eq.16)then
      j2(5)=jj
      j3(10)=jj
      j4(15)=jj
      endif
      if(kk.eq.17)then
      j3(5)=jj
      j4(16)=jj
      endif
      if(kk.eq.18)then
      j3(11)=jj
      j4(17)=jj
      endif
      if(kk.eq.19)then
      j4(18)=jj
      endif
      if(kk.eq.20)then
      j2(6)=jj
      j4(19)=jj
      endif
      if(kk.eq.21)then
      j2(7)=jj
      j4(20)=jj
      endif
      if(kk.eq.22)then
      j4(21)=jj
      endif
      if(kk.eq.23)then
      j4(22)=jj
      endif
      if(kk.eq.24)then
      j2(8)=jj
      j4(23)=jj
      endif

      endif
      endif
      enddo
      enddo
      enddo

      return
      end
************************************************************************
      function Bt4g0Full(iq,i1,i2,i3,i4,iqb)
      use KinData_mod
      implicit double precision (a-h,o-z)

!      fac=(4d0*pi*as*nc)**4*(nc**2-1d0)/nc

      Bt4g0Full = Bt4g0(iq,i1,i2,i3,i4,iqb)
     .          + Bt4g0(iq,i1,i2,i4,i3,iqb)
     .          + Bt4g0(iq,i1,i3,i2,i4,iqb)
     .          + Bt4g0(iq,i1,i3,i4,i2,iqb)
     .          + Bt4g0(iq,i1,i4,i2,i3,iqb)
     .          + Bt4g0(iq,i1,i4,i3,i2,iqb)
     .          + Bt4g0(iq,i2,i1,i3,i4,iqb)
     .          + Bt4g0(iq,i2,i1,i4,i3,iqb)
     .          + Bt4g0(iq,i2,i3,i1,i4,iqb)
     .          + Bt4g0(iq,i2,i3,i4,i1,iqb)
     .          + Bt4g0(iq,i2,i4,i1,i3,iqb)
     .          + Bt4g0(iq,i2,i4,i3,i1,iqb)
     .          + Bt4g0(iq,i3,i1,i2,i4,iqb)
     .          + Bt4g0(iq,i3,i1,i4,i2,iqb)
     .          + Bt4g0(iq,i3,i2,i1,i4,iqb)
     .          + Bt4g0(iq,i3,i2,i4,i1,iqb)
     .          + Bt4g0(iq,i3,i4,i1,i2,iqb)
     .          + Bt4g0(iq,i3,i4,i2,i1,iqb)
     .          + Bt4g0(iq,i4,i1,i2,i3,iqb)
     .          + Bt4g0(iq,i4,i1,i3,i2,iqb)
     .          + Bt4g0(iq,i4,i2,i1,i3,iqb)
     .          + Bt4g0(iq,i4,i2,i3,i1,iqb)
     .          + Bt4g0(iq,i4,i3,i1,i2,iqb)
     .          + Bt4g0(iq,i4,i3,i2,i1,iqb)

      return
      end
************************************************************************
      function Btt4g0Full(iq,i1,i2,i3,i4,iqb)
      use KinData_mod
      implicit double precision (a-h,o-z)

!      fac=(4d0*pi*as*nc)**4*(nc**2-1d0)/nc

      Btt4g0Full = Btt4g0(iq,i1,i2,i3,i4,iqb)
     .           + Btt4g0(iq,i1,i2,i4,i3,iqb)
     .           + Btt4g0(iq,i1,i3,i2,i4,iqb)
     .           + Btt4g0(iq,i1,i3,i4,i2,iqb)
     .           + Btt4g0(iq,i1,i4,i2,i3,iqb)
     .           + Btt4g0(iq,i1,i4,i3,i2,iqb)
     .           + Btt4g0(iq,i2,i3,i1,i4,iqb)
     .           + Btt4g0(iq,i2,i3,i4,i1,iqb)
     .           + Btt4g0(iq,i2,i4,i1,i3,iqb)
     .           + Btt4g0(iq,i2,i4,i3,i1,iqb)
     .           + Btt4g0(iq,i3,i4,i1,i2,iqb)
     .           + Btt4g0(iq,i3,i4,i2,i1,iqb)

      return
      end
************************************************************************
* Full R-term, summed over all 24 gluon permuations
      function R4g0(iq,i1,i2,i3,i4,iqb)
      use KinData_mod
      implicit double precision (a-h,o-z)

!      fac=(4d0*pi*as*nc)**4*(nc**2-1d0)/nc

      R4g0 = BR4g0ord(iq,i1,i2,i3,i4,iqb)
     .     + BR4g0ord(iq,i1,i2,i4,i3,iqb)
     .     + BR4g0ord(iq,i1,i3,i2,i4,iqb)
     .     + BR4g0ord(iq,i1,i3,i4,i2,iqb)
     .     + BR4g0ord(iq,i1,i4,i2,i3,iqb)
     .     + BR4g0ord(iq,i1,i4,i3,i2,iqb)
     .     + BR4g0ord(iq,i2,i1,i3,i4,iqb)
     .     + BR4g0ord(iq,i2,i1,i4,i3,iqb)
     .     + BR4g0ord(iq,i2,i3,i1,i4,iqb)
     .     + BR4g0ord(iq,i2,i3,i4,i1,iqb)
     .     + BR4g0ord(iq,i2,i4,i1,i3,iqb)
     .     + BR4g0ord(iq,i2,i4,i3,i1,iqb)
     .     + BR4g0ord(iq,i3,i1,i2,i4,iqb)
     .     + BR4g0ord(iq,i3,i1,i4,i2,iqb)
     .     + BR4g0ord(iq,i3,i2,i1,i4,iqb)
     .     + BR4g0ord(iq,i3,i2,i4,i1,iqb)
     .     + BR4g0ord(iq,i3,i4,i1,i2,iqb)
     .     + BR4g0ord(iq,i3,i4,i2,i1,iqb)
     .     + BR4g0ord(iq,i4,i1,i2,i3,iqb)
     .     + BR4g0ord(iq,i4,i1,i3,i2,iqb)
     .     + BR4g0ord(iq,i4,i2,i1,i3,iqb)
     .     + BR4g0ord(iq,i4,i2,i3,i1,iqb)
     .     + BR4g0ord(iq,i4,i3,i1,i2,iqb)
     .     + BR4g0ord(iq,i4,i3,i2,i1,iqb)

      return
      end
************************************************************************
* Wrapper function for the full SLC of FullB4g0
      function Bt4g0SLC(iq,i1,i2,i3,i4,iqb)
      use KinData_mod
      implicit double precision (a-h,o-z)

      Bt4g0SLC = - Bt4g0Full(iq,i1,i2,i3,i4,iqb)
     .           - R4g0(iq,i1,i2,i3,i4,iqb)
     .           + Bttt4g0(iq,i1,i2,i3,i4,iqb)

      return
      end
************************************************************************
* Wrapper function for the full SSLC of FullB4g0
      function Btt4g0SSLC(iq,i1,i2,i3,i4,iqb)
      use KinData_mod
      implicit double precision (a-h,o-z)

      Btt4g0SSLC = Btt4g0Full(iq,i1,i2,i3,i4,iqb)
     .           - 3d0*Bttt4g0(iq,i1,i2,i3,i4,iqb)

      return
      end
************************************************************************
* Wrapper function for the full SSSLC of FullB4g0
      function Bttt4g0SSSLC(iq,i1,i2,i3,i4,iqb)
      use KinData_mod
      implicit double precision (a-h,o-z)

      Bttt4g0SSSLC = - Bttt4g0(iq,i1,i2,i3,i4,iqb)

      return
      end
************************************************************************

