!************************************************************************
!  All tree-level amplitudes in this file are based on
!  
!  M.~L.~Mangano and S.~J.~Parke,
!  %``Multiparton amplitudes in gauge theories,''
!  Phys. Rept. \textbf{200} (1991), 301-367
!  doi:10.1016/0370-1573(91)90091-Y
!  [arXiv:hep-th/0509223 [hep-th]].
!
!  J.G.M. Kuijf,
!  Multiparton production at hadron colliders
!  PhD thesis, 
!  Leiden (1991).
************************************************************************
* 4 quark non-identical tree level matrix elements
* summed over colour orderings, summed and squared
*
* call with arguments i1=q, i2=Qb, i3=Q, i4=qb
*
      function FullC0g0(i1,i2,i3,i4)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)

      fac=(4d0*pi*as*nc)**2*(nc**2-1d0)/dble(nc)**2

      FullC0g0=fac*C0g0(i1,i2,i3,i4)

      return
      end
************************************************************************
* 4 quark identical tree level matrix elements
* summed over colour orderings, summed and squared
*
* call with arguments i1=q, i2=qb, i3=q, i4=qb
*
      function FullC0g0ID(i1,i2,i3,i4)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)

      FullC0g0ID=FullC0g0(i1,i2,i3,i4)
     .          +FullC0g0(i1,i4,i3,i2)
     .          -FullD0g0(i1,i2,i3,i4)


      return
      end
************************************************************************
* 4 quark identical tree level matrix elements
* summed over colour orderings, summed and squared
*
* call with arguments i1=q, i2=qb, i3=q, i4=qb
*
      function FullD0g0(i1,i2,i3,i4)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)

      fac=(4d0*pi*as*nc)**2*(nc**2-1d0)/dble(nc)**3

      wt= D0g0(i1,i2,i3,i4)

      FullD0g0=fac*wt

      return
      end
************************************************************************
* 4 quark non-identical tree level matrix elements
* one colour ordering, summed and squared
*
* call with arguments i1=q, i2=Qb, i3=Q, i4=qb
*
      function C0g0(i1,i2,i3,i4)
      use KinData_mod
      implicit double precision (a-h,o-z)

      s14=s4(i1,i4)
      s13=s4(i1,i3)
      s12=s4(i1,i2)

      C0g0=2d0*(s13**2+s12**2)/s14**2

      return
      end
************************************************************************
* 4 quark identical tree level matrix elements
* one colour ordering, summed and squared
*
* call with arguments i1=q, i2=qb, i3=q, i4=qb
*
      function D0g0(i1,i2,i3,i4)
      use KinData_mod
      implicit double precision (a-h,o-z)

      s12=s4(i1,i2)
      s13=s4(i1,i3)
      s14=s4(i1,i4)

      D0g0=4d0*s13**2/s12/s14

      return
      end



*****************************************************************************
*     Five parton tree-level 2q2q1g
*****************************************************************************
      function FullC1g0(iq1,iqb1,iq2,iqb2,ig)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      common /koppel/nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      parameter(pi=3.141592653589793238d0)

      fac=(4d0*pi*as*nc)**3*(nc**2-1d0)/dble(nc)**2

c      wt =
c     .+ C1g0a(iq1,ig,iqb2,iq2,iqb1)
c     .+ C1g0b(iq1,iqb2,iq2,ig,iqb1)

c     .+1d0/dble(nc)**2*(
c     .+ C1g0c(iq1,ig,iqb1,iq2,iqb2)
c     .+ C1g0d(iq1,iqb1,iq2,ig,iqb2)
c     .- 2d0*Ct1g0(iq1,iqb1,iq2,iqb2,ig))

      wt =
     .+ C1g0X(iq1,iqb1,iq2,iqb2,ig)

     .+1d0/dble(nc)**2*(
     .+ C1g0Y(iq1,iqb1,iq2,iqb2,ig))

      wt=fac*wt

      FullC1g0=wt

      return
      end

*****************************************************************************
*     Five parton tree-level 4q1g
*****************************************************************************
      function FullC1g0ID(iq1,iqb1,iq2,iqb2,ig)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      common /koppel/nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      parameter(pi=3.141592653589793238d0)

      wt = FullC1g0(iq1,iqb1,iq2,iqb2,ig)
     .   + FullC1g0(iq1,iqb2,iq2,iqb1,ig)
     .   + FullD1g0(iq1,iqb1,iq2,iqb2,ig)

      FullC1g0ID = wt

      return
      end
************************************************************************
* 1 gluon 4 quark identical tree level matrix elements
* summed over colour orderings, summed and squared
*
      function FullD1g0(iq1,iqb1,iq2,iqb2,ig)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)

      fac=(4d0*pi*as*nc)**3*(nc**2-1d0)/dble(nc)**3

c      wt =
c     .   + D1g0(iq1,iqb1,iq2,iqb2,ig)
c     .   + D1g0(iq1,iqb2,iq2,iqb1,ig)
c     .   - (nc**2+1d0)/nc**2*D1g0gam(iq1,iqb1,iq2,iqb2,ig)

      wt =
     .   + D1g0X(iq1,iqb1,iq2,iqb2,ig)
     .   - (nc**2+1d0)/nc**2*D1g0gam(iq1,iqb1,iq2,iqb2,ig)


      FullD1g0=fac*wt

      return
      end

************************************************************************
      function D1g0X(iq1,iqb1,iq2,iqb2,ig)
      use KinData_mod
      implicit double precision (a-h,o-z)

      D1g0X=D1g0(iq1,iqb1,iq2,iqb2,ig)
     .     +D1g0(iq1,iqb2,iq2,iqb1,ig)

c      write(*,*)"D1g0X  ",D1g0X,iq1,iqb1,iq2,iqb2,ig

      return
      end

**********************************************************************
* A wrapper function for the C1g0X matrix element symmetrised over the
* second quark pair. This is required for certain subtraction terms
* (e.g. qg initiated nf piece).
*
* qg configuration => symmetrised
* qbg configuration => symmetrised
* anything else => not symmetrised

      function sC1g0X(i1,i2,i3,i4,i5)
      use KinData_mod
      implicit double precision (a-h,o-y)

      if((i5.le.2).and.((i1.le.2).or.(i2.le.2)))then
         sC1g0X = 0.5d0*(C1g0X(i1,i2,i3,i4,i5)
     .                  +C1g0X(i1,i2,i4,i3,i5))
      else
         sC1g0X = C1g0X(i1,i2,i3,i4,i5)
      endif

      return
      end


**********************************************************************
* A wrapper function for the C1g0Y matrix element symmetrised over the
* second quark pair. This is required for certain subtraction terms
* (e.g. qg initiated nf piece).
*
* qg configuration => symmetrised
* qbg configuration => symmetrised
* anything else => not symmetrised

      function sC1g0Y(i1,i2,i3,i4,i5)
      use KinData_mod
      implicit double precision (a-h,o-y)

      if((i5.le.2).and.((i1.le.2).or.(i2.le.2)))then
         sC1g0Y = 0.5d0*(C1g0Y(i1,i2,i3,i4,i5)
     .                  +C1g0Y(i1,i2,i4,i3,i5))
      else
         sC1g0Y = C1g0Y(i1,i2,i3,i4,i5)
      endif

      return
      end



************************************************************************
      function C1g0X(iq1,iqb1,iq2,iqb2,ig)
      use KinData_mod
      implicit double precision (a-h,o-z)

      C1g0X=C1g0a(iq1,ig,iqb2,iq2,iqb1)
     .     +C1g0b(iq1,iqb2,iq2,ig,iqb1)
      return
      end
************************************************************************
      function C1g0Y(iq1,iqb1,iq2,iqb2,ig)
      use KinData_mod
      implicit double precision (a-h,o-z)

      C1g0Y=C1g0c(iq1,ig,iqb1,iq2,iqb2)
     .     +C1g0d(iq1,iqb1,iq2,ig,iqb2)
     .  - 2d0*C1g0gam(iq1,iqb1,iq2,iqb2,ig)

c      write(*,*)"C1g0Y  ",C1g0Y,iq1,iqb1,iq2,iqb2,ig

      return
      end

************************************************************************
      function C1g01(i1,i5,i4,i3,i2)
      use KinData_mod
      implicit double precision (a-h,o-z)

      C1g01=C1g0a(i1,i5,i4,i3,i2)
      return
      end
************************************************************************
      function C1g02(i1,i4,i3,i5,i2)
      use KinData_mod
      implicit double precision (a-h,o-z)

      C1g02=C1g0a(i3,i5,i2,i1,i4)
      return
      end

*****************************************************************************
*       helicity summed, squared amplitudes
*****************************************************************************
*       Qqb tree (1,0) ordering squared

      function C1g0a(iq1,ig,iqb2,iq2,iqb1)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      wt=0d0
      do ihel=1,4
      do iflip=0,1
        ztree = zzC1g0a(iq1,ig,iqb2,iq2,iqb1,ihel,iflip)
c        if(iflip.eq.0)write(*,*)1,ihel,ztree
        wt = wt + real(ztree*dconjg(ztree))
      enddo
      enddo

      C1g0a = wt
c      write(*,*)"C1g01",C1g01

      return
      end
*****************************************************************************
*       Qqb tree (0,1) ordering squared

      function C1g0b(iq1,iqb2,iq2,ig,iqb1)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      wt=0d0
      do ihel=1,4
      do iflip=0,1
        ztree = zzC1g0b(iq1,iqb2,iq2,ig,iqb1,ihel,iflip)
c        if(iflip.eq.0)write(*,*)2,ihel,ztree
        wt = wt + real(ztree*dconjg(ztree))
      enddo
      enddo

      C1g0b = wt
c      write(*,*)"C1g0b",C1g0b

      return
      end
*****************************************************************************
*       QQb tree (1,0) ordering squared

      function C1g0c(iq1,ig,iqb1,iq2,iqb2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      wt=0d0
      do ihel=1,4
      do iflip=0,1
        ztree = zzC1g0c(iq1,ig,iqb1,iq2,iqb2,ihel,iflip)
c        if(iflip.eq.0)write(*,*)3,ihel,ztree
        wt = wt + real(ztree*dconjg(ztree))
      enddo
      enddo

      C1g0c = wt
c      write(*,*)"C1g0c",C1g0c

      return
      end
*****************************************************************************
*       QQb tree (0,1) ordering squared

      function C1g0d(iq1,iqb1,iq2,ig,iqb2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      wt=0d0
      do ihel=1,4
      do iflip=0,1
        ztree = zzC1g0d(iq1,iqb1,iq2,ig,iqb2,ihel,iflip)
c        if(iflip.eq.0)write(*,*)4,ihel,ztree
        wt = wt + real(ztree*dconjg(ztree))
      enddo
      enddo

      C1g0d = wt
c      write(*,*)"C1g0d",C1g0d

      return
      end
*****************************************************************************
*       symmetrized tree squared
*       note Qqb symmetrized tree = QQb symmetrized tree

      function C1g0gam(iq1,iqb1,iq2,iqb2,ig)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      wt=0d0

      do ihel=1,4
      do iflip=0,1
        ztree = zzC1g0gam(iq1,iqb1,iq2,iqb2,ig,ihel,iflip)
        wt = wt + real(ztree*dconjg(ztree))
      enddo
      enddo

      C1g0gam = wt
c      write(*,*)"C1g0gam",C1g0gam

      return
      end
*****************************************************************************
*  interference between quark orderings

      function D1g0(iq1,iqb1,iq2,iqb2,ig)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      dimension ihel2(6)

      ihel2(1)=1
      ihel2(2)=5
      ihel2(3)=6
      ihel2(4)=4
      ihel2(5)=2
      ihel2(6)=3
      wt=0d0
      do ihel=1,6
      do iflip=0,1
        zwt11 = zzC1g0c(iq1,ig,iqb1,iq2,iqb2,ihel,iflip)
        zwt12 = zzC1g0a(iq1,ig,iqb1,iq2,iqb2,ihel2(ihel),iflip)

        zwt21 = zzC1g0d(iq1,iqb1,iq2,ig,iqb2,ihel,iflip)
        zwt22 = zzC1g0b(iq1,iqb1,iq2,ig,iqb2,ihel2(ihel),iflip)

c        write(*,*)ihel,iflip,2d0*real(zwt12*dconjg(zwt11))
c        write(*,*)ihel,iflip,2d0*real(zwt22*dconjg(zwt21))
c        write(*,*)zwt11
c        write(*,*)zwt12
c        write(*,*)zwt21
c        write(*,*)zwt22


        wt = wt + 2d0*real(zwt12*dconjg(zwt11))
        wt = wt + 2d0*real(zwt22*dconjg(zwt21))
      enddo
      enddo

      D1g0 = wt
c      write(*,*)"D1g0   ",D1g0,iq1,iqb1,iq2,iqb2,ig

      return
      end
*****************************************************************************
*  interference between quark orderings

      function D1g0gam(iq1,iqb1,iq2,iqb2,ig)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      dimension ihel2(6)

      ihel2(1)=1
      ihel2(2)=5
      ihel2(3)=6
      ihel2(4)=4
      ihel2(5)=2
      ihel2(6)=3

      wt=0d0
      do ihel=1,6
      do iflip=0,1
        ztree1 = zzC1g0gam(iq1,iqb1,iq2,iqb2,ig,ihel,iflip)
        ztree2 = zzC1g0gam(iq1,iqb2,iq2,iqb1,ig,ihel2(ihel),iflip)
        wt = wt + 2d0*real(ztree2*dconjg(ztree1))
      enddo
      enddo

      D1g0gam = wt
c      write(*,*)"D1g0gam",D1g0gam,iq1,iqb1,iq2,iqb2,ig

      return
      end

*****************************************************************************
*       tree-level 2q2Q1g amplitudes
*****************************************************************************
* (1,0) ordering Qqb tree amplitude (i.e leading colour amplitude,
* gluon in string between Q and qb, not q and qb)

      function zzC1g0a(iq1,ig,iqb2,iq2,iqb1,ihel,iflip)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

*     iflip = 0 ihel = 1 ppp
*     iflip = 0 ihel = 2 mpp
*     iflip = 0 ihel = 3 pmp
*     iflip = 0 ihel = 4 mmp
*         with A <-> B flip
*     iflip = 1 ihel = 1 mmm
*     iflip = 1 ihel = 2 pmm
*     iflip = 1 ihel = 3 mpm
*     iflip = 1 ihel = 4 ppm

      i1=iq1
      i2=iqb2
      i3=iq2
      i4=iqb1
      i5=ig

      if(iflip.eq.0)then

      za12=za5(i2,i4)
      za13=za5(i2,i1)
      za14=za5(i2,i3)
      za15=za5(i2,i5)
      za23=za5(i4,i1)
      za24=za5(i4,i3)
      za25=za5(i4,i5)
      za34=za5(i1,i3)
      za35=za5(i1,i5)
      za45=za5(i3,i5)

      zb12=zb5(i2,i4)
      zb13=zb5(i2,i1)
      zb14=zb5(i2,i3)
      zb15=zb5(i2,i5)
      zb23=zb5(i4,i1)
      zb24=zb5(i4,i3)
      zb25=zb5(i4,i5)
      zb34=zb5(i1,i3)
      zb35=zb5(i1,i5)
      zb45=zb5(i3,i5)

      elseif(iflip.eq.1)then

      za12=-zb5(i2,i4)
      za13=-zb5(i2,i1)
      za14=-zb5(i2,i3)
      za15=-zb5(i2,i5)
      za23=-zb5(i4,i1)
      za24=-zb5(i4,i3)
      za25=-zb5(i4,i5)
      za34=-zb5(i1,i3)
      za35=-zb5(i1,i5)
      za45=-zb5(i3,i5)

      zb12=-za5(i2,i4)
      zb13=-za5(i2,i1)
      zb14=-za5(i2,i3)
      zb15=-za5(i2,i5)
      zb23=-za5(i4,i1)
      zb24=-za5(i4,i3)
      zb25=-za5(i4,i5)
      zb34=-za5(i1,i3)
      zb35=-za5(i1,i5)
      zb45=-za5(i3,i5)

      endif
*
      zi=(0d0,1d0)

      zfac=zi*za13*za14**(-1)*za15**(-1)*za23**(-1)*za35**(-1)

      if(ihel.eq.1)then
         zwt = -za12**2
      elseif(ihel.eq.2)then
         zwt = +za24**2
      elseif(ihel.eq.3)then
         zwt = +za13**2
      elseif(ihel.eq.4)then
         zwt = -za34**2
      elseif(ihel.gt.4)then
         zwt = (0d0,0d0)
      endif
      zwt=zwt*zfac
      zzC1g0a= zwt


      return
      end
*****************************************************************************
* (0,1) ordering Qqb tree amplitude, i.e gluon in other string

      function zzC1g0b(iq1,iqb2,iq2,ig,iqb1,ihel,iflip)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      if(ihel.eq.1)then
      zwt= zzC1g0a(iq2,ig,iqb1,iq1,iqb2,1,iflip)
      elseif(ihel.eq.2)then
      zwt= zzC1g0a(iq2,ig,iqb1,iq1,iqb2,3,iflip)
      elseif(ihel.eq.3)then
      zwt= zzC1g0a(iq2,ig,iqb1,iq1,iqb2,2,iflip)
      elseif(ihel.eq.4)then
      zwt= zzC1g0a(iq2,ig,iqb1,iq1,iqb2,4,iflip)
      elseif(ihel.gt.4)then
      zwt = (0d0,0d0)
      endif

      zzC1g0b = zwt

      return
      end
*****************************************************************************
* (1,0) ordering QQb tree amplitude
* gluon in string between Q and qb, not q and qb)

      function zzC1g0c(iq1,ig,iqb1,iq2,iqb2,ihel,iflip)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

*     iflip = 0 ihel = 1 ppp
*     iflip = 0 ihel = 2 mpp
*     iflip = 0 ihel = 3 pmp
*     iflip = 0 ihel = 4 mmp
*         with A <-> B flip
*     iflip = 1 ihel = 1 mmm
*     iflip = 1 ihel = 2 pmm
*     iflip = 1 ihel = 3 mpm
*     iflip = 1 ihel = 4 ppm

      i1=iq1
      i2=iqb2
      i3=iq2
      i4=iqb1
      i5=ig

      if(iflip.eq.0)then

      za12=za5(i2,i4)
      za13=za5(i2,i1)
      za14=za5(i2,i3)
      za15=za5(i2,i5)
      za23=za5(i4,i1)
      za24=za5(i4,i3)
      za25=za5(i4,i5)
      za34=za5(i1,i3)
      za35=za5(i1,i5)
      za45=za5(i3,i5)

      zb12=zb5(i2,i4)
      zb13=zb5(i2,i1)
      zb14=zb5(i2,i3)
      zb15=zb5(i2,i5)
      zb23=zb5(i4,i1)
      zb24=zb5(i4,i3)
      zb25=zb5(i4,i5)
      zb34=zb5(i1,i3)
      zb35=zb5(i1,i5)
      zb45=zb5(i3,i5)

      elseif(iflip.eq.1)then

      za12=-zb5(i2,i4)
      za13=-zb5(i2,i1)
      za14=-zb5(i2,i3)
      za15=-zb5(i2,i5)
      za23=-zb5(i4,i1)
      za24=-zb5(i4,i3)
      za25=-zb5(i4,i5)
      za34=-zb5(i1,i3)
      za35=-zb5(i1,i5)
      za45=-zb5(i3,i5)

      zb12=-za5(i2,i4)
      zb13=-za5(i2,i1)
      zb14=-za5(i2,i3)
      zb15=-za5(i2,i5)
      zb23=-za5(i4,i1)
      zb24=-za5(i4,i3)
      zb25=-za5(i4,i5)
      zb34=-za5(i1,i3)
      zb35=-za5(i1,i5)
      zb45=-za5(i3,i5)

      endif
*
      zi=(0d0,1d0)
      zfac=zi*za14**(-1)*za35**(-1)*za25**(-1)

      if(ihel.eq.1)then
         zwt = -za12**2
      elseif(ihel.eq.2)then
         zwt = +za24**2
      elseif(ihel.eq.3)then
         zwt = +za13**2
      elseif(ihel.eq.4)then
         zwt = -za34**2
      elseif(ihel.gt.4)then
         zwt = (0d0,0d0)
      endif
      zwt=zwt*zfac
      zzC1g0c= zwt

      return
      end
*****************************************************************************
* (0,1) ordering QQb tree amplitude, i.e gluon in other string

      function zzC1g0d(iq1,iqb1,iq2,ig,iqb2,ihel,iflip)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      if(ihel.eq.1)then
      zwt= zzC1g0c(iq2,ig,iqb2,iq1,iqb1,1,iflip)
      elseif(ihel.eq.2)then
      zwt= zzC1g0c(iq2,ig,iqb2,iq1,iqb1,3,iflip)
      elseif(ihel.eq.3)then
      zwt= zzC1g0c(iq2,ig,iqb2,iq1,iqb1,2,iflip)
      elseif(ihel.eq.4)then
      zwt= zzC1g0c(iq2,ig,iqb2,iq1,iqb1,4,iflip)
      elseif(ihel.gt.4)then
      zwt = (0d0,0d0)
      endif

      zzC1g0d = zwt

      return
      end
*****************************************************************************
* symmetrized qQ amplitude (=symmetrized QQ amplitude)

      function zzC1g0gam(iq1,iqb1,iq2,iqb2,ig,ihel,iflip)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      zwt = zzC1g0a(iq1,ig,iqb2,iq2,iqb1,ihel,iflip)
     .    + zzC1g0b(iq1,iqb2,iq2,ig,iqb1,ihel,iflip)

      zzC1g0gam = zwt

      return
      end

************************************************************************
* non-identical 4 quark 2 gluon scattering
*
*                      q qb Q Qb g g
*
      function FullqQC2g0(iq1,iq2,iq3,iq4,ig1,ig2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16(z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      dimension zm(8,12),colm(12,12),tmp(12)

c     colour matrix

      data (colm(iaux,  1),iaux=1, 6)      / 24, 0, 3, -3, 0, 3/
      data (colm(iaux,  1),iaux=7, 12)     / 8, -1, 8, -1, 8, -1/

      data (colm(iaux,  2),iaux=1, 6)      / 0, 24, 0, 0, 3, 0/
      data (colm(iaux,  2),iaux=7, 12)     / -1, 8, 8, 8, 8, -1/

      data (colm(iaux,  3),iaux=1, 6)      / 3, 0, 24, 3, 0, -3/
      data (colm(iaux,  3),iaux=7, 12)     / 8, 8, 8, -1, -1, -1/

      data (colm(iaux,  4),iaux=1, 6)      / -3, 0, 3, 24, 0, 3/
      data (colm(iaux,  4),iaux=7, 12)     / -1, 8, -1, 8, -1, 8/

      data (colm(iaux,  5),iaux=1, 6)      / 0, 3, 0, 0, 24, 0/
      data (colm(iaux,  5),iaux=7, 12)     / 8, 8, -1, -1, 8, 8/

      data (colm(iaux,  6),iaux=1, 6)      / 3, 0, -3, 3, 0, 24/
      data (colm(iaux,  6),iaux=7, 12)     / -1, -1, -1, 8, 8, 8/

      data (colm(iaux,  7),iaux=1, 6)      / 8, -1, 8, -1, 8, -1/
      data (colm(iaux,  7),iaux=7, 12)     / 24, 0, 3, -3, 0, 3/

      data (colm(iaux,  8),iaux=1, 6)      / -1, 8, 8, 8, 8, -1/
      data (colm(iaux,  8),iaux=7, 12)     / 0, 24, 0, 0, 3, 0/

      data (colm(iaux,  9),iaux=1, 6)      / 8, 8, 8, -1, -1, -1/
      data (colm(iaux,  9),iaux=7, 12)     / 3, 0, 24, 3, 0, -3/

      data (colm(iaux,  10),iaux=1, 6)     / -1, 8, -1, 8, -1, 8/
      data (colm(iaux,  10),iaux=7, 12)    / -3, 0, 3, 24, 0, 3/

      data (colm(iaux,  11),iaux=1, 6)     / 8, 8, -1, -1, 8, 8/
      data (colm(iaux,  11),iaux=7, 12)    / 0, 3,  0, 0, 24, 0/

      data (colm(iaux,  12),iaux=1, 6)     / -1, -1, -1, 8, 8, 8/
      data (colm(iaux,  12),iaux=7, 12)    / 3, 0, -3, 3, 0, 24/

      iph1 = 1
      iph2 = 1
      if (ig1.le.2) iph1=-1
      if (ig2.le.2) iph2=-1
      iph = iph1*iph2

      do i=1,4
         zm(i,1) = zCt2g02(iq1,iq2,iq3,ig1,ig2,iq4,i)/nc
         zm(i,2) = zCt2g03(iq1,ig1,iq2,iq3,ig2,iq4,i)/nc
         zm(i,3) = zCt2g01(iq1,ig1,ig2,iq2,iq3,iq4,i)/nc
         zm(i,4) = zCt2g02(iq1,iq2,iq3,ig2,ig1,iq4,i)/nc
         zm(i,5) = zCt2g03(iq1,ig2,iq2,iq3,ig1,iq4,i)/nc
         zm(i,6) = zCt2g01(iq1,ig2,ig1,iq2,iq3,iq4,i)/nc

         zm(i,7) = zC2g02(iq1,iq4,iq3,ig1,ig2,iq2,i)
         zm(i,8) = zC2g03(iq1,ig1,iq4,iq3,ig2,iq2,i)
         zm(i,9) = zC2g01(iq1,ig1,ig2,iq4,iq3,iq2,i)
         zm(i,10)= zC2g02(iq1,iq4,iq3,ig2,ig1,iq2,i)
         zm(i,11)= zC2g03(iq1,ig2,iq4,iq3,ig1,iq2,i)
         zm(i,12)= zC2g01(iq1,ig2,ig1,iq4,iq3,iq2,i)
      enddo

      do i=5,8
         zm(i,1) = zCt2g02(iq1,iq2,iq3,ig1,ig2,iq4,i)/nc
         zm(i,2) = zCt2g03(iq1,ig1,iq2,iq3,ig2,iq4,i)/nc
         zm(i,3) = zCt2g01(iq1,ig1,ig2,iq2,iq3,iq4,i)/nc
         zm(i,4) = iph*dconjg(zCt2g02(iq1,iq2,iq3,ig2,ig1,iq4,13-i))/nc
         zm(i,5) = iph*dconjg(zCt2g03(iq1,ig2,iq2,iq3,ig1,iq4,13-i))/nc
         zm(i,6) = iph*dconjg(zCt2g01(iq1,ig2,ig1,iq2,iq3,iq4,13-i))/nc

         zm(i,7) = zC2g02(iq1,iq4,iq3,ig1,ig2,iq2,i)
         zm(i,8) = zC2g03(iq1,ig1,iq4,iq3,ig2,iq2,i)
         zm(i,9) = zC2g01(iq1,ig1,ig2,iq4,iq3,iq2,i)
         zm(i,10)= iph*dconjg(zC2g02(iq1,iq4,iq3,ig2,ig1,iq2,13-i))
         zm(i,11)= iph*dconjg(zC2g03(iq1,ig2,iq4,iq3,ig1,iq2,13-i))
         zm(i,12)= iph*dconjg(zC2g01(iq1,ig2,ig1,iq4,iq3,iq2,13-i))
      enddo


      ilc=0  ! 2 for SLC Ct-type; 1 for leading colour, 0 for full colour

      if(ilc.eq.1)then

      wt=0d0
      do i=1,8
         do j=7,12
            wt=wt+2d0*dreal(zm(i,j)*dconjg(zm(i,j)))
         enddo
      enddo
      fac=(4d0*pi*as)**4*(nc**2-1d0)*nc**2

      elseif(ilc.eq.2)then
         wt=0d0
         do i=1,8
         do j=1,6
            wt=wt+2d0*dreal(zm(i,j)*dconjg(zm(i,j)))
         enddo
      enddo
      fac=(4d0*pi*as)**4*(nc**2-1d0)*nc**2
      
      elseif(ilc.eq.0)then
         ztmp=(0d0,0d0)
         do i=1,8
            do j=1,12
               do k=1,12
                  ztmp=ztmp+2d0*zm(i,j)*colm(j,k)*dconjg(zm(i,k))
               enddo
            enddo
         enddo
         wt = dreal(ztmp)
         fac=(4d0*pi*as)**4*(nc**2-1d0)/nc
       endif

      FullqQC2g0=fac*wt
c      write(*,*)"FullC2g0",FullC2g0
      return
      end
************************************************************************
* non-identical 4 quark 2 gluon scattering used to test qQ subtraction
* term against
*                      q Qb Q qb g g
*
      function FullqQC2g0LC(iq1,iq2,iq3,iq4,ig1,ig2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16(z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      dimension zm(8,12)

      idebug=0

      fac=(4d0*pi*as)**4*(nc**2-1d0)*nc**2

      wt =
     .   + C2g01(iq1,ig1,ig2,iq2,iq3,iq4)
     .   + C2g01(iq1,ig2,ig1,iq2,iq3,iq4)
     .   + C2g02(iq1,iq2,iq3,ig1,ig2,iq4)
     .   + C2g02(iq1,iq2,iq3,ig2,ig1,iq4)
     .   + C2g03(iq1,ig1,iq2,iq3,ig2,iq4)
     .   + C2g03(iq1,ig2,iq2,iq3,ig1,iq4)

      if(idebug.eq.1)then
       write(*,*)iq1,ig1,ig2,iq2,iq3,iq4,C2g01(iq1,ig1,ig2,iq2,iq3,iq4)
       write(*,*)iq1,ig2,ig1,iq2,iq3,iq4,C2g01(iq1,ig2,ig1,iq2,iq3,iq4)
       write(*,*)iq1,iq2,iq3,ig1,ig2,iq4,C2g02(iq1,iq2,iq3,ig1,ig2,iq4)
       write(*,*)iq1,iq2,iq3,ig2,ig1,iq4,C2g02(iq1,iq2,iq3,ig2,ig1,iq4)
       write(*,*)iq1,ig1,iq2,iq3,ig2,iq4,C2g03(iq1,ig1,iq2,iq3,ig2,iq4)
       write(*,*)iq1,ig2,iq2,iq3,ig1,iq4,C2g03(iq1,ig2,iq2,iq3,ig1,iq4)
       write(*,*)
      endif


      FullqQC2g0LC=fac*wt
c      write(*,*)"testqQC2g0",testqQC2g0
      return
      end
*****************************************************************************
* full non-identical 4 quark 2 gluon scattering used to test qQ subtraction
* term against
*                      q qb Q Qb g g
*
*****************************************************************************    
      function FullC2g0(iq1,iq2,iq3,iq4,ig1,ig2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16(z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      dimension zm(8,12)

      idebug=0

      fac=(4d0*pi*as)**4*(nc**2-1d0)*nc**2
      wt =
     .        ( C2g01(iq1,ig1,ig2,iq4,iq3,iq2)
     .        + C2g01(iq1,ig2,ig1,iq4,iq3,iq2)
     .        + C2g02(iq1,iq4,iq3,ig1,ig2,iq2)
     .        + C2g02(iq1,iq4,iq3,ig2,ig1,iq2)
     .        + C2g03(iq1,ig1,iq4,iq3,ig2,iq2)
     .        + C2g03(iq1,ig2,iq4,iq3,ig1,iq2) )
     .     -1d0/nc**2* (
     .          Ctb2g012(iq1,iq4,iq3,iq2,ig1,ig2)
     .         +Ctb2g03(iq1,iq4,iq3,iq2,ig1,ig2) )
     .     + 1d0/nc**2 *
     .        ( Ct2g01(iq1,ig1,ig2,iq4,iq3,iq2)
     .        + Ct2g01(iq1,ig2,ig1,iq4,iq3,iq2)
     .        + Ct2g02(iq1,iq4,iq3,ig1,ig2,iq2)
     .        + Ct2g02(iq1,iq4,iq3,ig2,ig1,iq2)
     .        + Ct2g03(iq1,ig1,iq4,iq3,ig2,iq2)
     .        + Ct2g03(iq1,ig2,iq4,iq3,ig1,iq2) )
     .     + 1d0/nc**2 *
     .        ( Ctt2g01(iq1,ig1,ig2,iq4,iq3,iq2)
     .        + Ctt2g01(iq1,ig2,ig1,iq4,iq3,iq2)
     .        + Ctt2g02(iq1,iq4,iq3,ig1,ig2,iq2)
     .        + Ctt2g02(iq1,iq4,iq3,ig2,ig1,iq2)
     .        + Ctt2g03(iq1,ig1,iq4,iq3,ig2,iq2)
     .        + Ctt2g03(iq1,ig2,iq4,iq3,ig1,iq2) )
     .     +1d0/nc**4* (
     .         +Cttb2g0(iq1,iq4,iq3,iq2,ig1,ig2)
     .         -Cttb2g012(iq1,iq4,iq3,iq2,ig1,ig2)
     .         -Cttb2g03(iq1,iq4,iq3,iq2,ig1,ig2) 
     .     )

      FullC2g0=fac*wt
c      write(*,*)"testqQC2g0",testqQC2g0
      return
      end

      function FullD2g0(iq1,iq2,iq3,iq4,ig1,ig2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16(z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      dimension zm(8,12)

      idebug=0

      fac=-(4d0*pi*as)**4*(nc**2-1d0)*nc
      wt =
     .        ( D2g01(iq1,ig1,ig2,iq4,iq3,iq2)
     .        + D2g01(iq1,ig2,ig1,iq4,iq3,iq2)
     .        + D2g02(iq1,iq4,iq3,ig1,ig2,iq2)
     .        + D2g02(iq1,iq4,iq3,ig2,ig1,iq2)
     .        + D2g03(iq1,ig1,iq4,iq3,ig2,iq2)
     .        + D2g03(iq1,ig2,iq4,iq3,ig1,iq2)
     .        + Db2g012(iq1,iq4,iq3,iq2,ig1,ig2)
     .        + Db2g012(iq1,iq4,iq3,iq2,ig2,ig1)
     .        + Db2g03(iq1,iq4,iq3,iq2,ig1,ig2)
     .        + Dtb2g0(iq1,iq4,iq3,iq2,ig1,ig2)
     .        + Dtb2g0(iq1,iq4,iq3,iq2,ig2,ig1) )
     .     - 1d0/nc**2* (
     .          Dt2g0(iq1,iq4,iq3,iq2,ig1,ig2)
     .        + Dtt2g012(iq1,iq4,iq3,iq2,ig1,ig2)
     .        + Dtt2g03(iq1,iq4,iq3,iq2,ig1,ig2) )
     .     + 1d0/nc**2 * (
     .          Dttb2g012(iq1,iq4,iq3,iq2,ig1,ig2)
     .        + Dttb2g012(iq1,iq4,iq3,iq2,ig2,ig1)
     .        + Dttb2g03(iq1,iq4,iq3,iq2,ig1,ig2)
     .        + Dttb2g0(iq1,iq4,iq3,iq2,ig2,ig1)
     .        + Dttb2g0(iq1,iq4,iq3,iq2,ig1,ig2) ) 
     .     -1d0/nc**4 *
     .          Dttt2g0(iq1,iq4,iq3,iq2,ig1,ig2)
      FullD2g0=fac*wt
c      write(*,*)"testqQC2g0",testqQC2g0
      return
      end

      
*****************************************************************************
*     Six parton tree-level 4q2g
*****************************************************************************
*                      q qb Q Qb g g
*
*****************************************************************************
      function FullC2g0ID(iq1,iqb1,iq2,iqb2,ig1,ig2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      common /koppel/nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      parameter(pi=3.141592653589793238d0)

      wt = 0d0
      
      wt = FullC2g0(iq1,iqb1,iq2,iqb2,ig1,ig2)
     .   + FullC2g0(iq1,iqb2,iq2,iqb1,ig1,ig2)
     .   + FullD2g0(iq1,iqb1,iq2,iqb2,ig1,ig2)

      FullC2g0ID = wt

      return
      end
      
      
**********************************************
* Wrapper functions for returning the normal
* matrix element for situations where either of the quarks
* in the second quark line are in the initial state.
* Otherwise it will symmetrise over the quark pair.
*
*****************************************************************************
      function sC2g0X(iq1,iqb1,iq2,iqb2,ig1,ig2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      if((iq2.le.2).or.(iqb2.le.2))then
         sC2g0X=C2g0X(iq1,iqb1,iq2,iqb2,ig1,ig2)
      elseif((ig1.le.2).and.(ig2.le.2))then
         sC2g0X=C2g0X(iq1,iqb1,iq2,iqb2,ig1,ig2)
      else
         sC2g0X=0.5d0*(C2g0X(iq1,iqb1,iq2,iqb2,ig1,ig2)
     .                +C2g0X(iq1,iqb1,iqb2,iq2,ig1,ig2))
      endif

      return
      end
*****************************************************************************
      function sCt2g0(iq1,iqb1,iq2,iqb2,ig1,ig2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      if((iq2.le.2).or.(iqb2.le.2))then
         sCt2g0=Ct2g0(iq1,iqb1,iq2,iqb2,ig1,ig2)
      elseif((ig1.le.2).and.(ig2.le.2))then
         sCt2g0=Ct2g0(iq1,iqb1,iq2,iqb2,ig1,ig2)
      else
         sCt2g0=0.5d0*(Ct2g0(iq1,iqb1,iq2,iqb2,ig1,ig2)
     .                +Ct2g0(iq1,iqb1,iqb2,iq2,ig1,ig2))
      endif

      return
      end
*****************************************************************************
      function sCtt2g0(iq1,iqb1,iq2,iqb2,ig1,ig2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      if((iq2.le.2).or.(iqb2.le.2))then
         sCtt2g0=Ctt2g0(iq1,iqb1,iq2,iqb2,ig1,ig2)
      elseif((ig1.le.2).and.(ig2.le.2))then
         sCtt2g0=Ctt2g0(iq1,iqb1,iq2,iqb2,ig1,ig2)
      else
         sCtt2g0=0.5d0*(Ctt2g0(iq1,iqb1,iq2,iqb2,ig1,ig2)
     .                 +Ctt2g0(iq1,iqb1,iqb2,iq2,ig1,ig2))
      endif

      return
      end


************************************************************************
* non-identical 4 quark 2 gluon scattering used to test qQ subtraction
* term against
*                      q qb Q Qb g g
*
      function C2g0X(iq1,iqb1,iq2,iqb2,ig1,ig2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16(z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      dimension zm(8,12)

      C2g0X =
     .   + C2g01(iq1,ig1,ig2,iqb2,iq2,iqb1)
     .   + C2g02(iq1,iqb2,iq2,ig1,ig2,iqb1)
     .   + C2g03(iq1,ig1,iqb2,iq2,ig2,iqb1)

      return
      end


************************************************************************
* non-identical 4 quark 2 gluon scattering used to test gg NF^2 X topology
* subtraction term
*
      function ggC2g0X(iq1,ig1,ig2,iq2,iq3,iq4)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16(z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)

      idebug=0

      wt = C2g01(iq1,ig1,ig2,iq2,iq3,iq4)

      if(idebug.eq.1)then
       write(*,*)iq1,ig1,ig2,iq2,iq3,iq4,C2g01(iq1,ig1,ig2,iq2,iq3,iq4)
       write(*,*)
      endif

      ggC2g0X=wt

      return
      end

************************************************************************
* non-identical 4 quark 2 gluon scattering used to test gg NF^2 Y topology
* subtraction term
*
      function ggC2g0Y(iq1,ig1,iq2,iq3,ig2,iq4)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16(z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)

      idebug=0

       wt = C2g03(iq1,ig1,iq2,iq3,ig2,iq4)

      if(idebug.eq.1)then
       write(*,*)iq1,ig1,iq2,iq3,ig2,iq4,C2g03(iq1,ig1,iq2,iq3,ig2,iq4)
       write(*,*)
      endif

      ggC2g0Y=wt
      return
      end

************************************************************************
*  identical 4 quark 2 gluon scattering
*
*                      q qb q qb g g
*
      function FullqqC2g0id(iq1,iq2,iq3,iq4,ig1,ig2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16(z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      dimension zm(12,12),zm1(12,12),zm2(12,12),zm3(12,12),zm4(12,12),colm(12,12),ihel(12)

c     colour matrix

      data (colm(iaux,  1),iaux=1, 6)      / 24, 0, 3, -3, 0, 3/
      data (colm(iaux,  1),iaux=7, 12)     / 8, -1, 8, -1, 8, -1/

      data (colm(iaux,  2),iaux=1, 6)      / 0, 24, 0, 0, 3, 0/
      data (colm(iaux,  2),iaux=7, 12)     / -1, 8, 8, 8, 8, -1/

      data (colm(iaux,  3),iaux=1, 6)      / 3, 0, 24, 3, 0, -3/
      data (colm(iaux,  3),iaux=7, 12)     / 8, 8, 8, -1, -1, -1/

      data (colm(iaux,  4),iaux=1, 6)      / -3, 0, 3, 24, 0, 3/
      data (colm(iaux,  4),iaux=7, 12)     / -1, 8, -1, 8, -1, 8/

      data (colm(iaux,  5),iaux=1, 6)      / 0, 3, 0, 0, 24, 0/
      data (colm(iaux,  5),iaux=7, 12)     / 8, 8, -1, -1, 8, 8/

      data (colm(iaux,  6),iaux=1, 6)      / 3, 0, -3, 3, 0, 24/
      data (colm(iaux,  6),iaux=7, 12)     / -1, -1, -1, 8, 8, 8/

      data (colm(iaux,  7),iaux=1, 6)      / 8, -1, 8, -1, 8, -1/
      data (colm(iaux,  7),iaux=7, 12)     / 24, 0, 3, -3, 0, 3/

      data (colm(iaux,  8),iaux=1, 6)      / -1, 8, 8, 8, 8, -1/
      data (colm(iaux,  8),iaux=7, 12)     / 0, 24, 0, 0, 3, 0/

      data (colm(iaux,  9),iaux=1, 6)      / 8, 8, 8, -1, -1, -1/
      data (colm(iaux,  9),iaux=7, 12)     / 3, 0, 24, 3, 0, -3/

      data (colm(iaux,  10),iaux=1, 6)     / -1, 8, -1, 8, -1, 8/
      data (colm(iaux,  10),iaux=7, 12)    / -3, 0, 3, 24, 0, 3/

      data (colm(iaux,  11),iaux=1, 6)     / 8, 8, -1, -1, 8, 8/
      data (colm(iaux,  11),iaux=7, 12)    / 0, 3,  0, 0, 24, 0/

      data (colm(iaux,  12),iaux=1, 6)     / -1, -1, -1, 8, 8, 8/
      data (colm(iaux,  12),iaux=7, 12)    / 3, 0, -3, 3, 0, 24/

      do i=1,8
         do j=1,12
            zm1(i,j)=(0d0,0d0)
            zm2(i,j)=(0d0,0d0)
            zm3(i,j)=(0d0,0d0)
            zm4(i,j)=(0d0,0d0)
         enddo
      enddo

      ihel(1)=1
      ihel(2)=3
      ihel(3)=2
      ihel(4)=4
      ihel(5)=5
      ihel(6)=7
      ihel(7)=6
      ihel(8)=8


      do i=1,4
         zm1(i,1) = zCt2g02(iq1,iq2,iq3,ig1,ig2,iq4,i)/nc
         zm1(i,2) = zCt2g03(iq1,ig1,iq2,iq3,ig2,iq4,i)/nc
         zm1(i,3) = zCt2g01(iq1,ig1,ig2,iq2,iq3,iq4,i)/nc
         zm1(i,4) = zCt2g02(iq1,iq2,iq3,ig2,ig1,iq4,i)/nc
         zm1(i,5) = zCt2g03(iq1,ig2,iq2,iq3,ig1,iq4,i)/nc
         zm1(i,6) = zCt2g01(iq1,ig2,ig1,iq2,iq3,iq4,i)/nc

         zm1(i,7) = zC2g02(iq1,iq4,iq3,ig1,ig2,iq2,i)
         zm1(i,8) = zC2g03(iq1,ig1,iq4,iq3,ig2,iq2,i)
         zm1(i,9) = zC2g01(iq1,ig1,ig2,iq4,iq3,iq2,i)
         zm1(i,10)= zC2g02(iq1,iq4,iq3,ig2,ig1,iq2,i)
         zm1(i,11)= zC2g03(iq1,ig2,iq4,iq3,ig1,iq2,i)
         zm1(i,12)= zC2g01(iq1,ig2,ig1,iq4,iq3,iq2,i)
      enddo

      do i=5,8
         zm1(i,1) = zCt2g02(iq1,iq2,iq3,ig1,ig2,iq4,i)/nc
         zm1(i,2) = zCt2g03(iq1,ig1,iq2,iq3,ig2,iq4,i)/nc
         zm1(i,3) = zCt2g01(iq1,ig1,ig2,iq2,iq3,iq4,i)/nc
         zm1(i,4) = dconjg(zCt2g02(iq1,iq2,iq3,ig2,ig1,iq4,13-i))/nc
         zm1(i,5) = dconjg(zCt2g03(iq1,ig2,iq2,iq3,ig1,iq4,13-i))/nc
         zm1(i,6) = dconjg(zCt2g01(iq1,ig2,ig1,iq2,iq3,iq4,13-i))/nc

         zm1(i,7) = zC2g02(iq1,iq4,iq3,ig1,ig2,iq2,i)
         zm1(i,8) = zC2g03(iq1,ig1,iq4,iq3,ig2,iq2,i)
         zm1(i,9) = zC2g01(iq1,ig1,ig2,iq4,iq3,iq2,i)
         zm1(i,10)= dconjg(zC2g02(iq1,iq4,iq3,ig2,ig1,iq2,13-i))
         zm1(i,11)= dconjg(zC2g03(iq1,ig2,iq4,iq3,ig1,iq2,13-i))
         zm1(i,12)= dconjg(zC2g01(iq1,ig2,ig1,iq4,iq3,iq2,13-i))
      enddo

      do i=1,4
         zm2(i,1) = zCt2g02(iq1,iq4,iq3,ig1,ig2,iq2,i)/nc
         zm2(i,2) = zCt2g03(iq1,ig1,iq4,iq3,ig2,iq2,i)/nc
         zm2(i,3) = zCt2g01(iq1,ig1,ig2,iq4,iq3,iq2,i)/nc
         zm2(i,4) = zCt2g02(iq1,iq4,iq3,ig2,ig1,iq2,i)/nc
         zm2(i,5) = zCt2g03(iq1,ig2,iq4,iq3,ig1,iq2,i)/nc
         zm2(i,6) = zCt2g01(iq1,ig2,ig1,iq4,iq3,iq2,i)/nc

         zm2(i,7) = zC2g02(iq1,iq2,iq3,ig1,ig2,iq4,i)
         zm2(i,8) = zC2g03(iq1,ig1,iq2,iq3,ig2,iq4,i)
         zm2(i,9) = zC2g01(iq1,ig1,ig2,iq2,iq3,iq4,i)
         zm2(i,10)= zC2g02(iq1,iq2,iq3,ig2,ig1,iq4,i)
         zm2(i,11)= zC2g03(iq1,ig2,iq2,iq3,ig1,iq4,i)
         zm2(i,12)= zC2g01(iq1,ig2,ig1,iq2,iq3,iq4,i)
      enddo

      do i=5,8
         zm2(i,1) = zCt2g02(iq1,iq4,iq3,ig1,ig2,iq2,i)/nc
         zm2(i,2) = zCt2g03(iq1,ig1,iq4,iq3,ig2,iq2,i)/nc
         zm2(i,3) = zCt2g01(iq1,ig1,ig2,iq4,iq3,iq2,i)/nc
         zm2(i,4) = dconjg(zCt2g02(iq1,iq4,iq3,ig2,ig1,iq2,13-i))/nc
         zm2(i,5) = dconjg(zCt2g03(iq1,ig2,iq4,iq3,ig1,iq2,13-i))/nc
         zm2(i,6) = dconjg(zCt2g01(iq1,ig2,ig1,iq4,iq3,iq2,13-i))/nc

         zm2(i,7) = zC2g02(iq1,iq2,iq3,ig1,ig2,iq4,i)
         zm2(i,8) = zC2g03(iq1,ig1,iq2,iq3,ig2,iq4,i)
         zm2(i,9) = zC2g01(iq1,ig1,ig2,iq2,iq3,iq4,i)
         zm2(i,10)= dconjg(zC2g02(iq1,iq2,iq3,ig2,ig1,iq4,13-i))
         zm2(i,11)= dconjg(zC2g03(iq1,ig2,iq2,iq3,ig1,iq4,13-i))
         zm2(i,12)= dconjg(zC2g01(iq1,ig2,ig1,iq2,iq3,iq4,13-i))
      enddo

      do i=1,8
         do j=1,12
            zm3(i,j)=zm2(ihel(i),j)
         enddo
c         zm3(11,i)=dconjg(zm3(11,i))
      enddo

      do i=1,8
c         zm4(i, 1)=zm1(i, 6)-zm3(i,12)
c         zm4(i, 2)=zm1(i, 5)-zm3(i,11)
c         zm4(i, 3)=zm1(i, 4)-zm3(i,10)
c         zm4(i, 4)=zm1(i, 3)-zm3(i, 9)
c         zm4(i, 5)=zm1(i, 2)-zm3(i, 8)
c         zm4(i, 6)=zm1(i, 1)-zm3(i, 7)
c         zm4(i, 7)=zm1(i,10)-zm3(i, 4)
c         zm4(i, 8)=zm1(i, 8)-zm3(i, 2)
c         zm4(i, 9)=zm1(i,12)-zm3(i, 6)
c         zm4(i,10)=zm1(i, 7)-zm3(i, 1)
c         zm4(i,11)=zm1(i,11)-zm3(i, 5)
c         zm4(i,12)=zm1(i, 9)-zm3(i, 3)
         zm4(i, 1)=zm1(i, 6)-zm3(i,12)
         zm4(i, 2)=zm1(i, 5)-zm3(i,11)
         zm4(i, 3)=zm1(i, 4)-zm3(i,10)
         zm4(i, 4)=zm1(i, 3)-zm3(i, 9)
         zm4(i, 5)=zm1(i, 2)-zm3(i, 8)
         zm4(i, 6)=zm1(i, 1)-zm3(i, 7)
         zm4(i, 7)=zm1(i,10)-zm3(i, 4)
         zm4(i, 8)=zm1(i, 8)-zm3(i, 2)
         zm4(i, 9)=zm1(i,12)-zm3(i, 6)
         zm4(i,10)=zm1(i, 7)-zm3(i, 1)
         zm4(i,11)=zm1(i,11)-zm3(i, 5)
         zm4(i,12)=zm1(i, 9)-zm3(i, 3)
      enddo


      ztmp=(0d0,0d0)
         do i=1,8
            do j=1,12
               do k=1,12
                  ztmp=ztmp+2d0*zm4(i,j)*colm(j,k)*dconjg(zm4(i,k))
               enddo
            enddo
         enddo
      wt = dreal(ztmp)

      fac=(4d0*pi*as)**4*(nc**2-1d0)/nc

      FullqqC2g0id=fac*wt
c      write(*,*)"FullqqC2g0id",FullqqC2g0
      return
      end
************************************************************************
* identical 4 quark 2 gluon scattering used to test qq subtraction term
*                      q qb q qb g g
*
      function qqC2g0idLC(iq1,iq2,iq3,iq4,ig1,ig2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16(z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      dimension zm(8,12)

      idebug=0

      fac=(4d0*pi*as)**4*(nc**2-1d0)*nc**2

      wt =
     .   + C2g01(iq1,ig1,ig2,iq2,iq3,iq4)
     .   + C2g01(iq1,ig2,ig1,iq2,iq3,iq4)
     .   + C2g02(iq1,iq2,iq3,ig1,ig2,iq4)
     .   + C2g02(iq1,iq2,iq3,ig2,ig1,iq4)
     .   + C2g03(iq1,ig1,iq2,iq3,ig2,iq4)
     .   + C2g03(iq1,ig2,iq2,iq3,ig1,iq4)

     .   + C2g01(iq1,ig1,ig2,iq4,iq3,iq2)
     .   + C2g01(iq1,ig2,ig1,iq4,iq3,iq2)
     .   + C2g02(iq1,iq4,iq3,ig1,ig2,iq2)
     .   + C2g02(iq1,iq4,iq3,ig2,ig1,iq2)
     .   + C2g03(iq1,ig1,iq4,iq3,ig2,iq2)
     .   + C2g03(iq1,ig2,iq4,iq3,ig1,iq2)

      if(idebug.eq.1)then
       write(*,*)iq1,ig1,ig2,iq2,iq3,iq4,C2g01(iq1,ig1,ig2,iq2,iq3,iq4)
       write(*,*)iq1,ig2,ig1,iq2,iq3,iq4,C2g01(iq1,ig2,ig1,iq2,iq3,iq4)
       write(*,*)iq1,iq2,iq3,ig1,ig2,iq4,C2g02(iq1,iq2,iq3,ig1,ig2,iq4)
       write(*,*)iq1,iq2,iq3,ig2,ig1,iq4,C2g02(iq1,iq2,iq3,ig2,ig1,iq4)
       write(*,*)iq1,ig1,iq2,iq3,ig2,iq4,C2g03(iq1,ig1,iq2,iq3,ig2,iq4)
       write(*,*)iq1,ig2,iq2,iq3,ig1,iq4,C2g03(iq1,ig2,iq2,iq3,ig1,iq4)

       write(*,*)iq1,ig1,ig2,iq4,iq3,iq2,C2g01(iq1,ig1,ig2,iq4,iq3,iq2)
       write(*,*)iq1,ig2,ig1,iq4,iq3,iq2,C2g01(iq1,ig2,ig1,iq4,iq3,iq2)
       write(*,*)iq1,iq4,iq3,ig1,ig2,iq2,C2g02(iq1,iq4,iq3,ig1,ig2,iq2)
       write(*,*)iq1,iq4,iq3,ig2,ig1,iq2,C2g02(iq1,iq4,iq3,ig2,ig1,iq2)
       write(*,*)iq1,ig1,iq4,iq3,ig2,iq2,C2g03(iq1,ig1,iq4,iq3,ig2,iq2)
       write(*,*)iq1,ig2,iq4,iq3,ig1,iq2,C2g03(iq1,ig2,iq4,iq3,ig1,iq2)
      endif


      qqC2g0idLC=fac*wt
c      write(*,*)"testqQC2g0",testqQC2g0
      return
      end

************************************************************************
* non-identical 4 quark 2 gluon squared amplitude (2,0) configuration
*
*                      q g g Qb Q qb
*
      function C2g01(iq1,ig1,ig2,iq2,iq3,iq4)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

c      write(*,*)" hello C2g01"
      wt=0d0

      do ihel=1,8
      ztmp=zC2g01(iq1,ig1,ig2,iq2,iq3,iq4,ihel)
      wt=wt+dreal(dconjg(ztmp)*ztmp)
      enddo

      C2g01=2d0*wt

c      write(*,*)"C2g01",C2g01
*
      return
      end

************************************************************************
* non-identical 4 quark 2 gluon squared amplitude (0,2) configuration
*
*                      q Qb Q g g qb
*
      function C2g02(iq1,iq2,iq3,ig1,ig2,iq4)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

c      write(*,*)" hello C2g02"
      wt=0d0

      do ihel=1,8
      ztmp=zC2g02(iq1,iq2,iq3,ig1,ig2,iq4,ihel)
      wt=wt+dreal(dconjg(ztmp)*ztmp)
      enddo

      C2g02=2D0*wt

*      write(*,*)"C2g02",C2g02
*
      return
      end
************************************************************************
* non-identical 4 quark 2 gluon squared amplitude (1,1) configuration
*
*                      q g Qb Q g qb
*
      function C2g03(iq1,ig1,iq2,iq3,ig2,iq4)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

c      write(*,*)" hello C2g02"
      wt=0d0

      do ihel=1,8
      ztmp=zC2g03(iq1,ig1,iq2,iq3,ig2,iq4,ihel)
      wt=wt+dreal(dconjg(ztmp)*ztmp)
      enddo

      C2g03=2D0*wt

c      write(*,*)"C2g03",C2g03
*
      return
      end

************************************************************************
*     non-identical 4 quark 2 gluon squared amplitude (0,2)+(2,0) configuration
*     interference of LC amplitude       
*
*                      q qb Q Qb g g
*
      function Ctb2g012(iq1,iq2,iq3,iq4,ig1,ig2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      iph1 = 1
      iph2 = 1
      if (ig1.le.2) iph1=-1
      if (ig2.le.2) iph2=-1
      iph = iph1*iph2
c      write(*,*)" hello Ctb2g012"
      wt=0d0
      do ihel=1,4
         ztmp  =  zC2g02(iq1,iq2,iq3,ig1,ig2,iq4,ihel)
     .           +zC2g02(iq1,iq2,iq3,ig2,ig1,iq4,ihel)
     .           -zC2g01(iq1,ig1,ig2,iq2,iq3,iq4,ihel)
     .           -zC2g01(iq1,ig2,ig1,iq2,iq3,iq4,ihel)    
         wt=wt+dreal(conjg(ztmp)*ztmp)
      enddo
      do ihel=5,8
         ztmp  = zC2g02(iq1,iq2,iq3,ig1,ig2,iq4,ihel)
     .          -zC2g01(iq1,ig1,ig2,iq2,iq3,iq4,ihel)
     .          +iph*dconjg(zC2g02(iq1,iq2,iq3,ig2,ig1,iq4,13-ihel))
     .          -iph*dconjg(zC2g01(iq1,ig2,ig1,iq2,iq3,iq4,13-ihel))
         wt=wt+dreal(conjg(ztmp)*ztmp)
      enddo
      
      Ctb2g012=2D0*wt

c      write(*,*)"Ctb2g012",Ctb2g012

      return
      end

      
************************************************************************
*     non-identical 4 quark 2 gluon squared amplitude (1,1) configuration
*     interference of LC amplitude       
*
*                      q qb Q Qb g g
*
      function Ctb2g03(iq1,iq2,iq3,iq4,ig1,ig2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      iph1 = 1
      iph2 = 1
      if (ig1.le.2) iph1=-1
      if (ig2.le.2) iph2=-1
      iph = iph1*iph2
c      write(*,*)" hello Ctb2g03"
      wt=0d0

      do ihel=1,4
         ztmp  =  zC2g03(iq1,ig1,iq2,iq3,ig2,iq4,ihel)
     .           -zC2g03(iq1,ig2,iq2,iq3,ig1,iq4,ihel)
         wt=wt+dreal(conjg(ztmp)*ztmp)
      enddo
      do ihel=5,8
         ztmp  = zC2g03(iq1,ig1,iq2,iq3,ig2,iq4,ihel)
     .          -iph*conjg(zC2g03(iq1,ig2,iq2,iq3,ig1,iq4,13-ihel))
      wt=wt+dreal(conjg(ztmp)*ztmp)
      enddo
      
      Ctb2g03=2D0*wt

c      write(*,*)"Ctb2g03",Ctb2g03
*
      return
      end

************************************************************************
* non-identical 4 quark 2 gluon squared amplitude (2,0) configuration
*
      function Ct2g01(iq1,ig1,ig2,iq2,iq3,iq4)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

c      write(*,*)" hello Ct2g01"
      wt=0d0


      do ihel=1,8
      ztmp=zCt2g01(iq1,ig1,ig2,iq4,iq3,iq2,ihel)
      wt=wt+dreal(dconjg(ztmp)*ztmp)
      enddo

      Ct2g01=2d0*wt

*      write(*,*)"Ct2g01",Ct2g01
*
      return
      end

************************************************************************
* non-identical 4 quark 2 gluon squared amplitude (0,2) configuration
*
      function Ct2g02(iq1,iq2,iq3,ig1,ig2,iq4)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

c      write(*,*)" hello Ct2g02"
      wt=0d0

      do ihel=1,8
      ztmp=zCt2g02(iq1,iq4,iq3,ig1,ig2,iq2,ihel)
      wt=wt+dreal(dconjg(ztmp)*ztmp)
      enddo

      Ct2g02=2D0*wt

*      write(*,*)"Ct2g02",Ct2g02
*
      return
      end
************************************************************************
* non-identical 4 quark 2 gluon squared amplitude (1,1) configuration
*
      function Ct2g03(iq1,ig1,iq2,iq3,ig2,iq4)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

c      write(*,*)" hello Ct2g03"
      wt=0d0

      do ihel=1,8
      ztmp=zCt2g03(iq1,ig1,iq4,iq3,ig2,iq2,ihel)
      wt=wt+dreal(dconjg(ztmp)*ztmp)
      enddo

      Ct2g03=2d0*wt

*      write(*,*)"Ct2g03",Ct2g03
*
      return
      end
************************************************************************
* non-identical 4 quark 2 gluon squared amplitude (2,0) configuration
*
* LC interference with full SLC      
      function Ctt2g01(iq1,ig1,ig2,iq2,iq3,iq4)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

c      write(*,*)" hello Ct2g01"
      wt=0d0


      do ihel=1,8
      ztmp1=zC2g01(iq1,ig1,ig2,iq2,iq3,iq4,ihel) 
      ztmp2=(zCt2g01(iq1,ig1,ig2,iq4,iq3,iq2,ihel)
     .      +zCt2g02(iq1,iq4,iq3,ig1,ig2,iq2,ihel)
     .      +zCt2g03(iq1,ig1,iq4,iq3,ig2,iq2,ihel))
      wt=wt+2d0*dreal(dconjg(ztmp1)*ztmp2)
c      if (ihel.le.4) write(6,*) ihel,abs(ztmp2/ztmp3)
      enddo

      Ctt2g01=2D0*wt

*      write(*,*)"Ct2g01",Ct2g01
*
      return
      end

************************************************************************
* non-identical 4 quark 2 gluon squared amplitude (0,2) configuration
*
*     LC interference with full SLC
      function Ctt2g02(iq1,iq2,iq3,ig1,ig2,iq4)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      iph1 = 1
      iph2 = 1
      if (ig1.le.2) iph1=-1
      if (ig2.le.2) iph2=-1
      iph = iph1*iph2
c      write(*,*)" hello Ct2g02"
      wt=0d0

      do ihel=1,4
      ztmp1=zC2g02(iq1,iq2,iq3,ig1,ig2,iq4,ihel)
      ztmp2=(zCt2g01(iq1,ig1,ig2,iq4,iq3,iq2,ihel)
     .      +zCt2g02(iq1,iq4,iq3,ig1,ig2,iq2,ihel)
     .      +zCt2g03(iq1,ig2,iq4,iq3,ig1,iq2,ihel))
      wt=wt+2d0*dreal(dconjg(ztmp1)*ztmp2)
      enddo
      do ihel=5,8
      ztmp1=zC2g02(iq1,iq2,iq3,ig1,ig2,iq4,ihel)
      ztmp2=(zCt2g01(iq1,ig1,ig2,iq4,iq3,iq2,ihel)
     .      +zCt2g02(iq1,iq4,iq3,ig1,ig2,iq2,ihel)
     .      +iph*dconjg(zCt2g03(iq1,ig2,iq4,iq3,ig1,iq2,13-ihel)))
      wt=wt+2d0*dreal(dconjg(ztmp1)*ztmp2)
      enddo

      Ctt2g02=2D0*wt

*      write(*,*)"Ct2g02",Ct2g02
*
      return
      end
************************************************************************
* non-identical 4 quark 2 gluon squared amplitude (1,1) configuration
*
*     LC interference with full SLC
      function Ctt2g03(iq1,ig1,iq2,iq3,ig2,iq4)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      iph1 = 1
      iph2 = 1
      if (ig1.le.2) iph1=-1
      if (ig2.le.2) iph2=-1
      iph = iph1*iph2
c      write(*,*)" hello Ct2g03"
      wt=0d0

      do ihel=1,4
      ztmp1=zC2g03(iq1,ig1,iq2,iq3,ig2,iq4,ihel)
      ztmp2=(zCt2g01(iq1,ig1,ig2,iq4,iq3,iq2,ihel)
     .      +zCt2g02(iq1,iq4,iq3,ig2,ig1,iq2,ihel)
     .      +zCt2g03(iq1,ig1,iq4,iq3,ig2,iq2,ihel)
     .      +zCt2g03(iq1,ig2,iq4,iq3,ig1,iq2,ihel))
      wt=wt+2d0*dreal(dconjg(ztmp1)*ztmp2)
      enddo
      do ihel=5,8
      ztmp1=zC2g03(iq1,ig1,iq2,iq3,ig2,iq4,ihel)
      ztmp2=(zCt2g01(iq1,ig1,ig2,iq4,iq3,iq2,ihel)
     .      +iph*dconjg(zCt2g02(iq1,iq4,iq3,ig2,ig1,iq2,13-ihel))
     .      +zCt2g03(iq1,ig1,iq4,iq3,ig2,iq2,ihel)
     .      +iph*dconjg(zCt2g03(iq1,ig2,iq4,iq3,ig1,iq2,13-ihel)))
      wt=wt+2d0*dreal(dconjg(ztmp1)*ztmp2)
      enddo

      Ctt2g03=2d0*wt

*      write(*,*)"Ct2g03",Ct2g03
*
      return
      end
************************************************************************
*     non-identical 4 quark 2 gluon squared amplitude configuration
*     square of abelian 4 quark 2 photon amplitude       
*
*                      q qb Q Qb g g
*
      function Cttb2g0(iq1,iq2,iq3,iq4,ig1,ig2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      iph1 = 1
      iph2 = 1
      if (ig1.le.2) iph1=-1
      if (ig2.le.2) iph2=-1
      iph = iph1*iph2
c      write(*,*)" hello Cttb2g0"
      wt=0d0
      do ihel=1,4
         ztmp  = +zCt2g02(iq1,iq4,iq3,ig1,ig2,iq2,ihel)
     .           +zCt2g02(iq1,iq4,iq3,ig2,ig1,iq2,ihel)
     .           +zCt2g01(iq1,ig1,ig2,iq4,iq3,iq2,ihel)
     .           +zCt2g01(iq1,ig2,ig1,iq4,iq3,iq2,ihel)
     .           +zCt2g03(iq1,ig1,iq4,iq3,ig2,iq2,ihel)
     .           +zCt2g03(iq1,ig2,iq4,iq3,ig1,iq2,ihel)
         wt=wt+2d0*dreal(conjg(ztmp)*ztmp)
      enddo
      do ihel=5,8
         ztmp  =+zCt2g02(iq1,iq4,iq3,ig1,ig2,iq2,ihel)
     .          +iph*conjg(zCt2g02(iq1,iq4,iq3,ig2,ig1,iq2,13-ihel))
     .          +zCt2g01(iq1,ig1,ig2,iq4,iq3,iq2,ihel)
     .          +iph*conjg(zCt2g01(iq1,ig2,ig1,iq4,iq3,iq2,13-ihel))
     .          +zCt2g03(iq1,ig1,iq4,iq3,ig2,iq2,ihel)
     .          +iph*conjg(zCt2g03(iq1,ig2,iq4,iq3,ig1,iq2,13-ihel))
         wt=wt+2d0*dreal(conjg(ztmp)*ztmp)
      enddo
      
      Cttb2g0=2D0*wt
*      write(*,*)"Cttb2g0",Cttb2g0

      return
      end

************************************************************************
*     non-identical 4 quark 2 gluon squared amplitude (0,2)+(2,0) configuration
*     interference of SLC amplitude       
*
*                      q qb Q Qb g g
*
      function Cttb2g012(iq1,iq2,iq3,iq4,ig1,ig2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      iph1 = 1
      iph2 = 1
      if (ig1.le.2) iph1=-1
      if (ig2.le.2) iph2=-1
      iph = iph1*iph2
c      write(*,*)" hello Ctb2g012"
      wt=0d0
      do ihel=1,4
         ztmp  =  zCt2g02(iq1,iq4,iq3,ig1,ig2,iq2,ihel)
     .           +zCt2g02(iq1,iq4,iq3,ig2,ig1,iq2,ihel)
     .           -zCt2g01(iq1,ig1,ig2,iq4,iq3,iq2,ihel)
     .           -zCt2g01(iq1,ig2,ig1,iq4,iq3,iq2,ihel)    
         wt=wt+dreal(conjg(ztmp)*ztmp)
      enddo
      do ihel=5,8
         ztmp  = zCt2g02(iq1,iq4,iq3,ig1,ig2,iq2,ihel)
     .          -zCt2g01(iq1,ig1,ig2,iq4,iq3,iq2,ihel)
     .          +iph*dconjg(zCt2g02(iq1,iq4,iq3,ig2,ig1,iq2,13-ihel))
     .          -iph*dconjg(zCt2g01(iq1,ig2,ig1,iq4,iq3,iq2,13-ihel))
         wt=wt+dreal(conjg(ztmp)*ztmp)
      enddo
      
      Cttb2g012=2D0*wt

*      write(*,*)"Cttb2g012",Cttb2g012

      return
      end

      
************************************************************************
*     non-identical 4 quark 2 gluon squared amplitude (1,1) configuration
*     interference of SLC amplitude       
*
*                      q qb Q Qb g g
*
      function Cttb2g03(iq1,iq2,iq3,iq4,ig1,ig2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      iph1 = 1
      iph2 = 1
      if (ig1.le.2) iph1=-1
      if (ig2.le.2) iph2=-1
      iph = iph1*iph2
c      write(*,*)" hello Cttb2g03"
      wt=0d0

      do ihel=1,4
         ztmp  =  zCt2g03(iq1,ig1,iq4,iq3,ig2,iq2,ihel)
     .           -zCt2g03(iq1,ig2,iq4,iq3,ig1,iq2,ihel)
         wt=wt+dreal(conjg(ztmp)*ztmp)
      enddo
      do ihel=5,8
         ztmp  = zCt2g03(iq1,ig1,iq4,iq3,ig2,iq2,ihel)
     .          -iph*conjg(zCt2g03(iq1,ig2,iq4,iq3,ig1,iq2,13-ihel))
      wt=wt+dreal(conjg(ztmp)*ztmp)
      enddo
      
      Cttb2g03=2D0*wt

c      write(*,*)"Cttb2g03",Cttb2g03
*
      return
      end

************************************************************************
* identical 4 quark 2 gluon squared amplitude (2,0) configuration
*
*                      q g g Qb Q qb
*
      function D2g01(iq1,ig1,ig2,iq2,iq3,iq4)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

c      write(*,*)" hello C2g01"
      wt=0d0

      do ihel=1,3,2
      ztmp1=zC2g01(iq1,ig1,ig2,iq2,iq3,iq4,ihel)
      ztmp2=zCt2g01(iq1,ig1,ig2,iq2,iq3,iq4,ihel)
      wt=wt+2d0*dreal(dconjg(ztmp1)*ztmp2)
      ztmp1=zCt2g01(iq1,ig1,ig2,iq4,iq3,iq2,ihel)
      ztmp2=zC2g01(iq1,ig1,ig2,iq4,iq3,iq2,ihel)
      wt=wt+2d0*dreal(dconjg(ztmp1)*ztmp2)
      enddo
      do ihel=5,8,3
      ztmp1=zC2g01(iq1,ig1,ig2,iq2,iq3,iq4,ihel)
      ztmp2=zCt2g01(iq1,ig1,ig2,iq2,iq3,iq4,ihel)
      wt=wt+2d0*dreal(dconjg(ztmp1)*ztmp2)
      ztmp1=zCt2g01(iq1,ig1,ig2,iq4,iq3,iq2,ihel)
      ztmp2=zC2g01(iq1,ig1,ig2,iq4,iq3,iq2,ihel)
      wt=wt+2d0*dreal(dconjg(ztmp1)*ztmp2)
      enddo

      D2g01=2d0*wt

c      write(*,*)"D2g01",D2g01
*
      return
      end

************************************************************************
* identical 4 quark 2 gluon squared amplitude (0,2) configuration
*
*                      q Qb Q g g qb
*
      function D2g02(iq1,iq2,iq3,ig1,ig2,iq4)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

c      write(*,*)" hello C2g02"
      wt=0d0

      do ihel=1,3,2
      ztmp1=zC2g02(iq1,iq2,iq3,ig1,ig2,iq4,ihel)
      ztmp2=zCt2g02(iq1,iq2,iq3,ig1,ig2,iq4,ihel)
      wt=wt+2d0*dreal(dconjg(ztmp1)*ztmp2)
      ztmp1=zCt2g02(iq1,iq4,iq3,ig1,ig2,iq2,ihel)
      ztmp2=zC2g02(iq1,iq4,iq3,ig1,ig2,iq2,ihel)
      wt=wt+2d0*dreal(dconjg(ztmp1)*ztmp2)
      enddo
      do ihel=5,8,3
      ztmp1=zC2g02(iq1,iq2,iq3,ig1,ig2,iq4,ihel)
      ztmp2=zCt2g02(iq1,iq2,iq3,ig1,ig2,iq4,ihel)
      wt=wt+2d0*dreal(dconjg(ztmp1)*ztmp2)
      ztmp1=zCt2g02(iq1,iq4,iq3,ig1,ig2,iq2,ihel)
      ztmp2=zC2g02(iq1,iq4,iq3,ig1,ig2,iq2,ihel)
      wt=wt+2d0*dreal(dconjg(ztmp1)*ztmp2)
      enddo

      D2g02=2D0*wt

c      write(*,*)"D2g02",D2g02
*
      return
      end
************************************************************************
* identical 4 quark 2 gluon squared amplitude (1,1) configuration
*
*                      q g Qb Q g qb
*
      function D2g03(iq1,ig1,iq2,iq3,ig2,iq4)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

c      write(*,*)" hello C2g02"
      wt=0d0

      do ihel=1,3,2
      ztmp1=zC2g03(iq1,ig1,iq2,iq3,ig2,iq4,ihel)
      ztmp2=zCt2g03(iq1,ig1,iq2,iq3,ig2,iq4,ihel)
      wt=wt+2d0*dreal(dconjg(ztmp1)*ztmp2)
      ztmp1=zCt2g03(iq1,ig1,iq4,iq3,ig2,iq2,ihel)
      ztmp2=zC2g03(iq1,ig1,iq4,iq3,ig2,iq2,ihel)
      wt=wt+2d0*dreal(dconjg(ztmp1)*ztmp2)
      enddo
      do ihel=5,8,3
      ztmp1=zC2g03(iq1,ig1,iq2,iq3,ig2,iq4,ihel)
      ztmp2=zCt2g03(iq1,ig1,iq2,iq3,ig2,iq4,ihel)
      wt=wt+2d0*dreal(dconjg(ztmp1)*ztmp2)
      ztmp1=zCt2g03(iq1,ig1,iq4,iq3,ig2,iq2,ihel)
      ztmp2=zC2g03(iq1,ig1,iq4,iq3,ig2,iq2,ihel)
      wt=wt+2d0*dreal(dconjg(ztmp1)*ztmp2)
      enddo

      D2g03=2D0*wt

c      write(*,*)"D2g03",D2g03
*
      return
      end
      
************************************************************************
*     identical 4 quark 2 gluon squared amplitude (0,2)+(2,0) configuration
*     interference of LC amplitude       
*
*                      q qb Q Qb g g
*
      function Db2g012(iq1,iq2,iq3,iq4,ig1,ig2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

c      write(*,*)" hello Ctb2g012"
      wt=0d0
      do ihel=1,3,2
         ztmp1  =  zC2g01(iq1,ig1,ig2,iq2,iq3,iq4,ihel)
     .           + zC2g02(iq1,iq2,iq3,ig1,ig2,iq4,ihel)
         ztmp2  =  zC2g01(iq1,ig1,ig2,iq4,iq3,iq2,ihel)
     .           + zC2g02(iq1,iq4,iq3,ig1,ig2,iq2,ihel)
         wt=wt+2d0*dreal(conjg(ztmp1)*ztmp2)
      enddo
      do ihel=5,8,3
         ztmp1  =  zC2g01(iq1,ig1,ig2,iq2,iq3,iq4,ihel)
     .           + zC2g02(iq1,iq2,iq3,ig1,ig2,iq4,ihel)
         ztmp2  =  zC2g01(iq1,ig1,ig2,iq4,iq3,iq2,ihel)
     .           + zC2g02(iq1,iq4,iq3,ig1,ig2,iq2,ihel)
         wt=wt+2d0*dreal(conjg(ztmp1)*ztmp2)
      enddo
      
      Db2g012=2D0*wt

c      write(*,*)"Db2g012",Db2g012

      return
      end


************************************************************************
*     identical 4 quark 2 gluon squared amplitude (1,1) configuration
*     interference of LC amplitude       
*
*                      q qb Q Qb g g
*
      function Db2g03(iq1,iq2,iq3,iq4,ig1,ig2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      iph1 = 1
      iph2 = 1
      if (ig1.le.2) iph1=-1
      if (ig2.le.2) iph2=-1
      iph = iph1*iph2
c      write(*,*)" hello Db2g03"
      wt=0d0

      do ihel=1,3,2
         ztmp1  =  zC2g03(iq1,ig1,iq2,iq3,ig2,iq4,ihel)
     .            +zC2g03(iq1,ig2,iq2,iq3,ig1,iq4,ihel)
         ztmp2  =  zC2g03(iq1,ig1,iq4,iq3,ig2,iq2,ihel)
     .            +zC2g03(iq1,ig2,iq4,iq3,ig1,iq2,ihel)
         wt=wt+2d0*dreal(conjg(ztmp1)*ztmp2)
      enddo
      do ihel=5,8,3
         ztmp1  = zC2g03(iq1,ig1,iq2,iq3,ig2,iq4,ihel)
     .           +iph*conjg(zC2g03(iq1,ig2,iq2,iq3,ig1,iq4,13-ihel))
         ztmp2  = zC2g03(iq1,ig1,iq4,iq3,ig2,iq2,ihel)
     .           +iph*conjg(zC2g03(iq1,ig2,iq4,iq3,ig1,iq2,13-ihel))
      wt=wt+2d0*dreal(conjg(ztmp1)*ztmp2)
      enddo
      
      Db2g03=2D0*wt

c      write(*,*)"Db2g03",Db2g03

      return
      end


**************************************************
*     identical 4 quark 2 gluon squared amplitude 
*     generic interference of LC amplitude       
*
*                      q qb Q Qb g g
*
      function Dtb2g0(iq1,iq2,iq3,iq4,ig1,ig2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      iph1 = 1
      iph2 = 1
      if (ig1.le.2) iph1=-1
      if (ig2.le.2) iph2=-1
      iph = iph1*iph2
c      write(*,*)" hello Db2g03"
      wt=0d0

      do ihel=1,3,2
         ztmp1 =  zC2g02(iq1,iq4,iq3,ig2,ig1,iq2,ihel)
     .           +zC2g01(iq1,ig1,ig2,iq4,iq3,iq2,ihel)
         ztmp2 =  zC2g03(iq1,ig1,iq2,iq3,ig2,iq4,ihel)
         wt=wt+2d0*dreal(conjg(ztmp1)*ztmp2)
         ztmp1 =  zC2g02(iq1,iq2,iq3,ig2,ig1,iq4,ihel)
     .           +zC2g01(iq1,ig1,ig2,iq2,iq3,iq4,ihel)
         ztmp2 =  zC2g03(iq1,ig1,iq4,iq3,ig2,iq2,ihel)
         wt=wt+2d0*dreal(conjg(ztmp1)*ztmp2)
      enddo
      do ihel=5,8,3
         ztmp1 =+iph*conjg(zC2g02(iq1,iq4,iq3,ig2,ig1,iq2,13-ihel))
     .          +zC2g01(iq1,ig1,ig2,iq4,iq3,iq2,ihel)
         ztmp2 =+zC2g03(iq1,ig1,iq2,iq3,ig2,iq4,ihel)
         wt=wt+2d0*dreal(conjg(ztmp1)*ztmp2)
         ztmp1 =+iph*conjg(zC2g02(iq1,iq2,iq3,ig2,ig1,iq4,13-ihel))
     .          +zC2g01(iq1,ig1,ig2,iq2,iq3,iq4,ihel)
         ztmp2 =+zC2g03(iq1,ig1,iq4,iq3,ig2,iq2,ihel)
         wt=wt+2d0*dreal(conjg(ztmp1)*ztmp2)
      enddo
      
      Dtb2g0=2D0*wt

c      write(*,*)"Dtb2g0",Dtb2g0

      return
      end

************************************************************************
*     identical 4 quark 2 gluon squared amplitude configuration
*     square-interference of LC 4 quark 2 photon amplitude       
*
*                      q qb Q Qb g g
*
      function Dt2g0(iq1,iq2,iq3,iq4,ig1,ig2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      iph1 = 1
      iph2 = 1
      if (ig1.le.2) iph1=-1
      if (ig2.le.2) iph2=-1
      iph = iph1*iph2
c      write(*,*)" hello Cttb2g0"
      wt=0d0
      do ihel=1,3,2
         ztmp1 = +zC2g02(iq1,iq4,iq3,ig1,ig2,iq2,ihel)
     .           +zC2g02(iq1,iq4,iq3,ig2,ig1,iq2,ihel)
     .           +zC2g01(iq1,ig1,ig2,iq4,iq3,iq2,ihel)
     .           +zC2g01(iq1,ig2,ig1,iq4,iq3,iq2,ihel)
     .           +zC2g03(iq1,ig1,iq4,iq3,ig2,iq2,ihel)
     .           +zC2g03(iq1,ig2,iq4,iq3,ig1,iq2,ihel)
         ztmp2 = +zC2g02(iq1,iq2,iq3,ig1,ig2,iq4,ihel)
     .           +zC2g02(iq1,iq2,iq3,ig2,ig1,iq4,ihel)
     .           +zC2g01(iq1,ig1,ig2,iq2,iq3,iq4,ihel)
     .           +zC2g01(iq1,ig2,ig1,iq2,iq3,iq4,ihel)
     .           +zC2g03(iq1,ig1,iq2,iq3,ig2,iq4,ihel)
     .           +zC2g03(iq1,ig2,iq2,iq3,ig1,iq4,ihel)
         wt=wt+2d0*dreal(conjg(ztmp1)*ztmp2)
      enddo
      do ihel=5,8,3
         ztmp1 =+zC2g02(iq1,iq4,iq3,ig1,ig2,iq2,ihel)
     .          +iph*conjg(zC2g02(iq1,iq4,iq3,ig2,ig1,iq2,13-ihel))
     .          +zC2g01(iq1,ig1,ig2,iq4,iq3,iq2,ihel)
     .          +iph*conjg(zC2g01(iq1,ig2,ig1,iq4,iq3,iq2,13-ihel))
     .          +zC2g03(iq1,ig1,iq4,iq3,ig2,iq2,ihel)
     .          +iph*conjg(zC2g03(iq1,ig2,iq4,iq3,ig1,iq2,13-ihel))
         ztmp2 =+zC2g02(iq1,iq2,iq3,ig1,ig2,iq4,ihel)
     .          +iph*conjg(zC2g02(iq1,iq2,iq3,ig2,ig1,iq4,13-ihel))
     .          +zC2g01(iq1,ig1,ig2,iq2,iq3,iq4,ihel)
     .          +iph*conjg(zC2g01(iq1,ig2,ig1,iq2,iq3,iq4,13-ihel))
     .          +zC2g03(iq1,ig1,iq2,iq3,ig2,iq4,ihel)
     .          +iph*conjg(zC2g03(iq1,ig2,iq2,iq3,ig1,iq4,13-ihel))
         wt=wt+2d0*dreal(conjg(ztmp1)*ztmp2)
      enddo
      
      Dt2g0=2D0*wt
c      write(*,*)"Dt2g0",Dt2g0

      return
      end

************************************************************************
*     identical 4 quark 2 gluon squared amplitude (0,2)+(2,0) configuration
*     interference of LC/SLC amplitude       
*
*                      q qb Q Qb g g
*
      function Dtt2g012(iq1,iq2,iq3,iq4,ig1,ig2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      iph1 = 1
      iph2 = 1
      if (ig1.le.2) iph1=-1
      if (ig2.le.2) iph2=-1
      iph = iph1*iph2
c      write(*,*)" hello Cttb2g0"
      wt=0d0
      do ihel=1,3,2
         ztmp1 = -zC2g02(iq1,iq4,iq3,ig1,ig2,iq2,ihel)
     .           -zC2g02(iq1,iq4,iq3,ig2,ig1,iq2,ihel)
     .           +zC2g01(iq1,ig1,ig2,iq4,iq3,iq2,ihel)
     .           +zC2g01(iq1,ig2,ig1,iq4,iq3,iq2,ihel)
         ztmp2 = -zCt2g02(iq1,iq4,iq3,ig1,ig2,iq2,ihel)
     .           -zCt2g02(iq1,iq4,iq3,ig2,ig1,iq2,ihel)
     .           +zCt2g01(iq1,ig1,ig2,iq4,iq3,iq2,ihel)
     .           +zCt2g01(iq1,ig2,ig1,iq4,iq3,iq2,ihel)
         wt=wt+2d0*dreal(conjg(ztmp1)*ztmp2)
         ztmp1 = -zC2g02(iq1,iq2,iq3,ig1,ig2,iq4,ihel)
     .           -zC2g02(iq1,iq2,iq3,ig2,ig1,iq4,ihel)
     .           +zC2g01(iq1,ig1,ig2,iq2,iq3,iq4,ihel)
     .           +zC2g01(iq1,ig2,ig1,iq2,iq3,iq4,ihel)
         ztmp2 = -zCt2g02(iq1,iq2,iq3,ig1,ig2,iq4,ihel)
     .           -zCt2g02(iq1,iq2,iq3,ig2,ig1,iq4,ihel)
     .           +zCt2g01(iq1,ig1,ig2,iq2,iq3,iq4,ihel)
     .           +zCt2g01(iq1,ig2,ig1,iq2,iq3,iq4,ihel)
         wt=wt+2d0*dreal(conjg(ztmp1)*ztmp2)
      enddo
      do ihel=5,8,3
         ztmp1 =-zC2g02(iq1,iq4,iq3,ig1,ig2,iq2,ihel)
     .          -iph*conjg(zC2g02(iq1,iq4,iq3,ig2,ig1,iq2,13-ihel))
     .          +zC2g01(iq1,ig1,ig2,iq4,iq3,iq2,ihel)
     .          +iph*conjg(zC2g01(iq1,ig2,ig1,iq4,iq3,iq2,13-ihel))
         ztmp2 =-zCt2g02(iq1,iq4,iq3,ig1,ig2,iq2,ihel)
     .          -iph*conjg(zCt2g02(iq1,iq4,iq3,ig2,ig1,iq2,13-ihel))
     .          +zCt2g01(iq1,ig1,ig2,iq4,iq3,iq2,ihel)
     .          +iph*conjg(zCt2g01(iq1,ig2,ig1,iq4,iq3,iq2,13-ihel))
         wt=wt+2d0*dreal(conjg(ztmp1)*ztmp2)
         ztmp1 =-zC2g02(iq1,iq2,iq3,ig1,ig2,iq4,ihel)
     .          -iph*conjg(zC2g02(iq1,iq2,iq3,ig2,ig1,iq4,13-ihel))
     .          +zC2g01(iq1,ig1,ig2,iq2,iq3,iq4,ihel)
     .          +iph*conjg(zC2g01(iq1,ig2,ig1,iq2,iq3,iq4,13-ihel))
         ztmp2 =-zCt2g02(iq1,iq2,iq3,ig1,ig2,iq4,ihel)
     .          -iph*conjg(zCt2g02(iq1,iq2,iq3,ig2,ig1,iq4,13-ihel))
     .          +zCt2g01(iq1,ig1,ig2,iq2,iq3,iq4,ihel)
     .          +iph*conjg(zCt2g01(iq1,ig2,ig1,iq2,iq3,iq4,13-ihel))
         wt=wt+2d0*dreal(conjg(ztmp1)*ztmp2)
      enddo
      
      Dtt2g012=2D0*wt
c      write(*,*)"Dtt2g012",Dtt2g012

      return
      end

************************************************************************
*     identical 4 quark 2 gluon squared amplitude (1,1) configuration
*     interference of LC/SLC amplitudes       
*
*                      q qb Q Qb g g
*
      function Dtt2g03(iq1,iq2,iq3,iq4,ig1,ig2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      iph1 = 1
      iph2 = 1
      if (ig1.le.2) iph1=-1
      if (ig2.le.2) iph2=-1
      iph = iph1*iph2
c      write(*,*)" hello Db2g03"
      wt=0d0

      do ihel=1,3,2
         ztmp1  =  zC2g03(iq1,ig1,iq2,iq3,ig2,iq4,ihel)
     .            -zC2g03(iq1,ig2,iq2,iq3,ig1,iq4,ihel)
         ztmp2  =  zCt2g03(iq1,ig1,iq2,iq3,ig2,iq4,ihel)
     .            -zCt2g03(iq1,ig2,iq2,iq3,ig1,iq4,ihel)
         wt=wt+2d0*dreal(conjg(ztmp1)*ztmp2)
         ztmp1  =  zC2g03(iq1,ig1,iq4,iq3,ig2,iq2,ihel)
     .            -zC2g03(iq1,ig2,iq4,iq3,ig1,iq2,ihel)
         ztmp2  =  zCt2g03(iq1,ig1,iq4,iq3,ig2,iq2,ihel)
     .            -zCt2g03(iq1,ig2,iq4,iq3,ig1,iq2,ihel)
         wt=wt+2d0*dreal(conjg(ztmp1)*ztmp2)
      enddo
      do ihel=5,8,3
         ztmp1  = zC2g03(iq1,ig1,iq2,iq3,ig2,iq4,ihel)
     .           -iph*conjg(zC2g03(iq1,ig2,iq2,iq3,ig1,iq4,13-ihel))
         ztmp2  = zCt2g03(iq1,ig1,iq2,iq3,ig2,iq4,ihel)
     .           -iph*conjg(zCt2g03(iq1,ig2,iq2,iq3,ig1,iq4,13-ihel))
         wt=wt+2d0*dreal(conjg(ztmp1)*ztmp2)
         ztmp1  = zC2g03(iq1,ig1,iq4,iq3,ig2,iq2,ihel)
     .           -iph*conjg(zC2g03(iq1,ig2,iq4,iq3,ig1,iq2,13-ihel))
         ztmp2  = zCt2g03(iq1,ig1,iq4,iq3,ig2,iq2,ihel)
     .           -iph*conjg(zCt2g03(iq1,ig2,iq4,iq3,ig1,iq2,13-ihel))
         wt=wt+2d0*dreal(conjg(ztmp1)*ztmp2)
      enddo
      
      Dtt2g03=2D0*wt

c      write(*,*)"Dtt2g03",Dtt2g03

      return
      end

      
************************************************************************
*     identical 4 quark 2 gluon squared amplitude (0,2)+(2,0) configuration
*     interference of SLC amplitude       
*
*                      q qb Q Qb g g
*
      function Dttb2g012(iq1,iq2,iq3,iq4,ig1,ig2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

c      write(*,*)" hello Ctb2g012"
      wt=0d0
      do ihel=1,3,2
         ztmp1  =  zCt2g01(iq1,ig1,ig2,iq2,iq3,iq4,ihel)
     .           + zCt2g02(iq1,iq2,iq3,ig1,ig2,iq4,ihel)
         ztmp2  =  zCt2g01(iq1,ig1,ig2,iq4,iq3,iq2,ihel)
     .           + zCt2g02(iq1,iq4,iq3,ig1,ig2,iq2,ihel)
         wt=wt+2d0*dreal(conjg(ztmp1)*ztmp2)
      enddo
      do ihel=5,8,3
         ztmp1  =  zCt2g01(iq1,ig1,ig2,iq2,iq3,iq4,ihel)
     .           + zCt2g02(iq1,iq2,iq3,ig1,ig2,iq4,ihel)
         ztmp2  =  zCt2g01(iq1,ig1,ig2,iq4,iq3,iq2,ihel)
     .           + zCt2g02(iq1,iq4,iq3,ig1,ig2,iq2,ihel)
         wt=wt+2d0*dreal(conjg(ztmp1)*ztmp2)
      enddo
      
      Dttb2g012=2D0*wt

c      write(*,*)"Dttb2g012",Dttb2g012

      return
      end


************************************************************************
*     identical 4 quark 2 gluon squared amplitude (1,1) configuration
*     interference of SLC amplitude       
*
*                      q qb Q Qb g g
*
      function Dttb2g03(iq1,iq2,iq3,iq4,ig1,ig2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      iph1 = 1
      iph2 = 1
      if (ig1.le.2) iph1=-1
      if (ig2.le.2) iph2=-1
      iph = iph1*iph2
c      write(*,*)" hello Db2g03"
      wt=0d0

      do ihel=1,3,2
         ztmp1  =  zCt2g03(iq1,ig1,iq2,iq3,ig2,iq4,ihel)
     .            +zCt2g03(iq1,ig2,iq2,iq3,ig1,iq4,ihel)
         ztmp2  =  zCt2g03(iq1,ig1,iq4,iq3,ig2,iq2,ihel)
     .            +zCt2g03(iq1,ig2,iq4,iq3,ig1,iq2,ihel)
         wt=wt+2d0*dreal(conjg(ztmp1)*ztmp2)
      enddo
      do ihel=5,8,3
         ztmp1  = zCt2g03(iq1,ig1,iq2,iq3,ig2,iq4,ihel)
     .           +iph*conjg(zCt2g03(iq1,ig2,iq2,iq3,ig1,iq4,13-ihel))
         ztmp2  = zCt2g03(iq1,ig1,iq4,iq3,ig2,iq2,ihel)
     .           +iph*conjg(zCt2g03(iq1,ig2,iq4,iq3,ig1,iq2,13-ihel))
      wt=wt+2d0*dreal(conjg(ztmp1)*ztmp2)
      enddo
      
      Dttb2g03=2D0*wt

c      write(*,*)"Dttb2g03",Dttb2g03

      return
      end

**************************************************
*     identical 4 quark 2 gluon squared amplitude 
*     generic interference of SLC amplitude       
*
*                      q qb Q Qb g g
*
      function Dttb2g0(iq1,iq2,iq3,iq4,ig1,ig2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      iph1 = 1
      iph2 = 1
      if (ig1.le.2) iph1=-1
      if (ig2.le.2) iph2=-1
      iph = iph1*iph2
c      write(*,*)" hello Db2g03"
      wt=0d0

      do ihel=1,3,2
         ztmp1 =  zCt2g02(iq1,iq4,iq3,ig2,ig1,iq2,ihel)
     .           +zCt2g01(iq1,ig1,ig2,iq4,iq3,iq2,ihel)
         ztmp2 =  zCt2g03(iq1,ig1,iq2,iq3,ig2,iq4,ihel)
         wt=wt+2d0*dreal(conjg(ztmp1)*ztmp2)
         ztmp1 =  zCt2g02(iq1,iq2,iq3,ig2,ig1,iq4,ihel)
     .           +zCt2g01(iq1,ig1,ig2,iq2,iq3,iq4,ihel)
         ztmp2 =  zCt2g03(iq1,ig1,iq4,iq3,ig2,iq2,ihel)
         wt=wt+2d0*dreal(conjg(ztmp1)*ztmp2)
      enddo
      do ihel=5,8,3
         ztmp1 =+iph*conjg(zCt2g02(iq1,iq4,iq3,ig2,ig1,iq2,13-ihel))
     .          +zCt2g01(iq1,ig1,ig2,iq4,iq3,iq2,ihel)
         ztmp2 =+zCt2g03(iq1,ig1,iq2,iq3,ig2,iq4,ihel)
         wt=wt+2d0*dreal(conjg(ztmp1)*ztmp2)
         ztmp1 =+iph*conjg(zCt2g02(iq1,iq2,iq3,ig2,ig1,iq4,13-ihel))
     .          +zCt2g01(iq1,ig1,ig2,iq2,iq3,iq4,ihel)
         ztmp2 =+zCt2g03(iq1,ig1,iq4,iq3,ig2,iq2,ihel)
         wt=wt+2d0*dreal(conjg(ztmp1)*ztmp2)
      enddo
      
      Dttb2g0=2D0*wt

c      write(*,*)"Dttb2g0",Dttb2g0

      return
      end



************************************************************************
*     identical 4 quark 2 gluon squared amplitude configuration
*     square-interference of SLC 4 quark 2 photon amplitude       
*
*                      q qb Q Qb g g
*
      function Dttt2g0(iq1,iq2,iq3,iq4,ig1,ig2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      iph1 = 1
      iph2 = 1
      if (ig1.le.2) iph1=-1
      if (ig2.le.2) iph2=-1
      iph = iph1*iph2
c      write(*,*)" hello Cttb2g0"
      wt=0d0
      do ihel=1,3,2
         ztmp1 = +zCt2g02(iq1,iq4,iq3,ig1,ig2,iq2,ihel)
     .           +zCt2g02(iq1,iq4,iq3,ig2,ig1,iq2,ihel)
     .           +zCt2g01(iq1,ig1,ig2,iq4,iq3,iq2,ihel)
     .           +zCt2g01(iq1,ig2,ig1,iq4,iq3,iq2,ihel)
     .           +zCt2g03(iq1,ig1,iq4,iq3,ig2,iq2,ihel)
     .           +zCt2g03(iq1,ig2,iq4,iq3,ig1,iq2,ihel)
         ztmp2 = +zCt2g02(iq1,iq2,iq3,ig1,ig2,iq4,ihel)
     .           +zCt2g02(iq1,iq2,iq3,ig2,ig1,iq4,ihel)
     .           +zCt2g01(iq1,ig1,ig2,iq2,iq3,iq4,ihel)
     .           +zCt2g01(iq1,ig2,ig1,iq2,iq3,iq4,ihel)
     .           +zCt2g03(iq1,ig1,iq2,iq3,ig2,iq4,ihel)
     .           +zCt2g03(iq1,ig2,iq2,iq3,ig1,iq4,ihel)
         wt=wt+2d0*dreal(conjg(ztmp1)*ztmp2)
      enddo
      do ihel=5,8,3
         ztmp1 =+zCt2g02(iq1,iq4,iq3,ig1,ig2,iq2,ihel)
     .          +iph*conjg(zCt2g02(iq1,iq4,iq3,ig2,ig1,iq2,13-ihel))
     .          +zCt2g01(iq1,ig1,ig2,iq4,iq3,iq2,ihel)
     .          +iph*conjg(zCt2g01(iq1,ig2,ig1,iq4,iq3,iq2,13-ihel))
     .          +zCt2g03(iq1,ig1,iq4,iq3,ig2,iq2,ihel)
     .          +iph*conjg(zCt2g03(iq1,ig2,iq4,iq3,ig1,iq2,13-ihel))
         ztmp2 =+zCt2g02(iq1,iq2,iq3,ig1,ig2,iq4,ihel)
     .          +iph*conjg(zCt2g02(iq1,iq2,iq3,ig2,ig1,iq4,13-ihel))
     .          +zCt2g01(iq1,ig1,ig2,iq2,iq3,iq4,ihel)
     .          +iph*conjg(zCt2g01(iq1,ig2,ig1,iq2,iq3,iq4,13-ihel))
     .          +zCt2g03(iq1,ig1,iq2,iq3,ig2,iq4,ihel)
     .          +iph*conjg(zCt2g03(iq1,ig2,iq2,iq3,ig1,iq4,13-ihel))
         wt=wt+2d0*dreal(conjg(ztmp1)*ztmp2)
      enddo
      
      Dttt2g0=2D0*wt
c      write(*,*)"Dttt2g0",Dttt2g0

      return
      end



      
******
******
************************************************************
* non-identical 4 quark 2 gluon amplitude (2,0) configuration
*
      function zC2g01(iq1,ig1,ig2,iq4,iq3,iq2,ihel)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)

c      write(*,*)" hello zC2g01"

      s12=s6(iq1,iq4)
      s13=s6(iq1,iq3)
      s14=s6(iq1,iq2)
      s15=s6(iq1,ig1)
      s16=s6(iq1,ig2)
      s23=s6(iq4,iq3)
      s24=s6(iq4,iq2)
      s25=s6(iq4,ig1)
      s26=s6(iq4,ig2)
      s34=s6(iq3,iq2)
      s35=s6(iq3,ig1)
      s36=s6(iq3,ig2)
      s45=s6(iq2,ig1)
      s46=s6(iq2,ig2)
      s56=s6(ig1,ig2)

      s125 = s12 + s15 + s25
      s126 = s12 + s16 + s26
      s145 = s14 + s15 + s45
      s156 = s15 + s16 + s56
      s256 = s25 + s26 + s56

      za12=zA6(iq1,iq4)
      za13=zA6(iq1,iq3)
      za14=zA6(iq1,iq2)
      za15=zA6(iq1,ig1)
      za16=zA6(iq1,ig2)
      za21=zA6(iq4,iq1)
      za23=zA6(iq4,iq3)
      za24=zA6(iq4,iq2)
      za25=zA6(iq4,ig1)
      za26=zA6(iq4,ig2)
      za31=zA6(iq3,iq1)
      za32=zA6(iq3,iq4)
      za34=zA6(iq3,iq2)
      za35=zA6(iq3,ig1)
      za36=zA6(iq3,ig2)
      za41=zA6(iq2,iq1)
      za42=zA6(iq2,iq4)
      za43=zA6(iq2,iq3)
      za45=zA6(iq2,ig1)
      za46=zA6(iq2,ig2)
      za51=zA6(ig1,iq1)
      za52=zA6(ig1,iq4)
      za53=zA6(ig1,iq3)
      za54=zA6(ig1,iq2)
      za56=zA6(ig1,ig2)
      za61=zA6(ig2,iq1)
      za62=zA6(ig2,iq4)
      za63=zA6(ig2,iq3)
      za64=zA6(ig2,iq2)
      za65=zA6(ig2,ig1)

      zb12=zb6(iq1,iq4)
      zb13=zb6(iq1,iq3)
      zb14=zb6(iq1,iq2)
      zb15=zb6(iq1,ig1)
      zb16=zb6(iq1,ig2)
      zb21=zb6(iq4,iq1)
      zb23=zb6(iq4,iq3)
      zb24=zb6(iq4,iq2)
      zb25=zb6(iq4,ig1)
      zb26=zb6(iq4,ig2)
      zb31=zb6(iq3,iq1)
      zb32=zb6(iq3,iq4)
      zb34=zb6(iq3,iq2)
      zb35=zb6(iq3,ig1)
      zb36=zb6(iq3,ig2)
      zb41=zb6(iq2,iq1)
      zb42=zb6(iq2,iq4)
      zb43=zb6(iq2,iq3)
      zb45=zb6(iq2,ig1)
      zb46=zb6(iq2,ig2)
      zb51=zb6(ig1,iq1)
      zb52=zb6(ig1,iq4)
      zb53=zb6(ig1,iq3)
      zb54=zb6(ig1,iq2)
      zb56=zb6(ig1,ig2)
      zb61=zb6(ig2,iq1)
      zb62=zb6(ig2,iq4)
      zb63=zb6(ig2,iq3)
      zb64=zb6(ig2,iq2)
      zb65=zb6(ig2,ig1)

* MHV amplitudes

      zden = za12/za15/za56/za62/za23/za41
                                      !(q,g,g,Q;q,Q)
      if(ihel.eq.1) znum = -za42**2   !(+,+,+,-;+,-)
      if(ihel.eq.2) znum = +za43**2   !(+,+,+,-;-,+)
      if(ihel.eq.3) znum = -za13**2   !(-,+,+,+;-,+)
      if(ihel.eq.4) znum = +za12**2   !(-,+,+,+;+,-)


* NMHV amplitudes

      if(ihel.le.4) zC2g01 = znum*zden

      if(ihel.eq.5)then               !(+,+,-,-;+,-)
         znum1 = za61*za42*zb15**2*(zb31*za16+zb35*za56)
         znum2 = za26**2*zb31*zb52*(zb52*za24+zb56*za64)
         znum3 = za26**2*zb15**2*(zb34*za41+zb35*za51)*
     .           (zb23*za34+zb26*za64)
         znum4 = s145*za26*za61*za42*zb15*zb31*zb52
     .         - za26**2*za61*zb15*zb65*zb31*(zb23*za34+zb26*za64)
     .         - za26*za56*za42*zb15**2*zb52*(zb34*za41+zb35*za51)
       endif

       if(ihel.eq.6)then               !(+,+,-,-;-,+)
         znum1 = za34*za16*zb15**2*(zb21*za16+zb25*za56)
         znum2 = za43*za26*zb52**2*(zb12*za26+zb15*za56)
         znum3 = za63*za62*zb15**2*(zb24*za41+zb25*za51)*
     .           (zb23*za34+zb26*za64)
         znum4 = -s145*za61*za62*za43*zb15*zb21*zb52
     .         - za62*za43*za65*zb15**2*zb52*(zb24*za41+zb25*za51)
     .         - za62*za63*za61*zb65*zb15*zb21*(zb23*za34+zb26*za64)
       endif

       if(ihel.eq.7)then               !(-,+,-,+;+,-)
         znum1 = za61**2*zb15*zb34*(zb51*za12+zb56*za62)
         znum2 = za26**2*zb43*zb25*(zb52*za21+zb56*za61)
         znum3 = za26**2*zb51*zb54*(zb23*za31+zb26*za61)*
     .           (zb34*za41+zb35*za51)
         znum4 = -s145*za26*za12*za61*zb51*zb52*zb34
     .         - za26**2*za61*zb56*zb34*zb51*(zb23*za31+zb26*za61)
     .         - za12*za26*za56*zb51*zb52*zb54*(zb34*za41+zb35*za51)
       endif

       if(ihel.eq.8)then               !(-,+,-,+;-,+)
         znum1 = za61**2*zb24*zb15*(zb51*za13+zb56*za63)
         znum2 = za13*za26*zb25**2*(zb42*za26+zb45*za56)
         znum3 = za26*za36*zb45*zb15*(zb23*za31+zb26*za61)*
     .           (zb24*za41+zb25*za51)
         znum4 = -s145*za16*za31*za26*zb15*zb25*zb24
     .         - za26*za36*za16*zb56*zb24*zb15*(zb23*za31+zb26*za61)
     .         - za26*za31*za56*zb15*zb25*zb45*(zb24*za41+zb25*za51)
       endif

       zden1 = s15*s56*s23*s156
       zden2 = s14*s56*s26*s256
       zden3 = s14*s15*s26*s23*s145
       zden4 = s14*s15*s56*s26*s23

       if(ihel.gt.4) zC2g01 = (znum1/zden1 + znum2/zden2
     .                      +  znum3/zden3 + znum4/zden4)

       zC2g01=zC2g01

*      write(*,*)"zC2g01",zC2g01
      return
      end
************************************************************************
* non-identical 4 quark 2 gluon amplitude (0,2) configuration
*
      function zC2g02(iq1,iq2,iq3,ig1,ig2,iq4,ihel)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

c      write(*,*)" hello zC2g02"

      if(ihel.eq.1) zC2g02 = zC2g01(iq3,ig1,ig2,iq4,iq1,iq2,1)
      if(ihel.eq.2) zC2g02 = zC2g01(iq3,ig1,ig2,iq4,iq1,iq2,4)
      if(ihel.eq.3) zC2g02 = zC2g01(iq3,ig1,ig2,iq4,iq1,iq2,3)
      if(ihel.eq.4) zC2g02 = zC2g01(iq3,ig1,ig2,iq4,iq1,iq2,2)
      if(ihel.eq.5) zC2g02 = zC2g01(iq3,ig1,ig2,iq4,iq1,iq2,5)
      if(ihel.eq.6) zC2g02 = zC2g01(iq3,ig1,ig2,iq4,iq1,iq2,7)
      if(ihel.eq.7) zC2g02 = zC2g01(iq3,ig1,ig2,iq4,iq1,iq2,6)
      if(ihel.eq.8) zC2g02 = zC2g01(iq3,ig1,ig2,iq4,iq1,iq2,8)


c      write(*,*)"zC2g02",zC2g02

      return
      end
************************************************************************
* non-identical 4 quark 2 gluon amplitude (1,1) configuration
*
      function zC2g03(iq1,ig1,iq4,iq3,ig2,iq2,ihel)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)

c      write(*,*)" hello zC2g03"

      s12=s6(iq1,iq4)
      s13=s6(iq1,iq3)
      s14=s6(iq1,iq2)
      s15=s6(iq1,ig1)
      s16=s6(iq1,ig2)
      s23=s6(iq4,iq3)
      s24=s6(iq4,iq2)
      s25=s6(iq4,ig1)
      s26=s6(iq4,ig2)
      s34=s6(iq3,iq2)
      s35=s6(iq3,ig1)
      s36=s6(iq3,ig2)
      s45=s6(iq2,ig1)
      s46=s6(iq2,ig2)
      s56=s6(ig1,ig2)

      s125 = s12 + s15 + s25
      s126 = s12 + s16 + s26
      s145 = s14 + s15 + s45
      s156 = s15 + s16 + s56
      s256 = s25 + s26 + s56
      s235 = s23 + s25 + s35

      za12=zA6(iq1,iq4)
      za13=zA6(iq1,iq3)
      za14=zA6(iq1,iq2)
      za15=zA6(iq1,ig1)
      za16=zA6(iq1,ig2)
      za21=zA6(iq4,iq1)
      za23=zA6(iq4,iq3)
      za24=zA6(iq4,iq2)
      za25=zA6(iq4,ig1)
      za26=zA6(iq4,ig2)
      za31=zA6(iq3,iq1)
      za32=zA6(iq3,iq4)
      za34=zA6(iq3,iq2)
      za35=zA6(iq3,ig1)
      za36=zA6(iq3,ig2)
      za41=zA6(iq2,iq1)
      za42=zA6(iq2,iq4)
      za43=zA6(iq2,iq3)
      za45=zA6(iq2,ig1)
      za46=zA6(iq2,ig2)
      za51=zA6(ig1,iq1)
      za52=zA6(ig1,iq4)
      za53=zA6(ig1,iq3)
      za54=zA6(ig1,iq2)
      za56=zA6(ig1,ig2)
      za61=zA6(ig2,iq1)
      za62=zA6(ig2,iq4)
      za63=zA6(ig2,iq3)
      za64=zA6(ig2,iq2)
      za65=zA6(ig2,ig1)

      zb12=zb6(iq1,iq4)
      zb13=zb6(iq1,iq3)
      zb14=zb6(iq1,iq2)
      zb15=zb6(iq1,ig1)
      zb16=zb6(iq1,ig2)
      zb21=zb6(iq4,iq1)
      zb23=zb6(iq4,iq3)
      zb24=zb6(iq4,iq2)
      zb25=zb6(iq4,ig1)
      zb26=zb6(iq4,ig2)
      zb31=zb6(iq3,iq1)
      zb32=zb6(iq3,iq4)
      zb34=zb6(iq3,iq2)
      zb35=zb6(iq3,ig1)
      zb36=zb6(iq3,ig2)
      zb41=zb6(iq2,iq1)
      zb42=zb6(iq2,iq4)
      zb43=zb6(iq2,iq3)
      zb45=zb6(iq2,ig1)
      zb46=zb6(iq2,ig2)
      zb51=zb6(ig1,iq1)
      zb52=zb6(ig1,iq4)
      zb53=zb6(ig1,iq3)
      zb54=zb6(ig1,iq2)
      zb56=zb6(ig1,ig2)
      zb61=zb6(ig2,iq1)
      zb62=zb6(ig2,iq4)
      zb63=zb6(ig2,iq3)
      zb64=zb6(ig2,iq2)
      zb65=zb6(ig2,ig1)

* MHV amplitudes

      zden = za12*za34/za15/za52/za23/za36/za64/za41
                                      !(q,g,Q;q,g,Q)
      if(ihel.eq.1) znum = -za24**2   !(+,+,-;+,-,-)
      if(ihel.eq.2) znum = +za34**2   !(+,+,-;-,-,+)
      if(ihel.eq.3) znum = -za13**2   !(-,+,+;-,-,+)
      if(ihel.eq.4) znum = +za12**2   !(-,+,+;+,-,-)

* NMHV amplitudes

      if(ihel.le.4) zC2g03 = znum*zden

      if(ihel.eq.5)then               !(+,+,-;+,-,-)
         znum1 = za26*za36*zb51**2*(za14*zb43+za15*zb53)*
     .           (za41*zb13+za45*zb53)
         znum2 = za46**2*zb53*zb52*(za21*zb14+za26*zb64)*
     .           (za24*zb41+za26*zb61)
         znum3 = za46*za12*za36*zb51*zb25*zb34*(za41*zb13+za45*zb53)*
     .           (za23*zb31+za25*zb51)
       endif

       if(ihel.eq.6)then               !(+,+,-;-,-,+)
         znum1 = za63**2*zb51**2*(za14*zb43+za15*zb53)*
     .           (za41*zb12+za45*zb52)
         znum2 = za46**2*zb52**2*(za34*zb41+za36*zb61)*
     .           (za21*zb14+za26*zb64)
         znum3 = za12*za46*za36*zb51*zb25*zb34*(za41*zb12+za45*zb52)*
     .           (za32*zb21+za35*zb51)
       endif

       if(ihel.eq.7)then               !(-,+,+;+,-,-)
         znum1 = za26*za63*zb54*zb15*(za41*zb34+za15*zb53)**2
         znum2 = za61*za64*zb53*zb52*(za21*zb14+za26*zb64)**2
         znum3 = za46*za12*za36*zb51*zb25*zb34*(za14*zb43+za15*zb53)*
     .           (za23*zb34+za25*zb54)
       endif

       if(ihel.eq.8)then               !(-,+,+;-,-,+)
         znum1 = za36**2*zb51*zb54*(za14*zb43+za15*zb53)*
     .           (za14*zb42+za15*zb52)
         znum2 = za64*za61*zb52**2*(za31*zb14+za36*zb64)*
     .           (za21*zb14+za26*zb64)
         znum3 = za46*za12*za36*zb51*zb25*zb34*(za13*zb32+za16*zb62)*
     .           (za31*zb14+za36*zb64)
       endif

       zden1 = s14*s15*s23*s36*s145
       zden2 = s14*s25*s23*s46*s235
       zden3 = s14*s15*s25*s23*s36*s46

       if(ihel.gt.4) zC2g03 = (znum1/zden1 + znum2/zden2
     .                       + znum3/zden3)

       zC2g03=zC2g03

*      write(*,*)"zC2g03",zC2g03
      return
      end
************************************************************************
* non-identical 4 quark 2 gluon sub-leading amplitude (2,0) configuration
*
      function zCt2g01(iq1,ig1,ig2,iq4,iq3,iq2,ihel)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)

c      write(*,*)" hello zCt2g01"

      s12=s6(iq1,iq4)
      s13=s6(iq1,iq3)
      s14=s6(iq1,iq2)
      s15=s6(iq1,ig1)
      s16=s6(iq1,ig2)
      s23=s6(iq4,iq3)
      s24=s6(iq4,iq2)
      s25=s6(iq4,ig1)
      s26=s6(iq4,ig2)
      s34=s6(iq3,iq2)
      s35=s6(iq3,ig1)
      s36=s6(iq3,ig2)
      s45=s6(iq2,ig1)
      s46=s6(iq2,ig2)
      s56=s6(ig1,ig2)

      s156 = s15 + s16 + s56
      s256 = s25 + s26 + s56

      za12=zA6(iq1,iq4)
      za13=zA6(iq1,iq3)
      za14=zA6(iq1,iq2)
      za15=zA6(iq1,ig1)
      za16=zA6(iq1,ig2)
      za21=zA6(iq4,iq1)
      za23=zA6(iq4,iq3)
      za24=zA6(iq4,iq2)
      za25=zA6(iq4,ig1)
      za26=zA6(iq4,ig2)
      za31=zA6(iq3,iq1)
      za32=zA6(iq3,iq4)
      za34=zA6(iq3,iq2)
      za35=zA6(iq3,ig1)
      za36=zA6(iq3,ig2)
      za41=zA6(iq2,iq1)
      za42=zA6(iq2,iq4)
      za43=zA6(iq2,iq3)
      za45=zA6(iq2,ig1)
      za46=zA6(iq2,ig2)
      za51=zA6(ig1,iq1)
      za52=zA6(ig1,iq4)
      za53=zA6(ig1,iq3)
      za54=zA6(ig1,iq2)
      za56=zA6(ig1,ig2)
      za61=zA6(ig2,iq1)
      za62=zA6(ig2,iq4)
      za63=zA6(ig2,iq3)
      za64=zA6(ig2,iq2)
      za65=zA6(ig2,ig1)

      zb12=zb6(iq1,iq4)
      zb13=zb6(iq1,iq3)
      zb14=zb6(iq1,iq2)
      zb15=zb6(iq1,ig1)
      zb16=zb6(iq1,ig2)
      zb21=zb6(iq4,iq1)
      zb23=zb6(iq4,iq3)
      zb24=zb6(iq4,iq2)
      zb25=zb6(iq4,ig1)
      zb26=zb6(iq4,ig2)
      zb31=zb6(iq3,iq1)
      zb32=zb6(iq3,iq4)
      zb34=zb6(iq3,iq2)
      zb35=zb6(iq3,ig1)
      zb36=zb6(iq3,ig2)
      zb41=zb6(iq2,iq1)
      zb42=zb6(iq2,iq4)
      zb43=zb6(iq2,iq3)
      zb45=zb6(iq2,ig1)
      zb46=zb6(iq2,ig2)
      zb51=zb6(ig1,iq1)
      zb52=zb6(ig1,iq4)
      zb53=zb6(ig1,iq3)
      zb54=zb6(ig1,iq2)
      zb56=zb6(ig1,ig2)
      zb61=zb6(ig2,iq1)
      zb62=zb6(ig2,iq4)
      zb63=zb6(ig2,iq3)
      zb64=zb6(ig2,iq2)
      zb65=zb6(ig2,ig1)

* MHV amplitudes

      zden = za15*za56*za62*za34
                                      !(q,g,g,Q;q,Q)
      if(ihel.eq.1) znum = -za42**2   !(+,+,+,-;+,-)
      if(ihel.eq.2) znum = +za23**2   !(+,+,+,-;-,+)
      if(ihel.eq.3) znum = -za13**2   !(-,+,+,+;-,+)
      if(ihel.eq.4) znum = +za14**2   !(-,+,+,+;+,-)

      if(ihel.le.4) zCt2g01 = znum/zden

* NMHV amplitudes

      if(ihel.eq.5)then               !(+,+,-,-;+,-)
         znum1 = za61*za42*zb15*(zb31*za16+zb35*za56)
         znum2 = (zb31*za16+zb35*za56)*(zb52*za24+zb56*za64)
         znum3 = za62*zb13*zb25*(zb52*za24+zb56*za64)
       endif

       if(ihel.eq.6)then               !(+,+,-,-;-,+)
         znum1 = za61*za32*zb15*(zb41*za16+zb45*za56)
         znum2 = (zb41*za16+zb45*za56)*(zb52*za23+zb56*za63)
         znum3 = za62*zb14*zb25*(zb52*za23+zb56*za63)
       endif

       if(ihel.eq.7)then               !(-,+,-,+;+,-)
         znum1 = za61**2*zb23*(zb51*za14+zb56*za64)
         znum2 = za61*za14*zb32*zb25
         znum3 = za41*zb25**2*(zb32*za26+zb35*za56)
       endif

       if(ihel.eq.8)then               !(-,+,-,+;-,+)
         znum1 = za61**2*zb24*(zb51*za13+zb56*za63)
         znum2 = za61*za13*zb42*zb25
         znum3 = za31*zb25**2*(zb42*za26+zb45*za56)
       endif

       zden1 = s34*s56*s156*za15
       zden2 = s34*s56*za15*zb62
       zden3 = s34*s56*s256*zb62

       if(ihel.gt.4) zCt2g01 = (znum1/zden1 + znum2/zden2
     .                           + znum3/zden3)

       zCt2g01=-1d0*zCt2g01

*      write(*,*)"zCt2g01",zCt2g01
      return
      end
************************************************************************
* non-identical 4 quark 2 gluon sub-leading amplitude (0,2) configuration
*
      function zCt2g02(iq1,iq2,iq3,ig1,ig2,iq4,ihel)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      if(ihel.eq.1) zCt2g02 = zCt2g01(iq3,ig1,ig2,iq4,iq1,iq2,1)
      if(ihel.eq.2) zCt2g02 = zCt2g01(iq3,ig1,ig2,iq4,iq1,iq2,4)
      if(ihel.eq.3) zCt2g02 = zCt2g01(iq3,ig1,ig2,iq4,iq1,iq2,3)
      if(ihel.eq.4) zCt2g02 = zCt2g01(iq3,ig1,ig2,iq4,iq1,iq2,2)
      if(ihel.eq.5) zCt2g02 = zCt2g01(iq3,ig1,ig2,iq4,iq1,iq2,5)
      if(ihel.eq.6) zCt2g02 = zCt2g01(iq3,ig1,ig2,iq4,iq1,iq2,7)
      if(ihel.eq.7) zCt2g02 = zCt2g01(iq3,ig1,ig2,iq4,iq1,iq2,6)
      if(ihel.eq.8) zCt2g02 = zCt2g01(iq3,ig1,ig2,iq4,iq1,iq2,8)

*      write(*,*)" hello zCt2g02"
      return
      end
************************************************************************
* non-identical 4 quark 2 gluon sub-leading amplitude (1,1) configuration
*
      function zCt2g03(iq1,ig1,iq4,iq3,ig2,iq2,ihel)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)

c      write(*,*)" hello zCt2g03"

      s12=s6(iq1,iq4)
      s13=s6(iq1,iq3)
      s14=s6(iq1,iq2)
      s15=s6(iq1,ig1)
      s16=s6(iq1,ig2)
      s23=s6(iq4,iq3)
      s24=s6(iq4,iq2)
      s25=s6(iq4,ig1)
      s26=s6(iq4,ig2)
      s34=s6(iq3,iq2)
      s35=s6(iq3,ig1)
      s36=s6(iq3,ig2)
      s45=s6(iq2,ig1)
      s46=s6(iq2,ig2)
      s56=s6(ig1,ig2)

      s346 = s34 + s36 + s46

      za12=zA6(iq1,iq4)
      za13=zA6(iq1,iq3)
      za14=zA6(iq1,iq2)
      za15=zA6(iq1,ig1)
      za16=zA6(iq1,ig2)
      za21=zA6(iq4,iq1)
      za23=zA6(iq4,iq3)
      za24=zA6(iq4,iq2)
      za25=zA6(iq4,ig1)
      za26=zA6(iq4,ig2)
      za31=zA6(iq3,iq1)
      za32=zA6(iq3,iq4)
      za34=zA6(iq3,iq2)
      za35=zA6(iq3,ig1)
      za36=zA6(iq3,ig2)
      za41=zA6(iq2,iq1)
      za42=zA6(iq2,iq4)
      za43=zA6(iq2,iq3)
      za45=zA6(iq2,ig1)
      za46=zA6(iq2,ig2)
      za51=zA6(ig1,iq1)
      za52=zA6(ig1,iq4)
      za53=zA6(ig1,iq3)
      za54=zA6(ig1,iq2)
      za56=zA6(ig1,ig2)
      za61=zA6(ig2,iq1)
      za62=zA6(ig2,iq4)
      za63=zA6(ig2,iq3)
      za64=zA6(ig2,iq2)
      za65=zA6(ig2,ig1)

      zb12=zb6(iq1,iq4)
      zb13=zb6(iq1,iq3)
      zb14=zb6(iq1,iq2)
      zb15=zb6(iq1,ig1)
      zb16=zb6(iq1,ig2)
      zb21=zb6(iq4,iq1)
      zb23=zb6(iq4,iq3)
      zb24=zb6(iq4,iq2)
      zb25=zb6(iq4,ig1)
      zb26=zb6(iq4,ig2)
      zb31=zb6(iq3,iq1)
      zb32=zb6(iq3,iq4)
      zb34=zb6(iq3,iq2)
      zb35=zb6(iq3,ig1)
      zb36=zb6(iq3,ig2)
      zb41=zb6(iq2,iq1)
      zb42=zb6(iq2,iq4)
      zb43=zb6(iq2,iq3)
      zb45=zb6(iq2,ig1)
      zb46=zb6(iq2,ig2)
      zb51=zb6(ig1,iq1)
      zb52=zb6(ig1,iq4)
      zb53=zb6(ig1,iq3)
      zb54=zb6(ig1,iq2)
      zb56=zb6(ig1,ig2)
      zb61=zb6(ig2,iq1)
      zb62=zb6(ig2,iq4)
      zb63=zb6(ig2,iq3)
      zb64=zb6(ig2,iq2)
      zb65=zb6(ig2,ig1)

* MHV amplitudes

      if(ihel.le.4) zden = za15*za52*za36*za64
                                      !(q,g,Q;q,g,Q)
      if(ihel.eq.1) znum = -za24**2   !(+,+,-;+,+,-)
      if(ihel.eq.2) znum = +za23**2   !(+,+,-;-,+,+)
      if(ihel.eq.3) znum = -za13**2   !(-,+,+;-,+,+)
      if(ihel.eq.4) znum = +za14**2   !(-,+,+;+,+,-)

* NMHV amplitudes

      if(ihel.eq.5)then
         znum = -(zb34*za42+zb36*za62)**2
       endif

       if(ihel.eq.6)then
         znum = (zb43*za32+zb46*za62)**2
       endif

       if(ihel.eq.7)then
         znum = (zb34*za41+zb36*za61)**2
       endif

       if(ihel.eq.8)then
         znum = -(zb43*za31+zb46*za61)**2
       endif

       if(ihel.gt.4) zden = za15*za52*zb36*zb64*s346

       zCt2g03 = znum/zden

       zCt2g03=-1d0*zCt2g03

*      write(*,*)"zCt2g03",zCt2g03
      return
      end
************************************************************************
* Full SLC (1/N^2) colour level of FullC2g0
      function Ct2g0(iq1,iq2,iq3,iq4,ig1,ig2)
      implicit real*8 (a-h,o-z)

      Ct2g0 = -1d0*(
     .          Ctb2g012(iq1,iq4,iq3,iq2,ig1,ig2)
     .         +Ctb2g03(iq1,iq4,iq3,iq2,ig1,ig2) )
     .     + 1d0 *
     .        ( Ct2g01(iq1,ig1,ig2,iq4,iq3,iq2)
     .        + Ct2g01(iq1,ig2,ig1,iq4,iq3,iq2)
     .        + Ct2g02(iq1,iq4,iq3,ig1,ig2,iq2)
     .        + Ct2g02(iq1,iq4,iq3,ig2,ig1,iq2)
     .        + Ct2g03(iq1,ig1,iq4,iq3,ig2,iq2)
     .        + Ct2g03(iq1,ig2,iq4,iq3,ig1,iq2) )
     .     + 1d0 *
     .        ( Ctt2g01(iq1,ig1,ig2,iq4,iq3,iq2)
     .        + Ctt2g01(iq1,ig2,ig1,iq4,iq3,iq2)
     .        + Ctt2g02(iq1,iq4,iq3,ig1,ig2,iq2)
     .        + Ctt2g02(iq1,iq4,iq3,ig2,ig1,iq2)
     .        + Ctt2g03(iq1,ig1,iq4,iq3,ig2,iq2)
     .        + Ctt2g03(iq1,ig2,iq4,iq3,ig1,iq2) )

      return
      end
*************************************************************************
* Full SSLC (1/N^4) colour level of FullC2g0
      function Ctt2g0(iq1,iq2,iq3,iq4,ig1,ig2)
      implicit real*8 (a-h,o-z)
      
      Ctt2g0 = Cttb2g0(iq1,iq4,iq3,iq2,ig1,ig2)
     .         -Cttb2g012(iq1,iq4,iq3,iq2,ig1,ig2)
     .         -Cttb2g03(iq1,iq4,iq3,iq2,ig1,ig2)
      return
      end function
*************************************************************************
* Full LC (N^0) colour level of FullD2g0
      function D2g0full(iq1,iq2,iq3,iq4,ig1,ig2)
      implicit real*8 (a-h,o-z)

!      fac=(4d0*pi*as)**4*(nc**2-1d0)*nc
! minus sign of fac moved to D2g0full
      D2g0full = -1d0*(
     .         + D2g01(iq1,ig1,ig2,iq4,iq3,iq2)
     .         + D2g01(iq1,ig2,ig1,iq4,iq3,iq2)
     .         + D2g02(iq1,iq4,iq3,ig1,ig2,iq2)
     .         + D2g02(iq1,iq4,iq3,ig2,ig1,iq2)
     .         + D2g03(iq1,ig1,iq4,iq3,ig2,iq2)
     .         + D2g03(iq1,ig2,iq4,iq3,ig1,iq2)
     .         + Db2g012(iq1,iq4,iq3,iq2,ig1,ig2)
     .         + Db2g012(iq1,iq4,iq3,iq2,ig2,ig1)
     .         + Db2g03(iq1,iq4,iq3,iq2,ig1,ig2)
     .         + Dtb2g0(iq1,iq4,iq3,iq2,ig1,ig2)
     .         + Dtb2g0(iq1,iq4,iq3,iq2,ig2,ig1)
     .         )

      return
      end 
*************************************************************************
* Full SLC (1/N^2) colour level of FullD2g0
      function Dt2g0full(iq1,iq2,iq3,iq4,ig1,ig2)
      implicit real*8 (a-h,o-z)

!      fac=(4d0*pi*as)**4*(nc**2-1d0)*nc
! minus sign of fac moved to Dt2g0full
      Dt2g0full = 1d0*(
     .          + Dt2g0(iq1,iq4,iq3,iq2,ig1,ig2)
     .          + Dtt2g012(iq1,iq4,iq3,iq2,ig1,ig2)
     .          + Dtt2g03(iq1,iq4,iq3,iq2,ig1,ig2) )
     .          - 1d0*(
     .          + Dttb2g012(iq1,iq4,iq3,iq2,ig1,ig2)
     .          + Dttb2g012(iq1,iq4,iq3,iq2,ig2,ig1)
     .          + Dttb2g03(iq1,iq4,iq3,iq2,ig1,ig2)
     .          + Dttb2g0(iq1,iq4,iq3,iq2,ig2,ig1)
     .          + Dttb2g0(iq1,iq4,iq3,iq2,ig1,ig2) )

      return
      end
*************************************************************************
* Full SSLC (1/N^4) colour level of FullD2g0
      function Dtt2g0full(iq1,iq2,iq3,iq4,ig1,ig2)
      implicit real*8 (a-h,o-z)

!      fac=(4d0*pi*as)**4*(nc**2-1d0)*nc
! minus sign of fac moved to Dtt2g0full
      Dtt2g0full = Dttt2g0(iq1,iq4,iq3,iq2,ig1,ig2)

      return
      end
************************************************************************
      function C2g0(iq1,iq2,iq3,iq4,ig1,ig2)
      implicit real*8 (a-h,o-z)

      C2g0= C2g01(iq1,ig1,ig2,iq4,iq3,iq2)
     .        + C2g01(iq1,ig2,ig1,iq4,iq3,iq2)
     .        + C2g02(iq1,iq4,iq3,ig1,ig2,iq2)
     .        + C2g02(iq1,iq4,iq3,ig2,ig1,iq2)
     .        + C2g03(iq1,ig1,iq4,iq3,ig2,iq2)
     .        + C2g03(iq1,ig2,iq4,iq3,ig1,iq2) 

      return
      end
*************************************************************************
      function C1g0(iq1,iqb1,iq2,iqb2,ig)
      use KinData_mod
      implicit double precision (a-h,o-z)

      C1g0=C1g0X(iq1,iqb1,iq2,iqb2,ig)
      return
      end
*************************************************************************
      function Ct1g0(iq,iqb,iQ2,iQb2,ig)
      implicit real*8 (a-h,o-y)
      implicit complex*16(z)
      
      Ct1g0=-C1g0Y(iq,iqb,iQ2,iQb2,ig)
      return
      end
*************************************************************************
***** New C-type interference terms needed as reduced matrix element
      function C1g0ab(iq1,iqb1,iq2,iqb2,ig)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      wt=0d0
      do ihel=1,4
      do iflip=0,1
        ztreea = zzC1g0a(iq1,ig,iqb2,iq2,iqb1,ihel,iflip)
        ztreeb = zzC1g0b(iq1,iqb2,iq2,ig,iqb1,ihel,iflip)
        wt = wt + 2d0*real(ztreea*dconjg(ztreeb))
      enddo
      enddo

      C1g0ab = wt

      return
      end
*************************************************************************
      function C1g0ac(iq1,iqb1,iq2,iqb2,ig)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      wt=0d0
      do ihel=1,4
      do iflip=0,1
        ztreea = zzC1g0a(iq1,ig,iqb2,iq2,iqb1,ihel,iflip)
        ztreec = zzC1g0c(iq1,ig,iqb1,iq2,iqb2,ihel,iflip)
        wt = wt + 2d0*real(ztreea*dconjg(ztreec))
      enddo
      enddo

      C1g0ac = wt

      return
      end
*************************************************************************
      function C1g0ad(iq1,iqb1,iq2,iqb2,ig)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      wt=0d0
      do ihel=1,4
      do iflip=0,1
        ztreea = zzC1g0a(iq1,ig,iqb2,iq2,iqb1,ihel,iflip)
        ztreed = zzC1g0d(iq1,iqb1,iq2,ig,iqb2,ihel,iflip)
        wt = wt + 2d0*real(ztreea*dconjg(ztreed))
      enddo
      enddo

      C1g0ad = wt

      return
      end
*************************************************************************
      function C1g0bc(iq1,iqb1,iq2,iqb2,ig)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      wt=0d0
      do ihel=1,4
      do iflip=0,1
        ztreeb = zzC1g0b(iq1,iqb2,iq2,ig,iqb1,ihel,iflip)
        ztreec = zzC1g0c(iq1,ig,iqb1,iq2,iqb2,ihel,iflip)
        wt = wt + 2d0*real(ztreeb*dconjg(ztreec))
      enddo
      enddo

      C1g0bc = wt

      return
      end
*************************************************************************
      function C1g0bd(iq1,iqb1,iq2,iqb2,ig)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      wt=0d0
      do ihel=1,4
      do iflip=0,1
        ztreeb = zzC1g0b(iq1,iqb2,iq2,ig,iqb1,ihel,iflip)
        ztreed = zzC1g0d(iq1,iqb1,iq2,ig,iqb2,ihel,iflip)
        wt = wt + 2d0*real(ztreeb*dconjg(ztreed))
      enddo
      enddo

      C1g0bd = wt

      return
      end
*************************************************************************
      function C1g0cd(iq1,iqb1,iq2,iqb2,ig)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      wt=0d0
      do ihel=1,4
      do iflip=0,1
        ztreec = zzC1g0c(iq1,ig,iqb1,iq2,iqb2,ihel,iflip)
        ztreed = zzC1g0d(iq1,iqb1,iq2,ig,iqb2,ihel,iflip)
        wt = wt + 2d0*real(ztreec*dconjg(ztreed))
      enddo
      enddo

      C1g0cd = wt

      return
      end
*************************************************************************
***** D1g0inti is same as C1g0i, but with iqb1<->iqb2 for one of its amp      
      function D1g0inta(iq1,iqb1,iq2,iqb2,ig)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      dimension ihel2(6)

      ihel2(1)=1
      ihel2(2)=5
      ihel2(3)=6
      ihel2(4)=4
      ihel2(5)=2
      ihel2(6)=3
      wt=0d0
      do ihel=1,6
      do iflip=0,1
        zwt1 = zzC1g0a(iq1,ig,iqb2,iq2,iqb1,ihel,iflip)
        zwt2 = zzC1g0a(iq1,ig,iqb1,iq2,iqb2,ihel2(ihel),iflip)
        wt = wt + 2d0*real(zwt1*dconjg(zwt2))
      enddo
      enddo

      D1g0inta = wt

      return
      end
*************************************************************************
      function D1g0intb(iq1,iqb1,iq2,iqb2,ig)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      dimension ihel2(6)

      ihel2(1)=1
      ihel2(2)=5
      ihel2(3)=6
      ihel2(4)=4
      ihel2(5)=2
      ihel2(6)=3
      wt=0d0
      do ihel=1,6
      do iflip=0,1
        zwt1 = zzC1g0b(iq1,iqb2,iq2,ig,iqb1,ihel,iflip)
        zwt2 = zzC1g0b(iq1,iqb1,iq2,ig,iqb2,ihel2(ihel),iflip)
        wt = wt + 2d0*real(zwt1*dconjg(zwt2))
      enddo
      enddo

      D1g0intb = wt

      return
      end
*************************************************************************
      function D1g0intc(iq1,iqb1,iq2,iqb2,ig)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      dimension ihel2(6)

      ihel2(1)=1
      ihel2(2)=5
      ihel2(3)=6
      ihel2(4)=4
      ihel2(5)=2
      ihel2(6)=3
      wt=0d0
      do ihel=1,6
      do iflip=0,1
        zwt1 = zzC1g0c(iq1,ig,iqb1,iq2,iqb2,ihel,iflip)
        zwt2 = zzC1g0c(iq1,ig,iqb2,iq2,iqb1,ihel2(ihel),iflip)
        wt = wt + 2d0*real(zwt1*dconjg(zwt2))
      enddo
      enddo

      D1g0intc = wt

      return
      end
*************************************************************************
      function D1g0intd(iq1,iqb1,iq2,iqb2,ig)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      dimension ihel2(6)

      ihel2(1)=1
      ihel2(2)=5
      ihel2(3)=6
      ihel2(4)=4
      ihel2(5)=2
      ihel2(6)=3
      wt=0d0
      do ihel=1,6
      do iflip=0,1
        zwt1 = zzC1g0d(iq1,iqb1,iq2,ig,iqb2,ihel,iflip)
        zwt2 = zzC1g0d(iq1,iqb2,iq2,ig,iqb1,ihel2(ihel),iflip)
        wt = wt + 2d0*real(zwt1*dconjg(zwt2))
      enddo
      enddo

      D1g0intd = wt

      return
      end
*************************************************************************
***** D1g0intij has zzC1g0i with its args in normal order and zzC1g0j in 'wrong' order (=iqb1<->iqb2) 
      function D1g0intab(iq1,iqb1,iq2,iqb2,ig)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      dimension ihel2(6)

      ihel2(1)=1
      ihel2(2)=5
      ihel2(3)=6
      ihel2(4)=4
      ihel2(5)=2
      ihel2(6)=3
      wt=0d0
      do ihel=1,6
      do iflip=0,1
        zwt1 = zzC1g0a(iq1,ig,iqb2,iq2,iqb1,ihel,iflip)
        zwt2 = zzC1g0b(iq1,iqb1,iq2,ig,iqb2,ihel2(ihel),iflip)
        wt = wt + 2d0*real(zwt1*dconjg(zwt2))
      enddo
      enddo

      D1g0intab = wt

      return
      end      
*************************************************************************
      function D1g0intac(iq1,iqb1,iq2,iqb2,ig)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      dimension ihel2(6)

      ihel2(1)=1
      ihel2(2)=5
      ihel2(3)=6
      ihel2(4)=4
      ihel2(5)=2
      ihel2(6)=3
      wt=0d0
      do ihel=1,6
      do iflip=0,1
        zwt1 = zzC1g0a(iq1,ig,iqb2,iq2,iqb1,ihel,iflip)
        zwt2 = zzC1g0c(iq1,ig,iqb2,iq2,iqb1,ihel2(ihel),iflip)
        wt = wt + 2d0*real(zwt1*dconjg(zwt2))
      enddo
      enddo

      D1g0intac = wt

      return
      end
*************************************************************************
      function D1g0intad(iq1,iqb1,iq2,iqb2,ig)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      dimension ihel2(6)

      ihel2(1)=1
      ihel2(2)=5
      ihel2(3)=6
      ihel2(4)=4
      ihel2(5)=2
      ihel2(6)=3
      wt=0d0
      do ihel=1,6
      do iflip=0,1
        zwt1 = zzC1g0a(iq1,ig,iqb2,iq2,iqb1,ihel,iflip)
        zwt2 = zzC1g0d(iq1,iqb2,iq2,ig,iqb1,ihel2(ihel),iflip)
        wt = wt + 2d0*real(zwt1*dconjg(zwt2))
      enddo
      enddo

      D1g0intad = wt

      return
      end
*************************************************************************
      function D1g0intba(iq1,iqb1,iq2,iqb2,ig)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      dimension ihel2(6)

      ihel2(1)=1
      ihel2(2)=5
      ihel2(3)=6
      ihel2(4)=4
      ihel2(5)=2
      ihel2(6)=3
      wt=0d0
      do ihel=1,6
      do iflip=0,1
        zwt1 = zzC1g0b(iq1,iqb2,iq2,ig,iqb1,ihel,iflip)
        zwt2 = zzC1g0a(iq1,ig,iqb1,iq2,iqb2,ihel2(ihel),iflip)
        wt = wt + 2d0*real(zwt1*dconjg(zwt2))
      enddo
      enddo

      D1g0intba = wt

      return
      end
*************************************************************************
      function D1g0intbc(iq1,iqb1,iq2,iqb2,ig)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      dimension ihel2(6)

      ihel2(1)=1
      ihel2(2)=5
      ihel2(3)=6
      ihel2(4)=4
      ihel2(5)=2
      ihel2(6)=3
      wt=0d0
      do ihel=1,6
      do iflip=0,1
        zwt1 = zzC1g0b(iq1,iqb2,iq2,ig,iqb1,ihel,iflip)
        zwt2 = zzC1g0c(iq1,ig,iqb2,iq2,iqb1,ihel2(ihel),iflip)
        wt = wt + 2d0*real(zwt1*dconjg(zwt2))
      enddo
      enddo

      D1g0intbc = wt

      return
      end
*************************************************************************
      function D1g0intbd(iq1,iqb1,iq2,iqb2,ig)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      dimension ihel2(6)

      ihel2(1)=1
      ihel2(2)=5
      ihel2(3)=6
      ihel2(4)=4
      ihel2(5)=2
      ihel2(6)=3
      wt=0d0
      do ihel=1,6
      do iflip=0,1
        zwt1 = zzC1g0b(iq1,iqb2,iq2,ig,iqb1,ihel,iflip)
        zwt2 = zzC1g0d(iq1,iqb2,iq2,ig,iqb1,ihel2(ihel),iflip)
        wt = wt + 2d0*real(zwt1*dconjg(zwt2))
      enddo
      enddo

      D1g0intbd = wt

      return
      end
*************************************************************************
      function D1g0intca(iq1,iqb1,iq2,iqb2,ig)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      dimension ihel2(6)

      ihel2(1)=1
      ihel2(2)=5
      ihel2(3)=6
      ihel2(4)=4
      ihel2(5)=2
      ihel2(6)=3
      wt=0d0
      do ihel=1,6
      do iflip=0,1
        zwt1 = zzC1g0c(iq1,ig,iqb1,iq2,iqb2,ihel,iflip)
        zwt2 = zzC1g0a(iq1,ig,iqb1,iq2,iqb2,ihel2(ihel),iflip)
        wt = wt + 2d0*real(zwt1*dconjg(zwt2))
      enddo
      enddo

      D1g0intca = wt

      return
      end
*************************************************************************
      function D1g0intcb(iq1,iqb1,iq2,iqb2,ig)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      dimension ihel2(6)

      ihel2(1)=1
      ihel2(2)=5
      ihel2(3)=6
      ihel2(4)=4
      ihel2(5)=2
      ihel2(6)=3
      wt=0d0
      do ihel=1,6
      do iflip=0,1
        zwt1 = zzC1g0c(iq1,ig,iqb1,iq2,iqb2,ihel,iflip)
        zwt2 = zzC1g0b(iq1,iqb1,iq2,ig,iqb2,ihel2(ihel),iflip)
        wt = wt + 2d0*real(zwt1*dconjg(zwt2))
      enddo
      enddo

      D1g0intcb = wt

      return
      end
*************************************************************************
      function D1g0intcd(iq1,iqb1,iq2,iqb2,ig)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      dimension ihel2(6)

      ihel2(1)=1
      ihel2(2)=5
      ihel2(3)=6
      ihel2(4)=4
      ihel2(5)=2
      ihel2(6)=3
      wt=0d0
      do ihel=1,6
      do iflip=0,1
        zwt1 = zzC1g0c(iq1,ig,iqb1,iq2,iqb2,ihel,iflip)
        zwt2 = zzC1g0d(iq1,iqb2,iq2,ig,iqb1,ihel2(ihel),iflip)
        wt = wt + 2d0*real(zwt1*dconjg(zwt2))
      enddo
      enddo

      D1g0intcd = wt

      return
      end
*************************************************************************
      function D1g0intda(iq1,iqb1,iq2,iqb2,ig)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      dimension ihel2(6)

      ihel2(1)=1
      ihel2(2)=5
      ihel2(3)=6
      ihel2(4)=4
      ihel2(5)=2
      ihel2(6)=3
      wt=0d0
      do ihel=1,6
      do iflip=0,1
        zwt1 = zzC1g0d(iq1,iqb1,iq2,ig,iqb2,ihel,iflip)
        zwt2 = zzC1g0a(iq1,ig,iqb1,iq2,iqb2,ihel2(ihel),iflip)
        wt = wt + 2d0*real(zwt1*dconjg(zwt2))
      enddo
      enddo

      D1g0intda = wt

      return
      end
*************************************************************************
      function D1g0intdb(iq1,iqb1,iq2,iqb2,ig)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      dimension ihel2(6)

      ihel2(1)=1
      ihel2(2)=5
      ihel2(3)=6
      ihel2(4)=4
      ihel2(5)=2
      ihel2(6)=3
      wt=0d0
      do ihel=1,6
      do iflip=0,1
        zwt1 = zzC1g0d(iq1,iqb1,iq2,ig,iqb2,ihel,iflip)
        zwt2 = zzC1g0b(iq1,iqb1,iq2,ig,iqb2,ihel2(ihel),iflip)
        wt = wt + 2d0*real(zwt1*dconjg(zwt2))
      enddo
      enddo

      D1g0intdb = wt

      return
      end
*************************************************************************
      function D1g0intdc(iq1,iqb1,iq2,iqb2,ig)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      dimension ihel2(6)

      ihel2(1)=1
      ihel2(2)=5
      ihel2(3)=6
      ihel2(4)=4
      ihel2(5)=2
      ihel2(6)=3
      wt=0d0
      do ihel=1,6
      do iflip=0,1
        zwt1 = zzC1g0d(iq1,iqb1,iq2,ig,iqb2,ihel,iflip)
        zwt2 = zzC1g0c(iq1,ig,iqb2,iq2,iqb1,ihel2(ihel),iflip)
        wt = wt + 2d0*real(zwt1*dconjg(zwt2))
      enddo
      enddo

      D1g0intdc = wt

      return
      end
*************************************************************************

