****************************************************************
* quark pair tree level matrix elements for q + Qb -> g + g + W+ -> nu + e+
*
*
* summed over colour orderings, summed and squared
*
* q(i1) + Qb(i4) -> g(i2) + g(i3) + W+ -> g(i2) + nu(i4) + e+(i5)
*
* If the overall charge of the process is negative then the momenta of the electron
* and neutrino are reversed.
*
* Including overall factors
*

      function FullB2g0W(i1,i2,i3,i4,i5,i6)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)
      common /BWFlav/ nf1,nf2
!$omp threadprivate(/BWFlav/)
      common /eweakW/ amz,stw,ewfac
!$omp threadprivate(/eweakW/)


      fac = get_avg(i1,i4)

      fac = fac*(nc**2 - 1)*nc/16d0*(4d0*pi*amz/stw*4d0*pi*as)**2*16d0

      if((i1.le.2).and.(i4.le.2))then
         fac = 0.5d0*fac ! identical quark factor
      endif

      FullB2g0W =+B2g0W(i1,i2,i3,i4,i5,i6)
     .           +B2g0W(i1,i3,i2,i4,i5,i6)
     .           -(1d0/nc**2)*Bt2g0W(i1,i2,i3,i4,i5,i6)

      FullB2g0W = fac*FullB2g0W

c     write(*,*) "FullB2g0W",FullB2g0W,i1,i2,i3,i4,i5,i6

      return
      end

********************************************************************
*
* Quark pair tree level matrix element for
*
*     q(i1) + Qb(i4) ->  g(i3) + g(i4) + nu(i5) + e+(i6)
*
* One colour ordering, summed over spins and squared. Excluding overall factors.
*
* If the overall charge of the process is negative then the momenta of the electron
* and neutrino are reversed.
*


      function B2g0W(i1,i2,i3,i4,i5,i6)
      use KinData_mod
      use B2gZ, only : zampB2g0Z
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /BWFlav/ nf1,nf2
!$omp threadprivate(/BWFlav/)
      common /eweakW/ amz,stw,ewfac
!$omp threadprivate(/eweakW/)
      complex*16 zampB2g0WLL(2,2)

      ! The amplitudes for the Z and W processes are the same

      zampB2g0WLL = zampB2g0Z(i1,i2,i3,i4,i5,i6)


      B2g0W = 0d0

      do i=1,2
         do j=1,2
            B2g0W = B2g0W + abs(zampB2g0WLL(i,j))**2
         enddo
      enddo

      s56 = s6(i5,i6)

      call eweakW1qq(nf1,nf2,s56)

      B2g0W = ewfac*B2g0W

      return
      end

********************************************************************
*
* Quark pair tree level matrix element for
*
*     q + qb ->  g + g + e- + e+
*
* One colour ordering, summed over spins and squared
*
*     q(1) g(2) g(3) qbar(4) e-(5) e+(6)
*
* The flavour of the quark pair in this matrix element is determined from the flav variable in the common block /B2gFlav/
*
* Note: Each of the four components of these matrix element are generated separately. This is not, in general, necessary at tree level (LL = RR, RL = LR). However, exploiting this symmetry generates a very small numerical error in it's implementation for the subtraction term for the B2g1Z matrix element. This numerical error prevents explicit pole cancellation.

      function Bt2g0W(i1,i2,i3,i4,i5,i6)
      use KinData_mod
      use B2gZ, only : zampB2g0Z
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /BWFlav/ nf1,nf2
!$omp threadprivate(/BWFlav/)
      common /eweakW/ amz,stw,ewfac
!$omp threadprivate(/eweakW/)
      complex*16 zampBt2g0WLLa(2,2),zampBt2g0WLLb(2,2)

      do ii=1,2
         do jj=1,2
            zampBt2g0WLLa(ii,jj) = dcmplx(0d0,0d0)
            zampBt2g0WLLb(ii,jj) = dcmplx(0d0,0d0)
         enddo
      enddo

      ! The amplitudes for the Z and W processes are the same

      zampBt2g0WLLa = zampB2g0Z(i1,i2,i3,i4,i5,i6)
      zampBt2g0WLLb = zampB2g0Z(i1,i3,i2,i4,i5,i6)

      Bt2g0W = 0d0

      do i=1,2
         do j=1,2
            zampBt2g0W = zampBt2g0WLLa(i,j) + zampBt2g0WLLb(j,i)
            Bt2g0W = Bt2g0W + abs(zampBt2g0W)**2
         enddo
      enddo

      s56 = s6(i5,i6)

      call eweakW1qq(nf1,nf2,s56)

      Bt2g0W = ewfac*Bt2g0W

      return
      end

********************************************************************
