****************************************************
* Quark pair tree level leading colour matrix element for
*
*     q + qb ->  g + g + e- + e+
*
* summed over colour orderings, summed and squared
*
*     q(1) g(2) g(3) qbar(4) e-(5) e+(6)
*

      function FullB2g0Z(i1,i2,i3,i4,i5,i6)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)
      common /BZFlav/ nfB2g
!$omp threadprivate(/BZFlav/)
      common /eweakZ/ amz,zewfac(4),zewnull(4)
!$omp threadprivate(/eweakZ/)

      fac = get_avg(i1,i4)

      fac = fac*(nc**2 - 1)*nc/4d0*(4d0*pi*amz*4d0*pi*as)**2*16d0

      if((i1.le.2).and.(i4.le.2))then
         fac = 0.5d0*fac ! identical quark factor
      endif




      if((i2.le.2).and.(i3.le.2))then ! gg initial state is an nf type contribution
         nup = 2
         ndo = 3
         nfB2g = 2 ! Up type contribution
         FullB2g0Z = nup*(B2g0Z(i1,i2,i3,i4,i5,i6)
     .                   +B2g0Z(i1,i3,i2,i4,i5,i6)
     .                   -(1d0/nc**2)*Bt2g0Z(i1,i2,i3,i4,i5,i6))

         nfB2g = 1 ! down type contribution
         FullB2g0Z = FullB2g0Z + ndo*(B2g0Z(i1,i2,i3,i4,i5,i6)
     .                               +B2g0Z(i1,i3,i2,i4,i5,i6)
     .                               -(1d0/nc**2)*Bt2g0Z(i1,i2,i3,i4,i5,i6))

      else ! All other configurations
      FullB2g0Z =+B2g0Z(i1,i2,i3,i4,i5,i6)
     .           +B2g0Z(i1,i3,i2,i4,i5,i6)
     .           -(1d0/nc**2)*Bt2g0Z(i1,i2,i3,i4,i5,i6)
      endif

      FullB2g0Z = fac*FullB2g0Z

c     write(*,*) "FullB2g0Z",FullB2g0Z,i1,i2,i3,i4,i5,i6

      return
      end

********************************************************************
*
* Quark pair tree level matrix element for
*
*     q + qb ->  g + g + e- + e+
*
* One colour ordering, summed over spins and squared
*
*     q(1) g(2) g(3) qbar(4) e-(5) e+(6)
*
* The flavour of the quark pair in this matrix element is determined from the flav variable in the common block /BZFlav/
*
* Note: Each of the four components of these matrix element are generated separately. This is not, in general, necessary at tree level (LL = RR, RL = LR). However, exploiting this symmetry generates a very small numerical error in its implementation for the subtraction term for the B2g1Z matrix element. This numerical error prevents explicit pole cancellation.

      function B2g0Z(i1,i2,i3,i4,i5,i6)
      use KinData_mod
      use B2gZ, only : zampB2g0Z
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      double precision B2g0Zvec(4)
      common /eweakZ/ amz,zewfac(4),zewnull(4)
!$omp threadprivate(/eweakZ/)
      common /BZFlav/ nfB2g
!$omp threadprivate(/BZFlav/)
      complex*16 zampB2g0ZLL(2,2),zampB2g0ZLR(2,2)
     .          ,zampB2g0ZRL(2,2),zampB2g0ZRR(2,2)


      zampB2g0ZLL = zampB2g0Z(i1,i2,i3,i4,i5,i6)
      zampB2g0ZLR = zampB2g0Z(i1,i2,i3,i4,i6,i5)
      zampB2g0ZRL = zampB2g0Z(i4,i3,i2,i1,i5,i6)
      zampB2g0ZRR = zampB2g0Z(i4,i3,i2,i1,i6,i5)

      B2g0Zvec=0d0

      do i=1,2
         do j=1,2
            B2g0Zvec(1) = B2g0Zvec(1) + abs(zampB2g0ZLL(i,j))**2
            B2g0Zvec(2) = B2g0Zvec(2) + abs(zampB2g0ZLR(i,j))**2
            B2g0Zvec(3) = B2g0Zvec(3) + abs(zampB2g0ZRL(i,j))**2
            B2g0Zvec(4) = B2g0Zvec(4) + abs(zampB2g0ZRR(i,j))**2
         enddo
      enddo

      s56 = s6(i5,i6)

      call eweakV1qq(nfB2g,s56)

      B2g0Zvec = abs(zewfac)**2*B2g0Zvec

      B2g0Z = sum(B2g0Zvec)

      return
      end

************************************************************************
* A wrapper function for B2g0Z which inherits the flavour of the first
* pair of quarks from the C0g0Z common block.
*
* This must be called in the maple code for 4 quark processes where the
* reduced matrix element has the flavour of the first pair of quarks


      function f1B2g0Z(i1,i2,i3,i4,i5,i6)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /CZFlav/nf1,nf2
!$omp threadprivate(/CZFlav/)
      common /BZFlav/nf1g
!$omp threadprivate(/BZFlav/)

      if(nf1.ne.0)then
         nf1g = nf1
         f1B2g0Z = B2g0Z(i1,i2,i3,i4,i5,i6)
      else
         write(*,*) "Error in f1B2g0Z: flavours have not been initialised"
         stop
      endif

      return
      end

************************************************************************
* A wrapper function for B2g0Z which inherits the flavour of the second
* pair of quarks from the C0g0Z common block.
*
* This must be called in the maple code for 4 quark processes where the
* reduced matrix element has the flavour of the second pair of quarks
*

      function f2B2g0Z(i1,i2,i3,i4,i5,i6)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /CZFlav/nf1,nf2
!$omp threadprivate(/CZFlav/)
      common /BZFlav/nf1g
!$omp threadprivate(/BZFlav/)

      if(nf2.ne.0)then
         nf1g = nf2
         f2B2g0Z = B2g0Z(i1,i2,i3,i4,i5,i6)
      else
         write(*,*) "Error in f2B2g0Z: flavours have not been initialised"
         stop
      endif

      return
      end


*******************************************************************
* Symmetrised B2g0Z across the qqb pair.
*
*
      function sB2g0Z(i1,i2,i3,i4,i5,i6)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      sB2g0Z = 0.5d0*(B2g0Z(i1,i2,i3,i4,i5,i6)
     .               +B2g0Z(i4,i3,i2,i1,i5,i6))

      return
      end

*******************************************************************
* Conditionally symmetrised B2g0Z across the qqb pair.
*
*
      function csB2g0Z(i1,i2,i3,i4,i5,i6)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      if(i2.eq.1.or.i3.eq.1) then
         csB2g0Z = 0.5d0*(B2g0Z(i1,i2,i3,i4,i5,i6)
     .        +B2g0Z(i4,i3,i2,i1,i5,i6))
      else
         csB2g0Z = B2g0Z(i1,i2,i3,i4,i5,i6)
      endif

      return
      end

*******************************************************************
* Symmetrised Bt2g0Z across the qqb pair.
*
*
      function sBt2g0Z(i1,i2,i3,i4,i5,i6)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      sBt2g0Z = 0.5d0*(Bt2g0Z(i1,i2,i3,i4,i5,i6)
     .                +Bt2g0Z(i4,i3,i2,i1,i5,i6))

      return
      end

*******************************************************************
* Conditionally symmetrised Bt2g0Z across the qqb pair.
*
*
      function csBt2g0Z(i1,i2,i3,i4,i5,i6)
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      if(i2.eq.1.or.i3.eq.1) then
         csBt2g0Z = 0.5d0*(Bt2g0Z(i1,i2,i3,i4,i5,i6)
     .        +Bt2g0Z(i4,i3,i2,i1,i5,i6))
      else
         csBt2g0Z = Bt2g0Z(i1,i2,i3,i4,i5,i6)
      endif

      return
      end

*******************************************************************
* Symmetrised B2g0Z across the qqb pair, taking the flavour from
* the first quark pair of the C-type matrix elements.
*
*

      function sf1B2g0Z(i1,i2,i3,i4,i5,i6)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      sf1B2g0Z = 0.5d0*(f1B2g0Z(i1,i2,i3,i4,i5,i6)
     .                 +f1B2g0Z(i4,i3,i2,i1,i5,i6))

      return
      end

*******************************************************************
* Symmetrised B2g0Z across the qqb pair, taking the flavour from the
* second quark pair of the C-type matrix elements.
*

      function sf2B2g0Z(i1,i2,i3,i4,i5,i6)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      sf2B2g0Z = 0.5d0*(f2B2g0Z(i1,i2,i3,i4,i5,i6)
     .                 +f2B2g0Z(i4,i3,i2,i1,i5,i6))

      return
      end


*******************************************************************
* A flavour averaged version of B2g0Z. This is required in certain
* subtraction terms at RV
*

      function fB2g0Z(i1,i2,i3,i4,i5,i6)
      use KinData_mod
      use B2gZ, only : zampB2g0Z
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer ni(2)
      double precision B2g0Zvec(4),ewfacstore(4)
      complex*16 zampB2g0ZLL(2,2),zampB2g0ZLR(2,2)
     .          ,zampB2g0ZRL(2,2),zampB2g0ZRR(2,2)
      data ni /3,2/ ! this is nup and ndown, hard coded for now
      common /koppel/nf,nc,b0
      common /BZFlav/ nfB2g
!$omp threadprivate(/BZFlav/)
      common /eweakZ/ amz,zewfac(4),zewnull(4)
!$omp threadprivate(/eweakZ/)

      zampB2g0ZLL = zampB2g0Z(i1,i2,i3,i4,i5,i6)
      zampB2g0ZLR = zampB2g0Z(i1,i2,i3,i4,i6,i5)
      zampB2g0ZRL = zampB2g0Z(i4,i3,i2,i1,i5,i6)
      zampB2g0ZRR = zampB2g0Z(i4,i3,i2,i1,i6,i5)

      B2g0Zvec=0d0

      do i=1,2
         do j=1,2
            B2g0Zvec(1) = B2g0Zvec(1) + abs(zampB2g0ZLL(i,j))**2
            B2g0Zvec(2) = B2g0Zvec(2) + abs(zampB2g0ZLR(i,j))**2
            B2g0Zvec(3) = B2g0Zvec(3) + abs(zampB2g0ZRL(i,j))**2
            B2g0Zvec(4) = B2g0Zvec(4) + abs(zampB2g0ZRR(i,j))**2
         enddo
      enddo

      s56 = s6(i5,i6)

      ewfacstore = 0d0

      do i=1,2
         call eweakV1qq(i,s56)
         ewfacstore = ewfacstore + (dble(ni(i))/dble(nf))*abs(zewfac)**2
      enddo


      B2g0Zvec = ewfacstore*B2g0Zvec

      fB2g0Z = sum(B2g0Zvec)


      return
      end

********************************************************************


