      module h3pC_Coeff
      !To reconstruct the h3p
      ! amplitude
      !----------
      implicit none
      save

      complex*16, dimension(3,3,10) :: alphaCC
      complex*16, dimension(3,3,10) :: betaCC
      complex*16, dimension(2,10) :: gammaCC

      logical :: catanisub
      integer :: ieorder

      real*8 :: pi,zeta3
      parameter(pi=3.141592653589793238d0)
      parameter(zeta3=1.202056903159594285d0)

      contains

      subroutine setCataniSub(log)
      implicit none
      !Sets catanisub
      !--------------
      logical, intent(in) :: log


      catanisub = log

      end subroutine setCataniSub

!#############################################################

      subroutine setIEorder(ieord)
      implicit none
      !Sets ieorder
      !------------
      integer, intent(in) :: ieord


      ieorder = ieord

      end subroutine setIEorder

!#############################################################

      subroutine initG2LC(s)
      implicit none
      ! initialises alpha,beta
      ! at two loops
      !-------------
      real*8, dimension(3,3), intent(in) :: s
      !internal
      real*8 :: v,u13,u23
      real*8 :: x,y,z
      real*8 :: s123
      complex*16, dimension(6) :: a2a13,a2a23,b2a13,b2a23,b4a13,b4a23
      complex*16, dimension(6) :: a1a13,a1a23
      complex*16, dimension(6) :: b1a12,b1a21,b1a13,b1a31,b1a23,b1a32

      

      s123 = s(1,2) + s(1,3) + s(2,3)

      if (s(1,2).ge.0d0.and.s(1,3).le.0d0.and.s(2,3).le.0d0) then

      u13 = -s(1,3)/s(1,2)
      u23 = -s(2,3)/s(1,2)
      v   =  s123  /s(1,2)

      alphaCC(:,:,5:10) = (0.d0,0.d0)
      betaCC(:,:,5:10)  = (0.d0,0.d0)

      call alpha2a2L(u13,v,a2a13)
      call alpha2a2L(u23,v,a2a23)
      call beta2a2L( u13,v,b2a13)
      call beta2a2L( u23,v,b2a23)
      call beta4a2L( u13,v,b4a13)
      call beta4a2L( u23,v,b4a23)

      alphaCC(1,2,5:10) = a2a13
      alphaCC(2,1,5:10) = a2a23
      alphaCC(1,3,5:10) = alphaCC(2,1,5:10)
      alphaCC(3,1,5:10) = alphaCC(2,1,5:10)
      alphaCC(2,3,5:10) = alphaCC(1,2,5:10)
      alphaCC(3,2,5:10) = alphaCC(1,2,5:10)
      alphaCC(1,1,5:10) = (0.d0,0.d0)
      alphaCC(2,2,5:10) = (0.d0,0.d0)
      alphaCC(3,3,5:10) = (0.d0,0.d0)

      betaCC(1,2,5:10)  =  b2a13
      betaCC(2,1,5:10)  =  b2a23
      betaCC(1,3,5:10)  =  b4a23
      betaCC(3,1,5:10)  =  betaCC(1,3,5:10)
      betaCC(2,3,5:10)  =  b4a13
      betaCC(3,2,5:10)  =  betaCC(2,3,5:10)
      betaCC(1,1,5:10)  = (0.d0,0.d0)
      betaCC(2,2,5:10)  = (0.d0,0.d0)
      betaCC(3,3,5:10)  = (0.d0,0.d0)

      elseif (s(1,2).le.0d0.and.s(1,3).ge.0d0.and.s(2,3).le.0d0) then

      u13 = -s(1,2)/s(1,3)
      u23 = -s(2,3)/s(1,3)
      v   =  s123  /s(1,3)

      alphaCC(:,:,5:10) = (0.d0,0.d0)
      betaCC(:,:,5:10)  = (0.d0,0.d0)

      call alpha2a2L(u13,v,a2a13)
      call alpha2a2L(u23,v,a2a23)
      call beta2a2L( u13,v,b2a13)
      call beta2a2L( u23,v,b2a23)
      call beta4a2L( u13,v,b4a13)
      call beta4a2L( u23,v,b4a23)

      alphaCC(1,3,5:10) = a2a13
      alphaCC(3,1,5:10) = a2a23
      alphaCC(1,2,5:10) = alphaCC(3,1,5:10)
      alphaCC(2,1,5:10) = alphaCC(3,1,5:10)
      alphaCC(3,2,5:10) = alphaCC(1,3,5:10)
      alphaCC(2,3,5:10) = alphaCC(1,3,5:10)
      alphaCC(1,1,5:10) = (0.d0,0.d0)
      alphaCC(2,2,5:10) = (0.d0,0.d0)
      alphaCC(3,3,5:10) = (0.d0,0.d0)

      betaCC(1,3,5:10)  =  b2a13
      betaCC(3,1,5:10)  =  b2a23
      betaCC(1,2,5:10)  =  b4a23
      betaCC(2,1,5:10)  =  betaCC(1,2,5:10)
      betaCC(3,2,5:10)  =  b4a13
      betaCC(2,3,5:10)  =  betaCC(3,2,5:10)
      betaCC(1,1,5:10)  = (0.d0,0.d0)
      betaCC(2,2,5:10)  = (0.d0,0.d0)
      betaCC(3,3,5:10)  = (0.d0,0.d0)

      elseif (s(1,2).le.0d0.and.s(1,3).le.0d0.and.s(2,3).ge.0d0) then

      u13 = -s(1,3)/s(2,3)
      u23 = -s(1,2)/s(2,3)
      v   =  s123  /s(2,3)

      alphaCC(:,:,5:10) = (0.d0,0.d0)
      betaCC(:,:,5:10)  = (0.d0,0.d0)

      call alpha2a2L(u13,v,a2a13)
      call alpha2a2L(u23,v,a2a23)
      call beta2a2L( u13,v,b2a13)
      call beta2a2L( u23,v,b2a23)
      call beta4a2L( u13,v,b4a13)
      call beta4a2L( u23,v,b4a23)

      alphaCC(3,2,5:10) = a2a13
      alphaCC(2,3,5:10) = a2a23
      alphaCC(3,1,5:10) = alphaCC(2,3,5:10)
      alphaCC(1,3,5:10) = alphaCC(2,3,5:10)
      alphaCC(2,1,5:10) = alphaCC(3,2,5:10)
      alphaCC(1,2,5:10) = alphaCC(3,2,5:10)
      alphaCC(1,1,5:10) = (0.d0,0.d0)
      alphaCC(2,2,5:10) = (0.d0,0.d0)
      alphaCC(3,3,5:10) = (0.d0,0.d0)

      betaCC(3,2,5:10)  =  b2a13
      betaCC(2,3,5:10)  =  b2a23
      betaCC(3,1,5:10)  =  b4a23
      betaCC(1,3,5:10)  =  betaCC(3,1,5:10)
      betaCC(2,1,5:10)  =  b4a13
      betaCC(1,2,5:10)  =  betaCC(2,1,5:10)
      betaCC(1,1,5:10)  = (0.d0,0.d0)
      betaCC(2,2,5:10)  = (0.d0,0.d0)
      betaCC(3,3,5:10)  = (0.d0,0.d0)

      elseif(s(1,2).ge.0d0.and.s(1,3).ge.0d0.and.s(2,3).ge.0d0) then
         y = s(1,3)/s123
         z = s(2,3)/s123
         x = 1d0-y-z
         
         alphaCC(:,:,5:10) = (0.d0,0.d0)
         betaCC(:,:,5:10)  = (0.d0,0.d0)
         
         call alpha1a2L(y,z,a1a13)
         call alpha1a2L(z,y,a1a23)
         call  beta1a2L(y,z,b1a12)
         call  beta1a2L(z,y,b1a21)
         call  beta1a2L(x,z,b1a13)
         call  beta1a2L(z,x,b1a31)
         call  beta1a2L(x,y,b1a23)
         call  beta1a2L(y,x,b1a32)

      alphaCC(1,2,5:10) = a1a13
      alphaCC(2,1,5:10) = a1a23
      alphaCC(1,3,5:10) = alphaCC(2,1,5:10)
      alphaCC(3,1,5:10) = alphaCC(2,1,5:10)
      alphaCC(2,3,5:10) = alphaCC(1,2,5:10)
      alphaCC(3,2,5:10) = alphaCC(1,2,5:10)
      alphaCC(1,1,5:10) = (0.d0,0.d0)
      alphaCC(2,2,5:10) = (0.d0,0.d0)
      alphaCC(3,3,5:10) = (0.d0,0.d0)
      
      betaCC(1,2,5:10)  =  b1a12
      betaCC(2,1,5:10)  =  b1a21
      betaCC(1,3,5:10)  =  b1a13
      betaCC(3,1,5:10)  =  b1a31
      betaCC(2,3,5:10)  =  b1a23
      betaCC(3,2,5:10)  =  b1a32
      betaCC(1,1,5:10)  = (0.d0,0.d0)
      betaCC(2,2,5:10)  = (0.d0,0.d0)
      betaCC(3,3,5:10)  = (0.d0,0.d0)
      else
         write(6,*) 'wrong kinematic region in initG2LC'
      endif   


      !123 beta_2(123)  <=>  1+2->3+H   "u2" = -s13/s12  "v2" = s123/s12
      !213 beta_2(213)  <=>  2+1->3+H   "u2" = -s23/s12  "v2" = s123/s12
      !132 beta_3(132)  <=>  1+2->3+H   "u3" = -s23/s12  "v3" = s123/s12
      !231 beta_3(231)  <=>  2+1->3+H   "u3" = -s13/s12  "v3" = s123/s12
      !312 beta_4(312)  <=>  1+2->3+H   "u4" = -s23/s12  "v4" = s123/s12
      !321 beta_4(321)  <=>  2+1->3+H   "u4" = -s13/s12  "v4" = s123/s12
      !
      !as beta_2(123) <=> 1+2->3+H
      !   beta_3(123) <=> 1+3->2+H
      !   beta_4(123) <=> 2+3->1+H
      !
      !and beta_3 = beta_4 (unlike gamma)

      end subroutine initG2LC

!#############################################################

      subroutine initQ2LC(s,i1,i2,i3)
      implicit none
      !Initialises gamma
      ! at two loops
      !If c(3)=0 g in final state qqb->gh
      !If c(3)=1 g in initial state qg->qh/qbg->qbh
      !--------------------------------------------
      integer, intent(in) :: i1,i2,i3
      real*8, dimension(3,3), intent(in) :: s
      !internal
      real*8 :: v,u,u1,u2
      real*8 :: y,z
      real*8 :: s123
      complex*16, dimension(6) :: gam1,gam2


      s123 = s(i1,i2) + s(i1,i3) + s(i2,i3)

      gammaCC(:,5:10) = (0.d0,0.d0)

      if (s(i1,i2).ge.0d0.and.s(i1,i3).le.0d0.and.s(i2,i3).le.0d0) then
         u1 = -s(i1,i3)/s(i1,i2)
         u2 = -s(i2,i3)/s(i1,i2)
         v =    s123/s(i1,i2)

         call gamma2a2L(u1,v,gam1)
         call gamma2a2L(u2,v,gam2)

      elseif (s(i1,i2).le.0d0.and.s(i1,i3).le.0d0.and.s(i2,i3).ge.0d0) then
         u = -s(i1,i3)/s(i2,i3)
         v =    s123/s(i2,i3)

         call gamma4a2L(u,v,gam1)
         call gamma3a2L(u,v,gam2)

      elseif (s(i1,i2).le.0d0.and.s(i1,i3).ge.0d0.and.s(i2,i3).le.0d0) then
         u = -s(i2,i3)/s(i1,i3)
         v =    s123/s(i1,i3)

         call gamma3a2L(u,v,gam1)
         call gamma4a2L(u,v,gam2)

      elseif(s(i1,i2).ge.0d0.and.s(i1,i3).ge.0d0.and.s(i2,i3).ge.0d0) then
         y = s(i1,i3)/s123
         z = s(i2,i3)/s123
         call gamma1a2L(y,z,gam1)
         call gamma1a2L(z,y,gam2)
      else
         write(6,*) 'wrong kinematic region in initQ2LC'
      endif

      gammaCC(1,5:10) = gam1
      gammaCC(2,5:10) = gam2

      end subroutine initQ2LC

!#############################################################

      subroutine initG1LC(s)
      implicit none
      ! initialises alpha,beta
      ! at one loop
      !-------------
      real*8, dimension(3,3), intent(in) :: s
      !internal
      real*8 :: v,u13,u23
      real*8 :: s123
      real*8 :: x,y,z
      complex*16, dimension(3) :: a2a13,a2a23,b2a13,b2a23,b4a13,b4a23
      complex*16, dimension(3) :: a1a13,a1a23
      complex*16, dimension(3) :: b1a12,b1a21,b1a13,b1a31,b1a23,b1a32


      s123 = s(1,2) + s(1,3) + s(2,3)

      if (s(1,2).ge.0d0.and.s(1,3).le.0d0.and.s(2,3).le.0d0) then
      u13 = -s(1,3)/s(1,2)
      u23 = -s(2,3)/s(1,2)
      v   =  s123 /s(1,2)

      alphaCC(:,:,2:4) = (0.d0,0.d0)
      betaCC(:,:,2:4)  = (0.d0,0.d0)

      call alpha2a1L(u13,v,a2a13)
      call alpha2a1L(u23,v,a2a23)
      call  beta2a1L(u13,v,b2a13)
      call  beta2a1L(u23,v,b2a23)
      call  beta4a1L(u13,v,b4a13)
      call  beta4a1L(u23,v,b4a23)

      alphaCC(1,2,2:4) = a2a13
      alphaCC(2,1,2:4) = a2a23
      alphaCC(1,3,2:4) = alphaCC(2,1,2:4)
      alphaCC(3,1,2:4) = alphaCC(2,1,2:4)
      alphaCC(2,3,2:4) = alphaCC(1,2,2:4)
      alphaCC(3,2,2:4) = alphaCC(1,2,2:4)
      alphaCC(1,1,2:4) = (0.d0,0.d0)
      alphaCC(2,2,2:4) = (0.d0,0.d0)
      alphaCC(3,3,2:4) = (0.d0,0.d0)

      betaCC(1,2,2:4)  =  b2a13
      betaCC(2,1,2:4)  =  b2a23
      betaCC(1,3,2:4)  =  b4a23
      betaCC(3,1,2:4)  =  betaCC(1,3,2:4)
      betaCC(2,3,2:4)  =  b4a13
      betaCC(3,2,2:4)  =  betaCC(2,3,2:4)
      betaCC(1,1,2:4)  = (0.d0,0.d0)
      betaCC(2,2,2:4)  = (0.d0,0.d0)
      betaCC(3,3,2:4)  = (0.d0,0.d0)

      elseif (s(1,2).le.0d0.and.s(1,3).ge.0d0.and.s(2,3).le.0d0) then
      u13 = -s(1,2)/s(1,3)
      u23 = -s(2,3)/s(1,3)
      v   =  s123 /s(1,3)

      alphaCC(:,:,2:4) = (0.d0,0.d0)
      betaCC(:,:,2:4)  = (0.d0,0.d0)

      call alpha2a1L(u13,v,a2a13)
      call alpha2a1L(u23,v,a2a23)
      call  beta2a1L(u13,v,b2a13)
      call  beta2a1L(u23,v,b2a23)
      call  beta4a1L(u13,v,b4a13)
      call  beta4a1L(u23,v,b4a23)

      alphaCC(1,3,2:4) = a2a13
      alphaCC(3,1,2:4) = a2a23
      alphaCC(1,2,2:4) = alphaCC(3,1,2:4)
      alphaCC(2,1,2:4) = alphaCC(3,1,2:4)
      alphaCC(2,3,2:4) = alphaCC(1,3,2:4)
      alphaCC(3,2,2:4) = alphaCC(1,3,2:4)
      alphaCC(1,1,2:4) = (0.d0,0.d0)
      alphaCC(2,2,2:4) = (0.d0,0.d0)
      alphaCC(3,3,2:4) = (0.d0,0.d0)

      betaCC(1,3,2:4)  =  b2a13
      betaCC(3,1,2:4)  =  b2a23
      betaCC(1,2,2:4)  =  b4a23
      betaCC(2,1,2:4)  =  betaCC(1,2,2:4)
      betaCC(3,2,2:4)  =  b4a13
      betaCC(2,3,2:4)  =  betaCC(3,2,2:4)
      betaCC(1,1,2:4)  = (0.d0,0.d0)
      betaCC(2,2,2:4)  = (0.d0,0.d0)
      betaCC(3,3,2:4)  = (0.d0,0.d0)

      elseif (s(1,2).le.0d0.and.s(1,3).le.0d0.and.s(2,3).ge.0d0) then
      u13 = -s(1,3)/s(2,3)
      u23 = -s(1,2)/s(2,3)
      v   =  s123 /s(2,3)

      alphaCC(:,:,2:4) = (0.d0,0.d0)
      betaCC(:,:,2:4)  = (0.d0,0.d0)

      call alpha2a1L(u13,v,a2a13)
      call alpha2a1L(u23,v,a2a23)
      call  beta2a1L(u13,v,b2a13)
      call  beta2a1L(u23,v,b2a23)
      call  beta4a1L(u13,v,b4a13)
      call  beta4a1L(u23,v,b4a23)

      alphaCC(3,2,2:4) = a2a13
      alphaCC(2,3,2:4) = a2a23
      alphaCC(3,1,2:4) = alphaCC(2,3,2:4)
      alphaCC(1,3,2:4) = alphaCC(2,3,2:4)
      alphaCC(2,1,2:4) = alphaCC(3,2,2:4)
      alphaCC(1,2,2:4) = alphaCC(3,2,2:4)
      alphaCC(1,1,2:4) = (0.d0,0.d0)
      alphaCC(2,2,2:4) = (0.d0,0.d0)
      alphaCC(3,3,2:4) = (0.d0,0.d0)

      betaCC(3,2,2:4)  =  b2a13
      betaCC(2,3,2:4)  =  b2a23
      betaCC(3,1,2:4)  =  b4a23
      betaCC(1,3,2:4)  =  betaCC(3,1,2:4)
      betaCC(2,1,2:4)  =  b4a13
      betaCC(1,2,2:4)  =  betaCC(2,1,2:4)
      betaCC(1,1,2:4)  = (0.d0,0.d0)
      betaCC(2,2,2:4)  = (0.d0,0.d0)
      betaCC(3,3,2:4)  = (0.d0,0.d0)

      elseif(s(1,2).ge.0d0.and.s(1,3).ge.0d0.and.s(2,3).ge.0d0) then
         y = s(1,3)/s123
         z = s(2,3)/s123
         x = 1d0-y-z
         
         alphaCC(:,:,2:4) = (0.d0,0.d0)
         betaCC(:,:,2:4)  = (0.d0,0.d0)
         
         call alpha1a1L(y,z,a1a13)
         call alpha1a1L(z,y,a1a23)
         call  beta1a1L(y,z,b1a12)
         call  beta1a1L(z,y,b1a21)
         call  beta1a1L(x,z,b1a13)
         call  beta1a1L(z,x,b1a31)
         call  beta1a1L(x,y,b1a23)
         call  beta1a1L(y,x,b1a32)

      alphaCC(1,2,2:4) = a1a13
      alphaCC(2,1,2:4) = a1a23
      alphaCC(1,3,2:4) = alphaCC(2,1,2:4)
      alphaCC(3,1,2:4) = alphaCC(2,1,2:4)
      alphaCC(2,3,2:4) = alphaCC(1,2,2:4)
      alphaCC(3,2,2:4) = alphaCC(1,2,2:4)
      alphaCC(1,1,2:4) = (0.d0,0.d0)
      alphaCC(2,2,2:4) = (0.d0,0.d0)
      alphaCC(3,3,2:4) = (0.d0,0.d0)

      betaCC(1,2,2:4)  =  b1a12
      betaCC(2,1,2:4)  =  b1a21
      betaCC(1,3,2:4)  =  b1a13
      betaCC(3,1,2:4)  =  b1a31
      betaCC(2,3,2:4)  =  b1a23
      betaCC(3,2,2:4)  =  b1a32
      betaCC(1,1,2:4)  = (0.d0,0.d0)
      betaCC(2,2,2:4)  = (0.d0,0.d0)
      betaCC(3,3,2:4)  = (0.d0,0.d0)
      else
         write(6,*) 'wrong kinematic region in initG1LC'
      endif   
      end subroutine initG1LC

!#############################################################

      subroutine initQ1LC(s,i1,i2,i3)
      implicit none
      !Initialises gamma
      ! at one loop
      !If c(3)=0 g in final state qqb->gh
      !If c(3)=1 g in initial state qg->qh/qbg->qbh
      !--------------------------------------------
      integer, intent(in) :: i1,i2,i3
      real*8, dimension(3,3), intent(in) :: s
      !internal
      real*8 :: v,u,u1,u2
      real*8 :: y,z
      real*8 :: s123
      complex*16, dimension(3) :: gam1,gam2


      s123 = s(i1,i2) + s(i1,i3) + s(i2,i3)

      gammaCC(:,2:4) = (0.d0,0.d0)

      if (s(i1,i2).ge.0d0.and.s(i1,i3).le.0d0.and.s(i2,i3).le.0d0) then
         u1 = -s(i1,i3)/s(i1,i2)
         u2 = -s(i2,i3)/s(i1,i2)
         v =    s123/s(i1,i2)

         call gamma2a1L(u1,v,gam1)
         call gamma2a1L(u2,v,gam2)

      elseif (s(i1,i2).le.0d0.and.s(i1,i3).le.0d0.and.s(i2,i3).ge.0d0) then
         u = -s(i1,i3)/s(i2,i3)
         v =    s123/s(i2,i3)

         call gamma4a1L(u,v,gam1)
         call gamma3a1L(u,v,gam2)

      elseif (s(i1,i2).le.0d0.and.s(i1,i3).ge.0d0.and.s(i2,i3).le.0d0) then
         u = -s(i2,i3)/s(i1,i3)
         v =    s123/s(i1,i3)

         call gamma3a1L(u,v,gam1)
         call gamma4a1L(u,v,gam2)

      elseif(s(i1,i2).ge.0d0.and.s(i1,i3).ge.0d0.and.s(i2,i3).ge.0d0) then
         y = s(i1,i3)/s123
         z = s(i2,i3)/s123
         call gamma1a1L(y,z,gam1)
         call gamma1a1L(z,y,gam2)
      else
         write(6,*) 'wrong kinematic region in initQ1LC'
      endif
      
      gammaCC(1,2:4) = gam1
      gammaCC(2,2:4) = gam2

      end subroutine initQ1LC

!#############################################################

      subroutine initG0LC(s)
      implicit none
! initialises alpha,beta
! at tree level
!--------------
      real*8, dimension(3,3), intent(in) :: s
!internal
      complex*16 :: coeff


      coeff = (1.d0,0.d0)

      alphaCC(:,:,1) = (0.d0,0.d0)
      betaCC(:,:,1)  = (0.d0,0.d0)

      alphaCC(1,2,1) = coeff
      alphaCC(2,1,1) = coeff
      alphaCC(1,3,1) = coeff
      alphaCC(3,1,1) = coeff
      alphaCC(2,3,1) = coeff
      alphaCC(3,2,1) = coeff
      alphaCC(1,1,1) = (0.d0,0.d0)
      alphaCC(2,2,1) = (0.d0,0.d0)
      alphaCC(3,3,1) = (0.d0,0.d0)

      betaCC(1,2,1)  = coeff
      betaCC(2,1,1)  = coeff
      betaCC(1,3,1)  = coeff
      betaCC(3,1,1)  = coeff
      betaCC(2,3,1)  = coeff
      betaCC(3,2,1)  = coeff
      betaCC(1,1,1)  = (0.d0,0.d0)
      betaCC(2,2,1)  = (0.d0,0.d0)
      betaCC(3,3,1)  = (0.d0,0.d0)

      end subroutine initG0LC

!#############################################################

      subroutine initQ0LC(s,i1,i2,i3)
      implicit none
      !Initialises gamma
      ! at tree level
      !--------------
      real*8, dimension(3,3), intent(in) :: s
      integer, intent(in) :: i1,i2,i3
      !internal
      real*8 :: v,u
      real*8 :: s123


      s123 = s(1,2) + s(1,3) + s(2,3)

      gammaCC(:,1) = (0.d0,0.d0)

      gammaCC(1,1) = (1.d0,0.d0)
      gammaCC(2,1) = (1.d0,0.d0)

      end subroutine initQ0LC

!#############################################################

      subroutine alpha2a1L(u,v,col)
      implicit none
      !returns alpha(2a+)
      ! at one Loop (Catani-subtracted)
      !--------------------------------
      real*8, intent(in) :: u,v
      complex*16, dimension(3), intent(out) :: col
      !internal
      complex*16 :: alph
      complex*16 :: ci
      real*8 :: omu,omv,omumv,upv
      real*8, dimension(0:1) :: HPL1
      real*8, dimension(0:1,0:1) :: HPL2
      real*8, dimension(0:1,0:1,0:1) :: HPL3
      real*8, dimension(0:1,0:1,0:1,0:1) :: HPL4
      real*8, dimension(0:3) :: GPL1
      real*8, dimension(0:3,0:3) :: GPL2
      real*8, dimension(0:3,0:3,0:3) :: GPL3
      real*8, dimension(0:3,0:3,0:3,0:3) :: GPL4
      complex*16 :: col1,col2,col3


      omu   = 1.0d0 - u
      omv   = 1.0d0     - v
      omumv = 1.0d0 - u - v
      upv   =         u + v

      ci = (0.d0,1.d0)

      call tdhpl(dble(u),dble(v),4,GPL1,GPL2,GPL3,GPL4,HPL1,HPL2,HPL3,HPL4)

      if(ieorder>=2) then
         col1 = - 3.0d0/2.0d0

         col2 = 0.0d0

         col3 = 0.0d0
      elseif(ieorder==1) then
         col1 = - 11.0d0/4.0d0
     .          - 1.0d0/2.0d0*pi*ci
     .          - 3.0d0/2.0d0*HPL1(0)
     .          - 1.0d0/2.0d0*HPL1(1)
     .          + 1.0d0/2.0d0*GPL1(0)
     .          + 1.0d0/2.0d0*GPL1(2)

         col2 = 0

         col3 = + 1.0d0/2.0d0
      else
         if(catanisub) then
            col1 = - 1.0d0/6.0d0*v**(-2)
     .           + 1.0d0/6.0d0*v**(-1)
     .           + 11.0d0/12.0d0*pi*ci
     .           + 5.0d0/12.0d0*pi**2
     .           + 1.0d0/6.0d0*u*v**(-2)
     .           - 1.0d0/6.0d0*u*v**(-1)
     .           - 1.0d0/6.0d0*u**2*v**(-2)
     .           + 11.0d0/4.0d0*HPL1(0)
     .           + HPL1(0)*pi*ci
     .           + HPL1(0)*GPL1(0)
     .           - HPL1(0)*GPL1(1)
     .           + HPL1(0)*GPL1(2)
     .           - HPL1(0)*GPL1(3)
     .           + 11.0d0/12.0d0*HPL1(1)
     .           + 1.0d0/2.0d0*HPL1(1)*pi*ci
            col1 = col1 + 1.0d0/2.0d0*HPL1(1)*GPL1(0)
     .           - HPL1(1)*GPL1(1)
     .           - HPL2(0,0)
     .           - 11.0d0/12.0d0*GPL1(0)
     .           - 1.0d0/2.0d0*GPL1(0)*pi*ci
     .           + GPL1(1)*pi*ci
     .           - 11.0d0/12.0d0*GPL1(2)
     .           - 1.0d0/2.0d0*GPL1(2)*pi*ci
     .           + GPL1(3)*pi*ci
     .           - 1.0d0/2.0d0*GPL2(0,2)
     .           + GPL2(1,2)
     .           - 1.0d0/2.0d0*GPL2(2,0)
     .           + GPL2(3,0)

            col2 = (0.d0,0.d0)

            col3 = + 1.0d0/6.0d0*v**(-2)
     .           - 1.0d0/6.0d0*v**(-1)
     .           - 1.0d0/6.0d0*pi*ci
     .           - 1.0d0/6.0d0*u*v**(-2)
     .           + 1.0d0/6.0d0*u*v**(-1)
     .           + 1.0d0/6.0d0*u**2*v**(-2)
     .           - 1.0d0/2.0d0*HPL1(0)
     .           - 1.0d0/6.0d0*HPL1(1)
     .           + 1.0d0/6.0d0*GPL1(0)
     .           + 1.0d0/6.0d0*GPL1(2)
         else
            col1 = - 1.0d0/6.0d0*v**(-2)
     .           + 1.0d0/6.0d0*v**(-1)
     .           + 19.0d0/24.0d0*pi**2
     .           + 1.0d0/6.0d0*u*v**(-2)
     .           - 1.0d0/6.0d0*u*v**(-1)
     .           - 1.0d0/6.0d0*u**2*v**(-2)
     .           + 1.0d0/2.0d0*HPL1(0)*pi*ci
     .           + 3.0d0/2.0d0*HPL1(0)*GPL1(0)
     .           - HPL1(0)*GPL1(1)
     .           + 3.0d0/2.0d0*HPL1(0)*GPL1(2)
     .           - HPL1(0)*GPL1(3)
     .           + 1.0d0/2.0d0*HPL1(1)*pi*ci
     .           + 1.0d0/2.0d0*HPL1(1)*GPL1(0)
     .           - HPL1(1)*GPL1(1)
     .           + 1.0d0/2.0d0*HPL1(1)*GPL1(2)
            col1 = col1 - 5.0d0/2.0d0*HPL2(0,0)
     .           - 1.0d0/2.0d0*HPL2(0,1)
     .           - 1.0d0/2.0d0*HPL2(1,0)
     .           - 1.0d0/2.0d0*HPL2(1,1)
     .           - 1.0d0/2.0d0*GPL1(0)*pi*ci
     .           + GPL1(1)*pi*ci
     .           - 1.0d0/2.0d0*GPL1(2)*pi*ci
     .           + GPL1(3)*pi*ci
     .           - 1.0d0/2.0d0*GPL2(0,0)
     .           - 1.0d0/2.0d0*GPL2(0,2)
     .           + GPL2(1,2)
     .           - 1.0d0/2.0d0*GPL2(2,0)
     .           - 1.0d0/2.0d0*GPL2(2,2)
     .           + GPL2(3,0)

            col2 = (0.d0,0.d0)

            col3 = + 1.0d0/6.0d0*v**(-2)
     .           - 1.0d0/6.0d0*v**(-1)
     .           - 1.0d0/6.0d0*u*v**(-2)
     .           + 1.0d0/6.0d0*u*v**(-1)
     .           + 1.0d0/6.0d0*u**2*v**(-2)
         endif
      endif

      col(1) = col1
      col(2) = col2
      col(3) = col3

      end subroutine alpha2a1L

!#############################################################

      subroutine beta2a1L(u,v,col)
      implicit none
      !returns beta(2a+)
      ! at one Loop (Catani-subtracted)
      !--------------------------------
      real*8, intent(in) :: u,v
      complex*16, dimension(3), intent(out) :: col
      !internal
      complex*16 :: beta
      complex*16 :: ci
      real*8 :: omu,omv,omumv,upv
      real*8, dimension(0:1) :: HPL1
      real*8, dimension(0:1,0:1) :: HPL2
      real*8, dimension(0:1,0:1,0:1) :: HPL3
      real*8, dimension(0:1,0:1,0:1,0:1) :: HPL4
      real*8, dimension(0:3) :: GPL1
      real*8, dimension(0:3,0:3) :: GPL2
      real*8, dimension(0:3,0:3,0:3) :: GPL3
      real*8, dimension(0:3,0:3,0:3,0:3) :: GPL4
      complex*16 :: col1,col2,col3


      omu   = 1.0d0 - u
      omv   = 1.0d0     - v
      omumv = 1.0d0 - u - v
      upv   =         u + v

      ci = (0.d0,1.d0)

      call tdhpl(dble(u),dble(v),4,GPL1,GPL2,GPL3,GPL4,HPL1,HPL2,HPL3,HPL4)

      if(ieorder>=2) then
         col1 = - 3.0d0/2.0d0

         col2 = 0.0d0

         col3 = 0.0d0
      elseif(ieorder==1) then
         col1 = - 11.0d0/4.0d0
     .          - 1.0d0/2.0d0*pi*ci
     .          - 3.0d0/2.0d0*HPL1(0)
     .          - 1.0d0/2.0d0*HPL1(1)
     .          + 1.0d0/2.0d0*GPL1(0)
     .          + 1.0d0/2.0d0*GPL1(2)

         col2 = 0

         col3 = + 1.0d0/2.0d0
      else
         if(catanisub) then
            col1 = + 11.0d0/12.0d0*pi*ci
     .           + 5.0d0/12.0d0*pi**2
     .           + 1.0d0/6.0d0*u
     .           - 1.0d0/6.0d0*u*v
     .           - 1.0d0/6.0d0*u**2
     .           + 11.0d0/4.0d0*HPL1(0)
     .           + HPL1(0)*pi*ci
     .           + HPL1(0)*GPL1(0)
     .           - HPL1(0)*GPL1(1)
     .           + HPL1(0)*GPL1(2)
     .           - HPL1(0)*GPL1(3)
     .           + 11.0d0/12.0d0*HPL1(1)
     .           + 1.0d0/2.0d0*HPL1(1)*pi*ci
     .           + 1.0d0/2.0d0*HPL1(1)*GPL1(0)
     .           - HPL1(1)*GPL1(1)
            col1 = col1 - HPL2(0,0)
     .           - 11.0d0/12.0d0*GPL1(0)
     .           - 1.0d0/2.0d0*GPL1(0)*pi*ci
     .           + GPL1(1)*pi*ci
     .           - 11.0d0/12.0d0*GPL1(2)
     .           - 1.0d0/2.0d0*GPL1(2)*pi*ci
     .           + GPL1(3)*pi*ci
     .           - 1.0d0/2.0d0*GPL2(0,2)
     .           + GPL2(1,2)
     .           - 1.0d0/2.0d0*GPL2(2,0)
     .           + GPL2(3,0)

            col2 = (0.d0,0.d0)

            col3 = - 1.0d0/6.0d0*pi*ci
     .           - 1.0d0/6.0d0*u
     .           + 1.0d0/6.0d0*u*v
     .           + 1.0d0/6.0d0*u**2
     .           - 1.0d0/2.0d0*HPL1(0)
     .           - 1.0d0/6.0d0*HPL1(1)
     .           + 1.0d0/6.0d0*GPL1(0)
     .           + 1.0d0/6.0d0*GPL1(2)
         else
            col1 = + 19.0d0/24.0d0*pi**2
     .           + 1.0d0/6.0d0*u
     .           - 1.0d0/6.0d0*u*v
     .           - 1.0d0/6.0d0*u**2
     .           + 1.0d0/2.0d0*HPL1(0)*pi*ci
     .           + 3.0d0/2.0d0*HPL1(0)*GPL1(0)
     .           - HPL1(0)*GPL1(1)
     .           + 3.0d0/2.0d0*HPL1(0)*GPL1(2)
     .           - HPL1(0)*GPL1(3)
     .           + 1.0d0/2.0d0*HPL1(1)*pi*ci
     .           + 1.0d0/2.0d0*HPL1(1)*GPL1(0)
     .           - HPL1(1)*GPL1(1)
     .           + 1.0d0/2.0d0*HPL1(1)*GPL1(2)
     .           - 5.0d0/2.0d0*HPL2(0,0)
     .           - 1.0d0/2.0d0*HPL2(0,1)
            col1 = col1 - 1.0d0/2.0d0*HPL2(1,0)
     .           - 1.0d0/2.0d0*HPL2(1,1)
     .           - 1.0d0/2.0d0*GPL1(0)*pi*ci
     .           + GPL1(1)*pi*ci
     .           - 1.0d0/2.0d0*GPL1(2)*pi*ci
     .           + GPL1(3)*pi*ci
     .           - 1.0d0/2.0d0*GPL2(0,0)
     .           - 1.0d0/2.0d0*GPL2(0,2)
     .           + GPL2(1,2)
     .           - 1.0d0/2.0d0*GPL2(2,0)
     .           - 1.0d0/2.0d0*GPL2(2,2)
     .           + GPL2(3,0)

            col2 = (0.d0,0.d0)

            col3 = - 1.0d0/6.0d0*u
     .           + 1.0d0/6.0d0*u*v
     .           + 1.0d0/6.0d0*u**2
         endif
      endif

      col(1) = col1
      col(2) = col2
      col(3) = col3

      end subroutine beta2a1L

!#############################################################

      subroutine beta4a1L(u,v,col)
      implicit none
      !returns beta(4a+)
      ! at one Loop (Catani-subtracted)
      !--------------------------------
      real*8, intent(in) :: u,v
      complex*16, dimension(3), intent(out) :: col
      !internal
      complex*16 :: beta
      complex*16 :: ci
      real*8 :: omu,omv,omumv,upv
      real*8, dimension(0:1) :: HPL1
      real*8, dimension(0:1,0:1) :: HPL2
      real*8, dimension(0:1,0:1,0:1) :: HPL3
      real*8, dimension(0:1,0:1,0:1,0:1) :: HPL4
      real*8, dimension(0:3) :: GPL1
      real*8, dimension(0:3,0:3) :: GPL2
      real*8, dimension(0:3,0:3,0:3) :: GPL3
      real*8, dimension(0:3,0:3,0:3,0:3) :: GPL4
      complex*16 :: col1,col2,col3


      omu   = 1.0d0 - u
      omv   = 1.0d0     - v
      omumv = 1.0d0 - u - v
      upv   =         u + v

      ci = (0.d0,1.d0)

      call tdhpl(dble(u),dble(v),4,GPL1,GPL2,GPL3,GPL4,HPL1,HPL2,HPL3,HPL4)

      if(ieorder>=2) then
         col1 = - 3.0d0/2.0d0

         col2 = 0.0d0

         col3 = 0.0d0
      elseif(ieorder==1) then
         col1 = - 11.0d0/4.0d0
     .          - 1.0d0/2.0d0*pi*ci
     .          - 3.0d0/2.0d0*HPL1(0)
     .          - 1.0d0/2.0d0*HPL1(1)
     .          + 1.0d0/2.0d0*GPL1(0)
     .          + 1.0d0/2.0d0*GPL1(2)

         col2 = 0

         col3 = + 1.0d0/2.0d0
      else
         if(catanisub) then
            col1 = - 1.0d0/6.0d0*omumv**(-2)
     .           + 1.0d0/6.0d0*omumv**(-1)
     .           + 11.0d0/12.0d0*pi*ci
     .           + 5.0d0/12.0d0*pi**2
     .           + 1.0d0/6.0d0*v*omumv**(-2)
     .           + 11.0d0/4.0d0*HPL1(0)
     .           + HPL1(0)*pi*ci
     .           + HPL1(0)*GPL1(0)
     .           - HPL1(0)*GPL1(1)
     .           + HPL1(0)*GPL1(2)
     .           - HPL1(0)*GPL1(3)
     .           + 11.0d0/12.0d0*HPL1(1)
     .           + 1.0d0/2.0d0*HPL1(1)*pi*ci
     .           + 1.0d0/2.0d0*HPL1(1)*GPL1(0)
     .           - HPL1(1)*GPL1(1)
            col1 = col1 - HPL2(0,0)
     .           - 11.0d0/12.0d0*GPL1(0)
     .           - 1.0d0/2.0d0*GPL1(0)*pi*ci
     .           + GPL1(1)*pi*ci
     .           - 11.0d0/12.0d0*GPL1(2)
     .           - 1.0d0/2.0d0*GPL1(2)*pi*ci
     .           + GPL1(3)*pi*ci
     .           - 1.0d0/2.0d0*GPL2(0,2)
     .           + GPL2(1,2)
     .           - 1.0d0/2.0d0*GPL2(2,0)
     .           + GPL2(3,0)

            col2 = (0.d0,0.d0)

            col3 = + 1.0d0/6.0d0*omumv**(-2)
     .           - 1.0d0/6.0d0*omumv**(-1)
     .           - 1.0d0/6.0d0*pi*ci
     .           - 1.0d0/6.0d0*v*omumv**(-2)
     .           - 1.0d0/2.0d0*HPL1(0)
     .           - 1.0d0/6.0d0*HPL1(1)
     .           + 1.0d0/6.0d0*GPL1(0)
     .           + 1.0d0/6.0d0*GPL1(2)
         else
            col1 = - 1.0d0/6.0d0*omumv**(-2)
     .           + 1.0d0/6.0d0*omumv**(-1)
     .           + 19.0d0/24.0d0*pi**2
     .           + 1.0d0/6.0d0*v*omumv**(-2)
     .           + 1.0d0/2.0d0*HPL1(0)*pi*ci
     .           + 3.0d0/2.0d0*HPL1(0)*GPL1(0)
     .           - HPL1(0)*GPL1(1)
     .           + 3.0d0/2.0d0*HPL1(0)*GPL1(2)
     .           - HPL1(0)*GPL1(3)
     .           + 1.0d0/2.0d0*HPL1(1)*pi*ci
     .           + 1.0d0/2.0d0*HPL1(1)*GPL1(0)
     .           - HPL1(1)*GPL1(1)
     .           + 1.0d0/2.0d0*HPL1(1)*GPL1(2)
     .           - 5.0d0/2.0d0*HPL2(0,0)
     .           - 1.0d0/2.0d0*HPL2(0,1)
            col1 = col1 - 1.0d0/2.0d0*HPL2(1,0)
     .           - 1.0d0/2.0d0*HPL2(1,1)
     .           - 1.0d0/2.0d0*GPL1(0)*pi*ci
     .           + GPL1(1)*pi*ci
     .           - 1.0d0/2.0d0*GPL1(2)*pi*ci
     .           + GPL1(3)*pi*ci
     .           - 1.0d0/2.0d0*GPL2(0,0)
     .           - 1.0d0/2.0d0*GPL2(0,2)
     .           + GPL2(1,2)
     .           - 1.0d0/2.0d0*GPL2(2,0)
     .           - 1.0d0/2.0d0*GPL2(2,2)
     .           + GPL2(3,0)

            col2 = (0.d0,0.d0)

            col3 = + 1.0d0/6.0d0*omumv**(-2)
     .           - 1.0d0/6.0d0*omumv**(-1)
     .           - 1.0d0/6.0d0*v*omumv**(-2)
         endif
      endif

      col(1) = col1
      col(2) = col2
      col(3) = col3

      end subroutine beta4a1L

!#############################################################

      subroutine gamma2a1L(u,v,col)
      implicit none
      !Returns gamma(2a+) at one-loop
      !------------------------------
      real*8, intent(in) :: u,v
      complex*16, dimension(3), intent(out) :: col
      !internal
      complex*16 :: gam
      complex*16 :: ci
      real*8 :: omu,omv,omumv,upv
      real*8, dimension(0:1) :: HPL1
      real*8, dimension(0:1,0:1) :: HPL2
      real*8, dimension(0:1,0:1,0:1) :: HPL3
      real*8, dimension(0:1,0:1,0:1,0:1) :: HPL4
      real*8, dimension(0:3) :: GPL1
      real*8, dimension(0:3,0:3) :: GPL2
      real*8, dimension(0:3,0:3,0:3) :: GPL3
      real*8, dimension(0:3,0:3,0:3,0:3) :: GPL4
      complex*16 :: col1,col2,col3


      omu   = 1.0d0 - u
      omv   = 1.0d0     - v
      omumv = 1.0d0 - u - v
      upv   =         u + v

      ci = (0.d0,1.d0)

      call tdhpl(dble(u),dble(v),4,GPL1,GPL2,GPL3,GPL4,HPL1,HPL2,HPL3,HPL4)

      if(ieorder>=2) then
         col1 = 1.0d0

         col2 = - 1.0d0/2.0d0

         col3 = 0.0d0
      elseif(ieorder==1) then
         col1 = + 5.0d0/3.0d0
     .          + HPL1(0)
     .          + 1.0d0/2.0d0*HPL1(1)
     .          - 1.0d0/2.0d0*GPL1(0)
     .          - 1.0d0/2.0d0*GPL1(2)

         col2 = - 3.0d0/4.0d0
     .          - 1.0d0/2.0d0*pi*ci
     .          - 1.0d0/2.0d0*HPL1(0)

         col3 = - 1.0d0/6.0d0
      else
         if(catanisub) then
            col1 = - 89.0d0/36.0d0
     .           - 1.0d0/4.0d0*u**(-1)*omumv
     .           - 1.0d0/4.0d0*u**(-1)*v
     .           - 13.0d0/12.0d0*pi*ci
     .           - 1.0d0/12.0d0*pi**2
     .           - 11.0d0/4.0d0*HPL1(0)
     .           - 1.0d0/2.0d0*HPL1(0)*pi*ci
     .           - 1.0d0/2.0d0*HPL1(0)*GPL1(0)
     .           + 1.0d0/2.0d0*HPL1(0)*GPL1(1)
     .           - 1.0d0/2.0d0*HPL1(0)*GPL1(2)
     .           + 1.0d0/2.0d0*HPL1(0)*GPL1(3)
     .           - 5.0d0/6.0d0*HPL1(1)
     .           + 1.0d0/2.0d0*HPL1(1)*GPL1(1)
     .           + 1.0d0/2.0d0*HPL2(0,0)
     .           - 1.0d0/2.0d0*HPL2(1,0)
            col1 = col1 + 5.0d0/6.0d0*GPL1(0)
     .           - 1.0d0/2.0d0*GPL1(1)*pi*ci
     .           + 5.0d0/6.0d0*GPL1(2)
     .           - 1.0d0/2.0d0*GPL1(3)*pi*ci
     .           - 1.0d0/2.0d0*GPL2(1,2)
     .           - 1.0d0/2.0d0*GPL2(3,0)

            col2 = - 9.0d0/4.0d0
     .           - 1.0d0/4.0d0*u**(-1)*omumv
     .           - 1.0d0/4.0d0*u**(-1)*v
     .           + 1.0d0/3.0d0*pi**2
     .           + 1.0d0/2.0d0*HPL1(0)*pi*ci
     .           + 1.0d0/2.0d0*HPL1(0)*GPL1(0)
     .           - 1.0d0/2.0d0*HPL1(0)*GPL1(1)
     .           + 1.0d0/2.0d0*HPL1(0)*GPL1(2)
     .           - 1.0d0/2.0d0*HPL1(0)*GPL1(3)
     .           + 1.0d0/2.0d0*HPL1(1)*pi*ci
     .           + 1.0d0/2.0d0*HPL1(1)*GPL1(0)
     .           - 1.0d0/2.0d0*HPL1(1)*GPL1(1)
     .           - 1.0d0/2.0d0*HPL2(0,0)
     .           - 1.0d0/2.0d0*HPL2(1,0)
     .           - 1.0d0/2.0d0*GPL1(0)*pi*ci
            col2 = col2 + 1.0d0/2.0d0*GPL1(1)*pi*ci
     .           - 1.0d0/2.0d0*GPL1(2)*pi*ci
     .           + 1.0d0/2.0d0*GPL1(3)*pi*ci
     .           - 1.0d0/2.0d0*GPL2(0,2)
     .           + 1.0d0/2.0d0*GPL2(1,2)
     .           - 1.0d0/2.0d0*GPL2(2,0)
     .           + 1.0d0/2.0d0*GPL2(3,0)

            col3 = + 5.0d0/9.0d0
     .           + 1.0d0/3.0d0*pi*ci
     .           + 1.0d0/2.0d0*HPL1(0)
     .           + 1.0d0/12.0d0*HPL1(1)
     .           - 1.0d0/12.0d0*GPL1(0)
     .           - 1.0d0/12.0d0*GPL1(2)
         else
            col1 = - 20.0d0/9.0d0
     .           - 1.0d0/4.0d0*u**(-1)
     .           - 13.0d0/12.0d0*pi*ci
     .           - 1.0d0/6.0d0*pi**2
     .           - 13.0d0/12.0d0*HPL1(0)
     .           - 1.0d0/2.0d0*HPL1(0)*pi*ci
     .           - HPL1(0)*GPL1(0)
     .           + 1.0d0/2.0d0*HPL1(0)*GPL1(1)
     .           - HPL1(0)*GPL1(2)
     .           + 1.0d0/2.0d0*HPL1(0)*GPL1(3)
     .           + 1.0d0/2.0d0*HPL1(1)*GPL1(1)
     .           - 1.0d0/2.0d0*HPL1(1)*GPL1(2)
     .           + 3.0d0/2.0d0*HPL2(0,0)
     .           + 1.0d0/2.0d0*HPL2(0,1)
     .           + 1.0d0/2.0d0*HPL2(1,1)
            col1 = col1 - 1.0d0/2.0d0*GPL1(1)*pi*ci
     .           - 1.0d0/2.0d0*GPL1(3)*pi*ci
     .           + 1.0d0/2.0d0*GPL2(0,0)
     .           - 1.0d0/2.0d0*GPL2(1,2)
     .           + 1.0d0/2.0d0*GPL2(2,2)
     .           - 1.0d0/2.0d0*GPL2(3,0)

            col2 = - 2.0d0
     .           - 1.0d0/4.0d0*u**(-1)
     .           - 3.0d0/4.0d0*pi*ci
     .           + 5.0d0/8.0d0*pi**2
     .           - 3.0d0/4.0d0*HPL1(0)
     .           + 1.0d0/2.0d0*HPL1(0)*GPL1(0)
     .           - 1.0d0/2.0d0*HPL1(0)*GPL1(1)
     .           + 1.0d0/2.0d0*HPL1(0)*GPL1(2)
     .           - 1.0d0/2.0d0*HPL1(0)*GPL1(3)
     .           + 1.0d0/2.0d0*HPL1(1)*pi*ci
     .           + 1.0d0/2.0d0*HPL1(1)*GPL1(0)
     .           - 1.0d0/2.0d0*HPL1(1)*GPL1(1)
     .           - HPL2(0,0)
     .           - 1.0d0/2.0d0*HPL2(1,0)
     .           - 1.0d0/2.0d0*GPL1(0)*pi*ci
            col2 = col2 + 1.0d0/2.0d0*GPL1(1)*pi*ci
     .           - 1.0d0/2.0d0*GPL1(2)*pi*ci
     .           + 1.0d0/2.0d0*GPL1(3)*pi*ci
     .           - 1.0d0/2.0d0*GPL2(0,2)
     .           + 1.0d0/2.0d0*GPL2(1,2)
     .           - 1.0d0/2.0d0*GPL2(2,0)
     .           + 1.0d0/2.0d0*GPL2(3,0)

            col3 = + 5.0d0/9.0d0
     .           + 1.0d0/3.0d0*pi*ci
     .           + 1.0d0/3.0d0*HPL1(0)
         endif
      endif

      col(1) = col1*(-1.d0)
      col(2) = col2*(-1.d0)
      col(3) = col3*(-1.d0)     !To be consistent with gamma_0=1

      end subroutine gamma2a1L

!#############################################################

      subroutine gamma3a1L(u,v,col)
      implicit none
      !Returns gamma(3a+) at one-loop
      !------------------------------
      real*8, intent(in) :: u,v
      complex*16, dimension(3), intent(out) :: col
      !internal
      complex*16 :: gam
      complex*16 :: ci
      real*8 :: omu,omv,omumv,upv
      real*8, dimension(0:1) :: HPL1
      real*8, dimension(0:1,0:1) :: HPL2
      real*8, dimension(0:1,0:1,0:1) :: HPL3
      real*8, dimension(0:1,0:1,0:1,0:1) :: HPL4
      real*8, dimension(0:3) :: GPL1
      real*8, dimension(0:3,0:3) :: GPL2
      real*8, dimension(0:3,0:3,0:3) :: GPL3
      real*8, dimension(0:3,0:3,0:3,0:3) :: GPL4
      complex*16 :: col1,col2,col3


      omu   = 1.0d0 - u
      omv   = 1.0d0     - v
      omumv = 1.0d0 - u - v
      upv   =         u + v

      ci = (0.d0,1.d0)

      call tdhpl(dble(u),dble(v),4,GPL1,GPL2,GPL3,GPL4,HPL1,HPL2,HPL3,HPL4)

      if(ieorder>=2) then
         col1 = 1.0d0

         col2 = - 1.0d0/2.0d0

         col3 = 0.0d0
      elseif(ieorder==1) then
         col1 = + 5.0d0/3.0d0
     .          + 1.0d0/2.0d0*pi*ci
     .          + HPL1(0)
     .          - 1.0d0/2.0d0*GPL1(0)

         col2 = - 3.0d0/4.0d0
     .          - 1.0d0/2.0d0*HPL1(0)
     .          - 1.0d0/2.0d0*HPL1(1)
     .          + 1.0d0/2.0d0*GPL1(2)

         col3 = - 1.0d0/6.0d0
      else
         if(catanisub) then
            col1 = - 89.0d0/36.0d0
     .           - 5.0d0/6.0d0*pi*ci
     .           - 1.0d0/3.0d0*pi**2
     .           + 1.0d0/4.0d0*v
     .           + 1.0d0/4.0d0*u
     .           - 11.0d0/4.0d0*HPL1(0)
     .           - 1.0d0/2.0d0*HPL1(0)*pi*ci
     .           - 1.0d0/2.0d0*HPL1(0)*GPL1(0)
     .           + HPL1(0)*GPL1(1)
     .           - HPL1(0)*GPL1(2)
     .           + 1.0d0/2.0d0*HPL1(0)*GPL1(3)
     .           - 13.0d0/12.0d0*HPL1(1)
     .           - 1.0d0/2.0d0*HPL1(1)*pi*ci
     .           - 1.0d0/2.0d0*HPL1(1)*GPL1(0)
     .           + HPL1(1)*GPL1(1)
            col1 = col1 + 1.0d0/2.0d0*HPL2(0,0)
     .           + 1.0d0/2.0d0*HPL2(1,0)
     .           + 5.0d0/6.0d0*GPL1(0)
     .           + 1.0d0/2.0d0*GPL1(0)*pi*ci
     .           - GPL1(1)*pi*ci
     .           + 13.0d0/12.0d0*GPL1(2)
     .           + 1.0d0/2.0d0*GPL1(2)*pi*ci
     .           - 1.0d0/2.0d0*GPL1(3)*pi*ci
     .           + 1.0d0/2.0d0*GPL2(0,2)
     .           - GPL2(1,2)
     .           + 1.0d0/2.0d0*GPL2(2,0)
     .           - 1.0d0/2.0d0*GPL2(3,0)

            col2 = - 9.0d0/4.0d0
     .           + 1.0d0/12.0d0*pi**2
     .           + 1.0d0/4.0d0*v
     .           + 1.0d0/4.0d0*u
     .           + 1.0d0/2.0d0*HPL1(0)*pi*ci
     .           + 1.0d0/2.0d0*HPL1(0)*GPL1(0)
     .           - 1.0d0/2.0d0*HPL1(0)*GPL1(3)
     .           - 1.0d0/2.0d0*HPL2(0,0)
     .           + 1.0d0/2.0d0*HPL2(1,0)
     .           + 1.0d0/2.0d0*GPL1(3)*pi*ci
     .           + 1.0d0/2.0d0*GPL2(3,0)

            col3 = + 5.0d0/9.0d0
     .           + 1.0d0/12.0d0*pi*ci
     .           + 1.0d0/2.0d0*HPL1(0)
     .           + 1.0d0/3.0d0*HPL1(1)
     .           - 1.0d0/12.0d0*GPL1(0)
     .           - 1.0d0/3.0d0*GPL1(2)
         else
            col1 = - 89.0d0/36.0d0
     .           - 2.0d0/3.0d0*pi**2
     .           + 1.0d0/4.0d0*v
     .           + 1.0d0/4.0d0*u
     .           - 13.0d0/12.0d0*HPL1(0)
     .           - HPL1(0)*GPL1(0)
     .           + HPL1(0)*GPL1(1)
     .           - HPL1(0)*GPL1(2)
     .           + 1.0d0/2.0d0*HPL1(0)*GPL1(3)
     .           - 13.0d0/12.0d0*HPL1(1)
     .           - 1.0d0/2.0d0*HPL1(1)*pi*ci
     .           - 1.0d0/2.0d0*HPL1(1)*GPL1(0)
     .           + HPL1(1)*GPL1(1)
     .           + 3.0d0/2.0d0*HPL2(0,0)
     .           + 1.0d0/2.0d0*HPL2(1,0)
            col1 = col1 + 1.0d0/2.0d0*GPL1(0)*pi*ci
     .           - GPL1(1)*pi*ci
     .           + 13.0d0/12.0d0*GPL1(2)
     .           + 1.0d0/2.0d0*GPL1(2)*pi*ci
     .           - 1.0d0/2.0d0*GPL1(3)*pi*ci
     .           + 1.0d0/2.0d0*GPL2(0,0)
     .           + 1.0d0/2.0d0*GPL2(0,2)
     .           - GPL2(1,2)
     .           + 1.0d0/2.0d0*GPL2(2,0)
     .           - 1.0d0/2.0d0*GPL2(3,0)

            col2 = - 9.0d0/4.0d0
     .           + 1.0d0/8.0d0*pi**2
     .           + 1.0d0/4.0d0*v
     .           + 1.0d0/4.0d0*u
     .           - 3.0d0/4.0d0*HPL1(0)
     .           + 1.0d0/2.0d0*HPL1(0)*pi*ci
     .           + 1.0d0/2.0d0*HPL1(0)*GPL1(0)
     .           + 1.0d0/2.0d0*HPL1(0)*GPL1(2)
     .           - 1.0d0/2.0d0*HPL1(0)*GPL1(3)
     .           - 3.0d0/4.0d0*HPL1(1)
     .           + 1.0d0/2.0d0*HPL1(1)*GPL1(2)
     .           - HPL2(0,0)
     .           - 1.0d0/2.0d0*HPL2(0,1)
     .           - 1.0d0/2.0d0*HPL2(1,1)
     .           + 3.0d0/4.0d0*GPL1(2)
            col2 = col2 + 1.0d0/2.0d0*GPL1(3)*pi*ci
     .           - 1.0d0/2.0d0*GPL2(2,2)
     .           + 1.0d0/2.0d0*GPL2(3,0)

            col3 = + 5.0d0/9.0d0
     .           + 1.0d0/3.0d0*HPL1(0)
     .           + 1.0d0/3.0d0*HPL1(1)
     .           - 1.0d0/3.0d0*GPL1(2)
         endif
      endif

      col(1) = col1*(-1.d0)
      col(2) = col2*(-1.d0)
      col(3) = col3*(-1.d0)     !To be consistent with gamma_0=1

      end subroutine gamma3a1L

!#############################################################

      subroutine gamma4a1L(u,v,col)
      implicit none
      !Returns gamma(4a+) at one-loop
      !------------------------------
      real*8, intent(in) :: u,v
      complex*16, dimension(3), intent(out) :: col
      !internal
      complex*16 :: gam
      complex*16 :: ci
      real*8 :: omu,omv,omumv,upv
      real*8, dimension(0:1) :: HPL1
      real*8, dimension(0:1,0:1) :: HPL2
      real*8, dimension(0:1,0:1,0:1) :: HPL3
      real*8, dimension(0:1,0:1,0:1,0:1) :: HPL4
      real*8, dimension(0:3) :: GPL1
      real*8, dimension(0:3,0:3) :: GPL2
      real*8, dimension(0:3,0:3,0:3) :: GPL3
      real*8, dimension(0:3,0:3,0:3,0:3) :: GPL4
      complex*16 :: col1,col2,col3


      omu   = 1.0d0 - u
      omv   = 1.0d0     - v
      omumv = 1.0d0 - u - v
      upv   =         u + v

      ci = (0.d0,1.d0)

      call tdhpl(dble(u),dble(v),4,GPL1,GPL2,GPL3,GPL4,HPL1,HPL2,HPL3,HPL4)

      if(ieorder>=2) then
         col1 = 1.0d0

         col2 = - 1.0d0/2.0d0

         col3 = 0.0d0
      elseif(ieorder==1) then
         col1 = + 5.0d0/3.0d0
     .          + 1.0d0/2.0d0*pi*ci
     .          + HPL1(0)
     .          - 1.0d0/2.0d0*GPL1(0)

         col2 = - 3.0d0/4.0d0
     .          - 1.0d0/2.0d0*HPL1(0)
     .          - 1.0d0/2.0d0*HPL1(1)
     .          + 1.0d0/2.0d0*GPL1(2)

         col3 = - 1.0d0/6.0d0
      else
         if(catanisub) then
            col1 = - 89.0d0/36.0d0
     .           + 1.0d0/4.0d0*u**(-1)
     .           - 1.0d0/4.0d0*u**(-1)*v
     .           - 5.0d0/6.0d0*pi*ci
     .           - 1.0d0/3.0d0*pi**2
     .           - 11.0d0/4.0d0*HPL1(0)
     .           - 1.0d0/2.0d0*HPL1(0)*pi*ci
     .           - 1.0d0/2.0d0*HPL1(0)*GPL1(0)
     .           + HPL1(0)*GPL1(1)
     .           - HPL1(0)*GPL1(2)
     .           + 1.0d0/2.0d0*HPL1(0)*GPL1(3)
     .           - 13.0d0/12.0d0*HPL1(1)
     .           - 1.0d0/2.0d0*HPL1(1)*pi*ci
     .           - 1.0d0/2.0d0*HPL1(1)*GPL1(0)
     .           + HPL1(1)*GPL1(1)
            col1 = col1 + 1.0d0/2.0d0*HPL2(0,0)
     .           + 1.0d0/2.0d0*HPL2(1,0)
     .           + 5.0d0/6.0d0*GPL1(0)
     .           + 1.0d0/2.0d0*GPL1(0)*pi*ci
     .           - GPL1(1)*pi*ci
     .           + 13.0d0/12.0d0*GPL1(2)
     .           + 1.0d0/2.0d0*GPL1(2)*pi*ci
     .           - 1.0d0/2.0d0*GPL1(3)*pi*ci
     .           + 1.0d0/2.0d0*GPL2(0,2)
     .           - GPL2(1,2)
     .           + 1.0d0/2.0d0*GPL2(2,0)
     .           - 1.0d0/2.0d0*GPL2(3,0)

            col2 = - 9.0d0/4.0d0
     .           + 1.0d0/4.0d0*u**(-1)
     .           - 1.0d0/4.0d0*u**(-1)*v
     .           + 1.0d0/12.0d0*pi**2
     .           + 1.0d0/2.0d0*HPL1(0)*pi*ci
     .           + 1.0d0/2.0d0*HPL1(0)*GPL1(0)
     .           - 1.0d0/2.0d0*HPL1(0)*GPL1(3)
     .           - 1.0d0/2.0d0*HPL2(0,0)
     .           + 1.0d0/2.0d0*HPL2(1,0)
     .           + 1.0d0/2.0d0*GPL1(3)*pi*ci
     .           + 1.0d0/2.0d0*GPL2(3,0)

            col3 = + 5.0d0/9.0d0
     .           + 1.0d0/12.0d0*pi*ci
     .           + 1.0d0/2.0d0*HPL1(0)
     .           + 1.0d0/3.0d0*HPL1(1)
     .           - 1.0d0/12.0d0*GPL1(0)
     .           - 1.0d0/3.0d0*GPL1(2)
         else
            col1 = - 89.0d0/36.0d0
     .           + 1.0d0/4.0d0*u**(-1)
     .           - 1.0d0/4.0d0*u**(-1)*v
     .           - 2.0d0/3.0d0*pi**2
     .           - 13.0d0/12.0d0*HPL1(0)
     .           - HPL1(0)*GPL1(0)
     .           + HPL1(0)*GPL1(1)
     .           - HPL1(0)*GPL1(2)
     .           + 1.0d0/2.0d0*HPL1(0)*GPL1(3)
     .           - 13.0d0/12.0d0*HPL1(1)
     .           - 1.0d0/2.0d0*HPL1(1)*pi*ci
     .           - 1.0d0/2.0d0*HPL1(1)*GPL1(0)
     .           + HPL1(1)*GPL1(1)
     .           + 3.0d0/2.0d0*HPL2(0,0)
     .           + 1.0d0/2.0d0*HPL2(1,0)
            col1 = col1 + 1.0d0/2.0d0*GPL1(0)*pi*ci
     .           - GPL1(1)*pi*ci
     .           + 13.0d0/12.0d0*GPL1(2)
     .           + 1.0d0/2.0d0*GPL1(2)*pi*ci
     .           - 1.0d0/2.0d0*GPL1(3)*pi*ci
     .           + 1.0d0/2.0d0*GPL2(0,0)
     .           + 1.0d0/2.0d0*GPL2(0,2)
     .           - GPL2(1,2)
     .           + 1.0d0/2.0d0*GPL2(2,0)
     .           - 1.0d0/2.0d0*GPL2(3,0)

            col2 = - 9.0d0/4.0d0
     .           + 1.0d0/4.0d0*u**(-1)
     .           - 1.0d0/4.0d0*u**(-1)*v
     .           + 1.0d0/8.0d0*pi**2
     .           - 3.0d0/4.0d0*HPL1(0)
     .           + 1.0d0/2.0d0*HPL1(0)*pi*ci
     .           + 1.0d0/2.0d0*HPL1(0)*GPL1(0)
     .           + 1.0d0/2.0d0*HPL1(0)*GPL1(2)
     .           - 1.0d0/2.0d0*HPL1(0)*GPL1(3)
     .           - 3.0d0/4.0d0*HPL1(1)
     .           + 1.0d0/2.0d0*HPL1(1)*GPL1(2)
     .           - HPL2(0,0)
     .           - 1.0d0/2.0d0*HPL2(0,1)
     .           - 1.0d0/2.0d0*HPL2(1,1)
     .           + 3.0d0/4.0d0*GPL1(2)
            col2 = col2 + 1.0d0/2.0d0*GPL1(3)*pi*ci
     .           - 1.0d0/2.0d0*GPL2(2,2)
     .           + 1.0d0/2.0d0*GPL2(3,0)

            col3 = + 5.0d0/9.0d0
     .           + 1.0d0/3.0d0*HPL1(0)
     .           + 1.0d0/3.0d0*HPL1(1)
     .           - 1.0d0/3.0d0*GPL1(2)
         endif
      endif

      col(1) = col1*(-1.d0)
      col(2) = col2*(-1.d0)
      col(3) = col3*(-1.d0)     !To be consistent with gamma_0=1

      end subroutine gamma4a1L

!#############################################################

      subroutine alpha2a2L(u,v,col)
      implicit none
      !returns alpha(2a+)
      ! at two Loops
      !-------------
      real*8, intent(in) :: u,v
      complex*16, dimension(6), intent(out) :: col
      !internal
      complex*16 :: alph
      complex*16 :: ci
      real*8 :: omu,omv,omumv,upv
      real*8, dimension(0:1) :: HPL1
      real*8, dimension(0:1,0:1) :: HPL2
      real*8, dimension(0:1,0:1,0:1) :: HPL3
      real*8, dimension(0:1,0:1,0:1,0:1) :: HPL4
      real*8, dimension(0:3) :: GPL1
      real*8, dimension(0:3,0:3) :: GPL2
      real*8, dimension(0:3,0:3,0:3) :: GPL3
      real*8, dimension(0:3,0:3,0:3,0:3) :: GPL4
      complex*16 :: col1,col2,col3,col4,col5,col6

      omu   = 1.0d0 - u
      omv   = 1.0d0     - v
      omumv = 1.0d0 - u - v
      upv   =         u + v

      ci = (0.d0,1.d0)

      call tdhpl(dble(u),dble(v),4,GPL1,GPL2,GPL3,GPL4,HPL1,HPL2,HPL3,HPL4)

      if(ieorder==4) then
         col1 = 9.0d0/8.0d0

         col2 = 0.0d0

         col3 = 0.0d0

         col4 = 0.0d0

         col5 = 0.0d0

         col6 = 0.0d0
      elseif(ieorder==3) then
         col1 = + 99.0d0/16.0d0
     .          + 3.0d0/4.0d0*pi*ci
     .          + 9.0d0/4.0d0*HPL1(0)
     .          + 3.0d0/4.0d0*HPL1(1)
     .          - 3.0d0/4.0d0*GPL1(0)
     .          - 3.0d0/4.0d0*GPL1(2)

         col2 = 0

         col3 = 0

         col4 = 0

         col5 = - 9.0d0/8.0d0

         col6 = 0
      else
         col1 = + 1321.0d0/216.0d0
     .        - 5.0d0/6.0d0*u**(-1)*v**(-2)*pi*ci
     .        + 5.0d0/3.0d0*u**(-1)*v**(-1)*pi*ci
     .        - 5.0d0/6.0d0*u**(-1)*pi*ci
     .        - 5.0d0/36.0d0*v**(-2)*omumv**(-2)*pi**2
     .        - 5.0d0/6.0d0*v**(-2)*omumv**(-1)*pi*ci
     .        + 5.0d0/18.0d0*v**(-2)*omumv**(-1)*pi**2
     .        - 247.0d0/108.0d0*v**(-2)
     .        + 139.0d0/72.0d0*v**(-2)*pi*ci
     .        - 5.0d0/24.0d0*v**(-2)*pi**2
     .        + 5.0d0/18.0d0*v**(-1)*omumv**(-2)*pi**2
     .        + 5.0d0/3.0d0*v**(-1)*omumv**(-1)*pi*ci
     .        - 5.0d0/8.0d0*v**(-1)*omumv**(-1)*pi**2
     .        + 247.0d0/108.0d0*v**(-1)
     .        - 265.0d0/72.0d0*v**(-1)*pi*ci
         col1 = col1 + 5.0d0/12.0d0*v**(-1)*pi**2
     .        - 5.0d0/36.0d0*omumv**(-2)*pi**2
     .        - 5.0d0/6.0d0*omumv**(-1)*pi*ci
     .        + 25.0d0/72.0d0*omumv**(-1)*pi**2
     .        + 149.0d0/24.0d0*pi*ci
     .        + 935.0d0/864.0d0*pi**2
     .        + 121.0d0/288.0d0*pi**3*ci
     .        + 49.0d0/2880.0d0*pi**4
     .        + 143.0d0/48.0d0*zeta3
     .        + 1.0d0/4.0d0*zeta3*pi*ci
     .        + 247.0d0/108.0d0*u*v**(-2)
     .        + 11.0d0/24.0d0*u*v**(-2)*pi*ci
     .        + 11.0d0/72.0d0*u*v**(-2)*pi**2
     .        - 247.0d0/108.0d0*u*v**(-1)
     .        + 3.0d0/8.0d0*u*v**(-1)*pi*ci
         col1 = col1 - 113.0d0/72.0d0*u*v**(-1)*pi**2
     .        - 5.0d0/6.0d0*u*pi*ci
     .        + 25.0d0/18.0d0*u*pi**2
     .        - 247.0d0/108.0d0*u**2*v**(-2)
     .        - 11.0d0/24.0d0*u**2*v**(-2)*pi*ci
     .        - 53.0d0/72.0d0*u**2*v**(-2)*pi**2
     .        - 5.0d0/6.0d0*u**2*v**(-1)*pi*ci
     .        + 5.0d0/2.0d0*u**2*v**(-1)*pi**2
     .        - 5.0d0/9.0d0*u**2*pi**2
     .        + 10.0d0/9.0d0*u**3*v**(-2)*pi**2
     .        - 10.0d0/9.0d0*u**3*v**(-1)*pi**2
     .        - 5.0d0/9.0d0*u**4*v**(-2)*pi**2
     .        + 25.0d0/8.0d0*HPL1(0)
     .        - 5.0d0/6.0d0*HPL1(0)*u**(-1)*v**(-1)
     .        + 5.0d0/6.0d0*HPL1(0)*u**(-1)
         col1 = col1 - 5.0d0/6.0d0*HPL1(0)*v**(-2)*omumv**(-2)*pi*ci
     .        + 5.0d0/3.0d0*HPL1(0)*v**(-2)*omumv**(-1)*pi*ci
     .        - 55.0d0/72.0d0*HPL1(0)*v**(-2)
     .        - HPL1(0)*v**(-2)*pi*ci
     .        + 5.0d0/3.0d0*HPL1(0)*v**(-1)*omumv**(-2)*pi*ci
     .        - 5.0d0/6.0d0*HPL1(0)*v**(-1)*omumv**(-1)
     .        - 15.0d0/4.0d0*HPL1(0)*v**(-1)*omumv**(-1)*pi*ci
     .        + 175.0d0/72.0d0*HPL1(0)*v**(-1)
     .        + 9.0d0/4.0d0*HPL1(0)*v**(-1)*pi*ci
     .        - 5.0d0/6.0d0*HPL1(0)*omumv**(-2)*pi*ci
     .        + 5.0d0/6.0d0*HPL1(0)*omumv**(-1)
     .        + 25.0d0/12.0d0*HPL1(0)*omumv**(-1)*pi*ci
     .        + 1141.0d0/144.0d0*HPL1(0)*pi*ci
     .        + 11.0d0/32.0d0*HPL1(0)*pi**2
     .        + 1.0d0/4.0d0*HPL1(0)*pi**3*ci
         col1 = col1 + 3.0d0/4.0d0*HPL1(0)*zeta3
     .        + 55.0d0/72.0d0*HPL1(0)*u*v**(-2)
     .        + 1.0d0/6.0d0*HPL1(0)*u*v**(-2)*pi*ci
     .        - 115.0d0/72.0d0*HPL1(0)*u*v**(-1)
     .        - 29.0d0/12.0d0*HPL1(0)*u*v**(-1)*pi*ci
     .        + 5.0d0/6.0d0*HPL1(0)*u
     .        + 25.0d0/12.0d0*HPL1(0)*u*pi*ci
     .        - 55.0d0/72.0d0*HPL1(0)*u**2*v**(-2)
     .        - 7.0d0/6.0d0*HPL1(0)*u**2*v**(-2)*pi*ci
     .        + 5.0d0/6.0d0*HPL1(0)*u**2*v**(-1)
     .        + 15.0d0/4.0d0*HPL1(0)*u**2*v**(-1)*pi*ci
     .        - 5.0d0/6.0d0*HPL1(0)*u**2*pi*ci
     .        + 5.0d0/3.0d0*HPL1(0)*u**3*v**(-2)*pi*ci
     .        - 5.0d0/3.0d0*HPL1(0)*u**3*v**(-1)*pi*ci
     .        - 5.0d0/6.0d0*HPL1(0)*u**4*v**(-2)*pi*ci
         col1 = col1 - 23.0d0/48.0d0*HPL1(0)*GPL1(0)
     .        - 5.0d0/6.0d0*HPL1(0)*GPL1(0)*v**(-2)*omumv**(-2)
     .        + 5.0d0/3.0d0*HPL1(0)*GPL1(0)*v**(-2)*omumv**(-1)
     .        - HPL1(0)*GPL1(0)*v**(-2)
     .        + 5.0d0/3.0d0*HPL1(0)*GPL1(0)*v**(-1)*omumv**(-2)
     .        - 15.0d0/4.0d0*HPL1(0)*GPL1(0)*v**(-1)*omumv**(-1)
     .        + 9.0d0/4.0d0*HPL1(0)*GPL1(0)*v**(-1)
     .        - 5.0d0/6.0d0*HPL1(0)*GPL1(0)*omumv**(-2)
     .        + 25.0d0/12.0d0*HPL1(0)*GPL1(0)*omumv**(-1)
     .        - 55.0d0/24.0d0*HPL1(0)*GPL1(0)*pi*ci
     .        + 3.0d0/4.0d0*HPL1(0)*GPL1(0)*pi**2
     .        + 1.0d0/6.0d0*HPL1(0)*GPL1(0)*u*v**(-2)
     .        - 29.0d0/12.0d0*HPL1(0)*GPL1(0)*u*v**(-1)
     .        + 25.0d0/12.0d0*HPL1(0)*GPL1(0)*u
     .        - 7.0d0/6.0d0*HPL1(0)*GPL1(0)*u**2*v**(-2)
         col1 = col1 + 15.0d0/4.0d0*HPL1(0)*GPL1(0)*u**2*v**(-1)
     .        - 5.0d0/6.0d0*HPL1(0)*GPL1(0)*u**2
     .        + 5.0d0/3.0d0*HPL1(0)*GPL1(0)*u**3*v**(-2)
     .        - 5.0d0/3.0d0*HPL1(0)*GPL1(0)*u**3*v**(-1)
     .        - 5.0d0/6.0d0*HPL1(0)*GPL1(0)*u**4*v**(-2)
     .        - 67.0d0/18.0d0*HPL1(0)*GPL1(1)
     .        + 5.0d0/6.0d0*HPL1(0)*GPL1(1)*u**(-2)*v**(-2)
     .        - 5.0d0/3.0d0*HPL1(0)*GPL1(1)*u**(-2)*v**(-1)
     .        + 5.0d0/6.0d0*HPL1(0)*GPL1(1)*u**(-2)
     .        - 5.0d0/3.0d0*HPL1(0)*GPL1(1)*u**(-1)*v**(-2)
     .        + 15.0d0/4.0d0*HPL1(0)*GPL1(1)*u**(-1)*v**(-1)
     .        - 25.0d0/12.0d0*HPL1(0)*GPL1(1)*u**(-1)
     .        + 7.0d0/6.0d0*HPL1(0)*GPL1(1)*v**(-2)
     .        - 29.0d0/12.0d0*HPL1(0)*GPL1(1)*v**(-1)
     .        + 11.0d0/12.0d0*HPL1(0)*GPL1(1)*pi*ci
         col1 = col1 - 7.0d0/12.0d0*HPL1(0)*GPL1(1)*pi**2
     .        - 1.0d0/2.0d0*HPL1(0)*GPL1(1)*u*v**(-2)
     .        + 29.0d0/12.0d0*HPL1(0)*GPL1(1)*u*v**(-1)
     .        - 25.0d0/12.0d0*HPL1(0)*GPL1(1)*u
     .        + 7.0d0/6.0d0*HPL1(0)*GPL1(1)*u**2*v**(-2)
     .        - 15.0d0/4.0d0*HPL1(0)*GPL1(1)*u**2*v**(-1)
     .        + 5.0d0/6.0d0*HPL1(0)*GPL1(1)*u**2
     .        - 5.0d0/3.0d0*HPL1(0)*GPL1(1)*u**3*v**(-2)
     .        + 5.0d0/3.0d0*HPL1(0)*GPL1(1)*u**3*v**(-1)
     .        + 5.0d0/6.0d0*HPL1(0)*GPL1(1)*u**4*v**(-2)
     .        - 23.0d0/48.0d0*HPL1(0)*GPL1(2)
     .        - 5.0d0/6.0d0*HPL1(0)*GPL1(2)*u**(-2)*v**(-2)
     .        + 5.0d0/3.0d0*HPL1(0)*GPL1(2)*u**(-2)*v**(-1)
     .        - 5.0d0/6.0d0*HPL1(0)*GPL1(2)*u**(-2)
     .        + 5.0d0/3.0d0*HPL1(0)*GPL1(2)*u**(-1)*v**(-2)
         col1 = col1 - 15.0d0/4.0d0*HPL1(0)*GPL1(2)*u**(-1)*v**(-1)
     .        + 25.0d0/12.0d0*HPL1(0)*GPL1(2)*u**(-1)
     .        - 7.0d0/6.0d0*HPL1(0)*GPL1(2)*v**(-2)
     .        + 29.0d0/12.0d0*HPL1(0)*GPL1(2)*v**(-1)
     .        + 55.0d0/24.0d0*HPL1(0)*GPL1(2)*pi*ci
     .        + 1.0d0/12.0d0*HPL1(0)*GPL1(2)*pi**2
     .        + 1.0d0/2.0d0*HPL1(0)*GPL1(2)*u*v**(-2)
     .        - 29.0d0/12.0d0*HPL1(0)*GPL1(2)*u*v**(-1)
     .        + 25.0d0/12.0d0*HPL1(0)*GPL1(2)*u
     .        - 7.0d0/6.0d0*HPL1(0)*GPL1(2)*u**2*v**(-2)
     .        + 15.0d0/4.0d0*HPL1(0)*GPL1(2)*u**2*v**(-1)
     .        - 5.0d0/6.0d0*HPL1(0)*GPL1(2)*u**2
     .        + 5.0d0/3.0d0*HPL1(0)*GPL1(2)*u**3*v**(-2)
     .        - 5.0d0/3.0d0*HPL1(0)*GPL1(2)*u**3*v**(-1)
     .        - 5.0d0/6.0d0*HPL1(0)*GPL1(2)*u**4*v**(-2)
         col1 = col1 - 67.0d0/18.0d0*HPL1(0)*GPL1(3)
     .        + 5.0d0/6.0d0*HPL1(0)*GPL1(3)*v**(-2)*omumv**(-2)
     .        - 5.0d0/3.0d0*HPL1(0)*GPL1(3)*v**(-2)*omumv**(-1)
     .        + HPL1(0)*GPL1(3)*v**(-2)
     .        - 5.0d0/3.0d0*HPL1(0)*GPL1(3)*v**(-1)*omumv**(-2)
     .        + 15.0d0/4.0d0*HPL1(0)*GPL1(3)*v**(-1)*omumv**(-1)
     .        - 9.0d0/4.0d0*HPL1(0)*GPL1(3)*v**(-1)
     .        + 5.0d0/6.0d0*HPL1(0)*GPL1(3)*omumv**(-2)
     .        - 25.0d0/12.0d0*HPL1(0)*GPL1(3)*omumv**(-1)
     .        - 11.0d0/12.0d0*HPL1(0)*GPL1(3)*pi*ci
     .        - 5.0d0/4.0d0*HPL1(0)*GPL1(3)*pi**2
     .        - 1.0d0/6.0d0*HPL1(0)*GPL1(3)*u*v**(-2)
     .        + 29.0d0/12.0d0*HPL1(0)*GPL1(3)*u*v**(-1)
     .        - 25.0d0/12.0d0*HPL1(0)*GPL1(3)*u
     .        + 7.0d0/6.0d0*HPL1(0)*GPL1(3)*u**2*v**(-2)
         col1 = col1 - 15.0d0/4.0d0*HPL1(0)*GPL1(3)*u**2*v**(-1)
     .        + 5.0d0/6.0d0*HPL1(0)*GPL1(3)*u**2
     .        - 5.0d0/3.0d0*HPL1(0)*GPL1(3)*u**3*v**(-2)
     .        + 5.0d0/3.0d0*HPL1(0)*GPL1(3)*u**3*v**(-1)
     .        + 5.0d0/6.0d0*HPL1(0)*GPL1(3)*u**4*v**(-2)
     .        - 11.0d0/3.0d0*HPL1(0)*GPL2(0,0)
     .        - HPL1(0)*GPL2(0,0)*pi*ci
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(0,1)*pi*ci
     .        - 77.0d0/24.0d0*HPL1(0)*GPL2(0,2)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(0,2)*pi*ci
     .        + 55.0d0/12.0d0*HPL1(0)*GPL2(0,3)
     .        + 3.0d0/2.0d0*HPL1(0)*GPL2(0,3)*pi*ci
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(1,0)*pi*ci
     .        + 11.0d0/6.0d0*HPL1(0)*GPL2(1,1)
     .        + 11.0d0/3.0d0*HPL1(0)*GPL2(1,2)
         col1 = col1 + 1.0d0/2.0d0*HPL1(0)*GPL2(1,2)*pi*ci
     .        - 77.0d0/24.0d0*HPL1(0)*GPL2(2,0)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(2,0)*pi*ci
     .        + 55.0d0/12.0d0*HPL1(0)*GPL2(2,1)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(2,1)*pi*ci
     .        - 11.0d0/3.0d0*HPL1(0)*GPL2(2,2)
     .        + HPL1(0)*GPL2(2,2)*pi*ci
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(2,3)*pi*ci
     .        + 11.0d0/3.0d0*HPL1(0)*GPL2(3,0)
     .        + 5.0d0/2.0d0*HPL1(0)*GPL2(3,0)*pi*ci
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(3,2)*pi*ci
     .        + 11.0d0/6.0d0*HPL1(0)*GPL2(3,3)
     .        - 2.0d0*HPL1(0)*GPL2(3,3)*pi*ci
     .        - HPL1(0)*GPL3(0,0,1)
     .        + HPL1(0)*GPL3(0,0,3)
         col1 = col1 - HPL1(0)*GPL3(0,1,0)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(0,1,2)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(0,2,1)
     .        - HPL1(0)*GPL3(0,2,2)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(0,2,3)
     .        + HPL1(0)*GPL3(0,3,0)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(0,3,2)
     .        - 2.0d0*HPL1(0)*GPL3(0,3,3)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(1,0,2)
     .        + 2.0d0*HPL1(0)*GPL3(1,1,1)
     .        - 2.0d0*HPL1(0)*GPL3(1,1,2)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(1,2,0)
     .        - 2.0d0*HPL1(0)*GPL3(1,2,1)
     .        + 2.0d0*HPL1(0)*GPL3(1,2,2)
     .        - HPL1(0)*GPL3(2,0,0)
         col1 = col1 + 1.0d0/2.0d0*HPL1(0)*GPL3(2,0,1)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(2,0,3)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(2,1,0)
     .        - 2.0d0*HPL1(0)*GPL3(2,1,1)
     .        + HPL1(0)*GPL3(2,1,2)
     .        + HPL1(0)*GPL3(2,2,1)
     .        - HPL1(0)*GPL3(2,2,3)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(2,3,0)
     .        - HPL1(0)*GPL3(2,3,2)
     .        + 2.0d0*HPL1(0)*GPL3(3,0,0)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(3,0,2)
     .        - 2.0d0*HPL1(0)*GPL3(3,0,3)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(3,2,0)
     .        - 2.0d0*HPL1(0)*GPL3(3,3,0)
     .        + 2.0d0*HPL1(0)*GPL3(3,3,3)
         col1 = col1 - 1.0d0/8.0d0*HPL1(1)
     .        + 5.0d0/6.0d0*HPL1(1)*u**(-1)*v**(-2)
     .        - 5.0d0/3.0d0*HPL1(1)*u**(-1)*v**(-1)
     .        + 5.0d0/6.0d0*HPL1(1)*u**(-1)
     .        - 101.0d0/72.0d0*HPL1(1)*v**(-2)
     .        - 1.0d0/12.0d0*HPL1(1)*v**(-2)*pi*ci
     .        + 227.0d0/72.0d0*HPL1(1)*v**(-1)
     .        + 1.0d0/12.0d0*HPL1(1)*v**(-1)*pi*ci
     .        + 389.0d0/144.0d0*HPL1(1)*pi*ci
     .        - 55.0d0/288.0d0*HPL1(1)*pi**2
     .        + 1.0d0/8.0d0*HPL1(1)*pi**3*ci
     .        + 1.0d0/4.0d0*HPL1(1)*zeta3
     .        + 49.0d0/72.0d0*HPL1(1)*u*v**(-2)
     .        + 1.0d0/4.0d0*HPL1(1)*u*v**(-2)*pi*ci
     .        - 227.0d0/72.0d0*HPL1(1)*u*v**(-1)
         col1 = col1 - 7.0d0/3.0d0*HPL1(1)*u*v**(-1)*pi*ci
     .        + 5.0d0/6.0d0*HPL1(1)*u
     .        + 25.0d0/12.0d0*HPL1(1)*u*pi*ci
     .        - 101.0d0/72.0d0*HPL1(1)*u**2*v**(-2)
     .        - 13.0d0/12.0d0*HPL1(1)*u**2*v**(-2)*pi*ci
     .        + 5.0d0/3.0d0*HPL1(1)*u**2*v**(-1)
     .        + 15.0d0/4.0d0*HPL1(1)*u**2*v**(-1)*pi*ci
     .        - 5.0d0/6.0d0*HPL1(1)*u**2*pi*ci
     .        + 5.0d0/6.0d0*HPL1(1)*u**3*v**(-2)
     .        + 5.0d0/3.0d0*HPL1(1)*u**3*v**(-2)*pi*ci
     .        - 5.0d0/3.0d0*HPL1(1)*u**3*v**(-1)*pi*ci
     .        - 5.0d0/6.0d0*HPL1(1)*u**4*v**(-2)*pi*ci
     .        + 49.0d0/48.0d0*HPL1(1)*GPL1(0)
     .        - 1.0d0/12.0d0*HPL1(1)*GPL1(0)*v**(-2)
     .        + 1.0d0/12.0d0*HPL1(1)*GPL1(0)*v**(-1)
         col1 = col1 - 11.0d0/8.0d0*HPL1(1)*GPL1(0)*pi*ci
     .        + 3.0d0/8.0d0*HPL1(1)*GPL1(0)*pi**2
     .        + 1.0d0/4.0d0*HPL1(1)*GPL1(0)*u*v**(-2)
     .        - 7.0d0/3.0d0*HPL1(1)*GPL1(0)*u*v**(-1)
     .        + 25.0d0/12.0d0*HPL1(1)*GPL1(0)*u
     .        - 13.0d0/12.0d0*HPL1(1)*GPL1(0)*u**2*v**(-2)
     .        + 15.0d0/4.0d0*HPL1(1)*GPL1(0)*u**2*v**(-1)
     .        - 5.0d0/6.0d0*HPL1(1)*GPL1(0)*u**2
     .        + 5.0d0/3.0d0*HPL1(1)*GPL1(0)*u**3*v**(-2)
     .        - 5.0d0/3.0d0*HPL1(1)*GPL1(0)*u**3*v**(-1)
     .        - 5.0d0/6.0d0*HPL1(1)*GPL1(0)*u**4*v**(-2)
     .        - 67.0d0/18.0d0*HPL1(1)*GPL1(1)
     .        + 5.0d0/6.0d0*HPL1(1)*GPL1(1)*u**(-2)*v**(-2)
     .        - 5.0d0/3.0d0*HPL1(1)*GPL1(1)*u**(-2)*v**(-1)
     .        + 5.0d0/6.0d0*HPL1(1)*GPL1(1)*u**(-2)
         col1 = col1 - 5.0d0/3.0d0*HPL1(1)*GPL1(1)*u**(-1)*v**(-2)
     .        + 15.0d0/4.0d0*HPL1(1)*GPL1(1)*u**(-1)*v**(-1)
     .        - 25.0d0/12.0d0*HPL1(1)*GPL1(1)*u**(-1)
     .        + 7.0d0/6.0d0*HPL1(1)*GPL1(1)*v**(-2)
     .        - 29.0d0/12.0d0*HPL1(1)*GPL1(1)*v**(-1)
     .        - 11.0d0/12.0d0*HPL1(1)*GPL1(1)*pi*ci
     .        - 5.0d0/12.0d0*HPL1(1)*GPL1(1)*pi**2
     .        - 1.0d0/2.0d0*HPL1(1)*GPL1(1)*u*v**(-2)
     .        + 29.0d0/12.0d0*HPL1(1)*GPL1(1)*u*v**(-1)
     .        - 25.0d0/12.0d0*HPL1(1)*GPL1(1)*u
     .        + 7.0d0/6.0d0*HPL1(1)*GPL1(1)*u**2*v**(-2)
     .        - 15.0d0/4.0d0*HPL1(1)*GPL1(1)*u**2*v**(-1)
     .        + 5.0d0/6.0d0*HPL1(1)*GPL1(1)*u**2
     .        - 5.0d0/3.0d0*HPL1(1)*GPL1(1)*u**3*v**(-2)
     .        + 5.0d0/3.0d0*HPL1(1)*GPL1(1)*u**3*v**(-1)
         col1 = col1 + 5.0d0/6.0d0*HPL1(1)*GPL1(1)*u**4*v**(-2)
     .        - 121.0d0/48.0d0*HPL1(1)*GPL1(2)
     .        - 11.0d0/6.0d0*HPL1(1)*GPL1(2)*pi*ci
     .        - 1.0d0/12.0d0*HPL1(1)*GPL1(2)*pi**2
     .        - 1.0d0/2.0d0*HPL1(1)*GPL1(3)*pi**2
     .        - 11.0d0/6.0d0*HPL1(1)*GPL2(0,0)
     .        - 1.0d0/2.0d0*HPL1(1)*GPL2(0,0)*pi*ci
     .        - 11.0d0/6.0d0*HPL1(1)*GPL2(0,2)
     .        - 1.0d0/2.0d0*HPL1(1)*GPL2(0,2)*pi*ci
     .        + 1.0d0/2.0d0*HPL1(1)*GPL2(0,3)*pi*ci
     .        + HPL1(1)*GPL2(1,0)*pi*ci
     .        + 11.0d0/6.0d0*HPL1(1)*GPL2(1,1)
     .        - 2.0d0*HPL1(1)*GPL2(1,1)*pi*ci
     .        + 11.0d0/3.0d0*HPL1(1)*GPL2(1,2)
     .        + HPL1(1)*GPL2(1,2)*pi*ci
         col1 = col1 - 11.0d0/6.0d0*HPL1(1)*GPL2(2,0)
     .        - 1.0d0/2.0d0*HPL1(1)*GPL2(2,0)*pi*ci
     .        + 55.0d0/12.0d0*HPL1(1)*GPL2(2,1)
     .        + 1.0d0/2.0d0*HPL1(1)*GPL2(2,1)*pi*ci
     .        - HPL1(1)*GPL2(2,3)*pi*ci
     .        + HPL1(1)*GPL2(3,0)*pi*ci
     .        - HPL1(1)*GPL3(0,0,1)
     .        - 1.0d0/2.0d0*HPL1(1)*GPL3(0,0,2)
     .        - HPL1(1)*GPL3(0,1,0)
     .        + HPL1(1)*GPL3(0,1,2)
     .        - 1.0d0/2.0d0*HPL1(1)*GPL3(0,2,0)
     .        + 1.0d0/2.0d0*HPL1(1)*GPL3(0,2,1)
     .        + 1.0d0/2.0d0*HPL1(1)*GPL3(0,3,0)
     .        + HPL1(1)*GPL3(1,0,2)
     .        + 2.0d0*HPL1(1)*GPL3(1,1,1)
         col1 = col1 - 2.0d0*HPL1(1)*GPL3(1,1,2)
     .        + HPL1(1)*GPL3(1,2,0)
     .        - 2.0d0*HPL1(1)*GPL3(1,2,1)
     .        - 1.0d0/2.0d0*HPL1(1)*GPL3(2,0,0)
     .        + 1.0d0/2.0d0*HPL1(1)*GPL3(2,0,1)
     .        + 1.0d0/2.0d0*HPL1(1)*GPL3(2,1,0)
     .        - 2.0d0*HPL1(1)*GPL3(2,1,1)
     .        + HPL1(1)*GPL3(2,2,1)
     .        - HPL1(1)*GPL3(2,3,0)
     .        + HPL1(1)*GPL3(3,0,0)
     .        + 1279.0d0/144.0d0*HPL2(0,0)
     .        + 5.0d0/6.0d0*HPL2(0,0)*v**(-2)*omumv**(-2)
     .        - 5.0d0/3.0d0*HPL2(0,0)*v**(-2)*omumv**(-1)
     .        + HPL2(0,0)*v**(-2)
     .        - 5.0d0/3.0d0*HPL2(0,0)*v**(-1)*omumv**(-2)
         col1 = col1 + 15.0d0/4.0d0*HPL2(0,0)*v**(-1)*omumv**(-1)
     .        - 9.0d0/4.0d0*HPL2(0,0)*v**(-1)
     .        + 5.0d0/6.0d0*HPL2(0,0)*omumv**(-2)
     .        - 25.0d0/12.0d0*HPL2(0,0)*omumv**(-1)
     .        + 11.0d0/2.0d0*HPL2(0,0)*pi*ci
     .        - 5.0d0/4.0d0*HPL2(0,0)*pi**2
     .        - 1.0d0/6.0d0*HPL2(0,0)*u*v**(-2)
     .        + 29.0d0/12.0d0*HPL2(0,0)*u*v**(-1)
     .        - 25.0d0/12.0d0*HPL2(0,0)*u
     .        + 7.0d0/6.0d0*HPL2(0,0)*u**2*v**(-2)
     .        - 15.0d0/4.0d0*HPL2(0,0)*u**2*v**(-1)
     .        + 5.0d0/6.0d0*HPL2(0,0)*u**2
     .        - 5.0d0/3.0d0*HPL2(0,0)*u**3*v**(-2)
     .        + 5.0d0/3.0d0*HPL2(0,0)*u**3*v**(-1)
     .        + 5.0d0/6.0d0*HPL2(0,0)*u**4*v**(-2)
         col1 = col1 + 121.0d0/12.0d0*HPL2(0,0)*GPL1(0)
     .        + 5.0d0/2.0d0*HPL2(0,0)*GPL1(0)*pi*ci
     .        - 11.0d0/2.0d0*HPL2(0,0)*GPL1(1)
     .        - HPL2(0,0)*GPL1(1)*pi*ci
     .        + 11.0d0/2.0d0*HPL2(0,0)*GPL1(2)
     .        + 1.0d0/2.0d0*HPL2(0,0)*GPL1(2)*pi*ci
     .        - 11.0d0/3.0d0*HPL2(0,0)*GPL1(3)
     .        - 3.0d0*HPL2(0,0)*GPL1(3)*pi*ci
     .        + 2.0d0*HPL2(0,0)*GPL2(0,0)
     .        + 1.0d0/2.0d0*HPL2(0,0)*GPL2(0,2)
     .        - 2.0d0*HPL2(0,0)*GPL2(0,3)
     .        - HPL2(0,0)*GPL2(1,2)
     .        + 1.0d0/2.0d0*HPL2(0,0)*GPL2(2,0)
     .        - 3.0d0*HPL2(0,0)*GPL2(3,0)
     .        + 2.0d0*HPL2(0,0)*GPL2(3,3)
         col1 = col1 + 605.0d0/144.0d0*HPL2(0,1)
     .        + 77.0d0/24.0d0*HPL2(0,1)*pi*ci
     .        - 1.0d0/2.0d0*HPL2(0,1)*pi**2
     .        + 77.0d0/24.0d0*HPL2(0,1)*GPL1(0)
     .        + HPL2(0,1)*GPL1(0)*pi*ci
     .        - 11.0d0/2.0d0*HPL2(0,1)*GPL1(1)
     .        - 1.0d0/2.0d0*HPL2(0,1)*GPL1(1)*pi*ci
     .        - 11.0d0/12.0d0*HPL2(0,1)*GPL1(2)
     .        - 1.0d0/2.0d0*HPL2(0,1)*GPL1(2)*pi*ci
     .        - 1.0d0/2.0d0*HPL2(0,1)*GPL1(3)*pi*ci
     .        + HPL2(0,1)*GPL2(0,0)
     .        - 1.0d0/2.0d0*HPL2(0,1)*GPL2(0,1)
     .        + 1.0d0/2.0d0*HPL2(0,1)*GPL2(0,2)
     .        - 1.0d0/2.0d0*HPL2(0,1)*GPL2(0,3)
     .        - 1.0d0/2.0d0*HPL2(0,1)*GPL2(1,0)
         col1 = col1 - 1.0d0/2.0d0*HPL2(0,1)*GPL2(2,0)
     .        + HPL2(0,1)*GPL2(2,1)
     .        - HPL2(0,1)*GPL2(2,2)
     .        + HPL2(0,1)*GPL2(2,3)
     .        - 1.0d0/2.0d0*HPL2(0,1)*GPL2(3,0)
     .        + 605.0d0/144.0d0*HPL2(1,0)
     .        - 5.0d0/6.0d0*HPL2(1,0)*v**(-2)*omumv**(-2)
     .        + 5.0d0/3.0d0*HPL2(1,0)*v**(-2)*omumv**(-1)
     .        - 5.0d0/6.0d0*HPL2(1,0)*v**(-2)
     .        + 5.0d0/3.0d0*HPL2(1,0)*v**(-1)*omumv**(-2)
     .        - 15.0d0/4.0d0*HPL2(1,0)*v**(-1)*omumv**(-1)
     .        + 25.0d0/12.0d0*HPL2(1,0)*v**(-1)
     .        - 5.0d0/6.0d0*HPL2(1,0)*omumv**(-2)
     .        + 25.0d0/12.0d0*HPL2(1,0)*omumv**(-1)
     .        + 55.0d0/24.0d0*HPL2(1,0)*pi*ci
         col1 = col1 - 1.0d0/2.0d0*HPL2(1,0)*pi**2
     .        - 1.0d0/3.0d0*HPL2(1,0)*u*v**(-2)
     .        + 9.0d0/4.0d0*HPL2(1,0)*u*v**(-1)
     .        - 25.0d0/12.0d0*HPL2(1,0)*u
     .        + HPL2(1,0)*u**2*v**(-2)
     .        - 15.0d0/4.0d0*HPL2(1,0)*u**2*v**(-1)
     .        + 5.0d0/6.0d0*HPL2(1,0)*u**2
     .        - 5.0d0/3.0d0*HPL2(1,0)*u**3*v**(-2)
     .        + 5.0d0/3.0d0*HPL2(1,0)*u**3*v**(-1)
     .        + 5.0d0/6.0d0*HPL2(1,0)*u**4*v**(-2)
     .        + 77.0d0/24.0d0*HPL2(1,0)*GPL1(0)
     .        + HPL2(1,0)*GPL1(0)*pi*ci
     .        - 11.0d0/3.0d0*HPL2(1,0)*GPL1(1)
     .        - 1.0d0/2.0d0*HPL2(1,0)*GPL1(1)*pi*ci
     .        + 11.0d0/3.0d0*HPL2(1,0)*GPL1(2)
         col1 = col1 - 1.0d0/2.0d0*HPL2(1,0)*GPL1(2)*pi*ci
     .        - 1.0d0/2.0d0*HPL2(1,0)*GPL1(3)*pi*ci
     .        + HPL2(1,0)*GPL2(0,0)
     .        - 1.0d0/2.0d0*HPL2(1,0)*GPL2(0,1)
     .        + 1.0d0/2.0d0*HPL2(1,0)*GPL2(0,2)
     .        - 1.0d0/2.0d0*HPL2(1,0)*GPL2(0,3)
     .        - 1.0d0/2.0d0*HPL2(1,0)*GPL2(1,0)
     .        + 2.0d0*HPL2(1,0)*GPL2(1,1)
     .        - 2.0d0*HPL2(1,0)*GPL2(1,2)
     .        - 1.0d0/2.0d0*HPL2(1,0)*GPL2(2,0)
     .        - HPL2(1,0)*GPL2(2,1)
     .        + HPL2(1,0)*GPL2(2,2)
     .        + HPL2(1,0)*GPL2(2,3)
     .        - 1.0d0/2.0d0*HPL2(1,0)*GPL2(3,0)
     .        + 121.0d0/48.0d0*HPL2(1,1)
         col1 = col1 + 11.0d0/6.0d0*HPL2(1,1)*pi*ci
     .        - 1.0d0/4.0d0*HPL2(1,1)*pi**2
     .        + 11.0d0/6.0d0*HPL2(1,1)*GPL1(0)
     .        + 1.0d0/2.0d0*HPL2(1,1)*GPL1(0)*pi*ci
     .        - 11.0d0/3.0d0*HPL2(1,1)*GPL1(1)
     .        - HPL2(1,1)*GPL1(1)*pi*ci
     .        + 1.0d0/2.0d0*HPL2(1,1)*GPL2(0,0)
     .        - HPL2(1,1)*GPL2(0,1)
     .        - HPL2(1,1)*GPL2(1,0)
     .        + 2.0d0*HPL2(1,1)*GPL2(1,1)
     .        - 121.0d0/12.0d0*HPL3(0,0,0)
     .        - 3.0d0*HPL3(0,0,0)*pi*ci
     .        - 3.0d0*HPL3(0,0,0)*GPL1(0)
     .        + HPL3(0,0,0)*GPL1(1)
     .        - HPL3(0,0,0)*GPL1(2)
         col1 = col1 + 3.0d0*HPL3(0,0,0)*GPL1(3)
     .        - 1.0d0/2.0d0*HPL3(0,0,1)*pi*ci
     .        - 1.0d0/2.0d0*HPL3(0,0,1)*GPL1(0)
     .        + HPL3(0,0,1)*GPL1(1)
     .        - 1.0d0/2.0d0*HPL3(0,1,0)*pi*ci
     .        - 1.0d0/2.0d0*HPL3(0,1,0)*GPL1(0)
     .        + HPL3(0,1,0)*GPL1(1)
     .        - 1.0d0/2.0d0*HPL3(1,0,0)*pi*ci
     .        - 1.0d0/2.0d0*HPL3(1,0,0)*GPL1(0)
     .        + HPL3(1,0,0)*GPL1(1)
     .        + 11.0d0/12.0d0*HPL3(1,0,1)
     .        - 11.0d0/6.0d0*HPL3(1,1,0)
     .        + 2.0d0*HPL3(1,1,0)*GPL1(1)
     .        - 2.0d0*HPL3(1,1,0)*GPL1(2)
     .        + 3.0d0*HPL4(0,0,0,0)
         col1 = col1 - 13.0d0/8.0d0*GPL1(0)
     .        - 5.0d0/6.0d0*GPL1(0)*v**(-2)*omumv**(-1)
     .        + 31.0d0/24.0d0*GPL1(0)*v**(-2)
     .        + 1.0d0/12.0d0*GPL1(0)*v**(-2)*pi*ci
     .        + 5.0d0/3.0d0*GPL1(0)*v**(-1)*omumv**(-1)
     .        - 31.0d0/24.0d0*GPL1(0)*v**(-1)
     .        - 1.0d0/12.0d0*GPL1(0)*v**(-1)*pi*ci
     .        - 5.0d0/6.0d0*GPL1(0)*omumv**(-1)
     .        - 389.0d0/144.0d0*GPL1(0)*pi*ci
     .        - 77.0d0/288.0d0*GPL1(0)*pi**2
     .        - 1.0d0/8.0d0*GPL1(0)*pi**3*ci
     .        - 1.0d0/4.0d0*GPL1(0)*zeta3
     .        + 3.0d0/8.0d0*GPL1(0)*u*v**(-2)
     .        - 1.0d0/4.0d0*GPL1(0)*u*v**(-2)*pi*ci
     .        - 145.0d0/72.0d0*GPL1(0)*u*v**(-1)
         col1 = col1 + 7.0d0/3.0d0*GPL1(0)*u*v**(-1)*pi*ci
     .        - 25.0d0/12.0d0*GPL1(0)*u*pi*ci
     .        - 79.0d0/72.0d0*GPL1(0)*u**2*v**(-2)
     .        + 13.0d0/12.0d0*GPL1(0)*u**2*v**(-2)*pi*ci
     .        + 5.0d0/6.0d0*GPL1(0)*u**2*v**(-1)
     .        - 15.0d0/4.0d0*GPL1(0)*u**2*v**(-1)*pi*ci
     .        + 5.0d0/6.0d0*GPL1(0)*u**2*pi*ci
     .        + 5.0d0/6.0d0*GPL1(0)*u**3*v**(-2)
     .        - 5.0d0/3.0d0*GPL1(0)*u**3*v**(-2)*pi*ci
     .        + 5.0d0/3.0d0*GPL1(0)*u**3*v**(-1)*pi*ci
     .        + 5.0d0/6.0d0*GPL1(0)*u**4*v**(-2)*pi*ci
     .        - 5.0d0/6.0d0*GPL1(1)*u**(-2)*v**(-2)*pi*ci
     .        + 5.0d0/3.0d0*GPL1(1)*u**(-2)*v**(-1)*pi*ci
     .        - 5.0d0/6.0d0*GPL1(1)*u**(-2)*pi*ci
     .        + 5.0d0/3.0d0*GPL1(1)*u**(-1)*v**(-2)*pi*ci
         col1 = col1 - 15.0d0/4.0d0*GPL1(1)*u**(-1)*v**(-1)*pi*ci
     .        + 25.0d0/12.0d0*GPL1(1)*u**(-1)*pi*ci
     .        - 7.0d0/6.0d0*GPL1(1)*v**(-2)*pi*ci
     .        + 29.0d0/12.0d0*GPL1(1)*v**(-1)*pi*ci
     .        + 67.0d0/18.0d0*GPL1(1)*pi*ci
     .        - 77.0d0/36.0d0*GPL1(1)*pi**2
     .        + 1.0d0/4.0d0*GPL1(1)*pi**3*ci
     .        - 2.0d0*GPL1(1)*zeta3
     .        + 1.0d0/2.0d0*GPL1(1)*u*v**(-2)*pi*ci
     .        - 29.0d0/12.0d0*GPL1(1)*u*v**(-1)*pi*ci
     .        + 25.0d0/12.0d0*GPL1(1)*u*pi*ci
     .        - 7.0d0/6.0d0*GPL1(1)*u**2*v**(-2)*pi*ci
     .        + 15.0d0/4.0d0*GPL1(1)*u**2*v**(-1)*pi*ci
     .        - 5.0d0/6.0d0*GPL1(1)*u**2*pi*ci
     .        + 5.0d0/3.0d0*GPL1(1)*u**3*v**(-2)*pi*ci
         col1 = col1 - 5.0d0/3.0d0*GPL1(1)*u**3*v**(-1)*pi*ci
     .        - 5.0d0/6.0d0*GPL1(1)*u**4*v**(-2)*pi*ci
     .        + 1.0d0/8.0d0*GPL1(2)
     .        - 5.0d0/6.0d0*GPL1(2)*u**(-1)*v**(-2)
     .        + 5.0d0/3.0d0*GPL1(2)*u**(-1)*v**(-1)
     .        - 5.0d0/6.0d0*GPL1(2)*u**(-1)
     .        + 101.0d0/72.0d0*GPL1(2)*v**(-2)
     .        + 1.0d0/12.0d0*GPL1(2)*v**(-2)*pi*ci
     .        - 227.0d0/72.0d0*GPL1(2)*v**(-1)
     .        - 1.0d0/12.0d0*GPL1(2)*v**(-1)*pi*ci
     .        - 389.0d0/144.0d0*GPL1(2)*pi*ci
     .        + 143.0d0/288.0d0*GPL1(2)*pi**2
     .        - 1.0d0/8.0d0*GPL1(2)*pi**3*ci
     .        + 7.0d0/4.0d0*GPL1(2)*zeta3
     .        - 49.0d0/72.0d0*GPL1(2)*u*v**(-2)
         col1 = col1 - 1.0d0/4.0d0*GPL1(2)*u*v**(-2)*pi*ci
     .        + 227.0d0/72.0d0*GPL1(2)*u*v**(-1)
     .        + 7.0d0/3.0d0*GPL1(2)*u*v**(-1)*pi*ci
     .        - 5.0d0/6.0d0*GPL1(2)*u
     .        - 25.0d0/12.0d0*GPL1(2)*u*pi*ci
     .        + 101.0d0/72.0d0*GPL1(2)*u**2*v**(-2)
     .        + 13.0d0/12.0d0*GPL1(2)*u**2*v**(-2)*pi*ci
     .        - 5.0d0/3.0d0*GPL1(2)*u**2*v**(-1)
     .        - 15.0d0/4.0d0*GPL1(2)*u**2*v**(-1)*pi*ci
     .        + 5.0d0/6.0d0*GPL1(2)*u**2*pi*ci
     .        - 5.0d0/6.0d0*GPL1(2)*u**3*v**(-2)
     .        - 5.0d0/3.0d0*GPL1(2)*u**3*v**(-2)*pi*ci
     .        + 5.0d0/3.0d0*GPL1(2)*u**3*v**(-1)*pi*ci
     .        + 5.0d0/6.0d0*GPL1(2)*u**4*v**(-2)*pi*ci
     .        - 5.0d0/6.0d0*GPL1(3)*v**(-2)*omumv**(-2)*pi*ci
         col1 = col1 + 5.0d0/3.0d0*GPL1(3)*v**(-2)*omumv**(-1)*pi*ci
     .        - GPL1(3)*v**(-2)*pi*ci
     .        + 5.0d0/3.0d0*GPL1(3)*v**(-1)*omumv**(-2)*pi*ci
     .        - 15.0d0/4.0d0*GPL1(3)*v**(-1)*omumv**(-1)*pi*ci
     .        + 9.0d0/4.0d0*GPL1(3)*v**(-1)*pi*ci
     .        - 5.0d0/6.0d0*GPL1(3)*omumv**(-2)*pi*ci
     .        + 25.0d0/12.0d0*GPL1(3)*omumv**(-1)*pi*ci
     .        + 67.0d0/18.0d0*GPL1(3)*pi*ci
     .        - 22.0d0/9.0d0*GPL1(3)*pi**2
     .        + 1.0d0/4.0d0*GPL1(3)*pi**3*ci
     .        + 1.0d0/6.0d0*GPL1(3)*u*v**(-2)*pi*ci
     .        - 29.0d0/12.0d0*GPL1(3)*u*v**(-1)*pi*ci
     .        + 25.0d0/12.0d0*GPL1(3)*u*pi*ci
     .        - 7.0d0/6.0d0*GPL1(3)*u**2*v**(-2)*pi*ci
     .        + 15.0d0/4.0d0*GPL1(3)*u**2*v**(-1)*pi*ci
         col1 = col1 - 5.0d0/6.0d0*GPL1(3)*u**2*pi*ci
     .        + 5.0d0/3.0d0*GPL1(3)*u**3*v**(-2)*pi*ci
     .        - 5.0d0/3.0d0*GPL1(3)*u**3*v**(-1)*pi*ci
     .        - 5.0d0/6.0d0*GPL1(3)*u**4*v**(-2)*pi*ci
     .        + 121.0d0/48.0d0*GPL2(0,0)
     .        + 11.0d0/6.0d0*GPL2(0,0)*pi*ci
     .        - 1.0d0/4.0d0*GPL2(0,0)*pi**2
     .        - 1.0d0/2.0d0*GPL2(0,1)*pi**2
     .        - 49.0d0/48.0d0*GPL2(0,2)
     .        + 1.0d0/12.0d0*GPL2(0,2)*v**(-2)
     .        - 1.0d0/12.0d0*GPL2(0,2)*v**(-1)
     .        + 11.0d0/8.0d0*GPL2(0,2)*pi*ci
     .        - 3.0d0/8.0d0*GPL2(0,2)*pi**2
     .        - 1.0d0/4.0d0*GPL2(0,2)*u*v**(-2)
     .        + 7.0d0/3.0d0*GPL2(0,2)*u*v**(-1)
         col1 = col1 - 25.0d0/12.0d0*GPL2(0,2)*u
     .        + 13.0d0/12.0d0*GPL2(0,2)*u**2*v**(-2)
     .        - 15.0d0/4.0d0*GPL2(0,2)*u**2*v**(-1)
     .        + 5.0d0/6.0d0*GPL2(0,2)*u**2
     .        - 5.0d0/3.0d0*GPL2(0,2)*u**3*v**(-2)
     .        + 5.0d0/3.0d0*GPL2(0,2)*u**3*v**(-1)
     .        + 5.0d0/6.0d0*GPL2(0,2)*u**4*v**(-2)
     .        - 55.0d0/12.0d0*GPL2(0,3)*pi*ci
     .        + 1.0d0/6.0d0*GPL2(0,3)*pi**2
     .        + 1.0d0/2.0d0*GPL2(1,0)*pi**2
     .        - 11.0d0/6.0d0*GPL2(1,1)*pi*ci
     .        - 1.0d0/3.0d0*GPL2(1,1)*pi**2
     .        + 67.0d0/18.0d0*GPL2(1,2)
     .        - 5.0d0/6.0d0*GPL2(1,2)*u**(-2)*v**(-2)
     .        + 5.0d0/3.0d0*GPL2(1,2)*u**(-2)*v**(-1)
         col1 = col1 - 5.0d0/6.0d0*GPL2(1,2)*u**(-2)
     .        + 5.0d0/3.0d0*GPL2(1,2)*u**(-1)*v**(-2)
     .        - 15.0d0/4.0d0*GPL2(1,2)*u**(-1)*v**(-1)
     .        + 25.0d0/12.0d0*GPL2(1,2)*u**(-1)
     .        - 7.0d0/6.0d0*GPL2(1,2)*v**(-2)
     .        + 29.0d0/12.0d0*GPL2(1,2)*v**(-1)
     .        + 11.0d0/12.0d0*GPL2(1,2)*pi*ci
     .        + 5.0d0/12.0d0*GPL2(1,2)*pi**2
     .        + 1.0d0/2.0d0*GPL2(1,2)*u*v**(-2)
     .        - 29.0d0/12.0d0*GPL2(1,2)*u*v**(-1)
     .        + 25.0d0/12.0d0*GPL2(1,2)*u
     .        - 7.0d0/6.0d0*GPL2(1,2)*u**2*v**(-2)
     .        + 15.0d0/4.0d0*GPL2(1,2)*u**2*v**(-1)
     .        - 5.0d0/6.0d0*GPL2(1,2)*u**2
     .        + 5.0d0/3.0d0*GPL2(1,2)*u**3*v**(-2)
         col1 = col1 - 5.0d0/3.0d0*GPL2(1,2)*u**3*v**(-1)
     .        - 5.0d0/6.0d0*GPL2(1,2)*u**4*v**(-2)
     .        - 49.0d0/48.0d0*GPL2(2,0)
     .        + 1.0d0/12.0d0*GPL2(2,0)*v**(-2)
     .        - 1.0d0/12.0d0*GPL2(2,0)*v**(-1)
     .        + 11.0d0/8.0d0*GPL2(2,0)*pi*ci
     .        - 3.0d0/8.0d0*GPL2(2,0)*pi**2
     .        - 1.0d0/4.0d0*GPL2(2,0)*u*v**(-2)
     .        + 7.0d0/3.0d0*GPL2(2,0)*u*v**(-1)
     .        - 25.0d0/12.0d0*GPL2(2,0)*u
     .        + 13.0d0/12.0d0*GPL2(2,0)*u**2*v**(-2)
     .        - 15.0d0/4.0d0*GPL2(2,0)*u**2*v**(-1)
     .        + 5.0d0/6.0d0*GPL2(2,0)*u**2
     .        - 5.0d0/3.0d0*GPL2(2,0)*u**3*v**(-2)
     .        + 5.0d0/3.0d0*GPL2(2,0)*u**3*v**(-1)
         col1 = col1 + 5.0d0/6.0d0*GPL2(2,0)*u**4*v**(-2)
     .        - 55.0d0/12.0d0*GPL2(2,1)*pi*ci
     .        - 1.0d0/6.0d0*GPL2(2,1)*pi**2
     .        + 121.0d0/48.0d0*GPL2(2,2)
     .        + 11.0d0/6.0d0*GPL2(2,2)*pi*ci
     .        + 1.0d0/12.0d0*GPL2(2,2)*pi**2
     .        - 1.0d0/2.0d0*GPL2(2,3)*pi**2
     .        + 67.0d0/18.0d0*GPL2(3,0)
     .        - 5.0d0/6.0d0*GPL2(3,0)*v**(-2)*omumv**(-2)
     .        + 5.0d0/3.0d0*GPL2(3,0)*v**(-2)*omumv**(-1)
     .        - GPL2(3,0)*v**(-2)
     .        + 5.0d0/3.0d0*GPL2(3,0)*v**(-1)*omumv**(-2)
     .        - 15.0d0/4.0d0*GPL2(3,0)*v**(-1)*omumv**(-1)
     .        + 9.0d0/4.0d0*GPL2(3,0)*v**(-1)
     .        - 5.0d0/6.0d0*GPL2(3,0)*omumv**(-2)
         col1 = col1 + 25.0d0/12.0d0*GPL2(3,0)*omumv**(-1)
     .        + 11.0d0/12.0d0*GPL2(3,0)*pi*ci
     .        + 3.0d0/4.0d0*GPL2(3,0)*pi**2
     .        + 1.0d0/6.0d0*GPL2(3,0)*u*v**(-2)
     .        - 29.0d0/12.0d0*GPL2(3,0)*u*v**(-1)
     .        + 25.0d0/12.0d0*GPL2(3,0)*u
     .        - 7.0d0/6.0d0*GPL2(3,0)*u**2*v**(-2)
     .        + 15.0d0/4.0d0*GPL2(3,0)*u**2*v**(-1)
     .        - 5.0d0/6.0d0*GPL2(3,0)*u**2
     .        + 5.0d0/3.0d0*GPL2(3,0)*u**3*v**(-2)
     .        - 5.0d0/3.0d0*GPL2(3,0)*u**3*v**(-1)
     .        - 5.0d0/6.0d0*GPL2(3,0)*u**4*v**(-2)
     .        + 1.0d0/2.0d0*GPL2(3,2)*pi**2
     .        - 11.0d0/6.0d0*GPL2(3,3)*pi*ci
     .        - 2.0d0/3.0d0*GPL2(3,3)*pi**2
         col1 = col1 + GPL3(0,0,1)*pi*ci
     .        + 11.0d0/6.0d0*GPL3(0,0,2)
     .        + 1.0d0/2.0d0*GPL3(0,0,2)*pi*ci
     .        - GPL3(0,0,3)*pi*ci
     .        + GPL3(0,1,0)*pi*ci
     .        + 11.0d0/6.0d0*GPL3(0,2,0)
     .        + 1.0d0/2.0d0*GPL3(0,2,0)*pi*ci
     .        - 1.0d0/2.0d0*GPL3(0,2,1)*pi*ci
     .        + 11.0d0/6.0d0*GPL3(0,2,2)
     .        + 1.0d0/2.0d0*GPL3(0,2,2)*pi*ci
     .        - 1.0d0/2.0d0*GPL3(0,2,3)*pi*ci
     .        - 55.0d0/12.0d0*GPL3(0,3,0)
     .        - 1.0d0/2.0d0*GPL3(0,3,0)*pi*ci
     .        - 1.0d0/2.0d0*GPL3(0,3,2)*pi*ci
     .        + 2.0d0*GPL3(0,3,3)*pi*ci
         col1 = col1 - GPL3(1,0,2)*pi*ci
     .        - 2.0d0*GPL3(1,1,1)*pi*ci
     .        - 11.0d0/6.0d0*GPL3(1,1,2)
     .        + 2.0d0*GPL3(1,1,2)*pi*ci
     .        - GPL3(1,2,0)*pi*ci
     .        + 2.0d0*GPL3(1,2,1)*pi*ci
     .        - 11.0d0/3.0d0*GPL3(1,2,2)
     .        - GPL3(1,2,2)*pi*ci
     .        + 11.0d0/6.0d0*GPL3(2,0,0)
     .        + 1.0d0/2.0d0*GPL3(2,0,0)*pi*ci
     .        - 1.0d0/2.0d0*GPL3(2,0,1)*pi*ci
     .        + 11.0d0/6.0d0*GPL3(2,0,2)
     .        + 1.0d0/2.0d0*GPL3(2,0,2)*pi*ci
     .        - 1.0d0/2.0d0*GPL3(2,0,3)*pi*ci
     .        - 1.0d0/2.0d0*GPL3(2,1,0)*pi*ci
         col1 = col1 + 2.0d0*GPL3(2,1,1)*pi*ci
     .        - 55.0d0/12.0d0*GPL3(2,1,2)
     .        - 1.0d0/2.0d0*GPL3(2,1,2)*pi*ci
     .        + 11.0d0/6.0d0*GPL3(2,2,0)
     .        + 1.0d0/2.0d0*GPL3(2,2,0)*pi*ci
     .        - GPL3(2,2,1)*pi*ci
     .        + GPL3(2,2,3)*pi*ci
     .        + GPL3(2,3,2)*pi*ci
     .        - 11.0d0/3.0d0*GPL3(3,0,0)
     .        - GPL3(3,0,0)*pi*ci
     .        - GPL3(3,0,2)*pi*ci
     .        + 2.0d0*GPL3(3,0,3)*pi*ci
     .        - GPL3(3,2,0)*pi*ci
     .        - 11.0d0/6.0d0*GPL3(3,3,0)
     .        + 2.0d0*GPL3(3,3,0)*pi*ci
         col1 = col1 - 2.0d0*GPL3(3,3,3)*pi*ci
     .        + GPL4(0,0,1,2)
     .        + 1.0d0/2.0d0*GPL4(0,0,2,2)
     .        - GPL4(0,0,3,0)
     .        + GPL4(0,1,0,2)
     .        + GPL4(0,1,2,0)
     .        - GPL4(0,1,2,2)
     .        + 1.0d0/2.0d0*GPL4(0,2,0,2)
     .        - 1.0d0/2.0d0*GPL4(0,2,1,2)
     .        + 1.0d0/2.0d0*GPL4(0,2,2,0)
     .        - 1.0d0/2.0d0*GPL4(0,2,3,0)
     .        - 1.0d0/2.0d0*GPL4(0,3,0,2)
     .        - 1.0d0/2.0d0*GPL4(0,3,2,0)
     .        + 2.0d0*GPL4(0,3,3,0)
     .        - GPL4(1,0,2,2)
         col1 = col1 - 2.0d0*GPL4(1,1,1,2)
     .        + 2.0d0*GPL4(1,1,2,2)
     .        - GPL4(1,2,0,2)
     .        + 2.0d0*GPL4(1,2,1,2)
     .        - GPL4(1,2,2,0)
     .        + 1.0d0/2.0d0*GPL4(2,0,0,2)
     .        - 1.0d0/2.0d0*GPL4(2,0,1,2)
     .        + 1.0d0/2.0d0*GPL4(2,0,2,0)
     .        - 1.0d0/2.0d0*GPL4(2,0,3,0)
     .        - 1.0d0/2.0d0*GPL4(2,1,0,2)
     .        + 2.0d0*GPL4(2,1,1,2)
     .        - 1.0d0/2.0d0*GPL4(2,1,2,0)
     .        + 1.0d0/2.0d0*GPL4(2,2,0,0)
     .        - GPL4(2,2,1,2)
     .        + GPL4(2,2,3,0)
         col1 = col1 - GPL4(2,3,0,0)
     .        + GPL4(2,3,0,2)
     .        + GPL4(2,3,2,0)
     .        - GPL4(3,0,0,2)
     .        - GPL4(3,0,2,0)
     .        + 2.0d0*GPL4(3,0,3,0)
     .        - GPL4(3,2,0,0)
     .        + 2.0d0*GPL4(3,3,0,0)
     .        - 2.0d0*GPL4(3,3,3,0)

         col2 = (0.d0,0.d0)

         col3 = (0.d0,0.d0)

         col4 = + 67.0d0/48.0d0
     .        + 1.0d0/12.0d0*u**(-1)*v**(-2)*pi*ci
     .        - 1.0d0/6.0d0*u**(-1)*v**(-1)*pi*ci
     .        + 1.0d0/12.0d0*u**(-1)*pi*ci
     .        + 1.0d0/72.0d0*v**(-2)*omumv**(-2)*pi**2
     .        + 1.0d0/12.0d0*v**(-2)*omumv**(-1)*pi*ci
     .        - 1.0d0/36.0d0*v**(-2)*omumv**(-1)*pi**2
     .        + 1.0d0/6.0d0*v**(-2)
     .        - 5.0d0/24.0d0*v**(-2)*pi*ci
     .        + 1.0d0/72.0d0*v**(-2)*pi**2
     .        - 1.0d0/36.0d0*v**(-1)*omumv**(-2)*pi**2
     .        - 1.0d0/6.0d0*v**(-1)*omumv**(-1)*pi*ci
     .        - 1.0d0/6.0d0*v**(-1)
     .        + 1.0d0/36.0d0*v**(-1)*pi**2
     .        + 1.0d0/72.0d0*omumv**(-2)*pi**2
         col4 = col4 + 1.0d0/12.0d0*omumv**(-1)*pi*ci
     .        + 1.0d0/36.0d0*omumv**(-1)*pi**2
     .        + 11.0d0/24.0d0*pi*ci
     .        - zeta3
     .        - 1.0d0/6.0d0*u*v**(-2)
     .        + 1.0d0/12.0d0*u*v**(-2)*pi**2
     .        + 1.0d0/6.0d0*u*v**(-1)
     .        - 1.0d0/12.0d0*u*v**(-1)*pi*ci
     .        - 2.0d0/9.0d0*u*v**(-1)*pi**2
     .        + 1.0d0/12.0d0*u*pi*ci
     .        + 1.0d0/9.0d0*u*pi**2
     .        + 1.0d0/6.0d0*u**2*v**(-2)
     .        - 1.0d0/18.0d0*u**2*v**(-2)*pi**2
     .        + 1.0d0/12.0d0*u**2*v**(-1)*pi*ci
     .        + 1.0d0/18.0d0*u**2*pi**2
         col4 = col4 - 1.0d0/9.0d0*u**3*v**(-2)*pi**2
     .        + 1.0d0/9.0d0*u**3*v**(-1)*pi**2
     .        + 1.0d0/18.0d0*u**4*v**(-2)*pi**2
     .        - 5.0d0/24.0d0*HPL1(0)
     .        + 1.0d0/12.0d0*HPL1(0)*u**(-1)*v**(-1)
     .        - 1.0d0/12.0d0*HPL1(0)*u**(-1)
     .        + 1.0d0/12.0d0*HPL1(0)*v**(-2)*omumv**(-2)*pi*ci
     .        - 1.0d0/6.0d0*HPL1(0)*v**(-2)*omumv**(-1)*pi*ci
     .        + 1.0d0/12.0d0*HPL1(0)*v**(-2)*pi*ci
     .        - 1.0d0/6.0d0*HPL1(0)*v**(-1)*omumv**(-2)*pi*ci
     .        + 1.0d0/12.0d0*HPL1(0)*v**(-1)*omumv**(-1)
     .        - 1.0d0/6.0d0*HPL1(0)*v**(-1)
     .        + 1.0d0/6.0d0*HPL1(0)*v**(-1)*pi*ci
     .        + 1.0d0/12.0d0*HPL1(0)*omumv**(-2)*pi*ci
     .        - 1.0d0/12.0d0*HPL1(0)*omumv**(-1)
         col4 = col4 + 1.0d0/6.0d0*HPL1(0)*omumv**(-1)*pi*ci
     .        + 1.0d0/12.0d0*HPL1(0)*u*v**(-1)
     .        - 1.0d0/3.0d0*HPL1(0)*u*v**(-1)*pi*ci
     .        - 1.0d0/12.0d0*HPL1(0)*u
     .        + 1.0d0/6.0d0*HPL1(0)*u*pi*ci
     .        - 1.0d0/12.0d0*HPL1(0)*u**2*v**(-2)*pi*ci
     .        - 1.0d0/12.0d0*HPL1(0)*u**2*v**(-1)
     .        + 1.0d0/12.0d0*HPL1(0)*u**2*pi*ci
     .        - 1.0d0/6.0d0*HPL1(0)*u**3*v**(-2)*pi*ci
     .        + 1.0d0/6.0d0*HPL1(0)*u**3*v**(-1)*pi*ci
     .        + 1.0d0/12.0d0*HPL1(0)*u**4*v**(-2)*pi*ci
     .        + 1.0d0/12.0d0*HPL1(0)*GPL1(0)*v**(-2)*omumv**(-2)
     .        - 1.0d0/6.0d0*HPL1(0)*GPL1(0)*v**(-2)*omumv**(-1)
     .        + 1.0d0/12.0d0*HPL1(0)*GPL1(0)*v**(-2)
     .        - 1.0d0/6.0d0*HPL1(0)*GPL1(0)*v**(-1)*omumv**(-2)
         col4 = col4 + 1.0d0/6.0d0*HPL1(0)*GPL1(0)*v**(-1)
     .        + 1.0d0/12.0d0*HPL1(0)*GPL1(0)*omumv**(-2)
     .        + 1.0d0/6.0d0*HPL1(0)*GPL1(0)*omumv**(-1)
     .        - 1.0d0/3.0d0*HPL1(0)*GPL1(0)*u*v**(-1)
     .        + 1.0d0/6.0d0*HPL1(0)*GPL1(0)*u
     .        - 1.0d0/12.0d0*HPL1(0)*GPL1(0)*u**2*v**(-2)
     .        + 1.0d0/12.0d0*HPL1(0)*GPL1(0)*u**2
     .        - 1.0d0/6.0d0*HPL1(0)*GPL1(0)*u**3*v**(-2)
     .        + 1.0d0/6.0d0*HPL1(0)*GPL1(0)*u**3*v**(-1)
     .        + 1.0d0/12.0d0*HPL1(0)*GPL1(0)*u**4*v**(-2)
     .        - 1.0d0/12.0d0*HPL1(0)*GPL1(1)*u**(-2)*v**(-2)
     .        + 1.0d0/6.0d0*HPL1(0)*GPL1(1)*u**(-2)*v**(-1)
     .        - 1.0d0/12.0d0*HPL1(0)*GPL1(1)*u**(-2)
     .        + 1.0d0/6.0d0*HPL1(0)*GPL1(1)*u**(-1)*v**(-2)
     .        - 1.0d0/6.0d0*HPL1(0)*GPL1(1)*u**(-1)
         col4 = col4 + 1.0d0/12.0d0*HPL1(0)*GPL1(1)*v**(-2)
     .        - 1.0d0/3.0d0*HPL1(0)*GPL1(1)*v**(-1)
     .        - 1.0d0/3.0d0*HPL1(0)*GPL1(1)*u*v**(-2)
     .        + 1.0d0/3.0d0*HPL1(0)*GPL1(1)*u*v**(-1)
     .        - 1.0d0/6.0d0*HPL1(0)*GPL1(1)*u
     .        + 1.0d0/12.0d0*HPL1(0)*GPL1(1)*u**2*v**(-2)
     .        - 1.0d0/12.0d0*HPL1(0)*GPL1(1)*u**2
     .        + 1.0d0/6.0d0*HPL1(0)*GPL1(1)*u**3*v**(-2)
     .        - 1.0d0/6.0d0*HPL1(0)*GPL1(1)*u**3*v**(-1)
     .        - 1.0d0/12.0d0*HPL1(0)*GPL1(1)*u**4*v**(-2)
     .        + 1.0d0/12.0d0*HPL1(0)*GPL1(2)*u**(-2)*v**(-2)
     .        - 1.0d0/6.0d0*HPL1(0)*GPL1(2)*u**(-2)*v**(-1)
     .        + 1.0d0/12.0d0*HPL1(0)*GPL1(2)*u**(-2)
     .        - 1.0d0/6.0d0*HPL1(0)*GPL1(2)*u**(-1)*v**(-2)
     .        + 1.0d0/6.0d0*HPL1(0)*GPL1(2)*u**(-1)
         col4 = col4 - 1.0d0/12.0d0*HPL1(0)*GPL1(2)*v**(-2)
     .        + 1.0d0/3.0d0*HPL1(0)*GPL1(2)*v**(-1)
     .        + 1.0d0/3.0d0*HPL1(0)*GPL1(2)*u*v**(-2)
     .        - 1.0d0/3.0d0*HPL1(0)*GPL1(2)*u*v**(-1)
     .        + 1.0d0/6.0d0*HPL1(0)*GPL1(2)*u
     .        - 1.0d0/12.0d0*HPL1(0)*GPL1(2)*u**2*v**(-2)
     .        + 1.0d0/12.0d0*HPL1(0)*GPL1(2)*u**2
     .        - 1.0d0/6.0d0*HPL1(0)*GPL1(2)*u**3*v**(-2)
     .        + 1.0d0/6.0d0*HPL1(0)*GPL1(2)*u**3*v**(-1)
     .        + 1.0d0/12.0d0*HPL1(0)*GPL1(2)*u**4*v**(-2)
     .        - 1.0d0/12.0d0*HPL1(0)*GPL1(3)*v**(-2)*omumv**(-2)
     .        + 1.0d0/6.0d0*HPL1(0)*GPL1(3)*v**(-2)*omumv**(-1)
     .        - 1.0d0/12.0d0*HPL1(0)*GPL1(3)*v**(-2)
     .        + 1.0d0/6.0d0*HPL1(0)*GPL1(3)*v**(-1)*omumv**(-2)
     .        - 1.0d0/6.0d0*HPL1(0)*GPL1(3)*v**(-1)
         col4 = col4 - 1.0d0/12.0d0*HPL1(0)*GPL1(3)*omumv**(-2)
     .        - 1.0d0/6.0d0*HPL1(0)*GPL1(3)*omumv**(-1)
     .        + 1.0d0/3.0d0*HPL1(0)*GPL1(3)*u*v**(-1)
     .        - 1.0d0/6.0d0*HPL1(0)*GPL1(3)*u
     .        + 1.0d0/12.0d0*HPL1(0)*GPL1(3)*u**2*v**(-2)
     .        - 1.0d0/12.0d0*HPL1(0)*GPL1(3)*u**2
     .        + 1.0d0/6.0d0*HPL1(0)*GPL1(3)*u**3*v**(-2)
     .        - 1.0d0/6.0d0*HPL1(0)*GPL1(3)*u**3*v**(-1)
     .        - 1.0d0/12.0d0*HPL1(0)*GPL1(3)*u**4*v**(-2)
     .        - 5.0d0/24.0d0*HPL1(1)
     .        - 1.0d0/12.0d0*HPL1(1)*u**(-1)*v**(-2)
     .        + 1.0d0/6.0d0*HPL1(1)*u**(-1)*v**(-1)
     .        - 1.0d0/12.0d0*HPL1(1)*u**(-1)
     .        + 1.0d0/8.0d0*HPL1(1)*v**(-2)
     .        + 1.0d0/12.0d0*HPL1(1)*v**(-1)
         col4 = col4 - 1.0d0/12.0d0*HPL1(1)*u*v**(-2)
     .        + 1.0d0/6.0d0*HPL1(1)*u*v**(-2)*pi*ci
     .        - 1.0d0/12.0d0*HPL1(1)*u*v**(-1)
     .        - 1.0d0/3.0d0*HPL1(1)*u*v**(-1)*pi*ci
     .        - 1.0d0/12.0d0*HPL1(1)*u
     .        + 1.0d0/6.0d0*HPL1(1)*u*pi*ci
     .        + 1.0d0/8.0d0*HPL1(1)*u**2*v**(-2)
     .        - 1.0d0/12.0d0*HPL1(1)*u**2*v**(-2)*pi*ci
     .        - 1.0d0/6.0d0*HPL1(1)*u**2*v**(-1)
     .        + 1.0d0/12.0d0*HPL1(1)*u**2*pi*ci
     .        - 1.0d0/12.0d0*HPL1(1)*u**3*v**(-2)
     .        - 1.0d0/6.0d0*HPL1(1)*u**3*v**(-2)*pi*ci
     .        + 1.0d0/6.0d0*HPL1(1)*u**3*v**(-1)*pi*ci
     .        + 1.0d0/12.0d0*HPL1(1)*u**4*v**(-2)*pi*ci
     .        + 1.0d0/6.0d0*HPL1(1)*GPL1(0)*u*v**(-2)
         col4 = col4 - 1.0d0/3.0d0*HPL1(1)*GPL1(0)*u*v**(-1)
     .        + 1.0d0/6.0d0*HPL1(1)*GPL1(0)*u
     .        - 1.0d0/12.0d0*HPL1(1)*GPL1(0)*u**2*v**(-2)
     .        + 1.0d0/12.0d0*HPL1(1)*GPL1(0)*u**2
     .        - 1.0d0/6.0d0*HPL1(1)*GPL1(0)*u**3*v**(-2)
     .        + 1.0d0/6.0d0*HPL1(1)*GPL1(0)*u**3*v**(-1)
     .        + 1.0d0/12.0d0*HPL1(1)*GPL1(0)*u**4*v**(-2)
     .        - 1.0d0/12.0d0*HPL1(1)*GPL1(1)*u**(-2)*v**(-2)
     .        + 1.0d0/6.0d0*HPL1(1)*GPL1(1)*u**(-2)*v**(-1)
     .        - 1.0d0/12.0d0*HPL1(1)*GPL1(1)*u**(-2)
     .        + 1.0d0/6.0d0*HPL1(1)*GPL1(1)*u**(-1)*v**(-2)
     .        - 1.0d0/6.0d0*HPL1(1)*GPL1(1)*u**(-1)
     .        + 1.0d0/12.0d0*HPL1(1)*GPL1(1)*v**(-2)
     .        - 1.0d0/3.0d0*HPL1(1)*GPL1(1)*v**(-1)
     .        - 1.0d0/3.0d0*HPL1(1)*GPL1(1)*u*v**(-2)
         col4 = col4 + 1.0d0/3.0d0*HPL1(1)*GPL1(1)*u*v**(-1)
     .        - 1.0d0/6.0d0*HPL1(1)*GPL1(1)*u
     .        + 1.0d0/12.0d0*HPL1(1)*GPL1(1)*u**2*v**(-2)
     .        - 1.0d0/12.0d0*HPL1(1)*GPL1(1)*u**2
     .        + 1.0d0/6.0d0*HPL1(1)*GPL1(1)*u**3*v**(-2)
     .        - 1.0d0/6.0d0*HPL1(1)*GPL1(1)*u**3*v**(-1)
     .        - 1.0d0/12.0d0*HPL1(1)*GPL1(1)*u**4*v**(-2)
     .        - 1.0d0/12.0d0*HPL2(0,0)*v**(-2)*omumv**(-2)
     .        + 1.0d0/6.0d0*HPL2(0,0)*v**(-2)*omumv**(-1)
     .        - 1.0d0/12.0d0*HPL2(0,0)*v**(-2)
     .        + 1.0d0/6.0d0*HPL2(0,0)*v**(-1)*omumv**(-2)
     .        - 1.0d0/6.0d0*HPL2(0,0)*v**(-1)
     .        - 1.0d0/12.0d0*HPL2(0,0)*omumv**(-2)
     .        - 1.0d0/6.0d0*HPL2(0,0)*omumv**(-1)
     .        + 1.0d0/3.0d0*HPL2(0,0)*u*v**(-1)
         col4 = col4 - 1.0d0/6.0d0*HPL2(0,0)*u
     .        + 1.0d0/12.0d0*HPL2(0,0)*u**2*v**(-2)
     .        - 1.0d0/12.0d0*HPL2(0,0)*u**2
     .        + 1.0d0/6.0d0*HPL2(0,0)*u**3*v**(-2)
     .        - 1.0d0/6.0d0*HPL2(0,0)*u**3*v**(-1)
     .        - 1.0d0/12.0d0*HPL2(0,0)*u**4*v**(-2)
     .        + 1.0d0/12.0d0*HPL2(1,0)*v**(-2)*omumv**(-2)
     .        - 1.0d0/6.0d0*HPL2(1,0)*v**(-2)*omumv**(-1)
     .        + 1.0d0/12.0d0*HPL2(1,0)*v**(-2)
     .        - 1.0d0/6.0d0*HPL2(1,0)*v**(-1)*omumv**(-2)
     .        + 1.0d0/6.0d0*HPL2(1,0)*v**(-1)
     .        + 1.0d0/12.0d0*HPL2(1,0)*omumv**(-2)
     .        + 1.0d0/6.0d0*HPL2(1,0)*omumv**(-1)
     .        - 1.0d0/3.0d0*HPL2(1,0)*u*v**(-2)
     .        + 1.0d0/3.0d0*HPL2(1,0)*u*v**(-1)
         col4 = col4 - 1.0d0/6.0d0*HPL2(1,0)*u
     .        + 1.0d0/12.0d0*HPL2(1,0)*u**2*v**(-2)
     .        - 1.0d0/12.0d0*HPL2(1,0)*u**2
     .        + 1.0d0/6.0d0*HPL2(1,0)*u**3*v**(-2)
     .        - 1.0d0/6.0d0*HPL2(1,0)*u**3*v**(-1)
     .        - 1.0d0/12.0d0*HPL2(1,0)*u**4*v**(-2)
     .        + 1.0d0/12.0d0*GPL1(0)*v**(-2)*omumv**(-1)
     .        - 1.0d0/12.0d0*GPL1(0)*v**(-2)
     .        - 1.0d0/6.0d0*GPL1(0)*v**(-1)*omumv**(-1)
     .        + 1.0d0/12.0d0*GPL1(0)*v**(-1)
     .        + 1.0d0/12.0d0*GPL1(0)*omumv**(-1)
     .        - 1.0d0/12.0d0*GPL1(0)*u*v**(-2)
     .        - 1.0d0/6.0d0*GPL1(0)*u*v**(-2)*pi*ci
     .        - 1.0d0/6.0d0*GPL1(0)*u*v**(-1)
     .        + 1.0d0/3.0d0*GPL1(0)*u*v**(-1)*pi*ci
         col4 = col4 - 1.0d0/6.0d0*GPL1(0)*u*pi*ci
     .        + 1.0d0/8.0d0*GPL1(0)*u**2*v**(-2)
     .        + 1.0d0/12.0d0*GPL1(0)*u**2*v**(-2)*pi*ci
     .        - 1.0d0/12.0d0*GPL1(0)*u**2*v**(-1)
     .        - 1.0d0/12.0d0*GPL1(0)*u**2*pi*ci
     .        - 1.0d0/12.0d0*GPL1(0)*u**3*v**(-2)
     .        + 1.0d0/6.0d0*GPL1(0)*u**3*v**(-2)*pi*ci
     .        - 1.0d0/6.0d0*GPL1(0)*u**3*v**(-1)*pi*ci
     .        - 1.0d0/12.0d0*GPL1(0)*u**4*v**(-2)*pi*ci
     .        + 1.0d0/12.0d0*GPL1(1)*u**(-2)*v**(-2)*pi*ci
     .        - 1.0d0/6.0d0*GPL1(1)*u**(-2)*v**(-1)*pi*ci
     .        + 1.0d0/12.0d0*GPL1(1)*u**(-2)*pi*ci
     .        - 1.0d0/6.0d0*GPL1(1)*u**(-1)*v**(-2)*pi*ci
     .        + 1.0d0/6.0d0*GPL1(1)*u**(-1)*pi*ci
     .        - 1.0d0/12.0d0*GPL1(1)*v**(-2)*pi*ci
         col4 = col4 + 1.0d0/3.0d0*GPL1(1)*v**(-1)*pi*ci
     .        + 1.0d0/3.0d0*GPL1(1)*u*v**(-2)*pi*ci
     .        - 1.0d0/3.0d0*GPL1(1)*u*v**(-1)*pi*ci
     .        + 1.0d0/6.0d0*GPL1(1)*u*pi*ci
     .        - 1.0d0/12.0d0*GPL1(1)*u**2*v**(-2)*pi*ci
     .        + 1.0d0/12.0d0*GPL1(1)*u**2*pi*ci
     .        - 1.0d0/6.0d0*GPL1(1)*u**3*v**(-2)*pi*ci
     .        + 1.0d0/6.0d0*GPL1(1)*u**3*v**(-1)*pi*ci
     .        + 1.0d0/12.0d0*GPL1(1)*u**4*v**(-2)*pi*ci
     .        + 5.0d0/24.0d0*GPL1(2)
     .        + 1.0d0/12.0d0*GPL1(2)*u**(-1)*v**(-2)
     .        - 1.0d0/6.0d0*GPL1(2)*u**(-1)*v**(-1)
     .        + 1.0d0/12.0d0*GPL1(2)*u**(-1)
     .        - 1.0d0/8.0d0*GPL1(2)*v**(-2)
     .        - 1.0d0/12.0d0*GPL1(2)*v**(-1)
         col4 = col4 + 1.0d0/12.0d0*GPL1(2)*u*v**(-2)
     .        - 1.0d0/6.0d0*GPL1(2)*u*v**(-2)*pi*ci
     .        + 1.0d0/12.0d0*GPL1(2)*u*v**(-1)
     .        + 1.0d0/3.0d0*GPL1(2)*u*v**(-1)*pi*ci
     .        + 1.0d0/12.0d0*GPL1(2)*u
     .        - 1.0d0/6.0d0*GPL1(2)*u*pi*ci
     .        - 1.0d0/8.0d0*GPL1(2)*u**2*v**(-2)
     .        + 1.0d0/12.0d0*GPL1(2)*u**2*v**(-2)*pi*ci
     .        + 1.0d0/6.0d0*GPL1(2)*u**2*v**(-1)
     .        - 1.0d0/12.0d0*GPL1(2)*u**2*pi*ci
     .        + 1.0d0/12.0d0*GPL1(2)*u**3*v**(-2)
     .        + 1.0d0/6.0d0*GPL1(2)*u**3*v**(-2)*pi*ci
     .        - 1.0d0/6.0d0*GPL1(2)*u**3*v**(-1)*pi*ci
     .        - 1.0d0/12.0d0*GPL1(2)*u**4*v**(-2)*pi*ci
     .        + 1.0d0/12.0d0*GPL1(3)*v**(-2)*omumv**(-2)*pi*ci
         col4 = col4 - 1.0d0/6.0d0*GPL1(3)*v**(-2)*omumv**(-1)*pi*ci
     .        + 1.0d0/12.0d0*GPL1(3)*v**(-2)*pi*ci
     .        - 1.0d0/6.0d0*GPL1(3)*v**(-1)*omumv**(-2)*pi*ci
     .        + 1.0d0/6.0d0*GPL1(3)*v**(-1)*pi*ci
     .        + 1.0d0/12.0d0*GPL1(3)*omumv**(-2)*pi*ci
     .        + 1.0d0/6.0d0*GPL1(3)*omumv**(-1)*pi*ci
     .        - 1.0d0/3.0d0*GPL1(3)*u*v**(-1)*pi*ci
     .        + 1.0d0/6.0d0*GPL1(3)*u*pi*ci
     .        - 1.0d0/12.0d0*GPL1(3)*u**2*v**(-2)*pi*ci
     .        + 1.0d0/12.0d0*GPL1(3)*u**2*pi*ci
     .        - 1.0d0/6.0d0*GPL1(3)*u**3*v**(-2)*pi*ci
     .        + 1.0d0/6.0d0*GPL1(3)*u**3*v**(-1)*pi*ci
     .        + 1.0d0/12.0d0*GPL1(3)*u**4*v**(-2)*pi*ci
     .        - 1.0d0/6.0d0*GPL2(0,2)*u*v**(-2)
     .        + 1.0d0/3.0d0*GPL2(0,2)*u*v**(-1)
         col4 = col4 - 1.0d0/6.0d0*GPL2(0,2)*u
     .        + 1.0d0/12.0d0*GPL2(0,2)*u**2*v**(-2)
     .        - 1.0d0/12.0d0*GPL2(0,2)*u**2
     .        + 1.0d0/6.0d0*GPL2(0,2)*u**3*v**(-2)
     .        - 1.0d0/6.0d0*GPL2(0,2)*u**3*v**(-1)
     .        - 1.0d0/12.0d0*GPL2(0,2)*u**4*v**(-2)
     .        + 1.0d0/12.0d0*GPL2(1,2)*u**(-2)*v**(-2)
     .        - 1.0d0/6.0d0*GPL2(1,2)*u**(-2)*v**(-1)
     .        + 1.0d0/12.0d0*GPL2(1,2)*u**(-2)
     .        - 1.0d0/6.0d0*GPL2(1,2)*u**(-1)*v**(-2)
     .        + 1.0d0/6.0d0*GPL2(1,2)*u**(-1)
     .        - 1.0d0/12.0d0*GPL2(1,2)*v**(-2)
     .        + 1.0d0/3.0d0*GPL2(1,2)*v**(-1)
     .        + 1.0d0/3.0d0*GPL2(1,2)*u*v**(-2)
     .        - 1.0d0/3.0d0*GPL2(1,2)*u*v**(-1)
         col4 = col4 + 1.0d0/6.0d0*GPL2(1,2)*u
     .        - 1.0d0/12.0d0*GPL2(1,2)*u**2*v**(-2)
     .        + 1.0d0/12.0d0*GPL2(1,2)*u**2
     .        - 1.0d0/6.0d0*GPL2(1,2)*u**3*v**(-2)
     .        + 1.0d0/6.0d0*GPL2(1,2)*u**3*v**(-1)
     .        + 1.0d0/12.0d0*GPL2(1,2)*u**4*v**(-2)
     .        - 1.0d0/6.0d0*GPL2(2,0)*u*v**(-2)
     .        + 1.0d0/3.0d0*GPL2(2,0)*u*v**(-1)
     .        - 1.0d0/6.0d0*GPL2(2,0)*u
     .        + 1.0d0/12.0d0*GPL2(2,0)*u**2*v**(-2)
     .        - 1.0d0/12.0d0*GPL2(2,0)*u**2
     .        + 1.0d0/6.0d0*GPL2(2,0)*u**3*v**(-2)
     .        - 1.0d0/6.0d0*GPL2(2,0)*u**3*v**(-1)
     .        - 1.0d0/12.0d0*GPL2(2,0)*u**4*v**(-2)
     .        + 1.0d0/12.0d0*GPL2(3,0)*v**(-2)*omumv**(-2)
         col4 = col4 - 1.0d0/6.0d0*GPL2(3,0)*v**(-2)*omumv**(-1)
     .        + 1.0d0/12.0d0*GPL2(3,0)*v**(-2)
     .        - 1.0d0/6.0d0*GPL2(3,0)*v**(-1)*omumv**(-2)
     .        + 1.0d0/6.0d0*GPL2(3,0)*v**(-1)
     .        + 1.0d0/12.0d0*GPL2(3,0)*omumv**(-2)
     .        + 1.0d0/6.0d0*GPL2(3,0)*omumv**(-1)
     .        - 1.0d0/3.0d0*GPL2(3,0)*u*v**(-1)
     .        + 1.0d0/6.0d0*GPL2(3,0)*u
     .        - 1.0d0/12.0d0*GPL2(3,0)*u**2*v**(-2)
     .        + 1.0d0/12.0d0*GPL2(3,0)*u**2
     .        - 1.0d0/6.0d0*GPL2(3,0)*u**3*v**(-2)
     .        + 1.0d0/6.0d0*GPL2(3,0)*u**3*v**(-1)
     .        + 1.0d0/12.0d0*GPL2(3,0)*u**4*v**(-2)

         col5 = - 1781.0d0/432.0d0
     .        + 11.0d0/12.0d0*u**(-1)*v**(-2)*pi*ci
     .        - 11.0d0/6.0d0*u**(-1)*v**(-1)*pi*ci
     .        + 11.0d0/12.0d0*u**(-1)*pi*ci
     .        + 11.0d0/72.0d0*v**(-2)*omumv**(-2)*pi**2
     .        + 11.0d0/12.0d0*v**(-2)*omumv**(-1)*pi*ci
     .        - 11.0d0/36.0d0*v**(-2)*omumv**(-1)*pi**2
     .        + 275.0d0/108.0d0*v**(-2)
     .        - 19.0d0/9.0d0*v**(-2)*pi*ci
     .        + 2.0d0/9.0d0*v**(-2)*pi**2
     .        - 11.0d0/36.0d0*v**(-1)*omumv**(-2)*pi**2
     .        - 11.0d0/6.0d0*v**(-1)*omumv**(-1)*pi*ci
     .        + 1.0d0/2.0d0*v**(-1)*omumv**(-1)*pi**2
     .        - 275.0d0/108.0d0*v**(-1)
     .        + 209.0d0/72.0d0*v**(-1)*pi*ci
         col5 = col5 - 19.0d0/72.0d0*v**(-1)*pi**2
     .        + 11.0d0/72.0d0*omumv**(-2)*pi**2
     .        + 11.0d0/12.0d0*omumv**(-1)*pi*ci
     .        - 7.0d0/36.0d0*omumv**(-1)*pi**2
     .        - 611.0d0/216.0d0*pi*ci
     .        + 55.0d0/864.0d0*pi**2
     .        - 11.0d0/144.0d0*pi**3*ci
     .        - 37.0d0/24.0d0*zeta3
     .        - 275.0d0/108.0d0*u*v**(-2)
     .        - 13.0d0/24.0d0*u*v**(-2)*pi*ci
     .        - 5.0d0/72.0d0*u*v**(-2)*pi**2
     .        + 275.0d0/108.0d0*u*v**(-1)
     .        - 3.0d0/8.0d0*u*v**(-1)*pi*ci
     .        + 61.0d0/72.0d0*u*v**(-1)*pi**2
     .        + 11.0d0/12.0d0*u*pi*ci
         col5 = col5 - 7.0d0/9.0d0*u*pi**2
     .        + 275.0d0/108.0d0*u**2*v**(-2)
     .        + 13.0d0/24.0d0*u**2*v**(-2)*pi*ci
     .        + 49.0d0/72.0d0*u**2*v**(-2)*pi**2
     .        + 11.0d0/12.0d0*u**2*v**(-1)*pi*ci
     .        - 2.0d0*u**2*v**(-1)*pi**2
     .        + 11.0d0/18.0d0*u**2*pi**2
     .        - 11.0d0/9.0d0*u**3*v**(-2)*pi**2
     .        + 11.0d0/9.0d0*u**3*v**(-1)*pi**2
     .        + 11.0d0/18.0d0*u**4*v**(-2)*pi**2
     .        - 149.0d0/72.0d0*HPL1(0)
     .        + 11.0d0/12.0d0*HPL1(0)*u**(-1)*v**(-1)
     .        - 11.0d0/12.0d0*HPL1(0)*u**(-1)
     .        + 11.0d0/12.0d0*HPL1(0)*v**(-2)*omumv**(-2)*pi*ci
     .        - 11.0d0/6.0d0*HPL1(0)*v**(-2)*omumv**(-1)*pi*ci
         col5 = col5 + 65.0d0/72.0d0*HPL1(0)*v**(-2)
     .        + 13.0d0/12.0d0*HPL1(0)*v**(-2)*pi*ci
     .        - 11.0d0/6.0d0*HPL1(0)*v**(-1)*omumv**(-2)*pi*ci
     .        + 11.0d0/12.0d0*HPL1(0)*v**(-1)*omumv**(-1)
     .        + 3.0d0*HPL1(0)*v**(-1)*omumv**(-1)*pi*ci
     .        - 197.0d0/72.0d0*HPL1(0)*v**(-1)
     .        - 4.0d0/3.0d0*HPL1(0)*v**(-1)*pi*ci
     .        + 11.0d0/12.0d0*HPL1(0)*omumv**(-2)*pi*ci
     .        - 11.0d0/12.0d0*HPL1(0)*omumv**(-1)
     .        - 7.0d0/6.0d0*HPL1(0)*omumv**(-1)*pi*ci
     .        - 25.0d0/12.0d0*HPL1(0)*pi*ci
     .        - 1.0d0/16.0d0*HPL1(0)*pi**2
     .        - 65.0d0/72.0d0*HPL1(0)*u*v**(-2)
     .        - 1.0d0/6.0d0*HPL1(0)*u*v**(-2)*pi*ci
     .        + 131.0d0/72.0d0*HPL1(0)*u*v**(-1)
         col5 = col5 + 4.0d0/3.0d0*HPL1(0)*u*v**(-1)*pi*ci
     .        - 11.0d0/12.0d0*HPL1(0)*u
     .        - 7.0d0/6.0d0*HPL1(0)*u*pi*ci
     .        + 65.0d0/72.0d0*HPL1(0)*u**2*v**(-2)
     .        + 13.0d0/12.0d0*HPL1(0)*u**2*v**(-2)*pi*ci
     .        - 11.0d0/12.0d0*HPL1(0)*u**2*v**(-1)
     .        - 3.0d0*HPL1(0)*u**2*v**(-1)*pi*ci
     .        + 11.0d0/12.0d0*HPL1(0)*u**2*pi*ci
     .        - 11.0d0/6.0d0*HPL1(0)*u**3*v**(-2)*pi*ci
     .        + 11.0d0/6.0d0*HPL1(0)*u**3*v**(-1)*pi*ci
     .        + 11.0d0/12.0d0*HPL1(0)*u**4*v**(-2)*pi*ci
     .        + 35.0d0/36.0d0*HPL1(0)*GPL1(0)
     .        + 11.0d0/12.0d0*HPL1(0)*GPL1(0)*v**(-2)*omumv**(-2)
     .        - 11.0d0/6.0d0*HPL1(0)*GPL1(0)*v**(-2)*omumv**(-1)
     .        + 13.0d0/12.0d0*HPL1(0)*GPL1(0)*v**(-2)
         col5 = col5 - 11.0d0/6.0d0*HPL1(0)*GPL1(0)*v**(-1)*omumv**(-2)
     .        + 3.0d0*HPL1(0)*GPL1(0)*v**(-1)*omumv**(-1)
     .        - 4.0d0/3.0d0*HPL1(0)*GPL1(0)*v**(-1)
     .        + 11.0d0/12.0d0*HPL1(0)*GPL1(0)*omumv**(-2)
     .        - 7.0d0/6.0d0*HPL1(0)*GPL1(0)*omumv**(-1)
     .        + 5.0d0/12.0d0*HPL1(0)*GPL1(0)*pi*ci
     .        - 1.0d0/6.0d0*HPL1(0)*GPL1(0)*u*v**(-2)
     .        + 4.0d0/3.0d0*HPL1(0)*GPL1(0)*u*v**(-1)
     .        - 7.0d0/6.0d0*HPL1(0)*GPL1(0)*u
     .        + 13.0d0/12.0d0*HPL1(0)*GPL1(0)*u**2*v**(-2)
     .        - 3.0d0*HPL1(0)*GPL1(0)*u**2*v**(-1)
     .        + 11.0d0/12.0d0*HPL1(0)*GPL1(0)*u**2
     .        - 11.0d0/6.0d0*HPL1(0)*GPL1(0)*u**3*v**(-2)
     .        + 11.0d0/6.0d0*HPL1(0)*GPL1(0)*u**3*v**(-1)
     .        + 11.0d0/12.0d0*HPL1(0)*GPL1(0)*u**4*v**(-2)
         col5 = col5 + 5.0d0/9.0d0*HPL1(0)*GPL1(1)
     .        - 11.0d0/12.0d0*HPL1(0)*GPL1(1)*u**(-2)*v**(-2)
     .        + 11.0d0/6.0d0*HPL1(0)*GPL1(1)*u**(-2)*v**(-1)
     .        - 11.0d0/12.0d0*HPL1(0)*GPL1(1)*u**(-2)
     .        + 11.0d0/6.0d0*HPL1(0)*GPL1(1)*u**(-1)*v**(-2)
     .        - 3.0d0*HPL1(0)*GPL1(1)*u**(-1)*v**(-1)
     .        + 7.0d0/6.0d0*HPL1(0)*GPL1(1)*u**(-1)
     .        - 13.0d0/12.0d0*HPL1(0)*GPL1(1)*v**(-2)
     .        + 4.0d0/3.0d0*HPL1(0)*GPL1(1)*v**(-1)
     .        - 1.0d0/6.0d0*HPL1(0)*GPL1(1)*pi*ci
     .        + 1.0d0/6.0d0*HPL1(0)*GPL1(1)*u*v**(-2)
     .        - 4.0d0/3.0d0*HPL1(0)*GPL1(1)*u*v**(-1)
     .        + 7.0d0/6.0d0*HPL1(0)*GPL1(1)*u
     .        - 13.0d0/12.0d0*HPL1(0)*GPL1(1)*u**2*v**(-2)
     .        + 3.0d0*HPL1(0)*GPL1(1)*u**2*v**(-1)
         col5 = col5 - 11.0d0/12.0d0*HPL1(0)*GPL1(1)*u**2
     .        + 11.0d0/6.0d0*HPL1(0)*GPL1(1)*u**3*v**(-2)
     .        - 11.0d0/6.0d0*HPL1(0)*GPL1(1)*u**3*v**(-1)
     .        - 11.0d0/12.0d0*HPL1(0)*GPL1(1)*u**4*v**(-2)
     .        + 35.0d0/36.0d0*HPL1(0)*GPL1(2)
     .        + 11.0d0/12.0d0*HPL1(0)*GPL1(2)*u**(-2)*v**(-2)
     .        - 11.0d0/6.0d0*HPL1(0)*GPL1(2)*u**(-2)*v**(-1)
     .        + 11.0d0/12.0d0*HPL1(0)*GPL1(2)*u**(-2)
     .        - 11.0d0/6.0d0*HPL1(0)*GPL1(2)*u**(-1)*v**(-2)
     .        + 3.0d0*HPL1(0)*GPL1(2)*u**(-1)*v**(-1)
     .        - 7.0d0/6.0d0*HPL1(0)*GPL1(2)*u**(-1)
     .        + 13.0d0/12.0d0*HPL1(0)*GPL1(2)*v**(-2)
     .        - 4.0d0/3.0d0*HPL1(0)*GPL1(2)*v**(-1)
     .        - 5.0d0/12.0d0*HPL1(0)*GPL1(2)*pi*ci
     .        - 1.0d0/6.0d0*HPL1(0)*GPL1(2)*u*v**(-2)
         col5 = col5 + 4.0d0/3.0d0*HPL1(0)*GPL1(2)*u*v**(-1)
     .        - 7.0d0/6.0d0*HPL1(0)*GPL1(2)*u
     .        + 13.0d0/12.0d0*HPL1(0)*GPL1(2)*u**2*v**(-2)
     .        - 3.0d0*HPL1(0)*GPL1(2)*u**2*v**(-1)
     .        + 11.0d0/12.0d0*HPL1(0)*GPL1(2)*u**2
     .        - 11.0d0/6.0d0*HPL1(0)*GPL1(2)*u**3*v**(-2)
     .        + 11.0d0/6.0d0*HPL1(0)*GPL1(2)*u**3*v**(-1)
     .        + 11.0d0/12.0d0*HPL1(0)*GPL1(2)*u**4*v**(-2)
     .        + 5.0d0/9.0d0*HPL1(0)*GPL1(3)
     .        - 11.0d0/12.0d0*HPL1(0)*GPL1(3)*v**(-2)*omumv**(-2)
     .        + 11.0d0/6.0d0*HPL1(0)*GPL1(3)*v**(-2)*omumv**(-1)
     .        - 13.0d0/12.0d0*HPL1(0)*GPL1(3)*v**(-2)
     .        + 11.0d0/6.0d0*HPL1(0)*GPL1(3)*v**(-1)*omumv**(-2)
     .        - 3.0d0*HPL1(0)*GPL1(3)*v**(-1)*omumv**(-1)
     .        + 4.0d0/3.0d0*HPL1(0)*GPL1(3)*v**(-1)
         col5 = col5 - 11.0d0/12.0d0*HPL1(0)*GPL1(3)*omumv**(-2)
     .        + 7.0d0/6.0d0*HPL1(0)*GPL1(3)*omumv**(-1)
     .        + 1.0d0/6.0d0*HPL1(0)*GPL1(3)*pi*ci
     .        + 1.0d0/6.0d0*HPL1(0)*GPL1(3)*u*v**(-2)
     .        - 4.0d0/3.0d0*HPL1(0)*GPL1(3)*u*v**(-1)
     .        + 7.0d0/6.0d0*HPL1(0)*GPL1(3)*u
     .        - 13.0d0/12.0d0*HPL1(0)*GPL1(3)*u**2*v**(-2)
     .        + 3.0d0*HPL1(0)*GPL1(3)*u**2*v**(-1)
     .        - 11.0d0/12.0d0*HPL1(0)*GPL1(3)*u**2
     .        + 11.0d0/6.0d0*HPL1(0)*GPL1(3)*u**3*v**(-2)
     .        - 11.0d0/6.0d0*HPL1(0)*GPL1(3)*u**3*v**(-1)
     .        - 11.0d0/12.0d0*HPL1(0)*GPL1(3)*u**4*v**(-2)
     .        + 2.0d0/3.0d0*HPL1(0)*GPL2(0,0)
     .        + 7.0d0/12.0d0*HPL1(0)*GPL2(0,2)
     .        - 5.0d0/6.0d0*HPL1(0)*GPL2(0,3)
         col5 = col5 - 1.0d0/3.0d0*HPL1(0)*GPL2(1,1)
     .        - 2.0d0/3.0d0*HPL1(0)*GPL2(1,2)
     .        + 7.0d0/12.0d0*HPL1(0)*GPL2(2,0)
     .        - 5.0d0/6.0d0*HPL1(0)*GPL2(2,1)
     .        + 2.0d0/3.0d0*HPL1(0)*GPL2(2,2)
     .        - 2.0d0/3.0d0*HPL1(0)*GPL2(3,0)
     .        - 1.0d0/3.0d0*HPL1(0)*GPL2(3,3)
     .        - 35.0d0/216.0d0*HPL1(1)
     .        - 11.0d0/12.0d0*HPL1(1)*u**(-1)*v**(-2)
     .        + 11.0d0/6.0d0*HPL1(1)*u**(-1)*v**(-1)
     .        - 11.0d0/12.0d0*HPL1(1)*u**(-1)
     .        + 14.0d0/9.0d0*HPL1(1)*v**(-2)
     .        + 1.0d0/12.0d0*HPL1(1)*v**(-2)*pi*ci
     .        - 169.0d0/72.0d0*HPL1(1)*v**(-1)
     .        - 1.0d0/12.0d0*HPL1(1)*v**(-1)*pi*ci
         col5 = col5 - 7.0d0/12.0d0*HPL1(1)*pi*ci
     .        + 5.0d0/144.0d0*HPL1(1)*pi**2
     .        - 53.0d0/72.0d0*HPL1(1)*u*v**(-2)
     .        - 1.0d0/12.0d0*HPL1(1)*u*v**(-2)*pi*ci
     .        + 169.0d0/72.0d0*HPL1(1)*u*v**(-1)
     .        + 5.0d0/4.0d0*HPL1(1)*u*v**(-1)*pi*ci
     .        - 11.0d0/12.0d0*HPL1(1)*u
     .        - 7.0d0/6.0d0*HPL1(1)*u*pi*ci
     .        + 14.0d0/9.0d0*HPL1(1)*u**2*v**(-2)
     .        + HPL1(1)*u**2*v**(-2)*pi*ci
     .        - 11.0d0/6.0d0*HPL1(1)*u**2*v**(-1)
     .        - 3.0d0*HPL1(1)*u**2*v**(-1)*pi*ci
     .        + 11.0d0/12.0d0*HPL1(1)*u**2*pi*ci
     .        - 11.0d0/12.0d0*HPL1(1)*u**3*v**(-2)
     .        - 11.0d0/6.0d0*HPL1(1)*u**3*v**(-2)*pi*ci
         col5 = col5 + 11.0d0/6.0d0*HPL1(1)*u**3*v**(-1)*pi*ci
     .        + 11.0d0/12.0d0*HPL1(1)*u**4*v**(-2)*pi*ci
     .        + 1.0d0/36.0d0*HPL1(1)*GPL1(0)
     .        + 1.0d0/12.0d0*HPL1(1)*GPL1(0)*v**(-2)
     .        - 1.0d0/12.0d0*HPL1(1)*GPL1(0)*v**(-1)
     .        + 1.0d0/4.0d0*HPL1(1)*GPL1(0)*pi*ci
     .        - 1.0d0/12.0d0*HPL1(1)*GPL1(0)*u*v**(-2)
     .        + 5.0d0/4.0d0*HPL1(1)*GPL1(0)*u*v**(-1)
     .        - 7.0d0/6.0d0*HPL1(1)*GPL1(0)*u
     .        + HPL1(1)*GPL1(0)*u**2*v**(-2)
     .        - 3.0d0*HPL1(1)*GPL1(0)*u**2*v**(-1)
     .        + 11.0d0/12.0d0*HPL1(1)*GPL1(0)*u**2
     .        - 11.0d0/6.0d0*HPL1(1)*GPL1(0)*u**3*v**(-2)
     .        + 11.0d0/6.0d0*HPL1(1)*GPL1(0)*u**3*v**(-1)
     .        + 11.0d0/12.0d0*HPL1(1)*GPL1(0)*u**4*v**(-2)
         col5 = col5 + 5.0d0/9.0d0*HPL1(1)*GPL1(1)
     .        - 11.0d0/12.0d0*HPL1(1)*GPL1(1)*u**(-2)*v**(-2)
     .        + 11.0d0/6.0d0*HPL1(1)*GPL1(1)*u**(-2)*v**(-1)
     .        - 11.0d0/12.0d0*HPL1(1)*GPL1(1)*u**(-2)
     .        + 11.0d0/6.0d0*HPL1(1)*GPL1(1)*u**(-1)*v**(-2)
     .        - 3.0d0*HPL1(1)*GPL1(1)*u**(-1)*v**(-1)
     .        + 7.0d0/6.0d0*HPL1(1)*GPL1(1)*u**(-1)
     .        - 13.0d0/12.0d0*HPL1(1)*GPL1(1)*v**(-2)
     .        + 4.0d0/3.0d0*HPL1(1)*GPL1(1)*v**(-1)
     .        + 1.0d0/6.0d0*HPL1(1)*GPL1(1)*pi*ci
     .        + 1.0d0/6.0d0*HPL1(1)*GPL1(1)*u*v**(-2)
     .        - 4.0d0/3.0d0*HPL1(1)*GPL1(1)*u*v**(-1)
     .        + 7.0d0/6.0d0*HPL1(1)*GPL1(1)*u
     .        - 13.0d0/12.0d0*HPL1(1)*GPL1(1)*u**2*v**(-2)
     .        + 3.0d0*HPL1(1)*GPL1(1)*u**2*v**(-1)
         col5 = col5 - 11.0d0/12.0d0*HPL1(1)*GPL1(1)*u**2
     .        + 11.0d0/6.0d0*HPL1(1)*GPL1(1)*u**3*v**(-2)
     .        - 11.0d0/6.0d0*HPL1(1)*GPL1(1)*u**3*v**(-1)
     .        - 11.0d0/12.0d0*HPL1(1)*GPL1(1)*u**4*v**(-2)
     .        + 11.0d0/12.0d0*HPL1(1)*GPL1(2)
     .        + 1.0d0/3.0d0*HPL1(1)*GPL1(2)*pi*ci
     .        + 1.0d0/3.0d0*HPL1(1)*GPL2(0,0)
     .        + 1.0d0/3.0d0*HPL1(1)*GPL2(0,2)
     .        - 1.0d0/3.0d0*HPL1(1)*GPL2(1,1)
     .        - 2.0d0/3.0d0*HPL1(1)*GPL2(1,2)
     .        + 1.0d0/3.0d0*HPL1(1)*GPL2(2,0)
     .        - 5.0d0/6.0d0*HPL1(1)*GPL2(2,1)
     .        - 145.0d0/36.0d0*HPL2(0,0)
     .        - 11.0d0/12.0d0*HPL2(0,0)*v**(-2)*omumv**(-2)
     .        + 11.0d0/6.0d0*HPL2(0,0)*v**(-2)*omumv**(-1)
         col5 = col5 - 13.0d0/12.0d0*HPL2(0,0)*v**(-2)
     .        + 11.0d0/6.0d0*HPL2(0,0)*v**(-1)*omumv**(-2)
     .        - 3.0d0*HPL2(0,0)*v**(-1)*omumv**(-1)
     .        + 4.0d0/3.0d0*HPL2(0,0)*v**(-1)
     .        - 11.0d0/12.0d0*HPL2(0,0)*omumv**(-2)
     .        + 7.0d0/6.0d0*HPL2(0,0)*omumv**(-1)
     .        - HPL2(0,0)*pi*ci
     .        + 1.0d0/6.0d0*HPL2(0,0)*u*v**(-2)
     .        - 4.0d0/3.0d0*HPL2(0,0)*u*v**(-1)
     .        + 7.0d0/6.0d0*HPL2(0,0)*u
     .        - 13.0d0/12.0d0*HPL2(0,0)*u**2*v**(-2)
     .        + 3.0d0*HPL2(0,0)*u**2*v**(-1)
     .        - 11.0d0/12.0d0*HPL2(0,0)*u**2
     .        + 11.0d0/6.0d0*HPL2(0,0)*u**3*v**(-2)
     .        - 11.0d0/6.0d0*HPL2(0,0)*u**3*v**(-1)
         col5 = col5 - 11.0d0/12.0d0*HPL2(0,0)*u**4*v**(-2)
     .        - 11.0d0/6.0d0*HPL2(0,0)*GPL1(0)
     .        + HPL2(0,0)*GPL1(1)
     .        - HPL2(0,0)*GPL1(2)
     .        + 2.0d0/3.0d0*HPL2(0,0)*GPL1(3)
     .        - 55.0d0/36.0d0*HPL2(0,1)
     .        - 7.0d0/12.0d0*HPL2(0,1)*pi*ci
     .        - 7.0d0/12.0d0*HPL2(0,1)*GPL1(0)
     .        + HPL2(0,1)*GPL1(1)
     .        + 1.0d0/6.0d0*HPL2(0,1)*GPL1(2)
     .        - 55.0d0/36.0d0*HPL2(1,0)
     .        + 11.0d0/12.0d0*HPL2(1,0)*v**(-2)*omumv**(-2)
     .        - 11.0d0/6.0d0*HPL2(1,0)*v**(-2)*omumv**(-1)
     .        + 11.0d0/12.0d0*HPL2(1,0)*v**(-2)
     .        - 11.0d0/6.0d0*HPL2(1,0)*v**(-1)*omumv**(-2)
         col5 = col5 + 3.0d0*HPL2(1,0)*v**(-1)*omumv**(-1)
     .        - 7.0d0/6.0d0*HPL2(1,0)*v**(-1)
     .        + 11.0d0/12.0d0*HPL2(1,0)*omumv**(-2)
     .        - 7.0d0/6.0d0*HPL2(1,0)*omumv**(-1)
     .        - 5.0d0/12.0d0*HPL2(1,0)*pi*ci
     .        - 7.0d0/6.0d0*HPL2(1,0)*u*v**(-1)
     .        + 7.0d0/6.0d0*HPL2(1,0)*u
     .        - 11.0d0/12.0d0*HPL2(1,0)*u**2*v**(-2)
     .        + 3.0d0*HPL2(1,0)*u**2*v**(-1)
     .        - 11.0d0/12.0d0*HPL2(1,0)*u**2
     .        + 11.0d0/6.0d0*HPL2(1,0)*u**3*v**(-2)
     .        - 11.0d0/6.0d0*HPL2(1,0)*u**3*v**(-1)
     .        - 11.0d0/12.0d0*HPL2(1,0)*u**4*v**(-2)
     .        - 7.0d0/12.0d0*HPL2(1,0)*GPL1(0)
     .        + 2.0d0/3.0d0*HPL2(1,0)*GPL1(1)
         col5 = col5 - 2.0d0/3.0d0*HPL2(1,0)*GPL1(2)
     .        - 11.0d0/12.0d0*HPL2(1,1)
     .        - 1.0d0/3.0d0*HPL2(1,1)*pi*ci
     .        - 1.0d0/3.0d0*HPL2(1,1)*GPL1(0)
     .        + 2.0d0/3.0d0*HPL2(1,1)*GPL1(1)
     .        + 11.0d0/6.0d0*HPL3(0,0,0)
     .        - 1.0d0/6.0d0*HPL3(1,0,1)
     .        + 1.0d0/3.0d0*HPL3(1,1,0)
     .        + 103.0d0/108.0d0*GPL1(0)
     .        + 11.0d0/12.0d0*GPL1(0)*v**(-2)*omumv**(-1)
     .        - 35.0d0/24.0d0*GPL1(0)*v**(-2)
     .        - 1.0d0/12.0d0*GPL1(0)*v**(-2)*pi*ci
     .        - 11.0d0/6.0d0*GPL1(0)*v**(-1)*omumv**(-1)
     .        + 35.0d0/24.0d0*GPL1(0)*v**(-1)
     .        + 1.0d0/12.0d0*GPL1(0)*v**(-1)*pi*ci
         col5 = col5 + 11.0d0/12.0d0*GPL1(0)*omumv**(-1)
     .        + 7.0d0/12.0d0*GPL1(0)*pi*ci
     .        + 7.0d0/144.0d0*GPL1(0)*pi**2
     .        - 3.0d0/8.0d0*GPL1(0)*u*v**(-2)
     .        + 1.0d0/12.0d0*GPL1(0)*u*v**(-2)*pi*ci
     .        + 77.0d0/72.0d0*GPL1(0)*u*v**(-1)
     .        - 5.0d0/4.0d0*GPL1(0)*u*v**(-1)*pi*ci
     .        + 7.0d0/6.0d0*GPL1(0)*u*pi*ci
     .        + 43.0d0/36.0d0*GPL1(0)*u**2*v**(-2)
     .        - GPL1(0)*u**2*v**(-2)*pi*ci
     .        - 11.0d0/12.0d0*GPL1(0)*u**2*v**(-1)
     .        + 3.0d0*GPL1(0)*u**2*v**(-1)*pi*ci
     .        - 11.0d0/12.0d0*GPL1(0)*u**2*pi*ci
     .        - 11.0d0/12.0d0*GPL1(0)*u**3*v**(-2)
     .        + 11.0d0/6.0d0*GPL1(0)*u**3*v**(-2)*pi*ci
         col5 = col5 - 11.0d0/6.0d0*GPL1(0)*u**3*v**(-1)*pi*ci
     .        - 11.0d0/12.0d0*GPL1(0)*u**4*v**(-2)*pi*ci
     .        + 11.0d0/12.0d0*GPL1(1)*u**(-2)*v**(-2)*pi*ci
     .        - 11.0d0/6.0d0*GPL1(1)*u**(-2)*v**(-1)*pi*ci
     .        + 11.0d0/12.0d0*GPL1(1)*u**(-2)*pi*ci
     .        - 11.0d0/6.0d0*GPL1(1)*u**(-1)*v**(-2)*pi*ci
     .        + 3.0d0*GPL1(1)*u**(-1)*v**(-1)*pi*ci
     .        - 7.0d0/6.0d0*GPL1(1)*u**(-1)*pi*ci
     .        + 13.0d0/12.0d0*GPL1(1)*v**(-2)*pi*ci
     .        - 4.0d0/3.0d0*GPL1(1)*v**(-1)*pi*ci
     .        - 5.0d0/9.0d0*GPL1(1)*pi*ci
     .        + 7.0d0/18.0d0*GPL1(1)*pi**2
     .        - 1.0d0/6.0d0*GPL1(1)*u*v**(-2)*pi*ci
     .        + 4.0d0/3.0d0*GPL1(1)*u*v**(-1)*pi*ci
     .        - 7.0d0/6.0d0*GPL1(1)*u*pi*ci
         col5 = col5 + 13.0d0/12.0d0*GPL1(1)*u**2*v**(-2)*pi*ci
     .        - 3.0d0*GPL1(1)*u**2*v**(-1)*pi*ci
     .        + 11.0d0/12.0d0*GPL1(1)*u**2*pi*ci
     .        - 11.0d0/6.0d0*GPL1(1)*u**3*v**(-2)*pi*ci
     .        + 11.0d0/6.0d0*GPL1(1)*u**3*v**(-1)*pi*ci
     .        + 11.0d0/12.0d0*GPL1(1)*u**4*v**(-2)*pi*ci
     .        + 35.0d0/216.0d0*GPL1(2)
     .        + 11.0d0/12.0d0*GPL1(2)*u**(-1)*v**(-2)
     .        - 11.0d0/6.0d0*GPL1(2)*u**(-1)*v**(-1)
     .        + 11.0d0/12.0d0*GPL1(2)*u**(-1)
     .        - 14.0d0/9.0d0*GPL1(2)*v**(-2)
     .        - 1.0d0/12.0d0*GPL1(2)*v**(-2)*pi*ci
     .        + 169.0d0/72.0d0*GPL1(2)*v**(-1)
     .        + 1.0d0/12.0d0*GPL1(2)*v**(-1)*pi*ci
     .        + 7.0d0/12.0d0*GPL1(2)*pi*ci
         col5 = col5 - 13.0d0/144.0d0*GPL1(2)*pi**2
     .        + 53.0d0/72.0d0*GPL1(2)*u*v**(-2)
     .        + 1.0d0/12.0d0*GPL1(2)*u*v**(-2)*pi*ci
     .        - 169.0d0/72.0d0*GPL1(2)*u*v**(-1)
     .        - 5.0d0/4.0d0*GPL1(2)*u*v**(-1)*pi*ci
     .        + 11.0d0/12.0d0*GPL1(2)*u
     .        + 7.0d0/6.0d0*GPL1(2)*u*pi*ci
     .        - 14.0d0/9.0d0*GPL1(2)*u**2*v**(-2)
     .        - GPL1(2)*u**2*v**(-2)*pi*ci
     .        + 11.0d0/6.0d0*GPL1(2)*u**2*v**(-1)
     .        + 3.0d0*GPL1(2)*u**2*v**(-1)*pi*ci
     .        - 11.0d0/12.0d0*GPL1(2)*u**2*pi*ci
     .        + 11.0d0/12.0d0*GPL1(2)*u**3*v**(-2)
     .        + 11.0d0/6.0d0*GPL1(2)*u**3*v**(-2)*pi*ci
     .        - 11.0d0/6.0d0*GPL1(2)*u**3*v**(-1)*pi*ci
         col5 = col5 - 11.0d0/12.0d0*GPL1(2)*u**4*v**(-2)*pi*ci
     .        + 11.0d0/12.0d0*GPL1(3)*v**(-2)*omumv**(-2)*pi*ci
     .        - 11.0d0/6.0d0*GPL1(3)*v**(-2)*omumv**(-1)*pi*ci
     .        + 13.0d0/12.0d0*GPL1(3)*v**(-2)*pi*ci
     .        - 11.0d0/6.0d0*GPL1(3)*v**(-1)*omumv**(-2)*pi*ci
     .        + 3.0d0*GPL1(3)*v**(-1)*omumv**(-1)*pi*ci
     .        - 4.0d0/3.0d0*GPL1(3)*v**(-1)*pi*ci
     .        + 11.0d0/12.0d0*GPL1(3)*omumv**(-2)*pi*ci
     .        - 7.0d0/6.0d0*GPL1(3)*omumv**(-1)*pi*ci
     .        - 5.0d0/9.0d0*GPL1(3)*pi*ci
     .        + 4.0d0/9.0d0*GPL1(3)*pi**2
     .        - 1.0d0/6.0d0*GPL1(3)*u*v**(-2)*pi*ci
     .        + 4.0d0/3.0d0*GPL1(3)*u*v**(-1)*pi*ci
     .        - 7.0d0/6.0d0*GPL1(3)*u*pi*ci
     .        + 13.0d0/12.0d0*GPL1(3)*u**2*v**(-2)*pi*ci
         col5 = col5 - 3.0d0*GPL1(3)*u**2*v**(-1)*pi*ci
     .        + 11.0d0/12.0d0*GPL1(3)*u**2*pi*ci
     .        - 11.0d0/6.0d0*GPL1(3)*u**3*v**(-2)*pi*ci
     .        + 11.0d0/6.0d0*GPL1(3)*u**3*v**(-1)*pi*ci
     .        + 11.0d0/12.0d0*GPL1(3)*u**4*v**(-2)*pi*ci
     .        - 11.0d0/12.0d0*GPL2(0,0)
     .        - 1.0d0/3.0d0*GPL2(0,0)*pi*ci
     .        - 1.0d0/36.0d0*GPL2(0,2)
     .        - 1.0d0/12.0d0*GPL2(0,2)*v**(-2)
     .        + 1.0d0/12.0d0*GPL2(0,2)*v**(-1)
     .        - 1.0d0/4.0d0*GPL2(0,2)*pi*ci
     .        + 1.0d0/12.0d0*GPL2(0,2)*u*v**(-2)
     .        - 5.0d0/4.0d0*GPL2(0,2)*u*v**(-1)
     .        + 7.0d0/6.0d0*GPL2(0,2)*u
     .        - GPL2(0,2)*u**2*v**(-2)
         col5 = col5 + 3.0d0*GPL2(0,2)*u**2*v**(-1)
     .        - 11.0d0/12.0d0*GPL2(0,2)*u**2
     .        + 11.0d0/6.0d0*GPL2(0,2)*u**3*v**(-2)
     .        - 11.0d0/6.0d0*GPL2(0,2)*u**3*v**(-1)
     .        - 11.0d0/12.0d0*GPL2(0,2)*u**4*v**(-2)
     .        + 5.0d0/6.0d0*GPL2(0,3)*pi*ci
     .        + 1.0d0/3.0d0*GPL2(1,1)*pi*ci
     .        - 5.0d0/9.0d0*GPL2(1,2)
     .        + 11.0d0/12.0d0*GPL2(1,2)*u**(-2)*v**(-2)
     .        - 11.0d0/6.0d0*GPL2(1,2)*u**(-2)*v**(-1)
     .        + 11.0d0/12.0d0*GPL2(1,2)*u**(-2)
     .        - 11.0d0/6.0d0*GPL2(1,2)*u**(-1)*v**(-2)
     .        + 3.0d0*GPL2(1,2)*u**(-1)*v**(-1)
     .        - 7.0d0/6.0d0*GPL2(1,2)*u**(-1)
     .        + 13.0d0/12.0d0*GPL2(1,2)*v**(-2)
         col5 = col5 - 4.0d0/3.0d0*GPL2(1,2)*v**(-1)
     .        - 1.0d0/6.0d0*GPL2(1,2)*pi*ci
     .        - 1.0d0/6.0d0*GPL2(1,2)*u*v**(-2)
     .        + 4.0d0/3.0d0*GPL2(1,2)*u*v**(-1)
     .        - 7.0d0/6.0d0*GPL2(1,2)*u
     .        + 13.0d0/12.0d0*GPL2(1,2)*u**2*v**(-2)
     .        - 3.0d0*GPL2(1,2)*u**2*v**(-1)
     .        + 11.0d0/12.0d0*GPL2(1,2)*u**2
     .        - 11.0d0/6.0d0*GPL2(1,2)*u**3*v**(-2)
     .        + 11.0d0/6.0d0*GPL2(1,2)*u**3*v**(-1)
     .        + 11.0d0/12.0d0*GPL2(1,2)*u**4*v**(-2)
     .        - 1.0d0/36.0d0*GPL2(2,0)
     .        - 1.0d0/12.0d0*GPL2(2,0)*v**(-2)
     .        + 1.0d0/12.0d0*GPL2(2,0)*v**(-1)
     .        - 1.0d0/4.0d0*GPL2(2,0)*pi*ci
         col5 = col5 + 1.0d0/12.0d0*GPL2(2,0)*u*v**(-2)
     .        - 5.0d0/4.0d0*GPL2(2,0)*u*v**(-1)
     .        + 7.0d0/6.0d0*GPL2(2,0)*u
     .        - GPL2(2,0)*u**2*v**(-2)
     .        + 3.0d0*GPL2(2,0)*u**2*v**(-1)
     .        - 11.0d0/12.0d0*GPL2(2,0)*u**2
     .        + 11.0d0/6.0d0*GPL2(2,0)*u**3*v**(-2)
     .        - 11.0d0/6.0d0*GPL2(2,0)*u**3*v**(-1)
     .        - 11.0d0/12.0d0*GPL2(2,0)*u**4*v**(-2)
     .        + 5.0d0/6.0d0*GPL2(2,1)*pi*ci
     .        - 11.0d0/12.0d0*GPL2(2,2)
     .        - 1.0d0/3.0d0*GPL2(2,2)*pi*ci
     .        - 5.0d0/9.0d0*GPL2(3,0)
     .        + 11.0d0/12.0d0*GPL2(3,0)*v**(-2)*omumv**(-2)
     .        - 11.0d0/6.0d0*GPL2(3,0)*v**(-2)*omumv**(-1)
         col5 = col5 + 13.0d0/12.0d0*GPL2(3,0)*v**(-2)
     .        - 11.0d0/6.0d0*GPL2(3,0)*v**(-1)*omumv**(-2)
     .        + 3.0d0*GPL2(3,0)*v**(-1)*omumv**(-1)
     .        - 4.0d0/3.0d0*GPL2(3,0)*v**(-1)
     .        + 11.0d0/12.0d0*GPL2(3,0)*omumv**(-2)
     .        - 7.0d0/6.0d0*GPL2(3,0)*omumv**(-1)
     .        - 1.0d0/6.0d0*GPL2(3,0)*pi*ci
     .        - 1.0d0/6.0d0*GPL2(3,0)*u*v**(-2)
     .        + 4.0d0/3.0d0*GPL2(3,0)*u*v**(-1)
     .        - 7.0d0/6.0d0*GPL2(3,0)*u
     .        + 13.0d0/12.0d0*GPL2(3,0)*u**2*v**(-2)
     .        - 3.0d0*GPL2(3,0)*u**2*v**(-1)
     .        + 11.0d0/12.0d0*GPL2(3,0)*u**2
     .        - 11.0d0/6.0d0*GPL2(3,0)*u**3*v**(-2)
     .        + 11.0d0/6.0d0*GPL2(3,0)*u**3*v**(-1)
         col5 = col5 + 11.0d0/12.0d0*GPL2(3,0)*u**4*v**(-2)
     .        + 1.0d0/3.0d0*GPL2(3,3)*pi*ci
     .        - 1.0d0/3.0d0*GPL3(0,0,2)
     .        - 1.0d0/3.0d0*GPL3(0,2,0)
     .        - 1.0d0/3.0d0*GPL3(0,2,2)
     .        + 5.0d0/6.0d0*GPL3(0,3,0)
     .        + 1.0d0/3.0d0*GPL3(1,1,2)
     .        + 2.0d0/3.0d0*GPL3(1,2,2)
     .        - 1.0d0/3.0d0*GPL3(2,0,0)
     .        - 1.0d0/3.0d0*GPL3(2,0,2)
     .        + 5.0d0/6.0d0*GPL3(2,1,2)
     .        - 1.0d0/3.0d0*GPL3(2,2,0)
     .        + 2.0d0/3.0d0*GPL3(3,0,0)
     .        + 1.0d0/3.0d0*GPL3(3,3,0)

         col6 = - 5.0d0/54.0d0*v**(-2)
     .        - 1.0d0/36.0d0*v**(-2)*pi*ci
     .        + 5.0d0/54.0d0*v**(-1)
     .        + 1.0d0/36.0d0*v**(-1)*pi*ci
     .        + 5.0d0/54.0d0*pi*ci
     .        - 5.0d0/144.0d0*pi**2
     .        + 5.0d0/54.0d0*u*v**(-2)
     .        + 1.0d0/12.0d0*u*v**(-2)*pi*ci
     .        - 5.0d0/54.0d0*u*v**(-1)
     .        - 1.0d0/12.0d0*u*v**(-1)*pi*ci
     .        - 5.0d0/54.0d0*u**2*v**(-2)
     .        - 1.0d0/12.0d0*u**2*v**(-2)*pi*ci
     .        + 5.0d0/18.0d0*HPL1(0)
     .        - 5.0d0/36.0d0*HPL1(0)*v**(-2)
     .        + 5.0d0/36.0d0*HPL1(0)*v**(-1)
         col6 = col6 + 5.0d0/36.0d0*HPL1(0)*pi*ci
     .        + 5.0d0/36.0d0*HPL1(0)*u*v**(-2)
     .        - 5.0d0/36.0d0*HPL1(0)*u*v**(-1)
     .        - 5.0d0/36.0d0*HPL1(0)*u**2*v**(-2)
     .        - 5.0d0/36.0d0*HPL1(0)*GPL1(0)
     .        - 5.0d0/36.0d0*HPL1(0)*GPL1(2)
     .        + 5.0d0/54.0d0*HPL1(1)
     .        - 1.0d0/36.0d0*HPL1(1)*v**(-2)
     .        + 1.0d0/36.0d0*HPL1(1)*v**(-1)
     .        + 1.0d0/36.0d0*HPL1(1)*pi*ci
     .        - 1.0d0/36.0d0*HPL1(1)*u*v**(-2)
     .        - 1.0d0/36.0d0*HPL1(1)*u*v**(-1)
     .        - 1.0d0/36.0d0*HPL1(1)*u**2*v**(-2)
     .        - 1.0d0/36.0d0*HPL1(1)*GPL1(0)
     .        - 1.0d0/12.0d0*HPL1(1)*GPL1(2)
         col6 = col6 + 5.0d0/12.0d0*HPL2(0,0)
     .        + 5.0d0/36.0d0*HPL2(0,1)
     .        + 5.0d0/36.0d0*HPL2(1,0)
     .        + 1.0d0/12.0d0*HPL2(1,1)
     .        - 5.0d0/54.0d0*GPL1(0)
     .        + 1.0d0/12.0d0*GPL1(0)*v**(-2)
     .        - 1.0d0/12.0d0*GPL1(0)*v**(-1)
     .        - 1.0d0/36.0d0*GPL1(0)*pi*ci
     .        - 1.0d0/12.0d0*GPL1(0)*u*v**(-2)
     .        + 1.0d0/36.0d0*GPL1(0)*u*v**(-1)
     .        + 1.0d0/36.0d0*GPL1(0)*u**2*v**(-2)
     .        - 5.0d0/54.0d0*GPL1(2)
     .        + 1.0d0/36.0d0*GPL1(2)*v**(-2)
     .        - 1.0d0/36.0d0*GPL1(2)*v**(-1)
     .        - 1.0d0/36.0d0*GPL1(2)*pi*ci
         col6 = col6 + 1.0d0/36.0d0*GPL1(2)*u*v**(-2)
     .        + 1.0d0/36.0d0*GPL1(2)*u*v**(-1)
     .        + 1.0d0/36.0d0*GPL1(2)*u**2*v**(-2)
     .        + 1.0d0/12.0d0*GPL2(0,0)
     .        + 1.0d0/36.0d0*GPL2(0,2)
     .        + 1.0d0/36.0d0*GPL2(2,0)
     .        + 1.0d0/12.0d0*GPL2(2,2)
      endif

      col(1) = col1
      col(2)=  col2
      col(3) = col3
      col(4) = col4
      col(5) = col5
      col(6) = col6

      end subroutine alpha2a2L

!#############################################################

      subroutine beta2a2L(u,v,col)
      implicit none
      !returns beta(2a+)
      ! at two Loops
      !-------------
      real*8, intent(in) :: u,v
      complex*16, dimension(6), intent(out) :: col
      !internal
      complex*16 :: beta
      complex*16 :: ci
      real*8 :: omu,omv,omumv,upv
      real*8, dimension(0:1) :: HPL1
      real*8, dimension(0:1,0:1) :: HPL2
      real*8, dimension(0:1,0:1,0:1) :: HPL3
      real*8, dimension(0:1,0:1,0:1,0:1) :: HPL4
      real*8, dimension(0:3) :: GPL1
      real*8, dimension(0:3,0:3) :: GPL2
      real*8, dimension(0:3,0:3,0:3) :: GPL3
      real*8, dimension(0:3,0:3,0:3,0:3) :: GPL4
      complex*16 :: col1,col2,col3,col4,col5,col6


      omu   = 1.0d0 - u
      omv   = 1.0d0     - v
      omumv = 1.0d0 - u - v
      upv   =         u + v

      ci = (0.d0,1.d0)

      call tdhpl(dble(u),dble(v),4,GPL1,GPL2,GPL3,GPL4,HPL1,HPL2,HPL3,HPL4)

      if(ieorder==4) then
         col1 = 9.0d0/8.0d0

         col2 = 0.0d0

         col3 = 0.0d0

         col4 = 0.0d0

         col5 = 0.0d0

         col6 = 0.0d0
      elseif(ieorder==3) then
         col1 = + 99.0d0/16.0d0
     .          + 3.0d0/4.0d0*pi*ci
     .          + 9.0d0/4.0d0*HPL1(0)
     .          + 3.0d0/4.0d0*HPL1(1)
     .          - 3.0d0/4.0d0*GPL1(0)
     .          - 3.0d0/4.0d0*GPL1(2)

         col2 = 0

         col3 = 0

         col4 = 0

         col5 = - 9.0d0/8.0d0

         col6 = 0
      else
         col1 = + 943.0d0/216.0d0
     .        - 5.0d0/6.0d0*u**(-1)*pi*ci
     .        + 5.0d0/3.0d0*u**(-1)*v*pi*ci
     .        - 5.0d0/6.0d0*u**(-1)*v**2*pi*ci
     .        - 5.0d0/36.0d0*omumv**(-2)*pi**2
     .        - 5.0d0/6.0d0*omumv**(-1)*pi*ci
     .        - 5.0d0/72.0d0*omumv**(-1)*pi**2
     .        + 37.0d0/24.0d0*pi*ci
     .        + 1115.0d0/864.0d0*pi**2
     .        + 209.0d0/288.0d0*pi**3*ci
     .        + 49.0d0/2880.0d0*pi**4
     .        - 11.0d0/16.0d0*zeta3
     .        + 1.0d0/4.0d0*zeta3*pi*ci
     .        + 4.0d0/3.0d0*v*omu**(-1)
     .        + 5.0d0/3.0d0*v*omu**(-1)*pi*ci
         col1 = col1 + 5.0d0/18.0d0*v*omumv**(-2)*pi**2
     .        + 5.0d0/3.0d0*v*omumv**(-1)*pi*ci
     .        + 5.0d0/72.0d0*v*omumv**(-1)*pi**2
     .        + 4.0d0/3.0d0*v*upv**(-1)
     .        + 5.0d0/3.0d0*v*upv**(-1)*pi*ci
     .        - 11.0d0/12.0d0*v
     .        - 4.0d0/3.0d0*v*pi*ci
     .        - 1.0d0/3.0d0*v*pi**3*ci
     .        + 2.0d0*v*zeta3
     .        + 4.0d0/3.0d0*v**2*omu**(-2)*pi*ci
     .        - 5.0d0/36.0d0*v**2*omumv**(-2)*pi**2
     .        - 5.0d0/6.0d0*v**2*omumv**(-1)*pi*ci
     .        + 4.0d0/3.0d0*v**2*upv**(-2)*pi*ci
     .        - 7.0d0/4.0d0*v**2*pi*ci
     .        + 1.0d0/12.0d0*v**2*pi**3*ci
         col1 = col1 - 1.0d0/2.0d0*v**2*zeta3
     .        - 1.0d0/18.0d0*v**3*pi**3*ci
     .        + 1.0d0/3.0d0*v**3*zeta3
     .        + 337.0d0/108.0d0*u
     .        + 11.0d0/24.0d0*u*pi*ci
     .        + 101.0d0/72.0d0*u*pi**2
     .        + 2.0d0*u*zeta3
     .        - 337.0d0/108.0d0*u*v
     .        + 49.0d0/24.0d0*u*v*pi*ci
     .        - 65.0d0/72.0d0*u*v*pi**2
     .        + 1.0d0/6.0d0*u*v*pi**3*ci
     .        - u*v*zeta3
     .        - 5.0d0/2.0d0*u*v**2*pi*ci
     .        - 4.0d0/9.0d0*u*v**2*pi**2
     .        - 1.0d0/6.0d0*u*v**2*pi**3*ci
         col1 = col1 + u*v**2*zeta3
     .        - 337.0d0/108.0d0*u**2
     .        - 11.0d0/24.0d0*u**2*pi*ci
     .        - 221.0d0/72.0d0*u**2*pi**2
     .        - 1.0d0/2.0d0*u**2*zeta3
     .        - 5.0d0/2.0d0*u**2*v*pi*ci
     .        + 26.0d0/9.0d0*u**2*v*pi**2
     .        - 1.0d0/6.0d0*u**2*v*pi**3*ci
     .        + u**2*v*zeta3
     .        - 5.0d0/3.0d0*u**2*v**2*pi**2
     .        + 10.0d0/3.0d0*u**3*pi**2
     .        + 1.0d0/3.0d0*u**3*zeta3
     .        - 10.0d0/3.0d0*u**3*v*pi**2
     .        - 5.0d0/3.0d0*u**4*pi**2
     .        + 185.0d0/24.0d0*HPL1(0)
         col1 = col1 - 5.0d0/6.0d0*HPL1(0)*u**(-1)*v
     .        + 5.0d0/6.0d0*HPL1(0)*u**(-1)*v**2
     .        - 5.0d0/6.0d0*HPL1(0)*omumv**(-2)*pi*ci
     .        - 5.0d0/12.0d0*HPL1(0)*omumv**(-1)*pi*ci
     .        + 1321.0d0/144.0d0*HPL1(0)*pi*ci
     .        - 11.0d0/96.0d0*HPL1(0)*pi**2
     .        + 1.0d0/4.0d0*HPL1(0)*pi**3*ci
     .        + 3.0d0/4.0d0*HPL1(0)*zeta3
     .        - 5.0d0/3.0d0*HPL1(0)*v*omu**(-1)
     .        + 5.0d0/3.0d0*HPL1(0)*v*omumv**(-2)*pi*ci
     .        - 5.0d0/6.0d0*HPL1(0)*v*omumv**(-1)
     .        + 5.0d0/12.0d0*HPL1(0)*v*omumv**(-1)*pi*ci
     .        - 5.0d0/3.0d0*HPL1(0)*v*upv**(-1)
     .        + 4.0d0/3.0d0*HPL1(0)*v
     .        - 1.0d0/6.0d0*HPL1(0)*v*pi*ci
         col1 = col1 - 4.0d0/3.0d0*HPL1(0)*v**2*omu**(-2)
     .        - 5.0d0/6.0d0*HPL1(0)*v**2*omumv**(-2)*pi*ci
     .        + 5.0d0/6.0d0*HPL1(0)*v**2*omumv**(-1)
     .        - 4.0d0/3.0d0*HPL1(0)*v**2*upv**(-2)
     .        + 7.0d0/4.0d0*HPL1(0)*v**2
     .        + 1.0d0/6.0d0*HPL1(0)*v**2*pi*ci
     .        + 55.0d0/72.0d0*HPL1(0)*u
     .        + 25.0d0/12.0d0*HPL1(0)*u*pi*ci
     .        - 2.0d0/3.0d0*HPL1(0)*u*pi**2
     .        - 235.0d0/72.0d0*HPL1(0)*u*v
     .        - 13.0d0/12.0d0*HPL1(0)*u*v*pi*ci
     .        + 5.0d0/2.0d0*HPL1(0)*u*v**2
     .        - 2.0d0/3.0d0*HPL1(0)*u*v**2*pi*ci
     .        - 55.0d0/72.0d0*HPL1(0)*u**2
     .        - 55.0d0/12.0d0*HPL1(0)*u**2*pi*ci
         col1 = col1 + 1.0d0/6.0d0*HPL1(0)*u**2*pi**2
     .        + 5.0d0/2.0d0*HPL1(0)*u**2*v
     .        + 13.0d0/3.0d0*HPL1(0)*u**2*v*pi*ci
     .        - 5.0d0/2.0d0*HPL1(0)*u**2*v**2*pi*ci
     .        + 5.0d0*HPL1(0)*u**3*pi*ci
     .        - 1.0d0/9.0d0*HPL1(0)*u**3*pi**2
     .        - 5.0d0*HPL1(0)*u**3*v*pi*ci
     .        - 5.0d0/2.0d0*HPL1(0)*u**4*pi*ci
     .        + 37.0d0/48.0d0*HPL1(0)*GPL1(0)
     .        - 5.0d0/6.0d0*HPL1(0)*GPL1(0)*omumv**(-2)
     .        - 5.0d0/12.0d0*HPL1(0)*GPL1(0)*omumv**(-1)
     .        - 55.0d0/24.0d0*HPL1(0)*GPL1(0)*pi*ci
     .        + 3.0d0/4.0d0*HPL1(0)*GPL1(0)*pi**2
     .        + 5.0d0/3.0d0*HPL1(0)*GPL1(0)*v*omumv**(-2)
     .        + 5.0d0/12.0d0*HPL1(0)*GPL1(0)*v*omumv**(-1)
         col1 = col1 - 5.0d0/6.0d0*HPL1(0)*GPL1(0)*v**2*omumv**(-2)
     .        + 25.0d0/12.0d0*HPL1(0)*GPL1(0)*u
     .        + HPL1(0)*GPL1(0)*u*pi*ci
     .        - 17.0d0/12.0d0*HPL1(0)*GPL1(0)*u*v
     .        - 2.0d0/3.0d0*HPL1(0)*GPL1(0)*u*v**2
     .        - 55.0d0/12.0d0*HPL1(0)*GPL1(0)*u**2
     .        - 1.0d0/4.0d0*HPL1(0)*GPL1(0)*u**2*pi*ci
     .        + 13.0d0/3.0d0*HPL1(0)*GPL1(0)*u**2*v
     .        - 5.0d0/2.0d0*HPL1(0)*GPL1(0)*u**2*v**2
     .        + 5.0d0*HPL1(0)*GPL1(0)*u**3
     .        + 1.0d0/6.0d0*HPL1(0)*GPL1(0)*u**3*pi*ci
     .        - 5.0d0*HPL1(0)*GPL1(0)*u**3*v
     .        - 5.0d0/2.0d0*HPL1(0)*GPL1(0)*u**4
     .        - 179.0d0/36.0d0*HPL1(0)*GPL1(1)
     .        + 5.0d0/6.0d0*HPL1(0)*GPL1(1)*u**(-2)
         col1 = col1 - 5.0d0/3.0d0*HPL1(0)*GPL1(1)*u**(-2)*v
     .        + 5.0d0/6.0d0*HPL1(0)*GPL1(1)*u**(-2)*v**2
     .        + 5.0d0/12.0d0*HPL1(0)*GPL1(1)*u**(-1)
     .        - 5.0d0/12.0d0*HPL1(0)*GPL1(1)*u**(-1)*v
     .        - 7.0d0/12.0d0*HPL1(0)*GPL1(1)*pi**2
     .        - 1.0d0/6.0d0*HPL1(0)*GPL1(1)*v
     .        + 1.0d0/6.0d0*HPL1(0)*GPL1(1)*v**2
     .        - 25.0d0/12.0d0*HPL1(0)*GPL1(1)*u
     .        + 7.0d0/4.0d0*HPL1(0)*GPL1(1)*u*v
     .        + 2.0d0/3.0d0*HPL1(0)*GPL1(1)*u*v**2
     .        + 55.0d0/12.0d0*HPL1(0)*GPL1(1)*u**2
     .        - 13.0d0/3.0d0*HPL1(0)*GPL1(1)*u**2*v
     .        + 5.0d0/2.0d0*HPL1(0)*GPL1(1)*u**2*v**2
     .        - 5.0d0*HPL1(0)*GPL1(1)*u**3
     .        + 5.0d0*HPL1(0)*GPL1(1)*u**3*v
         col1 = col1 + 5.0d0/2.0d0*HPL1(0)*GPL1(1)*u**4
     .        + 37.0d0/48.0d0*HPL1(0)*GPL1(2)
     .        - 5.0d0/6.0d0*HPL1(0)*GPL1(2)*u**(-2)
     .        + 5.0d0/3.0d0*HPL1(0)*GPL1(2)*u**(-2)*v
     .        - 5.0d0/6.0d0*HPL1(0)*GPL1(2)*u**(-2)*v**2
     .        - 5.0d0/12.0d0*HPL1(0)*GPL1(2)*u**(-1)
     .        + 5.0d0/12.0d0*HPL1(0)*GPL1(2)*u**(-1)*v
     .        + 11.0d0/8.0d0*HPL1(0)*GPL1(2)*pi*ci
     .        + 1.0d0/12.0d0*HPL1(0)*GPL1(2)*pi**2
     .        + HPL1(0)*GPL1(2)*v*pi*ci
     .        - 1.0d0/4.0d0*HPL1(0)*GPL1(2)*v**2*pi*ci
     .        + 1.0d0/6.0d0*HPL1(0)*GPL1(2)*v**3*pi*ci
     .        + 25.0d0/12.0d0*HPL1(0)*GPL1(2)*u
     .        + HPL1(0)*GPL1(2)*u*pi*ci
     .        - 17.0d0/12.0d0*HPL1(0)*GPL1(2)*u*v
         col1 = col1 - 1.0d0/2.0d0*HPL1(0)*GPL1(2)*u*v*pi*ci
     .        - 2.0d0/3.0d0*HPL1(0)*GPL1(2)*u*v**2
     .        + 1.0d0/2.0d0*HPL1(0)*GPL1(2)*u*v**2*pi*ci
     .        - 55.0d0/12.0d0*HPL1(0)*GPL1(2)*u**2
     .        - 1.0d0/4.0d0*HPL1(0)*GPL1(2)*u**2*pi*ci
     .        + 13.0d0/3.0d0*HPL1(0)*GPL1(2)*u**2*v
     .        + 1.0d0/2.0d0*HPL1(0)*GPL1(2)*u**2*v*pi*ci
     .        - 5.0d0/2.0d0*HPL1(0)*GPL1(2)*u**2*v**2
     .        + 5.0d0*HPL1(0)*GPL1(2)*u**3
     .        + 1.0d0/6.0d0*HPL1(0)*GPL1(2)*u**3*pi*ci
     .        - 5.0d0*HPL1(0)*GPL1(2)*u**3*v
     .        - 5.0d0/2.0d0*HPL1(0)*GPL1(2)*u**4
     .        - 179.0d0/36.0d0*HPL1(0)*GPL1(3)
     .        + 5.0d0/6.0d0*HPL1(0)*GPL1(3)*omumv**(-2)
     .        + 5.0d0/12.0d0*HPL1(0)*GPL1(3)*omumv**(-1)
         col1 = col1 + 11.0d0/6.0d0*HPL1(0)*GPL1(3)*pi*ci
     .        - 5.0d0/4.0d0*HPL1(0)*GPL1(3)*pi**2
     .        - 5.0d0/3.0d0*HPL1(0)*GPL1(3)*v*omumv**(-2)
     .        - 5.0d0/12.0d0*HPL1(0)*GPL1(3)*v*omumv**(-1)
     .        + 1.0d0/6.0d0*HPL1(0)*GPL1(3)*v
     .        - 2.0d0*HPL1(0)*GPL1(3)*v*pi*ci
     .        + 5.0d0/6.0d0*HPL1(0)*GPL1(3)*v**2*omumv**(-2)
     .        - 1.0d0/6.0d0*HPL1(0)*GPL1(3)*v**2
     .        + 1.0d0/2.0d0*HPL1(0)*GPL1(3)*v**2*pi*ci
     .        - 1.0d0/3.0d0*HPL1(0)*GPL1(3)*v**3*pi*ci
     .        - 25.0d0/12.0d0*HPL1(0)*GPL1(3)*u
     .        - 2.0d0*HPL1(0)*GPL1(3)*u*pi*ci
     .        + 13.0d0/12.0d0*HPL1(0)*GPL1(3)*u*v
     .        + HPL1(0)*GPL1(3)*u*v*pi*ci
     .        + 2.0d0/3.0d0*HPL1(0)*GPL1(3)*u*v**2
         col1 = col1 - HPL1(0)*GPL1(3)*u*v**2*pi*ci
     .        + 55.0d0/12.0d0*HPL1(0)*GPL1(3)*u**2
     .        + 1.0d0/2.0d0*HPL1(0)*GPL1(3)*u**2*pi*ci
     .        - 13.0d0/3.0d0*HPL1(0)*GPL1(3)*u**2*v
     .        - HPL1(0)*GPL1(3)*u**2*v*pi*ci
     .        + 5.0d0/2.0d0*HPL1(0)*GPL1(3)*u**2*v**2
     .        - 5.0d0*HPL1(0)*GPL1(3)*u**3
     .        - 1.0d0/3.0d0*HPL1(0)*GPL1(3)*u**3*pi*ci
     .        + 5.0d0*HPL1(0)*GPL1(3)*u**3*v
     .        + 5.0d0/2.0d0*HPL1(0)*GPL1(3)*u**4
     .        - 11.0d0/3.0d0*HPL1(0)*GPL2(0,0)
     .        - HPL1(0)*GPL2(0,0)*pi*ci
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(0,1)*pi*ci
     .        + HPL1(0)*GPL2(0,1)*u
     .        - 1.0d0/4.0d0*HPL1(0)*GPL2(0,1)*u**2
         col1 = col1 + 1.0d0/6.0d0*HPL1(0)*GPL2(0,1)*u**3
     .        - 77.0d0/24.0d0*HPL1(0)*GPL2(0,2)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(0,2)*pi*ci
     .        + 55.0d0/12.0d0*HPL1(0)*GPL2(0,3)
     .        + 3.0d0/2.0d0*HPL1(0)*GPL2(0,3)*pi*ci
     .        - HPL1(0)*GPL2(0,3)*u
     .        + 1.0d0/4.0d0*HPL1(0)*GPL2(0,3)*u**2
     .        - 1.0d0/6.0d0*HPL1(0)*GPL2(0,3)*u**3
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(1,0)*pi*ci
     .        + HPL1(0)*GPL2(1,0)*u
     .        - 1.0d0/4.0d0*HPL1(0)*GPL2(1,0)*u**2
     .        + 1.0d0/6.0d0*HPL1(0)*GPL2(1,0)*u**3
     .        - 2.0d0*HPL1(0)*GPL2(1,1)*u
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(1,1)*u**2
     .        - 1.0d0/3.0d0*HPL1(0)*GPL2(1,1)*u**3
         col1 = col1 + 55.0d0/12.0d0*HPL1(0)*GPL2(1,2)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(1,2)*pi*ci
     .        + HPL1(0)*GPL2(1,2)*u
     .        - 1.0d0/4.0d0*HPL1(0)*GPL2(1,2)*u**2
     .        + 1.0d0/6.0d0*HPL1(0)*GPL2(1,2)*u**3
     .        - 77.0d0/24.0d0*HPL1(0)*GPL2(2,0)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(2,0)*pi*ci
     .        + 11.0d0/3.0d0*HPL1(0)*GPL2(2,1)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(2,1)*pi*ci
     .        + HPL1(0)*GPL2(2,1)*v
     .        - 1.0d0/4.0d0*HPL1(0)*GPL2(2,1)*v**2
     .        + 1.0d0/6.0d0*HPL1(0)*GPL2(2,1)*v**3
     .        + HPL1(0)*GPL2(2,1)*u
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(2,1)*u*v
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(2,1)*u*v**2
         col1 = col1 - 1.0d0/4.0d0*HPL1(0)*GPL2(2,1)*u**2
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(2,1)*u**2*v
     .        + 1.0d0/6.0d0*HPL1(0)*GPL2(2,1)*u**3
     .        - 11.0d0/3.0d0*HPL1(0)*GPL2(2,2)
     .        + HPL1(0)*GPL2(2,2)*pi*ci
     .        + 11.0d0/12.0d0*HPL1(0)*GPL2(2,3)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(2,3)*pi*ci
     .        - HPL1(0)*GPL2(2,3)*v
     .        + 1.0d0/4.0d0*HPL1(0)*GPL2(2,3)*v**2
     .        - 1.0d0/6.0d0*HPL1(0)*GPL2(2,3)*v**3
     .        - HPL1(0)*GPL2(2,3)*u
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(2,3)*u*v
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(2,3)*u*v**2
     .        + 1.0d0/4.0d0*HPL1(0)*GPL2(2,3)*u**2
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(2,3)*u**2*v
         col1 = col1 - 1.0d0/6.0d0*HPL1(0)*GPL2(2,3)*u**3
     .        + 11.0d0/2.0d0*HPL1(0)*GPL2(3,0)
     .        + 5.0d0/2.0d0*HPL1(0)*GPL2(3,0)*pi*ci
     .        - HPL1(0)*GPL2(3,0)*v
     .        + 1.0d0/4.0d0*HPL1(0)*GPL2(3,0)*v**2
     .        - 1.0d0/6.0d0*HPL1(0)*GPL2(3,0)*v**3
     .        - HPL1(0)*GPL2(3,0)*u
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(3,0)*u*v
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(3,0)*u*v**2
     .        + 1.0d0/4.0d0*HPL1(0)*GPL2(3,0)*u**2
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(3,0)*u**2*v
     .        - 1.0d0/6.0d0*HPL1(0)*GPL2(3,0)*u**3
     .        + 11.0d0/12.0d0*HPL1(0)*GPL2(3,2)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(3,2)*pi*ci
     .        - HPL1(0)*GPL2(3,2)*v
         col1 = col1 + 1.0d0/4.0d0*HPL1(0)*GPL2(3,2)*v**2
     .        - 1.0d0/6.0d0*HPL1(0)*GPL2(3,2)*v**3
     .        - HPL1(0)*GPL2(3,2)*u
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(3,2)*u*v
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(3,2)*u*v**2
     .        + 1.0d0/4.0d0*HPL1(0)*GPL2(3,2)*u**2
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(3,2)*u**2*v
     .        - 1.0d0/6.0d0*HPL1(0)*GPL2(3,2)*u**3
     .        - 11.0d0/6.0d0*HPL1(0)*GPL2(3,3)
     .        - 2.0d0*HPL1(0)*GPL2(3,3)*pi*ci
     .        + 2.0d0*HPL1(0)*GPL2(3,3)*v
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(3,3)*v**2
     .        + 1.0d0/3.0d0*HPL1(0)*GPL2(3,3)*v**3
     .        + 2.0d0*HPL1(0)*GPL2(3,3)*u
     .        - HPL1(0)*GPL2(3,3)*u*v
         col1 = col1 + HPL1(0)*GPL2(3,3)*u*v**2
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(3,3)*u**2
     .        + HPL1(0)*GPL2(3,3)*u**2*v
     .        + 1.0d0/3.0d0*HPL1(0)*GPL2(3,3)*u**3
     .        - HPL1(0)*GPL3(0,0,1)
     .        + HPL1(0)*GPL3(0,0,3)
     .        - HPL1(0)*GPL3(0,1,0)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(0,1,2)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(0,2,1)
     .        - HPL1(0)*GPL3(0,2,2)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(0,2,3)
     .        + HPL1(0)*GPL3(0,3,0)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(0,3,2)
     .        - 2.0d0*HPL1(0)*GPL3(0,3,3)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(1,0,2)
         col1 = col1 + 2.0d0*HPL1(0)*GPL3(1,1,1)
     .        - 2.0d0*HPL1(0)*GPL3(1,1,2)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(1,2,0)
     .        - 2.0d0*HPL1(0)*GPL3(1,2,1)
     .        + 2.0d0*HPL1(0)*GPL3(1,2,2)
     .        - HPL1(0)*GPL3(2,0,0)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(2,0,1)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(2,0,3)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(2,1,0)
     .        - 2.0d0*HPL1(0)*GPL3(2,1,1)
     .        + HPL1(0)*GPL3(2,1,2)
     .        + HPL1(0)*GPL3(2,2,1)
     .        - HPL1(0)*GPL3(2,2,3)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(2,3,0)
     .        - HPL1(0)*GPL3(2,3,2)
         col1 = col1 + 2.0d0*HPL1(0)*GPL3(3,0,0)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(3,0,2)
     .        - 2.0d0*HPL1(0)*GPL3(3,0,3)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(3,2,0)
     .        - 2.0d0*HPL1(0)*GPL3(3,3,0)
     .        + 2.0d0*HPL1(0)*GPL3(3,3,3)
     .        + 19.0d0/8.0d0*HPL1(1)
     .        + 5.0d0/6.0d0*HPL1(1)*u**(-1)
     .        - 5.0d0/3.0d0*HPL1(1)*u**(-1)*v
     .        + 5.0d0/6.0d0*HPL1(1)*u**(-1)*v**2
     .        + 389.0d0/144.0d0*HPL1(1)*pi*ci
     .        - 55.0d0/288.0d0*HPL1(1)*pi**2
     .        + 1.0d0/8.0d0*HPL1(1)*pi**3*ci
     .        + 1.0d0/4.0d0*HPL1(1)*zeta3
     .        - 5.0d0/3.0d0*HPL1(1)*v*omu**(-1)
         col1 = col1 + 1.0d0/2.0d0*HPL1(1)*v
     .        - 4.0d0/3.0d0*HPL1(1)*v**2*omu**(-2)
     .        + 7.0d0/4.0d0*HPL1(1)*v**2
     .        + 203.0d0/72.0d0*HPL1(1)*u
     .        + 2.0d0*HPL1(1)*u*pi*ci
     .        - 155.0d0/72.0d0*HPL1(1)*u*v
     .        - 4.0d0/3.0d0*HPL1(1)*u*v*pi*ci
     .        + 5.0d0/2.0d0*HPL1(1)*u*v**2
     .        - 2.0d0/3.0d0*HPL1(1)*u*v**2*pi*ci
     .        - 281.0d0/72.0d0*HPL1(1)*u**2
     .        - 9.0d0/2.0d0*HPL1(1)*u**2*pi*ci
     .        + 5.0d0*HPL1(1)*u**2*v
     .        + 13.0d0/3.0d0*HPL1(1)*u**2*v*pi*ci
     .        - 5.0d0/2.0d0*HPL1(1)*u**2*v**2*pi*ci
     .        + 5.0d0/2.0d0*HPL1(1)*u**3
         col1 = col1 + 5.0d0*HPL1(1)*u**3*pi*ci
     .        - 5.0d0*HPL1(1)*u**3*v*pi*ci
     .        - 5.0d0/2.0d0*HPL1(1)*u**4*pi*ci
     .        + 49.0d0/48.0d0*HPL1(1)*GPL1(0)
     .        - 11.0d0/8.0d0*HPL1(1)*GPL1(0)*pi*ci
     .        + 3.0d0/8.0d0*HPL1(1)*GPL1(0)*pi**2
     .        + 2.0d0*HPL1(1)*GPL1(0)*u
     .        - 4.0d0/3.0d0*HPL1(1)*GPL1(0)*u*v
     .        - 2.0d0/3.0d0*HPL1(1)*GPL1(0)*u*v**2
     .        - 9.0d0/2.0d0*HPL1(1)*GPL1(0)*u**2
     .        + 13.0d0/3.0d0*HPL1(1)*GPL1(0)*u**2*v
     .        - 5.0d0/2.0d0*HPL1(1)*GPL1(0)*u**2*v**2
     .        + 5.0d0*HPL1(1)*GPL1(0)*u**3
     .        - 5.0d0*HPL1(1)*GPL1(0)*u**3*v
     .        - 5.0d0/2.0d0*HPL1(1)*GPL1(0)*u**4
         col1 = col1 - 179.0d0/36.0d0*HPL1(1)*GPL1(1)
     .        + 5.0d0/6.0d0*HPL1(1)*GPL1(1)*u**(-2)
     .        - 5.0d0/3.0d0*HPL1(1)*GPL1(1)*u**(-2)*v
     .        + 5.0d0/6.0d0*HPL1(1)*GPL1(1)*u**(-2)*v**2
     .        + 5.0d0/12.0d0*HPL1(1)*GPL1(1)*u**(-1)
     .        - 5.0d0/12.0d0*HPL1(1)*GPL1(1)*u**(-1)*v
     .        - 11.0d0/12.0d0*HPL1(1)*GPL1(1)*pi*ci
     .        - 5.0d0/12.0d0*HPL1(1)*GPL1(1)*pi**2
     .        - 1.0d0/6.0d0*HPL1(1)*GPL1(1)*v
     .        + 1.0d0/6.0d0*HPL1(1)*GPL1(1)*v**2
     .        - 25.0d0/12.0d0*HPL1(1)*GPL1(1)*u
     .        + HPL1(1)*GPL1(1)*u*pi*ci
     .        + 7.0d0/4.0d0*HPL1(1)*GPL1(1)*u*v
     .        + 2.0d0/3.0d0*HPL1(1)*GPL1(1)*u*v**2
     .        + 55.0d0/12.0d0*HPL1(1)*GPL1(1)*u**2
         col1 = col1 - 1.0d0/4.0d0*HPL1(1)*GPL1(1)*u**2*pi*ci
     .        - 13.0d0/3.0d0*HPL1(1)*GPL1(1)*u**2*v
     .        + 5.0d0/2.0d0*HPL1(1)*GPL1(1)*u**2*v**2
     .        - 5.0d0*HPL1(1)*GPL1(1)*u**3
     .        + 1.0d0/6.0d0*HPL1(1)*GPL1(1)*u**3*pi*ci
     .        + 5.0d0*HPL1(1)*GPL1(1)*u**3*v
     .        + 5.0d0/2.0d0*HPL1(1)*GPL1(1)*u**4
     .        - 121.0d0/48.0d0*HPL1(1)*GPL1(2)
     .        - 11.0d0/6.0d0*HPL1(1)*GPL1(2)*pi*ci
     .        - 1.0d0/12.0d0*HPL1(1)*GPL1(2)*pi**2
     .        + 11.0d0/12.0d0*HPL1(1)*GPL1(3)*pi*ci
     .        - 1.0d0/2.0d0*HPL1(1)*GPL1(3)*pi**2
     .        - HPL1(1)*GPL1(3)*v*pi*ci
     .        + 1.0d0/4.0d0*HPL1(1)*GPL1(3)*v**2*pi*ci
     .        - 1.0d0/6.0d0*HPL1(1)*GPL1(3)*v**3*pi*ci
         col1 = col1 - HPL1(1)*GPL1(3)*u*pi*ci
     .        + 1.0d0/2.0d0*HPL1(1)*GPL1(3)*u*v*pi*ci
     .        - 1.0d0/2.0d0*HPL1(1)*GPL1(3)*u*v**2*pi*ci
     .        + 1.0d0/4.0d0*HPL1(1)*GPL1(3)*u**2*pi*ci
     .        - 1.0d0/2.0d0*HPL1(1)*GPL1(3)*u**2*v*pi*ci
     .        - 1.0d0/6.0d0*HPL1(1)*GPL1(3)*u**3*pi*ci
     .        - 11.0d0/6.0d0*HPL1(1)*GPL2(0,0)
     .        - 1.0d0/2.0d0*HPL1(1)*GPL2(0,0)*pi*ci
     .        + HPL1(1)*GPL2(0,1)*u
     .        - 1.0d0/4.0d0*HPL1(1)*GPL2(0,1)*u**2
     .        + 1.0d0/6.0d0*HPL1(1)*GPL2(0,1)*u**3
     .        - 11.0d0/6.0d0*HPL1(1)*GPL2(0,2)
     .        - 1.0d0/2.0d0*HPL1(1)*GPL2(0,2)*pi*ci
     .        + 1.0d0/2.0d0*HPL1(1)*GPL2(0,3)*pi*ci
     .        + HPL1(1)*GPL2(1,0)*pi*ci
         col1 = col1 + HPL1(1)*GPL2(1,0)*u
     .        - 1.0d0/4.0d0*HPL1(1)*GPL2(1,0)*u**2
     .        + 1.0d0/6.0d0*HPL1(1)*GPL2(1,0)*u**3
     .        - 2.0d0*HPL1(1)*GPL2(1,1)*pi*ci
     .        - 2.0d0*HPL1(1)*GPL2(1,1)*u
     .        + 1.0d0/2.0d0*HPL1(1)*GPL2(1,1)*u**2
     .        - 1.0d0/3.0d0*HPL1(1)*GPL2(1,1)*u**3
     .        + 11.0d0/3.0d0*HPL1(1)*GPL2(1,2)
     .        + HPL1(1)*GPL2(1,2)*pi*ci
     .        - 11.0d0/6.0d0*HPL1(1)*GPL2(2,0)
     .        - 1.0d0/2.0d0*HPL1(1)*GPL2(2,0)*pi*ci
     .        + 11.0d0/3.0d0*HPL1(1)*GPL2(2,1)
     .        + 1.0d0/2.0d0*HPL1(1)*GPL2(2,1)*pi*ci
     .        + HPL1(1)*GPL2(2,1)*v
     .        - 1.0d0/4.0d0*HPL1(1)*GPL2(2,1)*v**2
         col1 = col1 + 1.0d0/6.0d0*HPL1(1)*GPL2(2,1)*v**3
     .        + HPL1(1)*GPL2(2,1)*u
     .        - 1.0d0/2.0d0*HPL1(1)*GPL2(2,1)*u*v
     .        + 1.0d0/2.0d0*HPL1(1)*GPL2(2,1)*u*v**2
     .        - 1.0d0/4.0d0*HPL1(1)*GPL2(2,1)*u**2
     .        + 1.0d0/2.0d0*HPL1(1)*GPL2(2,1)*u**2*v
     .        + 1.0d0/6.0d0*HPL1(1)*GPL2(2,1)*u**3
     .        - HPL1(1)*GPL2(2,3)*pi*ci
     .        + 11.0d0/12.0d0*HPL1(1)*GPL2(3,0)
     .        + HPL1(1)*GPL2(3,0)*pi*ci
     .        - HPL1(1)*GPL2(3,0)*v
     .        + 1.0d0/4.0d0*HPL1(1)*GPL2(3,0)*v**2
     .        - 1.0d0/6.0d0*HPL1(1)*GPL2(3,0)*v**3
     .        - HPL1(1)*GPL2(3,0)*u
     .        + 1.0d0/2.0d0*HPL1(1)*GPL2(3,0)*u*v
         col1 = col1 - 1.0d0/2.0d0*HPL1(1)*GPL2(3,0)*u*v**2
     .        + 1.0d0/4.0d0*HPL1(1)*GPL2(3,0)*u**2
     .        - 1.0d0/2.0d0*HPL1(1)*GPL2(3,0)*u**2*v
     .        - 1.0d0/6.0d0*HPL1(1)*GPL2(3,0)*u**3
     .        - HPL1(1)*GPL3(0,0,1)
     .        - 1.0d0/2.0d0*HPL1(1)*GPL3(0,0,2)
     .        - HPL1(1)*GPL3(0,1,0)
     .        + HPL1(1)*GPL3(0,1,2)
     .        - 1.0d0/2.0d0*HPL1(1)*GPL3(0,2,0)
     .        + 1.0d0/2.0d0*HPL1(1)*GPL3(0,2,1)
     .        + 1.0d0/2.0d0*HPL1(1)*GPL3(0,3,0)
     .        + HPL1(1)*GPL3(1,0,2)
     .        + 2.0d0*HPL1(1)*GPL3(1,1,1)
     .        - 2.0d0*HPL1(1)*GPL3(1,1,2)
     .        + HPL1(1)*GPL3(1,2,0)
         col1 = col1 - 2.0d0*HPL1(1)*GPL3(1,2,1)
     .        - 1.0d0/2.0d0*HPL1(1)*GPL3(2,0,0)
     .        + 1.0d0/2.0d0*HPL1(1)*GPL3(2,0,1)
     .        + 1.0d0/2.0d0*HPL1(1)*GPL3(2,1,0)
     .        - 2.0d0*HPL1(1)*GPL3(2,1,1)
     .        + HPL1(1)*GPL3(2,2,1)
     .        - HPL1(1)*GPL3(2,3,0)
     .        + HPL1(1)*GPL3(3,0,0)
     .        + 1099.0d0/144.0d0*HPL2(0,0)
     .        + 5.0d0/6.0d0*HPL2(0,0)*omumv**(-2)
     .        + 5.0d0/12.0d0*HPL2(0,0)*omumv**(-1)
     .        + 77.0d0/12.0d0*HPL2(0,0)*pi*ci
     .        - 5.0d0/4.0d0*HPL2(0,0)*pi**2
     .        - 5.0d0/3.0d0*HPL2(0,0)*v*omumv**(-2)
     .        - 5.0d0/12.0d0*HPL2(0,0)*v*omumv**(-1)
         col1 = col1 + 1.0d0/6.0d0*HPL2(0,0)*v
     .        - HPL2(0,0)*v*pi*ci
     .        + 5.0d0/6.0d0*HPL2(0,0)*v**2*omumv**(-2)
     .        - 1.0d0/6.0d0*HPL2(0,0)*v**2
     .        + 1.0d0/4.0d0*HPL2(0,0)*v**2*pi*ci
     .        - 1.0d0/6.0d0*HPL2(0,0)*v**3*pi*ci
     .        - 25.0d0/12.0d0*HPL2(0,0)*u
     .        - 2.0d0*HPL2(0,0)*u*pi*ci
     .        + 13.0d0/12.0d0*HPL2(0,0)*u*v
     .        + 1.0d0/2.0d0*HPL2(0,0)*u*v*pi*ci
     .        + 2.0d0/3.0d0*HPL2(0,0)*u*v**2
     .        - 1.0d0/2.0d0*HPL2(0,0)*u*v**2*pi*ci
     .        + 55.0d0/12.0d0*HPL2(0,0)*u**2
     .        + 1.0d0/2.0d0*HPL2(0,0)*u**2*pi*ci
     .        - 13.0d0/3.0d0*HPL2(0,0)*u**2*v
         col1 = col1 - 1.0d0/2.0d0*HPL2(0,0)*u**2*v*pi*ci
     .        + 5.0d0/2.0d0*HPL2(0,0)*u**2*v**2
     .        - 5.0d0*HPL2(0,0)*u**3
     .        - 1.0d0/3.0d0*HPL2(0,0)*u**3*pi*ci
     .        + 5.0d0*HPL2(0,0)*u**3*v
     .        + 5.0d0/2.0d0*HPL2(0,0)*u**4
     .        + 121.0d0/12.0d0*HPL2(0,0)*GPL1(0)
     .        + 5.0d0/2.0d0*HPL2(0,0)*GPL1(0)*pi*ci
     .        - HPL2(0,0)*GPL1(0)*u
     .        + 1.0d0/4.0d0*HPL2(0,0)*GPL1(0)*u**2
     .        - 1.0d0/6.0d0*HPL2(0,0)*GPL1(0)*u**3
     .        - 11.0d0/2.0d0*HPL2(0,0)*GPL1(1)
     .        - HPL2(0,0)*GPL1(1)*pi*ci
     .        + 77.0d0/12.0d0*HPL2(0,0)*GPL1(2)
     .        + 1.0d0/2.0d0*HPL2(0,0)*GPL1(2)*pi*ci
         col1 = col1 - HPL2(0,0)*GPL1(2)*v
     .        + 1.0d0/4.0d0*HPL2(0,0)*GPL1(2)*v**2
     .        - 1.0d0/6.0d0*HPL2(0,0)*GPL1(2)*v**3
     .        - HPL2(0,0)*GPL1(2)*u
     .        + 1.0d0/2.0d0*HPL2(0,0)*GPL1(2)*u*v
     .        - 1.0d0/2.0d0*HPL2(0,0)*GPL1(2)*u*v**2
     .        + 1.0d0/4.0d0*HPL2(0,0)*GPL1(2)*u**2
     .        - 1.0d0/2.0d0*HPL2(0,0)*GPL1(2)*u**2*v
     .        - 1.0d0/6.0d0*HPL2(0,0)*GPL1(2)*u**3
     .        - 22.0d0/3.0d0*HPL2(0,0)*GPL1(3)
     .        - 3.0d0*HPL2(0,0)*GPL1(3)*pi*ci
     .        + 2.0d0*HPL2(0,0)*GPL1(3)*v
     .        - 1.0d0/2.0d0*HPL2(0,0)*GPL1(3)*v**2
     .        + 1.0d0/3.0d0*HPL2(0,0)*GPL1(3)*v**3
     .        + 2.0d0*HPL2(0,0)*GPL1(3)*u
         col1 = col1 - HPL2(0,0)*GPL1(3)*u*v
     .        + HPL2(0,0)*GPL1(3)*u*v**2
     .        - 1.0d0/2.0d0*HPL2(0,0)*GPL1(3)*u**2
     .        + HPL2(0,0)*GPL1(3)*u**2*v
     .        + 1.0d0/3.0d0*HPL2(0,0)*GPL1(3)*u**3
     .        + 2.0d0*HPL2(0,0)*GPL2(0,0)
     .        + 1.0d0/2.0d0*HPL2(0,0)*GPL2(0,2)
     .        - 2.0d0*HPL2(0,0)*GPL2(0,3)
     .        - HPL2(0,0)*GPL2(1,2)
     .        + 1.0d0/2.0d0*HPL2(0,0)*GPL2(2,0)
     .        - 3.0d0*HPL2(0,0)*GPL2(3,0)
     .        + 2.0d0*HPL2(0,0)*GPL2(3,3)
     .        + 605.0d0/144.0d0*HPL2(0,1)
     .        + 77.0d0/24.0d0*HPL2(0,1)*pi*ci
     .        - 1.0d0/2.0d0*HPL2(0,1)*pi**2
         col1 = col1 + 1.0d0/6.0d0*HPL2(0,1)*v
     .        - 1.0d0/6.0d0*HPL2(0,1)*v**2
     .        - HPL2(0,1)*u*pi*ci
     .        - 1.0d0/3.0d0*HPL2(0,1)*u*v
     .        + 1.0d0/4.0d0*HPL2(0,1)*u**2*pi*ci
     .        - 1.0d0/6.0d0*HPL2(0,1)*u**3*pi*ci
     .        + 77.0d0/24.0d0*HPL2(0,1)*GPL1(0)
     .        + HPL2(0,1)*GPL1(0)*pi*ci
     .        - HPL2(0,1)*GPL1(0)*u
     .        + 1.0d0/4.0d0*HPL2(0,1)*GPL1(0)*u**2
     .        - 1.0d0/6.0d0*HPL2(0,1)*GPL1(0)*u**3
     .        - 55.0d0/12.0d0*HPL2(0,1)*GPL1(1)
     .        - 1.0d0/2.0d0*HPL2(0,1)*GPL1(1)*pi*ci
     .        + HPL2(0,1)*GPL1(1)*u
     .        - 1.0d0/4.0d0*HPL2(0,1)*GPL1(1)*u**2
         col1 = col1 + 1.0d0/6.0d0*HPL2(0,1)*GPL1(1)*u**3
     .        - 1.0d0/2.0d0*HPL2(0,1)*GPL1(2)*pi*ci
     .        - HPL2(0,1)*GPL1(2)*v
     .        + 1.0d0/4.0d0*HPL2(0,1)*GPL1(2)*v**2
     .        - 1.0d0/6.0d0*HPL2(0,1)*GPL1(2)*v**3
     .        - HPL2(0,1)*GPL1(2)*u
     .        + 1.0d0/2.0d0*HPL2(0,1)*GPL1(2)*u*v
     .        - 1.0d0/2.0d0*HPL2(0,1)*GPL1(2)*u*v**2
     .        + 1.0d0/4.0d0*HPL2(0,1)*GPL1(2)*u**2
     .        - 1.0d0/2.0d0*HPL2(0,1)*GPL1(2)*u**2*v
     .        - 1.0d0/6.0d0*HPL2(0,1)*GPL1(2)*u**3
     .        - 11.0d0/12.0d0*HPL2(0,1)*GPL1(3)
     .        - 1.0d0/2.0d0*HPL2(0,1)*GPL1(3)*pi*ci
     .        + HPL2(0,1)*GPL1(3)*v
     .        - 1.0d0/4.0d0*HPL2(0,1)*GPL1(3)*v**2
         col1 = col1 + 1.0d0/6.0d0*HPL2(0,1)*GPL1(3)*v**3
     .        + HPL2(0,1)*GPL1(3)*u
     .        - 1.0d0/2.0d0*HPL2(0,1)*GPL1(3)*u*v
     .        + 1.0d0/2.0d0*HPL2(0,1)*GPL1(3)*u*v**2
     .        - 1.0d0/4.0d0*HPL2(0,1)*GPL1(3)*u**2
     .        + 1.0d0/2.0d0*HPL2(0,1)*GPL1(3)*u**2*v
     .        + 1.0d0/6.0d0*HPL2(0,1)*GPL1(3)*u**3
     .        + HPL2(0,1)*GPL2(0,0)
     .        - 1.0d0/2.0d0*HPL2(0,1)*GPL2(0,1)
     .        + 1.0d0/2.0d0*HPL2(0,1)*GPL2(0,2)
     .        - 1.0d0/2.0d0*HPL2(0,1)*GPL2(0,3)
     .        - 1.0d0/2.0d0*HPL2(0,1)*GPL2(1,0)
     .        - 1.0d0/2.0d0*HPL2(0,1)*GPL2(2,0)
     .        + HPL2(0,1)*GPL2(2,1)
     .        - HPL2(0,1)*GPL2(2,2)
         col1 = col1 + HPL2(0,1)*GPL2(2,3)
     .        - 1.0d0/2.0d0*HPL2(0,1)*GPL2(3,0)
     .        + 785.0d0/144.0d0*HPL2(1,0)
     .        - 5.0d0/6.0d0*HPL2(1,0)*omumv**(-2)
     .        - 5.0d0/12.0d0*HPL2(1,0)*omumv**(-1)
     .        + 77.0d0/24.0d0*HPL2(1,0)*pi*ci
     .        - 1.0d0/2.0d0*HPL2(1,0)*pi**2
     .        + 5.0d0/3.0d0*HPL2(1,0)*v*omumv**(-2)
     .        + 5.0d0/12.0d0*HPL2(1,0)*v*omumv**(-1)
     .        - HPL2(1,0)*v*pi*ci
     .        - 5.0d0/6.0d0*HPL2(1,0)*v**2*omumv**(-2)
     .        + 1.0d0/4.0d0*HPL2(1,0)*v**2*pi*ci
     .        - 1.0d0/6.0d0*HPL2(1,0)*v**3*pi*ci
     .        - 23.0d0/12.0d0*HPL2(1,0)*u
     .        - HPL2(1,0)*u*pi*ci
         col1 = col1 + 5.0d0/4.0d0*HPL2(1,0)*u*v
     .        + 1.0d0/2.0d0*HPL2(1,0)*u*v*pi*ci
     .        + 2.0d0/3.0d0*HPL2(1,0)*u*v**2
     .        - 1.0d0/2.0d0*HPL2(1,0)*u*v**2*pi*ci
     .        + 53.0d0/12.0d0*HPL2(1,0)*u**2
     .        + 1.0d0/4.0d0*HPL2(1,0)*u**2*pi*ci
     .        - 13.0d0/3.0d0*HPL2(1,0)*u**2*v
     .        - 1.0d0/2.0d0*HPL2(1,0)*u**2*v*pi*ci
     .        + 5.0d0/2.0d0*HPL2(1,0)*u**2*v**2
     .        - 5.0d0*HPL2(1,0)*u**3
     .        - 1.0d0/6.0d0*HPL2(1,0)*u**3*pi*ci
     .        + 5.0d0*HPL2(1,0)*u**3*v
     .        + 5.0d0/2.0d0*HPL2(1,0)*u**4
     .        + 77.0d0/24.0d0*HPL2(1,0)*GPL1(0)
     .        + HPL2(1,0)*GPL1(0)*pi*ci
         col1 = col1 - 55.0d0/12.0d0*HPL2(1,0)*GPL1(1)
     .        - 1.0d0/2.0d0*HPL2(1,0)*GPL1(1)*pi*ci
     .        - HPL2(1,0)*GPL1(1)*u
     .        + 1.0d0/4.0d0*HPL2(1,0)*GPL1(1)*u**2
     .        - 1.0d0/6.0d0*HPL2(1,0)*GPL1(1)*u**3
     .        + 11.0d0/3.0d0*HPL2(1,0)*GPL1(2)
     .        - 1.0d0/2.0d0*HPL2(1,0)*GPL1(2)*pi*ci
     .        - 11.0d0/12.0d0*HPL2(1,0)*GPL1(3)
     .        - 1.0d0/2.0d0*HPL2(1,0)*GPL1(3)*pi*ci
     .        + HPL2(1,0)*GPL1(3)*v
     .        - 1.0d0/4.0d0*HPL2(1,0)*GPL1(3)*v**2
     .        + 1.0d0/6.0d0*HPL2(1,0)*GPL1(3)*v**3
     .        + HPL2(1,0)*GPL1(3)*u
     .        - 1.0d0/2.0d0*HPL2(1,0)*GPL1(3)*u*v
     .        + 1.0d0/2.0d0*HPL2(1,0)*GPL1(3)*u*v**2
         col1 = col1 - 1.0d0/4.0d0*HPL2(1,0)*GPL1(3)*u**2
     .        + 1.0d0/2.0d0*HPL2(1,0)*GPL1(3)*u**2*v
     .        + 1.0d0/6.0d0*HPL2(1,0)*GPL1(3)*u**3
     .        + HPL2(1,0)*GPL2(0,0)
     .        - 1.0d0/2.0d0*HPL2(1,0)*GPL2(0,1)
     .        + 1.0d0/2.0d0*HPL2(1,0)*GPL2(0,2)
     .        - 1.0d0/2.0d0*HPL2(1,0)*GPL2(0,3)
     .        - 1.0d0/2.0d0*HPL2(1,0)*GPL2(1,0)
     .        + 2.0d0*HPL2(1,0)*GPL2(1,1)
     .        - 2.0d0*HPL2(1,0)*GPL2(1,2)
     .        - 1.0d0/2.0d0*HPL2(1,0)*GPL2(2,0)
     .        - HPL2(1,0)*GPL2(2,1)
     .        + HPL2(1,0)*GPL2(2,2)
     .        + HPL2(1,0)*GPL2(2,3)
     .        - 1.0d0/2.0d0*HPL2(1,0)*GPL2(3,0)
         col1 = col1 + 121.0d0/48.0d0*HPL2(1,1)
     .        + 11.0d0/6.0d0*HPL2(1,1)*pi*ci
     .        - 1.0d0/4.0d0*HPL2(1,1)*pi**2
     .        + 11.0d0/6.0d0*HPL2(1,1)*GPL1(0)
     .        + 1.0d0/2.0d0*HPL2(1,1)*GPL1(0)*pi*ci
     .        - 11.0d0/3.0d0*HPL2(1,1)*GPL1(1)
     .        - HPL2(1,1)*GPL1(1)*pi*ci
     .        + 1.0d0/2.0d0*HPL2(1,1)*GPL2(0,0)
     .        - HPL2(1,1)*GPL2(0,1)
     .        - HPL2(1,1)*GPL2(1,0)
     .        + 2.0d0*HPL2(1,1)*GPL2(1,1)
     .        - 143.0d0/12.0d0*HPL3(0,0,0)
     .        - 3.0d0*HPL3(0,0,0)*pi*ci
     .        + HPL3(0,0,0)*v
     .        - 1.0d0/4.0d0*HPL3(0,0,0)*v**2
         col1 = col1 + 1.0d0/6.0d0*HPL3(0,0,0)*v**3
     .        + 2.0d0*HPL3(0,0,0)*u
     .        - 1.0d0/2.0d0*HPL3(0,0,0)*u*v
     .        + 1.0d0/2.0d0*HPL3(0,0,0)*u*v**2
     .        - 1.0d0/2.0d0*HPL3(0,0,0)*u**2
     .        + 1.0d0/2.0d0*HPL3(0,0,0)*u**2*v
     .        + 1.0d0/3.0d0*HPL3(0,0,0)*u**3
     .        - 3.0d0*HPL3(0,0,0)*GPL1(0)
     .        + HPL3(0,0,0)*GPL1(1)
     .        - HPL3(0,0,0)*GPL1(2)
     .        + 3.0d0*HPL3(0,0,0)*GPL1(3)
     .        - 11.0d0/12.0d0*HPL3(0,0,1)
     .        - 1.0d0/2.0d0*HPL3(0,0,1)*pi*ci
     .        + HPL3(0,0,1)*v
     .        - 1.0d0/4.0d0*HPL3(0,0,1)*v**2
         col1 = col1 + 1.0d0/6.0d0*HPL3(0,0,1)*v**3
     .        + HPL3(0,0,1)*u
     .        - 1.0d0/2.0d0*HPL3(0,0,1)*u*v
     .        + 1.0d0/2.0d0*HPL3(0,0,1)*u*v**2
     .        - 1.0d0/4.0d0*HPL3(0,0,1)*u**2
     .        + 1.0d0/2.0d0*HPL3(0,0,1)*u**2*v
     .        + 1.0d0/6.0d0*HPL3(0,0,1)*u**3
     .        - 1.0d0/2.0d0*HPL3(0,0,1)*GPL1(0)
     .        + HPL3(0,0,1)*GPL1(1)
     .        - 11.0d0/12.0d0*HPL3(0,1,0)
     .        - 1.0d0/2.0d0*HPL3(0,1,0)*pi*ci
     .        + HPL3(0,1,0)*u
     .        - 1.0d0/4.0d0*HPL3(0,1,0)*u**2
     .        + 1.0d0/6.0d0*HPL3(0,1,0)*u**3
     .        - 1.0d0/2.0d0*HPL3(0,1,0)*GPL1(0)
         col1 = col1 + HPL3(0,1,0)*GPL1(1)
     .        - 11.0d0/12.0d0*HPL3(1,0,0)
     .        - 1.0d0/2.0d0*HPL3(1,0,0)*pi*ci
     .        + HPL3(1,0,0)*v
     .        - 1.0d0/4.0d0*HPL3(1,0,0)*v**2
     .        + 1.0d0/6.0d0*HPL3(1,0,0)*v**3
     .        + HPL3(1,0,0)*u
     .        - 1.0d0/2.0d0*HPL3(1,0,0)*u*v
     .        + 1.0d0/2.0d0*HPL3(1,0,0)*u*v**2
     .        - 1.0d0/4.0d0*HPL3(1,0,0)*u**2
     .        + 1.0d0/2.0d0*HPL3(1,0,0)*u**2*v
     .        + 1.0d0/6.0d0*HPL3(1,0,0)*u**3
     .        - 1.0d0/2.0d0*HPL3(1,0,0)*GPL1(0)
     .        + HPL3(1,0,0)*GPL1(1)
     .        + HPL3(1,0,1)*v
         col1 = col1 - 1.0d0/4.0d0*HPL3(1,0,1)*v**2
     .        + 1.0d0/6.0d0*HPL3(1,0,1)*v**3
     .        + HPL3(1,0,1)*u
     .        - 1.0d0/2.0d0*HPL3(1,0,1)*u*v
     .        + 1.0d0/2.0d0*HPL3(1,0,1)*u*v**2
     .        - 1.0d0/4.0d0*HPL3(1,0,1)*u**2
     .        + 1.0d0/2.0d0*HPL3(1,0,1)*u**2*v
     .        + 1.0d0/6.0d0*HPL3(1,0,1)*u**3
     .        - 11.0d0/6.0d0*HPL3(1,1,0)
     .        + 2.0d0*HPL3(1,1,0)*GPL1(1)
     .        - 2.0d0*HPL3(1,1,0)*GPL1(2)
     .        + 3.0d0*HPL4(0,0,0,0)
     .        - 91.0d0/24.0d0*GPL1(0)
     .        - 5.0d0/6.0d0*GPL1(0)*omumv**(-1)
     .        - 389.0d0/144.0d0*GPL1(0)*pi*ci
         col1 = col1 - 77.0d0/288.0d0*GPL1(0)*pi**2
     .        - 1.0d0/8.0d0*GPL1(0)*pi**3*ci
     .        - 1.0d0/4.0d0*GPL1(0)*zeta3
     .        + 5.0d0/3.0d0*GPL1(0)*v*omumv**(-1)
     .        + 5.0d0/3.0d0*GPL1(0)*v*upv**(-1)
     .        - 5.0d0/6.0d0*GPL1(0)*v
     .        - 5.0d0/6.0d0*GPL1(0)*v**2*omumv**(-1)
     .        + 4.0d0/3.0d0*GPL1(0)*v**2*upv**(-2)
     .        + 181.0d0/72.0d0*GPL1(0)*u
     .        - 2.0d0*GPL1(0)*u*pi*ci
     .        + 1.0d0/3.0d0*GPL1(0)*u*pi**2
     .        + 47.0d0/72.0d0*GPL1(0)*u*v
     .        + 4.0d0/3.0d0*GPL1(0)*u*v*pi*ci
     .        + 2.0d0/3.0d0*GPL1(0)*u*v**2*pi*ci
     .        - 259.0d0/72.0d0*GPL1(0)*u**2
         col1 = col1 + 9.0d0/2.0d0*GPL1(0)*u**2*pi*ci
     .        - 1.0d0/12.0d0*GPL1(0)*u**2*pi**2
     .        + 5.0d0/2.0d0*GPL1(0)*u**2*v
     .        - 13.0d0/3.0d0*GPL1(0)*u**2*v*pi*ci
     .        + 5.0d0/2.0d0*GPL1(0)*u**2*v**2*pi*ci
     .        + 5.0d0/2.0d0*GPL1(0)*u**3
     .        - 5.0d0*GPL1(0)*u**3*pi*ci
     .        + 1.0d0/18.0d0*GPL1(0)*u**3*pi**2
     .        + 5.0d0*GPL1(0)*u**3*v*pi*ci
     .        + 5.0d0/2.0d0*GPL1(0)*u**4*pi*ci
     .        - 5.0d0/6.0d0*GPL1(1)*u**(-2)*pi*ci
     .        + 5.0d0/3.0d0*GPL1(1)*u**(-2)*v*pi*ci
     .        - 5.0d0/6.0d0*GPL1(1)*u**(-2)*v**2*pi*ci
     .        - 5.0d0/12.0d0*GPL1(1)*u**(-1)*pi*ci
     .        + 5.0d0/12.0d0*GPL1(1)*u**(-1)*v*pi*ci
         col1 = col1 + 179.0d0/36.0d0*GPL1(1)*pi*ci
     .        - 11.0d0/4.0d0*GPL1(1)*pi**2
     .        + 1.0d0/4.0d0*GPL1(1)*pi**3*ci
     .        - 2.0d0*GPL1(1)*zeta3
     .        + 1.0d0/6.0d0*GPL1(1)*v*pi*ci
     .        - 1.0d0/6.0d0*GPL1(1)*v**2*pi*ci
     .        + 25.0d0/12.0d0*GPL1(1)*u*pi*ci
     .        + 1.0d0/3.0d0*GPL1(1)*u*pi**2
     .        - 7.0d0/4.0d0*GPL1(1)*u*v*pi*ci
     .        - 2.0d0/3.0d0*GPL1(1)*u*v**2*pi*ci
     .        - 55.0d0/12.0d0*GPL1(1)*u**2*pi*ci
     .        - 1.0d0/12.0d0*GPL1(1)*u**2*pi**2
     .        + 13.0d0/3.0d0*GPL1(1)*u**2*v*pi*ci
     .        - 5.0d0/2.0d0*GPL1(1)*u**2*v**2*pi*ci
     .        + 5.0d0*GPL1(1)*u**3*pi*ci
         col1 = col1 + 1.0d0/18.0d0*GPL1(1)*u**3*pi**2
     .        - 5.0d0*GPL1(1)*u**3*v*pi*ci
     .        - 5.0d0/2.0d0*GPL1(1)*u**4*pi*ci
     .        - 19.0d0/8.0d0*GPL1(2)
     .        - 5.0d0/6.0d0*GPL1(2)*u**(-1)
     .        + 5.0d0/3.0d0*GPL1(2)*u**(-1)*v
     .        - 5.0d0/6.0d0*GPL1(2)*u**(-1)*v**2
     .        - 389.0d0/144.0d0*GPL1(2)*pi*ci
     .        + 143.0d0/288.0d0*GPL1(2)*pi**2
     .        - 1.0d0/8.0d0*GPL1(2)*pi**3*ci
     .        + 7.0d0/4.0d0*GPL1(2)*zeta3
     .        + 5.0d0/3.0d0*GPL1(2)*v*omu**(-1)
     .        - 1.0d0/2.0d0*GPL1(2)*v
     .        + 4.0d0/3.0d0*GPL1(2)*v**2*omu**(-2)
     .        - 7.0d0/4.0d0*GPL1(2)*v**2
         col1 = col1 - 203.0d0/72.0d0*GPL1(2)*u
     .        - 2.0d0*GPL1(2)*u*pi*ci
     .        + 155.0d0/72.0d0*GPL1(2)*u*v
     .        + 4.0d0/3.0d0*GPL1(2)*u*v*pi*ci
     .        - 5.0d0/2.0d0*GPL1(2)*u*v**2
     .        + 2.0d0/3.0d0*GPL1(2)*u*v**2*pi*ci
     .        + 281.0d0/72.0d0*GPL1(2)*u**2
     .        + 9.0d0/2.0d0*GPL1(2)*u**2*pi*ci
     .        - 5.0d0*GPL1(2)*u**2*v
     .        - 13.0d0/3.0d0*GPL1(2)*u**2*v*pi*ci
     .        + 5.0d0/2.0d0*GPL1(2)*u**2*v**2*pi*ci
     .        - 5.0d0/2.0d0*GPL1(2)*u**3
     .        - 5.0d0*GPL1(2)*u**3*pi*ci
     .        + 5.0d0*GPL1(2)*u**3*v*pi*ci
     .        + 5.0d0/2.0d0*GPL1(2)*u**4*pi*ci
         col1 = col1 - 5.0d0/6.0d0*GPL1(3)*omumv**(-2)*pi*ci
     .        - 5.0d0/12.0d0*GPL1(3)*omumv**(-1)*pi*ci
     .        + 179.0d0/36.0d0*GPL1(3)*pi*ci
     .        - 77.0d0/36.0d0*GPL1(3)*pi**2
     .        + 1.0d0/4.0d0*GPL1(3)*pi**3*ci
     .        + 5.0d0/3.0d0*GPL1(3)*v*omumv**(-2)*pi*ci
     .        + 5.0d0/12.0d0*GPL1(3)*v*omumv**(-1)*pi*ci
     .        - 1.0d0/6.0d0*GPL1(3)*v*pi*ci
     .        - 2.0d0/3.0d0*GPL1(3)*v*pi**2
     .        - 5.0d0/6.0d0*GPL1(3)*v**2*omumv**(-2)*pi*ci
     .        + 1.0d0/6.0d0*GPL1(3)*v**2*pi*ci
     .        + 1.0d0/6.0d0*GPL1(3)*v**2*pi**2
     .        - 1.0d0/9.0d0*GPL1(3)*v**3*pi**2
     .        + 25.0d0/12.0d0*GPL1(3)*u*pi*ci
     .        - 2.0d0/3.0d0*GPL1(3)*u*pi**2
         col1 = col1 - 13.0d0/12.0d0*GPL1(3)*u*v*pi*ci
     .        + 1.0d0/3.0d0*GPL1(3)*u*v*pi**2
     .        - 2.0d0/3.0d0*GPL1(3)*u*v**2*pi*ci
     .        - 1.0d0/3.0d0*GPL1(3)*u*v**2*pi**2
     .        - 55.0d0/12.0d0*GPL1(3)*u**2*pi*ci
     .        + 1.0d0/6.0d0*GPL1(3)*u**2*pi**2
     .        + 13.0d0/3.0d0*GPL1(3)*u**2*v*pi*ci
     .        - 1.0d0/3.0d0*GPL1(3)*u**2*v*pi**2
     .        - 5.0d0/2.0d0*GPL1(3)*u**2*v**2*pi*ci
     .        + 5.0d0*GPL1(3)*u**3*pi*ci
     .        - 1.0d0/9.0d0*GPL1(3)*u**3*pi**2
     .        - 5.0d0*GPL1(3)*u**3*v*pi*ci
     .        - 5.0d0/2.0d0*GPL1(3)*u**4*pi*ci
     .        + 121.0d0/48.0d0*GPL2(0,0)
     .        + 11.0d0/6.0d0*GPL2(0,0)*pi*ci
         col1 = col1 - 1.0d0/4.0d0*GPL2(0,0)*pi**2
     .        - 1.0d0/2.0d0*GPL2(0,1)*pi**2
     .        - GPL2(0,1)*u*pi*ci
     .        + 1.0d0/4.0d0*GPL2(0,1)*u**2*pi*ci
     .        - 1.0d0/6.0d0*GPL2(0,1)*u**3*pi*ci
     .        - 49.0d0/48.0d0*GPL2(0,2)
     .        + 11.0d0/8.0d0*GPL2(0,2)*pi*ci
     .        - 3.0d0/8.0d0*GPL2(0,2)*pi**2
     .        - 2.0d0*GPL2(0,2)*u
     .        + 4.0d0/3.0d0*GPL2(0,2)*u*v
     .        + 2.0d0/3.0d0*GPL2(0,2)*u*v**2
     .        + 9.0d0/2.0d0*GPL2(0,2)*u**2
     .        - 13.0d0/3.0d0*GPL2(0,2)*u**2*v
     .        + 5.0d0/2.0d0*GPL2(0,2)*u**2*v**2
     .        - 5.0d0*GPL2(0,2)*u**3
         col1 = col1 + 5.0d0*GPL2(0,2)*u**3*v
     .        + 5.0d0/2.0d0*GPL2(0,2)*u**4
     .        - 55.0d0/12.0d0*GPL2(0,3)*pi*ci
     .        + 1.0d0/6.0d0*GPL2(0,3)*pi**2
     .        + GPL2(0,3)*u*pi*ci
     .        - 1.0d0/4.0d0*GPL2(0,3)*u**2*pi*ci
     .        + 1.0d0/6.0d0*GPL2(0,3)*u**3*pi*ci
     .        + 1.0d0/2.0d0*GPL2(1,0)*pi**2
     .        - GPL2(1,0)*u*pi*ci
     .        + 1.0d0/4.0d0*GPL2(1,0)*u**2*pi*ci
     .        - 1.0d0/6.0d0*GPL2(1,0)*u**3*pi*ci
     .        - 1.0d0/3.0d0*GPL2(1,1)*pi**2
     .        + 2.0d0*GPL2(1,1)*u*pi*ci
     .        - 1.0d0/2.0d0*GPL2(1,1)*u**2*pi*ci
     .        + 1.0d0/3.0d0*GPL2(1,1)*u**3*pi*ci
         col1 = col1 + 179.0d0/36.0d0*GPL2(1,2)
     .        - 5.0d0/6.0d0*GPL2(1,2)*u**(-2)
     .        + 5.0d0/3.0d0*GPL2(1,2)*u**(-2)*v
     .        - 5.0d0/6.0d0*GPL2(1,2)*u**(-2)*v**2
     .        - 5.0d0/12.0d0*GPL2(1,2)*u**(-1)
     .        + 5.0d0/12.0d0*GPL2(1,2)*u**(-1)*v
     .        + 11.0d0/12.0d0*GPL2(1,2)*pi*ci
     .        + 5.0d0/12.0d0*GPL2(1,2)*pi**2
     .        + 1.0d0/6.0d0*GPL2(1,2)*v
     .        - 1.0d0/6.0d0*GPL2(1,2)*v**2
     .        + 25.0d0/12.0d0*GPL2(1,2)*u
     .        - GPL2(1,2)*u*pi*ci
     .        - 7.0d0/4.0d0*GPL2(1,2)*u*v
     .        - 2.0d0/3.0d0*GPL2(1,2)*u*v**2
     .        - 55.0d0/12.0d0*GPL2(1,2)*u**2
         col1 = col1 + 1.0d0/4.0d0*GPL2(1,2)*u**2*pi*ci
     .        + 13.0d0/3.0d0*GPL2(1,2)*u**2*v
     .        - 5.0d0/2.0d0*GPL2(1,2)*u**2*v**2
     .        + 5.0d0*GPL2(1,2)*u**3
     .        - 1.0d0/6.0d0*GPL2(1,2)*u**3*pi*ci
     .        - 5.0d0*GPL2(1,2)*u**3*v
     .        - 5.0d0/2.0d0*GPL2(1,2)*u**4
     .        - 49.0d0/48.0d0*GPL2(2,0)
     .        + 11.0d0/8.0d0*GPL2(2,0)*pi*ci
     .        - 3.0d0/8.0d0*GPL2(2,0)*pi**2
     .        - 2.0d0*GPL2(2,0)*u
     .        + 4.0d0/3.0d0*GPL2(2,0)*u*v
     .        + 2.0d0/3.0d0*GPL2(2,0)*u*v**2
     .        + 9.0d0/2.0d0*GPL2(2,0)*u**2
     .        - 13.0d0/3.0d0*GPL2(2,0)*u**2*v
         col1 = col1 + 5.0d0/2.0d0*GPL2(2,0)*u**2*v**2
     .        - 5.0d0*GPL2(2,0)*u**3
     .        + 5.0d0*GPL2(2,0)*u**3*v
     .        + 5.0d0/2.0d0*GPL2(2,0)*u**4
     .        - 11.0d0/3.0d0*GPL2(2,1)*pi*ci
     .        - 1.0d0/6.0d0*GPL2(2,1)*pi**2
     .        - GPL2(2,1)*v*pi*ci
     .        + 1.0d0/4.0d0*GPL2(2,1)*v**2*pi*ci
     .        - 1.0d0/6.0d0*GPL2(2,1)*v**3*pi*ci
     .        - GPL2(2,1)*u*pi*ci
     .        + 1.0d0/2.0d0*GPL2(2,1)*u*v*pi*ci
     .        - 1.0d0/2.0d0*GPL2(2,1)*u*v**2*pi*ci
     .        + 1.0d0/4.0d0*GPL2(2,1)*u**2*pi*ci
     .        - 1.0d0/2.0d0*GPL2(2,1)*u**2*v*pi*ci
     .        - 1.0d0/6.0d0*GPL2(2,1)*u**3*pi*ci
         col1 = col1 + 121.0d0/48.0d0*GPL2(2,2)
     .        + 11.0d0/6.0d0*GPL2(2,2)*pi*ci
     .        + 1.0d0/12.0d0*GPL2(2,2)*pi**2
     .        - 11.0d0/12.0d0*GPL2(2,3)*pi*ci
     .        - 1.0d0/2.0d0*GPL2(2,3)*pi**2
     .        + GPL2(2,3)*v*pi*ci
     .        - 1.0d0/4.0d0*GPL2(2,3)*v**2*pi*ci
     .        + 1.0d0/6.0d0*GPL2(2,3)*v**3*pi*ci
     .        + GPL2(2,3)*u*pi*ci
     .        - 1.0d0/2.0d0*GPL2(2,3)*u*v*pi*ci
     .        + 1.0d0/2.0d0*GPL2(2,3)*u*v**2*pi*ci
     .        - 1.0d0/4.0d0*GPL2(2,3)*u**2*pi*ci
     .        + 1.0d0/2.0d0*GPL2(2,3)*u**2*v*pi*ci
     .        + 1.0d0/6.0d0*GPL2(2,3)*u**3*pi*ci
     .        + 179.0d0/36.0d0*GPL2(3,0)
         col1 = col1 - 5.0d0/6.0d0*GPL2(3,0)*omumv**(-2)
     .        - 5.0d0/12.0d0*GPL2(3,0)*omumv**(-1)
     .        + 3.0d0/4.0d0*GPL2(3,0)*pi**2
     .        + 5.0d0/3.0d0*GPL2(3,0)*v*omumv**(-2)
     .        + 5.0d0/12.0d0*GPL2(3,0)*v*omumv**(-1)
     .        - 1.0d0/6.0d0*GPL2(3,0)*v
     .        + GPL2(3,0)*v*pi*ci
     .        - 5.0d0/6.0d0*GPL2(3,0)*v**2*omumv**(-2)
     .        + 1.0d0/6.0d0*GPL2(3,0)*v**2
     .        - 1.0d0/4.0d0*GPL2(3,0)*v**2*pi*ci
     .        + 1.0d0/6.0d0*GPL2(3,0)*v**3*pi*ci
     .        + 25.0d0/12.0d0*GPL2(3,0)*u
     .        + GPL2(3,0)*u*pi*ci
     .        - 13.0d0/12.0d0*GPL2(3,0)*u*v
     .        - 1.0d0/2.0d0*GPL2(3,0)*u*v*pi*ci
         col1 = col1 - 2.0d0/3.0d0*GPL2(3,0)*u*v**2
     .        + 1.0d0/2.0d0*GPL2(3,0)*u*v**2*pi*ci
     .        - 55.0d0/12.0d0*GPL2(3,0)*u**2
     .        - 1.0d0/4.0d0*GPL2(3,0)*u**2*pi*ci
     .        + 13.0d0/3.0d0*GPL2(3,0)*u**2*v
     .        + 1.0d0/2.0d0*GPL2(3,0)*u**2*v*pi*ci
     .        - 5.0d0/2.0d0*GPL2(3,0)*u**2*v**2
     .        + 5.0d0*GPL2(3,0)*u**3
     .        + 1.0d0/6.0d0*GPL2(3,0)*u**3*pi*ci
     .        - 5.0d0*GPL2(3,0)*u**3*v
     .        - 5.0d0/2.0d0*GPL2(3,0)*u**4
     .        - 11.0d0/12.0d0*GPL2(3,2)*pi*ci
     .        + 1.0d0/2.0d0*GPL2(3,2)*pi**2
     .        + GPL2(3,2)*v*pi*ci
     .        - 1.0d0/4.0d0*GPL2(3,2)*v**2*pi*ci
         col1 = col1 + 1.0d0/6.0d0*GPL2(3,2)*v**3*pi*ci
     .        + GPL2(3,2)*u*pi*ci
     .        - 1.0d0/2.0d0*GPL2(3,2)*u*v*pi*ci
     .        + 1.0d0/2.0d0*GPL2(3,2)*u*v**2*pi*ci
     .        - 1.0d0/4.0d0*GPL2(3,2)*u**2*pi*ci
     .        + 1.0d0/2.0d0*GPL2(3,2)*u**2*v*pi*ci
     .        + 1.0d0/6.0d0*GPL2(3,2)*u**3*pi*ci
     .        + 11.0d0/6.0d0*GPL2(3,3)*pi*ci
     .        - 2.0d0/3.0d0*GPL2(3,3)*pi**2
     .        - 2.0d0*GPL2(3,3)*v*pi*ci
     .        + 1.0d0/2.0d0*GPL2(3,3)*v**2*pi*ci
     .        - 1.0d0/3.0d0*GPL2(3,3)*v**3*pi*ci
     .        - 2.0d0*GPL2(3,3)*u*pi*ci
     .        + GPL2(3,3)*u*v*pi*ci
     .        - GPL2(3,3)*u*v**2*pi*ci
         col1 = col1 + 1.0d0/2.0d0*GPL2(3,3)*u**2*pi*ci
     .        - GPL2(3,3)*u**2*v*pi*ci
     .        - 1.0d0/3.0d0*GPL2(3,3)*u**3*pi*ci
     .        + GPL3(0,0,1)*pi*ci
     .        + 11.0d0/6.0d0*GPL3(0,0,2)
     .        + 1.0d0/2.0d0*GPL3(0,0,2)*pi*ci
     .        - GPL3(0,0,3)*pi*ci
     .        + GPL3(0,1,0)*pi*ci
     .        - GPL3(0,1,2)*u
     .        + 1.0d0/4.0d0*GPL3(0,1,2)*u**2
     .        - 1.0d0/6.0d0*GPL3(0,1,2)*u**3
     .        + 11.0d0/6.0d0*GPL3(0,2,0)
     .        + 1.0d0/2.0d0*GPL3(0,2,0)*pi*ci
     .        - 1.0d0/2.0d0*GPL3(0,2,1)*pi*ci
     .        + 11.0d0/6.0d0*GPL3(0,2,2)
         col1 = col1 + 1.0d0/2.0d0*GPL3(0,2,2)*pi*ci
     .        - 1.0d0/2.0d0*GPL3(0,2,3)*pi*ci
     .        - 55.0d0/12.0d0*GPL3(0,3,0)
     .        - 1.0d0/2.0d0*GPL3(0,3,0)*pi*ci
     .        + GPL3(0,3,0)*u
     .        - 1.0d0/4.0d0*GPL3(0,3,0)*u**2
     .        + 1.0d0/6.0d0*GPL3(0,3,0)*u**3
     .        - 1.0d0/2.0d0*GPL3(0,3,2)*pi*ci
     .        + 2.0d0*GPL3(0,3,3)*pi*ci
     .        - GPL3(1,0,2)*pi*ci
     .        - GPL3(1,0,2)*u
     .        + 1.0d0/4.0d0*GPL3(1,0,2)*u**2
     .        - 1.0d0/6.0d0*GPL3(1,0,2)*u**3
     .        - 2.0d0*GPL3(1,1,1)*pi*ci
     .        + 2.0d0*GPL3(1,1,2)*pi*ci
         col1 = col1 + 2.0d0*GPL3(1,1,2)*u
     .        - 1.0d0/2.0d0*GPL3(1,1,2)*u**2
     .        + 1.0d0/3.0d0*GPL3(1,1,2)*u**3
     .        - GPL3(1,2,0)*pi*ci
     .        - GPL3(1,2,0)*u
     .        + 1.0d0/4.0d0*GPL3(1,2,0)*u**2
     .        - 1.0d0/6.0d0*GPL3(1,2,0)*u**3
     .        + 2.0d0*GPL3(1,2,1)*pi*ci
     .        - 11.0d0/3.0d0*GPL3(1,2,2)
     .        - GPL3(1,2,2)*pi*ci
     .        + 11.0d0/6.0d0*GPL3(2,0,0)
     .        + 1.0d0/2.0d0*GPL3(2,0,0)*pi*ci
     .        - 1.0d0/2.0d0*GPL3(2,0,1)*pi*ci
     .        + 11.0d0/6.0d0*GPL3(2,0,2)
     .        + 1.0d0/2.0d0*GPL3(2,0,2)*pi*ci
         col1 = col1 - 1.0d0/2.0d0*GPL3(2,0,3)*pi*ci
     .        - 1.0d0/2.0d0*GPL3(2,1,0)*pi*ci
     .        + 2.0d0*GPL3(2,1,1)*pi*ci
     .        - 11.0d0/3.0d0*GPL3(2,1,2)
     .        - 1.0d0/2.0d0*GPL3(2,1,2)*pi*ci
     .        - GPL3(2,1,2)*v
     .        + 1.0d0/4.0d0*GPL3(2,1,2)*v**2
     .        - 1.0d0/6.0d0*GPL3(2,1,2)*v**3
     .        - GPL3(2,1,2)*u
     .        + 1.0d0/2.0d0*GPL3(2,1,2)*u*v
     .        - 1.0d0/2.0d0*GPL3(2,1,2)*u*v**2
     .        + 1.0d0/4.0d0*GPL3(2,1,2)*u**2
     .        - 1.0d0/2.0d0*GPL3(2,1,2)*u**2*v
     .        - 1.0d0/6.0d0*GPL3(2,1,2)*u**3
     .        + 11.0d0/6.0d0*GPL3(2,2,0)
         col1 = col1 + 1.0d0/2.0d0*GPL3(2,2,0)*pi*ci
     .        - GPL3(2,2,1)*pi*ci
     .        + GPL3(2,2,3)*pi*ci
     .        - 11.0d0/12.0d0*GPL3(2,3,0)
     .        + GPL3(2,3,0)*v
     .        - 1.0d0/4.0d0*GPL3(2,3,0)*v**2
     .        + 1.0d0/6.0d0*GPL3(2,3,0)*v**3
     .        + GPL3(2,3,0)*u
     .        - 1.0d0/2.0d0*GPL3(2,3,0)*u*v
     .        + 1.0d0/2.0d0*GPL3(2,3,0)*u*v**2
     .        - 1.0d0/4.0d0*GPL3(2,3,0)*u**2
     .        + 1.0d0/2.0d0*GPL3(2,3,0)*u**2*v
     .        + 1.0d0/6.0d0*GPL3(2,3,0)*u**3
     .        + GPL3(2,3,2)*pi*ci
     .        - 11.0d0/3.0d0*GPL3(3,0,0)
         col1 = col1 - GPL3(3,0,0)*pi*ci
     .        - 11.0d0/12.0d0*GPL3(3,0,2)
     .        - GPL3(3,0,2)*pi*ci
     .        + GPL3(3,0,2)*v
     .        - 1.0d0/4.0d0*GPL3(3,0,2)*v**2
     .        + 1.0d0/6.0d0*GPL3(3,0,2)*v**3
     .        + GPL3(3,0,2)*u
     .        - 1.0d0/2.0d0*GPL3(3,0,2)*u*v
     .        + 1.0d0/2.0d0*GPL3(3,0,2)*u*v**2
     .        - 1.0d0/4.0d0*GPL3(3,0,2)*u**2
     .        + 1.0d0/2.0d0*GPL3(3,0,2)*u**2*v
     .        + 1.0d0/6.0d0*GPL3(3,0,2)*u**3
     .        + 2.0d0*GPL3(3,0,3)*pi*ci
     .        - 11.0d0/12.0d0*GPL3(3,2,0)
     .        - GPL3(3,2,0)*pi*ci
         col1 = col1 + GPL3(3,2,0)*v
     .        - 1.0d0/4.0d0*GPL3(3,2,0)*v**2
     .        + 1.0d0/6.0d0*GPL3(3,2,0)*v**3
     .        + GPL3(3,2,0)*u
     .        - 1.0d0/2.0d0*GPL3(3,2,0)*u*v
     .        + 1.0d0/2.0d0*GPL3(3,2,0)*u*v**2
     .        - 1.0d0/4.0d0*GPL3(3,2,0)*u**2
     .        + 1.0d0/2.0d0*GPL3(3,2,0)*u**2*v
     .        + 1.0d0/6.0d0*GPL3(3,2,0)*u**3
     .        + 11.0d0/6.0d0*GPL3(3,3,0)
     .        + 2.0d0*GPL3(3,3,0)*pi*ci
     .        - 2.0d0*GPL3(3,3,0)*v
     .        + 1.0d0/2.0d0*GPL3(3,3,0)*v**2
     .        - 1.0d0/3.0d0*GPL3(3,3,0)*v**3
     .        - 2.0d0*GPL3(3,3,0)*u
         col1 = col1 + GPL3(3,3,0)*u*v
     .        - GPL3(3,3,0)*u*v**2
     .        + 1.0d0/2.0d0*GPL3(3,3,0)*u**2
     .        - GPL3(3,3,0)*u**2*v
     .        - 1.0d0/3.0d0*GPL3(3,3,0)*u**3
     .        - 2.0d0*GPL3(3,3,3)*pi*ci
     .        + GPL4(0,0,1,2)
     .        + 1.0d0/2.0d0*GPL4(0,0,2,2)
     .        - GPL4(0,0,3,0)
     .        + GPL4(0,1,0,2)
     .        + GPL4(0,1,2,0)
     .        - GPL4(0,1,2,2)
     .        + 1.0d0/2.0d0*GPL4(0,2,0,2)
     .        - 1.0d0/2.0d0*GPL4(0,2,1,2)
     .        + 1.0d0/2.0d0*GPL4(0,2,2,0)
         col1 = col1 - 1.0d0/2.0d0*GPL4(0,2,3,0)
     .        - 1.0d0/2.0d0*GPL4(0,3,0,2)
     .        - 1.0d0/2.0d0*GPL4(0,3,2,0)
     .        + 2.0d0*GPL4(0,3,3,0)
     .        - GPL4(1,0,2,2)
     .        - 2.0d0*GPL4(1,1,1,2)
     .        + 2.0d0*GPL4(1,1,2,2)
     .        - GPL4(1,2,0,2)
     .        + 2.0d0*GPL4(1,2,1,2)
     .        - GPL4(1,2,2,0)
     .        + 1.0d0/2.0d0*GPL4(2,0,0,2)
     .        - 1.0d0/2.0d0*GPL4(2,0,1,2)
     .        + 1.0d0/2.0d0*GPL4(2,0,2,0)
     .        - 1.0d0/2.0d0*GPL4(2,0,3,0)
     .        - 1.0d0/2.0d0*GPL4(2,1,0,2)
         col1 = col1 + 2.0d0*GPL4(2,1,1,2)
     .        - 1.0d0/2.0d0*GPL4(2,1,2,0)
     .        + 1.0d0/2.0d0*GPL4(2,2,0,0)
     .        - GPL4(2,2,1,2)
     .        + GPL4(2,2,3,0)
     .        - GPL4(2,3,0,0)
     .        + GPL4(2,3,0,2)
     .        + GPL4(2,3,2,0)
     .        - GPL4(3,0,0,2)
     .        - GPL4(3,0,2,0)
     .        + 2.0d0*GPL4(3,0,3,0)
     .        - GPL4(3,2,0,0)
     .        + 2.0d0*GPL4(3,3,0,0)
     .        - 2.0d0*GPL4(3,3,3,0)

         col2 = (0.d0,0.d0)

         col3 = (0.d0,0.d0)

         col4 = + 19.0d0/16.0d0
     .        + 1.0d0/12.0d0*u**(-1)*pi*ci
     .        - 1.0d0/6.0d0*u**(-1)*v*pi*ci
     .        + 1.0d0/12.0d0*u**(-1)*v**2*pi*ci
     .        + 1.0d0/72.0d0*omumv**(-2)*pi**2
     .        + 1.0d0/12.0d0*omumv**(-1)*pi*ci
     .        - 1.0d0/18.0d0*omumv**(-1)*pi**2
     .        - 3.0d0/8.0d0*pi*ci
     .        + 1.0d0/24.0d0*pi**2
     .        - zeta3
     .        + 1.0d0/4.0d0*v*omu**(-1)
     .        - 1.0d0/36.0d0*v*omumv**(-2)*pi**2
     .        - 1.0d0/6.0d0*v*omumv**(-1)*pi*ci
     .        + 1.0d0/18.0d0*v*omumv**(-1)*pi**2
     .        + 1.0d0/4.0d0*v*upv**(-1)
         col4 = col4 - 7.0d0/24.0d0*v
     .        + 1.0d0/3.0d0*v*pi*ci
     .        + 1.0d0/4.0d0*v**2*omu**(-2)*pi*ci
     .        + 1.0d0/72.0d0*v**2*omumv**(-2)*pi**2
     .        + 1.0d0/12.0d0*v**2*omumv**(-1)*pi*ci
     .        + 1.0d0/4.0d0*v**2*upv**(-2)*pi*ci
     .        - 5.0d0/24.0d0*v**2*pi*ci
     .        - 1.0d0/4.0d0*u
     .        + 1.0d0/9.0d0*u*pi**2
     .        + 1.0d0/4.0d0*u*v
     .        - 1.0d0/4.0d0*u*v*pi*ci
     .        + 1.0d0/9.0d0*u*v*pi**2
     .        + 1.0d0/4.0d0*u*v**2*pi*ci
     .        - 2.0d0/9.0d0*u*v**2*pi**2
     .        + 1.0d0/4.0d0*u**2
         col4 = col4 + 1.0d0/18.0d0*u**2*pi**2
     .        + 1.0d0/4.0d0*u**2*v*pi*ci
     .        - 5.0d0/9.0d0*u**2*v*pi**2
     .        + 1.0d0/6.0d0*u**2*v**2*pi**2
     .        - 1.0d0/3.0d0*u**3*pi**2
     .        + 1.0d0/3.0d0*u**3*v*pi**2
     .        + 1.0d0/6.0d0*u**4*pi**2
     .        + 1.0d0/4.0d0*HPL1(0)
     .        + 1.0d0/12.0d0*HPL1(0)*u**(-1)*v
     .        - 1.0d0/12.0d0*HPL1(0)*u**(-1)*v**2
     .        + 1.0d0/12.0d0*HPL1(0)*omumv**(-2)*pi*ci
     .        - 1.0d0/3.0d0*HPL1(0)*omumv**(-1)*pi*ci
     .        + 1.0d0/4.0d0*HPL1(0)*pi*ci
     .        - 1.0d0/6.0d0*HPL1(0)*v*omumv**(-2)*pi*ci
     .        + 1.0d0/12.0d0*HPL1(0)*v*omumv**(-1)
         col4 = col4 + 1.0d0/3.0d0*HPL1(0)*v*omumv**(-1)*pi*ci
     .        - 1.0d0/3.0d0*HPL1(0)*v
     .        - 1.0d0/4.0d0*HPL1(0)*v**2*omu**(-2)
     .        + 1.0d0/12.0d0*HPL1(0)*v**2*omumv**(-2)*pi*ci
     .        - 1.0d0/12.0d0*HPL1(0)*v**2*omumv**(-1)
     .        - 1.0d0/4.0d0*HPL1(0)*v**2*upv**(-2)
     .        + 5.0d0/24.0d0*HPL1(0)*v**2
     .        + 1.0d0/6.0d0*HPL1(0)*u*pi*ci
     .        + 1.0d0/4.0d0*HPL1(0)*u*v
     .        + 1.0d0/6.0d0*HPL1(0)*u*v*pi*ci
     .        - 1.0d0/4.0d0*HPL1(0)*u*v**2
     .        - 1.0d0/3.0d0*HPL1(0)*u*v**2*pi*ci
     .        + 1.0d0/12.0d0*HPL1(0)*u**2*pi*ci
     .        - 1.0d0/4.0d0*HPL1(0)*u**2*v
     .        - 5.0d0/6.0d0*HPL1(0)*u**2*v*pi*ci
         col4 = col4 + 1.0d0/4.0d0*HPL1(0)*u**2*v**2*pi*ci
     .        - 1.0d0/2.0d0*HPL1(0)*u**3*pi*ci
     .        + 1.0d0/2.0d0*HPL1(0)*u**3*v*pi*ci
     .        + 1.0d0/4.0d0*HPL1(0)*u**4*pi*ci
     .        + 1.0d0/4.0d0*HPL1(0)*GPL1(0)
     .        + 1.0d0/12.0d0*HPL1(0)*GPL1(0)*omumv**(-2)
     .        - 1.0d0/3.0d0*HPL1(0)*GPL1(0)*omumv**(-1)
     .        - 1.0d0/6.0d0*HPL1(0)*GPL1(0)*v*omumv**(-2)
     .        + 1.0d0/3.0d0*HPL1(0)*GPL1(0)*v*omumv**(-1)
     .        + 1.0d0/12.0d0*HPL1(0)*GPL1(0)*v**2*omumv**(-2)
     .        + 1.0d0/6.0d0*HPL1(0)*GPL1(0)*u
     .        + 1.0d0/6.0d0*HPL1(0)*GPL1(0)*u*v
     .        - 1.0d0/3.0d0*HPL1(0)*GPL1(0)*u*v**2
     .        + 1.0d0/12.0d0*HPL1(0)*GPL1(0)*u**2
     .        - 5.0d0/6.0d0*HPL1(0)*GPL1(0)*u**2*v
         col4 = col4 + 1.0d0/4.0d0*HPL1(0)*GPL1(0)*u**2*v**2
     .        - 1.0d0/2.0d0*HPL1(0)*GPL1(0)*u**3
     .        + 1.0d0/2.0d0*HPL1(0)*GPL1(0)*u**3*v
     .        + 1.0d0/4.0d0*HPL1(0)*GPL1(0)*u**4
     .        - 1.0d0/4.0d0*HPL1(0)*GPL1(1)
     .        - 1.0d0/12.0d0*HPL1(0)*GPL1(1)*u**(-2)
     .        + 1.0d0/6.0d0*HPL1(0)*GPL1(1)*u**(-2)*v
     .        - 1.0d0/12.0d0*HPL1(0)*GPL1(1)*u**(-2)*v**2
     .        + 1.0d0/3.0d0*HPL1(0)*GPL1(1)*u**(-1)
     .        - 1.0d0/3.0d0*HPL1(0)*GPL1(1)*u**(-1)*v
     .        - 1.0d0/6.0d0*HPL1(0)*GPL1(1)*u
     .        - 1.0d0/6.0d0*HPL1(0)*GPL1(1)*u*v
     .        + 1.0d0/3.0d0*HPL1(0)*GPL1(1)*u*v**2
     .        - 1.0d0/12.0d0*HPL1(0)*GPL1(1)*u**2
     .        + 5.0d0/6.0d0*HPL1(0)*GPL1(1)*u**2*v
         col4 = col4 - 1.0d0/4.0d0*HPL1(0)*GPL1(1)*u**2*v**2
     .        + 1.0d0/2.0d0*HPL1(0)*GPL1(1)*u**3
     .        - 1.0d0/2.0d0*HPL1(0)*GPL1(1)*u**3*v
     .        - 1.0d0/4.0d0*HPL1(0)*GPL1(1)*u**4
     .        + 1.0d0/4.0d0*HPL1(0)*GPL1(2)
     .        + 1.0d0/12.0d0*HPL1(0)*GPL1(2)*u**(-2)
     .        - 1.0d0/6.0d0*HPL1(0)*GPL1(2)*u**(-2)*v
     .        + 1.0d0/12.0d0*HPL1(0)*GPL1(2)*u**(-2)*v**2
     .        - 1.0d0/3.0d0*HPL1(0)*GPL1(2)*u**(-1)
     .        + 1.0d0/3.0d0*HPL1(0)*GPL1(2)*u**(-1)*v
     .        + 1.0d0/6.0d0*HPL1(0)*GPL1(2)*u
     .        + 1.0d0/6.0d0*HPL1(0)*GPL1(2)*u*v
     .        - 1.0d0/3.0d0*HPL1(0)*GPL1(2)*u*v**2
     .        + 1.0d0/12.0d0*HPL1(0)*GPL1(2)*u**2
     .        - 5.0d0/6.0d0*HPL1(0)*GPL1(2)*u**2*v
         col4 = col4 + 1.0d0/4.0d0*HPL1(0)*GPL1(2)*u**2*v**2
     .        - 1.0d0/2.0d0*HPL1(0)*GPL1(2)*u**3
     .        + 1.0d0/2.0d0*HPL1(0)*GPL1(2)*u**3*v
     .        + 1.0d0/4.0d0*HPL1(0)*GPL1(2)*u**4
     .        - 1.0d0/4.0d0*HPL1(0)*GPL1(3)
     .        - 1.0d0/12.0d0*HPL1(0)*GPL1(3)*omumv**(-2)
     .        + 1.0d0/3.0d0*HPL1(0)*GPL1(3)*omumv**(-1)
     .        + 1.0d0/6.0d0*HPL1(0)*GPL1(3)*v*omumv**(-2)
     .        - 1.0d0/3.0d0*HPL1(0)*GPL1(3)*v*omumv**(-1)
     .        - 1.0d0/12.0d0*HPL1(0)*GPL1(3)*v**2*omumv**(-2)
     .        - 1.0d0/6.0d0*HPL1(0)*GPL1(3)*u
     .        - 1.0d0/6.0d0*HPL1(0)*GPL1(3)*u*v
     .        + 1.0d0/3.0d0*HPL1(0)*GPL1(3)*u*v**2
     .        - 1.0d0/12.0d0*HPL1(0)*GPL1(3)*u**2
     .        + 5.0d0/6.0d0*HPL1(0)*GPL1(3)*u**2*v
         col4 = col4 - 1.0d0/4.0d0*HPL1(0)*GPL1(3)*u**2*v**2
     .        + 1.0d0/2.0d0*HPL1(0)*GPL1(3)*u**3
     .        - 1.0d0/2.0d0*HPL1(0)*GPL1(3)*u**3*v
     .        - 1.0d0/4.0d0*HPL1(0)*GPL1(3)*u**4
     .        + 7.0d0/24.0d0*HPL1(1)
     .        - 1.0d0/12.0d0*HPL1(1)*u**(-1)
     .        + 1.0d0/6.0d0*HPL1(1)*u**(-1)*v
     .        - 1.0d0/12.0d0*HPL1(1)*u**(-1)*v**2
     .        - 1.0d0/4.0d0*HPL1(1)*v
     .        - 1.0d0/4.0d0*HPL1(1)*v**2*omu**(-2)
     .        + 5.0d0/24.0d0*HPL1(1)*v**2
     .        - 1.0d0/12.0d0*HPL1(1)*u
     .        + 1.0d0/6.0d0*HPL1(1)*u*pi*ci
     .        + 7.0d0/12.0d0*HPL1(1)*u*v
     .        + 1.0d0/6.0d0*HPL1(1)*u*v*pi*ci
         col4 = col4 - 1.0d0/4.0d0*HPL1(1)*u*v**2
     .        - 1.0d0/3.0d0*HPL1(1)*u*v**2*pi*ci
     .        + 3.0d0/8.0d0*HPL1(1)*u**2
     .        + 1.0d0/12.0d0*HPL1(1)*u**2*pi*ci
     .        - 1.0d0/2.0d0*HPL1(1)*u**2*v
     .        - 5.0d0/6.0d0*HPL1(1)*u**2*v*pi*ci
     .        + 1.0d0/4.0d0*HPL1(1)*u**2*v**2*pi*ci
     .        - 1.0d0/4.0d0*HPL1(1)*u**3
     .        - 1.0d0/2.0d0*HPL1(1)*u**3*pi*ci
     .        + 1.0d0/2.0d0*HPL1(1)*u**3*v*pi*ci
     .        + 1.0d0/4.0d0*HPL1(1)*u**4*pi*ci
     .        + 1.0d0/6.0d0*HPL1(1)*GPL1(0)*u
     .        + 1.0d0/6.0d0*HPL1(1)*GPL1(0)*u*v
     .        - 1.0d0/3.0d0*HPL1(1)*GPL1(0)*u*v**2
     .        + 1.0d0/12.0d0*HPL1(1)*GPL1(0)*u**2
         col4 = col4 - 5.0d0/6.0d0*HPL1(1)*GPL1(0)*u**2*v
     .        + 1.0d0/4.0d0*HPL1(1)*GPL1(0)*u**2*v**2
     .        - 1.0d0/2.0d0*HPL1(1)*GPL1(0)*u**3
     .        + 1.0d0/2.0d0*HPL1(1)*GPL1(0)*u**3*v
     .        + 1.0d0/4.0d0*HPL1(1)*GPL1(0)*u**4
     .        - 1.0d0/4.0d0*HPL1(1)*GPL1(1)
     .        - 1.0d0/12.0d0*HPL1(1)*GPL1(1)*u**(-2)
     .        + 1.0d0/6.0d0*HPL1(1)*GPL1(1)*u**(-2)*v
     .        - 1.0d0/12.0d0*HPL1(1)*GPL1(1)*u**(-2)*v**2
     .        + 1.0d0/3.0d0*HPL1(1)*GPL1(1)*u**(-1)
     .        - 1.0d0/3.0d0*HPL1(1)*GPL1(1)*u**(-1)*v
     .        - 1.0d0/6.0d0*HPL1(1)*GPL1(1)*u
     .        - 1.0d0/6.0d0*HPL1(1)*GPL1(1)*u*v
     .        + 1.0d0/3.0d0*HPL1(1)*GPL1(1)*u*v**2
     .        - 1.0d0/12.0d0*HPL1(1)*GPL1(1)*u**2
         col4 = col4 + 5.0d0/6.0d0*HPL1(1)*GPL1(1)*u**2*v
     .        - 1.0d0/4.0d0*HPL1(1)*GPL1(1)*u**2*v**2
     .        + 1.0d0/2.0d0*HPL1(1)*GPL1(1)*u**3
     .        - 1.0d0/2.0d0*HPL1(1)*GPL1(1)*u**3*v
     .        - 1.0d0/4.0d0*HPL1(1)*GPL1(1)*u**4
     .        - 1.0d0/4.0d0*HPL2(0,0)
     .        - 1.0d0/12.0d0*HPL2(0,0)*omumv**(-2)
     .        + 1.0d0/3.0d0*HPL2(0,0)*omumv**(-1)
     .        + 1.0d0/6.0d0*HPL2(0,0)*v*omumv**(-2)
     .        - 1.0d0/3.0d0*HPL2(0,0)*v*omumv**(-1)
     .        - 1.0d0/12.0d0*HPL2(0,0)*v**2*omumv**(-2)
     .        - 1.0d0/6.0d0*HPL2(0,0)*u
     .        - 1.0d0/6.0d0*HPL2(0,0)*u*v
     .        + 1.0d0/3.0d0*HPL2(0,0)*u*v**2
     .        - 1.0d0/12.0d0*HPL2(0,0)*u**2
         col4 = col4 + 5.0d0/6.0d0*HPL2(0,0)*u**2*v
     .        - 1.0d0/4.0d0*HPL2(0,0)*u**2*v**2
     .        + 1.0d0/2.0d0*HPL2(0,0)*u**3
     .        - 1.0d0/2.0d0*HPL2(0,0)*u**3*v
     .        - 1.0d0/4.0d0*HPL2(0,0)*u**4
     .        + 1.0d0/4.0d0*HPL2(1,0)
     .        + 1.0d0/12.0d0*HPL2(1,0)*omumv**(-2)
     .        - 1.0d0/3.0d0*HPL2(1,0)*omumv**(-1)
     .        - 1.0d0/6.0d0*HPL2(1,0)*v*omumv**(-2)
     .        + 1.0d0/3.0d0*HPL2(1,0)*v*omumv**(-1)
     .        + 1.0d0/12.0d0*HPL2(1,0)*v**2*omumv**(-2)
     .        - 1.0d0/6.0d0*HPL2(1,0)*u
     .        - 1.0d0/6.0d0*HPL2(1,0)*u*v
     .        + 1.0d0/3.0d0*HPL2(1,0)*u*v**2
     .        - 1.0d0/12.0d0*HPL2(1,0)*u**2
         col4 = col4 + 5.0d0/6.0d0*HPL2(1,0)*u**2*v
     .        - 1.0d0/4.0d0*HPL2(1,0)*u**2*v**2
     .        + 1.0d0/2.0d0*HPL2(1,0)*u**3
     .        - 1.0d0/2.0d0*HPL2(1,0)*u**3*v
     .        - 1.0d0/4.0d0*HPL2(1,0)*u**4
     .        - 1.0d0/3.0d0*GPL1(0)
     .        + 1.0d0/12.0d0*GPL1(0)*omumv**(-1)
     .        - 1.0d0/6.0d0*GPL1(0)*v*omumv**(-1)
     .        + 1.0d0/12.0d0*GPL1(0)*v
     .        + 1.0d0/12.0d0*GPL1(0)*v**2*omumv**(-1)
     .        + 1.0d0/4.0d0*GPL1(0)*v**2*upv**(-2)
     .        - 1.0d0/12.0d0*GPL1(0)*u
     .        - 1.0d0/6.0d0*GPL1(0)*u*pi*ci
     .        + 1.0d0/3.0d0*GPL1(0)*u*v
     .        - 1.0d0/6.0d0*GPL1(0)*u*v*pi*ci
         col4 = col4 + 1.0d0/3.0d0*GPL1(0)*u*v**2*pi*ci
     .        + 3.0d0/8.0d0*GPL1(0)*u**2
     .        - 1.0d0/12.0d0*GPL1(0)*u**2*pi*ci
     .        - 1.0d0/4.0d0*GPL1(0)*u**2*v
     .        + 5.0d0/6.0d0*GPL1(0)*u**2*v*pi*ci
     .        - 1.0d0/4.0d0*GPL1(0)*u**2*v**2*pi*ci
     .        - 1.0d0/4.0d0*GPL1(0)*u**3
     .        + 1.0d0/2.0d0*GPL1(0)*u**3*pi*ci
     .        - 1.0d0/2.0d0*GPL1(0)*u**3*v*pi*ci
     .        - 1.0d0/4.0d0*GPL1(0)*u**4*pi*ci
     .        + 1.0d0/12.0d0*GPL1(1)*u**(-2)*pi*ci
     .        - 1.0d0/6.0d0*GPL1(1)*u**(-2)*v*pi*ci
     .        + 1.0d0/12.0d0*GPL1(1)*u**(-2)*v**2*pi*ci
     .        - 1.0d0/3.0d0*GPL1(1)*u**(-1)*pi*ci
     .        + 1.0d0/3.0d0*GPL1(1)*u**(-1)*v*pi*ci
         col4 = col4 + 1.0d0/4.0d0*GPL1(1)*pi*ci
     .        + 1.0d0/6.0d0*GPL1(1)*u*pi*ci
     .        + 1.0d0/6.0d0*GPL1(1)*u*v*pi*ci
     .        - 1.0d0/3.0d0*GPL1(1)*u*v**2*pi*ci
     .        + 1.0d0/12.0d0*GPL1(1)*u**2*pi*ci
     .        - 5.0d0/6.0d0*GPL1(1)*u**2*v*pi*ci
     .        + 1.0d0/4.0d0*GPL1(1)*u**2*v**2*pi*ci
     .        - 1.0d0/2.0d0*GPL1(1)*u**3*pi*ci
     .        + 1.0d0/2.0d0*GPL1(1)*u**3*v*pi*ci
     .        + 1.0d0/4.0d0*GPL1(1)*u**4*pi*ci
     .        - 7.0d0/24.0d0*GPL1(2)
     .        + 1.0d0/12.0d0*GPL1(2)*u**(-1)
     .        - 1.0d0/6.0d0*GPL1(2)*u**(-1)*v
     .        + 1.0d0/12.0d0*GPL1(2)*u**(-1)*v**2
     .        + 1.0d0/4.0d0*GPL1(2)*v
         col4 = col4 + 1.0d0/4.0d0*GPL1(2)*v**2*omu**(-2)
     .        - 5.0d0/24.0d0*GPL1(2)*v**2
     .        + 1.0d0/12.0d0*GPL1(2)*u
     .        - 1.0d0/6.0d0*GPL1(2)*u*pi*ci
     .        - 7.0d0/12.0d0*GPL1(2)*u*v
     .        - 1.0d0/6.0d0*GPL1(2)*u*v*pi*ci
     .        + 1.0d0/4.0d0*GPL1(2)*u*v**2
     .        + 1.0d0/3.0d0*GPL1(2)*u*v**2*pi*ci
     .        - 3.0d0/8.0d0*GPL1(2)*u**2
     .        - 1.0d0/12.0d0*GPL1(2)*u**2*pi*ci
     .        + 1.0d0/2.0d0*GPL1(2)*u**2*v
     .        + 5.0d0/6.0d0*GPL1(2)*u**2*v*pi*ci
     .        - 1.0d0/4.0d0*GPL1(2)*u**2*v**2*pi*ci
     .        + 1.0d0/4.0d0*GPL1(2)*u**3
     .        + 1.0d0/2.0d0*GPL1(2)*u**3*pi*ci
         col4 = col4 - 1.0d0/2.0d0*GPL1(2)*u**3*v*pi*ci
     .        - 1.0d0/4.0d0*GPL1(2)*u**4*pi*ci
     .        + 1.0d0/12.0d0*GPL1(3)*omumv**(-2)*pi*ci
     .        - 1.0d0/3.0d0*GPL1(3)*omumv**(-1)*pi*ci
     .        + 1.0d0/4.0d0*GPL1(3)*pi*ci
     .        - 1.0d0/6.0d0*GPL1(3)*v*omumv**(-2)*pi*ci
     .        + 1.0d0/3.0d0*GPL1(3)*v*omumv**(-1)*pi*ci
     .        + 1.0d0/12.0d0*GPL1(3)*v**2*omumv**(-2)*pi*ci
     .        + 1.0d0/6.0d0*GPL1(3)*u*pi*ci
     .        + 1.0d0/6.0d0*GPL1(3)*u*v*pi*ci
     .        - 1.0d0/3.0d0*GPL1(3)*u*v**2*pi*ci
     .        + 1.0d0/12.0d0*GPL1(3)*u**2*pi*ci
     .        - 5.0d0/6.0d0*GPL1(3)*u**2*v*pi*ci
     .        + 1.0d0/4.0d0*GPL1(3)*u**2*v**2*pi*ci
     .        - 1.0d0/2.0d0*GPL1(3)*u**3*pi*ci
         col4 = col4 + 1.0d0/2.0d0*GPL1(3)*u**3*v*pi*ci
     .        + 1.0d0/4.0d0*GPL1(3)*u**4*pi*ci
     .        - 1.0d0/6.0d0*GPL2(0,2)*u
     .        - 1.0d0/6.0d0*GPL2(0,2)*u*v
     .        + 1.0d0/3.0d0*GPL2(0,2)*u*v**2
     .        - 1.0d0/12.0d0*GPL2(0,2)*u**2
     .        + 5.0d0/6.0d0*GPL2(0,2)*u**2*v
     .        - 1.0d0/4.0d0*GPL2(0,2)*u**2*v**2
     .        + 1.0d0/2.0d0*GPL2(0,2)*u**3
     .        - 1.0d0/2.0d0*GPL2(0,2)*u**3*v
     .        - 1.0d0/4.0d0*GPL2(0,2)*u**4
     .        + 1.0d0/4.0d0*GPL2(1,2)
     .        + 1.0d0/12.0d0*GPL2(1,2)*u**(-2)
     .        - 1.0d0/6.0d0*GPL2(1,2)*u**(-2)*v
     .        + 1.0d0/12.0d0*GPL2(1,2)*u**(-2)*v**2
         col4 = col4 - 1.0d0/3.0d0*GPL2(1,2)*u**(-1)
     .        + 1.0d0/3.0d0*GPL2(1,2)*u**(-1)*v
     .        + 1.0d0/6.0d0*GPL2(1,2)*u
     .        + 1.0d0/6.0d0*GPL2(1,2)*u*v
     .        - 1.0d0/3.0d0*GPL2(1,2)*u*v**2
     .        + 1.0d0/12.0d0*GPL2(1,2)*u**2
     .        - 5.0d0/6.0d0*GPL2(1,2)*u**2*v
     .        + 1.0d0/4.0d0*GPL2(1,2)*u**2*v**2
     .        - 1.0d0/2.0d0*GPL2(1,2)*u**3
     .        + 1.0d0/2.0d0*GPL2(1,2)*u**3*v
     .        + 1.0d0/4.0d0*GPL2(1,2)*u**4
     .        - 1.0d0/6.0d0*GPL2(2,0)*u
     .        - 1.0d0/6.0d0*GPL2(2,0)*u*v
     .        + 1.0d0/3.0d0*GPL2(2,0)*u*v**2
     .        - 1.0d0/12.0d0*GPL2(2,0)*u**2
         col4 = col4 + 5.0d0/6.0d0*GPL2(2,0)*u**2*v
     .        - 1.0d0/4.0d0*GPL2(2,0)*u**2*v**2
     .        + 1.0d0/2.0d0*GPL2(2,0)*u**3
     .        - 1.0d0/2.0d0*GPL2(2,0)*u**3*v
     .        - 1.0d0/4.0d0*GPL2(2,0)*u**4
     .        + 1.0d0/4.0d0*GPL2(3,0)
     .        + 1.0d0/12.0d0*GPL2(3,0)*omumv**(-2)
     .        - 1.0d0/3.0d0*GPL2(3,0)*omumv**(-1)
     .        - 1.0d0/6.0d0*GPL2(3,0)*v*omumv**(-2)
     .        + 1.0d0/3.0d0*GPL2(3,0)*v*omumv**(-1)
     .        + 1.0d0/12.0d0*GPL2(3,0)*v**2*omumv**(-2)
     .        + 1.0d0/6.0d0*GPL2(3,0)*u
     .        + 1.0d0/6.0d0*GPL2(3,0)*u*v
     .        - 1.0d0/3.0d0*GPL2(3,0)*u*v**2
     .        + 1.0d0/12.0d0*GPL2(3,0)*u**2
         col4 = col4 - 5.0d0/6.0d0*GPL2(3,0)*u**2*v
     .        + 1.0d0/4.0d0*GPL2(3,0)*u**2*v**2
     .        - 1.0d0/2.0d0*GPL2(3,0)*u**3
     .        + 1.0d0/2.0d0*GPL2(3,0)*u**3*v
     .        + 1.0d0/4.0d0*GPL2(3,0)*u**4

         col5 = - 1115.0d0/432.0d0
     .        + 11.0d0/12.0d0*u**(-1)*pi*ci
     .        - 11.0d0/6.0d0*u**(-1)*v*pi*ci
     .        + 11.0d0/12.0d0*u**(-1)*v**2*pi*ci
     .        + 11.0d0/72.0d0*omumv**(-2)*pi**2
     .        + 11.0d0/12.0d0*omumv**(-1)*pi*ci
     .        - 1.0d0/9.0d0*omumv**(-1)*pi**2
     .        - 431.0d0/216.0d0*pi*ci
     .        + 19.0d0/864.0d0*pi**2
     .        - 19.0d0/144.0d0*pi**3*ci
     .        - 7.0d0/8.0d0*zeta3
     .        - 13.0d0/12.0d0*v*omu**(-1)
     .        - 1.0d0/6.0d0*v*omu**(-1)*pi*ci
     .        - 11.0d0/36.0d0*v*omumv**(-2)*pi**2
     .        - 11.0d0/6.0d0*v*omumv**(-1)*pi*ci
         col5 = col5 + 1.0d0/9.0d0*v*omumv**(-1)*pi**2
     .        - 13.0d0/12.0d0*v*upv**(-1)
     .        - 1.0d0/6.0d0*v*upv**(-1)*pi*ci
     .        + 5.0d0/8.0d0*v
     .        + 11.0d0/12.0d0*v*pi*ci
     .        + 1.0d0/12.0d0*v*pi**3*ci
     .        - 1.0d0/2.0d0*v*zeta3
     .        - 13.0d0/12.0d0*v**2*omu**(-2)*pi*ci
     .        + 11.0d0/72.0d0*v**2*omumv**(-2)*pi**2
     .        + 11.0d0/12.0d0*v**2*omumv**(-1)*pi*ci
     .        - 13.0d0/12.0d0*v**2*upv**(-2)*pi*ci
     .        + 37.0d0/24.0d0*v**2*pi*ci
     .        - 1.0d0/12.0d0*v**2*pi**3*ci
     .        + 1.0d0/2.0d0*v**2*zeta3
     .        + 1.0d0/18.0d0*v**3*pi**3*ci
         col5 = col5 - 1.0d0/3.0d0*v**3*zeta3
     .        - 187.0d0/54.0d0*u
     .        - 13.0d0/24.0d0*u*pi*ci
     .        - 19.0d0/24.0d0*u*pi**2
     .        - 1.0d0/2.0d0*u*zeta3
     .        + 187.0d0/54.0d0*u*v
     .        - 53.0d0/24.0d0*u*v*pi*ci
     .        + 37.0d0/72.0d0*u*v*pi**2
     .        - 1.0d0/6.0d0*u*v*pi**3*ci
     .        + u*v*zeta3
     .        + 11.0d0/4.0d0*u*v**2*pi*ci
     .        + 2.0d0/9.0d0*u*v**2*pi**2
     .        + 1.0d0/6.0d0*u*v**2*pi**3*ci
     .        - u*v**2*zeta3
     .        + 187.0d0/54.0d0*u**2
         col5 = col5 + 13.0d0/24.0d0*u**2*pi*ci
     .        + 21.0d0/8.0d0*u**2*pi**2
     .        + 1.0d0/2.0d0*u**2*zeta3
     .        + 11.0d0/4.0d0*u**2*v*pi*ci
     .        - 31.0d0/9.0d0*u**2*v*pi**2
     .        + 1.0d0/6.0d0*u**2*v*pi**3*ci
     .        - u**2*v*zeta3
     .        + 11.0d0/6.0d0*u**2*v**2*pi**2
     .        - 11.0d0/3.0d0*u**3*pi**2
     .        - 1.0d0/3.0d0*u**3*zeta3
     .        + 11.0d0/3.0d0*u**3*v*pi**2
     .        + 11.0d0/6.0d0*u**4*pi**2
     .        - 71.0d0/18.0d0*HPL1(0)
     .        + 11.0d0/12.0d0*HPL1(0)*u**(-1)*v
     .        - 11.0d0/12.0d0*HPL1(0)*u**(-1)*v**2
         col5 = col5 + 11.0d0/12.0d0*HPL1(0)*omumv**(-2)*pi*ci
     .        - 2.0d0/3.0d0*HPL1(0)*omumv**(-1)*pi*ci
     .        - 7.0d0/3.0d0*HPL1(0)*pi*ci
     .        + 1.0d0/48.0d0*HPL1(0)*pi**2
     .        + 1.0d0/6.0d0*HPL1(0)*v*omu**(-1)
     .        - 11.0d0/6.0d0*HPL1(0)*v*omumv**(-2)*pi*ci
     .        + 11.0d0/12.0d0*HPL1(0)*v*omumv**(-1)
     .        + 2.0d0/3.0d0*HPL1(0)*v*omumv**(-1)*pi*ci
     .        + 1.0d0/6.0d0*HPL1(0)*v*upv**(-1)
     .        - 11.0d0/12.0d0*HPL1(0)*v
     .        + 1.0d0/6.0d0*HPL1(0)*v*pi*ci
     .        + 13.0d0/12.0d0*HPL1(0)*v**2*omu**(-2)
     .        + 11.0d0/12.0d0*HPL1(0)*v**2*omumv**(-2)*pi*ci
     .        - 11.0d0/12.0d0*HPL1(0)*v**2*omumv**(-1)
     .        + 13.0d0/12.0d0*HPL1(0)*v**2*upv**(-2)
         col5 = col5 - 37.0d0/24.0d0*HPL1(0)*v**2
     .        - 1.0d0/6.0d0*HPL1(0)*v**2*pi*ci
     .        - 65.0d0/72.0d0*HPL1(0)*u
     .        - 7.0d0/6.0d0*HPL1(0)*u*pi*ci
     .        + 1.0d0/6.0d0*HPL1(0)*u*pi**2
     .        + 263.0d0/72.0d0*HPL1(0)*u*v
     .        + 1.0d0/2.0d0*HPL1(0)*u*v*pi*ci
     .        - 11.0d0/4.0d0*HPL1(0)*u*v**2
     .        + 1.0d0/3.0d0*HPL1(0)*u*v**2*pi*ci
     .        + 65.0d0/72.0d0*HPL1(0)*u**2
     .        + 47.0d0/12.0d0*HPL1(0)*u**2*pi*ci
     .        - 1.0d0/6.0d0*HPL1(0)*u**2*pi**2
     .        - 11.0d0/4.0d0*HPL1(0)*u**2*v
     .        - 31.0d0/6.0d0*HPL1(0)*u**2*v*pi*ci
     .        + 11.0d0/4.0d0*HPL1(0)*u**2*v**2*pi*ci
         col5 = col5 - 11.0d0/2.0d0*HPL1(0)*u**3*pi*ci
     .        + 1.0d0/9.0d0*HPL1(0)*u**3*pi**2
     .        + 11.0d0/2.0d0*HPL1(0)*u**3*v*pi*ci
     .        + 11.0d0/4.0d0*HPL1(0)*u**4*pi*ci
     .        + 13.0d0/18.0d0*HPL1(0)*GPL1(0)
     .        + 11.0d0/12.0d0*HPL1(0)*GPL1(0)*omumv**(-2)
     .        - 2.0d0/3.0d0*HPL1(0)*GPL1(0)*omumv**(-1)
     .        + 5.0d0/12.0d0*HPL1(0)*GPL1(0)*pi*ci
     .        - 11.0d0/6.0d0*HPL1(0)*GPL1(0)*v*omumv**(-2)
     .        + 2.0d0/3.0d0*HPL1(0)*GPL1(0)*v*omumv**(-1)
     .        + 11.0d0/12.0d0*HPL1(0)*GPL1(0)*v**2*omumv**(-2)
     .        - 7.0d0/6.0d0*HPL1(0)*GPL1(0)*u
     .        - 1.0d0/4.0d0*HPL1(0)*GPL1(0)*u*pi*ci
     .        + 5.0d0/6.0d0*HPL1(0)*GPL1(0)*u*v
     .        + 1.0d0/3.0d0*HPL1(0)*GPL1(0)*u*v**2
         col5 = col5 + 47.0d0/12.0d0*HPL1(0)*GPL1(0)*u**2
     .        + 1.0d0/4.0d0*HPL1(0)*GPL1(0)*u**2*pi*ci
     .        - 31.0d0/6.0d0*HPL1(0)*GPL1(0)*u**2*v
     .        + 11.0d0/4.0d0*HPL1(0)*GPL1(0)*u**2*v**2
     .        - 11.0d0/2.0d0*HPL1(0)*GPL1(0)*u**3
     .        - 1.0d0/6.0d0*HPL1(0)*GPL1(0)*u**3*pi*ci
     .        + 11.0d0/2.0d0*HPL1(0)*GPL1(0)*u**3*v
     .        + 11.0d0/4.0d0*HPL1(0)*GPL1(0)*u**4
     .        + 29.0d0/36.0d0*HPL1(0)*GPL1(1)
     .        - 11.0d0/12.0d0*HPL1(0)*GPL1(1)*u**(-2)
     .        + 11.0d0/6.0d0*HPL1(0)*GPL1(1)*u**(-2)*v
     .        - 11.0d0/12.0d0*HPL1(0)*GPL1(1)*u**(-2)*v**2
     .        + 2.0d0/3.0d0*HPL1(0)*GPL1(1)*u**(-1)
     .        - 2.0d0/3.0d0*HPL1(0)*GPL1(1)*u**(-1)*v
     .        + 1.0d0/6.0d0*HPL1(0)*GPL1(1)*v
         col5 = col5 - 1.0d0/6.0d0*HPL1(0)*GPL1(1)*v**2
     .        + 7.0d0/6.0d0*HPL1(0)*GPL1(1)*u
     .        - 7.0d0/6.0d0*HPL1(0)*GPL1(1)*u*v
     .        - 1.0d0/3.0d0*HPL1(0)*GPL1(1)*u*v**2
     .        - 47.0d0/12.0d0*HPL1(0)*GPL1(1)*u**2
     .        + 31.0d0/6.0d0*HPL1(0)*GPL1(1)*u**2*v
     .        - 11.0d0/4.0d0*HPL1(0)*GPL1(1)*u**2*v**2
     .        + 11.0d0/2.0d0*HPL1(0)*GPL1(1)*u**3
     .        - 11.0d0/2.0d0*HPL1(0)*GPL1(1)*u**3*v
     .        - 11.0d0/4.0d0*HPL1(0)*GPL1(1)*u**4
     .        + 13.0d0/18.0d0*HPL1(0)*GPL1(2)
     .        + 11.0d0/12.0d0*HPL1(0)*GPL1(2)*u**(-2)
     .        - 11.0d0/6.0d0*HPL1(0)*GPL1(2)*u**(-2)*v
     .        + 11.0d0/12.0d0*HPL1(0)*GPL1(2)*u**(-2)*v**2
     .        - 2.0d0/3.0d0*HPL1(0)*GPL1(2)*u**(-1)
         col5 = col5 + 2.0d0/3.0d0*HPL1(0)*GPL1(2)*u**(-1)*v
     .        - 1.0d0/4.0d0*HPL1(0)*GPL1(2)*pi*ci
     .        - 1.0d0/4.0d0*HPL1(0)*GPL1(2)*v*pi*ci
     .        + 1.0d0/4.0d0*HPL1(0)*GPL1(2)*v**2*pi*ci
     .        - 1.0d0/6.0d0*HPL1(0)*GPL1(2)*v**3*pi*ci
     .        - 7.0d0/6.0d0*HPL1(0)*GPL1(2)*u
     .        - 1.0d0/4.0d0*HPL1(0)*GPL1(2)*u*pi*ci
     .        + 5.0d0/6.0d0*HPL1(0)*GPL1(2)*u*v
     .        + 1.0d0/2.0d0*HPL1(0)*GPL1(2)*u*v*pi*ci
     .        + 1.0d0/3.0d0*HPL1(0)*GPL1(2)*u*v**2
     .        - 1.0d0/2.0d0*HPL1(0)*GPL1(2)*u*v**2*pi*ci
     .        + 47.0d0/12.0d0*HPL1(0)*GPL1(2)*u**2
     .        + 1.0d0/4.0d0*HPL1(0)*GPL1(2)*u**2*pi*ci
     .        - 31.0d0/6.0d0*HPL1(0)*GPL1(2)*u**2*v
     .        - 1.0d0/2.0d0*HPL1(0)*GPL1(2)*u**2*v*pi*ci
         col5 = col5 + 11.0d0/4.0d0*HPL1(0)*GPL1(2)*u**2*v**2
     .        - 11.0d0/2.0d0*HPL1(0)*GPL1(2)*u**3
     .        - 1.0d0/6.0d0*HPL1(0)*GPL1(2)*u**3*pi*ci
     .        + 11.0d0/2.0d0*HPL1(0)*GPL1(2)*u**3*v
     .        + 11.0d0/4.0d0*HPL1(0)*GPL1(2)*u**4
     .        + 29.0d0/36.0d0*HPL1(0)*GPL1(3)
     .        - 11.0d0/12.0d0*HPL1(0)*GPL1(3)*omumv**(-2)
     .        + 2.0d0/3.0d0*HPL1(0)*GPL1(3)*omumv**(-1)
     .        - 1.0d0/3.0d0*HPL1(0)*GPL1(3)*pi*ci
     .        + 11.0d0/6.0d0*HPL1(0)*GPL1(3)*v*omumv**(-2)
     .        - 2.0d0/3.0d0*HPL1(0)*GPL1(3)*v*omumv**(-1)
     .        - 1.0d0/6.0d0*HPL1(0)*GPL1(3)*v
     .        + 1.0d0/2.0d0*HPL1(0)*GPL1(3)*v*pi*ci
     .        - 11.0d0/12.0d0*HPL1(0)*GPL1(3)*v**2*omumv**(-2)
     .        + 1.0d0/6.0d0*HPL1(0)*GPL1(3)*v**2
         col5 = col5 - 1.0d0/2.0d0*HPL1(0)*GPL1(3)*v**2*pi*ci
     .        + 1.0d0/3.0d0*HPL1(0)*GPL1(3)*v**3*pi*ci
     .        + 7.0d0/6.0d0*HPL1(0)*GPL1(3)*u
     .        + 1.0d0/2.0d0*HPL1(0)*GPL1(3)*u*pi*ci
     .        - 1.0d0/2.0d0*HPL1(0)*GPL1(3)*u*v
     .        - HPL1(0)*GPL1(3)*u*v*pi*ci
     .        - 1.0d0/3.0d0*HPL1(0)*GPL1(3)*u*v**2
     .        + HPL1(0)*GPL1(3)*u*v**2*pi*ci
     .        - 47.0d0/12.0d0*HPL1(0)*GPL1(3)*u**2
     .        - 1.0d0/2.0d0*HPL1(0)*GPL1(3)*u**2*pi*ci
     .        + 31.0d0/6.0d0*HPL1(0)*GPL1(3)*u**2*v
     .        + HPL1(0)*GPL1(3)*u**2*v*pi*ci
     .        - 11.0d0/4.0d0*HPL1(0)*GPL1(3)*u**2*v**2
     .        + 11.0d0/2.0d0*HPL1(0)*GPL1(3)*u**3
     .        + 1.0d0/3.0d0*HPL1(0)*GPL1(3)*u**3*pi*ci
         col5 = col5 - 11.0d0/2.0d0*HPL1(0)*GPL1(3)*u**3*v
     .        - 11.0d0/4.0d0*HPL1(0)*GPL1(3)*u**4
     .        + 2.0d0/3.0d0*HPL1(0)*GPL2(0,0)
     .        - 1.0d0/4.0d0*HPL1(0)*GPL2(0,1)*u
     .        + 1.0d0/4.0d0*HPL1(0)*GPL2(0,1)*u**2
     .        - 1.0d0/6.0d0*HPL1(0)*GPL2(0,1)*u**3
     .        + 7.0d0/12.0d0*HPL1(0)*GPL2(0,2)
     .        - 5.0d0/6.0d0*HPL1(0)*GPL2(0,3)
     .        + 1.0d0/4.0d0*HPL1(0)*GPL2(0,3)*u
     .        - 1.0d0/4.0d0*HPL1(0)*GPL2(0,3)*u**2
     .        + 1.0d0/6.0d0*HPL1(0)*GPL2(0,3)*u**3
     .        - 1.0d0/4.0d0*HPL1(0)*GPL2(1,0)*u
     .        + 1.0d0/4.0d0*HPL1(0)*GPL2(1,0)*u**2
     .        - 1.0d0/6.0d0*HPL1(0)*GPL2(1,0)*u**3
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(1,1)*u
         col5 = col5 - 1.0d0/2.0d0*HPL1(0)*GPL2(1,1)*u**2
     .        + 1.0d0/3.0d0*HPL1(0)*GPL2(1,1)*u**3
     .        - 5.0d0/6.0d0*HPL1(0)*GPL2(1,2)
     .        - 1.0d0/4.0d0*HPL1(0)*GPL2(1,2)*u
     .        + 1.0d0/4.0d0*HPL1(0)*GPL2(1,2)*u**2
     .        - 1.0d0/6.0d0*HPL1(0)*GPL2(1,2)*u**3
     .        + 7.0d0/12.0d0*HPL1(0)*GPL2(2,0)
     .        - 2.0d0/3.0d0*HPL1(0)*GPL2(2,1)
     .        - 1.0d0/4.0d0*HPL1(0)*GPL2(2,1)*v
     .        + 1.0d0/4.0d0*HPL1(0)*GPL2(2,1)*v**2
     .        - 1.0d0/6.0d0*HPL1(0)*GPL2(2,1)*v**3
     .        - 1.0d0/4.0d0*HPL1(0)*GPL2(2,1)*u
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(2,1)*u*v
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(2,1)*u*v**2
     .        + 1.0d0/4.0d0*HPL1(0)*GPL2(2,1)*u**2
         col5 = col5 - 1.0d0/2.0d0*HPL1(0)*GPL2(2,1)*u**2*v
     .        - 1.0d0/6.0d0*HPL1(0)*GPL2(2,1)*u**3
     .        + 2.0d0/3.0d0*HPL1(0)*GPL2(2,2)
     .        - 1.0d0/6.0d0*HPL1(0)*GPL2(2,3)
     .        + 1.0d0/4.0d0*HPL1(0)*GPL2(2,3)*v
     .        - 1.0d0/4.0d0*HPL1(0)*GPL2(2,3)*v**2
     .        + 1.0d0/6.0d0*HPL1(0)*GPL2(2,3)*v**3
     .        + 1.0d0/4.0d0*HPL1(0)*GPL2(2,3)*u
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(2,3)*u*v
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(2,3)*u*v**2
     .        - 1.0d0/4.0d0*HPL1(0)*GPL2(2,3)*u**2
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(2,3)*u**2*v
     .        + 1.0d0/6.0d0*HPL1(0)*GPL2(2,3)*u**3
     .        - HPL1(0)*GPL2(3,0)
     .        + 1.0d0/4.0d0*HPL1(0)*GPL2(3,0)*v
         col5 = col5 - 1.0d0/4.0d0*HPL1(0)*GPL2(3,0)*v**2
     .        + 1.0d0/6.0d0*HPL1(0)*GPL2(3,0)*v**3
     .        + 1.0d0/4.0d0*HPL1(0)*GPL2(3,0)*u
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(3,0)*u*v
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(3,0)*u*v**2
     .        - 1.0d0/4.0d0*HPL1(0)*GPL2(3,0)*u**2
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(3,0)*u**2*v
     .        + 1.0d0/6.0d0*HPL1(0)*GPL2(3,0)*u**3
     .        - 1.0d0/6.0d0*HPL1(0)*GPL2(3,2)
     .        + 1.0d0/4.0d0*HPL1(0)*GPL2(3,2)*v
     .        - 1.0d0/4.0d0*HPL1(0)*GPL2(3,2)*v**2
     .        + 1.0d0/6.0d0*HPL1(0)*GPL2(3,2)*v**3
     .        + 1.0d0/4.0d0*HPL1(0)*GPL2(3,2)*u
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(3,2)*u*v
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(3,2)*u*v**2
         col5 = col5 - 1.0d0/4.0d0*HPL1(0)*GPL2(3,2)*u**2
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(3,2)*u**2*v
     .        + 1.0d0/6.0d0*HPL1(0)*GPL2(3,2)*u**3
     .        + 1.0d0/3.0d0*HPL1(0)*GPL2(3,3)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(3,3)*v
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(3,3)*v**2
     .        - 1.0d0/3.0d0*HPL1(0)*GPL2(3,3)*v**3
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(3,3)*u
     .        + HPL1(0)*GPL2(3,3)*u*v
     .        - HPL1(0)*GPL2(3,3)*u*v**2
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(3,3)*u**2
     .        - HPL1(0)*GPL2(3,3)*u**2*v
     .        - 1.0d0/3.0d0*HPL1(0)*GPL2(3,3)*u**3
     .        - 143.0d0/216.0d0*HPL1(1)
     .        - 11.0d0/12.0d0*HPL1(1)*u**(-1)
         col5 = col5 + 11.0d0/6.0d0*HPL1(1)*u**(-1)*v
     .        - 11.0d0/12.0d0*HPL1(1)*u**(-1)*v**2
     .        - 7.0d0/12.0d0*HPL1(1)*pi*ci
     .        + 5.0d0/144.0d0*HPL1(1)*pi**2
     .        + 1.0d0/6.0d0*HPL1(1)*v*omu**(-1)
     .        + 13.0d0/12.0d0*HPL1(1)*v**2*omu**(-2)
     .        - 37.0d0/24.0d0*HPL1(1)*v**2
     .        - 157.0d0/72.0d0*HPL1(1)*u
     .        - 13.0d0/12.0d0*HPL1(1)*u*pi*ci
     .        + 199.0d0/72.0d0*HPL1(1)*u*v
     .        + 3.0d0/4.0d0*HPL1(1)*u*v*pi*ci
     .        - 11.0d0/4.0d0*HPL1(1)*u*v**2
     .        + 1.0d0/3.0d0*HPL1(1)*u*v**2*pi*ci
     .        + 155.0d0/36.0d0*HPL1(1)*u**2
     .        + 23.0d0/6.0d0*HPL1(1)*u**2*pi*ci
         col5 = col5 - 11.0d0/2.0d0*HPL1(1)*u**2*v
     .        - 31.0d0/6.0d0*HPL1(1)*u**2*v*pi*ci
     .        + 11.0d0/4.0d0*HPL1(1)*u**2*v**2*pi*ci
     .        - 11.0d0/4.0d0*HPL1(1)*u**3
     .        - 11.0d0/2.0d0*HPL1(1)*u**3*pi*ci
     .        + 11.0d0/2.0d0*HPL1(1)*u**3*v*pi*ci
     .        + 11.0d0/4.0d0*HPL1(1)*u**4*pi*ci
     .        + 1.0d0/36.0d0*HPL1(1)*GPL1(0)
     .        + 1.0d0/4.0d0*HPL1(1)*GPL1(0)*pi*ci
     .        - 13.0d0/12.0d0*HPL1(1)*GPL1(0)*u
     .        + 3.0d0/4.0d0*HPL1(1)*GPL1(0)*u*v
     .        + 1.0d0/3.0d0*HPL1(1)*GPL1(0)*u*v**2
     .        + 23.0d0/6.0d0*HPL1(1)*GPL1(0)*u**2
     .        - 31.0d0/6.0d0*HPL1(1)*GPL1(0)*u**2*v
     .        + 11.0d0/4.0d0*HPL1(1)*GPL1(0)*u**2*v**2
         col5 = col5 - 11.0d0/2.0d0*HPL1(1)*GPL1(0)*u**3
     .        + 11.0d0/2.0d0*HPL1(1)*GPL1(0)*u**3*v
     .        + 11.0d0/4.0d0*HPL1(1)*GPL1(0)*u**4
     .        + 29.0d0/36.0d0*HPL1(1)*GPL1(1)
     .        - 11.0d0/12.0d0*HPL1(1)*GPL1(1)*u**(-2)
     .        + 11.0d0/6.0d0*HPL1(1)*GPL1(1)*u**(-2)*v
     .        - 11.0d0/12.0d0*HPL1(1)*GPL1(1)*u**(-2)*v**2
     .        + 2.0d0/3.0d0*HPL1(1)*GPL1(1)*u**(-1)
     .        - 2.0d0/3.0d0*HPL1(1)*GPL1(1)*u**(-1)*v
     .        + 1.0d0/6.0d0*HPL1(1)*GPL1(1)*pi*ci
     .        + 1.0d0/6.0d0*HPL1(1)*GPL1(1)*v
     .        - 1.0d0/6.0d0*HPL1(1)*GPL1(1)*v**2
     .        + 7.0d0/6.0d0*HPL1(1)*GPL1(1)*u
     .        - 1.0d0/4.0d0*HPL1(1)*GPL1(1)*u*pi*ci
     .        - 7.0d0/6.0d0*HPL1(1)*GPL1(1)*u*v
         col5 = col5 - 1.0d0/3.0d0*HPL1(1)*GPL1(1)*u*v**2
     .        - 47.0d0/12.0d0*HPL1(1)*GPL1(1)*u**2
     .        + 1.0d0/4.0d0*HPL1(1)*GPL1(1)*u**2*pi*ci
     .        + 31.0d0/6.0d0*HPL1(1)*GPL1(1)*u**2*v
     .        - 11.0d0/4.0d0*HPL1(1)*GPL1(1)*u**2*v**2
     .        + 11.0d0/2.0d0*HPL1(1)*GPL1(1)*u**3
     .        - 1.0d0/6.0d0*HPL1(1)*GPL1(1)*u**3*pi*ci
     .        - 11.0d0/2.0d0*HPL1(1)*GPL1(1)*u**3*v
     .        - 11.0d0/4.0d0*HPL1(1)*GPL1(1)*u**4
     .        + 11.0d0/12.0d0*HPL1(1)*GPL1(2)
     .        + 1.0d0/3.0d0*HPL1(1)*GPL1(2)*pi*ci
     .        - 1.0d0/6.0d0*HPL1(1)*GPL1(3)*pi*ci
     .        + 1.0d0/4.0d0*HPL1(1)*GPL1(3)*v*pi*ci
     .        - 1.0d0/4.0d0*HPL1(1)*GPL1(3)*v**2*pi*ci
     .        + 1.0d0/6.0d0*HPL1(1)*GPL1(3)*v**3*pi*ci
         col5 = col5 + 1.0d0/4.0d0*HPL1(1)*GPL1(3)*u*pi*ci
     .        - 1.0d0/2.0d0*HPL1(1)*GPL1(3)*u*v*pi*ci
     .        + 1.0d0/2.0d0*HPL1(1)*GPL1(3)*u*v**2*pi*ci
     .        - 1.0d0/4.0d0*HPL1(1)*GPL1(3)*u**2*pi*ci
     .        + 1.0d0/2.0d0*HPL1(1)*GPL1(3)*u**2*v*pi*ci
     .        + 1.0d0/6.0d0*HPL1(1)*GPL1(3)*u**3*pi*ci
     .        + 1.0d0/3.0d0*HPL1(1)*GPL2(0,0)
     .        - 1.0d0/4.0d0*HPL1(1)*GPL2(0,1)*u
     .        + 1.0d0/4.0d0*HPL1(1)*GPL2(0,1)*u**2
     .        - 1.0d0/6.0d0*HPL1(1)*GPL2(0,1)*u**3
     .        + 1.0d0/3.0d0*HPL1(1)*GPL2(0,2)
     .        - 1.0d0/4.0d0*HPL1(1)*GPL2(1,0)*u
     .        + 1.0d0/4.0d0*HPL1(1)*GPL2(1,0)*u**2
     .        - 1.0d0/6.0d0*HPL1(1)*GPL2(1,0)*u**3
     .        + 1.0d0/2.0d0*HPL1(1)*GPL2(1,1)*u
         col5 = col5 - 1.0d0/2.0d0*HPL1(1)*GPL2(1,1)*u**2
     .        + 1.0d0/3.0d0*HPL1(1)*GPL2(1,1)*u**3
     .        - 2.0d0/3.0d0*HPL1(1)*GPL2(1,2)
     .        + 1.0d0/3.0d0*HPL1(1)*GPL2(2,0)
     .        - 2.0d0/3.0d0*HPL1(1)*GPL2(2,1)
     .        - 1.0d0/4.0d0*HPL1(1)*GPL2(2,1)*v
     .        + 1.0d0/4.0d0*HPL1(1)*GPL2(2,1)*v**2
     .        - 1.0d0/6.0d0*HPL1(1)*GPL2(2,1)*v**3
     .        - 1.0d0/4.0d0*HPL1(1)*GPL2(2,1)*u
     .        + 1.0d0/2.0d0*HPL1(1)*GPL2(2,1)*u*v
     .        - 1.0d0/2.0d0*HPL1(1)*GPL2(2,1)*u*v**2
     .        + 1.0d0/4.0d0*HPL1(1)*GPL2(2,1)*u**2
     .        - 1.0d0/2.0d0*HPL1(1)*GPL2(2,1)*u**2*v
     .        - 1.0d0/6.0d0*HPL1(1)*GPL2(2,1)*u**3
     .        - 1.0d0/6.0d0*HPL1(1)*GPL2(3,0)
         col5 = col5 + 1.0d0/4.0d0*HPL1(1)*GPL2(3,0)*v
     .        - 1.0d0/4.0d0*HPL1(1)*GPL2(3,0)*v**2
     .        + 1.0d0/6.0d0*HPL1(1)*GPL2(3,0)*v**3
     .        + 1.0d0/4.0d0*HPL1(1)*GPL2(3,0)*u
     .        - 1.0d0/2.0d0*HPL1(1)*GPL2(3,0)*u*v
     .        + 1.0d0/2.0d0*HPL1(1)*GPL2(3,0)*u*v**2
     .        - 1.0d0/4.0d0*HPL1(1)*GPL2(3,0)*u**2
     .        + 1.0d0/2.0d0*HPL1(1)*GPL2(3,0)*u**2*v
     .        + 1.0d0/6.0d0*HPL1(1)*GPL2(3,0)*u**3
     .        - 34.0d0/9.0d0*HPL2(0,0)
     .        - 11.0d0/12.0d0*HPL2(0,0)*omumv**(-2)
     .        + 2.0d0/3.0d0*HPL2(0,0)*omumv**(-1)
     .        - 7.0d0/6.0d0*HPL2(0,0)*pi*ci
     .        + 11.0d0/6.0d0*HPL2(0,0)*v*omumv**(-2)
     .        - 2.0d0/3.0d0*HPL2(0,0)*v*omumv**(-1)
         col5 = col5 - 1.0d0/6.0d0*HPL2(0,0)*v
     .        + 1.0d0/4.0d0*HPL2(0,0)*v*pi*ci
     .        - 11.0d0/12.0d0*HPL2(0,0)*v**2*omumv**(-2)
     .        + 1.0d0/6.0d0*HPL2(0,0)*v**2
     .        - 1.0d0/4.0d0*HPL2(0,0)*v**2*pi*ci
     .        + 1.0d0/6.0d0*HPL2(0,0)*v**3*pi*ci
     .        + 7.0d0/6.0d0*HPL2(0,0)*u
     .        + 1.0d0/2.0d0*HPL2(0,0)*u*pi*ci
     .        - 1.0d0/2.0d0*HPL2(0,0)*u*v
     .        - 1.0d0/2.0d0*HPL2(0,0)*u*v*pi*ci
     .        - 1.0d0/3.0d0*HPL2(0,0)*u*v**2
     .        + 1.0d0/2.0d0*HPL2(0,0)*u*v**2*pi*ci
     .        - 47.0d0/12.0d0*HPL2(0,0)*u**2
     .        - 1.0d0/2.0d0*HPL2(0,0)*u**2*pi*ci
     .        + 31.0d0/6.0d0*HPL2(0,0)*u**2*v
         col5 = col5 + 1.0d0/2.0d0*HPL2(0,0)*u**2*v*pi*ci
     .        - 11.0d0/4.0d0*HPL2(0,0)*u**2*v**2
     .        + 11.0d0/2.0d0*HPL2(0,0)*u**3
     .        + 1.0d0/3.0d0*HPL2(0,0)*u**3*pi*ci
     .        - 11.0d0/2.0d0*HPL2(0,0)*u**3*v
     .        - 11.0d0/4.0d0*HPL2(0,0)*u**4
     .        - 11.0d0/6.0d0*HPL2(0,0)*GPL1(0)
     .        + 1.0d0/4.0d0*HPL2(0,0)*GPL1(0)*u
     .        - 1.0d0/4.0d0*HPL2(0,0)*GPL1(0)*u**2
     .        + 1.0d0/6.0d0*HPL2(0,0)*GPL1(0)*u**3
     .        + HPL2(0,0)*GPL1(1)
     .        - 7.0d0/6.0d0*HPL2(0,0)*GPL1(2)
     .        + 1.0d0/4.0d0*HPL2(0,0)*GPL1(2)*v
     .        - 1.0d0/4.0d0*HPL2(0,0)*GPL1(2)*v**2
     .        + 1.0d0/6.0d0*HPL2(0,0)*GPL1(2)*v**3
         col5 = col5 + 1.0d0/4.0d0*HPL2(0,0)*GPL1(2)*u
     .        - 1.0d0/2.0d0*HPL2(0,0)*GPL1(2)*u*v
     .        + 1.0d0/2.0d0*HPL2(0,0)*GPL1(2)*u*v**2
     .        - 1.0d0/4.0d0*HPL2(0,0)*GPL1(2)*u**2
     .        + 1.0d0/2.0d0*HPL2(0,0)*GPL1(2)*u**2*v
     .        + 1.0d0/6.0d0*HPL2(0,0)*GPL1(2)*u**3
     .        + 4.0d0/3.0d0*HPL2(0,0)*GPL1(3)
     .        - 1.0d0/2.0d0*HPL2(0,0)*GPL1(3)*v
     .        + 1.0d0/2.0d0*HPL2(0,0)*GPL1(3)*v**2
     .        - 1.0d0/3.0d0*HPL2(0,0)*GPL1(3)*v**3
     .        - 1.0d0/2.0d0*HPL2(0,0)*GPL1(3)*u
     .        + HPL2(0,0)*GPL1(3)*u*v
     .        - HPL2(0,0)*GPL1(3)*u*v**2
     .        + 1.0d0/2.0d0*HPL2(0,0)*GPL1(3)*u**2
     .        - HPL2(0,0)*GPL1(3)*u**2*v
         col5 = col5 - 1.0d0/3.0d0*HPL2(0,0)*GPL1(3)*u**3
     .        - 55.0d0/36.0d0*HPL2(0,1)
     .        - 7.0d0/12.0d0*HPL2(0,1)*pi*ci
     .        - 1.0d0/6.0d0*HPL2(0,1)*v
     .        + 1.0d0/6.0d0*HPL2(0,1)*v**2
     .        + 1.0d0/4.0d0*HPL2(0,1)*u*pi*ci
     .        + 1.0d0/3.0d0*HPL2(0,1)*u*v
     .        - 1.0d0/4.0d0*HPL2(0,1)*u**2*pi*ci
     .        + 1.0d0/6.0d0*HPL2(0,1)*u**3*pi*ci
     .        - 7.0d0/12.0d0*HPL2(0,1)*GPL1(0)
     .        + 1.0d0/4.0d0*HPL2(0,1)*GPL1(0)*u
     .        - 1.0d0/4.0d0*HPL2(0,1)*GPL1(0)*u**2
     .        + 1.0d0/6.0d0*HPL2(0,1)*GPL1(0)*u**3
     .        + 5.0d0/6.0d0*HPL2(0,1)*GPL1(1)
     .        - 1.0d0/4.0d0*HPL2(0,1)*GPL1(1)*u
         col5 = col5 + 1.0d0/4.0d0*HPL2(0,1)*GPL1(1)*u**2
     .        - 1.0d0/6.0d0*HPL2(0,1)*GPL1(1)*u**3
     .        + 1.0d0/4.0d0*HPL2(0,1)*GPL1(2)*v
     .        - 1.0d0/4.0d0*HPL2(0,1)*GPL1(2)*v**2
     .        + 1.0d0/6.0d0*HPL2(0,1)*GPL1(2)*v**3
     .        + 1.0d0/4.0d0*HPL2(0,1)*GPL1(2)*u
     .        - 1.0d0/2.0d0*HPL2(0,1)*GPL1(2)*u*v
     .        + 1.0d0/2.0d0*HPL2(0,1)*GPL1(2)*u*v**2
     .        - 1.0d0/4.0d0*HPL2(0,1)*GPL1(2)*u**2
     .        + 1.0d0/2.0d0*HPL2(0,1)*GPL1(2)*u**2*v
     .        + 1.0d0/6.0d0*HPL2(0,1)*GPL1(2)*u**3
     .        + 1.0d0/6.0d0*HPL2(0,1)*GPL1(3)
     .        - 1.0d0/4.0d0*HPL2(0,1)*GPL1(3)*v
     .        + 1.0d0/4.0d0*HPL2(0,1)*GPL1(3)*v**2
     .        - 1.0d0/6.0d0*HPL2(0,1)*GPL1(3)*v**3
         col5 = col5 - 1.0d0/4.0d0*HPL2(0,1)*GPL1(3)*u
     .        + 1.0d0/2.0d0*HPL2(0,1)*GPL1(3)*u*v
     .        - 1.0d0/2.0d0*HPL2(0,1)*GPL1(3)*u*v**2
     .        + 1.0d0/4.0d0*HPL2(0,1)*GPL1(3)*u**2
     .        - 1.0d0/2.0d0*HPL2(0,1)*GPL1(3)*u**2*v
     .        - 1.0d0/6.0d0*HPL2(0,1)*GPL1(3)*u**3
     .        - 16.0d0/9.0d0*HPL2(1,0)
     .        + 11.0d0/12.0d0*HPL2(1,0)*omumv**(-2)
     .        - 2.0d0/3.0d0*HPL2(1,0)*omumv**(-1)
     .        - 7.0d0/12.0d0*HPL2(1,0)*pi*ci
     .        - 11.0d0/6.0d0*HPL2(1,0)*v*omumv**(-2)
     .        + 2.0d0/3.0d0*HPL2(1,0)*v*omumv**(-1)
     .        + 1.0d0/4.0d0*HPL2(1,0)*v*pi*ci
     .        + 11.0d0/12.0d0*HPL2(1,0)*v**2*omumv**(-2)
     .        - 1.0d0/4.0d0*HPL2(1,0)*v**2*pi*ci
         col5 = col5 + 1.0d0/6.0d0*HPL2(1,0)*v**3*pi*ci
     .        + HPL2(1,0)*u
     .        + 1.0d0/4.0d0*HPL2(1,0)*u*pi*ci
     .        - 2.0d0/3.0d0*HPL2(1,0)*u*v
     .        - 1.0d0/2.0d0*HPL2(1,0)*u*v*pi*ci
     .        - 1.0d0/3.0d0*HPL2(1,0)*u*v**2
     .        + 1.0d0/2.0d0*HPL2(1,0)*u*v**2*pi*ci
     .        - 15.0d0/4.0d0*HPL2(1,0)*u**2
     .        - 1.0d0/4.0d0*HPL2(1,0)*u**2*pi*ci
     .        + 31.0d0/6.0d0*HPL2(1,0)*u**2*v
     .        + 1.0d0/2.0d0*HPL2(1,0)*u**2*v*pi*ci
     .        - 11.0d0/4.0d0*HPL2(1,0)*u**2*v**2
     .        + 11.0d0/2.0d0*HPL2(1,0)*u**3
     .        + 1.0d0/6.0d0*HPL2(1,0)*u**3*pi*ci
     .        - 11.0d0/2.0d0*HPL2(1,0)*u**3*v
         col5 = col5 - 11.0d0/4.0d0*HPL2(1,0)*u**4
     .        - 7.0d0/12.0d0*HPL2(1,0)*GPL1(0)
     .        + 5.0d0/6.0d0*HPL2(1,0)*GPL1(1)
     .        + 1.0d0/4.0d0*HPL2(1,0)*GPL1(1)*u
     .        - 1.0d0/4.0d0*HPL2(1,0)*GPL1(1)*u**2
     .        + 1.0d0/6.0d0*HPL2(1,0)*GPL1(1)*u**3
     .        - 2.0d0/3.0d0*HPL2(1,0)*GPL1(2)
     .        + 1.0d0/6.0d0*HPL2(1,0)*GPL1(3)
     .        - 1.0d0/4.0d0*HPL2(1,0)*GPL1(3)*v
     .        + 1.0d0/4.0d0*HPL2(1,0)*GPL1(3)*v**2
     .        - 1.0d0/6.0d0*HPL2(1,0)*GPL1(3)*v**3
     .        - 1.0d0/4.0d0*HPL2(1,0)*GPL1(3)*u
     .        + 1.0d0/2.0d0*HPL2(1,0)*GPL1(3)*u*v
     .        - 1.0d0/2.0d0*HPL2(1,0)*GPL1(3)*u*v**2
     .        + 1.0d0/4.0d0*HPL2(1,0)*GPL1(3)*u**2
         col5 = col5 - 1.0d0/2.0d0*HPL2(1,0)*GPL1(3)*u**2*v
     .        - 1.0d0/6.0d0*HPL2(1,0)*GPL1(3)*u**3
     .        - 11.0d0/12.0d0*HPL2(1,1)
     .        - 1.0d0/3.0d0*HPL2(1,1)*pi*ci
     .        - 1.0d0/3.0d0*HPL2(1,1)*GPL1(0)
     .        + 2.0d0/3.0d0*HPL2(1,1)*GPL1(1)
     .        + 13.0d0/6.0d0*HPL3(0,0,0)
     .        - 1.0d0/4.0d0*HPL3(0,0,0)*v
     .        + 1.0d0/4.0d0*HPL3(0,0,0)*v**2
     .        - 1.0d0/6.0d0*HPL3(0,0,0)*v**3
     .        - 1.0d0/2.0d0*HPL3(0,0,0)*u
     .        + 1.0d0/2.0d0*HPL3(0,0,0)*u*v
     .        - 1.0d0/2.0d0*HPL3(0,0,0)*u*v**2
     .        + 1.0d0/2.0d0*HPL3(0,0,0)*u**2
     .        - 1.0d0/2.0d0*HPL3(0,0,0)*u**2*v
         col5 = col5 - 1.0d0/3.0d0*HPL3(0,0,0)*u**3
     .        + 1.0d0/6.0d0*HPL3(0,0,1)
     .        - 1.0d0/4.0d0*HPL3(0,0,1)*v
     .        + 1.0d0/4.0d0*HPL3(0,0,1)*v**2
     .        - 1.0d0/6.0d0*HPL3(0,0,1)*v**3
     .        - 1.0d0/4.0d0*HPL3(0,0,1)*u
     .        + 1.0d0/2.0d0*HPL3(0,0,1)*u*v
     .        - 1.0d0/2.0d0*HPL3(0,0,1)*u*v**2
     .        + 1.0d0/4.0d0*HPL3(0,0,1)*u**2
     .        - 1.0d0/2.0d0*HPL3(0,0,1)*u**2*v
     .        - 1.0d0/6.0d0*HPL3(0,0,1)*u**3
     .        + 1.0d0/6.0d0*HPL3(0,1,0)
     .        - 1.0d0/4.0d0*HPL3(0,1,0)*u
     .        + 1.0d0/4.0d0*HPL3(0,1,0)*u**2
     .        - 1.0d0/6.0d0*HPL3(0,1,0)*u**3
         col5 = col5 + 1.0d0/6.0d0*HPL3(1,0,0)
     .        - 1.0d0/4.0d0*HPL3(1,0,0)*v
     .        + 1.0d0/4.0d0*HPL3(1,0,0)*v**2
     .        - 1.0d0/6.0d0*HPL3(1,0,0)*v**3
     .        - 1.0d0/4.0d0*HPL3(1,0,0)*u
     .        + 1.0d0/2.0d0*HPL3(1,0,0)*u*v
     .        - 1.0d0/2.0d0*HPL3(1,0,0)*u*v**2
     .        + 1.0d0/4.0d0*HPL3(1,0,0)*u**2
     .        - 1.0d0/2.0d0*HPL3(1,0,0)*u**2*v
     .        - 1.0d0/6.0d0*HPL3(1,0,0)*u**3
     .        - 1.0d0/4.0d0*HPL3(1,0,1)*v
     .        + 1.0d0/4.0d0*HPL3(1,0,1)*v**2
     .        - 1.0d0/6.0d0*HPL3(1,0,1)*v**3
     .        - 1.0d0/4.0d0*HPL3(1,0,1)*u
     .        + 1.0d0/2.0d0*HPL3(1,0,1)*u*v
         col5 = col5 - 1.0d0/2.0d0*HPL3(1,0,1)*u*v**2
     .        + 1.0d0/4.0d0*HPL3(1,0,1)*u**2
     .        - 1.0d0/2.0d0*HPL3(1,0,1)*u**2*v
     .        - 1.0d0/6.0d0*HPL3(1,0,1)*u**3
     .        + 1.0d0/3.0d0*HPL3(1,1,0)
     .        + 139.0d0/108.0d0*GPL1(0)
     .        + 11.0d0/12.0d0*GPL1(0)*omumv**(-1)
     .        + 7.0d0/12.0d0*GPL1(0)*pi*ci
     .        + 7.0d0/144.0d0*GPL1(0)*pi**2
     .        - 11.0d0/6.0d0*GPL1(0)*v*omumv**(-1)
     .        - 1.0d0/6.0d0*GPL1(0)*v*upv**(-1)
     .        + 11.0d0/12.0d0*GPL1(0)*v
     .        + 11.0d0/12.0d0*GPL1(0)*v**2*omumv**(-1)
     .        - 13.0d0/12.0d0*GPL1(0)*v**2*upv**(-2)
     .        - 131.0d0/72.0d0*GPL1(0)*u
         col5 = col5 + 13.0d0/12.0d0*GPL1(0)*u*pi*ci
     .        - 1.0d0/12.0d0*GPL1(0)*u*pi**2
     .        - 25.0d0/72.0d0*GPL1(0)*u*v
     .        - 3.0d0/4.0d0*GPL1(0)*u*v*pi*ci
     .        - 1.0d0/3.0d0*GPL1(0)*u*v**2*pi*ci
     .        + 71.0d0/18.0d0*GPL1(0)*u**2
     .        - 23.0d0/6.0d0*GPL1(0)*u**2*pi*ci
     .        + 1.0d0/12.0d0*GPL1(0)*u**2*pi**2
     .        - 11.0d0/4.0d0*GPL1(0)*u**2*v
     .        + 31.0d0/6.0d0*GPL1(0)*u**2*v*pi*ci
     .        - 11.0d0/4.0d0*GPL1(0)*u**2*v**2*pi*ci
     .        - 11.0d0/4.0d0*GPL1(0)*u**3
     .        + 11.0d0/2.0d0*GPL1(0)*u**3*pi*ci
     .        - 1.0d0/18.0d0*GPL1(0)*u**3*pi**2
     .        - 11.0d0/2.0d0*GPL1(0)*u**3*v*pi*ci
         col5 = col5 - 11.0d0/4.0d0*GPL1(0)*u**4*pi*ci
     .        + 11.0d0/12.0d0*GPL1(1)*u**(-2)*pi*ci
     .        - 11.0d0/6.0d0*GPL1(1)*u**(-2)*v*pi*ci
     .        + 11.0d0/12.0d0*GPL1(1)*u**(-2)*v**2*pi*ci
     .        - 2.0d0/3.0d0*GPL1(1)*u**(-1)*pi*ci
     .        + 2.0d0/3.0d0*GPL1(1)*u**(-1)*v*pi*ci
     .        - 29.0d0/36.0d0*GPL1(1)*pi*ci
     .        + 1.0d0/2.0d0*GPL1(1)*pi**2
     .        - 1.0d0/6.0d0*GPL1(1)*v*pi*ci
     .        + 1.0d0/6.0d0*GPL1(1)*v**2*pi*ci
     .        - 7.0d0/6.0d0*GPL1(1)*u*pi*ci
     .        - 1.0d0/12.0d0*GPL1(1)*u*pi**2
     .        + 7.0d0/6.0d0*GPL1(1)*u*v*pi*ci
     .        + 1.0d0/3.0d0*GPL1(1)*u*v**2*pi*ci
     .        + 47.0d0/12.0d0*GPL1(1)*u**2*pi*ci
         col5 = col5 + 1.0d0/12.0d0*GPL1(1)*u**2*pi**2
     .        - 31.0d0/6.0d0*GPL1(1)*u**2*v*pi*ci
     .        + 11.0d0/4.0d0*GPL1(1)*u**2*v**2*pi*ci
     .        - 11.0d0/2.0d0*GPL1(1)*u**3*pi*ci
     .        - 1.0d0/18.0d0*GPL1(1)*u**3*pi**2
     .        + 11.0d0/2.0d0*GPL1(1)*u**3*v*pi*ci
     .        + 11.0d0/4.0d0*GPL1(1)*u**4*pi*ci
     .        + 143.0d0/216.0d0*GPL1(2)
     .        + 11.0d0/12.0d0*GPL1(2)*u**(-1)
     .        - 11.0d0/6.0d0*GPL1(2)*u**(-1)*v
     .        + 11.0d0/12.0d0*GPL1(2)*u**(-1)*v**2
     .        + 7.0d0/12.0d0*GPL1(2)*pi*ci
     .        - 13.0d0/144.0d0*GPL1(2)*pi**2
     .        - 1.0d0/6.0d0*GPL1(2)*v*omu**(-1)
     .        - 13.0d0/12.0d0*GPL1(2)*v**2*omu**(-2)
         col5 = col5 + 37.0d0/24.0d0*GPL1(2)*v**2
     .        + 157.0d0/72.0d0*GPL1(2)*u
     .        + 13.0d0/12.0d0*GPL1(2)*u*pi*ci
     .        - 199.0d0/72.0d0*GPL1(2)*u*v
     .        - 3.0d0/4.0d0*GPL1(2)*u*v*pi*ci
     .        + 11.0d0/4.0d0*GPL1(2)*u*v**2
     .        - 1.0d0/3.0d0*GPL1(2)*u*v**2*pi*ci
     .        - 155.0d0/36.0d0*GPL1(2)*u**2
     .        - 23.0d0/6.0d0*GPL1(2)*u**2*pi*ci
     .        + 11.0d0/2.0d0*GPL1(2)*u**2*v
     .        + 31.0d0/6.0d0*GPL1(2)*u**2*v*pi*ci
     .        - 11.0d0/4.0d0*GPL1(2)*u**2*v**2*pi*ci
     .        + 11.0d0/4.0d0*GPL1(2)*u**3
     .        + 11.0d0/2.0d0*GPL1(2)*u**3*pi*ci
     .        - 11.0d0/2.0d0*GPL1(2)*u**3*v*pi*ci
         col5 = col5 - 11.0d0/4.0d0*GPL1(2)*u**4*pi*ci
     .        + 11.0d0/12.0d0*GPL1(3)*omumv**(-2)*pi*ci
     .        - 2.0d0/3.0d0*GPL1(3)*omumv**(-1)*pi*ci
     .        - 29.0d0/36.0d0*GPL1(3)*pi*ci
     .        + 7.0d0/18.0d0*GPL1(3)*pi**2
     .        - 11.0d0/6.0d0*GPL1(3)*v*omumv**(-2)*pi*ci
     .        + 2.0d0/3.0d0*GPL1(3)*v*omumv**(-1)*pi*ci
     .        + 1.0d0/6.0d0*GPL1(3)*v*pi*ci
     .        + 1.0d0/6.0d0*GPL1(3)*v*pi**2
     .        + 11.0d0/12.0d0*GPL1(3)*v**2*omumv**(-2)*pi*ci
     .        - 1.0d0/6.0d0*GPL1(3)*v**2*pi*ci
     .        - 1.0d0/6.0d0*GPL1(3)*v**2*pi**2
     .        + 1.0d0/9.0d0*GPL1(3)*v**3*pi**2
     .        - 7.0d0/6.0d0*GPL1(3)*u*pi*ci
     .        + 1.0d0/6.0d0*GPL1(3)*u*pi**2
         col5 = col5 + 1.0d0/2.0d0*GPL1(3)*u*v*pi*ci
     .        - 1.0d0/3.0d0*GPL1(3)*u*v*pi**2
     .        + 1.0d0/3.0d0*GPL1(3)*u*v**2*pi*ci
     .        + 1.0d0/3.0d0*GPL1(3)*u*v**2*pi**2
     .        + 47.0d0/12.0d0*GPL1(3)*u**2*pi*ci
     .        - 1.0d0/6.0d0*GPL1(3)*u**2*pi**2
     .        - 31.0d0/6.0d0*GPL1(3)*u**2*v*pi*ci
     .        + 1.0d0/3.0d0*GPL1(3)*u**2*v*pi**2
     .        + 11.0d0/4.0d0*GPL1(3)*u**2*v**2*pi*ci
     .        - 11.0d0/2.0d0*GPL1(3)*u**3*pi*ci
     .        + 1.0d0/9.0d0*GPL1(3)*u**3*pi**2
     .        + 11.0d0/2.0d0*GPL1(3)*u**3*v*pi*ci
     .        + 11.0d0/4.0d0*GPL1(3)*u**4*pi*ci
     .        - 11.0d0/12.0d0*GPL2(0,0)
     .        - 1.0d0/3.0d0*GPL2(0,0)*pi*ci
         col5 = col5 + 1.0d0/4.0d0*GPL2(0,1)*u*pi*ci
     .        - 1.0d0/4.0d0*GPL2(0,1)*u**2*pi*ci
     .        + 1.0d0/6.0d0*GPL2(0,1)*u**3*pi*ci
     .        - 1.0d0/36.0d0*GPL2(0,2)
     .        - 1.0d0/4.0d0*GPL2(0,2)*pi*ci
     .        + 13.0d0/12.0d0*GPL2(0,2)*u
     .        - 3.0d0/4.0d0*GPL2(0,2)*u*v
     .        - 1.0d0/3.0d0*GPL2(0,2)*u*v**2
     .        - 23.0d0/6.0d0*GPL2(0,2)*u**2
     .        + 31.0d0/6.0d0*GPL2(0,2)*u**2*v
     .        - 11.0d0/4.0d0*GPL2(0,2)*u**2*v**2
     .        + 11.0d0/2.0d0*GPL2(0,2)*u**3
     .        - 11.0d0/2.0d0*GPL2(0,2)*u**3*v
     .        - 11.0d0/4.0d0*GPL2(0,2)*u**4
     .        + 5.0d0/6.0d0*GPL2(0,3)*pi*ci
         col5 = col5 - 1.0d0/4.0d0*GPL2(0,3)*u*pi*ci
     .        + 1.0d0/4.0d0*GPL2(0,3)*u**2*pi*ci
     .        - 1.0d0/6.0d0*GPL2(0,3)*u**3*pi*ci
     .        + 1.0d0/4.0d0*GPL2(1,0)*u*pi*ci
     .        - 1.0d0/4.0d0*GPL2(1,0)*u**2*pi*ci
     .        + 1.0d0/6.0d0*GPL2(1,0)*u**3*pi*ci
     .        - 1.0d0/2.0d0*GPL2(1,1)*u*pi*ci
     .        + 1.0d0/2.0d0*GPL2(1,1)*u**2*pi*ci
     .        - 1.0d0/3.0d0*GPL2(1,1)*u**3*pi*ci
     .        - 29.0d0/36.0d0*GPL2(1,2)
     .        + 11.0d0/12.0d0*GPL2(1,2)*u**(-2)
     .        - 11.0d0/6.0d0*GPL2(1,2)*u**(-2)*v
     .        + 11.0d0/12.0d0*GPL2(1,2)*u**(-2)*v**2
     .        - 2.0d0/3.0d0*GPL2(1,2)*u**(-1)
     .        + 2.0d0/3.0d0*GPL2(1,2)*u**(-1)*v
         col5 = col5 - 1.0d0/6.0d0*GPL2(1,2)*pi*ci
     .        - 1.0d0/6.0d0*GPL2(1,2)*v
     .        + 1.0d0/6.0d0*GPL2(1,2)*v**2
     .        - 7.0d0/6.0d0*GPL2(1,2)*u
     .        + 1.0d0/4.0d0*GPL2(1,2)*u*pi*ci
     .        + 7.0d0/6.0d0*GPL2(1,2)*u*v
     .        + 1.0d0/3.0d0*GPL2(1,2)*u*v**2
     .        + 47.0d0/12.0d0*GPL2(1,2)*u**2
     .        - 1.0d0/4.0d0*GPL2(1,2)*u**2*pi*ci
     .        - 31.0d0/6.0d0*GPL2(1,2)*u**2*v
     .        + 11.0d0/4.0d0*GPL2(1,2)*u**2*v**2
     .        - 11.0d0/2.0d0*GPL2(1,2)*u**3
     .        + 1.0d0/6.0d0*GPL2(1,2)*u**3*pi*ci
     .        + 11.0d0/2.0d0*GPL2(1,2)*u**3*v
     .        + 11.0d0/4.0d0*GPL2(1,2)*u**4
         col5 = col5 - 1.0d0/36.0d0*GPL2(2,0)
     .        - 1.0d0/4.0d0*GPL2(2,0)*pi*ci
     .        + 13.0d0/12.0d0*GPL2(2,0)*u
     .        - 3.0d0/4.0d0*GPL2(2,0)*u*v
     .        - 1.0d0/3.0d0*GPL2(2,0)*u*v**2
     .        - 23.0d0/6.0d0*GPL2(2,0)*u**2
     .        + 31.0d0/6.0d0*GPL2(2,0)*u**2*v
     .        - 11.0d0/4.0d0*GPL2(2,0)*u**2*v**2
     .        + 11.0d0/2.0d0*GPL2(2,0)*u**3
     .        - 11.0d0/2.0d0*GPL2(2,0)*u**3*v
     .        - 11.0d0/4.0d0*GPL2(2,0)*u**4
     .        + 2.0d0/3.0d0*GPL2(2,1)*pi*ci
     .        + 1.0d0/4.0d0*GPL2(2,1)*v*pi*ci
     .        - 1.0d0/4.0d0*GPL2(2,1)*v**2*pi*ci
     .        + 1.0d0/6.0d0*GPL2(2,1)*v**3*pi*ci
         col5 = col5 + 1.0d0/4.0d0*GPL2(2,1)*u*pi*ci
     .        - 1.0d0/2.0d0*GPL2(2,1)*u*v*pi*ci
     .        + 1.0d0/2.0d0*GPL2(2,1)*u*v**2*pi*ci
     .        - 1.0d0/4.0d0*GPL2(2,1)*u**2*pi*ci
     .        + 1.0d0/2.0d0*GPL2(2,1)*u**2*v*pi*ci
     .        + 1.0d0/6.0d0*GPL2(2,1)*u**3*pi*ci
     .        - 11.0d0/12.0d0*GPL2(2,2)
     .        - 1.0d0/3.0d0*GPL2(2,2)*pi*ci
     .        + 1.0d0/6.0d0*GPL2(2,3)*pi*ci
     .        - 1.0d0/4.0d0*GPL2(2,3)*v*pi*ci
     .        + 1.0d0/4.0d0*GPL2(2,3)*v**2*pi*ci
     .        - 1.0d0/6.0d0*GPL2(2,3)*v**3*pi*ci
     .        - 1.0d0/4.0d0*GPL2(2,3)*u*pi*ci
     .        + 1.0d0/2.0d0*GPL2(2,3)*u*v*pi*ci
     .        - 1.0d0/2.0d0*GPL2(2,3)*u*v**2*pi*ci
         col5 = col5 + 1.0d0/4.0d0*GPL2(2,3)*u**2*pi*ci
     .        - 1.0d0/2.0d0*GPL2(2,3)*u**2*v*pi*ci
     .        - 1.0d0/6.0d0*GPL2(2,3)*u**3*pi*ci
     .        - 29.0d0/36.0d0*GPL2(3,0)
     .        + 11.0d0/12.0d0*GPL2(3,0)*omumv**(-2)
     .        - 2.0d0/3.0d0*GPL2(3,0)*omumv**(-1)
     .        - 11.0d0/6.0d0*GPL2(3,0)*v*omumv**(-2)
     .        + 2.0d0/3.0d0*GPL2(3,0)*v*omumv**(-1)
     .        + 1.0d0/6.0d0*GPL2(3,0)*v
     .        - 1.0d0/4.0d0*GPL2(3,0)*v*pi*ci
     .        + 11.0d0/12.0d0*GPL2(3,0)*v**2*omumv**(-2)
     .        - 1.0d0/6.0d0*GPL2(3,0)*v**2
     .        + 1.0d0/4.0d0*GPL2(3,0)*v**2*pi*ci
     .        - 1.0d0/6.0d0*GPL2(3,0)*v**3*pi*ci
     .        - 7.0d0/6.0d0*GPL2(3,0)*u
         col5 = col5 - 1.0d0/4.0d0*GPL2(3,0)*u*pi*ci
     .        + 1.0d0/2.0d0*GPL2(3,0)*u*v
     .        + 1.0d0/2.0d0*GPL2(3,0)*u*v*pi*ci
     .        + 1.0d0/3.0d0*GPL2(3,0)*u*v**2
     .        - 1.0d0/2.0d0*GPL2(3,0)*u*v**2*pi*ci
     .        + 47.0d0/12.0d0*GPL2(3,0)*u**2
     .        + 1.0d0/4.0d0*GPL2(3,0)*u**2*pi*ci
     .        - 31.0d0/6.0d0*GPL2(3,0)*u**2*v
     .        - 1.0d0/2.0d0*GPL2(3,0)*u**2*v*pi*ci
     .        + 11.0d0/4.0d0*GPL2(3,0)*u**2*v**2
     .        - 11.0d0/2.0d0*GPL2(3,0)*u**3
     .        - 1.0d0/6.0d0*GPL2(3,0)*u**3*pi*ci
     .        + 11.0d0/2.0d0*GPL2(3,0)*u**3*v
     .        + 11.0d0/4.0d0*GPL2(3,0)*u**4
     .        + 1.0d0/6.0d0*GPL2(3,2)*pi*ci
         col5 = col5 - 1.0d0/4.0d0*GPL2(3,2)*v*pi*ci
     .        + 1.0d0/4.0d0*GPL2(3,2)*v**2*pi*ci
     .        - 1.0d0/6.0d0*GPL2(3,2)*v**3*pi*ci
     .        - 1.0d0/4.0d0*GPL2(3,2)*u*pi*ci
     .        + 1.0d0/2.0d0*GPL2(3,2)*u*v*pi*ci
     .        - 1.0d0/2.0d0*GPL2(3,2)*u*v**2*pi*ci
     .        + 1.0d0/4.0d0*GPL2(3,2)*u**2*pi*ci
     .        - 1.0d0/2.0d0*GPL2(3,2)*u**2*v*pi*ci
     .        - 1.0d0/6.0d0*GPL2(3,2)*u**3*pi*ci
     .        - 1.0d0/3.0d0*GPL2(3,3)*pi*ci
     .        + 1.0d0/2.0d0*GPL2(3,3)*v*pi*ci
     .        - 1.0d0/2.0d0*GPL2(3,3)*v**2*pi*ci
     .        + 1.0d0/3.0d0*GPL2(3,3)*v**3*pi*ci
     .        + 1.0d0/2.0d0*GPL2(3,3)*u*pi*ci
     .        - GPL2(3,3)*u*v*pi*ci
         col5 = col5 + GPL2(3,3)*u*v**2*pi*ci
     .        - 1.0d0/2.0d0*GPL2(3,3)*u**2*pi*ci
     .        + GPL2(3,3)*u**2*v*pi*ci
     .        + 1.0d0/3.0d0*GPL2(3,3)*u**3*pi*ci
     .        - 1.0d0/3.0d0*GPL3(0,0,2)
     .        + 1.0d0/4.0d0*GPL3(0,1,2)*u
     .        - 1.0d0/4.0d0*GPL3(0,1,2)*u**2
     .        + 1.0d0/6.0d0*GPL3(0,1,2)*u**3
     .        - 1.0d0/3.0d0*GPL3(0,2,0)
     .        - 1.0d0/3.0d0*GPL3(0,2,2)
     .        + 5.0d0/6.0d0*GPL3(0,3,0)
     .        - 1.0d0/4.0d0*GPL3(0,3,0)*u
     .        + 1.0d0/4.0d0*GPL3(0,3,0)*u**2
     .        - 1.0d0/6.0d0*GPL3(0,3,0)*u**3
     .        + 1.0d0/4.0d0*GPL3(1,0,2)*u
         col5 = col5 - 1.0d0/4.0d0*GPL3(1,0,2)*u**2
     .        + 1.0d0/6.0d0*GPL3(1,0,2)*u**3
     .        - 1.0d0/2.0d0*GPL3(1,1,2)*u
     .        + 1.0d0/2.0d0*GPL3(1,1,2)*u**2
     .        - 1.0d0/3.0d0*GPL3(1,1,2)*u**3
     .        + 1.0d0/4.0d0*GPL3(1,2,0)*u
     .        - 1.0d0/4.0d0*GPL3(1,2,0)*u**2
     .        + 1.0d0/6.0d0*GPL3(1,2,0)*u**3
     .        + 2.0d0/3.0d0*GPL3(1,2,2)
     .        - 1.0d0/3.0d0*GPL3(2,0,0)
     .        - 1.0d0/3.0d0*GPL3(2,0,2)
     .        + 2.0d0/3.0d0*GPL3(2,1,2)
     .        + 1.0d0/4.0d0*GPL3(2,1,2)*v
     .        - 1.0d0/4.0d0*GPL3(2,1,2)*v**2
     .        + 1.0d0/6.0d0*GPL3(2,1,2)*v**3
         col5 = col5 + 1.0d0/4.0d0*GPL3(2,1,2)*u
     .        - 1.0d0/2.0d0*GPL3(2,1,2)*u*v
     .        + 1.0d0/2.0d0*GPL3(2,1,2)*u*v**2
     .        - 1.0d0/4.0d0*GPL3(2,1,2)*u**2
     .        + 1.0d0/2.0d0*GPL3(2,1,2)*u**2*v
     .        + 1.0d0/6.0d0*GPL3(2,1,2)*u**3
     .        - 1.0d0/3.0d0*GPL3(2,2,0)
     .        + 1.0d0/6.0d0*GPL3(2,3,0)
     .        - 1.0d0/4.0d0*GPL3(2,3,0)*v
     .        + 1.0d0/4.0d0*GPL3(2,3,0)*v**2
     .        - 1.0d0/6.0d0*GPL3(2,3,0)*v**3
     .        - 1.0d0/4.0d0*GPL3(2,3,0)*u
     .        + 1.0d0/2.0d0*GPL3(2,3,0)*u*v
     .        - 1.0d0/2.0d0*GPL3(2,3,0)*u*v**2
     .        + 1.0d0/4.0d0*GPL3(2,3,0)*u**2
         col5 = col5 - 1.0d0/2.0d0*GPL3(2,3,0)*u**2*v
     .        - 1.0d0/6.0d0*GPL3(2,3,0)*u**3
     .        + 2.0d0/3.0d0*GPL3(3,0,0)
     .        + 1.0d0/6.0d0*GPL3(3,0,2)
     .        - 1.0d0/4.0d0*GPL3(3,0,2)*v
     .        + 1.0d0/4.0d0*GPL3(3,0,2)*v**2
     .        - 1.0d0/6.0d0*GPL3(3,0,2)*v**3
     .        - 1.0d0/4.0d0*GPL3(3,0,2)*u
     .        + 1.0d0/2.0d0*GPL3(3,0,2)*u*v
     .        - 1.0d0/2.0d0*GPL3(3,0,2)*u*v**2
     .        + 1.0d0/4.0d0*GPL3(3,0,2)*u**2
     .        - 1.0d0/2.0d0*GPL3(3,0,2)*u**2*v
     .        - 1.0d0/6.0d0*GPL3(3,0,2)*u**3
     .        + 1.0d0/6.0d0*GPL3(3,2,0)
     .        - 1.0d0/4.0d0*GPL3(3,2,0)*v
         col5 = col5 + 1.0d0/4.0d0*GPL3(3,2,0)*v**2
     .        - 1.0d0/6.0d0*GPL3(3,2,0)*v**3
     .        - 1.0d0/4.0d0*GPL3(3,2,0)*u
     .        + 1.0d0/2.0d0*GPL3(3,2,0)*u*v
     .        - 1.0d0/2.0d0*GPL3(3,2,0)*u*v**2
     .        + 1.0d0/4.0d0*GPL3(3,2,0)*u**2
     .        - 1.0d0/2.0d0*GPL3(3,2,0)*u**2*v
     .        - 1.0d0/6.0d0*GPL3(3,2,0)*u**3
     .        - 1.0d0/3.0d0*GPL3(3,3,0)
     .        + 1.0d0/2.0d0*GPL3(3,3,0)*v
     .        - 1.0d0/2.0d0*GPL3(3,3,0)*v**2
     .        + 1.0d0/3.0d0*GPL3(3,3,0)*v**3
     .        + 1.0d0/2.0d0*GPL3(3,3,0)*u
     .        - GPL3(3,3,0)*u*v
     .        + GPL3(3,3,0)*u*v**2
         col5 = col5 - 1.0d0/2.0d0*GPL3(3,3,0)*u**2
     .        + GPL3(3,3,0)*u**2*v
     .        + 1.0d0/3.0d0*GPL3(3,3,0)*u**3

         col6 = + 5.0d0/54.0d0*pi*ci
     .        - 5.0d0/144.0d0*pi**2
     .        + 5.0d0/54.0d0*u
     .        + 1.0d0/12.0d0*u*pi*ci
     .        - 5.0d0/54.0d0*u*v
     .        - 1.0d0/12.0d0*u*v*pi*ci
     .        - 5.0d0/54.0d0*u**2
     .        - 1.0d0/12.0d0*u**2*pi*ci
     .        + 5.0d0/18.0d0*HPL1(0)
     .        + 5.0d0/36.0d0*HPL1(0)*pi*ci
     .        + 5.0d0/36.0d0*HPL1(0)*u
     .        - 5.0d0/36.0d0*HPL1(0)*u*v
     .        - 5.0d0/36.0d0*HPL1(0)*u**2
     .        - 5.0d0/36.0d0*HPL1(0)*GPL1(0)
     .        - 5.0d0/36.0d0*HPL1(0)*GPL1(2)
         col6 = col6 + 5.0d0/54.0d0*HPL1(1)
     .        + 1.0d0/36.0d0*HPL1(1)*pi*ci
     .        + 1.0d0/36.0d0*HPL1(1)*u
     .        - 1.0d0/36.0d0*HPL1(1)*u*v
     .        - 1.0d0/36.0d0*HPL1(1)*u**2
     .        - 1.0d0/36.0d0*HPL1(1)*GPL1(0)
     .        - 1.0d0/12.0d0*HPL1(1)*GPL1(2)
     .        + 5.0d0/12.0d0*HPL2(0,0)
     .        + 5.0d0/36.0d0*HPL2(0,1)
     .        + 5.0d0/36.0d0*HPL2(1,0)
     .        + 1.0d0/12.0d0*HPL2(1,1)
     .        - 5.0d0/54.0d0*GPL1(0)
     .        - 1.0d0/36.0d0*GPL1(0)*pi*ci
     .        - 1.0d0/36.0d0*GPL1(0)*u
     .        + 1.0d0/36.0d0*GPL1(0)*u*v
         col6 = col6 + 1.0d0/36.0d0*GPL1(0)*u**2
     .        - 5.0d0/54.0d0*GPL1(2)
     .        - 1.0d0/36.0d0*GPL1(2)*pi*ci
     .        - 1.0d0/36.0d0*GPL1(2)*u
     .        + 1.0d0/36.0d0*GPL1(2)*u*v
     .        + 1.0d0/36.0d0*GPL1(2)*u**2
     .        + 1.0d0/12.0d0*GPL2(0,0)
     .        + 1.0d0/36.0d0*GPL2(0,2)
     .        + 1.0d0/36.0d0*GPL2(2,0)
     .        + 1.0d0/12.0d0*GPL2(2,2)
      endif

      col(1) = col1
      col(2) = col2
      col(3) = col3
      col(4) = col4
      col(5) = col5
      col(6) = col6

      end subroutine beta2a2L

!#############################################################

      subroutine beta4a2L(u,v,col)
      implicit none
      !returns beta(4a+)
      ! at two Loops
      !-------------
      real*8, intent(in) :: u,v
      complex*16, dimension(6), intent(out) :: col
      !internal
      complex*16 :: beta
      complex*16 :: ci
      real*8 :: omu,omv,omumv,upv
      real*8, dimension(0:1) :: HPL1
      real*8, dimension(0:1,0:1) :: HPL2
      real*8, dimension(0:1,0:1,0:1) :: HPL3
      real*8, dimension(0:1,0:1,0:1,0:1) :: HPL4
      real*8, dimension(0:3) :: GPL1
      real*8, dimension(0:3,0:3) :: GPL2
      real*8, dimension(0:3,0:3,0:3) :: GPL3
      real*8, dimension(0:3,0:3,0:3,0:3) :: GPL4
      complex*16 :: col1,col2,col3,col4,col5,col6


      omu   = 1.0d0 - u
      omv   = 1.0d0     - v
      omumv = 1.0d0 - u - v
      upv   =         u + v

      ci = (0.d0,1.d0)

      call tdhpl(dble(u),dble(v),4,GPL1,GPL2,GPL3,GPL4,HPL1,HPL2,HPL3,HPL4)

      if(ieorder==4) then
         col1 = 9.0d0/8.0d0

         col2 = 0.0d0

         col3 = 0.0d0

         col4 = 0.0d0

         col5 = 0.0d0

         col6 = 0.0d0
      elseif(ieorder==3) then
         col1 = + 99.0d0/16.0d0
     .          + 3.0d0/4.0d0*pi*ci
     .          + 9.0d0/4.0d0*HPL1(0)
     .          + 3.0d0/4.0d0*HPL1(1)
     .          - 3.0d0/4.0d0*GPL1(0)
     .          - 3.0d0/4.0d0*GPL1(2)

         col2 = 0

         col3 = 0

         col4 = 0

         col5 = - 9.0d0/8.0d0

         col6 = 0
      else
         col1 = + 1231.0d0/216.0d0
     .        - 5.0d0/6.0d0*u**(-1)*pi*ci
     .        - 4.0d0/3.0d0*omv**(-1)
     .        - 5.0d0/12.0d0*omumv**(-4)*pi**2
     .        - 5.0d0/2.0d0*omumv**(-3)*pi*ci
     .        + 5.0d0/6.0d0*omumv**(-3)*pi**2
     .        + 1.0d0/36.0d0*omumv**(-3)*pi**3*ci
     .        - 337.0d0/108.0d0*omumv**(-2)
     .        + 259.0d0/72.0d0*omumv**(-2)*pi*ci
     .        - 31.0d0/36.0d0*omumv**(-2)*pi**2
     .        - 1.0d0/24.0d0*omumv**(-2)*pi**3*ci
     .        + 337.0d0/108.0d0*omumv**(-1)
     .        - 181.0d0/72.0d0*omumv**(-1)*pi*ci
     .        + 4.0d0/9.0d0*omumv**(-1)*pi**2
     .        + 1.0d0/6.0d0*omumv**(-1)*pi**3*ci
         col1 = col1 + 37.0d0/8.0d0*pi*ci
     .        + 935.0d0/864.0d0*pi**2
     .        + 55.0d0/96.0d0*pi**3*ci
     .        + 49.0d0/2880.0d0*pi**4
     .        - 1.0d0/3.0d0*zeta3*omumv**(-3)
     .        + 1.0d0/2.0d0*zeta3*omumv**(-2)
     .        - 2.0d0*zeta3*omumv**(-1)
     .        + 55.0d0/48.0d0*zeta3
     .        + 1.0d0/4.0d0*zeta3*pi*ci
     .        + 5.0d0/6.0d0*v*omumv**(-4)*pi**2
     .        + 5.0d0*v*omumv**(-3)*pi*ci
     .        - 13.0d0/18.0d0*v*omumv**(-3)*pi**2
     .        - 1.0d0/12.0d0*v*omumv**(-3)*pi**3*ci
     .        + 337.0d0/108.0d0*v*omumv**(-2)
     .        - 133.0d0/72.0d0*v*omumv**(-2)*pi*ci
         col1 = col1 + 2.0d0/9.0d0*v*omumv**(-2)*pi**2
     .        + 1.0d0/12.0d0*v*omumv**(-2)*pi**3*ci
     .        + 11.0d0/12.0d0*v*omumv**(-1)
     .        + 1.0d0/2.0d0*v*omumv**(-1)*pi*ci
     .        + 1.0d0/36.0d0*v*omumv**(-1)*pi**2
     .        - 1.0d0/6.0d0*v*omumv**(-1)*pi**3*ci
     .        + 4.0d0/3.0d0*v*upv**(-1)
     .        + 5.0d0/3.0d0*v*upv**(-1)*pi*ci
     .        - 5.0d0/6.0d0*v*pi*ci
     .        - 5.0d0/12.0d0*v**2*omumv**(-4)*pi**2
     .        - 5.0d0/2.0d0*v**2*omumv**(-3)*pi*ci
     .        - 1.0d0/9.0d0*v**2*omumv**(-3)*pi**2
     .        + 1.0d0/12.0d0*v**2*omumv**(-3)*pi**3*ci
     .        - 7.0d0/4.0d0*v**2*omumv**(-2)*pi*ci
     .        + 1.0d0/36.0d0*v**2*omumv**(-2)*pi**2
         col1 = col1 - 1.0d0/24.0d0*v**2*omumv**(-2)*pi**3*ci
     .        - 5.0d0/6.0d0*v**2*omumv**(-1)*pi*ci
     .        + 4.0d0/3.0d0*v**2*upv**(-2)*pi*ci
     .        - 1.0d0/36.0d0*v**3*omumv**(-3)*pi**3*ci
     .        + 25.0d0/18.0d0*u*pi**2
     .        - 5.0d0/9.0d0*u**2*pi**2
     .        + 113.0d0/24.0d0*HPL1(0)
     .        - 5.0d0/6.0d0*HPL1(0)*u**(-1)*omv**(-1)
     .        + 5.0d0/6.0d0*HPL1(0)*u**(-1)
     .        - 4.0d0/3.0d0*HPL1(0)*omv**(-2)
     .        - 5.0d0/6.0d0*HPL1(0)*omv**(-1)*omumv**(-1)
     .        + 13.0d0/3.0d0*HPL1(0)*omv**(-1)
     .        - 5.0d0/2.0d0*HPL1(0)*omumv**(-4)*pi*ci
     .        + 5.0d0*HPL1(0)*omumv**(-3)*pi*ci
     .        - 1.0d0/18.0d0*HPL1(0)*omumv**(-3)*pi**2
         col1 = col1 - 55.0d0/72.0d0*HPL1(0)*omumv**(-2)
     .        - 55.0d0/12.0d0*HPL1(0)*omumv**(-2)*pi*ci
     .        + 1.0d0/12.0d0*HPL1(0)*omumv**(-2)*pi**2
     .        + 115.0d0/72.0d0*HPL1(0)*omumv**(-1)
     .        + 25.0d0/12.0d0*HPL1(0)*omumv**(-1)*pi*ci
     .        - 1.0d0/3.0d0*HPL1(0)*omumv**(-1)*pi**2
     .        + 1141.0d0/144.0d0*HPL1(0)*pi*ci
     .        + 55.0d0/288.0d0*HPL1(0)*pi**2
     .        + 1.0d0/4.0d0*HPL1(0)*pi**3*ci
     .        + 3.0d0/4.0d0*HPL1(0)*zeta3
     .        + 5.0d0*HPL1(0)*v*omumv**(-4)*pi*ci
     .        - 5.0d0/2.0d0*HPL1(0)*v*omumv**(-3)
     .        - 13.0d0/3.0d0*HPL1(0)*v*omumv**(-3)*pi*ci
     .        - 1.0d0/12.0d0*HPL1(0)*v*omumv**(-3)*pi**2
     .        + 235.0d0/72.0d0*HPL1(0)*v*omumv**(-2)
         col1 = col1 + 17.0d0/12.0d0*HPL1(0)*v*omumv**(-2)*pi*ci
     .        + 1.0d0/12.0d0*HPL1(0)*v*omumv**(-2)*pi**2
     .        - 1.0d0/2.0d0*HPL1(0)*v*omumv**(-1)
     .        - 1.0d0/6.0d0*HPL1(0)*v*omumv**(-1)*pi**2
     .        - 5.0d0/3.0d0*HPL1(0)*v*upv**(-1)
     .        + 5.0d0/6.0d0*HPL1(0)*v
     .        - 5.0d0/2.0d0*HPL1(0)*v**2*omumv**(-4)*pi*ci
     .        + 5.0d0/2.0d0*HPL1(0)*v**2*omumv**(-3)
     .        - 2.0d0/3.0d0*HPL1(0)*v**2*omumv**(-3)*pi*ci
     .        + 1.0d0/12.0d0*HPL1(0)*v**2*omumv**(-3)*pi**2
     .        + 7.0d0/4.0d0*HPL1(0)*v**2*omumv**(-2)
     .        - 1.0d0/24.0d0*HPL1(0)*v**2*omumv**(-2)*pi**2
     .        + 5.0d0/6.0d0*HPL1(0)*v**2*omumv**(-1)
     .        - 4.0d0/3.0d0*HPL1(0)*v**2*upv**(-2)
     .        - 1.0d0/36.0d0*HPL1(0)*v**3*omumv**(-3)*pi**2
         col1 = col1 + 25.0d0/12.0d0*HPL1(0)*u*pi*ci
     .        - 5.0d0/6.0d0*HPL1(0)*u**2*pi*ci
     .        - 23.0d0/48.0d0*HPL1(0)*GPL1(0)
     .        - 5.0d0/2.0d0*HPL1(0)*GPL1(0)*omumv**(-4)
     .        + 5.0d0*HPL1(0)*GPL1(0)*omumv**(-3)
     .        - 55.0d0/12.0d0*HPL1(0)*GPL1(0)*omumv**(-2)
     .        + 25.0d0/12.0d0*HPL1(0)*GPL1(0)*omumv**(-1)
     .        - 55.0d0/24.0d0*HPL1(0)*GPL1(0)*pi*ci
     .        + 3.0d0/4.0d0*HPL1(0)*GPL1(0)*pi**2
     .        + 5.0d0*HPL1(0)*GPL1(0)*v*omumv**(-4)
     .        - 13.0d0/3.0d0*HPL1(0)*GPL1(0)*v*omumv**(-3)
     .        + 17.0d0/12.0d0*HPL1(0)*GPL1(0)*v*omumv**(-2)
     .        - 5.0d0/2.0d0*HPL1(0)*GPL1(0)*v**2*omumv**(-4)
     .        - 2.0d0/3.0d0*HPL1(0)*GPL1(0)*v**2*omumv**(-3)
     .        + 25.0d0/12.0d0*HPL1(0)*GPL1(0)*u
         col1 = col1 - 5.0d0/6.0d0*HPL1(0)*GPL1(0)*u**2
     .        - 67.0d0/18.0d0*HPL1(0)*GPL1(1)
     .        + 5.0d0/6.0d0*HPL1(0)*GPL1(1)*u**(-2)
     .        - 25.0d0/12.0d0*HPL1(0)*GPL1(1)*u**(-1)
     .        + 1.0d0/6.0d0*HPL1(0)*GPL1(1)*omumv**(-2)
     .        - 1.0d0/6.0d0*HPL1(0)*GPL1(1)*omumv**(-1)
     .        + 11.0d0/12.0d0*HPL1(0)*GPL1(1)*pi*ci
     .        - 7.0d0/12.0d0*HPL1(0)*GPL1(1)*pi**2
     .        - 1.0d0/6.0d0*HPL1(0)*GPL1(1)*v*omumv**(-2)
     .        - 25.0d0/12.0d0*HPL1(0)*GPL1(1)*u
     .        + 5.0d0/6.0d0*HPL1(0)*GPL1(1)*u**2
     .        - 23.0d0/48.0d0*HPL1(0)*GPL1(2)
     .        - 5.0d0/6.0d0*HPL1(0)*GPL1(2)*u**(-2)
     .        + 25.0d0/12.0d0*HPL1(0)*GPL1(2)*u**(-1)
     .        - 1.0d0/6.0d0*HPL1(0)*GPL1(2)*omumv**(-2)
         col1 = col1 + 1.0d0/6.0d0*HPL1(0)*GPL1(2)*omumv**(-1)
     .        + 11.0d0/24.0d0*HPL1(0)*GPL1(2)*pi*ci
     .        + 1.0d0/12.0d0*HPL1(0)*GPL1(2)*pi**2
     .        + 1.0d0/6.0d0*HPL1(0)*GPL1(2)*v*omumv**(-2)
     .        + 25.0d0/12.0d0*HPL1(0)*GPL1(2)*u
     .        - 5.0d0/6.0d0*HPL1(0)*GPL1(2)*u**2
     .        - 67.0d0/18.0d0*HPL1(0)*GPL1(3)
     .        + 5.0d0/2.0d0*HPL1(0)*GPL1(3)*omumv**(-4)
     .        - 5.0d0*HPL1(0)*GPL1(3)*omumv**(-3)
     .        + 1.0d0/6.0d0*HPL1(0)*GPL1(3)*omumv**(-3)*pi*ci
     .        + 55.0d0/12.0d0*HPL1(0)*GPL1(3)*omumv**(-2)
     .        - 1.0d0/4.0d0*HPL1(0)*GPL1(3)*omumv**(-2)*pi*ci
     .        - 25.0d0/12.0d0*HPL1(0)*GPL1(3)*omumv**(-1)
     .        + HPL1(0)*GPL1(3)*omumv**(-1)*pi*ci
     .        + 11.0d0/12.0d0*HPL1(0)*GPL1(3)*pi*ci
         col1 = col1 - 5.0d0/4.0d0*HPL1(0)*GPL1(3)*pi**2
     .        - 5.0d0*HPL1(0)*GPL1(3)*v*omumv**(-4)
     .        + 13.0d0/3.0d0*HPL1(0)*GPL1(3)*v*omumv**(-3)
     .        - 7.0d0/4.0d0*HPL1(0)*GPL1(3)*v*omumv**(-2)
     .        + 1.0d0/6.0d0*HPL1(0)*GPL1(3)*v*omumv**(-1)
     .        + 5.0d0/2.0d0*HPL1(0)*GPL1(3)*v**2*omumv**(-4)
     .        + 2.0d0/3.0d0*HPL1(0)*GPL1(3)*v**2*omumv**(-3)
     .        + 1.0d0/6.0d0*HPL1(0)*GPL1(3)*v**2*omumv**(-2)
     .        - 25.0d0/12.0d0*HPL1(0)*GPL1(3)*u
     .        + 5.0d0/6.0d0*HPL1(0)*GPL1(3)*u**2
     .        - 11.0d0/3.0d0*HPL1(0)*GPL2(0,0)
     .        - HPL1(0)*GPL2(0,0)*pi*ci
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(0,1)*pi*ci
     .        - 77.0d0/24.0d0*HPL1(0)*GPL2(0,2)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(0,2)*pi*ci
         col1 = col1 + 11.0d0/3.0d0*HPL1(0)*GPL2(0,3)
     .        + 1.0d0/6.0d0*HPL1(0)*GPL2(0,3)*omumv**(-3)
     .        - 1.0d0/4.0d0*HPL1(0)*GPL2(0,3)*omumv**(-2)
     .        + HPL1(0)*GPL2(0,3)*omumv**(-1)
     .        + 3.0d0/2.0d0*HPL1(0)*GPL2(0,3)*pi*ci
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(0,3)*v*omumv**(-3)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(0,3)*v*omumv**(-2)
     .        - HPL1(0)*GPL2(0,3)*v*omumv**(-1)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(0,3)*v**2*omumv**(-3)
     .        - 1.0d0/4.0d0*HPL1(0)*GPL2(0,3)*v**2*omumv**(-2)
     .        - 1.0d0/6.0d0*HPL1(0)*GPL2(0,3)*v**3*omumv**(-3)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(1,0)*pi*ci
     .        + 11.0d0/6.0d0*HPL1(0)*GPL2(1,1)
     .        + 11.0d0/3.0d0*HPL1(0)*GPL2(1,2)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(1,2)*pi*ci
         col1 = col1 - 77.0d0/24.0d0*HPL1(0)*GPL2(2,0)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(2,0)*pi*ci
     .        + 11.0d0/4.0d0*HPL1(0)*GPL2(2,1)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(2,1)*pi*ci
     .        - 11.0d0/3.0d0*HPL1(0)*GPL2(2,2)
     .        + HPL1(0)*GPL2(2,2)*pi*ci
     .        + 11.0d0/12.0d0*HPL1(0)*GPL2(2,3)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(2,3)*pi*ci
     .        + 55.0d0/12.0d0*HPL1(0)*GPL2(3,0)
     .        + 1.0d0/6.0d0*HPL1(0)*GPL2(3,0)*omumv**(-3)
     .        - 1.0d0/4.0d0*HPL1(0)*GPL2(3,0)*omumv**(-2)
     .        + HPL1(0)*GPL2(3,0)*omumv**(-1)
     .        + 5.0d0/2.0d0*HPL1(0)*GPL2(3,0)*pi*ci
     .        + 11.0d0/12.0d0*HPL1(0)*GPL2(3,2)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(3,2)*pi*ci
         col1 = col1 - 1.0d0/3.0d0*HPL1(0)*GPL2(3,3)*omumv**(-3)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(3,3)*omumv**(-2)
     .        - 2.0d0*HPL1(0)*GPL2(3,3)*omumv**(-1)
     .        - 2.0d0*HPL1(0)*GPL2(3,3)*pi*ci
     .        - HPL1(0)*GPL3(0,0,1)
     .        + HPL1(0)*GPL3(0,0,3)
     .        - HPL1(0)*GPL3(0,1,0)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(0,1,2)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(0,2,1)
     .        - HPL1(0)*GPL3(0,2,2)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(0,2,3)
     .        + HPL1(0)*GPL3(0,3,0)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(0,3,2)
     .        - 2.0d0*HPL1(0)*GPL3(0,3,3)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(1,0,2)
         col1 = col1 + 2.0d0*HPL1(0)*GPL3(1,1,1)
     .        - 2.0d0*HPL1(0)*GPL3(1,1,2)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(1,2,0)
     .        - 2.0d0*HPL1(0)*GPL3(1,2,1)
     .        + 2.0d0*HPL1(0)*GPL3(1,2,2)
     .        - HPL1(0)*GPL3(2,0,0)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(2,0,1)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(2,0,3)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(2,1,0)
     .        - 2.0d0*HPL1(0)*GPL3(2,1,1)
     .        + HPL1(0)*GPL3(2,1,2)
     .        + HPL1(0)*GPL3(2,2,1)
     .        - HPL1(0)*GPL3(2,2,3)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(2,3,0)
     .        - HPL1(0)*GPL3(2,3,2)
         col1 = col1 + 2.0d0*HPL1(0)*GPL3(3,0,0)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(3,0,2)
     .        - 2.0d0*HPL1(0)*GPL3(3,0,3)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(3,2,0)
     .        - 2.0d0*HPL1(0)*GPL3(3,3,0)
     .        + 2.0d0*HPL1(0)*GPL3(3,3,3)
     .        - 1.0d0/8.0d0*HPL1(1)
     .        + 5.0d0/6.0d0*HPL1(1)*u**(-1)
     .        + 1.0d0/18.0d0*HPL1(1)*omumv**(-3)*pi**2
     .        - 11.0d0/24.0d0*HPL1(1)*omumv**(-2)
     .        - 1.0d0/12.0d0*HPL1(1)*omumv**(-2)*pi*ci
     .        - 1.0d0/12.0d0*HPL1(1)*omumv**(-2)*pi**2
     .        + 11.0d0/24.0d0*HPL1(1)*omumv**(-1)
     .        + 1.0d0/12.0d0*HPL1(1)*omumv**(-1)*pi*ci
     .        + 1.0d0/3.0d0*HPL1(1)*omumv**(-1)*pi**2
         col1 = col1 + 389.0d0/144.0d0*HPL1(1)*pi*ci
     .        - 143.0d0/288.0d0*HPL1(1)*pi**2
     .        + 1.0d0/8.0d0*HPL1(1)*pi**3*ci
     .        + 1.0d0/4.0d0*HPL1(1)*zeta3
     .        - 1.0d0/6.0d0*HPL1(1)*v*omumv**(-3)*pi**2
     .        + 11.0d0/24.0d0*HPL1(1)*v*omumv**(-2)
     .        + 1.0d0/12.0d0*HPL1(1)*v*omumv**(-2)*pi*ci
     .        + 1.0d0/6.0d0*HPL1(1)*v*omumv**(-2)*pi**2
     .        - 1.0d0/3.0d0*HPL1(1)*v*omumv**(-1)*pi**2
     .        + 1.0d0/6.0d0*HPL1(1)*v**2*omumv**(-3)*pi**2
     .        - 1.0d0/12.0d0*HPL1(1)*v**2*omumv**(-2)*pi**2
     .        - 1.0d0/18.0d0*HPL1(1)*v**3*omumv**(-3)*pi**2
     .        + 5.0d0/6.0d0*HPL1(1)*u
     .        + 25.0d0/12.0d0*HPL1(1)*u*pi*ci
     .        - 5.0d0/6.0d0*HPL1(1)*u**2*pi*ci
         col1 = col1 + 49.0d0/48.0d0*HPL1(1)*GPL1(0)
     .        - 1.0d0/12.0d0*HPL1(1)*GPL1(0)*omumv**(-2)
     .        + 1.0d0/12.0d0*HPL1(1)*GPL1(0)*omumv**(-1)
     .        - 11.0d0/8.0d0*HPL1(1)*GPL1(0)*pi*ci
     .        + 3.0d0/8.0d0*HPL1(1)*GPL1(0)*pi**2
     .        + 1.0d0/12.0d0*HPL1(1)*GPL1(0)*v*omumv**(-2)
     .        + 25.0d0/12.0d0*HPL1(1)*GPL1(0)*u
     .        - 5.0d0/6.0d0*HPL1(1)*GPL1(0)*u**2
     .        - 67.0d0/18.0d0*HPL1(1)*GPL1(1)
     .        + 5.0d0/6.0d0*HPL1(1)*GPL1(1)*u**(-2)
     .        - 25.0d0/12.0d0*HPL1(1)*GPL1(1)*u**(-1)
     .        + 1.0d0/6.0d0*HPL1(1)*GPL1(1)*omumv**(-2)
     .        - 1.0d0/6.0d0*HPL1(1)*GPL1(1)*omumv**(-1)
     .        - 11.0d0/12.0d0*HPL1(1)*GPL1(1)*pi*ci
     .        - 5.0d0/12.0d0*HPL1(1)*GPL1(1)*pi**2
         col1 = col1 - 1.0d0/6.0d0*HPL1(1)*GPL1(1)*v*omumv**(-2)
     .        - 25.0d0/12.0d0*HPL1(1)*GPL1(1)*u
     .        + 5.0d0/6.0d0*HPL1(1)*GPL1(1)*u**2
     .        - 121.0d0/48.0d0*HPL1(1)*GPL1(2)
     .        - 11.0d0/6.0d0*HPL1(1)*GPL1(2)*pi*ci
     .        - 1.0d0/12.0d0*HPL1(1)*GPL1(2)*pi**2
     .        + 11.0d0/12.0d0*HPL1(1)*GPL1(3)*pi*ci
     .        - 1.0d0/2.0d0*HPL1(1)*GPL1(3)*pi**2
     .        - 11.0d0/6.0d0*HPL1(1)*GPL2(0,0)
     .        - 1.0d0/2.0d0*HPL1(1)*GPL2(0,0)*pi*ci
     .        - 11.0d0/6.0d0*HPL1(1)*GPL2(0,2)
     .        - 1.0d0/2.0d0*HPL1(1)*GPL2(0,2)*pi*ci
     .        + 1.0d0/2.0d0*HPL1(1)*GPL2(0,3)*pi*ci
     .        + HPL1(1)*GPL2(1,0)*pi*ci
     .        + 11.0d0/6.0d0*HPL1(1)*GPL2(1,1)
         col1 = col1 - 2.0d0*HPL1(1)*GPL2(1,1)*pi*ci
     .        + 11.0d0/3.0d0*HPL1(1)*GPL2(1,2)
     .        + HPL1(1)*GPL2(1,2)*pi*ci
     .        - 11.0d0/6.0d0*HPL1(1)*GPL2(2,0)
     .        - 1.0d0/2.0d0*HPL1(1)*GPL2(2,0)*pi*ci
     .        + 11.0d0/4.0d0*HPL1(1)*GPL2(2,1)
     .        + 1.0d0/2.0d0*HPL1(1)*GPL2(2,1)*pi*ci
     .        - HPL1(1)*GPL2(2,3)*pi*ci
     .        + 11.0d0/12.0d0*HPL1(1)*GPL2(3,0)
     .        + HPL1(1)*GPL2(3,0)*pi*ci
     .        - HPL1(1)*GPL3(0,0,1)
     .        - 1.0d0/2.0d0*HPL1(1)*GPL3(0,0,2)
     .        - HPL1(1)*GPL3(0,1,0)
     .        + HPL1(1)*GPL3(0,1,2)
     .        - 1.0d0/2.0d0*HPL1(1)*GPL3(0,2,0)
         col1 = col1 + 1.0d0/2.0d0*HPL1(1)*GPL3(0,2,1)
     .        + 1.0d0/2.0d0*HPL1(1)*GPL3(0,3,0)
     .        + HPL1(1)*GPL3(1,0,2)
     .        + 2.0d0*HPL1(1)*GPL3(1,1,1)
     .        - 2.0d0*HPL1(1)*GPL3(1,1,2)
     .        + HPL1(1)*GPL3(1,2,0)
     .        - 2.0d0*HPL1(1)*GPL3(1,2,1)
     .        - 1.0d0/2.0d0*HPL1(1)*GPL3(2,0,0)
     .        + 1.0d0/2.0d0*HPL1(1)*GPL3(2,0,1)
     .        + 1.0d0/2.0d0*HPL1(1)*GPL3(2,1,0)
     .        - 2.0d0*HPL1(1)*GPL3(2,1,1)
     .        + HPL1(1)*GPL3(2,2,1)
     .        - HPL1(1)*GPL3(2,3,0)
     .        + HPL1(1)*GPL3(3,0,0)
     .        + 1279.0d0/144.0d0*HPL2(0,0)
         col1 = col1 + 5.0d0/2.0d0*HPL2(0,0)*omumv**(-4)
     .        - 5.0d0*HPL2(0,0)*omumv**(-3)
     .        + 1.0d0/6.0d0*HPL2(0,0)*omumv**(-3)*pi*ci
     .        + 55.0d0/12.0d0*HPL2(0,0)*omumv**(-2)
     .        - 1.0d0/4.0d0*HPL2(0,0)*omumv**(-2)*pi*ci
     .        - 25.0d0/12.0d0*HPL2(0,0)*omumv**(-1)
     .        + HPL2(0,0)*omumv**(-1)*pi*ci
     .        + 11.0d0/2.0d0*HPL2(0,0)*pi*ci
     .        - 5.0d0/4.0d0*HPL2(0,0)*pi**2
     .        - 5.0d0*HPL2(0,0)*v*omumv**(-4)
     .        + 13.0d0/3.0d0*HPL2(0,0)*v*omumv**(-3)
     .        - 1.0d0/2.0d0*HPL2(0,0)*v*omumv**(-3)*pi*ci
     .        - 7.0d0/4.0d0*HPL2(0,0)*v*omumv**(-2)
     .        + 1.0d0/2.0d0*HPL2(0,0)*v*omumv**(-2)*pi*ci
     .        + 1.0d0/6.0d0*HPL2(0,0)*v*omumv**(-1)
         col1 = col1 - HPL2(0,0)*v*omumv**(-1)*pi*ci
     .        + 5.0d0/2.0d0*HPL2(0,0)*v**2*omumv**(-4)
     .        + 2.0d0/3.0d0*HPL2(0,0)*v**2*omumv**(-3)
     .        + 1.0d0/2.0d0*HPL2(0,0)*v**2*omumv**(-3)*pi*ci
     .        + 1.0d0/6.0d0*HPL2(0,0)*v**2*omumv**(-2)
     .        - 1.0d0/4.0d0*HPL2(0,0)*v**2*omumv**(-2)*pi*ci
     .        - 1.0d0/6.0d0*HPL2(0,0)*v**3*omumv**(-3)*pi*ci
     .        - 25.0d0/12.0d0*HPL2(0,0)*u
     .        + 5.0d0/6.0d0*HPL2(0,0)*u**2
     .        + 55.0d0/6.0d0*HPL2(0,0)*GPL1(0)
     .        + 1.0d0/6.0d0*HPL2(0,0)*GPL1(0)*omumv**(-3)
     .        - 1.0d0/4.0d0*HPL2(0,0)*GPL1(0)*omumv**(-2)
     .        + HPL2(0,0)*GPL1(0)*omumv**(-1)
     .        + 5.0d0/2.0d0*HPL2(0,0)*GPL1(0)*pi*ci
     .        - 1.0d0/2.0d0*HPL2(0,0)*GPL1(0)*v*omumv**(-3)
         col1 = col1 + 1.0d0/2.0d0*HPL2(0,0)*GPL1(0)*v*omumv**(-2)
     .        - HPL2(0,0)*GPL1(0)*v*omumv**(-1)
     .        + 1.0d0/2.0d0*HPL2(0,0)*GPL1(0)*v**2*omumv**(-3)
     .        - 1.0d0/4.0d0*HPL2(0,0)*GPL1(0)*v**2*omumv**(-2)
     .        - 1.0d0/6.0d0*HPL2(0,0)*GPL1(0)*v**3*omumv**(-3)
     .        - 11.0d0/2.0d0*HPL2(0,0)*GPL1(1)
     .        - HPL2(0,0)*GPL1(1)*pi*ci
     .        + 77.0d0/12.0d0*HPL2(0,0)*GPL1(2)
     .        + 1.0d0/2.0d0*HPL2(0,0)*GPL1(2)*pi*ci
     .        - 11.0d0/2.0d0*HPL2(0,0)*GPL1(3)
     .        - 1.0d0/3.0d0*HPL2(0,0)*GPL1(3)*omumv**(-3)
     .        + 1.0d0/2.0d0*HPL2(0,0)*GPL1(3)*omumv**(-2)
     .        - 2.0d0*HPL2(0,0)*GPL1(3)*omumv**(-1)
     .        - 3.0d0*HPL2(0,0)*GPL1(3)*pi*ci
     .        + 2.0d0*HPL2(0,0)*GPL2(0,0)
         col1 = col1 + 1.0d0/2.0d0*HPL2(0,0)*GPL2(0,2)
     .        - 2.0d0*HPL2(0,0)*GPL2(0,3)
     .        - HPL2(0,0)*GPL2(1,2)
     .        + 1.0d0/2.0d0*HPL2(0,0)*GPL2(2,0)
     .        - 3.0d0*HPL2(0,0)*GPL2(3,0)
     .        + 2.0d0*HPL2(0,0)*GPL2(3,3)
     .        + 605.0d0/144.0d0*HPL2(0,1)
     .        + 77.0d0/24.0d0*HPL2(0,1)*pi*ci
     .        - 1.0d0/2.0d0*HPL2(0,1)*pi**2
     .        + 77.0d0/24.0d0*HPL2(0,1)*GPL1(0)
     .        + HPL2(0,1)*GPL1(0)*pi*ci
     .        - 11.0d0/2.0d0*HPL2(0,1)*GPL1(1)
     .        - 1.0d0/2.0d0*HPL2(0,1)*GPL1(1)*pi*ci
     .        + 11.0d0/12.0d0*HPL2(0,1)*GPL1(2)
     .        - 1.0d0/2.0d0*HPL2(0,1)*GPL1(2)*pi*ci
         col1 = col1 - 11.0d0/12.0d0*HPL2(0,1)*GPL1(3)
     .        - 1.0d0/2.0d0*HPL2(0,1)*GPL1(3)*pi*ci
     .        + HPL2(0,1)*GPL2(0,0)
     .        - 1.0d0/2.0d0*HPL2(0,1)*GPL2(0,1)
     .        + 1.0d0/2.0d0*HPL2(0,1)*GPL2(0,2)
     .        - 1.0d0/2.0d0*HPL2(0,1)*GPL2(0,3)
     .        - 1.0d0/2.0d0*HPL2(0,1)*GPL2(1,0)
     .        - 1.0d0/2.0d0*HPL2(0,1)*GPL2(2,0)
     .        + HPL2(0,1)*GPL2(2,1)
     .        - HPL2(0,1)*GPL2(2,2)
     .        + HPL2(0,1)*GPL2(2,3)
     .        - 1.0d0/2.0d0*HPL2(0,1)*GPL2(3,0)
     .        + 605.0d0/144.0d0*HPL2(1,0)
     .        - 5.0d0/2.0d0*HPL2(1,0)*omumv**(-4)
     .        + 5.0d0*HPL2(1,0)*omumv**(-3)
         col1 = col1 + 1.0d0/6.0d0*HPL2(1,0)*omumv**(-3)*pi*ci
     .        - 53.0d0/12.0d0*HPL2(1,0)*omumv**(-2)
     .        - 1.0d0/4.0d0*HPL2(1,0)*omumv**(-2)*pi*ci
     .        + 23.0d0/12.0d0*HPL2(1,0)*omumv**(-1)
     .        + HPL2(1,0)*omumv**(-1)*pi*ci
     .        + 55.0d0/24.0d0*HPL2(1,0)*pi*ci
     .        - 1.0d0/2.0d0*HPL2(1,0)*pi**2
     .        + 5.0d0*HPL2(1,0)*v*omumv**(-4)
     .        - 13.0d0/3.0d0*HPL2(1,0)*v*omumv**(-3)
     .        - 1.0d0/2.0d0*HPL2(1,0)*v*omumv**(-3)*pi*ci
     .        + 11.0d0/12.0d0*HPL2(1,0)*v*omumv**(-2)
     .        + 1.0d0/2.0d0*HPL2(1,0)*v*omumv**(-2)*pi*ci
     .        + 1.0d0/6.0d0*HPL2(1,0)*v*omumv**(-1)
     .        - HPL2(1,0)*v*omumv**(-1)*pi*ci
     .        - 5.0d0/2.0d0*HPL2(1,0)*v**2*omumv**(-4)
         col1 = col1 - 2.0d0/3.0d0*HPL2(1,0)*v**2*omumv**(-3)
     .        + 1.0d0/2.0d0*HPL2(1,0)*v**2*omumv**(-3)*pi*ci
     .        + 1.0d0/6.0d0*HPL2(1,0)*v**2*omumv**(-2)
     .        - 1.0d0/4.0d0*HPL2(1,0)*v**2*omumv**(-2)*pi*ci
     .        - 1.0d0/6.0d0*HPL2(1,0)*v**3*omumv**(-3)*pi*ci
     .        - 25.0d0/12.0d0*HPL2(1,0)*u
     .        + 5.0d0/6.0d0*HPL2(1,0)*u**2
     .        + 55.0d0/24.0d0*HPL2(1,0)*GPL1(0)
     .        + 1.0d0/6.0d0*HPL2(1,0)*GPL1(0)*omumv**(-3)
     .        - 1.0d0/4.0d0*HPL2(1,0)*GPL1(0)*omumv**(-2)
     .        + HPL2(1,0)*GPL1(0)*omumv**(-1)
     .        + HPL2(1,0)*GPL1(0)*pi*ci
     .        - 1.0d0/2.0d0*HPL2(1,0)*GPL1(0)*v*omumv**(-3)
     .        + 1.0d0/2.0d0*HPL2(1,0)*GPL1(0)*v*omumv**(-2)
     .        - HPL2(1,0)*GPL1(0)*v*omumv**(-1)
         col1 = col1 + 1.0d0/2.0d0*HPL2(1,0)*GPL1(0)*v**2*omumv**(-3)
     .        - 1.0d0/4.0d0*HPL2(1,0)*GPL1(0)*v**2*omumv**(-2)
     .        - 1.0d0/6.0d0*HPL2(1,0)*GPL1(0)*v**3*omumv**(-3)
     .        - 11.0d0/3.0d0*HPL2(1,0)*GPL1(1)
     .        - 1.0d0/2.0d0*HPL2(1,0)*GPL1(1)*pi*ci
     .        + 11.0d0/4.0d0*HPL2(1,0)*GPL1(2)
     .        - 1.0d0/2.0d0*HPL2(1,0)*GPL1(2)*pi*ci
     .        - 11.0d0/12.0d0*HPL2(1,0)*GPL1(3)
     .        - 1.0d0/2.0d0*HPL2(1,0)*GPL1(3)*pi*ci
     .        + HPL2(1,0)*GPL2(0,0)
     .        - 1.0d0/2.0d0*HPL2(1,0)*GPL2(0,1)
     .        + 1.0d0/2.0d0*HPL2(1,0)*GPL2(0,2)
     .        - 1.0d0/2.0d0*HPL2(1,0)*GPL2(0,3)
     .        - 1.0d0/2.0d0*HPL2(1,0)*GPL2(1,0)
     .        + 2.0d0*HPL2(1,0)*GPL2(1,1)
         col1 = col1 - 2.0d0*HPL2(1,0)*GPL2(1,2)
     .        - 1.0d0/2.0d0*HPL2(1,0)*GPL2(2,0)
     .        - HPL2(1,0)*GPL2(2,1)
     .        + HPL2(1,0)*GPL2(2,2)
     .        + HPL2(1,0)*GPL2(2,3)
     .        - 1.0d0/2.0d0*HPL2(1,0)*GPL2(3,0)
     .        + 121.0d0/48.0d0*HPL2(1,1)
     .        + 11.0d0/6.0d0*HPL2(1,1)*pi*ci
     .        - 1.0d0/4.0d0*HPL2(1,1)*pi**2
     .        + 11.0d0/6.0d0*HPL2(1,1)*GPL1(0)
     .        + 1.0d0/2.0d0*HPL2(1,1)*GPL1(0)*pi*ci
     .        - 11.0d0/3.0d0*HPL2(1,1)*GPL1(1)
     .        - HPL2(1,1)*GPL1(1)*pi*ci
     .        + 1.0d0/2.0d0*HPL2(1,1)*GPL2(0,0)
     .        - HPL2(1,1)*GPL2(0,1)
         col1 = col1 - HPL2(1,1)*GPL2(1,0)
     .        + 2.0d0*HPL2(1,1)*GPL2(1,1)
     .        - 121.0d0/12.0d0*HPL3(0,0,0)
     .        - 1.0d0/3.0d0*HPL3(0,0,0)*omumv**(-3)
     .        + 1.0d0/2.0d0*HPL3(0,0,0)*omumv**(-2)
     .        - 2.0d0*HPL3(0,0,0)*omumv**(-1)
     .        - 3.0d0*HPL3(0,0,0)*pi*ci
     .        + 1.0d0/2.0d0*HPL3(0,0,0)*v*omumv**(-3)
     .        - 1.0d0/2.0d0*HPL3(0,0,0)*v*omumv**(-2)
     .        + HPL3(0,0,0)*v*omumv**(-1)
     .        - 1.0d0/2.0d0*HPL3(0,0,0)*v**2*omumv**(-3)
     .        + 1.0d0/4.0d0*HPL3(0,0,0)*v**2*omumv**(-2)
     .        + 1.0d0/6.0d0*HPL3(0,0,0)*v**3*omumv**(-3)
     .        - 3.0d0*HPL3(0,0,0)*GPL1(0)
     .        + HPL3(0,0,0)*GPL1(1)
         col1 = col1 - HPL3(0,0,0)*GPL1(2)
     .        + 3.0d0*HPL3(0,0,0)*GPL1(3)
     .        - 11.0d0/12.0d0*HPL3(0,0,1)
     .        - 1.0d0/2.0d0*HPL3(0,0,1)*pi*ci
     .        - 1.0d0/2.0d0*HPL3(0,0,1)*GPL1(0)
     .        + HPL3(0,0,1)*GPL1(1)
     .        - 11.0d0/12.0d0*HPL3(0,1,0)
     .        - 1.0d0/2.0d0*HPL3(0,1,0)*pi*ci
     .        - 1.0d0/2.0d0*HPL3(0,1,0)*v*omumv**(-3)
     .        + 1.0d0/2.0d0*HPL3(0,1,0)*v*omumv**(-2)
     .        - HPL3(0,1,0)*v*omumv**(-1)
     .        + 1.0d0/2.0d0*HPL3(0,1,0)*v**2*omumv**(-3)
     .        - 1.0d0/4.0d0*HPL3(0,1,0)*v**2*omumv**(-2)
     .        - 1.0d0/6.0d0*HPL3(0,1,0)*v**3*omumv**(-3)
     .        - 1.0d0/2.0d0*HPL3(0,1,0)*GPL1(0)
         col1 = col1 + HPL3(0,1,0)*GPL1(1)
     .        - 11.0d0/12.0d0*HPL3(1,0,0)
     .        - 1.0d0/2.0d0*HPL3(1,0,0)*pi*ci
     .        - 1.0d0/2.0d0*HPL3(1,0,0)*GPL1(0)
     .        + HPL3(1,0,0)*GPL1(1)
     .        - 11.0d0/3.0d0*HPL3(1,1,0)
     .        + 1.0d0/3.0d0*HPL3(1,1,0)*omumv**(-3)
     .        - 1.0d0/2.0d0*HPL3(1,1,0)*omumv**(-2)
     .        + 2.0d0*HPL3(1,1,0)*omumv**(-1)
     .        - HPL3(1,1,0)*v*omumv**(-3)
     .        + HPL3(1,1,0)*v*omumv**(-2)
     .        - 2.0d0*HPL3(1,1,0)*v*omumv**(-1)
     .        + HPL3(1,1,0)*v**2*omumv**(-3)
     .        - 1.0d0/2.0d0*HPL3(1,1,0)*v**2*omumv**(-2)
     .        - 1.0d0/3.0d0*HPL3(1,1,0)*v**3*omumv**(-3)
         col1 = col1 + 2.0d0*HPL3(1,1,0)*GPL1(1)
     .        - 2.0d0*HPL3(1,1,0)*GPL1(2)
     .        + 3.0d0*HPL4(0,0,0,0)
     .        - 77.0d0/24.0d0*GPL1(0)
     .        - 5.0d0/2.0d0*GPL1(0)*omumv**(-3)
     .        + 1.0d0/36.0d0*GPL1(0)*omumv**(-3)*pi**2
     .        + 281.0d0/72.0d0*GPL1(0)*omumv**(-2)
     .        + 1.0d0/12.0d0*GPL1(0)*omumv**(-2)*pi*ci
     .        - 1.0d0/24.0d0*GPL1(0)*omumv**(-2)*pi**2
     .        - 203.0d0/72.0d0*GPL1(0)*omumv**(-1)
     .        - 1.0d0/12.0d0*GPL1(0)*omumv**(-1)*pi*ci
     .        + 1.0d0/6.0d0*GPL1(0)*omumv**(-1)*pi**2
     .        - 389.0d0/144.0d0*GPL1(0)*pi*ci
     .        - 121.0d0/288.0d0*GPL1(0)*pi**2
     .        - 1.0d0/8.0d0*GPL1(0)*pi**3*ci
         col1 = col1 - 1.0d0/4.0d0*GPL1(0)*zeta3
     .        + 5.0d0*GPL1(0)*v*omumv**(-3)
     .        - 1.0d0/12.0d0*GPL1(0)*v*omumv**(-3)*pi**2
     .        - 155.0d0/72.0d0*GPL1(0)*v*omumv**(-2)
     .        - 1.0d0/12.0d0*GPL1(0)*v*omumv**(-2)*pi*ci
     .        + 1.0d0/12.0d0*GPL1(0)*v*omumv**(-2)*pi**2
     .        + 1.0d0/2.0d0*GPL1(0)*v*omumv**(-1)
     .        - 1.0d0/6.0d0*GPL1(0)*v*omumv**(-1)*pi**2
     .        + 5.0d0/3.0d0*GPL1(0)*v*upv**(-1)
     .        - 5.0d0/6.0d0*GPL1(0)*v
     .        - 5.0d0/2.0d0*GPL1(0)*v**2*omumv**(-3)
     .        + 1.0d0/12.0d0*GPL1(0)*v**2*omumv**(-3)*pi**2
     .        - 7.0d0/4.0d0*GPL1(0)*v**2*omumv**(-2)
     .        - 1.0d0/24.0d0*GPL1(0)*v**2*omumv**(-2)*pi**2
     .        - 5.0d0/6.0d0*GPL1(0)*v**2*omumv**(-1)
         col1 = col1 + 4.0d0/3.0d0*GPL1(0)*v**2*upv**(-2)
     .        - 1.0d0/36.0d0*GPL1(0)*v**3*omumv**(-3)*pi**2
     .        + 5.0d0/6.0d0*GPL1(0)*u
     .        - 25.0d0/12.0d0*GPL1(0)*u*pi*ci
     .        + 5.0d0/6.0d0*GPL1(0)*u**2*pi*ci
     .        - 5.0d0/6.0d0*GPL1(1)*u**(-2)*pi*ci
     .        + 25.0d0/12.0d0*GPL1(1)*u**(-1)*pi*ci
     .        - 1.0d0/6.0d0*GPL1(1)*omumv**(-2)*pi*ci
     .        + 1.0d0/6.0d0*GPL1(1)*omumv**(-1)*pi*ci
     .        + 67.0d0/18.0d0*GPL1(1)*pi*ci
     .        - 77.0d0/36.0d0*GPL1(1)*pi**2
     .        + 1.0d0/4.0d0*GPL1(1)*pi**3*ci
     .        - 2.0d0*GPL1(1)*zeta3
     .        + 1.0d0/6.0d0*GPL1(1)*v*omumv**(-2)*pi*ci
     .        + 25.0d0/12.0d0*GPL1(1)*u*pi*ci
         col1 = col1 - 5.0d0/6.0d0*GPL1(1)*u**2*pi*ci
     .        + 1.0d0/8.0d0*GPL1(2)
     .        - 5.0d0/6.0d0*GPL1(2)*u**(-1)
     .        + 11.0d0/24.0d0*GPL1(2)*omumv**(-2)
     .        + 1.0d0/12.0d0*GPL1(2)*omumv**(-2)*pi*ci
     .        - 11.0d0/24.0d0*GPL1(2)*omumv**(-1)
     .        - 1.0d0/12.0d0*GPL1(2)*omumv**(-1)*pi*ci
     .        - 389.0d0/144.0d0*GPL1(2)*pi*ci
     .        + 55.0d0/288.0d0*GPL1(2)*pi**2
     .        - 1.0d0/8.0d0*GPL1(2)*pi**3*ci
     .        + 7.0d0/4.0d0*GPL1(2)*zeta3
     .        - 11.0d0/24.0d0*GPL1(2)*v*omumv**(-2)
     .        - 1.0d0/12.0d0*GPL1(2)*v*omumv**(-2)*pi*ci
     .        - 5.0d0/6.0d0*GPL1(2)*u
     .        - 25.0d0/12.0d0*GPL1(2)*u*pi*ci
         col1 = col1 + 5.0d0/6.0d0*GPL1(2)*u**2*pi*ci
     .        - 5.0d0/2.0d0*GPL1(3)*omumv**(-4)*pi*ci
     .        + 5.0d0*GPL1(3)*omumv**(-3)*pi*ci
     .        - 1.0d0/18.0d0*GPL1(3)*omumv**(-3)*pi**2
     .        - 55.0d0/12.0d0*GPL1(3)*omumv**(-2)*pi*ci
     .        + 1.0d0/12.0d0*GPL1(3)*omumv**(-2)*pi**2
     .        + 25.0d0/12.0d0*GPL1(3)*omumv**(-1)*pi*ci
     .        - 1.0d0/3.0d0*GPL1(3)*omumv**(-1)*pi**2
     .        + 67.0d0/18.0d0*GPL1(3)*pi*ci
     .        - 11.0d0/6.0d0*GPL1(3)*pi**2
     .        + 1.0d0/4.0d0*GPL1(3)*pi**3*ci
     .        + 5.0d0*GPL1(3)*v*omumv**(-4)*pi*ci
     .        - 13.0d0/3.0d0*GPL1(3)*v*omumv**(-3)*pi*ci
     .        + 7.0d0/4.0d0*GPL1(3)*v*omumv**(-2)*pi*ci
     .        - 1.0d0/6.0d0*GPL1(3)*v*omumv**(-1)*pi*ci
         col1 = col1 - 5.0d0/2.0d0*GPL1(3)*v**2*omumv**(-4)*pi*ci
     .        - 2.0d0/3.0d0*GPL1(3)*v**2*omumv**(-3)*pi*ci
     .        - 1.0d0/6.0d0*GPL1(3)*v**2*omumv**(-2)*pi*ci
     .        + 25.0d0/12.0d0*GPL1(3)*u*pi*ci
     .        - 5.0d0/6.0d0*GPL1(3)*u**2*pi*ci
     .        + 121.0d0/48.0d0*GPL2(0,0)
     .        + 11.0d0/6.0d0*GPL2(0,0)*pi*ci
     .        - 1.0d0/4.0d0*GPL2(0,0)*pi**2
     .        - 1.0d0/2.0d0*GPL2(0,1)*pi**2
     .        - 49.0d0/48.0d0*GPL2(0,2)
     .        + 1.0d0/12.0d0*GPL2(0,2)*omumv**(-2)
     .        - 1.0d0/12.0d0*GPL2(0,2)*omumv**(-1)
     .        + 11.0d0/8.0d0*GPL2(0,2)*pi*ci
     .        - 3.0d0/8.0d0*GPL2(0,2)*pi**2
     .        - 1.0d0/12.0d0*GPL2(0,2)*v*omumv**(-2)
         col1 = col1 - 25.0d0/12.0d0*GPL2(0,2)*u
     .        + 5.0d0/6.0d0*GPL2(0,2)*u**2
     .        - 1.0d0/6.0d0*GPL2(0,3)*omumv**(-3)*pi*ci
     .        + 1.0d0/4.0d0*GPL2(0,3)*omumv**(-2)*pi*ci
     .        - GPL2(0,3)*omumv**(-1)*pi*ci
     .        - 11.0d0/3.0d0*GPL2(0,3)*pi*ci
     .        + 1.0d0/6.0d0*GPL2(0,3)*pi**2
     .        + 1.0d0/2.0d0*GPL2(0,3)*v*omumv**(-3)*pi*ci
     .        - 1.0d0/2.0d0*GPL2(0,3)*v*omumv**(-2)*pi*ci
     .        + GPL2(0,3)*v*omumv**(-1)*pi*ci
     .        - 1.0d0/2.0d0*GPL2(0,3)*v**2*omumv**(-3)*pi*ci
     .        + 1.0d0/4.0d0*GPL2(0,3)*v**2*omumv**(-2)*pi*ci
     .        + 1.0d0/6.0d0*GPL2(0,3)*v**3*omumv**(-3)*pi*ci
     .        + 1.0d0/2.0d0*GPL2(1,0)*pi**2
     .        - 11.0d0/6.0d0*GPL2(1,1)*pi*ci
         col1 = col1 - 1.0d0/3.0d0*GPL2(1,1)*pi**2
     .        + 67.0d0/18.0d0*GPL2(1,2)
     .        - 5.0d0/6.0d0*GPL2(1,2)*u**(-2)
     .        + 25.0d0/12.0d0*GPL2(1,2)*u**(-1)
     .        - 1.0d0/6.0d0*GPL2(1,2)*omumv**(-2)
     .        + 1.0d0/6.0d0*GPL2(1,2)*omumv**(-1)
     .        + 11.0d0/12.0d0*GPL2(1,2)*pi*ci
     .        + 5.0d0/12.0d0*GPL2(1,2)*pi**2
     .        + 1.0d0/6.0d0*GPL2(1,2)*v*omumv**(-2)
     .        + 25.0d0/12.0d0*GPL2(1,2)*u
     .        - 5.0d0/6.0d0*GPL2(1,2)*u**2
     .        - 49.0d0/48.0d0*GPL2(2,0)
     .        + 1.0d0/12.0d0*GPL2(2,0)*omumv**(-2)
     .        - 1.0d0/12.0d0*GPL2(2,0)*omumv**(-1)
     .        + 11.0d0/8.0d0*GPL2(2,0)*pi*ci
         col1 = col1 - 3.0d0/8.0d0*GPL2(2,0)*pi**2
     .        - 1.0d0/12.0d0*GPL2(2,0)*v*omumv**(-2)
     .        - 25.0d0/12.0d0*GPL2(2,0)*u
     .        + 5.0d0/6.0d0*GPL2(2,0)*u**2
     .        - 11.0d0/4.0d0*GPL2(2,1)*pi*ci
     .        - 1.0d0/6.0d0*GPL2(2,1)*pi**2
     .        + 121.0d0/48.0d0*GPL2(2,2)
     .        + 11.0d0/6.0d0*GPL2(2,2)*pi*ci
     .        + 1.0d0/12.0d0*GPL2(2,2)*pi**2
     .        - 11.0d0/12.0d0*GPL2(2,3)*pi*ci
     .        - 1.0d0/2.0d0*GPL2(2,3)*pi**2
     .        + 67.0d0/18.0d0*GPL2(3,0)
     .        - 5.0d0/2.0d0*GPL2(3,0)*omumv**(-4)
     .        + 5.0d0*GPL2(3,0)*omumv**(-3)
     .        - 55.0d0/12.0d0*GPL2(3,0)*omumv**(-2)
         col1 = col1 + 25.0d0/12.0d0*GPL2(3,0)*omumv**(-1)
     .        + 3.0d0/4.0d0*GPL2(3,0)*pi**2
     .        + 5.0d0*GPL2(3,0)*v*omumv**(-4)
     .        - 13.0d0/3.0d0*GPL2(3,0)*v*omumv**(-3)
     .        + 7.0d0/4.0d0*GPL2(3,0)*v*omumv**(-2)
     .        - 1.0d0/6.0d0*GPL2(3,0)*v*omumv**(-1)
     .        - 5.0d0/2.0d0*GPL2(3,0)*v**2*omumv**(-4)
     .        - 2.0d0/3.0d0*GPL2(3,0)*v**2*omumv**(-3)
     .        - 1.0d0/6.0d0*GPL2(3,0)*v**2*omumv**(-2)
     .        + 25.0d0/12.0d0*GPL2(3,0)*u
     .        - 5.0d0/6.0d0*GPL2(3,0)*u**2
     .        - 11.0d0/12.0d0*GPL2(3,2)*pi*ci
     .        + 1.0d0/2.0d0*GPL2(3,2)*pi**2
     .        + 1.0d0/3.0d0*GPL2(3,3)*omumv**(-3)*pi*ci
     .        - 1.0d0/2.0d0*GPL2(3,3)*omumv**(-2)*pi*ci
         col1 = col1 + 2.0d0*GPL2(3,3)*omumv**(-1)*pi*ci
     .        - 2.0d0/3.0d0*GPL2(3,3)*pi**2
     .        + GPL3(0,0,1)*pi*ci
     .        + 11.0d0/6.0d0*GPL3(0,0,2)
     .        + 1.0d0/2.0d0*GPL3(0,0,2)*pi*ci
     .        - GPL3(0,0,3)*pi*ci
     .        + GPL3(0,1,0)*pi*ci
     .        + 11.0d0/6.0d0*GPL3(0,2,0)
     .        + 1.0d0/2.0d0*GPL3(0,2,0)*pi*ci
     .        - 1.0d0/2.0d0*GPL3(0,2,1)*pi*ci
     .        + 11.0d0/6.0d0*GPL3(0,2,2)
     .        + 1.0d0/2.0d0*GPL3(0,2,2)*pi*ci
     .        - 1.0d0/2.0d0*GPL3(0,2,3)*pi*ci
     .        - 11.0d0/3.0d0*GPL3(0,3,0)
     .        - 1.0d0/6.0d0*GPL3(0,3,0)*omumv**(-3)
         col1 = col1 + 1.0d0/4.0d0*GPL3(0,3,0)*omumv**(-2)
     .        - GPL3(0,3,0)*omumv**(-1)
     .        - 1.0d0/2.0d0*GPL3(0,3,0)*pi*ci
     .        + 1.0d0/2.0d0*GPL3(0,3,0)*v*omumv**(-3)
     .        - 1.0d0/2.0d0*GPL3(0,3,0)*v*omumv**(-2)
     .        + GPL3(0,3,0)*v*omumv**(-1)
     .        - 1.0d0/2.0d0*GPL3(0,3,0)*v**2*omumv**(-3)
     .        + 1.0d0/4.0d0*GPL3(0,3,0)*v**2*omumv**(-2)
     .        + 1.0d0/6.0d0*GPL3(0,3,0)*v**3*omumv**(-3)
     .        - 1.0d0/2.0d0*GPL3(0,3,2)*pi*ci
     .        + 2.0d0*GPL3(0,3,3)*pi*ci
     .        - GPL3(1,0,2)*pi*ci
     .        - 2.0d0*GPL3(1,1,1)*pi*ci
     .        - 11.0d0/6.0d0*GPL3(1,1,2)
     .        + 2.0d0*GPL3(1,1,2)*pi*ci
         col1 = col1 - GPL3(1,2,0)*pi*ci
     .        + 2.0d0*GPL3(1,2,1)*pi*ci
     .        - 11.0d0/3.0d0*GPL3(1,2,2)
     .        - GPL3(1,2,2)*pi*ci
     .        + 11.0d0/6.0d0*GPL3(2,0,0)
     .        + 1.0d0/2.0d0*GPL3(2,0,0)*pi*ci
     .        - 1.0d0/2.0d0*GPL3(2,0,1)*pi*ci
     .        + 11.0d0/6.0d0*GPL3(2,0,2)
     .        + 1.0d0/2.0d0*GPL3(2,0,2)*pi*ci
     .        - 1.0d0/2.0d0*GPL3(2,0,3)*pi*ci
     .        - 1.0d0/2.0d0*GPL3(2,1,0)*pi*ci
     .        + 2.0d0*GPL3(2,1,1)*pi*ci
     .        - 11.0d0/4.0d0*GPL3(2,1,2)
     .        - 1.0d0/2.0d0*GPL3(2,1,2)*pi*ci
     .        + 11.0d0/6.0d0*GPL3(2,2,0)
         col1 = col1 + 1.0d0/2.0d0*GPL3(2,2,0)*pi*ci
     .        - GPL3(2,2,1)*pi*ci
     .        + GPL3(2,2,3)*pi*ci
     .        - 11.0d0/12.0d0*GPL3(2,3,0)
     .        + GPL3(2,3,2)*pi*ci
     .        - 11.0d0/3.0d0*GPL3(3,0,0)
     .        - GPL3(3,0,0)*pi*ci
     .        - 11.0d0/12.0d0*GPL3(3,0,2)
     .        - GPL3(3,0,2)*pi*ci
     .        + 2.0d0*GPL3(3,0,3)*pi*ci
     .        - 11.0d0/12.0d0*GPL3(3,2,0)
     .        - GPL3(3,2,0)*pi*ci
     .        + 1.0d0/3.0d0*GPL3(3,3,0)*omumv**(-3)
     .        - 1.0d0/2.0d0*GPL3(3,3,0)*omumv**(-2)
     .        + 2.0d0*GPL3(3,3,0)*omumv**(-1)
         col1 = col1 + 2.0d0*GPL3(3,3,0)*pi*ci
     .        - 2.0d0*GPL3(3,3,3)*pi*ci
     .        + GPL4(0,0,1,2)
     .        + 1.0d0/2.0d0*GPL4(0,0,2,2)
     .        - GPL4(0,0,3,0)
     .        + GPL4(0,1,0,2)
     .        + GPL4(0,1,2,0)
     .        - GPL4(0,1,2,2)
     .        + 1.0d0/2.0d0*GPL4(0,2,0,2)
     .        - 1.0d0/2.0d0*GPL4(0,2,1,2)
     .        + 1.0d0/2.0d0*GPL4(0,2,2,0)
     .        - 1.0d0/2.0d0*GPL4(0,2,3,0)
     .        - 1.0d0/2.0d0*GPL4(0,3,0,2)
     .        - 1.0d0/2.0d0*GPL4(0,3,2,0)
     .        + 2.0d0*GPL4(0,3,3,0)
         col1 = col1 - GPL4(1,0,2,2)
     .        - 2.0d0*GPL4(1,1,1,2)
     .        + 2.0d0*GPL4(1,1,2,2)
     .        - GPL4(1,2,0,2)
     .        + 2.0d0*GPL4(1,2,1,2)
     .        - GPL4(1,2,2,0)
     .        + 1.0d0/2.0d0*GPL4(2,0,0,2)
     .        - 1.0d0/2.0d0*GPL4(2,0,1,2)
     .        + 1.0d0/2.0d0*GPL4(2,0,2,0)
     .        - 1.0d0/2.0d0*GPL4(2,0,3,0)
     .        - 1.0d0/2.0d0*GPL4(2,1,0,2)
     .        + 2.0d0*GPL4(2,1,1,2)
     .        - 1.0d0/2.0d0*GPL4(2,1,2,0)
     .        + 1.0d0/2.0d0*GPL4(2,2,0,0)
     .        - GPL4(2,2,1,2)
         col1 = col1 + GPL4(2,2,3,0)
     .        - GPL4(2,3,0,0)
     .        + GPL4(2,3,0,2)
     .        + GPL4(2,3,2,0)
     .        - GPL4(3,0,0,2)
     .        - GPL4(3,0,2,0)
     .        + 2.0d0*GPL4(3,0,3,0)
     .        - GPL4(3,2,0,0)
     .        + 2.0d0*GPL4(3,3,0,0)
     .        - 2.0d0*GPL4(3,3,3,0)

         col2 = (0.d0,0.d0)

         col3 = (0.d0,0.d0)

         col4 = + 23.0d0/16.0d0
     .        + 1.0d0/12.0d0*u**(-1)*pi*ci
     .        - 1.0d0/4.0d0*omv**(-1)
     .        + 1.0d0/24.0d0*omumv**(-4)*pi**2
     .        + 1.0d0/4.0d0*omumv**(-3)*pi*ci
     .        - 1.0d0/12.0d0*omumv**(-3)*pi**2
     .        + 1.0d0/4.0d0*omumv**(-2)
     .        - 3.0d0/8.0d0*omumv**(-2)*pi*ci
     .        + 1.0d0/72.0d0*omumv**(-2)*pi**2
     .        - 1.0d0/4.0d0*omumv**(-1)
     .        + 1.0d0/12.0d0*omumv**(-1)*pi*ci
     .        + 1.0d0/36.0d0*omumv**(-1)*pi**2
     .        + 1.0d0/4.0d0*pi*ci
     .        - zeta3
     .        - 1.0d0/12.0d0*v*omumv**(-4)*pi**2
         col4 = col4 - 1.0d0/2.0d0*v*omumv**(-3)*pi*ci
     .        + 5.0d0/36.0d0*v*omumv**(-3)*pi**2
     .        - 1.0d0/4.0d0*v*omumv**(-2)
     .        + 7.0d0/12.0d0*v*omumv**(-2)*pi*ci
     .        - 1.0d0/36.0d0*v*omumv**(-2)*pi**2
     .        + 7.0d0/24.0d0*v*omumv**(-1)
     .        - 1.0d0/4.0d0*v*omumv**(-1)*pi*ci
     .        + 1.0d0/4.0d0*v*upv**(-1)
     .        + 1.0d0/12.0d0*v*pi*ci
     .        + 1.0d0/24.0d0*v**2*omumv**(-4)*pi**2
     .        + 1.0d0/4.0d0*v**2*omumv**(-3)*pi*ci
     .        - 1.0d0/18.0d0*v**2*omumv**(-3)*pi**2
     .        - 5.0d0/24.0d0*v**2*omumv**(-2)*pi*ci
     .        + 1.0d0/12.0d0*v**2*omumv**(-1)*pi*ci
     .        + 1.0d0/4.0d0*v**2*upv**(-2)*pi*ci
         col4 = col4 + 1.0d0/9.0d0*u*pi**2
     .        + 1.0d0/18.0d0*u**2*pi**2
     .        + 1.0d0/12.0d0*HPL1(0)*u**(-1)*omv**(-1)
     .        - 1.0d0/12.0d0*HPL1(0)*u**(-1)
     .        - 1.0d0/4.0d0*HPL1(0)*omv**(-2)
     .        + 1.0d0/12.0d0*HPL1(0)*omv**(-1)*omumv**(-1)
     .        + 1.0d0/2.0d0*HPL1(0)*omv**(-1)
     .        + 1.0d0/4.0d0*HPL1(0)*omumv**(-4)*pi*ci
     .        - 1.0d0/2.0d0*HPL1(0)*omumv**(-3)*pi*ci
     .        + 1.0d0/12.0d0*HPL1(0)*omumv**(-2)*pi*ci
     .        - 1.0d0/12.0d0*HPL1(0)*omumv**(-1)
     .        + 1.0d0/6.0d0*HPL1(0)*omumv**(-1)*pi*ci
     .        - 1.0d0/2.0d0*HPL1(0)*v*omumv**(-4)*pi*ci
     .        + 1.0d0/4.0d0*HPL1(0)*v*omumv**(-3)
     .        + 5.0d0/6.0d0*HPL1(0)*v*omumv**(-3)*pi*ci
         col4 = col4 - 1.0d0/4.0d0*HPL1(0)*v*omumv**(-2)
     .        - 1.0d0/6.0d0*HPL1(0)*v*omumv**(-2)*pi*ci
     .        + 1.0d0/4.0d0*HPL1(0)*v*omumv**(-1)
     .        - 1.0d0/12.0d0*HPL1(0)*v
     .        + 1.0d0/4.0d0*HPL1(0)*v**2*omumv**(-4)*pi*ci
     .        - 1.0d0/4.0d0*HPL1(0)*v**2*omumv**(-3)
     .        - 1.0d0/3.0d0*HPL1(0)*v**2*omumv**(-3)*pi*ci
     .        + 5.0d0/24.0d0*HPL1(0)*v**2*omumv**(-2)
     .        - 1.0d0/12.0d0*HPL1(0)*v**2*omumv**(-1)
     .        - 1.0d0/4.0d0*HPL1(0)*v**2*upv**(-2)
     .        + 1.0d0/6.0d0*HPL1(0)*u*pi*ci
     .        + 1.0d0/12.0d0*HPL1(0)*u**2*pi*ci
     .        + 1.0d0/4.0d0*HPL1(0)*GPL1(0)*omumv**(-4)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL1(0)*omumv**(-3)
     .        + 1.0d0/12.0d0*HPL1(0)*GPL1(0)*omumv**(-2)
         col4 = col4 + 1.0d0/6.0d0*HPL1(0)*GPL1(0)*omumv**(-1)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL1(0)*v*omumv**(-4)
     .        + 5.0d0/6.0d0*HPL1(0)*GPL1(0)*v*omumv**(-3)
     .        - 1.0d0/6.0d0*HPL1(0)*GPL1(0)*v*omumv**(-2)
     .        + 1.0d0/4.0d0*HPL1(0)*GPL1(0)*v**2*omumv**(-4)
     .        - 1.0d0/3.0d0*HPL1(0)*GPL1(0)*v**2*omumv**(-3)
     .        + 1.0d0/6.0d0*HPL1(0)*GPL1(0)*u
     .        + 1.0d0/12.0d0*HPL1(0)*GPL1(0)*u**2
     .        - 1.0d0/12.0d0*HPL1(0)*GPL1(1)*u**(-2)
     .        - 1.0d0/6.0d0*HPL1(0)*GPL1(1)*u**(-1)
     .        - 1.0d0/6.0d0*HPL1(0)*GPL1(1)*u
     .        - 1.0d0/12.0d0*HPL1(0)*GPL1(1)*u**2
     .        + 1.0d0/12.0d0*HPL1(0)*GPL1(2)*u**(-2)
     .        + 1.0d0/6.0d0*HPL1(0)*GPL1(2)*u**(-1)
     .        + 1.0d0/6.0d0*HPL1(0)*GPL1(2)*u
         col4 = col4 + 1.0d0/12.0d0*HPL1(0)*GPL1(2)*u**2
     .        - 1.0d0/4.0d0*HPL1(0)*GPL1(3)*omumv**(-4)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL1(3)*omumv**(-3)
     .        - 1.0d0/12.0d0*HPL1(0)*GPL1(3)*omumv**(-2)
     .        - 1.0d0/6.0d0*HPL1(0)*GPL1(3)*omumv**(-1)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL1(3)*v*omumv**(-4)
     .        - 5.0d0/6.0d0*HPL1(0)*GPL1(3)*v*omumv**(-3)
     .        + 1.0d0/6.0d0*HPL1(0)*GPL1(3)*v*omumv**(-2)
     .        - 1.0d0/4.0d0*HPL1(0)*GPL1(3)*v**2*omumv**(-4)
     .        + 1.0d0/3.0d0*HPL1(0)*GPL1(3)*v**2*omumv**(-3)
     .        - 1.0d0/6.0d0*HPL1(0)*GPL1(3)*u
     .        - 1.0d0/12.0d0*HPL1(0)*GPL1(3)*u**2
     .        - 5.0d0/24.0d0*HPL1(1)
     .        - 1.0d0/12.0d0*HPL1(1)*u**(-1)
     .        - 1.0d0/12.0d0*HPL1(1)*u
         col4 = col4 + 1.0d0/6.0d0*HPL1(1)*u*pi*ci
     .        + 1.0d0/12.0d0*HPL1(1)*u**2*pi*ci
     .        + 1.0d0/6.0d0*HPL1(1)*GPL1(0)*u
     .        + 1.0d0/12.0d0*HPL1(1)*GPL1(0)*u**2
     .        - 1.0d0/12.0d0*HPL1(1)*GPL1(1)*u**(-2)
     .        - 1.0d0/6.0d0*HPL1(1)*GPL1(1)*u**(-1)
     .        - 1.0d0/6.0d0*HPL1(1)*GPL1(1)*u
     .        - 1.0d0/12.0d0*HPL1(1)*GPL1(1)*u**2
     .        - 1.0d0/4.0d0*HPL2(0,0)*omumv**(-4)
     .        + 1.0d0/2.0d0*HPL2(0,0)*omumv**(-3)
     .        - 1.0d0/12.0d0*HPL2(0,0)*omumv**(-2)
     .        - 1.0d0/6.0d0*HPL2(0,0)*omumv**(-1)
     .        + 1.0d0/2.0d0*HPL2(0,0)*v*omumv**(-4)
     .        - 5.0d0/6.0d0*HPL2(0,0)*v*omumv**(-3)
     .        + 1.0d0/6.0d0*HPL2(0,0)*v*omumv**(-2)
         col4 = col4 - 1.0d0/4.0d0*HPL2(0,0)*v**2*omumv**(-4)
     .        + 1.0d0/3.0d0*HPL2(0,0)*v**2*omumv**(-3)
     .        - 1.0d0/6.0d0*HPL2(0,0)*u
     .        - 1.0d0/12.0d0*HPL2(0,0)*u**2
     .        + 1.0d0/4.0d0*HPL2(1,0)*omumv**(-4)
     .        - 1.0d0/2.0d0*HPL2(1,0)*omumv**(-3)
     .        + 1.0d0/12.0d0*HPL2(1,0)*omumv**(-2)
     .        + 1.0d0/6.0d0*HPL2(1,0)*omumv**(-1)
     .        - 1.0d0/2.0d0*HPL2(1,0)*v*omumv**(-4)
     .        + 5.0d0/6.0d0*HPL2(1,0)*v*omumv**(-3)
     .        - 1.0d0/6.0d0*HPL2(1,0)*v*omumv**(-2)
     .        + 1.0d0/4.0d0*HPL2(1,0)*v**2*omumv**(-4)
     .        - 1.0d0/3.0d0*HPL2(1,0)*v**2*omumv**(-3)
     .        - 1.0d0/6.0d0*HPL2(1,0)*u
     .        - 1.0d0/12.0d0*HPL2(1,0)*u**2
         col4 = col4 - 5.0d0/24.0d0*GPL1(0)
     .        + 1.0d0/4.0d0*GPL1(0)*omumv**(-3)
     .        - 3.0d0/8.0d0*GPL1(0)*omumv**(-2)
     .        + 1.0d0/12.0d0*GPL1(0)*omumv**(-1)
     .        - 1.0d0/2.0d0*GPL1(0)*v*omumv**(-3)
     .        + 7.0d0/12.0d0*GPL1(0)*v*omumv**(-2)
     .        - 1.0d0/4.0d0*GPL1(0)*v*omumv**(-1)
     .        + 1.0d0/12.0d0*GPL1(0)*v
     .        + 1.0d0/4.0d0*GPL1(0)*v**2*omumv**(-3)
     .        - 5.0d0/24.0d0*GPL1(0)*v**2*omumv**(-2)
     .        + 1.0d0/12.0d0*GPL1(0)*v**2*omumv**(-1)
     .        + 1.0d0/4.0d0*GPL1(0)*v**2*upv**(-2)
     .        - 1.0d0/12.0d0*GPL1(0)*u
     .        - 1.0d0/6.0d0*GPL1(0)*u*pi*ci
     .        - 1.0d0/12.0d0*GPL1(0)*u**2*pi*ci
         col4 = col4 + 1.0d0/12.0d0*GPL1(1)*u**(-2)*pi*ci
     .        + 1.0d0/6.0d0*GPL1(1)*u**(-1)*pi*ci
     .        + 1.0d0/6.0d0*GPL1(1)*u*pi*ci
     .        + 1.0d0/12.0d0*GPL1(1)*u**2*pi*ci
     .        + 5.0d0/24.0d0*GPL1(2)
     .        + 1.0d0/12.0d0*GPL1(2)*u**(-1)
     .        + 1.0d0/12.0d0*GPL1(2)*u
     .        - 1.0d0/6.0d0*GPL1(2)*u*pi*ci
     .        - 1.0d0/12.0d0*GPL1(2)*u**2*pi*ci
     .        + 1.0d0/4.0d0*GPL1(3)*omumv**(-4)*pi*ci
     .        - 1.0d0/2.0d0*GPL1(3)*omumv**(-3)*pi*ci
     .        + 1.0d0/12.0d0*GPL1(3)*omumv**(-2)*pi*ci
     .        + 1.0d0/6.0d0*GPL1(3)*omumv**(-1)*pi*ci
     .        - 1.0d0/2.0d0*GPL1(3)*v*omumv**(-4)*pi*ci
     .        + 5.0d0/6.0d0*GPL1(3)*v*omumv**(-3)*pi*ci
         col4 = col4 - 1.0d0/6.0d0*GPL1(3)*v*omumv**(-2)*pi*ci
     .        + 1.0d0/4.0d0*GPL1(3)*v**2*omumv**(-4)*pi*ci
     .        - 1.0d0/3.0d0*GPL1(3)*v**2*omumv**(-3)*pi*ci
     .        + 1.0d0/6.0d0*GPL1(3)*u*pi*ci
     .        + 1.0d0/12.0d0*GPL1(3)*u**2*pi*ci
     .        - 1.0d0/6.0d0*GPL2(0,2)*u
     .        - 1.0d0/12.0d0*GPL2(0,2)*u**2
     .        + 1.0d0/12.0d0*GPL2(1,2)*u**(-2)
     .        + 1.0d0/6.0d0*GPL2(1,2)*u**(-1)
     .        + 1.0d0/6.0d0*GPL2(1,2)*u
     .        + 1.0d0/12.0d0*GPL2(1,2)*u**2
     .        - 1.0d0/6.0d0*GPL2(2,0)*u
     .        - 1.0d0/12.0d0*GPL2(2,0)*u**2
     .        + 1.0d0/4.0d0*GPL2(3,0)*omumv**(-4)
     .        - 1.0d0/2.0d0*GPL2(3,0)*omumv**(-3)
         col4 = col4 + 1.0d0/12.0d0*GPL2(3,0)*omumv**(-2)
     .        + 1.0d0/6.0d0*GPL2(3,0)*omumv**(-1)
     .        - 1.0d0/2.0d0*GPL2(3,0)*v*omumv**(-4)
     .        + 5.0d0/6.0d0*GPL2(3,0)*v*omumv**(-3)
     .        - 1.0d0/6.0d0*GPL2(3,0)*v*omumv**(-2)
     .        + 1.0d0/4.0d0*GPL2(3,0)*v**2*omumv**(-4)
     .        - 1.0d0/3.0d0*GPL2(3,0)*v**2*omumv**(-3)
     .        + 1.0d0/6.0d0*GPL2(3,0)*u
     .        + 1.0d0/12.0d0*GPL2(3,0)*u**2

         col5 = - 1583.0d0/432.0d0
     .        + 11.0d0/12.0d0*u**(-1)*pi*ci
     .        + 13.0d0/12.0d0*omv**(-1)
     .        + 11.0d0/24.0d0*omumv**(-4)*pi**2
     .        + 11.0d0/4.0d0*omumv**(-3)*pi*ci
     .        - 11.0d0/12.0d0*omumv**(-3)*pi**2
     .        - 1.0d0/36.0d0*omumv**(-3)*pi**3*ci
     .        + 187.0d0/54.0d0*omumv**(-2)
     .        - 71.0d0/18.0d0*omumv**(-2)*pi*ci
     .        + 3.0d0/4.0d0*omumv**(-2)*pi**2
     .        + 1.0d0/24.0d0*omumv**(-2)*pi**3*ci
     .        - 187.0d0/54.0d0*omumv**(-1)
     .        + 131.0d0/72.0d0*omumv**(-1)*pi*ci
     .        - 7.0d0/24.0d0*omumv**(-1)*pi**2
     .        - 1.0d0/24.0d0*omumv**(-1)*pi**3*ci
         col5 = col5 - 119.0d0/54.0d0*pi*ci
     .        + 55.0d0/864.0d0*pi**2
     .        - 5.0d0/48.0d0*pi**3*ci
     .        + 1.0d0/3.0d0*zeta3*omumv**(-3)
     .        - 1.0d0/2.0d0*zeta3*omumv**(-2)
     .        + 1.0d0/2.0d0*zeta3*omumv**(-1)
     .        - 29.0d0/24.0d0*zeta3
     .        - 11.0d0/12.0d0*v*omumv**(-4)*pi**2
     .        - 11.0d0/2.0d0*v*omumv**(-3)*pi*ci
     .        + 31.0d0/36.0d0*v*omumv**(-3)*pi**2
     .        + 1.0d0/12.0d0*v*omumv**(-3)*pi**3*ci
     .        - 187.0d0/54.0d0*v*omumv**(-2)
     .        + 173.0d0/72.0d0*v*omumv**(-2)*pi*ci
     .        - 1.0d0/8.0d0*v*omumv**(-2)*pi**2
     .        - 1.0d0/12.0d0*v*omumv**(-2)*pi**3*ci
         col5 = col5 - 5.0d0/8.0d0*v*omumv**(-1)
     .        - 1.0d0/36.0d0*v*omumv**(-1)*pi**2
     .        + 1.0d0/24.0d0*v*omumv**(-1)*pi**3*ci
     .        - 13.0d0/12.0d0*v*upv**(-1)
     .        - 1.0d0/6.0d0*v*upv**(-1)*pi*ci
     .        + 11.0d0/12.0d0*v*pi*ci
     .        + 11.0d0/24.0d0*v**2*omumv**(-4)*pi**2
     .        + 11.0d0/4.0d0*v**2*omumv**(-3)*pi*ci
     .        + 1.0d0/18.0d0*v**2*omumv**(-3)*pi**2
     .        - 1.0d0/12.0d0*v**2*omumv**(-3)*pi**3*ci
     .        + 37.0d0/24.0d0*v**2*omumv**(-2)*pi*ci
     .        - 1.0d0/36.0d0*v**2*omumv**(-2)*pi**2
     .        + 1.0d0/24.0d0*v**2*omumv**(-2)*pi**3*ci
     .        + 11.0d0/12.0d0*v**2*omumv**(-1)*pi*ci
     .        - 13.0d0/12.0d0*v**2*upv**(-2)*pi*ci
         col5 = col5 + 1.0d0/36.0d0*v**3*omumv**(-3)*pi**3*ci
     .        - 7.0d0/9.0d0*u*pi**2
     .        + 11.0d0/18.0d0*u**2*pi**2
     .        - 97.0d0/36.0d0*HPL1(0)
     .        + 11.0d0/12.0d0*HPL1(0)*u**(-1)*omv**(-1)
     .        - 11.0d0/12.0d0*HPL1(0)*u**(-1)
     .        + 13.0d0/12.0d0*HPL1(0)*omv**(-2)
     .        + 11.0d0/12.0d0*HPL1(0)*omv**(-1)*omumv**(-1)
     .        - 7.0d0/3.0d0*HPL1(0)*omv**(-1)
     .        + 11.0d0/4.0d0*HPL1(0)*omumv**(-4)*pi*ci
     .        - 11.0d0/2.0d0*HPL1(0)*omumv**(-3)*pi*ci
     .        + 1.0d0/18.0d0*HPL1(0)*omumv**(-3)*pi**2
     .        + 65.0d0/72.0d0*HPL1(0)*omumv**(-2)
     .        + 47.0d0/12.0d0*HPL1(0)*omumv**(-2)*pi*ci
     .        - 1.0d0/12.0d0*HPL1(0)*omumv**(-2)*pi**2
         col5 = col5 - 131.0d0/72.0d0*HPL1(0)*omumv**(-1)
     .        - 7.0d0/6.0d0*HPL1(0)*omumv**(-1)*pi*ci
     .        + 1.0d0/12.0d0*HPL1(0)*omumv**(-1)*pi**2
     .        - 25.0d0/12.0d0*HPL1(0)*pi*ci
     .        - 5.0d0/144.0d0*HPL1(0)*pi**2
     .        - 11.0d0/2.0d0*HPL1(0)*v*omumv**(-4)*pi*ci
     .        + 11.0d0/4.0d0*HPL1(0)*v*omumv**(-3)
     .        + 31.0d0/6.0d0*HPL1(0)*v*omumv**(-3)*pi*ci
     .        + 1.0d0/12.0d0*HPL1(0)*v*omumv**(-3)*pi**2
     .        - 263.0d0/72.0d0*HPL1(0)*v*omumv**(-2)
     .        - 5.0d0/6.0d0*HPL1(0)*v*omumv**(-2)*pi*ci
     .        - 1.0d0/12.0d0*HPL1(0)*v*omumv**(-2)*pi**2
     .        + 1.0d0/24.0d0*HPL1(0)*v*omumv**(-1)*pi**2
     .        + 1.0d0/6.0d0*HPL1(0)*v*upv**(-1)
     .        - 11.0d0/12.0d0*HPL1(0)*v
         col5 = col5 + 11.0d0/4.0d0*HPL1(0)*v**2*omumv**(-4)*pi*ci
     .        - 11.0d0/4.0d0*HPL1(0)*v**2*omumv**(-3)
     .        + 1.0d0/3.0d0*HPL1(0)*v**2*omumv**(-3)*pi*ci
     .        - 1.0d0/12.0d0*HPL1(0)*v**2*omumv**(-3)*pi**2
     .        - 37.0d0/24.0d0*HPL1(0)*v**2*omumv**(-2)
     .        + 1.0d0/24.0d0*HPL1(0)*v**2*omumv**(-2)*pi**2
     .        - 11.0d0/12.0d0*HPL1(0)*v**2*omumv**(-1)
     .        + 13.0d0/12.0d0*HPL1(0)*v**2*upv**(-2)
     .        + 1.0d0/36.0d0*HPL1(0)*v**3*omumv**(-3)*pi**2
     .        - 7.0d0/6.0d0*HPL1(0)*u*pi*ci
     .        + 11.0d0/12.0d0*HPL1(0)*u**2*pi*ci
     .        + 35.0d0/36.0d0*HPL1(0)*GPL1(0)
     .        + 11.0d0/4.0d0*HPL1(0)*GPL1(0)*omumv**(-4)
     .        - 11.0d0/2.0d0*HPL1(0)*GPL1(0)*omumv**(-3)
     .        + 47.0d0/12.0d0*HPL1(0)*GPL1(0)*omumv**(-2)
         col5 = col5 - 7.0d0/6.0d0*HPL1(0)*GPL1(0)*omumv**(-1)
     .        + 5.0d0/12.0d0*HPL1(0)*GPL1(0)*pi*ci
     .        - 11.0d0/2.0d0*HPL1(0)*GPL1(0)*v*omumv**(-4)
     .        + 31.0d0/6.0d0*HPL1(0)*GPL1(0)*v*omumv**(-3)
     .        - 5.0d0/6.0d0*HPL1(0)*GPL1(0)*v*omumv**(-2)
     .        + 11.0d0/4.0d0*HPL1(0)*GPL1(0)*v**2*omumv**(-4)
     .        + 1.0d0/3.0d0*HPL1(0)*GPL1(0)*v**2*omumv**(-3)
     .        - 7.0d0/6.0d0*HPL1(0)*GPL1(0)*u
     .        + 11.0d0/12.0d0*HPL1(0)*GPL1(0)*u**2
     .        + 5.0d0/9.0d0*HPL1(0)*GPL1(1)
     .        - 11.0d0/12.0d0*HPL1(0)*GPL1(1)*u**(-2)
     .        + 7.0d0/6.0d0*HPL1(0)*GPL1(1)*u**(-1)
     .        - 1.0d0/6.0d0*HPL1(0)*GPL1(1)*omumv**(-2)
     .        + 1.0d0/6.0d0*HPL1(0)*GPL1(1)*omumv**(-1)
     .        - 1.0d0/6.0d0*HPL1(0)*GPL1(1)*pi*ci
         col5 = col5 + 1.0d0/6.0d0*HPL1(0)*GPL1(1)*v*omumv**(-2)
     .        + 7.0d0/6.0d0*HPL1(0)*GPL1(1)*u
     .        - 11.0d0/12.0d0*HPL1(0)*GPL1(1)*u**2
     .        + 35.0d0/36.0d0*HPL1(0)*GPL1(2)
     .        + 11.0d0/12.0d0*HPL1(0)*GPL1(2)*u**(-2)
     .        - 7.0d0/6.0d0*HPL1(0)*GPL1(2)*u**(-1)
     .        + 1.0d0/6.0d0*HPL1(0)*GPL1(2)*omumv**(-2)
     .        - 1.0d0/6.0d0*HPL1(0)*GPL1(2)*omumv**(-1)
     .        - 1.0d0/12.0d0*HPL1(0)*GPL1(2)*pi*ci
     .        - 1.0d0/6.0d0*HPL1(0)*GPL1(2)*v*omumv**(-2)
     .        - 7.0d0/6.0d0*HPL1(0)*GPL1(2)*u
     .        + 11.0d0/12.0d0*HPL1(0)*GPL1(2)*u**2
     .        + 5.0d0/9.0d0*HPL1(0)*GPL1(3)
     .        - 11.0d0/4.0d0*HPL1(0)*GPL1(3)*omumv**(-4)
     .        + 11.0d0/2.0d0*HPL1(0)*GPL1(3)*omumv**(-3)
         col5 = col5 - 1.0d0/6.0d0*HPL1(0)*GPL1(3)*omumv**(-3)*pi*ci
     .        - 47.0d0/12.0d0*HPL1(0)*GPL1(3)*omumv**(-2)
     .        + 1.0d0/4.0d0*HPL1(0)*GPL1(3)*omumv**(-2)*pi*ci
     .        + 7.0d0/6.0d0*HPL1(0)*GPL1(3)*omumv**(-1)
     .        - 1.0d0/4.0d0*HPL1(0)*GPL1(3)*omumv**(-1)*pi*ci
     .        - 1.0d0/6.0d0*HPL1(0)*GPL1(3)*pi*ci
     .        + 11.0d0/2.0d0*HPL1(0)*GPL1(3)*v*omumv**(-4)
     .        - 31.0d0/6.0d0*HPL1(0)*GPL1(3)*v*omumv**(-3)
     .        + 7.0d0/6.0d0*HPL1(0)*GPL1(3)*v*omumv**(-2)
     .        - 1.0d0/6.0d0*HPL1(0)*GPL1(3)*v*omumv**(-1)
     .        - 11.0d0/4.0d0*HPL1(0)*GPL1(3)*v**2*omumv**(-4)
     .        - 1.0d0/3.0d0*HPL1(0)*GPL1(3)*v**2*omumv**(-3)
     .        - 1.0d0/6.0d0*HPL1(0)*GPL1(3)*v**2*omumv**(-2)
     .        + 7.0d0/6.0d0*HPL1(0)*GPL1(3)*u
     .        - 11.0d0/12.0d0*HPL1(0)*GPL1(3)*u**2
         col5 = col5 + 2.0d0/3.0d0*HPL1(0)*GPL2(0,0)
     .        + 7.0d0/12.0d0*HPL1(0)*GPL2(0,2)
     .        - 2.0d0/3.0d0*HPL1(0)*GPL2(0,3)
     .        - 1.0d0/6.0d0*HPL1(0)*GPL2(0,3)*omumv**(-3)
     .        + 1.0d0/4.0d0*HPL1(0)*GPL2(0,3)*omumv**(-2)
     .        - 1.0d0/4.0d0*HPL1(0)*GPL2(0,3)*omumv**(-1)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(0,3)*v*omumv**(-3)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(0,3)*v*omumv**(-2)
     .        + 1.0d0/4.0d0*HPL1(0)*GPL2(0,3)*v*omumv**(-1)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(0,3)*v**2*omumv**(-3)
     .        + 1.0d0/4.0d0*HPL1(0)*GPL2(0,3)*v**2*omumv**(-2)
     .        + 1.0d0/6.0d0*HPL1(0)*GPL2(0,3)*v**3*omumv**(-3)
     .        - 1.0d0/3.0d0*HPL1(0)*GPL2(1,1)
     .        - 2.0d0/3.0d0*HPL1(0)*GPL2(1,2)
     .        + 7.0d0/12.0d0*HPL1(0)*GPL2(2,0)
         col5 = col5 - 1.0d0/2.0d0*HPL1(0)*GPL2(2,1)
     .        + 2.0d0/3.0d0*HPL1(0)*GPL2(2,2)
     .        - 1.0d0/6.0d0*HPL1(0)*GPL2(2,3)
     .        - 5.0d0/6.0d0*HPL1(0)*GPL2(3,0)
     .        - 1.0d0/6.0d0*HPL1(0)*GPL2(3,0)*omumv**(-3)
     .        + 1.0d0/4.0d0*HPL1(0)*GPL2(3,0)*omumv**(-2)
     .        - 1.0d0/4.0d0*HPL1(0)*GPL2(3,0)*omumv**(-1)
     .        - 1.0d0/6.0d0*HPL1(0)*GPL2(3,2)
     .        + 1.0d0/3.0d0*HPL1(0)*GPL2(3,3)*omumv**(-3)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(3,3)*omumv**(-2)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(3,3)*omumv**(-1)
     .        - 35.0d0/216.0d0*HPL1(1)
     .        - 11.0d0/12.0d0*HPL1(1)*u**(-1)
     .        - 1.0d0/18.0d0*HPL1(1)*omumv**(-3)*pi**2
     .        + 13.0d0/24.0d0*HPL1(1)*omumv**(-2)
         col5 = col5 + 1.0d0/12.0d0*HPL1(1)*omumv**(-2)*pi*ci
     .        + 1.0d0/12.0d0*HPL1(1)*omumv**(-2)*pi**2
     .        - 13.0d0/24.0d0*HPL1(1)*omumv**(-1)
     .        - 1.0d0/12.0d0*HPL1(1)*omumv**(-1)*pi*ci
     .        - 1.0d0/12.0d0*HPL1(1)*omumv**(-1)*pi**2
     .        - 7.0d0/12.0d0*HPL1(1)*pi*ci
     .        + 13.0d0/144.0d0*HPL1(1)*pi**2
     .        + 1.0d0/6.0d0*HPL1(1)*v*omumv**(-3)*pi**2
     .        - 13.0d0/24.0d0*HPL1(1)*v*omumv**(-2)
     .        - 1.0d0/12.0d0*HPL1(1)*v*omumv**(-2)*pi*ci
     .        - 1.0d0/6.0d0*HPL1(1)*v*omumv**(-2)*pi**2
     .        + 1.0d0/12.0d0*HPL1(1)*v*omumv**(-1)*pi**2
     .        - 1.0d0/6.0d0*HPL1(1)*v**2*omumv**(-3)*pi**2
     .        + 1.0d0/12.0d0*HPL1(1)*v**2*omumv**(-2)*pi**2
     .        + 1.0d0/18.0d0*HPL1(1)*v**3*omumv**(-3)*pi**2
         col5 = col5 - 11.0d0/12.0d0*HPL1(1)*u
     .        - 7.0d0/6.0d0*HPL1(1)*u*pi*ci
     .        + 11.0d0/12.0d0*HPL1(1)*u**2*pi*ci
     .        + 1.0d0/36.0d0*HPL1(1)*GPL1(0)
     .        + 1.0d0/12.0d0*HPL1(1)*GPL1(0)*omumv**(-2)
     .        - 1.0d0/12.0d0*HPL1(1)*GPL1(0)*omumv**(-1)
     .        + 1.0d0/4.0d0*HPL1(1)*GPL1(0)*pi*ci
     .        - 1.0d0/12.0d0*HPL1(1)*GPL1(0)*v*omumv**(-2)
     .        - 7.0d0/6.0d0*HPL1(1)*GPL1(0)*u
     .        + 11.0d0/12.0d0*HPL1(1)*GPL1(0)*u**2
     .        + 5.0d0/9.0d0*HPL1(1)*GPL1(1)
     .        - 11.0d0/12.0d0*HPL1(1)*GPL1(1)*u**(-2)
     .        + 7.0d0/6.0d0*HPL1(1)*GPL1(1)*u**(-1)
     .        - 1.0d0/6.0d0*HPL1(1)*GPL1(1)*omumv**(-2)
     .        + 1.0d0/6.0d0*HPL1(1)*GPL1(1)*omumv**(-1)
         col5 = col5 + 1.0d0/6.0d0*HPL1(1)*GPL1(1)*pi*ci
     .        + 1.0d0/6.0d0*HPL1(1)*GPL1(1)*v*omumv**(-2)
     .        + 7.0d0/6.0d0*HPL1(1)*GPL1(1)*u
     .        - 11.0d0/12.0d0*HPL1(1)*GPL1(1)*u**2
     .        + 11.0d0/12.0d0*HPL1(1)*GPL1(2)
     .        + 1.0d0/3.0d0*HPL1(1)*GPL1(2)*pi*ci
     .        - 1.0d0/6.0d0*HPL1(1)*GPL1(3)*pi*ci
     .        + 1.0d0/3.0d0*HPL1(1)*GPL2(0,0)
     .        + 1.0d0/3.0d0*HPL1(1)*GPL2(0,2)
     .        - 1.0d0/3.0d0*HPL1(1)*GPL2(1,1)
     .        - 2.0d0/3.0d0*HPL1(1)*GPL2(1,2)
     .        + 1.0d0/3.0d0*HPL1(1)*GPL2(2,0)
     .        - 1.0d0/2.0d0*HPL1(1)*GPL2(2,1)
     .        - 1.0d0/6.0d0*HPL1(1)*GPL2(3,0)
     .        - 145.0d0/36.0d0*HPL2(0,0)
         col5 = col5 - 11.0d0/4.0d0*HPL2(0,0)*omumv**(-4)
     .        + 11.0d0/2.0d0*HPL2(0,0)*omumv**(-3)
     .        - 1.0d0/6.0d0*HPL2(0,0)*omumv**(-3)*pi*ci
     .        - 47.0d0/12.0d0*HPL2(0,0)*omumv**(-2)
     .        + 1.0d0/4.0d0*HPL2(0,0)*omumv**(-2)*pi*ci
     .        + 7.0d0/6.0d0*HPL2(0,0)*omumv**(-1)
     .        - 1.0d0/4.0d0*HPL2(0,0)*omumv**(-1)*pi*ci
     .        - HPL2(0,0)*pi*ci
     .        + 11.0d0/2.0d0*HPL2(0,0)*v*omumv**(-4)
     .        - 31.0d0/6.0d0*HPL2(0,0)*v*omumv**(-3)
     .        + 1.0d0/2.0d0*HPL2(0,0)*v*omumv**(-3)*pi*ci
     .        + 7.0d0/6.0d0*HPL2(0,0)*v*omumv**(-2)
     .        - 1.0d0/2.0d0*HPL2(0,0)*v*omumv**(-2)*pi*ci
     .        - 1.0d0/6.0d0*HPL2(0,0)*v*omumv**(-1)
     .        + 1.0d0/4.0d0*HPL2(0,0)*v*omumv**(-1)*pi*ci
         col5 = col5 - 11.0d0/4.0d0*HPL2(0,0)*v**2*omumv**(-4)
     .        - 1.0d0/3.0d0*HPL2(0,0)*v**2*omumv**(-3)
     .        - 1.0d0/2.0d0*HPL2(0,0)*v**2*omumv**(-3)*pi*ci
     .        - 1.0d0/6.0d0*HPL2(0,0)*v**2*omumv**(-2)
     .        + 1.0d0/4.0d0*HPL2(0,0)*v**2*omumv**(-2)*pi*ci
     .        + 1.0d0/6.0d0*HPL2(0,0)*v**3*omumv**(-3)*pi*ci
     .        + 7.0d0/6.0d0*HPL2(0,0)*u
     .        - 11.0d0/12.0d0*HPL2(0,0)*u**2
     .        - 5.0d0/3.0d0*HPL2(0,0)*GPL1(0)
     .        - 1.0d0/6.0d0*HPL2(0,0)*GPL1(0)*omumv**(-3)
     .        + 1.0d0/4.0d0*HPL2(0,0)*GPL1(0)*omumv**(-2)
     .        - 1.0d0/4.0d0*HPL2(0,0)*GPL1(0)*omumv**(-1)
     .        + 1.0d0/2.0d0*HPL2(0,0)*GPL1(0)*v*omumv**(-3)
     .        - 1.0d0/2.0d0*HPL2(0,0)*GPL1(0)*v*omumv**(-2)
     .        + 1.0d0/4.0d0*HPL2(0,0)*GPL1(0)*v*omumv**(-1)
         col5 = col5 - 1.0d0/2.0d0*HPL2(0,0)*GPL1(0)*v**2*omumv**(-3)
     .        + 1.0d0/4.0d0*HPL2(0,0)*GPL1(0)*v**2*omumv**(-2)
     .        + 1.0d0/6.0d0*HPL2(0,0)*GPL1(0)*v**3*omumv**(-3)
     .        + HPL2(0,0)*GPL1(1)
     .        - 7.0d0/6.0d0*HPL2(0,0)*GPL1(2)
     .        + HPL2(0,0)*GPL1(3)
     .        + 1.0d0/3.0d0*HPL2(0,0)*GPL1(3)*omumv**(-3)
     .        - 1.0d0/2.0d0*HPL2(0,0)*GPL1(3)*omumv**(-2)
     .        + 1.0d0/2.0d0*HPL2(0,0)*GPL1(3)*omumv**(-1)
     .        - 55.0d0/36.0d0*HPL2(0,1)
     .        - 7.0d0/12.0d0*HPL2(0,1)*pi*ci
     .        - 7.0d0/12.0d0*HPL2(0,1)*GPL1(0)
     .        + HPL2(0,1)*GPL1(1)
     .        - 1.0d0/6.0d0*HPL2(0,1)*GPL1(2)
     .        + 1.0d0/6.0d0*HPL2(0,1)*GPL1(3)
         col5 = col5 - 55.0d0/36.0d0*HPL2(1,0)
     .        + 11.0d0/4.0d0*HPL2(1,0)*omumv**(-4)
     .        - 11.0d0/2.0d0*HPL2(1,0)*omumv**(-3)
     .        - 1.0d0/6.0d0*HPL2(1,0)*omumv**(-3)*pi*ci
     .        + 15.0d0/4.0d0*HPL2(1,0)*omumv**(-2)
     .        + 1.0d0/4.0d0*HPL2(1,0)*omumv**(-2)*pi*ci
     .        - HPL2(1,0)*omumv**(-1)
     .        - 1.0d0/4.0d0*HPL2(1,0)*omumv**(-1)*pi*ci
     .        - 5.0d0/12.0d0*HPL2(1,0)*pi*ci
     .        - 11.0d0/2.0d0*HPL2(1,0)*v*omumv**(-4)
     .        + 31.0d0/6.0d0*HPL2(1,0)*v*omumv**(-3)
     .        + 1.0d0/2.0d0*HPL2(1,0)*v*omumv**(-3)*pi*ci
     .        - 1.0d0/3.0d0*HPL2(1,0)*v*omumv**(-2)
     .        - 1.0d0/2.0d0*HPL2(1,0)*v*omumv**(-2)*pi*ci
     .        - 1.0d0/6.0d0*HPL2(1,0)*v*omumv**(-1)
         col5 = col5 + 1.0d0/4.0d0*HPL2(1,0)*v*omumv**(-1)*pi*ci
     .        + 11.0d0/4.0d0*HPL2(1,0)*v**2*omumv**(-4)
     .        + 1.0d0/3.0d0*HPL2(1,0)*v**2*omumv**(-3)
     .        - 1.0d0/2.0d0*HPL2(1,0)*v**2*omumv**(-3)*pi*ci
     .        - 1.0d0/6.0d0*HPL2(1,0)*v**2*omumv**(-2)
     .        + 1.0d0/4.0d0*HPL2(1,0)*v**2*omumv**(-2)*pi*ci
     .        + 1.0d0/6.0d0*HPL2(1,0)*v**3*omumv**(-3)*pi*ci
     .        + 7.0d0/6.0d0*HPL2(1,0)*u
     .        - 11.0d0/12.0d0*HPL2(1,0)*u**2
     .        - 5.0d0/12.0d0*HPL2(1,0)*GPL1(0)
     .        - 1.0d0/6.0d0*HPL2(1,0)*GPL1(0)*omumv**(-3)
     .        + 1.0d0/4.0d0*HPL2(1,0)*GPL1(0)*omumv**(-2)
     .        - 1.0d0/4.0d0*HPL2(1,0)*GPL1(0)*omumv**(-1)
     .        + 1.0d0/2.0d0*HPL2(1,0)*GPL1(0)*v*omumv**(-3)
     .        - 1.0d0/2.0d0*HPL2(1,0)*GPL1(0)*v*omumv**(-2)
         col5 = col5 + 1.0d0/4.0d0*HPL2(1,0)*GPL1(0)*v*omumv**(-1)
     .        - 1.0d0/2.0d0*HPL2(1,0)*GPL1(0)*v**2*omumv**(-3)
     .        + 1.0d0/4.0d0*HPL2(1,0)*GPL1(0)*v**2*omumv**(-2)
     .        + 1.0d0/6.0d0*HPL2(1,0)*GPL1(0)*v**3*omumv**(-3)
     .        + 2.0d0/3.0d0*HPL2(1,0)*GPL1(1)
     .        - 1.0d0/2.0d0*HPL2(1,0)*GPL1(2)
     .        + 1.0d0/6.0d0*HPL2(1,0)*GPL1(3)
     .        - 11.0d0/12.0d0*HPL2(1,1)
     .        - 1.0d0/3.0d0*HPL2(1,1)*pi*ci
     .        - 1.0d0/3.0d0*HPL2(1,1)*GPL1(0)
     .        + 2.0d0/3.0d0*HPL2(1,1)*GPL1(1)
     .        + 11.0d0/6.0d0*HPL3(0,0,0)
     .        + 1.0d0/3.0d0*HPL3(0,0,0)*omumv**(-3)
     .        - 1.0d0/2.0d0*HPL3(0,0,0)*omumv**(-2)
     .        + 1.0d0/2.0d0*HPL3(0,0,0)*omumv**(-1)
         col5 = col5 - 1.0d0/2.0d0*HPL3(0,0,0)*v*omumv**(-3)
     .        + 1.0d0/2.0d0*HPL3(0,0,0)*v*omumv**(-2)
     .        - 1.0d0/4.0d0*HPL3(0,0,0)*v*omumv**(-1)
     .        + 1.0d0/2.0d0*HPL3(0,0,0)*v**2*omumv**(-3)
     .        - 1.0d0/4.0d0*HPL3(0,0,0)*v**2*omumv**(-2)
     .        - 1.0d0/6.0d0*HPL3(0,0,0)*v**3*omumv**(-3)
     .        + 1.0d0/6.0d0*HPL3(0,0,1)
     .        + 1.0d0/6.0d0*HPL3(0,1,0)
     .        + 1.0d0/2.0d0*HPL3(0,1,0)*v*omumv**(-3)
     .        - 1.0d0/2.0d0*HPL3(0,1,0)*v*omumv**(-2)
     .        + 1.0d0/4.0d0*HPL3(0,1,0)*v*omumv**(-1)
     .        - 1.0d0/2.0d0*HPL3(0,1,0)*v**2*omumv**(-3)
     .        + 1.0d0/4.0d0*HPL3(0,1,0)*v**2*omumv**(-2)
     .        + 1.0d0/6.0d0*HPL3(0,1,0)*v**3*omumv**(-3)
     .        + 1.0d0/6.0d0*HPL3(1,0,0)
         col5 = col5 + 2.0d0/3.0d0*HPL3(1,1,0)
     .        - 1.0d0/3.0d0*HPL3(1,1,0)*omumv**(-3)
     .        + 1.0d0/2.0d0*HPL3(1,1,0)*omumv**(-2)
     .        - 1.0d0/2.0d0*HPL3(1,1,0)*omumv**(-1)
     .        + HPL3(1,1,0)*v*omumv**(-3)
     .        - HPL3(1,1,0)*v*omumv**(-2)
     .        + 1.0d0/2.0d0*HPL3(1,1,0)*v*omumv**(-1)
     .        - HPL3(1,1,0)*v**2*omumv**(-3)
     .        + 1.0d0/2.0d0*HPL3(1,1,0)*v**2*omumv**(-2)
     .        + 1.0d0/3.0d0*HPL3(1,1,0)*v**3*omumv**(-3)
     .        + 341.0d0/216.0d0*GPL1(0)
     .        + 11.0d0/4.0d0*GPL1(0)*omumv**(-3)
     .        - 1.0d0/36.0d0*GPL1(0)*omumv**(-3)*pi**2
     .        - 155.0d0/36.0d0*GPL1(0)*omumv**(-2)
     .        - 1.0d0/12.0d0*GPL1(0)*omumv**(-2)*pi*ci
         col5 = col5 + 1.0d0/24.0d0*GPL1(0)*omumv**(-2)*pi**2
     .        + 157.0d0/72.0d0*GPL1(0)*omumv**(-1)
     .        + 1.0d0/12.0d0*GPL1(0)*omumv**(-1)*pi*ci
     .        - 1.0d0/24.0d0*GPL1(0)*omumv**(-1)*pi**2
     .        + 7.0d0/12.0d0*GPL1(0)*pi*ci
     .        + 11.0d0/144.0d0*GPL1(0)*pi**2
     .        - 11.0d0/2.0d0*GPL1(0)*v*omumv**(-3)
     .        + 1.0d0/12.0d0*GPL1(0)*v*omumv**(-3)*pi**2
     .        + 199.0d0/72.0d0*GPL1(0)*v*omumv**(-2)
     .        + 1.0d0/12.0d0*GPL1(0)*v*omumv**(-2)*pi*ci
     .        - 1.0d0/12.0d0*GPL1(0)*v*omumv**(-2)*pi**2
     .        + 1.0d0/24.0d0*GPL1(0)*v*omumv**(-1)*pi**2
     .        - 1.0d0/6.0d0*GPL1(0)*v*upv**(-1)
     .        + 11.0d0/12.0d0*GPL1(0)*v
     .        + 11.0d0/4.0d0*GPL1(0)*v**2*omumv**(-3)
         col5 = col5 - 1.0d0/12.0d0*GPL1(0)*v**2*omumv**(-3)*pi**2
     .        + 37.0d0/24.0d0*GPL1(0)*v**2*omumv**(-2)
     .        + 1.0d0/24.0d0*GPL1(0)*v**2*omumv**(-2)*pi**2
     .        + 11.0d0/12.0d0*GPL1(0)*v**2*omumv**(-1)
     .        - 13.0d0/12.0d0*GPL1(0)*v**2*upv**(-2)
     .        + 1.0d0/36.0d0*GPL1(0)*v**3*omumv**(-3)*pi**2
     .        - 11.0d0/12.0d0*GPL1(0)*u
     .        + 7.0d0/6.0d0*GPL1(0)*u*pi*ci
     .        - 11.0d0/12.0d0*GPL1(0)*u**2*pi*ci
     .        + 11.0d0/12.0d0*GPL1(1)*u**(-2)*pi*ci
     .        - 7.0d0/6.0d0*GPL1(1)*u**(-1)*pi*ci
     .        + 1.0d0/6.0d0*GPL1(1)*omumv**(-2)*pi*ci
     .        - 1.0d0/6.0d0*GPL1(1)*omumv**(-1)*pi*ci
     .        - 5.0d0/9.0d0*GPL1(1)*pi*ci
     .        + 7.0d0/18.0d0*GPL1(1)*pi**2
         col5 = col5 - 1.0d0/6.0d0*GPL1(1)*v*omumv**(-2)*pi*ci
     .        - 7.0d0/6.0d0*GPL1(1)*u*pi*ci
     .        + 11.0d0/12.0d0*GPL1(1)*u**2*pi*ci
     .        + 35.0d0/216.0d0*GPL1(2)
     .        + 11.0d0/12.0d0*GPL1(2)*u**(-1)
     .        - 13.0d0/24.0d0*GPL1(2)*omumv**(-2)
     .        - 1.0d0/12.0d0*GPL1(2)*omumv**(-2)*pi*ci
     .        + 13.0d0/24.0d0*GPL1(2)*omumv**(-1)
     .        + 1.0d0/12.0d0*GPL1(2)*omumv**(-1)*pi*ci
     .        + 7.0d0/12.0d0*GPL1(2)*pi*ci
     .        - 5.0d0/144.0d0*GPL1(2)*pi**2
     .        + 13.0d0/24.0d0*GPL1(2)*v*omumv**(-2)
     .        + 1.0d0/12.0d0*GPL1(2)*v*omumv**(-2)*pi*ci
     .        + 11.0d0/12.0d0*GPL1(2)*u
     .        + 7.0d0/6.0d0*GPL1(2)*u*pi*ci
         col5 = col5 - 11.0d0/12.0d0*GPL1(2)*u**2*pi*ci
     .        + 11.0d0/4.0d0*GPL1(3)*omumv**(-4)*pi*ci
     .        - 11.0d0/2.0d0*GPL1(3)*omumv**(-3)*pi*ci
     .        + 1.0d0/18.0d0*GPL1(3)*omumv**(-3)*pi**2
     .        + 47.0d0/12.0d0*GPL1(3)*omumv**(-2)*pi*ci
     .        - 1.0d0/12.0d0*GPL1(3)*omumv**(-2)*pi**2
     .        - 7.0d0/6.0d0*GPL1(3)*omumv**(-1)*pi*ci
     .        + 1.0d0/12.0d0*GPL1(3)*omumv**(-1)*pi**2
     .        - 5.0d0/9.0d0*GPL1(3)*pi*ci
     .        + 1.0d0/3.0d0*GPL1(3)*pi**2
     .        - 11.0d0/2.0d0*GPL1(3)*v*omumv**(-4)*pi*ci
     .        + 31.0d0/6.0d0*GPL1(3)*v*omumv**(-3)*pi*ci
     .        - 7.0d0/6.0d0*GPL1(3)*v*omumv**(-2)*pi*ci
     .        + 1.0d0/6.0d0*GPL1(3)*v*omumv**(-1)*pi*ci
     .        + 11.0d0/4.0d0*GPL1(3)*v**2*omumv**(-4)*pi*ci
         col5 = col5 + 1.0d0/3.0d0*GPL1(3)*v**2*omumv**(-3)*pi*ci
     .        + 1.0d0/6.0d0*GPL1(3)*v**2*omumv**(-2)*pi*ci
     .        - 7.0d0/6.0d0*GPL1(3)*u*pi*ci
     .        + 11.0d0/12.0d0*GPL1(3)*u**2*pi*ci
     .        - 11.0d0/12.0d0*GPL2(0,0)
     .        - 1.0d0/3.0d0*GPL2(0,0)*pi*ci
     .        - 1.0d0/36.0d0*GPL2(0,2)
     .        - 1.0d0/12.0d0*GPL2(0,2)*omumv**(-2)
     .        + 1.0d0/12.0d0*GPL2(0,2)*omumv**(-1)
     .        - 1.0d0/4.0d0*GPL2(0,2)*pi*ci
     .        + 1.0d0/12.0d0*GPL2(0,2)*v*omumv**(-2)
     .        + 7.0d0/6.0d0*GPL2(0,2)*u
     .        - 11.0d0/12.0d0*GPL2(0,2)*u**2
     .        + 1.0d0/6.0d0*GPL2(0,3)*omumv**(-3)*pi*ci
     .        - 1.0d0/4.0d0*GPL2(0,3)*omumv**(-2)*pi*ci
         col5 = col5 + 1.0d0/4.0d0*GPL2(0,3)*omumv**(-1)*pi*ci
     .        + 2.0d0/3.0d0*GPL2(0,3)*pi*ci
     .        - 1.0d0/2.0d0*GPL2(0,3)*v*omumv**(-3)*pi*ci
     .        + 1.0d0/2.0d0*GPL2(0,3)*v*omumv**(-2)*pi*ci
     .        - 1.0d0/4.0d0*GPL2(0,3)*v*omumv**(-1)*pi*ci
     .        + 1.0d0/2.0d0*GPL2(0,3)*v**2*omumv**(-3)*pi*ci
     .        - 1.0d0/4.0d0*GPL2(0,3)*v**2*omumv**(-2)*pi*ci
     .        - 1.0d0/6.0d0*GPL2(0,3)*v**3*omumv**(-3)*pi*ci
     .        + 1.0d0/3.0d0*GPL2(1,1)*pi*ci
     .        - 5.0d0/9.0d0*GPL2(1,2)
     .        + 11.0d0/12.0d0*GPL2(1,2)*u**(-2)
     .        - 7.0d0/6.0d0*GPL2(1,2)*u**(-1)
     .        + 1.0d0/6.0d0*GPL2(1,2)*omumv**(-2)
     .        - 1.0d0/6.0d0*GPL2(1,2)*omumv**(-1)
     .        - 1.0d0/6.0d0*GPL2(1,2)*pi*ci
         col5 = col5 - 1.0d0/6.0d0*GPL2(1,2)*v*omumv**(-2)
     .        - 7.0d0/6.0d0*GPL2(1,2)*u
     .        + 11.0d0/12.0d0*GPL2(1,2)*u**2
     .        - 1.0d0/36.0d0*GPL2(2,0)
     .        - 1.0d0/12.0d0*GPL2(2,0)*omumv**(-2)
     .        + 1.0d0/12.0d0*GPL2(2,0)*omumv**(-1)
     .        - 1.0d0/4.0d0*GPL2(2,0)*pi*ci
     .        + 1.0d0/12.0d0*GPL2(2,0)*v*omumv**(-2)
     .        + 7.0d0/6.0d0*GPL2(2,0)*u
     .        - 11.0d0/12.0d0*GPL2(2,0)*u**2
     .        + 1.0d0/2.0d0*GPL2(2,1)*pi*ci
     .        - 11.0d0/12.0d0*GPL2(2,2)
     .        - 1.0d0/3.0d0*GPL2(2,2)*pi*ci
     .        + 1.0d0/6.0d0*GPL2(2,3)*pi*ci
     .        - 5.0d0/9.0d0*GPL2(3,0)
         col5 = col5 + 11.0d0/4.0d0*GPL2(3,0)*omumv**(-4)
     .        - 11.0d0/2.0d0*GPL2(3,0)*omumv**(-3)
     .        + 47.0d0/12.0d0*GPL2(3,0)*omumv**(-2)
     .        - 7.0d0/6.0d0*GPL2(3,0)*omumv**(-1)
     .        - 11.0d0/2.0d0*GPL2(3,0)*v*omumv**(-4)
     .        + 31.0d0/6.0d0*GPL2(3,0)*v*omumv**(-3)
     .        - 7.0d0/6.0d0*GPL2(3,0)*v*omumv**(-2)
     .        + 1.0d0/6.0d0*GPL2(3,0)*v*omumv**(-1)
     .        + 11.0d0/4.0d0*GPL2(3,0)*v**2*omumv**(-4)
     .        + 1.0d0/3.0d0*GPL2(3,0)*v**2*omumv**(-3)
     .        + 1.0d0/6.0d0*GPL2(3,0)*v**2*omumv**(-2)
     .        - 7.0d0/6.0d0*GPL2(3,0)*u
     .        + 11.0d0/12.0d0*GPL2(3,0)*u**2
     .        + 1.0d0/6.0d0*GPL2(3,2)*pi*ci
     .        - 1.0d0/3.0d0*GPL2(3,3)*omumv**(-3)*pi*ci
         col5 = col5 + 1.0d0/2.0d0*GPL2(3,3)*omumv**(-2)*pi*ci
     .        - 1.0d0/2.0d0*GPL2(3,3)*omumv**(-1)*pi*ci
     .        - 1.0d0/3.0d0*GPL3(0,0,2)
     .        - 1.0d0/3.0d0*GPL3(0,2,0)
     .        - 1.0d0/3.0d0*GPL3(0,2,2)
     .        + 2.0d0/3.0d0*GPL3(0,3,0)
     .        + 1.0d0/6.0d0*GPL3(0,3,0)*omumv**(-3)
     .        - 1.0d0/4.0d0*GPL3(0,3,0)*omumv**(-2)
     .        + 1.0d0/4.0d0*GPL3(0,3,0)*omumv**(-1)
     .        - 1.0d0/2.0d0*GPL3(0,3,0)*v*omumv**(-3)
     .        + 1.0d0/2.0d0*GPL3(0,3,0)*v*omumv**(-2)
     .        - 1.0d0/4.0d0*GPL3(0,3,0)*v*omumv**(-1)
     .        + 1.0d0/2.0d0*GPL3(0,3,0)*v**2*omumv**(-3)
     .        - 1.0d0/4.0d0*GPL3(0,3,0)*v**2*omumv**(-2)
     .        - 1.0d0/6.0d0*GPL3(0,3,0)*v**3*omumv**(-3)
         col5 = col5 + 1.0d0/3.0d0*GPL3(1,1,2)
     .        + 2.0d0/3.0d0*GPL3(1,2,2)
     .        - 1.0d0/3.0d0*GPL3(2,0,0)
     .        - 1.0d0/3.0d0*GPL3(2,0,2)
     .        + 1.0d0/2.0d0*GPL3(2,1,2)
     .        - 1.0d0/3.0d0*GPL3(2,2,0)
     .        + 1.0d0/6.0d0*GPL3(2,3,0)
     .        + 2.0d0/3.0d0*GPL3(3,0,0)
     .        + 1.0d0/6.0d0*GPL3(3,0,2)
     .        + 1.0d0/6.0d0*GPL3(3,2,0)
     .        - 1.0d0/3.0d0*GPL3(3,3,0)*omumv**(-3)
     .        + 1.0d0/2.0d0*GPL3(3,3,0)*omumv**(-2)
     .        - 1.0d0/2.0d0*GPL3(3,3,0)*omumv**(-1)

         col6 = - 5.0d0/54.0d0*omumv**(-2)
     .        - 1.0d0/36.0d0*omumv**(-2)*pi*ci
     .        + 5.0d0/54.0d0*omumv**(-1)
     .        + 1.0d0/36.0d0*omumv**(-1)*pi*ci
     .        + 5.0d0/54.0d0*pi*ci
     .        - 5.0d0/144.0d0*pi**2
     .        + 5.0d0/54.0d0*v*omumv**(-2)
     .        + 1.0d0/36.0d0*v*omumv**(-2)*pi*ci
     .        + 5.0d0/18.0d0*HPL1(0)
     .        - 5.0d0/36.0d0*HPL1(0)*omumv**(-2)
     .        + 5.0d0/36.0d0*HPL1(0)*omumv**(-1)
     .        + 5.0d0/36.0d0*HPL1(0)*pi*ci
     .        + 5.0d0/36.0d0*HPL1(0)*v*omumv**(-2)
     .        - 5.0d0/36.0d0*HPL1(0)*GPL1(0)
     .        - 5.0d0/36.0d0*HPL1(0)*GPL1(2)
         col6 = col6 + 5.0d0/54.0d0*HPL1(1)
     .        - 1.0d0/12.0d0*HPL1(1)*omumv**(-2)
     .        + 1.0d0/12.0d0*HPL1(1)*omumv**(-1)
     .        + 1.0d0/36.0d0*HPL1(1)*pi*ci
     .        + 1.0d0/12.0d0*HPL1(1)*v*omumv**(-2)
     .        - 1.0d0/36.0d0*HPL1(1)*GPL1(0)
     .        - 1.0d0/12.0d0*HPL1(1)*GPL1(2)
     .        + 5.0d0/12.0d0*HPL2(0,0)
     .        + 5.0d0/36.0d0*HPL2(0,1)
     .        + 5.0d0/36.0d0*HPL2(1,0)
     .        + 1.0d0/12.0d0*HPL2(1,1)
     .        - 5.0d0/54.0d0*GPL1(0)
     .        + 1.0d0/36.0d0*GPL1(0)*omumv**(-2)
     .        - 1.0d0/36.0d0*GPL1(0)*omumv**(-1)
     .        - 1.0d0/36.0d0*GPL1(0)*pi*ci
         col6 = col6 - 1.0d0/36.0d0*GPL1(0)*v*omumv**(-2)
     .        - 5.0d0/54.0d0*GPL1(2)
     .        + 1.0d0/12.0d0*GPL1(2)*omumv**(-2)
     .        - 1.0d0/12.0d0*GPL1(2)*omumv**(-1)
     .        - 1.0d0/36.0d0*GPL1(2)*pi*ci
     .        - 1.0d0/12.0d0*GPL1(2)*v*omumv**(-2)
     .        + 1.0d0/12.0d0*GPL2(0,0)
     .        + 1.0d0/36.0d0*GPL2(0,2)
     .        + 1.0d0/36.0d0*GPL2(2,0)
     .        + 1.0d0/12.0d0*GPL2(2,2)
      endif

      col(1) = col1
      col(2) = col2
      col(3) = col3
      col(4) = col4
      col(5) = col5
      col(6) = col6

      end subroutine beta4a2L

!#############################################################

      subroutine gamma2a2L(u,v,col)
      implicit none
      !Returns gamma(2a+) at one-loop
      !------------------------------
      real*8, intent(in) :: u,v
      complex*16, dimension(6), intent(out) :: col
      !internal
      complex*16 :: gam
      complex*16 :: ci
      real*8 :: omu,omv,omumv,upv
      real*8, dimension(0:1) :: HPL1
      real*8, dimension(0:1,0:1) :: HPL2
      real*8, dimension(0:1,0:1,0:1) :: HPL3
      real*8, dimension(0:1,0:1,0:1,0:1) :: HPL4
      real*8, dimension(0:3) :: GPL1
      real*8, dimension(0:3,0:3) :: GPL2
      real*8, dimension(0:3,0:3,0:3) :: GPL3
      real*8, dimension(0:3,0:3,0:3,0:3) :: GPL4
      complex*16 :: col1,col2,col3,col4,col5,col6


      omu   = 1.0d0 - u
      omv   = 1.0d0     - v
      omumv = 1.0d0 - u - v
      upv   =         u + v

      ci = (0.d0,1.d0)

      call tdhpl(dble(u),dble(v),4,GPL1,GPL2,GPL3,GPL4,HPL1,HPL2,HPL3,HPL4)

      if(ieorder==4) then
         col1 = - 1.0d0/2.0d0

         col2 = 1.0d0/2.0d0

         col3 = - 1.0d0/8.0d0

         col4 = 0.0d0

         col5 = 0.0d0

         col6 = 0.0d0
      elseif(ieorder==3) then
         col1 = - 73.0d0/24.0d0
     .          - HPL1(0)
     .          - 1.0d0/2.0d0*HPL1(1)
     .          + 1.0d0/2.0d0*GPL1(0)
     .          + 1.0d0/2.0d0*GPL1(2)

         col2 = + 109.0d0/48.0d0
     .          + 1.0d0/2.0d0*pi*ci
     .          + HPL1(0)
     .          + 1.0d0/4.0d0*HPL1(1)
     .          - 1.0d0/4.0d0*GPL1(0)
     .          - 1.0d0/4.0d0*GPL1(2)

         col3 = - 3.0d0/8.0d0
     .          - 1.0d0/4.0d0*pi*ci
     .          - 1.0d0/4.0d0*HPL1(0)

         col4 = - 5.0d0/24.0d0

         col5 = + 5.0d0/12.0d0

         col6 = 0
      else
      col1 =
     +  - 235525.d0/10368.d0
     +  - 9.d0/4.d0*pi*zeta3*ci
     +  + 5.d0/4.d0*pi*u**(-2)*v*ci
     +  - 5.d0/4.d0*pi*u**(-2)*ci
     +  + 19.d0/48.d0*pi*u**(-1)*v*ci
     +  + 7.d0/8.d0*pi*u**(-1)*ci
     +  + 19.d0/48.d0*pi*v*omu**(-1)*ci
     +
      col1 = col1 + 5.d0/8.d0*pi*omumv**(-1)*ci
     +  - 10181.d0/864.d0*pi*ci
     +  - 1.d0/8.d0*pi**2*u**(-1)*v
     +  - 1.d0/48.d0*pi**2*u**(-1)*omv**(-1)
     +  - 1.d0/16.d0*pi**2*u**(-1)
     +  - 1.d0/48.d0*pi**2*omv**(-1)*omumv**(-1)
     +  + 5.d0/48.d0*pi**2*omumv**(-2)
     +  - 1.d0/3.d0*pi**2*omumv**(-1)
     +
      col1 = col1 + 913.d0/864.d0*pi**2
     +  + 35.d0/144.d0*pi**3*ci
     +  - 11.d0/160.d0*pi**4
     +  + 67.d0/72.d0*zeta3
     +  - 269.d0/72.d0*u**(-1)
     +  + 4.d0/3.d0*omv**(-1)
     +  - 12917.d0/864.d0*HPL1(0)
     +  - 1.d0/4.d0*HPL1(0)*pi*u**(-1)*v*ci
     +
      col1 = col1 - 1.d0/8.d0*HPL1(0)*pi*u**(-1)*omv**(-1)*ci
     +  - 1.d0/8.d0*HPL1(0)*pi*omv**(-1)*omumv**(-1)*ci
     +  + 5.d0/8.d0*HPL1(0)*pi*omumv**(-2)*ci
     +  - 2*HPL1(0)*pi*omumv**(-1)*ci
     +  - 31.d0/4.d0*HPL1(0)*pi*ci
     +  + 13.d0/24.d0*HPL1(0)*pi**2
     +  + 1.d0/12.d0*HPL1(0)*pi**3*ci
     +  - 5.d0/2.d0*HPL1(0)*zeta3
     +
      col1 = col1 - 5.d0/4.d0*HPL1(0)*u**(-2)*v
     +  - 19.d0/48.d0*HPL1(0)*u**(-1)*v
     +  + 5.d0/8.d0*HPL1(0)*u**(-1)*omv**(-1)
     +  - 85.d0/48.d0*HPL1(0)*u**(-1)
     +  - 19.d0/48.d0*HPL1(0)*v*omu**(-1)
     +  + 4.d0/3.d0*HPL1(0)*omv**(-2)
     +  + 5.d0/8.d0*HPL1(0)*omv**(-1)*omumv**(-1)
     +  - 13.d0/3.d0*HPL1(0)*omv**(-1)
     +
      col1 = col1 - 5.d0/8.d0*HPL1(0)*omumv**(-1)
     +  + 149.d0/144.d0*HPL1(0)*GPL1(0)
     +  - 1.d0/8.d0*HPL1(0)*GPL1(0)*pi*ci
     +  + 1.d0/12.d0*HPL1(0)*GPL1(0)*pi**2
     +  - 1.d0/8.d0*HPL1(0)*GPL1(0)*u**(-1)*v
     +  - 1.d0/8.d0*HPL1(0)*GPL1(0)*u**(-1)*omv**(-1)
     +  - 1.d0/8.d0*HPL1(0)*GPL1(0)*omv**(-1)*omumv**(-1)
     +  + 5.d0/8.d0*HPL1(0)*GPL1(0)*omumv**(-2)
     +
      col1 = col1 - 2*HPL1(0)*GPL1(0)*omumv**(-1)
     +  + 401.d0/144.d0*HPL1(0)*GPL1(1)
     +  - 1.d0/8.d0*HPL1(0)*GPL1(1)*pi*u**(-2)*ci
     +  + 1.d0/2.d0*HPL1(0)*GPL1(1)*pi*u**(-1)*ci
     +  - 5.d0/6.d0*HPL1(0)*GPL1(1)*pi*ci
     +  - 5.d0/24.d0*HPL1(0)*GPL1(1)*pi**2
     +  - 5.d0/4.d0*HPL1(0)*GPL1(1)*u**(-3)*v
     +  + 5.d0/4.d0*HPL1(0)*GPL1(1)*u**(-3)
     +
      col1 = col1 + 1.d0/6.d0*HPL1(0)*GPL1(1)*u**(-2)*v
     +  - 25.d0/12.d0*HPL1(0)*GPL1(1)*u**(-2)
     +  + 25.d0/12.d0*HPL1(0)*GPL1(1)*u**(-1)
     +  + 149.d0/144.d0*HPL1(0)*GPL1(2)
     +  - 1.d0/4.d0*HPL1(0)*GPL1(2)*pi*u**(-2)*v*ci
     +  + 1.d0/8.d0*HPL1(0)*GPL1(2)*pi*u**(-2)*v**2*ci
     +  + 1.d0/8.d0*HPL1(0)*GPL1(2)*pi*u**(-2)*ci
     +  + 1.d0/2.d0*HPL1(0)*GPL1(2)*pi*u**(-1)*v*ci
     +
      col1 = col1 - 1.d0/2.d0*HPL1(0)*GPL1(2)*pi*u**(-1)*ci
     +  - 5.d0/2.d0*HPL1(0)*GPL1(2)*pi*ci
     +  + 19.d0/24.d0*HPL1(0)*GPL1(2)*pi**2
     +  + 5.d0/4.d0*HPL1(0)*GPL1(2)*u**(-3)*v
     +  - 5.d0/4.d0*HPL1(0)*GPL1(2)*u**(-3)
     +  - 1.d0/6.d0*HPL1(0)*GPL1(2)*u**(-2)*v
     +  + 25.d0/12.d0*HPL1(0)*GPL1(2)*u**(-2)
     +  - 1.d0/8.d0*HPL1(0)*GPL1(2)*u**(-1)*v
     +
      col1 = col1 - 25.d0/12.d0*HPL1(0)*GPL1(2)*u**(-1)
     +  + 401.d0/144.d0*HPL1(0)*GPL1(3)
     +  + 1.d0/12.d0*HPL1(0)*GPL1(3)*pi*ci
     +  - 1.d0/12.d0*HPL1(0)*GPL1(3)*pi**2
     +  + 1.d0/8.d0*HPL1(0)*GPL1(3)*u**(-1)*v
     +  + 1.d0/8.d0*HPL1(0)*GPL1(3)*u**(-1)*omv**(-1)
     +  + 1.d0/8.d0*HPL1(0)*GPL1(3)*omv**(-1)*omumv**(-1)
     +  - 5.d0/8.d0*HPL1(0)*GPL1(3)*omumv**(-2)
     +
      col1 = col1 + 2*HPL1(0)*GPL1(3)*omumv**(-1)
     +  + 7.d0/4.d0*HPL1(0)*GPL2(0,0)
     +  + 25.d0/24.d0*HPL1(0)*GPL2(0,1)
     +  + 1.d0/2.d0*HPL1(0)*GPL2(0,1)*pi*ci
     +  + 1.d0/8.d0*HPL1(0)*GPL2(0,1)*u**(-2)
     +  - 1.d0/2.d0*HPL1(0)*GPL2(0,1)*u**(-1)
     +  - 5.d0/8.d0*HPL1(0)*GPL2(0,2)
     +  - HPL1(0)*GPL2(0,2)*pi*ci
     +
      col1 = col1 - 1.d0/8.d0*HPL1(0)*GPL2(0,2)*u**(-2)
     +  + 1.d0/2.d0*HPL1(0)*GPL2(0,2)*u**(-1)
     +  - 4.d0/3.d0*HPL1(0)*GPL2(0,3)
     +  - 1.d0/2.d0*HPL1(0)*GPL2(0,3)*pi*ci
     +  - 5.d0/12.d0*HPL1(0)*GPL2(1,0)
     +  + 1.d0/4.d0*HPL1(0)*GPL2(1,0)*pi*ci
     +  - 13.d0/24.d0*HPL1(0)*GPL2(1,1)
     +  - 1.d0/8.d0*HPL1(0)*GPL2(1,1)*u**(-2)
     +
      col1 = col1 + 1.d0/2.d0*HPL1(0)*GPL2(1,1)*u**(-1)
     +  - 13.d0/6.d0*HPL1(0)*GPL2(1,2)
     +  - 5.d0/8.d0*HPL1(0)*GPL2(2,0)
     +  - HPL1(0)*GPL2(2,0)*pi*ci
     +  - 41.d0/24.d0*HPL1(0)*GPL2(2,1)
     +  - 1.d0/4.d0*HPL1(0)*GPL2(2,1)*u**(-2)*v
     +  + 1.d0/8.d0*HPL1(0)*GPL2(2,1)*u**(-2)*v**2
     +  + 1.d0/2.d0*HPL1(0)*GPL2(2,1)*u**(-1)*v
     +
      col1 = col1 + 17.d0/8.d0*HPL1(0)*GPL2(2,2)
     +  - 1.d0/4.d0*HPL1(0)*GPL2(2,2)*pi*ci
     +  + 1.d0/8.d0*HPL1(0)*GPL2(2,2)*u**(-2)
     +  - 1.d0/2.d0*HPL1(0)*GPL2(2,2)*u**(-1)
     +  + 25.d0/24.d0*HPL1(0)*GPL2(2,3)
     +  + 3.d0/4.d0*HPL1(0)*GPL2(2,3)*pi*ci
     +  - 43.d0/24.d0*HPL1(0)*GPL2(3,0)
     +  - 1.d0/4.d0*HPL1(0)*GPL2(3,0)*pi*ci
     +
      col1 = col1 - 5.d0/12.d0*HPL1(0)*GPL2(3,2)
     +  - 11.d0/12.d0*HPL1(0)*GPL2(3,3)
     +  + 1.d0/2.d0*HPL1(0)*GPL2(3,3)*pi*ci
     +  + 1.d0/4.d0*HPL1(0)*GPL3(0,0,1)
     +  - 1.d0/4.d0*HPL1(0)*GPL3(0,0,2)
     +  + 1.d0/2.d0*HPL1(0)*GPL3(0,1,0)
     +  - 1.d0/4.d0*HPL1(0)*GPL3(0,1,1)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(0,1,2)
     +
      col1 = col1 - HPL1(0)*GPL3(0,2,0)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(0,2,1)
     +  - 1.d0/4.d0*HPL1(0)*GPL3(0,2,2)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(0,2,3)
     +  - 1.d0/4.d0*HPL1(0)*GPL3(0,3,0)
     +  - 1.d0/4.d0*HPL1(0)*GPL3(0,3,2)
     +  + 1.d0/2.d0*HPL1(0)*GPL3(0,3,3)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(1,0,1)
     +
      col1 = col1 - 1.d0/2.d0*HPL1(0)*GPL3(1,1,1)
     +  + 1.d0/2.d0*HPL1(0)*GPL3(1,1,2)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(1,2,0)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(1,2,1)
     +  - 1.d0/2.d0*HPL1(0)*GPL3(1,2,2)
     +  - 1.d0/4.d0*HPL1(0)*GPL3(2,0,0)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(2,0,1)
     +  - HPL1(0)*GPL3(2,0,2)
     +
      col1 = col1 + 1.d0/4.d0*HPL1(0)*GPL3(2,0,3)
     +  - 1.d0/4.d0*HPL1(0)*GPL3(2,1,0)
     +  + 1.d0/2.d0*HPL1(0)*GPL3(2,1,1)
     +  - 1.d0/4.d0*HPL1(0)*GPL3(2,1,2)
     +  - 1.d0/4.d0*HPL1(0)*GPL3(2,2,0)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(2,2,3)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(2,3,0)
     +  + 1.d0/2.d0*HPL1(0)*GPL3(2,3,2)
     +
      col1 = col1 - 1.d0/4.d0*HPL1(0)*GPL3(2,3,3)
     +  - 1.d0/2.d0*HPL1(0)*GPL3(3,0,0)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(3,0,2)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(3,0,3)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(3,2,3)
     +  + 1.d0/2.d0*HPL1(0)*GPL3(3,3,0)
     +  - 1.d0/2.d0*HPL1(0)*GPL3(3,3,3)
     +  - 37.d0/12.d0*HPL1(1)
     +
      col1 = col1 - 1.d0/8.d0*HPL1(1)*pi*u**(-1)*v*ci
     +  - 1.d0/8.d0*HPL1(1)*pi*u**(-1)*ci
     +  - 103.d0/144.d0*HPL1(1)*pi*ci
     +  + 25.d0/288.d0*HPL1(1)*pi**2
     +  + 1.d0/6.d0*HPL1(1)*pi**3*ci
     +  - 5.d0/2.d0*HPL1(1)*zeta3
     +  - 5.d0/4.d0*HPL1(1)*u**(-2)*v
     +  + 5.d0/4.d0*HPL1(1)*u**(-2)
     +
      col1 = col1 - 19.d0/48.d0*HPL1(1)*u**(-1)*v
     +  - 65.d0/48.d0*HPL1(1)*u**(-1)
     +  - 19.d0/48.d0*HPL1(1)*v*omu**(-1)
     +  + 127.d0/144.d0*HPL1(1)*GPL1(0)
     +  + 3.d0/8.d0*HPL1(1)*GPL1(0)*pi*ci
     +  + 5.d0/24.d0*HPL1(1)*GPL1(0)*pi**2
     +  - 1.d0/8.d0*HPL1(1)*GPL1(0)*u**(-1)*v
     +  + 401.d0/144.d0*HPL1(1)*GPL1(1)
     +
      col1 = col1 - 1.d0/8.d0*HPL1(1)*GPL1(1)*pi*u**(-2)*ci
     +  + 1.d0/2.d0*HPL1(1)*GPL1(1)*pi*u**(-1)*ci
     +  - 1.d0/4.d0*HPL1(1)*GPL1(1)*pi*ci
     +  - 1.d0/8.d0*HPL1(1)*GPL1(1)*pi**2
     +  - 5.d0/4.d0*HPL1(1)*GPL1(1)*u**(-3)*v
     +  + 5.d0/4.d0*HPL1(1)*GPL1(1)*u**(-3)
     +  + 1.d0/6.d0*HPL1(1)*GPL1(1)*u**(-2)*v
     +  - 25.d0/12.d0*HPL1(1)*GPL1(1)*u**(-2)
     +
      col1 = col1 + 25.d0/12.d0*HPL1(1)*GPL1(1)*u**(-1)
     +  + 20.d0/9.d0*HPL1(1)*GPL1(2)
     +  + 3.d0/8.d0*HPL1(1)*GPL1(2)*pi*ci
     +  + 1.d0/12.d0*HPL1(1)*GPL1(2)*pi**2
     +  - 1.d0/8.d0*HPL1(1)*GPL1(2)*u**(-1)
     +  - 5.d0/12.d0*HPL1(1)*GPL1(3)*pi*ci
     +  + 1.d0/24.d0*HPL1(1)*GPL1(3)*pi**2
     +  - 1.d0/4.d0*HPL1(1)*GPL2(0,0)*pi*ci
     +
      col1 = col1 + 25.d0/24.d0*HPL1(1)*GPL2(0,1)
     +  + 3.d0/4.d0*HPL1(1)*GPL2(0,1)*pi*ci
     +  + 1.d0/8.d0*HPL1(1)*GPL2(0,1)*u**(-2)
     +  - 1.d0/2.d0*HPL1(1)*GPL2(0,1)*u**(-1)
     +  + 3.d0/8.d0*HPL1(1)*GPL2(0,2)
     +  - 1.d0/4.d0*HPL1(1)*GPL2(0,2)*pi*ci
     +  - 1.d0/4.d0*HPL1(1)*GPL2(0,3)*pi*ci
     +  - 5.d0/12.d0*HPL1(1)*GPL2(1,0)
     +
      col1 = col1 - 13.d0/24.d0*HPL1(1)*GPL2(1,1)
     +  + 1.d0/2.d0*HPL1(1)*GPL2(1,1)*pi*ci
     +  - 1.d0/8.d0*HPL1(1)*GPL2(1,1)*u**(-2)
     +  + 1.d0/2.d0*HPL1(1)*GPL2(1,1)*u**(-1)
     +  - 17.d0/8.d0*HPL1(1)*GPL2(1,2)
     +  - 1.d0/8.d0*HPL1(1)*GPL2(1,2)*u**(-2)
     +  + 1.d0/2.d0*HPL1(1)*GPL2(1,2)*u**(-1)
     +  + 3.d0/8.d0*HPL1(1)*GPL2(2,0)
     +
      col1 = col1 - 1.d0/4.d0*HPL1(1)*GPL2(2,0)*pi*ci
     +  - 41.d0/24.d0*HPL1(1)*GPL2(2,1)
     +  - 1.d0/4.d0*HPL1(1)*GPL2(2,1)*pi*ci
     +  - 1.d0/4.d0*HPL1(1)*GPL2(2,1)*u**(-2)*v
     +  + 1.d0/8.d0*HPL1(1)*GPL2(2,1)*u**(-2)*v**2
     +  + 1.d0/2.d0*HPL1(1)*GPL2(2,1)*u**(-1)*v
     +  + 1.d0/2.d0*HPL1(1)*GPL2(2,3)*pi*ci
     +  - 5.d0/12.d0*HPL1(1)*GPL2(3,0)
     +
      col1 = col1 + 1.d0/4.d0*HPL1(1)*GPL3(0,0,1)
     +  - 1.d0/4.d0*HPL1(1)*GPL3(0,0,2)
     +  + 1.d0/2.d0*HPL1(1)*GPL3(0,1,0)
     +  - 1.d0/4.d0*HPL1(1)*GPL3(0,1,1)
     +  + 1.d0/4.d0*HPL1(1)*GPL3(0,1,2)
     +  - 1.d0/4.d0*HPL1(1)*GPL3(0,2,0)
     +  + 1.d0/4.d0*HPL1(1)*GPL3(0,2,1)
     +  - 1.d0/4.d0*HPL1(1)*GPL3(0,3,0)
     +
      col1 = col1 + 1.d0/4.d0*HPL1(1)*GPL3(1,0,1)
     +  - 1.d0/2.d0*HPL1(1)*GPL3(1,1,1)
     +  + 1.d0/2.d0*HPL1(1)*GPL3(1,1,2)
     +  + 1.d0/4.d0*HPL1(1)*GPL3(1,2,1)
     +  - 1.d0/4.d0*HPL1(1)*GPL3(2,0,0)
     +  + 1.d0/4.d0*HPL1(1)*GPL3(2,0,1)
     +  - 1.d0/4.d0*HPL1(1)*GPL3(2,1,0)
     +  + 1.d0/2.d0*HPL1(1)*GPL3(2,1,1)
     +
      col1 = col1 + 1.d0/2.d0*HPL1(1)*GPL3(2,3,0)
     +  - 707.d0/72.d0*HPL2(0,0)
     +  - 53.d0/24.d0*HPL2(0,0)*pi*ci
     +  - 1.d0/12.d0*HPL2(0,0)*pi**2
     +  + 1.d0/8.d0*HPL2(0,0)*u**(-1)*v
     +  + 1.d0/8.d0*HPL2(0,0)*u**(-1)*omv**(-1)
     +  + 1.d0/8.d0*HPL2(0,0)*omv**(-1)*omumv**(-1)
     +  - 5.d0/8.d0*HPL2(0,0)*omumv**(-2)
     +
      col1 = col1 + 2*HPL2(0,0)*omumv**(-1)
     +  - 49.d0/12.d0*HPL2(0,0)*GPL1(0)
     +  - 1.d0/4.d0*HPL2(0,0)*GPL1(0)*pi*ci
     +  + 11.d0/4.d0*HPL2(0,0)*GPL1(1)
     +  - 41.d0/24.d0*HPL2(0,0)*GPL1(2)
     +  + 3.d0/4.d0*HPL2(0,0)*GPL1(2)*pi*ci
     +  + 11.d0/6.d0*HPL2(0,0)*GPL1(3)
     +  + 1.d0/4.d0*HPL2(0,0)*GPL1(3)*pi*ci
     +
      col1 = col1 - 1.d0/2.d0*HPL2(0,0)*GPL2(0,0)
     +  + 3.d0/4.d0*HPL2(0,0)*GPL2(0,2)
     +  + 1.d0/2.d0*HPL2(0,0)*GPL2(0,3)
     +  - 1.d0/2.d0*HPL2(0,0)*GPL2(1,0)
     +  + 1.d0/2.d0*HPL2(0,0)*GPL2(1,2)
     +  + 3.d0/4.d0*HPL2(0,0)*GPL2(2,0)
     +  - 1.d0/4.d0*HPL2(0,0)*GPL2(2,2)
     +  - 1.d0/4.d0*HPL2(0,0)*GPL2(2,3)
     +
      col1 = col1 + 3.d0/4.d0*HPL2(0,0)*GPL2(3,0)
     +  - 1.d0/4.d0*HPL2(0,0)*GPL2(3,2)
     +  - 1.d0/2.d0*HPL2(0,0)*GPL2(3,3)
     +  - 275.d0/72.d0*HPL2(0,1)
     +  - 5.d0/12.d0*HPL2(0,1)*pi*ci
     +  + 1.d0/24.d0*HPL2(0,1)*pi**2
     +  + 1.d0/8.d0*HPL2(0,1)*u**(-1)*v
     +  - 5.d0/12.d0*HPL2(0,1)*GPL1(0)
     +
      col1 = col1 + 65.d0/24.d0*HPL2(0,1)*GPL1(1)
     +  - 1.d0/4.d0*HPL2(0,1)*GPL1(1)*pi*ci
     +  + 1.d0/8.d0*HPL2(0,1)*GPL1(1)*u**(-2)
     +  - 1.d0/2.d0*HPL2(0,1)*GPL1(1)*u**(-1)
     +  - 5.d0/12.d0*HPL2(0,1)*GPL1(2)
     +  + 3.d0/4.d0*HPL2(0,1)*GPL1(2)*pi*ci
     +  + 1.d0/4.d0*HPL2(0,1)*GPL1(2)*u**(-2)*v
     +  - 1.d0/8.d0*HPL2(0,1)*GPL1(2)*u**(-2)*v**2
     +
      col1 = col1 - 1.d0/8.d0*HPL2(0,1)*GPL1(2)*u**(-2)
     +  - 1.d0/2.d0*HPL2(0,1)*GPL1(2)*u**(-1)*v
     +  + 1.d0/2.d0*HPL2(0,1)*GPL1(2)*u**(-1)
     +  + 5.d0/12.d0*HPL2(0,1)*GPL1(3)
     +  - 1.d0/4.d0*HPL2(0,1)*GPL1(3)*pi*ci
     +  + 1.d0/4.d0*HPL2(0,1)*GPL2(0,3)
     +  - 1.d0/4.d0*HPL2(0,1)*GPL2(1,0)
     +  + 1.d0/4.d0*HPL2(0,1)*GPL2(1,2)
     +
      col1 = col1 + 3.d0/4.d0*HPL2(0,1)*GPL2(2,0)
     +  - 1.d0/4.d0*HPL2(0,1)*GPL2(2,1)
     +  - 1.d0/2.d0*HPL2(0,1)*GPL2(2,3)
     +  - 1.d0/4.d0*HPL2(0,1)*GPL2(3,0)
     +  - 335.d0/48.d0*HPL2(1,0)
     +  - 5.d0/12.d0*HPL2(1,0)*pi*ci
     +  - 3.d0/8.d0*HPL2(1,0)*pi**2
     +  - 1.d0/8.d0*HPL2(1,0)*u**(-1)*omv**(-1)
     +
      col1 = col1 + 1.d0/8.d0*HPL2(1,0)*u**(-1)
     +  - 1.d0/8.d0*HPL2(1,0)*omv**(-1)*omumv**(-1)
     +  + 5.d0/8.d0*HPL2(1,0)*omumv**(-2)
     +  - 2*HPL2(1,0)*omumv**(-1)
     +  + 35.d0/24.d0*HPL2(1,0)*GPL1(0)
     +  + 1.d0/2.d0*HPL2(1,0)*GPL1(0)*pi*ci
     +  + 13.d0/6.d0*HPL2(1,0)*GPL1(1)
     +  - 31.d0/24.d0*HPL2(1,0)*GPL1(2)
     +
      col1 = col1 - 1.d0/4.d0*HPL2(1,0)*GPL1(2)*pi*ci
     +  - 1.d0/4.d0*HPL2(1,0)*GPL1(2)*u**(-2)*v
     +  + 1.d0/8.d0*HPL2(1,0)*GPL1(2)*u**(-2)*v**2
     +  + 1.d0/8.d0*HPL2(1,0)*GPL1(2)*u**(-2)
     +  + 1.d0/2.d0*HPL2(1,0)*GPL1(2)*u**(-1)*v
     +  - 1.d0/2.d0*HPL2(1,0)*GPL1(2)*u**(-1)
     +  + 5.d0/12.d0*HPL2(1,0)*GPL1(3)
     +  + 1.d0/4.d0*HPL2(1,0)*GPL2(0,0)
     +
      col1 = col1 - 1.d0/4.d0*HPL2(1,0)*GPL2(0,1)
     +  - 1.d0/4.d0*HPL2(1,0)*GPL2(0,2)
     +  + 1.d0/4.d0*HPL2(1,0)*GPL2(0,3)
     +  - 1.d0/4.d0*HPL2(1,0)*GPL2(1,0)
     +  - 1.d0/2.d0*HPL2(1,0)*GPL2(1,1)
     +  + 3.d0/4.d0*HPL2(1,0)*GPL2(1,2)
     +  + 1.d0/2.d0*HPL2(1,0)*GPL2(2,0)
     +  + 1.d0/4.d0*HPL2(1,0)*GPL2(2,1)
     +
      col1 = col1 - 1.d0/2.d0*HPL2(1,0)*GPL2(2,3)
     +  - 1.d0/4.d0*HPL2(1,0)*GPL2(3,2)
     +  - 20.d0/9.d0*HPL2(1,1)
     +  - 3.d0/8.d0*HPL2(1,1)*pi*ci
     +  + 1.d0/4.d0*HPL2(1,1)*pi**2
     +  + 1.d0/8.d0*HPL2(1,1)*u**(-1)
     +  - 3.d0/8.d0*HPL2(1,1)*GPL1(0)
     +  + 1.d0/4.d0*HPL2(1,1)*GPL1(0)*pi*ci
     +
      col1 = col1 + 17.d0/8.d0*HPL2(1,1)*GPL1(1)
     +  + 1.d0/8.d0*HPL2(1,1)*GPL1(1)*u**(-2)
     +  - 1.d0/2.d0*HPL2(1,1)*GPL1(1)*u**(-1)
     +  + 1.d0/4.d0*HPL2(1,1)*GPL2(0,0)
     +  - 1.d0/4.d0*HPL2(1,1)*GPL2(0,1)
     +  - 1.d0/2.d0*HPL2(1,1)*GPL2(1,1)
     +  + 33.d0/8.d0*HPL3(0,0,0)
     +  + 1.d0/4.d0*HPL3(0,0,0)*pi*ci
     +
      col1 = col1 + 3.d0/4.d0*HPL3(0,0,0)*GPL1(0)
     +  - 1.d0/2.d0*HPL3(0,0,0)*GPL1(1)
     +  + 1.d0/4.d0*HPL3(0,0,0)*GPL1(2)
     +  - 3.d0/4.d0*HPL3(0,0,0)*GPL1(3)
     +  + 5.d0/12.d0*HPL3(0,0,1)
     +  - 1.d0/4.d0*HPL3(0,0,1)*pi*ci
     +  - 1.d0/4.d0*HPL3(0,0,1)*GPL1(0)
     +  - 1.d0/2.d0*HPL3(0,0,1)*GPL1(1)
     +
      col1 = col1 + 1.d0/2.d0*HPL3(0,0,1)*GPL1(3)
     +  - 15.d0/8.d0*HPL3(0,1,0)
     +  - 1.d0/2.d0*HPL3(0,1,0)*GPL1(1)
     +  + 1.d0/4.d0*HPL3(0,1,0)*GPL1(3)
     +  - 91.d0/24.d0*HPL3(1,0,0)
     +  - 3.d0/4.d0*HPL3(1,0,0)*pi*ci
     +  - 3.d0/4.d0*HPL3(1,0,0)*GPL1(0)
     +  - 1.d0/2.d0*HPL3(1,0,0)*GPL1(1)
     +
      col1 = col1 + 1.d0/4.d0*HPL3(1,0,0)*GPL1(2)
     +  + 1.d0/4.d0*HPL3(1,0,0)*GPL1(3)
     +  - 5.d0/12.d0*HPL3(1,0,1)
     +  - 1.d0/4.d0*HPL3(1,0,1)*pi*ci
     +  - 1.d0/4.d0*HPL3(1,0,1)*GPL1(0)
     +  - 1.d0/4.d0*HPL3(1,0,1)*GPL1(1)
     +  + 3.d0/4.d0*HPL3(1,0,1)*GPL1(2)
     +  + 55.d0/24.d0*HPL3(1,1,0)
     +
      col1 = col1 + 3.d0/4.d0*HPL3(1,1,0)*pi*ci
     +  - 3.d0/4.d0*HPL3(1,1,0)*GPL1(1)
     +  + 3.d0/4.d0*HPL3(1,1,0)*GPL1(2)
     +  + 1.d0/4.d0*HPL3(1,1,0)*GPL1(3)
     +  - 3.d0/4.d0*HPL4(0,0,0,0)
     +  + 1.d0/2.d0*HPL4(0,0,0,1)
     +  + 1.d0/4.d0*HPL4(0,0,1,0)
     +  + 1.d0/4.d0*HPL4(0,1,0,0)
     +
      col1 = col1 + 1.d0/4.d0*HPL4(0,1,1,0)
     +  + 3.d0/4.d0*HPL4(1,0,0,0)
     +  - 1.d0/4.d0*HPL4(1,1,0,0)
     +  - 3.d0/4.d0*HPL4(1,1,0,1)
     +  + 5.d0/4.d0*HPL4(1,1,1,0)
     +  + 37.d0/12.d0*GPL1(0)
     +  + 1.d0/8.d0*GPL1(0)*pi*u**(-1)*v*ci
     +  + 103.d0/144.d0*GPL1(0)*pi*ci
     +
      col1 = col1 + 47.d0/96.d0*GPL1(0)*pi**2
     +  + 1.d0/12.d0*GPL1(0)*pi**3*ci
     +  + 1.d0/2.d0*GPL1(0)*zeta3
     +  + 2.d0/3.d0*GPL1(0)*u**(-1)
     +  + 5.d0/8.d0*GPL1(0)*omumv**(-1)
     +  + 5.d0/4.d0*GPL1(1)*pi*u**(-3)*v*ci
     +  - 5.d0/4.d0*GPL1(1)*pi*u**(-3)*ci
     +  - 1.d0/6.d0*GPL1(1)*pi*u**(-2)*v*ci
     +
      col1 = col1 + 25.d0/12.d0*GPL1(1)*pi*u**(-2)*ci
     +  - 25.d0/12.d0*GPL1(1)*pi*u**(-1)*ci
     +  - 401.d0/144.d0*GPL1(1)*pi*ci
     +  - 5.d0/48.d0*GPL1(1)*pi**2*u**(-2)
     +  + 5.d0/12.d0*GPL1(1)*pi**2*u**(-1)
     +  + 91.d0/144.d0*GPL1(1)*pi**2
     +  + 1.d0/12.d0*GPL1(1)*pi**3*ci
     +  + 1.d0/4.d0*GPL1(1)*zeta3
     +
      col1 = col1 + 37.d0/12.d0*GPL1(2)
     +  + 1.d0/8.d0*GPL1(2)*pi*u**(-1)*v*ci
     +  + 1.d0/8.d0*GPL1(2)*pi*u**(-1)*ci
     +  + 103.d0/144.d0*GPL1(2)*pi*ci
     +  - 1.d0/12.d0*GPL1(2)*pi**2*u**(-2)*v
     +  + 1.d0/24.d0*GPL1(2)*pi**2*u**(-2)*v**2
     +  + 1.d0/24.d0*GPL1(2)*pi**2*u**(-2)
     +  + 1.d0/6.d0*GPL1(2)*pi**2*u**(-1)*v
     +
      col1 = col1 - 1.d0/6.d0*GPL1(2)*pi**2*u**(-1)
     +  + 7.d0/32.d0*GPL1(2)*pi**2
     +  - 1.d0/6.d0*GPL1(2)*pi**3*ci
     +  + 1.d0/2.d0*GPL1(2)*zeta3
     +  + 5.d0/4.d0*GPL1(2)*u**(-2)*v
     +  - 5.d0/4.d0*GPL1(2)*u**(-2)
     +  + 19.d0/48.d0*GPL1(2)*u**(-1)*v
     +  + 65.d0/48.d0*GPL1(2)*u**(-1)
     +
      col1 = col1 + 19.d0/48.d0*GPL1(2)*v*omu**(-1)
     +  - 1.d0/8.d0*GPL1(3)*pi*u**(-1)*v*ci
     +  - 1.d0/8.d0*GPL1(3)*pi*u**(-1)*omv**(-1)*ci
     +  - 1.d0/8.d0*GPL1(3)*pi*omv**(-1)*omumv**(-1)*ci
     +  + 5.d0/8.d0*GPL1(3)*pi*omumv**(-2)*ci
     +  - 2*GPL1(3)*pi*omumv**(-1)*ci
     +  - 401.d0/144.d0*GPL1(3)*pi*ci
     +  + 61.d0/72.d0*GPL1(3)*pi**2
     +
      col1 = col1 + 1.d0/12.d0*GPL1(3)*pi**3*ci
     +  - 1.d0/2.d0*GPL1(3)*zeta3
     +  - 20.d0/9.d0*GPL2(0,0)
     +  - 1.d0/24.d0*GPL2(0,0)*pi**2
     +  - 1.d0/8.d0*GPL2(0,1)*pi*u**(-2)*ci
     +  + 1.d0/2.d0*GPL2(0,1)*pi*u**(-1)*ci
     +  - 25.d0/24.d0*GPL2(0,1)*pi*ci
     +  + 13.d0/24.d0*GPL2(0,1)*pi**2
     +
      col1 = col1 - 127.d0/144.d0*GPL2(0,2)
     +  - 3.d0/8.d0*GPL2(0,2)*pi*ci
     +  - 5.d0/24.d0*GPL2(0,2)*pi**2
     +  + 1.d0/8.d0*GPL2(0,2)*u**(-1)*v
     +  + 4.d0/3.d0*GPL2(0,3)*pi*ci
     +  - 1.d0/6.d0*GPL2(0,3)*pi**2
     +  + 5.d0/12.d0*GPL2(1,0)*pi*ci
     +  + 1.d0/8.d0*GPL2(1,1)*pi*u**(-2)*ci
     +
      col1 = col1 - 1.d0/2.d0*GPL2(1,1)*pi*u**(-1)*ci
     +  + 13.d0/24.d0*GPL2(1,1)*pi*ci
     +  + 1.d0/12.d0*GPL2(1,1)*pi**2
     +  - 401.d0/144.d0*GPL2(1,2)
     +  + 1.d0/8.d0*GPL2(1,2)*pi*u**(-2)*ci
     +  - 1.d0/2.d0*GPL2(1,2)*pi*u**(-1)*ci
     +  + 1.d0/4.d0*GPL2(1,2)*pi*ci
     +  + 1.d0/8.d0*GPL2(1,2)*pi**2
     +
      col1 = col1 + 5.d0/4.d0*GPL2(1,2)*u**(-3)*v
     +  - 5.d0/4.d0*GPL2(1,2)*u**(-3)
     +  - 1.d0/6.d0*GPL2(1,2)*u**(-2)*v
     +  + 25.d0/12.d0*GPL2(1,2)*u**(-2)
     +  - 25.d0/12.d0*GPL2(1,2)*u**(-1)
     +  - 127.d0/144.d0*GPL2(2,0)
     +  - 3.d0/8.d0*GPL2(2,0)*pi*ci
     +  - 5.d0/24.d0*GPL2(2,0)*pi**2
     +
      col1 = col1 + 1.d0/8.d0*GPL2(2,0)*u**(-1)*v
     +  + 1.d0/4.d0*GPL2(2,1)*pi*u**(-2)*v*ci
     +  - 1.d0/8.d0*GPL2(2,1)*pi*u**(-2)*v**2*ci
     +  - 1.d0/2.d0*GPL2(2,1)*pi*u**(-1)*v*ci
     +  + 41.d0/24.d0*GPL2(2,1)*pi*ci
     +  - 1.d0/12.d0*GPL2(2,1)*pi**2
     +  - 20.d0/9.d0*GPL2(2,2)
     +  - 3.d0/8.d0*GPL2(2,2)*pi*ci
     +
      col1 = col1 - 1.d0/12.d0*GPL2(2,2)*pi**2
     +  + 1.d0/8.d0*GPL2(2,2)*u**(-1)
     +  - 25.d0/24.d0*GPL2(2,3)*pi*ci
     +  + 7.d0/12.d0*GPL2(2,3)*pi**2
     +  - 401.d0/144.d0*GPL2(3,0)
     +  - 1.d0/8.d0*GPL2(3,0)*pi*ci
     +  + 1.d0/12.d0*GPL2(3,0)*pi**2
     +  - 1.d0/8.d0*GPL2(3,0)*u**(-1)*v
     +
      col1 = col1 - 1.d0/8.d0*GPL2(3,0)*u**(-1)*omv**(-1)
     +  - 1.d0/8.d0*GPL2(3,0)*omv**(-1)*omumv**(-1)
     +  + 5.d0/8.d0*GPL2(3,0)*omumv**(-2)
     +  - 2*GPL2(3,0)*omumv**(-1)
     +  + 5.d0/12.d0*GPL2(3,2)*pi*ci
     +  - 1.d0/24.d0*GPL2(3,2)*pi**2
     +  + 11.d0/12.d0*GPL2(3,3)*pi*ci
     +  + 1.d0/6.d0*GPL2(3,3)*pi**2
     +
      col1 = col1 - 1.d0/4.d0*GPL3(0,0,1)*pi*ci
     +  + 1.d0/4.d0*GPL3(0,0,2)*pi*ci
     +  - 1.d0/2.d0*GPL3(0,1,0)*pi*ci
     +  + 1.d0/4.d0*GPL3(0,1,1)*pi*ci
     +  - 25.d0/24.d0*GPL3(0,1,2)
     +  - 3.d0/4.d0*GPL3(0,1,2)*pi*ci
     +  - 1.d0/8.d0*GPL3(0,1,2)*u**(-2)
     +  + 1.d0/2.d0*GPL3(0,1,2)*u**(-1)
     +
      col1 = col1 + 1.d0/4.d0*GPL3(0,2,0)*pi*ci
     +  - 1.d0/4.d0*GPL3(0,2,1)*pi*ci
     +  - 3.d0/8.d0*GPL3(0,2,2)
     +  + 1.d0/4.d0*GPL3(0,2,2)*pi*ci
     +  - 1.d0/4.d0*GPL3(0,2,3)*pi*ci
     +  + 4.d0/3.d0*GPL3(0,3,0)
     +  + 1.d0/4.d0*GPL3(0,3,0)*pi*ci
     +  + 1.d0/4.d0*GPL3(0,3,2)*pi*ci
     +
      col1 = col1 - 1.d0/2.d0*GPL3(0,3,3)*pi*ci
     +  - 1.d0/4.d0*GPL3(1,0,1)*pi*ci
     +  + 5.d0/12.d0*GPL3(1,0,2)
     +  + 1.d0/2.d0*GPL3(1,1,1)*pi*ci
     +  + 13.d0/24.d0*GPL3(1,1,2)
     +  - 1.d0/2.d0*GPL3(1,1,2)*pi*ci
     +  + 1.d0/8.d0*GPL3(1,1,2)*u**(-2)
     +  - 1.d0/2.d0*GPL3(1,1,2)*u**(-1)
     +
      col1 = col1 + 5.d0/12.d0*GPL3(1,2,0)
     +  - 1.d0/4.d0*GPL3(1,2,1)*pi*ci
     +  + 17.d0/8.d0*GPL3(1,2,2)
     +  + 1.d0/8.d0*GPL3(1,2,2)*u**(-2)
     +  - 1.d0/2.d0*GPL3(1,2,2)*u**(-1)
     +  + 1.d0/4.d0*GPL3(2,0,0)*pi*ci
     +  - 1.d0/4.d0*GPL3(2,0,1)*pi*ci
     +  - 3.d0/8.d0*GPL3(2,0,2)
     +
      col1 = col1 + 1.d0/4.d0*GPL3(2,0,2)*pi*ci
     +  - 1.d0/4.d0*GPL3(2,0,3)*pi*ci
     +  + 1.d0/4.d0*GPL3(2,1,0)*pi*ci
     +  - 1.d0/2.d0*GPL3(2,1,1)*pi*ci
     +  + 41.d0/24.d0*GPL3(2,1,2)
     +  + 1.d0/4.d0*GPL3(2,1,2)*pi*ci
     +  + 1.d0/4.d0*GPL3(2,1,2)*u**(-2)*v
     +  - 1.d0/8.d0*GPL3(2,1,2)*u**(-2)*v**2
     +
      col1 = col1 - 1.d0/2.d0*GPL3(2,1,2)*u**(-1)*v
     +  - 3.d0/8.d0*GPL3(2,2,0)
     +  + 1.d0/4.d0*GPL3(2,2,0)*pi*ci
     +  - 1.d0/4.d0*GPL3(2,2,3)*pi*ci
     +  - 25.d0/24.d0*GPL3(2,3,0)
     +  - 3.d0/4.d0*GPL3(2,3,0)*pi*ci
     +  - 1.d0/2.d0*GPL3(2,3,2)*pi*ci
     +  + 1.d0/4.d0*GPL3(2,3,3)*pi*ci
     +
      col1 = col1 + 7.d0/4.d0*GPL3(3,0,0)
     +  + 5.d0/12.d0*GPL3(3,0,2)
     +  - 1.d0/4.d0*GPL3(3,0,3)*pi*ci
     +  + 5.d0/12.d0*GPL3(3,2,0)
     +  - 1.d0/4.d0*GPL3(3,2,3)*pi*ci
     +  + 11.d0/12.d0*GPL3(3,3,0)
     +  - 1.d0/2.d0*GPL3(3,3,0)*pi*ci
     +  + 1.d0/2.d0*GPL3(3,3,3)*pi*ci
     +
      col1 = col1 - 1.d0/4.d0*GPL4(0,0,1,2)
     +  + 1.d0/4.d0*GPL4(0,0,2,2)
     +  - 1.d0/2.d0*GPL4(0,1,0,2)
     +  + 1.d0/4.d0*GPL4(0,1,1,2)
     +  - 1.d0/2.d0*GPL4(0,1,2,0)
     +  - 1.d0/4.d0*GPL4(0,1,2,2)
     +  + 1.d0/4.d0*GPL4(0,2,0,2)
     +  - 1.d0/4.d0*GPL4(0,2,1,2)
     +
      col1 = col1 + 1.d0/4.d0*GPL4(0,2,2,0)
     +  - 1.d0/4.d0*GPL4(0,2,3,0)
     +  + 1.d0/4.d0*GPL4(0,3,0,2)
     +  + 1.d0/4.d0*GPL4(0,3,2,0)
     +  - 1.d0/2.d0*GPL4(0,3,3,0)
     +  - 1.d0/4.d0*GPL4(1,0,1,2)
     +  + 1.d0/2.d0*GPL4(1,1,1,2)
     +  - 1.d0/2.d0*GPL4(1,1,2,2)
     +
      col1 = col1 - 1.d0/4.d0*GPL4(1,2,1,2)
     +  + 1.d0/4.d0*GPL4(2,0,0,2)
     +  - 1.d0/4.d0*GPL4(2,0,1,2)
     +  + 1.d0/4.d0*GPL4(2,0,2,0)
     +  - 1.d0/4.d0*GPL4(2,0,3,0)
     +  + 1.d0/4.d0*GPL4(2,1,0,2)
     +  - 1.d0/2.d0*GPL4(2,1,1,2)
     +  + 1.d0/4.d0*GPL4(2,1,2,0)
     +
      col1 = col1 + 1.d0/4.d0*GPL4(2,2,0,0)
     +  - 1.d0/4.d0*GPL4(2,2,3,0)
     +  - 1.d0/4.d0*GPL4(2,3,0,0)
     +  - 1.d0/2.d0*GPL4(2,3,0,2)
     +  - 1.d0/2.d0*GPL4(2,3,2,0)
     +  + 1.d0/4.d0*GPL4(2,3,3,0)
     +  - 1.d0/4.d0*GPL4(3,0,3,0)
     +  - 1.d0/4.d0*GPL4(3,2,3,0)
     +
      col1 = col1 - 1.d0/2.d0*GPL4(3,3,0,0)
     +  + 1.d0/2.d0*GPL4(3,3,3,0)
     +

      col2 =
     +  - 38435.d0/2592.d0
     +  + 1.d0/2.d0*pi*zeta3*ci
     +  + 1.d0/8.d0*pi*u**(-2)*v*ci
     +  - 1.d0/8.d0*pi*u**(-2)*ci
     +  + 7.d0/24.d0*pi*u**(-1)*v*ci
     +  - 1.d0/4.d0*pi*u**(-1)*ci
     +  + 2.d0/3.d0*pi*v*omu**(-1)*ci
     +
      col2 = col2 + 1.d0/16.d0*pi*omumv**(-1)*ci
     +  - 589.d0/108.d0*pi*ci
     +  - 1.d0/24.d0*pi**2*u**(-1)*v
     +  - 1.d0/48.d0*pi**2*u**(-1)
     +  - 1.d0/4.d0*pi**2*u
     +  - 1.d0/4.d0*pi**2*v
     +  - 1.d0/16.d0*pi**2*upv**(-1)
     +  + 1.d0/96.d0*pi**2*omumv**(-2)
     +
      col2 = col2 + 1.d0/24.d0*pi**2*omumv**(-1)
     +  + 2129.d0/864.d0*pi**2
     +  + 163.d0/288.d0*pi**3*ci
     +  + 271.d0/2880.d0*pi**4
     +  + 683.d0/144.d0*zeta3
     +  - 139.d0/36.d0*u**(-1)
     +  - 949.d0/108.d0*HPL1(0)
     +  - 1.d0/8.d0*HPL1(0)*pi*u**(-1)*v*ci
     +
      col2 = col2 + 1.d0/8.d0*HPL1(0)*pi*u**(-1)*ci
     +  - 3.d0/8.d0*HPL1(0)*pi*u*ci
     +  - 3.d0/8.d0*HPL1(0)*pi*v*ci
     +  - 1.d0/8.d0*HPL1(0)*pi*upv**(-1)*ci
     +  + 1.d0/16.d0*HPL1(0)*pi*omumv**(-2)*ci
     +  + 1.d0/4.d0*HPL1(0)*pi*omumv**(-1)*ci
     +  + 383.d0/144.d0*HPL1(0)*pi*ci
     +  - 17.d0/288.d0*HPL1(0)*pi**2
     +
      col2 = col2 + 3.d0/8.d0*HPL1(0)*pi**3*ci
     +  + 1.d0/4.d0*HPL1(0)*zeta3
     +  - 1.d0/8.d0*HPL1(0)*u**(-2)*v
     +  - 7.d0/24.d0*HPL1(0)*u**(-1)*v
     +  + 1.d0/16.d0*HPL1(0)*u**(-1)*omv**(-1)
     +  - 29.d0/24.d0*HPL1(0)*u**(-1)
     +  - 2.d0/3.d0*HPL1(0)*v*omu**(-1)
     +  + 1.d0/16.d0*HPL1(0)*omv**(-1)*omumv**(-1)
     +
      col2 = col2 - 1.d0/16.d0*HPL1(0)*omumv**(-1)
     +  + 383.d0/144.d0*HPL1(0)*GPL1(0)
     +  - 17.d0/12.d0*HPL1(0)*GPL1(0)*pi*ci
     +  + 7.d0/8.d0*HPL1(0)*GPL1(0)*pi**2
     +  - 3.d0/8.d0*HPL1(0)*GPL1(0)*u
     +  - 3.d0/8.d0*HPL1(0)*GPL1(0)*v
     +  + 1.d0/16.d0*HPL1(0)*GPL1(0)*omumv**(-2)
     +  + 1.d0/4.d0*HPL1(0)*GPL1(0)*omumv**(-1)
     +
      col2 = col2 - 383.d0/144.d0*HPL1(0)*GPL1(1)
     +  + 1.d0/12.d0*HPL1(0)*GPL1(1)*pi*ci
     +  - 3.d0/4.d0*HPL1(0)*GPL1(1)*pi**2
     +  - 1.d0/8.d0*HPL1(0)*GPL1(1)*u**(-3)*v
     +  + 1.d0/8.d0*HPL1(0)*GPL1(1)*u**(-3)
     +  - 1.d0/2.d0*HPL1(0)*GPL1(1)*u**(-2)*v
     +  - 1.d0/16.d0*HPL1(0)*GPL1(1)*u**(-2)
     +  + 1.d0/8.d0*HPL1(0)*GPL1(1)*u**(-1)*v**(-1)
     +
      col2 = col2 - 1.d0/8.d0*HPL1(0)*GPL1(1)*u**(-1)*v
     +  + 3.d0/8.d0*HPL1(0)*GPL1(1)*u
     +  - 1.d0/8.d0*HPL1(0)*GPL1(1)*v**(-1)*upv**(-1)
     +  + 3.d0/8.d0*HPL1(0)*GPL1(1)*v
     +  - 1.d0/8.d0*HPL1(0)*GPL1(1)*upv**(-1)
     +  + 383.d0/144.d0*HPL1(0)*GPL1(2)
     +  - 1.d0/4.d0*HPL1(0)*GPL1(2)*pi*u**(-2)*v*ci
     +  + 1.d0/8.d0*HPL1(0)*GPL1(2)*pi*u**(-2)*v**2*ci
     +
      col2 = col2 + 1.d0/8.d0*HPL1(0)*GPL1(2)*pi*u**(-2)*ci
     +  - 1.d0/4.d0*HPL1(0)*GPL1(2)*pi*u**(-1)*v*ci
     +  + 1.d0/4.d0*HPL1(0)*GPL1(2)*pi*u**(-1)*ci
     +  - 1.d0/8.d0*HPL1(0)*GPL1(2)*pi*upv**(-2)*ci
     +  - 1.d0/4.d0*HPL1(0)*GPL1(2)*pi*upv**(-1)*ci
     +  - 1.d0/2.d0*HPL1(0)*GPL1(2)*pi*ci
     +  + HPL1(0)*GPL1(2)*pi**2
     +  + 1.d0/8.d0*HPL1(0)*GPL1(2)*u**(-3)*v
     +
      col2 = col2 - 1.d0/8.d0*HPL1(0)*GPL1(2)*u**(-3)
     +  + 1.d0/2.d0*HPL1(0)*GPL1(2)*u**(-2)*v
     +  + 1.d0/16.d0*HPL1(0)*GPL1(2)*u**(-2)
     +  - 1.d0/8.d0*HPL1(0)*GPL1(2)*u**(-1)*v**(-1)
     +  + 1.d0/8.d0*HPL1(0)*GPL1(2)*u**(-1)
     +  - 3.d0/8.d0*HPL1(0)*GPL1(2)*u
     +  + 1.d0/8.d0*HPL1(0)*GPL1(2)*v**(-1)*upv**(-1)
     +  - 3.d0/8.d0*HPL1(0)*GPL1(2)*v
     +
      col2 = col2 - 383.d0/144.d0*HPL1(0)*GPL1(3)
     +  - 5.d0/6.d0*HPL1(0)*GPL1(3)*pi*ci
     +  - 9.d0/8.d0*HPL1(0)*GPL1(3)*pi**2
     +  + 3.d0/8.d0*HPL1(0)*GPL1(3)*u
     +  + 3.d0/8.d0*HPL1(0)*GPL1(3)*v
     +  - 1.d0/16.d0*HPL1(0)*GPL1(3)*omumv**(-2)
     +  - 1.d0/4.d0*HPL1(0)*GPL1(3)*omumv**(-1)
     +  - 7.d0/4.d0*HPL1(0)*GPL2(0,0)
     +
      col2 = col2 - HPL1(0)*GPL2(0,0)*pi*ci
     +  + 7.d0/6.d0*HPL1(0)*GPL2(0,1)
     +  + 1.d0/4.d0*HPL1(0)*GPL2(0,1)*pi*ci
     +  - 3.d0/4.d0*HPL1(0)*GPL2(0,1)*u**(-1)
     +  - 1.d0/8.d0*HPL1(0)*GPL2(0,1)*upv**(-2)
     +  - 1.d0/4.d0*HPL1(0)*GPL2(0,1)*upv**(-1)
     +  - 25.d0/8.d0*HPL1(0)*GPL2(0,2)
     +  - 3.d0/2.d0*HPL1(0)*GPL2(0,2)*pi*ci
     +
      col2 = col2 + 3.d0/4.d0*HPL1(0)*GPL2(0,2)*u**(-1)
     +  + 1.d0/8.d0*HPL1(0)*GPL2(0,2)*upv**(-2)
     +  + 1.d0/4.d0*HPL1(0)*GPL2(0,2)*upv**(-1)
     +  + 4.d0/3.d0*HPL1(0)*GPL2(0,3)
     +  + 3.d0/4.d0*HPL1(0)*GPL2(0,3)*pi*ci
     +  + 5.d0/12.d0*HPL1(0)*GPL2(1,0)
     +  + 3.d0/4.d0*HPL1(0)*GPL2(1,0)*pi*ci
     +  + 1.d0/6.d0*HPL1(0)*GPL2(1,1)
     +
      col2 = col2 - 3.d0/4.d0*HPL1(0)*GPL2(1,1)*u**(-1)
     +  + 43.d0/24.d0*HPL1(0)*GPL2(1,2)
     +  + 1.d0/2.d0*HPL1(0)*GPL2(1,2)*pi*ci
     +  + 3.d0/4.d0*HPL1(0)*GPL2(1,2)*u**(-1)
     +  - 25.d0/8.d0*HPL1(0)*GPL2(2,0)
     +  - 3.d0/2.d0*HPL1(0)*GPL2(2,0)*pi*ci
     +  + 4.d0/3.d0*HPL1(0)*GPL2(2,1)
     +  - 1.d0/4.d0*HPL1(0)*GPL2(2,1)*pi*ci
     +
      col2 = col2 - 1.d0/4.d0*HPL1(0)*GPL2(2,1)*u**(-2)*v
     +  + 1.d0/8.d0*HPL1(0)*GPL2(2,1)*u**(-2)*v**2
     +  + 1.d0/8.d0*HPL1(0)*GPL2(2,1)*u**(-2)
     +  - 1.d0/4.d0*HPL1(0)*GPL2(2,1)*u**(-1)*v
     +  + HPL1(0)*GPL2(2,1)*u**(-1)
     +  - 1.d0/8.d0*HPL1(0)*GPL2(2,1)*upv**(-2)
     +  - 1.d0/4.d0*HPL1(0)*GPL2(2,1)*upv**(-1)
     +  - HPL1(0)*GPL2(2,2)
     +
      col2 = col2 - 3.d0/4.d0*HPL1(0)*GPL2(2,2)*u**(-1)
     +  + 5.d0/12.d0*HPL1(0)*GPL2(2,3)
     +  + 1.d0/4.d0*HPL1(0)*GPL2(2,3)*pi*ci
     +  + 43.d0/24.d0*HPL1(0)*GPL2(3,0)
     +  + 7.d0/4.d0*HPL1(0)*GPL2(3,0)*pi*ci
     +  - 1.d0/3.d0*HPL1(0)*GPL2(3,2)
     +  + HPL1(0)*GPL2(3,2)*pi*ci
     +  + 11.d0/12.d0*HPL1(0)*GPL2(3,3)
     +
      col2 = col2 - HPL1(0)*GPL2(3,3)*pi*ci
     +  - 1.d0/2.d0*HPL1(0)*GPL3(0,0,1)
     +  - 1.d0/2.d0*HPL1(0)*GPL3(0,0,2)
     +  + 1.d0/2.d0*HPL1(0)*GPL3(0,0,3)
     +  - 1.d0/4.d0*HPL1(0)*GPL3(0,1,0)
     +  + 1.d0/2.d0*HPL1(0)*GPL3(0,1,2)
     +  - 3.d0/4.d0*HPL1(0)*GPL3(0,2,0)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(0,2,1)
     +
      col2 = col2 - HPL1(0)*GPL3(0,2,2)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(0,2,3)
     +  + 1.d0/2.d0*HPL1(0)*GPL3(0,3,0)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(0,3,2)
     +  - HPL1(0)*GPL3(0,3,3)
     +  + 3.d0/4.d0*HPL1(0)*GPL3(1,0,2)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(1,0,3)
     +  + HPL1(0)*GPL3(1,1,1)
     +
      col2 = col2 - HPL1(0)*GPL3(1,1,2)
     +  + 1.d0/2.d0*HPL1(0)*GPL3(1,2,0)
     +  - HPL1(0)*GPL3(1,2,1)
     +  + HPL1(0)*GPL3(1,2,2)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(1,2,3)
     +  - HPL1(0)*GPL3(2,0,0)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(2,0,1)
     +  - 3.d0/4.d0*HPL1(0)*GPL3(2,0,2)
     +
      col2 = col2 + 1.d0/4.d0*HPL1(0)*GPL3(2,0,3)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(2,1,0)
     +  - HPL1(0)*GPL3(2,1,1)
     +  + 1.d0/2.d0*HPL1(0)*GPL3(2,1,2)
     +  - 1.d0/2.d0*HPL1(0)*GPL3(2,2,0)
     +  + 1.d0/2.d0*HPL1(0)*GPL3(2,2,1)
     +  - 1.d0/2.d0*HPL1(0)*GPL3(2,2,3)
     +  + 1.d0/2.d0*HPL1(0)*GPL3(2,3,0)
     +
      col2 = col2 - 1.d0/4.d0*HPL1(0)*GPL3(2,3,2)
     +  + HPL1(0)*GPL3(3,0,0)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(3,0,1)
     +  + 1.d0/2.d0*HPL1(0)*GPL3(3,0,2)
     +  - HPL1(0)*GPL3(3,0,3)
     +  + 3.d0/4.d0*HPL1(0)*GPL3(3,2,0)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(3,2,1)
     +  - HPL1(0)*GPL3(3,3,0)
     +
      col2 = col2 + HPL1(0)*GPL3(3,3,3)
     +  - 31.d0/24.d0*HPL1(1)
     +  - 1.d0/8.d0*HPL1(1)*pi*u**(-1)*ci
     +  - 3.d0/8.d0*HPL1(1)*pi*u*ci
     +  - 3.d0/8.d0*HPL1(1)*pi*v*ci
     +  + 527.d0/144.d0*HPL1(1)*pi*ci
     +  + 1.d0/16.d0*HPL1(1)*pi**2*upv**(-2)
     +  + 1.d0/8.d0*HPL1(1)*pi**2*upv**(-1)
     +
      col2 = col2 - 17.d0/24.d0*HPL1(1)*pi**2
     +  + 1.d0/2.d0*HPL1(1)*pi**3*ci
     +  - 5.d0/4.d0*HPL1(1)*zeta3
     +  - 1.d0/8.d0*HPL1(1)*u**(-2)*v
     +  + 1.d0/8.d0*HPL1(1)*u**(-2)
     +  - 7.d0/24.d0*HPL1(1)*u**(-1)*v
     +  - 11.d0/48.d0*HPL1(1)*u**(-1)
     +  - 2.d0/3.d0*HPL1(1)*v*omu**(-1)
     +
      col2 = col2 + 527.d0/144.d0*HPL1(1)*GPL1(0)
     +  - 11.d0/24.d0*HPL1(1)*GPL1(0)*pi*ci
     +  + 23.d0/24.d0*HPL1(1)*GPL1(0)*pi**2
     +  - 1.d0/8.d0*HPL1(1)*GPL1(0)*u**(-1)
     +  - 3.d0/8.d0*HPL1(1)*GPL1(0)*u
     +  - 3.d0/8.d0*HPL1(1)*GPL1(0)*v
     +  - 383.d0/144.d0*HPL1(1)*GPL1(1)
     +  + 3.d0/4.d0*HPL1(1)*GPL1(1)*pi*u**(-1)*ci
     +
      col2 = col2 - 7.d0/8.d0*HPL1(1)*GPL1(1)*pi*ci
     +  - 2.d0/3.d0*HPL1(1)*GPL1(1)*pi**2
     +  - 1.d0/8.d0*HPL1(1)*GPL1(1)*u**(-3)*v
     +  + 1.d0/8.d0*HPL1(1)*GPL1(1)*u**(-3)
     +  - 1.d0/2.d0*HPL1(1)*GPL1(1)*u**(-2)*v
     +  - 1.d0/16.d0*HPL1(1)*GPL1(1)*u**(-2)
     +  + 1.d0/8.d0*HPL1(1)*GPL1(1)*u**(-1)*v**(-1)
     +  - 1.d0/8.d0*HPL1(1)*GPL1(1)*u**(-1)*v
     +
      col2 = col2 + 3.d0/8.d0*HPL1(1)*GPL1(1)*u
     +  - 1.d0/8.d0*HPL1(1)*GPL1(1)*v**(-1)*upv**(-1)
     +  + 3.d0/8.d0*HPL1(1)*GPL1(1)*v
     +  - 1.d0/8.d0*HPL1(1)*GPL1(1)*upv**(-1)
     +  - HPL1(1)*GPL1(2)*pi*ci
     +  + 1.d0/3.d0*HPL1(1)*GPL1(2)*pi**2
     +  - 1.d0/3.d0*HPL1(1)*GPL1(3)*pi*ci
     +  - 5.d0/8.d0*HPL1(1)*GPL1(3)*pi**2
     +
      col2 = col2 - 7.d0/4.d0*HPL1(1)*GPL2(0,0)
     +  - HPL1(1)*GPL2(0,0)*pi*ci
     +  + 7.d0/6.d0*HPL1(1)*GPL2(0,1)
     +  + 3.d0/4.d0*HPL1(1)*GPL2(0,1)*pi*ci
     +  - 3.d0/4.d0*HPL1(1)*GPL2(0,1)*u**(-1)
     +  - 1.d0/8.d0*HPL1(1)*GPL2(0,1)*upv**(-2)
     +  - 1.d0/4.d0*HPL1(1)*GPL2(0,1)*upv**(-1)
     +  - HPL1(1)*GPL2(0,2)
     +
      col2 = col2 - HPL1(1)*GPL2(0,2)*pi*ci
     +  + 1.d0/4.d0*HPL1(1)*GPL2(0,3)*pi*ci
     +  + 5.d0/12.d0*HPL1(1)*GPL2(1,0)
     +  + HPL1(1)*GPL2(1,0)*pi*ci
     +  + 1.d0/6.d0*HPL1(1)*GPL2(1,1)
     +  - HPL1(1)*GPL2(1,1)*pi*ci
     +  - 3.d0/4.d0*HPL1(1)*GPL2(1,1)*u**(-1)
     +  + HPL1(1)*GPL2(1,2)
     +
      col2 = col2 + HPL1(1)*GPL2(1,2)*pi*ci
     +  + 3.d0/4.d0*HPL1(1)*GPL2(1,2)*u**(-1)
     +  - HPL1(1)*GPL2(2,0)
     +  - HPL1(1)*GPL2(2,0)*pi*ci
     +  + 4.d0/3.d0*HPL1(1)*GPL2(2,1)
     +  + 1.d0/4.d0*HPL1(1)*GPL2(2,1)*pi*ci
     +  - 1.d0/4.d0*HPL1(1)*GPL2(2,1)*u**(-2)*v
     +  + 1.d0/8.d0*HPL1(1)*GPL2(2,1)*u**(-2)*v**2
     +
      col2 = col2 + 1.d0/8.d0*HPL1(1)*GPL2(2,1)*u**(-2)
     +  - 1.d0/4.d0*HPL1(1)*GPL2(2,1)*u**(-1)*v
     +  + HPL1(1)*GPL2(2,1)*u**(-1)
     +  - 1.d0/8.d0*HPL1(1)*GPL2(2,1)*upv**(-2)
     +  - 1.d0/4.d0*HPL1(1)*GPL2(2,1)*upv**(-1)
     +  - 1.d0/4.d0*HPL1(1)*GPL2(2,3)*pi*ci
     +  - 1.d0/3.d0*HPL1(1)*GPL2(3,0)
     +  + HPL1(1)*GPL2(3,0)*pi*ci
     +
      col2 = col2 - 1.d0/2.d0*HPL1(1)*GPL3(0,0,1)
     +  - HPL1(1)*GPL3(0,0,2)
     +  - 1.d0/4.d0*HPL1(1)*GPL3(0,1,0)
     +  + HPL1(1)*GPL3(0,1,2)
     +  - HPL1(1)*GPL3(0,2,0)
     +  + 1.d0/4.d0*HPL1(1)*GPL3(0,2,1)
     +  + 1.d0/4.d0*HPL1(1)*GPL3(0,3,0)
     +  + HPL1(1)*GPL3(1,0,2)
     +
      col2 = col2 + HPL1(1)*GPL3(1,1,1)
     +  - HPL1(1)*GPL3(1,1,2)
     +  + HPL1(1)*GPL3(1,2,0)
     +  - HPL1(1)*GPL3(1,2,1)
     +  - HPL1(1)*GPL3(2,0,0)
     +  + 1.d0/4.d0*HPL1(1)*GPL3(2,0,1)
     +  + 1.d0/4.d0*HPL1(1)*GPL3(2,1,0)
     +  - HPL1(1)*GPL3(2,1,1)
     +
      col2 = col2 + 1.d0/2.d0*HPL1(1)*GPL3(2,2,1)
     +  - 1.d0/4.d0*HPL1(1)*GPL3(2,3,0)
     +  + HPL1(1)*GPL3(3,0,0)
     +  + 1.d0/4.d0*HPL1(1)*GPL3(3,0,1)
     +  + 1.d0/4.d0*HPL1(1)*GPL3(3,2,1)
     +  - 383.d0/144.d0*HPL2(0,0)
     +  + 35.d0/24.d0*HPL2(0,0)*pi*ci
     +  - 9.d0/8.d0*HPL2(0,0)*pi**2
     +
      col2 = col2 + 3.d0/8.d0*HPL2(0,0)*u
     +  + 3.d0/8.d0*HPL2(0,0)*v
     +  - 1.d0/16.d0*HPL2(0,0)*omumv**(-2)
     +  - 1.d0/4.d0*HPL2(0,0)*omumv**(-1)
     +  + 49.d0/12.d0*HPL2(0,0)*GPL1(0)
     +  + 7.d0/4.d0*HPL2(0,0)*GPL1(0)*pi*ci
     +  - 11.d0/4.d0*HPL2(0,0)*GPL1(1)
     +  - 3.d0/4.d0*HPL2(0,0)*GPL1(1)*pi*ci
     +
      col2 = col2 + 19.d0/6.d0*HPL2(0,0)*GPL1(2)
     +  + HPL2(0,0)*GPL1(2)*pi*ci
     +  - 11.d0/6.d0*HPL2(0,0)*GPL1(3)
     +  - 2*HPL2(0,0)*GPL1(3)*pi*ci
     +  + HPL2(0,0)*GPL2(0,0)
     +  + 3.d0/4.d0*HPL2(0,0)*GPL2(0,2)
     +  - HPL2(0,0)*GPL2(0,3)
     +  - 1.d0/4.d0*HPL2(0,0)*GPL2(1,0)
     +
      col2 = col2 - 1.d0/4.d0*HPL2(0,0)*GPL2(1,2)
     +  + 3.d0/4.d0*HPL2(0,0)*GPL2(2,0)
     +  - 3.d0/2.d0*HPL2(0,0)*GPL2(3,0)
     +  - 1.d0/2.d0*HPL2(0,0)*GPL2(3,2)
     +  + HPL2(0,0)*GPL2(3,3)
     +  + 47.d0/24.d0*HPL2(0,1)*pi*ci
     +  - 5.d0/8.d0*HPL2(0,1)*pi**2
     +  + 1.d0/8.d0*HPL2(0,1)*u**(-1)*v
     +
      col2 = col2 - 1.d0/8.d0*HPL2(0,1)*u**(-1)
     +  + 1.d0/8.d0*HPL2(0,1)*upv**(-1)
     +  + 47.d0/24.d0*HPL2(0,1)*GPL1(0)
     +  + HPL2(0,1)*GPL1(0)*pi*ci
     +  - 47.d0/24.d0*HPL2(0,1)*GPL1(1)
     +  - 3.d0/4.d0*HPL2(0,1)*GPL1(1)*pi*ci
     +  - 1.d0/3.d0*HPL2(0,1)*GPL1(2)
     +  + 1.d0/2.d0*HPL2(0,1)*GPL1(2)*pi*ci
     +
      col2 = col2 + 1.d0/4.d0*HPL2(0,1)*GPL1(2)*u**(-2)*v
     +  - 1.d0/8.d0*HPL2(0,1)*GPL1(2)*u**(-2)*v**2
     +  - 1.d0/8.d0*HPL2(0,1)*GPL1(2)*u**(-2)
     +  + 1.d0/4.d0*HPL2(0,1)*GPL1(2)*u**(-1)*v
     +  - 1.d0/4.d0*HPL2(0,1)*GPL1(2)*u**(-1)
     +  + 1.d0/8.d0*HPL2(0,1)*GPL1(2)*upv**(-2)
     +  + 1.d0/4.d0*HPL2(0,1)*GPL1(2)*upv**(-1)
     +  + 1.d0/3.d0*HPL2(0,1)*GPL1(3)
     +
      col2 = col2 - 3.d0/4.d0*HPL2(0,1)*GPL1(3)*pi*ci
     +  + HPL2(0,1)*GPL2(0,0)
     +  - 1.d0/2.d0*HPL2(0,1)*GPL2(0,1)
     +  + 3.d0/4.d0*HPL2(0,1)*GPL2(0,2)
     +  - 1.d0/4.d0*HPL2(0,1)*GPL2(0,3)
     +  - 3.d0/4.d0*HPL2(0,1)*GPL2(1,0)
     +  + 1.d0/2.d0*HPL2(0,1)*GPL2(2,0)
     +  + 1.d0/2.d0*HPL2(0,1)*GPL2(2,1)
     +
      col2 = col2 - 1.d0/2.d0*HPL2(0,1)*GPL2(2,2)
     +  + 1.d0/4.d0*HPL2(0,1)*GPL2(2,3)
     +  - 3.d0/4.d0*HPL2(0,1)*GPL2(3,0)
     +  - 1.d0/4.d0*HPL2(0,1)*GPL2(3,2)
     +  - 671.d0/144.d0*HPL2(1,0)
     +  + 1.d0/8.d0*HPL2(1,0)*pi*upv**(-2)*ci
     +  + 1.d0/4.d0*HPL2(1,0)*pi*upv**(-1)*ci
     +  + 1.d0/2.d0*HPL2(1,0)*pi*ci
     +
      col2 = col2 - HPL2(1,0)*pi**2
     +  - 1.d0/8.d0*HPL2(1,0)*u**(-1)*v
     +  + 3.d0/8.d0*HPL2(1,0)*u**(-1)
     +  + 3.d0/8.d0*HPL2(1,0)*u
     +  + 3.d0/8.d0*HPL2(1,0)*v
     +  - 1.d0/8.d0*HPL2(1,0)*upv**(-1)
     +  + 1.d0/16.d0*HPL2(1,0)*omumv**(-2)
     +  + 1.d0/4.d0*HPL2(1,0)*omumv**(-1)
     +
      col2 = col2 + 25.d0/8.d0*HPL2(1,0)*GPL1(0)
     +  + 3.d0/2.d0*HPL2(1,0)*GPL1(0)*pi*ci
     +  - 43.d0/24.d0*HPL2(1,0)*GPL1(1)
     +  - 1.d0/2.d0*HPL2(1,0)*GPL1(1)*pi*ci
     +  - 3.d0/4.d0*HPL2(1,0)*GPL1(1)*u**(-1)
     +  + HPL2(1,0)*GPL1(2)
     +  - 1.d0/4.d0*HPL2(1,0)*GPL1(2)*u**(-2)*v
     +  + 1.d0/8.d0*HPL2(1,0)*GPL1(2)*u**(-2)*v**2
     +
      col2 = col2 + 1.d0/8.d0*HPL2(1,0)*GPL1(2)*u**(-2)
     +  - 1.d0/4.d0*HPL2(1,0)*GPL1(2)*u**(-1)*v
     +  + 1.d0/4.d0*HPL2(1,0)*GPL1(2)*u**(-1)
     +  - 1.d0/8.d0*HPL2(1,0)*GPL1(2)*upv**(-2)
     +  - 1.d0/4.d0*HPL2(1,0)*GPL1(2)*upv**(-1)
     +  + 1.d0/3.d0*HPL2(1,0)*GPL1(3)
     +  - HPL2(1,0)*GPL1(3)*pi*ci
     +  + HPL2(1,0)*GPL2(0,0)
     +
      col2 = col2 - 1.d0/2.d0*HPL2(1,0)*GPL2(0,1)
     +  + 1.d0/2.d0*HPL2(1,0)*GPL2(0,2)
     +  - 1.d0/4.d0*HPL2(1,0)*GPL2(0,3)
     +  - 1.d0/2.d0*HPL2(1,0)*GPL2(1,0)
     +  + HPL2(1,0)*GPL2(1,1)
     +  - 5.d0/4.d0*HPL2(1,0)*GPL2(1,2)
     +  + 1.d0/4.d0*HPL2(1,0)*GPL2(2,0)
     +  - 1.d0/2.d0*HPL2(1,0)*GPL2(2,1)
     +
      col2 = col2 + 1.d0/2.d0*HPL2(1,0)*GPL2(2,2)
     +  + 1.d0/4.d0*HPL2(1,0)*GPL2(2,3)
     +  - 3.d0/4.d0*HPL2(1,0)*GPL2(3,0)
     +  + 1.d0/4.d0*HPL2(1,0)*GPL2(3,2)
     +  + HPL2(1,1)*pi*ci
     +  - 1.d0/3.d0*HPL2(1,1)*pi**2
     +  + HPL2(1,1)*GPL1(0)
     +  + HPL2(1,1)*GPL1(0)*pi*ci
     +
      col2 = col2 - HPL2(1,1)*GPL1(1)
     +  - HPL2(1,1)*GPL1(1)*pi*ci
     +  - 3.d0/4.d0*HPL2(1,1)*GPL1(1)*u**(-1)
     +  + HPL2(1,1)*GPL2(0,0)
     +  - HPL2(1,1)*GPL2(0,1)
     +  - HPL2(1,1)*GPL2(1,0)
     +  + HPL2(1,1)*GPL2(1,1)
     +  - 33.d0/8.d0*HPL3(0,0,0)
     +
      col2 = col2 - 2*HPL3(0,0,0)*pi*ci
     +  - 3.d0/2.d0*HPL3(0,0,0)*GPL1(0)
     +  + 1.d0/4.d0*HPL3(0,0,0)*GPL1(1)
     +  - 1.d0/4.d0*HPL3(0,0,0)*GPL1(2)
     +  + 3.d0/2.d0*HPL3(0,0,0)*GPL1(3)
     +  + 1.d0/3.d0*HPL3(0,0,1)
     +  - 3.d0/4.d0*HPL3(0,0,1)*pi*ci
     +  - 3.d0/4.d0*HPL3(0,0,1)*GPL1(0)
     +
      col2 = col2 + 1.d0/2.d0*HPL3(0,0,1)*GPL1(1)
     +  - 1.d0/4.d0*HPL3(0,0,1)*GPL1(2)
     +  + 1.d0/2.d0*HPL3(0,0,1)*GPL1(3)
     +  - 47.d0/24.d0*HPL3(0,1,0)
     +  - HPL3(0,1,0)*pi*ci
     +  - 3.d0/4.d0*HPL3(0,1,0)*GPL1(0)
     +  + 3.d0/4.d0*HPL3(0,1,0)*GPL1(1)
     +  - 1.d0/2.d0*HPL3(0,1,0)*GPL1(2)
     +
      col2 = col2 + 1.d0/2.d0*HPL3(0,1,0)*GPL1(3)
     +  - 19.d0/6.d0*HPL3(1,0,0)
     +  - HPL3(1,0,0)*pi*ci
     +  - 3.d0/4.d0*HPL3(1,0,0)*GPL1(0)
     +  + 1.d0/4.d0*HPL3(1,0,0)*GPL1(1)
     +  + 1.d0/2.d0*HPL3(1,0,0)*GPL1(3)
     +  + 1.d0/3.d0*HPL3(1,0,1)
     +  - 1.d0/2.d0*HPL3(1,0,1)*pi*ci
     +
      col2 = col2 - 1.d0/8.d0*HPL3(1,0,1)*upv**(-2)
     +  - 1.d0/4.d0*HPL3(1,0,1)*upv**(-1)
     +  - 1.d0/2.d0*HPL3(1,0,1)*GPL1(0)
     +  + 1.d0/4.d0*HPL3(1,0,1)*GPL1(2)
     +  + 1.d0/4.d0*HPL3(1,0,1)*GPL1(3)
     +  - HPL3(1,1,0)
     +  + 1.d0/8.d0*HPL3(1,1,0)*upv**(-2)
     +  + 1.d0/4.d0*HPL3(1,1,0)*upv**(-1)
     +
      col2 = col2 - 1.d0/4.d0*HPL3(1,1,0)*GPL1(0)
     +  + 5.d0/4.d0*HPL3(1,1,0)*GPL1(1)
     +  - 3.d0/4.d0*HPL3(1,1,0)*GPL1(2)
     +  - 1.d0/4.d0*HPL3(1,1,0)*GPL1(3)
     +  + 3.d0/2.d0*HPL4(0,0,0,0)
     +  + 1.d0/2.d0*HPL4(0,0,0,1)
     +  + 1.d0/2.d0*HPL4(0,0,1,0)
     +  + 1.d0/2.d0*HPL4(0,1,0,0)
     +
      col2 = col2 + 1.d0/4.d0*HPL4(0,1,0,1)
     +  - 1.d0/4.d0*HPL4(0,1,1,0)
     +  + 1.d0/4.d0*HPL4(1,0,0,0)
     +  + 1.d0/4.d0*HPL4(1,0,0,1)
     +  + 1.d0/2.d0*HPL4(1,0,1,0)
     +  - 1.d0/4.d0*HPL4(1,1,0,1)
     +  + 3.d0/4.d0*HPL4(1,1,1,0)
     +  + 49.d0/24.d0*GPL1(0)
     +
      col2 = col2 + 1.d0/8.d0*GPL1(0)*pi*u**(-1)*ci
     +  + 3.d0/8.d0*GPL1(0)*pi*u*ci
     +  + 3.d0/8.d0*GPL1(0)*pi*v*ci
     +  - 527.d0/144.d0*GPL1(0)*pi*ci
     +  + 31.d0/72.d0*GPL1(0)*pi**2
     +  - 1.d0/3.d0*GPL1(0)*pi**3*ci
     +  + 2.d0/3.d0*GPL1(0)*u**(-1)
     +  + 1.d0/16.d0*GPL1(0)*omumv**(-1)
     +
      col2 = col2 + 1.d0/8.d0*GPL1(1)*pi*u**(-3)*v*ci
     +  - 1.d0/8.d0*GPL1(1)*pi*u**(-3)*ci
     +  + 1.d0/2.d0*GPL1(1)*pi*u**(-2)*v*ci
     +  + 1.d0/16.d0*GPL1(1)*pi*u**(-2)*ci
     +  - 1.d0/8.d0*GPL1(1)*pi*u**(-1)*v**(-1)*ci
     +  + 1.d0/8.d0*GPL1(1)*pi*u**(-1)*v*ci
     +  - 3.d0/8.d0*GPL1(1)*pi*u*ci
     +  + 1.d0/8.d0*GPL1(1)*pi*v**(-1)*upv**(-1)*ci
     +
      col2 = col2 - 3.d0/8.d0*GPL1(1)*pi*v*ci
     +  + 1.d0/8.d0*GPL1(1)*pi*upv**(-1)*ci
     +  + 383.d0/144.d0*GPL1(1)*pi*ci
     +  + 1.d0/8.d0*GPL1(1)*pi**2*u**(-1)
     +  - 95.d0/72.d0*GPL1(1)*pi**2
     +  + 11.d0/24.d0*GPL1(1)*pi**3*ci
     +  - 5.d0/4.d0*GPL1(1)*zeta3
     +  + 31.d0/24.d0*GPL1(2)
     +
      col2 = col2 + 1.d0/8.d0*GPL1(2)*pi*u**(-1)*ci
     +  + 3.d0/8.d0*GPL1(2)*pi*u*ci
     +  + 3.d0/8.d0*GPL1(2)*pi*v*ci
     +  - 527.d0/144.d0*GPL1(2)*pi*ci
     +  - 1.d0/12.d0*GPL1(2)*pi**2*u**(-2)*v
     +  + 1.d0/24.d0*GPL1(2)*pi**2*u**(-2)*v**2
     +  + 1.d0/24.d0*GPL1(2)*pi**2*u**(-2)
     +  - 1.d0/12.d0*GPL1(2)*pi**2*u**(-1)*v
     +
      col2 = col2 + 1.d0/12.d0*GPL1(2)*pi**2*u**(-1)
     +  - 1.d0/16.d0*GPL1(2)*pi**2*upv**(-2)
     +  - 1.d0/8.d0*GPL1(2)*pi**2*upv**(-1)
     +  + 17.d0/24.d0*GPL1(2)*pi**2
     +  - 1.d0/2.d0*GPL1(2)*pi**3*ci
     +  + 5.d0/4.d0*GPL1(2)*zeta3
     +  + 1.d0/8.d0*GPL1(2)*u**(-2)*v
     +  - 1.d0/8.d0*GPL1(2)*u**(-2)
     +
      col2 = col2 + 7.d0/24.d0*GPL1(2)*u**(-1)*v
     +  + 11.d0/48.d0*GPL1(2)*u**(-1)
     +  + 2.d0/3.d0*GPL1(2)*v*omu**(-1)
     +  - 3.d0/8.d0*GPL1(3)*pi*u*ci
     +  - 3.d0/8.d0*GPL1(3)*pi*v*ci
     +  + 1.d0/16.d0*GPL1(3)*pi*omumv**(-2)*ci
     +  + 1.d0/4.d0*GPL1(3)*pi*omumv**(-1)*ci
     +  + 383.d0/144.d0*GPL1(3)*pi*ci
     +
      col2 = col2 - 115.d0/72.d0*GPL1(3)*pi**2
     +  + 3.d0/8.d0*GPL1(3)*pi**3*ci
     +  + 7.d0/4.d0*GPL2(0,0)*pi*ci
     +  - 1.d0/2.d0*GPL2(0,0)*pi**2
     +  + 3.d0/4.d0*GPL2(0,1)*pi*u**(-1)*ci
     +  + 1.d0/8.d0*GPL2(0,1)*pi*upv**(-2)*ci
     +  + 1.d0/4.d0*GPL2(0,1)*pi*upv**(-1)*ci
     +  - 7.d0/6.d0*GPL2(0,1)*pi*ci
     +
      col2 = col2 + 1.d0/4.d0*GPL2(0,1)*pi**2
     +  - 527.d0/144.d0*GPL2(0,2)
     +  + 11.d0/24.d0*GPL2(0,2)*pi*ci
     +  - 23.d0/24.d0*GPL2(0,2)*pi**2
     +  + 1.d0/8.d0*GPL2(0,2)*u**(-1)
     +  + 3.d0/8.d0*GPL2(0,2)*u
     +  + 3.d0/8.d0*GPL2(0,2)*v
     +  - 4.d0/3.d0*GPL2(0,3)*pi*ci
     +
      col2 = col2 + 1.d0/12.d0*GPL2(0,3)*pi**2
     +  - 5.d0/12.d0*GPL2(1,0)*pi*ci
     +  + 7.d0/12.d0*GPL2(1,0)*pi**2
     +  + 3.d0/4.d0*GPL2(1,1)*pi*u**(-1)*ci
     +  - 1.d0/6.d0*GPL2(1,1)*pi*ci
     +  - 1.d0/6.d0*GPL2(1,1)*pi**2
     +  + 383.d0/144.d0*GPL2(1,2)
     +  - 3.d0/4.d0*GPL2(1,2)*pi*u**(-1)*ci
     +
      col2 = col2 + 7.d0/8.d0*GPL2(1,2)*pi*ci
     +  + 2.d0/3.d0*GPL2(1,2)*pi**2
     +  + 1.d0/8.d0*GPL2(1,2)*u**(-3)*v
     +  - 1.d0/8.d0*GPL2(1,2)*u**(-3)
     +  + 1.d0/2.d0*GPL2(1,2)*u**(-2)*v
     +  + 1.d0/16.d0*GPL2(1,2)*u**(-2)
     +  - 1.d0/8.d0*GPL2(1,2)*u**(-1)*v**(-1)
     +  + 1.d0/8.d0*GPL2(1,2)*u**(-1)*v
     +
      col2 = col2 - 3.d0/8.d0*GPL2(1,2)*u
     +  + 1.d0/8.d0*GPL2(1,2)*v**(-1)*upv**(-1)
     +  - 3.d0/8.d0*GPL2(1,2)*v
     +  + 1.d0/8.d0*GPL2(1,2)*upv**(-1)
     +  - 527.d0/144.d0*GPL2(2,0)
     +  + 11.d0/24.d0*GPL2(2,0)*pi*ci
     +  - 23.d0/24.d0*GPL2(2,0)*pi**2
     +  + 1.d0/8.d0*GPL2(2,0)*u**(-1)
     +
      col2 = col2 + 3.d0/8.d0*GPL2(2,0)*u
     +  + 3.d0/8.d0*GPL2(2,0)*v
     +  + 1.d0/4.d0*GPL2(2,1)*pi*u**(-2)*v*ci
     +  - 1.d0/8.d0*GPL2(2,1)*pi*u**(-2)*v**2*ci
     +  - 1.d0/8.d0*GPL2(2,1)*pi*u**(-2)*ci
     +  + 1.d0/4.d0*GPL2(2,1)*pi*u**(-1)*v*ci
     +  - GPL2(2,1)*pi*u**(-1)*ci
     +  + 1.d0/8.d0*GPL2(2,1)*pi*upv**(-2)*ci
     +
      col2 = col2 + 1.d0/4.d0*GPL2(2,1)*pi*upv**(-1)*ci
     +  - 4.d0/3.d0*GPL2(2,1)*pi*ci
     +  - 1.d0/12.d0*GPL2(2,1)*pi**2
     +  + GPL2(2,2)*pi*ci
     +  - 1.d0/3.d0*GPL2(2,2)*pi**2
     +  - 5.d0/12.d0*GPL2(2,3)*pi*ci
     +  + 1.d0/4.d0*GPL2(2,3)*pi**2
     +  + 383.d0/144.d0*GPL2(3,0)
     +
      col2 = col2 + 7.d0/8.d0*GPL2(3,0)*pi*ci
     +  + 7.d0/8.d0*GPL2(3,0)*pi**2
     +  - 3.d0/8.d0*GPL2(3,0)*u
     +  - 3.d0/8.d0*GPL2(3,0)*v
     +  + 1.d0/16.d0*GPL2(3,0)*omumv**(-2)
     +  + 1.d0/4.d0*GPL2(3,0)*omumv**(-1)
     +  + 1.d0/3.d0*GPL2(3,2)*pi*ci
     +  + 5.d0/8.d0*GPL2(3,2)*pi**2
     +
      col2 = col2 - 11.d0/12.d0*GPL2(3,3)*pi*ci
     +  - 1.d0/3.d0*GPL2(3,3)*pi**2
     +  + 1.d0/2.d0*GPL3(0,0,1)*pi*ci
     +  + 7.d0/4.d0*GPL3(0,0,2)
     +  + GPL3(0,0,2)*pi*ci
     +  - 1.d0/2.d0*GPL3(0,0,3)*pi*ci
     +  + 1.d0/4.d0*GPL3(0,1,0)*pi*ci
     +  - 7.d0/6.d0*GPL3(0,1,2)
     +
      col2 = col2 - 3.d0/4.d0*GPL3(0,1,2)*pi*ci
     +  + 3.d0/4.d0*GPL3(0,1,2)*u**(-1)
     +  + 1.d0/8.d0*GPL3(0,1,2)*upv**(-2)
     +  + 1.d0/4.d0*GPL3(0,1,2)*upv**(-1)
     +  + 7.d0/4.d0*GPL3(0,2,0)
     +  + GPL3(0,2,0)*pi*ci
     +  - 1.d0/4.d0*GPL3(0,2,1)*pi*ci
     +  + GPL3(0,2,2)
     +
      col2 = col2 + GPL3(0,2,2)*pi*ci
     +  - 1.d0/4.d0*GPL3(0,2,3)*pi*ci
     +  - 4.d0/3.d0*GPL3(0,3,0)
     +  - 1.d0/4.d0*GPL3(0,3,0)*pi*ci
     +  - 1.d0/4.d0*GPL3(0,3,2)*pi*ci
     +  + GPL3(0,3,3)*pi*ci
     +  - 5.d0/12.d0*GPL3(1,0,2)
     +  - GPL3(1,0,2)*pi*ci
     +
      col2 = col2 - 1.d0/4.d0*GPL3(1,0,3)*pi*ci
     +  - GPL3(1,1,1)*pi*ci
     +  - 1.d0/6.d0*GPL3(1,1,2)
     +  + GPL3(1,1,2)*pi*ci
     +  + 3.d0/4.d0*GPL3(1,1,2)*u**(-1)
     +  - 5.d0/12.d0*GPL3(1,2,0)
     +  - GPL3(1,2,0)*pi*ci
     +  + GPL3(1,2,1)*pi*ci
     +
      col2 = col2 - GPL3(1,2,2)
     +  - GPL3(1,2,2)*pi*ci
     +  - 3.d0/4.d0*GPL3(1,2,2)*u**(-1)
     +  - 1.d0/4.d0*GPL3(1,2,3)*pi*ci
     +  + 7.d0/4.d0*GPL3(2,0,0)
     +  + GPL3(2,0,0)*pi*ci
     +  - 1.d0/4.d0*GPL3(2,0,1)*pi*ci
     +  + GPL3(2,0,2)
     +
      col2 = col2 + GPL3(2,0,2)*pi*ci
     +  - 1.d0/4.d0*GPL3(2,0,3)*pi*ci
     +  - 1.d0/4.d0*GPL3(2,1,0)*pi*ci
     +  + GPL3(2,1,1)*pi*ci
     +  - 4.d0/3.d0*GPL3(2,1,2)
     +  - 1.d0/4.d0*GPL3(2,1,2)*pi*ci
     +  + 1.d0/4.d0*GPL3(2,1,2)*u**(-2)*v
     +  - 1.d0/8.d0*GPL3(2,1,2)*u**(-2)*v**2
     +
      col2 = col2 - 1.d0/8.d0*GPL3(2,1,2)*u**(-2)
     +  + 1.d0/4.d0*GPL3(2,1,2)*u**(-1)*v
     +  - GPL3(2,1,2)*u**(-1)
     +  + 1.d0/8.d0*GPL3(2,1,2)*upv**(-2)
     +  + 1.d0/4.d0*GPL3(2,1,2)*upv**(-1)
     +  + GPL3(2,2,0)
     +  + GPL3(2,2,0)*pi*ci
     +  - 1.d0/2.d0*GPL3(2,2,1)*pi*ci
     +
      col2 = col2 + 1.d0/2.d0*GPL3(2,2,3)*pi*ci
     +  - 5.d0/12.d0*GPL3(2,3,0)
     +  - 3.d0/4.d0*GPL3(2,3,0)*pi*ci
     +  + 1.d0/4.d0*GPL3(2,3,2)*pi*ci
     +  - 7.d0/4.d0*GPL3(3,0,0)
     +  - GPL3(3,0,0)*pi*ci
     +  - 1.d0/4.d0*GPL3(3,0,1)*pi*ci
     +  + 1.d0/3.d0*GPL3(3,0,2)
     +
      col2 = col2 - GPL3(3,0,2)*pi*ci
     +  + GPL3(3,0,3)*pi*ci
     +  + 1.d0/3.d0*GPL3(3,2,0)
     +  - GPL3(3,2,0)*pi*ci
     +  - 1.d0/4.d0*GPL3(3,2,1)*pi*ci
     +  - 11.d0/12.d0*GPL3(3,3,0)
     +  + GPL3(3,3,0)*pi*ci
     +  - GPL3(3,3,3)*pi*ci
     +
      col2 = col2 + 1.d0/2.d0*GPL4(0,0,1,2)
     +  + GPL4(0,0,2,2)
     +  - 1.d0/2.d0*GPL4(0,0,3,0)
     +  + 1.d0/4.d0*GPL4(0,1,0,2)
     +  + 1.d0/4.d0*GPL4(0,1,2,0)
     +  - GPL4(0,1,2,2)
     +  + GPL4(0,2,0,2)
     +  - 1.d0/4.d0*GPL4(0,2,1,2)
     +
      col2 = col2 + GPL4(0,2,2,0)
     +  - 1.d0/4.d0*GPL4(0,2,3,0)
     +  - 1.d0/4.d0*GPL4(0,3,0,2)
     +  - 1.d0/4.d0*GPL4(0,3,2,0)
     +  + GPL4(0,3,3,0)
     +  - GPL4(1,0,2,2)
     +  - 1.d0/4.d0*GPL4(1,0,3,0)
     +  - GPL4(1,1,1,2)
     +
      col2 = col2 + GPL4(1,1,2,2)
     +  - GPL4(1,2,0,2)
     +  + GPL4(1,2,1,2)
     +  - GPL4(1,2,2,0)
     +  - 1.d0/4.d0*GPL4(1,2,3,0)
     +  + GPL4(2,0,0,2)
     +  - 1.d0/4.d0*GPL4(2,0,1,2)
     +  + GPL4(2,0,2,0)
     +
      col2 = col2 - 1.d0/4.d0*GPL4(2,0,3,0)
     +  - 1.d0/4.d0*GPL4(2,1,0,2)
     +  + GPL4(2,1,1,2)
     +  - 1.d0/4.d0*GPL4(2,1,2,0)
     +  + GPL4(2,2,0,0)
     +  - 1.d0/2.d0*GPL4(2,2,1,2)
     +  + 1.d0/2.d0*GPL4(2,2,3,0)
     +  - GPL4(2,3,0,0)
     +
      col2 = col2 + 1.d0/4.d0*GPL4(2,3,0,2)
     +  + 1.d0/4.d0*GPL4(2,3,2,0)
     +  - GPL4(3,0,0,2)
     +  - 1.d0/4.d0*GPL4(3,0,1,2)
     +  - GPL4(3,0,2,0)
     +  + GPL4(3,0,3,0)
     +  - GPL4(3,2,0,0)
     +  - 1.d0/4.d0*GPL4(3,2,1,2)
     +
      col2 = col2 + GPL4(3,3,0,0)
     +  - GPL4(3,3,3,0)
     +

      col3 =
     +  - 255.d0/128.d0
     +  + 5.d0/2.d0*pi*zeta3*ci
     +  - 5.d0/8.d0*pi*u**(-2)*v*ci
     +  + 5.d0/8.d0*pi*u**(-2)*ci
     +  - 11.d0/16.d0*pi*u**(-1)*v*ci
     +  + 1.d0/8.d0*pi*u**(-1)*ci
     +  - 17.d0/16.d0*pi*v*omu**(-1)*ci
     +
      col3 = col3 - 5.d0/16.d0*pi*omumv**(-1)*ci
     +  + 3.d0/32.d0*pi*ci
     +  + 1.d0/12.d0*pi**2*u**(-1)*v
     +  + 1.d0/48.d0*pi**2*u**(-1)*omv**(-1)
     +  + 1.d0/24.d0*pi**2*u**(-1)
     +  + 1.d0/4.d0*pi**2*u
     +  + 1.d0/4.d0*pi**2*v
     +  - 1.d0/16.d0*pi**2*upv**(-1)
     +
      col3 = col3 + 1.d0/48.d0*pi**2*omv**(-1)*omumv**(-1)
     +  - 5.d0/96.d0*pi**2*omumv**(-2)
     +  + 1.d0/6.d0*pi**2*omumv**(-1)
     +  + 1.d0/32.d0*pi**2
     +  - 1.d0/8.d0*pi**3*ci
     +  + 7.d0/48.d0*pi**4
     +  + 3.d0/2.d0*zeta3
     +  - 5.d0/8.d0*u**(-1)
     +
      col3 = col3 + 3.d0/32.d0*HPL1(0)
     +  + 1.d0/8.d0*HPL1(0)*pi*u**(-1)*v*ci
     +  + 1.d0/8.d0*HPL1(0)*pi*u**(-1)*omv**(-1)*ci
     +  + 1.d0/8.d0*HPL1(0)*pi*u**(-1)*ci
     +  + 3.d0/8.d0*HPL1(0)*pi*u*ci
     +  + 3.d0/8.d0*HPL1(0)*pi*v*ci
     +  - 1.d0/8.d0*HPL1(0)*pi*upv**(-1)*ci
     +  + 1.d0/8.d0*HPL1(0)*pi*omv**(-1)*omumv**(-1)*ci
     +
      col3 = col3 - 5.d0/16.d0*HPL1(0)*pi*omumv**(-2)*ci
     +  + HPL1(0)*pi*omumv**(-1)*ci
     +  + 1.d0/2.d0*HPL1(0)*pi*ci
     +  - 1.d0/8.d0*HPL1(0)*pi**2
     +  + 1.d0/24.d0*HPL1(0)*pi**3*ci
     +  + 2*HPL1(0)*zeta3
     +  + 5.d0/8.d0*HPL1(0)*u**(-2)*v
     +  + 11.d0/16.d0*HPL1(0)*u**(-1)*v
     +
      col3 = col3 - 5.d0/16.d0*HPL1(0)*u**(-1)*omv**(-1)
     +  + 5.d0/16.d0*HPL1(0)*u**(-1)
     +  + 17.d0/16.d0*HPL1(0)*v*omu**(-1)
     +  - 5.d0/16.d0*HPL1(0)*omv**(-1)*omumv**(-1)
     +  + 5.d0/16.d0*HPL1(0)*omumv**(-1)
     +  + 1.d0/2.d0*HPL1(0)*GPL1(0)
     +  + 1.d0/24.d0*HPL1(0)*GPL1(0)*pi**2
     +  + 1.d0/8.d0*HPL1(0)*GPL1(0)*u**(-1)*v
     +
      col3 = col3 + 1.d0/8.d0*HPL1(0)*GPL1(0)*u**(-1)*omv**(-1)
     +  + 3.d0/8.d0*HPL1(0)*GPL1(0)*u
     +  + 3.d0/8.d0*HPL1(0)*GPL1(0)*v
     +  + 1.d0/8.d0*HPL1(0)*GPL1(0)*omv**(-1)*omumv**(-1)
     +  - 5.d0/16.d0*HPL1(0)*GPL1(0)*omumv**(-2)
     +  + HPL1(0)*GPL1(0)*omumv**(-1)
     +  - 1.d0/2.d0*HPL1(0)*GPL1(1)
     +  + 1.d0/8.d0*HPL1(0)*GPL1(1)*pi*u**(-2)*ci
     +
      col3 = col3 + 1.d0/4.d0*HPL1(0)*GPL1(1)*pi*u**(-1)*ci
     +  + 1.d0/24.d0*HPL1(0)*GPL1(1)*pi**2
     +  + 5.d0/8.d0*HPL1(0)*GPL1(1)*u**(-3)*v
     +  - 5.d0/8.d0*HPL1(0)*GPL1(1)*u**(-3)
     +  + 1.d0/4.d0*HPL1(0)*GPL1(1)*u**(-2)*v
     +  + 7.d0/16.d0*HPL1(0)*GPL1(1)*u**(-2)
     +  + 1.d0/8.d0*HPL1(0)*GPL1(1)*u**(-1)*v**(-1)
     +  - 1.d0/8.d0*HPL1(0)*GPL1(1)*u**(-1)*v
     +
      col3 = col3 - 3.d0/4.d0*HPL1(0)*GPL1(1)*u**(-1)
     +  - 3.d0/8.d0*HPL1(0)*GPL1(1)*u
     +  - 1.d0/8.d0*HPL1(0)*GPL1(1)*v**(-1)*upv**(-1)
     +  - 3.d0/8.d0*HPL1(0)*GPL1(1)*v
     +  - 1.d0/8.d0*HPL1(0)*GPL1(1)*upv**(-1)
     +  + 1.d0/2.d0*HPL1(0)*GPL1(2)
     +  - 1.d0/8.d0*HPL1(0)*GPL1(2)*pi*upv**(-2)*ci
     +  - 1.d0/4.d0*HPL1(0)*GPL1(2)*pi*upv**(-1)*ci
     +
      col3 = col3 + 3.d0/8.d0*HPL1(0)*GPL1(2)*pi*ci
     +  + 1.d0/8.d0*HPL1(0)*GPL1(2)*pi**2
     +  - 5.d0/8.d0*HPL1(0)*GPL1(2)*u**(-3)*v
     +  + 5.d0/8.d0*HPL1(0)*GPL1(2)*u**(-3)
     +  - 1.d0/4.d0*HPL1(0)*GPL1(2)*u**(-2)*v
     +  - 7.d0/16.d0*HPL1(0)*GPL1(2)*u**(-2)
     +  - 1.d0/8.d0*HPL1(0)*GPL1(2)*u**(-1)*v**(-1)
     +  + 1.d0/8.d0*HPL1(0)*GPL1(2)*u**(-1)*v
     +
      col3 = col3 + 7.d0/8.d0*HPL1(0)*GPL1(2)*u**(-1)
     +  + 3.d0/8.d0*HPL1(0)*GPL1(2)*u
     +  + 1.d0/8.d0*HPL1(0)*GPL1(2)*v**(-1)*upv**(-1)
     +  + 3.d0/8.d0*HPL1(0)*GPL1(2)*v
     +  - 1.d0/2.d0*HPL1(0)*GPL1(3)
     +  + 5.d0/24.d0*HPL1(0)*GPL1(3)*pi**2
     +  - 1.d0/8.d0*HPL1(0)*GPL1(3)*u**(-1)*v
     +  - 1.d0/8.d0*HPL1(0)*GPL1(3)*u**(-1)*omv**(-1)
     +
      col3 = col3 - 3.d0/8.d0*HPL1(0)*GPL1(3)*u
     +  - 3.d0/8.d0*HPL1(0)*GPL1(3)*v
     +  - 1.d0/8.d0*HPL1(0)*GPL1(3)*omv**(-1)*omumv**(-1)
     +  + 5.d0/16.d0*HPL1(0)*GPL1(3)*omumv**(-2)
     +  - HPL1(0)*GPL1(3)*omumv**(-1)
     +  - 3.d0/8.d0*HPL1(0)*GPL2(0,1)
     +  + 1.d0/4.d0*HPL1(0)*GPL2(0,1)*pi*ci
     +  - 1.d0/8.d0*HPL1(0)*GPL2(0,1)*u**(-2)
     +
      col3 = col3 - 1.d0/4.d0*HPL1(0)*GPL2(0,1)*u**(-1)
     +  - 1.d0/8.d0*HPL1(0)*GPL2(0,1)*upv**(-2)
     +  - 1.d0/4.d0*HPL1(0)*GPL2(0,1)*upv**(-1)
     +  + 3.d0/8.d0*HPL1(0)*GPL2(0,2)
     +  + 1.d0/8.d0*HPL1(0)*GPL2(0,2)*u**(-2)
     +  + 1.d0/4.d0*HPL1(0)*GPL2(0,2)*u**(-1)
     +  + 1.d0/8.d0*HPL1(0)*GPL2(0,2)*upv**(-2)
     +  + 1.d0/4.d0*HPL1(0)*GPL2(0,2)*upv**(-1)
     +
      col3 = col3 - 1.d0/4.d0*HPL1(0)*GPL2(0,3)*pi*ci
     +  + 3.d0/8.d0*HPL1(0)*GPL2(1,1)
     +  + 1.d0/8.d0*HPL1(0)*GPL2(1,1)*u**(-2)
     +  + 1.d0/4.d0*HPL1(0)*GPL2(1,1)*u**(-1)
     +  - 3.d0/8.d0*HPL1(0)*GPL2(1,2)
     +  + 3.d0/8.d0*HPL1(0)*GPL2(2,0)
     +  - 3.d0/8.d0*HPL1(0)*GPL2(2,1)
     +  + 1.d0/4.d0*HPL1(0)*GPL2(2,1)*pi*ci
     +
      col3 = col3 + 1.d0/8.d0*HPL1(0)*GPL2(2,1)*u**(-2)
     +  + 1.d0/4.d0*HPL1(0)*GPL2(2,1)*u**(-1)
     +  - 1.d0/8.d0*HPL1(0)*GPL2(2,1)*upv**(-2)
     +  - 1.d0/4.d0*HPL1(0)*GPL2(2,1)*upv**(-1)
     +  + 3.d0/8.d0*HPL1(0)*GPL2(2,2)
     +  - 3.d0/4.d0*HPL1(0)*GPL2(2,2)*pi*ci
     +  - 1.d0/8.d0*HPL1(0)*GPL2(2,2)*u**(-2)
     +  - 1.d0/4.d0*HPL1(0)*GPL2(2,2)*u**(-1)
     +
      col3 = col3 - 3.d0/8.d0*HPL1(0)*GPL2(2,3)
     +  - 1.d0/2.d0*HPL1(0)*GPL2(3,0)*pi*ci
     +  + 1.d0/2.d0*HPL1(0)*GPL2(3,2)*pi*ci
     +  + 1.d0/2.d0*HPL1(0)*GPL2(3,3)*pi*ci
     +  + 1.d0/4.d0*HPL1(0)*GPL3(0,0,1)
     +  - 1.d0/4.d0*HPL1(0)*GPL3(0,0,2)
     +  - 1.d0/2.d0*HPL1(0)*GPL3(0,0,3)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(0,1,0)
     +
      col3 = col3 + 1.d0/4.d0*HPL1(0)*GPL3(0,1,1)
     +  - 1.d0/4.d0*HPL1(0)*GPL3(0,1,2)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(0,2,0)
     +  - 1.d0/2.d0*HPL1(0)*GPL3(0,2,1)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(0,2,2)
     +  - 1.d0/2.d0*HPL1(0)*GPL3(0,2,3)
     +  - 1.d0/4.d0*HPL1(0)*GPL3(0,3,0)
     +  + 1.d0/2.d0*HPL1(0)*GPL3(0,3,3)
     +
      col3 = col3 - 1.d0/4.d0*HPL1(0)*GPL3(1,0,1)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(1,0,2)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(1,0,3)
     +  - 1.d0/2.d0*HPL1(0)*GPL3(1,1,1)
     +  + 1.d0/2.d0*HPL1(0)*GPL3(1,1,2)
     +  - 1.d0/4.d0*HPL1(0)*GPL3(1,2,0)
     +  + 3.d0/4.d0*HPL1(0)*GPL3(1,2,1)
     +  - 1.d0/2.d0*HPL1(0)*GPL3(1,2,2)
     +
      col3 = col3 + 1.d0/4.d0*HPL1(0)*GPL3(1,2,3)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(2,0,0)
     +  - 1.d0/2.d0*HPL1(0)*GPL3(2,0,1)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(2,0,2)
     +  - 1.d0/2.d0*HPL1(0)*GPL3(2,0,3)
     +  + 1.d0/2.d0*HPL1(0)*GPL3(2,1,1)
     +  - 1.d0/4.d0*HPL1(0)*GPL3(2,1,2)
     +  - 1.d0/4.d0*HPL1(0)*GPL3(2,2,0)
     +
      col3 = col3 - 1.d0/2.d0*HPL1(0)*GPL3(2,2,1)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(2,2,3)
     +  - 1.d0/4.d0*HPL1(0)*GPL3(2,3,0)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(2,3,2)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(2,3,3)
     +  - 1.d0/2.d0*HPL1(0)*GPL3(3,0,0)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(3,0,1)
     +  - 1.d0/4.d0*HPL1(0)*GPL3(3,0,2)
     +
      col3 = col3 + 3.d0/4.d0*HPL1(0)*GPL3(3,0,3)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(3,2,0)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(3,2,1)
     +  - 1.d0/4.d0*HPL1(0)*GPL3(3,2,3)
     +  + 1.d0/2.d0*HPL1(0)*GPL3(3,3,0)
     +  - 1.d0/2.d0*HPL1(0)*GPL3(3,3,3)
     +  - 3.d0/8.d0*HPL1(1)
     +  + 1.d0/8.d0*HPL1(1)*pi*u**(-1)*v*ci
     +
      col3 = col3 + 3.d0/8.d0*HPL1(1)*pi*u*ci
     +  + 3.d0/8.d0*HPL1(1)*pi*v*ci
     +  + 1.d0/2.d0*HPL1(1)*pi*ci
     +  + 1.d0/16.d0*HPL1(1)*pi**2*upv**(-2)
     +  + 1.d0/8.d0*HPL1(1)*pi**2*upv**(-1)
     +  - 1.d0/4.d0*HPL1(1)*pi**2
     +  + 5.d0/24.d0*HPL1(1)*pi**3*ci
     +  + HPL1(1)*zeta3
     +
      col3 = col3 + 5.d0/8.d0*HPL1(1)*u**(-2)*v
     +  - 5.d0/8.d0*HPL1(1)*u**(-2)
     +  + 11.d0/16.d0*HPL1(1)*u**(-1)*v
     +  - 1.d0/8.d0*HPL1(1)*u**(-1)
     +  + 17.d0/16.d0*HPL1(1)*v*omu**(-1)
     +  + 1.d0/2.d0*HPL1(1)*GPL1(0)
     +  + 3.d0/8.d0*HPL1(1)*GPL1(0)*pi*ci
     +  + 3.d0/8.d0*HPL1(1)*GPL1(0)*pi**2
     +
      col3 = col3 + 1.d0/8.d0*HPL1(1)*GPL1(0)*u**(-1)*v
     +  - 1.d0/8.d0*HPL1(1)*GPL1(0)*u**(-1)
     +  + 3.d0/8.d0*HPL1(1)*GPL1(0)*u
     +  + 3.d0/8.d0*HPL1(1)*GPL1(0)*v
     +  - 1.d0/2.d0*HPL1(1)*GPL1(1)
     +  + 1.d0/8.d0*HPL1(1)*GPL1(1)*pi*u**(-2)*ci
     +  + 1.d0/4.d0*HPL1(1)*GPL1(1)*pi*u**(-1)*ci
     +  - 3.d0/8.d0*HPL1(1)*GPL1(1)*pi*ci
     +
      col3 = col3 - 1.d0/8.d0*HPL1(1)*GPL1(1)*pi**2
     +  + 5.d0/8.d0*HPL1(1)*GPL1(1)*u**(-3)*v
     +  - 5.d0/8.d0*HPL1(1)*GPL1(1)*u**(-3)
     +  + 1.d0/4.d0*HPL1(1)*GPL1(1)*u**(-2)*v
     +  + 7.d0/16.d0*HPL1(1)*GPL1(1)*u**(-2)
     +  + 1.d0/8.d0*HPL1(1)*GPL1(1)*u**(-1)*v**(-1)
     +  - 1.d0/8.d0*HPL1(1)*GPL1(1)*u**(-1)*v
     +  - 3.d0/4.d0*HPL1(1)*GPL1(1)*u**(-1)
     +
      col3 = col3 - 3.d0/8.d0*HPL1(1)*GPL1(1)*u
     +  - 1.d0/8.d0*HPL1(1)*GPL1(1)*v**(-1)*upv**(-1)
     +  - 3.d0/8.d0*HPL1(1)*GPL1(1)*v
     +  - 1.d0/8.d0*HPL1(1)*GPL1(1)*upv**(-1)
     +  + 3.d0/8.d0*HPL1(1)*GPL1(2)*pi*ci
     +  + 1.d0/3.d0*HPL1(1)*GPL1(2)*pi**2
     +  + 1.d0/8.d0*HPL1(1)*GPL1(2)*u**(-1)
     +  - 1.d0/6.d0*HPL1(1)*GPL1(3)*pi**2
     +
      col3 = col3 - 1.d0/4.d0*HPL1(1)*GPL2(0,0)*pi*ci
     +  - 3.d0/8.d0*HPL1(1)*GPL2(0,1)
     +  - 1.d0/8.d0*HPL1(1)*GPL2(0,1)*u**(-2)
     +  - 1.d0/4.d0*HPL1(1)*GPL2(0,1)*u**(-1)
     +  - 1.d0/8.d0*HPL1(1)*GPL2(0,1)*upv**(-2)
     +  - 1.d0/4.d0*HPL1(1)*GPL2(0,1)*upv**(-1)
     +  + 3.d0/8.d0*HPL1(1)*GPL2(0,2)
     +  - 1.d0/4.d0*HPL1(1)*GPL2(0,2)*pi*ci
     +
      col3 = col3 + 3.d0/8.d0*HPL1(1)*GPL2(1,1)
     +  + 1.d0/2.d0*HPL1(1)*GPL2(1,1)*pi*ci
     +  + 1.d0/8.d0*HPL1(1)*GPL2(1,1)*u**(-2)
     +  + 1.d0/4.d0*HPL1(1)*GPL2(1,1)*u**(-1)
     +  - 3.d0/8.d0*HPL1(1)*GPL2(1,2)
     +  + 1.d0/8.d0*HPL1(1)*GPL2(1,2)*u**(-2)
     +  + 1.d0/4.d0*HPL1(1)*GPL2(1,2)*u**(-1)
     +  + 3.d0/8.d0*HPL1(1)*GPL2(2,0)
     +
      col3 = col3 - 1.d0/4.d0*HPL1(1)*GPL2(2,0)*pi*ci
     +  - 3.d0/8.d0*HPL1(1)*GPL2(2,1)
     +  + 1.d0/8.d0*HPL1(1)*GPL2(2,1)*u**(-2)
     +  + 1.d0/4.d0*HPL1(1)*GPL2(2,1)*u**(-1)
     +  - 1.d0/8.d0*HPL1(1)*GPL2(2,1)*upv**(-2)
     +  - 1.d0/4.d0*HPL1(1)*GPL2(2,1)*upv**(-1)
     +  + 1.d0/4.d0*HPL1(1)*GPL2(2,3)*pi*ci
     +  + 1.d0/4.d0*HPL1(1)*GPL3(0,0,1)
     +
      col3 = col3 - 1.d0/4.d0*HPL1(1)*GPL3(0,0,2)
     +  + 1.d0/4.d0*HPL1(1)*GPL3(0,1,0)
     +  + 1.d0/4.d0*HPL1(1)*GPL3(0,1,1)
     +  - 1.d0/4.d0*HPL1(1)*GPL3(0,1,2)
     +  - 1.d0/4.d0*HPL1(1)*GPL3(0,2,0)
     +  - 1.d0/2.d0*HPL1(1)*GPL3(0,2,1)
     +  - 1.d0/4.d0*HPL1(1)*GPL3(1,0,1)
     +  - 1.d0/2.d0*HPL1(1)*GPL3(1,1,1)
     +
      col3 = col3 + 1.d0/2.d0*HPL1(1)*GPL3(1,1,2)
     +  + 3.d0/4.d0*HPL1(1)*GPL3(1,2,1)
     +  - 1.d0/4.d0*HPL1(1)*GPL3(2,0,0)
     +  - 1.d0/2.d0*HPL1(1)*GPL3(2,0,1)
     +  + 1.d0/2.d0*HPL1(1)*GPL3(2,1,1)
     +  - 1.d0/2.d0*HPL1(1)*GPL3(2,2,1)
     +  + 1.d0/4.d0*HPL1(1)*GPL3(2,3,0)
     +  + 1.d0/4.d0*HPL1(1)*GPL3(3,0,1)
     +
      col3 = col3 + 1.d0/4.d0*HPL1(1)*GPL3(3,2,1)
     +  - 1.d0/2.d0*HPL2(0,0)
     +  + 5.d0/24.d0*HPL2(0,0)*pi**2
     +  - 1.d0/8.d0*HPL2(0,0)*u**(-1)*v
     +  - 1.d0/8.d0*HPL2(0,0)*u**(-1)*omv**(-1)
     +  - 3.d0/8.d0*HPL2(0,0)*u
     +  - 3.d0/8.d0*HPL2(0,0)*v
     +  - 1.d0/8.d0*HPL2(0,0)*omv**(-1)*omumv**(-1)
     +
      col3 = col3 + 5.d0/16.d0*HPL2(0,0)*omumv**(-2)
     +  - HPL2(0,0)*omumv**(-1)
     +  - 1.d0/2.d0*HPL2(0,0)*GPL1(0)*pi*ci
     +  + 1.d0/4.d0*HPL2(0,0)*GPL1(1)*pi*ci
     +  - 3.d0/8.d0*HPL2(0,0)*GPL1(2)
     +  - 1.d0/4.d0*HPL2(0,0)*GPL1(2)*pi*ci
     +  + 3.d0/4.d0*HPL2(0,0)*GPL1(3)*pi*ci
     +  - 1.d0/2.d0*HPL2(0,0)*GPL2(0,0)
     +
      col3 = col3 - 1.d0/2.d0*HPL2(0,0)*GPL2(0,2)
     +  + 1.d0/2.d0*HPL2(0,0)*GPL2(0,3)
     +  + 1.d0/4.d0*HPL2(0,0)*GPL2(1,0)
     +  + 1.d0/4.d0*HPL2(0,0)*GPL2(1,2)
     +  - 1.d0/2.d0*HPL2(0,0)*GPL2(2,0)
     +  + 1.d0/4.d0*HPL2(0,0)*GPL2(2,2)
     +  + 1.d0/4.d0*HPL2(0,0)*GPL2(2,3)
     +  + 3.d0/4.d0*HPL2(0,0)*GPL2(3,0)
     +
      col3 = col3 - 1.d0/4.d0*HPL2(0,0)*GPL2(3,2)
     +  - 1.d0/2.d0*HPL2(0,0)*GPL2(3,3)
     +  - 1.d0/6.d0*HPL2(0,1)*pi**2
     +  - 1.d0/8.d0*HPL2(0,1)*u**(-1)
     +  + 1.d0/8.d0*HPL2(0,1)*upv**(-1)
     +  - 1.d0/8.d0*HPL2(0,1)*GPL1(1)*u**(-2)
     +  - 1.d0/4.d0*HPL2(0,1)*GPL1(1)*u**(-1)
     +  + 1.d0/4.d0*HPL2(0,1)*GPL1(2)*pi*ci
     +
      col3 = col3 + 1.d0/8.d0*HPL2(0,1)*GPL1(2)*upv**(-2)
     +  + 1.d0/4.d0*HPL2(0,1)*GPL1(2)*upv**(-1)
     +  + 1.d0/4.d0*HPL2(0,1)*GPL2(0,2)
     +  - 1.d0/4.d0*HPL2(0,1)*GPL2(1,2)
     +  + 1.d0/4.d0*HPL2(0,1)*GPL2(2,0)
     +  - 1.d0/4.d0*HPL2(0,1)*GPL2(2,1)
     +  + 1.d0/2.d0*HPL2(0,1)*GPL2(2,2)
     +  - 1.d0/4.d0*HPL2(0,1)*GPL2(2,3)
     +
      col3 = col3 - 1.d0/4.d0*HPL2(0,1)*GPL2(3,2)
     +  - 1.d0/2.d0*HPL2(1,0)
     +  + 1.d0/8.d0*HPL2(1,0)*pi*upv**(-2)*ci
     +  + 1.d0/4.d0*HPL2(1,0)*pi*upv**(-1)*ci
     +  - 3.d0/8.d0*HPL2(1,0)*pi*ci
     +  - 1.d0/8.d0*HPL2(1,0)*pi**2
     +  - 1.d0/8.d0*HPL2(1,0)*u**(-1)*v
     +  + 1.d0/8.d0*HPL2(1,0)*u**(-1)*omv**(-1)
     +
      col3 = col3 + 1.d0/4.d0*HPL2(1,0)*u**(-1)
     +  - 3.d0/8.d0*HPL2(1,0)*u
     +  - 3.d0/8.d0*HPL2(1,0)*v
     +  - 1.d0/8.d0*HPL2(1,0)*upv**(-1)
     +  + 1.d0/8.d0*HPL2(1,0)*omv**(-1)*omumv**(-1)
     +  - 5.d0/16.d0*HPL2(1,0)*omumv**(-2)
     +  + HPL2(1,0)*omumv**(-1)
     +  - 3.d0/8.d0*HPL2(1,0)*GPL1(0)
     +
      col3 = col3 + 3.d0/8.d0*HPL2(1,0)*GPL1(1)
     +  - 3.d0/8.d0*HPL2(1,0)*GPL1(2)
     +  + 3.d0/4.d0*HPL2(1,0)*GPL1(2)*pi*ci
     +  - 1.d0/8.d0*HPL2(1,0)*GPL1(2)*upv**(-2)
     +  - 1.d0/4.d0*HPL2(1,0)*GPL1(2)*upv**(-1)
     +  - 1.d0/2.d0*HPL2(1,0)*GPL1(3)*pi*ci
     +  - 1.d0/4.d0*HPL2(1,0)*GPL2(0,0)
     +  + 1.d0/4.d0*HPL2(1,0)*GPL2(0,1)
     +
      col3 = col3 + 1.d0/4.d0*HPL2(1,0)*GPL2(0,2)
     +  + 1.d0/4.d0*HPL2(1,0)*GPL2(1,0)
     +  - 1.d0/2.d0*HPL2(1,0)*GPL2(1,1)
     +  + 1.d0/4.d0*HPL2(1,0)*GPL2(2,0)
     +  + 1.d0/4.d0*HPL2(1,0)*GPL2(2,1)
     +  - 1.d0/2.d0*HPL2(1,0)*GPL2(2,2)
     +  - 1.d0/4.d0*HPL2(1,0)*GPL2(2,3)
     +  - 1.d0/4.d0*HPL2(1,0)*GPL2(3,0)
     +
      col3 = col3 + 1.d0/2.d0*HPL2(1,0)*GPL2(3,2)
     +  - 3.d0/8.d0*HPL2(1,1)*pi*ci
     +  - 1.d0/3.d0*HPL2(1,1)*pi**2
     +  - 1.d0/8.d0*HPL2(1,1)*u**(-1)
     +  - 3.d0/8.d0*HPL2(1,1)*GPL1(0)
     +  + 1.d0/4.d0*HPL2(1,1)*GPL1(0)*pi*ci
     +  + 3.d0/8.d0*HPL2(1,1)*GPL1(1)
     +  - 1.d0/8.d0*HPL2(1,1)*GPL1(1)*u**(-2)
     +
      col3 = col3 - 1.d0/4.d0*HPL2(1,1)*GPL1(1)*u**(-1)
     +  + 1.d0/4.d0*HPL2(1,1)*GPL2(0,0)
     +  + 1.d0/4.d0*HPL2(1,1)*GPL2(0,1)
     +  - 1.d0/2.d0*HPL2(1,1)*GPL2(1,1)
     +  + 3.d0/4.d0*HPL3(0,0,0)*pi*ci
     +  + 3.d0/4.d0*HPL3(0,0,0)*GPL1(0)
     +  - 1.d0/4.d0*HPL3(0,0,0)*GPL1(1)
     +  + 1.d0/2.d0*HPL3(0,0,0)*GPL1(2)
     +
      col3 = col3 - 3.d0/4.d0*HPL3(0,0,0)*GPL1(3)
     +  - 1.d0/4.d0*HPL3(0,0,1)*GPL1(2)
     +  - 1.d0/2.d0*HPL3(0,1,0)*pi*ci
     +  - 1.d0/4.d0*HPL3(0,1,0)*GPL1(0)
     +  + 1.d0/4.d0*HPL3(0,1,0)*GPL1(1)
     +  - 1.d0/2.d0*HPL3(0,1,0)*GPL1(2)
     +  + 1.d0/4.d0*HPL3(0,1,0)*GPL1(3)
     +  + 3.d0/8.d0*HPL3(1,0,0)
     +
      col3 = col3 + 1.d0/4.d0*HPL3(1,0,0)*pi*ci
     +  + 1.d0/2.d0*HPL3(1,0,0)*GPL1(0)
     +  - 1.d0/4.d0*HPL3(1,0,0)*GPL1(1)
     +  - 1.d0/4.d0*HPL3(1,0,0)*GPL1(2)
     +  + 1.d0/4.d0*HPL3(1,0,0)*GPL1(3)
     +  - 1.d0/4.d0*HPL3(1,0,1)*pi*ci
     +  - 1.d0/8.d0*HPL3(1,0,1)*upv**(-2)
     +  - 1.d0/4.d0*HPL3(1,0,1)*upv**(-1)
     +
      col3 = col3 - 1.d0/4.d0*HPL3(1,0,1)*GPL1(0)
     +  + 1.d0/4.d0*HPL3(1,0,1)*GPL1(1)
     +  - 1.d0/2.d0*HPL3(1,0,1)*GPL1(2)
     +  + 1.d0/4.d0*HPL3(1,0,1)*GPL1(3)
     +  + 3.d0/8.d0*HPL3(1,1,0)
     +  - 3.d0/4.d0*HPL3(1,1,0)*pi*ci
     +  + 1.d0/8.d0*HPL3(1,1,0)*upv**(-2)
     +  + 1.d0/4.d0*HPL3(1,1,0)*upv**(-1)
     +
      col3 = col3 - 1.d0/4.d0*HPL3(1,1,0)*GPL1(0)
     +  + 1.d0/2.d0*HPL3(1,1,0)*GPL1(2)
     +  - 1.d0/2.d0*HPL3(1,1,0)*GPL1(3)
     +  - 3.d0/4.d0*HPL4(0,0,0,0)
     +  + 1.d0/4.d0*HPL4(0,0,1,0)
     +  + 1.d0/4.d0*HPL4(0,1,0,0)
     +  + 1.d0/4.d0*HPL4(0,1,0,1)
     +  - 1.d0/2.d0*HPL4(0,1,1,0)
     +
      col3 = col3 - 1.d0/2.d0*HPL4(1,0,0,0)
     +  + 1.d0/4.d0*HPL4(1,0,0,1)
     +  + 1.d0/2.d0*HPL4(1,0,1,0)
     +  + 1.d0/4.d0*HPL4(1,1,0,0)
     +  + 1.d0/2.d0*HPL4(1,1,0,1)
     +  - 1.d0/2.d0*HPL4(1,1,1,0)
     +  - 3.d0/8.d0*GPL1(0)
     +  - 1.d0/8.d0*GPL1(0)*pi*u**(-1)*v*ci
     +
      col3 = col3 + 1.d0/8.d0*GPL1(0)*pi*u**(-1)*ci
     +  - 3.d0/8.d0*GPL1(0)*pi*u*ci
     +  - 3.d0/8.d0*GPL1(0)*pi*v*ci
     +  - 1.d0/2.d0*GPL1(0)*pi*ci
     +  + 1.d0/16.d0*GPL1(0)*pi**2
     +  - 7.d0/24.d0*GPL1(0)*pi**3*ci
     +  - 1.d0/4.d0*GPL1(0)*zeta3
     +  - 5.d0/16.d0*GPL1(0)*omumv**(-1)
     +
      col3 = col3 - 5.d0/8.d0*GPL1(1)*pi*u**(-3)*v*ci
     +  + 5.d0/8.d0*GPL1(1)*pi*u**(-3)*ci
     +  - 1.d0/4.d0*GPL1(1)*pi*u**(-2)*v*ci
     +  - 7.d0/16.d0*GPL1(1)*pi*u**(-2)*ci
     +  - 1.d0/8.d0*GPL1(1)*pi*u**(-1)*v**(-1)*ci
     +  + 1.d0/8.d0*GPL1(1)*pi*u**(-1)*v*ci
     +  + 3.d0/4.d0*GPL1(1)*pi*u**(-1)*ci
     +  + 3.d0/8.d0*GPL1(1)*pi*u*ci
     +
      col3 = col3 + 1.d0/8.d0*GPL1(1)*pi*v**(-1)*upv**(-1)*ci
     +  + 3.d0/8.d0*GPL1(1)*pi*v*ci
     +  + 1.d0/8.d0*GPL1(1)*pi*upv**(-1)*ci
     +  + 1.d0/2.d0*GPL1(1)*pi*ci
     +  + 5.d0/48.d0*GPL1(1)*pi**2*u**(-2)
     +  + 5.d0/24.d0*GPL1(1)*pi**2*u**(-1)
     +  - 1.d0/16.d0*GPL1(1)*pi**2
     +  + 1.d0/8.d0*GPL1(1)*pi**3*ci
     +
      col3 = col3 + 1.d0/2.d0*GPL1(1)*zeta3
     +  + 3.d0/8.d0*GPL1(2)
     +  - 1.d0/8.d0*GPL1(2)*pi*u**(-1)*v*ci
     +  - 3.d0/8.d0*GPL1(2)*pi*u*ci
     +  - 3.d0/8.d0*GPL1(2)*pi*v*ci
     +  - 1.d0/2.d0*GPL1(2)*pi*ci
     +  - 1.d0/16.d0*GPL1(2)*pi**2*upv**(-2)
     +  - 1.d0/8.d0*GPL1(2)*pi**2*upv**(-1)
     +
      col3 = col3 + 1.d0/4.d0*GPL1(2)*pi**2
     +  - 5.d0/24.d0*GPL1(2)*pi**3*ci
     +  - GPL1(2)*zeta3
     +  - 5.d0/8.d0*GPL1(2)*u**(-2)*v
     +  + 5.d0/8.d0*GPL1(2)*u**(-2)
     +  - 11.d0/16.d0*GPL1(2)*u**(-1)*v
     +  + 1.d0/8.d0*GPL1(2)*u**(-1)
     +  - 17.d0/16.d0*GPL1(2)*v*omu**(-1)
     +
      col3 = col3 + 1.d0/8.d0*GPL1(3)*pi*u**(-1)*v*ci
     +  + 1.d0/8.d0*GPL1(3)*pi*u**(-1)*omv**(-1)*ci
     +  + 3.d0/8.d0*GPL1(3)*pi*u*ci
     +  + 3.d0/8.d0*GPL1(3)*pi*v*ci
     +  + 1.d0/8.d0*GPL1(3)*pi*omv**(-1)*omumv**(-1)*ci
     +  - 5.d0/16.d0*GPL1(3)*pi*omumv**(-2)*ci
     +  + GPL1(3)*pi*omumv**(-1)*ci
     +  + 1.d0/2.d0*GPL1(3)*pi*ci
     +
      col3 = col3 + 1.d0/24.d0*GPL1(3)*pi**3*ci
     +  + 1.d0/2.d0*GPL1(3)*zeta3
     +  - 5.d0/24.d0*GPL2(0,0)*pi**2
     +  + 1.d0/8.d0*GPL2(0,1)*pi*u**(-2)*ci
     +  + 1.d0/4.d0*GPL2(0,1)*pi*u**(-1)*ci
     +  + 1.d0/8.d0*GPL2(0,1)*pi*upv**(-2)*ci
     +  + 1.d0/4.d0*GPL2(0,1)*pi*upv**(-1)*ci
     +  + 3.d0/8.d0*GPL2(0,1)*pi*ci
     +
      col3 = col3 + 5.d0/24.d0*GPL2(0,1)*pi**2
     +  - 1.d0/2.d0*GPL2(0,2)
     +  - 3.d0/8.d0*GPL2(0,2)*pi*ci
     +  - 3.d0/8.d0*GPL2(0,2)*pi**2
     +  - 1.d0/8.d0*GPL2(0,2)*u**(-1)*v
     +  + 1.d0/8.d0*GPL2(0,2)*u**(-1)
     +  - 3.d0/8.d0*GPL2(0,2)*u
     +  - 3.d0/8.d0*GPL2(0,2)*v
     +
      col3 = col3 + 1.d0/12.d0*GPL2(0,3)*pi**2
     +  + 1.d0/12.d0*GPL2(1,0)*pi**2
     +  - 1.d0/8.d0*GPL2(1,1)*pi*u**(-2)*ci
     +  - 1.d0/4.d0*GPL2(1,1)*pi*u**(-1)*ci
     +  - 3.d0/8.d0*GPL2(1,1)*pi*ci
     +  + 1.d0/12.d0*GPL2(1,1)*pi**2
     +  + 1.d0/2.d0*GPL2(1,2)
     +  - 1.d0/8.d0*GPL2(1,2)*pi*u**(-2)*ci
     +
      col3 = col3 - 1.d0/4.d0*GPL2(1,2)*pi*u**(-1)*ci
     +  + 3.d0/8.d0*GPL2(1,2)*pi*ci
     +  + 1.d0/8.d0*GPL2(1,2)*pi**2
     +  - 5.d0/8.d0*GPL2(1,2)*u**(-3)*v
     +  + 5.d0/8.d0*GPL2(1,2)*u**(-3)
     +  - 1.d0/4.d0*GPL2(1,2)*u**(-2)*v
     +  - 7.d0/16.d0*GPL2(1,2)*u**(-2)
     +  - 1.d0/8.d0*GPL2(1,2)*u**(-1)*v**(-1)
     +
      col3 = col3 + 1.d0/8.d0*GPL2(1,2)*u**(-1)*v
     +  + 3.d0/4.d0*GPL2(1,2)*u**(-1)
     +  + 3.d0/8.d0*GPL2(1,2)*u
     +  + 1.d0/8.d0*GPL2(1,2)*v**(-1)*upv**(-1)
     +  + 3.d0/8.d0*GPL2(1,2)*v
     +  + 1.d0/8.d0*GPL2(1,2)*upv**(-1)
     +  - 1.d0/2.d0*GPL2(2,0)
     +  - 3.d0/8.d0*GPL2(2,0)*pi*ci
     +
      col3 = col3 - 3.d0/8.d0*GPL2(2,0)*pi**2
     +  - 1.d0/8.d0*GPL2(2,0)*u**(-1)*v
     +  + 1.d0/8.d0*GPL2(2,0)*u**(-1)
     +  - 3.d0/8.d0*GPL2(2,0)*u
     +  - 3.d0/8.d0*GPL2(2,0)*v
     +  - 1.d0/8.d0*GPL2(2,1)*pi*u**(-2)*ci
     +  - 1.d0/4.d0*GPL2(2,1)*pi*u**(-1)*ci
     +  + 1.d0/8.d0*GPL2(2,1)*pi*upv**(-2)*ci
     +
      col3 = col3 + 1.d0/4.d0*GPL2(2,1)*pi*upv**(-1)*ci
     +  + 3.d0/8.d0*GPL2(2,1)*pi*ci
     +  + 1.d0/6.d0*GPL2(2,1)*pi**2
     +  - 3.d0/8.d0*GPL2(2,2)*pi*ci
     +  - 1.d0/3.d0*GPL2(2,2)*pi**2
     +  - 1.d0/8.d0*GPL2(2,2)*u**(-1)
     +  + 3.d0/8.d0*GPL2(2,3)*pi*ci
     +  + 1.d0/6.d0*GPL2(2,3)*pi**2
     +
      col3 = col3 + 1.d0/2.d0*GPL2(3,0)
     +  + 1.d0/24.d0*GPL2(3,0)*pi**2
     +  + 1.d0/8.d0*GPL2(3,0)*u**(-1)*v
     +  + 1.d0/8.d0*GPL2(3,0)*u**(-1)*omv**(-1)
     +  + 3.d0/8.d0*GPL2(3,0)*u
     +  + 3.d0/8.d0*GPL2(3,0)*v
     +  + 1.d0/8.d0*GPL2(3,0)*omv**(-1)*omumv**(-1)
     +  - 5.d0/16.d0*GPL2(3,0)*omumv**(-2)
     +
      col3 = col3 + GPL2(3,0)*omumv**(-1)
     +  + 1.d0/6.d0*GPL2(3,2)*pi**2
     +  + 1.d0/6.d0*GPL2(3,3)*pi**2
     +  - 1.d0/4.d0*GPL3(0,0,1)*pi*ci
     +  + 1.d0/4.d0*GPL3(0,0,2)*pi*ci
     +  + 1.d0/2.d0*GPL3(0,0,3)*pi*ci
     +  - 1.d0/4.d0*GPL3(0,1,0)*pi*ci
     +  - 1.d0/4.d0*GPL3(0,1,1)*pi*ci
     +
      col3 = col3 + 3.d0/8.d0*GPL3(0,1,2)
     +  + 1.d0/8.d0*GPL3(0,1,2)*u**(-2)
     +  + 1.d0/4.d0*GPL3(0,1,2)*u**(-1)
     +  + 1.d0/8.d0*GPL3(0,1,2)*upv**(-2)
     +  + 1.d0/4.d0*GPL3(0,1,2)*upv**(-1)
     +  + 1.d0/4.d0*GPL3(0,2,0)*pi*ci
     +  + 1.d0/2.d0*GPL3(0,2,1)*pi*ci
     +  - 3.d0/8.d0*GPL3(0,2,2)
     +
      col3 = col3 + 1.d0/4.d0*GPL3(0,2,2)*pi*ci
     +  + 1.d0/2.d0*GPL3(0,2,3)*pi*ci
     +  - 1.d0/2.d0*GPL3(0,3,3)*pi*ci
     +  + 1.d0/4.d0*GPL3(1,0,1)*pi*ci
     +  - 1.d0/4.d0*GPL3(1,0,3)*pi*ci
     +  + 1.d0/2.d0*GPL3(1,1,1)*pi*ci
     +  - 3.d0/8.d0*GPL3(1,1,2)
     +  - 1.d0/2.d0*GPL3(1,1,2)*pi*ci
     +
      col3 = col3 - 1.d0/8.d0*GPL3(1,1,2)*u**(-2)
     +  - 1.d0/4.d0*GPL3(1,1,2)*u**(-1)
     +  - 3.d0/4.d0*GPL3(1,2,1)*pi*ci
     +  + 3.d0/8.d0*GPL3(1,2,2)
     +  - 1.d0/8.d0*GPL3(1,2,2)*u**(-2)
     +  - 1.d0/4.d0*GPL3(1,2,2)*u**(-1)
     +  - 1.d0/4.d0*GPL3(1,2,3)*pi*ci
     +  + 1.d0/4.d0*GPL3(2,0,0)*pi*ci
     +
      col3 = col3 + 1.d0/2.d0*GPL3(2,0,1)*pi*ci
     +  - 3.d0/8.d0*GPL3(2,0,2)
     +  + 1.d0/4.d0*GPL3(2,0,2)*pi*ci
     +  + 1.d0/2.d0*GPL3(2,0,3)*pi*ci
     +  - 1.d0/2.d0*GPL3(2,1,1)*pi*ci
     +  + 3.d0/8.d0*GPL3(2,1,2)
     +  - 1.d0/8.d0*GPL3(2,1,2)*u**(-2)
     +  - 1.d0/4.d0*GPL3(2,1,2)*u**(-1)
     +
      col3 = col3 + 1.d0/8.d0*GPL3(2,1,2)*upv**(-2)
     +  + 1.d0/4.d0*GPL3(2,1,2)*upv**(-1)
     +  - 3.d0/8.d0*GPL3(2,2,0)
     +  + 1.d0/4.d0*GPL3(2,2,0)*pi*ci
     +  + 1.d0/2.d0*GPL3(2,2,1)*pi*ci
     +  - 1.d0/4.d0*GPL3(2,2,3)*pi*ci
     +  + 3.d0/8.d0*GPL3(2,3,0)
     +  - 1.d0/4.d0*GPL3(2,3,2)*pi*ci
     +
      col3 = col3 - 1.d0/4.d0*GPL3(2,3,3)*pi*ci
     +  - 1.d0/4.d0*GPL3(3,0,1)*pi*ci
     +  - 3.d0/4.d0*GPL3(3,0,3)*pi*ci
     +  - 1.d0/4.d0*GPL3(3,2,1)*pi*ci
     +  + 1.d0/4.d0*GPL3(3,2,3)*pi*ci
     +  - 1.d0/2.d0*GPL3(3,3,0)*pi*ci
     +  + 1.d0/2.d0*GPL3(3,3,3)*pi*ci
     +  - 1.d0/4.d0*GPL4(0,0,1,2)
     +
      col3 = col3 + 1.d0/4.d0*GPL4(0,0,2,2)
     +  + 1.d0/2.d0*GPL4(0,0,3,0)
     +  - 1.d0/4.d0*GPL4(0,1,0,2)
     +  - 1.d0/4.d0*GPL4(0,1,1,2)
     +  - 1.d0/4.d0*GPL4(0,1,2,0)
     +  + 1.d0/4.d0*GPL4(0,1,2,2)
     +  + 1.d0/4.d0*GPL4(0,2,0,2)
     +  + 1.d0/2.d0*GPL4(0,2,1,2)
     +
      col3 = col3 + 1.d0/4.d0*GPL4(0,2,2,0)
     +  + 1.d0/2.d0*GPL4(0,2,3,0)
     +  - 1.d0/2.d0*GPL4(0,3,3,0)
     +  + 1.d0/4.d0*GPL4(1,0,1,2)
     +  - 1.d0/4.d0*GPL4(1,0,3,0)
     +  + 1.d0/2.d0*GPL4(1,1,1,2)
     +  - 1.d0/2.d0*GPL4(1,1,2,2)
     +  - 3.d0/4.d0*GPL4(1,2,1,2)
     +
      col3 = col3 - 1.d0/4.d0*GPL4(1,2,3,0)
     +  + 1.d0/4.d0*GPL4(2,0,0,2)
     +  + 1.d0/2.d0*GPL4(2,0,1,2)
     +  + 1.d0/4.d0*GPL4(2,0,2,0)
     +  + 1.d0/2.d0*GPL4(2,0,3,0)
     +  - 1.d0/2.d0*GPL4(2,1,1,2)
     +  + 1.d0/4.d0*GPL4(2,2,0,0)
     +  + 1.d0/2.d0*GPL4(2,2,1,2)
     +
      col3 = col3 - 1.d0/4.d0*GPL4(2,2,3,0)
     +  + 1.d0/4.d0*GPL4(2,3,0,0)
     +  - 1.d0/4.d0*GPL4(2,3,0,2)
     +  - 1.d0/4.d0*GPL4(2,3,2,0)
     +  - 1.d0/4.d0*GPL4(2,3,3,0)
     +  - 1.d0/4.d0*GPL4(3,0,1,2)
     +  - 3.d0/4.d0*GPL4(3,0,3,0)
     +  - 1.d0/4.d0*GPL4(3,2,1,2)
     +
      col3 = col3 + 1.d0/4.d0*GPL4(3,2,3,0)
     +  - 1.d0/2.d0*GPL4(3,3,0,0)
     +  + 1.d0/2.d0*GPL4(3,3,3,0)
     +

      col4 =
     +  + 377.d0/2592.d0
     +  + 1.d0/12.d0*pi*u**(-1)*v*ci
     +  + 1.d0/12.d0*pi*v*omu**(-1)*ci
     +  + 155.d0/216.d0*pi*ci
     +  - 211.d0/432.d0*pi**2
     +  - 5.d0/144.d0*pi**3*ci
     +  + 131.d0/72.d0*zeta3
     +
      col4 = col4 + 19.d0/36.d0*u**(-1)
     +  + 1.d0/4.d0*omv**(-1)
     +  + 281.d0/216.d0*HPL1(0)
     +  - 29.d0/36.d0*HPL1(0)*pi*ci
     +  - 5.d0/144.d0*HPL1(0)*pi**2
     +  - 1.d0/12.d0*HPL1(0)*u**(-1)*v
     +  + 5.d0/24.d0*HPL1(0)*u**(-1)
     +  - 1.d0/12.d0*HPL1(0)*v*omu**(-1)
     +
      col4 = col4 + 1.d0/4.d0*HPL1(0)*omv**(-2)
     +  - 1.d0/2.d0*HPL1(0)*omv**(-1)
     +  - 29.d0/36.d0*HPL1(0)*GPL1(0)
     +  + 7.d0/24.d0*HPL1(0)*GPL1(0)*pi*ci
     +  + 29.d0/36.d0*HPL1(0)*GPL1(1)
     +  - 1.d0/12.d0*HPL1(0)*GPL1(1)*pi*ci
     +  - 29.d0/36.d0*HPL1(0)*GPL1(2)
     +  + 1.d0/8.d0*HPL1(0)*GPL1(2)*pi*ci
     +
      col4 = col4 + 29.d0/36.d0*HPL1(0)*GPL1(3)
     +  + 1.d0/12.d0*HPL1(0)*GPL1(3)*pi*ci
     +  + 1.d0/4.d0*HPL1(0)*GPL2(0,0)
     +  - 1.d0/24.d0*HPL1(0)*GPL2(0,1)
     +  + 1.d0/2.d0*HPL1(0)*GPL2(0,2)
     +  - 5.d0/24.d0*HPL1(0)*GPL2(0,3)
     +  - 1.d0/24.d0*HPL1(0)*GPL2(1,0)
     +  - 1.d0/6.d0*HPL1(0)*GPL2(1,1)
     +
      col4 = col4 - 7.d0/24.d0*HPL1(0)*GPL2(1,2)
     +  + 1.d0/2.d0*HPL1(0)*GPL2(2,0)
     +  - 5.d0/24.d0*HPL1(0)*GPL2(2,1)
     +  + 1.d0/4.d0*HPL1(0)*GPL2(2,2)
     +  - 1.d0/24.d0*HPL1(0)*GPL2(2,3)
     +  - 7.d0/24.d0*HPL1(0)*GPL2(3,0)
     +  - 1.d0/24.d0*HPL1(0)*GPL2(3,2)
     +  - 1.d0/6.d0*HPL1(0)*GPL2(3,3)
     +
      col4 = col4 + 1.d0/6.d0*HPL1(1)
     +  - 29.d0/36.d0*HPL1(1)*pi*ci
     +  + 1.d0/12.d0*HPL1(1)*pi**2
     +  - 1.d0/12.d0*HPL1(1)*u**(-1)*v
     +  + 5.d0/48.d0*HPL1(1)*u**(-1)
     +  - 1.d0/12.d0*HPL1(1)*v*omu**(-1)
     +  - 29.d0/36.d0*HPL1(1)*GPL1(0)
     +  + 1.d0/12.d0*HPL1(1)*GPL1(0)*pi*ci
     +
      col4 = col4 + 29.d0/36.d0*HPL1(1)*GPL1(1)
     +  + 1.d0/8.d0*HPL1(1)*GPL1(1)*pi*ci
     +  + 1.d0/4.d0*HPL1(1)*GPL1(2)*pi*ci
     +  - 1.d0/24.d0*HPL1(1)*GPL1(3)*pi*ci
     +  + 1.d0/4.d0*HPL1(1)*GPL2(0,0)
     +  - 1.d0/24.d0*HPL1(1)*GPL2(0,1)
     +  + 1.d0/4.d0*HPL1(1)*GPL2(0,2)
     +  - 1.d0/24.d0*HPL1(1)*GPL2(1,0)
     +
      col4 = col4 - 1.d0/6.d0*HPL1(1)*GPL2(1,1)
     +  - 1.d0/4.d0*HPL1(1)*GPL2(1,2)
     +  + 1.d0/4.d0*HPL1(1)*GPL2(2,0)
     +  - 5.d0/24.d0*HPL1(1)*GPL2(2,1)
     +  - 1.d0/24.d0*HPL1(1)*GPL2(3,0)
     +  + 29.d0/36.d0*HPL2(0,0)
     +  - 1.d0/3.d0*HPL2(0,0)*pi*ci
     +  - 17.d0/24.d0*HPL2(0,0)*GPL1(0)
     +
      col4 = col4 + 1.d0/2.d0*HPL2(0,0)*GPL1(1)
     +  - 13.d0/24.d0*HPL2(0,0)*GPL1(2)
     +  + 1.d0/3.d0*HPL2(0,0)*GPL1(3)
     +  - 11.d0/24.d0*HPL2(0,1)*pi*ci
     +  - 11.d0/24.d0*HPL2(0,1)*GPL1(0)
     +  + 11.d0/24.d0*HPL2(0,1)*GPL1(1)
     +  - 1.d0/24.d0*HPL2(0,1)*GPL1(2)
     +  + 1.d0/24.d0*HPL2(0,1)*GPL1(3)
     +
      col4 = col4 + 29.d0/36.d0*HPL2(1,0)
     +  - 1.d0/8.d0*HPL2(1,0)*pi*ci
     +  - 1.d0/2.d0*HPL2(1,0)*GPL1(0)
     +  + 7.d0/24.d0*HPL2(1,0)*GPL1(1)
     +  - 1.d0/4.d0*HPL2(1,0)*GPL1(2)
     +  + 1.d0/24.d0*HPL2(1,0)*GPL1(3)
     +  - 1.d0/4.d0*HPL2(1,1)*pi*ci
     +  - 1.d0/4.d0*HPL2(1,1)*GPL1(0)
     +
      col4 = col4 + 1.d0/4.d0*HPL2(1,1)*GPL1(1)
     +  + 3.d0/4.d0*HPL3(0,0,0)
     +  + 1.d0/24.d0*HPL3(0,0,1)
     +  + 11.d0/24.d0*HPL3(0,1,0)
     +  + 13.d0/24.d0*HPL3(1,0,0)
     +  + 1.d0/24.d0*HPL3(1,0,1)
     +  + 1.d0/4.d0*HPL3(1,1,0)
     +  - 1.d0/6.d0*GPL1(0)
     +
      col4 = col4 + 29.d0/36.d0*GPL1(0)*pi*ci
     +  - 1.d0/18.d0*GPL1(0)*pi**2
     +  - 5.d0/48.d0*GPL1(0)*u**(-1)
     +  - 29.d0/36.d0*GPL1(1)*pi*ci
     +  + 7.d0/36.d0*GPL1(1)*pi**2
     +  - 1.d0/6.d0*GPL1(2)
     +  + 29.d0/36.d0*GPL1(2)*pi*ci
     +  - 1.d0/12.d0*GPL1(2)*pi**2
     +
      col4 = col4 + 1.d0/12.d0*GPL1(2)*u**(-1)*v
     +  - 5.d0/48.d0*GPL1(2)*u**(-1)
     +  + 1.d0/12.d0*GPL1(2)*v*omu**(-1)
     +  - 29.d0/36.d0*GPL1(3)*pi*ci
     +  + 2.d0/9.d0*GPL1(3)*pi**2
     +  - 1.d0/4.d0*GPL2(0,0)*pi*ci
     +  + 1.d0/24.d0*GPL2(0,1)*pi*ci
     +  + 29.d0/36.d0*GPL2(0,2)
     +
      col4 = col4 - 1.d0/12.d0*GPL2(0,2)*pi*ci
     +  + 5.d0/24.d0*GPL2(0,3)*pi*ci
     +  + 1.d0/24.d0*GPL2(1,0)*pi*ci
     +  + 1.d0/6.d0*GPL2(1,1)*pi*ci
     +  - 29.d0/36.d0*GPL2(1,2)
     +  - 1.d0/8.d0*GPL2(1,2)*pi*ci
     +  + 29.d0/36.d0*GPL2(2,0)
     +  - 1.d0/12.d0*GPL2(2,0)*pi*ci
     +
      col4 = col4 + 5.d0/24.d0*GPL2(2,1)*pi*ci
     +  - 1.d0/4.d0*GPL2(2,2)*pi*ci
     +  + 1.d0/24.d0*GPL2(2,3)*pi*ci
     +  - 29.d0/36.d0*GPL2(3,0)
     +  - 1.d0/8.d0*GPL2(3,0)*pi*ci
     +  + 1.d0/24.d0*GPL2(3,2)*pi*ci
     +  + 1.d0/6.d0*GPL2(3,3)*pi*ci
     +  - 1.d0/4.d0*GPL3(0,0,2)
     +
      col4 = col4 + 1.d0/24.d0*GPL3(0,1,2)
     +  - 1.d0/4.d0*GPL3(0,2,0)
     +  - 1.d0/4.d0*GPL3(0,2,2)
     +  + 5.d0/24.d0*GPL3(0,3,0)
     +  + 1.d0/24.d0*GPL3(1,0,2)
     +  + 1.d0/6.d0*GPL3(1,1,2)
     +  + 1.d0/24.d0*GPL3(1,2,0)
     +  + 1.d0/4.d0*GPL3(1,2,2)
     +
      col4 = col4 - 1.d0/4.d0*GPL3(2,0,0)
     +  - 1.d0/4.d0*GPL3(2,0,2)
     +  + 5.d0/24.d0*GPL3(2,1,2)
     +  - 1.d0/4.d0*GPL3(2,2,0)
     +  + 1.d0/24.d0*GPL3(2,3,0)
     +  + 1.d0/4.d0*GPL3(3,0,0)
     +  + 1.d0/24.d0*GPL3(3,0,2)
     +  + 1.d0/24.d0*GPL3(3,2,0)
     +
      col4 = col4 + 1.d0/6.d0*GPL3(3,3,0)
     +

      col5 =
     +  + 8785.d0/864.d0
     +  - 1.d0/2.d0*pi*u**(-2)*v*ci
     +  + 1.d0/2.d0*pi*u**(-2)*ci
     +  - 1.d0/12.d0*pi*u**(-1)*v*ci
     +  - 1.d0/2.d0*pi*u**(-1)*ci
     +  - 1.d0/12.d0*pi*v*omu**(-1)*ci
     +  - 1.d0/4.d0*pi*omumv**(-1)*ci
     +
      col5 = col5 + 1147.d0/216.d0*pi*ci
     +  - 1.d0/24.d0*pi**2*omumv**(-2)
     +  + 1.d0/12.d0*pi**2*omumv**(-1)
     +  - 647.d0/864.d0*pi**2
     +  - 1.d0/18.d0*pi**3*ci
     +  + 61.d0/36.d0*zeta3
     +  + 37.d0/36.d0*u**(-1)
     +  - 13.d0/12.d0*omv**(-1)
     +
      col5 = col5 + 1379.d0/216.d0*HPL1(0)
     +  - 1.d0/4.d0*HPL1(0)*pi*omumv**(-2)*ci
     +  + 1.d0/2.d0*HPL1(0)*pi*omumv**(-1)*ci
     +  + 215.d0/72.d0*HPL1(0)*pi*ci
     +  - 1.d0/6.d0*HPL1(0)*pi**2
     +  + 1.d0/2.d0*HPL1(0)*u**(-2)*v
     +  + 1.d0/12.d0*HPL1(0)*u**(-1)*v
     +  - 1.d0/4.d0*HPL1(0)*u**(-1)*omv**(-1)
     +
      col5 = col5 + 11.d0/24.d0*HPL1(0)*u**(-1)
     +  + 1.d0/12.d0*HPL1(0)*v*omu**(-1)
     +  - 13.d0/12.d0*HPL1(0)*omv**(-2)
     +  - 1.d0/4.d0*HPL1(0)*omv**(-1)*omumv**(-1)
     +  + 7.d0/3.d0*HPL1(0)*omv**(-1)
     +  + 1.d0/4.d0*HPL1(0)*omumv**(-1)
     +  - 25.d0/48.d0*HPL1(0)*GPL1(0)
     +  + 1.d0/8.d0*HPL1(0)*GPL1(0)*pi*ci
     +
      col5 = col5 - 1.d0/4.d0*HPL1(0)*GPL1(0)*omumv**(-2)
     +  + 1.d0/2.d0*HPL1(0)*GPL1(0)*omumv**(-1)
     +  - 5.d0/9.d0*HPL1(0)*GPL1(1)
     +  + 1.d0/12.d0*HPL1(0)*GPL1(1)*pi*ci
     +  + 1.d0/2.d0*HPL1(0)*GPL1(1)*u**(-3)*v
     +  - 1.d0/2.d0*HPL1(0)*GPL1(1)*u**(-3)
     +  - 1.d0/6.d0*HPL1(0)*GPL1(1)*u**(-2)*v
     +  + 5.d0/6.d0*HPL1(0)*GPL1(1)*u**(-2)
     +
      col5 = col5 - 7.d0/12.d0*HPL1(0)*GPL1(1)*u**(-1)
     +  - 25.d0/48.d0*HPL1(0)*GPL1(2)
     +  + 5.d0/8.d0*HPL1(0)*GPL1(2)*pi*ci
     +  - 1.d0/2.d0*HPL1(0)*GPL1(2)*u**(-3)*v
     +  + 1.d0/2.d0*HPL1(0)*GPL1(2)*u**(-3)
     +  + 1.d0/6.d0*HPL1(0)*GPL1(2)*u**(-2)*v
     +  - 5.d0/6.d0*HPL1(0)*GPL1(2)*u**(-2)
     +  + 7.d0/12.d0*HPL1(0)*GPL1(2)*u**(-1)
     +
      col5 = col5 - 5.d0/9.d0*HPL1(0)*GPL1(3)
     +  - 1.d0/12.d0*HPL1(0)*GPL1(3)*pi*ci
     +  + 1.d0/4.d0*HPL1(0)*GPL1(3)*omumv**(-2)
     +  - 1.d0/2.d0*HPL1(0)*GPL1(3)*omumv**(-1)
     +  - 1.d0/4.d0*HPL1(0)*GPL2(0,0)
     +  - 7.d0/24.d0*HPL1(0)*GPL2(0,1)
     +  + 1.d0/4.d0*HPL1(0)*GPL2(0,2)
     +  + 5.d0/24.d0*HPL1(0)*GPL2(0,3)
     +
      col5 = col5 + 1.d0/24.d0*HPL1(0)*GPL2(1,0)
     +  + 1.d0/6.d0*HPL1(0)*GPL2(1,1)
     +  + 7.d0/24.d0*HPL1(0)*GPL2(1,2)
     +  + 1.d0/4.d0*HPL1(0)*GPL2(2,0)
     +  + 5.d0/24.d0*HPL1(0)*GPL2(2,1)
     +  - 1.d0/4.d0*HPL1(0)*GPL2(2,2)
     +  - 7.d0/24.d0*HPL1(0)*GPL2(2,3)
     +  + 7.d0/24.d0*HPL1(0)*GPL2(3,0)
     +
      col5 = col5 + 1.d0/24.d0*HPL1(0)*GPL2(3,2)
     +  + 1.d0/6.d0*HPL1(0)*GPL2(3,3)
     +  + 251.d0/216.d0*HPL1(1)
     +  + 53.d0/144.d0*HPL1(1)*pi*ci
     +  - 1.d0/18.d0*HPL1(1)*pi**2
     +  + 1.d0/2.d0*HPL1(1)*u**(-2)*v
     +  - 1.d0/2.d0*HPL1(1)*u**(-2)
     +  + 1.d0/12.d0*HPL1(1)*u**(-1)*v
     +
      col5 = col5 + 29.d0/48.d0*HPL1(1)*u**(-1)
     +  + 1.d0/12.d0*HPL1(1)*v*omu**(-1)
     +  - 5.d0/36.d0*HPL1(1)*GPL1(0)
     +  - 5.d0/9.d0*HPL1(1)*GPL1(1)
     +  - 1.d0/8.d0*HPL1(1)*GPL1(1)*pi*ci
     +  + 1.d0/2.d0*HPL1(1)*GPL1(1)*u**(-3)*v
     +  - 1.d0/2.d0*HPL1(1)*GPL1(1)*u**(-3)
     +  - 1.d0/6.d0*HPL1(1)*GPL1(1)*u**(-2)*v
     +
      col5 = col5 + 5.d0/6.d0*HPL1(1)*GPL1(1)*u**(-2)
     +  - 7.d0/12.d0*HPL1(1)*GPL1(1)*u**(-1)
     +  - 41.d0/72.d0*HPL1(1)*GPL1(2)
     +  + 1.d0/24.d0*HPL1(1)*GPL1(3)*pi*ci
     +  - 7.d0/24.d0*HPL1(1)*GPL2(0,1)
     +  + 1.d0/24.d0*HPL1(1)*GPL2(1,0)
     +  + 1.d0/6.d0*HPL1(1)*GPL2(1,1)
     +  + 1.d0/4.d0*HPL1(1)*GPL2(1,2)
     +
      col5 = col5 + 5.d0/24.d0*HPL1(1)*GPL2(2,1)
     +  + 1.d0/24.d0*HPL1(1)*GPL2(3,0)
     +  + 145.d0/36.d0*HPL2(0,0)
     +  + 1.d0/3.d0*HPL2(0,0)*pi*ci
     +  + 1.d0/4.d0*HPL2(0,0)*omumv**(-2)
     +  - 1.d0/2.d0*HPL2(0,0)*omumv**(-1)
     +  + 17.d0/24.d0*HPL2(0,0)*GPL1(0)
     +  - 1.d0/2.d0*HPL2(0,0)*GPL1(1)
     +
      col5 = col5 + 5.d0/24.d0*HPL2(0,0)*GPL1(2)
     +  - 1.d0/3.d0*HPL2(0,0)*GPL1(3)
     +  + 155.d0/144.d0*HPL2(0,1)
     +  + 1.d0/24.d0*HPL2(0,1)*pi*ci
     +  + 1.d0/24.d0*HPL2(0,1)*GPL1(0)
     +  - 11.d0/24.d0*HPL2(0,1)*GPL1(1)
     +  + 1.d0/24.d0*HPL2(0,1)*GPL1(2)
     +  - 1.d0/24.d0*HPL2(0,1)*GPL1(3)
     +
      col5 = col5 + 235.d0/144.d0*HPL2(1,0)
     +  + 1.d0/24.d0*HPL2(1,0)*pi*ci
     +  - 1.d0/4.d0*HPL2(1,0)*omumv**(-2)
     +  + 1.d0/2.d0*HPL2(1,0)*omumv**(-1)
     +  - 1.d0/3.d0*HPL2(1,0)*GPL1(0)
     +  - 7.d0/24.d0*HPL2(1,0)*GPL1(1)
     +  + 1.d0/6.d0*HPL2(1,0)*GPL1(2)
     +  - 1.d0/24.d0*HPL2(1,0)*GPL1(3)
     +
      col5 = col5 + 41.d0/72.d0*HPL2(1,1)
     +  - 1.d0/4.d0*HPL2(1,1)*GPL1(1)
     +  - 3.d0/4.d0*HPL3(0,0,0)
     +  - 1.d0/24.d0*HPL3(0,0,1)
     +  + 3.d0/8.d0*HPL3(0,1,0)
     +  + 19.d0/24.d0*HPL3(1,0,0)
     +  + 1.d0/24.d0*HPL3(1,0,1)
     +  - 5.d0/12.d0*HPL3(1,1,0)
     +
      col5 = col5 - 251.d0/216.d0*GPL1(0)
     +  - 53.d0/144.d0*GPL1(0)*pi*ci
     +  - 1.d0/12.d0*GPL1(0)*pi**2
     +  - 5.d0/48.d0*GPL1(0)*u**(-1)
     +  - 1.d0/4.d0*GPL1(0)*omumv**(-1)
     +  - 1.d0/2.d0*GPL1(1)*pi*u**(-3)*v*ci
     +  + 1.d0/2.d0*GPL1(1)*pi*u**(-3)*ci
     +  + 1.d0/6.d0*GPL1(1)*pi*u**(-2)*v*ci
     +
      col5 = col5 - 5.d0/6.d0*GPL1(1)*pi*u**(-2)*ci
     +  + 7.d0/12.d0*GPL1(1)*pi*u**(-1)*ci
     +  + 5.d0/9.d0*GPL1(1)*pi*ci
     +  - 7.d0/36.d0*GPL1(1)*pi**2
     +  - 251.d0/216.d0*GPL1(2)
     +  - 53.d0/144.d0*GPL1(2)*pi*ci
     +  - 1.d0/2.d0*GPL1(2)*u**(-2)*v
     +  + 1.d0/2.d0*GPL1(2)*u**(-2)
     +
      col5 = col5 - 1.d0/12.d0*GPL1(2)*u**(-1)*v
     +  - 29.d0/48.d0*GPL1(2)*u**(-1)
     +  - 1.d0/12.d0*GPL1(2)*v*omu**(-1)
     +  - 1.d0/4.d0*GPL1(3)*pi*omumv**(-2)*ci
     +  + 1.d0/2.d0*GPL1(3)*pi*omumv**(-1)*ci
     +  + 5.d0/9.d0*GPL1(3)*pi*ci
     +  - 2.d0/9.d0*GPL1(3)*pi**2
     +  + 41.d0/72.d0*GPL2(0,0)
     +
      col5 = col5 + 7.d0/24.d0*GPL2(0,1)*pi*ci
     +  + 5.d0/36.d0*GPL2(0,2)
     +  - 5.d0/24.d0*GPL2(0,3)*pi*ci
     +  - 1.d0/24.d0*GPL2(1,0)*pi*ci
     +  - 1.d0/6.d0*GPL2(1,1)*pi*ci
     +  + 5.d0/9.d0*GPL2(1,2)
     +  + 1.d0/8.d0*GPL2(1,2)*pi*ci
     +  - 1.d0/2.d0*GPL2(1,2)*u**(-3)*v
     +
      col5 = col5 + 1.d0/2.d0*GPL2(1,2)*u**(-3)
     +  + 1.d0/6.d0*GPL2(1,2)*u**(-2)*v
     +  - 5.d0/6.d0*GPL2(1,2)*u**(-2)
     +  + 7.d0/12.d0*GPL2(1,2)*u**(-1)
     +  + 5.d0/36.d0*GPL2(2,0)
     +  - 5.d0/24.d0*GPL2(2,1)*pi*ci
     +  + 41.d0/72.d0*GPL2(2,2)
     +  + 7.d0/24.d0*GPL2(2,3)*pi*ci
     +
      col5 = col5 + 5.d0/9.d0*GPL2(3,0)
     +  + 1.d0/8.d0*GPL2(3,0)*pi*ci
     +  - 1.d0/4.d0*GPL2(3,0)*omumv**(-2)
     +  + 1.d0/2.d0*GPL2(3,0)*omumv**(-1)
     +  - 1.d0/24.d0*GPL2(3,2)*pi*ci
     +  - 1.d0/6.d0*GPL2(3,3)*pi*ci
     +  + 7.d0/24.d0*GPL3(0,1,2)
     +  - 5.d0/24.d0*GPL3(0,3,0)
     +
      col5 = col5 - 1.d0/24.d0*GPL3(1,0,2)
     +  - 1.d0/6.d0*GPL3(1,1,2)
     +  - 1.d0/24.d0*GPL3(1,2,0)
     +  - 1.d0/4.d0*GPL3(1,2,2)
     +  - 5.d0/24.d0*GPL3(2,1,2)
     +  + 7.d0/24.d0*GPL3(2,3,0)
     +  - 1.d0/4.d0*GPL3(3,0,0)
     +  - 1.d0/24.d0*GPL3(3,0,2)
     +
      col5 = col5 - 1.d0/24.d0*GPL3(3,2,0)
     +  - 1.d0/6.d0*GPL3(3,3,0)
     +

      col6 =
     +  - 25.d0/81.d0
     +  - 10.d0/27.d0*pi*ci
     +  + 47.d0/432.d0*pi**2
     +  - 5.d0/9.d0*HPL1(0)
     +  - 5.d0/18.d0*HPL1(0)*pi*ci
     +  + 5.d0/72.d0*HPL1(0)*GPL1(0)
     +  + 5.d0/72.d0*HPL1(0)*GPL1(2)
     +
      col6 = col6 - 5.d0/54.d0*HPL1(1)
     +  - 1.d0/36.d0*HPL1(1)*pi*ci
     +  + 1.d0/144.d0*HPL1(1)*GPL1(0)
     +  + 5.d0/144.d0*HPL1(1)*GPL1(2)
     +  - 5.d0/12.d0*HPL2(0,0)
     +  - 5.d0/72.d0*HPL2(0,1)
     +  - 5.d0/72.d0*HPL2(1,0)
     +  - 5.d0/144.d0*HPL2(1,1)
     +
      col6 = col6 + 5.d0/54.d0*GPL1(0)
     +  + 1.d0/36.d0*GPL1(0)*pi*ci
     +  + 5.d0/54.d0*GPL1(2)
     +  + 1.d0/36.d0*GPL1(2)*pi*ci
     +  - 5.d0/144.d0*GPL2(0,0)
     +  - 1.d0/144.d0*GPL2(0,2)
     +  - 1.d0/144.d0*GPL2(2,0)
     +  - 5.d0/144.d0*GPL2(2,2)
     +


      endif

      col(1) = col1*(-1.d0)
      col(2) = col2*(-1.d0)
      col(3) = col3*(-1.d0)
      col(4) = col4*(-1.d0)
      col(5) = col5*(-1.d0)
      col(6) = col6*(-1.d0)     !To be consistent with gamma_0=1

      end subroutine gamma2a2L

!#############################################################

      subroutine gamma3a2L(u,v,col)
      implicit none
      !Returns gamma(3a+) at one-loop
      !------------------------------
      real*8, intent(in) :: u,v
      complex*16, dimension(6), intent(out) :: col
      !internal
      complex*16 :: gam
      complex*16 :: ci
      real*8 :: omu,omv,omumv,upv
      real*8, dimension(0:1) :: HPL1
      real*8, dimension(0:1,0:1) :: HPL2
      real*8, dimension(0:1,0:1,0:1) :: HPL3
      real*8, dimension(0:1,0:1,0:1,0:1) :: HPL4
      real*8, dimension(0:3) :: GPL1
      real*8, dimension(0:3,0:3) :: GPL2
      real*8, dimension(0:3,0:3,0:3) :: GPL3
      real*8, dimension(0:3,0:3,0:3,0:3) :: GPL4
      complex*16 :: col1,col2,col3,col4,col5,col6


      omu   = 1.0d0 - u
      omv   = 1.0d0     - v
      omumv = 1.0d0 - u - v
      upv   =         u + v

      ci = (0.d0,1.d0)

      call tdhpl(dble(u),dble(v),4,GPL1,GPL2,GPL3,GPL4,HPL1,HPL2,HPL3,HPL4)

      if(ieorder==4) then
         col1 = - 1.0d0/2.0d0

         col2 = 1.0d0/2.0d0

         col3 = - 1.0d0/8.0d0

         col4 = 0.0d0

         col5 = 0.0d0

         col6 = 0.0d0
      elseif(ieorder==3) then
         col1 = - 73.0d0/24.0d0
     .          - 1.0d0/2.0d0*pi*ci
     .          - HPL1(0)
     .          + 1.0d0/2.0d0*GPL1(0)

         col2 = + 109.0d0/48.0d0
     .          + 1.0d0/4.0d0*pi*ci
     .          + HPL1(0)
     .          + 1.0d0/2.0d0*HPL1(1)
     .          - 1.0d0/4.0d0*GPL1(0)
     .          - 1.0d0/2.0d0*GPL1(2)

         col3 = - 3.0d0/8.0d0
     .          - 1.0d0/4.0d0*HPL1(0)
     .          - 1.0d0/4.0d0*HPL1(1)
     .          + 1.0d0/4.0d0*GPL1(2)

         col4 = - 5.0d0/24.0d0

         col5 = + 5.0d0/12.0d0

         col6 = 0
      else
         col1 = - 260437.0d0/10368.0d0
     .        + 5.0d0/8.0d0*u**(-1)*pi*ci
     .        - 5.0d0/4.0d0*u**(-1)*v*pi*ci
     .        + 5.0d0/8.0d0*u**(-1)*v**2*pi*ci
     .        - 35.0d0/8.0d0*pi*ci
     .        - 1577.0d0/864.0d0*pi**2
     .        - 55.0d0/96.0d0*pi**3*ci
     .        - 3.0d0/80.0d0*pi**4
     .        + 37.0d0/18.0d0*zeta3
     .        + 5.0d0/4.0d0*zeta3*pi*ci
     .        - 4.0d0/3.0d0*v*omu**(-1)
     .        - 5.0d0/3.0d0*v*omu**(-1)*pi*ci
     .        + 19.0d0/48.0d0*v*upv**(-1)*pi*ci
     .        + 269.0d0/72.0d0*v
     .        - 17.0d0/48.0d0*v*pi*ci
         col1 = col1 + 53.0d0/144.0d0*v*pi**2
     .        + 1.0d0/12.0d0*v*pi**3*ci
     .        - 1.0d0/2.0d0*v*zeta3
     .        - 4.0d0/3.0d0*v**2*omu**(-2)*pi*ci
     .        + 5.0d0/4.0d0*v**2*pi*ci
     .        + 4.0d0/9.0d0*v**2*pi**2
     .        + 1.0d0/24.0d0*v**2*pi**3*ci
     .        - 1.0d0/4.0d0*v**2*zeta3
     .        + 269.0d0/72.0d0*u
     .        + 2.0d0/3.0d0*u*pi*ci
     .        + 10.0d0/9.0d0*u*pi**2
     .        - 1.0d0/2.0d0*u*zeta3
     .        + 5.0d0/4.0d0*u*v*pi*ci
     .        - 2.0d0/3.0d0*u*v*pi**2
     .        + 1.0d0/12.0d0*u*v*pi**3*ci
         col1 = col1 - 1.0d0/2.0d0*u*v*zeta3
     .        + 5.0d0/6.0d0*u*v**2*pi**2
     .        - 10.0d0/9.0d0*u**2*pi**2
     .        - 1.0d0/4.0d0*u**2*zeta3
     .        + 5.0d0/3.0d0*u**2*v*pi**2
     .        + 5.0d0/6.0d0*u**3*pi**2
     .        - 16499.0d0/864.0d0*HPL1(0)
     .        + 5.0d0/8.0d0*HPL1(0)*u**(-1)*v
     .        - 5.0d0/8.0d0*HPL1(0)*u**(-1)*v**2
     .        - 377.0d0/48.0d0*HPL1(0)*pi*ci
     .        - 1.0d0/16.0d0*HPL1(0)*pi**2
     .        - 7.0d0/24.0d0*HPL1(0)*pi**3*ci
     .        - 5.0d0/2.0d0*HPL1(0)*zeta3
     .        + 5.0d0/3.0d0*HPL1(0)*v*omu**(-1)
     .        - 19.0d0/48.0d0*HPL1(0)*v*upv**(-1)
         col1 = col1 + 13.0d0/6.0d0*HPL1(0)*v
     .        + 7.0d0/12.0d0*HPL1(0)*v*pi*ci
     .        - 1.0d0/12.0d0*HPL1(0)*v*pi**2
     .        + 4.0d0/3.0d0*HPL1(0)*v**2*omu**(-2)
     .        - 5.0d0/4.0d0*HPL1(0)*v**2
     .        + 2.0d0/3.0d0*HPL1(0)*v**2*pi*ci
     .        - 1.0d0/24.0d0*HPL1(0)*v**2*pi**2
     .        + 55.0d0/48.0d0*HPL1(0)*u
     .        + 5.0d0/3.0d0*HPL1(0)*u*pi*ci
     .        + 1.0d0/12.0d0*HPL1(0)*u*pi**2
     .        - 5.0d0/4.0d0*HPL1(0)*u*v
     .        - HPL1(0)*u*v*pi*ci
     .        - 1.0d0/12.0d0*HPL1(0)*u*v*pi**2
     .        + 5.0d0/4.0d0*HPL1(0)*u*v**2*pi*ci
     .        - 5.0d0/3.0d0*HPL1(0)*u**2*pi*ci
         col1 = col1 + 1.0d0/24.0d0*HPL1(0)*u**2*pi**2
     .        + 5.0d0/2.0d0*HPL1(0)*u**2*v*pi*ci
     .        + 5.0d0/4.0d0*HPL1(0)*u**3*pi*ci
     .        - 31.0d0/144.0d0*HPL1(0)*GPL1(0)
     .        + 7.0d0/4.0d0*HPL1(0)*GPL1(0)*pi*ci
     .        - 13.0d0/24.0d0*HPL1(0)*GPL1(0)*pi**2
     .        + 17.0d0/24.0d0*HPL1(0)*GPL1(0)*v
     .        + 1.0d0/4.0d0*HPL1(0)*GPL1(0)*v*pi*ci
     .        + 2.0d0/3.0d0*HPL1(0)*GPL1(0)*v**2
     .        + 1.0d0/8.0d0*HPL1(0)*GPL1(0)*v**2*pi*ci
     .        + 5.0d0/3.0d0*HPL1(0)*GPL1(0)*u
     .        - HPL1(0)*GPL1(0)*u*v
     .        + 1.0d0/4.0d0*HPL1(0)*GPL1(0)*u*v*pi*ci
     .        + 5.0d0/4.0d0*HPL1(0)*GPL1(0)*u*v**2
     .        - 5.0d0/3.0d0*HPL1(0)*GPL1(0)*u**2
         col1 = col1 + 5.0d0/2.0d0*HPL1(0)*GPL1(0)*u**2*v
     .        + 5.0d0/4.0d0*HPL1(0)*GPL1(0)*u**3
     .        + 313.0d0/36.0d0*HPL1(0)*GPL1(1)
     .        - 5.0d0/8.0d0*HPL1(0)*GPL1(1)*u**(-2)
     .        + 5.0d0/4.0d0*HPL1(0)*GPL1(1)*u**(-2)*v
     .        - 5.0d0/8.0d0*HPL1(0)*GPL1(1)*u**(-2)*v**2
     .        - 7.0d0/8.0d0*HPL1(0)*GPL1(1)*u**(-1)
     .        + HPL1(0)*GPL1(1)*u**(-1)*v
     .        - 1.0d0/8.0d0*HPL1(0)*GPL1(1)*u**(-1)*v**2
     .        - 1.0d0/12.0d0*HPL1(0)*GPL1(1)*pi*ci
     .        + 7.0d0/12.0d0*HPL1(0)*GPL1(1)*pi**2
     .        - 5.0d0/6.0d0*HPL1(0)*GPL1(1)*v
     .        - 2.0d0/3.0d0*HPL1(0)*GPL1(1)*v**2
     .        - 5.0d0/3.0d0*HPL1(0)*GPL1(1)*u
     .        + HPL1(0)*GPL1(1)*u*v
         col1 = col1 - 5.0d0/4.0d0*HPL1(0)*GPL1(1)*u*v**2
     .        + 5.0d0/3.0d0*HPL1(0)*GPL1(1)*u**2
     .        - 5.0d0/2.0d0*HPL1(0)*GPL1(1)*u**2*v
     .        - 5.0d0/4.0d0*HPL1(0)*GPL1(1)*u**3
     .        - 179.0d0/48.0d0*HPL1(0)*GPL1(2)
     .        + 5.0d0/8.0d0*HPL1(0)*GPL1(2)*u**(-2)
     .        - 5.0d0/4.0d0*HPL1(0)*GPL1(2)*u**(-2)*v
     .        + 5.0d0/8.0d0*HPL1(0)*GPL1(2)*u**(-2)*v**2
     .        + 7.0d0/8.0d0*HPL1(0)*GPL1(2)*u**(-1)
     .        - HPL1(0)*GPL1(2)*u**(-1)*v
     .        + 1.0d0/8.0d0*HPL1(0)*GPL1(2)*u**(-1)*v**2
     .        - 9.0d0/8.0d0*HPL1(0)*GPL1(2)*pi*ci
     .        - 5.0d0/12.0d0*HPL1(0)*GPL1(2)*pi**2
     .        + 17.0d0/24.0d0*HPL1(0)*GPL1(2)*v
     .        + 2.0d0/3.0d0*HPL1(0)*GPL1(2)*v**2
         col1 = col1 + 5.0d0/3.0d0*HPL1(0)*GPL1(2)*u
     .        - HPL1(0)*GPL1(2)*u*v
     .        + 5.0d0/4.0d0*HPL1(0)*GPL1(2)*u*v**2
     .        - 5.0d0/3.0d0*HPL1(0)*GPL1(2)*u**2
     .        + 5.0d0/2.0d0*HPL1(0)*GPL1(2)*u**2*v
     .        + 5.0d0/4.0d0*HPL1(0)*GPL1(2)*u**3
     .        + 581.0d0/144.0d0*HPL1(0)*GPL1(3)
     .        + 1.0d0/8.0d0*HPL1(0)*GPL1(3)*pi*ci
     .        + 19.0d0/24.0d0*HPL1(0)*GPL1(3)*pi**2
     .        - 7.0d0/12.0d0*HPL1(0)*GPL1(3)*v
     .        + 1.0d0/4.0d0*HPL1(0)*GPL1(3)*v*pi*ci
     .        - 2.0d0/3.0d0*HPL1(0)*GPL1(3)*v**2
     .        + 1.0d0/8.0d0*HPL1(0)*GPL1(3)*v**2*pi*ci
     .        - 5.0d0/3.0d0*HPL1(0)*GPL1(3)*u
     .        + 1.0d0/4.0d0*HPL1(0)*GPL1(3)*u*pi*ci
         col1 = col1 + HPL1(0)*GPL1(3)*u*v
     .        + 1.0d0/4.0d0*HPL1(0)*GPL1(3)*u*v*pi*ci
     .        - 5.0d0/4.0d0*HPL1(0)*GPL1(3)*u*v**2
     .        + 5.0d0/3.0d0*HPL1(0)*GPL1(3)*u**2
     .        + 1.0d0/8.0d0*HPL1(0)*GPL1(3)*u**2*pi*ci
     .        - 5.0d0/2.0d0*HPL1(0)*GPL1(3)*u**2*v
     .        - 5.0d0/4.0d0*HPL1(0)*GPL1(3)*u**3
     .        + 7.0d0/4.0d0*HPL1(0)*GPL2(0,0)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(0,0)*pi*ci
     .        + 1.0d0/4.0d0*HPL1(0)*GPL2(0,0)*v
     .        + 1.0d0/8.0d0*HPL1(0)*GPL2(0,0)*v**2
     .        + 1.0d0/4.0d0*HPL1(0)*GPL2(0,0)*u
     .        + 1.0d0/4.0d0*HPL1(0)*GPL2(0,0)*u*v
     .        + 1.0d0/8.0d0*HPL1(0)*GPL2(0,0)*u**2
     .        - 1.0d0/12.0d0*HPL1(0)*GPL2(0,1)
         col1 = col1 + 1.0d0/4.0d0*HPL1(0)*GPL2(0,1)*pi*ci
     .        - 1.0d0/4.0d0*HPL1(0)*GPL2(0,1)*v
     .        - 1.0d0/8.0d0*HPL1(0)*GPL2(0,1)*v**2
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(0,1)*u
     .        - 1.0d0/4.0d0*HPL1(0)*GPL2(0,1)*u*v
     .        - 1.0d0/4.0d0*HPL1(0)*GPL2(0,1)*u**2
     .        + 19.0d0/8.0d0*HPL1(0)*GPL2(0,2)
     .        + HPL1(0)*GPL2(0,2)*pi*ci
     .        + 1.0d0/4.0d0*HPL1(0)*GPL2(0,2)*v
     .        + 1.0d0/8.0d0*HPL1(0)*GPL2(0,2)*v**2
     .        + 1.0d0/4.0d0*HPL1(0)*GPL2(0,2)*u
     .        + 1.0d0/4.0d0*HPL1(0)*GPL2(0,2)*u*v
     .        + 1.0d0/8.0d0*HPL1(0)*GPL2(0,2)*u**2
     .        - 41.0d0/24.0d0*HPL1(0)*GPL2(0,3)
     .        - 1.0d0/4.0d0*HPL1(0)*GPL2(0,3)*pi*ci
         col1 = col1 - 1.0d0/4.0d0*HPL1(0)*GPL2(0,3)*v
     .        - 1.0d0/8.0d0*HPL1(0)*GPL2(0,3)*v**2
     .        - 1.0d0/4.0d0*HPL1(0)*GPL2(0,3)*u*v
     .        - 11.0d0/24.0d0*HPL1(0)*GPL2(1,0)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(1,0)*pi*ci
     .        - 1.0d0/4.0d0*HPL1(0)*GPL2(1,0)*u
     .        - 1.0d0/8.0d0*HPL1(0)*GPL2(1,0)*u**2
     .        + 13.0d0/12.0d0*HPL1(0)*GPL2(1,1)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(1,1)*u
     .        + 1.0d0/4.0d0*HPL1(0)*GPL2(1,1)*u**2
     .        - 127.0d0/24.0d0*HPL1(0)*GPL2(1,2)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(1,2)*pi*ci
     .        - 1.0d0/4.0d0*HPL1(0)*GPL2(1,2)*u
     .        - 1.0d0/8.0d0*HPL1(0)*GPL2(1,2)*u**2
     .        + 23.0d0/6.0d0*HPL1(0)*GPL2(2,0)
         col1 = col1 + 3.0d0/4.0d0*HPL1(0)*GPL2(2,0)*pi*ci
     .        + 1.0d0/4.0d0*HPL1(0)*GPL2(2,0)*v
     .        + 1.0d0/8.0d0*HPL1(0)*GPL2(2,0)*v**2
     .        + 1.0d0/4.0d0*HPL1(0)*GPL2(2,0)*u
     .        + 1.0d0/4.0d0*HPL1(0)*GPL2(2,0)*u*v
     .        + 1.0d0/8.0d0*HPL1(0)*GPL2(2,0)*u**2
     .        - 35.0d0/8.0d0*HPL1(0)*GPL2(2,1)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(2,1)*pi*ci
     .        - 1.0d0/4.0d0*HPL1(0)*GPL2(2,1)*v
     .        - 1.0d0/8.0d0*HPL1(0)*GPL2(2,1)*v**2
     .        - 1.0d0/4.0d0*HPL1(0)*GPL2(2,1)*u
     .        - 1.0d0/4.0d0*HPL1(0)*GPL2(2,1)*u*v
     .        - 1.0d0/8.0d0*HPL1(0)*GPL2(2,1)*u**2
     .        + 4.0d0*HPL1(0)*GPL2(2,2)
     .        - HPL1(0)*GPL2(2,2)*pi*ci
         col1 = col1 - 13.0d0/12.0d0*HPL1(0)*GPL2(2,3)
     .        - 13.0d0/6.0d0*HPL1(0)*GPL2(3,0)
     .        - HPL1(0)*GPL2(3,0)*pi*ci
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(3,2)*pi*ci
     .        + 1.0d0/4.0d0*HPL1(0)*GPL2(3,2)*v
     .        + 1.0d0/8.0d0*HPL1(0)*GPL2(3,2)*v**2
     .        + 1.0d0/4.0d0*HPL1(0)*GPL2(3,2)*u
     .        + 1.0d0/4.0d0*HPL1(0)*GPL2(3,2)*u*v
     .        + 1.0d0/8.0d0*HPL1(0)*GPL2(3,2)*u**2
     .        - 1.0d0/6.0d0*HPL1(0)*GPL2(3,3)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(3,3)*pi*ci
     .        - 1.0d0/4.0d0*HPL1(0)*GPL2(3,3)*v
     .        - 1.0d0/8.0d0*HPL1(0)*GPL2(3,3)*v**2
     .        - 1.0d0/4.0d0*HPL1(0)*GPL2(3,3)*u
     .        - 1.0d0/4.0d0*HPL1(0)*GPL2(3,3)*u*v
         col1 = col1 - 1.0d0/8.0d0*HPL1(0)*GPL2(3,3)*u**2
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(0,0,2)
     .        + 3.0d0/4.0d0*HPL1(0)*GPL3(0,1,0)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL3(0,1,2)
     .        + 1.0d0/4.0d0*HPL1(0)*GPL3(0,2,0)
     .        - 1.0d0/4.0d0*HPL1(0)*GPL3(0,2,1)
     .        + HPL1(0)*GPL3(0,2,2)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL3(0,2,3)
     .        - 1.0d0/4.0d0*HPL1(0)*GPL3(0,3,0)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(0,3,3)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL3(1,0,2)
     .        - 2.0d0*HPL1(0)*GPL3(1,1,1)
     .        + 2.0d0*HPL1(0)*GPL3(1,1,2)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL3(1,2,0)
     .        + 2.0d0*HPL1(0)*GPL3(1,2,1)
         col1 = col1 - 2.0d0*HPL1(0)*GPL3(1,2,2)
     .        + 3.0d0/4.0d0*HPL1(0)*GPL3(2,0,0)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL3(2,0,1)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(2,0,2)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL3(2,0,3)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL3(2,1,0)
     .        + 2.0d0*HPL1(0)*GPL3(2,1,1)
     .        - HPL1(0)*GPL3(2,1,2)
     .        - HPL1(0)*GPL3(2,2,1)
     .        + HPL1(0)*GPL3(2,2,3)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL3(2,3,0)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(2,3,2)
     .        + 1.0d0/4.0d0*HPL1(0)*GPL3(2,3,3)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL3(3,0,0)
     .        - 1.0d0/4.0d0*HPL1(0)*GPL3(3,0,1)
         col1 = col1 - 1.0d0/4.0d0*HPL1(0)*GPL3(3,0,2)
     .        + 1.0d0/4.0d0*HPL1(0)*GPL3(3,0,3)
     .        - 1.0d0/4.0d0*HPL1(0)*GPL3(3,2,0)
     .        - 1.0d0/4.0d0*HPL1(0)*GPL3(3,2,1)
     .        - 1.0d0/4.0d0*HPL1(0)*GPL3(3,2,3)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(3,3,0)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL3(3,3,3)
     .        - 9965.0d0/864.0d0*HPL1(1)
     .        - 5.0d0/8.0d0*HPL1(1)*u**(-1)
     .        + 5.0d0/4.0d0*HPL1(1)*u**(-1)*v
     .        - 5.0d0/8.0d0*HPL1(1)*u**(-1)*v**2
     .        - 5.0d0*HPL1(1)*pi*ci
     .        + 61.0d0/144.0d0*HPL1(1)*pi**2
     .        - 7.0d0/24.0d0*HPL1(1)*pi**3*ci
     .        + 1.0d0/2.0d0*HPL1(1)*zeta3
         col1 = col1 + 5.0d0/3.0d0*HPL1(1)*v*omu**(-1)
     .        + HPL1(1)*v
     .        + 11.0d0/24.0d0*HPL1(1)*v*pi*ci
     .        - 1.0d0/12.0d0*HPL1(1)*v*pi**2
     .        + 4.0d0/3.0d0*HPL1(1)*v**2*omu**(-2)
     .        - 5.0d0/4.0d0*HPL1(1)*v**2
     .        + 2.0d0/3.0d0*HPL1(1)*v**2*pi*ci
     .        - 1.0d0/24.0d0*HPL1(1)*v**2*pi**2
     .        + 13.0d0/8.0d0*HPL1(1)*u
     .        + 37.0d0/24.0d0*HPL1(1)*u*pi*ci
     .        - 1.0d0/12.0d0*HPL1(1)*u*pi**2
     .        - 5.0d0/2.0d0*HPL1(1)*u*v
     .        - HPL1(1)*u*v*pi*ci
     .        - 1.0d0/12.0d0*HPL1(1)*u*v*pi**2
     .        + 5.0d0/4.0d0*HPL1(1)*u*v**2*pi*ci
         col1 = col1 - 5.0d0/4.0d0*HPL1(1)*u**2
     .        - 5.0d0/3.0d0*HPL1(1)*u**2*pi*ci
     .        - 1.0d0/24.0d0*HPL1(1)*u**2*pi**2
     .        + 5.0d0/2.0d0*HPL1(1)*u**2*v*pi*ci
     .        + 5.0d0/4.0d0*HPL1(1)*u**3*pi*ci
     .        - 115.0d0/36.0d0*HPL1(1)*GPL1(0)
     .        + 13.0d0/12.0d0*HPL1(1)*GPL1(0)*pi*ci
     .        - 5.0d0/12.0d0*HPL1(1)*GPL1(0)*pi**2
     .        + 11.0d0/24.0d0*HPL1(1)*GPL1(0)*v
     .        + 1.0d0/4.0d0*HPL1(1)*GPL1(0)*v*pi*ci
     .        + 2.0d0/3.0d0*HPL1(1)*GPL1(0)*v**2
     .        + 1.0d0/8.0d0*HPL1(1)*GPL1(0)*v**2*pi*ci
     .        + 37.0d0/24.0d0*HPL1(1)*GPL1(0)*u
     .        + 1.0d0/4.0d0*HPL1(1)*GPL1(0)*u*pi*ci
     .        - HPL1(1)*GPL1(0)*u*v
         col1 = col1 + 1.0d0/4.0d0*HPL1(1)*GPL1(0)*u*v*pi*ci
     .        + 5.0d0/4.0d0*HPL1(1)*GPL1(0)*u*v**2
     .        - 5.0d0/3.0d0*HPL1(1)*GPL1(0)*u**2
     .        + 1.0d0/8.0d0*HPL1(1)*GPL1(0)*u**2*pi*ci
     .        + 5.0d0/2.0d0*HPL1(1)*GPL1(0)*u**2*v
     .        + 5.0d0/4.0d0*HPL1(1)*GPL1(0)*u**3
     .        + 313.0d0/36.0d0*HPL1(1)*GPL1(1)
     .        - 5.0d0/8.0d0*HPL1(1)*GPL1(1)*u**(-2)
     .        + 5.0d0/4.0d0*HPL1(1)*GPL1(1)*u**(-2)*v
     .        - 5.0d0/8.0d0*HPL1(1)*GPL1(1)*u**(-2)*v**2
     .        - 7.0d0/8.0d0*HPL1(1)*GPL1(1)*u**(-1)
     .        + HPL1(1)*GPL1(1)*u**(-1)*v
     .        - 1.0d0/8.0d0*HPL1(1)*GPL1(1)*u**(-1)*v**2
     .        + 1.0d0/8.0d0*HPL1(1)*GPL1(1)*pi*ci
     .        + 5.0d0/12.0d0*HPL1(1)*GPL1(1)*pi**2
         col1 = col1 - 5.0d0/6.0d0*HPL1(1)*GPL1(1)*v
     .        - 2.0d0/3.0d0*HPL1(1)*GPL1(1)*v**2
     .        - 5.0d0/3.0d0*HPL1(1)*GPL1(1)*u
     .        - 1.0d0/4.0d0*HPL1(1)*GPL1(1)*u*pi*ci
     .        + HPL1(1)*GPL1(1)*u*v
     .        - 5.0d0/4.0d0*HPL1(1)*GPL1(1)*u*v**2
     .        + 5.0d0/3.0d0*HPL1(1)*GPL1(1)*u**2
     .        - 1.0d0/8.0d0*HPL1(1)*GPL1(1)*u**2*pi*ci
     .        - 5.0d0/2.0d0*HPL1(1)*GPL1(1)*u**2*v
     .        - 5.0d0/4.0d0*HPL1(1)*GPL1(1)*u**3
     .        + 455.0d0/144.0d0*HPL1(1)*GPL1(2)
     .        + 2.0d0*HPL1(1)*GPL1(2)*pi*ci
     .        + 1.0d0/8.0d0*HPL1(1)*GPL1(2)*pi**2
     .        + 1.0d0/3.0d0*HPL1(1)*GPL1(3)*pi**2
     .        + 1.0d0/4.0d0*HPL1(1)*GPL1(3)*v*pi*ci
         col1 = col1 + 1.0d0/8.0d0*HPL1(1)*GPL1(3)*v**2*pi*ci
     .        + 1.0d0/4.0d0*HPL1(1)*GPL1(3)*u*pi*ci
     .        + 1.0d0/4.0d0*HPL1(1)*GPL1(3)*u*v*pi*ci
     .        + 1.0d0/8.0d0*HPL1(1)*GPL1(3)*u**2*pi*ci
     .        + 11.0d0/8.0d0*HPL1(1)*GPL2(0,0)
     .        + 1.0d0/2.0d0*HPL1(1)*GPL2(0,0)*pi*ci
     .        + 1.0d0/4.0d0*HPL1(1)*GPL2(0,0)*v
     .        + 1.0d0/8.0d0*HPL1(1)*GPL2(0,0)*v**2
     .        + 1.0d0/4.0d0*HPL1(1)*GPL2(0,0)*u
     .        + 1.0d0/4.0d0*HPL1(1)*GPL2(0,0)*u*v
     .        + 1.0d0/8.0d0*HPL1(1)*GPL2(0,0)*u**2
     .        - 1.0d0/12.0d0*HPL1(1)*GPL2(0,1)
     .        - 1.0d0/4.0d0*HPL1(1)*GPL2(0,1)*pi*ci
     .        - 1.0d0/4.0d0*HPL1(1)*GPL2(0,1)*v
     .        - 1.0d0/8.0d0*HPL1(1)*GPL2(0,1)*v**2
         col1 = col1 - 1.0d0/2.0d0*HPL1(1)*GPL2(0,1)*u
     .        - 1.0d0/4.0d0*HPL1(1)*GPL2(0,1)*u*v
     .        - 1.0d0/4.0d0*HPL1(1)*GPL2(0,1)*u**2
     .        + 2.0d0*HPL1(1)*GPL2(0,2)
     .        + 1.0d0/2.0d0*HPL1(1)*GPL2(0,2)*pi*ci
     .        - 11.0d0/24.0d0*HPL1(1)*GPL2(1,0)
     .        - HPL1(1)*GPL2(1,0)*pi*ci
     .        - 1.0d0/4.0d0*HPL1(1)*GPL2(1,0)*u
     .        - 1.0d0/8.0d0*HPL1(1)*GPL2(1,0)*u**2
     .        + 13.0d0/12.0d0*HPL1(1)*GPL2(1,1)
     .        + 2.0d0*HPL1(1)*GPL2(1,1)*pi*ci
     .        + 1.0d0/2.0d0*HPL1(1)*GPL2(1,1)*u
     .        + 1.0d0/4.0d0*HPL1(1)*GPL2(1,1)*u**2
     .        - 4.0d0*HPL1(1)*GPL2(1,2)
     .        - HPL1(1)*GPL2(1,2)*pi*ci
         col1 = col1 + 2.0d0*HPL1(1)*GPL2(2,0)
     .        + 1.0d0/2.0d0*HPL1(1)*GPL2(2,0)*pi*ci
     .        - 35.0d0/8.0d0*HPL1(1)*GPL2(2,1)
     .        - 1.0d0/2.0d0*HPL1(1)*GPL2(2,1)*pi*ci
     .        - 1.0d0/4.0d0*HPL1(1)*GPL2(2,1)*v
     .        - 1.0d0/8.0d0*HPL1(1)*GPL2(2,1)*v**2
     .        - 1.0d0/4.0d0*HPL1(1)*GPL2(2,1)*u
     .        - 1.0d0/4.0d0*HPL1(1)*GPL2(2,1)*u*v
     .        - 1.0d0/8.0d0*HPL1(1)*GPL2(2,1)*u**2
     .        + 1.0d0/2.0d0*HPL1(1)*GPL2(2,3)*pi*ci
     .        - 1.0d0/2.0d0*HPL1(1)*GPL2(3,0)*pi*ci
     .        + 1.0d0/4.0d0*HPL1(1)*GPL2(3,0)*v
     .        + 1.0d0/8.0d0*HPL1(1)*GPL2(3,0)*v**2
     .        + 1.0d0/4.0d0*HPL1(1)*GPL2(3,0)*u
     .        + 1.0d0/4.0d0*HPL1(1)*GPL2(3,0)*u*v
         col1 = col1 + 1.0d0/8.0d0*HPL1(1)*GPL2(3,0)*u**2
     .        + 1.0d0/2.0d0*HPL1(1)*GPL3(0,0,2)
     .        + 3.0d0/4.0d0*HPL1(1)*GPL3(0,1,0)
     .        - HPL1(1)*GPL3(0,1,2)
     .        + 1.0d0/2.0d0*HPL1(1)*GPL3(0,2,0)
     .        - 1.0d0/4.0d0*HPL1(1)*GPL3(0,2,1)
     .        - HPL1(1)*GPL3(1,0,2)
     .        - 2.0d0*HPL1(1)*GPL3(1,1,1)
     .        + 2.0d0*HPL1(1)*GPL3(1,1,2)
     .        - HPL1(1)*GPL3(1,2,0)
     .        + 2.0d0*HPL1(1)*GPL3(1,2,1)
     .        + 1.0d0/2.0d0*HPL1(1)*GPL3(2,0,0)
     .        - 1.0d0/2.0d0*HPL1(1)*GPL3(2,0,1)
     .        - 1.0d0/2.0d0*HPL1(1)*GPL3(2,1,0)
     .        + 2.0d0*HPL1(1)*GPL3(2,1,1)
         col1 = col1 - HPL1(1)*GPL3(2,2,1)
     .        + 1.0d0/2.0d0*HPL1(1)*GPL3(2,3,0)
     .        - 1.0d0/2.0d0*HPL1(1)*GPL3(3,0,0)
     .        - 1.0d0/4.0d0*HPL1(1)*GPL3(3,0,1)
     .        - 1.0d0/4.0d0*HPL1(1)*GPL3(3,2,1)
     .        - 617.0d0/72.0d0*HPL2(0,0)
     .        - 13.0d0/6.0d0*HPL2(0,0)*pi*ci
     .        + 19.0d0/24.0d0*HPL2(0,0)*pi**2
     .        - 7.0d0/12.0d0*HPL2(0,0)*v
     .        - 2.0d0/3.0d0*HPL2(0,0)*v**2
     .        - 5.0d0/3.0d0*HPL2(0,0)*u
     .        + 1.0d0/4.0d0*HPL2(0,0)*u*pi*ci
     .        + HPL2(0,0)*u*v
     .        - 5.0d0/4.0d0*HPL2(0,0)*u*v**2
     .        + 5.0d0/3.0d0*HPL2(0,0)*u**2
         col1 = col1 + 1.0d0/8.0d0*HPL2(0,0)*u**2*pi*ci
     .        - 5.0d0/2.0d0*HPL2(0,0)*u**2*v
     .        - 5.0d0/4.0d0*HPL2(0,0)*u**3
     .        - 107.0d0/24.0d0*HPL2(0,0)*GPL1(0)
     .        - HPL2(0,0)*GPL1(0)*pi*ci
     .        - 1.0d0/4.0d0*HPL2(0,0)*GPL1(0)*v
     .        - 1.0d0/8.0d0*HPL2(0,0)*GPL1(0)*v**2
     .        - 1.0d0/4.0d0*HPL2(0,0)*GPL1(0)*u*v
     .        + 11.0d0/2.0d0*HPL2(0,0)*GPL1(1)
     .        + HPL2(0,0)*GPL1(1)*pi*ci
     .        - 79.0d0/12.0d0*HPL2(0,0)*GPL1(2)
     .        - HPL2(0,0)*GPL1(2)*pi*ci
     .        + 31.0d0/12.0d0*HPL2(0,0)*GPL1(3)
     .        + 5.0d0/4.0d0*HPL2(0,0)*GPL1(3)*pi*ci
     .        - 1.0d0/4.0d0*HPL2(0,0)*GPL1(3)*v
         col1 = col1 - 1.0d0/8.0d0*HPL2(0,0)*GPL1(3)*v**2
     .        - 1.0d0/4.0d0*HPL2(0,0)*GPL1(3)*u
     .        - 1.0d0/4.0d0*HPL2(0,0)*GPL1(3)*u*v
     .        - 1.0d0/8.0d0*HPL2(0,0)*GPL1(3)*u**2
     .        - 1.0d0/2.0d0*HPL2(0,0)*GPL2(0,0)
     .        - HPL2(0,0)*GPL2(0,2)
     .        + 1.0d0/2.0d0*HPL2(0,0)*GPL2(0,3)
     .        + HPL2(0,0)*GPL2(1,2)
     .        - 1.0d0/2.0d0*HPL2(0,0)*GPL2(2,0)
     .        + 1.0d0/4.0d0*HPL2(0,0)*GPL2(2,3)
     .        + 3.0d0/4.0d0*HPL2(0,0)*GPL2(3,0)
     .        + 1.0d0/4.0d0*HPL2(0,0)*GPL2(3,2)
     .        - 1.0d0/2.0d0*HPL2(0,0)*GPL2(3,3)
     .        - 715.0d0/144.0d0*HPL2(0,1)
     .        - 55.0d0/24.0d0*HPL2(0,1)*pi*ci
         col1 = col1 + 1.0d0/3.0d0*HPL2(0,1)*pi**2
     .        + 1.0d0/8.0d0*HPL2(0,1)*v
     .        + 1.0d0/4.0d0*HPL2(0,1)*u*pi*ci
     .        + 1.0d0/8.0d0*HPL2(0,1)*u**2*pi*ci
     .        - 55.0d0/24.0d0*HPL2(0,1)*GPL1(0)
     .        - 1.0d0/2.0d0*HPL2(0,1)*GPL1(0)*pi*ci
     .        + 1.0d0/4.0d0*HPL2(0,1)*GPL1(0)*u
     .        + 1.0d0/8.0d0*HPL2(0,1)*GPL1(0)*u**2
     .        + 101.0d0/24.0d0*HPL2(0,1)*GPL1(1)
     .        + 1.0d0/2.0d0*HPL2(0,1)*GPL1(1)*pi*ci
     .        - 1.0d0/4.0d0*HPL2(0,1)*GPL1(1)*u
     .        - 1.0d0/8.0d0*HPL2(0,1)*GPL1(1)*u**2
     .        + 3.0d0/8.0d0*HPL2(0,1)*GPL1(2)
     .        + 1.0d0/4.0d0*HPL2(0,1)*GPL1(2)*v
     .        + 1.0d0/8.0d0*HPL2(0,1)*GPL1(2)*v**2
         col1 = col1 + 1.0d0/4.0d0*HPL2(0,1)*GPL1(2)*u
     .        + 1.0d0/4.0d0*HPL2(0,1)*GPL1(2)*u*v
     .        + 1.0d0/8.0d0*HPL2(0,1)*GPL1(2)*u**2
     .        + 1.0d0/2.0d0*HPL2(0,1)*GPL1(3)*pi*ci
     .        - 1.0d0/4.0d0*HPL2(0,1)*GPL1(3)*v
     .        - 1.0d0/8.0d0*HPL2(0,1)*GPL1(3)*v**2
     .        - 1.0d0/4.0d0*HPL2(0,1)*GPL1(3)*u
     .        - 1.0d0/4.0d0*HPL2(0,1)*GPL1(3)*u*v
     .        - 1.0d0/8.0d0*HPL2(0,1)*GPL1(3)*u**2
     .        - 1.0d0/2.0d0*HPL2(0,1)*GPL2(0,0)
     .        + 1.0d0/2.0d0*HPL2(0,1)*GPL2(0,1)
     .        - 3.0d0/4.0d0*HPL2(0,1)*GPL2(0,2)
     .        + 1.0d0/2.0d0*HPL2(0,1)*GPL2(1,0)
     .        - HPL2(0,1)*GPL2(2,1)
     .        + HPL2(0,1)*GPL2(2,2)
         col1 = col1 - 1.0d0/2.0d0*HPL2(0,1)*GPL2(2,3)
     .        + 1.0d0/2.0d0*HPL2(0,1)*GPL2(3,0)
     .        + 1.0d0/4.0d0*HPL2(0,1)*GPL2(3,2)
     .        - 49.0d0/72.0d0*HPL2(1,0)
     .        - 9.0d0/8.0d0*HPL2(1,0)*pi*ci
     .        + 5.0d0/8.0d0*HPL2(1,0)*pi**2
     .        - 1.0d0/3.0d0*HPL2(1,0)*v
     .        - 2.0d0/3.0d0*HPL2(1,0)*v**2
     .        - 37.0d0/24.0d0*HPL2(1,0)*u
     .        + HPL2(1,0)*u*v
     .        - 5.0d0/4.0d0*HPL2(1,0)*u*v**2
     .        + 5.0d0/3.0d0*HPL2(1,0)*u**2
     .        - 5.0d0/2.0d0*HPL2(1,0)*u**2*v
     .        - 5.0d0/4.0d0*HPL2(1,0)*u**3
     .        - 41.0d0/12.0d0*HPL2(1,0)*GPL1(0)
         col1 = col1 - HPL2(1,0)*GPL1(0)*pi*ci
     .        - 1.0d0/4.0d0*HPL2(1,0)*GPL1(0)*v
     .        - 1.0d0/8.0d0*HPL2(1,0)*GPL1(0)*v**2
     .        - 1.0d0/4.0d0*HPL2(1,0)*GPL1(0)*u
     .        - 1.0d0/4.0d0*HPL2(1,0)*GPL1(0)*u*v
     .        - 1.0d0/8.0d0*HPL2(1,0)*GPL1(0)*u**2
     .        + 127.0d0/24.0d0*HPL2(1,0)*GPL1(1)
     .        + 1.0d0/2.0d0*HPL2(1,0)*GPL1(1)*pi*ci
     .        + 1.0d0/4.0d0*HPL2(1,0)*GPL1(1)*u
     .        + 1.0d0/8.0d0*HPL2(1,0)*GPL1(1)*u**2
     .        - 35.0d0/12.0d0*HPL2(1,0)*GPL1(2)
     .        + 3.0d0/4.0d0*HPL2(1,0)*GPL1(2)*pi*ci
     .        + 1.0d0/2.0d0*HPL2(1,0)*GPL1(3)*pi*ci
     .        - 1.0d0/4.0d0*HPL2(1,0)*GPL1(3)*v
     .        - 1.0d0/8.0d0*HPL2(1,0)*GPL1(3)*v**2
         col1 = col1 - 1.0d0/4.0d0*HPL2(1,0)*GPL1(3)*u
     .        - 1.0d0/4.0d0*HPL2(1,0)*GPL1(3)*u*v
     .        - 1.0d0/8.0d0*HPL2(1,0)*GPL1(3)*u**2
     .        - 3.0d0/4.0d0*HPL2(1,0)*GPL2(0,0)
     .        + 1.0d0/2.0d0*HPL2(1,0)*GPL2(0,1)
     .        - 3.0d0/4.0d0*HPL2(1,0)*GPL2(0,2)
     .        + 1.0d0/2.0d0*HPL2(1,0)*GPL2(1,0)
     .        - 2.0d0*HPL2(1,0)*GPL2(1,1)
     .        + 2.0d0*HPL2(1,0)*GPL2(1,2)
     .        + 1.0d0/4.0d0*HPL2(1,0)*GPL2(2,0)
     .        + HPL2(1,0)*GPL2(2,1)
     .        - HPL2(1,0)*GPL2(2,2)
     .        - 1.0d0/2.0d0*HPL2(1,0)*GPL2(2,3)
     .        + 1.0d0/4.0d0*HPL2(1,0)*GPL2(3,0)
     .        - 455.0d0/144.0d0*HPL2(1,1)
         col1 = col1 - 2.0d0*HPL2(1,1)*pi*ci
     .        - 1.0d0/24.0d0*HPL2(1,1)*pi**2
     .        - 2.0d0*HPL2(1,1)*GPL1(0)
     .        - 1.0d0/2.0d0*HPL2(1,1)*GPL1(0)*pi*ci
     .        + 4.0d0*HPL2(1,1)*GPL1(1)
     .        + HPL2(1,1)*GPL1(1)*pi*ci
     .        - 1.0d0/2.0d0*HPL2(1,1)*GPL2(0,0)
     .        + HPL2(1,1)*GPL2(0,1)
     .        + HPL2(1,1)*GPL2(1,0)
     .        - 2.0d0*HPL2(1,1)*GPL2(1,1)
     .        + 39.0d0/8.0d0*HPL3(0,0,0)
     .        + 5.0d0/4.0d0*HPL3(0,0,0)*pi*ci
     .        - 1.0d0/4.0d0*HPL3(0,0,0)*u
     .        - 1.0d0/8.0d0*HPL3(0,0,0)*u**2
     .        + 3.0d0/4.0d0*HPL3(0,0,0)*GPL1(0)
         col1 = col1 - HPL3(0,0,0)*GPL1(1)
     .        + 5.0d0/4.0d0*HPL3(0,0,0)*GPL1(2)
     .        - 3.0d0/4.0d0*HPL3(0,0,0)*GPL1(3)
     .        + 1.0d0/2.0d0*HPL3(0,0,1)*pi*ci
     .        - 1.0d0/4.0d0*HPL3(0,0,1)*v
     .        - 1.0d0/8.0d0*HPL3(0,0,1)*v**2
     .        - 1.0d0/4.0d0*HPL3(0,0,1)*u
     .        - 1.0d0/4.0d0*HPL3(0,0,1)*u*v
     .        - 1.0d0/8.0d0*HPL3(0,0,1)*u**2
     .        + 1.0d0/2.0d0*HPL3(0,0,1)*GPL1(0)
     .        - HPL3(0,0,1)*GPL1(1)
     .        + 1.0d0/2.0d0*HPL3(0,0,1)*GPL1(2)
     .        - 1.0d0/2.0d0*HPL3(0,0,1)*GPL1(3)
     .        + 55.0d0/24.0d0*HPL3(0,1,0)
     .        + 1.0d0/2.0d0*HPL3(0,1,0)*pi*ci
         col1 = col1 - 1.0d0/4.0d0*HPL3(0,1,0)*u
     .        - 1.0d0/8.0d0*HPL3(0,1,0)*u**2
     .        + 1.0d0/4.0d0*HPL3(0,1,0)*GPL1(0)
     .        - HPL3(0,1,0)*GPL1(1)
     .        + 1.0d0/2.0d0*HPL3(0,1,0)*GPL1(2)
     .        - 1.0d0/4.0d0*HPL3(0,1,0)*GPL1(3)
     .        + 23.0d0/6.0d0*HPL3(1,0,0)
     .        + HPL3(1,0,0)*pi*ci
     .        + HPL3(1,0,0)*GPL1(0)
     .        - HPL3(1,0,0)*GPL1(1)
     .        - 1.0d0/4.0d0*HPL3(1,0,0)*GPL1(3)
     .        + 1.0d0/4.0d0*HPL3(1,0,1)*pi*ci
     .        - 1.0d0/4.0d0*HPL3(1,0,1)*v
     .        - 1.0d0/8.0d0*HPL3(1,0,1)*v**2
     .        - 1.0d0/4.0d0*HPL3(1,0,1)*u
         col1 = col1 - 1.0d0/4.0d0*HPL3(1,0,1)*u*v
     .        - 1.0d0/8.0d0*HPL3(1,0,1)*u**2
     .        + 1.0d0/4.0d0*HPL3(1,0,1)*GPL1(0)
     .        - 1.0d0/2.0d0*HPL3(1,0,1)*GPL1(2)
     .        - 1.0d0/4.0d0*HPL3(1,0,1)*GPL1(3)
     .        + 19.0d0/8.0d0*HPL3(1,1,0)
     .        - 3.0d0/4.0d0*HPL3(1,1,0)*pi*ci
     .        - 2.0d0*HPL3(1,1,0)*GPL1(1)
     .        + 7.0d0/4.0d0*HPL3(1,1,0)*GPL1(2)
     .        - 3.0d0/4.0d0*HPL4(0,0,0,0)
     .        - 1.0d0/2.0d0*HPL4(0,0,0,1)
     .        - 1.0d0/4.0d0*HPL4(0,0,1,0)
     .        - 1.0d0/4.0d0*HPL4(0,1,0,0)
     .        - 1.0d0/4.0d0*HPL4(0,1,0,1)
     .        - 3.0d0/4.0d0*HPL4(1,0,0,0)
         col1 = col1 + 1.0d0/2.0d0*HPL4(1,1,0,0)
     .        + 3.0d0/4.0d0*HPL4(1,1,0,1)
     .        - HPL4(1,1,1,0)
     .        + 51.0d0/16.0d0*GPL1(0)
     .        + 59.0d0/12.0d0*GPL1(0)*pi*ci
     .        - 29.0d0/96.0d0*GPL1(0)*pi**2
     .        + 1.0d0/24.0d0*GPL1(0)*pi**3*ci
     .        + 1.0d0/2.0d0*GPL1(0)*zeta3
     .        + 19.0d0/48.0d0*GPL1(0)*v*upv**(-1)
     .        - 1.0d0/2.0d0*GPL1(0)*v
     .        - 7.0d0/12.0d0*GPL1(0)*v*pi*ci
     .        + 1.0d0/6.0d0*GPL1(0)*v*pi**2
     .        - 2.0d0/3.0d0*GPL1(0)*v**2*pi*ci
     .        + 1.0d0/12.0d0*GPL1(0)*v**2*pi**2
     .        + 55.0d0/48.0d0*GPL1(0)*u
         col1 = col1 - 5.0d0/3.0d0*GPL1(0)*u*pi*ci
     .        + 1.0d0/12.0d0*GPL1(0)*u*pi**2
     .        - 5.0d0/4.0d0*GPL1(0)*u*v
     .        + GPL1(0)*u*v*pi*ci
     .        + 1.0d0/6.0d0*GPL1(0)*u*v*pi**2
     .        - 5.0d0/4.0d0*GPL1(0)*u*v**2*pi*ci
     .        - 5.0d0/4.0d0*GPL1(0)*u**2
     .        + 5.0d0/3.0d0*GPL1(0)*u**2*pi*ci
     .        + 1.0d0/24.0d0*GPL1(0)*u**2*pi**2
     .        - 5.0d0/2.0d0*GPL1(0)*u**2*v*pi*ci
     .        - 5.0d0/4.0d0*GPL1(0)*u**3*pi*ci
     .        + 5.0d0/8.0d0*GPL1(1)*u**(-2)*pi*ci
     .        - 5.0d0/4.0d0*GPL1(1)*u**(-2)*v*pi*ci
     .        + 5.0d0/8.0d0*GPL1(1)*u**(-2)*v**2*pi*ci
     .        + 7.0d0/8.0d0*GPL1(1)*u**(-1)*pi*ci
         col1 = col1 - GPL1(1)*u**(-1)*v*pi*ci
     .        + 1.0d0/8.0d0*GPL1(1)*u**(-1)*v**2*pi*ci
     .        - 313.0d0/36.0d0*GPL1(1)*pi*ci
     .        + 179.0d0/72.0d0*GPL1(1)*pi**2
     .        - 1.0d0/4.0d0*GPL1(1)*pi**3*ci
     .        + 2.0d0*GPL1(1)*zeta3
     .        + 5.0d0/6.0d0*GPL1(1)*v*pi*ci
     .        + 2.0d0/3.0d0*GPL1(1)*v**2*pi*ci
     .        + 5.0d0/3.0d0*GPL1(1)*u*pi*ci
     .        - 1.0d0/12.0d0*GPL1(1)*u*pi**2
     .        - GPL1(1)*u*v*pi*ci
     .        + 5.0d0/4.0d0*GPL1(1)*u*v**2*pi*ci
     .        - 5.0d0/3.0d0*GPL1(1)*u**2*pi*ci
     .        - 1.0d0/24.0d0*GPL1(1)*u**2*pi**2
     .        + 5.0d0/2.0d0*GPL1(1)*u**2*v*pi*ci
         col1 = col1 + 5.0d0/4.0d0*GPL1(1)*u**3*pi*ci
     .        + 9965.0d0/864.0d0*GPL1(2)
     .        + 5.0d0/8.0d0*GPL1(2)*u**(-1)
     .        - 5.0d0/4.0d0*GPL1(2)*u**(-1)*v
     .        + 5.0d0/8.0d0*GPL1(2)*u**(-1)*v**2
     .        + 5.0d0*GPL1(2)*pi*ci
     .        - 83.0d0/144.0d0*GPL1(2)*pi**2
     .        + 1.0d0/4.0d0*GPL1(2)*pi**3*ci
     .        - 3.0d0/4.0d0*GPL1(2)*zeta3
     .        - 5.0d0/3.0d0*GPL1(2)*v*omu**(-1)
     .        - GPL1(2)*v
     .        - 11.0d0/24.0d0*GPL1(2)*v*pi*ci
     .        + 1.0d0/12.0d0*GPL1(2)*v*pi**2
     .        - 4.0d0/3.0d0*GPL1(2)*v**2*omu**(-2)
     .        + 5.0d0/4.0d0*GPL1(2)*v**2
         col1 = col1 - 2.0d0/3.0d0*GPL1(2)*v**2*pi*ci
     .        + 1.0d0/24.0d0*GPL1(2)*v**2*pi**2
     .        - 13.0d0/8.0d0*GPL1(2)*u
     .        - 37.0d0/24.0d0*GPL1(2)*u*pi*ci
     .        + 1.0d0/12.0d0*GPL1(2)*u*pi**2
     .        + 5.0d0/2.0d0*GPL1(2)*u*v
     .        + GPL1(2)*u*v*pi*ci
     .        + 1.0d0/12.0d0*GPL1(2)*u*v*pi**2
     .        - 5.0d0/4.0d0*GPL1(2)*u*v**2*pi*ci
     .        + 5.0d0/4.0d0*GPL1(2)*u**2
     .        + 5.0d0/3.0d0*GPL1(2)*u**2*pi*ci
     .        + 1.0d0/24.0d0*GPL1(2)*u**2*pi**2
     .        - 5.0d0/2.0d0*GPL1(2)*u**2*v*pi*ci
     .        - 5.0d0/4.0d0*GPL1(2)*u**3*pi*ci
     .        - 581.0d0/144.0d0*GPL1(3)*pi*ci
         col1 = col1 + 25.0d0/18.0d0*GPL1(3)*pi**2
     .        - 7.0d0/24.0d0*GPL1(3)*pi**3*ci
     .        - 1.0d0/2.0d0*GPL1(3)*zeta3
     .        + 7.0d0/12.0d0*GPL1(3)*v*pi*ci
     .        + 1.0d0/12.0d0*GPL1(3)*v*pi**2
     .        + 2.0d0/3.0d0*GPL1(3)*v**2*pi*ci
     .        + 1.0d0/24.0d0*GPL1(3)*v**2*pi**2
     .        + 5.0d0/3.0d0*GPL1(3)*u*pi*ci
     .        + 1.0d0/12.0d0*GPL1(3)*u*pi**2
     .        - GPL1(3)*u*v*pi*ci
     .        + 1.0d0/12.0d0*GPL1(3)*u*v*pi**2
     .        + 5.0d0/4.0d0*GPL1(3)*u*v**2*pi*ci
     .        - 5.0d0/3.0d0*GPL1(3)*u**2*pi*ci
     .        + 1.0d0/24.0d0*GPL1(3)*u**2*pi**2
     .        + 5.0d0/2.0d0*GPL1(3)*u**2*v*pi*ci
         col1 = col1 + 5.0d0/4.0d0*GPL1(3)*u**3*pi*ci
     .        - 151.0d0/72.0d0*GPL2(0,0)
     .        - 11.0d0/8.0d0*GPL2(0,0)*pi*ci
     .        + 5.0d0/24.0d0*GPL2(0,0)*pi**2
     .        - 1.0d0/8.0d0*GPL2(0,0)*v
     .        - 1.0d0/4.0d0*GPL2(0,0)*v*pi*ci
     .        - 1.0d0/8.0d0*GPL2(0,0)*v**2*pi*ci
     .        - 1.0d0/8.0d0*GPL2(0,0)*u
     .        - 1.0d0/4.0d0*GPL2(0,0)*u*pi*ci
     .        - 1.0d0/4.0d0*GPL2(0,0)*u*v*pi*ci
     .        - 1.0d0/8.0d0*GPL2(0,0)*u**2*pi*ci
     .        + 1.0d0/12.0d0*GPL2(0,1)*pi*ci
     .        + 1.0d0/4.0d0*GPL2(0,1)*pi**2
     .        + 1.0d0/4.0d0*GPL2(0,1)*v*pi*ci
     .        + 1.0d0/8.0d0*GPL2(0,1)*v**2*pi*ci
         col1 = col1 + 1.0d0/2.0d0*GPL2(0,1)*u*pi*ci
     .        + 1.0d0/4.0d0*GPL2(0,1)*u*v*pi*ci
     .        + 1.0d0/4.0d0*GPL2(0,1)*u**2*pi*ci
     .        + 115.0d0/36.0d0*GPL2(0,2)
     .        - 13.0d0/12.0d0*GPL2(0,2)*pi*ci
     .        + 3.0d0/8.0d0*GPL2(0,2)*pi**2
     .        - 11.0d0/24.0d0*GPL2(0,2)*v
     .        - 1.0d0/4.0d0*GPL2(0,2)*v*pi*ci
     .        - 2.0d0/3.0d0*GPL2(0,2)*v**2
     .        - 1.0d0/8.0d0*GPL2(0,2)*v**2*pi*ci
     .        - 37.0d0/24.0d0*GPL2(0,2)*u
     .        - 1.0d0/4.0d0*GPL2(0,2)*u*pi*ci
     .        + GPL2(0,2)*u*v
     .        - 1.0d0/4.0d0*GPL2(0,2)*u*v*pi*ci
     .        - 5.0d0/4.0d0*GPL2(0,2)*u*v**2
         col1 = col1 + 5.0d0/3.0d0*GPL2(0,2)*u**2
     .        - 1.0d0/8.0d0*GPL2(0,2)*u**2*pi*ci
     .        - 5.0d0/2.0d0*GPL2(0,2)*u**2*v
     .        - 5.0d0/4.0d0*GPL2(0,2)*u**3
     .        + 41.0d0/24.0d0*GPL2(0,3)*pi*ci
     .        + 1.0d0/12.0d0*GPL2(0,3)*pi**2
     .        + 1.0d0/4.0d0*GPL2(0,3)*v*pi*ci
     .        + 1.0d0/8.0d0*GPL2(0,3)*v**2*pi*ci
     .        + 1.0d0/4.0d0*GPL2(0,3)*u*v*pi*ci
     .        + 11.0d0/24.0d0*GPL2(1,0)*pi*ci
     .        - 1.0d0/2.0d0*GPL2(1,0)*pi**2
     .        + 1.0d0/4.0d0*GPL2(1,0)*u*pi*ci
     .        + 1.0d0/8.0d0*GPL2(1,0)*u**2*pi*ci
     .        - 13.0d0/12.0d0*GPL2(1,1)*pi*ci
     .        + 1.0d0/3.0d0*GPL2(1,1)*pi**2
         col1 = col1 - 1.0d0/2.0d0*GPL2(1,1)*u*pi*ci
     .        - 1.0d0/4.0d0*GPL2(1,1)*u**2*pi*ci
     .        - 313.0d0/36.0d0*GPL2(1,2)
     .        + 5.0d0/8.0d0*GPL2(1,2)*u**(-2)
     .        - 5.0d0/4.0d0*GPL2(1,2)*u**(-2)*v
     .        + 5.0d0/8.0d0*GPL2(1,2)*u**(-2)*v**2
     .        + 7.0d0/8.0d0*GPL2(1,2)*u**(-1)
     .        - GPL2(1,2)*u**(-1)*v
     .        + 1.0d0/8.0d0*GPL2(1,2)*u**(-1)*v**2
     .        - 1.0d0/8.0d0*GPL2(1,2)*pi*ci
     .        - 5.0d0/12.0d0*GPL2(1,2)*pi**2
     .        + 5.0d0/6.0d0*GPL2(1,2)*v
     .        + 2.0d0/3.0d0*GPL2(1,2)*v**2
     .        + 5.0d0/3.0d0*GPL2(1,2)*u
     .        + 1.0d0/4.0d0*GPL2(1,2)*u*pi*ci
         col1 = col1 - GPL2(1,2)*u*v
     .        + 5.0d0/4.0d0*GPL2(1,2)*u*v**2
     .        - 5.0d0/3.0d0*GPL2(1,2)*u**2
     .        + 1.0d0/8.0d0*GPL2(1,2)*u**2*pi*ci
     .        + 5.0d0/2.0d0*GPL2(1,2)*u**2*v
     .        + 5.0d0/4.0d0*GPL2(1,2)*u**3
     .        + 115.0d0/36.0d0*GPL2(2,0)
     .        - 13.0d0/12.0d0*GPL2(2,0)*pi*ci
     .        + 5.0d0/12.0d0*GPL2(2,0)*pi**2
     .        - 11.0d0/24.0d0*GPL2(2,0)*v
     .        - 1.0d0/4.0d0*GPL2(2,0)*v*pi*ci
     .        - 2.0d0/3.0d0*GPL2(2,0)*v**2
     .        - 1.0d0/8.0d0*GPL2(2,0)*v**2*pi*ci
     .        - 37.0d0/24.0d0*GPL2(2,0)*u
     .        - 1.0d0/4.0d0*GPL2(2,0)*u*pi*ci
         col1 = col1 + GPL2(2,0)*u*v
     .        - 1.0d0/4.0d0*GPL2(2,0)*u*v*pi*ci
     .        - 5.0d0/4.0d0*GPL2(2,0)*u*v**2
     .        + 5.0d0/3.0d0*GPL2(2,0)*u**2
     .        - 1.0d0/8.0d0*GPL2(2,0)*u**2*pi*ci
     .        - 5.0d0/2.0d0*GPL2(2,0)*u**2*v
     .        - 5.0d0/4.0d0*GPL2(2,0)*u**3
     .        + 35.0d0/8.0d0*GPL2(2,1)*pi*ci
     .        + 1.0d0/6.0d0*GPL2(2,1)*pi**2
     .        + 1.0d0/4.0d0*GPL2(2,1)*v*pi*ci
     .        + 1.0d0/8.0d0*GPL2(2,1)*v**2*pi*ci
     .        + 1.0d0/4.0d0*GPL2(2,1)*u*pi*ci
     .        + 1.0d0/4.0d0*GPL2(2,1)*u*v*pi*ci
     .        + 1.0d0/8.0d0*GPL2(2,1)*u**2*pi*ci
     .        - 455.0d0/144.0d0*GPL2(2,2)
         col1 = col1 - 2.0d0*GPL2(2,2)*pi*ci
     .        - 1.0d0/12.0d0*GPL2(2,2)*pi**2
     .        + 13.0d0/12.0d0*GPL2(2,3)*pi*ci
     .        + 1.0d0/6.0d0*GPL2(2,3)*pi**2
     .        - 581.0d0/144.0d0*GPL2(3,0)
     .        - 13.0d0/24.0d0*GPL2(3,0)*pi*ci
     .        - 13.0d0/24.0d0*GPL2(3,0)*pi**2
     .        + 7.0d0/12.0d0*GPL2(3,0)*v
     .        + 2.0d0/3.0d0*GPL2(3,0)*v**2
     .        + 5.0d0/3.0d0*GPL2(3,0)*u
     .        - GPL2(3,0)*u*v
     .        + 5.0d0/4.0d0*GPL2(3,0)*u*v**2
     .        - 5.0d0/3.0d0*GPL2(3,0)*u**2
     .        + 5.0d0/2.0d0*GPL2(3,0)*u**2*v
     .        + 5.0d0/4.0d0*GPL2(3,0)*u**3
         col1 = col1 - 1.0d0/3.0d0*GPL2(3,2)*pi**2
     .        - 1.0d0/4.0d0*GPL2(3,2)*v*pi*ci
     .        - 1.0d0/8.0d0*GPL2(3,2)*v**2*pi*ci
     .        - 1.0d0/4.0d0*GPL2(3,2)*u*pi*ci
     .        - 1.0d0/4.0d0*GPL2(3,2)*u*v*pi*ci
     .        - 1.0d0/8.0d0*GPL2(3,2)*u**2*pi*ci
     .        + 1.0d0/6.0d0*GPL2(3,3)*pi*ci
     .        + 1.0d0/6.0d0*GPL2(3,3)*pi**2
     .        + 1.0d0/4.0d0*GPL2(3,3)*v*pi*ci
     .        + 1.0d0/8.0d0*GPL2(3,3)*v**2*pi*ci
     .        + 1.0d0/4.0d0*GPL2(3,3)*u*pi*ci
     .        + 1.0d0/4.0d0*GPL2(3,3)*u*v*pi*ci
     .        + 1.0d0/8.0d0*GPL2(3,3)*u**2*pi*ci
     .        - 11.0d0/8.0d0*GPL3(0,0,2)
     .        - 1.0d0/2.0d0*GPL3(0,0,2)*pi*ci
         col1 = col1 - 1.0d0/4.0d0*GPL3(0,0,2)*v
     .        - 1.0d0/8.0d0*GPL3(0,0,2)*v**2
     .        - 1.0d0/4.0d0*GPL3(0,0,2)*u
     .        - 1.0d0/4.0d0*GPL3(0,0,2)*u*v
     .        - 1.0d0/8.0d0*GPL3(0,0,2)*u**2
     .        - 3.0d0/4.0d0*GPL3(0,1,0)*pi*ci
     .        + 1.0d0/12.0d0*GPL3(0,1,2)
     .        + 1.0d0/4.0d0*GPL3(0,1,2)*pi*ci
     .        + 1.0d0/4.0d0*GPL3(0,1,2)*v
     .        + 1.0d0/8.0d0*GPL3(0,1,2)*v**2
     .        + 1.0d0/2.0d0*GPL3(0,1,2)*u
     .        + 1.0d0/4.0d0*GPL3(0,1,2)*u*v
     .        + 1.0d0/4.0d0*GPL3(0,1,2)*u**2
     .        - 11.0d0/8.0d0*GPL3(0,2,0)
     .        - 1.0d0/2.0d0*GPL3(0,2,0)*pi*ci
         col1 = col1 - 1.0d0/4.0d0*GPL3(0,2,0)*v
     .        - 1.0d0/8.0d0*GPL3(0,2,0)*v**2
     .        - 1.0d0/4.0d0*GPL3(0,2,0)*u
     .        - 1.0d0/4.0d0*GPL3(0,2,0)*u*v
     .        - 1.0d0/8.0d0*GPL3(0,2,0)*u**2
     .        + 1.0d0/4.0d0*GPL3(0,2,1)*pi*ci
     .        - 2.0d0*GPL3(0,2,2)
     .        - 1.0d0/2.0d0*GPL3(0,2,2)*pi*ci
     .        + 1.0d0/2.0d0*GPL3(0,2,3)*pi*ci
     .        + 41.0d0/24.0d0*GPL3(0,3,0)
     .        + 1.0d0/4.0d0*GPL3(0,3,0)*v
     .        + 1.0d0/8.0d0*GPL3(0,3,0)*v**2
     .        + 1.0d0/4.0d0*GPL3(0,3,0)*u*v
     .        - 1.0d0/2.0d0*GPL3(0,3,3)*pi*ci
     .        + 11.0d0/24.0d0*GPL3(1,0,2)
         col1 = col1 + GPL3(1,0,2)*pi*ci
     .        + 1.0d0/4.0d0*GPL3(1,0,2)*u
     .        + 1.0d0/8.0d0*GPL3(1,0,2)*u**2
     .        + 2.0d0*GPL3(1,1,1)*pi*ci
     .        - 13.0d0/12.0d0*GPL3(1,1,2)
     .        - 2.0d0*GPL3(1,1,2)*pi*ci
     .        - 1.0d0/2.0d0*GPL3(1,1,2)*u
     .        - 1.0d0/4.0d0*GPL3(1,1,2)*u**2
     .        + 11.0d0/24.0d0*GPL3(1,2,0)
     .        + GPL3(1,2,0)*pi*ci
     .        + 1.0d0/4.0d0*GPL3(1,2,0)*u
     .        + 1.0d0/8.0d0*GPL3(1,2,0)*u**2
     .        - 2.0d0*GPL3(1,2,1)*pi*ci
     .        + 4.0d0*GPL3(1,2,2)
     .        + GPL3(1,2,2)*pi*ci
         col1 = col1 - 11.0d0/8.0d0*GPL3(2,0,0)
     .        - 1.0d0/2.0d0*GPL3(2,0,0)*pi*ci
     .        - 1.0d0/4.0d0*GPL3(2,0,0)*v
     .        - 1.0d0/8.0d0*GPL3(2,0,0)*v**2
     .        - 1.0d0/4.0d0*GPL3(2,0,0)*u
     .        - 1.0d0/4.0d0*GPL3(2,0,0)*u*v
     .        - 1.0d0/8.0d0*GPL3(2,0,0)*u**2
     .        + 1.0d0/2.0d0*GPL3(2,0,1)*pi*ci
     .        - 2.0d0*GPL3(2,0,2)
     .        - 1.0d0/2.0d0*GPL3(2,0,2)*pi*ci
     .        + 1.0d0/2.0d0*GPL3(2,0,3)*pi*ci
     .        + 1.0d0/2.0d0*GPL3(2,1,0)*pi*ci
     .        - 2.0d0*GPL3(2,1,1)*pi*ci
     .        + 35.0d0/8.0d0*GPL3(2,1,2)
     .        + 1.0d0/2.0d0*GPL3(2,1,2)*pi*ci
         col1 = col1 + 1.0d0/4.0d0*GPL3(2,1,2)*v
     .        + 1.0d0/8.0d0*GPL3(2,1,2)*v**2
     .        + 1.0d0/4.0d0*GPL3(2,1,2)*u
     .        + 1.0d0/4.0d0*GPL3(2,1,2)*u*v
     .        + 1.0d0/8.0d0*GPL3(2,1,2)*u**2
     .        - 2.0d0*GPL3(2,2,0)
     .        - 1.0d0/2.0d0*GPL3(2,2,0)*pi*ci
     .        + GPL3(2,2,1)*pi*ci
     .        - GPL3(2,2,3)*pi*ci
     .        + 13.0d0/12.0d0*GPL3(2,3,0)
     .        + 1.0d0/4.0d0*GPL3(2,3,0)*pi*ci
     .        - 1.0d0/2.0d0*GPL3(2,3,2)*pi*ci
     .        - 1.0d0/4.0d0*GPL3(2,3,3)*pi*ci
     .        + 7.0d0/4.0d0*GPL3(3,0,0)
     .        + 1.0d0/2.0d0*GPL3(3,0,0)*pi*ci
         col1 = col1 + 1.0d0/4.0d0*GPL3(3,0,0)*v
     .        + 1.0d0/8.0d0*GPL3(3,0,0)*v**2
     .        + 1.0d0/4.0d0*GPL3(3,0,0)*u
     .        + 1.0d0/4.0d0*GPL3(3,0,0)*u*v
     .        + 1.0d0/8.0d0*GPL3(3,0,0)*u**2
     .        + 1.0d0/4.0d0*GPL3(3,0,1)*pi*ci
     .        + 1.0d0/2.0d0*GPL3(3,0,2)*pi*ci
     .        - 1.0d0/4.0d0*GPL3(3,0,2)*v
     .        - 1.0d0/8.0d0*GPL3(3,0,2)*v**2
     .        - 1.0d0/4.0d0*GPL3(3,0,2)*u
     .        - 1.0d0/4.0d0*GPL3(3,0,2)*u*v
     .        - 1.0d0/8.0d0*GPL3(3,0,2)*u**2
     .        - 1.0d0/4.0d0*GPL3(3,0,3)*pi*ci
     .        + 1.0d0/2.0d0*GPL3(3,2,0)*pi*ci
     .        - 1.0d0/4.0d0*GPL3(3,2,0)*v
         col1 = col1 - 1.0d0/8.0d0*GPL3(3,2,0)*v**2
     .        - 1.0d0/4.0d0*GPL3(3,2,0)*u
     .        - 1.0d0/4.0d0*GPL3(3,2,0)*u*v
     .        - 1.0d0/8.0d0*GPL3(3,2,0)*u**2
     .        + 1.0d0/4.0d0*GPL3(3,2,1)*pi*ci
     .        + 1.0d0/4.0d0*GPL3(3,2,3)*pi*ci
     .        + 1.0d0/6.0d0*GPL3(3,3,0)
     .        - 1.0d0/2.0d0*GPL3(3,3,0)*pi*ci
     .        + 1.0d0/4.0d0*GPL3(3,3,0)*v
     .        + 1.0d0/8.0d0*GPL3(3,3,0)*v**2
     .        + 1.0d0/4.0d0*GPL3(3,3,0)*u
     .        + 1.0d0/4.0d0*GPL3(3,3,0)*u*v
     .        + 1.0d0/8.0d0*GPL3(3,3,0)*u**2
     .        + 1.0d0/2.0d0*GPL3(3,3,3)*pi*ci
     .        - 1.0d0/2.0d0*GPL4(0,0,2,2)
         col1 = col1 - 3.0d0/4.0d0*GPL4(0,1,0,2)
     .        - 3.0d0/4.0d0*GPL4(0,1,2,0)
     .        + GPL4(0,1,2,2)
     .        - 1.0d0/2.0d0*GPL4(0,2,0,2)
     .        + 1.0d0/4.0d0*GPL4(0,2,1,2)
     .        - 1.0d0/2.0d0*GPL4(0,2,2,0)
     .        + 1.0d0/2.0d0*GPL4(0,2,3,0)
     .        - 1.0d0/2.0d0*GPL4(0,3,3,0)
     .        + GPL4(1,0,2,2)
     .        + 2.0d0*GPL4(1,1,1,2)
     .        - 2.0d0*GPL4(1,1,2,2)
     .        + GPL4(1,2,0,2)
     .        - 2.0d0*GPL4(1,2,1,2)
     .        + GPL4(1,2,2,0)
     .        - 1.0d0/2.0d0*GPL4(2,0,0,2)
         col1 = col1 + 1.0d0/2.0d0*GPL4(2,0,1,2)
     .        - 1.0d0/2.0d0*GPL4(2,0,2,0)
     .        + 1.0d0/2.0d0*GPL4(2,0,3,0)
     .        + 1.0d0/2.0d0*GPL4(2,1,0,2)
     .        - 2.0d0*GPL4(2,1,1,2)
     .        + 1.0d0/2.0d0*GPL4(2,1,2,0)
     .        - 1.0d0/2.0d0*GPL4(2,2,0,0)
     .        + GPL4(2,2,1,2)
     .        - GPL4(2,2,3,0)
     .        + 3.0d0/4.0d0*GPL4(2,3,0,0)
     .        - 1.0d0/2.0d0*GPL4(2,3,0,2)
     .        - 1.0d0/2.0d0*GPL4(2,3,2,0)
     .        - 1.0d0/4.0d0*GPL4(2,3,3,0)
     .        + 1.0d0/2.0d0*GPL4(3,0,0,2)
     .        + 1.0d0/4.0d0*GPL4(3,0,1,2)
         col1 = col1 + 1.0d0/2.0d0*GPL4(3,0,2,0)
     .        - 1.0d0/4.0d0*GPL4(3,0,3,0)
     .        + 1.0d0/2.0d0*GPL4(3,2,0,0)
     .        + 1.0d0/4.0d0*GPL4(3,2,1,2)
     .        + 1.0d0/4.0d0*GPL4(3,2,3,0)
     .        - 1.0d0/2.0d0*GPL4(3,3,0,0)
     .        + 1.0d0/2.0d0*GPL4(3,3,3,0)

         col2 = - 48443.0d0/2592.0d0
     .        + 1.0d0/16.0d0*u**(-1)*pi*ci
     .        - 1.0d0/8.0d0*u**(-1)*v*pi*ci
     .        + 1.0d0/16.0d0*u**(-1)*v**2*pi*ci
     .        - 1.0d0/16.0d0*omumv**(-1)*pi**2
     .        - 133.0d0/48.0d0*pi*ci
     .        + 29.0d0/216.0d0*pi**2
     .        + 1.0d0/4.0d0*pi**3*ci
     .        - 209.0d0/2880.0d0*pi**4
     .        + 629.0d0/144.0d0*zeta3
     .        - 1.0d0/4.0d0*zeta3*pi*ci
     .        + 1.0d0/16.0d0*v*omumv**(-1)*pi**2
     .        + 2.0d0/3.0d0*v*upv**(-1)*pi*ci
     .        + 139.0d0/36.0d0*v
     .        + 5.0d0/16.0d0*v*pi*ci
         col2 = col2 - 5.0d0/16.0d0*v*pi**2
     .        + 1.0d0/8.0d0*v**2*pi*ci
     .        + 7.0d0/24.0d0*v**2*pi**2
     .        - 1.0d0/48.0d0*u*omv**(-1)*pi**2
     .        - 1.0d0/6.0d0*u*omv**(-1)*pi**3*ci
     .        + 139.0d0/36.0d0*u
     .        + 2.0d0/3.0d0*u*pi*ci
     .        + 11.0d0/48.0d0*u*pi**2
     .        + 1.0d0/6.0d0*u*pi**3*ci
     .        - 1.0d0/2.0d0*u*zeta3*omv**(-1)
     .        + 1.0d0/8.0d0*u*v*pi*ci
     .        + 1.0d0/12.0d0*u*v*pi**2
     .        + 1.0d0/12.0d0*u*v**2*pi**2
     .        + 1.0d0/24.0d0*u**2*omv**(-2)*pi**3*ci
     .        - 1.0d0/12.0d0*u**2*omv**(-1)*pi**2
         col2 = col2 - 5.0d0/24.0d0*u**2*pi**2
     .        - 1.0d0/24.0d0*u**2*pi**3*ci
     .        + 1.0d0/8.0d0*u**2*zeta3*omv**(-2)
     .        + 1.0d0/6.0d0*u**2*v*pi**2
     .        + 1.0d0/12.0d0*u**3*pi**2
     .        - 4291.0d0/432.0d0*HPL1(0)
     .        + 1.0d0/16.0d0*HPL1(0)*u**(-1)*v
     .        - 1.0d0/16.0d0*HPL1(0)*u**(-1)*v**2
     .        - 3.0d0/8.0d0*HPL1(0)*omumv**(-1)*pi*ci
     .        + 187.0d0/72.0d0*HPL1(0)*pi*ci
     .        - 215.0d0/288.0d0*HPL1(0)*pi**2
     .        - 1.0d0/8.0d0*HPL1(0)*pi**3*ci
     .        + 1.0d0/4.0d0*HPL1(0)*zeta3
     .        + 3.0d0/8.0d0*HPL1(0)*v*omumv**(-1)*pi*ci
     .        - 2.0d0/3.0d0*HPL1(0)*v*upv**(-1)
         col2 = col2 + 3.0d0/2.0d0*HPL1(0)*v
     .        - 3.0d0/8.0d0*HPL1(0)*v*pi*ci
     .        - 1.0d0/4.0d0*HPL1(0)*v*pi**2
     .        - 1.0d0/8.0d0*HPL1(0)*v**2
     .        + 7.0d0/16.0d0*HPL1(0)*v**2*pi*ci
     .        - 1.0d0/8.0d0*HPL1(0)*u*omv**(-1)*pi*ci
     .        + 1.0d0/3.0d0*HPL1(0)*u*omv**(-1)*pi**2
     .        + 55.0d0/48.0d0*HPL1(0)*u
     .        + 1.0d0/2.0d0*HPL1(0)*u*pi*ci
     .        - 7.0d0/12.0d0*HPL1(0)*u*pi**2
     .        - 1.0d0/8.0d0*HPL1(0)*u*v
     .        + 1.0d0/8.0d0*HPL1(0)*u*v*pi*ci
     .        + 1.0d0/8.0d0*HPL1(0)*u*v**2*pi*ci
     .        - 1.0d0/12.0d0*HPL1(0)*u**2*omv**(-2)*pi**2
     .        - 1.0d0/8.0d0*HPL1(0)*u**2*omv**(-1)*pi*ci
         col2 = col2 - 5.0d0/16.0d0*HPL1(0)*u**2*pi*ci
     .        + 1.0d0/12.0d0*HPL1(0)*u**2*pi**2
     .        + 1.0d0/4.0d0*HPL1(0)*u**2*v*pi*ci
     .        + 1.0d0/8.0d0*HPL1(0)*u**3*pi*ci
     .        + 187.0d0/72.0d0*HPL1(0)*GPL1(0)
     .        - 3.0d0/8.0d0*HPL1(0)*GPL1(0)*omumv**(-1)
     .        + 1.0d0/6.0d0*HPL1(0)*GPL1(0)*pi*ci
     .        - 1.0d0/8.0d0*HPL1(0)*GPL1(0)*pi**2
     .        + 3.0d0/8.0d0*HPL1(0)*GPL1(0)*v*omumv**(-1)
     .        - 3.0d0/8.0d0*HPL1(0)*GPL1(0)*v
     .        + 3.0d0/4.0d0*HPL1(0)*GPL1(0)*v*pi*ci
     .        + 7.0d0/16.0d0*HPL1(0)*GPL1(0)*v**2
     .        - 1.0d0/2.0d0*HPL1(0)*GPL1(0)*u*omv**(-1)*pi*ci
     .        + 3.0d0/8.0d0*HPL1(0)*GPL1(0)*u
     .        + 5.0d0/4.0d0*HPL1(0)*GPL1(0)*u*pi*ci
         col2 = col2 + 1.0d0/8.0d0*HPL1(0)*GPL1(0)*u*v
     .        + 1.0d0/8.0d0*HPL1(0)*GPL1(0)*u*v**2
     .        + 1.0d0/8.0d0*HPL1(0)*GPL1(0)*u**2*omv**(-2)*pi*ci
     .        - 1.0d0/8.0d0*HPL1(0)*GPL1(0)*u**2*omv**(-1)
     .        - 5.0d0/16.0d0*HPL1(0)*GPL1(0)*u**2
     .        - 1.0d0/8.0d0*HPL1(0)*GPL1(0)*u**2*pi*ci
     .        + 1.0d0/4.0d0*HPL1(0)*GPL1(0)*u**2*v
     .        + 1.0d0/8.0d0*HPL1(0)*GPL1(0)*u**3
     .        + 3.0d0/2.0d0*HPL1(0)*GPL1(1)
     .        - 1.0d0/16.0d0*HPL1(0)*GPL1(1)*u**(-2)
     .        + 1.0d0/8.0d0*HPL1(0)*GPL1(1)*u**(-2)*v
     .        - 1.0d0/16.0d0*HPL1(0)*GPL1(1)*u**(-2)*v**2
     .        + 3.0d0/8.0d0*HPL1(0)*GPL1(1)*u**(-1)
     .        - 3.0d0/8.0d0*HPL1(0)*GPL1(1)*u**(-1)*v
     .        + 5.0d0/8.0d0*HPL1(0)*GPL1(1)*v
         col2 = col2 - 7.0d0/16.0d0*HPL1(0)*GPL1(1)*v**2
     .        - 1.0d0/8.0d0*HPL1(0)*GPL1(1)*u*omv**(-1)
     .        - 1.0d0/8.0d0*HPL1(0)*GPL1(1)*u*v
     .        - 1.0d0/8.0d0*HPL1(0)*GPL1(1)*u*v**2
     .        + 1.0d0/8.0d0*HPL1(0)*GPL1(1)*u**2*omv**(-1)
     .        + 5.0d0/16.0d0*HPL1(0)*GPL1(1)*u**2
     .        - 1.0d0/4.0d0*HPL1(0)*GPL1(1)*u**2*v
     .        - 1.0d0/8.0d0*HPL1(0)*GPL1(1)*u**3
     .        - 3.0d0/2.0d0*HPL1(0)*GPL1(2)
     .        + 1.0d0/16.0d0*HPL1(0)*GPL1(2)*u**(-2)
     .        - 1.0d0/8.0d0*HPL1(0)*GPL1(2)*u**(-2)*v
     .        + 1.0d0/16.0d0*HPL1(0)*GPL1(2)*u**(-2)*v**2
     .        - 3.0d0/8.0d0*HPL1(0)*GPL1(2)*u**(-1)
     .        + 3.0d0/8.0d0*HPL1(0)*GPL1(2)*u**(-1)*v
     .        - 3.0d0/4.0d0*HPL1(0)*GPL1(2)*pi*ci
         col2 = col2 - 5.0d0/8.0d0*HPL1(0)*GPL1(2)*v
     .        + 3.0d0/4.0d0*HPL1(0)*GPL1(2)*v*pi*ci
     .        + 7.0d0/16.0d0*HPL1(0)*GPL1(2)*v**2
     .        + 1.0d0/2.0d0*HPL1(0)*GPL1(2)*u*omv**(-1)*pi*ci
     .        + 1.0d0/8.0d0*HPL1(0)*GPL1(2)*u
     .        + 3.0d0/4.0d0*HPL1(0)*GPL1(2)*u*pi*ci
     .        + 1.0d0/8.0d0*HPL1(0)*GPL1(2)*u*v
     .        + 1.0d0/8.0d0*HPL1(0)*GPL1(2)*u*v**2
     .        - 1.0d0/8.0d0*HPL1(0)*GPL1(2)*u**2*omv**(-2)*pi*ci
     .        - 1.0d0/8.0d0*HPL1(0)*GPL1(2)*u**2*omv**(-1)
     .        - 5.0d0/16.0d0*HPL1(0)*GPL1(2)*u**2
     .        + 1.0d0/4.0d0*HPL1(0)*GPL1(2)*u**2*v
     .        + 1.0d0/8.0d0*HPL1(0)*GPL1(2)*u**3
     .        - 187.0d0/72.0d0*HPL1(0)*GPL1(3)
     .        + 3.0d0/8.0d0*HPL1(0)*GPL1(3)*omumv**(-1)
         col2 = col2 + 5.0d0/8.0d0*HPL1(0)*GPL1(3)*pi*ci
     .        - 1.0d0/8.0d0*HPL1(0)*GPL1(3)*pi**2
     .        - 3.0d0/8.0d0*HPL1(0)*GPL1(3)*v*omumv**(-1)
     .        + 3.0d0/8.0d0*HPL1(0)*GPL1(3)*v
     .        - 3.0d0/4.0d0*HPL1(0)*GPL1(3)*v*pi*ci
     .        - 7.0d0/16.0d0*HPL1(0)*GPL1(3)*v**2
     .        + 1.0d0/8.0d0*HPL1(0)*GPL1(3)*u*omv**(-1)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL1(3)*u
     .        - 3.0d0/4.0d0*HPL1(0)*GPL1(3)*u*pi*ci
     .        - 1.0d0/8.0d0*HPL1(0)*GPL1(3)*u*v
     .        - 1.0d0/8.0d0*HPL1(0)*GPL1(3)*u*v**2
     .        + 1.0d0/8.0d0*HPL1(0)*GPL1(3)*u**2*omv**(-1)
     .        + 5.0d0/16.0d0*HPL1(0)*GPL1(3)*u**2
     .        - 1.0d0/4.0d0*HPL1(0)*GPL1(3)*u**2*v
     .        - 1.0d0/8.0d0*HPL1(0)*GPL1(3)*u**3
         col2 = col2 - 7.0d0/4.0d0*HPL1(0)*GPL2(0,0)
     .        + 3.0d0/4.0d0*HPL1(0)*GPL2(0,0)*v
     .        + 3.0d0/4.0d0*HPL1(0)*GPL2(0,0)*u
     .        + 3.0d0/4.0d0*HPL1(0)*GPL2(0,1)
     .        - 1.0d0/4.0d0*HPL1(0)*GPL2(0,1)*pi*ci
     .        - 3.0d0/4.0d0*HPL1(0)*GPL2(0,1)*v
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(0,1)*u*omv**(-1)
     .        - 1.0d0/4.0d0*HPL1(0)*GPL2(0,1)*u
     .        + 1.0d0/8.0d0*HPL1(0)*GPL2(0,1)*u**2*omv**(-2)
     .        - 1.0d0/8.0d0*HPL1(0)*GPL2(0,1)*u**2
     .        - 9.0d0/8.0d0*HPL1(0)*GPL2(0,2)
     .        + 3.0d0/4.0d0*HPL1(0)*GPL2(0,2)*v
     .        + 3.0d0/4.0d0*HPL1(0)*GPL2(0,2)*u
     .        + 25.0d0/12.0d0*HPL1(0)*GPL2(0,3)
     .        + 3.0d0/4.0d0*HPL1(0)*GPL2(0,3)*pi*ci
         col2 = col2 - 3.0d0/4.0d0*HPL1(0)*GPL2(0,3)*v
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(0,3)*u*omv**(-1)
     .        - 5.0d0/4.0d0*HPL1(0)*GPL2(0,3)*u
     .        - 1.0d0/8.0d0*HPL1(0)*GPL2(0,3)*u**2*omv**(-2)
     .        + 1.0d0/8.0d0*HPL1(0)*GPL2(0,3)*u**2
     .        - 3.0d0/8.0d0*HPL1(0)*GPL2(1,0)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(1,0)*u
     .        - 1.0d0/8.0d0*HPL1(0)*GPL2(1,0)*u**2
     .        + 3.0d0/4.0d0*HPL1(0)*GPL2(1,1)
     .        - HPL1(0)*GPL2(1,1)*u
     .        + 1.0d0/4.0d0*HPL1(0)*GPL2(1,1)*u**2
     .        - 3.0d0/8.0d0*HPL1(0)*GPL2(1,2)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(1,2)*u
     .        - 1.0d0/8.0d0*HPL1(0)*GPL2(1,2)*u**2
     .        - 3.0d0/8.0d0*HPL1(0)*GPL2(2,0)
         col2 = col2 + 3.0d0/4.0d0*HPL1(0)*GPL2(2,0)*v
     .        + 3.0d0/4.0d0*HPL1(0)*GPL2(2,0)*u
     .        - 3.0d0/8.0d0*HPL1(0)*GPL2(2,1)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(2,1)*u*omv**(-1)
     .        - 1.0d0/8.0d0*HPL1(0)*GPL2(2,1)*u**2*omv**(-2)
     .        + 3.0d0/4.0d0*HPL1(0)*GPL2(2,3)
     .        - 3.0d0/4.0d0*HPL1(0)*GPL2(2,3)*v
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(2,3)*u*omv**(-1)
     .        - 3.0d0/4.0d0*HPL1(0)*GPL2(2,3)*u
     .        + 1.0d0/8.0d0*HPL1(0)*GPL2(2,3)*u**2*omv**(-2)
     .        + 61.0d0/24.0d0*HPL1(0)*GPL2(3,0)
     .        + 3.0d0/4.0d0*HPL1(0)*GPL2(3,0)*pi*ci
     .        - 3.0d0/4.0d0*HPL1(0)*GPL2(3,0)*v
     .        - 3.0d0/4.0d0*HPL1(0)*GPL2(3,0)*u
     .        + 3.0d0/8.0d0*HPL1(0)*GPL2(3,2)
         col2 = col2 - 7.0d0/12.0d0*HPL1(0)*GPL2(3,3)
     .        - HPL1(0)*GPL2(3,3)*pi*ci
     .        + 3.0d0/4.0d0*HPL1(0)*GPL2(3,3)*v
     .        + 3.0d0/4.0d0*HPL1(0)*GPL2(3,3)*u
     .        - 1.0d0/2.0d0*HPL1(0)*GPL3(0,0,1)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(0,0,2)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(0,0,3)
     .        - 1.0d0/4.0d0*HPL1(0)*GPL3(0,1,0)
     .        + 1.0d0/4.0d0*HPL1(0)*GPL3(0,2,0)
     .        - 1.0d0/4.0d0*HPL1(0)*GPL3(0,2,1)
     .        + 1.0d0/4.0d0*HPL1(0)*GPL3(0,2,3)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(0,3,0)
     .        + 1.0d0/4.0d0*HPL1(0)*GPL3(0,3,2)
     .        - HPL1(0)*GPL3(0,3,3)
     .        + HPL1(0)*GPL3(3,0,0)
         col2 = col2 + 1.0d0/4.0d0*HPL1(0)*GPL3(3,0,1)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL3(3,0,2)
     .        - HPL1(0)*GPL3(3,0,3)
     .        - 1.0d0/4.0d0*HPL1(0)*GPL3(3,2,0)
     .        + 1.0d0/4.0d0*HPL1(0)*GPL3(3,2,1)
     .        - HPL1(0)*GPL3(3,3,0)
     .        + HPL1(0)*GPL3(3,3,3)
     .        - 2491.0d0/432.0d0*HPL1(1)
     .        - 1.0d0/16.0d0*HPL1(1)*u**(-1)
     .        + 1.0d0/8.0d0*HPL1(1)*u**(-1)*v
     .        - 1.0d0/16.0d0*HPL1(1)*u**(-1)*v**2
     .        - 15.0d0/16.0d0*HPL1(1)*pi*ci
     .        + 119.0d0/288.0d0*HPL1(1)*pi**2
     .        + 1.0d0/12.0d0*HPL1(1)*pi**3*ci
     .        + 3.0d0/2.0d0*HPL1(1)*zeta3
         col2 = col2 + 7.0d0/16.0d0*HPL1(1)*v
     .        - 1.0d0/2.0d0*HPL1(1)*v*pi*ci
     .        - 1.0d0/4.0d0*HPL1(1)*v*pi**2
     .        - 1.0d0/8.0d0*HPL1(1)*v**2
     .        + 7.0d0/16.0d0*HPL1(1)*v**2*pi*ci
     .        - 1.0d0/12.0d0*HPL1(1)*u*omv**(-1)*pi**2
     .        + 1.0d0/2.0d0*HPL1(1)*u
     .        + 1.0d0/4.0d0*HPL1(1)*u*pi*ci
     .        - 1.0d0/4.0d0*HPL1(1)*u*pi**2
     .        - 1.0d0/4.0d0*HPL1(1)*u*v
     .        + 1.0d0/8.0d0*HPL1(1)*u*v*pi*ci
     .        + 1.0d0/8.0d0*HPL1(1)*u*v**2*pi*ci
     .        + 1.0d0/48.0d0*HPL1(1)*u**2*omv**(-2)*pi**2
     .        - 1.0d0/8.0d0*HPL1(1)*u**2*omv**(-1)*pi*ci
     .        - 1.0d0/8.0d0*HPL1(1)*u**2
         col2 = col2 - 5.0d0/16.0d0*HPL1(1)*u**2*pi*ci
     .        + 1.0d0/4.0d0*HPL1(1)*u**2*v*pi*ci
     .        + 1.0d0/8.0d0*HPL1(1)*u**3*pi*ci
     .        - 15.0d0/16.0d0*HPL1(1)*GPL1(0)
     .        - 3.0d0/4.0d0*HPL1(1)*GPL1(0)*pi*ci
     .        + 1.0d0/24.0d0*HPL1(1)*GPL1(0)*pi**2
     .        - 1.0d0/2.0d0*HPL1(1)*GPL1(0)*v
     .        + 3.0d0/4.0d0*HPL1(1)*GPL1(0)*v*pi*ci
     .        + 7.0d0/16.0d0*HPL1(1)*GPL1(0)*v**2
     .        + 1.0d0/4.0d0*HPL1(1)*GPL1(0)*u
     .        + 3.0d0/4.0d0*HPL1(1)*GPL1(0)*u*pi*ci
     .        + 1.0d0/8.0d0*HPL1(1)*GPL1(0)*u*v
     .        + 1.0d0/8.0d0*HPL1(1)*GPL1(0)*u*v**2
     .        - 1.0d0/8.0d0*HPL1(1)*GPL1(0)*u**2*omv**(-1)
     .        - 5.0d0/16.0d0*HPL1(1)*GPL1(0)*u**2
         col2 = col2 + 1.0d0/4.0d0*HPL1(1)*GPL1(0)*u**2*v
     .        + 1.0d0/8.0d0*HPL1(1)*GPL1(0)*u**3
     .        + 3.0d0/2.0d0*HPL1(1)*GPL1(1)
     .        - 1.0d0/16.0d0*HPL1(1)*GPL1(1)*u**(-2)
     .        + 1.0d0/8.0d0*HPL1(1)*GPL1(1)*u**(-2)*v
     .        - 1.0d0/16.0d0*HPL1(1)*GPL1(1)*u**(-2)*v**2
     .        + 3.0d0/8.0d0*HPL1(1)*GPL1(1)*u**(-1)
     .        - 3.0d0/8.0d0*HPL1(1)*GPL1(1)*u**(-1)*v
     .        - 3.0d0/8.0d0*HPL1(1)*GPL1(1)*pi*ci
     .        + 5.0d0/8.0d0*HPL1(1)*GPL1(1)*v
     .        - 7.0d0/16.0d0*HPL1(1)*GPL1(1)*v**2
     .        - 1.0d0/8.0d0*HPL1(1)*GPL1(1)*u*omv**(-1)
     .        + 1.0d0/2.0d0*HPL1(1)*GPL1(1)*u*pi*ci
     .        - 1.0d0/8.0d0*HPL1(1)*GPL1(1)*u*v
     .        - 1.0d0/8.0d0*HPL1(1)*GPL1(1)*u*v**2
         col2 = col2 + 1.0d0/8.0d0*HPL1(1)*GPL1(1)*u**2*omv**(-1)
     .        + 5.0d0/16.0d0*HPL1(1)*GPL1(1)*u**2
     .        - 1.0d0/8.0d0*HPL1(1)*GPL1(1)*u**2*pi*ci
     .        - 1.0d0/4.0d0*HPL1(1)*GPL1(1)*u**2*v
     .        - 1.0d0/8.0d0*HPL1(1)*GPL1(1)*u**3
     .        + 3.0d0/8.0d0*HPL1(1)*GPL1(3)*pi*ci
     .        - 1.0d0/8.0d0*HPL1(1)*GPL1(3)*pi**2
     .        - 3.0d0/4.0d0*HPL1(1)*GPL2(0,0)
     .        + 3.0d0/4.0d0*HPL1(1)*GPL2(0,0)*v
     .        + 3.0d0/4.0d0*HPL1(1)*GPL2(0,0)*u
     .        + 3.0d0/4.0d0*HPL1(1)*GPL2(0,1)
     .        - 1.0d0/4.0d0*HPL1(1)*GPL2(0,1)*pi*ci
     .        - 3.0d0/4.0d0*HPL1(1)*GPL2(0,1)*v
     .        - 1.0d0/2.0d0*HPL1(1)*GPL2(0,1)*u*omv**(-1)
     .        - 1.0d0/4.0d0*HPL1(1)*GPL2(0,1)*u
         col2 = col2 + 1.0d0/8.0d0*HPL1(1)*GPL2(0,1)*u**2*omv**(-2)
     .        - 1.0d0/8.0d0*HPL1(1)*GPL2(0,1)*u**2
     .        + 1.0d0/4.0d0*HPL1(1)*GPL2(0,3)*pi*ci
     .        - 3.0d0/8.0d0*HPL1(1)*GPL2(1,0)
     .        + 1.0d0/2.0d0*HPL1(1)*GPL2(1,0)*u
     .        - 1.0d0/8.0d0*HPL1(1)*GPL2(1,0)*u**2
     .        + 3.0d0/4.0d0*HPL1(1)*GPL2(1,1)
     .        - HPL1(1)*GPL2(1,1)*u
     .        + 1.0d0/4.0d0*HPL1(1)*GPL2(1,1)*u**2
     .        - 3.0d0/8.0d0*HPL1(1)*GPL2(2,1)
     .        + 1.0d0/2.0d0*HPL1(1)*GPL2(2,1)*u*omv**(-1)
     .        - 1.0d0/8.0d0*HPL1(1)*GPL2(2,1)*u**2*omv**(-2)
     .        + 3.0d0/8.0d0*HPL1(1)*GPL2(3,0)
     .        - 1.0d0/2.0d0*HPL1(1)*GPL3(0,0,1)
     .        - 1.0d0/4.0d0*HPL1(1)*GPL3(0,1,0)
         col2 = col2 - 1.0d0/4.0d0*HPL1(1)*GPL3(0,2,1)
     .        + 1.0d0/4.0d0*HPL1(1)*GPL3(0,3,0)
     .        + 1.0d0/4.0d0*HPL1(1)*GPL3(3,0,1)
     .        + 1.0d0/4.0d0*HPL1(1)*GPL3(3,2,1)
     .        - 187.0d0/72.0d0*HPL2(0,0)
     .        + 3.0d0/8.0d0*HPL2(0,0)*omumv**(-1)
     .        + 35.0d0/12.0d0*HPL2(0,0)*pi*ci
     .        - 1.0d0/8.0d0*HPL2(0,0)*pi**2
     .        - 3.0d0/8.0d0*HPL2(0,0)*v*omumv**(-1)
     .        + 3.0d0/8.0d0*HPL2(0,0)*v
     .        - 3.0d0/4.0d0*HPL2(0,0)*v*pi*ci
     .        - 7.0d0/16.0d0*HPL2(0,0)*v**2
     .        + 1.0d0/8.0d0*HPL2(0,0)*u*omv**(-1)
     .        + 1.0d0/2.0d0*HPL2(0,0)*u*omv**(-1)*pi*ci
     .        - 1.0d0/2.0d0*HPL2(0,0)*u
         col2 = col2 - 5.0d0/4.0d0*HPL2(0,0)*u*pi*ci
     .        - 1.0d0/8.0d0*HPL2(0,0)*u*v
     .        - 1.0d0/8.0d0*HPL2(0,0)*u*v**2
     .        - 1.0d0/8.0d0*HPL2(0,0)*u**2*omv**(-2)*pi*ci
     .        + 1.0d0/8.0d0*HPL2(0,0)*u**2*omv**(-1)
     .        + 5.0d0/16.0d0*HPL2(0,0)*u**2
     .        + 1.0d0/8.0d0*HPL2(0,0)*u**2*pi*ci
     .        - 1.0d0/4.0d0*HPL2(0,0)*u**2*v
     .        - 1.0d0/8.0d0*HPL2(0,0)*u**3
     .        + 29.0d0/6.0d0*HPL2(0,0)*GPL1(0)
     .        + 3.0d0/4.0d0*HPL2(0,0)*GPL1(0)*pi*ci
     .        - 3.0d0/4.0d0*HPL2(0,0)*GPL1(0)*v
     .        + 1.0d0/2.0d0*HPL2(0,0)*GPL1(0)*u*omv**(-1)
     .        - 5.0d0/4.0d0*HPL2(0,0)*GPL1(0)*u
     .        - 1.0d0/8.0d0*HPL2(0,0)*GPL1(0)*u**2*omv**(-2)
         col2 = col2 + 1.0d0/8.0d0*HPL2(0,0)*GPL1(0)*u**2
     .        + 3.0d0/4.0d0*HPL2(0,0)*GPL1(2)
     .        - 3.0d0/4.0d0*HPL2(0,0)*GPL1(2)*v
     .        - 1.0d0/2.0d0*HPL2(0,0)*GPL1(2)*u*omv**(-1)
     .        - 3.0d0/4.0d0*HPL2(0,0)*GPL1(2)*u
     .        + 1.0d0/8.0d0*HPL2(0,0)*GPL1(2)*u**2*omv**(-2)
     .        - 10.0d0/3.0d0*HPL2(0,0)*GPL1(3)
     .        - HPL2(0,0)*GPL1(3)*pi*ci
     .        + 3.0d0/4.0d0*HPL2(0,0)*GPL1(3)*v
     .        + 3.0d0/4.0d0*HPL2(0,0)*GPL1(3)*u
     .        + HPL2(0,0)*GPL2(0,0)
     .        - 1.0d0/4.0d0*HPL2(0,0)*GPL2(0,2)
     .        - HPL2(0,0)*GPL2(0,3)
     .        - 3.0d0/2.0d0*HPL2(0,0)*GPL2(3,0)
     .        + 1.0d0/2.0d0*HPL2(0,0)*GPL2(3,2)
         col2 = col2 + HPL2(0,0)*GPL2(3,3)
     .        + 3.0d0/8.0d0*HPL2(0,1)*pi*ci
     .        - 1.0d0/8.0d0*HPL2(0,1)*pi**2
     .        + 1.0d0/8.0d0*HPL2(0,1)*u*omv**(-1)
     .        + 1.0d0/2.0d0*HPL2(0,1)*u*omv**(-1)*pi*ci
     .        - 1.0d0/8.0d0*HPL2(0,1)*u
     .        - 1.0d0/2.0d0*HPL2(0,1)*u*pi*ci
     .        - 1.0d0/8.0d0*HPL2(0,1)*u**2*omv**(-2)*pi*ci
     .        + 1.0d0/8.0d0*HPL2(0,1)*u**2*pi*ci
     .        + 3.0d0/8.0d0*HPL2(0,1)*GPL1(0)
     .        + 1.0d0/2.0d0*HPL2(0,1)*GPL1(0)*u*omv**(-1)
     .        - 1.0d0/2.0d0*HPL2(0,1)*GPL1(0)*u
     .        - 1.0d0/8.0d0*HPL2(0,1)*GPL1(0)*u**2*omv**(-2)
     .        + 1.0d0/8.0d0*HPL2(0,1)*GPL1(0)*u**2
     .        - 3.0d0/8.0d0*HPL2(0,1)*GPL1(1)
         col2 = col2 + 1.0d0/2.0d0*HPL2(0,1)*GPL1(1)*u
     .        - 1.0d0/8.0d0*HPL2(0,1)*GPL1(1)*u**2
     .        + 3.0d0/8.0d0*HPL2(0,1)*GPL1(2)
     .        - 1.0d0/2.0d0*HPL2(0,1)*GPL1(2)*u*omv**(-1)
     .        + 1.0d0/8.0d0*HPL2(0,1)*GPL1(2)*u**2*omv**(-2)
     .        - 3.0d0/8.0d0*HPL2(0,1)*GPL1(3)
     .        + 1.0d0/4.0d0*HPL2(0,1)*GPL1(3)*pi*ci
     .        + 1.0d0/4.0d0*HPL2(0,1)*GPL2(0,2)
     .        - 1.0d0/4.0d0*HPL2(0,1)*GPL2(0,3)
     .        + 1.0d0/4.0d0*HPL2(0,1)*GPL2(3,0)
     .        - 1.0d0/4.0d0*HPL2(0,1)*GPL2(3,2)
     .        + 161.0d0/36.0d0*HPL2(1,0)
     .        - 3.0d0/8.0d0*HPL2(1,0)*omumv**(-1)
     .        + 3.0d0*HPL2(1,0)*pi*ci
     .        - 3.0d0/8.0d0*HPL2(1,0)*pi**2
         col2 = col2 + 3.0d0/8.0d0*HPL2(1,0)*v*omumv**(-1)
     .        + 5.0d0/8.0d0*HPL2(1,0)*v
     .        - 3.0d0/4.0d0*HPL2(1,0)*v*pi*ci
     .        - 7.0d0/16.0d0*HPL2(1,0)*v**2
     .        - 1.0d0/2.0d0*HPL2(1,0)*u*omv**(-1)*pi*ci
     .        - 1.0d0/8.0d0*HPL2(1,0)*u
     .        - 3.0d0/4.0d0*HPL2(1,0)*u*pi*ci
     .        - 1.0d0/8.0d0*HPL2(1,0)*u*v
     .        - 1.0d0/8.0d0*HPL2(1,0)*u*v**2
     .        + 1.0d0/8.0d0*HPL2(1,0)*u**2*omv**(-2)*pi*ci
     .        + 1.0d0/8.0d0*HPL2(1,0)*u**2*omv**(-1)
     .        + 5.0d0/16.0d0*HPL2(1,0)*u**2
     .        - 1.0d0/4.0d0*HPL2(1,0)*u**2*v
     .        - 1.0d0/8.0d0*HPL2(1,0)*u**3
     .        + 17.0d0/24.0d0*HPL2(1,0)*GPL1(0)
         col2 = col2 + 1.0d0/2.0d0*HPL2(1,0)*GPL1(0)*pi*ci
     .        - 3.0d0/4.0d0*HPL2(1,0)*GPL1(0)*v
     .        - 3.0d0/4.0d0*HPL2(1,0)*GPL1(0)*u
     .        + 3.0d0/8.0d0*HPL2(1,0)*GPL1(1)
     .        - 1.0d0/2.0d0*HPL2(1,0)*GPL1(1)*u
     .        + 1.0d0/8.0d0*HPL2(1,0)*GPL1(1)*u**2
     .        + 3.0d0/8.0d0*HPL2(1,0)*GPL1(2)
     .        - 3.0d0/8.0d0*HPL2(1,0)*GPL1(3)
     .        - 1.0d0/4.0d0*HPL2(1,0)*GPL2(0,3)
     .        + 1.0d0/4.0d0*HPL2(1,0)*GPL2(3,0)
     .        + 1.0d0/4.0d0*HPL2(1,0)*GPL2(3,2)
     .        - 1.0d0/8.0d0*HPL2(1,1)*pi**2
     .        - 45.0d0/8.0d0*HPL3(0,0,0)
     .        - HPL3(0,0,0)*pi*ci
     .        + 3.0d0/4.0d0*HPL3(0,0,0)*v
         col2 = col2 - 1.0d0/2.0d0*HPL3(0,0,0)*u*omv**(-1)
     .        + 5.0d0/4.0d0*HPL3(0,0,0)*u
     .        + 1.0d0/8.0d0*HPL3(0,0,0)*u**2*omv**(-2)
     .        - 1.0d0/8.0d0*HPL3(0,0,0)*u**2
     .        - 3.0d0/2.0d0*HPL3(0,0,0)*GPL1(0)
     .        + 3.0d0/2.0d0*HPL3(0,0,0)*GPL1(3)
     .        - 3.0d0/8.0d0*HPL3(0,0,1)
     .        + 1.0d0/4.0d0*HPL3(0,0,1)*pi*ci
     .        + 1.0d0/4.0d0*HPL3(0,0,1)*GPL1(0)
     .        - 1.0d0/2.0d0*HPL3(0,0,1)*GPL1(3)
     .        + 23.0d0/12.0d0*HPL3(0,1,0)
     .        - 1.0d0/2.0d0*HPL3(0,1,0)*u*omv**(-1)
     .        + 1.0d0/2.0d0*HPL3(0,1,0)*u
     .        + 1.0d0/8.0d0*HPL3(0,1,0)*u**2*omv**(-2)
     .        - 1.0d0/8.0d0*HPL3(0,1,0)*u**2
         col2 = col2 + 1.0d0/4.0d0*HPL3(0,1,0)*GPL1(0)
     .        - 1.0d0/2.0d0*HPL3(0,1,0)*GPL1(3)
     .        + 2.0d0*HPL3(1,0,0)
     .        - 1.0d0/4.0d0*HPL3(1,0,0)*pi*ci
     .        + 3.0d0/4.0d0*HPL3(1,0,0)*v
     .        + 1.0d0/2.0d0*HPL3(1,0,0)*u*omv**(-1)
     .        + 3.0d0/4.0d0*HPL3(1,0,0)*u
     .        - 1.0d0/8.0d0*HPL3(1,0,0)*u**2*omv**(-2)
     .        + 1.0d0/4.0d0*HPL3(1,0,0)*GPL1(0)
     .        - 1.0d0/2.0d0*HPL3(1,0,0)*GPL1(3)
     .        - 1.0d0/2.0d0*HPL3(1,0,1)*pi*ci
     .        + 1.0d0/2.0d0*HPL3(1,0,1)*u*omv**(-1)
     .        - 1.0d0/8.0d0*HPL3(1,0,1)*u**2*omv**(-2)
     .        - 1.0d0/2.0d0*HPL3(1,0,1)*GPL1(0)
     .        + 1.0d0/4.0d0*HPL3(1,0,1)*GPL1(3)
         col2 = col2 + 11.0d0/12.0d0*HPL3(1,1,0)
     .        - 1.0d0/4.0d0*HPL3(1,1,0)*GPL1(0)
     .        - 1.0d0/4.0d0*HPL3(1,1,0)*GPL1(3)
     .        + 3.0d0/2.0d0*HPL4(0,0,0,0)
     .        - 1.0d0/2.0d0*HPL4(0,0,0,1)
     .        - 1.0d0/2.0d0*HPL4(0,0,1,0)
     .        - 1.0d0/2.0d0*HPL4(0,1,0,0)
     .        + 1.0d0/4.0d0*HPL4(0,1,0,1)
     .        - 1.0d0/4.0d0*HPL4(0,1,1,0)
     .        - 1.0d0/4.0d0*HPL4(1,0,0,0)
     .        + 1.0d0/2.0d0*HPL4(1,0,0,1)
     .        + 3.0d0/4.0d0*HPL4(1,0,1,0)
     .        + 1.0d0/4.0d0*HPL4(1,1,0,0)
     .        - 1.0d0/4.0d0*HPL4(1,1,0,1)
     .        - HPL4(1,1,1,0)
         col2 = col2 + 67.0d0/48.0d0*GPL1(0)
     .        + 15.0d0/16.0d0*GPL1(0)*pi*ci
     .        - 91.0d0/144.0d0*GPL1(0)*pi**2
     .        + 1.0d0/12.0d0*GPL1(0)*pi**3*ci
     .        + 2.0d0/3.0d0*GPL1(0)*v*upv**(-1)
     .        - 19.0d0/48.0d0*GPL1(0)*v
     .        + 1.0d0/2.0d0*GPL1(0)*v*pi*ci
     .        + 1.0d0/2.0d0*GPL1(0)*v*pi**2
     .        - 7.0d0/16.0d0*GPL1(0)*v**2*pi*ci
     .        - 1.0d0/12.0d0*GPL1(0)*u*omv**(-1)*pi**2
     .        + 1.0d0/48.0d0*GPL1(0)*u
     .        - 1.0d0/4.0d0*GPL1(0)*u*pi*ci
     .        + 2.0d0/3.0d0*GPL1(0)*u*pi**2
     .        - 1.0d0/8.0d0*GPL1(0)*u*v
     .        - 1.0d0/8.0d0*GPL1(0)*u*v*pi*ci
         col2 = col2 - 1.0d0/8.0d0*GPL1(0)*u*v**2*pi*ci
     .        + 1.0d0/48.0d0*GPL1(0)*u**2*omv**(-2)*pi**2
     .        + 1.0d0/8.0d0*GPL1(0)*u**2*omv**(-1)*pi*ci
     .        - 1.0d0/8.0d0*GPL1(0)*u**2
     .        + 5.0d0/16.0d0*GPL1(0)*u**2*pi*ci
     .        - 1.0d0/24.0d0*GPL1(0)*u**2*pi**2
     .        - 1.0d0/4.0d0*GPL1(0)*u**2*v*pi*ci
     .        - 1.0d0/8.0d0*GPL1(0)*u**3*pi*ci
     .        + 1.0d0/16.0d0*GPL1(1)*u**(-2)*pi*ci
     .        - 1.0d0/8.0d0*GPL1(1)*u**(-2)*v*pi*ci
     .        + 1.0d0/16.0d0*GPL1(1)*u**(-2)*v**2*pi*ci
     .        - 3.0d0/8.0d0*GPL1(1)*u**(-1)*pi*ci
     .        + 3.0d0/8.0d0*GPL1(1)*u**(-1)*v*pi*ci
     .        - 3.0d0/2.0d0*GPL1(1)*pi*ci
     .        - 1.0d0/8.0d0*GPL1(1)*pi**2
         col2 = col2 - 5.0d0/8.0d0*GPL1(1)*v*pi*ci
     .        + 7.0d0/16.0d0*GPL1(1)*v**2*pi*ci
     .        + 1.0d0/8.0d0*GPL1(1)*u*omv**(-1)*pi*ci
     .        + 1.0d0/6.0d0*GPL1(1)*u*pi**2
     .        + 1.0d0/8.0d0*GPL1(1)*u*v*pi*ci
     .        + 1.0d0/8.0d0*GPL1(1)*u*v**2*pi*ci
     .        - 1.0d0/8.0d0*GPL1(1)*u**2*omv**(-1)*pi*ci
     .        - 5.0d0/16.0d0*GPL1(1)*u**2*pi*ci
     .        - 1.0d0/24.0d0*GPL1(1)*u**2*pi**2
     .        + 1.0d0/4.0d0*GPL1(1)*u**2*v*pi*ci
     .        + 1.0d0/8.0d0*GPL1(1)*u**3*pi*ci
     .        + 2491.0d0/432.0d0*GPL1(2)
     .        + 1.0d0/16.0d0*GPL1(2)*u**(-1)
     .        - 1.0d0/8.0d0*GPL1(2)*u**(-1)*v
     .        + 1.0d0/16.0d0*GPL1(2)*u**(-1)*v**2
         col2 = col2 + 15.0d0/16.0d0*GPL1(2)*pi*ci
     .        - 25.0d0/96.0d0*GPL1(2)*pi**2
     .        - 1.0d0/4.0d0*GPL1(2)*zeta3
     .        - 7.0d0/16.0d0*GPL1(2)*v
     .        + 1.0d0/2.0d0*GPL1(2)*v*pi*ci
     .        + 1.0d0/4.0d0*GPL1(2)*v*pi**2
     .        + 1.0d0/8.0d0*GPL1(2)*v**2
     .        - 7.0d0/16.0d0*GPL1(2)*v**2*pi*ci
     .        + 1.0d0/12.0d0*GPL1(2)*u*omv**(-1)*pi**2
     .        - 1.0d0/2.0d0*GPL1(2)*u
     .        - 1.0d0/4.0d0*GPL1(2)*u*pi*ci
     .        + 1.0d0/4.0d0*GPL1(2)*u*pi**2
     .        + 1.0d0/4.0d0*GPL1(2)*u*v
     .        - 1.0d0/8.0d0*GPL1(2)*u*v*pi*ci
     .        - 1.0d0/8.0d0*GPL1(2)*u*v**2*pi*ci
         col2 = col2 - 1.0d0/48.0d0*GPL1(2)*u**2*omv**(-2)*pi**2
     .        + 1.0d0/8.0d0*GPL1(2)*u**2*omv**(-1)*pi*ci
     .        + 1.0d0/8.0d0*GPL1(2)*u**2
     .        + 5.0d0/16.0d0*GPL1(2)*u**2*pi*ci
     .        - 1.0d0/4.0d0*GPL1(2)*u**2*v*pi*ci
     .        - 1.0d0/8.0d0*GPL1(2)*u**3*pi*ci
     .        - 3.0d0/8.0d0*GPL1(3)*omumv**(-1)*pi*ci
     .        + 187.0d0/72.0d0*GPL1(3)*pi*ci
     .        - 41.0d0/36.0d0*GPL1(3)*pi**2
     .        - 1.0d0/8.0d0*GPL1(3)*pi**3*ci
     .        + 3.0d0/8.0d0*GPL1(3)*v*omumv**(-1)*pi*ci
     .        - 3.0d0/8.0d0*GPL1(3)*v*pi*ci
     .        - 1.0d0/4.0d0*GPL1(3)*v*pi**2
     .        + 7.0d0/16.0d0*GPL1(3)*v**2*pi*ci
     .        - 1.0d0/8.0d0*GPL1(3)*u*omv**(-1)*pi*ci
         col2 = col2 + 1.0d0/2.0d0*GPL1(3)*u*pi*ci
     .        - 1.0d0/4.0d0*GPL1(3)*u*pi**2
     .        + 1.0d0/8.0d0*GPL1(3)*u*v*pi*ci
     .        + 1.0d0/8.0d0*GPL1(3)*u*v**2*pi*ci
     .        - 1.0d0/8.0d0*GPL1(3)*u**2*omv**(-1)*pi*ci
     .        - 5.0d0/16.0d0*GPL1(3)*u**2*pi*ci
     .        + 1.0d0/4.0d0*GPL1(3)*u**2*v*pi*ci
     .        + 1.0d0/8.0d0*GPL1(3)*u**3*pi*ci
     .        + 3.0d0/4.0d0*GPL2(0,0)*pi*ci
     .        - 3.0d0/4.0d0*GPL2(0,0)*v*pi*ci
     .        - 3.0d0/4.0d0*GPL2(0,0)*u*pi*ci
     .        - 3.0d0/4.0d0*GPL2(0,1)*pi*ci
     .        - 1.0d0/4.0d0*GPL2(0,1)*pi**2
     .        + 3.0d0/4.0d0*GPL2(0,1)*v*pi*ci
     .        + 1.0d0/2.0d0*GPL2(0,1)*u*omv**(-1)*pi*ci
         col2 = col2 + 1.0d0/4.0d0*GPL2(0,1)*u*pi*ci
     .        - 1.0d0/8.0d0*GPL2(0,1)*u**2*omv**(-2)*pi*ci
     .        + 1.0d0/8.0d0*GPL2(0,1)*u**2*pi*ci
     .        + 15.0d0/16.0d0*GPL2(0,2)
     .        + 3.0d0/4.0d0*GPL2(0,2)*pi*ci
     .        - 1.0d0/24.0d0*GPL2(0,2)*pi**2
     .        + 1.0d0/2.0d0*GPL2(0,2)*v
     .        - 3.0d0/4.0d0*GPL2(0,2)*v*pi*ci
     .        - 7.0d0/16.0d0*GPL2(0,2)*v**2
     .        - 1.0d0/4.0d0*GPL2(0,2)*u
     .        - 3.0d0/4.0d0*GPL2(0,2)*u*pi*ci
     .        - 1.0d0/8.0d0*GPL2(0,2)*u*v
     .        - 1.0d0/8.0d0*GPL2(0,2)*u*v**2
     .        + 1.0d0/8.0d0*GPL2(0,2)*u**2*omv**(-1)
     .        + 5.0d0/16.0d0*GPL2(0,2)*u**2
         col2 = col2 - 1.0d0/4.0d0*GPL2(0,2)*u**2*v
     .        - 1.0d0/8.0d0*GPL2(0,2)*u**3
     .        - 25.0d0/12.0d0*GPL2(0,3)*pi*ci
     .        + 1.0d0/12.0d0*GPL2(0,3)*pi**2
     .        + 3.0d0/4.0d0*GPL2(0,3)*v*pi*ci
     .        - 1.0d0/2.0d0*GPL2(0,3)*u*omv**(-1)*pi*ci
     .        + 5.0d0/4.0d0*GPL2(0,3)*u*pi*ci
     .        + 1.0d0/8.0d0*GPL2(0,3)*u**2*omv**(-2)*pi*ci
     .        - 1.0d0/8.0d0*GPL2(0,3)*u**2*pi*ci
     .        + 3.0d0/8.0d0*GPL2(1,0)*pi*ci
     .        - 1.0d0/2.0d0*GPL2(1,0)*u*pi*ci
     .        + 1.0d0/8.0d0*GPL2(1,0)*u**2*pi*ci
     .        - 3.0d0/4.0d0*GPL2(1,1)*pi*ci
     .        + GPL2(1,1)*u*pi*ci
     .        - 1.0d0/4.0d0*GPL2(1,1)*u**2*pi*ci
         col2 = col2 - 3.0d0/2.0d0*GPL2(1,2)
     .        + 1.0d0/16.0d0*GPL2(1,2)*u**(-2)
     .        - 1.0d0/8.0d0*GPL2(1,2)*u**(-2)*v
     .        + 1.0d0/16.0d0*GPL2(1,2)*u**(-2)*v**2
     .        - 3.0d0/8.0d0*GPL2(1,2)*u**(-1)
     .        + 3.0d0/8.0d0*GPL2(1,2)*u**(-1)*v
     .        + 3.0d0/8.0d0*GPL2(1,2)*pi*ci
     .        - 5.0d0/8.0d0*GPL2(1,2)*v
     .        + 7.0d0/16.0d0*GPL2(1,2)*v**2
     .        + 1.0d0/8.0d0*GPL2(1,2)*u*omv**(-1)
     .        - 1.0d0/2.0d0*GPL2(1,2)*u*pi*ci
     .        + 1.0d0/8.0d0*GPL2(1,2)*u*v
     .        + 1.0d0/8.0d0*GPL2(1,2)*u*v**2
     .        - 1.0d0/8.0d0*GPL2(1,2)*u**2*omv**(-1)
     .        - 5.0d0/16.0d0*GPL2(1,2)*u**2
         col2 = col2 + 1.0d0/8.0d0*GPL2(1,2)*u**2*pi*ci
     .        + 1.0d0/4.0d0*GPL2(1,2)*u**2*v
     .        + 1.0d0/8.0d0*GPL2(1,2)*u**3
     .        + 15.0d0/16.0d0*GPL2(2,0)
     .        + 3.0d0/4.0d0*GPL2(2,0)*pi*ci
     .        + 1.0d0/2.0d0*GPL2(2,0)*v
     .        - 3.0d0/4.0d0*GPL2(2,0)*v*pi*ci
     .        - 7.0d0/16.0d0*GPL2(2,0)*v**2
     .        - 1.0d0/4.0d0*GPL2(2,0)*u
     .        - 3.0d0/4.0d0*GPL2(2,0)*u*pi*ci
     .        - 1.0d0/8.0d0*GPL2(2,0)*u*v
     .        - 1.0d0/8.0d0*GPL2(2,0)*u*v**2
     .        + 1.0d0/8.0d0*GPL2(2,0)*u**2*omv**(-1)
     .        + 5.0d0/16.0d0*GPL2(2,0)*u**2
     .        - 1.0d0/4.0d0*GPL2(2,0)*u**2*v
         col2 = col2 - 1.0d0/8.0d0*GPL2(2,0)*u**3
     .        + 3.0d0/8.0d0*GPL2(2,1)*pi*ci
     .        - 1.0d0/2.0d0*GPL2(2,1)*u*omv**(-1)*pi*ci
     .        + 1.0d0/8.0d0*GPL2(2,1)*u**2*omv**(-2)*pi*ci
     .        - 3.0d0/4.0d0*GPL2(2,3)*pi*ci
     .        + 3.0d0/4.0d0*GPL2(2,3)*v*pi*ci
     .        + 1.0d0/2.0d0*GPL2(2,3)*u*omv**(-1)*pi*ci
     .        + 3.0d0/4.0d0*GPL2(2,3)*u*pi*ci
     .        - 1.0d0/8.0d0*GPL2(2,3)*u**2*omv**(-2)*pi*ci
     .        + 187.0d0/72.0d0*GPL2(3,0)
     .        - 3.0d0/8.0d0*GPL2(3,0)*omumv**(-1)
     .        + 1.0d0/6.0d0*GPL2(3,0)*pi*ci
     .        - 1.0d0/8.0d0*GPL2(3,0)*pi**2
     .        + 3.0d0/8.0d0*GPL2(3,0)*v*omumv**(-1)
     .        - 3.0d0/8.0d0*GPL2(3,0)*v
         col2 = col2 + 3.0d0/4.0d0*GPL2(3,0)*v*pi*ci
     .        + 7.0d0/16.0d0*GPL2(3,0)*v**2
     .        - 1.0d0/8.0d0*GPL2(3,0)*u*omv**(-1)
     .        + 1.0d0/2.0d0*GPL2(3,0)*u
     .        + 3.0d0/4.0d0*GPL2(3,0)*u*pi*ci
     .        + 1.0d0/8.0d0*GPL2(3,0)*u*v
     .        + 1.0d0/8.0d0*GPL2(3,0)*u*v**2
     .        - 1.0d0/8.0d0*GPL2(3,0)*u**2*omv**(-1)
     .        - 5.0d0/16.0d0*GPL2(3,0)*u**2
     .        + 1.0d0/4.0d0*GPL2(3,0)*u**2*v
     .        + 1.0d0/8.0d0*GPL2(3,0)*u**3
     .        - 3.0d0/8.0d0*GPL2(3,2)*pi*ci
     .        + 1.0d0/8.0d0*GPL2(3,2)*pi**2
     .        + 7.0d0/12.0d0*GPL2(3,3)*pi*ci
     .        - 1.0d0/3.0d0*GPL2(3,3)*pi**2
         col2 = col2 - 3.0d0/4.0d0*GPL2(3,3)*v*pi*ci
     .        - 3.0d0/4.0d0*GPL2(3,3)*u*pi*ci
     .        + 1.0d0/2.0d0*GPL3(0,0,1)*pi*ci
     .        + 3.0d0/4.0d0*GPL3(0,0,2)
     .        - 3.0d0/4.0d0*GPL3(0,0,2)*v
     .        - 3.0d0/4.0d0*GPL3(0,0,2)*u
     .        - 1.0d0/2.0d0*GPL3(0,0,3)*pi*ci
     .        + 1.0d0/4.0d0*GPL3(0,1,0)*pi*ci
     .        - 3.0d0/4.0d0*GPL3(0,1,2)
     .        + 1.0d0/4.0d0*GPL3(0,1,2)*pi*ci
     .        + 3.0d0/4.0d0*GPL3(0,1,2)*v
     .        + 1.0d0/2.0d0*GPL3(0,1,2)*u*omv**(-1)
     .        + 1.0d0/4.0d0*GPL3(0,1,2)*u
     .        - 1.0d0/8.0d0*GPL3(0,1,2)*u**2*omv**(-2)
     .        + 1.0d0/8.0d0*GPL3(0,1,2)*u**2
         col2 = col2 + 3.0d0/4.0d0*GPL3(0,2,0)
     .        - 3.0d0/4.0d0*GPL3(0,2,0)*v
     .        - 3.0d0/4.0d0*GPL3(0,2,0)*u
     .        + 1.0d0/4.0d0*GPL3(0,2,1)*pi*ci
     .        - 1.0d0/4.0d0*GPL3(0,2,3)*pi*ci
     .        - 25.0d0/12.0d0*GPL3(0,3,0)
     .        - 1.0d0/4.0d0*GPL3(0,3,0)*pi*ci
     .        + 3.0d0/4.0d0*GPL3(0,3,0)*v
     .        - 1.0d0/2.0d0*GPL3(0,3,0)*u*omv**(-1)
     .        + 5.0d0/4.0d0*GPL3(0,3,0)*u
     .        + 1.0d0/8.0d0*GPL3(0,3,0)*u**2*omv**(-2)
     .        - 1.0d0/8.0d0*GPL3(0,3,0)*u**2
     .        - 1.0d0/4.0d0*GPL3(0,3,2)*pi*ci
     .        + GPL3(0,3,3)*pi*ci
     .        + 3.0d0/8.0d0*GPL3(1,0,2)
         col2 = col2 - 1.0d0/2.0d0*GPL3(1,0,2)*u
     .        + 1.0d0/8.0d0*GPL3(1,0,2)*u**2
     .        - 3.0d0/4.0d0*GPL3(1,1,2)
     .        + GPL3(1,1,2)*u
     .        - 1.0d0/4.0d0*GPL3(1,1,2)*u**2
     .        + 3.0d0/8.0d0*GPL3(1,2,0)
     .        - 1.0d0/2.0d0*GPL3(1,2,0)*u
     .        + 1.0d0/8.0d0*GPL3(1,2,0)*u**2
     .        + 3.0d0/4.0d0*GPL3(2,0,0)
     .        - 3.0d0/4.0d0*GPL3(2,0,0)*v
     .        - 3.0d0/4.0d0*GPL3(2,0,0)*u
     .        + 3.0d0/8.0d0*GPL3(2,1,2)
     .        - 1.0d0/2.0d0*GPL3(2,1,2)*u*omv**(-1)
     .        + 1.0d0/8.0d0*GPL3(2,1,2)*u**2*omv**(-2)
     .        - 3.0d0/4.0d0*GPL3(2,3,0)
         col2 = col2 + 3.0d0/4.0d0*GPL3(2,3,0)*v
     .        + 1.0d0/2.0d0*GPL3(2,3,0)*u*omv**(-1)
     .        + 3.0d0/4.0d0*GPL3(2,3,0)*u
     .        - 1.0d0/8.0d0*GPL3(2,3,0)*u**2*omv**(-2)
     .        - 7.0d0/4.0d0*GPL3(3,0,0)
     .        + 3.0d0/4.0d0*GPL3(3,0,0)*v
     .        + 3.0d0/4.0d0*GPL3(3,0,0)*u
     .        - 1.0d0/4.0d0*GPL3(3,0,1)*pi*ci
     .        - 3.0d0/8.0d0*GPL3(3,0,2)
     .        + GPL3(3,0,3)*pi*ci
     .        - 3.0d0/8.0d0*GPL3(3,2,0)
     .        - 1.0d0/4.0d0*GPL3(3,2,1)*pi*ci
     .        + 7.0d0/12.0d0*GPL3(3,3,0)
     .        + GPL3(3,3,0)*pi*ci
     .        - 3.0d0/4.0d0*GPL3(3,3,0)*v
         col2 = col2 - 3.0d0/4.0d0*GPL3(3,3,0)*u
     .        - GPL3(3,3,3)*pi*ci
     .        + 1.0d0/2.0d0*GPL4(0,0,1,2)
     .        - 1.0d0/2.0d0*GPL4(0,0,3,0)
     .        + 1.0d0/4.0d0*GPL4(0,1,0,2)
     .        + 1.0d0/4.0d0*GPL4(0,1,2,0)
     .        + 1.0d0/4.0d0*GPL4(0,2,1,2)
     .        - 1.0d0/4.0d0*GPL4(0,2,3,0)
     .        - 1.0d0/4.0d0*GPL4(0,3,0,2)
     .        - 1.0d0/4.0d0*GPL4(0,3,2,0)
     .        + GPL4(0,3,3,0)
     .        - 1.0d0/4.0d0*GPL4(3,0,1,2)
     .        + GPL4(3,0,3,0)
     .        - 1.0d0/4.0d0*GPL4(3,2,1,2)
     .        + GPL4(3,3,0,0)
         col2 = col2 - GPL4(3,3,3,0)

         col3 = - 335.0d0/128.0d0
     .        - 5.0d0/16.0d0*u**(-1)*pi*ci
     .        + 5.0d0/8.0d0*u**(-1)*v*pi*ci
     .        - 5.0d0/16.0d0*u**(-1)*v**2*pi*ci
     .        + 1.0d0/16.0d0*omumv**(-1)*pi**2
     .        + 11.0d0/16.0d0*pi*ci
     .        + 43.0d0/96.0d0*pi**2
     .        + 1.0d0/16.0d0*pi**3*ci
     .        - 5.0d0/96.0d0*pi**4
     .        + 3.0d0*zeta3
     .        - 7.0d0/4.0d0*zeta3*pi*ci
     .        - 1.0d0/16.0d0*v*omumv**(-1)*pi**2
     .        - 17.0d0/16.0d0*v*upv**(-1)*pi*ci
     .        + 5.0d0/8.0d0*v
     .        + v*pi*ci
         col3 = col3 - 5.0d0/8.0d0*v*pi**2
     .        + 1.0d0/6.0d0*v*pi**3*ci
     .        - v*zeta3
     .        - 5.0d0/8.0d0*v**2*pi*ci
     .        - 1.0d0/24.0d0*v**2*pi**2
     .        - 1.0d0/24.0d0*v**2*pi**3*ci
     .        + 1.0d0/4.0d0*v**2*zeta3
     .        - 1.0d0/48.0d0*u*omv**(-1)*pi**2
     .        - 1.0d0/6.0d0*u*omv**(-1)*pi**3*ci
     .        + 5.0d0/8.0d0*u
     .        - 53.0d0/48.0d0*u*pi**2
     .        + 1.0d0/6.0d0*u*pi**3*ci
     .        - 1.0d0/2.0d0*u*zeta3*omv**(-1)
     .        - u*zeta3
     .        - 5.0d0/8.0d0*u*v*pi*ci
         col3 = col3 + 11.0d0/12.0d0*u*v*pi**2
     .        - 1.0d0/12.0d0*u*v*pi**3*ci
     .        + 1.0d0/2.0d0*u*v*zeta3
     .        - 5.0d0/12.0d0*u*v**2*pi**2
     .        + 1.0d0/24.0d0*u**2*omv**(-2)*pi**3*ci
     .        - 1.0d0/12.0d0*u**2*omv**(-1)*pi**2
     .        + 23.0d0/24.0d0*u**2*pi**2
     .        - 1.0d0/24.0d0*u**2*pi**3*ci
     .        + 1.0d0/8.0d0*u**2*zeta3*omv**(-2)
     .        + 1.0d0/4.0d0*u**2*zeta3
     .        - 5.0d0/6.0d0*u**2*v*pi**2
     .        - 5.0d0/12.0d0*u**3*pi**2
     .        + 3.0d0/32.0d0*HPL1(0)
     .        - 5.0d0/16.0d0*HPL1(0)*u**(-1)*v
     .        + 5.0d0/16.0d0*HPL1(0)*u**(-1)*v**2
         col3 = col3 + 3.0d0/8.0d0*HPL1(0)*omumv**(-1)*pi*ci
     .        + 23.0d0/16.0d0*HPL1(0)*pi*ci
     .        - 3.0d0/8.0d0*HPL1(0)*pi**2
     .        - 1.0d0/12.0d0*HPL1(0)*pi**3*ci
     .        + 2.0d0*HPL1(0)*zeta3
     .        - 3.0d0/8.0d0*HPL1(0)*v*omumv**(-1)*pi*ci
     .        + 17.0d0/16.0d0*HPL1(0)*v*upv**(-1)
     .        - HPL1(0)*v
     .        - 7.0d0/8.0d0*HPL1(0)*v*pi*ci
     .        - 1.0d0/6.0d0*HPL1(0)*v*pi**2
     .        + 5.0d0/8.0d0*HPL1(0)*v**2
     .        - 1.0d0/16.0d0*HPL1(0)*v**2*pi*ci
     .        + 1.0d0/24.0d0*HPL1(0)*v**2*pi**2
     .        - 1.0d0/8.0d0*HPL1(0)*u*omv**(-1)*pi*ci
     .        + 1.0d0/3.0d0*HPL1(0)*u*omv**(-1)*pi**2
         col3 = col3 - 3.0d0/2.0d0*HPL1(0)*u*pi*ci
     .        - 1.0d0/6.0d0*HPL1(0)*u*pi**2
     .        + 5.0d0/8.0d0*HPL1(0)*u*v
     .        + 11.0d0/8.0d0*HPL1(0)*u*v*pi*ci
     .        + 1.0d0/12.0d0*HPL1(0)*u*v*pi**2
     .        - 5.0d0/8.0d0*HPL1(0)*u*v**2*pi*ci
     .        - 1.0d0/12.0d0*HPL1(0)*u**2*omv**(-2)*pi**2
     .        - 1.0d0/8.0d0*HPL1(0)*u**2*omv**(-1)*pi*ci
     .        + 23.0d0/16.0d0*HPL1(0)*u**2*pi*ci
     .        + 1.0d0/24.0d0*HPL1(0)*u**2*pi**2
     .        - 5.0d0/4.0d0*HPL1(0)*u**2*v*pi*ci
     .        - 5.0d0/8.0d0*HPL1(0)*u**3*pi*ci
     .        + 23.0d0/16.0d0*HPL1(0)*GPL1(0)
     .        + 3.0d0/8.0d0*HPL1(0)*GPL1(0)*omumv**(-1)
     .        + 3.0d0/8.0d0*HPL1(0)*GPL1(0)*pi*ci
         col3 = col3 - 1.0d0/3.0d0*HPL1(0)*GPL1(0)*pi**2
     .        - 3.0d0/8.0d0*HPL1(0)*GPL1(0)*v*omumv**(-1)
     .        - HPL1(0)*GPL1(0)*v
     .        + 1.0d0/2.0d0*HPL1(0)*GPL1(0)*v*pi*ci
     .        - 1.0d0/16.0d0*HPL1(0)*GPL1(0)*v**2
     .        - 1.0d0/8.0d0*HPL1(0)*GPL1(0)*v**2*pi*ci
     .        - 1.0d0/2.0d0*HPL1(0)*GPL1(0)*u*omv**(-1)*pi*ci
     .        - 13.0d0/8.0d0*HPL1(0)*GPL1(0)*u
     .        + 1.0d0/2.0d0*HPL1(0)*GPL1(0)*u*pi*ci
     .        + 11.0d0/8.0d0*HPL1(0)*GPL1(0)*u*v
     .        - 1.0d0/4.0d0*HPL1(0)*GPL1(0)*u*v*pi*ci
     .        - 5.0d0/8.0d0*HPL1(0)*GPL1(0)*u*v**2
     .        + 1.0d0/8.0d0*HPL1(0)*GPL1(0)*u**2*omv**(-2)*pi*ci
     .        - 1.0d0/8.0d0*HPL1(0)*GPL1(0)*u**2*omv**(-1)
     .        + 23.0d0/16.0d0*HPL1(0)*GPL1(0)*u**2
         col3 = col3 - 1.0d0/8.0d0*HPL1(0)*GPL1(0)*u**2*pi*ci
     .        - 5.0d0/4.0d0*HPL1(0)*GPL1(0)*u**2*v
     .        - 5.0d0/8.0d0*HPL1(0)*GPL1(0)*u**3
     .        - 2.0d0*HPL1(0)*GPL1(1)
     .        + 5.0d0/16.0d0*HPL1(0)*GPL1(1)*u**(-2)
     .        - 5.0d0/8.0d0*HPL1(0)*GPL1(1)*u**(-2)*v
     .        + 5.0d0/16.0d0*HPL1(0)*GPL1(1)*u**(-2)*v**2
     .        + 1.0d0/2.0d0*HPL1(0)*GPL1(1)*u**(-1)
     .        - 5.0d0/8.0d0*HPL1(0)*GPL1(1)*u**(-1)*v
     .        + 1.0d0/8.0d0*HPL1(0)*GPL1(1)*u**(-1)*v**2
     .        + 11.0d0/8.0d0*HPL1(0)*GPL1(1)*v
     .        + 1.0d0/16.0d0*HPL1(0)*GPL1(1)*v**2
     .        - 1.0d0/8.0d0*HPL1(0)*GPL1(1)*u*omv**(-1)
     .        + 2.0d0*HPL1(0)*GPL1(1)*u
     .        - 11.0d0/8.0d0*HPL1(0)*GPL1(1)*u*v
         col3 = col3 + 5.0d0/8.0d0*HPL1(0)*GPL1(1)*u*v**2
     .        + 1.0d0/8.0d0*HPL1(0)*GPL1(1)*u**2*omv**(-1)
     .        - 23.0d0/16.0d0*HPL1(0)*GPL1(1)*u**2
     .        + 5.0d0/4.0d0*HPL1(0)*GPL1(1)*u**2*v
     .        + 5.0d0/8.0d0*HPL1(0)*GPL1(1)*u**3
     .        + 2.0d0*HPL1(0)*GPL1(2)
     .        - 5.0d0/16.0d0*HPL1(0)*GPL1(2)*u**(-2)
     .        + 5.0d0/8.0d0*HPL1(0)*GPL1(2)*u**(-2)*v
     .        - 5.0d0/16.0d0*HPL1(0)*GPL1(2)*u**(-2)*v**2
     .        - 1.0d0/2.0d0*HPL1(0)*GPL1(2)*u**(-1)
     .        + 5.0d0/8.0d0*HPL1(0)*GPL1(2)*u**(-1)*v
     .        - 1.0d0/8.0d0*HPL1(0)*GPL1(2)*u**(-1)*v**2
     .        - 3.0d0/4.0d0*HPL1(0)*GPL1(2)*pi*ci
     .        + 1.0d0/3.0d0*HPL1(0)*GPL1(2)*pi**2
     .        - 5.0d0/4.0d0*HPL1(0)*GPL1(2)*v
         col3 = col3 - 1.0d0/16.0d0*HPL1(0)*GPL1(2)*v**2
     .        + 1.0d0/2.0d0*HPL1(0)*GPL1(2)*u*omv**(-1)*pi*ci
     .        - 15.0d0/8.0d0*HPL1(0)*GPL1(2)*u
     .        + 11.0d0/8.0d0*HPL1(0)*GPL1(2)*u*v
     .        - 5.0d0/8.0d0*HPL1(0)*GPL1(2)*u*v**2
     .        - 1.0d0/8.0d0*HPL1(0)*GPL1(2)*u**2*omv**(-2)*pi*ci
     .        - 1.0d0/8.0d0*HPL1(0)*GPL1(2)*u**2*omv**(-1)
     .        + 23.0d0/16.0d0*HPL1(0)*GPL1(2)*u**2
     .        - 5.0d0/4.0d0*HPL1(0)*GPL1(2)*u**2*v
     .        - 5.0d0/8.0d0*HPL1(0)*GPL1(2)*u**3
     .        - 23.0d0/16.0d0*HPL1(0)*GPL1(3)
     .        - 3.0d0/8.0d0*HPL1(0)*GPL1(3)*omumv**(-1)
     .        - 3.0d0/4.0d0*HPL1(0)*GPL1(3)*pi*ci
     .        + 1.0d0/3.0d0*HPL1(0)*GPL1(3)*pi**2
     .        + 3.0d0/8.0d0*HPL1(0)*GPL1(3)*v*omumv**(-1)
         col3 = col3 + 7.0d0/8.0d0*HPL1(0)*GPL1(3)*v
     .        + 1.0d0/2.0d0*HPL1(0)*GPL1(3)*v*pi*ci
     .        + 1.0d0/16.0d0*HPL1(0)*GPL1(3)*v**2
     .        - 1.0d0/8.0d0*HPL1(0)*GPL1(3)*v**2*pi*ci
     .        + 1.0d0/8.0d0*HPL1(0)*GPL1(3)*u*omv**(-1)
     .        + 3.0d0/2.0d0*HPL1(0)*GPL1(3)*u
     .        + 1.0d0/2.0d0*HPL1(0)*GPL1(3)*u*pi*ci
     .        - 11.0d0/8.0d0*HPL1(0)*GPL1(3)*u*v
     .        - 1.0d0/4.0d0*HPL1(0)*GPL1(3)*u*v*pi*ci
     .        + 5.0d0/8.0d0*HPL1(0)*GPL1(3)*u*v**2
     .        + 1.0d0/8.0d0*HPL1(0)*GPL1(3)*u**2*omv**(-1)
     .        - 23.0d0/16.0d0*HPL1(0)*GPL1(3)*u**2
     .        - 1.0d0/8.0d0*HPL1(0)*GPL1(3)*u**2*pi*ci
     .        + 5.0d0/4.0d0*HPL1(0)*GPL1(3)*u**2*v
     .        + 5.0d0/8.0d0*HPL1(0)*GPL1(3)*u**3
         col3 = col3 + 1.0d0/2.0d0*HPL1(0)*GPL2(0,0)*pi*ci
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(0,0)*v
     .        - 1.0d0/8.0d0*HPL1(0)*GPL2(0,0)*v**2
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(0,0)*u
     .        - 1.0d0/4.0d0*HPL1(0)*GPL2(0,0)*u*v
     .        - 1.0d0/8.0d0*HPL1(0)*GPL2(0,0)*u**2
     .        + 3.0d0/4.0d0*HPL1(0)*GPL2(0,1)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(0,1)*v
     .        + 1.0d0/8.0d0*HPL1(0)*GPL2(0,1)*v**2
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(0,1)*u*omv**(-1)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(0,1)*u
     .        + 1.0d0/4.0d0*HPL1(0)*GPL2(0,1)*u*v
     .        + 1.0d0/8.0d0*HPL1(0)*GPL2(0,1)*u**2*omv**(-2)
     .        + 1.0d0/8.0d0*HPL1(0)*GPL2(0,1)*u**2
     .        - 3.0d0/8.0d0*HPL1(0)*GPL2(0,2)
         col3 = col3 - 1.0d0/2.0d0*HPL1(0)*GPL2(0,2)*pi*ci
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(0,2)*v
     .        - 1.0d0/8.0d0*HPL1(0)*GPL2(0,2)*v**2
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(0,2)*u
     .        - 1.0d0/4.0d0*HPL1(0)*GPL2(0,2)*u*v
     .        - 1.0d0/8.0d0*HPL1(0)*GPL2(0,2)*u**2
     .        - 3.0d0/8.0d0*HPL1(0)*GPL2(0,3)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(0,3)*pi*ci
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(0,3)*v
     .        + 1.0d0/8.0d0*HPL1(0)*GPL2(0,3)*v**2
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(0,3)*u*omv**(-1)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(0,3)*u
     .        + 1.0d0/4.0d0*HPL1(0)*GPL2(0,3)*u*v
     .        - 1.0d0/8.0d0*HPL1(0)*GPL2(0,3)*u**2*omv**(-2)
     .        + 1.0d0/8.0d0*HPL1(0)*GPL2(0,3)*u**2
         col3 = col3 - 3.0d0/4.0d0*HPL1(0)*GPL2(2,0)
     .        - 1.0d0/4.0d0*HPL1(0)*GPL2(2,0)*pi*ci
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(2,0)*v
     .        - 1.0d0/8.0d0*HPL1(0)*GPL2(2,0)*v**2
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(2,0)*u
     .        - 1.0d0/4.0d0*HPL1(0)*GPL2(2,0)*u*v
     .        - 1.0d0/8.0d0*HPL1(0)*GPL2(2,0)*u**2
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(2,1)*v
     .        + 1.0d0/8.0d0*HPL1(0)*GPL2(2,1)*v**2
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(2,1)*u*omv**(-1)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(2,1)*u
     .        + 1.0d0/4.0d0*HPL1(0)*GPL2(2,1)*u*v
     .        - 1.0d0/8.0d0*HPL1(0)*GPL2(2,1)*u**2*omv**(-2)
     .        + 1.0d0/8.0d0*HPL1(0)*GPL2(2,1)*u**2
     .        + 3.0d0/4.0d0*HPL1(0)*GPL2(2,3)
         col3 = col3 + 1.0d0/2.0d0*HPL1(0)*GPL2(2,3)*pi*ci
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(2,3)*u*omv**(-1)
     .        + 1.0d0/8.0d0*HPL1(0)*GPL2(2,3)*u**2*omv**(-2)
     .        - 3.0d0/8.0d0*HPL1(0)*GPL2(3,0)
     .        - 3.0d0/4.0d0*HPL1(0)*GPL2(3,0)*pi*ci
     .        - 3.0d0/8.0d0*HPL1(0)*GPL2(3,2)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(3,2)*v
     .        - 1.0d0/8.0d0*HPL1(0)*GPL2(3,2)*v**2
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(3,2)*u
     .        - 1.0d0/4.0d0*HPL1(0)*GPL2(3,2)*u*v
     .        - 1.0d0/8.0d0*HPL1(0)*GPL2(3,2)*u**2
     .        + 3.0d0/4.0d0*HPL1(0)*GPL2(3,3)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(3,3)*pi*ci
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(3,3)*v
     .        + 1.0d0/8.0d0*HPL1(0)*GPL2(3,3)*v**2
         col3 = col3 - 1.0d0/2.0d0*HPL1(0)*GPL2(3,3)*u
     .        + 1.0d0/4.0d0*HPL1(0)*GPL2(3,3)*u*v
     .        + 1.0d0/8.0d0*HPL1(0)*GPL2(3,3)*u**2
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(0,0,1)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL3(0,0,3)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL3(0,2,1)
     .        + 1.0d0/4.0d0*HPL1(0)*GPL3(0,2,3)
     .        - 1.0d0/4.0d0*HPL1(0)*GPL3(0,3,0)
     .        - 1.0d0/4.0d0*HPL1(0)*GPL3(0,3,2)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(0,3,3)
     .        + 1.0d0/4.0d0*HPL1(0)*GPL3(2,0,0)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL3(2,0,2)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(2,3,2)
     .        - 1.0d0/4.0d0*HPL1(0)*GPL3(2,3,3)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL3(3,0,0)
         col3 = col3 + 1.0d0/2.0d0*HPL1(0)*GPL3(3,0,1)
     .        - 3.0d0/4.0d0*HPL1(0)*GPL3(3,0,2)
     .        + 3.0d0/4.0d0*HPL1(0)*GPL3(3,0,3)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL3(3,2,0)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(3,2,1)
     .        + 1.0d0/4.0d0*HPL1(0)*GPL3(3,2,3)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(3,3,0)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL3(3,3,3)
     .        + 17.0d0/32.0d0*HPL1(1)
     .        + 5.0d0/16.0d0*HPL1(1)*u**(-1)
     .        - 5.0d0/8.0d0*HPL1(1)*u**(-1)*v
     .        + 5.0d0/16.0d0*HPL1(1)*u**(-1)*v**2
     .        + 15.0d0/16.0d0*HPL1(1)*pi*ci
     .        + 1.0d0/8.0d0*HPL1(1)*pi**2
     .        + 1.0d0/4.0d0*HPL1(1)*pi**3*ci
         col3 = col3 + 3.0d0/4.0d0*HPL1(1)*zeta3
     .        - 17.0d0/16.0d0*HPL1(1)*v
     .        - 7.0d0/8.0d0*HPL1(1)*v*pi*ci
     .        - 1.0d0/6.0d0*HPL1(1)*v*pi**2
     .        + 5.0d0/8.0d0*HPL1(1)*v**2
     .        - 1.0d0/16.0d0*HPL1(1)*v**2*pi*ci
     .        + 1.0d0/24.0d0*HPL1(1)*v**2*pi**2
     .        - 1.0d0/12.0d0*HPL1(1)*u*omv**(-1)*pi**2
     .        - 11.0d0/8.0d0*HPL1(1)*u
     .        - 13.0d0/8.0d0*HPL1(1)*u*pi*ci
     .        - 1.0d0/6.0d0*HPL1(1)*u*pi**2
     .        + 5.0d0/4.0d0*HPL1(1)*u*v
     .        + 11.0d0/8.0d0*HPL1(1)*u*v*pi*ci
     .        + 1.0d0/12.0d0*HPL1(1)*u*v*pi**2
     .        - 5.0d0/8.0d0*HPL1(1)*u*v**2*pi*ci
         col3 = col3 + 1.0d0/48.0d0*HPL1(1)*u**2*omv**(-2)*pi**2
     .        - 1.0d0/8.0d0*HPL1(1)*u**2*omv**(-1)*pi*ci
     .        + 5.0d0/8.0d0*HPL1(1)*u**2
     .        + 23.0d0/16.0d0*HPL1(1)*u**2*pi*ci
     .        + 1.0d0/24.0d0*HPL1(1)*u**2*pi**2
     .        - 5.0d0/4.0d0*HPL1(1)*u**2*v*pi*ci
     .        - 5.0d0/8.0d0*HPL1(1)*u**3*pi*ci
     .        + 15.0d0/16.0d0*HPL1(1)*GPL1(0)
     .        - 3.0d0/8.0d0*HPL1(1)*GPL1(0)*pi*ci
     .        + 1.0d0/12.0d0*HPL1(1)*GPL1(0)*pi**2
     .        - 7.0d0/8.0d0*HPL1(1)*GPL1(0)*v
     .        + 1.0d0/2.0d0*HPL1(1)*GPL1(0)*v*pi*ci
     .        - 1.0d0/16.0d0*HPL1(1)*GPL1(0)*v**2
     .        - 1.0d0/8.0d0*HPL1(1)*GPL1(0)*v**2*pi*ci
     .        - 13.0d0/8.0d0*HPL1(1)*GPL1(0)*u
         col3 = col3 + 1.0d0/2.0d0*HPL1(1)*GPL1(0)*u*pi*ci
     .        + 11.0d0/8.0d0*HPL1(1)*GPL1(0)*u*v
     .        - 1.0d0/4.0d0*HPL1(1)*GPL1(0)*u*v*pi*ci
     .        - 5.0d0/8.0d0*HPL1(1)*GPL1(0)*u*v**2
     .        - 1.0d0/8.0d0*HPL1(1)*GPL1(0)*u**2*omv**(-1)
     .        + 23.0d0/16.0d0*HPL1(1)*GPL1(0)*u**2
     .        - 1.0d0/8.0d0*HPL1(1)*GPL1(0)*u**2*pi*ci
     .        - 5.0d0/4.0d0*HPL1(1)*GPL1(0)*u**2*v
     .        - 5.0d0/8.0d0*HPL1(1)*GPL1(0)*u**3
     .        - 2.0d0*HPL1(1)*GPL1(1)
     .        + 5.0d0/16.0d0*HPL1(1)*GPL1(1)*u**(-2)
     .        - 5.0d0/8.0d0*HPL1(1)*GPL1(1)*u**(-2)*v
     .        + 5.0d0/16.0d0*HPL1(1)*GPL1(1)*u**(-2)*v**2
     .        + 1.0d0/2.0d0*HPL1(1)*GPL1(1)*u**(-1)
     .        - 5.0d0/8.0d0*HPL1(1)*GPL1(1)*u**(-1)*v
         col3 = col3 + 1.0d0/8.0d0*HPL1(1)*GPL1(1)*u**(-1)*v**2
     .        + 11.0d0/8.0d0*HPL1(1)*GPL1(1)*v
     .        + 1.0d0/16.0d0*HPL1(1)*GPL1(1)*v**2
     .        - 1.0d0/8.0d0*HPL1(1)*GPL1(1)*u*omv**(-1)
     .        + 2.0d0*HPL1(1)*GPL1(1)*u
     .        - 11.0d0/8.0d0*HPL1(1)*GPL1(1)*u*v
     .        + 5.0d0/8.0d0*HPL1(1)*GPL1(1)*u*v**2
     .        + 1.0d0/8.0d0*HPL1(1)*GPL1(1)*u**2*omv**(-1)
     .        - 23.0d0/16.0d0*HPL1(1)*GPL1(1)*u**2
     .        + 5.0d0/4.0d0*HPL1(1)*GPL1(1)*u**2*v
     .        + 5.0d0/8.0d0*HPL1(1)*GPL1(1)*u**3
     .        - 1.0d0/24.0d0*HPL1(1)*GPL1(2)*pi**2
     .        - 3.0d0/8.0d0*HPL1(1)*GPL1(3)*pi*ci
     .        + 1.0d0/24.0d0*HPL1(1)*GPL1(3)*pi**2
     .        + 1.0d0/2.0d0*HPL1(1)*GPL1(3)*v*pi*ci
         col3 = col3 - 1.0d0/8.0d0*HPL1(1)*GPL1(3)*v**2*pi*ci
     .        + 1.0d0/2.0d0*HPL1(1)*GPL1(3)*u*pi*ci
     .        - 1.0d0/4.0d0*HPL1(1)*GPL1(3)*u*v*pi*ci
     .        - 1.0d0/8.0d0*HPL1(1)*GPL1(3)*u**2*pi*ci
     .        - 3.0d0/8.0d0*HPL1(1)*GPL2(0,0)
     .        + 1.0d0/2.0d0*HPL1(1)*GPL2(0,0)*v
     .        - 1.0d0/8.0d0*HPL1(1)*GPL2(0,0)*v**2
     .        + 1.0d0/2.0d0*HPL1(1)*GPL2(0,0)*u
     .        - 1.0d0/4.0d0*HPL1(1)*GPL2(0,0)*u*v
     .        - 1.0d0/8.0d0*HPL1(1)*GPL2(0,0)*u**2
     .        + 3.0d0/4.0d0*HPL1(1)*GPL2(0,1)
     .        - 1.0d0/2.0d0*HPL1(1)*GPL2(0,1)*v
     .        + 1.0d0/8.0d0*HPL1(1)*GPL2(0,1)*v**2
     .        - 1.0d0/2.0d0*HPL1(1)*GPL2(0,1)*u*omv**(-1)
     .        - 1.0d0/2.0d0*HPL1(1)*GPL2(0,1)*u
         col3 = col3 + 1.0d0/4.0d0*HPL1(1)*GPL2(0,1)*u*v
     .        + 1.0d0/8.0d0*HPL1(1)*GPL2(0,1)*u**2*omv**(-2)
     .        + 1.0d0/8.0d0*HPL1(1)*GPL2(0,1)*u**2
     .        - 1.0d0/4.0d0*HPL1(1)*GPL2(0,3)*pi*ci
     .        - 1.0d0/2.0d0*HPL1(1)*GPL2(2,1)*v
     .        + 1.0d0/8.0d0*HPL1(1)*GPL2(2,1)*v**2
     .        + 1.0d0/2.0d0*HPL1(1)*GPL2(2,1)*u*omv**(-1)
     .        - 1.0d0/2.0d0*HPL1(1)*GPL2(2,1)*u
     .        + 1.0d0/4.0d0*HPL1(1)*GPL2(2,1)*u*v
     .        - 1.0d0/8.0d0*HPL1(1)*GPL2(2,1)*u**2*omv**(-2)
     .        + 1.0d0/8.0d0*HPL1(1)*GPL2(2,1)*u**2
     .        + 1.0d0/2.0d0*HPL1(1)*GPL2(2,3)*pi*ci
     .        - 3.0d0/8.0d0*HPL1(1)*GPL2(3,0)
     .        - 1.0d0/2.0d0*HPL1(1)*GPL2(3,0)*pi*ci
     .        + 1.0d0/2.0d0*HPL1(1)*GPL2(3,0)*v
         col3 = col3 - 1.0d0/8.0d0*HPL1(1)*GPL2(3,0)*v**2
     .        + 1.0d0/2.0d0*HPL1(1)*GPL2(3,0)*u
     .        - 1.0d0/4.0d0*HPL1(1)*GPL2(3,0)*u*v
     .        - 1.0d0/8.0d0*HPL1(1)*GPL2(3,0)*u**2
     .        + 1.0d0/2.0d0*HPL1(1)*GPL3(0,0,1)
     .        - 1.0d0/2.0d0*HPL1(1)*GPL3(0,2,1)
     .        - 1.0d0/4.0d0*HPL1(1)*GPL3(0,3,0)
     .        + 1.0d0/2.0d0*HPL1(1)*GPL3(2,3,0)
     .        - 1.0d0/2.0d0*HPL1(1)*GPL3(3,0,0)
     .        + 1.0d0/2.0d0*HPL1(1)*GPL3(3,0,1)
     .        + 1.0d0/2.0d0*HPL1(1)*GPL3(3,2,1)
     .        - 23.0d0/16.0d0*HPL2(0,0)
     .        - 3.0d0/8.0d0*HPL2(0,0)*omumv**(-1)
     .        - 3.0d0/4.0d0*HPL2(0,0)*pi*ci
     .        + 1.0d0/3.0d0*HPL2(0,0)*pi**2
         col3 = col3 + 3.0d0/8.0d0*HPL2(0,0)*v*omumv**(-1)
     .        + 7.0d0/8.0d0*HPL2(0,0)*v
     .        + 1.0d0/16.0d0*HPL2(0,0)*v**2
     .        + 1.0d0/8.0d0*HPL2(0,0)*u*omv**(-1)
     .        + 1.0d0/2.0d0*HPL2(0,0)*u*omv**(-1)*pi*ci
     .        + 3.0d0/2.0d0*HPL2(0,0)*u
     .        - 11.0d0/8.0d0*HPL2(0,0)*u*v
     .        + 5.0d0/8.0d0*HPL2(0,0)*u*v**2
     .        - 1.0d0/8.0d0*HPL2(0,0)*u**2*omv**(-2)*pi*ci
     .        + 1.0d0/8.0d0*HPL2(0,0)*u**2*omv**(-1)
     .        - 23.0d0/16.0d0*HPL2(0,0)*u**2
     .        + 5.0d0/4.0d0*HPL2(0,0)*u**2*v
     .        + 5.0d0/8.0d0*HPL2(0,0)*u**3
     .        - 3.0d0/8.0d0*HPL2(0,0)*GPL1(0)
     .        - 3.0d0/4.0d0*HPL2(0,0)*GPL1(0)*pi*ci
         col3 = col3 - 1.0d0/2.0d0*HPL2(0,0)*GPL1(0)*v
     .        + 1.0d0/8.0d0*HPL2(0,0)*GPL1(0)*v**2
     .        + 1.0d0/2.0d0*HPL2(0,0)*GPL1(0)*u*omv**(-1)
     .        - 1.0d0/2.0d0*HPL2(0,0)*GPL1(0)*u
     .        + 1.0d0/4.0d0*HPL2(0,0)*GPL1(0)*u*v
     .        - 1.0d0/8.0d0*HPL2(0,0)*GPL1(0)*u**2*omv**(-2)
     .        + 1.0d0/8.0d0*HPL2(0,0)*GPL1(0)*u**2
     .        + 3.0d0/4.0d0*HPL2(0,0)*GPL1(2)
     .        + 1.0d0/2.0d0*HPL2(0,0)*GPL1(2)*pi*ci
     .        - 1.0d0/2.0d0*HPL2(0,0)*GPL1(2)*u*omv**(-1)
     .        + 1.0d0/8.0d0*HPL2(0,0)*GPL1(2)*u**2*omv**(-2)
     .        + 3.0d0/4.0d0*HPL2(0,0)*GPL1(3)
     .        + 3.0d0/4.0d0*HPL2(0,0)*GPL1(3)*pi*ci
     .        - 1.0d0/2.0d0*HPL2(0,0)*GPL1(3)*v
     .        + 1.0d0/8.0d0*HPL2(0,0)*GPL1(3)*v**2
         col3 = col3 - 1.0d0/2.0d0*HPL2(0,0)*GPL1(3)*u
     .        + 1.0d0/4.0d0*HPL2(0,0)*GPL1(3)*u*v
     .        + 1.0d0/8.0d0*HPL2(0,0)*GPL1(3)*u**2
     .        - 1.0d0/2.0d0*HPL2(0,0)*GPL2(0,0)
     .        + 1.0d0/4.0d0*HPL2(0,0)*GPL2(0,2)
     .        + 1.0d0/2.0d0*HPL2(0,0)*GPL2(0,3)
     .        - 1.0d0/4.0d0*HPL2(0,0)*GPL2(2,3)
     .        + 3.0d0/4.0d0*HPL2(0,0)*GPL2(3,0)
     .        + 1.0d0/4.0d0*HPL2(0,0)*GPL2(3,2)
     .        - 1.0d0/2.0d0*HPL2(0,0)*GPL2(3,3)
     .        - 3.0d0/8.0d0*HPL2(0,1)*pi*ci
     .        + 1.0d0/24.0d0*HPL2(0,1)*pi**2
     .        - 1.0d0/8.0d0*HPL2(0,1)*v
     .        + 1.0d0/8.0d0*HPL2(0,1)*u*omv**(-1)
     .        + 1.0d0/2.0d0*HPL2(0,1)*u*omv**(-1)*pi*ci
         col3 = col3 - 1.0d0/8.0d0*HPL2(0,1)*u
     .        - 1.0d0/8.0d0*HPL2(0,1)*u**2*omv**(-2)*pi*ci
     .        - 3.0d0/8.0d0*HPL2(0,1)*GPL1(0)
     .        - 1.0d0/2.0d0*HPL2(0,1)*GPL1(0)*pi*ci
     .        + 1.0d0/2.0d0*HPL2(0,1)*GPL1(0)*u*omv**(-1)
     .        - 1.0d0/8.0d0*HPL2(0,1)*GPL1(0)*u**2*omv**(-2)
     .        + 1.0d0/2.0d0*HPL2(0,1)*GPL1(2)*pi*ci
     .        + 1.0d0/2.0d0*HPL2(0,1)*GPL1(2)*v
     .        - 1.0d0/8.0d0*HPL2(0,1)*GPL1(2)*v**2
     .        - 1.0d0/2.0d0*HPL2(0,1)*GPL1(2)*u*omv**(-1)
     .        + 1.0d0/2.0d0*HPL2(0,1)*GPL1(2)*u
     .        - 1.0d0/4.0d0*HPL2(0,1)*GPL1(2)*u*v
     .        + 1.0d0/8.0d0*HPL2(0,1)*GPL1(2)*u**2*omv**(-2)
     .        - 1.0d0/8.0d0*HPL2(0,1)*GPL1(2)*u**2
     .        + 3.0d0/8.0d0*HPL2(0,1)*GPL1(3)
         col3 = col3 + 1.0d0/4.0d0*HPL2(0,1)*GPL1(3)*pi*ci
     .        - 1.0d0/2.0d0*HPL2(0,1)*GPL1(3)*v
     .        + 1.0d0/8.0d0*HPL2(0,1)*GPL1(3)*v**2
     .        - 1.0d0/2.0d0*HPL2(0,1)*GPL1(3)*u
     .        + 1.0d0/4.0d0*HPL2(0,1)*GPL1(3)*u*v
     .        + 1.0d0/8.0d0*HPL2(0,1)*GPL1(3)*u**2
     .        - 1.0d0/2.0d0*HPL2(0,1)*GPL2(0,0)
     .        + 1.0d0/2.0d0*HPL2(0,1)*GPL2(0,2)
     .        + 1.0d0/4.0d0*HPL2(0,1)*GPL2(0,3)
     .        + 1.0d0/2.0d0*HPL2(0,1)*GPL2(2,0)
     .        - 1.0d0/2.0d0*HPL2(0,1)*GPL2(2,3)
     .        + 1.0d0/4.0d0*HPL2(0,1)*GPL2(3,0)
     .        - 1.0d0/2.0d0*HPL2(0,1)*GPL2(3,2)
     .        - 9.0d0/16.0d0*HPL2(1,0)
     .        + 3.0d0/8.0d0*HPL2(1,0)*omumv**(-1)
         col3 = col3 + 3.0d0/4.0d0*HPL2(1,0)*pi*ci
     .        - 1.0d0/2.0d0*HPL2(1,0)*pi**2
     .        - 3.0d0/8.0d0*HPL2(1,0)*v*omumv**(-1)
     .        + 7.0d0/8.0d0*HPL2(1,0)*v
     .        + 1.0d0/16.0d0*HPL2(1,0)*v**2
     .        - 1.0d0/2.0d0*HPL2(1,0)*u*omv**(-1)*pi*ci
     .        + 7.0d0/4.0d0*HPL2(1,0)*u
     .        - 11.0d0/8.0d0*HPL2(1,0)*u*v
     .        + 5.0d0/8.0d0*HPL2(1,0)*u*v**2
     .        + 1.0d0/8.0d0*HPL2(1,0)*u**2*omv**(-2)*pi*ci
     .        + 1.0d0/8.0d0*HPL2(1,0)*u**2*omv**(-1)
     .        - 23.0d0/16.0d0*HPL2(1,0)*u**2
     .        + 5.0d0/4.0d0*HPL2(1,0)*u**2*v
     .        + 5.0d0/8.0d0*HPL2(1,0)*u**3
     .        + 3.0d0/4.0d0*HPL2(1,0)*GPL1(0)
         col3 = col3 + 1.0d0/2.0d0*HPL2(1,0)*GPL1(0)*pi*ci
     .        - 1.0d0/2.0d0*HPL2(1,0)*GPL1(0)*v
     .        + 1.0d0/8.0d0*HPL2(1,0)*GPL1(0)*v**2
     .        - 1.0d0/2.0d0*HPL2(1,0)*GPL1(0)*u
     .        + 1.0d0/4.0d0*HPL2(1,0)*GPL1(0)*u*v
     .        + 1.0d0/8.0d0*HPL2(1,0)*GPL1(0)*u**2
     .        - 3.0d0/8.0d0*HPL2(1,0)*GPL1(2)
     .        - 1.0d0/4.0d0*HPL2(1,0)*GPL1(2)*pi*ci
     .        + 3.0d0/8.0d0*HPL2(1,0)*GPL1(3)
     .        - 1.0d0/2.0d0*HPL2(1,0)*GPL1(3)*v
     .        + 1.0d0/8.0d0*HPL2(1,0)*GPL1(3)*v**2
     .        - 1.0d0/2.0d0*HPL2(1,0)*GPL1(3)*u
     .        + 1.0d0/4.0d0*HPL2(1,0)*GPL1(3)*u*v
     .        + 1.0d0/8.0d0*HPL2(1,0)*GPL1(3)*u**2
     .        - 1.0d0/4.0d0*HPL2(1,0)*GPL2(0,0)
         col3 = col3 + 1.0d0/4.0d0*HPL2(1,0)*GPL2(0,2)
     .        + 1.0d0/4.0d0*HPL2(1,0)*GPL2(0,3)
     .        + 1.0d0/4.0d0*HPL2(1,0)*GPL2(2,0)
     .        - 1.0d0/2.0d0*HPL2(1,0)*GPL2(2,3)
     .        + 1.0d0/2.0d0*HPL2(1,0)*GPL2(3,0)
     .        + 1.0d0/4.0d0*HPL2(1,0)*GPL2(3,2)
     .        + 1.0d0/6.0d0*HPL2(1,1)*pi**2
     .        + 3.0d0/4.0d0*HPL3(0,0,0)
     .        + 3.0d0/4.0d0*HPL3(0,0,0)*pi*ci
     .        - 1.0d0/2.0d0*HPL3(0,0,0)*u*omv**(-1)
     .        + 1.0d0/8.0d0*HPL3(0,0,0)*u**2*omv**(-2)
     .        + 3.0d0/4.0d0*HPL3(0,0,0)*GPL1(0)
     .        - 1.0d0/4.0d0*HPL3(0,0,0)*GPL1(2)
     .        - 3.0d0/4.0d0*HPL3(0,0,0)*GPL1(3)
     .        + 3.0d0/8.0d0*HPL3(0,0,1)
         col3 = col3 + 1.0d0/4.0d0*HPL3(0,0,1)*pi*ci
     .        - 1.0d0/2.0d0*HPL3(0,0,1)*v
     .        + 1.0d0/8.0d0*HPL3(0,0,1)*v**2
     .        - 1.0d0/2.0d0*HPL3(0,0,1)*u
     .        + 1.0d0/4.0d0*HPL3(0,0,1)*u*v
     .        + 1.0d0/8.0d0*HPL3(0,0,1)*u**2
     .        + 1.0d0/4.0d0*HPL3(0,0,1)*GPL1(0)
     .        - 1.0d0/2.0d0*HPL3(0,0,1)*GPL1(2)
     .        + 3.0d0/8.0d0*HPL3(0,1,0)
     .        - 1.0d0/2.0d0*HPL3(0,1,0)*u*omv**(-1)
     .        + 1.0d0/8.0d0*HPL3(0,1,0)*u**2*omv**(-2)
     .        + 1.0d0/2.0d0*HPL3(0,1,0)*GPL1(0)
     .        - 1.0d0/2.0d0*HPL3(0,1,0)*GPL1(2)
     .        - 1.0d0/4.0d0*HPL3(0,1,0)*GPL1(3)
     .        - 3.0d0/4.0d0*HPL3(1,0,0)
         col3 = col3 - 3.0d0/4.0d0*HPL3(1,0,0)*pi*ci
     .        + 1.0d0/2.0d0*HPL3(1,0,0)*u*omv**(-1)
     .        - 1.0d0/8.0d0*HPL3(1,0,0)*u**2*omv**(-2)
     .        - 1.0d0/4.0d0*HPL3(1,0,0)*GPL1(0)
     .        - 1.0d0/4.0d0*HPL3(1,0,0)*GPL1(3)
     .        - 3.0d0/4.0d0*HPL3(1,0,1)*pi*ci
     .        - 1.0d0/2.0d0*HPL3(1,0,1)*v
     .        + 1.0d0/8.0d0*HPL3(1,0,1)*v**2
     .        + 1.0d0/2.0d0*HPL3(1,0,1)*u*omv**(-1)
     .        - 1.0d0/2.0d0*HPL3(1,0,1)*u
     .        + 1.0d0/4.0d0*HPL3(1,0,1)*u*v
     .        - 1.0d0/8.0d0*HPL3(1,0,1)*u**2*omv**(-2)
     .        + 1.0d0/8.0d0*HPL3(1,0,1)*u**2
     .        - 3.0d0/4.0d0*HPL3(1,0,1)*GPL1(0)
     .        + 1.0d0/2.0d0*HPL3(1,0,1)*GPL1(2)
         col3 = col3 + 1.0d0/2.0d0*HPL3(1,0,1)*GPL1(3)
     .        + 3.0d0/8.0d0*HPL3(1,1,0)
     .        + 3.0d0/4.0d0*HPL3(1,1,0)*pi*ci
     .        - 1.0d0/4.0d0*HPL3(1,1,0)*GPL1(0)
     .        + 1.0d0/4.0d0*HPL3(1,1,0)*GPL1(2)
     .        - 1.0d0/4.0d0*HPL3(1,1,0)*GPL1(3)
     .        - 3.0d0/4.0d0*HPL4(0,0,0,0)
     .        - 1.0d0/4.0d0*HPL4(0,0,1,0)
     .        - 1.0d0/4.0d0*HPL4(0,1,0,0)
     .        + 1.0d0/2.0d0*HPL4(0,1,0,1)
     .        - 1.0d0/4.0d0*HPL4(0,1,1,0)
     .        + 1.0d0/2.0d0*HPL4(1,0,0,0)
     .        + 1.0d0/2.0d0*HPL4(1,0,0,1)
     .        + 3.0d0/4.0d0*HPL4(1,0,1,0)
     .        - 1.0d0/4.0d0*HPL4(1,1,0,0)
         col3 = col3 - HPL4(1,1,0,1)
     .        + 9.0d0/8.0d0*GPL1(0)
     .        - 17.0d0/16.0d0*GPL1(0)*pi*ci
     .        - 1.0d0/8.0d0*GPL1(0)*pi**2
     .        + 1.0d0/6.0d0*GPL1(0)*pi**3*ci
     .        - 1.0d0/4.0d0*GPL1(0)*zeta3
     .        - 17.0d0/16.0d0*GPL1(0)*v*upv**(-1)
     .        - 1.0d0/16.0d0*GPL1(0)*v
     .        + GPL1(0)*v*pi*ci
     .        + 1.0d0/3.0d0*GPL1(0)*v*pi**2
     .        + 1.0d0/16.0d0*GPL1(0)*v**2*pi*ci
     .        - 1.0d0/12.0d0*GPL1(0)*v**2*pi**2
     .        - 1.0d0/12.0d0*GPL1(0)*u*omv**(-1)*pi**2
     .        - 11.0d0/8.0d0*GPL1(0)*u
     .        + 7.0d0/4.0d0*GPL1(0)*u*pi*ci
         col3 = col3 + 1.0d0/3.0d0*GPL1(0)*u*pi**2
     .        + 5.0d0/8.0d0*GPL1(0)*u*v
     .        - 11.0d0/8.0d0*GPL1(0)*u*v*pi*ci
     .        - 1.0d0/6.0d0*GPL1(0)*u*v*pi**2
     .        + 5.0d0/8.0d0*GPL1(0)*u*v**2*pi*ci
     .        + 1.0d0/48.0d0*GPL1(0)*u**2*omv**(-2)*pi**2
     .        + 1.0d0/8.0d0*GPL1(0)*u**2*omv**(-1)*pi*ci
     .        + 5.0d0/8.0d0*GPL1(0)*u**2
     .        - 23.0d0/16.0d0*GPL1(0)*u**2*pi*ci
     .        - 1.0d0/12.0d0*GPL1(0)*u**2*pi**2
     .        + 5.0d0/4.0d0*GPL1(0)*u**2*v*pi*ci
     .        + 5.0d0/8.0d0*GPL1(0)*u**3*pi*ci
     .        - 5.0d0/16.0d0*GPL1(1)*u**(-2)*pi*ci
     .        + 5.0d0/8.0d0*GPL1(1)*u**(-2)*v*pi*ci
     .        - 5.0d0/16.0d0*GPL1(1)*u**(-2)*v**2*pi*ci
         col3 = col3 - 1.0d0/2.0d0*GPL1(1)*u**(-1)*pi*ci
     .        + 5.0d0/8.0d0*GPL1(1)*u**(-1)*v*pi*ci
     .        - 1.0d0/8.0d0*GPL1(1)*u**(-1)*v**2*pi*ci
     .        + 2.0d0*GPL1(1)*pi*ci
     .        - 11.0d0/8.0d0*GPL1(1)*v*pi*ci
     .        - 1.0d0/16.0d0*GPL1(1)*v**2*pi*ci
     .        + 1.0d0/8.0d0*GPL1(1)*u*omv**(-1)*pi*ci
     .        - 2.0d0*GPL1(1)*u*pi*ci
     .        + 11.0d0/8.0d0*GPL1(1)*u*v*pi*ci
     .        - 5.0d0/8.0d0*GPL1(1)*u*v**2*pi*ci
     .        - 1.0d0/8.0d0*GPL1(1)*u**2*omv**(-1)*pi*ci
     .        + 23.0d0/16.0d0*GPL1(1)*u**2*pi*ci
     .        - 5.0d0/4.0d0*GPL1(1)*u**2*v*pi*ci
     .        - 5.0d0/8.0d0*GPL1(1)*u**3*pi*ci
     .        - 17.0d0/32.0d0*GPL1(2)
         col3 = col3 - 5.0d0/16.0d0*GPL1(2)*u**(-1)
     .        + 5.0d0/8.0d0*GPL1(2)*u**(-1)*v
     .        - 5.0d0/16.0d0*GPL1(2)*u**(-1)*v**2
     .        - 15.0d0/16.0d0*GPL1(2)*pi*ci
     .        - 1.0d0/8.0d0*GPL1(2)*pi**2
     .        - 1.0d0/8.0d0*GPL1(2)*pi**3*ci
     .        - 5.0d0/4.0d0*GPL1(2)*zeta3
     .        + 17.0d0/16.0d0*GPL1(2)*v
     .        + 7.0d0/8.0d0*GPL1(2)*v*pi*ci
     .        + 1.0d0/6.0d0*GPL1(2)*v*pi**2
     .        - 5.0d0/8.0d0*GPL1(2)*v**2
     .        + 1.0d0/16.0d0*GPL1(2)*v**2*pi*ci
     .        - 1.0d0/24.0d0*GPL1(2)*v**2*pi**2
     .        + 1.0d0/12.0d0*GPL1(2)*u*omv**(-1)*pi**2
     .        + 11.0d0/8.0d0*GPL1(2)*u
         col3 = col3 + 13.0d0/8.0d0*GPL1(2)*u*pi*ci
     .        + 1.0d0/6.0d0*GPL1(2)*u*pi**2
     .        - 5.0d0/4.0d0*GPL1(2)*u*v
     .        - 11.0d0/8.0d0*GPL1(2)*u*v*pi*ci
     .        - 1.0d0/12.0d0*GPL1(2)*u*v*pi**2
     .        + 5.0d0/8.0d0*GPL1(2)*u*v**2*pi*ci
     .        - 1.0d0/48.0d0*GPL1(2)*u**2*omv**(-2)*pi**2
     .        + 1.0d0/8.0d0*GPL1(2)*u**2*omv**(-1)*pi*ci
     .        - 5.0d0/8.0d0*GPL1(2)*u**2
     .        - 23.0d0/16.0d0*GPL1(2)*u**2*pi*ci
     .        - 1.0d0/24.0d0*GPL1(2)*u**2*pi**2
     .        + 5.0d0/4.0d0*GPL1(2)*u**2*v*pi*ci
     .        + 5.0d0/8.0d0*GPL1(2)*u**3*pi*ci
     .        + 3.0d0/8.0d0*GPL1(3)*omumv**(-1)*pi*ci
     .        + 23.0d0/16.0d0*GPL1(3)*pi*ci
         col3 = col3 - 1.0d0/4.0d0*GPL1(3)*pi**2
     .        - 1.0d0/12.0d0*GPL1(3)*pi**3*ci
     .        + 1.0d0/2.0d0*GPL1(3)*zeta3
     .        - 3.0d0/8.0d0*GPL1(3)*v*omumv**(-1)*pi*ci
     .        - 7.0d0/8.0d0*GPL1(3)*v*pi*ci
     .        + 1.0d0/6.0d0*GPL1(3)*v*pi**2
     .        - 1.0d0/16.0d0*GPL1(3)*v**2*pi*ci
     .        - 1.0d0/24.0d0*GPL1(3)*v**2*pi**2
     .        - 1.0d0/8.0d0*GPL1(3)*u*omv**(-1)*pi*ci
     .        - 3.0d0/2.0d0*GPL1(3)*u*pi*ci
     .        + 1.0d0/6.0d0*GPL1(3)*u*pi**2
     .        + 11.0d0/8.0d0*GPL1(3)*u*v*pi*ci
     .        - 1.0d0/12.0d0*GPL1(3)*u*v*pi**2
     .        - 5.0d0/8.0d0*GPL1(3)*u*v**2*pi*ci
     .        - 1.0d0/8.0d0*GPL1(3)*u**2*omv**(-1)*pi*ci
         col3 = col3 + 23.0d0/16.0d0*GPL1(3)*u**2*pi*ci
     .        - 1.0d0/24.0d0*GPL1(3)*u**2*pi**2
     .        - 5.0d0/4.0d0*GPL1(3)*u**2*v*pi*ci
     .        - 5.0d0/8.0d0*GPL1(3)*u**3*pi*ci
     .        - 1.0d0/8.0d0*GPL2(0,0)
     .        + 3.0d0/8.0d0*GPL2(0,0)*pi*ci
     .        + 1.0d0/24.0d0*GPL2(0,0)*pi**2
     .        + 1.0d0/8.0d0*GPL2(0,0)*v
     .        - 1.0d0/2.0d0*GPL2(0,0)*v*pi*ci
     .        + 1.0d0/8.0d0*GPL2(0,0)*v**2*pi*ci
     .        + 1.0d0/8.0d0*GPL2(0,0)*u
     .        - 1.0d0/2.0d0*GPL2(0,0)*u*pi*ci
     .        + 1.0d0/4.0d0*GPL2(0,0)*u*v*pi*ci
     .        + 1.0d0/8.0d0*GPL2(0,0)*u**2*pi*ci
     .        - 3.0d0/4.0d0*GPL2(0,1)*pi*ci
         col3 = col3 + 1.0d0/2.0d0*GPL2(0,1)*v*pi*ci
     .        - 1.0d0/8.0d0*GPL2(0,1)*v**2*pi*ci
     .        + 1.0d0/2.0d0*GPL2(0,1)*u*omv**(-1)*pi*ci
     .        + 1.0d0/2.0d0*GPL2(0,1)*u*pi*ci
     .        - 1.0d0/4.0d0*GPL2(0,1)*u*v*pi*ci
     .        - 1.0d0/8.0d0*GPL2(0,1)*u**2*omv**(-2)*pi*ci
     .        - 1.0d0/8.0d0*GPL2(0,1)*u**2*pi*ci
     .        - 15.0d0/16.0d0*GPL2(0,2)
     .        + 3.0d0/8.0d0*GPL2(0,2)*pi*ci
     .        - 1.0d0/24.0d0*GPL2(0,2)*pi**2
     .        + 7.0d0/8.0d0*GPL2(0,2)*v
     .        - 1.0d0/2.0d0*GPL2(0,2)*v*pi*ci
     .        + 1.0d0/16.0d0*GPL2(0,2)*v**2
     .        + 1.0d0/8.0d0*GPL2(0,2)*v**2*pi*ci
     .        + 13.0d0/8.0d0*GPL2(0,2)*u
         col3 = col3 - 1.0d0/2.0d0*GPL2(0,2)*u*pi*ci
     .        - 11.0d0/8.0d0*GPL2(0,2)*u*v
     .        + 1.0d0/4.0d0*GPL2(0,2)*u*v*pi*ci
     .        + 5.0d0/8.0d0*GPL2(0,2)*u*v**2
     .        + 1.0d0/8.0d0*GPL2(0,2)*u**2*omv**(-1)
     .        - 23.0d0/16.0d0*GPL2(0,2)*u**2
     .        + 1.0d0/8.0d0*GPL2(0,2)*u**2*pi*ci
     .        + 5.0d0/4.0d0*GPL2(0,2)*u**2*v
     .        + 5.0d0/8.0d0*GPL2(0,2)*u**3
     .        + 3.0d0/8.0d0*GPL2(0,3)*pi*ci
     .        - 1.0d0/6.0d0*GPL2(0,3)*pi**2
     .        + 1.0d0/2.0d0*GPL2(0,3)*v*pi*ci
     .        - 1.0d0/8.0d0*GPL2(0,3)*v**2*pi*ci
     .        - 1.0d0/2.0d0*GPL2(0,3)*u*omv**(-1)*pi*ci
     .        + 1.0d0/2.0d0*GPL2(0,3)*u*pi*ci
         col3 = col3 - 1.0d0/4.0d0*GPL2(0,3)*u*v*pi*ci
     .        + 1.0d0/8.0d0*GPL2(0,3)*u**2*omv**(-2)*pi*ci
     .        - 1.0d0/8.0d0*GPL2(0,3)*u**2*pi*ci
     .        + 2.0d0*GPL2(1,2)
     .        - 5.0d0/16.0d0*GPL2(1,2)*u**(-2)
     .        + 5.0d0/8.0d0*GPL2(1,2)*u**(-2)*v
     .        - 5.0d0/16.0d0*GPL2(1,2)*u**(-2)*v**2
     .        - 1.0d0/2.0d0*GPL2(1,2)*u**(-1)
     .        + 5.0d0/8.0d0*GPL2(1,2)*u**(-1)*v
     .        - 1.0d0/8.0d0*GPL2(1,2)*u**(-1)*v**2
     .        - 11.0d0/8.0d0*GPL2(1,2)*v
     .        - 1.0d0/16.0d0*GPL2(1,2)*v**2
     .        + 1.0d0/8.0d0*GPL2(1,2)*u*omv**(-1)
     .        - 2.0d0*GPL2(1,2)*u
     .        + 11.0d0/8.0d0*GPL2(1,2)*u*v
         col3 = col3 - 5.0d0/8.0d0*GPL2(1,2)*u*v**2
     .        - 1.0d0/8.0d0*GPL2(1,2)*u**2*omv**(-1)
     .        + 23.0d0/16.0d0*GPL2(1,2)*u**2
     .        - 5.0d0/4.0d0*GPL2(1,2)*u**2*v
     .        - 5.0d0/8.0d0*GPL2(1,2)*u**3
     .        - 15.0d0/16.0d0*GPL2(2,0)
     .        + 3.0d0/8.0d0*GPL2(2,0)*pi*ci
     .        - 1.0d0/24.0d0*GPL2(2,0)*pi**2
     .        + 7.0d0/8.0d0*GPL2(2,0)*v
     .        - 1.0d0/2.0d0*GPL2(2,0)*v*pi*ci
     .        + 1.0d0/16.0d0*GPL2(2,0)*v**2
     .        + 1.0d0/8.0d0*GPL2(2,0)*v**2*pi*ci
     .        + 13.0d0/8.0d0*GPL2(2,0)*u
     .        - 1.0d0/2.0d0*GPL2(2,0)*u*pi*ci
     .        - 11.0d0/8.0d0*GPL2(2,0)*u*v
         col3 = col3 + 1.0d0/4.0d0*GPL2(2,0)*u*v*pi*ci
     .        + 5.0d0/8.0d0*GPL2(2,0)*u*v**2
     .        + 1.0d0/8.0d0*GPL2(2,0)*u**2*omv**(-1)
     .        - 23.0d0/16.0d0*GPL2(2,0)*u**2
     .        + 1.0d0/8.0d0*GPL2(2,0)*u**2*pi*ci
     .        + 5.0d0/4.0d0*GPL2(2,0)*u**2*v
     .        + 5.0d0/8.0d0*GPL2(2,0)*u**3
     .        + 1.0d0/2.0d0*GPL2(2,1)*v*pi*ci
     .        - 1.0d0/8.0d0*GPL2(2,1)*v**2*pi*ci
     .        - 1.0d0/2.0d0*GPL2(2,1)*u*omv**(-1)*pi*ci
     .        + 1.0d0/2.0d0*GPL2(2,1)*u*pi*ci
     .        - 1.0d0/4.0d0*GPL2(2,1)*u*v*pi*ci
     .        + 1.0d0/8.0d0*GPL2(2,1)*u**2*omv**(-2)*pi*ci
     .        - 1.0d0/8.0d0*GPL2(2,1)*u**2*pi*ci
     .        - 3.0d0/4.0d0*GPL2(2,3)*pi*ci
         col3 = col3 + 1.0d0/3.0d0*GPL2(2,3)*pi**2
     .        + 1.0d0/2.0d0*GPL2(2,3)*u*omv**(-1)*pi*ci
     .        - 1.0d0/8.0d0*GPL2(2,3)*u**2*omv**(-2)*pi*ci
     .        + 23.0d0/16.0d0*GPL2(3,0)
     .        + 3.0d0/8.0d0*GPL2(3,0)*omumv**(-1)
     .        + 3.0d0/8.0d0*GPL2(3,0)*pi*ci
     .        - 1.0d0/3.0d0*GPL2(3,0)*pi**2
     .        - 3.0d0/8.0d0*GPL2(3,0)*v*omumv**(-1)
     .        - 7.0d0/8.0d0*GPL2(3,0)*v
     .        - 1.0d0/16.0d0*GPL2(3,0)*v**2
     .        - 1.0d0/8.0d0*GPL2(3,0)*u*omv**(-1)
     .        - 3.0d0/2.0d0*GPL2(3,0)*u
     .        + 11.0d0/8.0d0*GPL2(3,0)*u*v
     .        - 5.0d0/8.0d0*GPL2(3,0)*u*v**2
     .        - 1.0d0/8.0d0*GPL2(3,0)*u**2*omv**(-1)
         col3 = col3 + 23.0d0/16.0d0*GPL2(3,0)*u**2
     .        - 5.0d0/4.0d0*GPL2(3,0)*u**2*v
     .        - 5.0d0/8.0d0*GPL2(3,0)*u**3
     .        + 3.0d0/8.0d0*GPL2(3,2)*pi*ci
     .        - 1.0d0/24.0d0*GPL2(3,2)*pi**2
     .        - 1.0d0/2.0d0*GPL2(3,2)*v*pi*ci
     .        + 1.0d0/8.0d0*GPL2(3,2)*v**2*pi*ci
     .        - 1.0d0/2.0d0*GPL2(3,2)*u*pi*ci
     .        + 1.0d0/4.0d0*GPL2(3,2)*u*v*pi*ci
     .        + 1.0d0/8.0d0*GPL2(3,2)*u**2*pi*ci
     .        - 3.0d0/4.0d0*GPL2(3,3)*pi*ci
     .        + 1.0d0/6.0d0*GPL2(3,3)*pi**2
     .        + 1.0d0/2.0d0*GPL2(3,3)*v*pi*ci
     .        - 1.0d0/8.0d0*GPL2(3,3)*v**2*pi*ci
     .        + 1.0d0/2.0d0*GPL2(3,3)*u*pi*ci
         col3 = col3 - 1.0d0/4.0d0*GPL2(3,3)*u*v*pi*ci
     .        - 1.0d0/8.0d0*GPL2(3,3)*u**2*pi*ci
     .        - 1.0d0/2.0d0*GPL3(0,0,1)*pi*ci
     .        + 3.0d0/8.0d0*GPL3(0,0,2)
     .        - 1.0d0/2.0d0*GPL3(0,0,2)*v
     .        + 1.0d0/8.0d0*GPL3(0,0,2)*v**2
     .        - 1.0d0/2.0d0*GPL3(0,0,2)*u
     .        + 1.0d0/4.0d0*GPL3(0,0,2)*u*v
     .        + 1.0d0/8.0d0*GPL3(0,0,2)*u**2
     .        + 1.0d0/2.0d0*GPL3(0,0,3)*pi*ci
     .        - 3.0d0/4.0d0*GPL3(0,1,2)
     .        + 1.0d0/2.0d0*GPL3(0,1,2)*v
     .        - 1.0d0/8.0d0*GPL3(0,1,2)*v**2
     .        + 1.0d0/2.0d0*GPL3(0,1,2)*u*omv**(-1)
     .        + 1.0d0/2.0d0*GPL3(0,1,2)*u
         col3 = col3 - 1.0d0/4.0d0*GPL3(0,1,2)*u*v
     .        - 1.0d0/8.0d0*GPL3(0,1,2)*u**2*omv**(-2)
     .        - 1.0d0/8.0d0*GPL3(0,1,2)*u**2
     .        + 3.0d0/8.0d0*GPL3(0,2,0)
     .        - 1.0d0/2.0d0*GPL3(0,2,0)*v
     .        + 1.0d0/8.0d0*GPL3(0,2,0)*v**2
     .        - 1.0d0/2.0d0*GPL3(0,2,0)*u
     .        + 1.0d0/4.0d0*GPL3(0,2,0)*u*v
     .        + 1.0d0/8.0d0*GPL3(0,2,0)*u**2
     .        + 1.0d0/2.0d0*GPL3(0,2,1)*pi*ci
     .        - 1.0d0/4.0d0*GPL3(0,2,3)*pi*ci
     .        + 3.0d0/8.0d0*GPL3(0,3,0)
     .        + 1.0d0/4.0d0*GPL3(0,3,0)*pi*ci
     .        + 1.0d0/2.0d0*GPL3(0,3,0)*v
     .        - 1.0d0/8.0d0*GPL3(0,3,0)*v**2
         col3 = col3 - 1.0d0/2.0d0*GPL3(0,3,0)*u*omv**(-1)
     .        + 1.0d0/2.0d0*GPL3(0,3,0)*u
     .        - 1.0d0/4.0d0*GPL3(0,3,0)*u*v
     .        + 1.0d0/8.0d0*GPL3(0,3,0)*u**2*omv**(-2)
     .        - 1.0d0/8.0d0*GPL3(0,3,0)*u**2
     .        + 1.0d0/4.0d0*GPL3(0,3,2)*pi*ci
     .        - 1.0d0/2.0d0*GPL3(0,3,3)*pi*ci
     .        + 3.0d0/8.0d0*GPL3(2,0,0)
     .        - 1.0d0/2.0d0*GPL3(2,0,0)*v
     .        + 1.0d0/8.0d0*GPL3(2,0,0)*v**2
     .        - 1.0d0/2.0d0*GPL3(2,0,0)*u
     .        + 1.0d0/4.0d0*GPL3(2,0,0)*u*v
     .        + 1.0d0/8.0d0*GPL3(2,0,0)*u**2
     .        + 1.0d0/2.0d0*GPL3(2,1,2)*v
     .        - 1.0d0/8.0d0*GPL3(2,1,2)*v**2
         col3 = col3 - 1.0d0/2.0d0*GPL3(2,1,2)*u*omv**(-1)
     .        + 1.0d0/2.0d0*GPL3(2,1,2)*u
     .        - 1.0d0/4.0d0*GPL3(2,1,2)*u*v
     .        + 1.0d0/8.0d0*GPL3(2,1,2)*u**2*omv**(-2)
     .        - 1.0d0/8.0d0*GPL3(2,1,2)*u**2
     .        - 3.0d0/4.0d0*GPL3(2,3,0)
     .        - 1.0d0/4.0d0*GPL3(2,3,0)*pi*ci
     .        + 1.0d0/2.0d0*GPL3(2,3,0)*u*omv**(-1)
     .        - 1.0d0/8.0d0*GPL3(2,3,0)*u**2*omv**(-2)
     .        - 1.0d0/2.0d0*GPL3(2,3,2)*pi*ci
     .        + 1.0d0/4.0d0*GPL3(2,3,3)*pi*ci
     .        + 1.0d0/2.0d0*GPL3(3,0,0)*pi*ci
     .        + 1.0d0/2.0d0*GPL3(3,0,0)*v
     .        - 1.0d0/8.0d0*GPL3(3,0,0)*v**2
     .        + 1.0d0/2.0d0*GPL3(3,0,0)*u
         col3 = col3 - 1.0d0/4.0d0*GPL3(3,0,0)*u*v
     .        - 1.0d0/8.0d0*GPL3(3,0,0)*u**2
     .        - 1.0d0/2.0d0*GPL3(3,0,1)*pi*ci
     .        + 3.0d0/8.0d0*GPL3(3,0,2)
     .        + 1.0d0/2.0d0*GPL3(3,0,2)*pi*ci
     .        - 1.0d0/2.0d0*GPL3(3,0,2)*v
     .        + 1.0d0/8.0d0*GPL3(3,0,2)*v**2
     .        - 1.0d0/2.0d0*GPL3(3,0,2)*u
     .        + 1.0d0/4.0d0*GPL3(3,0,2)*u*v
     .        + 1.0d0/8.0d0*GPL3(3,0,2)*u**2
     .        - 3.0d0/4.0d0*GPL3(3,0,3)*pi*ci
     .        + 3.0d0/8.0d0*GPL3(3,2,0)
     .        + 1.0d0/2.0d0*GPL3(3,2,0)*pi*ci
     .        - 1.0d0/2.0d0*GPL3(3,2,0)*v
     .        + 1.0d0/8.0d0*GPL3(3,2,0)*v**2
         col3 = col3 - 1.0d0/2.0d0*GPL3(3,2,0)*u
     .        + 1.0d0/4.0d0*GPL3(3,2,0)*u*v
     .        + 1.0d0/8.0d0*GPL3(3,2,0)*u**2
     .        - 1.0d0/2.0d0*GPL3(3,2,1)*pi*ci
     .        - 1.0d0/4.0d0*GPL3(3,2,3)*pi*ci
     .        - 3.0d0/4.0d0*GPL3(3,3,0)
     .        - 1.0d0/2.0d0*GPL3(3,3,0)*pi*ci
     .        + 1.0d0/2.0d0*GPL3(3,3,0)*v
     .        - 1.0d0/8.0d0*GPL3(3,3,0)*v**2
     .        + 1.0d0/2.0d0*GPL3(3,3,0)*u
     .        - 1.0d0/4.0d0*GPL3(3,3,0)*u*v
     .        - 1.0d0/8.0d0*GPL3(3,3,0)*u**2
     .        + 1.0d0/2.0d0*GPL3(3,3,3)*pi*ci
     .        - 1.0d0/2.0d0*GPL4(0,0,1,2)
     .        + 1.0d0/2.0d0*GPL4(0,0,3,0)
         col3 = col3 + 1.0d0/2.0d0*GPL4(0,2,1,2)
     .        - 1.0d0/4.0d0*GPL4(0,2,3,0)
     .        + 1.0d0/4.0d0*GPL4(0,3,0,2)
     .        + 1.0d0/4.0d0*GPL4(0,3,2,0)
     .        - 1.0d0/2.0d0*GPL4(0,3,3,0)
     .        + 1.0d0/4.0d0*GPL4(2,3,0,0)
     .        - 1.0d0/2.0d0*GPL4(2,3,0,2)
     .        - 1.0d0/2.0d0*GPL4(2,3,2,0)
     .        + 1.0d0/4.0d0*GPL4(2,3,3,0)
     .        + 1.0d0/2.0d0*GPL4(3,0,0,2)
     .        - 1.0d0/2.0d0*GPL4(3,0,1,2)
     .        + 1.0d0/2.0d0*GPL4(3,0,2,0)
     .        - 3.0d0/4.0d0*GPL4(3,0,3,0)
     .        + 1.0d0/2.0d0*GPL4(3,2,0,0)
     .        - 1.0d0/2.0d0*GPL4(3,2,1,2)
         col3 = col3 - 1.0d0/4.0d0*GPL4(3,2,3,0)
     .        - 1.0d0/2.0d0*GPL4(3,3,0,0)
     .        + 1.0d0/2.0d0*GPL4(3,3,3,0)

         col4 = + 2393.0d0/2592.0d0
     .        + 13.0d0/48.0d0*pi*ci
     .        - 37.0d0/432.0d0*pi**2
     .        - 1.0d0/16.0d0*pi**3*ci
     .        + 131.0d0/72.0d0*zeta3
     .        - 1.0d0/4.0d0*v*omu**(-1)
     .        + 1.0d0/12.0d0*v*upv**(-1)*pi*ci
     .        - 19.0d0/36.0d0*v
     .        - 3.0d0/16.0d0*v*pi*ci
     .        - 1.0d0/4.0d0*v**2*omu**(-2)*pi*ci
     .        - 19.0d0/36.0d0*u
     .        - 5.0d0/48.0d0*u*pi*ci
     .        + 34.0d0/27.0d0*HPL1(0)
     .        - 29.0d0/36.0d0*HPL1(0)*pi*ci
     .        + 31.0d0/144.0d0*HPL1(0)*pi**2
         col4 = col4 - 1.0d0/12.0d0*HPL1(0)*v*upv**(-1)
     .        - 1.0d0/8.0d0*HPL1(0)*v
     .        + 1.0d0/4.0d0*HPL1(0)*v**2*omu**(-2)
     .        - 5.0d0/24.0d0*HPL1(0)*u
     .        - 29.0d0/36.0d0*HPL1(0)*GPL1(0)
     .        - 1.0d0/6.0d0*HPL1(0)*GPL1(0)*pi*ci
     .        + 29.0d0/36.0d0*HPL1(0)*GPL1(3)
     .        + 1.0d0/8.0d0*HPL1(0)*GPL1(3)*pi*ci
     .        + 1.0d0/4.0d0*HPL1(0)*GPL2(0,0)
     .        - 5.0d0/24.0d0*HPL1(0)*GPL2(0,3)
     .        - 7.0d0/24.0d0*HPL1(0)*GPL2(3,0)
     .        - 1.0d0/6.0d0*HPL1(0)*GPL2(3,3)
     .        + 155.0d0/216.0d0*HPL1(1)
     .        - 1.0d0/144.0d0*HPL1(1)*pi**2
     .        + 1.0d0/4.0d0*HPL1(1)*v**2*omu**(-2)
         col4 = col4 + 29.0d0/36.0d0*HPL2(0,0)
     .        - 7.0d0/24.0d0*HPL2(0,0)*pi*ci
     .        - 17.0d0/24.0d0*HPL2(0,0)*GPL1(0)
     .        + 1.0d0/3.0d0*HPL2(0,0)*GPL1(3)
     .        - 29.0d0/36.0d0*HPL2(1,0)
     .        - 3.0d0/8.0d0*HPL2(1,0)*pi*ci
     .        + 1.0d0/24.0d0*HPL2(1,0)*GPL1(0)
     .        + 3.0d0/4.0d0*HPL3(0,0,0)
     .        - 5.0d0/12.0d0*HPL3(0,1,0)
     .        - 1.0d0/2.0d0*HPL3(1,0,0)
     .        - 1.0d0/6.0d0*HPL3(1,1,0)
     .        - 13.0d0/48.0d0*GPL1(0)
     .        + 1.0d0/144.0d0*GPL1(0)*pi**2
     .        + 1.0d0/12.0d0*GPL1(0)*v*upv**(-1)
     .        + 1.0d0/48.0d0*GPL1(0)*v
         col4 = col4 + 5.0d0/48.0d0*GPL1(0)*u
     .        - 155.0d0/216.0d0*GPL1(2)
     .        - 1.0d0/48.0d0*GPL1(2)*pi**2
     .        - 1.0d0/4.0d0*GPL1(2)*v**2*omu**(-2)
     .        - 29.0d0/36.0d0*GPL1(3)*pi*ci
     .        + 19.0d0/72.0d0*GPL1(3)*pi**2
     .        + 5.0d0/24.0d0*GPL2(0,3)*pi*ci
     .        - 29.0d0/36.0d0*GPL2(3,0)
     .        - 1.0d0/6.0d0*GPL2(3,0)*pi*ci
     .        + 1.0d0/6.0d0*GPL2(3,3)*pi*ci
     .        + 5.0d0/24.0d0*GPL3(0,3,0)
     .        + 1.0d0/4.0d0*GPL3(3,0,0)
     .        + 1.0d0/6.0d0*GPL3(3,3,0)

         col5 = + 8737.0d0/864.0d0
     .        - 1.0d0/4.0d0*u**(-1)*pi*ci
     .        + 1.0d0/2.0d0*u**(-1)*v*pi*ci
     .        - 1.0d0/4.0d0*u**(-1)*v**2*pi*ci
     .        + 655.0d0/432.0d0*pi*ci
     .        + 223.0d0/864.0d0*pi**2
     .        + 5.0d0/48.0d0*pi**3*ci
     .        + 61.0d0/36.0d0*zeta3
     .        + 13.0d0/12.0d0*v*omu**(-1)
     .        + 1.0d0/6.0d0*v*omu**(-1)*pi*ci
     .        - 1.0d0/12.0d0*v*upv**(-1)*pi*ci
     .        - 37.0d0/36.0d0*v
     .        + 11.0d0/48.0d0*v*pi*ci
     .        - 1.0d0/18.0d0*v*pi**2
     .        + 13.0d0/12.0d0*v**2*omu**(-2)*pi*ci
         col5 = col5 - 1.0d0/2.0d0*v**2*pi*ci
     .        - 1.0d0/9.0d0*v**2*pi**2
     .        - 37.0d0/36.0d0*u
     .        - 5.0d0/48.0d0*u*pi*ci
     .        - 5.0d0/18.0d0*u*pi**2
     .        - 1.0d0/2.0d0*u*v*pi*ci
     .        + 1.0d0/3.0d0*u*v*pi**2
     .        - 1.0d0/3.0d0*u*v**2*pi**2
     .        + 4.0d0/9.0d0*u**2*pi**2
     .        - 2.0d0/3.0d0*u**2*v*pi**2
     .        - 1.0d0/3.0d0*u**3*pi**2
     .        + 847.0d0/108.0d0*HPL1(0)
     .        - 1.0d0/4.0d0*HPL1(0)*u**(-1)*v
     .        + 1.0d0/4.0d0*HPL1(0)*u**(-1)*v**2
     .        + 271.0d0/144.0d0*HPL1(0)*pi*ci
         col5 = col5 - 1.0d0/6.0d0*HPL1(0)*v*omu**(-1)
     .        + 1.0d0/12.0d0*HPL1(0)*v*upv**(-1)
     .        - 13.0d0/24.0d0*HPL1(0)*v
     .        - 1.0d0/12.0d0*HPL1(0)*v*pi*ci
     .        - 13.0d0/12.0d0*HPL1(0)*v**2*omu**(-2)
     .        + 1.0d0/2.0d0*HPL1(0)*v**2
     .        - 1.0d0/6.0d0*HPL1(0)*v**2*pi*ci
     .        - 5.0d0/24.0d0*HPL1(0)*u
     .        - 5.0d0/12.0d0*HPL1(0)*u*pi*ci
     .        + 1.0d0/2.0d0*HPL1(0)*u*v
     .        + 1.0d0/2.0d0*HPL1(0)*u*v*pi*ci
     .        - 1.0d0/2.0d0*HPL1(0)*u*v**2*pi*ci
     .        + 2.0d0/3.0d0*HPL1(0)*u**2*pi*ci
     .        - HPL1(0)*u**2*v*pi*ci
     .        - 1.0d0/2.0d0*HPL1(0)*u**3*pi*ci
         col5 = col5 - 13.0d0/48.0d0*HPL1(0)*GPL1(0)
     .        - 1.0d0/4.0d0*HPL1(0)*GPL1(0)*pi*ci
     .        - 1.0d0/12.0d0*HPL1(0)*GPL1(0)*v
     .        - 1.0d0/6.0d0*HPL1(0)*GPL1(0)*v**2
     .        - 5.0d0/12.0d0*HPL1(0)*GPL1(0)*u
     .        + 1.0d0/2.0d0*HPL1(0)*GPL1(0)*u*v
     .        - 1.0d0/2.0d0*HPL1(0)*GPL1(0)*u*v**2
     .        + 2.0d0/3.0d0*HPL1(0)*GPL1(0)*u**2
     .        - HPL1(0)*GPL1(0)*u**2*v
     .        - 1.0d0/2.0d0*HPL1(0)*GPL1(0)*u**3
     .        - 29.0d0/18.0d0*HPL1(0)*GPL1(1)
     .        + 1.0d0/4.0d0*HPL1(0)*GPL1(1)*u**(-2)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL1(1)*u**(-2)*v
     .        + 1.0d0/4.0d0*HPL1(0)*GPL1(1)*u**(-2)*v**2
     .        + 1.0d0/12.0d0*HPL1(0)*GPL1(1)*pi*ci
         col5 = col5 + 1.0d0/12.0d0*HPL1(0)*GPL1(1)*v
     .        + 1.0d0/6.0d0*HPL1(0)*GPL1(1)*v**2
     .        + 5.0d0/12.0d0*HPL1(0)*GPL1(1)*u
     .        - 1.0d0/2.0d0*HPL1(0)*GPL1(1)*u*v
     .        + 1.0d0/2.0d0*HPL1(0)*GPL1(1)*u*v**2
     .        - 2.0d0/3.0d0*HPL1(0)*GPL1(1)*u**2
     .        + HPL1(0)*GPL1(1)*u**2*v
     .        + 1.0d0/2.0d0*HPL1(0)*GPL1(1)*u**3
     .        - 59.0d0/72.0d0*HPL1(0)*GPL1(2)
     .        - 1.0d0/4.0d0*HPL1(0)*GPL1(2)*u**(-2)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL1(2)*u**(-2)*v
     .        - 1.0d0/4.0d0*HPL1(0)*GPL1(2)*u**(-2)*v**2
     .        - 1.0d0/12.0d0*HPL1(0)*GPL1(2)*v
     .        - 1.0d0/6.0d0*HPL1(0)*GPL1(2)*v**2
     .        - 5.0d0/12.0d0*HPL1(0)*GPL1(2)*u
         col5 = col5 + 1.0d0/2.0d0*HPL1(0)*GPL1(2)*u*v
     .        - 1.0d0/2.0d0*HPL1(0)*GPL1(2)*u*v**2
     .        + 2.0d0/3.0d0*HPL1(0)*GPL1(2)*u**2
     .        - HPL1(0)*GPL1(2)*u**2*v
     .        - 1.0d0/2.0d0*HPL1(0)*GPL1(2)*u**3
     .        - 29.0d0/36.0d0*HPL1(0)*GPL1(3)
     .        - 1.0d0/8.0d0*HPL1(0)*GPL1(3)*pi*ci
     .        + 1.0d0/12.0d0*HPL1(0)*GPL1(3)*v
     .        + 1.0d0/6.0d0*HPL1(0)*GPL1(3)*v**2
     .        + 5.0d0/12.0d0*HPL1(0)*GPL1(3)*u
     .        - 1.0d0/2.0d0*HPL1(0)*GPL1(3)*u*v
     .        + 1.0d0/2.0d0*HPL1(0)*GPL1(3)*u*v**2
     .        - 2.0d0/3.0d0*HPL1(0)*GPL1(3)*u**2
     .        + HPL1(0)*GPL1(3)*u**2*v
     .        + 1.0d0/2.0d0*HPL1(0)*GPL1(3)*u**3
         col5 = col5 - 1.0d0/4.0d0*HPL1(0)*GPL2(0,0)
     .        + 1.0d0/12.0d0*HPL1(0)*GPL2(0,1)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(0,2)
     .        + 5.0d0/24.0d0*HPL1(0)*GPL2(0,3)
     .        + 1.0d0/12.0d0*HPL1(0)*GPL2(1,0)
     .        - 1.0d0/3.0d0*HPL1(0)*GPL2(1,1)
     .        + 7.0d0/6.0d0*HPL1(0)*GPL2(1,2)
     .        - 5.0d0/6.0d0*HPL1(0)*GPL2(2,0)
     .        + HPL1(0)*GPL2(2,1)
     .        - HPL1(0)*GPL2(2,2)
     .        + 1.0d0/3.0d0*HPL1(0)*GPL2(2,3)
     .        + 7.0d0/24.0d0*HPL1(0)*GPL2(3,0)
     .        + 1.0d0/6.0d0*HPL1(0)*GPL2(3,3)
     .        + 1093.0d0/216.0d0*HPL1(1)
     .        + 1.0d0/4.0d0*HPL1(1)*u**(-1)
         col5 = col5 - 1.0d0/2.0d0*HPL1(1)*u**(-1)*v
     .        + 1.0d0/4.0d0*HPL1(1)*u**(-1)*v**2
     .        + 169.0d0/144.0d0*HPL1(1)*pi*ci
     .        + 1.0d0/72.0d0*HPL1(1)*pi**2
     .        - 1.0d0/6.0d0*HPL1(1)*v*omu**(-1)
     .        - 1.0d0/4.0d0*HPL1(1)*v
     .        - 1.0d0/12.0d0*HPL1(1)*v*pi*ci
     .        - 13.0d0/12.0d0*HPL1(1)*v**2*omu**(-2)
     .        + 1.0d0/2.0d0*HPL1(1)*v**2
     .        - 1.0d0/6.0d0*HPL1(1)*v**2*pi*ci
     .        - 1.0d0/2.0d0*HPL1(1)*u
     .        - 5.0d0/12.0d0*HPL1(1)*u*pi*ci
     .        + HPL1(1)*u*v
     .        + 1.0d0/2.0d0*HPL1(1)*u*v*pi*ci
     .        - 1.0d0/2.0d0*HPL1(1)*u*v**2*pi*ci
         col5 = col5 + 1.0d0/2.0d0*HPL1(1)*u**2
     .        + 2.0d0/3.0d0*HPL1(1)*u**2*pi*ci
     .        - HPL1(1)*u**2*v*pi*ci
     .        - 1.0d0/2.0d0*HPL1(1)*u**3*pi*ci
     .        + 7.0d0/16.0d0*HPL1(1)*GPL1(0)
     .        - 1.0d0/3.0d0*HPL1(1)*GPL1(0)*pi*ci
     .        - 1.0d0/12.0d0*HPL1(1)*GPL1(0)*v
     .        - 1.0d0/6.0d0*HPL1(1)*GPL1(0)*v**2
     .        - 5.0d0/12.0d0*HPL1(1)*GPL1(0)*u
     .        + 1.0d0/2.0d0*HPL1(1)*GPL1(0)*u*v
     .        - 1.0d0/2.0d0*HPL1(1)*GPL1(0)*u*v**2
     .        + 2.0d0/3.0d0*HPL1(1)*GPL1(0)*u**2
     .        - HPL1(1)*GPL1(0)*u**2*v
     .        - 1.0d0/2.0d0*HPL1(1)*GPL1(0)*u**3
     .        - 29.0d0/18.0d0*HPL1(1)*GPL1(1)
         col5 = col5 + 1.0d0/4.0d0*HPL1(1)*GPL1(1)*u**(-2)
     .        - 1.0d0/2.0d0*HPL1(1)*GPL1(1)*u**(-2)*v
     .        + 1.0d0/4.0d0*HPL1(1)*GPL1(1)*u**(-2)*v**2
     .        + 1.0d0/4.0d0*HPL1(1)*GPL1(1)*pi*ci
     .        + 1.0d0/12.0d0*HPL1(1)*GPL1(1)*v
     .        + 1.0d0/6.0d0*HPL1(1)*GPL1(1)*v**2
     .        + 5.0d0/12.0d0*HPL1(1)*GPL1(1)*u
     .        - 1.0d0/2.0d0*HPL1(1)*GPL1(1)*u*v
     .        + 1.0d0/2.0d0*HPL1(1)*GPL1(1)*u*v**2
     .        - 2.0d0/3.0d0*HPL1(1)*GPL1(1)*u**2
     .        + HPL1(1)*GPL1(1)*u**2*v
     .        + 1.0d0/2.0d0*HPL1(1)*GPL1(1)*u**3
     .        - 61.0d0/36.0d0*HPL1(1)*GPL1(2)
     .        - 1.0d0/2.0d0*HPL1(1)*GPL1(2)*pi*ci
     .        - 1.0d0/4.0d0*HPL1(1)*GPL2(0,0)
         col5 = col5 + 1.0d0/12.0d0*HPL1(1)*GPL2(0,1)
     .        - 1.0d0/2.0d0*HPL1(1)*GPL2(0,2)
     .        + 1.0d0/12.0d0*HPL1(1)*GPL2(1,0)
     .        - 1.0d0/3.0d0*HPL1(1)*GPL2(1,1)
     .        + HPL1(1)*GPL2(1,2)
     .        - 1.0d0/2.0d0*HPL1(1)*GPL2(2,0)
     .        + HPL1(1)*GPL2(2,1)
     .        + 34.0d0/9.0d0*HPL2(0,0)
     .        + 7.0d0/24.0d0*HPL2(0,0)*pi*ci
     .        + 1.0d0/12.0d0*HPL2(0,0)*v
     .        + 1.0d0/6.0d0*HPL2(0,0)*v**2
     .        + 5.0d0/12.0d0*HPL2(0,0)*u
     .        - 1.0d0/2.0d0*HPL2(0,0)*u*v
     .        + 1.0d0/2.0d0*HPL2(0,0)*u*v**2
     .        - 2.0d0/3.0d0*HPL2(0,0)*u**2
         col5 = col5 + HPL2(0,0)*u**2*v
     .        + 1.0d0/2.0d0*HPL2(0,0)*u**3
     .        + 17.0d0/24.0d0*HPL2(0,0)*GPL1(0)
     .        - HPL2(0,0)*GPL1(1)
     .        + 4.0d0/3.0d0*HPL2(0,0)*GPL1(2)
     .        - 1.0d0/3.0d0*HPL2(0,0)*GPL1(3)
     .        + 175.0d0/72.0d0*HPL2(0,1)
     .        + 5.0d0/12.0d0*HPL2(0,1)*pi*ci
     .        + 5.0d0/12.0d0*HPL2(0,1)*GPL1(0)
     .        - 5.0d0/6.0d0*HPL2(0,1)*GPL1(1)
     .        + 13.0d0/8.0d0*HPL2(1,0)
     .        + 3.0d0/8.0d0*HPL2(1,0)*pi*ci
     .        + 1.0d0/12.0d0*HPL2(1,0)*v
     .        + 1.0d0/6.0d0*HPL2(1,0)*v**2
     .        + 5.0d0/12.0d0*HPL2(1,0)*u
         col5 = col5 - 1.0d0/2.0d0*HPL2(1,0)*u*v
     .        + 1.0d0/2.0d0*HPL2(1,0)*u*v**2
     .        - 2.0d0/3.0d0*HPL2(1,0)*u**2
     .        + HPL2(1,0)*u**2*v
     .        + 1.0d0/2.0d0*HPL2(1,0)*u**3
     .        + 19.0d0/24.0d0*HPL2(1,0)*GPL1(0)
     .        - 7.0d0/6.0d0*HPL2(1,0)*GPL1(1)
     .        + 2.0d0/3.0d0*HPL2(1,0)*GPL1(2)
     .        + 61.0d0/36.0d0*HPL2(1,1)
     .        + 1.0d0/2.0d0*HPL2(1,1)*pi*ci
     .        + 1.0d0/2.0d0*HPL2(1,1)*GPL1(0)
     .        - HPL2(1,1)*GPL1(1)
     .        - 3.0d0/4.0d0*HPL3(0,0,0)
     .        - 5.0d0/12.0d0*HPL3(0,1,0)
     .        - 5.0d0/6.0d0*HPL3(1,0,0)
         col5 = col5 - 1.0d0/2.0d0*HPL3(1,1,0)
     .        - 547.0d0/432.0d0*GPL1(0)
     .        - 17.0d0/18.0d0*GPL1(0)*pi*ci
     .        + 1.0d0/48.0d0*GPL1(0)*pi**2
     .        - 1.0d0/12.0d0*GPL1(0)*v*upv**(-1)
     .        + 3.0d0/16.0d0*GPL1(0)*v
     .        + 1.0d0/12.0d0*GPL1(0)*v*pi*ci
     .        + 1.0d0/6.0d0*GPL1(0)*v**2*pi*ci
     .        - 19.0d0/48.0d0*GPL1(0)*u
     .        + 5.0d0/12.0d0*GPL1(0)*u*pi*ci
     .        + 1.0d0/2.0d0*GPL1(0)*u*v
     .        - 1.0d0/2.0d0*GPL1(0)*u*v*pi*ci
     .        + 1.0d0/2.0d0*GPL1(0)*u*v**2*pi*ci
     .        + 1.0d0/2.0d0*GPL1(0)*u**2
     .        - 2.0d0/3.0d0*GPL1(0)*u**2*pi*ci
         col5 = col5 + GPL1(0)*u**2*v*pi*ci
     .        + 1.0d0/2.0d0*GPL1(0)*u**3*pi*ci
     .        - 1.0d0/4.0d0*GPL1(1)*u**(-2)*pi*ci
     .        + 1.0d0/2.0d0*GPL1(1)*u**(-2)*v*pi*ci
     .        - 1.0d0/4.0d0*GPL1(1)*u**(-2)*v**2*pi*ci
     .        + 29.0d0/18.0d0*GPL1(1)*pi*ci
     .        - 13.0d0/36.0d0*GPL1(1)*pi**2
     .        - 1.0d0/12.0d0*GPL1(1)*v*pi*ci
     .        - 1.0d0/6.0d0*GPL1(1)*v**2*pi*ci
     .        - 5.0d0/12.0d0*GPL1(1)*u*pi*ci
     .        + 1.0d0/2.0d0*GPL1(1)*u*v*pi*ci
     .        - 1.0d0/2.0d0*GPL1(1)*u*v**2*pi*ci
     .        + 2.0d0/3.0d0*GPL1(1)*u**2*pi*ci
     .        - GPL1(1)*u**2*v*pi*ci
     .        - 1.0d0/2.0d0*GPL1(1)*u**3*pi*ci
         col5 = col5 - 1093.0d0/216.0d0*GPL1(2)
     .        - 1.0d0/4.0d0*GPL1(2)*u**(-1)
     .        + 1.0d0/2.0d0*GPL1(2)*u**(-1)*v
     .        - 1.0d0/4.0d0*GPL1(2)*u**(-1)*v**2
     .        - 169.0d0/144.0d0*GPL1(2)*pi*ci
     .        + 1.0d0/72.0d0*GPL1(2)*pi**2
     .        + 1.0d0/6.0d0*GPL1(2)*v*omu**(-1)
     .        + 1.0d0/4.0d0*GPL1(2)*v
     .        + 1.0d0/12.0d0*GPL1(2)*v*pi*ci
     .        + 13.0d0/12.0d0*GPL1(2)*v**2*omu**(-2)
     .        - 1.0d0/2.0d0*GPL1(2)*v**2
     .        + 1.0d0/6.0d0*GPL1(2)*v**2*pi*ci
     .        + 1.0d0/2.0d0*GPL1(2)*u
     .        + 5.0d0/12.0d0*GPL1(2)*u*pi*ci
     .        - GPL1(2)*u*v
         col5 = col5 - 1.0d0/2.0d0*GPL1(2)*u*v*pi*ci
     .        + 1.0d0/2.0d0*GPL1(2)*u*v**2*pi*ci
     .        - 1.0d0/2.0d0*GPL1(2)*u**2
     .        - 2.0d0/3.0d0*GPL1(2)*u**2*pi*ci
     .        + GPL1(2)*u**2*v*pi*ci
     .        + 1.0d0/2.0d0*GPL1(2)*u**3*pi*ci
     .        + 29.0d0/36.0d0*GPL1(3)*pi*ci
     .        - 19.0d0/72.0d0*GPL1(3)*pi**2
     .        - 1.0d0/12.0d0*GPL1(3)*v*pi*ci
     .        - 1.0d0/6.0d0*GPL1(3)*v**2*pi*ci
     .        - 5.0d0/12.0d0*GPL1(3)*u*pi*ci
     .        + 1.0d0/2.0d0*GPL1(3)*u*v*pi*ci
     .        - 1.0d0/2.0d0*GPL1(3)*u*v**2*pi*ci
     .        + 2.0d0/3.0d0*GPL1(3)*u**2*pi*ci
     .        - GPL1(3)*u**2*v*pi*ci
         col5 = col5 - 1.0d0/2.0d0*GPL1(3)*u**3*pi*ci
     .        + 41.0d0/72.0d0*GPL2(0,0)
     .        + 1.0d0/4.0d0*GPL2(0,0)*pi*ci
     .        - 1.0d0/12.0d0*GPL2(0,1)*pi*ci
     .        - 7.0d0/16.0d0*GPL2(0,2)
     .        + 1.0d0/3.0d0*GPL2(0,2)*pi*ci
     .        + 1.0d0/12.0d0*GPL2(0,2)*v
     .        + 1.0d0/6.0d0*GPL2(0,2)*v**2
     .        + 5.0d0/12.0d0*GPL2(0,2)*u
     .        - 1.0d0/2.0d0*GPL2(0,2)*u*v
     .        + 1.0d0/2.0d0*GPL2(0,2)*u*v**2
     .        - 2.0d0/3.0d0*GPL2(0,2)*u**2
     .        + GPL2(0,2)*u**2*v
     .        + 1.0d0/2.0d0*GPL2(0,2)*u**3
     .        - 5.0d0/24.0d0*GPL2(0,3)*pi*ci
         col5 = col5 - 1.0d0/12.0d0*GPL2(1,0)*pi*ci
     .        + 1.0d0/3.0d0*GPL2(1,1)*pi*ci
     .        + 29.0d0/18.0d0*GPL2(1,2)
     .        - 1.0d0/4.0d0*GPL2(1,2)*u**(-2)
     .        + 1.0d0/2.0d0*GPL2(1,2)*u**(-2)*v
     .        - 1.0d0/4.0d0*GPL2(1,2)*u**(-2)*v**2
     .        - 1.0d0/4.0d0*GPL2(1,2)*pi*ci
     .        - 1.0d0/12.0d0*GPL2(1,2)*v
     .        - 1.0d0/6.0d0*GPL2(1,2)*v**2
     .        - 5.0d0/12.0d0*GPL2(1,2)*u
     .        + 1.0d0/2.0d0*GPL2(1,2)*u*v
     .        - 1.0d0/2.0d0*GPL2(1,2)*u*v**2
     .        + 2.0d0/3.0d0*GPL2(1,2)*u**2
     .        - GPL2(1,2)*u**2*v
     .        - 1.0d0/2.0d0*GPL2(1,2)*u**3
         col5 = col5 - 7.0d0/16.0d0*GPL2(2,0)
     .        + 1.0d0/3.0d0*GPL2(2,0)*pi*ci
     .        + 1.0d0/12.0d0*GPL2(2,0)*v
     .        + 1.0d0/6.0d0*GPL2(2,0)*v**2
     .        + 5.0d0/12.0d0*GPL2(2,0)*u
     .        - 1.0d0/2.0d0*GPL2(2,0)*u*v
     .        + 1.0d0/2.0d0*GPL2(2,0)*u*v**2
     .        - 2.0d0/3.0d0*GPL2(2,0)*u**2
     .        + GPL2(2,0)*u**2*v
     .        + 1.0d0/2.0d0*GPL2(2,0)*u**3
     .        - GPL2(2,1)*pi*ci
     .        + 61.0d0/36.0d0*GPL2(2,2)
     .        + 1.0d0/2.0d0*GPL2(2,2)*pi*ci
     .        - 1.0d0/3.0d0*GPL2(2,3)*pi*ci
     .        + 29.0d0/36.0d0*GPL2(3,0)
         col5 = col5 + 1.0d0/6.0d0*GPL2(3,0)*pi*ci
     .        - 1.0d0/12.0d0*GPL2(3,0)*v
     .        - 1.0d0/6.0d0*GPL2(3,0)*v**2
     .        - 5.0d0/12.0d0*GPL2(3,0)*u
     .        + 1.0d0/2.0d0*GPL2(3,0)*u*v
     .        - 1.0d0/2.0d0*GPL2(3,0)*u*v**2
     .        + 2.0d0/3.0d0*GPL2(3,0)*u**2
     .        - GPL2(3,0)*u**2*v
     .        - 1.0d0/2.0d0*GPL2(3,0)*u**3
     .        - 1.0d0/6.0d0*GPL2(3,3)*pi*ci
     .        + 1.0d0/4.0d0*GPL3(0,0,2)
     .        - 1.0d0/12.0d0*GPL3(0,1,2)
     .        + 1.0d0/4.0d0*GPL3(0,2,0)
     .        + 1.0d0/2.0d0*GPL3(0,2,2)
     .        - 5.0d0/24.0d0*GPL3(0,3,0)
         col5 = col5 - 1.0d0/12.0d0*GPL3(1,0,2)
     .        + 1.0d0/3.0d0*GPL3(1,1,2)
     .        - 1.0d0/12.0d0*GPL3(1,2,0)
     .        - GPL3(1,2,2)
     .        + 1.0d0/4.0d0*GPL3(2,0,0)
     .        + 1.0d0/2.0d0*GPL3(2,0,2)
     .        - GPL3(2,1,2)
     .        + 1.0d0/2.0d0*GPL3(2,2,0)
     .        - 1.0d0/3.0d0*GPL3(2,3,0)
     .        - 1.0d0/4.0d0*GPL3(3,0,0)
     .        - 1.0d0/6.0d0*GPL3(3,3,0)

         col6 = - 25.0d0/81.0d0
     .        - 5.0d0/54.0d0*pi*ci
     .        + 13.0d0/864.0d0*pi**2
     .        - 5.0d0/9.0d0*HPL1(0)
     .        - 5.0d0/72.0d0*HPL1(0)*pi*ci
     .        + 5.0d0/72.0d0*HPL1(0)*GPL1(0)
     .        + 5.0d0/18.0d0*HPL1(0)*GPL1(2)
     .        - 10.0d0/27.0d0*HPL1(1)
     .        - 1.0d0/36.0d0*HPL1(1)*pi*ci
     .        + 1.0d0/36.0d0*HPL1(1)*GPL1(0)
     .        + 2.0d0/9.0d0*HPL1(1)*GPL1(2)
     .        - 5.0d0/12.0d0*HPL2(0,0)
     .        - 5.0d0/18.0d0*HPL2(0,1)
     .        - 5.0d0/18.0d0*HPL2(1,0)
     .        - 2.0d0/9.0d0*HPL2(1,1)
         col6 = col6 + 5.0d0/54.0d0*GPL1(0)
     .        + 1.0d0/144.0d0*GPL1(0)*pi*ci
     .        + 10.0d0/27.0d0*GPL1(2)
     .        + 1.0d0/36.0d0*GPL1(2)*pi*ci
     .        - 5.0d0/144.0d0*GPL2(0,0)
     .        - 1.0d0/36.0d0*GPL2(0,2)
     .        - 1.0d0/36.0d0*GPL2(2,0)
     .        - 2.0d0/9.0d0*GPL2(2,2)
      endif

      col(1) = col1*(-1.d0)
      col(2) = col2*(-1.d0)
      col(3) = col3*(-1.d0)
      col(4) = col4*(-1.d0)
      col(5) = col5*(-1.d0)
      col(6) = col6*(-1.d0)     !To be consistent with gamma_0=1

      end subroutine gamma3a2L

!#############################################################

      subroutine gamma4a2L(u,v,col)
      implicit none
      !Returns gamma(4a+) at one-loop
      !------------------------------
      real*8, intent(in) :: u,v
      complex*16, dimension(6), intent(out) :: col
      !internal
      complex*16 :: gam
      complex*16 :: ci
      real*8 :: omu,omv,omumv,upv
      real*8, dimension(0:1) :: HPL1
      real*8, dimension(0:1,0:1) :: HPL2
      real*8, dimension(0:1,0:1,0:1) :: HPL3
      real*8, dimension(0:1,0:1,0:1,0:1) :: HPL4
      real*8, dimension(0:3) :: GPL1
      real*8, dimension(0:3,0:3) :: GPL2
      real*8, dimension(0:3,0:3,0:3) :: GPL3
      real*8, dimension(0:3,0:3,0:3,0:3) :: GPL4
      complex*16 :: col1,col2,col3,col4,col5,col6


      omu   = 1.0d0 - u
      omv   = 1.0d0     - v
      omumv = 1.0d0 - u - v
      upv   =         u + v

      ci = (0.d0,1.d0)

      call tdhpl(dble(u),dble(v),4,GPL1,GPL2,GPL3,GPL4,HPL1,HPL2,HPL3,HPL4)

      if(ieorder==4) then
         col1 = - 1.0d0/2.0d0

         col2 = 1.0d0/2.0d0

         col3 = - 1.0d0/8.0d0

         col4 = 0.0d0

         col5 = 0.0d0

         col6 = 0.0d0
      elseif(ieorder==3) then
         col1 = - 73.0d0/24.0d0
     .          - 1.0d0/2.0d0*pi*ci
     .          - HPL1(0)
     .          + 1.0d0/2.0d0*GPL1(0)

         col2 = + 109.0d0/48.0d0
     .          + 1.0d0/4.0d0*pi*ci
     .          + HPL1(0)
     .          + 1.0d0/2.0d0*HPL1(1)
     .          - 1.0d0/4.0d0*GPL1(0)
     .          - 1.0d0/2.0d0*GPL1(2)

         col3 = - 3.0d0/8.0d0
     .          - 1.0d0/4.0d0*HPL1(0)
     .          - 1.0d0/4.0d0*HPL1(1)
     .          + 1.0d0/4.0d0*GPL1(2)

         col4 = - 5.0d0/24.0d0

         col5 = + 5.0d0/12.0d0

         col6 = 0
      else
         col1 = - 260437.0d0/10368.0d0
     .        + 5.0d0/4.0d0*u**(-2)*pi*ci
     .        - 5.0d0/2.0d0*u**(-2)*v*pi*ci
     .        + 5.0d0/4.0d0*u**(-2)*v**2*pi*ci
     .        + 269.0d0/72.0d0*u**(-1)
     .        - 55.0d0/48.0d0*u**(-1)*pi*ci
     .        + 7.0d0/48.0d0*u**(-1)*pi**2
     .        - 269.0d0/72.0d0*u**(-1)*v
     .        + 25.0d0/48.0d0*u**(-1)*v*pi*ci
     .        - 1.0d0/6.0d0*u**(-1)*v*pi**2
     .        + 5.0d0/8.0d0*u**(-1)*v**2*pi*ci
     .        + 1.0d0/12.0d0*u**(-1)*v**2*pi**2
     .        - 51.0d0/16.0d0*pi*ci
     .        - 1847.0d0/864.0d0*pi**2
     .        - 55.0d0/96.0d0*pi**3*ci
         col1 = col1 - 3.0d0/80.0d0*pi**4
     .        + 67.0d0/72.0d0*zeta3
     .        + 5.0d0/4.0d0*zeta3*pi*ci
     .        - 4.0d0/3.0d0*v*omu**(-1)
     .        - 5.0d0/3.0d0*v*omu**(-1)*pi*ci
     .        + 5.0d0/8.0d0*v*pi*ci
     .        + 2.0d0/3.0d0*v*pi**2
     .        - 4.0d0/3.0d0*v**2*omu**(-2)*pi*ci
     .        + 5.0d0/12.0d0*v**2*pi**2
     .        + 7.0d0/12.0d0*u*pi**2
     .        + 5.0d0/6.0d0*u*v*pi**2
     .        + 5.0d0/12.0d0*u**2*pi**2
     .        - 16499.0d0/864.0d0*HPL1(0)
     .        + 5.0d0/4.0d0*HPL1(0)*u**(-2)*v
     .        - 5.0d0/4.0d0*HPL1(0)*u**(-2)*v**2
         col1 = col1 + 55.0d0/48.0d0*HPL1(0)*u**(-1)
     .        + 1.0d0/8.0d0*HPL1(0)*u**(-1)*pi*ci
     .        - 13.0d0/6.0d0*HPL1(0)*u**(-1)*v
     .        - 1.0d0/4.0d0*HPL1(0)*u**(-1)*v*pi*ci
     .        - 5.0d0/8.0d0*HPL1(0)*u**(-1)*v**2
     .        + 1.0d0/8.0d0*HPL1(0)*u**(-1)*v**2*pi*ci
     .        - 395.0d0/48.0d0*HPL1(0)*pi*ci
     .        + 3.0d0/16.0d0*HPL1(0)*pi**2
     .        - 7.0d0/24.0d0*HPL1(0)*pi**3*ci
     .        - 5.0d0/2.0d0*HPL1(0)*zeta3
     .        + 5.0d0/3.0d0*HPL1(0)*v*omu**(-1)
     .        + 19.0d0/48.0d0*HPL1(0)*v*omv**(-1)
     .        - 5.0d0/8.0d0*HPL1(0)*v
     .        + HPL1(0)*v*pi*ci
     .        + 4.0d0/3.0d0*HPL1(0)*v**2*omu**(-2)
         col1 = col1 + 5.0d0/8.0d0*HPL1(0)*v**2*pi*ci
     .        + 7.0d0/8.0d0*HPL1(0)*u*pi*ci
     .        + 5.0d0/4.0d0*HPL1(0)*u*v*pi*ci
     .        + 5.0d0/8.0d0*HPL1(0)*u**2*pi*ci
     .        - 85.0d0/144.0d0*HPL1(0)*GPL1(0)
     .        + 1.0d0/8.0d0*HPL1(0)*GPL1(0)*u**(-1)
     .        - 3.0d0/8.0d0*HPL1(0)*GPL1(0)*u**(-1)*v
     .        + 1.0d0/8.0d0*HPL1(0)*GPL1(0)*u**(-1)*v**2
     .        + 11.0d0/8.0d0*HPL1(0)*GPL1(0)*pi*ci
     .        - 13.0d0/24.0d0*HPL1(0)*GPL1(0)*pi**2
     .        + HPL1(0)*GPL1(0)*v
     .        + 5.0d0/8.0d0*HPL1(0)*GPL1(0)*v**2
     .        + 7.0d0/8.0d0*HPL1(0)*GPL1(0)*u
     .        + 5.0d0/4.0d0*HPL1(0)*GPL1(0)*u*v
     .        + 5.0d0/8.0d0*HPL1(0)*GPL1(0)*u**2
         col1 = col1 + 313.0d0/36.0d0*HPL1(0)*GPL1(1)
     .        - 5.0d0/4.0d0*HPL1(0)*GPL1(1)*u**(-3)
     .        + 5.0d0/2.0d0*HPL1(0)*GPL1(1)*u**(-3)*v
     .        - 5.0d0/4.0d0*HPL1(0)*GPL1(1)*u**(-3)*v**2
     .        + 5.0d0/3.0d0*HPL1(0)*GPL1(1)*u**(-2)
     .        + 1.0d0/8.0d0*HPL1(0)*GPL1(1)*u**(-2)*pi*ci
     .        - HPL1(0)*GPL1(1)*u**(-2)*v
     .        - 2.0d0/3.0d0*HPL1(0)*GPL1(1)*u**(-2)*v**2
     .        - 5.0d0/3.0d0*HPL1(0)*GPL1(1)*u**(-1)
     .        + 1.0d0/4.0d0*HPL1(0)*GPL1(1)*u**(-1)*pi*ci
     .        + 5.0d0/6.0d0*HPL1(0)*GPL1(1)*u**(-1)*v
     .        - 1.0d0/8.0d0*HPL1(0)*GPL1(1)*u**(-1)*v**2
     .        - 11.0d0/24.0d0*HPL1(0)*GPL1(1)*pi*ci
     .        + 7.0d0/12.0d0*HPL1(0)*GPL1(1)*pi**2
     .        - HPL1(0)*GPL1(1)*v
         col1 = col1 - 5.0d0/8.0d0*HPL1(0)*GPL1(1)*v**2
     .        - 7.0d0/8.0d0*HPL1(0)*GPL1(1)*u
     .        - 5.0d0/4.0d0*HPL1(0)*GPL1(1)*u*v
     .        - 5.0d0/8.0d0*HPL1(0)*GPL1(1)*u**2
     .        - 179.0d0/48.0d0*HPL1(0)*GPL1(2)
     .        + 5.0d0/4.0d0*HPL1(0)*GPL1(2)*u**(-3)
     .        - 5.0d0/2.0d0*HPL1(0)*GPL1(2)*u**(-3)*v
     .        + 5.0d0/4.0d0*HPL1(0)*GPL1(2)*u**(-3)*v**2
     .        - 5.0d0/3.0d0*HPL1(0)*GPL1(2)*u**(-2)
     .        - 1.0d0/8.0d0*HPL1(0)*GPL1(2)*u**(-2)*pi*ci
     .        + HPL1(0)*GPL1(2)*u**(-2)*v
     .        + 1.0d0/4.0d0*HPL1(0)*GPL1(2)*u**(-2)*v*pi*ci
     .        + 2.0d0/3.0d0*HPL1(0)*GPL1(2)*u**(-2)*v**2
     .        - 1.0d0/8.0d0*HPL1(0)*GPL1(2)*u**(-2)*v**2*pi*ci
     .        + 5.0d0/3.0d0*HPL1(0)*GPL1(2)*u**(-1)
         col1 = col1 - 1.0d0/4.0d0*HPL1(0)*GPL1(2)*u**(-1)*pi*ci
     .        - 17.0d0/24.0d0*HPL1(0)*GPL1(2)*u**(-1)*v
     .        + 1.0d0/4.0d0*HPL1(0)*GPL1(2)*u**(-1)*v*pi*ci
     .        + 1.0d0/8.0d0*HPL1(0)*GPL1(2)*u**(-1)*v**2
     .        - 3.0d0/4.0d0*HPL1(0)*GPL1(2)*pi*ci
     .        - 5.0d0/12.0d0*HPL1(0)*GPL1(2)*pi**2
     .        + HPL1(0)*GPL1(2)*v
     .        + 5.0d0/8.0d0*HPL1(0)*GPL1(2)*v**2
     .        + 7.0d0/8.0d0*HPL1(0)*GPL1(2)*u
     .        + 5.0d0/4.0d0*HPL1(0)*GPL1(2)*u*v
     .        + 5.0d0/8.0d0*HPL1(0)*GPL1(2)*u**2
     .        + 635.0d0/144.0d0*HPL1(0)*GPL1(3)
     .        - 1.0d0/8.0d0*HPL1(0)*GPL1(3)*u**(-1)
     .        + 3.0d0/8.0d0*HPL1(0)*GPL1(3)*u**(-1)*v
     .        - 1.0d0/8.0d0*HPL1(0)*GPL1(3)*u**(-1)*v**2
         col1 = col1 + 7.0d0/8.0d0*HPL1(0)*GPL1(3)*pi*ci
     .        + 19.0d0/24.0d0*HPL1(0)*GPL1(3)*pi**2
     .        - HPL1(0)*GPL1(3)*v
     .        - 5.0d0/8.0d0*HPL1(0)*GPL1(3)*v**2
     .        - 7.0d0/8.0d0*HPL1(0)*GPL1(3)*u
     .        - 5.0d0/4.0d0*HPL1(0)*GPL1(3)*u*v
     .        - 5.0d0/8.0d0*HPL1(0)*GPL1(3)*u**2
     .        + 7.0d0/4.0d0*HPL1(0)*GPL2(0,0)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(0,0)*pi*ci
     .        - 29.0d0/24.0d0*HPL1(0)*GPL2(0,1)
     .        + 1.0d0/8.0d0*HPL1(0)*GPL2(0,1)*u**(-2)
     .        - 1.0d0/4.0d0*HPL1(0)*GPL2(0,1)*u**(-2)*v
     .        + 1.0d0/8.0d0*HPL1(0)*GPL2(0,1)*u**(-2)*v**2
     .        + 1.0d0/4.0d0*HPL1(0)*GPL2(0,1)*u**(-1)
     .        - 1.0d0/4.0d0*HPL1(0)*GPL2(0,1)*u**(-1)*v
         col1 = col1 + 1.0d0/4.0d0*HPL1(0)*GPL2(0,1)*pi*ci
     .        + 25.0d0/8.0d0*HPL1(0)*GPL2(0,2)
     .        - 1.0d0/8.0d0*HPL1(0)*GPL2(0,2)*u**(-2)
     .        + 1.0d0/4.0d0*HPL1(0)*GPL2(0,2)*u**(-2)*v
     .        - 1.0d0/8.0d0*HPL1(0)*GPL2(0,2)*u**(-2)*v**2
     .        - 1.0d0/4.0d0*HPL1(0)*GPL2(0,2)*u**(-1)
     .        + 1.0d0/4.0d0*HPL1(0)*GPL2(0,2)*u**(-1)*v
     .        + HPL1(0)*GPL2(0,2)*pi*ci
     .        - 4.0d0/3.0d0*HPL1(0)*GPL2(0,3)
     .        - 1.0d0/4.0d0*HPL1(0)*GPL2(0,3)*pi*ci
     .        - 5.0d0/6.0d0*HPL1(0)*GPL2(1,0)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(1,0)*pi*ci
     .        + 13.0d0/12.0d0*HPL1(0)*GPL2(1,1)
     .        + 1.0d0/4.0d0*HPL1(0)*GPL2(1,1)*u**(-2)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(1,1)*u**(-1)
         col1 = col1 - 127.0d0/24.0d0*HPL1(0)*GPL2(1,2)
     .        - 1.0d0/8.0d0*HPL1(0)*GPL2(1,2)*u**(-2)
     .        - 1.0d0/4.0d0*HPL1(0)*GPL2(1,2)*u**(-1)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(1,2)*pi*ci
     .        + 101.0d0/24.0d0*HPL1(0)*GPL2(2,0)
     .        + 3.0d0/4.0d0*HPL1(0)*GPL2(2,0)*pi*ci
     .        - 35.0d0/8.0d0*HPL1(0)*GPL2(2,1)
     .        - 1.0d0/8.0d0*HPL1(0)*GPL2(2,1)*u**(-2)
     .        + 1.0d0/4.0d0*HPL1(0)*GPL2(2,1)*u**(-2)*v
     .        - 1.0d0/8.0d0*HPL1(0)*GPL2(2,1)*u**(-2)*v**2
     .        - 1.0d0/4.0d0*HPL1(0)*GPL2(2,1)*u**(-1)
     .        + 1.0d0/4.0d0*HPL1(0)*GPL2(2,1)*u**(-1)*v
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(2,1)*pi*ci
     .        + 4.0d0*HPL1(0)*GPL2(2,2)
     .        - HPL1(0)*GPL2(2,2)*pi*ci
         col1 = col1 - 13.0d0/12.0d0*HPL1(0)*GPL2(2,3)
     .        - 43.0d0/24.0d0*HPL1(0)*GPL2(3,0)
     .        - HPL1(0)*GPL2(3,0)*pi*ci
     .        + 3.0d0/8.0d0*HPL1(0)*GPL2(3,2)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(3,2)*pi*ci
     .        - 11.0d0/12.0d0*HPL1(0)*GPL2(3,3)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(3,3)*pi*ci
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(0,0,2)
     .        + 3.0d0/4.0d0*HPL1(0)*GPL3(0,1,0)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL3(0,1,2)
     .        + 1.0d0/4.0d0*HPL1(0)*GPL3(0,2,0)
     .        - 1.0d0/4.0d0*HPL1(0)*GPL3(0,2,1)
     .        + HPL1(0)*GPL3(0,2,2)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL3(0,2,3)
     .        - 1.0d0/4.0d0*HPL1(0)*GPL3(0,3,0)
         col1 = col1 + 1.0d0/2.0d0*HPL1(0)*GPL3(0,3,3)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL3(1,0,2)
     .        - 2.0d0*HPL1(0)*GPL3(1,1,1)
     .        + 2.0d0*HPL1(0)*GPL3(1,1,2)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL3(1,2,0)
     .        + 2.0d0*HPL1(0)*GPL3(1,2,1)
     .        - 2.0d0*HPL1(0)*GPL3(1,2,2)
     .        + 3.0d0/4.0d0*HPL1(0)*GPL3(2,0,0)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL3(2,0,1)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(2,0,2)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL3(2,0,3)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL3(2,1,0)
     .        + 2.0d0*HPL1(0)*GPL3(2,1,1)
     .        - HPL1(0)*GPL3(2,1,2)
     .        - HPL1(0)*GPL3(2,2,1)
         col1 = col1 + HPL1(0)*GPL3(2,2,3)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL3(2,3,0)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(2,3,2)
     .        + 1.0d0/4.0d0*HPL1(0)*GPL3(2,3,3)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL3(3,0,0)
     .        - 1.0d0/4.0d0*HPL1(0)*GPL3(3,0,1)
     .        - 1.0d0/4.0d0*HPL1(0)*GPL3(3,0,2)
     .        + 1.0d0/4.0d0*HPL1(0)*GPL3(3,0,3)
     .        - 1.0d0/4.0d0*HPL1(0)*GPL3(3,2,0)
     .        - 1.0d0/4.0d0*HPL1(0)*GPL3(3,2,1)
     .        - 1.0d0/4.0d0*HPL1(0)*GPL3(3,2,3)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(3,3,0)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL3(3,3,3)
     .        - 9965.0d0/864.0d0*HPL1(1)
     .        - 5.0d0/4.0d0*HPL1(1)*u**(-2)
         col1 = col1 + 5.0d0/2.0d0*HPL1(1)*u**(-2)*v
     .        - 5.0d0/4.0d0*HPL1(1)*u**(-2)*v**2
     .        + 13.0d0/8.0d0*HPL1(1)*u**(-1)
     .        + 1.0d0/8.0d0*HPL1(1)*u**(-1)*pi*ci
     .        - HPL1(1)*u**(-1)*v
     .        - 1.0d0/4.0d0*HPL1(1)*u**(-1)*v*pi*ci
     .        - 5.0d0/8.0d0*HPL1(1)*u**(-1)*v**2
     .        + 1.0d0/8.0d0*HPL1(1)*u**(-1)*v**2*pi*ci
     .        - 11.0d0/2.0d0*HPL1(1)*pi*ci
     .        + 13.0d0/36.0d0*HPL1(1)*pi**2
     .        - 7.0d0/24.0d0*HPL1(1)*pi**3*ci
     .        + 1.0d0/2.0d0*HPL1(1)*zeta3
     .        + 5.0d0/3.0d0*HPL1(1)*v*omu**(-1)
     .        - 5.0d0/4.0d0*HPL1(1)*v
     .        + HPL1(1)*v*pi*ci
         col1 = col1 + 4.0d0/3.0d0*HPL1(1)*v**2*omu**(-2)
     .        + 5.0d0/8.0d0*HPL1(1)*v**2*pi*ci
     .        - 5.0d0/8.0d0*HPL1(1)*u
     .        + 7.0d0/8.0d0*HPL1(1)*u*pi*ci
     .        + 5.0d0/4.0d0*HPL1(1)*u*v*pi*ci
     .        + 5.0d0/8.0d0*HPL1(1)*u**2*pi*ci
     .        - 133.0d0/36.0d0*HPL1(1)*GPL1(0)
     .        + 1.0d0/8.0d0*HPL1(1)*GPL1(0)*u**(-1)
     .        - 1.0d0/4.0d0*HPL1(1)*GPL1(0)*u**(-1)*v
     .        + 1.0d0/8.0d0*HPL1(1)*GPL1(0)*u**(-1)*v**2
     .        + 35.0d0/24.0d0*HPL1(1)*GPL1(0)*pi*ci
     .        - 5.0d0/12.0d0*HPL1(1)*GPL1(0)*pi**2
     .        + HPL1(1)*GPL1(0)*v
     .        + 5.0d0/8.0d0*HPL1(1)*GPL1(0)*v**2
     .        + 7.0d0/8.0d0*HPL1(1)*GPL1(0)*u
         col1 = col1 + 5.0d0/4.0d0*HPL1(1)*GPL1(0)*u*v
     .        + 5.0d0/8.0d0*HPL1(1)*GPL1(0)*u**2
     .        + 313.0d0/36.0d0*HPL1(1)*GPL1(1)
     .        - 5.0d0/4.0d0*HPL1(1)*GPL1(1)*u**(-3)
     .        + 5.0d0/2.0d0*HPL1(1)*GPL1(1)*u**(-3)*v
     .        - 5.0d0/4.0d0*HPL1(1)*GPL1(1)*u**(-3)*v**2
     .        + 5.0d0/3.0d0*HPL1(1)*GPL1(1)*u**(-2)
     .        - HPL1(1)*GPL1(1)*u**(-2)*v
     .        - 2.0d0/3.0d0*HPL1(1)*GPL1(1)*u**(-2)*v**2
     .        - 5.0d0/3.0d0*HPL1(1)*GPL1(1)*u**(-1)
     .        + 5.0d0/6.0d0*HPL1(1)*GPL1(1)*u**(-1)*v
     .        - 1.0d0/8.0d0*HPL1(1)*GPL1(1)*u**(-1)*v**2
     .        - 1.0d0/4.0d0*HPL1(1)*GPL1(1)*pi*ci
     .        + 5.0d0/12.0d0*HPL1(1)*GPL1(1)*pi**2
     .        - HPL1(1)*GPL1(1)*v
         col1 = col1 - 5.0d0/8.0d0*HPL1(1)*GPL1(1)*v**2
     .        - 7.0d0/8.0d0*HPL1(1)*GPL1(1)*u
     .        - 5.0d0/4.0d0*HPL1(1)*GPL1(1)*u*v
     .        - 5.0d0/8.0d0*HPL1(1)*GPL1(1)*u**2
     .        + 455.0d0/144.0d0*HPL1(1)*GPL1(2)
     .        + 2.0d0*HPL1(1)*GPL1(2)*pi*ci
     .        + 1.0d0/8.0d0*HPL1(1)*GPL1(2)*pi**2
     .        + 3.0d0/8.0d0*HPL1(1)*GPL1(3)*pi*ci
     .        + 1.0d0/3.0d0*HPL1(1)*GPL1(3)*pi**2
     .        + 7.0d0/4.0d0*HPL1(1)*GPL2(0,0)
     .        + 1.0d0/2.0d0*HPL1(1)*GPL2(0,0)*pi*ci
     .        - 29.0d0/24.0d0*HPL1(1)*GPL2(0,1)
     .        + 1.0d0/8.0d0*HPL1(1)*GPL2(0,1)*u**(-2)
     .        - 1.0d0/4.0d0*HPL1(1)*GPL2(0,1)*u**(-2)*v
     .        + 1.0d0/8.0d0*HPL1(1)*GPL2(0,1)*u**(-2)*v**2
         col1 = col1 + 1.0d0/4.0d0*HPL1(1)*GPL2(0,1)*u**(-1)
     .        - 1.0d0/4.0d0*HPL1(1)*GPL2(0,1)*u**(-1)*v
     .        - 1.0d0/4.0d0*HPL1(1)*GPL2(0,1)*pi*ci
     .        + 2.0d0*HPL1(1)*GPL2(0,2)
     .        + 1.0d0/2.0d0*HPL1(1)*GPL2(0,2)*pi*ci
     .        - 5.0d0/6.0d0*HPL1(1)*GPL2(1,0)
     .        - HPL1(1)*GPL2(1,0)*pi*ci
     .        + 13.0d0/12.0d0*HPL1(1)*GPL2(1,1)
     .        + 1.0d0/4.0d0*HPL1(1)*GPL2(1,1)*u**(-2)
     .        + 1.0d0/2.0d0*HPL1(1)*GPL2(1,1)*u**(-1)
     .        + 2.0d0*HPL1(1)*GPL2(1,1)*pi*ci
     .        - 4.0d0*HPL1(1)*GPL2(1,2)
     .        - HPL1(1)*GPL2(1,2)*pi*ci
     .        + 2.0d0*HPL1(1)*GPL2(2,0)
     .        + 1.0d0/2.0d0*HPL1(1)*GPL2(2,0)*pi*ci
         col1 = col1 - 35.0d0/8.0d0*HPL1(1)*GPL2(2,1)
     .        - 1.0d0/8.0d0*HPL1(1)*GPL2(2,1)*u**(-2)
     .        + 1.0d0/4.0d0*HPL1(1)*GPL2(2,1)*u**(-2)*v
     .        - 1.0d0/8.0d0*HPL1(1)*GPL2(2,1)*u**(-2)*v**2
     .        - 1.0d0/4.0d0*HPL1(1)*GPL2(2,1)*u**(-1)
     .        + 1.0d0/4.0d0*HPL1(1)*GPL2(2,1)*u**(-1)*v
     .        - 1.0d0/2.0d0*HPL1(1)*GPL2(2,1)*pi*ci
     .        + 1.0d0/2.0d0*HPL1(1)*GPL2(2,3)*pi*ci
     .        + 3.0d0/8.0d0*HPL1(1)*GPL2(3,0)
     .        - 1.0d0/2.0d0*HPL1(1)*GPL2(3,0)*pi*ci
     .        + 1.0d0/2.0d0*HPL1(1)*GPL3(0,0,2)
     .        + 3.0d0/4.0d0*HPL1(1)*GPL3(0,1,0)
     .        - HPL1(1)*GPL3(0,1,2)
     .        + 1.0d0/2.0d0*HPL1(1)*GPL3(0,2,0)
     .        - 1.0d0/4.0d0*HPL1(1)*GPL3(0,2,1)
         col1 = col1 - HPL1(1)*GPL3(1,0,2)
     .        - 2.0d0*HPL1(1)*GPL3(1,1,1)
     .        + 2.0d0*HPL1(1)*GPL3(1,1,2)
     .        - HPL1(1)*GPL3(1,2,0)
     .        + 2.0d0*HPL1(1)*GPL3(1,2,1)
     .        + 1.0d0/2.0d0*HPL1(1)*GPL3(2,0,0)
     .        - 1.0d0/2.0d0*HPL1(1)*GPL3(2,0,1)
     .        - 1.0d0/2.0d0*HPL1(1)*GPL3(2,1,0)
     .        + 2.0d0*HPL1(1)*GPL3(2,1,1)
     .        - HPL1(1)*GPL3(2,2,1)
     .        + 1.0d0/2.0d0*HPL1(1)*GPL3(2,3,0)
     .        - 1.0d0/2.0d0*HPL1(1)*GPL3(3,0,0)
     .        - 1.0d0/4.0d0*HPL1(1)*GPL3(3,0,1)
     .        - 1.0d0/4.0d0*HPL1(1)*GPL3(3,2,1)
     .        - 295.0d0/36.0d0*HPL2(0,0)
         col1 = col1 - 1.0d0/8.0d0*HPL2(0,0)*u**(-1)
     .        + 3.0d0/8.0d0*HPL2(0,0)*u**(-1)*v
     .        - 1.0d0/8.0d0*HPL2(0,0)*u**(-1)*v**2
     .        - 17.0d0/12.0d0*HPL2(0,0)*pi*ci
     .        + 19.0d0/24.0d0*HPL2(0,0)*pi**2
     .        - HPL2(0,0)*v
     .        - 5.0d0/8.0d0*HPL2(0,0)*v**2
     .        - 7.0d0/8.0d0*HPL2(0,0)*u
     .        - 5.0d0/4.0d0*HPL2(0,0)*u*v
     .        - 5.0d0/8.0d0*HPL2(0,0)*u**2
     .        - 49.0d0/12.0d0*HPL2(0,0)*GPL1(0)
     .        - HPL2(0,0)*GPL1(0)*pi*ci
     .        + 11.0d0/2.0d0*HPL2(0,0)*GPL1(1)
     .        + HPL2(0,0)*GPL1(1)*pi*ci
     .        - 79.0d0/12.0d0*HPL2(0,0)*GPL1(2)
         col1 = col1 - HPL2(0,0)*GPL1(2)*pi*ci
     .        + 11.0d0/6.0d0*HPL2(0,0)*GPL1(3)
     .        + 5.0d0/4.0d0*HPL2(0,0)*GPL1(3)*pi*ci
     .        - 1.0d0/2.0d0*HPL2(0,0)*GPL2(0,0)
     .        - HPL2(0,0)*GPL2(0,2)
     .        + 1.0d0/2.0d0*HPL2(0,0)*GPL2(0,3)
     .        + HPL2(0,0)*GPL2(1,2)
     .        - 1.0d0/2.0d0*HPL2(0,0)*GPL2(2,0)
     .        + 1.0d0/4.0d0*HPL2(0,0)*GPL2(2,3)
     .        + 3.0d0/4.0d0*HPL2(0,0)*GPL2(3,0)
     .        + 1.0d0/4.0d0*HPL2(0,0)*GPL2(3,2)
     .        - 1.0d0/2.0d0*HPL2(0,0)*GPL2(3,3)
     .        - 715.0d0/144.0d0*HPL2(0,1)
     .        - 1.0d0/8.0d0*HPL2(0,1)*u**(-1)*v
     .        - 23.0d0/12.0d0*HPL2(0,1)*pi*ci
         col1 = col1 + 1.0d0/3.0d0*HPL2(0,1)*pi**2
     .        - 23.0d0/12.0d0*HPL2(0,1)*GPL1(0)
     .        - 1.0d0/2.0d0*HPL2(0,1)*GPL1(0)*pi*ci
     .        + 101.0d0/24.0d0*HPL2(0,1)*GPL1(1)
     .        - 1.0d0/8.0d0*HPL2(0,1)*GPL1(1)*u**(-2)
     .        - 1.0d0/4.0d0*HPL2(0,1)*GPL1(1)*u**(-1)
     .        + 1.0d0/2.0d0*HPL2(0,1)*GPL1(1)*pi*ci
     .        + 3.0d0/8.0d0*HPL2(0,1)*GPL1(2)
     .        + 1.0d0/8.0d0*HPL2(0,1)*GPL1(2)*u**(-2)
     .        - 1.0d0/4.0d0*HPL2(0,1)*GPL1(2)*u**(-2)*v
     .        + 1.0d0/8.0d0*HPL2(0,1)*GPL1(2)*u**(-2)*v**2
     .        + 1.0d0/4.0d0*HPL2(0,1)*GPL1(2)*u**(-1)
     .        - 1.0d0/4.0d0*HPL2(0,1)*GPL1(2)*u**(-1)*v
     .        - 3.0d0/8.0d0*HPL2(0,1)*GPL1(3)
     .        + 1.0d0/2.0d0*HPL2(0,1)*GPL1(3)*pi*ci
         col1 = col1 - 1.0d0/2.0d0*HPL2(0,1)*GPL2(0,0)
     .        + 1.0d0/2.0d0*HPL2(0,1)*GPL2(0,1)
     .        - 3.0d0/4.0d0*HPL2(0,1)*GPL2(0,2)
     .        + 1.0d0/2.0d0*HPL2(0,1)*GPL2(1,0)
     .        - HPL2(0,1)*GPL2(2,1)
     .        + HPL2(0,1)*GPL2(2,2)
     .        - 1.0d0/2.0d0*HPL2(0,1)*GPL2(2,3)
     .        + 1.0d0/2.0d0*HPL2(0,1)*GPL2(3,0)
     .        + 1.0d0/4.0d0*HPL2(0,1)*GPL2(3,2)
     .        - 11.0d0/36.0d0*HPL2(1,0)
     .        + 1.0d0/8.0d0*HPL2(1,0)*u**(-1)*v
     .        - 1.0d0/8.0d0*HPL2(1,0)*u**(-1)*v**2
     .        - 9.0d0/8.0d0*HPL2(1,0)*pi*ci
     .        + 5.0d0/8.0d0*HPL2(1,0)*pi**2
     .        - HPL2(1,0)*v
         col1 = col1 - 5.0d0/8.0d0*HPL2(1,0)*v**2
     .        - 7.0d0/8.0d0*HPL2(1,0)*u
     .        - 5.0d0/4.0d0*HPL2(1,0)*u*v
     .        - 5.0d0/8.0d0*HPL2(1,0)*u**2
     .        - 91.0d0/24.0d0*HPL2(1,0)*GPL1(0)
     .        - HPL2(1,0)*GPL1(0)*pi*ci
     .        + 127.0d0/24.0d0*HPL2(1,0)*GPL1(1)
     .        + 1.0d0/8.0d0*HPL2(1,0)*GPL1(1)*u**(-2)
     .        + 1.0d0/4.0d0*HPL2(1,0)*GPL1(1)*u**(-1)
     .        + 1.0d0/2.0d0*HPL2(1,0)*GPL1(1)*pi*ci
     .        - 35.0d0/12.0d0*HPL2(1,0)*GPL1(2)
     .        + 3.0d0/4.0d0*HPL2(1,0)*GPL1(2)*pi*ci
     .        - 3.0d0/8.0d0*HPL2(1,0)*GPL1(3)
     .        + 1.0d0/2.0d0*HPL2(1,0)*GPL1(3)*pi*ci
     .        - 3.0d0/4.0d0*HPL2(1,0)*GPL2(0,0)
         col1 = col1 + 1.0d0/2.0d0*HPL2(1,0)*GPL2(0,1)
     .        - 3.0d0/4.0d0*HPL2(1,0)*GPL2(0,2)
     .        + 1.0d0/2.0d0*HPL2(1,0)*GPL2(1,0)
     .        - 2.0d0*HPL2(1,0)*GPL2(1,1)
     .        + 2.0d0*HPL2(1,0)*GPL2(1,2)
     .        + 1.0d0/4.0d0*HPL2(1,0)*GPL2(2,0)
     .        + HPL2(1,0)*GPL2(2,1)
     .        - HPL2(1,0)*GPL2(2,2)
     .        - 1.0d0/2.0d0*HPL2(1,0)*GPL2(2,3)
     .        + 1.0d0/4.0d0*HPL2(1,0)*GPL2(3,0)
     .        - 455.0d0/144.0d0*HPL2(1,1)
     .        - 2.0d0*HPL2(1,1)*pi*ci
     .        - 1.0d0/24.0d0*HPL2(1,1)*pi**2
     .        - 2.0d0*HPL2(1,1)*GPL1(0)
     .        - 1.0d0/2.0d0*HPL2(1,1)*GPL1(0)*pi*ci
         col1 = col1 + 4.0d0*HPL2(1,1)*GPL1(1)
     .        + HPL2(1,1)*GPL1(1)*pi*ci
     .        - 1.0d0/2.0d0*HPL2(1,1)*GPL2(0,0)
     .        + HPL2(1,1)*GPL2(0,1)
     .        + HPL2(1,1)*GPL2(1,0)
     .        - 2.0d0*HPL2(1,1)*GPL2(1,1)
     .        + 33.0d0/8.0d0*HPL3(0,0,0)
     .        + 5.0d0/4.0d0*HPL3(0,0,0)*pi*ci
     .        + 3.0d0/4.0d0*HPL3(0,0,0)*GPL1(0)
     .        - HPL3(0,0,0)*GPL1(1)
     .        + 5.0d0/4.0d0*HPL3(0,0,0)*GPL1(2)
     .        - 3.0d0/4.0d0*HPL3(0,0,0)*GPL1(3)
     .        - 3.0d0/8.0d0*HPL3(0,0,1)
     .        + 1.0d0/2.0d0*HPL3(0,0,1)*pi*ci
     .        + 1.0d0/2.0d0*HPL3(0,0,1)*GPL1(0)
         col1 = col1 - HPL3(0,0,1)*GPL1(1)
     .        + 1.0d0/2.0d0*HPL3(0,0,1)*GPL1(2)
     .        - 1.0d0/2.0d0*HPL3(0,0,1)*GPL1(3)
     .        + 23.0d0/12.0d0*HPL3(0,1,0)
     .        + 1.0d0/2.0d0*HPL3(0,1,0)*pi*ci
     .        + 1.0d0/4.0d0*HPL3(0,1,0)*GPL1(0)
     .        - HPL3(0,1,0)*GPL1(1)
     .        + 1.0d0/2.0d0*HPL3(0,1,0)*GPL1(2)
     .        - 1.0d0/4.0d0*HPL3(0,1,0)*GPL1(3)
     .        + 23.0d0/6.0d0*HPL3(1,0,0)
     .        + HPL3(1,0,0)*pi*ci
     .        + HPL3(1,0,0)*GPL1(0)
     .        - HPL3(1,0,0)*GPL1(1)
     .        - 1.0d0/4.0d0*HPL3(1,0,0)*GPL1(3)
     .        - 3.0d0/8.0d0*HPL3(1,0,1)
         col1 = col1 + 1.0d0/4.0d0*HPL3(1,0,1)*pi*ci
     .        + 1.0d0/4.0d0*HPL3(1,0,1)*GPL1(0)
     .        - 1.0d0/2.0d0*HPL3(1,0,1)*GPL1(2)
     .        - 1.0d0/4.0d0*HPL3(1,0,1)*GPL1(3)
     .        + 11.0d0/4.0d0*HPL3(1,1,0)
     .        - 3.0d0/4.0d0*HPL3(1,1,0)*pi*ci
     .        - 2.0d0*HPL3(1,1,0)*GPL1(1)
     .        + 7.0d0/4.0d0*HPL3(1,1,0)*GPL1(2)
     .        - 3.0d0/4.0d0*HPL4(0,0,0,0)
     .        - 1.0d0/2.0d0*HPL4(0,0,0,1)
     .        - 1.0d0/4.0d0*HPL4(0,0,1,0)
     .        - 1.0d0/4.0d0*HPL4(0,1,0,0)
     .        - 1.0d0/4.0d0*HPL4(0,1,0,1)
     .        - 3.0d0/4.0d0*HPL4(1,0,0,0)
     .        + 1.0d0/2.0d0*HPL4(1,1,0,0)
         col1 = col1 + 3.0d0/4.0d0*HPL4(1,1,0,1)
     .        - HPL4(1,1,1,0)
     .        + 35.0d0/8.0d0*GPL1(0)
     .        - 2.0d0/3.0d0*GPL1(0)*u**(-1)
     .        - 1.0d0/8.0d0*GPL1(0)*u**(-1)*pi*ci
     .        + 2.0d0/3.0d0*GPL1(0)*u**(-1)*v
     .        + 1.0d0/4.0d0*GPL1(0)*u**(-1)*v*pi*ci
     .        - 1.0d0/8.0d0*GPL1(0)*u**(-1)*v**2*pi*ci
     .        + 127.0d0/24.0d0*GPL1(0)*pi*ci
     .        - 17.0d0/96.0d0*GPL1(0)*pi**2
     .        + 1.0d0/24.0d0*GPL1(0)*pi**3*ci
     .        + 1.0d0/2.0d0*GPL1(0)*zeta3
     .        - 5.0d0/8.0d0*GPL1(0)*v
     .        - GPL1(0)*v*pi*ci
     .        - 5.0d0/8.0d0*GPL1(0)*v**2*pi*ci
         col1 = col1 - 5.0d0/8.0d0*GPL1(0)*u
     .        - 7.0d0/8.0d0*GPL1(0)*u*pi*ci
     .        - 5.0d0/4.0d0*GPL1(0)*u*v*pi*ci
     .        - 5.0d0/8.0d0*GPL1(0)*u**2*pi*ci
     .        + 5.0d0/4.0d0*GPL1(1)*u**(-3)*pi*ci
     .        - 5.0d0/2.0d0*GPL1(1)*u**(-3)*v*pi*ci
     .        + 5.0d0/4.0d0*GPL1(1)*u**(-3)*v**2*pi*ci
     .        - 5.0d0/3.0d0*GPL1(1)*u**(-2)*pi*ci
     .        + 1.0d0/12.0d0*GPL1(1)*u**(-2)*pi**2
     .        + GPL1(1)*u**(-2)*v*pi*ci
     .        + 2.0d0/3.0d0*GPL1(1)*u**(-2)*v**2*pi*ci
     .        + 5.0d0/3.0d0*GPL1(1)*u**(-1)*pi*ci
     .        + 1.0d0/6.0d0*GPL1(1)*u**(-1)*pi**2
     .        - 5.0d0/6.0d0*GPL1(1)*u**(-1)*v*pi*ci
     .        + 1.0d0/8.0d0*GPL1(1)*u**(-1)*v**2*pi*ci
         col1 = col1 - 313.0d0/36.0d0*GPL1(1)*pi*ci
     .        + 19.0d0/9.0d0*GPL1(1)*pi**2
     .        - 1.0d0/4.0d0*GPL1(1)*pi**3*ci
     .        + 2.0d0*GPL1(1)*zeta3
     .        + GPL1(1)*v*pi*ci
     .        + 5.0d0/8.0d0*GPL1(1)*v**2*pi*ci
     .        + 7.0d0/8.0d0*GPL1(1)*u*pi*ci
     .        + 5.0d0/4.0d0*GPL1(1)*u*v*pi*ci
     .        + 5.0d0/8.0d0*GPL1(1)*u**2*pi*ci
     .        + 9965.0d0/864.0d0*GPL1(2)
     .        + 5.0d0/4.0d0*GPL1(2)*u**(-2)
     .        - 1.0d0/48.0d0*GPL1(2)*u**(-2)*pi**2
     .        - 5.0d0/2.0d0*GPL1(2)*u**(-2)*v
     .        + 1.0d0/24.0d0*GPL1(2)*u**(-2)*v*pi**2
     .        + 5.0d0/4.0d0*GPL1(2)*u**(-2)*v**2
         col1 = col1 - 1.0d0/48.0d0*GPL1(2)*u**(-2)*v**2*pi**2
     .        - 13.0d0/8.0d0*GPL1(2)*u**(-1)
     .        - 1.0d0/8.0d0*GPL1(2)*u**(-1)*pi*ci
     .        - 1.0d0/24.0d0*GPL1(2)*u**(-1)*pi**2
     .        + GPL1(2)*u**(-1)*v
     .        + 1.0d0/4.0d0*GPL1(2)*u**(-1)*v*pi*ci
     .        + 1.0d0/24.0d0*GPL1(2)*u**(-1)*v*pi**2
     .        + 5.0d0/8.0d0*GPL1(2)*u**(-1)*v**2
     .        - 1.0d0/8.0d0*GPL1(2)*u**(-1)*v**2*pi*ci
     .        + 11.0d0/2.0d0*GPL1(2)*pi*ci
     .        - 7.0d0/18.0d0*GPL1(2)*pi**2
     .        + 1.0d0/4.0d0*GPL1(2)*pi**3*ci
     .        - 3.0d0/4.0d0*GPL1(2)*zeta3
     .        - 5.0d0/3.0d0*GPL1(2)*v*omu**(-1)
     .        + 5.0d0/4.0d0*GPL1(2)*v
         col1 = col1 - GPL1(2)*v*pi*ci
     .        - 4.0d0/3.0d0*GPL1(2)*v**2*omu**(-2)
     .        - 5.0d0/8.0d0*GPL1(2)*v**2*pi*ci
     .        + 5.0d0/8.0d0*GPL1(2)*u
     .        - 7.0d0/8.0d0*GPL1(2)*u*pi*ci
     .        - 5.0d0/4.0d0*GPL1(2)*u*v*pi*ci
     .        - 5.0d0/8.0d0*GPL1(2)*u**2*pi*ci
     .        + 1.0d0/8.0d0*GPL1(3)*u**(-1)*pi*ci
     .        - 3.0d0/8.0d0*GPL1(3)*u**(-1)*v*pi*ci
     .        + 1.0d0/8.0d0*GPL1(3)*u**(-1)*v**2*pi*ci
     .        - 635.0d0/144.0d0*GPL1(3)*pi*ci
     .        + 59.0d0/36.0d0*GPL1(3)*pi**2
     .        - 7.0d0/24.0d0*GPL1(3)*pi**3*ci
     .        - 1.0d0/2.0d0*GPL1(3)*zeta3
     .        + GPL1(3)*v*pi*ci
         col1 = col1 + 5.0d0/8.0d0*GPL1(3)*v**2*pi*ci
     .        + 7.0d0/8.0d0*GPL1(3)*u*pi*ci
     .        + 5.0d0/4.0d0*GPL1(3)*u*v*pi*ci
     .        + 5.0d0/8.0d0*GPL1(3)*u**2*pi*ci
     .        - 20.0d0/9.0d0*GPL2(0,0)
     .        - 7.0d0/4.0d0*GPL2(0,0)*pi*ci
     .        + 5.0d0/24.0d0*GPL2(0,0)*pi**2
     .        - 1.0d0/8.0d0*GPL2(0,1)*u**(-2)*pi*ci
     .        + 1.0d0/4.0d0*GPL2(0,1)*u**(-2)*v*pi*ci
     .        - 1.0d0/8.0d0*GPL2(0,1)*u**(-2)*v**2*pi*ci
     .        - 1.0d0/4.0d0*GPL2(0,1)*u**(-1)*pi*ci
     .        + 1.0d0/4.0d0*GPL2(0,1)*u**(-1)*v*pi*ci
     .        + 29.0d0/24.0d0*GPL2(0,1)*pi*ci
     .        + 1.0d0/4.0d0*GPL2(0,1)*pi**2
     .        + 133.0d0/36.0d0*GPL2(0,2)
         col1 = col1 - 1.0d0/8.0d0*GPL2(0,2)*u**(-1)
     .        + 1.0d0/4.0d0*GPL2(0,2)*u**(-1)*v
     .        - 1.0d0/8.0d0*GPL2(0,2)*u**(-1)*v**2
     .        - 35.0d0/24.0d0*GPL2(0,2)*pi*ci
     .        + 3.0d0/8.0d0*GPL2(0,2)*pi**2
     .        - GPL2(0,2)*v
     .        - 5.0d0/8.0d0*GPL2(0,2)*v**2
     .        - 7.0d0/8.0d0*GPL2(0,2)*u
     .        - 5.0d0/4.0d0*GPL2(0,2)*u*v
     .        - 5.0d0/8.0d0*GPL2(0,2)*u**2
     .        + 4.0d0/3.0d0*GPL2(0,3)*pi*ci
     .        + 1.0d0/12.0d0*GPL2(0,3)*pi**2
     .        + 5.0d0/6.0d0*GPL2(1,0)*pi*ci
     .        - 1.0d0/2.0d0*GPL2(1,0)*pi**2
     .        - 1.0d0/4.0d0*GPL2(1,1)*u**(-2)*pi*ci
         col1 = col1 - 1.0d0/2.0d0*GPL2(1,1)*u**(-1)*pi*ci
     .        - 13.0d0/12.0d0*GPL2(1,1)*pi*ci
     .        + 1.0d0/3.0d0*GPL2(1,1)*pi**2
     .        - 313.0d0/36.0d0*GPL2(1,2)
     .        + 5.0d0/4.0d0*GPL2(1,2)*u**(-3)
     .        - 5.0d0/2.0d0*GPL2(1,2)*u**(-3)*v
     .        + 5.0d0/4.0d0*GPL2(1,2)*u**(-3)*v**2
     .        - 5.0d0/3.0d0*GPL2(1,2)*u**(-2)
     .        + GPL2(1,2)*u**(-2)*v
     .        + 2.0d0/3.0d0*GPL2(1,2)*u**(-2)*v**2
     .        + 5.0d0/3.0d0*GPL2(1,2)*u**(-1)
     .        - 5.0d0/6.0d0*GPL2(1,2)*u**(-1)*v
     .        + 1.0d0/8.0d0*GPL2(1,2)*u**(-1)*v**2
     .        + 1.0d0/4.0d0*GPL2(1,2)*pi*ci
     .        - 5.0d0/12.0d0*GPL2(1,2)*pi**2
         col1 = col1 + GPL2(1,2)*v
     .        + 5.0d0/8.0d0*GPL2(1,2)*v**2
     .        + 7.0d0/8.0d0*GPL2(1,2)*u
     .        + 5.0d0/4.0d0*GPL2(1,2)*u*v
     .        + 5.0d0/8.0d0*GPL2(1,2)*u**2
     .        + 133.0d0/36.0d0*GPL2(2,0)
     .        - 1.0d0/8.0d0*GPL2(2,0)*u**(-1)
     .        + 1.0d0/4.0d0*GPL2(2,0)*u**(-1)*v
     .        - 1.0d0/8.0d0*GPL2(2,0)*u**(-1)*v**2
     .        - 35.0d0/24.0d0*GPL2(2,0)*pi*ci
     .        + 5.0d0/12.0d0*GPL2(2,0)*pi**2
     .        - GPL2(2,0)*v
     .        - 5.0d0/8.0d0*GPL2(2,0)*v**2
     .        - 7.0d0/8.0d0*GPL2(2,0)*u
     .        - 5.0d0/4.0d0*GPL2(2,0)*u*v
         col1 = col1 - 5.0d0/8.0d0*GPL2(2,0)*u**2
     .        + 1.0d0/8.0d0*GPL2(2,1)*u**(-2)*pi*ci
     .        - 1.0d0/4.0d0*GPL2(2,1)*u**(-2)*v*pi*ci
     .        + 1.0d0/8.0d0*GPL2(2,1)*u**(-2)*v**2*pi*ci
     .        + 1.0d0/4.0d0*GPL2(2,1)*u**(-1)*pi*ci
     .        - 1.0d0/4.0d0*GPL2(2,1)*u**(-1)*v*pi*ci
     .        + 35.0d0/8.0d0*GPL2(2,1)*pi*ci
     .        + 1.0d0/6.0d0*GPL2(2,1)*pi**2
     .        - 455.0d0/144.0d0*GPL2(2,2)
     .        - 2.0d0*GPL2(2,2)*pi*ci
     .        - 1.0d0/12.0d0*GPL2(2,2)*pi**2
     .        + 13.0d0/12.0d0*GPL2(2,3)*pi*ci
     .        + 1.0d0/6.0d0*GPL2(2,3)*pi**2
     .        - 635.0d0/144.0d0*GPL2(3,0)
     .        + 1.0d0/8.0d0*GPL2(3,0)*u**(-1)
         col1 = col1 - 3.0d0/8.0d0*GPL2(3,0)*u**(-1)*v
     .        + 1.0d0/8.0d0*GPL2(3,0)*u**(-1)*v**2
     .        - 11.0d0/12.0d0*GPL2(3,0)*pi*ci
     .        - 13.0d0/24.0d0*GPL2(3,0)*pi**2
     .        + GPL2(3,0)*v
     .        + 5.0d0/8.0d0*GPL2(3,0)*v**2
     .        + 7.0d0/8.0d0*GPL2(3,0)*u
     .        + 5.0d0/4.0d0*GPL2(3,0)*u*v
     .        + 5.0d0/8.0d0*GPL2(3,0)*u**2
     .        - 3.0d0/8.0d0*GPL2(3,2)*pi*ci
     .        - 1.0d0/3.0d0*GPL2(3,2)*pi**2
     .        + 11.0d0/12.0d0*GPL2(3,3)*pi*ci
     .        + 1.0d0/6.0d0*GPL2(3,3)*pi**2
     .        - 7.0d0/4.0d0*GPL3(0,0,2)
     .        - 1.0d0/2.0d0*GPL3(0,0,2)*pi*ci
         col1 = col1 - 3.0d0/4.0d0*GPL3(0,1,0)*pi*ci
     .        + 29.0d0/24.0d0*GPL3(0,1,2)
     .        - 1.0d0/8.0d0*GPL3(0,1,2)*u**(-2)
     .        + 1.0d0/4.0d0*GPL3(0,1,2)*u**(-2)*v
     .        - 1.0d0/8.0d0*GPL3(0,1,2)*u**(-2)*v**2
     .        - 1.0d0/4.0d0*GPL3(0,1,2)*u**(-1)
     .        + 1.0d0/4.0d0*GPL3(0,1,2)*u**(-1)*v
     .        + 1.0d0/4.0d0*GPL3(0,1,2)*pi*ci
     .        - 7.0d0/4.0d0*GPL3(0,2,0)
     .        - 1.0d0/2.0d0*GPL3(0,2,0)*pi*ci
     .        + 1.0d0/4.0d0*GPL3(0,2,1)*pi*ci
     .        - 2.0d0*GPL3(0,2,2)
     .        - 1.0d0/2.0d0*GPL3(0,2,2)*pi*ci
     .        + 1.0d0/2.0d0*GPL3(0,2,3)*pi*ci
     .        + 4.0d0/3.0d0*GPL3(0,3,0)
         col1 = col1 - 1.0d0/2.0d0*GPL3(0,3,3)*pi*ci
     .        + 5.0d0/6.0d0*GPL3(1,0,2)
     .        + GPL3(1,0,2)*pi*ci
     .        + 2.0d0*GPL3(1,1,1)*pi*ci
     .        - 13.0d0/12.0d0*GPL3(1,1,2)
     .        - 1.0d0/4.0d0*GPL3(1,1,2)*u**(-2)
     .        - 1.0d0/2.0d0*GPL3(1,1,2)*u**(-1)
     .        - 2.0d0*GPL3(1,1,2)*pi*ci
     .        + 5.0d0/6.0d0*GPL3(1,2,0)
     .        + GPL3(1,2,0)*pi*ci
     .        - 2.0d0*GPL3(1,2,1)*pi*ci
     .        + 4.0d0*GPL3(1,2,2)
     .        + GPL3(1,2,2)*pi*ci
     .        - 7.0d0/4.0d0*GPL3(2,0,0)
     .        - 1.0d0/2.0d0*GPL3(2,0,0)*pi*ci
         col1 = col1 + 1.0d0/2.0d0*GPL3(2,0,1)*pi*ci
     .        - 2.0d0*GPL3(2,0,2)
     .        - 1.0d0/2.0d0*GPL3(2,0,2)*pi*ci
     .        + 1.0d0/2.0d0*GPL3(2,0,3)*pi*ci
     .        + 1.0d0/2.0d0*GPL3(2,1,0)*pi*ci
     .        - 2.0d0*GPL3(2,1,1)*pi*ci
     .        + 35.0d0/8.0d0*GPL3(2,1,2)
     .        + 1.0d0/8.0d0*GPL3(2,1,2)*u**(-2)
     .        - 1.0d0/4.0d0*GPL3(2,1,2)*u**(-2)*v
     .        + 1.0d0/8.0d0*GPL3(2,1,2)*u**(-2)*v**2
     .        + 1.0d0/4.0d0*GPL3(2,1,2)*u**(-1)
     .        - 1.0d0/4.0d0*GPL3(2,1,2)*u**(-1)*v
     .        + 1.0d0/2.0d0*GPL3(2,1,2)*pi*ci
     .        - 2.0d0*GPL3(2,2,0)
     .        - 1.0d0/2.0d0*GPL3(2,2,0)*pi*ci
         col1 = col1 + GPL3(2,2,1)*pi*ci
     .        - GPL3(2,2,3)*pi*ci
     .        + 13.0d0/12.0d0*GPL3(2,3,0)
     .        + 1.0d0/4.0d0*GPL3(2,3,0)*pi*ci
     .        - 1.0d0/2.0d0*GPL3(2,3,2)*pi*ci
     .        - 1.0d0/4.0d0*GPL3(2,3,3)*pi*ci
     .        + 7.0d0/4.0d0*GPL3(3,0,0)
     .        + 1.0d0/2.0d0*GPL3(3,0,0)*pi*ci
     .        + 1.0d0/4.0d0*GPL3(3,0,1)*pi*ci
     .        - 3.0d0/8.0d0*GPL3(3,0,2)
     .        + 1.0d0/2.0d0*GPL3(3,0,2)*pi*ci
     .        - 1.0d0/4.0d0*GPL3(3,0,3)*pi*ci
     .        - 3.0d0/8.0d0*GPL3(3,2,0)
     .        + 1.0d0/2.0d0*GPL3(3,2,0)*pi*ci
     .        + 1.0d0/4.0d0*GPL3(3,2,1)*pi*ci
         col1 = col1 + 1.0d0/4.0d0*GPL3(3,2,3)*pi*ci
     .        + 11.0d0/12.0d0*GPL3(3,3,0)
     .        - 1.0d0/2.0d0*GPL3(3,3,0)*pi*ci
     .        + 1.0d0/2.0d0*GPL3(3,3,3)*pi*ci
     .        - 1.0d0/2.0d0*GPL4(0,0,2,2)
     .        - 3.0d0/4.0d0*GPL4(0,1,0,2)
     .        - 3.0d0/4.0d0*GPL4(0,1,2,0)
     .        + GPL4(0,1,2,2)
     .        - 1.0d0/2.0d0*GPL4(0,2,0,2)
     .        + 1.0d0/4.0d0*GPL4(0,2,1,2)
     .        - 1.0d0/2.0d0*GPL4(0,2,2,0)
     .        + 1.0d0/2.0d0*GPL4(0,2,3,0)
     .        - 1.0d0/2.0d0*GPL4(0,3,3,0)
     .        + GPL4(1,0,2,2)
     .        + 2.0d0*GPL4(1,1,1,2)
         col1 = col1 - 2.0d0*GPL4(1,1,2,2)
     .        + GPL4(1,2,0,2)
     .        - 2.0d0*GPL4(1,2,1,2)
     .        + GPL4(1,2,2,0)
     .        - 1.0d0/2.0d0*GPL4(2,0,0,2)
     .        + 1.0d0/2.0d0*GPL4(2,0,1,2)
     .        - 1.0d0/2.0d0*GPL4(2,0,2,0)
     .        + 1.0d0/2.0d0*GPL4(2,0,3,0)
     .        + 1.0d0/2.0d0*GPL4(2,1,0,2)
     .        - 2.0d0*GPL4(2,1,1,2)
     .        + 1.0d0/2.0d0*GPL4(2,1,2,0)
     .        - 1.0d0/2.0d0*GPL4(2,2,0,0)
     .        + GPL4(2,2,1,2)
     .        - GPL4(2,2,3,0)
     .        + 3.0d0/4.0d0*GPL4(2,3,0,0)
         col1 = col1 - 1.0d0/2.0d0*GPL4(2,3,0,2)
     .        - 1.0d0/2.0d0*GPL4(2,3,2,0)
     .        - 1.0d0/4.0d0*GPL4(2,3,3,0)
     .        + 1.0d0/2.0d0*GPL4(3,0,0,2)
     .        + 1.0d0/4.0d0*GPL4(3,0,1,2)
     .        + 1.0d0/2.0d0*GPL4(3,0,2,0)
     .        - 1.0d0/4.0d0*GPL4(3,0,3,0)
     .        + 1.0d0/2.0d0*GPL4(3,2,0,0)
     .        + 1.0d0/4.0d0*GPL4(3,2,1,2)
     .        + 1.0d0/4.0d0*GPL4(3,2,3,0)
     .        - 1.0d0/2.0d0*GPL4(3,3,0,0)
     .        + 1.0d0/2.0d0*GPL4(3,3,3,0)

         col2 = - 48443.0d0/2592.0d0
     .        + 1.0d0/8.0d0*u**(-2)*pi*ci
     .        - 1.0d0/4.0d0*u**(-2)*v*pi*ci
     .        + 1.0d0/8.0d0*u**(-2)*v**2*pi*ci
     .        + 139.0d0/36.0d0*u**(-1)
     .        - 1.0d0/48.0d0*u**(-1)*pi*ci
     .        + 1.0d0/48.0d0*u**(-1)*pi**2
     .        - 139.0d0/36.0d0*u**(-1)*v
     .        - 1.0d0/24.0d0*u**(-1)*v*pi*ci
     .        + 1.0d0/16.0d0*u**(-1)*v*pi**2
     .        + 1.0d0/16.0d0*u**(-1)*v**2*pi*ci
     .        + 1.0d0/16.0d0*omumv**(-1)*pi**2
     .        - 1.0d0/16.0d0*upv**(-1)*pi**2
     .        - 67.0d0/48.0d0*pi*ci
     .        + 139.0d0/432.0d0*pi**2
         col2 = col2 + 1.0d0/8.0d0*pi**3*ci
     .        - 209.0d0/2880.0d0*pi**4
     .        + 683.0d0/144.0d0*zeta3
     .        - 1.0d0/4.0d0*zeta3*pi*ci
     .        + 1.0d0/16.0d0*v*pi*ci
     .        - 1.0d0/4.0d0*v*pi**2
     .        + 1.0d0/24.0d0*v**2*pi**2
     .        - 1.0d0/4.0d0*u*pi**2
     .        + 1.0d0/12.0d0*u*v*pi**2
     .        + 1.0d0/24.0d0*u**2*pi**2
     .        - 4291.0d0/432.0d0*HPL1(0)
     .        + 1.0d0/8.0d0*HPL1(0)*u**(-2)*v
     .        - 1.0d0/8.0d0*HPL1(0)*u**(-2)*v**2
     .        + 55.0d0/48.0d0*HPL1(0)*u**(-1)
     .        + 1.0d0/8.0d0*HPL1(0)*u**(-1)*pi*ci
         col2 = col2 - 3.0d0/2.0d0*HPL1(0)*u**(-1)*v
     .        - 1.0d0/16.0d0*HPL1(0)*u**(-1)*v**2
     .        + 3.0d0/8.0d0*HPL1(0)*omumv**(-1)*pi*ci
     .        - 1.0d0/8.0d0*HPL1(0)*upv**(-1)*pi*ci
     .        + 187.0d0/72.0d0*HPL1(0)*pi*ci
     .        - 143.0d0/288.0d0*HPL1(0)*pi**2
     .        - 1.0d0/8.0d0*HPL1(0)*pi**3*ci
     .        + 1.0d0/4.0d0*HPL1(0)*zeta3
     .        + 2.0d0/3.0d0*HPL1(0)*v*omv**(-1)
     .        - 1.0d0/16.0d0*HPL1(0)*v
     .        - 3.0d0/8.0d0*HPL1(0)*v*pi*ci
     .        + 1.0d0/16.0d0*HPL1(0)*v**2*pi*ci
     .        - 3.0d0/8.0d0*HPL1(0)*u*pi*ci
     .        + 1.0d0/8.0d0*HPL1(0)*u*v*pi*ci
     .        + 1.0d0/16.0d0*HPL1(0)*u**2*pi*ci
         col2 = col2 + 187.0d0/72.0d0*HPL1(0)*GPL1(0)
     .        + 3.0d0/8.0d0*HPL1(0)*GPL1(0)*omumv**(-1)
     .        + 1.0d0/6.0d0*HPL1(0)*GPL1(0)*pi*ci
     .        - 1.0d0/8.0d0*HPL1(0)*GPL1(0)*pi**2
     .        - 3.0d0/8.0d0*HPL1(0)*GPL1(0)*v
     .        + 1.0d0/16.0d0*HPL1(0)*GPL1(0)*v**2
     .        - 3.0d0/8.0d0*HPL1(0)*GPL1(0)*u
     .        + 1.0d0/8.0d0*HPL1(0)*GPL1(0)*u*v
     .        + 1.0d0/16.0d0*HPL1(0)*GPL1(0)*u**2
     .        + 3.0d0/2.0d0*HPL1(0)*GPL1(1)
     .        - 1.0d0/8.0d0*HPL1(0)*GPL1(1)*u**(-3)
     .        + 1.0d0/4.0d0*HPL1(0)*GPL1(1)*u**(-3)*v
     .        - 1.0d0/8.0d0*HPL1(0)*GPL1(1)*u**(-3)*v**2
     .        + 5.0d0/16.0d0*HPL1(0)*GPL1(1)*u**(-2)
     .        + 1.0d0/8.0d0*HPL1(0)*GPL1(1)*u**(-2)*pi*ci
         col2 = col2 + 1.0d0/8.0d0*HPL1(0)*GPL1(1)*u**(-2)*v
     .        - 7.0d0/16.0d0*HPL1(0)*GPL1(1)*u**(-2)*v**2
     .        + 1.0d0/8.0d0*HPL1(0)*GPL1(1)*u**(-1)*v**(-1)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL1(1)*u**(-1)*pi*ci
     .        - 5.0d0/8.0d0*HPL1(0)*GPL1(1)*u**(-1)*v
     .        - 1.0d0/8.0d0*HPL1(0)*GPL1(1)*v**(-1)*upv**(-1)
     .        - 1.0d0/8.0d0*HPL1(0)*GPL1(1)*upv**(-1)
     .        + 3.0d0/8.0d0*HPL1(0)*GPL1(1)*pi*ci
     .        + 3.0d0/8.0d0*HPL1(0)*GPL1(1)*v
     .        - 1.0d0/16.0d0*HPL1(0)*GPL1(1)*v**2
     .        + 3.0d0/8.0d0*HPL1(0)*GPL1(1)*u
     .        - 1.0d0/8.0d0*HPL1(0)*GPL1(1)*u*v
     .        - 1.0d0/16.0d0*HPL1(0)*GPL1(1)*u**2
     .        - 3.0d0/2.0d0*HPL1(0)*GPL1(2)
     .        + 1.0d0/8.0d0*HPL1(0)*GPL1(2)*u**(-3)
         col2 = col2 - 1.0d0/4.0d0*HPL1(0)*GPL1(2)*u**(-3)*v
     .        + 1.0d0/8.0d0*HPL1(0)*GPL1(2)*u**(-3)*v**2
     .        - 5.0d0/16.0d0*HPL1(0)*GPL1(2)*u**(-2)
     .        - 1.0d0/8.0d0*HPL1(0)*GPL1(2)*u**(-2)*v
     .        + 7.0d0/16.0d0*HPL1(0)*GPL1(2)*u**(-2)*v**2
     .        - 1.0d0/8.0d0*HPL1(0)*GPL1(2)*u**(-1)*v**(-1)
     .        + 1.0d0/8.0d0*HPL1(0)*GPL1(2)*u**(-1)
     .        + 5.0d0/8.0d0*HPL1(0)*GPL1(2)*u**(-1)*v
     .        + 1.0d0/8.0d0*HPL1(0)*GPL1(2)*v**(-1)*upv**(-1)
     .        - 1.0d0/8.0d0*HPL1(0)*GPL1(2)*upv**(-2)*pi*ci
     .        + 1.0d0/2.0d0*HPL1(0)*GPL1(2)*upv**(-1)*pi*ci
     .        - 3.0d0/8.0d0*HPL1(0)*GPL1(2)*v
     .        + 1.0d0/16.0d0*HPL1(0)*GPL1(2)*v**2
     .        - 3.0d0/8.0d0*HPL1(0)*GPL1(2)*u
     .        + 1.0d0/8.0d0*HPL1(0)*GPL1(2)*u*v
         col2 = col2 + 1.0d0/16.0d0*HPL1(0)*GPL1(2)*u**2
     .        - 187.0d0/72.0d0*HPL1(0)*GPL1(3)
     .        - 3.0d0/8.0d0*HPL1(0)*GPL1(3)*omumv**(-1)
     .        - 1.0d0/8.0d0*HPL1(0)*GPL1(3)*pi*ci
     .        - 1.0d0/8.0d0*HPL1(0)*GPL1(3)*pi**2
     .        + 3.0d0/8.0d0*HPL1(0)*GPL1(3)*v
     .        - 1.0d0/16.0d0*HPL1(0)*GPL1(3)*v**2
     .        + 3.0d0/8.0d0*HPL1(0)*GPL1(3)*u
     .        - 1.0d0/8.0d0*HPL1(0)*GPL1(3)*u*v
     .        - 1.0d0/16.0d0*HPL1(0)*GPL1(3)*u**2
     .        - 7.0d0/4.0d0*HPL1(0)*GPL2(0,0)
     .        - 9.0d0/8.0d0*HPL1(0)*GPL2(0,1)
     .        + 3.0d0/4.0d0*HPL1(0)*GPL2(0,1)*u**(-1)
     .        - 3.0d0/4.0d0*HPL1(0)*GPL2(0,1)*u**(-1)*v
     .        - 1.0d0/8.0d0*HPL1(0)*GPL2(0,1)*upv**(-2)
         col2 = col2 + 1.0d0/2.0d0*HPL1(0)*GPL2(0,1)*upv**(-1)
     .        - 1.0d0/4.0d0*HPL1(0)*GPL2(0,1)*pi*ci
     .        + 3.0d0/4.0d0*HPL1(0)*GPL2(0,2)
     .        - 3.0d0/4.0d0*HPL1(0)*GPL2(0,2)*u**(-1)
     .        + 3.0d0/4.0d0*HPL1(0)*GPL2(0,2)*u**(-1)*v
     .        + 1.0d0/8.0d0*HPL1(0)*GPL2(0,2)*upv**(-2)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(0,2)*upv**(-1)
     .        + 4.0d0/3.0d0*HPL1(0)*GPL2(0,3)
     .        + 3.0d0/4.0d0*HPL1(0)*GPL2(0,3)*pi*ci
     .        + 3.0d0/4.0d0*HPL1(0)*GPL2(1,1)
     .        + 1.0d0/4.0d0*HPL1(0)*GPL2(1,1)*u**(-2)
     .        - HPL1(0)*GPL2(1,1)*u**(-1)
     .        - 3.0d0/8.0d0*HPL1(0)*GPL2(1,2)
     .        - 1.0d0/8.0d0*HPL1(0)*GPL2(1,2)*u**(-2)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(1,2)*u**(-1)
         col2 = col2 + 3.0d0/8.0d0*HPL1(0)*GPL2(2,0)
     .        - 3.0d0/8.0d0*HPL1(0)*GPL2(2,1)
     .        - 1.0d0/8.0d0*HPL1(0)*GPL2(2,1)*upv**(-2)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(2,1)*upv**(-1)
     .        - 3.0d0/8.0d0*HPL1(0)*GPL2(2,3)
     .        + 43.0d0/24.0d0*HPL1(0)*GPL2(3,0)
     .        + 3.0d0/4.0d0*HPL1(0)*GPL2(3,0)*pi*ci
     .        + 3.0d0/8.0d0*HPL1(0)*GPL2(3,2)
     .        + 11.0d0/12.0d0*HPL1(0)*GPL2(3,3)
     .        - HPL1(0)*GPL2(3,3)*pi*ci
     .        - 1.0d0/2.0d0*HPL1(0)*GPL3(0,0,1)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(0,0,2)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(0,0,3)
     .        - 1.0d0/4.0d0*HPL1(0)*GPL3(0,1,0)
     .        + 1.0d0/4.0d0*HPL1(0)*GPL3(0,2,0)
         col2 = col2 - 1.0d0/4.0d0*HPL1(0)*GPL3(0,2,1)
     .        + 1.0d0/4.0d0*HPL1(0)*GPL3(0,2,3)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(0,3,0)
     .        + 1.0d0/4.0d0*HPL1(0)*GPL3(0,3,2)
     .        - HPL1(0)*GPL3(0,3,3)
     .        + HPL1(0)*GPL3(3,0,0)
     .        + 1.0d0/4.0d0*HPL1(0)*GPL3(3,0,1)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL3(3,0,2)
     .        - HPL1(0)*GPL3(3,0,3)
     .        - 1.0d0/4.0d0*HPL1(0)*GPL3(3,2,0)
     .        + 1.0d0/4.0d0*HPL1(0)*GPL3(3,2,1)
     .        - HPL1(0)*GPL3(3,3,0)
     .        + HPL1(0)*GPL3(3,3,3)
     .        - 2491.0d0/432.0d0*HPL1(1)
     .        - 1.0d0/8.0d0*HPL1(1)*u**(-2)
         col2 = col2 + 1.0d0/4.0d0*HPL1(1)*u**(-2)*v
     .        - 1.0d0/8.0d0*HPL1(1)*u**(-2)*v**2
     .        + 1.0d0/2.0d0*HPL1(1)*u**(-1)
     .        - 1.0d0/8.0d0*HPL1(1)*u**(-1)*pi*ci
     .        - 7.0d0/16.0d0*HPL1(1)*u**(-1)*v
     .        + 1.0d0/8.0d0*HPL1(1)*u**(-1)*v*pi*ci
     .        - 1.0d0/16.0d0*HPL1(1)*u**(-1)*v**2
     .        + 1.0d0/16.0d0*HPL1(1)*upv**(-2)*pi**2
     .        - 1.0d0/4.0d0*HPL1(1)*upv**(-1)*pi**2
     .        - 9.0d0/16.0d0*HPL1(1)*pi*ci
     .        + 47.0d0/288.0d0*HPL1(1)*pi**2
     .        + 1.0d0/12.0d0*HPL1(1)*pi**3*ci
     .        + 3.0d0/2.0d0*HPL1(1)*zeta3
     .        - 1.0d0/8.0d0*HPL1(1)*v
     .        - 3.0d0/8.0d0*HPL1(1)*v*pi*ci
         col2 = col2 + 1.0d0/16.0d0*HPL1(1)*v**2*pi*ci
     .        - 1.0d0/16.0d0*HPL1(1)*u
     .        - 3.0d0/8.0d0*HPL1(1)*u*pi*ci
     .        + 1.0d0/8.0d0*HPL1(1)*u*v*pi*ci
     .        + 1.0d0/16.0d0*HPL1(1)*u**2*pi*ci
     .        - 9.0d0/16.0d0*HPL1(1)*GPL1(0)
     .        - 1.0d0/8.0d0*HPL1(1)*GPL1(0)*u**(-1)
     .        + 1.0d0/8.0d0*HPL1(1)*GPL1(0)*u**(-1)*v
     .        + 1.0d0/24.0d0*HPL1(1)*GPL1(0)*pi**2
     .        - 3.0d0/8.0d0*HPL1(1)*GPL1(0)*v
     .        + 1.0d0/16.0d0*HPL1(1)*GPL1(0)*v**2
     .        - 3.0d0/8.0d0*HPL1(1)*GPL1(0)*u
     .        + 1.0d0/8.0d0*HPL1(1)*GPL1(0)*u*v
     .        + 1.0d0/16.0d0*HPL1(1)*GPL1(0)*u**2
     .        + 3.0d0/2.0d0*HPL1(1)*GPL1(1)
         col2 = col2 - 1.0d0/8.0d0*HPL1(1)*GPL1(1)*u**(-3)
     .        + 1.0d0/4.0d0*HPL1(1)*GPL1(1)*u**(-3)*v
     .        - 1.0d0/8.0d0*HPL1(1)*GPL1(1)*u**(-3)*v**2
     .        + 5.0d0/16.0d0*HPL1(1)*GPL1(1)*u**(-2)
     .        + 1.0d0/8.0d0*HPL1(1)*GPL1(1)*u**(-2)*v
     .        - 7.0d0/16.0d0*HPL1(1)*GPL1(1)*u**(-2)*v**2
     .        + 1.0d0/8.0d0*HPL1(1)*GPL1(1)*u**(-1)*v**(-1)
     .        - 5.0d0/8.0d0*HPL1(1)*GPL1(1)*u**(-1)*v
     .        - 1.0d0/8.0d0*HPL1(1)*GPL1(1)*v**(-1)*upv**(-1)
     .        - 1.0d0/8.0d0*HPL1(1)*GPL1(1)*upv**(-1)
     .        + 3.0d0/8.0d0*HPL1(1)*GPL1(1)*v
     .        - 1.0d0/16.0d0*HPL1(1)*GPL1(1)*v**2
     .        + 3.0d0/8.0d0*HPL1(1)*GPL1(1)*u
     .        - 1.0d0/8.0d0*HPL1(1)*GPL1(1)*u*v
     .        - 1.0d0/16.0d0*HPL1(1)*GPL1(1)*u**2
         col2 = col2 + 3.0d0/8.0d0*HPL1(1)*GPL1(3)*pi*ci
     .        - 1.0d0/8.0d0*HPL1(1)*GPL1(3)*pi**2
     .        - 9.0d0/8.0d0*HPL1(1)*GPL2(0,1)
     .        + 3.0d0/4.0d0*HPL1(1)*GPL2(0,1)*u**(-1)
     .        - 3.0d0/4.0d0*HPL1(1)*GPL2(0,1)*u**(-1)*v
     .        - 1.0d0/8.0d0*HPL1(1)*GPL2(0,1)*upv**(-2)
     .        + 1.0d0/2.0d0*HPL1(1)*GPL2(0,1)*upv**(-1)
     .        - 1.0d0/4.0d0*HPL1(1)*GPL2(0,1)*pi*ci
     .        + 1.0d0/4.0d0*HPL1(1)*GPL2(0,3)*pi*ci
     .        + 3.0d0/4.0d0*HPL1(1)*GPL2(1,1)
     .        + 1.0d0/4.0d0*HPL1(1)*GPL2(1,1)*u**(-2)
     .        - HPL1(1)*GPL2(1,1)*u**(-1)
     .        - 3.0d0/8.0d0*HPL1(1)*GPL2(2,1)
     .        - 1.0d0/8.0d0*HPL1(1)*GPL2(2,1)*upv**(-2)
     .        + 1.0d0/2.0d0*HPL1(1)*GPL2(2,1)*upv**(-1)
         col2 = col2 + 3.0d0/8.0d0*HPL1(1)*GPL2(3,0)
     .        - 1.0d0/2.0d0*HPL1(1)*GPL3(0,0,1)
     .        - 1.0d0/4.0d0*HPL1(1)*GPL3(0,1,0)
     .        - 1.0d0/4.0d0*HPL1(1)*GPL3(0,2,1)
     .        + 1.0d0/4.0d0*HPL1(1)*GPL3(0,3,0)
     .        + 1.0d0/4.0d0*HPL1(1)*GPL3(3,0,1)
     .        + 1.0d0/4.0d0*HPL1(1)*GPL3(3,2,1)
     .        - 187.0d0/72.0d0*HPL2(0,0)
     .        - 3.0d0/8.0d0*HPL2(0,0)*omumv**(-1)
     .        + 13.0d0/6.0d0*HPL2(0,0)*pi*ci
     .        - 1.0d0/8.0d0*HPL2(0,0)*pi**2
     .        + 3.0d0/8.0d0*HPL2(0,0)*v
     .        - 1.0d0/16.0d0*HPL2(0,0)*v**2
     .        + 3.0d0/8.0d0*HPL2(0,0)*u
     .        - 1.0d0/8.0d0*HPL2(0,0)*u*v
         col2 = col2 - 1.0d0/16.0d0*HPL2(0,0)*u**2
     .        + 49.0d0/12.0d0*HPL2(0,0)*GPL1(0)
     .        + 3.0d0/4.0d0*HPL2(0,0)*GPL1(0)*pi*ci
     .        - 3.0d0/8.0d0*HPL2(0,0)*GPL1(2)
     .        - 11.0d0/6.0d0*HPL2(0,0)*GPL1(3)
     .        - HPL2(0,0)*GPL1(3)*pi*ci
     .        + HPL2(0,0)*GPL2(0,0)
     .        - 1.0d0/4.0d0*HPL2(0,0)*GPL2(0,2)
     .        - HPL2(0,0)*GPL2(0,3)
     .        - 3.0d0/2.0d0*HPL2(0,0)*GPL2(3,0)
     .        + 1.0d0/2.0d0*HPL2(0,0)*GPL2(3,2)
     .        + HPL2(0,0)*GPL2(3,3)
     .        - 1.0d0/8.0d0*HPL2(0,1)*u**(-1)
     .        + 1.0d0/8.0d0*HPL2(0,1)*upv**(-1)
     .        + 3.0d0/8.0d0*HPL2(0,1)*pi*ci
         col2 = col2 - 1.0d0/8.0d0*HPL2(0,1)*pi**2
     .        + 3.0d0/8.0d0*HPL2(0,1)*GPL1(0)
     .        - 3.0d0/8.0d0*HPL2(0,1)*GPL1(1)
     .        - 1.0d0/8.0d0*HPL2(0,1)*GPL1(1)*u**(-2)
     .        + 1.0d0/2.0d0*HPL2(0,1)*GPL1(1)*u**(-1)
     .        + 3.0d0/8.0d0*HPL2(0,1)*GPL1(2)
     .        + 1.0d0/8.0d0*HPL2(0,1)*GPL1(2)*upv**(-2)
     .        - 1.0d0/2.0d0*HPL2(0,1)*GPL1(2)*upv**(-1)
     .        - 3.0d0/8.0d0*HPL2(0,1)*GPL1(3)
     .        + 1.0d0/4.0d0*HPL2(0,1)*GPL1(3)*pi*ci
     .        + 1.0d0/4.0d0*HPL2(0,1)*GPL2(0,2)
     .        - 1.0d0/4.0d0*HPL2(0,1)*GPL2(0,3)
     .        + 1.0d0/4.0d0*HPL2(0,1)*GPL2(3,0)
     .        - 1.0d0/4.0d0*HPL2(0,1)*GPL2(3,2)
     .        + 67.0d0/18.0d0*HPL2(1,0)
         col2 = col2 + 3.0d0/8.0d0*HPL2(1,0)*u**(-1)
     .        - 1.0d0/4.0d0*HPL2(1,0)*u**(-1)*v
     .        + 3.0d0/8.0d0*HPL2(1,0)*omumv**(-1)
     .        + 1.0d0/8.0d0*HPL2(1,0)*upv**(-2)*pi*ci
     .        - 1.0d0/8.0d0*HPL2(1,0)*upv**(-1)
     .        - 1.0d0/2.0d0*HPL2(1,0)*upv**(-1)*pi*ci
     .        + 3.0d0/2.0d0*HPL2(1,0)*pi*ci
     .        - 3.0d0/8.0d0*HPL2(1,0)*pi**2
     .        + 3.0d0/8.0d0*HPL2(1,0)*v
     .        - 1.0d0/16.0d0*HPL2(1,0)*v**2
     .        + 3.0d0/8.0d0*HPL2(1,0)*u
     .        - 1.0d0/8.0d0*HPL2(1,0)*u*v
     .        - 1.0d0/16.0d0*HPL2(1,0)*u**2
     .        - 19.0d0/24.0d0*HPL2(1,0)*GPL1(0)
     .        + 1.0d0/2.0d0*HPL2(1,0)*GPL1(0)*pi*ci
         col2 = col2 + 3.0d0/8.0d0*HPL2(1,0)*GPL1(1)
     .        + 1.0d0/8.0d0*HPL2(1,0)*GPL1(1)*u**(-2)
     .        - 1.0d0/2.0d0*HPL2(1,0)*GPL1(1)*u**(-1)
     .        - 3.0d0/4.0d0*HPL2(1,0)*GPL1(2)
     .        + 3.0d0/4.0d0*HPL2(1,0)*GPL1(2)*u**(-1)
     .        - 3.0d0/4.0d0*HPL2(1,0)*GPL1(2)*u**(-1)*v
     .        - 1.0d0/8.0d0*HPL2(1,0)*GPL1(2)*upv**(-2)
     .        + 1.0d0/2.0d0*HPL2(1,0)*GPL1(2)*upv**(-1)
     .        - 3.0d0/8.0d0*HPL2(1,0)*GPL1(3)
     .        - 1.0d0/4.0d0*HPL2(1,0)*GPL2(0,3)
     .        + 1.0d0/4.0d0*HPL2(1,0)*GPL2(3,0)
     .        + 1.0d0/4.0d0*HPL2(1,0)*GPL2(3,2)
     .        - 1.0d0/8.0d0*HPL2(1,1)*pi**2
     .        - 33.0d0/8.0d0*HPL3(0,0,0)
     .        - HPL3(0,0,0)*pi*ci
         col2 = col2 - 3.0d0/2.0d0*HPL3(0,0,0)*GPL1(0)
     .        + 3.0d0/2.0d0*HPL3(0,0,0)*GPL1(3)
     .        - 3.0d0/8.0d0*HPL3(0,0,1)
     .        + 1.0d0/4.0d0*HPL3(0,0,1)*pi*ci
     .        + 1.0d0/4.0d0*HPL3(0,0,1)*GPL1(0)
     .        - 1.0d0/2.0d0*HPL3(0,0,1)*GPL1(3)
     .        + 23.0d0/12.0d0*HPL3(0,1,0)
     .        + 1.0d0/4.0d0*HPL3(0,1,0)*GPL1(0)
     .        - 1.0d0/2.0d0*HPL3(0,1,0)*GPL1(3)
     .        + 25.0d0/8.0d0*HPL3(1,0,0)
     .        - 1.0d0/4.0d0*HPL3(1,0,0)*pi*ci
     .        + 1.0d0/4.0d0*HPL3(1,0,0)*GPL1(0)
     .        - 1.0d0/2.0d0*HPL3(1,0,0)*GPL1(3)
     .        - 1.0d0/8.0d0*HPL3(1,0,1)*upv**(-2)
     .        + 1.0d0/2.0d0*HPL3(1,0,1)*upv**(-1)
         col2 = col2 - 1.0d0/2.0d0*HPL3(1,0,1)*pi*ci
     .        - 1.0d0/2.0d0*HPL3(1,0,1)*GPL1(0)
     .        + 1.0d0/4.0d0*HPL3(1,0,1)*GPL1(3)
     .        + 13.0d0/24.0d0*HPL3(1,1,0)
     .        + 1.0d0/8.0d0*HPL3(1,1,0)*upv**(-2)
     .        - 1.0d0/2.0d0*HPL3(1,1,0)*upv**(-1)
     .        - 1.0d0/4.0d0*HPL3(1,1,0)*GPL1(0)
     .        - 1.0d0/4.0d0*HPL3(1,1,0)*GPL1(3)
     .        + 3.0d0/2.0d0*HPL4(0,0,0,0)
     .        - 1.0d0/2.0d0*HPL4(0,0,0,1)
     .        - 1.0d0/2.0d0*HPL4(0,0,1,0)
     .        - 1.0d0/2.0d0*HPL4(0,1,0,0)
     .        + 1.0d0/4.0d0*HPL4(0,1,0,1)
     .        - 1.0d0/4.0d0*HPL4(0,1,1,0)
     .        - 1.0d0/4.0d0*HPL4(1,0,0,0)
         col2 = col2 + 1.0d0/2.0d0*HPL4(1,0,0,1)
     .        + 3.0d0/4.0d0*HPL4(1,0,1,0)
     .        + 1.0d0/4.0d0*HPL4(1,1,0,0)
     .        - 1.0d0/4.0d0*HPL4(1,1,0,1)
     .        - HPL4(1,1,1,0)
     .        + 133.0d0/48.0d0*GPL1(0)
     .        - 2.0d0/3.0d0*GPL1(0)*u**(-1)
     .        + 1.0d0/8.0d0*GPL1(0)*u**(-1)*pi*ci
     .        + 2.0d0/3.0d0*GPL1(0)*u**(-1)*v
     .        - 1.0d0/8.0d0*GPL1(0)*u**(-1)*v*pi*ci
     .        + 9.0d0/16.0d0*GPL1(0)*pi*ci
     .        - 7.0d0/36.0d0*GPL1(0)*pi**2
     .        + 1.0d0/12.0d0*GPL1(0)*pi**3*ci
     .        - 1.0d0/16.0d0*GPL1(0)*v
     .        + 3.0d0/8.0d0*GPL1(0)*v*pi*ci
         col2 = col2 - 1.0d0/16.0d0*GPL1(0)*v**2*pi*ci
     .        - 1.0d0/16.0d0*GPL1(0)*u
     .        + 3.0d0/8.0d0*GPL1(0)*u*pi*ci
     .        - 1.0d0/8.0d0*GPL1(0)*u*v*pi*ci
     .        - 1.0d0/16.0d0*GPL1(0)*u**2*pi*ci
     .        + 1.0d0/8.0d0*GPL1(1)*u**(-3)*pi*ci
     .        - 1.0d0/4.0d0*GPL1(1)*u**(-3)*v*pi*ci
     .        + 1.0d0/8.0d0*GPL1(1)*u**(-3)*v**2*pi*ci
     .        - 5.0d0/16.0d0*GPL1(1)*u**(-2)*pi*ci
     .        + 1.0d0/12.0d0*GPL1(1)*u**(-2)*pi**2
     .        - 1.0d0/8.0d0*GPL1(1)*u**(-2)*v*pi*ci
     .        + 7.0d0/16.0d0*GPL1(1)*u**(-2)*v**2*pi*ci
     .        - 1.0d0/8.0d0*GPL1(1)*u**(-1)*v**(-1)*pi*ci
     .        - 1.0d0/3.0d0*GPL1(1)*u**(-1)*pi**2
     .        + 5.0d0/8.0d0*GPL1(1)*u**(-1)*v*pi*ci
         col2 = col2 + 1.0d0/8.0d0*GPL1(1)*v**(-1)*upv**(-1)*pi*ci
     .        + 1.0d0/8.0d0*GPL1(1)*upv**(-1)*pi*ci
     .        - 3.0d0/2.0d0*GPL1(1)*pi*ci
     .        + 1.0d0/4.0d0*GPL1(1)*pi**2
     .        - 3.0d0/8.0d0*GPL1(1)*v*pi*ci
     .        + 1.0d0/16.0d0*GPL1(1)*v**2*pi*ci
     .        - 3.0d0/8.0d0*GPL1(1)*u*pi*ci
     .        + 1.0d0/8.0d0*GPL1(1)*u*v*pi*ci
     .        + 1.0d0/16.0d0*GPL1(1)*u**2*pi*ci
     .        + 2491.0d0/432.0d0*GPL1(2)
     .        + 1.0d0/8.0d0*GPL1(2)*u**(-2)
     .        - 1.0d0/4.0d0*GPL1(2)*u**(-2)*v
     .        + 1.0d0/8.0d0*GPL1(2)*u**(-2)*v**2
     .        - 1.0d0/2.0d0*GPL1(2)*u**(-1)
     .        + 1.0d0/8.0d0*GPL1(2)*u**(-1)*pi*ci
         col2 = col2 + 1.0d0/8.0d0*GPL1(2)*u**(-1)*pi**2
     .        + 7.0d0/16.0d0*GPL1(2)*u**(-1)*v
     .        - 1.0d0/8.0d0*GPL1(2)*u**(-1)*v*pi*ci
     .        - 1.0d0/8.0d0*GPL1(2)*u**(-1)*v*pi**2
     .        + 1.0d0/16.0d0*GPL1(2)*u**(-1)*v**2
     .        - 1.0d0/16.0d0*GPL1(2)*upv**(-2)*pi**2
     .        + 1.0d0/4.0d0*GPL1(2)*upv**(-1)*pi**2
     .        + 9.0d0/16.0d0*GPL1(2)*pi*ci
     .        - 25.0d0/96.0d0*GPL1(2)*pi**2
     .        - 1.0d0/4.0d0*GPL1(2)*zeta3
     .        + 1.0d0/8.0d0*GPL1(2)*v
     .        + 3.0d0/8.0d0*GPL1(2)*v*pi*ci
     .        - 1.0d0/16.0d0*GPL1(2)*v**2*pi*ci
     .        + 1.0d0/16.0d0*GPL1(2)*u
     .        + 3.0d0/8.0d0*GPL1(2)*u*pi*ci
         col2 = col2 - 1.0d0/8.0d0*GPL1(2)*u*v*pi*ci
     .        - 1.0d0/16.0d0*GPL1(2)*u**2*pi*ci
     .        + 3.0d0/8.0d0*GPL1(3)*omumv**(-1)*pi*ci
     .        + 187.0d0/72.0d0*GPL1(3)*pi*ci
     .        - 8.0d0/9.0d0*GPL1(3)*pi**2
     .        - 1.0d0/8.0d0*GPL1(3)*pi**3*ci
     .        - 3.0d0/8.0d0*GPL1(3)*v*pi*ci
     .        + 1.0d0/16.0d0*GPL1(3)*v**2*pi*ci
     .        - 3.0d0/8.0d0*GPL1(3)*u*pi*ci
     .        + 1.0d0/8.0d0*GPL1(3)*u*v*pi*ci
     .        + 1.0d0/16.0d0*GPL1(3)*u**2*pi*ci
     .        - 3.0d0/4.0d0*GPL2(0,1)*u**(-1)*pi*ci
     .        + 3.0d0/4.0d0*GPL2(0,1)*u**(-1)*v*pi*ci
     .        + 1.0d0/8.0d0*GPL2(0,1)*upv**(-2)*pi*ci
     .        - 1.0d0/2.0d0*GPL2(0,1)*upv**(-1)*pi*ci
         col2 = col2 + 9.0d0/8.0d0*GPL2(0,1)*pi*ci
     .        - 1.0d0/4.0d0*GPL2(0,1)*pi**2
     .        + 9.0d0/16.0d0*GPL2(0,2)
     .        + 1.0d0/8.0d0*GPL2(0,2)*u**(-1)
     .        - 1.0d0/8.0d0*GPL2(0,2)*u**(-1)*v
     .        - 1.0d0/24.0d0*GPL2(0,2)*pi**2
     .        + 3.0d0/8.0d0*GPL2(0,2)*v
     .        - 1.0d0/16.0d0*GPL2(0,2)*v**2
     .        + 3.0d0/8.0d0*GPL2(0,2)*u
     .        - 1.0d0/8.0d0*GPL2(0,2)*u*v
     .        - 1.0d0/16.0d0*GPL2(0,2)*u**2
     .        - 4.0d0/3.0d0*GPL2(0,3)*pi*ci
     .        + 1.0d0/12.0d0*GPL2(0,3)*pi**2
     .        - 1.0d0/4.0d0*GPL2(1,1)*u**(-2)*pi*ci
     .        + GPL2(1,1)*u**(-1)*pi*ci
         col2 = col2 - 3.0d0/4.0d0*GPL2(1,1)*pi*ci
     .        - 3.0d0/2.0d0*GPL2(1,2)
     .        + 1.0d0/8.0d0*GPL2(1,2)*u**(-3)
     .        - 1.0d0/4.0d0*GPL2(1,2)*u**(-3)*v
     .        + 1.0d0/8.0d0*GPL2(1,2)*u**(-3)*v**2
     .        - 5.0d0/16.0d0*GPL2(1,2)*u**(-2)
     .        - 1.0d0/8.0d0*GPL2(1,2)*u**(-2)*v
     .        + 7.0d0/16.0d0*GPL2(1,2)*u**(-2)*v**2
     .        - 1.0d0/8.0d0*GPL2(1,2)*u**(-1)*v**(-1)
     .        + 5.0d0/8.0d0*GPL2(1,2)*u**(-1)*v
     .        + 1.0d0/8.0d0*GPL2(1,2)*v**(-1)*upv**(-1)
     .        + 1.0d0/8.0d0*GPL2(1,2)*upv**(-1)
     .        - 3.0d0/8.0d0*GPL2(1,2)*v
     .        + 1.0d0/16.0d0*GPL2(1,2)*v**2
     .        - 3.0d0/8.0d0*GPL2(1,2)*u
         col2 = col2 + 1.0d0/8.0d0*GPL2(1,2)*u*v
     .        + 1.0d0/16.0d0*GPL2(1,2)*u**2
     .        + 9.0d0/16.0d0*GPL2(2,0)
     .        + 1.0d0/8.0d0*GPL2(2,0)*u**(-1)
     .        - 1.0d0/8.0d0*GPL2(2,0)*u**(-1)*v
     .        + 3.0d0/8.0d0*GPL2(2,0)*v
     .        - 1.0d0/16.0d0*GPL2(2,0)*v**2
     .        + 3.0d0/8.0d0*GPL2(2,0)*u
     .        - 1.0d0/8.0d0*GPL2(2,0)*u*v
     .        - 1.0d0/16.0d0*GPL2(2,0)*u**2
     .        + 1.0d0/8.0d0*GPL2(2,1)*upv**(-2)*pi*ci
     .        - 1.0d0/2.0d0*GPL2(2,1)*upv**(-1)*pi*ci
     .        + 3.0d0/8.0d0*GPL2(2,1)*pi*ci
     .        + 3.0d0/8.0d0*GPL2(2,3)*pi*ci
     .        + 187.0d0/72.0d0*GPL2(3,0)
         col2 = col2 + 3.0d0/8.0d0*GPL2(3,0)*omumv**(-1)
     .        + 1.0d0/6.0d0*GPL2(3,0)*pi*ci
     .        - 1.0d0/8.0d0*GPL2(3,0)*pi**2
     .        - 3.0d0/8.0d0*GPL2(3,0)*v
     .        + 1.0d0/16.0d0*GPL2(3,0)*v**2
     .        - 3.0d0/8.0d0*GPL2(3,0)*u
     .        + 1.0d0/8.0d0*GPL2(3,0)*u*v
     .        + 1.0d0/16.0d0*GPL2(3,0)*u**2
     .        - 3.0d0/8.0d0*GPL2(3,2)*pi*ci
     .        + 1.0d0/8.0d0*GPL2(3,2)*pi**2
     .        - 11.0d0/12.0d0*GPL2(3,3)*pi*ci
     .        - 1.0d0/3.0d0*GPL2(3,3)*pi**2
     .        + 1.0d0/2.0d0*GPL3(0,0,1)*pi*ci
     .        - 1.0d0/2.0d0*GPL3(0,0,3)*pi*ci
     .        + 1.0d0/4.0d0*GPL3(0,1,0)*pi*ci
         col2 = col2 + 9.0d0/8.0d0*GPL3(0,1,2)
     .        - 3.0d0/4.0d0*GPL3(0,1,2)*u**(-1)
     .        + 3.0d0/4.0d0*GPL3(0,1,2)*u**(-1)*v
     .        + 1.0d0/8.0d0*GPL3(0,1,2)*upv**(-2)
     .        - 1.0d0/2.0d0*GPL3(0,1,2)*upv**(-1)
     .        + 1.0d0/4.0d0*GPL3(0,1,2)*pi*ci
     .        + 1.0d0/4.0d0*GPL3(0,2,1)*pi*ci
     .        - 1.0d0/4.0d0*GPL3(0,2,3)*pi*ci
     .        - 4.0d0/3.0d0*GPL3(0,3,0)
     .        - 1.0d0/4.0d0*GPL3(0,3,0)*pi*ci
     .        - 1.0d0/4.0d0*GPL3(0,3,2)*pi*ci
     .        + GPL3(0,3,3)*pi*ci
     .        - 3.0d0/4.0d0*GPL3(1,1,2)
     .        - 1.0d0/4.0d0*GPL3(1,1,2)*u**(-2)
     .        + GPL3(1,1,2)*u**(-1)
         col2 = col2 + 3.0d0/8.0d0*GPL3(2,1,2)
     .        + 1.0d0/8.0d0*GPL3(2,1,2)*upv**(-2)
     .        - 1.0d0/2.0d0*GPL3(2,1,2)*upv**(-1)
     .        + 3.0d0/8.0d0*GPL3(2,3,0)
     .        - 7.0d0/4.0d0*GPL3(3,0,0)
     .        - 1.0d0/4.0d0*GPL3(3,0,1)*pi*ci
     .        - 3.0d0/8.0d0*GPL3(3,0,2)
     .        + GPL3(3,0,3)*pi*ci
     .        - 3.0d0/8.0d0*GPL3(3,2,0)
     .        - 1.0d0/4.0d0*GPL3(3,2,1)*pi*ci
     .        - 11.0d0/12.0d0*GPL3(3,3,0)
     .        + GPL3(3,3,0)*pi*ci
     .        - GPL3(3,3,3)*pi*ci
     .        + 1.0d0/2.0d0*GPL4(0,0,1,2)
     .        - 1.0d0/2.0d0*GPL4(0,0,3,0)
         col2 = col2 + 1.0d0/4.0d0*GPL4(0,1,0,2)
     .        + 1.0d0/4.0d0*GPL4(0,1,2,0)
     .        + 1.0d0/4.0d0*GPL4(0,2,1,2)
     .        - 1.0d0/4.0d0*GPL4(0,2,3,0)
     .        - 1.0d0/4.0d0*GPL4(0,3,0,2)
     .        - 1.0d0/4.0d0*GPL4(0,3,2,0)
     .        + GPL4(0,3,3,0)
     .        - 1.0d0/4.0d0*GPL4(3,0,1,2)
     .        + GPL4(3,0,3,0)
     .        - 1.0d0/4.0d0*GPL4(3,2,1,2)
     .        + GPL4(3,3,0,0)
     .        - GPL4(3,3,3,0)

         col3 = - 335.0d0/128.0d0
     .        - 5.0d0/8.0d0*u**(-2)*pi*ci
     .        + 5.0d0/4.0d0*u**(-2)*v*pi*ci
     .        - 5.0d0/8.0d0*u**(-2)*v**2*pi*ci
     .        + 5.0d0/8.0d0*u**(-1)
     .        + 11.0d0/8.0d0*u**(-1)*pi*ci
     .        - 1.0d0/8.0d0*u**(-1)*pi**2
     .        - 5.0d0/8.0d0*u**(-1)*v
     .        - 17.0d0/16.0d0*u**(-1)*v*pi*ci
     .        + 11.0d0/48.0d0*u**(-1)*v*pi**2
     .        - 5.0d0/16.0d0*u**(-1)*v**2*pi*ci
     .        - 1.0d0/12.0d0*u**(-1)*v**2*pi**2
     .        - 1.0d0/16.0d0*omumv**(-1)*pi**2
     .        - 1.0d0/16.0d0*upv**(-1)*pi**2
     .        - 9.0d0/8.0d0*pi*ci
         col3 = col3 + 55.0d0/96.0d0*pi**2
     .        + 1.0d0/16.0d0*pi**3*ci
     .        - 5.0d0/96.0d0*pi**4
     .        + 3.0d0/2.0d0*zeta3
     .        - 7.0d0/4.0d0*zeta3*pi*ci
     .        - 5.0d0/16.0d0*v*pi*ci
     .        - 5.0d0/12.0d0*v*pi**2
     .        - 5.0d0/24.0d0*v**2*pi**2
     .        - 1.0d0/3.0d0*u*pi**2
     .        - 5.0d0/12.0d0*u*v*pi**2
     .        - 5.0d0/24.0d0*u**2*pi**2
     .        + 3.0d0/32.0d0*HPL1(0)
     .        - 5.0d0/8.0d0*HPL1(0)*u**(-2)*v
     .        + 5.0d0/8.0d0*HPL1(0)*u**(-2)*v**2
     .        + HPL1(0)*u**(-1)*v
         col3 = col3 + 1.0d0/4.0d0*HPL1(0)*u**(-1)*v*pi*ci
     .        + 5.0d0/16.0d0*HPL1(0)*u**(-1)*v**2
     .        - 1.0d0/8.0d0*HPL1(0)*u**(-1)*v**2*pi*ci
     .        - 3.0d0/8.0d0*HPL1(0)*omumv**(-1)*pi*ci
     .        - 1.0d0/8.0d0*HPL1(0)*upv**(-1)*pi*ci
     .        + 29.0d0/16.0d0*HPL1(0)*pi*ci
     .        - 1.0d0/8.0d0*HPL1(0)*pi**2
     .        - 1.0d0/12.0d0*HPL1(0)*pi**3*ci
     .        + 2.0d0*HPL1(0)*zeta3
     .        - 17.0d0/16.0d0*HPL1(0)*v*omv**(-1)
     .        + 5.0d0/16.0d0*HPL1(0)*v
     .        - 5.0d0/8.0d0*HPL1(0)*v*pi*ci
     .        - 5.0d0/16.0d0*HPL1(0)*v**2*pi*ci
     .        - 1.0d0/2.0d0*HPL1(0)*u*pi*ci
     .        - 5.0d0/8.0d0*HPL1(0)*u*v*pi*ci
         col3 = col3 - 5.0d0/16.0d0*HPL1(0)*u**2*pi*ci
     .        + 29.0d0/16.0d0*HPL1(0)*GPL1(0)
     .        - 1.0d0/8.0d0*HPL1(0)*GPL1(0)*u**(-1)
     .        + 3.0d0/8.0d0*HPL1(0)*GPL1(0)*u**(-1)*v
     .        - 1.0d0/8.0d0*HPL1(0)*GPL1(0)*u**(-1)*v**2
     .        - 3.0d0/8.0d0*HPL1(0)*GPL1(0)*omumv**(-1)
     .        - 1.0d0/3.0d0*HPL1(0)*GPL1(0)*pi**2
     .        - 5.0d0/8.0d0*HPL1(0)*GPL1(0)*v
     .        - 5.0d0/16.0d0*HPL1(0)*GPL1(0)*v**2
     .        - 1.0d0/2.0d0*HPL1(0)*GPL1(0)*u
     .        - 5.0d0/8.0d0*HPL1(0)*GPL1(0)*u*v
     .        - 5.0d0/16.0d0*HPL1(0)*GPL1(0)*u**2
     .        - 2.0d0*HPL1(0)*GPL1(1)
     .        + 5.0d0/8.0d0*HPL1(0)*GPL1(1)*u**(-3)
     .        - 5.0d0/4.0d0*HPL1(0)*GPL1(1)*u**(-3)*v
         col3 = col3 + 5.0d0/8.0d0*HPL1(0)*GPL1(1)*u**(-3)*v**2
     .        - 23.0d0/16.0d0*HPL1(0)*GPL1(1)*u**(-2)
     .        + 11.0d0/8.0d0*HPL1(0)*GPL1(1)*u**(-2)*v
     .        + 1.0d0/16.0d0*HPL1(0)*GPL1(1)*u**(-2)*v**2
     .        + 1.0d0/8.0d0*HPL1(0)*GPL1(1)*u**(-1)*v**(-1)
     .        + 2.0d0*HPL1(0)*GPL1(1)*u**(-1)
     .        - 11.0d0/8.0d0*HPL1(0)*GPL1(1)*u**(-1)*v
     .        + 1.0d0/8.0d0*HPL1(0)*GPL1(1)*u**(-1)*v**2
     .        - 1.0d0/8.0d0*HPL1(0)*GPL1(1)*v**(-1)*upv**(-1)
     .        - 1.0d0/8.0d0*HPL1(0)*GPL1(1)*upv**(-1)
     .        + 5.0d0/8.0d0*HPL1(0)*GPL1(1)*v
     .        + 5.0d0/16.0d0*HPL1(0)*GPL1(1)*v**2
     .        + 1.0d0/2.0d0*HPL1(0)*GPL1(1)*u
     .        + 5.0d0/8.0d0*HPL1(0)*GPL1(1)*u*v
     .        + 5.0d0/16.0d0*HPL1(0)*GPL1(1)*u**2
         col3 = col3 + 2.0d0*HPL1(0)*GPL1(2)
     .        - 5.0d0/8.0d0*HPL1(0)*GPL1(2)*u**(-3)
     .        + 5.0d0/4.0d0*HPL1(0)*GPL1(2)*u**(-3)*v
     .        - 5.0d0/8.0d0*HPL1(0)*GPL1(2)*u**(-3)*v**2
     .        + 23.0d0/16.0d0*HPL1(0)*GPL1(2)*u**(-2)
     .        + 1.0d0/8.0d0*HPL1(0)*GPL1(2)*u**(-2)*pi*ci
     .        - 11.0d0/8.0d0*HPL1(0)*GPL1(2)*u**(-2)*v
     .        - 1.0d0/4.0d0*HPL1(0)*GPL1(2)*u**(-2)*v*pi*ci
     .        - 1.0d0/16.0d0*HPL1(0)*GPL1(2)*u**(-2)*v**2
     .        + 1.0d0/8.0d0*HPL1(0)*GPL1(2)*u**(-2)*v**2*pi*ci
     .        - 1.0d0/8.0d0*HPL1(0)*GPL1(2)*u**(-1)*v**(-1)
     .        - 15.0d0/8.0d0*HPL1(0)*GPL1(2)*u**(-1)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL1(2)*u**(-1)*pi*ci
     .        + 5.0d0/4.0d0*HPL1(0)*GPL1(2)*u**(-1)*v
     .        + 1.0d0/2.0d0*HPL1(0)*GPL1(2)*u**(-1)*v*pi*ci
         col3 = col3 - 1.0d0/8.0d0*HPL1(0)*GPL1(2)*u**(-1)*v**2
     .        + 1.0d0/8.0d0*HPL1(0)*GPL1(2)*v**(-1)*upv**(-1)
     .        - 1.0d0/8.0d0*HPL1(0)*GPL1(2)*upv**(-2)*pi*ci
     .        + 1.0d0/2.0d0*HPL1(0)*GPL1(2)*upv**(-1)*pi*ci
     .        - 3.0d0/8.0d0*HPL1(0)*GPL1(2)*pi*ci
     .        + 1.0d0/3.0d0*HPL1(0)*GPL1(2)*pi**2
     .        - 5.0d0/8.0d0*HPL1(0)*GPL1(2)*v
     .        - 5.0d0/16.0d0*HPL1(0)*GPL1(2)*v**2
     .        - 1.0d0/2.0d0*HPL1(0)*GPL1(2)*u
     .        - 5.0d0/8.0d0*HPL1(0)*GPL1(2)*u*v
     .        - 5.0d0/16.0d0*HPL1(0)*GPL1(2)*u**2
     .        - 29.0d0/16.0d0*HPL1(0)*GPL1(3)
     .        + 1.0d0/8.0d0*HPL1(0)*GPL1(3)*u**(-1)
     .        - 3.0d0/8.0d0*HPL1(0)*GPL1(3)*u**(-1)*v
     .        + 1.0d0/8.0d0*HPL1(0)*GPL1(3)*u**(-1)*v**2
         col3 = col3 + 3.0d0/8.0d0*HPL1(0)*GPL1(3)*omumv**(-1)
     .        + 1.0d0/3.0d0*HPL1(0)*GPL1(3)*pi**2
     .        + 5.0d0/8.0d0*HPL1(0)*GPL1(3)*v
     .        + 5.0d0/16.0d0*HPL1(0)*GPL1(3)*v**2
     .        + 1.0d0/2.0d0*HPL1(0)*GPL1(3)*u
     .        + 5.0d0/8.0d0*HPL1(0)*GPL1(3)*u*v
     .        + 5.0d0/16.0d0*HPL1(0)*GPL1(3)*u**2
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(0,0)*pi*ci
     .        - 3.0d0/4.0d0*HPL1(0)*GPL2(0,1)
     .        - 1.0d0/8.0d0*HPL1(0)*GPL2(0,1)*u**(-2)
     .        + 1.0d0/4.0d0*HPL1(0)*GPL2(0,1)*u**(-2)*v
     .        - 1.0d0/8.0d0*HPL1(0)*GPL2(0,1)*u**(-2)*v**2
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(0,1)*u**(-1)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(0,1)*u**(-1)*v
     .        - 1.0d0/8.0d0*HPL1(0)*GPL2(0,1)*upv**(-2)
         col3 = col3 + 1.0d0/2.0d0*HPL1(0)*GPL2(0,1)*upv**(-1)
     .        + 3.0d0/4.0d0*HPL1(0)*GPL2(0,2)
     .        + 1.0d0/8.0d0*HPL1(0)*GPL2(0,2)*u**(-2)
     .        - 1.0d0/4.0d0*HPL1(0)*GPL2(0,2)*u**(-2)*v
     .        + 1.0d0/8.0d0*HPL1(0)*GPL2(0,2)*u**(-2)*v**2
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(0,2)*u**(-1)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(0,2)*u**(-1)*v
     .        + 1.0d0/8.0d0*HPL1(0)*GPL2(0,2)*upv**(-2)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(0,2)*upv**(-1)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(0,2)*pi*ci
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(0,3)*pi*ci
     .        - 3.0d0/8.0d0*HPL1(0)*GPL2(2,0)
     .        - 1.0d0/4.0d0*HPL1(0)*GPL2(2,0)*pi*ci
     .        + 1.0d0/8.0d0*HPL1(0)*GPL2(2,1)*u**(-2)
     .        - 1.0d0/4.0d0*HPL1(0)*GPL2(2,1)*u**(-2)*v
         col3 = col3 + 1.0d0/8.0d0*HPL1(0)*GPL2(2,1)*u**(-2)*v**2
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(2,1)*u**(-1)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(2,1)*u**(-1)*v
     .        - 1.0d0/8.0d0*HPL1(0)*GPL2(2,1)*upv**(-2)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(2,1)*upv**(-1)
     .        + 3.0d0/8.0d0*HPL1(0)*GPL2(2,3)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(2,3)*pi*ci
     .        - 3.0d0/4.0d0*HPL1(0)*GPL2(3,0)*pi*ci
     .        + 1.0d0/2.0d0*HPL1(0)*GPL2(3,3)*pi*ci
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(0,0,1)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL3(0,0,3)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL3(0,2,1)
     .        + 1.0d0/4.0d0*HPL1(0)*GPL3(0,2,3)
     .        - 1.0d0/4.0d0*HPL1(0)*GPL3(0,3,0)
     .        - 1.0d0/4.0d0*HPL1(0)*GPL3(0,3,2)
         col3 = col3 + 1.0d0/2.0d0*HPL1(0)*GPL3(0,3,3)
     .        + 1.0d0/4.0d0*HPL1(0)*GPL3(2,0,0)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL3(2,0,2)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(2,3,2)
     .        - 1.0d0/4.0d0*HPL1(0)*GPL3(2,3,3)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL3(3,0,0)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(3,0,1)
     .        - 3.0d0/4.0d0*HPL1(0)*GPL3(3,0,2)
     .        + 3.0d0/4.0d0*HPL1(0)*GPL3(3,0,3)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL3(3,2,0)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(3,2,1)
     .        + 1.0d0/4.0d0*HPL1(0)*GPL3(3,2,3)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL3(3,3,0)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL3(3,3,3)
     .        + 17.0d0/32.0d0*HPL1(1)
         col3 = col3 + 5.0d0/8.0d0*HPL1(1)*u**(-2)
     .        - 5.0d0/4.0d0*HPL1(1)*u**(-2)*v
     .        + 5.0d0/8.0d0*HPL1(1)*u**(-2)*v**2
     .        - 11.0d0/8.0d0*HPL1(1)*u**(-1)
     .        - 1.0d0/4.0d0*HPL1(1)*u**(-1)*pi*ci
     .        + 17.0d0/16.0d0*HPL1(1)*u**(-1)*v
     .        + 3.0d0/8.0d0*HPL1(1)*u**(-1)*v*pi*ci
     .        + 5.0d0/16.0d0*HPL1(1)*u**(-1)*v**2
     .        - 1.0d0/8.0d0*HPL1(1)*u**(-1)*v**2*pi*ci
     .        + 1.0d0/16.0d0*HPL1(1)*upv**(-2)*pi**2
     .        - 1.0d0/4.0d0*HPL1(1)*upv**(-1)*pi**2
     .        + 17.0d0/16.0d0*HPL1(1)*pi*ci
     .        + 3.0d0/16.0d0*HPL1(1)*pi**2
     .        + 1.0d0/4.0d0*HPL1(1)*pi**3*ci
     .        + 3.0d0/4.0d0*HPL1(1)*zeta3
         col3 = col3 + 5.0d0/8.0d0*HPL1(1)*v
     .        - 5.0d0/8.0d0*HPL1(1)*v*pi*ci
     .        - 5.0d0/16.0d0*HPL1(1)*v**2*pi*ci
     .        + 5.0d0/16.0d0*HPL1(1)*u
     .        - 1.0d0/2.0d0*HPL1(1)*u*pi*ci
     .        - 5.0d0/8.0d0*HPL1(1)*u*v*pi*ci
     .        - 5.0d0/16.0d0*HPL1(1)*u**2*pi*ci
     .        + 17.0d0/16.0d0*HPL1(1)*GPL1(0)
     .        - 1.0d0/4.0d0*HPL1(1)*GPL1(0)*u**(-1)
     .        + 3.0d0/8.0d0*HPL1(1)*GPL1(0)*u**(-1)*v
     .        - 1.0d0/8.0d0*HPL1(1)*GPL1(0)*u**(-1)*v**2
     .        + 1.0d0/12.0d0*HPL1(1)*GPL1(0)*pi**2
     .        - 5.0d0/8.0d0*HPL1(1)*GPL1(0)*v
     .        - 5.0d0/16.0d0*HPL1(1)*GPL1(0)*v**2
     .        - 1.0d0/2.0d0*HPL1(1)*GPL1(0)*u
         col3 = col3 - 5.0d0/8.0d0*HPL1(1)*GPL1(0)*u*v
     .        - 5.0d0/16.0d0*HPL1(1)*GPL1(0)*u**2
     .        - 2.0d0*HPL1(1)*GPL1(1)
     .        + 5.0d0/8.0d0*HPL1(1)*GPL1(1)*u**(-3)
     .        - 5.0d0/4.0d0*HPL1(1)*GPL1(1)*u**(-3)*v
     .        + 5.0d0/8.0d0*HPL1(1)*GPL1(1)*u**(-3)*v**2
     .        - 23.0d0/16.0d0*HPL1(1)*GPL1(1)*u**(-2)
     .        + 11.0d0/8.0d0*HPL1(1)*GPL1(1)*u**(-2)*v
     .        + 1.0d0/16.0d0*HPL1(1)*GPL1(1)*u**(-2)*v**2
     .        + 1.0d0/8.0d0*HPL1(1)*GPL1(1)*u**(-1)*v**(-1)
     .        + 2.0d0*HPL1(1)*GPL1(1)*u**(-1)
     .        - 11.0d0/8.0d0*HPL1(1)*GPL1(1)*u**(-1)*v
     .        + 1.0d0/8.0d0*HPL1(1)*GPL1(1)*u**(-1)*v**2
     .        - 1.0d0/8.0d0*HPL1(1)*GPL1(1)*v**(-1)*upv**(-1)
     .        - 1.0d0/8.0d0*HPL1(1)*GPL1(1)*upv**(-1)
         col3 = col3 + 5.0d0/8.0d0*HPL1(1)*GPL1(1)*v
     .        + 5.0d0/16.0d0*HPL1(1)*GPL1(1)*v**2
     .        + 1.0d0/2.0d0*HPL1(1)*GPL1(1)*u
     .        + 5.0d0/8.0d0*HPL1(1)*GPL1(1)*u*v
     .        + 5.0d0/16.0d0*HPL1(1)*GPL1(1)*u**2
     .        - 1.0d0/24.0d0*HPL1(1)*GPL1(2)*pi**2
     .        + 1.0d0/24.0d0*HPL1(1)*GPL1(3)*pi**2
     .        - 3.0d0/4.0d0*HPL1(1)*GPL2(0,1)
     .        - 1.0d0/8.0d0*HPL1(1)*GPL2(0,1)*u**(-2)
     .        + 1.0d0/4.0d0*HPL1(1)*GPL2(0,1)*u**(-2)*v
     .        - 1.0d0/8.0d0*HPL1(1)*GPL2(0,1)*u**(-2)*v**2
     .        + 1.0d0/2.0d0*HPL1(1)*GPL2(0,1)*u**(-1)
     .        - 1.0d0/2.0d0*HPL1(1)*GPL2(0,1)*u**(-1)*v
     .        - 1.0d0/8.0d0*HPL1(1)*GPL2(0,1)*upv**(-2)
     .        + 1.0d0/2.0d0*HPL1(1)*GPL2(0,1)*upv**(-1)
         col3 = col3 - 1.0d0/4.0d0*HPL1(1)*GPL2(0,3)*pi*ci
     .        + 1.0d0/8.0d0*HPL1(1)*GPL2(2,1)*u**(-2)
     .        - 1.0d0/4.0d0*HPL1(1)*GPL2(2,1)*u**(-2)*v
     .        + 1.0d0/8.0d0*HPL1(1)*GPL2(2,1)*u**(-2)*v**2
     .        - 1.0d0/2.0d0*HPL1(1)*GPL2(2,1)*u**(-1)
     .        + 1.0d0/2.0d0*HPL1(1)*GPL2(2,1)*u**(-1)*v
     .        - 1.0d0/8.0d0*HPL1(1)*GPL2(2,1)*upv**(-2)
     .        + 1.0d0/2.0d0*HPL1(1)*GPL2(2,1)*upv**(-1)
     .        + 1.0d0/2.0d0*HPL1(1)*GPL2(2,3)*pi*ci
     .        - 1.0d0/2.0d0*HPL1(1)*GPL2(3,0)*pi*ci
     .        + 1.0d0/2.0d0*HPL1(1)*GPL3(0,0,1)
     .        - 1.0d0/2.0d0*HPL1(1)*GPL3(0,2,1)
     .        - 1.0d0/4.0d0*HPL1(1)*GPL3(0,3,0)
     .        + 1.0d0/2.0d0*HPL1(1)*GPL3(2,3,0)
     .        - 1.0d0/2.0d0*HPL1(1)*GPL3(3,0,0)
         col3 = col3 + 1.0d0/2.0d0*HPL1(1)*GPL3(3,0,1)
     .        + 1.0d0/2.0d0*HPL1(1)*GPL3(3,2,1)
     .        - 29.0d0/16.0d0*HPL2(0,0)
     .        + 1.0d0/8.0d0*HPL2(0,0)*u**(-1)
     .        - 3.0d0/8.0d0*HPL2(0,0)*u**(-1)*v
     .        + 1.0d0/8.0d0*HPL2(0,0)*u**(-1)*v**2
     .        + 3.0d0/8.0d0*HPL2(0,0)*omumv**(-1)
     .        + 1.0d0/3.0d0*HPL2(0,0)*pi**2
     .        + 5.0d0/8.0d0*HPL2(0,0)*v
     .        + 5.0d0/16.0d0*HPL2(0,0)*v**2
     .        + 1.0d0/2.0d0*HPL2(0,0)*u
     .        + 5.0d0/8.0d0*HPL2(0,0)*u*v
     .        + 5.0d0/16.0d0*HPL2(0,0)*u**2
     .        - 3.0d0/4.0d0*HPL2(0,0)*GPL1(0)*pi*ci
     .        + 3.0d0/8.0d0*HPL2(0,0)*GPL1(2)
         col3 = col3 + 1.0d0/2.0d0*HPL2(0,0)*GPL1(2)*pi*ci
     .        + 3.0d0/4.0d0*HPL2(0,0)*GPL1(3)*pi*ci
     .        - 1.0d0/2.0d0*HPL2(0,0)*GPL2(0,0)
     .        + 1.0d0/4.0d0*HPL2(0,0)*GPL2(0,2)
     .        + 1.0d0/2.0d0*HPL2(0,0)*GPL2(0,3)
     .        - 1.0d0/4.0d0*HPL2(0,0)*GPL2(2,3)
     .        + 3.0d0/4.0d0*HPL2(0,0)*GPL2(3,0)
     .        + 1.0d0/4.0d0*HPL2(0,0)*GPL2(3,2)
     .        - 1.0d0/2.0d0*HPL2(0,0)*GPL2(3,3)
     .        - 1.0d0/8.0d0*HPL2(0,1)*u**(-1)
     .        + 1.0d0/8.0d0*HPL2(0,1)*u**(-1)*v
     .        + 1.0d0/8.0d0*HPL2(0,1)*upv**(-1)
     .        + 1.0d0/24.0d0*HPL2(0,1)*pi**2
     .        - 1.0d0/2.0d0*HPL2(0,1)*GPL1(0)*pi*ci
     .        - 1.0d0/8.0d0*HPL2(0,1)*GPL1(2)*u**(-2)
         col3 = col3 + 1.0d0/4.0d0*HPL2(0,1)*GPL1(2)*u**(-2)*v
     .        - 1.0d0/8.0d0*HPL2(0,1)*GPL1(2)*u**(-2)*v**2
     .        + 1.0d0/2.0d0*HPL2(0,1)*GPL1(2)*u**(-1)
     .        - 1.0d0/2.0d0*HPL2(0,1)*GPL1(2)*u**(-1)*v
     .        + 1.0d0/8.0d0*HPL2(0,1)*GPL1(2)*upv**(-2)
     .        - 1.0d0/2.0d0*HPL2(0,1)*GPL1(2)*upv**(-1)
     .        + 1.0d0/2.0d0*HPL2(0,1)*GPL1(2)*pi*ci
     .        + 1.0d0/4.0d0*HPL2(0,1)*GPL1(3)*pi*ci
     .        - 1.0d0/2.0d0*HPL2(0,1)*GPL2(0,0)
     .        + 1.0d0/2.0d0*HPL2(0,1)*GPL2(0,2)
     .        + 1.0d0/4.0d0*HPL2(0,1)*GPL2(0,3)
     .        + 1.0d0/2.0d0*HPL2(0,1)*GPL2(2,0)
     .        - 1.0d0/2.0d0*HPL2(0,1)*GPL2(2,3)
     .        + 1.0d0/4.0d0*HPL2(0,1)*GPL2(3,0)
     .        - 1.0d0/2.0d0*HPL2(0,1)*GPL2(3,2)
         col3 = col3 - 3.0d0/16.0d0*HPL2(1,0)
     .        + 3.0d0/8.0d0*HPL2(1,0)*u**(-1)
     .        - 3.0d0/8.0d0*HPL2(1,0)*u**(-1)*v
     .        + 1.0d0/8.0d0*HPL2(1,0)*u**(-1)*v**2
     .        - 3.0d0/8.0d0*HPL2(1,0)*omumv**(-1)
     .        + 1.0d0/8.0d0*HPL2(1,0)*upv**(-2)*pi*ci
     .        - 1.0d0/8.0d0*HPL2(1,0)*upv**(-1)
     .        - 1.0d0/2.0d0*HPL2(1,0)*upv**(-1)*pi*ci
     .        + 3.0d0/4.0d0*HPL2(1,0)*pi*ci
     .        - 1.0d0/2.0d0*HPL2(1,0)*pi**2
     .        + 5.0d0/8.0d0*HPL2(1,0)*v
     .        + 5.0d0/16.0d0*HPL2(1,0)*v**2
     .        + 1.0d0/2.0d0*HPL2(1,0)*u
     .        + 5.0d0/8.0d0*HPL2(1,0)*u*v
     .        + 5.0d0/16.0d0*HPL2(1,0)*u**2
         col3 = col3 + 3.0d0/8.0d0*HPL2(1,0)*GPL1(0)
     .        + 1.0d0/2.0d0*HPL2(1,0)*GPL1(0)*pi*ci
     .        - 3.0d0/4.0d0*HPL2(1,0)*GPL1(2)
     .        - 1.0d0/8.0d0*HPL2(1,0)*GPL1(2)*upv**(-2)
     .        + 1.0d0/2.0d0*HPL2(1,0)*GPL1(2)*upv**(-1)
     .        - 1.0d0/4.0d0*HPL2(1,0)*GPL1(2)*pi*ci
     .        - 1.0d0/4.0d0*HPL2(1,0)*GPL2(0,0)
     .        + 1.0d0/4.0d0*HPL2(1,0)*GPL2(0,2)
     .        + 1.0d0/4.0d0*HPL2(1,0)*GPL2(0,3)
     .        + 1.0d0/4.0d0*HPL2(1,0)*GPL2(2,0)
     .        - 1.0d0/2.0d0*HPL2(1,0)*GPL2(2,3)
     .        + 1.0d0/2.0d0*HPL2(1,0)*GPL2(3,0)
     .        + 1.0d0/4.0d0*HPL2(1,0)*GPL2(3,2)
     .        + 1.0d0/6.0d0*HPL2(1,1)*pi**2
     .        + 3.0d0/4.0d0*HPL3(0,0,0)*pi*ci
         col3 = col3 + 3.0d0/4.0d0*HPL3(0,0,0)*GPL1(0)
     .        - 1.0d0/4.0d0*HPL3(0,0,0)*GPL1(2)
     .        - 3.0d0/4.0d0*HPL3(0,0,0)*GPL1(3)
     .        + 1.0d0/4.0d0*HPL3(0,0,1)*pi*ci
     .        + 1.0d0/4.0d0*HPL3(0,0,1)*GPL1(0)
     .        - 1.0d0/2.0d0*HPL3(0,0,1)*GPL1(2)
     .        + 1.0d0/2.0d0*HPL3(0,1,0)*GPL1(0)
     .        - 1.0d0/2.0d0*HPL3(0,1,0)*GPL1(2)
     .        - 1.0d0/4.0d0*HPL3(0,1,0)*GPL1(3)
     .        - 3.0d0/8.0d0*HPL3(1,0,0)
     .        - 3.0d0/4.0d0*HPL3(1,0,0)*pi*ci
     .        - 1.0d0/4.0d0*HPL3(1,0,0)*GPL1(0)
     .        - 1.0d0/4.0d0*HPL3(1,0,0)*GPL1(3)
     .        - 3.0d0/8.0d0*HPL3(1,0,1)
     .        - 1.0d0/8.0d0*HPL3(1,0,1)*upv**(-2)
         col3 = col3 + 1.0d0/2.0d0*HPL3(1,0,1)*upv**(-1)
     .        - 3.0d0/4.0d0*HPL3(1,0,1)*pi*ci
     .        - 3.0d0/4.0d0*HPL3(1,0,1)*GPL1(0)
     .        + 1.0d0/2.0d0*HPL3(1,0,1)*GPL1(2)
     .        + 1.0d0/2.0d0*HPL3(1,0,1)*GPL1(3)
     .        + 9.0d0/8.0d0*HPL3(1,1,0)
     .        + 1.0d0/8.0d0*HPL3(1,1,0)*upv**(-2)
     .        - 1.0d0/2.0d0*HPL3(1,1,0)*upv**(-1)
     .        + 3.0d0/4.0d0*HPL3(1,1,0)*pi*ci
     .        - 1.0d0/4.0d0*HPL3(1,1,0)*GPL1(0)
     .        + 1.0d0/4.0d0*HPL3(1,1,0)*GPL1(2)
     .        - 1.0d0/4.0d0*HPL3(1,1,0)*GPL1(3)
     .        - 3.0d0/4.0d0*HPL4(0,0,0,0)
     .        - 1.0d0/4.0d0*HPL4(0,0,1,0)
     .        - 1.0d0/4.0d0*HPL4(0,1,0,0)
         col3 = col3 + 1.0d0/2.0d0*HPL4(0,1,0,1)
     .        - 1.0d0/4.0d0*HPL4(0,1,1,0)
     .        + 1.0d0/2.0d0*HPL4(1,0,0,0)
     .        + 1.0d0/2.0d0*HPL4(1,0,0,1)
     .        + 3.0d0/4.0d0*HPL4(1,0,1,0)
     .        - 1.0d0/4.0d0*HPL4(1,1,0,0)
     .        - HPL4(1,1,0,1)
     .        - 11.0d0/16.0d0*GPL1(0)
     .        + 1.0d0/4.0d0*GPL1(0)*u**(-1)*pi*ci
     .        - 3.0d0/8.0d0*GPL1(0)*u**(-1)*v*pi*ci
     .        + 1.0d0/8.0d0*GPL1(0)*u**(-1)*v**2*pi*ci
     .        - 17.0d0/16.0d0*GPL1(0)*pi*ci
     .        + 1.0d0/16.0d0*GPL1(0)*pi**2
     .        + 1.0d0/6.0d0*GPL1(0)*pi**3*ci
     .        - 1.0d0/4.0d0*GPL1(0)*zeta3
         col3 = col3 + 5.0d0/16.0d0*GPL1(0)*v
     .        + 5.0d0/8.0d0*GPL1(0)*v*pi*ci
     .        + 5.0d0/16.0d0*GPL1(0)*v**2*pi*ci
     .        + 5.0d0/16.0d0*GPL1(0)*u
     .        + 1.0d0/2.0d0*GPL1(0)*u*pi*ci
     .        + 5.0d0/8.0d0*GPL1(0)*u*v*pi*ci
     .        + 5.0d0/16.0d0*GPL1(0)*u**2*pi*ci
     .        - 5.0d0/8.0d0*GPL1(1)*u**(-3)*pi*ci
     .        + 5.0d0/4.0d0*GPL1(1)*u**(-3)*v*pi*ci
     .        - 5.0d0/8.0d0*GPL1(1)*u**(-3)*v**2*pi*ci
     .        + 23.0d0/16.0d0*GPL1(1)*u**(-2)*pi*ci
     .        - 11.0d0/8.0d0*GPL1(1)*u**(-2)*v*pi*ci
     .        - 1.0d0/16.0d0*GPL1(1)*u**(-2)*v**2*pi*ci
     .        - 1.0d0/8.0d0*GPL1(1)*u**(-1)*v**(-1)*pi*ci
     .        - 2.0d0*GPL1(1)*u**(-1)*pi*ci
         col3 = col3 + 11.0d0/8.0d0*GPL1(1)*u**(-1)*v*pi*ci
     .        - 1.0d0/8.0d0*GPL1(1)*u**(-1)*v**2*pi*ci
     .        + 1.0d0/8.0d0*GPL1(1)*v**(-1)*upv**(-1)*pi*ci
     .        + 1.0d0/8.0d0*GPL1(1)*upv**(-1)*pi*ci
     .        + 2.0d0*GPL1(1)*pi*ci
     .        - 5.0d0/8.0d0*GPL1(1)*v*pi*ci
     .        - 5.0d0/16.0d0*GPL1(1)*v**2*pi*ci
     .        - 1.0d0/2.0d0*GPL1(1)*u*pi*ci
     .        - 5.0d0/8.0d0*GPL1(1)*u*v*pi*ci
     .        - 5.0d0/16.0d0*GPL1(1)*u**2*pi*ci
     .        - 17.0d0/32.0d0*GPL1(2)
     .        - 5.0d0/8.0d0*GPL1(2)*u**(-2)
     .        + 1.0d0/48.0d0*GPL1(2)*u**(-2)*pi**2
     .        + 5.0d0/4.0d0*GPL1(2)*u**(-2)*v
     .        - 1.0d0/24.0d0*GPL1(2)*u**(-2)*v*pi**2
         col3 = col3 - 5.0d0/8.0d0*GPL1(2)*u**(-2)*v**2
     .        + 1.0d0/48.0d0*GPL1(2)*u**(-2)*v**2*pi**2
     .        + 11.0d0/8.0d0*GPL1(2)*u**(-1)
     .        + 1.0d0/4.0d0*GPL1(2)*u**(-1)*pi*ci
     .        - 1.0d0/12.0d0*GPL1(2)*u**(-1)*pi**2
     .        - 17.0d0/16.0d0*GPL1(2)*u**(-1)*v
     .        - 3.0d0/8.0d0*GPL1(2)*u**(-1)*v*pi*ci
     .        + 1.0d0/12.0d0*GPL1(2)*u**(-1)*v*pi**2
     .        - 5.0d0/16.0d0*GPL1(2)*u**(-1)*v**2
     .        + 1.0d0/8.0d0*GPL1(2)*u**(-1)*v**2*pi*ci
     .        - 1.0d0/16.0d0*GPL1(2)*upv**(-2)*pi**2
     .        + 1.0d0/4.0d0*GPL1(2)*upv**(-1)*pi**2
     .        - 17.0d0/16.0d0*GPL1(2)*pi*ci
     .        - 1.0d0/16.0d0*GPL1(2)*pi**2
     .        - 1.0d0/8.0d0*GPL1(2)*pi**3*ci
         col3 = col3 - 5.0d0/4.0d0*GPL1(2)*zeta3
     .        - 5.0d0/8.0d0*GPL1(2)*v
     .        + 5.0d0/8.0d0*GPL1(2)*v*pi*ci
     .        + 5.0d0/16.0d0*GPL1(2)*v**2*pi*ci
     .        - 5.0d0/16.0d0*GPL1(2)*u
     .        + 1.0d0/2.0d0*GPL1(2)*u*pi*ci
     .        + 5.0d0/8.0d0*GPL1(2)*u*v*pi*ci
     .        + 5.0d0/16.0d0*GPL1(2)*u**2*pi*ci
     .        - 1.0d0/8.0d0*GPL1(3)*u**(-1)*pi*ci
     .        + 3.0d0/8.0d0*GPL1(3)*u**(-1)*v*pi*ci
     .        - 1.0d0/8.0d0*GPL1(3)*u**(-1)*v**2*pi*ci
     .        - 3.0d0/8.0d0*GPL1(3)*omumv**(-1)*pi*ci
     .        + 29.0d0/16.0d0*GPL1(3)*pi*ci
     .        - 1.0d0/12.0d0*GPL1(3)*pi**3*ci
     .        + 1.0d0/2.0d0*GPL1(3)*zeta3
         col3 = col3 - 5.0d0/8.0d0*GPL1(3)*v*pi*ci
     .        - 5.0d0/16.0d0*GPL1(3)*v**2*pi*ci
     .        - 1.0d0/2.0d0*GPL1(3)*u*pi*ci
     .        - 5.0d0/8.0d0*GPL1(3)*u*v*pi*ci
     .        - 5.0d0/16.0d0*GPL1(3)*u**2*pi*ci
     .        + 1.0d0/24.0d0*GPL2(0,0)*pi**2
     .        + 1.0d0/8.0d0*GPL2(0,1)*u**(-2)*pi*ci
     .        - 1.0d0/4.0d0*GPL2(0,1)*u**(-2)*v*pi*ci
     .        + 1.0d0/8.0d0*GPL2(0,1)*u**(-2)*v**2*pi*ci
     .        - 1.0d0/2.0d0*GPL2(0,1)*u**(-1)*pi*ci
     .        + 1.0d0/2.0d0*GPL2(0,1)*u**(-1)*v*pi*ci
     .        + 1.0d0/8.0d0*GPL2(0,1)*upv**(-2)*pi*ci
     .        - 1.0d0/2.0d0*GPL2(0,1)*upv**(-1)*pi*ci
     .        + 3.0d0/4.0d0*GPL2(0,1)*pi*ci
     .        - 17.0d0/16.0d0*GPL2(0,2)
         col3 = col3 + 1.0d0/4.0d0*GPL2(0,2)*u**(-1)
     .        - 3.0d0/8.0d0*GPL2(0,2)*u**(-1)*v
     .        + 1.0d0/8.0d0*GPL2(0,2)*u**(-1)*v**2
     .        - 1.0d0/24.0d0*GPL2(0,2)*pi**2
     .        + 5.0d0/8.0d0*GPL2(0,2)*v
     .        + 5.0d0/16.0d0*GPL2(0,2)*v**2
     .        + 1.0d0/2.0d0*GPL2(0,2)*u
     .        + 5.0d0/8.0d0*GPL2(0,2)*u*v
     .        + 5.0d0/16.0d0*GPL2(0,2)*u**2
     .        - 1.0d0/6.0d0*GPL2(0,3)*pi**2
     .        + 2.0d0*GPL2(1,2)
     .        - 5.0d0/8.0d0*GPL2(1,2)*u**(-3)
     .        + 5.0d0/4.0d0*GPL2(1,2)*u**(-3)*v
     .        - 5.0d0/8.0d0*GPL2(1,2)*u**(-3)*v**2
     .        + 23.0d0/16.0d0*GPL2(1,2)*u**(-2)
         col3 = col3 - 11.0d0/8.0d0*GPL2(1,2)*u**(-2)*v
     .        - 1.0d0/16.0d0*GPL2(1,2)*u**(-2)*v**2
     .        - 1.0d0/8.0d0*GPL2(1,2)*u**(-1)*v**(-1)
     .        - 2.0d0*GPL2(1,2)*u**(-1)
     .        + 11.0d0/8.0d0*GPL2(1,2)*u**(-1)*v
     .        - 1.0d0/8.0d0*GPL2(1,2)*u**(-1)*v**2
     .        + 1.0d0/8.0d0*GPL2(1,2)*v**(-1)*upv**(-1)
     .        + 1.0d0/8.0d0*GPL2(1,2)*upv**(-1)
     .        - 5.0d0/8.0d0*GPL2(1,2)*v
     .        - 5.0d0/16.0d0*GPL2(1,2)*v**2
     .        - 1.0d0/2.0d0*GPL2(1,2)*u
     .        - 5.0d0/8.0d0*GPL2(1,2)*u*v
     .        - 5.0d0/16.0d0*GPL2(1,2)*u**2
     .        - 17.0d0/16.0d0*GPL2(2,0)
     .        + 1.0d0/4.0d0*GPL2(2,0)*u**(-1)
         col3 = col3 - 3.0d0/8.0d0*GPL2(2,0)*u**(-1)*v
     .        + 1.0d0/8.0d0*GPL2(2,0)*u**(-1)*v**2
     .        - 1.0d0/24.0d0*GPL2(2,0)*pi**2
     .        + 5.0d0/8.0d0*GPL2(2,0)*v
     .        + 5.0d0/16.0d0*GPL2(2,0)*v**2
     .        + 1.0d0/2.0d0*GPL2(2,0)*u
     .        + 5.0d0/8.0d0*GPL2(2,0)*u*v
     .        + 5.0d0/16.0d0*GPL2(2,0)*u**2
     .        - 1.0d0/8.0d0*GPL2(2,1)*u**(-2)*pi*ci
     .        + 1.0d0/4.0d0*GPL2(2,1)*u**(-2)*v*pi*ci
     .        - 1.0d0/8.0d0*GPL2(2,1)*u**(-2)*v**2*pi*ci
     .        + 1.0d0/2.0d0*GPL2(2,1)*u**(-1)*pi*ci
     .        - 1.0d0/2.0d0*GPL2(2,1)*u**(-1)*v*pi*ci
     .        + 1.0d0/8.0d0*GPL2(2,1)*upv**(-2)*pi*ci
     .        - 1.0d0/2.0d0*GPL2(2,1)*upv**(-1)*pi*ci
         col3 = col3 - 3.0d0/8.0d0*GPL2(2,3)*pi*ci
     .        + 1.0d0/3.0d0*GPL2(2,3)*pi**2
     .        + 29.0d0/16.0d0*GPL2(3,0)
     .        - 1.0d0/8.0d0*GPL2(3,0)*u**(-1)
     .        + 3.0d0/8.0d0*GPL2(3,0)*u**(-1)*v
     .        - 1.0d0/8.0d0*GPL2(3,0)*u**(-1)*v**2
     .        - 3.0d0/8.0d0*GPL2(3,0)*omumv**(-1)
     .        - 1.0d0/3.0d0*GPL2(3,0)*pi**2
     .        - 5.0d0/8.0d0*GPL2(3,0)*v
     .        - 5.0d0/16.0d0*GPL2(3,0)*v**2
     .        - 1.0d0/2.0d0*GPL2(3,0)*u
     .        - 5.0d0/8.0d0*GPL2(3,0)*u*v
     .        - 5.0d0/16.0d0*GPL2(3,0)*u**2
     .        - 1.0d0/24.0d0*GPL2(3,2)*pi**2
     .        + 1.0d0/6.0d0*GPL2(3,3)*pi**2
         col3 = col3 - 1.0d0/2.0d0*GPL3(0,0,1)*pi*ci
     .        + 1.0d0/2.0d0*GPL3(0,0,3)*pi*ci
     .        + 3.0d0/4.0d0*GPL3(0,1,2)
     .        + 1.0d0/8.0d0*GPL3(0,1,2)*u**(-2)
     .        - 1.0d0/4.0d0*GPL3(0,1,2)*u**(-2)*v
     .        + 1.0d0/8.0d0*GPL3(0,1,2)*u**(-2)*v**2
     .        - 1.0d0/2.0d0*GPL3(0,1,2)*u**(-1)
     .        + 1.0d0/2.0d0*GPL3(0,1,2)*u**(-1)*v
     .        + 1.0d0/8.0d0*GPL3(0,1,2)*upv**(-2)
     .        - 1.0d0/2.0d0*GPL3(0,1,2)*upv**(-1)
     .        + 1.0d0/2.0d0*GPL3(0,2,1)*pi*ci
     .        - 1.0d0/4.0d0*GPL3(0,2,3)*pi*ci
     .        + 1.0d0/4.0d0*GPL3(0,3,0)*pi*ci
     .        + 1.0d0/4.0d0*GPL3(0,3,2)*pi*ci
     .        - 1.0d0/2.0d0*GPL3(0,3,3)*pi*ci
         col3 = col3 - 1.0d0/8.0d0*GPL3(2,1,2)*u**(-2)
     .        + 1.0d0/4.0d0*GPL3(2,1,2)*u**(-2)*v
     .        - 1.0d0/8.0d0*GPL3(2,1,2)*u**(-2)*v**2
     .        + 1.0d0/2.0d0*GPL3(2,1,2)*u**(-1)
     .        - 1.0d0/2.0d0*GPL3(2,1,2)*u**(-1)*v
     .        + 1.0d0/8.0d0*GPL3(2,1,2)*upv**(-2)
     .        - 1.0d0/2.0d0*GPL3(2,1,2)*upv**(-1)
     .        - 3.0d0/8.0d0*GPL3(2,3,0)
     .        - 1.0d0/4.0d0*GPL3(2,3,0)*pi*ci
     .        - 1.0d0/2.0d0*GPL3(2,3,2)*pi*ci
     .        + 1.0d0/4.0d0*GPL3(2,3,3)*pi*ci
     .        + 1.0d0/2.0d0*GPL3(3,0,0)*pi*ci
     .        - 1.0d0/2.0d0*GPL3(3,0,1)*pi*ci
     .        + 1.0d0/2.0d0*GPL3(3,0,2)*pi*ci
     .        - 3.0d0/4.0d0*GPL3(3,0,3)*pi*ci
         col3 = col3 + 1.0d0/2.0d0*GPL3(3,2,0)*pi*ci
     .        - 1.0d0/2.0d0*GPL3(3,2,1)*pi*ci
     .        - 1.0d0/4.0d0*GPL3(3,2,3)*pi*ci
     .        - 1.0d0/2.0d0*GPL3(3,3,0)*pi*ci
     .        + 1.0d0/2.0d0*GPL3(3,3,3)*pi*ci
     .        - 1.0d0/2.0d0*GPL4(0,0,1,2)
     .        + 1.0d0/2.0d0*GPL4(0,0,3,0)
     .        + 1.0d0/2.0d0*GPL4(0,2,1,2)
     .        - 1.0d0/4.0d0*GPL4(0,2,3,0)
     .        + 1.0d0/4.0d0*GPL4(0,3,0,2)
     .        + 1.0d0/4.0d0*GPL4(0,3,2,0)
     .        - 1.0d0/2.0d0*GPL4(0,3,3,0)
     .        + 1.0d0/4.0d0*GPL4(2,3,0,0)
     .        - 1.0d0/2.0d0*GPL4(2,3,0,2)
     .        - 1.0d0/2.0d0*GPL4(2,3,2,0)
         col3 = col3 + 1.0d0/4.0d0*GPL4(2,3,3,0)
     .        + 1.0d0/2.0d0*GPL4(3,0,0,2)
     .        - 1.0d0/2.0d0*GPL4(3,0,1,2)
     .        + 1.0d0/2.0d0*GPL4(3,0,2,0)
     .        - 3.0d0/4.0d0*GPL4(3,0,3,0)
     .        + 1.0d0/2.0d0*GPL4(3,2,0,0)
     .        - 1.0d0/2.0d0*GPL4(3,2,1,2)
     .        - 1.0d0/4.0d0*GPL4(3,2,3,0)
     .        - 1.0d0/2.0d0*GPL4(3,3,0,0)
     .        + 1.0d0/2.0d0*GPL4(3,3,3,0)

         col4 = + 2393.0d0/2592.0d0
     .        - 19.0d0/36.0d0*u**(-1)
     .        - 5.0d0/48.0d0*u**(-1)*pi*ci
     .        + 19.0d0/36.0d0*u**(-1)*v
     .        + 5.0d0/48.0d0*u**(-1)*v*pi*ci
     .        + 13.0d0/48.0d0*pi*ci
     .        - 37.0d0/432.0d0*pi**2
     .        - 1.0d0/16.0d0*pi**3*ci
     .        + 131.0d0/72.0d0*zeta3
     .        - 1.0d0/4.0d0*v*omu**(-1)
     .        - 1.0d0/4.0d0*v**2*omu**(-2)*pi*ci
     .        + 34.0d0/27.0d0*HPL1(0)
     .        - 5.0d0/24.0d0*HPL1(0)*u**(-1)
     .        + 1.0d0/8.0d0*HPL1(0)*u**(-1)*v
     .        - 29.0d0/36.0d0*HPL1(0)*pi*ci
         col4 = col4 + 31.0d0/144.0d0*HPL1(0)*pi**2
     .        + 1.0d0/12.0d0*HPL1(0)*v*omv**(-1)
     .        + 1.0d0/4.0d0*HPL1(0)*v**2*omu**(-2)
     .        - 29.0d0/36.0d0*HPL1(0)*GPL1(0)
     .        - 1.0d0/6.0d0*HPL1(0)*GPL1(0)*pi*ci
     .        + 29.0d0/36.0d0*HPL1(0)*GPL1(3)
     .        + 1.0d0/8.0d0*HPL1(0)*GPL1(3)*pi*ci
     .        + 1.0d0/4.0d0*HPL1(0)*GPL2(0,0)
     .        - 5.0d0/24.0d0*HPL1(0)*GPL2(0,3)
     .        - 7.0d0/24.0d0*HPL1(0)*GPL2(3,0)
     .        - 1.0d0/6.0d0*HPL1(0)*GPL2(3,3)
     .        + 155.0d0/216.0d0*HPL1(1)
     .        - 1.0d0/144.0d0*HPL1(1)*pi**2
     .        + 1.0d0/4.0d0*HPL1(1)*v**2*omu**(-2)
     .        + 29.0d0/36.0d0*HPL2(0,0)
         col4 = col4 - 7.0d0/24.0d0*HPL2(0,0)*pi*ci
     .        - 17.0d0/24.0d0*HPL2(0,0)*GPL1(0)
     .        + 1.0d0/3.0d0*HPL2(0,0)*GPL1(3)
     .        - 29.0d0/36.0d0*HPL2(1,0)
     .        - 3.0d0/8.0d0*HPL2(1,0)*pi*ci
     .        + 1.0d0/24.0d0*HPL2(1,0)*GPL1(0)
     .        + 3.0d0/4.0d0*HPL3(0,0,0)
     .        - 5.0d0/12.0d0*HPL3(0,1,0)
     .        - 1.0d0/2.0d0*HPL3(1,0,0)
     .        - 1.0d0/6.0d0*HPL3(1,1,0)
     .        - 13.0d0/48.0d0*GPL1(0)
     .        + 5.0d0/48.0d0*GPL1(0)*u**(-1)
     .        - 5.0d0/48.0d0*GPL1(0)*u**(-1)*v
     .        + 1.0d0/144.0d0*GPL1(0)*pi**2
     .        - 155.0d0/216.0d0*GPL1(2)
         col4 = col4 - 1.0d0/48.0d0*GPL1(2)*pi**2
     .        - 1.0d0/4.0d0*GPL1(2)*v**2*omu**(-2)
     .        - 29.0d0/36.0d0*GPL1(3)*pi*ci
     .        + 19.0d0/72.0d0*GPL1(3)*pi**2
     .        + 5.0d0/24.0d0*GPL2(0,3)*pi*ci
     .        - 29.0d0/36.0d0*GPL2(3,0)
     .        - 1.0d0/6.0d0*GPL2(3,0)*pi*ci
     .        + 1.0d0/6.0d0*GPL2(3,3)*pi*ci
     .        + 5.0d0/24.0d0*GPL3(0,3,0)
     .        + 1.0d0/4.0d0*GPL3(3,0,0)
     .        + 1.0d0/6.0d0*GPL3(3,3,0)

         col5 = + 8737.0d0/864.0d0
     .        - 1.0d0/2.0d0*u**(-2)*pi*ci
     .        + u**(-2)*v*pi*ci
     .        - 1.0d0/2.0d0*u**(-2)*v**2*pi*ci
     .        - 37.0d0/36.0d0*u**(-1)
     .        + 19.0d0/48.0d0*u**(-1)*pi*ci
     .        + 37.0d0/36.0d0*u**(-1)*v
     .        - 7.0d0/48.0d0*u**(-1)*v*pi*ci
     .        - 1.0d0/4.0d0*u**(-1)*v**2*pi*ci
     .        + 547.0d0/432.0d0*pi*ci
     .        + 223.0d0/864.0d0*pi**2
     .        + 5.0d0/48.0d0*pi**3*ci
     .        + 61.0d0/36.0d0*zeta3
     .        + 13.0d0/12.0d0*v*omu**(-1)
     .        + 1.0d0/6.0d0*v*omu**(-1)*pi*ci
         col5 = col5 - 1.0d0/4.0d0*v*pi*ci
     .        + 13.0d0/12.0d0*v**2*omu**(-2)*pi*ci
     .        - 1.0d0/6.0d0*v**2*pi**2
     .        - 1.0d0/3.0d0*u*v*pi**2
     .        - 1.0d0/6.0d0*u**2*pi**2
     .        + 847.0d0/108.0d0*HPL1(0)
     .        - 1.0d0/2.0d0*HPL1(0)*u**(-2)*v
     .        + 1.0d0/2.0d0*HPL1(0)*u**(-2)*v**2
     .        - 5.0d0/24.0d0*HPL1(0)*u**(-1)
     .        + 13.0d0/24.0d0*HPL1(0)*u**(-1)*v
     .        + 1.0d0/4.0d0*HPL1(0)*u**(-1)*v**2
     .        + 271.0d0/144.0d0*HPL1(0)*pi*ci
     .        - 1.0d0/6.0d0*HPL1(0)*v*omu**(-1)
     .        - 1.0d0/12.0d0*HPL1(0)*v*omv**(-1)
     .        + 1.0d0/4.0d0*HPL1(0)*v
         col5 = col5 - 13.0d0/12.0d0*HPL1(0)*v**2*omu**(-2)
     .        - 1.0d0/4.0d0*HPL1(0)*v**2*pi*ci
     .        - 1.0d0/2.0d0*HPL1(0)*u*v*pi*ci
     .        - 1.0d0/4.0d0*HPL1(0)*u**2*pi*ci
     .        - 13.0d0/48.0d0*HPL1(0)*GPL1(0)
     .        - 1.0d0/4.0d0*HPL1(0)*GPL1(0)*pi*ci
     .        - 1.0d0/4.0d0*HPL1(0)*GPL1(0)*v**2
     .        - 1.0d0/2.0d0*HPL1(0)*GPL1(0)*u*v
     .        - 1.0d0/4.0d0*HPL1(0)*GPL1(0)*u**2
     .        - 29.0d0/18.0d0*HPL1(0)*GPL1(1)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL1(1)*u**(-3)
     .        - HPL1(0)*GPL1(1)*u**(-3)*v
     .        + 1.0d0/2.0d0*HPL1(0)*GPL1(1)*u**(-3)*v**2
     .        - 2.0d0/3.0d0*HPL1(0)*GPL1(1)*u**(-2)
     .        + 1.0d0/2.0d0*HPL1(0)*GPL1(1)*u**(-2)*v
         col5 = col5 + 1.0d0/6.0d0*HPL1(0)*GPL1(1)*u**(-2)*v**2
     .        + 5.0d0/12.0d0*HPL1(0)*GPL1(1)*u**(-1)
     .        - 1.0d0/12.0d0*HPL1(0)*GPL1(1)*u**(-1)*v
     .        + 1.0d0/12.0d0*HPL1(0)*GPL1(1)*pi*ci
     .        + 1.0d0/4.0d0*HPL1(0)*GPL1(1)*v**2
     .        + 1.0d0/2.0d0*HPL1(0)*GPL1(1)*u*v
     .        + 1.0d0/4.0d0*HPL1(0)*GPL1(1)*u**2
     .        - 59.0d0/72.0d0*HPL1(0)*GPL1(2)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL1(2)*u**(-3)
     .        + HPL1(0)*GPL1(2)*u**(-3)*v
     .        - 1.0d0/2.0d0*HPL1(0)*GPL1(2)*u**(-3)*v**2
     .        + 2.0d0/3.0d0*HPL1(0)*GPL1(2)*u**(-2)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL1(2)*u**(-2)*v
     .        - 1.0d0/6.0d0*HPL1(0)*GPL1(2)*u**(-2)*v**2
     .        - 5.0d0/12.0d0*HPL1(0)*GPL1(2)*u**(-1)
         col5 = col5 + 1.0d0/12.0d0*HPL1(0)*GPL1(2)*u**(-1)*v
     .        - 1.0d0/4.0d0*HPL1(0)*GPL1(2)*v**2
     .        - 1.0d0/2.0d0*HPL1(0)*GPL1(2)*u*v
     .        - 1.0d0/4.0d0*HPL1(0)*GPL1(2)*u**2
     .        - 29.0d0/36.0d0*HPL1(0)*GPL1(3)
     .        - 1.0d0/8.0d0*HPL1(0)*GPL1(3)*pi*ci
     .        + 1.0d0/4.0d0*HPL1(0)*GPL1(3)*v**2
     .        + 1.0d0/2.0d0*HPL1(0)*GPL1(3)*u*v
     .        + 1.0d0/4.0d0*HPL1(0)*GPL1(3)*u**2
     .        - 1.0d0/4.0d0*HPL1(0)*GPL2(0,0)
     .        + 1.0d0/12.0d0*HPL1(0)*GPL2(0,1)
     .        - 1.0d0/2.0d0*HPL1(0)*GPL2(0,2)
     .        + 5.0d0/24.0d0*HPL1(0)*GPL2(0,3)
     .        + 1.0d0/12.0d0*HPL1(0)*GPL2(1,0)
     .        - 1.0d0/3.0d0*HPL1(0)*GPL2(1,1)
         col5 = col5 + 7.0d0/6.0d0*HPL1(0)*GPL2(1,2)
     .        - 5.0d0/6.0d0*HPL1(0)*GPL2(2,0)
     .        + HPL1(0)*GPL2(2,1)
     .        - HPL1(0)*GPL2(2,2)
     .        + 1.0d0/3.0d0*HPL1(0)*GPL2(2,3)
     .        + 7.0d0/24.0d0*HPL1(0)*GPL2(3,0)
     .        + 1.0d0/6.0d0*HPL1(0)*GPL2(3,3)
     .        + 1093.0d0/216.0d0*HPL1(1)
     .        + 1.0d0/2.0d0*HPL1(1)*u**(-2)
     .        - HPL1(1)*u**(-2)*v
     .        + 1.0d0/2.0d0*HPL1(1)*u**(-2)*v**2
     .        - 1.0d0/2.0d0*HPL1(1)*u**(-1)
     .        + 1.0d0/4.0d0*HPL1(1)*u**(-1)*v
     .        + 1.0d0/4.0d0*HPL1(1)*u**(-1)*v**2
     .        + 169.0d0/144.0d0*HPL1(1)*pi*ci
         col5 = col5 + 1.0d0/72.0d0*HPL1(1)*pi**2
     .        - 1.0d0/6.0d0*HPL1(1)*v*omu**(-1)
     .        + 1.0d0/2.0d0*HPL1(1)*v
     .        - 13.0d0/12.0d0*HPL1(1)*v**2*omu**(-2)
     .        - 1.0d0/4.0d0*HPL1(1)*v**2*pi*ci
     .        + 1.0d0/4.0d0*HPL1(1)*u
     .        - 1.0d0/2.0d0*HPL1(1)*u*v*pi*ci
     .        - 1.0d0/4.0d0*HPL1(1)*u**2*pi*ci
     .        + 7.0d0/16.0d0*HPL1(1)*GPL1(0)
     .        - 1.0d0/3.0d0*HPL1(1)*GPL1(0)*pi*ci
     .        - 1.0d0/4.0d0*HPL1(1)*GPL1(0)*v**2
     .        - 1.0d0/2.0d0*HPL1(1)*GPL1(0)*u*v
     .        - 1.0d0/4.0d0*HPL1(1)*GPL1(0)*u**2
     .        - 29.0d0/18.0d0*HPL1(1)*GPL1(1)
     .        + 1.0d0/2.0d0*HPL1(1)*GPL1(1)*u**(-3)
         col5 = col5 - HPL1(1)*GPL1(1)*u**(-3)*v
     .        + 1.0d0/2.0d0*HPL1(1)*GPL1(1)*u**(-3)*v**2
     .        - 2.0d0/3.0d0*HPL1(1)*GPL1(1)*u**(-2)
     .        + 1.0d0/2.0d0*HPL1(1)*GPL1(1)*u**(-2)*v
     .        + 1.0d0/6.0d0*HPL1(1)*GPL1(1)*u**(-2)*v**2
     .        + 5.0d0/12.0d0*HPL1(1)*GPL1(1)*u**(-1)
     .        - 1.0d0/12.0d0*HPL1(1)*GPL1(1)*u**(-1)*v
     .        + 1.0d0/4.0d0*HPL1(1)*GPL1(1)*pi*ci
     .        + 1.0d0/4.0d0*HPL1(1)*GPL1(1)*v**2
     .        + 1.0d0/2.0d0*HPL1(1)*GPL1(1)*u*v
     .        + 1.0d0/4.0d0*HPL1(1)*GPL1(1)*u**2
     .        - 61.0d0/36.0d0*HPL1(1)*GPL1(2)
     .        - 1.0d0/2.0d0*HPL1(1)*GPL1(2)*pi*ci
     .        - 1.0d0/4.0d0*HPL1(1)*GPL2(0,0)
     .        + 1.0d0/12.0d0*HPL1(1)*GPL2(0,1)
         col5 = col5 - 1.0d0/2.0d0*HPL1(1)*GPL2(0,2)
     .        + 1.0d0/12.0d0*HPL1(1)*GPL2(1,0)
     .        - 1.0d0/3.0d0*HPL1(1)*GPL2(1,1)
     .        + HPL1(1)*GPL2(1,2)
     .        - 1.0d0/2.0d0*HPL1(1)*GPL2(2,0)
     .        + HPL1(1)*GPL2(2,1)
     .        + 34.0d0/9.0d0*HPL2(0,0)
     .        + 7.0d0/24.0d0*HPL2(0,0)*pi*ci
     .        + 1.0d0/4.0d0*HPL2(0,0)*v**2
     .        + 1.0d0/2.0d0*HPL2(0,0)*u*v
     .        + 1.0d0/4.0d0*HPL2(0,0)*u**2
     .        + 17.0d0/24.0d0*HPL2(0,0)*GPL1(0)
     .        - HPL2(0,0)*GPL1(1)
     .        + 4.0d0/3.0d0*HPL2(0,0)*GPL1(2)
     .        - 1.0d0/3.0d0*HPL2(0,0)*GPL1(3)
         col5 = col5 + 175.0d0/72.0d0*HPL2(0,1)
     .        + 5.0d0/12.0d0*HPL2(0,1)*pi*ci
     .        + 5.0d0/12.0d0*HPL2(0,1)*GPL1(0)
     .        - 5.0d0/6.0d0*HPL2(0,1)*GPL1(1)
     .        + 13.0d0/8.0d0*HPL2(1,0)
     .        + 3.0d0/8.0d0*HPL2(1,0)*pi*ci
     .        + 1.0d0/4.0d0*HPL2(1,0)*v**2
     .        + 1.0d0/2.0d0*HPL2(1,0)*u*v
     .        + 1.0d0/4.0d0*HPL2(1,0)*u**2
     .        + 19.0d0/24.0d0*HPL2(1,0)*GPL1(0)
     .        - 7.0d0/6.0d0*HPL2(1,0)*GPL1(1)
     .        + 2.0d0/3.0d0*HPL2(1,0)*GPL1(2)
     .        + 61.0d0/36.0d0*HPL2(1,1)
     .        + 1.0d0/2.0d0*HPL2(1,1)*pi*ci
     .        + 1.0d0/2.0d0*HPL2(1,1)*GPL1(0)
         col5 = col5 - HPL2(1,1)*GPL1(1)
     .        - 3.0d0/4.0d0*HPL3(0,0,0)
     .        - 5.0d0/12.0d0*HPL3(0,1,0)
     .        - 5.0d0/6.0d0*HPL3(1,0,0)
     .        - 1.0d0/2.0d0*HPL3(1,1,0)
     .        - 655.0d0/432.0d0*GPL1(0)
     .        + 5.0d0/48.0d0*GPL1(0)*u**(-1)
     .        - 5.0d0/48.0d0*GPL1(0)*u**(-1)*v
     .        - 17.0d0/18.0d0*GPL1(0)*pi*ci
     .        + 1.0d0/48.0d0*GPL1(0)*pi**2
     .        + 1.0d0/4.0d0*GPL1(0)*v
     .        + 1.0d0/4.0d0*GPL1(0)*v**2*pi*ci
     .        + 1.0d0/4.0d0*GPL1(0)*u
     .        + 1.0d0/2.0d0*GPL1(0)*u*v*pi*ci
     .        + 1.0d0/4.0d0*GPL1(0)*u**2*pi*ci
         col5 = col5 - 1.0d0/2.0d0*GPL1(1)*u**(-3)*pi*ci
     .        + GPL1(1)*u**(-3)*v*pi*ci
     .        - 1.0d0/2.0d0*GPL1(1)*u**(-3)*v**2*pi*ci
     .        + 2.0d0/3.0d0*GPL1(1)*u**(-2)*pi*ci
     .        - 1.0d0/2.0d0*GPL1(1)*u**(-2)*v*pi*ci
     .        - 1.0d0/6.0d0*GPL1(1)*u**(-2)*v**2*pi*ci
     .        - 5.0d0/12.0d0*GPL1(1)*u**(-1)*pi*ci
     .        + 1.0d0/12.0d0*GPL1(1)*u**(-1)*v*pi*ci
     .        + 29.0d0/18.0d0*GPL1(1)*pi*ci
     .        - 13.0d0/36.0d0*GPL1(1)*pi**2
     .        - 1.0d0/4.0d0*GPL1(1)*v**2*pi*ci
     .        - 1.0d0/2.0d0*GPL1(1)*u*v*pi*ci
     .        - 1.0d0/4.0d0*GPL1(1)*u**2*pi*ci
     .        - 1093.0d0/216.0d0*GPL1(2)
     .        - 1.0d0/2.0d0*GPL1(2)*u**(-2)
         col5 = col5 + GPL1(2)*u**(-2)*v
     .        - 1.0d0/2.0d0*GPL1(2)*u**(-2)*v**2
     .        + 1.0d0/2.0d0*GPL1(2)*u**(-1)
     .        - 1.0d0/4.0d0*GPL1(2)*u**(-1)*v
     .        - 1.0d0/4.0d0*GPL1(2)*u**(-1)*v**2
     .        - 169.0d0/144.0d0*GPL1(2)*pi*ci
     .        + 1.0d0/72.0d0*GPL1(2)*pi**2
     .        + 1.0d0/6.0d0*GPL1(2)*v*omu**(-1)
     .        - 1.0d0/2.0d0*GPL1(2)*v
     .        + 13.0d0/12.0d0*GPL1(2)*v**2*omu**(-2)
     .        + 1.0d0/4.0d0*GPL1(2)*v**2*pi*ci
     .        - 1.0d0/4.0d0*GPL1(2)*u
     .        + 1.0d0/2.0d0*GPL1(2)*u*v*pi*ci
     .        + 1.0d0/4.0d0*GPL1(2)*u**2*pi*ci
     .        + 29.0d0/36.0d0*GPL1(3)*pi*ci
         col5 = col5 - 19.0d0/72.0d0*GPL1(3)*pi**2
     .        - 1.0d0/4.0d0*GPL1(3)*v**2*pi*ci
     .        - 1.0d0/2.0d0*GPL1(3)*u*v*pi*ci
     .        - 1.0d0/4.0d0*GPL1(3)*u**2*pi*ci
     .        + 41.0d0/72.0d0*GPL2(0,0)
     .        + 1.0d0/4.0d0*GPL2(0,0)*pi*ci
     .        - 1.0d0/12.0d0*GPL2(0,1)*pi*ci
     .        - 7.0d0/16.0d0*GPL2(0,2)
     .        + 1.0d0/3.0d0*GPL2(0,2)*pi*ci
     .        + 1.0d0/4.0d0*GPL2(0,2)*v**2
     .        + 1.0d0/2.0d0*GPL2(0,2)*u*v
     .        + 1.0d0/4.0d0*GPL2(0,2)*u**2
     .        - 5.0d0/24.0d0*GPL2(0,3)*pi*ci
     .        - 1.0d0/12.0d0*GPL2(1,0)*pi*ci
     .        + 1.0d0/3.0d0*GPL2(1,1)*pi*ci
         col5 = col5 + 29.0d0/18.0d0*GPL2(1,2)
     .        - 1.0d0/2.0d0*GPL2(1,2)*u**(-3)
     .        + GPL2(1,2)*u**(-3)*v
     .        - 1.0d0/2.0d0*GPL2(1,2)*u**(-3)*v**2
     .        + 2.0d0/3.0d0*GPL2(1,2)*u**(-2)
     .        - 1.0d0/2.0d0*GPL2(1,2)*u**(-2)*v
     .        - 1.0d0/6.0d0*GPL2(1,2)*u**(-2)*v**2
     .        - 5.0d0/12.0d0*GPL2(1,2)*u**(-1)
     .        + 1.0d0/12.0d0*GPL2(1,2)*u**(-1)*v
     .        - 1.0d0/4.0d0*GPL2(1,2)*pi*ci
     .        - 1.0d0/4.0d0*GPL2(1,2)*v**2
     .        - 1.0d0/2.0d0*GPL2(1,2)*u*v
     .        - 1.0d0/4.0d0*GPL2(1,2)*u**2
     .        - 7.0d0/16.0d0*GPL2(2,0)
     .        + 1.0d0/3.0d0*GPL2(2,0)*pi*ci
         col5 = col5 + 1.0d0/4.0d0*GPL2(2,0)*v**2
     .        + 1.0d0/2.0d0*GPL2(2,0)*u*v
     .        + 1.0d0/4.0d0*GPL2(2,0)*u**2
     .        - GPL2(2,1)*pi*ci
     .        + 61.0d0/36.0d0*GPL2(2,2)
     .        + 1.0d0/2.0d0*GPL2(2,2)*pi*ci
     .        - 1.0d0/3.0d0*GPL2(2,3)*pi*ci
     .        + 29.0d0/36.0d0*GPL2(3,0)
     .        + 1.0d0/6.0d0*GPL2(3,0)*pi*ci
     .        - 1.0d0/4.0d0*GPL2(3,0)*v**2
     .        - 1.0d0/2.0d0*GPL2(3,0)*u*v
     .        - 1.0d0/4.0d0*GPL2(3,0)*u**2
     .        - 1.0d0/6.0d0*GPL2(3,3)*pi*ci
     .        + 1.0d0/4.0d0*GPL3(0,0,2)
     .        - 1.0d0/12.0d0*GPL3(0,1,2)
         col5 = col5 + 1.0d0/4.0d0*GPL3(0,2,0)
     .        + 1.0d0/2.0d0*GPL3(0,2,2)
     .        - 5.0d0/24.0d0*GPL3(0,3,0)
     .        - 1.0d0/12.0d0*GPL3(1,0,2)
     .        + 1.0d0/3.0d0*GPL3(1,1,2)
     .        - 1.0d0/12.0d0*GPL3(1,2,0)
     .        - GPL3(1,2,2)
     .        + 1.0d0/4.0d0*GPL3(2,0,0)
     .        + 1.0d0/2.0d0*GPL3(2,0,2)
     .        - GPL3(2,1,2)
     .        + 1.0d0/2.0d0*GPL3(2,2,0)
     .        - 1.0d0/3.0d0*GPL3(2,3,0)
     .        - 1.0d0/4.0d0*GPL3(3,0,0)
     .        - 1.0d0/6.0d0*GPL3(3,3,0)

         col6 = - 25.0d0/81.0d0
     .        - 5.0d0/54.0d0*pi*ci
     .        + 13.0d0/864.0d0*pi**2
     .        - 5.0d0/9.0d0*HPL1(0)
     .        - 5.0d0/72.0d0*HPL1(0)*pi*ci
     .        + 5.0d0/72.0d0*HPL1(0)*GPL1(0)
     .        + 5.0d0/18.0d0*HPL1(0)*GPL1(2)
     .        - 10.0d0/27.0d0*HPL1(1)
     .        - 1.0d0/36.0d0*HPL1(1)*pi*ci
     .        + 1.0d0/36.0d0*HPL1(1)*GPL1(0)
     .        + 2.0d0/9.0d0*HPL1(1)*GPL1(2)
     .        - 5.0d0/12.0d0*HPL2(0,0)
     .        - 5.0d0/18.0d0*HPL2(0,1)
     .        - 5.0d0/18.0d0*HPL2(1,0)
     .        - 2.0d0/9.0d0*HPL2(1,1)
         col6 = col6 + 5.0d0/54.0d0*GPL1(0)
     .        + 1.0d0/144.0d0*GPL1(0)*pi*ci
     .        + 10.0d0/27.0d0*GPL1(2)
     .        + 1.0d0/36.0d0*GPL1(2)*pi*ci
     .        - 5.0d0/144.0d0*GPL2(0,0)
     .        - 1.0d0/36.0d0*GPL2(0,2)
     .        - 1.0d0/36.0d0*GPL2(2,0)
     .        - 2.0d0/9.0d0*GPL2(2,2)
      endif

      col(1) = col1*(-1.d0)
      col(2) = col2*(-1.d0)
      col(3) = col3*(-1.d0)
      col(4) = col4*(-1.d0)
      col(5) = col5*(-1.d0)
      col(6) = col6*(-1.d0)     !To be consistent with gamma_0=1

      end subroutine gamma4a2L

      subroutine alpha1a1L(u,v,col)
      implicit none
      !returns alpha(1a+)
      ! at one Loop (Catani-subtracted)
      !--------------------------------
      real*8, intent(in) :: u,v
      complex*16, dimension(3), intent(out) :: col
      !internal
      complex*16 :: alph
      complex*16 :: ci
      real*8 :: omu,omv,omumv,upv
      real*8, dimension(0:1) :: HPL1
      real*8, dimension(0:1,0:1) :: HPL2
      real*8, dimension(0:1,0:1,0:1) :: HPL3
      real*8, dimension(0:1,0:1,0:1,0:1) :: HPL4
      real*8, dimension(0:3) :: GPL1
      real*8, dimension(0:3,0:3) :: GPL2
      real*8, dimension(0:3,0:3,0:3) :: GPL3
      real*8, dimension(0:3,0:3,0:3,0:3) :: GPL4
      complex*16 :: col1,col2,col3


      omu   = 1.0d0 - u
      omv   = 1.0d0     - v
      omumv = 1.0d0 - u - v
      upv   =         u + v

      ci = (0.d0,1.d0)

      call tdhpl(dble(u),dble(v),4,GPL1,GPL2,GPL3,GPL4,HPL1,HPL2,HPL3,HPL4)

      if(ieorder>=2) then
         col1 = - 3.0d0/2.0d0

         col2 = 0.0d0

         col3 = 0.0d0
      elseif(ieorder==1) then
      col1 = 
     &     - 11.D0/4.D0
     &     - 3.D0/2.D0*ci*pi
     &     + 1.D0/2.D0*GPL1(0)
     &     + 1.D0/2.D0*GPL1(2)
     &     + 1.D0/2.D0*HPL1(0)
     &     - 1.D0/2.D0*HPL1(1)
         col2 = 0d0

         col3 = + 1.0d0/2.0d0
      else
         if(catanisub) then
      col1 = - 1.D0/12.D0*pi**2
     &  + 11.D0/4.D0*ci*pi
     &  + 1.D0/6.D0*v
     &  - 1.D0/6.D0*v**2
     &  + 1.D0/6.D0*u
     &  - 1.D0/6.D0*u*v
     &  - 1.D0/6.D0*u**2
     &  - 11.D0/12.D0*GPL1(0)
     &  - 1.D0/2.D0*GPL1(0)*HPL1(0)
     &  + 1.D0/2.D0*GPL1(0)*HPL1(1)
     &  - 11.D0/12.D0*GPL1(2)
     &  - 1.D0/2.D0*GPL1(2)*HPL1(0)
     &  - GPL1(3)*HPL1(1)
     &  - 1.D0/2.D0*GPL2(0,2)
     &  + GPL2(1,0)
     &
      col1 = col1 - 1.D0/2.D0*GPL2(2,0)
     &  + GPL2(3,2)
     &  - 11.D0/12.D0*HPL1(0)
     &  + 11.D0/12.D0*HPL1(1)
     &  - 1.D0/2.D0*HPL2(0,1)
     &  - 1.D0/2.D0*HPL2(1,0)
     &

      col2 = (0d0,0d0)

      col3 = - 1.D0/2.D0*ci*pi
     &  - 1.D0/6.D0*v
     &  + 1.D0/6.D0*v**2
     &  - 1.D0/6.D0*u
     &  + 1.D0/6.D0*u*v
     &  + 1.D0/6.D0*u**2
     &  + 1.D0/6.D0*GPL1(0)
     &  + 1.D0/6.D0*GPL1(2)
     &  + 1.D0/6.D0*HPL1(0)
     &  - 1.D0/6.D0*HPL1(1)
         else
      col1 = + 19.D0/24.D0*pi**2
     &     + 1.D0/6.D0*v
     &     - 1.D0/6.D0*v**2
     &     + 1.D0/6.D0*u
     &     - 1.D0/6.D0*u*v
     &     - 1.D0/6.D0*u**2
     &     + 1.D0/2.D0*GPL1(0)*ci*pi
     &     - 1.D0/2.D0*GPL1(0)*HPL1(0)
     &     + 1.D0/2.D0*GPL1(0)*HPL1(1)
     &     + 1.D0/2.D0*GPL1(2)*ci*pi
     &     - 1.D0/2.D0*GPL1(2)*HPL1(0)
     &     + 1.D0/2.D0*GPL1(2)*HPL1(1)
     &     - GPL1(3)*HPL1(1)
     &     - 1.D0/2.D0*GPL2(0,0)
     &     - 1.D0/2.D0*GPL2(0,2)
     &
      col1 = col1 + GPL2(1,0)
     &     - 1.D0/2.D0*GPL2(2,0)
     &     - 1.D0/2.D0*GPL2(2,2)
     &     + GPL2(3,2)
     &     + 1.D0/2.D0*HPL1(0)*ci*pi
     &     - 1.D0/2.D0*HPL1(1)*ci*pi
     &     - 1.D0/2.D0*HPL2(0,0)
     &     - 1.D0/2.D0*HPL2(0,1)
     &     - 1.D0/2.D0*HPL2(1,0)
     &     - 1.D0/2.D0*HPL2(1,1)
     &

      col2 = 0d0

      col3 = - 1.D0/6.D0*v
     &     + 1.D0/6.D0*v**2
     &     - 1.D0/6.D0*u
     &     + 1.D0/6.D0*u*v
     &     + 1.D0/6.D0*u**2
         
         endif
      endif

      col(1) = col1
      col(2) = col2
      col(3) = col3

      end subroutine alpha1a1L

!#############################################################

      subroutine beta1a1L(u,v,col)
      implicit none
      !returns beta(1a+)
      ! at one Loop (Catani-subtracted)
      !--------------------------------
      real*8, intent(in) :: u,v
      complex*16, dimension(3), intent(out) :: col
      !internal
      complex*16 :: beta
      complex*16 :: ci
      real*8 :: omu,omv,omumv,upv
      real*8, dimension(0:1) :: HPL1
      real*8, dimension(0:1,0:1) :: HPL2
      real*8, dimension(0:1,0:1,0:1) :: HPL3
      real*8, dimension(0:1,0:1,0:1,0:1) :: HPL4
      real*8, dimension(0:3) :: GPL1
      real*8, dimension(0:3,0:3) :: GPL2
      real*8, dimension(0:3,0:3,0:3) :: GPL3
      real*8, dimension(0:3,0:3,0:3,0:3) :: GPL4
      complex*16 :: col1,col2,col3


      omu   = 1.0d0 - u
      omv   = 1.0d0     - v
      omumv = 1.0d0 - u - v
      upv   =         u + v

      ci = (0.d0,1.d0)

      call tdhpl(dble(u),dble(v),4,GPL1,GPL2,GPL3,GPL4,HPL1,HPL2,HPL3,HPL4)

      if(ieorder>=2) then
         col1 = - 3.0d0/2.0d0

         col2 = 0.0d0

         col3 = 0.0d0
      elseif(ieorder==1) then
      col1 = 
     &     - 11.D0/4.D0
     &     - 3.D0/2.D0*ci*pi
     &     + 1.D0/2.D0*GPL1(0)
     &     + 1.D0/2.D0*GPL1(2)
     &     + 1.D0/2.D0*HPL1(0)
     &     - 1.D0/2.D0*HPL1(1)

         col2 = 0d0

         col3 = + 1.0d0/2.0d0
      else
         if(catanisub) then
      col1 = - 1.D0/12.D0*pi**2
     &  + 11.D0/4.D0*ci*pi
     &  + 1.D0/6.D0*v*omumv**(-2)
     &  - 1.D0/6.D0*v*omumv**(-1)
     &  - 1.D0/6.D0*v**2*omumv**(-2)
     &  - 11.D0/12.D0*GPL1(0)
     &  - 1.D0/2.D0*GPL1(0)*HPL1(0)
     &  + 1.D0/2.D0*GPL1(0)*HPL1(1)
     &  - 11.D0/12.D0*GPL1(2)
     &  - 1.D0/2.D0*GPL1(2)*HPL1(0)
     &  - GPL1(3)*HPL1(1)
     &  - 1.D0/2.D0*GPL2(0,2)
     &  + GPL2(1,0)
     &  - 1.D0/2.D0*GPL2(2,0)
     &  + GPL2(3,2)
     &
      col1 = col1 - 11.D0/12.D0*HPL1(0)
     &  + 11.D0/12.D0*HPL1(1)
     &  - 1.D0/2.D0*HPL2(0,1)
     &  - 1.D0/2.D0*HPL2(1,0)
     &

      col2 = 0d0

      col3 = - 1.D0/2.D0*ci*pi
     &  - 1.D0/6.D0*v*omumv**(-2)
     &  + 1.D0/6.D0*v*omumv**(-1)
     &  + 1.D0/6.D0*v**2*omumv**(-2)
     &  + 1.D0/6.D0*GPL1(0)
     &  + 1.D0/6.D0*GPL1(2)
     &  + 1.D0/6.D0*HPL1(0)
     &  - 1.D0/6.D0*HPL1(1)
         else
      col1 = 19.D0/24.D0*pi**2
     &     + 1.D0/6.D0*v*omumv**(-2)
     &     - 1.D0/6.D0*v*omumv**(-1)
     &     - 1.D0/6.D0*v**2*omumv**(-2)
     &     + 1.D0/2.D0*GPL1(0)*ci*pi
     &     - 1.D0/2.D0*GPL1(0)*HPL1(0)
     &     + 1.D0/2.D0*GPL1(0)*HPL1(1)
     &     + 1.D0/2.D0*GPL1(2)*ci*pi
     &     - 1.D0/2.D0*GPL1(2)*HPL1(0)
     &     + 1.D0/2.D0*GPL1(2)*HPL1(1)
     &     - GPL1(3)*HPL1(1)
     &     - 1.D0/2.D0*GPL2(0,0)
     &     - 1.D0/2.D0*GPL2(0,2)
     &     + GPL2(1,0)
     &     - 1.D0/2.D0*GPL2(2,0)
     &
      col1 = col1 - 1.D0/2.D0*GPL2(2,2)
     &     + GPL2(3,2)
     &     + 1.D0/2.D0*HPL1(0)*ci*pi
     &     - 1.D0/2.D0*HPL1(1)*ci*pi
     &     - 1.D0/2.D0*HPL2(0,0)
     &     - 1.D0/2.D0*HPL2(0,1)
     &     - 1.D0/2.D0*HPL2(1,0)
     &     - 1.D0/2.D0*HPL2(1,1)
     &

      col2 = 0d0

      col3 = - 1.D0/6.D0*v*omumv**(-2)
     &     + 1.D0/6.D0*v*omumv**(-1)
     &     + 1.D0/6.D0*v**2*omumv**(-2)
     &

         endif
      endif

      col(1) = col1
      col(2) = col2
      col(3) = col3

      end subroutine beta1a1L

!#############################################################

      subroutine gamma1a1L(u,v,col)
      implicit none
      !Returns gamma(1a+) at one-loop
      !------------------------------
      real*8, intent(in) :: u,v
      complex*16, dimension(3), intent(out) :: col
      !internal
      complex*16 :: gam
      complex*16 :: ci
      real*8 :: omu,omv,omumv,upv
      real*8, dimension(0:1) :: HPL1
      real*8, dimension(0:1,0:1) :: HPL2
      real*8, dimension(0:1,0:1,0:1) :: HPL3
      real*8, dimension(0:1,0:1,0:1,0:1) :: HPL4
      real*8, dimension(0:3) :: GPL1
      real*8, dimension(0:3,0:3) :: GPL2
      real*8, dimension(0:3,0:3,0:3) :: GPL3
      real*8, dimension(0:3,0:3,0:3,0:3) :: GPL4
      complex*16 :: col1,col2,col3


      omu   = 1.0d0 - u
      omv   = 1.0d0     - v
      omumv = 1.0d0 - u - v
      upv   =         u + v

      ci = (0.d0,1.d0)

      call tdhpl(dble(u),dble(v),4,GPL1,GPL2,GPL3,GPL4,HPL1,HPL2,HPL3,HPL4)

      if(ieorder>=2) then
         col1 = 1.0d0

         col2 = - 1.0d0/2.0d0

         col3 = 0.0d0
      elseif(ieorder==1) then
         col1 =
     &     + 5.D0/3.D0
     &     + ci*pi
     &     - 1.D0/2.D0*GPL1(0)
     &     - 1.D0/2.D0*HPL1(0)

         col2 =
     &     - 3.D0/4.D0
     &     - 1.D0/2.D0*ci*pi
     &     + 1.D0/2.D0*GPL1(2)
     &     - 1.D0/2.D0*HPL1(1)
         
         col3 = - 1.0d0/6.0d0
      else
         if(catanisub) then
      col1 = - 89.D0/36.D0
     &  + 1.D0/4.D0*u**(-1)
     &  - 1.D0/4.D0*u**(-1)*v
     &  - 11.D0/4.D0*ci*pi
     &  + 5.D0/6.D0*GPL1(0)
     &  - 1.D0/2.D0*GPL1(0)*HPL1(1)
     &  + 13.D0/12.D0*GPL1(2)
     &  + 1.D0/2.D0*GPL1(2)*HPL1(0)
     &  + GPL1(3)*HPL1(1)
     &  + 1.D0/2.D0*GPL2(0,2)
     &  - 1.D0/2.D0*GPL2(1,0)
     &  + 1.D0/2.D0*GPL2(2,0)
     &  - GPL2(3,2)
     &  + 5.D0/6.D0*HPL1(0)
     &  - 13.D0/12.D0*HPL1(1)
     &
      col1 = col1 + 1.D0/2.D0*HPL2(0,1)
     &

      col2 = - 9.D0/4.D0
     &  + 1.D0/4.D0*u**(-1)
     &  - 1.D0/4.D0*u**(-1)*v
     &  - 1.D0/12.D0*pi**2
     &  - 1.D0/2.D0*GPL1(0)*HPL1(0)
     &  + 1.D0/2.D0*GPL2(1,0)
     &  - 1.D0/2.D0*HPL2(1,0)
     &

      col3 = + 5.D0/9.D0
     &  + 1.D0/2.D0*ci*pi
     &  - 1.D0/12.D0*GPL1(0)
     &  - 1.D0/3.D0*GPL1(2)
     &  - 1.D0/12.D0*HPL1(0)
     &  + 1.D0/3.D0*HPL1(1)
         else
      col1 = - 89.D0/36.D0
     &     + 1.D0/4.D0*u**(-1)
     &     - 1.D0/4.D0*u**(-1)*v
     &     - 7.D0/12.D0*pi**2
     &     - 13.D0/12.D0*ci*pi
     &     - 1.D0/2.D0*GPL1(0)*ci*pi
     &     - 1.D0/2.D0*GPL1(0)*HPL1(1)
     &     + 13.D0/12.D0*GPL1(2)
     &     + 1.D0/2.D0*GPL1(2)*HPL1(0)
     &     + GPL1(3)*HPL1(1)
     &     + 1.D0/2.D0*GPL2(0,0)
     &     + 1.D0/2.D0*GPL2(0,2)
     &     - 1.D0/2.D0*GPL2(1,0)
     &     + 1.D0/2.D0*GPL2(2,0)
     &     - GPL2(3,2)
     &
      col1 = col1 - 1.D0/2.D0*HPL1(0)*ci*pi
     &     - 13.D0/12.D0*HPL1(1)
     &     + 1.D0/2.D0*HPL2(0,0)
     &     + 1.D0/2.D0*HPL2(0,1)
     &

      col2 = - 9.D0/4.D0
     &     + 1.D0/4.D0*u**(-1)
     &     - 1.D0/4.D0*u**(-1)*v
     &     + 5.D0/24.D0*pi**2
     &     - 3.D0/4.D0*ci*pi
     &     - 1.D0/2.D0*GPL1(0)*HPL1(0)
     &     + 3.D0/4.D0*GPL1(2)
     &     + 1.D0/2.D0*GPL1(2)*ci*pi
     &     + 1.D0/2.D0*GPL1(2)*HPL1(1)
     &     + 1.D0/2.D0*GPL2(1,0)
     &     - 1.D0/2.D0*GPL2(2,2)
     &     - 3.D0/4.D0*HPL1(1)
     &     - 1.D0/2.D0*HPL1(1)*ci*pi
     &     - 1.D0/2.D0*HPL2(1,0)
     &     - 1.D0/2.D0*HPL2(1,1)
     &

      col3 = + 5.D0/9.D0
     &     + 1.D0/3.D0*ci*pi
     &     - 1.D0/3.D0*GPL1(2)
     &     + 1.D0/3.D0*HPL1(1)
         endif
      endif

      col(1) = col1*(-1.d0)
      col(2) = col2*(-1.d0)
      col(3) = col3*(-1.d0)     !To be consistent with gamma_0=1

      end subroutine gamma1a1L

!#############################################################


!#############################################################

      subroutine alpha1a2L(u,v,col)
      implicit none
      !returns alpha(1a+)
      ! at two Loops
      !-------------
      real*8, intent(in) :: u,v
      complex*16, dimension(6), intent(out) :: col
      !internal
      complex*16 :: alph
      complex*16 :: ci
      real*8 :: omu,omv,omumv,upv
      real*8, dimension(0:1) :: HPL1
      real*8, dimension(0:1,0:1) :: HPL2
      real*8, dimension(0:1,0:1,0:1) :: HPL3
      real*8, dimension(0:1,0:1,0:1,0:1) :: HPL4
      real*8, dimension(0:3) :: GPL1
      real*8, dimension(0:3,0:3) :: GPL2
      real*8, dimension(0:3,0:3,0:3) :: GPL3
      real*8, dimension(0:3,0:3,0:3,0:3) :: GPL4
      complex*16 :: col1,col2,col3,col4,col5,col6


      omu   = 1.0d0 - u
      omv   = 1.0d0     - v
      omumv = 1.0d0 - u - v
      upv   =         u + v

      ci = (0.d0,1.d0)

      call tdhpl(dble(u),dble(v),4,GPL1,GPL2,GPL3,GPL4,HPL1,HPL2,HPL3,HPL4)

      if(ieorder==4) then
         col1 = 9.0d0/8.0d0

         col2 = 0.0d0

         col3 = 0.0d0

         col4 = 0.0d0

         col5 = 0.0d0

         col6 = 0.0d0
      elseif(ieorder==3) then
         col1 = + 99.0d0/16.0d0
     .          + 9.0d0/4.0d0*pi*ci
     .          - 3.0d0/4.0d0*HPL1(0)
     .          + 3.0d0/4.0d0*HPL1(1)
     .          - 3.0d0/4.0d0*GPL1(0)
     .          - 3.0d0/4.0d0*GPL1(2)

         col2 = 0

         col3 = 0

         col4 = 0

         col5 = - 9.0d0/8.0d0

         col6 = 0
      else
      col1 = + 1321.D0/216.D0
     &  - 5029.D0/864.D0*pi**2
     &  + 49.D0/2880.D0*pi**4
     &  + 185.D0/24.D0*ci*pi
     &  - 77.D0/288.D0*ci*pi**3
     &  - 11.D0/16.D0*zeta3
     &  + 3.D0/4.D0*zeta3*ci*pi
     &  + 247.D0/108.D0*v
     &  + 25.D0/72.D0*v*pi**2*omumv**(-1)
     &  - 13.D0/36.D0*v*pi**2
     &  + 55.D0/72.D0*v*ci*pi
     &  - 247.D0/108.D0*v**2
     &  + 5.D0/36.D0*v**2*pi**2*omumv**(-2)
     &  - 5.D0/8.D0*v**2*pi**2*omumv**(-1)
     &  + 11.D0/72.D0*v**2*pi**2
     &
      col1 = col1 - 55.D0/72.D0*v**2*ci*pi
     &  - 5.D0/18.D0*v**3*pi**2*omumv**(-2)
     &  + 5.D0/18.D0*v**3*pi**2*omumv**(-1)
     &  + 5.D0/36.D0*v**4*pi**2*omumv**(-2)
     &  + 247.D0/108.D0*u
     &  - 1.D0/72.D0*u*pi**2
     &  + 55.D0/72.D0*u*ci*pi
     &  - 247.D0/108.D0*u*v
     &  - 1.D0/72.D0*u*v*pi**2
     &  - 55.D0/72.D0*u*v*ci*pi
     &  - 247.D0/108.D0*u**2
     &  + 1.D0/72.D0*u**2*pi**2
     &  - 55.D0/72.D0*u**2*ci*pi
     &  - 13.D0/8.D0*GPL1(0)
     &  + 55.D0/288.D0*GPL1(0)*pi**2
     &
      col1 = col1 - 605.D0/144.D0*GPL1(0)*ci*pi
     &  - 1.D0/4.D0*GPL1(0)*zeta3
     &  + 5.D0/6.D0*GPL1(0)*v*omumv**(-1)
     &  - 31.D0/24.D0*GPL1(0)*v
     &  - 5.D0/3.D0*GPL1(0)*v**2*omumv**(-1)
     &  + 31.D0/24.D0*GPL1(0)*v**2
     &  + 5.D0/6.D0*GPL1(0)*v**3*omumv**(-1)
     &  + 145.D0/72.D0*GPL1(0)*u
     &  - 3.D0/8.D0*GPL1(0)*u*v
     &  + 5.D0/6.D0*GPL1(0)*u**2*v**(-1)
     &  - 79.D0/72.D0*GPL1(0)*u**2
     &  - 5.D0/6.D0*GPL1(0)*u**3*v**(-1)
     &  - 49.D0/48.D0*GPL1(0)*HPL1(0)
     &  + 1.D0/8.D0*GPL1(0)*HPL1(0)*pi**2
     &  - 55.D0/24.D0*GPL1(0)*HPL1(0)*ci*pi
     &
      col1 = col1 + 25.D0/12.D0*GPL1(0)*HPL1(0)*v*omumv**(-1)
     &  - 13.D0/6.D0*GPL1(0)*HPL1(0)*v
     &  + 5.D0/6.D0*GPL1(0)*HPL1(0)*v**2*omumv**(-2)
     &  - 15.D0/4.D0*GPL1(0)*HPL1(0)*v**2*omumv**(-1)
     &  + 11.D0/12.D0*GPL1(0)*HPL1(0)*v**2
     &  - 5.D0/3.D0*GPL1(0)*HPL1(0)*v**3*omumv**(-2)
     &  + 5.D0/3.D0*GPL1(0)*HPL1(0)*v**3*omumv**(-1)
     &  + 5.D0/6.D0*GPL1(0)*HPL1(0)*v**4*omumv**(-2)
     &  - 1.D0/12.D0*GPL1(0)*HPL1(0)*u
     &  - 1.D0/12.D0*GPL1(0)*HPL1(0)*u*v
     &  + 1.D0/12.D0*GPL1(0)*HPL1(0)*u**2
     &  + 49.D0/48.D0*GPL1(0)*HPL1(1)
     &  - 1.D0/8.D0*GPL1(0)*HPL1(1)*pi**2
     &  + 55.D0/24.D0*GPL1(0)*HPL1(1)*ci*pi
     &  + 1.D0/12.D0*GPL1(0)*HPL1(1)*v
     &
      col1 = col1 - 1.D0/12.D0*GPL1(0)*HPL1(1)*v**2
     &  - 25.D0/12.D0*GPL1(0)*HPL1(1)*u*v**(-1)
     &  + 7.D0/3.D0*GPL1(0)*HPL1(1)*u
     &  - 1.D0/4.D0*GPL1(0)*HPL1(1)*u*v
     &  - 5.D0/6.D0*GPL1(0)*HPL1(1)*u**2*v**(-2)
     &  + 15.D0/4.D0*GPL1(0)*HPL1(1)*u**2*v**(-1)
     &  - 13.D0/12.D0*GPL1(0)*HPL1(1)*u**2
     &  + 5.D0/3.D0*GPL1(0)*HPL1(1)*u**3*v**(-2)
     &  - 5.D0/3.D0*GPL1(0)*HPL1(1)*u**3*v**(-1)
     &  - 5.D0/6.D0*GPL1(0)*HPL1(1)*u**4*v**(-2)
     &  + 11.D0/6.D0*GPL1(0)*HPL2(0,0)
     &  + 11.D0/24.D0*GPL1(0)*HPL2(0,1)
     &  + 11.D0/24.D0*GPL1(0)*HPL2(1,0)
     &  + 11.D0/6.D0*GPL1(0)*HPL2(1,1)
     &  + 1.D0/2.D0*GPL1(0)*HPL3(0,0,1)
     &
      col1 = col1 + 1.D0/2.D0*GPL1(0)*HPL3(0,1,0)
     &  - 1.D0/2.D0*GPL1(0)*HPL3(0,1,1)
     &  + 1.D0/2.D0*GPL1(0)*HPL3(1,0,0)
     &  - 1.D0/2.D0*GPL1(0)*HPL3(1,0,1)
     &  - 1.D0/2.D0*GPL1(0)*HPL3(1,1,0)
     &  + 11.D0/36.D0*GPL1(1)*pi**2
     &  + 1.D0/8.D0*GPL1(2)
     &  + 5.D0/6.D0*GPL1(2)*u**(-1)*v
     &  - 5.D0/3.D0*GPL1(2)*u**(-1)*v**2
     &  + 5.D0/6.D0*GPL1(2)*u**(-1)*v**3
     &  - 55.D0/96.D0*GPL1(2)*pi**2
     &  - 605.D0/144.D0*GPL1(2)*ci*pi
     &  - 1.D0/4.D0*GPL1(2)*zeta3
     &  - 227.D0/72.D0*GPL1(2)*v
     &  + 101.D0/72.D0*GPL1(2)*v**2
     &
      col1 = col1 + 5.D0/6.D0*GPL1(2)*u*v**(-1)
     &  - 227.D0/72.D0*GPL1(2)*u
     &  + 49.D0/72.D0*GPL1(2)*u*v
     &  - 5.D0/3.D0*GPL1(2)*u**2*v**(-1)
     &  + 101.D0/72.D0*GPL1(2)*u**2
     &  + 5.D0/6.D0*GPL1(2)*u**3*v**(-1)
     &  - 49.D0/48.D0*GPL1(2)*HPL1(0)
     &  + 5.D0/6.D0*GPL1(2)*HPL1(0)*u**(-2)*v**2
     &  - 5.D0/3.D0*GPL1(2)*HPL1(0)*u**(-2)*v**3
     &  + 5.D0/6.D0*GPL1(2)*HPL1(0)*u**(-2)*v**4
     &  + 25.D0/12.D0*GPL1(2)*HPL1(0)*u**(-1)*v
     &  - 15.D0/4.D0*GPL1(2)*HPL1(0)*u**(-1)*v**2
     &  + 5.D0/3.D0*GPL1(2)*HPL1(0)*u**(-1)*v**3
     &  + 1.D0/8.D0*GPL1(2)*HPL1(0)*pi**2
     &  - 55.D0/24.D0*GPL1(2)*HPL1(0)*ci*pi
     &
      col1 = col1 - 7.D0/3.D0*GPL1(2)*HPL1(0)*v
     &  + 13.D0/12.D0*GPL1(2)*HPL1(0)*v**2
     &  - 1.D0/12.D0*GPL1(2)*HPL1(0)*u
     &  + 1.D0/4.D0*GPL1(2)*HPL1(0)*u*v
     &  + 1.D0/12.D0*GPL1(2)*HPL1(0)*u**2
     &  - 121.D0/48.D0*GPL1(2)*HPL1(1)
     &  + 1.D0/3.D0*GPL1(2)*HPL1(1)*pi**2
     &  + 11.D0/6.D0*GPL1(2)*HPL2(0,0)
     &  + 11.D0/4.D0*GPL1(2)*HPL2(0,1)
     &  - 11.D0/6.D0*GPL1(2)*HPL2(1,0)
     &  - 3.D0/2.D0*GPL1(2)*HPL3(0,0,1)
     &  + 1.D0/2.D0*GPL1(2)*HPL3(0,1,0)
     &  + 1.D0/2.D0*GPL1(2)*HPL3(1,0,0)
     &  + 2*GPL1(2)*HPL3(1,1,0)
     &  - 67.D0/18.D0*GPL1(3)*HPL1(1)
     &
      col1 = col1 + 5.D0/6.D0*GPL1(3)*HPL1(1)*u**(-2)*v**2
     &  - 5.D0/3.D0*GPL1(3)*HPL1(1)*u**(-2)*v**3
     &  + 5.D0/6.D0*GPL1(3)*HPL1(1)*u**(-2)*v**4
     &  + 25.D0/12.D0*GPL1(3)*HPL1(1)*u**(-1)*v
     &  - 15.D0/4.D0*GPL1(3)*HPL1(1)*u**(-1)*v**2
     &  + 5.D0/3.D0*GPL1(3)*HPL1(1)*u**(-1)*v**3
     &  - 1.D0/12.D0*GPL1(3)*HPL1(1)*pi**2
     &  - 55.D0/12.D0*GPL1(3)*HPL1(1)*ci*pi
     &  - 29.D0/12.D0*GPL1(3)*HPL1(1)*v
     &  + 7.D0/6.D0*GPL1(3)*HPL1(1)*v**2
     &  + 25.D0/12.D0*GPL1(3)*HPL1(1)*u*v**(-1)
     &  - 29.D0/12.D0*GPL1(3)*HPL1(1)*u
     &  + 1.D0/2.D0*GPL1(3)*HPL1(1)*u*v
     &  + 5.D0/6.D0*GPL1(3)*HPL1(1)*u**2*v**(-2)
     &  - 15.D0/4.D0*GPL1(3)*HPL1(1)*u**2*v**(-1)
     &
      col1 = col1 + 7.D0/6.D0*GPL1(3)*HPL1(1)*u**2
     &  - 5.D0/3.D0*GPL1(3)*HPL1(1)*u**3*v**(-2)
     &  + 5.D0/3.D0*GPL1(3)*HPL1(1)*u**3*v**(-1)
     &  + 5.D0/6.D0*GPL1(3)*HPL1(1)*u**4*v**(-2)
     &  + 11.D0/6.D0*GPL1(3)*HPL2(0,1)
     &  - 11.D0/3.D0*GPL1(3)*HPL2(1,1)
     &  + 2*GPL1(3)*HPL3(0,0,1)
     &  + GPL1(3)*HPL3(0,1,1)
     &  + GPL1(3)*HPL3(1,0,1)
     &  - GPL1(3)*HPL3(1,1,0)
     &  + 121.D0/48.D0*GPL2(0,0)
     &  + 11.D0/6.D0*GPL2(0,0)*HPL1(0)
     &  - 11.D0/6.D0*GPL2(0,0)*HPL1(1)
     &  + 1.D0/2.D0*GPL2(0,0)*HPL2(0,0)
     &  - 1.D0/2.D0*GPL2(0,0)*HPL2(0,1)
     &
      col1 = col1 - 1.D0/2.D0*GPL2(0,0)*HPL2(1,0)
     &  + 1.D0/2.D0*GPL2(0,0)*HPL2(1,1)
     &  - 1.D0/3.D0*GPL2(0,1)*pi**2
     &  - 49.D0/48.D0*GPL2(0,2)
     &  + 1.D0/8.D0*GPL2(0,2)*pi**2
     &  - 55.D0/24.D0*GPL2(0,2)*ci*pi
     &  - 1.D0/12.D0*GPL2(0,2)*v
     &  + 1.D0/12.D0*GPL2(0,2)*v**2
     &  + 25.D0/12.D0*GPL2(0,2)*u*v**(-1)
     &  - 7.D0/3.D0*GPL2(0,2)*u
     &  + 1.D0/4.D0*GPL2(0,2)*u*v
     &  + 5.D0/6.D0*GPL2(0,2)*u**2*v**(-2)
     &  - 15.D0/4.D0*GPL2(0,2)*u**2*v**(-1)
     &  + 13.D0/12.D0*GPL2(0,2)*u**2
     &  - 5.D0/3.D0*GPL2(0,2)*u**3*v**(-2)
     &
      col1 = col1 + 5.D0/3.D0*GPL2(0,2)*u**3*v**(-1)
     &  + 5.D0/6.D0*GPL2(0,2)*u**4*v**(-2)
     &  - 11.D0/24.D0*GPL2(0,2)*HPL1(0)
     &  - 11.D0/6.D0*GPL2(0,2)*HPL1(1)
     &  + 1.D0/2.D0*GPL2(0,2)*HPL2(0,0)
     &  + 1.D0/2.D0*GPL2(0,3)*HPL2(0,1)
     &  + 1.D0/2.D0*GPL2(0,3)*HPL2(1,0)
     &  - GPL2(0,3)*HPL2(1,1)
     &  + 67.D0/18.D0*GPL2(1,0)
     &  - 1.D0/4.D0*GPL2(1,0)*pi**2
     &  + 55.D0/12.D0*GPL2(1,0)*ci*pi
     &  - 25.D0/12.D0*GPL2(1,0)*v*omumv**(-1)
     &  + 9.D0/4.D0*GPL2(1,0)*v
     &  - 5.D0/6.D0*GPL2(1,0)*v**2*omumv**(-2)
     &  + 15.D0/4.D0*GPL2(1,0)*v**2*omumv**(-1)
     &
      col1 = col1 - GPL2(1,0)*v**2
     &  + 5.D0/3.D0*GPL2(1,0)*v**3*omumv**(-2)
     &  - 5.D0/3.D0*GPL2(1,0)*v**3*omumv**(-1)
     &  - 5.D0/6.D0*GPL2(1,0)*v**4*omumv**(-2)
     &  - 25.D0/12.D0*GPL2(1,0)*u*v**(-1)
     &  + 29.D0/12.D0*GPL2(1,0)*u
     &  - 1.D0/6.D0*GPL2(1,0)*u*v
     &  - 5.D0/6.D0*GPL2(1,0)*u**2*v**(-2)
     &  + 15.D0/4.D0*GPL2(1,0)*u**2*v**(-1)
     &  - 7.D0/6.D0*GPL2(1,0)*u**2
     &  + 5.D0/3.D0*GPL2(1,0)*u**3*v**(-2)
     &  - 5.D0/3.D0*GPL2(1,0)*u**3*v**(-1)
     &  - 5.D0/6.D0*GPL2(1,0)*u**4*v**(-2)
     &  - 1.D0/2.D0*GPL2(1,0)*HPL2(0,1)
     &  - 1.D0/2.D0*GPL2(1,0)*HPL2(1,0)
     &
      col1 = col1 + 1.D0/3.D0*GPL2(1,1)*pi**2
     &  - 49.D0/48.D0*GPL2(2,0)
     &  + 1.D0/8.D0*GPL2(2,0)*pi**2
     &  - 55.D0/24.D0*GPL2(2,0)*ci*pi
     &  - 1.D0/12.D0*GPL2(2,0)*v
     &  + 1.D0/12.D0*GPL2(2,0)*v**2
     &  + 25.D0/12.D0*GPL2(2,0)*u*v**(-1)
     &  - 7.D0/3.D0*GPL2(2,0)*u
     &  + 1.D0/4.D0*GPL2(2,0)*u*v
     &  + 5.D0/6.D0*GPL2(2,0)*u**2*v**(-2)
     &  - 15.D0/4.D0*GPL2(2,0)*u**2*v**(-1)
     &  + 13.D0/12.D0*GPL2(2,0)*u**2
     &  - 5.D0/3.D0*GPL2(2,0)*u**3*v**(-2)
     &  + 5.D0/3.D0*GPL2(2,0)*u**3*v**(-1)
     &  + 5.D0/6.D0*GPL2(2,0)*u**4*v**(-2)
     &
      col1 = col1 - 11.D0/24.D0*GPL2(2,0)*HPL1(0)
     &  - 11.D0/6.D0*GPL2(2,0)*HPL1(1)
     &  + 1.D0/2.D0*GPL2(2,0)*HPL2(0,0)
     &  + GPL2(2,0)*HPL2(0,1)
     &  + GPL2(2,0)*HPL2(1,0)
     &  + 121.D0/48.D0*GPL2(2,2)
     &  - 1.D0/3.D0*GPL2(2,2)*pi**2
     &  + 11.D0/6.D0*GPL2(2,2)*HPL1(0)
     &  + 1.D0/2.D0*GPL2(2,2)*HPL2(0,0)
     &  + GPL2(2,2)*HPL2(0,1)
     &  - GPL2(2,2)*HPL2(1,0)
     &  + 55.D0/12.D0*GPL2(2,3)*HPL1(1)
     &  - 3.D0/2.D0*GPL2(2,3)*HPL2(0,1)
     &  + 1.D0/2.D0*GPL2(2,3)*HPL2(1,0)
     &  - 1.D0/2.D0*GPL2(3,0)*HPL2(0,1)
     &
      col1 = col1 - 1.D0/2.D0*GPL2(3,0)*HPL2(1,0)
     &  - GPL2(3,0)*HPL2(1,1)
     &  + 67.D0/18.D0*GPL2(3,2)
     &  - 5.D0/6.D0*GPL2(3,2)*u**(-2)*v**2
     &  + 5.D0/3.D0*GPL2(3,2)*u**(-2)*v**3
     &  - 5.D0/6.D0*GPL2(3,2)*u**(-2)*v**4
     &  - 25.D0/12.D0*GPL2(3,2)*u**(-1)*v
     &  + 15.D0/4.D0*GPL2(3,2)*u**(-1)*v**2
     &  - 5.D0/3.D0*GPL2(3,2)*u**(-1)*v**3
     &  + 1.D0/12.D0*GPL2(3,2)*pi**2
     &  + 55.D0/12.D0*GPL2(3,2)*ci*pi
     &  + 29.D0/12.D0*GPL2(3,2)*v
     &  - 7.D0/6.D0*GPL2(3,2)*v**2
     &  - 25.D0/12.D0*GPL2(3,2)*u*v**(-1)
     &  + 29.D0/12.D0*GPL2(3,2)*u
     &
      col1 = col1 - 1.D0/2.D0*GPL2(3,2)*u*v
     &  - 5.D0/6.D0*GPL2(3,2)*u**2*v**(-2)
     &  + 15.D0/4.D0*GPL2(3,2)*u**2*v**(-1)
     &  - 7.D0/6.D0*GPL2(3,2)*u**2
     &  + 5.D0/3.D0*GPL2(3,2)*u**3*v**(-2)
     &  - 5.D0/3.D0*GPL2(3,2)*u**3*v**(-1)
     &  - 5.D0/6.D0*GPL2(3,2)*u**4*v**(-2)
     &  + 11.D0/3.D0*GPL2(3,2)*HPL1(1)
     &  - GPL2(3,2)*HPL2(0,1)
     &  + GPL2(3,2)*HPL2(1,0)
     &  + 11.D0/6.D0*GPL2(3,3)*HPL1(1)
     &  + 2*GPL2(3,3)*HPL2(0,1)
     &  + 2*GPL2(3,3)*HPL2(1,1)
     &  + 11.D0/6.D0*GPL3(0,0,2)
     &  - 1.D0/2.D0*GPL3(0,0,2)*HPL1(0)
     &
      col1 = col1 - 1.D0/2.D0*GPL3(0,0,2)*HPL1(1)
     &  - GPL3(0,0,3)*HPL1(1)
     &  - 55.D0/12.D0*GPL3(0,1,0)
     &  - 1.D0/2.D0*GPL3(0,1,0)*HPL1(0)
     &  + 1.D0/2.D0*GPL3(0,1,0)*HPL1(1)
     &  + 11.D0/6.D0*GPL3(0,2,0)
     &  - 1.D0/2.D0*GPL3(0,2,0)*HPL1(0)
     &  - 1.D0/2.D0*GPL3(0,2,0)*HPL1(1)
     &  + 11.D0/6.D0*GPL3(0,2,2)
     &  + 1.D0/2.D0*GPL3(0,2,2)*HPL1(0)
     &  + 1.D0/2.D0*GPL3(0,2,3)*HPL1(1)
     &  - GPL3(0,3,0)*HPL1(1)
     &  - 1.D0/2.D0*GPL3(0,3,2)*HPL1(0)
     &  + GPL3(0,3,2)*HPL1(1)
     &  - 11.D0/3.D0*GPL3(1,0,0)
     &
      col1 = col1 - GPL3(1,0,0)*HPL1(0)
     &  + GPL3(1,0,0)*HPL1(1)
     &  + 1.D0/2.D0*GPL3(1,0,2)*HPL1(0)
     &  - 11.D0/6.D0*GPL3(1,1,0)
     &  + 1.D0/2.D0*GPL3(1,2,0)*HPL1(0)
     &  + 11.D0/6.D0*GPL3(2,0,0)
     &  + 1.D0/2.D0*GPL3(2,0,0)*HPL1(0)
     &  - 1.D0/2.D0*GPL3(2,0,0)*HPL1(1)
     &  + 11.D0/6.D0*GPL3(2,0,2)
     &  - 1.D0/2.D0*GPL3(2,0,2)*HPL1(0)
     &  + 1.D0/2.D0*GPL3(2,0,3)*HPL1(1)
     &  - 1.D0/2.D0*GPL3(2,1,0)*HPL1(0)
     &  - GPL3(2,1,0)*HPL1(1)
     &  + 11.D0/6.D0*GPL3(2,2,0)
     &  - 1.D0/2.D0*GPL3(2,2,0)*HPL1(0)
     &
      col1 = col1 + GPL3(2,2,3)*HPL1(1)
     &  + 1.D0/2.D0*GPL3(2,3,0)*HPL1(1)
     &  - 55.D0/12.D0*GPL3(2,3,2)
     &  - 1.D0/2.D0*GPL3(2,3,2)*HPL1(0)
     &  - 2*GPL3(2,3,3)*HPL1(1)
     &  + 1.D0/2.D0*GPL3(3,0,2)*HPL1(0)
     &  + GPL3(3,0,2)*HPL1(1)
     &  + 1.D0/2.D0*GPL3(3,2,0)*HPL1(0)
     &  + GPL3(3,2,0)*HPL1(1)
     &  - 11.D0/3.D0*GPL3(3,2,2)
     &  - GPL3(3,2,2)*HPL1(0)
     &  - 2*GPL3(3,2,3)*HPL1(1)
     &  - 11.D0/6.D0*GPL3(3,3,2)
     &  - 2*GPL3(3,3,2)*HPL1(1)
     &  + 2*GPL3(3,3,3)*HPL1(1)
     &
      col1 = col1 - GPL4(0,0,1,0)
     &  + 1.D0/2.D0*GPL4(0,0,2,2)
     &  + GPL4(0,0,3,2)
     &  - 1.D0/2.D0*GPL4(0,1,0,2)
     &  + 2*GPL4(0,1,1,0)
     &  - 1.D0/2.D0*GPL4(0,1,2,0)
     &  + 1.D0/2.D0*GPL4(0,2,0,2)
     &  - 1.D0/2.D0*GPL4(0,2,1,0)
     &  + 1.D0/2.D0*GPL4(0,2,2,0)
     &  - 1.D0/2.D0*GPL4(0,2,3,2)
     &  + GPL4(0,3,0,2)
     &  + GPL4(0,3,2,0)
     &  - GPL4(0,3,2,2)
     &  - GPL4(1,0,0,2)
     &  + 2*GPL4(1,0,1,0)
     &
      col1 = col1 - GPL4(1,0,2,0)
     &  + 2*GPL4(1,1,0,0)
     &  - 2*GPL4(1,1,1,0)
     &  - GPL4(1,2,0,0)
     &  + 1.D0/2.D0*GPL4(2,0,0,2)
     &  - 1.D0/2.D0*GPL4(2,0,1,0)
     &  + 1.D0/2.D0*GPL4(2,0,2,0)
     &  - 1.D0/2.D0*GPL4(2,0,3,2)
     &  - GPL4(2,1,0,0)
     &  + GPL4(2,1,0,2)
     &  + GPL4(2,1,2,0)
     &  + 1.D0/2.D0*GPL4(2,2,0,0)
     &  + GPL4(2,2,1,0)
     &  - GPL4(2,2,3,2)
     &  - 1.D0/2.D0*GPL4(2,3,0,2)
     &
      col1 = col1 - 1.D0/2.D0*GPL4(2,3,2,0)
     &  + 2*GPL4(2,3,3,2)
     &  - GPL4(3,0,2,2)
     &  - GPL4(3,2,0,2)
     &  - GPL4(3,2,2,0)
     &  + 2*GPL4(3,2,3,2)
     &  + 2*GPL4(3,3,2,2)
     &  - 2*GPL4(3,3,3,2)
     &  - 13.D0/8.D0*HPL1(0)
     &  + 5.D0/6.D0*HPL1(0)*u**(-1)*v**2
     &  - 5.D0/6.D0*HPL1(0)*u**(-1)*v**3
     &  + 55.D0/288.D0*HPL1(0)*pi**2
     &  - 605.D0/144.D0*HPL1(0)*ci*pi
     &  - 1.D0/4.D0*HPL1(0)*zeta3
     &  + 145.D0/72.D0*HPL1(0)*v
     &
      col1 = col1 + 5.D0/6.D0*HPL1(0)*v**2*omumv**(-1)
     &  - 139.D0/72.D0*HPL1(0)*v**2
     &  - 5.D0/6.D0*HPL1(0)*v**3*omumv**(-1)
     &  - 11.D0/24.D0*HPL1(0)*u
     &  + 11.D0/24.D0*HPL1(0)*u*v
     &  + 11.D0/24.D0*HPL1(0)*u**2
     &  - 1.D0/8.D0*HPL1(1)
     &  - 5.D0/6.D0*HPL1(1)*u**(-1)*v
     &  + 5.D0/3.D0*HPL1(1)*u**(-1)*v**2
     &  - 5.D0/6.D0*HPL1(1)*u**(-1)*v**3
     &  + 77.D0/288.D0*HPL1(1)*pi**2
     &  + 605.D0/144.D0*HPL1(1)*ci*pi
     &  + 1.D0/4.D0*HPL1(1)*zeta3
     &  + 227.D0/72.D0*HPL1(1)*v
     &  - 101.D0/72.D0*HPL1(1)*v**2
     &
      col1 = col1 - 5.D0/6.D0*HPL1(1)*u*v**(-1)
     &  + 227.D0/72.D0*HPL1(1)*u
     &  - 49.D0/72.D0*HPL1(1)*u*v
     &  + 5.D0/3.D0*HPL1(1)*u**2*v**(-1)
     &  - 101.D0/72.D0*HPL1(1)*u**2
     &  - 5.D0/6.D0*HPL1(1)*u**3*v**(-1)
     &  + 121.D0/48.D0*HPL2(0,0)
     &  - 389.D0/144.D0*HPL2(0,1)
     &  + 1.D0/8.D0*HPL2(0,1)*pi**2
     &  - 55.D0/24.D0*HPL2(0,1)*ci*pi
     &  - 1.D0/12.D0*HPL2(0,1)*v
     &  + 1.D0/12.D0*HPL2(0,1)*v**2
     &  + 25.D0/12.D0*HPL2(0,1)*u*v**(-1)
     &  - 7.D0/3.D0*HPL2(0,1)*u
     &  + 1.D0/4.D0*HPL2(0,1)*u*v
     &
      col1 = col1 + 5.D0/6.D0*HPL2(0,1)*u**2*v**(-2)
     &  - 15.D0/4.D0*HPL2(0,1)*u**2*v**(-1)
     &  + 13.D0/12.D0*HPL2(0,1)*u**2
     &  - 5.D0/3.D0*HPL2(0,1)*u**3*v**(-2)
     &  + 5.D0/3.D0*HPL2(0,1)*u**3*v**(-1)
     &  + 5.D0/6.D0*HPL2(0,1)*u**4*v**(-2)
     &  - 389.D0/144.D0*HPL2(1,0)
     &  + 1.D0/8.D0*HPL2(1,0)*pi**2
     &  - 55.D0/24.D0*HPL2(1,0)*ci*pi
     &  + 25.D0/12.D0*HPL2(1,0)*v*omumv**(-1)
     &  - 13.D0/6.D0*HPL2(1,0)*v
     &  + 5.D0/6.D0*HPL2(1,0)*v**2*omumv**(-2)
     &  - 15.D0/4.D0*HPL2(1,0)*v**2*omumv**(-1)
     &  + 11.D0/12.D0*HPL2(1,0)*v**2
     &  - 5.D0/3.D0*HPL2(1,0)*v**3*omumv**(-2)
     &
      col1 = col1 + 5.D0/3.D0*HPL2(1,0)*v**3*omumv**(-1)
     &  + 5.D0/6.D0*HPL2(1,0)*v**4*omumv**(-2)
     &  - 1.D0/12.D0*HPL2(1,0)*u
     &  - 1.D0/12.D0*HPL2(1,0)*u*v
     &  + 1.D0/12.D0*HPL2(1,0)*u**2
     &  + 121.D0/48.D0*HPL2(1,1)
     &  + 11.D0/4.D0*HPL3(0,1,0)
     &  - 11.D0/6.D0*HPL3(0,1,1)
     &  + 11.D0/6.D0*HPL3(1,0,0)
     &  - 11.D0/4.D0*HPL3(1,0,1)
     &  + 1.D0/2.D0*HPL4(0,0,1,1)
     &  + 1.D0/2.D0*HPL4(0,1,0,1)
     &  + 1.D0/2.D0*HPL4(0,1,1,0)
     &  + 1.D0/2.D0*HPL4(1,0,0,1)
     &  + 1.D0/2.D0*HPL4(1,0,1,0)
     &
      col1 = col1 + 1.D0/2.D0*HPL4(1,1,0,0)
     &

      col2 = 0d0

      col3 = 0d0

      col4 = + 67.D0/48.D0
     &  + 1.D0/4.D0*ci*pi
     &  - zeta3
     &  - 1.D0/6.D0*v
     &  + 1.D0/36.D0*v*pi**2*omumv**(-1)
     &  - 1.D0/36.D0*v*pi**2
     &  + 1.D0/6.D0*v**2
     &  - 1.D0/72.D0*v**2*pi**2*omumv**(-2)
     &  - 1.D0/72.D0*v**2*pi**2
     &  + 1.D0/36.D0*v**3*pi**2*omumv**(-2)
     &  - 1.D0/36.D0*v**3*pi**2*omumv**(-1)
     &  - 1.D0/72.D0*v**4*pi**2*omumv**(-2)
     &  - 1.D0/6.D0*u
     &  + 1.D0/6.D0*u*v
     &  - 1.D0/36.D0*u*v*pi**2
     &
      col4 = col4 + 1.D0/6.D0*u**2
     &  - 1.D0/12.D0*GPL1(0)*v*omumv**(-1)
     &  + 1.D0/12.D0*GPL1(0)*v
     &  + 1.D0/6.D0*GPL1(0)*v**2*omumv**(-1)
     &  - 1.D0/12.D0*GPL1(0)*v**2
     &  - 1.D0/12.D0*GPL1(0)*v**3*omumv**(-1)
     &  + 1.D0/6.D0*GPL1(0)*u
     &  + 1.D0/12.D0*GPL1(0)*u*v
     &  - 1.D0/12.D0*GPL1(0)*u**2*v**(-1)
     &  + 1.D0/8.D0*GPL1(0)*u**2
     &  + 1.D0/12.D0*GPL1(0)*u**3*v**(-1)
     &  + 1.D0/6.D0*GPL1(0)*HPL1(0)*v*omumv**(-1)
     &  - 1.D0/6.D0*GPL1(0)*HPL1(0)*v
     &  - 1.D0/12.D0*GPL1(0)*HPL1(0)*v**2*omumv**(-2)
     &  - 1.D0/12.D0*GPL1(0)*HPL1(0)*v**2
     &
      col4 = col4 + 1.D0/6.D0*GPL1(0)*HPL1(0)*v**3*omumv**(-2)
     &  - 1.D0/6.D0*GPL1(0)*HPL1(0)*v**3*omumv**(-1)
     &  - 1.D0/12.D0*GPL1(0)*HPL1(0)*v**4*omumv**(-2)
     &  - 1.D0/6.D0*GPL1(0)*HPL1(0)*u*v
     &  - 1.D0/6.D0*GPL1(0)*HPL1(1)*u*v**(-1)
     &  + 1.D0/3.D0*GPL1(0)*HPL1(1)*u
     &  - 1.D0/6.D0*GPL1(0)*HPL1(1)*u*v
     &  + 1.D0/12.D0*GPL1(0)*HPL1(1)*u**2*v**(-2)
     &  - 1.D0/12.D0*GPL1(0)*HPL1(1)*u**2
     &  - 1.D0/6.D0*GPL1(0)*HPL1(1)*u**3*v**(-2)
     &  + 1.D0/6.D0*GPL1(0)*HPL1(1)*u**3*v**(-1)
     &  + 1.D0/12.D0*GPL1(0)*HPL1(1)*u**4*v**(-2)
     &  + 5.D0/24.D0*GPL1(2)
     &  - 1.D0/12.D0*GPL1(2)*u**(-1)*v
     &  + 1.D0/6.D0*GPL1(2)*u**(-1)*v**2
     &
      col4 = col4 - 1.D0/12.D0*GPL1(2)*u**(-1)*v**3
     &  - 1.D0/12.D0*GPL1(2)*v
     &  - 1.D0/8.D0*GPL1(2)*v**2
     &  - 1.D0/12.D0*GPL1(2)*u*v**(-1)
     &  - 1.D0/12.D0*GPL1(2)*u
     &  - 1.D0/12.D0*GPL1(2)*u*v
     &  + 1.D0/6.D0*GPL1(2)*u**2*v**(-1)
     &  - 1.D0/8.D0*GPL1(2)*u**2
     &  - 1.D0/12.D0*GPL1(2)*u**3*v**(-1)
     &  - 1.D0/12.D0*GPL1(2)*HPL1(0)*u**(-2)*v**2
     &  + 1.D0/6.D0*GPL1(2)*HPL1(0)*u**(-2)*v**3
     &  - 1.D0/12.D0*GPL1(2)*HPL1(0)*u**(-2)*v**4
     &  + 1.D0/6.D0*GPL1(2)*HPL1(0)*u**(-1)*v
     &  - 1.D0/6.D0*GPL1(2)*HPL1(0)*u**(-1)*v**3
     &  - 1.D0/3.D0*GPL1(2)*HPL1(0)*v
     &
      col4 = col4 + 1.D0/12.D0*GPL1(2)*HPL1(0)*v**2
     &  + 1.D0/6.D0*GPL1(2)*HPL1(0)*u*v
     &  - 1.D0/12.D0*GPL1(3)*HPL1(1)*u**(-2)*v**2
     &  + 1.D0/6.D0*GPL1(3)*HPL1(1)*u**(-2)*v**3
     &  - 1.D0/12.D0*GPL1(3)*HPL1(1)*u**(-2)*v**4
     &  + 1.D0/6.D0*GPL1(3)*HPL1(1)*u**(-1)*v
     &  - 1.D0/6.D0*GPL1(3)*HPL1(1)*u**(-1)*v**3
     &  - 1.D0/3.D0*GPL1(3)*HPL1(1)*v
     &  + 1.D0/12.D0*GPL1(3)*HPL1(1)*v**2
     &  + 1.D0/6.D0*GPL1(3)*HPL1(1)*u*v**(-1)
     &  - 1.D0/3.D0*GPL1(3)*HPL1(1)*u
     &  + 1.D0/3.D0*GPL1(3)*HPL1(1)*u*v
     &  - 1.D0/12.D0*GPL1(3)*HPL1(1)*u**2*v**(-2)
     &  + 1.D0/12.D0*GPL1(3)*HPL1(1)*u**2
     &  + 1.D0/6.D0*GPL1(3)*HPL1(1)*u**3*v**(-2)
     &
      col4 = col4 - 1.D0/6.D0*GPL1(3)*HPL1(1)*u**3*v**(-1)
     &  - 1.D0/12.D0*GPL1(3)*HPL1(1)*u**4*v**(-2)
     &  + 1.D0/6.D0*GPL2(0,2)*u*v**(-1)
     &  - 1.D0/3.D0*GPL2(0,2)*u
     &  + 1.D0/6.D0*GPL2(0,2)*u*v
     &  - 1.D0/12.D0*GPL2(0,2)*u**2*v**(-2)
     &  + 1.D0/12.D0*GPL2(0,2)*u**2
     &  + 1.D0/6.D0*GPL2(0,2)*u**3*v**(-2)
     &  - 1.D0/6.D0*GPL2(0,2)*u**3*v**(-1)
     &  - 1.D0/12.D0*GPL2(0,2)*u**4*v**(-2)
     &  - 1.D0/6.D0*GPL2(1,0)*v*omumv**(-1)
     &  + 1.D0/6.D0*GPL2(1,0)*v
     &  + 1.D0/12.D0*GPL2(1,0)*v**2*omumv**(-2)
     &  + 1.D0/12.D0*GPL2(1,0)*v**2
     &  - 1.D0/6.D0*GPL2(1,0)*v**3*omumv**(-2)
     &
      col4 = col4 + 1.D0/6.D0*GPL2(1,0)*v**3*omumv**(-1)
     &  + 1.D0/12.D0*GPL2(1,0)*v**4*omumv**(-2)
     &  - 1.D0/6.D0*GPL2(1,0)*u*v**(-1)
     &  + 1.D0/3.D0*GPL2(1,0)*u
     &  + 1.D0/12.D0*GPL2(1,0)*u**2*v**(-2)
     &  - 1.D0/12.D0*GPL2(1,0)*u**2
     &  - 1.D0/6.D0*GPL2(1,0)*u**3*v**(-2)
     &  + 1.D0/6.D0*GPL2(1,0)*u**3*v**(-1)
     &  + 1.D0/12.D0*GPL2(1,0)*u**4*v**(-2)
     &  + 1.D0/6.D0*GPL2(2,0)*u*v**(-1)
     &  - 1.D0/3.D0*GPL2(2,0)*u
     &  + 1.D0/6.D0*GPL2(2,0)*u*v
     &  - 1.D0/12.D0*GPL2(2,0)*u**2*v**(-2)
     &  + 1.D0/12.D0*GPL2(2,0)*u**2
     &  + 1.D0/6.D0*GPL2(2,0)*u**3*v**(-2)
     &
      col4 = col4 - 1.D0/6.D0*GPL2(2,0)*u**3*v**(-1)
     &  - 1.D0/12.D0*GPL2(2,0)*u**4*v**(-2)
     &  + 1.D0/12.D0*GPL2(3,2)*u**(-2)*v**2
     &  - 1.D0/6.D0*GPL2(3,2)*u**(-2)*v**3
     &  + 1.D0/12.D0*GPL2(3,2)*u**(-2)*v**4
     &  - 1.D0/6.D0*GPL2(3,2)*u**(-1)*v
     &  + 1.D0/6.D0*GPL2(3,2)*u**(-1)*v**3
     &  + 1.D0/3.D0*GPL2(3,2)*v
     &  - 1.D0/12.D0*GPL2(3,2)*v**2
     &  - 1.D0/6.D0*GPL2(3,2)*u*v**(-1)
     &  + 1.D0/3.D0*GPL2(3,2)*u
     &  - 1.D0/3.D0*GPL2(3,2)*u*v
     &  + 1.D0/12.D0*GPL2(3,2)*u**2*v**(-2)
     &  - 1.D0/12.D0*GPL2(3,2)*u**2
     &  - 1.D0/6.D0*GPL2(3,2)*u**3*v**(-2)
     &
      col4 = col4 + 1.D0/6.D0*GPL2(3,2)*u**3*v**(-1)
     &  + 1.D0/12.D0*GPL2(3,2)*u**4*v**(-2)
     &  - 1.D0/12.D0*HPL1(0)*u**(-1)*v**2
     &  + 1.D0/12.D0*HPL1(0)*u**(-1)*v**3
     &  + 1.D0/6.D0*HPL1(0)*v
     &  - 1.D0/12.D0*HPL1(0)*v**2*omumv**(-1)
     &  + 5.D0/24.D0*HPL1(0)*v**2
     &  + 1.D0/12.D0*HPL1(0)*v**3*omumv**(-1)
     &  - 5.D0/24.D0*HPL1(1)
     &  + 1.D0/12.D0*HPL1(1)*u**(-1)*v
     &  - 1.D0/6.D0*HPL1(1)*u**(-1)*v**2
     &  + 1.D0/12.D0*HPL1(1)*u**(-1)*v**3
     &  + 1.D0/12.D0*HPL1(1)*v
     &  + 1.D0/8.D0*HPL1(1)*v**2
     &  + 1.D0/12.D0*HPL1(1)*u*v**(-1)
     &
      col4 = col4 + 1.D0/12.D0*HPL1(1)*u
     &  + 1.D0/12.D0*HPL1(1)*u*v
     &  - 1.D0/6.D0*HPL1(1)*u**2*v**(-1)
     &  + 1.D0/8.D0*HPL1(1)*u**2
     &  + 1.D0/12.D0*HPL1(1)*u**3*v**(-1)
     &  + 1.D0/6.D0*HPL2(0,1)*u*v**(-1)
     &  - 1.D0/3.D0*HPL2(0,1)*u
     &  + 1.D0/6.D0*HPL2(0,1)*u*v
     &  - 1.D0/12.D0*HPL2(0,1)*u**2*v**(-2)
     &  + 1.D0/12.D0*HPL2(0,1)*u**2
     &  + 1.D0/6.D0*HPL2(0,1)*u**3*v**(-2)
     &  - 1.D0/6.D0*HPL2(0,1)*u**3*v**(-1)
     &  - 1.D0/12.D0*HPL2(0,1)*u**4*v**(-2)
     &  + 1.D0/6.D0*HPL2(1,0)*v*omumv**(-1)
     &  - 1.D0/6.D0*HPL2(1,0)*v
     &
      col4 = col4 - 1.D0/12.D0*HPL2(1,0)*v**2*omumv**(-2)
     &  - 1.D0/12.D0*HPL2(1,0)*v**2
     &  + 1.D0/6.D0*HPL2(1,0)*v**3*omumv**(-2)
     &  - 1.D0/6.D0*HPL2(1,0)*v**3*omumv**(-1)
     &  - 1.D0/12.D0*HPL2(1,0)*v**4*omumv**(-2)
     &  - 1.D0/6.D0*HPL2(1,0)*u*v
     &

      col5 = - 1781.D0/432.D0
     &  + 1879.D0/864.D0*pi**2
     &  - 71.D0/18.D0*ci*pi
     &  + 7.D0/144.D0*ci*pi**3
     &  - 7.D0/8.D0*zeta3
     &  - 275.D0/108.D0*v
     &  - 7.D0/36.D0*v*pi**2*omumv**(-1)
     &  + 5.D0/24.D0*v*pi**2
     &  - 65.D0/72.D0*v*ci*pi
     &  + 275.D0/108.D0*v**2
     &  - 11.D0/72.D0*v**2*pi**2*omumv**(-2)
     &  + 1.D0/2.D0*v**2*pi**2*omumv**(-1)
     &  - 1.D0/6.D0*v**2*pi**2
     &  + 65.D0/72.D0*v**2*ci*pi
     &  + 11.D0/36.D0*v**3*pi**2*omumv**(-2)
     &
      col5 = col5 - 11.D0/36.D0*v**3*pi**2*omumv**(-1)
     &  - 11.D0/72.D0*v**4*pi**2*omumv**(-2)
     &  - 275.D0/108.D0*u
     &  + 1.D0/72.D0*u*pi**2
     &  - 65.D0/72.D0*u*ci*pi
     &  + 275.D0/108.D0*u*v
     &  - 1.D0/72.D0*u*v*pi**2
     &  + 65.D0/72.D0*u*v*ci*pi
     &  + 275.D0/108.D0*u**2
     &  - 1.D0/72.D0*u**2*pi**2
     &  + 65.D0/72.D0*u**2*ci*pi
     &  + 103.D0/108.D0*GPL1(0)
     &  - 5.D0/144.D0*GPL1(0)*pi**2
     &  + 55.D0/36.D0*GPL1(0)*ci*pi
     &  - 11.D0/12.D0*GPL1(0)*v*omumv**(-1)
     &
      col5 = col5 + 35.D0/24.D0*GPL1(0)*v
     &  + 11.D0/6.D0*GPL1(0)*v**2*omumv**(-1)
     &  - 35.D0/24.D0*GPL1(0)*v**2
     &  - 11.D0/12.D0*GPL1(0)*v**3*omumv**(-1)
     &  - 77.D0/72.D0*GPL1(0)*u
     &  + 3.D0/8.D0*GPL1(0)*u*v
     &  - 11.D0/12.D0*GPL1(0)*u**2*v**(-1)
     &  + 43.D0/36.D0*GPL1(0)*u**2
     &  + 11.D0/12.D0*GPL1(0)*u**3*v**(-1)
     &  - 1.D0/36.D0*GPL1(0)*HPL1(0)
     &  + 5.D0/12.D0*GPL1(0)*HPL1(0)*ci*pi
     &  - 7.D0/6.D0*GPL1(0)*HPL1(0)*v*omumv**(-1)
     &  + 5.D0/4.D0*GPL1(0)*HPL1(0)*v
     &  - 11.D0/12.D0*GPL1(0)*HPL1(0)*v**2*omumv**(-2)
     &  + 3*GPL1(0)*HPL1(0)*v**2*omumv**(-1)
     &
      col5 = col5 - GPL1(0)*HPL1(0)*v**2
     &  + 11.D0/6.D0*GPL1(0)*HPL1(0)*v**3*omumv**(-2)
     &  - 11.D0/6.D0*GPL1(0)*HPL1(0)*v**3*omumv**(-1)
     &  - 11.D0/12.D0*GPL1(0)*HPL1(0)*v**4*omumv**(-2)
     &  + 1.D0/12.D0*GPL1(0)*HPL1(0)*u
     &  - 1.D0/12.D0*GPL1(0)*HPL1(0)*u*v
     &  - 1.D0/12.D0*GPL1(0)*HPL1(0)*u**2
     &  + 1.D0/36.D0*GPL1(0)*HPL1(1)
     &  - 5.D0/12.D0*GPL1(0)*HPL1(1)*ci*pi
     &  - 1.D0/12.D0*GPL1(0)*HPL1(1)*v
     &  + 1.D0/12.D0*GPL1(0)*HPL1(1)*v**2
     &  + 7.D0/6.D0*GPL1(0)*HPL1(1)*u*v**(-1)
     &  - 5.D0/4.D0*GPL1(0)*HPL1(1)*u
     &  + 1.D0/12.D0*GPL1(0)*HPL1(1)*u*v
     &  + 11.D0/12.D0*GPL1(0)*HPL1(1)*u**2*v**(-2)
     &
      col5 = col5 - 3*GPL1(0)*HPL1(1)*u**2*v**(-1)
     &  + GPL1(0)*HPL1(1)*u**2
     &  - 11.D0/6.D0*GPL1(0)*HPL1(1)*u**3*v**(-2)
     &  + 11.D0/6.D0*GPL1(0)*HPL1(1)*u**3*v**(-1)
     &  + 11.D0/12.D0*GPL1(0)*HPL1(1)*u**4*v**(-2)
     &  - 1.D0/3.D0*GPL1(0)*HPL2(0,0)
     &  - 1.D0/12.D0*GPL1(0)*HPL2(0,1)
     &  - 1.D0/12.D0*GPL1(0)*HPL2(1,0)
     &  - 1.D0/3.D0*GPL1(0)*HPL2(1,1)
     &  - 1.D0/18.D0*GPL1(1)*pi**2
     &  + 35.D0/216.D0*GPL1(2)
     &  - 11.D0/12.D0*GPL1(2)*u**(-1)*v
     &  + 11.D0/6.D0*GPL1(2)*u**(-1)*v**2
     &  - 11.D0/12.D0*GPL1(2)*u**(-1)*v**3
     &  + 5.D0/48.D0*GPL1(2)*pi**2
     &
      col5 = col5 + 55.D0/36.D0*GPL1(2)*ci*pi
     &  + 169.D0/72.D0*GPL1(2)*v
     &  - 14.D0/9.D0*GPL1(2)*v**2
     &  - 11.D0/12.D0*GPL1(2)*u*v**(-1)
     &  + 169.D0/72.D0*GPL1(2)*u
     &  - 53.D0/72.D0*GPL1(2)*u*v
     &  + 11.D0/6.D0*GPL1(2)*u**2*v**(-1)
     &  - 14.D0/9.D0*GPL1(2)*u**2
     &  - 11.D0/12.D0*GPL1(2)*u**3*v**(-1)
     &  - 1.D0/36.D0*GPL1(2)*HPL1(0)
     &  - 11.D0/12.D0*GPL1(2)*HPL1(0)*u**(-2)*v**2
     &  + 11.D0/6.D0*GPL1(2)*HPL1(0)*u**(-2)*v**3
     &  - 11.D0/12.D0*GPL1(2)*HPL1(0)*u**(-2)*v**4
     &  - 7.D0/6.D0*GPL1(2)*HPL1(0)*u**(-1)*v
     &  + 3*GPL1(2)*HPL1(0)*u**(-1)*v**2
     &
      col5 = col5 - 11.D0/6.D0*GPL1(2)*HPL1(0)*u**(-1)*v**3
     &  + 5.D0/12.D0*GPL1(2)*HPL1(0)*ci*pi
     &  + 5.D0/4.D0*GPL1(2)*HPL1(0)*v
     &  - GPL1(2)*HPL1(0)*v**2
     &  + 1.D0/12.D0*GPL1(2)*HPL1(0)*u
     &  - 1.D0/12.D0*GPL1(2)*HPL1(0)*u*v
     &  - 1.D0/12.D0*GPL1(2)*HPL1(0)*u**2
     &  + 11.D0/12.D0*GPL1(2)*HPL1(1)
     &  - 1.D0/3.D0*GPL1(2)*HPL2(0,0)
     &  - 1.D0/2.D0*GPL1(2)*HPL2(0,1)
     &  + 1.D0/3.D0*GPL1(2)*HPL2(1,0)
     &  + 5.D0/9.D0*GPL1(3)*HPL1(1)
     &  - 11.D0/12.D0*GPL1(3)*HPL1(1)*u**(-2)*v**2
     &  + 11.D0/6.D0*GPL1(3)*HPL1(1)*u**(-2)*v**3
     &  - 11.D0/12.D0*GPL1(3)*HPL1(1)*u**(-2)*v**4
     &
      col5 = col5 - 7.D0/6.D0*GPL1(3)*HPL1(1)*u**(-1)*v
     &  + 3*GPL1(3)*HPL1(1)*u**(-1)*v**2
     &  - 11.D0/6.D0*GPL1(3)*HPL1(1)*u**(-1)*v**3
     &  + 5.D0/6.D0*GPL1(3)*HPL1(1)*ci*pi
     &  + 4.D0/3.D0*GPL1(3)*HPL1(1)*v
     &  - 13.D0/12.D0*GPL1(3)*HPL1(1)*v**2
     &  - 7.D0/6.D0*GPL1(3)*HPL1(1)*u*v**(-1)
     &  + 4.D0/3.D0*GPL1(3)*HPL1(1)*u
     &  - 1.D0/6.D0*GPL1(3)*HPL1(1)*u*v
     &  - 11.D0/12.D0*GPL1(3)*HPL1(1)*u**2*v**(-2)
     &  + 3*GPL1(3)*HPL1(1)*u**2*v**(-1)
     &  - 13.D0/12.D0*GPL1(3)*HPL1(1)*u**2
     &  + 11.D0/6.D0*GPL1(3)*HPL1(1)*u**3*v**(-2)
     &  - 11.D0/6.D0*GPL1(3)*HPL1(1)*u**3*v**(-1)
     &  - 11.D0/12.D0*GPL1(3)*HPL1(1)*u**4*v**(-2)
     &
      col5 = col5 - 1.D0/3.D0*GPL1(3)*HPL2(0,1)
     &  + 2.D0/3.D0*GPL1(3)*HPL2(1,1)
     &  - 11.D0/12.D0*GPL2(0,0)
     &  - 1.D0/3.D0*GPL2(0,0)*HPL1(0)
     &  + 1.D0/3.D0*GPL2(0,0)*HPL1(1)
     &  - 1.D0/36.D0*GPL2(0,2)
     &  + 5.D0/12.D0*GPL2(0,2)*ci*pi
     &  + 1.D0/12.D0*GPL2(0,2)*v
     &  - 1.D0/12.D0*GPL2(0,2)*v**2
     &  - 7.D0/6.D0*GPL2(0,2)*u*v**(-1)
     &  + 5.D0/4.D0*GPL2(0,2)*u
     &  - 1.D0/12.D0*GPL2(0,2)*u*v
     &  - 11.D0/12.D0*GPL2(0,2)*u**2*v**(-2)
     &  + 3*GPL2(0,2)*u**2*v**(-1)
     &  - GPL2(0,2)*u**2
     &
      col5 = col5 + 11.D0/6.D0*GPL2(0,2)*u**3*v**(-2)
     &  - 11.D0/6.D0*GPL2(0,2)*u**3*v**(-1)
     &  - 11.D0/12.D0*GPL2(0,2)*u**4*v**(-2)
     &  + 1.D0/12.D0*GPL2(0,2)*HPL1(0)
     &  + 1.D0/3.D0*GPL2(0,2)*HPL1(1)
     &  - 5.D0/9.D0*GPL2(1,0)
     &  - 5.D0/6.D0*GPL2(1,0)*ci*pi
     &  + 7.D0/6.D0*GPL2(1,0)*v*omumv**(-1)
     &  - 4.D0/3.D0*GPL2(1,0)*v
     &  + 11.D0/12.D0*GPL2(1,0)*v**2*omumv**(-2)
     &  - 3*GPL2(1,0)*v**2*omumv**(-1)
     &  + 13.D0/12.D0*GPL2(1,0)*v**2
     &  - 11.D0/6.D0*GPL2(1,0)*v**3*omumv**(-2)
     &  + 11.D0/6.D0*GPL2(1,0)*v**3*omumv**(-1)
     &  + 11.D0/12.D0*GPL2(1,0)*v**4*omumv**(-2)
     &
      col5 = col5 + 7.D0/6.D0*GPL2(1,0)*u*v**(-1)
     &  - 4.D0/3.D0*GPL2(1,0)*u
     &  + 1.D0/6.D0*GPL2(1,0)*u*v
     &  + 11.D0/12.D0*GPL2(1,0)*u**2*v**(-2)
     &  - 3*GPL2(1,0)*u**2*v**(-1)
     &  + 13.D0/12.D0*GPL2(1,0)*u**2
     &  - 11.D0/6.D0*GPL2(1,0)*u**3*v**(-2)
     &  + 11.D0/6.D0*GPL2(1,0)*u**3*v**(-1)
     &  + 11.D0/12.D0*GPL2(1,0)*u**4*v**(-2)
     &  - 1.D0/36.D0*GPL2(2,0)
     &  + 5.D0/12.D0*GPL2(2,0)*ci*pi
     &  + 1.D0/12.D0*GPL2(2,0)*v
     &  - 1.D0/12.D0*GPL2(2,0)*v**2
     &  - 7.D0/6.D0*GPL2(2,0)*u*v**(-1)
     &  + 5.D0/4.D0*GPL2(2,0)*u
     &
      col5 = col5 - 1.D0/12.D0*GPL2(2,0)*u*v
     &  - 11.D0/12.D0*GPL2(2,0)*u**2*v**(-2)
     &  + 3*GPL2(2,0)*u**2*v**(-1)
     &  - GPL2(2,0)*u**2
     &  + 11.D0/6.D0*GPL2(2,0)*u**3*v**(-2)
     &  - 11.D0/6.D0*GPL2(2,0)*u**3*v**(-1)
     &  - 11.D0/12.D0*GPL2(2,0)*u**4*v**(-2)
     &  + 1.D0/12.D0*GPL2(2,0)*HPL1(0)
     &  + 1.D0/3.D0*GPL2(2,0)*HPL1(1)
     &  - 11.D0/12.D0*GPL2(2,2)
     &  - 1.D0/3.D0*GPL2(2,2)*HPL1(0)
     &  - 5.D0/6.D0*GPL2(2,3)*HPL1(1)
     &  - 5.D0/9.D0*GPL2(3,2)
     &  + 11.D0/12.D0*GPL2(3,2)*u**(-2)*v**2
     &  - 11.D0/6.D0*GPL2(3,2)*u**(-2)*v**3
     &
      col5 = col5 + 11.D0/12.D0*GPL2(3,2)*u**(-2)*v**4
     &  + 7.D0/6.D0*GPL2(3,2)*u**(-1)*v
     &  - 3*GPL2(3,2)*u**(-1)*v**2
     &  + 11.D0/6.D0*GPL2(3,2)*u**(-1)*v**3
     &  - 5.D0/6.D0*GPL2(3,2)*ci*pi
     &  - 4.D0/3.D0*GPL2(3,2)*v
     &  + 13.D0/12.D0*GPL2(3,2)*v**2
     &  + 7.D0/6.D0*GPL2(3,2)*u*v**(-1)
     &  - 4.D0/3.D0*GPL2(3,2)*u
     &  + 1.D0/6.D0*GPL2(3,2)*u*v
     &  + 11.D0/12.D0*GPL2(3,2)*u**2*v**(-2)
     &  - 3*GPL2(3,2)*u**2*v**(-1)
     &  + 13.D0/12.D0*GPL2(3,2)*u**2
     &  - 11.D0/6.D0*GPL2(3,2)*u**3*v**(-2)
     &  + 11.D0/6.D0*GPL2(3,2)*u**3*v**(-1)
     &
      col5 = col5 + 11.D0/12.D0*GPL2(3,2)*u**4*v**(-2)
     &  - 2.D0/3.D0*GPL2(3,2)*HPL1(1)
     &  - 1.D0/3.D0*GPL2(3,3)*HPL1(1)
     &  - 1.D0/3.D0*GPL3(0,0,2)
     &  + 5.D0/6.D0*GPL3(0,1,0)
     &  - 1.D0/3.D0*GPL3(0,2,0)
     &  - 1.D0/3.D0*GPL3(0,2,2)
     &  + 2.D0/3.D0*GPL3(1,0,0)
     &  + 1.D0/3.D0*GPL3(1,1,0)
     &  - 1.D0/3.D0*GPL3(2,0,0)
     &  - 1.D0/3.D0*GPL3(2,0,2)
     &  - 1.D0/3.D0*GPL3(2,2,0)
     &  + 5.D0/6.D0*GPL3(2,3,2)
     &  + 2.D0/3.D0*GPL3(3,2,2)
     &  + 1.D0/3.D0*GPL3(3,3,2)
     &
      col5 = col5 + 103.D0/108.D0*HPL1(0)
     &  - 11.D0/12.D0*HPL1(0)*u**(-1)*v**2
     &  + 11.D0/12.D0*HPL1(0)*u**(-1)*v**3
     &  - 5.D0/144.D0*HPL1(0)*pi**2
     &  + 55.D0/36.D0*HPL1(0)*ci*pi
     &  - 77.D0/72.D0*HPL1(0)*v
     &  - 11.D0/12.D0*HPL1(0)*v**2*omumv**(-1)
     &  + 19.D0/9.D0*HPL1(0)*v**2
     &  + 11.D0/12.D0*HPL1(0)*v**3*omumv**(-1)
     &  + 13.D0/24.D0*HPL1(0)*u
     &  - 13.D0/24.D0*HPL1(0)*u*v
     &  - 13.D0/24.D0*HPL1(0)*u**2
     &  - 35.D0/216.D0*HPL1(1)
     &  + 11.D0/12.D0*HPL1(1)*u**(-1)*v
     &  - 11.D0/6.D0*HPL1(1)*u**(-1)*v**2
     &
      col5 = col5 + 11.D0/12.D0*HPL1(1)*u**(-1)*v**3
     &  - 7.D0/144.D0*HPL1(1)*pi**2
     &  - 55.D0/36.D0*HPL1(1)*ci*pi
     &  - 169.D0/72.D0*HPL1(1)*v
     &  + 14.D0/9.D0*HPL1(1)*v**2
     &  + 11.D0/12.D0*HPL1(1)*u*v**(-1)
     &  - 169.D0/72.D0*HPL1(1)*u
     &  + 53.D0/72.D0*HPL1(1)*u*v
     &  - 11.D0/6.D0*HPL1(1)*u**2*v**(-1)
     &  + 14.D0/9.D0*HPL1(1)*u**2
     &  + 11.D0/12.D0*HPL1(1)*u**3*v**(-1)
     &  - 11.D0/12.D0*HPL2(0,0)
     &  + 7.D0/12.D0*HPL2(0,1)
     &  + 5.D0/12.D0*HPL2(0,1)*ci*pi
     &  + 1.D0/12.D0*HPL2(0,1)*v
     &
      col5 = col5 - 1.D0/12.D0*HPL2(0,1)*v**2
     &  - 7.D0/6.D0*HPL2(0,1)*u*v**(-1)
     &  + 5.D0/4.D0*HPL2(0,1)*u
     &  - 1.D0/12.D0*HPL2(0,1)*u*v
     &  - 11.D0/12.D0*HPL2(0,1)*u**2*v**(-2)
     &  + 3*HPL2(0,1)*u**2*v**(-1)
     &  - HPL2(0,1)*u**2
     &  + 11.D0/6.D0*HPL2(0,1)*u**3*v**(-2)
     &  - 11.D0/6.D0*HPL2(0,1)*u**3*v**(-1)
     &  - 11.D0/12.D0*HPL2(0,1)*u**4*v**(-2)
     &  + 7.D0/12.D0*HPL2(1,0)
     &  + 5.D0/12.D0*HPL2(1,0)*ci*pi
     &  - 7.D0/6.D0*HPL2(1,0)*v*omumv**(-1)
     &  + 5.D0/4.D0*HPL2(1,0)*v
     &  - 11.D0/12.D0*HPL2(1,0)*v**2*omumv**(-2)
     &
      col5 = col5 + 3*HPL2(1,0)*v**2*omumv**(-1)
     &  - HPL2(1,0)*v**2
     &  + 11.D0/6.D0*HPL2(1,0)*v**3*omumv**(-2)
     &  - 11.D0/6.D0*HPL2(1,0)*v**3*omumv**(-1)
     &  - 11.D0/12.D0*HPL2(1,0)*v**4*omumv**(-2)
     &  + 1.D0/12.D0*HPL2(1,0)*u
     &  - 1.D0/12.D0*HPL2(1,0)*u*v
     &  - 1.D0/12.D0*HPL2(1,0)*u**2
     &  - 11.D0/12.D0*HPL2(1,1)
     &  - 1.D0/2.D0*HPL3(0,1,0)
     &  + 1.D0/3.D0*HPL3(0,1,1)
     &  - 1.D0/3.D0*HPL3(1,0,0)
     &  + 1.D0/2.D0*HPL3(1,0,1)
     &

      col6 = - 29.D0/144.D0*pi**2
     &  + 5.D0/18.D0*ci*pi
     &  + 5.D0/54.D0*v
     &  + 5.D0/36.D0*v*ci*pi
     &  - 5.D0/54.D0*v**2
     &  - 5.D0/36.D0*v**2*ci*pi
     &  + 5.D0/54.D0*u
     &  + 5.D0/36.D0*u*ci*pi
     &  - 5.D0/54.D0*u*v
     &  - 5.D0/36.D0*u*v*ci*pi
     &  - 5.D0/54.D0*u**2
     &  - 5.D0/36.D0*u**2*ci*pi
     &  - 5.D0/54.D0*GPL1(0)
     &  - 5.D0/36.D0*GPL1(0)*ci*pi
     &  - 1.D0/12.D0*GPL1(0)*v
     &
      col6 = col6 + 1.D0/12.D0*GPL1(0)*v**2
     &  - 1.D0/36.D0*GPL1(0)*u
     &  + 1.D0/12.D0*GPL1(0)*u*v
     &  + 1.D0/36.D0*GPL1(0)*u**2
     &  + 1.D0/36.D0*GPL1(0)*HPL1(0)
     &  - 1.D0/36.D0*GPL1(0)*HPL1(1)
     &  - 5.D0/54.D0*GPL1(2)
     &  - 5.D0/36.D0*GPL1(2)*ci*pi
     &  - 1.D0/36.D0*GPL1(2)*v
     &  + 1.D0/36.D0*GPL1(2)*v**2
     &  - 1.D0/36.D0*GPL1(2)*u
     &  - 1.D0/36.D0*GPL1(2)*u*v
     &  + 1.D0/36.D0*GPL1(2)*u**2
     &  + 1.D0/36.D0*GPL1(2)*HPL1(0)
     &  - 1.D0/12.D0*GPL1(2)*HPL1(1)
     &
      col6 = col6 + 1.D0/12.D0*GPL2(0,0)
     &  + 1.D0/36.D0*GPL2(0,2)
     &  + 1.D0/36.D0*GPL2(2,0)
     &  + 1.D0/12.D0*GPL2(2,2)
     &  - 5.D0/54.D0*HPL1(0)
     &  - 5.D0/36.D0*HPL1(0)*ci*pi
     &  - 1.D0/36.D0*HPL1(0)*v
     &  + 1.D0/36.D0*HPL1(0)*v**2
     &  - 1.D0/12.D0*HPL1(0)*u
     &  + 1.D0/12.D0*HPL1(0)*u*v
     &  + 1.D0/12.D0*HPL1(0)*u**2
     &  + 5.D0/54.D0*HPL1(1)
     &  + 5.D0/36.D0*HPL1(1)*ci*pi
     &  + 1.D0/36.D0*HPL1(1)*v
     &  - 1.D0/36.D0*HPL1(1)*v**2
     &
      col6 = col6 + 1.D0/36.D0*HPL1(1)*u
     &  + 1.D0/36.D0*HPL1(1)*u*v
     &  - 1.D0/36.D0*HPL1(1)*u**2
     &  + 1.D0/12.D0*HPL2(0,0)
     &  - 1.D0/36.D0*HPL2(0,1)
     &  - 1.D0/36.D0*HPL2(1,0)
     &  + 1.D0/12.D0*HPL2(1,1)

      endif

      col(1) = col1
      col(2)=  col2
      col(3) = col3
      col(4) = col4
      col(5) = col5
      col(6) = col6

      end subroutine alpha1a2L

!#############################################################

      subroutine beta1a2L(u,v,col)
      implicit none
      !returns beta(1a+)
      ! at two Loops
      !-------------
      real*8, intent(in) :: u,v
      complex*16, dimension(6), intent(out) :: col
      !internal
      complex*16 :: beta
      complex*16 :: ci
      real*8 :: omu,omv,omumv,upv
      real*8, dimension(0:1) :: HPL1
      real*8, dimension(0:1,0:1) :: HPL2
      real*8, dimension(0:1,0:1,0:1) :: HPL3
      real*8, dimension(0:1,0:1,0:1,0:1) :: HPL4
      real*8, dimension(0:3) :: GPL1
      real*8, dimension(0:3,0:3) :: GPL2
      real*8, dimension(0:3,0:3,0:3) :: GPL3
      real*8, dimension(0:3,0:3,0:3,0:3) :: GPL4
      complex*16 :: col1,col2,col3,col4,col5,col6


      omu   = 1.0d0 - u
      omv   = 1.0d0     - v
      omumv = 1.0d0 - u - v
      upv   =         u + v

      ci = (0.d0,1.d0)

      call tdhpl(dble(u),dble(v),4,GPL1,GPL2,GPL3,GPL4,HPL1,HPL2,HPL3,HPL4)

      if(ieorder==4) then
         col1 = 9.0d0/8.0d0

         col2 = 0.0d0

         col3 = 0.0d0

         col4 = 0.0d0

         col5 = 0.0d0

         col6 = 0.0d0
      elseif(ieorder==3) then
         col1 = + 99.0d0/16.0d0
     .          + 9.0d0/4.0d0*pi*ci
     .          - 3.0d0/4.0d0*HPL1(0)
     .          + 3.0d0/4.0d0*HPL1(1)
     .          - 3.0d0/4.0d0*GPL1(0)
     .          - 3.0d0/4.0d0*GPL1(2)

         col2 = 0

         col3 = 0

         col4 = 0

         col5 = - 9.0d0/8.0d0

         col6 = 0
      else
      col1 = + 943.D0/216.D0
     &  + 4.D0/3.D0*omu**(-1)
     &  + 4.D0/3.D0*omv**(-1)
     &  - 11.D0/12.D0*omumv**(-1)
     &  - 1.D0/36.D0*pi**2*omumv**(-2)
     &  + 1.D0/36.D0*pi**2*omumv**(-1)
     &  - 5029.D0/864.D0*pi**2
     &  + 49.D0/2880.D0*pi**4
     &  + 185.D0/24.D0*ci*pi
     &  - 77.D0/288.D0*ci*pi**3
     &  + 1.D0/3.D0*zeta3*omumv**(-3)
     &  - 1.D0/2.D0*zeta3*omumv**(-2)
     &  + 2*zeta3*omumv**(-1)
     &  - 121.D0/48.D0*zeta3
     &  + 3.D0/4.D0*zeta3*ci*pi
     &
      col1 = col1 + 337.D0/108.D0*v*omumv**(-2)
     &  - 337.D0/108.D0*v*omumv**(-1)
     &  - 1.D0/9.D0*v*pi**2*omumv**(-3)
     &  - 1.D0/6.D0*v*pi**2*omumv**(-2)
     &  + 5.D0/18.D0*v*pi**2*omumv**(-1)
     &  + 55.D0/72.D0*v*ci*pi*omumv**(-2)
     &  - 55.D0/72.D0*v*ci*pi*omumv**(-1)
     &  - v*zeta3*omumv**(-3)
     &  + v*zeta3*omumv**(-2)
     &  - 337.D0/108.D0*v**2*omumv**(-2)
     &  + 5.D0/12.D0*v**2*pi**2*omumv**(-4)
     &  - 13.D0/18.D0*v**2*pi**2*omumv**(-3)
     &  + 25.D0/36.D0*v**2*pi**2*omumv**(-2)
     &  - 55.D0/72.D0*v**2*ci*pi*omumv**(-2)
     &  + v**2*zeta3*omumv**(-3)
     &
      col1 = col1 - 5.D0/6.D0*v**3*pi**2*omumv**(-4)
     &  + 5.D0/6.D0*v**3*pi**2*omumv**(-3)
     &  + 5.D0/12.D0*v**4*pi**2*omumv**(-4)
     &  - 77.D0/24.D0*GPL1(0)
     &  + 5.D0/6.D0*GPL1(0)*v**(-1)*omumv**(-1)
     &  - 5.D0/6.D0*GPL1(0)*v**(-1)
     &  + 4.D0/3.D0*GPL1(0)*omu**(-2)
     &  + 5.D0/3.D0*GPL1(0)*omu**(-1)
     &  - 7.D0/4.D0*GPL1(0)*omumv**(-2)
     &  - 1.D0/2.D0*GPL1(0)*omumv**(-1)
     &  - 1.D0/36.D0*GPL1(0)*pi**2*omumv**(-3)
     &  + 1.D0/24.D0*GPL1(0)*pi**2*omumv**(-2)
     &  - 1.D0/6.D0*GPL1(0)*pi**2*omumv**(-1)
     &  + 11.D0/32.D0*GPL1(0)*pi**2
     &  - 605.D0/144.D0*GPL1(0)*ci*pi
     &
      col1 = col1 - 1.D0/4.D0*GPL1(0)*zeta3
     &  + 5.D0/2.D0*GPL1(0)*v*omumv**(-3)
     &  - 155.D0/72.D0*GPL1(0)*v*omumv**(-2)
     &  + 203.D0/72.D0*GPL1(0)*v*omumv**(-1)
     &  + 1.D0/12.D0*GPL1(0)*v*pi**2*omumv**(-3)
     &  - 1.D0/12.D0*GPL1(0)*v*pi**2*omumv**(-2)
     &  + 1.D0/6.D0*GPL1(0)*v*pi**2*omumv**(-1)
     &  - 5*GPL1(0)*v**2*omumv**(-3)
     &  + 281.D0/72.D0*GPL1(0)*v**2*omumv**(-2)
     &  - 1.D0/12.D0*GPL1(0)*v**2*pi**2*omumv**(-3)
     &  + 1.D0/24.D0*GPL1(0)*v**2*pi**2*omumv**(-2)
     &  + 5.D0/2.D0*GPL1(0)*v**3*omumv**(-3)
     &  + 1.D0/36.D0*GPL1(0)*v**3*pi**2*omumv**(-3)
     &  - 5.D0/6.D0*GPL1(0)*u*v**(-1)
     &  - 49.D0/48.D0*GPL1(0)*HPL1(0)
     &
      col1 = col1 + 1.D0/8.D0*GPL1(0)*HPL1(0)*pi**2
     &  - 55.D0/24.D0*GPL1(0)*HPL1(0)*ci*pi
     &  - 2.D0/3.D0*GPL1(0)*HPL1(0)*v*omumv**(-3)
     &  - 4.D0/3.D0*GPL1(0)*HPL1(0)*v*omumv**(-2)
     &  + 2*GPL1(0)*HPL1(0)*v*omumv**(-1)
     &  + 5.D0/2.D0*GPL1(0)*HPL1(0)*v**2*omumv**(-4)
     &  - 13.D0/3.D0*GPL1(0)*HPL1(0)*v**2*omumv**(-3)
     &  + 9.D0/2.D0*GPL1(0)*HPL1(0)*v**2*omumv**(-2)
     &  - 5*GPL1(0)*HPL1(0)*v**3*omumv**(-4)
     &  + 5*GPL1(0)*HPL1(0)*v**3*omumv**(-3)
     &  + 5.D0/2.D0*GPL1(0)*HPL1(0)*v**4*omumv**(-4)
     &  + 49.D0/48.D0*GPL1(0)*HPL1(1)
     &  - 1.D0/8.D0*GPL1(0)*HPL1(1)*pi**2
     &  + 55.D0/24.D0*GPL1(0)*HPL1(1)*ci*pi
     &  + 1.D0/12.D0*GPL1(0)*HPL1(1)*v*omumv**(-2)
     &
      col1 = col1 - 1.D0/12.D0*GPL1(0)*HPL1(1)*v*omumv**(-1)
     &  - 1.D0/12.D0*GPL1(0)*HPL1(1)*v**2*omumv**(-2)
     &  - 25.D0/12.D0*GPL1(0)*HPL1(1)*u*v**(-1)
     &  - 5.D0/6.D0*GPL1(0)*HPL1(1)*u**2*v**(-2)
     &  + 11.D0/6.D0*GPL1(0)*HPL2(0,0)
     &  + 11.D0/24.D0*GPL1(0)*HPL2(0,1)
     &  + 11.D0/8.D0*GPL1(0)*HPL2(1,0)
     &  - 1.D0/6.D0*GPL1(0)*HPL2(1,0)*omumv**(-3)
     &  + 1.D0/4.D0*GPL1(0)*HPL2(1,0)*omumv**(-2)
     &  - GPL1(0)*HPL2(1,0)*omumv**(-1)
     &  + 1.D0/2.D0*GPL1(0)*HPL2(1,0)*v*omumv**(-3)
     &  - 1.D0/2.D0*GPL1(0)*HPL2(1,0)*v*omumv**(-2)
     &  + GPL1(0)*HPL2(1,0)*v*omumv**(-1)
     &  - 1.D0/2.D0*GPL1(0)*HPL2(1,0)*v**2*omumv**(-3)
     &  + 1.D0/4.D0*GPL1(0)*HPL2(1,0)*v**2*omumv**(-2)
     &
      col1 = col1 + 1.D0/6.D0*GPL1(0)*HPL2(1,0)*v**3*omumv**(-3)
     &  + 11.D0/6.D0*GPL1(0)*HPL2(1,1)
     &  + 1.D0/2.D0*GPL1(0)*HPL3(0,0,1)
     &  + 1.D0/2.D0*GPL1(0)*HPL3(0,1,0)
     &  - 1.D0/2.D0*GPL1(0)*HPL3(0,1,1)
     &  + 1.D0/2.D0*GPL1(0)*HPL3(1,0,0)
     &  - 1.D0/2.D0*GPL1(0)*HPL3(1,0,1)
     &  - 1.D0/2.D0*GPL1(0)*HPL3(1,1,0)
     &  + 1.D0/3.D0*GPL1(1)*v*pi**2*omumv**(-1)
     &  + 1.D0/12.D0*GPL1(1)*v**2*pi**2*omumv**(-2)
     &  + 1.D0/18.D0*GPL1(1)*v**3*pi**2*omumv**(-3)
     &  + 1.D0/8.D0*GPL1(2)
     &  + 5.D0/6.D0*GPL1(2)*u**(-1)*v
     &  + 11.D0/288.D0*GPL1(2)*pi**2
     &  - 605.D0/144.D0*GPL1(2)*ci*pi
     &
      col1 = col1 - 1.D0/4.D0*GPL1(2)*zeta3
     &  - 11.D0/24.D0*GPL1(2)*v*omumv**(-2)
     &  + 11.D0/24.D0*GPL1(2)*v*omumv**(-1)
     &  + 11.D0/24.D0*GPL1(2)*v**2*omumv**(-2)
     &  + 5.D0/6.D0*GPL1(2)*u*v**(-1)
     &  - 49.D0/48.D0*GPL1(2)*HPL1(0)
     &  + 5.D0/6.D0*GPL1(2)*HPL1(0)*u**(-2)*v**2
     &  + 25.D0/12.D0*GPL1(2)*HPL1(0)*u**(-1)*v
     &  + 1.D0/8.D0*GPL1(2)*HPL1(0)*pi**2
     &  - 55.D0/24.D0*GPL1(2)*HPL1(0)*ci*pi
     &  - 1.D0/12.D0*GPL1(2)*HPL1(0)*v*omumv**(-2)
     &  + 1.D0/12.D0*GPL1(2)*HPL1(0)*v*omumv**(-1)
     &  + 1.D0/12.D0*GPL1(2)*HPL1(0)*v**2*omumv**(-2)
     &  - 121.D0/48.D0*GPL1(2)*HPL1(1)
     &  + 1.D0/3.D0*GPL1(2)*HPL1(1)*pi**2
     &
      col1 = col1 + 11.D0/6.D0*GPL1(2)*HPL2(0,0)
     &  + 11.D0/12.D0*GPL1(2)*HPL2(0,1)
     &  - 11.D0/12.D0*GPL1(2)*HPL2(1,0)
     &  - 3.D0/2.D0*GPL1(2)*HPL3(0,0,1)
     &  + 1.D0/2.D0*GPL1(2)*HPL3(0,1,0)
     &  + 1.D0/2.D0*GPL1(2)*HPL3(1,0,0)
     &  + 2*GPL1(2)*HPL3(1,1,0)
     &  - 67.D0/18.D0*GPL1(3)*HPL1(1)
     &  + 5.D0/6.D0*GPL1(3)*HPL1(1)*u**(-2)*v**2
     &  + 25.D0/12.D0*GPL1(3)*HPL1(1)*u**(-1)*v
     &  - 1.D0/12.D0*GPL1(3)*HPL1(1)*pi**2
     &  - 55.D0/12.D0*GPL1(3)*HPL1(1)*ci*pi
     &  - 1.D0/6.D0*GPL1(3)*HPL1(1)*v*omumv**(-2)
     &  + 1.D0/6.D0*GPL1(3)*HPL1(1)*v*omumv**(-1)
     &  + 1.D0/6.D0*GPL1(3)*HPL1(1)*v**2*omumv**(-2)
     &
      col1 = col1 + 25.D0/12.D0*GPL1(3)*HPL1(1)*u*v**(-1)
     &  + 5.D0/6.D0*GPL1(3)*HPL1(1)*u**2*v**(-2)
     &  + 11.D0/6.D0*GPL1(3)*HPL2(0,1)
     &  - 11.D0/3.D0*GPL1(3)*HPL2(1,1)
     &  + 2*GPL1(3)*HPL3(0,0,1)
     &  + GPL1(3)*HPL3(0,1,1)
     &  + GPL1(3)*HPL3(1,0,1)
     &  - GPL1(3)*HPL3(1,1,0)
     &  + 121.D0/48.D0*GPL2(0,0)
     &  + 11.D0/6.D0*GPL2(0,0)*HPL1(0)
     &  - 11.D0/6.D0*GPL2(0,0)*HPL1(1)
     &  + 1.D0/2.D0*GPL2(0,0)*HPL2(0,0)
     &  - 1.D0/2.D0*GPL2(0,0)*HPL2(0,1)
     &  - 1.D0/2.D0*GPL2(0,0)*HPL2(1,0)
     &  + 1.D0/2.D0*GPL2(0,0)*HPL2(1,1)
     &
      col1 = col1 - 1.D0/3.D0*GPL2(0,1)*pi**2
     &  - 49.D0/48.D0*GPL2(0,2)
     &  + 1.D0/8.D0*GPL2(0,2)*pi**2
     &  - 55.D0/24.D0*GPL2(0,2)*ci*pi
     &  - 1.D0/12.D0*GPL2(0,2)*v*omumv**(-2)
     &  + 1.D0/12.D0*GPL2(0,2)*v*omumv**(-1)
     &  + 1.D0/12.D0*GPL2(0,2)*v**2*omumv**(-2)
     &  + 25.D0/12.D0*GPL2(0,2)*u*v**(-1)
     &  + 5.D0/6.D0*GPL2(0,2)*u**2*v**(-2)
     &  - 11.D0/24.D0*GPL2(0,2)*HPL1(0)
     &  - 11.D0/6.D0*GPL2(0,2)*HPL1(1)
     &  + 1.D0/2.D0*GPL2(0,2)*HPL2(0,0)
     &  + 1.D0/2.D0*GPL2(0,3)*HPL2(0,1)
     &  + 1.D0/2.D0*GPL2(0,3)*HPL2(1,0)
     &  - GPL2(0,3)*HPL2(1,1)
     &
      col1 = col1 + 67.D0/18.D0*GPL2(1,0)
     &  - 1.D0/6.D0*GPL2(1,0)*omumv**(-2)
     &  + 1.D0/6.D0*GPL2(1,0)*omumv**(-1)
     &  - 1.D0/4.D0*GPL2(1,0)*pi**2
     &  + 55.D0/12.D0*GPL2(1,0)*ci*pi
     &  + 2.D0/3.D0*GPL2(1,0)*v*omumv**(-3)
     &  + 7.D0/4.D0*GPL2(1,0)*v*omumv**(-2)
     &  - 25.D0/12.D0*GPL2(1,0)*v*omumv**(-1)
     &  - 5.D0/2.D0*GPL2(1,0)*v**2*omumv**(-4)
     &  + 13.D0/3.D0*GPL2(1,0)*v**2*omumv**(-3)
     &  - 55.D0/12.D0*GPL2(1,0)*v**2*omumv**(-2)
     &  + 5*GPL2(1,0)*v**3*omumv**(-4)
     &  - 5*GPL2(1,0)*v**3*omumv**(-3)
     &  - 5.D0/2.D0*GPL2(1,0)*v**4*omumv**(-4)
     &  - 25.D0/12.D0*GPL2(1,0)*u*v**(-1)
     &
      col1 = col1 - 5.D0/6.D0*GPL2(1,0)*u**2*v**(-2)
     &  + GPL2(1,0)*HPL1(0)*v*omumv**(-1)
     &  + 1.D0/4.D0*GPL2(1,0)*HPL1(0)*v**2*omumv**(-2)
     &  + 1.D0/6.D0*GPL2(1,0)*HPL1(0)*v**3*omumv**(-3)
     &  + 11.D0/12.D0*GPL2(1,0)*HPL1(1)
     &  - 1.D0/2.D0*GPL2(1,0)*HPL2(0,1)
     &  - 1.D0/2.D0*GPL2(1,0)*HPL2(1,0)
     &  + 1.D0/3.D0*GPL2(1,1)*pi**2
     &  - 49.D0/48.D0*GPL2(2,0)
     &  + 1.D0/8.D0*GPL2(2,0)*pi**2
     &  - 55.D0/24.D0*GPL2(2,0)*ci*pi
     &  - 1.D0/12.D0*GPL2(2,0)*v*omumv**(-2)
     &  + 1.D0/12.D0*GPL2(2,0)*v*omumv**(-1)
     &  + 1.D0/12.D0*GPL2(2,0)*v**2*omumv**(-2)
     &  + 25.D0/12.D0*GPL2(2,0)*u*v**(-1)
     &
      col1 = col1 + 5.D0/6.D0*GPL2(2,0)*u**2*v**(-2)
     &  - 11.D0/24.D0*GPL2(2,0)*HPL1(0)
     &  - 11.D0/6.D0*GPL2(2,0)*HPL1(1)
     &  + 1.D0/2.D0*GPL2(2,0)*HPL2(0,0)
     &  + GPL2(2,0)*HPL2(0,1)
     &  + GPL2(2,0)*HPL2(1,0)
     &  + 121.D0/48.D0*GPL2(2,2)
     &  - 1.D0/3.D0*GPL2(2,2)*pi**2
     &  + 11.D0/6.D0*GPL2(2,2)*HPL1(0)
     &  + 1.D0/2.D0*GPL2(2,2)*HPL2(0,0)
     &  + GPL2(2,2)*HPL2(0,1)
     &  - GPL2(2,2)*HPL2(1,0)
     &  + 11.D0/4.D0*GPL2(2,3)*HPL1(1)
     &  - 3.D0/2.D0*GPL2(2,3)*HPL2(0,1)
     &  + 1.D0/2.D0*GPL2(2,3)*HPL2(1,0)
     &
      col1 = col1 - 1.D0/2.D0*GPL2(3,0)*HPL2(0,1)
     &  - 1.D0/2.D0*GPL2(3,0)*HPL2(1,0)
     &  - GPL2(3,0)*HPL2(1,1)
     &  + 67.D0/18.D0*GPL2(3,2)
     &  - 5.D0/6.D0*GPL2(3,2)*u**(-2)*v**2
     &  - 25.D0/12.D0*GPL2(3,2)*u**(-1)*v
     &  + 1.D0/12.D0*GPL2(3,2)*pi**2
     &  + 55.D0/12.D0*GPL2(3,2)*ci*pi
     &  + 1.D0/6.D0*GPL2(3,2)*v*omumv**(-2)
     &  - 1.D0/6.D0*GPL2(3,2)*v*omumv**(-1)
     &  - 1.D0/6.D0*GPL2(3,2)*v**2*omumv**(-2)
     &  - 25.D0/12.D0*GPL2(3,2)*u*v**(-1)
     &  - 5.D0/6.D0*GPL2(3,2)*u**2*v**(-2)
     &  + 11.D0/3.D0*GPL2(3,2)*HPL1(1)
     &  - GPL2(3,2)*HPL2(0,1)
     &
      col1 = col1 + GPL2(3,2)*HPL2(1,0)
     &  + 11.D0/6.D0*GPL2(3,3)*HPL1(1)
     &  + 2*GPL2(3,3)*HPL2(0,1)
     &  + 2*GPL2(3,3)*HPL2(1,1)
     &  + 11.D0/6.D0*GPL3(0,0,2)
     &  - 1.D0/2.D0*GPL3(0,0,2)*HPL1(0)
     &  - 1.D0/2.D0*GPL3(0,0,2)*HPL1(1)
     &  - GPL3(0,0,3)*HPL1(1)
     &  - 11.D0/3.D0*GPL3(0,1,0)
     &  - 1.D0/6.D0*GPL3(0,1,0)*omumv**(-3)
     &  + 1.D0/4.D0*GPL3(0,1,0)*omumv**(-2)
     &  - GPL3(0,1,0)*omumv**(-1)
     &  + 1.D0/2.D0*GPL3(0,1,0)*v*omumv**(-3)
     &  - 1.D0/2.D0*GPL3(0,1,0)*v*omumv**(-2)
     &  + GPL3(0,1,0)*v*omumv**(-1)
     &
      col1 = col1 - 1.D0/2.D0*GPL3(0,1,0)*v**2*omumv**(-3)
     &  + 1.D0/4.D0*GPL3(0,1,0)*v**2*omumv**(-2)
     &  + 1.D0/6.D0*GPL3(0,1,0)*v**3*omumv**(-3)
     &  - 1.D0/2.D0*GPL3(0,1,0)*HPL1(0)
     &  + 1.D0/2.D0*GPL3(0,1,0)*HPL1(1)
     &  + 11.D0/6.D0*GPL3(0,2,0)
     &  - 1.D0/2.D0*GPL3(0,2,0)*HPL1(0)
     &  - 1.D0/2.D0*GPL3(0,2,0)*HPL1(1)
     &  + 11.D0/6.D0*GPL3(0,2,2)
     &  + 1.D0/2.D0*GPL3(0,2,2)*HPL1(0)
     &  + 1.D0/2.D0*GPL3(0,2,3)*HPL1(1)
     &  - GPL3(0,3,0)*HPL1(1)
     &  - 1.D0/2.D0*GPL3(0,3,2)*HPL1(0)
     &  + GPL3(0,3,2)*HPL1(1)
     &  - 11.D0/3.D0*GPL3(1,0,0)
     &
      col1 = col1 - GPL3(1,0,0)*HPL1(0)
     &  + GPL3(1,0,0)*HPL1(1)
     &  - 11.D0/12.D0*GPL3(1,0,2)
     &  + 1.D0/2.D0*GPL3(1,0,2)*HPL1(0)
     &  - 2*GPL3(1,1,0)*v*omumv**(-1)
     &  - 1.D0/2.D0*GPL3(1,1,0)*v**2*omumv**(-2)
     &  - 1.D0/3.D0*GPL3(1,1,0)*v**3*omumv**(-3)
     &  - 11.D0/12.D0*GPL3(1,2,0)
     &  + 1.D0/2.D0*GPL3(1,2,0)*HPL1(0)
     &  + 11.D0/6.D0*GPL3(2,0,0)
     &  + 1.D0/2.D0*GPL3(2,0,0)*HPL1(0)
     &  - 1.D0/2.D0*GPL3(2,0,0)*HPL1(1)
     &  + 11.D0/6.D0*GPL3(2,0,2)
     &  - 1.D0/2.D0*GPL3(2,0,2)*HPL1(0)
     &  + 1.D0/2.D0*GPL3(2,0,3)*HPL1(1)
     &
      col1 = col1 - 11.D0/12.D0*GPL3(2,1,0)
     &  - 1.D0/2.D0*GPL3(2,1,0)*HPL1(0)
     &  - GPL3(2,1,0)*HPL1(1)
     &  + 11.D0/6.D0*GPL3(2,2,0)
     &  - 1.D0/2.D0*GPL3(2,2,0)*HPL1(0)
     &  + GPL3(2,2,3)*HPL1(1)
     &  + 1.D0/2.D0*GPL3(2,3,0)*HPL1(1)
     &  - 11.D0/4.D0*GPL3(2,3,2)
     &  - 1.D0/2.D0*GPL3(2,3,2)*HPL1(0)
     &  - 2*GPL3(2,3,3)*HPL1(1)
     &  + 1.D0/2.D0*GPL3(3,0,2)*HPL1(0)
     &  + GPL3(3,0,2)*HPL1(1)
     &  + 1.D0/2.D0*GPL3(3,2,0)*HPL1(0)
     &  + GPL3(3,2,0)*HPL1(1)
     &  - 11.D0/3.D0*GPL3(3,2,2)
     &
      col1 = col1 - GPL3(3,2,2)*HPL1(0)
     &  - 2*GPL3(3,2,3)*HPL1(1)
     &  - 11.D0/6.D0*GPL3(3,3,2)
     &  - 2*GPL3(3,3,2)*HPL1(1)
     &  + 2*GPL3(3,3,3)*HPL1(1)
     &  - GPL4(0,0,1,0)
     &  + 1.D0/2.D0*GPL4(0,0,2,2)
     &  + GPL4(0,0,3,2)
     &  - 1.D0/2.D0*GPL4(0,1,0,2)
     &  + 2*GPL4(0,1,1,0)
     &  - 1.D0/2.D0*GPL4(0,1,2,0)
     &  + 1.D0/2.D0*GPL4(0,2,0,2)
     &  - 1.D0/2.D0*GPL4(0,2,1,0)
     &  + 1.D0/2.D0*GPL4(0,2,2,0)
     &  - 1.D0/2.D0*GPL4(0,2,3,2)
     &
      col1 = col1 + GPL4(0,3,0,2)
     &  + GPL4(0,3,2,0)
     &  - GPL4(0,3,2,2)
     &  - GPL4(1,0,0,2)
     &  + 2*GPL4(1,0,1,0)
     &  - GPL4(1,0,2,0)
     &  + 2*GPL4(1,1,0,0)
     &  - 2*GPL4(1,1,1,0)
     &  - GPL4(1,2,0,0)
     &  + 1.D0/2.D0*GPL4(2,0,0,2)
     &  - 1.D0/2.D0*GPL4(2,0,1,0)
     &  + 1.D0/2.D0*GPL4(2,0,2,0)
     &  - 1.D0/2.D0*GPL4(2,0,3,2)
     &  - GPL4(2,1,0,0)
     &  + GPL4(2,1,0,2)
     &
      col1 = col1 + GPL4(2,1,2,0)
     &  + 1.D0/2.D0*GPL4(2,2,0,0)
     &  + GPL4(2,2,1,0)
     &  - GPL4(2,2,3,2)
     &  - 1.D0/2.D0*GPL4(2,3,0,2)
     &  - 1.D0/2.D0*GPL4(2,3,2,0)
     &  + 2*GPL4(2,3,3,2)
     &  - GPL4(3,0,2,2)
     &  - GPL4(3,2,0,2)
     &  - GPL4(3,2,2,0)
     &  + 2*GPL4(3,2,3,2)
     &  + 2*GPL4(3,3,2,2)
     &  - 2*GPL4(3,3,3,2)
     &  - 37.D0/8.D0*HPL1(0)
     &  + 5.D0/6.D0*HPL1(0)*u**(-1)*omv**(-1)
     &
      col1 = col1 - 5.D0/6.D0*HPL1(0)*u**(-1)
     &  - 5.D0/6.D0*HPL1(0)*u**(-1)*v
     &  + 4.D0/3.D0*HPL1(0)*omv**(-2)
     &  + 5.D0/6.D0*HPL1(0)*omv**(-1)*omumv**(-1)
     &  + 5.D0/3.D0*HPL1(0)*omv**(-1)
     &  - 5.D0/6.D0*HPL1(0)*omumv**(-1)
     &  + 55.D0/288.D0*HPL1(0)*pi**2
     &  - 605.D0/144.D0*HPL1(0)*ci*pi
     &  - 1.D0/4.D0*HPL1(0)*zeta3
     &  - 47.D0/72.D0*HPL1(0)*v*omumv**(-2)
     &  - 181.D0/72.D0*HPL1(0)*v*omumv**(-1)
     &  - 1.D0/6.D0*HPL1(0)*v*pi**2*omumv**(-1)
     &  + 5.D0/2.D0*HPL1(0)*v**2*omumv**(-3)
     &  - 259.D0/72.D0*HPL1(0)*v**2*omumv**(-2)
     &  - 1.D0/24.D0*HPL1(0)*v**2*pi**2*omumv**(-2)
     &
      col1 = col1 - 5.D0/2.D0*HPL1(0)*v**3*omumv**(-3)
     &  - 1.D0/36.D0*HPL1(0)*v**3*pi**2*omumv**(-3)
     &  - 1.D0/8.D0*HPL1(1)
     &  - 5.D0/6.D0*HPL1(1)*u**(-1)*v
     &  - 1.D0/18.D0*HPL1(1)*pi**2*omumv**(-3)
     &  + 1.D0/12.D0*HPL1(1)*pi**2*omumv**(-2)
     &  - 1.D0/3.D0*HPL1(1)*pi**2*omumv**(-1)
     &  + 77.D0/288.D0*HPL1(1)*pi**2
     &  + 605.D0/144.D0*HPL1(1)*ci*pi
     &  + 1.D0/4.D0*HPL1(1)*zeta3
     &  + 11.D0/24.D0*HPL1(1)*v*omumv**(-2)
     &  - 11.D0/24.D0*HPL1(1)*v*omumv**(-1)
     &  + 1.D0/6.D0*HPL1(1)*v*pi**2*omumv**(-3)
     &  - 1.D0/6.D0*HPL1(1)*v*pi**2*omumv**(-2)
     &  + 1.D0/3.D0*HPL1(1)*v*pi**2*omumv**(-1)
     &
      col1 = col1 - 11.D0/24.D0*HPL1(1)*v**2*omumv**(-2)
     &  - 1.D0/6.D0*HPL1(1)*v**2*pi**2*omumv**(-3)
     &  + 1.D0/12.D0*HPL1(1)*v**2*pi**2*omumv**(-2)
     &  + 1.D0/18.D0*HPL1(1)*v**3*pi**2*omumv**(-3)
     &  - 5.D0/6.D0*HPL1(1)*u*v**(-1)
     &  + 121.D0/48.D0*HPL2(0,0)
     &  - 389.D0/144.D0*HPL2(0,1)
     &  + 1.D0/8.D0*HPL2(0,1)*pi**2
     &  - 55.D0/24.D0*HPL2(0,1)*ci*pi
     &  - 1.D0/12.D0*HPL2(0,1)*v*omumv**(-2)
     &  + 1.D0/12.D0*HPL2(0,1)*v*omumv**(-1)
     &  + 1.D0/12.D0*HPL2(0,1)*v**2*omumv**(-2)
     &  + 25.D0/12.D0*HPL2(0,1)*u*v**(-1)
     &  + 5.D0/6.D0*HPL2(0,1)*u**2*v**(-2)
     &  - 389.D0/144.D0*HPL2(1,0)
     &
      col1 = col1 - 1.D0/6.D0*HPL2(1,0)*omumv**(-2)
     &  + 1.D0/6.D0*HPL2(1,0)*omumv**(-1)
     &  + 1.D0/8.D0*HPL2(1,0)*pi**2
     &  - 55.D0/24.D0*HPL2(1,0)*ci*pi
     &  - 2.D0/3.D0*HPL2(1,0)*v*omumv**(-3)
     &  - HPL2(1,0)*v*omumv**(-2)
     &  + 2*HPL2(1,0)*v*omumv**(-1)
     &  + 5.D0/2.D0*HPL2(1,0)*v**2*omumv**(-4)
     &  - 13.D0/3.D0*HPL2(1,0)*v**2*omumv**(-3)
     &  + 9.D0/2.D0*HPL2(1,0)*v**2*omumv**(-2)
     &  - 5*HPL2(1,0)*v**3*omumv**(-4)
     &  + 5*HPL2(1,0)*v**3*omumv**(-3)
     &  + 5.D0/2.D0*HPL2(1,0)*v**4*omumv**(-4)
     &  + 121.D0/48.D0*HPL2(1,1)
     &  + 11.D0/4.D0*HPL3(0,1,0)
     &
      col1 = col1 + HPL3(0,1,0)*v*omumv**(-1)
     &  + 1.D0/4.D0*HPL3(0,1,0)*v**2*omumv**(-2)
     &  + 1.D0/6.D0*HPL3(0,1,0)*v**3*omumv**(-3)
     &  - 11.D0/6.D0*HPL3(0,1,1)
     &  + 11.D0/6.D0*HPL3(1,0,0)
     &  - 11.D0/6.D0*HPL3(1,0,1)
     &  + 11.D0/6.D0*HPL3(1,1,0)
     &  - 1.D0/3.D0*HPL3(1,1,0)*omumv**(-3)
     &  + 1.D0/2.D0*HPL3(1,1,0)*omumv**(-2)
     &  - 2*HPL3(1,1,0)*omumv**(-1)
     &  + HPL3(1,1,0)*v*omumv**(-3)
     &  - HPL3(1,1,0)*v*omumv**(-2)
     &  + 2*HPL3(1,1,0)*v*omumv**(-1)
     &  - HPL3(1,1,0)*v**2*omumv**(-3)
     &  + 1.D0/2.D0*HPL3(1,1,0)*v**2*omumv**(-2)
     &
      col1 = col1 + 1.D0/3.D0*HPL3(1,1,0)*v**3*omumv**(-3)
     &  + 1.D0/2.D0*HPL4(0,0,1,1)
     &  + 1.D0/2.D0*HPL4(0,1,0,1)
     &  + 1.D0/2.D0*HPL4(0,1,1,0)
     &  + 1.D0/2.D0*HPL4(1,0,0,1)
     &  + 1.D0/2.D0*HPL4(1,0,1,0)
     &  + 1.D0/2.D0*HPL4(1,1,0,0)
     &

      col2 = 0

      col3 = 0

      col4 = + 19.D0/16.D0
     &  + 1.D0/4.D0*omu**(-1)
     &  + 1.D0/4.D0*omv**(-1)
     &  - 7.D0/24.D0*omumv**(-1)
     &  + 1.D0/4.D0*ci*pi
     &  - zeta3
     &  - 1.D0/4.D0*v*omumv**(-2)
     &  + 1.D0/4.D0*v*omumv**(-1)
     &  - 1.D0/18.D0*v*pi**2*omumv**(-3)
     &  + 1.D0/36.D0*v*pi**2*omumv**(-2)
     &  + 1.D0/36.D0*v*pi**2*omumv**(-1)
     &  + 1.D0/4.D0*v**2*omumv**(-2)
     &  - 1.D0/24.D0*v**2*pi**2*omumv**(-4)
     &  + 5.D0/36.D0*v**2*pi**2*omumv**(-3)
     &  - 1.D0/72.D0*v**2*pi**2*omumv**(-2)
     &
      col4 = col4 + 1.D0/12.D0*v**3*pi**2*omumv**(-4)
     &  - 1.D0/12.D0*v**3*pi**2*omumv**(-3)
     &  - 1.D0/24.D0*v**4*pi**2*omumv**(-4)
     &  - 5.D0/24.D0*GPL1(0)
     &  - 1.D0/12.D0*GPL1(0)*v**(-1)*omumv**(-1)
     &  + 1.D0/12.D0*GPL1(0)*v**(-1)
     &  + 1.D0/4.D0*GPL1(0)*omu**(-2)
     &  - 5.D0/24.D0*GPL1(0)*omumv**(-2)
     &  + 1.D0/4.D0*GPL1(0)*omumv**(-1)
     &  - 1.D0/4.D0*GPL1(0)*v*omumv**(-3)
     &  + 7.D0/12.D0*GPL1(0)*v*omumv**(-2)
     &  - 1.D0/12.D0*GPL1(0)*v*omumv**(-1)
     &  + 1.D0/2.D0*GPL1(0)*v**2*omumv**(-3)
     &  - 3.D0/8.D0*GPL1(0)*v**2*omumv**(-2)
     &  - 1.D0/4.D0*GPL1(0)*v**3*omumv**(-3)
     &
      col4 = col4 + 1.D0/12.D0*GPL1(0)*u*v**(-1)
     &  - 1.D0/3.D0*GPL1(0)*HPL1(0)*v*omumv**(-3)
     &  + 1.D0/6.D0*GPL1(0)*HPL1(0)*v*omumv**(-2)
     &  + 1.D0/6.D0*GPL1(0)*HPL1(0)*v*omumv**(-1)
     &  - 1.D0/4.D0*GPL1(0)*HPL1(0)*v**2*omumv**(-4)
     &  + 5.D0/6.D0*GPL1(0)*HPL1(0)*v**2*omumv**(-3)
     &  - 1.D0/12.D0*GPL1(0)*HPL1(0)*v**2*omumv**(-2)
     &  + 1.D0/2.D0*GPL1(0)*HPL1(0)*v**3*omumv**(-4)
     &  - 1.D0/2.D0*GPL1(0)*HPL1(0)*v**3*omumv**(-3)
     &  - 1.D0/4.D0*GPL1(0)*HPL1(0)*v**4*omumv**(-4)
     &  - 1.D0/6.D0*GPL1(0)*HPL1(1)*u*v**(-1)
     &  + 1.D0/12.D0*GPL1(0)*HPL1(1)*u**2*v**(-2)
     &  + 5.D0/24.D0*GPL1(2)
     &  - 1.D0/12.D0*GPL1(2)*u**(-1)*v
     &  - 1.D0/12.D0*GPL1(2)*u*v**(-1)
     &
      col4 = col4 - 1.D0/12.D0*GPL1(2)*HPL1(0)*u**(-2)*v**2
     &  + 1.D0/6.D0*GPL1(2)*HPL1(0)*u**(-1)*v
     &  - 1.D0/12.D0*GPL1(3)*HPL1(1)*u**(-2)*v**2
     &  + 1.D0/6.D0*GPL1(3)*HPL1(1)*u**(-1)*v
     &  + 1.D0/6.D0*GPL1(3)*HPL1(1)*u*v**(-1)
     &  - 1.D0/12.D0*GPL1(3)*HPL1(1)*u**2*v**(-2)
     &  + 1.D0/6.D0*GPL2(0,2)*u*v**(-1)
     &  - 1.D0/12.D0*GPL2(0,2)*u**2*v**(-2)
     &  + 1.D0/3.D0*GPL2(1,0)*v*omumv**(-3)
     &  - 1.D0/6.D0*GPL2(1,0)*v*omumv**(-2)
     &  - 1.D0/6.D0*GPL2(1,0)*v*omumv**(-1)
     &  + 1.D0/4.D0*GPL2(1,0)*v**2*omumv**(-4)
     &  - 5.D0/6.D0*GPL2(1,0)*v**2*omumv**(-3)
     &  + 1.D0/12.D0*GPL2(1,0)*v**2*omumv**(-2)
     &  - 1.D0/2.D0*GPL2(1,0)*v**3*omumv**(-4)
     &
      col4 = col4 + 1.D0/2.D0*GPL2(1,0)*v**3*omumv**(-3)
     &  + 1.D0/4.D0*GPL2(1,0)*v**4*omumv**(-4)
     &  - 1.D0/6.D0*GPL2(1,0)*u*v**(-1)
     &  + 1.D0/12.D0*GPL2(1,0)*u**2*v**(-2)
     &  + 1.D0/6.D0*GPL2(2,0)*u*v**(-1)
     &  - 1.D0/12.D0*GPL2(2,0)*u**2*v**(-2)
     &  + 1.D0/12.D0*GPL2(3,2)*u**(-2)*v**2
     &  - 1.D0/6.D0*GPL2(3,2)*u**(-1)*v
     &  - 1.D0/6.D0*GPL2(3,2)*u*v**(-1)
     &  + 1.D0/12.D0*GPL2(3,2)*u**2*v**(-2)
     &  - 1.D0/4.D0*HPL1(0)
     &  - 1.D0/12.D0*HPL1(0)*u**(-1)*omv**(-1)
     &  + 1.D0/12.D0*HPL1(0)*u**(-1)
     &  + 1.D0/12.D0*HPL1(0)*u**(-1)*v
     &  + 1.D0/4.D0*HPL1(0)*omv**(-2)
     &
      col4 = col4 - 1.D0/12.D0*HPL1(0)*omv**(-1)*omumv**(-1)
     &  + 1.D0/12.D0*HPL1(0)*omumv**(-1)
     &  - 1.D0/3.D0*HPL1(0)*v*omumv**(-2)
     &  + 1.D0/12.D0*HPL1(0)*v*omumv**(-1)
     &  - 1.D0/4.D0*HPL1(0)*v**2*omumv**(-3)
     &  + 3.D0/8.D0*HPL1(0)*v**2*omumv**(-2)
     &  + 1.D0/4.D0*HPL1(0)*v**3*omumv**(-3)
     &  - 5.D0/24.D0*HPL1(1)
     &  + 1.D0/12.D0*HPL1(1)*u**(-1)*v
     &  + 1.D0/12.D0*HPL1(1)*u*v**(-1)
     &  + 1.D0/6.D0*HPL2(0,1)*u*v**(-1)
     &  - 1.D0/12.D0*HPL2(0,1)*u**2*v**(-2)
     &  - 1.D0/3.D0*HPL2(1,0)*v*omumv**(-3)
     &  + 1.D0/6.D0*HPL2(1,0)*v*omumv**(-2)
     &  + 1.D0/6.D0*HPL2(1,0)*v*omumv**(-1)
     &
      col4 = col4 - 1.D0/4.D0*HPL2(1,0)*v**2*omumv**(-4)
     &  + 5.D0/6.D0*HPL2(1,0)*v**2*omumv**(-3)
     &  - 1.D0/12.D0*HPL2(1,0)*v**2*omumv**(-2)
     &  + 1.D0/2.D0*HPL2(1,0)*v**3*omumv**(-4)
     &  - 1.D0/2.D0*HPL2(1,0)*v**3*omumv**(-3)
     &  - 1.D0/4.D0*HPL2(1,0)*v**4*omumv**(-4)
     &

      col5 = - 1115.D0/432.D0
     &  - 13.D0/12.D0*omu**(-1)
     &  - 13.D0/12.D0*omv**(-1)
     &  + 5.D0/8.D0*omumv**(-1)
     &  + 1.D0/36.D0*pi**2*omumv**(-2)
     &  - 1.D0/36.D0*pi**2*omumv**(-1)
     &  + 1879.D0/864.D0*pi**2
     &  - 71.D0/18.D0*ci*pi
     &  + 7.D0/144.D0*ci*pi**3
     &  - 1.D0/3.D0*zeta3*omumv**(-3)
     &  + 1.D0/2.D0*zeta3*omumv**(-2)
     &  - 1.D0/2.D0*zeta3*omumv**(-1)
     &  - 13.D0/24.D0*zeta3
     &  - 187.D0/54.D0*v*omumv**(-2)
     &  + 187.D0/54.D0*v*omumv**(-1)
     &
      col5 = col5 + 1.D0/18.D0*v*pi**2*omumv**(-3)
     &  + 5.D0/72.D0*v*pi**2*omumv**(-2)
     &  - 1.D0/8.D0*v*pi**2*omumv**(-1)
     &  - 65.D0/72.D0*v*ci*pi*omumv**(-2)
     &  + 65.D0/72.D0*v*ci*pi*omumv**(-1)
     &  + v*zeta3*omumv**(-3)
     &  - v*zeta3*omumv**(-2)
     &  + 187.D0/54.D0*v**2*omumv**(-2)
     &  - 11.D0/24.D0*v**2*pi**2*omumv**(-4)
     &  + 31.D0/36.D0*v**2*pi**2*omumv**(-3)
     &  - 7.D0/12.D0*v**2*pi**2*omumv**(-2)
     &  + 65.D0/72.D0*v**2*ci*pi*omumv**(-2)
     &  - v**2*zeta3*omumv**(-3)
     &  + 11.D0/12.D0*v**3*pi**2*omumv**(-4)
     &  - 11.D0/12.D0*v**3*pi**2*omumv**(-3)
     &
      col5 = col5 - 11.D0/24.D0*v**4*pi**2*omumv**(-4)
     &  + 341.D0/216.D0*GPL1(0)
     &  - 11.D0/12.D0*GPL1(0)*v**(-1)*omumv**(-1)
     &  + 11.D0/12.D0*GPL1(0)*v**(-1)
     &  - 13.D0/12.D0*GPL1(0)*omu**(-2)
     &  - 1.D0/6.D0*GPL1(0)*omu**(-1)
     &  + 37.D0/24.D0*GPL1(0)*omumv**(-2)
     &  + 1.D0/36.D0*GPL1(0)*pi**2*omumv**(-3)
     &  - 1.D0/24.D0*GPL1(0)*pi**2*omumv**(-2)
     &  + 1.D0/24.D0*GPL1(0)*pi**2*omumv**(-1)
     &  - 1.D0/16.D0*GPL1(0)*pi**2
     &  + 55.D0/36.D0*GPL1(0)*ci*pi
     &  - 11.D0/4.D0*GPL1(0)*v*omumv**(-3)
     &  + 199.D0/72.D0*GPL1(0)*v*omumv**(-2)
     &  - 157.D0/72.D0*GPL1(0)*v*omumv**(-1)
     &
      col5 = col5 - 1.D0/12.D0*GPL1(0)*v*pi**2*omumv**(-3)
     &  + 1.D0/12.D0*GPL1(0)*v*pi**2*omumv**(-2)
     &  - 1.D0/24.D0*GPL1(0)*v*pi**2*omumv**(-1)
     &  + 11.D0/2.D0*GPL1(0)*v**2*omumv**(-3)
     &  - 155.D0/36.D0*GPL1(0)*v**2*omumv**(-2)
     &  + 1.D0/12.D0*GPL1(0)*v**2*pi**2*omumv**(-3)
     &  - 1.D0/24.D0*GPL1(0)*v**2*pi**2*omumv**(-2)
     &  - 11.D0/4.D0*GPL1(0)*v**3*omumv**(-3)
     &  - 1.D0/36.D0*GPL1(0)*v**3*pi**2*omumv**(-3)
     &  + 11.D0/12.D0*GPL1(0)*u*v**(-1)
     &  - 1.D0/36.D0*GPL1(0)*HPL1(0)
     &  + 5.D0/12.D0*GPL1(0)*HPL1(0)*ci*pi
     &  + 1.D0/3.D0*GPL1(0)*HPL1(0)*v*omumv**(-3)
     &  + 3.D0/4.D0*GPL1(0)*HPL1(0)*v*omumv**(-2)
     &  - 13.D0/12.D0*GPL1(0)*HPL1(0)*v*omumv**(-1)
     &
      col5 = col5 - 11.D0/4.D0*GPL1(0)*HPL1(0)*v**2*omumv**(-4)
     &  + 31.D0/6.D0*GPL1(0)*HPL1(0)*v**2*omumv**(-3)
     &  - 23.D0/6.D0*GPL1(0)*HPL1(0)*v**2*omumv**(-2)
     &  + 11.D0/2.D0*GPL1(0)*HPL1(0)*v**3*omumv**(-4)
     &  - 11.D0/2.D0*GPL1(0)*HPL1(0)*v**3*omumv**(-3)
     &  - 11.D0/4.D0*GPL1(0)*HPL1(0)*v**4*omumv**(-4)
     &  + 1.D0/36.D0*GPL1(0)*HPL1(1)
     &  - 5.D0/12.D0*GPL1(0)*HPL1(1)*ci*pi
     &  - 1.D0/12.D0*GPL1(0)*HPL1(1)*v*omumv**(-2)
     &  + 1.D0/12.D0*GPL1(0)*HPL1(1)*v*omumv**(-1)
     &  + 1.D0/12.D0*GPL1(0)*HPL1(1)*v**2*omumv**(-2)
     &  + 7.D0/6.D0*GPL1(0)*HPL1(1)*u*v**(-1)
     &  + 11.D0/12.D0*GPL1(0)*HPL1(1)*u**2*v**(-2)
     &  - 1.D0/3.D0*GPL1(0)*HPL2(0,0)
     &  - 1.D0/12.D0*GPL1(0)*HPL2(0,1)
     &
      col5 = col5 - 1.D0/4.D0*GPL1(0)*HPL2(1,0)
     &  + 1.D0/6.D0*GPL1(0)*HPL2(1,0)*omumv**(-3)
     &  - 1.D0/4.D0*GPL1(0)*HPL2(1,0)*omumv**(-2)
     &  + 1.D0/4.D0*GPL1(0)*HPL2(1,0)*omumv**(-1)
     &  - 1.D0/2.D0*GPL1(0)*HPL2(1,0)*v*omumv**(-3)
     &  + 1.D0/2.D0*GPL1(0)*HPL2(1,0)*v*omumv**(-2)
     &  - 1.D0/4.D0*GPL1(0)*HPL2(1,0)*v*omumv**(-1)
     &  + 1.D0/2.D0*GPL1(0)*HPL2(1,0)*v**2*omumv**(-3)
     &  - 1.D0/4.D0*GPL1(0)*HPL2(1,0)*v**2*omumv**(-2)
     &  - 1.D0/6.D0*GPL1(0)*HPL2(1,0)*v**3*omumv**(-3)
     &  - 1.D0/3.D0*GPL1(0)*HPL2(1,1)
     &  - 1.D0/12.D0*GPL1(1)*v*pi**2*omumv**(-1)
     &  - 1.D0/12.D0*GPL1(1)*v**2*pi**2*omumv**(-2)
     &  - 1.D0/18.D0*GPL1(1)*v**3*pi**2*omumv**(-3)
     &  + 35.D0/216.D0*GPL1(2)
     &
      col5 = col5 - 11.D0/12.D0*GPL1(2)*u**(-1)*v
     &  - 1.D0/144.D0*GPL1(2)*pi**2
     &  + 55.D0/36.D0*GPL1(2)*ci*pi
     &  + 13.D0/24.D0*GPL1(2)*v*omumv**(-2)
     &  - 13.D0/24.D0*GPL1(2)*v*omumv**(-1)
     &  - 13.D0/24.D0*GPL1(2)*v**2*omumv**(-2)
     &  - 11.D0/12.D0*GPL1(2)*u*v**(-1)
     &  - 1.D0/36.D0*GPL1(2)*HPL1(0)
     &  - 11.D0/12.D0*GPL1(2)*HPL1(0)*u**(-2)*v**2
     &  - 7.D0/6.D0*GPL1(2)*HPL1(0)*u**(-1)*v
     &  + 5.D0/12.D0*GPL1(2)*HPL1(0)*ci*pi
     &  + 1.D0/12.D0*GPL1(2)*HPL1(0)*v*omumv**(-2)
     &  - 1.D0/12.D0*GPL1(2)*HPL1(0)*v*omumv**(-1)
     &  - 1.D0/12.D0*GPL1(2)*HPL1(0)*v**2*omumv**(-2)
     &  + 11.D0/12.D0*GPL1(2)*HPL1(1)
     &
      col5 = col5 - 1.D0/3.D0*GPL1(2)*HPL2(0,0)
     &  - 1.D0/6.D0*GPL1(2)*HPL2(0,1)
     &  + 1.D0/6.D0*GPL1(2)*HPL2(1,0)
     &  + 5.D0/9.D0*GPL1(3)*HPL1(1)
     &  - 11.D0/12.D0*GPL1(3)*HPL1(1)*u**(-2)*v**2
     &  - 7.D0/6.D0*GPL1(3)*HPL1(1)*u**(-1)*v
     &  + 5.D0/6.D0*GPL1(3)*HPL1(1)*ci*pi
     &  + 1.D0/6.D0*GPL1(3)*HPL1(1)*v*omumv**(-2)
     &  - 1.D0/6.D0*GPL1(3)*HPL1(1)*v*omumv**(-1)
     &  - 1.D0/6.D0*GPL1(3)*HPL1(1)*v**2*omumv**(-2)
     &  - 7.D0/6.D0*GPL1(3)*HPL1(1)*u*v**(-1)
     &  - 11.D0/12.D0*GPL1(3)*HPL1(1)*u**2*v**(-2)
     &  - 1.D0/3.D0*GPL1(3)*HPL2(0,1)
     &  + 2.D0/3.D0*GPL1(3)*HPL2(1,1)
     &  - 11.D0/12.D0*GPL2(0,0)
     &
      col5 = col5 - 1.D0/3.D0*GPL2(0,0)*HPL1(0)
     &  + 1.D0/3.D0*GPL2(0,0)*HPL1(1)
     &  - 1.D0/36.D0*GPL2(0,2)
     &  + 5.D0/12.D0*GPL2(0,2)*ci*pi
     &  + 1.D0/12.D0*GPL2(0,2)*v*omumv**(-2)
     &  - 1.D0/12.D0*GPL2(0,2)*v*omumv**(-1)
     &  - 1.D0/12.D0*GPL2(0,2)*v**2*omumv**(-2)
     &  - 7.D0/6.D0*GPL2(0,2)*u*v**(-1)
     &  - 11.D0/12.D0*GPL2(0,2)*u**2*v**(-2)
     &  + 1.D0/12.D0*GPL2(0,2)*HPL1(0)
     &  + 1.D0/3.D0*GPL2(0,2)*HPL1(1)
     &  - 5.D0/9.D0*GPL2(1,0)
     &  + 1.D0/6.D0*GPL2(1,0)*omumv**(-2)
     &  - 1.D0/6.D0*GPL2(1,0)*omumv**(-1)
     &  - 5.D0/6.D0*GPL2(1,0)*ci*pi
     &
      col5 = col5 - 1.D0/3.D0*GPL2(1,0)*v*omumv**(-3)
     &  - 7.D0/6.D0*GPL2(1,0)*v*omumv**(-2)
     &  + 7.D0/6.D0*GPL2(1,0)*v*omumv**(-1)
     &  + 11.D0/4.D0*GPL2(1,0)*v**2*omumv**(-4)
     &  - 31.D0/6.D0*GPL2(1,0)*v**2*omumv**(-3)
     &  + 47.D0/12.D0*GPL2(1,0)*v**2*omumv**(-2)
     &  - 11.D0/2.D0*GPL2(1,0)*v**3*omumv**(-4)
     &  + 11.D0/2.D0*GPL2(1,0)*v**3*omumv**(-3)
     &  + 11.D0/4.D0*GPL2(1,0)*v**4*omumv**(-4)
     &  + 7.D0/6.D0*GPL2(1,0)*u*v**(-1)
     &  + 11.D0/12.D0*GPL2(1,0)*u**2*v**(-2)
     &  - 1.D0/4.D0*GPL2(1,0)*HPL1(0)*v*omumv**(-1)
     &  - 1.D0/4.D0*GPL2(1,0)*HPL1(0)*v**2*omumv**(-2)
     &  - 1.D0/6.D0*GPL2(1,0)*HPL1(0)*v**3*omumv**(-3)
     &  - 1.D0/6.D0*GPL2(1,0)*HPL1(1)
     &
      col5 = col5 - 1.D0/36.D0*GPL2(2,0)
     &  + 5.D0/12.D0*GPL2(2,0)*ci*pi
     &  + 1.D0/12.D0*GPL2(2,0)*v*omumv**(-2)
     &  - 1.D0/12.D0*GPL2(2,0)*v*omumv**(-1)
     &  - 1.D0/12.D0*GPL2(2,0)*v**2*omumv**(-2)
     &  - 7.D0/6.D0*GPL2(2,0)*u*v**(-1)
     &  - 11.D0/12.D0*GPL2(2,0)*u**2*v**(-2)
     &  + 1.D0/12.D0*GPL2(2,0)*HPL1(0)
     &  + 1.D0/3.D0*GPL2(2,0)*HPL1(1)
     &  - 11.D0/12.D0*GPL2(2,2)
     &  - 1.D0/3.D0*GPL2(2,2)*HPL1(0)
     &  - 1.D0/2.D0*GPL2(2,3)*HPL1(1)
     &  - 5.D0/9.D0*GPL2(3,2)
     &  + 11.D0/12.D0*GPL2(3,2)*u**(-2)*v**2
     &  + 7.D0/6.D0*GPL2(3,2)*u**(-1)*v
     &
      col5 = col5 - 5.D0/6.D0*GPL2(3,2)*ci*pi
     &  - 1.D0/6.D0*GPL2(3,2)*v*omumv**(-2)
     &  + 1.D0/6.D0*GPL2(3,2)*v*omumv**(-1)
     &  + 1.D0/6.D0*GPL2(3,2)*v**2*omumv**(-2)
     &  + 7.D0/6.D0*GPL2(3,2)*u*v**(-1)
     &  + 11.D0/12.D0*GPL2(3,2)*u**2*v**(-2)
     &  - 2.D0/3.D0*GPL2(3,2)*HPL1(1)
     &  - 1.D0/3.D0*GPL2(3,3)*HPL1(1)
     &  - 1.D0/3.D0*GPL3(0,0,2)
     &  + 2.D0/3.D0*GPL3(0,1,0)
     &  + 1.D0/6.D0*GPL3(0,1,0)*omumv**(-3)
     &  - 1.D0/4.D0*GPL3(0,1,0)*omumv**(-2)
     &  + 1.D0/4.D0*GPL3(0,1,0)*omumv**(-1)
     &  - 1.D0/2.D0*GPL3(0,1,0)*v*omumv**(-3)
     &  + 1.D0/2.D0*GPL3(0,1,0)*v*omumv**(-2)
     &
      col5 = col5 - 1.D0/4.D0*GPL3(0,1,0)*v*omumv**(-1)
     &  + 1.D0/2.D0*GPL3(0,1,0)*v**2*omumv**(-3)
     &  - 1.D0/4.D0*GPL3(0,1,0)*v**2*omumv**(-2)
     &  - 1.D0/6.D0*GPL3(0,1,0)*v**3*omumv**(-3)
     &  - 1.D0/3.D0*GPL3(0,2,0)
     &  - 1.D0/3.D0*GPL3(0,2,2)
     &  + 2.D0/3.D0*GPL3(1,0,0)
     &  + 1.D0/6.D0*GPL3(1,0,2)
     &  + 1.D0/2.D0*GPL3(1,1,0)*v*omumv**(-1)
     &  + 1.D0/2.D0*GPL3(1,1,0)*v**2*omumv**(-2)
     &  + 1.D0/3.D0*GPL3(1,1,0)*v**3*omumv**(-3)
     &  + 1.D0/6.D0*GPL3(1,2,0)
     &  - 1.D0/3.D0*GPL3(2,0,0)
     &  - 1.D0/3.D0*GPL3(2,0,2)
     &  + 1.D0/6.D0*GPL3(2,1,0)
     &
      col5 = col5 - 1.D0/3.D0*GPL3(2,2,0)
     &  + 1.D0/2.D0*GPL3(2,3,2)
     &  + 2.D0/3.D0*GPL3(3,2,2)
     &  + 1.D0/3.D0*GPL3(3,3,2)
     &  + 119.D0/54.D0*HPL1(0)
     &  - 11.D0/12.D0*HPL1(0)*u**(-1)*omv**(-1)
     &  + 11.D0/12.D0*HPL1(0)*u**(-1)
     &  + 11.D0/12.D0*HPL1(0)*u**(-1)*v
     &  - 13.D0/12.D0*HPL1(0)*omv**(-2)
     &  - 11.D0/12.D0*HPL1(0)*omv**(-1)*omumv**(-1)
     &  - 1.D0/6.D0*HPL1(0)*omv**(-1)
     &  + 11.D0/12.D0*HPL1(0)*omumv**(-1)
     &  - 5.D0/144.D0*HPL1(0)*pi**2
     &  + 55.D0/36.D0*HPL1(0)*ci*pi
     &  + 25.D0/72.D0*HPL1(0)*v*omumv**(-2)
     &
      col5 = col5 + 131.D0/72.D0*HPL1(0)*v*omumv**(-1)
     &  + 1.D0/24.D0*HPL1(0)*v*pi**2*omumv**(-1)
     &  - 11.D0/4.D0*HPL1(0)*v**2*omumv**(-3)
     &  + 71.D0/18.D0*HPL1(0)*v**2*omumv**(-2)
     &  + 1.D0/24.D0*HPL1(0)*v**2*pi**2*omumv**(-2)
     &  + 11.D0/4.D0*HPL1(0)*v**3*omumv**(-3)
     &  + 1.D0/36.D0*HPL1(0)*v**3*pi**2*omumv**(-3)
     &  - 35.D0/216.D0*HPL1(1)
     &  + 11.D0/12.D0*HPL1(1)*u**(-1)*v
     &  + 1.D0/18.D0*HPL1(1)*pi**2*omumv**(-3)
     &  - 1.D0/12.D0*HPL1(1)*pi**2*omumv**(-2)
     &  + 1.D0/12.D0*HPL1(1)*pi**2*omumv**(-1)
     &  - 7.D0/144.D0*HPL1(1)*pi**2
     &  - 55.D0/36.D0*HPL1(1)*ci*pi
     &  - 13.D0/24.D0*HPL1(1)*v*omumv**(-2)
     &
      col5 = col5 + 13.D0/24.D0*HPL1(1)*v*omumv**(-1)
     &  - 1.D0/6.D0*HPL1(1)*v*pi**2*omumv**(-3)
     &  + 1.D0/6.D0*HPL1(1)*v*pi**2*omumv**(-2)
     &  - 1.D0/12.D0*HPL1(1)*v*pi**2*omumv**(-1)
     &  + 13.D0/24.D0*HPL1(1)*v**2*omumv**(-2)
     &  + 1.D0/6.D0*HPL1(1)*v**2*pi**2*omumv**(-3)
     &  - 1.D0/12.D0*HPL1(1)*v**2*pi**2*omumv**(-2)
     &  - 1.D0/18.D0*HPL1(1)*v**3*pi**2*omumv**(-3)
     &  + 11.D0/12.D0*HPL1(1)*u*v**(-1)
     &  - 11.D0/12.D0*HPL2(0,0)
     &  + 7.D0/12.D0*HPL2(0,1)
     &  + 5.D0/12.D0*HPL2(0,1)*ci*pi
     &  + 1.D0/12.D0*HPL2(0,1)*v*omumv**(-2)
     &  - 1.D0/12.D0*HPL2(0,1)*v*omumv**(-1)
     &  - 1.D0/12.D0*HPL2(0,1)*v**2*omumv**(-2)
     &
      col5 = col5 - 7.D0/6.D0*HPL2(0,1)*u*v**(-1)
     &  - 11.D0/12.D0*HPL2(0,1)*u**2*v**(-2)
     &  + 7.D0/12.D0*HPL2(1,0)
     &  + 1.D0/6.D0*HPL2(1,0)*omumv**(-2)
     &  - 1.D0/6.D0*HPL2(1,0)*omumv**(-1)
     &  + 5.D0/12.D0*HPL2(1,0)*ci*pi
     &  + 1.D0/3.D0*HPL2(1,0)*v*omumv**(-3)
     &  + 5.D0/12.D0*HPL2(1,0)*v*omumv**(-2)
     &  - 13.D0/12.D0*HPL2(1,0)*v*omumv**(-1)
     &  - 11.D0/4.D0*HPL2(1,0)*v**2*omumv**(-4)
     &  + 31.D0/6.D0*HPL2(1,0)*v**2*omumv**(-3)
     &  - 23.D0/6.D0*HPL2(1,0)*v**2*omumv**(-2)
     &  + 11.D0/2.D0*HPL2(1,0)*v**3*omumv**(-4)
     &  - 11.D0/2.D0*HPL2(1,0)*v**3*omumv**(-3)
     &  - 11.D0/4.D0*HPL2(1,0)*v**4*omumv**(-4)
     &
      col5 = col5 - 11.D0/12.D0*HPL2(1,1)
     &  - 1.D0/2.D0*HPL3(0,1,0)
     &  - 1.D0/4.D0*HPL3(0,1,0)*v*omumv**(-1)
     &  - 1.D0/4.D0*HPL3(0,1,0)*v**2*omumv**(-2)
     &  - 1.D0/6.D0*HPL3(0,1,0)*v**3*omumv**(-3)
     &  + 1.D0/3.D0*HPL3(0,1,1)
     &  - 1.D0/3.D0*HPL3(1,0,0)
     &  + 1.D0/3.D0*HPL3(1,0,1)
     &  - 1.D0/3.D0*HPL3(1,1,0)
     &  + 1.D0/3.D0*HPL3(1,1,0)*omumv**(-3)
     &  - 1.D0/2.D0*HPL3(1,1,0)*omumv**(-2)
     &  + 1.D0/2.D0*HPL3(1,1,0)*omumv**(-1)
     &  - HPL3(1,1,0)*v*omumv**(-3)
     &  + HPL3(1,1,0)*v*omumv**(-2)
     &  - 1.D0/2.D0*HPL3(1,1,0)*v*omumv**(-1)
     &
      col5 = col5 + HPL3(1,1,0)*v**2*omumv**(-3)
     &  - 1.D0/2.D0*HPL3(1,1,0)*v**2*omumv**(-2)
     &  - 1.D0/3.D0*HPL3(1,1,0)*v**3*omumv**(-3)
     &

      col6 = - 29.D0/144.D0*pi**2
     &  + 5.D0/18.D0*ci*pi
     &  + 5.D0/54.D0*v*omumv**(-2)
     &  - 5.D0/54.D0*v*omumv**(-1)
     &  + 5.D0/36.D0*v*ci*pi*omumv**(-2)
     &  - 5.D0/36.D0*v*ci*pi*omumv**(-1)
     &  - 5.D0/54.D0*v**2*omumv**(-2)
     &  - 5.D0/36.D0*v**2*ci*pi*omumv**(-2)
     &  - 5.D0/54.D0*GPL1(0)
     &  - 5.D0/36.D0*GPL1(0)*ci*pi
     &  - 1.D0/36.D0*GPL1(0)*v*omumv**(-2)
     &  + 1.D0/36.D0*GPL1(0)*v*omumv**(-1)
     &  + 1.D0/36.D0*GPL1(0)*v**2*omumv**(-2)
     &  + 1.D0/36.D0*GPL1(0)*HPL1(0)
     &  - 1.D0/36.D0*GPL1(0)*HPL1(1)
     &
      col6 = col6 - 5.D0/54.D0*GPL1(2)
     &  - 5.D0/36.D0*GPL1(2)*ci*pi
     &  - 1.D0/12.D0*GPL1(2)*v*omumv**(-2)
     &  + 1.D0/12.D0*GPL1(2)*v*omumv**(-1)
     &  + 1.D0/12.D0*GPL1(2)*v**2*omumv**(-2)
     &  + 1.D0/36.D0*GPL1(2)*HPL1(0)
     &  - 1.D0/12.D0*GPL1(2)*HPL1(1)
     &  + 1.D0/12.D0*GPL2(0,0)
     &  + 1.D0/36.D0*GPL2(0,2)
     &  + 1.D0/36.D0*GPL2(2,0)
     &  + 1.D0/12.D0*GPL2(2,2)
     &  - 5.D0/54.D0*HPL1(0)
     &  - 5.D0/36.D0*HPL1(0)*ci*pi
     &  - 1.D0/36.D0*HPL1(0)*v*omumv**(-2)
     &  + 1.D0/36.D0*HPL1(0)*v*omumv**(-1)
     &
      col6 = col6 + 1.D0/36.D0*HPL1(0)*v**2*omumv**(-2)
     &  + 5.D0/54.D0*HPL1(1)
     &  + 5.D0/36.D0*HPL1(1)*ci*pi
     &  + 1.D0/12.D0*HPL1(1)*v*omumv**(-2)
     &  - 1.D0/12.D0*HPL1(1)*v*omumv**(-1)
     &  - 1.D0/12.D0*HPL1(1)*v**2*omumv**(-2)
     &  + 1.D0/12.D0*HPL2(0,0)
     &  - 1.D0/36.D0*HPL2(0,1)
     &  - 1.D0/36.D0*HPL2(1,0)
     &  + 1.D0/12.D0*HPL2(1,1)

      endif

      col(1) = col1
      col(2) = col2
      col(3) = col3
      col(4) = col4
      col(5) = col5
      col(6) = col6

      end subroutine beta1a2L
!#############################################################

      subroutine gamma1a2L(u,v,col)
      implicit none
      !Returns gamma(1a+) at one-loop
      !------------------------------
      real*8, intent(in) :: u,v
      complex*16, dimension(6), intent(out) :: col
      !internal
      complex*16 :: gam
      complex*16 :: ci
      real*8 :: omu,omv,omumv,upv
      real*8, dimension(0:1) :: HPL1
      real*8, dimension(0:1,0:1) :: HPL2
      real*8, dimension(0:1,0:1,0:1) :: HPL3
      real*8, dimension(0:1,0:1,0:1,0:1) :: HPL4
      real*8, dimension(0:3) :: GPL1
      real*8, dimension(0:3,0:3) :: GPL2
      real*8, dimension(0:3,0:3,0:3) :: GPL3
      real*8, dimension(0:3,0:3,0:3,0:3) :: GPL4
      complex*16 :: col1,col2,col3,col4,col5,col6


      omu   = 1.0d0 - u
      omv   = 1.0d0     - v
      omumv = 1.0d0 - u - v
      upv   =         u + v

      ci = (0.d0,1.d0)

      call tdhpl(dble(u),dble(v),4,GPL1,GPL2,GPL3,GPL4,HPL1,HPL2,HPL3,HPL4)

      if(ieorder==4) then
         col1 = - 1.0d0/2.0d0

         col2 = 1.0d0/2.0d0

         col3 = - 1.0d0/8.0d0

         col4 = 0.0d0

         col5 = 0.0d0

         col6 = 0.0d0
      elseif(ieorder==3) then
         col1 = - 73.0d0/24.0d0
     .          - 1.0d0*pi*ci
     .          + 1.0d0/2.0d0*HPL1(0)
     .          + 1.0d0/2.0d0*GPL1(0)

         col2 = + 109.0d0/48.0d0
     .          + 1.0d0*pi*ci
     .          - 1.0d0/4.0d0*HPL1(0)
     .          - 1.0d0/4.0d0*GPL1(0)
     .          - 1.0d0/2.0d0*GPL1(2)
     .          + 1.0d0/2.0d0*HPL1(1)

         col3 = - 3.0d0/8.0d0
     .          - 1.0d0/4.0d0*pi*ci
     .          + 1.0d0/4.0d0*GPL1(2)
     .          - 1.0d0/4.0d0*HPL1(1)


         col4 = - 5.0d0/24.0d0

         col5 = + 5.0d0/12.0d0

         col6 = 0
      else

         col1 =
     +  - 235525.d0/10368.d0
     +  - 9.d0/4.d0*pi*zeta3*ci
     +  + 5.d0/4.d0*pi*u**(-2)*v*ci
     +  - 5.d0/4.d0*pi*u**(-2)*ci
     +  + 19.d0/48.d0*pi*u**(-1)*v*ci
     +  + 7.d0/8.d0*pi*u**(-1)*ci
     +  + 19.d0/48.d0*pi*v*omu**(-1)*ci
     +
      col1 = col1 + 5.d0/8.d0*pi*omumv**(-1)*ci
     +  - 10181.d0/864.d0*pi*ci
     +  - 1.d0/8.d0*pi**2*u**(-1)*v
     +  - 1.d0/48.d0*pi**2*u**(-1)*omv**(-1)
     +  - 1.d0/16.d0*pi**2*u**(-1)
     +  - 1.d0/48.d0*pi**2*omv**(-1)*omumv**(-1)
     +  + 5.d0/48.d0*pi**2*omumv**(-2)
     +  - 1.d0/3.d0*pi**2*omumv**(-1)
     +
      col1 = col1 + 913.d0/864.d0*pi**2
     +  + 35.d0/144.d0*pi**3*ci
     +  - 11.d0/160.d0*pi**4
     +  + 67.d0/72.d0*zeta3
     +  - 269.d0/72.d0*u**(-1)
     +  + 4.d0/3.d0*omv**(-1)
     +  - 12917.d0/864.d0*HPL1(0)
     +  - 1.d0/4.d0*HPL1(0)*pi*u**(-1)*v*ci
     +
      col1 = col1 - 1.d0/8.d0*HPL1(0)*pi*u**(-1)*omv**(-1)*ci
     +  - 1.d0/8.d0*HPL1(0)*pi*omv**(-1)*omumv**(-1)*ci
     +  + 5.d0/8.d0*HPL1(0)*pi*omumv**(-2)*ci
     +  - 2*HPL1(0)*pi*omumv**(-1)*ci
     +  - 31.d0/4.d0*HPL1(0)*pi*ci
     +  + 13.d0/24.d0*HPL1(0)*pi**2
     +  + 1.d0/12.d0*HPL1(0)*pi**3*ci
     +  - 5.d0/2.d0*HPL1(0)*zeta3
     +
      col1 = col1 - 5.d0/4.d0*HPL1(0)*u**(-2)*v
     +  - 19.d0/48.d0*HPL1(0)*u**(-1)*v
     +  + 5.d0/8.d0*HPL1(0)*u**(-1)*omv**(-1)
     +  - 85.d0/48.d0*HPL1(0)*u**(-1)
     +  - 19.d0/48.d0*HPL1(0)*v*omu**(-1)
     +  + 4.d0/3.d0*HPL1(0)*omv**(-2)
     +  + 5.d0/8.d0*HPL1(0)*omv**(-1)*omumv**(-1)
     +  - 13.d0/3.d0*HPL1(0)*omv**(-1)
     +
      col1 = col1 - 5.d0/8.d0*HPL1(0)*omumv**(-1)
     +  + 149.d0/144.d0*HPL1(0)*GPL1(0)
     +  - 1.d0/8.d0*HPL1(0)*GPL1(0)*pi*ci
     +  + 1.d0/12.d0*HPL1(0)*GPL1(0)*pi**2
     +  - 1.d0/8.d0*HPL1(0)*GPL1(0)*u**(-1)*v
     +  - 1.d0/8.d0*HPL1(0)*GPL1(0)*u**(-1)*omv**(-1)
     +  - 1.d0/8.d0*HPL1(0)*GPL1(0)*omv**(-1)*omumv**(-1)
     +  + 5.d0/8.d0*HPL1(0)*GPL1(0)*omumv**(-2)
     +
      col1 = col1 - 2*HPL1(0)*GPL1(0)*omumv**(-1)
     +  + 401.d0/144.d0*HPL1(0)*GPL1(1)
     +  - 1.d0/8.d0*HPL1(0)*GPL1(1)*pi*u**(-2)*ci
     +  + 1.d0/2.d0*HPL1(0)*GPL1(1)*pi*u**(-1)*ci
     +  - 5.d0/6.d0*HPL1(0)*GPL1(1)*pi*ci
     +  - 5.d0/24.d0*HPL1(0)*GPL1(1)*pi**2
     +  - 5.d0/4.d0*HPL1(0)*GPL1(1)*u**(-3)*v
     +  + 5.d0/4.d0*HPL1(0)*GPL1(1)*u**(-3)
     +
      col1 = col1 + 1.d0/6.d0*HPL1(0)*GPL1(1)*u**(-2)*v
     +  - 25.d0/12.d0*HPL1(0)*GPL1(1)*u**(-2)
     +  + 25.d0/12.d0*HPL1(0)*GPL1(1)*u**(-1)
     +  + 149.d0/144.d0*HPL1(0)*GPL1(2)
     +  - 1.d0/4.d0*HPL1(0)*GPL1(2)*pi*u**(-2)*v*ci
     +  + 1.d0/8.d0*HPL1(0)*GPL1(2)*pi*u**(-2)*v**2*ci
     +  + 1.d0/8.d0*HPL1(0)*GPL1(2)*pi*u**(-2)*ci
     +  + 1.d0/2.d0*HPL1(0)*GPL1(2)*pi*u**(-1)*v*ci
     +
      col1 = col1 - 1.d0/2.d0*HPL1(0)*GPL1(2)*pi*u**(-1)*ci
     +  - 5.d0/2.d0*HPL1(0)*GPL1(2)*pi*ci
     +  + 19.d0/24.d0*HPL1(0)*GPL1(2)*pi**2
     +  + 5.d0/4.d0*HPL1(0)*GPL1(2)*u**(-3)*v
     +  - 5.d0/4.d0*HPL1(0)*GPL1(2)*u**(-3)
     +  - 1.d0/6.d0*HPL1(0)*GPL1(2)*u**(-2)*v
     +  + 25.d0/12.d0*HPL1(0)*GPL1(2)*u**(-2)
     +  - 1.d0/8.d0*HPL1(0)*GPL1(2)*u**(-1)*v
     +
      col1 = col1 - 25.d0/12.d0*HPL1(0)*GPL1(2)*u**(-1)
     +  + 401.d0/144.d0*HPL1(0)*GPL1(3)
     +  + 1.d0/12.d0*HPL1(0)*GPL1(3)*pi*ci
     +  - 1.d0/12.d0*HPL1(0)*GPL1(3)*pi**2
     +  + 1.d0/8.d0*HPL1(0)*GPL1(3)*u**(-1)*v
     +  + 1.d0/8.d0*HPL1(0)*GPL1(3)*u**(-1)*omv**(-1)
     +  + 1.d0/8.d0*HPL1(0)*GPL1(3)*omv**(-1)*omumv**(-1)
     +  - 5.d0/8.d0*HPL1(0)*GPL1(3)*omumv**(-2)
     +
      col1 = col1 + 2*HPL1(0)*GPL1(3)*omumv**(-1)
     +  + 7.d0/4.d0*HPL1(0)*GPL2(0,0)
     +  + 25.d0/24.d0*HPL1(0)*GPL2(0,1)
     +  + 1.d0/2.d0*HPL1(0)*GPL2(0,1)*pi*ci
     +  + 1.d0/8.d0*HPL1(0)*GPL2(0,1)*u**(-2)
     +  - 1.d0/2.d0*HPL1(0)*GPL2(0,1)*u**(-1)
     +  - 5.d0/8.d0*HPL1(0)*GPL2(0,2)
     +  - HPL1(0)*GPL2(0,2)*pi*ci
     +
      col1 = col1 - 1.d0/8.d0*HPL1(0)*GPL2(0,2)*u**(-2)
     +  + 1.d0/2.d0*HPL1(0)*GPL2(0,2)*u**(-1)
     +  - 4.d0/3.d0*HPL1(0)*GPL2(0,3)
     +  - 1.d0/2.d0*HPL1(0)*GPL2(0,3)*pi*ci
     +  - 5.d0/12.d0*HPL1(0)*GPL2(1,0)
     +  + 1.d0/4.d0*HPL1(0)*GPL2(1,0)*pi*ci
     +  - 13.d0/24.d0*HPL1(0)*GPL2(1,1)
     +  - 1.d0/8.d0*HPL1(0)*GPL2(1,1)*u**(-2)
     +
      col1 = col1 + 1.d0/2.d0*HPL1(0)*GPL2(1,1)*u**(-1)
     +  - 13.d0/6.d0*HPL1(0)*GPL2(1,2)
     +  - 5.d0/8.d0*HPL1(0)*GPL2(2,0)
     +  - HPL1(0)*GPL2(2,0)*pi*ci
     +  - 41.d0/24.d0*HPL1(0)*GPL2(2,1)
     +  - 1.d0/4.d0*HPL1(0)*GPL2(2,1)*u**(-2)*v
     +  + 1.d0/8.d0*HPL1(0)*GPL2(2,1)*u**(-2)*v**2
     +  + 1.d0/2.d0*HPL1(0)*GPL2(2,1)*u**(-1)*v
     +
      col1 = col1 + 17.d0/8.d0*HPL1(0)*GPL2(2,2)
     +  - 1.d0/4.d0*HPL1(0)*GPL2(2,2)*pi*ci
     +  + 1.d0/8.d0*HPL1(0)*GPL2(2,2)*u**(-2)
     +  - 1.d0/2.d0*HPL1(0)*GPL2(2,2)*u**(-1)
     +  + 25.d0/24.d0*HPL1(0)*GPL2(2,3)
     +  + 3.d0/4.d0*HPL1(0)*GPL2(2,3)*pi*ci
     +  - 43.d0/24.d0*HPL1(0)*GPL2(3,0)
     +  - 1.d0/4.d0*HPL1(0)*GPL2(3,0)*pi*ci
     +
      col1 = col1 - 5.d0/12.d0*HPL1(0)*GPL2(3,2)
     +  - 11.d0/12.d0*HPL1(0)*GPL2(3,3)
     +  + 1.d0/2.d0*HPL1(0)*GPL2(3,3)*pi*ci
     +  + 1.d0/4.d0*HPL1(0)*GPL3(0,0,1)
     +  - 1.d0/4.d0*HPL1(0)*GPL3(0,0,2)
     +  + 1.d0/2.d0*HPL1(0)*GPL3(0,1,0)
     +  - 1.d0/4.d0*HPL1(0)*GPL3(0,1,1)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(0,1,2)
     +
      col1 = col1 - HPL1(0)*GPL3(0,2,0)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(0,2,1)
     +  - 1.d0/4.d0*HPL1(0)*GPL3(0,2,2)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(0,2,3)
     +  - 1.d0/4.d0*HPL1(0)*GPL3(0,3,0)
     +  - 1.d0/4.d0*HPL1(0)*GPL3(0,3,2)
     +  + 1.d0/2.d0*HPL1(0)*GPL3(0,3,3)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(1,0,1)
     +
      col1 = col1 - 1.d0/2.d0*HPL1(0)*GPL3(1,1,1)
     +  + 1.d0/2.d0*HPL1(0)*GPL3(1,1,2)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(1,2,0)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(1,2,1)
     +  - 1.d0/2.d0*HPL1(0)*GPL3(1,2,2)
     +  - 1.d0/4.d0*HPL1(0)*GPL3(2,0,0)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(2,0,1)
     +  - HPL1(0)*GPL3(2,0,2)
     +
      col1 = col1 + 1.d0/4.d0*HPL1(0)*GPL3(2,0,3)
     +  - 1.d0/4.d0*HPL1(0)*GPL3(2,1,0)
     +  + 1.d0/2.d0*HPL1(0)*GPL3(2,1,1)
     +  - 1.d0/4.d0*HPL1(0)*GPL3(2,1,2)
     +  - 1.d0/4.d0*HPL1(0)*GPL3(2,2,0)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(2,2,3)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(2,3,0)
     +  + 1.d0/2.d0*HPL1(0)*GPL3(2,3,2)
     +
      col1 = col1 - 1.d0/4.d0*HPL1(0)*GPL3(2,3,3)
     +  - 1.d0/2.d0*HPL1(0)*GPL3(3,0,0)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(3,0,2)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(3,0,3)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(3,2,3)
     +  + 1.d0/2.d0*HPL1(0)*GPL3(3,3,0)
     +  - 1.d0/2.d0*HPL1(0)*GPL3(3,3,3)
     +  - 37.d0/12.d0*HPL1(1)
     +
      col1 = col1 - 1.d0/8.d0*HPL1(1)*pi*u**(-1)*v*ci
     +  - 1.d0/8.d0*HPL1(1)*pi*u**(-1)*ci
     +  - 103.d0/144.d0*HPL1(1)*pi*ci
     +  + 25.d0/288.d0*HPL1(1)*pi**2
     +  + 1.d0/6.d0*HPL1(1)*pi**3*ci
     +  - 5.d0/2.d0*HPL1(1)*zeta3
     +  - 5.d0/4.d0*HPL1(1)*u**(-2)*v
     +  + 5.d0/4.d0*HPL1(1)*u**(-2)
     +
      col1 = col1 - 19.d0/48.d0*HPL1(1)*u**(-1)*v
     +  - 65.d0/48.d0*HPL1(1)*u**(-1)
     +  - 19.d0/48.d0*HPL1(1)*v*omu**(-1)
     +  + 127.d0/144.d0*HPL1(1)*GPL1(0)
     +  + 3.d0/8.d0*HPL1(1)*GPL1(0)*pi*ci
     +  + 5.d0/24.d0*HPL1(1)*GPL1(0)*pi**2
     +  - 1.d0/8.d0*HPL1(1)*GPL1(0)*u**(-1)*v
     +  + 401.d0/144.d0*HPL1(1)*GPL1(1)
     +
      col1 = col1 - 1.d0/8.d0*HPL1(1)*GPL1(1)*pi*u**(-2)*ci
     +  + 1.d0/2.d0*HPL1(1)*GPL1(1)*pi*u**(-1)*ci
     +  - 1.d0/4.d0*HPL1(1)*GPL1(1)*pi*ci
     +  - 1.d0/8.d0*HPL1(1)*GPL1(1)*pi**2
     +  - 5.d0/4.d0*HPL1(1)*GPL1(1)*u**(-3)*v
     +  + 5.d0/4.d0*HPL1(1)*GPL1(1)*u**(-3)
     +  + 1.d0/6.d0*HPL1(1)*GPL1(1)*u**(-2)*v
     +  - 25.d0/12.d0*HPL1(1)*GPL1(1)*u**(-2)
     +
      col1 = col1 + 25.d0/12.d0*HPL1(1)*GPL1(1)*u**(-1)
     +  + 20.d0/9.d0*HPL1(1)*GPL1(2)
     +  + 3.d0/8.d0*HPL1(1)*GPL1(2)*pi*ci
     +  + 1.d0/12.d0*HPL1(1)*GPL1(2)*pi**2
     +  - 1.d0/8.d0*HPL1(1)*GPL1(2)*u**(-1)
     +  - 5.d0/12.d0*HPL1(1)*GPL1(3)*pi*ci
     +  + 1.d0/24.d0*HPL1(1)*GPL1(3)*pi**2
     +  - 1.d0/4.d0*HPL1(1)*GPL2(0,0)*pi*ci
     +
      col1 = col1 + 25.d0/24.d0*HPL1(1)*GPL2(0,1)
     +  + 3.d0/4.d0*HPL1(1)*GPL2(0,1)*pi*ci
     +  + 1.d0/8.d0*HPL1(1)*GPL2(0,1)*u**(-2)
     +  - 1.d0/2.d0*HPL1(1)*GPL2(0,1)*u**(-1)
     +  + 3.d0/8.d0*HPL1(1)*GPL2(0,2)
     +  - 1.d0/4.d0*HPL1(1)*GPL2(0,2)*pi*ci
     +  - 1.d0/4.d0*HPL1(1)*GPL2(0,3)*pi*ci
     +  - 5.d0/12.d0*HPL1(1)*GPL2(1,0)
     +
      col1 = col1 - 13.d0/24.d0*HPL1(1)*GPL2(1,1)
     +  + 1.d0/2.d0*HPL1(1)*GPL2(1,1)*pi*ci
     +  - 1.d0/8.d0*HPL1(1)*GPL2(1,1)*u**(-2)
     +  + 1.d0/2.d0*HPL1(1)*GPL2(1,1)*u**(-1)
     +  - 17.d0/8.d0*HPL1(1)*GPL2(1,2)
     +  - 1.d0/8.d0*HPL1(1)*GPL2(1,2)*u**(-2)
     +  + 1.d0/2.d0*HPL1(1)*GPL2(1,2)*u**(-1)
     +  + 3.d0/8.d0*HPL1(1)*GPL2(2,0)
     +
      col1 = col1 - 1.d0/4.d0*HPL1(1)*GPL2(2,0)*pi*ci
     +  - 41.d0/24.d0*HPL1(1)*GPL2(2,1)
     +  - 1.d0/4.d0*HPL1(1)*GPL2(2,1)*pi*ci
     +  - 1.d0/4.d0*HPL1(1)*GPL2(2,1)*u**(-2)*v
     +  + 1.d0/8.d0*HPL1(1)*GPL2(2,1)*u**(-2)*v**2
     +  + 1.d0/2.d0*HPL1(1)*GPL2(2,1)*u**(-1)*v
     +  + 1.d0/2.d0*HPL1(1)*GPL2(2,3)*pi*ci
     +  - 5.d0/12.d0*HPL1(1)*GPL2(3,0)
     +
      col1 = col1 + 1.d0/4.d0*HPL1(1)*GPL3(0,0,1)
     +  - 1.d0/4.d0*HPL1(1)*GPL3(0,0,2)
     +  + 1.d0/2.d0*HPL1(1)*GPL3(0,1,0)
     +  - 1.d0/4.d0*HPL1(1)*GPL3(0,1,1)
     +  + 1.d0/4.d0*HPL1(1)*GPL3(0,1,2)
     +  - 1.d0/4.d0*HPL1(1)*GPL3(0,2,0)
     +  + 1.d0/4.d0*HPL1(1)*GPL3(0,2,1)
     +  - 1.d0/4.d0*HPL1(1)*GPL3(0,3,0)
     +
      col1 = col1 + 1.d0/4.d0*HPL1(1)*GPL3(1,0,1)
     +  - 1.d0/2.d0*HPL1(1)*GPL3(1,1,1)
     +  + 1.d0/2.d0*HPL1(1)*GPL3(1,1,2)
     +  + 1.d0/4.d0*HPL1(1)*GPL3(1,2,1)
     +  - 1.d0/4.d0*HPL1(1)*GPL3(2,0,0)
     +  + 1.d0/4.d0*HPL1(1)*GPL3(2,0,1)
     +  - 1.d0/4.d0*HPL1(1)*GPL3(2,1,0)
     +  + 1.d0/2.d0*HPL1(1)*GPL3(2,1,1)
     +
      col1 = col1 + 1.d0/2.d0*HPL1(1)*GPL3(2,3,0)
     +  - 707.d0/72.d0*HPL2(0,0)
     +  - 53.d0/24.d0*HPL2(0,0)*pi*ci
     +  - 1.d0/12.d0*HPL2(0,0)*pi**2
     +  + 1.d0/8.d0*HPL2(0,0)*u**(-1)*v
     +  + 1.d0/8.d0*HPL2(0,0)*u**(-1)*omv**(-1)
     +  + 1.d0/8.d0*HPL2(0,0)*omv**(-1)*omumv**(-1)
     +  - 5.d0/8.d0*HPL2(0,0)*omumv**(-2)
     +
      col1 = col1 + 2*HPL2(0,0)*omumv**(-1)
     +  - 49.d0/12.d0*HPL2(0,0)*GPL1(0)
     +  - 1.d0/4.d0*HPL2(0,0)*GPL1(0)*pi*ci
     +  + 11.d0/4.d0*HPL2(0,0)*GPL1(1)
     +  - 41.d0/24.d0*HPL2(0,0)*GPL1(2)
     +  + 3.d0/4.d0*HPL2(0,0)*GPL1(2)*pi*ci
     +  + 11.d0/6.d0*HPL2(0,0)*GPL1(3)
     +  + 1.d0/4.d0*HPL2(0,0)*GPL1(3)*pi*ci
     +
      col1 = col1 - 1.d0/2.d0*HPL2(0,0)*GPL2(0,0)
     +  + 3.d0/4.d0*HPL2(0,0)*GPL2(0,2)
     +  + 1.d0/2.d0*HPL2(0,0)*GPL2(0,3)
     +  - 1.d0/2.d0*HPL2(0,0)*GPL2(1,0)
     +  + 1.d0/2.d0*HPL2(0,0)*GPL2(1,2)
     +  + 3.d0/4.d0*HPL2(0,0)*GPL2(2,0)
     +  - 1.d0/4.d0*HPL2(0,0)*GPL2(2,2)
     +  - 1.d0/4.d0*HPL2(0,0)*GPL2(2,3)
     +
      col1 = col1 + 3.d0/4.d0*HPL2(0,0)*GPL2(3,0)
     +  - 1.d0/4.d0*HPL2(0,0)*GPL2(3,2)
     +  - 1.d0/2.d0*HPL2(0,0)*GPL2(3,3)
     +  - 275.d0/72.d0*HPL2(0,1)
     +  - 5.d0/12.d0*HPL2(0,1)*pi*ci
     +  + 1.d0/24.d0*HPL2(0,1)*pi**2
     +  + 1.d0/8.d0*HPL2(0,1)*u**(-1)*v
     +  - 5.d0/12.d0*HPL2(0,1)*GPL1(0)
     +
      col1 = col1 + 65.d0/24.d0*HPL2(0,1)*GPL1(1)
     +  - 1.d0/4.d0*HPL2(0,1)*GPL1(1)*pi*ci
     +  + 1.d0/8.d0*HPL2(0,1)*GPL1(1)*u**(-2)
     +  - 1.d0/2.d0*HPL2(0,1)*GPL1(1)*u**(-1)
     +  - 5.d0/12.d0*HPL2(0,1)*GPL1(2)
     +  + 3.d0/4.d0*HPL2(0,1)*GPL1(2)*pi*ci
     +  + 1.d0/4.d0*HPL2(0,1)*GPL1(2)*u**(-2)*v
     +  - 1.d0/8.d0*HPL2(0,1)*GPL1(2)*u**(-2)*v**2
     +
      col1 = col1 - 1.d0/8.d0*HPL2(0,1)*GPL1(2)*u**(-2)
     +  - 1.d0/2.d0*HPL2(0,1)*GPL1(2)*u**(-1)*v
     +  + 1.d0/2.d0*HPL2(0,1)*GPL1(2)*u**(-1)
     +  + 5.d0/12.d0*HPL2(0,1)*GPL1(3)
     +  - 1.d0/4.d0*HPL2(0,1)*GPL1(3)*pi*ci
     +  + 1.d0/4.d0*HPL2(0,1)*GPL2(0,3)
     +  - 1.d0/4.d0*HPL2(0,1)*GPL2(1,0)
     +  + 1.d0/4.d0*HPL2(0,1)*GPL2(1,2)
     +
      col1 = col1 + 3.d0/4.d0*HPL2(0,1)*GPL2(2,0)
     +  - 1.d0/4.d0*HPL2(0,1)*GPL2(2,1)
     +  - 1.d0/2.d0*HPL2(0,1)*GPL2(2,3)
     +  - 1.d0/4.d0*HPL2(0,1)*GPL2(3,0)
     +  - 335.d0/48.d0*HPL2(1,0)
     +  - 5.d0/12.d0*HPL2(1,0)*pi*ci
     +  - 3.d0/8.d0*HPL2(1,0)*pi**2
     +  - 1.d0/8.d0*HPL2(1,0)*u**(-1)*omv**(-1)
     +
      col1 = col1 + 1.d0/8.d0*HPL2(1,0)*u**(-1)
     +  - 1.d0/8.d0*HPL2(1,0)*omv**(-1)*omumv**(-1)
     +  + 5.d0/8.d0*HPL2(1,0)*omumv**(-2)
     +  - 2*HPL2(1,0)*omumv**(-1)
     +  + 35.d0/24.d0*HPL2(1,0)*GPL1(0)
     +  + 1.d0/2.d0*HPL2(1,0)*GPL1(0)*pi*ci
     +  + 13.d0/6.d0*HPL2(1,0)*GPL1(1)
     +  - 31.d0/24.d0*HPL2(1,0)*GPL1(2)
     +
      col1 = col1 - 1.d0/4.d0*HPL2(1,0)*GPL1(2)*pi*ci
     +  - 1.d0/4.d0*HPL2(1,0)*GPL1(2)*u**(-2)*v
     +  + 1.d0/8.d0*HPL2(1,0)*GPL1(2)*u**(-2)*v**2
     +  + 1.d0/8.d0*HPL2(1,0)*GPL1(2)*u**(-2)
     +  + 1.d0/2.d0*HPL2(1,0)*GPL1(2)*u**(-1)*v
     +  - 1.d0/2.d0*HPL2(1,0)*GPL1(2)*u**(-1)
     +  + 5.d0/12.d0*HPL2(1,0)*GPL1(3)
     +  + 1.d0/4.d0*HPL2(1,0)*GPL2(0,0)
     +
      col1 = col1 - 1.d0/4.d0*HPL2(1,0)*GPL2(0,1)
     +  - 1.d0/4.d0*HPL2(1,0)*GPL2(0,2)
     +  + 1.d0/4.d0*HPL2(1,0)*GPL2(0,3)
     +  - 1.d0/4.d0*HPL2(1,0)*GPL2(1,0)
     +  - 1.d0/2.d0*HPL2(1,0)*GPL2(1,1)
     +  + 3.d0/4.d0*HPL2(1,0)*GPL2(1,2)
     +  + 1.d0/2.d0*HPL2(1,0)*GPL2(2,0)
     +  + 1.d0/4.d0*HPL2(1,0)*GPL2(2,1)
     +
      col1 = col1 - 1.d0/2.d0*HPL2(1,0)*GPL2(2,3)
     +  - 1.d0/4.d0*HPL2(1,0)*GPL2(3,2)
     +  - 20.d0/9.d0*HPL2(1,1)
     +  - 3.d0/8.d0*HPL2(1,1)*pi*ci
     +  + 1.d0/4.d0*HPL2(1,1)*pi**2
     +  + 1.d0/8.d0*HPL2(1,1)*u**(-1)
     +  - 3.d0/8.d0*HPL2(1,1)*GPL1(0)
     +  + 1.d0/4.d0*HPL2(1,1)*GPL1(0)*pi*ci
     +
      col1 = col1 + 17.d0/8.d0*HPL2(1,1)*GPL1(1)
     +  + 1.d0/8.d0*HPL2(1,1)*GPL1(1)*u**(-2)
     +  - 1.d0/2.d0*HPL2(1,1)*GPL1(1)*u**(-1)
     +  + 1.d0/4.d0*HPL2(1,1)*GPL2(0,0)
     +  - 1.d0/4.d0*HPL2(1,1)*GPL2(0,1)
     +  - 1.d0/2.d0*HPL2(1,1)*GPL2(1,1)
     +  + 33.d0/8.d0*HPL3(0,0,0)
     +  + 1.d0/4.d0*HPL3(0,0,0)*pi*ci
     +
      col1 = col1 + 3.d0/4.d0*HPL3(0,0,0)*GPL1(0)
     +  - 1.d0/2.d0*HPL3(0,0,0)*GPL1(1)
     +  + 1.d0/4.d0*HPL3(0,0,0)*GPL1(2)
     +  - 3.d0/4.d0*HPL3(0,0,0)*GPL1(3)
     +  + 5.d0/12.d0*HPL3(0,0,1)
     +  - 1.d0/4.d0*HPL3(0,0,1)*pi*ci
     +  - 1.d0/4.d0*HPL3(0,0,1)*GPL1(0)
     +  - 1.d0/2.d0*HPL3(0,0,1)*GPL1(1)
     +
      col1 = col1 + 1.d0/2.d0*HPL3(0,0,1)*GPL1(3)
     +  - 15.d0/8.d0*HPL3(0,1,0)
     +  - 1.d0/2.d0*HPL3(0,1,0)*GPL1(1)
     +  + 1.d0/4.d0*HPL3(0,1,0)*GPL1(3)
     +  - 91.d0/24.d0*HPL3(1,0,0)
     +  - 3.d0/4.d0*HPL3(1,0,0)*pi*ci
     +  - 3.d0/4.d0*HPL3(1,0,0)*GPL1(0)
     +  - 1.d0/2.d0*HPL3(1,0,0)*GPL1(1)
     +
      col1 = col1 + 1.d0/4.d0*HPL3(1,0,0)*GPL1(2)
     +  + 1.d0/4.d0*HPL3(1,0,0)*GPL1(3)
     +  - 5.d0/12.d0*HPL3(1,0,1)
     +  - 1.d0/4.d0*HPL3(1,0,1)*pi*ci
     +  - 1.d0/4.d0*HPL3(1,0,1)*GPL1(0)
     +  - 1.d0/4.d0*HPL3(1,0,1)*GPL1(1)
     +  + 3.d0/4.d0*HPL3(1,0,1)*GPL1(2)
     +  + 55.d0/24.d0*HPL3(1,1,0)
     +
      col1 = col1 + 3.d0/4.d0*HPL3(1,1,0)*pi*ci
     +  - 3.d0/4.d0*HPL3(1,1,0)*GPL1(1)
     +  + 3.d0/4.d0*HPL3(1,1,0)*GPL1(2)
     +  + 1.d0/4.d0*HPL3(1,1,0)*GPL1(3)
     +  - 3.d0/4.d0*HPL4(0,0,0,0)
     +  + 1.d0/2.d0*HPL4(0,0,0,1)
     +  + 1.d0/4.d0*HPL4(0,0,1,0)
     +  + 1.d0/4.d0*HPL4(0,1,0,0)
     +
      col1 = col1 + 1.d0/4.d0*HPL4(0,1,1,0)
     +  + 3.d0/4.d0*HPL4(1,0,0,0)
     +  - 1.d0/4.d0*HPL4(1,1,0,0)
     +  - 3.d0/4.d0*HPL4(1,1,0,1)
     +  + 5.d0/4.d0*HPL4(1,1,1,0)
     +  + 37.d0/12.d0*GPL1(0)
     +  + 1.d0/8.d0*GPL1(0)*pi*u**(-1)*v*ci
     +  + 103.d0/144.d0*GPL1(0)*pi*ci
     +
      col1 = col1 + 47.d0/96.d0*GPL1(0)*pi**2
     +  + 1.d0/12.d0*GPL1(0)*pi**3*ci
     +  + 1.d0/2.d0*GPL1(0)*zeta3
     +  + 2.d0/3.d0*GPL1(0)*u**(-1)
     +  + 5.d0/8.d0*GPL1(0)*omumv**(-1)
     +  + 5.d0/4.d0*GPL1(1)*pi*u**(-3)*v*ci
     +  - 5.d0/4.d0*GPL1(1)*pi*u**(-3)*ci
     +  - 1.d0/6.d0*GPL1(1)*pi*u**(-2)*v*ci
     +
      col1 = col1 + 25.d0/12.d0*GPL1(1)*pi*u**(-2)*ci
     +  - 25.d0/12.d0*GPL1(1)*pi*u**(-1)*ci
     +  - 401.d0/144.d0*GPL1(1)*pi*ci
     +  - 5.d0/48.d0*GPL1(1)*pi**2*u**(-2)
     +  + 5.d0/12.d0*GPL1(1)*pi**2*u**(-1)
     +  + 91.d0/144.d0*GPL1(1)*pi**2
     +  + 1.d0/12.d0*GPL1(1)*pi**3*ci
     +  + 1.d0/4.d0*GPL1(1)*zeta3
     +
      col1 = col1 + 37.d0/12.d0*GPL1(2)
     +  + 1.d0/8.d0*GPL1(2)*pi*u**(-1)*v*ci
     +  + 1.d0/8.d0*GPL1(2)*pi*u**(-1)*ci
     +  + 103.d0/144.d0*GPL1(2)*pi*ci
     +  - 1.d0/12.d0*GPL1(2)*pi**2*u**(-2)*v
     +  + 1.d0/24.d0*GPL1(2)*pi**2*u**(-2)*v**2
     +  + 1.d0/24.d0*GPL1(2)*pi**2*u**(-2)
     +  + 1.d0/6.d0*GPL1(2)*pi**2*u**(-1)*v
     +
      col1 = col1 - 1.d0/6.d0*GPL1(2)*pi**2*u**(-1)
     +  + 7.d0/32.d0*GPL1(2)*pi**2
     +  - 1.d0/6.d0*GPL1(2)*pi**3*ci
     +  + 1.d0/2.d0*GPL1(2)*zeta3
     +  + 5.d0/4.d0*GPL1(2)*u**(-2)*v
     +  - 5.d0/4.d0*GPL1(2)*u**(-2)
     +  + 19.d0/48.d0*GPL1(2)*u**(-1)*v
     +  + 65.d0/48.d0*GPL1(2)*u**(-1)
     +
      col1 = col1 + 19.d0/48.d0*GPL1(2)*v*omu**(-1)
     +  - 1.d0/8.d0*GPL1(3)*pi*u**(-1)*v*ci
     +  - 1.d0/8.d0*GPL1(3)*pi*u**(-1)*omv**(-1)*ci
     +  - 1.d0/8.d0*GPL1(3)*pi*omv**(-1)*omumv**(-1)*ci
     +  + 5.d0/8.d0*GPL1(3)*pi*omumv**(-2)*ci
     +  - 2*GPL1(3)*pi*omumv**(-1)*ci
     +  - 401.d0/144.d0*GPL1(3)*pi*ci
     +  + 61.d0/72.d0*GPL1(3)*pi**2
     +
      col1 = col1 + 1.d0/12.d0*GPL1(3)*pi**3*ci
     +  - 1.d0/2.d0*GPL1(3)*zeta3
     +  - 20.d0/9.d0*GPL2(0,0)
     +  - 1.d0/24.d0*GPL2(0,0)*pi**2
     +  - 1.d0/8.d0*GPL2(0,1)*pi*u**(-2)*ci
     +  + 1.d0/2.d0*GPL2(0,1)*pi*u**(-1)*ci
     +  - 25.d0/24.d0*GPL2(0,1)*pi*ci
     +  + 13.d0/24.d0*GPL2(0,1)*pi**2
     +
      col1 = col1 - 127.d0/144.d0*GPL2(0,2)
     +  - 3.d0/8.d0*GPL2(0,2)*pi*ci
     +  - 5.d0/24.d0*GPL2(0,2)*pi**2
     +  + 1.d0/8.d0*GPL2(0,2)*u**(-1)*v
     +  + 4.d0/3.d0*GPL2(0,3)*pi*ci
     +  - 1.d0/6.d0*GPL2(0,3)*pi**2
     +  + 5.d0/12.d0*GPL2(1,0)*pi*ci
     +  + 1.d0/8.d0*GPL2(1,1)*pi*u**(-2)*ci
     +
      col1 = col1 - 1.d0/2.d0*GPL2(1,1)*pi*u**(-1)*ci
     +  + 13.d0/24.d0*GPL2(1,1)*pi*ci
     +  + 1.d0/12.d0*GPL2(1,1)*pi**2
     +  - 401.d0/144.d0*GPL2(1,2)
     +  + 1.d0/8.d0*GPL2(1,2)*pi*u**(-2)*ci
     +  - 1.d0/2.d0*GPL2(1,2)*pi*u**(-1)*ci
     +  + 1.d0/4.d0*GPL2(1,2)*pi*ci
     +  + 1.d0/8.d0*GPL2(1,2)*pi**2
     +
      col1 = col1 + 5.d0/4.d0*GPL2(1,2)*u**(-3)*v
     +  - 5.d0/4.d0*GPL2(1,2)*u**(-3)
     +  - 1.d0/6.d0*GPL2(1,2)*u**(-2)*v
     +  + 25.d0/12.d0*GPL2(1,2)*u**(-2)
     +  - 25.d0/12.d0*GPL2(1,2)*u**(-1)
     +  - 127.d0/144.d0*GPL2(2,0)
     +  - 3.d0/8.d0*GPL2(2,0)*pi*ci
     +  - 5.d0/24.d0*GPL2(2,0)*pi**2
     +
      col1 = col1 + 1.d0/8.d0*GPL2(2,0)*u**(-1)*v
     +  + 1.d0/4.d0*GPL2(2,1)*pi*u**(-2)*v*ci
     +  - 1.d0/8.d0*GPL2(2,1)*pi*u**(-2)*v**2*ci
     +  - 1.d0/2.d0*GPL2(2,1)*pi*u**(-1)*v*ci
     +  + 41.d0/24.d0*GPL2(2,1)*pi*ci
     +  - 1.d0/12.d0*GPL2(2,1)*pi**2
     +  - 20.d0/9.d0*GPL2(2,2)
     +  - 3.d0/8.d0*GPL2(2,2)*pi*ci
     +
      col1 = col1 - 1.d0/12.d0*GPL2(2,2)*pi**2
     +  + 1.d0/8.d0*GPL2(2,2)*u**(-1)
     +  - 25.d0/24.d0*GPL2(2,3)*pi*ci
     +  + 7.d0/12.d0*GPL2(2,3)*pi**2
     +  - 401.d0/144.d0*GPL2(3,0)
     +  - 1.d0/8.d0*GPL2(3,0)*pi*ci
     +  + 1.d0/12.d0*GPL2(3,0)*pi**2
     +  - 1.d0/8.d0*GPL2(3,0)*u**(-1)*v
     +
      col1 = col1 - 1.d0/8.d0*GPL2(3,0)*u**(-1)*omv**(-1)
     +  - 1.d0/8.d0*GPL2(3,0)*omv**(-1)*omumv**(-1)
     +  + 5.d0/8.d0*GPL2(3,0)*omumv**(-2)
     +  - 2*GPL2(3,0)*omumv**(-1)
     +  + 5.d0/12.d0*GPL2(3,2)*pi*ci
     +  - 1.d0/24.d0*GPL2(3,2)*pi**2
     +  + 11.d0/12.d0*GPL2(3,3)*pi*ci
     +  + 1.d0/6.d0*GPL2(3,3)*pi**2
     +
      col1 = col1 - 1.d0/4.d0*GPL3(0,0,1)*pi*ci
     +  + 1.d0/4.d0*GPL3(0,0,2)*pi*ci
     +  - 1.d0/2.d0*GPL3(0,1,0)*pi*ci
     +  + 1.d0/4.d0*GPL3(0,1,1)*pi*ci
     +  - 25.d0/24.d0*GPL3(0,1,2)
     +  - 3.d0/4.d0*GPL3(0,1,2)*pi*ci
     +  - 1.d0/8.d0*GPL3(0,1,2)*u**(-2)
     +  + 1.d0/2.d0*GPL3(0,1,2)*u**(-1)
     +
      col1 = col1 + 1.d0/4.d0*GPL3(0,2,0)*pi*ci
     +  - 1.d0/4.d0*GPL3(0,2,1)*pi*ci
     +  - 3.d0/8.d0*GPL3(0,2,2)
     +  + 1.d0/4.d0*GPL3(0,2,2)*pi*ci
     +  - 1.d0/4.d0*GPL3(0,2,3)*pi*ci
     +  + 4.d0/3.d0*GPL3(0,3,0)
     +  + 1.d0/4.d0*GPL3(0,3,0)*pi*ci
     +  + 1.d0/4.d0*GPL3(0,3,2)*pi*ci
     +
      col1 = col1 - 1.d0/2.d0*GPL3(0,3,3)*pi*ci
     +  - 1.d0/4.d0*GPL3(1,0,1)*pi*ci
     +  + 5.d0/12.d0*GPL3(1,0,2)
     +  + 1.d0/2.d0*GPL3(1,1,1)*pi*ci
     +  + 13.d0/24.d0*GPL3(1,1,2)
     +  - 1.d0/2.d0*GPL3(1,1,2)*pi*ci
     +  + 1.d0/8.d0*GPL3(1,1,2)*u**(-2)
     +  - 1.d0/2.d0*GPL3(1,1,2)*u**(-1)
     +
      col1 = col1 + 5.d0/12.d0*GPL3(1,2,0)
     +  - 1.d0/4.d0*GPL3(1,2,1)*pi*ci
     +  + 17.d0/8.d0*GPL3(1,2,2)
     +  + 1.d0/8.d0*GPL3(1,2,2)*u**(-2)
     +  - 1.d0/2.d0*GPL3(1,2,2)*u**(-1)
     +  + 1.d0/4.d0*GPL3(2,0,0)*pi*ci
     +  - 1.d0/4.d0*GPL3(2,0,1)*pi*ci
     +  - 3.d0/8.d0*GPL3(2,0,2)
     +
      col1 = col1 + 1.d0/4.d0*GPL3(2,0,2)*pi*ci
     +  - 1.d0/4.d0*GPL3(2,0,3)*pi*ci
     +  + 1.d0/4.d0*GPL3(2,1,0)*pi*ci
     +  - 1.d0/2.d0*GPL3(2,1,1)*pi*ci
     +  + 41.d0/24.d0*GPL3(2,1,2)
     +  + 1.d0/4.d0*GPL3(2,1,2)*pi*ci
     +  + 1.d0/4.d0*GPL3(2,1,2)*u**(-2)*v
     +  - 1.d0/8.d0*GPL3(2,1,2)*u**(-2)*v**2
     +
      col1 = col1 - 1.d0/2.d0*GPL3(2,1,2)*u**(-1)*v
     +  - 3.d0/8.d0*GPL3(2,2,0)
     +  + 1.d0/4.d0*GPL3(2,2,0)*pi*ci
     +  - 1.d0/4.d0*GPL3(2,2,3)*pi*ci
     +  - 25.d0/24.d0*GPL3(2,3,0)
     +  - 3.d0/4.d0*GPL3(2,3,0)*pi*ci
     +  - 1.d0/2.d0*GPL3(2,3,2)*pi*ci
     +  + 1.d0/4.d0*GPL3(2,3,3)*pi*ci
     +
      col1 = col1 + 7.d0/4.d0*GPL3(3,0,0)
     +  + 5.d0/12.d0*GPL3(3,0,2)
     +  - 1.d0/4.d0*GPL3(3,0,3)*pi*ci
     +  + 5.d0/12.d0*GPL3(3,2,0)
     +  - 1.d0/4.d0*GPL3(3,2,3)*pi*ci
     +  + 11.d0/12.d0*GPL3(3,3,0)
     +  - 1.d0/2.d0*GPL3(3,3,0)*pi*ci
     +  + 1.d0/2.d0*GPL3(3,3,3)*pi*ci
     +
      col1 = col1 - 1.d0/4.d0*GPL4(0,0,1,2)
     +  + 1.d0/4.d0*GPL4(0,0,2,2)
     +  - 1.d0/2.d0*GPL4(0,1,0,2)
     +  + 1.d0/4.d0*GPL4(0,1,1,2)
     +  - 1.d0/2.d0*GPL4(0,1,2,0)
     +  - 1.d0/4.d0*GPL4(0,1,2,2)
     +  + 1.d0/4.d0*GPL4(0,2,0,2)
     +  - 1.d0/4.d0*GPL4(0,2,1,2)
     +
      col1 = col1 + 1.d0/4.d0*GPL4(0,2,2,0)
     +  - 1.d0/4.d0*GPL4(0,2,3,0)
     +  + 1.d0/4.d0*GPL4(0,3,0,2)
     +  + 1.d0/4.d0*GPL4(0,3,2,0)
     +  - 1.d0/2.d0*GPL4(0,3,3,0)
     +  - 1.d0/4.d0*GPL4(1,0,1,2)
     +  + 1.d0/2.d0*GPL4(1,1,1,2)
     +  - 1.d0/2.d0*GPL4(1,1,2,2)
     +
      col1 = col1 - 1.d0/4.d0*GPL4(1,2,1,2)
     +  + 1.d0/4.d0*GPL4(2,0,0,2)
     +  - 1.d0/4.d0*GPL4(2,0,1,2)
     +  + 1.d0/4.d0*GPL4(2,0,2,0)
     +  - 1.d0/4.d0*GPL4(2,0,3,0)
     +  + 1.d0/4.d0*GPL4(2,1,0,2)
     +  - 1.d0/2.d0*GPL4(2,1,1,2)
     +  + 1.d0/4.d0*GPL4(2,1,2,0)
     +
      col1 = col1 + 1.d0/4.d0*GPL4(2,2,0,0)
     +  - 1.d0/4.d0*GPL4(2,2,3,0)
     +  - 1.d0/4.d0*GPL4(2,3,0,0)
     +  - 1.d0/2.d0*GPL4(2,3,0,2)
     +  - 1.d0/2.d0*GPL4(2,3,2,0)
     +  + 1.d0/4.d0*GPL4(2,3,3,0)
     +  - 1.d0/4.d0*GPL4(3,0,3,0)
     +  - 1.d0/4.d0*GPL4(3,2,3,0)
     +
      col1 = col1 - 1.d0/2.d0*GPL4(3,3,0,0)
     +  + 1.d0/2.d0*GPL4(3,3,3,0)
     +

      col2 =
     +  - 38435.d0/2592.d0
     +  + 1.d0/2.d0*pi*zeta3*ci
     +  + 1.d0/8.d0*pi*u**(-2)*v*ci
     +  - 1.d0/8.d0*pi*u**(-2)*ci
     +  + 7.d0/24.d0*pi*u**(-1)*v*ci
     +  - 1.d0/4.d0*pi*u**(-1)*ci
     +  + 2.d0/3.d0*pi*v*omu**(-1)*ci
     +
      col2 = col2 + 1.d0/16.d0*pi*omumv**(-1)*ci
     +  - 589.d0/108.d0*pi*ci
     +  - 1.d0/24.d0*pi**2*u**(-1)*v
     +  - 1.d0/48.d0*pi**2*u**(-1)
     +  - 1.d0/4.d0*pi**2*u
     +  - 1.d0/4.d0*pi**2*v
     +  - 1.d0/16.d0*pi**2*upv**(-1)
     +  + 1.d0/96.d0*pi**2*omumv**(-2)
     +
      col2 = col2 + 1.d0/24.d0*pi**2*omumv**(-1)
     +  + 2129.d0/864.d0*pi**2
     +  + 163.d0/288.d0*pi**3*ci
     +  + 271.d0/2880.d0*pi**4
     +  + 683.d0/144.d0*zeta3
     +  - 139.d0/36.d0*u**(-1)
     +  - 949.d0/108.d0*HPL1(0)
     +  - 1.d0/8.d0*HPL1(0)*pi*u**(-1)*v*ci
     +
      col2 = col2 + 1.d0/8.d0*HPL1(0)*pi*u**(-1)*ci
     +  - 3.d0/8.d0*HPL1(0)*pi*u*ci
     +  - 3.d0/8.d0*HPL1(0)*pi*v*ci
     +  - 1.d0/8.d0*HPL1(0)*pi*upv**(-1)*ci
     +  + 1.d0/16.d0*HPL1(0)*pi*omumv**(-2)*ci
     +  + 1.d0/4.d0*HPL1(0)*pi*omumv**(-1)*ci
     +  + 383.d0/144.d0*HPL1(0)*pi*ci
     +  - 17.d0/288.d0*HPL1(0)*pi**2
     +
      col2 = col2 + 3.d0/8.d0*HPL1(0)*pi**3*ci
     +  + 1.d0/4.d0*HPL1(0)*zeta3
     +  - 1.d0/8.d0*HPL1(0)*u**(-2)*v
     +  - 7.d0/24.d0*HPL1(0)*u**(-1)*v
     +  + 1.d0/16.d0*HPL1(0)*u**(-1)*omv**(-1)
     +  - 29.d0/24.d0*HPL1(0)*u**(-1)
     +  - 2.d0/3.d0*HPL1(0)*v*omu**(-1)
     +  + 1.d0/16.d0*HPL1(0)*omv**(-1)*omumv**(-1)
     +
      col2 = col2 - 1.d0/16.d0*HPL1(0)*omumv**(-1)
     +  + 383.d0/144.d0*HPL1(0)*GPL1(0)
     +  - 17.d0/12.d0*HPL1(0)*GPL1(0)*pi*ci
     +  + 7.d0/8.d0*HPL1(0)*GPL1(0)*pi**2
     +  - 3.d0/8.d0*HPL1(0)*GPL1(0)*u
     +  - 3.d0/8.d0*HPL1(0)*GPL1(0)*v
     +  + 1.d0/16.d0*HPL1(0)*GPL1(0)*omumv**(-2)
     +  + 1.d0/4.d0*HPL1(0)*GPL1(0)*omumv**(-1)
     +
      col2 = col2 - 383.d0/144.d0*HPL1(0)*GPL1(1)
     +  + 1.d0/12.d0*HPL1(0)*GPL1(1)*pi*ci
     +  - 3.d0/4.d0*HPL1(0)*GPL1(1)*pi**2
     +  - 1.d0/8.d0*HPL1(0)*GPL1(1)*u**(-3)*v
     +  + 1.d0/8.d0*HPL1(0)*GPL1(1)*u**(-3)
     +  - 1.d0/2.d0*HPL1(0)*GPL1(1)*u**(-2)*v
     +  - 1.d0/16.d0*HPL1(0)*GPL1(1)*u**(-2)
     +  + 1.d0/8.d0*HPL1(0)*GPL1(1)*u**(-1)*v**(-1)
     +
      col2 = col2 - 1.d0/8.d0*HPL1(0)*GPL1(1)*u**(-1)*v
     +  + 3.d0/8.d0*HPL1(0)*GPL1(1)*u
     +  - 1.d0/8.d0*HPL1(0)*GPL1(1)*v**(-1)*upv**(-1)
     +  + 3.d0/8.d0*HPL1(0)*GPL1(1)*v
     +  - 1.d0/8.d0*HPL1(0)*GPL1(1)*upv**(-1)
     +  + 383.d0/144.d0*HPL1(0)*GPL1(2)
     +  - 1.d0/4.d0*HPL1(0)*GPL1(2)*pi*u**(-2)*v*ci
     +  + 1.d0/8.d0*HPL1(0)*GPL1(2)*pi*u**(-2)*v**2*ci
     +
      col2 = col2 + 1.d0/8.d0*HPL1(0)*GPL1(2)*pi*u**(-2)*ci
     +  - 1.d0/4.d0*HPL1(0)*GPL1(2)*pi*u**(-1)*v*ci
     +  + 1.d0/4.d0*HPL1(0)*GPL1(2)*pi*u**(-1)*ci
     +  - 1.d0/8.d0*HPL1(0)*GPL1(2)*pi*upv**(-2)*ci
     +  - 1.d0/4.d0*HPL1(0)*GPL1(2)*pi*upv**(-1)*ci
     +  - 1.d0/2.d0*HPL1(0)*GPL1(2)*pi*ci
     +  + HPL1(0)*GPL1(2)*pi**2
     +  + 1.d0/8.d0*HPL1(0)*GPL1(2)*u**(-3)*v
     +
      col2 = col2 - 1.d0/8.d0*HPL1(0)*GPL1(2)*u**(-3)
     +  + 1.d0/2.d0*HPL1(0)*GPL1(2)*u**(-2)*v
     +  + 1.d0/16.d0*HPL1(0)*GPL1(2)*u**(-2)
     +  - 1.d0/8.d0*HPL1(0)*GPL1(2)*u**(-1)*v**(-1)
     +  + 1.d0/8.d0*HPL1(0)*GPL1(2)*u**(-1)
     +  - 3.d0/8.d0*HPL1(0)*GPL1(2)*u
     +  + 1.d0/8.d0*HPL1(0)*GPL1(2)*v**(-1)*upv**(-1)
     +  - 3.d0/8.d0*HPL1(0)*GPL1(2)*v
     +
      col2 = col2 - 383.d0/144.d0*HPL1(0)*GPL1(3)
     +  - 5.d0/6.d0*HPL1(0)*GPL1(3)*pi*ci
     +  - 9.d0/8.d0*HPL1(0)*GPL1(3)*pi**2
     +  + 3.d0/8.d0*HPL1(0)*GPL1(3)*u
     +  + 3.d0/8.d0*HPL1(0)*GPL1(3)*v
     +  - 1.d0/16.d0*HPL1(0)*GPL1(3)*omumv**(-2)
     +  - 1.d0/4.d0*HPL1(0)*GPL1(3)*omumv**(-1)
     +  - 7.d0/4.d0*HPL1(0)*GPL2(0,0)
     +
      col2 = col2 - HPL1(0)*GPL2(0,0)*pi*ci
     +  + 7.d0/6.d0*HPL1(0)*GPL2(0,1)
     +  + 1.d0/4.d0*HPL1(0)*GPL2(0,1)*pi*ci
     +  - 3.d0/4.d0*HPL1(0)*GPL2(0,1)*u**(-1)
     +  - 1.d0/8.d0*HPL1(0)*GPL2(0,1)*upv**(-2)
     +  - 1.d0/4.d0*HPL1(0)*GPL2(0,1)*upv**(-1)
     +  - 25.d0/8.d0*HPL1(0)*GPL2(0,2)
     +  - 3.d0/2.d0*HPL1(0)*GPL2(0,2)*pi*ci
     +
      col2 = col2 + 3.d0/4.d0*HPL1(0)*GPL2(0,2)*u**(-1)
     +  + 1.d0/8.d0*HPL1(0)*GPL2(0,2)*upv**(-2)
     +  + 1.d0/4.d0*HPL1(0)*GPL2(0,2)*upv**(-1)
     +  + 4.d0/3.d0*HPL1(0)*GPL2(0,3)
     +  + 3.d0/4.d0*HPL1(0)*GPL2(0,3)*pi*ci
     +  + 5.d0/12.d0*HPL1(0)*GPL2(1,0)
     +  + 3.d0/4.d0*HPL1(0)*GPL2(1,0)*pi*ci
     +  + 1.d0/6.d0*HPL1(0)*GPL2(1,1)
     +
      col2 = col2 - 3.d0/4.d0*HPL1(0)*GPL2(1,1)*u**(-1)
     +  + 43.d0/24.d0*HPL1(0)*GPL2(1,2)
     +  + 1.d0/2.d0*HPL1(0)*GPL2(1,2)*pi*ci
     +  + 3.d0/4.d0*HPL1(0)*GPL2(1,2)*u**(-1)
     +  - 25.d0/8.d0*HPL1(0)*GPL2(2,0)
     +  - 3.d0/2.d0*HPL1(0)*GPL2(2,0)*pi*ci
     +  + 4.d0/3.d0*HPL1(0)*GPL2(2,1)
     +  - 1.d0/4.d0*HPL1(0)*GPL2(2,1)*pi*ci
     +
      col2 = col2 - 1.d0/4.d0*HPL1(0)*GPL2(2,1)*u**(-2)*v
     +  + 1.d0/8.d0*HPL1(0)*GPL2(2,1)*u**(-2)*v**2
     +  + 1.d0/8.d0*HPL1(0)*GPL2(2,1)*u**(-2)
     +  - 1.d0/4.d0*HPL1(0)*GPL2(2,1)*u**(-1)*v
     +  + HPL1(0)*GPL2(2,1)*u**(-1)
     +  - 1.d0/8.d0*HPL1(0)*GPL2(2,1)*upv**(-2)
     +  - 1.d0/4.d0*HPL1(0)*GPL2(2,1)*upv**(-1)
     +  - HPL1(0)*GPL2(2,2)
     +
      col2 = col2 - 3.d0/4.d0*HPL1(0)*GPL2(2,2)*u**(-1)
     +  + 5.d0/12.d0*HPL1(0)*GPL2(2,3)
     +  + 1.d0/4.d0*HPL1(0)*GPL2(2,3)*pi*ci
     +  + 43.d0/24.d0*HPL1(0)*GPL2(3,0)
     +  + 7.d0/4.d0*HPL1(0)*GPL2(3,0)*pi*ci
     +  - 1.d0/3.d0*HPL1(0)*GPL2(3,2)
     +  + HPL1(0)*GPL2(3,2)*pi*ci
     +  + 11.d0/12.d0*HPL1(0)*GPL2(3,3)
     +
      col2 = col2 - HPL1(0)*GPL2(3,3)*pi*ci
     +  - 1.d0/2.d0*HPL1(0)*GPL3(0,0,1)
     +  - 1.d0/2.d0*HPL1(0)*GPL3(0,0,2)
     +  + 1.d0/2.d0*HPL1(0)*GPL3(0,0,3)
     +  - 1.d0/4.d0*HPL1(0)*GPL3(0,1,0)
     +  + 1.d0/2.d0*HPL1(0)*GPL3(0,1,2)
     +  - 3.d0/4.d0*HPL1(0)*GPL3(0,2,0)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(0,2,1)
     +
      col2 = col2 - HPL1(0)*GPL3(0,2,2)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(0,2,3)
     +  + 1.d0/2.d0*HPL1(0)*GPL3(0,3,0)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(0,3,2)
     +  - HPL1(0)*GPL3(0,3,3)
     +  + 3.d0/4.d0*HPL1(0)*GPL3(1,0,2)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(1,0,3)
     +  + HPL1(0)*GPL3(1,1,1)
     +
      col2 = col2 - HPL1(0)*GPL3(1,1,2)
     +  + 1.d0/2.d0*HPL1(0)*GPL3(1,2,0)
     +  - HPL1(0)*GPL3(1,2,1)
     +  + HPL1(0)*GPL3(1,2,2)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(1,2,3)
     +  - HPL1(0)*GPL3(2,0,0)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(2,0,1)
     +  - 3.d0/4.d0*HPL1(0)*GPL3(2,0,2)
     +
      col2 = col2 + 1.d0/4.d0*HPL1(0)*GPL3(2,0,3)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(2,1,0)
     +  - HPL1(0)*GPL3(2,1,1)
     +  + 1.d0/2.d0*HPL1(0)*GPL3(2,1,2)
     +  - 1.d0/2.d0*HPL1(0)*GPL3(2,2,0)
     +  + 1.d0/2.d0*HPL1(0)*GPL3(2,2,1)
     +  - 1.d0/2.d0*HPL1(0)*GPL3(2,2,3)
     +  + 1.d0/2.d0*HPL1(0)*GPL3(2,3,0)
     +
      col2 = col2 - 1.d0/4.d0*HPL1(0)*GPL3(2,3,2)
     +  + HPL1(0)*GPL3(3,0,0)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(3,0,1)
     +  + 1.d0/2.d0*HPL1(0)*GPL3(3,0,2)
     +  - HPL1(0)*GPL3(3,0,3)
     +  + 3.d0/4.d0*HPL1(0)*GPL3(3,2,0)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(3,2,1)
     +  - HPL1(0)*GPL3(3,3,0)
     +
      col2 = col2 + HPL1(0)*GPL3(3,3,3)
     +  - 31.d0/24.d0*HPL1(1)
     +  - 1.d0/8.d0*HPL1(1)*pi*u**(-1)*ci
     +  - 3.d0/8.d0*HPL1(1)*pi*u*ci
     +  - 3.d0/8.d0*HPL1(1)*pi*v*ci
     +  + 527.d0/144.d0*HPL1(1)*pi*ci
     +  + 1.d0/16.d0*HPL1(1)*pi**2*upv**(-2)
     +  + 1.d0/8.d0*HPL1(1)*pi**2*upv**(-1)
     +
      col2 = col2 - 17.d0/24.d0*HPL1(1)*pi**2
     +  + 1.d0/2.d0*HPL1(1)*pi**3*ci
     +  - 5.d0/4.d0*HPL1(1)*zeta3
     +  - 1.d0/8.d0*HPL1(1)*u**(-2)*v
     +  + 1.d0/8.d0*HPL1(1)*u**(-2)
     +  - 7.d0/24.d0*HPL1(1)*u**(-1)*v
     +  - 11.d0/48.d0*HPL1(1)*u**(-1)
     +  - 2.d0/3.d0*HPL1(1)*v*omu**(-1)
     +
      col2 = col2 + 527.d0/144.d0*HPL1(1)*GPL1(0)
     +  - 11.d0/24.d0*HPL1(1)*GPL1(0)*pi*ci
     +  + 23.d0/24.d0*HPL1(1)*GPL1(0)*pi**2
     +  - 1.d0/8.d0*HPL1(1)*GPL1(0)*u**(-1)
     +  - 3.d0/8.d0*HPL1(1)*GPL1(0)*u
     +  - 3.d0/8.d0*HPL1(1)*GPL1(0)*v
     +  - 383.d0/144.d0*HPL1(1)*GPL1(1)
     +  + 3.d0/4.d0*HPL1(1)*GPL1(1)*pi*u**(-1)*ci
     +
      col2 = col2 - 7.d0/8.d0*HPL1(1)*GPL1(1)*pi*ci
     +  - 2.d0/3.d0*HPL1(1)*GPL1(1)*pi**2
     +  - 1.d0/8.d0*HPL1(1)*GPL1(1)*u**(-3)*v
     +  + 1.d0/8.d0*HPL1(1)*GPL1(1)*u**(-3)
     +  - 1.d0/2.d0*HPL1(1)*GPL1(1)*u**(-2)*v
     +  - 1.d0/16.d0*HPL1(1)*GPL1(1)*u**(-2)
     +  + 1.d0/8.d0*HPL1(1)*GPL1(1)*u**(-1)*v**(-1)
     +  - 1.d0/8.d0*HPL1(1)*GPL1(1)*u**(-1)*v
     +
      col2 = col2 + 3.d0/8.d0*HPL1(1)*GPL1(1)*u
     +  - 1.d0/8.d0*HPL1(1)*GPL1(1)*v**(-1)*upv**(-1)
     +  + 3.d0/8.d0*HPL1(1)*GPL1(1)*v
     +  - 1.d0/8.d0*HPL1(1)*GPL1(1)*upv**(-1)
     +  - HPL1(1)*GPL1(2)*pi*ci
     +  + 1.d0/3.d0*HPL1(1)*GPL1(2)*pi**2
     +  - 1.d0/3.d0*HPL1(1)*GPL1(3)*pi*ci
     +  - 5.d0/8.d0*HPL1(1)*GPL1(3)*pi**2
     +
      col2 = col2 - 7.d0/4.d0*HPL1(1)*GPL2(0,0)
     +  - HPL1(1)*GPL2(0,0)*pi*ci
     +  + 7.d0/6.d0*HPL1(1)*GPL2(0,1)
     +  + 3.d0/4.d0*HPL1(1)*GPL2(0,1)*pi*ci
     +  - 3.d0/4.d0*HPL1(1)*GPL2(0,1)*u**(-1)
     +  - 1.d0/8.d0*HPL1(1)*GPL2(0,1)*upv**(-2)
     +  - 1.d0/4.d0*HPL1(1)*GPL2(0,1)*upv**(-1)
     +  - HPL1(1)*GPL2(0,2)
     +
      col2 = col2 - HPL1(1)*GPL2(0,2)*pi*ci
     +  + 1.d0/4.d0*HPL1(1)*GPL2(0,3)*pi*ci
     +  + 5.d0/12.d0*HPL1(1)*GPL2(1,0)
     +  + HPL1(1)*GPL2(1,0)*pi*ci
     +  + 1.d0/6.d0*HPL1(1)*GPL2(1,1)
     +  - HPL1(1)*GPL2(1,1)*pi*ci
     +  - 3.d0/4.d0*HPL1(1)*GPL2(1,1)*u**(-1)
     +  + HPL1(1)*GPL2(1,2)
     +
      col2 = col2 + HPL1(1)*GPL2(1,2)*pi*ci
     +  + 3.d0/4.d0*HPL1(1)*GPL2(1,2)*u**(-1)
     +  - HPL1(1)*GPL2(2,0)
     +  - HPL1(1)*GPL2(2,0)*pi*ci
     +  + 4.d0/3.d0*HPL1(1)*GPL2(2,1)
     +  + 1.d0/4.d0*HPL1(1)*GPL2(2,1)*pi*ci
     +  - 1.d0/4.d0*HPL1(1)*GPL2(2,1)*u**(-2)*v
     +  + 1.d0/8.d0*HPL1(1)*GPL2(2,1)*u**(-2)*v**2
     +
      col2 = col2 + 1.d0/8.d0*HPL1(1)*GPL2(2,1)*u**(-2)
     +  - 1.d0/4.d0*HPL1(1)*GPL2(2,1)*u**(-1)*v
     +  + HPL1(1)*GPL2(2,1)*u**(-1)
     +  - 1.d0/8.d0*HPL1(1)*GPL2(2,1)*upv**(-2)
     +  - 1.d0/4.d0*HPL1(1)*GPL2(2,1)*upv**(-1)
     +  - 1.d0/4.d0*HPL1(1)*GPL2(2,3)*pi*ci
     +  - 1.d0/3.d0*HPL1(1)*GPL2(3,0)
     +  + HPL1(1)*GPL2(3,0)*pi*ci
     +
      col2 = col2 - 1.d0/2.d0*HPL1(1)*GPL3(0,0,1)
     +  - HPL1(1)*GPL3(0,0,2)
     +  - 1.d0/4.d0*HPL1(1)*GPL3(0,1,0)
     +  + HPL1(1)*GPL3(0,1,2)
     +  - HPL1(1)*GPL3(0,2,0)
     +  + 1.d0/4.d0*HPL1(1)*GPL3(0,2,1)
     +  + 1.d0/4.d0*HPL1(1)*GPL3(0,3,0)
     +  + HPL1(1)*GPL3(1,0,2)
     +
      col2 = col2 + HPL1(1)*GPL3(1,1,1)
     +  - HPL1(1)*GPL3(1,1,2)
     +  + HPL1(1)*GPL3(1,2,0)
     +  - HPL1(1)*GPL3(1,2,1)
     +  - HPL1(1)*GPL3(2,0,0)
     +  + 1.d0/4.d0*HPL1(1)*GPL3(2,0,1)
     +  + 1.d0/4.d0*HPL1(1)*GPL3(2,1,0)
     +  - HPL1(1)*GPL3(2,1,1)
     +
      col2 = col2 + 1.d0/2.d0*HPL1(1)*GPL3(2,2,1)
     +  - 1.d0/4.d0*HPL1(1)*GPL3(2,3,0)
     +  + HPL1(1)*GPL3(3,0,0)
     +  + 1.d0/4.d0*HPL1(1)*GPL3(3,0,1)
     +  + 1.d0/4.d0*HPL1(1)*GPL3(3,2,1)
     +  - 383.d0/144.d0*HPL2(0,0)
     +  + 35.d0/24.d0*HPL2(0,0)*pi*ci
     +  - 9.d0/8.d0*HPL2(0,0)*pi**2
     +
      col2 = col2 + 3.d0/8.d0*HPL2(0,0)*u
     +  + 3.d0/8.d0*HPL2(0,0)*v
     +  - 1.d0/16.d0*HPL2(0,0)*omumv**(-2)
     +  - 1.d0/4.d0*HPL2(0,0)*omumv**(-1)
     +  + 49.d0/12.d0*HPL2(0,0)*GPL1(0)
     +  + 7.d0/4.d0*HPL2(0,0)*GPL1(0)*pi*ci
     +  - 11.d0/4.d0*HPL2(0,0)*GPL1(1)
     +  - 3.d0/4.d0*HPL2(0,0)*GPL1(1)*pi*ci
     +
      col2 = col2 + 19.d0/6.d0*HPL2(0,0)*GPL1(2)
     +  + HPL2(0,0)*GPL1(2)*pi*ci
     +  - 11.d0/6.d0*HPL2(0,0)*GPL1(3)
     +  - 2*HPL2(0,0)*GPL1(3)*pi*ci
     +  + HPL2(0,0)*GPL2(0,0)
     +  + 3.d0/4.d0*HPL2(0,0)*GPL2(0,2)
     +  - HPL2(0,0)*GPL2(0,3)
     +  - 1.d0/4.d0*HPL2(0,0)*GPL2(1,0)
     +
      col2 = col2 - 1.d0/4.d0*HPL2(0,0)*GPL2(1,2)
     +  + 3.d0/4.d0*HPL2(0,0)*GPL2(2,0)
     +  - 3.d0/2.d0*HPL2(0,0)*GPL2(3,0)
     +  - 1.d0/2.d0*HPL2(0,0)*GPL2(3,2)
     +  + HPL2(0,0)*GPL2(3,3)
     +  + 47.d0/24.d0*HPL2(0,1)*pi*ci
     +  - 5.d0/8.d0*HPL2(0,1)*pi**2
     +  + 1.d0/8.d0*HPL2(0,1)*u**(-1)*v
     +
      col2 = col2 - 1.d0/8.d0*HPL2(0,1)*u**(-1)
     +  + 1.d0/8.d0*HPL2(0,1)*upv**(-1)
     +  + 47.d0/24.d0*HPL2(0,1)*GPL1(0)
     +  + HPL2(0,1)*GPL1(0)*pi*ci
     +  - 47.d0/24.d0*HPL2(0,1)*GPL1(1)
     +  - 3.d0/4.d0*HPL2(0,1)*GPL1(1)*pi*ci
     +  - 1.d0/3.d0*HPL2(0,1)*GPL1(2)
     +  + 1.d0/2.d0*HPL2(0,1)*GPL1(2)*pi*ci
     +
      col2 = col2 + 1.d0/4.d0*HPL2(0,1)*GPL1(2)*u**(-2)*v
     +  - 1.d0/8.d0*HPL2(0,1)*GPL1(2)*u**(-2)*v**2
     +  - 1.d0/8.d0*HPL2(0,1)*GPL1(2)*u**(-2)
     +  + 1.d0/4.d0*HPL2(0,1)*GPL1(2)*u**(-1)*v
     +  - 1.d0/4.d0*HPL2(0,1)*GPL1(2)*u**(-1)
     +  + 1.d0/8.d0*HPL2(0,1)*GPL1(2)*upv**(-2)
     +  + 1.d0/4.d0*HPL2(0,1)*GPL1(2)*upv**(-1)
     +  + 1.d0/3.d0*HPL2(0,1)*GPL1(3)
     +
      col2 = col2 - 3.d0/4.d0*HPL2(0,1)*GPL1(3)*pi*ci
     +  + HPL2(0,1)*GPL2(0,0)
     +  - 1.d0/2.d0*HPL2(0,1)*GPL2(0,1)
     +  + 3.d0/4.d0*HPL2(0,1)*GPL2(0,2)
     +  - 1.d0/4.d0*HPL2(0,1)*GPL2(0,3)
     +  - 3.d0/4.d0*HPL2(0,1)*GPL2(1,0)
     +  + 1.d0/2.d0*HPL2(0,1)*GPL2(2,0)
     +  + 1.d0/2.d0*HPL2(0,1)*GPL2(2,1)
     +
      col2 = col2 - 1.d0/2.d0*HPL2(0,1)*GPL2(2,2)
     +  + 1.d0/4.d0*HPL2(0,1)*GPL2(2,3)
     +  - 3.d0/4.d0*HPL2(0,1)*GPL2(3,0)
     +  - 1.d0/4.d0*HPL2(0,1)*GPL2(3,2)
     +  - 671.d0/144.d0*HPL2(1,0)
     +  + 1.d0/8.d0*HPL2(1,0)*pi*upv**(-2)*ci
     +  + 1.d0/4.d0*HPL2(1,0)*pi*upv**(-1)*ci
     +  + 1.d0/2.d0*HPL2(1,0)*pi*ci
     +
      col2 = col2 - HPL2(1,0)*pi**2
     +  - 1.d0/8.d0*HPL2(1,0)*u**(-1)*v
     +  + 3.d0/8.d0*HPL2(1,0)*u**(-1)
     +  + 3.d0/8.d0*HPL2(1,0)*u
     +  + 3.d0/8.d0*HPL2(1,0)*v
     +  - 1.d0/8.d0*HPL2(1,0)*upv**(-1)
     +  + 1.d0/16.d0*HPL2(1,0)*omumv**(-2)
     +  + 1.d0/4.d0*HPL2(1,0)*omumv**(-1)
     +
      col2 = col2 + 25.d0/8.d0*HPL2(1,0)*GPL1(0)
     +  + 3.d0/2.d0*HPL2(1,0)*GPL1(0)*pi*ci
     +  - 43.d0/24.d0*HPL2(1,0)*GPL1(1)
     +  - 1.d0/2.d0*HPL2(1,0)*GPL1(1)*pi*ci
     +  - 3.d0/4.d0*HPL2(1,0)*GPL1(1)*u**(-1)
     +  + HPL2(1,0)*GPL1(2)
     +  - 1.d0/4.d0*HPL2(1,0)*GPL1(2)*u**(-2)*v
     +  + 1.d0/8.d0*HPL2(1,0)*GPL1(2)*u**(-2)*v**2
     +
      col2 = col2 + 1.d0/8.d0*HPL2(1,0)*GPL1(2)*u**(-2)
     +  - 1.d0/4.d0*HPL2(1,0)*GPL1(2)*u**(-1)*v
     +  + 1.d0/4.d0*HPL2(1,0)*GPL1(2)*u**(-1)
     +  - 1.d0/8.d0*HPL2(1,0)*GPL1(2)*upv**(-2)
     +  - 1.d0/4.d0*HPL2(1,0)*GPL1(2)*upv**(-1)
     +  + 1.d0/3.d0*HPL2(1,0)*GPL1(3)
     +  - HPL2(1,0)*GPL1(3)*pi*ci
     +  + HPL2(1,0)*GPL2(0,0)
     +
      col2 = col2 - 1.d0/2.d0*HPL2(1,0)*GPL2(0,1)
     +  + 1.d0/2.d0*HPL2(1,0)*GPL2(0,2)
     +  - 1.d0/4.d0*HPL2(1,0)*GPL2(0,3)
     +  - 1.d0/2.d0*HPL2(1,0)*GPL2(1,0)
     +  + HPL2(1,0)*GPL2(1,1)
     +  - 5.d0/4.d0*HPL2(1,0)*GPL2(1,2)
     +  + 1.d0/4.d0*HPL2(1,0)*GPL2(2,0)
     +  - 1.d0/2.d0*HPL2(1,0)*GPL2(2,1)
     +
      col2 = col2 + 1.d0/2.d0*HPL2(1,0)*GPL2(2,2)
     +  + 1.d0/4.d0*HPL2(1,0)*GPL2(2,3)
     +  - 3.d0/4.d0*HPL2(1,0)*GPL2(3,0)
     +  + 1.d0/4.d0*HPL2(1,0)*GPL2(3,2)
     +  + HPL2(1,1)*pi*ci
     +  - 1.d0/3.d0*HPL2(1,1)*pi**2
     +  + HPL2(1,1)*GPL1(0)
     +  + HPL2(1,1)*GPL1(0)*pi*ci
     +
      col2 = col2 - HPL2(1,1)*GPL1(1)
     +  - HPL2(1,1)*GPL1(1)*pi*ci
     +  - 3.d0/4.d0*HPL2(1,1)*GPL1(1)*u**(-1)
     +  + HPL2(1,1)*GPL2(0,0)
     +  - HPL2(1,1)*GPL2(0,1)
     +  - HPL2(1,1)*GPL2(1,0)
     +  + HPL2(1,1)*GPL2(1,1)
     +  - 33.d0/8.d0*HPL3(0,0,0)
     +
      col2 = col2 - 2*HPL3(0,0,0)*pi*ci
     +  - 3.d0/2.d0*HPL3(0,0,0)*GPL1(0)
     +  + 1.d0/4.d0*HPL3(0,0,0)*GPL1(1)
     +  - 1.d0/4.d0*HPL3(0,0,0)*GPL1(2)
     +  + 3.d0/2.d0*HPL3(0,0,0)*GPL1(3)
     +  + 1.d0/3.d0*HPL3(0,0,1)
     +  - 3.d0/4.d0*HPL3(0,0,1)*pi*ci
     +  - 3.d0/4.d0*HPL3(0,0,1)*GPL1(0)
     +
      col2 = col2 + 1.d0/2.d0*HPL3(0,0,1)*GPL1(1)
     +  - 1.d0/4.d0*HPL3(0,0,1)*GPL1(2)
     +  + 1.d0/2.d0*HPL3(0,0,1)*GPL1(3)
     +  - 47.d0/24.d0*HPL3(0,1,0)
     +  - HPL3(0,1,0)*pi*ci
     +  - 3.d0/4.d0*HPL3(0,1,0)*GPL1(0)
     +  + 3.d0/4.d0*HPL3(0,1,0)*GPL1(1)
     +  - 1.d0/2.d0*HPL3(0,1,0)*GPL1(2)
     +
      col2 = col2 + 1.d0/2.d0*HPL3(0,1,0)*GPL1(3)
     +  - 19.d0/6.d0*HPL3(1,0,0)
     +  - HPL3(1,0,0)*pi*ci
     +  - 3.d0/4.d0*HPL3(1,0,0)*GPL1(0)
     +  + 1.d0/4.d0*HPL3(1,0,0)*GPL1(1)
     +  + 1.d0/2.d0*HPL3(1,0,0)*GPL1(3)
     +  + 1.d0/3.d0*HPL3(1,0,1)
     +  - 1.d0/2.d0*HPL3(1,0,1)*pi*ci
     +
      col2 = col2 - 1.d0/8.d0*HPL3(1,0,1)*upv**(-2)
     +  - 1.d0/4.d0*HPL3(1,0,1)*upv**(-1)
     +  - 1.d0/2.d0*HPL3(1,0,1)*GPL1(0)
     +  + 1.d0/4.d0*HPL3(1,0,1)*GPL1(2)
     +  + 1.d0/4.d0*HPL3(1,0,1)*GPL1(3)
     +  - HPL3(1,1,0)
     +  + 1.d0/8.d0*HPL3(1,1,0)*upv**(-2)
     +  + 1.d0/4.d0*HPL3(1,1,0)*upv**(-1)
     +
      col2 = col2 - 1.d0/4.d0*HPL3(1,1,0)*GPL1(0)
     +  + 5.d0/4.d0*HPL3(1,1,0)*GPL1(1)
     +  - 3.d0/4.d0*HPL3(1,1,0)*GPL1(2)
     +  - 1.d0/4.d0*HPL3(1,1,0)*GPL1(3)
     +  + 3.d0/2.d0*HPL4(0,0,0,0)
     +  + 1.d0/2.d0*HPL4(0,0,0,1)
     +  + 1.d0/2.d0*HPL4(0,0,1,0)
     +  + 1.d0/2.d0*HPL4(0,1,0,0)
     +
      col2 = col2 + 1.d0/4.d0*HPL4(0,1,0,1)
     +  - 1.d0/4.d0*HPL4(0,1,1,0)
     +  + 1.d0/4.d0*HPL4(1,0,0,0)
     +  + 1.d0/4.d0*HPL4(1,0,0,1)
     +  + 1.d0/2.d0*HPL4(1,0,1,0)
     +  - 1.d0/4.d0*HPL4(1,1,0,1)
     +  + 3.d0/4.d0*HPL4(1,1,1,0)
     +  + 49.d0/24.d0*GPL1(0)
     +
      col2 = col2 + 1.d0/8.d0*GPL1(0)*pi*u**(-1)*ci
     +  + 3.d0/8.d0*GPL1(0)*pi*u*ci
     +  + 3.d0/8.d0*GPL1(0)*pi*v*ci
     +  - 527.d0/144.d0*GPL1(0)*pi*ci
     +  + 31.d0/72.d0*GPL1(0)*pi**2
     +  - 1.d0/3.d0*GPL1(0)*pi**3*ci
     +  + 2.d0/3.d0*GPL1(0)*u**(-1)
     +  + 1.d0/16.d0*GPL1(0)*omumv**(-1)
     +
      col2 = col2 + 1.d0/8.d0*GPL1(1)*pi*u**(-3)*v*ci
     +  - 1.d0/8.d0*GPL1(1)*pi*u**(-3)*ci
     +  + 1.d0/2.d0*GPL1(1)*pi*u**(-2)*v*ci
     +  + 1.d0/16.d0*GPL1(1)*pi*u**(-2)*ci
     +  - 1.d0/8.d0*GPL1(1)*pi*u**(-1)*v**(-1)*ci
     +  + 1.d0/8.d0*GPL1(1)*pi*u**(-1)*v*ci
     +  - 3.d0/8.d0*GPL1(1)*pi*u*ci
     +  + 1.d0/8.d0*GPL1(1)*pi*v**(-1)*upv**(-1)*ci
     +
      col2 = col2 - 3.d0/8.d0*GPL1(1)*pi*v*ci
     +  + 1.d0/8.d0*GPL1(1)*pi*upv**(-1)*ci
     +  + 383.d0/144.d0*GPL1(1)*pi*ci
     +  + 1.d0/8.d0*GPL1(1)*pi**2*u**(-1)
     +  - 95.d0/72.d0*GPL1(1)*pi**2
     +  + 11.d0/24.d0*GPL1(1)*pi**3*ci
     +  - 5.d0/4.d0*GPL1(1)*zeta3
     +  + 31.d0/24.d0*GPL1(2)
     +
      col2 = col2 + 1.d0/8.d0*GPL1(2)*pi*u**(-1)*ci
     +  + 3.d0/8.d0*GPL1(2)*pi*u*ci
     +  + 3.d0/8.d0*GPL1(2)*pi*v*ci
     +  - 527.d0/144.d0*GPL1(2)*pi*ci
     +  - 1.d0/12.d0*GPL1(2)*pi**2*u**(-2)*v
     +  + 1.d0/24.d0*GPL1(2)*pi**2*u**(-2)*v**2
     +  + 1.d0/24.d0*GPL1(2)*pi**2*u**(-2)
     +  - 1.d0/12.d0*GPL1(2)*pi**2*u**(-1)*v
     +
      col2 = col2 + 1.d0/12.d0*GPL1(2)*pi**2*u**(-1)
     +  - 1.d0/16.d0*GPL1(2)*pi**2*upv**(-2)
     +  - 1.d0/8.d0*GPL1(2)*pi**2*upv**(-1)
     +  + 17.d0/24.d0*GPL1(2)*pi**2
     +  - 1.d0/2.d0*GPL1(2)*pi**3*ci
     +  + 5.d0/4.d0*GPL1(2)*zeta3
     +  + 1.d0/8.d0*GPL1(2)*u**(-2)*v
     +  - 1.d0/8.d0*GPL1(2)*u**(-2)
     +
      col2 = col2 + 7.d0/24.d0*GPL1(2)*u**(-1)*v
     +  + 11.d0/48.d0*GPL1(2)*u**(-1)
     +  + 2.d0/3.d0*GPL1(2)*v*omu**(-1)
     +  - 3.d0/8.d0*GPL1(3)*pi*u*ci
     +  - 3.d0/8.d0*GPL1(3)*pi*v*ci
     +  + 1.d0/16.d0*GPL1(3)*pi*omumv**(-2)*ci
     +  + 1.d0/4.d0*GPL1(3)*pi*omumv**(-1)*ci
     +  + 383.d0/144.d0*GPL1(3)*pi*ci
     +
      col2 = col2 - 115.d0/72.d0*GPL1(3)*pi**2
     +  + 3.d0/8.d0*GPL1(3)*pi**3*ci
     +  + 7.d0/4.d0*GPL2(0,0)*pi*ci
     +  - 1.d0/2.d0*GPL2(0,0)*pi**2
     +  + 3.d0/4.d0*GPL2(0,1)*pi*u**(-1)*ci
     +  + 1.d0/8.d0*GPL2(0,1)*pi*upv**(-2)*ci
     +  + 1.d0/4.d0*GPL2(0,1)*pi*upv**(-1)*ci
     +  - 7.d0/6.d0*GPL2(0,1)*pi*ci
     +
      col2 = col2 + 1.d0/4.d0*GPL2(0,1)*pi**2
     +  - 527.d0/144.d0*GPL2(0,2)
     +  + 11.d0/24.d0*GPL2(0,2)*pi*ci
     +  - 23.d0/24.d0*GPL2(0,2)*pi**2
     +  + 1.d0/8.d0*GPL2(0,2)*u**(-1)
     +  + 3.d0/8.d0*GPL2(0,2)*u
     +  + 3.d0/8.d0*GPL2(0,2)*v
     +  - 4.d0/3.d0*GPL2(0,3)*pi*ci
     +
      col2 = col2 + 1.d0/12.d0*GPL2(0,3)*pi**2
     +  - 5.d0/12.d0*GPL2(1,0)*pi*ci
     +  + 7.d0/12.d0*GPL2(1,0)*pi**2
     +  + 3.d0/4.d0*GPL2(1,1)*pi*u**(-1)*ci
     +  - 1.d0/6.d0*GPL2(1,1)*pi*ci
     +  - 1.d0/6.d0*GPL2(1,1)*pi**2
     +  + 383.d0/144.d0*GPL2(1,2)
     +  - 3.d0/4.d0*GPL2(1,2)*pi*u**(-1)*ci
     +
      col2 = col2 + 7.d0/8.d0*GPL2(1,2)*pi*ci
     +  + 2.d0/3.d0*GPL2(1,2)*pi**2
     +  + 1.d0/8.d0*GPL2(1,2)*u**(-3)*v
     +  - 1.d0/8.d0*GPL2(1,2)*u**(-3)
     +  + 1.d0/2.d0*GPL2(1,2)*u**(-2)*v
     +  + 1.d0/16.d0*GPL2(1,2)*u**(-2)
     +  - 1.d0/8.d0*GPL2(1,2)*u**(-1)*v**(-1)
     +  + 1.d0/8.d0*GPL2(1,2)*u**(-1)*v
     +
      col2 = col2 - 3.d0/8.d0*GPL2(1,2)*u
     +  + 1.d0/8.d0*GPL2(1,2)*v**(-1)*upv**(-1)
     +  - 3.d0/8.d0*GPL2(1,2)*v
     +  + 1.d0/8.d0*GPL2(1,2)*upv**(-1)
     +  - 527.d0/144.d0*GPL2(2,0)
     +  + 11.d0/24.d0*GPL2(2,0)*pi*ci
     +  - 23.d0/24.d0*GPL2(2,0)*pi**2
     +  + 1.d0/8.d0*GPL2(2,0)*u**(-1)
     +
      col2 = col2 + 3.d0/8.d0*GPL2(2,0)*u
     +  + 3.d0/8.d0*GPL2(2,0)*v
     +  + 1.d0/4.d0*GPL2(2,1)*pi*u**(-2)*v*ci
     +  - 1.d0/8.d0*GPL2(2,1)*pi*u**(-2)*v**2*ci
     +  - 1.d0/8.d0*GPL2(2,1)*pi*u**(-2)*ci
     +  + 1.d0/4.d0*GPL2(2,1)*pi*u**(-1)*v*ci
     +  - GPL2(2,1)*pi*u**(-1)*ci
     +  + 1.d0/8.d0*GPL2(2,1)*pi*upv**(-2)*ci
     +
      col2 = col2 + 1.d0/4.d0*GPL2(2,1)*pi*upv**(-1)*ci
     +  - 4.d0/3.d0*GPL2(2,1)*pi*ci
     +  - 1.d0/12.d0*GPL2(2,1)*pi**2
     +  + GPL2(2,2)*pi*ci
     +  - 1.d0/3.d0*GPL2(2,2)*pi**2
     +  - 5.d0/12.d0*GPL2(2,3)*pi*ci
     +  + 1.d0/4.d0*GPL2(2,3)*pi**2
     +  + 383.d0/144.d0*GPL2(3,0)
     +
      col2 = col2 + 7.d0/8.d0*GPL2(3,0)*pi*ci
     +  + 7.d0/8.d0*GPL2(3,0)*pi**2
     +  - 3.d0/8.d0*GPL2(3,0)*u
     +  - 3.d0/8.d0*GPL2(3,0)*v
     +  + 1.d0/16.d0*GPL2(3,0)*omumv**(-2)
     +  + 1.d0/4.d0*GPL2(3,0)*omumv**(-1)
     +  + 1.d0/3.d0*GPL2(3,2)*pi*ci
     +  + 5.d0/8.d0*GPL2(3,2)*pi**2
     +
      col2 = col2 - 11.d0/12.d0*GPL2(3,3)*pi*ci
     +  - 1.d0/3.d0*GPL2(3,3)*pi**2
     +  + 1.d0/2.d0*GPL3(0,0,1)*pi*ci
     +  + 7.d0/4.d0*GPL3(0,0,2)
     +  + GPL3(0,0,2)*pi*ci
     +  - 1.d0/2.d0*GPL3(0,0,3)*pi*ci
     +  + 1.d0/4.d0*GPL3(0,1,0)*pi*ci
     +  - 7.d0/6.d0*GPL3(0,1,2)
     +
      col2 = col2 - 3.d0/4.d0*GPL3(0,1,2)*pi*ci
     +  + 3.d0/4.d0*GPL3(0,1,2)*u**(-1)
     +  + 1.d0/8.d0*GPL3(0,1,2)*upv**(-2)
     +  + 1.d0/4.d0*GPL3(0,1,2)*upv**(-1)
     +  + 7.d0/4.d0*GPL3(0,2,0)
     +  + GPL3(0,2,0)*pi*ci
     +  - 1.d0/4.d0*GPL3(0,2,1)*pi*ci
     +  + GPL3(0,2,2)
     +
      col2 = col2 + GPL3(0,2,2)*pi*ci
     +  - 1.d0/4.d0*GPL3(0,2,3)*pi*ci
     +  - 4.d0/3.d0*GPL3(0,3,0)
     +  - 1.d0/4.d0*GPL3(0,3,0)*pi*ci
     +  - 1.d0/4.d0*GPL3(0,3,2)*pi*ci
     +  + GPL3(0,3,3)*pi*ci
     +  - 5.d0/12.d0*GPL3(1,0,2)
     +  - GPL3(1,0,2)*pi*ci
     +
      col2 = col2 - 1.d0/4.d0*GPL3(1,0,3)*pi*ci
     +  - GPL3(1,1,1)*pi*ci
     +  - 1.d0/6.d0*GPL3(1,1,2)
     +  + GPL3(1,1,2)*pi*ci
     +  + 3.d0/4.d0*GPL3(1,1,2)*u**(-1)
     +  - 5.d0/12.d0*GPL3(1,2,0)
     +  - GPL3(1,2,0)*pi*ci
     +  + GPL3(1,2,1)*pi*ci
     +
      col2 = col2 - GPL3(1,2,2)
     +  - GPL3(1,2,2)*pi*ci
     +  - 3.d0/4.d0*GPL3(1,2,2)*u**(-1)
     +  - 1.d0/4.d0*GPL3(1,2,3)*pi*ci
     +  + 7.d0/4.d0*GPL3(2,0,0)
     +  + GPL3(2,0,0)*pi*ci
     +  - 1.d0/4.d0*GPL3(2,0,1)*pi*ci
     +  + GPL3(2,0,2)
     +
      col2 = col2 + GPL3(2,0,2)*pi*ci
     +  - 1.d0/4.d0*GPL3(2,0,3)*pi*ci
     +  - 1.d0/4.d0*GPL3(2,1,0)*pi*ci
     +  + GPL3(2,1,1)*pi*ci
     +  - 4.d0/3.d0*GPL3(2,1,2)
     +  - 1.d0/4.d0*GPL3(2,1,2)*pi*ci
     +  + 1.d0/4.d0*GPL3(2,1,2)*u**(-2)*v
     +  - 1.d0/8.d0*GPL3(2,1,2)*u**(-2)*v**2
     +
      col2 = col2 - 1.d0/8.d0*GPL3(2,1,2)*u**(-2)
     +  + 1.d0/4.d0*GPL3(2,1,2)*u**(-1)*v
     +  - GPL3(2,1,2)*u**(-1)
     +  + 1.d0/8.d0*GPL3(2,1,2)*upv**(-2)
     +  + 1.d0/4.d0*GPL3(2,1,2)*upv**(-1)
     +  + GPL3(2,2,0)
     +  + GPL3(2,2,0)*pi*ci
     +  - 1.d0/2.d0*GPL3(2,2,1)*pi*ci
     +
      col2 = col2 + 1.d0/2.d0*GPL3(2,2,3)*pi*ci
     +  - 5.d0/12.d0*GPL3(2,3,0)
     +  - 3.d0/4.d0*GPL3(2,3,0)*pi*ci
     +  + 1.d0/4.d0*GPL3(2,3,2)*pi*ci
     +  - 7.d0/4.d0*GPL3(3,0,0)
     +  - GPL3(3,0,0)*pi*ci
     +  - 1.d0/4.d0*GPL3(3,0,1)*pi*ci
     +  + 1.d0/3.d0*GPL3(3,0,2)
     +
      col2 = col2 - GPL3(3,0,2)*pi*ci
     +  + GPL3(3,0,3)*pi*ci
     +  + 1.d0/3.d0*GPL3(3,2,0)
     +  - GPL3(3,2,0)*pi*ci
     +  - 1.d0/4.d0*GPL3(3,2,1)*pi*ci
     +  - 11.d0/12.d0*GPL3(3,3,0)
     +  + GPL3(3,3,0)*pi*ci
     +  - GPL3(3,3,3)*pi*ci
     +
      col2 = col2 + 1.d0/2.d0*GPL4(0,0,1,2)
     +  + GPL4(0,0,2,2)
     +  - 1.d0/2.d0*GPL4(0,0,3,0)
     +  + 1.d0/4.d0*GPL4(0,1,0,2)
     +  + 1.d0/4.d0*GPL4(0,1,2,0)
     +  - GPL4(0,1,2,2)
     +  + GPL4(0,2,0,2)
     +  - 1.d0/4.d0*GPL4(0,2,1,2)
     +
      col2 = col2 + GPL4(0,2,2,0)
     +  - 1.d0/4.d0*GPL4(0,2,3,0)
     +  - 1.d0/4.d0*GPL4(0,3,0,2)
     +  - 1.d0/4.d0*GPL4(0,3,2,0)
     +  + GPL4(0,3,3,0)
     +  - GPL4(1,0,2,2)
     +  - 1.d0/4.d0*GPL4(1,0,3,0)
     +  - GPL4(1,1,1,2)
     +
      col2 = col2 + GPL4(1,1,2,2)
     +  - GPL4(1,2,0,2)
     +  + GPL4(1,2,1,2)
     +  - GPL4(1,2,2,0)
     +  - 1.d0/4.d0*GPL4(1,2,3,0)
     +  + GPL4(2,0,0,2)
     +  - 1.d0/4.d0*GPL4(2,0,1,2)
     +  + GPL4(2,0,2,0)
     +
      col2 = col2 - 1.d0/4.d0*GPL4(2,0,3,0)
     +  - 1.d0/4.d0*GPL4(2,1,0,2)
     +  + GPL4(2,1,1,2)
     +  - 1.d0/4.d0*GPL4(2,1,2,0)
     +  + GPL4(2,2,0,0)
     +  - 1.d0/2.d0*GPL4(2,2,1,2)
     +  + 1.d0/2.d0*GPL4(2,2,3,0)
     +  - GPL4(2,3,0,0)
     +
      col2 = col2 + 1.d0/4.d0*GPL4(2,3,0,2)
     +  + 1.d0/4.d0*GPL4(2,3,2,0)
     +  - GPL4(3,0,0,2)
     +  - 1.d0/4.d0*GPL4(3,0,1,2)
     +  - GPL4(3,0,2,0)
     +  + GPL4(3,0,3,0)
     +  - GPL4(3,2,0,0)
     +  - 1.d0/4.d0*GPL4(3,2,1,2)
     +
      col2 = col2 + GPL4(3,3,0,0)
     +  - GPL4(3,3,3,0)
     +

      col3 =
     +  - 255.d0/128.d0
     +  + 5.d0/2.d0*pi*zeta3*ci
     +  - 5.d0/8.d0*pi*u**(-2)*v*ci
     +  + 5.d0/8.d0*pi*u**(-2)*ci
     +  - 11.d0/16.d0*pi*u**(-1)*v*ci
     +  + 1.d0/8.d0*pi*u**(-1)*ci
     +  - 17.d0/16.d0*pi*v*omu**(-1)*ci
     +
      col3 = col3 - 5.d0/16.d0*pi*omumv**(-1)*ci
     +  + 3.d0/32.d0*pi*ci
     +  + 1.d0/12.d0*pi**2*u**(-1)*v
     +  + 1.d0/48.d0*pi**2*u**(-1)*omv**(-1)
     +  + 1.d0/24.d0*pi**2*u**(-1)
     +  + 1.d0/4.d0*pi**2*u
     +  + 1.d0/4.d0*pi**2*v
     +  - 1.d0/16.d0*pi**2*upv**(-1)
     +
      col3 = col3 + 1.d0/48.d0*pi**2*omv**(-1)*omumv**(-1)
     +  - 5.d0/96.d0*pi**2*omumv**(-2)
     +  + 1.d0/6.d0*pi**2*omumv**(-1)
     +  + 1.d0/32.d0*pi**2
     +  - 1.d0/8.d0*pi**3*ci
     +  + 7.d0/48.d0*pi**4
     +  + 3.d0/2.d0*zeta3
     +  - 5.d0/8.d0*u**(-1)
     +
      col3 = col3 + 3.d0/32.d0*HPL1(0)
     +  + 1.d0/8.d0*HPL1(0)*pi*u**(-1)*v*ci
     +  + 1.d0/8.d0*HPL1(0)*pi*u**(-1)*omv**(-1)*ci
     +  + 1.d0/8.d0*HPL1(0)*pi*u**(-1)*ci
     +  + 3.d0/8.d0*HPL1(0)*pi*u*ci
     +  + 3.d0/8.d0*HPL1(0)*pi*v*ci
     +  - 1.d0/8.d0*HPL1(0)*pi*upv**(-1)*ci
     +  + 1.d0/8.d0*HPL1(0)*pi*omv**(-1)*omumv**(-1)*ci
     +
      col3 = col3 - 5.d0/16.d0*HPL1(0)*pi*omumv**(-2)*ci
     +  + HPL1(0)*pi*omumv**(-1)*ci
     +  + 1.d0/2.d0*HPL1(0)*pi*ci
     +  - 1.d0/8.d0*HPL1(0)*pi**2
     +  + 1.d0/24.d0*HPL1(0)*pi**3*ci
     +  + 2*HPL1(0)*zeta3
     +  + 5.d0/8.d0*HPL1(0)*u**(-2)*v
     +  + 11.d0/16.d0*HPL1(0)*u**(-1)*v
     +
      col3 = col3 - 5.d0/16.d0*HPL1(0)*u**(-1)*omv**(-1)
     +  + 5.d0/16.d0*HPL1(0)*u**(-1)
     +  + 17.d0/16.d0*HPL1(0)*v*omu**(-1)
     +  - 5.d0/16.d0*HPL1(0)*omv**(-1)*omumv**(-1)
     +  + 5.d0/16.d0*HPL1(0)*omumv**(-1)
     +  + 1.d0/2.d0*HPL1(0)*GPL1(0)
     +  + 1.d0/24.d0*HPL1(0)*GPL1(0)*pi**2
     +  + 1.d0/8.d0*HPL1(0)*GPL1(0)*u**(-1)*v
     +
      col3 = col3 + 1.d0/8.d0*HPL1(0)*GPL1(0)*u**(-1)*omv**(-1)
     +  + 3.d0/8.d0*HPL1(0)*GPL1(0)*u
     +  + 3.d0/8.d0*HPL1(0)*GPL1(0)*v
     +  + 1.d0/8.d0*HPL1(0)*GPL1(0)*omv**(-1)*omumv**(-1)
     +  - 5.d0/16.d0*HPL1(0)*GPL1(0)*omumv**(-2)
     +  + HPL1(0)*GPL1(0)*omumv**(-1)
     +  - 1.d0/2.d0*HPL1(0)*GPL1(1)
     +  + 1.d0/8.d0*HPL1(0)*GPL1(1)*pi*u**(-2)*ci
     +
      col3 = col3 + 1.d0/4.d0*HPL1(0)*GPL1(1)*pi*u**(-1)*ci
     +  + 1.d0/24.d0*HPL1(0)*GPL1(1)*pi**2
     +  + 5.d0/8.d0*HPL1(0)*GPL1(1)*u**(-3)*v
     +  - 5.d0/8.d0*HPL1(0)*GPL1(1)*u**(-3)
     +  + 1.d0/4.d0*HPL1(0)*GPL1(1)*u**(-2)*v
     +  + 7.d0/16.d0*HPL1(0)*GPL1(1)*u**(-2)
     +  + 1.d0/8.d0*HPL1(0)*GPL1(1)*u**(-1)*v**(-1)
     +  - 1.d0/8.d0*HPL1(0)*GPL1(1)*u**(-1)*v
     +
      col3 = col3 - 3.d0/4.d0*HPL1(0)*GPL1(1)*u**(-1)
     +  - 3.d0/8.d0*HPL1(0)*GPL1(1)*u
     +  - 1.d0/8.d0*HPL1(0)*GPL1(1)*v**(-1)*upv**(-1)
     +  - 3.d0/8.d0*HPL1(0)*GPL1(1)*v
     +  - 1.d0/8.d0*HPL1(0)*GPL1(1)*upv**(-1)
     +  + 1.d0/2.d0*HPL1(0)*GPL1(2)
     +  - 1.d0/8.d0*HPL1(0)*GPL1(2)*pi*upv**(-2)*ci
     +  - 1.d0/4.d0*HPL1(0)*GPL1(2)*pi*upv**(-1)*ci
     +
      col3 = col3 + 3.d0/8.d0*HPL1(0)*GPL1(2)*pi*ci
     +  + 1.d0/8.d0*HPL1(0)*GPL1(2)*pi**2
     +  - 5.d0/8.d0*HPL1(0)*GPL1(2)*u**(-3)*v
     +  + 5.d0/8.d0*HPL1(0)*GPL1(2)*u**(-3)
     +  - 1.d0/4.d0*HPL1(0)*GPL1(2)*u**(-2)*v
     +  - 7.d0/16.d0*HPL1(0)*GPL1(2)*u**(-2)
     +  - 1.d0/8.d0*HPL1(0)*GPL1(2)*u**(-1)*v**(-1)
     +  + 1.d0/8.d0*HPL1(0)*GPL1(2)*u**(-1)*v
     +
      col3 = col3 + 7.d0/8.d0*HPL1(0)*GPL1(2)*u**(-1)
     +  + 3.d0/8.d0*HPL1(0)*GPL1(2)*u
     +  + 1.d0/8.d0*HPL1(0)*GPL1(2)*v**(-1)*upv**(-1)
     +  + 3.d0/8.d0*HPL1(0)*GPL1(2)*v
     +  - 1.d0/2.d0*HPL1(0)*GPL1(3)
     +  + 5.d0/24.d0*HPL1(0)*GPL1(3)*pi**2
     +  - 1.d0/8.d0*HPL1(0)*GPL1(3)*u**(-1)*v
     +  - 1.d0/8.d0*HPL1(0)*GPL1(3)*u**(-1)*omv**(-1)
     +
      col3 = col3 - 3.d0/8.d0*HPL1(0)*GPL1(3)*u
     +  - 3.d0/8.d0*HPL1(0)*GPL1(3)*v
     +  - 1.d0/8.d0*HPL1(0)*GPL1(3)*omv**(-1)*omumv**(-1)
     +  + 5.d0/16.d0*HPL1(0)*GPL1(3)*omumv**(-2)
     +  - HPL1(0)*GPL1(3)*omumv**(-1)
     +  - 3.d0/8.d0*HPL1(0)*GPL2(0,1)
     +  + 1.d0/4.d0*HPL1(0)*GPL2(0,1)*pi*ci
     +  - 1.d0/8.d0*HPL1(0)*GPL2(0,1)*u**(-2)
     +
      col3 = col3 - 1.d0/4.d0*HPL1(0)*GPL2(0,1)*u**(-1)
     +  - 1.d0/8.d0*HPL1(0)*GPL2(0,1)*upv**(-2)
     +  - 1.d0/4.d0*HPL1(0)*GPL2(0,1)*upv**(-1)
     +  + 3.d0/8.d0*HPL1(0)*GPL2(0,2)
     +  + 1.d0/8.d0*HPL1(0)*GPL2(0,2)*u**(-2)
     +  + 1.d0/4.d0*HPL1(0)*GPL2(0,2)*u**(-1)
     +  + 1.d0/8.d0*HPL1(0)*GPL2(0,2)*upv**(-2)
     +  + 1.d0/4.d0*HPL1(0)*GPL2(0,2)*upv**(-1)
     +
      col3 = col3 - 1.d0/4.d0*HPL1(0)*GPL2(0,3)*pi*ci
     +  + 3.d0/8.d0*HPL1(0)*GPL2(1,1)
     +  + 1.d0/8.d0*HPL1(0)*GPL2(1,1)*u**(-2)
     +  + 1.d0/4.d0*HPL1(0)*GPL2(1,1)*u**(-1)
     +  - 3.d0/8.d0*HPL1(0)*GPL2(1,2)
     +  + 3.d0/8.d0*HPL1(0)*GPL2(2,0)
     +  - 3.d0/8.d0*HPL1(0)*GPL2(2,1)
     +  + 1.d0/4.d0*HPL1(0)*GPL2(2,1)*pi*ci
     +
      col3 = col3 + 1.d0/8.d0*HPL1(0)*GPL2(2,1)*u**(-2)
     +  + 1.d0/4.d0*HPL1(0)*GPL2(2,1)*u**(-1)
     +  - 1.d0/8.d0*HPL1(0)*GPL2(2,1)*upv**(-2)
     +  - 1.d0/4.d0*HPL1(0)*GPL2(2,1)*upv**(-1)
     +  + 3.d0/8.d0*HPL1(0)*GPL2(2,2)
     +  - 3.d0/4.d0*HPL1(0)*GPL2(2,2)*pi*ci
     +  - 1.d0/8.d0*HPL1(0)*GPL2(2,2)*u**(-2)
     +  - 1.d0/4.d0*HPL1(0)*GPL2(2,2)*u**(-1)
     +
      col3 = col3 - 3.d0/8.d0*HPL1(0)*GPL2(2,3)
     +  - 1.d0/2.d0*HPL1(0)*GPL2(3,0)*pi*ci
     +  + 1.d0/2.d0*HPL1(0)*GPL2(3,2)*pi*ci
     +  + 1.d0/2.d0*HPL1(0)*GPL2(3,3)*pi*ci
     +  + 1.d0/4.d0*HPL1(0)*GPL3(0,0,1)
     +  - 1.d0/4.d0*HPL1(0)*GPL3(0,0,2)
     +  - 1.d0/2.d0*HPL1(0)*GPL3(0,0,3)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(0,1,0)
     +
      col3 = col3 + 1.d0/4.d0*HPL1(0)*GPL3(0,1,1)
     +  - 1.d0/4.d0*HPL1(0)*GPL3(0,1,2)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(0,2,0)
     +  - 1.d0/2.d0*HPL1(0)*GPL3(0,2,1)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(0,2,2)
     +  - 1.d0/2.d0*HPL1(0)*GPL3(0,2,3)
     +  - 1.d0/4.d0*HPL1(0)*GPL3(0,3,0)
     +  + 1.d0/2.d0*HPL1(0)*GPL3(0,3,3)
     +
      col3 = col3 - 1.d0/4.d0*HPL1(0)*GPL3(1,0,1)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(1,0,2)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(1,0,3)
     +  - 1.d0/2.d0*HPL1(0)*GPL3(1,1,1)
     +  + 1.d0/2.d0*HPL1(0)*GPL3(1,1,2)
     +  - 1.d0/4.d0*HPL1(0)*GPL3(1,2,0)
     +  + 3.d0/4.d0*HPL1(0)*GPL3(1,2,1)
     +  - 1.d0/2.d0*HPL1(0)*GPL3(1,2,2)
     +
      col3 = col3 + 1.d0/4.d0*HPL1(0)*GPL3(1,2,3)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(2,0,0)
     +  - 1.d0/2.d0*HPL1(0)*GPL3(2,0,1)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(2,0,2)
     +  - 1.d0/2.d0*HPL1(0)*GPL3(2,0,3)
     +  + 1.d0/2.d0*HPL1(0)*GPL3(2,1,1)
     +  - 1.d0/4.d0*HPL1(0)*GPL3(2,1,2)
     +  - 1.d0/4.d0*HPL1(0)*GPL3(2,2,0)
     +
      col3 = col3 - 1.d0/2.d0*HPL1(0)*GPL3(2,2,1)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(2,2,3)
     +  - 1.d0/4.d0*HPL1(0)*GPL3(2,3,0)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(2,3,2)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(2,3,3)
     +  - 1.d0/2.d0*HPL1(0)*GPL3(3,0,0)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(3,0,1)
     +  - 1.d0/4.d0*HPL1(0)*GPL3(3,0,2)
     +
      col3 = col3 + 3.d0/4.d0*HPL1(0)*GPL3(3,0,3)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(3,2,0)
     +  + 1.d0/4.d0*HPL1(0)*GPL3(3,2,1)
     +  - 1.d0/4.d0*HPL1(0)*GPL3(3,2,3)
     +  + 1.d0/2.d0*HPL1(0)*GPL3(3,3,0)
     +  - 1.d0/2.d0*HPL1(0)*GPL3(3,3,3)
     +  - 3.d0/8.d0*HPL1(1)
     +  + 1.d0/8.d0*HPL1(1)*pi*u**(-1)*v*ci
     +
      col3 = col3 + 3.d0/8.d0*HPL1(1)*pi*u*ci
     +  + 3.d0/8.d0*HPL1(1)*pi*v*ci
     +  + 1.d0/2.d0*HPL1(1)*pi*ci
     +  + 1.d0/16.d0*HPL1(1)*pi**2*upv**(-2)
     +  + 1.d0/8.d0*HPL1(1)*pi**2*upv**(-1)
     +  - 1.d0/4.d0*HPL1(1)*pi**2
     +  + 5.d0/24.d0*HPL1(1)*pi**3*ci
     +  + HPL1(1)*zeta3
     +
      col3 = col3 + 5.d0/8.d0*HPL1(1)*u**(-2)*v
     +  - 5.d0/8.d0*HPL1(1)*u**(-2)
     +  + 11.d0/16.d0*HPL1(1)*u**(-1)*v
     +  - 1.d0/8.d0*HPL1(1)*u**(-1)
     +  + 17.d0/16.d0*HPL1(1)*v*omu**(-1)
     +  + 1.d0/2.d0*HPL1(1)*GPL1(0)
     +  + 3.d0/8.d0*HPL1(1)*GPL1(0)*pi*ci
     +  + 3.d0/8.d0*HPL1(1)*GPL1(0)*pi**2
     +
      col3 = col3 + 1.d0/8.d0*HPL1(1)*GPL1(0)*u**(-1)*v
     +  - 1.d0/8.d0*HPL1(1)*GPL1(0)*u**(-1)
     +  + 3.d0/8.d0*HPL1(1)*GPL1(0)*u
     +  + 3.d0/8.d0*HPL1(1)*GPL1(0)*v
     +  - 1.d0/2.d0*HPL1(1)*GPL1(1)
     +  + 1.d0/8.d0*HPL1(1)*GPL1(1)*pi*u**(-2)*ci
     +  + 1.d0/4.d0*HPL1(1)*GPL1(1)*pi*u**(-1)*ci
     +  - 3.d0/8.d0*HPL1(1)*GPL1(1)*pi*ci
     +
      col3 = col3 - 1.d0/8.d0*HPL1(1)*GPL1(1)*pi**2
     +  + 5.d0/8.d0*HPL1(1)*GPL1(1)*u**(-3)*v
     +  - 5.d0/8.d0*HPL1(1)*GPL1(1)*u**(-3)
     +  + 1.d0/4.d0*HPL1(1)*GPL1(1)*u**(-2)*v
     +  + 7.d0/16.d0*HPL1(1)*GPL1(1)*u**(-2)
     +  + 1.d0/8.d0*HPL1(1)*GPL1(1)*u**(-1)*v**(-1)
     +  - 1.d0/8.d0*HPL1(1)*GPL1(1)*u**(-1)*v
     +  - 3.d0/4.d0*HPL1(1)*GPL1(1)*u**(-1)
     +
      col3 = col3 - 3.d0/8.d0*HPL1(1)*GPL1(1)*u
     +  - 1.d0/8.d0*HPL1(1)*GPL1(1)*v**(-1)*upv**(-1)
     +  - 3.d0/8.d0*HPL1(1)*GPL1(1)*v
     +  - 1.d0/8.d0*HPL1(1)*GPL1(1)*upv**(-1)
     +  + 3.d0/8.d0*HPL1(1)*GPL1(2)*pi*ci
     +  + 1.d0/3.d0*HPL1(1)*GPL1(2)*pi**2
     +  + 1.d0/8.d0*HPL1(1)*GPL1(2)*u**(-1)
     +  - 1.d0/6.d0*HPL1(1)*GPL1(3)*pi**2
     +
      col3 = col3 - 1.d0/4.d0*HPL1(1)*GPL2(0,0)*pi*ci
     +  - 3.d0/8.d0*HPL1(1)*GPL2(0,1)
     +  - 1.d0/8.d0*HPL1(1)*GPL2(0,1)*u**(-2)
     +  - 1.d0/4.d0*HPL1(1)*GPL2(0,1)*u**(-1)
     +  - 1.d0/8.d0*HPL1(1)*GPL2(0,1)*upv**(-2)
     +  - 1.d0/4.d0*HPL1(1)*GPL2(0,1)*upv**(-1)
     +  + 3.d0/8.d0*HPL1(1)*GPL2(0,2)
     +  - 1.d0/4.d0*HPL1(1)*GPL2(0,2)*pi*ci
     +
      col3 = col3 + 3.d0/8.d0*HPL1(1)*GPL2(1,1)
     +  + 1.d0/2.d0*HPL1(1)*GPL2(1,1)*pi*ci
     +  + 1.d0/8.d0*HPL1(1)*GPL2(1,1)*u**(-2)
     +  + 1.d0/4.d0*HPL1(1)*GPL2(1,1)*u**(-1)
     +  - 3.d0/8.d0*HPL1(1)*GPL2(1,2)
     +  + 1.d0/8.d0*HPL1(1)*GPL2(1,2)*u**(-2)
     +  + 1.d0/4.d0*HPL1(1)*GPL2(1,2)*u**(-1)
     +  + 3.d0/8.d0*HPL1(1)*GPL2(2,0)
     +
      col3 = col3 - 1.d0/4.d0*HPL1(1)*GPL2(2,0)*pi*ci
     +  - 3.d0/8.d0*HPL1(1)*GPL2(2,1)
     +  + 1.d0/8.d0*HPL1(1)*GPL2(2,1)*u**(-2)
     +  + 1.d0/4.d0*HPL1(1)*GPL2(2,1)*u**(-1)
     +  - 1.d0/8.d0*HPL1(1)*GPL2(2,1)*upv**(-2)
     +  - 1.d0/4.d0*HPL1(1)*GPL2(2,1)*upv**(-1)
     +  + 1.d0/4.d0*HPL1(1)*GPL2(2,3)*pi*ci
     +  + 1.d0/4.d0*HPL1(1)*GPL3(0,0,1)
     +
      col3 = col3 - 1.d0/4.d0*HPL1(1)*GPL3(0,0,2)
     +  + 1.d0/4.d0*HPL1(1)*GPL3(0,1,0)
     +  + 1.d0/4.d0*HPL1(1)*GPL3(0,1,1)
     +  - 1.d0/4.d0*HPL1(1)*GPL3(0,1,2)
     +  - 1.d0/4.d0*HPL1(1)*GPL3(0,2,0)
     +  - 1.d0/2.d0*HPL1(1)*GPL3(0,2,1)
     +  - 1.d0/4.d0*HPL1(1)*GPL3(1,0,1)
     +  - 1.d0/2.d0*HPL1(1)*GPL3(1,1,1)
     +
      col3 = col3 + 1.d0/2.d0*HPL1(1)*GPL3(1,1,2)
     +  + 3.d0/4.d0*HPL1(1)*GPL3(1,2,1)
     +  - 1.d0/4.d0*HPL1(1)*GPL3(2,0,0)
     +  - 1.d0/2.d0*HPL1(1)*GPL3(2,0,1)
     +  + 1.d0/2.d0*HPL1(1)*GPL3(2,1,1)
     +  - 1.d0/2.d0*HPL1(1)*GPL3(2,2,1)
     +  + 1.d0/4.d0*HPL1(1)*GPL3(2,3,0)
     +  + 1.d0/4.d0*HPL1(1)*GPL3(3,0,1)
     +
      col3 = col3 + 1.d0/4.d0*HPL1(1)*GPL3(3,2,1)
     +  - 1.d0/2.d0*HPL2(0,0)
     +  + 5.d0/24.d0*HPL2(0,0)*pi**2
     +  - 1.d0/8.d0*HPL2(0,0)*u**(-1)*v
     +  - 1.d0/8.d0*HPL2(0,0)*u**(-1)*omv**(-1)
     +  - 3.d0/8.d0*HPL2(0,0)*u
     +  - 3.d0/8.d0*HPL2(0,0)*v
     +  - 1.d0/8.d0*HPL2(0,0)*omv**(-1)*omumv**(-1)
     +
      col3 = col3 + 5.d0/16.d0*HPL2(0,0)*omumv**(-2)
     +  - HPL2(0,0)*omumv**(-1)
     +  - 1.d0/2.d0*HPL2(0,0)*GPL1(0)*pi*ci
     +  + 1.d0/4.d0*HPL2(0,0)*GPL1(1)*pi*ci
     +  - 3.d0/8.d0*HPL2(0,0)*GPL1(2)
     +  - 1.d0/4.d0*HPL2(0,0)*GPL1(2)*pi*ci
     +  + 3.d0/4.d0*HPL2(0,0)*GPL1(3)*pi*ci
     +  - 1.d0/2.d0*HPL2(0,0)*GPL2(0,0)
     +
      col3 = col3 - 1.d0/2.d0*HPL2(0,0)*GPL2(0,2)
     +  + 1.d0/2.d0*HPL2(0,0)*GPL2(0,3)
     +  + 1.d0/4.d0*HPL2(0,0)*GPL2(1,0)
     +  + 1.d0/4.d0*HPL2(0,0)*GPL2(1,2)
     +  - 1.d0/2.d0*HPL2(0,0)*GPL2(2,0)
     +  + 1.d0/4.d0*HPL2(0,0)*GPL2(2,2)
     +  + 1.d0/4.d0*HPL2(0,0)*GPL2(2,3)
     +  + 3.d0/4.d0*HPL2(0,0)*GPL2(3,0)
     +
      col3 = col3 - 1.d0/4.d0*HPL2(0,0)*GPL2(3,2)
     +  - 1.d0/2.d0*HPL2(0,0)*GPL2(3,3)
     +  - 1.d0/6.d0*HPL2(0,1)*pi**2
     +  - 1.d0/8.d0*HPL2(0,1)*u**(-1)
     +  + 1.d0/8.d0*HPL2(0,1)*upv**(-1)
     +  - 1.d0/8.d0*HPL2(0,1)*GPL1(1)*u**(-2)
     +  - 1.d0/4.d0*HPL2(0,1)*GPL1(1)*u**(-1)
     +  + 1.d0/4.d0*HPL2(0,1)*GPL1(2)*pi*ci
     +
      col3 = col3 + 1.d0/8.d0*HPL2(0,1)*GPL1(2)*upv**(-2)
     +  + 1.d0/4.d0*HPL2(0,1)*GPL1(2)*upv**(-1)
     +  + 1.d0/4.d0*HPL2(0,1)*GPL2(0,2)
     +  - 1.d0/4.d0*HPL2(0,1)*GPL2(1,2)
     +  + 1.d0/4.d0*HPL2(0,1)*GPL2(2,0)
     +  - 1.d0/4.d0*HPL2(0,1)*GPL2(2,1)
     +  + 1.d0/2.d0*HPL2(0,1)*GPL2(2,2)
     +  - 1.d0/4.d0*HPL2(0,1)*GPL2(2,3)
     +
      col3 = col3 - 1.d0/4.d0*HPL2(0,1)*GPL2(3,2)
     +  - 1.d0/2.d0*HPL2(1,0)
     +  + 1.d0/8.d0*HPL2(1,0)*pi*upv**(-2)*ci
     +  + 1.d0/4.d0*HPL2(1,0)*pi*upv**(-1)*ci
     +  - 3.d0/8.d0*HPL2(1,0)*pi*ci
     +  - 1.d0/8.d0*HPL2(1,0)*pi**2
     +  - 1.d0/8.d0*HPL2(1,0)*u**(-1)*v
     +  + 1.d0/8.d0*HPL2(1,0)*u**(-1)*omv**(-1)
     +
      col3 = col3 + 1.d0/4.d0*HPL2(1,0)*u**(-1)
     +  - 3.d0/8.d0*HPL2(1,0)*u
     +  - 3.d0/8.d0*HPL2(1,0)*v
     +  - 1.d0/8.d0*HPL2(1,0)*upv**(-1)
     +  + 1.d0/8.d0*HPL2(1,0)*omv**(-1)*omumv**(-1)
     +  - 5.d0/16.d0*HPL2(1,0)*omumv**(-2)
     +  + HPL2(1,0)*omumv**(-1)
     +  - 3.d0/8.d0*HPL2(1,0)*GPL1(0)
     +
      col3 = col3 + 3.d0/8.d0*HPL2(1,0)*GPL1(1)
     +  - 3.d0/8.d0*HPL2(1,0)*GPL1(2)
     +  + 3.d0/4.d0*HPL2(1,0)*GPL1(2)*pi*ci
     +  - 1.d0/8.d0*HPL2(1,0)*GPL1(2)*upv**(-2)
     +  - 1.d0/4.d0*HPL2(1,0)*GPL1(2)*upv**(-1)
     +  - 1.d0/2.d0*HPL2(1,0)*GPL1(3)*pi*ci
     +  - 1.d0/4.d0*HPL2(1,0)*GPL2(0,0)
     +  + 1.d0/4.d0*HPL2(1,0)*GPL2(0,1)
     +
      col3 = col3 + 1.d0/4.d0*HPL2(1,0)*GPL2(0,2)
     +  + 1.d0/4.d0*HPL2(1,0)*GPL2(1,0)
     +  - 1.d0/2.d0*HPL2(1,0)*GPL2(1,1)
     +  + 1.d0/4.d0*HPL2(1,0)*GPL2(2,0)
     +  + 1.d0/4.d0*HPL2(1,0)*GPL2(2,1)
     +  - 1.d0/2.d0*HPL2(1,0)*GPL2(2,2)
     +  - 1.d0/4.d0*HPL2(1,0)*GPL2(2,3)
     +  - 1.d0/4.d0*HPL2(1,0)*GPL2(3,0)
     +
      col3 = col3 + 1.d0/2.d0*HPL2(1,0)*GPL2(3,2)
     +  - 3.d0/8.d0*HPL2(1,1)*pi*ci
     +  - 1.d0/3.d0*HPL2(1,1)*pi**2
     +  - 1.d0/8.d0*HPL2(1,1)*u**(-1)
     +  - 3.d0/8.d0*HPL2(1,1)*GPL1(0)
     +  + 1.d0/4.d0*HPL2(1,1)*GPL1(0)*pi*ci
     +  + 3.d0/8.d0*HPL2(1,1)*GPL1(1)
     +  - 1.d0/8.d0*HPL2(1,1)*GPL1(1)*u**(-2)
     +
      col3 = col3 - 1.d0/4.d0*HPL2(1,1)*GPL1(1)*u**(-1)
     +  + 1.d0/4.d0*HPL2(1,1)*GPL2(0,0)
     +  + 1.d0/4.d0*HPL2(1,1)*GPL2(0,1)
     +  - 1.d0/2.d0*HPL2(1,1)*GPL2(1,1)
     +  + 3.d0/4.d0*HPL3(0,0,0)*pi*ci
     +  + 3.d0/4.d0*HPL3(0,0,0)*GPL1(0)
     +  - 1.d0/4.d0*HPL3(0,0,0)*GPL1(1)
     +  + 1.d0/2.d0*HPL3(0,0,0)*GPL1(2)
     +
      col3 = col3 - 3.d0/4.d0*HPL3(0,0,0)*GPL1(3)
     +  - 1.d0/4.d0*HPL3(0,0,1)*GPL1(2)
     +  - 1.d0/2.d0*HPL3(0,1,0)*pi*ci
     +  - 1.d0/4.d0*HPL3(0,1,0)*GPL1(0)
     +  + 1.d0/4.d0*HPL3(0,1,0)*GPL1(1)
     +  - 1.d0/2.d0*HPL3(0,1,0)*GPL1(2)
     +  + 1.d0/4.d0*HPL3(0,1,0)*GPL1(3)
     +  + 3.d0/8.d0*HPL3(1,0,0)
     +
      col3 = col3 + 1.d0/4.d0*HPL3(1,0,0)*pi*ci
     +  + 1.d0/2.d0*HPL3(1,0,0)*GPL1(0)
     +  - 1.d0/4.d0*HPL3(1,0,0)*GPL1(1)
     +  - 1.d0/4.d0*HPL3(1,0,0)*GPL1(2)
     +  + 1.d0/4.d0*HPL3(1,0,0)*GPL1(3)
     +  - 1.d0/4.d0*HPL3(1,0,1)*pi*ci
     +  - 1.d0/8.d0*HPL3(1,0,1)*upv**(-2)
     +  - 1.d0/4.d0*HPL3(1,0,1)*upv**(-1)
     +
      col3 = col3 - 1.d0/4.d0*HPL3(1,0,1)*GPL1(0)
     +  + 1.d0/4.d0*HPL3(1,0,1)*GPL1(1)
     +  - 1.d0/2.d0*HPL3(1,0,1)*GPL1(2)
     +  + 1.d0/4.d0*HPL3(1,0,1)*GPL1(3)
     +  + 3.d0/8.d0*HPL3(1,1,0)
     +  - 3.d0/4.d0*HPL3(1,1,0)*pi*ci
     +  + 1.d0/8.d0*HPL3(1,1,0)*upv**(-2)
     +  + 1.d0/4.d0*HPL3(1,1,0)*upv**(-1)
     +
      col3 = col3 - 1.d0/4.d0*HPL3(1,1,0)*GPL1(0)
     +  + 1.d0/2.d0*HPL3(1,1,0)*GPL1(2)
     +  - 1.d0/2.d0*HPL3(1,1,0)*GPL1(3)
     +  - 3.d0/4.d0*HPL4(0,0,0,0)
     +  + 1.d0/4.d0*HPL4(0,0,1,0)
     +  + 1.d0/4.d0*HPL4(0,1,0,0)
     +  + 1.d0/4.d0*HPL4(0,1,0,1)
     +  - 1.d0/2.d0*HPL4(0,1,1,0)
     +
      col3 = col3 - 1.d0/2.d0*HPL4(1,0,0,0)
     +  + 1.d0/4.d0*HPL4(1,0,0,1)
     +  + 1.d0/2.d0*HPL4(1,0,1,0)
     +  + 1.d0/4.d0*HPL4(1,1,0,0)
     +  + 1.d0/2.d0*HPL4(1,1,0,1)
     +  - 1.d0/2.d0*HPL4(1,1,1,0)
     +  - 3.d0/8.d0*GPL1(0)
     +  - 1.d0/8.d0*GPL1(0)*pi*u**(-1)*v*ci
     +
      col3 = col3 + 1.d0/8.d0*GPL1(0)*pi*u**(-1)*ci
     +  - 3.d0/8.d0*GPL1(0)*pi*u*ci
     +  - 3.d0/8.d0*GPL1(0)*pi*v*ci
     +  - 1.d0/2.d0*GPL1(0)*pi*ci
     +  + 1.d0/16.d0*GPL1(0)*pi**2
     +  - 7.d0/24.d0*GPL1(0)*pi**3*ci
     +  - 1.d0/4.d0*GPL1(0)*zeta3
     +  - 5.d0/16.d0*GPL1(0)*omumv**(-1)
     +
      col3 = col3 - 5.d0/8.d0*GPL1(1)*pi*u**(-3)*v*ci
     +  + 5.d0/8.d0*GPL1(1)*pi*u**(-3)*ci
     +  - 1.d0/4.d0*GPL1(1)*pi*u**(-2)*v*ci
     +  - 7.d0/16.d0*GPL1(1)*pi*u**(-2)*ci
     +  - 1.d0/8.d0*GPL1(1)*pi*u**(-1)*v**(-1)*ci
     +  + 1.d0/8.d0*GPL1(1)*pi*u**(-1)*v*ci
     +  + 3.d0/4.d0*GPL1(1)*pi*u**(-1)*ci
     +  + 3.d0/8.d0*GPL1(1)*pi*u*ci
     +
      col3 = col3 + 1.d0/8.d0*GPL1(1)*pi*v**(-1)*upv**(-1)*ci
     +  + 3.d0/8.d0*GPL1(1)*pi*v*ci
     +  + 1.d0/8.d0*GPL1(1)*pi*upv**(-1)*ci
     +  + 1.d0/2.d0*GPL1(1)*pi*ci
     +  + 5.d0/48.d0*GPL1(1)*pi**2*u**(-2)
     +  + 5.d0/24.d0*GPL1(1)*pi**2*u**(-1)
     +  - 1.d0/16.d0*GPL1(1)*pi**2
     +  + 1.d0/8.d0*GPL1(1)*pi**3*ci
     +
      col3 = col3 + 1.d0/2.d0*GPL1(1)*zeta3
     +  + 3.d0/8.d0*GPL1(2)
     +  - 1.d0/8.d0*GPL1(2)*pi*u**(-1)*v*ci
     +  - 3.d0/8.d0*GPL1(2)*pi*u*ci
     +  - 3.d0/8.d0*GPL1(2)*pi*v*ci
     +  - 1.d0/2.d0*GPL1(2)*pi*ci
     +  - 1.d0/16.d0*GPL1(2)*pi**2*upv**(-2)
     +  - 1.d0/8.d0*GPL1(2)*pi**2*upv**(-1)
     +
      col3 = col3 + 1.d0/4.d0*GPL1(2)*pi**2
     +  - 5.d0/24.d0*GPL1(2)*pi**3*ci
     +  - GPL1(2)*zeta3
     +  - 5.d0/8.d0*GPL1(2)*u**(-2)*v
     +  + 5.d0/8.d0*GPL1(2)*u**(-2)
     +  - 11.d0/16.d0*GPL1(2)*u**(-1)*v
     +  + 1.d0/8.d0*GPL1(2)*u**(-1)
     +  - 17.d0/16.d0*GPL1(2)*v*omu**(-1)
     +
      col3 = col3 + 1.d0/8.d0*GPL1(3)*pi*u**(-1)*v*ci
     +  + 1.d0/8.d0*GPL1(3)*pi*u**(-1)*omv**(-1)*ci
     +  + 3.d0/8.d0*GPL1(3)*pi*u*ci
     +  + 3.d0/8.d0*GPL1(3)*pi*v*ci
     +  + 1.d0/8.d0*GPL1(3)*pi*omv**(-1)*omumv**(-1)*ci
     +  - 5.d0/16.d0*GPL1(3)*pi*omumv**(-2)*ci
     +  + GPL1(3)*pi*omumv**(-1)*ci
     +  + 1.d0/2.d0*GPL1(3)*pi*ci
     +
      col3 = col3 + 1.d0/24.d0*GPL1(3)*pi**3*ci
     +  + 1.d0/2.d0*GPL1(3)*zeta3
     +  - 5.d0/24.d0*GPL2(0,0)*pi**2
     +  + 1.d0/8.d0*GPL2(0,1)*pi*u**(-2)*ci
     +  + 1.d0/4.d0*GPL2(0,1)*pi*u**(-1)*ci
     +  + 1.d0/8.d0*GPL2(0,1)*pi*upv**(-2)*ci
     +  + 1.d0/4.d0*GPL2(0,1)*pi*upv**(-1)*ci
     +  + 3.d0/8.d0*GPL2(0,1)*pi*ci
     +
      col3 = col3 + 5.d0/24.d0*GPL2(0,1)*pi**2
     +  - 1.d0/2.d0*GPL2(0,2)
     +  - 3.d0/8.d0*GPL2(0,2)*pi*ci
     +  - 3.d0/8.d0*GPL2(0,2)*pi**2
     +  - 1.d0/8.d0*GPL2(0,2)*u**(-1)*v
     +  + 1.d0/8.d0*GPL2(0,2)*u**(-1)
     +  - 3.d0/8.d0*GPL2(0,2)*u
     +  - 3.d0/8.d0*GPL2(0,2)*v
     +
      col3 = col3 + 1.d0/12.d0*GPL2(0,3)*pi**2
     +  + 1.d0/12.d0*GPL2(1,0)*pi**2
     +  - 1.d0/8.d0*GPL2(1,1)*pi*u**(-2)*ci
     +  - 1.d0/4.d0*GPL2(1,1)*pi*u**(-1)*ci
     +  - 3.d0/8.d0*GPL2(1,1)*pi*ci
     +  + 1.d0/12.d0*GPL2(1,1)*pi**2
     +  + 1.d0/2.d0*GPL2(1,2)
     +  - 1.d0/8.d0*GPL2(1,2)*pi*u**(-2)*ci
     +
      col3 = col3 - 1.d0/4.d0*GPL2(1,2)*pi*u**(-1)*ci
     +  + 3.d0/8.d0*GPL2(1,2)*pi*ci
     +  + 1.d0/8.d0*GPL2(1,2)*pi**2
     +  - 5.d0/8.d0*GPL2(1,2)*u**(-3)*v
     +  + 5.d0/8.d0*GPL2(1,2)*u**(-3)
     +  - 1.d0/4.d0*GPL2(1,2)*u**(-2)*v
     +  - 7.d0/16.d0*GPL2(1,2)*u**(-2)
     +  - 1.d0/8.d0*GPL2(1,2)*u**(-1)*v**(-1)
     +
      col3 = col3 + 1.d0/8.d0*GPL2(1,2)*u**(-1)*v
     +  + 3.d0/4.d0*GPL2(1,2)*u**(-1)
     +  + 3.d0/8.d0*GPL2(1,2)*u
     +  + 1.d0/8.d0*GPL2(1,2)*v**(-1)*upv**(-1)
     +  + 3.d0/8.d0*GPL2(1,2)*v
     +  + 1.d0/8.d0*GPL2(1,2)*upv**(-1)
     +  - 1.d0/2.d0*GPL2(2,0)
     +  - 3.d0/8.d0*GPL2(2,0)*pi*ci
     +
      col3 = col3 - 3.d0/8.d0*GPL2(2,0)*pi**2
     +  - 1.d0/8.d0*GPL2(2,0)*u**(-1)*v
     +  + 1.d0/8.d0*GPL2(2,0)*u**(-1)
     +  - 3.d0/8.d0*GPL2(2,0)*u
     +  - 3.d0/8.d0*GPL2(2,0)*v
     +  - 1.d0/8.d0*GPL2(2,1)*pi*u**(-2)*ci
     +  - 1.d0/4.d0*GPL2(2,1)*pi*u**(-1)*ci
     +  + 1.d0/8.d0*GPL2(2,1)*pi*upv**(-2)*ci
     +
      col3 = col3 + 1.d0/4.d0*GPL2(2,1)*pi*upv**(-1)*ci
     +  + 3.d0/8.d0*GPL2(2,1)*pi*ci
     +  + 1.d0/6.d0*GPL2(2,1)*pi**2
     +  - 3.d0/8.d0*GPL2(2,2)*pi*ci
     +  - 1.d0/3.d0*GPL2(2,2)*pi**2
     +  - 1.d0/8.d0*GPL2(2,2)*u**(-1)
     +  + 3.d0/8.d0*GPL2(2,3)*pi*ci
     +  + 1.d0/6.d0*GPL2(2,3)*pi**2
     +
      col3 = col3 + 1.d0/2.d0*GPL2(3,0)
     +  + 1.d0/24.d0*GPL2(3,0)*pi**2
     +  + 1.d0/8.d0*GPL2(3,0)*u**(-1)*v
     +  + 1.d0/8.d0*GPL2(3,0)*u**(-1)*omv**(-1)
     +  + 3.d0/8.d0*GPL2(3,0)*u
     +  + 3.d0/8.d0*GPL2(3,0)*v
     +  + 1.d0/8.d0*GPL2(3,0)*omv**(-1)*omumv**(-1)
     +  - 5.d0/16.d0*GPL2(3,0)*omumv**(-2)
     +
      col3 = col3 + GPL2(3,0)*omumv**(-1)
     +  + 1.d0/6.d0*GPL2(3,2)*pi**2
     +  + 1.d0/6.d0*GPL2(3,3)*pi**2
     +  - 1.d0/4.d0*GPL3(0,0,1)*pi*ci
     +  + 1.d0/4.d0*GPL3(0,0,2)*pi*ci
     +  + 1.d0/2.d0*GPL3(0,0,3)*pi*ci
     +  - 1.d0/4.d0*GPL3(0,1,0)*pi*ci
     +  - 1.d0/4.d0*GPL3(0,1,1)*pi*ci
     +
      col3 = col3 + 3.d0/8.d0*GPL3(0,1,2)
     +  + 1.d0/8.d0*GPL3(0,1,2)*u**(-2)
     +  + 1.d0/4.d0*GPL3(0,1,2)*u**(-1)
     +  + 1.d0/8.d0*GPL3(0,1,2)*upv**(-2)
     +  + 1.d0/4.d0*GPL3(0,1,2)*upv**(-1)
     +  + 1.d0/4.d0*GPL3(0,2,0)*pi*ci
     +  + 1.d0/2.d0*GPL3(0,2,1)*pi*ci
     +  - 3.d0/8.d0*GPL3(0,2,2)
     +
      col3 = col3 + 1.d0/4.d0*GPL3(0,2,2)*pi*ci
     +  + 1.d0/2.d0*GPL3(0,2,3)*pi*ci
     +  - 1.d0/2.d0*GPL3(0,3,3)*pi*ci
     +  + 1.d0/4.d0*GPL3(1,0,1)*pi*ci
     +  - 1.d0/4.d0*GPL3(1,0,3)*pi*ci
     +  + 1.d0/2.d0*GPL3(1,1,1)*pi*ci
     +  - 3.d0/8.d0*GPL3(1,1,2)
     +  - 1.d0/2.d0*GPL3(1,1,2)*pi*ci
     +
      col3 = col3 - 1.d0/8.d0*GPL3(1,1,2)*u**(-2)
     +  - 1.d0/4.d0*GPL3(1,1,2)*u**(-1)
     +  - 3.d0/4.d0*GPL3(1,2,1)*pi*ci
     +  + 3.d0/8.d0*GPL3(1,2,2)
     +  - 1.d0/8.d0*GPL3(1,2,2)*u**(-2)
     +  - 1.d0/4.d0*GPL3(1,2,2)*u**(-1)
     +  - 1.d0/4.d0*GPL3(1,2,3)*pi*ci
     +  + 1.d0/4.d0*GPL3(2,0,0)*pi*ci
     +
      col3 = col3 + 1.d0/2.d0*GPL3(2,0,1)*pi*ci
     +  - 3.d0/8.d0*GPL3(2,0,2)
     +  + 1.d0/4.d0*GPL3(2,0,2)*pi*ci
     +  + 1.d0/2.d0*GPL3(2,0,3)*pi*ci
     +  - 1.d0/2.d0*GPL3(2,1,1)*pi*ci
     +  + 3.d0/8.d0*GPL3(2,1,2)
     +  - 1.d0/8.d0*GPL3(2,1,2)*u**(-2)
     +  - 1.d0/4.d0*GPL3(2,1,2)*u**(-1)
     +
      col3 = col3 + 1.d0/8.d0*GPL3(2,1,2)*upv**(-2)
     +  + 1.d0/4.d0*GPL3(2,1,2)*upv**(-1)
     +  - 3.d0/8.d0*GPL3(2,2,0)
     +  + 1.d0/4.d0*GPL3(2,2,0)*pi*ci
     +  + 1.d0/2.d0*GPL3(2,2,1)*pi*ci
     +  - 1.d0/4.d0*GPL3(2,2,3)*pi*ci
     +  + 3.d0/8.d0*GPL3(2,3,0)
     +  - 1.d0/4.d0*GPL3(2,3,2)*pi*ci
     +
      col3 = col3 - 1.d0/4.d0*GPL3(2,3,3)*pi*ci
     +  - 1.d0/4.d0*GPL3(3,0,1)*pi*ci
     +  - 3.d0/4.d0*GPL3(3,0,3)*pi*ci
     +  - 1.d0/4.d0*GPL3(3,2,1)*pi*ci
     +  + 1.d0/4.d0*GPL3(3,2,3)*pi*ci
     +  - 1.d0/2.d0*GPL3(3,3,0)*pi*ci
     +  + 1.d0/2.d0*GPL3(3,3,3)*pi*ci
     +  - 1.d0/4.d0*GPL4(0,0,1,2)
     +
      col3 = col3 + 1.d0/4.d0*GPL4(0,0,2,2)
     +  + 1.d0/2.d0*GPL4(0,0,3,0)
     +  - 1.d0/4.d0*GPL4(0,1,0,2)
     +  - 1.d0/4.d0*GPL4(0,1,1,2)
     +  - 1.d0/4.d0*GPL4(0,1,2,0)
     +  + 1.d0/4.d0*GPL4(0,1,2,2)
     +  + 1.d0/4.d0*GPL4(0,2,0,2)
     +  + 1.d0/2.d0*GPL4(0,2,1,2)
     +
      col3 = col3 + 1.d0/4.d0*GPL4(0,2,2,0)
     +  + 1.d0/2.d0*GPL4(0,2,3,0)
     +  - 1.d0/2.d0*GPL4(0,3,3,0)
     +  + 1.d0/4.d0*GPL4(1,0,1,2)
     +  - 1.d0/4.d0*GPL4(1,0,3,0)
     +  + 1.d0/2.d0*GPL4(1,1,1,2)
     +  - 1.d0/2.d0*GPL4(1,1,2,2)
     +  - 3.d0/4.d0*GPL4(1,2,1,2)
     +
      col3 = col3 - 1.d0/4.d0*GPL4(1,2,3,0)
     +  + 1.d0/4.d0*GPL4(2,0,0,2)
     +  + 1.d0/2.d0*GPL4(2,0,1,2)
     +  + 1.d0/4.d0*GPL4(2,0,2,0)
     +  + 1.d0/2.d0*GPL4(2,0,3,0)
     +  - 1.d0/2.d0*GPL4(2,1,1,2)
     +  + 1.d0/4.d0*GPL4(2,2,0,0)
     +  + 1.d0/2.d0*GPL4(2,2,1,2)
     +
      col3 = col3 - 1.d0/4.d0*GPL4(2,2,3,0)
     +  + 1.d0/4.d0*GPL4(2,3,0,0)
     +  - 1.d0/4.d0*GPL4(2,3,0,2)
     +  - 1.d0/4.d0*GPL4(2,3,2,0)
     +  - 1.d0/4.d0*GPL4(2,3,3,0)
     +  - 1.d0/4.d0*GPL4(3,0,1,2)
     +  - 3.d0/4.d0*GPL4(3,0,3,0)
     +  - 1.d0/4.d0*GPL4(3,2,1,2)
     +
      col3 = col3 + 1.d0/4.d0*GPL4(3,2,3,0)
     +  - 1.d0/2.d0*GPL4(3,3,0,0)
     +  + 1.d0/2.d0*GPL4(3,3,3,0)
     +

      col4 =
     +  + 377.d0/2592.d0
     +  + 1.d0/12.d0*pi*u**(-1)*v*ci
     +  + 1.d0/12.d0*pi*v*omu**(-1)*ci
     +  + 155.d0/216.d0*pi*ci
     +  - 211.d0/432.d0*pi**2
     +  - 5.d0/144.d0*pi**3*ci
     +  + 131.d0/72.d0*zeta3
     +
      col4 = col4 + 19.d0/36.d0*u**(-1)
     +  + 1.d0/4.d0*omv**(-1)
     +  + 281.d0/216.d0*HPL1(0)
     +  - 29.d0/36.d0*HPL1(0)*pi*ci
     +  - 5.d0/144.d0*HPL1(0)*pi**2
     +  - 1.d0/12.d0*HPL1(0)*u**(-1)*v
     +  + 5.d0/24.d0*HPL1(0)*u**(-1)
     +  - 1.d0/12.d0*HPL1(0)*v*omu**(-1)
     +
      col4 = col4 + 1.d0/4.d0*HPL1(0)*omv**(-2)
     +  - 1.d0/2.d0*HPL1(0)*omv**(-1)
     +  - 29.d0/36.d0*HPL1(0)*GPL1(0)
     +  + 7.d0/24.d0*HPL1(0)*GPL1(0)*pi*ci
     +  + 29.d0/36.d0*HPL1(0)*GPL1(1)
     +  - 1.d0/12.d0*HPL1(0)*GPL1(1)*pi*ci
     +  - 29.d0/36.d0*HPL1(0)*GPL1(2)
     +  + 1.d0/8.d0*HPL1(0)*GPL1(2)*pi*ci
     +
      col4 = col4 + 29.d0/36.d0*HPL1(0)*GPL1(3)
     +  + 1.d0/12.d0*HPL1(0)*GPL1(3)*pi*ci
     +  + 1.d0/4.d0*HPL1(0)*GPL2(0,0)
     +  - 1.d0/24.d0*HPL1(0)*GPL2(0,1)
     +  + 1.d0/2.d0*HPL1(0)*GPL2(0,2)
     +  - 5.d0/24.d0*HPL1(0)*GPL2(0,3)
     +  - 1.d0/24.d0*HPL1(0)*GPL2(1,0)
     +  - 1.d0/6.d0*HPL1(0)*GPL2(1,1)
     +
      col4 = col4 - 7.d0/24.d0*HPL1(0)*GPL2(1,2)
     +  + 1.d0/2.d0*HPL1(0)*GPL2(2,0)
     +  - 5.d0/24.d0*HPL1(0)*GPL2(2,1)
     +  + 1.d0/4.d0*HPL1(0)*GPL2(2,2)
     +  - 1.d0/24.d0*HPL1(0)*GPL2(2,3)
     +  - 7.d0/24.d0*HPL1(0)*GPL2(3,0)
     +  - 1.d0/24.d0*HPL1(0)*GPL2(3,2)
     +  - 1.d0/6.d0*HPL1(0)*GPL2(3,3)
     +
      col4 = col4 + 1.d0/6.d0*HPL1(1)
     +  - 29.d0/36.d0*HPL1(1)*pi*ci
     +  + 1.d0/12.d0*HPL1(1)*pi**2
     +  - 1.d0/12.d0*HPL1(1)*u**(-1)*v
     +  + 5.d0/48.d0*HPL1(1)*u**(-1)
     +  - 1.d0/12.d0*HPL1(1)*v*omu**(-1)
     +  - 29.d0/36.d0*HPL1(1)*GPL1(0)
     +  + 1.d0/12.d0*HPL1(1)*GPL1(0)*pi*ci
     +
      col4 = col4 + 29.d0/36.d0*HPL1(1)*GPL1(1)
     +  + 1.d0/8.d0*HPL1(1)*GPL1(1)*pi*ci
     +  + 1.d0/4.d0*HPL1(1)*GPL1(2)*pi*ci
     +  - 1.d0/24.d0*HPL1(1)*GPL1(3)*pi*ci
     +  + 1.d0/4.d0*HPL1(1)*GPL2(0,0)
     +  - 1.d0/24.d0*HPL1(1)*GPL2(0,1)
     +  + 1.d0/4.d0*HPL1(1)*GPL2(0,2)
     +  - 1.d0/24.d0*HPL1(1)*GPL2(1,0)
     +
      col4 = col4 - 1.d0/6.d0*HPL1(1)*GPL2(1,1)
     +  - 1.d0/4.d0*HPL1(1)*GPL2(1,2)
     +  + 1.d0/4.d0*HPL1(1)*GPL2(2,0)
     +  - 5.d0/24.d0*HPL1(1)*GPL2(2,1)
     +  - 1.d0/24.d0*HPL1(1)*GPL2(3,0)
     +  + 29.d0/36.d0*HPL2(0,0)
     +  - 1.d0/3.d0*HPL2(0,0)*pi*ci
     +  - 17.d0/24.d0*HPL2(0,0)*GPL1(0)
     +
      col4 = col4 + 1.d0/2.d0*HPL2(0,0)*GPL1(1)
     +  - 13.d0/24.d0*HPL2(0,0)*GPL1(2)
     +  + 1.d0/3.d0*HPL2(0,0)*GPL1(3)
     +  - 11.d0/24.d0*HPL2(0,1)*pi*ci
     +  - 11.d0/24.d0*HPL2(0,1)*GPL1(0)
     +  + 11.d0/24.d0*HPL2(0,1)*GPL1(1)
     +  - 1.d0/24.d0*HPL2(0,1)*GPL1(2)
     +  + 1.d0/24.d0*HPL2(0,1)*GPL1(3)
     +
      col4 = col4 + 29.d0/36.d0*HPL2(1,0)
     +  - 1.d0/8.d0*HPL2(1,0)*pi*ci
     +  - 1.d0/2.d0*HPL2(1,0)*GPL1(0)
     +  + 7.d0/24.d0*HPL2(1,0)*GPL1(1)
     +  - 1.d0/4.d0*HPL2(1,0)*GPL1(2)
     +  + 1.d0/24.d0*HPL2(1,0)*GPL1(3)
     +  - 1.d0/4.d0*HPL2(1,1)*pi*ci
     +  - 1.d0/4.d0*HPL2(1,1)*GPL1(0)
     +
      col4 = col4 + 1.d0/4.d0*HPL2(1,1)*GPL1(1)
     +  + 3.d0/4.d0*HPL3(0,0,0)
     +  + 1.d0/24.d0*HPL3(0,0,1)
     +  + 11.d0/24.d0*HPL3(0,1,0)
     +  + 13.d0/24.d0*HPL3(1,0,0)
     +  + 1.d0/24.d0*HPL3(1,0,1)
     +  + 1.d0/4.d0*HPL3(1,1,0)
     +  - 1.d0/6.d0*GPL1(0)
     +
      col4 = col4 + 29.d0/36.d0*GPL1(0)*pi*ci
     +  - 1.d0/18.d0*GPL1(0)*pi**2
     +  - 5.d0/48.d0*GPL1(0)*u**(-1)
     +  - 29.d0/36.d0*GPL1(1)*pi*ci
     +  + 7.d0/36.d0*GPL1(1)*pi**2
     +  - 1.d0/6.d0*GPL1(2)
     +  + 29.d0/36.d0*GPL1(2)*pi*ci
     +  - 1.d0/12.d0*GPL1(2)*pi**2
     +
      col4 = col4 + 1.d0/12.d0*GPL1(2)*u**(-1)*v
     +  - 5.d0/48.d0*GPL1(2)*u**(-1)
     +  + 1.d0/12.d0*GPL1(2)*v*omu**(-1)
     +  - 29.d0/36.d0*GPL1(3)*pi*ci
     +  + 2.d0/9.d0*GPL1(3)*pi**2
     +  - 1.d0/4.d0*GPL2(0,0)*pi*ci
     +  + 1.d0/24.d0*GPL2(0,1)*pi*ci
     +  + 29.d0/36.d0*GPL2(0,2)
     +
      col4 = col4 - 1.d0/12.d0*GPL2(0,2)*pi*ci
     +  + 5.d0/24.d0*GPL2(0,3)*pi*ci
     +  + 1.d0/24.d0*GPL2(1,0)*pi*ci
     +  + 1.d0/6.d0*GPL2(1,1)*pi*ci
     +  - 29.d0/36.d0*GPL2(1,2)
     +  - 1.d0/8.d0*GPL2(1,2)*pi*ci
     +  + 29.d0/36.d0*GPL2(2,0)
     +  - 1.d0/12.d0*GPL2(2,0)*pi*ci
     +
      col4 = col4 + 5.d0/24.d0*GPL2(2,1)*pi*ci
     +  - 1.d0/4.d0*GPL2(2,2)*pi*ci
     +  + 1.d0/24.d0*GPL2(2,3)*pi*ci
     +  - 29.d0/36.d0*GPL2(3,0)
     +  - 1.d0/8.d0*GPL2(3,0)*pi*ci
     +  + 1.d0/24.d0*GPL2(3,2)*pi*ci
     +  + 1.d0/6.d0*GPL2(3,3)*pi*ci
     +  - 1.d0/4.d0*GPL3(0,0,2)
     +
      col4 = col4 + 1.d0/24.d0*GPL3(0,1,2)
     +  - 1.d0/4.d0*GPL3(0,2,0)
     +  - 1.d0/4.d0*GPL3(0,2,2)
     +  + 5.d0/24.d0*GPL3(0,3,0)
     +  + 1.d0/24.d0*GPL3(1,0,2)
     +  + 1.d0/6.d0*GPL3(1,1,2)
     +  + 1.d0/24.d0*GPL3(1,2,0)
     +  + 1.d0/4.d0*GPL3(1,2,2)
     +
      col4 = col4 - 1.d0/4.d0*GPL3(2,0,0)
     +  - 1.d0/4.d0*GPL3(2,0,2)
     +  + 5.d0/24.d0*GPL3(2,1,2)
     +  - 1.d0/4.d0*GPL3(2,2,0)
     +  + 1.d0/24.d0*GPL3(2,3,0)
     +  + 1.d0/4.d0*GPL3(3,0,0)
     +  + 1.d0/24.d0*GPL3(3,0,2)
     +  + 1.d0/24.d0*GPL3(3,2,0)
     +
      col4 = col4 + 1.d0/6.d0*GPL3(3,3,0)
     +

      col5 =
     +  + 8785.d0/864.d0
     +  - 1.d0/2.d0*pi*u**(-2)*v*ci
     +  + 1.d0/2.d0*pi*u**(-2)*ci
     +  - 1.d0/12.d0*pi*u**(-1)*v*ci
     +  - 1.d0/2.d0*pi*u**(-1)*ci
     +  - 1.d0/12.d0*pi*v*omu**(-1)*ci
     +  - 1.d0/4.d0*pi*omumv**(-1)*ci
     +
      col5 = col5 + 1147.d0/216.d0*pi*ci
     +  - 1.d0/24.d0*pi**2*omumv**(-2)
     +  + 1.d0/12.d0*pi**2*omumv**(-1)
     +  - 647.d0/864.d0*pi**2
     +  - 1.d0/18.d0*pi**3*ci
     +  + 61.d0/36.d0*zeta3
     +  + 37.d0/36.d0*u**(-1)
     +  - 13.d0/12.d0*omv**(-1)
     +
      col5 = col5 + 1379.d0/216.d0*HPL1(0)
     +  - 1.d0/4.d0*HPL1(0)*pi*omumv**(-2)*ci
     +  + 1.d0/2.d0*HPL1(0)*pi*omumv**(-1)*ci
     +  + 215.d0/72.d0*HPL1(0)*pi*ci
     +  - 1.d0/6.d0*HPL1(0)*pi**2
     +  + 1.d0/2.d0*HPL1(0)*u**(-2)*v
     +  + 1.d0/12.d0*HPL1(0)*u**(-1)*v
     +  - 1.d0/4.d0*HPL1(0)*u**(-1)*omv**(-1)
     +
      col5 = col5 + 11.d0/24.d0*HPL1(0)*u**(-1)
     +  + 1.d0/12.d0*HPL1(0)*v*omu**(-1)
     +  - 13.d0/12.d0*HPL1(0)*omv**(-2)
     +  - 1.d0/4.d0*HPL1(0)*omv**(-1)*omumv**(-1)
     +  + 7.d0/3.d0*HPL1(0)*omv**(-1)
     +  + 1.d0/4.d0*HPL1(0)*omumv**(-1)
     +  - 25.d0/48.d0*HPL1(0)*GPL1(0)
     +  + 1.d0/8.d0*HPL1(0)*GPL1(0)*pi*ci
     +
      col5 = col5 - 1.d0/4.d0*HPL1(0)*GPL1(0)*omumv**(-2)
     +  + 1.d0/2.d0*HPL1(0)*GPL1(0)*omumv**(-1)
     +  - 5.d0/9.d0*HPL1(0)*GPL1(1)
     +  + 1.d0/12.d0*HPL1(0)*GPL1(1)*pi*ci
     +  + 1.d0/2.d0*HPL1(0)*GPL1(1)*u**(-3)*v
     +  - 1.d0/2.d0*HPL1(0)*GPL1(1)*u**(-3)
     +  - 1.d0/6.d0*HPL1(0)*GPL1(1)*u**(-2)*v
     +  + 5.d0/6.d0*HPL1(0)*GPL1(1)*u**(-2)
     +
      col5 = col5 - 7.d0/12.d0*HPL1(0)*GPL1(1)*u**(-1)
     +  - 25.d0/48.d0*HPL1(0)*GPL1(2)
     +  + 5.d0/8.d0*HPL1(0)*GPL1(2)*pi*ci
     +  - 1.d0/2.d0*HPL1(0)*GPL1(2)*u**(-3)*v
     +  + 1.d0/2.d0*HPL1(0)*GPL1(2)*u**(-3)
     +  + 1.d0/6.d0*HPL1(0)*GPL1(2)*u**(-2)*v
     +  - 5.d0/6.d0*HPL1(0)*GPL1(2)*u**(-2)
     +  + 7.d0/12.d0*HPL1(0)*GPL1(2)*u**(-1)
     +
      col5 = col5 - 5.d0/9.d0*HPL1(0)*GPL1(3)
     +  - 1.d0/12.d0*HPL1(0)*GPL1(3)*pi*ci
     +  + 1.d0/4.d0*HPL1(0)*GPL1(3)*omumv**(-2)
     +  - 1.d0/2.d0*HPL1(0)*GPL1(3)*omumv**(-1)
     +  - 1.d0/4.d0*HPL1(0)*GPL2(0,0)
     +  - 7.d0/24.d0*HPL1(0)*GPL2(0,1)
     +  + 1.d0/4.d0*HPL1(0)*GPL2(0,2)
     +  + 5.d0/24.d0*HPL1(0)*GPL2(0,3)
     +
      col5 = col5 + 1.d0/24.d0*HPL1(0)*GPL2(1,0)
     +  + 1.d0/6.d0*HPL1(0)*GPL2(1,1)
     +  + 7.d0/24.d0*HPL1(0)*GPL2(1,2)
     +  + 1.d0/4.d0*HPL1(0)*GPL2(2,0)
     +  + 5.d0/24.d0*HPL1(0)*GPL2(2,1)
     +  - 1.d0/4.d0*HPL1(0)*GPL2(2,2)
     +  - 7.d0/24.d0*HPL1(0)*GPL2(2,3)
     +  + 7.d0/24.d0*HPL1(0)*GPL2(3,0)
     +
      col5 = col5 + 1.d0/24.d0*HPL1(0)*GPL2(3,2)
     +  + 1.d0/6.d0*HPL1(0)*GPL2(3,3)
     +  + 251.d0/216.d0*HPL1(1)
     +  + 53.d0/144.d0*HPL1(1)*pi*ci
     +  - 1.d0/18.d0*HPL1(1)*pi**2
     +  + 1.d0/2.d0*HPL1(1)*u**(-2)*v
     +  - 1.d0/2.d0*HPL1(1)*u**(-2)
     +  + 1.d0/12.d0*HPL1(1)*u**(-1)*v
     +
      col5 = col5 + 29.d0/48.d0*HPL1(1)*u**(-1)
     +  + 1.d0/12.d0*HPL1(1)*v*omu**(-1)
     +  - 5.d0/36.d0*HPL1(1)*GPL1(0)
     +  - 5.d0/9.d0*HPL1(1)*GPL1(1)
     +  - 1.d0/8.d0*HPL1(1)*GPL1(1)*pi*ci
     +  + 1.d0/2.d0*HPL1(1)*GPL1(1)*u**(-3)*v
     +  - 1.d0/2.d0*HPL1(1)*GPL1(1)*u**(-3)
     +  - 1.d0/6.d0*HPL1(1)*GPL1(1)*u**(-2)*v
     +
      col5 = col5 + 5.d0/6.d0*HPL1(1)*GPL1(1)*u**(-2)
     +  - 7.d0/12.d0*HPL1(1)*GPL1(1)*u**(-1)
     +  - 41.d0/72.d0*HPL1(1)*GPL1(2)
     +  + 1.d0/24.d0*HPL1(1)*GPL1(3)*pi*ci
     +  - 7.d0/24.d0*HPL1(1)*GPL2(0,1)
     +  + 1.d0/24.d0*HPL1(1)*GPL2(1,0)
     +  + 1.d0/6.d0*HPL1(1)*GPL2(1,1)
     +  + 1.d0/4.d0*HPL1(1)*GPL2(1,2)
     +
      col5 = col5 + 5.d0/24.d0*HPL1(1)*GPL2(2,1)
     +  + 1.d0/24.d0*HPL1(1)*GPL2(3,0)
     +  + 145.d0/36.d0*HPL2(0,0)
     +  + 1.d0/3.d0*HPL2(0,0)*pi*ci
     +  + 1.d0/4.d0*HPL2(0,0)*omumv**(-2)
     +  - 1.d0/2.d0*HPL2(0,0)*omumv**(-1)
     +  + 17.d0/24.d0*HPL2(0,0)*GPL1(0)
     +  - 1.d0/2.d0*HPL2(0,0)*GPL1(1)
     +
      col5 = col5 + 5.d0/24.d0*HPL2(0,0)*GPL1(2)
     +  - 1.d0/3.d0*HPL2(0,0)*GPL1(3)
     +  + 155.d0/144.d0*HPL2(0,1)
     +  + 1.d0/24.d0*HPL2(0,1)*pi*ci
     +  + 1.d0/24.d0*HPL2(0,1)*GPL1(0)
     +  - 11.d0/24.d0*HPL2(0,1)*GPL1(1)
     +  + 1.d0/24.d0*HPL2(0,1)*GPL1(2)
     +  - 1.d0/24.d0*HPL2(0,1)*GPL1(3)
     +
      col5 = col5 + 235.d0/144.d0*HPL2(1,0)
     +  + 1.d0/24.d0*HPL2(1,0)*pi*ci
     +  - 1.d0/4.d0*HPL2(1,0)*omumv**(-2)
     +  + 1.d0/2.d0*HPL2(1,0)*omumv**(-1)
     +  - 1.d0/3.d0*HPL2(1,0)*GPL1(0)
     +  - 7.d0/24.d0*HPL2(1,0)*GPL1(1)
     +  + 1.d0/6.d0*HPL2(1,0)*GPL1(2)
     +  - 1.d0/24.d0*HPL2(1,0)*GPL1(3)
     +
      col5 = col5 + 41.d0/72.d0*HPL2(1,1)
     +  - 1.d0/4.d0*HPL2(1,1)*GPL1(1)
     +  - 3.d0/4.d0*HPL3(0,0,0)
     +  - 1.d0/24.d0*HPL3(0,0,1)
     +  + 3.d0/8.d0*HPL3(0,1,0)
     +  + 19.d0/24.d0*HPL3(1,0,0)
     +  + 1.d0/24.d0*HPL3(1,0,1)
     +  - 5.d0/12.d0*HPL3(1,1,0)
     +
      col5 = col5 - 251.d0/216.d0*GPL1(0)
     +  - 53.d0/144.d0*GPL1(0)*pi*ci
     +  - 1.d0/12.d0*GPL1(0)*pi**2
     +  - 5.d0/48.d0*GPL1(0)*u**(-1)
     +  - 1.d0/4.d0*GPL1(0)*omumv**(-1)
     +  - 1.d0/2.d0*GPL1(1)*pi*u**(-3)*v*ci
     +  + 1.d0/2.d0*GPL1(1)*pi*u**(-3)*ci
     +  + 1.d0/6.d0*GPL1(1)*pi*u**(-2)*v*ci
     +
      col5 = col5 - 5.d0/6.d0*GPL1(1)*pi*u**(-2)*ci
     +  + 7.d0/12.d0*GPL1(1)*pi*u**(-1)*ci
     +  + 5.d0/9.d0*GPL1(1)*pi*ci
     +  - 7.d0/36.d0*GPL1(1)*pi**2
     +  - 251.d0/216.d0*GPL1(2)
     +  - 53.d0/144.d0*GPL1(2)*pi*ci
     +  - 1.d0/2.d0*GPL1(2)*u**(-2)*v
     +  + 1.d0/2.d0*GPL1(2)*u**(-2)
     +
      col5 = col5 - 1.d0/12.d0*GPL1(2)*u**(-1)*v
     +  - 29.d0/48.d0*GPL1(2)*u**(-1)
     +  - 1.d0/12.d0*GPL1(2)*v*omu**(-1)
     +  - 1.d0/4.d0*GPL1(3)*pi*omumv**(-2)*ci
     +  + 1.d0/2.d0*GPL1(3)*pi*omumv**(-1)*ci
     +  + 5.d0/9.d0*GPL1(3)*pi*ci
     +  - 2.d0/9.d0*GPL1(3)*pi**2
     +  + 41.d0/72.d0*GPL2(0,0)
     +
      col5 = col5 + 7.d0/24.d0*GPL2(0,1)*pi*ci
     +  + 5.d0/36.d0*GPL2(0,2)
     +  - 5.d0/24.d0*GPL2(0,3)*pi*ci
     +  - 1.d0/24.d0*GPL2(1,0)*pi*ci
     +  - 1.d0/6.d0*GPL2(1,1)*pi*ci
     +  + 5.d0/9.d0*GPL2(1,2)
     +  + 1.d0/8.d0*GPL2(1,2)*pi*ci
     +  - 1.d0/2.d0*GPL2(1,2)*u**(-3)*v
     +
      col5 = col5 + 1.d0/2.d0*GPL2(1,2)*u**(-3)
     +  + 1.d0/6.d0*GPL2(1,2)*u**(-2)*v
     +  - 5.d0/6.d0*GPL2(1,2)*u**(-2)
     +  + 7.d0/12.d0*GPL2(1,2)*u**(-1)
     +  + 5.d0/36.d0*GPL2(2,0)
     +  - 5.d0/24.d0*GPL2(2,1)*pi*ci
     +  + 41.d0/72.d0*GPL2(2,2)
     +  + 7.d0/24.d0*GPL2(2,3)*pi*ci
     +
      col5 = col5 + 5.d0/9.d0*GPL2(3,0)
     +  + 1.d0/8.d0*GPL2(3,0)*pi*ci
     +  - 1.d0/4.d0*GPL2(3,0)*omumv**(-2)
     +  + 1.d0/2.d0*GPL2(3,0)*omumv**(-1)
     +  - 1.d0/24.d0*GPL2(3,2)*pi*ci
     +  - 1.d0/6.d0*GPL2(3,3)*pi*ci
     +  + 7.d0/24.d0*GPL3(0,1,2)
     +  - 5.d0/24.d0*GPL3(0,3,0)
     +
      col5 = col5 - 1.d0/24.d0*GPL3(1,0,2)
     +  - 1.d0/6.d0*GPL3(1,1,2)
     +  - 1.d0/24.d0*GPL3(1,2,0)
     +  - 1.d0/4.d0*GPL3(1,2,2)
     +  - 5.d0/24.d0*GPL3(2,1,2)
     +  + 7.d0/24.d0*GPL3(2,3,0)
     +  - 1.d0/4.d0*GPL3(3,0,0)
     +  - 1.d0/24.d0*GPL3(3,0,2)
     +
      col5 = col5 - 1.d0/24.d0*GPL3(3,2,0)
     +  - 1.d0/6.d0*GPL3(3,3,0)
     +

      col6 =
     +  - 25.d0/81.d0
     +  - 10.d0/27.d0*pi*ci
     +  + 47.d0/432.d0*pi**2
     +  - 5.d0/9.d0*HPL1(0)
     +  - 5.d0/18.d0*HPL1(0)*pi*ci
     +  + 5.d0/72.d0*HPL1(0)*GPL1(0)
     +  + 5.d0/72.d0*HPL1(0)*GPL1(2)
     +
      col6 = col6 - 5.d0/54.d0*HPL1(1)
     +  - 1.d0/36.d0*HPL1(1)*pi*ci
     +  + 1.d0/144.d0*HPL1(1)*GPL1(0)
     +  + 5.d0/144.d0*HPL1(1)*GPL1(2)
     +  - 5.d0/12.d0*HPL2(0,0)
     +  - 5.d0/72.d0*HPL2(0,1)
     +  - 5.d0/72.d0*HPL2(1,0)
     +  - 5.d0/144.d0*HPL2(1,1)
     +
      col6 = col6 + 5.d0/54.d0*GPL1(0)
     +  + 1.d0/36.d0*GPL1(0)*pi*ci
     +  + 5.d0/54.d0*GPL1(2)
     +  + 1.d0/36.d0*GPL1(2)*pi*ci
     +  - 5.d0/144.d0*GPL2(0,0)
     +  - 1.d0/144.d0*GPL2(0,2)
     +  - 1.d0/144.d0*GPL2(2,0)
     +  - 5.d0/144.d0*GPL2(2,2)
     +


      endif

      col(1) = col1*(-1.d0)
      col(2) = col2*(-1.d0)
      col(3) = col3*(-1.d0)
      col(4) = col4*(-1.d0)
      col(5) = col5*(-1.d0)
      col(6) = col6*(-1.d0)     !To be consistent with gamma_0=1

      end subroutine gamma1a2L



      
      end module h3pC_Coeff
