************************************************************************
!XC 1-loop amplitudes based on MCFM-6.8 source code 
!  J.~M.~Campbell and R.~K.~Ellis,
!  %``An Update on vector boson pair production at hadron colliders,''
!  Phys.\ Rev.\ D {\bf 60} (1999) 113006
!  doi:10.1103/PhysRevD.60.113006
!  [hep-ph/9905386].

!  J.~M.~Campbell, R.~K.~Ellis and W.~T.~Giele,
!  %``A Multi-Threaded Version of MCFM,''
!  Eur.\ Phys.\ J.\ C {\bf 75} (2015) no.6,  246
!  doi:10.1140/epjc/s10052-015-3461-2
!  [arXiv:1503.06182 [physics.comp-ph]].

!XC The corresponding reference for 1-loop amplitudes are 
!  S.~Badger, J.~M.~Campbell, R.~K.~Ellis and C.~Williams,
!  %``Analytic results for the one-loop NMHV Hqqgg amplitude,''
!  JHEP {\bf 0912} (2009) 035
!  doi:10.1088/1126-6708/2009/12/035
!  [arXiv:0910.4481 [hep-ph]].

!  S.~Badger, E.~W.~Nigel Glover, P.~Mastrolia and C.~Williams,
!  %``One-loop Higgs plus four gluon amplitudes: Full analytic results,''
!  JHEP {\bf 1001} (2010) 036
!  doi:10.1007/JHEP01(2010)036
!  [arXiv:0909.4475 [hep-ph]].

!  L.~J.~Dixon and Y.~Sofianatos,
!  %``Analytic one-loop amplitudes for a Higgs boson plus four partons,''
!  JHEP {\bf 0908} (2009) 058
!  doi:10.1088/1126-6708/2009/08/058
!  [arXiv:0906.0008 [hep-ph]].

!  E.~W.~N.~Glover, P.~Mastrolia and C.~Williams,
!  %``One-loop phi-MHV amplitudes using the unitarity bootstrap: The General helicity case,''
!  JHEP {\bf 0808} (2008) 017
!  doi:10.1088/1126-6708/2008/08/017
!  [arXiv:0804.4149 [hep-ph]].

!  S.~D.~Badger, E.~W.~N.~Glover and K.~Risager,
!  %``One-loop phi-MHV amplitudes using the unitarity bootstrap,''
!  JHEP {\bf 0707} (2007) 066
!  doi:10.1088/1126-6708/2007/07/066
!  [arXiv:0704.3914 [hep-ph]].

!XC To improve numerical stability, more details in this code are explained in
!  X.~Chen,
!  %``Hadronic Production of a Higgs Boson in Association with a Jet at Next-to-Next-to-Leading Order,''
!  http://etheses.dur.ac.uk/11158/1/PhDthesisXuanChen.pdf 
************************************************************************
*
* two gluon one higgs zero quark 1-loop level matrix elements
**
* summed over colour orderings, summed and squared
*
************************************************************************
      function FullA2g1H(i1,i2,i3,i4,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /order/ieorder

      stop "set value for chgg correctly!"
      fac = (4d0*pi*as*nc)**1*(nc**2d0-1d0)*2d0*chgg**2/4d0

      FullA2g1H=
     .         +A2g1H(i1,i2,i3,i4,renscale2)
     .         +Ah2g1H(i1,i2,i3,i4,renscale2)*dfloat(nf)/nc

      FullA2g1H=fac*FullA2g1H
      return
      end
***********************************************************
      function A2g1H(i1,i2,i3,i4,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      common /order/ieorder
      double precision logg,lns,s,t,u,xlf,subuv,Delta
     . xn,epivn,pisq,V,cf
      common /npars/npars,nspec,n

      if(nspec.eq.2)then
      s   = s4(i1,i2)
      elseif(nspec.eq.3)then
      s   = s5(i1,i2)
      elseif(nspec.eq.4)then
      s   = s6(i1,i2)
      else
        write(*,*)"A2g1H spectator unknown"
      endif

      pisq=pi**2
      lns=dlog(s/renscale2)
      logg=A2g0H(i1,i2,i3,i4)

      if(ieorder.eq.-2)then
        A2g1H=-2d0*logg
      elseif(ieorder.eq.-1)then
        A2g1H=
     .  +2d0*logg*(lns-11d0/6d0)

      elseif(ieorder.eq.0)then

        A2g1H=
     .  +2d0*logg
     .  *(-0.5d0*lns**2
     .  +1d0/2d0*pisq)
     .  +2d0*logg
     .  *(pisq*1/12d0)
*XC c_gamma fix up
      endif

      return
      end
***********************************************************
      function Ah2g1H(i1,i2,i3,i4,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      common /order/ieorder
      double precision logg

      logg=A2g0H(i1,i2,i3,i4)

      if(ieorder.eq.-2)then
        Ah2g1H=0d0

      elseif(ieorder.eq.-1)then
        Ah2g1H=2d0/3d0*logg

      elseif(ieorder.eq.0)then
        Ah2g1H=0d0

      endif

      return
      end
************************************************************************
*
* three gluon one higgs zero quark 1-loop level matrix elements
**
* summed over colour orderings, summed and squared
*
************************************************************************
      function FullA3g1H(i1,i2,i3,i4,i5,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /order/ieorder

      stop "set value for chgg correctly!"
      fac = (4d0*pi*as*nc)**2*(nc**2d0-1d0)*2d0*chgg**2/4d0

      FullA3g1H=
     .         +A3g1H(i1,i2,i3,i4,i5,renscale2)
     .         +Ah3g1H(i1,i2,i3,i4,i5,renscale2)*dfloat(nf)/nc

      FullA3g1H=fac*FullA3g1H
      return
      end
***********************************************************
* MM: valid in arbitrary kinematics, taken from 9707448
      function A3g1H(i1,i2,i3,i4,i5,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /order/ieorder
      common /npars/npars,nspec,n
      real*8 mhsq
      real*8, pointer :: s12,s13,s23
      integer ipset

      ipset = nspec+3
      s12 => kin(ipset)%s(i1,i2)
      s13 => kin(ipset)%s(i1,i3)
      s23 => kin(ipset)%s(i2,i3)

      mhsq=s12+s13+s23

      tree = A3g0H(i1,i2,i3,i4,i5)

      zlogs12 = zzlog(-mhsq) - zzlog(-s12)
      zlogs13 = zzlog(-mhsq) - zzlog(-s13)
      zlogs23 = zzlog(-mhsq) - zzlog(-s23)
      zlogmhsq = zzlog(renscale2) - zzlog(-mhsq)

      if(ieorder.eq.-2)then
        A3g1H=-3d0*tree

      elseif(ieorder.eq.-1)then
        A3g1H= tree*dreal(
     .         -zlogs12
     .         -zlogs13
     .         -zlogs23
     .         -3d0*zlogmhsq
     .         -3d0*11d0/6d0  !renormalization
     .         )

      elseif(ieorder.eq.0)then

      !Using complete dli2 function in src/specials/specials.f for arbitrary argument
      zdilog12 = dli2(1d0-s12/mhsq)
      zdilog13 = dli2(1d0-s13/mhsq)
      zdilog23 = dli2(1d0-s23/mhsq)

      A3g1H=

     .  +tree*dreal(
     .         -2d0*(zdilog12+zdilog13+zdilog23)
     .         -1d0/2d0*zlogs12**2
     .         -1d0/2d0*zlogs13**2
     .         -1d0/2d0*zlogs23**2
     .         -3d0/2d0*zlogmhsq**2
     .         -zlogs12*zlogs13
     .         -zlogs13*zlogs23
     .         -zlogs23*zlogs12
     .         -zlogs12*zlogmhsq
     .         -zlogs13*zlogmhsq
     .         -zlogs23*zlogmhsq
     .         +1d0/2d0*pi**2
     .      )
     .   +2d0/3d0*mhsq*(1d0+mhsq/s12+mhsq/s13+mhsq/s23)
     .   +tree*pi**2/4d0 !r_gamma

      endif

      return
      end
***********************************************************
* MM: valid in arbitrary kinematics, taken from 9707448
      function Ah3g1H(i1,i2,i3,i4,i5,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      parameter(pi=3.141592653589793238d0)
      common /order/ieorder
      common /npars/npars,nspec,n
      double precision mhsq
      real*8, pointer :: s12,s13,s23
      integer ipset

      ipset = nspec+3
      s12 => kin(ipset)%s(i1,i2)
      s13 => kin(ipset)%s(i1,i3)
      s23 => kin(ipset)%s(i2,i3)

      mhsq=s12+s13+s23

      tree=A3g0H(i1,i2,i3,i4,i5)

      if(ieorder.eq.-2)then
        Ah3g1H=0d0

      elseif(ieorder.eq.-1)then
        Ah3g1H=+tree  != 3d0/3d0*tree

      elseif(ieorder.eq.0)then
        Ah3g1H=
     .  -2d0/3d0*mhsq*(1d0+mhsq/s12+mhsq/s13+mhsq/s23)
      endif

      return
      end
************************************************************************
*
* four gluon one higgs zero quark 1-loop level matrix elements
**
* summed over colour orderings, summed and squared
*
*
* Using the result in MCFM for different helicity configuration
*
*************************************************************************
      function FullA4g1H(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /order/ieorder

      stop "set value for chgg correctly!"
      fac=(4d0*pi*as*nc)**3*(nc**2-1d0)*2d0*chgg**2/4d0


      FullA4g1H=
     .         (
     .          +A4g1H(i1,i2,i3,i4,i5,i6,renscale2)
     .          +A4g1H(i1,i2,i4,i3,i5,i6,renscale2)
     .          +A4g1H(i1,i4,i2,i3,i5,i6,renscale2)

     .          +Ah4g1H(i1,i2,i3,i4,i5,i6,renscale2)*dfloat(nf)/nc
     .          +Ah4g1H(i1,i2,i4,i3,i5,i6,renscale2)*dfloat(nf)/nc
     .          +Ah4g1H(i1,i4,i2,i3,i5,i6,renscale2)*dfloat(nf)/nc
     .         )

      FullA4g1H=fac*FullA4g1H
      return
      end
*************************************************************************
      function FullA4g1HCL(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /order/ieorder

      stop "set value for chgg correctly!"
      fac=(4d0*pi*as*nc)**3*(nc**2-1d0)*2d0*chgg**2/4d0

      FullA4g1HCL=
     .         (
     .          +A4g1H(i1,i2,i3,i4,i5,i6,renscale2)
     .          +A4g1H(i1,i2,i4,i3,i5,i6,renscale2)
     .          +A4g1H(i1,i4,i2,i3,i5,i6,renscale2)
     .         )

      FullA4g1HCL=fac*FullA4g1HCL
      return
      end
************************************************************************
      function A4g1H(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /order/ieorder
      integer NHEL(4,16),PERMa(6),PERMb(6),PERMc(6),PERMd(6)

      DATA (NHEL(IHEL,   1),IHEL=1, 4) /  1, 1, 1, 1/!
      DATA (NHEL(IHEL,   2),IHEL=1, 4) / -1,-1,-1,-1/

      DATA (NHEL(IHEL,   3),IHEL=1, 4) /  1,-1,-1,-1/
      DATA (NHEL(IHEL,   4),IHEL=1, 4) / -1, 1,-1,-1/
      DATA (NHEL(IHEL,   5),IHEL=1, 4) / -1,-1, 1,-1/
      DATA (NHEL(IHEL,   6),IHEL=1, 4) / -1,-1,-1, 1/

      DATA (NHEL(IHEL,   7),IHEL=1, 4) / -1, 1, 1, 1/!
      DATA (NHEL(IHEL,   8),IHEL=1, 4) /  1,-1, 1, 1/!
      DATA (NHEL(IHEL,   9),IHEL=1, 4) /  1, 1,-1, 1/!
      DATA (NHEL(IHEL,  10),IHEL=1, 4) /  1, 1, 1,-1/!

      DATA (NHEL(IHEL,  11),IHEL=1, 4) / -1,-1, 1, 1/
      DATA (NHEL(IHEL,  12),IHEL=1, 4) /  1,-1,-1, 1/
      DATA (NHEL(IHEL,  13),IHEL=1, 4) /  1, 1,-1,-1/
      DATA (NHEL(IHEL,  14),IHEL=1, 4) / -1, 1, 1,-1/

      DATA (NHEL(IHEL,  15),IHEL=1, 4) / -1, 1,-1, 1/
      DATA (NHEL(IHEL,  16),IHEL=1, 4) /  1,-1, 1,-1/

        perma(1)=i1
        perma(2)=i2
        perma(3)=i3
        perma(4)=i4
        perma(5)=i5
        perma(6)=i6

        permb(1)=i2
        permb(2)=i3
        permb(3)=i4
        permb(4)=i1
        permb(5)=i5
        permb(6)=i6

        permc(1)=i3
        permc(2)=i4
        permc(3)=i1
        permc(4)=i2
        permc(5)=i5
        permc(6)=i6

        permd(1)=i4
        permd(2)=i1
        permd(3)=i2
        permd(4)=i3
        permd(5)=i5
        permd(6)=i6

       zamp=0d0
       temp=0d0
       do i=1,16
           if(i.eq.1) then
            zamp=dble(zA4g1Hppppcl(perma(1),renscale2)*zA4g0Hmmmm(perma(1)))
           endif
           if(i.eq.2) then
            zamp=dble(zA4g1Hmmmmcl(perma(1),renscale2)*Dconjg(zA4g0Hmmmm(perma(1))))
           endif
           if(i.eq.3) then
            zamp=dble(zA4g1Hpmmmcl(perma(1),renscale2)*Dconjg(zA4g0Hpmmm(perma(1))))
           endif
           if(i.eq.4) then
            zamp=dble(zA4g1Hpmmmcl(permb(1),renscale2)*Dconjg(zA4g0Hpmmm(permb(1))))
           endif
           if(i.eq.5) then
            zamp=dble(zA4g1Hpmmmcl(permc(1),renscale2)*Dconjg(zA4g0Hpmmm(permc(1))))
           endif
           if(i.eq.6) then
            zamp=dble(zA4g1Hpmmmcl(permd(1),renscale2)*Dconjg(zA4g0Hpmmm(permd(1))))
           endif
           if(i.eq.7) then
            zamp=dble(zA4g1Hmpppcl(perma(1),renscale2)*zA4g0Hpmmm(perma(1)))
           endif
           if(i.eq.8) then
            zamp=dble(zA4g1Hmpppcl(permb(1),renscale2)*zA4g0Hpmmm(permb(1)))
           endif
           if(i.eq.9) then
            zamp=dble(zA4g1Hmpppcl(permc(1),renscale2)*zA4g0Hpmmm(permc(1)))
           endif
           if(i.eq.10) then
            zamp=dble(zA4g1Hmpppcl(permd(1),renscale2)*zA4g0Hpmmm(permd(1)))
           endif
           if(i.eq.11) then
            zamp=dble(zA4g1Hmmppcl(perma(1),renscale2)*Dconjg(zA4g0Hmmpp(perma(1))))
           endif
           if(i.eq.12) then
            zamp=dble(zA4g1Hmmppcl(permd(1),renscale2)*Dconjg(zA4g0Hmmpp(permd(1))))
           endif
           if(i.eq.13) then
            zamp=dble(zA4g1Hmmppcl(permc(1),renscale2)*Dconjg(zA4g0Hmmpp(permc(1))))
           endif
           if(i.eq.14) then
            zamp=dble(zA4g1Hmmppcl(permb(1),renscale2)*Dconjg(zA4g0Hmmpp(permb(1))))
           endif
           if(i.eq.15) then
            zamp=dble(zA4g1Hmpmpcl(perma(1),renscale2)*Dconjg(zA4g0Hmpmp(perma(1))))
           endif
           if(i.eq.16) then
            zamp=dble(zA4g1Hmpmpcl(permb(1),renscale2)*Dconjg(zA4g0Hmpmp(permb(1))))
           endif
          temp=temp+zamp
       enddo

      if(ieorder.eq.0)then
        ren=4d0*pi**2/12d0
      elseif(ieorder.eq.-1)then
        ren=-4d0*11d0/6d0
      else
        ren=0d0
      endif

         A4g1H=temp+A4g0H(i1,i2,i3,i4,i5,i6)*ren
*XC add c_gamma fix up

   10 format(" Ah5g0   ",e12.4,6i3)
      return
      end
************************************************************************
      function Ah4g1H(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /koppel/ nf,nc,b0
      common /order/ieorder

      integer NHEL(4,16),PERMa(6),PERMb(6),PERMc(6),PERMd(6)

      DATA (NHEL(IHEL,   1),IHEL=1, 4) /  1, 1, 1, 1/!
      DATA (NHEL(IHEL,   2),IHEL=1, 4) / -1,-1,-1,-1/

      DATA (NHEL(IHEL,   3),IHEL=1, 4) /  1,-1,-1,-1/
      DATA (NHEL(IHEL,   4),IHEL=1, 4) / -1, 1,-1,-1/
      DATA (NHEL(IHEL,   5),IHEL=1, 4) / -1,-1, 1,-1/
      DATA (NHEL(IHEL,   6),IHEL=1, 4) / -1,-1,-1, 1/

      DATA (NHEL(IHEL,   7),IHEL=1, 4) / -1, 1, 1, 1/!
      DATA (NHEL(IHEL,   8),IHEL=1, 4) /  1,-1, 1, 1/!
      DATA (NHEL(IHEL,   9),IHEL=1, 4) /  1, 1,-1, 1/!
      DATA (NHEL(IHEL,  10),IHEL=1, 4) /  1, 1, 1,-1/!

      DATA (NHEL(IHEL,  11),IHEL=1, 4) / -1,-1, 1, 1/
      DATA (NHEL(IHEL,  12),IHEL=1, 4) /  1,-1,-1, 1/
      DATA (NHEL(IHEL,  13),IHEL=1, 4) /  1, 1,-1,-1/
      DATA (NHEL(IHEL,  14),IHEL=1, 4) / -1, 1, 1,-1/

      DATA (NHEL(IHEL,  15),IHEL=1, 4) / -1, 1,-1, 1/
      DATA (NHEL(IHEL,  16),IHEL=1, 4) /  1,-1, 1,-1/

        perma(1)=i1
        perma(2)=i2
        perma(3)=i3
        perma(4)=i4
        perma(5)=i5
        perma(6)=i6

        permb(1)=i2
        permb(2)=i3
        permb(3)=i4
        permb(4)=i1
        permb(5)=i5
        permb(6)=i6

        permc(1)=i3
        permc(2)=i4
        permc(3)=i1
        permc(4)=i2
        permc(5)=i5
        permc(6)=i6

        permd(1)=i4
        permd(2)=i1
        permd(3)=i2
        permd(4)=i3
        permd(5)=i5
        permd(6)=i6

       zamp=0d0
       temp=0d0
       do i=1,16
           if(i.eq.1) then
            zamp=dble(zA4g1Hppppnf(perma(1))*zA4g0Hmmmm(perma(1)))
           endif
           if(i.eq.2) then
            zamp=dble(zA4g1Hmmmmnf(perma(1))*Dconjg(zA4g0Hmmmm(perma(1))))
           endif
           if(i.eq.3) then
            zamp=dble(zA4g1Hpmmmnf(perma(1))*Dconjg(zA4g0Hpmmm(perma(1))))
           endif
           if(i.eq.4) then
            zamp=dble(zA4g1Hpmmmnf(permb(1))*Dconjg(zA4g0Hpmmm(permb(1))))
           endif
           if(i.eq.5) then
            zamp=dble(zA4g1Hpmmmnf(permc(1))*Dconjg(zA4g0Hpmmm(permc(1))))
           endif
           if(i.eq.6) then
            zamp=dble(zA4g1Hpmmmnf(permd(1))*Dconjg(zA4g0Hpmmm(permd(1))))
           endif
           if(i.eq.7) then
            zamp=dble(zA4g1Hmpppnf(perma(1))*zA4g0Hpmmm(perma(1)))
           endif
           if(i.eq.8) then
            zamp=dble(zA4g1Hmpppnf(permb(1))*zA4g0Hpmmm(permb(1)))
           endif
           if(i.eq.9) then
            zamp=dble(zA4g1Hmpppnf(permc(1))*zA4g0Hpmmm(permc(1)))
           endif
           if(i.eq.10) then
            zamp=dble(zA4g1Hmpppnf(permd(1))*zA4g0Hpmmm(permd(1)))
           endif
           if(i.eq.11) then
            zamp=dble(zA4g1Hmmppnf(perma(1),renscale2)*Dconjg(zA4g0Hmmpp(perma(1))))
           endif
           if(i.eq.12) then
            zamp=dble(zA4g1Hmmppnf(permd(1),renscale2)*Dconjg(zA4g0Hmmpp(permd(1))))
           endif
           if(i.eq.13) then
            zamp=dble(zA4g1Hmmppnf(permc(1),renscale2)*Dconjg(zA4g0Hmmpp(permc(1))))
           endif
           if(i.eq.14) then
            zamp=dble(zA4g1Hmmppnf(permb(1),renscale2)*Dconjg(zA4g0Hmmpp(permb(1))))
           endif
           if(i.eq.15) then
            zamp=dble(zA4g1Hmpmpnf(perma(1),renscale2)*Dconjg(zA4g0Hmpmp(perma(1))))
           endif
           if(i.eq.16) then
            zamp=dble(zA4g1Hmpmpnf(permb(1),renscale2)*Dconjg(zA4g0Hmpmp(permb(1))))
           endif
          temp=temp+zamp
       enddo
         A4g1Hnf=temp

      if(ieorder.eq.-1)then
        ren=4d0/3d0
      else
        ren=0d0
      endif

         Ah4g1H=temp+A4g0H(i1,i2,i3,i4,i5,i6)*ren

   10 format(" Ah5g0   ",e12.4,6i3)
      return
      end
*****************************************************************************************
*****************************************************************************************
****************************** Library for 1-loop amplitudes ****************************
*****************************************************************************************
*****************************************************************************************
      complex*16 function zA4g1Hmmmmcl(PERM,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer  perm(6)
      zA4g1Hmmmmcl=zA4g1HmmmmCC(perm(1),renscale2)
     .          +zA4g1HmmmmNCCcl(perm(1))

      return
      end
*****************************************************************************************
      complex*16 function zA4g1Hmmmmnf(PERM)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer  perm(6)
      zA4g1Hmmmmnf=
     .          +zA4g1HmmmmNCCnf(perm(1))

      return
      end
*****************************************************************************************
      complex*16 function zA4g1HmmmmCC(PERM,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /order/ieorder
      integer j1,j2,j3,j4,j5,i,ii(7),perm(6)
      double complex F31m,F42me,F41m
      double precision sc(4,4)
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      integer ipset
      data ii/1,2,3,4,1,2,3/
      save ii

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

cw--- set up 's-comma' products
      sc(1,1)=0d0
      sc(1,2)=s12
      sc(1,3)=s12+s13+s23 !t123
      sc(1,4)=s12+s13+s14+s23+s24+s34
               !t1234
      sc(2,1)=sc(1,4) !s(j1,j2)+s(j1,j3)+s(j1,j4)+s(j2,j3)+s(j2,j4)+s(j3,j4)
      sc(2,2)=0d0
      sc(2,3)=s23
      sc(2,4)=s23+s24+s34 !t234

      sc(3,1)=s34+s31+s41 !t341
      sc(3,2)=sc(1,4) !s(j1,j2)+s(j1,j3)+s(j1,j4)+s(j2,j3)+s(j2,j4)+s(j3,j4)
      sc(3,3)=0d0
      sc(3,4)=s34

      sc(4,1)=s41
      sc(4,2)=s41+s42+s12 !t412
      sc(4,3)=sc(1,4) !s(j1,j2)+s(j1,j3)+s(j1,j4)+s(j2,j3)+s(j2,j4)+s(j3,j4)
      sc(4,4)=0d0

      zA4g1HmmmmCC=(0d0,0d0)
      do i=1,4
cw--- NOTE: Arguments of F41m , F42m have been changed to be consistent
cw---       with later papers: F41m(s,t;Psq) -> F41m(Psq;s,t)
cw---                          F42me(s,t;Psq,Qsq) -> F42me(Psq,Qsq;s,t)

      zA4g1HmmmmCC=zA4g1HmmmmCC
     .  +F31m(sc(ii(i),ii(i+2)),renscale2)-F31m(sc(ii(i),ii(i+3)),renscale2)
     . -0.5d0*F42me(sc(ii(i),ii(i+3)),sc(ii(i+1),ii(i+2)),
     .              sc(ii(i),ii(i+2)),sc(ii(i+1),ii(i+3)),renscale2)
     . -0.5d0*F41m(sc(ii(i),ii(i+2)),sc(ii(i),ii(i+1)),
     .                               sc(ii(i+1),ii(i+2)),renscale2)
      enddo

      zA4g1HmmmmCC=zA4g0Hmmmm(perm(1))*zA4g1HmmmmCC
      return
      end
*****************************************************************************************
      complex*16 function zA4g1HmmmmNCCcl(PERM)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer PERM(6),PERMa(6),PERMb(6),PERMc(6),PERMd(6)
      common /order/ieorder

        i1=perm(1)
        i2=perm(2)
        i3=perm(3)
        i4=perm(4)
        i5=perm(5)
        i6=perm(6)

        perma(1)=i1
        perma(2)=i2
        perma(3)=i3
        perma(4)=i4
        perma(5)=i5
        perma(6)=i6

        permb(1)=i2
        permb(2)=i3
        permb(3)=i4
        permb(4)=i1
        permb(5)=i5
        permb(6)=i6

        permc(1)=i3
        permc(2)=i4
        permc(3)=i1
        permc(4)=i2
        permc(5)=i5
        permc(6)=i6

        permd(1)=i4
        permd(2)=i1
        permd(3)=i2
        permd(4)=i3
        permd(5)=i5
        permd(6)=i6


      zA4g1HmmmmNCCcl=1d0/3d0
     . *(+zA4g1Hmmmmsub(perma(1))
     .   +zA4g1Hmmmmsub(permb(1))
     .   +zA4g1Hmmmmsub(permc(1))
     .   +zA4g1Hmmmmsub(permd(1)))

      return
      end
*****************************************************************************************
      complex*16 function zA4g1HmmmmNCCnf(PERM)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer PERM(6),PERMa(6),PERMb(6),PERMc(6),PERMd(6)
      common /order/ieorder

        i1=perm(1)
        i2=perm(2)
        i3=perm(3)
        i4=perm(4)
        i5=perm(5)
        i6=perm(6)

        perma(1)=i1
        perma(2)=i2
        perma(3)=i3
        perma(4)=i4
        perma(5)=i5
        perma(6)=i6

        permb(1)=i2
        permb(2)=i3
        permb(3)=i4
        permb(4)=i1
        permb(5)=i5
        permb(6)=i6

        permc(1)=i3
        permc(2)=i4
        permc(3)=i1
        permc(4)=i2
        permc(5)=i5
        permc(6)=i6

        permd(1)=i4
        permd(2)=i1
        permd(3)=i2
        permd(4)=i3
        permd(5)=i5
        permd(6)=i6


      zA4g1HmmmmNCCnf=-1d0/3d0
     . *(+zA4g1Hmmmmsub(perma(1))
     .   +zA4g1Hmmmmsub(permb(1))
     .   +zA4g1Hmmmmsub(permc(1))
     .   +zA4g1Hmmmmsub(permd(1)))

      return
      end
*****************************************************************************************
      complex*16 function zA4g1Hmmmmsub(PERM)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /order/ieorder
      integer j1,j2,j3,j4,j5,ieorder,perm(6)
      double complex zab2
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43
      integer ipset

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za34 => kin(ipset)%zA(j3,j4)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za43 => kin(ipset)%zA(j4,j3)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb34 => kin(ipset)%zB(j3,j4)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb43 => kin(ipset)%zB(j4,j3)

      t123=s12+s23+s31
      t234=s23+s34+s24

      z2ab4132=zab2(j4,j1,j3,j2,ipset)

      if(ieorder.eq.-2)then
        zA4g1Hmmmmsub=0d0
      elseif(ieorder.eq.-1)then
        zA4g1Hmmmmsub=0d0
      elseif(ieorder.eq.0)then
        zA4g1Hmmmmsub=-s13*z2ab4132**2
     .   /(t123*zb12**2*zb23**2)
     .   +(za34/zb12)**2
     .   +2d0*za34*za41/(zb12*zb23)
     .   +(s12*s34+t123*t234-s12**2)
     .   /(2d0*zb12*zb23*zb34*zb41)
      endif

      return
      end
*****************************************************************************************
      complex*16 function zA4g1Hppppcl(PERM,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer  perm(6)

      zA4g1Hppppcl=zA4g1HppppCC(perm(1),renscale2)
     .          +zA4g1HppppNCCcl(perm(1))
      return
      end
*****************************************************************************************
      complex*16 function zA4g1Hppppnf(PERM)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer  perm(6)

      zA4g1Hppppnf=
     .          +zA4g1HppppNCCnf(perm(1))
      return
      end
*****************************************************************************************
      complex*16 function zA4g1HppppCC(PERM,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /order/ieorder
      integer j1,j2,j3,j4,j5,i,ii(7),perm(6)
      double complex F31m,F42me,F41m
      double precision sc(4,4)
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      integer ipset
      data ii/1,2,3,4,1,2,3/
      save ii

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

cw--- set up 's-comma' products
      sc(1,1)=0d0
      sc(1,2)=s12
      sc(1,3)=s12+s13+s23 !t123
      sc(1,4)=s12+s13+s14+s23+s24+s34
               !t1234
      sc(2,1)=sc(1,4) !s(j1,j2)+s(j1,j3)+s(j1,j4)+s(j2,j3)+s(j2,j4)+s(j3,j4)
      sc(2,2)=0d0
      sc(2,3)=s23
      sc(2,4)=s23+s24+s34 !t234

      sc(3,1)=s34+s31+s41 !t341
      sc(3,2)=sc(1,4) !s(j1,j2)+s(j1,j3)+s(j1,j4)+s(j2,j3)+s(j2,j4)+s(j3,j4)
      sc(3,3)=0d0
      sc(3,4)=s34

      sc(4,1)=s41
      sc(4,2)=s41+s42+s12 !t412
      sc(4,3)=sc(1,4) !s(j1,j2)+s(j1,j3)+s(j1,j4)+s(j2,j3)+s(j2,j4)+s(j3,j4)
      sc(4,4)=0d0

      zA4g1HppppCC=(0d0,0d0)
      do i=1,4
cw--- NOTE: Arguments of F41m , F42m have been changed to be consistent
cw---       with later papers: F41m(s,t;Psq) -> F41m(Psq;s,t)
cw---                          F42me(s,t;Psq,Qsq) -> F42me(Psq,Qsq;s,t)

      zA4g1HppppCC=zA4g1HppppCC
     .  +F31m(sc(ii(i),ii(i+2)),renscale2)-F31m(sc(ii(i),ii(i+3)),renscale2)
     . -0.5d0*F42me(sc(ii(i),ii(i+3)),sc(ii(i+1),ii(i+2)),
     .              sc(ii(i),ii(i+2)),sc(ii(i+1),ii(i+3)),renscale2)
     . -0.5d0*F41m(sc(ii(i),ii(i+2)),sc(ii(i),ii(i+1)),
     .                               sc(ii(i+1),ii(i+2)),renscale2)
      enddo

      zA4g1HppppCC=conjg(zA4g0Hmmmm(perm(1)))*zA4g1HppppCC

      return
      end
*****************************************************************************************
      complex*16 function zA4g1HppppNCCcl(PERM)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer PERM(6),PERMa(6),PERMb(6),PERMc(6),PERMd(6)
      common /order/ieorder

        i1=perm(1)
        i2=perm(2)
        i3=perm(3)
        i4=perm(4)
        i5=perm(5)
        i6=perm(6)

        perma(1)=i1
        perma(2)=i2
        perma(3)=i3
        perma(4)=i4
        perma(5)=i5
        perma(6)=i6

        permb(1)=i2
        permb(2)=i3
        permb(3)=i4
        permb(4)=i1
        permb(5)=i5
        permb(6)=i6

        permc(1)=i3
        permc(2)=i4
        permc(3)=i1
        permc(4)=i2
        permc(5)=i5
        permc(6)=i6

        permd(1)=i4
        permd(2)=i1
        permd(3)=i2
        permd(4)=i3
        permd(5)=i5
        permd(6)=i6

      zA4g1HppppNCCcl=1d0/3d0
     . *(+zA4g1Hppppsub(perma(1))
     .   +zA4g1Hppppsub(permb(1))
     .   +zA4g1Hppppsub(permc(1))
     .   +zA4g1Hppppsub(permd(1)))

      return
      end
*****************************************************************************************
      complex*16 function zA4g1HppppNCCnf(PERM)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer PERM(6),PERMa(6),PERMb(6),PERMc(6),PERMd(6)
      common /order/ieorder

        i1=perm(1)
        i2=perm(2)
        i3=perm(3)
        i4=perm(4)
        i5=perm(5)
        i6=perm(6)

        perma(1)=i1
        perma(2)=i2
        perma(3)=i3
        perma(4)=i4
        perma(5)=i5
        perma(6)=i6

        permb(1)=i2
        permb(2)=i3
        permb(3)=i4
        permb(4)=i1
        permb(5)=i5
        permb(6)=i6

        permc(1)=i3
        permc(2)=i4
        permc(3)=i1
        permc(4)=i2
        permc(5)=i5
        permc(6)=i6

        permd(1)=i4
        permd(2)=i1
        permd(3)=i2
        permd(4)=i3
        permd(5)=i5
        permd(6)=i6

      zA4g1HppppNCCnf=-1d0/3d0
     . *(+zA4g1Hppppsub(perma(1))
     .   +zA4g1Hppppsub(permb(1))
     .   +zA4g1Hppppsub(permc(1))
     .   +zA4g1Hppppsub(permd(1)))

      return
      end
*****************************************************************************************
      complex*16 function zA4g1Hppppsub(PERM)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /order/ieorder
      integer j1,j2,j3,j4,j5,ieorder,perm(6)
      double complex zab2
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43
      integer ipset

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za34 => kin(ipset)%zA(j3,j4)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za43 => kin(ipset)%zA(j4,j3)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb34 => kin(ipset)%zB(j3,j4)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb43 => kin(ipset)%zB(j4,j3)

      t123=s12+s23+s31
      t234=s23+s34+s24

      z2ba4132=zba2(j4,j1,j3,j2,ipset)

      if(ieorder.eq.-2)then
        zA4g1Hppppsub=0d0
      elseif(ieorder.eq.-1)then
        zA4g1Hppppsub=0d0
      elseif(ieorder.eq.0)then
        zA4g1Hppppsub=-s13*z2ba4132**2
     .   /(t123*za12**2*za23**2)
     .   +(zb34/za12)**2
     .   +2d0*zb34*zb41/(za12*za23)
     .   +(s12*s34+t123*t234-s12**2)
     .   /(2d0*za12*za23*za34*za41)
      endif

      return
      end
************************************************************************
cw--- Results taken from an early draft of:
cw---   S.~Badger, E.~W.~N.~Glover, P.~Mastrolia and C.~Williams,
cw---   %``One-loop Higgs plus four gluon amplitudes: Full analytic results,''
cw---   arXiv:0909.4475 [hep-ph].
cw---   %%CITATION = ARXIV:0909.4475;%%
      double complex function zA4g1Hpmmmcl(PERM,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer perm(6),j1,j2,j3,j4,j5,j6
      common /order/ieorder
      double complex V4,FR4pmmmcl,FR4pmmmnf,F31m
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      integer ipset

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      V4 = -zA4g0Hpmmm(perm(1))*(
     . +F31m(s12,renscale2)+F31m(s23,renscale2)+F31m(s34,renscale2)
     . +F31m(s41,renscale2))

      zA4g1Hpmmmcl=V4-FR4pmmmcl(perm(1))

      return
      end
************************************************************************
cw--- Results taken from an early draft of:
cw---   S.~Badger, E.~W.~N.~Glover, P.~Mastrolia and C.~Williams,
cw---   %``One-loop Higgs plus four gluon amplitudes: Full analytic results,''
cw---   arXiv:0909.4475 [hep-ph].
cw---   %%CITATION = ARXIV:0909.4475;%%
      double complex function zA4g1Hpmmmnf(PERM)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer perm(6),j1,j2,j3,j4,j5,j6
      double complex V4,FR4pmmmcl,FR4pmmmnf

      zA4g1Hpmmmnf=
     .             -FR4pmmmnf(perm(1))

      return
      end
************************************************************************
      double complex function FR4pmmmcl(perm)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer perm(6),perma(6),permb(6),j1,j2,j3,j4,j5,j6
      common /order/ieorder
      double complex FR4unsymcl,FR4unsymnf

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      perma(1)=j1
      perma(2)=j2
      perma(3)=j3
      perma(4)=j4
      perma(5)=j5
      perma(6)=j6

      permb(1)=j1
      permb(2)=j4
      permb(3)=j3
      permb(4)=j2
      permb(5)=j5
      permb(6)=j6

      if(ieorder.eq.-2)then
        FR4pmmmcl=0d0
      elseif(ieorder.eq.-1)then
        FR4pmmmcl=0d0
      elseif(ieorder.eq.0)then
        FR4pmmmcl=FR4unsymcl(permb(1))
     .           +FR4unsymcl(perma(1))
      endif

      return
      end
************************************************************************
      double complex function FR4pmmmnf(perm)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer perm(6),perma(6),permb(6),j1,j2,j3,j4,j5,j6
      common /order/ieorder
      double complex FR4unsymnf

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      perma(1)=j1
      perma(2)=j2
      perma(3)=j3
      perma(4)=j4
      perma(5)=j5
      perma(6)=j6

      permb(1)=j1
      permb(2)=j4
      permb(3)=j3
      permb(4)=j2
      permb(5)=j5
      permb(6)=j6

      if(ieorder.eq.-2)then
        FR4pmmmnf=0d0
      elseif(ieorder.eq.-1)then
        FR4pmmmnf=0d0
      elseif(ieorder.eq.0)then
        FR4pmmmnf=
     .         +FR4unsymnf(permb(1))+FR4unsymnf(perma(1))
      endif

      return
      end
************************************************************************
*XC use the result of zA4g1Hpmmm but only take complex conjugation of za and zb
*XC leave other complex number invariant to get -+++ helicity configurations
      double complex function zA4g1Hmpppcl(PERM,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer perm(6),j1,j2,j3,j4,j5,j6
      common /order/ieorder
      double complex V4,FR4mpppcl,FR4mpppnf,F31m
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      integer ipset

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      V4 = -conjg(zA4g0Hpmmm(perm(1)))*(
     . +F31m(s12,renscale2)+F31m(s23,renscale2)+F31m(s34,renscale2)
     . +F31m(s41,renscale2))

      zA4g1Hmpppcl=V4-FR4mpppcl(perm(1))

      return
      end
************************************************************************
*XC use the result of zA4g1Hpmmm but only take complex conjugation of za and zb
*XC leave other complex number invariant to get -+++ helicity configurations
      double complex function zA4g1Hmpppnf(PERM)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer perm(6),j1,j2,j3,j4,j5,j6
      common /order/ieorder
      double complex V4,FR4mpppcl,FR4mpppnf

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      zA4g1Hmpppnf=
     .             -FR4mpppnf(perm(1))

      return
      end
************************************************************************
*XC use the result of FR4pmmm but only take complex conjugation of za and zb
*XC leave other complex number invariant to get -+++ helicity configurations

      double complex function FR4mpppcl(perm)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer perm(6),perma(6),permb(6),j1,j2,j3,j4,j5,j6
      common /order/ieorder
      double complex FR4unsymmodcl,FR4unsymmodnf
      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      perma(1)=j1
      perma(2)=j2
      perma(3)=j3
      perma(4)=j4
      perma(5)=j5
      perma(6)=j6

      permb(1)=j1
      permb(2)=j4
      permb(3)=j3
      permb(4)=j2
      permb(5)=j5
      permb(6)=j6

      if(ieorder.eq.-2)then
        FR4mpppcl=0d0
      elseif(ieorder.eq.-1)then
        FR4mpppcl=0d0
      elseif(ieorder.eq.0)then
        FR4mpppcl=FR4unsymmodcl(perma(1))+FR4unsymmodcl(permb(1))
      endif

      return
      end
************************************************************************
*XC use the result of FR4pmmm but only take complex conjugation of za and zb
*XC leave other complex number invariant to get -+++ helicity configurations
      double complex function FR4mpppnf(perm)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer perm(6),perma(6),permb(6),j1,j2,j3,j4,j5,j6
      common /order/ieorder
      double complex FR4unsymmod,FR4unsymmodnf
      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      perma(1)=j1
      perma(2)=j2
      perma(3)=j3
      perma(4)=j4
      perma(5)=j5
      perma(6)=j6

      permb(1)=j1
      permb(2)=j4
      permb(3)=j3
      permb(4)=j2
      permb(5)=j5
      permb(6)=j6

      if(ieorder.eq.-2)then
        FR4mpppnf=0d0
      elseif(ieorder.eq.-1)then
        FR4mpppnf=0d0
      elseif(ieorder.eq.0)then
        FR4mpppnf=
     .         +FR4unsymmodnf(perma(1))+FR4unsymmodnf(permb(1))
      endif

      return
      end
************************************************************************
      complex*16 function zA4g1Hmmppcl(PERM,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer perm(6),perma(6),permb(6),j1,j2,j3,j4,j5,j6

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      perma(1)=j1
      perma(2)=j2
      perma(3)=j3
      perma(4)=j4
      perma(5)=j5
      perma(6)=j6

      permb(1)=j3
      permb(2)=j4
      permb(3)=j1
      permb(4)=j2
      permb(5)=j5
      permb(6)=j6

cw--- 0704.3914v3 Eqs. (2.4) and (2.6)
cw--- Note: c.c. is equivalent to interchanging za and zb
cw--- spinor product in MCFM is in spinorz.f spinoru.f/Need/src
      zA4g1Hmmppcl=zAphi4g1mmppcl(PERMa(1),renscale2)
     .            +zAcphi4g1mmppcl(PERMb(1),renscale2)

      return
      end
************************************************************************
      complex*16 function zA4g1Hmmppnf(PERM,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer perm(6),perma(6),permb(6),j1,j2,j3,j4,j5,j6

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      perma(1)=j1
      perma(2)=j2
      perma(3)=j3
      perma(4)=j4
      perma(5)=j5
      perma(6)=j6

      permb(1)=j3
      permb(2)=j4
      permb(3)=j1
      permb(4)=j2
      permb(5)=j5
      permb(6)=j6

cw--- 0704.3914v3 Eqs. (2.4) and (2.6)
cw--- Note: c.c. is equivalent to interchanging za and zb
cw--- spinor product in MCFM is in spinorz.f spinoru.f/Need/src
      zA4g1Hmmppnf=
     .          +zAphi4g1mmppnf(PERMa(1),renscale2)
     .          +zAcphi4g1mmppnf(PERMb(1),renscale2)

      return
      end
************************************************************************
cw--- Results taken from:
cw---   S.~D.~Badger, E.~W.~N.~Glover and K.~Risager,
cw---   %``One-loop phigggg-MHV amplitudes using the unitarity bootstrap,''
cw---   JHEP {\bf 0707}, 066 (2007)
cw---   [arXiv:0704.3914 [hep-ph]].
cw--- with the sign of all terms proportional to Np or b0 reversed

      double complex function zAphi4g1mmppcl(perm,renscale2)
      use KinData_mod
      implicit none
      double complex C4mmpphatcl,Rhat4mmppnf,Rhat4mmppcl
CW     arXiv:0704.3914v3, Eq.(5.13)
      integer perm(6),ieorder
      common /order/ieorder
      double precision renscale2

cw---- version using "hatting" procedure
      zAphi4g1mmppcl=C4mmpphatcl(perm(1),renscale2)
     &              +Rhat4mmppcl(perm(1))

      return
      end
************************************************************************
cw--- Results taken from:
cw---   S.~D.~Badger, E.~W.~N.~Glover and K.~Risager,
cw---   %``One-loop phigggg-MHV amplitudes using the unitarity bootstrap,''
cw---   JHEP {\bf 0707}, 066 (2007)
cw---   [arXiv:0704.3914 [hep-ph]].
cw--- with the sign of all terms proportional to Np or b0 reversed

      double complex function zAphi4g1mmppnf(perm,renscale2)
      use KinData_mod
      implicit none
      double complex Rhat4mmppnf,Rhat4mmppcl,C4mmpphatnf
CW     arXiv:0704.3914v3, Eq.(5.13)
      integer perm(6),ieorder
      double precision renscale2
      common /order/ieorder

cw---- version using "hatting" procedure
      zAphi4g1mmppnf=
     .            +C4mmpphatnf(perm(1),renscale2)
     &            +Rhat4mmppnf(perm(1))

      return
      end
************************************************************************
      double complex function Rhat4mmppcl(PERM)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
C     arXiv:0704.3914v3, Eq.(5.12) (factor of 16 pi^2 removed)
      integer perm(6),perma(6),permb(6),j1,j2,j3,j4,j5,j6
      double precision ss3
      common /order/ieorder
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43
      integer ipset

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      perma(1)=j1
      perma(2)=j2
      perma(3)=j3
      perma(4)=j4
      perma(5)=j5
      perma(6)=j6

      permb(1)=j3
      permb(2)=j4
      permb(3)=j1
      permb(4)=j2
      permb(5)=j5
      permb(6)=j6

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za34 => kin(ipset)%zA(j3,j4)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za43 => kin(ipset)%zA(j4,j3)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb34 => kin(ipset)%zB(j3,j4)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb43 => kin(ipset)%zB(j4,j3)

      z2ab1243=zab2(j1,j2,j4,j3,ipset)
      z2ab3123=zab2(j3,j1,j2,j3,ipset)
      z2ab2134=zab2(j2,j1,j3,j4,ipset)
      z2ab4124=zab2(j4,j1,j2,j4,ipset)

      s3s234=ss3(j2,j3,j4,ipset)
      s3s341=ss3(j3,j4,j1,ipset)

cw--- Note: implicit use of A0(A,...)=A0phigggg(...)-A0phiggggdagger(...)
cw---       and the fact that A0phiggggdagger=(c.c. of A0phigggg)
      if(ieorder.eq.-2)then
        Rhat4mmppcl=0d0
      elseif(ieorder.eq.-1)then
        Rhat4mmppcl=0d0
      elseif(ieorder.eq.0)then
        Rhat4mmppcl=2d0
     &   *(zAphi4g0mmpp(perma(1))
     &    -conjg(zAphi4g0mmpp(permb(1))))
     &   +1d0/3d0*zb43/za34
     &   *(-za23*z2ab1243**2
     &   /(za34*zb43*zb32*s3s234)
     &    +za41*z2ab3123
     &   /(za34*zb12*zb32)
     &    -za14*z2ab2134**2
     &   /(za34*zb43*zb41*s3s341)
     &    +za32*z2ab4124
     &   /(za34*zb12*zb41)
     &    +za12**2/(za34*zb43)-za12/zb12
     &    -za12*z2ab2134
     &   /(2d0*zb41*s3s341)
     &    +za12*z2ab1243
     &   /(2d0*zb32*s3s234)
     &    +za12**2/(2d0*s23)+za12**2/(2d0*s41))
cw--- MODIFIED: overall sign of Rhat
      endif
      Rhat4mmppcl=-Rhat4mmppcl

      return
      end
************************************************************************
      double complex function Rhat4mmppnf(PERM)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
C     arXiv:0704.3914v3, Eq.(5.12) (factor of 16 pi^2 removed)
      integer perm(6),perma(6),permb(6),j1,j2,j3,j4,j5,j6
      double precision ss3
      common /order/ieorder
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43
      integer ipset

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za34 => kin(ipset)%zA(j3,j4)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za43 => kin(ipset)%zA(j4,j3)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb34 => kin(ipset)%zB(j3,j4)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb43 => kin(ipset)%zB(j4,j3)

      z2ab1243=zab2(j1,j2,j4,j3,ipset)
      z2ab3123=zab2(j3,j1,j2,j3,ipset)
      z2ab2134=zab2(j2,j1,j3,j4,ipset)
      z2ab4124=zab2(j4,j1,j2,j4,ipset)

      s3s234=ss3(j2,j3,j4,ipset)
      s3s341=ss3(j3,j4,j1,ipset)

cw--- Note: implicit use of A0(A,...)=A0phigggg(...)-A0phiggggdagger(...)
cw---       and the fact that A0phiggggdagger=(c.c. of A0phigggg)
      if(ieorder.eq.-2)then
        Rhat4mmppnf=0d0
      elseif(ieorder.eq.-1)then
        Rhat4mmppnf=0d0
      elseif(ieorder.eq.0)then
        Rhat4mmppnf=
     &   -1d0/3d0*zb43/za34
     &   *(-za23*z2ab1243**2
     &   /(za34*zb43*zb32*s3s234)
     &    +za41*z2ab3123
     &   /(za34*zb12*zb32)
     &    -za14*z2ab2134**2
     &   /(za34*zb43*zb41*s3s341)
     &    +za32*z2ab4124
     &   /(za34*zb12*zb41)
     &    +za12**2/(za34*zb43)-za12/zb12
     &    -za12*z2ab2134
     &   /(2d0*zb41*s3s341)
     &    +za12*z2ab1243
     &   /(2d0*zb32*s3s234)
     &    +za12**2/(2d0*s23)+za12**2/(2d0*s41))
cw--- MODIFIED: overall sign of Rhat
      endif
      Rhat4mmppnf=-Rhat4mmppnf

      return
      end
************************************************************************
cw--- This is the hatted second version of the
cw--- function presented in the paper
      double complex function C4mmpphatcl(PERM,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
CW     arXiv:0704.3914v3, Eq.(3.24) (factor of C_\Gamma removed)
      integer j1,j2,j3,j4,j5,j6,i,ii(7),perm(6)
      double complex F31m,F42me,F41m,sum,
     . BGRL3hat,BGRL1,zab2,zba2
      double precision sc(4,4)
      common /order/ieorder
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43
      integer ipset
      data ii/1,2,3,4,1,2,3/
      save ii

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za34 => kin(ipset)%zA(j3,j4)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za43 => kin(ipset)%zA(j4,j3)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb34 => kin(ipset)%zB(j3,j4)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb43 => kin(ipset)%zB(j4,j3)

cw--- set up 's-comma' products
      sc(1,1)=0d0
      sc(1,2)=s12
      sc(1,3)=s12+s13+s23
      sc(1,4)=s12+s13+s14+s23+s24+s34
      sc(2,1)=sc(1,4) !s(j1,j2)+s(j1,j3)+s(j1,j4)+s(j2,j3)+s(j2,j4)+s(j3,j4)
      sc(2,2)=0d0
      sc(2,3)=s23
      sc(2,4)=s23+s24+s34
      sc(3,1)=s34+s31+s41
      sc(3,2)=sc(1,4) !s(j1,j2)+s(j1,j3)+s(j1,j4)+s(j2,j3)+s(j2,j4)+s(j3,j4)
      sc(3,3)=0d0
      sc(3,4)=s34
      sc(4,1)=s41
      sc(4,2)=s41+s42+s12
      sc(4,3)=sc(1,4) !s(j1,j2)+s(j1,j3)+s(j1,j4)+s(j2,j3)+s(j2,j4)+s(j3,j4)
      sc(4,4)=0d0

      z2ab1243=zab2(j1,j2,j4,j3,ipset)
      z2ab3123=zab2(j3,j1,j2,j3,ipset)
      z2ab2134=zab2(j2,j1,j3,j4,ipset)
      z2ab4124=zab2(j4,j1,j2,j4,ipset)
      z2ab1234=zab2(j1,j2,j3,j4,ipset)

      z2ba3412=zba2(j3,j4,j1,j2,ipset)

      s3s234=ss3(j2,j3,j4,ipset)
      s3s341=ss3(j3,j4,j1,ipset)

      sum=(0d0,0d0)
      do i=1,4
cw--- note: corrected third argument of F42me from
cw---       s_(i,j+1) to s_(i,j-1) [with j=i+3 here]
      sum=sum
     . +F31m(sc(ii(i),ii(i+2)),renscale2)-F31m(sc(ii(i),ii(i+3)),renscale2)
     . -0.5d0*F42me(sc(ii(i),ii(i+3)),sc(ii(i+1),ii(i+2))
     .             ,sc(ii(i),ii(i+2)),sc(ii(i+1),ii(i+3)),renscale2)
     . -0.5d0*F41m(sc(ii(i),ii(i+2)),sc(ii(i),ii(i+1)),
     .                               sc(ii(i+1),ii(i+2)),renscale2)
      enddo

cw--- implementation of Eq. (3.24)

      if(ieorder.eq.-2)then
        C4mmpphatcl=zAphi4g0mmpp(perm(1))*sum
      elseif(ieorder.eq.-1)then
        C4mmpphatcl=zAphi4g0mmpp(perm(1))*sum
      elseif(ieorder.eq.0)then
        C4mmpphatcl=zAphi4g0mmpp(perm(1))*sum-(
     .   +1d0/(za23*za34*za41)*(
     .   1d0/3d0*za14*zb43*za32
     .   *za13*z2ba3412
     .   *(za13*z2ba3412-za14*zb43*za32)
     .   *BGRL3hat(sc(3,1),sc(4,1))
     .   +11d0/3d0*za12**2*za14*zb43*za32
     .   *BGRL1(sc(3,1),sc(4,1))
     .   +1d0/3d0
     .   *za14*zb43*za32*z2ab1234*za42
     .   *(z2ab1234*za42-za14*zb43*za32)
     .   *BGRL3hat(sc(2,4),sc(2,3))
     .   +11d0/3d0*za12**2*za14*zb43*za32
     .  *BGRL1(sc(2,4),sc(2,3))))

      endif

      return
      end
************************************************************************
cw--- This is the hatted second version of the
cw--- function presented in the paper
      double complex function C4mmpphatnf(PERM,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
CW     arXiv:0704.3914v3, Eq.(3.24) (factor of C_\Gamma removed)
      integer j1,j2,j3,j4,j5,j6,i,ii(7),perm(6)
      double complex F31m,F42me,F41m,sum,
     . BGRL3hat,BGRL1,zab2,zba2
      common /koppel/ nf,nc,b0
      double precision Np,bb0,sc(4,4),renscale2
      common /order/ieorder
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43
      integer ipset
      data ii/1,2,3,4,1,2,3/
      save ii

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za34 => kin(ipset)%zA(j3,j4)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za43 => kin(ipset)%zA(j4,j3)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb34 => kin(ipset)%zB(j3,j4)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb43 => kin(ipset)%zB(j4,j3)

cw--- set up 's-comma' products
      sc(1,1)=0d0
      sc(1,2)=s12
      sc(1,3)=s12+s13+s23
      sc(1,4)=s12+s13+s14+s23+s24+s34
      sc(2,1)=sc(1,4) !s(j1,j2)+s(j1,j3)+s(j1,j4)+s(j2,j3)+s(j2,j4)+s(j3,j4)
      sc(2,2)=0d0
      sc(2,3)=s23
      sc(2,4)=s23+s24+s34
      sc(3,1)=s34+s31+s41
      sc(3,2)=sc(1,4) !s(j1,j2)+s(j1,j3)+s(j1,j4)+s(j2,j3)+s(j2,j4)+s(j3,j4)
      sc(3,3)=0d0
      sc(3,4)=s34
      sc(4,1)=s41
      sc(4,2)=s41+s42+s12
      sc(4,3)=sc(1,4) !s(j1,j2)+s(j1,j3)+s(j1,j4)+s(j2,j3)+s(j2,j4)+s(j3,j4)
      sc(4,4)=0d0

      z2ab1243=zab2(j1,j2,j4,j3,ipset)
      z2ab3123=zab2(j3,j1,j2,j3,ipset)
      z2ab2134=zab2(j2,j1,j3,j4,ipset)
      z2ab4124=zab2(j4,j1,j2,j4,ipset)
      z2ab1234=zab2(j1,j2,j3,j4,ipset)

      z2ba3412=zba2(j3,j4,j1,j2,ipset)

      s3s234=ss3(j2,j3,j4,ipset)
      s3s341=ss3(j3,j4,j1,ipset)

      Np=-2d0
      bb0=-2d0/3d0

      sum=(0d0,0d0)
      do i=1,4
cw--- note: corrected third argument of F42me from
cw---       s_(i,j+1) to s_(i,j-1) [with j=i+3 here]
      sum=sum
     . +F31m(sc(ii(i),ii(i+2)),renscale2)-F31m(sc(ii(i),ii(i+3)),renscale2)
     . -0.5d0*F42me(sc(ii(i),ii(i+3)),sc(ii(i+1),ii(i+2))
     .             ,sc(ii(i),ii(i+2)),sc(ii(i+1),ii(i+3)),renscale2)
     . -0.5d0*F41m(sc(ii(i),ii(i+2)),sc(ii(i),ii(i+1)),
     .                               sc(ii(i+1),ii(i+2)),renscale2)
      enddo

cw--- implementation of Eq. (3.24)

      if(ieorder.eq.-2)then
        C4mmpphatnf=0d0
      elseif(ieorder.eq.-1)then
        C4mmpphatnf=0d0
      elseif(ieorder.eq.0)then
        C4mmpphatnf=-(
     .   +1d0/(za23*za34*za41)*(
     .   Np/6d0*za14*zb43*za32
     .   *za13*z2ba3412
     .   *(za13*z2ba3412-za14*zb43*za32)
     .   *BGRL3hat(sc(3,1),sc(4,1))
     .   +bb0*za12**2*za14*zb43*za32
     .   *BGRL1(sc(3,1),sc(4,1))
     .   +Np/6d0
     .   *za14*zb43*za32*z2ab1234*za42
     .   *(z2ab1234*za42-za14*zb43*za32)
     .   *BGRL3hat(sc(2,4),sc(2,3))
     .   +bb0*za12**2*za14*zb43*za32
     .  *BGRL1(sc(2,4),sc(2,3))))
      endif

      return
      end
************************************************************************
      double complex function zAcphi4g1mmppcl(perm,renscale2)
      use KinData_mod
      implicit none
      double complex C4mmpphatmodcl,Rhat4mmppmodcl,Rhat4mmppmodnf
      integer perm(6),ieorder
      double precision renscale2
      common /order/ieorder

      zAcphi4g1mmppcl=C4mmpphatmodcl(perm(1),renscale2)
     &               +Rhat4mmppmodcl(perm(1))

      return
      end
************************************************************************
      double complex function zAcphi4g1mmppnf(perm,renscale2)
      use KinData_mod
      implicit none
      double complex C4mmpphatmodcl,C4mmpphatmodnf,Rhat4mmppmodnf
      double precision renscale2
      integer perm(6),ieorder
      common /order/ieorder
      zAcphi4g1mmppnf=
     .            +C4mmpphatmodnf(perm(1),renscale2)
     &            +Rhat4mmppmodnf(perm(1))

      return
      end
************************************************************************
      double complex function C4mmpphatmodcl(PERM,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer j1,j2,j3,j4,j5,j6,i,ii(7),perm(6)
      double complex F31m,F42me,F41m,sum,
     . BGRL3hat,BGRL1,zab2,zba2
      double precision sc(4,4)
      common /order/ieorder
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43
      integer ipset
      data ii/1,2,3,4,1,2,3/
      save ii

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za34 => kin(ipset)%zA(j3,j4)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za43 => kin(ipset)%zA(j4,j3)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb34 => kin(ipset)%zB(j3,j4)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb43 => kin(ipset)%zB(j4,j3)

      sc(1,1)=0d0
      sc(1,2)=s12
      sc(1,3)=s12+s13+s23
      sc(1,4)=s12+s13+s14+s23+s24+s34
      sc(2,1)=sc(1,4) !s(j1,j2)+s(j1,j3)+s(j1,j4)+s(j2,j3)+s(j2,j4)+s(j3,j4)
      sc(2,2)=0d0
      sc(2,3)=s23
      sc(2,4)=s23+s24+s34
      sc(3,1)=s34+s31+s41
      sc(3,2)=sc(1,4) !s(j1,j2)+s(j1,j3)+s(j1,j4)+s(j2,j3)+s(j2,j4)+s(j3,j4)
      sc(3,3)=0d0
      sc(3,4)=s34
      sc(4,1)=s41
      sc(4,2)=s41+s42+s12
      sc(4,3)=sc(1,4) !s(j1,j2)+s(j1,j3)+s(j1,j4)+s(j2,j3)+s(j2,j4)+s(j3,j4)
      sc(4,4)=0d0

      z2ab1243=zab2(j1,j2,j4,j3,ipset)
      z2ab3123=zab2(j3,j1,j2,j3,ipset)
      z2ab2134=zab2(j2,j1,j3,j4,ipset)
      z2ab4124=zab2(j4,j1,j2,j4,ipset)
      z2ab1234=zab2(j1,j2,j3,j4,ipset)
      z2ab3412=zab2(j3,j4,j1,j2,ipset)

      z2ba3412=zba2(j3,j4,j1,j2,ipset)
      z2ba1234=zba2(j1,j2,j3,j4,ipset)

      s3s234=ss3(j2,j3,j4,ipset)
      s3s341=ss3(j3,j4,j1,ipset)

      sum=(0d0,0d0)
      do i=1,4
      sum=sum
     . +F31m(sc(ii(i),ii(i+2)),renscale2)-F31m(sc(ii(i),ii(i+3)),renscale2)
     . -0.5d0*F42me(sc(ii(i),ii(i+3)),sc(ii(i+1),ii(i+2))
     .             ,sc(ii(i),ii(i+2)),sc(ii(i+1),ii(i+3)),renscale2)
     . -0.5d0*F41m(sc(ii(i),ii(i+2)),sc(ii(i),ii(i+1)),
     .                               sc(ii(i+1),ii(i+2)),renscale2)
      enddo

      if(ieorder.eq.-2)then
        C4mmpphatmodcl=conjg(zAphi4g0mmpp(perm(1)))*sum
      elseif(ieorder.eq.-1)then
        C4mmpphatmodcl=conjg(zAphi4g0mmpp(perm(1)))*sum
      elseif(ieorder.eq.0)then
        C4mmpphatmodcl=conjg(zAphi4g0mmpp(perm(1)))*sum-(
     .   +1d0/(zb23*zb34*zb41)*(
     .   1d0/3d0*zb14*za43*zb32
     .   *zb13*z2ab3412
     .   *(zb13*z2ab3412-zb14*za43*zb32)
     .   *BGRL3hat(sc(3,1),sc(4,1))
     .   +11d0/3d0*zb12**2*zb14*za43*zb32
     .   *BGRL1(sc(3,1),sc(4,1))
     .   +1d0/3d0
     .   *zb14*za43*zb32*z2ba1234*zb42
     .   *(z2ba1234*zb42-zb14*za43*zb32)
     .   *BGRL3hat(sc(2,4),sc(2,3))
     .   +11d0/3d0*zb12**2*zb14*za43*zb32
     .  *BGRL1(sc(2,4),sc(2,3))))

      endif

      return
      end
************************************************************************
      double complex function C4mmpphatmodnf(PERM,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer j1,j2,j3,j4,j5,j6,i,ii(7),perm(6)
      double complex F31m,F42me,F41m,sum,
     . BGRL3hat,BGRL1,zab2,zba2
      double precision sc(4,4)
      common /order/ieorder
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43
      data ii/1,2,3,4,1,2,3/
      save ii

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za34 => kin(ipset)%zA(j3,j4)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za43 => kin(ipset)%zA(j4,j3)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb34 => kin(ipset)%zB(j3,j4)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb43 => kin(ipset)%zB(j4,j3)

      sc(1,1)=0d0
      sc(1,2)=s12
      sc(1,3)=s12+s13+s23
      sc(1,4)=s12+s13+s14+s23+s24+s34
      sc(2,1)=sc(1,4) !s(j1,j2)+s(j1,j3)+s(j1,j4)+s(j2,j3)+s(j2,j4)+s(j3,j4)
      sc(2,2)=0d0
      sc(2,3)=s23
      sc(2,4)=s23+s24+s34
      sc(3,1)=s34+s31+s41
      sc(3,2)=sc(1,4) !s(j1,j2)+s(j1,j3)+s(j1,j4)+s(j2,j3)+s(j2,j4)+s(j3,j4)
      sc(3,3)=0d0
      sc(3,4)=s34
      sc(4,1)=s41
      sc(4,2)=s41+s42+s12
      sc(4,3)=sc(1,4) !s(j1,j2)+s(j1,j3)+s(j1,j4)+s(j2,j3)+s(j2,j4)+s(j3,j4)
      sc(4,4)=0d0

      z2ab1243=zab2(j1,j2,j4,j3,ipset)
      z2ab3123=zab2(j3,j1,j2,j3,ipset)
      z2ab2134=zab2(j2,j1,j3,j4,ipset)
      z2ab4124=zab2(j4,j1,j2,j4,ipset)
      z2ab1234=zab2(j1,j2,j3,j4,ipset)
      z2ab3412=zab2(j3,j4,j1,j2,ipset)

      z2ba3412=zba2(j3,j4,j1,j2,ipset)
      z2ba1234=zba2(j1,j2,j3,j4,ipset)

      s3s234=ss3(j2,j3,j4,ipset)
      s3s341=ss3(j3,j4,j1,ipset)

      sum=(0d0,0d0)
      do i=1,4
      sum=sum
     . +F31m(sc(ii(i),ii(i+2)),renscale2)-F31m(sc(ii(i),ii(i+3)),renscale2)
     . -0.5d0*F42me(sc(ii(i),ii(i+3)),sc(ii(i+1),ii(i+2))
     .             ,sc(ii(i),ii(i+2)),sc(ii(i+1),ii(i+3)),renscale2)
     . -0.5d0*F41m(sc(ii(i),ii(i+2)),sc(ii(i),ii(i+1)),
     .                               sc(ii(i+1),ii(i+2)),renscale2)
      enddo

      if(ieorder.eq.-2)then
        C4mmpphatmodnf=0d0
      elseif(ieorder.eq.-1)then
        C4mmpphatmodnf=0d0
      elseif(ieorder.eq.0)then
        C4mmpphatmodnf=-(
     .   +1d0/(zb23*zb34*zb41)*(
     .   -1d0/3d0*zb14*za43*zb32
     .   *zb13*z2ab3412
     .   *(zb13*z2ab3412-zb14*za43*zb32)
     .   *BGRL3hat(sc(3,1),sc(4,1))
     .   -2d0/3d0*zb12**2*zb14*za43*zb32
     .   *BGRL1(sc(3,1),sc(4,1))
     .   -1d0/3d0
     .   *zb14*za43*zb32*z2ba1234*zb42
     .   *(z2ba1234*zb42-zb14*za43*zb32)
     .   *BGRL3hat(sc(2,4),sc(2,3))
     .   -2d0/3d0*zb12**2*zb14*za43*zb32
     .  *BGRL1(sc(2,4),sc(2,3))))

      endif

      return
      end

************************************************************************
      double complex function Rhat4mmppmodcl(PERM)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer perm(6),perma(6),permb(6),j1,j2,j3,j4,j5,j6
      double precision ss3
      common /order/ieorder
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      perma(1)=j1
      perma(2)=j2
      perma(3)=j3
      perma(4)=j4
      perma(5)=j5
      perma(6)=j6

      permb(1)=j3
      permb(2)=j4
      permb(3)=j1
      permb(4)=j2
      permb(5)=j5
      permb(6)=j6

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za34 => kin(ipset)%zA(j3,j4)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za43 => kin(ipset)%zA(j4,j3)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb34 => kin(ipset)%zB(j3,j4)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb43 => kin(ipset)%zB(j4,j3)

      z2ab1243=zab2(j1,j2,j4,j3,ipset)
      z2ab3123=zab2(j3,j1,j2,j3,ipset)
      z2ab2134=zab2(j2,j1,j3,j4,ipset)
      z2ab4124=zab2(j4,j1,j2,j4,ipset)
      z2ab1234=zab2(j1,j2,j3,j4,ipset)
      z2ab3412=zab2(j3,j4,j1,j2,ipset)

      z2ba3412=zba2(j3,j4,j1,j2,ipset)
      z2ba1234=zba2(j1,j2,j3,j4,ipset)
      z2ba1243=zba2(j1,j2,j4,j3,ipset)
      z2ba3123=zba2(j3,j1,j2,j3,ipset)
      z2ba2134=zba2(j2,j1,j3,j4,ipset)
      z2ba4124=zba2(j4,j1,j2,j4,ipset)

      s3s234=ss3(j2,j3,j4,ipset)
      s3s341=ss3(j3,j4,j1,ipset)

      if(ieorder.eq.-2)then
        Rhat4mmppmodcl=0d0
      elseif(ieorder.eq.-1)then
        Rhat4mmppmodcl=0d0
      elseif(ieorder.eq.0)then
        Rhat4mmppmodcl=2d0
     &   *(conjg(zAphi4g0mmpp(perma(1)))
     &    -zAphi4g0mmpp(permb(1)))
     &   +1d0/3d0*za43/zb34
     &   *(-zb23*z2ba1243**2
     &   /(zb34*za43*za32*s3s234)
     &    +zb41*z2ba3123
     &   /(zb34*za12*za32)
     &    -zb14*z2ba2134**2
     &   /(zb34*za43*za41*s3s341)
     &    +zb32*z2ba4124
     &   /(zb34*za12*za41)
     &    +zb12**2/(zb34*za43)-zb12/za12
     &    -zb12*z2ba2134
     &   /(2d0*za41*s3s341)
     &    +zb12*z2ba1243
     &   /(2d0*za32*s3s234)
     &    +zb12**2/(2d0*s23)+zb12**2/(2d0*s41))
      endif
      Rhat4mmppmodcl=-Rhat4mmppmodcl

      return
      end
************************************************************************
      double complex function Rhat4mmppmodnf(PERM)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer perm(6),perma(6),permb(6),j1,j2,j3,j4,j5,j6
      double precision ss3
      common /order/ieorder
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za34 => kin(ipset)%zA(j3,j4)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za43 => kin(ipset)%zA(j4,j3)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb34 => kin(ipset)%zB(j3,j4)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb43 => kin(ipset)%zB(j4,j3)

      z2ab1243=zab2(j1,j2,j4,j3,ipset)
      z2ab3123=zab2(j3,j1,j2,j3,ipset)
      z2ab2134=zab2(j2,j1,j3,j4,ipset)
      z2ab4124=zab2(j4,j1,j2,j4,ipset)
      z2ab1234=zab2(j1,j2,j3,j4,ipset)
      z2ab3412=zab2(j3,j4,j1,j2,ipset)

      z2ba3412=zba2(j3,j4,j1,j2,ipset)
      z2ba1234=zba2(j1,j2,j3,j4,ipset)
      z2ba1243=zba2(j1,j2,j4,j3,ipset)
      z2ba3123=zba2(j3,j1,j2,j3,ipset)
      z2ba2134=zba2(j2,j1,j3,j4,ipset)
      z2ba4124=zba2(j4,j1,j2,j4,ipset)

      s3s234=ss3(j2,j3,j4,ipset)
      s3s341=ss3(j3,j4,j1,ipset)

      if(ieorder.eq.-2)then
        Rhat4mmppmodnf=0d0
      elseif(ieorder.eq.-1)then
        Rhat4mmppmodnf=0d0
      elseif(ieorder.eq.0)then
        Rhat4mmppmodnf=
     &   -1d0/3d0*za43/zb34
     &   *(-zb23*z2ba1243**2
     &   /(zb34*za43*za32*s3s234)
     &    +zb41*z2ba3123
     &   /(zb34*za12*za32)
     &    -zb14*z2ba2134**2
     &   /(zb34*za43*za41*s3s341)
     &    +zb32*z2ba4124
     &   /(zb34*za12*za41)
     &    +zb12**2/(zb34*za43)-zb12/za12
     &    -zb12*z2ba2134
     &   /(2d0*za41*s3s341)
     &    +zb12*z2ba1243
     &   /(2d0*za32*s3s234)
     &    +zb12**2/(2d0*s23)+zb12**2/(2d0*s41))
      endif
      Rhat4mmppmodnf=-Rhat4mmppmodnf

      return
      end
************************************************************************
      double complex function zA4g1Hmpmpcl(PERM,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer perm(6),perma(6),permb(6),j1,j2,j3,j4,j5,j6

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      perma(1)=j1
      perma(2)=j2
      perma(3)=j3
      perma(4)=j4
      perma(5)=j5
      perma(6)=j6

      permb(1)=j2
      permb(2)=j3
      permb(3)=j4
      permb(4)=j1
      permb(5)=j5
      permb(6)=j6

cw--- 0704.3914v3 Eqs. (2.4) and (2.6)
cw--- Note: c.c. is equivalent to interchanging za and zb
      zA4g1Hmpmpcl=zAphi4g1mpmpcl(PERMa(1),renscale2)
     .            +zAcphi4g1mpmpcl(PERMb(1),renscale2)

      return
      end
************************************************************************
      double complex function zA4g1Hmpmpnf(PERM,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer perm(6),perma(6),permb(6),j1,j2,j3,j4,j5,j6

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      perma(1)=j1
      perma(2)=j2
      perma(3)=j3
      perma(4)=j4
      perma(5)=j5
      perma(6)=j6

      permb(1)=j2
      permb(2)=j3
      permb(3)=j4
      permb(4)=j1
      permb(5)=j5
      permb(6)=j6

cw--- 0704.3914v3 Eqs. (2.4) and (2.6)
cw--- Note: c.c. is equivalent to interchanging za and zb
      zA4g1Hmpmpnf=
     .          +zAphi4g1mpmpnf(PERMa(1))
     .          +zAcphi4g1mpmpnf(PERMb(1),renscale2)

      return
      end

************************************************************************
cw--- Results taken from:
cw---   E.~W.~N.~Glover, P.~Mastrolia and C.~Williams,
cw---   %``One-loop phigggg-MHV amplitudes using the unitarity bootstrap: the general
cw---   %helicity case,''
cw---   JHEP {\bf 0808}, 017 (2008)
cw---   [arXiv:0804.4149 [hep-ph]].

cw--- with the sign of rational terms reversed

cw---  Note that a factor of c_\Gamma is missing in Eq. (5.1)

      double complex function zAphi4g1mpmpcl(PERM,renscale2)
CW     arXiv:0804.4149v3, Eq.(5.15)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer perm(6)
      double complex C4mpmpcl,C4mpmpnf,Rhat4mpmpcl,Rhat4mpmpnf,CR4mpmpcl,CR4mpmpnf

      zAphi4g1mpmpcl=C4mpmpcl(perm(1),renscale2)
c     &              +CR4mpmpcl(perm(1))
*XC combined CR4mpmpcl with C4mpmpcl function by replacing the L2 and L3 functions in C4mpmpcl by L2hat and L3hat
*XC this could improve the numerical stability
     &              +Rhat4mpmpcl(perm(1))

      return
      end
************************************************************************
cw--- Results taken from:
cw---   E.~W.~N.~Glover, P.~Mastrolia and C.~Williams,
cw---   %``One-loop phigggg-MHV amplitudes using the unitarity bootstrap: the general
cw---   %helicity case,''
cw---   JHEP {\bf 0808}, 017 (2008)
cw---   [arXiv:0804.4149 [hep-ph]].

cw--- with the sign of rational terms reversed

cw---  Note that a factor of c_\Gamma is missing in Eq. (5.1)

      double complex function zAphi4g1mpmpnf(PERM)
CW     arXiv:0804.4149v3, Eq.(5.15)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer perm(6)
      double complex C4mpmpnf,Rhat4mpmpcl,Rhat4mpmpnf,CR4mpmpcl,CR4mpmpnf

      zAphi4g1mpmpnf=
     .            +C4mpmpnf(perm(1))
c     &            +CR4mpmpnf(perm(1))
*XC combined CR4mpmpnf with C4mpmpnf function by replacing the L2 and L3 functions in C4mpmpnf by L2hat and L3hat
*XC this could improve the numerical stability
     &            +Rhat4mpmpnf(perm(1))

      return
      end

************************************************************************
      double complex function C4mpmpcl(PERM,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
CW     arXiv:0804.4149v3, Eq.(5.1) (factor of C_\Gamma removed)
      common /order/ieorder
      integer j1,j2,j3,j4,j5,j6,i,ii(7),perm(6)
      double complex C4mpmpsubcl,C4mpmpsubnf,zAphi4g0mpmp,F31m,F42me,F41m,sum
      double precision sc(4,4)
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43
      data ii/1,2,3,4,1,2,3/
      save ii

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

c--- set up 's-comma' products
      sc(1,1)=0d0
      sc(1,2)=s12
      sc(1,3)=s12+s13+s23
      sc(1,4)=s12+s13+s14+s23+s24+s34
      sc(2,1)=sc(1,4) !s(j1,j2)+s(j1,j3)+s(j1,j4)+s(j2,j3)+s(j2,j4)+s(j3,j4)
      sc(2,2)=0d0
      sc(2,3)=s23
      sc(2,4)=s23+s24+s34
      sc(3,1)=s34+s31+s41
      sc(3,2)=sc(1,4) !s(j1,j2)+s(j1,j3)+s(j1,j4)+s(j2,j3)+s(j2,j4)+s(j3,j4)
      sc(3,3)=0d0
      sc(3,4)=s34
      sc(4,1)=s41
      sc(4,2)=s41+s42+s12
      sc(4,3)=sc(1,4) !s(j1,j2)+s(j1,j3)+s(j1,j4)+s(j2,j3)+s(j2,j4)+s(j3,j4)
      sc(4,4)=0d0

      sum=(0d0,0d0)
      do i=1,4
      sum=sum
     . -0.5d0*F42me(sc(ii(i),ii(i+3)),sc(ii(i+1),ii(i+2))
     .             ,sc(ii(i+1),ii(i+3)),sc(ii(i),ii(i+2)),renscale2)
     . -0.5d0*F41m(sc(ii(i),ii(i+2)),sc(ii(i),ii(i+1)),
     .                               sc(ii(i+1),ii(i+2)),renscale2)
     . +F31m(sc(ii(i),ii(i+2)),renscale2)-F31m(sc(ii(i),ii(i+3)),renscale2)
      enddo

      if(ieorder.eq.-2)then
        C4mpmpcl=sum*zAphi4g0mpmp(PERM(1))
      elseif(ieorder.eq.-1)then
        C4mpmpcl=sum*zAphi4g0mpmp(PERM(1))
      elseif(ieorder.eq.0)then
        sum=sum
     .   +C4mpmpsubcl(j1,j2,j3,j4)
     .   +C4mpmpsubcl(j1,j4,j3,j2)
     .   +C4mpmpsubcl(j3,j2,j1,j4)
     .   +C4mpmpsubcl(j3,j4,j1,j2)

        C4mpmpcl=sum*zAphi4g0mpmp(PERM(1))
      endif

      return
      end
************************************************************************
      double complex function C4mpmpnf(PERM)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
CW     arXiv:0804.4149v3, Eq.(5.1) (factor of C_\Gamma removed)
      common /order/ieorder
      integer j1,j2,j3,j4,j5,j6,i,ii(7),perm(6)
      double complex C4mpmpsubcl,C4mpmpsubnf,zAphi4g0mpmp,sum
      double precision sc(4,4)
      data ii/1,2,3,4,1,2,3/
      save ii

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      if(ieorder.eq.-2)then
        C4mpmpnf=0d0
      elseif(ieorder.eq.-1)then
        C4mpmpnf=0d0
      elseif(ieorder.eq.0)then
        sum=(0d0,0d0)
        sum=
     .   +C4mpmpsubnf(j1,j2,j3,j4)
     .   +C4mpmpsubnf(j1,j4,j3,j2)
     .   +C4mpmpsubnf(j3,j2,j1,j4)
     .   +C4mpmpsubnf(j3,j4,j1,j2)

        C4mpmpnf=sum*zAphi4g0mpmp(PERM(1))
      endif

      return
      end
************************************************************************
      double complex function C4mpmpsubcl(j1,j2,j3,j4)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
CW     arXiv:0804.4149v3, Eq.(5.1) (factor of C_\Gamma removed)
      integer j1,j2,j3,j4
      double complex trm,trm3241,trm3421,BGRL3,BGRL2,BGRL1,F41mF
      double precision s234
CW     c.f. arXiv:0804.4149v3 Eq.(3.38)
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za34 => kin(ipset)%zA(j3,j4)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za43 => kin(ipset)%zA(j4,j3)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb34 => kin(ipset)%zB(j3,j4)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb43 => kin(ipset)%zB(j4,j3)

      s234=s23+s24+s34
      trm3241=za32*zb24*za41*zb13
      trm3421=za34*zb42*za21*zb13

cw--- MODIFIED: added an overall factor of (-1) here
      C4mpmpsubcl=
     . +4d0*(trm3241*trm3421/(2d0*(s24*s13)**2)
     . *F41mF(s234,s23,s34)
     . -trm3241*trm3421/(s24*s13**2)
     . *BGRL1(s23,s234))
     . +2d0*(-0.5d0*(trm3241*trm3421/(s24*s13)**2)**2
     . *F41mF(s234,s23,s34)
     . -trm3241*trm3421/s13**4
     . *(+trm3241**2/(3d0*s24)*BGRL3hat(s23,s234)
     . +trm3421*trm3241/(2d0*s24**2)*BGRL2hat(s23,s234)
     . -trm3421*trm3241/(s24**3)*BGRL1(s23,s234)))

      return
      end
************************************************************************
      double complex function C4mpmpsubnf(j1,j2,j3,j4)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
CW     arXiv:0804.4149v3, Eq.(5.1) (factor of C_\Gamma removed)
      integer j1,j2,j3,j4
      double complex trm,trm3241,trm3421,BGRL3,BGRL2,BGRL1,F41mF
      double precision s234
CW     c.f. arXiv:0804.4149v3 Eq.(3.38)
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za34 => kin(ipset)%zA(j3,j4)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za43 => kin(ipset)%zA(j4,j3)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb34 => kin(ipset)%zB(j3,j4)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb43 => kin(ipset)%zB(j4,j3)

      s234=s23+s24+s34
      trm3241=za32*zb24*za41*zb13
      trm3421=za34*zb42*za21*zb13

cw--- MODIFIED: added an overall factor of (-1) here
      C4mpmpsubnf=-1d0*(trm3241*trm3421/(2d0*(s24*s13)**2)
     . *F41mF(s234,s23,s34)
     . -trm3241*trm3421/(s24*s13**2)
     . *BGRL1(s23,s234))
     . -2d0*(-0.5d0*(trm3241*trm3421/(s24*s13)**2)**2
     . *F41mF(s234,s23,s34)
     . -trm3241*trm3421/s13**4
     . *(+trm3241**2/(3d0*s24)*BGRL3hat(s23,s234)
     . +trm3421*trm3241/(2d0*s24**2)*BGRL2hat(s23,s234)
     . -trm3421*trm3241/(s24**3)*BGRL1(s23,s234)))

      return
      end

************************************************************************
      Double complex function Rhat4mpmpcl(PERM)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
CW     arXiv:0804.4149v3, Eq.(5.17)
cw---                    (factor of C_\Gamma = 1/(16 pi^2) removed)
      common /order/ieorder
      double complex im
      double precision ss3
      integer perm(6),perma(6),permb(6),j1,j2,j3,j4,j5,j6
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      perma(1)=j1
      perma(2)=j2
      perma(3)=j3
      perma(4)=j4
      perma(5)=j5
      perma(6)=j6

      permb(1)=j2
      permb(2)=j3
      permb(3)=j4
      permb(4)=j1
      permb(5)=j5
      permb(6)=j6

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za34 => kin(ipset)%zA(j3,j4)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za43 => kin(ipset)%zA(j4,j3)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb34 => kin(ipset)%zB(j3,j4)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb43 => kin(ipset)%zB(j4,j3)

      z2ab1243=zab2(j1,j2,j4,j3,ipset)
      z2ab3123=zab2(j3,j1,j2,j3,ipset)
      z2ab2134=zab2(j2,j1,j3,j4,ipset)
      z2ab4124=zab2(j4,j1,j2,j4,ipset)
      z2ab1234=zab2(j1,j2,j3,j4,ipset)
      z2ab3412=zab2(j3,j4,j1,j2,ipset)

      z2ba3412=zba2(j3,j4,j1,j2,ipset)
      z2ba1234=zba2(j1,j2,j3,j4,ipset)
      z2ba1243=zba2(j1,j2,j4,j3,ipset)
      z2ba3123=zba2(j3,j1,j2,j3,ipset)
      z2ba2134=zba2(j2,j1,j3,j4,ipset)
      z2ba4124=zba2(j4,j1,j2,j4,ipset)

      s3s234=ss3(j2,j3,j4,ipset)
      s3s341=ss3(j3,j4,j1,ipset)
      s3s412=ss3(j4,j1,j2,ipset)

      im=(0d0,1d0)

c--- Note: implicit use of A0(A,...)=-i*[A0phi(...)-A0phidagger(...)]
c---       and the fact that A0phidagger=(c.c. of A0phi)

      if(ieorder.eq.-2)then
        Rhat4mpmpcl=0d0
      elseif(ieorder.eq.-1)then
        Rhat4mpmpcl=0d0
      elseif(ieorder.eq.0)then
        Rhat4mpmpcl=
     .   -2d0*(-im)*(zAphi4g0mpmp(PERMa(1))
     .              -conjg(zAphi4g0mpmp(PERMb(1))))
     .   +1d0/6d0*zb24**4/(zb12*zb23*zb34*zb41)
     .   *(-s23*s34/(s24*s3s412)
     .     +3d0*s23*s34/s24**2
     .     -s12*s41/(s24*s3s234)
     .     +3d0*s12*s41/s24**2)
      endif

      return
      end
************************************************************************
      Double complex function Rhat4mpmpnf(PERM)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
CW     arXiv:0804.4149v3, Eq.(5.17)
cw---                    (factor of C_\Gamma = 1/(16 pi^2) removed)
      common /order/ieorder
      double complex im
      double precision ss3
      integer perm(6),perma(6),permb(6),j1,j2,j3,j4,j5,j6
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za34 => kin(ipset)%zA(j3,j4)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za43 => kin(ipset)%zA(j4,j3)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb34 => kin(ipset)%zB(j3,j4)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb43 => kin(ipset)%zB(j4,j3)

      z2ab1243=zab2(j1,j2,j4,j3,ipset)
      z2ab3123=zab2(j3,j1,j2,j3,ipset)
      z2ab2134=zab2(j2,j1,j3,j4,ipset)
      z2ab4124=zab2(j4,j1,j2,j4,ipset)
      z2ab1234=zab2(j1,j2,j3,j4,ipset)
      z2ab3412=zab2(j3,j4,j1,j2,ipset)

      z2ba3412=zba2(j3,j4,j1,j2,ipset)
      z2ba1234=zba2(j1,j2,j3,j4,ipset)
      z2ba1243=zba2(j1,j2,j4,j3,ipset)
      z2ba3123=zba2(j3,j1,j2,j3,ipset)
      z2ba2134=zba2(j2,j1,j3,j4,ipset)
      z2ba4124=zba2(j4,j1,j2,j4,ipset)

      s3s234=ss3(j2,j3,j4,ipset)
      s3s341=ss3(j3,j4,j1,ipset)
      s3s412=ss3(j4,j1,j2,ipset)

      im=(0d0,1d0)

c--- Note: implicit use of A0(A,...)=-i*[A0phi(...)-A0phidagger(...)]
c---       and the fact that A0phidagger=(c.c. of A0phi)

      if(ieorder.eq.-2)then
        Rhat4mpmpnf=0d0
      elseif(ieorder.eq.-1)then
        Rhat4mpmpnf=0d0
      elseif(ieorder.eq.0)then
        Rhat4mpmpnf=
     .   -1d0/6d0*zb24**4/(zb12*zb23*zb34*zb41)
     .   *(-s23*s34/(s24*s3s412)
     .     +3d0*s23*s34/s24**2
     .     -s12*s41/(s24*s3s234)
     .     +3d0*s12*s41/s24**2)
      endif

      return
      end
************************************************************************
      double complex function CR4mpmpcl(PERM)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
CW     arXiv:0804.4149v3, Eq.(5.2)
cw---                    (factor of C_\Gamma = 1/(16 pi^2) removed)
      integer j1,j2,j3,j4,j5,j6,perm(6),ieorder
      double complex CR4mpmpsubcl
      common /order/ieorder

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      if(ieorder.eq.-2)then
        CR4mpmpcl=0d0
      elseif(ieorder.eq.-1)then
        CR4mpmpcl=0d0
      elseif(ieorder.eq.0)then
        CR4mpmpcl=
     .   +CR4mpmpsubcl(j1,j2,j3,j4)+CR4mpmpsubcl(j1,j4,j3,j2)
     .   +CR4mpmpsubcl(j3,j2,j1,j4)+CR4mpmpsubcl(j3,j4,j1,j2)
      endif

      return
      end
************************************************************************
      double complex function CR4mpmpnf(PERM)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
CW     arXiv:0804.4149v3, Eq.(5.2)
cw---                    (factor of C_\Gamma = 1/(16 pi^2) removed)
      integer j1,j2,j3,j4,j5,j6,perm(6),ieorder
      double complex CR4mpmpsubnf
      common /order/ieorder

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      if(ieorder.eq.-2)then
        CR4mpmpnf=0d0
      elseif(ieorder.eq.-1)then
        CR4mpmpnf=0d0
      elseif(ieorder.eq.0)then
        CR4mpmpnf=
     .   +CR4mpmpsubnf(j1,j2,j3,j4)+CR4mpmpsubnf(j1,j4,j3,j2)
     .   +CR4mpmpsubnf(j3,j2,j1,j4)+CR4mpmpsubnf(j3,j4,j1,j2)
      endif

      return
      end
************************************************************************
      double complex function CR4mpmpsubcl(j1,j2,j3,j4)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
CW     arXiv:0804.4149v3, Eq.(5.2)
cw---                    (factor of C_\Gamma = 1/(16 pi^2) removed)
      integer j1,j2,j3,j4
      double precision ss3
      double complex zab3
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za34 => kin(ipset)%zA(j3,j4)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za43 => kin(ipset)%zA(j4,j3)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb34 => kin(ipset)%zB(j3,j4)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb43 => kin(ipset)%zB(j4,j3)

      z2ab1243=zab2(j1,j2,j4,j3,ipset)
      z2ab3123=zab2(j3,j1,j2,j3,ipset)
      z2ab2134=zab2(j2,j1,j3,j4,ipset)
      z2ab4124=zab2(j4,j1,j2,j4,ipset)
      z2ab1234=zab2(j1,j2,j3,j4,ipset)
      z2ab3412=zab2(j3,j4,j1,j2,ipset)

      z2ba3412=zba2(j3,j4,j1,j2,ipset)
      z2ba1234=zba2(j1,j2,j3,j4,ipset)
      z2ba1243=zba2(j1,j2,j4,j3,ipset)
      z2ba3123=zba2(j3,j1,j2,j3,ipset)
      z2ba2134=zba2(j2,j1,j3,j4,ipset)
      z2ba4124=zba2(j4,j1,j2,j4,ipset)
      z3ab3241=zab3(j3,j2,j4,j1,ipset)

      s3s234=ss3(j2,j3,j4,ipset)
      s3s341=ss3(j3,j4,j1,ipset)
      s3s412=ss3(j4,j1,j2,ipset)

      CR4mpmpsubcl=2d0/(2d0*za12*za23*za34*za41)
     . *(-z3ab3241**3*za34*za21
     . /(3d0*za42*(s3s234-s23)**2)
     . -(z3ab3241*za34*za21/za42)**2
     . /(2d0*(s3s234-s23)))
     . *(1d0/s23+1d0/s3s234)
      return
      end
************************************************************************
      double complex function CR4mpmpsubnf(j1,j2,j3,j4)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
CW     arXiv:0804.4149v3, Eq.(5.2)
cw---                    (factor of C_\Gamma = 1/(16 pi^2) removed)
      integer j1,j2,j3,j4
      double precision ss3
      double complex zab3
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za34 => kin(ipset)%zA(j3,j4)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za43 => kin(ipset)%zA(j4,j3)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb34 => kin(ipset)%zB(j3,j4)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb43 => kin(ipset)%zB(j4,j3)

      z2ab1243=zab2(j1,j2,j4,j3,ipset)
      z2ab3123=zab2(j3,j1,j2,j3,ipset)
      z2ab2134=zab2(j2,j1,j3,j4,ipset)
      z2ab4124=zab2(j4,j1,j2,j4,ipset)
      z2ab1234=zab2(j1,j2,j3,j4,ipset)
      z2ab3412=zab2(j3,j4,j1,j2,ipset)

      z2ba3412=zba2(j3,j4,j1,j2,ipset)
      z2ba1234=zba2(j1,j2,j3,j4,ipset)
      z2ba1243=zba2(j1,j2,j4,j3,ipset)
      z2ba3123=zba2(j3,j1,j2,j3,ipset)
      z2ba2134=zba2(j2,j1,j3,j4,ipset)
      z2ba4124=zba2(j4,j1,j2,j4,ipset)
      z3ab3241=zab3(j3,j2,j4,j1,ipset)

      s3s234=ss3(j2,j3,j4,ipset)
      s3s341=ss3(j3,j4,j1,ipset)
      s3s412=ss3(j4,j1,j2,ipset)

      CR4mpmpsubnf=-2d0/(2d0*za12*za23*za34*za41)
     . *(-z3ab3241**3*za34*za21
     . /(3d0*za42*(s3s234-s23)**2)
     . -(z3ab3241*za34*za21/za42)**2
     . /(2d0*(s3s234-s23)))
     . *(1d0/s23+1d0/s3s234)
      return
      end
************************************************************************
      double complex function zAcphi4g1mpmpcl(PERM,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer perm(6)
      double complex C4mpmpmodcl,Rhat4mpmpmodcl,CR4mpmpmodcl,CR4mpmpmodnf,
     .               Rhat4mpmpmodnf

      zAcphi4g1mpmpcl=C4mpmpmodcl(perm(1),renscale2)
c     &             +CR4mpmpmodcl(perm(1))
*XC combined CR4mpmpmodcl with C4mpmpmodcl function by replacing the L2 and L3 functions in C4mpmpmodcl by L2hat and L3hat
*XC this could improve the numerical stability
     &             +Rhat4mpmpmodcl(perm(1))

      return
      end
************************************************************************
      double complex function zAcphi4g1mpmpnf(PERM,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer perm(6)
      double complex C4mpmpmodnf,Rhat4mpmpmodcl,CR4mpmpmodcl,CR4mpmpmodnf,
     .               Rhat4mpmpmodnf
      double precision renscale2

      zAcphi4g1mpmpnf=
     .             +C4mpmpmodnf(perm(1),renscale2)
c     &             +CR4mpmpmodnf(perm(1))
*XC combined CR4mpmpmodnf with C4mpmpmodnf function by replacing the L2 and L3 functions in C4mpmpmodnf by L2hat and L3hat
*XC this could improve the numerical stability
     &             +Rhat4mpmpmodnf(perm(1))

      return
      end
************************************************************************
      double complex function C4mpmpmodcl(PERM,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /order/ieorder
      integer j1,j2,j3,j4,j5,j6,i,ii(7),perm(6)
      double complex C4mpmpsubmodcl,zAphi4g0mpmp,F31m,F42me,F41m,sum
      double precision sc(4,4)
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      data ii/1,2,3,4,1,2,3/
      save ii

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      sc(1,1)=0d0
      sc(1,2)=s12
      sc(1,3)=s12+s13+s23
      sc(1,4)=s12+s13+s14+s23+s24+s34
      sc(2,1)=sc(1,4) !s(j1,j2)+s(j1,j3)+s(j1,j4)+s(j2,j3)+s(j2,j4)+s(j3,j4)
      sc(2,2)=0d0
      sc(2,3)=s23
      sc(2,4)=s23+s24+s34
      sc(3,1)=s34+s31+s41
      sc(3,2)=sc(1,4) !s(j1,j2)+s(j1,j3)+s(j1,j4)+s(j2,j3)+s(j2,j4)+s(j3,j4)
      sc(3,3)=0d0
      sc(3,4)=s34
      sc(4,1)=s41
      sc(4,2)=s41+s42+s12
      sc(4,3)=sc(1,4) !s(j1,j2)+s(j1,j3)+s(j1,j4)+s(j2,j3)+s(j2,j4)+s(j3,j4)
      sc(4,4)=0d0

      sum=(0d0,0d0)
      do i=1,4
      sum=sum
     . -0.5d0*F42me(sc(ii(i),ii(i+3)),sc(ii(i+1),ii(i+2))
     .             ,sc(ii(i+1),ii(i+3)),sc(ii(i),ii(i+2)),renscale2)
     . -0.5d0*F41m(sc(ii(i),ii(i+2)),sc(ii(i),ii(i+1)),
     .                               sc(ii(i+1),ii(i+2)),renscale2)
     . +F31m(sc(ii(i),ii(i+2)),renscale2)-F31m(sc(ii(i),ii(i+3)),renscale2)
      enddo

      if(ieorder.eq.-2)then
        C4mpmpmodcl=sum*conjg(zAphi4g0mpmp(PERM(1)))
      elseif(ieorder.eq.-1)then
        C4mpmpmodcl=sum*conjg(zAphi4g0mpmp(PERM(1)))
      elseif(ieorder.eq.0)then
        sum=sum
     .   +C4mpmpsubmodcl(j1,j2,j3,j4)
     .   +C4mpmpsubmodcl(j1,j4,j3,j2)
     .   +C4mpmpsubmodcl(j3,j2,j1,j4)
     .   +C4mpmpsubmodcl(j3,j4,j1,j2)
        C4mpmpmodcl=sum*conjg(zAphi4g0mpmp(PERM(1)))
      endif

      return
      end
************************************************************************
      double complex function C4mpmpmodnf(PERM,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /order/ieorder
      integer j1,j2,j3,j4,j5,j6,i,ii(7),perm(6)
      double complex C4mpmpsubmodnf,zAphi4g0mpmp,F31m,F42me,F41m,sum
      double precision sc(4,4)
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      data ii/1,2,3,4,1,2,3/
      save ii

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      sc(1,1)=0d0
      sc(1,2)=s12
      sc(1,3)=s12+s13+s23
      sc(1,4)=s12+s13+s14+s23+s24+s34
      sc(2,1)=sc(1,4) !s(j1,j2)+s(j1,j3)+s(j1,j4)+s(j2,j3)+s(j2,j4)+s(j3,j4)
      sc(2,2)=0d0
      sc(2,3)=s23
      sc(2,4)=s23+s24+s34
      sc(3,1)=s34+s31+41
      sc(3,2)=sc(1,4) !s(j1,j2)+s(j1,j3)+s(j1,j4)+s(j2,j3)+s(j2,j4)+s(j3,j4)
      sc(3,3)=0d0
      sc(3,4)=s34
      sc(4,1)=s41
      sc(4,2)=s41+s42+s12
      sc(4,3)=sc(1,4) !s(j1,j2)+s(j1,j3)+s(j1,j4)+s(j2,j3)+s(j2,j4)+s(j3,j4)
      sc(4,4)=0d0

      sum=(0d0,0d0)
      do i=1,4
      sum=sum
     . -0.5d0*F42me(sc(ii(i),ii(i+3)),sc(ii(i+1),ii(i+2))
     .             ,sc(ii(i+1),ii(i+3)),sc(ii(i),ii(i+2)),renscale2)
     . -0.5d0*F41m(sc(ii(i),ii(i+2)),sc(ii(i),ii(i+1)),
     .                               sc(ii(i+1),ii(i+2)),renscale2)
     . +F31m(sc(ii(i),ii(i+2)),renscale2)-F31m(sc(ii(i),ii(i+3)),renscale2)
      enddo

      if(ieorder.eq.-2)then
        C4mpmpmodnf=0d0
      elseif(ieorder.eq.-1)then
        C4mpmpmodnf=0d0
      elseif(ieorder.eq.0)then
        sum=
     .   +C4mpmpsubmodnf(j1,j2,j3,j4)
     .   +C4mpmpsubmodnf(j1,j4,j3,j2)
     .   +C4mpmpsubmodnf(j3,j2,j1,j4)
     .   +C4mpmpsubmodnf(j3,j4,j1,j2)
        C4mpmpmodnf=sum*conjg(zAphi4g0mpmp(PERM(1)))
      endif

      return
      end
************************************************************************
      double complex function C4mpmpsubmodcl(j1,j2,j3,j4)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer j1,j2,j3,j4
      double complex trmmod,trm3241,trm3421,BGRL3,BGRL2,BGRL1,F41mF
      double precision s234
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za34 => kin(ipset)%zA(j3,j4)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za43 => kin(ipset)%zA(j4,j3)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb34 => kin(ipset)%zB(j3,j4)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb43 => kin(ipset)%zB(j4,j3)

      s234=s23+s24+s34
      trm3241=zb32*za24*zb41*za13
      trm3421=zb34*za42*zb21*za13

      z2ab1243=zab2(j1,j2,j4,j3,ipset)
      z2ab3123=zab2(j3,j1,j2,j3,ipset)
      z2ab2134=zab2(j2,j1,j3,j4,ipset)
      z2ab4124=zab2(j4,j1,j2,j4,ipset)
      z2ab1234=zab2(j1,j2,j3,j4,ipset)
      z2ab3412=zab2(j3,j4,j1,j2,ipset)

      z2ba3412=zba2(j3,j4,j1,j2,ipset)
      z2ba1234=zba2(j1,j2,j3,j4,ipset)
      z2ba1243=zba2(j1,j2,j4,j3,ipset)
      z2ba3123=zba2(j3,j1,j2,j3,ipset)
      z2ba2134=zba2(j2,j1,j3,j4,ipset)
      z2ba4124=zba2(j4,j1,j2,j4,ipset)
      z3ab3241=zab3(j3,j2,j4,j1,ipset)

      s3s234=ss3(j2,j3,j4,ipset)
      s3s341=ss3(j3,j4,j1,ipset)
      s3s412=ss3(j4,j1,j2,ipset)

      C4mpmpsubmodcl=+4d0*(trm3241*trm3421/(2d0*(s24*s13)**2)
     . *F41mF(s234,s23,s34)
     . -trm3241*trm3421/(s24*s13**2)
     . *BGRL1(s23,s234))
     . +2d0*(-0.5d0*(trm3241*trm3421/(s24*s13)**2)**2
     . *F41mF(s234,s23,s34)
     . -trm3241*trm3421/s13**4
     . *(+trm3241**2/(3d0*s24)*BGRL3hat(s23,s234)
     . +trm3421*trm3241/(2d0*s24**2)*BGRL2hat(s23,s234)
     . -trm3421*trm3241/(s24**3)*BGRL1(s23,s234)))

      return
      end
************************************************************************
      double complex function C4mpmpsubmodnf(j1,j2,j3,j4)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer j1,j2,j3,j4
      double complex trmmod,trm3241,trm3421,BGRL3,BGRL2,BGRL1,F41mF
      double precision s234
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za34 => kin(ipset)%zA(j3,j4)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za43 => kin(ipset)%zA(j4,j3)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb34 => kin(ipset)%zB(j3,j4)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb43 => kin(ipset)%zB(j4,j3)

      s234=s23+s24+s34
      trm3241=zb32*za24*zb41*za13
      trm3421=zb34*za42*zb21*za13

      z2ab1243=zab2(j1,j2,j4,j3,ipset)
      z2ab3123=zab2(j3,j1,j2,j3,ipset)
      z2ab2134=zab2(j2,j1,j3,j4,ipset)
      z2ab4124=zab2(j4,j1,j2,j4,ipset)
      z2ab1234=zab2(j1,j2,j3,j4,ipset)
      z2ab3412=zab2(j3,j4,j1,j2,ipset)

      z2ba3412=zba2(j3,j4,j1,j2,ipset)
      z2ba1234=zba2(j1,j2,j3,j4,ipset)
      z2ba1243=zba2(j1,j2,j4,j3,ipset)
      z2ba3123=zba2(j3,j1,j2,j3,ipset)
      z2ba2134=zba2(j2,j1,j3,j4,ipset)
      z2ba4124=zba2(j4,j1,j2,j4,ipset)

      z3ab3241=zab3(j3,j2,j4,j1,ipset)

      s3s234=ss3(j2,j3,j4,ipset)
      s3s341=ss3(j3,j4,j1,ipset)
      s3s412=ss3(j4,j1,j2,ipset)

      C4mpmpsubmodnf=-(trm3241*trm3421/(2d0*(s24*s13)**2)
     . *F41mF(s234,s23,s34)
     . -trm3241*trm3421/(s24*s13**2)
     . *BGRL1(s23,s234))
     . -2d0*(-0.5d0*(trm3241*trm3421/(s24*s13)**2)**2
     . *F41mF(s234,s23,s34)
     . -trm3241*trm3421/s13**4
     . *(+trm3241**2/(3d0*s24)*BGRL3hat(s23,s234)
     . +trm3421*trm3241/(2d0*s24**2)*BGRL2hat(s23,s234)
     . -trm3421*trm3241/(s24**3)*BGRL1(s23,s234)))

      return
      end

************************************************************************
      Double complex function Rhat4mpmpmodcl(PERM)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /order/ieorder
      double complex im
      double precision ss3
      integer perm(6),perma(6),permb(6),j1,j2,j3,j4,j5,j6
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      perma(1)=j1
      perma(2)=j2
      perma(3)=j3
      perma(4)=j4
      perma(5)=j5
      perma(6)=j6

      permb(1)=j2
      permb(2)=j3
      permb(3)=j4
      permb(4)=j1
      permb(5)=j5
      permb(6)=j6

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za34 => kin(ipset)%zA(j3,j4)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za43 => kin(ipset)%zA(j4,j3)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb34 => kin(ipset)%zB(j3,j4)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb43 => kin(ipset)%zB(j4,j3)

      z2ab1243=zab2(j1,j2,j4,j3,ipset)
      z2ab3123=zab2(j3,j1,j2,j3,ipset)
      z2ab2134=zab2(j2,j1,j3,j4,ipset)
      z2ab4124=zab2(j4,j1,j2,j4,ipset)
      z2ab1234=zab2(j1,j2,j3,j4,ipset)
      z2ab3412=zab2(j3,j4,j1,j2,ipset)

      z2ba3412=zba2(j3,j4,j1,j2,ipset)
      z2ba1234=zba2(j1,j2,j3,j4,ipset)
      z2ba1243=zba2(j1,j2,j4,j3,ipset)
      z2ba3123=zba2(j3,j1,j2,j3,ipset)
      z2ba2134=zba2(j2,j1,j3,j4,ipset)
      z2ba4124=zba2(j4,j1,j2,j4,ipset)
      z3ab3241=zab3(j3,j2,j4,j1,ipset)

      s3s234=ss3(j2,j3,j4,ipset)
      s3s341=ss3(j3,j4,j1,ipset)
      s3s412=ss3(j4,j1,j2,ipset)

      im=(0d0,1d0)

      if(ieorder.eq.-2)then
        Rhat4mpmpmodcl=0d0
      elseif(ieorder.eq.-1)then
        Rhat4mpmpmodcl=0d0
      elseif(ieorder.eq.0)then
        Rhat4mpmpmodcl=
     .   -2d0*(-im)*(conjg(zAphi4g0mpmp(PERMa(1)))
     .              -zAphi4g0mpmp(PERMb(1)))
     .   +1d0/6d0*za24**4/(za12*za23*za34*za41)
     .   *(-s23*s34/(s24*s3s412)
     .     +3d0*s23*s34/s24**2
     .     -s12*s41/(s24*s3s234)
     .     +3d0*s12*s41/s24**2)
      endif

      return
      end
************************************************************************
      Double complex function Rhat4mpmpmodnf(PERM)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /order/ieorder
      double complex im
      double precision ss3
      integer perm(6),perma(6),permb(6),j1,j2,j3,j4,j5,j6
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za34 => kin(ipset)%zA(j3,j4)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za43 => kin(ipset)%zA(j4,j3)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb34 => kin(ipset)%zB(j3,j4)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb43 => kin(ipset)%zB(j4,j3)

      z2ab1243=zab2(j1,j2,j4,j3,ipset)
      z2ab3123=zab2(j3,j1,j2,j3,ipset)
      z2ab2134=zab2(j2,j1,j3,j4,ipset)
      z2ab4124=zab2(j4,j1,j2,j4,ipset)
      z2ab1234=zab2(j1,j2,j3,j4,ipset)
      z2ab3412=zab2(j3,j4,j1,j2,ipset)

      z2ba3412=zba2(j3,j4,j1,j2,ipset)
      z2ba1234=zba2(j1,j2,j3,j4,ipset)
      z2ba1243=zba2(j1,j2,j4,j3,ipset)
      z2ba3123=zba2(j3,j1,j2,j3,ipset)
      z2ba2134=zba2(j2,j1,j3,j4,ipset)
      z2ba4124=zba2(j4,j1,j2,j4,ipset)
      z3ab3241=zab3(j3,j2,j4,j1,ipset)

      s3s234=ss3(j2,j3,j4,ipset)
      s3s341=ss3(j3,j4,j1,ipset)
      s3s412=ss3(j4,j1,j2,ipset)

      im=(0d0,1d0)

      if(ieorder.eq.-2)then
        Rhat4mpmpmodnf=0d0
      elseif(ieorder.eq.-1)then
        Rhat4mpmpmodnf=0d0
      elseif(ieorder.eq.0)then
        Rhat4mpmpmodnf=
     .   -1d0/6d0*za24**4/(za12*za23*za34*za41)
     .   *(-s23*s34/(s24*s3s412)
     .     +3d0*s23*s34/s24**2
     .     -s12*s41/(s24*s3s234)
     .     +3d0*s12*s41/s24**2)
      endif

      return
      end
************************************************************************
      double complex function CR4mpmpmodcl(PERM)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer j1,j2,j3,j4,j5,j6,perm(6),ieorder
      double complex CR4mpmpsubmodcl
      common /order/ieorder

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      if(ieorder.eq.-2)then
        CR4mpmpmodcl=0d0
      elseif(ieorder.eq.-1)then
        CR4mpmpmodcl=0d0
      elseif(ieorder.eq.0)then
        CR4mpmpmodcl=
     .   +CR4mpmpsubmodcl(j1,j2,j3,j4)+CR4mpmpsubmodcl(j1,j4,j3,j2)
     .   +CR4mpmpsubmodcl(j3,j2,j1,j4)+CR4mpmpsubmodcl(j3,j4,j1,j2)
      endif

      return
      end
************************************************************************
      double complex function CR4mpmpmodnf(PERM)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer j1,j2,j3,j4,j5,j6,perm(6),ieorder
      double complex CR4mpmpsubmodnf
      common /order/ieorder

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      if(ieorder.eq.-2)then
        CR4mpmpmodnf=0d0
      elseif(ieorder.eq.-1)then
        CR4mpmpmodnf=0d0
      elseif(ieorder.eq.0)then
        CR4mpmpmodnf=
     .   +CR4mpmpsubmodnf(j1,j2,j3,j4)+CR4mpmpsubmodnf(j1,j4,j3,j2)
     .   +CR4mpmpsubmodnf(j3,j2,j1,j4)+CR4mpmpsubmodnf(j3,j4,j1,j2)
      endif

      return
      end
************************************************************************
      double complex function CR4mpmpsubmodcl(j1,j2,j3,j4)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer j1,j2,j3,j4
      double precision ss3
      double complex zab3
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za34 => kin(ipset)%zA(j3,j4)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za43 => kin(ipset)%zA(j4,j3)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb34 => kin(ipset)%zB(j3,j4)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb43 => kin(ipset)%zB(j4,j3)

      z2ab1243=zab2(j1,j2,j4,j3,ipset)
      z2ab3123=zab2(j3,j1,j2,j3,ipset)
      z2ab2134=zab2(j2,j1,j3,j4,ipset)
      z2ab4124=zab2(j4,j1,j2,j4,ipset)
      z2ab1234=zab2(j1,j2,j3,j4,ipset)
      z2ab3412=zab2(j3,j4,j1,j2,ipset)

      z2ba3412=zba2(j3,j4,j1,j2,ipset)
      z2ba1234=zba2(j1,j2,j3,j4,ipset)
      z2ba1243=zba2(j1,j2,j4,j3,ipset)
      z2ba3123=zba2(j3,j1,j2,j3,ipset)
      z2ba2134=zba2(j2,j1,j3,j4,ipset)
      z2ba4124=zba2(j4,j1,j2,j4,ipset)
      z3ab3241=zab3(j3,j2,j4,j1,ipset)
      z3ba3241=zba3(j3,j2,j4,j1,ipset)

      s3s234=ss3(j2,j3,j4,ipset)
      s3s341=ss3(j3,j4,j1,ipset)
      s3s412=ss3(j4,j1,j2,ipset)

      CR4mpmpsubmodcl=2d0/(2d0*zb12*zb23*zb34*zb41)
     . *(-z3ba3241**3*zb34*zb21
     . /(3d0*zb42*(s3s234-s23)**2)
     . -(z3ba3241*zb34*zb21/zb42)**2
     . /(2d0*(s3s234-s23)))
     . *(1d0/s23+1d0/s3s234)
      return
      end
************************************************************************
      double complex function CR4mpmpsubmodnf(j1,j2,j3,j4)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer j1,j2,j3,j4
      double precision ss3
      double complex zab3
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za34 => kin(ipset)%zA(j3,j4)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za43 => kin(ipset)%zA(j4,j3)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb34 => kin(ipset)%zB(j3,j4)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb43 => kin(ipset)%zB(j4,j3)

      z2ab1243=zab2(j1,j2,j4,j3,ipset)
      z2ab3123=zab2(j3,j1,j2,j3,ipset)
      z2ab2134=zab2(j2,j1,j3,j4,ipset)
      z2ab4124=zab2(j4,j1,j2,j4,ipset)
      z2ab1234=zab2(j1,j2,j3,j4,ipset)
      z2ab3412=zab2(j3,j4,j1,j2,ipset)

      z2ba3412=zba2(j3,j4,j1,j2,ipset)
      z2ba1234=zba2(j1,j2,j3,j4,ipset)
      z2ba1243=zba2(j1,j2,j4,j3,ipset)
      z2ba3123=zba2(j3,j1,j2,j3,ipset)
      z2ba2134=zba2(j2,j1,j3,j4,ipset)
      z2ba4124=zba2(j4,j1,j2,j4,ipset)
      z3ab3241=zab3(j3,j2,j4,j1,ipset)
      z3ba3241=zba3(j3,j2,j4,j1,ipset)

      s3s234=ss3(j2,j3,j4,ipset)
      s3s341=ss3(j3,j4,j1,ipset)
      s3s412=ss3(j4,j1,j2,ipset)

      CR4mpmpsubmodnf=-2d0/(2d0*zb12*zb23*zb34*zb41)
     . *(-z3ba3241**3*zb34*zb21
     . /(3d0*zb42*(s3s234-s23)**2)
     . -(z3ba3241*zb34*zb21/zb42)**2
     . /(2d0*(s3s234-s23)))
     . *(1d0/s23+1d0/s3s234)
      return
      end
*****************************************************************************************
*****************************************************************************************
*********************** Library for 1-loop sub layer amplitudes *************************
*****************************************************************************************
*****************************************************************************************
      double complex function FR4unsymcl(perm)
cw--- This function is the sum of the finite cut-constructible
cw--- and rational terms from arXiv:0909.4475 [hep-ph].
cw--- i.e. it is the sum of F4 in Eq. (5.12) and R4 in Eq. (5.13)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer j1,j2,j3,j4,j5,j6,j,perm(6)
      double complex zab2,W,W1,W2,W3,d1,d2,d3,
     . BGRL0,BGRL1,BGRL2hat,BGRL3hat,F41mF_BGMW,F33m,lnrat,gammap,gammam
      double precision mhsq,s123,s234,s134,s124
      double complex k1sq,k2sq,k1Dk2,gamma,factor,coef3mass,a1,a2,a3,a4
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za34 => kin(ipset)%zA(j3,j4)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za43 => kin(ipset)%zA(j4,j3)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb34 => kin(ipset)%zB(j3,j4)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb43 => kin(ipset)%zB(j4,j3)

      z2ab1243=zab2(j1,j2,j4,j3,ipset)
      z2ab3123=zab2(j3,j1,j2,j3,ipset)
      z2ab2134=zab2(j2,j1,j3,j4,ipset)
      z2ab4124=zab2(j4,j1,j2,j4,ipset)
      z2ab1234=zab2(j1,j2,j3,j4,ipset)
      z2ab3412=zab2(j3,j4,j1,j2,ipset)
      z2ab1342=zab2(j1,j3,j4,j2,ipset)
      z2ab2341=zab2(j2,j3,j4,j1,ipset)
      z2ab2143=zab2(j2,j1,j4,j3,ipset)
      z2ab3142=zab2(j3,j1,j4,j2,ipset)
      z2ab3241=zab2(j3,j2,j4,j1,ipset)
      z2ab3124=zab2(j3,j1,j2,j4,ipset)
      z2ab4123=zab2(j4,j1,j2,j3,ipset)
      z2ab4321=zab2(j4,j3,j2,j1,ipset)
      z2ab4231=zab2(j4,j2,j3,j1,ipset)

      z2ba3412=zba2(j3,j4,j1,j2,ipset)
      z2ba1234=zba2(j1,j2,j3,j4,ipset)
      z2ba1243=zba2(j1,j2,j4,j3,ipset)
      z2ba3123=zba2(j3,j1,j2,j3,ipset)
      z2ba2134=zba2(j2,j1,j3,j4,ipset)
      z2ba4124=zba2(j4,j1,j2,j4,ipset)

      z3ab3241=zab3(j3,j2,j4,j1,ipset)
      z3ba3241=zba3(j3,j2,j4,j1,ipset)

      s3s234=ss3(j2,j3,j4,ipset)
      s3s341=ss3(j3,j4,j1,ipset)
      s3s412=ss3(j4,j1,j2,ipset)

      mhsq=s12+s13+s14+s23+s24+s34
      s234=s23+s34+s24
      s134=s13+s34+s14
      s124=s12+s24+s14
      s123=s12+s23+s13
      W1=W(mhsq,s234,s12,s23,s34,s14)
      W2=W(mhsq,s134,s23,s34,s14,s12)
      W3=W(mhsq,s124,s34,s14,s12,s23)

      k1sq=dcmplx(mhsq)
      k2sq=dcmplx(s12)
      k1Dk2=-dcmplx(s12+0.5d0*(s13+s14+s23+s24))
      coef3mass=(0d0,0d0)

      gammap=k1Dk2+sqrt(k1Dk2**2-k1sq*k2sq)
      gammam=2d0*k1Dk2-gammap

      factor=gammap/(gammap**2-k1sq*k2sq)
      a1=factor*(-gammap-k1sq)
      a2=a1
      a3=factor*(-gammap)
      a4=a3

*XC rewrite (a2*s6(j1,j2)+a3*s6(j1,j3)+a4*s6(j1,j4))=d1K=d1 for better numerical stability in soft limit
      d1=-factor*s12*(gammap+mhsq+mhsq*(s13+s14)/gammam)
*XC rewrite (a1*s6(j1,j3)+a2*s6(j2,j3)+a4*s6(j3,j4))=d3K=d2 for better numerical stability in soft limit
      d2=-factor*s34*(mhsq*(s13+s23)/(gammam+mhsq)+gammap)
*XC rewrite (a1*s6(j1,j4)+a2*s6(j2,j4)+a3*s6(j3,j4))=d4K=d3 for better numerical stability in soft limit
      d3=-factor*s34*(mhsq*(s14+s24)/(gammam+mhsq)+gammap)

      coef3mass=coef3mass+mhsq**2*za34**3
     &  *(+a3*za23*zb31+a4*za24*zb41)
     &  *(+a1*za21*zb13+a4*za24*zb43)
     &  *(+a1*za21*zb14+a3*za23*zb34)
     & /(gammap*(gammap+mhsq)*za12
     &  *d1   !D1
     &  *d2   !D2
     &  *d3   !D3
     .  )

      factor=gammam/(gammam**2-k1sq*k2sq)
      a1=factor*(-gammam-k1sq)
      a2=a1
      a3=factor*(-gammam)
      a4=a3

*XC rewrite (a1*s6(j1,j4)+a2*s6(j2,j4)+a3*s6(j3,j4))=d4K=d3 for better numerical stability in soft limit
      d3=-factor*s34*(mhsq*(s14+s24)/(gammap+mhsq)+gammam)
*XC rewrite (a1*s6(j1,j3)+a2*s6(j2,j3)+a4*s6(j3,j4))=d3K=d2 for better numerical stability in soft limit
      d2=-factor*s34*(mhsq*(s13+s23)/(gammap+mhsq)+gammam)
*XC rewrite (a2*s6(j1,j2)+a3*s6(j1,j3)+a4*s6(j1,j4))=d1K=d1 for better numerical stability in soft limit
      d1=-factor*s12*(gammam+mhsq+mhsq*(s13+s14)/gammap)

      coef3mass=coef3mass+mhsq**2*za34**3
     &  *(+a3*za23*zb31+a4*za24*zb41)
     &  *(+a1*za21*zb13+a4*za24*zb43)
     &  *(+a1*za21*zb14+a3*za23*zb34)
     & /(gammam*(gammam+mhsq)*za12
     &  *d1   !D1
     &  *d2   !D2
     &  *d3   !D3
     .  )

cw---  Eq.(5.12)
      FR4unsymcl=-s234**3
     . /(4d0*z2ab1342*z2ab1234*zb23*zb34)*W1
     .   *(-1d0)

     . +(z2ab2341**3
     . /(2d0*s134*z2ab2143*zb34*zb41)
     .  +za34**3*mhsq**2
     .  /(2d0*s134*z2ab1342*z2ab3142*za41))*W2

     . +0.25d0/s124
     . *(z2ab3241**4
     . /(z2ab3142*z2ab3124*zb21*zb41)
     . + za24**4*mhsq**2
     . /(za12*za14*z2ab2143*z2ab4123))*W3
     .   *(-1d0)*(1d0)

     . +coef3mass*F33m(mhsq,s12,s34)

      FR4unsymcl=FR4unsymcl
     . +(2d0*z2ab3241**2/(s124*zb24**2)
     . *F41mF_BGMW(s124,s12,s14)*(-0.5d0)
     . +4d0*za24*z2ab3241**2/(s124*zb42)
     . *BGRL1(s124,s12)
     . -4d0*za23*z2ab4321**2/(s123*zb32)
     . *BGRL1(s123,s12))

      FR4unsymcl=FR4unsymcl
     . +(zb12*zb41*z2ab3142*z2ab3124
     . /(2d0*s124*zb24**4)*F41mF_BGMW(s124,s12,s14)

     . +(za34*zb41)**2/zb42**2/3d0*(
     . s24*s12*(2d0*BGRL3hat(s124,s12)+1d0/s124*BGRL2hat(s124,s12))
     . +3d0*s12*(BGRL2hat(s124,s12)+1d0/s124*BGRL1(s124,s12))
     . +6d0*s12**2/s24/s124*BGRL1(s124,s12))
     . -za34*zb41*za32*zb21/zb42**2/3d0*(
     . +s24*(BGRL2hat(s124,s12)+2d0/s124*BGRL1(s124,s12))
     . +6d0/s124*BGRL0(s124,s12)
     . +12d0*s12/s24/s124*BGRL0(s124,s12))
     . +(za32*zb21)**2/zb42**2/3d0*(
     . -s24/s124*BGRL1(s124,s12)
     . +6d0*s14/s24/s124*BGRL0(s124,s12)
     . +3d0/s124*BGRL0(s124,s12))
*XC above block is the rewrited result of BGRL3hat(s124,s12), BGRL2hat(s124,s12), BGRL1hat(s124,s12), BGRL0hat(s124,s12) terms
*XC in (5.12) of 0909.4475v2.

     . -2d0*s123*za23*(za34*zb31)**2/(3d0*zb32)
     . *BGRL3hat(s123,s12)
     . -za23*za34*zb31*z2ab4231/(3d0*zb32)
     . *BGRL2hat(s123,s12)
     . +za23*z2ab4231**2/(3d0*s123*zb32)
     . *BGRL1(s123,s12))

c-----Add rational piece Eq.(5.13)
      FR4unsymcl=FR4unsymcl
     . -0.5d0*(-za23*za34*z2ab4231*zb31
     . /(3d0*s123*za12*zb21*zb32)
     . -z2ab3241**2/(s124*zb42**2)

     . -za24*za34*za32*zb21*zb41
     . /(3d0*s124*s12*zb42)
     . +za24*(za34*zb41)**2
     . /(3d0*s124*s124*zb42)
     . +(zb14*za43)**2/(s124*zb42**2)
*XC above block is the combination of the third and fourth term in (5.13) of 0909.4475

     . -za24*(s23*s24+s23*s34+s24*s34)
     . /(3d0*za12*za14*zb23*zb34*zb42)
     . +z2ab2341*z2ab4231
     . /(3d0*s234*zb23*zb34)
     . -2d0*zb12*za23*zb31**2
     . /(3d0*zb23**2*zb41*zb34))

      return
      end
************************************************************************
      double complex function FR4unsymnf(perm)
cw--- This function is the sum of the finite cut-constructible
cw--- and rational terms from arXiv:0909.4475 [hep-ph].
cw--- i.e. it is the sum of F4 in Eq. (5.12) and R4 in Eq. (5.13)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer j1,j2,j3,j4,j5,j6,j,perm(6)
      double complex zab2,W,W1,W2,W3,
     . BGRL0,BGRL1,BGRL2hat,BGRL3hat,F41mF_BGMW,F33m,lnrat
      double precision mhsq,s123,s234,s134,s124
      double complex k1sq,k2sq,k1Dk2,gamma,factor,coef3mass,a1,a2,a3,a4
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za34 => kin(ipset)%zA(j3,j4)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za43 => kin(ipset)%zA(j4,j3)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb34 => kin(ipset)%zB(j3,j4)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb43 => kin(ipset)%zB(j4,j3)

      z2ab1243=zab2(j1,j2,j4,j3,ipset)
      z2ab3123=zab2(j3,j1,j2,j3,ipset)
      z2ab2134=zab2(j2,j1,j3,j4,ipset)
      z2ab4124=zab2(j4,j1,j2,j4,ipset)
      z2ab1234=zab2(j1,j2,j3,j4,ipset)
      z2ab3412=zab2(j3,j4,j1,j2,ipset)
      z2ab1342=zab2(j1,j3,j4,j2,ipset)
      z2ab2341=zab2(j2,j3,j4,j1,ipset)
      z2ab2143=zab2(j2,j1,j4,j3,ipset)
      z2ab3142=zab2(j3,j1,j4,j2,ipset)
      z2ab3241=zab2(j3,j2,j4,j1,ipset)
      z2ab3124=zab2(j3,j1,j2,j4,ipset)
      z2ab4123=zab2(j4,j1,j2,j3,ipset)
      z2ab4321=zab2(j4,j3,j2,j1,ipset)
      z2ab4231=zab2(j4,j2,j3,j1,ipset)

      z2ba3412=zba2(j3,j4,j1,j2,ipset)
      z2ba1234=zba2(j1,j2,j3,j4,ipset)
      z2ba1243=zba2(j1,j2,j4,j3,ipset)
      z2ba3123=zba2(j3,j1,j2,j3,ipset)
      z2ba2134=zba2(j2,j1,j3,j4,ipset)
      z2ba4124=zba2(j4,j1,j2,j4,ipset)
      z3ab3241=zab3(j3,j2,j4,j1,ipset)
      z3ba3241=zba3(j3,j2,j4,j1,ipset)

      s3s234=ss3(j2,j3,j4,ipset)
      s3s341=ss3(j3,j4,j1,ipset)
      s3s412=ss3(j4,j1,j2,ipset)

      mhsq=s12+s13+s14+s23+s24+s34
      s234=s23+s34+s24
      s134=s13+s34+s14
      s124=s12+s24+s14
      s123=s12+s23+s13

cw---  Eq.(5.12)
      FR4unsymnf=
     . -1d0/4d0
     . *(2d0*z2ab3241**2/(s124*zb24**2)
     . *F41mF_BGMW(s124,s12,s14)*(-0.5d0)
     . +4d0*za24*z2ab3241**2/(s124*zb42)
     . *BGRL1(s124,s12)
     . -4d0*za23*z2ab4321**2/(s123*zb32)
     . *BGRL1(s123,s12))

      FR4unsymnf=FR4unsymnf
     . -(zb12*zb41*z2ab3142*z2ab3124
     . /(2d0*s124*zb24**4)*F41mF_BGMW(s124,s12,s14)

     . +(za34*zb41)**2/zb42**2/3d0*(
     . s24*s12*(2d0*BGRL3hat(s124,s12)+1d0/s124*BGRL2hat(s124,s12))
     . +3d0*s12*(BGRL2hat(s124,s12)+1d0/s124*BGRL1(s124,s12))
     . +6d0*s12**2/s24/s124*BGRL1(s124,s12))
     . -za34*zb41*za32*zb21/zb42**2/3d0*(
     . +s24*(BGRL2hat(s124,s12)+2d0/s124*BGRL1(s124,s12))
     . +6d0/s124*BGRL0(s124,s12)
     . +12d0*s12/s24/s124*BGRL0(s124,s12))
     . +(za32*zb21)**2/zb42**2/3d0*(
     . -s24/s124*BGRL1(s124,s12)
     . +6d0*s14/s24/s124*BGRL0(s124,s12)
     . +3d0/s124*BGRL0(s124,s12))
*XC above block is the rewrited result of BGRL3hat(s124,s12), BGRL2hat(s124,s12), BGRL1hat(s124,s12), BGRL0hat(s124,s12) terms
*XC in (5.12) of 0909.4475v2. New rational terms from the rewritting is moved to the rational block in below.

     . -2d0*s123*za23*(za34*zb31)**2/(3d0*zb32)
     . *BGRL3hat(s123,s12)
     . -za23*za34*zb31*z2ab4231/(3d0*zb32)
     . *BGRL2hat(s123,s12)
     . +za23*z2ab4231**2/(3d0*s123*zb32)
     . *BGRL1(s123,s12))

c-----Add rational piece Eq.(5.13)
      FR4unsymnf=FR4unsymnf
     . +0.5d0*(-za23*za34*z2ab4231*zb31
     . /(3d0*s123*za12*zb21*zb32)
     . -z2ab3241**2/(s124*zb42**2)

     . -za24*za34*za32*zb21*zb41
     . /(3d0*s124*s12*zb42)
     . +za24*(za34*zb41)**2
     . /(3d0*s124*s124*zb42)
     . +(zb14*za43)**2/(s124*zb42**2)
*XC above block is the combination of the third and fourth term in (5.13) of 0909.4475

     . -za24*(s23*s24+s23*s34+s24*s34)
     . /(3d0*za12*za14*zb23*zb34*zb42)
     . +z2ab2341*z2ab4231
     . /(3d0*s234*zb23*zb34)
     . -2d0*zb12*za23*zb31**2
     . /(3d0*zb23**2*zb41*zb34))

      return
      end
************************************************************************
      double complex function FR4unsymmodcl(perm)
*XC This function is the FR4unsym function with za and zb getting swapped
*XC This function for the purpose of generating -+++ helicity configurations
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer j1,j2,j3,j4,j5,j6,j,perm(6)
      double complex zab2,W,W1,W2,W3,
     . BGRL0,BGRL1,BGRL2hat,BGRL3hat,F41mF_BGMW,F33m,lnrat,zf33m,zf41mf
      double precision mhsq,s123,s234,s134,s124
      double complex k1sq,k2sq,k1Dk2,gamma,factor,coef3massmod,a1,a2,a3,a4
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za34 => kin(ipset)%zA(j3,j4)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za43 => kin(ipset)%zA(j4,j3)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb34 => kin(ipset)%zB(j3,j4)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb43 => kin(ipset)%zB(j4,j3)

      z2ab1243=zab2(j1,j2,j4,j3,ipset)
      z2ab3123=zab2(j3,j1,j2,j3,ipset)
      z2ab2134=zab2(j2,j1,j3,j4,ipset)
      z2ab4124=zab2(j4,j1,j2,j4,ipset)
      z2ab1234=zab2(j1,j2,j3,j4,ipset)
      z2ab3412=zab2(j3,j4,j1,j2,ipset)
      z2ab1342=zab2(j1,j3,j4,j2,ipset)
      z2ab2341=zab2(j2,j3,j4,j1,ipset)
      z2ab2143=zab2(j2,j1,j4,j3,ipset)
      z2ab3142=zab2(j3,j1,j4,j2,ipset)
      z2ab3241=zab2(j3,j2,j4,j1,ipset)
      z2ab3124=zab2(j3,j1,j2,j4,ipset)
      z2ab4123=zab2(j4,j1,j2,j3,ipset)
      z2ab4321=zab2(j4,j3,j2,j1,ipset)
      z2ab4231=zab2(j4,j2,j3,j1,ipset)

      z2ba3412=zba2(j3,j4,j1,j2,ipset)
      z2ba1234=zba2(j1,j2,j3,j4,ipset)
      z2ba1243=zba2(j1,j2,j4,j3,ipset)
      z2ba3123=zba2(j3,j1,j2,j3,ipset)
      z2ba2134=zba2(j2,j1,j3,j4,ipset)
      z2ba4124=zba2(j4,j1,j2,j4,ipset)
      z2ba1342=zba2(j1,j3,j4,j2,ipset)
      z2ba2341=zba2(j2,j3,j4,j1,ipset)
      z2ba2143=zba2(j2,j1,j4,j3,ipset)
      z2ba3142=zba2(j3,j1,j4,j2,ipset)
      z2ba3241=zba2(j3,j2,j4,j1,ipset)
      z2ba3124=zba2(j3,j1,j2,j4,ipset)
      z2ba4123=zba2(j4,j1,j2,j3,ipset)
      z2ba4321=zba2(j4,j3,j2,j1,ipset)
      z2ba4231=zba2(j4,j2,j3,j1,ipset)
      z3ab3241=zab3(j3,j2,j4,j1,ipset)
      z3ba3241=zba3(j3,j2,j4,j1,ipset)

      s3s234=ss3(j2,j3,j4,ipset)
      s3s341=ss3(j3,j4,j1,ipset)
      s3s412=ss3(j4,j1,j2,ipset)

      mhsq=s12+s13+s14+s23+s24+s34
      s234=s23+s34+s24
      s134=s13+s34+s14
      s124=s12+s24+s14
      s123=s12+s23+s13

      W1=W(mhsq,s234,s12,s23,s34,s14)
      W2=W(mhsq,s134,s23,s34,s14,s12)
      W3=W(mhsq,s124,s34,s14,s12,s23)

      k1sq=dcmplx(mhsq)
      k2sq=dcmplx(s12)
      k1Dk2=-dcmplx(s12+0.5d0*(s13+s14+s23+s24))
      coef3massmod=(0d0,0d0)

      gammap=k1Dk2+sqrt(k1Dk2**2-k1sq*k2sq)
      gammam=2d0*k1Dk2-gammap

      factor=gammap/(gammap**2-k1sq*k2sq)
      a1=factor*(-gammap-k1sq)
      a2=a1
      a3=factor*(-gammap)
      a4=a3

*XC rewrite (a2*s6(j1,j2)+a3*s6(j1,j3)+a4*s6(j1,j4))=d1K=d1 for better numerical stability in soft limit
      d1=-factor*s12*(gammap+mhsq+mhsq*(s13+s14)/gammam)
*XC rewrite (a1*s6(j1,j3)+a2*s6(j2,j3)+a4*s6(j3,j4))=d3K=d2 for better numerical stability in soft limit
      d2=-factor*s34*(mhsq*(s13+s23)/(gammam+mhsq)+gammap)
*XC rewrite (a1*s6(j1,j4)+a2*s6(j2,j4)+a3*s6(j3,j4))=d4K=d3 for better numerical stability in soft limit
      d3=-factor*s34*(mhsq*(s14+s24)/(gammam+mhsq)+gammap)

      coef3massmod=coef3massmod+mhsq**2*zb34**3
     &  *(+a3*zb23*za31+a4*zb24*za41)
     &  *(+a1*zb21*za13+a4*zb24*za43)
     &  *(+a1*zb21*za14+a3*zb23*za34)
     & /(gammap*(gammap+mhsq)*zb12
     &  *d1
     &  *d2
     &  *d3
     .  )

      factor=gammam/(gammam**2-k1sq*k2sq)
      a1=factor*(-gammam-k1sq)
      a2=a1
      a3=factor*(-gammam)
      a4=a3

*XC rewrite (a1*s6(j1,j4)+a2*s6(j2,j4)+a3*s6(j3,j4))=d4K=d3 for better numerical stability in soft limit
      d3=-factor*s34*(mhsq*(s14+s24)/(gammap+mhsq)+gammam)
*XC rewrite (a1*s6(j1,j3)+a2*s6(j2,j3)+a4*s6(j3,j4))=d2 for better numerical stability in soft limit
      d2=-factor*s34*(mhsq*(s13+s23)/(gammap+mhsq)+gammam)
*XC rewrite (a2*s6(j1,j2)+a3*s6(j1,j3)+a4*s6(j1,j4))=d1K=d1 for better numerical stability in soft limit
      d1=-factor*s12*(gammam+mhsq+mhsq*(s13+s14)/gammap)

      coef3massmod=coef3massmod+mhsq**2*zb34**3
     &  *(+a3*zb23*za31+a4*zb24*za41)
     &  *(+a1*zb21*za13+a4*zb24*za43)
     &  *(+a1*zb21*za14+a3*zb23*za34)
     & /(gammam*(gammam+mhsq)*zb12
     &  *d1
     &  *d2
     &  *d3
     .  )

      FR4unsymmodcl=-s234**3
     . /(4d0*z2ba1342*z2ba1234*za23*za34)*W1
     .   *(-1d0)

     . +(z2ba2341**3
     . /(2d0*s134*z2ba2143*za34*za41)
     .  +zb34**3*mhsq**2
     .  /(2d0*s134*z2ba1342*z2ba3142*zb41))*W2

     . +0.25d0/s124
     . *(z2ba3241**4
     . /(z2ba3142*z2ba3124*za21*za41)
     . + zb24**4*mhsq**2
     . /(zb12*zb14*z2ba2143*z2ba4123))*W3
     .   *(-1d0)*(1d0)

     . +coef3massmod*F33m(mhsq,s12,s34)

      FR4unsymmodcl=FR4unsymmodcl
     . +(2d0*z2ba3241**2/(s124*za24**2)
     . *F41mF_BGMW(s124,s12,s14)*(-0.5d0)
     . +4d0*zb24*z2ba3241**2/(s124*za42)
     . *BGRL1(s124,s12)
     . -4d0*zb23*z2ba4321**2/(s123*za32)
     . *BGRL1(s123,s12))

      FR4unsymmodcl=FR4unsymmodcl
     . +(za12*za41*z2ba3142*z2ba3124
     . /(2d0*s124*za24**4)*F41mF_BGMW(s124,s12,s14)

     . +(zb34*za41)**2/za42**2/3d0*(
     . s24*s12*(2d0*BGRL3hat(s124,s12)+1d0/s124*BGRL2hat(s124,s12))
     . +3d0*s12*(BGRL2hat(s124,s12)+1d0/s124*BGRL1(s124,s12))
     . +6d0*s12**2/s24/s124*BGRL1(s124,s12))
     . -zb34*za41*zb32*za21/za42**2/3d0*(
     . +s24*(BGRL2hat(s124,s12)+2d0/s124*BGRL1(s124,s12))
     . +6d0/s124*BGRL0(s124,s12)
     . +12d0*s12/s24/s124*BGRL0(s124,s12))
     . +(zb32*za21)**2/za42**2/3d0*(
     . -s24/s124*BGRL1(s124,s12)
     . +6d0*s14/s24/s124*BGRL0(s124,s12)
     . +3d0/s124*BGRL0(s124,s12))
*XC above block is the rewrited result of BGRL3hat(s124,s12), BGRL2hat(s124,s12), BGRL1hat(s124,s12), BGRL0hat(s124,s12) terms
*XC in (5.12) of 0909.4475v2. New rational terms from the rewritting is moved to the rational block in below.

     . -2d0*s123*zb23*(zb34*za31)**2/(3d0*za32)
     . *BGRL3hat(s123,s12)
     . -zb23*zb34*za31*z2ba4231/(3d0*za32)
     . *BGRL2hat(s123,s12)
     . +zb23*z2ba4231**2/(3d0*s123*za32)
     . *BGRL1(s123,s12))

      FR4unsymmodcl=FR4unsymmodcl
     . -0.5d0*(-zb23*zb34*z2ba4231*za31
     . /(3d0*s123*zb12*za21*za32)
     . -z2ba3241**2/(s124*za42**2)

     . -zb24*zb34*zb32*za21*za41
     . /(3d0*s12*s124*za42)
     . +zb24*(zb34*za41)**2
     . /(3d0*s124*s124*za42)
     . +(za14*zb43)**2/(s124*za42**2)
*XC above block is the combination of the third and fourth term in (5.13) of 0909.4475

     . -zb24*(s23*s24+s23*s34+s24*s34)
     . /(3d0*zb12*zb14*za23*za34*za42)
     . +z2ba2341*z2ba4231
     . /(3d0*s234*za23*za34)
     . -2d0*za12*zb23*za31**2
     . /(3d0*za23**2*za41*za34))

      return
      end
************************************************************************
      double complex function FR4unsymmodnf(perm)
*XC This function is the FR4unsym function with za and zb getting swapped
*XC This function for the purpose of generating -+++ helicity configurations
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer j1,j2,j3,j4,j5,j6,j,perm(6)
      double complex zab2,W,W1,W2,W3,
     . BGRL0,BGRL1,BGRL2hat,BGRL3hat,F41mF_BGMW,F33m,lnrat,zf33m,zf41mf
      double precision mhsq,s123,s234,s134,s124
      double complex k1sq,k2sq,k1Dk2,gamma,factor,coef3massmod,a1,a2,a3,a4
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za34 => kin(ipset)%zA(j3,j4)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za43 => kin(ipset)%zA(j4,j3)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb34 => kin(ipset)%zB(j3,j4)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb43 => kin(ipset)%zB(j4,j3)

      z2ab1243=zab2(j1,j2,j4,j3,ipset)
      z2ab3123=zab2(j3,j1,j2,j3,ipset)
      z2ab2134=zab2(j2,j1,j3,j4,ipset)
      z2ab4124=zab2(j4,j1,j2,j4,ipset)
      z2ab1234=zab2(j1,j2,j3,j4,ipset)
      z2ab3412=zab2(j3,j4,j1,j2,ipset)
      z2ab1342=zab2(j1,j3,j4,j2,ipset)
      z2ab2341=zab2(j2,j3,j4,j1,ipset)
      z2ab2143=zab2(j2,j1,j4,j3,ipset)
      z2ab3142=zab2(j3,j1,j4,j2,ipset)
      z2ab3241=zab2(j3,j2,j4,j1,ipset)
      z2ab3124=zab2(j3,j1,j2,j4,ipset)
      z2ab4123=zab2(j4,j1,j2,j3,ipset)
      z2ab4321=zab2(j4,j3,j2,j1,ipset)
      z2ab4231=zab2(j4,j2,j3,j1,ipset)

      z2ba3412=zba2(j3,j4,j1,j2,ipset)
      z2ba1234=zba2(j1,j2,j3,j4,ipset)
      z2ba1243=zba2(j1,j2,j4,j3,ipset)
      z2ba3123=zba2(j3,j1,j2,j3,ipset)
      z2ba2134=zba2(j2,j1,j3,j4,ipset)
      z2ba4124=zba2(j4,j1,j2,j4,ipset)
      z2ba1342=zba2(j1,j3,j4,j2,ipset)
      z2ba2341=zba2(j2,j3,j4,j1,ipset)
      z2ba2143=zba2(j2,j1,j4,j3,ipset)
      z2ba3142=zba2(j3,j1,j4,j2,ipset)
      z2ba3241=zba2(j3,j2,j4,j1,ipset)
      z2ba3124=zba2(j3,j1,j2,j4,ipset)
      z2ba4123=zba2(j4,j1,j2,j3,ipset)
      z2ba4321=zba2(j4,j3,j2,j1,ipset)
      z2ba4231=zba2(j4,j2,j3,j1,ipset)
      z3ab3241=zab3(j3,j2,j4,j1,ipset)
      z3ba3241=zba3(j3,j2,j4,j1,ipset)

      s3s234=ss3(j2,j3,j4,ipset)
      s3s341=ss3(j3,j4,j1,ipset)
      s3s412=ss3(j4,j1,j2,ipset)

      mhsq=s12+s13+s14+s23+s24+s34
      s234=s23+s34+s24
      s134=s13+s34+s14
      s124=s12+s24+s14
      s123=s12+s23+s13

      FR4unsymmodnf=
     . +(-1d0/4d0)
     . *(2d0*z2ba3241**2/(s124*za24**2)
     . *F41mF_BGMW(s124,s12,s14)*(-0.5d0)
     . +4d0*zb24*z2ba3241**2/(s124*za42)
     . *BGRL1(s124,s12)
     . -4d0*zb23*z2ba4321**2/(s123*za32)
     . *BGRL1(s123,s12))

      FR4unsymmodnf=FR4unsymmodnf
     . -(za12*za41*z2ba3142*z2ba3124
     . /(2d0*s124*za24**4)*F41mF_BGMW(s124,s12,s14)

     . +(zb34*za41)**2/za42**2/3d0*(
     . s24*s12*(2d0*BGRL3hat(s124,s12)+1d0/s124*BGRL2hat(s124,s12))
     . +3d0*s12*(BGRL2hat(s124,s12)+1d0/s124*BGRL1(s124,s12))
     . +6d0*s12**2/s24/s124*BGRL1(s124,s12))
     . -zb34*za41*zb32*za21/za42**2/3d0*(
     . +s24*(BGRL2hat(s124,s12)+2d0/s124*BGRL1(s124,s12))
     . +6d0/s124*BGRL0(s124,s12)
     . +12d0*s12/s24/s124*BGRL0(s124,s12))
     . +(zb32*za21)**2/za42**2/3d0*(
     . -s24/s124*BGRL1(s124,s12)
     . +6d0*s14/s24/s124*BGRL0(s124,s12)
     . +3d0/s124*BGRL0(s124,s12))
*XC above block is the rewrited result of BGRL3hat(s124,s12), BGRL2hat(s124,s12), BGRL1hat(s124,s12), BGRL0hat(s124,s12) terms
*XC in (5.12) of 0909.4475v2. New rational terms from the rewritting is moved to the rational block in below.

     . -2d0*s123*zb23*(zb34*za31)**2/(3d0*za32)
     . *BGRL3hat(s123,s12)
     . -zb23*zb34*za31*z2ba4231/(3d0*za32)
     . *BGRL2hat(s123,s12)
     . +zb23*z2ba4231**2/(3d0*s123*za32)
     . *BGRL1(s123,s12))

      FR4unsymmodnf=FR4unsymmodnf
     . +0.5d0*(-zb23*zb34*z2ba4231*za31
     . /(3d0*s123*zb12*za21*za32)
     . -z2ba3241**2/(s124*za42**2)

     . -zb24*zb34*zb32*za21*za41
     . /(3d0*s12*s124*za42)
     . +zb24*(zb34*za41)**2
     . /(3d0*s124*s124*za42)
     . +(za14*zb43)**2/(s124*za42**2)
*XC above block is the combination of the third and fourth term in (5.13) of 0909.4475

     . -zb24*(s23*s24+s23*s34+s24*s34)
     . /(3d0*zb12*zb14*za23*za34*za42)
     . +z2ba2341*z2ba4231
     . /(3d0*s234*za23*za34)
     . -2d0*za12*zb23*za31**2
     . /(3d0*za23**2*za41*za34))

      return
      end


