************************************************************************
!XC 1-loop amplitudes in this file are based on MCFM-6.8 source code
!  J.~M.~Campbell and R.~K.~Ellis,
!  %``An Update on vector boson pair production at hadron colliders,''
!  Phys.\ Rev.\ D {\bf 60} (1999) 113006
!  doi:10.1103/PhysRevD.60.113006
!  [hep-ph/9905386].

!  J.~M.~Campbell, R.~K.~Ellis and W.~T.~Giele,
!  %``A Multi-Threaded Version of MCFM,''
!  Eur.\ Phys.\ J.\ C {\bf 75} (2015) no.6,  246
!  doi:10.1140/epjc/s10052-015-3461-2
!  [arXiv:1503.06182 [physics.comp-ph]].

!XC The corresponding reference for 1-loop amplitudes are 
!  S.~Badger, J.~M.~Campbell, R.~K.~Ellis and C.~Williams,
!  %``Analytic results for the one-loop NMHV Hqqgg amplitude,''
!  JHEP {\bf 0912} (2009) 035
!  doi:10.1088/1126-6708/2009/12/035
!  [arXiv:0910.4481 [hep-ph]].

!  S.~Badger, E.~W.~Nigel Glover, P.~Mastrolia and C.~Williams,
!  %``One-loop Higgs plus four gluon amplitudes: Full analytic results,''
!  JHEP {\bf 1001} (2010) 036
!  doi:10.1007/JHEP01(2010)036
!  [arXiv:0909.4475 [hep-ph]].

!  E.~W.~N.~Glover, P.~Mastrolia and C.~Williams,
!  %``One-loop phi-MHV amplitudes using the unitarity bootstrap: The General helicity case,''
!  JHEP {\bf 0808} (2008) 017
!  doi:10.1088/1126-6708/2008/08/017
!  [arXiv:0804.4149 [hep-ph]].

!  S.~D.~Badger, E.~W.~N.~Glover and K.~Risager,
!  %``One-loop phi-MHV amplitudes using the unitarity bootstrap,''
!  JHEP {\bf 0707} (2007) 066
!  doi:10.1088/1126-6708/2007/07/066
!  [arXiv:0704.3914 [hep-ph]].

!XC To improve numerical stability, more details in this code are explained in
!  X.~Chen,
!  %``Hadronic Production of a Higgs Boson in Association with a Jet at Next-to-Next-to-Leading Order,''
!  http://etheses.dur.ac.uk/11158/1/PhDthesisXuanChen.pdf 
************************************************************************
* 2 quark 1 gluon one higgs 1-loop level matrix elements
*
* summed over colour orderings, summed and squared
*
* Using the result in MCFM for different helicity configuration
************************************************************************
      function FullB1g1H(iq1,i3,iqbar2,i4,i5,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /order/ieorder
      double precision virtgg,virtqa,virtaq,virtqg,virtgq,
     . logg,loqa,loqg,logq,ddilog,Li2s,Li2t,Li2u,
     . lnm,lns,lnt,lnu,ln2t,ln2u,mhsq,s,t,u,xlf,subuv,Delta
     . xn,epivn,pisq,V,cf,musq

      stop "set value for chgg correctly!"
      fac = (4d0*pi*as)**2d0*nc*(nc**2d0-1d0)*chgg**2d0/4d0
      musq=renscale2

      FullB1g1H=
     .         +B1g1H(iq1,i3,iqbar2,i4,i5,renscale2)
     .         -Bt1g1H(iq1,i3,iqbar2,i4,i5,renscale2)/nc**2
     .         +Bh1g1H(iq1,i3,iqbar2,i4,i5,renscale2)*nf/nc

      FullB1g1H=fac*FullB1g1H

      return
      end
************************************************************************
* NG: valid in arbitrary kinematics, taken from 9707448
      function B1g1H(iq,ig,iqb,i4,i5,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /order/ieorder
      common /npars/npars,nspec,n
      real*8 mhsq
      real*8, pointer :: sqqb,sqg,sqbg,sgq,sgqb
      integer ipset

      ipset = nspec+3
      sqqb => kin(ipset)%s(iq,iqb)
      sqg  => kin(ipset)%s(iq, ig)
      sqbg => kin(ipset)%s(iqb,ig)
      sgq  => sqg
      sgqb => sqbg

      mhsq=sqqb+sqg+sqbg
      flip=1d0
      if((iq.eq.1).or.(iq.eq.2))flip=-flip
      if((iqb.eq.1).or.(iqb.eq.2))flip=-flip


      tree = B1g0H(iq,ig,iqb,i4,i5)

      zlogsqqb = zzlog(-mhsq) - zzlog(-sqqb)
      zlogsqg  = zzlog(-mhsq) - zzlog(-sqg)
      zlogsqbg = zzlog(-mhsq) - zzlog(-sqbg)
      zlogmhsq = zzlog(renscale2) - zzlog(-mhsq)

      if(ieorder.eq.-2)then
        B1g1H=-2d0*tree

      elseif(ieorder.eq.-1)then
        B1g1H= tree*dreal(
     .         -zlogsqg
     .         -zlogsqbg
     .         -2d0*zlogmhsq
     .         +13d0/6d0
     .         -3d0*11d0/6d0  !renormalization
     .         )

      elseif(ieorder.eq.0)then

      !Using complete dli2 function in src/specials/specials.f for arbitrary argument
      zdilogqqb = dli2(1d0-sqqb/mhsq)
      zdilogqg  = dli2(1d0-sqg/mhsq)
      zdilogqbg = dli2(1d0-sqbg/mhsq)

        B1g1H= tree*dreal(
     .         -(2d0*zdilogqqb+zdilogqg+zdilogqbg)
     .         -1d0/2d0*zlogsqg**2
     .         -1d0/2d0*zlogsqbg**2
     .         -2d0/2d0*zlogmhsq**2
     .         -zlogsqg*zlogmhsq
     .         -zlogsqbg*zlogmhsq
     .         -zlogsqg*zlogsqqb
     .         -zlogsqbg*zlogsqqb
     .         +13d0/6d0*zlogsqqb
     .         +13d0/6d0*zlogmhsq
     .         +40d0/9d0
     .         +1d0/3d0*pi**2
     .      )
     .   -sqg*flip-sqbg*flip
     .   +tree*pi**2/6d0 !r_gamma

      endif

      return
      end
************************************************************************
* NG: valid in arbitrary kinematics, taken from 9707448
      function Bh1g1H(iq,ig,iqb,i4,i5,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /order/ieorder
      common /npars/npars,nspec,n
      real*8 mhsq
      real*8, pointer :: sqqb,sqg,sqbg,sgq,sgqb
      integer ipset

      ipset = nspec+3
      sqqb => kin(ipset)%s(iq,iqb)
      sqg  => kin(ipset)%s(iq, ig)
      sqbg => kin(ipset)%s(iqb,ig)
      sgq  => sqg
      sgqb => sqbg

      mhsq=sqqb+sqg+sqbg
      flip=1d0
      if((iq.eq.1).or.(iq.eq.2))flip=-flip
      if((iqb.eq.1).or.(iqb.eq.2))flip=-flip

      tree = B1g0H(iq,ig,iqb,i4,i5)

      zlogsqqb = zzlog(-mhsq) - zzlog(-sqqb)
      zlogmhsq = zzlog(renscale2) - zzlog(-mhsq)

      if(ieorder.eq.-2)then
        Bh1g1H=0d0

      elseif(ieorder.eq.-1)then
        Bh1g1H= tree*(
     .         -2d0/3d0
     .         +3d0*2d0/6d0  !renormalization
     .         )

      elseif(ieorder.eq.0)then

        Bh1g1H= tree*dreal(
     .         -2d0/3d0*zlogsqqb
     .         -2d0/3d0*zlogmhsq
     .         -10d0/9d0
     .      )

      endif

      return
      end
************************************************************************
* NG: valid in arbitrary kinematics, taken from 9707448
*
      function Bt1g1H(iq,ig,iqb,i4,i5,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /order/ieorder
      common /npars/npars,nspec,n
      real*8 mhsq
      real*8, pointer :: sqqb,sqg,sqbg,sgq,sgqb
      integer ipset

      ipset = nspec+3
      sqqb => kin(ipset)%s(iq,iqb)
      sqg  => kin(ipset)%s(iq, ig)
      sqbg => kin(ipset)%s(iqb,ig)
      sgq  => sqg
      sgqb => sqbg

      mhsq=sqqb+sqg+sqbg
      flip=1d0
      if((iq.eq.1).or.(iq.eq.2))flip=-flip
      if((iqb.eq.1).or.(iqb.eq.2))flip=-flip


      tree = B1g0H(iq,ig,iqb,i4,i5)

      zlogsqqb = zzlog(-mhsq) - zzlog(-sqqb)
      zlogsqg  = zzlog(-mhsq) - zzlog(-sqg)
      zlogsqbg = zzlog(-mhsq) - zzlog(-sqbg)
      zlogmhsq = zzlog(renscale2) - zzlog(-mhsq)

      if(ieorder.eq.-2)then
        Bt1g1H=1d0*tree

      elseif(ieorder.eq.-1)then
        Bt1g1H= tree*dreal(
     .         +zlogsqqb
     .         +zlogmhsq
     .         +3d0/2d0
     .         )

      elseif(ieorder.eq.0)then

      !Using complete dli2 function in src/specials/specials.f for arbitrary argument
      zdilogqqb = dli2(1d0-sqqb/mhsq)
      zdilogqg  = dli2(1d0-sqg/mhsq)
      zdilogqbg = dli2(1d0-sqbg/mhsq)

        Bt1g1H= tree*dreal(
     .         +(zdilogqg+zdilogqbg)
     .         +1d0/2d0*zlogsqqb**2
     .         +1d0/2d0*zlogmhsq**2
     .         +zlogsqqb*zlogmhsq
     .         +zlogsqg*zlogsqbg
     .         +3d0/2d0*zlogsqqb
     .         +3d0/2d0*zlogmhsq
     .         +4d0
     .         -1d0/6d0*pi**2
     .      )
     .   -sqg*flip-sqbg*flip
     .   -tree*pi**2/12d0 !r_gamma

      endif

* NG: note that 9707448 gives coefficient of 1/N^2 relative to B1g1H
*     while we are looking for coefficient of -1/N^2 relative to B1g1H

      Bt1g1H = -Bt1g1H

      return
      end

************************************************************************
* 2 quark 2 gluon one higgs 1-loop level matrix elements
*
* summed over colour orderings, summed and squared
*
* Using the result in MCFM for different helicity configuration
************************************************************************
      function FullB2g1H(i1,i3,i4,i2,i5,i6,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /order/ieorder

      stop "set value for chgg correctly!"
      fac=(4d0*pi*as*nc)**3*(nc**2-1d0)/nc*chgg**2/4d0
*      write(*,*)'hello FullB2g1H'

      FullB2g1H=
     .         (
     .          +B2g1H(i1,i3,i4,i2,i5,i6,renscale2)
     .          -Bt2g1H(i1,i3,i4,i2,i5,i6,renscale2)/nc**2
     .          +Bh2g1H(i1,i3,i4,i2,i5,i6,renscale2)*dfloat(nf)/nc

     .          +B2g1H(i1,i4,i3,i2,i5,i6,renscale2)
     .          -Bt2g1H(i1,i4,i3,i2,i5,i6,renscale2)/nc**2
     .          +Bh2g1H(i1,i4,i3,i2,i5,i6,renscale2)*dfloat(nf)/nc

     .          -1d0/nc**2*(Btt2g1H(i1,i3,i4,i2,i5,i6,renscale2)
     .                     -Bttt2g1H(i1,i3,i4,i2,i5,i6,renscale2)/nc**2
     .                     +Btth2g1H(i1,i3,i4,i2,i5,i6,renscale2)*dfloat(nf)/nc)

     .          +1d0/nc**2*Bhh2g1H(i1,i3,i4,i2,i5,i6,renscale2)
     .          )

      FullB2g1H=fac*FullB2g1H
      return
      end
************************************************************************
      function Btt2g1HF(i1,i3,i4,i2,i5,i6,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /order/ieorder


      Btt2g1HF=Btt2g1H(i1,i3,i4,i2,i5,i6,renscale2)-Bhh2g1H(i1,i3,i4,i2,i5,i6,renscale2)

      return
      end
************************************************************************
      function FullB2g1HCL(i1,i3,i4,i2,i5,i6,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /order/ieorder

      stop "set value for chgg correctly!"
      fac=(4d0*pi*as*nc)**3*(nc**2-1d0)/nc*chgg**2/4d0
*      write(*,*)'hello FullB2g1H'

      FullB2g1HCL=
     .         (
     .          +B2g1H(i1,i3,i4,i2,i5,i6,renscale2)
     .          +B2g1H(i1,i4,i3,i2,i5,i6,renscale2)
     .          )

*      FullB2g1HCL=fac*FullB2g1HCL
      return
      end
************************************************************************
*XC B2g1H code is based on MCFM-6.8
*XC one colour ordering, summed over helicities
*XC 1-loop*tree^{+} structure
      function B2g1H(i1,i3,i4,i2,i5,i6,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /order/ieorder
      common /koppel/ nf,nc,b0
      integer NHEL(4,8),perm(6),permb(6),permm(6)
      DATA (NHEL(IHEL,   1),IHEL=1, 4) / -1, 1, 1, 1/!
      DATA (NHEL(IHEL,   2),IHEL=1, 4) / -1, 1,-1,-1/
      DATA (NHEL(IHEL,   3),IHEL=1, 4) / -1, 1,-1, 1/
      DATA (NHEL(IHEL,   4),IHEL=1, 4) / -1, 1, 1,-1/

      DATA (NHEL(IHEL,   5),IHEL=1, 4) /  1,-1,-1,-1/
      DATA (NHEL(IHEL,   6),IHEL=1, 4) /  1,-1, 1, 1/
      DATA (NHEL(IHEL,   7),IHEL=1, 4) /  1,-1, 1,-1/!
      DATA (NHEL(IHEL,   8),IHEL=1, 4) /  1,-1,-1, 1/!
        perm(1)=i1
        perm(2)=i2
        perm(3)=i4
        perm(4)=i3
        perm(5)=i5
        perm(6)=i6

        permb(1)=i1
        permb(2)=i2
        permb(3)=i3
        permb(4)=i4
        permb(5)=i5
        permb(6)=i6

        permm(1)=i1
        permm(2)=i3
        permm(3)=i4
        permm(4)=i2
        permm(5)=i5
        permm(6)=i6

       wt=0d0
       temp=0d0
       do i=1,8
           if(i.eq.1) then
            wt=
     .                +dble((
     .                      +zB2g1Hmpppcl(perm(1),renscale2)
     .                     )*conjg(zB2g0Hmppp(permm(1))))
           endif
           if(i.eq.2) then
            wt=
     .                +dble((
     .                      +zB2g1Hmpmmcl(perm(1),renscale2)
     .                     )*conjg(zB2g0Hmmmp(permm(1))))
           endif
           if(i.eq.3) then
            wt=
     .                +dble((
     .                      +zB2g1Hmpmpcl(perm(1),renscale2)
     .                      )*conjg(zB2g0Hmpmp(permm(1))))
           endif
           if(i.eq.4) then
            wt=
     .                +dble((
     .                      +zB2g1Hmppmcl(perm(1),renscale2)
     .                      )*conjg(zB2g0Hmmpp(permm(1))))

           endif
           if(i.eq.5) then
            wt=
     .                +dble((
     .                      -zB2g1Hpmmmcl(perm(1),renscale2)
     .                      )*conjg(zB2g0Hpmmm(permm(1))))
           endif
           if(i.eq.6) then
            wt=
     .                +dble((
     .                      -zB2g1Hpmppcl(perm(1),renscale2)
     .                      )*conjg(zB2g0Hpppm(permm(1))))
           endif
           if(i.eq.7) then
            wt=
     .                +dble((
     .                      -zB2g1Hpmpmcl(perm(1),renscale2)
     .                      )*conjg(zB2g0Hpmpm(permm(1))))
           endif
           if(i.eq.8) then
            wt=
     .                +dble((
     .                      -zB2g1Hpmmpcl(perm(1),renscale2)
     .                      )*conjg(zB2g0Hppmm(permm(1))))
           endif
          temp=temp+wt
       enddo

      if(ieorder.eq.0)then
        ren=3d0*pi**2/12d0
      elseif(ieorder.eq.-1)then
        ren=-4d0*11d0/6d0
      elseif(ieorder.eq.-2)then
        ren=0d0
      endif

         B2g1H=temp+B2g0H(i1,i3,i4,i2,i5,i6)*ren
*XC c_gamma fix up
   10 format(" B2g1H   ",e12.4,6i3)
      return
      end
************************************************************************
*XC Bt2g1H code is based on MCFM-6.8
*XC one colour ordering, summed over helicities
*XC 1-loop*tree^{+} structure
      function Bt2g1H(i1,i3,i4,i2,i5,i6,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /order/ieorder
      common /koppel/ nf,nc,b0
      integer NHEL(4,8),perm(6),permm(6)
      DATA (NHEL(IHEL,   1),IHEL=1, 4) / -1, 1, 1, 1/!
      DATA (NHEL(IHEL,   2),IHEL=1, 4) / -1, 1,-1,-1/
      DATA (NHEL(IHEL,   3),IHEL=1, 4) / -1, 1,-1, 1/
      DATA (NHEL(IHEL,   4),IHEL=1, 4) / -1, 1, 1,-1/

      DATA (NHEL(IHEL,   5),IHEL=1, 4) /  1,-1,-1,-1/
      DATA (NHEL(IHEL,   6),IHEL=1, 4) /  1,-1, 1, 1/
      DATA (NHEL(IHEL,   7),IHEL=1, 4) /  1,-1, 1,-1/!
      DATA (NHEL(IHEL,   8),IHEL=1, 4) /  1,-1,-1, 1/!
        perm(1)=i1
        perm(2)=i2
        perm(3)=i4
        perm(4)=i3
        perm(5)=i5
        perm(6)=i6

        permm(1)=i1
        permm(2)=i3
        permm(3)=i4
        permm(4)=i2
        permm(5)=i5
        permm(6)=i6

       zamp=0d0
       temp=0d0
       do i=1,8
           if(i.eq.1) then
            zamp=
     .                +dble((
     .                      +zB2g1Hmpppsl(perm(1),renscale2)
     .                     )*conjg(zB2g0Hmppp(permm(1))))
           endif
           if(i.eq.2) then
            zamp=
     .                +dble((
     .                      +zB2g1Hmpmmsl(perm(1),renscale2)
     .                     )*conjg(zB2g0Hmmmp(permm(1))))
           endif
           if(i.eq.3) then
            zamp=
     .                +dble((
     .                      +zB2g1Hmpmpsl(perm(1),renscale2)
     .                      )*conjg(zB2g0Hmpmp(permm(1))))
           endif
           if(i.eq.4) then
            zamp=
     .                +dble((
     .                      +zB2g1Hmppmsl(perm(1),renscale2)
     .                      )*conjg(zB2g0Hmmpp(permm(1))))

           endif
           if(i.eq.5) then
            zamp=
     .                +dble((
     .                      -zB2g1Hpmmmsl(perm(1),renscale2)
     .                      )*conjg(zB2g0Hpmmm(permm(1))))
           endif
           if(i.eq.6) then
            zamp=
     .                +dble((
     .                      -zB2g1Hpmppsl(perm(1),renscale2)
     .                      )*conjg(zB2g0Hpppm(permm(1))))
           endif
           if(i.eq.7) then
            zamp=
     .                +dble((
     .                      -zB2g1Hpmpmsl(perm(1),renscale2)
     .                      )*conjg(zB2g0Hpmpm(permm(1))))
           endif
           if(i.eq.8) then
            zamp=
     .                +dble((
     .                      -zB2g1Hpmmpsl(perm(1),renscale2)
     .                      )*conjg(zB2g0Hppmm(permm(1))))
           endif
          temp=temp+zamp
       enddo


      if(ieorder.eq.0)then
        ren=pi**2/12d0
      elseif(ieorder.eq.-1)then
        zl12=zlnrat(renscale2,-s6(i1,i2))
        ren=-3d0/2d0-zl12
      elseif(ieorder.eq.-2)then
        ren=-1d0
      else
        ren=0d0
      endif

         Bt2g1H=temp+B2g0H(i1,i3,i4,i2,i5,i6)*ren
*XC c_gamma fix up
   10 format(" B2g1H   ",e12.4,6i3)
      return
      end
************************************************************************
*XC Bh2g1H code is based on MCFM-6.8
*XC one colour ordering, summed over helicities
*XC 1-loop*tree^{+} structure
      function Bh2g1H(i1,i3,i4,i2,i5,i6,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /koppel/ nf,nc,b0
      common /order/ieorder

      integer NHEL(4,8),perm(6),permm(6)
      DATA (NHEL(IHEL,   1),IHEL=1, 4) / -1, 1, 1, 1/!
      DATA (NHEL(IHEL,   2),IHEL=1, 4) / -1, 1,-1,-1/
      DATA (NHEL(IHEL,   3),IHEL=1, 4) / -1, 1,-1, 1/
      DATA (NHEL(IHEL,   4),IHEL=1, 4) / -1, 1, 1,-1/

      DATA (NHEL(IHEL,   5),IHEL=1, 4) /  1,-1,-1,-1/
      DATA (NHEL(IHEL,   6),IHEL=1, 4) /  1,-1, 1, 1/
      DATA (NHEL(IHEL,   7),IHEL=1, 4) /  1,-1, 1,-1/!
      DATA (NHEL(IHEL,   8),IHEL=1, 4) /  1,-1,-1, 1/!
        perm(1)=i1
        perm(2)=i2
        perm(3)=i4
        perm(4)=i3
        perm(5)=i5
        perm(6)=i6

        permm(1)=i1
        permm(2)=i3
        permm(3)=i4
        permm(4)=i2
        permm(5)=i5
        permm(6)=i6

       zamp=0d0
       temp=0d0
       do i=1,8
           if(i.eq.1) then
            zamp=
     .                +dble((
     .                      +zB2g1Hmpppnf(perm(1),renscale2)
     .                     )*conjg(zB2g0Hmppp(permm(1))))
           endif
           if(i.eq.2) then
            zamp=
     .                +dble((
     .                      +zB2g1Hmpmmnf(perm(1),renscale2)
     .                     )*conjg(zB2g0Hmmmp(permm(1))))
           endif
           if(i.eq.3) then
            zamp=
     .                +dble((
     .                      +zB2g1Hmpmpnf(perm(1),renscale2)
     .                      )*conjg(zB2g0Hmpmp(permm(1))))
           endif
           if(i.eq.4) then
            zamp=
     .                +dble((
     .                      +zB2g1Hmppmnf(perm(1),renscale2)
     .                      )*conjg(zB2g0Hmmpp(permm(1))))

           endif
           if(i.eq.5) then
            zamp=
     .                +dble((
     .                      -zB2g1Hpmmmnf(perm(1),renscale2)
     .                      )*conjg(zB2g0Hpmmm(permm(1))))
           endif
           if(i.eq.6) then
            zamp=
     .                +dble((
     .                      -zB2g1Hpmppnf(perm(1),renscale2)
     .                      )*conjg(zB2g0Hpppm(permm(1))))
           endif
           if(i.eq.7) then
            zamp=
     .                +dble((
     .                      -zB2g1Hpmpmnf(perm(1),renscale2)
     .                      )*conjg(zB2g0Hpmpm(permm(1))))
           endif
           if(i.eq.8) then
            zamp=
     .                +dble((
     .                      -zB2g1Hpmmpnf(perm(1),renscale2)
     .                      )*conjg(zB2g0Hppmm(permm(1))))
           endif
          temp=temp+zamp
       enddo

      if(ieorder.eq.-1)then
        ren=2d0/3d0
      else
        ren=0d0
      endif

         Bh2g1H=temp+B2g0H(i1,i3,i4,i2,i5,i6)*ren

   10 format(" B2g1H   ",e12.4,6i3)
      return
      end
************************************************************************
*XC Btt2g1H code is based on MCFM-6.8
*XC one colour ordering, summed over helicities
*XC 1-loop*tree^{+} structure
      function Btt2g1H(i1,i3,i4,i2,i5,i6,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /order/ieorder
      common /koppel/ nf,nc,b0
      common /npars/npars,nspec,n

      integer NHEL(4,8),perm(6),permb(6),permm(6),permn(6)
      DATA (NHEL(IHEL,   1),IHEL=1, 4) / -1, 1, 1, 1/!
      DATA (NHEL(IHEL,   2),IHEL=1, 4) / -1, 1,-1,-1/
      DATA (NHEL(IHEL,   3),IHEL=1, 4) / -1, 1,-1, 1/
      DATA (NHEL(IHEL,   4),IHEL=1, 4) / -1, 1, 1,-1/

      DATA (NHEL(IHEL,   5),IHEL=1, 4) /  1,-1,-1,-1/
      DATA (NHEL(IHEL,   6),IHEL=1, 4) /  1,-1, 1, 1/
      DATA (NHEL(IHEL,   7),IHEL=1, 4) /  1,-1, 1,-1/!
      DATA (NHEL(IHEL,   8),IHEL=1, 4) /  1,-1,-1, 1/!
        perm(1)=i1
        perm(2)=i2
        perm(3)=i3
        perm(4)=i4
        perm(5)=i5
        perm(6)=i6

        permb(1)=i1
        permb(2)=i2
        permb(3)=i4
        permb(4)=i3
        permb(5)=i5
        permb(6)=i6

        permm(1)=i1
        permm(2)=i4
        permm(3)=i3
        permm(4)=i2
        permm(5)=i5
        permm(6)=i6

        permn(1)=i1
        permn(2)=i3
        permn(3)=i4
        permn(4)=i2
        permn(5)=i5
        permn(6)=i6

      if(nspec.eq.2)then
        s34=s6(i3,i4)
        s14=s6(i1,i4)
        s24=s6(i2,i4)
        s13=s6(i1,i3)
        s23=s6(i2,i3)
      elseif(nspec.eq.3)then
        s34=s7(i3,i4)
        s14=s7(i1,i4)
        s24=s7(i2,i4)
        s13=s7(i1,i3)
        s23=s7(i2,i3)
      elseif(nspec.eq.4)then
        s34=s8(i3,i4)
        s14=s8(i1,i4)
        s24=s8(i2,i4)
        s13=s8(i1,i3)
        s23=s8(i2,i3)
      else
        write(*,*)"Btt2g1H spectator unknown"
      endif


       zamp=0d0
       temp=0d0
       do i=1,8
           if(i.eq.1) then
            zamp=
     .                +dble(
     .                (zB2g1Hmpppcl(perm(1),renscale2)

     .                +zB2g1Hmpppcl(permb(1),renscale2)

     .                )*conjg(zB2g0Hmppp(permm(1))+zB2g0Hmppp(permn(1))))
           endif
           if(i.eq.2) then
            zamp=
     .                +dble(
     .                (zB2g1Hmpmmcl(perm(1),renscale2)

     .                +zB2g1Hmpmmcl(permb(1),renscale2)

     .                )*conjg(zB2g0Hmmmp(permm(1))+zB2g0Hmmmp(permn(1))))
           endif
           if(i.eq.3) then
            zamp=
     .                +dble(
     .                (zB2g1Hmpmpcl(perm(1),renscale2)

     .                +zB2g1Hmppmcl(permb(1),renscale2)
     .                )*conjg(zB2g0Hmpmp(permm(1))+zB2g0Hmmpp(permn(1))))

           endif
           if(i.eq.4) then
            zamp=
     .                +dble(
     .                (zB2g1Hmppmcl(perm(1),renscale2)

     .                +zB2g1Hmpmpcl(permb(1),renscale2)

     .                )*conjg(zB2g0Hmmpp(permm(1))+zB2g0Hmpmp(permn(1))))
*XC helicity configuration 3&4 cross talk to each other when combine the two colour ordering together
           endif
           if(i.eq.5) then
            zamp=
     .                +dble(
     .                -(zB2g1Hpmmmcl(perm(1),renscale2)

     .                 +zB2g1Hpmmmcl(permb(1),renscale2)
     .                )*conjg(zB2g0Hpmmm(permm(1))+zB2g0Hpmmm(permn(1))))
           endif
           if(i.eq.6) then
            zamp=
     .                +dble(
     .                -(zB2g1Hpmppcl(perm(1),renscale2)

     .                 +zB2g1Hpmppcl(permb(1),renscale2)
     .                )*conjg(zB2g0Hpppm(permm(1))+zB2g0Hpppm(permn(1))))
           endif
           if(i.eq.7) then
            zamp=
     .                +dble(
     .                -(zB2g1Hpmpmcl(perm(1),renscale2)

     .                 +zB2g1Hpmmpcl(permb(1),renscale2)
     .                )*conjg(zB2g0Hpmpm(permm(1))+zB2g0Hppmm(permn(1))))

           endif
           if(i.eq.8) then
            zamp=
     .                +dble(
     .                -(zB2g1Hpmmpcl(perm(1),renscale2)

     .                 +zB2g1Hpmpmcl(permb(1),renscale2)
     .                )*conjg(zB2g0Hppmm(permm(1))+zB2g0Hpmpm(permn(1))))
*XC helicity configuration 7&8 cross talk to each other when combine the two colour ordering together

           endif
          temp=temp+zamp
       enddo

      if(ieorder.eq.0)then
        ren=3d0*pi**2/12d0
      elseif(ieorder.eq.-1)then
        ren=0d0
      else
        ren=0d0
      endif

      if(ieorder.eq.-2)then
        temp=-3d0*Bt2g0H(i1,i3,i4,i2,i5,i6)
      elseif(ieorder.eq.-1)then
        zl34=zlnrat(renscale2,-s34)
        zl14=zlnrat(renscale2,-s14)
        zl24=zlnrat(renscale2,-s24)
        zl13=zlnrat(renscale2,-s13)
        zl23=zlnrat(renscale2,-s23)
        temp=(-31d0/6d0
     .        -        zl34
     .        -1d0/2d0*zl14
     .        -1d0/2d0*zl13
     .        -1d0/2d0*zl24
     .        -1d0/2d0*zl23
     .       )*Bt2g0H(i1,i3,i4,i2,i5,i6)
      endif
*XC rewrite the pole structure using above formular for better numerical stability in unresolved limits

         Btt2g1H=temp+Bt2g0H(i1,i3,i4,i2,i5,i6)*ren
*XC c_gamma fix up in ieorder=0
   10 format(" B2g1H   ",e12.4,6i3)
      return
      end
************************************************************************
*XC Bttt2g1H code is based on MCFM-6.8
*XC one colour ordering, summed over helicities
*XC 1-loop*tree^{+} structure
      function Bttt2g1H(i1,i3,i4,i2,i5,i6,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /order/ieorder
      common /koppel/ nf,nc,b0
      integer NHEL(4,8),perm(6),permb(6),permm(6),permn(6)
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s31,s23
      integer ipset

      DATA (NHEL(IHEL,   1),IHEL=1, 4) / -1, 1, 1, 1/!
      DATA (NHEL(IHEL,   2),IHEL=1, 4) / -1, 1,-1,-1/
      DATA (NHEL(IHEL,   3),IHEL=1, 4) / -1, 1,-1, 1/
      DATA (NHEL(IHEL,   4),IHEL=1, 4) / -1, 1, 1,-1/

      DATA (NHEL(IHEL,   5),IHEL=1, 4) /  1,-1,-1,-1/
      DATA (NHEL(IHEL,   6),IHEL=1, 4) /  1,-1, 1, 1/
      DATA (NHEL(IHEL,   7),IHEL=1, 4) /  1,-1, 1,-1/!
      DATA (NHEL(IHEL,   8),IHEL=1, 4) /  1,-1,-1, 1/!

      ipset = nspec+4
      s12 => kin(ipset)%s(i1,i2)

        perm(1)=i1
        perm(2)=i2
        perm(3)=i3
        perm(4)=i4
        perm(5)=i5
        perm(6)=i6

        permb(1)=i1
        permb(2)=i2
        permb(3)=i4
        permb(4)=i3
        permb(5)=i5
        permb(6)=i6

        permm(1)=i1
        permm(2)=i4
        permm(3)=i3
        permm(4)=i2
        permm(5)=i5
        permm(6)=i6

        permn(1)=i1
        permn(2)=i3
        permn(3)=i4
        permn(4)=i2
        permn(5)=i5
        permn(6)=i6


       zamp=0d0
       temp=0d0
       do i=1,8
           if(i.eq.1) then
            zamp=
     .                +dble(
     .                (
     .                +zB2g1Hmpppsl(perm(1),renscale2)

     .                +zB2g1Hmpppsl(permb(1),renscale2)

     .                )*conjg(zB2g0Hmppp(permm(1))+zB2g0Hmppp(permn(1))))
           endif
           if(i.eq.2) then
            zamp=
     .                +dble(
     .                (
     .                +zB2g1Hmpmmsl(perm(1),renscale2)

     .                +zB2g1Hmpmmsl(permb(1),renscale2)

     .                )*conjg(zB2g0Hmmmp(permm(1))+zB2g0Hmmmp(permn(1))))
           endif
           if(i.eq.3) then
            zamp=
     .                +dble(
     .                (
     .                +zB2g1Hmpmpsl(perm(1),renscale2)

     .                +zB2g1Hmppmsl(permb(1),renscale2)
     .                )*conjg(zB2g0Hmpmp(permm(1))+zB2g0Hmmpp(permn(1))))

           endif
           if(i.eq.4) then
            zamp=
     .                +dble(
     .                (
     .                +zB2g1Hmppmsl(perm(1),renscale2)

     .                +zB2g1Hmpmpsl(permb(1),renscale2)

     .                )*conjg(zB2g0Hmmpp(permm(1))+zB2g0Hmpmp(permn(1))))
*XC helicity configuration 3&4 cross talk to each other when combine the two colour ordering together
           endif
           if(i.eq.5) then
            zamp=
     .                +dble(
     .                -(
     .                 +zB2g1Hpmmmsl(perm(1),renscale2)

     .                 +zB2g1Hpmmmsl(permb(1),renscale2)
     .                )*conjg(zB2g0Hpmmm(permm(1))+zB2g0Hpmmm(permn(1))))
           endif
           if(i.eq.6) then
            zamp=
     .                +dble(
     .                -(
     .                 +zB2g1Hpmppsl(perm(1),renscale2)

     .                 +zB2g1Hpmppsl(permb(1),renscale2)
     .                )*conjg(zB2g0Hpppm(permm(1))+zB2g0Hpppm(permn(1))))
           endif
           if(i.eq.7) then
            zamp=
     .                +dble(
     .                -(
     .                 +zB2g1Hpmpmsl(perm(1),renscale2)

     .                 +zB2g1Hpmmpsl(permb(1),renscale2)
     .                )*conjg(zB2g0Hpmpm(permm(1))+zB2g0Hppmm(permn(1))))

           endif
           if(i.eq.8) then
            zamp=
     .                +dble(
     .                -(
     .                 +zB2g1Hpmmpsl(perm(1),renscale2)

     .                 +zB2g1Hpmpmsl(permb(1),renscale2)
     .                )*conjg(zB2g0Hppmm(permm(1))+zB2g0Hpmpm(permn(1))))
*XC helicity configuration 7&8 cross talk to each other when combine the two colour ordering together

           endif
          temp=temp+zamp
       enddo

      if(ieorder.eq.0)then
        ren=pi**2/12d0
      elseif(ieorder.eq.-1)then
        zl12=zlnrat(renscale2,-s12)
        ren=-3d0/2d0-zl12
      elseif(ieorder.eq.-2)then
        ren=-1d0
      else
        ren=0d0
      endif

         Bttt2g1H=temp+Bt2g0H(i1,i3,i4,i2,i5,i6)*ren
*XC c_gamma fix up
   10 format(" B2g1H   ",e12.4,6i3)
      return
      end
************************************************************************
*XC Btth2g1H code is based on MCFM-6.8
*XC one colour ordering, summed over helicities
*XC 1-loop*tree^{+} structure
      function Btth2g1H(i1,i3,i4,i2,i5,i6,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /koppel/ nf,nc,b0
      common /order/ieorder

      integer NHEL(4,8),perm(6),permb(6),permm(6),permn(6)
      DATA (NHEL(IHEL,   1),IHEL=1, 4) / -1, 1, 1, 1/!
      DATA (NHEL(IHEL,   2),IHEL=1, 4) / -1, 1,-1,-1/
      DATA (NHEL(IHEL,   3),IHEL=1, 4) / -1, 1,-1, 1/
      DATA (NHEL(IHEL,   4),IHEL=1, 4) / -1, 1, 1,-1/

      DATA (NHEL(IHEL,   5),IHEL=1, 4) /  1,-1,-1,-1/
      DATA (NHEL(IHEL,   6),IHEL=1, 4) /  1,-1, 1, 1/
      DATA (NHEL(IHEL,   7),IHEL=1, 4) /  1,-1, 1,-1/!
      DATA (NHEL(IHEL,   8),IHEL=1, 4) /  1,-1,-1, 1/!
        perm(1)=i1
        perm(2)=i2
        perm(3)=i3
        perm(4)=i4
        perm(5)=i5
        perm(6)=i6

        permb(1)=i1
        permb(2)=i2
        permb(3)=i4
        permb(4)=i3
        permb(5)=i5
        permb(6)=i6

        permm(1)=i1
        permm(2)=i4
        permm(3)=i3
        permm(4)=i2
        permm(5)=i5
        permm(6)=i6

        permn(1)=i1
        permn(2)=i3
        permn(3)=i4
        permn(4)=i2
        permn(5)=i5
        permn(6)=i6


       zamp=0d0
       temp=0d0
       do i=1,8
           if(i.eq.1) then
            zamp=
     .                +dble(
     .                (
     .                +zB2g1Hmpppnf(perm(1),renscale2)

     .                +zB2g1Hmpppnf(permb(1),renscale2)

     .                )*conjg(zB2g0Hmppp(permm(1))+zB2g0Hmppp(permn(1))))
           endif
           if(i.eq.2) then
            zamp=
     .                +dble(
     .                (
     .                +zB2g1Hmpmmnf(perm(1),renscale2)

     .                +zB2g1Hmpmmnf(permb(1),renscale2)

     .                )*conjg(zB2g0Hmmmp(permm(1))+zB2g0Hmmmp(permn(1))))
           endif
           if(i.eq.3) then
            zamp=
     .                +dble(
     .                (
     .                +zB2g1Hmpmpnf(perm(1),renscale2)

     .                +zB2g1Hmppmnf(permb(1),renscale2)
     .                )*conjg(zB2g0Hmpmp(permm(1))+zB2g0Hmmpp(permn(1))))

           endif
           if(i.eq.4) then
            zamp=
     .                +dble(
     .                (
     .                +zB2g1Hmppmnf(perm(1),renscale2)

     .                +zB2g1Hmpmpnf(permb(1),renscale2)

     .                )*conjg(zB2g0Hmmpp(permm(1))+zB2g0Hmpmp(permn(1))))
*XC helicity configuration 3&4 cross talk to each other when combine the two colour ordering together
           endif
           if(i.eq.5) then
            zamp=
     .                +dble(
     .                -(
     .                 +zB2g1Hpmmmnf(perm(1),renscale2)

     .                 +zB2g1Hpmmmnf(permb(1),renscale2)
     .                )*conjg(zB2g0Hpmmm(permm(1))+zB2g0Hpmmm(permn(1))))
           endif
           if(i.eq.6) then
            zamp=
     .                +dble(
     .                -(
     .                 +zB2g1Hpmppnf(perm(1),renscale2)

     .                 +zB2g1Hpmppnf(permb(1),renscale2)
     .                )*conjg(zB2g0Hpppm(permm(1))+zB2g0Hpppm(permn(1))))
           endif
           if(i.eq.7) then
            zamp=
     .                +dble(
     .                -(
     .                 +zB2g1Hpmpmnf(perm(1),renscale2)

     .                 +zB2g1Hpmmpnf(permb(1),renscale2)
     .                )*conjg(zB2g0Hpmpm(permm(1))+zB2g0Hppmm(permn(1))))

           endif
           if(i.eq.8) then
            zamp=
     .                +dble(
     .                -(
     .                 +zB2g1Hpmmpnf(perm(1),renscale2)

     .                 +zB2g1Hpmpmnf(permb(1),renscale2)
     .                )*conjg(zB2g0Hppmm(permm(1))+zB2g0Hpmpm(permn(1))))
*XC helicity configuration 7&8 cross talk to each other when combine the two colour ordering together

           endif
          temp=temp+zamp
       enddo

      if(ieorder.eq.-1)then
        ren=2d0/3d0
      else
        ren=0d0
      endif

         Btth2g1H=temp+Bt2g0H(i1,i3,i4,i2,i5,i6)*ren

   10 format(" B2g1H   ",e12.4,6i3)
      return
      end

************************************************************************
*XC Bhh2g1H code is based on MCFM-6.8
*XC one colour ordering, summed over helicities
*XC 1-loop*tree^{+} structure
      function Bhh2g1H(i1,i3,i4,i2,i5,i6,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /koppel/ nf,nc,b0
      common /order/ieorder
      integer NHEL(4,8),perm(6),permm(6),permn(6)
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      common /npars/npars,nspec,n
      integer ipset

      DATA (NHEL(IHEL,   1),IHEL=1, 4) / -1, 1, 1, 1/!
      DATA (NHEL(IHEL,   2),IHEL=1, 4) / -1, 1,-1,-1/
      DATA (NHEL(IHEL,   3),IHEL=1, 4) / -1, 1,-1, 1/
      DATA (NHEL(IHEL,   4),IHEL=1, 4) / -1, 1, 1,-1/

      DATA (NHEL(IHEL,   5),IHEL=1, 4) /  1,-1,-1,-1/
      DATA (NHEL(IHEL,   6),IHEL=1, 4) /  1,-1, 1, 1/
      DATA (NHEL(IHEL,   7),IHEL=1, 4) /  1,-1, 1,-1/!
      DATA (NHEL(IHEL,   8),IHEL=1, 4) /  1,-1,-1, 1/!

      ipset = nspec+4

        perm(1)=i1
        perm(2)=i2
        perm(3)=i3
        perm(4)=i4
        perm(5)=i5
        perm(6)=i6

        permm(1)=i1
        permm(2)=i4
        permm(3)=i3
        permm(4)=i2
        permm(5)=i5
        permm(6)=i6

        permn(1)=i1
        permn(2)=i3
        permn(3)=i4
        permn(4)=i2
        permn(5)=i5
        permn(6)=i6

      s12 => kin(ipset)%s(i1,i2)
      s13 => kin(ipset)%s(i1,i3)
      s14 => kin(ipset)%s(i1,i4)
      s23 => kin(ipset)%s(i2,i3)
      s24 => kin(ipset)%s(i2,i4)
      s34 => kin(ipset)%s(i3,i4)

      s21 => kin(ipset)%s(i2,i1)
      s31 => kin(ipset)%s(i3,i1)
      s41 => kin(ipset)%s(i4,i1)
      s32 => kin(ipset)%s(i3,i2)
      s42 => kin(ipset)%s(i4,i2)
      s43 => kin(ipset)%s(i4,i3)

       zamp=0d0
       temp=0d0
       do i=1,8
           if(i.eq.1) then
            zamp=
     .                +dble(
     .                +zBt2g1Hmppp(perm(1),renscale2)
     .          *conjg(zB2g0Hmppp(permm(1))+zB2g0Hmppp(permn(1))))
           endif
           if(i.eq.2) then
            zamp=
     .                +dble(
     .                +zBt2g1Hmpmm(perm(1),renscale2)
     .          *conjg(zB2g0Hmmmp(permm(1))+zB2g0Hmmmp(permn(1))))
           endif
           if(i.eq.3) then
            zamp=
     .                +dble(
     .                +zBt2g1Hmpmp(perm(1),renscale2)
     .          *conjg(zB2g0Hmpmp(permm(1))+zB2g0Hmmpp(permn(1))))
           endif
           if(i.eq.4) then
            zamp=
     .                +dble(
     .                +zBt2g1Hmppm(perm(1),renscale2)
     .          *conjg(zB2g0Hmmpp(permm(1))+zB2g0Hmpmp(permn(1))))
*XC helicity configuration 3&4 cross talk to each other when combine the two colour ordering together

           endif
           if(i.eq.5) then
            zamp=
     .                +dble(
     .                -zBt2g1Hpmmm(perm(1),renscale2)
     .          *conjg(zB2g0Hpmmm(permm(1))+zB2g0Hpmmm(permn(1))))
           endif
           if(i.eq.6) then
            zamp=
     .                +dble(
     .                -zBt2g1Hpmpp(perm(1),renscale2)
     .          *conjg(zB2g0Hpppm(permm(1))+zB2g0Hpppm(permn(1))))
           endif
           if(i.eq.7) then
            zamp=
     .                +dble(
     .                -zBt2g1Hpmpm(perm(1),renscale2)
     .          *conjg(zB2g0Hpmpm(permm(1))+zB2g0Hppmm(permn(1))))
           endif
           if(i.eq.8) then
            zamp=
     .                +dble(
     .                -zBt2g1Hpmmp(perm(1),renscale2)
     .          *conjg(zB2g0Hppmm(permm(1))+zB2g0Hpmpm(permn(1))))
*XC helicity configuration 7&8 cross talk to each other when combine the two colour ordering together

           endif
          temp=temp+zamp
       enddo


      if(ieorder.eq.-2)then
        temp=0d0
      elseif(ieorder.eq.-1)then
        zl12=zlnrat(renscale2,-s12)
        zl34=zlnrat(renscale2,-s34)
        zl14=zlnrat(renscale2,-s14)
        zl24=zlnrat(renscale2,-s24)
        zl13=zlnrat(renscale2,-s13)
        zl23=zlnrat(renscale2,-s23)
        temp=(
     .        -        zl34
     .        -        zl12
     .        +1d0/2d0*zl14
     .        +1d0/2d0*zl13
     .        +1d0/2d0*zl24
     .        +1d0/2d0*zl23
     .       )*Bt2g0H(i1,i3,i4,i2,i5,i6)
      endif
*XC rewrite the pole structure using above formular for better numerical stability in unresolved limits

         Bhh2g1H=temp

   10 format(" B2g1H   ",e12.4,6i3)
      return
      end

*****************************************************************************************
*****************************************************************************************
******************* Library for 2 quarks 2 gluon  1-loop amplitudes *********************
*****************************************************************************************
*****************************************************************************************
      double complex function zB2g1Hmpppcl(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer  perm(6),permm(6),permmb(6)
*      write(*,*)'hello zB2g1Hmppp'
      permm(1)=perm(1)
      permm(2)=perm(2)
      permm(3)=perm(3)
      permm(4)=perm(4)
      permm(5)=perm(5)
      permm(6)=perm(6)

      permmb(1)=permm(2)
      permmb(2)=permm(1)
      permmb(3)=permm(4)
      permmb(4)=permm(3)
      permmb(5)=permm(5)
      permmb(6)=permm(6)

      zB2g1Hmpppcl=
     .   +zA41phiAQggmpppcl(permm(1))
     .   -zA41cphiAQggmpmmcl(permmb(1),renscale2)

      return
      end
*****************************************************************************************
      double complex function zB2g1Hmpppsl(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer  perm(6),permm(6),permmb(6)
*      write(*,*)'hello zB2g1Hmpppsl'
      permm(1)=perm(1)
      permm(2)=perm(2)
      permm(3)=perm(3)
      permm(4)=perm(4)
      permm(5)=perm(5)
      permm(6)=perm(6)

      permmb(1)=permm(2)
      permmb(2)=permm(1)
      permmb(3)=permm(4)
      permmb(4)=permm(3)
      permmb(5)=permm(5)
      permmb(6)=permm(6)

      zB2g1Hmpppsl=
     .   +zA41phiAQggmpppsl(permm(1))
     .   -zA41cphiAQggmpmmsl(permmb(1),renscale2)

      return
      end
*****************************************************************************************
      double complex function zB2g1Hmpppnf(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer  perm(6),permm(6),permmb(6)
*      write(*,*)'hello zB2g1Hmpppnf'
      permm(1)=perm(1)
      permm(2)=perm(2)
      permm(3)=perm(3)
      permm(4)=perm(4)
      permm(5)=perm(5)
      permm(6)=perm(6)

      permmb(1)=permm(2)
      permmb(2)=permm(1)
      permmb(3)=permm(4)
      permmb(4)=permm(3)
      permmb(5)=permm(5)
      permmb(6)=permm(6)

      zB2g1Hmpppnf=
     .   +zA41phiAQggmpppnf(permm(1))

     .   -zA41cphiAQggmpmmnf(permmb(1),renscale2)

      return
      end
************************************************************************
      double complex function zB2g1Hmpmmcl(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer  perm(6),permm(6),permmb(6)
*      write(*,*)'hello zB2g1Hmpmmcl'
      permm(1)=perm(1)
      permm(2)=perm(2)
      permm(3)=perm(3)
      permm(4)=perm(4)
      permm(5)=perm(5)
      permm(6)=perm(6)

      permmb(1)=permm(2)
      permmb(2)=permm(1)
      permmb(3)=permm(4)
      permmb(4)=permm(3)
      permmb(5)=permm(5)
      permmb(6)=permm(6)

      zB2g1Hmpmmcl=
     .    +zA41phiAQggmpmmcl(permm(1),renscale2)

     .   -zA41cphiAQggmpppcl(permmb(1))

      return
      end
************************************************************************
      double complex function zB2g1Hmpmmsl(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer  perm(6),permm(6),permmb(6)
*      write(*,*)'hello zB2g1Hmpmmsl'
      permm(1)=perm(1)
      permm(2)=perm(2)
      permm(3)=perm(3)
      permm(4)=perm(4)
      permm(5)=perm(5)
      permm(6)=perm(6)

      permmb(1)=permm(2)
      permmb(2)=permm(1)
      permmb(3)=permm(4)
      permmb(4)=permm(3)
      permmb(5)=permm(5)
      permmb(6)=permm(6)

      zB2g1Hmpmmsl=
     .    +zA41phiAQggmpmmsl(permm(1),renscale2)

     .    -zA41cphiAQggmpppsl(permmb(1))

      return
      end
************************************************************************
      double complex function zB2g1Hmpmmnf(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer  perm(6),permm(6),permmb(6)
*      write(*,*)'hello zB2g1Hmpmmnf'
      permm(1)=perm(1)
      permm(2)=perm(2)
      permm(3)=perm(3)
      permm(4)=perm(4)
      permm(5)=perm(5)
      permm(6)=perm(6)

      permmb(1)=permm(2)
      permmb(2)=permm(1)
      permmb(3)=permm(4)
      permmb(4)=permm(3)
      permmb(5)=permm(5)
      permmb(6)=permm(6)

      zB2g1Hmpmmnf=
     .    +zA41phiAQggmpmmnf(permm(1),renscale2)

     .   -zA41cphiAQggmpppnf(permmb(1))

      return
      end
************************************************************************
      double complex function zB2g1Hmpmpcl(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer  perm(6),permm(6),permmb(6)
*      write(*,*)'hello zB2g1Hmpmpcl'
      permm(1)=perm(1)
      permm(2)=perm(2)
      permm(3)=perm(3)
      permm(4)=perm(4)
      permm(5)=perm(5)
      permm(6)=perm(6)

      permmb(1)=permm(2)
      permmb(2)=permm(1)
      permmb(3)=permm(4)
      permmb(4)=permm(3)
      permmb(5)=permm(5)
      permmb(6)=permm(6)

      zB2g1Hmpmpcl=
     .    +zA41phiAQggmpmpcl(permm(1),renscale2)

     .   -zA41cphiAQggmpmpcl(permmb(1),renscale2)

      return
      end
************************************************************************
      double complex function zB2g1Hmpmpsl(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer  perm(6),permm(6),permmb(6)
*      write(*,*)'hello zB2g1Hmpmpsl'
      permm(1)=perm(1)
      permm(2)=perm(2)
      permm(3)=perm(3)
      permm(4)=perm(4)
      permm(5)=perm(5)
      permm(6)=perm(6)

      permmb(1)=permm(2)
      permmb(2)=permm(1)
      permmb(3)=permm(4)
      permmb(4)=permm(3)
      permmb(5)=permm(5)
      permmb(6)=permm(6)

      zB2g1Hmpmpsl=
     .    +zA41phiAQggmpmpsl(permm(1),renscale2)

     .   -zA41cphiAQggmpmpsl(permmb(1),renscale2)

      return
      end
************************************************************************
      double complex function zB2g1Hmpmpnf(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer  perm(6),permm(6),permmb(6)
*      write(*,*)'hello zB2g1Hmpmpnf'
      permm(1)=perm(1)
      permm(2)=perm(2)
      permm(3)=perm(3)
      permm(4)=perm(4)
      permm(5)=perm(5)
      permm(6)=perm(6)

      permmb(1)=permm(2)
      permmb(2)=permm(1)
      permmb(3)=permm(4)
      permmb(4)=permm(3)
      permmb(5)=permm(5)
      permmb(6)=permm(6)

      zB2g1Hmpmpnf=
     .    +zA41phiAQggmpmpnf(permm(1),renscale2)

     .    -zA41cphiAQggmpmpnf(permmb(1),renscale2)

      return
      end
************************************************************************
      double complex function zB2g1Hmppmcl(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer  perm(6),permm(6),permmb(6)
*      write(*,*)'hello zB2g1Hmppmcl'
      permm(1)=perm(1)
      permm(2)=perm(2)
      permm(3)=perm(3)
      permm(4)=perm(4)
      permm(5)=perm(5)
      permm(6)=perm(6)

      permmb(1)=permm(2)
      permmb(2)=permm(1)
      permmb(3)=permm(4)
      permmb(4)=permm(3)
      permmb(5)=permm(5)
      permmb(6)=permm(6)

      zB2g1Hmppmcl=
     .   +zA41phiAQggmppmcl(perm(1),renscale2)

     .   -zA41cphiAQggmppmcl(permmb(1),renscale2)

      return
      end
************************************************************************
      double complex function zB2g1Hmppmsl(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer  perm(6),permm(6),permmb(6)
*      write(*,*)'hello zB2g1Hmppmsl'
      permm(1)=perm(1)
      permm(2)=perm(2)
      permm(3)=perm(3)
      permm(4)=perm(4)
      permm(5)=perm(5)
      permm(6)=perm(6)

      permmb(1)=permm(2)
      permmb(2)=permm(1)
      permmb(3)=permm(4)
      permmb(4)=permm(3)
      permmb(5)=permm(5)
      permmb(6)=permm(6)

      zB2g1Hmppmsl=
     .   +zA41phiAQggmppmsl(perm(1),renscale2)

     .   -zA41cphiAQggmppmsl(permmb(1),renscale2)

      return
      end
************************************************************************
      double complex function zB2g1Hmppmnf(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer  perm(6),permm(6),permmb(6)
*      write(*,*)'hello zB2g1Hmppmnf'
      permm(1)=perm(1)
      permm(2)=perm(2)
      permm(3)=perm(3)
      permm(4)=perm(4)
      permm(5)=perm(5)
      permm(6)=perm(6)

      permmb(1)=permm(2)
      permmb(2)=permm(1)
      permmb(3)=permm(4)
      permmb(4)=permm(3)
      permmb(5)=permm(5)
      permmb(6)=permm(6)

      zB2g1Hmppmnf=
     .   +zA41phiAQggmppmnf(perm(1),renscale2)

     .   -zA41cphiAQggmppmnf(permmb(1),renscale2)

      return
      end
*****************************************************************************************
      double complex function zB2g1Hpmmmcl(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer  perm(6),permm(6),permmb(6)
*      write(*,*)'hello zB2g1Hpmmmcl'
      permm(1)=perm(1)
      permm(2)=perm(2)
      permm(3)=perm(3)
      permm(4)=perm(4)
      permm(5)=perm(5)
      permm(6)=perm(6)

      permmb(1)=permm(2)
      permmb(2)=permm(1)
      permmb(3)=permm(4)
      permmb(4)=permm(3)
      permmb(5)=permm(5)
      permmb(6)=permm(6)

      zB2g1Hpmmmcl=
     .   -zA41cphiAQggmpppcl(permm(1))

     .   +zA41phiAQggmpmmcl(permmb(1),renscale2)

      return
      end
*****************************************************************************************
      double complex function zB2g1Hpmmmsl(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer  perm(6),permm(6),permmb(6)
*      write(*,*)'hello zB2g1Hpmmmsl'
      permm(1)=perm(1)
      permm(2)=perm(2)
      permm(3)=perm(3)
      permm(4)=perm(4)
      permm(5)=perm(5)
      permm(6)=perm(6)

      permmb(1)=permm(2)
      permmb(2)=permm(1)
      permmb(3)=permm(4)
      permmb(4)=permm(3)
      permmb(5)=permm(5)
      permmb(6)=permm(6)

      zB2g1Hpmmmsl=
     .   -zA41cphiAQggmpppsl(permm(1))

     .   +zA41phiAQggmpmmsl(permmb(1),renscale2)

      return
      end
*****************************************************************************************
      double complex function zB2g1Hpmmmnf(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer  perm(6),permm(6),permmb(6)
*      write(*,*)'hello zB2g1Hpmmmnf'
      permm(1)=perm(1)
      permm(2)=perm(2)
      permm(3)=perm(3)
      permm(4)=perm(4)
      permm(5)=perm(5)
      permm(6)=perm(6)

      permmb(1)=permm(2)
      permmb(2)=permm(1)
      permmb(3)=permm(4)
      permmb(4)=permm(3)
      permmb(5)=permm(5)
      permmb(6)=permm(6)

      zB2g1Hpmmmnf=
     .   -zA41cphiAQggmpppnf(permm(1))

     .   +zA41phiAQggmpmmnf(permmb(1),renscale2)

      return
      end
************************************************************************
      double complex function zB2g1Hpmppcl(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer  perm(6),permm(6),permmb(6)
*      write(*,*)'hello zB2g1Hpmppcl'
      permm(1)=perm(1)
      permm(2)=perm(2)
      permm(3)=perm(3)
      permm(4)=perm(4)
      permm(5)=perm(5)
      permm(6)=perm(6)

      permmb(1)=permm(2)
      permmb(2)=permm(1)
      permmb(3)=permm(4)
      permmb(4)=permm(3)
      permmb(5)=permm(5)
      permmb(6)=permm(6)

      zB2g1Hpmppcl=
     .    -zA41cphiAQggmpmmcl(permm(1),renscale2)

     .    +zA41phiAQggmpppcl(permmb(1))

      return
      end
************************************************************************
      double complex function zB2g1Hpmppsl(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer  perm(6),permm(6),permmb(6)
*      write(*,*)'hello zB2g1Hpmppsl'
      permm(1)=perm(1)
      permm(2)=perm(2)
      permm(3)=perm(3)
      permm(4)=perm(4)
      permm(5)=perm(5)
      permm(6)=perm(6)

      permmb(1)=permm(2)
      permmb(2)=permm(1)
      permmb(3)=permm(4)
      permmb(4)=permm(3)
      permmb(5)=permm(5)
      permmb(6)=permm(6)

      zB2g1Hpmppsl=
     .    -zA41cphiAQggmpmmsl(permm(1),renscale2)

     .    +zA41phiAQggmpppsl(permmb(1))

      return
      end
************************************************************************
      double complex function zB2g1Hpmppnf(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer  perm(6),permm(6),permmb(6)
*      write(*,*)'hello zB2g1Hpmppnf'
      permm(1)=perm(1)
      permm(2)=perm(2)
      permm(3)=perm(3)
      permm(4)=perm(4)
      permm(5)=perm(5)
      permm(6)=perm(6)

      permmb(1)=permm(2)
      permmb(2)=permm(1)
      permmb(3)=permm(4)
      permmb(4)=permm(3)
      permmb(5)=permm(5)
      permmb(6)=permm(6)

      zB2g1Hpmppnf=
     .    -zA41cphiAQggmpmmnf(permm(1),renscale2)

     .    +zA41phiAQggmpppnf(permmb(1))

      return
      end
************************************************************************
      double complex function zB2g1Hpmpmcl(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer  perm(6),permm(6),permmb(6)
*      write(*,*)'hello zB2g1Hpmpmcl'
      permm(1)=perm(1)
      permm(2)=perm(2)
      permm(3)=perm(3)
      permm(4)=perm(4)
      permm(5)=perm(5)
      permm(6)=perm(6)

      permmb(1)=permm(2)
      permmb(2)=permm(1)
      permmb(3)=permm(4)
      permmb(4)=permm(3)
      permmb(5)=permm(5)
      permmb(6)=permm(6)

      zB2g1Hpmpmcl=
     .    -zA41cphiAQggmpmpcl(permm(1),renscale2)

     .    +zA41phiAQggmpmpcl(permmb(1),renscale2)


      return
      end
************************************************************************
      double complex function zB2g1Hpmpmsl(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer  perm(6),permm(6),permmb(6)
*      write(*,*)'hello zB2g1Hpmpmsl'
      permm(1)=perm(1)
      permm(2)=perm(2)
      permm(3)=perm(3)
      permm(4)=perm(4)
      permm(5)=perm(5)
      permm(6)=perm(6)

      permmb(1)=permm(2)
      permmb(2)=permm(1)
      permmb(3)=permm(4)
      permmb(4)=permm(3)
      permmb(5)=permm(5)
      permmb(6)=permm(6)

      zB2g1Hpmpmsl=
     .    -zA41cphiAQggmpmpsl(permm(1),renscale2)

     .    +zA41phiAQggmpmpsl(permmb(1),renscale2)

      return
      end
************************************************************************
      double complex function zB2g1Hpmpmnf(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer  perm(6),permm(6),permmb(6)
*      write(*,*)'hello zB2g1Hpmpmnf'
      permm(1)=perm(1)
      permm(2)=perm(2)
      permm(3)=perm(3)
      permm(4)=perm(4)
      permm(5)=perm(5)
      permm(6)=perm(6)

      permmb(1)=permm(2)
      permmb(2)=permm(1)
      permmb(3)=permm(4)
      permmb(4)=permm(3)
      permmb(5)=permm(5)
      permmb(6)=permm(6)

      zB2g1Hpmpmnf=
     .    -zA41cphiAQggmpmpnf(permm(1),renscale2)

     .    +zA41phiAQggmpmpnf(permmb(1),renscale2)

      return
      end

************************************************************************
      double complex function zB2g1Hpmmpcl(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer  perm(6),permm(6),permmb(6)
*      write(*,*)'hello zB2g1Hpmmpcl'
      permm(1)=perm(1)
      permm(2)=perm(2)
      permm(3)=perm(3)
      permm(4)=perm(4)
      permm(5)=perm(5)
      permm(6)=perm(6)

      permmb(1)=permm(2)
      permmb(2)=permm(1)
      permmb(3)=permm(4)
      permmb(4)=permm(3)
      permmb(5)=permm(5)
      permmb(6)=permm(6)

      zB2g1Hpmmpcl=
     .    -zA41cphiAQggmppmcl(perm(1),renscale2)

     .    +zA41phiAQggmppmcl(permmb(1),renscale2)

      return
      end
************************************************************************
      double complex function zB2g1Hpmmpsl(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer  perm(6),permm(6),permmb(6)
*      write(*,*)'hello zB2g1Hpmmpsl'
      permm(1)=perm(1)
      permm(2)=perm(2)
      permm(3)=perm(3)
      permm(4)=perm(4)
      permm(5)=perm(5)
      permm(6)=perm(6)

      permmb(1)=permm(2)
      permmb(2)=permm(1)
      permmb(3)=permm(4)
      permmb(4)=permm(3)
      permmb(5)=permm(5)
      permmb(6)=permm(6)

      zB2g1Hpmmpsl=
     .    -zA41cphiAQggmppmsl(perm(1),renscale2)

     .    +zA41phiAQggmppmsl(permmb(1),renscale2)

      return
      end
************************************************************************
      double complex function zB2g1Hpmmpnf(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer  perm(6),permm(6),permmb(6)
*      write(*,*)'hello zB2g1Hpmmpnf'
      permm(1)=perm(1)
      permm(2)=perm(2)
      permm(3)=perm(3)
      permm(4)=perm(4)
      permm(5)=perm(5)
      permm(6)=perm(6)

      permmb(1)=permm(2)
      permmb(2)=permm(1)
      permmb(3)=permm(4)
      permmb(4)=permm(3)
      permmb(5)=permm(5)
      permmb(6)=permm(6)

      zB2g1Hpmmpnf=
     .    -zA41cphiAQggmppmnf(perm(1),renscale2)

     .    +zA41phiAQggmppmnf(permmb(1),renscale2)

      return
      end
************************************************************************
      double complex function zBt2g1Hmppp(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer  perm(6),permm(6)
*      write(*,*)'hello zBt2g1Hmppp'
      permm(1)=perm(2)
      permm(2)=perm(1)
      permm(3)=perm(4)
      permm(4)=perm(3)
      permm(5)=perm(5)
      permm(6)=perm(6)

      zBt2g1Hmppp=
     .  -zA43cphiAQggmpmm(permm(1),renscale2)

      return
      end
************************************************************************
      double complex function zBt2g1Hmpmm(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer  perm(6),permm(6)
*      write(*,*)'hello zBt2g1Hmpmm'

      zBt2g1Hmpmm=
     .    zA43phiAQggmpmm(perm(1),renscale2)

      return
      end
************************************************************************
      double complex function zBt2g1Hmpmp(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer  perm(6),permm(6)
*      write(*,*)'hello zBt2g1Hmpmp'
      permm(1)=perm(2)
      permm(2)=perm(1)
      permm(3)=perm(4)
      permm(4)=perm(3)
      permm(5)=perm(5)
      permm(6)=perm(6)

      zBt2g1Hmpmp=
     .    zA43phiAQggmpmp(perm(1),renscale2)
     .   -zA43cphiAQggmpmp(permm(1),renscale2)

      return
      end
************************************************************************
      double complex function zBt2g1Hmppm(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer  perm(6),permm(6)
*      write(*,*)'hello zBt2g1Hmppm'
      permm(1)=perm(2)
      permm(2)=perm(1)
      permm(3)=perm(4)
      permm(4)=perm(3)
      permm(5)=perm(5)
      permm(6)=perm(6)

      zBt2g1Hmppm=
     .    zA43phiAQggmppm(perm(1),renscale2)
     .   -zA43cphiAQggmppm(permm(1),renscale2)

      return
      end
************************************************************************
      double complex function zBt2g1Hpmmm(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer  perm(6),permm(6)
*      write(*,*)'hello zBt2g1Hpmmm'
      permm(1)=perm(2)
      permm(2)=perm(1)
      permm(3)=perm(4)
      permm(4)=perm(3)
      permm(5)=perm(5)
      permm(6)=perm(6)

      zBt2g1Hpmmm=
     .   zA43phiAQggmpmm(permm(1),renscale2)

      return
      end
************************************************************************
      double complex function zBt2g1Hpmpp(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer  perm(6),permm(6)
*      write(*,*)'hello zBt2g1Hpmpp'

      zBt2g1Hpmpp=
     .    -zA43cphiAQggmpmm(perm(1),renscale2)

      return
      end
************************************************************************
      double complex function zBt2g1Hpmpm(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer  perm(6),permm(6)
*      write(*,*)'hello zBt2g1Hmpmp'
      permm(1)=perm(2)
      permm(2)=perm(1)
      permm(3)=perm(4)
      permm(4)=perm(3)
      permm(5)=perm(5)
      permm(6)=perm(6)

      zBt2g1Hpmpm=
     .   -zA43cphiAQggmpmp(perm(1),renscale2)
     .   +zA43phiAQggmpmp(permm(1),renscale2)

      return
      end
************************************************************************
      double complex function zBt2g1Hpmmp(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      integer  perm(6),permm(6)
*      write(*,*)'hello zBt2g1Hpmmp'
      permm(1)=perm(2)
      permm(2)=perm(1)
      permm(3)=perm(4)
      permm(4)=perm(3)
      permm(5)=perm(5)
      permm(6)=perm(6)

      zBt2g1Hpmmp=
     .   -zA43cphiAQggmppm(perm(1),renscale2)
     .   +zA43phiAQggmppm(permm(1),renscale2)

      return
      end
************************************************************************
      double complex function zA41phiAQggmpppcl(perm)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /order/ieorder
      common /koppel/ nf,nc,b0
      integer  perm(6)
      double complex zA1phiAQggmpppL,zA1phiAQggmpppR,zA1phiAQggmpppF
*      write(*,*)'hello zA41phiAQggmppp'

c--- implementation of arXiv:0906.0008, Eq.(2.21)

      if(ieorder.eq.-2)then
      zA41phiAQggmpppcl=0d0
      elseif(ieorder.eq.-1)then
      zA41phiAQggmpppcl=0d0
      elseif(ieorder.eq.0)then

      zA41phiAQggmpppcl=
     .     +zA1phiAQggmpppL(perm(1))

      endif
      return
      end
************************************************************************
      double complex function zA41phiAQggmpppsl(perm)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /order/ieorder
      common /koppel/ nf,nc,b0
      integer  perm(6)
      double complex zA1phiAQggmpppL,zA1phiAQggmpppR,zA1phiAQggmpppF
*      write(*,*)'hello zA41phiAQggmppp'

c--- implementation of arXiv:0906.0008, Eq.(2.21)

      if(ieorder.eq.-2)then
      zA41phiAQggmpppsl=0d0
      elseif(ieorder.eq.-1)then
      zA41phiAQggmpppsl=0d0
      elseif(ieorder.eq.0)then

      zA41phiAQggmpppsl=
     .     zA1phiAQggmpppR(perm(1))

      endif
      return
      end
************************************************************************
      double complex function zA41phiAQggmpppnf(perm)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /order/ieorder
      common /koppel/ nf,nc,b0
      integer  perm(6)
      double complex zA1phiAQggmpppL,zA1phiAQggmpppR,zA1phiAQggmpppF
*      write(*,*)'hello zA41phiAQggmppp'

c--- implementation of arXiv:0906.0008, Eq.(2.21)

      if(ieorder.eq.-2)then
      zA41phiAQggmpppnf=0d0
      elseif(ieorder.eq.-1)then
      zA41phiAQggmpppnf=0d0
      elseif(ieorder.eq.0)then

      zA41phiAQggmpppnf=
     .     +zA1phiAQggmpppF(perm(1))

      endif
      return
      end
************************************************************************
      double complex function zA41cphiAQggmpppcl(perm)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /order/ieorder
      common /koppel/ nf,nc,b0
      integer  perm(6)
      double complex zA1cphiAQggmpppL,zA1cphiAQggmpppR,zA1cphiAQggmpppF
*      write(*,*)'hello zA41cphiAQggmppp'

c--- implementation of arXiv:0906.0008, Eq.(2.21)

      if(ieorder.eq.-2)then

      zA41cphiAQggmpppcl=0d0

      elseif(ieorder.eq.-1)then

      zA41cphiAQggmpppcl=0d0

      elseif(ieorder.eq.0)then

      zA41cphiAQggmpppcl=
     .     +zA1cphiAQggmpppL(perm(1))

      endif
      return
      end
************************************************************************
      double complex function zA41cphiAQggmpppsl(perm)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /order/ieorder
      common /koppel/ nf,nc,b0
      integer  perm(6)
      double complex zA1cphiAQggmpppL,zA1cphiAQggmpppR,zA1cphiAQggmpppF
*      write(*,*)'hello zA41cphiAQggmppp'

c--- implementation of arXiv:0906.0008, Eq.(2.21)

      if(ieorder.eq.-2)then

      zA41cphiAQggmpppsl=0d0

      elseif(ieorder.eq.-1)then

      zA41cphiAQggmpppsl=0d0

      elseif(ieorder.eq.0)then

      zA41cphiAQggmpppsl=
     .     zA1cphiAQggmpppR(perm(1))

      endif
      return
      end
************************************************************************
      double complex function zA41cphiAQggmpppnf(perm)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /order/ieorder
      common /koppel/ nf,nc,b0
      integer  perm(6)
      double complex zA1cphiAQggmpppL,zA1cphiAQggmpppR,zA1cphiAQggmpppF
*      write(*,*)'hello zA41cphiAQggmppp'

c--- implementation of arXiv:0906.0008, Eq.(2.21)

      if(ieorder.eq.-2)then

      zA41cphiAQggmpppnf=0d0

      elseif(ieorder.eq.-1)then

      zA41cphiAQggmpppnf=0d0

      elseif(ieorder.eq.0)then

      zA41cphiAQggmpppnf=
     .     +zA1cphiAQggmpppF(perm(1))

      endif
      return
      end

************************************************************************
      double complex function zA41phiAQggmpmmcl(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /koppel/ nf,nc,b0
      integer  perm(6)
      double complex zA1phiAQggmpmmL

c--- implementation of arXiv:0906.0008, Eq.(2.21)
*      write(*,*)'hello zA41phiAQggmpmm'

      zA41phiAQggmpmmcl=
     .     +zA1phiAQggmpmmL(perm(1),renscale2)

      return
      end
************************************************************************
      double complex function zA41phiAQggmpmmsl(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /koppel/ nf,nc,b0
      integer  perm(6)
      double complex zA1phiAQggmpmmR

c--- implementation of arXiv:0906.0008, Eq.(2.21)
*      write(*,*)'hello zA41phiAQggmpmm'

      zA41phiAQggmpmmsl=
     .     zA1phiAQggmpmmR(perm(1),renscale2)
      return
      end
************************************************************************
      double complex function zA41phiAQggmpmmnf(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /koppel/ nf,nc,b0
      integer  perm(6)
      double complex zA1phiAQggmpmmF

c--- implementation of arXiv:0906.0008, Eq.(2.21)
*      write(*,*)'hello zA41phiAQggmpmm'

      zA41phiAQggmpmmnf=
     .     +zA1phiAQggmpmmF(perm(1),renscale2)

      return
      end

************************************************************************
      double complex function zA41cphiAQggmpmmcl(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /koppel/ nf,nc,b0
      integer  perm(6)
      double complex zA1cphiAQggmpmmL

c--- implementation of arXiv:0906.0008, Eq.(2.21)
*      write(*,*)'hello zA41cphiAQggmpmm'

      zA41cphiAQggmpmmcl=
     .     +zA1cphiAQggmpmmL(perm(1),renscale2)

      return
      end
************************************************************************
      double complex function zA41cphiAQggmpmmsl(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /koppel/ nf,nc,b0
      integer  perm(6)
      double complex zA1cphiAQggmpmmR

c--- implementation of arXiv:0906.0008, Eq.(2.21)
*      write(*,*)'hello zA41cphiAQggmpmm'

      zA41cphiAQggmpmmsl=
     .     zA1cphiAQggmpmmR(perm(1),renscale2)
      return
      end
************************************************************************
      double complex function zA41cphiAQggmpmmnf(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /koppel/ nf,nc,b0
      integer  perm(6)
      double complex zA1cphiAQggmpmmF

c--- implementation of arXiv:0906.0008, Eq.(2.21)
*      write(*,*)'hello zA41cphiAQggmpmm'

      zA41cphiAQggmpmmnf=
     .     +zA1cphiAQggmpmmF(perm(1),renscale2)

      return
      end
************************************************************************
      double complex function zA41phiAQggmpmpcl(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /koppel/ nf,nc,b0
      integer  perm(6)
      double complex zA1phiAQggmpmpL
*      write(*,*)'hello zA41phiAQggmpmp'
c--- implementation of arXiv:0906.0008, Eq.(2.21)
      zA41phiAQggmpmpcl=
     .     +zA1phiAQggmpmpL(perm(1),renscale2)

      return
      end
************************************************************************
      double complex function zA41phiAQggmpmpsl(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /koppel/ nf,nc,b0
      integer  perm(6)
      double complex zA1phiAQggmpmpR
*      write(*,*)'hello zA41phiAQggmpmp'
c--- implementation of arXiv:0906.0008, Eq.(2.21)
      zA41phiAQggmpmpsl=
     .     zA1phiAQggmpmpR(perm(1),renscale2)

      return
      end
************************************************************************
      double complex function zA41phiAQggmpmpnf(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /koppel/ nf,nc,b0
      integer  perm(6)
      double complex zA1phiAQggmpmpF
*      write(*,*)'hello zA41phiAQggmpmp'
c--- implementation of arXiv:0906.0008, Eq.(2.21)
      zA41phiAQggmpmpnf=
     .     +zA1phiAQggmpmpF(perm(1),renscale2)

      return
      end

************************************************************************
      double complex function zA41cphiAQggmpmpcl(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /koppel/ nf,nc,b0
      integer  perm(6)
      double complex zA1cphiAQggmpmpL
*      write(*,*)'hello zA41cphiAQggmpmp'
c--- implementation of arXiv:0906.0008, Eq.(2.21)
      zA41cphiAQggmpmpcl=
     .     +zA1cphiAQggmpmpL(perm(1),renscale2)

      return
      end
************************************************************************
      double complex function zA41cphiAQggmpmpsl(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /koppel/ nf,nc,b0
      integer  perm(6)
      double complex zA1cphiAQggmpmpR
*      write(*,*)'hello zA41cphiAQggmpmp'
c--- implementation of arXiv:0906.0008, Eq.(2.21)
      zA41cphiAQggmpmpsl=
     .     zA1cphiAQggmpmpR(perm(1),renscale2)

      return
      end
************************************************************************
      double complex function zA41cphiAQggmpmpnf(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /koppel/ nf,nc,b0
      integer  perm(6)
      double complex zA1cphiAQggmpmpF
*      write(*,*)'hello zA41cphiAQggmpmp'
c--- implementation of arXiv:0906.0008, Eq.(2.21)
      zA41cphiAQggmpmpnf=
     .     +zA1cphiAQggmpmpF(perm(1),renscale2)

      return
      end

************************************************************************
      double complex function zA41phiAQggmppmcl(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /koppel/ nf,nc,b0
      integer  perm(6)
      double complex zA1phiAQggmppmL
*      write(*,*)'hello zA41phiAQggmppm'

c--- implementation of arXiv:0906.0008, Eq.(2.21)
      zA41phiAQggmppmcl=
     .     +zA1phiAQggmppmL(perm(1),renscale2)

      return
      end
************************************************************************
      double complex function zA41phiAQggmppmsl(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /koppel/ nf,nc,b0
      integer  perm(6)
      double complex zA1phiAQggmppmR
*      write(*,*)'hello zA41phiAQggmppm'

c--- implementation of arXiv:0906.0008, Eq.(2.21)
      zA41phiAQggmppmsl=
     .     zA1phiAQggmppmR(perm(1),renscale2)

      return
      end
************************************************************************
      double complex function zA41phiAQggmppmnf(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /koppel/ nf,nc,b0
      integer  perm(6)
      double complex zA1phiAQggmppmL,zA1phiAQggmppmF
*      write(*,*)'hello zA41phiAQggmppm'

c--- implementation of arXiv:0906.0008, Eq.(2.21)
      zA41phiAQggmppmnf=
     .     +zA1phiAQggmppmF(perm(1),renscale2)

      return
      end

************************************************************************
      double complex function zA41cphiAQggmppmcl(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /koppel/ nf,nc,b0
      integer  perm(6)
      double complex zA1cphiAQggmppmL
*      write(*,*)'hello zA41phiAQggmppm'

c--- implementation of arXiv:0906.0008, Eq.(2.21)
      zA41cphiAQggmppmcl=
     .     +zA1cphiAQggmppmL(perm(1),renscale2)

      return
      end
************************************************************************
      double complex function zA41cphiAQggmppmsl(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /koppel/ nf,nc,b0
      integer  perm(6)
      double complex zA1cphiAQggmppmR
*      write(*,*)'hello zA41phiAQggmppm'

c--- implementation of arXiv:0906.0008, Eq.(2.21)
      zA41cphiAQggmppmsl=
     .     zA1cphiAQggmppmR(perm(1),renscale2)

      return
      end
************************************************************************
      double complex function zA41cphiAQggmppmnf(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /koppel/ nf,nc,b0
      integer  perm(6)
      double complex zA1cphiAQggmppmL
*      write(*,*)'hello zA41phiAQggmppm'

c--- implementation of arXiv:0906.0008, Eq.(2.21)
      zA41cphiAQggmppmnf=
     .     +zA1cphiAQggmppmF(perm(1),renscale2)

      return
      end

************************************************************************
      double complex function zA43cphiAQggmpmm(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /koppel/ nf,nc,b0
      integer j,j1,j2,j3,j4,j5,j6,perm(6),permm(6)
*      write(*,*)'hello zA43cphiAQggmpmm'
      permm(1)=perm(1)
      permm(2)=perm(2)
      permm(3)=perm(4)
      permm(4)=perm(3)
      permm(5)=perm(5)
      permm(6)=perm(6)

      zA43cphiAQggmpmm=
     & +zA43cphiAQggmpmm_unsym(perm(1),renscale2)
     & +zA43cphiAQggmpmm_unsym(permm(1),renscale2)
      return
      end
************************************************************************
      double complex function zA43phiAQggmpmm(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /koppel/ nf,nc,b0
      integer j,j1,j2,j3,j4,j5,j6,perm(6),permm(6)
*      write(*,*)'hello zA43phiAQggmpmm'
      permm(1)=perm(1)
      permm(2)=perm(2)
      permm(3)=perm(4)
      permm(4)=perm(3)
      permm(5)=perm(5)
      permm(6)=perm(6)

      zA43phiAQggmpmm=
     & +zA43phiAQggmpmm_unsym(perm(1),renscale2)
     & +zA43phiAQggmpmm_unsym(permm(1),renscale2)
      return
      end
************************************************************************
      double complex function zA43phiAQggmpmp(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /koppel/ nf,nc,b0
      integer j,j1,j2,j3,j4,j5,j6,perm(6),permm(6)
*      write(*,*)'hello zA43phiAQggmpmp'
      permm(1)=perm(1)
      permm(2)=perm(2)
      permm(3)=perm(4)
      permm(4)=perm(3)
      permm(5)=perm(5)
      permm(6)=perm(6)

      zA43phiAQggmpmp=
     & +zA43phiAQggmpmp_unsym(perm(1),renscale2)
     & +zA43phiAQggmppm_unsym(permm(1),renscale2)
      return
      end
************************************************************************
      double complex function zA43cphiAQggmpmp(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /koppel/ nf,nc,b0
      integer j,j1,j2,j3,j4,j5,j6,perm(6),permm(6)
*      write(*,*)'hello zA43cphiAQggmpmp'

      permm(1)=perm(1)
      permm(2)=perm(2)
      permm(3)=perm(4)
      permm(4)=perm(3)
      permm(5)=perm(5)
      permm(6)=perm(6)

      zA43cphiAQggmpmp=
     & +zA43cphiAQggmpmp_unsym(perm(1),renscale2)
     & +zA43cphiAQggmppm_unsym(permm(1),renscale2)
      return
      end
************************************************************************
      double complex function zA43phiAQggmppm(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /koppel/ nf,nc,b0
      integer j,j1,j2,j3,j4,j5,j6,perm(6),permm(6)
*      write(*,*)'hello zA43phiAQggmppm'

      permm(1)=perm(1)
      permm(2)=perm(2)
      permm(3)=perm(4)
      permm(4)=perm(3)
      permm(5)=perm(5)
      permm(6)=perm(6)

      zA43phiAQggmppm=
     & +zA43phiAQggmppm_unsym(perm(1),renscale2)
     & +zA43phiAQggmpmp_unsym(permm(1),renscale2)
      return
      end
************************************************************************
      double complex function zA43cphiAQggmppm(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /koppel/ nf,nc,b0
      integer j,j1,j2,j3,j4,j5,j6,perm(6),permm(6)
*      write(*,*)'hello zA43phiAQggmppm'
      permm(1)=perm(1)
      permm(2)=perm(2)
      permm(3)=perm(4)
      permm(4)=perm(3)
      permm(5)=perm(5)
      permm(6)=perm(6)

      zA43cphiAQggmppm=
     & +zA43cphiAQggmppm_unsym(perm(1),renscale2)
     & +zA43cphiAQggmpmp_unsym(permm(1),renscale2)
      return
      end
*****************************************************************************************
*****************************************************************************************
******************** Library for 1-loop sub-amplitudes with ieorder**********************
*****************************************************************************************
*****************************************************************************************
      double complex function zA1cphiAQggmpmmL(perm,renscale2)
c--- This is an implementation of Eq. (5.2) in
c---  S.~Badger, John.~M.~Campbell, R.~Keith Ellis and Ciaran Williams
c---  "Analytic results for the one-loop NMHV H-qbar-q-g-g amplitude."
c---   preprint DESY 09-180, FERMILAB-PUB-09-505-T, IPPP/09/86
c---   arXiv: 0910.4481 [hep-ph]
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /order/ieorder
      common /koppel/ nf,nc,b0

      integer j,j1,j2,j3,j4,j5,j6,perm(6),permm(6)
      double complex V1L,zB2g0Hmmmp,lnrat,zba2,Lsm1,Lsm1_2mht,czip
      double complex sum,l23,l34,l41,l12,coef3m1234,coef3m1423,
     & S1,S2,K1DK2,a1,a2,a3,a4,gamma,gammap,gammam,factor,I3m,
     & BGRL1,BGRL2hat,BGRL3hat,za(5,5),zb(5,5),d1,d2
      double precision s123,s234,s124,s134,mhsq,epinv,deltar,musq
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43
      integer ipset

      musq=renscale2
      epinv=0d0
      deltar=1d0
      czip=(0d0,0d0)

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      permm(1)=perm(1)
      permm(2)=perm(4)
      permm(3)=perm(3)
      permm(4)=perm(2)
      permm(5)=perm(5)
      permm(6)=perm(6)

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zB(j1,j2)
      za13 => kin(ipset)%zB(j1,j3)
      za14 => kin(ipset)%zB(j1,j4)
      za23 => kin(ipset)%zB(j2,j3)
      za24 => kin(ipset)%zB(j2,j4)
      za34 => kin(ipset)%zB(j3,j4)

      za21 => kin(ipset)%zB(j2,j1)
      za31 => kin(ipset)%zB(j3,j1)
      za41 => kin(ipset)%zB(j4,j1)
      za32 => kin(ipset)%zB(j3,j2)
      za42 => kin(ipset)%zB(j4,j2)
      za43 => kin(ipset)%zB(j4,j3)

      zb12 => kin(ipset)%zA(j1,j2)
      zb13 => kin(ipset)%zA(j1,j3)
      zb14 => kin(ipset)%zA(j1,j4)
      zb23 => kin(ipset)%zA(j2,j3)
      zb24 => kin(ipset)%zA(j2,j4)
      zb34 => kin(ipset)%zA(j3,j4)

      zb21 => kin(ipset)%zA(j2,j1)
      zb31 => kin(ipset)%zA(j3,j1)
      zb41 => kin(ipset)%zA(j4,j1)
      zb32 => kin(ipset)%zA(j3,j2)
      zb42 => kin(ipset)%zA(j4,j2)
      zb43 => kin(ipset)%zA(j4,j3)

      z2ba2143=zba2(j2,j1,j4,j3,ipset)
      z2ba1342=zba2(j1,j3,j4,j2,ipset)
      z2ba1234=zba2(j1,j2,j3,j4,ipset)
      z2ba4123=zba2(j4,j1,j2,j3,ipset)
      z2ba3142=zba2(j3,j1,j4,j2,ipset)
      z2ba3124=zba2(j3,j1,j2,j4,ipset)
      z2ba4231=zba2(j4,j2,j3,j1,ipset)
      z2ba4132=zba2(j4,j1,j3,j2,ipset)
      z2ba3241=zba2(j3,j2,j4,j1,ipset)
      z2ba4134=zba2(j4,j1,j3,j4,ipset)

      mhsq=s12+s13+s14+s23+s24+s34
      s123=s12+s13+s23
      s124=s12+s14+s24
      s134=s13+s14+s34
      s234=s23+s24+s34

      l12=lnrat(musq,-s12)
      l23=lnrat(musq,-s23)
      l34=lnrat(musq,-s34)
      l41=lnrat(musq,-s14)

      if(ieorder.eq.-2)then
      V1L=-3d0
      zA1cphiAQggmpmmL=zB2g0Hpppm(permm(1))*V1L
      elseif(ieorder.eq.-1)then
      V1L=
     . -l23
     . -l34
     . -l41
     . +13d0/6d0
      zA1cphiAQggmpmmL=zB2g0Hpppm(permm(1))*V1L
      elseif(ieorder.eq.0)then
      V1L=
     . -epinv**2-epinv*l23-0.5d0*l23**2
     . -epinv**2-epinv*l34-0.5d0*l34**2
     . -epinv**2-epinv*l41-0.5d0*l41**2
     . +13d0/6d0*(epinv+l12)+119d0/18d0-deltar/6d0

      zA1cphiAQggmpmmL=zB2g0Hpppm(permm(1))*V1L
      sum=
     & -s134**2/(zb41*zb34*z2ba2143)
     & *Lsm1(-s14,-s134,-s34,-s134)

     & -z2ba1342**2/(z2ba1234*zb23*zb34)
     & *Lsm1(-s34,-s234,-s23,-s234)

     &  +(mhsq**2*za14**2*za24
     & /(za12*z2ba2143*z2ba4123*s124)
     &   -z2ba3142**3
     & /(zb12*zb24*z2ba3124*s124))
     & *Lsm1(-s14,-s124,-s12,-s124)

     & +(zb23**2*z2ba4231**3
     & /(zb12*zb13**3*z2ba4123*s123)
     & -mhsq**2*za13**3
     & /(za12*z2ba1234*z2ba3124*s123))
     & *Lsm1(-s12,-s123,-s23,-s123)

     &   +za34*s134**2
     & /(zb34*za34*zb14*z2ba2143)
     & *Lsm1_2mht(s12,s134,s34,mhsq)

     & -z2ba1342**2
     & /(zb34*zb23*z2ba1234)
     & *Lsm1_2mht(s12,s234,s34,mhsq)

     & +(z2ba4132**3
     & /(zb12*zb23*z2ba4123*s123)
     & -mhsq**2*za13**3
     & /(za12*z2ba1234*z2ba3124*s123))
     & *Lsm1_2mht(s34,s123,s12,mhsq)

     & +(mhsq**2*za14**2*za24
     & /(za12*z2ba2143*z2ba4123*s124)
     & -z2ba3241*z2ba3142**2
     & /(z2ba3124*zb14*zb12*s124))
     & *Lsm1_2mht(s34,s124,s12,mhsq)

     &  +(z2ba4132**3
     & /(zb12*zb23*z2ba4123*s123)
     & -mhsq**2*za13**3
     & /(za12*z2ba1234*z2ba3124*s123))
     & *Lsm1_2mht(s14,s123,s23,mhsq)

     & +(mhsq**2*za24*za14**2
     & /(za12*z2ba2143*z2ba4123*s124)
     & -z2ba3142**2*z2ba3241
     & /(zb12*zb14*z2ba3124*s124))
     & *Lsm1_2mht(s23,s124,s14,mhsq)

C-----Now for three mass triangles
C-----Deal with the 12-34 case first
C-----K1=-(p1+p2+p3+p4)
C-----K2=-(p1+p2)
C     K1flat=gamma/(gamma**2-S1*S2)*(gamma*K1-S1*K2)
C     K1flat=gamma/(gamma**2-S1*S2)*((S1-gamma)*(p1+p2)-gamma*((p3+p4))
C----solve for gamma_+ and gamma_-
      S1=dcmplx(mhsq)
      S2=dcmplx(s12)
      K1DK2=dcmplx(s12+0.5d0*(s13+s14+s23+s24))

C-gamma+ = K1DK2+sqrt(K1DK2**2-S1*S2)
C-gamma- = K1DK2-sqrt(K1DK2**2-S1*S2)

      coef3m1234=czip
      gammap=K1DK2+sqrt(K1DK2**2-S1*S2)
      gammam=2d0*K1DK2-gammap
C -- calculate the projections of K1 flat on k1,k2,k3,k4 called a1,a2,a3,a4
      factor=gammap/(gammap**2-S1*S2)
      a1=factor*(S1-gammap)
      a2=a1
      a3=-factor*gammap
      a4=a3

*XC rewrite (a1*s14+a2*s24+a3*s34)=d4K=d1 for better numerical stability in 3 soft limit
      d1=factor*s34*mhsq/gammam/(gammam-mhsq)
     .  *(mhsq*s12*s34/(s12-gammap)+gammam*(s13+s23+s34))
*XC rewrite (a1*s13+a2*s23+a4*s34)=d3K=d2 for better numerical stability in 4 soft limit
      d2=factor*s34*mhsq/gammam/(gammam-mhsq)
     .  *(mhsq*s12*s34/(s12-gammap)+gammam*(s14+s24+s34))

      coef3m1234=coef3m1234
     & +mhsq**2*za34**3/(za12*gammap*(gammap-mhsq))
     & *(a2*za12*zb23+a4*za14*zb43)   ! zab2(k1,k1f,k3)
     & *(a2*za12*zb24+a3*za13*zb34)   ! zab2(k1,k1f,k4)
     * /d2                                                ! (2*k3.k1f)^-1*
     * /d1

C----switch to other solution

      factor=gammam/(gammam**2-S1*S2)
      a1=factor*(S1-gammam)
      a2=a1
      a3=-factor*gammam
      a4=a3

*XC rewrite (a1*s14+a2*s24+a3*s34)=d4K=d1 for better numerical stability in 3 soft limit
      d1=factor*s34*mhsq/gammap/(gammap-mhsq)
     .  *(mhsq*s12*s34/(s12-gammam)+gammap*(s13+s23+s34))
*XC rewrite (a1*s13+a2*s23+a4*s34)=d3K=d2 for better numerical stability in 4 soft limit
      d2=factor*s34*mhsq/gammap/(gammap-mhsq)
     .  *(mhsq*s12*s34/(s12-gammam)+gammap*(s14+s24+s34))

      coef3m1234=coef3m1234
     & +mhsq**2*za34**3/(za12*gammam*(gammam-mhsq))
     & *(a2*za12*zb23+a4*za14*zb43)   ! zab2(k1,k1f,k3)
     & *(a2*za12*zb24+a3*za13*zb34)   ! zab2(k1,k1f,k4)
     * /d2                                                ! (2*k3.k1f)^-1*
     * /d1

C-----Now deal with the 14-23 case
C-----K1=-(p1+p2+p3+p4)
C-----K2=-(p1+p4)
C     K1flat=gamma/(gamma**2-S1*S2)*(gamma*K1-S1*K2)
C     K1flat=gamma/(gamma**2-S1*S2)*((S1-gamma)*(p1+p4)-gamma*((p2+p3))

C----solve for gamma_+ and gamma_-
      S1=dcmplx(mhsq)
      S2=dcmplx(s14)
      K1DK2=dcmplx(s14+0.5d0*(s12+s13+s24+s34))

C-gamma+ = K1DK2+sqrt(K1DK2**2-S1*S2)
C-gamma- = K1DK2-sqrt(K1DK2**2-S1*S2)

      coef3m1423=czip
      gammap=K1DK2+sqrt(K1DK2**2-S1*S2)
      gammam=2d0*K1DK2-gammap

      factor=gammap/(gammap**2-S1*S2)
      a1=factor*(S1-gammap)
      a4=a1
      a3=-factor*gammap
      a2=a3

*XC rewrite (a1*s12+a3*s23+a4*s24)=d2K=d1 for better numerical stability in 3 soft limit
      d1=factor*s23*mhsq*gammap/(gammam-mhsq)
     .  *(s23/(s14-gammap)+(s13+s23+s34)/gammap)
*XC rewrite (a2*s12+a3*s13+a4*s14)=d1K=d2 for better numerical stability in 4 soft limit
      d2=factor*s14*mhsq/gammam/(gammam-mhsq)
     .  *(-gammam*s23+(mhsq-gammam)*(s12+s13))

      coef3m1423=coef3m1423
     &  -mhsq**2*za14**2/(2d0*gammap*(gammap-mhsq))
     & *(a2*za32*zb21+a4*za34*zb41)   ! *zab2(k3,k1f,k1)
     & *(a1*za31*zb12+a4*za34*zb42)   ! *zab2(k3,k1f,k2)
     * /d2                                                ! (2*k1.k1f)^-1*
     * /d1                                                ! (2*k2.k1f)^-1*

      factor=gammam/(gammam**2-S1*S2)
      a1=factor*(S1-gammam)
      a4=a1
      a3=-factor*gammam
      a2=a3

*XC rewrite (a1*s12+a3*s23+a4*s24)=d2K=d1 for better numerical stability in 3 soft limit
      d1=factor*s23*mhsq*gammam/(gammap-mhsq)
     .  *(s23/(s14-gammam)+(s13+s23+s34)/gammam)
*XC rewrite (a2*s12+a3*s13+a4*s14)=d1K=d2 for better numerical stability in 4 soft limit
      d2=factor*s14*mhsq/gammap/(gammap-mhsq)
     .  *(-gammap*s23+(mhsq-gammap)*(s12+s13))

      coef3m1423=coef3m1423
     &  -mhsq**2*za14**2/(2d0*gammam*(gammam-mhsq))
     & *(a2*za32*zb21+a4*za34*zb41)   ! *zab2(k3,k1f,k1)
     & *(a1*za31*zb12+a4*za34*zb42)   ! *zab2(k3,k1f,k2)
     * /d2                                                ! (2*k1.k1f)^-1*
     * /d1                                                ! (2*k2.k1f)^-1*

      sum=sum-coef3m1234*I3m(mhsq,s12,s34)
     .       -coef3m1423*I3m(mhsq,s14,s23)

      sum=sum
     & -2d0/3d0*za13**2*za34*z2ba4123*zb12
     & *BGRL3hat(s123,s12)

     & +1d0/6d0*za34*za13
     & *(z2ba4132*zb13-3d0*z2ba4231*zb23)
     & /zb13
     * *BGRL2hat(s123,s12)

     & +za13
     & *(0.5d0*z2ba4132*z2ba4123*zb12*zb13
     & -z2ba4231**2*zb23**2
     & -8d0/3d0*z2ba4132**2*zb13**2)
     & /(s123*zb13**2*zb23)
     & *BGRL1(s123,s12)

     & -2d0/3d0*s124*za34**2*za14*zb42
     & *BGRL3hat(s124,s12)

     & +za34*za14
     & *(1d0/3d0*z2ba3142*zb14
     * -0.5d0*z2ba3124*zb12)/zb14
     & *BGRL2hat(s124,s12)

     & +z2ba3142*(3d0/2d0*s124*za34
     & +11d0/3d0*z2ba3142*za42)/(s124*zb14)
     & *BGRL1(s124,s12)

     & +0.5d0*za14*za13*z2ba4231*zb12/zb31
     & *BGRL2hat(s123,s23)

     &-za13*z2ba4231*(3d0/2d0*z2ba4132*zb13
     &+z2ba4231*zb23)/(s123*zb13**2)
     & *BGRL1(s123,s23)

     & +0.5d0*s234*za14*za34*zb42/zb43
     & *BGRL2hat(s234,s23)

     & +3d0/2d0*za34*z2ba1342/zb43
     & *BGRL1(s234,s23)

      zA1cphiAQggmpmmL=zA1cphiAQggmpmmL+sum

c--- now add the rational pieces
      sum=
     . za34*z2ba3142
     .  *(2d0*za24*zb42-za12*zb21)
     .  /(12d0*s124*za12*zb21*zb41)
     . +(za23*z2ba4132**2*(
     .    3d0*za12*zb21-2d0*za23*zb32)
     .  -2d0*za13**2*za24*z2ba4231
     .      *zb21*zb32)
     .  /(12d0*s123*za12*za23*zb21*zb31*zb32)
     . +5d0*za34**2/(12d0*za23*zb31)
     . +5d0*za34*z2ba4132
     .  /(6d0*za23*zb31*zb32)
     . +z2ba4132**2
     .  /(6d0*za12*zb21*zb31*zb32)
     . -za13*za14*za24*zb21
     .  /(3d0*za12*za23*zb31*zb32)
     . -za13*za34/(12d0*za12*zb41)
     . -za34**2*zb42/(6d0*za12*zb21*zb41)
     . +za13*za24*z2ba4134
     .  /(4d0*za12*za23*zb31*zb43)
     . -za13*z2ba4134/(3d0*za12*zb41*zb43)
     . -5d0*za14**2*zb41/(12d0*za12*zb31*zb43)
     . +za14**2*zb42/(6d0*za12*zb32*zb43)

      zA1cphiAQggmpmmL=zA1cphiAQggmpmmL+sum
      endif

      return
      end
************************************************************************
      double complex function zA1cphiAQggmpmmR(perm,renscale2)
c--- This is an implementation of Eq. (5.10) in
c---  S.~Badger, John.~M.~Campbell, R.~Keith Ellis and Ciaran Williams
c---  "Analytic results for the one-loop NMHV H-qbar-q-g-g amplitude."
c---   preprint DESY 09-180, FERMILAB-PUB-09-505-T, IPPP/09/86
c---   arXiv: 0910.4481 [hep-ph]
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /order/ieorder
      common /koppel/ nf,nc,b0
      integer j,j1,j2,j3,j4,j5,j6,perm(6),permm(6)
      double complex VR,zB2g0Hmmmp,lnrat,zab2,Lsm1,Lsm1_2mht
      double complex sum,l12,coef3m1423,
     & S1,S2,K1DK2,a1,a2,a3,a4,gamma,factor,I3m,
     & BGRL1,BGRL2hat,za(5,5),zb(5,5)
      double precision s123,s234,s124,s134,mhsq,epinv,deltar,musq
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43
      integer ipset

      musq=renscale2
      epinv=0d0
      deltar=1d0
      czip=(0d0,0d0)

      permm(1)=perm(1)
      permm(2)=perm(4)
      permm(3)=perm(3)
      permm(4)=perm(2)
      permm(5)=perm(5)
      permm(6)=perm(6)

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zB(j1,j2)
      za13 => kin(ipset)%zB(j1,j3)
      za14 => kin(ipset)%zB(j1,j4)
      za23 => kin(ipset)%zB(j2,j3)
      za24 => kin(ipset)%zB(j2,j4)
      za34 => kin(ipset)%zB(j3,j4)

      za21 => kin(ipset)%zB(j2,j1)
      za31 => kin(ipset)%zB(j3,j1)
      za41 => kin(ipset)%zB(j4,j1)
      za32 => kin(ipset)%zB(j3,j2)
      za42 => kin(ipset)%zB(j4,j2)
      za43 => kin(ipset)%zB(j4,j3)

      zb12 => kin(ipset)%zA(j1,j2)
      zb13 => kin(ipset)%zA(j1,j3)
      zb14 => kin(ipset)%zA(j1,j4)
      zb23 => kin(ipset)%zA(j2,j3)
      zb24 => kin(ipset)%zA(j2,j4)
      zb34 => kin(ipset)%zA(j3,j4)

      zb21 => kin(ipset)%zA(j2,j1)
      zb31 => kin(ipset)%zA(j3,j1)
      zb41 => kin(ipset)%zA(j4,j1)
      zb32 => kin(ipset)%zA(j3,j2)
      zb42 => kin(ipset)%zA(j4,j2)
      zb43 => kin(ipset)%zA(j4,j3)

      z2ba2143=zba2(j2,j1,j4,j3,ipset)
      z2ba1342=zba2(j1,j3,j4,j2,ipset)
      z2ba1234=zba2(j1,j2,j3,j4,ipset)
      z2ba4123=zba2(j4,j1,j2,j3,ipset)
      z2ba3142=zba2(j3,j1,j4,j2,ipset)
      z2ba3124=zba2(j3,j1,j2,j4,ipset)
      z2ba4231=zba2(j4,j2,j3,j1,ipset)
      z2ba4132=zba2(j4,j1,j3,j2,ipset)
      z2ba3241=zba2(j3,j2,j4,j1,ipset)
      z2ba4134=zba2(j4,j1,j3,j4,ipset)

      mhsq=s12+s13+s14+s23+s24+s34
      s123=s12+s13+s23
      s124=s12+s14+s24
      s134=s13+s14+s34
      s234=s23+s24+s34
      l12=lnrat(musq,-s12)

      if(ieorder.eq.-2)then
      VR=-1d0
      zA1cphiAQggmpmmR=0d0!zB2g0Hpppm(permm(1))*VR
      elseif(ieorder.eq.-1)then
      VR=-l12
     .   -3d0/2d0
      zA1cphiAQggmpmmR=0d0!zB2g0Hpppm(permm(1))*VR
      elseif(ieorder.eq.0)then
      VR=-epinv**2-epinv*l12-0.5d0*l12**2
     .   -3d0/2d0*(epinv+l12)-7d0/2d0-deltar/2d0

      zA1cphiAQggmpmmR=zB2g0Hpppm(permm(1))*VR
      sum=
     &  +zb12**2*z2ba4123**2/(zb13**3*zb23*s123)
     & *Lsm1(-s12,-s123,-s23,-s123)

     & +z2ba3142**2/(zb14*zb24*s124)
     & *Lsm1(-s14,-s124,-s12,-s124)

     & -z2ba1342**2/(zb23*zb34*z2ba1234)
     & *Lsm1_2mht(s14,s234,s23,mhsq)

     & +s134**2/(zb14*zb34*z2ba2143)
     & *Lsm1_2mht(s23,s134,s14,mhsq)

C-----Now deal with the 14-23 case
C-----K1=-(p1+p2+p3+p4)
C-----K2=-(p1+p4)
C     K1flat=gamma/(gamma**2-S1*S2)*(gamma*K1-S1*K2)
C     K1flat=gamma/(gamma**2-S1*S2)*((S1-gamma)*(p1+p4)-gamma*((p2+p3))

C----solve for gamma_+ and gamma_-
      S1=dcmplx(mhsq)
      S2=dcmplx(s14)
      K1DK2=dcmplx(s14+0.5d0*(s12+s13+s24+s34))

C-gamma+ = K1DK2+sqrt(K1DK2**2-S1*S2)
C-gamma- = K1DK2-sqrt(K1DK2**2-S1*S2)

      coef3m1423=czip
      gammap=K1DK2+sqrt(K1DK2**2-S1*S2)
      gammam=2d0*K1DK2-gammap

      factor=gammap/(gammap**2-S1*S2)
      a1=factor*(S1-gammap)
      a4=a1
      a3=-factor*gammap
      a2=a3

*XC rewrite (a1*s12+a3*s23+a4*s24)=d2K=d1 for better numerical stability in 3 soft limit
      d1=factor*s23*mhsq*gammap/(gammam-mhsq)
     .  *(s23/(s14-gammap)+(s13+s23+s34)/gammap)
*XC rewrite (a2*s12+a3*s13+a4*s14)=d1K=d2 for better numerical stability in 4 soft limit
      d2=factor*s14*mhsq/gammam/(gammam-mhsq)
     .  *(-gammam*s23+(mhsq-gammam)*(s12+s13))

      coef3m1423=coef3m1423
     &  -mhsq**2*za14**2/(2d0*gammap*(gammap-mhsq))
     & *(a2*za32*zb21+a4*za34*zb41)   ! *zab2(k3,k1f,k1)
     & *(a1*za31*zb12+a4*za34*zb42)   ! *zab2(k3,k1f,k2)
     * /d2                                                ! (2*k1.k1f)^-1*
     * /d1                                                ! (2*k2.k1f)^-1*

      factor=gammam/(gammam**2-S1*S2)
      a1=factor*(S1-gammam)
      a4=a1
      a3=-factor*gammam
      a2=a3

*XC rewrite (a1*s12+a3*s23+a4*s24)=d2K=d1 for better numerical stability in 3 soft limit
      d1=factor*s23*mhsq*gammam/(gammap-mhsq)
     .  *(s23/(s14-gammam)+(s13+s23+s34)/gammam)
*XC rewrite (a2*s12+a3*s13+a4*s14)=d1K=d2 for better numerical stability in 4 soft limit
      d2=factor*s14*mhsq/gammap/(gammap-mhsq)
     .  *(-gammap*s23+(mhsq-gammap)*(s12+s13))

      coef3m1423=coef3m1423
     &  -mhsq**2*za14**2/(2d0*gammam*(gammam-mhsq))
     & *(a2*za32*zb21+a4*za34*zb41)   ! *zab2(k3,k1f,k1)
     & *(a1*za31*zb12+a4*za34*zb42)   ! *zab2(k3,k1f,k2)
     * /d2                                                ! (2*k1.k1f)^-1*
     * /d1                                                ! (2*k2.k1f)^-1*

      sum=sum-coef3m1423*I3m(mhsq,s14,s23)

      sum=sum
     & -0.5d0*(za14*zb12*z2ba3124)**2
     & /(zb14*zb24*s124)
     & *BGRL2hat(s124,s12)

     & +2d0*za34*z2ba3142/zb14
     & *BGRL1(s124,s12)

     & +0.5d0*z2ba3142**2/(zb14*zb24*s124)
     & *lnrat(-s124,-s12)

     & -0.5d0*(za14*zb24*s234)**2
     & /(zb23*zb34*z2ba1234)
     & *BGRL2hat(s234,s23)

     & -2d0*za34*z2ba1342/zb34
     & *BGRL1(s234,s23)

     & +0.5d0*z2ba1342**2
     & /(zb23*zb34*z2ba1234)
     & *lnrat(-s234,-s23)

     &-0.5d0*(za12*zb12*z2ba4231)**2*zb23
     & /(zb13**3*s123)
     & *BGRL2hat(s123,s23)

     & +2d0*za13*zb12*z2ba4123*z2ba4231
     & /(za23*zb13**2*zb23)
     & *BGRL1(s123,s23)

     & +(-2d0*za13*zb12*z2ba4123*z2ba4231
     & /(zb13**2*za23*zb23*s123)
     & +0.5d0*z2ba4231**2*zb23
     & /(zb13**3*s123))
     & *lnrat(-s123,-s23)

     & -0.5d0*(za13*zb12*z2ba4123)**2
     & /(zb13*zb23*s123)
     & *BGRL2hat(s123,s12)

     & +za34*zb12*z2ba4123
     & *(-2d0*za13*zb13-za23*zb23)
     & /(za23*zb13**2*zb23)
     & *BGRL1(s123,s12)

     & +zb12*z2ba4123
     & *(za23*z2ba4132+2d0*za13*z2ba4231)
     & /(zb13**2*za23*zb23*s123)
     & *lnrat(-s123,-s12)

      zA1cphiAQggmpmmR=zA1cphiAQggmpmmR+sum
c--- now add the rational pieces
      sum=
     .-(za24**2*zb21**2)/(2d0*za23*zb31**3)
     .+(z2ba4123**2*zb21**2)
     .  /(2d0*s123*zb31**3*zb32)
     .-(za14**2*zb21)/(2*za12*zb31*zb32)
     .+(zb21*(za13**2*za23
     .  *z2ba4123**2*zb31**2
     .+za12**3*z2ba4231**2*zb21*zb32))
     .  /(4d0*s123**2*za12*za23*zb31**3*zb32)
     .+z2ba3142**2/(2d0*s124*zb41*zb42)
     .-(za13**2*zb21)/(2d0*za12*zb41*zb42)
     .+(za14**2*z2ba3124**2*zb21)
     .  /(4d0*s124**2*za12*zb41*zb42)
     .-(za13*za14*zb42)/(2d0*z2ba1234*zb43)
     .-(s234*za14**2*zb42**2)
     .  /(4d0*za23*z2ba1234*zb32**2*zb43)
     .-(za14**2*zb42**2)
     .  /(2d0*z2ba1234*zb32*zb43)

      zA1cphiAQggmpmmR=zA1cphiAQggmpmmR+sum
      endif
      return
      end
************************************************************************
      double complex function zA1cphiAQggmpmmF(perm,renscale2)
c--- This is an implementation of Eq. (5.13) in
c---  S.~Badger, John.~M.~Campbell, R.~Keith Ellis and Ciaran Williams
c---  "Analytic results for the one-loop NMHV H-qbar-q-g-g amplitude."
c---   preprint DESY 09-180, FERMILAB-PUB-09-505-T, IPPP/09/86
c---   arXiv: 0910.4481 [hep-ph]
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /order/ieorder
      common /koppel/ nf,nc,b0
      integer j,j1,j2,j3,j4,j5,j6,perm(6),permm(6)
      double complex zB2g0Hmmmp,lnrat,za(5,5),zb(5,5)
      double complex l12,zab2,BGRL1,BGRL2hat,BGRL3hat
      double precision s123,s124,mhsq,epinv,deltar,musq
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43
      integer ipset

      musq=renscale2
      epinv=0d0
      deltar=1d0
      czip=(0d0,0d0)

      permm(1)=perm(1)
      permm(2)=perm(4)
      permm(3)=perm(3)
      permm(4)=perm(2)
      permm(5)=perm(5)
      permm(6)=perm(6)

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zB(j1,j2)
      za13 => kin(ipset)%zB(j1,j3)
      za14 => kin(ipset)%zB(j1,j4)
      za23 => kin(ipset)%zB(j2,j3)
      za24 => kin(ipset)%zB(j2,j4)
      za34 => kin(ipset)%zB(j3,j4)

      za21 => kin(ipset)%zB(j2,j1)
      za31 => kin(ipset)%zB(j3,j1)
      za41 => kin(ipset)%zB(j4,j1)
      za32 => kin(ipset)%zB(j3,j2)
      za42 => kin(ipset)%zB(j4,j2)
      za43 => kin(ipset)%zB(j4,j3)

      zb12 => kin(ipset)%zA(j1,j2)
      zb13 => kin(ipset)%zA(j1,j3)
      zb14 => kin(ipset)%zA(j1,j4)
      zb23 => kin(ipset)%zA(j2,j3)
      zb24 => kin(ipset)%zA(j2,j4)
      zb34 => kin(ipset)%zA(j3,j4)

      zb21 => kin(ipset)%zA(j2,j1)
      zb31 => kin(ipset)%zA(j3,j1)
      zb41 => kin(ipset)%zA(j4,j1)
      zb32 => kin(ipset)%zA(j3,j2)
      zb42 => kin(ipset)%zA(j4,j2)
      zb43 => kin(ipset)%zA(j4,j3)

      z2ba2143=zba2(j2,j1,j4,j3,ipset)
      z2ba1342=zba2(j1,j3,j4,j2,ipset)
      z2ba1234=zba2(j1,j2,j3,j4,ipset)
      z2ba4123=zba2(j4,j1,j2,j3,ipset)
      z2ba3142=zba2(j3,j1,j4,j2,ipset)
      z2ba3124=zba2(j3,j1,j2,j4,ipset)
      z2ba4231=zba2(j4,j2,j3,j1,ipset)
      z2ba4132=zba2(j4,j1,j3,j2,ipset)
      z2ba3241=zba2(j3,j2,j4,j1,ipset)
      z2ba4134=zba2(j4,j1,j3,j4,ipset)

      s123=s12+s13+s23
      s124=s12+s14+s24

      l12=lnrat(musq,-s12)

      if(ieorder.eq.-2)then
          zA1cphiAQggmpmmF=0d0
      elseif(ieorder.eq.-1)then
          zA1cphiAQggmpmmF=0d0
      elseif(ieorder.eq.0)then
      zA1cphiAQggmpmmF=zB2g0Hpppm(permm(1))
     . *(-2d0/3d0*(epinv+l12)-10d0/9d0)
     & +2d0/3d0*za13*z2ba4132**2
     & /(za12*zb12*zb23*s123)*lnrat(-s123,-s12)
     & -2d0/3d0*(s24-s124)*z2ba3142**2
     & /(za12*zb14*zb24*zb12*s124)*lnrat(-s124,-s12)
     & -2d0/3d0*za13*z2ba4132**2
     & /(za12*zb23*zb12)*BGRL1(s123,s12)
     & +2d0/3d0*za14*z2ba3142**2
     & /(za12*zb24*zb12)*BGRL1(s124,s12)
     & +za13*za34*z2ba4132/3d0*BGRL2hat(s123,s12)
     & +za14*za34*z2ba3142/3d0*BGRL2hat(s124,s12)
     & +2d0/3d0*za13**2*za34*zb12*z2ba4123
     & *BGRL3hat(s123,s12)
     & +2d0/3d0*za14**2*za34*zb12*z2ba3124
     & *BGRL3hat(s124,s12)
      zA1cphiAQggmpmmF=zA1cphiAQggmpmmF
     & -za13*za34*z2ba4132
     & /(6d0*za12*zb12*s123)
     & -za14*za34*z2ba3142
     & /(6d0*za12*zb12*s124)
     & +(
     &  -za13*za14*zb12*zb34)
     & /(3d0*za12*zb12*zb34**2)
      endif
      return
      end
************************************************************************
      double complex function zA1phiAQggmpppL(perm)
C     implementation of arXiv:0906.0008v1, Eq. 4.16
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /order/ieorder
      common /koppel/ nf,nc,b0
      integer j1,j2,j3,j4,j5,j6,perm(6)
      double complex zab2
      double precision ss3,mhsq
*      write(*,*)'hello zA1phiAQggmpppL'
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43
      integer ipset

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za34 => kin(ipset)%zA(j3,j4)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za43 => kin(ipset)%zA(j4,j3)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb34 => kin(ipset)%zB(j3,j4)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb43 => kin(ipset)%zB(j4,j3)

      mhsq=s12+s13+s14+s23+s24+s34

      z2ab1342=zab2(j1,j3,j4,j2,ipset)
      z2ab3142=zab2(j3,j1,j4,j2,ipset)
      z2ab1234=zab2(j1,j2,j3,j4,ipset)
      z2ab2134=zab2(j2,j1,j3,j4,ipset)
      z2ab3124=zab2(j3,j1,j2,j4,ipset)

      s123=ss3(j1,j2,j3,ipset)
      s412=ss3(j4,j1,j2,ipset)

      zA1phiAQggmpppL=
     . 0.5d0*za12*z2ab1342/(za23*za34*za41)
     . +0.5d0*za13*zb34/(za23*za34)
     . +2d0*z2ab1342**2/(za34*za41*z2ab3142)
     . -2d0*z2ab1234**2*z2ab2134
     . /(za12*za23*s123*z2ab3124)
     . -2d0*zb24**3*mhsq**2
     . /(zb12*s412*z2ab3124*z2ab3142)
     . -1d0/3d0*za13*zb34*za41/(za12*za34**2)

      return
      end
************************************************************************
      double complex function zA1phiAQggmpppR(perm)
C     implementation of arXiv:0906.0008v1, Eq. 4.17
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /order/ieorder
      common /koppel/ nf,nc,b0
      integer j1,j2,j3,j4,j5,j6,perm(6)
      double complex zab2
      double precision ss3,mhsq
*      write(*,*)'hello zA1phiAQggmpppR'
      common /npars/npars,nspec,n
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43
      integer ipset

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      ipset = nspec+4

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za34 => kin(ipset)%zA(j3,j4)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za43 => kin(ipset)%zA(j4,j3)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb34 => kin(ipset)%zB(j3,j4)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb43 => kin(ipset)%zB(j4,j3)

      z2ab1342=zab2(j1,j3,j4,j2,ipset)
      z2ab3142=zab2(j3,j1,j4,j2,ipset)
      z2ab1234=zab2(j1,j2,j3,j4,ipset)
      z2ab2134=zab2(j2,j1,j3,j4,ipset)
      z2ab3124=zab2(j3,j1,j2,j4,ipset)

      zA1phiAQggmpppR=
     . -0.5d0*z2ab1234/(za23*za34)
     . -0.5d0*za12*zb23*za31
     .  /(za23*za34*za41)

      return
      end
************************************************************************
      double complex function zA1phiAQggmpppF(perm)
C     implementation of arXiv:0906.0008v1, Eq. 4.18
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /order/ieorder
      common /koppel/ nf,nc,b0
      integer j1,j2,j3,j4,j5,j6,perm(6)
*      write(*,*)'hello zA1phiAQggmpppF'
      common /npars/npars,nspec,n
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43
      integer ipset

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      ipset = nspec+4

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za34 => kin(ipset)%zA(j3,j4)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za43 => kin(ipset)%zA(j4,j3)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb34 => kin(ipset)%zB(j3,j4)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb43 => kin(ipset)%zB(j4,j3)

      zA1phiAQggmpppF=
     . +1d0/3d0*za13*zb34*za41/(za12*za34**2)

      return
      end
************************************************************************
      double complex function zA1cphiAQggmpppL(perm)
C     implementation of arXiv:0906.0008v1, Eq. 4.16
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /order/ieorder
      common /koppel/ nf,nc,b0
      integer j1,j2,j3,j4,j5,j6,perm(6)
      double precision ss3,mhsq
*      write(*,*)'hello zA1cphiAQggmpppL'
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43
      integer ipset

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za34 => kin(ipset)%zA(j3,j4)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za43 => kin(ipset)%zA(j4,j3)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb34 => kin(ipset)%zB(j3,j4)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb43 => kin(ipset)%zB(j4,j3)

      mhsq=s12+s13+s14+s23+s24+s34

      z2ba2143=zba2(j2,j1,j4,j3,ipset)
      z2ba1342=zba2(j1,j3,j4,j2,ipset)
      z2ba1234=zba2(j1,j2,j3,j4,ipset)
      z2ba4123=zba2(j4,j1,j2,j3,ipset)
      z2ba3142=zba2(j3,j1,j4,j2,ipset)
      z2ba3124=zba2(j3,j1,j2,j4,ipset)
      z2ba4231=zba2(j4,j2,j3,j1,ipset)
      z2ba4132=zba2(j4,j1,j3,j2,ipset)
      z2ba3241=zba2(j3,j2,j4,j1,ipset)
      z2ba4134=zba2(j4,j1,j3,j4,ipset)
      z2ba2134=zba2(j2,j1,j3,j4,ipset)

      s123=ss3(j1,j2,j3,ipset)
      s412=ss3(j4,j1,j2,ipset)

      zA1cphiAQggmpppL=
     . 0.5d0*zb12*z2ba1342/(zb23*zb34*zb41)
     . +0.5d0*zb13*za34/(zb23*zb34)
     . +2d0*z2ba1342**2/(zb34*zb41*z2ba3142)
     . -2d0*z2ba1234**2*z2ba2134
     . /(zb12*zb23*s123*z2ba3124)
     . -2d0*za24**3*mhsq**2
     . /(za12*s412*z2ba3124*z2ba3142)
     . -1d0/3d0*zb13*za34*zb41/(zb12*zb34**2)

      return
      end
************************************************************************
      double complex function zA1cphiAQggmpppR(perm)
C     implementation of arXiv:0906.0008v1, Eq. 4.17
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /order/ieorder
      common /koppel/ nf,nc,b0
      integer j1,j2,j3,j4,j5,j6,perm(6)
      double precision ss3,mhsq
*      write(*,*)'hello zA1cphiAQggmpppR'
      common /npars/npars,nspec,n
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43
      integer ipset

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      ipset = nspec+4

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za34 => kin(ipset)%zA(j3,j4)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za43 => kin(ipset)%zA(j4,j3)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb34 => kin(ipset)%zB(j3,j4)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb43 => kin(ipset)%zB(j4,j3)

      z2ba1234=zba2(j1,j2,j3,j4,ipset)

      zA1cphiAQggmpppR=
     . -0.5d0*z2ba1234/(zb23*zb34)
     . -0.5d0*zb12*za23*zb31
     .  /(zb23*zb34*zb41)

      return
      end
************************************************************************
      double complex function zA1cphiAQggmpppF(perm)
C     implementation of arXiv:0906.0008v1, Eq. 4.18
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /order/ieorder
      common /koppel/ nf,nc,b0
      integer j1,j2,j3,j4,j5,j6,perm(6)
*      write(*,*)'hello zA1cphiAQggmpppF'
      common /npars/npars,nspec,n
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43
      integer ipset

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      ipset = nspec+4

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za34 => kin(ipset)%zA(j3,j4)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za43 => kin(ipset)%zA(j4,j3)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb34 => kin(ipset)%zB(j3,j4)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb43 => kin(ipset)%zB(j4,j3)

      zA1cphiAQggmpppF=
     . +1d0/3d0*zb13*za34*zb41/(zb12*zb34**2)

      return
      end
************************************************************************
      double complex function zA1phiAQggmpmmL(perm,renscale2)
c--- This is an implementation of Eq. (5.2) in
c---  S.~Badger, John.~M.~Campbell, R.~Keith Ellis and Ciaran Williams
c---  "Analytic results for the one-loop NMHV H-qbar-q-g-g amplitude."
c---   preprint DESY 09-180, FERMILAB-PUB-09-505-T, IPPP/09/86
c---   arXiv: 0910.4481 [hep-ph]
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /order/ieorder
      common /koppel/ nf,nc,b0
      integer j,j1,j2,j3,j4,j5,j6,perm(6),permm(6)
      double complex V1L,zB2g0Hmmmp,lnrat,zab2,Lsm1,Lsm1_2mht,czip
      double complex sum,l23,l34,l41,l12,coef3m1234,coef3m1423,
     & S1,S2,K1DK2,a1,a2,a3,a4,gamma,gammap,gammam,factor,I3m,d1,d2,
     & BGRL1,BGRL2hat,BGRL3hat,za(5,5),zb(5,5)
      double precision s123,s234,s124,s134,mhsq,epinv,deltar,musq
*      write(*,*)'hello zA1phiAQggmpmmL'
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43
      integer ipset

      musq=renscale2
      epinv=0d0
      deltar=1d0
      czip=(0d0,0d0)

      permm(1)=perm(1)
      permm(2)=perm(4)
      permm(3)=perm(3)
      permm(4)=perm(2)
      permm(5)=perm(5)
      permm(6)=perm(6)

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za34 => kin(ipset)%zA(j3,j4)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za43 => kin(ipset)%zA(j4,j3)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb34 => kin(ipset)%zB(j3,j4)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb43 => kin(ipset)%zB(j4,j3)

      z2ab2143=zab2(j2,j1,j4,j3,ipset)
      z2ab1342=zab2(j1,j3,j4,j2,ipset)
      z2ab1234=zab2(j1,j2,j3,j4,ipset)
      z2ab4123=zab2(j4,j1,j2,j3,ipset)
      z2ab3124=zab2(j3,j1,j2,j4,ipset)
      z2ab4231=zab2(j4,j2,j3,j1,ipset)
      z2ab4132=zab2(j4,j1,j3,j2,ipset)
      z2ab3241=zab2(j3,j2,j4,j1,ipset)
      z2ab3142=zab2(j3,j1,j4,j2,ipset)
      z2ab4134=zab2(j4,j1,j3,j4,ipset)

      mhsq=s12+s13+s14+s23+s24+s34
      s123=s12+s13+s23
      s124=s12+s14+s24
      s134=s13+s14+s34
      s234=s23+s24+s34

      l12=lnrat(musq,-s12)
      l23=lnrat(musq,-s23)
      l34=lnrat(musq,-s34)
      l41=lnrat(musq,-s14)

      if(ieorder.eq.-2)then
      V1L=-3d0
      zA1phiAQggmpmmL=zB2g0Hmmmp(permm(1))*V1L
      elseif(ieorder.eq.-1)then
      V1L=
     . -l23
     . -l34
     . -l41
     . +13d0/6d0
      zA1phiAQggmpmmL=zB2g0Hmmmp(permm(1))*V1L
      elseif(ieorder.eq.0)then

      V1L=
     . -epinv**2-epinv*l23-0.5d0*l23**2
     . -epinv**2-epinv*l34-0.5d0*l34**2
     . -epinv**2-epinv*l41-0.5d0*l41**2
     . +13d0/6d0*(epinv+l12)+119d0/18d0-deltar/6d0

      zA1phiAQggmpmmL=zB2g0Hmmmp(permm(1))*V1L

      sum=
     & -s134**2/(zb41*zb34*z2ab2143)
     & *Lsm1(-s14,-s134,-s34,-s134)

     & -z2ab1342**2/(z2ab1234*zb23*zb34)
     & *Lsm1(-s34,-s234,-s23,-s234)

     &  +(mhsq**2*za14**2*za24
     & /(za12*z2ab2143*z2ab4123*s124)
     &   -z2ab3142**3
     & /(zb12*zb24*z2ab3124*s124))
     & *Lsm1(-s14,-s124,-s12,-s124)

     & +(zb23**2*z2ab4231**3
     & /(zb12*zb13**3*z2ab4123*s123)
     & -mhsq**2*za13**3
     & /(za12*z2ab1234*z2ab3124*s123))
     & *Lsm1(-s12,-s123,-s23,-s123)

     &   +za34*s134**2
     & /(zb34*za34*zb14*z2ab2143)
     & *Lsm1_2mht(s12,s134,s34,mhsq)

     & -z2ab1342**2
     & /(zb34*zb23*z2ab1234)
     & *Lsm1_2mht(s12,s234,s34,mhsq)

     & +(z2ab4132**3
     & /(zb12*zb23*z2ab4123*s123)
     & -mhsq**2*za13**3
     & /(za12*z2ab1234*z2ab3124*s123))
     & *Lsm1_2mht(s34,s123,s12,mhsq)

     & +(mhsq**2*za14**2*za24
     & /(za12*z2ab2143*z2ab4123*s124)
     & -z2ab3241*z2ab3142**2
     & /(z2ab3124*zb14*zb12*s124))
     & *Lsm1_2mht(s34,s124,s12,mhsq)

     &  +(z2ab4132**3
     & /(zb12*zb23*z2ab4123*s123)
     & -mhsq**2*za13**3
     & /(za12*z2ab1234*z2ab3124*s123))
     & *Lsm1_2mht(s14,s123,s23,mhsq)

     & +(mhsq**2*za24*za14**2
     & /(za12*z2ab2143*z2ab4123*s124)
     & -z2ab3142**2*z2ab3241
     & /(zb12*zb14*z2ab3124*s124))
     & *Lsm1_2mht(s23,s124,s14,mhsq)

C-----Now for three mass triangles
C-----Deal with the 12-34 case first
C-----K1=-(p1+p2+p3+p4)
C-----K2=-(p1+p2)
C     K1flat=gamma/(gamma**2-S1*S2)*(gamma*K1-S1*K2)
C     K1flat=gamma/(gamma**2-S1*S2)*((S1-gamma)*(p1+p2)-gamma*((p3+p4))
C----solve for gamma_+ and gamma_-
      S1=dcmplx(mhsq)
      S2=dcmplx(s12)
      K1DK2=dcmplx(s12+0.5d0*(s13+s14+s23+s24))

C-gamma+ = K1DK2+sqrt(K1DK2**2-S1*S2)
C-gamma- = K1DK2-sqrt(K1DK2**2-S1*S2)

      coef3m1234=czip
      gammap=K1DK2+sqrt(K1DK2**2-S1*S2)
      gammam=2d0*K1DK2-gammap
C -- calculate the projections of K1 flat on k1,k2,k3,k4 called a1,a2,a3,a4
      factor=gammap/(gammap**2-S1*S2)
      a1=factor*(S1-gammap)
      a2=a1
      a3=-factor*gammap
      a4=a3

*XC rewrite (a1*s14+a2*s24+a3*s34)=d4K=d1 for better numerical stability in 3 soft limit
      d1=factor*s34*mhsq/gammam/(gammam-mhsq)
     .  *(mhsq*s12*s34/(s12-gammap)+gammam*(s13+s23+s34))
*XC rewrite (a1*s13+a2*s23+a4*s34)=d3K=d2 for better numerical stability in 4 soft limit
      d2=factor*s34*mhsq/gammam/(gammam-mhsq)
     .  *(mhsq*s12*s34/(s12-gammap)+gammam*(s14+s24+s34))

      coef3m1234=coef3m1234
     & +mhsq**2*za34**3/(za12*gammap*(gammap-mhsq))
     & *(a2*za12*zb23+a4*za14*zb43)   ! zab2(k1,k1f,k3)
     & *(a2*za12*zb24+a3*za13*zb34)   ! zab2(k1,k1f,k4)
     * /d2                                                ! (2*k3.k1f)^-1*
     * /d1
C----switch to other solution

      factor=gammam/(gammam**2-S1*S2)
      a1=factor*(S1-gammam)
      a2=a1
      a3=-factor*gammam
      a4=a3

*XC rewrite (a1*s14+a2*s24+a3*s34)=d4K=d1 for better numerical stability in 3 soft limit
      d1=factor*s34*mhsq/gammap/(gammap-mhsq)
     .  *(mhsq*s12*s34/(s12-gammam)+gammap*(s13+s23+s34))
*XC rewrite (a1*s13+a2*s23+a4*s34)=d3K=d2 for better numerical stability in 4 soft limit
      d2=factor*s34*mhsq/gammap/(gammap-mhsq)
     .  *(mhsq*s12*s34/(s12-gammam)+gammap*(s14+s24+s34))

      coef3m1234=coef3m1234
     & +mhsq**2*za34**3/(za12*gammam*(gammam-mhsq))
     & *(a2*za12*zb23+a4*za14*zb43)   ! zab2(k1,k1f,k3)
     & *(a2*za12*zb24+a3*za13*zb34)   ! zab2(k1,k1f,k4)
     * /d2                                                ! (2*k3.k1f)^-1*
     * /d1

C-----Now deal with the 14-23 case
C-----K1=-(p1+p2+p3+p4)
C-----K2=-(p1+p4)
C     K1flat=gamma/(gamma**2-S1*S2)*(gamma*K1-S1*K2)
C     K1flat=gamma/(gamma**2-S1*S2)*((S1-gamma)*(p1+p4)-gamma*((p2+p3))

C----solve for gamma_+ and gamma_-
      S1=dcmplx(mhsq)
      S2=dcmplx(s14)
      K1DK2=dcmplx(s14+0.5d0*(s12+s13+s24+s34))

C-gamma+ = K1DK2+sqrt(K1DK2**2-S1*S2)
C-gamma- = K1DK2-sqrt(K1DK2**2-S1*S2)

      coef3m1423=czip
      gammap=K1DK2+sqrt(K1DK2**2-S1*S2)
      gammam=2d0*K1DK2-gammap

      factor=gammap/(gammap**2-S1*S2)
      a1=factor*(S1-gammap)
      a4=a1
      a3=-factor*gammap
      a2=a3

*XC rewrite (a1*s12+a3*s23+a4*s24)=d2K=d1 for better numerical stability in 3 soft limit
      d1=factor*s23*mhsq*gammap/(gammam-mhsq)
     .  *(s23/(s14-gammap)+(s13+s23+s34)/gammap)
*XC rewrite (a2*s12+a3*s13+a4*s14)=d1K=d2 for better numerical stability in 4 soft limit
      d2=factor*s14*mhsq/gammam/(gammam-mhsq)
     .  *(-gammam*s23+(mhsq-gammam)*(s12+s13))

      coef3m1423=coef3m1423
     &  -mhsq**2*za14**2/(2d0*gammap*(gammap-mhsq))
     & *(a2*za32*zb21+a4*za34*zb41)   ! *zab2(k3,k1f,k1)
     & *(a1*za31*zb12+a4*za34*zb42)   ! *zab2(k3,k1f,k2)
     * /d2                                                ! (2*k1.k1f)^-1*
     * /d1                                                ! (2*k2.k1f)^-1*

      factor=gammam/(gammam**2-S1*S2)
      a1=factor*(S1-gammam)
      a4=a1
      a3=-factor*gammam
      a2=a3

*XC rewrite (a1*s12+a3*s23+a4*s24)=d2K=d1 for better numerical stability in 3 soft limit
      d1=factor*s23*mhsq*gammam/(gammap-mhsq)
     .  *(s23/(s14-gammam)+(s13+s23+s34)/gammam)
*XC rewrite (a2*s12+a3*s13+a4*s14)=d1K=d2 for better numerical stability in 4 soft limit
      d2=factor*s14*mhsq/gammap/(gammap-mhsq)
     .  *(-gammap*s23+(mhsq-gammap)*(s12+s13))

      coef3m1423=coef3m1423
     &  -mhsq**2*za14**2/(2d0*gammam*(gammam-mhsq))
     & *(a2*za32*zb21+a4*za34*zb41)   ! *zab2(k3,k1f,k1)
     & *(a1*za31*zb12+a4*za34*zb42)   ! *zab2(k3,k1f,k2)
     * /d2                                                ! (2*k1.k1f)^-1*
     * /d1                                                ! (2*k2.k1f)^-1*

      sum=sum-coef3m1234*I3m(mhsq,s12,s34)
     .       -coef3m1423*I3m(mhsq,s14,s23)

      sum=sum
     & -2d0/3d0*za13**2*za34*z2ab4123*zb12
     & *BGRL3hat(s123,s12)

     & +1d0/6d0*za34*za13
     & *(z2ab4132*zb13-3d0*z2ab4231*zb23)
     & /zb13
     * *BGRL2hat(s123,s12)

     & +za13
     & *(0.5d0*z2ab4132*z2ab4123*zb12*zb13
     & -z2ab4231**2*zb23**2
     & -8d0/3d0*z2ab4132**2*zb13**2)
     & /(s123*zb13**2*zb23)
     & *BGRL1(s123,s12)

     & -2d0/3d0*s124*za34**2*za14*zb42
     & *BGRL3hat(s124,s12)

     & +za34*za14
     & *(1d0/3d0*z2ab3142*zb14
     * -0.5d0*z2ab3124*zb12)/zb14
     & *BGRL2hat(s124,s12)

     & +z2ab3142*(3d0/2d0*s124*za34
     & +11d0/3d0*z2ab3142*za42)/(s124*zb14)
     & *BGRL1(s124,s12)

     & +0.5d0*za14*za13*z2ab4231*zb12/zb31
     & *BGRL2hat(s123,s23)

     &-za13*z2ab4231*(3d0/2d0*z2ab4132*zb13
     &+z2ab4231*zb23)/(s123*zb13**2)
     & *BGRL1(s123,s23)

     & +0.5d0*s234*za14*za34*zb42/zb43
     & *BGRL2hat(s234,s23)

     & +3d0/2d0*za34*z2ab1342/zb43
     & *BGRL1(s234,s23)

      zA1phiAQggmpmmL=zA1phiAQggmpmmL+sum

c--- now add the rational pieces
      sum=
     . za34*z2ab3142
     .  *(2d0*za24*zb42-za12*zb21)
     .  /(12d0*s124*za12*zb21*zb41)
     . +(za23*z2ab4132**2*(
     .    3d0*za12*zb21-2d0*za23*zb32)
     .  -2d0*za13**2*za24*z2ab4231
     .      *zb21*zb32)
     .  /(12d0*s123*za12*za23*zb21*zb31*zb32)
     . +5d0*za34**2/(12d0*za23*zb31)
     . +5d0*za34*z2ab4132
     .  /(6d0*za23*zb31*zb32)
     . +z2ab4132**2
     .  /(6d0*za12*zb21*zb31*zb32)
     . -za13*za14*za24*zb21
     .  /(3d0*za12*za23*zb31*zb32)
     . -za13*za34/(12d0*za12*zb41)
     . -za34**2*zb42/(6d0*za12*zb21*zb41)
     . +za13*za24*z2ab4134
     .  /(4d0*za12*za23*zb31*zb43)
     . -za13*z2ab4134/(3d0*za12*zb41*zb43)
     . -5d0*za14**2*zb41/(12d0*za12*zb31*zb43)
     . +za14**2*zb42/(6d0*za12*zb32*zb43)

      zA1phiAQggmpmmL=zA1phiAQggmpmmL+sum

      endif
      return
      end
************************************************************************
      double complex function zA1phiAQggmpmmR(perm,renscale2)
c--- This is an implementation of Eq. (5.10) in
c---  S.~Badger, John.~M.~Campbell, R.~Keith Ellis and Ciaran Williams
c---  "Analytic results for the one-loop NMHV H-qbar-q-g-g amplitude."
c---   preprint DESY 09-180, FERMILAB-PUB-09-505-T, IPPP/09/86
c---   arXiv: 0910.4481 [hep-ph]
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /order/ieorder
      common /koppel/ nf,nc,b0
      integer j,j1,j2,j3,j4,j5,j6,perm(6),permm(6)
      double complex VR,zB2g0Hmmmp,lnrat,zab2,Lsm1,Lsm1_2mht
      double complex sum,l12,coef3m1423,
     & S1,S2,K1DK2,a1,a2,a3,a4,gamma,factor,I3m,
     & BGRL1,BGRL2hat,za(5,5),zb(5,5)
      double precision s123,s234,s124,s134,mhsq,epinv,deltar,musq
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43
      integer ipset

      musq=renscale2
      epinv=0d0
      deltar=1d0
      czip=(0d0,0d0)

      permm(1)=perm(1)
      permm(2)=perm(4)
      permm(3)=perm(3)
      permm(4)=perm(2)
      permm(5)=perm(5)
      permm(6)=perm(6)

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za34 => kin(ipset)%zA(j3,j4)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za43 => kin(ipset)%zA(j4,j3)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb34 => kin(ipset)%zB(j3,j4)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb43 => kin(ipset)%zB(j4,j3)

      z2ab2143=zab2(j2,j1,j4,j3,ipset)
      z2ab1342=zab2(j1,j3,j4,j2,ipset)
      z2ab1234=zab2(j1,j2,j3,j4,ipset)
      z2ab4123=zab2(j4,j1,j2,j3,ipset)
      z2ab3124=zab2(j3,j1,j2,j4,ipset)
      z2ab4231=zab2(j4,j2,j3,j1,ipset)
      z2ab4132=zab2(j4,j1,j3,j2,ipset)
      z2ab3241=zab2(j3,j2,j4,j1,ipset)
      z2ab3142=zab2(j3,j1,j4,j2,ipset)
      z2ab4134=zab2(j4,j1,j3,j4,ipset)

      mhsq=s12+s13+s14+s23+s24+s34
      s123=s12+s13+s23
      s124=s12+s14+s24
      s134=s13+s14+s34
      s234=s23+s24+s34
      l12=lnrat(musq,-s12)

      if(ieorder.eq.-2)then
      VR=-1d0
      zA1phiAQggmpmmR=0d0!zB2g0Hmmmp(permm(1))*VR
      elseif(ieorder.eq.-1)then
      VR=
     . -l12
     . -3d0/2d0
      zA1phiAQggmpmmR=0d0!zB2g0Hmmmp(permm(1))*VR
      elseif(ieorder.eq.0)then
      VR=-epinv**2-epinv*l12-0.5d0*l12**2
     .   -3d0/2d0*(epinv+l12)-7d0/2d0-deltar/2d0

      zA1phiAQggmpmmR=zB2g0Hmmmp(permm(1))*VR
      sum=
     &  +zb12**2*z2ab4123**2/(zb13**3*zb23*s123)
     & *Lsm1(-s12,-s123,-s23,-s123)

     & +z2ab3142**2/(zb14*zb24*s124)
     & *Lsm1(-s14,-s124,-s12,-s124)

     & -z2ab1342**2/(zb23*zb34*z2ab1234)
     & *Lsm1_2mht(s14,s234,s23,mhsq)

     & +s134**2/(zb14*zb34*z2ab2143)
     & *Lsm1_2mht(s23,s134,s14,mhsq)

C-----Now deal with the 14-23 case
C-----K1=-(p1+p2+p3+p4)
C-----K2=-(p1+p4)
C     K1flat=gamma/(gamma**2-S1*S2)*(gamma*K1-S1*K2)
C     K1flat=gamma/(gamma**2-S1*S2)*((S1-gamma)*(p1+p4)-gamma*((p2+p3))

C----solve for gamma_+ and gamma_-
      S1=dcmplx(mhsq)
      S2=dcmplx(s14)
      K1DK2=dcmplx(s14+0.5d0*(s12+s13+s24+s34))

C-gamma+ = K1DK2+sqrt(K1DK2**2-S1*S2)
C-gamma- = K1DK2-sqrt(K1DK2**2-S1*S2)

      coef3m1423=czip
      gammap=K1DK2+sqrt(K1DK2**2-S1*S2)
      gammam=2d0*K1DK2-gammap

      factor=gammap/(gammap**2-S1*S2)
      a1=factor*(S1-gammap)
      a4=a1
      a3=-factor*gammap
      a2=a3

*XC rewrite (a1*s12+a3*s23+a4*s24)=d2K=d1 for better numerical stability in 3 soft limit
      d1=factor*s23*mhsq*gammap/(gammam-mhsq)
     .  *(s23/(s14-gammap)+(s13+s23+s34)/gammap)
*XC rewrite (a2*s12+a3*s13+a4*s14)=d1K=d2 for better numerical stability in 4 soft limit
      d2=factor*s14*mhsq/gammam/(gammam-mhsq)
     .  *(-gammam*s23+(mhsq-gammam)*(s12+s13))

      coef3m1423=coef3m1423
     &  -mhsq**2*za14**2/(2d0*gammap*(gammap-mhsq))
     & *(a2*za32*zb21+a4*za34*zb41)   ! *zab2(k3,k1f,k1)
     & *(a1*za31*zb12+a4*za34*zb42)   ! *zab2(k3,k1f,k2)
     * /d2                                                ! (2*k1.k1f)^-1*
     * /d1                                                ! (2*k2.k1f)^-1*

      factor=gammam/(gammam**2-S1*S2)
      a1=factor*(S1-gammam)
      a4=a1
      a3=-factor*gammam
      a2=a3

*XC rewrite (a1*s12+a3*s23+a4*s24)=d2K=d1 for better numerical stability in 3 soft limit
      d1=factor*s23*mhsq*gammam/(gammap-mhsq)
     .  *(s23/(s14-gammam)+(s13+s23+s34)/gammam)
*XC rewrite (a2*s12+a3*s13+a4*s14)=d1K=d2 for better numerical stability in 4 soft limit
      d2=factor*s14*mhsq/gammap/(gammap-mhsq)
     .  *(-gammap*s23+(mhsq-gammap)*(s12+s13))

      coef3m1423=coef3m1423
     &  -mhsq**2*za14**2/(2d0*gammam*(gammam-mhsq))
     & *(a2*za32*zb21+a4*za34*zb41)   ! *zab2(k3,k1f,k1)
     & *(a1*za31*zb12+a4*za34*zb42)   ! *zab2(k3,k1f,k2)
     * /d2                                                ! (2*k1.k1f)^-1*
     * /d1                                                ! (2*k2.k1f)^-1*

      sum=sum-coef3m1423*I3m(mhsq,s14,s23)

      sum=sum
     & -0.5d0*(za14*zb12*z2ab3124)**2
     & /(zb14*zb24*s124)
     & *BGRL2hat(s124,s12)

     & +2d0*za34*z2ab3142/zb14
     & *BGRL1(s124,s12)

     & +0.5d0*z2ab3142**2/(zb14*zb24*s124)
     & *lnrat(-s124,-s12)

     & -0.5d0*(za14*zb24*s234)**2
     & /(zb23*zb34*z2ab1234)
     & *BGRL2hat(s234,s23)

     & -2d0*za34*z2ab1342/zb34
     & *BGRL1(s234,s23)

     & +0.5d0*z2ab1342**2
     & /(zb23*zb34*z2ab1234)
     & *lnrat(-s234,-s23)

     &-0.5d0*(za12*zb12*z2ab4231)**2*zb23
     & /(zb13**3*s123)
     & *BGRL2hat(s123,s23)

     & +2d0*za13*zb12*z2ab4123*z2ab4231
     & /(za23*zb13**2*zb23)
     & *BGRL1(s123,s23)

     & +(-2d0*za13*zb12*z2ab4123*z2ab4231
     & /(zb13**2*za23*zb23*s123)
     & +0.5d0*z2ab4231**2*zb23
     & /(zb13**3*s123))
     & *lnrat(-s123,-s23)

     & -0.5d0*(za13*zb12*z2ab4123)**2
     & /(zb13*zb23*s123)
     & *BGRL2hat(s123,s12)

     & +za34*zb12*z2ab4123
     & *(-2d0*za13*zb13-za23*zb23)
     & /(za23*zb13**2*zb23)
     & *BGRL1(s123,s12)

     & +zb12*z2ab4123
     & *(za23*z2ab4132+2d0*za13*z2ab4231)
     & /(zb13**2*za23*zb23*s123)
     & *lnrat(-s123,-s12)
      zA1phiAQggmpmmR=zA1phiAQggmpmmR+sum
c--- now add the rational pieces
      sum=
     .-(za24**2*zb21**2)/(2d0*za23*zb31**3)
     .+(z2ab4123**2*zb21**2)
     .  /(2d0*s123*zb31**3*zb32)
     .-(za14**2*zb21)/(2*za12*zb31*zb32)
     .+(zb21*(za13**2*za23
     .  *z2ab4123**2*zb31**2
     .+za12**3*z2ab4231**2*zb21*zb32))
     .  /(4d0*s123**2*za12*za23*zb31**3*zb32)
     .+z2ab3142**2/(2d0*s124*zb41*zb42)
     .-(za13**2*zb21)/(2d0*za12*zb41*zb42)
     .+(za14**2*z2ab3124**2*zb21)
     .  /(4d0*s124**2*za12*zb41*zb42)
     .-(za13*za14*zb42)/(2d0*z2ab1234*zb43)
     .-(s234*za14**2*zb42**2)
     .  /(4d0*za23*z2ab1234*zb32**2*zb43)
     .-(za14**2*zb42**2)
     .  /(2d0*z2ab1234*zb32*zb43)

      zA1phiAQggmpmmR=zA1phiAQggmpmmR+sum
      endif
      return
      end
************************************************************************
      double complex function zA1phiAQggmpmmF(perm,renscale2)
c--- This is an implementation of Eq. (5.13) in
c---  S.~Badger, John.~M.~Campbell, R.~Keith Ellis and Ciaran Williams
c---  "Analytic results for the one-loop NMHV H-qbar-q-g-g amplitude."
c---   preprint DESY 09-180, FERMILAB-PUB-09-505-T, IPPP/09/86
c---   arXiv: 0910.4481 [hep-ph]
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /order/ieorder
      common /koppel/ nf,nc,b0
      integer j,j1,j2,j3,j4,j5,j6,perm(6),permm(6)
      double complex zB2g0Hmmmp,lnrat,za(5,5),zb(5,5)
      double complex l12,zab2,BGRL1,BGRL2hat,BGRL3hat
      double precision s123,s124,mhsq,epinv,deltar,musq
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43
      integer ipset

      musq=renscale2
      epinv=0d0
      deltar=1d0
      czip=(0d0,0d0)

      permm(1)=perm(1)
      permm(2)=perm(4)
      permm(3)=perm(3)
      permm(4)=perm(2)
      permm(5)=perm(5)
      permm(6)=perm(6)

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za34 => kin(ipset)%zA(j3,j4)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za43 => kin(ipset)%zA(j4,j3)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb34 => kin(ipset)%zB(j3,j4)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb43 => kin(ipset)%zB(j4,j3)

      z2ab2143=zab2(j2,j1,j4,j3,ipset)
      z2ab1342=zab2(j1,j3,j4,j2,ipset)
      z2ab1234=zab2(j1,j2,j3,j4,ipset)
      z2ab4123=zab2(j4,j1,j2,j3,ipset)
      z2ab3124=zab2(j3,j1,j2,j4,ipset)
      z2ab4231=zab2(j4,j2,j3,j1,ipset)
      z2ab4132=zab2(j4,j1,j3,j2,ipset)
      z2ab3241=zab2(j3,j2,j4,j1,ipset)
      z2ab3142=zab2(j3,j1,j4,j2,ipset)
      z2ab4134=zab2(j4,j1,j3,j4,ipset)

      s123=s12+s13+s23
      s124=s12+s14+s24

      l12=lnrat(musq,-s12)

      if(ieorder.eq.-2)then
      zA1phiAQggmpmmF=0d0
      elseif(ieorder.eq.-1)then
      zA1phiAQggmpmmF=0d0
      elseif(ieorder.eq.0)then
      zA1phiAQggmpmmF=zB2g0Hmmmp(permm(1))
     . *(-2d0/3d0*(epinv+l12)-10d0/9d0)
     & +2d0/3d0*za13*z2ab4132**2
     & /(za12*zb12*zb23*s123)*lnrat(-s123,-s12)
     & -2d0/3d0*(s24-s124)*z2ab3142**2
     & /(za12*zb14*zb24*zb12*s124)*lnrat(-s124,-s12)
     & -2d0/3d0*za13*z2ab4132**2
     & /(za12*zb23*zb12)*BGRL1(s123,s12)
     & +2d0/3d0*za14*z2ab3142**2
     & /(za12*zb24*zb12)*BGRL1(s124,s12)
     & +za13*za34*z2ab4132/3d0*BGRL2hat(s123,s12)
     & +za14*za34*z2ab3142/3d0*BGRL2hat(s124,s12)
     & +2d0/3d0*za13**2*za34*zb12*z2ab4123
     & *BGRL3hat(s123,s12)
     & +2d0/3d0*za14**2*za34*zb12*z2ab3124
     & *BGRL3hat(s124,s12)
      zA1phiAQggmpmmF=zA1phiAQggmpmmF
     & -za13*za34*z2ab4132
     & /(6d0*za12*zb12*s123)
     & -za14*za34*z2ab3142
     & /(6d0*za12*zb12*s124)
     & +(
     &  -za13*za14*zb12*zb34)
     & /(3d0*za12*zb12*zb34**2)
      endif
      return
      end
************************************************************************
      double complex function zA1phiAQggmpmpL(perm,renscale2)
C     implementation of arXiv:0906.0008v1, Eq. 4.24
c--- the function defined in this routine is in fact (-i*A_4),
c---   i.e. complete LHS
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /order/ieorder
      common /koppel/ nf,nc,b0
      integer j,j1,j2,j3,j4,j5,j6,perm(6),permm(6)
      double complex za(5,5),zb(5,5)
      double precision s(5,5),epinv,deltar,musq
      double complex zab2,V1L,L2,L1,L0,lnrat,sum,
     . zAphiq2gqmpmp
      double complex l23,l34,l41,l12,Lsm1_2me,Lsm1DS
      double precision ss3,mhsq,s123,s234,s341,s412
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43
      integer ipset

      musq=renscale2
      epinv=0d0
      deltar=1d0
      czip=(0d0,0d0)
*      write(*,*)'hello zA1phiAQggmpmpL'

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      permm(1)=perm(1)
      permm(2)=perm(4)
      permm(3)=perm(3)
      permm(4)=perm(2)
      permm(5)=perm(5)
      permm(6)=perm(6)

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za34 => kin(ipset)%zA(j3,j4)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za43 => kin(ipset)%zA(j4,j3)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb34 => kin(ipset)%zB(j3,j4)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb43 => kin(ipset)%zB(j4,j3)

      s123=ss3(j1,j2,j3,ipset)
      s234=ss3(j2,j3,j4,ipset)
      s341=ss3(j3,j4,j1,ipset)
      s412=ss3(j4,j1,j2,ipset)

      z2ab2143=zab2(j2,j1,j4,j3,ipset)
      z2ab1342=zab2(j1,j3,j4,j2,ipset)
      z2ab1234=zab2(j1,j2,j3,j4,ipset)
      z2ab4123=zab2(j4,j1,j2,j3,ipset)
      z2ab3124=zab2(j3,j1,j2,j4,ipset)
      z2ab4231=zab2(j4,j2,j3,j1,ipset)
      z2ab4132=zab2(j4,j1,j3,j2,ipset)
      z2ab3241=zab2(j3,j2,j4,j1,ipset)
      z2ab3142=zab2(j3,j1,j4,j2,ipset)
      z2ab4134=zab2(j4,j1,j3,j4,ipset)

      mhsq=s12+s13+s14+s23+s24+s34
      l12=lnrat(musq,-s12)
      l23=lnrat(musq,-s23)
      l34=lnrat(musq,-s34)
      l41=lnrat(musq,-s41)

      if(ieorder.eq.-2)then
      V1L=-3d0
      zA1phiAQggmpmpL=zAphiq2gqmpmp(permm(1))*V1L
      elseif(ieorder.eq.-1)then
      V1L=
     . -l23
     . -l34
     . -l41
     . +13d0/6d0
      zA1phiAQggmpmpL=zAphiq2gqmpmp(permm(1))*V1L
      elseif(ieorder.eq.0)then
      V1L=
     . -epinv**2-epinv*l23-0.5d0*l23**2
     . -epinv**2-epinv*l34-0.5d0*l34**2
     . -epinv**2-epinv*l41-0.5d0*l41**2
     . +13d0/6d0*(epinv+l12)
     . +119d0/18d0-deltar/6d0
     . -Lsm1_2me(s123,s234,s23,mhsq)
     . -Lsm1_2me(s341,s412,s41,mhsq)
     . -Lsm1_2me(s412,s123,s12,mhsq)

      zA1phiAQggmpmpL=zAphiq2gqmpmp(permm(1))
     . *(V1L-13d0/6d0*lnrat(-s412,-s12)
     .      -Lsm1DS(s34,s41,s341)
     .      -Lsm1DS(s12,s23,s123))

      sum=
     . +za14**2*za23**3/(za12*za34*za24**3)
     .  *(Lsm1DS(s23,s34,s234)+Lsm1DS(s41,s12,s412))
     . +2d0/3d0*za12**2*za34**2*zb24**3/za14
     .  *L2(-s412,-s12)/s12**3
     . -0.5d0*za12*za23*za34*zb24**2/za24
     .  *L1(-s234,-s34)/s34**2
     . +(0.5d0*za14*z2ab3124**2/za24
     .  -1d0/3d0*za13*za14*z2ab3124**2
     .          /(za12*za34)
     .  -2d0/3d0*za13*za12*za34*zb24**2/za14)
     .  *L1(-s412,-s12)/s12**2
     . -(za12*zb24*z2ab3142**2/(za14*zb12)
     .  +0.5d0*za14*za23**2*zb24**2/za24)
     .  *L1(-s412,-s41)/s41**2
     . -(zb24*za34*z2ab1234**2/(za14*zb34)
     .  +0.5d0*za14*za23**2*zb24**2/za24)
     .  *L1(-s234,-s23)/s23**2
     . +(3d0*za13**2*z2ab3124/(za12*za34)
     .  +2d0*za13*z2ab3124**2
     .      /(za12*za34*zb14)
     .  +1d0/3d0*za13**2*zb24/za14
     .  -z2ab3124**2/(zb14*za24))
     .  *L0(-s412,-s12)/s12
     . +3d0*za23*za13*zb24/za24
     .  *(L0(-s234,-s23)/s23+L0(-s412,-s41)/s41)
     . +za23*zb24*(za12*za34
     .                      +2d0*za14*za23)/za24**2
     .  *L0(-s234,-s34)/s34
     . -(1d0/3d0*za13**3/(za12*za34*za14)
     .  +0.5d0*za23*za13**2/(za12*za24*za34)
     .  +za23**2*zb24/(za24**2*zb14)
     .  +2d0*za23**3*za14*zb24
     .      /(za24**2*za34*za12*zb14))
     .  *lnrat(-s412,-s12)
     . +za12**2*za34*zb24
     .  /(za24**2*za14*zb34)*lnrat(-s234,-s23)
     . +za34**2*za12*zb24
     .  /(za24**2*za14*zb12)*lnrat(-s412,-s41)
     . -5d0/6d0*za13**2*zb24/(s12*za14)
     . -1d0/3d0*za13**2*z2ab3124
     .  /(s12*za12*za34)
     . -1d0/3d0*za13*zb24
     .  *(2d0*za34*zb42+za31*zb12)
     .  /(s412*za14*zb12)
     . +0.5d0*(
     .  zb24*z2ab3124*z2ab3241
     .  /(s412*zb14*zb12*za24)
     . -za13**2*zb14/(s12*za24)
     . -za13*zb24*za34/(za14*zb12*za24)
     . +za12*zb24**2/(zb23*zb34*za24) )
     . +za13*zb24*z2ab1234
     .  /(s23*za14*zb34)
     . -za13*zb24*z2ab3142
     .  /(s41*za14*zb12)
     . -zb24**2*za34*za23
     .  /(s41*za24*zb12)
     . -2d0*zAcphiq2gqmpmp(permm(1))

      zA1phiAQggmpmpL=zA1phiAQggmpmpL+sum
      endif
      return
      end
************************************************************************
      double complex function zA1phiAQggmpmpR(perm,renscale2)
C     implementation of arXiv:0906.0008v1, Eq. 4.25
c--- the function defined in this routine is in fact (-i*A_4),
c---   i.e. complete LHS
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /order/ieorder
      common /koppel/ nf,nc,b0
      integer j,j1,j2,j3,j4,j5,j6,perm(6),permm(6)
      double complex zab2,VR,Lsm1_2me,lnrat,l12,L0,L1,
     & Lsm1DS,za(5,5),zb(5,5)
      double precision s(5,5),ss3,mhsq,s341,s234,s412,epinv,deltar,musq
*      write(*,*)'hello zA1phiAQggmpmpR'
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43
      integer ipset

      musq=renscale2
      epinv=0d0
      deltar=1d0
      czip=(0d0,0d0)

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      permm(1)=perm(1)
      permm(2)=perm(4)
      permm(3)=perm(3)
      permm(4)=perm(2)
      permm(5)=perm(5)
      permm(6)=perm(6)

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za34 => kin(ipset)%zA(j3,j4)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za43 => kin(ipset)%zA(j4,j3)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb34 => kin(ipset)%zB(j3,j4)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb43 => kin(ipset)%zB(j4,j3)

      z2ab2143=zab2(j2,j1,j4,j3,ipset)
      z2ab1342=zab2(j1,j3,j4,j2,ipset)
      z2ab1234=zab2(j1,j2,j3,j4,ipset)
      z2ab4123=zab2(j4,j1,j2,j3,ipset)
      z2ab3124=zab2(j3,j1,j2,j4,ipset)
      z2ab4231=zab2(j4,j2,j3,j1,ipset)
      z2ab4132=zab2(j4,j1,j3,j2,ipset)
      z2ab3241=zab2(j3,j2,j4,j1,ipset)
      z2ab3142=zab2(j3,j1,j4,j2,ipset)
      z2ab4134=zab2(j4,j1,j3,j4,ipset)

      mhsq=s12+s13+s14+s23+s24+s34
      l12=lnrat(musq,-s12)
      s341=ss3(j3,j4,j1,ipset)
      s234=ss3(j2,j3,j4,ipset)
      s412=ss3(j4,j1,j2,ipset)

      if(ieorder.eq.-2)then
      VR=-1d0
      zA1phiAQggmpmpR=0d0!zAphiq2gqmpmp(permm(1))*VR
      elseif(ieorder.eq.-1)then
      VR=-l12
     .   -3d0/2d0
      zA1phiAQggmpmpR=0d0!zAphiq2gqmpmp(permm(1))*VR
      elseif(ieorder.eq.0)then
      VR=-epinv**2-epinv*l12-0.5d0*l12**2
     .   -3d0/2d0*(epinv+l12)-7d0/2d0-deltar/2d0
     .   -Lsm1_2me(s234,s341,s34,mhsq)
      zA1phiAQggmpmpR=
     . +za12**2*za34**2/(za14*za24**3)
     . *(Lsm1DS(s23,s34,s234)+Lsm1DS(s41,s12,s412))
     . -0.5d0*za12**2*za34**2*zb24**2
     .       /(za14*za24)*L1(-s412,-s12)/s12**2
     . +zb24*za34*z2ab1234**2/(za14*zb34)
     .       *L1(-s234,-s23)/s23**2
     . -(za12*za34*zb24**3/zb23
     .  +0.5d0*za23**3*z2ab1342**2
     .       /(za12*za34*za24))
     .  *L1(-s234,-s34)/s34**2
     . -0.5d0*za14*za23**2*zb24**2/za24
     .       *(L1(-s412,-s41)/s41**2
     .        -L1(-s234,-s23)/s23**2)
     . -za12**2*za34**2*zb24/(za14*za24**2)
     .       *L0(-s412,-s12)/s12
     . +za23*zb24*(
     .  2d0*za12*za34+za14*za23)/(za24**2)
     .       *L0(-s412,-s41)/s41
     . -za12**2*za34*zb24
     .  /(za24**2*za14*zb34)*lnrat(-s234,-s23)
     . +(za34*za12*zb24/(za24**2*zb23)
     .  +0.5d0*za23*za13**2/(za12*za24*za34))
     .       *lnrat(-s234,-s34)
     . -0.5d0*zb24*z2ab3124*z2ab3142
     .  /(s41*s412*zb12)
     . -0.5d0*zb24**2*za34*za23
     .  /(s41*za24*zb12)
     . -0.5d0*(za13*za23**2*z2ab1342)
     .  /(s34*za34*za12*za24)
     . +0.5d0*za23*zb24*z2ab1342*(s23+s34)
     .  /(s34*s234*zb23*za24)
     . +0.5d0*zb24**2*z2ab1234/(s234*zb23*zb34)
     . -za12*zb24*za34*z2ab1234
     .  /(s23*za14*zb34*za24)
     . +za13*zb24/(zb23*za24)

      zA1phiAQggmpmpR=zA1phiAQggmpmpR
     .             +zAphiq2gqmpmp(permm(1))*VR
      endif
      return
      end
************************************************************************
      double complex function zA1phiAQggmpmpF(perm,renscale2)
C     implementation of arXiv:0906.0008v1, Eq. 4.26
c--- the function defined in this routine is in fact (-i*A_4),
c---   i.e. complete LHS
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /order/ieorder
      common /koppel/ nf,nc,b0
      integer j,j1,j2,j3,j4,j5,j6,perm(6),permm(6)
      double complex l12,zab2,L2,lnrat,zAphiq2gqmpmp
      double precision s412,mhsq,epinv,deltar,musq
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43
      integer ipset

      musq=renscale2
      epinv=0d0
      deltar=1d0
      czip=(0d0,0d0)

*      write(*,*)'hello zA1phiAQggmpmpF'
      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      permm(1)=perm(1)
      permm(2)=perm(4)
      permm(3)=perm(3)
      permm(4)=perm(2)
      permm(5)=perm(5)
      permm(6)=perm(6)

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za34 => kin(ipset)%zA(j3,j4)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za43 => kin(ipset)%zA(j4,j3)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb34 => kin(ipset)%zB(j3,j4)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb43 => kin(ipset)%zB(j4,j3)

      s412=s41+s42+s12
      l12=lnrat(musq,-s12)

      z2ab3124=zab2(j3,j1,j2,j4,ipset)
      z2ab3142=zab2(j3,j1,j4,j2,ipset)

      if(ieorder.eq.-2)then
      zA1phiAQggmpmpF=0d0
      elseif(ieorder.eq.-1)then
      zA1phiAQggmpmpF=0d0
      elseif(ieorder.eq.0)then
      zA1phiAQggmpmpF=zAphiq2gqmpmp(permm(1))
     . *(-2d0/3d0*(epinv+l12)-10d0/9d0)
     .-1d0/3d0*(za14**2*z2ab3124**3/(za12*za34)
     .         +za12**2*zb24**3*za34**2/za14)
     .        *L2(-s412,-s12)/s12**3
     .+1d0/3d0*zAphiq2gqmpmp(permm(1))*lnrat(-s412,-s12)
     .+0.5d0*za13**2*zb24/(s12*za14)
     .+1d0/6d0*za13*(za31*zb12*z2ab3142
     .                   -(za34*zb42)**2)
     .                  /(zb12*za34*za14*s412)
     .+1d0/6d0*za13**3*s412
     .        /(s12*za12*za34*za14)
      endif
      return
      end
************************************************************************
      double complex function zA1cphiAQggmpmpL(perm,renscale2)
C     implementation of arXiv:0906.0008v1, Eq. 4.24
c--- the function defined in this routine is in fact (-i*A_4),
c---   i.e. complete LHS
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /order/ieorder
      common /koppel/ nf,nc,b0
      integer j,j1,j2,j3,j4,j5,j6,perm(6),permm(6)
      double complex za(5,5),zb(5,5)
      double precision s(5,5),epinv,deltar,musq
      double complex V1L,L2,L1,L0,lnrat,sum,
     . zAphiq2gqmpmp
      double complex l23,l34,l41,l12,Lsm1_2me,Lsm1DS
      double precision ss3,mhsq,s123,s234,s341,s412
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43
      integer ipset

      musq=renscale2
      epinv=0d0
      deltar=1d0
      czip=(0d0,0d0)
*      write(*,*)'hello zA1cphiAQggmpmpL'

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      permm(1)=perm(1)
      permm(2)=perm(4)
      permm(3)=perm(3)
      permm(4)=perm(2)
      permm(5)=perm(5)
      permm(6)=perm(6)

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zB(j1,j2)
      za13 => kin(ipset)%zB(j1,j3)
      za14 => kin(ipset)%zB(j1,j4)
      za23 => kin(ipset)%zB(j2,j3)
      za24 => kin(ipset)%zB(j2,j4)
      za34 => kin(ipset)%zB(j3,j4)

      za21 => kin(ipset)%zB(j2,j1)
      za31 => kin(ipset)%zB(j3,j1)
      za41 => kin(ipset)%zB(j4,j1)
      za32 => kin(ipset)%zB(j3,j2)
      za42 => kin(ipset)%zB(j4,j2)
      za43 => kin(ipset)%zB(j4,j3)

      zb12 => kin(ipset)%zA(j1,j2)
      zb13 => kin(ipset)%zA(j1,j3)
      zb14 => kin(ipset)%zA(j1,j4)
      zb23 => kin(ipset)%zA(j2,j3)
      zb24 => kin(ipset)%zA(j2,j4)
      zb34 => kin(ipset)%zA(j3,j4)

      zb21 => kin(ipset)%zA(j2,j1)
      zb31 => kin(ipset)%zA(j3,j1)
      zb41 => kin(ipset)%zA(j4,j1)
      zb32 => kin(ipset)%zA(j3,j2)
      zb42 => kin(ipset)%zA(j4,j2)
      zb43 => kin(ipset)%zA(j4,j3)

      s123=ss3(j1,j2,j3,ipset)
      s234=ss3(j2,j3,j4,ipset)
      s341=ss3(j3,j4,j1,ipset)
      s412=ss3(j4,j1,j2,ipset)

      z2ba2143=zba2(j2,j1,j4,j3,ipset)
      z2ba1342=zba2(j1,j3,j4,j2,ipset)
      z2ba1234=zba2(j1,j2,j3,j4,ipset)
      z2ba4123=zba2(j4,j1,j2,j3,ipset)
      z2ba3142=zba2(j3,j1,j4,j2,ipset)
      z2ba3124=zba2(j3,j1,j2,j4,ipset)
      z2ba4231=zba2(j4,j2,j3,j1,ipset)
      z2ba4132=zba2(j4,j1,j3,j2,ipset)
      z2ba3241=zba2(j3,j2,j4,j1,ipset)
      z2ba4134=zba2(j4,j1,j3,j4,ipset)
      z2ba2134=zba2(j2,j1,j3,j4,ipset)

      mhsq=s12+s13+s14+s23+s24+s34
      l12=lnrat(musq,-s12)
      l23=lnrat(musq,-s23)
      l34=lnrat(musq,-s34)
      l41=lnrat(musq,-s41)

      if(ieorder.eq.-2)then
      V1L=-3d0
      zA1cphiAQggmpmpL=zAphiq2gqpmpm(permm(1))*V1L
      elseif(ieorder.eq.-1)then
      V1L=
     . -l23
     . -l34
     . -l41
     . +13d0/6d0
      zA1cphiAQggmpmpL=zAphiq2gqpmpm(permm(1))*V1L
      elseif(ieorder.eq.0)then
      V1L=
     . -epinv**2-epinv*l23-0.5d0*l23**2
     . -epinv**2-epinv*l34-0.5d0*l34**2
     . -epinv**2-epinv*l41-0.5d0*l41**2
     . +13d0/6d0*(epinv+l12)
     . +119d0/18d0-deltar/6d0
     . -Lsm1_2me(s123,s234,s23,mhsq)
     . -Lsm1_2me(s341,s412,s41,mhsq)
     . -Lsm1_2me(s412,s123,s12,mhsq)

      zA1cphiAQggmpmpL=zAphiq2gqpmpm(permm(1))
     . *(V1L-13d0/6d0*lnrat(-s412,-s12)
     .      -Lsm1DS(s34,s41,s341)
     .      -Lsm1DS(s12,s23,s123))

      sum=
     . +za14**2*za23**3/(za12*za34*za24**3)
     .  *(Lsm1DS(s23,s34,s234)+Lsm1DS(s41,s12,s412))
     . +2d0/3d0*za12**2*za34**2*zb24**3/za14
     .  *L2(-s412,-s12)/s12**3
     . -0.5d0*za12*za23*za34*zb24**2/za24
     .  *L1(-s234,-s34)/s34**2
     . +(0.5d0*za14*z2ba3124**2/za24
     .  -1d0/3d0*za13*za14*z2ba3124**2
     .          /(za12*za34)
     .  -2d0/3d0*za13*za12*za34*zb24**2/za14)
     .  *L1(-s412,-s12)/s12**2
     . -(za12*zb24*z2ba3142**2/(za14*zb12)
     .  +0.5d0*za14*za23**2*zb24**2/za24)
     .  *L1(-s412,-s41)/s41**2
     . -(zb24*za34*z2ba1234**2/(za14*zb34)
     .  +0.5d0*za14*za23**2*zb24**2/za24)
     .  *L1(-s234,-s23)/s23**2
     . +(3d0*za13**2*z2ba3124/(za12*za34)
     .  +2d0*za13*z2ba3124**2
     .      /(za12*za34*zb14)
     .  +1d0/3d0*za13**2*zb24/za14
     .  -z2ba3124**2/(zb14*za24))
     .  *L0(-s412,-s12)/s12
     . +3d0*za23*za13*zb24/za24
     .  *(L0(-s234,-s23)/s23+L0(-s412,-s41)/s41)
     . +za23*zb24*(za12*za34
     .                      +2d0*za14*za23)/za24**2
     .  *L0(-s234,-s34)/s34
     . -(1d0/3d0*za13**3/(za12*za34*za14)
     .  +0.5d0*za23*za13**2/(za12*za24*za34)
     .  +za23**2*zb24/(za24**2*zb14)
     .  +2d0*za23**3*za14*zb24
     .      /(za24**2*za34*za12*zb14))
     .  *lnrat(-s412,-s12)
     . +za12**2*za34*zb24
     .  /(za24**2*za14*zb34)*lnrat(-s234,-s23)
     . +za34**2*za12*zb24
     .  /(za24**2*za14*zb12)*lnrat(-s412,-s41)
     . -5d0/6d0*za13**2*zb24/(s12*za14)
     . -1d0/3d0*za13**2*z2ba3124
     .  /(s12*za12*za34)
     . -1d0/3d0*za13*zb24
     .  *(2d0*za34*zb42+za31*zb12)
     .  /(s412*za14*zb12)
     . +0.5d0*(
     .  zb24*z2ba3124*z2ba3241
     .  /(s412*zb14*zb12*za24)
     . -za13**2*zb14/(s12*za24)
     . -za13*zb24*za34/(za14*zb12*za24)
     . +za12*zb24**2/(zb23*zb34*za24) )
     . +za13*zb24*z2ba1234
     .  /(s23*za14*zb34)
     . -za13*zb24*z2ba3142
     .  /(s41*za14*zb12)
     . -zb24**2*za34*za23
     .  /(s41*za24*zb12)
     . -2d0*zAcphiq2gqpmpm(permm(1))

      zA1cphiAQggmpmpL=zA1cphiAQggmpmpL+sum
      endif
      return
      end
************************************************************************
      double complex function zA1cphiAQggmpmpR(perm,renscale2)
C     implementation of arXiv:0906.0008v1, Eq. 4.25
c--- the function defined in this routine is in fact (-i*A_4),
c---   i.e. complete LHS
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /order/ieorder
      common /koppel/ nf,nc,b0
      integer j,j1,j2,j3,j4,j5,j6,perm(6),permm(6)
      double complex VR,Lsm1_2me,lnrat,l12,L0,L1,
     & Lsm1DS,za(5,5),zb(5,5)
      double precision s(5,5),ss3,mhsq,s341,s234,s412,epinv,deltar,musq
*      write(*,*)'hello zA1cphiAQggmpmpR'
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43
      integer ipset

      musq=renscale2
      epinv=0d0
      deltar=1d0
      czip=(0d0,0d0)

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      permm(1)=perm(1)
      permm(2)=perm(4)
      permm(3)=perm(3)
      permm(4)=perm(2)
      permm(5)=perm(5)
      permm(6)=perm(6)

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zB(j1,j2)
      za13 => kin(ipset)%zB(j1,j3)
      za14 => kin(ipset)%zB(j1,j4)
      za23 => kin(ipset)%zB(j2,j3)
      za24 => kin(ipset)%zB(j2,j4)
      za34 => kin(ipset)%zB(j3,j4)

      za21 => kin(ipset)%zB(j2,j1)
      za31 => kin(ipset)%zB(j3,j1)
      za41 => kin(ipset)%zB(j4,j1)
      za32 => kin(ipset)%zB(j3,j2)
      za42 => kin(ipset)%zB(j4,j2)
      za43 => kin(ipset)%zB(j4,j3)

      zb12 => kin(ipset)%zA(j1,j2)
      zb13 => kin(ipset)%zA(j1,j3)
      zb14 => kin(ipset)%zA(j1,j4)
      zb23 => kin(ipset)%zA(j2,j3)
      zb24 => kin(ipset)%zA(j2,j4)
      zb34 => kin(ipset)%zA(j3,j4)

      zb21 => kin(ipset)%zA(j2,j1)
      zb31 => kin(ipset)%zA(j3,j1)
      zb41 => kin(ipset)%zA(j4,j1)
      zb32 => kin(ipset)%zA(j3,j2)
      zb42 => kin(ipset)%zA(j4,j2)
      zb43 => kin(ipset)%zA(j4,j3)

      z2ba2143=zba2(j2,j1,j4,j3,ipset)
      z2ba1342=zba2(j1,j3,j4,j2,ipset)
      z2ba1234=zba2(j1,j2,j3,j4,ipset)
      z2ba4123=zba2(j4,j1,j2,j3,ipset)
      z2ba3142=zba2(j3,j1,j4,j2,ipset)
      z2ba3124=zba2(j3,j1,j2,j4,ipset)
      z2ba4231=zba2(j4,j2,j3,j1,ipset)
      z2ba4132=zba2(j4,j1,j3,j2,ipset)
      z2ba3241=zba2(j3,j2,j4,j1,ipset)
      z2ba4134=zba2(j4,j1,j3,j4,ipset)
      z2ba2134=zba2(j2,j1,j3,j4,ipset)

      mhsq=s12+s13+s14+s23+s24+s34
      l12=lnrat(musq,-s12)
      s341=ss3(j3,j4,j1,ipset)
      s234=ss3(j2,j3,j4,ipset)
      s412=ss3(j4,j1,j2,ipset)

      if(ieorder.eq.-2)then
      VR=-1d0
      zA1cphiAQggmpmpR=0d0!zAphiq2gqpmpm(permm(1))*VR
      elseif(ieorder.eq.-1)then
      VR=-l12
     .   -3d0/2d0
      zA1cphiAQggmpmpR=0d0!zAphiq2gqpmpm(permm(1))*VR
      elseif(ieorder.eq.0)then
      VR=-epinv**2-epinv*l12-0.5d0*l12**2
     .   -3d0/2d0*(epinv+l12)-7d0/2d0-deltar/2d0
     .   -Lsm1_2me(s234,s341,s34,mhsq)
      zA1cphiAQggmpmpR=
     . +za12**2*za34**2/(za14*za24**3)
     . *(Lsm1DS(s23,s34,s234)+Lsm1DS(s41,s12,s412))
     . -0.5d0*za12**2*za34**2*zb24**2
     .       /(za14*za24)*L1(-s412,-s12)/s12**2
     . +zb24*za34*z2ba1234**2/(za14*zb34)
     .       *L1(-s234,-s23)/s23**2
     . -(za12*za34*zb24**3/zb23
     .  +0.5d0*za23**3*z2ba1342**2
     .       /(za12*za34*za24))
     .  *L1(-s234,-s34)/s34**2
     . -0.5d0*za14*za23**2*zb24**2/za24
     .       *(L1(-s412,-s41)/s41**2
     .        -L1(-s234,-s23)/s23**2)
     . -za12**2*za34**2*zb24/(za14*za24**2)
     .       *L0(-s412,-s12)/s12
     . +za23*zb24*(
     .  2d0*za12*za34+za14*za23)/(za24**2)
     .       *L0(-s412,-s41)/s41
     . -za12**2*za34*zb24
     .  /(za24**2*za14*zb34)*lnrat(-s234,-s23)
     . +(za34*za12*zb24/(za24**2*zb23)
     .  +0.5d0*za23*za13**2/(za12*za24*za34))
     .       *lnrat(-s234,-s34)
     . -0.5d0*zb24*z2ba3124*z2ba3142
     .  /(s41*s412*zb12)
     . -0.5d0*zb24**2*za34*za23
     .  /(s41*za24*zb12)
     . -0.5d0*(za13*za23**2*z2ba1342)
     .  /(s34*za34*za12*za24)
     . +0.5d0*za23*zb24*z2ba1342*(s23+s34)
     .  /(s34*s234*zb23*za24)
     . +0.5d0*zb24**2*z2ba1234/(s234*zb23*zb34)
     . -za12*zb24*za34*z2ba1234
     .  /(s23*za14*zb34*za24)
     . +za13*zb24/(zb23*za24)

      zA1cphiAQggmpmpR=zA1cphiAQggmpmpR
     .             +zAphiq2gqpmpm(permm(1))*VR
      endif
      return
      end
************************************************************************
      double complex function zA1cphiAQggmpmpF(perm,renscale2)
C     implementation of arXiv:0906.0008v1, Eq. 4.26
c--- the function defined in this routine is in fact (-i*A_4),
c---   i.e. complete LHS
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /order/ieorder
      common /koppel/ nf,nc,b0
      integer j,j1,j2,j3,j4,j5,j6,perm(6),permm(6)
      double complex l12,zab2,L2,lnrat,zAphiq2gqmpmp
      double precision s412,mhsq,epinv,deltar,musq
      common /npars/npars,nspec,n
*      write(*,*)'hello zA1phiAQggmpmmF'
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43
      integer ipset

      musq=renscale2
      epinv=0d0
      deltar=1d0
      czip=(0d0,0d0)

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      permm(1)=perm(1)
      permm(2)=perm(4)
      permm(3)=perm(3)
      permm(4)=perm(2)
      permm(5)=perm(5)
      permm(6)=perm(6)

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za34 => kin(ipset)%zA(j3,j4)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za43 => kin(ipset)%zA(j4,j3)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb34 => kin(ipset)%zB(j3,j4)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb43 => kin(ipset)%zB(j4,j3)

      z2ba2143=zba2(j2,j1,j4,j3,ipset)
      z2ba1342=zba2(j1,j3,j4,j2,ipset)
      z2ba1234=zba2(j1,j2,j3,j4,ipset)
      z2ba4123=zba2(j4,j1,j2,j3,ipset)
      z2ba3142=zba2(j3,j1,j4,j2,ipset)
      z2ba3124=zba2(j3,j1,j2,j4,ipset)
      z2ba4231=zba2(j4,j2,j3,j1,ipset)
      z2ba4132=zba2(j4,j1,j3,j2,ipset)
      z2ba3241=zba2(j3,j2,j4,j1,ipset)
      z2ba4134=zba2(j4,j1,j3,j4,ipset)
      z2ba2134=zba2(j2,j1,j3,j4,ipset)

      s412=s41+s42+s12
      l12=lnrat(musq,-s12)

      if(ieorder.eq.-2)then
      zA1cphiAQggmpmpF=0d0
      elseif(ieorder.eq.-1)then
      zA1cphiAQggmpmpF=0d0
      elseif(ieorder.eq.0)then

      zA1cphiAQggmpmpF=zAphiq2gqpmpm(permm(1))
     . *(-2d0/3d0*(epinv+l12)-10d0/9d0)
     .-1d0/3d0*(zb14**2*z2ba3124**3/(zb12*zb34)
     .         +zb12**2*za24**3*zb34**2/zb14)
     .        *L2(-s412,-s12)/s12**3
     .+1d0/3d0*zAphiq2gqpmpm(permm(1))*lnrat(-s412,-s12)
     .+0.5d0*zb13**2*za24/(s12*zb14)
     .+1d0/6d0*zb13*(zb31*za12*z2ba3142
     .                   -(zb34*za42)**2)
     .                  /(za12*zb34*zb14*s412)
     .+1d0/6d0*zb13**3*s412
     .        /(s12*zb12*zb34*zb14)
      endif
      return
      end
************************************************************************
      double complex function zA1phiAQggmppmL(perm,renscale2)
C     implementation of arXiv:0906.0008v1, Eq. 4.19
c--- the function defined in this routine is in fact (-i*A_4),
c---   i.e. complete LHS
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /order/ieorder
      common /koppel/ nf,nc,b0
      integer j,j1,j2,j3,j4,j5,j6,perm(6),permm(6)
      double complex za(5,5),zb(5,5)
      double precision s(5,5),epinv,deltar,musq
      double complex V1L,L2,L1,L0,lnrat,sum
      double complex l23,l34,l41,l12,Lsm1_2me,Lsm1DS
      double precision mhsq,s123,s234,s341,s412
*      write(*,*)'hello zA1phiAQggmppmL'
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43
      integer ipset

      musq=renscale2
      epinv=0d0
      deltar=1d0
      czip=(0d0,0d0)

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      permm(1)=perm(1)
      permm(2)=perm(4)
      permm(3)=perm(3)
      permm(4)=perm(2)
      permm(5)=perm(5)
      permm(6)=perm(6)

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za34 => kin(ipset)%zA(j3,j4)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za43 => kin(ipset)%zA(j4,j3)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb34 => kin(ipset)%zB(j3,j4)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb43 => kin(ipset)%zB(j4,j3)

      s123=ss3(j1,j2,j3,ipset)
      s234=ss3(j2,j3,j4,ipset)
      s341=ss3(j3,j4,j1,ipset)
      s412=ss3(j4,j1,j2,ipset)

      z2ab2143=zab2(j2,j1,j4,j3,ipset)
      z2ab1342=zab2(j1,j3,j4,j2,ipset)
      z2ab1234=zab2(j1,j2,j3,j4,ipset)
      z2ab4123=zab2(j4,j1,j2,j3,ipset)
      z2ab3142=zab2(j3,j1,j4,j2,ipset)
      z2ab3124=zab2(j3,j1,j2,j4,ipset)
      z2ab4231=zab2(j4,j2,j3,j1,ipset)
      z2ab4132=zab2(j4,j1,j3,j2,ipset)
      z2ab3241=zab2(j3,j2,j4,j1,ipset)
      z2ab4134=zab2(j4,j1,j3,j4,ipset)
      z2ab2134=zab2(j2,j1,j3,j4,ipset)

      mhsq=s12+s13+s14+s23+s24+s34
      l12=lnrat(musq,-s12)
      l23=lnrat(musq,-s23)
      l34=lnrat(musq,-s34)
      l41=lnrat(musq,-s41)

      if(ieorder.eq.-2)then
      V1L=-3d0
      zA1phiAQggmppmL=zAphiq2gqmmpp(permm(1))*V1L
      elseif(ieorder.eq.-1)then
      V1L=
     . -l23
     . -l34
     . -l41
     . +13d0/6d0
      zA1phiAQggmppmL=zAphiq2gqmmpp(permm(1))*V1L
      elseif(ieorder.eq.0)then

      V1L=
     . -epinv**2-epinv*l23-0.5d0*l23**2
     . -epinv**2-epinv*l34-0.5d0*l34**2
     . -epinv**2-epinv*l41-0.5d0*l41**2
     . +13d0/6d0*(epinv+l12)
     . +119d0/18d0-deltar/6d0
     . -Lsm1_2me(s123,s234,s23,mhsq)
     . -Lsm1_2me(s341,s412,s41,mhsq)
     . -Lsm1_2me(s412,s123,s12,mhsq)
      zA1phiAQggmppmL=zAphiq2gqmmpp(permm(1))
     . *(V1L-Lsm1DS(s23,s34,s234)
     .      -Lsm1DS(s41,s12,s412))
      sum=
     . +za14**3/(za12*za34*za13)
     . *(Lsm1DS(s12,s23,s123)
     .  +Lsm1DS(s34,s41,s341))
     . +(4d0/3d0*za13**2*z2ab4123**3/(za12*za34)
     . -za12*zb23**2*za34*z2ab4123
     . -1d0/3d0*za12**2*zb23**3*za34**2/za13)
     . *L2(-s123,-s12)/s12**3
     . +(0.5d0*za13**2*za24*z2ab4123**2
     . /(za12*za23*za34)
     . +za13*za14*z2ab4123**2/(za12*za34)
     . +0.5d0*za12*za34*zb23**2*za14/za13)
     . *L1(-s123,-s12)/s12**2
     . -0.5d0*za12*za34*za24*zb23**2
     . /za23*L1(-s234,-s34)/s34**2
     . +za14**2*z2ab4123/(za12*za34)
     . *L0(-s123,-s12)/s12

     . -2d0*za14*za24*zb23/za23
     . *(L0(-s123,-s12)/s12
     .  +L0(-s234,-s34)/s34)

c--- check: looks like there should be a factor of
c---        (-im) in front of A0 here
     . -5d0/6d0*(2d0*zAphiq2gqmmpp(permm(1))
     . +za14**3/(za12*za34*za13))
     . *lnrat(-s123,-s12)

     . +5d0/6d0*za14**2*z2ab4123
     . /(s12*za12*za34)
     . -1d0/6d0*za14**2*zb23*za34
     . /(za23*za13*z2ab4132)
     . +2d0/3d0*za14*zb23*za34
     . /(zb12*za23*za13)

     . -2d0/3d0*za14*za24*z2ab4132
     . /(s12*za23*za34)
     . +1d0/3d0*zb23*z2ab4132*z2ab4231
     . /(s123*zb12**2*za23)

     . -1d0/6d0*z2ab4231
     . *(za41*zb12+2d0*za43*zb32)
     . *(2d0*za41*zb12+za43*zb32)**2
     . /(s123*zb12**2*za23*za34*z2ab4132)

     . +0.5d0*zb23*z2ab2143
     . /(zb14*za23*zb34)

     . +0.5d0*z2ab4132*z2ab4123
     . /(s123*za23*zb12)

     . -0.5d0*za14**2*zb13/(s12*za23)

     . -2d0*zAcphiq2gqmmpp(permm(1))

      zA1phiAQggmppmL=zA1phiAQggmppmL+sum
      endif
      return
      end
************************************************************************
      double complex function zA1phiAQggmppmR(perm,renscale2)
C     implementation of arXiv:0906.0008v1, Eq. 4.21
c--- the function defined in this routine is in fact (-i*A_4),
c---   i.e. complete LHS
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /order/ieorder
      common /koppel/ nf,nc,b0
      integer j,j1,j2,j3,j4,j5,j6,perm(6),permm(6)
      double complex za(5,5),zb(5,5)
      double precision s(5,5),epinv,deltar,musq
      double complex VR,A0phiAQggmppm,Lsm1_2me,lnrat,l12,L0,L1,
     . Lsm1DS
      double precision mhsq,s123,s234,s341
*      write(*,*)'hello zA1phiAQggmppmR'
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43
      integer ipset

      musq=renscale2
      epinv=0d0
      deltar=1d0
      czip=(0d0,0d0)

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      permm(1)=perm(1)
      permm(2)=perm(4)
      permm(3)=perm(3)
      permm(4)=perm(2)
      permm(5)=perm(5)
      permm(6)=perm(6)

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za34 => kin(ipset)%zA(j3,j4)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za43 => kin(ipset)%zA(j4,j3)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb34 => kin(ipset)%zB(j3,j4)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb43 => kin(ipset)%zB(j4,j3)

      z2ab2143=zab2(j2,j1,j4,j3,ipset)
      z2ab1342=zab2(j1,j3,j4,j2,ipset)
      z2ab1234=zab2(j1,j2,j3,j4,ipset)
      z2ab4123=zab2(j4,j1,j2,j3,ipset)
      z2ab3142=zab2(j3,j1,j4,j2,ipset)
      z2ab3124=zab2(j3,j1,j2,j4,ipset)
      z2ab4231=zab2(j4,j2,j3,j1,ipset)
      z2ab4132=zab2(j4,j1,j3,j2,ipset)
      z2ab3241=zab2(j3,j2,j4,j1,ipset)
      z2ab4134=zab2(j4,j1,j3,j4,ipset)
      z2ab2134=zab2(j2,j1,j3,j4,ipset)
      z2ab2341=zab2(j2,j3,j4,j1,ipset)

      s123=ss3(j1,j2,j3,ipset)
      s234=ss3(j2,j3,j4,ipset)
      s341=ss3(j3,j4,j1,ipset)

      mhsq=s12+s13+s14+s23+s24+s34
      l12=lnrat(musq,-s12)

      if(ieorder.eq.-2)then
      VR=-1d0
      zA1phiAQggmppmR=0d0!zAphiq2gqmmpp(permm(1))*VR
      elseif(ieorder.eq.-1)then
      VR=-l12
     .   -3d0/2d0
      zA1phiAQggmppmR=0d0!zAphiq2gqmmpp(permm(1))*VR
      elseif(ieorder.eq.0)then

      VR=-epinv**2-epinv*l12-0.5d0*l12**2
     .   -3d0/2d0*(epinv+l12)-7d0/2d0-deltar/2d0
     .   -Lsm1_2me(s234,s341,s34,mhsq)

      zA1phiAQggmppmR=
     . za14**2/(za23*za13)*
     .  (Lsm1DS(s12,s23,s123)+Lsm1DS(s34,s41,s341))
     . -0.5d0*za12**2*zb23**2*za34**2
     .  /(za23*za13)*L1(-s123,-s12)/(s12**2)
     . +0.5d0*za24**3*z2ab1342**2
     .  /(za12*za23*za34)
     .  *L1(-s234,-s34)/(s34**2)
     . -2d0*za12*za34*za14*zb23
     .  /(za23*za13)*L0(-s123,-s12)/s12
     . -2d0*za14*za24*zb23/za23
     .  *L0(-s234,-s34)/s34
     . -3d0/2d0*za14**2/(za23*za13)
     .  *lnrat(-s123,-s12)
     . +1d0/2d0*zAphiq2gqmmpp(permm(1))*lnrat(-s234,-s34)
     . +0.5d0*(
     .  za14*zb23*za34/(zb12*za23*za13)
     . +zb23*zb13*z2ab2143
     .  /(zb34*zb14*z2ab2341)
     . -z2ab4132*z2ab4123
     .  /(s123*za23*zb12)
     . +za14**2*za24**2*(s21+s23+s24)
     .  /(za12*za23*za34**2*z2ab2143)
     . -s341**2*zb23*za24**3
     .  /(s34*za23*za34
     .    *z2ab2143*z2ab2341)
     .        )

      zA1phiAQggmppmR=zA1phiAQggmppmR
     .             +zAphiq2gqmmpp(permm(1))*VR
      endif
      return
      end
************************************************************************
      double complex function zA1phiAQggmppmF(perm,renscale2)
C     implementation of arXiv:0906.0008v1, Eq. 4.23
c--- the function defined in this routine is in fact (-i*A_4),
c---   i.e. complete LHS
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /order/ieorder
      common /koppel/ nf,nc,b0
      integer j,j1,j2,j3,j4,j5,j6,perm(6),permm(6)
      double complex za(5,5),zb(5,5)
      double precision s(5,5),epinv,deltar,musq
      double complex l12,zab2,L2,lnrat,A0phiAQggmppm
      double precision ss3
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43
      integer ipset

*      write(*,*)'hello zA1phiAQggmppmF'
      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      permm(1)=perm(1)
      permm(2)=perm(4)
      permm(3)=perm(3)
      permm(4)=perm(2)
      permm(5)=perm(5)
      permm(6)=perm(6)

      musq=renscale2
      epinv=0d0
      deltar=1d0
      czip=(0d0,0d0)

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za34 => kin(ipset)%zA(j3,j4)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za43 => kin(ipset)%zA(j4,j3)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb34 => kin(ipset)%zB(j3,j4)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb43 => kin(ipset)%zB(j4,j3)

      z2ab2143=zab2(j2,j1,j4,j3,ipset)
      z2ab1342=zab2(j1,j3,j4,j2,ipset)
      z2ab1234=zab2(j1,j2,j3,j4,ipset)
      z2ab4123=zab2(j4,j1,j2,j3,ipset)
      z2ab3142=zab2(j3,j1,j4,j2,ipset)
      z2ab3124=zab2(j3,j1,j2,j4,ipset)
      z2ab4231=zab2(j4,j2,j3,j1,ipset)
      z2ab4132=zab2(j4,j1,j3,j2,ipset)
      z2ab3241=zab2(j3,j2,j4,j1,ipset)
      z2ab4134=zab2(j4,j1,j3,j4,ipset)
      z2ab2134=zab2(j2,j1,j3,j4,ipset)
      z2ab2341=zab2(j2,j3,j4,j1,ipset)

      s123=ss3(j1,j2,j3,ipset)

      l12=lnrat(musq,-s12)

      if(ieorder.eq.-2)then
      zA1phiAQggmppmF=0d0
      elseif(ieorder.eq.-1)then
      zA1phiAQggmppmF=0d0
      elseif(ieorder.eq.0)then
      zA1phiAQggmppmF=zAphiq2gqmmpp(permm(1))
     . *(-2d0/3d0*(epinv+l12)-10d0/9d0)
     .+1d0/3d0*(za12**2*zb23**3*za34**2/za13
     .         -z2ab4123**3*za13**2/(za12*za34))
     .        *L2(-s123,-s12)/s12**3
     .-1d0/3d0*(za14**2/(za23*za13)
     .         +za14**2*za24/(za12*za23*za34))
     .        *lnrat(-s123,-s12)
     .-0.5d0*za14**2*zb23/(s12*za13)
     .+1d0/6d0*za14*(za41*zb12*z2ab4132
     .                   -(za43*zb32)**2)
     .                  /(zb12*za34*za13*s123)
     .+1d0/6d0*za14**3*s123
     .        /(s12*za12*za34*za13)
      endif
      return
      end
************************************************************************
      double complex function zA1cphiAQggmppmL(perm,renscale2)
C     implementation of arXiv:0906.0008v1, Eq. 4.19
c--- the function defined in this routine is in fact (-i*A_4),
c---   i.e. complete LHS
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /order/ieorder
      common /koppel/ nf,nc,b0
      integer j,j1,j2,j3,j4,j5,j6,perm(6),permm(6)
      double complex za(5,5),zb(5,5)
      double precision s(5,5),epinv,deltar,musq
      double complex V1L,L2,L1,L0,lnrat,sum
      double complex l23,l34,l41,l12,Lsm1_2me,Lsm1DS
      double precision mhsq,s123,s234,s341,s412
*      write(*,*)'hello zA1phiAQggmppmL'
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43
      integer ipset

      musq=renscale2
      epinv=0d0
      deltar=1d0
      czip=(0d0,0d0)

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      permm(1)=perm(1)
      permm(2)=perm(4)
      permm(3)=perm(3)
      permm(4)=perm(2)
      permm(5)=perm(5)
      permm(6)=perm(6)

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zB(j1,j2)
      za13 => kin(ipset)%zB(j1,j3)
      za14 => kin(ipset)%zB(j1,j4)
      za23 => kin(ipset)%zB(j2,j3)
      za24 => kin(ipset)%zB(j2,j4)
      za34 => kin(ipset)%zB(j3,j4)

      za21 => kin(ipset)%zB(j2,j1)
      za31 => kin(ipset)%zB(j3,j1)
      za41 => kin(ipset)%zB(j4,j1)
      za32 => kin(ipset)%zB(j3,j2)
      za42 => kin(ipset)%zB(j4,j2)
      za43 => kin(ipset)%zB(j4,j3)

      zb12 => kin(ipset)%zA(j1,j2)
      zb13 => kin(ipset)%zA(j1,j3)
      zb14 => kin(ipset)%zA(j1,j4)
      zb23 => kin(ipset)%zA(j2,j3)
      zb24 => kin(ipset)%zA(j2,j4)
      zb34 => kin(ipset)%zA(j3,j4)

      zb21 => kin(ipset)%zA(j2,j1)
      zb31 => kin(ipset)%zA(j3,j1)
      zb41 => kin(ipset)%zA(j4,j1)
      zb32 => kin(ipset)%zA(j3,j2)
      zb42 => kin(ipset)%zA(j4,j2)
      zb43 => kin(ipset)%zA(j4,j3)

      s123=ss3(j1,j2,j3,ipset)
      s234=ss3(j2,j3,j4,ipset)
      s341=ss3(j3,j4,j1,ipset)
      s412=ss3(j4,j1,j2,ipset)

      z2ba2143=zba2(j2,j1,j4,j3,ipset)
      z2ba1342=zba2(j1,j3,j4,j2,ipset)
      z2ba1234=zba2(j1,j2,j3,j4,ipset)
      z2ba4123=zba2(j4,j1,j2,j3,ipset)
      z2ba3142=zba2(j3,j1,j4,j2,ipset)
      z2ba3124=zba2(j3,j1,j2,j4,ipset)
      z2ba4231=zba2(j4,j2,j3,j1,ipset)
      z2ba4132=zba2(j4,j1,j3,j2,ipset)
      z2ba3241=zba2(j3,j2,j4,j1,ipset)
      z2ba4134=zba2(j4,j1,j3,j4,ipset)
      z2ba2134=zba2(j2,j1,j3,j4,ipset)

      mhsq=s12+s13+s14+s23+s24+s34
      l12=lnrat(musq,-s12)
      l23=lnrat(musq,-s23)
      l34=lnrat(musq,-s34)
      l41=lnrat(musq,-s41)

      if(ieorder.eq.-2)then
      V1L=-3d0
      zA1cphiAQggmppmL=zAphiq2gqppmm(permm(1))*V1L
      elseif(ieorder.eq.-1)then
      V1L=
     . -l23
     . -l34
     . -l41
     . +13d0/6d0
      zA1cphiAQggmppmL=zAphiq2gqppmm(permm(1))*V1L
      elseif(ieorder.eq.0)then

      V1L=
     . -epinv**2-epinv*l23-0.5d0*l23**2
     . -epinv**2-epinv*l34-0.5d0*l34**2
     . -epinv**2-epinv*l41-0.5d0*l41**2
     . +13d0/6d0*(epinv+l12)
     . +119d0/18d0-deltar/6d0
     . -Lsm1_2me(s123,s234,s23,mhsq)
     . -Lsm1_2me(s341,s412,s41,mhsq)
     . -Lsm1_2me(s412,s123,s12,mhsq)

      zA1cphiAQggmppmL=zAphiq2gqppmm(permm(1))
     . *(V1L-Lsm1DS(s23,s34,s234)
     .      -Lsm1DS(s41,s12,s412))
      sum=
     . +za14**3/(za12*za34*za13)
     . *(Lsm1DS(s12,s23,s123)
     .  +Lsm1DS(s34,s41,s341))
     . +(4d0/3d0*za13**2*z2ba4123**3/(za12*za34)
     . -za12*zb23**2*za34*z2ba4123
     . -1d0/3d0*za12**2*zb23**3*za34**2/za13)
     . *L2(-s123,-s12)/s12**3
     . +(0.5d0*za13**2*za24*z2ba4123**2
     . /(za12*za23*za34)
     . +za13*za14*z2ba4123**2/(za12*za34)
     . +0.5d0*za12*za34*zb23**2*za14/za13)
     . *L1(-s123,-s12)/s12**2
     . -0.5d0*za12*za34*za24*zb23**2
     . /za23*L1(-s234,-s34)/s34**2
     . +za14**2*z2ba4123/(za12*za34)
     . *L0(-s123,-s12)/s12

     . -2d0*za14*za24*zb23/za23
     . *(L0(-s123,-s12)/s12
     .  +L0(-s234,-s34)/s34)

c--- check: looks like there should be a factor of
c---        (-im) in front of A0 here
     . -5d0/6d0*(2d0*zAphiq2gqppmm(permm(1))
     . +za14**3/(za12*za34*za13))
     . *lnrat(-s123,-s12)

     . +5d0/6d0*za14**2*z2ba4123
     . /(s12*za12*za34)
     . -1d0/6d0*za14**2*zb23*za34
     . /(za23*za13*z2ba4132)
     . +2d0/3d0*za14*zb23*za34
     . /(zb12*za23*za13)

     . -2d0/3d0*za14*za24*z2ba4132
     . /(s12*za23*za34)
     . +1d0/3d0*zb23*z2ba4132*z2ba4231
     . /(s123*zb12**2*za23)

     . -1d0/6d0*z2ba4231
     . *(za41*zb12+2d0*za43*zb32)
     . *(2d0*za41*zb12+za43*zb32)**2
     . /(s123*zb12**2*za23*za34*z2ba4132)

     . +0.5d0*zb23*z2ba2143
     . /(zb14*za23*zb34)

     . +0.5d0*z2ba4132*z2ba4123
     . /(s123*za23*zb12)

     . -0.5d0*za14**2*zb13/(s12*za23)

     . -2d0*zAcphiq2gqppmm(permm(1))

      zA1cphiAQggmppmL=zA1cphiAQggmppmL+sum
      endif
      return
      end
************************************************************************
      double complex function zA1cphiAQggmppmR(perm,renscale2)
C     implementation of arXiv:0906.0008v1, Eq. 4.21
c--- the function defined in this routine is in fact (-i*A_4),
c---   i.e. complete LHS
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /order/ieorder
      common /koppel/ nf,nc,b0
      integer j,j1,j2,j3,j4,j5,j6,perm(6),permm(6)
      double complex za(5,5),zb(5,5)
      double precision s(5,5),epinv,deltar,musq
      double complex VR,Lsm1_2me,lnrat,l12,L0,L1,
     . Lsm1DS
      double precision mhsq,s123,s234,s341
      common /npars/npars,nspec,n
*      write(*,*)'hello zA1cphiAQggmppmR'
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43
      integer ipset

      musq=renscale2
      epinv=0d0
      deltar=1d0
      czip=(0d0,0d0)

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      permm(1)=perm(1)
      permm(2)=perm(4)
      permm(3)=perm(3)
      permm(4)=perm(2)
      permm(5)=perm(5)
      permm(6)=perm(6)

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zB(j1,j2)
      za13 => kin(ipset)%zB(j1,j3)
      za14 => kin(ipset)%zB(j1,j4)
      za23 => kin(ipset)%zB(j2,j3)
      za24 => kin(ipset)%zB(j2,j4)
      za34 => kin(ipset)%zB(j3,j4)

      za21 => kin(ipset)%zB(j2,j1)
      za31 => kin(ipset)%zB(j3,j1)
      za41 => kin(ipset)%zB(j4,j1)
      za32 => kin(ipset)%zB(j3,j2)
      za42 => kin(ipset)%zB(j4,j2)
      za43 => kin(ipset)%zB(j4,j3)

      zb12 => kin(ipset)%zA(j1,j2)
      zb13 => kin(ipset)%zA(j1,j3)
      zb14 => kin(ipset)%zA(j1,j4)
      zb23 => kin(ipset)%zA(j2,j3)
      zb24 => kin(ipset)%zA(j2,j4)
      zb34 => kin(ipset)%zA(j3,j4)

      zb21 => kin(ipset)%zA(j2,j1)
      zb31 => kin(ipset)%zA(j3,j1)
      zb41 => kin(ipset)%zA(j4,j1)
      zb32 => kin(ipset)%zA(j3,j2)
      zb42 => kin(ipset)%zA(j4,j2)
      zb43 => kin(ipset)%zA(j4,j3)

      s123=ss3(j1,j2,j3,ipset)
      s234=ss3(j2,j3,j4,ipset)
      s341=ss3(j3,j4,j1,ipset)

      z2ba2143=zba2(j2,j1,j4,j3,ipset)
      z2ba1342=zba2(j1,j3,j4,j2,ipset)
      z2ba1234=zba2(j1,j2,j3,j4,ipset)
      z2ba4123=zba2(j4,j1,j2,j3,ipset)
      z2ba3142=zba2(j3,j1,j4,j2,ipset)
      z2ba3124=zba2(j3,j1,j2,j4,ipset)
      z2ba4231=zba2(j4,j2,j3,j1,ipset)
      z2ba4132=zba2(j4,j1,j3,j2,ipset)
      z2ba3241=zba2(j3,j2,j4,j1,ipset)
      z2ba4134=zba2(j4,j1,j3,j4,ipset)
      z2ba2134=zba2(j2,j1,j3,j4,ipset)
      z2ba2341=zba2(j2,j3,j4,j1,ipset)

      mhsq=s12+s13+s14+s23+s24+s34
      l12=lnrat(musq,-s12)

      if(ieorder.eq.-2)then
      VR=-1d0
      zA1cphiAQggmppmR=0d0!zAphiq2gqppmm(permm(1))*VR
      elseif(ieorder.eq.-1)then
      VR=-l12
     .   -3d0/2d0
      zA1cphiAQggmppmR=0d0!zAphiq2gqppmm(permm(1))*VR
      elseif(ieorder.eq.0)then

      VR=-epinv**2-epinv*l12-0.5d0*l12**2
     .   -3d0/2d0*(epinv+l12)-7d0/2d0-deltar/2d0
     .   -Lsm1_2me(s234,s341,s34,mhsq)

      zA1cphiAQggmppmR=
     . za14**2/(za23*za13)*
     .  (Lsm1DS(s12,s23,s123)+Lsm1DS(s34,s41,s341))
     . -0.5d0*za12**2*zb23**2*za34**2
     .  /(za23*za13)*L1(-s123,-s12)/(s12**2)
     . +0.5d0*za24**3*z2ba1342**2
     .  /(za12*za23*za34)
     .  *L1(-s234,-s34)/(s34**2)
     . -2d0*za12*za34*za14*zb23
     .  /(za23*za13)*L0(-s123,-s12)/s12
     . -2d0*za14*za24*zb23/za23
     .  *L0(-s234,-s34)/s34
     . -3d0/2d0*za14**2/(za23*za13)
     .  *lnrat(-s123,-s12)
     . +1d0/2d0*zAphiq2gqppmm(permm(1))*lnrat(-s234,-s34)
     . +0.5d0*(
     .  za14*zb23*za34/(zb12*za23*za13)
     . +zb23*zb13*z2ba2143
     .  /(zb34*zb14*z2ba2341)
     . -z2ba4132*z2ba4123
     .  /(s123*za23*zb12)
     . +za14**2*za24**2*(s21+s23+s24)
     .  /(za12*za23*za34**2*z2ba2143)
     . -s341**2*zb23*za24**3
     .  /(s34*za23*za34
     .    *z2ba2143*z2ba2341)
     .        )

      zA1cphiAQggmppmR=zA1cphiAQggmppmR
     .             +zAphiq2gqppmm(permm(1))*VR
      endif
      return
      end
************************************************************************
      double complex function zA1cphiAQggmppmF(perm,renscale2)
C     implementation of arXiv:0906.0008v1, Eq. 4.23
c--- the function defined in this routine is in fact (-i*A_4),
c---   i.e. complete LHS
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /order/ieorder
      common /koppel/ nf,nc,b0
      integer j,j1,j2,j3,j4,j5,j6,perm(6),permm(6)
      double complex za(5,5),zb(5,5)
      double precision s(5,5),epinv,deltar,musq
      double complex l12,zab2,L2,lnrat
      double precision ss3
*      write(*,*)'hello zA1cphiAQggmppmF'
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43
      integer ipset

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      permm(1)=perm(1)
      permm(2)=perm(4)
      permm(3)=perm(3)
      permm(4)=perm(2)
      permm(5)=perm(5)
      permm(6)=perm(6)

      musq=renscale2
      epinv=0d0
      deltar=1d0
      czip=(0d0,0d0)

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za34 => kin(ipset)%zA(j3,j4)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za43 => kin(ipset)%zA(j4,j3)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb34 => kin(ipset)%zB(j3,j4)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb43 => kin(ipset)%zB(j4,j3)

      s123=ss3(j1,j2,j3,ipset)
      s234=ss3(j2,j3,j4,ipset)
      s341=ss3(j3,j4,j1,ipset)

      z2ba2143=zba2(j2,j1,j4,j3,ipset)
      z2ba1342=zba2(j1,j3,j4,j2,ipset)
      z2ba1234=zba2(j1,j2,j3,j4,ipset)
      z2ba4123=zba2(j4,j1,j2,j3,ipset)
      z2ba3142=zba2(j3,j1,j4,j2,ipset)
      z2ba3124=zba2(j3,j1,j2,j4,ipset)
      z2ba4231=zba2(j4,j2,j3,j1,ipset)
      z2ba4132=zba2(j4,j1,j3,j2,ipset)
      z2ba3241=zba2(j3,j2,j4,j1,ipset)
      z2ba4134=zba2(j4,j1,j3,j4,ipset)
      z2ba2134=zba2(j2,j1,j3,j4,ipset)
      z2ba2341=zba2(j2,j3,j4,j1,ipset)

      l12=lnrat(musq,-s12)

      if(ieorder.eq.-2)then
      zA1cphiAQggmppmF=0d0
      elseif(ieorder.eq.-1)then
      zA1cphiAQggmppmF=0d0
      elseif(ieorder.eq.0)then
      zA1cphiAQggmppmF=zAphiq2gqppmm(permm(1))
     . *(-2d0/3d0*(epinv+l12)-10d0/9d0)
     .+1d0/3d0*(zb12**2*za23**3*zb34**2/zb13
     .         -z2ba4123**3*zb13**2/(zb12*zb34))
     .        *L2(-s123,-s12)/s12**3
     .-1d0/3d0*(zb14**2/(zb23*zb13)
     .         +zb14**2*zb24/(zb12*zb23*zb34))
     .        *lnrat(-s123,-s12)
     .-0.5d0*zb14**2*za23/(s12*zb13)
     .+1d0/6d0*zb14*(zb41*za12*z2ba4132
     .                   -(zb43*za32)**2)
     .                  /(za12*zb34*zb13*s123)
     .+1d0/6d0*zb14**3*s123
     .        /(s12*zb12*zb34*zb13)
      endif
      return
      end
************************************************************************
      double complex function zA43phiAQggmpmm_unsym(perm,renscale2)
c--- This is an implementation of Eq. (5.17) in
c---  S.~Badger, John.~M.~Campbell, R.~Keith Ellis and Ciaran Williams
c---  "Analytic results for the one-loop NMHV H-qbar-q-g-g amplitude."
c---   preprint DESY 09-180, FERMILAB-PUB-09-505-T, IPPP/09/86
c---   arXiv: 0910.4481 [hep-ph]
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /order/ieorder
      common /koppel/ nf,nc,b0
      integer j,j1,j2,j3,j4,j5,j6,perm(6),permm(6)
      double complex za(5,5),zb(5,5)
      double precision s(5,5),epinv,deltar,musq
      double complex V5L,lnrat,sum
      double complex l34,l12,l24,l13,Lsm1,Lsm1_2mht
      double precision ss3,mhsq,s123,s234,s341,s412
*      write(*,*)'hello zA43phiAQggmpmm_unsym'
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43
      integer ipset

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      permm(1)=perm(1)
      permm(2)=perm(4)
      permm(3)=perm(3)
      permm(4)=perm(2)
      permm(5)=perm(5)
      permm(6)=perm(6)

      musq=renscale2
      epinv=0d0
      deltar=1d0
      czip=(0d0,0d0)

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za34 => kin(ipset)%zA(j3,j4)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za43 => kin(ipset)%zA(j4,j3)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb34 => kin(ipset)%zB(j3,j4)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb43 => kin(ipset)%zB(j4,j3)

      s123=ss3(j1,j2,j3,ipset)
      s234=ss3(j2,j3,j4,ipset)
      s341=ss3(j3,j4,j1,ipset)
      s412=ss3(j4,j1,j2,ipset)

      z2ab2143=zab2(j2,j1,j4,j3,ipset)
      z2ab1342=zab2(j1,j3,j4,j2,ipset)
      z2ab1234=zab2(j1,j2,j3,j4,ipset)
      z2ab4123=zab2(j4,j1,j2,j3,ipset)
      z2ab3142=zab2(j3,j1,j4,j2,ipset)
      z2ab3124=zab2(j3,j1,j2,j4,ipset)
      z2ab4231=zab2(j4,j2,j3,j1,ipset)
      z2ab4132=zab2(j4,j1,j3,j2,ipset)
      z2ab3241=zab2(j3,j2,j4,j1,ipset)
      z2ab4134=zab2(j4,j1,j3,j4,ipset)
      z2ab2134=zab2(j2,j1,j3,j4,ipset)
      z2ab2341=zab2(j2,j3,j4,j1,ipset)
      z2ab1243=zab2(j1,j2,j4,j3,ipset)

      mhsq=s12+s13+s14+s23+s24+s34
      l12=lnrat(musq,-s12)
      l24=lnrat(musq,-s24)
      l34=lnrat(musq,-s34)
      l13=lnrat(musq,-s13)

      if(ieorder.eq.-2)then
      V5L=0d0
      zA43phiAQggmpmm_unsym=zB2g0Hmmmp(permm(1))*V5L
      elseif(ieorder.eq.-1)then
      V5L=
     & -l12
     & -l34
     & +l13
     & +l24
      zA43phiAQggmpmm_unsym=zB2g0Hmmmp(permm(1))*V5L
      elseif(ieorder.eq.0)then
c--- new representation of poles
      V5L=
     & -epinv**2-epinv*l12-0.5d0*l12**2
     & -epinv**2-epinv*l34-0.5d0*l34**2
     & +epinv**2+epinv*l13+0.5d0*l13**2
     & +epinv**2+epinv*l24+0.5d0*l24**2

      sum=+zB2g0Hmmmp(permm(1))*V5L

      sum=sum
     & +((z2ab4123**2*zb12**2/zb23
     & +zb23**2*z2ab4231**3
     &  /(zb12*z2ab4123))/zb13**3
     & -mhsq**2*za13**3
     &  /(za12*z2ab1234*z2ab3124))
     & *Lsm1(-s12,-s123,-s23,-s123)/s123

     & +(mhsq**2*za14**2*za24
     &  /(za12*z2ab2143*z2ab4123)
     & -z2ab3142**2*z2ab3241
     &  /(zb14*z2ab3124*zb12))
     & *Lsm1(-s12,-s412,-s14,-s412)/s412

     & +(mhsq**2*za13**2/(z2ab1234*z2ab2134)
     & -z2ab4132**2/(zb13*zb23))
     & *Lsm1(-s13,-s123,-s23,-s123)/s123

     & +s341**2
     & /(zb13*zb34*z2ab2134)
     & *(Lsm1(-s13,-s341,-s14,-s341)
     &  +Lsm1_2mht(s23,s341,s14,mhsq))

     & +s341**2
     & /(zb14*zb34*z2ab2143)
     & *(Lsm1(-s14,-s341,-s34,-s341)
     &  +Lsm1_2mht(s12,s341,s34,mhsq))

     & -z2ab1342**2
     & /(zb23*zb34*z2ab1234)
     & *(Lsm1(-s23,-s234,-s34,-s234)
     &  +Lsm1_2mht(s12,s234,s34,mhsq))

     & +zb24**2*z2ab1243**2
     & /(zb23*zb34**3*z2ab1234)
     & *Lsm1(-s23,-s234,-s24,-s234)

     & -z2ab1342**2
     & /(z2ab1243*zb24*zb34)
     & *Lsm1_2mht(s14,s234,s23,mhsq)

     & +(-mhsq**2*za13**2*za23
     & /(za12*z2ab2134*z2ab3124)
     & +z2ab4132**2/z2ab4123
     & *z2ab4231/(zb12*zb13))
     & *Lsm1_2mht(s14,s123,s23,mhsq)/s123

     & +(mhsq**2*za13**3
     & /(za12*z2ab3124*z2ab1234)
     & -z2ab4132**3/(z2ab4123*zb12*zb23))
     & *Lsm1_2mht(s24,s123,s13,mhsq)/s123

     & +(-mhsq**2*za13**2
     & /(z2ab2134*z2ab1234)
     & +z2ab4132**2/(zb13*zb23))
     & *Lsm1_2mht(s34,s123,s12,mhsq)/s123

      zA43phiAQggmpmm_unsym=sum
      endif
      return
      end
************************************************************************
      double complex function zA43cphiAQggmpmm_unsym(perm,renscale2)
c--- This is an implementation of Eq. (5.17) in
c---  S.~Badger, John.~M.~Campbell, R.~Keith Ellis and Ciaran Williams
c---  "Analytic results for the one-loop NMHV H-qbar-q-g-g amplitude."
c---   preprint DESY 09-180, FERMILAB-PUB-09-505-T, IPPP/09/86
c---   arXiv: 0910.4481 [hep-ph]
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /order/ieorder
      common /koppel/ nf,nc,b0
      integer j,j1,j2,j3,j4,j5,j6,perm(6),permm(6)
      double complex za(5,5),zb(5,5)
      double precision s(5,5),epinv,deltar,musq
      double complex V5L,
     . A0phiAQggmpmm,A0phiAgQgmmpm,lnrat,sum
      double complex l34,l12,l24,l13,Lsm1,Lsm1_2mht
      double precision ss3,mhsq,s123,s234,s341,s412
*      write(*,*)'hello zA43cphiAQggmpmm_unsym'
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43
      integer ipset

      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      permm(1)=perm(1)
      permm(2)=perm(4)
      permm(3)=perm(3)
      permm(4)=perm(2)
      permm(5)=perm(5)
      permm(6)=perm(6)

      musq=renscale2
      epinv=0d0
      deltar=1d0
      czip=(0d0,0d0)

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zB(j1,j2)
      za13 => kin(ipset)%zB(j1,j3)
      za14 => kin(ipset)%zB(j1,j4)
      za23 => kin(ipset)%zB(j2,j3)
      za24 => kin(ipset)%zB(j2,j4)
      za34 => kin(ipset)%zB(j3,j4)

      za21 => kin(ipset)%zB(j2,j1)
      za31 => kin(ipset)%zB(j3,j1)
      za41 => kin(ipset)%zB(j4,j1)
      za32 => kin(ipset)%zB(j3,j2)
      za42 => kin(ipset)%zB(j4,j2)
      za43 => kin(ipset)%zB(j4,j3)

      zb12 => kin(ipset)%zA(j1,j2)
      zb13 => kin(ipset)%zA(j1,j3)
      zb14 => kin(ipset)%zA(j1,j4)
      zb23 => kin(ipset)%zA(j2,j3)
      zb24 => kin(ipset)%zA(j2,j4)
      zb34 => kin(ipset)%zA(j3,j4)

      zb21 => kin(ipset)%zA(j2,j1)
      zb31 => kin(ipset)%zA(j3,j1)
      zb41 => kin(ipset)%zA(j4,j1)
      zb32 => kin(ipset)%zA(j3,j2)
      zb42 => kin(ipset)%zA(j4,j2)
      zb43 => kin(ipset)%zA(j4,j3)

      s123=ss3(j1,j2,j3,ipset)
      s234=ss3(j2,j3,j4,ipset)
      s341=ss3(j3,j4,j1,ipset)
      s412=ss3(j4,j1,j2,ipset)

      z2ba2143=zba2(j2,j1,j4,j3,ipset)
      z2ba1342=zba2(j1,j3,j4,j2,ipset)
      z2ba1234=zba2(j1,j2,j3,j4,ipset)
      z2ba4123=zba2(j4,j1,j2,j3,ipset)
      z2ba3142=zba2(j3,j1,j4,j2,ipset)
      z2ba3124=zba2(j3,j1,j2,j4,ipset)
      z2ba4231=zba2(j4,j2,j3,j1,ipset)
      z2ba4132=zba2(j4,j1,j3,j2,ipset)
      z2ba3241=zba2(j3,j2,j4,j1,ipset)
      z2ba4134=zba2(j4,j1,j3,j4,ipset)
      z2ba2134=zba2(j2,j1,j3,j4,ipset)
      z2ba2341=zba2(j2,j3,j4,j1,ipset)
      z2ba1243=zba2(j1,j2,j4,j3,ipset)

      mhsq=s12+s13+s14+s23+s24+s34
      l12=lnrat(musq,-s12)
      l24=lnrat(musq,-s24)
      l34=lnrat(musq,-s34)
      l13=lnrat(musq,-s13)
      if(ieorder.eq.-2)then
      V5L=0d0
      zA43cphiAQggmpmm_unsym=zB2g0Hpppm(permm(1))*V5L
      elseif(ieorder.eq.-1)then
      V5L=
     & -l12
     & -l34
     & +l13
     & +l24
      zA43cphiAQggmpmm_unsym=zB2g0Hpppm(permm(1))*V5L
      elseif(ieorder.eq.0)then
c--- new representation of poles
      V5L=
     & -epinv**2-epinv*l12-0.5d0*l12**2
     & -epinv**2-epinv*l34-0.5d0*l34**2
     & +epinv**2+epinv*l13+0.5d0*l13**2
     & +epinv**2+epinv*l24+0.5d0*l24**2

      sum=+zB2g0Hpppm(permm(1))*V5L

      sum=sum
     & +((z2ba4123**2*zb12**2/zb23
     & +zb23**2*z2ba4231**3
     &  /(zb12*z2ba4123))/zb13**3
     & -mhsq**2*za13**3
     &  /(za12*z2ba1234*z2ba3124))
     & *Lsm1(-s12,-s123,-s23,-s123)/s123

     & +(mhsq**2*za14**2*za24
     &  /(za12*z2ba2143*z2ba4123)
     & -z2ba3142**2*z2ba3241
     &  /(zb14*z2ba3124*zb12))
     & *Lsm1(-s12,-s412,-s14,-s412)/s412

     & +(mhsq**2*za13**2/(z2ba1234*z2ba2134)
     & -z2ba4132**2/(zb13*zb23))
     & *Lsm1(-s13,-s123,-s23,-s123)/s123

     & +s341**2
     & /(zb13*zb34*z2ba2134)
     & *(Lsm1(-s13,-s341,-s14,-s341)
     &  +Lsm1_2mht(s23,s341,s14,mhsq))

     & +s341**2
     & /(zb14*zb34*z2ba2143)
     & *(Lsm1(-s14,-s341,-s34,-s341)
     &  +Lsm1_2mht(s12,s341,s34,mhsq))

     & -z2ba1342**2
     & /(zb23*zb34*z2ba1234)
     & *(Lsm1(-s23,-s234,-s34,-s234)
     &  +Lsm1_2mht(s12,s234,s34,mhsq))

     & +zb24**2*z2ba1243**2
     & /(zb23*zb34**3*z2ba1234)
     & *Lsm1(-s23,-s234,-s24,-s234)

     & -z2ba1342**2
     & /(z2ba1243*zb24*zb34)
     & *Lsm1_2mht(s14,s234,s23,mhsq)

     & +(-mhsq**2*za13**2*za23
     & /(za12*z2ba2134*z2ba3124)
     & +z2ba4132**2/z2ba4123
     & *z2ba4231/(zb12*zb13))
     & *Lsm1_2mht(s14,s123,s23,mhsq)/s123

     & +(mhsq**2*za13**3
     & /(za12*z2ba3124*z2ba1234)
     & -z2ba4132**3/(z2ba4123*zb12*zb23))
     & *Lsm1_2mht(s24,s123,s13,mhsq)/s123

     & +(-mhsq**2*za13**2
     & /(z2ba2134*z2ba1234)
     & +z2ba4132**2/(zb13*zb23))
     & *Lsm1_2mht(s34,s123,s12,mhsq)/s123

      zA43cphiAQggmpmm_unsym=sum
      endif
      return
      end
************************************************************************
c--- The expression below corresponds to one half of A4;3:
c---   Aleft(a1,q2,g3-,g4+)+Aright(a1,q2,g3-,g4+)+Aleft(a1,g3-,q2,g4+)

c--- It is obtained by summing the expressions given in:
c---  L.~J.~Dixon and Y.~Sofianatos,
c---  %``Analytic one-loop amplitudes for a Higgs boson plus four partons,''
c---  arXiv:0906.0008 [hep-ph].
      double complex function zA43phiAQggmpmp_unsym(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /order/ieorder
      common /koppel/ nf,nc,b0
      integer j,j1,j2,j3,j4,j5,j6,perm(6),permm(6)
      double complex za(5,5),zb(5,5)
      double precision s(5,5),epinv,deltar,musq
      double complex V5L,V6L,A0phiAQggmpmp,lnrat
      double complex l13,l34,l12,l24,Lsm1DS,Lsm1,Lsm1_2me
      double precision ss3,mhsq,s123,s234,s341,s412
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43
      integer ipset

      musq=renscale2
      epinv=0d0
      deltar=1d0
      czip=(0d0,0d0)
      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      permm(1)=perm(1)
      permm(2)=perm(4)
      permm(3)=perm(3)
      permm(4)=perm(2)
      permm(5)=perm(5)
      permm(6)=perm(6)

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za34 => kin(ipset)%zA(j3,j4)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za43 => kin(ipset)%zA(j4,j3)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb34 => kin(ipset)%zB(j3,j4)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb43 => kin(ipset)%zB(j4,j3)

      s123=ss3(j1,j2,j3,ipset)
      s234=ss3(j2,j3,j4,ipset)
      s341=ss3(j3,j4,j1,ipset)
      s412=ss3(j4,j1,j2,ipset)
      mhsq=s12+s13+s14+s23+s24+s34
      l12=lnrat(musq,-s12)
      l13=lnrat(musq,-s13)
      l24=lnrat(musq,-s24)
      l34=lnrat(musq,-s34)

      if(ieorder.eq.-2)then
      V5L=0d0
      zA43phiAQggmpmp_unsym=zAphiq2gqmpmp(permm(1))*V5L
      elseif(ieorder.eq.-1)then
      V5L=
     & -l12
     & -l34
     & +l13
     & +l24
      zA43phiAQggmpmp_unsym=zAphiq2gqmpmp(permm(1))*V5L
      elseif(ieorder.eq.0)then
c--- This is the same function as in the mpmm amplitude
      V5L=
     & -epinv**2-epinv*l12-0.5d0*l12**2
     & -epinv**2-epinv*l34-0.5d0*l34**2
     & +epinv**2+epinv*l13+0.5d0*l13**2
     & +epinv**2+epinv*l24+0.5d0*l24**2

c--- These are additional boxes
      V6L=
     . +Lsm1_2me(s341,s123,s13,mhsq)
     . +Lsm1_2me(s412,s234,s24,mhsq)
     . -Lsm1_2me(s412,s123,s12,mhsq)
     . -Lsm1_2me(s234,s341,s34,mhsq)
     . +Lsm1DS(s24,s41,s412)
     . +Lsm1DS(s13,s32,s123)
     . +Lsm1DS(s24,s23,s234)
     . +Lsm1DS(s13,s14,s341)

      zA43phiAQggmpmp_unsym=
     .  zAphiq2gqmpmp(permm(1))
     .  *(V5L+V6L
     .   -Lsm1DS(s34,s41,s341)
     .   -Lsm1DS(s12,s23,s123))

     . +(za14**2*za23**3/(za12*za34*za24**3)
     .  +za12**2*za34**2/(za14*za24**3))
     .  *(Lsm1DS(s23,s34,s234)
     .   +Lsm1DS(s41,s12,s412))
      endif
      return
      end
************************************************************************
c--- The expression below corresponds to one half of A4;3:
c---   Aleft(a1,q2,g3+,g4-)+Aright(a1,q2,g3+,g4-)+Aleft(a1,g3+,q2,g4-)

c--- It is obtained by summing the expressions given in:
c---  L.~J.~Dixon and Y.~Sofianatos,
c---  %``Analytic one-loop amplitudes for a Higgs boson plus four partons,''
c---  arXiv:0906.0008 [hep-ph].
      double complex function zA43phiAQggmppm_unsym(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /order/ieorder
      common /koppel/ nf,nc,b0
      integer j,j1,j2,j3,j4,j5,j6,perm(6),permm(6)
      double complex za(5,5),zb(5,5)
      double precision s(5,5),epinv,deltar,musq

      double complex V5L,V6l,A0phiAQggmppm,lnrat
      double complex l13,l34,l12,l24,Lsm1DS,Lsm1,Lsm1_2me
      double precision mhsq,s123,s234,s341,s412
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43
      integer ipset

      musq=renscale2
      epinv=0d0
      deltar=1d0
      czip=(0d0,0d0)
      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      permm(1)=perm(1)
      permm(2)=perm(4)
      permm(3)=perm(3)
      permm(4)=perm(2)
      permm(5)=perm(5)
      permm(6)=perm(6)

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za34 => kin(ipset)%zA(j3,j4)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za43 => kin(ipset)%zA(j4,j3)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb34 => kin(ipset)%zB(j3,j4)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb43 => kin(ipset)%zB(j4,j3)

      s123=ss3(j1,j2,j3,ipset)
      s234=ss3(j2,j3,j4,ipset)
      s341=ss3(j3,j4,j1,ipset)
      s412=ss3(j4,j1,j2,ipset)
      mhsq=s12+s13+s14+s23+s24+s34
      l12=lnrat(musq,-s12)
      l13=lnrat(musq,-s13)
      l24=lnrat(musq,-s24)
      l34=lnrat(musq,-s34)

      if(ieorder.eq.-2)then
      V5L=0d0
      zA43phiAQggmppm_unsym=zAphiq2gqmmpp(permm(1))*V5L
      elseif(ieorder.eq.-1)then
      V5L=
     & -l12
     & -l34
     & +l13
     & +l24
      zA43phiAQggmppm_unsym=zAphiq2gqmmpp(permm(1))*V5L
      elseif(ieorder.eq.0)then
c--- This is the same function as in the mpmm amplitude
      V5L=
     & -epinv**2-epinv*l12-0.5d0*l12**2
     & -epinv**2-epinv*l34-0.5d0*l34**2
     & +epinv**2+epinv*l13+0.5d0*l13**2
     & +epinv**2+epinv*l24+0.5d0*l24**2
c--- These are additional boxes
      V6L=
     . +Lsm1_2me(s341,s123,s13,mhsq)
     . +Lsm1_2me(s412,s234,s24,mhsq)
     . -Lsm1_2me(s412,s123,s12,mhsq)
     . -Lsm1_2me(s234,s341,s34,mhsq)
     . +Lsm1DS(s24,s41,s412)
     . +Lsm1DS(s13,s32,s123)
     . +Lsm1DS(s24,s23,s234)
     . +Lsm1DS(s13,s14,s341)

      zA43phiAQggmppm_unsym=
     .  zAphiq2gqmmpp(permm(1))
     .  *(V5L+V6L
     .    -Lsm1DS(s23,s34,s234)
     .    -Lsm1DS(s41,s12,s412))

     . +(za14**3/(za12*za34*za13)
     .  +za14**2/(za23*za13))
     .  *(Lsm1DS(s12,s23,s123)
     .   +Lsm1DS(s34,s41,s341))
      endif
      return
      end
************************************************************************
c--- The expression below corresponds to one half of A4;3:
c---   Aleft(a1,q2,g3-,g4+)+Aright(a1,q2,g3-,g4+)+Aleft(a1,g3-,q2,g4+)

c--- It is obtained by summing the expressions given in:
c---  L.~J.~Dixon and Y.~Sofianatos,
c---  %``Analytic one-loop amplitudes for a Higgs boson plus four partons,''
c---  arXiv:0906.0008 [hep-ph].
      double complex function zA43cphiAQggmpmp_unsym(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /order/ieorder
      common /koppel/ nf,nc,b0
      integer j,j1,j2,j3,j4,j5,j6,perm(6),permm(6)
      double complex za(5,5),zb(5,5)
      double precision s(5,5),epinv,deltar,musq
      double complex V5L,V6L,A0phiAQggmpmp,lnrat
      double complex l13,l34,l12,l24,Lsm1DS,Lsm1,Lsm1_2me
      double precision ss3,mhsq,s123,s234,s341,s412
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43
      integer ipset

      musq=renscale2
      epinv=0d0
      deltar=1d0
      czip=(0d0,0d0)
      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      permm(1)=perm(1)
      permm(2)=perm(4)
      permm(3)=perm(3)
      permm(4)=perm(2)
      permm(5)=perm(5)
      permm(6)=perm(6)

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za34 => kin(ipset)%zA(j3,j4)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za43 => kin(ipset)%zA(j4,j3)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb34 => kin(ipset)%zB(j3,j4)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb43 => kin(ipset)%zB(j4,j3)

      s123=ss3(j1,j2,j3,ipset)
      s234=ss3(j2,j3,j4,ipset)
      s341=ss3(j3,j4,j1,ipset)
      s412=ss3(j4,j1,j2,ipset)
      mhsq=s12+s13+s14+s23+s24+s34
      l12=lnrat(musq,-s12)
      l13=lnrat(musq,-s13)
      l24=lnrat(musq,-s24)
      l34=lnrat(musq,-s34)

      if(ieorder.eq.-2)then
      V5L=0d0
      zA43cphiAQggmpmp_unsym=zAphiq2gqpmpm(permm(1))*V5L
      elseif(ieorder.eq.-1)then
      V5L=
     & -l12
     & -l34
     & +l13
     & +l24
      zA43cphiAQggmpmp_unsym=zAphiq2gqpmpm(permm(1))*V5L
      elseif(ieorder.eq.0)then
c--- This is the same function as in the mpmm amplitude
      V5L=
     & -epinv**2-epinv*l12-0.5d0*l12**2
     & -epinv**2-epinv*l34-0.5d0*l34**2
     & +epinv**2+epinv*l13+0.5d0*l13**2
     & +epinv**2+epinv*l24+0.5d0*l24**2

c--- These are additional boxes
      V6L=
     . +Lsm1_2me(s341,s123,s13,mhsq)
     . +Lsm1_2me(s412,s234,s24,mhsq)
     . -Lsm1_2me(s412,s123,s12,mhsq)
     . -Lsm1_2me(s234,s341,s34,mhsq)
     . +Lsm1DS(s24,s41,s412)
     . +Lsm1DS(s13,s32,s123)
     . +Lsm1DS(s24,s23,s234)
     . +Lsm1DS(s13,s14,s341)

      zA43cphiAQggmpmp_unsym=
     .  zAphiq2gqpmpm(permm(1))
     .  *(V5L+V6L
     .   -Lsm1DS(s34,s41,s341)
     .   -Lsm1DS(s12,s23,s123))

     . +(zb14**2*zb23**3/(zb12*zb34*zb24**3)
     .  +zb12**2*zb34**2/(zb14*zb24**3))
     .  *(Lsm1DS(s23,s34,s234)
     .   +Lsm1DS(s41,s12,s412))
      endif
      return
      end
************************************************************************
c--- The expression below corresponds to one half of A4;3:
c---   Aleft(a1,q2,g3+,g4-)+Aright(a1,q2,g3+,g4-)+Aleft(a1,g3+,q2,g4-)

c--- It is obtained by summing the expressions given in:
c---  L.~J.~Dixon and Y.~Sofianatos,
c---  %``Analytic one-loop amplitudes for a Higgs boson plus four partons,''
c---  arXiv:0906.0008 [hep-ph].
      double complex function zA43cphiAQggmppm_unsym(perm,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /order/ieorder
      common /koppel/ nf,nc,b0
      integer j,j1,j2,j3,j4,j5,j6,perm(6),permm(6)
      double complex za(5,5),zb(5,5)
      double precision s(5,5),epinv,deltar,musq
      double complex V5L,V6l,A0phiAQggmppm,lnrat
      double complex l13,l34,l12,l24,Lsm1DS,Lsm1,Lsm1_2me
      double precision mhsq,s123,s234,s341,s412
      common /npars/npars,nspec,n
      real*8, pointer :: s12,s13,s14,s23,s24,s34,s21,s31,s41,s32,s42,s43
      complex*16, pointer :: za12,za13,za14,za23,za24,za34
      complex*16, pointer :: za21,za31,za41,za32,za42,za43
      complex*16, pointer :: zb12,zb13,zb14,zb23,zb24,zb34
      complex*16, pointer :: zb21,zb31,zb41,zb32,zb42,zb43
      integer ipset

      musq=renscale2
      epinv=0d0
      deltar=1d0
      czip=(0d0,0d0)
      j1=perm(1)
      j2=perm(2)
      j3=perm(3)
      j4=perm(4)
      j5=perm(5)
      j6=perm(6)

      permm(1)=perm(1)
      permm(2)=perm(4)
      permm(3)=perm(3)
      permm(4)=perm(2)
      permm(5)=perm(5)
      permm(6)=perm(6)

      ipset = nspec+4

      s12 => kin(ipset)%s(j1,j2)
      s13 => kin(ipset)%s(j1,j3)
      s14 => kin(ipset)%s(j1,j4)
      s23 => kin(ipset)%s(j2,j3)
      s24 => kin(ipset)%s(j2,j4)
      s34 => kin(ipset)%s(j3,j4)

      s21 => kin(ipset)%s(j2,j1)
      s31 => kin(ipset)%s(j3,j1)
      s41 => kin(ipset)%s(j4,j1)
      s32 => kin(ipset)%s(j3,j2)
      s42 => kin(ipset)%s(j4,j2)
      s43 => kin(ipset)%s(j4,j3)

      za12 => kin(ipset)%zA(j1,j2)
      za13 => kin(ipset)%zA(j1,j3)
      za14 => kin(ipset)%zA(j1,j4)
      za23 => kin(ipset)%zA(j2,j3)
      za24 => kin(ipset)%zA(j2,j4)
      za34 => kin(ipset)%zA(j3,j4)

      za21 => kin(ipset)%zA(j2,j1)
      za31 => kin(ipset)%zA(j3,j1)
      za41 => kin(ipset)%zA(j4,j1)
      za32 => kin(ipset)%zA(j3,j2)
      za42 => kin(ipset)%zA(j4,j2)
      za43 => kin(ipset)%zA(j4,j3)

      zb12 => kin(ipset)%zB(j1,j2)
      zb13 => kin(ipset)%zB(j1,j3)
      zb14 => kin(ipset)%zB(j1,j4)
      zb23 => kin(ipset)%zB(j2,j3)
      zb24 => kin(ipset)%zB(j2,j4)
      zb34 => kin(ipset)%zB(j3,j4)

      zb21 => kin(ipset)%zB(j2,j1)
      zb31 => kin(ipset)%zB(j3,j1)
      zb41 => kin(ipset)%zB(j4,j1)
      zb32 => kin(ipset)%zB(j3,j2)
      zb42 => kin(ipset)%zB(j4,j2)
      zb43 => kin(ipset)%zB(j4,j3)

      s123=ss3(j1,j2,j3,ipset)
      s234=ss3(j2,j3,j4,ipset)
      s341=ss3(j3,j4,j1,ipset)
      s412=ss3(j4,j1,j2,ipset)
      mhsq=s12+s13+s14+s23+s24+s34
      l12=lnrat(musq,-s12)
      l13=lnrat(musq,-s13)
      l24=lnrat(musq,-s24)
      l34=lnrat(musq,-s34)

      if(ieorder.eq.-2)then
      V5L=0d0
      zA43cphiAQggmppm_unsym=zAphiq2gqppmm(permm(1))*V5L
      elseif(ieorder.eq.-1)then
      V5L=
     & -l12
     & -l34
     & +l13
     & +l24
      zA43cphiAQggmppm_unsym=zAphiq2gqppmm(permm(1))*V5L
      elseif(ieorder.eq.0)then
c--- This is the same function as in the mpmm amplitude
      V5L=
     & -epinv**2-epinv*l12-0.5d0*l12**2
     & -epinv**2-epinv*l34-0.5d0*l34**2
     & +epinv**2+epinv*l13+0.5d0*l13**2
     & +epinv**2+epinv*l24+0.5d0*l24**2

c--- These are additional boxes
      V6L=
     . +Lsm1_2me(s341,s123,s13,mhsq)
     . +Lsm1_2me(s412,s234,s24,mhsq)
     . -Lsm1_2me(s412,s123,s12,mhsq)
     . -Lsm1_2me(s234,s341,s34,mhsq)
     . +Lsm1DS(s24,s41,s412)
     . +Lsm1DS(s13,s32,s123)
     . +Lsm1DS(s24,s23,s234)
     . +Lsm1DS(s13,s14,s341)

      zA43cphiAQggmppm_unsym=
     .  zAphiq2gqppmm(permm(1))
     .  *(V5L+V6L
     .    -Lsm1DS(s23,s34,s234)
     .    -Lsm1DS(s41,s12,s412))

     . +(zb14**3/(zb12*zb34*zb13)
     .  +zb14**2/(zb23*zb13))
     .  *(Lsm1DS(s12,s23,s123)
     .   +Lsm1DS(s34,s41,s341))
      endif
      return
      end





