***************************************************
      module B2gZ

      contains

********************************************************************
*
* Quark pair tree level helicity amplitude for
*
*     q + qb ->  g + g + e- + e+
*
*     q(1) g(2) g(3) qbar(4) e-(5) e+(6)
*
* One colour ordering and helicity, ignoring overall factors
*
* Returns a 2x2 array where each element denotes the combinations of
* the helicities of the gluons. The elements are given by
*
* (1,1) => gluon 2 left handed, gluon 3 left handed
* (1,2) => gluon 2 left handed, gluon 3 right handed
* (2,1) => gluon 2 right handed, gluon 3 left handed
* (2,2) => gluon 2 right handed, gluon 3 right handed

      function zampB2g0Z(i1,i2,i3,i4,i5,i6)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)
      complex*16 zampB2g0Z(2,2)

      s123 =  s6(i1,i2) + s6(i1,i3) + s6(i2,i3)
      s423 =  s6(i4,i2) + s6(i4,i3) + s6(i2,i3)
      s23  =  s6(i2,i3)
      s56  =  s6(i5,i6)

      za45 = zA6(i4,i5)
      za23 = zA6(i2,i3)
      za43 = zA6(i4,i3)
      za12 = zA6(i1,i2)
      za65 = zA6(i6,i5)

      zb42 = zB6(i4,i2)
      za35 = zA6(i3,i5)
      zb23 = zB6(i2,i3)
      za13 = zA6(i1,i3)
      zb16 = zB6(i1,i6)
      zb26 = zB6(i2,i6)
      zb16 = zB6(i1,i6)
      zb43 = zB6(i4,i3)
      zb12 = zB6(i1,i2)
      za45 = zA6(i4,i5)
      zb43 = zB6(i4,i3)

      zb36 = zB6(i3,i6)
      za52 = zA6(i5,i2)
      zb13 = zB6(i1,i3)
      za54 = zA6(i5,i4)
      za42 = zA6(i4,i2)
      za43 = zA6(i4,i3)
      zb16 = zB6(i1,i6)
      zb12 = zB6(i1,i2)

      zb65 = zB6(i6,i5)

      zampB2g0Z(1,1) = -zb16**2/(zb23*zb43*zb12*zb65)

      zb1 =  za12*zb16-za23*zb36
      zb2 =  za45*zb43-za52*zb23

      zx  = -zb13**2*za54*zb1/(zb12*s123*s23*s56)
      zy  =  za42**2*zb16*zb2/(za43*s423*s23*s56)
      zz  =  zb13*za42*zb16*za45/(za43*zb12*s23*s56)

      zampB2g0Z(1,2) = zx+zy+zz

      zb1 =  za45*zb42-za35*zb23
      zb2 =  za13*zb16+za23*zb26
      zx  =  za43*zb42*zb16*zb1/(zb43*s423*s23*s56)
      zy  =  za13*zb12*za45*zb2/(za12*s123*s23*s56)
      zz  =  zb1*zb2/(zb43*za12*s23*s56)

      zampB2g0Z(2,1) = zx+zy+zz

      zampB2g0Z(2,2) = -za45**2/(za23*za43*za12*za65)

c     write(*,*)"zampB2g0Z",zampB2g0Z

      return
      end function

********************************************************************
*
* Quark pair one-loop helicity amplitude for
*
*     q + qb ->  g + g + e- + e+
*
*     q(1) g(2) g(3) qbar(4) e-(5) e+(6)
*
* One colour ordering and helicity, ignoring overall factors
*
* Returns a 2x2 array where each element denotes the combinations of
* the helicities of the gluons. The elements are given by
*
* (1,1) => gluon 2 left handed, gluon 3 left handed
* (1,2) => gluon 2 left handed, gluon 3 right handed
* (2,1) => gluon 2 right handed, gluon 3 left handed
* (2,2) => gluon 2 right handed, gluon 3 right handed

      function zampB2g1Z(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)
      common /order/iorder
      complex*16 zampB2g1Z(2,2),zFsc(2,2),zFcc(2,2),zTree(2,2)

      if (iorder .ne. 0) then
         zFcc = (0d0,0d0)
         zFsc = (0d0,0d0)
      else
c     Different convention from MCFM amplitudes, fix at some point
          zFcc = zFccqggqb(i4,i2,i3,i1,i6,i5,renscale2)
          zFsc = zFscqggqb(i4,i2,i3,i1,i6,i5,renscale2)
      endif

      zTree = zampB2g0Z(i1,i2,i3,i4,i5,i6)

      zl12 = zzLogRat(renscale2,-s6(i1,i2))
      zl34 = zzLogRat(renscale2,-s6(i3,i4))
      zl23 = zzLogRat(renscale2,-s6(i2,i3))
      zl56 = zzLogRat(renscale2,-s6(i5,i6))

      if(iorder.eq.-2)then
         zIPoles = -3d0
      elseif(iorder.eq.-1)then
         zIPoles = -real(zl12+zl34+zl23)-1.5d0
      else
         zIPoles =-0.5d0*(zl12**2+zl34**2+zl23**2)-1.5d0*zl56-3.5d0
      endif

      zampB2g1Z=(zIPoles*zTree)+zFcc+zFsc

      return
      end function

********************************************************************
*
* Quark pair one-loop helicity amplitude for
*
*     q + qb ->  g + g + e- + e+
*
*     q(1) g(2) g(3) qbar(4) e-(5) e+(6)
*
* One colour ordering and helicity, ignoring overall factors
*
* Returns a 2x2 array where each element denotes the combinations of
* the helicities of the gluons. The elements are given by
*
* (1,1) => gluon 2 left handed, gluon 3 left handed
* (1,2) => gluon 2 left handed, gluon 3 right handed
* (2,1) => gluon 2 right handed, gluon 3 left handed
* (2,2) => gluon 2 right handed, gluon 3 right handed

      function zampBt2g1Z(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)
      common /order/iorder
      complex*16 zampBt2g1Z(2,2),zTree(2),ztFcc(2),ztFsc(2)

      do ii=1,2
         do jj=1,2
            zampBt2g1Z(ii,jj) = (0d0,0d0)
         enddo
      enddo

      call swapzAB_kin(6)

      if (iorder .ne. 0) then
         ztFcc = (0d0,0d0)
         ztFsc = (0d0,0d0)
      else
c     Different convention from MCFM amplitudes, fix at some point
         ztFsc = zFscqgqbg(i4,i2,i3,i1,i6,i5,renscale2)
         ztFcc = zFccqgqbg(i4,i2,i3,i1,i6,i5,renscale2)
      endif

      zTree = zTreeqgqbg(i4,i2,i3,i1,i6,i5)
      call swapzAB_kin(6)
      zl13 = zzLogRat(renscale2,-s6(i1,i3))
      zl34 = zzLogRat(renscale2,-s6(i3,i4))
      zl56 = zzLogRat(renscale2,-s6(i5,i6))

      if(iorder.eq.-2)then
         zIPoles = -2d0
      elseif(iorder.eq.-1)then
         zIPoles = -real(zl13+zl34)-1.5d0
      else
         zIPoles =-0.5d0*(zl13**2+zl34**2)-1.5d0*zl56-3.5d0
      endif

c      call swapzAB_kin(6)
c      z = zFscqpqp(i4,i2,i3,i1,i6,i5,renscale2)
c      call swapzAB_kin(6)
c      z = zFscqpqm(i1,i2,i3,i4,i5,i6,renscale2)
c      print *, abs(z)


      zampBt2g1Z(2,1) = (zIPoles*zTree(1))+ztFcc(1)+ztFsc(1)
      zampBt2g1Z(2,2) = (zIPoles*zTree(2))+ztFcc(2)+ztFsc(2)

      if (iorder .ne. 0) then
         ztFcc = (0d0,0d0)
         ztFsc = (0d0,0d0)
      else
c     Different convention from MCFM amplitudes, fix at some point
         ztFsc = zFscqgqbg(i1,i2,i3,i4,i5,i6,renscale2)
         ztFcc = zFccqgqbg(i1,i2,i3,i4,i5,i6,renscale2)
      endif

      zTree = zTreeqgqbg(i1,i2,i3,i4,i5,i6)

      zampBt2g1Z(1,2) = (zIPoles*zTree(1))+ztFcc(1)+ztFsc(1)
      zampBt2g1Z(1,1) = (zIPoles*zTree(2))+ztFcc(2)+ztFsc(2)


      return
      end function



********************************************************************
*
* Quark pair one-loop helicity amplitude for
*
*     q + qb ->  g + g + e- + e+
*
*     q(1) g(2) g(3) qbar(4) e-(5) e+(6)
*
* One colour ordering and helicity, ignoring overall factors
*
* Returns a 2x2 array where each element denotes the combinations of
* the helicities of the gluons. The elements are given by
*
* (1,1) => gluon 2 left handed, gluon 3 left handed
* (1,2) => gluon 2 left handed, gluon 3 right handed
* (2,1) => gluon 2 right handed, gluon 3 left handed
* (2,2) => gluon 2 right handed, gluon 3 right handed

      function zampBtt2g1Z(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)
      common /order/iorder
      complex*16 zampBtt2g1Z(2,2),zTree(2,2),zttFcc(2,2),zttFsc(2,2)

      if (iorder .ne. 0) then
         zttFsc = (0d0,0d0)
         zttFcc = (0d0,0d0)
      else
         zttFsc = zFscqqbgg(i1,i2,i3,i4,i5,i6,renscale2)
         zttFcc = zFccqqbgg(i1,i2,i3,i4,i5,i6,renscale2)
      endif

      zTree = zampB2g0Z(i1,i2,i3,i4,i5,i6)
      zl14 = zzLogRat(renscale2,-s6(i1,i4))
      zl56 = zzLogRat(renscale2,-s6(i5,i6))

      if(iorder.eq.-2)then
         zIPoles = -1d0
      elseif(iorder.eq.-1)then
         zIPoles = -real(zl14)-1.5d0
      else
         zIPoles =-0.5d0*(zl14**2)-1.5d0*zl56-3.5d0
      endif

      zampBtt2g1Z = (zIPoles*zTree)+zttFsc+zttFcc

      return
      end function

********************************************************************
*
* Quark pair one-loop helicity amplitude for
*
*     q + qb ->  g + g + e- + e+
*
*     q(1) g(2) g(3) qbar(4) e-(5) e+(6)
*
* One colour ordering and helicity, ignoring overall factors
*
* Returns a 2x2 array where each element denotes the combinations of
* the helicities of the gluons. The elements are given by
*
* (1,1) => gluon 2 left handed, gluon 3 left handed
* (1,2) => gluon 2 left handed, gluon 3 right handed
* (2,1) => gluon 2 right handed, gluon 3 left handed
* (2,2) => gluon 2 right handed, gluon 3 right handed

      function zampB2g1Zvec(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)
      common /order/iorder
      complex*16 zFvf(2,2),zFvs(2,2)
      complex*16 zampB2g1Zvec(2,2)

      if (iorder .ne. 0) then
         zFvf = (0d0,0d0)
         zFvs = (0d0,0d0)
      else
         zFvf = zFvfqqbgg(i1,i2,i3,i4,i5,i6,renscale2)
         zFvs = zFvsqqbgg(i1,i2,i3,i4,i5,i6,renscale2)
      endif

c      zampB2g1Zvec = -dconjg(zFvf+zFvs)
      zampB2g1Zvec =-(zFvf+zFvs)

      return
      end function


******************************************************************
* A special set of tree level primitive tree level amplitudes
*
* q(i1) + qb(i4) -> g(i2) + g(i3) +  e-(i5) +  e+(i6)
*
* For ordering q,g,qb,g (NOTE: THIS IS NOT COLOUR ORDERING)
*
* (1) => g+g-
* (2) => g+g+
*


      function zTreeqgqbg(i1,i2,i3,i4,i5,i6)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)
      complex*16 zTreeqgqbg(2)

      t(i1,i2,i3) = s6(i1,i2) + s6(i2,i3) + s6(i1,i3)

      za14 = zA6(i1,i4)
      za45 = zA6(i4,i5)
      za13 = zA6(i1,i3)
      za12 = zA6(i1,i2)
      za34 = zA6(i3,i4)
      za42 = zA6(i4,i2)
      za56 = zA6(i5,i6)

      zb14 = zB6(i1,i4)
      za25 = zA6(i2,i5)
      zb12 = zB6(i1,i2)
      zb16 = zB6(i1,i6)
      zb36 = zB6(i3,i6)
      s56  = s6(i5,i6)
      zb42 = zB6(i4,i2)
      zb13 = zB6(i1,i3)
      za32 = zA6(i3,i2)
      zb34 = zB6(i3,i4)
      zb32 = zB6(i3,i2)

      zTreeqgqbg(2)=
     .(-za14*za45**2)/
     .(za13*za12*za34*za42*za56)

      zTreeqgqbg(1)=
     .(((za45*zb14+za25*zb12)*
     .(-(za14*zb16)-za34*zb36))/
     .(s56*za13*za34*zb12*zb42)+
     .(za45*zb13*(-(za12*zb16)-za32*zb36))/
     .(s56*za13*zb12*t(i1,i3,i2))-
     .(za42*zb16*(za45*zb34+za25*zb32))/
     .(s56*za34*zb42*t(i3,i4,i2)))


      return
      end function

******************************************************************
* Generates the Fcc finite helicity amplitude contributions for the process
*
* q(i1) + qb(i4) -> g(i2) + g(i3) +  e-(i5) +  e+(i6)
*
* For primitive amplitude ordering qgqbg
*
* (2) => g+g+
* (1) => g+g-
*

      function zFccqgqbg(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)
      complex*16 zFccqgqbg(2)

      do i=1,2
         zFccqgqbg(i) = 0d0
      enddo

      zFccqgqbg(2) = zFccqpqp(i1,i2,i3,i4,i5,i6,renscale2)
      zFccqgqbg(1) = zFccqpqm(i1,i2,i3,i4,i5,i6,renscale2)

      return
      end function

******************************************************************
* Generates the Fsc finite helicity amplitude contributions for the process
*
* q(i1) + qb(i4) -> g(i2) + g(i3) +  e-(i5) +  e+(i6)
*
*
* For primitive amplitude ordering qgqbg
*
* (2) => g+g+
* (1) => g+g-
*

      function zFscqgqbg(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)
      complex*16 zFscqgqbg(2)

      do i=1,2
         zFscqgqbg(i) = 0d0
      enddo

      zFscqgqbg(2) = zFscqpqp(i1,i2,i3,i4,i5,i6,renscale2)
      zFscqgqbg(1) = zFscqpqm(i1,i2,i3,i4,i5,i6,renscale2)

      return
      end function



******************************************************************
* Generates the Fcc finite helicity amplitude contributions for the process
*
* q(i1) + qb(i4) -> g(i2) + g(i3) +  e-(i5) +  e+(i6)
*
* At one loop for the qggqb colour ordering. Returns a 2x2 array with the following key
*
* (1,1) => g-g-
* (1,2) => g-g+
* (2,1) => g+g-
* (2,2) => g+g+-
*

      function zFccqggqb(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)
      complex*16 zFccqggqb(2,2)


      t(i1,i2,i3) = s6(i1,i2) + s6(i2,i3) + s6(i1,i3)

      za46 = zA6(i4,i6)
      zb43 = zB6(i4,i3)
      za42 = zA6(i4,i2)
      zb23 = zB6(i2,i3)
      za41 = zA6(i4,i1)
      zb31 = zB6(i3,i1)
      za26 = zA6(i2,i6)
      za16 = zA6(i1,i6)
      za65 = zA6(i6,i5)
      za43 = zA6(i4,i3)
      zb21 = zB6(i2,i1)
      za21 = zA6(i2,i1)
      zb41 = zB6(i4,i1)
      za23 = zA6(i2,i3)
      zb45 = zB6(i4,i5)
      zb35 = zB6(i3,i5)
      zb15 = zB6(i1,i5)
      zb65 = zB6(i6,i5)
      zb42 = zB6(i4,i2)
      za31 = zA6(i3,i1)
      zb25 = zB6(i2,i5)
      za36 = zA6(i3,i6)
      s65  = s6(i6,i5)
      s31  = s6(i3,i1)
      s42  = s6(i4,i2)
      s23  = s6(i2,i3)
      s41  = s6(i4,i1)
c ************
      s43  = s6(i4,i3)
      s21  = s6(i2,i1)
      zb32 = zB6(i3,i2)
      za32 = zA6(i3,i2)
      s32  = s6(i3,i2)
      za46 = zA6(i4,i6)
c ***************
      s13  = s6(i1,i3)
      s24  = s6(i2,i4)
      s56  = s6(i5,i6)
      zb13 = zB6(i1,i3)
      zb24 = zB6(i2,i4)
      zb56 = zB6(i5,i6)
      zb34 = zB6(i3,i4)
      za13 = zA6(i1,i3)
      za12 = zA6(i1,i2)

      zFccqggqb(1,2)=
     .(2d0*zL0d(-t(i2,i3,i1),-s65)*za46*zb43*
     .(za42*zb23-za41*zb31)*
     .(-(za26*zb23)+za16*zb31))/
     .(s65*za65*zb23*
     .(-(za43*zb23)-za41*zb21)*
     .(za42*zb21+za43*zb31))-
     .(2d0*zL0d(-t(i4,i2,i3),-s65)*za21*
     .(-(za42*zb41)+za23*zb31)*
     .(-(za42*zb45)+za23*zb35)*zb15)/
     .(s65*za23*(-(za43*zb41)-za23*zb21)*
     .(za42*zb21+za43*zb31)*zb65)+
     .zLsm1(-s42,-t(i4,i2,i3),-s23,-t(i4,i2,i3))*
     .((za16**2*zb43**3)/
     .(za65*zb42*(za21*zb42+za31*zb43)*
     .zb23*t(i4,i2,i3))+
     .(za42**3*(-(za43*zb45)-za23*zb25)**2)/
     .(za43**3*za23*(za42*zb21+za43*zb31)*
     .zb65*t(i4,i2,i3))-
     .(za42*za23*(za42*zb25+za43*zb35)**2)/
     .(za43**3*(za42*zb21+za43*zb31)*zb65*
     .t(i4,i2,i3)))+(2d0*za42*zb43*
     .(-(za42*zb45)+za23*zb35)*
     .((zL0d(-t(i4,i2,i3),-s42)*
     .(-(za43*zb45)-za23*zb25))/
     .(s42*(-(za43*zb41)-za23*zb21))+
     .(zL0d(-t(i4,i2,i3),-s23)*
     .(za42*zb25+za43*zb35))/
     .(s23*(za42*zb21+za43*zb31))))/
     .(za43*zb65*t(i4,i2,i3))

      zFccqggqb(1,2)=zFccqggqb(1,2)+
     .zLsm1_2mh(s31,t(i4,i2,i3),s42,s65)*
     .((za16**2*zb43**3)/
     .(za65*zb42*(za21*zb42+za31*zb43)*
     .zb23*t(i4,i2,i3))+
     .((-(za43*zb45)-za23*zb25)**2*
     .(-(za42*zb41)+za23*zb31)**3)/
     .(za23*(-(za43*zb41)-za23*zb21)**3*
     .(za42*zb21+za43*zb31)*zb65*t(i4,i2,i3))-
     .(za23*(-(za42*zb41)+za23*zb31)*zb15**2*t(i4,i2,i3))/
     .((-(za43*zb41)-za23*zb21)**3*
     .(za42*zb21+za43*zb31)*zb65))+
     .(zI3m(s42,s31,s65)*zb43*
     .(s65*za21*(2d0*za42*za36*zb25*zb31+
     .(-s42-s31+s65)*za46*zb15)+
     .2d0*za42*za46*za16*
     .((-s42-s31+s65)*zb41-
     .2d0*za23*zb42*zb31)*zb65-
     .2d0*za42*za26*zb25*
     .(s65*(-s42-s31+s65)+
     .(s42-s31-s65)*t(i4,i2,i3))))/
     .((s42**2-2d0*s42*s31+s31**2-2d0*s42*s65-
     .2d0*s31*s65+s65**2)*
     .(-(za43*zb23)-za41*zb21)*
     .(-(za43*zb41)-za23*zb21))+
     .(2d0*zzlogRat(-s42,-s65)*(-2d0*(s41-s23)*za42*zb43*
     .(-(za46*zb45)-za26*zb25)-
     .(za21*zb45*((s41-s23)*
     .((s42-s31-s65)*za42*zb25-
     .(-s42-s31+s65)*za46*zb65)+
     .(-(za43*zb23)-za41*zb21)*
     .(-((s42-s31-s65)*za42*zb45)-
     .(-s42-s31+s65)*za26*zb65)))/
     .(za31*zb65)+
     .(zb43*(-(za42*
     .(za36*zb43+za16*zb41))-
     .za23*(-(za46*zb43)-za26*zb23)-
     .za21*(-(za46*zb41)-za26*zb21))*
     .(-(za42*za65*zb25)+za46*t(i4,i2,i1)))/
     .za65))/
     .((s42**2-2d0*s42*s31+s31**2-2d0*s42*s65-
     .2d0*s31*s65+s65**2)*
     .(-(za43*zb23)-za41*zb21)*
     .(-(za43*zb41)-za23*zb21))

      zFccqggqb(1,2)=zFccqggqb(1,2)+
     .(2d0*zzlogRat(-s31,-s65)*(-2d0*(s41-s23)*za21*zb31*
     .(-(za36*zb35)-za16*zb15)+
     .(za16*zb43*((-(za43*zb41)-
     .za23*zb21)*
     .((-s42+s31-s65)*za16*zb31+
     .(-s42-s31+s65)*za65*zb35)+
     .(s41-s23)*
     .(-((-s42+s31-s65)*za36*zb31)+
     .(-s42-s31+s65)*za65*zb15)))/
     .(za65*zb42)+
     .(za21*((-(za41*zb45)-za21*zb25)*
     .zb31+zb43*
     .(za43*zb35+za41*zb15)+
     .zb23*(za23*zb35+za21*zb15))*
     .(-(za36*zb31*zb65)+zb15*t(i4,i3,i1)))/
     .zb65))/
     .((s42**2-2d0*s42*s31+s31**2-2d0*s42*s65-
     .2d0*s31*s65+s65**2)*
     .(-(za43*zb23)-za41*zb21)*
     .(-(za43*zb41)-za23*zb21))+
     .zLsm1(-s31,-t(i2,i3,i1),-s23,-t(i2,i3,i1))*
     .(((za36*zb23+za16*zb21)**2*zb31**3)/
     .(za65*zb23*zb21**3*
     .(za42*zb21+za43*zb31)*t(i2,i3,i1))-
     .(zb23*zb31*(-(za26*zb21)-za36*zb31)**2)/(za65*zb21**3*
     .(za42*zb21+za43*zb31)*t(i2,i3,i1))+
     .(za21**3*zb45**2)/
     .(za23*za31*(za21*zb42+za31*zb43)*
     .zb65*t(i2,i3,i1)))+
     .(2d0*za21*zb31*(-(za26*zb23)+za16*zb31)*
     .((zL0d(-t(i2,i3,i1),-s31)*
     .(za36*zb23+za16*zb21))/
     .(s31*(-(za43*zb23)-za41*zb21))+
     .(zL0d(-t(i2,i3,i1),-s23)*
     .(-(za26*zb21)-za36*zb31))/
     .(s23*(za42*zb21+za43*zb31))))/
     .(za65*zb21*t(i2,i3,i1))

      zFccqggqb(1,2)=zFccqggqb(1,2)+
     .zLsm1_2mh(s42,t(i2,i3,i1),s31,s65)*
     .(((za36*zb23+za16*zb21)**2*
     .(za42*zb23-za41*zb31)**3)/
     .(za65*zb23*(-(za43*zb23)-za41*zb21)**3d0*(za42*zb21+za43*zb31)*t(i2,i3,i1))+
     .(za21**3*zb45**2)/
     .(za23*za31*(za21*zb42+za31*zb43)*
     .zb65*t(i2,i3,i1))-
     .(za46**2*zb23*(za42*zb23-za41*zb31)*
     .t(i2,i3,i1))/
     .(za65*(-(za43*zb23)-za41*zb21)**3*
     .(za42*zb21+za43*zb31)))+
     .zI3m(s31,s42,s65)*
     .(((za46*za16*zb43*
     .(-(za42*zb42)-za43*zb43-
     .za41*zb41)*
     .(-(za43*zb41)-za23*zb21)*zb31)/
     .(za65*zb42)-
     .(s41*(-s42-s31+s65)*za16*zb45*
     .(za42*zb21+za43*zb31))/
     .(2d0*za31*zb42)+
     .(za46*za26*zb43*
     .(-(za43*zb41)-za23*zb21)*
     .(za42*zb23-za41*zb31))/za65+
     .za42*za43*za16*zb41*zb31*zb35+
     .((s41-s23)*za42*zb31*
     .(-(za26*zb25)-za36*zb35))/2+
     .(za42*za46*za16*zb41**2*zb23*zb65)/
     .zb42+za42*zb23*zb15*
     .(za42*(za26*zb42+za16*zb41)+
     .za26*t(i4,i2,i3)))/
     .(2d0*(-(za43*zb23)-za41*zb21)*
     .(-(za43*zb41)-za23*zb21)*
     .(za42*zb21+za43*zb31))-
     .(2d0*za46*za21*zb45*zb31*
     .(za42*zb23-za41*zb31))/
     .((-(za43*zb23)-za41*zb21)*
     .(za42*zb21+za43*zb31)*t(i2,i3,i1))-
     .((((za42*zb23-za41*zb31)*
     .(-(za26*zb23)+za16*zb31)**2)/
     .(za65*zb23*
     .(-(za43*zb23)-za41*zb21)*
     .(za42*zb21+za43*zb31))+
     .(za21**3*zb45**2)/
     .(za23*za31*
     .(za21*zb42+za31*zb43)*zb65))*
     .(2d0*s31*s65+(s42-s31-s65)*t(i2,i3,i1)))/
     .(2d0*t(i2,i3,i1)**2))

      zFccqggqb(1,2)=zFccqggqb(1,2)
     .-(zI3m(s31,s42,s65)*za21*
     .(2d0*za65*zb45*(-((-s42-s31+s65)*
     .za41)+2d0*za42*za31*zb23)*zb31*
     .zb15-s65*zb43*
     .(2d0*za42*za36*zb25*zb31+
     .(-s42-s31+s65)*za46*zb15)+
     .2d0*za36*zb31*zb35*
     .(s65*(-s42-s31+s65)+
     .(-s42+s31-s65)*t(i2,i3,i1))))/
     .((s42**2-2d0*s42*s31+s31**2-2d0*s42*s65-
     .2d0*s31*s65+s65**2)*
     .(-(za43*zb23)-za41*zb21)*
     .(-(za43*zb41)-za23*zb21))+
     .zI3m(s42,s31,s65)*
     .((-2d0*za42*za16*zb43*
     .(-(za42*zb41)+za23*zb31)*zb15)/
     .((-(za43*zb41)-za23*zb21)*
     .(za42*zb21+za43*zb31)*t(i4,i2,i3))-
     .(((za16**2*zb43**3)/
     .(za65*zb42*
     .(za21*zb42+za31*zb43)*zb23)+
     .((-(za42*zb41)+za23*zb31)*
     .(-(za42*zb45)+za23*zb35)**2)/
     .(za23*(-(za43*zb41)-za23*zb21)*
     .(za42*zb21+za43*zb31)*zb65))*
     .(2d0*s42*s65+(-s42+s31-s65)*t(i4,i2,i3)))/
     .(2d0*t(i4,i2,i3)**2)+(za42*za41*za26*zb45*
     .zb21*zb31-
     .(s41*(-s42-s31+s65)*za16*zb45*
     .(za42*zb21+za43*zb31))/
     .(2d0*za31*zb42)+
     .((s41-s23)*za42*zb31*
     .(-(za26*zb25)-za36*zb35))/2+
     .(za41**2*za23*za65*zb45*zb31*zb15)/
     .za31+(za42*za21*zb45*
     .(-(za43*zb23)-za41*zb21)*
     .(-(za41*zb41)-za21*zb21-
     .za31*zb31)*zb15)/(za31*zb65)+
     .(za21*(-(za43*zb23)-za41*zb21)*
     .(-(za42*zb41)+za23*zb31)*zb35*
     .zb15)/zb65+
     .za46*za23*zb31*
     .(-(zb31*(-(za41*zb45)-za31*zb35))+
     .zb35*t(i2,i3,i1)))/
     .(2d0*(-(za43*zb23)-za41*zb21)*
     .(-(za43*zb41)-za23*zb21)*
     .(za42*zb21+za43*zb31)))



      zFccqggqb(2,1)=
     .(zI3m(s42,s31,s65)*zb42*
     .(-(s42*s31*za16**2)+
     .(-(za41*(za36*zb43+za16*zb41))-
     .za21*(za36*zb23+za16*zb21))**2))/
     .(2d0*za42*za65*(za21*zb42+za31*zb43)*
     .(-(za41*zb43)-za21*zb23)*zb31)+
     .(zI3m(s31,s42,s65)*za31*
     .(-(s42*s31*zb45**2)+
     .(zb43*(-(za43*zb45)-za23*zb25)+
     .zb41*(-(za41*zb45)-za21*zb25))**2))/
     .(2d0*za42*(za21*zb42+za31*zb43)*
     .(-(za23*zb43)-za21*zb41)*zb31*zb65)-
     .(2d0*za43*(-(za43*zb45)-za23*zb25)*
     .((zL0d(-s23,-t(i4,i2,i3))*
     .(za42*zb42*zb25+za43*zb42*zb35)
     .)/t(i4,i2,i3)**2+(zL0d(-s65,-t(i4,i2,i3))*za31*zb15)/
     .(za23*t(i4,i2,i3))))/
     .(za42*(za42*zb21+za43*zb31)*zb65)+
     .zLsm1(-s42,-t(i4,i2,i3),-s23,-t(i4,i2,i3))*
     .((za16**2*zb42**3)/
     .(za65*zb43*(za21*zb42+za31*zb43)*
     .zb23*t(i4,i2,i3))+
     .(za43*(-(za43*zb45)-za23*zb25)**2)/
     .(za42*za23*(za42*zb21+za43*zb31)*
     .zb65*t(i4,i2,i3)))+
     .zLsm1_2mht(s31,t(i4,i2,i3),s65,s42)*
     .((za16**2*zb42**2*(-(za41*zb42)+
     .za31*zb23))/
     .(za65*(za21*zb42+za31*zb43)*zb23*
     .(-(za41*zb43)-za21*zb23)*t(i4,i2,i3))+
     .(za43*(-(za43*zb45)-za23*zb25)**2)/
     .(za42*za23*(za42*zb21+za43*zb31)*
     .zb65*t(i4,i2,i3)))+
     .(2d0*zb21*(za36*zb23+za16*zb21)*
     .((zL0d(-s23,-t(i2,i3,i1))*
     .(za26*za31*zb21+za31*za36*zb31)
     .)/t(i2,i3,i1)**2+(zL0d(-s65,-t(i2,i3,i1))*za46*zb42)/
     .(zb23*t(i2,i3,i1))))/
     .(za65*zb31*(za42*zb21+za43*zb31))+
     .zLsm1(-s31,-t(i2,i3,i1),-s23,-t(i2,i3,i1))*
     .((zb21*(za36*zb23+za16*zb21)**2)/
     .(za65*zb23*zb31*
     .(za42*zb21+za43*zb31)*t(i2,i3,i1))+
     .(za31**3*zb45**2)/
     .(za23*za21*(za21*zb42+za31*zb43)*
     .zb65*t(i2,i3,i1)))+
     .zLsm1_2mht(s42,t(i2,i3,i1),s65,s31)*
     .((zb21*(za36*zb23+za16*zb21)**2)/
     .(za65*zb23*zb31*
     .(za42*zb21+za43*zb31)*t(i2,i3,i1))+
     .(za31**2*(za23*zb42-za31*zb41)*zb45**2)/
     .(za23*(za21*zb42+za31*zb43)*
     .(-(za23*zb43)-za21*zb41)*zb65*t(i2,i3,i1)))

      zFccqggqb(2,2) = -(((-zLsm1(-s42,-t(i4,i2,i3),-s23,-t(i4,i2,i3))-
     .zLsm1(-s23,-t(i2,i3,i1),-s31,-t(i2,i3,i1))-
     .zLsm1_2me(t(i4,i2,i3),t(i2,i3,i1),s23,s65))*za16**2)/
     .(za42*za23*za31*za65))-
     .(2d0*zL0d(-t(i2,i3,i1),-s31)*za26*za16*zb23)/
     .(s31*za42*za23*za65)-
     .(2d0*zL0d(-s65,-t(i2,i3,i1))*za46*za16*
     .(za21*zb42+za31*zb43))/
     .(za42*za23*za31*za65*t(i2,i3,i1))

      zFccqggqb(1,1) = -(((-zLsm1(-s13,-t(i1,i3,i2),-s32,-t(i1,i3,i2))-
     .zLsm1(-s32,-t(i3,i2,i4),-s24,-t(i3,i2,i4))-
     .zLsm1_2me(t(i1,i3,i2),t(i3,i2,i4),s32,s56))*zb45**2)
     ./
     .(zb13*zb32*zb24*zb56))-
     .(2d0*zL0d(-t(i3,i2,i4),-s24)*zb35*zb45*za32)/
     .(s24*zb13*zb32*zb56)-
     .(2d0*zL0d(-s56,-t(i3,i2,i4))*zb15*zb45*
     .(zb34*za13+zb24*za12))/
     .(zb13*zb32*zb24*zb56*t(i3,i2,i4))



      return
      end function

******************************************************************
* Generates the Fsc finite helicity amplitude contributions for the process
*
* q(i1) + qb(i4) -> g(i2) + g(i3) +  e-(i5) +  e+(i6)
*
* At one loop for the qggqb colour ordering. Returns a 2x2 array with the following key
*
* (1,1) => g-g-
* (1,2) => g-g+
* (2,1) => g+g-
* (2,2) => g+g+-
*


      function zFscqggqb(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)
      complex*16 zFscqggqb(2,2)

      t(i1,i2,i3) = s6(i1,i2) + s6(i2,i3) + s6(i1,i3)

      za46 = zA6(i4,i6)
      za23 = zA6(i2,i3)
      zb43 = zB6(i4,i3)
      za21 = zA6(i2,i1)
      zb41 = zB6(i4,i1)
      za41 = zA6(i4,i1)
      zb23 = zB6(i2,i3)
      s42  = s6(i4,i2)
      s31  = s6(i3,i1)
      s65  = s6(i6,i5)
      zb42 = zB6(i4,i2)
      za65 = zA6(i6,i5)
      zb25 = zB6(i2,i5)
      za31 = zA6(i3,i1)
      za43 = zA6(i4,i3)
      zb21 = zB6(i2,i1)
      za42 = zA6(i4,i2)
      zb31 = zB6(i3,i1)
      zb15 = zB6(i1,i5)
      za36 = zA6(i3,i6)
      zb65 = zB6(i6,i5)
      zb45 = zB6(i4,i5)
      zb35 = zB6(i3,i5)
      s23  = s6(i2,i3)
      za16 = zA6(i1,i6)
      za26 = zA6(i2,i6)
      za26 = zA6(i2,i6)
      za32 = zA6(i3,i2)
      zb32 = zB6(i3,i2)
      s32  = s6(i3,i2)
c     ***************
      s56  = s6(i5,i6)
      zb34 = zB6(i3,i4)
      za13 = zA6(i1,i3)
      zb24 = zB6(i2,i4)
      za12 = zA6(i1,i2)
      s24  = s6(i2,i4)
      zb13 = zB6(i1,i3)
      zb56 = zB6(i5,i6)
      zb12 = zB6(i1,i2)
      za24 = zA6(i2,i4)
      za56 = zA6(i5,i6)


      zFscqggqb(1,2)=
     .-(za46*(-(za23*zb43)-za21*zb41)*
     .(-(za41*zb43)-za21*zb23)*
     .((s42-s31-s65)*za46*zb42-
     .(-s42-s31+s65)*za65*zb25))/
     .(2d0*(s42**2-2d0*s42*s31+s31**2-2d0*s42*s65-
     .2d0*s31*s65+s65**2)*za31*za65*zb42*
     .(-(za43*zb23)-za41*zb21)*
     .(za42*zb21+za43*zb31))-
     .(za46*zb43*(-(za46*
     .(za21*zb42+za31*zb43))-
     .za21*za65*zb25))/
     .(2d0*za31*za65*zb42*
     .(-(za43*zb23)-za41*zb21)*
     .(za42*zb21+za43*zb31))+
     .((-(za23*zb43)-za21*zb41)*
     .(-(za41*zb43)-za21*zb23)*zb15*
     .(-((-s42+s31-s65)*za31*zb15)+
     .(-s42-s31+s65)*za36*zb65))/
     .(2d0*(s42**2-2d0*s42*s31+s31**2-2d0*s42*s65-
     .2d0*s31*s65+s65**2)*za31*zb42*
     .(-(za43*zb41)-za23*zb21)*
     .(za42*zb21+za43*zb31)*zb65)-
     .(za21*zb15*(-((za21*zb42+za31*zb43)*
     .zb15)-za36*zb43*zb65))/
     .(2d0*za31*zb42*(-(za43*zb41)-za23*zb21)*
     .(za42*zb21+za43*zb31)*zb65)+
     .(zL1d(-t(i4,i2,i3),-s42)*za42**2*zb43**2*
     .(-(za43*zb45)-za23*zb25)**2)/
     .(2d0*s42**2*za43*(-(za43*zb41)-za23*zb21)*
     .zb65*t(i4,i2,i3))-(zb43**2*
     .(za42*zb25+za43*zb35)**2)/
     .(2d0*za43*zb42*zb23*
     .(za42*zb21+za43*zb31)*zb65*t(i4,i2,i3))+
     .(za42*za23*zb43**2*
     .(za42*zb25+za43*zb35)**2*
     .(-zL1d(-t(i4,i2,i3),-s23)/(2d0*s23**2)+
     .zLs1d(-s42,-t(i4,i2,i3),-s23,-t(i4,i2,i3))/t(i4,i2,i3)**2))/
     .(za43*(za42*zb21+za43*zb31)*zb65*
     .t(i4,i2,i3))+(zL0d(-t(i4,i2,i3),-s42)*za42*za23*zb43*
     .zb15**2*t(i4,i2,i3))/
     .(s42*(-(za43*zb41)-za23*zb21)**2*
     .(za42*zb21+za43*zb31)*zb65)+
     .(zLsm1_2mh(s31,t(i4,i2,i3),s42,s65)*za23*
     .(-(za42*zb41)+za23*zb31)*zb15**2*t(i4,i2,i3))/
     .((-(za43*zb41)-za23*zb21)**3*
     .(za42*zb21+za43*zb31)*zb65)+
     .(za21*(-(za42*zb41)+za23*zb31)*zb15**2*
     .(-(zL1d(-s65,-t(i4,i2,i3))*za21)/(2d0*za23*t(i4,i2,i3)**2)+
     .zL0d(-s65,-t(i4,i2,i3))/
     .((-(za43*zb41)-za23*zb21)*t(i4,i2,i3)))*
     .t(i4,i2,i3))/
     .((-(za43*zb41)-za23*zb21)*
     .(za42*zb21+za43*zb31)*zb65)+
     .(zzlogRat(-s31,-s65)*zb15*
     .((-3d0*za42*(-(za23*zb43)-za21*zb41)*
     .(-(za41*zb43)-za21*zb23)*zb31*
     .((s42-s31-s65)*za36+
     .2d0*za31*za65*zb15))/
     .(s42**2-2d0*s42*s31+s31**2-2d0*s42*s65-
     .2d0*s31*s65+s65**2)**2-
     .(za21*zb43*zb15)/(2d0*zb42*zb65)-
     .((-(za23*zb43)-za21*zb41)*
     .(-(za41*zb43)-za21*zb23)*
     .(-((-(za43*zb45)-za23*zb25)*
     .zb31)+(-3d0*s42-s31+s65)*zb15)
     .+(-s42+s31-s65)*za21*zb31*
     .(-((za21*zb42+za31*zb43)*zb15)-
     .za36*zb43*zb65)-
     .2d0*zb15*t(i4,i2,i3)*
     .(2d0*(-(za23*zb43)+za21*zb41)*
     .(-(za41*zb43)-za21*zb23)+
     .(za21*zb43+
     .(za23*zb41*
     .(-(za41*zb43)-za21*zb23))/
     .(-(za43*zb41)-za23*zb21))*
     .(t(i4,i2,i3)-t(i4,i2,i1))))/
     .(2d0*(s42**2-2d0*s42*s31+s31**2-
     .2d0*s42*s65-2d0*s31*s65+s65**2)*zb42*
     .zb65)))/
     .((-(za43*zb41)-za23*zb21)*
     .(za42*zb21+za43*zb31))

      zFscqggqb(1,2)=zFscqggqb(1,2)
     .+(zzlogRat(-s42,-s65)*za42*zb15*
     .((3d0*(-(za23*zb43)-za21*zb41)*
     .(-(za41*zb43)-za21*zb23)*
     .(-((-s42+s31-s65)*za36*zb31)+
     .(-s42-s31+s65)*za65*zb15))/
     .((s42**2-2d0*s42*s31+s31**2-
     .2d0*s42*s65-2d0*s31*s65+s65**2)**2*
     .(-(za43*zb41)-za23*zb21))+
     .((-(za23*zb43)-za21*zb41)*zb15)/
     .(2d0*(-(za43*zb41)-za23*zb21)**2*zb65)+
     .((-(za23*zb43)-za21*zb41)*
     .(-(za41*zb43)-za21*zb23)*
     .(za31*zb15-za36*zb65)-
     .(s42-s31-s65)*za21*
     .(-((za21*zb42+za31*zb43)*zb15)-
     .za36*zb43*zb65)-
     .((-s42+s31-s65)*za31*
     .(-(za23*zb43)-za21*zb41)*zb15*
     .(t(i4,i2,i3)-t(i4,i2,i1)))/
     .(-(za43*zb41)-za23*zb21))/
     .(2d0*(s42**2-2d0*s42*s31+s31**2-
     .2d0*s42*s65-2d0*s31*s65+s65**2)*za31*
     .(-(za43*zb41)-za23*zb21)*zb65)))/
     .(za42*zb21+za43*zb31)


      zFscqggqb(1,2)=zFscqggqb(1,2)
     .+(zI3m(s42,s31,s65)*za42*zb15*
     .(-(za21*za65*zb43*zb35)+
     .((-(za41*zb43)-za21*zb23)*
     .(za36*(-(za23*zb43)-za21*zb41)*
     .zb31+3d0*za23*za65*zb43*zb15))/
     .(-(za43*zb41)-za23*zb21)+
     .(3d0*za65*(-(za23*zb43)-za21*zb41)*
     .(-(za41*zb43)-za21*zb23)*zb31*
     .(-((-s42+s31-s65)*za31*zb15)+
     .(-s42-s31+s65)*za36*zb65))/
     .((s42**2-2d0*s42*s31+s31**2-
     .2d0*s42*s65-2d0*s31*s65+s65**2)*
     .(-(za43*zb41)-za23*zb21))-
     .(za23*za65*zb41*
     .(-(za41*zb43)-za21*zb23)*zb15*
     .(t(i4,i2,i3)-t(i4,i2,i1)))/
     .(-(za43*zb41)-za23*zb21)**2))/
     .((s42**2-2d0*s42*s31+s31**2-2d0*s42*s65-
     .2d0*s31*s65+s65**2)*
     .(za42*zb21+za43*zb31))


      zFscqggqb(1,2)=zFscqggqb(1,2)
     .+(zL1d(-t(i2,i3,i1),-s31)*za21**2*
     .(za36*zb23+za16*zb21)**2*zb31**2)/
     .(2d0*s31**2*za65*zb21*
     .(-(za43*zb23)-za41*zb21)*t(i2,i3,i1))


      zFscqggqb(1,2)=zFscqggqb(1,2)
     .-(za21**2*(-(za26*zb21)-za36*zb31)**2)/
     .(2d0*za23*za31*za65*zb21*
     .(za42*zb21+za43*zb31)*t(i2,i3,i1))


      zFscqggqb(1,2)=zFscqggqb(1,2)
     .+(za21**2*zb23*zb31*
     .(-(za26*zb21)-za36*zb31)**2*
     .(-zL1d(-t(i2,i3,i1),-s23)/(2d0*s23**2)+
     .zLs1d(-s31,-t(i2,i3,i1),-s23,-t(i2,i3,i1))/t(i2,i3,i1)**2))/
     .(za65*zb21*(za42*zb21+za43*zb31)*
     .t(i2,i3,i1))


      zFscqggqb(1,2)=zFscqggqb(1,2)
     .+(zL0d(-t(i2,i3,i1),-s31)*za46**2*za21*zb23*
     .zb31*t(i2,i3,i1))/
     .(s31*za65*
     .(-(za43*zb23)-za41*zb21)**2*
     .(za42*zb21+za43*zb31))

      zFscqggqb(1,2)=zFscqggqb(1,2)
     .+(zLsm1_2mh(s42,t(i2,i3,i1),s31,s65)*za46**2*
     .zb23*
     .(za42*zb23-za41*zb31)*t(i2,i3,i1))/
     .(za65*(-(za43*zb23)-za41*zb21)**3*
     .(za42*zb21+za43*zb31))

      zFscqggqb(1,2)=zFscqggqb(1,2)
     .+(za46**2*zb43*
     .(za42*zb23-za41*zb31)*
     .(-(zL1d(-s65,-t(i2,i3,i1))*zb43)/
     .(2d0*zb23*t(i2,i3,i1)**2)+
     .zL0d(-s65,-t(i2,i3,i1))/
     .((-(za43*zb23)-za41*zb21)*t(i2,i3,i1)))*
     .t(i2,i3,i1))/
     .(za65*(-(za43*zb23)-za41*zb21)*
     .(za42*zb21+za43*zb31))


      zFscqggqb(1,2)=zFscqggqb(1,2)
     .-(zI3m(s31,s42,s65)*za46*zb31*
     .(za21*za26*zb43*zb65+
     .(3d0*za42*(-(za23*zb43)-za21*zb41)*
     .(-(za41*zb43)-za21*zb23)*
     .((s42-s31-s65)*za46*zb42-
     .(-s42-s31+s65)*za65*zb25)*zb65
     .)/((s42**2-2d0*s42*s31+s31**2-2d0*s42*
     .s65-
     .2d0*s31*s65+s65**2)*
     .(-(za43*zb23)-za41*zb21))+
     .((-(za23*zb43)-za21*zb41)*
     .(-(za42*(-(za41*zb43)-za21*zb23)*
     .zb25)-3d0*za46*za21*zb23*zb65))/
     .(-(za43*zb23)-za41*zb21)+
     .(za41*za46*(-(za23*zb43)-
     .za21*zb41)*zb23*zb65*
     .(-t(i4,i3,i1)+t(i2,i3,i1)))/
     .(-(za43*zb23)-za41*zb21)**2))/
     .((s42**2-2d0*s42*s31+s31**2-2d0*s42*s65-
     .2d0*s31*s65+s65**2)*
     .(za42*zb21+za43*zb31))

      zFscqggqb(1,2)=zFscqggqb(1,2)
     .-(zzlogRat(-s31,-s65)*za46*zb31*
     .(-(za46*(-(za41*zb43)-za21*zb23))/
     .(2d0*za65*(-(za43*zb23)-za41*zb21)**2)+
     .(3d0*(-(za23*zb43)-za21*zb41)*
     .(-(za41*zb43)-za21*zb23)*
     .((s42-s31-s65)*za42*zb25-
     .(-s42-s31+s65)*za46*zb65))/
     .((s42**2-2d0*s42*s31+s31**2-
     .2d0*s42*s65-2d0*s31*s65+s65**2)**2*
     .(-(za43*zb23)-za41*zb21))+
     .((-(za23*zb43)-za21*zb41)*
     .(-(za41*zb43)-za21*zb23)*
     .(-(za46*zb42)+za65*zb25)+
     .(-s42+s31-s65)*zb43*
     .(-(za46*(za21*zb42+za31*zb43))-
     .za21*za65*zb25)+
     .((s42-s31-s65)*za46*zb42*
     .(-(za41*zb43)-za21*zb23)*
     .(-t(i4,i3,i1)+t(i2,i3,i1)))/
     .(-(za43*zb23)-za41*zb21))/
     .(2d0*(s42**2-2d0*s42*s31+s31**2-
     .2d0*s42*s65-2d0*s31*s65+s65**2)*za65*
     .zb42*(-(za43*zb23)-za41*zb21))))/
     .(za42*zb21+za43*zb31)


      zFscqggqb(1,2)=zFscqggqb(1,2)
     .+(zzlogRat(-s42,-s65)*za46*
     .(-(za46*za21*zb43)/(2d0*za31*za65)-
     .(3d0*za42*(-(za23*zb43)-za21*zb41)*
     .(-(za41*zb43)-za21*zb23)*zb31*
     .((-s42+s31-s65)*zb25+
     .2d0*za46*zb42*zb65))/
     .(s42**2-2d0*s42*s31+s31**2-2d0*s42*s65-
     .2d0*s31*s65+s65**2)**2-
     .((-(za23*zb43)-za21*zb41)*
     .(-(za41*zb43)-za21*zb23)*
     .((-s42-3d0*s31+s65)*za46+
     .za42*(za36*zb23+za16*zb21))+
     .(s42-s31-s65)*za42*zb43*
     .(-(za46*(za21*zb42+za31*zb43))-
     .za21*za65*zb25)-
     .2d0*za46*t(i2,i3,i1)*
     .(2d0*(-(za23*zb43)-za21*zb41)*
     .(za41*zb43-za21*zb23)+
     .(za21*zb43+
     .(za41*(-(za23*zb43)-
     .za21*zb41)*zb23)/
     .(-(za43*zb23)-za41*zb21))*
     .(-t(i4,i3,i1)+t(i2,i3,i1))))/
     .(2d0*(s42**2-2d0*s42*s31+s31**2-
     .2d0*s42*s65-2d0*s31*s65+s65**2)*za31*za65)))/
     .((-(za43*zb23)-za41*zb21)*
     .(za42*zb21+za43*zb31))

      zFscqggqb(2,1) = za36**2/(2d0*za42*za23*za65*zb31)+
     .zb25**2/(2d0*za42*zb23*zb31*zb65)-
     .(za43*((zL1d(-t(i4,i2,i3),-s23)*za23**2*zb42**2*
     .(za42*zb25+za43*zb35)**2)/s23**2+
     .zL1d(-s65,-t(i4,i2,i3))*za31**2*zb15**2))/
     .(2d0*za42*za23*(za42*zb21+za43*zb31)*
     .zb65*t(i4,i2,i3))-(zb21*
     .(zL1d(-s65,-t(i2,i3,i1))*za46**2*zb42**2+
     .(zL1d(-t(i2,i3,i1),-s23)*za31**2*zb23**2*
     .(-(za26*zb21)-za36*zb31)**2)/s23**2))/
     .(2d0*za65*zb23*zb31*
     .(za42*zb21+za43*zb31)*t(i2,i3,i1))

      zFscqggqb(2,2)=-((za16**2*((zL1d(-s65,-t(i2,i3,i1))*
     .(za46*za21*zb42+
     .za46*za31*zb43)**2)/
     .(2d0*za16**2*t(i2,i3,i1)**2)-
     .(zL1d(-s31,-t(i2,i3,i1))*za26**2*za31**2*zb23**2)/
     .(2d0*za16**2*t(i2,i3,i1)**2)))/
     .(za42*za23*za31*za65))+
     .(-((za16*(za43*zb43*zb45+
     .za23*zb43*zb25))/t(i4,i2,i3))+
     .(zb45*(-(za26*za31*zb23)+
     .za31*za16*zb31))/t(i2,i3,i1))/
     .(3d0*s65*za23**2)+(-((za26*za16*zb23)/
     .(za42*za23*za65*t(i2,i3,i1)))+
     .(-(za21*zb25)-za31*zb35)**2/
     .(za42*za23*za31*zb65*t(i2,i3,i1))-
     .(za16*zb23*(-(za26*zb23)+za16*zb31))/
     .(za42*za65*t(i4,i2,i3)*t(i2,i3,i1))+
     .(za21*zb45*zb23*
     .(za42*zb25+za43*zb35))/
     .(za42*za23*zb65*t(i4,i2,i3)*t(i2,i3,i1)))/2

      zFscqggqb(1,1) = -((zb45**2*((zL1d(-s56,-t(i3,i2,i4))*
     .(zb15*zb34*za13+
     .zb15*zb24*za12)**2)/
     .(2d0*zb45**2*t(i3,i2,i4)**2)-
     .(zL1d(-s24,-t(i3,i2,i4))*zb35**2*zb24**2*za32**2)/
     .(2d0*zb45**2*t(i3,i2,i4)**2)))/
     .(zb13*zb32*zb24*zb56))+
     .(-((zb45*(zb12*za12*za16+
     .zb32*za12*za36))/t(i1,i3,i2))+
     .(za16*(-(zb35*zb24*za32)+
     .zb24*zb45*za24))/t(i3,i2,i4))/
     .(3d0*s56*zb32**2)+(-((zb35*zb45*za32)/
     .(zb13*zb32*zb56*t(i3,i2,i4)))+
     .(-(zb34*za36)-zb24*za26)**2/
     .(zb13*zb32*zb24*za56*t(i3,i2,i4))-
     .(zb45*za32*(-(zb35*za32)+zb45*za24))/
     .(zb13*zb56*t(i1,i3,i2)*t(i3,i2,i4))+
     .(zb34*za16*za32*
     .(zb13*za36+zb12*za26))/
     .(zb13*zb32*za56*t(i1,i3,i2)*t(i3,i2,i4)))/2

      return
      end function

      function zFccqqbgg(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)
      complex*16 zFccqqbgg(2,2)

c      "this swapping of i2<->i3 could indicate a problem"
c     TG 30/06/23: the swap is correct. The code implments the colour ordering
c     zFccqq(q,ga,gb,qb)=A6(q,qb,ga,gb) (Dixon 12.3) as  zFccqq(q,gb,ga,qb,l,lb)
c     so  zFccqq(i1,i3,i2,i4,i5,i6,renscale2) is the colour ordering i1,i2,i3,i4
      
      call swapzAB_kin(6)
      zFccqqbgg(1,1) = zFccqqpp(i4,i2,i3,i1,i6,i5,renscale2)
      call swapzAB_kin(6)
      zFccqqbgg(2,1) = zFccqqmp(i1,i3,i2,i4,i5,i6,renscale2) ! Need to check this
      zFccqqbgg(1,2) = zFccqqpm(i1,i3,i2,i4,i5,i6,renscale2) !
      zFccqqbgg(2,2) = zFccqqpp(i1,i3,i2,i4,i5,i6,renscale2)

      return
      end function

      function zFscqqbgg(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)
      complex*16 zFscqqbgg(2,2)

c      "this swapping of i2<->i3 could indicate a problem"

      call swapzAB_kin(6)
      zFscqqbgg(1,1) = zFscqqpp(i4,i2,i3,i1,i6,i5,renscale2)
      call swapzAB_kin(6)
      zFscqqbgg(2,1) = zFscqqmp(i1,i3,i2,i4,i5,i6,renscale2) ! Need to check this
      zFscqqbgg(1,2) = zFscqqpm(i1,i3,i2,i4,i5,i6,renscale2) !
      zFscqqbgg(2,2) = zFscqqpp(i1,i3,i2,i4,i5,i6,renscale2)

      return
      end function

      function zFaxqqbgg(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      use Parameters_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)
      double precision :: emt,etwidth,emb,ebwidth,emc,ecwidth,emtau,etauwidth
      common /Qmass/ emt,etwidth,emb,ebwidth,emc,ecwidth,emtau,etauwidth
      complex*16 zFaxqqbgg(2,2)

      if (emt.eq.0d0)then
         write(*,*) "Error in axial vector contribution to B2g1Z,
     .the mass of the top has not been defined"
         stop
      endif

      emtsq = emt**2
      zFaxqqbgg(1,2) = zFaxmp(i1,i4,i2,i3,i5,i6,emtsq)
      zFaxqqbgg(2,1) = zFaxpm(i1,i4,i2,i3,i5,i6,emtsq)
      zFaxqqbgg(2,2) = zFaxpp(i1,i4,i2,i3,i5,i6,emtsq)
      call swapzAB_kin(6)
      zFaxqqbgg(1,1) = zFaxpp(i4,i1,i3,i2,i6,i5,emtsq)
      call swapzAB_kin(6)

      return
      end function

      function zFaxpp(i1,i2,i3,i4,i5,i6,emtsq)
      use KinData_mod
      implicit double precision(a-h,o-y)
      implicit complex*16 (z)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

c     statement functions

      t(j1,j2,j3) = s6(j1,j2) + s6(j2,j3) + s6(j1,j3)

      s56  = s6(i5,i6)
      za25 = zA6(i2,i5)
      za12 = zA6(i1,i2)
      za34 = zA6(i3,i4)
      za56 = zA6(i5,i6)
      s12  = s6(i1,i2)
      za24 = zA6(i2,i4)
      za35 = zA6(i3,i5)
      za23 = zA6(i2,i3)
      za45 = zA6(i4,i5)
      zb13 = zB6(i1,i3)
      zb14 = zB6(i1,i4)
      s34  = s6(i3,i4)
      zb36 = zB6(i3,i6)
      zb46 = zB6(i4,i6)
      s14  = s6(i1,i4)
      za13 = zA6(i1,i3)

CC comment out emtsq power corection terms for large mt limit

      zFaxpp=
     .(zzlogRat(-t(i1,i2,i3),-s56)*za25**2)/(za12*za34**2*za56)-
     .(zzlogRat(-t(i1,i2,i4),-s56)*za25**2)/(za12*za34**2*za56)-
     .(zLsm1_2me(t(i1,i2,i3),t(i1,i2,i4),s12,s56)*za25*
     .(za24*za35+za23*za45))/
     .(2d0*za12*za34**3*za56)-
     .(zL0d(-t(i1,i2,i3),-s12)*za25*za35*zb13)/
     .(s12*za34**2*za56)+
     .(zL0d(-t(i1,i2,i4),-s12)*za25*za45*zb14)/
     .(s12*za34**2*za56)+
     .(((zL1d(-t(i1,i2,i4),-s56)*s34)/s56**2+
     .zL0d(-t(i1,i2,i4),-s56)/s56)*za23*za25*zb36)/
     .(za12*za34**2)-(zL1d(-t(i1,i2,i4),-s56)*za23*za25*
     .zb13*zb36)/(s56**2*za24*za34)-
     .(((zL1d(-t(i1,i2,i3),-s56)*s34)/s56**2+
     .zL0d(-t(i1,i2,i3),-s56)/s56)*za24*za25*zb46)/
     .(za12*za34**2)+(zL1d(-t(i1,i2,i3),-s56)*(s14+s34)*
     .za25*zb46)/(s56**2*za13*za34)
C      .-(za25*(((-(za23*zb13)-za24*zb14)*zb36)/
C      .za24+(zb46*t(i1,i3,i4))/za13))/
C      .(12d0*emtsq*s56*za34)

      return
      end function

      function zFaxpm(i1,i2,i3,i4,i5,i6,emtsq)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

c     statement functions

      t(j1,j2,j3) = s6(j1,j2) + s6(j2,j3) + s6(j1,j3)

      s56  = s6(i5,i6)
      s34  = s6(i3,i4)
      za25 = zA6(i2,i5)
      zb12 = zB6(i1,i2)
      za45 = zA6(i4,i5)
      zb14 = zB6(i1,i4)
      za56 = zA6(i5,i6)
      za13 = zA6(i1,i3)
      za23 = zA6(i2,i3)
      zb24 = zB6(i2,i4)
      s12  = s6(i1,i2)
      za24 = zA6(i2,i4)
      za15 = zA6(i1,i5)
      zb13 = zB6(i1,i3)
      za34 = zA6(i3,i4)
      zb23 = zB6(i2,i3)
      zb34 = zB6(i3,i4)
      zb36 = zB6(i3,i6)
      za14 = zA6(i1,i4)
      zb16 = zB6(i1,i6)
      za35 = zA6(i3,i5)
      zb46 = zB6(i4,i6)
      za12 = zA6(i1,i2)
      zb56 = zB6(i5,i6)
      zb26 = zB6(i2,i6)
      zb45 = zB6(i4,i5)
      za36 = zA6(i3,i6)
      zb35 = zB6(i3,i5)
      za46 = zA6(i4,i6)

CC comment out emtsq power corection terms for large mt limit

      zFaxpm=
     .(zzlogRat(-s56,-s34)*(za25*zb12+za45*zb14)**2)/
     .(za56*zb12*(-(za13*zb14)-za23*zb24)**2)+
     .(zL0d(-t(i1,i2,i4),-s12)*za24*
     .(za25*zb12+za45*zb14)*
     .(-(za15*zb14)-za25*zb24))/
     .(s12*za56*(-(za13*zb14)-za23*zb24)**2)-
     .(zL0d(-t(i1,i2,i4),-s56)*zb13*
     .(za25*zb12+za45*zb14)*
     .(-(za15*zb14)-za25*zb24)*
     .(-(za13*zb12)-za34*zb24))/
     .(s56*za56*zb12*zb24*
     .(-(za13*zb14)-za23*zb24)**2)+
C      .(zb13*(-(za25*zb23)+za45*zb34)*zb36)/
C      .(12d0*emtsq*s56*zb24*zb34)-
C      .(za24*za45*(-(za14*zb16)-za34*zb36))/
C      .(12d0*emtsq*s56*za13*za34)+
     .(za24*za35*(-(za14*zb16)-za34*zb36))/
     .(s56*za13*za34*
     .(-(za13*zb14)-za23*zb24))-
     .(zb13*(-(za25*zb23)+za45*zb34)*zb46)/
     .(s56*zb24*(-(za13*zb14)-za23*zb24)*
     .zb34)+(za45*zb13*
     .(-(((s12-s34-s56)*za24*za35)/
     .((s12**2-2d0*s12*s34+s34**2-
     .2d0*s12*s56-2d0*s34*s56+s56**2)*
     .za34*za56))+
     .((-s12+s34-s56)*za35*zb13)/
     .((s12**2-2d0*s12*s34+s34**2-
     .2d0*s12*s56-2d0*s34*s56+s56**2)*za56*
     .zb12)-(2d0*za24*zb46)/
     .(s12**2-2d0*s12*s34+s34**2-2d0*s12*s56-
     .2d0*s34*s56+s56**2)+
     .(zb13*zb46)/(s56*zb12*zb34)-
     .((-s12-s34+s56)*zb13*zb46)/
     .((s12**2-2d0*s12*s34+s34**2-2d0*s12*s56-
     .2d0*s34*s56+s56**2)*zb12*zb34)))/
     .(-(za13*zb14)-za23*zb24)+
     .(za24*zb36*((za24*za35)/
     .(s56*za12*za34)-
     .((-s12-s34+s56)*za24*za35)/
     .((s12**2-2d0*s12*s34+s34**2-
     .2d0*s12*s56-2d0*s34*s56+s56**2)*za12*
     .za34)-(2d0*za35*zb13)/
     .(s12**2-2d0*s12*s34+s34**2-2d0*s12*s56-
     .2d0*s34*s56+s56**2)+
     .((-s12+s34-s56)*za24*zb46)/
     .((s12**2-2d0*s12*s34+s34**2-
     .2d0*s12*s56-2d0*s34*s56+s56**2)*za12*
     .zb56)-((s12-s34-s56)*zb13*zb46)/
     .((s12**2-2d0*s12*s34+s34**2-2d0*s12*s56-
     .2d0*s34*s56+s56**2)*zb34*zb56)))/
     .(-(za13*zb14)-za23*zb24)-
     .(zL0d(-t(i1,i2,i3),-s12)*zb13*
     .(-(za13*zb16)-za23*zb26)*
     .(-(za12*zb16)+za23*zb36))/
     .(s12*(-(za13*zb14)-za23*zb24)**2*zb56)-
     .(zL0d(-t(i1,i2,i3),-s56)*za24*
     .(-(za13*zb16)-za23*zb26)*
     .(za12*zb24+za13*zb34)*
     .(-(za12*zb16)+za23*zb36))/
     .(s56*za12*za13*
     .(-(za13*zb14)-za23*zb24)**2*zb56)+
     .(zzlogRat(-s56,-s34)*(-(za12*zb16)+za23*zb36)**2)/
     .(za12*(-(za13*zb14)-za23*zb24)**2*zb56)-
     .(zL1d(-s56,-t(i1,i2,i3))*za14*za24*
     .(-(za12*zb16)+za23*zb36)*zb46)/
     .(za12*za13*(-(za13*zb14)-za23*zb24)*
     .zb56*t(i1,i2,i3))-(zLsm1_2mh(s34,t(i1,i2,i3),s12,
     .s56)*((-(za13*zb16)-za23*zb26)**2*
     .(-(za12*zb14)+za23*zb34)**2-
     .za23**2*zb46**2*t(i1,i2,i3)**2))/
     .(2d0*za12*(-(za13*zb14)-za23*zb24)**4*zb56)

      zFaxpm=zFaxpm
     .+zzlogRat(-s12,-s34)*((-6d0*zb12*
     .((-s12+s34-s56)*za25-
     .2d0*za12*za56*zb16)*
     .(-(za14*zb13)-za24*zb23)*
     .(-(za12*zb16)+za23*zb36))/
     .((s12**2-2d0*s12*s34+s34**2-2d0*s12*s56-
     .2d0*s34*s56+s56**2)**2*
     .(-(za13*zb14)-za23*zb24))-
     .(za24*zb13*zb36**2)/
     .((s12**2-2d0*s12*s34+s34**2-2d0*s12*s56-
     .2d0*s34*s56+s56**2)*zb34*zb56)-
     .(zb13*(-(za12*zb16)+za23*zb36)*zb46)/
     .((-(za13*zb14)-za23*zb24)**2*zb34*zb56)+(zb14*
     .(-(za14*zb13)-za24*zb23)*
     .(-(za12*zb16)+za23*zb36)*
     .(3d0*(-(za13*zb14*zb36)-
     .za23*zb24*zb36)-
     .zb46*(t(i1,i2,i3)-t(i1,i2,i4))))/
     .((s12**2-2d0*s12*s34+s34**2-2d0*s12*s56-
     .2d0*s34*s56+s56**2)*
     .(-(za13*zb14)-za23*zb24)**2*zb34*zb56))


      zFaxpm=zFaxpm
     .+zI3m(s12,s34,s56)*
     .(-((za24*za45*zb13*zb36)/
     .(s12**2-2d0*s12*s34+s34**2-2d0*s12*s56-
     .2d0*s34*s56+s56**2))-
     .(3d0*(-s12+s34-s56)*
     .((s12-s34-s56)*za25*zb12+
     .(-s12-s34+s56)*za56*zb16)*
     .(-(za14*zb13)-za24*zb23)*
     .(-(za12*zb16)+za23*zb36))/
     .((s12**2-2d0*s12*s34+s34**2-2d0*s12*s56-
     .2d0*s34*s56+s56**2)**2*
     .(-(za13*zb14)-za23*zb24))-
     .(3d0*(-(za14*zb13)-za24*zb23)*
     .(-(za12*za25*zb12*zb16)-
     .za23*za45*zb14*zb36-
     .za24*za35*zb13*zb46-
     .za25*za56*zb16*zb56))/
     .(2d0*(s12**2-2d0*s12*s34+s34**2-2d0*s12*s56-
     .2d0*s34*s56+s56**2)*
     .(-(za13*zb14)-za23*zb24))-
     .(za45*zb13*(-(za12*zb16)+za23*zb36))/
     .(2d0*(-(za13*zb14)-za23*zb24)*t(i1,i2,i3))+
     .(za35*zb14*(-(za14*zb13)-za24*zb23)*
     .(-(za12*zb16)+za23*zb36)*
     .(t(i1,i2,i3)-t(i1,i2,i4)))/
     .((s12**2-2d0*s12*s34+s34**2-2d0*s12*s56-
     .2d0*s34*s56+s56**2)*
     .(-(za13*zb14)-za23*zb24)**2))


      zFaxpm=zFaxpm
     .-(zL1d(-s56,-t(i1,i2,i4))*za35*zb13*
     .(za25*zb12+za45*zb14)*zb23)/
     .(za56*zb12*zb24*
     .(-(za13*zb14)-za23*zb24)*t(i1,i2,i4))


      zFaxpm=zFaxpm
     .-(zLsm1_2mh(s34,t(i1,i2,i4),s12,s56)*
     .((za23*zb12-za34*zb14)**2*
     .(-(za15*zb14)-za25*zb24)**2-
     .za35**2*zb14**2*t(i1,i2,i4)**2))/
     .(2d0*za56*zb12*(-(za13*zb14)-za23*zb24)**4)


      zFaxpm=zFaxpm
     .+zI3m(s12,s34,s56)*
     .(-((za24*za45*zb13*zb36)/
     .(s12**2-2d0*s12*s34+s34**2-2d0*s12*s56-
     .2d0*s34*s56+s56**2))-
     .(3d0*(-s12+s34-s56)*
     .(za25*zb12+za45*zb14)*
     .(-(za14*zb13)-za24*zb23)*
     .(-((s12-s34-s56)*za12*zb16)-
     .(-s12-s34+s56)*za25*zb56))/
     .((s12**2-2d0*s12*s34+s34**2-2d0*s12*s56-
     .2d0*s34*s56+s56**2)**2*
     .(-(za13*zb14)-za23*zb24))-
     .(3d0*(-(za14*zb13)-za24*zb23)*
     .(-(za12*za25*zb12*zb16)-
     .za23*za45*zb14*zb36-
     .za24*za35*zb13*zb46-
     .za25*za56*zb16*zb56))/
     .(2d0*(s12**2-2d0*s12*s34+s34**2-2d0*s12*s56-
     .2d0*s34*s56+s56**2)*
     .(-(za13*zb14)-za23*zb24))-
     .(za24*(za25*zb12+za45*zb14)*zb36)/
     .(2d0*(-(za13*zb14)-za23*zb24)*t(i1,i2,i4))+
     .(za23*(za25*zb12+za45*zb14)*
     .(-(za14*zb13)-za24*zb23)*zb46*
     .(-t(i1,i2,i3)+t(i1,i2,i4)))/
     .((s12**2-2d0*s12*s34+s34**2-2d0*s12*s56-
     .2d0*s34*s56+s56**2)*
     .(-(za13*zb14)-za23*zb24)**2))


      zFaxpm=zFaxpm
     .+zzlogRat(-s12,-s34)*(-((za24*za45**2*zb13)/
     .((s12**2-2d0*s12*s34+s34**2-2d0*s12*s56-
     .2d0*s34*s56+s56**2)*za34*za56))-
     .(za24*za35*(za25*zb12+za45*zb14))/
     .(za34*za56*(-(za13*zb14)-za23*zb24)**
     .2)+(6d0*za12*(za25*zb12+za45*zb14)*
     .(-(za14*zb13)-za24*zb23)*
     .((-s12+s34-s56)*zb16-
     .2d0*za25*zb12*zb56))/
     .((s12**2-2d0*s12*s34+s34**2-2d0*s12*s56-
     .2d0*s34*s56+s56**2)**2*
     .(-(za13*zb14)-za23*zb24))+
     .(za23*(za25*zb12+za45*zb14)*
     .(-(za14*zb13)-za24*zb23)*
     .(3d0*(-(za13*za45*zb14)-
     .za23*za45*zb24)-
     .za35*(-t(i1,i2,i3)+t(i1,i2,i4))))/
     .((s12**2-2d0*s12*s34+s34**2-2d0*s12*s56-
     .2d0*s34*s56+s56**2)*za34*za56*
     .(-(za13*zb14)-za23*zb24)**2))


      zFaxpm=zFaxpm
     .+zzlogRat(-s56,-s34)*((za45*zb13**2*zb36)/
     .((s12**2-2d0*s12*s34+s34**2-2d0*s12*s56-
     .2d0*s34*s56+s56**2)*zb12*zb34)+
     .(zb14*(za35*zb13-za56*zb16)*zb36)/
     .(zb12*zb34*(-(za35*zb45)-za36*zb46)**
     .2)+(6d0*(za35*zb13-za56*zb16)*
     .(-((-s12+s34-s56)*za25)+
     .2d0*za12*za56*zb16)*
     .(-(za45*zb35)-za46*zb36)*zb56)/
     .((s12**2-2d0*s12*s34+s34**2-2d0*s12*s56-
     .2d0*s34*s56+s56**2)**2*
     .(-(za35*zb45)-za36*zb46))+
     .((za35*zb13-za56*zb16)*
     .(-(za45*zb35)-za46*zb36)*zb46*
     .(3d0*(za35*zb13*zb45+
     .za36*zb13*zb46)+
     .zb14*(t(i3,i5,i6)-t(i4,i5,i6))))/
     .((s12**2-2d0*s12*s34+s34**2-2d0*s12*s56-
     .2d0*s34*s56+s56**2)*zb12*zb34*
     .(-(za35*zb45)-za36*zb46)**2))


      zFaxpm=zFaxpm
     .+zzlogRat(-s56,-s34)*((za24**2*za45*zb36)/
     .((s12**2-2d0*s12*s34+s34**2-2d0*s12*s56-
     .2d0*s34*s56+s56**2)*za12*za34)+
     .(za23*za45*(za24*zb46+za25*zb56))/
     .(za12*za34*(-(za35*zb45)-za36*zb46)**2)
     .-(6d0*za56*(-(za45*zb35)-za46*zb36)*
     .(za24*zb46+za25*zb56)*
     .(-((-s12+s34-s56)*zb16)+
     .2d0*za25*zb12*zb56))/
     .((s12**2-2d0*s12*s34+s34**2-2d0*s12*s56-
     .2d0*s34*s56+s56**2)**2*
     .(-(za35*zb45)-za36*zb46))+
     .(za35*(-(za45*zb35)-za46*zb36)*
     .(za24*zb46+za25*zb56)*
     .(3d0*(za24*za35*zb45+
     .za24*za36*zb46)+
     .za23*(-t(i3,i5,i6)+t(i4,i5,i6))))/
     .((s12**2-2d0*s12*s34+s34**2-2d0*s12*s56-
     .2d0*s34*s56+s56**2)*za12*za34*
     .(-(za35*zb45)-za36*zb46)**2))

      return
      end function

      function zFaxmp(i1,i2,i3,i4,i5,i6,emtsq)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

      t(j1,j2,j3)=s6(j1,j2)+s6(j2,j3)+s6(j3,j1)

      za23 = zA6(i2,i3)
      za35 = zA6(i3,i5)
      zb16 = zB6(i1,i6)
      s56  = s6(i5,i6)
      za24 = zA6(i2,i4)
      za34 = zA6(i3,i4)
      s34  = s6(i3,i4)
      za25 = zA6(i2,i5)
      zb12 = zB6(i1,i2)
      zb13 = zB6(i1,i3)
      za56 = zA6(i5,i6)
      za14 = zA6(i1,i4)
      zb23 = zB6(i2,i3)
      za45 = zA6(i4,i5)
      s12  = s6(i1,i2)
      za15 = zA6(i1,i5)
      zb14 = zB6(i1,i4)
      zb36 = zB6(i3,i6)
      zb34 = zB6(i3,i4)
      zb46 = zB6(i4,i6)
      za12 = zA6(i1,i2)
      zb56 = zB6(i5,i6)
      zb26 = zB6(i2,i6)
      za13 = zA6(i1,i3)
      zb24 = zB6(i2,i4)
      zb35 = zB6(i3,i5)
      za46 = zA6(i4,i6)
      zb45 = zB6(i4,i5)
      za36 = zA6(i3,i6)

CC comment out emtsq power corection terms for large mt limit

      zFaxmp=
C      .(za23**2*za35*zb16)/(12d0*emtsq*s56*za24*za34)
     .-(zzlogRat(-s56,-s34)*(za25*zb12+za35*zb13)**2)/
     .(za56*zb12*(-(za14*zb13)-za24*zb23)**2)-
     .(za23**2*za45*zb16)/
     .(s56*za24*za34*
     .(-(za14*zb13)-za24*zb23))-
     .(zL0d(-t(i1,i2,i3),-s12)*za23*
     .(za25*zb12+za35*zb13)*
     .(-(za15*zb13)-za25*zb23))/
     .(s12*za56*(-(za14*zb13)-za24*zb23)**2)+
     .(zL0d(-t(i1,i2,i3),-s56)*(za24*zb12+za34*zb13)*
     .(za25*zb12+za35*zb13)*zb14*
     .(-(za15*zb13)-za25*zb23))/
     .(s56*za56*zb12*zb13*
     .(-(za14*zb13)-za24*zb23)**2)+
     .(za25*zb14**2*zb36)/
     .(s56*zb13*(-(za14*zb13)-za24*zb23)*
     .zb34)-(za35*zb14*
     .(((s12-s34-s56)*za23*za45)/
     .((s12**2-2d0*s12*s34+s34**2-
     .2d0*s12*s56-2d0*s34*s56+s56**2)*za34*
     .za56)+((-s12+s34-s56)*za45*
     .zb14)/
     .((s12**2-2d0*s12*s34+s34**2-
     .2d0*s12*s56-2d0*s34*s56+s56**2)*za56*
     .zb12)-(2d0*za23*zb36)/
     .(s12**2-2d0*s12*s34+s34**2-2d0*s12*s56-
     .2d0*s34*s56+s56**2)-
     .(zb14*zb36)/(s56*zb12*zb34)+
     .((-s12-s34+s56)*zb14*zb36)/
     .((s12**2-2d0*s12*s34+s34**2-2d0*s12*s56-
     .2d0*s34*s56+s56**2)*zb12*zb34)))/
     .(-(za14*zb13)-za24*zb23)-
C      .(za25*zb14**2*zb46)/(12d0*emtsq*s56*zb13*zb34)-
     .(za23*zb46*(-((za23*za45)/
     .(s56*za12*za34))+
     .((-s12-s34+s56)*za23*za45)/
     .((s12**2-2d0*s12*s34+s34**2-
     .2d0*s12*s56-2d0*s34*s56+s56**2)*za12*
     .za34)-(2d0*za45*zb14)/
     .(s12**2-2d0*s12*s34+s34**2-2d0*s12*s56-
     .2d0*s34*s56+s56**2)+
     .((-s12+s34-s56)*za23*zb36)/
     .((s12**2-2d0*s12*s34+s34**2-
     .2d0*s12*s56-2d0*s34*s56+s56**2)*za12*
     .zb56)+((s12-s34-s56)*zb14*zb36)/
     .((s12**2-2d0*s12*s34+s34**2-2d0*s12*s56-
     .2d0*s34*s56+s56**2)*zb34*zb56)))/
     .(-(za14*zb13)-za24*zb23)+
     .(zL0d(-t(i1,i2,i4),-s12)*zb14*
     .(-(za14*zb16)-za24*zb26)*
     .(-(za12*zb16)+za24*zb46))/
     .(s12*(-(za14*zb13)-za24*zb23)**2*zb56)+
     .(zL0d(-t(i1,i2,i4),-s56)*za23*
     .(-(za14*zb16)-za24*zb26)*
     .(-(za12*zb13)-za24*zb34)*
     .(-(za12*zb16)+za24*zb46))/
     .(s56*za12*za24*
     .(-(za14*zb13)-za24*zb23)**2*zb56)-
     .(zzlogRat(-s56,-s34)*(-(za12*zb16)+za24*zb46)**2)/
     .(za12*(-(za14*zb13)-za24*zb23)**2*zb56)+
     .(zL1d(-s56,-t(i1,i2,i3))*za45*
     .(za25*zb12+za35*zb13)*zb14**2)/
     .(za56*zb12*zb13*
     .(-(za14*zb13)-za24*zb23)*t(i1,i2,i3))+
     .(zLsm1_2mh(s34,t(i1,i2,i3),s12,s56)*
     .((za24*zb12+za34*zb13)**2*
     .(-(za15*zb13)-za25*zb23)**2-
     .za45**2*zb13**2*t(i1,i2,i3)**2))/
     .(2d0*za56*zb12*(-(za14*zb13)-za24*zb23)**4)


      zFaxmp=zFaxmp
     .-zzlogRat(-s12,-s34)*((za23*za35**2*zb14)/
     .((s12**2-2d0*s12*s34+s34**2-2d0*s12*s56-
     .2d0*s34*s56+s56**2)*za34*za56)+
     .(za23*za45*(za25*zb12+za35*zb13))/
     .(za34*za56*(-(za14*zb13)-za24*zb23)**
     .2)+(6d0*za12*(za25*zb12+za35*zb13)*
     .(-(za13*zb14)-za23*zb24)*
     .((-s12+s34-s56)*zb16-
     .2d0*za25*zb12*zb56))/
     .((s12**2-2d0*s12*s34+s34**2-2d0*s12*s56-
     .2d0*s34*s56+s56**2)**2*
     .(-(za14*zb13)-za24*zb23))-
     .(za24*(za25*zb12+za35*zb13)*
     .(-(za13*zb14)-za23*zb24)*
     .(3d0*(-(za14*za35*zb13)-
     .za24*za35*zb23)-
     .za45*(t(i1,i2,i3)-t(i1,i2,i4))))/
     .((s12**2-2d0*s12*s34+s34**2-2d0*s12*s56-
     .2d0*s34*s56+s56**2)*za34*za56*
     .(-(za14*zb13)-za24*zb23)**2))


      zFaxmp=zFaxmp
     .-zI3m(s12,s34,s56)*
     .(-((za23*za35*zb14*zb46)/
     .(s12**2-2d0*s12*s34+s34**2-2d0*s12*s56-
     .2d0*s34*s56+s56**2))-
     .(3d0*(-s12+s34-s56)*
     .(za25*zb12+za35*zb13)*
     .(-(za13*zb14)-za23*zb24)*
     .(-((s12-s34-s56)*za12*zb16)-
     .(-s12-s34+s56)*za25*zb56))/
     .((s12**2-2d0*s12*s34+s34**2-2d0*s12*s56-
     .2d0*s34*s56+s56**2)**2*
     .(-(za14*zb13)-za24*zb23))-
     .(3d0*(-(za13*zb14)-za23*zb24)*
     .(-(za12*za25*zb12*zb16)-
     .za23*za45*zb14*zb36-
     .za24*za35*zb13*zb46-
     .za25*za56*zb16*zb56))/
     .(2d0*(s12**2-2d0*s12*s34+s34**2-2d0*s12*s56-
     .2d0*s34*s56+s56**2)*
     .(-(za14*zb13)-za24*zb23))-
     .(za23*(za25*zb12+za35*zb13)*zb46)/
     .(2d0*(-(za14*zb13)-za24*zb23)*t(i1,i2,i3))+
     .(za24*(za25*zb12+za35*zb13)*
     .(-(za13*zb14)-za23*zb24)*zb36*
     .(t(i1,i2,i3)-t(i1,i2,i4)))/
     .((s12**2-2d0*s12*s34+s34**2-2d0*s12*s56-
     .2d0*s34*s56+s56**2)*
     .(-(za14*zb13)-za24*zb23)**2))


      zFaxmp=zFaxmp
     .+(zL1d(-s56,-t(i1,i2,i4))*za23**2*zb36*
     .(-(za12*zb16)+za24*zb46))/
     .(za12*za24*(-(za14*zb13)-za24*zb23)*
     .zb56*t(i1,i2,i4))


      zFaxmp=zFaxmp
     .+(zLsm1_2mh(s34,t(i1,i2,i4),s12,
     .s56)*((-(za14*zb16)-za24*zb26)**2*
     .(-(za12*zb13)-za24*zb34)**2-
     .za24**2*zb36**2*t(i1,i2,i4)**2))/
     .(2d0*za12*(-(za14*zb13)-za24*zb23)**4*zb56)


      zFaxmp=zFaxmp
     .-zI3m(s12,s34,s56)*
     .(-((za23*za35*zb14*zb46)/
     .(s12**2-2d0*s12*s34+s34**2-2d0*s12*s56-
     .2d0*s34*s56+s56**2))-
     .(3d0*(-s12+s34-s56)*
     .((s12-s34-s56)*za25*zb12+
     .(-s12-s34+s56)*za56*zb16)*
     .(-(za13*zb14)-za23*zb24)*
     .(-(za12*zb16)+za24*zb46))/
     .((s12**2-2d0*s12*s34+s34**2-2d0*s12*s56-
     .2d0*s34*s56+s56**2)**2*
     .(-(za14*zb13)-za24*zb23))-
     .(3d0*(-(za13*zb14)-za23*zb24)*
     .(-(za12*za25*zb12*zb16)-
     .za23*za45*zb14*zb36-
     .za24*za35*zb13*zb46-
     .za25*za56*zb16*zb56))/
     .(2d0*(s12**2-2d0*s12*s34+s34**2-2d0*s12*s56-
     .2d0*s34*s56+s56**2)*
     .(-(za14*zb13)-za24*zb23))-
     .(za35*zb14*(-(za12*zb16)+za24*zb46))/
     .(2d0*(-(za14*zb13)-za24*zb23)*t(i1,i2,i4))+
     .(za45*zb13*(-(za13*zb14)-za23*zb24)*
     .(-(za12*zb16)+za24*zb46)*
     .(-t(i1,i2,i3)+t(i1,i2,i4)))/
     .((s12**2-2d0*s12*s34+s34**2-2d0*s12*s56-
     .2d0*s34*s56+s56**2)*
     .(-(za14*zb13)-za24*zb23)**2))


      zFaxmp=zFaxmp
     .-zzlogRat(-s12,-s34)*((-6d0*zb12*
     .((-s12+s34-s56)*za25-
     .2d0*za12*za56*zb16)*
     .(-(za13*zb14)-za23*zb24)*
     .(-(za12*zb16)+za24*zb46))/
     .((s12**2-2d0*s12*s34+s34**2-2d0*s12*s56-
     .2d0*s34*s56+s56**2)**2*
     .(-(za14*zb13)-za24*zb23))+
     .(za23*zb14*zb46**2)/
     .((s12**2-2d0*s12*s34+s34**2-2d0*s12*s56-
     .2d0*s34*s56+s56**2)*zb34*zb56)+
     .(zb14*zb36*(-(za12*zb16)+za24*zb46))/
     .((-(za14*zb13)-za24*zb23)**2*zb34*
     .zb56)-(zb13*
     .(-(za13*zb14)-za23*zb24)*
     .(-(za12*zb16)+za24*zb46)*
     .(3d0*(-(za14*zb13*zb46)-
     .za24*zb23*zb46)-
     .zb36*(-t(i1,i2,i3)+t(i1,i2,i4))))/
     .((s12**2-2d0*s12*s34+s34**2-2d0*s12*s56-
     .2d0*s34*s56+s56**2)*
     .(-(za14*zb13)-za24*zb23)**2*zb34*zb56))


      zFaxmp=zFaxmp
     .-zzlogRat(-s56,-s34)*(-((za23**2*za35*zb46)/
     .((s12**2-2d0*s12*s34+s34**2-2d0*s12*s56-
     .2d0*s34*s56+s56**2)*za12*za34))-
     .(za24*za35*(za23*zb36+za25*zb56))/
     .(za12*za34*(-(za45*zb35)-za46*zb36)**
     .2)-(6d0*za56*(-(za35*zb45)-za36*zb46)*
     .(za23*zb36+za25*zb56)*
     .(-((-s12+s34-s56)*zb16)+
     .2d0*za25*zb12*zb56))/
     .((s12**2-2d0*s12*s34+s34**2-2d0*s12*s56-
     .2d0*s34*s56+s56**2)**2*
     .(-(za45*zb35)-za46*zb36))-
     .(za45*(-(za35*zb45)-za36*zb46)*
     .(za23*zb36+za25*zb56)*
     .(3d0*(za23*za45*zb35+
     .za23*za46*zb36)+
     .za24*(t(i3,i5,i6)-t(i4,i5,i6))))/
     .((s12**2-2d0*s12*s34+s34**2-2d0*s12*s56-
     .2d0*s34*s56+s56**2)*za12*za34*
     .(-(za45*zb35)-za46*zb36)**2))


      zFaxmp=zFaxmp
     .-zzlogRat(-s56,-s34)*(-((za35*zb14**2*zb46)/
     .((s12**2-2d0*s12*s34+s34**2-2d0*s12*s56-
     .2d0*s34*s56+s56**2)*zb12*zb34))-
     .(zb13*(za45*zb14-za56*zb16)*zb46)/
     .(zb12*zb34*(-(za45*zb35)-za46*zb36)**2)
     .+(6d0*(za45*zb14-za56*zb16)*
     .(-((-s12+s34-s56)*za25)+
     .2d0*za12*za56*zb16)*
     .(-(za35*zb45)-za36*zb46)*zb56)/
     .((s12**2-2d0*s12*s34+s34**2-2d0*s12*s56-
     .2d0*s34*s56+s56**2)**2*
     .(-(za45*zb35)-za46*zb36))-
     .((za45*zb14-za56*zb16)*zb36*
     .(-(za35*zb45)-za36*zb46)*
     .(3d0*(za45*zb14*zb35+
     .za46*zb14*zb36)+
     .zb13*(-t(i3,i5,i6)+t(i4,i5,i6))))/
     .((s12**2-2d0*s12*s34+s34**2-2d0*s12*s56-
     .2d0*s34*s56+s56**2)*zb12*zb34*
     .(-(za45*zb35)-za46*zb36)**2))


      return
      end function



      function zFvsqqbgg(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)
      complex*16 zFvsqqbgg(2,2)

c     statement functions

      t(j1,j2,j3) = s6(j1,j2) + s6(j2,j3) + s6(j1,j3)


      zBrackppa(j1,j2,j3,j4,j5,j6) = zA6(j1,j2)
     . *(zA6(j5,j3)*zB6(j3,j1))**2/zA6(j5,j6)/zA6(j3,j4)**2
     . *zL1d(-t(j1,j2,j3),-s6(j1,j2))/s6(j1,j2)**2
     . +zA6(j5,j2)*zA6(j5,j3)*zB6(j3,j1)/zA6(j5,j6)/zA6(j3,j4)**2
     . *zL0d(-t(j1,j2,j3),-s6(j1,j2))/s6(j1,j2)

      zBrackpp(j1,j2,j3,j4,j5,j6) =
     . -zA6(j2,j3)*zA6(j2,j4)*zA6(j3,j5)*zA6(j4,j5)
     . /zA6(j1,j2)/zA6(j5,j6)/zA6(j3,j4)**4
     . *zLsm1_2me(t(j1,j2,j3),t(j1,j2,j4),s6(j1,j2),s6(j5,j6))
     . -(zA6(j2,j4)*zA6(j3,j5)+zA6(j2,j3)*zA6(j4,j5))
     . /zA6(j1,j2)/zA6(j5,j6)/zA6(j3,j4)**3
     . *(zA6(j5,j3)*zB6(j3,j1)*zA6(j1,j2)
     . *zL0d(-t(j1,j2,j3),-s6(j1,j2))/s6(j1,j2)
     . +zA6(j5,j6)*zB6(j6,j4)*zA6(j4,j2)
     . *zL0d(-t(j1,j2,j3),-s6(j5,j6))/s6(j5,j6)
     . +0.5d0*zA6(j5,j2)*(zzlogRat(-t(j1,j2,j3),-t(j1,j2,j4))))
     . -zBrackppa(j1,j2,j3,j4,j5,j6)-zBrackppa(j6,j5,j3,j4,j2,j1)
     . +0.5d0/zA6(j3,j4)**2
     . *(zA6(j2,j5)**2/zA6(j1,j2)/zA6(j5,j6)
     . -zB6(j1,j6)**2/zB6(j1,j2)/zB6(j5,j6))
c     swap za <-> zb
      call swapzAB_kin(6)
      zFvsqqbgg(1,1) = zBrackpp(i4,i1,i2,i3,i6,i5)
     .                +zBrackpp(i4,i1,i3,i2,i6,i5)

      zFvsqqbgg(1,2) = zBrackpm(i4,i1,i2,i3,i6,i5)
      zFvsqqbgg(2,1) = zBrackpm(i4,i1,i3,i2,i6,i5)

      call swapzAB_kin(6)
c     swap back

      zFvsqqbgg(1,2) = zFvsqqbgg(1,2)
     .                +zBrackpm(i1,i4,i3,i2,i5,i6)
      zFvsqqbgg(2,1) = zFvsqqbgg(2,1)
     .                +zBrackpm(i1,i4,i2,i3,i5,i6)

      zFvsqqbgg(2,2) = zBrackpp(i1,i4,i3,i2,i5,i6)
     .                +zBrackpp(i1,i4,i2,i3,i5,i6)

      return
      end function

      function zFvfqqbgg(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)
      complex*16 zFvfqqbgg(2,2)

cc      zFvfqqbgg(1,2) = zFvfpm(i4,i1,i2,i3,i6,i5)
cc      zFvfqqbgg(2,1) = zFvfpm(i4,i1,i3,i2,i6,i5)
cc      zFvfqqbgg(1,1) = zFvfpp(i4,i1,i2,i3,i6,i5)
cc      call swapzAB_kin(6)
cc      zFvfqqbgg(2,2) = zFvfpp(i1,i4,i3,i2,i5,i6)
cc      call swapzAB_kin(6)
      call swapzAB_kin(6)
      zFvfqqbgg(1,2) = zFvfpm(i4,i1,i2,i3,i6,i5)
      zFvfqqbgg(1,1) = zFvfpp(i4,i1,i2,i3,i6,i5)
      call swapzAB_kin(6)
      zFvfqqbgg(2,1) = zFvfpm(i1,i4,i2,i3,i5,i6)
      zFvfqqbgg(2,2) = zFvfpp(i1,i4,i2,i3,i5,i6)

      return
      end function

      function zFvfpm(i1,i2,i3,i4,i5,i6)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

c     statement function

      t(j1,j2,j3)=s6(j1,j2)+s6(j2,j3)+s6(j3,j1)
      zab2(j1,j2,j3,j4)=zA6(j1,j2)*zB6(j2,j4)+zA6(j1,j3)*zB6(j3,j4)

      zI3m123456=zI3m(s6(i1,i2),s6(i3,i4),s6(i5,i6))
      zI3m563412=zI3m123456

      za56 = zA6(i5,i6)
      zb12 = zB6(i1,i2)
      za12 = zA6(i1,i2)
      zb56 = zB6(i5,i6)
      za45 = zA6(i4,i5)
      zb13 = zB6(i1,i3)
      za24 = zA6(i2,i4)
      zb36 = zB6(i3,i6)
      s34  = s6(i3,i4)
      s12  = s6(i1,i2)
      s56  = s6(i5,i6)

      zFvfpm=
     & -zab2(i5,i2,i4,i1)**2/(za56*zb12*zab2(i3,i1,i2,i4)**2)
     & *zLsm1_2mh(s34,t(i1,i2,i4),s12,s56)
     & -zab2(i2,i1,i3,i6)**2/(za12*zb56*zab2(i3,i1,i2,i4)**2)
     & *zLsm1_2mh(s34,t(i1,i2,i3),s12,s56)

     & -zI3m123456*za45*zb13
     & *zab2(i2,i1,i3,i6)/(2d0*zab2(i3,i1,i2,i4)*t(i1,i2,i3))

     & -zI3m123456*za24*zb36
     & *zab2(i5,i2,i4,i1)/(2d0*zab2(i3,i1,i2,i4)*t(i1,i2,i4))

     & +zI3m563412*za24*zb36
     & *zab2(i5,i3,i6,i1)/(2d0*zab2(i3,i1,i2,i4)*t(i3,i5,i6))

     & +zI3m563412*za45*zb13
     & *zab2(i2,i4,i5,i6)/(2d0*zab2(i3,i1,i2,i4)*t(i4,i5,i6))

      return
      end function



      function zFvfpp(i1,i2,i3,i4,i5,i6)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

c     statement function
      t(j1,j2,j3)=s6(j1,j2)+s6(j2,j3)+s6(j3,j1)

      za25 = zA6(i2,i5)
      za12 = zA6(i1,i2)
      za56 = zA6(i5,i6)
      za34 = zA6(i3,i4)
      s12  = s6(i1,i2)
      s56  = s6(i5,i6)

      zFvfpp=-za25**2/(za12*za56*za34**2)
     & *zLsm1_2me(t(i1,i2,i3),t(i1,i2,i4),s12,s56)

      return
      end function

      function zBrackpm(i1,i2,i3,i4,i5,i6)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

c     Statement functions

      t(j1,j2,j3)=s6(j1,j2)+s6(j2,j3)+s6(j3,j1)
      edelta(j1,j2,j3,j4,j5,j6)=s6(j1,j2)-s6(j3,j4)-s6(j5,j6)
      zab2(j1,j2,j3,j4)=zA6(j1,j2)*zB6(j2,j4)+zA6(j1,j3)*zB6(j3,j4)

      eIDelta=1d0/(s6(i1,i2)**2+s6(i3,i4)**2+s6(i5,i6)**2
     . -2d0*(s6(i1,i2)*s6(i3,i4)+s6(i1,i2)*s6(i5,i6)+s6(i5,i6)*s6(i3,i4)))

      za23 = zA6(i2,i3)
      zb46 = zB6(i4,i6)
      za12 = zA6(i1,i2)
      zb56 = zB6(i5,i6)
      s34  = s6(i3,i4)
      s12  = s6(i1,i2)
      s56  = s6(i5,i6)
      zb12 = zB6(i1,i2)
      za56 = zA6(i5,i6)
      zb31 = zB6(i3,i1)
      za54 = zA6(i5,i4)
      za24 = zA6(i2,i4)
      zb41 = zB6(i4,i1)
      za53 = zA6(i5,i3)
      zb36 = zB6(i3,i6)
      zb14 = zB6(i1,i4)
      za35 = zA6(i3,i5)
      zb13 = zB6(i1,i3)
      za45 = zA6(i4,i5)
      zb16 = zB6(i1,i6)
      za25 = zA6(i2,i5)
      za21 = zA6(i2,i1)

      zBrackpm=
     & -2d0*za23*zb46*zab2(i3,i1,i2,i6)
     & *zab2(i2,i5,i6,i4)*t(i1,i2,i3)
     & /za12/zb56/zab2(i3,i1,i2,i4)**4
     & *zLsm1_2mh(s34,t(i1,i2,i3),s12,s56)

     &+(2d0*za23*zb46*(zb12*za56*za23*zb46

     & -zab2(i5,i2,i4,i1)*zab2(i3,i1,i2,i4))*zab2(i4,i1,i2,i3)
     & *(t(i1,i2,i3)-t(i1,i2,i4))/zab2(i3,i1,i2,i4)**3*eIDelta

     & -3d0*(s34*edelta(i3,i4,i1,i2,i5,i6)*zab2(i2,i3,i4,i1)
     &  *zab2(i5,i3,i4,i6)*eIDelta
     & -za23*zb31*za54*zb46
     & -za24*zb41*za53*zb36)

     & *zab2(i4,i1,i2,i3)/zab2(i3,i1,i2,i4)*eIDelta
     & -zb14*za35*za23*zb46
     & *zab2(i4,i1,i2,i3)**2/zab2(i3,i1,i2,i4)**2*eIDelta
     & -zb13*za45*za24*zb36*eIDelta)
     & *zI3m(s12,s34,s56)

     & +(2d0*za23*zb46*t(i1,i2,i3)*zab2(i2,i1,i4,i6)
     & /za12/zb56/zab2(i3,i1,i2,i4)**3

     & -(zab2(i2,i1,i4,i6)**2
     &  +2d0*za23*zb36*za24*zb46)
     & /za12/zb56/zab2(i3,i1,i2,i4)**2)
     & *zzlogRat(-t(i1,i2,i3),-s34)

     &  +zBrackpma(i1,i2,i3,i4,i5,i6,1)
     &  +zBrackpma(i5,i6,i4,i3,i1,i2,2)

     &  +zb16*zab2(i4,i1,i2,i3)
     &  *(zb16*edelta(i3,i4,i5,i6,i1,i2)
     & -2d0*zb12*za25*zb56)
     & /zb12/zb56/zab2(i3,i1,i2,i4)*eIDelta

     &  +(zab2(i2,i1,i4,i6)**2+za21*zb16*za25*zb56)
     & /za12/zb56/zab2(i3,i1,i2,i4)**2


      return
      end function

      function zBrackpma(i1,i2,i3,i4,i5,i6,iswap)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

      if (iswap.eq.1) then
         zBrackpma = zBrackpma1(i1,i2,i3,i4,i5,i6)
      else
c     za <-> zb
         call swapzAB_kin(6)
         zBrackpma = zBrackpma1(i1,i2,i3,i4,i5,i6)
c     swap back
         call swapzAB_kin(6)
      endif

      return
      end function

      function zBrackpma1(i1,i2,i3,i4,i5,i6)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)


c     Statement functions

      t(j1,j2,j3)=s6(j1,j2)+s6(j2,j3)+s6(j3,j1)
      edelta(j1,j2,j3,j4,j5,j6)=s6(j1,j2)-s6(j3,j4)-s6(j5,j6)
      zab2(j1,j2,j3,j4)=zA6(j1,j2)*zB6(j2,j4)+zA6(j1,j3)*zB6(j3,j4)

      eIDelta=1d0/(s6(i1,i2)**2+s6(i3,i4)**2+s6(i5,i6)**2
     . -2d0*(s6(i1,i2)*s6(i3,i4)+s6(i1,i2)*s6(i5,i6)+s6(i5,i6)*s6(i3,i4)))

      zb12 = zB6(i1,i2)
      za23 = zA6(i2,i3)
      zb36 = zB6(i3,i6)
      zb56 = zB6(i5,i6)
      s12  = s6(i1,i2)
      za32 = zA6(i3,i2)
      zb21 = zB6(i2,i1)
      zb46 = zB6(i4,i6)
      za25 = zA6(i2,i5)
      za21 = zA6(i2,i1)
      zb16 = zB6(i1,i6)
      za65 = zA6(i6,i5)
      za56 = zA6(i5,i6)
      za52 = zA6(i5,i2)
      za12 = zA6(i1,i2)
      s34  = s6(i3,i4)

      zBrackpma1 = 2d0*zb12*za23*zb36
     & /zb56/zab2(i3,i1,i2,i4)**2
     & *(zab2(i3,i1,i2,i6)*zab2(i2,i1,i3,i4)/zab2(i3,i1,i2,i4)
     & *zL0d(-t(i1,i2,i3),-s12)/s12
     & -za23*zb36
     & *(zL0d(-t(i1,i2,i3),-s12)/s12
     & -1d0/2d0*zL1d(-t(i1,i2,i3),-s12)/s12))

     & -(3d0*edelta(i5,i6,i3,i4,i1,i2)*zab2(i2,i3,i4,i1)
     & *zab2(i5,i3,i4,i6)*zab2(i4,i1,i2,i3)
     & /zab2(i3,i1,i2,i4)*eIDelta**2

     & +2d0*za32*zb21*zb46*(t(i1,i2,i3)-t(i1,i2,i4))
     & *(za25*t(i1,i2,i3)+za21*zb16*za65)
     & /zab2(i3,i1,i2,i4)**3*eIDelta

     & -(2d0*za23*zb36*(t(i1,i2,i3)-t(i1,i2,i4))
     & +edelta(i1,i2,i3,i4,i5,i6)
     & *(za25*t(i1,i2,i3)
     & +za21*zb16*za65)/za56)
     & *za52*zb21/zab2(i3,i1,i2,i4)**2*eIDelta

     & +0.5d0*(za12*zb16**2/zb56
     &  +zb12*za25**2/za56
     & -2d0*za25*zb16)*zab2(i4,i1,i2,i3)
     & /zab2(i3,i1,i2,i4)*eIDelta)
     & *(zzlogRat(-s12,-s34))

      return
      end function

      function zFccqqmp(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

c     Statement functions

      t(j1,j2,j3)=s6(j1,j2)+s6(j2,j3)+s6(j3,j1)

      s14  = s6(i1,i4)
      s42  = s6(i4,i2)
      s13  = s6(i1,i3)
      s56  = s6(i5,i6)
      za45 = zA6(i4,i5)
      zb14 = zB6(i1,i4)
      za25 = zA6(i2,i5)
      zb12 = zB6(i1,i2)
      za56 = zA6(i5,i6)
      za43 = zA6(i4,i3)
      za23 = zA6(i2,i3)
      zb42 = zB6(i4,i2)
      za13 = zA6(i1,i3)
      za15 = zA6(i1,i5)
      zb13 = zB6(i1,i3)
      zb43 = zB6(i4,i3)
      zb23 = zB6(i2,i3)
      za12 = zA6(i1,i2)
      za14 = zA6(i1,i4)
      za42 = zA6(i4,i2)
      zb16 = zB6(i1,i6)
      zb36 = zB6(i3,i6)
      zb56 = zB6(i5,i6)
      zb46 = zB6(i4,i6)
      s23  = s6(i2,i3)
      za35 = zA6(i3,i5)
      zb26 = zB6(i2,i6)
      s43  = s6(i4,i3)
      s12  = s6(i1,i2)

      zFccqqmp=
     .((zLsm1(-s14,-t(i1,i4,i2),-s42,-t(i1,i4,i2))+
     .zLsm1_2mht(s13,t(i1,i4,i2),s42,s56))*
     .(za45*zb14+za25*zb12)**2)/
     .(za56*(za43*zb14+za23*zb12)*zb42*
     .(-(za13*zb12)-za43*zb42))-
     .(2d0*zL0d(-t(i4,i2,i3),-s56)*za15*zb13*zb43*
     .(-(za45*zb43)-za25*zb23))/
     .(s56*za56*zb42*
     .(-(za12*zb42)-za13*zb43)*zb23)-
     .(zI3m(s13,s42,s56)*zb13*
     .(-(s13*s42*za45**2)+
     .(-(za14*(za45*zb14+za25*zb12))+
     .za43*(-(za45*zb43)-za25*zb23))**2))/
     .(2d0*za13*za56*(-(za42*zb12)-za43*zb13)*
     .zb42*(-(za14*zb12)-za43*zb23))-
     .((zLsm1(-s14,-t(i1,i4,i3),-s13,-t(i1,i4,i3))+
     .zLsm1_2mht(s42,t(i1,i4,i3),s13,s56))*
     .(-(za14*zb16)+za43*zb36)**2)/
     .(za13*(-(za13*zb12)-za43*zb42)*
     .(-(za14*zb12)-za43*zb23)*zb56)-
     .(2d0*zL0d(-t(i1,i2,i3),-s56)*za12*za42*zb46*
     .(-(za12*zb16)+za23*zb36))/
     .(s56*za13*za23*
     .(-(za12*zb42)-za13*zb43)*zb56)+
     .(zI3m(s42,s13,s56)*za42*
     .(-(s13*s42*zb16**2)+
     .(zb14*(-(za14*zb16)+za43*zb36)+
     .zb12*(-(za12*zb16)+za23*zb36))**2))/
     .(2d0*za13*(za43*zb14+za23*zb12)*
     .(-(za42*zb12)-za43*zb13)*zb42*zb56)+
     .zLsm1_2mh(s23,t(i1,i4,i2),s14,s56)*
     .(-(((za43*zb14+za23*zb12)*
     .(-(za15*zb12)-za45*zb42)**2)/
     .(za56*zb42*(-(za13*zb12)-
     .za43*zb42)**3))+
     .(za35**2*zb12**2*t(i1,i4,i2)**2)/
     .(za56*(za43*zb14+za23*zb12)*zb42*
     .(-(za13*zb12)-za43*zb42)**3))

      zFccqqmp=zFccqqmp+
     .zLsm1_2mh(s23,t(i1,i4,i3),s14,s56)*
     .(((-(za13*zb16)-za43*zb46)**2*
     .(-(za14*zb12)-za43*zb23))/
     .(za13*(-(za13*zb12)-za43*zb42)**3*
     .zb56)-(za43**2*zb26**2*t(i1,i4,i3)**2)/
     .(za13*(-(za13*zb12)-za43*zb42)**3*
     .(-(za14*zb12)-za43*zb23)*zb56))+
     .(zLsm1(-s13,-t(i1,i2,i3),-s23,-t(i1,i2,i3))*za45**2*
     .zb13**2)/
     .(za56*zb12*(-(za14*zb12)-za43*zb23)*
     .t(i1,i2,i3))-(2d0*zL0d(-t(i1,i2,i3),-s23)*za12*zb13*
     .(za12*zb26+za13*zb36)*
     .(-(za12*zb16)+za23*zb36))/
     .(s23*za13*(-(za12*zb42)-za13*zb43)*
     .zb56*t(i1,i2,i3))+zLsm1_2mh(s14,t(i1,i2,i3),s23,s56)*(-((za45**2*zb13**2*
     .(-(za14*zb13)+za42*zb23))/
     .(za56*(-(za42*zb12)-za43*zb13)*
     .zb23*(-(za14*zb12)-za43*zb23)*
     .t(i1,i2,i3)))-(za12*
     .(-(za12*zb14)-za23*zb43)**2*
     .(za12*zb26+za13*zb36)**2)/
     .(za13*za23*(-(za12*zb42)-za13*zb43)**
     .3d0*zb56*t(i1,i2,i3))+
     .(za12**3*zb46**2*t(i1,i2,i3))/
     .(za13*za23*(-(za12*zb42)-za13*zb43)**3*
     .zb56))

      zFccqqmp=zFccqqmp+
     .zI3m(s14,s23,s56)*
     .((za14*za25*zb13*
     .(-(za14*zb16)+za43*zb36))/
     .(za13*(-(za14*zb12)-za43*zb23)*t(i1,i4,i3))
     .+(4d0*za14*za43*za25*zb13*zb46*zb23-
     .s42*za15*zb16*
     .(-3d0*za14*zb13+za42*zb23)-
     .s23*za14*za25*zb13*zb26+
     .s43*za42*(-(za15*zb13)-za45*zb43)*
     .zb26+(za15*za43*zb13-
     .za43*za45*zb43)*
     .(-(za12*zb13*zb26)-
     .za42*zb43*zb26)-
     .(za14*za23*zb13*zb42*
     .(-2d0*za14*za25*zb26+
     .za45*(za14*zb46-za12*zb26)))/
     .za13-za43*
     .(za25*zb42+za35*zb43)*
     .(-(za14*zb13)-za42*zb23)*zb36-
     .za35*(-(s43*za14*zb13)+
     .s14*za42*zb23)*zb36-
     .s13*za25*zb23*
     .(za42*zb26+za43*zb36)+
     .(-(za12*za42*zb12)-
     .za12*za43*zb13)*
     .(-(za15*zb13*zb26)-
     .za45*zb42*zb36)-
     .zb13*(-(za14*zb12)-za43*zb23)*
     .(za12*za45*zb46+
     .2d0*za15*(-(za12*zb16)-za23*zb36))+
     .2d0*za42*za45*zb23*
     .(-(za13*zb16*zb43)-
     .zb46*(s14+t(i1,i4,i3))))/
     .(2d0*(-(za13*zb12)-za43*zb42)*
     .(-(za12*zb42)-za13*zb43)*
     .(-(za14*zb12)-za43*zb23))+
     .(za45*zb13*(-(za14*zb13)+za42*zb23)*
     .(-(za12*zb16)+za23*zb36))/
     .((-(za14*zb12)-za43*zb23)*t(i1,i2,i3)**2)-
     .(2d0*za12**2*za45*zb13*zb46)/
     .(za13*(-(za12*zb42)-za13*zb43)*t(i1,i2,i3))
     .+(za12*za42*za45*zb13*zb16)/
     .(2d0*za23*(-(za14*zb12)-za43*zb23)*
     .t(i1,i2,i3))+(za45*zb13*
     .(-(za14*zb13)+za42*zb23)*
     .(-(za45*zb43)-za25*zb23))/
     .(2d0*za56*zb23*(-(za14*zb12)-
     .za43*zb23)*t(i1,i2,i3))+
     .(za12*(-(za12*zb16)+za23*zb36)**2*
     .(2d0*s23*s56+(s14-s23-s56)*t(i1,i2,i3)))/
     .(2d0*za13*za23*(-(za12*zb42)-
     .za13*zb43)*zb56*t(i1,i2,i3)**2)+
     .(-((za12*za42*za45*zb16*zb43)/za23)+
     .za42*(-(za15*zb13)-2d0*za25*zb23)*
     .zb36+(2d0*za14*za56*zb36*
     .(-(za12*zb16)+za23*zb36))/za13-
     .(za45*zb13*
     .(-(za15*zb13*
     .(za14*zb43+za12*zb23))-
     .za14*za56*zb43*zb36))/
     .(za56*zb23)+
     .((s14-s23-s56)*za12*za42*zb26*
     .(-(za12*zb16)+za23*zb36))/
     .(za13*za23*zb56)-
     .(za12*za45*(-(za12*zb16*
     .(s56-t(i1,i4,i3)))+
     .za23*zb36*(s56+t(i1,i2,i3))))/
     .(za13*za23))/
     .(2d0*(-(za12*zb42)-za13*zb43)*
     .(-(za14*zb12)-za43*zb23)))

      zFccqqmp=zFccqqmp+
     .(2d0*zL0d(-t(i4,i2,i3),-s23)*za42*zb43*
     .(za25*zb42+za35*zb43)*
     .(-(za45*zb43)-za25*zb23))/
     .(s23*za56*zb42*
     .(-(za12*zb42)-za13*zb43)*t(i4,i2,i3))-
     .(zLsm1(-s42,-t(i4,i2,i3),-s23,-t(i4,i2,i3))*za42**2*
     .zb16**2)/
     .(za43*(za43*zb14+za23*zb12)*zb56*
     .t(i4,i2,i3))+zLsm1_2mh(s14,t(i4,i2,i3),s23,s56)*
     .(-((zb43*(za25*zb42+za35*zb43)**2*
     .(za14*zb43+za12*zb23)**2)/
     .(za56*zb42*(-(za12*zb42)-
     .za13*zb43)**3*zb23*t(i4,i2,i3)))-
     .(za42**2*(za42*zb14-za23*zb13)*zb16**2)/
     .(za23*(za43*zb14+za23*zb12)*
     .(-(za42*zb12)-za43*zb13)*zb56*t(i4,i2,i3)
     .)+(za15**2*zb43**3*t(i4,i2,i3))/
     .(za56*zb42*(-(za12*zb42)-za13*zb43)**3*
     .zb23))

      zFccqqmp=zFccqqmp+
     .zI3m(s14,s23,s56)*
     .(-((za42*zb14*(za45*zb14+za25*zb12)*
     .zb36)/
     .((za43*zb14+za23*zb12)*zb42*t(i1,i4,i2)))
     .+(-(s13*za45*(3d0*za42*zb14-za23*zb13)*
     .zb46)+s12*za35*zb13*
     .(-(za12*zb16)-za42*zb46)+
     .(-(za42*zb12*zb43)-
     .za43*zb13*zb43)*
     .(-(za13*za25*zb16)-
     .za42*za35*zb46)+
     .(-(za12*za35*zb13)-
     .za42*za35*zb43)*
     .(-(za12*zb12*zb16)+
     .za42*zb12*zb46)-
     .za42*(za43*zb14+za23*zb12)*
     .(za15*zb16*zb43+
     .2d0*zb46*(-(za45*zb43)+za25*zb23))-
     .za25*(s12*za42*zb14-
     .s14*za23*zb13)*zb26+
     .s23*za42*za35*zb14*zb36+
     .4d0*za15*za42*za23*zb14*zb12*zb36+
     .s42*za23*(za25*zb12+za35*zb13)*
     .zb36-za25*zb12*
     .(za42*zb14+za23*zb13)*
     .(za12*zb26+za13*zb36)-
     .(za13*za42*zb14*zb23*
     .(zb16*(-(za15*zb14)-za35*zb43)+
     .2d0*za35*zb14*zb36))/zb42-
     .2d0*za23*zb13*zb16*
     .(-(za12*za45*zb42)-
     .za15*(s14+t(i1,i4,i2))))/
     .(2d0*(za43*zb14+za23*zb12)*
     .(-(za13*zb12)-za43*zb42)*
     .(-(za12*zb42)-za13*zb43))+
     .(za42*(za42*zb14-za23*zb13)*zb16*
     .(-(za45*zb43)-za25*zb23))/
     .((za43*zb14+za23*zb12)*t(i4,i2,i3)**2)-
     .(2d0*za15*za42*zb16*zb43**2)/
     .(zb42*(-(za12*zb42)-za13*zb43)*t(i4,i2,i3))
     .-(za42*za45*zb13*zb16*zb43)/
     .(2d0*(za43*zb14+za23*zb12)*zb23*t(i4,i2,i3))+
     .(za42*(za42*zb14-za23*zb13)*zb16*
     .(-(za12*zb16)+za23*zb36))/
     .(2d0*za23*(za43*zb14+za23*zb12)*zb56*
     .t(i4,i2,i3))+(zb43*
     .(-(za45*zb43)-za25*zb23)**2*
     .(2d0*s23*s56+(s14-s23-s56)*t(i4,i2,i3)))/
     .(2d0*za56*zb42*(-(za12*zb42)-za13*zb43)*
     .zb23*t(i4,i2,i3)**2)+
     .((za12*za45*zb13*zb16*zb43)/zb23+
     .((s14-s23-s56)*za35*zb13*zb43*
     .(-(za45*zb43)-za25*zb23))/
     .(za56*zb42*zb23)+
     .za25*zb13*(-(za42*zb46)+
     .2d0*za23*zb36)+
     .(2d0*za25*zb14*
     .(-(za45*zb43)-za25*zb23)*zb56)/
     .zb42-(za42*zb16*
     .(-(za42*(-(za12*zb14)-za23*zb43)*
     .zb46)-za12*za25*zb14*zb56))/
     .(za23*zb56)+
     .(zb16*zb43*(-(za45*zb43*
     .(s56-t(i1,i4,i2)))-
     .za25*zb23*(s56+t(i4,i2,i3))))/
     .(zb42*zb23))/
     .(2d0*(za43*zb14+za23*zb12)*
     .(-(za12*zb42)-za13*zb43)))


      return
      end function

      function zFccqqpm(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

      t(j1,j2,j3)=s6(j1,j2)+s6(j2,j3)+s6(j3,j1)

      s14  = s6(i1,i4)
      s13  = s6(i1,i3)
      zb14 = zB6(i1,i4)
      za15 = zA6(i1,i5)
      zb13 = zB6(i1,i3)
      za45 = zA6(i4,i5)
      zb43 = zB6(i4,i3)
      za56 = zA6(i5,i6)
      za12 = zA6(i1,i2)
      za42 = zA6(i4,i2)
      za23 = zA6(i2,i3)
      za35 = zA6(i3,i5)
      s56  = s6(i5,i6)
      zb12 = zB6(i1,i2)
      za14 = zA6(i1,i4)
      zb42 = zB6(i4,i2)
      za13 = zA6(i1,i3)
      zb23 = zB6(i2,i3)
      s42  = s6(i4,i2)
      zb16 = zB6(i1,i6)
      zb46 = zB6(i4,i6)
      zb56 = zB6(i5,i6)
      zb26 = zB6(i2,i6)
      za43 = zA6(i4,i3)
      za25 = zA6(i2,i5)
      zb36 = zB6(i3,i6)
      s23  = s6(i2,i3)
      s43  = s6(i4,i3)
      s12  = s6(i1,i2)

      zFccqqpm=
     .zLsm1(-s14,-t(i1,i4,i3),-s13,-t(i1,i4,i3))*
     .(-((zb14**2*(-(za15*zb13)-za45*zb43)**2)/
     .(za56*zb13*zb43**2*
     .(-(za12*zb13)-za42*zb43)*
     .(-(za12*zb14)-za23*zb43)))+
     .(zb13*(-(za15*zb14)+za35*zb43)**2)/
     .(za56*zb43**2*(-(za12*zb13)-za42*zb43)*
     .(-(za12*zb14)-za23*zb43)))-
     .(2d0*zL0d(-t(i4,i2,i3),-s56)*za15*zb12*
     .(za14*zb42-za13*zb23)*
     .(-(za45*zb42)+za35*zb23))/
     .(s56*za56*(-(za12*zb42)-za13*zb43)*
     .zb23*(za14*zb43+za12*zb23))+
     .zLsm1(-s14,-t(i1,i4,i2),-s42,-t(i1,i4,i2))*
     .((za14**2*(-(za12*zb16)-za42*zb46)**2)/
     .(za12**2*za42*(-(za12*zb13)-
     .za42*zb43)*(za14*zb43+za12*zb23)*
     .zb56)-(za42*
     .(za14*zb46+za12*zb26)**2)/
     .(za12**2*(-(za12*zb13)-za42*zb43)*
     .(za14*zb43+za12*zb23)*zb56))-
     .(2d0*zL0d(-t(i1,i2,i3),-s56)*za43*
     .(-(za13*zb14)+za23*zb42)*zb46*
     .(-(za13*zb16)-za23*zb26))/
     .(s56*za23*(-(za12*zb42)-za13*zb43)*
     .(-(za12*zb14)-za23*zb43)*zb56)

      zFccqqpm=zFccqqpm+
     .zI3m(s13,s42,s56)*
     .((za15*za43*zb12*zb46+
     .((-(za13*zb12)-za43*zb42)*
     .(za12*za45*zb16*zb43+
     .za12*zb13*
     .(-(za45*zb46)+za25*zb26)))/
     .(-(za12*zb13)-za42*zb43)+
     .((-(za13*zb12)+za43*zb42)*
     .(-(za15*zb16)-za35*zb36))/2)/
     .((-(za12*zb14)-za23*zb43)*
     .(za14*zb43+za12*zb23))-
     .(za35*zb12*(-(za14*zb16)+za42*zb26))/
     .((-(za12*zb13)-za42*zb43)*t(i1,i4,i2)))+
     .zLsm1_2mh(s13,t(i1,i4,i2),s42,s56)*
     .(-(((-(za14*zb13)+za42*zb23)**2*
     .(za14*zb46+za12*zb26)**2)/
     .(za42*(-(za12*zb13)-za42*zb43)*
     .(za14*zb43+za12*zb23)**3*zb56))+
     .(za14**2*zb36**2*t(i1,i4,i2)**2)/
     .(za42*(-(za12*zb13)-za42*zb43)*
     .(za14*zb43+za12*zb23)**3*zb56))+
     .zLsm1_2mh(s23,t(i1,i4,i2),s14,s56)*
     .(((-(za12*zb16)-za42*zb46)**2*
     .(-(za14*zb13)+za42*zb23)**2)/
     .(za42*(-(za12*zb13)-za42*zb43)**3*
     .(za14*zb43+za12*zb23)*zb56)-
     .(za42*zb36**2*t(i1,i4,i2)**2)/
     .((-(za12*zb13)-za42*zb43)**3*
     .(za14*zb43+za12*zb23)*zb56))+
     .(2d0*zzlogRat(-s42,-s56)*(-2d0*(s14-s23)*za43*zb42*
     .(-(za45*zb46)-za25*zb26)+
     .(za45*zb12*((s14-s23)*
     .((-s13+s42-s56)*za25*zb42+
     .(-s13-s42+s56)*za56*zb46)+
     .(-(za12*zb14)-za23*zb43)*
     .(-((-s13+s42-s56)*za45*zb42)+
     .(-s13-s42+s56)*za56*zb26)))/
     .(za56*zb13)-
     .(za43*(zb12*
     .(za14*zb46+za12*zb26)-
     .zb23*(-(za43*zb46)-za23*zb26)-
     .zb42*(-(za14*zb16)+za43*zb36))*
     .(za25*zb42*zb56+zb46*t(i1,i4,i2)))/zb56
     .))/((s13**2-2d0*s13*s42+s42**2-2d0*s13*s56-
     .2d0*s42*s56+s56**2)*
     .(-(za12*zb14)-za23*zb43)*
     .(za14*zb43+za12*zb23))

      zFccqqpm=zFccqqpm+
     .zI3m(s42,s13,s56)*
     .((za15*za43*zb12*zb46+
     .((za13*zb12-za43*zb42)*
     .(-(za45*zb46)-za25*zb26))/2+
     .((-(za13*zb12)-za43*zb42)*
     .(za12*za45*zb16*zb43+
     .za42*zb43*
     .(-(za15*zb16)+za35*zb36)))/
     .(-(za12*zb13)-za42*zb43))/
     .((-(za12*zb14)-za23*zb43)*
     .(za14*zb43+za12*zb23))-
     .(za43*(za45*zb14+za35*zb13)*zb26)/
     .((-(za12*zb13)-za42*zb43)*t(i1,i4,i3)))+
     .zLsm1_2mh(s42,t(i1,i4,i3),s13,s56)*
     .(((za42*zb14-za23*zb13)**2*
     .(-(za15*zb14)+za35*zb43)**2)/
     .(za56*zb13*(-(za12*zb13)-za42*zb43)*
     .(-(za12*zb14)-za23*zb43)**3)-
     .(za25**2*zb14**2*t(i1,i4,i3)**2)/
     .(za56*zb13*(-(za12*zb13)-za42*zb43)*
     .(-(za12*zb14)-za23*zb43)**3))+
     .zLsm1_2mh(s23,t(i1,i4,i3),s14,s56)*
     .(-(((za42*zb14-za23*zb13)**2*
     .(-(za15*zb13)-za45*zb43)**2)/
     .(za56*zb13*(-(za12*zb13)-
     .za42*zb43)**3*
     .(-(za12*zb14)-za23*zb43)))+
     .(za25**2*zb13*t(i1,i4,i3)**2)/
     .(za56*(-(za12*zb13)-za42*zb43)**3*
     .(-(za12*zb14)-za23*zb43)))+
     .(2d0*zzlogRat(-s13,-s56)*(-2d0*(s14-s23)*za13*zb12*
     .(-(za15*zb16)-za35*zb36)-
     .(za43*zb16*((s14-s23)*
     .((s13-s42-s56)*za13*zb36-
     .(-s13-s42+s56)*za15*zb56)+
     .(za14*zb43+za12*zb23)*
     .(-((s13-s42-s56)*za13*zb16)-
     .(-s13-s42+s56)*za35*zb56)))/
     .(za42*zb56)+
     .(zb12*(-(za13*
     .(za45*zb14+za25*zb12))+
     .za43*(-(za15*zb14)+za35*zb43)+
     .za23*(-(za15*zb12)+za35*zb23))*
     .(-(za13*za56*zb36)+za15*t(i1,i4,i3)))/
     .za56))/
     .((s13**2-2d0*s13*s42+s42**2-2d0*s13*s56-
     .2d0*s42*s56+s56**2)*
     .(-(za12*zb14)-za23*zb43)*
     .(za14*zb43+za12*zb23))

      zFccqqpm=zFccqqpm+
     .zLsm1(-s13,-t(i1,i2,i3),-s23,-t(i1,i2,i3))*
     .((za23**2*(za12*zb26+za13*zb36)**2)/
     .(za12**3*(-(za12*zb14)-za23*zb43)*
     .zb56*t(i1,i2,i3))-
     .(za13**2*(-(za12*zb16)+za23*zb36)**2)/
     .(za12**3*(-(za12*zb14)-za23*zb43)*zb56*
     .t(i1,i2,i3)))+(2d0*zL0d(-t(i1,i2,i3),-s23)*za13*zb12*
     .(-(za13*zb16)-za23*zb26)*
     .(za12*zb26+za13*zb36))/
     .(s23*za12*(-(za12*zb42)-za13*zb43)*
     .zb56*t(i1,i2,i3))+(2d0*zL0d(-t(i1,i2,i3),-s13)*za13*zb12*
     .(-(za13*zb16)-za23*zb26)*
     .(-(za12*zb16)+za23*zb36))/
     .(s13*za12*(-(za12*zb14)-za23*zb43)*
     .zb56*t(i1,i2,i3))+zLsm1_2mh(s14,t(i1,i2,i3),s23,
     .s56)*(-((za45**2*zb12**3)/
     .(za56*zb13*(-(za42*zb12)-
     .za43*zb13)*zb23*t(i1,i2,i3)))-
     .((-(za13*zb14)+za23*zb42)**3*
     .(za12*zb26+za13*zb36)**2)/
     .(za23*(-(za12*zb42)-za13*zb43)**3*
     .(-(za12*zb14)-za23*zb43)*zb56*t(i1,i2,i3)
     .)+(za13**2*(-(za13*zb14)+za23*zb42)*zb46**2*
     .t(i1,i2,i3))/
     .(za23*(-(za12*zb42)-za13*zb43)**3*
     .(-(za12*zb14)-za23*zb43)*zb56))+
     .(zI3m(s13,s42,s56)*zb12*
     .(-(s56*za43*((-s13-s42+s56)*za15*
     .zb46-2d0*za13*za25*zb42*zb36))+
     .2d0*za13*za15*za45*
     .((-s13-s42+s56)*zb14-
     .2d0*za23*zb13*zb42)*zb56-
     .2d0*za13*za35*zb36*
     .(s56*(-s13-s42+s56)+
     .(s13-s42-s56)*t(i1,i2,i3))))/
     .((s13**2-2d0*s13*s42+s42**2-2d0*s13*s56-
     .2d0*s42*s56+s56**2)*
     .(-(za12*zb14)-za23*zb43)*
     .(za14*zb43+za12*zb23))

      zFccqqpm=zFccqqpm+
     .zLsm1(-s42,-t(i4,i2,i3),-s23,-t(i4,i2,i3))*
     .(-(((za25*zb42+za35*zb43)**2*zb23**2)/
     .(za56*zb43**3*(za14*zb43+za12*zb23)*
     .t(i4,i2,i3)))+(zb42**2*
     .(-(za45*zb43)-za25*zb23)**2)/
     .(za56*zb43**3*(za14*zb43+za12*zb23)*
     .t(i4,i2,i3)))-(2d0*zL0d(-t(i4,i2,i3),-s23)*za43*zb42*
     .(za25*zb42+za35*zb43)*
     .(-(za45*zb42)+za35*zb23))/
     .(s23*za56*zb43*
     .(-(za12*zb42)-za13*zb43)*t(i4,i2,i3))-
     .(2d0*zL0d(-t(i4,i2,i3),-s42)*za43*zb42*
     .(-(za45*zb43)-za25*zb23)*
     .(-(za45*zb42)+za35*zb23))/
     .(s42*za56*zb43*(za14*zb43+za12*zb23)*
     .t(i4,i2,i3))+zLsm1_2mh(s14,t(i4,i2,i3),s23,s56)*
     .(-(((za25*zb42+za35*zb43)**2*
     .(za14*zb42-za13*zb23)**3)/
     .(za56*(-(za12*zb42)-za13*zb43)**3*
     .zb23*(za14*zb43+za12*zb23)*t(i4,i2,i3))
     .)-(za43**3*zb16**2)/
     .(za42*za23*(-(za42*zb12)-za43*zb13)*
     .zb56*t(i4,i2,i3))+
     .(za15**2*zb42**2*(za14*zb42-za13*zb23)*
     .t(i4,i2,i3))/
     .(za56*(-(za12*zb42)-za13*zb43)**3*zb23*
     .(za14*zb43+za12*zb23)))+
     .(zI3m(s42,s13,s56)*za43*
     .(-2d0*za56*zb16*zb42*zb46*
     .(-((-s13-s42+s56)*za14)+
     .2d0*za13*za42*zb23)-
     .s56*zb12*((-s13-s42+s56)*za15*
     .zb46-2d0*za13*za25*zb42*zb36)-
     .2d0*za25*zb42*zb26*
     .(s56*(-s13-s42+s56)+
     .(-s13+s42-s56)*t(i4,i2,i3))))/
     .((s13**2-2d0*s13*s42+s42**2-2d0*s13*s56-
     .2d0*s42*s56+s56**2)*
     .(-(za12*zb14)-za23*zb43)*
     .(za14*zb43+za12*zb23))

      zFccqqpm=zFccqqpm+
     .zI3m(s14,s23,s56)*
     .((za25*zb42*(s13*za13*zb16-
     .s43*za43*zb46)-
     .5d0*za12*za43*za25*zb14*zb42*zb26-
     .(-s12+s43)*za23*
     .(-(za15*zb14)+za35*zb43)*zb26-
     .za13*zb14*(-(s23*za15*zb16)-
     .2d0*(-((s12+s42)*za45*zb46)-
     .za42*za35*zb46*zb23)-
     .s43*za25*zb26+
     .(2d0*za42*zb14*zb23*
     .(-(za45*zb46)-za35*zb36))/zb13+
     .3d0*(za45*za23*zb43*zb26+
     .s12*za35*zb36)))/
     .(2d0*(-(za12*zb42)-za13*zb43)*
     .(-(za12*zb13)-za42*zb43)*
     .(-(za12*zb14)-za23*zb43))-
     .(za43*zb14*(za45*zb14+za35*zb13)*
     .zb26)/
     .(zb13*(-(za12*zb14)-za23*zb43)*t(i1,i4,i3))
     .-(za45*zb12**2*(-(za13*zb16)-za23*zb26))/
     .(zb13*t(i1,i2,i3)**2)+
     .(za45*zb12**2*(-(za45*zb42)+
     .za35*zb23))/
     .(2d0*za56*zb13*zb23*t(i1,i2,i3))+
     .(za43*zb12*zb16*
     .(-(za13*zb16)-za23*zb26))/
     .(2d0*za23*zb13*zb56*t(i1,i2,i3))+
     .((za15**2*zb12**3*
     .(-(za12*zb14)-za23*zb43))/
     .(za56*zb13*zb23)+
     .(zb14*((-s14+2d0*s12+s23-s56)*
     .za35-2d0*za43*za56*zb46)*zb26)/
     .zb13+(zb12*
     .(-(za12*za15*zb14*zb12)+
     .za12*za45*zb14*zb42-
     .za15*za23*zb12*zb43+
     .za45*za23*zb42*zb43)*zb26)/
     .(zb13*zb23)+
     .(za43**2*(-((s14-s23-s56)*zb14)+
     .za12*zb14*zb12+
     .za23*zb12*zb43)*zb46**2)/
     .(za23*zb13*zb56)+
     .(4d0*za13*za45*zb12*
     .(-(za13*zb14)+za23*zb42)*zb46)/
     .t(i1,i2,i3)+((-(za13*zb14)+za23*zb42)*
     .(-(za13*zb16)-za23*zb26)**2*
     .(2d0*s23*s56+(s14-s23-s56)*t(i1,i2,i3))
     .)/(za23*zb56*t(i1,i2,i3)**2)+
     .(za35*zb14*(za13*zb16-za43*zb46)*
     .(s13+t(i4,i2,i3)))/(za23*zb13))/
     .(2d0*(-(za12*zb42)-za13*zb43)*
     .(-(za12*zb14)-za23*zb43)))

      zFccqqpm=zFccqqpm+
     .zI3m(s14,s23,s56)*
     .(((s12-s43)*za35*zb23*
     .(za14*zb46+za12*zb26)+
     .za13*(-(s12*za15*zb12)+
     .s42*za45*zb42)*zb36+
     .5d0*za14*za13*za35*zb12*zb43*zb36+
     .za14*zb42*(-(s23*za45*zb46)+
     .(2d0*za14*za23*zb13*
     .(-(za15*zb16)-za25*zb26))/za42+
     .3d0*(-(za12*za35*zb16*zb23)+
     .s43*za25*zb26)-
     .2d0*(-((s13+s43)*za15*zb16)+
     .za15*za23*zb13*zb26)-
     .s12*za35*zb36))/
     .(2d0*(-(za12*zb42)-za13*zb43)*
     .(-(za12*zb13)-za42*zb43)*
     .(za14*zb43+za12*zb23))+
     .(za14*za35*zb12*
     .(-(za14*zb16)+za42*zb26))/
     .(za42*(za14*zb43+za12*zb23)*t(i1,i4,i2))-
     .(za43**2*zb16*(-(za45*zb42)+za35*zb23))/
     .(za42*t(i4,i2,i3)**2)+
     .(za43*za45*zb12*
     .(-(za45*zb42)+za35*zb23))/
     .(2d0*za42*za56*zb23*t(i4,i2,i3))+
     .(za43**2*zb16*(-(za13*zb16)-za23*zb26))/
     .(2d0*za42*za23*zb56*t(i4,i2,i3))+
     .((za15**2*zb12**2*
     .((s14-s23-s56)*za14-
     .za14*za43*zb43-za12*za43*zb23
     .))/(za42*za56*zb23)-
     .(za43*za35*(-(za14*za13*zb16*
     .zb43)+za14*za43*zb43*zb46-
     .za12*za13*zb16*zb23+
     .za12*za43*zb46*zb23))/
     .(za42*za23)+
     .(za43**3*zb46**2*
     .(za14*zb43+za12*zb23))/
     .(za42*za23*zb56)-
     .(za14*za35*((-s14+2d0*s43+s23-s56)*
     .zb26+2d0*za15*zb12*zb56))/za42+
     .(za14*(-(za15*zb12)+za45*zb42)*
     .zb26*(s42+t(i1,i2,i3)))/(za42*zb23)+
     .(4d0*za15*za43*zb16*zb42*
     .(za14*zb42-za13*zb23))/t(i4,i2,i3)+
     .((za14*zb42-za13*zb23)*
     .(-(za45*zb42)+za35*zb23)**2*
     .(2d0*s23*s56+(s14-s23-s56)*t(i4,i2,i3)))/
     .(za56*zb23*t(i4,i2,i3)**2))/
     .(2d0*(-(za12*zb42)-za13*zb43)*
     .(za14*zb43+za12*zb23)))

      return
      end function

      function zFccqqpp(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

      t(j1,j2,j3)=s6(j1,j2)+s6(j2,j3)+s6(j3,j1)

      s13  = s6(i1,i3)
      s14  = s6(i1,i4)
      s56  = s6(i5,i6)
      za45 = zA6(i4,i5)
      za13 = zA6(i1,i3)
      za42 = zA6(i4,i2)
      za23 = zA6(i2,i3)
      za56 = zA6(i5,i6)
      s42  = s6(i4,i2)
      za15 = zA6(i1,i5)
      za14 = zA6(i1,i4)
      za25 = zA6(i2,i5)
      za12 = zA6(i1,i2)
      za43 = zA6(i4,i3)
      za35 = zA6(i3,i5)
      zb12 = zB6(i1,i2)
      zb13 = zB6(i1,i3)
      zb23 = zB6(i2,i3)

      zFccqqpp=
     .((zLsm1(-s13,-t(i1,i4,i3),-s14,-t(i1,i4,i3))+
     .zLsm1_2me(t(i1,i2,i3),t(i1,i4,i3),s13,s56))*za45**2)/
     .(za13*za42*za23*za56)-
     .(zLsm1(-s14,-t(i1,i4,i2),-s42,-t(i1,i4,i2))*za45*
     .(za15*za42-za14*za25))/
     .(za12*za13*za42*za23*za56)+
     .(zLsm1_2me(t(i1,i4,i2),t(i4,i2,i3),s42,s56)*za45*
     .(-(za15*za43)+za14*za35))/
     .(za13**2*za42*za23*za56)+
     .(zLsm1_2me(t(i1,i4,i3),t(i1,i4,i2),s14,s56)*za45*
     .(za43*za25+za42*za35))/
     .(za13*za42*za23**2*za56)-
     .(2d0*zL0d(-t(i4,i2,i3),-s56)*za15*za45*
     .(-(za42*zb12)-za43*zb13))/
     .(s56*za13*za42*za23*za56)-
     .(2d0*zL0d(-t(i4,i2,i3),-s42)*za45*za35*zb23)/
     .(s42*za13*za23*za56)

      return
      end function

      function zFscqqmp(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

      t(i1,i2,i3) = s6(i1,i2) + s6(i2,i3) + s6(i1,i3)

      za25 = zA6(i2,i5)
      za45 = zA6(i4,i5)
      za23 = zA6(i2,i3)
      za42 = zA6(i4,i2)
      za35 = zA6(i3,i5)
      za13 = zA6(i1,i3)
      za56 = zA6(i5,i6)
      zb12 = zB6(i1,i2)
      za43 = zA6(i4,i3)
      zb42 = zB6(i4,i2)
      s23  = s6(i2,i3)
      s14  = s6(i1,i4)
      s56  = s6(i5,i6)
      zb14 = zB6(i1,i4)
      za15 = zA6(i1,i5)
      za12 = zA6(i1,i2)
      zb13 = zB6(i1,i3)
      zb43 = zB6(i4,i3)
      zb23 = zB6(i2,i3)
      za14 = zA6(i1,i4)
      zb36 = zB6(i3,i6)
      zb16 = zB6(i1,i6)
      zb56 = zB6(i5,i6)
      zb46 = zB6(i4,i6)
      zb26 = zB6(i2,i6)

      zFscqqmp=
     .-(za25*(-(za45*za23)+za42*za35))/
     .(2d0*za13*za23*za56*
     .(-(za13*zb12)-za43*zb42))+
     .(zLsm1_2mh(s23,t(i1,i4,i2),s14,s56)*
     .(za43*zb14+za23*zb12)*
     .(-(za15*zb12)-za45*zb42)**2)/
     .(za56*zb42*(-(za13*zb12)-za43*zb42)**3)-
     .(zL0d(-t(i1,i4,i2),-s14)*za42*zb14*
     .(-(za15*zb12)-za45*zb42)**2)/
     .(s14*za56*zb42*
     .(-(za13*zb12)-za43*zb42)**2)+
     .(zzlogRat(-s14,-s56)*za35*
     .(-(za12*zb13)-za42*zb43)*
     .(-(za45*zb43)-za25*zb23))/
     .(2d0*s23*za56*(-(za13*zb12)-za43*zb42)*
     .(-(za12*zb42)-za13*zb43))-
     .(za25*((-s14-s23+s56)*za43+
     .2d0*za14*za23*zb12)*zb36)/
     .((s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)*za13*
     .(-(za13*zb12)-za43*zb42))-
     .(za25*((-s14-s23+s56)*zb12+
     .2d0*za43*zb14*zb23)*zb36)/
     .((s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)*zb42*
     .(-(za13*zb12)-za43*zb42))

      zFscqqmp=zFscqqmp+
     .(za12*(-((za25*(-((za45*
     .(-((s14-s23-s56)*za12*
     .zb14)+
     .(-s14+s23-s56)*za23*zb43
     .))/za56)+zb16*((-s14-s23+s56)*za12+
     .2d0*za14*za23*zb43)))/za23)+
     .(zb36*(-(za45*
     .((-s14-s23+s56)*zb43+
     .2d0*za12*zb14*zb23))-
     .(zb16*((s14-s23-s56)*za14*zb43-
     .(-s14+s23-s56)*za12*zb23))/
     .zb56))/zb23))/
     .(2d0*(s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)*za13*
     .(-(za12*zb42)-za13*zb43))+
     .(zb43*(-((za25*((za45*
     .(-((s14-s23-s56)*za12*zb14)+
     .(-s14+s23-s56)*za23*zb43))/
     .za56-zb16*
     .((-s14-s23+s56)*za12+
     .2d0*za14*za23*zb43)))/za23)+
     .(zb36*(za45*
     .((-s14-s23+s56)*zb43+
     .2d0*za12*zb14*zb23)+
     .(zb16*((s14-s23-s56)*za14*
     .zb43-
     .(-s14+s23-s56)*za12*zb23))/
     .zb56))/zb23))/
     .(2d0*(s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)*zb42*
     .(-(za12*zb42)-za13*zb43))

      zFscqqmp=zFscqqmp-
     .(((s14-s23-s56)*za43*zb14-
     .(-s14+s23-s56)*za23*zb12)*
     .((za25**2*zb23)/za56+
     .(za23*zb36**2)/zb56))/
     .(2d0*(s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)*za23*zb42*
     .(-(za13*zb12)-za43*zb42))+
     .((-((s14-s23-s56)*za14*zb12)+
     .(-s14+s23-s56)*za43*zb23)*
     .((za25**2*zb23)/za56+
     .(za23*zb36**2)/zb56))/
     .(2d0*(s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)*za13*
     .(-(za13*zb12)-za43*zb42)*zb23)-
     .(zL0d(-t(i1,i4,i3),-s14)*za14*zb13*
     .(-(za13*zb16)-za43*zb46)**2)/
     .(s14*za13*(-(za13*zb12)-za43*zb42)**2*
     .zb56)-(zLsm1_2mh(s23,t(i1,i4,i3),s14,s56)*
     .(-(za13*zb16)-za43*zb46)**2*
     .(-(za14*zb12)-za43*zb23))/
     .(za13*(-(za13*zb12)-za43*zb42)**3*zb56)-
     .((zb16*zb23+zb13*zb26)*zb36)/
     .(2d0*zb42*(-(za13*zb12)-za43*zb42)*zb23*
     .zb56)-(zzlogRat(-s14,-s56)*
     .(-(za12*zb13)-za42*zb43)*zb26*
     .(-(za12*zb16)+za23*zb36))/
     .(2d0*s23*(-(za13*zb12)-za43*zb42)*
     .(-(za12*zb42)-za13*zb43)*zb56)

      zFscqqmp=zFscqqmp-
     .(zzlogRat(-s14,-s56)*za14*
     .((-6d0*zb14*((-s14-s23+s56)*za43+
     .2d0*za14*za23*zb12)*
     .(-(za12*zb13)-za42*zb43)*
     .(-(za15*zb16)-za45*zb46))/
     .(s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)**2+
     .((-(za13*zb16)-za43*zb46)*
     .(-(zb16*zb23)-zb13*zb26))/
     .((-(za13*zb12)-za43*zb42)*zb23*
     .zb56)+(-((za25*
     .(2d0*((-s14+s23-s56)*za35*zb13+
     .(-s14-s23+s56)*za56*zb16)
     .-za25*(za43*zb14+za23*zb12)*zb23))/
     .za56)-((s14-s23-s56)*za43*
     .zb14*(za25*zb26-za35*zb36))/
     .(-(za13*zb12)-za43*zb42)-
     .(za42*zb14*
     .(-(za13*zb16)-za43*zb46)*zb36)/
     .zb56+(zb14*zb26*
     .(((s14-s23-s56)*za43*
     .(-(za12*zb13)-za42*zb43)*
     .zb26)/
     .(-(za13*zb12)-za43*zb42)-
     .(2d0*(s14-s23-s56)*za42-
     .za14*za23*zb13)*zb36))/
     .(zb23*zb56)-
     .((s14-s23-s56)*zb12*zb36*
     .(-(za12*zb16)-za42*zb46-
     .za25*zb56))/(zb23*zb56)+
     .za25*(-(za43*zb14*zb36)+
     .za25*zb12*zb56)+
     .4d0*za35*zb14*(za42*zb36+
     .(za43*za25*zb23*zb56)/
     .(-(za13*zb12)-za43*zb42)))/
     .(s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)))/
     .(2d0*za13*(-(za13*zb12)-za43*zb42))

      zFscqqmp=zFscqqmp+
     .(zzlogRat(-s14,-s56)*zb14*
     .(((za45*za23-za42*za35)*
     .(-(za15*zb12)-za45*zb42))/
     .(za23*za56*(-(za13*zb12)-
     .za43*zb42))+
     .(6d0*za14*(-(za12*zb13)-za42*zb43)*
     .(-(za15*zb16)-za45*zb46)*
     .((-s14-s23+s56)*zb12+
     .2d0*za43*zb14*zb23))/
     .(s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)**2+
     .(-((za14*za25*zb13*
     .(-(za15*zb12)-za45*zb42))/za56)
     .-(za14*za35*(((s14-s23-s56)*za35*
     .zb12*
     .(-(za12*zb13)-za42*zb43))/
     .(-(za13*zb12)-za43*zb42)-
     .za25*(2d0*(s14-s23-s56)*zb13-
     .za42*zb14*zb23)))/(za23*za56)
     .+((s14-s23-s56)*za14*zb12*
     .(-(za25*zb26)+za35*zb36))/
     .(-(za13*zb12)-za43*zb42)-
     .((s14-s23-s56)*za43*za25*
     .(-(za15*zb13)-za45*zb43+
     .za56*zb36))/(za23*za56)+
     .zb36*(za14*za25*zb12-
     .za43*za56*zb36)-
     .4d0*za14*zb26*
     .(za25*zb13+
     .(za23*za56*zb12*zb36)/
     .(-(za13*zb12)-za43*zb42))+
     .(zb36*(za23*
     .(-(za14*zb12)-za43*zb23)*zb36
     .+2d0*((-s14+s23-s56)*za42*zb26-
     .(-s14-s23+s56)*za45*zb56)))/zb56)/
     .(s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)))/
     .(2d0*zb42*(-(za13*zb12)-za43*zb42))+
     .(zL1d(-s56,-t(i1,i4,i2))*za35*zb13*zb36)/
     .(zb42*(-(za13*zb12)-za43*zb42)*t(i1,i4,i2))

      zFscqqmp=zFscqqmp-
     .(zL0d(-t(i1,i4,i2),-s56)*za35*zb13*
     .(-(za15*zb12)-za45*zb42)*t(i1,i4,i2))/
     .(s56*za56*zb42*
     .(-(za13*zb12)-za43*zb42)**2)-
     .(zzlogRat(-s23,-s56)*(-((za25*
     .(2d0*((-s14+s23-s56)*za35*zb13+
     .(-s14-s23+s56)*za56*zb16)+
     .(za43*zb14+za23*zb12)*
     .(-(za15*zb13)-za45*zb43))*zb23)/
     .za56)+(6d0*zb14*
     .((-s14-s23+s56)*za43+
     .2d0*za14*za23*zb12)*
     .(-(za12*zb13)-za42*zb43)*
     .(-(za15*zb16)-za45*zb46)*zb23)/
     .(s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)-
     .(zb36*(-(za43*zb14*
     .(-(za12*zb16)-za42*zb46)*zb23)
     .-za23*zb12*(-(za12*zb16)-za42*zb46)*
     .zb23+3d0*(-s14+s23-s56)*za23*
     .zb12*zb36+
     .2d0*za42*zb14*
     .((s14-s23-s56)*zb26-
     .2d0*za35*zb23*zb56)))/zb56+
     .(zb12*(-((za25*za35*zb23)/za56)-
     .(za23*zb26*zb36)/zb56)*
     .(-s14**2+2d0*s14*s23-s23**2+
     .2d0*s14*s56+2d0*s23*s56-s56**2+
     .(-s14+s23-s56)*(t(i1,i4,i2)-t(i1,i4,i3))))/
     .(-(za13*zb12)-za43*zb42)))/
     .(2d0*(s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)*zb42*
     .(-(za13*zb12)-za43*zb42))

      zFscqqmp=zFscqqmp-
     .(zI3m(s14,s23,s56)*za14*
     .((3d0*za23*zb14*
     .(-(za12*zb13)-za42*zb43)*
     .(-(za15*zb16)-za45*zb46)*
     .(-((s14-s23-s56)*za14*zb12)+
     .(-s14+s23-s56)*za43*zb23))/
     .(s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)**2-
     .((-(za12*zb13)-za42*zb43)*
     .(za23*zb12*
     .(-(za15*zb16)-za45*zb46)-
     .3d0*za43*za25*zb14*zb26)-
     .za42*zb14*
     .((s14-s23-s56)*za35-
     .za25*(-(za13*zb12)-
     .za43*zb42)-2d0*za23*za56*zb26)*
     .zb36+za25*zb13*
     .(-((-s14+s23-s56)*za23*zb26)-
     .(-s14-s23+s56)*za35*zb56)+
     .(za43*za35*zb14*
     .(-(za12*zb13*zb26)-
     .za42*zb43*zb26)*(t(i1,i4,i2)-t(i1,i4,i3)))/
     .(-(za13*zb12)-za43*zb42))/
     .(s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)+
     .(za25*zb13*(-(za13*zb16)-
     .za43*zb46))/t(i1,i4,i3)))/
     .(za13*(-(za13*zb12)-za43*zb42))

      zFscqqmp=zFscqqmp+
     .(zL1d(-s56,-t(i1,i4,i3))*za42*za25*zb26)/
     .(za13*(-(za13*zb12)-za43*zb42)*t(i1,i4,i3))+
     .(zL0d(-t(i1,i4,i3),-s56)*za42*
     .(-(za13*zb16)-za43*zb46)*zb26*t(i1,i4,i3))/
     .(s56*za13*(-(za13*zb12)-za43*zb42)**2*
     .zb56)-(zzlogRat(-s23,-s56)*
     .((6d0*za14*za23*
     .(-(za12*zb13)-za42*zb43)*
     .(-(za15*zb16)-za45*zb46)*
     .((-s14-s23+s56)*zb12+
     .2d0*za43*zb14*zb23))/
     .(s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)+
     .(za25*(-(za14*za23*zb12*
     .(-(za15*zb13)-za45*zb43))-
     .3d0*(-s14+s23-s56)*za43*za25*
     .zb23-za43*za23*
     .(-(za15*zb13)-za45*zb43)*zb23-
     .2d0*za14*zb13*
     .((s14-s23-s56)*za35-
     .2d0*za23*za56*zb26)))/za56-
     .(za23*zb36*((-(za12*zb16)-
     .za42*zb46)*
     .(-(za14*zb12)-za43*zb23)+
     .2d0*((-s14+s23-s56)*za42*zb26-
     .(-s14-s23+s56)*za45*zb56)))/
     .zb56+(za43*
     .(-((za25*za35*zb23)/za56)-
     .(za23*zb26*zb36)/zb56)*
     .(-s14**2+2d0*s14*s23-s23**2+
     .2d0*s14*s56+2d0*s23*s56-s56**2+
     .(-s14+s23-s56)*(-t(i1,i4,i2)+t(i1,i4,i3))))/
     .(-(za13*zb12)-za43*zb42)))/
     .(2d0*(s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)*za13*
     .(-(za13*zb12)-za43*zb42))

      zFscqqmp=zFscqqmp+
     .(zI3m(s14,s23,s56)*zb14*
     .((3d0*za14*((s14-s23-s56)*za43*zb14-
     .(-s14+s23-s56)*za23*zb12)*
     .(-(za12*zb13)-za42*zb43)*
     .(-(za15*zb16)-za45*zb46)*zb23)/
     .(s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)**2+
     .(za42*(-(za15*zb12)-za45*zb42)*
     .zb36)/t(i1,i4,i2)-
     .(za42*((-s14+s23-s56)*za35*zb23+
     .(-s14-s23+s56)*za56*zb26)*
     .zb36+(-(za12*zb13)-za42*zb43)*
     .(-(za43*(-(za15*zb16)-za45*zb46)*
     .zb23)+3d0*za14*za35*zb12*zb36)+
     .za14*za25*zb13*
     .((s14-s23-s56)*zb26-
     .(-(za13*zb12)-za43*zb42)*zb36-
     .2d0*za35*zb23*zb56)-
     .(za14*zb12*
     .(-(za12*za35*zb13)-
     .za42*za35*zb43)*zb26*
     .(-t(i1,i4,i2)+t(i1,i4,i3)))/
     .(-(za13*zb12)-za43*zb42))/
     .(s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)))/
     .(zb42*(-(za13*zb12)-za43*zb42))+
     .(za12*zb13*(za12*zb26+za13*zb36)*
     .((zL0d(-t(i1,i2,i3),-s23)*
     .((-2d0*za12*zb46)/
     .(-(za12*zb42)-za13*zb43)+
     .(-(za12*zb16)+za23*zb36)/t(i1,i2,i3)))/
     .s23-(zL1d(-s23,-t(i1,i2,i3))*za12*zb16)/t(i1,i2,i3)**2
     .))/(2d0*za13*(-(za12*zb42)-za13*zb43)*zb56)

      zFscqqmp=zFscqqmp+
     .(za12*za42*zb46**2*
     .((-2d0*zL0d(-s56,-t(i1,i2,i3))*za12)/
     .(-(za12*zb42)-za13*zb43)+
     .(zL1d(-s56,-t(i1,i2,i3))*za42)/t(i1,i2,i3)))/
     .(2d0*za13*za23*(-(za12*zb42)-za13*zb43)*
     .zb56)-(za12*zb36*
     .(-(zb36/zb23)+
     .(-(za12*zb16)+za23*zb36)/t(i1,i2,i3)))/
     .(2d0*za13*(-(za12*zb42)-za13*zb43)*zb56)-
     .(zLsm1_2mh(s14,t(i1,i2,i3),s23,s56)*za12**3*zb46**2*
     .t(i1,i2,i3))/
     .(za13*za23*(-(za12*zb42)-za13*zb43)*
     .*3*
     .zb56)+(zI3m(s14,s23,s56)*zb43*
     .((s14*(-(za12*zb13)-za42*zb43)*
     .(-(za15*zb16)-za45*zb46))/2-
     .za14*zb13*((-(za12*zb14)-
     .za23*zb43)*
     .(-(za15*zb16)-za45*zb46)-
     .za15*za23*zb14*zb36)-
     .(za12*za15*(-(za42*zb12)-
     .za43*zb13)*zb43*
     .(-(zb42*(-(za12*zb16)-za42*zb46))-
     .zb43*(-(za13*zb16)-za43*zb46)-
     .zb14*(za12*zb26+za13*zb36)))/
     .(-(za12*zb42)-za13*zb43)+
     .za15*za42*(zb43*
     .(-(zb12*(-(za12*zb16)-
     .za42*zb46))-
     .zb13*(-(za13*zb16)-za43*zb46)+
     .zb14*(za42*zb26+za43*zb36))-
     .zb14*(-(zb13*
     .(za12*zb26+za13*zb36))-
     .zb43*(za42*zb26+za43*zb36)))+
     .(3d0*s14*(s14-s23-s56)*
     .(-(za12*zb13)-za42*zb43)*
     .(-(za15*zb16)-za45*zb46)*
     .(t(i1,i2,i3)-t(i4,i2,i3)))/
     .(2d0*(s14**2-2d0*s14*s23+s23**2-
     .2d0*s14*s56-2d0*s23*s56+s56**2))))/
     .((s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)*zb42*
     .(-(za12*zb42)-za13*zb43))

      zFscqqmp=zFscqqmp+
     .(zzlogRat(-s23,-s56)*zb43*
     .(-2d0*(-(za12*zb13)+za42*zb43)*
     .(-(za15*zb16)-za45*zb46)+
     .4d0*(-(za14*za25*zb13*zb46)+
     .za12*za45*zb14*zb36)+
     .(-(za12*zb13)-za42*zb43+
     .(4d0*s56*za12*zb43)/
     .(-(za12*zb42)-za13*zb43))*
     .(-((za15*za45*zb14)/za56)+
     .(za14*zb16*zb46)/zb56)-
     .(-s14+s23-s56)*
     .((za15*za25*zb13)/za56+
     .(2d0*za15*zb43*
     .(za15*za42*zb14-
     .za15*za23*zb13-
     .za42*za56*zb46))/
     .(za56*(-(za12*zb42)-za13*zb43))-
     .(za42*zb46*zb36)/zb56)+
     .(8d0*s14*za15*za25*zb43*zb56)/
     .(-(za12*zb42)-za13*zb43)+
     .(3d0*(-s14-s23+s56)*
     .(-(za12*zb13)-za42*zb43)*
     .(-(za15*zb16)-za45*zb46)*
     .(t(i1,i2,i3)-t(i4,i2,i3)))/
     .(s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)))/
     .(2d0*(s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)*zb42*
     .(-(za12*zb42)-za13*zb43))-
     .(za42*zb43*(za25*zb42+za35*zb43)*
     .((zL0d(-t(i4,i2,i3),-s23)*
     .((-2d0*za15*zb43)/
     .(-(za12*zb42)-za13*zb43)+
     .(-(za45*zb43)-za25*zb23)/t(i4,i2,i3)))/
     .s23-(zL1d(-s23,-t(i4,i2,i3))*za45*zb43)/t(i4,i2,i3)**2
     .))/(2d0*za56*zb42*(-(za12*zb42)-za13*zb43))

      zFscqqmp=zFscqqmp+
     .(za15**2*zb13*zb43*
     .((-2d0*zL0d(-s56,-t(i4,i2,i3))*zb43)/
     .(-(za12*zb42)-za13*zb43)+
     .(zL1d(-s56,-t(i4,i2,i3))*zb13)/t(i4,i2,i3)))/
     .(2d0*za56*zb42*(-(za12*zb42)-za13*zb43)*
     .zb23)+(za25*zb43*
     .(za25/za23+(-(za45*zb43)-za25*zb23)/
     .t(i4,i2,i3)))/
     .(2d0*za56*zb42*(-(za12*zb42)-za13*zb43))-
     .(zLsm1_2mh(s14,t(i4,i2,i3),s23,s56)*za15**2*zb43**3*
     .t(i4,i2,i3))/
     .(za56*zb42*(-(za12*zb42)-za13*zb43)**3*
     .zb23)+(zzlogRat(-s14,-s56)*zb43*
     .(((s14-s23-s56)*zb14*
     .(-(za45*(-(za15*za42)+za14*za25)*
     .zb43)+(za12*za15**2*
     .(-(za42*zb12)-za43*zb13)*zb43
     .)/(-(za12*zb42)-za13*zb43)-
     .za15*za45*
     .(-(za12*zb13)-za42*zb43)))/
     .((s14**2-2d0*s14*s23+s23**2-
     .2d0*s14*s56-2d0*s23*s56+s56**2)*za56)
     .-(2d0*(-s14-s23+s56)*za14*za25*
     .(zb16*zb43+zb13*zb46))/
     .(s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)-
     .(za12*zb43*(-((za15*za45*zb14)/
     .za56)-(za14*zb16*zb46)/zb56))/
     .(-(za12*zb42)-za13*zb43)-
     .(za42*zb46*zb36)/zb56+
     .((s14-s23-s56)*za14*
     .(zb13*zb46*
     .(-(za12*zb16)-za42*zb46)-
     .za12*zb43*
     .(zb16**2+((-(za42*zb12)-
     .za43*zb13)*zb46**2)/
     .(-(za12*zb42)-za13*zb43))+
     .zb14*zb36*
     .(-(za42*zb46)+za23*zb36)))/
     .((s14**2-2d0*s14*s23+s23**2-
     .2d0*s14*s56-2d0*s23*s56+s56**2)*zb56)
     .+(6d0*s14*s23*(-(za12*zb13)-za42*zb43)*
     .(-(za15*zb16)-za45*zb46)*
     .(t(i1,i2,i3)-t(i4,i2,i3)))/
     .(s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)**2-
     .(s14*((za25**2*zb23)/za56+
     .(za23*zb36**2)/zb56)*t(i4,i2,i3))/
     .(s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)))/
     .(2d0*s23*zb42*(-(za12*zb42)-za13*zb43))

      zFscqqmp=zFscqqmp+
     .(zzlogRat(-s14,-s56)*za12*
     .((za15*za25*zb13)/za56+
     .((s14-s23-s56)*zb14*
     .(za15*za42*
     .(-(za15*zb13)-za45*zb43)-
     .za12*zb43*
     .(za45**2+(za15**2*
     .(-(za42*zb12)-za43*zb13))/
     .(-(za12*zb42)-za13*zb43))-
     .za14*za25*
     .(-(za15*zb13)-za25*zb23)))/
     .((s14**2-2d0*s14*s23+s23**2-
     .2d0*s14*s56-2d0*s23*s56+s56**2)*za56)
     .+(2d0*(-s14-s23+s56)*
     .(za15*za42+za12*za45)*zb14*zb36
     .)/(s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)-
     .(za12*zb43*(-((za15*za45*zb14)/
     .za56)-(za14*zb16*zb46)/zb56))/
     .(-(za12*zb42)-za13*zb43)+
     .((s14-s23-s56)*za14*
     .(-(zb16*(-(za12*zb13)-za42*zb43)*
     .zb46)+(za12*
     .(-(za42*zb12)-za43*zb13)*
     .zb43*zb46**2)/
     .(-(za12*zb42)-za13*zb43)-
     .za12*zb16*
     .(-(zb13*zb46)-zb14*zb36)))/
     .((s14**2-2d0*s14*s23+s23**2-
     .2d0*s14*s56-2d0*s23*s56+s56**2)*zb56)
     .-(s14*((za25**2*zb23)/za56+
     .(za23*zb36**2)/zb56)*t(i1,i2,i3))/
     .(s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)+
     .(6d0*s14*s23*(-(za12*zb13)-za42*zb43)*
     .(-(za15*zb16)-za45*zb46)*
     .(-t(i1,i2,i3)+t(i4,i2,i3)))/
     .(s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)**2))/
     .(2d0*s23*za13*(-(za12*zb42)-za13*zb43))

      zFscqqmp=zFscqqmp+
     .(zI3m(s14,s23,s56)*za12*
     .(-((za12*(-(za42*zb12)-za43*zb13)*
     .zb43*(-(za12*
     .(-(za15*zb12)-za45*zb42))-
     .za13*(-(za15*zb13)-
     .za45*zb43)+
     .za14*(za25*zb42+za35*zb43))*
     .zb46)/(-(za12*zb42)-za13*zb43))+
     .zb13*(za12*(-(za14*
     .(za25*zb12+za35*zb13))-
     .za42*(-(za15*zb12)-za45*zb42)-
     .za43*(-(za15*zb13)-za45*zb43))+
     .za14*(-(za12*
     .(za25*zb12+za35*zb13))-
     .za42*(za25*zb42+za35*zb43)))*
     .zb46+(s14*
     .(-(za12*zb13)-za42*zb43)*
     .(-(za15*zb16)-za45*zb46))/2+
     .za42*zb14*(-(za14*za25*zb46*zb23)+
     .(-(za15*zb16)-za45*zb46)*
     .(za14*zb43+za12*zb23))+
     .(3d0*s14*(s14-s23-s56)*
     .(-(za12*zb13)-za42*zb43)*
     .(-(za15*zb16)-za45*zb46)*
     .(-t(i1,i2,i3)+t(i4,i2,i3)))/
     .(2d0*(s14**2-2d0*s14*s23+s23**2-
     .2d0*s14*s56-2d0*s23*s56+s56**2))))/
     .((s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)*za13*
     .(-(za12*zb42)-za13*zb43))

      zFscqqmp=zFscqqmp+
     .(zzlogRat(-s23,-s56)*za12*
     .(-2d0*(za12*zb13-za42*zb43)*
     .(-(za15*zb16)-za45*zb46)-
     .(8d0*s14*za12*za56*zb46*zb36)/
     .(-(za12*zb42)-za13*zb43)+
     .4d0*(-(za14*za25*zb16*zb43)+
     .za15*za42*zb14*zb36)+
     .(-(za12*zb13)-za42*zb43+
     .(4d0*s56*za12*zb43)/
     .(-(za12*zb42)-za13*zb43))*
     .((za15*za45*zb14)/za56-
     .(za14*zb16*zb46)/zb56)-
     .(-s14+s23-s56)*
     .((za15*za25*zb13)/za56-
     .(za42*zb46*zb36)/zb56-
     .(2d0*za12*zb46*
     .(-(za14*zb13*zb46)+
     .za42*zb46*zb23+
     .za15*zb13*zb56))/
     .((-(za12*zb42)-za13*zb43)*zb56))+
     .(3d0*(-s14-s23+s56)*
     .(-(za12*zb13)-za42*zb43)*
     .(-(za15*zb16)-za45*zb46)*
     .(-t(i1,i2,i3)+t(i4,i2,i3)))/
     .(s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)))/
     .(2d0*(s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)*za13*
     .(-(za12*zb42)-za13*zb43))

      return
      end function

      function zFscqqpm(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

      t(j1,j2,j3)=s6(j1,j2)+s6(j2,j3)+s6(j3,j1)

      s14  = s6(i1,i4)
      s13  = s6(i1,i3)
      zb13 = zB6(i1,i3)
      za15 = zA6(i1,i5)
      zb14 = zB6(i1,i4)
      za35 = zA6(i3,i5)
      zb43 = zB6(i4,i3)
      za56 = zA6(i5,i6)
      za12 = zA6(i1,i2)
      za42 = zA6(i4,i2)
      za23 = zA6(i2,i3)
      za43 = zA6(i4,i3)
      za45 = zA6(i4,i5)
      za13 = zA6(i1,i3)
      zb12 = zB6(i1,i2)
      zb42 = zB6(i4,i2)
      za14 = zA6(i1,i4)
      zb23 = zB6(i2,i3)
      s23  = s6(i2,i3)
      s56  = s6(i5,i6)
      zb16 = zB6(i1,i6)
      za25 = zA6(i2,i5)
      s42  = s6(i4,i2)
      zb26 = zB6(i2,i6)
      zb56 = zB6(i5,i6)
      zb36 = zB6(i3,i6)
      zb46 = zB6(i4,i6)
      zb15 = zB6(i1,i5)
      za46 = zA6(i4,i6)

      zFscqqpm=
     .-((zLsm1(-s14,-t(i1,i4,i3),-s13,-t(i1,i4,i3))*zb13*
     .(-(za15*zb14)+za35*zb43)**2)/
     .(za56*zb43**2*(-(za12*zb13)-za42*zb43)*
     .(-(za12*zb14)-za23*zb43)))-
     .(za43*(-((zL0d(-t(i1,i4,i3),-s14)*zb14*
     .(-(za15*zb13)-za45*zb43)**2)/
     .(s14*(-(za12*zb13)-za42*zb43)))-
     .(zL0d(-t(i1,i4,i3),-s13)*zb13*
     .(-(za15*zb14)+za35*zb43)**2)/
     .(s13*(-(za12*zb14)-za23*zb43))))/
     .(za56*zb43*(-(za12*zb13)-za42*zb43))+
     .(za15*za35*(-(za13*zb12)-za43*zb42))/
     .(2d0*za23*za56*(-(za12*zb42)-za13*zb43)*
     .(za14*zb43+za12*zb23))+
     .(za15*(-(za13*zb12)-za43*zb42)*
     .((za13*((s14-s23-s56)*za45*zb14+
     .(-s14-s23+s56)*za56*zb16))/za23
     .-((-s14-s23+s56)*za45+
     .2d0*za14*(za25*zb12+za35*zb13))*zb42))/
     .(2d0*(s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)*za56*
     .(-(za12*zb42)-za13*zb43)*
     .(za14*zb43+za12*zb23))-
     .(zzlogRat(-s14,-s56)*za25*
     .(-(za13*zb12)-za43*zb42)*
     .(-(za45*zb42)+za35*zb23))/
     .(2d0*s23*za56*(-(za12*zb42)-za13*zb43)*
     .(-(za12*zb13)-za42*zb43))-
     .(za25*(-(za13*zb12)-za43*zb42)*
     .((za25*zb14*((-s14-s23+s56)*za43+
     .2d0*za14*za23*zb12))/za23-
     .((-(za15*zb13)-za45*zb43)*
     .((-s14-s23+s56)*zb12+
     .2d0*za43*zb14*zb23))/zb23))/
     .(2d0*(s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)*za56*
     .(-(za12*zb13)-za42*zb43)*
     .(-(za12*zb14)-za23*zb43))+
     .((-(za42*zb12)-za43*zb13)*
     .(((s13-s42-s56)*za45*za35)/
     .(za42*za56)+
     .((-s13-s42+s56)*za45*zb16)/
     .(2d0*za42*zb13)+za35*zb26))/
     .(2d0*(s13**2-2d0*s13*s42+s42**2-2d0*s13*s56-
     .2d0*s42*s56+s56**2)*
     .(-(za12*zb13)-za42*zb43))+
     .((-(za42*zb12)-za43*zb13)*
     .(((-s13-s42+s56)*za45*zb16)/
     .(2d0*za42*zb13)+za35*zb26-
     .((-s13+s42-s56)*zb16*zb26)/
     .(zb13*zb56)))/
     .(2d0*(s13**2-2d0*s13*s42+s42**2-2d0*s13*s56-
     .2d0*s42*s56+s56**2)*
     .(-(za12*zb13)-za42*zb43))+
     .((za43**2*za25**2*zb14)/(za42*za23*za56)-
     .(za25*zb12*zb26)/zb23+
     .((za42*zb14-za23*zb13)*zb16*zb26)/
     .(zb13*zb56))/
     .(2d0*(-(za12*zb13)-za42*zb43)*
     .(-(za12*zb14)-za23*zb43))+
     .(-((za45*za35*(-(za14*zb13)+za42*zb23))/
     .(za42*za56))+
     .(za43*za35*zb36)/za23-
     .(za14*zb12**2*zb36**2)/(zb13*zb23*zb56))/
     .(2d0*(-(za12*zb13)-za42*zb43)*
     .(za14*zb43+za12*zb23))+
     .((-(za13*zb12)-za43*zb42)*zb46*zb26)/
     .(2d0*(-(za12*zb42)-za13*zb43)*
     .(-(za12*zb14)-za23*zb43)*zb23*zb56)+
     .(zLsm1(-s14,-t(i1,i4,i2),-s42,-t(i1,i4,i2))*za42*
     .(za14*zb46+za12*zb26)**2)/
     .(za12**2*(-(za12*zb13)-za42*zb43)*
     .(za14*zb43+za12*zb23)*zb56)+
     .(zb12*((zL0d(-t(i1,i4,i2),-s14)*za14*
     .(-(za12*zb16)-za42*zb46)**2)/
     .(s14*(-(za12*zb13)-za42*zb43))-
     .(zL0d(-t(i1,i4,i2),-s42)*za42*
     .(za14*zb46+za12*zb26)**2)/
     .(s42*(za14*zb43+za12*zb23))))/
     .(za12*(-(za12*zb13)-za42*zb43)*zb56)


      zFscqqpm=zFscqqpm+
     .(zzlogRat(-s14,-s56)*(-(za13*zb12)-za43*zb42)*
     .(-(za13*zb16)-za23*zb26)*zb36)/
     .(2d0*s23*(-(za12*zb42)-za13*zb43)*
     .(-(za12*zb13)-za42*zb43)*zb56)+
     .((-(za13*zb12)-za43*zb42)*zb36*
     .((((-s14-s23+s56)*za43+
     .2d0*za14*za23*zb12)*
     .(-(za12*zb16)-za42*zb46))/za23+
     .(za14*((-s14-s23+s56)*zb12+
     .2d0*za43*zb14*zb23)*zb36)/zb23))/
     .(2d0*(s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)*
     .(-(za12*zb13)-za42*zb43)*
     .(za14*zb43+za12*zb23)*zb56)-
     .((-(za13*zb12)-za43*zb42)*zb46*
     .(-(za13*((-s14-s23+s56)*zb16-
     .2d0*zb14*(za42*zb26+za43*zb36)))-
     .(zb42*(-((s14-s23-s56)*za14*zb16)-
     .(-s14-s23+s56)*za45*zb56))/zb23)
     .)/(2d0*(s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)*
     .(-(za12*zb42)-za13*zb43)*
     .(-(za12*zb14)-za23*zb43)*zb56)+
     .zI3m(s13,s42,s56)*
     .(-(((-(za42*zb12)-za43*zb13)*
     .(-(za13*za45*zb16*zb42)+
     .(za35*(-2d0*za45*zb42-za56*zb26)*
     .zb56)/2))/
     .((s13**2-2d0*s13*s42+s42**2-2d0*s13*s56-
     .2d0*s42*s56+s56**2)*
     .(-(za12*zb13)-za42*zb43)))+
     .(3d0*za13*(-(za42*zb12)-za43*zb13)*zb42*
     .(((-s13-s42+s56)*za45*za56*zb16*
     .zb56)/2+za35*zb13*
     .(-(s56*za42*zb26)+
     .(s13-s42-s56)*za45*zb56)))/
     .((s13**2-2d0*s13*s42+s42**2-2d0*s13*s56-
     .2d0*s42*s56+s56**2)**2*
     .(-(za12*zb13)-za42*zb43)))



      zFscqqpm=zFscqqpm+
     .zI3m(s42,s13,s56)*
     .((3d0*za13*(-(za42*zb12)-za43*zb13)*zb42*
     .(za42*(-(s56*za35*zb13)-
     .(-s13+s42-s56)*za56*zb16)*
     .zb26+((-s13-s42+s56)*za45*
     .za56*zb16*zb56)/2))/
     .((s13**2-2d0*s13*s42+s42**2-2d0*s13*s56-
     .2d0*s42*s56+s56**2)**2*
     .(-(za12*zb13)-za42*zb43))-
     .((-(za42*zb12)-za43*zb13)*
     .(-(za13*za45*zb16*zb42)-
     .(za56*zb26*
     .(-2d0*za13*zb16+za35*zb56))/2))/
     .((s13**2-2d0*s13*s42+s42**2-2d0*s13*s56-
     .2d0*s42*s56+s56**2)*
     .(-(za12*zb13)-za42*zb43)))+
     .zzlogRat(-s13,-s56)*((4d0*za43*za45*zb16*zb42+
     .(2d0*za35*(-(za42*zb12)-za43*zb13)*
     .((s13-s42-s56)*za45+
     .2d0*za42*za56*zb26))/(za42*za56)+
     .((s13-s42-s56)*za43+
     .za13*za45*zb15+za13*za46*zb16)*
     .(-((za45*(za45*zb14+za25*zb12))/
     .(za42*za56))+(zb16*zb26)/zb56))/
     .(2d0*(s13**2-2d0*s13*s42+s42**2-2d0*s13*s56-
     .2d0*s42*s56+s56**2)*
     .(-(za12*zb13)-za42*zb43))+
     .(3d0*za13*(-(za42*zb12)-za43*zb13)*
     .(((s13-s42-s56)*za35*zb13+
     .(-s13-s42+s56)*za56*zb16)*zb26
     .-za45*zb42*((-s13+s42-s56)*zb16-
     .2d0*za35*zb13*zb56)))/
     .((s13**2-2d0*s13*s42+s42**2-2d0*s13*s56-
     .2d0*s42*s56+s56**2)**2*
     .(-(za12*zb13)-za42*zb43)))+
     .zzlogRat(-s42,-s56)*((3d0*(-(za42*zb12)-za43*zb13)*
     .zb42*(-(za13*zb16*
     .((s13-s42-s56)*za45+
     .2d0*za42*za56*zb26))+
     .za35*((-s13+s42-s56)*za42*zb26-
     .(-s13-s42+s56)*za45*zb56)))/
     .((s13**2-2d0*s13*s42+s42**2-2d0*s13*s56-
     .2d0*s42*s56+s56**2)**2*
     .(-(za12*zb13)-za42*zb43))+
     .(4d0*za13*za45*zb12*zb16+
     .((-s13+s42-s56)*zb12+
     .za45*zb15*zb42+za46*zb16*zb42)*
     .(-((za45*za35)/za56)+
     .(zb16*(-(za14*zb16)+za43*zb36))/
     .(zb13*zb56))-
     .(2d0*(-(za42*zb12)-za43*zb13)*zb26*
     .((-s13+s42-s56)*zb16-
     .2d0*za35*zb13*zb56))/(zb13*zb56))/
     .(2d0*(s13**2-2d0*s13*s42+s42**2-2d0*s13*s56-
     .2d0*s42*s56+s56**2)*
     .(-(za12*zb13)-za42*zb43)))

      zFscqqpm=zFscqqpm
     .+(zzlogRat(-s14,-s56)*za14*
     .((-6d0*zb14*((-s14-s23+s56)*za42-
     .2d0*za14*za23*zb13)*
     .(-(za13*zb12)-za43*zb42)*
     .(-(za15*zb16)-za45*zb46))/
     .(s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)**2-
     .((-(za12*zb16)-za42*zb46)*
     .(zb16*zb23-zb12*zb36))/
     .((-(za12*zb13)-za42*zb43)*zb23*
     .zb56)+(-((za35*
     .(2d0*((-s14+s23-s56)*za25*
     .zb12+
     .(-s14-s23+s56)*za56*zb16)
     .+za35*(za42*zb14-za23*zb13)*zb23))/
     .za56)-((s14-s23-s56)*za42*
     .zb14*(-(za25*zb26)+za35*zb36))/
     .(-(za12*zb13)-za42*zb43)-
     .(za43*zb14*
     .(-(za12*zb16)-za42*zb46)*zb26)/
     .zb56-(zb14*zb36*
     .(-((2d0*(s14-s23-s56)*za43+
     .za14*za23*zb12)*zb26)+
     .((s14-s23-s56)*za42*
     .(-(za13*zb12)-za43*zb42)*
     .zb36)/
     .(-(za12*zb13)-za42*zb43)))/
     .(zb23*zb56)+
     .((s14-s23-s56)*zb13*zb26*
     .(-(za13*zb16)-za43*zb46-
     .za35*zb56))/(zb23*zb56)+
     .za35*(-(za42*zb14*zb26)+
     .za35*zb13*zb56)+
     .4d0*za25*zb14*
     .(za43*zb26-
     .(za42*za35*zb23*zb56)/
     .(-(za12*zb13)-za42*zb43)))/
     .(s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)))/
     .(2d0*za12*(-(za12*zb13)-za42*zb43))



      zFscqqpm=zFscqqpm
     .-(zzlogRat(-s14,-s56)*zb14*
     .(-(((-(za45*za23)-za43*za25)*
     .(-(za15*zb13)-za45*zb43))/
     .(za23*za56*
     .(-(za12*zb13)-za42*zb43)))+
     .(6d0*za14*(-(za13*zb12)-za43*zb42)*
     .(-(za15*zb16)-za45*zb46)*
     .((-s14-s23+s56)*zb13-
     .2d0*za42*zb14*zb23))/
     .(s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)**2+
     .(-((za14*za35*zb12*
     .(-(za15*zb13)-za45*zb43))/za56)
     .+(za14*za25*(((s14-s23-s56)*za25*
     .zb13*
     .(-(za13*zb12)-za43*zb42))/
     .(-(za12*zb13)-za42*zb43)-
     .za35*(2d0*(s14-s23-s56)*zb12+
     .za43*zb14*zb23)))/(za23*za56)
     .+((s14-s23-s56)*za42*za35*
     .(-(za15*zb12)-za45*zb42+
     .za56*zb26))/(za23*za56)+
     .zb26*(za14*za35*zb13-
     .za42*za56*zb26)-
     .4d0*za14*(za35*zb12-
     .(za23*za56*zb13*zb26)/
     .(-(za12*zb13)-za42*zb43))*zb36+
     .((s14-s23-s56)*za14*zb13*
     .(za25*zb26-za35*zb36))/
     .(-(za12*zb13)-za42*zb43)+
     .(zb26*(-(za23*
     .(-(za14*zb13)+za42*zb23)*zb26)+
     .2d0*((-s14+s23-s56)*za43*zb36-
     .(-s14-s23+s56)*za45*zb56)))/zb56)/
     .(s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)))/
     .(2d0*zb43*(-(za12*zb13)-za42*zb43))


      zFscqqpm=zFscqqpm+
     .(zLsm1_2mh(s23,t(i1,i4,i2),s14,s56)*za42*zb36**2*
     .t(i1,i4,i2)**2)/
     .((-(za12*zb13)-za42*zb43)**3*
     .(za14*zb43+za12*zb23)*zb56)+
     .(zL0d(-t(i1,i4,i2),-s56)*za43*zb36**2*t(i1,i4,i2)**2)/
     .(s56*(-(za12*zb13)-za42*zb43)**2*
     .(za14*zb43+za12*zb23)*zb56)+
     .(zzlogRat(-s23,-s56)*((-6d0*za14*za23*
     .(-(za13*zb12)-za43*zb42)*
     .(-(za15*zb16)-za45*zb46)*
     .((-s14-s23+s56)*zb13-
     .2d0*za42*zb14*zb23))/
     .(s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)+
     .(za35*(za14*za23*zb13*
     .(-(za15*zb12)-za45*zb42)+
     .3d0*(-s14+s23-s56)*za42*za35*
     .zb23-za42*za23*
     .(-(za15*zb12)-za45*zb42)*zb23-
     .2d0*za14*zb12*
     .((s14-s23-s56)*za25+
     .2d0*za23*za56*zb36)))/za56+
     .(za23*zb26*((-(za13*zb16)-
     .za43*zb46)*
     .(-(za14*zb13)+za42*zb23)+
     .2d0*((-s14+s23-s56)*za43*zb36-
     .(-s14-s23+s56)*za45*zb56)))/
     .zb56+(za42*
     .((za25*za35*zb23)/za56+
     .(za23*zb26*zb36)/zb56)*
     .(-s14**2+2d0*s14*s23-s23**2+
     .2d0*s14*s56+2d0*s23*s56-s56**2+
     .(-s14+s23-s56)*(t(i1,i4,i2)-t(i1,i4,i3))))/
     .(-(za12*zb13)-za42*zb43)))/
     .(2d0*(s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)*za12*
     .(-(za12*zb13)-za42*zb43))-
     .(zLsm1_2mh(s23,t(i1,i4,i3),s14,s56)*za25**2*zb13*
     .t(i1,i4,i3)**2)/
     .(za56*(-(za12*zb13)-za42*zb43)**3*
     .(-(za12*zb14)-za23*zb43))-
     .(zL0d(-t(i1,i4,i3),-s56)*za25**2*zb12*t(i1,i4,i3)**2)/
     .(s56*za56*(-(za12*zb13)-za42*zb43)**2*
     .(-(za12*zb14)-za23*zb43))+
     .(zzlogRat(-s23,-s56)*((za35*
     .(2d0*((-s14+s23-s56)*za25*zb12+
     .(-s14-s23+s56)*za56*zb16)+
     .(za42*zb14-za23*zb13)*
     .(-(za15*zb12)-za45*zb42))*zb23)/
     .za56-(6d0*zb14*
     .((-s14-s23+s56)*za42-
     .2d0*za14*za23*zb13)*
     .(-(za13*zb12)-za43*zb42)*
     .(-(za15*zb16)-za45*zb46)*zb23)/
     .(s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)-
     .(zb26*(za42*zb14*
     .(-(za13*zb16)-za43*zb46)*zb23-
     .za23*zb13*
     .(-(za13*zb16)-za43*zb46)*zb23-
     .3d0*(-s14+s23-s56)*za23*zb13*
     .zb26+2d0*za43*zb14*
     .((s14-s23-s56)*zb36+
     .2d0*za25*zb23*zb56)))/zb56+
     .(zb13*((za25*za35*zb23)/za56+
     .(za23*zb26*zb36)/zb56)*
     .(-s14**2+2d0*s14*s23-s23**2+
     .2d0*s14*s56+2d0*s23*s56-s56**2+
     .(-s14+s23-s56)*(-t(i1,i4,i2)+t(i1,i4,i3))))/
     .(-(za12*zb13)-za42*zb43)))/
     .(2d0*(s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)*zb43*
     .(-(za12*zb13)-za42*zb43))+
     .(zLs1d(-s13,-t(i1,i2,i3),-s23,-t(i1,i2,i3))*za13**2*zb12**2*
     .(-(za12*zb16)+za23*zb36)**2)/
     .(za12*(-(za12*zb14)-za23*zb43)*zb56*
     .t(i1,i2,i3)**3)-(zb12**2*
     .(za12*zb26+za13*zb36)*
     .(-(za12*zb16)+za23*zb36))/
     .(2d0*za12*zb13*(-(za12*zb14)-za23*zb43)*
     .zb23*zb56*t(i1,i2,i3))


      zFscqqpm=zFscqqpm
     .-(za13*zb12**2*((zL1d(-t(i1,i2,i3),-s23)*za23*
     .(za12*zb26+za13*zb36)**2)/
     .(s23**2*(-(za12*zb42)-za13*zb43))+
     .(zL1d(-t(i1,i2,i3),-s13)*za13*
     .(-(za12*zb16)+za23*zb36)**2)/
     .(s13**2*(-(za12*zb14)-za23*zb43))))/
     .(2d0*za12*zb56*t(i1,i2,i3))-
     .(zLsm1_2mh(s14,t(i1,i2,i3),s23,s56)*za13**2*
     .(-(za13*zb14)+za23*zb42)*zb46**2*t(i1,i2,i3))/
     .(za23*(-(za12*zb42)-za13*zb43)**3*
     .(-(za12*zb14)-za23*zb43)*zb56)+
     .(zL0d(-t(i1,i2,i3),-s23)*za13**2*zb12*zb46**2*t(i1,i2,i3))/
     .(s23*(-(za12*zb42)-za13*zb43)**2*
     .(-(za12*zb14)-za23*zb43)*zb56)+
     .(za13*za43*(-(za13*zb14)+za23*zb42)*
     .zb46**2*((zL1d(-s56,-t(i1,i2,i3))*za43)/
     .(za13*t(i1,i2,i3)**2)-
     .(2d0*zL0d(-s56,-t(i1,i2,i3)))/
     .((-(za12*zb42)-za13*zb43)*t(i1,i2,i3)))*
     .t(i1,i2,i3))/
     .(2d0*za23*(-(za12*zb42)-za13*zb43)*
     .(-(za12*zb14)-za23*zb43)*zb56)+
     .(zzlogRat(-s42,-s56)*zb12*zb36*
     .(za12*za35+((-(za12*zb16)+
     .za23*zb36)*t(i1,i4,i2)-
     .(-(za12*zb16)-za42*zb46)*t(i1,i2,i3))/
     .(zb13*zb56)))/
     .(2d0*(-(za12*zb13)-za42*zb43)*
     .(-(za12*zb14)-za23*zb43)*
     .(za14*zb43+za12*zb23))

      ztmp=
     .-((zLsm1_2mh(s42,t(i1,i4,i3),s13,s56)*
     .(za42*zb14-za23*zb13)*
     .(-(za15*zb14)+za35*zb43)**2)/
     .(za56*zb43*
     .(-(za12*zb14)-za23*zb43)**3))

      ztmp=ztmp
     .-(zL0d(-t(i1,i4,i3),-s13)*za43*zb13*
     .(-(za15*zb14)+za35*zb43)**2)/
     .(s13*za56*zb43*
     .(-(za12*zb14)-za23*zb43)**2)+
     .(za45*((-s13-s42+s56)*zb14-
     .2d0*za23*zb13*zb42)*zb26)/
     .((s13**2-2d0*s13*s42+s42**2-
     .2d0*s13*s56-2d0*s42*s56+s56**2)*zb43*
     .(-(za12*zb14)-za23*zb43))+
     .((-((-s13+s42-s56)*za42*zb14)-
     .(s13-s42-s56)*za23*zb13)*
     .((za45**2*zb42)/za56+
     .(za42*zb26**2)/zb56))/
     .(2d0*(s13**2-2d0*s13*s42+s42**2-
     .2d0*s13*s56-2d0*s42*s56+s56**2)*za42*
     .zb43*(-(za12*zb14)-za23*zb43))


      ztmp=ztmp
     .-(zzlogRat(-s13,-s56)*zb13*
     .(((za43*za25+za42*za35)*
     .(-(za15*zb14)+za35*zb43))/
     .(za42*za56*
     .(-(za12*zb14)-za23*zb43))+
     .(6d0*za13*((-s13-s42+s56)*zb14-
     .2d0*za23*zb13*zb42)*
     .(-(za14*zb12)-za43*zb23)*
     .(-(za15*zb16)-za35*zb36))/
     .(s13**2-2d0*s13*s42+s42**2-
     .2d0*s13*s56-2d0*s42*s56+s56**2)**2+
     .(-((za13*za45*zb12*
     .(-(za15*zb14)+za35*zb43))/
     .za56)-
     .(za13*za25*
     .(-(za45*
     .(2d0*(s13-s42-s56)*zb12+
     .za43*zb13*zb42))+
     .((s13-s42-s56)*za25*
     .zb14*
     .(-(za14*zb12)-za43*zb23)
     .)/(-(za12*zb14)-za23*zb43)))/(za42*za56)
     .+((s13-s42-s56)*za13*zb14*
     .(-(za45*zb46)+za25*zb26))/
     .(-(za12*zb14)-za23*zb43)+
     .((s13-s42-s56)*za45*za23*
     .(-(za15*zb12)+za35*zb23+
     .za56*zb26))/(za42*za56)+
     .zb26*(za13*za45*zb14+
     .za23*za56*zb26)-
     .4d0*za13*zb46*
     .(za45*zb12+
     .(za42*za56*zb14*zb26)/
     .(-(za12*zb14)-za23*zb43))+
     .(zb26*(za42*
     .(-(za13*zb14)+za23*zb42)*
     .zb26+
     .2d0*(-((-s13+s42-s56)*za43*
     .zb46)-
     .(-s13-s42+s56)*za35*
     .zb56)))/zb56)/
     .(s13**2-2d0*s13*s42+s42**2-
     .2d0*s13*s56-2d0*s42*s56+s56**2)))/
     .(2d0*zb43*(-(za12*zb14)-za23*zb43))-
     .(zL1d(-s56,-t(i1,i4,i3))*za25*zb12*zb26)/
     .(zb43*(-(za12*zb14)-za23*zb43)*
     .t(i1,i4,i3))+(zL0d(-t(i1,i4,i3),-s56)*za25*zb12*
     .(-(za15*zb14)+za35*zb43)*t(i1,i4,i3))/
     .(s56*za56*zb43*
     .(-(za12*zb14)-za23*zb43)**2)


      ztmp=ztmp
     .+(zzlogRat(-s42,-s56)*(-((za45*zb42*
     .(2d0*((-s13+s42-s56)*za25*
     .zb12+
     .(-s13-s42+s56)*za56*
     .zb16)+
     .(za42*zb14-za23*zb13)*
     .(-(za15*zb12)+za35*zb23)))/
     .za56)+(6d0*
     .(-((-s13-s42+s56)*za23)+
     .2d0*za13*za42*zb14)*zb13*zb42*
     .(-(za14*zb12)-za43*zb23)*
     .(-(za15*zb16)-za35*zb36))/
     .(s13**2-2d0*s13*s42+s42**2-
     .2d0*s13*s56-2d0*s42*s56+s56**2)-
     .(zb26*(3d0*(-s13+s42-s56)*za42*
     .zb14*zb26-
     .za42*zb14*zb42*
     .(-(za14*zb16)+za43*zb36)+
     .za23*zb13*zb42*
     .(-(za14*zb16)+za43*zb36)-
     .2d0*za43*zb13*
     .((s13-s42-s56)*zb46-
     .2d0*za25*zb42*zb56)))/zb56+
     .(zb14*(-((za45*za25*zb42)/za56)-
     .(za42*zb46*zb26)/zb56)*
     .(-s13**2+2d0*s13*s42-s42**2+
     .2d0*s13*s56+2d0*s42*s56-s56**2+
     .(-s13+s42-s56)*
     .(t(i1,i4,i3)-t(i1,i2,i3))))/
     .(-(za12*zb14)-za23*zb43)))/
     .(2d0*(s13**2-2d0*s13*s42+s42**2-
     .2d0*s13*s56-2d0*s42*s56+s56**2)*zb43*
     .(-(za12*zb14)-za23*zb43))


      ztmp=ztmp
     .-(zI3m(s13,s42,s56)*zb13*
     .((3d0*za13*(-((-s13+s42-s56)*za42*
     .zb14)-
     .(s13-s42-s56)*za23*zb13)*
     .zb42*(-(za14*zb12)-za43*zb23)*
     .(-(za15*zb16)-za35*zb36))/
     .(s13**2-2d0*s13*s42+s42**2-
     .2d0*s13*s56-2d0*s42*s56+s56**2)**2-
     .(za43*(-(za15*zb14)+za35*zb43)*
     .zb26)/t(i1,i4,i3)-
     .(-(za43*((-s13+s42-s56)*za25*
     .zb42+
     .(-s13-s42+s56)*za56*zb46)*
     .zb26)+
     .(-(za14*zb12)-za43*zb23)*
     .(3d0*za13*za25*zb14*zb26+
     .za23*zb42*
     .(-(za15*zb16)-za35*zb36))+
     .za13*za45*zb12*
     .((s13-s42-s56)*zb46-
     .(-(za12*zb14)-za23*zb43)*
     .zb26-2d0*za25*zb42*zb56)-
     .(za13*zb14*zb46*
     .(-(za14*za25*zb12)-
     .za43*za25*zb23)*
     .(-t(i1,i4,i3)+t(i1,i2,i3)))/
     .(-(za12*zb14)-za23*zb43))/
     .(s13**2-2d0*s13*s42+s42**2-
     .2d0*s13*s56-2d0*s42*s56+s56**2)))/
     .(zb43*(-(za12*zb14)-za23*zb43))


c --- DSW. Introduce tmp to split up the above expression
      zFscqqpm=zFscqqpm
     .+((za42*zb14-za23*zb13)*zb43*(ztmp))/
     .(zb13*(-(za12*zb13)-za42*zb43))


      zFscqqpm=zFscqqpm+
     .zI3m(s14,s23,s56)*
     .((za43*za25*zb14*zb26)/
     .((-(za12*zb13)-za42*zb43)*
     .(-(za12*zb14)-za23*zb43))-
     .(3d0*za56*(-(za13*zb12)-za43*zb42)*
     .zb46*(-((za13*zb14*
     .(-(za42*zb12)-za43*zb13)*
     .(-((s14-s23-s56)*za14*zb46)+
     .(-s14-s23+s56)*za15*zb56))/
     .(-(za12*zb42)-za13*zb43))+
     .za43*zb14*
     .(-((s14-s23-s56)*za14*zb16)-
     .(-s14-s23+s56)*za45*zb56)))/
     .((s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)**2*
     .(-(za12*zb14)-za23*zb43))-
     .(3d0*s14*za25*(-(za13*zb12)-za43*zb42)*
     .zb56*(za56*zb14*
     .((s14-s23-s56)*za14*zb16+
     .(-s14-s23+s56)*za45*zb56)+
     .(-((s14-s23-s56)*za45*zb14)-
     .(-s14-s23+s56)*za56*zb16)*
     .t(i1,i4,i2)))/
     .((s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)**2*
     .(-(za12*zb13)-za42*zb43)*
     .(-(za12*zb14)-za23*zb43))-
     .(za25*zb14*((za14*za23*zb12+
     .za42*(-(za13*zb12)-za43*zb42)+
     .za43*(-(za13*zb13)-za43*zb43))*
     .zb26*t(i1,i4,i3)-
     .((-(za14*za23*zb13)+
     .za42*(-(za12*zb12)-
     .za42*zb42)+
     .za43*(-(za12*zb13)-za42*zb43)
     .)*(-(za13*zb12*zb36)-za43*zb42*zb36)*
     .t(i1,i4,i3))/(-(za12*zb13)-za42*zb43)+
     .(-(za13*zb12)-za43*zb42)*
     .(-((s14-s23-s56)*za14*zb16)-
     .(-s14-s23+s56)*za45*zb56-
     .(-(za14*zb16)+za43*zb36)*t(i1,i4,i2)+
     .(-(za14*zb16)+za42*zb26)*t(i1,i4,i3))-
     .za14*zb56*
     .(3d0*(za45*zb14+za35*zb13)*
     .(-(za13*zb12)-za43*zb42)-
     .za35*zb12*(-t(i1,i4,i2)+t(i1,i4,i3)))))/
     .((s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)*
     .(-(za12*zb13)-za42*zb43)*
     .(-(za12*zb14)-za23*zb43))+
     .(zb46*(-(za43*(za13*za56*zb12*zb16-
     .(za45*zb14+za56*zb16)*
     .(-(za13*zb12)-za43*zb42)-
     .za43*za56*zb14*zb26))+
     .(za13*(-(za42*zb12)-za43*zb13)*
     .(za15*zb14*
     .(-(za13*zb12)-za43*zb42)+
     .3d0*za13*za56*zb12*zb46-
     .(za13*za56*zb42*zb46*
     .(t(i1,i2,i3)-t(i4,i2,i3)))/
     .(-(za12*zb42)-za13*zb43)))/
     .(-(za12*zb42)-za13*zb43)))/
     .((s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)*
     .(-(za12*zb14)-za23*zb43)))


      zFscqqpm=zFscqqpm
     .-(zzlogRat(-s23,-s56)*zb42*
     .(-2d0*(-(za13*zb12)+za43*zb42)*
     .(-(za15*zb16)-za45*zb46)+
     .4d0*(-(za14*za35*zb12*zb46)+
     .za13*za45*zb14*zb26)+
     .(-(za13*zb12)-za43*zb42+
     .(4d0*s56*za13*zb42)/
     .(-(za12*zb42)-za13*zb43))*
     .(-((za15*za45*zb14)/za56)+
     .(za14*zb16*zb46)/zb56)-
     .(-s14+s23-s56)*
     .((za15*za35*zb12)/za56+
     .(2d0*za15*zb42*
     .(za15*za43*zb14+
     .za15*za23*zb12-
     .za43*za56*zb46))/
     .(za56*(-(za12*zb42)-za13*zb43))-
     .(za43*zb46*zb26)/zb56)+
     .(8d0*s14*za15*za35*zb42*zb56)/
     .(-(za12*zb42)-za13*zb43)+
     .(3d0*(-s14-s23+s56)*
     .(-(za13*zb12)-za43*zb42)*
     .(-(za15*zb16)-za45*zb46)*
     .(t(i1,i2,i3)-t(i4,i2,i3)))/
     .(s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)))/
     .(2d0*(s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)*zb43*
     .(-(za12*zb42)-za13*zb43))


      zFscqqpm=zFscqqpm-
     .(zLs1d(-s42,-t(i4,i2,i3),-s23,-t(i4,i2,i3))*za43**2*zb42**2*
     .(-(za45*zb43)-za25*zb23)**2)/
     .(za56*zb43*(za14*zb43+za12*zb23)*
     .t(i4,i2,i3)**3)-(za43**2*
     .(za25*zb42+za35*zb43)*
     .(-(za45*zb43)-za25*zb23))/
     .(2d0*za42*za23*za56*zb43*
     .(za14*zb43+za12*zb23)*t(i4,i2,i3))


      zFscqqpm=zFscqqpm
     .+(za43**2*zb42*(-((zL1d(-t(i4,i2,i3),-s23)*
     .(za25*zb42+za35*zb43)**2*zb23)/
     .(s23**2*(-(za12*zb42)-za13*zb43)))+
     .(zL1d(-t(i4,i2,i3),-s42)*zb42*
     .(-(za45*zb43)-za25*zb23)**2)/
     .(s42**2*(za14*zb43+za12*zb23))))/
     .(2d0*za56*zb43*t(i4,i2,i3))


      zFscqqpm=zFscqqpm-
     .(zL0d(-t(i4,i2,i3),-s23)*za15**2*za43*zb42**2*t(i4,i2,i3))/
     .(s23*za56*(-(za12*zb42)-za13*zb43)**2*
     .(za14*zb43+za12*zb23))


      zFscqqpm=zFscqqpm-
     .(zLsm1_2mh(s14,t(i4,i2,i3),s23,s56)*za15**2*zb42**2*
     .(za14*zb42-za13*zb23)*t(i4,i2,i3))/
     .(za56*(-(za12*zb42)-za13*zb43)**3*zb23*
     .(za14*zb43+za12*zb23))


      zFscqqpm=zFscqqpm
     .+(za15**2*zb12*zb42*
     .(za14*zb42-za13*zb23)*
     .((zL1d(-s56,-t(i4,i2,i3))*zb12)/(zb42*t(i4,i2,i3)**2)-
     .(2d0*zL0d(-s56,-t(i4,i2,i3)))/
     .((-(za12*zb42)-za13*zb43)*t(i4,i2,i3)))*
     .t(i4,i2,i3))/
     .(2d0*za56*(-(za12*zb42)-za13*zb43)*zb23*
     .(za14*zb43+za12*zb23))


      zFscqqpm=zFscqqpm
     .-(zzlogRat(-s14,-s56)*zb42*
     .(((s14-s23-s56)*zb14*
     .(-(za45*(-(za15*za43)+za14*za35)*
     .zb42)-za15*za45*
     .(-(za13*zb12)-za43*zb42)+
     .(za13*za15**2*
     .(-(za42*zb12)-za43*zb13)*zb42)/
     .(-(za12*zb42)-za13*zb43)))/
     .((s14**2-2d0*s14*s23+s23**2-
     .2d0*s14*s56-2d0*s23*s56+s56**2)*za56)
     .-(2d0*(-s14-s23+s56)*za14*za35*
     .(zb16*zb42+zb12*zb46))/
     .(s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)-
     .(za13*zb42*(-((za15*za45*zb14)/
     .za56)-(za14*zb16*zb46)/zb56))/
     .(-(za12*zb42)-za13*zb43)-
     .(za43*zb46*zb26)/zb56+
     .((s14-s23-s56)*za14*
     .(zb12*zb46*
     .(-(za13*zb16)-za43*zb46)-
     .za13*zb42*
     .(zb16**2+((-(za42*zb12)-
     .za43*zb13)*zb46**2)/
     .(-(za12*zb42)-za13*zb43))+
     .zb14*zb26*
     .(-(za43*zb46)-za23*zb26)))/
     .((s14**2-2d0*s14*s23+s23**2-
     .2d0*s14*s56-2d0*s23*s56+s56**2)*zb56)
     .+(6d0*s14*s23*(-(za13*zb12)-za43*zb42)*
     .(-(za15*zb16)-za45*zb46)*
     .(t(i1,i2,i3)-t(i4,i2,i3)))/
     .(s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)**2-
     .(s14*(-((za35**2*zb23)/za56)-
     .(za23*zb26**2)/zb56)*t(i4,i2,i3))/
     .(s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)))/
     .(2d0*s23*zb43*(-(za12*zb42)-za13*zb43))


      zFscqqpm=zFscqqpm-
     .(zzlogRat(-s14,-s56)*za13*
     .((za15*za35*zb12)/za56+
     .((s14-s23-s56)*zb14*
     .(za15*za43*
     .(-(za15*zb12)-za45*zb42)-
     .za13*zb42*
     .(za45**2+(za15**2*
     .(-(za42*zb12)-za43*zb13))/
     .(-(za12*zb42)-za13*zb43))-
     .za14*za35*
     .(-(za15*zb12)+za35*zb23)))/
     .((s14**2-2d0*s14*s23+s23**2-
     .2d0*s14*s56-2d0*s23*s56+s56**2)*za56)
     .+(2d0*(-s14-s23+s56)*
     .(za15*za43+za13*za45)*zb14*zb26
     .)/(s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)-
     .(za13*zb42*(-((za15*za45*zb14)/
     .za56)-(za14*zb16*zb46)/zb56))/
     .(-(za12*zb42)-za13*zb43)+
     .((s14-s23-s56)*za14*
     .(-(zb16*(-(za13*zb12)-za43*zb42)*
     .zb46)+(za13*
     .(-(za42*zb12)-za43*zb13)*
     .zb42*zb46**2)/
     .(-(za12*zb42)-za13*zb43)-
     .za13*zb16*
     .(-(zb12*zb46)-zb14*zb26)))/
     .((s14**2-2d0*s14*s23+s23**2-
     .2d0*s14*s56-2d0*s23*s56+s56**2)*zb56)
     .-(s14*(-((za35**2*zb23)/za56)-
     .(za23*zb26**2)/zb56)*t(i1,i2,i3))/
     .(s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)+
     .(6d0*s14*s23*(-(za13*zb12)-za43*zb42)*
     .(-(za15*zb16)-za45*zb46)*
     .(-t(i1,i2,i3)+t(i4,i2,i3)))/
     .(s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)**2))/
     .(2d0*s23*za12*(-(za12*zb42)-za13*zb43))


      zFscqqpm=zFscqqpm
     .-(zzlogRat(-s23,-s56)*za13*
     .(-2d0*(za13*zb12-za43*zb42)*
     .(-(za15*zb16)-za45*zb46)-
     .(8d0*s14*za13*za56*zb46*zb26)/
     .(-(za12*zb42)-za13*zb43)+
     .4d0*(-(za14*za35*zb16*zb42)+
     .za15*za43*zb14*zb26)+
     .(-(za13*zb12)-za43*zb42+
     .(4d0*s56*za13*zb42)/
     .(-(za12*zb42)-za13*zb43))*
     .((za15*za45*zb14)/za56-
     .(za14*zb16*zb46)/zb56)-
     .(-s14+s23-s56)*
     .((za15*za35*zb12)/za56-
     .(za43*zb46*zb26)/zb56-
     .(2d0*za13*zb46*
     .(-(za14*zb12*zb46)-
     .za43*zb46*zb23+
     .za15*zb12*zb56))/
     .((-(za12*zb42)-za13*zb43)*zb56))+
     .(3d0*(-s14-s23+s56)*
     .(-(za13*zb12)-za43*zb42)*
     .(-(za15*zb16)-za45*zb46)*
     .(-t(i1,i2,i3)+t(i4,i2,i3)))/
     .(s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)))/
     .(2d0*(s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)*za12*
     .(-(za12*zb42)-za13*zb43))


      zFscqqpm=zFscqqpm
     .+(zzlogRat(-s13,-s56)*za43*za25*
     .(zb43*zb26-((-(za45*zb43)-
     .za25*zb23)*t(i1,i4,i3)-
     .(-(za15*zb13)-za45*zb43)*t(i4,i2,i3))/
     .(za42*za56)))/
     .(2d0*(-(za12*zb13)-za42*zb43)*
     .(-(za12*zb14)-za23*zb43)*
     .(za14*zb43+za12*zb23))


      ztmp=
     .(za35*zb16*(-((-s13-s42+s56)*
     .za14)+2d0*za13*za42*zb23))/
     .((s13**2-2d0*s13*s42+s42**2-
     .2d0*s13*s56-2d0*s42*s56+s56**2)*za12*
     .(za14*zb43+za12*zb23))


      ztmp=ztmp
     .+(((s13-s42-s56)*za14*zb13+
     .(-s13+s42-s56)*za42*zb23)*
     .(-((za35**2*zb13)/za56)-
     .(za13*zb16**2)/zb56))/
     .(2d0*(s13**2-2d0*s13*s42+s42**2-
     .2d0*s13*s56-2d0*s42*s56+s56**2)*za12*
     .zb13*(za14*zb43+za12*zb23))


      ztmp=ztmp
     .+(zL0d(-t(i1,i4,i2),-s42)*za42*zb12*
     .(za14*zb46+za12*zb26)**2)/
     .(s42*za12*(za14*zb43+za12*zb23)**2*zb56)


      ztmp=ztmp
     .+(zLsm1_2mh(s13,t(i1,i4,i2),s42,s56)*
     .(-(za14*zb13)+za42*zb23)*
     .(za14*zb46+za12*zb26)**2)/
     .(za12*(za14*zb43+za12*zb23)**3*zb56)


      ztmp=ztmp
     .-(zzlogRat(-s42,-s56)*za42*
     .((6d0*(za43*zb14+za23*zb12)*zb42*
     .(-((-s13-s42+s56)*za14)+
     .2d0*za13*za42*zb23)*
     .(-(za45*zb46)-za25*zb26))/
     .(s13**2-2d0*s13*s42+s42**2-
     .2d0*s13*s56-2d0*s42*s56+s56**2)**2-
     .((za14*zb46+za12*zb26)*
     .(zb13*zb26+zb12*zb36))/
     .(zb13*(za14*zb43+za12*zb23)*
     .zb56)+(-((za35*
     .(za35*zb13*
     .(za14*zb42-za13*zb23)+
     .2d0*(-((s13-s42-s56)*za15*zb12)+
     .(-s13-s42+s56)*za56*
     .zb26)))/za56)-
     .((-s13+s42-s56)*za14*zb42*
     .(-(za15*zb16)+za35*zb36))/
     .(za14*zb43+za12*zb23)+
     .(za43*zb16*zb42*
     .(za14*zb46+za12*zb26))/
     .zb56+(zb42*zb36*
     .(-((2d0*(-s13+s42-s56)*za43+
     .za13*za42*zb12)*zb16)-
     .((-s13+s42-s56)*za14*
     .(za43*zb14+za23*zb12)*zb36)/
     .(za14*zb43+za12*zb23)))/
     .(zb13*zb56)+
     .((-s13+s42-s56)*zb16*zb23*
     .(-(za43*zb46)-za23*zb26-
     .za35*zb56))/(zb13*zb56)+
     .za35*(-(za14*zb16*zb42)+
     .za35*zb23*zb56)-
     .4d0*za15*zb42*(za43*zb16+
     .(za14*za35*zb13*zb56)/
     .(za14*zb43+za12*zb23)))/
     .(s13**2-2d0*s13*s42+s42**2-
     .2d0*s13*s56-2d0*s42*s56+s56**2)))/
     .(2d0*za12*(za14*zb43+za12*zb23))


      ztmp=ztmp
     .-(zL1d(-s56,-t(i1,i4,i2))*za43*za35*zb36)/
     .(za12*(za14*zb43+za12*zb23)*t(i1,i4,i2))


      ztmp=ztmp
     .-(zL0d(-t(i1,i4,i2),-s56)*za43*
     .(za14*zb46+za12*zb26)*zb36*
     .t(i1,i4,i2))/
     .(s56*za12*(za14*zb43+za12*zb23)**2*
     .zb56)


      ztmp=ztmp
     .+(zzlogRat(-s13,-s56)*
     .((6d0*za13*za42*
     .(za43*zb14+za23*zb12)*
     .(-2d0*za14*zb13*zb42+
     .(-s13-s42+s56)*zb23)*
     .(-(za45*zb46)-za25*zb26))/
     .(s13**2-2d0*s13*s42+s42**2-
     .2d0*s13*s56-2d0*s42*s56+s56**2)+
     .(za35*(-3d0*(s13-s42-s56)*za14*
     .za35*zb13+
     .za14*za13*
     .(za45*zb14+za25*zb12)*zb13-
     .za13*za42*
     .(za45*zb14+za25*zb12)*zb23-
     .2d0*za42*zb12*
     .((-s13+s42-s56)*za15+
     .2d0*za13*za56*zb36)))/za56+
     .(za13*zb16*
     .((-(za14*zb13)+za42*zb23)*
     .(-(za43*zb46)-za23*zb26)+
     .2d0*((s13-s42-s56)*za43*
     .zb36-
     .(-s13-s42+s56)*za45*zb56)
     .))/zb56-(za14*((za15*za35*zb13)/za56+
     .(za13*zb16*zb36)/zb56)*
     .(-s13**2+2d0*s13*s42-s42**2+
     .2d0*s13*s56+2d0*s42*s56-s56**2+
     .(s13-s42-s56)*(t(i1,i4,i2)-t(i4,i2,i3))
     .))/(za14*zb43+za12*zb23)))/
     .(2d0*(s13**2-2d0*s13*s42+s42**2-
     .2d0*s13*s56-2d0*s42*s56+s56**2)*za12*
     .(za14*zb43+za12*zb23))


      ztmp=ztmp
     .-(zI3m(s42,s13,s56)*za42*
     .((3d0*za13*(za43*zb14+za23*zb12)*
     .zb42*((s13-s42-s56)*za14*
     .zb13+
     .(-s13+s42-s56)*za42*zb23)*
     .(-(za45*zb46)-za25*zb26))/
     .(s13**2-2d0*s13*s42+s42**2-
     .2d0*s13*s56-2d0*s42*s56+s56**2)**2-
     .(za35*zb12*
     .(za14*zb46+za12*zb26))/t(i1,i4,i2)-
     .(za43*zb16*zb42*
     .((-s13+s42-s56)*za15-
     .za35*
     .(za14*zb43+za12*zb23)+
     .2d0*za13*za56*zb36)+
     .(za43*zb14+za23*zb12)*
     .(-(za13*zb23*
     .(-(za45*zb46)-za25*zb26))-
     .3d0*za14*za35*zb42*zb36)-
     .za35*zb12*
     .((s13-s42-s56)*za13*zb36-
     .(-s13-s42+s56)*za15*zb56)+
     .(za14*za15*zb42*
     .(za43*zb14*zb36+
     .za23*zb12*zb36)*
     .(-t(i1,i4,i2)+t(i4,i2,i3)))/
     .(za14*zb43+za12*zb23))/
     .(s13**2-2d0*s13*s42+s42**2-
     .2d0*s13*s56-2d0*s42*s56+s56**2)))/
     .(za12*(za14*zb43+za12*zb23))


c --- DSW. Introduce tmp to split up the above expression
      zFscqqpm=zFscqqpm
     .+(za12*(-(za14*zb13)+za42*zb23)*
     .(ztmp))/
     .(za42*(-(za12*zb13)-za42*zb43))


      zFscqqpm=zFscqqpm
     .+zI3m(s14,s23,s56)
     .*(-((za14*za35*zb12*zb36)/
     .((-(za12*zb13)-za42*zb43)*
     .(za14*zb43+za12*zb23)))+
     .(3d0*za15*(-(za13*zb12)-za43*zb42)*
     .(-(za14*zb12*
     .((s14-s23-s56)*za45*zb14+
     .(-s14-s23+s56)*za56*zb16))+
     .(za14*(-(za42*zb12)-za43*zb13)*
     .zb42*((s14-s23-s56)*za15*
     .zb14-(-s14-s23+s56)*za56*
     .zb46))/(-(za12*zb42)-za13*zb43)
     .)*zb56)/((s14**2-2d0*s14*s23+s23**2-
     .2d0*s14*s56-2d0*s23*s56+s56**2)**2*
     .(za14*zb43+za12*zb23))+
     .(3d0*s14*za56*(-(za13*zb12)-za43*zb42)*
     .zb36*(za14*(-((s14-s23-s56)*za45*
     .zb14)-
     .(-s14-s23+s56)*za56*zb16)*zb56
     .+((s14-s23-s56)*za14*zb16+
     .(-s14-s23+s56)*za45*zb56)*t(i1,i4,i3)
     .))/((s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)**2*
     .(-(za12*zb13)-za42*zb43)*
     .(za14*zb43+za12*zb23))+
     .(za14*zb36*(-(((-(za13*za25*zb12)-
     .za43*za25*zb42)*
     .(zb12*(-(za12*zb13)-
     .za42*zb43)+
     .zb13*(-(za13*zb13)-
     .za43*zb43)-za42*zb14*zb23
     .)*t(i1,i4,i2))/(-(za12*zb13)-za42*zb43))+
     .za35*(zb12*
     .(-(za12*zb12)-za42*zb42)+
     .zb13*(-(za13*zb12)-za43*zb42)+
     .za43*zb14*zb23)*t(i1,i4,i2)-
     .za56*zb14*
     .(3d0*(-(za13*zb12)-za43*zb42)*
     .(-(za14*zb16)+za42*zb26)-
     .za43*zb26*(t(i1,i4,i2)-t(i1,i4,i3)))+
     .(-(za13*zb12)-za43*zb42)*
     .((s14-s23-s56)*za45*zb14+
     .(-s14-s23+s56)*za56*zb16+
     .(za45*zb14+za35*zb13)*t(i1,i4,i2)-
     .(za45*zb14+za25*zb12)*t(i1,i4,i3))))/
     .((s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)*
     .(-(za12*zb13)-za42*zb43)*
     .(za14*zb43+za12*zb23))+
     .(za15*(-(zb12*(-(za14*za35*zb12*zb56)-
     .za43*za45*zb42*zb56-
     .(-(za13*zb12)-za43*zb42)*
     .(-(za14*zb16)-za45*zb56)))+
     .((-(za42*zb12)-za43*zb13)*zb42*
     .(-(za14*(-(za13*zb12)-za43*zb42)*
     .zb46)-3d0*za15*za43*zb42*zb56+
     .(za13*za15*zb42*zb56*
     .(-t(i1,i2,i3)+t(i4,i2,i3)))/
     .(-(za12*zb42)-za13*zb43)))/
     .(-(za12*zb42)-za13*zb43)))/
     .((s14**2-2d0*s14*s23+s23**2-2d0*s14*s56-
     .2d0*s23*s56+s56**2)*
     .(za14*zb43+za12*zb23)))

      return
      end function

      function zFscqqpp(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

      t(j1,j2,j3)=s6(j1,j2)+s6(j2,j3)+s6(j3,j1)

      za15 = zA6(i1,i5)
      za45 = zA6(i4,i5)
      s42  = s6(i4,i2)
      s56  = s6(i5,i6)
      za14 = zA6(i1,i4)
      za13 = zA6(i1,i3)
      za42 = zA6(i4,i2)
      s14  = s6(i1,i4)
      za23 = zA6(i2,i3)
      za12 = zA6(i1,i2)
      za56 = zA6(i5,i6)
      za43 = zA6(i4,i3)
      za35 = zA6(i3,i5)
      zb14 = zB6(i1,i4)
      za25 = zA6(i2,i5)
      zb12 = zB6(i1,i2)
      zb16 = zB6(i1,i6)
      zb13 = zB6(i1,i3)
      zb42 = zB6(i4,i2)
      zb43 = zB6(i4,i3)
      zb23 = zB6(i2,i3)
      zb26 = zB6(i2,i6)
      zb36 = zB6(i3,i6)
      zb56 = zB6(i5,i6)

      zFscqqpp=
     .-((za15*za45*((zzlogRat(-s42,-s56)*za14)/
     .(za13*za42)+zzlogRat(-s14,-s56)/za23))/
     .(za12*za13*za56))+
     .(zLsm1(-s14,-t(i1,i4,i2),-s42,-t(i1,i4,i2))*za15**2*
     .za42)/
     .(za12**2*za13*za23*za56)+
     .(zLsm1_2me(t(i1,i4,i2),t(i4,i2,i3),s42,s56)*za15**2*
     .za43**2)/(za13**3*za42*za23*za56)+
     .(zzlogRat(-t(i1,i4,i2),-s56)*za45**2)/
     .(za13*za42*za23*za56)+
     .(zzlogRat(-t(i4,i2,i3),-s56)*za45**2)/
     .(2d0*za13*za42*za23*za56)+
     .(zLsm1_2me(t(i1,i4,i3),t(i1,i4,i2),s14,s56)*za42*
     .za35**2)/(za13*za23**3*za56)-
     .(zL0d(-t(i1,i4,i2),-s42)*za14**2*
     .(za15*za45*zb14+za15*za25*zb12))/
     .(s42*za12*za13**2*za42*za56)+
     .(zL1d(-t(i4,i2,i3),-s56)*za14**2*za56*zb16**2)/
     .(2d0*s56**2*za13*za42*za23)+
     .(zL0d(-t(i4,i2,i3),-s56)*((za15**2*za43*
     .(-(za42*zb12)-za43*zb13))/
     .(za13**2*za42*za23*za56)+
     .(za14*za45*zb16)/(za13*za42*za23)))/
     .s56+(zL0d(-t(i1,i4,i2),-s14)*za42*
     .(-(za15*za25*zb12)-za45*za25*zb42))/
     .(s14*za12*za23**2*za56)-
     .(zL0d(-t(i1,i4,i3),-s14)*za43*za35*
     .(-(za15*zb13)-za45*zb43))/
     .(s14*za13*za23**2*za56)+
     .(zL0d(-t(i4,i2,i3),-s42)*(za15*za43+za13*za45)*
     .za35*zb23)/(s42*za13**2*za23*za56)-
     .(zL0d(-t(i1,i4,i3),-s56)*za43*za25*zb26)/
     .(s56*za13*za23**2)+
     .(zL1d(-t(i1,i4,i3),-s56)*za42*za56*zb26**2)/
     .(s56**2*za13*za23)+
     .(zL1d(-t(i1,i4,i2),-s56)*za43**2*za56*zb36**2)/
     .(s56**2*za13*za42*za23)+
     .(zL0d(-t(i1,i4,i2),-s56)*(-((za45*za35*
     .(-(za14*zb13)+za42*zb23))/
     .(za13*za42*za23*za56))-
     .(za43*(-(za15*za43)+2d0*za13*za45)*
     .zb36)/(za13**2*za42*za23)+
     .(za43*za35*zb36)/(za13*za23**2)))/s56+
     .(-(za45**2/(za13*za42*za23*za56))+
     .(za45*za35*zb23)/
     .(za13*za23*za56*t(i4,i2,i3))-
     .(za15*za45*zb12*zb23)/
     .(za13*za56*t(i1,i2,i3)*t(i4,i2,i3))-
     .(zb23*(-(za13*zb16)-za23*zb26)*
     .(za42*zb26+za43*zb36))/
     .(za13*za23*zb56*t(i1,i2,i3)*t(i4,i2,i3)))/2-
     .(zL1d(-s42,-t(i4,i2,i3))*za42*za35**2*zb23**2)/
     .(2d0*za13*za23*za56*t(i4,i2,i3)**2)

      return
      end function

      function zFccqpqm(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

      t(j1,j2,j3)=s6(j1,j2)+s6(j2,j3)+s6(j3,j1)

      s34  = s6(i3,i4)
      za25 = zA6(i2,i5)
      zb32 = zB6(i3,i2)
      za45 = zA6(i4,i5)
      zb34 = zB6(i3,i4)
      za56 = zA6(i5,i6)
      zb42 = zB6(i4,i2)
      za13 = zA6(i1,i3)
      za14 = zA6(i1,i4)
      s13  = s6(i1,i3)
      zb16 = zB6(i1,i6)
      za34 = zA6(i3,i4)
      zb36 = zB6(i3,i6)
      zb12 = zB6(i1,i2)
      zb56 = zB6(i5,i6)
      s42  = s6(i4,i2)
      s56  = s6(i5,i6)
      za42 = zA6(i4,i2)
      za12 = zA6(i1,i2)
      za32 = zA6(i3,i2)
      zb46 = zB6(i4,i6)
      zb14 = zB6(i1,i4)
      s12  = s6(i1,i2)
      zb26 = zB6(i2,i6)
      s14  = s6(i1,i4)
      s32  = s6(i3,i2)
      za35 = zA6(i3,i5)
      zb13 = zB6(i1,i3)
      za15 = zA6(i1,i5)

      zFccqpqm=
     .(2d0*zL0d(-t(i3,i4,i2),-s34)*za25*zb32*
     .(za45*zb34+za25*zb32))/
     .(s34*za56*zb42*(za13*zb32+za14*zb42))
     .+(zLsm1(-s13,-t(i1,i3,i4),-s34,-t(i1,i3,i4))*za14*
     .(-(za14*zb16)-za34*zb36)**2)/
     .(za13*za34*(-(za14*zb12)-za34*zb32)*
     .(za13*zb32+za14*zb42)*zb56)+
     .(zLsm1_2mh(s42,t(i1,i3,i4),s13,s56)*za14*
     .(-(za14*zb16)-za34*zb36)**2)/
     .(za13*za34*(-(za14*zb12)-za34*zb32)*
     .(za13*zb32+za14*zb42)*zb56)-
     .(2d0*zL0d(-t(i1,i3,i2),-s56)*za42*
     .(-(za12*zb16)-za32*zb36)*zb46)/
     .(s56*za13*(-(za13*zb14)-za32*zb42)*
     .zb56)+zLsm1_2mh(s12,t(i1,i3,i4),s34,s56)*
     .(-((za14*(-(za14*zb12)-za34*zb32)*
     .(za13*zb36+za14*zb46)**2)/
     .(za13*za34*(za13*zb32+za14*zb42)**3*
     .zb56))+(za14**3*zb26**2*t(i1,i3,i4)**2)/
     .(za13*za34*(-(za14*zb12)-za34*zb32)*
     .(za13*zb32+za14*zb42)**3*zb56))-
     .(2d0*zzlogRat(-s34,-s56)*(2d0*(s14-s32)*za42
     .*zb34*
     .(-(za35*zb36)-za45*zb46)+
     .(za45*zb13*(((-s12+s34-s56)*
     .za45*zb34+
     .(-s12-s34+s56)*za56*zb36)*
     .(-(za13*zb14)-za32*zb42)+
     .(s14-s32)*
     .(-((-s12+s34-s56)*za35*zb34)+
     .(-s12-s34+s56)*za56*zb46)))/
     .(za56*zb12)-
     .(za42*(zb13*
     .(za13*zb36+za14*zb46)-
     .zb32*(-(za32*zb36)-za42*zb46)+
     .zb34*(-(za14*zb16)+za42*zb26))*
     .(-(za35*zb34*zb56)+zb46*t(i1,i3,i4)))/
     .zb56))/
     .((s12**2-2d0*s12*s34+s34**2-2d0*s12*s56-
     .2d0*s34*s56+s56**2)*
     .(za13*zb32+za14*zb42)*
     .(-(za13*zb14)-za32*zb42))



      zFccqpqm=zFccqpqm-
     .(zLsm1(-s12,-t(i1,i3,i2),-s13,-t(i1,i3,i2))*za45**2*zb13**2)/
     .(za56*zb12*(-(za14*zb12)-za34*zb32)*
     .t(i1,i3,i2))-(2d0*zL0d(-t(i1,i3,i2),-s12)*za12*zb13*
     .(-(za12*zb16)-za32*zb36)*
     .(-(za13*zb16)+za32*zb26))/
     .(s12*za13*(-(za13*zb14)-za32*zb42)*
     .zb56*t(i1,i3,i2))-zLsm1_2mh(s34,t(i1,i3,i2),s12,
     .s56)*((za45**2*zb13**2)/
     .(za56*zb12*(-(za14*zb12)-za34*zb32)*
     .t(i1,i3,i2))+((-(za12*zb14)-za32*zb34)**2*
     .(-(za13*zb16)+za32*zb26)**2)/
     .(za13*(-(za13*zb14)-za32*zb42)**3*
     .zb56*t(i1,i3,i2))-
     .(za32**2*zb46**2*t(i1,i3,i2))/
     .(za13*(-(za13*zb14)-za32*zb42)**3*zb56)
     .)-(zI3m(s12,s34,s56)*zb13*
     .(-(s56*za42*((-s12-s34+s56)*za15*
     .zb46+2d0*za12*za35*zb34*zb26))+
     .2d0*za12*za15*za45*
     .((-s12-s34+s56)*zb14+
     .2d0*za32*zb12*zb34)*zb56-
     .2d0*za12*za25*zb26*
     .(s56*(-s12-s34+s56)+
     .(s12-s34-s56)*t(i1,i3,i2))))/
     .((s12**2-2d0*s12*s34+s34**2-2d0*s12*s56-
     .2d0*s34*s56+s56**2)*
     .(za13*zb32+za14*zb42)*
     .(-(za13*zb14)-za32*zb42))

      zFccqpqm=zFccqpqm+
     .zI3m(s12,s34,s56)*
     .((-((-s12+s34-s56)*za45*zb13*
     .(za45*zb14+za25*zb12))/
     .(2d0*za56*zb12)-
     .za42*zb16*(-(za15*zb13)+
     .za45*zb34)+
     .(za32*za45*zb13-za42*za25*zb12)*
     .zb36+(za12*za34*za45*zb13*zb46)/
     .za13+(za14*za25*zb13*
     .((s14-s32)*za13+
     .za14*(-(za13*zb14)-za32*zb42))*zb26)/
     .(za13*(za13*zb32+za14*zb42))-
     .((-(za12*zb16)-za32*zb36)*
     .(-2d0*s56*za14*zb16+
     .(-s12+s34-s56)*
     .(-(za14*zb16)-za34*zb36)))/
     .(2d0*za13*zb56))/
     .((-(za14*zb12)-za34*zb32)*
     .(-(za13*zb14)-za32*zb42))+
     .(za14*za25*zb13*
     .(-(za14*zb16)-za34*zb36))/
     .(za13*(-(za14*zb12)-za34*zb32)*t(i1,i3,i4))
     .-(2d0*za12*za32*za45*zb13*zb46)/
     .(za13*(-(za13*zb14)-za32*zb42)*t(i1,i3,i2))
     .+(((za45**2*zb13**2)/
     .(za56*zb12*
     .(-(za14*zb12)-za34*zb32))+
     .(-(za12*zb16)-za32*zb36)**2/
     .(za13*(-(za13*zb14)-za32*zb42)*
     .zb56))*(2d0*s12*s56+
     .(-s12+s34-s56)*t(i1,i3,i2)))/(2d0*t(i1,i3,i2)**2))-
     .(2d0*zzlogRat(-s12,-s56)*(-2d0*(s14-s32)*za12*zb13*
     .(-(za15*zb16)-za25*zb26)+
     .(za42*zb16*((s14-s32)*
     .((s12-s34-s56)*za12*zb26-
     .(-s12-s34+s56)*za15*zb56)+
     .(za13*zb32+za14*zb42)*
     .(-((s12-s34-s56)*za12*zb16)-
     .(-s12-s34+s56)*za25*zb56)))/
     .(za34*zb56)+
     .(zb13*(-(za12*
     .(za35*zb13+za45*zb14))+
     .za32*(-(za15*zb13)+za25*zb32)+
     .za42*(-(za15*zb14)+za25*zb42))*
     .(-(za12*za56*zb26)+za15*t(i1,i4,i2)))/
     .za56))/
     .((s12**2-2d0*s12*s34+s34**2-2d0*s12*s56-
     .2d0*s34*s56+s56**2)*
     .(za13*zb32+za14*zb42)*
     .(-(za13*zb14)-za32*zb42))

      zFccqpqm=zFccqpqm+
     .zLsm1_2mh(s13,t(i3,i4,i2),s42,s56)*
     .((za45*zb34+za25*zb32)**2/
     .(za56*zb42*(za13*zb32+za14*zb42)*
     .t(i3,i4,i2))-(za42**2*zb16**2)/
     .(za34*(-(za34*zb14)-za32*zb12)*zb56*
     .t(i3,i4,i2)))+zI3m(s13,s42,s56)*
     .((2d0*za42**2*zb16*(za45*zb34+za25*zb32))/
     .(za34*t(i3,i4,i2)**2)+
     .(za14*(-(za42*za25*zb13*zb16)+
     .((za32*zb13+za42*zb14)*
     .(za45*zb34+za25*zb32)*
     .(-(za14*zb16)-za34*zb36))/
     .(za13*zb32+za14*zb42)))/
     .(za34*t(i1,i3,i4)*t(i3,i4,i2)))+
     .(2d0*zL0d(-t(i3,i4,i2),-s56)*zb16*
     .(-(za14*zb34)-za12*zb32)*
     .(za45*zb34+za25*zb32))/
     .(s56*zb42*(za13*zb32+za14*zb42)*
     .t(i3,i4,i2))-(2d0*zzlogRat(-s34,-s56)*
     .(za45*zb34+za25*zb32)**2)/
     .(za56*zb42*(za13*zb32+za14*zb42)*
     .t(i3,i4,i2))+(zLsm1(-s34,-t(i3,i4,i2),-s42,-t(i3,i4,i2))*
     .(za45*zb34+za25*zb32)**2)/
     .(za56*zb42*(za13*zb32+za14*zb42)*
     .t(i3,i4,i2))-(zI3m(s34,s12,s56)*za42*
     .(2d0*za56*zb16*zb34*
     .(-((-s12-s34+s56)*za14)-
     .2d0*za12*za34*zb32)*zb46-
     .s56*zb13*((-s12-s34+s56)*za15*
     .zb46+2d0*za12*za35*zb34*zb26)+
     .2d0*za35*zb34*zb36*
     .(s56*(-s12-s34+s56)+
     .(-s12+s34-s56)*t(i3,i4,i2))))/
     .((s12**2-2d0*s12*s34+s34**2-2d0*s12*s56-
     .2d0*s34*s56+s56**2)*
     .(za13*zb32+za14*zb42)*
     .(-(za13*zb14)-za32*zb42))

      return
      end function

      function zFccqpqp(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

      t(j1,j2,j3)=s6(j1,j2)+s6(j2,j3)+s6(j3,j1)

      s12  = s6(i1,i2)
      s13  = s6(i1,i3)
      za45 = zA6(i4,i5)
      za12 = zA6(i1,i2)
      za34 = zA6(i3,i4)
      za32 = zA6(i3,i2)
      za56 = zA6(i5,i6)
      s34  = s6(i3,i4)
      za14 = zA6(i1,i4)
      za13 = zA6(i1,i3)
      za42 = zA6(i4,i2)
      s56  = s6(i5,i6)
      za15 = zA6(i1,i5)
      za25 = zA6(i2,i5)
      s42  = s6(i4,i2)
      za35 = zA6(i3,i5)
      zb13 = zB6(i1,i3)
      zb12 = zB6(i1,i2)
      zb32 = zB6(i3,i2)

      zFccqpqp=
     .(zLsm1(-s12,-t(i1,i3,i2),-s13,-t(i1,i3,i2))*za45**2)/
     .(za12*za34*za32*za56)+
     .(zLsm1(-s13,-t(i1,i3,i4),-s34,-t(i1,i3,i4))*za14*za45**2)/
     .(za13*za12*za34*za42*za56)+
     .(zLsm1_2me(t(i1,i3,i2),t(i1,i3,i4),s13,s56)*za14*
     .za45**2)/(za13*za12*za34*za42*za56)+
     .(zLsm1_2me(t(i1,i3,i4),t(i3,i4,i2),s34,s56)*za14*za45*
     .(-(za15*za42)+za14*za25))/
     .(za13*za12**2*za34*za42*za56)+
     .(zLsm1(-s34,-t(i3,i4,i2),-s42,-t(i3,i4,i2))*za45*
     .(-(za35*za42)+za34*za25))/
     .(za13*za32**2*za42*za56)-
     .(2d0*zL0d(-t(i3,i4,i2),-s56)*za14*za15*za45*
     .(za34*zb13-za42*zb12))/
     .(s56*za13*za12*za34*za42*za56)+
     .(2d0*zL0d(-t(i3,i4,i2),-s42)*za35*za45*zb32)/
     .(s42*za13*za32*za56)+
     .(2d0*zL0d(-t(i3,i4,i2),-s34)*za45*za25*zb32)/
     .(s34*za12*za32*za56)

      return
      end function


      function zFscqpqp(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

      t(j1,j2,j3)=s6(j1,j2)+s6(j2,j3)+s6(j3,j1)

      s34  = s6(i3,i4)
      s56  = s6(i5,i6)
      za14 = zA6(i1,i4)
      za15 = zA6(i1,i5)
      za42 = zA6(i4,i2)
      za13 = zA6(i1,i3)
      za12 = zA6(i1,i2)
      za34 = zA6(i3,i4)
      za56 = zA6(i5,i6)
      za45 = zA6(i4,i5)
      zb13 = zB6(i1,i3)
      zb12 = zB6(i1,i2)
      zb16 = zB6(i1,i6)
      zb32 = zB6(i3,i2)
      za25 = zA6(i2,i5)
      zb26 = zB6(i2,i6)
      s42  = s6(i4,i2)
      za35 = zA6(i3,i5)
      za32 = zA6(i3,i2)
      zb36 = zB6(i3,i6)
      zb56 = zB6(i5,i6)

      zFscqpqp=
     .(zLsm1_2me(t(i1,i3,i4),t(i3,i4,i2),s34,s56)*za14*za15**2*
     .za42)/(za13*za12**3*za34*za56)-
     .(za14*za45**2)/
     .(2d0*za13*za12*za34*za42*za56)+
     .(zzlogRat(-t(i3,i4,i2),-s56)*za14*za45**2)/
     .(2d0*za13*za12*za34*za42*za56)-
     .(zL0d(-t(i1,i3,i4),-s34)*za14*za15**2*zb13)/
     .(s34*za13*za12**2*za56)+
     .(zL0d(-t(i3,i4,i2),-s56)*za14*za15**2*
     .(za34*zb13-za42*zb12))/
     .(s56*za13*za12**2*za34*za56)+
     .(zL0d(-t(i3,i4,i2),-s56)*za14**2*za45*zb16)/
     .(s56*za13*za12*za34*za42)+
     .(zL1d(-t(i3,i4,i2),-s56)*za14**3*za56*zb16**2)/
     .(2d0*s56**2*za13*za12*za34*za42)+
     .(zL0d(-t(i3,i4,i2),-s34)*za15**2*za42*zb32)/
     .(s34*za13*za12**2*za56)-
     .(zL0d(-t(i1,i3,i4),-s56)*za14*za15*za25*
     .(-(za14*zb12)-za34*zb32))/
     .(s56*za13*za12**2*za34*za56)-
     .(zL0d(-t(i1,i3,i4),-s56)*za14*za45*zb26)/
     .(s56*za13*za12*za34)+
     .(zL1d(-t(i1,i3,i4),-s56)*za14*za42*za56*zb26**2)/
     .(s56**2*za13*za12*za34)+
     .(zL1d(-s42,-t(i3,i4,i2))*za35**2*za42*zb32**2)/
     .(2d0*za13*za32*za56*t(i3,i4,i2)**2)+
     .(zLs1d(-s34,-t(i3,i4,i2),-s42,-t(i3,i4,i2))*za35**2*za42*
     .zb32**2)/(za13*za32*za56*t(i3,i4,i2)**2)-
     .(zL1d(-s34,-t(i3,i4,i2))*za34*za25**2*zb32**2)/
     .(2d0*za12*za32*za56*t(i3,i4,i2)**2)-
     .(za15*za32*za45*zb32**2)/
     .(2d0*za13*za12*za56*t(i1,i3,i2)*t(i3,i4,i2))-
     .(zb32*(za13*zb36+za12*zb26)*
     .(-(za34*zb36)+za42*zb26))/
     .(2d0*za13*za12*zb56*t(i1,i3,i2)*t(i3,i4,i2))


      return
      end function

      function zFscqpqm(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

      t(j1,j2,j3)=s6(j1,j2)+s6(j2,j3)+s6(j3,j1)

      s34  = s6(i3,i4)
      s56  = s6(i5,i6)
      zb13 = zB6(i1,i3)
      za15 = zA6(i1,i5)
      za45 = zA6(i4,i5)
      zb14 = zB6(i1,i4)
      za25 = zA6(i2,i5)
      zb12 = zB6(i1,i2)
      za56 = zA6(i5,i6)
      za12 = zA6(i1,i2)
      zb16 = zB6(i1,i6)
      zb26 = zB6(i2,i6)
      za32 = zA6(i3,i2)
      zb36 = zB6(i3,i6)
      zb56 = zB6(i5,i6)
      za13 = zA6(i1,i3)
      zb32 = zB6(i3,i2)
      za14 = zA6(i1,i4)
      zb42 = zB6(i4,i2)
      s12  = s6(i1,i2)
      zb34 = zB6(i3,i4)
      za34 = zA6(i3,i4)
      zb46 = zB6(i4,i6)
      za42 = zA6(i4,i2)
      za35 = zA6(i3,i5)
      s42  = s6(i4,i2)
      s14  = s6(i1,i4)

      zFscqpqm=
     .-(zzlogRat(-s34,-s56)*zb13*
     .((za15*za45*zb14+za15*za25*zb12)/
     .za56-(-(za12*zb16*zb26)-
     .za32*zb36*zb26)/zb56))/
     .(2d0*zb12*(za13*zb32+za14*zb42)*
     .(-(za13*zb14)-za32*zb42))

      zfac=
     .-(za32*((za25*
     .(-((za45*
     .((s12-s34-s56)*za12*
     .zb14-
     .(-s12+s34-s56)*za32*
     .zb34))/za56)+
     .((-s12-s34+s56)*za32+
     .2d0*za12*za34*zb14)*zb36))/
     .za12-(zb16*
     .(-(za45*
     .(-((-s12-s34+s56)*zb14)-
     .2d0*za32*zb12*zb34))-
     .((-((-s12+s34-s56)*za34*
     .zb14)+
     .(s12-s34-s56)*za32*
     .zb12)*zb36)/zb56))/zb12))/
     .(2d0*(s12**2-2d0*s12*s34+s34**2-
     .2d0*s12*s56-2d0*s34*s56+s56**2)*za13*
     .(-(za13*zb14)-za32*zb42))+
     .(za32*zb13*(-(za13*zb16)+
     .za32*zb26)*
     .((zL0d(-t(i1,i3,i2),-s12)*
     .((-2d0*za32*zb46)/
     .(-(za13*zb14)-za32*zb42)+
     .(-(za12*zb16)-za32*zb36)/
     .t(i1,i3,i2)))/s12-
     .(zL1d(-s12,-t(i1,i3,i2))*za32*zb36)/t(i1,i3,i2)**2))/
     .(2d0*za13*(-(za13*zb14)-za32*zb42)*
     .zb56)
      zfac=zfac+(za32*za42*zb46**2*
     .((-2d0*zL0d(-s56,-t(i1,i3,i2))*za32)/
     .(-(za13*zb14)-za32*zb42)+
     .(zL1d(-s56,-t(i1,i3,i2))*za42)/t(i1,i3,i2)))/
     .(2d0*za13*za12*
     .(-(za13*zb14)-za32*zb42)*zb56)-
     .(zLsm1_2mh(s34,t(i1,i3,i2),s12,s56)*za32**3*
     .zb46**2*t(i1,i3,i2))/
     .(za13*za12*(-(za13*zb14)-
     .za32*zb42)**3*zb56)-
     .(zzlogRat(-s34,-s56)*za32*
     .(-((za35*za25*zb13)/za56)+
     .(2d0*(-s12-s34+s56)*
     .(za35*za42+za32*za45)*zb16*
     .zb34)/
     .(s12**2-2d0*s12*s34+s34**2-
     .2d0*s12*s56-2d0*s34*s56+s56**2)+
     .((-s12+s34-s56)*zb34*
     .(za35*za42*
     .(za35*zb13+za45*zb14)-
     .za34*za25*
     .(za35*zb13+za25*zb12)+
     .za32*zb14*
     .(za45**2+
     .(za35**2*
     .(-(za14*zb13)-za42*zb32))/
     .(-(za13*zb14)-za32*zb42))))/
     .((s12**2-2d0*s12*s34+s34**2-
     .2d0*s12*s56-2d0*s34*s56+s56**2)*
     .za56)+(za32*zb14*
     .(-((za35*za45*zb34)/za56)-
     .(za34*zb36*zb46)/zb56))/
     .(-(za13*zb14)-za32*zb42)+
     .((-s12+s34-s56)*za34*
     .(-((za32*zb13+za42*zb14)*
     .zb36*zb46)-
     .(za32*zb14*
     .(-(za14*zb13)-za42*zb32)*
     .zb46**2)/
     .(-(za13*zb14)-za32*zb42)-
     .za32*zb36*
     .(-(zb16*zb34)+zb13*zb46)))/
     .((s12**2-2d0*s12*s34+s34**2-
     .2d0*s12*s56-2d0*s34*s56+s56**2)*
     .zb56)-(s34*
     .(-((za25**2*zb12)/za56)-
     .(za12*zb16**2)/zb56)*t(i1,i3,i2))/
     .(s12**2-2d0*s12*s34+s34**2-
     .2d0*s12*s56-2d0*s34*s56+s56**2)+
     .(6d0*s12*s34*
     .(za32*zb13+za42*zb14)*
     .(-(za35*zb36)-za45*zb46)*
     .(-t(i1,i3,i2)+t(i1,i4,i2)))/
     .(s12**2-2d0*s12*s34+s34**2-
     .2d0*s12*s56-2d0*s34*s56+s56**2)**2))/
     .(2d0*s12*za13*(-(za13*zb14)-
     .za32*zb42))
      zfac=zfac-
     .(zI3m(s34,s12,s56)*za32*
     .((za32*zb14*
     .(-(za14*zb13)-za42*zb32)*
     .(za13*(za35*zb13+
     .za45*zb14)-
     .za32*(-(za35*zb32)-
     .za45*zb42)+
     .za34*(-(za15*zb14)+
     .za25*zb42))*zb46)/
     .(-(za13*zb14)-za32*zb42)-
     .zb13*(za32*
     .(za14*(za35*zb13+
     .za45*zb14)-za34*
     .(-(za15*zb13)+za25*zb32)-za42*
     .(-(za35*zb32)-za45*zb42))+
     .za34*(-(za32*
     .(-(za15*zb13)+za25*zb32))-za42*
     .(-(za15*zb14)+za25*zb42)))*
     .zb46+(s34*
     .(za32*zb13+za42*zb14)*
     .(-(za35*zb36)-za45*zb46))/2+
     .za42*zb34*
     .(za34*za25*zb12*zb46+
     .(-(za34*zb14)-za32*zb12)*
     .(-(za35*zb36)-za45*zb46))+
     .(3d0*s34*(-s12+s34-s56)*
     .(za32*zb13+za42*zb14)*
     .(-(za35*zb36)-za45*zb46)*
     .(-t(i1,i3,i2)+t(i1,i4,i2)))/
     .(2d0*(s12**2-2d0*s12*s34+s34**2-
     .2d0*s12*s56-2d0*s34*s56+s56**2))))/
     .((s12**2-2d0*s12*s34+s34**2-
     .2d0*s12*s56-2d0*s34*s56+s56**2)*za13*
     .(-(za13*zb14)-za32*zb42))
      zfac=zfac-
     .(zzlogRat(-s12,-s56)*za32*
     .(4d0*(za35*za42*zb16*zb34+
     .za34*za25*zb14*zb36)-
     .(8d0*s34*za32*za56*zb16*zb46)/
     .(-(za13*zb14)-za32*zb42)-
     .2d0*(-(za32*zb13)+za42*zb14)*
     .(-(za35*zb36)-za45*zb46)+
     .(za32*zb13+za42*zb14-
     .(4d0*s56*za32*zb14)/
     .(-(za13*zb14)-za32*zb42))*
     .((za35*za45*zb34)/za56-
     .(za34*zb36*zb46)/zb56)-
     .(s12-s34-s56)*
     .(-((za35*za25*zb13)/za56)-
     .(za42*zb16*zb46)/zb56-
     .(2d0*za32*zb46*
     .(za34*zb13*zb46-
     .za42*zb12*zb46-
     .za35*zb13*zb56))/
     .((-(za13*zb14)-za32*zb42)*zb56))+
     .(3d0*(-s12-s34+s56)*
     .(za32*zb13+za42*zb14)*
     .(-(za35*zb36)-za45*zb46)*
     .(-t(i1,i3,i2)+t(i1,i4,i2)))/
     .(s12**2-2d0*s12*s34+s34**2-
     .2d0*s12*s56-2d0*s34*s56+s56**2)))/
     .(2d0*(s12**2-2d0*s12*s34+s34**2-
     .2d0*s12*s56-2d0*s34*s56+s56**2)*za13*
     .(-(za13*zb14)-za32*zb42))

      zFscqpqm=zFscqpqm+za12/za32*zfac

      zFscqpqm=zFscqpqm-
     .(za15*zb13*((zL0d(-t(i3,i4,i2),-s56)*
     .(za45*zb34+za25*zb32))/(s56*za56)
     .+(zL1d(-s56,-t(i3,i4,i2))*(-(za14*zb16*zb34)-
     .za12*zb16*zb32))/t(i3,i4,i2)**2))/
     .(2d0*zb42*(za13*zb32+za14*zb42))+
     .(za42*zb34*(-(za35*zb32)-za45*zb42)*
     .((zL1d(-t(i3,i4,i2),-s34)*za25*zb32)/s34**2+
     .(zL0d(-t(i3,i4,i2),-s34)*
     .(za45*zb34+za25*zb32))/(s34*t(i3,i4,i2))
     .))/(2d0*za56*zb42*(za13*zb32+za14*zb42))+
     .(-((za14*za45*za25)/
     .(za13*za34*za56*
     .(za13*zb32+za14*zb42)))-
     .(za15*za45*zb13)/
     .(za34*za56*zb42*
     .(za13*zb32+za14*zb42))-
     .(za45**2*zb14**2)/
     .(za34*za56*zb12*zb42*
     .(-(za13*zb14)-za32*zb42))-
     .(s42*za15*za25)/
     .(za13*za56*(za13*zb32+za14*zb42)*
     .(-(za13*zb14)-za32*zb42))+
     .(za15*za42*za45*zb14)/
     .(za34*za56*(za13*zb32+za14*zb42)*
     .(-(za13*zb14)-za32*zb42))-
     .(za14*zb16**2)/
     .(za13*za34*zb12*zb42*zb56)-
     .(za14*zb13*zb16*zb26)/
     .(za34*zb12*zb42*
     .(za13*zb32+za14*zb42)*zb56)+
     .(za12*za42*zb46*zb26)/
     .(za13*(za13*zb32+za14*zb42)*
     .(-(za13*zb14)-za32*zb42)*zb56)-
     .(s14*za42*zb46*zb26)/
     .(za34*zb42*(za13*zb32+za14*zb42)*
     .(-(za13*zb14)-za32*zb42)*zb56)+
     .(za42*(za13*zb36+za14*zb46)*zb26)/
     .(za13*za34*zb42*
     .(za13*zb32+za14*zb42)*zb56)-
     .(za32*zb36*(-(za12*zb16)-za32*zb36))/
     .(za13*(-(za13*zb14)-za32*zb42)*zb56*
     .t(i1,i3,i2))-(za15*zb13*
     .(za45*zb34+za25*zb32))/
     .(za56*zb42*(za13*zb32+za14*zb42)*
     .t(i3,i4,i2)))/2d0

      zfac=
     .(za25*zb16*(-((-s12-s34+s56)*za14)-
     .2d0*za12*za34*zb32))/
     .((s12**2-2d0*s12*s34+s34**2-2d0*s12*s56-
     .2d0*s34*s56+s56**2)*za13*
     .(za13*zb32+za14*zb42))+
     .(((s12-s34-s56)*za14*zb12-
     .(-s12+s34-s56)*za34*zb32)*
     .(-((za25**2*zb12)/za56)-
     .(za12*zb16**2)/zb56))/
     .(2d0*(s12**2-2d0*s12*s34+s34**2-
     .2d0*s12*s56-2d0*s34*s56+s56**2)*za13*
     .zb12*(za13*zb32+za14*zb42))+
     .(zLsm1_2mh(s12,t(i1,i3,i4),s34,s56)*
     .(-(za14*zb12)-za34*zb32)*
     .(za13*zb36+za14*zb46)**2)/
     .(za13*(za13*zb32+za14*zb42)**3*zb56)
     .-(zL0d(-t(i1,i3,i4),-s34)*za34*zb13*
     .(za13*zb36+za14*zb46)**2)/
     .(s34*za13*(za13*zb32+za14*zb42)**2*
     .zb56)
      zfac=zfac+(zzlogRat(-s34,-s56)*za34*
     .((-6d0*(za32*zb13+za42*zb14)*zb34*
     .(-((-s12-s34+s56)*za14)-
     .2d0*za12*za34*zb32)*
     .(-(za35*zb36)-za45*zb46))/
     .(s12**2-2d0*s12*s34+s34**2-
     .2d0*s12*s56-2d0*s34*s56+s56**2)**2-
     .((za13*zb36+za14*zb46)*
     .(zb12*zb36+zb13*zb26))/
     .(zb12*(za13*zb32+za14*zb42)*
     .zb56)+(-((za25*
     .(za25*zb12*
     .(-(za14*zb34)-za12*zb32)
     .+2d0*(-((s12-s34-s56)*za15*zb13)+
     .(-s12-s34+s56)*za56*
     .zb36)))/za56)+
     .((-s12+s34-s56)*za14*zb34*
     .(-(za15*zb16)+za25*zb26))/
     .(za13*zb32+za14*zb42)-
     .(za42*zb16*zb34*
     .(za13*zb36+za14*zb46))/zb56
     .-(zb34*zb26*(-((2d0*(-s12+s34-s56)*
     .za42-za12*za34*zb13)*zb16)-
     .((-s12+s34-s56)*za14*
     .(za32*zb13+za42*zb14)*zb26)/
     .(za13*zb32+za14*zb42)))/
     .(zb12*zb56)+
     .((-s12+s34-s56)*zb16*zb32*
     .(-(za32*zb36)-za42*zb46-
     .za25*zb56))/(zb12*zb56)+
     .za25*(za14*zb16*zb34+
     .za25*zb32*zb56)+
     .4d0*za15*zb34*
     .(za42*zb16+
     .(za14*za25*zb12*zb56)/
     .(za13*zb32+za14*zb42)))/
     .(s12**2-2d0*s12*s34+s34**2-
     .2d0*s12*s56-2d0*s34*s56+s56**2)))/
     .(2d0*za13*(za13*zb32+za14*zb42))
      zfac=zfac-
     .(zL1d(-s56,-t(i1,i3,i4))*za42*za25*zb26)/
     .(za13*(za13*zb32+za14*zb42)*t(i1,i3,i4))
     .-(zL0d(-t(i1,i3,i4),-s56)*za42*
     .(za13*zb36+za14*zb46)*zb26*t(i1,i3,i4)
     .)/(s56*za13*(za13*zb32+za14*zb42)**2*
     .zb56)+(zzlogRat(-s12,-s56)*
     .((-6d0*za12*za34*
     .(za32*zb13+za42*zb14)*
     .(2d0*za14*zb12*zb34+
     .(-s12-s34+s56)*zb32)*
     .(-(za35*zb36)-za45*zb46))/
     .(s12**2-2d0*s12*s34+s34**2-
     .2d0*s12*s56-2d0*s34*s56+s56**2)+
     .(za25*(-3d0*(s12-s34-s56)*za14*
     .za25*zb12+
     .za14*za12*
     .(za35*zb13+za45*zb14)*zb12
     .+za12*za34*(za35*zb13+za45*zb14)*zb32+
     .2d0*za34*zb13*
     .((-s12+s34-s56)*za15+
     .2d0*za12*za56*zb26)))/za56+
     .(za12*zb16*
     .((-(za14*zb12)-za34*zb32)*
     .(-(za32*zb36)-za42*zb46)+
     .2d0*((s12-s34-s56)*za42*zb26-
     .(-s12-s34+s56)*za45*zb56))
     .)/zb56-(za14*((za15*za25*zb12)/za56+
     .(za12*zb16*zb26)/zb56)*
     .(-s12**2+2d0*s12*s34-s34**2+
     .2d0*s12*s56+2d0*s34*s56-s56**2+
     .(s12-s34-s56)*(t(i1,i3,i4)-t(i3,i4,i2)))
     .)/(za13*zb32+za14*zb42)))/
     .(2d0*(s12**2-2d0*s12*s34+s34**2-
     .2d0*s12*s56-2d0*s34*s56+s56**2)*za13*
     .(za13*zb32+za14*zb42))
      zfac=zfac+
     .(zI3m(s34,s12,s56)*za34*
     .((-3d0*za12*(za32*zb13+za42*zb14)*
     .zb34*((s12-s34-s56)*za14*
     .zb12-
     .(-s12+s34-s56)*za34*zb32)*
     .(-(za35*zb36)-za45*zb46))/
     .(s12**2-2d0*s12*s34+s34**2-
     .2d0*s12*s56-2d0*s34*s56+s56**2)**2-
     .(za25*zb13*
     .(za13*zb36+za14*zb46))/t(i1,i3,i4)-
     .(-(za42*zb16*zb34*
     .((-s12+s34-s56)*za15-
     .za25*
     .(za13*zb32+za14*zb42)+
     .2d0*za12*za56*zb26))+
     .(za32*zb13+za42*zb14)*
     .(-(za12*zb32*
     .(-(za35*zb36)-za45*zb46))+
     .3d0*za14*za25*zb34*zb26)-
     .za25*zb13*
     .((s12-s34-s56)*za12*zb26-
     .(-s12-s34+s56)*za15*zb56)-
     .(za14*za15*zb34*
     .(za32*zb13*zb26+
     .za42*zb14*zb26)*
     .(-t(i1,i3,i4)+t(i3,i4,i2)))/
     .(za13*zb32+za14*zb42))/
     .(s12**2-2d0*s12*s34+s34**2-
     .2d0*s12*s56-2d0*s34*s56+s56**2)))/
     .(za13*(za13*zb32+za14*zb42))

      zFscqpqm=zFscqpqm+za14/za34*zfac

      return
      end function

      function zFaxslpm(i1,i2,i3,i4,i5,i6,emtsq)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

      t(j1,j2,j3)=s6(j1,j2)+s6(j2,j3)+s6(j3,j1)

      s56  = s6(i5,i6)
      zb13 = zB6(i1,i3)
      za25 = zA6(i2,i5)
      zb12 = zB6(i1,i2)
      za45 = zA6(i4,i5)
      zb14 = zB6(i1,i4)
      zb36 = zB6(i3,i6)
      zb24 = zB6(i2,i4)
      za24 = zA6(i2,i4)
      za12 = zA6(i1,i2)
      zb16 = zB6(i1,i6)
      za23 = zA6(i2,i3)
      za13 = zA6(i1,i3)

CC comment out emtsq power corection terms for large mt limit

      zFaxslpm = (2d0*(
C      . +1d0/(24d0*emtsq)
     . -zL1d(-t(i1,i2,i4),-s56)/(2d0*s56))*zb13*
     .(za25*zb12+za45*zb14)*zb36)/
     .(s56*zb14*zb24)+
     .(2d0*(
C      . +1d0/(24d0*emtsq)
     . -zL1d(-t(i1,i2,i3),-s56)/(2d0*s56))*za24*za45*
     .(-(za12*zb16)+za23*zb36))/
     .(s56*za13*za23)

      return
      end function

      function zFaxslpp(i1,i2,i3,i4,i5,i6,emtsq)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

      t(j1,j2,j3)=s6(j1,j2)+s6(j2,j3)+s6(j3,j1)

      s56  = s6(i5,i6)
      za25 = zA6(i2,i5)
      za12 = zA6(i1,i2)
      zb13 = zB6(i1,i3)
      za24 = zA6(i2,i4)
      zb34 = zB6(i3,i4)
      zb36 = zB6(i3,i6)
      za14 = zA6(i1,i4)
      zb14 = zB6(i1,i4)
      za23 = zA6(i2,i3)
      zb46 = zB6(i4,i6)
      za13 = zA6(i1,i3)

CC comment out emtsq power corection terms for large mt limit

      zFaxslpp=(2d0*(
C      . +1d0/(24d0*emtsq)
     . -zL1d(-t(i1,i2,i4),-s56)/(2d0*s56))*za25*
     .(-(za12*zb13)-za24*zb34)*zb36)/
     .(s56*za14*za24)+
     .(2d0*(
C      . +1d0/(24d0*emtsq)
     . -zL1d(-t(i1,i2,i3),-s56)/(2d0*s56))*za25*
     .(-(za12*zb14)+za23*zb34)*zb46)/
     .(s56*za13*za23)


      return
      end function

      function zampLoopnf(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)

      za23 = zA6(i2,i3)
      s56  = s6(i5,i6)
      zb61 = zB6(i6,i1)
      za13 = zA6(i1,i3)
      zb62 = zB6(i6,i2)
      zb31 = zB6(i3,i1)
      za45 = zA6(i4,i5)
      s12  = s6(i1,i2)
      s13  = s6(i1,i3)
      s23  = s6(i2,i3)
      za54 = zA6(i5,i4)
      zb43 = zB6(i4,i3)
      za52 = zA6(i5,i2)
      zb23 = zB6(i2,i3)
      za34 = zA6(i3,i4)
      zb16 = zB6(i1,i6)
      s24  = s6(i2,i4)
      s34  = s6(i3,i4)

!      zampLoopnfA=(1d0/3d0)/za23**2/s56*(
!     .        -(zb61*za13+zb62*za23)*zb31
!     .         *za45/(s12+s13+s23)
!     .        +(za54*zb43+za52*zb23)*za34
!     .         *zb16/(s23+s24+s34))
      zampLoopnf=(1d0/3d0)/za23/s56*(
     .       zb16*za54/za23*(
     .          -za13*zb31/(s12+s13+s23)
     .          +zb43*za34/(s23+s24+s34) ) 
     .     -zb62*zb31*za45/(s12+s13+s23)
     .     +za52*(zb23/za23)*za34*zb16/(s23+s24+s34)
     .        ) 

      return
      end function

      function zampBh2g1Z(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)
      common /order/iorder
      complex*16 zampBh2g1Z(2,2)

      do ii=1,2
         do jj=1,2
            zampBh2g1Z = dcmplx(0d0,0d0)
         enddo
      enddo

      if(iorder.eq.0)then
         call swapzAB_kin(6)
         zampBh2g1Z(1,1) = zampLoopnf(i4,i2,i3,i1,i6,i5,renscale2)
         call swapzAB_kin(6)
         zampBh2g1Z(2,2) =-zampLoopnf(i1,i2,i3,i4,i5,i6,renscale2)
      endif

      return
      end function



      function zFaxslqqbgg(i1,i2,i3,i4,i5,i6,renscale2)
      use KinData_mod
      use Parameters_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /proc/ isub(7)
      double precision :: emt,etwidth,emb,ebwidth,emc,ecwidth,emtau,etauwidth
      common /Qmass/ emt,etwidth,emb,ebwidth,emc,ecwidth,emtau,etauwidth
      complex*16 zFaxslqqbgg(2,2)

      if (emt.eq.0d0)then
         write(*,*) "Error in axial vector contribution to B2g1Z,
     .the mass of the top has not been defined"
         stop
      endif

      emtsq = emt**2

      zFaxslqqbgg(2,1) = zFaxslpm(i1,i4,i2,i3,i5,i6,emtsq)
      zFaxslqqbgg(1,2) = zFaxslpm(i1,i4,i3,i2,i5,i6,emtsq)
      call swapzAB_kin(6)
c      zFaxslqqbgg(2,1) = zFaxslpm(i4,i1,i3,i2,i6,i5,emtsq)
c      zFaxslqqbgg(1,2) = zFaxslpm(i4,i1,i2,i3,i6,i5,emtsq)
      zFaxslqqbgg(1,1) = zFaxslpp(i4,i1,i2,i3,i6,i5,emtsq)
      call swapzAB_kin(6)
      zFaxslqqbgg(2,2) = zFaxslpp(i1,i4,i3,i2,i5,i6,emtsq)

      return
      end function

c      function zampB2g1Zax(i1,i2,i3,i4,i5,i6,renscale2)
c      use KinData_mod
c      use Parameters_mod
c      implicit double precision (a-h,o-y)
c      implicit complex*16 (z)
c      parameter(pi=3.141592653589793238d0)
c      common /koppel/ nf,nc,b0
c      common /strong/ as
c!$omp threadprivate(/strong/)
c      common /proc/ isub(7)
c      double precision :: emt,etwidth,emb,ebwidth,emc,ecwidth,emtau,etauwidth
c      common /Qmass/ emt,etwidth,emb,ebwidth,emc,ecwidth,emtau,etauwidth
c      complex*16 zFax(2,2),zFaxsl(2,2)
c
c
c      zFax = zFaxqqbgg(i1,i2,i3,i4,i5,i6,renscale2) ! zFaxqqbgg(i1,i3,i2,i4,i5,i6,renscale2)
c      zFaxsl = zFaxslqqbgg(i1,i2,i3,i4,i5,i6,renscale2)
c
c
c      zampB2g1Zax = 0d0
c      print*, "zampB2g1Zax: return value not set!"
c      stop
c
c      return
c      end function

      end module B2gZ
