!************************************************************************
!  All 1-loop amplitudes in this file are based on
!  Z.~Bern, L.~J.~Dixon and D.~A.~Kosower,
!  %``One loop corrections to two quark three gluon amplitudes,''
!  Nucl. Phys. B \textbf{437} (1995), 259-304
!  doi:10.1016/0550-3213(94)00542-M
!  [arXiv:hep-ph/9409393 [hep-ph]].
!  
!  A. Signer,
!  Helicity method for next-to-leading order corrections in QCD
!  PhD thesis, 
!  ETH-Z¨urich Nr. 11143 (1995).
************************************************************************
* three gluon two quark tree level
*
* one colour ordering, summed and squared
*
*
      function BB3g0(i1,i2,i3,i4,i5)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /scheme/ deltaR
      common /order/iorder
      deltaR=1d0
*      write(*,*)" hello BB3g0 ", iorder

*
      BB3g0=0d0

      do ihel=1,3
      do iflip=0,3
      Bb3g0 = BB3g0
     . + zB123tree(ihel,iflip,i1,i2,i3,i4,i5)*dconjg(zB123tree(ihel,iflip,i1,i2,i3,i4,i5))
      enddo
      enddo

c      write(*,*)"BB3g0",i1,i2,i3,i4,i5,BB3g0

      return
      end
************************************************************************
* three gluon two quark tree level, symmetrized over i2
* (external 'photon' i2)
* one colour ordering, summed and squared
*
*
      function BBt3g0(i1,i2,i3,i4,i5)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /scheme/ deltaR
      common /order/iorder
      deltaR=1d0
*      write(*,*)" hello BBt3g0 ", iorder

*
      BBt3g0=0d0
      do ihel=1,3
      do iflip=0,3
      BBt3g0 = BBt3g0
     . + zBt123tree(ihel,iflip,i1,i2,i3,i4,i5)*dconjg(zBt123tree(ihel,iflip,i1,i2,i3,i4,i5))
      enddo
      enddo

c      write(*,*)"BBt3g0",i1,i2,i3,i4,i5,BBt3g0

      return
      end
************************************************************************
* three gluon two quark tree level, symmetrized over all gluons
* (external 'photons')
* one colour ordering, summed and squared
*
*
      function BBtt3g0(i1,i2,i3,i4,i5)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /scheme/ deltaR
      common /order/iorder
      deltaR=1d0
*      write(*,*)" hello BBtt3g0 ", iorder

*
      BBtt3g0=0d0
      do ihel=1,3
      do iflip=0,3
      BBtt3g0 = BBtt3g0
     . + zBtt123tree(ihel,iflip,i1,i2,i3,i4,i5)*dconjg(zBtt123tree(ihel,iflip,i1,i2,i3,i4,i5))
      enddo
      enddo

c      write(*,*)"Btt3g0",i1,i2,i3,i4,i5,BBtt3g0,zBBtt3g0

      return
      end
************************************************************************
*
* three gluon two quark helicity tree amplitudes zB123tree
*
*
      function zB123tree(ihel,iflip,i1,i2,i3,i4,i5)
      use KinData_mod, only : zA5, zB5
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /order/iorder
      parameter(pi=3.141592653589793238d0)

*      iflip = 1 -> A(i,j) <-> B(j,i) and line reversal
*      ihel = 1  iflip = 0 pmppm
*      ihel = 2  iflip = 0 ppmpm
*      ihel = 3  iflip = 0 pppmm
*      ihel = 1  iflip = 1 pmmpm
*      ihel = 2  iflip = 1 pmpmm
*      ihel = 3  iflip = 1 ppmmm
*      ihel = 1  iflip = 2 mppmp
*      ihel = 2  iflip = 2 mpmpp
*      ihel = 3  iflip = 2 mmppp
*      ihel = 1  iflip = 3 mpmmp
*      ihel = 2  iflip = 3 mmpmp
*      ihel = 3  iflip = 3 mmmpp

      if(iflip.eq.0.or.iflip.eq.3)then
        j1=i1
        j2=i2
        j3=i3
        j4=i4
        j5=i5
      elseif(iflip.eq.1.or.iflip.eq.2)then
        j1=i5
        j2=i4
        j3=i3
        j4=i2
        j5=i1
      endif
      if(iflip.eq.0.or.iflip.eq.2)then
        za12=zA5(j1,j2)
        za21=zA5(j2,j1)
        za13=zA5(j1,j3)
        za31=zA5(j3,j1)
        za14=zA5(j1,j4)
        za41=zA5(j4,j1)
        za15=zA5(j1,j5)
        za51=zA5(j5,j1)
        za23=zA5(j2,j3)
        za32=zA5(j3,j2)
        za24=zA5(j2,j4)
        za42=zA5(j4,j2)
        za25=zA5(j2,j5)
        za52=zA5(j5,j2)
        za34=zA5(j3,j4)
        za43=zA5(j4,j3)
        za35=zA5(j3,j5)
        za53=zA5(j5,j3)
        za45=zA5(j4,j5)
        za54=zA5(j5,j4)

        zb12=zB5(j1,j2)
        zb21=zB5(j2,j1)
        zb13=zB5(j1,j3)
        zb31=zB5(j3,j1)
        zb14=zB5(j1,j4)
        zb41=zB5(j4,j1)
        zb15=zB5(j1,j5)
        zb51=zB5(j5,j1)
        zb23=zB5(j2,j3)
        zb32=zB5(j3,j2)
        zb24=zB5(j2,j4)
        zb42=zB5(j4,j2)
        zb25=zB5(j2,j5)
        zb52=zB5(j5,j2)
        zb34=zB5(j3,j4)
        zb43=zB5(j4,j3)
        zb35=zB5(j3,j5)
        zb53=zB5(j5,j3)
        zb45=zB5(j4,j5)
        zb54=zB5(j5,j4)
      elseif(iflip.eq.1.or.iflip.eq.3)then
        za12=zB5(j1,j2)
        za21=zB5(j2,j1)
        za13=zB5(j1,j3)
        za31=zB5(j3,j1)
        za14=zB5(j1,j4)
        za41=zB5(j4,j1)
        za15=zB5(j1,j5)
        za51=zB5(j5,j1)
        za23=zB5(j2,j3)
        za32=zB5(j3,j2)
        za24=zB5(j2,j4)
        za42=zB5(j4,j2)
        za25=zB5(j2,j5)
        za52=zB5(j5,j2)
        za34=zB5(j3,j4)
        za43=zB5(j4,j3)
        za35=zB5(j3,j5)
        za53=zB5(j5,j3)
        za45=zB5(j4,j5)
        za54=zB5(j5,j4)

        zb12=zA5(j1,j2)
        zb21=zA5(j2,j1)
        zb13=zA5(j1,j3)
        zb31=zA5(j3,j1)
        zb14=zA5(j1,j4)
        zb41=zA5(j4,j1)
        zb15=zA5(j1,j5)
        zb51=zA5(j5,j1)
        zb23=zA5(j2,j3)
        zb32=zA5(j3,j2)
        zb24=zA5(j2,j4)
        zb42=zA5(j4,j2)
        zb25=zA5(j2,j5)
        zb52=zA5(j5,j2)
        zb34=zA5(j3,j4)
        zb43=zA5(j4,j3)
        zb35=zA5(j3,j5)
        zb53=zA5(j5,j3)
        zb45=zA5(j4,j5)
        zb54=zA5(j5,j4)
      endif

      zi = (0d0,1d0)
*                           A B               B A               A B               B A
*                          12345             54321             54321             12345
*      ihel = 1  iflip = 0 pmppm   iflip = 1 pmmpm   iflip = 2 mppmp   iflip = 3 mpmmp
*      ihel = 2  iflip = 0 ppmpm   iflip = 1 pmpmm   iflip = 2 mpmpp   iflip = 3 mmpmp
*      ihel = 3  iflip = 0 pppmm   iflip = 1 ppmmm   iflip = 2 mmppp   iflip = 3 mmmpp

      if(ihel.eq.1)then
        ztree=za12*za52**(3)/(za12*za23*za34*za45*za51)
      elseif(ihel.eq.2)then
        ztree=za13*za53**(3)/(za12*za23*za34*za45*za51)
      elseif(ihel.eq.3)then
        ztree=za14*za54**(3)/(za12*za23*za34*za45*za51)
      endif

      zB123tree=ztree

c      write(*,*)"zB123tree ", ihel,iflip,zB123tree
      return
      end
************************************************************************
*
* three gluon two quark helicity tree amplitudes zBt123tree
*
*
      function zBt123tree(ihel,iflip,i1,i2,i3,i4,i5)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /order/iorder
      parameter(pi=3.141592653589793238d0)


*      iflip = 1 -> A(i,j) <-> B(j,i) and line reversal
*                           234      324      342
*      ihel = 1  iflip = 0 pmppm -> ppmpm -> pppmm   1 2 3
*      ihel = 2  iflip = 0 ppmpm -> pmppm -> pmppm   2 1 1
*      ihel = 3  iflip = 0 pppmm -> pppmm -> ppmpm   3 3 2
*      ihel = 1  iflip = 1 pmmpm -> pmmpm -> pmpmm   1 1 2
*      ihel = 2  iflip = 1 pmpmm -> ppmmm -> ppmmm   2 3 3
*      ihel = 3  iflip = 1 ppmmm -> pmpmm -> pmmpm   3 2 1
*
*      i1,i2,i3,i4,i5 -> i5,i4,i3,i2,i1
*
*      ihel = 1  iflip = 2 mppmp -> mppmp -> mpmpp   1 1 2
*      ihel = 2  iflip = 2 mpmpp -> mmppp -> mmppp   2 3 3
*      ihel = 3  iflip = 2 mmppp -> mpmpp -> mppmp   3 2 1
*      ihel = 1  iflip = 3 mpmmp -> mmpmp -> mmmpp   1 2 3
*      ihel = 2  iflip = 3 mmpmp -> mpmmp -> mpmmp   2 1 1
*      ihel = 3  iflip = 3 mmmpp -> mmmpp -> mmpmp   3 3 2

*                           A B               B A               A B               B A
*                          12345             54321             54321             12345
*      ihel = 1  iflip = 0 pmppm   iflip = 1 pmmpm   iflip = 2 mppmp   iflip = 3 mpmmp
*      ihel = 2  iflip = 0 ppmpm   iflip = 1 pmpmm   iflip = 2 mpmpp   iflip = 3 mmpmp
*      ihel = 3  iflip = 0 pppmm   iflip = 1 ppmmm   iflip = 2 mmppp   iflip = 3 mmmpp

      if(iflip.eq.0.or.iflip.eq.3)then
      if(ihel.eq.1)then

        ztree=zB123tree(1,iflip,i1,i2,i3,i4,i5)
     .       +zB123tree(2,iflip,i1,i3,i2,i4,i5)
     .       +zB123tree(3,iflip,i1,i3,i4,i2,i5)

      elseif(ihel.eq.2)then

        ztree=zB123tree(2,iflip,i1,i2,i3,i4,i5)
     .       +zB123tree(1,iflip,i1,i3,i2,i4,i5)
     .       +zB123tree(1,iflip,i1,i3,i4,i2,i5)

      elseif(ihel.eq.3)then

        ztree=zB123tree(3,iflip,i1,i2,i3,i4,i5)
     .       +zB123tree(3,iflip,i1,i3,i2,i4,i5)
     .       +zB123tree(2,iflip,i1,i3,i4,i2,i5)
      endif
      elseif(iflip.eq.1.or.iflip.eq.2)then
      if(ihel.eq.1)then

        ztree=zB123tree(1,iflip,i1,i2,i3,i4,i5)
     .       +zB123tree(1,iflip,i1,i3,i2,i4,i5)
     .       +zB123tree(2,iflip,i1,i3,i4,i2,i5)

      elseif(ihel.eq.2)then

        ztree=zB123tree(2,iflip,i1,i2,i3,i4,i5)
     .       +zB123tree(3,iflip,i1,i3,i2,i4,i5)
     .       +zB123tree(3,iflip,i1,i3,i4,i2,i5)

      elseif(ihel.eq.3)then

        ztree=zB123tree(3,iflip,i1,i2,i3,i4,i5)
     .       +zB123tree(2,iflip,i1,i3,i2,i4,i5)
     .       +zB123tree(1,iflip,i1,i3,i4,i2,i5)

      endif
      endif

      zBt123tree=ztree

c      write(*,*)"zBt123tree ",iorder, ihel,iflip,zBt123tree
      return
      end
************************************************************************
*
* three gluon two quark helicity tree amplitudes zBtt123tree
*
*
      function zBtt123tree(ihel,iflip,i1,i2,i3,i4,i5)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /order/iorder
      parameter(pi=3.141592653589793238d0)


*      iflip = 1 -> A(i,j) <-> B(j,i) and line reversal
*                           234      243      324      342      423      432
*      ihel = 1  iflip = 0 pmppm -> pmppm -> ppmpm -> pppmm -> ppmpm -> pppmm   1 1 2 3 2 3
*      ihel = 2  iflip = 0 ppmpm -> pppmm -> pmppm -> pmppm -> pppmm -> ppmpm   2 3 1 1 3 2
*      ihel = 3  iflip = 0 pppmm -> ppmpm -> pppmm -> ppmpm -> pmppm -> pmppm   3 2 3 2 1 1
*      ihel = 1  iflip = 1 pmmpm -> pmpmm -> pmmpm -> pmpmm -> ppmmm -> ppmmm   1 2 1 2 3 3
*      ihel = 2  iflip = 1 pmpmm -> pmmpm -> ppmmm -> ppmmm -> pmmpm -> pmpmm   2 1 3 3 1 2
*      ihel = 3  iflip = 1 ppmmm -> ppmmm -> pmpmm -> pmmpm -> pmpmm -> pppmm   3 3 2 1 2 1
*
*      i1,i2,i3,i4,i5 -> i5,i4,i3,i2,i1
*
*      ihel = 1  iflip = 2 mppmp -> mpmpp -> mppmp -> mpmpp -> mmppp -> mmppp   1 2 1 2 3 3
*      ihel = 2  iflip = 2 mpmpp -> mppmp -> mmppp -> mmppp -> mppmp -> mpmpp   2 1 3 3 1 2
*      ihel = 3  iflip = 2 mmppp -> mmppp -> mpmpp -> mppmp -> mpmpp -> mppmp   3 3 2 1 2 1
*      ihel = 1  iflip = 3 mpmmp -> mpmmp -> mmpmp -> mmmpp -> mmpmp -> mmmpp   1 1 2 3 2 3
*      ihel = 2  iflip = 3 mmpmp -> mmmpp -> mpmmp -> mpmmp -> mmmpp -> mmpmp   2 3 1 1 3 2
*      ihel = 3  iflip = 3 mmmpp -> mmpmp -> mmmpp -> mmpmp -> mpmmp -> mpmmp   3 2 3 2 1 1

*                           A B               B A               A B               B A
*                          12345             54321             54321             12345
*      ihel = 1  iflip = 0 pmppm   iflip = 1 pmmpm   iflip = 2 mppmp   iflip = 3 mpmmp
*      ihel = 2  iflip = 0 ppmpm   iflip = 1 pmpmm   iflip = 2 mpmpp   iflip = 3 mmpmp
*      ihel = 3  iflip = 0 pppmm   iflip = 1 ppmmm   iflip = 2 mmppp   iflip = 3 mmmpp


      if(iflip.eq.0.or.iflip.eq.3)then
      if(ihel.eq.1)then

        ztree=zB123tree(1,iflip,i1,i2,i3,i4,i5)
     .       +zB123tree(1,iflip,i1,i2,i4,i3,i5)
     .       +zB123tree(2,iflip,i1,i3,i2,i4,i5)
     .       +zB123tree(3,iflip,i1,i3,i4,i2,i5)
     .       +zB123tree(2,iflip,i1,i4,i2,i3,i5)
     .       +zB123tree(3,iflip,i1,i4,i3,i2,i5)

      elseif(ihel.eq.2)then

        ztree=zB123tree(2,iflip,i1,i2,i3,i4,i5)
     .       +zB123tree(3,iflip,i1,i2,i4,i3,i5)
     .       +zB123tree(1,iflip,i1,i3,i2,i4,i5)
     .       +zB123tree(1,iflip,i1,i3,i4,i2,i5)
     .       +zB123tree(3,iflip,i1,i4,i2,i3,i5)
     .       +zB123tree(2,iflip,i1,i4,i3,i2,i5)

      elseif(ihel.eq.3)then

        ztree=zB123tree(3,iflip,i1,i2,i3,i4,i5)
     .       +zB123tree(2,iflip,i1,i2,i4,i3,i5)
     .       +zB123tree(3,iflip,i1,i3,i2,i4,i5)
     .       +zB123tree(2,iflip,i1,i3,i4,i2,i5)
     .       +zB123tree(1,iflip,i1,i4,i2,i3,i5)
     .       +zB123tree(1,iflip,i1,i4,i3,i2,i5)

      endif
      elseif(iflip.eq.1.or.iflip.eq.2)then
      if(ihel.eq.1)then

        ztree=zB123tree(1,iflip,i1,i2,i3,i4,i5)
     .       +zB123tree(2,iflip,i1,i2,i4,i3,i5)
     .       +zB123tree(1,iflip,i1,i3,i2,i4,i5)
     .       +zB123tree(2,iflip,i1,i3,i4,i2,i5)
     .       +zB123tree(3,iflip,i1,i4,i2,i3,i5)
     .       +zB123tree(3,iflip,i1,i4,i3,i2,i5)

      elseif(ihel.eq.2)then

        ztree=zB123tree(2,iflip,i1,i2,i3,i4,i5)
     .       +zB123tree(1,iflip,i1,i2,i4,i3,i5)
     .       +zB123tree(3,iflip,i1,i3,i2,i4,i5)
     .       +zB123tree(3,iflip,i1,i3,i4,i2,i5)
     .       +zB123tree(1,iflip,i1,i4,i2,i3,i5)
     .       +zB123tree(2,iflip,i1,i4,i3,i2,i5)

      elseif(ihel.eq.3)then

        ztree=zB123tree(3,iflip,i1,i2,i3,i4,i5)
     .       +zB123tree(3,iflip,i1,i2,i4,i3,i5)
     .       +zB123tree(2,iflip,i1,i3,i2,i4,i5)
     .       +zB123tree(1,iflip,i1,i3,i4,i2,i5)
     .       +zB123tree(2,iflip,i1,i4,i2,i3,i5)
     .       +zB123tree(1,iflip,i1,i4,i3,i2,i5)

      endif
      endif

      zBtt123tree=ztree

c      write(*,*)"zBt123tree ",iorder, ihel,iflip,zBt123tree
      return
      end
************************************************************************
*
* three gluon two quark helicity tree amplitudes zB3g0d1tree
*
*   zB3g0d1tree = B0(1,2,3)+B0(1,3,2)+B0(2,3,1)+B0(3,2,1)
*
      function zB3g0d1tree(ihel,iflip,i1,i2,i3,i4,i5)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /order/iorder
      parameter(pi=3.141592653589793238d0)


*      iflip = 1 -> A(i,j) <-> B(j,i) and line reversal
*                           234      243      324      342      423      432
*      ihel = 1  iflip = 0 pmppm -> pmppm -> ppmpm -> pppmm -> ppmpm -> pppmm   1 1 2 3 2 3
*      ihel = 2  iflip = 0 ppmpm -> pppmm -> pmppm -> pmppm -> pppmm -> ppmpm   2 3 1 1 3 2
*      ihel = 3  iflip = 0 pppmm -> ppmpm -> pppmm -> ppmpm -> pmppm -> pmppm   3 2 3 2 1 1
*      ihel = 1  iflip = 1 pmmpm -> pmpmm -> pmmpm -> pmpmm -> ppmmm -> ppmmm   1 2 1 2 3 3
*      ihel = 2  iflip = 1 pmpmm -> pmmpm -> ppmmm -> ppmmm -> pmmpm -> pmpmm   2 1 3 3 1 2
*      ihel = 3  iflip = 1 ppmmm -> ppmmm -> pmpmm -> pmmpm -> pmpmm -> pppmm   3 3 2 1 2 1
*
*      i1,i2,i3,i4,i5 -> i5,i4,i3,i2,i1
*
*      ihel = 1  iflip = 2 mppmp -> mpmpp -> mppmp -> mpmpp -> mmppp -> mmppp   1 2 1 2 3 3
*      ihel = 2  iflip = 2 mpmpp -> mppmp -> mmppp -> mmppp -> mppmp -> mpmpp   2 1 3 3 1 2
*      ihel = 3  iflip = 2 mmppp -> mmppp -> mpmpp -> mppmp -> mpmpp -> mppmp   3 3 2 1 2 1
*      ihel = 1  iflip = 3 mpmmp -> mpmmp -> mmpmp -> mmmpp -> mmpmp -> mmmpp   1 1 2 3 2 3
*      ihel = 2  iflip = 3 mmpmp -> mmmpp -> mpmmp -> mpmmp -> mmmpp -> mmpmp   2 3 1 1 3 2
*      ihel = 3  iflip = 3 mmmpp -> mmpmp -> mmmpp -> mmpmp -> mpmmp -> mpmmp   3 2 3 2 1 1

*                           A B               B A               A B               B A
*                          12345             54321             54321             12345
*      ihel = 1  iflip = 0 pmppm   iflip = 1 pmmpm   iflip = 2 mppmp   iflip = 3 mpmmp
*      ihel = 2  iflip = 0 ppmpm   iflip = 1 pmpmm   iflip = 2 mpmpp   iflip = 3 mmpmp
*      ihel = 3  iflip = 0 pppmm   iflip = 1 ppmmm   iflip = 2 mmppp   iflip = 3 mmmpp


      if(iflip.eq.0.or.iflip.eq.3)then
      if(ihel.eq.1)then

        ztree=zB123tree(1,iflip,i1,i2,i3,i4,i5)
     .       +zB123tree(1,iflip,i1,i2,i4,i3,i5)
     .       +zB123tree(3,iflip,i1,i3,i4,i2,i5)
     .       +zB123tree(3,iflip,i1,i4,i3,i2,i5)

      elseif(ihel.eq.2)then

        ztree=zB123tree(2,iflip,i1,i2,i3,i4,i5)
     .       +zB123tree(3,iflip,i1,i2,i4,i3,i5)
     .       +zB123tree(1,iflip,i1,i3,i4,i2,i5)
     .       +zB123tree(2,iflip,i1,i4,i3,i2,i5)

      elseif(ihel.eq.3)then

        ztree=zB123tree(3,iflip,i1,i2,i3,i4,i5)
     .       +zB123tree(2,iflip,i1,i2,i4,i3,i5)
     .       +zB123tree(2,iflip,i1,i3,i4,i2,i5)
     .       +zB123tree(1,iflip,i1,i4,i3,i2,i5)

      endif
      elseif(iflip.eq.1.or.iflip.eq.2)then
      if(ihel.eq.1)then

        ztree=zB123tree(1,iflip,i1,i2,i3,i4,i5)
     .       +zB123tree(2,iflip,i1,i2,i4,i3,i5)
     .       +zB123tree(2,iflip,i1,i3,i4,i2,i5)
     .       +zB123tree(3,iflip,i1,i4,i3,i2,i5)

      elseif(ihel.eq.2)then

        ztree=zB123tree(2,iflip,i1,i2,i3,i4,i5)
     .       +zB123tree(1,iflip,i1,i2,i4,i3,i5)
     .       +zB123tree(3,iflip,i1,i3,i4,i2,i5)
     .       +zB123tree(2,iflip,i1,i4,i3,i2,i5)

      elseif(ihel.eq.3)then

        ztree=zB123tree(3,iflip,i1,i2,i3,i4,i5)
     .       +zB123tree(3,iflip,i1,i2,i4,i3,i5)
     .       +zB123tree(1,iflip,i1,i3,i4,i2,i5)
     .       +zB123tree(1,iflip,i1,i4,i3,i2,i5)

      endif
      endif

      zB3g0d1tree=ztree

*      write(*,*)"zB3g0d1tree ",iorder, ihel,iflip,zB3g0d1tree
      return
      end

************************************************************************
*
* three gluon two quark helicity tree amplitudes zB3g0d2tree
*
*   zB3g0d2tree = B0(1,3,2)+B0(2,1,3)+B0(2,3,1)+B0(3,1,2)
*
      function zB3g0d2tree(ihel,iflip,i1,i2,i3,i4,i5)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /order/iorder
      parameter(pi=3.141592653589793238d0)


*      iflip = 1 -> A(i,j) <-> B(j,i) and line reversal
*                           234      243      324      342      423      432
*      ihel = 1  iflip = 0 pmppm -> pmppm -> ppmpm -> pppmm -> ppmpm -> pppmm   1 1 2 3 2 3
*      ihel = 2  iflip = 0 ppmpm -> pppmm -> pmppm -> pmppm -> pppmm -> ppmpm   2 3 1 1 3 2
*      ihel = 3  iflip = 0 pppmm -> ppmpm -> pppmm -> ppmpm -> pmppm -> pmppm   3 2 3 2 1 1
*      ihel = 1  iflip = 1 pmmpm -> pmpmm -> pmmpm -> pmpmm -> ppmmm -> ppmmm   1 2 1 2 3 3
*      ihel = 2  iflip = 1 pmpmm -> pmmpm -> ppmmm -> ppmmm -> pmmpm -> pmpmm   2 1 3 3 1 2
*      ihel = 3  iflip = 1 ppmmm -> ppmmm -> pmpmm -> pmmpm -> pmpmm -> pppmm   3 3 2 1 2 1
*
*      i1,i2,i3,i4,i5 -> i5,i4,i3,i2,i1
*
*      ihel = 1  iflip = 2 mppmp -> mpmpp -> mppmp -> mpmpp -> mmppp -> mmppp   1 2 1 2 3 3
*      ihel = 2  iflip = 2 mpmpp -> mppmp -> mmppp -> mmppp -> mppmp -> mpmpp   2 1 3 3 1 2
*      ihel = 3  iflip = 2 mmppp -> mmppp -> mpmpp -> mppmp -> mpmpp -> mppmp   3 3 2 1 2 1
*      ihel = 1  iflip = 3 mpmmp -> mpmmp -> mmpmp -> mmmpp -> mmpmp -> mmmpp   1 1 2 3 2 3
*      ihel = 2  iflip = 3 mmpmp -> mmmpp -> mpmmp -> mpmmp -> mmmpp -> mmpmp   2 3 1 1 3 2
*      ihel = 3  iflip = 3 mmmpp -> mmpmp -> mmmpp -> mmpmp -> mpmmp -> mpmmp   3 2 3 2 1 1

*                           A B               B A               A B               B A
*                          12345             54321             54321             12345
*      ihel = 1  iflip = 0 pmppm   iflip = 1 pmmpm   iflip = 2 mppmp   iflip = 3 mpmmp
*      ihel = 2  iflip = 0 ppmpm   iflip = 1 pmpmm   iflip = 2 mpmpp   iflip = 3 mmpmp
*      ihel = 3  iflip = 0 pppmm   iflip = 1 ppmmm   iflip = 2 mmppp   iflip = 3 mmmpp


      if(iflip.eq.0.or.iflip.eq.3)then
      if(ihel.eq.1)then

        ztree=zB123tree(1,iflip,i1,i2,i4,i3,i5)
     .       +zB123tree(2,iflip,i1,i3,i2,i4,i5)
     .       +zB123tree(3,iflip,i1,i3,i4,i2,i5)
     .       +zB123tree(2,iflip,i1,i4,i2,i3,i5)

      elseif(ihel.eq.2)then

        ztree=zB123tree(3,iflip,i1,i2,i4,i3,i5)
     .       +zB123tree(1,iflip,i1,i3,i2,i4,i5)
     .       +zB123tree(1,iflip,i1,i3,i4,i2,i5)
     .       +zB123tree(3,iflip,i1,i4,i2,i3,i5)

      elseif(ihel.eq.3)then

        ztree=zB123tree(2,iflip,i1,i2,i4,i3,i5)
     .       +zB123tree(3,iflip,i1,i3,i2,i4,i5)
     .       +zB123tree(2,iflip,i1,i3,i4,i2,i5)
     .       +zB123tree(1,iflip,i1,i4,i2,i3,i5)

      endif
      elseif(iflip.eq.1.or.iflip.eq.2)then
      if(ihel.eq.1)then

        ztree=zB123tree(2,iflip,i1,i2,i4,i3,i5)
     .       +zB123tree(1,iflip,i1,i3,i2,i4,i5)
     .       +zB123tree(2,iflip,i1,i3,i4,i2,i5)
     .       +zB123tree(3,iflip,i1,i4,i2,i3,i5)

      elseif(ihel.eq.2)then

        ztree=zB123tree(1,iflip,i1,i2,i4,i3,i5)
     .       +zB123tree(3,iflip,i1,i3,i2,i4,i5)
     .       +zB123tree(3,iflip,i1,i3,i4,i2,i5)
     .       +zB123tree(1,iflip,i1,i4,i2,i3,i5)

      elseif(ihel.eq.3)then

        ztree=zB123tree(3,iflip,i1,i2,i4,i3,i5)
     .       +zB123tree(2,iflip,i1,i3,i2,i4,i5)
     .       +zB123tree(1,iflip,i1,i3,i4,i2,i5)
     .       +zB123tree(2,iflip,i1,i4,i2,i3,i5)

      endif
      endif

      zB3g0d2tree=ztree

c      write(*,*)"zB3g0d2tree ",iorder, ihel,iflip,zB3g0d2tree
      return
      end
************************************************************************
*
* three gluon two quark helicity tree amplitudes zB3g0d3tree
*
*   zB3g0d3tree = B0(1,2,3)+B0(2,1,3)+B0(3,1,2)+B0(3,2,1)
*
      function zB3g0d3tree(ihel,iflip,i1,i2,i3,i4,i5)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /order/iorder
      parameter(pi=3.141592653589793238d0)


*      iflip = 1 -> A(i,j) <-> B(j,i) and line reversal
*                           234      243      324      342      423      432
*      ihel = 1  iflip = 0 pmppm -> pmppm -> ppmpm -> pppmm -> ppmpm -> pppmm   1 1 2 3 2 3
*      ihel = 2  iflip = 0 ppmpm -> pppmm -> pmppm -> pmppm -> pppmm -> ppmpm   2 3 1 1 3 2
*      ihel = 3  iflip = 0 pppmm -> ppmpm -> pppmm -> ppmpm -> pmppm -> pmppm   3 2 3 2 1 1
*      ihel = 1  iflip = 1 pmmpm -> pmpmm -> pmmpm -> pmpmm -> ppmmm -> ppmmm   1 2 1 2 3 3
*      ihel = 2  iflip = 1 pmpmm -> pmmpm -> ppmmm -> ppmmm -> pmmpm -> pmpmm   2 1 3 3 1 2
*      ihel = 3  iflip = 1 ppmmm -> ppmmm -> pmpmm -> pmmpm -> pmpmm -> pppmm   3 3 2 1 2 1
*
*      i1,i2,i3,i4,i5 -> i5,i4,i3,i2,i1
*
*      ihel = 1  iflip = 2 mppmp -> mpmpp -> mppmp -> mpmpp -> mmppp -> mmppp   1 2 1 2 3 3
*      ihel = 2  iflip = 2 mpmpp -> mppmp -> mmppp -> mmppp -> mppmp -> mpmpp   2 1 3 3 1 2
*      ihel = 3  iflip = 2 mmppp -> mmppp -> mpmpp -> mppmp -> mpmpp -> mppmp   3 3 2 1 2 1
*      ihel = 1  iflip = 3 mpmmp -> mpmmp -> mmpmp -> mmmpp -> mmpmp -> mmmpp   1 1 2 3 2 3
*      ihel = 2  iflip = 3 mmpmp -> mmmpp -> mpmmp -> mpmmp -> mmmpp -> mmpmp   2 3 1 1 3 2
*      ihel = 3  iflip = 3 mmmpp -> mmpmp -> mmmpp -> mmpmp -> mpmmp -> mpmmp   3 2 3 2 1 1

*                           A B               B A               A B               B A
*                          12345             54321             54321             12345
*      ihel = 1  iflip = 0 pmppm   iflip = 1 pmmpm   iflip = 2 mppmp   iflip = 3 mpmmp
*      ihel = 2  iflip = 0 ppmpm   iflip = 1 pmpmm   iflip = 2 mpmpp   iflip = 3 mmpmp
*      ihel = 3  iflip = 0 pppmm   iflip = 1 ppmmm   iflip = 2 mmppp   iflip = 3 mmmpp


      if(iflip.eq.0.or.iflip.eq.3)then
      if(ihel.eq.1)then

        ztree=zB123tree(1,iflip,i1,i2,i3,i4,i5)
     .       +zB123tree(2,iflip,i1,i3,i2,i4,i5)
     .       +zB123tree(2,iflip,i1,i4,i2,i3,i5)
     .       +zB123tree(3,iflip,i1,i4,i3,i2,i5)

      elseif(ihel.eq.2)then

        ztree=zB123tree(2,iflip,i1,i2,i3,i4,i5)
     .       +zB123tree(1,iflip,i1,i3,i2,i4,i5)
     .       +zB123tree(3,iflip,i1,i4,i2,i3,i5)
     .       +zB123tree(2,iflip,i1,i4,i3,i2,i5)

      elseif(ihel.eq.3)then

        ztree=zB123tree(3,iflip,i1,i2,i3,i4,i5)
     .       +zB123tree(3,iflip,i1,i3,i2,i4,i5)
     .       +zB123tree(1,iflip,i1,i4,i2,i3,i5)
     .       +zB123tree(1,iflip,i1,i4,i3,i2,i5)

      endif
      elseif(iflip.eq.1.or.iflip.eq.2)then
      if(ihel.eq.1)then

        ztree=zB123tree(1,iflip,i1,i2,i3,i4,i5)
     .       +zB123tree(1,iflip,i1,i3,i2,i4,i5)
     .       +zB123tree(3,iflip,i1,i4,i2,i3,i5)
     .       +zB123tree(3,iflip,i1,i4,i3,i2,i5)

      elseif(ihel.eq.2)then

        ztree=zB123tree(2,iflip,i1,i2,i3,i4,i5)
     .       +zB123tree(3,iflip,i1,i3,i2,i4,i5)
     .       +zB123tree(1,iflip,i1,i4,i2,i3,i5)
     .       +zB123tree(2,iflip,i1,i4,i3,i2,i5)

      elseif(ihel.eq.3)then

        ztree=zB123tree(3,iflip,i1,i2,i3,i4,i5)
     .       +zB123tree(2,iflip,i1,i3,i2,i4,i5)
     .       +zB123tree(2,iflip,i1,i4,i2,i3,i5)
     .       +zB123tree(1,iflip,i1,i4,i3,i2,i5)

      endif
      endif

      zB3g0d3tree=ztree

c      write(*,*)"zB3g0d3tree ",iorder, ihel,iflip,zB3g0d3tree
      return
      end
************************************************************************
*
* three gluon two quark helicity tree amplitudes zBtt123tree
*
*   zB3g0qtree = B0(1,2,3)+B0(2,3,1)+B0(3,1,2)
*
      function zB3g0qtree(ihel,iflip,i1,i2,i3,i4,i5)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /order/iorder
      parameter(pi=3.141592653589793238d0)


*      iflip = 1 -> A(i,j) <-> B(j,i) and line reversal
*                           234      243      324      342      423      432
*      ihel = 1  iflip = 0 pmppm -> pmppm -> ppmpm -> pppmm -> ppmpm -> pppmm   1 1 2 3 2 3
*      ihel = 2  iflip = 0 ppmpm -> pppmm -> pmppm -> pmppm -> pppmm -> ppmpm   2 3 1 1 3 2
*      ihel = 3  iflip = 0 pppmm -> ppmpm -> pppmm -> ppmpm -> pmppm -> pmppm   3 2 3 2 1 1
*      ihel = 1  iflip = 1 pmmpm -> pmpmm -> pmmpm -> pmpmm -> ppmmm -> ppmmm   1 2 1 2 3 3
*      ihel = 2  iflip = 1 pmpmm -> pmmpm -> ppmmm -> ppmmm -> pmmpm -> pmpmm   2 1 3 3 1 2
*      ihel = 3  iflip = 1 ppmmm -> ppmmm -> pmpmm -> pmmpm -> pmpmm -> pppmm   3 3 2 1 2 1
*
*      i1,i2,i3,i4,i5 -> i5,i4,i3,i2,i1
*
*      ihel = 1  iflip = 2 mppmp -> mpmpp -> mppmp -> mpmpp -> mmppp -> mmppp   1 2 1 2 3 3
*      ihel = 2  iflip = 2 mpmpp -> mppmp -> mmppp -> mmppp -> mppmp -> mpmpp   2 1 3 3 1 2
*      ihel = 3  iflip = 2 mmppp -> mmppp -> mpmpp -> mppmp -> mpmpp -> mppmp   3 3 2 1 2 1
*      ihel = 1  iflip = 3 mpmmp -> mpmmp -> mmpmp -> mmmpp -> mmpmp -> mmmpp   1 1 2 3 2 3
*      ihel = 2  iflip = 3 mmpmp -> mmmpp -> mpmmp -> mpmmp -> mmmpp -> mmpmp   2 3 1 1 3 2
*      ihel = 3  iflip = 3 mmmpp -> mmpmp -> mmmpp -> mmpmp -> mpmmp -> mpmmp   3 2 3 2 1 1

*                           A B               B A               A B               B A
*                          12345             54321             54321             12345
*      ihel = 1  iflip = 0 pmppm   iflip = 1 pmmpm   iflip = 2 mppmp   iflip = 3 mpmmp
*      ihel = 2  iflip = 0 ppmpm   iflip = 1 pmpmm   iflip = 2 mpmpp   iflip = 3 mmpmp
*      ihel = 3  iflip = 0 pppmm   iflip = 1 ppmmm   iflip = 2 mmppp   iflip = 3 mmmpp


      if(iflip.eq.0.or.iflip.eq.3)then
      if(ihel.eq.1)then

        ztree=zB123tree(1,iflip,i1,i2,i3,i4,i5)
     .       +zB123tree(3,iflip,i1,i3,i4,i2,i5)
     .       +zB123tree(2,iflip,i1,i4,i2,i3,i5)

      elseif(ihel.eq.2)then

        ztree=zB123tree(2,iflip,i1,i2,i3,i4,i5)
     .       +zB123tree(1,iflip,i1,i3,i4,i2,i5)
     .       +zB123tree(3,iflip,i1,i4,i2,i3,i5)

      elseif(ihel.eq.3)then

        ztree=zB123tree(3,iflip,i1,i2,i3,i4,i5)
     .       +zB123tree(2,iflip,i1,i3,i4,i2,i5)
     .       +zB123tree(1,iflip,i1,i4,i2,i3,i5)

      endif
      elseif(iflip.eq.1.or.iflip.eq.2)then
      if(ihel.eq.1)then

        ztree=zB123tree(1,iflip,i1,i2,i3,i4,i5)
     .       +zB123tree(2,iflip,i1,i3,i4,i2,i5)
     .       +zB123tree(3,iflip,i1,i4,i2,i3,i5)

      elseif(ihel.eq.2)then

        ztree=zB123tree(2,iflip,i1,i2,i3,i4,i5)
     .       +zB123tree(3,iflip,i1,i3,i4,i2,i5)
     .       +zB123tree(1,iflip,i1,i4,i2,i3,i5)

      elseif(ihel.eq.3)then

        ztree=zB123tree(3,iflip,i1,i2,i3,i4,i5)
     .       +zB123tree(1,iflip,i1,i3,i4,i2,i5)
     .       +zB123tree(2,iflip,i1,i4,i2,i3,i5)

      endif
      endif

      zB3g0qtree=ztree

c      write(*,*)"zB3g0qtree ",iorder, ihel,iflip,zB3g0qtree
      return
      end
************************************************************************
*
* three gluon two quark helicity tree amplitudes zBtt123tree
*
*   zB3g0qtree = B0(1,3,2)+B0(2,1,3)+B0(3,2,1)
*
      function zB3g0qbtree(ihel,iflip,i1,i2,i3,i4,i5)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /order/iorder
      parameter(pi=3.141592653589793238d0)


*      iflip = 1 -> A(i,j) <-> B(j,i) and line reversal
*                           234      243      324      342      423      432
*      ihel = 1  iflip = 0 pmppm -> pmppm -> ppmpm -> pppmm -> ppmpm -> pppmm   1 1 2 3 2 3
*      ihel = 2  iflip = 0 ppmpm -> pppmm -> pmppm -> pmppm -> pppmm -> ppmpm   2 3 1 1 3 2
*      ihel = 3  iflip = 0 pppmm -> ppmpm -> pppmm -> ppmpm -> pmppm -> pmppm   3 2 3 2 1 1
*      ihel = 1  iflip = 1 pmmpm -> pmpmm -> pmmpm -> pmpmm -> ppmmm -> ppmmm   1 2 1 2 3 3
*      ihel = 2  iflip = 1 pmpmm -> pmmpm -> ppmmm -> ppmmm -> pmmpm -> pmpmm   2 1 3 3 1 2
*      ihel = 3  iflip = 1 ppmmm -> ppmmm -> pmpmm -> pmmpm -> pmpmm -> pppmm   3 3 2 1 2 1
*
*      i1,i2,i3,i4,i5 -> i5,i4,i3,i2,i1
*
*      ihel = 1  iflip = 2 mppmp -> mpmpp -> mppmp -> mpmpp -> mmppp -> mmppp   1 2 1 2 3 3
*      ihel = 2  iflip = 2 mpmpp -> mppmp -> mmppp -> mmppp -> mppmp -> mpmpp   2 1 3 3 1 2
*      ihel = 3  iflip = 2 mmppp -> mmppp -> mpmpp -> mppmp -> mpmpp -> mppmp   3 3 2 1 2 1
*      ihel = 1  iflip = 3 mpmmp -> mpmmp -> mmpmp -> mmmpp -> mmpmp -> mmmpp   1 1 2 3 2 3
*      ihel = 2  iflip = 3 mmpmp -> mmmpp -> mpmmp -> mpmmp -> mmmpp -> mmpmp   2 3 1 1 3 2
*      ihel = 3  iflip = 3 mmmpp -> mmpmp -> mmmpp -> mmpmp -> mpmmp -> mpmmp   3 2 3 2 1 1

*                           A B               B A               A B               B A
*                          12345             54321             54321             12345
*      ihel = 1  iflip = 0 pmppm   iflip = 1 pmmpm   iflip = 2 mppmp   iflip = 3 mpmmp
*      ihel = 2  iflip = 0 ppmpm   iflip = 1 pmpmm   iflip = 2 mpmpp   iflip = 3 mmpmp
*      ihel = 3  iflip = 0 pppmm   iflip = 1 ppmmm   iflip = 2 mmppp   iflip = 3 mmmpp


      if(iflip.eq.0.or.iflip.eq.3)then
      if(ihel.eq.1)then

        ztree=zB123tree(1,iflip,i1,i2,i4,i3,i5)
     .       +zB123tree(3,iflip,i1,i4,i3,i2,i5)
     .       +zB123tree(2,iflip,i1,i3,i2,i4,i5)

      elseif(ihel.eq.2)then

        ztree=zB123tree(3,iflip,i1,i2,i4,i3,i5)
     .       +zB123tree(2,iflip,i1,i4,i3,i2,i5)
     .       +zB123tree(1,iflip,i1,i3,i2,i4,i5)

      elseif(ihel.eq.3)then

        ztree=zB123tree(2,iflip,i1,i2,i4,i3,i5)
     .       +zB123tree(1,iflip,i1,i4,i3,i2,i5)
     .       +zB123tree(3,iflip,i1,i3,i2,i4,i5)

      endif
      elseif(iflip.eq.1.or.iflip.eq.2)then
      if(ihel.eq.1)then

        ztree=zB123tree(2,iflip,i1,i2,i4,i3,i5)
     .       +zB123tree(3,iflip,i1,i4,i3,i2,i5)
     .       +zB123tree(1,iflip,i1,i3,i2,i4,i5)

      elseif(ihel.eq.2)then

        ztree=zB123tree(1,iflip,i1,i2,i4,i3,i5)
     .       +zB123tree(2,iflip,i1,i4,i3,i2,i5)
     .       +zB123tree(3,iflip,i1,i3,i2,i4,i5)

      elseif(ihel.eq.3)then

        ztree=zB123tree(3,iflip,i1,i2,i4,i3,i5)
     .       +zB123tree(1,iflip,i1,i4,i3,i2,i5)
     .       +zB123tree(2,iflip,i1,i3,i2,i4,i5)

      endif
      endif

      zB3g0qbtree=ztree

*      write(*,*)"zB3g0qbtree ",iorder, ihel,iflip,zB3g0qbtree
      return
      end

************************************************************************
* three gluon two quark one loop matrix elements interfered with tree level
* summed over colour orderings, summed and squared
*
*                   q  g  g  g  qb
*
      function FullB3g1(i1,i2,i3,i4,i5,renscale2)
      implicit double precision (a-h,o-z)
      parameter(pi=3.141592653589793238d0)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)

c      write(*,*)" hello FullB3g1"
      fac=(as*nc/2d0/pi)*(4d0*pi*as*nc)**3*(nc**2-1d0)/nc

      FullB3g1= (+B3g1a(i1,i2,i3,i4,i5,renscale2)
     .           +B3g1a(i1,i2,i4,i3,i5,renscale2)
     .           +B3g1a(i1,i3,i2,i4,i5,renscale2)
     .           +B3g1a(i1,i3,i4,i2,i5,renscale2)
     .           +B3g1a(i1,i4,i2,i3,i5,renscale2)
     .           +B3g1a(i1,i4,i3,i2,i5,renscale2))
     .    +dble(nf)/dble(nc)*(
     .           +B3g1b(i1,i2,i3,i4,i5,renscale2)
     .           +B3g1b(i1,i2,i4,i3,i5,renscale2)
     .           +B3g1b(i1,i3,i2,i4,i5,renscale2)
     .           +B3g1b(i1,i3,i4,i2,i5,renscale2)
     .           +B3g1b(i1,i4,i2,i3,i5,renscale2)
     .           +B3g1b(i1,i4,i3,i2,i5,renscale2))
     .    +1d0/dble(nc)**2*(
     .           +B3g1c(i1,i2,i3,i4,i5,renscale2)
     .           +B3g1c(i1,i2,i4,i3,i5,renscale2)
     .           +B3g1c(i1,i3,i2,i4,i5,renscale2)
     .           +B3g1c(i1,i3,i4,i2,i5,renscale2)
     .           +B3g1c(i1,i4,i2,i3,i5,renscale2)
     .           +B3g1c(i1,i4,i3,i2,i5,renscale2))
     .    -1d0/dble(nc)**2*(
     .           +Bt3g1a(i1,i2,i3,i4,i5,renscale2)
     .           +Bt3g1a(i1,i2,i4,i3,i5,renscale2)
     .           +Bt3g1a(i1,i3,i2,i4,i5,renscale2)
     .           +Bt3g1a(i1,i3,i4,i2,i5,renscale2)
     .           +Bt3g1a(i1,i4,i2,i3,i5,renscale2)
     .           +Bt3g1a(i1,i4,i3,i2,i5,renscale2))
     .    -dble(nf)/dble(nc)**3*(
     .           +Bt3g1b(i1,i2,i3,i4,i5,renscale2)
     .           +Bt3g1b(i1,i2,i4,i3,i5,renscale2)
     .           +Bt3g1b(i1,i3,i2,i4,i5,renscale2)
     .           +Bt3g1b(i1,i3,i4,i2,i5,renscale2)
     .           +Bt3g1b(i1,i4,i2,i3,i5,renscale2)
     .           +Bt3g1b(i1,i4,i3,i2,i5,renscale2))
     .    -1d0/dble(nc)**4*(
     .           +Bt3g1c(i1,i2,i3,i4,i5,renscale2)
     .           +Bt3g1c(i1,i2,i4,i3,i5,renscale2)
     .           +Bt3g1c(i1,i3,i2,i4,i5,renscale2)
     .           +Bt3g1c(i1,i3,i4,i2,i5,renscale2)
     .           +Bt3g1c(i1,i4,i2,i3,i5,renscale2)
     .           +Bt3g1c(i1,i4,i3,i2,i5,renscale2))
     .     +1d0/dble(nc)**(2)*(1d0+1d0/dble(nc)**(2))
     .             *Btt3g1a(i1,i2,i3,i4,i5,renscale2)
     .     +dble(nf)/dble(nc)**(3)*(1d0+1d0/dble(nc)**(2))
     .             *Btt3g1b(i1,i2,i3,i4,i5,renscale2)
     .     +1d0/dble(nc)**(4)*(1d0+1d0/dble(nc)**(2))
     .             *Btt3g1c(i1,i2,i3,i4,i5,renscale2)
     .
     .     -1d0/dble(nc)**(2)*(B3g1d1(i1,i2,i3,i4,i5,renscale2)
     .                   +B3g1d2(i1,i2,i3,i4,i5,renscale2)
     .                   +B3g1d3(i1,i2,i3,i4,i5,renscale2)
     .                   +B3g1q(i1,i2,i3,i4,i5,renscale2)
     .                   +B3g1qb(i1,i2,i3,i4,i5,renscale2))
     .     -1d0*dble(nf)/dble(nc)**(3)*(Bh3g1q(i1,i2,i3,i4,i5,renscale2)
     .                             +Bh3g1qb(i1,i2,i3,i4,i5,renscale2))
     .     +1d0/dble(nc)**(4)*(Bt3g1d(i1,i2,i3,i4,i5,renscale2)
     .                  +2d0*Bt3g1q(i1,i2,i3,i4,i5,renscale2)
     .                  +2d0*Bt3g1qb(i1,i2,i3,i4,i5,renscale2))
     .     +1d0*dble(nf)/dble(nc)**(5)*(
     .                  +2d0*Bth3g1q(i1,i2,i3,i4,i5,renscale2)
     .                  +2d0*Bth3g1qb(i1,i2,i3,i4,i5,renscale2))


c       write(*,*),"Full",FullB3g1,fac
      FullB3g1=fac*FullB3g1
c      write(*,*)"Btt3g1a",Btt3g1a(i1,i2,i3,i4,i5,renscale2),
c     .                    Btt3g1a(i1,i2,i4,i3,i5,renscale2),
c     .                    Btt3g1a(i1,i3,i2,i4,i5,renscale2),
c     .                    Btt3g1a(i1,i3,i4,i2,i5,renscale2),
c     .                    Btt3g1a(i1,i4,i2,i3,i5,renscale2),
c     .                    Btt3g1a(i1,i4,i3,i2,i5,renscale2)

c      write(*,*)"B3g1dtype",B3g1d1(i1,i2,i3,i4,i5,renscale2)
c     .                     +B3g1d2(i1,i2,i3,i4,i5,renscale2)
c     .                     +B3g1d3(i1,i2,i3,i4,i5,renscale2),
c     .                      B3g1d1(i5,i4,i2,i3,i1,renscale2)
c     .                     +B3g1d2(i5,i4,i2,i3,i1,renscale2)
c     .                     +B3g1d3(i5,i4,i2,i3,i1,renscale2)
c      write(*,*)"B3g1qtype",B3g1q(i1,i3,i4,i2,i5,renscale2)
c     .                    +B3g1qb(i1,i3,i4,i2,i5,renscale2),
c     .                      B3g1q(i1,i2,i4,i3,i5,renscale2)
c     .                    +B3g1qb(i1,i2,i4,i3,i5,renscale2),
c     .                      B3g1q(i1,i3,i2,i4,i5,renscale2)
c     .                    +B3g1qb(i1,i3,i2,i4,i5,renscale2),
c     .                      B3g1q(i1,i4,i3,i2,i5,renscale2)
c     .                    +B3g1qb(i1,i4,i3,i2,i5,renscale2)
      return
      end
*

************************************************************************
* three gluon two quark 1-loop interfered with tree level
* colour structures that do not appear at one loop
* B3g1d1 = B1(1,2,3)*((B0c(1,2,3)+B0c(1,3,2)+B0c(2,3,1)+B0c(3,2,1))
*
*
      function B3g1d1(i1,i2,i3,i4,i5,renscale2)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /scheme/ deltaR
      common /order/iorder
      deltaR=1d0
*      write(*,*)" hello B3g1d1 ", iorder

*
      zwt = (0d0,0d0)
      do ihel=1,3
      do iflip=0,3
      zwt = zwt
     . + zB3g1AAd1(ihel,iflip,i1,i2,i3,i4,i5,renscale2)*dconjg(zB3g0d1tree(ihel,iflip,i1,i2,i3,i4,i5))
      enddo
      enddo


      B3g1d1=-2d0*dreal(zwt)

* factor of half from g^2/16/pi^2 = (as/2/pi)*1/2
         B3g1d1=B3g1d1/2d0
c      write(*,*)"B3g1d1",i1,i2,i3,i4,i5,B3g1d1
      return
      end
************************************************************************
* three gluon two quark 1-loop interfered with tree level
* colour structures that do not appear at one loop
* B3g1d2 = B1(1,2,3)*(B0c(1,3,2)+B0c(2,1,3)+B0c(2,3,1)+B0c(3,1,2))
*
*
      function B3g1d2(i1,i2,i3,i4,i5,renscale2)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /scheme/ deltaR
      common /order/iorder
      deltaR=1d0
*      write(*,*)" hello B3g1d2 ", iorder

*
      zwt = (0d0,0d0)
      do ihel=1,3
      do iflip=0,3
      zwt = zwt
     . + zB3g1AAd2(ihel,iflip,i1,i2,i3,i4,i5,renscale2)*dconjg(zB3g0d2tree(ihel,iflip,i1,i2,i3,i4,i5))
      enddo
      enddo


      B3g1d2=-2d0*dreal(zwt)

* factor of half from g^2/16/pi^2 = (as/2/pi)*1/2
         B3g1d2=B3g1d2/2d0
c      write(*,*)"B3g1d2",i1,i2,i3,i4,i5,B3g1d2
      return
      end
************************************************************************
* three gluon two quark 1-loop interfered with tree level
* colour structures that do not appear at one loop
* B3g1d3 = B1(1,2,3)*((B0c(1,2,3)+B0c(1,3,2)+B0c(2,3,1)+B0c(3,2,1))
*
*
      function B3g1d3(i1,i2,i3,i4,i5,renscale2)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /scheme/ deltaR
      common /order/iorder
      deltaR=1d0
*      write(*,*)" hello B3g1d3 ", iorder

*
      zwt = (0d0,0d0)
      do ihel=1,3
      do iflip=0,3
      zwt = zwt
     . + zB3g1AAd3(ihel,iflip,i1,i2,i3,i4,i5,renscale2)*dconjg(zB3g0d3tree(ihel,iflip,i1,i2,i3,i4,i5))
      enddo
      enddo

      B3g1d3=-2d0*dreal(zwt)

* factor of half from g^2/16/pi^2 = (as/2/pi)*1/2
         B3g1d3=B3g1d3/2d0
c      write(*,*)"B3g1d3",i1,i2,i3,i4,i5,B3g1d3
      return
      end
************************************************************************
* three gluon two quark 1-loop interfered with tree level
* colour structures that do not appear at one loop
* B3g1q = Bq(1,2,3)*((B0c(1,2,3)+B0c(2,3,1)+B0c(3,1,2))
*
*
      function B3g1q(i1,i2,i3,i4,i5,renscale2)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /scheme/ deltaR
      common /order/iorder
      deltaR=1d0
*      write(*,*)" hello B3g1q ", iorder

*
      zwt = (0d0,0d0)
      do ihel=1,3
      do iflip=0,3
      zwt = zwt
     . + zB3g1AAq(ihel,iflip,i1,i2,i3,i4,i5,renscale2)*dconjg(zB3g0qtree(ihel,iflip,i1,i2,i3,i4,i5))
      enddo
      enddo


      B3g1q=-2d0*dreal(zwt)

* factor of half from g^2/16/pi^2 = (as/2/pi)*1/2
         B3g1q=B3g1q/2d0
*      write(*,*)"B3g1q",i1,i2,i3,i4,i5,B3g1q
      return
      end
************************************************************************
* three gluon two quark 1-loop interfered with tree level
* colour structures that do not appear at one loop
* B3g1qb = Bqb(1,2,3)*((B0c(1,3,2)+B0c(2,1,3)+B0c(3,2,1))
*
*
      function B3g1qb(i1,i2,i3,i4,i5,renscale2)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /scheme/ deltaR
      common /order/iorder
      deltaR=1d0
*      write(*,*)" hello B3g1qb ", iorder

*
      zwt = (0d0,0d0)
      do ihel=1,3
      do iflip=0,3
      zwt = zwt
     . + zB3g1AAqb(ihel,iflip,i1,i2,i3,i4,i5,renscale2)*dconjg(zB3g0qbtree(ihel,iflip,i1,i2,i3,i4,i5))
      enddo
      enddo


      B3g1qb=-2d0*dreal(zwt)

* factor of half from g^2/16/pi^2 = (as/2/pi)*1/2
         B3g1qb=B3g1qb/2d0
c      write(*,*)"B3g1qb",i1,i2,i3,i4,i5,B3g1qb
      return
      end
************************************************************************
* three gluon two quark 1-loop interfered with tree level
* colour structures that do not appear at one loop
* Bh3g1q = Bhq(1,2,3)*((B0c(1,2,3)+B0c(2,3,1)+B0c(3,1,2))
*
*
      function Bh3g1q(i1,i2,i3,i4,i5,renscale2)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /scheme/ deltaR
      common /order/iorder
      deltaR=1d0
*      write(*,*)" hello Bh3g1q ", iorder

*
      zwt = (0d0,0d0)
      do ihel=1,3
      do iflip=0,3
      zwt = zwt
     . + zBh3g1AAq(ihel,iflip,i1,i2,i3,i4,i5,renscale2)*dconjg(zB3g0qtree(ihel,iflip,i1,i2,i3,i4,i5))
      enddo
      enddo


      Bh3g1q=-2d0*dreal(zwt)

* factor of half from g^2/16/pi^2 = (as/2/pi)*1/2
         Bh3g1q=Bh3g1q/2d0
c      write(*,*)"Bh3g1q",i1,i2,i3,i4,i5,Bh3g1q
      return
      end
************************************************************************
* three gluon two quark 1-loop interfered with tree level
* colour structures that do not appear at one loop
* Bh3g1qb = Bhqb(1,2,3)*((B0c(1,3,2)+B0c(2,1,3)+B0c(3,2,1))
*
*
      function Bh3g1qb(i1,i2,i3,i4,i5,renscale2)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /scheme/ deltaR
      common /order/iorder
      deltaR=1d0
*      write(*,*)" hello Bh3g1qb ", iorder

*
      zwt = (0d0,0d0)
      do ihel=1,3
      do iflip=0,3
      zwt = zwt
     . + zBh3g1AAqb(ihel,iflip,i1,i2,i3,i4,i5,renscale2)*dconjg(zB3g0qbtree(ihel,iflip,i1,i2,i3,i4,i5))
      enddo
      enddo

      Bh3g1qb=-2d0*dreal(zwt)

* factor of half from g^2/16/pi^2 = (as/2/pi)*1/2
         Bh3g1qb=Bh3g1qb/2d0
c      write(*,*)"Bh3g1qb",i1,i2,i3,i4,i5,Bh3g1qb
      return
      end
************************************************************************
* three gluon two quark 1-loop interfered with tree level
* colour structures that do not appear at one loop
* Bt3g1q = Bq(1,2,3)*Bq(1,2,3)*(B0c(1,2,3)+B0c(1,3,2)+B0c(2,3,1)+B0c(3,2,1)+B0c(2,1,3)+B0c(3,1,2))
*
*
      function Bt3g1q(i1,i2,i3,i4,i5,renscale2)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /scheme/ deltaR
      common /order/iorder
      deltaR=1d0
*      write(*,*)" hello Bt3g1q ", iorder

*
      zwt = (0d0,0d0)
      do ihel=1,3
      do iflip=0,3
      zwt = zwt
     . + zB3g1AAq(ihel,iflip,i1,i2,i3,i4,i5,renscale2)*dconjg(zBtt123tree(ihel,iflip,i1,i2,i3,i4,i5))
      enddo
      enddo

      Bt3g1q=-2d0*dreal(zwt)

* factor of half from g^2/16/pi^2 = (as/2/pi)*1/2
         Bt3g1q=Bt3g1q/2d0
c      write(*,*)"Bt3g1q",i1,i2,i3,i4,i5,Bt3g1q
      return
      end

      function Bth3g1q(i1,i2,i3,i4,i5,renscale2)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /scheme/ deltaR
      common /order/iorder
      deltaR=1d0
*      write(*,*)" hello Bt3g1q ", iorder

*
      zwt = (0d0,0d0)
      do ihel=1,3
      do iflip=0,3
      zwt = zwt
     . + zBh3g1AAq(ihel,iflip,i1,i2,i3,i4,i5,renscale2)*dconjg(zBtt123tree(ihel,iflip,i1,i2,i3,i4,i5))
      enddo
      enddo

      Bth3g1q=-2d0*dreal(zwt)

* factor of half from g^2/16/pi^2 = (as/2/pi)*1/2
         Bth3g1q=Bth3g1q/2d0
c      write(*,*)"Bt3g1q",i1,i2,i3,i4,i5,Bt3g1q
      return
      end
************************************************************************
* three gluon two quark 1-loop interfered with tree level
* colour structures that do not appear at one loop
* Bt3g1qb = Bqb(1,2,3)*Bq(1,2,3)*(B0c(1,2,3)+B0c(1,3,2)+B0c(2,3,1)+B0c(3,2,1)+B0c(2,1,3)+B0c(3,1,2))
*
*
      function Bt3g1qb(i1,i2,i3,i4,i5,renscale2)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /scheme/ deltaR
      common /order/iorder
      deltaR=1d0
*      write(*,*)" hello Bt3g1qb ", iorder

*
      zwt = (0d0,0d0)
      do ihel=1,3
      do iflip=0,3
      zwt = zwt
     . + zB3g1AAqb(ihel,iflip,i1,i2,i3,i4,i5,renscale2)*dconjg(zBtt123tree(ihel,iflip,i1,i2,i3,i4,i5))
      enddo
      enddo

      Bt3g1qb=-2d0*dreal(zwt)

* factor of half from g^2/16/pi^2 = (as/2/pi)*1/2
         Bt3g1qb=Bt3g1qb/2d0
c      write(*,*)"Bt3g1qb",i1,i2,i3,i4,i5,Bt3g1qb
      return
      end

      function Bth3g1qb(i1,i2,i3,i4,i5,renscale2)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /scheme/ deltaR
      common /order/iorder
      deltaR=1d0
*      write(*,*)" hello Bt3g1qb ", iorder

*
      zwt = (0d0,0d0)
      do ihel=1,3
      do iflip=0,3
      zwt = zwt
     . + zBh3g1AAqb(ihel,iflip,i1,i2,i3,i4,i5,renscale2)*dconjg(zBtt123tree(ihel,iflip,i1,i2,i3,i4,i5))
      enddo
      enddo

      Bth3g1qb=-2d0*dreal(zwt)

* factor of half from g^2/16/pi^2 = (as/2/pi)*1/2
         Bth3g1qb=Bth3g1qb/2d0
c      write(*,*)"Bt3g1qb",i1,i2,i3,i4,i5,Bt3g1qb
      return
      end

************************************************************************
* three gluon two quark 1-loop interfered with tree level
* colour structures that do not appear at one loop
* Bt3g1d = (B1(1,2,3)+B2(1,2,3)+B3(1,2,3))*(B0c(1,2,3)+B0c(1,3,2)+B0c(2,3,1)+B0c(3,2,1)+B0c(2,1,3)+B0c(3,1,2))
*
*
      function Bt3g1d(i1,i2,i3,i4,i5,renscale2)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /scheme/ deltaR
      common /order/iorder
      deltaR=1d0
*      write(*,*)" hello Bt3g1d ", iorder

*
      zwt = (0d0,0d0)
      do ihel=1,3
      do iflip=0,3
      zwt = zwt
     . + (zB3g1AAd1(ihel,iflip,i1,i2,i3,i4,i5,renscale2)
     .   +zB3g1AAd2(ihel,iflip,i1,i2,i3,i4,i5,renscale2)
     .   +zB3g1AAd3(ihel,iflip,i1,i2,i3,i4,i5,renscale2))
     .      *dconjg(zBtt123tree(ihel,iflip,i1,i2,i3,i4,i5))
      enddo
      enddo

      Bt3g1d=-2d0*dreal(zwt)

* factor of half from g^2/16/pi^2 = (as/2/pi)*1/2
         Bt3g1d=Bt3g1d/2d0
c      write(*,*)"Bt3g1d",i1,i2,i3,i4,i5,Bt3g1d
      return
      end
************************************************************************
* three gluon two quark LC 1-loop interfered with tree level
* (internal gluon loop, external gluons)
* one colour ordering, summed and squared
*
*
      function B3g1(i1,i2,i3,i4,i5,renscale2)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      B3g1 = B3g1a(i1,i2,i3,i4,i5,renscale2)
c      write(*,*)i1,i2,i3,i4,i5,B3g1
      return
      end
************************************************************************
* three gluon two quark NF 1-loop interfered with tree level
* (internal quark loop, external gluons)
* one colour ordering, summed and squared
*
*
      function Bh3g1(i1,i2,i3,i4,i5,renscale2)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      Bh3g1 = B3g1b(i1,i2,i3,i4,i5,renscale2)
*      write(*,*)i1,i2,i3,i4,i5,Bh3g1
      return
      end

************************************************************************
* three gluon two quark LC 1-loop interfered with tree level
* (internal gluon loop, external gluons)
* one colour ordering, summed and squared
*
*
      function B3g1a(i1,i2,i3,i4,i5,renscale2)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /scheme/ deltaR
      common /order/iorder
      deltaR=1d0
*      write(*,*)" hello B3g1a ", iorder

*
      zwt = (0d0,0d0)
      do ihel=1,3
      do iflip=0,3
      zwt = zwt
     . + zB3g1AAa(ihel,iflip,i1,i2,i3,i4,i5,renscale2)*dconjg(zB123tree(ihel,iflip,i1,i2,i3,i4,i5))
      enddo
c      write(*,*)
      enddo

      B3g1a=-2d0*dreal(zwt)

* factor of half from g^2/16/pi^2 = (as/2/pi)*1/2
         B3g1a=B3g1a/2d0
c      write(*,*)"B3g1",i1,i2,i3,i4,i5,B3g1
      return
      end
************************************************************************
* three gluon two quark NF 1-loop interfered with tree level
* (internal quark loop, external gluons)
* one colour ordering, summed and squared
*
*
      function B3g1b(i1,i2,i3,i4,i5,renscale2)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /scheme/ deltaR
      common /order/iorder
      deltaR=1d0
*      write(*,*)" hello B3g1b ", iorder

*
      zwt = (0d0,0d0)
      do ihel=1,3
      do iflip=0,3
      zwt = zwt
     . + zB3g1AAb(ihel,iflip,i1,i2,i3,i4,i5,renscale2)*dconjg(zB123tree(ihel,iflip,i1,i2,i3,i4,i5))
      enddo
      enddo

      B3g1b=-2d0*dreal(zwt)

* factor of half from g^2/16/pi^2 = (as/2/pi)*1/2
         B3g1b=B3g1b/2d0
c      write(*,*)"B3g1b",i1,i2,i3,i4,i5,B3g1b
      return
      end
************************************************************************
* three gluon two quark SLC 1-loop interfered with tree level
* (internal 'photon' loop, external gluons)
* one colour ordering, summed and squared
*
*
      function B3g1c(i1,i2,i3,i4,i5,renscale2)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /scheme/ deltaR
      common /order/iorder
      deltaR=1d0
*      write(*,*)" hello B3g1c ", iorder

*
      zwt = (0d0,0d0)
      do ihel=1,3
      do iflip=0,3
      zwt = zwt
     . + zB3g1AAc(ihel,iflip,i1,i2,i3,i4,i5,renscale2)*dconjg(zB123tree(ihel,iflip,i1,i2,i3,i4,i5))
      enddo
      enddo

      B3g1c=-2d0*dreal(zwt)

* factor of half from g^2/16/pi^2 = (as/2/pi)*1/2
         B3g1c=B3g1c/2d0
c      write(*,*)"B3g1c",i1,i2,i3,i4,i5,B3g1c

      return
      end
************************************************************************
* three gluon two quark LC 1-loop, symmetrized over i2, interfered with tree level
* (internal gluon loop, external 'photon' i2)
* one colour ordering, summed and squared
*
*
      function Bt3g1a(i1,i2,i3,i4,i5,renscale2)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /scheme/ deltaR
      common /order/iorder
      deltaR=1d0
*      write(*,*)" hello Bt3g1a ", iorder

*
      zwt = (0d0,0d0)
      do ihel=1,3
      do iflip=0,3
      zwt = zwt
     .  +zBt3g1AAa(ihel,iflip,i1,i2,i3,i4,i5,renscale2)*dconjg(zBt123tree(ihel,iflip,i1,i2,i3,i4,i5))
      enddo
      enddo

      Bt3g1a=-2d0*dreal(zwt)

* factor of half from g^2/16/pi^2 = (as/2/pi)*1/2
         Bt3g1a=Bt3g1a/2d0
*      write(*,*)"Bt3g1a",i1,i2,i3,i4,i5,Bt3g1a

      return
      end
************************************************************************
* three gluon two quark NF 1-loop, symmetrized over i2, interfered with tree level
* (internal quark loop, external 'photon' i2)
* one colour ordering, summed and squared
*
*
      function Bt3g1b(i1,i2,i3,i4,i5,renscale2)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /scheme/ deltaR
      common /order/iorder
      deltaR=1d0
*      write(*,*)" hello Bt3g1b ", iorder

*
      zwt = (0d0,0d0)
      do ihel=1,3
      do iflip=0,3
      zwt = zwt
     .  +zBt3g1AAb(ihel,iflip,i1,i2,i3,i4,i5,renscale2)*dconjg(zBt123tree(ihel,iflip,i1,i2,i3,i4,i5))
      enddo
      enddo

      Bt3g1b=-2d0*dreal(zwt)

* factor of half from g^2/16/pi^2 = (as/2/pi)*1/2
         Bt3g1b=Bt3g1b/2d0
c      write(*,*)"Bt3g1",i1,i2,i3,i4,i5,Bt3g1

      return
      end
************************************************************************
* three gluon two quark SLC 1-loop, symmetrized over i2, interfered with tree level
* (internal 'photon' loop, external 'photon' i2)
* one colour ordering, summed and squared
*
*
      function Bt3g1c(i1,i2,i3,i4,i5,renscale2)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /scheme/ deltaR
      common /order/iorder
      deltaR=1d0
*      write(*,*)" hello Bt3g1c ", iorder

*
      zwt = (0d0,0d0)

      do ihel=1,3
      do iflip=0,3
      zwt = zwt
     .  +zBt3g1AAc(ihel,iflip,i1,i2,i3,i4,i5,renscale2)*dconjg(zBt123tree(ihel,iflip,i1,i2,i3,i4,i5))
      enddo
      enddo
      Bt3g1c=-2d0*dreal(zwt)

* factor of half from g^2/16/pi^2 = (as/2/pi)*1/2
         Bt3g1c=Bt3g1c/2d0
c      write(*,*)"Bt3g1",i1,i2,i3,i4,i5,Bt3g1

      return
      end
************************************************************************
* three gluon two quark LC 1-loop, all gluons symmetrized, interfered with tree level
* (internal gluon loop, external 'photons')
* one colour ordering, summed and squared
*
*
      function Btt3g1a(i1,i2,i3,i4,i5,renscale2)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /scheme/ deltaR
      common /order/iorder
      deltaR=1d0
*      write(*,*)" hello BB3g0 ", iorder

*
      zwt = (0d0,0d0)
      do ihel=1,3
      do iflip=0,3
      zwt = zwt
     .  +zBtt3g1AAa(ihel,iflip,i1,i2,i3,i4,i5,renscale2)*dconjg(zBtt123tree(ihel,iflip,i1,i2,i3,i4,i5))
      enddo
      enddo

      Btt3g1a=-2d0*dreal(zwt)

* factor of half from g^2/16/pi^2 = (as/2/pi)*1/2
         Btt3g1a=Btt3g1a/2d0
c      write(*,*)"Btt3g1a",i1,i2,i3,i4,i5,Btt3g1a

      return
      end

************************************************************************
* three gluon two quark NF 1-loop, all gluons symmetrized, interfered with tree level
* (internal quark loop, external 'photons')
* one colour ordering, summed and squared
*
*
      function Btt3g1b(i1,i2,i3,i4,i5,renscale2)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /scheme/ deltaR
      common /order/iorder
      deltaR=1d0
*      write(*,*)" hello BB3g0 ", iorder

*
      zwt = (0d0,0d0)
      do ihel=1,3
      do iflip=0,3
      zwt = zwt
     .  +zBtt3g1AAb(ihel,iflip,i1,i2,i3,i4,i5,renscale2)*dconjg(zBtt123tree(ihel,iflip,i1,i2,i3,i4,i5))
      enddo
      enddo

      Btt3g1b=-2d0*dreal(zwt)

* factor of half from g^2/16/pi^2 = (as/2/pi)*1/2
         Btt3g1b=Btt3g1b/2d0
c      write(*,*)"Btt3g1b",i1,i2,i3,i4,i5,Btt3g1b

      return
      end
************************************************************************
* three gluon two quark SLC 1-loop, all gluons symmetrized, interfered with tree level
* (internal 'photon' loop, external 'photons')
* one colour ordering, summed and squared
*
*
      function Btt3g1c(i1,i2,i3,i4,i5,renscale2)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /scheme/ deltaR
      common /order/iorder
      deltaR=1d0
*      write(*,*)" hello BB3g0 ", iorder

*
      zwt = (0d0,0d0)
      do ihel=1,3
      do iflip=0,3
      zwt = zwt
     .  +zBtt3g1AAc(ihel,iflip,i1,i2,i3,i4,i5,renscale2)*dconjg(zBtt123tree(ihel,iflip,i1,i2,i3,i4,i5))
      enddo
      enddo

      Btt3g1c=-2d0*dreal(zwt)

* factor of half from g^2/16/pi^2 = (as/2/pi)*1/2
         Btt3g1c=Btt3g1c/2d0
c      write(*,*)"Btt3g1c",i1,i2,i3,i4,i5,Btt3g1c

      return
      end
************************************************************************
*
      function zB3g1AAa(ihel,iflip,i1,i2,i3,i4,i5,renscale2)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /scheme/ deltaR
      common /order/iorder

      zB3g1AAa     = zB123l(ihel,iflip,i1,i2,i3,i4,i5,renscale2)
     .             + zB123m(ihel,iflip,i1,i2,i3,i4,i5,renscale2)
     .             - zB123nf(ihel,iflip,i1,i2,i3,i4,i5,renscale2)

c      write(*,*)"l",ihel,iflip,zB123l(ihel,iflip,i1,i2,i3,i4,i5,renscale2)
c      write(*,*)"m",ihel,iflip,zB123m(ihel,iflip,i1,i2,i3,i4,i5,renscale2)
c      write(*,*)"f",ihel,iflip,zB123nf(ihel,iflip,i1,i2,i3,i4,i5,renscale2)



* renormalisation
      if(iorder.eq.-1)then
      zB3g1AAa =  zB3g1AAa +3d0/2d0*11d0/3d0*zB123tree(ihel,iflip,i1,i2,i3,i4,i5)
      endif
* CDR fixup
      if(iorder.eq.0)then
      zB3g1AAa =  zB3g1AAa +1d0/2d0*zB123tree(ihel,iflip,i1,i2,i3,i4,i5)
      endif
      return
      end
************************************************************************
*
      function zB3g1AAb(ihel,iflip,i1,i2,i3,i4,i5,renscale2)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /scheme/ deltaR
      common /order/iorder

      zB3g1AAb    =  zB123nf(ihel,iflip,i1,i2,i3,i4,i5,renscale2)
* renormalisation
      if(iorder.eq.-1)then
      zB3g1AAb =  zB3g1AAb -3d0/2d0*2d0/3d0*zB123tree(ihel,iflip,i1,i2,i3,i4,i5)
      endif
*      write(*,*)ihel,iflip,zB3g1AAb
      return
      end
************************************************************************
*
      function zB3g1AAc(ihel,iflip,i1,i2,i3,i4,i5,renscale2)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /scheme/ deltaR
      common /order/iorder

      zB3g1AAc    = zB123s(ihel,iflip,i1,i2,i3,i4,i5,renscale2)
     .            + zB123m(ihel,iflip,i1,i2,i3,i4,i5,renscale2)
* DRED to CDR fixup
      if(iorder.eq.0)then
      zB3g1AAc =  zB3g1AAc -1d0/2d0*zB123tree(ihel,iflip,i1,i2,i3,i4,i5)
      endif
      return
      end
************************************************************************
*
* three gluon two quark helicity loop amplitudes zBt3g1AAa
*
*
      function zBt3g1AAa(ihel,iflip,i1,i2,i3,i4,i5,renscale2)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /order/iorder
      parameter(pi=3.141592653589793238d0)


*      iflip = 1 -> A(i,j) <-> B(j,i) and line reversal
*                           234      324      342
*      ihel = 1  iflip = 0 pmppm -> ppmpm -> pppmm   1 2 3
*      ihel = 2  iflip = 0 ppmpm -> pmppm -> pmppm   2 1 1
*      ihel = 3  iflip = 0 pppmm -> pppmm -> ppmpm   3 3 2
*      ihel = 1  iflip = 1 pmmpm -> pmmpm -> pmpmm   1 1 2
*      ihel = 2  iflip = 1 pmpmm -> ppmmm -> ppmmm   2 3 3
*      ihel = 3  iflip = 1 ppmmm -> pmpmm -> pmmpm   3 2 1
*
*      i1,i2,i3,i4,i5 -> i5,i4,i3,i2,i1
*
*      ihel = 1  iflip = 2 mppmp -> mppmp -> mpmpp   1 1 2
*      ihel = 2  iflip = 2 mpmpp -> mmppp -> mmppp   2 3 3
*      ihel = 3  iflip = 2 mmppp -> mpmpp -> mppmp   3 2 1
*      ihel = 1  iflip = 3 mpmmp -> mmpmp -> mmmpp   1 2 3
*      ihel = 2  iflip = 3 mmpmp -> mpmmp -> mpmmp   2 1 1
*      ihel = 3  iflip = 3 mmmpp -> mmmpp -> mmpmp   3 3 2

*                           A B               B A               A B               B A
*                          12345             54321             54321             12345
*      ihel = 1  iflip = 0 pmppm   iflip = 1 pmmpm   iflip = 2 mppmp   iflip = 3 mpmmp
*      ihel = 2  iflip = 0 ppmpm   iflip = 1 pmpmm   iflip = 2 mpmpp   iflip = 3 mmpmp
*      ihel = 3  iflip = 0 pppmm   iflip = 1 ppmmm   iflip = 2 mmppp   iflip = 3 mmmpp

      if(iflip.eq.0.or.iflip.eq.3)then
      if(ihel.eq.1)then

        zwt=zB3g1AAa(1,iflip,i1,i2,i3,i4,i5,renscale2)
     .     +zB3g1AAa(2,iflip,i1,i3,i2,i4,i5,renscale2)
     .     +zB3g1AAa(3,iflip,i1,i3,i4,i2,i5,renscale2)

      elseif(ihel.eq.2)then

        zwt=zB3g1AAa(2,iflip,i1,i2,i3,i4,i5,renscale2)
     .     +zB3g1AAa(1,iflip,i1,i3,i2,i4,i5,renscale2)
     .     +zB3g1AAa(1,iflip,i1,i3,i4,i2,i5,renscale2)

      elseif(ihel.eq.3)then

        zwt=zB3g1AAa(3,iflip,i1,i2,i3,i4,i5,renscale2)
     .     +zB3g1AAa(3,iflip,i1,i3,i2,i4,i5,renscale2)
     .     +zB3g1AAa(2,iflip,i1,i3,i4,i2,i5,renscale2)
      endif
      elseif(iflip.eq.1.or.iflip.eq.2)then
      if(ihel.eq.1)then

        zwt=zB3g1AAa(1,iflip,i1,i2,i3,i4,i5,renscale2)
     .     +zB3g1AAa(1,iflip,i1,i3,i2,i4,i5,renscale2)
     .     +zB3g1AAa(2,iflip,i1,i3,i4,i2,i5,renscale2)

      elseif(ihel.eq.2)then

        zwt=zB3g1AAa(2,iflip,i1,i2,i3,i4,i5,renscale2)
     .     +zB3g1AAa(3,iflip,i1,i3,i2,i4,i5,renscale2)
     .     +zB3g1AAa(3,iflip,i1,i3,i4,i2,i5,renscale2)

      elseif(ihel.eq.3)then

        zwt=zB3g1AAa(3,iflip,i1,i2,i3,i4,i5,renscale2)
     .     +zB3g1AAa(2,iflip,i1,i3,i2,i4,i5,renscale2)
     .     +zB3g1AAa(1,iflip,i1,i3,i4,i2,i5,renscale2)

      endif
      endif

      zBt3g1AAa=zwt

c      write(*,*)"zBt3g1AAa ",iorder, ihel,iflip,zBt3g1AAa
      return
      end
************************************************************************
*
* three gluon two quark helicity loop amplitudes zBt3g1AAb
*
*
      function zBt3g1AAb(ihel,iflip,i1,i2,i3,i4,i5,renscale2)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /order/iorder
      parameter(pi=3.141592653589793238d0)


*      iflip = 1 -> A(i,j) <-> B(j,i) and line reversal
*                           234      324      342
*      ihel = 1  iflip = 0 pmppm -> ppmpm -> pppmm   1 2 3
*      ihel = 2  iflip = 0 ppmpm -> pmppm -> pmppm   2 1 1
*      ihel = 3  iflip = 0 pppmm -> pppmm -> ppmpm   3 3 2
*      ihel = 1  iflip = 1 pmmpm -> pmmpm -> pmpmm   1 1 2
*      ihel = 2  iflip = 1 pmpmm -> ppmmm -> ppmmm   2 3 3
*      ihel = 3  iflip = 1 ppmmm -> pmpmm -> pmmpm   3 2 1
*
*      i1,i2,i3,i4,i5 -> i5,i4,i3,i2,i1
*
*      ihel = 1  iflip = 2 mppmp -> mppmp -> mpmpp   1 1 2
*      ihel = 2  iflip = 2 mpmpp -> mmppp -> mmppp   2 3 3
*      ihel = 3  iflip = 2 mmppp -> mpmpp -> mppmp   3 2 1
*      ihel = 1  iflip = 3 mpmmp -> mmpmp -> mmmpp   1 2 3
*      ihel = 2  iflip = 3 mmpmp -> mpmmp -> mpmmp   2 1 1
*      ihel = 3  iflip = 3 mmmpp -> mmmpp -> mmpmp   3 3 2

*                           A B               B A               A B               B A
*                          12345             54321             54321             12345
*      ihel = 1  iflip = 0 pmppm   iflip = 1 pmmpm   iflip = 2 mppmp   iflip = 3 mpmmp
*      ihel = 2  iflip = 0 ppmpm   iflip = 1 pmpmm   iflip = 2 mpmpp   iflip = 3 mmpmp
*      ihel = 3  iflip = 0 pppmm   iflip = 1 ppmmm   iflip = 2 mmppp   iflip = 3 mmmpp

      if(iflip.eq.0.or.iflip.eq.3)then
      if(ihel.eq.1)then

        zwt=zB3g1AAb(1,iflip,i1,i2,i3,i4,i5,renscale2)
     .     +zB3g1AAb(2,iflip,i1,i3,i2,i4,i5,renscale2)
     .     +zB3g1AAb(3,iflip,i1,i3,i4,i2,i5,renscale2)

      elseif(ihel.eq.2)then

        zwt=zB3g1AAb(2,iflip,i1,i2,i3,i4,i5,renscale2)
     .     +zB3g1AAb(1,iflip,i1,i3,i2,i4,i5,renscale2)
     .     +zB3g1AAb(1,iflip,i1,i3,i4,i2,i5,renscale2)

      elseif(ihel.eq.3)then

        zwt=zB3g1AAb(3,iflip,i1,i2,i3,i4,i5,renscale2)
     .     +zB3g1AAb(3,iflip,i1,i3,i2,i4,i5,renscale2)
     .     +zB3g1AAb(2,iflip,i1,i3,i4,i2,i5,renscale2)
      endif
      elseif(iflip.eq.1.or.iflip.eq.2)then
      if(ihel.eq.1)then

        zwt=zB3g1AAb(1,iflip,i1,i2,i3,i4,i5,renscale2)
     .     +zB3g1AAb(1,iflip,i1,i3,i2,i4,i5,renscale2)
     .     +zB3g1AAb(2,iflip,i1,i3,i4,i2,i5,renscale2)

      elseif(ihel.eq.2)then

        zwt=zB3g1AAb(2,iflip,i1,i2,i3,i4,i5,renscale2)
     .     +zB3g1AAb(3,iflip,i1,i3,i2,i4,i5,renscale2)
     .     +zB3g1AAb(3,iflip,i1,i3,i4,i2,i5,renscale2)

      elseif(ihel.eq.3)then

        zwt=zB3g1AAb(3,iflip,i1,i2,i3,i4,i5,renscale2)
     .     +zB3g1AAb(2,iflip,i1,i3,i2,i4,i5,renscale2)
     .     +zB3g1AAb(1,iflip,i1,i3,i4,i2,i5,renscale2)

      endif
      endif

      zBt3g1AAb=zwt

c      write(*,*)"zBt3g1AAb ",iorder, ihel,iflip,zBt3g1AAb
      return
      end
************************************************************************
*
* three gluon two quark helicity loop amplitudes zBt3g1AAc
*
*
      function zBt3g1AAc(ihel,iflip,i1,i2,i3,i4,i5,renscale2)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /order/iorder
      parameter(pi=3.141592653589793238d0)


*      iflip = 1 -> A(i,j) <-> B(j,i) and line reversal
*                           234      324      342
*      ihel = 1  iflip = 0 pmppm -> ppmpm -> pppmm   1 2 3
*      ihel = 2  iflip = 0 ppmpm -> pmppm -> pmppm   2 1 1
*      ihel = 3  iflip = 0 pppmm -> pppmm -> ppmpm   3 3 2
*      ihel = 1  iflip = 1 pmmpm -> pmmpm -> pmpmm   1 1 2
*      ihel = 2  iflip = 1 pmpmm -> ppmmm -> ppmmm   2 3 3
*      ihel = 3  iflip = 1 ppmmm -> pmpmm -> pmmpm   3 2 1
*
*      i1,i2,i3,i4,i5 -> i5,i4,i3,i2,i1
*
*      ihel = 1  iflip = 2 mppmp -> mppmp -> mpmpp   1 1 2
*      ihel = 2  iflip = 2 mpmpp -> mmppp -> mmppp   2 3 3
*      ihel = 3  iflip = 2 mmppp -> mpmpp -> mppmp   3 2 1
*      ihel = 1  iflip = 3 mpmmp -> mmpmp -> mmmpp   1 2 3
*      ihel = 2  iflip = 3 mmpmp -> mpmmp -> mpmmp   2 1 1
*      ihel = 3  iflip = 3 mmmpp -> mmmpp -> mmpmp   3 3 2

*                           A B               B A               A B               B A
*                          12345             54321             54321             12345
*      ihel = 1  iflip = 0 pmppm   iflip = 1 pmmpm   iflip = 2 mppmp   iflip = 3 mpmmp
*      ihel = 2  iflip = 0 ppmpm   iflip = 1 pmpmm   iflip = 2 mpmpp   iflip = 3 mmpmp
*      ihel = 3  iflip = 0 pppmm   iflip = 1 ppmmm   iflip = 2 mmppp   iflip = 3 mmmpp

      if(iflip.eq.0.or.iflip.eq.3)then
      if(ihel.eq.1)then

        zwt=zB3g1AAc(1,iflip,i1,i2,i3,i4,i5,renscale2)
     .     +zB3g1AAc(2,iflip,i1,i3,i2,i4,i5,renscale2)
     .     +zB3g1AAc(3,iflip,i1,i3,i4,i2,i5,renscale2)

      elseif(ihel.eq.2)then

        zwt=zB3g1AAc(2,iflip,i1,i2,i3,i4,i5,renscale2)
     .     +zB3g1AAc(1,iflip,i1,i3,i2,i4,i5,renscale2)
     .     +zB3g1AAc(1,iflip,i1,i3,i4,i2,i5,renscale2)

      elseif(ihel.eq.3)then

        zwt=zB3g1AAc(3,iflip,i1,i2,i3,i4,i5,renscale2)
     .     +zB3g1AAc(3,iflip,i1,i3,i2,i4,i5,renscale2)
     .     +zB3g1AAc(2,iflip,i1,i3,i4,i2,i5,renscale2)
      endif
      elseif(iflip.eq.1.or.iflip.eq.2)then
      if(ihel.eq.1)then

        zwt=zB3g1AAc(1,iflip,i1,i2,i3,i4,i5,renscale2)
     .     +zB3g1AAc(1,iflip,i1,i3,i2,i4,i5,renscale2)
     .     +zB3g1AAc(2,iflip,i1,i3,i4,i2,i5,renscale2)

      elseif(ihel.eq.2)then

        zwt=zB3g1AAc(2,iflip,i1,i2,i3,i4,i5,renscale2)
     .     +zB3g1AAc(3,iflip,i1,i3,i2,i4,i5,renscale2)
     .     +zB3g1AAc(3,iflip,i1,i3,i4,i2,i5,renscale2)

      elseif(ihel.eq.3)then

        zwt=zB3g1AAc(3,iflip,i1,i2,i3,i4,i5,renscale2)
     .     +zB3g1AAc(2,iflip,i1,i3,i2,i4,i5,renscale2)
     .     +zB3g1AAc(1,iflip,i1,i3,i4,i2,i5,renscale2)

      endif
      endif

      zBt3g1AAc=zwt

c      write(*,*)"zBt3g1AAc ",iorder, ihel,iflip,zBt3g1AAc
      return
      end
************************************************************************
*
* three gluon two quark helicity loop amplitudes zBtt3g1AAa
*
*
      function zBtt3g1AAa(ihel,iflip,i1,i2,i3,i4,i5,renscale2)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /order/iorder
      parameter(pi=3.141592653589793238d0)


*      iflip = 1 -> A(i,j) <-> B(j,i) and line reversal
*                           234      243      324      342      423      432
*      ihel = 1  iflip = 0 pmppm -> pmppm -> ppmpm -> pppmm -> ppmpm -> pppmm   1 1 2 3 2 3
*      ihel = 2  iflip = 0 ppmpm -> pppmm -> pmppm -> pmppm -> pppmm -> ppmpm   2 3 1 1 3 2
*      ihel = 3  iflip = 0 pppmm -> ppmpm -> pppmm -> ppmpm -> pmppm -> pmppm   3 2 3 2 1 1
*      ihel = 1  iflip = 1 pmmpm -> pmpmm -> pmmpm -> pmpmm -> ppmmm -> ppmmm   1 2 1 2 3 3
*      ihel = 2  iflip = 1 pmpmm -> pmmpm -> ppmmm -> ppmmm -> pmmpm -> pmpmm   2 1 3 3 1 2
*      ihel = 3  iflip = 1 ppmmm -> ppmmm -> pmpmm -> pmmpm -> pmpmm -> pppmm   3 3 2 1 2 1
*
*      i1,i2,i3,i4,i5 -> i5,i4,i3,i2,i1
*
*      ihel = 1  iflip = 2 mppmp -> mpmpp -> mppmp -> mpmpp -> mmppp -> mmppp   1 2 1 2 3 3
*      ihel = 2  iflip = 2 mpmpp -> mppmp -> mmppp -> mmppp -> mppmp -> mpmpp   2 1 3 3 1 2
*      ihel = 3  iflip = 2 mmppp -> mmppp -> mpmpp -> mppmp -> mpmpp -> mppmp   3 3 2 1 2 1
*      ihel = 1  iflip = 3 mpmmp -> mpmmp -> mmpmp -> mmmpp -> mmpmp -> mmmpp   1 1 2 3 2 3
*      ihel = 2  iflip = 3 mmpmp -> mmmpp -> mpmmp -> mpmmp -> mmmpp -> mmpmp   2 3 1 1 3 2
*      ihel = 3  iflip = 3 mmmpp -> mmpmp -> mmmpp -> mmpmp -> mpmmp -> mpmmp   3 2 3 2 1 1

*                           A B               B A               A B               B A
*                          12345             54321             54321             12345
*      ihel = 1  iflip = 0 pmppm   iflip = 1 pmmpm   iflip = 2 mppmp   iflip = 3 mpmmp
*      ihel = 2  iflip = 0 ppmpm   iflip = 1 pmpmm   iflip = 2 mpmpp   iflip = 3 mmpmp
*      ihel = 3  iflip = 0 pppmm   iflip = 1 ppmmm   iflip = 2 mmppp   iflip = 3 mmmpp


      if(iflip.eq.0.or.iflip.eq.3)then
      if(ihel.eq.1)then

        zwt=zB3g1AAa(1,iflip,i1,i2,i3,i4,i5,renscale2)
     .     +zB3g1AAa(1,iflip,i1,i2,i4,i3,i5,renscale2)
     .     +zB3g1AAa(2,iflip,i1,i3,i2,i4,i5,renscale2)
     .     +zB3g1AAa(3,iflip,i1,i3,i4,i2,i5,renscale2)
     .     +zB3g1AAa(2,iflip,i1,i4,i2,i3,i5,renscale2)
     .     +zB3g1AAa(3,iflip,i1,i4,i3,i2,i5,renscale2)

      elseif(ihel.eq.2)then

        zwt=zB3g1AAa(2,iflip,i1,i2,i3,i4,i5,renscale2)
     .     +zB3g1AAa(3,iflip,i1,i2,i4,i3,i5,renscale2)
     .     +zB3g1AAa(1,iflip,i1,i3,i2,i4,i5,renscale2)
     .     +zB3g1AAa(1,iflip,i1,i3,i4,i2,i5,renscale2)
     .     +zB3g1AAa(3,iflip,i1,i4,i2,i3,i5,renscale2)
     .     +zB3g1AAa(2,iflip,i1,i4,i3,i2,i5,renscale2)

      elseif(ihel.eq.3)then

        zwt=zB3g1AAa(3,iflip,i1,i2,i3,i4,i5,renscale2)
     .     +zB3g1AAa(2,iflip,i1,i2,i4,i3,i5,renscale2)
     .     +zB3g1AAa(3,iflip,i1,i3,i2,i4,i5,renscale2)
     .     +zB3g1AAa(2,iflip,i1,i3,i4,i2,i5,renscale2)
     .     +zB3g1AAa(1,iflip,i1,i4,i2,i3,i5,renscale2)
     .     +zB3g1AAa(1,iflip,i1,i4,i3,i2,i5,renscale2)

      endif
      elseif(iflip.eq.1.or.iflip.eq.2)then
      if(ihel.eq.1)then

        zwt=zB3g1AAa(1,iflip,i1,i2,i3,i4,i5,renscale2)
     .     +zB3g1AAa(2,iflip,i1,i2,i4,i3,i5,renscale2)
     .     +zB3g1AAa(1,iflip,i1,i3,i2,i4,i5,renscale2)
     .     +zB3g1AAa(2,iflip,i1,i3,i4,i2,i5,renscale2)
     .     +zB3g1AAa(3,iflip,i1,i4,i2,i3,i5,renscale2)
     .     +zB3g1AAa(3,iflip,i1,i4,i3,i2,i5,renscale2)

      elseif(ihel.eq.2)then

        zwt=zB3g1AAa(2,iflip,i1,i2,i3,i4,i5,renscale2)
     .     +zB3g1AAa(1,iflip,i1,i2,i4,i3,i5,renscale2)
     .     +zB3g1AAa(3,iflip,i1,i3,i2,i4,i5,renscale2)
     .     +zB3g1AAa(3,iflip,i1,i3,i4,i2,i5,renscale2)
     .     +zB3g1AAa(1,iflip,i1,i4,i2,i3,i5,renscale2)
     .     +zB3g1AAa(2,iflip,i1,i4,i3,i2,i5,renscale2)

      elseif(ihel.eq.3)then

        zwt=zB3g1AAa(3,iflip,i1,i2,i3,i4,i5,renscale2)
     .     +zB3g1AAa(3,iflip,i1,i2,i4,i3,i5,renscale2)
     .     +zB3g1AAa(2,iflip,i1,i3,i2,i4,i5,renscale2)
     .     +zB3g1AAa(1,iflip,i1,i3,i4,i2,i5,renscale2)
     .     +zB3g1AAa(2,iflip,i1,i4,i2,i3,i5,renscale2)
     .     +zB3g1AAa(1,iflip,i1,i4,i3,i2,i5,renscale2)

      endif
      endif

      zBtt3g1AAa=zwt

c      write(*,*)"zBtt3g1AAa ",iorder, ihel,iflip,zBtt3g1AAa
      return
      end
************************************************************************
*
* three gluon two quark helicity loop amplitudes zBtt3g1AAb
*
*
      function zBtt3g1AAb(ihel,iflip,i1,i2,i3,i4,i5,renscale2)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /order/iorder
      parameter(pi=3.141592653589793238d0)


*      iflip = 1 -> A(i,j) <-> B(j,i) and line reversal
*                           234      243      324      342      423      432
*      ihel = 1  iflip = 0 pmppm -> pmppm -> ppmpm -> pppmm -> ppmpm -> pppmm   1 1 2 3 2 3
*      ihel = 2  iflip = 0 ppmpm -> pppmm -> pmppm -> pmppm -> pppmm -> ppmpm   2 3 1 1 3 2
*      ihel = 3  iflip = 0 pppmm -> ppmpm -> pppmm -> ppmpm -> pmppm -> pmppm   3 2 3 2 1 1
*      ihel = 1  iflip = 1 pmmpm -> pmpmm -> pmmpm -> pmpmm -> ppmmm -> ppmmm   1 2 1 2 3 3
*      ihel = 2  iflip = 1 pmpmm -> pmmpm -> ppmmm -> ppmmm -> pmmpm -> pmpmm   2 1 3 3 1 2
*      ihel = 3  iflip = 1 ppmmm -> ppmmm -> pmpmm -> pmmpm -> pmpmm -> pppmm   3 3 2 1 2 1
*
*      i1,i2,i3,i4,i5 -> i5,i4,i3,i2,i1
*
*      ihel = 1  iflip = 2 mppmp -> mpmpp -> mppmp -> mpmpp -> mmppp -> mmppp   1 2 1 2 3 3
*      ihel = 2  iflip = 2 mpmpp -> mppmp -> mmppp -> mmppp -> mppmp -> mpmpp   2 1 3 3 1 2
*      ihel = 3  iflip = 2 mmppp -> mmppp -> mpmpp -> mppmp -> mpmpp -> mppmp   3 3 2 1 2 1
*      ihel = 1  iflip = 3 mpmmp -> mpmmp -> mmpmp -> mmmpp -> mmpmp -> mmmpp   1 1 2 3 2 3
*      ihel = 2  iflip = 3 mmpmp -> mmmpp -> mpmmp -> mpmmp -> mmmpp -> mmpmp   2 3 1 1 3 2
*      ihel = 3  iflip = 3 mmmpp -> mmpmp -> mmmpp -> mmpmp -> mpmmp -> mpmmp   3 2 3 2 1 1

*                           A B               B A               A B               B A
*                          12345             54321             54321             12345
*      ihel = 1  iflip = 0 pmppm   iflip = 1 pmmpm   iflip = 2 mppmp   iflip = 3 mpmmp
*      ihel = 2  iflip = 0 ppmpm   iflip = 1 pmpmm   iflip = 2 mpmpp   iflip = 3 mmpmp
*      ihel = 3  iflip = 0 pppmm   iflip = 1 ppmmm   iflip = 2 mmppp   iflip = 3 mmmpp


      if(iflip.eq.0.or.iflip.eq.3)then
      if(ihel.eq.1)then

        zwt=zB3g1AAb(1,iflip,i1,i2,i3,i4,i5,renscale2)
     .     +zB3g1AAb(1,iflip,i1,i2,i4,i3,i5,renscale2)
     .     +zB3g1AAb(2,iflip,i1,i3,i2,i4,i5,renscale2)
     .     +zB3g1AAb(3,iflip,i1,i3,i4,i2,i5,renscale2)
     .     +zB3g1AAb(2,iflip,i1,i4,i2,i3,i5,renscale2)
     .     +zB3g1AAb(3,iflip,i1,i4,i3,i2,i5,renscale2)

      elseif(ihel.eq.2)then

        zwt=zB3g1AAb(2,iflip,i1,i2,i3,i4,i5,renscale2)
     .     +zB3g1AAb(3,iflip,i1,i2,i4,i3,i5,renscale2)
     .     +zB3g1AAb(1,iflip,i1,i3,i2,i4,i5,renscale2)
     .     +zB3g1AAb(1,iflip,i1,i3,i4,i2,i5,renscale2)
     .     +zB3g1AAb(3,iflip,i1,i4,i2,i3,i5,renscale2)
     .     +zB3g1AAb(2,iflip,i1,i4,i3,i2,i5,renscale2)

      elseif(ihel.eq.3)then

        zwt=zB3g1AAb(3,iflip,i1,i2,i3,i4,i5,renscale2)
     .     +zB3g1AAb(2,iflip,i1,i2,i4,i3,i5,renscale2)
     .     +zB3g1AAb(3,iflip,i1,i3,i2,i4,i5,renscale2)
     .     +zB3g1AAb(2,iflip,i1,i3,i4,i2,i5,renscale2)
     .     +zB3g1AAb(1,iflip,i1,i4,i2,i3,i5,renscale2)
     .     +zB3g1AAb(1,iflip,i1,i4,i3,i2,i5,renscale2)

      endif
      elseif(iflip.eq.1.or.iflip.eq.2)then
      if(ihel.eq.1)then

        zwt=zB3g1AAb(1,iflip,i1,i2,i3,i4,i5,renscale2)
     .     +zB3g1AAb(2,iflip,i1,i2,i4,i3,i5,renscale2)
     .     +zB3g1AAb(1,iflip,i1,i3,i2,i4,i5,renscale2)
     .     +zB3g1AAb(2,iflip,i1,i3,i4,i2,i5,renscale2)
     .     +zB3g1AAb(3,iflip,i1,i4,i2,i3,i5,renscale2)
     .     +zB3g1AAb(3,iflip,i1,i4,i3,i2,i5,renscale2)

      elseif(ihel.eq.2)then

        zwt=zB3g1AAb(2,iflip,i1,i2,i3,i4,i5,renscale2)
     .     +zB3g1AAb(1,iflip,i1,i2,i4,i3,i5,renscale2)
     .     +zB3g1AAb(3,iflip,i1,i3,i2,i4,i5,renscale2)
     .     +zB3g1AAb(3,iflip,i1,i3,i4,i2,i5,renscale2)
     .     +zB3g1AAb(1,iflip,i1,i4,i2,i3,i5,renscale2)
     .     +zB3g1AAb(2,iflip,i1,i4,i3,i2,i5,renscale2)

      elseif(ihel.eq.3)then

        zwt=zB3g1AAb(3,iflip,i1,i2,i3,i4,i5,renscale2)
     .     +zB3g1AAb(3,iflip,i1,i2,i4,i3,i5,renscale2)
     .     +zB3g1AAb(2,iflip,i1,i3,i2,i4,i5,renscale2)
     .     +zB3g1AAb(1,iflip,i1,i3,i4,i2,i5,renscale2)
     .     +zB3g1AAb(2,iflip,i1,i4,i2,i3,i5,renscale2)
     .     +zB3g1AAb(1,iflip,i1,i4,i3,i2,i5,renscale2)

      endif
      endif

      zBtt3g1AAb=zwt

c      write(*,*)"zBtt3g1AAb ",iorder, ihel,iflip,zBtt3g1AAb
      return
      end
************************************************************************
*
* three gluon two quark helicity loop amplitudes zBtt3g1AAc
*
*
      function zBtt3g1AAc(ihel,iflip,i1,i2,i3,i4,i5,renscale2)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /order/iorder
      parameter(pi=3.141592653589793238d0)


*      iflip = 1 -> A(i,j) <-> B(j,i) and line reversal
*                           234      243      324      342      423      432
*      ihel = 1  iflip = 0 pmppm -> pmppm -> ppmpm -> pppmm -> ppmpm -> pppmm   1 1 2 3 2 3
*      ihel = 2  iflip = 0 ppmpm -> pppmm -> pmppm -> pmppm -> pppmm -> ppmpm   2 3 1 1 3 2
*      ihel = 3  iflip = 0 pppmm -> ppmpm -> pppmm -> ppmpm -> pmppm -> pmppm   3 2 3 2 1 1
*      ihel = 1  iflip = 1 pmmpm -> pmpmm -> pmmpm -> pmpmm -> ppmmm -> ppmmm   1 2 1 2 3 3
*      ihel = 2  iflip = 1 pmpmm -> pmmpm -> ppmmm -> ppmmm -> pmmpm -> pmpmm   2 1 3 3 1 2
*      ihel = 3  iflip = 1 ppmmm -> ppmmm -> pmpmm -> pmmpm -> pmpmm -> pppmm   3 3 2 1 2 1
*
*      i1,i2,i3,i4,i5 -> i5,i4,i3,i2,i1
*
*      ihel = 1  iflip = 2 mppmp -> mpmpp -> mppmp -> mpmpp -> mmppp -> mmppp   1 2 1 2 3 3
*      ihel = 2  iflip = 2 mpmpp -> mppmp -> mmppp -> mmppp -> mppmp -> mpmpp   2 1 3 3 1 2
*      ihel = 3  iflip = 2 mmppp -> mmppp -> mpmpp -> mppmp -> mpmpp -> mppmp   3 3 2 1 2 1
*      ihel = 1  iflip = 3 mpmmp -> mpmmp -> mmpmp -> mmmpp -> mmpmp -> mmmpp   1 1 2 3 2 3
*      ihel = 2  iflip = 3 mmpmp -> mmmpp -> mpmmp -> mpmmp -> mmmpp -> mmpmp   2 3 1 1 3 2
*      ihel = 3  iflip = 3 mmmpp -> mmpmp -> mmmpp -> mmpmp -> mpmmp -> mpmmp   3 2 3 2 1 1

*                           A B               B A               A B               B A
*                          12345             54321             54321             12345
*      ihel = 1  iflip = 0 pmppm   iflip = 1 pmmpm   iflip = 2 mppmp   iflip = 3 mpmmp
*      ihel = 2  iflip = 0 ppmpm   iflip = 1 pmpmm   iflip = 2 mpmpp   iflip = 3 mmpmp
*      ihel = 3  iflip = 0 pppmm   iflip = 1 ppmmm   iflip = 2 mmppp   iflip = 3 mmmpp


      if(iflip.eq.0.or.iflip.eq.3)then
      if(ihel.eq.1)then

        zwt=zB3g1AAc(1,iflip,i1,i2,i3,i4,i5,renscale2)
     .     +zB3g1AAc(1,iflip,i1,i2,i4,i3,i5,renscale2)
     .     +zB3g1AAc(2,iflip,i1,i3,i2,i4,i5,renscale2)
     .     +zB3g1AAc(3,iflip,i1,i3,i4,i2,i5,renscale2)
     .     +zB3g1AAc(2,iflip,i1,i4,i2,i3,i5,renscale2)
     .     +zB3g1AAc(3,iflip,i1,i4,i3,i2,i5,renscale2)

      elseif(ihel.eq.2)then

        zwt=zB3g1AAc(2,iflip,i1,i2,i3,i4,i5,renscale2)
     .     +zB3g1AAc(3,iflip,i1,i2,i4,i3,i5,renscale2)
     .     +zB3g1AAc(1,iflip,i1,i3,i2,i4,i5,renscale2)
     .     +zB3g1AAc(1,iflip,i1,i3,i4,i2,i5,renscale2)
     .     +zB3g1AAc(3,iflip,i1,i4,i2,i3,i5,renscale2)
     .     +zB3g1AAc(2,iflip,i1,i4,i3,i2,i5,renscale2)

      elseif(ihel.eq.3)then

        zwt=zB3g1AAc(3,iflip,i1,i2,i3,i4,i5,renscale2)
     .     +zB3g1AAc(2,iflip,i1,i2,i4,i3,i5,renscale2)
     .     +zB3g1AAc(3,iflip,i1,i3,i2,i4,i5,renscale2)
     .     +zB3g1AAc(2,iflip,i1,i3,i4,i2,i5,renscale2)
     .     +zB3g1AAc(1,iflip,i1,i4,i2,i3,i5,renscale2)
     .     +zB3g1AAc(1,iflip,i1,i4,i3,i2,i5,renscale2)

      endif
      elseif(iflip.eq.1.or.iflip.eq.2)then
      if(ihel.eq.1)then

        zwt=zB3g1AAc(1,iflip,i1,i2,i3,i4,i5,renscale2)
     .     +zB3g1AAc(2,iflip,i1,i2,i4,i3,i5,renscale2)
     .     +zB3g1AAc(1,iflip,i1,i3,i2,i4,i5,renscale2)
     .     +zB3g1AAc(2,iflip,i1,i3,i4,i2,i5,renscale2)
     .     +zB3g1AAc(3,iflip,i1,i4,i2,i3,i5,renscale2)
     .     +zB3g1AAc(3,iflip,i1,i4,i3,i2,i5,renscale2)

      elseif(ihel.eq.2)then

        zwt=zB3g1AAc(2,iflip,i1,i2,i3,i4,i5,renscale2)
     .     +zB3g1AAc(1,iflip,i1,i2,i4,i3,i5,renscale2)
     .     +zB3g1AAc(3,iflip,i1,i3,i2,i4,i5,renscale2)
     .     +zB3g1AAc(3,iflip,i1,i3,i4,i2,i5,renscale2)
     .     +zB3g1AAc(1,iflip,i1,i4,i2,i3,i5,renscale2)
     .     +zB3g1AAc(2,iflip,i1,i4,i3,i2,i5,renscale2)

      elseif(ihel.eq.3)then

        zwt=zB3g1AAc(3,iflip,i1,i2,i3,i4,i5,renscale2)
     .     +zB3g1AAc(3,iflip,i1,i2,i4,i3,i5,renscale2)
     .     +zB3g1AAc(2,iflip,i1,i3,i2,i4,i5,renscale2)
     .     +zB3g1AAc(1,iflip,i1,i3,i4,i2,i5,renscale2)
     .     +zB3g1AAc(2,iflip,i1,i4,i2,i3,i5,renscale2)
     .     +zB3g1AAc(1,iflip,i1,i4,i3,i2,i5,renscale2)

      endif
      endif

      zBtt3g1AAc=zwt

c      write(*,*)"zBtt3g1AAc ",iorder, ihel,iflip,zBtt3g1AAc
      return
      end
************************************************************************
*
* three gluon two quark helicity amplitudes zB123l
*
*
      function zB123l(ihel,iflip,i1,i2,i3,i4,i5,renscale2)
      use KinData_mod, only : zA5, zB5, s5
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /order/iorder
      parameter(pi=3.141592653589793238d0)

*      iflip = 1 -> A(i,j) <-> B(j,i) and line reversal
*      ihel = 1  iflip = 0 pmppm
*      ihel = 2  iflip = 0 ppmpm
*      ihel = 3  iflip = 0 pppmm
*      ihel = 1  iflip = 1 pmmpm
*      ihel = 2  iflip = 1 pmpmm
*      ihel = 3  iflip = 1 ppmmm
*      ihel = 1  iflip = 2 mppmp
*      ihel = 2  iflip = 2 mpmpp
*      ihel = 3  iflip = 2 mmppp
*      ihel = 1  iflip = 3 mpmmp
*      ihel = 2  iflip = 3 mmpmp
*      ihel = 3  iflip = 3 mmmpp

      if(iflip.eq.0.or.iflip.eq.3)then
        j1=i1
        j2=i2
        j3=i3
        j4=i4
        j5=i5
      elseif(iflip.eq.1.or.iflip.eq.2)then
        j1=i5
        j2=i4
        j3=i3
        j4=i2
        j5=i1
      endif
      if(iflip.eq.0.or.iflip.eq.2)then
        za12=zA5(j1,j2)
        za21=zA5(j2,j1)
        za13=zA5(j1,j3)
        za31=zA5(j3,j1)
        za14=zA5(j1,j4)
        za41=zA5(j4,j1)
        za15=zA5(j1,j5)
        za51=zA5(j5,j1)
        za23=zA5(j2,j3)
        za32=zA5(j3,j2)
        za24=zA5(j2,j4)
        za42=zA5(j4,j2)
        za25=zA5(j2,j5)
        za52=zA5(j5,j2)
        za34=zA5(j3,j4)
        za43=zA5(j4,j3)
        za35=zA5(j3,j5)
        za53=zA5(j5,j3)
        za45=zA5(j4,j5)
        za54=zA5(j5,j4)

        zb12=zB5(j1,j2)
        zb21=zB5(j2,j1)
        zb13=zB5(j1,j3)
        zb31=zB5(j3,j1)
        zb14=zB5(j1,j4)
        zb41=zB5(j4,j1)
        zb15=zB5(j1,j5)
        zb51=zB5(j5,j1)
        zb23=zB5(j2,j3)
        zb32=zB5(j3,j2)
        zb24=zB5(j2,j4)
        zb42=zB5(j4,j2)
        zb25=zB5(j2,j5)
        zb52=zB5(j5,j2)
        zb34=zB5(j3,j4)
        zb43=zB5(j4,j3)
        zb35=zB5(j3,j5)
        zb53=zB5(j5,j3)
        zb45=zB5(j4,j5)
        zb54=zB5(j5,j4)

      elseif(iflip.eq.1.or.iflip.eq.3)then
        za12=zB5(j1,j2)
        za21=zB5(j2,j1)
        za13=zB5(j1,j3)
        za31=zB5(j3,j1)
        za14=zB5(j1,j4)
        za41=zB5(j4,j1)
        za15=zB5(j1,j5)
        za51=zB5(j5,j1)
        za23=zB5(j2,j3)
        za32=zB5(j3,j2)
        za24=zB5(j2,j4)
        za42=zB5(j4,j2)
        za25=zB5(j2,j5)
        za52=zB5(j5,j2)
        za34=zB5(j3,j4)
        za43=zB5(j4,j3)
        za35=zB5(j3,j5)
        za53=zB5(j5,j3)
        za45=zB5(j4,j5)
        za54=zB5(j5,j4)

        zb12=zA5(j1,j2)
        zb21=zA5(j2,j1)
        zb13=zA5(j1,j3)
        zb31=zA5(j3,j1)
        zb14=zA5(j1,j4)
        zb41=zA5(j4,j1)
        zb15=zA5(j1,j5)
        zb51=zA5(j5,j1)
        zb23=zA5(j2,j3)
        zb32=zA5(j3,j2)
        zb24=zA5(j2,j4)
        zb42=zA5(j4,j2)
        zb25=zA5(j2,j5)
        zb52=zA5(j5,j2)
        zb34=zA5(j3,j4)
        zb43=zA5(j4,j3)
        zb35=zA5(j3,j5)
        zb53=zA5(j5,j3)
        zb45=zA5(j4,j5)
        zb54=zA5(j5,j4)
      endif
        s12=s5(j1,j2)
        s13=s5(j1,j3)
        s14=s5(j1,j4)
        s15=s5(j1,j5)
        s21=s5(j2,j1)
        s23=s5(j2,j3)
        s24=s5(j2,j4)
        s25=s5(j2,j5)
        s31=s5(j3,j1)
        s32=s5(j3,j2)
        s34=s5(j3,j4)
        s35=s5(j3,j5)
        s41=s5(j4,j1)
        s42=s5(j4,j2)
        s43=s5(j4,j3)
        s45=s5(j4,j5)
        s51=s5(j5,j1)
        s52=s5(j5,j2)
        s53=s5(j5,j3)
        s54=s5(j5,j4)

        zi = (0d0,1d0)
        zone = (1d0,0d0)
*                           A B               B A               A B               B A
*                          12345             54321             54321             12345
*      ihel = 1  iflip = 0 pmppm   iflip = 1 pmmpm   iflip = 2 mppmp   iflip = 3 mpmmp
*      ihel = 2  iflip = 0 ppmpm   iflip = 1 pmpmm   iflip = 2 mpmpp   iflip = 3 mmpmp
*      ihel = 3  iflip = 0 pppmm   iflip = 1 ppmmm   iflip = 2 mmppp   iflip = 3 mmmpp

      if(ihel.eq.1)then

        ztree=za12*za52**(3)/(za12*za23*za34*za45*za51)

        ze0 = ztree
     . *( 0.5d0*(zzlog(renscale2) - zzlog(-s54))**(2)
     .  + 0.5d0*(zzlog(renscale2) - zzlog(-s12))**(2)
     .  + 0.5d0*(zzlog(renscale2) - zzlog(-s23))**(2)
     .  + 0.5d0*(zzlog(renscale2) - zzlog(-s34))**(2)
     .  + (3.d0/2.d0)*(zzlog(renscale2)- zzlog(-s54)) + (5.d0/2.d0))
     .  -3.d0*za52*za52*za53*za12
     .    *zL0(s54,s23,renscale2)/(za51*za54*za13*za23*za34)
     .  -3.d0*za52*za12*za12*zb14*zL1(s54,s23,renscale2)/(za13*za14*za23)
     .  -3.d0*za52*za12*za24*zb14*zL1(s51,s23,renscale2)/(za14*za23*za34)
     .  -3.d0*za52*za53*za12*zb13*zL1(s54,s12,renscale2)/(za54*za13*za34)
     .  + ztree*(zzCF(s25,s51,s12)
     .  + zzCF(s35,s54,s43)
     .  + (za54*za12)**(3)*zzCF(s14,s15,s54)/((za52*za14)**(3))
     .  + (za53*za12)**(3)*zzCF(s13,s12,s23)/((za52*za13)**(3))
     .  + zzCF(s24,s23,s34) )

        ze1 = ztree
     . *( zzlog(renscale2) - zzlog(-s54)
     .  + zzlog(renscale2) - zzlog(-s12)
     .  + zzlog(renscale2) - zzlog(-s23)
     .  + zzlog(renscale2) - zzlog(-s34) + 3d0/2d0*zone)

        ze2 = ztree*4.d0

      elseif(ihel.eq.2)then

        ztree=za13*za53**(3)/(za12*za23*za34*za45*za51)
        ze0 = ztree
     .  *(0.5d0*(zzlog(renscale2) - zzlog(-s54))**(2)
     .  + 0.5d0*(zzlog(renscale2) - zzlog(-s12))**(2)
     .  + 0.5d0*(zzlog(renscale2) - zzlog(-s23))**(2)
     .  + 0.5d0*(zzlog(renscale2) - zzlog(-s34))**(2)
     .  + (3.d0/2.d0)*(zzlog(renscale2) - zzlog(-s23)) + 5.d0/2.d0)
     .  + 2.d0*za53*za53*za13*za13
     .       *zL0(s51,s23,renscale2)/(za51*za12*za14*za23*za34)
     .  - za53**(3)*za13*zL0(s51,s23,renscale2)/(2.d0
     .       *za51*za54*za12*za23*za34)
     .  + za51*za13*za13*za34*zb14*zL1(s51,s23,renscale2)/(za12
     .       *za14**(2)*za23)
     .  - za53*za13*zb24*zL1(s51,s23,renscale2)/(za12*za14)
     .  - 3.d0*za53*za54*za13*zb24
     .       *zL1(s51,s23,renscale2)/(za51*za14*za24)
     .  - 3.d0*za53*za13*za13*zb14
     .       *zL1(s54,s23,renscale2)/(za12*za14*za23)
     .  - 3.d0*za52*za53*za13*zb24
     .       *zL1(s51,s34,renscale2)/(za51*za12*za24)
     .  + ztree*za53*za12*zzCF(s25,s51,s12)/(za52*za13)
     .  + ztree*zzCF(s35,s54,s43)
     .  + ztree*za54**(3)*za13**(3)
     .       *zzCF(s14,s15,s54)/(za53**(3)*za14**(3))
     .  + ztree*zzCF(s13,s12,s23)
     .  + ztree*zzCF(s24,s23,s34)
     .  - ztree*3.d0*za52*za43*za54*za23
     .       *zzCF(s24,s23,s34)/(za53*za42*za53*za24)
     .  - ztree*za51*za32*za52**(3)*za43**(3)
     .       *zzCF(s24,s23,s34)/(za52*za31*za53**(3)*za42**(3))

        ze1 = ztree
     . *( zzlog(renscale2) - zzlog(-s54)
     .  + zzlog(renscale2) - zzlog(-s12)
     .  + zzlog(renscale2) - zzlog(-s23)
     .  + zzlog(renscale2) - zzlog(-s34) + 3d0/2d0*zone)
        ze2 = ztree*(4.d0)

      elseif(ihel.eq.3)then

        ztree=za14*za54**(3)/(za12*za23*za34*za45*za51)

        ze0  = ztree
     .*(  0.5d0*(zzlog(renscale2)- zzlog(-s54))**(2)
     .  + 0.5d0*(zzlog(renscale2) - zzlog(-s12))**(2)
     .  + 0.5d0*(zzlog(renscale2) - zzlog(-s23))**(2)
     .  + 0.5d0*(zzlog(renscale2) - zzlog(-s34))**(2)
     .  + 3.d0/2.d0*(zzlog(renscale2) - zzlog(-s51)) + 5.d0/2.d0 )
     .  + 3.d0*za52*za54*za14*zb23*zL1(s51,s34,renscale2)/(za51*za12*za23)
     .  + ztree
     .*( za54*za12*zzCF(s25,s51,s12)/(za52*za14)
     . + za54*za13*zzCF(s35,s54,s43)/(za53*za14) + zzCF(s14,s15,s54)
     . + zzCF(s13,s12,s23)+ zzCF(s24,s23,s34) )

        ze1  = ztree
     .*( zzlog(renscale2) - zzlog(-s54)
     . + zzlog(renscale2) - zzlog(-s12)
     . + zzlog(renscale2) - zzlog(-s23)
     . + zzlog(renscale2) - zzlog(-s34) + 3d0/2d0*zone)
        ze2  = ztree*(4.d0)
      endif

* correction factor to convert from c_gamma prefactor
       ze0=ze0 - pi**2/12d0*ze2

      if(iorder.eq.-2)then
          zB123l=ze2
      elseif(iorder.eq.-1)then
          zB123l=ze1
      elseif(iorder.eq.0)then
          zB123l=ze0
      endif


c      write(*,*)"zB123l ", iorder,ihel,iflip,zB123l

      return
      end

************************************************************************
*
* three gluon two quark helicity amplitudes zB123m
*
*
      function zB123m(ihel,iflip,i1,i2,i3,i4,i5,renscale2)
      use KinData_mod, only : zA5, zB5, s5
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /order/iorder
      parameter(pi=3.141592653589793238d0)

*      iflip = 1 -> A(i,j) <-> B(j,i) and line reversal
*      ihel = 1  iflip = 0 pmppm
*      ihel = 2  iflip = 0 ppmpm
*      ihel = 3  iflip = 0 pppmm
*      ihel = 1  iflip = 1 pmmpm
*      ihel = 2  iflip = 1 pmpmm
*      ihel = 3  iflip = 1 ppmmm
*      ihel = 1  iflip = 2 mppmp
*      ihel = 2  iflip = 2 mpmpp
*      ihel = 3  iflip = 2 mmppp
*      ihel = 1  iflip = 3 mpmmp
*      ihel = 2  iflip = 3 mmpmp
*      ihel = 3  iflip = 3 mmmpp

      if(iflip.eq.0.or.iflip.eq.3)then
        j1=i1
        j2=i2
        j3=i3
        j4=i4
        j5=i5
      elseif(iflip.eq.1.or.iflip.eq.2)then
        j1=i5
        j2=i4
        j3=i3
        j4=i2
        j5=i1
      endif
      if(iflip.eq.0.or.iflip.eq.2)then
        za12=zA5(j1,j2)
        za21=zA5(j2,j1)
        za13=zA5(j1,j3)
        za31=zA5(j3,j1)
        za14=zA5(j1,j4)
        za41=zA5(j4,j1)
        za15=zA5(j1,j5)
        za51=zA5(j5,j1)
        za23=zA5(j2,j3)
        za32=zA5(j3,j2)
        za24=zA5(j2,j4)
        za42=zA5(j4,j2)
        za25=zA5(j2,j5)
        za52=zA5(j5,j2)
        za34=zA5(j3,j4)
        za43=zA5(j4,j3)
        za35=zA5(j3,j5)
        za53=zA5(j5,j3)
        za45=zA5(j4,j5)
        za54=zA5(j5,j4)

        zb12=zB5(j1,j2)
        zb21=zB5(j2,j1)
        zb13=zB5(j1,j3)
        zb31=zB5(j3,j1)
        zb14=zB5(j1,j4)
        zb41=zB5(j4,j1)
        zb15=zB5(j1,j5)
        zb51=zB5(j5,j1)
        zb23=zB5(j2,j3)
        zb32=zB5(j3,j2)
        zb24=zB5(j2,j4)
        zb42=zB5(j4,j2)
        zb25=zB5(j2,j5)
        zb52=zB5(j5,j2)
        zb34=zB5(j3,j4)
        zb43=zB5(j4,j3)
        zb35=zB5(j3,j5)
        zb53=zB5(j5,j3)
        zb45=zB5(j4,j5)
        zb54=zB5(j5,j4)

      elseif(iflip.eq.1.or.iflip.eq.3)then
        za12=zB5(j1,j2)
        za21=zB5(j2,j1)
        za13=zB5(j1,j3)
        za31=zB5(j3,j1)
        za14=zB5(j1,j4)
        za41=zB5(j4,j1)
        za15=zB5(j1,j5)
        za51=zB5(j5,j1)
        za23=zB5(j2,j3)
        za32=zB5(j3,j2)
        za24=zB5(j2,j4)
        za42=zB5(j4,j2)
        za25=zB5(j2,j5)
        za52=zB5(j5,j2)
        za34=zB5(j3,j4)
        za43=zB5(j4,j3)
        za35=zB5(j3,j5)
        za53=zB5(j5,j3)
        za45=zB5(j4,j5)
        za54=zB5(j5,j4)

        zb12=zA5(j1,j2)
        zb21=zA5(j2,j1)
        zb13=zA5(j1,j3)
        zb31=zA5(j3,j1)
        zb14=zA5(j1,j4)
        zb41=zA5(j4,j1)
        zb15=zA5(j1,j5)
        zb51=zA5(j5,j1)
        zb23=zA5(j2,j3)
        zb32=zA5(j3,j2)
        zb24=zA5(j2,j4)
        zb42=zA5(j4,j2)
        zb25=zA5(j2,j5)
        zb52=zA5(j5,j2)
        zb34=zA5(j3,j4)
        zb43=zA5(j4,j3)
        zb35=zA5(j3,j5)
        zb53=zA5(j5,j3)
        zb45=zA5(j4,j5)
        zb54=zA5(j5,j4)
      endif

        s12=s5(j1,j2)
        s13=s5(j1,j3)
        s14=s5(j1,j4)
        s15=s5(j1,j5)
        s21=s5(j2,j1)
        s23=s5(j2,j3)
        s24=s5(j2,j4)
        s25=s5(j2,j5)
        s31=s5(j3,j1)
        s32=s5(j3,j2)
        s34=s5(j3,j4)
        s35=s5(j3,j5)
        s41=s5(j4,j1)
        s42=s5(j4,j2)
        s43=s5(j4,j3)
        s45=s5(j4,j5)
        s51=s5(j5,j1)
        s52=s5(j5,j2)
        s53=s5(j5,j3)
        s54=s5(j5,j4)
        zi = (0d0,1d0)
        zone = (1d0,0d0)
*                           A B               B A               A B               B A
*                          12345             54321             54321             12345
*      ihel = 1  iflip = 0 pmppm   iflip = 1 pmmpm   iflip = 2 mppmp   iflip = 3 mpmmp
*      ihel = 2  iflip = 0 ppmpm   iflip = 1 pmpmm   iflip = 2 mpmpp   iflip = 3 mmpmp
*      ihel = 3  iflip = 0 pppmm   iflip = 1 ppmmm   iflip = 2 mmppp   iflip = 3 mmmpp


      if(ihel.eq.1)then
        ztree=za12*za52**(3)/(za12*za23*za34*za45*za51)

        ze0 = -1.d0*za52**(3)/(2.d0*za51*za54*za23*za34) ! original amplitude in Signer's thesis
     .  + za52*za53*za12*zb13/(2.d0*za54*za13*za34*s54)
     .  + za52*za12*za12*zb14/(2.d0*za13*za14*za23*s54)
     .  - za12*za12*zb13*zb14/(2.d0*za14*za34*zb54*s23)
     .  - za52*za54*za12*zb34/(2.d0*za51*za14*za34*s23)
     .  - za53*zb13*zb13*zb34/(2.d0*za34*zb12*zb23*s54)
     .  - za52**(3)*zL0(s51,s54,renscale2)/(za51*za54*za23*za34)
     .  - za52*za52*za12*zL0(s23,s51,renscale2)/(2.d0*za51*za14*za23*za34)
     .  - za52*za52*za53*za12
     .      *zL0(s23,s54,renscale2)/(2.d0*za51*za54*za13*za23*za34)
     .  + za52*za12*za24*zb14*zL1(s51,s23,renscale2)/(za14*za23*za34)
     .  - za51*za51*(za24**(3))*zb14
     .      *zL1(s51,s23,renscale2)/(za54*za14*za14*za23*za34)
     .  + za52*za52*zb34*zL1(s51,s23,renscale2)/(za51*za34)
     .  + za52*za53*za12*zb13*zL1(s54,s12,renscale2)/(za54*za13*za34)
     .  + za51*za53*za12*za23*zb13
     .      *zL1(s54,s12,renscale2)/(za54*za13*za13*za34)
     .  + za52*za12*za12*zb14*zL1(s54,s23,renscale2)/(za13*za14*za23)
     .  - za51*za51*za23*za23*zb13
     .      *zL1(s54,s23,renscale2)/(za54*za13*za13*za34)
     .  + za51*za12*za24*za24*zb14
     .      *zL1(s54,s23,renscale2)/(za14*za14*za23*za34)
     .  + za54*za23*za24*zb34*zb34
     .      *zL2(s51,s23,renscale2)/(2.d0*za14*za34)
     .  + za53*za53*za12*za12*zb13*zb13
     .      *zL2(s12,s54,renscale2)/(2.d0*za54*za13*za34)
     .  - za54*((za12)**(3))*zb14*zb14
     .      *zL2(s23,s54,renscale2)/(2.d0*za13*za14*za23)



        ztmp = -1.d0*za52**(3)/(2.d0*za51*za54*za23*za34) ! amplitude corrected to remove angular instability in s54
     .  + za52*za12*za12*zb14/(2.d0*za13*za14*za23*s54)
     .  - za12*za12*zb13*zb14/(2.d0*za14*za34*zb54*s23)
     .  - za52*za54*za12*zb34/(2.d0*za51*za14*za34*s23)
     .  - za53*zb13*zb13*zb34/(2.d0*za34*zb12*zb23*s54)
     .  - za52**(3)*zL0(s51,s54,renscale2)/(za51*za54*za23*za34)
     .  - za52*za52*za12*zL0(s23,s51,renscale2)/(2.d0*za51*za14*za23*za34)
     .  - za52*za52*za53*za12
     .      *zL0(s23,s54,renscale2)/(2.d0*za51*za54*za13*za23*za34)
     .  + za52*za12*za24*zb14*zL1(s51,s23,renscale2)/(za14*za23*za34)
     .  - za51*za51*(za24**(3))*zb14
     .      *zL1(s51,s23,renscale2)/(za54*za14*za14*za23*za34)
     .  + za52*za52*zb34*zL1(s51,s23,renscale2)/(za51*za34)
     .  + za52*za53*za12*zb13*zL1(s54,s12,renscale2)/(za54*za13*za34)
     .  + za51*za53*za12*za23*zb13
     .      *zL1(s54,s12,renscale2)/(za54*za13*za13*za34)
     .  + za52*za12*za12*zb14*zL1(s54,s23,renscale2)/(za13*za14*za23)
     .  - za51*za51*za23*za23*zb13
     .      *zL1(s54,s23,renscale2)/(za54*za13*za13*za34)
     .  + za51*za12*za24*za24*zb14
     .      *zL1(s54,s23,renscale2)/(za14*za14*za23*za34)
     .  + za54*za23*za24*zb34*zb34
     .      *zL2(s51,s23,renscale2)/(2.d0*za14*za34)
     .  - za54*((za12)**(3))*zb14*zb14
     .      *zL2(s23,s54,renscale2)/(2.d0*za13*za14*za23)

     .  +za53*za12*zb13/(2.d0*za54*za13*za34)*(-za53*za12*zb13*zL2(s54,s12,renscale2)
     .-zb41/zb45/zb21)

        ze0=ztmp

        ze1 = 0d0
        ze2 = 0d0

      elseif(ihel.eq.2)then

        ztree=za13*za53**(3)/(za12*za23*za34*za45*za51)

        ze0 = za52*za34*zb14*zb24**(2)/(2.d0*za24*zb34*s51*s23)
     .  + za53*za53*za13*zb14/(2.d0*za54*za12*za23*s51)
     .  - za13*za13*zb14*zb14/(2.d0*za12*za14*za23*zb51*zb34)
     .  + za13*zb52*zb14*zb14*zb24/(2.d0*za14*zb51*zb54*zb34*s23)
     .  - za52*zb24*zb24*s54/(2.d0*za12*za24*zb23*zb34*s51)
     .  + za53*za23*zb24*zL1(s51,s23,renscale2)/(za12*za24)
     .  +  za53*za23*zb24*zL1(s51,s34,renscale2)/(za12*za24)
     .  + za52*za23*za34*zb24*zL1(s51,s23,renscale2)/(za12*za24*za24)
     .  + za52*za23*za34*zb24*zL1(s51,s34,renscale2)/(za12*za24*za24)
     .  + za53*za13*za13*zb14*zL1(s54,s23,renscale2)/(za12*za14*za23)
     .  + za51*za13*za13*za34*zb14
     .      *zL1(s54,s23,renscale2)/(za12*za14*za14*za23)
     .  - za54*za23*za23*zb24*zb24*zL2(s51,s23,renscale2)/(2.d0*za12*za24)
     .  - za51*za51*za13*za34*za34*zb14*zb14
     .      *zL2(s23,s51,renscale2)/(2.d0*za54*za12*za14*za23)
     .  - za54*za13*za13*za13*zb14*zb14
     .      *zL2(s23,s54,renscale2)/(2.d0*za12*za14*za23)
     .  -za52*za23*za34*zb24*zb24*zL2(s34,s51,renscale2)/(2.d0*za12*za24)
        ze1 = 0d0
        ze2 = 0d0

      elseif(ihel.eq.3)then

        ztree=za14*za54**(3)/(za12*za23*za34*za45*za51)

        ze0 = -1.d0*zb13*zb23/(2.d0*za23*zb54*zb34)
     . - za54*za24*zb23*zL1(s51,s34,renscale2)/(za12*za23)
     . + za52*za24*za34*zb23*zb23*zL2(s51,s34,renscale2)/(2.d0*za12*za23)
        ze1 = 0d0
        ze2 = 0d0

      endif

* correction factor to convert from c_gamma prefactor
       ze0=ze0 - pi**2/12d0*ze2

      if(iorder.eq.-2)then
          zB123m=ze2
      elseif(iorder.eq.-1)then
          zB123m=ze1
      elseif(iorder.eq.0)then
          zB123m=ze0
      endif
c      write(*,*)"zB123m ",iorder, ihel,iflip,zB123m
      return
      end

************************************************************************
*
* three gluon two quark helicity amplitudes zB123nf
*
*
      function zB123nf(ihel,iflip,i1,i2,i3,i4,i5,renscale2)
      use KinData_mod, only : zA5, zB5, s5
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /order/iorder
      parameter(pi=3.141592653589793238d0)


*      iflip = 1 -> A(i,j) <-> B(j,i) and line reversal
*      ihel = 1  iflip = 0 pmppm
*      ihel = 2  iflip = 0 ppmpm
*      ihel = 3  iflip = 0 pppmm
*      ihel = 1  iflip = 1 pmmpm
*      ihel = 2  iflip = 1 pmpmm
*      ihel = 3  iflip = 1 ppmmm
*      ihel = 1  iflip = 2 mppmp
*      ihel = 2  iflip = 2 mpmpp
*      ihel = 3  iflip = 2 mmppp
*      ihel = 1  iflip = 3 mpmmp
*      ihel = 2  iflip = 3 mmpmp
*      ihel = 3  iflip = 3 mmmpp

      if(iflip.eq.0.or.iflip.eq.3)then
        j1=i1
        j2=i2
        j3=i3
        j4=i4
        j5=i5
      elseif(iflip.eq.1.or.iflip.eq.2)then
        j1=i5
        j2=i4
        j3=i3
        j4=i2
        j5=i1
      endif
      if(iflip.eq.0.or.iflip.eq.2)then
        za12=zA5(j1,j2)
        za21=zA5(j2,j1)
        za13=zA5(j1,j3)
        za31=zA5(j3,j1)
        za14=zA5(j1,j4)
        za41=zA5(j4,j1)
        za15=zA5(j1,j5)
        za51=zA5(j5,j1)
        za23=zA5(j2,j3)
        za32=zA5(j3,j2)
        za24=zA5(j2,j4)
        za42=zA5(j4,j2)
        za25=zA5(j2,j5)
        za52=zA5(j5,j2)
        za34=zA5(j3,j4)
        za43=zA5(j4,j3)
        za35=zA5(j3,j5)
        za53=zA5(j5,j3)
        za45=zA5(j4,j5)
        za54=zA5(j5,j4)

        zb12=zB5(j1,j2)
        zb21=zB5(j2,j1)
        zb13=zB5(j1,j3)
        zb31=zB5(j3,j1)
        zb14=zB5(j1,j4)
        zb41=zB5(j4,j1)
        zb15=zB5(j1,j5)
        zb51=zB5(j5,j1)
        zb23=zB5(j2,j3)
        zb32=zB5(j3,j2)
        zb24=zB5(j2,j4)
        zb42=zB5(j4,j2)
        zb25=zB5(j2,j5)
        zb52=zB5(j5,j2)
        zb34=zB5(j3,j4)
        zb43=zB5(j4,j3)
        zb35=zB5(j3,j5)
        zb53=zB5(j5,j3)
        zb45=zB5(j4,j5)
        zb54=zB5(j5,j4)

      elseif(iflip.eq.1.or.iflip.eq.3)then
        za12=zB5(j1,j2)
        za21=zB5(j2,j1)
        za13=zB5(j1,j3)
        za31=zB5(j3,j1)
        za14=zB5(j1,j4)
        za41=zB5(j4,j1)
        za15=zB5(j1,j5)
        za51=zB5(j5,j1)
        za23=zB5(j2,j3)
        za32=zB5(j3,j2)
        za24=zB5(j2,j4)
        za42=zB5(j4,j2)
        za25=zB5(j2,j5)
        za52=zB5(j5,j2)
        za34=zB5(j3,j4)
        za43=zB5(j4,j3)
        za35=zB5(j3,j5)
        za53=zB5(j5,j3)
        za45=zB5(j4,j5)
        za54=zB5(j5,j4)

        zb12=zA5(j1,j2)
        zb21=zA5(j2,j1)
        zb13=zA5(j1,j3)
        zb31=zA5(j3,j1)
        zb14=zA5(j1,j4)
        zb41=zA5(j4,j1)
        zb15=zA5(j1,j5)
        zb51=zA5(j5,j1)
        zb23=zA5(j2,j3)
        zb32=zA5(j3,j2)
        zb24=zA5(j2,j4)
        zb42=zA5(j4,j2)
        zb25=zA5(j2,j5)
        zb52=zA5(j5,j2)
        zb34=zA5(j3,j4)
        zb43=zA5(j4,j3)
        zb35=zA5(j3,j5)
        zb53=zA5(j5,j3)
        zb45=zA5(j4,j5)
        zb54=zA5(j5,j4)
      endif

        s12=s5(j1,j2)
        s13=s5(j1,j3)
        s14=s5(j1,j4)
        s15=s5(j1,j5)
        s21=s5(j2,j1)
        s23=s5(j2,j3)
        s24=s5(j2,j4)
        s25=s5(j2,j5)
        s31=s5(j3,j1)
        s32=s5(j3,j2)
        s34=s5(j3,j4)
        s35=s5(j3,j5)
        s41=s5(j4,j1)
        s42=s5(j4,j2)
        s43=s5(j4,j3)
        s45=s5(j4,j5)
        s51=s5(j5,j1)
        s52=s5(j5,j2)
        s53=s5(j5,j3)
        s54=s5(j5,j4)
        zi = (0d0,1d0)
        zone = (1d0,0d0)
*                           A B               B A               A B               B A
*                          12345             54321             54321             12345
*      ihel = 1  iflip = 0 pmppm   iflip = 1 pmmpm   iflip = 2 mppmp   iflip = 3 mpmmp
*      ihel = 2  iflip = 0 ppmpm   iflip = 1 pmpmm   iflip = 2 mpmpp   iflip = 3 mmpmp
*      ihel = 3  iflip = 0 pppmm   iflip = 1 ppmmm   iflip = 2 mmppp   iflip = 3 mmmpp


*      ihel = 1  iflip = 0 pmppm   iflip = 1 pmmpm
*      ihel = 2  iflip = 0 ppmpm   iflip = 1 pmpmm
*      ihel = 3  iflip = 0 pppmm   iflip = 1 ppmmm

      if(ihel.eq.1)then

        ztree=za12*za52**(3)/(za12*za23*za34*za45*za51)

        ze0 =
     .  + zb13**2*zb14/(3.d0*za34*zb51*zb12*zb23)
     .  + za52*zb13*zb34/(3.d0*za34*zb23*s51)
     .  - za52**2*zb34*zL1(s51,s23,renscale2)/(za51*za34)
     .  - za52*za54*za23*zb34**2*zL2(s51,s23,renscale2)/(3d0*za51*za34)
     .  + 2d0*za54*za23*za24*zb14*zb34**2
     .   *zL3(s51,s23,renscale2)/(3.d0*za34)
        ze1 = 0d0
        ze2 = 0d0

      elseif(ihel.eq.2)then

        ztree=za13*za53**(3)/(za12*za23*za34*za45*za51)

        ze0Sig = (za52*za54*zb24**(3))/(3.d0    ! Signer's thesis
     .      *za51*za24*zb23*zb34*s51)
     .  + 2.d0*za52*za54*za23*za34*zzCF(s24,s23,s34)/(za51*za24**(4))
     .  -za53*za53*zzCF(s24,s23,s34)/(za51*za24*za24)
     .  + za53*za53*zb24*zL1(s51,s23,renscale2)/(za51*za24)
     .  + za53*za53*zb24*zL1(s51,s34,renscale2)/(za51*za24)
     .  - 2.d0*za52*za54*za23*za34*zb24
     .      *zL1(s51,s23,renscale2)/(za51*za24**(3))
     .  - 2.d0*za52*za54*za23*za34*zb24
     .      *zL1(s51,s34,renscale2)/(za51*za24**(3))
     .  + za52*za54*za23*za34*zb24*zb24
     .      *zL2(s23,s51,renscale2)/(za51*za24*za24)
     .  + za52*za54*za23*za34*zb24*zb24
     .      *zL2(s34,s51,renscale2)/(za51*za24*za24)
     .  + 2.d0*za54*za54*za23*za23*zb24**(3)
     .      *zL3(s51,s23,renscale2)/(3.d0*za51*za24)
     .  + 2.d0*za52*za52*za34*za34*zb24**(3)
     .      *zL3(s51,s34,renscale2)/(3.d0*za51*za24)


        ze0BDK = -(za52*za54*zb24**(4))/(3.d0    ! BDK hep-ph/9409393v3
     .      * za51*zb23*zb34*s51**2)             ! doesn't work well in s51->0 (qqb) limit
     .
     .      + 2.d0*za53*za54*za23*zb24**(3)
     .      *zL3(s51,s23,renscale2)/(3.d0*za51)
     .
     .      + 2.d0*za52*za53*za34*zb24**(3)
     .      *zL3(s51,s34,renscale2)/(3.d0*za51)
     .
     .      - za53**2*zb24**2*zLss1(s23,s51,s34,s51,renscale2)/s51**2/za51
     .
     .      + 2d0*za52*za23*za34*za54*zb24**4
     .      *zLss3(s23,s51,s34,s51,renscale2)/s51**4/za51


        ze0JC = -(za52*za54*zb24**(4))/(3.d0    ! BDK rewritten to remove 1/s51**2 and 1/s51**4
     .      * za51*zb23*zb34*s51*(s51-s23-s34)) ! still less stable than Signer
     .
     .      + 2.d0*za53*za54*za23*zb24**(3)
     .      *zL3(s51,s23,renscale2)/(3.d0*za51)
     .
     .      + 2.d0*za52*za53*za34*zb24**(3)
     .      *zL3(s51,s34,renscale2)/(3.d0*za51)
     .
     .      - za53**2*zb24**2*zLsss1(s23,s51,s34,s51,renscale2)/za51
     .
     .      + 2d0*za52*za23*za34*za54*zb24**4
     .      *zLsss3(s23,s51,s34,s51,renscale2)/za51






        ze0old =
     .  + 2.d0*za52*za54*za23*za34*zzCF(s24,s23,s34)/(za51*za24**(4))
     .  - za53*za53*zzCF(s24,s23,s34)/(za51*za24*za24)
     .  + za53*za53*zb24*zL1(s51,s23,renscale2)/(za51*za24)
     .  + za53*za53*zb24*zL1(s51,s34,renscale2)/(za51*za24)
     .  - 2.d0*za52*za54*za23*za34*zb24*zL1(s51,s23,renscale2)/(za51*za24**(3))
     .  - 2.d0*za52*za54*za23*za34*zb24*zL1(s51,s34,renscale2)/(za51*za24**(3))

     .  + (za52*za54*zb24**(3))/(3.d0*za51*za24*zb23*zb34*s51)
     .  + za52*za54*za23*za34*zb24*zb24*zL2(s23,s51,renscale2)/(za51*za24*za24)
     .  + za52*za54*za23*za34*zb24*zb24*zL2(s34,s51,renscale2)/(za51*za24*za24)
     .  + 2.d0*za54*za54*za23*za23*zb24**(3)*zL3(s51,s23,renscale2)/(3.d0*za51*za24)
     .  + 2.d0*za52*za52*za34*za34*zb24**(3)*zL3(s51,s34,renscale2)/(3.d0*za51*za24)



        ze0new =
     .  + 2.d0*za52*za54*za23*za34*zzCF(s24,s23,s34)/(za51*za24**(4))
     .  - za53*za53*zzCF(s24,s23,s34)/(za51*za24*za24)
     .  + za53*za53*zb24*zL1(s51,s23,renscale2)/(za51*za24)
     .  + za53*za53*zb24*zL1(s51,s34,renscale2)/(za51*za24)
     .  - 2.d0*za52*za54*za23*za34*zb24*zL1(s51,s23,renscale2)/(za51*za24**(3))
     .  - 2.d0*za52*za54*za23*za34*zb24*zL1(s51,s34,renscale2)/(za51*za24**(3))

     .  + zb24**2/za51/za24*(
     .  + za52*za54*za23*za34*zL1(s23,s51,renscale2)/za24/(s51-s23)
     .  + za52*za54*za23*za34*zL1(s34,s51,renscale2)/za24/(s51-s34)
     .  + 2d0/3d0*za54*za54*za23*za23*zb24*zL1(s51,s23,renscale2)/(s23-s51)**2
     .  + 2d0/3d0*za52*za52*za34*za34*zb24*zL1(s51,s34,renscale2)/(s34-s51)**2

     .  + 1d0/3d0*za54*za54*za23*za23*zb24/s23/(s23-s51)**2
     .  + 1d0/3d0*za52*za52*za34*za34*zb24/s34/(s34-s51)**2

     .  - za52*za54*za23*za34/za24/s23/(s23-s51)
     .  - za52*za54*za23*za34/za24/s34/(s34-s51)

     .  + 1d0/3d0*za54*za54*za23*za23*zb24/s23**2/(s23-s51)
     .  + 1d0/3d0*za54*za54*za23*za23*zb24/s23/(s23-s51)**2

     .  + 1d0/3d0*za52*za52*za34*za34*zb24/s34**2/(s34-s51)
     .  + 1d0/3d0*za52*za52*za34*za34*zb24/s34/(s34-s51)**2
     .  )

     .  + zb24**2/za24**2/s51/zb32**2/zb43**2*(
     .  - 1d0/3d0*za54*za24*zb24*zb43*zb13
     .  - 1d0/3d0*za52*za24*zb24*zb23*zb13
c     .  + za52*za54*zb32*zb43*zb15
     .  + za52*za54*zb32*zb43*(za24*zb42+za34*zb43+za23*zb32)/za15
     .  )

c        write(*,*)"old ",ze0old
c        write(*,*)"new ",ze0new
c        write(*,*)

        ze0=ze0new
c        ze0=ze0old

        ze1 = 0d0
        ze2 = 0d0

      elseif(ihel.eq.3)then

        ztree=za14*za54**(3)/(za12*za23*za34*za45*za51)

        ze0 =
     . + zb13*zb23*s53/(3.d0*za23*zb54*zb34*s51)
     . - za54*zb52*zb13/(3.d0*za23*zb54*s51)
     . - 2.d0*za54**2*zb23/(3.d0*za51*za23*s51)
     . - za54**2*zb23*zL1(s51,s34,renscale2)/(za51*za23)
     . + za52*za54*za34*zb23**2*zL2(s34,s51,renscale2)/(za51*za23)
     . - 2.d0*za52**2*za34**2*zb23**(3)
     .    *zL3(s51,s34,renscale2)/(3.d0*za51*za23)   ! signer

        ze0n =
     . + zb13*zb23*s53/(3.d0*za23*zb54*zb34*s51)
     . - za54*zb52*zb13/(3.d0*za23*zb54*s51)
     . - za54**2*zb23*zL1(s51,s34,renscale2)/(za51*za23)
     . + za52*za54*za34*zb23**2*zL1(s34,s51,renscale2)/(za51*za23)/(s51-s34)
     . - za52*za54*za34*zb23**2/(za51*za23)/s34/(s34-s51)
     . - 2.d0*za52**2*za34**2*zb23**(3)/(3.d0*za51*za23)*
     .zL1(s51,s34,renscale2)/(s34-s51)**2
     . - 2.d0*za52**2*za34**2*zb23**(3)/(3.d0*za51*za23)*1d0/2d0*
     .(1d0/s34/s34/(s34-s51)+1d0/s34/(s34-s51)**2)
     . - 2.d0*za52**2*za34**2*zb23**(3)/(3.d0*za51*za23)*1d0/2d0/s34/(s34-s51)**2


     . + za52*zb23**2*zb13/(s51*za23*zb43**2*3d0)
     . + 2d0*za54*zb13*zb23/(za51*za23*zb15*zb43*3d0)

        ze0=ze0n



        ze1 = 0d0
        ze2 = 0d0
      endif

c      ze0 = ze0
c     . + ztree
c     . *( zzlog(renscale2) - zzlog(-s45)
c     .       + zzlog(renscale2) - zzlog(-s12))/6d0

c     . +  ztree
c     . *( zzlog(renscale2) - zzlog(-s23)
c     .       + zzlog(renscale2) - zzlog(-s34))/3d0


* correction factor to convert from c_gamma prefactor
       ze0=ze0 - pi**2/12d0*ze2

      if(iorder.eq.-2)then
          zB123nf=ze2
      elseif(iorder.eq.-1)then
          zB123nf=ze1
      elseif(iorder.eq.0)then
          zB123nf=ze0
c          write(*,*)ihel,iflip,zB123nf
      endif



      return
      end
************************************************************************
      function zLsss1(x1,x2,y1,y2,renscale2)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      r1=x1/x2
      r2=y1/y2

      zden = dcmplx(x2-x1-y1)

      zwt = (zLssm1(r1,r2,renscale2)/zden + zL1(x1,x2,renscale2)
     .     + zL1(y1,y2,renscale2))

      zLsss1=zwt/zden

      return
      end
************************************************************************
      function zLsss3(x1,x2,y1,y2,renscale2)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      r1=x1/x2
      r2=y1/y2

      zden = dcmplx(x2-x1-y1)

      zwt = zLssm1(r1,r2,renscale2)/zden**3 + zL1(x1,x2,renscale2)/zden**2
     .     + zL1(y1,y2,renscale2)/zden**2+ zL2(x1,x2,renscale2)/2d0/zden
     .     + zL2(y1,y2,renscale2)/2d0/zden+ zL3(x1,x2,renscale2)/3d0
     .     + zL3(y1,y2,renscale2)/3d0

      zLsss3=zwt/zden

      return
      end
************************************************************************
      function zLssm1(r1,r2,renscale2)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      parameter( Pi2_6 = 1.64493406684822643646d0 )

      omr1=1d0-r1
      omr2=1d0-r2
      zeta2=dcmplx(Pi2_6)


        ztmp1 = zh2(0,1,dcmplx(omr1))
        ztmp2 = zh2(0,1,dcmplx(omr2))

      zwt = ztmp1 + ztmp2 + zzlog(r1)*zzlog(r2) -zeta2

      zLssm1 = zwt

      return
      end
************************************************************************
      function zLss0(r1,r2,renscale2)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      zden = dcmplx(1d0-r1-r2)

      zwt = zLssm1(r1,r2,renscale2)

      zLss0=zwt/zden

      return
      end
************************************************************************
      function zLss1(x1,x2,y1,y2,renscale2)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      r1=x1/x2
      r2=y1/y2

      zden = dcmplx(1d0-r1-r2)

      zwt = (zLss0(r1,r2,renscale2) + x2*zL1(x1,x2,renscale2)
     .     + y2*zL1(y1,y2,renscale2))

      zLss1=zwt/zden

      return
      end
************************************************************************
      function zLss2(x1,x2,y1,y2,renscale2)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      r1=x1/x2
      r2=y1/y2

      zden = dcmplx(1d0-r1-r2)

      zwt =  zLss1(x1,x2,y1,y2,renscale2) + 1d0/2d0*x2**2*zL2(x1,x2,renscale2)
     .     + 1d0/2d0*y2**2*zL2(y1,y2,renscale2)

      zLss2=zwt/zden

      return
      end
************************************************************************
      function zLss3(x1,x2,y1,y2,renscale2)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      r1=x1/x2
      r2=y1/y2

      zden = dcmplx(1d0-r1-r2)

      zwt =  zLss2(x1,x2,y1,y2,renscale2) + 1d0/3d0*x2**3*zL3(x1,x2,renscale2)
     .     + 1d0/3d0*y2**3*zL3(y1,y2,renscale2)-dcmplx(1d0/6d0/r1)-dcmplx(1d0/6d0/r2)

      zLss3=zwt/zden

      return
      end
************************************************************************
*
* three gluon two quark helicity amplitudes zB123s
*
*
      function zB123s(ihel,iflip,i1,i2,i3,i4,i5,renscale2)
      use KinData_mod, only : zA5, zB5, s5
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      common /order/iorder
      parameter(pi=3.141592653589793238d0)

*      iflip = 1 -> A(i,j) <-> B(j,i) and line reversal
*      ihel = 1  iflip = 0 pmppm
*      ihel = 2  iflip = 0 ppmpm
*      ihel = 3  iflip = 0 pppmm
*      ihel = 1  iflip = 1 pmmpm
*      ihel = 2  iflip = 1 pmpmm
*      ihel = 3  iflip = 1 ppmmm
*      ihel = 1  iflip = 2 mppmp
*      ihel = 2  iflip = 2 mpmpp
*      ihel = 3  iflip = 2 mmppp
*      ihel = 1  iflip = 3 mpmmp
*      ihel = 2  iflip = 3 mmpmp
*      ihel = 3  iflip = 3 mmmpp

      if(iflip.eq.0.or.iflip.eq.3)then
        j1=i1
        j2=i2
        j3=i3
        j4=i4
        j5=i5
      elseif(iflip.eq.1.or.iflip.eq.2)then
        j1=i5
        j2=i4
        j3=i3
        j4=i2
        j5=i1
      endif
      if(iflip.eq.0.or.iflip.eq.2)then
        za12=zA5(j1,j2)
        za21=zA5(j2,j1)
        za13=zA5(j1,j3)
        za31=zA5(j3,j1)
        za14=zA5(j1,j4)
        za41=zA5(j4,j1)
        za15=zA5(j1,j5)
        za51=zA5(j5,j1)
        za23=zA5(j2,j3)
        za32=zA5(j3,j2)
        za24=zA5(j2,j4)
        za42=zA5(j4,j2)
        za25=zA5(j2,j5)
        za52=zA5(j5,j2)
        za34=zA5(j3,j4)
        za43=zA5(j4,j3)
        za35=zA5(j3,j5)
        za53=zA5(j5,j3)
        za45=zA5(j4,j5)
        za54=zA5(j5,j4)

        zb12=zB5(j1,j2)
        zb21=zB5(j2,j1)
        zb13=zB5(j1,j3)
        zb31=zB5(j3,j1)
        zb14=zB5(j1,j4)
        zb41=zB5(j4,j1)
        zb15=zB5(j1,j5)
        zb51=zB5(j5,j1)
        zb23=zB5(j2,j3)
        zb32=zB5(j3,j2)
        zb24=zB5(j2,j4)
        zb42=zB5(j4,j2)
        zb25=zB5(j2,j5)
        zb52=zB5(j5,j2)
        zb34=zB5(j3,j4)
        zb43=zB5(j4,j3)
        zb35=zB5(j3,j5)
        zb53=zB5(j5,j3)
        zb45=zB5(j4,j5)
        zb54=zB5(j5,j4)
      elseif(iflip.eq.1.or.iflip.eq.3)then
        za12=zB5(j1,j2)
        za21=zB5(j2,j1)
        za13=zB5(j1,j3)
        za31=zB5(j3,j1)
        za14=zB5(j1,j4)
        za41=zB5(j4,j1)
        za15=zB5(j1,j5)
        za51=zB5(j5,j1)
        za23=zB5(j2,j3)
        za32=zB5(j3,j2)
        za24=zB5(j2,j4)
        za42=zB5(j4,j2)
        za25=zB5(j2,j5)
        za52=zB5(j5,j2)
        za34=zB5(j3,j4)
        za43=zB5(j4,j3)
        za35=zB5(j3,j5)
        za53=zB5(j5,j3)
        za45=zB5(j4,j5)
        za54=zB5(j5,j4)

        zb12=zA5(j1,j2)
        zb21=zA5(j2,j1)
        zb13=zA5(j1,j3)
        zb31=zA5(j3,j1)
        zb14=zA5(j1,j4)
        zb41=zA5(j4,j1)
        zb15=zA5(j1,j5)
        zb51=zA5(j5,j1)
        zb23=zA5(j2,j3)
        zb32=zA5(j3,j2)
        zb24=zA5(j2,j4)
        zb42=zA5(j4,j2)
        zb25=zA5(j2,j5)
        zb52=zA5(j5,j2)
        zb34=zA5(j3,j4)
        zb43=zA5(j4,j3)
        zb35=zA5(j3,j5)
        zb53=zA5(j5,j3)
        zb45=zA5(j4,j5)
        zb54=zA5(j5,j4)
      endif

        s12=s5(j1,j2)
        s13=s5(j1,j3)
        s14=s5(j1,j4)
        s15=s5(j1,j5)
        s21=s5(j2,j1)
        s23=s5(j2,j3)
        s24=s5(j2,j4)
        s25=s5(j2,j5)
        s31=s5(j3,j1)
        s32=s5(j3,j2)
        s34=s5(j3,j4)
        s35=s5(j3,j5)
        s41=s5(j4,j1)
        s42=s5(j4,j2)
        s43=s5(j4,j3)
        s45=s5(j4,j5)
        s51=s5(j5,j1)
        s52=s5(j5,j2)
        s53=s5(j5,j3)
        s54=s5(j5,j4)

        zi = (0d0,1d0)
        zone = (1d0,0d0)
*                           A B               B A               A B               B A
*                          12345             54321             54321             12345
*      ihel = 1  iflip = 0 pmppm   iflip = 1 pmmpm   iflip = 2 mppmp   iflip = 3 mpmmp
*      ihel = 2  iflip = 0 ppmpm   iflip = 1 pmpmm   iflip = 2 mpmpp   iflip = 3 mmpmp
*      ihel = 3  iflip = 0 pppmm   iflip = 1 ppmmm   iflip = 2 mmppp   iflip = 3 mmmpp


*      ihel = 1  iflip = 0 pmppm   iflip = 1 pmmpm
*      ihel = 2  iflip = 0 ppmpm   iflip = 1 pmpmm
*      ihel = 3  iflip = 0 pppmm   iflip = 1 ppmmm

      if(ihel.eq.1)then

        ztree=za12*za52**(3)/(za12*za23*za34*za45*za51)

c --- Signer (5.11)
        ze0 = -ztree
     . *( 0.5d0*(zzlog(renscale2) - zzlog(-s15))**(2)
     .  + (3.d0/2.d0)*(zzlog(renscale2)- zzlog(-s45))
     .  + (7.d0/2.d0)*zone
     .  + (za51*za42)**(3)/(za52*za41)**(3)*zzCF(s14,s15,s54)
     .  - ((za53*za12)/(za52*za13)-1d0*zone
     .      +(za53*za12)/(za52*za13)*(za51*za32)/(za52*za31))
     .  *(za51*za32)/(za52*za31)*zzCF(s13,s12,s23))

        ze1 = -ztree*( zzlog(renscale2) - zzlog(-s15) + (3d0/2d0)*zone)

        ze2 = -ztree*1.d0

      elseif(ihel.eq.2)then

        ztree=za13*za53**(3)/(za12*za23*za34*za45*za51)

c --- Signer (5.15)
        ze0 = -ztree
     .  *(0.5d0*(zzlog(renscale2) - zzlog(-s15))**(2)
     .  + (3.d0/2.d0)*(zzlog(renscale2) - zzlog(-s15))
     .  + (7.d0/2.d0)*zone)
     .  - za51**2*za13*za34**2*zb14*zL1(s51,s23,renscale2)
     .     /(za54*za12*za14**(2)*za23)
     .  - ztree*(za51*za43/za53/za41)**(3)
     .       *zzCF(s14,s15,s54)
     .  - ztree*za51*za32/(za52*za31)
     .       *zzCF(s52,s15,s12)
     .  - ztree*za51*za32/(za52*za31)*(za52*za43/za53/za42)**(3)
     .       *zzCF(s24,s23,s34)

        ze1 = -ztree*( zzlog(renscale2) - zzlog(-s15) + (3d0/2d0)*zone)

        ze2 = -ztree*1.d0

      elseif(ihel.eq.3)then

        ztree=za14*za54**(3)/(za12*za23*za34*za45*za51)
c --- Signer (5.19)
        ze0  = -ztree
     .*(  0.5d0*(zzlog(renscale2)- zzlog(-s15))**(2)
     .  + 3.d0/2.d0*(zzlog(renscale2) - zzlog(-s15))
     .  + 7.d0/2.d0*zone
     . + za51*za42*zzCF(s52,s51,s12)/(za52*za41)
     . + za51*za43*zzCF(s53,s54,s43)/(za53*za41) )

        ze1  = -ztree*(zzlog(renscale2) - zzlog(-s15) + (3d0/2d0)*zone)

        ze2  = -ztree*1.d0
      endif

* correction factor to convert from c_gamma prefactor
       ze0=ze0 - pi**2/12d0*ze2

      if(iorder.eq.-2)then
          zB123s=ze2
      elseif(iorder.eq.-1)then
          zB123s=ze1
       elseif(iorder.eq.0)then
          zB123s=ze0
      endif

*      write(*,*)"zB123s ", iorder,ihel,iflip,zB123s

      return
      end
************************************************************************
*
      function zB3g1AAd1(ihel,iflip,i1,i2,i3,i4,i5,renscale2)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /scheme/ deltaR
      common /order/iorder

*     interpretation of Signer table 5.2
*
*      iflip = 1 -> A(i,j) <-> B(j,i) and q <-> qb
*      iflip = 2 -> q <-> qb
*      iflip = 3 -> A(i,j) <-> B(j,i)
*
*      ihel = 1  iflip = 0 pmppm   12345    ! U(2,3,4) given in Signer 5.25, 5.30, 5.31, 5.32, 5.33
*      ihel = 2  iflip = 0 ppmpm   12345    ! V(2,3,4) given in Signer 5.26, 5.34, 5.35, 5.36, 5.37
*      ihel = 3  iflip = 0 ppmpm   12435    ! V(2,4,3)
*      ihel = 1  iflip = 1 pmpmm   53421    ! V(2,4,3) A <-> B q <-> qb
*      ihel = 2  iflip = 1 pmpmm   54321    ! V(2,3,4) A <-> B q <-> qb
*      ihel = 3  iflip = 1 ppmmm   54321    ! U(2,3,4) A <-> B q <-> qb

*      ihel = 1  iflip = 2 mpmpp   53421    ! pmmpm A <-> B = V(2,4,3) q <-> qb
*      ihel = 2  iflip = 2 mpmpp   54321    ! pmpmm A <-> B = V(2,3,4) q <-> qb
*      ihel = 3  iflip = 2 mmppp   54321    ! ppmmm A <-> B = U(2,3,4) q <-> qb
*      ihel = 1  iflip = 3 mpmmp   12345    ! pmppm A <-> B = U(2,3,4) A <-> B
*      ihel = 2  iflip = 3 mmpmp   12345    ! ppmpm A <-> B = V(2,3,4) A <-> B
*      ihel = 3  iflip = 3 mmpmp   12435    ! pppmm A <-> B = V(2,4,3) A <-> B

* Signer thesis eq 5.23
* tree level factors absorbed into zB1231mpp etc

      if(iflip.eq.0.or.iflip.eq.3)then
      if(ihel.eq.1)then
      zwt      =   +2d0*zBnfmpp(iflip,i1,i2,i3,i4,i5,renscale2)
     .               +zB1231mpp(iflip,i1,i2,i3,i4,i5,renscale2)
     .               +zB1321mpp(iflip,i1,i2,i3,i4,i5,renscale2)
     .               +zB2311mpp(iflip,i1,i2,i3,i4,i5,renscale2)
     .               +zB3211mpp(iflip,i1,i2,i3,i4,i5,renscale2)
      elseif(ihel.eq.2)then
      zwt      =   +2d0*zBnfpmp(iflip,i1,i2,i3,i4,i5,renscale2)
     .               +zB1231pmp(iflip,i1,i2,i3,i4,i5,renscale2)
     .               +zB1321pmp(iflip,i1,i2,i3,i4,i5,renscale2)
     .               +zB2311pmp(iflip,i1,i2,i3,i4,i5,renscale2)
     .               +zB3211pmp(iflip,i1,i2,i3,i4,i5,renscale2)
      elseif(ihel.eq.3)then
      zwt      =   +2d0*zBnfpmp(iflip,i1,i2,i4,i3,i5,renscale2)
     .               +zB1231pmp(iflip,i1,i2,i4,i3,i5,renscale2)
     .               +zB1321pmp(iflip,i1,i2,i4,i3,i5,renscale2)
     .               +zB2311pmp(iflip,i1,i2,i4,i3,i5,renscale2)
     .               +zB3211pmp(iflip,i1,i2,i4,i3,i5,renscale2)
      endif
      elseif(iflip.eq.1.or.iflip.eq.2)then
      if(ihel.eq.1)then
      zwt      =   +2d0*zBnfpmp(iflip,i1,i3,i4,i2,i5,renscale2)
     .               +zB1231pmp(iflip,i1,i3,i4,i2,i5,renscale2)
     .               +zB1321pmp(iflip,i1,i3,i4,i2,i5,renscale2)
     .               +zB2311pmp(iflip,i1,i3,i4,i2,i5,renscale2)
     .               +zB3211pmp(iflip,i1,i3,i4,i2,i5,renscale2)
      elseif(ihel.eq.2)then
      zwt      =   +2d0*zBnfpmp(iflip,i1,i4,i3,i2,i5,renscale2)
     .               +zB1231pmp(iflip,i1,i4,i3,i2,i5,renscale2)
     .               +zB1321pmp(iflip,i1,i4,i3,i2,i5,renscale2)
     .               +zB2311pmp(iflip,i1,i4,i3,i2,i5,renscale2)
     .               +zB3211pmp(iflip,i1,i4,i3,i2,i5,renscale2)
      elseif(ihel.eq.3)then
      zwt      =   +2d0*zBnfmpp(iflip,i1,i4,i3,i2,i5,renscale2)
     .               +zB1231mpp(iflip,i1,i4,i3,i2,i5,renscale2)
     .               +zB1321mpp(iflip,i1,i4,i3,i2,i5,renscale2)
     .               +zB2311mpp(iflip,i1,i4,i3,i2,i5,renscale2)
     .               +zB3211mpp(iflip,i1,i4,i3,i2,i5,renscale2)
      endif
      endif

      zB3g1AAd1  = zwt

      return
      end
************************************************************************
*
      function zB3g1AAd2(ihel,iflip,i1,i2,i3,i4,i5,renscale2)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /scheme/ deltaR
      common /order/iorder

*     interpretation of Signer table 5.2
*
*      iflip = 1 -> A(i,j) <-> B(j,i) and q <-> qb
*      iflip = 2 -> q <-> qb
*      iflip = 3 -> A(i,j) <-> B(j,i)

*

*     interpretation of Signer table 5.2
*
*      iflip = 1 -> A(i,j) <-> B(j,i) and q <-> qb
*      iflip = 2 -> q <-> qb
*      iflip = 3 -> A(i,j) <-> B(j,i)
*
*      ihel = 1  iflip = 0 ppmpm   13245    ! V(3,2,4)
*      ihel = 2  iflip = 0 pmppm   13245    ! U(3,2,4)
*      ihel = 3  iflip = 0 ppmpm   13425    ! V(3,4,2)
*      ihel = 1  iflip = 1 pmpmm   53421    ! V(3,4,2) A <-> B q <-> qb
*      ihel = 2  iflip = 1 ppmmm   54231    ! U(3,2,4) A <-> B q <-> qb
*      ihel = 3  iflip = 1 pmpmm   54231    ! V(3,2,4) A <-> B q <-> qb

*      ihel = 1  iflip = 2 mpmpp   53421    ! pmmpm A <-> B = V(3,4,2) q <-> qb
*      ihel = 2  iflip = 2 mmppp   54231    ! pmpmm A <-> B = V(3,2,4) q <-> qb
*      ihel = 3  iflip = 2 mpmpp   54231    ! ppmmm A <-> B = V(3,2,4) q <-> qb
*      ihel = 1  iflip = 3 mmpmp   13245    ! pmppm A <-> B = V(3,2,4) A <-> B
*      ihel = 2  iflip = 3 mpmmp   13245    ! ppmpm A <-> B = U(3,2,4) A <-> B
*      ihel = 3  iflip = 3 mmpmp   13425    ! pppmm A <-> B = V(3,4,2) A <-> B


      if(iflip.eq.0.or.iflip.eq.3)then
      if(ihel.eq.1)then
      zwt      =   +2d0*zBnfpmp(iflip,i1,i3,i2,i4,i5,renscale2)
     .               +zB1231pmp(iflip,i1,i3,i2,i4,i5,renscale2)
     .               +zB1321pmp(iflip,i1,i3,i2,i4,i5,renscale2)
     .               +zB2311pmp(iflip,i1,i3,i2,i4,i5,renscale2)
     .               +zB3211pmp(iflip,i1,i3,i2,i4,i5,renscale2)
      elseif(ihel.eq.2)then
      zwt      =   +2d0*zBnfmpp(iflip,i1,i3,i2,i4,i5,renscale2)
     .               +zB1231mpp(iflip,i1,i3,i2,i4,i5,renscale2)
     .               +zB1321mpp(iflip,i1,i3,i2,i4,i5,renscale2)
     .               +zB2311mpp(iflip,i1,i3,i2,i4,i5,renscale2)
     .               +zB3211mpp(iflip,i1,i3,i2,i4,i5,renscale2)
      elseif(ihel.eq.3)then
      zwt      =   +2d0*zBnfpmp(iflip,i1,i3,i4,i2,i5,renscale2)
     .               +zB1231pmp(iflip,i1,i3,i4,i2,i5,renscale2)
     .               +zB1321pmp(iflip,i1,i3,i4,i2,i5,renscale2)
     .               +zB2311pmp(iflip,i1,i3,i4,i2,i5,renscale2)
     .               +zB3211pmp(iflip,i1,i3,i4,i2,i5,renscale2)
      endif
      elseif(iflip.eq.1.or.iflip.eq.2)then
      if(ihel.eq.1)then
      zwt      =   +2d0*zBnfpmp(iflip,i1,i2,i4,i3,i5,renscale2)
     .               +zB1231pmp(iflip,i1,i2,i4,i3,i5,renscale2)
     .               +zB1321pmp(iflip,i1,i2,i4,i3,i5,renscale2)
     .               +zB2311pmp(iflip,i1,i2,i4,i3,i5,renscale2)
     .               +zB3211pmp(iflip,i1,i2,i4,i3,i5,renscale2)
      elseif(ihel.eq.2)then
      zwt      =   +2d0*zBnfmpp(iflip,i1,i4,i2,i3,i5,renscale2)
     .               +zB1231mpp(iflip,i1,i4,i2,i3,i5,renscale2)
     .               +zB1321mpp(iflip,i1,i4,i2,i3,i5,renscale2)
     .               +zB2311mpp(iflip,i1,i4,i2,i3,i5,renscale2)
     .               +zB3211mpp(iflip,i1,i4,i2,i3,i5,renscale2)
      elseif(ihel.eq.3)then
      zwt      =   +2d0*zBnfpmp(iflip,i1,i4,i2,i3,i5,renscale2)
     .               +zB1231pmp(iflip,i1,i4,i2,i3,i5,renscale2)
     .               +zB1321pmp(iflip,i1,i4,i2,i3,i5,renscale2)
     .               +zB2311pmp(iflip,i1,i4,i2,i3,i5,renscale2)
     .               +zB3211pmp(iflip,i1,i4,i2,i3,i5,renscale2)
      endif
      endif

      zB3g1AAd2  = zwt

*      write(*,*)ihel,iflip,zB3g1AAd2
      return
      end
************************************************************************
*
      function zB3g1AAd3(ihel,iflip,i1,i2,i3,i4,i5,renscale2)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /scheme/ deltaR
      common /order/iorder

*     interpretation of Signer table 5.2
*
*      iflip = 1 -> A(i,j) <-> B(j,i) and q <-> qb
*      iflip = 2 -> q <-> qb
*      iflip = 3 -> A(i,j) <-> B(j,i)
*
*      ihel = 1  iflip = 0 pmppm   14235    ! V(4,2,3)
*      ihel = 2  iflip = 0 ppmpm   14325    ! V(4,3,2)
*      ihel = 3  iflip = 0 pppmm   14325    ! U(4,3,2)
*      ihel = 1  iflip = 1 pmmpm   52341    ! U(4,3,2) A <-> B q <-> qb
*      ihel = 2  iflip = 1 pmpmm   53241    ! V(4,3,2) A <-> B q <-> qb
*      ihel = 3  iflip = 1 ppmmm   53241    ! V(4,2,3) A <-> B q <-> qb

*      ihel = 1  iflip = 2 mppmp   52341    ! pmmpm A <-> B = U(4,3,2) q <-> qb
*      ihel = 2  iflip = 2 mpmpp   53241    ! pmpmm A <-> B = V(4,3,2) q <-> qb
*      ihel = 3  iflip = 2 mmppp   53241    ! ppmmm A <-> B = V(4,2,3) q <-> qb
*      ihel = 1  iflip = 3 mpmmp   14235    ! pmppm A <-> B = V(4,2,3) A <-> B
*      ihel = 2  iflip = 3 mmpmp   14325    ! ppmpm A <-> B = V(4,3,2) A <-> B
*      ihel = 3  iflip = 3 mmmpp   14325    ! pppmm A <-> B = U(4,3,2) A <-> B

      if(iflip.eq.0.or.iflip.eq.3)then
      if(ihel.eq.1)then
      zwt      =   +2d0*zBnfpmp(iflip,i1,i4,i2,i3,i5,renscale2)
     .               +zB1231pmp(iflip,i1,i4,i2,i3,i5,renscale2)
     .               +zB1321pmp(iflip,i1,i4,i2,i3,i5,renscale2)
     .               +zB2311pmp(iflip,i1,i4,i2,i3,i5,renscale2)
     .               +zB3211pmp(iflip,i1,i4,i2,i3,i5,renscale2)
      elseif(ihel.eq.2)then
      zwt      =   +2d0*zBnfpmp(iflip,i1,i4,i3,i2,i5,renscale2)
     .               +zB1231pmp(iflip,i1,i4,i3,i2,i5,renscale2)
     .               +zB1321pmp(iflip,i1,i4,i3,i2,i5,renscale2)
     .               +zB2311pmp(iflip,i1,i4,i3,i2,i5,renscale2)
     .               +zB3211pmp(iflip,i1,i4,i3,i2,i5,renscale2)
      elseif(ihel.eq.3)then
      zwt      =   +2d0*zBnfmpp(iflip,i1,i4,i3,i2,i5,renscale2)
     .               +zB1231mpp(iflip,i1,i4,i3,i2,i5,renscale2)
     .               +zB1321mpp(iflip,i1,i4,i3,i2,i5,renscale2)
     .               +zB2311mpp(iflip,i1,i4,i3,i2,i5,renscale2)
     .               +zB3211mpp(iflip,i1,i4,i3,i2,i5,renscale2)
      endif
      elseif(iflip.eq.1.or.iflip.eq.2)then
      if(ihel.eq.1)then
      zwt      =   +2d0*zBnfmpp(iflip,i1,i2,i3,i4,i5,renscale2)
     .               +zB1231mpp(iflip,i1,i2,i3,i4,i5,renscale2)
     .               +zB1321mpp(iflip,i1,i2,i3,i4,i5,renscale2)
     .               +zB2311mpp(iflip,i1,i2,i3,i4,i5,renscale2)
     .               +zB3211mpp(iflip,i1,i2,i3,i4,i5,renscale2)
      elseif(ihel.eq.2)then
      zwt      =   +2d0*zBnfpmp(iflip,i1,i2,i3,i4,i5,renscale2)
     .               +zB1231pmp(iflip,i1,i2,i3,i4,i5,renscale2)
     .               +zB1321pmp(iflip,i1,i2,i3,i4,i5,renscale2)
     .               +zB2311pmp(iflip,i1,i2,i3,i4,i5,renscale2)
     .               +zB3211pmp(iflip,i1,i2,i3,i4,i5,renscale2)
      elseif(ihel.eq.3)then
      zwt      =   +2d0*zBnfpmp(iflip,i1,i3,i2,i4,i5,renscale2)
     .               +zB1231pmp(iflip,i1,i3,i2,i4,i5,renscale2)
     .               +zB1321pmp(iflip,i1,i3,i2,i4,i5,renscale2)
     .               +zB2311pmp(iflip,i1,i3,i2,i4,i5,renscale2)
     .               +zB3211pmp(iflip,i1,i3,i2,i4,i5,renscale2)
      endif
      endif

      zB3g1AAd3  = zwt

*      write(*,*)ihel,iflip,zB3g1AAd3
      return
      end
************************************************************************
*
      function zB3g1AAq(ihel,iflip,i1,i2,i3,i4,i5,renscale2)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /scheme/ deltaR
      common /order/iorder
* Signer thesis eq. 5.22

*     interpretation of Signer table 5.2
*
*      iflip = 1 -> A(i,j) <-> B(j,i) and q <-> qb
*      iflip = 2 -> q <-> qb
*      iflip = 3 -> A(i,j) <-> B(j,i)
*
*      ihel = 1  iflip = 0 pmppm   12345    ! W(2,3,4) given in Signer 5.25, 5.27, 5.28, 5.29
*      ihel = 2  iflip = 0 pmppm   13425    ! W(3,4,2)
*      ihel = 3  iflip = 0 pmppm   14235    ! W(4,2,3)
*      ihel = 1  iflip = 1 pmmpm   53241    ! W(4,3,2) A <-> B q <-> qb
*      ihel = 2  iflip = 1 pmmpm   52431    ! W(3,2,4) A <-> B q <-> qb
*      ihel = 3  iflip = 1 pmmpm   54321    ! W(2,4,3) A <-> B q <-> qb

*      ihel = 1  iflip = 2 mppmp   53241    ! pmmpm A <-> B = W(4,3,2) q <-> qb
*      ihel = 2  iflip = 2 mppmp   52431    ! pmpmm A <-> B = W(3,2,4) q <-> qb
*      ihel = 3  iflip = 2 mppmp   54321    ! ppmmm A <-> B = W(2,4,3) q <-> qb
*      ihel = 1  iflip = 3 mpmmp   12345    ! pmppm A <-> B = W(2,3,4) A <-> B
*      ihel = 2  iflip = 3 mpmmp   13425    ! ppmpm A <-> B = W(3,4,2) A <-> B
*      ihel = 3  iflip = 3 mpmmp   14235    ! pppmm A <-> B = W(4,2,3) A <-> B

* Signer thesis eq 5.22
* tree level factors absorbed into zB123qmpp, zB231qmpp, zB312qmpp

      if(iflip.eq.0.or.iflip.eq.3)then
      if(ihel.eq.1)then
      zwt      =   -zBnfmpp(iflip,i1,i2,i3,i4,i5,renscale2)
     .           +zB123qmpp(iflip,i1,i2,i3,i4,i5,renscale2)
     .           +zB231qmpp(iflip,i1,i2,i3,i4,i5,renscale2)
     .           +zB312qmpp(iflip,i1,i2,i3,i4,i5,renscale2)
      elseif(ihel.eq.2)then
      zwt      =   -zBnfmpp(iflip,i1,i3,i4,i2,i5,renscale2)
     .           +zB123qmpp(iflip,i1,i3,i4,i2,i5,renscale2)
     .           +zB231qmpp(iflip,i1,i3,i4,i2,i5,renscale2)
     .           +zB312qmpp(iflip,i1,i3,i4,i2,i5,renscale2)
      elseif(ihel.eq.3)then
      zwt      =   -zBnfmpp(iflip,i1,i4,i2,i3,i5,renscale2)
     .           +zB123qmpp(iflip,i1,i4,i2,i3,i5,renscale2)
     .           +zB231qmpp(iflip,i1,i4,i2,i3,i5,renscale2)
     .           +zB312qmpp(iflip,i1,i4,i2,i3,i5,renscale2)
      endif
      elseif(iflip.eq.1.or.iflip.eq.2)then
      if(ihel.eq.1)then
      zwt      =   -zBnfmpp(iflip,i1,i2,i3,i4,i5,renscale2)
     .           +zB123qmpp(iflip,i1,i2,i3,i4,i5,renscale2)
     .           +zB231qmpp(iflip,i1,i2,i3,i4,i5,renscale2)
     .           +zB312qmpp(iflip,i1,i2,i3,i4,i5,renscale2)
      elseif(ihel.eq.2)then
      zwt      =   -zBnfmpp(iflip,i1,i4,i2,i3,i5,renscale2)
     .           +zB123qmpp(iflip,i1,i4,i2,i3,i5,renscale2)
     .           +zB231qmpp(iflip,i1,i4,i2,i3,i5,renscale2)
     .           +zB312qmpp(iflip,i1,i4,i2,i3,i5,renscale2)
      elseif(ihel.eq.3)then
      zwt      =   -zBnfmpp(iflip,i1,i3,i4,i2,i5,renscale2)
     .           +zB123qmpp(iflip,i1,i3,i4,i2,i5,renscale2)
     .           +zB231qmpp(iflip,i1,i3,i4,i2,i5,renscale2)
     .           +zB312qmpp(iflip,i1,i3,i4,i2,i5,renscale2)
      endif
      endif

      zB3g1AAq = zwt
*      write(*,*)ihel,iflip,zB3g1AAq
      return
      end
************************************************************************
*
      function zBh3g1AAq(ihel,iflip,i1,i2,i3,i4,i5,renscale2)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /scheme/ deltaR
      common /order/iorder

* Signer thesis eq. 5.22

*     interpretation of Signer table 5.2
*
*      iflip = 1 -> A(i,j) <-> B(j,i) and q <-> qb
*      iflip = 2 -> q <-> qb
*      iflip = 3 -> A(i,j) <-> B(j,i)
*
*      ihel = 1  iflip = 0 pmppm   12345    ! W(2,3,4) given in Signer 5.25, 5.27, 5.28, 5.29
*      ihel = 2  iflip = 0 pmppm   13425    ! W(3,4,2)
*      ihel = 3  iflip = 0 pmppm   14235    ! W(4,2,3)
*      ihel = 1  iflip = 1 pmmpm   53241    ! W(4,3,2) A <-> B q <-> qb
*      ihel = 2  iflip = 1 pmmpm   52431    ! W(3,2,4) A <-> B q <-> qb
*      ihel = 3  iflip = 1 pmmpm   54321    ! W(2,4,3) A <-> B q <-> qb

*      ihel = 1  iflip = 2 mppmp   53241    ! pmmpm A <-> B = W(4,3,2) q <-> qb
*      ihel = 2  iflip = 2 mppmp   52431    ! pmpmm A <-> B = W(3,2,4) q <-> qb
*      ihel = 3  iflip = 2 mppmp   54321    ! ppmmm A <-> B = W(2,4,3) q <-> qb
*      ihel = 1  iflip = 3 mpmmp   12345    ! pmppm A <-> B = W(2,3,4) A <-> B
*      ihel = 2  iflip = 3 mpmmp   13425    ! ppmpm A <-> B = W(3,4,2) A <-> B
*      ihel = 3  iflip = 3 mpmmp   14235    ! pppmm A <-> B = W(4,2,3) A <-> B

* Signer thesis eq 5.22
* tree level factors absorbed into zB123qmpp, zB231qmpp, zB312qmpp

      if(iflip.eq.0.or.iflip.eq.3)then
      if(ihel.eq.1)then
      zwt      =   +zBnfmpp(iflip,i1,i2,i3,i4,i5,renscale2)
      elseif(ihel.eq.2)then
      zwt      =   +zBnfmpp(iflip,i1,i3,i4,i2,i5,renscale2)
      elseif(ihel.eq.3)then
      zwt      =   +zBnfmpp(iflip,i1,i4,i2,i3,i5,renscale2)
      endif
      elseif(iflip.eq.1.or.iflip.eq.2)then
      if(ihel.eq.1)then
      zwt      =   +zBnfmpp(iflip,i1,i2,i3,i4,i5,renscale2)
      elseif(ihel.eq.2)then
      zwt      =   +zBnfmpp(iflip,i1,i4,i2,i3,i5,renscale2)
      elseif(ihel.eq.3)then
      zwt      =   +zBnfmpp(iflip,i1,i3,i4,i2,i5,renscale2)
      endif
      endif

      zBh3g1AAq = zwt


*      write(*,*)ihel,iflip,zBh3g1AAq
      return
      end
************************************************************************
*
      function zB3g1AAqb(ihel,iflip,i1,i2,i3,i4,i5,renscale2)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /scheme/ deltaR
      common /order/iorder

* Signer thesis eq. 5.22

*     interpretation of Signer table 5.2
*
*      iflip = 1 -> A(i,j) <-> B(j,i) and q <-> qb
*      iflip = 2 -> q <-> qb
*      iflip = 3 -> A(i,j) <-> B(j,i)
*
*      ihel = 1  iflip = 0 pmppm   12435    ! W(2,4,3) given in Signer 5.25, 5.27, 5.28, 5.29
*      ihel = 2  iflip = 0 pmppm   13245    ! W(3,2,4)
*      ihel = 3  iflip = 0 pmppm   14325    ! W(4,3,2)
*      ihel = 1  iflip = 1 pmmpm   53241    ! W(4,2,3) A <-> B q <-> qb
*      ihel = 2  iflip = 1 pmmpm   52431    ! W(3,4,2) A <-> B q <-> qb
*      ihel = 3  iflip = 1 pmmpm   54321    ! W(2,3,4) A <-> B q <-> qb

*      ihel = 1  iflip = 2 mppmp   53241    ! pmmpm A <-> B = W(4,2,3) q <-> qb
*      ihel = 2  iflip = 2 mppmp   52431    ! pmpmm A <-> B = W(3,4,2) q <-> qb
*      ihel = 3  iflip = 2 mppmp   54321    ! ppmmm A <-> B = W(2,3,4) q <-> qb
*      ihel = 1  iflip = 3 mpmmp   12435    ! pmppm A <-> B = W(2,4,3) A <-> B
*      ihel = 2  iflip = 3 mpmmp   13245    ! ppmpm A <-> B = W(3,2,4) A <-> B
*      ihel = 3  iflip = 3 mpmmp   14325    ! pppmm A <-> B = W(4,3,2) A <-> B

* Signer thesis eq 5.22
* tree level factors absorbed into zB123qmpp, zB231qmpp, zB312qmpp

      if(iflip.eq.0.or.iflip.eq.3)then
      if(ihel.eq.1)then
      zwt      =   -zBnfmpp(iflip,i1,i2,i4,i3,i5,renscale2)
     .           +zB123qmpp(iflip,i1,i2,i4,i3,i5,renscale2)
     .           +zB231qmpp(iflip,i1,i2,i4,i3,i5,renscale2)
     .           +zB312qmpp(iflip,i1,i2,i4,i3,i5,renscale2)
      elseif(ihel.eq.2)then
      zwt      =   -zBnfmpp(iflip,i1,i3,i2,i4,i5,renscale2)
     .           +zB123qmpp(iflip,i1,i3,i2,i4,i5,renscale2)
     .           +zB231qmpp(iflip,i1,i3,i2,i4,i5,renscale2)
     .           +zB312qmpp(iflip,i1,i3,i2,i4,i5,renscale2)
      elseif(ihel.eq.3)then
      zwt      =   -zBnfmpp(iflip,i1,i4,i3,i2,i5,renscale2)
     .           +zB123qmpp(iflip,i1,i4,i3,i2,i5,renscale2)
     .           +zB231qmpp(iflip,i1,i4,i3,i2,i5,renscale2)
     .           +zB312qmpp(iflip,i1,i4,i3,i2,i5,renscale2)
      endif
      elseif(iflip.eq.1.or.iflip.eq.2)then
      if(ihel.eq.1)then
      zwt      =   -zBnfmpp(iflip,i1,i3,i2,i4,i5,renscale2)
     .           +zB123qmpp(iflip,i1,i3,i2,i4,i5,renscale2)
     .           +zB231qmpp(iflip,i1,i3,i2,i4,i5,renscale2)
     .           +zB312qmpp(iflip,i1,i3,i2,i4,i5,renscale2)
      elseif(ihel.eq.2)then
      zwt      =   -zBnfmpp(iflip,i1,i2,i4,i3,i5,renscale2)
     .           +zB123qmpp(iflip,i1,i2,i4,i3,i5,renscale2)
     .           +zB231qmpp(iflip,i1,i2,i4,i3,i5,renscale2)
     .           +zB312qmpp(iflip,i1,i2,i4,i3,i5,renscale2)
      elseif(ihel.eq.3)then
      zwt      =   -zBnfmpp(iflip,i1,i4,i3,i2,i5,renscale2)
     .           +zB123qmpp(iflip,i1,i4,i3,i2,i5,renscale2)
     .           +zB231qmpp(iflip,i1,i4,i3,i2,i5,renscale2)
     .           +zB312qmpp(iflip,i1,i4,i3,i2,i5,renscale2)
      endif
      endif


      zB3g1AAqb = zwt
*      write(*,*)ihel,iflip,zB3g1AAqb
      return
      end
************************************************************************
*
      function zBh3g1AAqb(ihel,iflip,i1,i2,i3,i4,i5,renscale2)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      parameter(pi=3.141592653589793238d0)
      common /scheme/ deltaR
      common /order/iorder

*     interpretation of Signer table 5.2
*
*      iflip = 1 -> A(i,j) <-> B(j,i) and q <-> qb
*      iflip = 2 -> q <-> qb
*      iflip = 3 -> A(i,j) <-> B(j,i)
*
*      ihel = 1  iflip = 0 pmppm   12435    ! W(2,4,3) given in Signer 5.25, 5.27, 5.28, 5.29
*      ihel = 2  iflip = 0 pmppm   13245    ! W(3,2,4)
*      ihel = 3  iflip = 0 pmppm   14325    ! W(4,3,2)
*      ihel = 1  iflip = 1 pmmpm   53241    ! W(4,2,3) A <-> B q <-> qb
*      ihel = 2  iflip = 1 pmmpm   52431    ! W(3,4,2) A <-> B q <-> qb
*      ihel = 3  iflip = 1 pmmpm   54321    ! W(2,3,4) A <-> B q <-> qb

*      ihel = 1  iflip = 2 mppmp   53241    ! pmmpm A <-> B = W(4,2,3) q <-> qb
*      ihel = 2  iflip = 2 mppmp   52431    ! pmpmm A <-> B = W(3,4,2) q <-> qb
*      ihel = 3  iflip = 2 mppmp   54321    ! ppmmm A <-> B = W(2,3,4) q <-> qb
*      ihel = 1  iflip = 3 mpmmp   12435    ! pmppm A <-> B = W(2,4,3) A <-> B
*      ihel = 2  iflip = 3 mpmmp   13245    ! ppmpm A <-> B = W(3,2,4) A <-> B
*      ihel = 3  iflip = 3 mpmmp   14325    ! pppmm A <-> B = W(4,3,2) A <-> B

* Signer thesis eq 5.22
* tree level factors absorbed into zB123qmpp, zB231qmpp, zB312qmpp

      if(iflip.eq.0.or.iflip.eq.3)then
      if(ihel.eq.1)then
      zwt      =   +zBnfmpp(iflip,i1,i2,i4,i3,i5,renscale2)
      elseif(ihel.eq.2)then
      zwt      =   +zBnfmpp(iflip,i1,i3,i2,i4,i5,renscale2)
      elseif(ihel.eq.3)then
      zwt      =   +zBnfmpp(iflip,i1,i4,i3,i2,i5,renscale2)
      endif
      elseif(iflip.eq.1.or.iflip.eq.2)then
      if(ihel.eq.1)then
      zwt      =   +zBnfmpp(iflip,i1,i3,i2,i4,i5,renscale2)
      elseif(ihel.eq.2)then
      zwt      =   +zBnfmpp(iflip,i1,i2,i4,i3,i5,renscale2)
      elseif(ihel.eq.3)then
      zwt      =   +zBnfmpp(iflip,i1,i4,i3,i2,i5,renscale2)
      endif
      endif


      zBh3g1AAqb = zwt

*      write(*,*)ihel,iflip,zBh3g1AAqb
      return
      end

************************************************************************
*
* three gluon two quark helicity amplitudes zBnfmpp
*
*
      function zBnfmpp(iflip,i1,i2,i3,i4,i5,renscale2)
      use KinData_mod, only : zA5, zB5, s5
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      common /scheme/ deltaR
      common /order/iorder
      parameter(pi=3.141592653589793238d0)


      if(iflip.eq.0.or.iflip.eq.3)then
        j1=i1
        j2=i2
        j3=i3
        j4=i4
        j5=i5
      elseif(iflip.eq.1.or.iflip.eq.2)then
        j1=i5
        j2=i4
        j3=i3
        j4=i2
        j5=i1
      endif
      if(iflip.eq.0.or.iflip.eq.2)then
        za12=zA5(j1,j2)
        za21=zA5(j2,j1)
        za13=zA5(j1,j3)
        za31=zA5(j3,j1)
        za14=zA5(j1,j4)
        za41=zA5(j4,j1)
        za15=zA5(j1,j5)
        za51=zA5(j5,j1)
        za23=zA5(j2,j3)
        za32=zA5(j3,j2)
        za24=zA5(j2,j4)
        za42=zA5(j4,j2)
        za25=zA5(j2,j5)
        za52=zA5(j5,j2)
        za34=zA5(j3,j4)
        za43=zA5(j4,j3)
        za35=zA5(j3,j5)
        za53=zA5(j5,j3)
        za45=zA5(j4,j5)
        za54=zA5(j5,j4)

        zb12=zB5(j1,j2)
        zb21=zB5(j2,j1)
        zb13=zB5(j1,j3)
        zb31=zB5(j3,j1)
        zb14=zB5(j1,j4)
        zb41=zB5(j4,j1)
        zb15=zB5(j1,j5)
        zb51=zB5(j5,j1)
        zb23=zB5(j2,j3)
        zb32=zB5(j3,j2)
        zb24=zB5(j2,j4)
        zb42=zB5(j4,j2)
        zb25=zB5(j2,j5)
        zb52=zB5(j5,j2)
        zb34=zB5(j3,j4)
        zb43=zB5(j4,j3)
        zb35=zB5(j3,j5)
        zb53=zB5(j5,j3)
        zb45=zB5(j4,j5)
        zb54=zB5(j5,j4)

      elseif(iflip.eq.1.or.iflip.eq.3)then
        za12=zB5(j1,j2)
        za21=zB5(j2,j1)
        za13=zB5(j1,j3)
        za31=zB5(j3,j1)
        za14=zB5(j1,j4)
        za41=zB5(j4,j1)
        za15=zB5(j1,j5)
        za51=zB5(j5,j1)
        za23=zB5(j2,j3)
        za32=zB5(j3,j2)
        za24=zB5(j2,j4)
        za42=zB5(j4,j2)
        za25=zB5(j2,j5)
        za52=zB5(j5,j2)
        za34=zB5(j3,j4)
        za43=zB5(j4,j3)
        za35=zB5(j3,j5)
        za53=zB5(j5,j3)
        za45=zB5(j4,j5)
        za54=zB5(j5,j4)

        zb12=zA5(j1,j2)
        zb21=zA5(j2,j1)
        zb13=zA5(j1,j3)
        zb31=zA5(j3,j1)
        zb14=zA5(j1,j4)
        zb41=zA5(j4,j1)
        zb15=zA5(j1,j5)
        zb51=zA5(j5,j1)
        zb23=zA5(j2,j3)
        zb32=zA5(j3,j2)
        zb24=zA5(j2,j4)
        zb42=zA5(j4,j2)
        zb25=zA5(j2,j5)
        zb52=zA5(j5,j2)
        zb34=zA5(j3,j4)
        zb43=zA5(j4,j3)
        zb35=zA5(j3,j5)
        zb53=zA5(j5,j3)
        zb45=zA5(j4,j5)
        zb54=zA5(j5,j4)
      endif
        s12=s5(j1,j2)
        s13=s5(j1,j3)
        s14=s5(j1,j4)
        s15=s5(j1,j5)
        s21=s5(j2,j1)
        s23=s5(j2,j3)
        s24=s5(j2,j4)
        s25=s5(j2,j5)
        s31=s5(j3,j1)
        s32=s5(j3,j2)
        s34=s5(j3,j4)
        s35=s5(j3,j5)
        s41=s5(j4,j1)
        s42=s5(j4,j2)
        s43=s5(j4,j3)
        s45=s5(j4,j5)
        s51=s5(j5,j1)
        s52=s5(j5,j2)
        s53=s5(j5,j3)
        s54=s5(j5,j4)

        zi = (0d0,1d0)
        zone = (1d0,0d0)


*     eq 5.25 of Signer's thesis

        ze0 =  - za52*zb13*zb34/(za34*zb23*s51)
     .         + za53*za24*zb12*zb34**2/(za34**2*zb23*zb24*s51)
     .         - 2d0*za53*za54*za23*za24*zb34/(za51*za34**3)
     .           *( zL1(s51,s23,renscale2) + zL1(s51,s24,renscale2))
     .         + 2d0*za53*za54*za23*za24/(za51*za34**4)*zzCF(s34,s32,s24)
     .         + za52**2/(za51*za34**2)*zzCF(s34,s32,s24)
     .         - za54**2*za23**2*zb34**2/(za51*za34**2)
     .           * zL2(s51,s23,renscale2)
     .         - za53**2*za24**2*zb34**2/(za51*za34**2)
     .           * zL2(s51,s24,renscale2)


        ze0 = -ze0

        ze1 = 0d0

        ze2 = 0d0


* correction factor to convert from c_gamma prefactor
       ze0=ze0 - pi**2/12d0*ze2

      if(iorder.eq.-2)then
          zwt=ze2
      elseif(iorder.eq.-1)then
          zwt=ze1
      elseif(iorder.eq.0)then
          zwt=ze0
      endif

*      write(*,*)" zBnfmpp ",zwt
      zBnfmpp = zwt
      return
      end
************************************************************************
*
* three gluon two quark helicity amplitudes zBnfpmp
*
*


      function zBnfpmp(iflip,i1,i2,i3,i4,i5,renscale2)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

*     eq 5.26 of Signer's thesis
      if (iflip.eq.0.or.iflip.eq.3) then
         zBnfpmp = zBnfmpp(iflip,i1,i3,i2,i4,i5,renscale2)
      elseif (iflip.eq.1.or.iflip.eq.2) then
         zBnfpmp = zBnfmpp(iflip,i1,i2,i4,i3,i5,renscale2)
      endif
      return
      end

************************************************************************
*
* three gluon two quark helicity amplitudes zB123qmpp
*
*
      function zB123qmpp(iflip,i1,i2,i3,i4,i5,renscale2)
      use KinData_mod, only : zA5, zB5, s5
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      common /scheme/ deltaR
      common /order/iorder
      parameter(pi=3.141592653589793238d0)


      if(iflip.eq.0.or.iflip.eq.3)then
        j1=i1
        j2=i2
        j3=i3
        j4=i4
        j5=i5
      elseif(iflip.eq.1.or.iflip.eq.2)then
        j1=i5
        j2=i4
        j3=i3
        j4=i2
        j5=i1
      endif
      if(iflip.eq.0.or.iflip.eq.2)then
        za12=zA5(j1,j2)
        za21=zA5(j2,j1)
        za13=zA5(j1,j3)
        za31=zA5(j3,j1)
        za14=zA5(j1,j4)
        za41=zA5(j4,j1)
        za15=zA5(j1,j5)
        za51=zA5(j5,j1)
        za23=zA5(j2,j3)
        za32=zA5(j3,j2)
        za24=zA5(j2,j4)
        za42=zA5(j4,j2)
        za25=zA5(j2,j5)
        za52=zA5(j5,j2)
        za34=zA5(j3,j4)
        za43=zA5(j4,j3)
        za35=zA5(j3,j5)
        za53=zA5(j5,j3)
        za45=zA5(j4,j5)
        za54=zA5(j5,j4)

        zb12=zB5(j1,j2)
        zb21=zB5(j2,j1)
        zb13=zB5(j1,j3)
        zb31=zB5(j3,j1)
        zb14=zB5(j1,j4)
        zb41=zB5(j4,j1)
        zb15=zB5(j1,j5)
        zb51=zB5(j5,j1)
        zb23=zB5(j2,j3)
        zb32=zB5(j3,j2)
        zb24=zB5(j2,j4)
        zb42=zB5(j4,j2)
        zb25=zB5(j2,j5)
        zb52=zB5(j5,j2)
        zb34=zB5(j3,j4)
        zb43=zB5(j4,j3)
        zb35=zB5(j3,j5)
        zb53=zB5(j5,j3)
        zb45=zB5(j4,j5)
        zb54=zB5(j5,j4)

      elseif(iflip.eq.1.or.iflip.eq.3)then
        za12=zB5(j1,j2)
        za21=zB5(j2,j1)
        za13=zB5(j1,j3)
        za31=zB5(j3,j1)
        za14=zB5(j1,j4)
        za41=zB5(j4,j1)
        za15=zB5(j1,j5)
        za51=zB5(j5,j1)
        za23=zB5(j2,j3)
        za32=zB5(j3,j2)
        za24=zB5(j2,j4)
        za42=zB5(j4,j2)
        za25=zB5(j2,j5)
        za52=zB5(j5,j2)
        za34=zB5(j3,j4)
        za43=zB5(j4,j3)
        za35=zB5(j3,j5)
        za53=zB5(j5,j3)
        za45=zB5(j4,j5)
        za54=zB5(j5,j4)

        zb12=zA5(j1,j2)
        zb21=zA5(j2,j1)
        zb13=zA5(j1,j3)
        zb31=zA5(j3,j1)
        zb14=zA5(j1,j4)
        zb41=zA5(j4,j1)
        zb15=zA5(j1,j5)
        zb51=zA5(j5,j1)
        zb23=zA5(j2,j3)
        zb32=zA5(j3,j2)
        zb24=zA5(j2,j4)
        zb42=zA5(j4,j2)
        zb25=zA5(j2,j5)
        zb52=zA5(j5,j2)
        zb34=zA5(j3,j4)
        zb43=zA5(j4,j3)
        zb35=zA5(j3,j5)
        zb53=zA5(j5,j3)
        zb45=zA5(j4,j5)
        zb54=zA5(j5,j4)
      endif
        s12=s5(j1,j2)
        s13=s5(j1,j3)
        s14=s5(j1,j4)
        s15=s5(j1,j5)
        s21=s5(j2,j1)
        s23=s5(j2,j3)
        s24=s5(j2,j4)
        s25=s5(j2,j5)
        s31=s5(j3,j1)
        s32=s5(j3,j2)
        s34=s5(j3,j4)
        s35=s5(j3,j5)
        s41=s5(j4,j1)
        s42=s5(j4,j2)
        s43=s5(j4,j3)
        s45=s5(j4,j5)
        s51=s5(j5,j1)
        s52=s5(j5,j2)
        s53=s5(j5,j3)
        s54=s5(j5,j4)

        zi = (0d0,1d0)
        zone = (1d0,0d0)
****************


        ztree=za12*za52**(3)/(za12*za23*za34*za45*za51)

*     eq 5.27 of Signer's thesis
*     factor of tree level included

        ze0 =- 0.5d0*(zzlog(renscale2) - zzlog(-s15))**2
     .      + 0.5d0*(zzlog(renscale2) - zzlog(-s52))**2
     .      + 0.5d0*(zzlog(renscale2) - zzlog(-s14))**2
     .      - 0.5d0*(zzlog(renscale2) - zzlog(-s24))**2
     .      - za54*za23/za53/za24*zzCF(s52,s51,s12)
     .      - za51*za32/za52/za31*zzCF(s53,s51,s13)
     .      + za54*za13/za53/za14*(zzCF(s54,s51,s14) + zzCF(s13,s14,s43))
     .      - za12*za43/za13/za42*zzCF(s53,s51,s13)
     .      + zzCF(s53,s52,s23)
     .      - za52*za43/za53/za42*zzCF(s52,s54,s42)
     .      + za54*za12/za52/za14*zzCF(s12,s15,s52)
     .      - (za51*za42/za52/za41)**2*zzCF(s14,s15,s54)
     .      + za51*za43/za53/za41*(zzCF(s51,s54,s41) + zzCF(s34,s31,s14))
     .      + za51*za42/za52/za41*zzCF(s24,s25,s54)
     .      - za54*za23/za53/za24*zzCF(s23,s24,s43)
     .      + za54*za32/za52/za34*zzCF(s34,s32,s24)
     .       *( 2d0*za53*za12/za52/za13 + za53*za14/za54/za13
     .        - za54*za32/za52/za34*za51*za43/za53/za41)

        ze1 =
     .   - (zzlog(renscale2) - zzlog(-s15))
     .   + (zzlog(renscale2) - zzlog(-s52))
     .   + (zzlog(renscale2) - zzlog(-s14))
     .   - (zzlog(renscale2) - zzlog(-s24))


        ze2 = 0d0


* correction factor to convert from c_gamma prefactor
       ze0=ze0 - pi**2/12d0*ze2

      if(iorder.eq.-2)then
          zwt=ze2
      elseif(iorder.eq.-1)then
          zwt=ze1
      elseif(iorder.eq.0)then
          zwt=ze0
      endif

*      write(*,*)" zB123qmpp ",jflip,zwt,ztree,i1,i2,i3,i4,i5
      zB123qmpp = zwt*ztree
      return
      end

************************************************************************
*
* three gluon two quark helicity amplitudes zB231qmpp
*
*
      function zB231qmpp(iflip,i1,i2,i3,i4,i5,renscale2)
      use KinData_mod, only : zA5, zB5, s5
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      common /scheme/ deltaR
      common /order/iorder
      parameter(pi=3.141592653589793238d0)


      if(iflip.eq.0.or.iflip.eq.3)then
        j1=i1
        j2=i2
        j3=i3
        j4=i4
        j5=i5
      elseif(iflip.eq.1.or.iflip.eq.2)then
        j1=i5
        j2=i4
        j3=i3
        j4=i2
        j5=i1
      endif
      if(iflip.eq.0.or.iflip.eq.2)then
        za12=zA5(j1,j2)
        za21=zA5(j2,j1)
        za13=zA5(j1,j3)
        za31=zA5(j3,j1)
        za14=zA5(j1,j4)
        za41=zA5(j4,j1)
        za15=zA5(j1,j5)
        za51=zA5(j5,j1)
        za23=zA5(j2,j3)
        za32=zA5(j3,j2)
        za24=zA5(j2,j4)
        za42=zA5(j4,j2)
        za25=zA5(j2,j5)
        za52=zA5(j5,j2)
        za34=zA5(j3,j4)
        za43=zA5(j4,j3)
        za35=zA5(j3,j5)
        za53=zA5(j5,j3)
        za45=zA5(j4,j5)
        za54=zA5(j5,j4)

        zb12=zB5(j1,j2)
        zb21=zB5(j2,j1)
        zb13=zB5(j1,j3)
        zb31=zB5(j3,j1)
        zb14=zB5(j1,j4)
        zb41=zB5(j4,j1)
        zb15=zB5(j1,j5)
        zb51=zB5(j5,j1)
        zb23=zB5(j2,j3)
        zb32=zB5(j3,j2)
        zb24=zB5(j2,j4)
        zb42=zB5(j4,j2)
        zb25=zB5(j2,j5)
        zb52=zB5(j5,j2)
        zb34=zB5(j3,j4)
        zb43=zB5(j4,j3)
        zb35=zB5(j3,j5)
        zb53=zB5(j5,j3)
        zb45=zB5(j4,j5)
        zb54=zB5(j5,j4)

      elseif(iflip.eq.1.or.iflip.eq.3)then
        za12=zB5(j1,j2)
        za21=zB5(j2,j1)
        za13=zB5(j1,j3)
        za31=zB5(j3,j1)
        za14=zB5(j1,j4)
        za41=zB5(j4,j1)
        za15=zB5(j1,j5)
        za51=zB5(j5,j1)
        za23=zB5(j2,j3)
        za32=zB5(j3,j2)
        za24=zB5(j2,j4)
        za42=zB5(j4,j2)
        za25=zB5(j2,j5)
        za52=zB5(j5,j2)
        za34=zB5(j3,j4)
        za43=zB5(j4,j3)
        za35=zB5(j3,j5)
        za53=zB5(j5,j3)
        za45=zB5(j4,j5)
        za54=zB5(j5,j4)

        zb12=zA5(j1,j2)
        zb21=zA5(j2,j1)
        zb13=zA5(j1,j3)
        zb31=zA5(j3,j1)
        zb14=zA5(j1,j4)
        zb41=zA5(j4,j1)
        zb15=zA5(j1,j5)
        zb51=zA5(j5,j1)
        zb23=zA5(j2,j3)
        zb32=zA5(j3,j2)
        zb24=zA5(j2,j4)
        zb42=zA5(j4,j2)
        zb25=zA5(j2,j5)
        zb52=zA5(j5,j2)
        zb34=zA5(j3,j4)
        zb43=zA5(j4,j3)
        zb35=zA5(j3,j5)
        zb53=zA5(j5,j3)
        zb45=zA5(j4,j5)
        zb54=zA5(j5,j4)
      endif
        s12=s5(j1,j2)
        s13=s5(j1,j3)
        s14=s5(j1,j4)
        s15=s5(j1,j5)
        s21=s5(j2,j1)
        s23=s5(j2,j3)
        s24=s5(j2,j4)
        s25=s5(j2,j5)
        s31=s5(j3,j1)
        s32=s5(j3,j2)
        s34=s5(j3,j4)
        s35=s5(j3,j5)
        s41=s5(j4,j1)
        s42=s5(j4,j2)
        s43=s5(j4,j3)
        s45=s5(j4,j5)
        s51=s5(j5,j1)
        s52=s5(j5,j2)
        s53=s5(j5,j3)
        s54=s5(j5,j4)

        zi = (0d0,1d0)
        zone = (1d0,0d0)
************

        ztree=za12*za52**(3)/(za13*za34*za42*za25*za51)

*     eq 5.28 of Signer's thesis
*     factor of tree level included

        ze0 =  - 0.5d0*(zzlog(renscale2) - zzlog(-s15))**2
     .      + 0.5d0*(zzlog(renscale2) - zzlog(-s35))**2
     .      + 0.5d0*(zzlog(renscale2) - zzlog(-s12))**2
     .      - 0.5d0*(zzlog(renscale2) - zzlog(-s23))**2
     .      + za51*za43/za53/za41*za13*za42/za12/za43
     .       *(zzCF(s54,s52,s24) - zzCF(s51,s52,s21))
     .      - (za51*za42/za52/za41)**2
     .       *(zzCF(s14,s15,s54) + za14*za32/za12/za34*zzCF(s34,s32,s24))
     .      + za53*za12/za52/za13*za54*za12/za52/za14*zzCF(s13,s15,s53)
     .      - za12*za34/za14/za32*zzCF(s14,s15,s54)
     .      + (za54*za12/za52/za14)**2*zzCF(s14,s12,s24)
     .      + za51*za23/za53/za21*zzCF(s23,s21,s13)

        ze1 =
     .   - (zzlog(renscale2) - zzlog(-s15))
     .   + (zzlog(renscale2) - zzlog(-s35))
     .   + (zzlog(renscale2) - zzlog(-s12))
     .   - (zzlog(renscale2) - zzlog(-s23))
        ze2 = 0d0


* correction factor to convert from c_gamma prefactor
       ze0=ze0 - pi**2/12d0*ze2

      if(iorder.eq.-2)then
          zwt=ze2
      elseif(iorder.eq.-1)then
          zwt=ze1
      elseif(iorder.eq.0)then
          zwt=ze0
      endif

*      write(*,*)" zB231qmpp ",jflip,zwt,ztree
      zB231qmpp = zwt*ztree
      return
      end

************************************************************************
*
* three gluon two quark helicity amplitudes zB312qmpp
*
*
      function zB312qmpp(iflip,i1,i2,i3,i4,i5,renscale2)
      use KinData_mod, only : zA5, zB5, s5
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      common /scheme/ deltaR
      common /order/iorder
      parameter(pi=3.141592653589793238d0)


      if(iflip.eq.0.or.iflip.eq.3)then
        j1=i1
        j2=i2
        j3=i3
        j4=i4
        j5=i5
      elseif(iflip.eq.1.or.iflip.eq.2)then
        j1=i5
        j2=i4
        j3=i3
        j4=i2
        j5=i1
      endif
      if(iflip.eq.0.or.iflip.eq.2)then
        za12=zA5(j1,j2)
        za21=zA5(j2,j1)
        za13=zA5(j1,j3)
        za31=zA5(j3,j1)
        za14=zA5(j1,j4)
        za41=zA5(j4,j1)
        za15=zA5(j1,j5)
        za51=zA5(j5,j1)
        za23=zA5(j2,j3)
        za32=zA5(j3,j2)
        za24=zA5(j2,j4)
        za42=zA5(j4,j2)
        za25=zA5(j2,j5)
        za52=zA5(j5,j2)
        za34=zA5(j3,j4)
        za43=zA5(j4,j3)
        za35=zA5(j3,j5)
        za53=zA5(j5,j3)
        za45=zA5(j4,j5)
        za54=zA5(j5,j4)

        zb12=zB5(j1,j2)
        zb21=zB5(j2,j1)
        zb13=zB5(j1,j3)
        zb31=zB5(j3,j1)
        zb14=zB5(j1,j4)
        zb41=zB5(j4,j1)
        zb15=zB5(j1,j5)
        zb51=zB5(j5,j1)
        zb23=zB5(j2,j3)
        zb32=zB5(j3,j2)
        zb24=zB5(j2,j4)
        zb42=zB5(j4,j2)
        zb25=zB5(j2,j5)
        zb52=zB5(j5,j2)
        zb34=zB5(j3,j4)
        zb43=zB5(j4,j3)
        zb35=zB5(j3,j5)
        zb53=zB5(j5,j3)
        zb45=zB5(j4,j5)
        zb54=zB5(j5,j4)

      elseif(iflip.eq.1.or.iflip.eq.3)then
        za12=zB5(j1,j2)
        za21=zB5(j2,j1)
        za13=zB5(j1,j3)
        za31=zB5(j3,j1)
        za14=zB5(j1,j4)
        za41=zB5(j4,j1)
        za15=zB5(j1,j5)
        za51=zB5(j5,j1)
        za23=zB5(j2,j3)
        za32=zB5(j3,j2)
        za24=zB5(j2,j4)
        za42=zB5(j4,j2)
        za25=zB5(j2,j5)
        za52=zB5(j5,j2)
        za34=zB5(j3,j4)
        za43=zB5(j4,j3)
        za35=zB5(j3,j5)
        za53=zB5(j5,j3)
        za45=zB5(j4,j5)
        za54=zB5(j5,j4)

        zb12=zA5(j1,j2)
        zb21=zA5(j2,j1)
        zb13=zA5(j1,j3)
        zb31=zA5(j3,j1)
        zb14=zA5(j1,j4)
        zb41=zA5(j4,j1)
        zb15=zA5(j1,j5)
        zb51=zA5(j5,j1)
        zb23=zA5(j2,j3)
        zb32=zA5(j3,j2)
        zb24=zA5(j2,j4)
        zb42=zA5(j4,j2)
        zb25=zA5(j2,j5)
        zb52=zA5(j5,j2)
        zb34=zA5(j3,j4)
        zb43=zA5(j4,j3)
        zb35=zA5(j3,j5)
        zb53=zA5(j5,j3)
        zb45=zA5(j4,j5)
        zb54=zA5(j5,j4)
      endif
        s12=s5(j1,j2)
        s13=s5(j1,j3)
        s14=s5(j1,j4)
        s15=s5(j1,j5)
        s21=s5(j2,j1)
        s23=s5(j2,j3)
        s24=s5(j2,j4)
        s25=s5(j2,j5)
        s31=s5(j3,j1)
        s32=s5(j3,j2)
        s34=s5(j3,j4)
        s35=s5(j3,j5)
        s41=s5(j4,j1)
        s42=s5(j4,j2)
        s43=s5(j4,j3)
        s45=s5(j4,j5)
        s51=s5(j5,j1)
        s52=s5(j5,j2)
        s53=s5(j5,j3)
        s54=s5(j5,j4)

        zi = (0d0,1d0)
        zone = (1d0,0d0)

**************
*       pmppm

        ztree=za12*za52**(3)/(za14*za42*za23*za35*za51)

*     eq 5.29 of Signer's thesis
*     factor of tree level included

        ze0 = - 0.5d0*(zzlog(renscale2) - zzlog(-s15))**2
     .      + 0.5d0*(zzlog(renscale2) - zzlog(-s45))**2
     .      + 0.5d0*(zzlog(renscale2) - zzlog(-s13))**2
     .      - 0.5d0*(zzlog(renscale2) - zzlog(-s34))**2
     .      + za51*za32/za52/za31*zzCF(s51,s53,s31)
     .      - za54*za32/za52/za34*zzCF(s54,s53,s34)
     .      + (za53*za12/za52/za13*za51*za32/za52/za31
     .         -(za51*za32/za52/za31)**2)*zzCF(s13,s15,s53)
     .      - 2d0*za53*za12/za52/za13*za51*za42/za52/za41
     .       *zzCF(s14,s15,s54)
     .      + (za51*za32/za52/za31)**3*za13*za42/za12/za43
     .       *zzCF(s13,s12,s23)
     .      + za53*za12/za52/za13*zzCF(s12,s13,s32)
     .      - za53*za42/za52/za43*zzCF(s24,s23,s34)
     .      + za51*za32/za52/za31*(za54*za32/za52/za34)**2
     .       *zzCF(s34,s35,s54)
     .      + za53*za14/za54/za13*zzCF(s34,s32,s24)

        ze1 =
     .   - (zzlog(renscale2) - zzlog(-s15))
     .   + (zzlog(renscale2) - zzlog(-s45))
     .   + (zzlog(renscale2) - zzlog(-s13))
     .   - (zzlog(renscale2) - zzlog(-s34))

        ze2 = 0d0

* correction factor to convert from c_gamma prefactor
       ze0=ze0 - pi**2/12d0*ze2

      if(iorder.eq.-2)then
          zwt=ze2
      elseif(iorder.eq.-1)then
          zwt=ze1
      elseif(iorder.eq.0)then
          zwt=ze0
      endif

*      write(*,*)" zB312qmpp ",jflip,zwt,ztree
      zB312qmpp = zwt*ztree
      return
      end
************************************************************************
*
* three gluon two quark helicity amplitudes zB1231mpp
*
*
      function zB1231mpp(iflip,i1,i2,i3,i4,i5,renscale2)
      use KinData_mod, only : zA5, zB5, s5
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      common /scheme/ deltaR
      common /order/iorder
      parameter(pi=3.141592653589793238d0)


      if(iflip.eq.0.or.iflip.eq.3)then
        j1=i1
        j2=i2
        j3=i3
        j4=i4
        j5=i5
      elseif(iflip.eq.1.or.iflip.eq.2)then
        j1=i5
        j2=i4
        j3=i3
        j4=i2
        j5=i1
      endif
      if(iflip.eq.0.or.iflip.eq.2)then
        za12=zA5(j1,j2)
        za21=zA5(j2,j1)
        za13=zA5(j1,j3)
        za31=zA5(j3,j1)
        za14=zA5(j1,j4)
        za41=zA5(j4,j1)
        za15=zA5(j1,j5)
        za51=zA5(j5,j1)
        za23=zA5(j2,j3)
        za32=zA5(j3,j2)
        za24=zA5(j2,j4)
        za42=zA5(j4,j2)
        za25=zA5(j2,j5)
        za52=zA5(j5,j2)
        za34=zA5(j3,j4)
        za43=zA5(j4,j3)
        za35=zA5(j3,j5)
        za53=zA5(j5,j3)
        za45=zA5(j4,j5)
        za54=zA5(j5,j4)

        zb12=zB5(j1,j2)
        zb21=zB5(j2,j1)
        zb13=zB5(j1,j3)
        zb31=zB5(j3,j1)
        zb14=zB5(j1,j4)
        zb41=zB5(j4,j1)
        zb15=zB5(j1,j5)
        zb51=zB5(j5,j1)
        zb23=zB5(j2,j3)
        zb32=zB5(j3,j2)
        zb24=zB5(j2,j4)
        zb42=zB5(j4,j2)
        zb25=zB5(j2,j5)
        zb52=zB5(j5,j2)
        zb34=zB5(j3,j4)
        zb43=zB5(j4,j3)
        zb35=zB5(j3,j5)
        zb53=zB5(j5,j3)
        zb45=zB5(j4,j5)
        zb54=zB5(j5,j4)

      elseif(iflip.eq.1.or.iflip.eq.3)then
        za12=zB5(j1,j2)
        za21=zB5(j2,j1)
        za13=zB5(j1,j3)
        za31=zB5(j3,j1)
        za14=zB5(j1,j4)
        za41=zB5(j4,j1)
        za15=zB5(j1,j5)
        za51=zB5(j5,j1)
        za23=zB5(j2,j3)
        za32=zB5(j3,j2)
        za24=zB5(j2,j4)
        za42=zB5(j4,j2)
        za25=zB5(j2,j5)
        za52=zB5(j5,j2)
        za34=zB5(j3,j4)
        za43=zB5(j4,j3)
        za35=zB5(j3,j5)
        za53=zB5(j5,j3)
        za45=zB5(j4,j5)
        za54=zB5(j5,j4)

        zb12=zA5(j1,j2)
        zb21=zA5(j2,j1)
        zb13=zA5(j1,j3)
        zb31=zA5(j3,j1)
        zb14=zA5(j1,j4)
        zb41=zA5(j4,j1)
        zb15=zA5(j1,j5)
        zb51=zA5(j5,j1)
        zb23=zA5(j2,j3)
        zb32=zA5(j3,j2)
        zb24=zA5(j2,j4)
        zb42=zA5(j4,j2)
        zb25=zA5(j2,j5)
        zb52=zA5(j5,j2)
        zb34=zA5(j3,j4)
        zb43=zA5(j4,j3)
        zb35=zA5(j3,j5)
        zb53=zA5(j5,j3)
        zb45=zA5(j4,j5)
        zb54=zA5(j5,j4)
      endif
        s12=s5(j1,j2)
        s13=s5(j1,j3)
        s14=s5(j1,j4)
        s15=s5(j1,j5)
        s21=s5(j2,j1)
        s23=s5(j2,j3)
        s24=s5(j2,j4)
        s25=s5(j2,j5)
        s31=s5(j3,j1)
        s32=s5(j3,j2)
        s34=s5(j3,j4)
        s35=s5(j3,j5)
        s41=s5(j4,j1)
        s42=s5(j4,j2)
        s43=s5(j4,j3)
        s45=s5(j4,j5)
        s51=s5(j5,j1)
        s52=s5(j5,j2)
        s53=s5(j5,j3)
        s54=s5(j5,j4)

        zi = (0d0,1d0)
        zone = (1d0,0d0)

        ztree=za12*za52**(3)/(za12*za23*za34*za45*za51)

*     eq 5.30 of Signer's thesis
*     factor of tree level included

        ze0 = - 0.5d0*(zzlog(renscale2) - zzlog(-s52))**2
     .         + 0.5d0*(zzlog(renscale2) - zzlog(-s53))**2
     .         + 0.5d0*(zzlog(renscale2) - zzlog(-s24))**2
     .         - 0.5d0*(zzlog(renscale2) - zzlog(-s34))**2
     .        - za52*za43/za53/za42
     .         *( zzCF(s52,s51,s12)-zzCF(s52,s53,s32)-zzCF(s52,s54,s42)
     .           +zzCF(s34,s31,s14)-zzCF(s34,s32,s24))
     .        - za54*za23/za53/za24
     .         *( zzCF(s54,s51,s14)-zzCF(s54,s52,s24)-zzCF(s54,s53,s34))
     .        - za54*za12/za52/za14*zzCF(s13,s14,s43)
     .        - za51*za42/za52/za41*(zzCF(s24,s25,s54)-zzCF(s24,s21,s14))
     .        + ( za54*za23/za53/za24
     .           + za53*za12/za52/za13*za54*za23/za53/za24)
     .         * zzCF(s23,s24,s43)
     .        - zzCF(s24,s23,s34)
     .        + za51*za43/za53/za41*zzCF(s34,s35,s54)
     .        - 3d0*za54*za32/za52/za34*zzCF(s34,s32,s24)

        ze1 =
     .   - (zzlog(renscale2) - zzlog(-s52))
     .   + (zzlog(renscale2) - zzlog(-s53))
     .   + (zzlog(renscale2) - zzlog(-s24))
     .   - (zzlog(renscale2) - zzlog(-s34))
        ze2 = 0d0


* correction factor to convert from c_gamma prefactor
       ze0=ze0 - pi**2/12d0*ze2

      if(iorder.eq.-2)then
          zwt=ze2
      elseif(iorder.eq.-1)then
          zwt=ze1
      elseif(iorder.eq.0)then
          zwt=ze0
      endif

*      write(*,*)" zB1231mpp ",iorder,zwt,ztree,i1,i2,i3,i4,i5

      zB1231mpp = zwt*ztree
      return
      end
************************************************************************
*
* three gluon two quark helicity amplitudes zB1321mpp
*
*
      function zB1321mpp(iflip,i1,i2,i3,i4,i5,renscale2)
      use KinData_mod, only : zA5, zB5, s5
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      common /scheme/ deltaR
      common /order/iorder
      parameter(pi=3.141592653589793238d0)


      if(iflip.eq.0.or.iflip.eq.3)then
        j1=i1
        j2=i2
        j3=i3
        j4=i4
        j5=i5
      elseif(iflip.eq.1.or.iflip.eq.2)then
        j1=i5
        j2=i4
        j3=i3
        j4=i2
        j5=i1
      endif
      if(iflip.eq.0.or.iflip.eq.2)then
        za12=zA5(j1,j2)
        za21=zA5(j2,j1)
        za13=zA5(j1,j3)
        za31=zA5(j3,j1)
        za14=zA5(j1,j4)
        za41=zA5(j4,j1)
        za15=zA5(j1,j5)
        za51=zA5(j5,j1)
        za23=zA5(j2,j3)
        za32=zA5(j3,j2)
        za24=zA5(j2,j4)
        za42=zA5(j4,j2)
        za25=zA5(j2,j5)
        za52=zA5(j5,j2)
        za34=zA5(j3,j4)
        za43=zA5(j4,j3)
        za35=zA5(j3,j5)
        za53=zA5(j5,j3)
        za45=zA5(j4,j5)
        za54=zA5(j5,j4)

        zb12=zB5(j1,j2)
        zb21=zB5(j2,j1)
        zb13=zB5(j1,j3)
        zb31=zB5(j3,j1)
        zb14=zB5(j1,j4)
        zb41=zB5(j4,j1)
        zb15=zB5(j1,j5)
        zb51=zB5(j5,j1)
        zb23=zB5(j2,j3)
        zb32=zB5(j3,j2)
        zb24=zB5(j2,j4)
        zb42=zB5(j4,j2)
        zb25=zB5(j2,j5)
        zb52=zB5(j5,j2)
        zb34=zB5(j3,j4)
        zb43=zB5(j4,j3)
        zb35=zB5(j3,j5)
        zb53=zB5(j5,j3)
        zb45=zB5(j4,j5)
        zb54=zB5(j5,j4)

      elseif(iflip.eq.1.or.iflip.eq.3)then
        za12=zB5(j1,j2)
        za21=zB5(j2,j1)
        za13=zB5(j1,j3)
        za31=zB5(j3,j1)
        za14=zB5(j1,j4)
        za41=zB5(j4,j1)
        za15=zB5(j1,j5)
        za51=zB5(j5,j1)
        za23=zB5(j2,j3)
        za32=zB5(j3,j2)
        za24=zB5(j2,j4)
        za42=zB5(j4,j2)
        za25=zB5(j2,j5)
        za52=zB5(j5,j2)
        za34=zB5(j3,j4)
        za43=zB5(j4,j3)
        za35=zB5(j3,j5)
        za53=zB5(j5,j3)
        za45=zB5(j4,j5)
        za54=zB5(j5,j4)

        zb12=zA5(j1,j2)
        zb21=zA5(j2,j1)
        zb13=zA5(j1,j3)
        zb31=zA5(j3,j1)
        zb14=zA5(j1,j4)
        zb41=zA5(j4,j1)
        zb15=zA5(j1,j5)
        zb51=zA5(j5,j1)
        zb23=zA5(j2,j3)
        zb32=zA5(j3,j2)
        zb24=zA5(j2,j4)
        zb42=zA5(j4,j2)
        zb25=zA5(j2,j5)
        zb52=zA5(j5,j2)
        zb34=zA5(j3,j4)
        zb43=zA5(j4,j3)
        zb35=zA5(j3,j5)
        zb53=zA5(j5,j3)
        zb45=zA5(j4,j5)
        zb54=zA5(j5,j4)
      endif
        s12=s5(j1,j2)
        s13=s5(j1,j3)
        s14=s5(j1,j4)
        s15=s5(j1,j5)
        s21=s5(j2,j1)
        s23=s5(j2,j3)
        s24=s5(j2,j4)
        s25=s5(j2,j5)
        s31=s5(j3,j1)
        s32=s5(j3,j2)
        s34=s5(j3,j4)
        s35=s5(j3,j5)
        s41=s5(j4,j1)
        s42=s5(j4,j2)
        s43=s5(j4,j3)
        s45=s5(j4,j5)
        s51=s5(j5,j1)
        s52=s5(j5,j2)
        s53=s5(j5,j3)
        s54=s5(j5,j4)

        zi = (0d0,1d0)
        zone = (1d0,0d0)

        ztree=za12*za52**(3)/(za12*za24*za43*za35*za51)

*     eq 5.31 of Signer's thesis
*     factor of tree level included

        ze0 =  - 0.5d0*(zzlog(renscale2) - zzlog(-s25))**2
     .         + 0.5d0*(zzlog(renscale2) - zzlog(-s45))**2
     .         + 0.5d0*(zzlog(renscale2) - zzlog(-s23))**2
     .         - 0.5d0*(zzlog(renscale2) - zzlog(-s34))**2
     .        - za53*za24/za54/za23
     .         *( zzCF(s53,s51,s13)-zzCF(s53,s52,s23)-zzCF(s53,s54,s43))
     .        - za53*za12/za52/za13*zzCF(s14,s13,s34)
     .        - za51*za32/za52/za31*(zzCF(s23,s25,s53)-zzCF(s23,s21,s13))
     .        - 3d0*za51*za42/za52/za41*za53*za42/za52/za43
     .         * zzCF(s34,s35,s54)

        ze1 =
     .   - (zzlog(renscale2) - zzlog(-s25))
     .   + (zzlog(renscale2) - zzlog(-s45))
     .   + (zzlog(renscale2) - zzlog(-s23))
     .   - (zzlog(renscale2) - zzlog(-s34))
        ze2 = 0d0


* correction factor to convert from c_gamma prefactor
       ze0=ze0 - pi**2/12d0*ze2

      if(iorder.eq.-2)then
          zwt=ze2
      elseif(iorder.eq.-1)then
          zwt=ze1
      elseif(iorder.eq.0)then
          zwt=ze0
      endif
*      write(*,*)" zB1321mpp ",iorder,zwt,ztree,i1,i2,i3,i4,i5

*      write(*,*)" zB1321mpp ",jflip,zwt,ztree

      zB1321mpp = zwt*ztree

      return
      end
************************************************************************
*
* three gluon two quark helicity amplitudes zB2311mpp
*
*
      function zB2311mpp(iflip,i1,i2,i3,i4,i5,renscale2)
      use KinData_mod, only : zA5, zB5, s5
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      common /scheme/ deltaR
      common /order/iorder
      parameter(pi=3.141592653589793238d0)


      if(iflip.eq.0.or.iflip.eq.3)then
        j1=i1
        j2=i2
        j3=i3
        j4=i4
        j5=i5
      elseif(iflip.eq.1.or.iflip.eq.2)then
        j1=i5
        j2=i4
        j3=i3
        j4=i2
        j5=i1
      endif
      if(iflip.eq.0.or.iflip.eq.2)then
        za12=zA5(j1,j2)
        za21=zA5(j2,j1)
        za13=zA5(j1,j3)
        za31=zA5(j3,j1)
        za14=zA5(j1,j4)
        za41=zA5(j4,j1)
        za15=zA5(j1,j5)
        za51=zA5(j5,j1)
        za23=zA5(j2,j3)
        za32=zA5(j3,j2)
        za24=zA5(j2,j4)
        za42=zA5(j4,j2)
        za25=zA5(j2,j5)
        za52=zA5(j5,j2)
        za34=zA5(j3,j4)
        za43=zA5(j4,j3)
        za35=zA5(j3,j5)
        za53=zA5(j5,j3)
        za45=zA5(j4,j5)
        za54=zA5(j5,j4)

        zb12=zB5(j1,j2)
        zb21=zB5(j2,j1)
        zb13=zB5(j1,j3)
        zb31=zB5(j3,j1)
        zb14=zB5(j1,j4)
        zb41=zB5(j4,j1)
        zb15=zB5(j1,j5)
        zb51=zB5(j5,j1)
        zb23=zB5(j2,j3)
        zb32=zB5(j3,j2)
        zb24=zB5(j2,j4)
        zb42=zB5(j4,j2)
        zb25=zB5(j2,j5)
        zb52=zB5(j5,j2)
        zb34=zB5(j3,j4)
        zb43=zB5(j4,j3)
        zb35=zB5(j3,j5)
        zb53=zB5(j5,j3)
        zb45=zB5(j4,j5)
        zb54=zB5(j5,j4)

      elseif(iflip.eq.1.or.iflip.eq.3)then
        za12=zB5(j1,j2)
        za21=zB5(j2,j1)
        za13=zB5(j1,j3)
        za31=zB5(j3,j1)
        za14=zB5(j1,j4)
        za41=zB5(j4,j1)
        za15=zB5(j1,j5)
        za51=zB5(j5,j1)
        za23=zB5(j2,j3)
        za32=zB5(j3,j2)
        za24=zB5(j2,j4)
        za42=zB5(j4,j2)
        za25=zB5(j2,j5)
        za52=zB5(j5,j2)
        za34=zB5(j3,j4)
        za43=zB5(j4,j3)
        za35=zB5(j3,j5)
        za53=zB5(j5,j3)
        za45=zB5(j4,j5)
        za54=zB5(j5,j4)

        zb12=zA5(j1,j2)
        zb21=zA5(j2,j1)
        zb13=zA5(j1,j3)
        zb31=zA5(j3,j1)
        zb14=zA5(j1,j4)
        zb41=zA5(j4,j1)
        zb15=zA5(j1,j5)
        zb51=zA5(j5,j1)
        zb23=zA5(j2,j3)
        zb32=zA5(j3,j2)
        zb24=zA5(j2,j4)
        zb42=zA5(j4,j2)
        zb25=zA5(j2,j5)
        zb52=zA5(j5,j2)
        zb34=zA5(j3,j4)
        zb43=zA5(j4,j3)
        zb35=zA5(j3,j5)
        zb53=zA5(j5,j3)
        zb45=zA5(j4,j5)
        zb54=zA5(j5,j4)
      endif
        s12=s5(j1,j2)
        s13=s5(j1,j3)
        s14=s5(j1,j4)
        s15=s5(j1,j5)
        s21=s5(j2,j1)
        s23=s5(j2,j3)
        s24=s5(j2,j4)
        s25=s5(j2,j5)
        s31=s5(j3,j1)
        s32=s5(j3,j2)
        s34=s5(j3,j4)
        s35=s5(j3,j5)
        s41=s5(j4,j1)
        s42=s5(j4,j2)
        s43=s5(j4,j3)
        s45=s5(j4,j5)
        s51=s5(j5,j1)
        s52=s5(j5,j2)
        s53=s5(j5,j3)
        s54=s5(j5,j4)

        zi = (0d0,1d0)
        zone = (1d0,0d0)

        ztree=za12*za52**(3)/(za13*za34*za42*za25*za51)

*     eq 5.32 of Signer's thesis
*     factor of tree level included

        ze0 =  - 0.5d0*(zzlog(renscale2) - zzlog(-s12))**2
     .         + 0.5d0*(zzlog(renscale2) - zzlog(-s14))**2
     .         + 0.5d0*(zzlog(renscale2) - zzlog(-s23))**2
     .         - 0.5d0*(zzlog(renscale2) - zzlog(-s34))**2
     .        + za12*za34/za14/za32
     .         *( zzCF(s12,s13,s32)+zzCF(s12,s14,s42)-zzCF(s12,s15,s52))
     .        + (za54*za12/za52/za14)**2
     .         *( zzCF(s14,s15,s54)-zzCF(s14,s12,s24))
     .        - (za54*za32/za52/za34)**2*za51*za43/za53/za41
     .         * zzCF(s34,s35,s54)
     .        +( za54*za32/za52/za34-za53*za42/za52/za43
     .          + (za53*za42/za52/za43)**2*za12*za34/za14/za32)
     .         * zzCF(s34,s32,s24)
        ze1 =
     .   - (zzlog(renscale2) - zzlog(-s12))
     .   + (zzlog(renscale2) - zzlog(-s14))
     .   + (zzlog(renscale2) - zzlog(-s23))
     .   - (zzlog(renscale2) - zzlog(-s34))
        ze2 = 0d0


* correction factor to convert from c_gamma prefactor
       ze0=ze0 - pi**2/12d0*ze2

      if(iorder.eq.-2)then
          zwt=ze2
      elseif(iorder.eq.-1)then
          zwt=ze1
      elseif(iorder.eq.0)then
          zwt=ze0
      endif
*      write(*,*)" zB2311mpp ",iorder,zwt,ztree,i1,i2,i3,i4,i5

*      write(*,*)" zB2311mpp ",jflip,zwt,ztree

      zB2311mpp = zwt*ztree
      return
      end
************************************************************************
*
* three gluon two quark helicity amplitudes zB3211mpp
*
*
      function zB3211mpp(iflip,i1,i2,i3,i4,i5,renscale2)
      use KinData_mod, only : zA5, zB5, s5
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      common /scheme/ deltaR
      common /order/iorder
      parameter(pi=3.141592653589793238d0)


      if(iflip.eq.0.or.iflip.eq.3)then
        j1=i1
        j2=i2
        j3=i3
        j4=i4
        j5=i5
      elseif(iflip.eq.1.or.iflip.eq.2)then
        j1=i5
        j2=i4
        j3=i3
        j4=i2
        j5=i1
      endif
      if(iflip.eq.0.or.iflip.eq.2)then
        za12=zA5(j1,j2)
        za21=zA5(j2,j1)
        za13=zA5(j1,j3)
        za31=zA5(j3,j1)
        za14=zA5(j1,j4)
        za41=zA5(j4,j1)
        za15=zA5(j1,j5)
        za51=zA5(j5,j1)
        za23=zA5(j2,j3)
        za32=zA5(j3,j2)
        za24=zA5(j2,j4)
        za42=zA5(j4,j2)
        za25=zA5(j2,j5)
        za52=zA5(j5,j2)
        za34=zA5(j3,j4)
        za43=zA5(j4,j3)
        za35=zA5(j3,j5)
        za53=zA5(j5,j3)
        za45=zA5(j4,j5)
        za54=zA5(j5,j4)

        zb12=zB5(j1,j2)
        zb21=zB5(j2,j1)
        zb13=zB5(j1,j3)
        zb31=zB5(j3,j1)
        zb14=zB5(j1,j4)
        zb41=zB5(j4,j1)
        zb15=zB5(j1,j5)
        zb51=zB5(j5,j1)
        zb23=zB5(j2,j3)
        zb32=zB5(j3,j2)
        zb24=zB5(j2,j4)
        zb42=zB5(j4,j2)
        zb25=zB5(j2,j5)
        zb52=zB5(j5,j2)
        zb34=zB5(j3,j4)
        zb43=zB5(j4,j3)
        zb35=zB5(j3,j5)
        zb53=zB5(j5,j3)
        zb45=zB5(j4,j5)
        zb54=zB5(j5,j4)

      elseif(iflip.eq.1.or.iflip.eq.3)then
        za12=zB5(j1,j2)
        za21=zB5(j2,j1)
        za13=zB5(j1,j3)
        za31=zB5(j3,j1)
        za14=zB5(j1,j4)
        za41=zB5(j4,j1)
        za15=zB5(j1,j5)
        za51=zB5(j5,j1)
        za23=zB5(j2,j3)
        za32=zB5(j3,j2)
        za24=zB5(j2,j4)
        za42=zB5(j4,j2)
        za25=zB5(j2,j5)
        za52=zB5(j5,j2)
        za34=zB5(j3,j4)
        za43=zB5(j4,j3)
        za35=zB5(j3,j5)
        za53=zB5(j5,j3)
        za45=zB5(j4,j5)
        za54=zB5(j5,j4)

        zb12=zA5(j1,j2)
        zb21=zA5(j2,j1)
        zb13=zA5(j1,j3)
        zb31=zA5(j3,j1)
        zb14=zA5(j1,j4)
        zb41=zA5(j4,j1)
        zb15=zA5(j1,j5)
        zb51=zA5(j5,j1)
        zb23=zA5(j2,j3)
        zb32=zA5(j3,j2)
        zb24=zA5(j2,j4)
        zb42=zA5(j4,j2)
        zb25=zA5(j2,j5)
        zb52=zA5(j5,j2)
        zb34=zA5(j3,j4)
        zb43=zA5(j4,j3)
        zb35=zA5(j3,j5)
        zb53=zA5(j5,j3)
        zb45=zA5(j4,j5)
        zb54=zA5(j5,j4)
      endif
        s12=s5(j1,j2)
        s13=s5(j1,j3)
        s14=s5(j1,j4)
        s15=s5(j1,j5)
        s21=s5(j2,j1)
        s23=s5(j2,j3)
        s24=s5(j2,j4)
        s25=s5(j2,j5)
        s31=s5(j3,j1)
        s32=s5(j3,j2)
        s34=s5(j3,j4)
        s35=s5(j3,j5)
        s41=s5(j4,j1)
        s42=s5(j4,j2)
        s43=s5(j4,j3)
        s45=s5(j4,j5)
        s51=s5(j5,j1)
        s52=s5(j5,j2)
        s53=s5(j5,j3)
        s54=s5(j5,j4)

        zi = (0d0,1d0)
        zone = (1d0,0d0)

        ztree=za12*za52**(3)/(za14*za43*za32*za25*za51)

*     eq 5.33 of Signer's thesis
*     factor of tree level included

        ze0 =  - 0.5d0*(zzlog(renscale2) - zzlog(-s12))**2
     .         + 0.5d0*(zzlog(renscale2) - zzlog(-s13))**2
     .         + 0.5d0*(zzlog(renscale2) - zzlog(-s24))**2
     .         - 0.5d0*(zzlog(renscale2) - zzlog(-s34))**2
     .        + (za53*za12/za52/za13)**2
     .         *( zzCF(s13,s15,s53)-zzCF(s13,s12,s23))
     .        - zzCF(s24,s23,s34)
     .        + za12*za43/za13/za42*zzCF(s34,s31,s14)

        ze1 =
     .   - (zzlog(renscale2) - zzlog(-s12))
     .   + (zzlog(renscale2) - zzlog(-s13))
     .   + (zzlog(renscale2) - zzlog(-s24))
     .   - (zzlog(renscale2) - zzlog(-s34))
        ze2 = 0d0


* correction factor to convert from c_gamma prefactor
       ze0=ze0 - pi**2/12d0*ze2

      if(iorder.eq.-2)then
          zwt=ze2
      elseif(iorder.eq.-1)then
          zwt=ze1
      elseif(iorder.eq.0)then
          zwt=ze0
      endif
*      write(*,*)" zB3211mpp ",iorder,zwt,ztree,i1,i2,i3,i4,i5

*      write(*,*)" zB3211mpp ",jflip,zwt,ztree

      zB3211mpp = zwt*ztree
      return
      end
************************************************************************
*
* three gluon two quark helicity amplitudes zB1231pmp
*
*
      function zB1231pmp(iflip,i1,i2,i3,i4,i5,renscale2)
      use KinData_mod, only : zA5, zB5, s5
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      common /scheme/ deltaR
      common /order/iorder
      parameter(pi=3.141592653589793238d0)

      if(iflip.eq.0.or.iflip.eq.3)then
        j1=i1
        j2=i2
        j3=i3
        j4=i4
        j5=i5
      elseif(iflip.eq.1.or.iflip.eq.2)then
        j1=i5
        j2=i4
        j3=i3
        j4=i2
        j5=i1
      endif
      if(iflip.eq.0.or.iflip.eq.2)then
        za12=zA5(j1,j2)
        za21=zA5(j2,j1)
        za13=zA5(j1,j3)
        za31=zA5(j3,j1)
        za14=zA5(j1,j4)
        za41=zA5(j4,j1)
        za15=zA5(j1,j5)
        za51=zA5(j5,j1)
        za23=zA5(j2,j3)
        za32=zA5(j3,j2)
        za24=zA5(j2,j4)
        za42=zA5(j4,j2)
        za25=zA5(j2,j5)
        za52=zA5(j5,j2)
        za34=zA5(j3,j4)
        za43=zA5(j4,j3)
        za35=zA5(j3,j5)
        za53=zA5(j5,j3)
        za45=zA5(j4,j5)
        za54=zA5(j5,j4)

        zb12=zB5(j1,j2)
        zb21=zB5(j2,j1)
        zb13=zB5(j1,j3)
        zb31=zB5(j3,j1)
        zb14=zB5(j1,j4)
        zb41=zB5(j4,j1)
        zb15=zB5(j1,j5)
        zb51=zB5(j5,j1)
        zb23=zB5(j2,j3)
        zb32=zB5(j3,j2)
        zb24=zB5(j2,j4)
        zb42=zB5(j4,j2)
        zb25=zB5(j2,j5)
        zb52=zB5(j5,j2)
        zb34=zB5(j3,j4)
        zb43=zB5(j4,j3)
        zb35=zB5(j3,j5)
        zb53=zB5(j5,j3)
        zb45=zB5(j4,j5)
        zb54=zB5(j5,j4)

      elseif(iflip.eq.1.or.iflip.eq.3)then
        za12=zB5(j1,j2)
        za21=zB5(j2,j1)
        za13=zB5(j1,j3)
        za31=zB5(j3,j1)
        za14=zB5(j1,j4)
        za41=zB5(j4,j1)
        za15=zB5(j1,j5)
        za51=zB5(j5,j1)
        za23=zB5(j2,j3)
        za32=zB5(j3,j2)
        za24=zB5(j2,j4)
        za42=zB5(j4,j2)
        za25=zB5(j2,j5)
        za52=zB5(j5,j2)
        za34=zB5(j3,j4)
        za43=zB5(j4,j3)
        za35=zB5(j3,j5)
        za53=zB5(j5,j3)
        za45=zB5(j4,j5)
        za54=zB5(j5,j4)

        zb12=zA5(j1,j2)
        zb21=zA5(j2,j1)
        zb13=zA5(j1,j3)
        zb31=zA5(j3,j1)
        zb14=zA5(j1,j4)
        zb41=zA5(j4,j1)
        zb15=zA5(j1,j5)
        zb51=zA5(j5,j1)
        zb23=zA5(j2,j3)
        zb32=zA5(j3,j2)
        zb24=zA5(j2,j4)
        zb42=zA5(j4,j2)
        zb25=zA5(j2,j5)
        zb52=zA5(j5,j2)
        zb34=zA5(j3,j4)
        zb43=zA5(j4,j3)
        zb35=zA5(j3,j5)
        zb53=zA5(j5,j3)
        zb45=zA5(j4,j5)
        zb54=zA5(j5,j4)
      endif
        s12=s5(j1,j2)
        s13=s5(j1,j3)
        s14=s5(j1,j4)
        s15=s5(j1,j5)
        s21=s5(j2,j1)
        s23=s5(j2,j3)
        s24=s5(j2,j4)
        s25=s5(j2,j5)
        s31=s5(j3,j1)
        s32=s5(j3,j2)
        s34=s5(j3,j4)
        s35=s5(j3,j5)
        s41=s5(j4,j1)
        s42=s5(j4,j2)
        s43=s5(j4,j3)
        s45=s5(j4,j5)
        s51=s5(j5,j1)
        s52=s5(j5,j2)
        s53=s5(j5,j3)
        s54=s5(j5,j4)

        zi = (0d0,1d0)
        zone = (1d0,0d0)


        ztree=za13*za53**(3)/(za12*za23*za34*za45*za51)

*     eq 5.34 of Signer's thesis
*     factor of tree level included

        ze0 =  - 0.5d0*(zzlog(renscale2) - zzlog(-s52))**2
     .         + 0.5d0*(zzlog(renscale2) - zzlog(-s53))**2
     .         + 0.5d0*(zzlog(renscale2) - zzlog(-s24))**2
     .         - 0.5d0*(zzlog(renscale2) - zzlog(-s34))**2
     .         - za12*za43/za13/za42*(zzCF(s52,s51,s12)-zzCF(s34,s32,s24))
     .        + za53*za12/za52/za13
     .         *( zzCF(s53,s51,s13)-zzCF(s53,s52,s23)-zzCF(s24,s23,s34))
     .        + za51*za32/za52/za31*za51*za43/za53/za41
     .         *( zzCF(s51,s53,s31)-zzCF(s54,s53,s34)-zzCF(s51,s52,s21))
     .        + za52*za43/za53/za42*zzCF(s52,s54,s42)
     .        - zzCF(s53,s54,s43)
     .        + za54*za13/za53/za14*(zzCF(s13,s15,s53)-zzCF(s13,s12,s23))
     .        - za54*za23/za53/za24*(za54*za13/za53/za14)**2
     .         * zzCF(s14,s15,s54)
     .        - za51*za32/za52/za31*(za51*za43/za53/za41)**3
     .         * zzCF(s14,s13,s34)
     .        - za51*za43/za53/za41*(za52*za43/za53/za42)**2
     .         * zzCF(s24,s25,s54)
     .        + za51*za43/za53/za41*zzCF(s34,s35,s54)

        ze1 =
     .   - (zzlog(renscale2) - zzlog(-s52))
     .   + (zzlog(renscale2) - zzlog(-s53))
     .   + (zzlog(renscale2) - zzlog(-s24))
     .   - (zzlog(renscale2) - zzlog(-s34))

        ze2 = 0d0


* correction factor to convert from c_gamma prefactor
       ze0=ze0 - pi**2/12d0*ze2

      if(iorder.eq.-2)then
          zwt=ze2
      elseif(iorder.eq.-1)then
          zwt=ze1
      elseif(iorder.eq.0)then
          zwt=ze0
      endif
*      write(*,*)" zB1231pmp ",iorder,zwt,ztree,i1,i2,i3,i4,i5

*      write(*,*)" zB1231pmp ",jflip,zwt,ztree

      zB1231pmp = zwt*ztree
      return
      end
************************************************************************
*
* three gluon two quark helicity amplitudes zB1321pmp
*
*
      function zB1321pmp(iflip,i1,i2,i3,i4,i5,renscale2)
      use KinData_mod, only : zA5, zB5, s5
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      common /scheme/ deltaR
      common /order/iorder
      parameter(pi=3.141592653589793238d0)

      if(iflip.eq.0.or.iflip.eq.3)then
        j1=i1
        j2=i2
        j3=i3
        j4=i4
        j5=i5
      elseif(iflip.eq.1.or.iflip.eq.2)then
        j1=i5
        j2=i4
        j3=i3
        j4=i2
        j5=i1
      endif
      if(iflip.eq.0.or.iflip.eq.2)then
        za12=zA5(j1,j2)
        za21=zA5(j2,j1)
        za13=zA5(j1,j3)
        za31=zA5(j3,j1)
        za14=zA5(j1,j4)
        za41=zA5(j4,j1)
        za15=zA5(j1,j5)
        za51=zA5(j5,j1)
        za23=zA5(j2,j3)
        za32=zA5(j3,j2)
        za24=zA5(j2,j4)
        za42=zA5(j4,j2)
        za25=zA5(j2,j5)
        za52=zA5(j5,j2)
        za34=zA5(j3,j4)
        za43=zA5(j4,j3)
        za35=zA5(j3,j5)
        za53=zA5(j5,j3)
        za45=zA5(j4,j5)
        za54=zA5(j5,j4)

        zb12=zB5(j1,j2)
        zb21=zB5(j2,j1)
        zb13=zB5(j1,j3)
        zb31=zB5(j3,j1)
        zb14=zB5(j1,j4)
        zb41=zB5(j4,j1)
        zb15=zB5(j1,j5)
        zb51=zB5(j5,j1)
        zb23=zB5(j2,j3)
        zb32=zB5(j3,j2)
        zb24=zB5(j2,j4)
        zb42=zB5(j4,j2)
        zb25=zB5(j2,j5)
        zb52=zB5(j5,j2)
        zb34=zB5(j3,j4)
        zb43=zB5(j4,j3)
        zb35=zB5(j3,j5)
        zb53=zB5(j5,j3)
        zb45=zB5(j4,j5)
        zb54=zB5(j5,j4)

      elseif(iflip.eq.1.or.iflip.eq.3)then
        za12=zB5(j1,j2)
        za21=zB5(j2,j1)
        za13=zB5(j1,j3)
        za31=zB5(j3,j1)
        za14=zB5(j1,j4)
        za41=zB5(j4,j1)
        za15=zB5(j1,j5)
        za51=zB5(j5,j1)
        za23=zB5(j2,j3)
        za32=zB5(j3,j2)
        za24=zB5(j2,j4)
        za42=zB5(j4,j2)
        za25=zB5(j2,j5)
        za52=zB5(j5,j2)
        za34=zB5(j3,j4)
        za43=zB5(j4,j3)
        za35=zB5(j3,j5)
        za53=zB5(j5,j3)
        za45=zB5(j4,j5)
        za54=zB5(j5,j4)

        zb12=zA5(j1,j2)
        zb21=zA5(j2,j1)
        zb13=zA5(j1,j3)
        zb31=zA5(j3,j1)
        zb14=zA5(j1,j4)
        zb41=zA5(j4,j1)
        zb15=zA5(j1,j5)
        zb51=zA5(j5,j1)
        zb23=zA5(j2,j3)
        zb32=zA5(j3,j2)
        zb24=zA5(j2,j4)
        zb42=zA5(j4,j2)
        zb25=zA5(j2,j5)
        zb52=zA5(j5,j2)
        zb34=zA5(j3,j4)
        zb43=zA5(j4,j3)
        zb35=zA5(j3,j5)
        zb53=zA5(j5,j3)
        zb45=zA5(j4,j5)
        zb54=zA5(j5,j4)
      endif
        s12=s5(j1,j2)
        s13=s5(j1,j3)
        s14=s5(j1,j4)
        s15=s5(j1,j5)
        s21=s5(j2,j1)
        s23=s5(j2,j3)
        s24=s5(j2,j4)
        s25=s5(j2,j5)
        s31=s5(j3,j1)
        s32=s5(j3,j2)
        s34=s5(j3,j4)
        s35=s5(j3,j5)
        s41=s5(j4,j1)
        s42=s5(j4,j2)
        s43=s5(j4,j3)
        s45=s5(j4,j5)
        s51=s5(j5,j1)
        s52=s5(j5,j2)
        s53=s5(j5,j3)
        s54=s5(j5,j4)

        zi = (0d0,1d0)
        zone = (1d0,0d0)


        ztree=za13*za53**(3)/(za12*za24*za43*za35*za51)

*     eq 5.35 of Signer's thesis
*     factor of tree level included

        ze0 =  - 0.5d0*(zzlog(renscale2) - zzlog(-s25))**2
     .         + 0.5d0*(zzlog(renscale2) - zzlog(-s45))**2
     .         + 0.5d0*(zzlog(renscale2) - zzlog(-s23))**2
     .         - 0.5d0*(zzlog(renscale2) - zzlog(-s34))**2
     .        + za53*za12/za52/za13*zzCF(s54,s51,s14)
     .        - za54*za12/za52/za14*zzCF(s54,s52,s24)
     .        - za53*za14/za54/za13*zzCF(s54,s53,s34)
     .        +( za54*za23/za53/za24-za52*za43/za53/za42)
     .         *zzCF(s24,s23,s34)
     .        - zzCF(s23,s24,s43)

        ze1 =
     .   - (zzlog(renscale2) - zzlog(-s25))
     .   + (zzlog(renscale2) - zzlog(-s45))
     .   + (zzlog(renscale2) - zzlog(-s23))
     .   - (zzlog(renscale2) - zzlog(-s34))

        ze2 = 0d0


* correction factor to convert from c_gamma prefactor
       ze0=ze0 - pi**2/12d0*ze2

      if(iorder.eq.-2)then
          zwt=ze2
      elseif(iorder.eq.-1)then
          zwt=ze1
      elseif(iorder.eq.0)then
          zwt=ze0
      endif
*      write(*,*)" zB1321pmp ",iorder,zwt,ztree,i1,i2,i3,i4,i5

*      write(*,*)" zB1321pmp ",jflip,zwt,ztree

      zB1321pmp = zwt*ztree
      return
      end
************************************************************************
*
* three gluon two quark helicity amplitudes zB2311pmp
*
*
      function zB2311pmp(iflip,i1,i2,i3,i4,i5,renscale2)
      use KinData_mod, only : zA5, zB5, s5
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      common /scheme/ deltaR
      common /order/iorder
      parameter(pi=3.141592653589793238d0)


      if(iflip.eq.0.or.iflip.eq.3)then
        j1=i1
        j2=i2
        j3=i3
        j4=i4
        j5=i5
      elseif(iflip.eq.1.or.iflip.eq.2)then
        j1=i5
        j2=i4
        j3=i3
        j4=i2
        j5=i1
      endif
      if(iflip.eq.0.or.iflip.eq.2)then
        za12=zA5(j1,j2)
        za21=zA5(j2,j1)
        za13=zA5(j1,j3)
        za31=zA5(j3,j1)
        za14=zA5(j1,j4)
        za41=zA5(j4,j1)
        za15=zA5(j1,j5)
        za51=zA5(j5,j1)
        za23=zA5(j2,j3)
        za32=zA5(j3,j2)
        za24=zA5(j2,j4)
        za42=zA5(j4,j2)
        za25=zA5(j2,j5)
        za52=zA5(j5,j2)
        za34=zA5(j3,j4)
        za43=zA5(j4,j3)
        za35=zA5(j3,j5)
        za53=zA5(j5,j3)
        za45=zA5(j4,j5)
        za54=zA5(j5,j4)

        zb12=zB5(j1,j2)
        zb21=zB5(j2,j1)
        zb13=zB5(j1,j3)
        zb31=zB5(j3,j1)
        zb14=zB5(j1,j4)
        zb41=zB5(j4,j1)
        zb15=zB5(j1,j5)
        zb51=zB5(j5,j1)
        zb23=zB5(j2,j3)
        zb32=zB5(j3,j2)
        zb24=zB5(j2,j4)
        zb42=zB5(j4,j2)
        zb25=zB5(j2,j5)
        zb52=zB5(j5,j2)
        zb34=zB5(j3,j4)
        zb43=zB5(j4,j3)
        zb35=zB5(j3,j5)
        zb53=zB5(j5,j3)
        zb45=zB5(j4,j5)
        zb54=zB5(j5,j4)

      elseif(iflip.eq.1.or.iflip.eq.3)then
        za12=zB5(j1,j2)
        za21=zB5(j2,j1)
        za13=zB5(j1,j3)
        za31=zB5(j3,j1)
        za14=zB5(j1,j4)
        za41=zB5(j4,j1)
        za15=zB5(j1,j5)
        za51=zB5(j5,j1)
        za23=zB5(j2,j3)
        za32=zB5(j3,j2)
        za24=zB5(j2,j4)
        za42=zB5(j4,j2)
        za25=zB5(j2,j5)
        za52=zB5(j5,j2)
        za34=zB5(j3,j4)
        za43=zB5(j4,j3)
        za35=zB5(j3,j5)
        za53=zB5(j5,j3)
        za45=zB5(j4,j5)
        za54=zB5(j5,j4)

        zb12=zA5(j1,j2)
        zb21=zA5(j2,j1)
        zb13=zA5(j1,j3)
        zb31=zA5(j3,j1)
        zb14=zA5(j1,j4)
        zb41=zA5(j4,j1)
        zb15=zA5(j1,j5)
        zb51=zA5(j5,j1)
        zb23=zA5(j2,j3)
        zb32=zA5(j3,j2)
        zb24=zA5(j2,j4)
        zb42=zA5(j4,j2)
        zb25=zA5(j2,j5)
        zb52=zA5(j5,j2)
        zb34=zA5(j3,j4)
        zb43=zA5(j4,j3)
        zb35=zA5(j3,j5)
        zb53=zA5(j5,j3)
        zb45=zA5(j4,j5)
        zb54=zA5(j5,j4)
      endif
        s12=s5(j1,j2)
        s13=s5(j1,j3)
        s14=s5(j1,j4)
        s15=s5(j1,j5)
        s21=s5(j2,j1)
        s23=s5(j2,j3)
        s24=s5(j2,j4)
        s25=s5(j2,j5)
        s31=s5(j3,j1)
        s32=s5(j3,j2)
        s34=s5(j3,j4)
        s35=s5(j3,j5)
        s41=s5(j4,j1)
        s42=s5(j4,j2)
        s43=s5(j4,j3)
        s45=s5(j4,j5)
        s51=s5(j5,j1)
        s52=s5(j5,j2)
        s53=s5(j5,j3)
        s54=s5(j5,j4)

        zi = (0d0,1d0)
        zone = (1d0,0d0)


        ztree=za13*za53**(3)/(za13*za34*za42*za25*za51)

*     eq 5.36 of Signer's thesis
*     factor of tree level included

        ze0 =  - 0.5d0*(zzlog(renscale2) - zzlog(-s12))**2
     .         + 0.5d0*(zzlog(renscale2) - zzlog(-s14))**2
     .         + 0.5d0*(zzlog(renscale2) - zzlog(-s23))**2
     .         - 0.5d0*(zzlog(renscale2) - zzlog(-s34))**2
     .        - 3d0*za52*za43/za53/za42*zzCF(s24,s23,s34)
     .        - za52*za14/za54/za12*zzCF(s14,s12,s24)
     .        + za12*za34/za14/za32*za52*za13/za53/za12
     .         * zzCF(s34,s32,s24)
     .        + za52*za34/za54/za32*zzCF(s52,s53,s32)

        ze1 =
     .   - (zzlog(renscale2) - zzlog(-s12))
     .   + (zzlog(renscale2) - zzlog(-s14))
     .   + (zzlog(renscale2) - zzlog(-s23))
     .   - (zzlog(renscale2) - zzlog(-s34))

        ze2 = 0d0


* correction factor to convert from c_gamma prefactor
       ze0=ze0 - pi**2/12d0*ze2

      if(iorder.eq.-2)then
          zwt=ze2
      elseif(iorder.eq.-1)then
          zwt=ze1
      elseif(iorder.eq.0)then
          zwt=ze0
      endif
*      write(*,*)" zB2311pmp ",iorder,zwt,ztree,i1,i2,i3,i4,i5

*      write(*,*)" zB2311pmp ",jflip,zwt,ztree

      zB2311pmp = zwt*ztree
      return
      end
************************************************************************
*
* three gluon two quark helicity amplitudes zB3211pmp
*
*
      function zB3211pmp(iflip,i1,i2,i3,i4,i5,renscale2)
      use KinData_mod, only : zA5, zB5, s5
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      common /scheme/ deltaR
      common /order/iorder
      parameter(pi=3.141592653589793238d0)


      if(iflip.eq.0.or.iflip.eq.3)then
        j1=i1
        j2=i2
        j3=i3
        j4=i4
        j5=i5
      elseif(iflip.eq.1.or.iflip.eq.2)then
        j1=i5
        j2=i4
        j3=i3
        j4=i2
        j5=i1
      endif
      if(iflip.eq.0.or.iflip.eq.2)then
        za12=zA5(j1,j2)
        za21=zA5(j2,j1)
        za13=zA5(j1,j3)
        za31=zA5(j3,j1)
        za14=zA5(j1,j4)
        za41=zA5(j4,j1)
        za15=zA5(j1,j5)
        za51=zA5(j5,j1)
        za23=zA5(j2,j3)
        za32=zA5(j3,j2)
        za24=zA5(j2,j4)
        za42=zA5(j4,j2)
        za25=zA5(j2,j5)
        za52=zA5(j5,j2)
        za34=zA5(j3,j4)
        za43=zA5(j4,j3)
        za35=zA5(j3,j5)
        za53=zA5(j5,j3)
        za45=zA5(j4,j5)
        za54=zA5(j5,j4)

        zb12=zB5(j1,j2)
        zb21=zB5(j2,j1)
        zb13=zB5(j1,j3)
        zb31=zB5(j3,j1)
        zb14=zB5(j1,j4)
        zb41=zB5(j4,j1)
        zb15=zB5(j1,j5)
        zb51=zB5(j5,j1)
        zb23=zB5(j2,j3)
        zb32=zB5(j3,j2)
        zb24=zB5(j2,j4)
        zb42=zB5(j4,j2)
        zb25=zB5(j2,j5)
        zb52=zB5(j5,j2)
        zb34=zB5(j3,j4)
        zb43=zB5(j4,j3)
        zb35=zB5(j3,j5)
        zb53=zB5(j5,j3)
        zb45=zB5(j4,j5)
        zb54=zB5(j5,j4)

      elseif(iflip.eq.1.or.iflip.eq.3)then
        za12=zB5(j1,j2)
        za21=zB5(j2,j1)
        za13=zB5(j1,j3)
        za31=zB5(j3,j1)
        za14=zB5(j1,j4)
        za41=zB5(j4,j1)
        za15=zB5(j1,j5)
        za51=zB5(j5,j1)
        za23=zB5(j2,j3)
        za32=zB5(j3,j2)
        za24=zB5(j2,j4)
        za42=zB5(j4,j2)
        za25=zB5(j2,j5)
        za52=zB5(j5,j2)
        za34=zB5(j3,j4)
        za43=zB5(j4,j3)
        za35=zB5(j3,j5)
        za53=zB5(j5,j3)
        za45=zB5(j4,j5)
        za54=zB5(j5,j4)

        zb12=zA5(j1,j2)
        zb21=zA5(j2,j1)
        zb13=zA5(j1,j3)
        zb31=zA5(j3,j1)
        zb14=zA5(j1,j4)
        zb41=zA5(j4,j1)
        zb15=zA5(j1,j5)
        zb51=zA5(j5,j1)
        zb23=zA5(j2,j3)
        zb32=zA5(j3,j2)
        zb24=zA5(j2,j4)
        zb42=zA5(j4,j2)
        zb25=zA5(j2,j5)
        zb52=zA5(j5,j2)
        zb34=zA5(j3,j4)
        zb43=zA5(j4,j3)
        zb35=zA5(j3,j5)
        zb53=zA5(j5,j3)
        zb45=zA5(j4,j5)
        zb54=zA5(j5,j4)
      endif
        s12=s5(j1,j2)
        s13=s5(j1,j3)
        s14=s5(j1,j4)
        s15=s5(j1,j5)
        s21=s5(j2,j1)
        s23=s5(j2,j3)
        s24=s5(j2,j4)
        s25=s5(j2,j5)
        s31=s5(j3,j1)
        s32=s5(j3,j2)
        s34=s5(j3,j4)
        s35=s5(j3,j5)
        s41=s5(j4,j1)
        s42=s5(j4,j2)
        s43=s5(j4,j3)
        s45=s5(j4,j5)
        s51=s5(j5,j1)
        s52=s5(j5,j2)
        s53=s5(j5,j3)
        s54=s5(j5,j4)

        zi = (0d0,1d0)
        zone = (1d0,0d0)

        ztree=za13*za53**(3)/(za14*za43*za32*za25*za51)

*     eq 5.37 of Signer's thesis
*     factor of tree level included

        ze0 =  - 0.5d0*(zzlog(renscale2) - zzlog(-s12))**2
     .         + 0.5d0*(zzlog(renscale2) - zzlog(-s13))**2
     .         + 0.5d0*(zzlog(renscale2) - zzlog(-s24))**2
     .         - 0.5d0*(zzlog(renscale2) - zzlog(-s34))**2
     .        + (za52*za13/za53/za12)**2*za52*za43/za53/za42
     .         *( zzCF(s12,s13,s32)-zzCF(s12,s15,s52))
     .        + za12*za43/za13/za42*zzCF(s12,s14,s42)
     .        + (za54*za13/za53/za14)**3*za14*za23/za13/za24
     .         * zzCF(s14,s13,s34)
     .        - zzCF(s13,s14,s43)
     .        + za14*za23/za13/za24*zzCF(s23,s24,s43)
     .        + za51*za34/za54/za31*(zzCF(s34,s31,s14)-zzCF(s24,s21,s14))
     .        - za52*za13/za53/za12*(za52*za43/za53/za42)**2
     .         * zzCF(s24,s23,s34)

        ze1 =
     .   - (zzlog(renscale2) - zzlog(-s12))
     .   + (zzlog(renscale2) - zzlog(-s13))
     .   + (zzlog(renscale2) - zzlog(-s24))
     .   - (zzlog(renscale2) - zzlog(-s34))

        ze2 = 0d0

* correction factor to convert from c_gamma prefactor
       ze0=ze0 - pi**2/12d0*ze2

      if(iorder.eq.-2)then
          zwt=ze2
      elseif(iorder.eq.-1)then
          zwt=ze1
      elseif(iorder.eq.0)then
          zwt=ze0
      endif
*      write(*,*)" zB3211pmp ",iorder,zwt,ztree,i1,i2,i3,i4,i5

*      write(*,*)" zB3211pmp ",jflip,zwt,ztree

      zB3211pmp = zwt*ztree
      return
      end


c************************************************************************
c       FUNCTIONS
c**************************************************************************
      complex*16 function ztaylor(a,b,c)
      implicit real*8(a-h,o-y)
      implicit complex*16(z)

      abc = a+b+c
      u = (a+b) / a
      v = (a+b) / b
      zone=(1d0,0d0)
      za=a*zone
      zb=b*zone
      zc=c*zone
      zu=(za+zb)/za
      zv=(za+zb)/zb
      zabc=abc*zone

      zwt1 =
     . 0.1D1 / (zabc - za) * zzlog(abc / a) + 0.1D1 / (zabc - zb)
     #* zzlog(abc / b)

      zwt2 =
     .(zzlog(v) * zb ** 2 + zzlog(u) * za ** 2 - za * zb) / za ** 2
     #/ zb ** 2 / 0.2D1

      zwt3 =
     . -(0.2D1 * zzlog(v) * za * zb ** 3 + 0.2D1 * zzlog(v) * zb ** 4
     # + 0.2D1 * zzlog(u) * za ** 4 + 0.2D1 * zzlog(u) * za ** 3 * zb - 0.
     #2D1 * za ** 3 * zb - za ** 2 * zb ** 2 - 0.2D1 * za * zb ** 3) / z
     #a ** 3 / zb ** 3 / (za + zb) / 0.3D1

      zwt4 =
     . (0.6D1 * zzlog(v) * za ** 2 * zb ** 4 + 0.12D2 * zzlog(v) * za
     # * zb ** 5 + 0.6D1 * zzlog(v) * zb ** 6 + 0.6D1 * zzlog(u) * za ** 6
     # + 0.12D2 * zzlog(u) * za ** 5 * zb + 0.6D1 * zzlog(u) * za ** 4 * z
     #b ** 2 - 0.6D1 * za ** 5 * zb - 0.9D1 * za ** 4 * zb ** 2 - 0.2D1
     #* zb ** 3 * za ** 3 - 0.9D1 * za ** 2 * zb ** 4 - 0.6D1 * za * zb
     #** 5) / zb ** 4 / za ** 4 / (za + zb) ** 2 / 0.8D1


      zwt5 =
     . -(0.12D2 * zzlog(v) * za ** 3 * zb ** 5 + 0.36D2 * zzlog(v) *
     #za ** 2 * zb ** 6 + 0.36D2 * zzlog(v) * za * zb ** 7 + 0.12D2 * zzlog
     #(v) * zb ** 8 + 0.12D2 * zzlog(u) * za ** 8 + 0.36D2 * zzlog(u) * z
     #a ** 7 * zb + 0.36D2 * zzlog(u) * za ** 6 * zb ** 2 + 0.12D2 * zzlog(
     #u) * za ** 5 * zb ** 3 - 0.12D2 * za ** 7 * zb - 0.30D2 * za ** 6
     # * zb ** 2 - 0.22D2 * za ** 5 * zb ** 3 - 0.3D1 * za ** 4 * zb **
     #4 - 0.22D2 * za ** 3 * zb ** 5 - 0.30D2 * za ** 2 * zb ** 6 - 0.12
     #D2 * za * zb ** 7) / zb ** 5 / za ** 5 / (za + zb) ** 3 / 0.15D2

c      write(*,*)"order 1",dreal(zc*zwt1)
c      write(*,*)"order 2",dreal(zc**2*zwt2),
c      write(*,*)"order 3",dreal(zc**3*zwt3)
c      write(*,*)"order 4",dreal(zc**4*zwt4)
c      write(*,*)"order 5",dreal(zc**5*zwt5)

      zwt = zc*zwt1 + zc**2*zwt2 + zc**3*zwt3 + zc**4*zwt4 + zc**5*zwt5

      ztaylor=zwt

      return
      end
**************************************************************************
      complex*16 function zL0(a,b,renscale2)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
         zL0 = zzlog(-a)-zzlog(-b)
      return
      end
**************************************************************************
*
      complex*16 function zL1(a,b,renscale2)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      rat = b/a-1d0
      if(abs(rat).gt.1d-05)then
         zL1 = 1d0/(b-a)*zL0(a,b,renscale2)
      else  ! use taylor expansion in (b-a)/a if b->a
         zL1 = -1d0/a*(1d0-rat/2d0+rat**2/3d0
     .        -rat**3/4d0+rat**4/5d0-rat**5/6d0)
      endif
      return
      end
**************************************************************************
*
      complex*16 function zL2(a,b,renscale2)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      rat = b/a-1d0
      if(abs(rat).gt.1d-05)then
         zL2  =  1d0/((b-a)**2)*zL0(a,b,renscale2)
     .      + 1d0/(b*(b-a))
      else ! use taylor expansion in (b-a)/a if b->a
         zL2 = -1d0/2d0/a**2*(1d0-4d0/3d0*rat+3d0/2d0*rat**2
     .        -8d0/5d0*rat**3+5d0/3d0*rat**4-12d0/7d0*rat**5)
      endif
      return
      end
**************************************************************************
*
      complex*16 function zL3(a,b,renscale2)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      rat = b/a-1d0
      if(abs(rat).gt.1d-5)then
      zL3 = 1d0/((b-a)**3)*zL0(a,b,renscale2)
     .    + (a+b)/(2d0*a*b*(b-a)**2)
      else ! use taylor expansion in (b-a)/a if b->a
         zL3 = 1d0/6d0/a**3*(1d0-3d0/2d0*rat+9d0/5d0*rat**2
     .        -2d0*rat**3+15d0/7d0*rat**4-9d0/4d0*rat**5)
      endif
      return
      end
**************************************************************************
*
      complex*16 function zCF(a,b,c)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      zCF =zzCF(a-b-c,b,c)

      return
      end

**************************************************************************
*
      complex*16 function zzCF(a,b,c)
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      parameter( Pi2_6 = 1.64493406684822643646d0 )
c      common/debugdel0/ddel0
c      logical ddel0

      zone = (1d0,0d0)
      aa=a+b+c
      r1=(b-aa)/b
      r2=(c-aa)/c
      r3=-c/b
      r4=-b/c
      zrr1=1d0-aa/b
      zrr2=1d0-aa/c
      del=abs(a/aa)
      del0=1d-5

      zzr1=r1*zone
      zzr2=r2*zone
      zzr3=r3*zone
      zzr4=r4*zone
      if(r1.gt.0)then
        zh2temp1 = dconjg(zh2(0,1,zzr1))
      else
        zh2temp1 = zh2(0,1,zzr1)
      endif

      if(r2.gt.0)then
        zh2temp2 = dconjg(zh2(0,1,zzr2))
      else
        zh2temp2 = zh2(0,1,zzr2)
      endif

      if(r3.gt.0)then
         zh2temp3 = dconjg(zh2(0,1,zzr3))
      else
         zh2temp3 = zh2(0,1,zzr3)
      endif

      if(r4.gt.0)then
         zh2temp4 = dconjg(zh2(0,1,zzr4))
      else
         zh2temp4 = zh2(0,1,zzr4)
      endif

      if(del.ge.del0)then

* old formula
*       zwt = -zh2temp1 - zh2temp2
*     . - 0.5d0*(zzlog(b) - zzlog(c))**2 - Pi2_6*zone

* rewritten log and pi terms into dilogs
        zwt=-zh2temp1 - zh2temp2 +zh2temp3 + zh2temp4

c        ddel0 = .false.
        else
c           write(*,*)"del0 test failed"
        zwt = ztaylor(b,c,a)
c        ddel0 = .true.
        endif

        zzCF = zwt

*     wrapper to src/process/Z version of this function,
*     the analytic continuation of which is correct
c      zzCF = zLsm1(-b,-aa,-c,-aa)

      return
      end
************************************************************************
*
      function xIqqonev2(sij,renscale2)
      implicit real*8(a-h,o-y)
      common /order/ieorder
      parameter( pi = 3.14159265358979323846d0 )

      yy = dLog(dabs(renscale2/sij))
      e2=-1d0/2d0
      e1=-3d0/4d0
      e0=+1d0/24d0*pi**2

      if(ieorder.eq.-2)then
        xIqqonev2=e2
      elseif(ieorder.eq.-1)then
        xIqqonev2=e1+yy*e2
      elseif(ieorder.eq.0)then
        xIqqonev2=e0+yy*e1+1d0/2d0*yy**2*e2
      endif
      return
      end

********************************************************************
      function xIqgonev2(sij,renscale2)
      implicit real*8(a-h,o-y)
      common /order/ieorder
      parameter( pi = 3.14159265358979323846d0 )

      yy = dLog(dabs(renscale2/sij))
      e2=-1d0/2d0
      e1=-5d0/6d0
      e0=+1d0/24d0*pi**2

      if(ieorder.eq.-2)then
        xIqgonev2=e2
      elseif(ieorder.eq.-1)then
        xIqgonev2=e1+yy*e2
      elseif(ieorder.eq.0)then
        xIqgonev2=e0+yy*e1+1d0/2d0*yy**2*e2
      endif

*      write(*,*)" xIqgone ",xIqgone

      return
      end

      function xIggonev2(sij,renscale2)
      implicit real*8(a-h,o-y)
      common /order/ieorder
      parameter( pi = 3.14159265358979323846d0 )

      yy = dLog(dabs(renscale2/sij))
      e2=-1d0/2d0
      e1=-11d0/12d0
      e0=+1d0/24d0*pi**2

      if(ieorder.eq.-2)then
        xIggonev2=e2
      elseif(ieorder.eq.-1)then
        xIggonev2=e1+yy*e2
      elseif(ieorder.eq.0)then
        xIggonev2=e0+yy*e1+1d0/2d0*yy**2*e2
      endif

*      write(*,*)" xIggone ",xIggone

      return
      end

** single ordering gluon ordering for q gt g g qb one-loop amplitude
** Replaced with full SLC colour level of FullB3g1
*      function Bt3g1(i1,i2,i3,i4,i5,renscale2)
*      implicit real*8(a-h,o-y)
*      Bt3g1=     +B3g1c(i1,i2,i3,i4,i5,renscale2)
*     .           -Bt3g1a(i1,i2,i3,i4,i5,renscale2)
*     .           +1d0/6d0*Btt3g1a(i1,i2,i3,i4,i5,renscale2)
*      return
*      end


!>  Reorder terms into colour structure

**************************************************************************
* Full LC (fac*N^0) colour level of FullB3g1
      function B3g1full(i1,i2,i3,i4,i5,renscale2)
      double precision B3g1full,B3g1a,renscale2
      integer i1,i2,i3,i4,i5

      B3g1full= (+B3g1a(i1,i2,i3,i4,i5,renscale2)
     .           +B3g1a(i1,i2,i4,i3,i5,renscale2)
     .           +B3g1a(i1,i3,i2,i4,i5,renscale2)
     .           +B3g1a(i1,i3,i4,i2,i5,renscale2)
     .           +B3g1a(i1,i4,i2,i3,i5,renscale2)
     .           +B3g1a(i1,i4,i3,i2,i5,renscale2))

      return
      end
**************************************************************************
* Full LC nf (nf/N) colour level of FullB3g1
      function Bh3g1full(i1,i2,i3,i4,i5,renscale2)
      double precision Bh3g1full,B3g1b,renscale2
      integer i1,i2,i3,i4,i5

      Bh3g1full = +B3g1b(i1,i2,i3,i4,i5,renscale2)
     .            +B3g1b(i1,i2,i4,i3,i5,renscale2)
     .            +B3g1b(i1,i3,i2,i4,i5,renscale2)
     .            +B3g1b(i1,i3,i4,i2,i5,renscale2)
     .            +B3g1b(i1,i4,i2,i3,i5,renscale2)
     .            +B3g1b(i1,i4,i3,i2,i5,renscale2)

      return
      end
**************************************************************************
* Full SLC (1/N^2) colour level of FullB3g1    
      function Bt3g1(i1,i2,i3,i4,i5,renscale2)
      double precision Bt3g1,B3g1c,Bt3g1a,Btt3g1a,renscale2
      double precision B3g1d1,B3g1d2,B3g1d3,B3g1q,B3g1qb
      integer i1,i2,i3,i4,i5

      Bt3g1= +1d0*(
     .       +B3g1c(i1,i2,i3,i4,i5,renscale2)
     .       +B3g1c(i1,i2,i4,i3,i5,renscale2)
     .       +B3g1c(i1,i3,i2,i4,i5,renscale2)
     .       +B3g1c(i1,i3,i4,i2,i5,renscale2)
     .       +B3g1c(i1,i4,i2,i3,i5,renscale2)
     .       +B3g1c(i1,i4,i3,i2,i5,renscale2))
     .       -1d0*(
     .       +Bt3g1a(i1,i2,i3,i4,i5,renscale2)
     .       +Bt3g1a(i1,i2,i4,i3,i5,renscale2)
     .       +Bt3g1a(i1,i3,i2,i4,i5,renscale2)
     .       +Bt3g1a(i1,i3,i4,i2,i5,renscale2)
     .       +Bt3g1a(i1,i4,i2,i3,i5,renscale2)
     .       +Bt3g1a(i1,i4,i3,i2,i5,renscale2))
     .       +1d0*(
     .       +Btt3g1a(i1,i2,i3,i4,i5,renscale2))
     .       -1d0*(
     .       +B3g1d1(i1,i2,i3,i4,i5,renscale2)
     .       +B3g1d2(i1,i2,i3,i4,i5,renscale2)
     .       +B3g1d3(i1,i2,i3,i4,i5,renscale2)
     .       +B3g1q(i1,i2,i3,i4,i5,renscale2)
     .       +B3g1qb(i1,i2,i3,i4,i5,renscale2))

      return
      end
**************************************************************************
* Full SLC nf (nf/N^3) colour level of FullB3g1
      function Bth3g1(i1,i2,i3,i4,i5,renscale2)
      double precision Bth3g1,Bt3g1b,Btt3g1b,Bh3g1q,Bh3g1qb,renscale2
      integer i1,i2,i3,i4,i5

      Bth3g1 = -1d0*(
     .         +Bt3g1b(i1,i2,i3,i4,i5,renscale2)
     .         +Bt3g1b(i1,i2,i4,i3,i5,renscale2)
     .         +Bt3g1b(i1,i3,i2,i4,i5,renscale2)
     .         +Bt3g1b(i1,i3,i4,i2,i5,renscale2)
     .         +Bt3g1b(i1,i4,i2,i3,i5,renscale2)
     .         +Bt3g1b(i1,i4,i3,i2,i5,renscale2)
     .         )
     .         +Btt3g1b(i1,i2,i3,i4,i5,renscale2)
     .         -1d0*(
     .         +Bh3g1q(i1,i2,i3,i4,i5,renscale2)
     .         +Bh3g1qb(i1,i2,i3,i4,i5,renscale2)
     .         )

      return
      end
**************************************************************************
* Full SSLC (1/N^4) colour level of FullB3g1
      function Btt3g1(i1,i2,i3,i4,i5,renscale2)
      double precision Btt3g1,Bt3g1c,Btt3g1a,Btt3g1c,renscale2
      integer i1,i2,i3,i4,i5
      double precision Bt3g1d,Bt3g1q,Bt3g1qb

      Btt3g1 = -1d0*(
     .           +Bt3g1c(i1,i2,i3,i4,i5,renscale2)
     .           +Bt3g1c(i1,i2,i4,i3,i5,renscale2)
     .           +Bt3g1c(i1,i3,i2,i4,i5,renscale2)
     .           +Bt3g1c(i1,i3,i4,i2,i5,renscale2)
     .           +Bt3g1c(i1,i4,i2,i3,i5,renscale2)
     .           +Bt3g1c(i1,i4,i3,i2,i5,renscale2)
     .           -Btt3g1a(i1,i2,i3,i4,i5,renscale2)
     .           -Btt3g1c(i1,i2,i3,i4,i5,renscale2)
     .           -Bt3g1d(i1,i2,i3,i4,i5,renscale2)
     .     -2d0*Bt3g1q(i1,i2,i3,i4,i5,renscale2)
     .     -2d0*Bt3g1qb(i1,i2,i3,i4,i5,renscale2)
     . )

      return
      end
**************************************************************************
* Full SSLC nf (nf/N^5) colour level of FullB3g1
      function Btth3g1(i1,i2,i3,i4,i5,renscale2)
      double precision Btth3g1,Btt3g1b,Bth3g1q,Bth3g1qb,renscale2
      integer i1,i2,i3,i4,i5

      Btth3g1 = +1d0*Btt3g1b(i1,i2,i3,i4,i5,renscale2)
     .          +2d0*Bth3g1q(i1,i2,i3,i4,i5,renscale2)
     .          +2d0*Bth3g1qb(i1,i2,i3,i4,i5,renscale2)

      return
      end
**************************************************************************
* Full SSSLC (1/N^6) colour level of FullB3g1      
      function Bttt3g1(i1,i2,i3,i4,i5,renscale2)
      double precision Bttt3g1,Btt3g1c,renscale2
      integer i1,i2,i3,i4,i5
      Bttt3g1=Btt3g1c(i1,i2,i3,i4,i5,renscale2)
      return
      end
**************************************************************************

