!************************************************************************
!  All 1-loop amplitudes in this file are based on
!  Z.~Kunszt, A.~Signer and Z.~Trocsanyi,
!  %``One loop radiative corrections to the helicity amplitudes of QCD processes involving four quarks and one gluon,''
!  Phys. Lett. B \textbf{336} (1994), 529-536
!  doi:10.1016/0370-2693(94)90568-1
!  [arXiv:hep-ph/9405386 [hep-ph]].
!  
!  A. Signer,
!  Helicity method for next-to-leading order corrections in QCD
!  PhD thesis, 
!  ETH-Z¨urich Nr. 11143 (1995).
**********************************************
* Wrapper functions for returning the normal
* matrix element for situations where either of the quarks
* in the second quark line are in the initial state.
* Otherwise it will symmetrise over the quark pair.
*
*****************************************************************************
      function sC1g1X(iq1,iqb1,iq2,iqb2,ig,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      if((iq2.le.2).or.(iqb2.le.2))then
         sC1g1X=C1g1X(iq1,iqb1,iq2,iqb2,ig,renscale2)
      else
         sC1g1X=0.5d0*(C1g1X(iq1,iqb1,iq2,iqb2,ig,renscale2)
     .                +C1g1X(iq1,iqb1,iqb2,iq2,ig,renscale2))
      endif

      return
      end
*****************************************************************************
      function sCh1g1X(iq1,iqb1,iq2,iqb2,ig,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      if((iq2.le.2).or.(iqb2.le.2))then
         sCh1g1X=Ch1g1X(iq1,iqb1,iq2,iqb2,ig,renscale2)
      else
         sCh1g1X=0.5d0*(Ch1g1X(iq1,iqb1,iq2,iqb2,ig,renscale2)
     .                 +Ch1g1X(iq1,iqb1,iqb2,iq2,ig,renscale2))
      endif

      return
      end
*****************************************************************************
      function sCt1g1(iq1,iqb1,iq2,iqb2,ig,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      if((iq2.le.2).or.(iqb2.le.2))then
         sCt1g1=Ct1g1(iq1,iqb1,iq2,iqb2,ig,renscale2)
      else
         sCt1g1=0.5d0*(Ct1g1(iq1,iqb1,iq2,iqb2,ig,renscale2)
     .                +Ct1g1(iq1,iqb1,iqb2,iq2,ig,renscale2))
      endif

      return
      end
*****************************************************************************
      function sCtt1g1(iq1,iqb1,iq2,iqb2,ig,renscale2)
      use KinData_mod
      implicit double precision (a-h,o-y)
      implicit complex*16 (z)

      if((iq2.le.2).or.(iqb2.le.2))then
         sCtt1g1=Ctt1g1(iq1,iqb1,iq2,iqb2,ig,renscale2)
      else
         sCtt1g1=0.5d0*(Ctt1g1(iq1,iqb1,iq2,iqb2,ig,renscale2)
     .                 +Ctt1g1(iq1,iqb1,iqb2,iq2,ig,renscale2))
      endif

      return
      end



*****************************************************************************
*       Four parton 1-loop 4q matrix elemets
*****************************************************************************
* call with arguments i1=q, i2=qb, i3=q, i4=qb, i5=g

      function FullC0g1ID(i1,i2,i3,i4,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      parameter(pi=3.141592653589793238d0)

      FullC0g1ID=FullC0g1(i1,i2,i3,i4,renscale2)
     .          +FullC0g1(i1,i4,i3,i2,renscale2)
     .          -FullD0g1(i1,i2,i3,i4,renscale2)

      return
      end

*****************************************************************************
*       Five parton 1-loop 2q2Q1g matrix elemets
*****************************************************************************
      function FullC1g1(iq1,iqb1,iq2,iqb2,ig,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      parameter(pi=3.141592653589793238d0)

      fac=(as*nc/2d0/pi)*(4d0*pi*as*nc)**3*(nc**2-1d0)/dble(nc)**2

      wt =   C1g1X(iq1,iqb1,iq2,iqb2,ig,renscale2)

     .     +dble(nf)/dble(nc)*(
     .     + Ch1g1X(iq1,iqb1,iq2,iqb2,ig,renscale2))

     .    +1d0/dble(nc)**2*(
     .    + C1g1Y(iq1,iqb1,iq2,iqb2,ig,renscale2)
     .    + Ct1g1X(iq1,iqb1,iq2,iqb2,ig,renscale2)
     .    - C1g1gamX(iq1,iqb1,iq2,iqb2,ig,renscale2)
     .    - C1g1gamY(iq1,iqb1,iq2,iqb2,ig,renscale2))

     .    +dble(nf)/dble(nc)**3*(
     .    + Ch1g1Y(iq1,iqb1,iq2,iqb2,ig,renscale2)
     .    - 2d0*Ch1g1gamX(iq1,iqb1,iq2,iqb2,ig,renscale2))

     .     +1d0/dble(nc)**4*(
     .    + Ct1g1Y(iq1,iqb1,iq2,iqb2,ig,renscale2)
     .    - Ct1g1gamX(iq1,iqb1,iq2,iqb2,ig,renscale2)
     .    - Ct1g1gamY(iq1,iqb1,iq2,iqb2,ig,renscale2))

* old formula
c      wt =
c     .     + C1g1a(iq1,ig,iqb2,iq2,iqb1,renscale2)
c     .     + C1g1b(iq1,iqb2,iq2,ig,iqb1,renscale2)

c     .    +dble(nf)/dble(nc)*(
c     .     + Ch1g1a(iq1,ig,iqb2,iq2,iqb1,renscale2)
c     .     + Ch1g1b(iq1,iqb2,iq2,ig,iqb1,renscale2))

c     .    +1d0/dble(nc)**2*(
c     .     + Ct1g1a(iq1,ig,iqb2,iq2,iqb1,renscale2)
c     .     + Ct1g1b(iq1,iqb2,iq2,ig,iqb1,renscale2)
c     .     + C1g1c(iq1,ig,iqb1,iq2,iqb2,renscale2)
c     .     + C1g1d(iq1,iqb1,iq2,ig,iqb2,renscale2)
c     .     - C1g1gam1(iq1,iqb1,iq2,iqb2,ig,renscale2)
c     .     - C1g1gam2(iq1,iqb1,iq2,iqb2,ig,renscale2))

c     .     +dble(nf)/dble(nc)**3*(
c     .     + Ch1g1c(iq1,ig,iqb1,iq2,iqb2,renscale2)
c     .     + Ch1g1d(iq1,iqb1,iq2,ig,iqb2,renscale2)
c     .     - 2d0*Ch1g1gam(iq1,iqb1,iq2,iqb2,ig,renscale2))

c     .     +1d0/dble(nc)**4*(
c     .     + Ct1g1c(iq1,ig,iqb1,iq2,iqb2,renscale2)
c     .     + Ct1g1d(iq1,iqb1,iq2,ig,iqb2,renscale2)
c     .     - Ct1g1gam1(iq1,iqb1,iq2,iqb2,ig,renscale2)
c     .     - Ct1g1gam2(iq1,iqb1,iq2,iqb2,ig,renscale2))

      wt=fac*wt
      FullC1g1=wt

      return
      end
*****************************************************************************
*       Five parton 1-loop 4q1g matrix elemets
*****************************************************************************

      function FullC1g1ID(iq1,iqb1,iq2,iqb2,ig,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      parameter(pi=3.141592653589793238d0)

      FullC1g1ID=FullC1g1(iq1,iqb1,iq2,iqb2,ig,renscale2)
     .          +FullC1g1(iq1,iqb2,iq2,iqb1,ig,renscale2)
     .          +FullD1g1(iq1,iqb1,iq2,iqb2,ig,renscale2)

      return
      end
*****************************************************************************
* interference terms

      function FullD1g1(iq1,iqb1,iq2,iqb2,ig,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      common /koppel/ nf,nc,b0
      common /strong/ as
!$omp threadprivate(/strong/)
      parameter(pi=3.141592653589793238d0)

      fac=(as*nc/2d0/pi)*(4d0*pi*as*nc)**3*(nc**2-1d0)/dble(nc)**3

      wt =
     .     + D1g1(iq1,iqb1,iq2,iqb2,ig,renscale2)
     .     + D1g1(iq1,iqb2,iq2,iqb1,ig,renscale2)
     .     - D1g1gam1(iq1,iqb1,iq2,iqb2,ig,renscale2)
     .     - D1g1gam1(iq1,iqb2,iq2,iqb1,ig,renscale2)

     .     +1d0/dble(nc)**2*(
     .     + Dt1g1(iq1,iqb1,iq2,iqb2,ig,renscale2)
     .     + Dt1g1(iq1,iqb2,iq2,iqb1,ig,renscale2)
     .     - Dt1g1gam1(iq1,iqb1,iq2,iqb2,ig,renscale2)
     .     - Dt1g1gam1(iq1,iqb2,iq2,iqb1,ig,renscale2)
     .     - D1g1gam2(iq1,iqb1,iq2,iqb2,ig,renscale2)
     .     - D1g1gam2(iq1,iqb2,iq2,iqb1,ig,renscale2))

     .     -1d0/dble(nc)**4*(
     .     + Dt1g1gam2(iq1,iqb1,iq2,iqb2,ig,renscale2)
     .     + Dt1g1gam2(iq1,iqb2,iq2,iqb1,ig,renscale2))

     .     +dble(nf)/dble(nc)*(
     .     + Dh1g1(iq1,iqb1,iq2,iqb2,ig,renscale2)
     .     + Dh1g1(iq1,iqb2,iq2,iqb1,ig,renscale2))

     .     -dble(nf)*((dble(nc)**2+1d0)/dble(nc)**3)*(
     .     + Dh1g1gam(iq1,iqb1,iq2,iqb2,ig,renscale2)
     .     + Dh1g1gam(iq1,iqb2,iq2,iqb1,ig,renscale2))

      wt=fac*wt
      FullD1g1=wt

      return
      end
*****************************************************************************
*     summed 1-loop/tree interference terms
*****************************************************************************
* leading colour qQ 2q2q1g term

      function C1g1X(iq1,iqb1,iq2,iqb2,ig,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-z)

      wt =   C1g1a(iq1,ig,iqb2,iq2,iqb1,renscale2)
     .     + C1g1b(iq1,iqb2,iq2,ig,iqb1,renscale2)

      C1g1X = wt
      return
      end

*****************************************************************************
* leading NF qQ 2q2q1g term

      function Ch1g1X(iq1,iqb1,iq2,iqb2,ig,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-z)

      wt =   Ch1g1a(iq1,ig,iqb2,iq2,iqb1,renscale2)
     .     + Ch1g1b(iq1,iqb2,iq2,ig,iqb1,renscale2)

      Ch1g1X = wt
      return
      end
*****************************************************************************
* SLC qQ 2q2q1g term

      function Ct1g1X(iq1,iqb1,iq2,iqb2,ig,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-z)

      wt =   Ct1g1a(iq1,ig,iqb2,iq2,iqb1,renscale2)
     .     + Ct1g1b(iq1,iqb2,iq2,ig,iqb1,renscale2)

      Ct1g1X = wt
      return
      end
*****************************************************************************
* symmetrized LC qQ 2q2q1g term

      function C1g1gamX(iq1,iqb1,iq2,iqb2,ig,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-z)

      wt =   C1g1gam1(iq1,iqb1,iq2,iqb2,ig,renscale2)

      C1g1gamX = wt
      return
      end
*****************************************************************************
* symmetrized NF qQ or QQ 2q2q1g term

      function Ch1g1gamX(iq1,iqb1,iq2,iqb2,ig,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-z)

      wt =   Ch1g1gam(iq1,iqb1,iq2,iqb2,ig,renscale2)

      Ch1g1gamX = wt
      return
      end
*****************************************************************************
* symmetrized SLC qQ 2q2q1g term

      function Ct1g1gamX(iq1,iqb1,iq2,iqb2,ig,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-z)

      wt =   Ct1g1gam1(iq1,iqb1,iq2,iqb2,ig,renscale2)

      Ct1g1gamX = wt
      return
      end
*****************************************************************************
* LC QQ 2q2q1g term

      function C1g1Y(iq1,iqb1,iq2,iqb2,ig,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-z)

      wt =   C1g1c(iq1,ig,iqb1,iq2,iqb2,renscale2)
     .     + C1g1d(iq1,iqb1,iq2,ig,iqb2,renscale2)

      C1g1Y = wt
      return
      end
*****************************************************************************
* Leading NF QQ 2q2q1g term

      function Ch1g1Y(iq1,iqb1,iq2,iqb2,ig,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-z)

      wt =   Ch1g1c(iq1,ig,iqb1,iq2,iqb2,renscale2)
     .     + Ch1g1d(iq1,iqb1,iq2,ig,iqb2,renscale2)

      Ch1g1Y = wt
      return
      end
*****************************************************************************
* SLC QQ 2q2q1g term

      function Ct1g1Y(iq1,iqb1,iq2,iqb2,ig,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-z)

      wt =   Ct1g1c(iq1,ig,iqb1,iq2,iqb2,renscale2)
     .     + Ct1g1d(iq1,iqb1,iq2,ig,iqb2,renscale2)

      Ct1g1Y = wt
      return
      end
*****************************************************************************
* symmetrized LC QQ 2q2q1g term

      function C1g1gamY(iq1,iqb1,iq2,iqb2,ig,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-z)

      wt =   C1g1gam2(iq1,iqb1,iq2,iqb2,ig,renscale2)

      C1g1gamY = wt
      return
      end
*****************************************************************************
* symmetrized SLC QQ 2q2q1g term

      function Ct1g1gamY(iq1,iqb1,iq2,iqb2,ig,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-z)

      wt =   Ct1g1gam2(iq1,iqb1,iq2,iqb2,ig,renscale2)

      Ct1g1gamY = wt
      return
      end
*****************************************************************************
*  interference between (1,0) Qqb tree and leading colour piece of (1,0)
*  Qqb 1-loop amplitude

      function C1g1a(iq1,ig,iqb2,iq2,iqb1,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      wt=0d0
      do ihel=1,4
      do iflip=0,1
      ztree = zzC1g0a(iq1,ig,iqb2,iq2,iqb1,ihel,iflip)
      ztmp  = zzC1g1a(iq1,ig,iqb2,iq2,iqb1,ihel,iflip,renscale2)
      wt = wt + 2d0*real(ztmp*dconjg(ztree))
*      wt = wt + real(ztree*dconjg(ztree))
      enddo
      enddo

      C1g1a = wt
*      write(*,*)"C1g1a",C1g1a,C1g01(iq1,ig,iqb2,iq2,iqb1)
      return
      end

*****************************************************************************
*  interference between (0,1) Qqb tree and leading colour piece of (0,1)
*  Qqb 1-loop amplitude

      function C1g1b(iq1,iqb2,iq2,ig,iqb1,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      wt=0d0
      do ihel=1,4
      do iflip=0,1
      ztree = zzC1g0b(iq1,iqb2,iq2,ig,iqb1,ihel,iflip)
      ztmp  = zzC1g1b(iq1,iqb2,iq2,ig,iqb1,ihel,iflip,renscale2)
      wt = wt + 2d0*real(ztmp*dconjg(ztree))
*      wt = wt + real(ztree*dconjg(ztree))
      enddo
      enddo

      C1g1b = wt
*      write(*,*)"C1g1b",C1g1b,C1g0b(iq1,iqb2,iq2,ig,iqb1)
      return
      end

*****************************************************************************
*  interference between (1,0) Qqb tree and NF piece of (1,0) Qqb 1-loop amplitude

      function Ch1g1a(iq1,ig,iqb2,iq2,iqb1,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      wt=0d0
      do ihel=1,4
      do iflip=0,1
        ztree = zzC1g0a(iq1,ig,iqb2,iq2,iqb1,ihel,iflip)
        ztmp  = zzCh1g1a(iq1,ig,iqb2,iq2,iqb1,ihel,iflip,renscale2)
        wt = wt + 2d0*real(ztmp*dconjg(ztree))
      enddo
      enddo

      Ch1g1a = wt
c      write(*,*)"Ch1g1a",Ch1g1a

      return
      end
*****************************************************************************
*  interference between (0,1) Qqb tree and NF piece of (0,1) Qqb 1-loop amplitude

      function Ch1g1b(iq1,iqb2,iq2,ig,iqb1,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      wt=0d0
      do ihel=1,4
      do iflip=0,1
        ztree = zzC1g0b(iq1,iqb2,iq2,ig,iqb1,ihel,iflip)
        ztmp  = zzCh1g1b(iq1,iqb2,iq2,ig,iqb1,ihel,iflip,renscale2)
        wt = wt + 2d0*real(ztmp*dconjg(ztree))
      enddo
      enddo

      Ch1g1b = wt
c      write(*,*)"Ch1g1b",Ch1g1b

      return
      end
*****************************************************************************
*  interference between (1,0) Qqb tree and 1/N piece of (1,0) Qqb 1-loop amplitude

      function Ct1g1a(iq1,ig,iqb2,iq2,iqb1,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      wt=0d0
      do ihel=1,4
      do iflip=0,1
        ztree = zzC1g0a(iq1,ig,iqb2,iq2,iqb1,ihel,iflip)
        ztmp  = zzCt1g1a(iq1,ig,iqb2,iq2,iqb1,ihel,iflip,renscale2)
        wt = wt + 2d0*real(ztmp*dconjg(ztree))
      enddo
      enddo
      Ct1g1a = wt
c      write(*,*)"Ct1g1a",Ct1g1a

      return
      end
*****************************************************************************
*  interference between (0,1) Qqb tree and 1/N piece of (0,1) Qqb 1-loop amplitude

      function Ct1g1b(iq1,iqb2,iq2,ig,iqb1,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      wt=0d0
      do ihel=1,4
      do iflip=0,1
        ztree = zzC1g0b(iq1,iqb2,iq2,ig,iqb1,ihel,iflip)
        ztmp  = zzCt1g1b(iq1,iqb2,iq2,ig,iqb1,ihel,iflip,renscale2)
        wt = wt + 2d0*real(ztmp*dconjg(ztree))
      enddo
      enddo

      Ct1g1b = wt
c      write(*,*)"Ct1g1b",Ct1g1b

      return
      end
*****************************************************************************
*  interference between (1,0) QQb tree and leading colour piece of (1,0)
*  QQb 1-loop amplitude

      function C1g1c(iq1,ig,iqb1,iq2,iqb2,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      wt=0d0
      do ihel=1,4
      do iflip=0,1
        ztree = zzC1g0c(iq1,ig,iqb1,iq2,iqb2,ihel,iflip)
        ztmp  = zzC1g1c(iq1,ig,iqb1,iq2,iqb2,ihel,iflip,renscale2)
        wt = wt + 2d0*real(ztmp*dconjg(ztree))
      enddo
      enddo

      C1g1c = wt
c      write(*,*)"C1g1c",C1g1c
      return
      end
*****************************************************************************
*  interference between (0,1) QQb tree and leading colour piece of (0,1)
*  QQb 1-loop amplitude

      function C1g1d(iq1,iqb1,iq2,ig,iqb2,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      wt=0d0
      do ihel=1,4
      do iflip=0,1
        ztree = zzC1g0d(iq1,iqb1,iq2,ig,iqb2,ihel,iflip)
        ztmp  = zzC1g1d(iq1,iqb1,iq2,ig,iqb2,ihel,iflip,renscale2)
        wt = wt + 2d0*real(ztmp*dconjg(ztree))
      enddo
      enddo

      C1g1d = wt
c      write(*,*)"C1g1d",C1g1d
      return
      end
*****************************************************************************
*  interference between (1,0) QQb tree and NF piece of (1,0) QQb 1-loop amplitude

      function Ch1g1c(iq1,ig,iqb1,iq2,iqb2,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      wt=0d0
      do ihel=1,4
      do iflip=0,1
        ztree = zzC1g0c(iq1,ig,iqb1,iq2,iqb2,ihel,iflip)
        ztmp  = zzCh1g1c(iq1,ig,iqb1,iq2,iqb2,ihel,iflip,renscale2)
        wt = wt + 2d0*real(ztmp*dconjg(ztree))
      enddo
      enddo

      Ch1g1c = wt
c      write(*,*)"Ch1g1c",Ch1g1c

      return
      end
*****************************************************************************
*  interference between (0,1) QQb tree and NF piece of (0,1) QQb 1-loop amplitude

      function Ch1g1d(iq1,iqb1,iq2,ig,iqb2,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      wt=0d0
      do ihel=1,4
      do iflip=0,1
        ztree = zzC1g0d(iq1,iqb1,iq2,ig,iqb2,ihel,iflip)
        ztmp  = zzCh1g1d(iq1,iqb1,iq2,ig,iqb2,ihel,iflip,renscale2)
        wt = wt + 2d0*real(ztmp*dconjg(ztree))
      enddo
      enddo

      Ch1g1d = wt
c      write(*,*)"Ch1g1d",Ch1g1d

      return
      end
*****************************************************************************
*  interference between (1,0) QQb tree and sub-leading colour piece of (1,0)
*  QQb 1-loop amplitude

      function Ct1g1c(iq1,ig,iqb1,iq2,iqb2,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      wt=0d0
      do ihel=1,4
      do iflip=0,1
        ztree = zzC1g0c(iq1,ig,iqb1,iq2,iqb2,ihel,iflip)
        ztmp  = zzCt1g1c(iq1,ig,iqb1,iq2,iqb2,ihel,iflip,renscale2)
        wt = wt + 2d0*real(ztmp*dconjg(ztree))
      enddo
      enddo

      Ct1g1c = wt
c      write(*,*)"Ct1g1a",Ct1g1c

      return
      end
*****************************************************************************
*  interference between (0,1) QQb tree and sub-leading colour piece of (0,1)
*  QQb 1-loop amplitude

      function Ct1g1d(iq1,iqb1,iq2,ig,iqb2,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      wt=0d0
      do ihel=1,4
      do iflip=0,1
        ztree = zzC1g0d(iq1,iqb1,iq2,ig,iqb2,ihel,iflip)
        ztmp  = zzCt1g1d(iq1,iqb1,iq2,ig,iqb2,ihel,iflip,renscale2)
        wt = wt + 2d0*real(ztmp*dconjg(ztree))
      enddo
      enddo

      Ct1g1d = wt
c      write(*,*)"Ct1g1d",Ct1g1d

      return
      end
*****************************************************************************
*  interference between symmetrized tree and leading colour piece of Qqb symmetrized
*  1-loop amplitude

      function C1g1gam1(iq1,iqb1,iq2,iqb2,ig,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)


      wt=0d0
      do ihel=1,4
      do iflip=0,1
      ztree = zzC1g0gam(iq1,iqb1,iq2,iqb2,ig,ihel,iflip)

      zloop =
     . + zzC1g1a(iq1,ig,iqb2,iq2,iqb1,ihel,iflip,renscale2)
     . + zzC1g1b(iq1,iqb2,iq2,ig,iqb1,ihel,iflip,renscale2)
      wt = wt + 2d0*real(zloop*dconjg(ztree))
      enddo
      enddo

      C1g1gam1 = wt
c      write(*,*)"C1g1gam1",C1g1gam1

      return
      end
*****************************************************************************
*  interference between symmetrized tree and leading colour piece of QQb symmetrized
*  1-loop amplitude

      function C1g1gam2(iq1,iqb1,iq2,iqb2,ig,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)


      wt=0d0
      do ihel=1,4
      do iflip=0,1
      ztree = zzC1g0gam(iq1,iqb1,iq2,iqb2,ig,ihel,iflip)

      zloop =
     . + zzC1g1c(iq1,ig,iqb1,iq2,iqb2,ihel,iflip,renscale2)
     . + zzC1g1d(iq1,iqb1,iq2,ig,iqb2,ihel,iflip,renscale2)
      wt = wt + 2d0*real(zloop*dconjg(ztree))
      enddo
      enddo

      C1g1gam2 = wt
c      write(*,*)"C1g1gam2",C1g1gam2

      return
      end
*****************************************************************************
*  interference between symmetrized tree and NF piece of symmetrized
*  1-loop amplitude

      function Ch1g1gam(iq1,iqb1,iq2,iqb2,ig,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      wt=0d0
      do ihel=1,4
      do iflip=0,1
      ztree = zzC1g0gam(iq1,iqb1,iq2,iqb2,ig,ihel,iflip)

      zloop =
     .+ zzCh1g1c(iq1,ig,iqb1,iq2,iqb2,ihel,iflip,renscale2) !call c+d rather than a+b
     .+ zzCh1g1d(iq1,iqb1,iq2,ig,iqb2,ihel,iflip,renscale2) !because smaller formulae

      wt = wt + 2d0*real((zloop)*dconjg(ztree))
      enddo
      enddo

      Ch1g1gam = wt
c      write(*,*)"Ch1g1gam",Ch1g1gam

      return
      end
*****************************************************************************
*  interference between symmetrized tree and SLC piece of Qqb symmetrized
*  1-loop amplitude

      function Ct1g1gam1(iq1,iqb1,iq2,iqb2,ig,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)


      wt=0d0
      do ihel=1,4
      do iflip=0,1
      ztree = zzC1g0gam(iq1,iqb1,iq2,iqb2,ig,ihel,iflip)

      zloop =
     . + zzCt1g1a(iq1,ig,iqb2,iq2,iqb1,ihel,iflip,renscale2)
     . + zzCt1g1b(iq1,iqb2,iq2,ig,iqb1,ihel,iflip,renscale2)
      wt = wt + 2d0*real(zloop*dconjg(ztree))
      enddo
      enddo

      Ct1g1gam1 = wt
c      write(*,*)"Ct1g1gam1",Ct1g1gam1

      return
      end
*****************************************************************************
*  interference between symmetrized tree and SLC piece of QQb symmetrized
*  1-loop amplitude

      function Ct1g1gam2(iq1,iqb1,iq2,iqb2,ig,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)


      wt=0d0
      do ihel=1,4
      do iflip=0,1
      ztree = zzC1g0gam(iq1,iqb1,iq2,iqb2,ig,ihel,iflip)

      zloop =
     . + zzCt1g1c(iq1,ig,iqb1,iq2,iqb2,ihel,iflip,renscale2)
     . + zzCt1g1d(iq1,iqb1,iq2,ig,iqb2,ihel,iflip,renscale2)
      wt = wt + 2d0*real(zloop*dconjg(ztree))
      enddo
      enddo

      Ct1g1gam2 = wt
c      write(*,*)"Ct1g1gam2",Ct1g1gam2

      return
      end
*****************************************************************************
*  block of interferences between LC qQ and QQ 1-loop and trees with flipped
*  antiquark orderings

      function D1g1(iq1,iqb1,iq2,iqb2,ig,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      dimension ihel2(6)

      ihel2(1)=1
      ihel2(2)=5
      ihel2(3)=6
      ihel2(4)=4
      ihel2(5)=2
      ihel2(6)=3

      wt=0d0
      do ihel=1,6
      do iflip=0,1
        zwt11 = zzC1g0c(iq1,ig,iqb1,iq2,iqb2,ihel,iflip)
        zwt12 = zzC1g1a(iq1,ig,iqb1,iq2,iqb2,ihel2(ihel),iflip,renscale2)

        zwt21 = zzC1g0d(iq1,iqb1,iq2,ig,iqb2,ihel,iflip)
        zwt22 = zzC1g1b(iq1,iqb1,iq2,ig,iqb2,ihel2(ihel),iflip,renscale2)

        zwt31 = zzC1g0a(iq1,ig,iqb1,iq2,iqb2,ihel,iflip)
        zwt32 = zzC1g1c(iq1,ig,iqb1,iq2,iqb2,ihel2(ihel),iflip,renscale2)

        zwt41 = zzC1g0b(iq1,iqb1,iq2,ig,iqb2,ihel,iflip)
        zwt42 = zzC1g1d(iq1,iqb1,iq2,ig,iqb2,ihel2(ihel),iflip,renscale2)

        wt = wt + 2d0*real(zwt12*dconjg(zwt11))
        wt = wt + 2d0*real(zwt22*dconjg(zwt21))
        wt = wt + 2d0*real(zwt32*dconjg(zwt31))
        wt = wt + 2d0*real(zwt42*dconjg(zwt41))
      enddo
      enddo

      D1g1 = wt
c      write(*,*)"D1g1",D1g1

      return
      end
*****************************************************************************
*  block of interferences between NF qQ and QQ 1-loop and trees with flipped
*  antiquark orderings

      function Dh1g1(iq1,iqb1,iq2,iqb2,ig,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      dimension ihel2(6)

      ihel2(1)=1
      ihel2(2)=5
      ihel2(3)=6
      ihel2(4)=4
      ihel2(5)=2
      ihel2(6)=3

      wt=0d0
      do ihel=1,6
      do iflip=0,1
        zwt11 = zzC1g0c(iq1,ig,iqb1,iq2,iqb2,ihel,iflip)
        zwt12 = zzCh1g1a(iq1,ig,iqb1,iq2,iqb2,ihel2(ihel),iflip,renscale2)

        zwt21 = zzC1g0d(iq1,iqb1,iq2,ig,iqb2,ihel,iflip)
        zwt22 = zzCh1g1b(iq1,iqb1,iq2,ig,iqb2,ihel2(ihel),iflip,renscale2)

        zwt31 = zzC1g0a(iq1,ig,iqb1,iq2,iqb2,ihel,iflip)
        zwt32 = zzCh1g1c(iq1,ig,iqb1,iq2,iqb2,ihel2(ihel),iflip,renscale2)

        zwt41 = zzC1g0b(iq1,iqb1,iq2,ig,iqb2,ihel,iflip)
        zwt42 = zzCh1g1d(iq1,iqb1,iq2,ig,iqb2,ihel2(ihel),iflip,renscale2)

        wt = wt + 2d0*real(zwt12*dconjg(zwt11))
        wt = wt + 2d0*real(zwt22*dconjg(zwt21))
        wt = wt + 2d0*real(zwt32*dconjg(zwt31))
        wt = wt + 2d0*real(zwt42*dconjg(zwt41))
      enddo
      enddo

      Dh1g1 = wt
c      write(*,*)"Dh1g1",Dh1g1

      return
      end
*****************************************************************************
*  block of interferences between SLC qQ and QQ 1-loop and trees with flipped
*  antiquark orderings

      function Dt1g1(iq1,iqb1,iq2,iqb2,ig,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      dimension ihel2(6)

      ihel2(1)=1
      ihel2(2)=5
      ihel2(3)=6
      ihel2(4)=4
      ihel2(5)=2
      ihel2(6)=3

      wt=0d0
      do ihel=1,6
      do iflip=0,1
        zwt11 = zzC1g0c(iq1,ig,iqb1,iq2,iqb2,ihel,iflip)
        zwt12 = zzCt1g1a(iq1,ig,iqb1,iq2,iqb2,ihel2(ihel),iflip,renscale2)

        zwt21 = zzC1g0d(iq1,iqb1,iq2,ig,iqb2,ihel,iflip)
        zwt22 = zzCt1g1b(iq1,iqb1,iq2,ig,iqb2,ihel2(ihel),iflip,renscale2)

        zwt31 = zzC1g0a(iq1,ig,iqb1,iq2,iqb2,ihel,iflip)
        zwt32 = zzCt1g1c(iq1,ig,iqb1,iq2,iqb2,ihel2(ihel),iflip,renscale2)

        zwt41 = zzC1g0b(iq1,iqb1,iq2,ig,iqb2,ihel,iflip)
        zwt42 = zzCt1g1d(iq1,iqb1,iq2,ig,iqb2,ihel2(ihel),iflip,renscale2)

        wt = wt + 2d0*real(zwt12*dconjg(zwt11))
        wt = wt + 2d0*real(zwt22*dconjg(zwt21))
        wt = wt + 2d0*real(zwt32*dconjg(zwt31))
        wt = wt + 2d0*real(zwt42*dconjg(zwt41))
      enddo
      enddo

      Dt1g1 = wt
c      write(*,*)"Dt1g1",Dt1g1

      return
      end
*****************************************************************************
*  interference between symmetrized LC qQ 1-loop and symmetrized tree
*  with different quark orderings (antiquarks swapped)

      function D1g1gam1(iq1,iqb1,iq2,iqb2,ig,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      dimension ihel2(6)

      ihel2(1)=1
      ihel2(2)=5
      ihel2(3)=6
      ihel2(4)=4
      ihel2(5)=2
      ihel2(6)=3

      wt=0d0
      do ihel=1,6
      do iflip=0,1
        ztree1 = zzC1g0gam(iq1,iqb1,iq2,iqb2,ig,ihel,iflip)
        zloop = zzC1g1gam1(iq1,iqb2,iq2,iqb1,ig,ihel2(ihel),iflip,renscale2)
        wt = wt + 2d0*real(zloop*dconjg(ztree1))
      enddo
      enddo

      D1g1gam1 = wt
c      write(*,*)"D1g1gam1",D1g1gam1

      return
      end
*****************************************************************************
*  interference between symmetrized NF 1-loop and symmetrized tree
*  with different quark orderings (antiquarks swapped)

      function Dh1g1gam(iq1,iqb1,iq2,iqb2,ig,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      dimension ihel2(6)

      ihel2(1)=1
      ihel2(2)=5
      ihel2(3)=6
      ihel2(4)=4
      ihel2(5)=2
      ihel2(6)=3

      wt=0d0
      do ihel=1,6
      do iflip=0,1
        ztree1 = zzC1g0gam(iq1,iqb1,iq2,iqb2,ig,ihel,iflip)
        zloop = zzCh1g1gam(iq1,iqb2,iq2,iqb1,ig,ihel2(ihel),iflip,renscale2)
        wt = wt + 2d0*real(zloop*dconjg(ztree1))
      enddo
      enddo

      Dh1g1gam = wt
c      write(*,*)"Dh1g1gam",Dh1g1gam

      return
      end
*****************************************************************************
*  interference between symmetrized SLC qQ 1-loop and symmetrized tree
*  with different quark orderings (antiquarks swapped)

      function Dt1g1gam1(iq1,iqb1,iq2,iqb2,ig,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      dimension ihel2(6)

      ihel2(1)=1
      ihel2(2)=5
      ihel2(3)=6
      ihel2(4)=4
      ihel2(5)=2
      ihel2(6)=3

      wt=0d0
      do ihel=1,6
      do iflip=0,1
        ztree1 = zzC1g0gam(iq1,iqb1,iq2,iqb2,ig,ihel,iflip)
        zloop = zzCt1g1gam1(iq1,iqb2,iq2,iqb1,ig,ihel2(ihel),iflip,renscale2)
        wt = wt + 2d0*real(zloop*dconjg(ztree1))
      enddo
      enddo

      Dt1g1gam1 = wt
c      write(*,*)"Dt1g1gam1",Dt1g1gam1

      return
      end
*****************************************************************************
*  interference between symmetrized LC QQ 1-loop and symmetrized tree
*  with different quark orderings (antiquarks swapped)

      function D1g1gam2(iq1,iqb1,iq2,iqb2,ig,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      dimension ihel2(6)

      ihel2(1)=1
      ihel2(2)=5
      ihel2(3)=6
      ihel2(4)=4
      ihel2(5)=2
      ihel2(6)=3

      wt=0d0
      do ihel=1,6
      do iflip=0,1
        ztree1 = zzC1g0gam(iq1,iqb1,iq2,iqb2,ig,ihel,iflip)
        zloop = zzC1g1gam2(iq1,iqb2,iq2,iqb1,ig,ihel2(ihel),iflip,renscale2)
        wt = wt + 2d0*real(zloop*dconjg(ztree1))
      enddo
      enddo

      D1g1gam2 = wt
c      write(*,*)"D1g1gam2",D1g1gam2

      return
      end
*****************************************************************************
*  interference between symmetrized LC QQ 1-loop and symmetrized tree
*  with different quark orderings (antiquarks swapped)

      function Dt1g1gam2(iq1,iqb1,iq2,iqb2,ig,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      dimension ihel2(6)

      ihel2(1)=1
      ihel2(2)=5
      ihel2(3)=6
      ihel2(4)=4
      ihel2(5)=2
      ihel2(6)=3

      wt=0d0
      do ihel=1,6
      do iflip=0,1
        ztree1 = zzC1g0gam(iq1,iqb1,iq2,iqb2,ig,ihel,iflip)
        zloop = zzCt1g1gam2(iq1,iqb2,iq2,iqb1,ig,ihel2(ihel),iflip,renscale2)
        wt = wt + 2d0*real(zloop*dconjg(ztree1))
      enddo
      enddo

      Dt1g1gam2 = wt
c      write(*,*)"Dt1g1gam2",Dt1g1gam2
      return
      end
*****************************************************************************
*      1-loop amplitudes
*****************************************************************************
* leading colour (1,0) 1-loop Qqb amplitude

      function zzC1g1a(iq1,ig,iqb2,iq2,iqb1,ihel,iflip,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      common /order/iorder
      parameter(pi=3.141592653589793238d0)

      i1=iq1
      i2=iqb2
      i3=iq2
      i4=iqb1
      i5=ig

      s12=s5(i2,i4)
      s13=s5(i2,i1)
      s14=s5(i2,i3)
      s15=s5(i2,i5)
      s23=s5(i4,i1)
      s24=s5(i4,i3)
      s25=s5(i4,i5)
      s34=s5(i1,i3)
      s35=s5(i1,i5)
      s45=s5(i3,i5)

      if(iflip.eq.0)then

      za12=za5(i2,i4)
      za13=za5(i2,i1)
      za14=za5(i2,i3)
      za15=za5(i2,i5)
      za23=za5(i4,i1)
      za24=za5(i4,i3)
      za25=za5(i4,i5)
      za34=za5(i1,i3)
      za35=za5(i1,i5)
      za45=za5(i3,i5)

      zb12=zb5(i2,i4)
      zb13=zb5(i2,i1)
      zb14=zb5(i2,i3)
      zb15=zb5(i2,i5)
      zb23=zb5(i4,i1)
      zb24=zb5(i4,i3)
      zb25=zb5(i4,i5)
      zb34=zb5(i1,i3)
      zb35=zb5(i1,i5)
      zb45=zb5(i3,i5)

      elseif(iflip.eq.1)then

      za12=-zb5(i2,i4)
      za13=-zb5(i2,i1)
      za14=-zb5(i2,i3)
      za15=-zb5(i2,i5)
      za23=-zb5(i4,i1)
      za24=-zb5(i4,i3)
      za25=-zb5(i4,i5)
      za34=-zb5(i1,i3)
      za35=-zb5(i1,i5)
      za45=-zb5(i3,i5)

      zb12=-za5(i2,i4)
      zb13=-za5(i2,i1)
      zb14=-za5(i2,i3)
      zb15=-za5(i2,i5)
      zb23=-za5(i4,i1)
      zb24=-za5(i4,i3)
      zb25=-za5(i4,i5)
      zb34=-za5(i1,i3)
      zb35=-za5(i1,i5)
      zb45=-za5(i3,i5)

      endif

      zi=(0d0,1d0)
      ztree=zzC1g0a(i1,i5,i2,i3,i4,ihel,iflip)

      if(ihel.eq.1)then

      ze2 =  + ztree * (
     &     - 3.D0
     &     )

      ze1 =  + ztree * (
     &     - 29.D0/6.D0
     &     + zzlog( - s35)
     &     + zzlog( - s24)
     &     + zzlog( - s15)
     &     - 3.D0*zzlog(renscale2)
     &     )

      ze0 =  + zL0(s23,s14,renscale2) * (
     &     + zi*za12*za14**(-1)*za15*za23**(-1)*za24*za34*za35**(-1)*
     &    za45**(-2)
     &     - 1.D0/2.D0*zi*za12**2*za14**(-1)*za23**(-1)*za34*za35**(-1)
     &    *za45**(-1)
     &     )
      ze0 = ze0 + zL0(s23,s15,renscale2) * (
     &     - zi*za12*za14*za15**(-1)*za23**(-1)*za25*za45**(-2)
     &     + 3.D0/2.D0*zi*za12**2*za15**(-1)*za23**(-1)*za45**(-1)
     &     )
      ze0 = ze0 + zL0(s23,s24,renscale2) * (
     &     + zi*za12*za13*za15**(-1)*za23**(-1)*za25*za35**(-1)*
     &    za45**(-1)
     &     )
      ze0 = ze0 + zL0(s24,s15,renscale2) * (
     &     + zi*za12*za13*za15**(-1)*za23**(-1)*za24*za34**(-1)*
     &    za45**(-1)
     &     + 2.D0*zi*za12*za13*za15**(-1)*za34**(-1)*za35**(-1)
     &     )
      ze0 = ze0 + zL1(s23,s14,renscale2) * (
     &     - zi*za15*za24*zb35*za45**(-2)
     &     + 3.D0*zi*za13*za14**(-1)*za15*za24*za35**(-1)*zb35*
     &    za45**(-1)
     &     - 2.D0*zi*za13*za25*za35**(-1)*zb35*za45**(-1)
     &     - 3.D0/2.D0*zi*za12*za35**(-1)*zb45
     &     + 2.D0/3.D0*zi*za12**2*za14**(-1)*zb25*za35**(-1)
     &     )
      ze0 = ze0 + zL1(s23,s15,renscale2) * (
     &     + zi*za14**2*za15**(-1)*za24*za34**(-1)*zb34*za35*za45**(-2)
     &     + zi*za14**2*za15**(-1)*za23*za34**(-1)*zb34*za45**(-1)
     &     - 3.D0*zi*za13*za14*za15**(-1)*za24*za34**(-1)*zb34*
     &    za45**(-1)
     &     )
      ze0 = ze0 + zL1(s24,s15,renscale2) * (
     &     + 3.D0*zi*za13*za23*za34**(-1)*za35**(-1)*zb35
     &     + 2.D0*zi*za13**2*za15**(-1)*za24*za34**(-1)*zb34*za35**(-1)
     &     )
      ze0 = ze0 + zL2(s14,s23,renscale2) * (
     &     - 1.D0/2.D0*zi*za15*za23*zb35**2*za45**(-1)
     &     )
      ze0 = ze0 + zL2(s15,s23,renscale2) * (
     &     - 1.D0/2.D0*zi*za14**2*za15**(-1)*za23*zb34**2*za45**(-1)
     &     )
      ze0 = ze0 + zL2(s23,s14,renscale2) * (
     &     + 1.D0/2.D0*zi*za15*za23*za35**(-1)*zb35*zb45
     &     - 1.D0/3.D0*zi*za12*zb35*zb45
     &     )
      ze0 = ze0 + zL3(s23,s14,renscale2) * (
     &     + 2.D0/3.D0*zi*za15*za23*zb35**2*zb45
     &     )
      ze0 = ze0 + ztree * (
     &     + 29.D0/18.D0
     &     - 1.D0/2.D0*zzlog( - s35)**2
     &     - 1.D0/2.D0*zzlog( - s24)**2
     &     - 2.D0/3.D0*zzlog( - s23)
     &     - 1.D0/2.D0*zzlog( - s15)**2
     &     + 2.D0/3.D0*zzlog(renscale2)
     &     + zzlog(renscale2)*zzlog( - s35)
     &     + zzlog(renscale2)*zzlog( - s24)
     &     + zzlog(renscale2)*zzlog( - s15)
     &     - 3.D0/2.D0*zzlog(renscale2)**2
     &     )
      ze0 = ze0 - 1.D0/2.D0*zi*za12*s14**(-1)*za35**(-1)*zb45
     &     + 1.D0/2.D0*zi*za12**2*za14**(-1)*za23**(-1)*za34*za35**(-1)
     &    *za45**(-1)
     &     - zCF(s14,s35,s23)*zi*za12**2*za14**(-1)*za25**(-1)*
     &    za35**(-1)
     &     + zCF(s14,s35,s23)*zi*za12**2*za13*za14**(-1)*za15**(-1)*
     &    za23**(-1)*za35**(-1)
     &     + zCF(s15,s24,s23)*zi*za13**3*za14**(-1)*za15**(-1)*
     &    za23**(-1)*za24**2*za34**(-2)*za35**(-1)
     &     - zCF(s23,s14,s15)*zi*za15*za23**(-1)*za24**2*za45**(-3)
     &     + zCF(s23,s14,s15)*zi*za13*za14**(-1)*za15*za23**(-1)*
     &    za24**2*za35**(-1)*za45**(-2)
     &     + zCF(s24,s15,s35)*zi*za12**2*za13*za14**(-1)*za15**(-1)*
     &    za23**(-1)*za35**(-1)
     &     + zCF(s35,s14,s24)*zi*za12**2*za13*za14**(-1)*za15**(-1)*
     &    za23**(-1)*za35**(-1)


      elseif(ihel.eq.2)then

      ze2 =  + ztree * (
     &     - 3.D0
     &     )

      ze1 =  + ztree * (
     &     - 29.D0/6.D0
     &     + zzlog( - s35)
     &     + zzlog( - s24)
     &     + zzlog( - s15)
     &     - 3.D0*zzlog(renscale2)
     &     )

      ze0 =  + zi * (
     &     + 1.D0/2.D0*za15**(-1)*s14**(-1)*zb35**2*za45*zb23**(-1)
     &     + 1.D0/2.D0*za35**(-1)*za24*s14**(-1)*zb15
     &     + 1.D0/2.D0*za13*za14**(-1)*za15**(-1)*za23**(-1)*za35**(-1)
     &    *za24**2
     &     )
      ze0 = ze0 + zL0(s23,s14,renscale2)*zi * (
     &     - 3.D0/2.D0*za12*za14**(-1)*za15**(-1)*za23**(-1)*za24**2*
     &    za25**(-1)
     &     )
      ze0 = ze0 + zL1(s23,s14,renscale2)*zi * (
     &     - 2.D0/3.D0*za14**(-1)*za35**(-1)*za24**2*zb25
     &     + za15**(-1)*za24*zb35
     &     - 3.D0/2.D0*za13*za14**(-1)*za15**(-1)*za35**(-1)*za24*zb35*
     &    za45
     &     - 3.D0/2.D0*za12*za14**(-1)*za15**(-1)*za24*za25**(-1)*zb35*
     &    za45
     &     )
      ze0 = ze0 + zL2(s23,s14,renscale2)*zi * (
     &     + 1.D0/2.D0*za15**(-1)*za23*za35**(-1)*zb35*s35*za45
     &     - 1.D0/2.D0*za15**(-1)*za23*za35**(-1)*s15*zb35*za45
     &     + 1.D0/3.D0*za24*zb35*zb15
     &     )
      ze0 = ze0 + zL3(s23,s14,renscale2)*zi * (
     &     + 2.D0/3.D0*za23*zb35**2*za45*zb15
     &     )
      ze0 = ze0 + zCF(s14,s35,s23)*zi * (
     &     + za14**(-1)*za35**(-1)*za24**2*za25**(-1)
     &     )
      ze0 = ze0 + zCF(s14,s35,s23)*ztree * (
     &     - 1.D0
     &     )
      ze0 = ze0 + zCF(s15,s24,s23)*ztree * (
     &     - 1.D0
     &     )
      ze0 = ze0 + zCF(s23,s14,s15)*zi * (
     &     + za15**(-1)*za23**(-1)*za24**2*za45**(-1)
     &     )
      ze0 = ze0 + zCF(s23,s14,s15)*ztree * (
     &     - 1.D0
     &     )
      ze0 = ze0 + zCF(s24,s15,s35)*ztree * (
     &     - 1.D0
     &     )
      ze0 = ze0 + zCF(s35,s14,s24)*ztree * (
     &     - 1.D0
     &     )
      ze0 = ze0 + ztree * (
     &     + 29.D0/18.D0
     &     - 1.D0/2.D0*zzlog( - s35)**2
     &     - 1.D0/2.D0*zzlog( - s24)**2
     &     - 2.D0/3.D0*zzlog( - s23)
     &     - 1.D0/2.D0*zzlog( - s15)**2
     &     + 2.D0/3.D0*zzlog(renscale2)
     &     + zzlog(renscale2)*zzlog( - s35)
     &     + zzlog(renscale2)*zzlog( - s24)
     &     + zzlog(renscale2)*zzlog( - s15)
     &     - 3.D0/2.D0*zzlog(renscale2)**2
     &     )

      elseif(ihel.eq.3)then

      ze2 =  + ztree * (
     &     - 3.D0
     &     )

      ze1 =  + ztree * (
     &     - 29.D0/6.D0
     &     + zzlog( - s35)
     &     + zzlog( - s24)
     &     + zzlog( - s15)
     &     - 3.D0*zzlog(renscale2)
     &     )

      ze0 =  + zi * (
     &     + 1.D0/2.D0*za13**3*za14**(-1)*za15**(-1)*za23**(-1)*
     &    za35**(-1)
     &     )
      ze0 = ze0 + zL1(s23,s14,renscale2)*zi * (
     &     - za15*za23**(-1)*za34**2*za45**(-2)*zb45
     &     + 1.D0/2.D0*za13*za23**(-1)*za35*za24*za25**(-1)*za45**(-1)*
     &    zb45
     &     - 2.D0/3.D0*za13**2*za14**(-1)*za35**(-1)*zb25
     &     + 1.D0/2.D0*za13**2*za14**(-1)*za25**(-1)*zb35
     &     - za12**2*za14**(-1)*za35*za25**(-2)*zb25
     &     )
      ze0 = ze0 + zL1(s23,s15,renscale2)*zi * (
     &     - za15*za23**(-1)*za34**2*za45**(-2)*zb45
     &     + 3.D0/2.D0*za13*za23**(-1)*za34*za45**(-1)*zb45
     &     )
      ze0 = ze0 + zL1(s35,s14,renscale2)*zi * (
     &     - 3.D0/2.D0*za12*za13*za14**(-1)*za25**(-1)*zb25
     &     - za12**2*za14**(-1)*za35*za25**(-2)*zb25
     &     )
      ze0 = ze0 + zL2(s14,s23,renscale2)*zi * (
     &     + 1.D0/2.D0*za14*za35*zb25*za45**(-1)*zb45
     &     )
      ze0 = ze0 + zL2(s15,s23,renscale2)*zi * (
     &     - 1.D0/2.D0*za14*za34*za45**(-1)*zb45*zb24
     &     )
      ze0 = ze0 + zL2(s23,s14,renscale2)*zi * (
     &     + 1.D0/2.D0*za15*za23*za25**(-1)*zb25*zb45
     &     - 1.D0/3.D0*za13*zb25*zb45
     &     )
      ze0 = ze0 + zL2(s35,s14,renscale2)*zi * (
     &     + 1.D0/2.D0*za12*za23*za25**(-1)*zb25*zb24
     &     )
      ze0 = ze0 + zL3(s23,s14,renscale2)*zi * (
     &     - 2.D0/3.D0*za15*za23*zb25**2*zb45
     &     )
      ze0 = ze0 + zCF(s14,s35,s23)*zi * (
     &     + za12**2*za14**(-1)*za35*za25**(-3)
     &     )
      ze0 = ze0 + zCF(s14,s35,s23)*ztree * (
     &     - za12**2*za13**(-2)*za35**2*za25**(-2)
     &     )
      ze0 = ze0 + zCF(s15,s24,s23)*ztree * (
     &     - 1.D0
     &     )
      ze0 = ze0 + zCF(s23,s14,s15)*zi * (
     &     + za15*za23**(-1)*za34**2*za45**(-3)
     &     )
      ze0 = ze0 + zCF(s23,s14,s15)*ztree * (
     &     - za13**(-2)*za15**2*za34**2*za45**(-2)
     &     )
      ze0 = ze0 + zCF(s24,s15,s35)*ztree * (
     &     - 1.D0
     &     )
      ze0 = ze0 + zCF(s35,s14,s24)*ztree * (
     &     - 1.D0
     &     )
      ze0 = ze0 + ztree * (
     &     + 29.D0/18.D0
     &     - 1.D0/2.D0*zzlog( - s35)**2
     &     - 1.D0/2.D0*zzlog( - s24)**2
     &     - 2.D0/3.D0*zzlog( - s23)
     &     - 1.D0/2.D0*zzlog( - s15)**2
     &     + 2.D0/3.D0*zzlog(renscale2)
     &     + zzlog(renscale2)*zzlog( - s35)
     &     + zzlog(renscale2)*zzlog( - s24)
     &     + zzlog(renscale2)*zzlog( - s15)
     &     - 3.D0/2.D0*zzlog(renscale2)**2
     &     )


      elseif(ihel.eq.4)then

      ze2 =  + ztree * (
     &     - 3.D0
     &     )

      ze1 =  + ztree * (
     &     - 29.D0/6.D0
     &     + zzlog( - s35)
     &     + zzlog( - s24)
     &     + zzlog( - s15)
     &     - 3.D0*zzlog(renscale2)
     &     )

      ze0 =  + zi * (
     &     + 1.D0/2.D0*za15**(-1)*za34*s23**(-1)*zb25
     &     - 1.D0/2.D0*za12*za14**(-1)*za15**(-1)*za23**(-1)*za34**2*
     &    za25**(-1)
     &     )
      ze0 = ze0 + zL0(s14,s23,renscale2)*zi * (
     &     + 1.D0/2.D0*za12*za14**(-1)*za15**(-1)*za23**(-1)*za34**2*
     &    za25**(-1)
     &     + za12*za14**(-1)*za15**(-1)*za23**(-1)*za35*za24*za34*
     &    za25**(-2)
     &     )
      ze0 = ze0 + zL0(s14,s24,renscale2)*zi * (
     &     - za13*za14**(-1)*za15**(-1)*za35**(-1)*za34*za25**(-1)*za45
     &     )
      ze0 = ze0 + zL0(s14,s35,renscale2)*zi * (
     &     + 3.D0/2.D0*za14**(-1)*za35**(-1)*za34**2*za25**(-1)
     &     + za14**(-1)*za23*za35**(-1)*za34*za25**(-2)*za45
     &     )
      ze0 = ze0 + zL0(s24,s35,renscale2)*zi * (
     &     + za12**(-1)*za13*za14**(-1)*za35**(-1)*za24*za34*za25**(-1)
     &     + 2.D0*za12**(-1)*za13*za15**(-1)*za35**(-1)*za34
     &     )
      ze0 = ze0 + zL1(s14,s23,renscale2)*zi * (
     &     + 2.D0/3.D0*za15**(-1)*za23**(-1)*za34**2*zb45
     &     + 3.D0/2.D0*za15**(-1)*za34*zb25
     &     - za35*za24*za25**(-2)*zb15
     &     + 3.D0*za13*za15**(-1)*za23**(-1)*za35*za24*za25**(-1)*zb15
     &     - 2.D0*za13*za15**(-1)*za25**(-1)*za45*zb15
     &     )
      ze0 = ze0 + zL1(s14,s35,renscale2)*zi * (
     &     + za12**(-1)*za15*za23**2*za35**(-1)*za24*za25**(-2)*zb12
     &     + za12**(-1)*za14*za23**2*za35**(-1)*za25**(-1)*zb12
     &     - 3.D0*za12**(-1)*za13*za23*za35**(-1)*za24*za25**(-1)*zb12
     &     )
      ze0 = ze0 + zL1(s24,s35,renscale2)*zi * (
     &     - 3.D0*za12**(-1)*za13*za14*za15**(-1)*zb15
     &     + 2.D0*za12**(-1)*za13**2*za15**(-1)*za35**(-1)*za24*zb12
     &     )
      ze0 = ze0 + zL2(s14,s23,renscale2)*zi * (
     &     + 1.D0/3.D0*za34*zb25*zb15
     &     + 1.D0/2.D0*za14*za15**(-1)*za35*zb25*zb15
     &     )
      ze0 = ze0 + zL2(s23,s14,renscale2)*zi * (
     &     - 1.D0/2.D0*za14*za35*za25**(-1)*zb15**2
     &     )
      ze0 = ze0 + zL2(s35,s14,renscale2)*zi * (
     &     - 1.D0/2.D0*za14*za23**2*za35**(-1)*za25**(-1)*zb12**2
     &     )
      ze0 = ze0 + zL3(s14,s23,renscale2)*zi * (
     &     + 2.D0/3.D0*za14*za35*zb25*zb15**2
     &     )
      ze0 = ze0 + zCF(s14,s35,s23)*zi * (
     &     - za14**(-1)*za35*za24**2*za25**(-3)
     &     )
      ze0 = ze0 + zCF(s14,s35,s23)*ztree * (
     &     - za35**2*za24**2*za34**(-2)*za25**(-2)
     &     )
      ze0 = ze0 + zCF(s15,s24,s23)*ztree * (
     &     - 1.D0
     &     )
      ze0 = ze0 + zCF(s23,s14,s15)*zi * (
     &     - za15**(-1)*za23**(-1)*za34**2*za45**(-1)
     &     )
      ze0 = ze0 + zCF(s23,s14,s15)*ztree * (
     &     - 1.D0
     &     )
      ze0 = ze0 + zCF(s24,s15,s35)*ztree * (
     &     - 1.D0
     &     )
      ze0 = ze0 + zCF(s35,s14,s24)*ztree * (
     &     - za12**(-2)*za13**2*za24**2*za34**(-2)
     &     )
      ze0 = ze0 + ztree * (
     &     + 29.D0/18.D0
     &     - 1.D0/2.D0*zzlog( - s35)**2
     &     - 1.D0/2.D0*zzlog( - s24)**2
     &     - 1.D0/2.D0*zzlog( - s15)**2
     &     - 2.D0/3.D0*zzlog( - s14)
     &     + 2.D0/3.D0*zzlog(renscale2)
     &     + zzlog(renscale2)*zzlog( - s35)
     &     + zzlog(renscale2)*zzlog( - s24)
     &     + zzlog(renscale2)*zzlog( - s15)
     &     - 3.D0/2.D0*zzlog(renscale2)**2
     &     )
      elseif(ihel.gt.4)then
         ze2 = (0d0,0d0)
         ze1 = (0d0,0d0)
         ze0 = (0d0,0d0)
      endif

* correction factor to convert from c_gamma prefactor
      ze0 = ze0 - pi**2/12d0*ze2
* correction from finite renormalisations: eq 2.19 Signer's thesis
      ze0 = ze0 - ( 1d0/3d0+4d0/12d0 )*ztree


      if(iorder.eq.-2) zwt = ze2
      if(iorder.eq.-1) zwt = ze1
      if(iorder.eq.0)  zwt = ze0
      zwt=zwt/2d0
      zzC1g1a=zwt

      return
      end
*****************************************************************************
* sub-leading colour (1,0) 1-loop Qqb amplitude

      function zzCt1g1a(iq1,ig,iqb2,iq2,iqb1,ihel,iflip,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      common /order/iorder
      parameter(pi=3.141592653589793238d0)

      i1=iq1
      i2=iqb2
      i3=iq2
      i4=iqb1
      i5=ig

      s12=s5(i2,i4)
      s13=s5(i2,i1)
      s14=s5(i2,i3)
      s15=s5(i2,i5)
      s23=s5(i4,i1)
      s24=s5(i4,i3)
      s25=s5(i4,i5)
      s34=s5(i1,i3)
      s35=s5(i1,i5)
      s45=s5(i3,i5)

      if(iflip.eq.0)then

      za12=za5(i2,i4)
      za13=za5(i2,i1)
      za14=za5(i2,i3)
      za15=za5(i2,i5)
      za23=za5(i4,i1)
      za24=za5(i4,i3)
      za25=za5(i4,i5)
      za34=za5(i1,i3)
      za35=za5(i1,i5)
      za45=za5(i3,i5)

      zb12=zb5(i2,i4)
      zb13=zb5(i2,i1)
      zb14=zb5(i2,i3)
      zb15=zb5(i2,i5)
      zb23=zb5(i4,i1)
      zb24=zb5(i4,i3)
      zb25=zb5(i4,i5)
      zb34=zb5(i1,i3)
      zb35=zb5(i1,i5)
      zb45=zb5(i3,i5)

      elseif(iflip.eq.1)then

      za12=-zb5(i2,i4)
      za13=-zb5(i2,i1)
      za14=-zb5(i2,i3)
      za15=-zb5(i2,i5)
      za23=-zb5(i4,i1)
      za24=-zb5(i4,i3)
      za25=-zb5(i4,i5)
      za34=-zb5(i1,i3)
      za35=-zb5(i1,i5)
      za45=-zb5(i3,i5)

      zb12=-za5(i2,i4)
      zb13=-za5(i2,i1)
      zb14=-za5(i2,i3)
      zb15=-za5(i2,i5)
      zb23=-za5(i4,i1)
      zb24=-za5(i4,i3)
      zb25=-za5(i4,i5)
      zb34=-za5(i1,i3)
      zb35=-za5(i1,i5)
      zb45=-za5(i3,i5)

      endif

      zi=(0d0,1d0)
      ztree=zzC1g0a(i1,i5,i2,i3,i4,ihel,iflip)

      if(ihel.eq.1)then

      zte2 =  + ztree * (
     &     + 2.D0
     &     )

      zte1 =  + zi * (
     &     - zzlog( - s45)*za12**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     - zzlog( - s34)*za12**2*za15**(-1)*za23**(-1)*za45**(-1)
     &     - zzlog( - s25)*za12**2*za15**(-1)*za23**(-1)*za45**(-1)
     &     - zzlog( - s12)*za12**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     + zzlog( - s35)*za12**2*za15**(-1)*za23**(-1)*za45**(-1)
     &     + zzlog( - s24)*za12**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     + zzlog( - s24)*za12**2*za15**(-1)*za23**(-1)*za45**(-1)
     &     + zzlog( - s15)*za12**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     )
      zte1 = zte1 + ztree * (
     &     + 3.D0
     &     + zzlog( - s34)
     &     - zzlog( - s13)
     &     + zzlog( - s12)
     &     - zzlog( - s24)
     &     - zzlog( - s23)
     &     - zzlog( - s14)
     &     + 2.D0*zzlog(renscale2)
     &     )

      zte0 =  + zi * (
     &     - 1.D0/2.D0*za12*za35**(-1)*s14**(-1)*zb45
     &     + 1.D0/2.D0*za12**2*za14**(-1)*za23**(-1)*za35**(-1)*za34*
     &    za45**(-1)
     &     + 1.D0/2.D0*zzlog( - s45)**2*za12**2*za14**(-1)*za35**(-1)*
     &    za25**(-1)
     &     + 1.D0/2.D0*zzlog( - s34)**2*za12**2*za15**(-1)*za23**(-1)*
     &    za45**(-1)
     &     + 1.D0/2.D0*zzlog( - s25)**2*za12**2*za15**(-1)*za23**(-1)*
     &    za45**(-1)
     &     + 1.D0/2.D0*zzlog( - s12)**2*za12**2*za14**(-1)*za35**(-1)*
     &    za25**(-1)
     &     - 1.D0/2.D0*zzlog( - s35)**2*za12**2*za15**(-1)*za23**(-1)*
     &    za45**(-1)
     &     - 1.D0/2.D0*zzlog( - s24)**2*za12**2*za14**(-1)*za35**(-1)*
     &    za25**(-1)
     &     )
      zte0 = zte0 + zi * (
     &     - 1.D0/2.D0*zzlog( - s24)**2*za12**2*za15**(-1)*za23**(-1)*
     &    za45**(-1)
     &     - 1.D0/2.D0*zzlog( - s15)**2*za12**2*za14**(-1)*za35**(-1)*
     &    za25**(-1)
     &     - zzlog(renscale2)*zzlog( - s45)*za12**2*za14**(-1)*
     &    za35**(-1)*za25**(-1)
     &     - zzlog(renscale2)*zzlog( - s34)*za12**2*za15**(-1)*
     &    za23**(-1)*za45**(-1)
     &     - zzlog(renscale2)*zzlog( - s25)*za12**2*za15**(-1)*
     &    za23**(-1)*za45**(-1)
     &     - zzlog(renscale2)*zzlog( - s12)*za12**2*za14**(-1)*
     &    za35**(-1)*za25**(-1)
     &     + zzlog(renscale2)*zzlog( - s35)*za12**2*za15**(-1)*
     &    za23**(-1)*za45**(-1)
     &     )
      zte0 = zte0 + zi * (
     &     + zzlog(renscale2)*zzlog( - s24)*za12**2*za14**(-1)*
     &    za35**(-1)*za25**(-1)
     &     + zzlog(renscale2)*zzlog( - s24)*za12**2*za15**(-1)*
     &    za23**(-1)*za45**(-1)
     &     + zzlog(renscale2)*zzlog( - s15)*za12**2*za14**(-1)*
     &    za35**(-1)*za25**(-1)
     &     )
      zte0 = zte0 + zL0(s15,s14,renscale2)*zi * (
     &     + 2.D0*za12*za35**(-1)*za45**(-1)
     &     + 1.D0/2.D0*za12**2*za15**(-1)*za23**(-1)*za45**(-1)
     &     )
      zte0 = zte0 + zL0(s23,s14,renscale2)*zi * (
     &     + za12*za14*za15**(-1)*za23**(-1)*za25*za45**(-2)
     &     )
      zte0 = zte0 + zL0(s23,s15,renscale2)*zi * (
     &     - za12*za14*za15**(-1)*za23**(-1)*za25*za45**(-2)
     &     )
      zte0 = zte0 + zL1(s23,s14,renscale2)*zi * (
     &     - za15*za24*zb35*za45**(-2)
     &     - 2.D0*za13*za35**(-1)*za25*zb35*za45**(-1)
     &     - 3.D0/2.D0*za12*za35**(-1)*zb45
     &     )
      zte0 = zte0 + zL1(s23,s15,renscale2)*zi * (
     &     + za14**2*za15**(-1)*za35*za24*za34**(-1)*zb34*za45**(-2)
     &     + za14**2*za15**(-1)*za23*za34**(-1)*zb34*za45**(-1)
     &     )
      zte0 = zte0 + zL1(s24,s15,renscale2)*zi * (
     &     + 2.D0*za13**2*za15**(-1)*za35**(-1)*za24*za34**(-1)*zb34
     &     )
      zte0 = zte0 + zL2(s14,s23,renscale2)*zi * (
     &     - 1.D0/2.D0*za15*za23*zb35**2*za45**(-1)
     &     )
      zte0 = zte0 + zL2(s15,s23,renscale2)*zi * (
     &     - 1.D0/2.D0*za14**2*za15**(-1)*za23*zb34**2*za45**(-1)
     &     )
      zte0 = zte0 + zL2(s23,s14,renscale2)*zi * (
     &     + 1.D0/2.D0*za15*za23*za35**(-1)*zb35*zb45
     &     )
      zte0 = zte0 + zCF(s14,s23,s35)*zi * (
     &     - za12**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     )
      zte0 = zte0 + zCF(s14,s35,s23)*zi * (
     &     - za12**2*za15**(-1)*za23**(-1)*za45**(-1)
     &     )
      zte0 = zte0 + zCF(s14,s25,s23)*zi * (
     &     + za13**2*za15**(-1)*za23**(-1)*za35**(-2)*za25**2*
     &    za45**(-1)
     &     )
      zte0 = zte0 + zCF(s15,s24,s23)*zi * (
     &     - za13**2*za15**(-1)*za23**(-1)*za24**2*za34**(-2)*
     &    za45**(-1)
     &     )
      zte0 = zte0 + zCF(s15,s24,s23)*ztree * (
     &     + za12**(-2)*za13**2*za24**2*za34**(-2)
     &     )
      zte0 = zte0 + zCF(s15,s34,s23)*zi * (
     &     - za12**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     + za12**2*za15**(-1)*za23**(-1)*za45**(-1)
     &     )
      zte0 = zte0 + zCF(s15,s34,s23)*ztree * (
     &     - 1.D0
     &     )
      zte0 = zte0 + zCF(s23,s14,s15)*zi * (
     &     - za14**(-1)*za15**2*za35**(-1)*za24**2*za25**(-1)*
     &    za45**(-2)
     &     )
      zte0 = zte0 + zCF(s23,s14,s45)*zi * (
     &     + za12**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     )
      zte0 = zte0 + zCF(s23,s15,s14)*zi * (
     &     - za14**2*za15**(-1)*za23**(-1)*za25**2*za45**(-3)
     &     )
      zte0 = zte0 + zCF(s24,s15,s35)*ztree * (
     &     + 1.D0
     &     )
      zte0 = zte0 + zCF(s24,s15,s13)*zi * (
     &     - za13**2*za15**(-1)*za23**(-1)*za35**(-2)*za25**2*
     &    za45**(-1)
     &     )
      zte0 = zte0 + zCF(s24,s13,s35)*zi * (
     &     - za12**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     )
      zte0 = zte0 + zCF(s35,s14,s24)*zi * (
     &     - za12**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     )
      zte0 = zte0 + zCF(s35,s14,s24)*ztree * (
     &     + 1.D0
     &     )
      zte0 = zte0 + zCF(s35,s14,s12)*zi * (
     &     + za12**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     - za12**2*za15**(-1)*za23**(-1)*za45**(-1)
     &     )
      zte0 = zte0 + zCF(s35,s14,s12)*ztree * (
     &     - 1.D0
     &     )
      zte0 = zte0 + zCF(s12,s35,s45)*zi * (
     &     - za12**2*za15**(-1)*za23**(-1)*za45**(-1)
     &     )
      zte0 = zte0 + zCF(s12,s34,s35)*zi * (
     &     + za12**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     )
      zte0 = zte0 + zCF(s12,s45,s35)*ztree * (
     &     - 1.D0
     &     )
      zte0 = zte0 + zCF(s12,s45,s34)*zi * (
     &     + za12**2*za15**(-1)*za23**(-1)*za45**(-1)
     &     )
      zte0 = zte0 + zCF(s13,s24,s25)*zi * (
     &     - za14**(-1)*za15**2*za35**(-1)*za24**2*za25**(-1)*
     &    za45**(-2)
     &     )
      zte0 = zte0 + zCF(s13,s25,s45)*zi * (
     &     + za12**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     + za12**2*za15**(-1)*za23**(-1)*za45**(-1)
     &     )
      zte0 = zte0 + zCF(s13,s45,s24)*zi * (
     &     - za12**2*za15**(-1)*za23**(-1)*za45**(-1)
     &     )
      zte0 = zte0 + zCF(s13,s45,s25)*ztree * (
     &     + 1.D0
     &     )
      zte0 = zte0 + zCF(s25,s14,s13)*zi * (
     &     + za13**2*za15**(-1)*za23**(-1)*za24**2*za34**(-2)*
     &    za45**(-1)
     &     )
      zte0 = zte0 + zCF(s25,s14,s13)*ztree * (
     &     + za12**(-2)*za13**2*za24**2*za34**(-2)
     &     )
      zte0 = zte0 + zCF(s25,s14,s34)*ztree * (
     &     - 1.D0
     &     )
      zte0 = zte0 + zCF(s34,s15,s12)*zi * (
     &     + za12**2*za15**(-1)*za23**(-1)*za45**(-1)
     &     )
      zte0 = zte0 + zCF(s34,s15,s25)*zi * (
     &     - za12**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     )
      zte0 = zte0 + zCF(s34,s15,s25)*ztree * (
     &     - 1.D0
     &     )
      zte0 = zte0 + zCF(s34,s12,s25)*zi * (
     &     + za12**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     )
      zte0 = zte0 + zCF(s45,s12,s23)*ztree * (
     &     - 1.D0
     &     )
      zte0 = zte0 + zCF(s45,s13,s23)*zi * (
     &     + za12**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     )
      zte0 = zte0 + zCF(s45,s13,s23)*ztree * (
     &     + 1.D0
     &     )
      zte0 = zte0 + ztree * (
     &     + 13.D0/2.D0
     &     - 1.D0/2.D0*zzlog( - s34)**2
     &     + 1.D0/2.D0*zzlog( - s13)**2
     &     - 1.D0/2.D0*zzlog( - s12)**2
     &     + 1.D0/2.D0*zzlog( - s24)**2
     &     - 3.D0/2.D0*zzlog( - s23)
     &     + 1.D0/2.D0*zzlog( - s23)**2
     &     - 3.D0/2.D0*zzlog( - s14)
     &     + 1.D0/2.D0*zzlog( - s14)**2
     &     + 3.D0*zzlog(renscale2)
     &     + zzlog(renscale2)*zzlog( - s34)
     &     - zzlog(renscale2)*zzlog( - s13)
     &     + zzlog(renscale2)*zzlog( - s12)
     &     - zzlog(renscale2)*zzlog( - s24)
     &     )
      zte0 = zte0 + ztree * (
     &     - zzlog(renscale2)*zzlog( - s23)
     &     - zzlog(renscale2)*zzlog( - s14)
     &     + zzlog(renscale2)**2
     &     )

      elseif(ihel.eq.2)then

      zte2 =  + ztree * (
     &     + 2.D0
     &     )

      zte1 =  + zi * (
     &     + zzlog( - s45)*za14**(-1)*za35**(-1)*za24**2*za25**(-1)
     &     + zzlog( - s34)*za15**(-1)*za23**(-1)*za24**2*za45**(-1)
     &     + zzlog( - s25)*za15**(-1)*za23**(-1)*za24**2*za45**(-1)
     &     + zzlog( - s12)*za14**(-1)*za35**(-1)*za24**2*za25**(-1)
     &     - zzlog( - s35)*za15**(-1)*za23**(-1)*za24**2*za45**(-1)
     &     - zzlog( - s24)*za14**(-1)*za35**(-1)*za24**2*za25**(-1)
     &     - zzlog( - s24)*za15**(-1)*za23**(-1)*za24**2*za45**(-1)
     &     - zzlog( - s15)*za14**(-1)*za35**(-1)*za24**2*za25**(-1)
     &     )
      zte1 = zte1 + ztree * (
     &     + 3.D0
     &     + zzlog( - s34)
     &     - zzlog( - s13)
     &     + zzlog( - s12)
     &     - zzlog( - s24)
     &     - zzlog( - s23)
     &     - zzlog( - s14)
     &     + 2.D0*zzlog(renscale2)
     &     )

      zte0 =  + zi * (
     &     + 1.D0/2.D0*za15**(-1)*s14**(-1)*zb35**2*za45*zb23**(-1)
     &     + 1.D0/2.D0*za35**(-1)*za24*s14**(-1)*zb15
     &     + 1.D0/2.D0*za13*za14**(-1)*za15**(-1)*za23**(-1)*za35**(-1)
     &    *za24**2
     &     - 1.D0/2.D0*zzlog( - s45)**2*za14**(-1)*za35**(-1)*za24**2*
     &    za25**(-1)
     &     - 1.D0/2.D0*zzlog( - s34)**2*za15**(-1)*za23**(-1)*za24**2*
     &    za45**(-1)
     &     - 1.D0/2.D0*zzlog( - s25)**2*za15**(-1)*za23**(-1)*za24**2*
     &    za45**(-1)
     &     - 1.D0/2.D0*zzlog( - s12)**2*za14**(-1)*za35**(-1)*za24**2*
     &    za25**(-1)
     &     + 1.D0/2.D0*zzlog( - s35)**2*za15**(-1)*za23**(-1)*za24**2*
     &    za45**(-1)
     &     )
      zte0 = zte0 + zi * (
     &     + 1.D0/2.D0*zzlog( - s24)**2*za14**(-1)*za35**(-1)*za24**2*
     &    za25**(-1)
     &     + 1.D0/2.D0*zzlog( - s24)**2*za15**(-1)*za23**(-1)*za24**2*
     &    za45**(-1)
     &     + 1.D0/2.D0*zzlog( - s15)**2*za14**(-1)*za35**(-1)*za24**2*
     &    za25**(-1)
     &     + zzlog(renscale2)*zzlog( - s45)*za14**(-1)*za35**(-1)*
     &    za24**2*za25**(-1)
     &     + zzlog(renscale2)*zzlog( - s34)*za15**(-1)*za23**(-1)*
     &    za24**2*za45**(-1)
     &     + zzlog(renscale2)*zzlog( - s25)*za15**(-1)*za23**(-1)*
     &    za24**2*za45**(-1)
     &     + zzlog(renscale2)*zzlog( - s12)*za14**(-1)*za35**(-1)*
     &    za24**2*za25**(-1)
     &     )
      zte0 = zte0 + zi * (
     &     - zzlog(renscale2)*zzlog( - s35)*za15**(-1)*za23**(-1)*
     &    za24**2*za45**(-1)
     &     - zzlog(renscale2)*zzlog( - s24)*za14**(-1)*za35**(-1)*
     &    za24**2*za25**(-1)
     &     - zzlog(renscale2)*zzlog( - s24)*za15**(-1)*za23**(-1)*
     &    za24**2*za45**(-1)
     &     - zzlog(renscale2)*zzlog( - s15)*za14**(-1)*za35**(-1)*
     &    za24**2*za25**(-1)
     &     )
      zte0 = zte0 + zL0(s12,s23,renscale2)*zi * (
     &     + 2.D0*za15**(-1)*za35**(-1)*za24
     &     )
      zte0 = zte0 + zL1(s23,s14,renscale2)*zi * (
     &     + za15**(-1)*za24*zb35
     &     + 3.D0/2.D0*za35**(-1)*za24*zb15
     &     + 2.D0*za14*za15**(-1)*za35**(-1)*za25*zb15
     &     )
      zte0 = zte0 + zL1(s34,s12,renscale2)*zi * (
     &     - 2.D0*za15**(-1)*za35**(-1)*za34*za25*zb35
     &     )
      zte0 = zte0 + zL2(s23,s14,renscale2)*zi * (
     &     + 1.D0/2.D0*za15**(-1)*za23*za35**(-1)*zb35*s35*za45
     &     - 1.D0/2.D0*za15**(-1)*za23*za35**(-1)*s15*zb35*za45
     &     )
      zte0 = zte0 + zCF(s14,s23,s35)*zi * (
     &     + za14**(-1)*za35**(-1)*za24**2*za25**(-1)
     &     )
      zte0 = zte0 + zCF(s14,s35,s23)*zi * (
     &     + za15**(-1)*za23**(-1)*za24**2*za45**(-1)
     &     )
      zte0 = zte0 + zCF(s14,s25,s23)*zi * (
     &     - za15**(-1)*za23**(-1)*za35**(-2)*za34**2*za25**2*
     &    za45**(-1)
     &     )
      zte0 = zte0 + zCF(s15,s24,s23)*zi * (
     &     + za15**(-1)*za23**(-1)*za24**2*za45**(-1)
     &     )
      zte0 = zte0 + zCF(s15,s24,s23)*ztree * (
     &     + 1.D0
     &     )
      zte0 = zte0 + zCF(s15,s34,s23)*zi * (
     &     + za14**(-1)*za35**(-1)*za24**2*za25**(-1)
     &     - za15**(-1)*za23**(-1)*za24**2*za45**(-1)
     &     )
      zte0 = zte0 + zCF(s15,s34,s23)*ztree * (
     &     - 1.D0
     &     )
      zte0 = zte0 + zCF(s23,s14,s15)*zi * (
     &     + za14**(-1)*za35**(-1)*za24**2*za25**(-1)
     &     )
      zte0 = zte0 + zCF(s23,s14,s45)*zi * (
     &     - za12**2*za14**(-1)*za15**(-2)*za35**(-1)*za25**(-1)*
     &    za45**2
     &     )
      zte0 = zte0 + zCF(s23,s15,s14)*zi * (
     &     + za15**(-1)*za23**(-1)*za24**2*za45**(-1)
     &     )
      zte0 = zte0 + zCF(s24,s15,s35)*ztree * (
     &     + 1.D0
     &     )
      zte0 = zte0 + zCF(s24,s15,s13)*zi * (
     &     + za15**(-1)*za23**(-1)*za24**2*za45**(-1)
     &     )
      zte0 = zte0 + zCF(s24,s13,s35)*zi * (
     &     + za14**(-1)*za35**(-1)*za24**2*za25**(-1)
     &     )
      zte0 = zte0 + zCF(s35,s14,s24)*zi * (
     &     + za14**(-1)*za35**(-1)*za24**2*za25**(-1)
     &     )
      zte0 = zte0 + zCF(s35,s14,s24)*ztree * (
     &     + 1.D0
     &     )
      zte0 = zte0 + zCF(s35,s14,s12)*zi * (
     &     - za14**(-1)*za35**(-1)*za24**2*za25**(-1)
     &     + za15**(-1)*za23**(-1)*za24**2*za45**(-1)
     &     )
      zte0 = zte0 + zCF(s35,s14,s12)*ztree * (
     &     - 1.D0
     &     )
      zte0 = zte0 + zCF(s12,s35,s45)*zi * (
     &     + za15**(-1)*za23**(-1)*za24**2*za45**(-1)
     &     )
      zte0 = zte0 + zCF(s12,s34,s35)*zi * (
     &     - za14**(-1)*za35**(-1)*za24**2*za25**(-1)
     &     )
      zte0 = zte0 + zCF(s12,s45,s35)*ztree * (
     &     - 1.D0
     &     )
      zte0 = zte0 + zCF(s12,s45,s34)*zi * (
     &     - za15**(-1)*za23**(-1)*za35**(-2)*za34**2*za25**2*
     &    za45**(-1)
     &     )
      zte0 = zte0 + zCF(s13,s24,s25)*zi * (
     &     + za14**(-1)*za35**(-1)*za24**2*za25**(-1)
     &     )
      zte0 = zte0 + zCF(s13,s25,s45)*zi * (
     &     - za14**(-1)*za35**(-1)*za24**2*za25**(-1)
     &     - za15**(-1)*za23**(-1)*za24**2*za45**(-1)
     &     )
      zte0 = zte0 + zCF(s13,s45,s24)*zi * (
     &     + za15**(-1)*za23**(-1)*za24**2*za45**(-1)
     &     )
      zte0 = zte0 + zCF(s13,s45,s25)*ztree * (
     &     + 1.D0
     &     )
      zte0 = zte0 + zCF(s25,s14,s13)*zi * (
     &     - za15**(-1)*za23**(-1)*za24**2*za45**(-1)
     &     )
      zte0 = zte0 + zCF(s25,s14,s13)*ztree * (
     &     + 1.D0
     &     )
      zte0 = zte0 + zCF(s25,s14,s34)*ztree * (
     &     - za12**2*za13**(-2)*za24**(-2)*za34**2
     &     )
      zte0 = zte0 + zCF(s34,s15,s12)*zi * (
     &     - za15**(-1)*za23**(-1)*za24**2*za45**(-1)
     &     )
      zte0 = zte0 + zCF(s34,s15,s25)*zi * (
     &     + za14**(-1)*za35**(-1)*za24**2*za25**(-1)
     &     )
      zte0 = zte0 + zCF(s34,s15,s25)*ztree * (
     &     - 1.D0
     &     )
      zte0 = zte0 + zCF(s34,s12,s25)*zi * (
     &     - za12**2*za14**(-1)*za15**(-2)*za35**(-1)*za25**(-1)*
     &    za45**2
     &     )
      zte0 = zte0 + zCF(s45,s12,s23)*ztree * (
     &     - za12**2*za13**(-2)*za24**(-2)*za34**2
     &     )
      zte0 = zte0 + zCF(s45,s13,s23)*zi * (
     &     - za14**(-1)*za35**(-1)*za24**2*za25**(-1)
     &     )
      zte0 = zte0 + zCF(s45,s13,s23)*ztree * (
     &     + 1.D0
     &     )
      zte0 = zte0 + ztree * (
     &     + 13.D0/2.D0
     &     - 1.D0/2.D0*zzlog( - s34)**2
     &     + 1.D0/2.D0*zzlog( - s13)**2
     &     - 1.D0/2.D0*zzlog( - s12)**2
     &     + 1.D0/2.D0*zzlog( - s24)**2
     &     - 3.D0/2.D0*zzlog( - s23)
     &     + 1.D0/2.D0*zzlog( - s23)**2
     &     - 3.D0/2.D0*zzlog( - s14)
     &     + 1.D0/2.D0*zzlog( - s14)**2
     &     + 3.D0*zzlog(renscale2)
     &     + zzlog(renscale2)*zzlog( - s34)
     &     - zzlog(renscale2)*zzlog( - s13)
     &     + zzlog(renscale2)*zzlog( - s12)
     &     - zzlog(renscale2)*zzlog( - s24)
     &     )
      zte0 = zte0 + ztree * (
     &     - zzlog(renscale2)*zzlog( - s23)
     &     - zzlog(renscale2)*zzlog( - s14)
     &     + zzlog(renscale2)**2
     &     )

      elseif(ihel.eq.3)then

      zte2 =  + ztree * (
     &     + 2.D0
     &     )

      zte1 =  + zi * (
     &     + zzlog( - s45)*za13**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     + zzlog( - s34)*za13**2*za15**(-1)*za23**(-1)*za45**(-1)
     &     + zzlog( - s25)*za13**2*za15**(-1)*za23**(-1)*za45**(-1)
     &     + zzlog( - s12)*za13**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     - zzlog( - s35)*za13**2*za15**(-1)*za23**(-1)*za45**(-1)
     &     - zzlog( - s24)*za13**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     - zzlog( - s24)*za13**2*za15**(-1)*za23**(-1)*za45**(-1)
     &     - zzlog( - s15)*za13**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     )
      zte1 = zte1 + ztree * (
     &     + 3.D0
     &     + zzlog( - s34)
     &     - zzlog( - s13)
     &     + zzlog( - s12)
     &     - zzlog( - s24)
     &     - zzlog( - s23)
     &     - zzlog( - s14)
     &     + 2.D0*zzlog(renscale2)
     &     )

      zte0 =  + zi * (
     &     + 1.D0/2.D0*za15*s14**(-1)*zb25**2*za45**(-1)*zb23**(-1)
     &     + 1.D0/2.D0*za15*za23*s14**(-1)*za25**(-1)*zb25*za45**(-1)
     &     - 1.D0/2.D0*za13*za23*za35**(-1)*s14**(-1)*za25**(-1)*zb24
     &     - 1.D0/2.D0*za13**2*za14**(-1)*za23**(-1)*za35**(-1)*za34*
     &    za45**(-1)
     &     - 1.D0/2.D0*zzlog( - s45)**2*za13**2*za14**(-1)*za35**(-1)*
     &    za25**(-1)
     &     - 1.D0/2.D0*zzlog( - s34)**2*za13**2*za15**(-1)*za23**(-1)*
     &    za45**(-1)
     &     - 1.D0/2.D0*zzlog( - s25)**2*za13**2*za15**(-1)*za23**(-1)*
     &    za45**(-1)
     &     - 1.D0/2.D0*zzlog( - s12)**2*za13**2*za14**(-1)*za35**(-1)*
     &    za25**(-1)
     &     + 1.D0/2.D0*zzlog( - s35)**2*za13**2*za15**(-1)*za23**(-1)*
     &    za45**(-1)
     &     )
      zte0 = zte0 + zi * (
     &     + 1.D0/2.D0*zzlog( - s24)**2*za13**2*za14**(-1)*za35**(-1)*
     &    za25**(-1)
     &     + 1.D0/2.D0*zzlog( - s24)**2*za13**2*za15**(-1)*za23**(-1)*
     &    za45**(-1)
     &     + 1.D0/2.D0*zzlog( - s15)**2*za13**2*za14**(-1)*za35**(-1)*
     &    za25**(-1)
     &     + zzlog(renscale2)*zzlog( - s45)*za13**2*za14**(-1)*
     &    za35**(-1)*za25**(-1)
     &     + zzlog(renscale2)*zzlog( - s34)*za13**2*za15**(-1)*
     &    za23**(-1)*za45**(-1)
     &     + zzlog(renscale2)*zzlog( - s25)*za13**2*za15**(-1)*
     &    za23**(-1)*za45**(-1)
     &     + zzlog(renscale2)*zzlog( - s12)*za13**2*za14**(-1)*
     &    za35**(-1)*za25**(-1)
     &     )
      zte0 = zte0 + zi * (
     &     - zzlog(renscale2)*zzlog( - s35)*za13**2*za15**(-1)*
     &    za23**(-1)*za45**(-1)
     &     - zzlog(renscale2)*zzlog( - s24)*za13**2*za14**(-1)*
     &    za35**(-1)*za25**(-1)
     &     - zzlog(renscale2)*zzlog( - s24)*za13**2*za15**(-1)*
     &    za23**(-1)*za45**(-1)
     &     - zzlog(renscale2)*zzlog( - s15)*za13**2*za14**(-1)*
     &    za35**(-1)*za25**(-1)
     &     )
      zte0 = zte0 + zL0(s15,s14,renscale2)*zi * (
     &     - 1.D0/2.D0*za13**2*za15**(-1)*za23**(-1)*za45**(-1)
     &     )
      zte0 = zte0 + zL0(s23,s14,renscale2)*zi * (
     &     + za13*za14**(-1)*za15*za23*za35**(-1)*za25**(-2)
     &     - za13*za14*za15**(-1)*za23**(-1)*za35*za45**(-2)
     &     )
      zte0 = zte0 + zL0(s23,s15,renscale2)*zi * (
     &     + za14*za35*s23**(-1)*zb25*za45**(-2)
     &     )
      zte0 = zte0 + zL0(s35,s14,renscale2)*zi * (
     &     - za15*za23*s14**(-1)*za25**(-2)*zb45
     &     - 2.D0*za13*za23*za35**(-1)*za24**(-1)*za25**(-1)
     &     - 1.D0/2.D0*za13**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     )
      zte0 = zte0 + zL0(s12,s14,renscale2)*zi * (
     &     - 2.D0*za13*za35**(-1)*za24**(-1)*za34*za45**(-1)
     &     )
      zte0 = zte0 + zL0(s34,s15,renscale2)*zi * (
     &     + 2.D0*za13*za14*za15**(-1)*za24**(-1)*za45**(-1)
     &     )
      zte0 = zte0 + zL1(s23,s14,renscale2)*zi * (
     &     + 1.D0/2.D0*za14**(-1)*za15**2*za23*za35**(-1)*za34*
     &    za25**(-1)*zb25*za45**(-1)
     &     - za15*za34*zb25*za45**(-2)
     &     - za15*za23*za25**(-2)*zb45
     &     - 2.D0*za15*za23*za25**(-1)*zb25*za45**(-1)
     &     - 2.D0*za13*zb25*za45**(-1)
     &     )
      zte0 = zte0 + zL1(s23,s15,renscale2)*zi * (
     &     + 2.D0*za14**2*za15**(-1)*za23*za24**(-1)*za45**(-1)*zb24
     &     - za14**2*za35*s23**(-1)*za45**(-2)*zb15*zb24
     &     )
      zte0 = zte0 + zL1(s35,s14,renscale2)*zi * (
     &     - za15*za23**2*s14**(-1)*za25**(-2)*zb35*zb24
     &     + 2.D0*za14*za23**2*za35**(-1)*za24**(-1)*za25**(-1)*zb24
     &     )
      zte0 = zte0 + zL1(s35,s12,renscale2)*zi * (
     &     + 2.D0*za14*za24**(-1)*za34*za45**(-1)*zb45
     &     )
      zte0 = zte0 + zL1(s34,s15,renscale2)*zi * (
     &     - 2.D0*za12*za23*za24**(-1)*za25**(-1)*zb25
     &     )
      zte0 = zte0 + zL1(s34,s12,renscale2)*zi * (
     &     - 2.D0*za12*za35*za25**(-1)*zb25*za45**(-1)
     &     )
      zte0 = zte0 + zL2(s15,s23,renscale2)*zi * (
     &     + 1.D0/2.D0*za14**2*za15**(-1)*za23*za45**(-1)*zb24**2
     &     )
      zte0 = zte0 + zL2(s23,s14,renscale2)*zi * (
     &     + 1.D0/2.D0*za15*za23*za25**(-1)*zb25*za45**(-1)*s34
     &     - 1.D0/2.D0*za15*za23*za25**(-1)*zb25*za45**(-1)*s12
     &     )
      zte0 = zte0 + zL2(s35,s14,renscale2)*zi * (
     &     + 1.D0/2.D0*za14*za23**2*za35**(-1)*za25**(-1)*zb24**2
     &     )
      zte0 = zte0 + zCF(s14,s23,s35)*zi * (
     &     + za14**(-1)*za15**2*za23**2*za35**(-1)*za25**(-3)
     &     )
      zte0 = zte0 + zCF(s14,s35,s23)*zi * (
     &     + za12**2*za15**(-1)*za23**(-1)*za35**2*za25**(-2)*
     &    za45**(-1)
     &     )
      zte0 = zte0 + zCF(s14,s25,s23)*zi * (
     &     - za13**2*za15**(-1)*za23**(-1)*za45**(-1)
     &     )
      zte0 = zte0 + zCF(s15,s24,s23)*zi * (
     &     + za13**2*za15**(-1)*za23**(-1)*za45**(-1)
     &     )
      zte0 = zte0 + zCF(s15,s24,s23)*ztree * (
     &     + 1.D0
     &     )
      zte0 = zte0 + zCF(s15,s34,s23)*zi * (
     &     + za12**2*za14**(-1)*za35**(-1)*za24**(-2)*za34**2*
     &    za25**(-1)
     &     - za12**2*za15**(-1)*za23**(-1)*za24**(-2)*za34**2*
     &    za45**(-1)
     &     )
      zte0 = zte0 + zCF(s15,s34,s23)*ztree * (
     &     - za12**2*za13**(-2)*za24**(-2)*za34**2
     &     )
      zte0 = zte0 + zCF(s23,s14,s15)*zi * (
     &     + za14**(-1)*za15**2*za35**(-1)*za34**2*za25**(-1)*
     &    za45**(-2)
     &     )
      zte0 = zte0 + zCF(s23,s14,s45)*zi * (
     &     - za13**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     )
      zte0 = zte0 + zCF(s23,s15,s14)*zi * (
     &     + za14**2*za15**(-1)*za23**(-1)*za35**2*za45**(-3)
     &     )
      zte0 = zte0 + zCF(s24,s15,s35)*ztree * (
     &     + 1.D0
     &     )
      zte0 = zte0 + zCF(s24,s15,s13)*zi * (
     &     + za13**2*za15**(-1)*za23**(-1)*za45**(-1)
     &     )
      zte0 = zte0 + zCF(s24,s13,s35)*zi * (
     &     + za13**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     )
      zte0 = zte0 + zCF(s35,s14,s24)*zi * (
     &     + za13**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     )
      zte0 = zte0 + zCF(s35,s14,s24)*ztree * (
     &     + 1.D0
     &     )
      zte0 = zte0 + zCF(s35,s14,s12)*zi * (
     &     - za12**2*za14**(-1)*za35**(-1)*za24**(-2)*za34**2*
     &    za25**(-1)
     &     + za12**2*za15**(-1)*za23**(-1)*za24**(-2)*za34**2*
     &    za45**(-1)
     &     )
      zte0 = zte0 + zCF(s35,s14,s12)*ztree * (
     &     - za12**2*za13**(-2)*za24**(-2)*za34**2
     &     )
      zte0 = zte0 + zCF(s12,s35,s45)*zi * (
     &     + za13**2*za15**(-1)*za23**(-1)*za45**(-1)
     &     )
      zte0 = zte0 + zCF(s12,s34,s35)*zi * (
     &     - za14**(-1)*za15**2*za35**(-1)*za34**2*za25**(-1)*
     &    za45**(-2)
     &     )
      zte0 = zte0 + zCF(s12,s45,s35)*ztree * (
     &     - 1.D0
     &     )
      zte0 = zte0 + zCF(s12,s45,s34)*zi * (
     &     - za13**2*za15**(-1)*za23**(-1)*za45**(-1)
     &     )
      zte0 = zte0 + zCF(s13,s24,s25)*zi * (
     &     + za13**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     )
      zte0 = zte0 + zCF(s13,s25,s45)*zi * (
     &     - za13**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     - za13**2*za15**(-1)*za23**(-1)*za45**(-1)
     &     )
      zte0 = zte0 + zCF(s13,s45,s24)*zi * (
     &     + za13**2*za15**(-1)*za23**(-1)*za45**(-1)
     &     )
      zte0 = zte0 + zCF(s13,s45,s25)*ztree * (
     &     + 1.D0
     &     )
      zte0 = zte0 + zCF(s25,s14,s13)*zi * (
     &     - za13**2*za15**(-1)*za23**(-1)*za45**(-1)
     &     )
      zte0 = zte0 + zCF(s25,s14,s13)*ztree * (
     &     + 1.D0
     &     )
      zte0 = zte0 + zCF(s25,s14,s34)*ztree * (
     &     - 1.D0
     &     )
      zte0 = zte0 + zCF(s34,s15,s12)*zi * (
     &     - za12**2*za15**(-1)*za23**(-1)*za35**2*za25**(-2)*
     &    za45**(-1)
     &     )
      zte0 = zte0 + zCF(s34,s15,s25)*zi * (
     &     + za13**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     )
      zte0 = zte0 + zCF(s34,s15,s25)*ztree * (
     &     - 1.D0
     &     )
      zte0 = zte0 + zCF(s34,s12,s25)*zi * (
     &     - za13**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     )
      zte0 = zte0 + zCF(s45,s12,s23)*ztree * (
     &     - 1.D0
     &     )
      zte0 = zte0 + zCF(s45,s13,s23)*zi * (
     &     - za13**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     )
      zte0 = zte0 + zCF(s45,s13,s23)*ztree * (
     &     + 1.D0
     &     )
      zte0 = zte0 + ztree * (
     &     + 13.D0/2.D0
     &     - 1.D0/2.D0*zzlog( - s34)**2
     &     + 1.D0/2.D0*zzlog( - s13)**2
     &     - 1.D0/2.D0*zzlog( - s12)**2
     &     + 1.D0/2.D0*zzlog( - s24)**2
     &     - 3.D0/2.D0*zzlog( - s23)
     &     + 1.D0/2.D0*zzlog( - s23)**2
     &     - 3.D0/2.D0*zzlog( - s14)
     &     + 1.D0/2.D0*zzlog( - s14)**2
     &     + 3.D0*zzlog(renscale2)
     &     + zzlog(renscale2)*zzlog( - s34)
     &     - zzlog(renscale2)*zzlog( - s13)
     &     + zzlog(renscale2)*zzlog( - s12)
     &     - zzlog(renscale2)*zzlog( - s24)
     &     )
      zte0 = zte0 + ztree * (
     &     - zzlog(renscale2)*zzlog( - s23)
     &     - zzlog(renscale2)*zzlog( - s14)
     &     + zzlog(renscale2)**2
     &     )


      elseif(ihel.eq.4)then


      zte2 =  + ztree * (
     &     + 2.D0
     &     )

      zte1 =  + zi * (
     &     - zzlog( - s45)*za14**(-1)*za35**(-1)*za34**2*za25**(-1)
     &     - zzlog( - s34)*za15**(-1)*za23**(-1)*za34**2*za45**(-1)
     &     - zzlog( - s25)*za15**(-1)*za23**(-1)*za34**2*za45**(-1)
     &     - zzlog( - s12)*za14**(-1)*za35**(-1)*za34**2*za25**(-1)
     &     + zzlog( - s35)*za15**(-1)*za23**(-1)*za34**2*za45**(-1)
     &     + zzlog( - s24)*za14**(-1)*za35**(-1)*za34**2*za25**(-1)
     &     + zzlog( - s24)*za15**(-1)*za23**(-1)*za34**2*za45**(-1)
     &     + zzlog( - s15)*za14**(-1)*za35**(-1)*za34**2*za25**(-1)
     &     )
      zte1 = zte1 + ztree * (
     &     + 3.D0
     &     + zzlog( - s34)
     &     - zzlog( - s13)
     &     + zzlog( - s12)
     &     - zzlog( - s24)
     &     - zzlog( - s23)
     &     - zzlog( - s14)
     &     + 2.D0*zzlog(renscale2)
     &     )

      zte0 =  + zi * (
     &     + 1.D0/2.D0*za15**(-1)*za34*s23**(-1)*zb25
     &     - 1.D0/2.D0*za12*za14**(-1)*za15**(-1)*za23**(-1)*za34**2*
     &    za25**(-1)
     &     + 1.D0/2.D0*zzlog( - s45)**2*za14**(-1)*za35**(-1)*za34**2*
     &    za25**(-1)
     &     + 1.D0/2.D0*zzlog( - s34)**2*za15**(-1)*za23**(-1)*za34**2*
     &    za45**(-1)
     &     + 1.D0/2.D0*zzlog( - s25)**2*za15**(-1)*za23**(-1)*za34**2*
     &    za45**(-1)
     &     + 1.D0/2.D0*zzlog( - s12)**2*za14**(-1)*za35**(-1)*za34**2*
     &    za25**(-1)
     &     - 1.D0/2.D0*zzlog( - s35)**2*za15**(-1)*za23**(-1)*za34**2*
     &    za45**(-1)
     &     - 1.D0/2.D0*zzlog( - s24)**2*za14**(-1)*za35**(-1)*za34**2*
     &    za25**(-1)
     &     )
      zte0 = zte0 + zi * (
     &     - 1.D0/2.D0*zzlog( - s24)**2*za15**(-1)*za23**(-1)*za34**2*
     &    za45**(-1)
     &     - 1.D0/2.D0*zzlog( - s15)**2*za14**(-1)*za35**(-1)*za34**2*
     &    za25**(-1)
     &     - zzlog(renscale2)*zzlog( - s45)*za14**(-1)*za35**(-1)*
     &    za34**2*za25**(-1)
     &     - zzlog(renscale2)*zzlog( - s34)*za15**(-1)*za23**(-1)*
     &    za34**2*za45**(-1)
     &     - zzlog(renscale2)*zzlog( - s25)*za15**(-1)*za23**(-1)*
     &    za34**2*za45**(-1)
     &     - zzlog(renscale2)*zzlog( - s12)*za14**(-1)*za35**(-1)*
     &    za34**2*za25**(-1)
     &     + zzlog(renscale2)*zzlog( - s35)*za15**(-1)*za23**(-1)*
     &    za34**2*za45**(-1)
     &     )
      zte0 = zte0 + zi * (
     &     + zzlog(renscale2)*zzlog( - s24)*za14**(-1)*za35**(-1)*
     &    za34**2*za25**(-1)
     &     + zzlog(renscale2)*zzlog( - s24)*za15**(-1)*za23**(-1)*
     &    za34**2*za45**(-1)
     &     + zzlog(renscale2)*zzlog( - s15)*za14**(-1)*za35**(-1)*
     &    za34**2*za25**(-1)
     &     )
      zte0 = zte0 + zL0(s14,s23,renscale2)*zi * (
     &     - za14**(-1)*za23*za35**(-1)*za34*za25**(-2)*za45
     &     )
      zte0 = zte0 + zL0(s14,s35,renscale2)*zi * (
     &     + za14**(-1)*za23*za35**(-1)*za34*za25**(-2)*za45
     &     )
      zte0 = zte0 + zL0(s35,s23,renscale2)*zi * (
     &     + 1.D0/2.D0*za14**(-1)*za35**(-1)*za34**2*za25**(-1)
     &     - 2.D0*za15**(-1)*za34*za25**(-1)
     &     )
      zte0 = zte0 + zL1(s14,s23,renscale2)*zi * (
     &     + 3.D0/2.D0*za15**(-1)*za34*zb25
     &     - za35*za24*za25**(-2)*zb15
     &     - 2.D0*za13*za15**(-1)*za25**(-1)*za45*zb15
     &     )
      zte0 = zte0 + zL1(s14,s35,renscale2)*zi * (
     &     + za12**(-1)*za15*za23**2*za35**(-1)*za24*za25**(-2)*zb12
     &     + za12**(-1)*za14*za23**2*za35**(-1)*za25**(-1)*zb12
     &     )
      zte0 = zte0 + zL1(s24,s35,renscale2)*zi * (
     &     + 2.D0*za12**(-1)*za13**2*za15**(-1)*za35**(-1)*za24*zb12
     &     )
      zte0 = zte0 + zL2(s14,s23,renscale2)*zi * (
     &     + 1.D0/2.D0*za14*za15**(-1)*za35*zb25*zb15
     &     )
      zte0 = zte0 + zL2(s23,s14,renscale2)*zi * (
     &     - 1.D0/2.D0*za14*za35*za25**(-1)*zb15**2
     &     )
      zte0 = zte0 + zL2(s35,s14,renscale2)*zi * (
     &     - 1.D0/2.D0*za14*za23**2*za35**(-1)*za25**(-1)*zb12**2
     &     )
      zte0 = zte0 + zCF(s14,s23,s35)*zi * (
     &     - za14**(-1)*za23**2*za35**(-1)*za25**(-3)*za45**2
     &     )
      zte0 = zte0 + zCF(s14,s35,s23)*zi * (
     &     - za15**(-1)*za23**(-1)*za35**2*za24**2*za25**(-2)*
     &    za45**(-1)
     &     )
      zte0 = zte0 + zCF(s14,s25,s23)*zi * (
     &     + za15**(-1)*za23**(-1)*za34**2*za45**(-1)
     &     )
      zte0 = zte0 + zCF(s15,s24,s23)*zi * (
     &     - za15**(-1)*za23**(-1)*za34**2*za45**(-1)
     &     )
      zte0 = zte0 + zCF(s15,s24,s23)*ztree * (
     &     + 1.D0
     &     )
      zte0 = zte0 + zCF(s15,s34,s23)*zi * (
     &     - za14**(-1)*za35**(-1)*za34**2*za25**(-1)
     &     + za15**(-1)*za23**(-1)*za34**2*za45**(-1)
     &     )
      zte0 = zte0 + zCF(s15,s34,s23)*ztree * (
     &     - 1.D0
     &     )
      zte0 = zte0 + zCF(s23,s14,s15)*zi * (
     &     - za14**(-1)*za35**(-1)*za34**2*za25**(-1)
     &     )
      zte0 = zte0 + zCF(s23,s14,s45)*zi * (
     &     + za13**2*za14**(-1)*za15**(-2)*za35**(-1)*za25**(-1)*
     &    za45**2
     &     )
      zte0 = zte0 + zCF(s23,s15,s14)*zi * (
     &     - za15**(-1)*za23**(-1)*za34**2*za45**(-1)
     &     )
      zte0 = zte0 + zCF(s24,s15,s35)*ztree * (
     &     + 1.D0
     &     )
      zte0 = zte0 + zCF(s24,s15,s13)*zi * (
     &     - za15**(-1)*za23**(-1)*za34**2*za45**(-1)
     &     )
      zte0 = zte0 + zCF(s24,s13,s35)*zi * (
     &     - za13**2*za14**(-1)*za15**(-2)*za35**(-1)*za25**(-1)*
     &    za45**2
     &     )
      zte0 = zte0 + zCF(s35,s14,s24)*zi * (
     &     - za12**(-2)*za13**2*za14**(-1)*za35**(-1)*za24**2*
     &    za25**(-1)
     &     )
      zte0 = zte0 + zCF(s35,s14,s24)*ztree * (
     &     + za12**(-2)*za13**2*za24**2*za34**(-2)
     &     )
      zte0 = zte0 + zCF(s35,s14,s12)*zi * (
     &     + za14**(-1)*za35**(-1)*za34**2*za25**(-1)
     &     - za15**(-1)*za23**(-1)*za34**2*za45**(-1)
     &     )
      zte0 = zte0 + zCF(s35,s14,s12)*ztree * (
     &     - 1.D0
     &     )
      zte0 = zte0 + zCF(s12,s35,s45)*zi * (
     &     - za15**(-1)*za23**(-1)*za34**2*za45**(-1)
     &     )
      zte0 = zte0 + zCF(s12,s34,s35)*zi * (
     &     + za14**(-1)*za35**(-1)*za34**2*za25**(-1)
     &     )
      zte0 = zte0 + zCF(s12,s45,s35)*ztree * (
     &     - 1.D0
     &     )
      zte0 = zte0 + zCF(s12,s45,s34)*zi * (
     &     + za15**(-1)*za23**(-1)*za34**2*za45**(-1)
     &     )
      zte0 = zte0 + zCF(s13,s24,s25)*zi * (
     &     - za14**(-1)*za35**(-1)*za34**2*za25**(-1)
     &     )
      zte0 = zte0 + zCF(s13,s25,s45)*zi * (
     &     + za14**(-1)*za35**(-1)*za34**2*za25**(-1)
     &     + za15**(-1)*za23**(-1)*za34**2*za45**(-1)
     &     )
      zte0 = zte0 + zCF(s13,s45,s24)*zi * (
     &     - za15**(-1)*za23**(-1)*za35**2*za24**2*za25**(-2)*
     &    za45**(-1)
     &     )
      zte0 = zte0 + zCF(s13,s45,s25)*ztree * (
     &     + 1.D0
     &     )
      zte0 = zte0 + zCF(s25,s14,s13)*zi * (
     &     + za15**(-1)*za23**(-1)*za34**2*za45**(-1)
     &     )
      zte0 = zte0 + zCF(s25,s14,s13)*ztree * (
     &     + 1.D0
     &     )
      zte0 = zte0 + zCF(s25,s14,s34)*ztree * (
     &     - 1.D0
     &     )
      zte0 = zte0 + zCF(s34,s15,s12)*zi * (
     &     + za15**(-1)*za23**(-1)*za34**2*za45**(-1)
     &     )
      zte0 = zte0 + zCF(s34,s15,s25)*zi * (
     &     - za14**(-1)*za35**(-1)*za34**2*za25**(-1)
     &     )
      zte0 = zte0 + zCF(s34,s15,s25)*ztree * (
     &     - 1.D0
     &     )
      zte0 = zte0 + zCF(s34,s12,s25)*zi * (
     &     + za14**(-1)*za35**(-1)*za34**2*za25**(-1)
     &     )
      zte0 = zte0 + zCF(s45,s12,s23)*ztree * (
     &     - 1.D0
     &     )
      zte0 = zte0 + zCF(s45,s13,s23)*zi * (
     &     + za12**(-2)*za13**2*za14**(-1)*za35**(-1)*za24**2*
     &    za25**(-1)
     &     )
      zte0 = zte0 + zCF(s45,s13,s23)*ztree * (
     &     + za12**(-2)*za13**2*za24**2*za34**(-2)
     &     )
      zte0 = zte0 + ztree * (
     &     + 13.D0/2.D0
     &     - 1.D0/2.D0*zzlog( - s34)**2
     &     + 1.D0/2.D0*zzlog( - s13)**2
     &     - 1.D0/2.D0*zzlog( - s12)**2
     &     + 1.D0/2.D0*zzlog( - s24)**2
     &     - 3.D0/2.D0*zzlog( - s23)
     &     + 1.D0/2.D0*zzlog( - s23)**2
     &     - 3.D0/2.D0*zzlog( - s14)
     &     + 1.D0/2.D0*zzlog( - s14)**2
     &     + 3.D0*zzlog(renscale2)
     &     + zzlog(renscale2)*zzlog( - s34)
     &     - zzlog(renscale2)*zzlog( - s13)
     &     + zzlog(renscale2)*zzlog( - s12)
     &     - zzlog(renscale2)*zzlog( - s24)
     &     )
      zte0 = zte0 + ztree * (
     &     - zzlog(renscale2)*zzlog( - s23)
     &     - zzlog(renscale2)*zzlog( - s14)
     &     + zzlog(renscale2)**2
     &     )
      elseif(ihel.gt.4)then
         zte2 = (0d0,0d0)
         zte1 = (0d0,0d0)
         zte0 = (0d0,0d0)
      endif

* correction factor to convert from c_gamma prefactor
      zte0 = zte0 - pi**2/12d0*zte2
* correction from finite renormalisations: eq 2.19 Signer's thesis
      zte0 = zte0 - ( -4d0/4d0 )*ztree



      if(iorder.eq.-2) zwt = zte2
      if(iorder.eq.-1) zwt = zte1
      if(iorder.eq.0)  zwt = zte0
      zwt=zwt/2d0
      zzCt1g1a=zwt

      return
      end
*****************************************************************************
* NF (1,0) 1-loop Qqb amplitude

      function zzCh1g1a(iq1,ig,iqb2,iq2,iqb1,ihel,iflip,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      common /order/iorder
      parameter(pi=3.141592653589793238d0)

      i1=iq1
      i2=iqb2
      i3=iq2
      i4=iqb1
      i5=ig

      s12=s5(i2,i4)
      s13=s5(i2,i1)
      s14=s5(i2,i3)
      s15=s5(i2,i5)
      s23=s5(i4,i1)
      s24=s5(i4,i3)
      s25=s5(i4,i5)
      s34=s5(i1,i3)
      s35=s5(i1,i5)
      s45=s5(i3,i5)

      if(iflip.eq.0)then

      za12=za5(i2,i4)
      za13=za5(i2,i1)
      za14=za5(i2,i3)
      za15=za5(i2,i5)
      za23=za5(i4,i1)
      za24=za5(i4,i3)
      za25=za5(i4,i5)
      za34=za5(i1,i3)
      za35=za5(i1,i5)
      za45=za5(i3,i5)

      zb12=zb5(i2,i4)
      zb13=zb5(i2,i1)
      zb14=zb5(i2,i3)
      zb15=zb5(i2,i5)
      zb23=zb5(i4,i1)
      zb24=zb5(i4,i3)
      zb25=zb5(i4,i5)
      zb34=zb5(i1,i3)
      zb35=zb5(i1,i5)
      zb45=zb5(i3,i5)

      elseif(iflip.eq.1)then

      za12=-zb5(i2,i4)
      za13=-zb5(i2,i1)
      za14=-zb5(i2,i3)
      za15=-zb5(i2,i5)
      za23=-zb5(i4,i1)
      za24=-zb5(i4,i3)
      za25=-zb5(i4,i5)
      za34=-zb5(i1,i3)
      za35=-zb5(i1,i5)
      za45=-zb5(i3,i5)

      zb12=-za5(i2,i4)
      zb13=-za5(i2,i1)
      zb14=-za5(i2,i3)
      zb15=-za5(i2,i5)
      zb23=-za5(i4,i1)
      zb24=-za5(i4,i3)
      zb25=-za5(i4,i5)
      zb34=-za5(i1,i3)
      zb35=-za5(i1,i5)
      zb45=-za5(i3,i5)

      endif

      zi=(0d0,1d0)
      ztree=zzC1g0a(i1,i5,i2,i3,i4,ihel,iflip)

      if(ihel.eq.1)then

      zhe2 = 0

      zhe1 =  + ztree * (
     &     + 1.D0/3.D0
     &     )

      zhe0 =  + zL1(s23,s14,renscale2)*zi * (
     &     - 2.D0/3.D0*za12**2*za14**(-1)*za35**(-1)*zb25
     &     )
      zhe0 = zhe0 + zL2(s23,s14,renscale2)*zi * (
     &     + 1.D0/3.D0*za12*zb35*zb45
     &     )
      zhe0 = zhe0 + zL3(s23,s14,renscale2)*zi * (
     &     - 2.D0/3.D0*za15*za23*zb35**2*zb45
     &     )
      zhe0 = zhe0 + ztree * (
     &     - 10.D0/9.D0
     &     + 2.D0/3.D0*zzlog( - s23)
     &     - 2.D0/3.D0*zzlog(renscale2)
     &     )

      elseif(ihel.eq.2)then

      zhe2 = 0

      zhe1 =  + ztree * (
     &     + 1.D0/3.D0
     &     )

      zhe0 =  + zL0(s23,s14,renscale2)*zi * (
     &     + 2.D0/3.D0*za14**(-1)*za23**(-1)*za35**(-1)*za24**2*za34*
     &    za45**(-1)
     &     )
      zhe0 = zhe0 + zL1(s23,s14,renscale2)*zi * (
     &     - 2.D0/3.D0*za23**(-1)*za24**2*za45**(-1)*zb15
     &     )
      zhe0 = zhe0 + zL2(s14,s23,renscale2)*zi * (
     &     - 1.D0/3.D0*za24*zb35*zb15
     &     )
      zhe0 = zhe0 + zL3(s23,s14,renscale2)*zi * (
     &     - 2.D0/3.D0*za14*za25*zb35*zb15**2
     &     )
      zhe0 = zhe0 + ztree * (
     &     - 10.D0/9.D0
     &     + 2.D0/3.D0*zzlog( - s23)
     &     - 2.D0/3.D0*zzlog(renscale2)
     &     )

      elseif(ihel.eq.3)then

      zhe2 = 0

      zhe1 =  + ztree * (
     &     + 1.D0/3.D0
     &     )

      zhe0 =  + zL1(s23,s14,renscale2)*zi * (
     &     + 2.D0/3.D0*za13**2*za14**(-1)*za35**(-1)*zb25
     &     )
      zhe0 = zhe0 + zL2(s23,s14,renscale2)*zi * (
     &     + 1.D0/3.D0*za13*zb25*zb45
     &     )
      zhe0 = zhe0 + zL3(s23,s14,renscale2)*zi * (
     &     + 2.D0/3.D0*za15*za23*zb25**2*zb45
     &     )
      zhe0 = zhe0 + ztree * (
     &     - 10.D0/9.D0
     &     + 2.D0/3.D0*zzlog( - s23)
     &     - 2.D0/3.D0*zzlog(renscale2)
     &     )

      elseif(ihel.eq.4)then

      zhe2 = 0

      zhe1 =  + ztree * (
     &     + 1.D0/3.D0
     &     )

      zhe0 =  + zL1(s14,s23,renscale2)*zi * (
     &     - 2.D0/3.D0*za15**(-1)*za23**(-1)*za34**2*zb45
     &     )
      zhe0 = zhe0 + zL2(s14,s23,renscale2)*zi * (
     &     - 1.D0/3.D0*za34*zb25*zb15
     &     )
      zhe0 = zhe0 + zL3(s14,s23,renscale2)*zi * (
     &     - 2.D0/3.D0*za14*za35*zb25*zb15**2
     &     )
      zhe0 = zhe0 + ztree * (
     &     - 10.D0/9.D0
     &     + 2.D0/3.D0*zzlog( - s14)
     &     - 2.D0/3.D0*zzlog(renscale2)
     &     )
      elseif(ihel.gt.4)then
         zhe2 = (0d0,0d0)
         zhe1 = (0d0,0d0)
         zhe0 = (0d0,0d0)
      endif

* correction factor to convert from c_gamma prefactor
      zhe0 = zhe0 - pi**2/12d0*zhe2

      if(iorder.eq.-2) zwt = zhe2
      if(iorder.eq.-1) zwt = zhe1
      if(iorder.eq.0)  zwt = zhe0
      zwt=zwt/2d0
      zzCh1g1a=zwt

      return
      end
****************************************************************************
* leading colour (1,0) 1-loop QQb  amplitude

      function zzC1g1c(iq1,ig,iqb1,iq2,iqb2,ihel,iflip,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      common /order/iorder
      parameter(pi=3.141592653589793238d0)

      i1=iq1
      i2=iqb2
      i3=iq2
      i4=iqb1
      i5=ig

      s12=s5(i2,i4)
      s13=s5(i2,i1)
      s14=s5(i2,i3)
      s15=s5(i2,i5)
      s23=s5(i4,i1)
      s24=s5(i4,i3)
      s25=s5(i4,i5)
      s34=s5(i1,i3)
      s35=s5(i1,i5)
      s45=s5(i3,i5)

      if(iflip.eq.0)then

      za12=za5(i2,i4)
      za13=za5(i2,i1)
      za14=za5(i2,i3)
      za15=za5(i2,i5)
      za23=za5(i4,i1)
      za24=za5(i4,i3)
      za25=za5(i4,i5)
      za34=za5(i1,i3)
      za35=za5(i1,i5)
      za45=za5(i3,i5)

      zb12=zb5(i2,i4)
      zb13=zb5(i2,i1)
      zb14=zb5(i2,i3)
      zb15=zb5(i2,i5)
      zb23=zb5(i4,i1)
      zb24=zb5(i4,i3)
      zb25=zb5(i4,i5)
      zb34=zb5(i1,i3)
      zb35=zb5(i1,i5)
      zb45=zb5(i3,i5)

      elseif(iflip.eq.1)then

      za12=-zb5(i2,i4)
      za13=-zb5(i2,i1)
      za14=-zb5(i2,i3)
      za15=-zb5(i2,i5)
      za23=-zb5(i4,i1)
      za24=-zb5(i4,i3)
      za25=-zb5(i4,i5)
      za34=-zb5(i1,i3)
      za35=-zb5(i1,i5)
      za45=-zb5(i3,i5)

      zb12=-za5(i2,i4)
      zb13=-za5(i2,i1)
      zb14=-za5(i2,i3)
      zb15=-za5(i2,i5)
      zb23=-za5(i4,i1)
      zb24=-za5(i4,i3)
      zb25=-za5(i4,i5)
      zb34=-za5(i1,i3)
      zb35=-za5(i1,i5)
      zb45=-za5(i3,i5)

      endif

      zi=(0d0,1d0)
      ztree=zzC1g0c(i1,i5,i4,i3,i2,ihel,iflip)

      if(ihel.eq.1)then

      ze2 =  + ztree * (
     &     - 3.D0
     &     )

      ze1 =  + zi * (
     &     - zzlog( - s45)*za12**2*za13*za14**(-1)*za15**(-1)*
     &    za23**(-1)*za35**(-1)
     &     - zzlog( - s34)*za12**2*za14**(-1)*za23**(-1)*za24*
     &    za25**(-1)*za45**(-1)
     &     + zzlog( - s25)*za12**2*za13*za14**(-1)*za15**(-1)*
     &    za23**(-1)*za35**(-1)
     &     - zzlog( - s12)*za12**2*za13*za14**(-1)*za15**(-1)*
     &    za23**(-1)*za35**(-1)
     &     + zzlog( - s35)*za12**2*za14**(-1)*za23**(-1)*za24*
     &    za25**(-1)*za45**(-1)
     &     - zzlog( - s15)*za12**2*za14**(-1)*za23**(-1)*za24*
     &    za25**(-1)*za45**(-1)
     &     + zzlog( - s14)*za12**2*za14**(-1)*za23**(-1)*za24*
     &    za25**(-1)*za45**(-1)
     &     )
      ze1 = ze1 + zi * (
     &     + zzlog( - s14)*za12**2*za13*za14**(-1)*za15**(-1)*
     &    za23**(-1)*za35**(-1)
     &     )
      ze1 = ze1 + ztree * (
     &     - 29.D0/6.D0
     &     + zzlog( - s25)
     &     + zzlog( - s35)
     &     + zzlog( - s14)
     &     - 3.D0*zzlog(renscale2)
     &     )

      ze0 =  + zi * (
     &     - za12*za35**(-1)*s14**(-1)*zb45
     &     + 1.D0/2.D0*za12*za23*za35**(-1)*s14**(-1)*za25**(-1)*zb34
     &     + 1.D0/2.D0*zzlog( - s45)**2*za12**2*za13*za14**(-1)*
     &    za15**(-1)*za23**(-1)*za35**(-1)
     &     + 1.D0/2.D0*zzlog( - s34)**2*za12**2*za14**(-1)*za23**(-1)*
     &    za24*za25**(-1)*za45**(-1)
     &     - 1.D0/2.D0*zzlog( - s25)**2*za12**2*za13*za14**(-1)*
     &    za15**(-1)*za23**(-1)*za35**(-1)
     &     + 1.D0/2.D0*zzlog( - s12)**2*za12**2*za13*za14**(-1)*
     &    za15**(-1)*za23**(-1)*za35**(-1)
     &     - 1.D0/2.D0*zzlog( - s35)**2*za12**2*za14**(-1)*za23**(-1)*
     &    za24*za25**(-1)*za45**(-1)
     &     + 1.D0/2.D0*zzlog( - s15)**2*za12**2*za14**(-1)*za23**(-1)*
     &    za24*za25**(-1)*za45**(-1)
     &     )
      ze0 = ze0 + zi * (
     &     - 1.D0/2.D0*zzlog( - s14)**2*za12**2*za14**(-1)*za23**(-1)*
     &    za24*za25**(-1)*za45**(-1)
     &     - 1.D0/2.D0*zzlog( - s14)**2*za12**2*za13*za14**(-1)*
     &    za15**(-1)*za23**(-1)*za35**(-1)
     &     - zzlog(renscale2)*zzlog( - s45)*za12**2*za13*za14**(-1)*
     &    za15**(-1)*za23**(-1)*za35**(-1)
     &     - zzlog(renscale2)*zzlog( - s34)*za12**2*za14**(-1)*
     &    za23**(-1)*za24*za25**(-1)*za45**(-1)
     &     + zzlog(renscale2)*zzlog( - s25)*za12**2*za13*za14**(-1)*
     &    za15**(-1)*za23**(-1)*za35**(-1)
     &     - zzlog(renscale2)*zzlog( - s12)*za12**2*za13*za14**(-1)*
     &    za15**(-1)*za23**(-1)*za35**(-1)
     &     + zzlog(renscale2)*zzlog( - s35)*za12**2*za14**(-1)*
     &    za23**(-1)*za24*za25**(-1)*za45**(-1)
     &     )
      ze0 = ze0 + zi * (
     &     - zzlog(renscale2)*zzlog( - s15)*za12**2*za14**(-1)*
     &    za23**(-1)*za24*za25**(-1)*za45**(-1)
     &     + zzlog(renscale2)*zzlog( - s14)*za12**2*za14**(-1)*
     &    za23**(-1)*za24*za25**(-1)*za45**(-1)
     &     + zzlog(renscale2)*zzlog( - s14)*za12**2*za13*za14**(-1)*
     &    za15**(-1)*za23**(-1)*za35**(-1)
     &     )
      ze0 = ze0 + zL0(s23,s14,renscale2)*zi * (
     &     + za12*za13*za14**(-1)*za35**(-2)
     &     )
      ze0 = ze0 + zL1(s23,s14,renscale2)*zi * (
     &     - za13*za35**(-2)*za25*zb45
     &     )
      ze0 = ze0 + zL1(s25,s14,renscale2)*zi * (
     &     + za13**2*za14**(-1)*za35**(-2)*za25*zb35
     &     + 3.D0/2.D0*za12*za13*za14**(-1)*za35**(-1)*zb35
     &     )
      ze0 = ze0 + zL2(s23,s14,renscale2)*zi * (
     &     + za15*za23*za35**(-1)*zb35*zb45
     &     )
      ze0 = ze0 + zL2(s25,s14,renscale2)*zi * (
     &     + 1.D0/2.D0*za13*za23*za35**(-1)*zb34*zb35
     &     )
      ze0 = ze0 + zCF(s14,s35,s23)*zi * (
     &     - za12**2*za14**(-1)*za23**(-1)*za24*za25**(-1)*za45**(-1)
     &     )
      ze0 = ze0 + zCF(s14,s25,s23)*zi * (
     &     - za13**3*za14**(-1)*za15**(-1)*za23**(-1)*za35**(-3)*
     &    za25**2
     &     )
      ze0 = ze0 + zCF(s15,s34,s23)*zi * (
     &     + za12**2*za14**(-1)*za23**(-1)*za24*za25**(-1)*za45**(-1)
     &     )
      ze0 = ze0 + zCF(s23,s14,s15)*zi * (
     &     + za14**(-1)*za15**2*za23**(-1)*za24**3*za25**(-1)*
     &    za45**(-3)
     &     )
      ze0 = ze0 + zCF(s23,s14,s45)*zi * (
     &     - za12**2*za14**(-1)*za23**(-1)*za24*za25**(-1)*za45**(-1)
     &     )
      ze0 = ze0 + zCF(s23,s15,s14)*zi * (
     &     - za14*za23**(-1)*za24*za25*za45**(-3)
     &     + za14*za35**(-1)*za25*za45**(-2)
     &     - za13*za14*za15**(-1)*za23**(-1)*za35**(-1)*za25**2*
     &    za45**(-2)
     &     )
      ze0 = ze0 + zCF(s23,s15,s45)*zi * (
     &     - za12**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     + za12**2*za13*za14**(-1)*za15**(-1)*za23**(-1)*za35**(-1)
     &     )
      ze0 = ze0 + zCF(s23,s45,s14)*zi * (
     &     + za12**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     )
      ze0 = ze0 + zCF(s23,s45,s15)*zi * (
     &     + za12**2*za14**(-1)*za23**(-1)*za24*za25**(-1)*za45**(-1)
     &     )
      ze0 = ze0 + zCF(s35,s14,s12)*zi * (
     &     - za12**2*za14**(-1)*za23**(-1)*za24*za25**(-1)*za45**(-1)
     &     + za12**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     )
      ze0 = ze0 + zCF(s12,s35,s45)*zi * (
     &     + za12**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     )
      ze0 = ze0 + zCF(s12,s34,s45)*zi * (
     &     + za12**2*za14**(-1)*za23**(-1)*za24*za25**(-1)*za45**(-1)
     &     - za12**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     + za12**2*za13*za14**(-1)*za15**(-1)*za23**(-1)*za35**(-1)
     &     )
      ze0 = ze0 + zCF(s12,s45,s35)*zi * (
     &     - za12**2*za14**(-1)*za23**(-1)*za24*za25**(-1)*za45**(-1)
     &     )
      ze0 = ze0 + zCF(s25,s34,s14)*zi * (
     &     + za12**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     - za12**2*za13*za14**(-1)*za15**(-1)*za23**(-1)*za35**(-1)
     &     )
      ze0 = ze0 + zCF(s34,s15,s12)*zi * (
     &     + za12**2*za14**(-1)*za23**(-1)*za24*za25**(-1)*za45**(-1)
     &     - za12**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     + za12**2*za13*za14**(-1)*za15**(-1)*za23**(-1)*za35**(-1)
     &     )
      ze0 = ze0 + zCF(s34,s15,s25)*zi * (
     &     - za12**2*za13*za14**(-1)*za15**(-1)*za23**(-1)*za35**(-1)
     &     )
      ze0 = ze0 + zCF(s34,s25,s15)*zi * (
     &     + za12**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     )
      ze0 = ze0 + zCF(s45,s23,s12)*zi * (
     &     + za12**2*za13*za14**(-1)*za15**(-1)*za23**(-1)*za35**(-1)
     &     )
      ze0 = ze0 + ztree * (
     &     + 29.D0/18.D0
     &     - 1.D0/2.D0*zzlog( - s25)**2
     &     - 1.D0/2.D0*zzlog( - s35)**2
     &     - 2.D0/3.D0*zzlog( - s14)
     &     - 1.D0/2.D0*zzlog( - s14)**2
     &     + 2.D0/3.D0*zzlog(renscale2)
     &     + zzlog(renscale2)*zzlog( - s25)
     &     + zzlog(renscale2)*zzlog( - s35)
     &     + zzlog(renscale2)*zzlog( - s14)
     &     - 3.D0/2.D0*zzlog(renscale2)**2
     &     )


      elseif(ihel.eq.2)then

      ze2 =  + ztree * (
     &     - 3.D0
     &     )

      ze1 =  + zi * (
     &     + zzlog( - s45)*za13*za14**(-1)*za15**(-1)*za23**(-1)*
     &    za35**(-1)*za24**2
     &     + zzlog( - s34)*za14**(-1)*za23**(-1)*za24**3*za25**(-1)*
     &    za45**(-1)
     &     - zzlog( - s25)*za13*za14**(-1)*za15**(-1)*za23**(-1)*
     &    za35**(-1)*za24**2
     &     + zzlog( - s12)*za13*za14**(-1)*za15**(-1)*za23**(-1)*
     &    za35**(-1)*za24**2
     &     - zzlog( - s35)*za14**(-1)*za23**(-1)*za24**3*za25**(-1)*
     &    za45**(-1)
     &     + zzlog( - s15)*za14**(-1)*za23**(-1)*za24**3*za25**(-1)*
     &    za45**(-1)
     &     - zzlog( - s14)*za14**(-1)*za23**(-1)*za24**3*za25**(-1)*
     &    za45**(-1)
     &     )
      ze1 = ze1 + zi * (
     &     - zzlog( - s14)*za13*za14**(-1)*za15**(-1)*za23**(-1)*
     &    za35**(-1)*za24**2
     &     )
      ze1 = ze1 + ztree * (
     &     - 29.D0/6.D0
     &     + zzlog( - s25)
     &     + zzlog( - s35)
     &     + zzlog( - s14)
     &     - 3.D0*zzlog(renscale2)
     &     )

      ze0 =  + zi * (
     &     + za35**(-1)*za24*s14**(-1)*zb15
     &     - 1.D0/2.D0*zzlog( - s45)**2*za13*za14**(-1)*za15**(-1)*
     &    za23**(-1)*za35**(-1)*za24**2
     &     - 1.D0/2.D0*zzlog( - s34)**2*za14**(-1)*za23**(-1)*za24**3*
     &    za25**(-1)*za45**(-1)
     &     + 1.D0/2.D0*zzlog( - s25)**2*za13*za14**(-1)*za15**(-1)*
     &    za23**(-1)*za35**(-1)*za24**2
     &     - 1.D0/2.D0*zzlog( - s12)**2*za13*za14**(-1)*za15**(-1)*
     &    za23**(-1)*za35**(-1)*za24**2
     &     + 1.D0/2.D0*zzlog( - s35)**2*za14**(-1)*za23**(-1)*za24**3*
     &    za25**(-1)*za45**(-1)
     &     - 1.D0/2.D0*zzlog( - s15)**2*za14**(-1)*za23**(-1)*za24**3*
     &    za25**(-1)*za45**(-1)
     &     + 1.D0/2.D0*zzlog( - s14)**2*za14**(-1)*za23**(-1)*za24**3*
     &    za25**(-1)*za45**(-1)
     &     )
      ze0 = ze0 + zi * (
     &     + 1.D0/2.D0*zzlog( - s14)**2*za13*za14**(-1)*za15**(-1)*
     &    za23**(-1)*za35**(-1)*za24**2
     &     + zzlog(renscale2)*zzlog( - s45)*za13*za14**(-1)*za15**(-1)*
     &    za23**(-1)*za35**(-1)*za24**2
     &     + zzlog(renscale2)*zzlog( - s34)*za14**(-1)*za23**(-1)*
     &    za24**3*za25**(-1)*za45**(-1)
     &     - zzlog(renscale2)*zzlog( - s25)*za13*za14**(-1)*za15**(-1)*
     &    za23**(-1)*za35**(-1)*za24**2
     &     + zzlog(renscale2)*zzlog( - s12)*za13*za14**(-1)*za15**(-1)*
     &    za23**(-1)*za35**(-1)*za24**2
     &     - zzlog(renscale2)*zzlog( - s35)*za14**(-1)*za23**(-1)*
     &    za24**3*za25**(-1)*za45**(-1)
     &     + zzlog(renscale2)*zzlog( - s15)*za14**(-1)*za23**(-1)*
     &    za24**3*za25**(-1)*za45**(-1)
     &     )
      ze0 = ze0 + zi * (
     &     - zzlog(renscale2)*zzlog( - s14)*za14**(-1)*za23**(-1)*
     &    za24**3*za25**(-1)*za45**(-1)
     &     - zzlog(renscale2)*zzlog( - s14)*za13*za14**(-1)*za15**(-1)*
     &    za23**(-1)*za35**(-1)*za24**2
     &     )
      ze0 = ze0 + zL0(s23,s25,renscale2)*zi * (
     &     - za14**(-1)*za35**(-2)*za24*za34
     &     )
      ze0 = ze0 + zL0(s25,s14,renscale2)*zi * (
     &     + 3.D0/2.D0*za14**(-1)*za35**(-1)*za24**2*za25**(-1)
     &     )
      ze0 = ze0 + zL0(s34,s25,renscale2)*zi * (
     &     + za15**(-1)*za35**(-1)*za24
     &     )
      ze0 = ze0 + zL1(s23,s14,renscale2)*zi * (
     &     + za35**(-2)*za34*za25*zb15
     &     )
      ze0 = ze0 + zL1(s25,s14,renscale2)*zi * (
     &     + za13**(-1)*za15*za23**2*za35**(-2)*za34*za25**(-1)*zb13
     &     - 3.D0*za23*za35**(-1)*za24*za25**(-1)*zb13
     &     + 2.D0*za23**2*za35**(-2)*za25**(-1)*za45*zb13
     &     )
      ze0 = ze0 + zL1(s25,s34,renscale2)*zi * (
     &     + za12**2*za13**(-1)*za15**(-1)*za34*za25**(-1)*zb13
     &     )
      ze0 = ze0 + zL1(s34,s12,renscale2)*zi * (
     &     + za12*za15**(-1)*za35**(-1)*za45*zb15
     &     )
      ze0 = ze0 + zL2(s23,s14,renscale2)*zi * (
     &     + za23*za35**(-1)*zb35*za45*zb15
     &     )
      ze0 = ze0 + zL2(s25,s14,renscale2)*zi * (
     &     + 1.D0/2.D0*za14*za23**2*za35**(-1)*za25**(-1)*zb13**2
     &     )
      ze0 = ze0 + zCF(s14,s35,s23)*zi * (
     &     + za14**(-1)*za23**(-1)*za24**3*za25**(-1)*za45**(-1)
     &     )
      ze0 = ze0 + zCF(s14,s25,s23)*zi * (
     &     + za13*za14**(-1)*za15**(-1)*za23**(-1)*za35**(-3)*za34**2*
     &    za25**2
     &     )
      ze0 = ze0 + zCF(s15,s23,s34)*zi * (
     &     - za14**(-1)*za23**(-1)*za24**3*za25**(-1)*za45**(-1)
     &     )
      ze0 = ze0 + zCF(s23,s14,s15)*zi * (
     &     + za13*za14**(-1)*za15**(-1)*za23**(-1)*za35**(-1)*za24**2
     &     )
      ze0 = ze0 + zCF(s23,s14,s45)*zi * (
     &     - za12**2*za13*za14**(-1)*za15**(-3)*za23**(-1)*za35**(-1)*
     &    za45**2
     &     )
      ze0 = ze0 + zCF(s23,s15,s14)*zi * (
     &     - za14**(-1)*za35**(-1)*za24**2*za25**(-1)
     &     )
      ze0 = ze0 + zCF(s23,s15,s45)*zi * (
     &     - za13*za14**(-1)*za15**(-1)*za23**(-1)*za35**(-1)*za24**2
     &     )
      ze0 = ze0 + zCF(s23,s45,s14)*zi * (
     &     + za14*za15**(-2)*za23**(-1)*za24*za25*za45**(-1)
     &     - za14*za15**(-2)*za35**(-1)*za25
     &     + za13*za14*za15**(-3)*za23**(-1)*za35**(-1)*za25**2
     &     )
      ze0 = ze0 + zCF(s23,s45,s15)*zi * (
     &     - za14**(-1)*za23**(-1)*za24**3*za25**(-1)*za45**(-1)
     &     + za14**(-1)*za35**(-1)*za24**2*za25**(-1)
     &     )
      ze0 = ze0 + zCF(s35,s12,s14)*zi * (
     &     + za14**(-1)*za23**(-1)*za24**3*za25**(-1)*za45**(-1)
     &     - za14**(-1)*za35**(-1)*za24**2*za25**(-1)
     &     )
      ze0 = ze0 + zCF(s12,s35,s45)*zi * (
     &     - za14**(-1)*za35**(-1)*za24**2*za25**(-1)
     &     )
      ze0 = ze0 + zCF(s12,s45,s35)*zi * (
     &     + za14**(-1)*za23**(-1)*za24**3*za25**(-1)*za45**(-1)
     &     )
      ze0 = ze0 + zCF(s12,s45,s34)*zi * (
     &     - za14**(-1)*za23**(-1)*za35**(-2)*za24*za34**2*za25*
     &    za45**(-1)
     &     + za14**(-1)*za35**(-3)*za34**2*za25
     &     - za13*za14**(-1)*za15**(-1)*za23**(-1)*za35**(-3)*za34**2*
     &    za25**2
     &     )
      ze0 = ze0 + zCF(s25,s14,s34)*zi * (
     &     - za12**2*za13**(-2)*za14**(-1)*za35**(-1)*za34**2*
     &    za25**(-1)
     &     + za12**2*za13**(-1)*za14**(-1)*za15**(-1)*za23**(-1)*
     &    za35**(-1)*za34**2
     &     )
      ze0 = ze0 + zCF(s34,s15,s25)*zi * (
     &     + za13*za14**(-1)*za15**(-1)*za23**(-1)*za35**(-1)*za24**2
     &     )
      ze0 = ze0 + zCF(s34,s12,s15)*zi * (
     &     - za14**(-1)*za23**(-1)*za24**3*za25**(-1)*za45**(-1)
     &     + za14**(-1)*za35**(-1)*za24**2*za25**(-1)
     &     - za13*za14**(-1)*za15**(-1)*za23**(-1)*za35**(-1)*za24**2
     &     )
      ze0 = ze0 + zCF(s34,s25,s15)*zi * (
     &     - za14**(-1)*za35**(-1)*za24**2*za25**(-1)
     &     )
      ze0 = ze0 + zCF(s45,s12,s23)*zi * (
     &     - za12**2*za13**(-1)*za14**(-1)*za15**(-1)*za23**(-1)*
     &    za35**(-1)*za34**2
     &     )
      ze0 = ze0 + ztree * (
     &     + 29.D0/18.D0
     &     - 1.D0/2.D0*zzlog( - s25)**2
     &     - 1.D0/2.D0*zzlog( - s35)**2
     &     - 2.D0/3.D0*zzlog( - s14)
     &     - 1.D0/2.D0*zzlog( - s14)**2
     &     + 2.D0/3.D0*zzlog(renscale2)
     &     + zzlog(renscale2)*zzlog( - s25)
     &     + zzlog(renscale2)*zzlog( - s35)
     &     + zzlog(renscale2)*zzlog( - s14)
     &     - 3.D0/2.D0*zzlog(renscale2)**2
     &     )

      elseif(ihel.eq.3)then

      ze2 =  + ztree * (
     &     - 3.D0
     &     )

      ze1 =  + zi * (
     &     + zzlog( - s45)*za13**3*za14**(-1)*za15**(-1)*za23**(-1)*
     &    za35**(-1)
     &     + zzlog( - s34)*za13**2*za14**(-1)*za23**(-1)*za24*
     &    za25**(-1)*za45**(-1)
     &     - zzlog( - s25)*za13**3*za14**(-1)*za15**(-1)*za23**(-1)*
     &    za35**(-1)
     &     + zzlog( - s12)*za13**3*za14**(-1)*za15**(-1)*za23**(-1)*
     &    za35**(-1)
     &     - zzlog( - s35)*za13**2*za14**(-1)*za23**(-1)*za24*
     &    za25**(-1)*za45**(-1)
     &     + zzlog( - s15)*za13**2*za14**(-1)*za23**(-1)*za24*
     &    za25**(-1)*za45**(-1)
     &     - zzlog( - s14)*za13**2*za14**(-1)*za23**(-1)*za24*
     &    za25**(-1)*za45**(-1)
     &     )
      ze1 = ze1 + zi * (
     &     - zzlog( - s14)*za13**3*za14**(-1)*za15**(-1)*za23**(-1)*
     &    za35**(-1)
     &     )
      ze1 = ze1 + ztree * (
     &     - 29.D0/6.D0
     &     + zzlog( - s25)
     &     + zzlog( - s35)
     &     + zzlog( - s14)
     &     - 3.D0*zzlog(renscale2)
     &     )

      ze0 =  + zi * (
     &     + za13*s14**(-1)*za25**(-1)*zb45
     &     - 1.D0/2.D0*zzlog( - s45)**2*za13**3*za14**(-1)*za15**(-1)*
     &    za23**(-1)*za35**(-1)
     &     - 1.D0/2.D0*zzlog( - s34)**2*za13**2*za14**(-1)*za23**(-1)*
     &    za24*za25**(-1)*za45**(-1)
     &     + 1.D0/2.D0*zzlog( - s25)**2*za13**3*za14**(-1)*za15**(-1)*
     &    za23**(-1)*za35**(-1)
     &     - 1.D0/2.D0*zzlog( - s12)**2*za13**3*za14**(-1)*za15**(-1)*
     &    za23**(-1)*za35**(-1)
     &     + 1.D0/2.D0*zzlog( - s35)**2*za13**2*za14**(-1)*za23**(-1)*
     &    za24*za25**(-1)*za45**(-1)
     &     - 1.D0/2.D0*zzlog( - s15)**2*za13**2*za14**(-1)*za23**(-1)*
     &    za24*za25**(-1)*za45**(-1)
     &     + 1.D0/2.D0*zzlog( - s14)**2*za13**2*za14**(-1)*za23**(-1)*
     &    za24*za25**(-1)*za45**(-1)
     &     )
      ze0 = ze0 + zi * (
     &     + 1.D0/2.D0*zzlog( - s14)**2*za13**3*za14**(-1)*za15**(-1)*
     &    za23**(-1)*za35**(-1)
     &     + zzlog(renscale2)*zzlog( - s45)*za13**3*za14**(-1)*
     &    za15**(-1)*za23**(-1)*za35**(-1)
     &     + zzlog(renscale2)*zzlog( - s34)*za13**2*za14**(-1)*
     &    za23**(-1)*za24*za25**(-1)*za45**(-1)
     &     - zzlog(renscale2)*zzlog( - s25)*za13**3*za14**(-1)*
     &    za15**(-1)*za23**(-1)*za35**(-1)
     &     + zzlog(renscale2)*zzlog( - s12)*za13**3*za14**(-1)*
     &    za15**(-1)*za23**(-1)*za35**(-1)
     &     - zzlog(renscale2)*zzlog( - s35)*za13**2*za14**(-1)*
     &    za23**(-1)*za24*za25**(-1)*za45**(-1)
     &     + zzlog(renscale2)*zzlog( - s15)*za13**2*za14**(-1)*
     &    za23**(-1)*za24*za25**(-1)*za45**(-1)
     &     )
      ze0 = ze0 + zi * (
     &     - zzlog(renscale2)*zzlog( - s14)*za13**2*za14**(-1)*
     &    za23**(-1)*za24*za25**(-1)*za45**(-1)
     &     - zzlog(renscale2)*zzlog( - s14)*za13**3*za14**(-1)*
     &    za15**(-1)*za23**(-1)*za35**(-1)
     &     )
      ze0 = ze0 + zL0(s23,s35,renscale2)*zi * (
     &     - za12*za13*za14**(-1)*za25**(-2)
     &     )
      ze0 = ze0 + zL0(s35,s14,renscale2)*zi * (
     &     + 3.D0/2.D0*za13**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     )
      ze0 = ze0 + zL0(s12,s35,renscale2)*zi * (
     &     + za13*za25**(-1)*za45**(-1)
     &     )
      ze0 = ze0 + zL1(s23,s14,renscale2)*zi * (
     &     + za12*za35*za25**(-2)*zb45
     &     )
      ze0 = ze0 + zL1(s35,s14,renscale2)*zi * (
     &     + 2.D0*za15*za23**2*za35**(-1)*za25**(-2)*zb24
     &     - 3.D0*za13*za23*za35**(-1)*za25**(-1)*zb24
     &     + za12*za23**2*za35**(-1)*za24**(-1)*za25**(-2)*za45*zb24
     &     )
      ze0 = ze0 + zL1(s35,s12,renscale2)*zi * (
     &     + za12*za35**(-1)*za24**(-1)*za34**2*za45**(-1)*zb24
     &     )
      ze0 = ze0 + zL1(s12,s34,renscale2)*zi * (
     &     + za15*za34*za25**(-1)*za45**(-1)*zb45
     &     )
      ze0 = ze0 + zL2(s23,s14,renscale2)*zi * (
     &     + za15*za23*za25**(-1)*zb25*zb45
     &     )
      ze0 = ze0 + zL2(s35,s14,renscale2)*zi * (
     &     + 1.D0/2.D0*za14*za23**2*za35**(-1)*za25**(-1)*zb24**2
     &     )
      ze0 = ze0 + zCF(s14,s35,s23)*zi * (
     &     + za12**2*za14**(-1)*za23**(-1)*za35**2*za24*za25**(-3)*
     &    za45**(-1)
     &     )
      ze0 = ze0 + zCF(s14,s25,s23)*zi * (
     &     + za13**3*za14**(-1)*za15**(-1)*za23**(-1)*za35**(-1)
     &     )
      ze0 = ze0 + zCF(s15,s34,s23)*zi * (
     &     - za12**2*za14**(-1)*za23**(-1)*za24**(-1)*za34**2*
     &    za25**(-1)*za45**(-1)
     &     )
      ze0 = ze0 + zCF(s23,s14,s15)*zi * (
     &     - za14**(-1)*za15**2*za23**(-1)*za24*za34**2*za25**(-1)*
     &    za45**(-3)
     &     )
      ze0 = ze0 + zCF(s23,s14,s45)*zi * (
     &     + za13**2*za14**(-1)*za23**(-1)*za24*za25**(-1)*za45**(-1)
     &     )
      ze0 = ze0 + zCF(s23,s15,s14)*zi * (
     &     + za14*za23**(-1)*za35**2*za24*za25**(-1)*za45**(-3)
     &     - za14*za35*za25**(-1)*za45**(-2)
     &     + za13*za14*za15**(-1)*za23**(-1)*za35*za45**(-2)
     &     )
      ze0 = ze0 + zCF(s23,s15,s45)*zi * (
     &     + za13**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     - za13**3*za14**(-1)*za15**(-1)*za23**(-1)*za35**(-1)
     &     )
      ze0 = ze0 + zCF(s23,s45,s14)*zi * (
     &     - za13**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     )
      ze0 = ze0 + zCF(s23,s45,s15)*zi * (
     &     - za13**2*za14**(-1)*za23**(-1)*za24*za25**(-1)*za45**(-1)
     &     )
      ze0 = ze0 + zCF(s35,s14,s12)*zi * (
     &     + za12**2*za14**(-1)*za23**(-1)*za24**(-1)*za34**2*
     &    za25**(-1)*za45**(-1)
     &     - za12**2*za14**(-1)*za35**(-1)*za24**(-2)*za34**2*
     &    za25**(-1)
     &     )
      ze0 = ze0 + zCF(s12,s35,s45)*zi * (
     &     - za13**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     )
      ze0 = ze0 + zCF(s12,s34,s45)*zi * (
     &     - za13**2*za14**(-1)*za23**(-1)*za24*za25**(-1)*za45**(-1)
     &     + za13**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     - za13**3*za14**(-1)*za15**(-1)*za23**(-1)*za35**(-1)
     &     )
      ze0 = ze0 + zCF(s12,s45,s35)*zi * (
     &     + za13**2*za14**(-1)*za23**(-1)*za24*za25**(-1)*za45**(-1)
     &     )
      ze0 = ze0 + zCF(s25,s34,s14)*zi * (
     &     - za13**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     + za13**3*za14**(-1)*za15**(-1)*za23**(-1)*za35**(-1)
     &     )
      ze0 = ze0 + zCF(s34,s15,s12)*zi * (
     &     - za12**2*za14**(-1)*za23**(-1)*za35**2*za24*za25**(-3)*
     &    za45**(-1)
     &     + za12**2*za14**(-1)*za35*za25**(-3)
     &     - za12**2*za13*za14**(-1)*za15**(-1)*za23**(-1)*za35*
     &    za25**(-2)
     &     )
      ze0 = ze0 + zCF(s34,s15,s25)*zi * (
     &     + za13**3*za14**(-1)*za15**(-1)*za23**(-1)*za35**(-1)
     &     )
      ze0 = ze0 + zCF(s34,s25,s15)*zi * (
     &     - za13**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     )
      ze0 = ze0 + zCF(s45,s23,s12)*zi * (
     &     - za13**3*za14**(-1)*za15**(-1)*za23**(-1)*za35**(-1)
     &     )
      ze0 = ze0 + ztree * (
     &     + 29.D0/18.D0
     &     - 1.D0/2.D0*zzlog( - s25)**2
     &     - 1.D0/2.D0*zzlog( - s35)**2
     &     - 2.D0/3.D0*zzlog( - s14)
     &     - 1.D0/2.D0*zzlog( - s14)**2
     &     + 2.D0/3.D0*zzlog(renscale2)
     &     + zzlog(renscale2)*zzlog( - s25)
     &     + zzlog(renscale2)*zzlog( - s35)
     &     + zzlog(renscale2)*zzlog( - s14)
     &     - 3.D0/2.D0*zzlog(renscale2)**2
     &     )

      elseif(ihel.eq.4)then


      ze2 =  + ztree * (
     &     - 3.D0
     &     )

      ze1 =  + zi * (
     &     - zzlog( - s45)*za13*za14**(-1)*za15**(-1)*za23**(-1)*
     &    za35**(-1)*za34**2
     &     - zzlog( - s34)*za14**(-1)*za23**(-1)*za24*za34**2*
     &    za25**(-1)*za45**(-1)
     &     + zzlog( - s25)*za13*za14**(-1)*za15**(-1)*za23**(-1)*
     &    za35**(-1)*za34**2
     &     - zzlog( - s12)*za13*za14**(-1)*za15**(-1)*za23**(-1)*
     &    za35**(-1)*za34**2
     &     + zzlog( - s35)*za14**(-1)*za23**(-1)*za24*za34**2*
     &    za25**(-1)*za45**(-1)
     &     - zzlog( - s15)*za14**(-1)*za23**(-1)*za24*za34**2*
     &    za25**(-1)*za45**(-1)
     &     + zzlog( - s14)*za14**(-1)*za23**(-1)*za24*za34**2*
     &    za25**(-1)*za45**(-1)
     &     )
      ze1 = ze1 + zi * (
     &     + zzlog( - s14)*za13*za14**(-1)*za15**(-1)*za23**(-1)*
     &    za35**(-1)*za34**2
     &     )
      ze1 = ze1 + ztree * (
     &     - 29.D0/6.D0
     &     + zzlog( - s25)
     &     + zzlog( - s35)
     &     + zzlog( - s14)
     &     - 3.D0*zzlog(renscale2)
     &     )

      ze0 =  + zi * (
     &     - za34*s14**(-1)*za25**(-1)*zb15
     &     + 1.D0/2.D0*za23*za35**(-1)*za34*s14**(-1)*za25**(-1)*zb12
     &     + 1.D0/2.D0*zzlog( - s45)**2*za13*za14**(-1)*za15**(-1)*
     &    za23**(-1)*za35**(-1)*za34**2
     &     + 1.D0/2.D0*zzlog( - s34)**2*za14**(-1)*za23**(-1)*za24*
     &    za34**2*za25**(-1)*za45**(-1)
     &     - 1.D0/2.D0*zzlog( - s25)**2*za13*za14**(-1)*za15**(-1)*
     &    za23**(-1)*za35**(-1)*za34**2
     &     + 1.D0/2.D0*zzlog( - s12)**2*za13*za14**(-1)*za15**(-1)*
     &    za23**(-1)*za35**(-1)*za34**2
     &     - 1.D0/2.D0*zzlog( - s35)**2*za14**(-1)*za23**(-1)*za24*
     &    za34**2*za25**(-1)*za45**(-1)
     &     + 1.D0/2.D0*zzlog( - s15)**2*za14**(-1)*za23**(-1)*za24*
     &    za34**2*za25**(-1)*za45**(-1)
     &     )
      ze0 = ze0 + zi * (
     &     - 1.D0/2.D0*zzlog( - s14)**2*za14**(-1)*za23**(-1)*za24*
     &    za34**2*za25**(-1)*za45**(-1)
     &     - 1.D0/2.D0*zzlog( - s14)**2*za13*za14**(-1)*za15**(-1)*
     &    za23**(-1)*za35**(-1)*za34**2
     &     - zzlog(renscale2)*zzlog( - s45)*za13*za14**(-1)*za15**(-1)*
     &    za23**(-1)*za35**(-1)*za34**2
     &     - zzlog(renscale2)*zzlog( - s34)*za14**(-1)*za23**(-1)*za24*
     &    za34**2*za25**(-1)*za45**(-1)
     &     + zzlog(renscale2)*zzlog( - s25)*za13*za14**(-1)*za15**(-1)*
     &    za23**(-1)*za35**(-1)*za34**2
     &     - zzlog(renscale2)*zzlog( - s12)*za13*za14**(-1)*za15**(-1)*
     &    za23**(-1)*za35**(-1)*za34**2
     &     + zzlog(renscale2)*zzlog( - s35)*za14**(-1)*za23**(-1)*za24*
     &    za34**2*za25**(-1)*za45**(-1)
     &     )
      ze0 = ze0 + zi * (
     &     - zzlog(renscale2)*zzlog( - s15)*za14**(-1)*za23**(-1)*za24*
     &    za34**2*za25**(-1)*za45**(-1)
     &     + zzlog(renscale2)*zzlog( - s14)*za14**(-1)*za23**(-1)*za24*
     &    za34**2*za25**(-1)*za45**(-1)
     &     + zzlog(renscale2)*zzlog( - s14)*za13*za14**(-1)*za15**(-1)*
     &    za23**(-1)*za35**(-1)*za34**2
     &     )
      ze0 = ze0 + zL0(s23,s14,renscale2)*zi * (
     &     + za14**(-1)*za24*za34*za25**(-2)
     &     )
      ze0 = ze0 + zL1(s23,s14,renscale2)*zi * (
     &     - za35*za24*za25**(-2)*zb15
     &     )
      ze0 = ze0 + zL1(s35,s14,renscale2)*zi * (
     &     + 3.D0/2.D0*za14**(-1)*za24*za34*za25**(-1)*zb25
     &     + za14**(-1)*za35*za24**2*za25**(-2)*zb25
     &     )
      ze0 = ze0 + zL2(s23,s14,renscale2)*zi * (
     &     + za23*za25**(-1)*zb25*za45*zb15
     &     )
      ze0 = ze0 + zL2(s35,s14,renscale2)*zi * (
     &     + 1.D0/2.D0*za23*za24*za25**(-1)*zb25*zb12
     &     )
      ze0 = ze0 + zCF(s14,s35,s23)*zi * (
     &     - za14**(-1)*za23**(-1)*za35**2*za24**3*za25**(-3)*
     &    za45**(-1)
     &     )
      ze0 = ze0 + zCF(s14,s25,s23)*zi * (
     &     - za13*za14**(-1)*za15**(-1)*za23**(-1)*za35**(-1)*za34**2
     &     )
      ze0 = ze0 + zCF(s15,s23,s34)*zi * (
     &     + za14**(-1)*za23**(-1)*za24*za34**2*za25**(-1)*za45**(-1)
     &     )
      ze0 = ze0 + zCF(s23,s14,s15)*zi * (
     &     - za13*za14**(-1)*za15**(-1)*za23**(-1)*za35**(-1)*za34**2
     &     )
      ze0 = ze0 + zCF(s23,s14,s45)*zi * (
     &     + za13**3*za14**(-1)*za15**(-3)*za23**(-1)*za35**(-1)*
     &    za45**2
     &     )
      ze0 = ze0 + zCF(s23,s15,s14)*zi * (
     &     + za14**(-1)*za35**(-1)*za34**2*za25**(-1)
     &     )
      ze0 = ze0 + zCF(s23,s15,s45)*zi * (
     &     + za13*za14**(-1)*za15**(-1)*za23**(-1)*za35**(-1)*za34**2
     &     )
      ze0 = ze0 + zCF(s23,s45,s14)*zi * (
     &     - za14*za15**(-2)*za23**(-1)*za35**2*za24*za25**(-1)*
     &    za45**(-1)
     &     + za14*za15**(-2)*za35*za25**(-1)
     &     - za13*za14*za15**(-3)*za23**(-1)*za35
     &     )
      ze0 = ze0 + zCF(s23,s45,s15)*zi * (
     &     + za14**(-1)*za23**(-1)*za24*za34**2*za25**(-1)*za45**(-1)
     &     - za14**(-1)*za35**(-1)*za34**2*za25**(-1)
     &     )
      ze0 = ze0 + zCF(s35,s12,s14)*zi * (
     &     - za14**(-1)*za23**(-1)*za24*za34**2*za25**(-1)*za45**(-1)
     &     + za14**(-1)*za35**(-1)*za34**2*za25**(-1)
     &     )
      ze0 = ze0 + zCF(s12,s35,s45)*zi * (
     &     + za14**(-1)*za35**(-1)*za34**2*za25**(-1)
     &     )
      ze0 = ze0 + zCF(s12,s45,s35)*zi * (
     &     - za14**(-1)*za23**(-1)*za24*za34**2*za25**(-1)*za45**(-1)
     &     )
      ze0 = ze0 + zCF(s12,s45,s34)*zi * (
     &     + za14**(-1)*za23**(-1)*za24*za34**2*za25**(-1)*za45**(-1)
     &     - za14**(-1)*za35**(-1)*za34**2*za25**(-1)
     &     + za13*za14**(-1)*za15**(-1)*za23**(-1)*za35**(-1)*za34**2
     &     )
      ze0 = ze0 + zCF(s25,s14,s34)*zi * (
     &     + za14**(-1)*za35**(-1)*za34**2*za25**(-1)
     &     - za13*za14**(-1)*za15**(-1)*za23**(-1)*za35**(-1)*za34**2
     &     )
      ze0 = ze0 + zCF(s34,s15,s25)*zi * (
     &     - za13*za14**(-1)*za15**(-1)*za23**(-1)*za35**(-1)*za34**2
     &     )
      ze0 = ze0 + zCF(s34,s12,s15)*zi * (
     &     + za14**(-1)*za23**(-1)*za24*za34**2*za25**(-1)*za45**(-1)
     &     - za14**(-1)*za35**(-1)*za34**2*za25**(-1)
     &     + za13*za14**(-1)*za15**(-1)*za23**(-1)*za35**(-1)*za34**2
     &     )
      ze0 = ze0 + zCF(s34,s25,s15)*zi * (
     &     + za14**(-1)*za35**(-1)*za34**2*za25**(-1)
     &     )
      ze0 = ze0 + zCF(s45,s12,s23)*zi * (
     &     + za13*za14**(-1)*za15**(-1)*za23**(-1)*za35**(-1)*za34**2
     &     )
      ze0 = ze0 + ztree * (
     &     + 29.D0/18.D0
     &     - 1.D0/2.D0*zzlog( - s25)**2
     &     - 1.D0/2.D0*zzlog( - s35)**2
     &     - 2.D0/3.D0*zzlog( - s14)
     &     - 1.D0/2.D0*zzlog( - s14)**2
     &     + 2.D0/3.D0*zzlog(renscale2)
     &     + zzlog(renscale2)*zzlog( - s25)
     &     + zzlog(renscale2)*zzlog( - s35)
     &     + zzlog(renscale2)*zzlog( - s14)
     &     - 3.D0/2.D0*zzlog(renscale2)**2
     &     )
      elseif(ihel.gt.4)then
         ze2 = (0d0,0d0)
         ze1 = (0d0,0d0)
         ze0 = (0d0,0d0)
      endif

* correction factor to convert from c_gamma prefactor
      ze0 = ze0 - pi**2/12d0*ze2
* correction from finite renormalisations: eq 2.19 Signer's thesis
      ze0 = ze0 - ( 1d0/3d0+4d0/12d0 )*ztree


      if(iorder.eq.-2) zwt = ze2
      if(iorder.eq.-1) zwt = ze1
      if(iorder.eq.0)  zwt = ze0
      zwt=zwt/2d0
      zzC1g1c=zwt

      return
      end
*****************************************************************************
* sub-leading colour (1,0) 1-loop QQb  amplitude

      function zzCt1g1c(iq1,ig,iqb1,iq2,iqb2,ihel,iflip,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      common /order/iorder
      parameter(pi=3.141592653589793238d0)

      i1=iq1
      i2=iqb2
      i3=iq2
      i4=iqb1
      i5=ig

      s12=s5(i2,i4)
      s13=s5(i2,i1)
      s14=s5(i2,i3)
      s15=s5(i2,i5)
      s23=s5(i4,i1)
      s24=s5(i4,i3)
      s25=s5(i4,i5)
      s34=s5(i1,i3)
      s35=s5(i1,i5)
      s45=s5(i3,i5)

      if(iflip.eq.0)then

      za12=za5(i2,i4)
      za13=za5(i2,i1)
      za14=za5(i2,i3)
      za15=za5(i2,i5)
      za23=za5(i4,i1)
      za24=za5(i4,i3)
      za25=za5(i4,i5)
      za34=za5(i1,i3)
      za35=za5(i1,i5)
      za45=za5(i3,i5)

      zb12=zb5(i2,i4)
      zb13=zb5(i2,i1)
      zb14=zb5(i2,i3)
      zb15=zb5(i2,i5)
      zb23=zb5(i4,i1)
      zb24=zb5(i4,i3)
      zb25=zb5(i4,i5)
      zb34=zb5(i1,i3)
      zb35=zb5(i1,i5)
      zb45=zb5(i3,i5)

      elseif(iflip.eq.1)then

      za12=-zb5(i2,i4)
      za13=-zb5(i2,i1)
      za14=-zb5(i2,i3)
      za15=-zb5(i2,i5)
      za23=-zb5(i4,i1)
      za24=-zb5(i4,i3)
      za25=-zb5(i4,i5)
      za34=-zb5(i1,i3)
      za35=-zb5(i1,i5)
      za45=-zb5(i3,i5)

      zb12=-za5(i2,i4)
      zb13=-za5(i2,i1)
      zb14=-za5(i2,i3)
      zb15=-za5(i2,i5)
      zb23=-za5(i4,i1)
      zb24=-za5(i4,i3)
      zb25=-za5(i4,i5)
      zb34=-za5(i1,i3)
      zb35=-za5(i1,i5)
      zb45=-za5(i3,i5)

      endif

      zi=(0d0,1d0)
      ztree=zzC1g0c(i1,i5,i4,i3,i2,ihel,iflip)

      if(ihel.eq.1)then

      zte2 =  + ztree * (
     &     + 2.D0
     &     )

      zte1 =  + ztree * (
     &     + 3.D0
     &     + zzlog( - s34)
     &     - zzlog( - s13)
     &     + zzlog( - s12)
     &     - zzlog( - s24)
     &     - zzlog( - s23)
     &     - zzlog( - s14)
     &     + 2.D0*zzlog(renscale2)
     &     )

      zte0 =  + zi * (
     &     - za12*za35**(-1)*s14**(-1)*zb45
     &     + 1.D0/2.D0*za12*za23*za35**(-1)*s14**(-1)*za25**(-1)*zb34
     &     )
      zte0 = zte0 + zL0(s23,s14,renscale2)*zi * (
     &     + za12*za13*za14**(-1)*za35**(-2)
     &     )
      zte0 = zte0 + zL0(s23,s25,renscale2)*zi * (
     &     - 3.D0/2.D0*za12**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     )
      zte0 = zte0 + zL0(s25,s14,renscale2)*zi * (
     &     - za12*za13*za14**(-1)*za35**(-2)
     &     )
      zte0 = zte0 + zL0(s25,s13,renscale2)*zi * (
     &     - za12*za35**(-1)*za45**(-1)
     &     )
      zte0 = zte0 + zL1(s23,s14,renscale2)*zi * (
     &     - za13*za35**(-2)*za25*zb45
     &     )
      zte0 = zte0 + zL1(s24,s13,renscale2)*zi * (
     &     + za15*za35**(-1)*za24*za45**(-1)*zb45
     &     )
      zte0 = zte0 + zL1(s25,s14,renscale2)*zi * (
     &     + za13*za23**2*za35**(-2)*za34**(-1)*za25**(-1)*zb34*za45
     &     + 1.D0/2.D0*za12*za23*za35**(-1)*za25**(-1)*zb34
     &     )
      zte0 = zte0 + zL1(s25,s13,renscale2)*zi * (
     &     + za13*za24**2*za34**(-1)*za25**(-1)*zb34*za45**(-1)
     &     )
      zte0 = zte0 + zL2(s23,s14,renscale2)*zi * (
     &     + za15*za23*za35**(-1)*zb35*zb45
     &     )
      zte0 = zte0 + zL2(s25,s14,renscale2)*zi * (
     &     + 1.D0/2.D0*za13*za23*za35**(-1)*zb34*zb35
     &     )
      zte0 = zte0 + zCF(s14,s23,s35)*zi * (
     &     - za12**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     )
      zte0 = zte0 + zCF(s14,s23,s25)*zi * (
     &     - za14**(-1)*za15**2*za23**2*za35**(-3)*za25**(-1)
     &     )
      zte0 = zte0 + zCF(s24,s13,s35)*zi * (
     &     - za12**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     )
      zte0 = zte0 + zCF(s35,s14,s24)*zi * (
     &     - za12**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     )
      zte0 = zte0 + zCF(s35,s12,s14)*zi * (
     &     + za12**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     )
      zte0 = zte0 + zCF(s12,s34,s35)*zi * (
     &     + za12**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     )
      zte0 = zte0 + zCF(s13,s24,s25)*zi * (
     &     - za14**(-1)*za15**2*za35**(-1)*za24**2*za25**(-1)*
     &    za45**(-2)
     &     )
      zte0 = zte0 + zCF(s25,s14,s34)*zi * (
     &     + za12**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     )
      zte0 = zte0 + zCF(s25,s13,s14)*zi * (
     &     - za13**2*za14**(-1)*za35**(-1)*za24**2*za34**(-2)*
     &    za25**(-1)
     &     )
      zte0 = zte0 + zCF(s34,s12,s25)*zi * (
     &     + za12**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     )
      zte0 = zte0 + ztree * (
     &     + 13.D0/2.D0
     &     - 1.D0/2.D0*zzlog( - s34)**2
     &     + 1.D0/2.D0*zzlog( - s13)**2
     &     - 1.D0/2.D0*zzlog( - s12)**2
     &     + 1.D0/2.D0*zzlog( - s24)**2
     &     - 3.D0/2.D0*zzlog( - s23)
     &     + 1.D0/2.D0*zzlog( - s23)**2
     &     - 3.D0/2.D0*zzlog( - s14)
     &     + 1.D0/2.D0*zzlog( - s14)**2
     &     + 3.D0*zzlog(renscale2)
     &     + zzlog(renscale2)*zzlog( - s34)
     &     - zzlog(renscale2)*zzlog( - s13)
     &     + zzlog(renscale2)*zzlog( - s12)
     &     - zzlog(renscale2)*zzlog( - s24)
     &     )
      zte0 = zte0 + ztree * (
     &     - zzlog(renscale2)*zzlog( - s23)
     &     - zzlog(renscale2)*zzlog( - s14)
     &     + zzlog(renscale2)**2
     &     )

      elseif(ihel.eq.2)then

      zte2 =  + ztree * (
     &     + 2.D0
     &     )

      zte1 =  + ztree * (
     &     + 3.D0
     &     + zzlog( - s34)
     &     - zzlog( - s13)
     &     + zzlog( - s12)
     &     - zzlog( - s24)
     &     - zzlog( - s23)
     &     - zzlog( - s14)
     &     + 2.D0*zzlog(renscale2)
     &     )

      zte0 =  + zi * (
     &     + za35**(-1)*za24*s14**(-1)*zb15
     &     )
      zte0 = zte0 + zL0(s23,s25,renscale2)*zi * (
     &     - za14**(-1)*za35**(-2)*za24*za34
     &     + 3.D0/2.D0*za14**(-1)*za35**(-1)*za24**2*za25**(-1)
     &     )
      zte0 = zte0 + zL0(s34,s25,renscale2)*zi * (
     &     + za15**(-1)*za35**(-1)*za24
     &     )
      zte0 = zte0 + zL1(s23,s14,renscale2)*zi * (
     &     + za35**(-2)*za34*za25*zb15
     &     )
      zte0 = zte0 + zL1(s25,s14,renscale2)*zi * (
     &     + za13**(-1)*za15*za23**2*za35**(-2)*za34*za25**(-1)*zb13
     &     + 2.D0*za23**2*za35**(-2)*za25**(-1)*za45*zb13
     &     )
      zte0 = zte0 + zL1(s25,s34,renscale2)*zi * (
     &     + za12**2*za13**(-1)*za15**(-1)*za34*za25**(-1)*zb13
     &     )
      zte0 = zte0 + zL1(s34,s12,renscale2)*zi * (
     &     + za12*za15**(-1)*za35**(-1)*za45*zb15
     &     )
      zte0 = zte0 + zL2(s23,s14,renscale2)*zi * (
     &     + za23*za35**(-1)*zb35*za45*zb15
     &     )
      zte0 = zte0 + zL2(s25,s14,renscale2)*zi * (
     &     + 1.D0/2.D0*za14*za23**2*za35**(-1)*za25**(-1)*zb13**2
     &     )
      zte0 = zte0 + zCF(s14,s23,s35)*zi * (
     &     + za14**(-1)*za35**(-1)*za24**2*za25**(-1)
     &     )
      zte0 = zte0 + zCF(s14,s23,s25)*zi * (
     &     + za14**(-1)*za23**2*za35**(-3)*za25**(-1)*za45**2
     &     )
      zte0 = zte0 + zCF(s24,s13,s35)*zi * (
     &     + za14**(-1)*za35**(-1)*za24**2*za25**(-1)
     &     )
      zte0 = zte0 + zCF(s35,s14,s12)*zi * (
     &     - za14**(-1)*za35**(-1)*za24**2*za25**(-1)
     &     )
      zte0 = zte0 + zCF(s35,s24,s14)*zi * (
     &     + za14**(-1)*za35**(-1)*za24**2*za25**(-1)
     &     )
      zte0 = zte0 + zCF(s12,s34,s35)*zi * (
     &     - za14**(-1)*za35**(-1)*za24**2*za25**(-1)
     &     )
      zte0 = zte0 + zCF(s13,s24,s25)*zi * (
     &     + za14**(-1)*za35**(-1)*za24**2*za25**(-1)
     &     )
      zte0 = zte0 + zCF(s25,s14,s13)*zi * (
     &     + za14**(-1)*za35**(-1)*za24**2*za25**(-1)
     &     )
      zte0 = zte0 + zCF(s25,s14,s34)*zi * (
     &     - za12**2*za13**(-2)*za14**(-1)*za35**(-1)*za34**2*
     &    za25**(-1)
     &     )
      zte0 = zte0 + zCF(s34,s12,s25)*zi * (
     &     - za12**2*za14**(-1)*za15**(-2)*za35**(-1)*za25**(-1)*
     &    za45**2
     &     )
      zte0 = zte0 + ztree * (
     &     + 13.D0/2.D0
     &     - 1.D0/2.D0*zzlog( - s34)**2
     &     + 1.D0/2.D0*zzlog( - s13)**2
     &     - 1.D0/2.D0*zzlog( - s12)**2
     &     + 1.D0/2.D0*zzlog( - s24)**2
     &     - 3.D0/2.D0*zzlog( - s23)
     &     + 1.D0/2.D0*zzlog( - s23)**2
     &     - 3.D0/2.D0*zzlog( - s14)
     &     + 1.D0/2.D0*zzlog( - s14)**2
     &     + 3.D0*zzlog(renscale2)
     &     + zzlog(renscale2)*zzlog( - s34)
     &     - zzlog(renscale2)*zzlog( - s13)
     &     + zzlog(renscale2)*zzlog( - s12)
     &     - zzlog(renscale2)*zzlog( - s24)
     &     )
      zte0 = zte0 + ztree * (
     &     - zzlog(renscale2)*zzlog( - s23)
     &     - zzlog(renscale2)*zzlog( - s14)
     &     + zzlog(renscale2)**2
     &     )

      elseif(ihel.eq.3)then

      zte2 =  + ztree * (
     &     + 2.D0
     &     )

      zte1 =  + ztree * (
     &     + 3.D0
     &     + zzlog( - s34)
     &     - zzlog( - s13)
     &     + zzlog( - s12)
     &     - zzlog( - s24)
     &     - zzlog( - s23)
     &     - zzlog( - s14)
     &     + 2.D0*zzlog(renscale2)
     &     )

      zte0 =  + zi * (
     &     + za13*s14**(-1)*za25**(-1)*zb45
     &     )
      zte0 = zte0 + zL0(s23,s35,renscale2)*zi * (
     &     + 3.D0/2.D0*za13**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     - za12*za13*za14**(-1)*za25**(-2)
     &     )
      zte0 = zte0 + zL0(s12,s35,renscale2)*zi * (
     &     + za13*za25**(-1)*za45**(-1)
     &     )
      zte0 = zte0 + zL1(s23,s14,renscale2)*zi * (
     &     + za12*za35*za25**(-2)*zb45
     &     )
      zte0 = zte0 + zL1(s35,s14,renscale2)*zi * (
     &     + 2.D0*za15*za23**2*za35**(-1)*za25**(-2)*zb24
     &     + za12*za23**2*za35**(-1)*za24**(-1)*za25**(-2)*za45*zb24
     &     )
      zte0 = zte0 + zL1(s35,s12,renscale2)*zi * (
     &     + za12*za35**(-1)*za24**(-1)*za34**2*za45**(-1)*zb24
     &     )
      zte0 = zte0 + zL1(s12,s34,renscale2)*zi * (
     &     + za15*za34*za25**(-1)*za45**(-1)*zb45
     &     )
      zte0 = zte0 + zL2(s23,s14,renscale2)*zi * (
     &     + za15*za23*za25**(-1)*zb25*zb45
     &     )
      zte0 = zte0 + zL2(s35,s14,renscale2)*zi * (
     &     + 1.D0/2.D0*za14*za23**2*za35**(-1)*za25**(-1)*zb24**2
     &     )
      zte0 = zte0 + zCF(s14,s23,s35)*zi * (
     &     + za14**(-1)*za15**2*za23**2*za35**(-1)*za25**(-3)
     &     )
      zte0 = zte0 + zCF(s14,s23,s25)*zi * (
     &     + za13**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     )
      zte0 = zte0 + zCF(s24,s13,s35)*zi * (
     &     + za13**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     )
      zte0 = zte0 + zCF(s35,s14,s24)*zi * (
     &     + za13**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     )
      zte0 = zte0 + zCF(s35,s14,s12)*zi * (
     &     - za12**2*za14**(-1)*za35**(-1)*za24**(-2)*za34**2*
     &    za25**(-1)
     &     )
      zte0 = zte0 + zCF(s12,s34,s35)*zi * (
     &     - za14**(-1)*za15**2*za35**(-1)*za34**2*za25**(-1)*
     &    za45**(-2)
     &     )
      zte0 = zte0 + zCF(s13,s24,s25)*zi * (
     &     + za13**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     )
      zte0 = zte0 + zCF(s25,s14,s34)*zi * (
     &     - za13**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     )
      zte0 = zte0 + zCF(s25,s13,s14)*zi * (
     &     + za13**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     )
      zte0 = zte0 + zCF(s34,s12,s25)*zi * (
     &     - za13**2*za14**(-1)*za35**(-1)*za25**(-1)
     &     )
      zte0 = zte0 + ztree * (
     &     + 13.D0/2.D0
     &     - 1.D0/2.D0*zzlog( - s34)**2
     &     + 1.D0/2.D0*zzlog( - s13)**2
     &     - 1.D0/2.D0*zzlog( - s12)**2
     &     + 1.D0/2.D0*zzlog( - s24)**2
     &     - 3.D0/2.D0*zzlog( - s23)
     &     + 1.D0/2.D0*zzlog( - s23)**2
     &     - 3.D0/2.D0*zzlog( - s14)
     &     + 1.D0/2.D0*zzlog( - s14)**2
     &     + 3.D0*zzlog(renscale2)
     &     + zzlog(renscale2)*zzlog( - s34)
     &     - zzlog(renscale2)*zzlog( - s13)
     &     + zzlog(renscale2)*zzlog( - s12)
     &     - zzlog(renscale2)*zzlog( - s24)
     &     )
      zte0 = zte0 + ztree * (
     &     - zzlog(renscale2)*zzlog( - s23)
     &     - zzlog(renscale2)*zzlog( - s14)
     &     + zzlog(renscale2)**2
     &     )


      elseif(ihel.eq.4)then


      zte2 =  + ztree * (
     &     + 2.D0
     &     )

      zte1 =  + ztree * (
     &     + 3.D0
     &     + zzlog( - s34)
     &     - zzlog( - s13)
     &     + zzlog( - s12)
     &     - zzlog( - s24)
     &     - zzlog( - s23)
     &     - zzlog( - s14)
     &     + 2.D0*zzlog(renscale2)
     &     )

      zte0 =  + zi * (
     &     - za34*s14**(-1)*za25**(-1)*zb15
     &     + 1.D0/2.D0*za23*za35**(-1)*za34*s14**(-1)*za25**(-1)*zb12
     &     )
      zte0 = zte0 + zL0(s23,s14,renscale2)*zi * (
     &     + za14**(-1)*za24*za34*za25**(-2)
     &     )
      zte0 = zte0 + zL0(s23,s35,renscale2)*zi * (
     &     - 3.D0/2.D0*za14**(-1)*za35**(-1)*za34**2*za25**(-1)
     &     )
      zte0 = zte0 + zL0(s35,s14,renscale2)*zi * (
     &     - za14**(-1)*za24*za34*za25**(-2)
     &     )
      zte0 = zte0 + zL0(s35,s24,renscale2)*zi * (
     &     - za15**(-1)*za34*za25**(-1)
     &     )
      zte0 = zte0 + zL1(s23,s14,renscale2)*zi * (
     &     - za35*za24*za25**(-2)*zb15
     &     )
      zte0 = zte0 + zL1(s35,s14,renscale2)*zi * (
     &     + za12**(-1)*za15*za23**2*za35**(-1)*za24*za25**(-2)*zb12
     &     + 1.D0/2.D0*za23*za35**(-1)*za34*za25**(-1)*zb12
     &     )
      zte0 = zte0 + zL1(s35,s24,renscale2)*zi * (
     &     + za12**(-1)*za13**2*za15**(-1)*za35**(-1)*za24*zb12
     &     )
      zte0 = zte0 + zL1(s13,s24,renscale2)*zi * (
     &     + za13*za15**(-1)*za25**(-1)*za45*zb15
     &     )
      zte0 = zte0 + zL2(s23,s14,renscale2)*zi * (
     &     + za23*za25**(-1)*zb25*za45*zb15
     &     )
      zte0 = zte0 + zL2(s35,s14,renscale2)*zi * (
     &     + 1.D0/2.D0*za23*za24*za25**(-1)*zb25*zb12
     &     )
      zte0 = zte0 + zCF(s14,s23,s35)*zi * (
     &     - za14**(-1)*za23**2*za35**(-1)*za25**(-3)*za45**2
     &     )
      zte0 = zte0 + zCF(s14,s23,s25)*zi * (
     &     - za14**(-1)*za35**(-1)*za34**2*za25**(-1)
     &     )
      zte0 = zte0 + zCF(s24,s13,s35)*zi * (
     &     - za13**2*za14**(-1)*za15**(-2)*za35**(-1)*za25**(-1)*
     &    za45**2
     &     )
      zte0 = zte0 + zCF(s35,s14,s12)*zi * (
     &     + za14**(-1)*za35**(-1)*za34**2*za25**(-1)
     &     )
      zte0 = zte0 + zCF(s35,s24,s14)*zi * (
     &     - za12**(-2)*za13**2*za14**(-1)*za35**(-1)*za24**2*
     &    za25**(-1)
     &     )
      zte0 = zte0 + zCF(s12,s34,s35)*zi * (
     &     + za14**(-1)*za35**(-1)*za34**2*za25**(-1)
     &     )
      zte0 = zte0 + zCF(s13,s24,s25)*zi * (
     &     - za14**(-1)*za35**(-1)*za34**2*za25**(-1)
     &     )
      zte0 = zte0 + zCF(s25,s14,s13)*zi * (
     &     - za14**(-1)*za35**(-1)*za34**2*za25**(-1)
     &     )
      zte0 = zte0 + zCF(s25,s34,s14)*zi * (
     &     + za14**(-1)*za35**(-1)*za34**2*za25**(-1)
     &     )
      zte0 = zte0 + zCF(s34,s12,s25)*zi * (
     &     + za14**(-1)*za35**(-1)*za34**2*za25**(-1)
     &     )
      zte0 = zte0 + ztree * (
     &     + 13.D0/2.D0
     &     - 1.D0/2.D0*zzlog( - s34)**2
     &     + 1.D0/2.D0*zzlog( - s13)**2
     &     - 1.D0/2.D0*zzlog( - s12)**2
     &     + 1.D0/2.D0*zzlog( - s24)**2
     &     - 3.D0/2.D0*zzlog( - s23)
     &     + 1.D0/2.D0*zzlog( - s23)**2
     &     - 3.D0/2.D0*zzlog( - s14)
     &     + 1.D0/2.D0*zzlog( - s14)**2
     &     + 3.D0*zzlog(renscale2)
     &     + zzlog(renscale2)*zzlog( - s34)
     &     - zzlog(renscale2)*zzlog( - s13)
     &     + zzlog(renscale2)*zzlog( - s12)
     &     - zzlog(renscale2)*zzlog( - s24)
     &     )
      zte0 = zte0 + ztree * (
     &     - zzlog(renscale2)*zzlog( - s23)
     &     - zzlog(renscale2)*zzlog( - s14)
     &     + zzlog(renscale2)**2
     &     )
      elseif(ihel.gt.4)then
         zte2 = (0d0,0d0)
         zte1 = (0d0,0d0)
         zte0 = (0d0,0d0)
      endif

* correction factor to convert from c_gamma prefactor
      zte0 = zte0 - pi**2/12d0*zte2
* correction from finite renormalisations: eq 2.19 Signers thesis
      zte0 = zte0 - ( -4d0/4d0 )*ztree



      if(iorder.eq.-2) zwt = zte2
      if(iorder.eq.-1) zwt = zte1
      if(iorder.eq.0)  zwt = zte0
      zwt=zwt/2d0
      zzCt1g1c=zwt

      return
      end
*****************************************************************************
* NF (1,0) 1-loop QQb  amplitude

      function zzCh1g1c(iq1,ig,iqb1,iq2,iqb2,ihel,iflip,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)
      common /order/iorder
      parameter(pi=3.141592653589793238d0)

      i1=iq1
      i2=iqb2
      i3=iq2
      i4=iqb1
      i5=ig

      s12=s5(i2,i4)
      s13=s5(i2,i1)
      s14=s5(i2,i3)
      s15=s5(i2,i5)
      s23=s5(i4,i1)
      s24=s5(i4,i3)
      s25=s5(i4,i5)
      s34=s5(i1,i3)
      s35=s5(i1,i5)
      s45=s5(i3,i5)

      if(iflip.eq.0)then

      za12=za5(i2,i4)
      za13=za5(i2,i1)
      za14=za5(i2,i3)
      za15=za5(i2,i5)
      za23=za5(i4,i1)
      za24=za5(i4,i3)
      za25=za5(i4,i5)
      za34=za5(i1,i3)
      za35=za5(i1,i5)
      za45=za5(i3,i5)

      zb12=zb5(i2,i4)
      zb13=zb5(i2,i1)
      zb14=zb5(i2,i3)
      zb15=zb5(i2,i5)
      zb23=zb5(i4,i1)
      zb24=zb5(i4,i3)
      zb25=zb5(i4,i5)
      zb34=zb5(i1,i3)
      zb35=zb5(i1,i5)
      zb45=zb5(i3,i5)

      elseif(iflip.eq.1)then

      za12=-zb5(i2,i4)
      za13=-zb5(i2,i1)
      za14=-zb5(i2,i3)
      za15=-zb5(i2,i5)
      za23=-zb5(i4,i1)
      za24=-zb5(i4,i3)
      za25=-zb5(i4,i5)
      za34=-zb5(i1,i3)
      za35=-zb5(i1,i5)
      za45=-zb5(i3,i5)

      zb12=-za5(i2,i4)
      zb13=-za5(i2,i1)
      zb14=-za5(i2,i3)
      zb15=-za5(i2,i5)
      zb23=-za5(i4,i1)
      zb24=-za5(i4,i3)
      zb25=-za5(i4,i5)
      zb34=-za5(i1,i3)
      zb35=-za5(i1,i5)
      zb45=-za5(i3,i5)

      endif

      zi=(0d0,1d0)
      ztree=zzC1g0c(i1,i5,i4,i3,i2,ihel,iflip)

      if(ihel.eq.1)then

      zhe2 = 0

      zhe1 =  + ztree * (
     &     + 1.D0/3.D0
     &     )

      zhe0 =  + ztree * (
     &     - 10.D0/9.D0
     &     + 2.D0/3.D0*zzlog( - s14)
     &     - 2.D0/3.D0*zzlog(renscale2)
     &     )

      elseif(ihel.eq.2)then

      zhe2 = 0

      zhe1 =  + ztree * (
     &     + 1.D0/3.D0
     &     )

      zhe0 =  + ztree * (
     &     - 10.D0/9.D0
     &     + 2.D0/3.D0*zzlog( - s14)
     &     - 2.D0/3.D0*zzlog(renscale2)
     &     )


      elseif(ihel.eq.3)then

      zhe2 = 0

      zhe1 =  + ztree * (
     &     + 1.D0/3.D0
     &     )

      zhe0 =  + ztree * (
     &     - 10.D0/9.D0
     &     + 2.D0/3.D0*zzlog( - s14)
     &     - 2.D0/3.D0*zzlog(renscale2)
     &     )


      elseif(ihel.eq.4)then

      zhe2 = 0

      zhe1 =  + ztree * (
     &     + 1.D0/3.D0
     &     )

      zhe0 =  + ztree * (
     &     - 10.D0/9.D0
     &     + 2.D0/3.D0*zzlog( - s14)
     &     - 2.D0/3.D0*zzlog(renscale2)
     &     )
      elseif(ihel.gt.4)then
         zhe2 = (0d0,0d0)
         zhe1 = (0d0,0d0)
         zhe0 = (0d0,0d0)
      endif

* correction factor to convert from c_gamma prefactor
      zhe0 = zhe0 - pi**2/12d0*zhe2

      if(iorder.eq.-2) zwt = zhe2
      if(iorder.eq.-1) zwt = zhe1
      if(iorder.eq.0)  zwt = zhe0
      zwt=zwt/2d0
      zzCh1g1c=zwt

      return
      end
****************************************************************************
* leading colour (0,1) 1-loop Qqb amplitude

      function zzC1g1b(iq1,iqb2,iq2,ig,iqb1,ihel,iflip,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      if(ihel.eq.1)then
      zwt= zzC1g1a(iq2,ig,iqb1,iq1,iqb2,1,iflip,renscale2)
      elseif(ihel.eq.2)then
      zwt= zzC1g1a(iq2,ig,iqb1,iq1,iqb2,3,iflip,renscale2)
      elseif(ihel.eq.3)then
      zwt= zzC1g1a(iq2,ig,iqb1,iq1,iqb2,2,iflip,renscale2)
      elseif(ihel.eq.4)then
      zwt= zzC1g1a(iq2,ig,iqb1,iq1,iqb2,4,iflip,renscale2)
      elseif(ihel.gt.4)then
      zwt = (0d0,0d0)
      endif

      zzC1g1b = zwt

      return
      end
****************************************************************************
* leading NF (0,1) 1-loop Qqb amplitude

      function zzCh1g1b(iq1,iqb2,iq2,ig,iqb1,ihel,iflip,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      if(ihel.eq.1)then
      zwt= zzCh1g1a(iq2,ig,iqb1,iq1,iqb2,1,iflip,renscale2)
      elseif(ihel.eq.2)then
      zwt= zzCh1g1a(iq2,ig,iqb1,iq1,iqb2,3,iflip,renscale2)
      elseif(ihel.eq.3)then
      zwt= zzCh1g1a(iq2,ig,iqb1,iq1,iqb2,2,iflip,renscale2)
      elseif(ihel.eq.4)then
      zwt= zzCh1g1a(iq2,ig,iqb1,iq1,iqb2,4,iflip,renscale2)
      elseif(ihel.gt.4)then
      zwt = (0d0,0d0)
      endif

      zzCh1g1b = zwt

      return
      end
****************************************************************************
* sub leading colour (0,1) 1-loop Qqb amplitude

      function zzCt1g1b(iq1,iqb2,iq2,ig,iqb1,ihel,iflip,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      if(ihel.eq.1)then
      zwt= zzCt1g1a(iq2,ig,iqb1,iq1,iqb2,1,iflip,renscale2)
      elseif(ihel.eq.2)then
      zwt= zzCt1g1a(iq2,ig,iqb1,iq1,iqb2,3,iflip,renscale2)
      elseif(ihel.eq.3)then
      zwt= zzCt1g1a(iq2,ig,iqb1,iq1,iqb2,2,iflip,renscale2)
      elseif(ihel.eq.4)then
      zwt= zzCt1g1a(iq2,ig,iqb1,iq1,iqb2,4,iflip,renscale2)
      elseif(ihel.gt.4)then
      zwt = (0d0,0d0)
      endif

      zzCt1g1b = zwt

      return
      end
****************************************************************************
* leading colour (0,1) 1-loop QQb  amplitude

      function zzC1g1d(iq1,iqb1,iq2,ig,iqb2,ihel,iflip,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      if(ihel.eq.1)then
      zwt= zzC1g1c(iq2,ig,iqb2,iq1,iqb1,1,iflip,renscale2)
      elseif(ihel.eq.2)then
      zwt= zzC1g1c(iq2,ig,iqb2,iq1,iqb1,3,iflip,renscale2)
      elseif(ihel.eq.3)then
      zwt= zzC1g1c(iq2,ig,iqb2,iq1,iqb1,2,iflip,renscale2)
      elseif(ihel.eq.4)then
      zwt= zzC1g1c(iq2,ig,iqb2,iq1,iqb1,4,iflip,renscale2)
      elseif(ihel.gt.4)then
      zwt = (0d0,0d0)
      endif

      zzC1g1d = zwt

      return
      end
****************************************************************************
* leading NF (0,1) 1-loop QQb  amplitude

      function zzCh1g1d(iq1,iqb1,iq2,ig,iqb2,ihel,iflip,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      if(ihel.eq.1)then
      zwt= zzCh1g1c(iq2,ig,iqb2,iq1,iqb1,1,iflip,renscale2)
      elseif(ihel.eq.2)then
      zwt= zzCh1g1c(iq2,ig,iqb2,iq1,iqb1,3,iflip,renscale2)
      elseif(ihel.eq.3)then
      zwt= zzCh1g1c(iq2,ig,iqb2,iq1,iqb1,2,iflip,renscale2)
      elseif(ihel.eq.4)then
      zwt= zzCh1g1c(iq2,ig,iqb2,iq1,iqb1,4,iflip,renscale2)
      elseif(ihel.gt.4)then
      zwt = (0d0,0d0)
      endif

      zzCh1g1d = zwt

      return
      end
****************************************************************************
* sub leading colour (0,1) 1-loop QQb  amplitude

      function zzCt1g1d(iq1,iqb1,iq2,ig,iqb2,ihel,iflip,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      if(ihel.eq.1)then
      zwt= zzCt1g1c(iq2,ig,iqb2,iq1,iqb1,1,iflip,renscale2)
      elseif(ihel.eq.2)then
      zwt= zzCt1g1c(iq2,ig,iqb2,iq1,iqb1,3,iflip,renscale2)
      elseif(ihel.eq.3)then
      zwt= zzCt1g1c(iq2,ig,iqb2,iq1,iqb1,2,iflip,renscale2)
      elseif(ihel.eq.4)then
      zwt= zzCt1g1c(iq2,ig,iqb2,iq1,iqb1,4,iflip,renscale2)
      elseif(ihel.gt.4)then
      zwt = (0d0,0d0)
      endif

      zzCt1g1d = zwt

      return
      end
****************************************************************************
* symmetrized qQ LC 1-loop amplitude

      function zzC1g1gam1(iq1,iqb1,iq2,iqb2,ig,ihel,iflip,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      zwt = zzC1g1a(iq1,ig,iqb2,iq2,iqb1,ihel,iflip,renscale2)
     .    + zzC1g1b(iq1,iqb2,iq2,ig,iqb1,ihel,iflip,renscale2)

      zzC1g1gam1 = zwt

      return
      end
****************************************************************************
* symmetrized qQ NF 1-loop amplitude (=symmetrized QQ NF 1-loop amplitude)
* included for completeness

      function zzCh1g1gam1(iq1,iqb1,iq2,iqb2,ig,ihel,iflip,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      zwt = zzCh1g1a(iq1,ig,iqb2,iq2,iqb1,ihel,iflip,renscale2)
     .    + zzCh1g1b(iq1,iqb2,iq2,ig,iqb1,ihel,iflip,renscale2)

      zzCh1g1gam1 = zwt

      return
      end
****************************************************************************
* symmetrized qQ SLC 1-loop amplitude

      function zzCt1g1gam1(iq1,iqb1,iq2,iqb2,ig,ihel,iflip,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      zwt = zzCt1g1a(iq1,ig,iqb2,iq2,iqb1,ihel,iflip,renscale2)
     .    + zzCt1g1b(iq1,iqb2,iq2,ig,iqb1,ihel,iflip,renscale2)

      zzCt1g1gam1 = zwt

      return
      end
****************************************************************************
* symmetrized QQ LC 1-loop amplitude

      function zzC1g1gam2(iq1,iqb1,iq2,iqb2,ig,ihel,iflip,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      zwt = zzC1g1c(iq1,ig,iqb1,iq2,iqb2,ihel,iflip,renscale2)
     .    + zzC1g1d(iq1,iqb1,iq2,ig,iqb2,ihel,iflip,renscale2)

      zzC1g1gam2 = zwt

      return
      end
****************************************************************************
* symmetrized QQ NF 1-loop amplitude (=symmetrized qQ NF 1-loop amplitude)
* included for completeness

      function zzCh1g1gam2(iq1,iqb1,iq2,iqb2,ig,ihel,iflip,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      zwt = zzCh1g1c(iq1,ig,iqb1,iq2,iqb2,ihel,iflip,renscale2)
     .    + zzCh1g1d(iq1,iqb1,iq2,ig,iqb2,ihel,iflip,renscale2)

      zzCh1g1gam2 = zwt

      return
      end
****************************************************************************
* symmetrized QQ SLC 1-loop amplitude

      function zzCt1g1gam2(iq1,iqb1,iq2,iqb2,ig,ihel,iflip,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      zwt = zzCt1g1c(iq1,ig,iqb1,iq2,iqb2,ihel,iflip,renscale2)
     .    + zzCt1g1d(iq1,iqb1,iq2,ig,iqb2,ihel,iflip,renscale2)

      zzCt1g1gam2 = zwt

      return
      end
****************************************************************************
* symmetrized QQ NF 1-loop amplitude

      function zzCh1g1gam(iq1,iqb1,iq2,iqb2,ig,ihel,iflip,renscale2)
      use KinData_mod
      implicit real*8(a-h,o-y)
      implicit complex*16 (z)

      zwt = zzCh1g1c(iq1,ig,iqb1,iq2,iqb2,ihel,iflip,renscale2)
     .    + zzCh1g1d(iq1,iqb1,iq2,ig,iqb2,ihel,iflip,renscale2)

      zzCh1g1gam = zwt

      return
      end
****************************************************************************
* Full SLC (1/N^2) colour level of FullC1g1
      function Ct1g1(iq1,iqb1,iq2,iqb2,ig,renscale2)
      implicit real*8 (a-h,o-y)

      Ct1g1 = + C1g1Y(iq1,iqb1,iq2,iqb2,ig,renscale2)
     .        + Ct1g1X(iq1,iqb1,iq2,iqb2,ig,renscale2)
     .        - C1g1gamX(iq1,iqb1,iq2,iqb2,ig,renscale2)
     .        - C1g1gamY(iq1,iqb1,iq2,iqb2,ig,renscale2)

      return
      end
****************************************************************************
* Full SLC nf (nf/N^3) colour level of FullC1g1
      function Cth1g1(iq1,iqb1,iq2,iqb2,ig,renscale2)
      implicit real*8 (a-h,o-y)

      Cth1g1 = + Ch1g1Y(iq1,iqb1,iq2,iqb2,ig,renscale2)
     .         - 2d0*Ch1g1gamX(iq1,iqb1,iq2,iqb2,ig,renscale2)

      return
      end
****************************************************************************
* Full SSLC (1/N^4) colour level of FullC1g1
      function Ctt1g1(iq1,iqb1,iq2,iqb2,ig,renscale2)
      implicit real*8 (a-h,o-y)

      Ctt1g1 = + Ct1g1Y(iq1,iqb1,iq2,iqb2,ig,renscale2)
     .         - Ct1g1gamX(iq1,iqb1,iq2,iqb2,ig,renscale2)
     .         - Ct1g1gamY(iq1,iqb1,iq2,iqb2,ig,renscale2)
     
      return
      end 
****************************************************************************
* Full LC colour level of FullD1g1
      function D1g1full(iq1,iqb1,iq2,iqb2,ig,renscale2)
      implicit real*8 (a-h,o-y)

      D1g1full = + D1g1(iq1,iqb1,iq2,iqb2,ig,renscale2)
     .           + D1g1(iq1,iqb2,iq2,iqb1,ig,renscale2)
     .           - D1g1gam1(iq1,iqb1,iq2,iqb2,ig,renscale2)
     .           - D1g1gam1(iq1,iqb2,iq2,iqb1,ig,renscale2)

      return
      end
****************************************************************************
* Full LC nf (nf/N) colour level of FullD1g1
      function Dh1g1full(iq1,iqb1,iq2,iqb2,ig,renscale2)
      implicit real*8 (a-h,o-y)

      Dh1g1full = + Dh1g1(iq1,iqb1,iq2,iqb2,ig,renscale2)
     .            + Dh1g1(iq1,iqb2,iq2,iqb1,ig,renscale2)
     .            - Dh1g1gam(iq1,iqb1,iq2,iqb2,ig,renscale2)
     .            - Dh1g1gam(iq1,iqb2,iq2,iqb1,ig,renscale2)

      return
      end
****************************************************************************
* Full SLC colour level of FullD1g1
      function Dt1g1full(iq1,iqb1,iq2,iqb2,ig,renscale2)
      implicit real*8 (a-h,o-y)

      Dt1g1full = + Dt1g1(iq1,iqb1,iq2,iqb2,ig,renscale2)
     .            + Dt1g1(iq1,iqb2,iq2,iqb1,ig,renscale2)
     .            - Dt1g1gam1(iq1,iqb1,iq2,iqb2,ig,renscale2)
     .            - Dt1g1gam1(iq1,iqb2,iq2,iqb1,ig,renscale2)
     .            - D1g1gam2(iq1,iqb1,iq2,iqb2,ig,renscale2)
     .            - D1g1gam2(iq1,iqb2,iq2,iqb1,ig,renscale2)
    
      return
      end
****************************************************************************
* Full SLC nf (nf/N^3) colour level of FullD1g1
      function Dth1g1full(iq1,iqb1,iq2,iqb2,ig,renscale2)
      implicit real*8 (a-h,o-y)

      Dth1g1full = - Dh1g1gam(iq1,iqb1,iq2,iqb2,ig,renscale2)
     .             - Dh1g1gam(iq1,iqb2,iq2,iqb1,ig,renscale2)

      return
      end
****************************************************************************
* Full SSLC colour level of FullD1g1
      function Dtt1g1full(iq1,iqb1,iq2,iqb2,ig,renscale2)
      implicit real*8 (a-h,o-y)

      Dtt1g1full = - Dt1g1gam2(iq1,iqb1,iq2,iqb2,ig,renscale2)
     .             - Dt1g1gam2(iq1,iqb2,iq2,iqb1,ig,renscale2)

      return
      end
****************************************************************************
